// MacAppCompatibility.h
// Copyright  1997 Apple Computer, Inc. All rights reserved.
// Portions Copyright  1994-1996 Nick Nallick.

#ifndef __MacAppCompatibility__
#define __MacAppCompatibility__

#ifndef qMacAppVersion
#define qMacAppVersion	30
#endif

#ifndef MAVolatile
#if qHasVolatile
#define MAVolatile(type, id)				type volatile id
#else
#define MAVolatile(type, id)				type id; ((void) &id)
#endif
#endif

#ifndef MAVolatileInit
#if qHasVolatile
#define MAVolatileInit(type, id, value)		type volatile id(value)
#else
#define MAVolatileInit(type, id, value)		type id = value; ((void) &id)
#endif
#endif

#if qMacAppVersion == 30
#define HANDLE_BASED_OBJECTS
#endif

#ifdef HANDLE_BASED_OBJECTS

#ifndef DECLARE_CLASS
#define DECLARE_CLASS(name)
#endif

#ifndef DEFINE_CLASS
#define DEFINE_CLASS(name, firstBase)
#endif

#ifndef VIRTUAL
#define VIRTUAL			virtual	pascal
#endif

#ifndef MACAPP_METHOD
#define MACAPP_METHOD	pascal
#endif

#ifndef TRY
#define	TRY(fi)			if ((fi).Try())
#endif

#else	// ! HANDLE_BASED_OBJECTS

#ifndef DECLARE_CLASS
#define DECLARE_CLASS(name)				DeclareClass(name)
#endif

#ifndef DEFINE_CLASS
#define DEFINE_CLASS(name, firstBase)	DefineClass(name, firstBase)
#endif

#ifndef VIRTUAL
#define VIRTUAL			virtual
#endif

#ifndef MACAPP_METHOD
#define MACAPP_METHOD
#endif

#ifndef TRY
#define	TRY(fi)			Try(fi)
#endif

#ifndef TRUE
#define TRUE			true
#endif

#ifndef FALSE
#define FALSE			false
#endif

#endif

#if qMacAppVersion >= 33
#define gMacAppAppleEventIdleProc	TAppleEvent::fgIdleProc
#endif

#if qMacAppVersion > 33

#define BOOLEAN			bool
#define CONST			const
#define CVIEWPOINT		CViewPoint
#define CPOINT			CPoint_AC
#define CVIEWRECT		CViewRect
#define CRECT			CRect_AC
#define CSTREAM			CStream_AC
#define PROGRAMBREAK	ProgramBreak_AC
#define ITERCURRENT		Current
#define THROWIFNULL 	ThrowIfNULL_AC
#define THROWIFOSERR 	ThrowIfOSErr_AC
#define THROWIFRESERROR	ThrowIfResError_AC
#define THROWIFMEMERROR	ThrowIfMemError_AC
#define USERESFILE		UseResFile_AC
#define DISPOSEIFHANDLE DisposeIfHandle_AC
#define CSTR255			CStr255_AC

#else // earlier than Release 10

#define	BOOLEAN			Boolean
#define	CONST
#define CVIEWPOINT		VPoint
#define CPOINT			CPoint
#define CVIEWRECT		VRect
#define CRECT			CRect
#define CSTREAM			TStream
#define PROGRAMBREAK 	ProgramBreak
#define ITERCURRENT 	Current
#define THROWIFNULL		FailNIL
#define THROWIFOSERR	FailOSErr
#define THROWIFRESERROR	FailResError
#define THROWIFMEMERROR	FailMemError
#define USERESFILE		MAUseResFile
#define DISPOSEIFHANDLE	DisposeIfHandle
#define CSTR255			CStr255

#endif

// Inline functions

inline unsigned char TToolboxEvent_GetCharacter(TToolboxEvent* event)
{
#if qMacAppVersion > 33
	return event->GetText()[1];
#else
	return event->fCharacter;
#endif
}

inline TEventHandler* TBehavior_GetOwner(TBehavior* behavior)
{
#if qMacAppVersion > 33
	return behavior->GetOwner();
#else
	return behavior->fOwner;
#endif
}

inline void TBehavior_SetOwner(TBehavior* behavior, TEventHandler* newOwner)
{
#if qMacAppVersion > 33
	behavior->SetOwner(newOwner);
#else
	behavior->fOwner = newOwner;
#endif
}

#endif