//	UBehaviorApp.cp
//	Copyright  1992-1996 Nick Nallick.  All rights reserved.
//
//
//	Changes:
//
//			Add MacApp Release 13 compatibility
//		 	Make sure class gets declared and defined.
//			Add MacApp 3.3 compatibility.
//			Add MacApp 3.1 compatibility.
//

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#include "UBehaviorApp.h"


///////////////////////////////// TBehaviorApp /////////////////////////////////
//
//	The an application object that loads its behaviors from any 'BHVR' resource.
//

#pragma segment AInit

DEFINE_CLASS(TBehaviorApp, TApplication);

#if qMacAppVersion > 33
TBehaviorApp::TBehaviorApp(OSType itsMainFileType, OSType itsCreator) :
	TApplication(itsMainFileType, itsCreator)
{
	// can't call MakeBehaviors here because gViewServer hasn't been constructed yet.
}

void TBehaviorApp::DoInitialState()
{
	TApplication::DoInitialState();
	MakeBehaviors;
}

#else

MACAPP_METHOD
void TBehaviorApp::IBehaviorApp(OSType itsMainFileType, OSType itsCreator)
{
	IApplication(itsMainFileType, itsCreator);
	MakeBehaviors();
}

#endif // qMacAppVersion

void TBehaviorApp::MakeBehaviors()
{
	short curRsrcFile = CurResFile();
	const ResType kBehaviorType = 'BHVR';
	short rsrcCount = CountResources(kBehaviorType);	// try all 'BHVR' resources
	for (short i = 1; i <= rsrcCount; ++i)
	{
		SetResLoad(FALSE);				// don't load the resource until we know we need it
		Handle rsrc = GetIndResource(kBehaviorType, i);
		SetResLoad(TRUE);
		THROWIFNULL(rsrc);
		if (HomeResFile(rsrc) == curRsrcFile)		// only use the current resource file
		{
			MAVolatileInit(TStream*, stream, NULL);
#if qMacAppVersion <= 33
			FailInfo fi;
			TRY(fi)
#else
			try
#endif
			{
				TBehavior* behavior;
				LoadResource(rsrc);
				THROWIFRESERROR();
				stream = gViewServer->DoMakeViewStream(kNoResource, rsrc);	// create a stream
				short behaviorCount = stream->ReadInteger();		// number of behavior in resource
				while (behaviorCount-- > 0)
				{
#if qMacAppVersion <= 33
					if (stream->ReadStreamObject((TObject*&) behavior))
#else
					bool wasValid;
					behavior = dynamic_cast<TBehavior*> (stream->ReadStreamObject(wasValid));
					if (wasValid)
#endif
						this->AddBehavior(behavior);
				}
				FreeIfObject(stream);
#if qMacAppVersion <= 33
				fi.Success();
#endif
			}
#if qMacAppVersion <= 33
			else
#else
			catch (CException_AC& theException)
#endif
			{
				ReleaseResource(rsrc);
				FreeIfObject(stream);
#if qMacAppVersion <= 33
				fi.ReSignal();
#else
				throw;
#endif
			}
		}
		ReleaseResource(rsrc);
	}
}