//	UHandlerBehavior.h
//	Copyright  1992-1996 Nick Nallick.  All rights reserved.

#ifndef __UHandlerBehavior__
#define __UHandlerBehavior__

#ifndef __MacAppCompatibility__
#include "MacAppCompatibility.h"
#endif

const IDType	kHandlerBehaviorID =	'hdlr';

typedef enum
{
	kNullMsg,
	kAppleCommandMsg,
	kCommandKeyMsg,
	kEventMsg,
	kIdleMsg,
	kInitializeMsg,
	kKeyDownMsg,
	kKeyUpMsg,
	kMenuCommandMsg,
	kMouseDownMsg,
	kMouseUpMsg,
	kSetupMenuMsg,
	kUpdateMsg
} HandlerMessage;


class THandlerBehavior : public TBehavior
#if qMacAppVersion > 33
, MDependable_AC
#endif
{
#if qMacAppVersion > 30
	MA_DECLARE_CLASS;
#endif

	public:
								THandlerBehavior();
#if qMacAppVersion <= 33
		VIRTUAL void			IHandlerBehavior();
		VIRTUAL void			Free();
#else
		virtual 				~THandlerBehavior();
#endif

#ifdef HANDLE_BASED_OBJECTS
		VIRTUAL void			Initialize();
#endif

#if qMacAppVersion == 30 || qMacAppVersion == 31
		VIRTUAL void			DoAppleCommand(CommandNumber aCommandNumber, const AppleEvent& message, const AppleEvent& reply);
#else
		VIRTUAL BOOLEAN			DoScriptCommand(CommandNumber aCommandNumber, TAppleEvent* message, TAppleEvent* reply);
#endif

#if qMacAppVersion <= 33
		VIRTUAL void			DoBehaviorUpdate(ChangeID theChange, TObject* changedObject, TObject* changedBy, TDependencySpace* dependencySpace);
#else
		virtual void			DoUpdate(ChangeID_AC theChange, MDependable_AC* changedObject, void* changeData, CDependencySpace_AC* dependencySpace);
#endif
		VIRTUAL void			DoCommandKeyEvent(TToolboxEvent* event);
		VIRTUAL void			DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event);
		VIRTUAL BOOLEAN			DoIdle(IdlePhase phase);
		VIRTUAL void			DoKeyEvent(TToolboxEvent* event);
		VIRTUAL void			DoKeyUp(TToolboxEvent* event);
		VIRTUAL void			DoMenuCommand(CommandNumber aCommandNumber);
		VIRTUAL BOOLEAN			DoMouseCommand(CVIEWPOINT& theMouse, TToolboxEvent* event, CPOINT hysteresis);
		VIRTUAL BOOLEAN			DoMouseUp(CVIEWPOINT& theMouse, TToolboxEvent* event, CPOINT hysteresis);
		VIRTUAL void			DoSetupMenus();
		VIRTUAL void			DoUserAction(HandlerMessage message, long parameter, TEventHandler* target);
		VIRTUAL IDType			GetStandardSignature() CONST;
		VIRTUAL void			ReadFrom(CSTREAM* aStream);
		VIRTUAL void			WriteTo(CSTREAM* aStream) CONST;

				Handle			GetHandlerData();	// used by Ad Lib

	private:
				TEventHandler*	getTarget(void* data);
				void			performAction(TEventHandler* target, void* data);
				void			initHandlers();
		static	TEventHandler*	handleMessage(HandlerMessage message, long msgParam, void* scratch);

				Handle			fHandlerData;
				unsigned short	fHandlers;
				long			fDataSize;
				BOOLEAN*		fFreeFlagPtr;

	friend class THandlerInitMsgCmd;
};


extern void RegisterTHandlerBehavior();

#endif
	
