//----------------------------------------------------------------------------------------
// CAppleGuide_AC.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	Change History:
		09/03/97	bpa		Fixed up non-precompiled includes.
		08/24/97	bpa		Created.
*/

#ifndef __CAppleGuide_AC__
#include "CAppleGuide_AC.h"
#endif

// ACS

#ifndef __Configuration_AC__
#include "Configuration_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

//----------------------------------------------------------------------------------------

static pascal OSErr CoachReplyProc(Rect *pRect, Ptr name, long refCon)
{
	OSErr returnErr = noErr;
	try 
	{
		CRect_AC replyRect;
		CAppleGuide_AC* guide = reinterpret_cast<CAppleGuide_AC*> (refCon);
		guide->DoCoachReply(replyRect, name);
		*pRect = replyRect;
	}
	catch (CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		returnErr = theException.GetError();
	}
	catch (...)
	{
#if qDebug
		ProgramReport_AC("An unhandled exception in AppleGuide CoachReply almost went on to the Toolbox.");
#endif		
	}
	
	return returnErr;
}

//----------------------------------------------------------------------------------------

static pascal OSErr ContextReplyProc(ContextCheckDataPtr_AC pInputData, Size inputDataSize, Ptr *ppOutputData, Size *pOutputDataSize, AGAppInfoHdl hAppInfo)
{
	OSErr returnErr = noErr;
	try 
	{
		CAppleGuide_AC* guide = reinterpret_cast<CAppleGuide_AC*> ((*hAppInfo)->refCon);
		guide->DoContextReply(pInputData, inputDataSize, ppOutputData, *pOutputDataSize, hAppInfo);
	}
	catch (CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		returnErr = theException.GetError();
	}
	catch (...)
	{
#if qDebug
		ProgramReport_AC("An unhandled exception in AppleGuide ContextReply almost went on to the Toolbox.");
#endif		
	}
	return returnErr;
}

//----------------------------------------------------------------------------------------

CAppleGuide_AC::CAppleGuide_AC() :
	fCoachRefNum(0),
	fContextRefNum(0)
{
	// By default, this does nothing.
	// Subclass constructors may install one or both handlers by calling:
	//
	//	SetCoachMarkHandler();
	//	SetContextCheckHandler();
}

//----------------------------------------------------------------------------------------

CAppleGuide_AC::~CAppleGuide_AC()
{
	RemoveCoachMarkHandler();
	RemoveContextCheckHandler();
}

//----------------------------------------------------------------------------------------


void CAppleGuide_AC::SetCoachMarkHandler()
{
	if (HasAppleGuide_AC())
	{
		fCoachReplyUPP = NewCoachReplyProc(CoachReplyProc);
		::ThrowIfNULL_AC(fCoachReplyUPP);
		::ThrowIfOSErr_AC(::AGInstallCoachHandler(fCoachReplyUPP, (long) this, &fCoachRefNum));
	}
}

//----------------------------------------------------------------------------------------
	
void CAppleGuide_AC::SetContextCheckHandler(OSType contextType)
{
	if (HasAppleGuide_AC())
	{
		fContextReplyUPP = NewContextReplyProc(ContextReplyProc);
		::ThrowIfNULL_AC(fContextReplyUPP);
		::ThrowIfOSErr_AC(::AGInstallContextHandler(fContextReplyUPP, contextType, (long) this, &fContextRefNum));
	}
}

//----------------------------------------------------------------------------------------
	
void CAppleGuide_AC::RemoveCoachMarkHandler()
{
	if (HasAppleGuide_AC() && fCoachReplyUPP.get() != NULL)
	{
		AGRemoveCoachHandler(&fCoachRefNum);
		fCoachReplyUPP.reset();
	}
}

//----------------------------------------------------------------------------------------
	
void CAppleGuide_AC::RemoveContextCheckHandler()
{
	if (HasAppleGuide_AC() && fContextReplyUPP.get() != NULL)
	{
		AGRemoveContextHandler(&fContextRefNum);
		fContextReplyUPP.reset();
	}
}

//----------------------------------------------------------------------------------------

void CAppleGuide_AC::DoCoachReply(CRect_AC& replyRect, Ptr /*name*/)
{
#if qDebug
	ProgramBreak_AC("AppleGuide coach reply proc installed, but no implementation.");
#endif
	replyRect = CRect_AC::kZeroRect;
}
	
//----------------------------------------------------------------------------------------

void CAppleGuide_AC::DoContextReply(ContextCheckDataPtr_AC /*pInputData*/, Size /*inputDataSize*/, Ptr* ppOutputData, Size& pOutputDataSize, AGAppInfoHdl /*hAppInfo*/)
{
#if qDebug
	ProgramBreak_AC("AppleGuide context reply proc installed, but no implementation.");
#endif
	*ppOutputData = NULL;
	pOutputDataSize = 0;
}


//----------------------------------------------------------------------------------------

bool HasAppleGuide_AC() 
{
	static bool defined;
	static bool hasAppleGuide;
	if (!defined)
	{
		defined = true;
		hasAppleGuide = GestaltAttribute_AC(gestaltHelpMgrAttr, gestaltAppleGuidePresent);
	}
	return hasAppleGuide;

}


//----------------------------------------------------------------------------------------
// end of file

