//----------------------------------------------------------------------------------------
// UAppleGuideBehavior.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/24/97	bpa		Created.
*/

#ifndef __UAPPLEGUIDEBEHAVIOR__
#include "UAppleGuideBehavior.h"
#endif

// MacApp

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UAPPLEGUIDETYPES__
#include "UAppleGuideTypes.h"
#endif

#ifndef __UAPPLEGUIDECOMMANDS__
#include "UAppleGuideCommands.h"
#endif

//----------------------------------------------------------------------------------------

#if qSegments
class CTempA5
{
public:
	inline CTempA5() { fOldA5 = SetCurrentA5(); }
	
	inline ~CTempA5() { SetA5(fOldA5); }
	
	long fOldA5;
};
#else

class CTempA5 {};

#endif

//----------------------------------------------------------------------------------------

TAppleGuideBehavior::TAppleGuideBehavior()
{
	SetCoachMarkHandler();
	SetContextCheckHandler(gDispatcher->GetCreator());
}

//----------------------------------------------------------------------------------------

TAppleGuideBehavior::~TAppleGuideBehavior()
{
}

//----------------------------------------------------------------------------------------

void TAppleGuideBehavior::DoCoachReply(CRect_AC& replyRect, Ptr name)
{
	IDType viewID1;
	IDType viewID2;
	CViewRect coachFrame;
	long positionCodeH1;
	long positionCodeV1;
	long positionCodeH2;
	long positionCodeV2;
	long addPixels;
	short countParams;
	OSErr err = kAGErrItemNotFound;
	
	// Handle AppleGuide coachmark calls.
	// This method provides access to rectangles for drawing cochmarks.
	
	CTempA5 tempA5();
	
	countParams = sscanf( name, "%4c,%d,%d,%4c,%d,%d",
							(char*) &viewID1,
							&positionCodeH1,
							&positionCodeV1,
							(char*) &viewID2,
							&positionCodeH2,
							&positionCodeV2);
	
	switch (countParams)
	{
		case 1:
			if (FindFrameByViewID(viewID1, coachFrame))
				err = noErr;
			break;
		
		case 6:
			bool foundView1;
			bool foundView2;
			CViewRect viewFrame1;
			CViewRect viewFrame2;
			foundView1 = FindFrameByViewID(viewID1, viewFrame1);
			foundView2 = FindFrameByViewID(viewID2, viewFrame2);
			
			if (foundView1 && foundView2)
			{
				coachFrame[topLeft] = FramePositionToPoint(viewFrame1, positionCodeH1, positionCodeV1);
				coachFrame[botRight] = FramePositionToPoint(viewFrame2, positionCodeH2, positionCodeV2);
				coachFrame.Validate();
				
				// Make sure we have at least 30 x 30 pixels for the red arrow
				
				if (coachFrame.GetLength(hSel) < 30)
				{
					addPixels = (30 - coachFrame.GetLength(hSel)) / 2;
					coachFrame.right += addPixels;
					coachFrame.left	-= addPixels;
				}
				
				if (coachFrame.GetLength(vSel) < 30)
				{
					addPixels = (30 - coachFrame.GetLength(vSel)) / 2;
					coachFrame.bottom += addPixels;
					coachFrame.top -= addPixels;
				}
				
				err = noErr;
			}
			break;
	}
	
	replyRect = coachFrame.ToRect();
	
	ThrowIfOSErr_AC(err);
}
	
//----------------------------------------------------------------------------------------

void TAppleGuideBehavior::DoContextReply(ContextCheckDataPtr_AC pInputData, Size /*inputDataSize*/, Ptr* ppOutputData, Size& pOutputDataSize, AGAppInfoHdl /*hAppInfo*/)
{
	bool result = false;
	Ptr outputPtr = NULL;
	
	// Handle AppleGuide context check calls.
	// This method provides access to the internal state of the application from within AppleGuide.
	
	CTempA5 tempA5();

	if (pInputData)
	{
		switch (pInputData->eventID)
		{
			case kFrontWindowID:
				TWindow* aWindow = gDispatcher->GetActiveWindow(kNoFloaters);				
				if (aWindow)
				{	
					if (aWindow->GetIdentifier() == pInputData->data.identifier)
						result = true;
				}
				break;
			
			case kInModalState:
				result = gDispatcher->InModalState();
				break;
			
			case kViewIDVisible:
				TView* aView = FindViewByID(pInputData->data.identifier);
				if (aView)
					result = true;
				break;
		}
	}
	
	outputPtr = NewPtr(sizeof(short));
	
	if (outputPtr)
	{
		// Set the result and its size.
		
		* (short*) outputPtr = result;
		pOutputDataSize = sizeof(short);
	}
	
	*ppOutputData = outputPtr;
}

//----------------------------------------------------------------------------------------

bool TAppleGuideBehavior::DoScriptCommand(CommandNumber aCommandNumber,
										  TAppleEvent* message,
										  TAppleEvent* reply)
{
	bool handledCommand = false;
	switch (aCommandNumber)
	{
		case cChooseCommandMenu:
			gDispatcher->PostCommand(TH_new TChooseMenuCommand(aCommandNumber, message, reply));
			handledCommand = true;
			break;

		case cClickView:
			gDispatcher->PostCommand(TH_new TClickViewCommand(aCommandNumber, message, reply));
			handledCommand = true;
			break;

		default:
			handledCommand = TBehavior::DoScriptCommand(aCommandNumber, message, reply);
			break;
	}
	return handledCommand;
}

//----------------------------------------------------------------------------------------

TView* TAppleGuideBehavior::FindViewByID(IDType viewID) // static
{
	TWindow* aWindow = NULL;
	TView* aView = NULL;

	aWindow = gDispatcher->GetActiveWindow(kNoFloaters);	
	if (aWindow)
		aView = aWindow->FindSubView(viewID);
	
	return aView;
}

//----------------------------------------------------------------------------------------

bool TAppleGuideBehavior::FindFrameByViewID(IDType viewID, CViewRect& viewFrame)
{
	TView* aView = NULL;
	bool foundAView = false;
	
	aView = FindViewByID(viewID);	
	if (aView)
	{
		// Get the view's extent, and convert to global coordinates

		viewFrame = aView->GetVisibleExtent();
		
		CTempFocus tempFocus(aView);
			
		CPoint_AC viewPoint(viewFrame[topLeft].ToPoint());
		::LocalToGlobal(&viewPoint);
		viewFrame[topLeft] = CViewPoint(viewPoint);
		viewPoint = viewFrame[botRight].ToPoint();
		::LocalToGlobal(&viewPoint);
		viewFrame[botRight] = CViewPoint(viewPoint);
		
		foundAView = true;
	}
	
	return foundAView;
}

//----------------------------------------------------------------------------------------

CViewPoint TAppleGuideBehavior::FramePositionToPoint(CViewRect frame, short positionCodeH, short positionCodeV)
{
	long width;
	long height;
	double multiplier;
	VPoint position;
			
	width = frame.GetLength (hSel);
	multiplier = positionCodeH * 0.25;
	position.h = frame.left + width / 2 + width * multiplier;
	
	height = frame.GetLength (vSel);
	multiplier = positionCodeV * 0.25;
	position.v = frame.top + height / 2 + height * multiplier;
		
	return position;
}

//----------------------------------------------------------------------------------------
// end of file

