//----------------------------------------------------------------------------------------
// UAppleGuideCommands.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/24/97	bpa		Created.
*/

#ifndef __UAPPLEGUIDECOMMANDS__
#include "UAppleGuideCommands.h"
#endif

// MacApp

#ifndef __UAPPLEGUIDEBEHAVIOR__
#include "UAppleGuideBehavior.h"
#endif

#ifndef __UAAPPLEGUIDETYPES__
#include "UAppleGuideTypes.h"
#endif

//----------------------------------------------------------------------------------------

TChooseMenuCommand::TChooseMenuCommand(CommandNumber itsCommandNumber,
											TAppleEvent* message,
											TAppleEvent* /*reply*/) :
	TCommand(itsCommandNumber, GetContext(), kCantUndo, kDoesNotCauseChange, GetContext())
{
	long			actualSize;
	DescType		actualType;
		
	if (message)
	{
		message->ReadParameterPtr (kCommandMenuNumberKey,
											typeLongInteger,
											actualType,
											(Ptr) &fMenuCommand,
											sizeof (CommandNumber),
											actualSize);
	}
}

//----------------------------------------------------------------------------------------

void TChooseMenuCommand::DoIt()
{
	if (gDispatcher)
	{
		TEventHandler* target = gDispatcher->GetTarget();		
		if (target)
			target->HandleMenuCommand(fMenuCommand);
	}
}

//----------------------------------------------------------------------------------------

TClickViewCommand::TClickViewCommand(CommandNumber itsCommandNumber,
									TAppleEvent* message,
									TAppleEvent* /*reply*/) :
	TCommand(itsCommandNumber, GetContext(), kCantUndo, kDoesNotCauseChange, GetContext())
{
	long			actualSize;
	DescType		actualType;
		
	if (message)
	{
		message->ReadParameterPtr(kCommandMenuNumberKey,
											typeType,
											actualType,
											(Ptr) &fViewID,
											sizeof (IDType),
											actualSize);
	}
}

// ------------------------------------------------------------------------------------------

void TClickViewCommand::DoIt()
{
	TView* aView = TAppleGuideBehavior::FindViewByID(fViewID);
	
	if (aView)
	{
		CTempFocus tempFocus(aView);

		CViewRect viewExtent(aView->GetExtent());
		
		CViewRect revealExtent(viewExtent);
		revealExtent.Inset(CPoint_AC(50, 50));
		CViewPoint minToSee = revealExtent[botRight];
		
		aView->RevealRect(revealExtent, minToSee, kRedraw);		// Make sure the view is visible
		
		CViewPoint clickLoc(viewExtent.GetLength(hSel) / 2, viewExtent.GetLength(vSel) / 2);
		CPoint_AC globalClickLoc(clickLoc.ToPoint());
		
		LocalToGlobal(globalClickLoc);
		
		// Simulate a mouse-click
		EventRecord anEvent;
		anEvent.what = mouseDown;
		anEvent.message	= 0;
		anEvent.when = TickCount();
		anEvent.where = globalClickLoc;
		anEvent.modifiers = 0;
		
		CAutoPtr_AC<TToolboxEvent> aToolBoxEvent;
		aToolBoxEvent = TH_new TToolboxEvent(GetContext(), &anEvent);
			
		clickLoc = aView->LocalToRootView(clickLoc);
		TWindow* aWindow = aView->GetWindow();
			
		if (aWindow)
			aWindow->HandleMouseDown(clickLoc, aToolBoxEvent, CPoint(0, 0), kFirstClick_dragOrClick);
	}
}

//----------------------------------------------------------------------------------------
// end of file

