//----------------------------------------------------------------------------------------
// UDocument.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

// MacApp

//	#ifndef __UAPPLICATION__
//	#include "UApplication.h"
//	#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USCRIPTABLEOBJECT__
#include "UScriptableObject.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

//========================================================================================
// static data members
//========================================================================================
short TDocument::gNumUntitled = 1;							// call the first document "Untitled-1"

//========================================================================================
// CLASS TRevertCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TRevertCommand::TRevertCommand:
//----------------------------------------------------------------------------------------

TRevertCommand::TRevertCommand(TDocument* itsDocument)
	:	TCommand(cRevert, itsDocument, kCantUndo, kDoesNotCauseChange, itsDocument),

	fChangedDocument(itsDocument),
	fInProcessP(NULL)
{
	// Ask them if they are sure they want to revert.
	::ThrowIfOSErr_AC(MAInteractWithUser());

	ParamText(fChangedDocument->GetTitle(), gEmptyString, gEmptyString, gEmptyString);

	if (MacAppAlert(phRevert, NULL) != kYesButton)
		::ThrowIfOSErr_AC(userCanceledErr);
}

//----------------------------------------------------------------------------------------
// TRevertCommand::TRevertCommand
//----------------------------------------------------------------------------------------

TRevertCommand::TRevertCommand(TDocument* itsDocument,
							   TAppleEvent* /* message */,
							   TAppleEvent* /* reply */,
							   bool* statusP) :
	TCommand(cRevert, itsDocument, kCantUndo, kDoesNotCauseChange, itsDocument),

	fChangedDocument(itsDocument),
	fInProcessP(statusP)
{
	if (fInProcessP)
		*fInProcessP = true;
}

//----------------------------------------------------------------------------------------
// TRevertCommand destructor
//----------------------------------------------------------------------------------------

TRevertCommand::~TRevertCommand()
{
	if (fInProcessP)
		*fInProcessP = false;
}

//----------------------------------------------------------------------------------------
// TRevertCommand::DoIt:
//----------------------------------------------------------------------------------------

void TRevertCommand::DoIt()
{
	fChangedDocument->RevertDocument();
	fChangedDocument->ShowReverted();
	if (fInProcessP)
		*fInProcessP = false;
}

//----------------------------------------------------------------------------------------
// TRevertCommand::MakeAppleEvent:
//----------------------------------------------------------------------------------------

TAppleEvent* TRevertCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kAEMiscStandards, kAERevert, gServerAddress, kAEWaitReply));

	// Add the document as the direct object
	CTempDesc docDesc;
	fChangedDocument->MakeObjectSpecifier(docDesc, formName);
	theEvent->WriteParameter(keyDirectObject, docDesc);

	return theEvent.release();
}

//========================================================================================
// CLASS TSaveCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSaveCommand::TSaveCommand:
//----------------------------------------------------------------------------------------

TSaveCommand::TSaveCommand(TDocument* itsDocument, EventNumber saveType)
	:	TCommand(saveType, itsDocument,  kCantUndo, kDoesNotCauseChange, NULL),
		fDocument(itsDocument),
	fMessage(NULL),
	fReply(NULL)
{
}

//----------------------------------------------------------------------------------------
// TSaveCommand::TSaveCommand:
//----------------------------------------------------------------------------------------

TSaveCommand::TSaveCommand(	TDocument* itsDocument,
							TAppleEvent* message,
							TAppleEvent* reply,
							EventNumber saveType)
	// This doesn't actually prepare to save anything since we don't have any idea
	// what kind of document you have.
	:	TCommand(saveType, itsDocument, kCantUndo, kDoesNotCauseChange, NULL),
		fDocument(itsDocument),
	fMessage(message),
	fReply(reply)
{
}

//----------------------------------------------------------------------------------------
// TSaveCommand destructor
//----------------------------------------------------------------------------------------

TSaveCommand::~TSaveCommand()
{
	// NOTE: We deliberately do not dispose of fMessage and fReply.
}

//----------------------------------------------------------------------------------------
// TSaveCommand::DoIt:
//----------------------------------------------------------------------------------------

void TSaveCommand::DoIt()
{
	if (fDocument && fDocument->GetCommitOnSave())
		fDocument->CommitLastCommand();
}

//----------------------------------------------------------------------------------------
// TSaveCommand::MakeAppleEvent:
//----------------------------------------------------------------------------------------

TAppleEvent* TSaveCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kAECoreSuite, kAESave, gServerAddress, kAEWaitReply));

	CTempDesc theDocDesc;
	fDocument->MakeObjectSpecifier(theDocDesc, fDocument->GetSpecifierForm());
	theEvent->WriteParameter(keyDirectObject, theDocDesc);

	return theEvent.release();
}

//========================================================================================
// CLASS TCloseDocCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TCloseDocCommand constructor
//----------------------------------------------------------------------------------------

TCloseDocCommand::TCloseDocCommand(CommandNumber itsCommandNumber,
										TDocument* itsDocument) :
	TCommand(itsCommandNumber, itsDocument,  kCantUndo,
				   kDoesNotCauseChange, NULL),

	fDocument(itsDocument),
	fMessage(NULL),
	fReply(NULL),
	fSavingState(kUnknown)
{
	fUseAppleEvent = true;		// We want to default here.

	if (fDocument->IsChanged())
	{
		if (fDocument->GetAskOnClose())
		{
			short poseResult = fDocument->PoseSaveDialog();
			switch (poseResult)
			{
				case kNoButton:
					fSavingState = kDontSave;
					break;

				case kYesButton:
					fSavingState = kDoSave;
					break;

				case cancel:
					::ThrowIfOSErr_AC(userCanceledErr);
					break;
			}
		}
		else
			fSavingState = kDoSave;
	}
	else
		fSavingState = kDontSave;
}

//----------------------------------------------------------------------------------------
// TCloseDocCommand constructor
//----------------------------------------------------------------------------------------

TCloseDocCommand::TCloseDocCommand(TDocument* itsDocument,
										TAppleEvent* message,
										TAppleEvent* reply) :
	TCommand(cClose, itsDocument,  kCantUndo,
						kDoesNotCauseChange, NULL),

	fDocument(itsDocument),
	fMessage(message),
	fReply(reply),
	fSavingState(kDoSave)
{
	// Does the AppleEvent specifically say not to save?
	bool hasSaveOptions = message->HasParameter(keyAESaveOptions);
	if (hasSaveOptions)
	{
		// Get the save options
		DescType theSaveOption = message->ReadEnum(keyAESaveOptions);
		if (theSaveOption == kAEAsk)
			fSavingState = kAskSave;
		else if (theSaveOption == kAENo)
			fSavingState = kDontSave;
	}
	else if (fDocument->GetAskOnClose())
	{
		short poseResult = fDocument->PoseSaveDialog();
		switch (poseResult)
		{
			case kNoButton:
				fSavingState = kDontSave;
				break;

			case kYesButton:
				fSavingState = kDoSave;
				break;

			case cancel:
				::ThrowIfOSErr_AC(userCanceledErr);
				break;
		}
	}
}

//----------------------------------------------------------------------------------------
// TCloseDocCommand destructor
//----------------------------------------------------------------------------------------

TCloseDocCommand::~TCloseDocCommand()
{
	// NOTE: We deliberately do not dispose of fMessage and fReply.
}

//----------------------------------------------------------------------------------------
// TCloseDocCommand::DoIt:
//----------------------------------------------------------------------------------------

void TCloseDocCommand::DoIt()
{
	if (fDocument)
	{
		if (fSavingState != kDontSave && fDocument->GetCommitOnSave())
			fDocument->CommitLastCommand();

		// If there is actually some saving to do it should have already been done.
		fDocument->SetChangeCount(0);	// Force it to not save since it will have
										// already have been saved if desired by user.
		fDocument->Close();
		delete fDocument;
	}
}

//----------------------------------------------------------------------------------------
// TCloseDocCommand::MakeAppleEvent:
//----------------------------------------------------------------------------------------

TAppleEvent* TCloseDocCommand::MakeAppleEvent()
{
	TAppleEvent* returnVal = NULL;

	if (fSavingState != kCancelled)
	{
		CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kAECoreSuite, kAEClose, gServerAddress, kAEWaitReply));
		{
			CTempDesc theDocDesc;
			fDocument->MakeObjectSpecifier(theDocDesc, fDocument->GetSpecifierForm());
			theEvent->WriteParameter(keyDirectObject, theDocDesc);
		}
		// What kind of saving is desired.
		DescType theSaveOption = kAEAsk;
		if (fSavingState == kDoSave)
			theSaveOption = kAEYes;
		else if (fSavingState == kDontSave)
			theSaveOption = kAENo;
		theEvent->WriteEnum(keyAESaveOptions, theSaveOption);

		returnVal = theEvent.release();
	}

	return returnVal;
}

//========================================================================================
// CLASS TDocument
//========================================================================================

MA_DEFINE_CLASS(TDocument);

//----------------------------------------------------------------------------------------
// TDocument constructor
//----------------------------------------------------------------------------------------

TDocument::TDocument() :
	MScriptableObject(cDocument),
	TCommandHandler(gDispatcher),

	fPrintInfo(NULL),
	fChangeCount(0),
	fSavePrintInfo(true),
	fSaveUserSelection(true),
#if qAttachable
	fSaveAttachedScript(true),
#endif
	fSharePrintInfo(true),
	fReopenAlert(false),
	fCommitOnSave(true),
	fAskOnClose(true),
	fIsGhostDocument(false)
{
	gDispatcher->AddDocument(this);
}

//----------------------------------------------------------------------------------------
// TDocument::Free:
//----------------------------------------------------------------------------------------

TDocument::~TDocument()
{
	gDispatcher->DeleteDocument(this);

	for  (TemplateIterator_AC<TWindow*> iter1(&fWindowList); iter1; ++iter1)
		delete iter1.Current();

	for  (TemplateIterator_AC<TView*> iter2(&fViewList); iter2; ++iter2)
		delete iter2.Current();

	// Always drop my reference
	if (fSharePrintInfo)
		TPrintHandler::gPrintHandler->DeleteIfPrintInfo(fPrintInfo);
	fPrintInfo = NULL;
}

//----------------------------------------------------------------------------------------
// TDocument::AddView:
//----------------------------------------------------------------------------------------

void TDocument::AddView(TView* aView)
{
	// Protect against double installation and keep in synch with window list
	if (!fViewList.GetIdentityItemNo(aView))
		fViewList.Insert(aView);

	// Also insert this into the window list if it is a window.
	if (dynamic_cast<TWindow*>(aView)
		&& !fWindowList.GetIdentityItemNo(aView))
			fWindowList.Insert(aView);
}

//----------------------------------------------------------------------------------------
// TDocument::AddWindow:
//----------------------------------------------------------------------------------------

void TDocument::AddWindow(TWindow* aWindow)
{
	// Protect against double installation and keep in synch with window list
	// doesn't already exist in list

	if (!fWindowList.GetIdentityItemNo(aWindow))
		fWindowList.Insert(aWindow);

	// Also insert this into the view list.
	if (!fViewList.GetIdentityItemNo(aWindow))
		fViewList.Insert(aWindow);
}

//----------------------------------------------------------------------------------------
// TDocument::FindDocument:
//----------------------------------------------------------------------------------------

bool TDocument::FindDocument(TFile* /*aFile*/)
{
	return false;
}

//----------------------------------------------------------------------------------------
// TDocument::AttachPrintHandler:
//----------------------------------------------------------------------------------------

void TDocument::AttachPrintHandler(TPrintHandler* itsPrintHandler)
{
	if (itsPrintHandler)
		itsPrintHandler->fDocument = this;

	AddBehavior(TH_new TPrintMenuBehavior(itsPrintHandler));
	if (!fPrintInfo && fSharePrintInfo)
		fPrintInfo = itsPrintHandler->GetPrintInfo();
}

//----------------------------------------------------------------------------------------
// TDocument::DetachPrintHandler:
//----------------------------------------------------------------------------------------

void TDocument::DetachPrintHandler(TPrintHandler* itsPrintHandler)
{
	TBehavior* itsManager = itsPrintHandler->GetManager();
	if (itsManager)
	{
		RemoveBehavior(itsManager);
		delete itsManager;
	}
}

//----------------------------------------------------------------------------------------
// TDocument::Changed:
//----------------------------------------------------------------------------------------

void TDocument::Changed(ChangeID_AC theChange, void* changeData)	// override
{
	switch (theChange)
	{
		case cUndo:
			--fChangeCount;
			break;

		case mClosed:
			//don't want to up the the changed count on closing.
			break;

		default:
			// protect from rollover (it goes negative). If your document has
			// this many changes (over 2 billion) you are truly sick!
			SetChangeCount(Max_AC(GetChangeCount() + 1, 1));
			break;
	}
	MDependable_AC::Changed(theChange, changeData);	// Notify dependents
}

//----------------------------------------------------------------------------------------
// TDocument::OpenWindowCount:
//----------------------------------------------------------------------------------------

short TDocument::OpenWindowCount()
{
	short openDocWindows = 0;

	// See how many open windows this document has
	for (CWindowIterator iter(this); iter.Current(); ++iter)
		if (iter->CountAsOpen())
			++openDocWindows;

	return openDocWindows;
}

//----------------------------------------------------------------------------------------
// TDocument::CloseWindow:
//----------------------------------------------------------------------------------------

void TDocument::CloseWindow(TWindow* aWindow)
{
	// The default behavior is: if the window's fClosesDocument is true
	// OR if this is the document's last open window then close the
	// document, otherwise just close the window.  In either case
	// the action must be recordable.

	if (aWindow && (aWindow->GetDocument() == this))	// free window
	{
		if (aWindow->GetClosesDocument() ||
			(OpenWindowCount() <= 1
			 && !aWindow->GetFloats()
			 && aWindow->IsShown()))
			DoClose(cClose);			// The window will be closed and freed
		else								// as a side effect
			aWindow->DoClose(cClose);
	}
}

//----------------------------------------------------------------------------------------
// TDocument::Close:
//----------------------------------------------------------------------------------------

void TDocument::Close()
{
	Changed(mClosed,this);

	// Must never be called for a document related to a view in the Clipboard.
	for (CWindowIterator iter(this); iter.Current(); ++iter)
	{
		iter->Close();
		if (iter->GetFreeOnClosing())
			delete iter.Current();
	}
}

//----------------------------------------------------------------------------------------
// TDocument::CloseAndFree:
//----------------------------------------------------------------------------------------

void TDocument::CloseAndFree()
{
#if qDebug
	::ProgramBreak_AC("TDocument::CloseAndFree is deprecated: please call Close() and then if necessary free the object");
#endif

	Close();
	delete this;
}

//----------------------------------------------------------------------------------------
// TDocument::MakeCloseCommand:
//----------------------------------------------------------------------------------------
// Note that whatever command you chose to make here must be a descendant
// of TCloseDocCommand.

TCloseDocCommand* TDocument::MakeCloseCommand(CommandNumber itsCommandNumber)
{
	return TH_new TCloseDocCommand(itsCommandNumber, this);
}

TCloseDocCommand* TDocument::MakeCloseCommand(TAppleEvent* message, TAppleEvent* reply)
{
	return TH_new TCloseDocCommand(this, message, reply);
}

//----------------------------------------------------------------------------------------
// TDocument::DoClose:
//----------------------------------------------------------------------------------------
void TDocument::DoClose(CommandNumber /* aCommand */, bool useAppleEvent)
{
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	CAutoPtr_AC<TCloseDocCommand> aCloseCommand = MakeCloseCommand();

	aCloseCommand->SetUseAppleEvent(useAppleEvent);

	PostCommand(aCloseCommand);

	aCloseCommand.release();
}

//----------------------------------------------------------------------------------------
// TDocument::DeleteView:
//----------------------------------------------------------------------------------------

void TDocument::DeleteView(TView* viewToDelete)
{
	fViewList.Delete(viewToDelete);

	// Make sure the lists are in synch.
	fWindowList.Delete(viewToDelete);
}

//----------------------------------------------------------------------------------------
// TDocument::DeleteWindow:
//----------------------------------------------------------------------------------------

void TDocument::DeleteWindow(TWindow* windowToDelete)
{
	fWindowList.Delete(windowToDelete);

	// Make sure the lists are in synch.
	fViewList.Delete(windowToDelete);
}

//----------------------------------------------------------------------------------------
// TDocument::DoInitialState: Called for 'New' && 'Revert' [to blank] commands && for
// default open tool icon
//----------------------------------------------------------------------------------------

void TDocument::DoInitialState()
{
}

//----------------------------------------------------------------------------------------
// TDocument::DoMakeViews:
//
// 	E X A M P L E
//	{
//		TYOURView*	aYOURView;
//		aYOURView = new TYourView;
//		aYOURView->IYOURView(this, YOURExtentRect);
//		return aYOURView;
//	}
//----------------------------------------------------------------------------------------

#if qTemplateViews
void TDocument::DoMakeViews(bool forPrinting)
#else
void TDocument::DoMakeViews(bool /*forPrinting*/)
#endif
{
#if qTemplateViews
	TView * aView = NULL;

	if (forPrinting)						// Don't need window when Finder printing.
		aView = TViewServer::fgViewServer->DoCreateViews(this, NULL, kDefaultViewID, gZeroVPt);
	else
		aView = TViewServer::fgViewServer->NewTemplateWindow(kDefaultWindowID, this);

	InstallIfPrintHandler(TPrintHandler::gPrintHandler, aView);
#endif
}

//----------------------------------------------------------------------------------------
// TDocument::DoPostMakeViews:
//----------------------------------------------------------------------------------------

void TDocument::DoPostMakeViews(bool forPrinting)
{
	if (!forPrinting)
		ShowWindows();

	TDesignator* userSelection = GetUserSelection();
	if (userSelection)
		RevealSelection(userSelection);
}

//----------------------------------------------------------------------------------------
// TDocument::DoMenuCommand:
//----------------------------------------------------------------------------------------

void TDocument::DoMenuCommand(CommandNumber aCommandNumber)
{
	//====================================================================================
    // Some commands will be posted to perform actions that must _ALWAYS_ be available.
    // The allocation cannot be allowed to fail. So we do a temp allocation which by
    // definition cannot be allowed to fail. This strategy is used wherever we want to use
    // command objects but don't want to leave the user twisting in the breeze. NOTE:
    // Don't forget to allow for this memory in your mem! resource if you copy this style
    // in your own code.
    //====================================================================================

	switch (aCommandNumber)
	{
		case cUndo:
			if (!gDispatcher->DoUndoRedo(this, aCommandNumber, this))
				TCommandHandler::DoMenuCommand(aCommandNumber);
			break;

		case cSave:
			DoSave();
			break;

		case cSaveAs:
			DoSaveAs();
			break;

		case cSaveCopy:
			DoSaveCopy();
			break;

		case cRevert:
			DoRevert();
			break;

		default:
			TCommandHandler::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDocument::FreeData:
//----------------------------------------------------------------------------------------

void TDocument::FreeData()
{
}

//----------------------------------------------------------------------------------------
// TDocument::FreeFromClipboard:
//----------------------------------------------------------------------------------------

void TDocument::FreeFromClipboard()
{
	DeleteWindow(gClipboardMgr->fClipWindow);

	delete this;
}

//----------------------------------------------------------------------------------------
// TDocument::GetIsGhostDocument:
//----------------------------------------------------------------------------------------

bool TDocument::GetIsGhostDocument() const
{
	return fIsGhostDocument;
}

//----------------------------------------------------------------------------------------
// TDocument::SetIsGhostDocument:
//----------------------------------------------------------------------------------------

void TDocument::SetIsGhostDocument(const bool newIsGhostDocument)
{
	fIsGhostDocument = newIsGhostDocument;
}

//----------------------------------------------------------------------------------------
// TDocument::RegainControl:
//----------------------------------------------------------------------------------------

void TDocument::RegainControl()
{
}

//----------------------------------------------------------------------------------------
// TDocument::GetChangeCount:
//----------------------------------------------------------------------------------------

long TDocument::GetChangeCount()
{
	return fChangeCount;
}

// Added for 3.5:
//----------------------------------------------------------------------------------------
// TDocument::IsChanged:
//----------------------------------------------------------------------------------------

bool TDocument::IsChanged()
{
	return GetChangeCount() > 0;
}

//----------------------------------------------------------------------------------------
// TDocument::DoWriteData:
//----------------------------------------------------------------------------------------

void TDocument::DoWriteData(const OSType /* aScrapType */,
								   TDesignator* /* aDesignator */,
								   CStream_AC* /* aStream */)
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TDocument::DoReadData:
//----------------------------------------------------------------------------------------

void TDocument::DoReadData(const OSType /* aScrapType */,
								  TDesignator* /* aDesignator */,
								  CStream_AC* /* aStream */,
								  long /* count */)
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TDocument::GetUserSelection:
//----------------------------------------------------------------------------------------

TDesignator* TDocument::GetUserSelection()
{
	return fUserSelection;
}

//----------------------------------------------------------------------------------------
// TDocument::SetUserSelection:
//----------------------------------------------------------------------------------------

void TDocument::SetUserSelection(TDesignator* newSelection)
{
	if (newSelection != fUserSelection)
		fUserSelection = newSelection;
}

//----------------------------------------------------------------------------------------
// TDocument::UserSelectionChanged:
//----------------------------------------------------------------------------------------

void TDocument::UserSelectionChanged(TView* /* changedView */)
{
}

//----------------------------------------------------------------------------------------
// TDocument::RevealSelection:
//----------------------------------------------------------------------------------------

void TDocument::RevealSelection(TDesignator* /* designator */)
{
	// Default behavior: select the first window of the application.
	// Call TDocument::RevealSelection() to get this default behavior if you need it.

	TWindow* window = static_cast<TWindow*>(fWindowList.First());
	if (window && !window->IsActive())
		window->Select();
}

//----------------------------------------------------------------------------------------
// TDocument::RevealUndoRedo:
//----------------------------------------------------------------------------------------

void TDocument::RevealUndoRedo(TCommand* command) // override
{
	TCommandHandler::RevealUndoRedo(command);

	TWindow* window = static_cast<TWindow*>(fWindowList.First());
	if (window)
		window->Select();
}

//----------------------------------------------------------------------------------------
// TDocument::OpenAgain:
//----------------------------------------------------------------------------------------

void TDocument::OpenAgain(CommandNumber /*commandNumber*/,
						  TDocument* /*document*/)
{
	if (fReopenAlert)
	{
		OSErr interactErr = MAInteractWithUser();
		if (interactErr == noErr)
		{
			ParamText(fTitle, gEmptyString, gEmptyString, gEmptyString);
			StdAlert(phReopenDoc);
		}
		else if (interactErr != errAENoUserInteraction)
			::ThrowIfOSErr_AC(interactErr);
	}

	TWindow* window = static_cast<TWindow*>(fWindowList.First());
	if (window)
		window->Select();
}

//----------------------------------------------------------------------------------------
// TDocument::PoseSaveDialog:
//----------------------------------------------------------------------------------------

short TDocument::PoseSaveDialog()
{
	short returnVal = kNoButton;

	if (IsChanged())	// 3.5
	{
		::ThrowIfOSErr_AC(MAInteractWithUser());

		CStr255_AC reason(kIDBuzzString, gDispatcher->GetDone() ? bzQuitting : bzClosing);

		CStr255_AC apName;
		gDispatcher->GetApplicationName(apName);	// Get the application name
		ParamText(fTitle, reason, apName, gEmptyString);
		returnVal = MacAppAlert(phSaveChanges, NULL);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDocument::ReadDocument:
//----------------------------------------------------------------------------------------

void TDocument::ReadDocument(bool)
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TDocument::ReadStationery:
//----------------------------------------------------------------------------------------

void TDocument::ReadStationery(TFile* /*itsNewFile*/)
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TDocument::RevertDocument:
//----------------------------------------------------------------------------------------

void TDocument::RevertDocument()
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TDocument::Abandon:
//----------------------------------------------------------------------------------------

void TDocument::Abandon()
{
	// If your document needs to do some cleanup when its being abandoned then
	// put some code in an override of this method
}

//----------------------------------------------------------------------------------------
// TDocument::GetSaveLocation
//----------------------------------------------------------------------------------------

void TDocument::GetSaveLocation(CommandNumber /*itsCommandNumber*/, CAEDesc& /*theSaveDesc*/)
// Creates an CAEDesc describing where the document would be saved
{
}

//----------------------------------------------------------------------------------------
// TDocument::DoSave
//----------------------------------------------------------------------------------------

void TDocument::DoSave()
{
	// Generate a save document command.
	CTempAllocation	tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	TSaveCommand* aSaveCommand = TH_new TSaveCommand(this, cSave);

	aSaveCommand->SetUseAppleEvent(true);
	PostCommand(aSaveCommand);
}

//----------------------------------------------------------------------------------------
// TDocument::DoSaveAs
//----------------------------------------------------------------------------------------

void TDocument::DoSaveAs()
{
	// We don't really know what "saving as" means for a file-less document
	// so we're just going to call Save here.  Perhaps we ought to remove
	// Save As and Save A Copy from TDocument and only provide it at the
	// TFileBasedDocument level.
	DoSave();
}

//----------------------------------------------------------------------------------------
// TDocument::DoSaveCopy
//----------------------------------------------------------------------------------------

void TDocument::DoSaveCopy()
{
	// See note in implementation of DoSaveAs.
	DoSave();
}

//----------------------------------------------------------------------------------------
// TDocument::DoRevert
//----------------------------------------------------------------------------------------

void TDocument::DoRevert()
{
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	TRevertCommand* aRevertCommand = TH_new TRevertCommand(this);

	aRevertCommand->SetUseAppleEvent(true);
	PostCommand(aRevertCommand);
}

//----------------------------------------------------------------------------------------
// TDocument::SaveDocument:
//----------------------------------------------------------------------------------------

void TDocument::SaveDocument(CommandNumber)
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TDocument::SaveAgain:
//----------------------------------------------------------------------------------------

void TDocument::SaveAgain(CommandNumber,
								 TDocument* savingDoc)
{
	// Don't save the file if another one of the same name is already open.
	if (savingDoc != this)
		::ThrowException_AC(errSaveAgain, 0);
}

//----------------------------------------------------------------------------------------
// TDocument::GetTitle:
//----------------------------------------------------------------------------------------

CStr255_AC TDocument::GetTitle() const
{
	return fTitle;
}

//----------------------------------------------------------------------------------------
// TDocument::SetTitle:
//----------------------------------------------------------------------------------------

void TDocument::SetTitle(const CStr255_AC& aTitle)
{
	fTitle = aTitle;

	for (CWindowIterator iter(this); iter.Current(); ++iter)
		iter->SetTitleForDoc(aTitle);
}

//----------------------------------------------------------------------------------------
// TDocument::SetChangeCount:
//----------------------------------------------------------------------------------------

void TDocument::SetChangeCount(long newChangeCount)
{
	fChangeCount = newChangeCount;
}

//----------------------------------------------------------------------------------------
// TDocument::ShowReverted:
//----------------------------------------------------------------------------------------

void TDocument::ShowReverted()
{
	for (CWindowIterator iter(this); iter.Current(); ++iter)
		iter->ShowReverted();
}

//----------------------------------------------------------------------------------------
// TDocument::ShowWindows:
//----------------------------------------------------------------------------------------

void TDocument::ShowWindows()
{
	// Make the windows open from back to front
	for (CWindowIterator iter(this, kIterateBackward_AC); iter.Current(); ++iter)
		if (iter->GetOpenInitially())
			iter->Open();
}

//----------------------------------------------------------------------------------------
// TDocument::UntitledName:
//----------------------------------------------------------------------------------------

void TDocument::UntitledName(CStr255_AC& noName)
{
	short preInsert;
	short constChars;
	CStr255_AC num;

	if (gNumUntitled == 1) {
		GetIndString(noName, kIDBuzzString, bzFirstUntitled);
			++gNumUntitled;
	}
	else {
		GetIndString(noName, kIDBuzzString, bzUntitled);
		NumToString(gNumUntitled, num);
		if (ParseTitleTemplate(noName, preInsert, constChars) &&
		  SubstituteInTitle(noName, num, preInsert, constChars))
			++gNumUntitled;
	}
}

//----------------------------------------------------------------------------------------
// Scripting Support
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TDocument::GetObjectProperty:
//----------------------------------------------------------------------------------------

bool TDocument::GetObjectProperty(CAEDesc& thePropertyValue,
									 DescType whichProperty,
									 const CAEDesc& desiredType)
{
	bool hasProperty = true;

	try
	{
		switch (whichProperty)
		{
			case pIsModified:
				thePropertyValue.PutBoolean(IsChanged());
				break;

			case pName:
				thePropertyValue.PutString(GetTitle());
				break;

			case pIndex:
				{
					// locate the nth window and return its associated document
					hasProperty = false;
					MScriptableObject *myContainer = GetObjectsContainer();
					long numWindows = myContainer->CountContainedObjects(cWindow);
					for (long index = 1; index <= numWindows; ++index)
					{
						TWindow *indexedWindow =
								dynamic_cast_AC(TWindow*,myContainer->GetIndContainedObject(cWindow, index));

						CTempDesc objectDesc;
						objectDesc.PutObject(indexedWindow->GetDocument());
						if (CompareScriptableObjects(kAEEquals, objectDesc))
						{
							thePropertyValue.PutLong(index);
							hasProperty = true;
							break;
						}
					}
				}
				break;

			default:
				hasProperty = MScriptableObject::GetObjectProperty(thePropertyValue, whichProperty, desiredType);
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		hasProperty = false;
	}

	return hasProperty;
}

//----------------------------------------------------------------------------------------
// TDocument::GetSetPropertyInfo:
//----------------------------------------------------------------------------------------

void TDocument::GetSetPropertyInfo(DescType whichProperty,
								CommandNumber& cmdNum,
								bool& canUndo,
								bool& causesChange,
								TCommandHandler* &theContext)
{
	MScriptableObject::GetSetPropertyInfo(whichProperty, cmdNum, canUndo,
		causesChange, theContext);

	if (whichProperty == pName)
	{
		canUndo = false;
		causesChange = false;
	}
}

//----------------------------------------------------------------------------------------
// TDocument::SetObjectProperty:
//----------------------------------------------------------------------------------------

void TDocument::SetObjectProperty(const CAEDesc& thePropertyValue,
											 DescType whichProperty)
{
	switch (whichProperty)
	{
		case pName:
			{
				CStr255_AC theDocName;
				thePropertyValue.GetString(theDocName);
				SetTitle(theDocName);
			}
			break;

		case pIsModified:
			::ThrowIfOSErr_AC(errAECantSetReadOnly);
			break;

		default:
			MScriptableObject::SetObjectProperty(thePropertyValue, whichProperty);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDocument::GetContainedObject:
//----------------------------------------------------------------------------------------

MScriptableObject* TDocument::GetContainedObject(DescType desiredType,
											  DescType selectionForm,
											  const CAEDesc& selectionData)
{
	MScriptableObject * result = NULL;
	CAEDesc theSelectionData = selectionData;
	if ((desiredType == cProperty) && (theSelectionData.GetType() == pSelection))
		result = GetUserSelection();
	else
		result = MScriptableObject::GetContainedObject(desiredType, selectionForm, selectionData);
	return result;
}

//----------------------------------------------------------------------------------------
// TDocument::DoAESetData:
//----------------------------------------------------------------------------------------

void TDocument::DoAESetData(TAppleEvent* /* message */,
							TAppleEvent* /* reply */)
{
	// Handles the SetData AppleEvent from the Core Suite.
	// TDocument overrides this method so it can handle this event
	// and avoid an event not handled error, but
	// it doesn't actually do anything with the data.
}

//----------------------------------------------------------------------------------------
// TDocument::DoAEClose:
//----------------------------------------------------------------------------------------

void TDocument::DoAEClose(TAppleEvent* message,
						  TAppleEvent* reply)
{
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	TCloseDocCommand* aCloseCommand = MakeCloseCommand(message, reply);

	// Process the command immediately rather than posting it.
	// Process will delete command if it is not undoable
	aCloseCommand->Process();
}

//----------------------------------------------------------------------------------------
// TDocument::DoAERevert
//----------------------------------------------------------------------------------------

void TDocument::DoAERevert(TAppleEvent*	message,
						   TAppleEvent* reply)
{
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	bool inProcess = true;

	PostCommand(TH_new TRevertCommand(this, message, reply, &inProcess));

	// Now we're going modal until we're sure the command
	// has completed successfully.
	while (inProcess)
		gDispatcher->PollEvent(kAllowApplicationToSleep);
}

//----------------------------------------------------------------------------------------
// TDocument::DoAESave
//----------------------------------------------------------------------------------------

void TDocument::DoAESave(TAppleEvent* message, TAppleEvent* reply)
{
	// Generate a save document command.
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	TSaveCommand* aSaveCommand = TH_new TSaveCommand(this, message, reply, cSave);

	// Process the command immediately rather than posting it.
	aSaveCommand->Process();
}

//----------------------------------------------------------------------------------------
// TDocument::DoScriptCommand
//----------------------------------------------------------------------------------------

void TDocument::DoScriptCommand(CommandNumber 	aCommandNumber,
								TAppleEvent* 	message,
								TAppleEvent* 	reply)
{
	switch (aCommandNumber)
	{
		case cAEUndo:
		case cAERedo:
			if (CommandEnabled(cUndo))
				gDispatcher->UndoRedoInContext(this, aCommandNumber);
			break;

		case cAERevert:
			DoAERevert(message, reply);
			break;

		default:
			MScriptableObject::DoScriptCommand(aCommandNumber, message, reply);
			break;
	}
}

//----------------------------------------------------------------------------------------
// End of UDocument.cp

