//----------------------------------------------------------------------------------------
// CException_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CException_AC__
#define __CException_AC__

#ifndef __Core_AC__
#error "Include CoreAC.h to get CException_AC.h"
#endif

#ifndef __CoreSwitches_AC__
#include "CoreSwitches_AC.h"
#endif

#ifndef __ConditionalMacros_AC__
#include "ConditionalMacros_AC.h"
#endif

#ifndef __Types_AC__
#include "Types_AC.h"
#endif

#if defined(__MRC__)
#	ifndef __STDEXCEPT_H
#	include <stdexcept.h>
#	endif 
#elif defined(_MSC_VER)
#	include <exception>
#else	// defined(__MRC__)
#	ifndef _EXCEPTION_
#	include <exception.h>
#	endif
#endif 
	// defined(__MRC__)

#ifndef __climits__
#include <Limits.h>
#endif

//----------------------------------------------------------------------------------------
// Max and min error numbers
//----------------------------------------------------------------------------------------

enum
{
	minErr = SHRT_MIN,
	maxErr = SHRT_MAX,
	minACErr = LONG_MIN,
	maxACErr = LONG_MAX
};

//----------------------------------------------------------------------------------------
// class CException_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CException_AC	: public exception
{
public:
	typedef long Message;

	CException_AC();

	CException_AC(CError_AC inError, Message inMessage = 0);

	CException_AC(const CException_AC& inEx);

	virtual ~CException_AC();
	
	// operators
	CException_AC& operator = (const CException_AC& inEx);
	
	bool operator == (const CException_AC& inEx) const ;

	bool operator != (const CException_AC& inEx) const ;

	// error
	CError_AC GetError() const;
	
	virtual void SetError(CError_AC inError);
	
	// overides 	
	virtual const char* what() const throw();

	// optional message
	Message GetMessage() const;

	void SetMessage(Message inMessage);

	void SetMessage(short lowWord, short highWord);
	
	static long BuildMessage(short lowWord, short highWord);

// Debug Methods

#if qDebugExceptions
	void SetThrownFrom(const char* inFile, short inLine);
	
	const char* GetFile() const;
	
	short GetLine() const;

#endif
			
#if qDebugExceptions >= qDebugLevelTwo_AC

	virtual void DoThrowMessage();

	virtual void DoCatchMessage(const char* fileName,
								short lineNum,
								bool breakInDebugger = false);
		// keep going by default.

	static void SetDoMessageOnThrow(bool toTrue = true)
	{	fgDoThrowMessage = toTrue; }
	
	static bool GetDoMessageOnThrow()
	{	return fgDoThrowMessage; }

	static void SetDoMessageOnCatch(bool toTrue = true)
	{	fgDoCatchMessage = toTrue; }
	
	static bool GetDoMessageOnCatch()
	{	return fgDoCatchMessage; }

	class EXPORT_AC CTempDisableThrowMessage_AC
	{
	public:
		CTempDisableThrowMessage_AC()
			: fPrev(CException_AC::GetDoMessageOnThrow())
		{	CException_AC::SetDoMessageOnThrow(false); }
		~CTempDisableThrowMessage_AC()
		{	CException_AC::SetDoMessageOnThrow(fPrev); }

	private:
		bool fPrev;

	};

	class EXPORT_AC CTempDisableCatchMessage_AC
	{
	public:
		CTempDisableCatchMessage_AC()
			: fPrev(CException_AC::GetDoMessageOnCatch())
		{	CException_AC::SetDoMessageOnCatch(false); }
		~CTempDisableCatchMessage_AC()
		{	CException_AC::SetDoMessageOnCatch(fPrev); }

	private:
		bool fPrev;

	};

#endif

// terminate_handler methods

	static terminate_handler SetTerminateHandler( terminate_handler inHandler);
		// Sets default terminate_handler. Returns the previous handler.
	
	static void ResetTerminateHandler();
		// This sets the terminate_handler to DefaultTerminateHandler and
		// saves the previous terminate_handler in fgPreviousTerminateHandler.
	
	static terminate_handler GetPreviousTerminateHandler();
		// This gets the terminate_handler that was installed before CExceptions_AC
		// this is provided because we need to chain calls to the terminate handler.

private:

	static void DefaultTerminateHandler();

protected:
	Message 	fMessage;
	CError_AC 	fError;

	static terminate_handler fgPreviousTerminateHandler;

// For developers migrating from MacApp FailInfo, these data members
// preserve the old names and can be accessed by turning on qBackwardCompatible.	
#if qBackwardCompatible
public:	
	Message& message;
	CError_AC& error;
#endif

// Debug Data
#if qDebugExceptions
protected:
	char 			fFile[256];
	short			fLine;
#endif	
#if qDebugExceptions >= qDebugLevelTwo_AC
	static 	bool	fgDoThrowMessage;	
	static 	bool	fgDoCatchMessage;	
#endif

};


//----------------------------------------------------------------------------------------
// Inline methods
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// CException_AC::operator =
//----------------------------------------------------------------------------------------

inline CException_AC& CException_AC::operator = (const CException_AC& inEx) 
{ 	
	fMessage = inEx.fMessage; 
	fError = inEx.fError;
	
#if qDebugExceptions
	SetThrownFrom(inEx.fFile, inEx.fLine);
#endif

	return *this; 
}

//----------------------------------------------------------------------------------------
// default ctor
//----------------------------------------------------------------------------------------

inline CException_AC::CException_AC() 
	: fMessage(0),
	fError(0) 
#if qBackwardCompatible	
	, message(fMessage),
	error(fError)
#endif
{
#if qDebugExceptions
	SetThrownFrom("",0);
#endif
}

//----------------------------------------------------------------------------------------
// ctor
//----------------------------------------------------------------------------------------


inline CException_AC::CException_AC(CError_AC inError, Message inMessage) 
	: fMessage(inMessage),
	fError(inError)  
#if qBackwardCompatible
	, message(fMessage),
	error(fError)
#endif
{
#if qDebugExceptions
	SetThrownFrom("",0);
#endif
}

//----------------------------------------------------------------------------------------
// copy ctor
//----------------------------------------------------------------------------------------

inline CException_AC::CException_AC(const CException_AC& inEx) 
#if qBackwardCompatible
	: message(fMessage),
	error(fError)
#endif
{ 
	fMessage = inEx.fMessage; 
	fError = inEx.fError;
	
#if qDebugExceptions
	SetThrownFrom(inEx.fFile, inEx.fLine);
#endif
}

//----------------------------------------------------------------------------------------
// dtor
//----------------------------------------------------------------------------------------

inline CException_AC::~CException_AC() 
{
}

//----------------------------------------------------------------------------------------
// CException_AC::what
//----------------------------------------------------------------------------------------

inline const char* CException_AC::what() const throw() 
{ 	
	return "CException"; 
}

//----------------------------------------------------------------------------------------
// CException_AC::GetMessage
//----------------------------------------------------------------------------------------

inline CException_AC::Message CException_AC::GetMessage() const 
{ 	
	return fMessage; 
}

//----------------------------------------------------------------------------------------
// CException_AC::SetMessage
//----------------------------------------------------------------------------------------

inline void CException_AC::SetMessage(Message inMessage) 
{ 
	fMessage = inMessage; 
}

//----------------------------------------------------------------------------------------
// bool CException_AC::operator ==
//----------------------------------------------------------------------------------------

inline bool CException_AC::operator == (const CException_AC& inEx) const
{
	return fMessage == inEx.fMessage;
}

//----------------------------------------------------------------------------------------
// CException_AC::operator !=
//----------------------------------------------------------------------------------------

inline bool CException_AC::operator != (const CException_AC& inEx) const 
{
	return fMessage != inEx.fMessage;
}

//----------------------------------------------------------------------------------------
// CException_AC::GetError
//----------------------------------------------------------------------------------------

inline CError_AC CException_AC::GetError() const 
{ 
	return fError; 
}

//----------------------------------------------------------------------------------------
// Exception_AC::SetError
//----------------------------------------------------------------------------------------

inline void CException_AC::SetError(CError_AC inError) 
{ 
	fError = inError; 
}


//------------------------------------------------------------------------------------
// DEBUG METHODS
//------------------------------------------------------------------------------------
#if qDebugExceptions

inline const char* CException_AC::GetFile() const 
{
	return fFile;
}

inline short CException_AC::GetLine() const
{
	return fLine;
}

// helper macros

//----------------------------------------------------------------------------------------
// ThrowException_AC
//----------------------------------------------------------------------------------------

#define ThrowException_AC(err, msg) DebugThrowException_AC(err, msg,__FILE__, __LINE__ )
EXPORT_AC void DebugThrowException_AC(	CError_AC inErr, CException_AC::Message msg,  
										const char *inFileName, short inLineNum);

//----------------------------------------------------------------------------------------
// ThrowIfError_AC
//----------------------------------------------------------------------------------------

#define ThrowIfError_AC(e) DebugThrowIfError_AC((e), __FILE__, __LINE__ )
EXPORT_AC void DebugThrowIfError_AC( CError_AC inErr, const char* inFile, short inLine);

//----------------------------------------------------------------------------------------
// ThrowError_AC
//----------------------------------------------------------------------------------------

#define ThrowError_AC(e) DebugThrowError_AC((e),  __FILE__, __LINE__ )
EXPORT_AC void DebugThrowError_AC( CError_AC inErr, const char* inFile, short inLine);

//----------------------------------------------------------------------------------------
// ThrowIfMemError_AC
//----------------------------------------------------------------------------------------

#define ThrowIfMemError_AC() DebugThrowIfMemError_AC( __FILE__, __LINE__ )
EXPORT_AC void DebugThrowIfMemError_AC( const char* inFile, short inLine);

//----------------------------------------------------------------------------------------
// ThrowIfNULLResource_AC
//----------------------------------------------------------------------------------------

#define ThrowIfNULLResource_AC(r) DebugThrowIfNULLResource_AC(r, __FILE__, __LINE__ )
EXPORT_AC void DebugThrowIfNULLResource_AC(void* r, const char *inFileName, short inLineNum);

//----------------------------------------------------------------------------------------
// ThrowIfOSErr_AC
//----------------------------------------------------------------------------------------

#define ThrowIfOSErr_AC(e) DebugThrowIfOSErr_AC(e, __FILE__, __LINE__ )
EXPORT_AC void DebugThrowIfOSErr_AC(OSErr error, const char *inFileName, short inLineNum);

//----------------------------------------------------------------------------------------
// ThrowIfNonObject_AC
//----------------------------------------------------------------------------------------

#define ThrowIfNonObject_AC(obj) DebugThrowIfNonObject_AC(obj, __FILE__, __LINE__ )
EXPORT_AC void DebugThrowIfNonObject_AC(void* obj, const char *inFileName, short inLineNum);

//----------------------------------------------------------------------------------------
// ThrowIfResError_AC
//----------------------------------------------------------------------------------------

#define ThrowIfResError_AC() DebugThrowIfResError_AC(__FILE__, __LINE__ )
EXPORT_AC void DebugThrowIfResError_AC(const char *inFileName, short inLineNum);

//----------------------------------------------------------------------------------------
// ThrowNewMessage_AC
//----------------------------------------------------------------------------------------

#define ThrowNewMessage_AC(err, old, newMsg) DebugThrowNewMessage_AC(err, old, newMsg, __FILE__, __LINE__ )
EXPORT_AC void DebugThrowNewMessage_AC(	CError_AC inError, 
										CException_AC::Message oldMessage, 
										CException_AC::Message newMessage,
										const char *inFileName, short inLineNum);

//----------------------------------------------------------------------------------------
// ThrowIfNULL_AC
//----------------------------------------------------------------------------------------

#define ThrowIfNULL_AC(ptr) DebugThrowIfNULL_AC((ptr), __FILE__, __LINE__ )
EXPORT_AC void DebugThrowIfNULL_AC( const void* inPtr, const char* inFile, short inLine);

//----------------------------------------------------------------------------------------
// ThrowBadCast
//----------------------------------------------------------------------------------------

// this is a debug only method.

#define ThrowBadCast_AC(castTo, castFrom) DebugThrowBadCast_AC((castTo), (castFrom), __FILE__, __LINE__ )
EXPORT_AC void DebugThrowBadCast_AC(const void* castTo, const void* castFrom, const char* inFile, short inLine);

//----------------------------------------------------------------------------------------
// ThrowIfBadCast_AC
//----------------------------------------------------------------------------------------

#define ThrowIfBadCast_AC(ptr) DebugThrowIfBadCast_AC((ptr), __FILE__, __LINE__ )
inline void DebugThrowIfBadCast_AC( const void* inPtr, const char* inFile, short inLine)
{
	if(!inPtr)
		DebugThrowBadCast_AC("unknown", "unknown", inFile, inLine);
}

#else
//----------------------------------------------------------------------------------------
// NonDebug Functions
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// ThrowIfError_AC
//----------------------------------------------------------------------------------------
	
inline void ThrowIfError_AC(CError_AC inErr, CException_AC::Message msg)
{
	throw CException_AC(inErr, msg);
}

//----------------------------------------------------------------------------------------
// ThrowIfError_AC
//----------------------------------------------------------------------------------------

inline void ThrowIfError_AC( CError_AC inErr ) 
{ 
	if(inErr)
	{
		throw CException_AC(inErr); 
	}
}

//----------------------------------------------------------------------------------------
// ThrowError_AC
//----------------------------------------------------------------------------------------

inline void ThrowError_AC( CError_AC inErr  )
{	
	throw CException_AC(inErr);
}

//----------------------------------------------------------------------------------------
// ThrowNewMessage_AC
//----------------------------------------------------------------------------------------

inline void ThrowNewMessage_AC(	CError_AC inError, 
								CException_AC::Message oldMessage, 
								CException_AC::Message newMessage)
{
	throw CException_AC(inError, oldMessage ? oldMessage : newMessage);
}


//----------------------------------------------------------------------------------------
// ThrowNewMessage_AC
//----------------------------------------------------------------------------------------

inline void ThrowException_AC(	CError_AC inError, CException_AC::Message inMsg)
{
	throw CException_AC(inError, inMsg);
}
//----------------------------------------------------------------------------------------
// Other (implementation has header depenedencies so they're in the .cp file.)
//----------------------------------------------------------------------------------------

EXPORT_AC void ThrowIfMemError_AC();
EXPORT_AC void ThrowIfNULLResource_AC(void* r);
EXPORT_AC void ThrowIfOSErr_AC(OSErr error);
EXPORT_AC void ThrowIfNonObject_AC(void* obj);
EXPORT_AC void ThrowIfResError_AC();
EXPORT_AC void ThrowIfNULL_AC( const void* inPtr);
EXPORT_AC void ThrowIfBadCast_AC( const void* inPtr );

#endif 
	// qDebugExceptions

#if qDebugExceptions >= qDebugLevelTwo_AC

#define DoCatchMessage_AC(ex)				(ex).DoCatchMessage(__FILE__, __LINE__)
#define DoCatchMessageAndStop_AC(ex)		(ex).DoCatchMessage(__FILE__, __LINE__, false)
#define DoThrowMessage_AC(ex)				(ex).DoThrowMessage()

#else

inline void ExceptionMessageStub_AC(CException_AC&) {}
	// this should get optimized out of existance.
	
#define DoCatchMessage_AC(ex)				ExceptionMessageStub_AC((ex))
#define DoCatchMessageAndStop_AC(ex)		ExceptionMessageStub_AC((ex))
#define DoThrowMessage_AC(ex)				ExceptionMessageStub_AC((ex))
	
#endif
	// qDebugExceptions >= qDebugLevelTwo_AC
#endif
