//----------------------------------------------------------------------------------------
// CStream_AC.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein. Used by permission
//----------------------------------------------------------------------------------------

/*
	Note that CStream_AC (the class formerly known as TStream) used to provide built in streaming support
 	for some of the classes that now reside in the imaging suite. Because in the new ACS design this 
 	would be an illegal dependency, this support has been removed from CStream.
	
	To stream these imaging objects use the streaming support in ImagingStreaming_AC.h, which lives in the
	imaging suite.
*/

#ifndef __CStream_AC__
#define __CStream_AC__

#ifndef __Streams_AC__
#include "Streams_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

#ifndef __CStreamRep_AC__
#include "CStreamRep_AC.h"
#endif

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

#ifndef __CDataFilter_AC__
#include "CDataFilter_AC.h"
#endif

#ifndef __MStreamable_AC__
#include "MStreamable_AC.h"
#endif

#ifndef __CStreamContext_AC__
#include "CStreamContext_AC.h"
#endif

//----------------------------------------------------------------------------------------

class CStream_AC;
class CPascalStr_AC;

typedef struct ClassIDEntry
{
	long fOldID;
	long fNewID;
} *PClassIDEntry, **HClassArray;

//----------------------------------------------------------------------------------------
// The following declarations were moved out of the protected section of CStream_AC
// in order to suppress VC++ warnings
typedef CLinkedList_AC<CStream_AC*> CStreamList;
	// readability typedef
class EXPORT_AC CAutoPtr_AC<CStreamList>;
class EXPORT_AC CAutoPtr_AC<CStreamContext_AC>;
class EXPORT_AC TemplateTempHandle<HClassArray>;		

// Forward declarations added to suppress VC++ warnings
class EXPORT_AC CLinkedList_AC<CStream_AC*>;
class EXPORT_AC CRefCountingPtr_AC<CStreamRep_AC>;

//----------------------------------------------------------------------------------------

typedef SInt16 ResNumber;
const IDType	 kNoIdentifier_AC = '    ';
const ResNumber  kNoResource_AC = -1;


// passed to WriteStreamObject
enum
{
	kNoStandardObject, 
	kStandardObject
};

//  written by WriteStreamObject, read by ReadStreamObject 
enum
{
	kNullObject, 
	kLocalObjectNumber, 
	kClassIDAndClassname, 
	kSignatureAndClassname
};

// passed to TContext::Add()
enum
{
	kDontForceReplacement, 
	kForceReplacement
};

#define StreamInEnum_AC(e) 	(Byte&) (e)
#define StreamOutEnum_AC(e) (Byte) (e)
	// These are helpful macros since the compiler can't seem to decide what type
	// an enum is. We're assuming a byte value here for historical MacApp reasons.

//----------------------------------------------------------------------------------------

class EXPORT_AC CStream_AC
{
public:
	typedef CStream_AC& (*ManipulatorProc) (CStream_AC&);
		// This is a typedef for the manipulators like endl_AC or flush_AC.
		
protected:
	CStream_AC(CStreamRep_AC* inRep);
		// Giving the stream a rep is required.

public:
	virtual ~CStream_AC() = 0;

	//------------------------------------------------------------------------------------
	// Note - all reading and writing methods throw exception in case of an I/ O error.
	//------------------------------------------------------------------------------------

// writers

	CStream_AC& operator << (const char* inCString);
	CStream_AC& operator << (const unsigned char* inPString);
	CStream_AC& operator << (Byte inData);
	CStream_AC& operator << (char inData);
	CStream_AC& operator << (bool inData);
	CStream_AC& operator << (short inData);
	CStream_AC& operator << (unsigned short inData);
	CStream_AC& operator << (long inData);
	CStream_AC& operator << (unsigned long inData);
#if !qMSWindows
	CStream_AC& operator << (int inData);
#endif
	CStream_AC& operator << (unsigned int inData);
	CStream_AC& operator << (float inData);
	CStream_AC& operator << (double inData);
  	CStream_AC& operator << (const Handle aHandle);
   	CStream_AC& operator << (const CPascalStr_AC& data);
   	
   	CStream_AC& operator << (ManipulatorProc inManipulator);

	void WriteString(const char* inCString);
	void WriteString(const unsigned char* inPString);
	void WriteByte(Byte data);
	void WriteCharacter(char inData);
	void WriteBool(bool inData);
	void WriteBoolean(bool inBoolean);
	void WriteShort(short inData);
	void WriteUnsignedShort(unsigned short inData);
	void WriteInteger(short inData);
	void WriteUnsignedInteger(unsigned short inData);
	void WriteLong(long inData);
	void WriteUnsignedLong(unsigned long inData);
	void WriteFloat(float inData);
   	void WriteDouble(double inData);
	void WriteIDType(IDType data);
	void WriteHandle(const Handle aHandle); 
		// Locks the handle, dereferences it, and calls WriteBytes for the size of the
		// handle.  Restores the handle to the state it was in.
	void WriteString(const CPascalStr_AC& data); 
		// Write a CPascalStr_AC. This takes a CPascalStrPtr_AC, so it can accommodate any size CPascalStr_AC. 
		// this writes the byte len then the string
	void WriteWordAlign();
		// calls WriteBytes to bump ptr 1 byte to word align
	//------------------------------------------------------------------------------------
	// These methods are used to write an object to a stream.You must call WriteObject
    // first, then tell the object to write its specific data, and then call
    // WriteObjectSize. 
	//------------------------------------------------------------------------------------

	virtual void WriteObject(MStreamable_AC* data, long& sizePosition);
	virtual void WriteStdObject(MStreamable_AC* data, long& sizePosition);
	void WriteObjectSize(long sizePosition);
	virtual void WriteStreamObject(MStreamable_AC* data,  bool stdObject = kNoStandardObject);
		// This calls WriteObject, then data->WriteTo, and finally WriteObjectSize. 

	virtual void WriteBytes(const void*, long amt);

// readers

	CStream_AC& operator >> (Byte& inData);
	CStream_AC& operator >> (char& inData);
	CStream_AC& operator >> (bool& inData);
	CStream_AC& operator >> (short& inData);
	CStream_AC& operator >> (unsigned short& inData);
	CStream_AC& operator >> (long& inData);
	CStream_AC& operator >> (unsigned long& inData);
	CStream_AC& operator >> (float& inData);
   	CStream_AC& operator >> (double& inData);
   	CStream_AC& operator >> (Handle& inData);
#if !qMSWindows
   	CStream_AC&	operator >> (int& inInt);
#endif
   	CStream_AC&	operator >> (unsigned int& inInt);
  	
   	Byte 			ReadByte();
	char 			ReadCharacter();
	bool 			ReadBool();
	bool 			ReadBoolean();
	short 			ReadShort();
	unsigned short 	ReadUnsignedShort();
	short 			ReadInteger();
	unsigned short 	ReadUnsignedInteger();
	long 			ReadLong();
	unsigned long 	ReadUnsignedLong();
	float 			ReadFloat();
   	double 			ReadDouble();
	IDType 			ReadIDType();
	
	char* 			ReadString(char* inCharString, long inLen);
		// reads inLen-1 bytes from stream. Last byte reserved for terminator.

	Handle ReadHandle(); 
		// Read a handle from the stream and return it. 

	CStr255_AC ReadString(short maxSize); 
		// Read a CPascalStr_AC. maxSize is the amount of memory available, which is the maximum
        // CPascalStr_AC length + 1.If the size of the CPascalStr_AC in the stream is too large, then
        // it signals Failure with err=paramErr. 

	void ReadWordAlign();


	virtual MStreamable_AC* ReadObject(bool& wasValid);
		// Returns an uninitialized object; known is set to true if the object was valid
        // (i.e., the class of the object is known to the program. The function result
        // will be nil if the object was unknown or if nil was found in the stream. 

	virtual MStreamable_AC* ReadStdObject(bool& wasValid);
	virtual MStreamable_AC* ReadStreamObject(bool& wasValid);
		// If stdObject is true, calls ReadStdObject, else calls ReadObject. If the
		// result is not nil, calls ReadFrom on the new object. 

#if qBackwardCompatible
	inline void ReadString(CPascalStr_AC& newString, short maxSize) { newString = ReadString(maxSize); }
#endif

	virtual void ReadBytes(void*, long amt);

// piping	
	void PipeOutputTo(CStream_AC* inStream);
	
	void BreakPipeWith(CStream_AC* inStream);

	void PipeBytes(const void* inBytes, long inAmt);
	
// filtering
	
	void AddFilter(CDataFilter_AC* inFilter);
		// Filters are executed in FIFO order. If filters are left in the queue
		// they are deleted.
		
	void RemoveFilter(CDataFilter_AC* inFilter);
		// This does not delete the filter.
			
// CContext_AC mgmt.

	inline void SetContext(CStreamContext_AC* itsContext)
	{ fContext = itsContext; }

	inline CStreamContext_AC* GetContext() const
	{ return fContext; }		

// buffering support
 
	virtual CStream_AC& Flush();
		// For flushing buffered streams. This is sent to the pipes, and does nothing
		// on nonbuffered streams.

// positioning
	
	// Note: you shouldn't call these unless you are writing or reading into or out of 
	// the 'raw' stream reprensentation, otherwise using any of the Write, Read, or 
	// streaming operators will move the positions.

	virtual long GetWritePosition() const;

	virtual void SetWritePosition(long newPosition);
	
	void OffsetWritePosition(long offset)
	{	SetWritePosition( GetWritePosition() + offset ); }
		
	virtual void SetReadPosition(long pos);
	
	virtual long GetReadPosition() const;

	void OffsetReadPosition(long offset)
	{	SetReadPosition( GetReadPosition() + offset ); }
	
	void RewindPositions()
	{	SetReadPosition(0); SetWritePosition(0); }
	
protected:
	long LookupClassID(long id) const;

	void RegisterClassID(long oldID, long newID);

	void FilterReadBytes(void* inBytes, long& inAmt);
	
	const void* FilterWriteBytes(const void* inBytes, long& inAmt);

	virtual CStreamRep_AC* GetStreamRep() const
	{	return fStreamRep; }
	
	virtual void SetStreamRep(CStreamRep_AC* inRep)
	{	fStreamRep = inRep; }
		
protected:
	CDataFilter_AC*									fFilters;
		// list of filters

	CAutoPtr_AC<CStreamList>						fOutputPipes;
		// list of streams to pipe output to
		
	CAutoPtr_AC<CStreamContext_AC> 					fContext;
		// context keeps track of objects streamed in and out of the stream
	TemplateTempHandle<HClassArray> 				fClassMap;		
		// handle to class ID mapping table 
	short 											fClassMapSize;	
		// number of entries in the table 

private:
	CRefCountingPtr_AC<CStreamRep_AC>				fStreamRep;
};

//----------------------------------------------------------------------------------------

inline void CStream_AC::FilterReadBytes(void* inBytes, long& inAmt)
{
	CDataFilter_AC* walker = fFilters;
	while(walker)
	{	
		walker->FilterInput(inBytes, inAmt);	
		walker = walker->GetNextLink();
	}
}

//----------------------------------------------------------------------------------------

inline const void* CStream_AC::FilterWriteBytes(const void* inBytes, long& inAmt)
{
	CDataFilter_AC* walker = fFilters;
	while(walker)
	{	
		inBytes = walker->FilterOutput(inBytes, inAmt);	
		walker = walker->GetNextLink();
	}
	return inBytes;
}

//----------------------------------------------------------------------------------------
// WRITERS
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// Using macros here to hook into the overloaded filter operators
// which require the type of the inData.
//----------------------------------------------------------------------------------------


#define WriteStreamData_AC(inData, inSize)	\
	CDataFilter_AC* walker = fFilters;	\
	while(walker)	\
	{	\
		walker->FilterOutput(inData);	\
		walker = walker->GetNextLink();	\
	}	\
	WriteBytes(&inData, inSize)

//----------------------------------------------------------------------------------------

#define ReadStreamData_AC(inData, inSize)	\
	ReadBytes(&inData, inSize);	\
	CDataFilter_AC* walker = fFilters;	\
	while(walker)	\
	{	\
		walker->FilterInput(inData);	\
		walker = walker->GetNextLink();	\
	}	

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (Byte inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}	

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (char inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (bool inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (short inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (unsigned short inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (long inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (unsigned long inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (float inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (double inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (const Handle aHandle)
{
	WriteHandle(aHandle);
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (ManipulatorProc inManipulator)
{
	return (*inManipulator) (*this);
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (const unsigned char* inPString)
{
	WriteString(inPString);
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (const char* inCString)
{
	WriteString(inCString);
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (const CPascalStr_AC& inStr)
{	
	WriteString(inStr);
	return *this;
}

//----------------------------------------------------------------------------------------
#if !qMSWindows

inline CStream_AC& CStream_AC::operator << (int inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}
#endif

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator << (unsigned int inData)
{
	WriteStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteByte(Byte inData)
{		
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteCharacter(char inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteBool(bool inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteBoolean(bool inBoolean)
{
	WriteBool(inBoolean); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteShort(short inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteUnsignedShort(unsigned short inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteInteger(short inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteUnsignedInteger(unsigned short inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteLong(long inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteUnsignedLong(unsigned long inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteFloat(float inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteDouble(double inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteIDType(IDType inData)
{	
	WriteStreamData_AC(inData, sizeof(inData)); 
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::WriteWordAlign()
{		
	if (::Odd_AC(GetWritePosition()))
		WriteByte(0); 
}

//----------------------------------------------------------------------------------------
// READERS
//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (char& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (bool& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (short& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (unsigned short& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (long& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (unsigned long& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (float& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (double& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (Byte& inData)
{
	ReadStreamData_AC(inData, sizeof(inData));
	return *this;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (Handle& inData)
{
	inData = ReadHandle();
	return *this;
}

//----------------------------------------------------------------------------------------
#if !qMSWindows

inline CStream_AC& CStream_AC::operator >> (int& inData)
{
	ReadStreamData_AC(inData, sizeof(inData)); 
	return *this;
}
#endif

//----------------------------------------------------------------------------------------

inline CStream_AC& CStream_AC::operator >> (unsigned int& inData)
{
	ReadStreamData_AC(inData, sizeof(inData)); 
	return *this;
}

//----------------------------------------------------------------------------------------

inline Byte CStream_AC::ReadByte()
{
	Byte data;
	ReadStreamData_AC(data, sizeof(data));
	return data;
}

//----------------------------------------------------------------------------------------

inline char CStream_AC::ReadCharacter()
{
	char inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline bool CStream_AC::ReadBool()
{
	bool inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline bool CStream_AC::ReadBoolean()
{	
	return ReadBool(); 
}

//----------------------------------------------------------------------------------------

inline short CStream_AC::ReadShort()
{
	short inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline unsigned short CStream_AC::ReadUnsignedShort()
{
	unsigned short inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline short CStream_AC::ReadInteger()
{
	short inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline unsigned short CStream_AC::ReadUnsignedInteger()
{
	unsigned short inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline long CStream_AC::ReadLong()
{
	long inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline unsigned long CStream_AC::ReadUnsignedLong()
{
	unsigned long inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline float CStream_AC::ReadFloat()
{
	float inData;
	ReadStreamData_AC(inData, sizeof(inData));
	return inData;
}

//----------------------------------------------------------------------------------------

inline IDType CStream_AC::ReadIDType()
{
	IDType data;
	ReadStreamData_AC(data, sizeof(data));
	return data;
}

//----------------------------------------------------------------------------------------

inline void CStream_AC::ReadWordAlign()
{
	if (::Odd_AC(GetReadPosition()))
		ReadByte();
}

//----------------------------------------------------------------------------------------

inline char* CStream_AC::ReadString(char* inString, long inLen)
{
	ReadBytes(inString, inLen);
	inString[inLen] = '\0';
	return inString;
}

//----------------------------------------------------------------------------------------
// manipulators

inline CStream_AC& flush_AC (CStream_AC& inStream)
{
	inStream.Flush();
    return inStream;
}

#endif
