// UAppleEvents.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UAPPLEEVENTS__
#define __UAPPLEEVENTS__

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __TOOLBOX__
#include "Toolbox.h"
#endif

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

// Toolbox

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

//----------------------------------------------------------------------------------------
// Mask constants for testing sending mode.
//----------------------------------------------------------------------------------------

const long kAEReplyModeMask = 0x0000000F;		// Mask for reply mode
const long kAEInteractModeMask = 0x00000030;	// Mask for interact mode

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class CDynamicArray_AC;
class TFile;
class CStream_AC;
class MScriptableObject;
class CAEDesc;

class TemplateTempRoutineDescriptor<AEIdleUPP>;
class TemplateTempRoutineDescriptor<AEFilterUPP>;		

//----------------------------------------------------------------------------------------
// CAEDesc : is a toolbox compatible class for the toolbox struct AEDesc. It is bitwise
// compatible with the toolbox struct
// It provides lots of extra services. Use this instead of a bare AEDesc and you'll have
// access to those services.
//----------------------------------------------------------------------------------------

// Compatibility macro for setting the backward compatible fMessage field.
#if qBackwardCompatible
#define INIT_FMESSAGE :fMessage(*this)
#else
#define INIT_FMESSAGE
#endif

class EXPORT_MA CAEDesc : public AEDesc
{
public:
	CAEDesc();
	inline CAEDesc(const AEDesc& theDesc) INIT_FMESSAGE { Set(theDesc); }
	inline CAEDesc(const AEDesc* theDesc) INIT_FMESSAGE { Set(*theDesc); }
	inline CAEDesc(DescType theType, Handle theHandle) INIT_FMESSAGE { Set(theType, theHandle); }

	// Wrappers for toolbox functions
	
	bool CanCoerceDesc(DescType toType) const;

	CAEDesc CoerceDesc(DescType toType) const;
	inline void CoerceDesc(DescType toType, CAEDesc& result) const { result = CoerceDesc(toType); } // synonym for compatibility

	void CreateDesc(DescType typeCode, const void *dataPtr, Size dataSize);
	OSErr DisposeDesc();
	OSErr GetKeyPtr(AEKeyword theAEKeyword, DescType desiredType, DescType *typeCode,
					void *dataPtr, Size maximumSize, Size *actualSize) const;
	void PutKeyPtr(AEKeyword theAEKeyword, DescType typeCode,
				   const void *dataPtr, Size dataSize);

	// Conversion operator methods

	inline operator AEDesc&()				{ return *this; }
	inline operator const AEDesc&() const	{ return *this; }

	inline operator AEDesc*()				{ return this; }
	inline operator const AEDesc*() const	{ return this; }

	inline CAEDesc& operator=(const CAEDesc& theDesc) { Set(theDesc); return *this; }

	// Desc content comparison method

	bool EqualDesc(const CAEDesc& theDesc) const; 

	bool CompareDesc(const CAEDesc& theDesc, DescType compareHow) const; 

	// Methods to get and set data on the fAEDesc.

#if qPowerPC
	inline void Set(const AEDesc& theDesc) { *(double*)this = *(const double*)&theDesc; }
		// cheat, we know that we're eight bytes long
#else
	void Set(const AEDesc& theDesc);
#endif

	void Set(DescType theType, Handle theHandle);
	inline void SetDesc(DescType theType, Handle theHandle) { Set(theType, theHandle); } // synonym for compatibility

	inline DescType GetDescriptorType() const	{ return descriptorType; }

	inline Handle GetDataHandle() const	{ return dataHandle; }

	long GetDataSize() const;
	
	// The following methods treat the CAEDesc as a list of descriptors
	
	void CreateList();
		// Creates an AEList with no factoring information
		
	long CountListItems();
		// Returns the number of items in the list

	CAEDesc GetNthItem(long index, DescType desiredType);
	inline void GetNthItem(long index, DescType desiredType, CAEDesc& theDesc) { theDesc = GetNthItem(index, desiredType); } // synonym for compatibility
		// Gets the Nth AEDesc from the list

	void PutListItem(long index, CAEDesc& theDesc);
		// Puts theDesc into the list at the index position.

	OSErr DeleteListItem(long index);
		// Puts theDesc at the index position from the list.

	// The following methods are for storing and retrieving various data
	// types from CAEDescs
	
	CFSSpec_AC GetFSSpec() const;
	inline void GetFSSpec(CFSSpec_AC& fileSpec) const { fileSpec = GetFSSpec(); } // synonym for compatibility
	void PutFSSpec(const CFSSpec_AC& fileSpec);

	CRect_AC GetRect() const;
	inline void GetRect(CRect_AC& theRect) const { theRect = GetRect(); } // synonym for compatibility
	void PutRect(const CRect_AC& theRect);

	DescType GetType() const;
	void PutType(DescType theType);

	CStr255_AC GetString(long maximumSize = 255) const;
	inline void GetString(CStr255_AC& theString, long maximumSize = 255) const { theString = GetString(maximumSize); } // synonym for compatibility
	void PutString(const CStr255_AC& theString);

	bool GetBoolean() const;
	void PutBoolean(bool theBoolean);

	DescType GetEnum() const;
	void PutEnum(DescType theEnum);

	long GetLong() const;
	void PutLong(long theLong);

	unsigned long GetDateTime() const;
	void PutDateTime(unsigned long theDateTime);

	CRGBColor_AC GetRGBColor() const;
	inline void GetRGBColor(CRGBColor_AC& theColor) const { theColor = GetRGBColor(); } // synonym for compatibility
	void PutRGBColor(const CRGBColor_AC& theColor);

	MScriptableObject* GetObject() const;
	void PutObject(MScriptableObject* theObject);

	// Methods for dealing with records

	void CreateRecord();

	// The following GetKeyXXX and PutKeyXXX methods are for easy access to AERecords
	// The GetKeyXXX methods will not fail but will return false if the key did not exist.
	// This is so you can use them to attempt to retrieve optional keys in a record without
	// handling failure or checking first to see if they exist.

	bool GetKeyLong(DescType theKey,
					   long& theNum) const;
	void PutKeyLong(DescType theKey,
					long theNum);

	bool GetKeyRect(DescType theKey,
					   CRect_AC& theRect) const;
	void PutKeyRect(DescType theKey,
					const CRect_AC& theRect);

	bool GetKeyString(DescType theKey,
						 CStr255_AC& theStr,
						 long maximumSize = 255) const;
	void PutKeyString(DescType theKey,
					  const CStr255_AC& theStr);

	bool GetKeyDateTime(DescType theKey,
						   unsigned long& theDate) const;
	void PutKeyDateTime(DescType theKey,
						unsigned long theDate);

	bool GetKeyBoolean(DescType theKey,
						  bool& theBoolean) const;
	void PutKeyBoolean(DescType theKey,
					   bool theBoolean);

	bool GetKeyColor(DescType theKey,
						CRGBColor_AC& theColor) const;
	void PutKeyColor(DescType theKey,
					 const CRGBColor_AC& theColor);

	bool GetKeyEnum(DescType theKey,
					   DescType& theEnum) const;
	void PutKeyEnum(DescType theKey,
					DescType theEnum);
					
	bool GetKeyDesc(DescType theKey,
						DescType desiredType,
					   	CAEDesc& theAEDesc) const;
						
	void PutKeyDesc(DescType theKey,
					CAEDesc& theAEDesc);

	OSErr DeleteKeyDesc(DescType theKey);

	// Stream I/O methods

	void ReadFrom(CStream_AC* aStream);
	void WriteTo(CStream_AC* aStream) const;
	
	// Resource I/O methods
	
	void ReadFrom(TFile* aFile, ResType resourceType, short resourceID);
	void WriteTo(TFile* aFile,
					  short resourceID,
					  CStr255_AC& resourceName) const;

	//----------------------------------------------------------------------------------------
	// static data members
	//----------------------------------------------------------------------------------------
public:
	static CAEDesc fgNullDesc; 
#if qBackwardCompatible
public:
	AEDesc& fMessage;
#endif
};

//----------------------------------------------------------------------------------------
// CTempDesc
//----------------------------------------------------------------------------------------

class EXPORT_MA CTempDesc : public CAEDesc
{
	// This varient of CAEDesc is designed for the occasions when you need to create
	// a descriptor that you will only use within a limited scope. CTempDesc destructor
	// will call AEDisposeDesc for you to simplify cleanup.

public:
	inline CTempDesc() { }
	inline CTempDesc(const AEDesc& theDesc) : CAEDesc(theDesc) { }
	inline CTempDesc(DescType theType, Handle theHandle) : CAEDesc(theType, theHandle) { }
	// Initializes the fAEDesc

	inline ~CTempDesc() { if (dataHandle) DisposeDesc(); }
	// releases the descriptor.

	inline CAEDesc release() { CAEDesc returnVal = *this; dataHandle = NULL; return returnVal; }
		// Clears the owned reference to the data when returning it so it won't get released.
};

//----------------------------------------------------------------------------------------
// TAppleEvent
//----------------------------------------------------------------------------------------

class EXPORT_MA TAppleEvent : public CAEDesc
{
public:
	TAppleEvent(const AEEventClass itsEventClass = typeNull,
									const AEEventID itsEventID = typeNull,
									const AEAddressDesc& itsAddress = fgNullDesc,
									long itsSendingMode = kAENoReply);
		// Creates a new AppleEvent message and stores it in fMessage. Supplies
		// defaults for return ID, transaction ID, and sending priority.
		
	TAppleEvent(const AppleEvent& theMessage,
									   bool freeMessage);
		// Stores theMessage in fMessage. This may be a message or a reply.  If freeMessage is true
		// Free the message when freeing the object

	virtual ~TAppleEvent();
	
	virtual OSErr SendEvent();
		// Send the apple event. Will not throw an exception. Preferred method.
	
	virtual OSErr SendEvent(TAppleEvent*& replyEvent);
		// Send the apple event and get a reply event. Will not throw an exception. Preferred Method.

	virtual TAppleEvent* Send();
		// Send the apple event and return a reply. WILL throw an exception if an error occurs.
		// Present for compatibility - use SendEvent when possible.
		
	void GetAddress(AEAddressDesc& theAddress);
		// Return the target address of the message

	void GetProcessSerialNumber(ProcessSerialNumber& thePSN);
		// Return the Process Serial Number for the sender of the message

	inline short GetPriority()
	{ return fPriority; }
		// Return the sending priority of the message

	long GetReturnID();
		// Return the ID assigned to the message
	
	inline long GetSendingMode()
	{ return fSendingMode; }
		// Return the sending mode of the event

	inline long GetTimeoutVal()
	{ return fTimeoutVal; }
		// Return the timeout value (in ticks) used when sending the message

	long GetTransactionID();
		// Return the ID of the transaction the message belongs to

	void SetAddress(const AEAddressDesc& theAddress);
		// Assign the target address of the message

	void SetPriority(short thePriority);
		// Assign the priority with which the message will be sent

	void SetReturnID(long returnID);
		// Assign a new return ID to the message
		
	void SetSendingMode(long sendingMode);
		// Specify a new way of sending the message

	void SetTimeoutVal(long theTimeoutVal);
		// Assign the time (in ticks) to wait for reply if sending kAEWaitReply

	void SetTransactionID(long transactionID);
		// Assign a transaction ID to the message
	
	AEEventClass ReadEventClass();
		// Return the event class of the apple event
		
	AEEventID ReadEventID();
		// Return the event ID

	// READING - The following methods retrieve theData from fMessage based on the keyword
	short ReadShort(const AEKeyword theKey);
									
	long ReadLong(const AEKeyword theKey);
								 
	void ReadString(const AEKeyword theKey,
								 CStr255_AC& theData);
								 
	SectionHandle ReadSectionHandle(const AEKeyword theKey);

	void ReadPtrList(const AEKeyword theKey,
									const DescType theType,
									CDynamicArray_AC* theData);

	void ReadHandleList(const AEKeyword theKey,
									   const DescType theType,
									   CDynamicArray_AC* theData);

	DescType ReadType(const AEKeyword theKey);

	DescType ReadEnum(const AEKeyword theKey);

	bool ReadBoolean(const AEKeyword theKey);

	void ReadRect(const AEKeyword theKey,
						  CRect_AC& theRect);
						  
	void ReadError(long& errorNumber, CStr255_AC& errorString);

	// Allow raw read access to a parameter of the message

	OSErr ReadParameter(const AEKeyword theKey,
							   const DescType desiredType,
							   CAEDesc& theData);


	void ReadParameterPtr(const AEKeyword theKey,
										 const DescType desiredType,
										 DescType& actualType,
										 Ptr theData,
										 long maximumSize,
										 long& actualSize);

	// WRITING - The following methods store theData in fMessage with theKey
	void WriteShort(const AEKeyword theKey,
									 short theData);
									 
	void WriteLong(const AEKeyword theKey,
								  long theData);
								  
	void WriteString(const AEKeyword theKey,
								 	const CStr255_AC& theData);
								 
	void WriteSectionHandle(const AEKeyword theKey,
										   SectionHandle theData);
										   
	void WritePtrList(const AEKeyword theKey,
									 const DescType theType,
									 CDynamicArray_AC* theData);

	void WriteHandleList(const AEKeyword theKey,
										const DescType theType,
										CDynamicArray_AC* theData);

	void WriteEnum(const AEKeyword theKey,
						   DescType theEnum);

	void WriteType(const AEKeyword theKey,
						   DescType theType);

	void WriteBoolean(const AEKeyword theKey,
							  bool theBoolean);

	void WriteRect(const AEKeyword theKey,
						   const CRect_AC& theRect);

	// Allow raw access to a parameter of the message

	void WriteParameter(const AEKeyword theKey,
								const CAEDesc& theData);

	void WriteParameterPtr(const AEKeyword theKey,
										  const DescType typeCode,
										  Ptr theData,
										  long dataSize);

	void DeleteParameter(const AEKeyword theKey);
		// Delete the specified parameter from the message

	void CheckRequiredParameters();

	bool HasParameter(const AEKeyword theKey);

	DescType ParameterType(const AEKeyword theKey);

	bool WriteReplySpecifier(MScriptableObject* theReplyObject);
	// Use this whenever you need to return the object's specifier as the result of an event.
	// Makes an object specifier for an object and puts it in the keyAEResult parameter of this event.
	
	void WriteError(long errNum, short errorStringResource, short errorStringIndex);
	void WriteOSError(OSErr error);
		// Report an error back to the client using the errReasonID 'errs' resource.

	//----------------------------------------------------------------------------------------
	// static functions
	//----------------------------------------------------------------------------------------
public:
	static TAppleEvent* MakeCreateElementEvent(AEAddressDesc& targetAddress, AESendMode	sendMode, 
			CAEDesc& targetSpecifier, DescType newElementType, DescType	newElementPosition, 
			CAEDesc& initialData, CAEDesc& initialProperties);

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
public:
	long	fSendingMode;						// kAENoReply, kAEQueueReply, or kAEWaitReply
	long	fTimeoutVal;						// How long should we wait for a response?
	short	fPriority;							// kAENormalPriority or kAEHighPriority
	bool	fFreeMessage;						// If true then free the message when freeing the object
	
	//----------------------------------------------------------------------------------------
	// static data members
	//----------------------------------------------------------------------------------------
public:
	static TemplateTempRoutineDescriptor<AEIdleUPP> fgIdleProc;
	static TemplateTempRoutineDescriptor<AEFilterUPP> fgFilterProc;		
};
	
#endif
