// UDragDropBehavior
// Copyright  1994-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UDRAGDROPBEHAVIOR__
#define __UDRAGDROPBEHAVIOR__

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

const IDType kDragDropBehavior = 'drag';

//----------------------------------------------------------------------------------------
// class TDragDropBehavior
//----------------------------------------------------------------------------------------

class EXPORT_MA TDragDropBehavior : public TBehavior
{
public:
	TDragDropBehavior(IDType itsIdentifier = kNoIdentifier, 
											bool draggable = false, 
											bool droppable = false,
											short	dragMoveDeterminer = kNeverMove,
											IDType	dragMoveFamily = kNoIdentifier);
		// Constructor

	virtual ~TDragDropBehavior();
		// Destructor

	virtual void* Clone() const;
		// Clone
				
	void ReadFrom(CStream_AC* aStream);
		// Read the behavior's data from a stream

	void WriteTo(CStream_AC* aStream) const;
		// Write the behavior's data to a stream
		
	virtual void SetOwner(TEventHandler *itsOwner);
		// Override
	
	virtual IDType GetStandardSignature() const;
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Additional simple accessors/manipulators
	//------------------------------------------------------------------------------------

	inline bool IsDraggable() const { return fDraggable; }
	inline bool IsDroppable() const { return fDroppable; }
	inline short GetDragMoveDeterminer() const { return fDragMoveDeterminer; }
	inline IDType GetDragMoveFamily() const { return fDragMoveFamily; }
	
	inline void SetDraggable(bool isDraggable) { fDraggable = isDraggable; }
	inline void SetDroppable(bool isDroppable) { fDroppable = isDroppable; }
	inline void SetDragMoveDeterminer(short dragMoveDeterminer) { fDragMoveDeterminer = dragMoveDeterminer; }
	inline void SetDragMoveFamily(IDType dragMoveFamily) { fDragMoveFamily = dragMoveFamily; }

	//------------------------------------------------------------------------------------
	// Data members
	//------------------------------------------------------------------------------------

#if qBackwardCompatible
public:
#else
protected:
#endif

	bool	fDraggable;				// indicates whether or not the owner view's contents
									// can be dragged
	bool	fDroppable;				// indicates whether or not the owner view can receive
									// drops
	
	short	fDragMoveDeterminer;	// defines context for drag move. one of the following
									// values: kNeverMove, kMoveWithinView, kMoveWithinWindow,
									// kMoveWithinContext, kMoveWithinApplication
									
	IDType	fDragMoveFamily;		// Used with fDragMoveDeterminer to determine if a drag
									// operation should be treated as a copy or a move.
									// If the source and target of the drag are within
									// the context defined by the source view's fDragMoveDeterminer
									// and their fDragMoveFamily values match or are both
									// kNoID, it's a move.
}; // class TDragDropBehavior

#endif // __UDRAGDROPBEHAVIOR__

