//----------------------------------------------------------------------------------------
// UEditText.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UEDITTEXT__
#define __UEDITTEXT__

#ifndef __USTATICTEXT__
#include "UStaticText.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

#ifndef __UFLOATINGTEMANAGER__
#include "UFloatingTEManager.h"
#endif

//----------------------------------------------------------------------------------------
// Some constants
//----------------------------------------------------------------------------------------
enum
{
	kErrorHandled = -1,				// Don't post an alert, the user handled it

	kInvalidValueReasons = 300,		// CPascalStr_AC list used for invalid value alert
                                                
	phInvalidValue = 300			// Alert used when an invalid value is entered
};
	                                               
enum
{
	kValidValue = 0,						// Not looked up in the CPascalStr_AC list.
	kInvalidValue = 1,
	kValueTooSmall = 2,
	kValueTooLarge = 3,
	kNonNumericCharacters = 4,
	kTooManyCharacters = 5,
	kNoOfDefaultReasons = 5
};

//Forward Declarations

class TDialogTEView;
//----------------------------------------------------------------------------------------
// TEditText: Implements an edit text field.
//----------------------------------------------------------------------------------------

class EXPORT_MA TEditText : public TStaticText
{
	MA_DECLARE_CLASS;
	
public:
	TEditText(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								short itsMaxChars = 255);
		// Constructor
		
	virtual ~TEditText();
		// Un-installs the floating TEView, sets fTEView to NULL then calls Inherited::Free.

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	virtual void WriteSubViews(CStream_AC* aStream) const;


	virtual void DoSubstitution(CStr255_AC& theText);
		// Default is to do nothing.

	virtual CStr255_AC GetText() const;
	inline void GetText(CStr255_AC& itsTitle) const { itsTitle = GetText(); }	
		// If TEView not NULL then returns the extracted text, otherwise calls Inherited::GetText.

	virtual void ImageText(Ptr text,
								  long length,
								  const CViewRect& box,
								  short just);
		// Sets up the text to the parameters passed. Text is drawn in a 'box' , the text
        // is 'length' long.

	virtual bool HandleMouseDown(const CViewPoint& theMouse,
										   TToolboxEvent* event,
										   CPoint_AC hysteresis,
										   EMouseDownType mouseDownType = kDragOrClick);	// Override
	
	virtual void DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* event,
								CPoint_AC hysteresis);		// Override
						
	virtual void BecameWindowTarget();				// Override
		// Overridden to install the "floating" TDialogTEView
		
	virtual void BecameTarget();			// Override
		// Overridden to make the "floating" TDialogTEView the target
		
	virtual void SetTargetSelection(bool redraw);	// Override
		// Selects all the characters in this EditText
	
	virtual bool WantsToBeTarget();	// Override

	virtual void RestartEdit(const CStr255_AC& restartText);
		// This method resets the text of the view, as well as selecting the text. We go
        // to some extra trouble to ensure that this method doesn't do any drawing,
        // defering to window update time. The reasoning is that this method usually gets
        // called right after displaying an alert. The alert may have obscured this view,
        // thus including it in the window's update region. If we drew in this method we
        // would get flashing as this view would get redraw at update time. Unfortunately
        // TextEdit isn't very cooperative so we set the clipping to an empty CRect_AC to
        // ensure that TEActivate's drawing isn't seen.

	virtual void ChangeWrap(bool newAutoWrap, bool redraw);
		// Changes auto-wrapping behavior, redrawing if requested.Forwards into the
        // floating TE if necessary.

	virtual void SetJustification(short theJust, bool redraw);
		// Sets the justification of the text to theJust, redrawing is required.

	virtual void SetSelection(short selStart,
									 short selEnd,
									 bool redraw);
		// Sets the selection in the field starting at selStart and ending at selEnd,
        // redrawing if required.

	virtual void SetText(const CStr255_AC& theText, bool redrawImmediately);
		// Sets the displayed text to theText, redrawing if required.

	virtual void StartEdit(bool selectChars);
		// Install the selection, revealing the selection if required.

	virtual void MakeVisible();
		// Reveal the EditText, if scrolled out of view

	virtual bool Deselect();
		// Valdates and and deselects the edit text, if possible

	virtual void StopEdit();
		// Extracts the CPascalStr_AC from the TEView, then clears out the TEView before calling
        // SetText.

	virtual void InstallFloatingTEView(bool selectChars);
		// Attach the "floating" TDialogTEView to this EditText

	virtual void RemoveFloatingTEView();
		// Remove the "floating" TDialogTEView from this EditText.

	virtual TDialogTEView* GetTEView();
		// Returns a "floating" TDialogTEView to install in this EditText

	virtual void ReleaseTEView();
		// Returns the "floating" TDialogTEView to the server

	virtual long GetValidationError(); // Override
		// By default just returns true.

	virtual void ValidationFailed(long reason);	// Override
		// Displays an alert when validation fails.

	virtual void GetDefaultCursorRegion(const CViewPoint& localPoint,
									   RgnHandle cursorRegion); // Override
		// Sets the cursor region to the control area (i.e. including inset)

#if qDrag

	//------------------------------------------------------------------------------------
	// Drag/Drop Support
	//------------------------------------------------------------------------------------
								
	virtual RgnHandle DoMakeDragCursorRegion(); // Override
		// Provide a drag test region in local coordinates.

	virtual bool WillDrag(const CViewPoint& theMouse, const RgnHandle dragCursorRegion); // Override
		// Return true if a mouse click at theMouse would begin a drag

	virtual short GetWillDragCursorID();	// Override
		// display quickdraw arrow cursor
		
	virtual short GetIsDraggingCursorID();	// Override
		// display quickdraw arrow cursor

	virtual void DoAddDragContent(); // Override
		// Add text content

	virtual bool WillAcceptDrop(CDragItemIterator& dragItemIterator); // Override
		// Return true if the drag contains a single text item
	
	virtual void DoDragEnter(); // Override
		// Become the current drop target. Install a floating text edit view
		
	virtual RgnHandle DoMakeDropHiliteRegion(); // Override
		// Create a local region that will represent the drop area

	virtual void DoDragWithin(const CViewPoint& localMouse); // Override
		// Tell the fTEView to manage mouse tracking during a drag.

	virtual void DoDragLeave(); // Override
		// Notify the TEView that the drag has left
	
	virtual TCommand* DoMakeDragDropCommand(CommandNumber 		itsCommandNumber,
											CDragItemIterator& 	dragItemIterator);	// Override
		// Create a drag drop command.
		
	virtual void DoFulfillPromise(TDragItem* promisedItem); // Override
		// Provide promised flavor on demand
		
#endif // qDrag

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TDialogTEView* fTEView;

	ControlCharSet fControlChars;				// Control chars accepted in text

	short fMaxChars;
}; // class TEditText

//----------------------------------------------------------------------------------------
// Externally available function declaration.
//----------------------------------------------------------------------------------------

EXPORT_MA void InitUEditText();
	// This routine must be called if UEditText is used so that all the necessary
    // initialization can be performed.

EXPORT_MA TDialogTEView* AcquireFloatingTEView();
	// Called by EditTexts to obtain a "floating" TDialogTEView to install. Procedural
    // interface to TFloatingTEManager::AcquireFloatingTE.

EXPORT_MA void ReleaseFloatingTEView(TDialogTEView* aDialogTEView);
	// Called by EditTexts when deselecting. Procedural interface to
    // TFloatingTEManager::ReleaseFloatingTE.

//----------------------------------------------------------------------------------------
// Externally available variable declarations.
//----------------------------------------------------------------------------------------

EXPORT_MA extern TFloatingTEManager gFloatingTEManager;

#endif