// UEditionDocument.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UEDITIONDOCUMENT__
#define __UEDITIONDOCUMENT__

// ACS

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include "UFileBasedDocument.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __USECTION__
#include "USection.h"
#endif

// ACS

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

// Toolbox

#ifndef __EDITIONS__
#include <Editions.h>
#endif


//----------------------------------------------------------------------------------------
// Edition Manager menu command constants
//----------------------------------------------------------------------------------------

const CommandNumber cCreatePublisher = 601;		// the user has selected "Create
                                                // Publisher" from the Edit menu

const CommandNumber cSubscribeTo = 602;			// the user has selected Subscribe to from
                                                // the Edit menu

const CommandNumber cOptions = 603;				// the user has selected a publisher in
                                                // the document and then selected
                                                // Publisher Options from the Edit menu,
                                                // OR, the user has selected a subscriber
                                                // in the document and then selected
                                                // Subscriber Options from the Edit menu

const CommandNumber cBorders = 604;				// the user has selected Show/ Hide Borders
                                                // from the Edit menu

const CommandNumber cStopAllEditions = 605;		// the user has selected Stop all Editions
                                                // from the Edit menu

//----------------------------------------------------------------------------------------
// Edition Manager command constants
//----------------------------------------------------------------------------------------

const CommandNumber cCancelPublisher = 606;		// the user canceled this publisher

const CommandNumber cCancelSubscriber = 607;	// the user canceled this subscriber

//----------------------------------------------------------------------------------------
// miscellaneous constants
//----------------------------------------------------------------------------------------

const IDType kSectionBehavior = 'sect';

const IDType kSubscriberAdornerID = 'subs';

const IDType kPublisherAdornerID = 'publ';

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TDesignator;
class TFile;
class CList_AC;
class TPublisher;
class TSection;
class TSectionList;
class TSubscriber;
class TView;
class TEditionDocument;


//----------------------------------------------------------------------------------------
// CSectionIterator
//----------------------------------------------------------------------------------------

class EXPORT_MA CSectionIterator : public TemplateIterator_AC<TSection*>
{
public:
	CSectionIterator(const TEditionDocument* itsDocument,
								   ArrayIndex_AC itsLowBound,
								   ArrayIndex_AC itsHighBound,
								   bool itsForward);

	CSectionIterator(const TEditionDocument* itsDocument,
								   bool itsForward);

	CSectionIterator(const TEditionDocument* itsDocument);

	CSectionIterator(const TSectionList* itsSectionList);
};


//----------------------------------------------------------------------------------------
// TFileBasedDocument
//----------------------------------------------------------------------------------------

class EXPORT_MA TEditionDocument : public TFileBasedDocument
{
	MA_DECLARE_CLONE;
	
public:
	//------------------------------------------------------------------------------------
	// constructor/destructor methods
	//------------------------------------------------------------------------------------
	TEditionDocument(TFile* itsFile = NULL,
										 const OSType itsScrapType = kNoIdentifier,
										 const OSType editionCreator = kAllQuestionsType_AC);
		// Constructor

	virtual ~TEditionDocument();
		// frees this object

	virtual void FreeData();
		// frees the fSectionList


	//------------------------------------------------------------------------------------
	// saving documents
	//------------------------------------------------------------------------------------

	virtual void Close();
		// For the case in which the document's change count = 0, but there are "dirty"
        // sections, need to ask the user whether to save the document.

	virtual short PoseNewPublishersAlert();
		// Warns the user that he is about to close a document that hasn't been saved in
        // which he has created a new publisher. The user hasn't modified the document's
        // data.

	virtual short PoseMultPublishersAlert(TSection* aSection);
		// Warns the user that he is creating multiple publishers to the same edition.

	virtual short PoseSavingMultPublishersAlert(TSection* aSection);
		// Warns the user that he is saving multiple publishers to the same edition.

	virtual void Abandon();
		// Calls TSection.Delete on all dirty sections.

	virtual void DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes);
		// Bytes required to store SAVE file(s) on disk. When called, both parameters are
        // set to 0 for you.

	virtual void DoWrite(TFile* aFile, bool makingCopy);
		// writes out the sections and edition doc settings

	virtual void DoWriteSettings(TFile* aFile);
		// writes out the edition doc settings (Show/ Hide Border, Stop All Editions)

	virtual void DoRead(TFile* aFile, bool forPrinting);
		// reads in the sections and edition doc settings

	virtual void DoReadSettings(TFile* aFile);
		// reads in the edition doc settings (Show/ Hide Border, Stop All Editions)

	virtual void DoReadPublisher(TFile* aFile, short theID);
		// reads in a given publisher

	virtual void DoReadSubscriber(TFile* aFile, short theID);
		// reads in a given subscriber

	//------------------------------------------------------------------------------------
	// Revert
	//------------------------------------------------------------------------------------

	virtual void RevertDocument();
		// Calls Abandon() and then Inherited.

	//------------------------------------------------------------------------------------
	// Accessors
	//------------------------------------------------------------------------------------

	virtual void SetSectionBorders(bool state, bool invalidate);
		// sets the value of fShowSectionBorders to state and invalidates the document's windows
        // if "invalidate" is true

	virtual void SetStopAllEditions(bool state);
		// sets the value of fStopAllEditions to state and reads in any new subscribers if "state"
        // is true


	//------------------------------------------------------------------------------------
	// menu management methods
	//------------------------------------------------------------------------------------

	virtual void DoMenuCommand(CommandNumber aCommandNumber);

	virtual void DoSetupMenus();

	virtual bool CanPublishSelection();

	virtual bool CanSubscribe();


	//------------------------------------------------------------------------------------
	// edition manager support
	//------------------------------------------------------------------------------------

	virtual void DoAddSectionBehavior(TView* itsView);
		// creates and adds a TSectionBehavior to implement the Ed Mgr shortcuts.

	virtual void DoNewPublisher();

	virtual TPublisher* DoMakePublisher(TDesignator* itsDesignator,
											   SectionHandle itsSectionHandle,
											   short itsSectionID);


	virtual void DoNewPublisherDialog(NewPublisherReply& reply);

	virtual void DoMakePreview(TDesignator* itsDesignator,
									  FormatType& previewFormat,
									  Handle& preview);
	
	virtual void DisposeOfPreview(NewPublisherReply& reply);
		// Called from DoNewPublisher.  Default implementation diposes of the view
		// if it is of type 'TEXT' or 'PICT'.  Override this method to dispose of
		// your own special preview type.

	virtual void DoNewSubscriber();

	virtual TSubscriber* DoMakeSubscriber(TDesignator* itsDesignator,
												 SectionHandle itsSectionHandle,
												 short itsSectionID);

	virtual void DoNewSubscriberDialog(NewSubscriberReply& reply);

	virtual void DoSectionOptions();
		// poses the publisher or subscriber options dialog depending on what
        // GetSelectedSection returns

	virtual FormatType GetPublishPreference();
		// return 'PICT' to publish 'PICT' first, return 'TEXT' to publish 'TEXT' first

	virtual SignedByte GetSubscriberFormatsMask();

	virtual void GetNextEditionName(CStr63_AC& editionName);

	virtual void DoPublisherOptions(TPublisher* aPublisher);

	virtual void DoPublisherOptionsDialog(SectionOptionsReply& reply);

	virtual void DoSubscriberOptions(TSubscriber* aSubscriber);

	virtual void DoSubscriberOptionsDialog(SectionOptionsReply& reply);
	

	//------------------------------------------------------------------------------------
	// section management
	//------------------------------------------------------------------------------------

	virtual void AddSection(TSection* aSection);
		// adds the section to this document's section list

	virtual void AddSectionAndBorder(TSection* aSection);
		// adds the section to this document's section list and calls DoAddBorder to add
		// its border. This should be used when the view hierarchy has already been
		// constructed and we know that have a view to attach the border to.

	virtual void CancelSection(TSection* aSection, bool cancel);
		// calls aSection->CancelSection(cancel);

	virtual TSection* GetSelectedSection();
		// This returns the currently selected section, if any. Returns nil if > 1 section
        // is currently selected.

	virtual short GetUniqueSectRsrcID();
		// returns a unique 'sect' resource id for use as both the section ID and as the
        // rsrc ID

	virtual bool IsSectionSelected(TSection* aSection);
		// returns true if the given section is in the user selection

	virtual void RemoveSection(TSection* aSection);
		// this method should mark document as changed so that document removes this
        // section when saved (e.g. sipAlways)

	//------------------------------------------------------------------------------------
	// border management
	//------------------------------------------------------------------------------------

	virtual void DoPostMakeViews(bool forPrinting);
		// Calls DoAddBorder for all sections and then calls Inherited::DoPostMakeViews().

	virtual void DoAddBorder(TSection* aSection);
		// subclasses should override this to create a border to adorn this section

	virtual void DoAdjustBorder(TSection* aSection);
		// subclasses should override this to adjust the border which adorn this section

	virtual void DoDeleteBorder(TSection* aSection);
		// subclasses should override this to remove the border adorning this section

	virtual bool IsBorderShown(TSection* aSection);
		// returns true if (borders are shown) or (the user selection contains aSection)

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline TSectionList* GetSectionList() const { return &const_cast<TEditionDocument*>(this)->fSectionList; }
	inline bool GetShowSectionBorders() const { return fShowSectionBorders; }
	inline bool GetStopAllEditions() const { return fStopAllEditions; }

	inline OSType GetEditionCreatorSignature() const { return fEditionCreator; }
		// returns the creator signature used for the edition container file


#if qBackwardCompatible
public:
#else
protected:
#endif
	TSectionList fSectionList;					//  list of TSection objects

	OSType fEditionCreator;						// creator for publisher edition container file

	short fNextEditionNumber;					//  number used in "Untitled-n"

	bool fStopAllEditions;						//  false by default
	bool fShowSectionBorders;					//  true by default
	bool fNewPublishers;						//  are there any new publishers?

};


//----------------------------------------------------------------------------------------
// TNewSubscriberCommand: TNewSubscriberCommand creates a new subscriber and marks the
// document as changed.
//----------------------------------------------------------------------------------------

class EXPORT_MA TNewSubscriberCommand : public TCommand
{
public:
	TNewSubscriberCommand(CommandNumber itsCommandNumber = cNoCommand, TDocument* itsDocument = NULL);
		// constructor

	virtual ~TNewSubscriberCommand();
		// Destructor
		
	virtual void DoIt();
		// Create a new subscriber
};


//----------------------------------------------------------------------------------------
// TSectionBorderCommand: TSectionBorderCommand is an undoable command which changes the setting
// for the show/ hide border option for the current document when the user chooses Hide/
// Show Borders from the Edit menu
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionBorderCommand : public TCommand
{
public:
	TSectionBorderCommand(CommandNumber itsCommandNumber = cNoCommand, TDocument* itsDocument = NULL);
		// constructor

	virtual ~TSectionBorderCommand();
		// Destructor
		
	virtual void DoIt();
		// tell the document to show or hide the borders for all sections

	virtual void UndoIt();
		// tell the document to undo show or hide the borders for all sections

protected:
	bool fShowSectionBorders;				// value to set
};


//----------------------------------------------------------------------------------------
// TStopAllEditionsCommand: TStopAllEditionsCommand is an undoable command gets posted when the
// user chooses Stop all Editions from the Edit menu. This command toggles the setting for
// the documents fStopAllEditions field.
//----------------------------------------------------------------------------------------

class EXPORT_MA TStopAllEditionsCommand : public TCommand
{
public:
	TStopAllEditionsCommand(CommandNumber itsCommandNumber = cNoCommand, 
											TDocument* itsDocument = NULL);
		// constructor

	virtual ~TStopAllEditionsCommand();
		// Destructor
		
	virtual void DoIt();
		// tell the document to toggle the state of its fStopAllEditions flag

	virtual void UndoIt();
		// tell the document to toggle the state of its fStopAllEditions flag

protected:
	bool fStopAllEditions;					// value to set
};


//----------------------------------------------------------------------------------------
// TSectionAdorner: abstract superclass for section adorners
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionAdorner : public TAdorner
{
	MA_DECLARE_CLONE;
	
public:
	TSectionAdorner(IDType itsID = kNoIdentifier,
						TSection* itsSection = NULL,
						RgnHandle itsBorderRegion = NULL);
		// initializes the section adorner with priority kAdornAfter
		
	virtual ~TSectionAdorner();
		// frees fBorderRegion

	virtual void InvalidateAdorner(TView* itsView); // Override
		// Invalidates the adorner in the TAdornerMgr' TView.

	virtual void DrawBorder(const Pattern& whichPattern);	
		// Draws the adorner's border in the TAdornerMgr' TView.

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline RgnHandle GetBorderRegion() const { return fBorderRegion; }
	inline TSection* GetSection() const { return fSection; }

#if qBackwardCompatible
public:
#else
protected:
#endif

	CTempRegion_AC fBorderRegion;						// the region for the section border

	TSection* fSection;							// the section this adorner adorns
};


//----------------------------------------------------------------------------------------
// TSubscriberAdorner: A specialized adorner for imaging subscriber borders
//----------------------------------------------------------------------------------------

class EXPORT_MA TSubscriberAdorner : public TSectionAdorner
{
	MA_DECLARE_CLONE;
	
public:
	TSubscriberAdorner(IDType itsID = kNoIdentifier,
						TSubscriber* itsSubscriber = NULL,
						RgnHandle itsBorderRegion = NULL) :
	TSectionAdorner(itsID, itsSubscriber, itsBorderRegion)
	{ }

	virtual ~TSubscriberAdorner();
		// Destructor
		
	virtual void DoHighlightSelection(TView* itsView,
											 const CViewRect& area,
											 HLState fromHL,
											 HLState toHL); // Override
		// Adorns the selection before the view's DoHighlightSelection.
};


//----------------------------------------------------------------------------------------
// TPublisherAdorner
//----------------------------------------------------------------------------------------

class EXPORT_MA TPublisherAdorner : public TSectionAdorner
{
	MA_DECLARE_CLONE;
	
public:
	TPublisherAdorner(IDType itsID = kNoIdentifier,
						TPublisher* itsPublisher = NULL,
						RgnHandle itsBorderRegion = NULL) :
	TSectionAdorner(itsID, itsPublisher, itsBorderRegion)
	{ }


	virtual ~TPublisherAdorner();
		// Destructor
		
	virtual void DoHighlightSelection(TView* itsView,
											 const CViewRect& area,
											 HLState fromHL,
											 HLState toHL); // Override
		// Adorns the selection before the view's DoHighlightSelection.
};

//----------------------------------------------------------------------------------------
// TSectionBehavior
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionBehavior : public TBehavior
{
	MA_DECLARE_CLONE;
	
public:
	inline TSectionBehavior(IDType itsIdentifier = kNoIdentifier) : TBehavior(itsIdentifier) { }
		// constructor

	virtual ~TSectionBehavior();
		// Destructor
		
	virtual bool DoMouseCommand(CViewPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint_AC hysteresis);
		// Intercepts mouse commands and traps dbl-clicks in sections.
		// By default, calls Inherited.
	
	virtual IDType GetStandardSignature() const;
		// Returns this class's standard signature.
};

//----------------------------------------------------------------------------------------
// Externally avaibable routines
//----------------------------------------------------------------------------------------

EXPORT_MA void InitUEditionDocument();
	// call this routine to initialize this unit.

#endif


