//----------------------------------------------------------------------------------------
// UView.cpp
// Copyright  1987-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UVIEW__
#include "UView.h"
#endif

// ACS

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif

#if qDebug
	// for DelayFuture_AC
	#ifndef __AGA_AC__
	#include "AGA_AC.h"
	#endif
#endif

// MacApp

#ifndef __PLATFORMMEMORY__
#include "PlatformMemory.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UDRAG__
#include "UDragDrop.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#if qGXViews || qGXPrinting
#	ifndef __GXGRAPHICS__
#	include <GXGraphics.h>
#	endif
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


static const short kRgnHandleTooBig = 16 * 1024;// regions > 16K don't work too well

//========================================================================================
// static data members
//========================================================================================
TView* TView::gFocusedView;
TView* TView::fgSetCursorView;
bool TView::gDrawingPictScrap;
TView* TView::gDrawingPictScrapView;
bool TView::gPrinting;

#if qGXViews || qGXPrinting
gxGraphicsClient TView::gGXGraphicsClient;
#endif

//========================================================================================
// CLASS CSubViewIterator
//========================================================================================
#undef Inherited
#define Inherited CIterator_AC

//----------------------------------------------------------------------------------------
// CSubViewIterator::CSubViewIterator:
//----------------------------------------------------------------------------------------

CSubViewIterator::CSubViewIterator(const TView* theSuperView,
								   bool itsForward) :
	fCurrent(NULL),
	fFirstView(theSuperView ? theSuperView->GetSubView() : NULL),
	fIterateForward(itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CSubViewIterator::First:
//----------------------------------------------------------------------------------------

TView* CSubViewIterator::First()
{
	return fCurrent = fIterateForward ? const_cast<TView*>(fFirstView) : LastView();
}

//----------------------------------------------------------------------------------------
// CSubViewIterator::Next:
//----------------------------------------------------------------------------------------

TView* CSubViewIterator::Next()
{
	return fCurrent ? fCurrent = (fIterateForward ? NextView(fCurrent) : PreviousView(fCurrent)) : NULL;
}

//----------------------------------------------------------------------------------------
// CSubViewIterator::PreviousView:
//----------------------------------------------------------------------------------------

TView* CSubViewIterator::PreviousView(const TView* aView) const
{
	const TView * currentView = fFirstView;
	while (currentView && (NextView(currentView) != aView))
		currentView = NextView(currentView);

	return const_cast<TView*>(currentView);
}

//----------------------------------------------------------------------------------------
// CSubViewIterator::LastView:
//----------------------------------------------------------------------------------------

TView* CSubViewIterator::LastView() const
{
	const TView * currentView = fFirstView;
	while (currentView && NextView(currentView))
		currentView = NextView(currentView);

	return const_cast<TView*>(currentView);
}

//========================================================================================
// CLASS TView
//========================================================================================
#undef Inherited
#define Inherited TCommandHandler

MA_DEFINE_CLASS(TView);

//----------------------------------------------------------------------------------------
// TView constructor
//----------------------------------------------------------------------------------------

TView::TView(const CViewPoint& itsLocation,
				  const CViewPoint& itsSize,
				  SizeDeterminer itsHSizeDet,
				  SizeDeterminer itsVSizeDet) :
	TCommandHandler(NULL),

	fLocation(itsLocation),
	fSize (itsSize),

	fSuperView(NULL),
	fSubView(NULL),
	fNextView(NULL),
	fDocument(NULL),
	fUserArea(0),
	fTemplateVersion(kMARelease13ViewsTemplateVersion),
	fHelpIndex(1),
	fHelpID(kNoResource),
	fCursorID(kNoResource),
	fActiveHL(hlOn),
	fInactiveHL(hlDim),
	fHandlesCursor(true),
	fLetsSubViewsHandleCursor(true),
	fHandlesHelp(true),
	fHandlesFirstClicks(true),
	fLetsSubViewsHandleHelp(true),
	fShown(true),
	fWantsToBeTarget(false),
	fCoordinatesAreInvalid(true),
	fIsOpaque(false),
	fFrameInvalidateDeterminer(frameInvalidateDifference)
#if qDrag
	,
	fDraggable(false),
	fDroppable(false),
	fDragMoveDeterminer(kNeverMove),
	fDragMoveFamily(kNoIdentifier)
#endif // qDrag
#if qWriteTemplateViews
	,
	fTemplateUnknownData(NULL)
#endif // qWriteTemplateViews
{
	fSizeDeterminer[hSel] = itsHSizeDet;
	fSizeDeterminer[vSel] = itsVSizeDet;
}

//----------------------------------------------------------------------------------------
// TView::HandlePostCreate:
//----------------------------------------------------------------------------------------

void TView::HandlePostCreate(TDocument* itsDocument)
{
	if (!fDocument)
		BeInDocument(itsDocument);
#if qDebugMsg
	else if (fDocument)
		::ThrowIfNonObject_AC(itsDocument);
#endif

	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->HandlePostCreate(itsDocument);

	if (GetAdorners())
		for (CAdornerIterator adornIter(this); adornIter; ++adornIter)
			adornIter->AddedToView(this);

	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->DoPostCreate(itsDocument);

	DoPostCreate(itsDocument);
}

//----------------------------------------------------------------------------------------
// TView::DoPostCreate:
//----------------------------------------------------------------------------------------

void TView::DoPostCreate(TDocument* /* itsDocument */)
{
// No longer necessary to register this view with window if draggable here in
// DoPostCreate since the view server will call AddSubView which will take
// care of the registering.
#if 0
#if qDrag
	if (fDroppable && HasDragManager_AC())			// if this view receives drops, register it with its window
	{
		TWindow * itsWindow = GetWindow();
		if (itsWindow)
			itsWindow->RegisterDroppableView(this);
	}
#endif // qDrag
#endif
}

//----------------------------------------------------------------------------------------
// TView copy constructor
//----------------------------------------------------------------------------------------

TView::TView(const TView& inView) :
	TCommandHandler(inView),

	fLocation(inView.fLocation),
	fSize(inView.fSize),

	fSuperView(NULL),
	fSubView(NULL),
	fNextView(NULL),
	fDocument(NULL),
	fUserArea(inView.fUserArea),
	fTemplateVersion(inView.fTemplateVersion),
	fHelpIndex(inView.fHelpIndex),
	fHelpID(inView.fHelpID),
	fCursorID(inView.fCursorID),
	fActiveHL(inView.fActiveHL),
	fInactiveHL(inView.fInactiveHL),
	fHandlesCursor(inView.fHandlesCursor),
	fLetsSubViewsHandleCursor(inView.fLetsSubViewsHandleCursor),
	fHandlesHelp(inView.fHandlesHelp),
	fHandlesFirstClicks(inView.fHandlesFirstClicks),
	fLetsSubViewsHandleHelp(inView.fLetsSubViewsHandleHelp),
	fShown(inView.fShown),
	fWantsToBeTarget(inView.fWantsToBeTarget),
	fCoordinatesAreInvalid(inView.fCoordinatesAreInvalid),
	fIsOpaque(inView.fIsOpaque),
	fFrameInvalidateDeterminer(inView.fFrameInvalidateDeterminer)
#if qDrag
	,
	fDraggable(inView.fDraggable),
	fDroppable(inView.fDroppable),
	fDragMoveDeterminer(inView.fDragMoveDeterminer),
	fDragMoveFamily(inView.fDragMoveFamily)
#endif // qDrag
{
	fSizeDeterminer[hSel] = inView.fSizeDeterminer[hSel];
	fSizeDeterminer[vSel] = inView.fSizeDeterminer[vSel];

	if (inView.HasSubViews())
		for (CSubViewIterator iter(&inView); iter; ++iter)
			AddSubView(static_cast<TView*>(iter->Clone()));

	if (inView.fDrawingEnvironment)
		fDrawingEnvironment = static_cast<TDrawingEnvironment*>(inView.fDrawingEnvironment->Clone());

	for (CAdornerIterator iter(&inView); iter; ++iter)
		AddAdorner(iter->GetFreeOnDeletion() ? static_cast<TAdorner*>(iter->Clone()) : iter, inView.fAdorners->PriorityAt(iter.CurrentIndex()), false);
}

//----------------------------------------------------------------------------------------
// TView::Free:
//----------------------------------------------------------------------------------------

TView::~TView()
{
	while (fSubView)
		delete fSubView;

	InvalidateFocus();
	if (gDispatcher)
		gDispatcher->InvalidateMouseRegions();		// Must re-calc cursor and help rgn.

	if (fgSetCursorView == this)
		fgSetCursorView = NULL;

	if (gDispatcher && this == GetTarget())		// Fixup the target chain
		SetTarget(gDispatcher);

	if (fSuperView)
	{
		fSuperView->RemoveSubView(this);
		fSuperView = NULL;						// redundant, but safe
	}

	DeleteAllAdorners();

	RemoveFromDocument();
}

//----------------------------------------------------------------------------------------
// TView::DeleteAllAdorners:
//----------------------------------------------------------------------------------------

void TView::DeleteAllAdorners()
{
	if (fAdorners)
		for (CAdornerIterator iter(this); iter; ++iter)
		{
			iter->RemovedFromView(this);
			// anAdorner might get freed here
			if (iter->GetFreeOnDeletion())
				delete iter.Current();
		}
}

//----------------------------------------------------------------------------------------
// TView::HighlightAdorners:
//----------------------------------------------------------------------------------------

void TView::HighlightAdorners(const CViewRect& area,
							  HLState fromHL,
							  HLState toHL)
{
	PenState savePenState;
	HLState localFromHL = fromHL;
	HLState localToHL = toHL;

	// The direction of iteration is controlled by the following state transition diagram:
	//
	// toHL
	// 				  || 	OFF  	 ||		   DIM		  || 	  ON
	//  --------------||-------------||-------------------||-------------------
	//  		OFF	  || 	(NA)	 ||		 Forward 	  || 	Forward
	//  --------------||-------------||-------------------||-------------------
	//  fromHL  DIM	  ||   Backward	 ||		  (NA)		  || 	Forward
	//  --------------||-------------||-------------------||-------------------
	//  		ON	  ||   Backward	 ||		Backward	  || 	 (NA)
	//  --------------||-------------||-------------------||-------------------

	PenNormal();
	GetPenState(&savePenState);

	if ((fromHL == hlOn && (toHL == hlDim || toHL == hlOff)) || (fromHL == hlDim && toHL == hlOff))
	{
		for (CAdornerIterator iter(this, kIterateBackward_AC); iter; ++iter)
			if (iter->DoesAdorn(this))
			{
				SetPenState(&savePenState);
				iter->DoHighlightSelection(this, area, localFromHL, localToHL);
			}
	}
	else if (((fromHL == hlOff || fromHL == hlDim) && toHL == hlOn) || (fromHL == hlOff && toHL == hlDim))
	{
		for (CAdornerIterator iter(this); iter; ++iter)
			if (iter->DoesAdorn(this))
			{
				SetPenState(&savePenState);
				iter->DoHighlightSelection(this, area, localFromHL, localToHL);
			}
	}
}

//----------------------------------------------------------------------------------------
// TView::Activate:
//----------------------------------------------------------------------------------------

void TView::Activate(bool entering)
{
	if (fAdorners && IsDrawable())
	{
		CViewRect visRect(GetDrawableRect());
		if (entering)
			HighlightAdorners(visRect, fInactiveHL, fActiveHL);
		else
			HighlightAdorners(visRect, fActiveHL, fInactiveHL);
	}

	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->Activate(entering);
}

//----------------------------------------------------------------------------------------
// TView::AddAdorner:
//----------------------------------------------------------------------------------------

void TView::AddAdorner(TAdorner* anAdorner,
					   AdornPriority itsPriority,
					   bool invalidate)
{
	{
		CTempObjectAllocation tempObjectAllocation(true);
		fAdorners = MakeAdornerList();
		fAdorners->AddAdorner(anAdorner, itsPriority);
	}

	anAdorner->AddedToView(this);

	if (invalidate)
		anAdorner->InvalidateAdorner(this);
}

//----------------------------------------------------------------------------------------
// TView::DeleteAdorner:
//----------------------------------------------------------------------------------------

void TView::DeleteAdorner(TAdorner* anAdorner,
						  bool invalidate)
{
	if (fAdorners)
	{
		for (CAdornerIterator iter(this); iter; ++iter)
			if (iter == anAdorner)
			{
				if (invalidate)
					iter->InvalidateAdorner(this);
				iter->RemovedFromView(this);
				// anAdorner might get freed here
				if (iter->GetFreeOnDeletion())
					delete iter.Current();
				fAdorners->DeleteElementsAt(iter.CurrentIndex(), 1);
			}

		// dispose of fAdorners if it only contains the default gDrawAdorner
		if ((fAdorners->GetSize() == 1) && (AdornerAt(1) == &TAdorner::fgDrawAdorner))
			fAdorners.reset();
	}
}

//----------------------------------------------------------------------------------------
// TView::DeleteAdornerByID:
//----------------------------------------------------------------------------------------

void TView::DeleteAdornerByID(IDType itsID,
							  bool invalidate)
{
	if (fAdorners)
	{
		for (CAdornerIterator iter(this); iter; ++iter)
			if (iter->GetIdentifier() == itsID)
			{
				if (invalidate)
					iter->InvalidateAdorner(this);
				iter->RemovedFromView(this);
				// anAdorner might get freed here
				if (iter->GetFreeOnDeletion())
					delete iter.Current();
				fAdorners->DeleteElementsAt(iter.CurrentIndex(), 1);
			}

		// dispose of fAdorners if it only contains the default TAdorner::fgDrawAdorner
		if ((fAdorners->GetSize() == 1) && (AdornerAt(1) == &TAdorner::fgDrawAdorner))
			fAdorners.reset();
	}
}

//----------------------------------------------------------------------------------------
// TView::AdornerAt:
//----------------------------------------------------------------------------------------

TAdorner* TView::AdornerAt(ArrayIndex_AC i) const
{
	return fAdorners ? fAdorners->AdornerAt(i) : NULL;
}

//----------------------------------------------------------------------------------------
// TView::AdornerWithID:
//----------------------------------------------------------------------------------------

TAdorner* TView::AdornerWithID(IDType itsID) const
{
	TAdorner* returnVal = NULL;

	if (fAdorners)
		for (CAdornerIterator iter(this); iter; ++iter)
			if (iter->GetIdentifier() == itsID)
			{
				returnVal = iter;
				break;
			}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::Dim:
//----------------------------------------------------------------------------------------

void TView::Dim()
{
#if qDebug
	AssumeFocused();
#endif

	PenPat(&qd.gray);
	PenMode(patBic);
	PaintRect(&GetQDExtent());
}

//----------------------------------------------------------------------------------------
// TView::GetAdornExtent:
//----------------------------------------------------------------------------------------

CViewRect TView::GetAdornExtent()
{
	return GetExtent();
}

//----------------------------------------------------------------------------------------
// TView::GetEventNumber:
//----------------------------------------------------------------------------------------

EventNumber TView::GetEventNumber()
{
	return cNoEvent;
}

//----------------------------------------------------------------------------------------
// TView::Hilite:
//----------------------------------------------------------------------------------------

void TView::Hilite()
{
#if qDebug
	AssumeFocused();
#endif

	InvertRect(&GetQDExtent());
}

//----------------------------------------------------------------------------------------
// TView::MakeAdornerList:
//----------------------------------------------------------------------------------------

TAdornerList* TView::MakeAdornerList()
{
	TAdornerList* returnVal = NULL;

	if (fAdorners)
		returnVal = fAdorners;
	else
	{
		CAutoPtr_AC<TAdornerList> anAdornerList(TH_new TAdornerList);

		anAdornerList->AddAdorner(&TAdorner::fgDrawAdorner, kDrawView);// default "Draw" adorner

		returnVal = anAdornerList.release();
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::FindParentView
//----------------------------------------------------------------------------------------

TView* TView::FindParentView(IDType itsID)
{
	TView * parentView = this;

	while (parentView && parentView->fIdentifier != itsID)
		parentView = parentView->fSuperView;

	return parentView;
}

//----------------------------------------------------------------------------------------
// TView::FindPeerView
//----------------------------------------------------------------------------------------

TView* TView::FindPeerView(IDType itsID)
{
	TView * returnVal = NULL;

	for (CSubViewIterator iter(fSuperView); iter; ++iter)
		if (iter->fIdentifier == itsID)
		{
			returnVal = iter;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::InsertSubView:
//----------------------------------------------------------------------------------------

void TView::InsertSubView(TView* theSubView,
						  bool last)
{
#if qDebug
	::ThrowIfNonObject_AC(theSubView);
#endif

	if (last && fSubView)
	{
		// place at the tail of the list
		register TView* lastView = fSubView;
		register TView* nextView = lastView->fNextView;
		while (nextView)
		{
			lastView = nextView;
			nextView = lastView->fNextView;
		}
		lastView->fNextView = theSubView;
		theSubView->fNextView = NULL;
	}
	else										// first
	{
		theSubView->fNextView = fSubView;
		fSubView = theSubView;
	}
}

//----------------------------------------------------------------------------------------
// TView::InsertSubView:
//----------------------------------------------------------------------------------------

void TView::InsertSubView(TView* theSubView, TView* beforeView)
{
#if qDebug
	::ThrowIfNonObject_AC(theSubView);
#endif

	if (!fSubView)
		beforeView = NULL; // if there are no views, there is nothing to insert before
		
	if (fSubView == beforeView)
	{
		fSubView = theSubView;
	}
	else
	{
		register TView* lastView = fSubView;
		register TView* nextView = lastView->fNextView;
		while (nextView && nextView != beforeView)
		{
			lastView = nextView;
			nextView = lastView->fNextView;
		}
		lastView->fNextView = theSubView;
	}

	theSubView->fNextView = beforeView; // reconnect next to linked list
}

//----------------------------------------------------------------------------------------
// TView::DeleteSubView:
//----------------------------------------------------------------------------------------

void TView::DeleteSubView(TView* theSubView)
{
#if qDebug
	::ThrowIfNonObject_AC(theSubView);
#endif

	if (fSubView)
	{
		if (fSubView == theSubView)
			fSubView = theSubView->fNextView;
		else
		{
			register TView* lastView = fSubView;
			register TView* nextView = fSubView->fNextView;
			while (nextView && (nextView != theSubView))
			{
				lastView = nextView;
				nextView = lastView->fNextView;
			}

			if (nextView)
				lastView->fNextView = theSubView->fNextView;
			else
			{
#if qDebug
				::ProgramBreak_AC("Delete subview not in list!");
#endif

			}
		}

		theSubView->fNextView = NULL;
	}

}

//----------------------------------------------------------------------------------------
// TView::AddedASubView:
//----------------------------------------------------------------------------------------

void TView::AddedASubView(TView* newSubView)
{
	if (fSuperView)
		fSuperView->AddedASubView(newSubView);
}

//----------------------------------------------------------------------------------------
// TView::AddSubView:
//----------------------------------------------------------------------------------------

void TView::AddSubView(TView* theSubView)
{
	if (theSubView)
	{
		InsertSubView(theSubView);

	//	theSubView->BeInDocument(fDocument);
		theSubView->BeInSuperView(this);
		AddedASubView(theSubView);
	}
}

//----------------------------------------------------------------------------------------
// TView::AdjustFrame:
//----------------------------------------------------------------------------------------

void TView::AdjustFrame()
{
	CViewRect oldFrame(GetFrame());
	CViewRect newFrame(oldFrame);
	ComputeFrame(newFrame);

	if (newFrame != oldFrame)
	{
		SetFrame(newFrame, kInvalidate);
		DoPagination();
	}
	else if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->AdjustFrame();
}

//----------------------------------------------------------------------------------------
// TView::AssumeFocused:
//----------------------------------------------------------------------------------------
#if qDebug

void TView::AssumeFocused() const
{
	if (gAssumeFocused)
		if (!IsFocused())
		{
#if qDebugMsg
			ProgramReport_AC(false, ", ClassName=%s\n", typeid(*this).name());
			if (gFocusedView)
			{
				ProgramReport_AC("View actually focused is:\n");
				ProgramReport_AC(false, ", ClassName=%s\n", MA_GET_CLASS_NAME(*gFocusedView));
			}
#endif
			::ProgramReport_AC("Failed AssumeFocused");
		}
}
#endif

//----------------------------------------------------------------------------------------
// TView::AttachPrintHandler:
//----------------------------------------------------------------------------------------

void TView::AttachPrintHandler(TPrintHandler* itsPrintHandler)
{
	AddAdorner(&TAdorner::fgPrintAdorner, kAdornLast, false);// adorner calls DoDrawPrintFeedback
	AddBehavior(itsPrintHandler);
	DoCheckPrinter();
}

//----------------------------------------------------------------------------------------
// TView::DetachPrintHandler:
//----------------------------------------------------------------------------------------

void TView::DetachPrintHandler(TPrintHandler* itsPrintHandler)
{
	DeleteAdorner(&TAdorner::fgPrintAdorner, false);
	RemoveBehavior(itsPrintHandler);
	DoCheckPrinter();
}

//----------------------------------------------------------------------------------------
// TView::GetPrintHandler:
//----------------------------------------------------------------------------------------

TPrintHandler* TView::GetPrintHandler()
{
	// This can return NULL.

	return dynamic_cast<TPrintHandler*>(GetBehaviorWithIdentifier(kPrintBehaviorID));
}

//----------------------------------------------------------------------------------------
// TView::BeInPort:
//----------------------------------------------------------------------------------------

void TView::BeInPort(GrafPtr itsPort)
{
	if (itsPort && GetPrintHandler())
		DoCheckPrinter();

	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->BeInPort(itsPort);

	InvalidateCoordinates();
}

//----------------------------------------------------------------------------------------
// TView::BeInDocument:
//----------------------------------------------------------------------------------------

void TView::BeInDocument(TDocument* itsDocument)
{
	if (itsDocument != fDocument)
	{
		if (fDocument) {
			fDocument->DeleteView(this);
			fDocument = NULL;
		}
		if (itsDocument)
		{
			itsDocument->AddView(this);
			fDocument = itsDocument;
		}
	}
}

//----------------------------------------------------------------------------------------
// TView::RemoveFromDocument:
//----------------------------------------------------------------------------------------

void TView::RemoveFromDocument()
{
	if (fDocument)
	{
		fDocument->DeleteView(this);
		fDocument = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TView::BeInScroller:
//----------------------------------------------------------------------------------------

void TView::BeInScroller(TScroller* itsScroller)
{
	// convert the size to superview location.

	if (itsScroller)
		itsScroller->SetScrollLimits(fSize + fLocation, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TView::BeInSuperView:
//----------------------------------------------------------------------------------------

void TView::BeInSuperView(TView* itsSuperview)
{
	if (fSuperView != itsSuperview)
	{
		TView * oldSuperView = fSuperView;

		fSuperView = itsSuperview;
		InvalidateFocus();						// the subview may have been focused

		if (itsSuperview)
		{
			if (!fNextHandler)					// If necessary,
				fNextHandler = itsSuperview;	// hook this view up to an event hander chain.
			BeInPort(GetGrafPort());
		}
		else
		{
			// Removing this view from its superview
			if (fNextHandler == oldSuperView)	// If next event handler is the former superview
				fNextHandler = NULL;			// take this view out of the event handler chain.
			BeInPort(NULL);
		}
	}
}

//----------------------------------------------------------------------------------------
// TView::FindTarget:
//----------------------------------------------------------------------------------------

TView* TView::FindTarget()
{
	TView * foundView = NULL;

	if (WantsToBeTarget())
		foundView = this;
	else if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
		{
			foundView = iter->FindTarget();
			if (foundView)
				break;
		}

	return foundView;
}

//----------------------------------------------------------------------------------------
// TView::SetTargetSelection:
//----------------------------------------------------------------------------------------

void TView::SetTargetSelection(bool redraw)
{
	ScrollSelectionIntoView(redraw);
}

//----------------------------------------------------------------------------------------
// TView::WantsToBeTarget:
//----------------------------------------------------------------------------------------

bool TView::WantsToBeTarget()				// Override

{
	return fWantsToBeTarget;
}

//----------------------------------------------------------------------------------------
// TView::ContainsSubView:
//----------------------------------------------------------------------------------------

bool TView::ContainsSubView(TView* aSubView) const
{
	bool foundView = false;

	if (HasSubViews())
	{
		for (TView * viewToTest = aSubView; viewToTest; viewToTest = viewToTest->fSuperView)
		{
			if (viewToTest->fSuperView == this)
			{
				foundView = true;
				break;
			}
		}
	}

	return foundView;
}

//----------------------------------------------------------------------------------------
// TView::CalcMinFrame:
//----------------------------------------------------------------------------------------

CViewRect TView::CalcMinFrame()
{
	return GetFrame();
}

//----------------------------------------------------------------------------------------
// TView::ClipFurtherTo:
//----------------------------------------------------------------------------------------

void TView::ClipFurtherTo(RgnHandle rRgn,
						  CPoint_AC deltaOrg)
{
#if qDebug
	AssumeFocused();
#endif

	SectRgn(GetClipRegion(qd.thePort), rRgn, rRgn);
	if (deltaOrg != gZeroPt)
		OffsetRgn(rRgn, deltaOrg.h, deltaOrg.v);
	SetClip(rRgn);
}

//----------------------------------------------------------------------------------------
// TView::OffsetAndClipFurtherTo:
//----------------------------------------------------------------------------------------

void TView::OffsetAndClipFurtherTo(RgnHandle rRgn,
								   CPoint_AC deltaOrg)
{
#if qDebug
	AssumeFocused();
#endif

	// First offset the clip Rgn
	if (deltaOrg != gZeroPt)
		OffsetRgn(GetClipRegion(qd.thePort), deltaOrg.h, deltaOrg.v);

	SectRgn(GetClipRegion(qd.thePort), rRgn, rRgn);
	SetClip(rRgn);
}

//----------------------------------------------------------------------------------------
// TView::Close:
//----------------------------------------------------------------------------------------

void TView::Close()
{
	Changed(mClosed, this);

	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->Close();
}

//----------------------------------------------------------------------------------------
// TView::ComputeFrame:
//----------------------------------------------------------------------------------------

void TView::ComputeFrame(CViewRect& newFrame)
{
	CViewRect minFrame;								// will be calculated only when necessary
	bool haveMinFrame = false;

	// Adjust view size based on size determiners.
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		ViewCoordinate dimension;

		switch (fSizeDeterminer[vhs])
		{
			case sizeFixed:
				dimension = fSize[vhs];
				break;

			case sizeRelSuperView:
				dimension = newFrame.GetSize()[vhs];// newFrame set in
				// SuperViewChangedSize
				break;

			case sizeSuperView:
				if (fSuperView)
					dimension = fSuperView->fSize[vhs];
				else
				{
					if (!haveMinFrame)
					{
						minFrame = CalcMinFrame();
						haveMinFrame = true;
					}
					dimension = minFrame.GetSize()[vhs];// Set dimension to a default
				}
				break;

			case sizePage:
				{
					// If there is no printhandler we can defer until one is attached
					TPrintHandler * itsPrintHandler = GetPrintHandler();
					if (itsPrintHandler)
						dimension = itsPrintHandler->GetViewPerPage()[vhs];
					else
					{
						if (!haveMinFrame)
						{
							minFrame = CalcMinFrame();
							haveMinFrame = true;
						}
						dimension = minFrame.GetSize()[vhs];// Set dimension to a default
					}
				}
				break;

			case sizeFillPages:
				{
					if (!haveMinFrame)
					{
						minFrame = CalcMinFrame();
						haveMinFrame = true;
					}
					// If there is no printhandler we can defer until one is attached
					TPrintHandler * itsPrintHandler = GetPrintHandler();
					if (itsPrintHandler)
						dimension = RoundUp(minFrame.GetSize()[vhs], static_cast<short>(itsPrintHandler->GetViewPerPage()[vhs]));
					else
						dimension = minFrame.GetSize()[vhs];// Set dimension to a default
				}
				break;

			case sizeVariable:					// For sizeVariable dimension is already set correctly
			default:
				if (!haveMinFrame)
				{
					minFrame = CalcMinFrame();
					haveMinFrame = true;
				}
				dimension = minFrame.GetSize()[vhs];// Set dimension to a default
				break;
		}
		newFrame[botRight][vhs] = newFrame[topLeft][vhs] + dimension;
	}
}

//----------------------------------------------------------------------------------------
// TView::ContainsClipType:
//----------------------------------------------------------------------------------------

bool TView::ContainsClipType(ResType aType)
{
	long offset;

	return (GetScrap(NULL, aType, &offset) > 0);
}

//----------------------------------------------------------------------------------------
// TView::ContainsMouse:
//----------------------------------------------------------------------------------------

bool TView::ContainsMouse(const CViewPoint& theMouse)
{
	CTempRegion_AC extentRgn;
	GetExtentRegion(extentRgn);

	return (IsShown() && PtInRgn(ViewToQD(theMouse), extentRgn));
}

//----------------------------------------------------------------------------------------
// TView::CountSubViews:
//----------------------------------------------------------------------------------------

short TView::CountSubViews() const
{
	short count = 0;

	register TView* subView = fSubView;
	while (subView)
	{
		++count;
		subView = subView->fNextView;
	}

	return count;
}

//----------------------------------------------------------------------------------------
// TView::DoBreakFollowing:
//----------------------------------------------------------------------------------------

ViewCoordinate TView::DoBreakFollowing(VHSelect vhs,
									ViewCoordinate previousBreak,
									bool& automatic)
{
	ViewCoordinate returnVal = 0;

	TPrintHandler * itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
		returnVal = itsPrintHandler->BreakFollowing(vhs, previousBreak, automatic);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::DoCalcPageStrips:
//----------------------------------------------------------------------------------------

CViewPoint TView::DoCalcPageStrips()
{
	TPrintHandler * itsPrintHandler = GetPrintHandler();

	return itsPrintHandler ? itsPrintHandler->CalcPageStrips() : gZeroVPt;
}

//----------------------------------------------------------------------------------------
// TView::DoCalcViewPerPage:
//----------------------------------------------------------------------------------------

CViewPoint TView::DoCalcViewPerPage()
{
	TPrintHandler * itsPrintHandler = GetPrintHandler();

	return itsPrintHandler ? itsPrintHandler->CalcViewPerPage() : fSize;
}

//----------------------------------------------------------------------------------------
// TView::DoCheckPrinter:
//----------------------------------------------------------------------------------------

void TView::DoCheckPrinter()
{
	// If you are sizeVariable and are going to want to depend on the printed page geometry
	// then you will want to override this method and add an appropriate test

	TPrintHandler * itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
	{
		if (itsPrintHandler->fShowBreaks || fSizeDeterminer[vSel] == sizePage || fSizeDeterminer[vSel] == sizeFillPages || fSizeDeterminer[hSel] == sizePage || fSizeDeterminer[hSel] == sizeFillPages)
			itsPrintHandler->Update();
	}
}

//----------------------------------------------------------------------------------------
// TView::DoDrawPrintFeedback:
//----------------------------------------------------------------------------------------

void TView::DoDrawPrintFeedback(const CViewRect& area)
{
#if qDebug
	AssumeFocused();
#endif

	TPrintHandler * itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
		itsPrintHandler->DrawPrintFeedback(area);
}

//----------------------------------------------------------------------------------------
// TView::DoDrawPageBreak:
//----------------------------------------------------------------------------------------

void TView::DoDrawPageBreak(VHSelect vhs,
							short whichBreak,
							ViewCoordinate loc,
							bool automatic)
{
	TPrintHandler * itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
		itsPrintHandler->DrawPageBreak(vhs, whichBreak, loc, automatic);
}

//----------------------------------------------------------------------------------------
// TView::HandlesCursor:
//----------------------------------------------------------------------------------------

bool TView::HandlesCursor() const
{
	bool result = (fHandlesCursor && IsEnabled()) || LetsSubViewsHandleCursor()
#if qDrag
	||fDraggable
#endif
	;

	return result;
}

//----------------------------------------------------------------------------------------
// TView::LetsSubViewsHandleCursor:
//----------------------------------------------------------------------------------------

bool TView::LetsSubViewsHandleCursor() const
{
	return fLetsSubViewsHandleCursor && HasSubViews();
}

//----------------------------------------------------------------------------------------
// TView::HandlesHelp:
//----------------------------------------------------------------------------------------

bool TView::HandlesHelp() const
{
	return fHandlesHelp || LetsSubViewsHandleHelp();
}

//----------------------------------------------------------------------------------------
// TView::LetsSubViewsHandleHelp:
//----------------------------------------------------------------------------------------

bool TView::LetsSubViewsHandleHelp() const
{
	return fLetsSubViewsHandleHelp && HasSubViews();
}

//----------------------------------------------------------------------------------------
// TView::UpdateSelection:
//----------------------------------------------------------------------------------------

void TView::UpdateSelection(const CViewPoint&		/* theMouse */ ,
							TToolboxEvent*		/* event */ )
{
}

//----------------------------------------------------------------------------------------
// TView::DoHighlightSelection:
//----------------------------------------------------------------------------------------

void TView::DoHighlightSelection(HLState		/* fromHL */,
								 HLState		/* toHL */)
{
}

//----------------------------------------------------------------------------------------
// TView::GetCurrentHL:
//----------------------------------------------------------------------------------------

HLState TView::GetCurrentHL()
{
	return IsActive() ? fActiveHL : fInactiveHL;
}

//----------------------------------------------------------------------------------------
// TView::DoMouseCommand:
//----------------------------------------------------------------------------------------

void TView::DoMouseCommand(CViewPoint&				/* theMouse */ ,
						   TToolboxEvent*		/* event */ ,
						   CPoint_AC				/* hysteresis */)
{
}

//----------------------------------------------------------------------------------------
// TView::DoMouseUp:
//----------------------------------------------------------------------------------------

void TView::DoMouseUp(CViewPoint&					/* theMouse */ ,
					  TToolboxEvent*			/* event */ ,
					  CPoint_AC					/* hysteresis */)
{
}

//----------------------------------------------------------------------------------------
// TView::DoPagination:
//----------------------------------------------------------------------------------------

void TView::DoPagination()
{
	TPrintHandler * itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
		itsPrintHandler->RedoPageBreaks();
}

//----------------------------------------------------------------------------------------
// TView::DoPrinterChanged:
//----------------------------------------------------------------------------------------

void TView::DoPrinterChanged()
{
	TPrintHandler * itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
		itsPrintHandler->PrinterChanged();
}

//----------------------------------------------------------------------------------------
// TView::DoSetPageOffset:
//----------------------------------------------------------------------------------------

void TView::DoSetPageOffset(const CViewPoint& coord)
{
	TPrintHandler * itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
		itsPrintHandler->SetPageOffset(coord);
}

//----------------------------------------------------------------------------------------
// TView::DoSetCursor:
//----------------------------------------------------------------------------------------

void TView::DoSetCursor(const CViewPoint& localPoint,
						RgnHandle cursorRegion)
{
#if qDebug
	AssumeFocused();
#endif

#if qDrag
	RgnHandle dragCursorRegion = NULL;
	if (!HasDragManager_AC() || !DoSetDragCursor(localPoint, cursorRegion, dragCursorRegion))
	{
#endif
		ResNumber theCursorID = GetCursorID();
		if (theCursorID != kNoResource)
		{
			CPoint_AC localQDPoint(ViewToQD(localPoint));
			if (PtInRgn(localQDPoint, cursorRegion))
			{
				CursHandle aCursor = GetCursor(theCursorID);
				::ThrowIfNULLResource_AC((Handle)aCursor);
				SetCursor(*aCursor);
			}
			else
				SetCursor(&(qd.arrow));
		}
		else
			SetCursor(&(qd.arrow));

#if qDrag
		// remove the drag region from the cursor region so the cursor
		// will be updated if it moves into or out of the drag region
		if (dragCursorRegion)
			DiffRgn(cursorRegion, dragCursorRegion, cursorRegion);
	}
	DisposeIfRgnHandle(dragCursorRegion);
#endif

	// If you claim the cursor don't forget to _PROPERLY_ calculate the cursorRegion or
	// WaitNextEvent might give you a lovely mouse-moved event shower. (Lather up!)

	// P.S. In debug this is checked for in our original caller: TApplication.TrackMouse.

	// If you need to do any direct drawing or other good stuff put it in an override

	fgSetCursorView = this; // It's us, we claimed it
}

//----------------------------------------------------------------------------------------
// TView::DoUnsetCursor:
//----------------------------------------------------------------------------------------

void TView::DoUnsetCursor()
{
	// If you need to do any direct drawing or other good stuff put it in an override
}

//----------------------------------------------------------------------------------------
// TView::DoShowBalloon:
//----------------------------------------------------------------------------------------

void TView::DoShowBalloon(const CViewPoint&			/* localPoint */ ,
						  RgnHandle helpRegion,
						  const HMMessageRecord& helpMessage,
						  CPoint_AC localQDTip,
						  const CRect_AC&			/* localQDRect */ ,
						  short balloonVariant)
{
	OSErr err = (HMShowBalloon(&helpMessage, LocalToGlobal(localQDTip), NULL, NULL, 0, balloonVariant, kHMRegularWindow));
	if (err != hmBalloonAborted)
		::ThrowIfOSErr_AC(err);

	// Intersect with visible region. Excludes the balloon region.
	SectRgn(TWindow::GetVisRegion(qd.thePort), helpRegion, helpRegion);
	SectRgn(GetClipRegion(qd.thePort), helpRegion, helpRegion);
}

//----------------------------------------------------------------------------------------
// TView::DoShowHelp:
//----------------------------------------------------------------------------------------

void TView::DoShowHelp(const CViewPoint& localPoint,
					   RgnHandle helpRegion)
{
	if (GetHelpID() != kNoResource)
	{
		GetDefaultHelpRegion(localPoint, helpRegion);

		try
		{
			HMMessageRecord helpMessage;
			CPoint_AC tip;
			CRect_AC hotRect;
			short balloonVariant;

			GetHelpParameters(localPoint, helpRegion, helpMessage, tip, hotRect, balloonVariant);
			DoShowBalloon(localPoint, helpRegion, helpMessage, tip, hotRect, balloonVariant);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);

			if (theException.GetError() != hmSkippedBalloon)
				throw;

			// fall through
		}

	}
}

//----------------------------------------------------------------------------------------
// TView::GetDefaultCursorRegion:
//----------------------------------------------------------------------------------------
void TView::GetDefaultCursorRegion(const CViewPoint&/* localPoint */  ,
								   RgnHandle cursorRegion)
{
	// If we get all the way to this view without any other view having claimed the cursor
	// its because no superview handled it in a "HandleCursor" and no subview handled it
	// in a "DoSetCursor" thus there is a _very_ high probability that this view should
	// set the cursor. Just in case though, we remove the frames.
	//
	// If a containing view wishes to set the cursor for itself && all its subview, the
	// place to do that is in our caller: HandleCursor. See HandleCursor in the homebrew
	// controls TDialogView subclass in DemoDialogs.
	//
	// This strategy lets MacApp provide the _largest_ possible cursor rgn, as a default.
	// This way windows, controls, background views, etc can have nice big cursor rgns but
	// the developer can wrest control if they care to.

#if qDebug
	AssumeFocused();
#endif

	GetExtentRegion(cursorRegion);

	// We won't claim subviews' territory
	if (LetsSubViewsHandleCursor())
	{
		bool rgnHandleWasTooBig = false;
		CTempRegion_AC subViewFrameRgn;

		for (CSubViewIterator iter(this); iter; ++iter)
			if (iter->HandlesCursor() && !rgnHandleWasTooBig)
			{
				iter->GetExtentRegion(subViewFrameRgn);// Remember, the frame is in OUR
				// coordinate system
				iter->LocalToSuperView(subViewFrameRgn);
				DiffRgn(cursorRegion, subViewFrameRgn, cursorRegion);// Remove the subview from the claimed rgn

				if ((GetHandleSize((Handle)cursorRegion) > kRgnHandleTooBig) || (MemError() != noErr))
				{
					rgnHandleWasTooBig = true;
					SetEmptyRgn(cursorRegion);
				}
			}
	}
	// If you claim the cursor don't forget to _PROPERLY_ calculate the cursorRegion or
	// WaitNextEvent might give you a lovely mouse-moved event shower. (Lather up!)

	// P.S. In debug this is checked for in our original caller: TApplication.TrackCursor.
}

//----------------------------------------------------------------------------------------
// TView::GetDefaultHelpRegion:
//----------------------------------------------------------------------------------------
void TView::GetDefaultHelpRegion(const CViewPoint&	/* localPoint */ ,
								 RgnHandle helpRegion)
{
#if qDebug
	AssumeFocused();
#endif

	GetExtentRegion(helpRegion);

	// We won't claim subviews' territory
	if (LetsSubViewsHandleHelp())
	{
		bool rgnHandleWasTooBig = false;
		CTempRegion_AC subViewFrameRgn;

		for (CSubViewIterator iter(this); iter; ++iter)
			if (iter->HandlesHelp() && !rgnHandleWasTooBig)
			{
				iter->GetExtentRegion(subViewFrameRgn);// Remember, the frame is in OUR
				// coordinate system
				iter->LocalToSuperView(subViewFrameRgn);
				DiffRgn(helpRegion, subViewFrameRgn, helpRegion);// Remove the subview from the claimed rgn

				if ((GetHandleSize((Handle)helpRegion) > kRgnHandleTooBig) || (MemError() != noErr))
				{
					rgnHandleWasTooBig = true;
					SetEmptyRgn(helpRegion);
				}
			}
	}
}

//----------------------------------------------------------------------------------------
// TView::GetLocalOffsetInSuper:
//----------------------------------------------------------------------------------------

CViewPoint TView::GetLocalOffsetInSuper() const
{
	return fLocation - fTranslation;
}

//----------------------------------------------------------------------------------------
// TView::SetLocalOrigin:
//----------------------------------------------------------------------------------------

void TView::SetLocalOrigin(const CViewPoint& newOrigin,
						   bool redraw)
{
	if (fTranslation != -newOrigin)
	{
		fTranslation = -newOrigin;
		InvalidateFocus();
		InvalidateCoordinates();
		if (redraw)
			ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TView::GetUserSelection:
//----------------------------------------------------------------------------------------

TDesignator* TView::GetUserSelection()
{
	TDesignator*	theDesignator = NULL;

	if (fSuperView)
		theDesignator = fSuperView->GetUserSelection();
	else if (fDocument)
		theDesignator = fDocument->GetUserSelection();

	return theDesignator;
}

//----------------------------------------------------------------------------------------
// TView::SetUserSelection:
//----------------------------------------------------------------------------------------

void TView::SetUserSelection(TDesignator* newSelection)
{
	if (fSuperView)
		fSuperView->SetUserSelection(newSelection);
	else if (fDocument)
		fDocument->SetUserSelection(newSelection);
}

//----------------------------------------------------------------------------------------
// TView::UserSelectionChanged:
//----------------------------------------------------------------------------------------

void TView::UserSelectionChanged(TView* changedView)
{
	if (fSuperView)
		fSuperView->UserSelectionChanged(changedView);
	else if (fDocument)
		fDocument->UserSelectionChanged(changedView);
}

//----------------------------------------------------------------------------------------
// TView::ScrollSelectionIntoView:
//----------------------------------------------------------------------------------------

void TView::ScrollSelectionIntoView(bool redraw)
{
	// As a default, reveal the view's extent
	RevealRect(GetExtent(), fSize, redraw);
}

//----------------------------------------------------------------------------------------
// TView::DoMakeNewDrawingEnvironment:
//----------------------------------------------------------------------------------------

TDrawingEnvironment* TView::DoMakeNewDrawingEnvironment()
{
	return TH_new TDrawingEnvironment;
}

//----------------------------------------------------------------------------------------
// TView::CreateDrawingEnvironment:
//----------------------------------------------------------------------------------------

void TView::CreateDrawingEnvironment()
{
	if (!fDrawingEnvironment)
		fDrawingEnvironment = DoMakeNewDrawingEnvironment();
}

//----------------------------------------------------------------------------------------
// TView::SetDrawingEnvironment:
//----------------------------------------------------------------------------------------

void TView::SetDrawingEnvironment(TDrawingEnvironment* itsDrawingEnvironment,
								  bool invalidate)
{
	// protect against installing same drawing environment
	if (fDrawingEnvironment != itsDrawingEnvironment)
		fDrawingEnvironment = itsDrawingEnvironment;

	if (invalidate)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TView::PrepareForDrawing:
//----------------------------------------------------------------------------------------

void TView::PrepareForDrawing()
{
	if (fDrawingEnvironment)
		fDrawingEnvironment->Prepare();

	SetupDrawingEnvironment();
}

//----------------------------------------------------------------------------------------
// TView::SetupDrawingEnvironment:
//----------------------------------------------------------------------------------------

void TView::SetupDrawingEnvironment()
{
	if (fDrawingEnvironment)
		fDrawingEnvironment->Setup();
}

//----------------------------------------------------------------------------------------
// TView::CompleteDrawing:
//----------------------------------------------------------------------------------------

void TView::CompleteDrawing()
{
	if (fDrawingEnvironment)
		fDrawingEnvironment->Complete();
}

//----------------------------------------------------------------------------------------
// TView::Draw:
//----------------------------------------------------------------------------------------

void TView::Draw(const CViewRect&					/* area */ )
{
}

//----------------------------------------------------------------------------------------
// TView::HandleDrawFailed:
//----------------------------------------------------------------------------------------

bool TView::HandleDrawFailed(const CViewRect& area, const CException_AC& theException)
{
	//
	// Show the error then call DrawFailed() to paint in the region
	// Remembering to validate the region AFTER showing the error
	// This way the error won't keep recurring!
	//
	TWindow* theWindow = GetWindow();
	bool alertAndContinue = theWindow && theWindow->GetUpdating();

	NotifyDrawFailed(area, theException);

	RgnHandle theRealUpdateRgn;
	if (alertAndContinue)
	{
		RgnHandle theSavedUpdateRgn = TWindow::GetUpdateRegion(theWindow->GetWindowRef());
		theRealUpdateRgn = ((WindowPeek)theWindow->GetWindowRef())->updateRgn;
		UnionRgn(theSavedUpdateRgn, theRealUpdateRgn, theSavedUpdateRgn);
	}

	InvalidateFocus();
	if (Focus())
		DrawFailed(area, theException);

	if (alertAndContinue)
	{
		DiffRgn(theRealUpdateRgn, qd.thePort->clipRgn, theRealUpdateRgn);
		Validate(GetQDExtent());
	}

	return alertAndContinue;
}

//----------------------------------------------------------------------------------------
// TView::NotifyDrawFailed:
//----------------------------------------------------------------------------------------

void TView::NotifyDrawFailed(const CViewRect& /* area */, const CException_AC& theException)
{
	// The default is to just let the user know by posting an alert, however, you may
	// wish to update a status panel or something in an override.

	// We don't post the alert when NOT updating b/c we'll just take the default re-throwing
	// of the exception
	TWindow* theWindow = GetWindow();
	if(theWindow && theWindow->GetUpdating())
		gDispatcher->ShowError(theException.GetError(), theException.GetMessage());
}

//----------------------------------------------------------------------------------------
// TView::DrawFailed:
//----------------------------------------------------------------------------------------

void TView::DrawFailed(const CViewRect& /* area */, const CException_AC& /* theException */)
{
	PenNormal();
	FillRgn(qd.thePort->clipRgn, &qd.ltGray);
#if qDebug
	PenSize(2, 2);
	FrameRgn(qd.thePort->clipRgn);
	PenNormal();
#endif
}

//----------------------------------------------------------------------------------------
// TView::DrawContents:
//----------------------------------------------------------------------------------------

void TView::DrawContents()
{
	HandleDraw(GetExtent());
}

//----------------------------------------------------------------------------------------
// TView::HandleDraw:
//----------------------------------------------------------------------------------------

void TView::HandleDraw(const CViewRect& area)
{
	if (!(area & GetVisibleExtent()).Empty() && (IsShown()) && Focus())
	{
		// Even if the visible extent is not empty the Drawable rect might be.
		// The Focus (above) will set the clipping for us so we can use it to determine
		// what can be drawn.
		const RgnHandle currentClipRgn = GetClipRegion(qd.thePort);

		const CRect_AC clipQDRect((*currentClipRgn)->rgnBBox);
		if (!clipQDRect.Empty())				// can we draw anything at all thru the current clip?
		{
			const CViewRect drawArea(area & QDToView(clipQDRect));
			if (!drawArea.Empty())				// does the requested drawing area intersect the drawable area?
			{
				// As suggested by Ed Anson:
				// Check to see if the drawing area intersects the clipping region
				// before drawing.
				const CRect_AC drawQDArea(ViewToQD(drawArea));

				if (RectInRgn(&drawQDArea, currentClipRgn))// does the requested area intersect the drawable region?
				{
					RgnHandle originalClippedRgn = NULL;
					RgnHandle subviewClippedRgn = NULL;
					try
					{
						// remove any opaque subviews from the clip
						bool subviewClippingOccurred = false;
						if (HasSubViews())
						{
							originalClippedRgn = MakeNewRgn();
							::CopyRgn(currentClipRgn, originalClippedRgn); // we'll need this later
							if (ClipSubViews(currentClipRgn))
							{
								ClipFurtherTo(currentClipRgn, gZeroPt);

								subviewClippedRgn = MakeNewRgn();
								::CopyRgn(currentClipRgn, subviewClippedRgn); // we'll need this later

								subviewClippingOccurred = true;
							}
						}

						PrepareForDrawing();

						if (fAdorners)
							DrawAdorners(drawArea);
						else
							Draw(drawArea);

						// Give the behavior a chance to draw
						TBehavior * aBehavior = GetFirstEnabledBehavior();
						if (aBehavior)
							aBehavior->Draw(drawArea);

						// Draw all of the subviews
						if (HasSubViews())
						{
							if (subviewClippingOccurred && IsFocused())		// no-one else cares about the removal of my subviews
								::SetClip(originalClippedRgn);

							for (CSubViewIterator iter(this); iter; ++iter)
							{
								CViewRect subViewDrawArea(iter->SuperViewToLocal(drawArea));

								const CViewRect subViewDrawAreaExtent(subViewDrawArea & iter->GetExtent());
								if (!subViewDrawAreaExtent.Empty())
									iter->HandleDraw(subViewDrawAreaExtent);
							}
						}

						// note that we need to re-focus because we've been left focused on our subviews
						if (Focus() && subviewClippingOccurred)
							// remove any opaque subviews from the clip
							::SetClip(subviewClippedRgn);

						// add the adornments
						if (!(gPrinting || gDrawingPictScrap) && fAdorners)
							HighlightAdorners(drawArea, hlOff, GetCurrentHL());

						CompleteDrawing();

						if (subviewClippingOccurred && IsFocused())
							// remove any opaque subviews from the clip
							::SetClip(originalClippedRgn);
					}
					catch(CException_AC& theException)
					{
						DoCatchMessage_AC(theException);

						if (!HandleDrawFailed(area, theException))
						{
							DisposeIfRgnHandle(originalClippedRgn);
							DisposeIfRgnHandle(subviewClippedRgn);

							throw;
						}
					}

					DisposeIfRgnHandle(originalClippedRgn);
					DisposeIfRgnHandle(subviewClippedRgn);
				}
			}
		}

#if qDebug
		if (gShowInvalidations)					// special delay after drawing each view so you can watch it advance
			DelayFuture_AC(15);					// so you can see it
#endif

	}
}

//----------------------------------------------------------------------------------------
// TView::DrawAdorners:
//----------------------------------------------------------------------------------------

void TView::DrawAdorners(const CViewRect& area)
{
	PenState savePenState;
	GetPenState(&savePenState);

	for (CAdornerIterator iter(this); iter; ++iter)
		if (iter->DoesAdorn(this))
		{
			iter->Draw(this, area);
			SetPenState(&savePenState);
		}
}

//----------------------------------------------------------------------------------------
// TView::FindSubView:
//----------------------------------------------------------------------------------------

TView* TView::FindSubView(IDType itsIdentifier) const
{
	TView* returnVal = NULL;

	if (fIdentifier == itsIdentifier)
		returnVal = const_cast<TView*>(this);
	else if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
		{
			if (iter->fIdentifier == itsIdentifier)
			{
				returnVal = iter;
				break;
			}
			else if (iter->HasSubViews())
			{
				TView * foundSubView = iter->FindSubView(itsIdentifier);
				if (foundSubView)
				{
					returnVal = foundSubView;
					break;
				}
			}
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::Focus:
//----------------------------------------------------------------------------------------

bool TView::Focus()
{
	bool returnVal = true;

	if (IsFocused())							// Already focused.
	{
		GrafPtr		currentPort;
		GrafPtr		expectedPort = GetGrafPort();

		::GetPort(&currentPort);
		if (currentPort != expectedPort)
		{
			::SetPort(expectedPort);

			SetupDrawingEnvironment();
		}
	}
	else if (gDrawingPictScrapView == this)
	{
		gFocusedView = this;

		CRect_AC qdExtent(GetQDExtent());
		CPoint_AC qdOrigin(GetQDOrigin());
		::SetOrigin(qdOrigin.h, qdOrigin.v);
		::ClipRect(&qdExtent);

		SetupDrawingEnvironment();
	}
	else if (FocusOnSuperView())				// Do my own focus.
	{
		CPoint_AC qdOrigin(GetQDOrigin());
		::SetOrigin(qdOrigin.h, qdOrigin.v);	// Set the new QD origin.

		CPoint_AC deltaOrigin;
		if (TPrintHandler::gCurrPrintHandler && (TPrintHandler::gCurrPrintHandler->fView == this))
			deltaOrigin = qdOrigin - TPrintHandler::gCurrPrintHandler->GetQDOrigin();
		else if (fSuperView)
			deltaOrigin = qdOrigin - fSuperView->GetQDOrigin();
		else
			deltaOrigin.Clear();

		gFocusedView = this;					// Now we have the basic focus.

		// Add the shaping to the view's extent shape.
		CTempRegion_AC rRgn;
		GetExtentRegion(rRgn);
		OffsetAndClipFurtherTo(rRgn, deltaOrigin);

		// Add the clipping to the currently visible portion of the extent.
		GetVisibleRegion(rRgn);
		ClipSiblingViews(rRgn);
		ClipFurtherTo(rRgn, gZeroPt);

		SetupDrawingEnvironment();
	}
	else										// Can't focus
		returnVal = false;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::FocusOnSuperView:
//----------------------------------------------------------------------------------------

bool TView::FocusOnSuperView()
{
	bool returnVal = false;

	if (TPrintHandler::gCurrPrintHandler && (TPrintHandler::gCurrPrintHandler->fView == this))
		returnVal = TPrintHandler::gCurrPrintHandler->Focus();
	else if (fSuperView)
		returnVal = fSuperView->Focus();

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::ForceRedraw:
//----------------------------------------------------------------------------------------

void TView::ForceRedraw()
{
	if (IsVisible())
	{
		CTempRegion_AC itsExtentRgn;

		GetExtentRegion(itsExtentRgn);
		Invalidate(itsExtentRgn);
	}
}

//----------------------------------------------------------------------------------------
// TView::FreeFromClipboard:
//----------------------------------------------------------------------------------------

void TView::FreeFromClipboard()
{
	gClipboardMgr->Changed(mFreeClipboardView, this);

	if (fDocument)
		fDocument->FreeFromClipboard();

	else if (gClipboardMgr->fClipOrphanage != this)
		delete this;
}

//----------------------------------------------------------------------------------------
// TView::GetStandardSignature:
//----------------------------------------------------------------------------------------

IDType TView::GetStandardSignature() const			// override
{
	return kStdView;
}

//----------------------------------------------------------------------------------------
// TView::ReadFrom:
//----------------------------------------------------------------------------------------

void TView::ReadFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);					// read in TEventHandler info

	// Subclasses of TView should override either TView::ReadFields or TView::ReadSubViews,
	// not TView::ReadFrom.

	fTemplateVersion = aStream->ReadInteger();		// Moved from ReadFields.

	if (fTemplateVersion == kMARelease13ViewsTemplateVersion)
	{
		// read fields size and remember the current position.
		long sizePosition = aStream->GetReadPosition();
		long fieldsSize = aStream->ReadLong();
		ReadFields(aStream);						// read in fields polymorphically

#if qWriteTemplateViews
		if (aStream->GetReadPosition() < sizePosition + fieldsSize)
		{
			Size itsSize = sizePosition + fieldsSize - aStream->GetReadPosition();
			CTempHandle_AC aHandle = NewPermHandle_AC(itsSize);

			// Lock the handle and read into it from the stream.
			// We definitely need to lock the handle here because
			// the code in ReadBytes may move memory before reading
			// the data.
			CTempHandleLock_AC tempHandleLock(aHandle);
			::ThrowIfMemError_AC();
			aStream->ReadBytes(*(Handle)aHandle, itsSize);

			fTemplateUnknownData = aHandle;	// CAutoPtr ownership is transferred
		}
#if qDebug
		else if (aStream->GetReadPosition() > sizePosition + fieldsSize)
			::ProgramBreak_AC("###Stream position is greater than saved position in TView::ReadFrom.");
#endif // qDebug
#endif // qWriteTemplateViews

		// move the stream position past the end of our fields
		aStream->SetReadPosition(sizePosition + fieldsSize);

	}
	else if (fTemplateVersion == kMAThreeOhTemplateVersion)
	{
		// bump up the template version and read in fields directly.
		fTemplateVersion = kMARelease13ViewsTemplateVersion;
		ReadFields(aStream);						// read in fields polymorphically
	}
#if qDebug
	else
		::ProgramBreak_AC("###Unknown template version in TView::ReadFrom.");
#endif

	ReadSubViews(aStream);						// read in subviews
}

//----------------------------------------------------------------------------------------
// TView::ReadFields:
//----------------------------------------------------------------------------------------

void TView::ReadFields(CStream_AC* aStream)
{
	// template version moved to ReadFrom where it is needed for guard length protection.

	*aStream 	>> fLocation
				>> fSize
				>> StreamInEnum_AC(fSizeDeterminer[vSel])
				>> StreamInEnum_AC(fSizeDeterminer[hSel])
				>> fShown
				>> fWantsToBeTarget
				>> fHandlesCursor
				>> fLetsSubViewsHandleCursor
				>> fCursorID
				>> fHandlesHelp
				>> fLetsSubViewsHandleHelp
				>> fHelpID
				>> fHelpIndex;

#if 0
	fLocation = aStream->ReadLongPoint();
	fSize = aStream->ReadLongPoint();

	// formerly SizeDeterminers; however, sizeof(enum) is compiler-dependent
	fSizeDeterminer[vSel] = (SizeDeterminer)aStream->ReadByte();
	fSizeDeterminer[hSel] = (SizeDeterminer)aStream->ReadByte();

	fShown = aStream->ReadBoolean();
	fWantsToBeTarget = aStream->ReadBoolean();

	fHandlesCursor = aStream->ReadBoolean();
	fLetsSubViewsHandleCursor = aStream->ReadBoolean();
	fCursorID = aStream->ReadInteger();

	fHandlesHelp = aStream->ReadBoolean();
	fLetsSubViewsHandleHelp = aStream->ReadBoolean();
	fHelpID = aStream->ReadInteger();
	fHelpIndex = aStream->ReadInteger();
#endif

	bool wasValid;
	fDrawingEnvironment = dynamic_cast<TDrawingEnvironment*>(aStream->ReadStreamObject(wasValid));
	if (!wasValid)
	{
		fDrawingEnvironment = NULL;
#if qDebugMsg
		::ProgramBreak_AC("###Unknown TDrawingEnvironment in TView::ReadFields.");
#endif

	}

	// It so happens that some default adorners may attach themselves at constructor time.
	// We make sure to delete them here before the list is deleted by the auto-ptr assignment
	DeleteAllAdorners();
	
	fAdorners = dynamic_cast<TAdornerList*>(aStream->ReadStreamObject(wasValid));
	if (!wasValid)
	{
		fAdorners = NULL;
#if qDebugMsg
		::ProgramBreak_AC("###Unknown TAdornerList in TView::ReadFields.");
#endif

	}

	*aStream >> fUserArea;

	aStream->ReadIDType();					// read in idtype and ignore it
}

//----------------------------------------------------------------------------------------
// TView::ReadSubViews:
//----------------------------------------------------------------------------------------

void TView::ReadSubViews(CStream_AC* aStream)
{
	short numSubViews = aStream->ReadInteger();// read in count of subviews

	for (short count = 0; count < numSubViews; ++count)
	{
		bool wasValid;
		TView * aView = dynamic_cast<TView*>(aStream->ReadStreamObject(wasValid));
		if (wasValid && aView)
			AddSubView(aView);
#if qDebugMsg
		else
			::ProgramBreak_AC("###Unknown TView in TView::ReadSubViews: Attempting to create a TView.");
#endif
	}
}

//----------------------------------------------------------------------------------------
// TView::WriteTo:
//----------------------------------------------------------------------------------------

void TView::WriteTo(CStream_AC* aStream) const
{
	Inherited::WriteTo(aStream);				// write out TEventHandler info

	// Subclasses of TView should override either TView::WriteFields or TView::WriteSubViews,
	// not TView::WriteTo.

	if (qWriteTemplateViews)
	{
		aStream->WriteInteger(fTemplateVersion);			// Moved from WriteFields.
		long sizePosition = aStream->GetWritePosition();			// Remember the position of the fields size in the stream.
		aStream->WriteLong(LONG_MAX);						// Write a dummy size (maximum object size).

		WriteFields(aStream);								// write out fields polymorphically

#if qWriteTemplateViews
		if (fTemplateUnknownData)
		{
			Size itsSize = GetHandleSize(fTemplateUnknownData);

			// Write the contents of the handle.
			// We definitely need to lock the handle here because
			// the code in WriteBytes may move memory before reading
			// the data.
			CTempHandleLock_AC tempHandleLock(fTemplateUnknownData);
			::ThrowIfMemError_AC();
			aStream->WriteBytes(*fTemplateUnknownData, itsSize);
		}
#endif

		long currentPosition = aStream->GetWritePosition();		// Remember where we are.
		aStream->SetWritePosition(sizePosition);					// Go back to position of size.
		aStream->WriteLong(currentPosition - sizePosition);	// Write the size.
		aStream->SetWritePosition(currentPosition);				// Go back to where we were.

		WriteSubViews(aStream);								// write out subviews
	}
}

//----------------------------------------------------------------------------------------
// TView::WriteFields:
//----------------------------------------------------------------------------------------

void TView::WriteFields(CStream_AC* aStream) const
{
//	Byte itsHSizeDet = (Byte);
//	Byte itsVSizeDet = (Byte);
	// formerly SizeDeterminers; however, sizeof(enum) is compiler-dependent

	*aStream 	<< fLocation
				<< fSize
				<< StreamOutEnum_AC(fSizeDeterminer[vSel])
				<< StreamOutEnum_AC(fSizeDeterminer[hSel])
				<< fShown
				<< fWantsToBeTarget
				<< fHandlesCursor
				<< fLetsSubViewsHandleCursor
				<< fCursorID
				<< fHandlesHelp
				<< fLetsSubViewsHandleHelp
				<< fHelpID
				<< fHelpIndex;
#if 0
	aStream->WriteLongPoint(fLocation);
	aStream->WriteLongPoint(fSize);

	aStream->WriteBytes(&itsVSizeDet, sizeof(Byte));
	aStream->WriteBytes(&itsHSizeDet, sizeof(Byte));

	aStream->WriteBoolean(fShown);
	aStream->WriteBoolean(fWantsToBeTarget);

	aStream->WriteBoolean(fHandlesCursor);
	aStream->WriteBoolean(fLetsSubViewsHandleCursor);
	aStream->WriteInteger(fCursorID);

	aStream->WriteBoolean(fHandlesHelp);
	aStream->WriteBoolean(fLetsSubViewsHandleHelp);
	aStream->WriteInteger(fHelpID);
	aStream->WriteInteger(fHelpIndex);
#endif
	aStream->WriteStreamObject(fDrawingEnvironment);

	aStream->WriteStreamObject(fAdorners, kStandardObject);

	*aStream << fUserArea << GetStandardSignature();

#if 0
	aStream->WriteLong(fUserArea);

	aStream->WriteIDType(GetStandardSignature());// write it out again for Rez-ability
#endif
}

//----------------------------------------------------------------------------------------
// TView::WriteSubViews:
//----------------------------------------------------------------------------------------

void TView::WriteSubViews(CStream_AC* aStream) const
{
	aStream->WriteInteger(CountSubViews());		// write out count of subviews

	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			aStream->WriteStreamObject(iter, kStandardObject);
}

//----------------------------------------------------------------------------------------
// TView::GetContext:
//----------------------------------------------------------------------------------------

TCommandHandler* TView::GetContext(CommandNumber/* aCommandNumber */ )// Override
{
	return fDocument ? fDocument : dynamic_cast_AC(TCommandHandler*,GetWindow());
}

//----------------------------------------------------------------------------------------
// TView::GetExtent:
//----------------------------------------------------------------------------------------

CViewRect TView::GetExtent() const
{
	// Allow for the origin of the view's coordinate system.
	return CViewRect(gZeroVPt, fSize) - (GetLocalOffsetInSuper() - fLocation);
}

//----------------------------------------------------------------------------------------
// TView::GetExtentRegion:
//----------------------------------------------------------------------------------------

void TView::GetExtentRegion(RgnHandle itsExtentRgn)
{
	RectRgn(itsExtentRgn, &GetQDExtent());
}

//----------------------------------------------------------------------------------------
// TView::GetFrame:
//----------------------------------------------------------------------------------------

CViewRect TView::GetFrame() const
{
	return CViewRect(fLocation, fLocation + fSize);
}

//----------------------------------------------------------------------------------------
// TView::GetGrafPort:
//----------------------------------------------------------------------------------------

GrafPtr TView::GetGrafPort() const
{
	GrafPtr returnVal = NULL;

	if (fDrawingEnvironment)
		returnVal = fDrawingEnvironment->GetGrafPort();
	if (!returnVal)
	{
		if (gPrinting || gDrawingPictScrap)
			returnVal = qd.thePort;				//thePort assumed to be set by print handler
		else if (fSuperView)
			returnVal = fSuperView->GetGrafPort();
	}
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::GetPartCode:
//----------------------------------------------------------------------------------------

short TView::GetPartCode(const CViewPoint& theMouse) const
{
	return GetExtent().Contains(theMouse) ? inContent : inDesk;
}

//----------------------------------------------------------------------------------------
// TView::GetPrintExtent:
//----------------------------------------------------------------------------------------

CViewRect TView::GetPrintExtent()
{
	return GetExtent();
}

//----------------------------------------------------------------------------------------
// TView::GetQDExtent:
//----------------------------------------------------------------------------------------

CRect_AC TView::GetQDExtent()
{
	return ViewToQD(GetExtent());
}

//----------------------------------------------------------------------------------------
// TView::GetScroller:
//----------------------------------------------------------------------------------------

TScroller* TView::GetScroller(bool immediateSuperView)
{
	TScroller * returnVal = NULL;

	if (fSuperView)
	{
		TScroller * aScroller = fSuperView->GetScroller(immediateSuperView);
		if ((aScroller == fSuperView) || (!immediateSuperView))
			returnVal = aScroller;
	}
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::GetVisibleRect:
//----------------------------------------------------------------------------------------

CViewRect TView::GetVisibleRect()
{
	return QDToView(GetVisibleQDRect());
}

//----------------------------------------------------------------------------------------
// TView::GetVisibleQDRect:
//----------------------------------------------------------------------------------------

CRect_AC TView::GetVisibleQDRect()
{
	CTempRegion_AC visibleRgn;

	GetVisibleRegion(visibleRgn);

	return (*(RgnHandle)visibleRgn)->rgnBBox;
}

//----------------------------------------------------------------------------------------
// TView::GetVisibleRegion:
//----------------------------------------------------------------------------------------

void TView::GetVisibleRegion(RgnHandle visibleRgn)
{
	CViewRect visibleExtent(GetVisibleExtent());

	if (visibleExtent.Empty())
		SetEmptyRgn(visibleRgn);
	else
	{
		RectRgn(visibleRgn, &ViewToQD(visibleExtent));
		VisibleRgn(visibleRgn);
	}
}

//----------------------------------------------------------------------------------------
// TView::GetDrawableRect:
//----------------------------------------------------------------------------------------

CViewRect TView::GetDrawableRect()
{
	return QDToView(GetDrawableQDRect());
}

//----------------------------------------------------------------------------------------
// TView::GetDrawableQDRect:
//----------------------------------------------------------------------------------------

CRect_AC TView::GetDrawableQDRect()
{
	CTempRegion_AC drawableRgn;

	GetDrawableRegion(drawableRgn);

	return (*(RgnHandle)drawableRgn)->rgnBBox;
}

//----------------------------------------------------------------------------------------
// TView::GetDrawableRegion:
//----------------------------------------------------------------------------------------

void TView::GetDrawableRegion(RgnHandle drawRgn)
{
	GetVisibleRegion(drawRgn);
	if (!EmptyRgn(drawRgn))
		DrawableRgn(drawRgn);
}

//----------------------------------------------------------------------------------------
// TView::GetHelpID:
//----------------------------------------------------------------------------------------

ResNumber TView::GetHelpID()
{
	return fHelpID;
}

//----------------------------------------------------------------------------------------
// TView::GetHelpIndex:
//----------------------------------------------------------------------------------------

short TView::GetHelpIndex()
{
	return fHelpIndex;
}

//----------------------------------------------------------------------------------------
// TView::GetHelpState:
//----------------------------------------------------------------------------------------

short TView::GetHelpState()
{
	return IsEnabled() ? static_cast<short>(kHMEnabledItem) : static_cast<short>(kHMDisabledItem);
}

//----------------------------------------------------------------------------------------
// TView::GetCursorID:
//----------------------------------------------------------------------------------------

ResNumber TView::GetCursorID()
{
	return fCursorID;
}

//----------------------------------------------------------------------------------------
// TView::GetHelpParameters:
//----------------------------------------------------------------------------------------

void TView::GetHelpParameters(const CViewPoint& localPoint,
							  RgnHandle helpRegion,
							  HMMessageRecord& helpMessage,
							  CPoint_AC& localQDTip,
							  CRect_AC& localQDRect,
							  short& balloonVariant)
{
	gDispatcher->GetHelpParameters(GetHelpID(), GetHelpIndex(),// component number within resource
								   GetHelpState(),// view state: enabled, disabled etc.
								   helpMessage, localQDTip, localQDRect, balloonVariant);

	// Ignore the hotRect from the resource
	if (helpRegion)
	{
		localQDRect = (*helpRegion)->rgnBBox;
		if ((localQDTip.h < 0) || (localQDTip.v < 0))
		{
			//GetMouse(localQDTip);	 ???
			localQDTip = ViewToQD(localPoint);
		}
	}
}

//----------------------------------------------------------------------------------------
// TView::GetRootView:
//----------------------------------------------------------------------------------------

TView* TView::GetRootView()
{
	return fSuperView ? fSuperView->GetRootView() : this;
}

//----------------------------------------------------------------------------------------
// TView::GetWindow:
//----------------------------------------------------------------------------------------

TWindow* TView::GetWindow() const
{
	return fSuperView ? fSuperView->GetWindow() : NULL;
}

//----------------------------------------------------------------------------------------
// TView::GivePasteData:
//----------------------------------------------------------------------------------------

long TView::GivePasteData(Handle aDataHandle,
						  ResType dataType)
{
	// Make sure the scrap handle is grown out of permanent memory.	If it
	// was grown out of temporary memory it would eat away at our code space.
	bool savedTemp = TemporaryAllocation(false);

	long offset;
	long err = GetScrap(aDataHandle, dataType, &offset);

	savedTemp = PermAllocation(savedTemp);

#if qDebugMsg
	if (err < 0)
		ProgramReport_AC(false, "TView.GivePasteData got error code # %d from GetScrap for type %s\n", err, &dataType);
#endif

	return err;
}

//----------------------------------------------------------------------------------------
// TView::HandleCursor:
//----------------------------------------------------------------------------------------

void TView::HandleCursor(const CViewPoint& theMouse,
						 RgnHandle cursorRegion)
{

	// Assume if we got here then we already know the mouse is in this view
	if (LetsSubViewsHandleCursor())
		for (CSubViewIterator iter(this, kIterateBackward_AC); iter; ++iter)
		{
			CViewPoint subViewPt(iter->SuperViewToLocal(theMouse));

			if ((iter->ContainsMouse(subViewPt) && iter->HandlesCursor()))
			{
				iter->HandleCursor(subViewPt, cursorRegion);
				// bring the cursor Rgn back to our coordinate system.
				iter->LocalToSuperView(cursorRegion);
				break;
			}
		}

	if (EmptyRgn(cursorRegion) && Focus())
	{
		// First let the Behavior have a crack at handling the cursor
		bool proceed = true;

		GetDefaultCursorRegion(theMouse, cursorRegion);

		if (GetFirstBehavior())
			proceed = !dynamic_cast_AC(TBehavior*,GetFirstBehavior())->DoSetCursor(theMouse, cursorRegion);

		if (proceed)
			DoSetCursor(theMouse, cursorRegion);

		// Restrict the claimed cursor region to the visible extent of this view
		RestrictRegionToVisible(cursorRegion);

		fgSetCursorView = this; // in case someone's override of DoSetCursor forgot
	}
}

//----------------------------------------------------------------------------------------
// TView::HandleUnCursor:
//----------------------------------------------------------------------------------------

void TView::HandleUnCursor()
{
	if (Focus())
	{
		// First let the Behavior have a crack at handling the cursor
		bool proceed = true;

		if (GetFirstBehavior())
			proceed = !dynamic_cast_AC(TBehavior*,GetFirstBehavior())->DoUnsetCursor();

		if (proceed)
			DoUnsetCursor();
	}
}

//----------------------------------------------------------------------------------------
// TView::HandleHelp:
//----------------------------------------------------------------------------------------

void TView::HandleHelp(const CViewPoint& theMouse,
					   RgnHandle helpRegion)
{
	//Assume if we got here then we already know the mouse is in this view

	if (LetsSubViewsHandleHelp())
		for (CSubViewIterator iter(this, kIterateBackward_AC); iter; ++iter)
		{
			CViewPoint subViewPt(iter->SuperViewToLocal(theMouse));

			if (iter->ContainsMouse(subViewPt) && iter->HandlesHelp())
			{
				iter->HandleHelp(subViewPt, helpRegion);

				// bring the help Rgn back to our coordinate system.
				iter->LocalToSuperView(helpRegion);
				break;
			}
		}

	if (EmptyRgn(helpRegion) && HandlesHelp() && Focus())
	{
		DoShowHelp(theMouse, helpRegion);

		// Restrict the claimed help region to the visible extent of this view
		RestrictRegionToVisible(helpRegion);
	}
}

#if qWantsContextualMenus

//----------------------------------------------------------------------------------------
// TView::GetContextualMenu:
//----------------------------------------------------------------------------------------

void TView::GetContextualMenu(MenuRef& theMenuRef, CAEDesc& theSelection)
{
	// If no menu is supplied, the contextual menu manager will use a default one.
	theMenuRef = NULL;	// by default, need someone to override me
	bool result = false;
	TWindow * myWindow = GetWindow();

	// Default implementation is to return a description of the containing window.
	// Subclasses of TView that mix MScriptable object may choose to describe themselves rather
	// than their containing window.
	if (myWindow)
	{
		try
		{
			// formUniqueID isn't right, this is placeholder code.
			// A real example should at least create a text descriptor.
			if (!myWindow->MakeObjectSpecifier(theSelection, formUniqueID))
				theMenuRef = NULL;
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
		}

		// no resignal on failure
	}
}

#endif // qWantsContextualMenus

//----------------------------------------------------------------------------------------
// TView::HandleMouseDown:
//----------------------------------------------------------------------------------------

bool TView::HandleMouseDown(const CViewPoint& theMouse,
							   TToolboxEvent* event,
							   CPoint_AC hysteresis,
							   EMouseDownType mouseDownType)
{
	bool wasHandled = false;

	bool isFirstClick = (mouseDownType == kFirstClick_dragOnly || mouseDownType == kFirstClick_dragOrClick);
	bool canDoMouse = (mouseDownType == kDragOrClick || mouseDownType == kFirstClick_dragOrClick);

	if (HasSubViews())
		for (CSubViewIterator iter(this, kIterateBackward_AC); iter; ++iter)
		{
			CViewPoint subViewPt(iter->SuperViewToLocal(theMouse));

			if (iter->ContainsMouse(subViewPt))
				if (iter->HandleMouseDown(subViewPt, event, hysteresis, mouseDownType))
				{
					wasHandled = true;
					break;
				}
		}

	if (!wasHandled && Focus())								// see if we can handle it
	{
		CViewPoint tempMouse = theMouse;			// We do this so we can pass a const to DoMouseCommand
		CPoint_AC globalMouse(LocalToGlobal(ViewToQD(theMouse)));

		//  This is important to allowing the cursor to change when selecting
		if (gDispatcher->IsCursorRgnInvalid())
		{
			gDispatcher->TrackCursor(globalMouse);
			Focus();
		}

		if (canDoMouse && IsEnabled())
			UpdateSelection(tempMouse, event);

#if qDrag
		// if the user is dragging, and the view wants to drag, initiate a drag
		if (HasDragManager_AC())
		{
			RgnHandle dragCursorRegion = DoMakeDragCursorRegion();
			if (WillDrag(theMouse, dragCursorRegion))
			{
				dragCursorRegion = DisposeIfRgnHandle(dragCursorRegion);
				short cursorID = GetIsDraggingCursorID();
				if (cursorID != kNoResource)
				{
					CursHandle aCursor = GetCursor(cursorID);
					::ThrowIfNULLResource_AC((Handle)aCursor);
					SetCursor(*aCursor);
				}
				else
					SetCursor(&(qd.arrow));

				if (TDragDropSession::fgDragDropSession->UserIsDragging(globalMouse))
				{
					DoGetDragProxy()->HandleDrag(event);
					wasHandled = true;
				}
			}
			else
				dragCursorRegion = DisposeIfRgnHandle(dragCursorRegion);
		}
#endif // qDrag

		if (!wasHandled && isFirstClick)
		{
			TWindow * theWindow = GetWindow();
			if (theWindow)
			{
				if (!fHandlesFirstClicks)
					wasHandled = true;
				theWindow->Select();
				gDispatcher->UpdateAllWindows();
				Focus();
			}
		}

		if (!wasHandled && canDoMouse && IsEnabled())
		{
			if (WantsToBeTarget() && !BecomeTarget())
				wasHandled = true;				// don't allow subviews to handle it
			else
			{
				// Give behaviors a chance to handle the mouseDown
				TBehavior * aBehavior = GetFirstEnabledBehavior();
				if (aBehavior)
					wasHandled = aBehavior->DoMouseCommand(tempMouse, event, hysteresis);

				if (!wasHandled && IsEnabled())
				{
					DoMouseCommand(tempMouse, event, hysteresis);
					wasHandled = true;
				}
			}
		}

		if (wasHandled || GetDraggable())
			gDispatcher->InvalidateMouseRegions();
	}

	return wasHandled;
}

//----------------------------------------------------------------------------------------
// TView::HandleMouseUp:
//----------------------------------------------------------------------------------------

bool TView::HandleMouseUp(const CViewPoint& theMouse,
							 TToolboxEvent* event,
							 CPoint_AC hysteresis)
{
	bool wasHandled = false;

	if (HasSubViews())
		for (CSubViewIterator iter(this, kIterateBackward_AC); iter; ++iter)
		{
			CViewPoint subViewPt(iter->SuperViewToLocal(theMouse));

			if (iter->ContainsMouse(subViewPt))
				if (iter->HandleMouseUp(subViewPt, event, hysteresis))
				{
					wasHandled = true;
					break;
				}
		}

	if (!wasHandled && Focus())								// see if we can handle it
	{
		CViewPoint tempMouse(theMouse);				// We do this so we can pass a const to DoMouseCommand

		// Change added to support behaviors
		bool proceed = true;

		TBehavior * aBehavior = GetFirstEnabledBehavior();
		if (aBehavior)
			proceed = !aBehavior->DoMouseUp(tempMouse, event, hysteresis);

		if (proceed && IsEnabled())
		{
			DoMouseUp(tempMouse, event, hysteresis);
			wasHandled = true;
		}
		else
			wasHandled = !proceed;
	}

	return wasHandled;
}

//----------------------------------------------------------------------------------------
// TView::HasPendingUpdate:
//----------------------------------------------------------------------------------------

bool TView::HasPendingUpdate() const
{
	return fSuperView ? fSuperView->HasPendingUpdate() : false;
}

//----------------------------------------------------------------------------------------
// TView::InvalidateVRect:
//----------------------------------------------------------------------------------------

void TView::Invalidate(const CViewRect& badVRect)
{
	Invalidate(ViewToQD(badVRect));
}

//----------------------------------------------------------------------------------------
// TView::InvalidateRect:
//----------------------------------------------------------------------------------------

void TView::Invalidate(const CRect_AC& badRect)
{
	CTempRegion_AC badRectRgn;

	RectRgn(badRectRgn, &badRect);
	Invalidate(badRectRgn);
}

//----------------------------------------------------------------------------------------
// TView::InvalidateRegion:
//----------------------------------------------------------------------------------------

void TView::Invalidate(const RgnHandle badRgn)
{
	if (Focus())
	{
		CTempRegion_AC visibleBadRgn;

		RectRgn(visibleBadRgn, &ViewToQD(GetVisibleExtent()));
		SectRgn(badRgn, visibleBadRgn, visibleBadRgn);
		if (!EmptyRgn(visibleBadRgn))
			DoInvalidateRegion(visibleBadRgn);
	}
}

//----------------------------------------------------------------------------------------
// TView::DoInvalidateRegion:
//----------------------------------------------------------------------------------------

void TView::DoInvalidateRegion(const RgnHandle badRgn)
{
	if (fSuperView)
		fSuperView->DoInvalidateRegion(badRgn);	// see if someone knows how
}

//----------------------------------------------------------------------------------------
// TView::IsShown:
//----------------------------------------------------------------------------------------

bool TView::IsShown() const
{
	// Rewritten as single exit code. (It's smaller and faster object code too!)
	bool returnVal = fShown;

	if (!(gDrawingPictScrap && gDrawingPictScrapView == this) && !(TPrintHandler::gCurrPrintHandler && TPrintHandler::gCurrPrintHandler->fView == this) && fSuperView)
		returnVal = returnVal && fSuperView->IsShown();// A view can't be shown if its superview isn't shown

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::IsActive:
//----------------------------------------------------------------------------------------

bool TView::IsActive() const
{
	return fSuperView ? fSuperView->IsActive() : false;// Can't be "active" without a superview
}

//----------------------------------------------------------------------------------------
// TView::IsDrawable:
//----------------------------------------------------------------------------------------

bool TView::IsDrawable()
{
	// Rewritten as single exit code. (It's smaller and faster object code too!)
	// also avoid call to GetDrawableQDRect by checking the clipRgn directly.
	bool returnVal = false;

	if (IsShown() && Focus())					// leaves view focused if it is drawable
		returnVal = !EmptyRgn(GetClipRegion(qd.thePort));

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::IsVisible:
//----------------------------------------------------------------------------------------

bool TView::IsVisible()
{
	bool returnVal = false;

	if (IsShown())
		returnVal = IsFocused() ? !GetVisibleQDRect().Empty() : !GetVisibleExtent().Empty();

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::LocalToSuperView:
//----------------------------------------------------------------------------------------

CViewPoint TView::LocalToSuperView(const CViewPoint& thePoint) const
{
	return thePoint + GetLocalOffsetInSuper();
}

//----------------------------------------------------------------------------------------
// TView::LocalToSuperView:
//----------------------------------------------------------------------------------------

CViewRect TView::LocalToSuperView(const CViewRect& viewRect) const
{
	return viewRect + GetLocalOffsetInSuper();
}

//----------------------------------------------------------------------------------------
// TView::LocalToSuperView:
//----------------------------------------------------------------------------------------

void TView::LocalToSuperView(RgnHandle theRgn)
{
	CViewPoint localViewPt(LocalToSuperView(QDToView(gZeroPt)));	// get 0 in View Space

	CPoint_AC superQDPt(fSuperView ? fSuperView->ViewToQD(localViewPt) : localViewPt.ToPoint());

	OffsetRgn(theRgn, superQDPt.h, superQDPt.v);
}

//----------------------------------------------------------------------------------------
// TView::LocalToGlobal:
//----------------------------------------------------------------------------------------

CPoint_AC TView::LocalToGlobal(CPoint_AC thePoint) const
{
	CPoint_AC returnVal(thePoint);
	::LocalToGlobal(&returnVal);
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::LocalToGlobal:
//----------------------------------------------------------------------------------------

CViewPoint TView::LocalToGlobal(const CViewPoint& thePoint) const
{
	return thePoint + LocalToGlobal(gZeroPt);
}

//----------------------------------------------------------------------------------------
// TView::LocalToGlobal:
//----------------------------------------------------------------------------------------

CViewRect TView::LocalToGlobal(const CViewRect& viewRect) const
{
	return viewRect + LocalToGlobal(gZeroPt);
}

//----------------------------------------------------------------------------------------
// TView::LocalToGlobal:
//----------------------------------------------------------------------------------------

void TView::LocalToGlobal(RgnHandle theRgn)
{
	// Note that the intention of this function is to offset the region from _window_
	// coordinates into global coordinates.  It is not necessary to also offset based
	// on the view's offset within the window.
	CPoint_AC globalQDPt(LocalToGlobal(gZeroPt));
	OffsetRgn(theRgn, globalQDPt.h, globalQDPt.v);
}

//----------------------------------------------------------------------------------------
// TView::LocalToWindow:
//----------------------------------------------------------------------------------------

CViewPoint TView::LocalToRootView(const CViewPoint& thePoint) const
{
	return fSuperView ? fSuperView->LocalToRootView(LocalToSuperView(thePoint)) : thePoint;
}

//----------------------------------------------------------------------------------------
// TView::LocalToRootView:
//----------------------------------------------------------------------------------------

CViewRect TView::LocalToRootView(const CViewRect& viewRect) const
{
	return fSuperView ? fSuperView->LocalToRootView(LocalToSuperView(viewRect)) : viewRect;
}

//----------------------------------------------------------------------------------------
// TView::LocalToRootView:
//----------------------------------------------------------------------------------------

void TView::LocalToRootView(RgnHandle theRgn)
{
	CViewPoint localViewPt(LocalToRootView(QDToView(gZeroPt)));	// get 0 in view space

	TWindow * itsWindow = GetWindow();
	CPoint_AC windowQDPt(itsWindow ? itsWindow->ViewToQD(localViewPt) : localViewPt.ToPoint());

	OffsetRgn(theRgn, windowQDPt.h, windowQDPt.v);
}

//----------------------------------------------------------------------------------------
// TView::InvalidateFrameDifference:
//----------------------------------------------------------------------------------------

void TView::InvalidateFrameDifference(const CViewRect& oldFrame,
									  const CViewRect& newFrame)
{
	if (fSuperView && FocusOnSuperView())
	{
		CRect_AC oldQDFrame(fSuperView->ViewToQD(oldFrame));
		CTempRegion_AC oldFrameRgn;
		RectRgn(oldFrameRgn, &oldQDFrame);

		CRect_AC newQDFrame(fSuperView->ViewToQD(newFrame));
		CTempRegion_AC newFrameRgn;
		RectRgn(newFrameRgn, &newQDFrame);

		// If it didn't move then only the size difference is invalid if the view
		// doesn't have contents proportional to its size. Otherwise both the regions
		// are invalid
		if (oldFrame[topLeft] == newFrame[topLeft])
			XorRgn(oldFrameRgn, newFrameRgn, newFrameRgn);
		else
			UnionRgn(oldFrameRgn, newFrameRgn, newFrameRgn);

		fSuperView->Invalidate(newFrameRgn);
	}
}

//----------------------------------------------------------------------------------------
// TView::SetFrame:
//----------------------------------------------------------------------------------------

void TView::SetFrame(const CViewRect& newFrame,
					 bool invalidate)
{
	CViewRect oldFrame(GetFrame());

	if (newFrame != oldFrame)
	{
		// update the frame stored in the view
		fLocation = newFrame[topLeft];
		fSize = newFrame.GetSize();

		InvalidateFocus();						// Must re-focus because frame changed
		InvalidateCoordinates();				// Re-calc all the offsets

		gDispatcher->InvalidateMouseRegions();	// Must re-calc cursor and help regions

		if (invalidate)
		{
			switch (fFrameInvalidateDeterminer)
			{
				case frameInvalidateDifference:
					InvalidateFrameDifference(oldFrame, newFrame);
					break;
				case frameInvalidateWholeFrame:
					InvalidateFrameDifference(oldFrame, newFrame);
					ForceRedraw();
					break;
				default:
#if qDebug
					::ProgramBreak_AC("This view has an invalid frameInvalidateDeterminer (is the view corrupt or deleted?)");				
#endif
					;
			}
		}

		//  Let my adorners know that my frame changed
		NotifyAdornersOfFrameChange(oldFrame, newFrame, invalidate);

		//  Let my superview know that my frame changed
		if (fSuperView)
			fSuperView->SubViewChangedFrame(this, oldFrame, newFrame, invalidate);

		//  Let my subviews know that my frame changed
		NotifySubViewsOfFrameChange(oldFrame, newFrame, invalidate);

		// Let other interested items know about my frame change.
		Changed(mFrameChanged,this);
	}
}

//----------------------------------------------------------------------------------------
// TView::NotifyAdornersOfFrameChange:
//----------------------------------------------------------------------------------------

void TView::NotifyAdornersOfFrameChange(const CViewRect& oldFrame,
										const CViewRect& newFrame,
										bool invalidate)
{
	if (fAdorners && Focus())
		for (CAdornerIterator iter(this); iter; ++iter)
			iter->ViewChangedFrame(this, oldFrame, newFrame, invalidate);
}

//----------------------------------------------------------------------------------------
// TView::NotifySubViewsOfFrameChange:
//----------------------------------------------------------------------------------------

void TView::NotifySubViewsOfFrameChange(const CViewRect& oldFrame,
										const CViewRect& newFrame,
										bool invalidate)
{
	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->SuperViewChangedFrame(oldFrame, newFrame, invalidate);
}

//----------------------------------------------------------------------------------------
// TView::Locate:
//----------------------------------------------------------------------------------------

void TView::Locate(const CViewPoint& newLoc,
				   bool invalidate)
{
	SetFrame(CViewRect(newLoc, newLoc + fSize), invalidate);
}

//----------------------------------------------------------------------------------------
// TView::MakeFirstSubView:
//----------------------------------------------------------------------------------------

void TView::MakeFirstSubView(TView* theSubView)
{
	DeleteSubView(theSubView);
	InsertSubView(theSubView, kInsertSubViewFirst);
}

//----------------------------------------------------------------------------------------
// TView::MakeLastSubView:
//----------------------------------------------------------------------------------------

void TView::MakeLastSubView(TView* theSubView)
{
	DeleteSubView(theSubView);
	InsertSubView(theSubView);
}

//----------------------------------------------------------------------------------------
// TView::Open:
//----------------------------------------------------------------------------------------

void TView::Open()
{
	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->Open();
}

//----------------------------------------------------------------------------------------
// TView::PageInteriorChanged:
//----------------------------------------------------------------------------------------

void TView::PageInteriorChanged(const CViewRect&	/* newInterior */ )
{
}

//----------------------------------------------------------------------------------------
// TView::PostCommand:
//----------------------------------------------------------------------------------------

void TView::PostCommand(TCommand* command)
{
	if (command)
	{
		if (GetNextHandler())
			GetNextHandler()->PostCommand(command);
		else
		{
			// Make next best effort to post somewhere sensible. Try the document
			if (fDocument)
				fDocument->PostCommand(command);
			else
				TCommandHandler::PostCommand(command);
		}
	}
}

//----------------------------------------------------------------------------------------
// TView::RevealUndoRedo:
//----------------------------------------------------------------------------------------

void TView::RevealUndoRedo(TCommand* command)	// override
{
	Inherited::RevealUndoRedo(command);

	SetTargetSelection(kRedraw);

	TWindow * window = GetWindow();
	if (window)
		window->Select();
}

//----------------------------------------------------------------------------------------
// TView::QDToView:
//----------------------------------------------------------------------------------------

CViewPoint TView::QDToView(CPoint_AC qdPoint)
{
	return CViewPoint(qdPoint) + GetViewToQDOffset();
}

//----------------------------------------------------------------------------------------
// TView::QDToView:
//----------------------------------------------------------------------------------------

CViewRect TView::QDToView(const CRect_AC& qdRect)
{
	return CViewRect(qdRect) + GetViewToQDOffset();
}

//----------------------------------------------------------------------------------------
// TView::RemovedASubView:
//----------------------------------------------------------------------------------------

void TView::RemovedASubView(TView* theSubView)
{
	if (fSuperView)
		fSuperView->RemovedASubView(theSubView);
}

//----------------------------------------------------------------------------------------
// TView::RemoveSubView:
//----------------------------------------------------------------------------------------

void TView::RemoveSubView(TView* theSubView)
{
	DeleteSubView(theSubView);

	RemovedASubView(theSubView);
	theSubView->BeInSuperView(NULL);
	//theSubView->RemoveFromDocument();
}

//----------------------------------------------------------------------------------------
// TView::Resize:
//----------------------------------------------------------------------------------------

void TView::Resize(const CViewPoint& newSize,
				   bool invalidate)
{
	SetFrame(CViewRect(fLocation, fLocation + newSize), invalidate);
}

//----------------------------------------------------------------------------------------
// TView::RestrictRegionToVisible:
//----------------------------------------------------------------------------------------

void TView::RestrictRegionToVisible(RgnHandle aRegion)
{
#if qDebug
	AssumeFocused();
#endif

	CRect_AC itsVisibleExtent(ViewToQD(GetVisibleExtent()));

	CTempRegion_AC visibleExtentRgn;

	RectRgn(visibleExtentRgn, &itsVisibleExtent);
	SectRgn(aRegion, visibleExtentRgn, aRegion);

	// Intersect with visible region
	SectRgn(TWindow::GetVisRegion(qd.thePort), aRegion, aRegion);

	// Don't further restrict to clip region, it may have accumulated invalidated views
	// and if it did, we won't correctly set the cursor.
	// SectRgn(GetClipRegion(qd.thePort), aRegion, aRegion);
}

//----------------------------------------------------------------------------------------
// TView::RevealBottom:
//----------------------------------------------------------------------------------------

void TView::RevealBottom(bool redraw)
{
	RevealRect(CViewRect(fSize, fSize), gZeroVPt, redraw);
}

//----------------------------------------------------------------------------------------
// TView::RevealRect:
//----------------------------------------------------------------------------------------

void TView::RevealRect(const CViewRect& rectToReveal,
					   const CViewPoint& minToSee,
					   bool redraw)
{
	if (fSuperView)
		fSuperView->RevealRect(LocalToSuperView(rectToReveal), minToSee, redraw);
}

//----------------------------------------------------------------------------------------
// TView::RevealTop:
//----------------------------------------------------------------------------------------

void TView::RevealTop(bool redraw)
{
	RevealRect(gZeroVRect, gZeroVPt, redraw);
}

//----------------------------------------------------------------------------------------
// TView::Show:
//----------------------------------------------------------------------------------------

void TView::Show(bool state,
				 bool redraw)
{
	if (state != fShown)
	{
		if (redraw)
		{
			fShown = true;						// So that ForceRedraw works.
			ForceRedraw();
		}
		InvalidateFocus();
		gDispatcher->InvalidateMouseRegions();	// Must re-calc cursor and help regions
		fShown = state;
	}
}

//----------------------------------------------------------------------------------------
// TView::ShowReverted:
//----------------------------------------------------------------------------------------

void TView::ShowReverted()
{
	AdjustFrame();
	ForceRedraw();

	if (HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
			iter->ShowReverted();
}

//----------------------------------------------------------------------------------------
// TView::SubViewChangedFrame:
//----------------------------------------------------------------------------------------

void TView::SubViewChangedFrame(TView*			/* theSubView */ ,
								const CViewRect&	/* oldFrame */ ,
								const CViewRect&	/* newFrame */ ,
								bool			/* invalidate */)
{
	// Just a warning. If you override this method to change your frame based on your
	// subview changing its frame you will need to be careful if your subviews have size
	// determiners of sizeRelSuperView. If you do, you will find yourself in an infinite
	// loop. For an sample solution look at TIncludeView::SubViewChangedFrame. Essentially
	// you have to be careful not to renotify your subviews that your frame has changed
	// since you are probably changing your frame because they changed theirs. I think you
	// get the picture.
}

//----------------------------------------------------------------------------------------
// TView::SuperViewToLocal:
//----------------------------------------------------------------------------------------

CViewPoint TView::SuperViewToLocal(const CViewPoint& thePoint) const
{
	return thePoint - GetLocalOffsetInSuper();
}

//----------------------------------------------------------------------------------------
// TView::SuperViewToLocal:
//----------------------------------------------------------------------------------------

CViewRect TView::SuperViewToLocal(const CViewRect& viewRect) const
{
	return viewRect - GetLocalOffsetInSuper();
}

//----------------------------------------------------------------------------------------
// TView::SuperViewToLocal:
//----------------------------------------------------------------------------------------

void TView::SuperViewToLocal(RgnHandle theRgn)
{
	CViewPoint localViewPt(LocalToSuperView(QDToView(gZeroPt)));	// get 0 in View Space

	CPoint_AC superQDPt(fSuperView ? fSuperView->ViewToQD(localViewPt) : localViewPt.ToPoint());

	OffsetRgn(theRgn, -superQDPt.h, -superQDPt.v);
}

//----------------------------------------------------------------------------------------
// TView::SuperViewChangedFrame:
//----------------------------------------------------------------------------------------

void TView::SuperViewChangedFrame(const CViewRect& oldFrame,
								  const CViewRect& newFrame,
								  bool invalidate)
{
	if (oldFrame != newFrame)
	{
		CViewRect itsOldFrame(GetFrame());
		CViewRect itsNewFrame(itsOldFrame);

		for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
			switch (fSizeDeterminer[vhs])
			{
				case sizeRelSuperView:
					// need to capture the amount of change in the superview here for use
					// in ComputeFrame
					itsNewFrame[botRight][vhs] += newFrame.GetSize()[vhs] - oldFrame.GetSize()[vhs];
					break;
			}

		ComputeFrame(itsNewFrame);
		if (itsOldFrame != itsNewFrame)
		{
			SetFrame(itsNewFrame, invalidate);
			DoPagination();
		}
	}
}

//----------------------------------------------------------------------------------------
// TView::IsDoneTracking:
//----------------------------------------------------------------------------------------

bool TView::IsDoneTracking()
{
	// Default to mouse-down tracking
	return !StillDown();
}

//----------------------------------------------------------------------------------------
// TView::TrackConstrain:
//----------------------------------------------------------------------------------------

void TView::TrackConstrain(TrackPhase			/* aTrackPhase */,
						   const CViewPoint&		/* anchorPoint */ ,
						   const CViewPoint&		/* previousPoint */ ,
						   CViewPoint&				/* nextPoint */ ,
						   bool				/* mouseDidMove */)
{
}

//----------------------------------------------------------------------------------------
// TView::TrackFeedback:
//----------------------------------------------------------------------------------------

void TView::TrackFeedback(TrackPhase			/* aTrackPhase */,
						  const CViewPoint& anchorPoint,
						  const CViewPoint&			/* previousPoint */ ,
						  const CViewPoint& nextPoint,
						  bool mouseDidMove,
						  bool				/* turnItOn */)

// Use code like this to get a flickering rectangle.
{
	if (mouseDidMove)
	{
		CViewRect vr(anchorPoint, nextPoint);
		vr.Validate();

		PenPat(&qd.gray);
		FrameRect(&ViewToQD(vr));
	}
}

//----------------------------------------------------------------------------------------
// TView::TrackMouse:
//----------------------------------------------------------------------------------------

void TView::TrackMouse(TrackPhase				/* aTrackPhase */,
					   CViewPoint&					/* anchorPoint */ ,
					   CViewPoint&					/* previousPoint */ ,
					   CViewPoint&					/* nextPoint */ ,
					   bool					/* mouseDidMove */)
{
}

//----------------------------------------------------------------------------------------
// TView::Update:
//----------------------------------------------------------------------------------------

void TView::Update()
{
	if (fSuperView)
		fSuperView->Update();
	else
		DrawContents();
}

//----------------------------------------------------------------------------------------
// TView::GetValidationError:
//----------------------------------------------------------------------------------------

long TView::GetValidationError()
{
	return noErr;
}

//----------------------------------------------------------------------------------------
// TView::IsValid:
//----------------------------------------------------------------------------------------

bool TView::IsValid()
{
	bool returnVal = true;

	long result = GetValidationError();
	if (result != noErr)
	{
		ValidationFailed(result);
		returnVal = false;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::ValidationFailed:
//----------------------------------------------------------------------------------------

void TView::ValidationFailed(long				/* reason */)
{
}

//----------------------------------------------------------------------------------------
// TView::IsHierarchyValid:
//----------------------------------------------------------------------------------------

bool TView::IsHierarchyValid()
{
	bool result = IsValid();

	if (result && HasSubViews())
		for (CSubViewIterator iter(this); iter; ++iter)
		{
			result = iter->IsHierarchyValid();
			if (!result)
				break;
		}

	return result;
}

//----------------------------------------------------------------------------------------
// TView::Validate:
//----------------------------------------------------------------------------------------

void TView::Validate(const CViewRect& goodVRect)
{
	Validate(ViewToQD(goodVRect));
}

//----------------------------------------------------------------------------------------
// TView::Validate:
//----------------------------------------------------------------------------------------

void TView::Validate(const CRect_AC& goodRect)
{
	CTempRegion_AC goodRectRgn;

	RectRgn(goodRectRgn, &goodRect);
	Validate(goodRectRgn);
}

//----------------------------------------------------------------------------------------
// TView::Validate:
//----------------------------------------------------------------------------------------

void TView::Validate(const RgnHandle goodRgn)
{
	if (Focus())
	{
		CTempRegion_AC visibleGoodRgn;

		RectRgn(visibleGoodRgn, &ViewToQD(GetVisibleExtent()));
		SectRgn(goodRgn, visibleGoodRgn, visibleGoodRgn);
		if (!EmptyRgn(visibleGoodRgn))
			DoValidateRegion(visibleGoodRgn);
	}
}

//----------------------------------------------------------------------------------------
// TView::DoValidateRegion:
//----------------------------------------------------------------------------------------

void TView::DoValidateRegion(const RgnHandle goodRgn)
{
	if (fSuperView)
		fSuperView->DoValidateRegion(goodRgn);	// see if someone knows how
}

//----------------------------------------------------------------------------------------
// TView::ViewEnable:
//----------------------------------------------------------------------------------------

void TView::ViewEnable(bool state,
					   bool redraw)
{
	SetEnable(state);

	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TView::ViewToQD:
//----------------------------------------------------------------------------------------

CPoint_AC TView::ViewToQD(const CViewPoint& viewPt)
{
	return (viewPt - GetViewToQDOffset()).ToPoint();
}

//----------------------------------------------------------------------------------------
// TView::ViewToQD:
//----------------------------------------------------------------------------------------

CRect_AC TView::ViewToQD(const CViewRect& viewRect)
{
	return (viewRect - GetViewToQDOffset()).ToRect();
}

//----------------------------------------------------------------------------------------
// TView::WindowToLocal:
//----------------------------------------------------------------------------------------

CViewPoint TView::RootViewToLocal(const CViewPoint& thePoint) const
{
	return fSuperView ? SuperViewToLocal(fSuperView->RootViewToLocal(thePoint)) : thePoint;
}

//----------------------------------------------------------------------------------------
// TView::RootViewToLocal:
//----------------------------------------------------------------------------------------

CViewRect TView::RootViewToLocal(const CViewRect& viewRect) const
{
	return fSuperView ? SuperViewToLocal(fSuperView->RootViewToLocal(viewRect)) : viewRect;
}

//----------------------------------------------------------------------------------------
// TView::RootViewToLocal:
//----------------------------------------------------------------------------------------

void TView::RootViewToLocal(RgnHandle theRgn)
{
	if (fSuperView)
	{
		fSuperView->RootViewToLocal(theRgn);
		SuperViewToLocal(theRgn);
	}
}

//----------------------------------------------------------------------------------------
// TView::AsPict:
//----------------------------------------------------------------------------------------

PicHandle TView::AsPict()
{
	GrafPtr savedPort;
	GetPort(&savedPort);
	PicHandle pHndl = NULL;
	bool pictureIsOpen = false;

	CRect_AC aRect(savedPort->portRect);
	
	CRect_AC oldPortFrame = gWorkPort->portRect;
	SetPortWindowPort(gWorkPort);
	PortSize(aRect.GetLength(hSel), aRect.GetLength(vSel));
	MovePortTo(aRect.left, aRect.top);

	InvalidateFocus();
	gDrawingPictScrap = true;
	gDrawingPictScrapView = this;
	SetPortWindowPort(gWorkPort);
	BeInPort((GrafPtr)GetWindowPort(gWorkPort));

	try
	{
		if (Focus())
		{
			//Open new or old style, depending on QD
			CRect_AC qdExtent(GetQDExtent());
			if (qNeedsColorQD || HasColorQD_AC())
			{
				OpenCPicParams picParms;
				picParms.srcRect = qdExtent;
				picParms.hRes = 0x00480000;
				picParms.vRes = 0x00480000;
				picParms.version = -2;
				picParms.reserved1 = 0;
				picParms.reserved2 = 0;
				pHndl = OpenCPicture(&picParms);
			}
			else
				pHndl = OpenPicture(&qdExtent);
			pictureIsOpen = true;				// Failure handler needs to close it
			::ThrowIfNULL_AC(pHndl);

			::ClipRect(&qdExtent);
			DrawContents();

			ClosePicture();
			pictureIsOpen = false;				// Failure handler doesn't need to close it

			//  On the 128K ROMs the picFrame will be empty if drawing the
			// picture failed.
			if (((CRect_AC &)(*pHndl)->picFrame).Empty())
			{
#if qDebug
				ProgramReport_AC("Picture frame is empty!\n");
#endif

				::ThrowIfOSErr_AC(memFullErr);
			}

			InvalidateFocus();
		}
#if qDebug
		else									// can't focus
			::ProgramBreak_AC("Can't focus view while writing picture");
#endif
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		if (pHndl)
		{
			if (pictureIsOpen)
				ClosePicture();
			pHndl = DisposeIfPicHandle(pHndl);
		}

		PortSize(oldPortFrame.GetLength(hSel), oldPortFrame.GetLength(vSel));
		MovePortTo(oldPortFrame.right, oldPortFrame.top);
	
		gDrawingPictScrap = false;
		gDrawingPictScrapView = NULL;
		SetPortWindowPort(savedPort);
		BeInPort(savedPort);
		throw;
	}

	PortSize(oldPortFrame.GetLength(hSel), oldPortFrame.GetLength(vSel));
	MovePortTo(oldPortFrame.right, oldPortFrame.top);

	gDrawingPictScrap = false;
	gDrawingPictScrapView = NULL;
	SetPortWindowPort(savedPort);
	BeInPort(savedPort);

	return pHndl;
}


//----------------------------------------------------------------------------------------
// TView::WriteToDeskScrap:
//----------------------------------------------------------------------------------------

void TView::WriteToDeskScrap()
{
	PicHandle pHndl = NULL;

	try
	{
		pHndl = AsPict();
		::ThrowIfNULL_AC(pHndl);

		::ThrowIfOSErr_AC(gClipboardMgr->PutDeskScrapData(kPictType_AC, (Handle)pHndl));

		pHndl = DisposeIfPicHandle(pHndl);		// dispose of picture created by AsPict
		// so failure handler doesn't to kill it
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		if (pHndl)
		{
#if qDebug
			::ProgramBreak_AC("Failed to put PICT-type scrap");
#endif

			pHndl = DisposeIfPicHandle(pHndl);
		}
		throw;
	}

}

//----------------------------------------------------------------------------------------
// TView::GoAwayByUser:
//----------------------------------------------------------------------------------------

void TView::GoAwayByUser(const CViewPoint&			/* theMouse */ )
{
}

//----------------------------------------------------------------------------------------
// TView::MoveByUser:
//----------------------------------------------------------------------------------------

void TView::MoveByUser(const CViewPoint&			/* theMouse */ )
{
}

//----------------------------------------------------------------------------------------
// TView::ResizeByUser:
//----------------------------------------------------------------------------------------

void TView::ResizeByUser(const CViewPoint&			/* theMouse */ )
{
}

//----------------------------------------------------------------------------------------
// TView::SetHLPenState:
//----------------------------------------------------------------------------------------

void TView::SetHLPenState(HLState fromHL,
						  HLState toHL)
{
	short mode = patXor;						// every transition except hlOn <-> hlDim uses patXOR

	PatPtr pPat;
	if (fromHL + toHL == hlOffOn)
		pPat = &qd.black;
	else if (fromHL != toHL)
		pPat = &qd.gray;

	if (fromHL + toHL == hlDimOn)
		mode = notPatXor;

	PenMode(mode);
	PenPat(pPat);
}

//----------------------------------------------------------------------------------------
// TView::UpdateCoordinates:
//----------------------------------------------------------------------------------------

void TView::UpdateCoordinates()
{
	const short kMaxOriginFixup = 1024;			// The maximum amount by which to _fixup_
	// (not offset) the QD origin when
	// focusing. This allows patterns and
	// pixpats to be pinned to global space as
	// they were intended. See note below.
	// Larger values than this may be required
	// for _extremely_ complex pixpats (not
	// patterns as they are restricted to 8
	// bit repetition.) The reason that the
	// fixup is restricted is that you don't
	// want to run off the end of QD space and
	// we already have slop allowance of about
	// 2k in ViewCoords.p.

	// we can't update our coordinates until our superview updates theirs
	// however once a views coordinates are valid we don't have to walk up
	// the tree any further.
	if (fCoordinatesAreInvalid)
	{
		if (fSuperView)
			fSuperView->UpdateCoordinates();

		CPoint_AC oldQDOrigin(fQDOrigin);
		CViewPoint oldViewToQDOffset(fViewToQDOffset);
		CViewRect oldVisibleExtent(fVisibleExtent);

		if (fSuperView || (TPrintHandler::gCurrPrintHandler && (TPrintHandler::gCurrPrintHandler->fView == this)))
		{
			// First calculate and setup the new origins.
			// But first a word about this approach. (thanks Keith Rollins!) Problems with
			// pattern alignment when using 32bit views occur when virtual origin of the port
			// has coordinates that are not multiples of 8. This can occur if, for example,
			// you scroll a view by increments of other than 8, 16, or other multiple of 8.
			// The solution is to appropriately set the origin so that patterns are drawn
			// correctly aligned. This can be done by making sure that we set the origin to
			// some value that will cause QuickDraw to perform in exactly the same way as if
			// the origin were really at the value specified by fViewToQDOffset. Since classic
			// QuickDraw patterns repeat after 8 pixels, we can accomplish this task by making
			// sure that fViewToQDOffset is a multiple of 8, and setting the origin to
			// compensate by setting it to fViewToQDOffset % 8.
			// Unfortunately, this will only work for classic QuickDraw patterns that repeat
			// after 8 pixels. With PixPats, this period can be any power of 2. In order to
			// support them, we have to make sure that fViewToQDOffset must be only a multiple
			// of whatever power of 2 is being used. Ideally, therefore, we could like to make
			// the chunkiness of fViewToQDOffset to be as large as possible. On the other
			// hand, we can't make it too large, or else the amount by which we offset the
			// origin in TView.Focus may overflow QuickDraw's limits. Probably a value of 1024
			// or 2048 should be sufficient enough to accomodate the largest of PixPats a
			// developer may use, and still be small enough that we shouldn't overflow
			// QuickDraw.
			CViewPoint localOriginInSuper(GetLocalOffsetInSuper());

			CViewPoint actualViewToQDOffset;
			if (TPrintHandler::gCurrPrintHandler && (TPrintHandler::gCurrPrintHandler->fView == this))
				actualViewToQDOffset = TPrintHandler::gCurrPrintHandler->GetViewToQDOffset() + CViewPoint(TPrintHandler::gCurrPrintHandler->GetQDOrigin());
			else
				actualViewToQDOffset = fSuperView->fViewToQDOffset + CViewPoint(fSuperView->fQDOrigin) - localOriginInSuper;

			fViewToQDOffset = actualViewToQDOffset;// to allow testing of the Extent
			if (CViewRect(-kMaxQDCoord, -kMaxQDCoord, kMaxQDCoord, kMaxQDCoord).Contains(GetExtent()))
			{
				fQDOrigin = actualViewToQDOffset.ToPoint();
				fViewToQDOffset.Clear();
			}
			else
			{
				CPoint_AC deltaOrigin(static_cast<short>(actualViewToQDOffset.h % kMaxOriginFixup), static_cast<short>(actualViewToQDOffset.v % kMaxOriginFixup));
				fViewToQDOffset = actualViewToQDOffset - CViewPoint(deltaOrigin);
				fQDOrigin = deltaOrigin;
			}

			// Update my visible extent by intersecting my extent with my superview's visible extent

			// When printing, or drawing the scrap, the grafPort is supplied.
			if ((gDrawingPictScrapView == this) || (TPrintHandler::gCurrPrintHandler && TPrintHandler::gCurrPrintHandler->fView == this))
				fVisibleExtent = GetExtent();
			else
				fVisibleExtent = SuperViewToLocal(GetFrame() & fSuperView->GetVisibleExtent());
		}
		else
		{
			fViewToQDOffset.Clear();
			fQDOrigin.Clear();

			fVisibleExtent = GetExtent();
		}

		fCoordinatesAreInvalid = false;
	}

}

//----------------------------------------------------------------------------------------
// TView::InvalidateCoordinates:
//----------------------------------------------------------------------------------------
void TView::InvalidateCoordinates()
{
	if (!fCoordinatesAreInvalid)
	{
		fCoordinatesAreInvalid = true;

		for (CSubViewIterator iter(this); iter; ++iter)
			iter->InvalidateCoordinates();
	}
}

//----------------------------------------------------------------------------------------
// TView::GetVisibleExtent:
//----------------------------------------------------------------------------------------
CViewRect TView::GetVisibleExtent()
{
	if (fCoordinatesAreInvalid)
		UpdateCoordinates();

	return fVisibleExtent;
}

//----------------------------------------------------------------------------------------
// TView::GetQDOrigin:
//----------------------------------------------------------------------------------------
CPoint_AC TView::GetQDOrigin()
{
	if (fCoordinatesAreInvalid)
		UpdateCoordinates();

	return fQDOrigin;
}

//----------------------------------------------------------------------------------------
// TView::GetViewToQDOffset:
//----------------------------------------------------------------------------------------
CViewPoint TView::GetViewToQDOffset()
{
	if (fCoordinatesAreInvalid)
		UpdateCoordinates();

	return fViewToQDOffset;
}

//----------------------------------------------------------------------------------------
// TView::IsOpaque:
//----------------------------------------------------------------------------------------
bool TView::IsOpaque() const
{
	bool returnVal = fIsOpaque;

	if (!returnVal)
		for (CAdornerIterator iter(this); iter; ++iter)
			if (iter->DoesAdorn(this) && iter->IsOpaque())
			{
				returnVal = true;
				break;
			}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::ClipSubViews:
//----------------------------------------------------------------------------------------
bool TView::ClipSubViews(RgnHandle rRgn)
{
	bool returnVal = false;

	if (HasSubViews() && !::EmptyRgn(rRgn))
	{
	#if qDebug
		AssumeFocused();
	#endif

		// We won't claim subviews' territory
		bool rgnHandleWasTooBig = false;
		CTempRegion_AC subViewFrameRgn;
		CViewRect myVisExtent(GetVisibleExtent());

		for (CSubViewIterator iter(this); iter; ++iter)
			if (!rgnHandleWasTooBig && iter->IsOpaque() && !CViewRect(myVisExtent & iter->GetFrame()).Empty())
			{
				iter->GetExtentRegion(subViewFrameRgn);// Remember, the frame is in OUR
				// coordinate system
				iter->LocalToSuperView(subViewFrameRgn);
				::DiffRgn(rRgn, subViewFrameRgn, rRgn);// Remove the subview from the claimed rgn
				returnVal = true;	// did modify region

				if ((GetHandleSize((Handle)rRgn) > kRgnHandleTooBig) || (MemError() != noErr))
				{
					rgnHandleWasTooBig = true;
					break; // can't do any more after this due to size of resulting region
				}
			}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::ClipSiblingViews:
//----------------------------------------------------------------------------------------
bool TView::ClipSiblingViews(RgnHandle rRgn)
{
	bool returnVal = false;

	TView* iter = fNextView;
	if (iter && !::EmptyRgn(rRgn))
	{

	#if qDebug
		AssumeFocused();
	#endif

		// We won't claim sibling's territory
		bool rgnHandleWasTooBig = false;
		CTempRegion_AC sibViewFrameRgn;
		CViewRect myVisFrame(LocalToSuperView(GetVisibleExtent()));

		while (iter)
		{
			if (!rgnHandleWasTooBig && iter->IsOpaque() && !CViewRect(myVisFrame & iter->GetFrame()).Empty())
			{
				iter->GetExtentRegion(sibViewFrameRgn);// Remember, the frame is in OUR superview coordinate system
				iter->LocalToSuperView(sibViewFrameRgn);
				SuperViewToLocal(sibViewFrameRgn);
				::DiffRgn(rRgn, sibViewFrameRgn, rRgn);// Remove the subview from the claimed rgn
				returnVal = true;	// did modify region

				if ((GetHandleSize((Handle)rRgn) > kRgnHandleTooBig) || (MemError() != noErr))
				{
					rgnHandleWasTooBig = true;
					break; // can't do any more after this due to size of resulting region
				}
			}
			iter = iter->fNextView;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TView::ZoomByUser:
//----------------------------------------------------------------------------------------

void TView::ZoomByUser(const CViewPoint&			/* theMouse */ ,
					   short					/* partCode */)
{
}

//------------------------------------------------------------------------------------
// Drag and Drop Methods
//------------------------------------------------------------------------------------

#if qDrag
//----------------------------------------------------------------------------------------
// TView::SetDroppable:
//----------------------------------------------------------------------------------------

void TView::SetDroppable(bool isDroppable)
{
	if (HasDragManager_AC())
	{
		TWindow * itsWindow = GetWindow();
		if (isDroppable != fDroppable)
		{
			fDroppable = isDroppable;
			if (itsWindow)
			{
				if (fDroppable)
					itsWindow->RegisterDroppableView(this);
				else if (!fDroppable)
					itsWindow->UnregisterDroppableView(this);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TView::HandleDragCursor:
//----------------------------------------------------------------------------------------

void TView::HandleDragCursor(const CViewPoint& theMouse,
							 RgnHandle cursorRegion)
{
	if (HasSubViews())
		for (CSubViewIterator iter(this, kIterateBackward_AC); iter; ++iter)
		{
			CViewPoint subViewPt(iter->SuperViewToLocal(theMouse));

			if (iter->ContainsMouse(subViewPt) && (iter->DoGetDragProxy()->GetDraggable() || iter->HasSubViews()))
			{
				iter->HandleDragCursor(subViewPt, cursorRegion);
				// bring the cursor region back to our coordinate system
				iter->LocalToSuperView(cursorRegion);
				break;
			}
		}

	if (EmptyRgn(cursorRegion) && Focus())
	{
		GetExtentRegion(cursorRegion);

		if (HasSubViews())
		{
			CTempRegion_AC subViewFrameRgn;

			for (CSubViewIterator iter(this); iter; ++iter)
				if (iter->DoGetDragProxy()->GetDraggable() || iter->HasSubViews())
				{
					iter->GetExtentRegion(subViewFrameRgn);
					iter->LocalToSuperView(subViewFrameRgn);
					DiffRgn(cursorRegion, subViewFrameRgn, cursorRegion);
				}
		}

		RgnHandle dragCursorRegion = NULL;
		if (!DoSetDragCursor(theMouse, cursorRegion, dragCursorRegion))
		{
			if (dragCursorRegion)
				DiffRgn(cursorRegion, dragCursorRegion, cursorRegion);
			SetCursor(&(qd.arrow));
		}

		DisposeIfRgnHandle(dragCursorRegion);

		RestrictRegionToVisible(cursorRegion);
	}
}

//----------------------------------------------------------------------------------------
// TView::DoSetDragCursor:
//----------------------------------------------------------------------------------------

bool TView::DoSetDragCursor(const CViewPoint& localPoint,
							   RgnHandle cursorRegion,
							   RgnHandle& dragCursorRegion)
{
#if qDebug
	::Assertion_AC(dragCursorRegion == NULL, "dragCursorRegion should be nil");
	AssumeFocused();
#endif

	dragCursorRegion = DoMakeDragCursorRegion();
	bool usingDragCursor = WillDrag(localPoint, dragCursorRegion);
	if (usingDragCursor)
	{
		// cursor region is the intersection of the drag region
		// and the provided cursor region
		::ThrowIfNULL_AC(dragCursorRegion);
		SectRgn(cursorRegion, dragCursorRegion, cursorRegion);
		short cursorID = GetWillDragCursorID();
		if (cursorID != kNoResource)
		{
			CursHandle aCursor = GetCursor(cursorID);
			::ThrowIfNULLResource_AC((Handle)aCursor);
			SetCursor(*aCursor);
		}
		else
			SetCursor(&(qd.arrow));
	}

	return usingDragCursor;
}

//----------------------------------------------------------------------------------------
// TView::GetWillDragCursorID:
//----------------------------------------------------------------------------------------

short TView::GetWillDragCursorID()
{
	return kOpenHandCursor;
}

//----------------------------------------------------------------------------------------
// TView::GetIsDraggingCursorID:
//----------------------------------------------------------------------------------------

short TView::GetIsDraggingCursorID()
{
	return kClosedHandCursor;
}

//----------------------------------------------------------------------------------------
// TView::DoMakeDragCursorRegion:
//----------------------------------------------------------------------------------------

RgnHandle TView::DoMakeDragCursorRegion()
{
	RgnHandle dragCursorRegion = NULL;

	if (GetDraggable())
		dragCursorRegion = DoMakeDropHiliteRegion();// simple case

	return dragCursorRegion;
}

//----------------------------------------------------------------------------------------
// TView::WillDrag
//----------------------------------------------------------------------------------------

bool TView::WillDrag(const CViewPoint& localMouse,
						const RgnHandle dragCursorRegion)
{
	return dragCursorRegion && GetDraggable() ? PtInRgn(ViewToQD(localMouse), dragCursorRegion) : false;
}

//----------------------------------------------------------------------------------------
// TView::DoGetDragProxy:
//----------------------------------------------------------------------------------------

TView* TView::DoGetDragProxy()
{
	return this;								// default is to not provide a proxy
}

//----------------------------------------------------------------------------------------
// TView::HandleDrag:
//----------------------------------------------------------------------------------------

void TView::HandleDrag(TToolboxEvent* event)
{
	TDragDropSession::fgDragDropSession->ClearDrag();
	DoAddDragContent();

	CTempRegion_AC dragRegion(DoMakeDragOutlineRegion());

	TDragDropSession::fgDragDropSession->StartDrag(this, event, dragRegion);
}

//----------------------------------------------------------------------------------------
// TView::DoAddDragContent:
//----------------------------------------------------------------------------------------

void TView::DoAddDragContent()
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TView::DoMakeDragOutlineRegion
//----------------------------------------------------------------------------------------

RgnHandle TView::DoMakeDragOutlineRegion()
{
	// The drag outline region will usually match the drag cursor region.
	RgnHandle dragOutlineRegion = DoMakeDragCursorRegion();
	CTempRegion_AC tempRegion;

	CopyRgn(dragOutlineRegion, tempRegion);		// make it an outline
	InsetRgn(tempRegion, 1, 1);
	DiffRgn(dragOutlineRegion, tempRegion, dragOutlineRegion);

	LocalToGlobal(dragOutlineRegion);		// globalize

	return dragOutlineRegion;
}

//----------------------------------------------------------------------------------------
// TView::WillAcceptDrop:
//----------------------------------------------------------------------------------------

bool TView::WillAcceptDrop(CDragItemIterator&/* dragItemIterator */  )
{
	SubClassResponsibility();

	return false;
}

//----------------------------------------------------------------------------------------
// TView::DoDragEnter:
//----------------------------------------------------------------------------------------

void TView::DoDragEnter()
{
	// set up private data used to track drag
}

//----------------------------------------------------------------------------------------
// TView::DoMakeDropHiliteRegion:
//----------------------------------------------------------------------------------------

RgnHandle TView::DoMakeDropHiliteRegion()
{
	RgnHandle dropRegion = MakeNewRgn();
	RectRgn(dropRegion, &ViewToQD(GetVisibleExtent()));

	return dropRegion;
}

//----------------------------------------------------------------------------------------
// TView::DragWithin:
//----------------------------------------------------------------------------------------

void TView::DoDragWithin(const CViewPoint&			/*localMouse*/ )
{
	// update tracking data and visuals
}

//----------------------------------------------------------------------------------------
// TView::DoDragLeave:
//----------------------------------------------------------------------------------------

void TView::DoDragLeave()
{
	// clean up after the drag has left
}

//----------------------------------------------------------------------------------------
// TView::DoMakeDropLocationDescriptor:
//----------------------------------------------------------------------------------------

bool TView::DoMakeDropLocationDescriptor(CAEDesc& dropLocationDesc)
{
	// Default implementation is to return a description of the containing window.
	// Subclasses of TView that mix MScriptable object may choose to describe themselves rather
	// than their containing window.
	bool result = false;
	TWindow * myWindow = GetWindow();

	if (myWindow)
	{
		try
		{
			result = myWindow->MakeObjectSpecifier(dropLocationDesc, formAbsolutePosition);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
		}

		// no resignal on failure
	}
	return result;
}

//----------------------------------------------------------------------------------------
// TView::WillDragMove:
//----------------------------------------------------------------------------------------

bool TView::WillDragMove(TView* destinationView)
{
	// test the family id's first
	bool isMove = (GetDragMoveFamily() == destinationView->GetDragMoveFamily());

	if (isMove)
	{
		switch (GetDragMoveDeterminer())
		{
			case kNeverMove:
				isMove = false;
				break;

			case kMoveWithinView:
				isMove = (this == destinationView);
				break;

			case kMoveWithinWindow:
				isMove = (GetWindow() == destinationView->GetWindow());
				break;

			case kMoveWithinContext:
				isMove = (GetContext(cDragMove) == destinationView->GetContext(cDragMove));
				break;

			case kMoveWithinApplication:		// already true.
				break;

			default:
				isMove = false;					// unknown context value
				break;
		}
	}

	return isMove;
}

//----------------------------------------------------------------------------------------
// TView::DoMakeDragDropCommand:
//----------------------------------------------------------------------------------------

TCommand* TView::DoMakeDragDropCommand(CommandNumber/* itsCommandNumber */ ,
									   CDragItemIterator&/* dragItemIterator */  )
{
	return NULL;								// subview responsibility
}

//----------------------------------------------------------------------------------------
// TView::DoFulfillPromise:
//----------------------------------------------------------------------------------------

void TView::DoFulfillPromise(TDragItem*			/*promisedItem */ )
{
	// subview responsibility
}

#endif // qDrag

//========================================================================================
// CLASS TIncludeView
//========================================================================================
#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS(TIncludeView);

//----------------------------------------------------------------------------------------
// TIncludeView constructor.
//----------------------------------------------------------------------------------------

TIncludeView::TIncludeView(short includeRsrcID,
								const CViewPoint& itsSubViewOffset,
								const CStr255_AC& resourceName,
								short resourceAttributes) :
	TView(gZeroVPt, gZeroVPt, sizeSuperView, sizeSuperView),

	fResourceName(resourceName),
	fItsSubViewOffset(itsSubViewOffset),
	fIncludeRsrcID(includeRsrcID),
	fResourceAttributes(resourceAttributes)
{
	fEnabled = false;
}

//----------------------------------------------------------------------------------------
// TIncludeView destructor
//----------------------------------------------------------------------------------------

TIncludeView::~TIncludeView()
{
}

//----------------------------------------------------------------------------------------
// TIncludeView::GetStandardSignature:
//----------------------------------------------------------------------------------------

IDType TIncludeView::GetStandardSignature() const		// override
{
	return kStdIncludeAt;
}

//----------------------------------------------------------------------------------------
// TIncludeView::DoPostCreate:
//----------------------------------------------------------------------------------------

void TIncludeView::DoPostCreate(TDocument* itsDocument)// override
{
	Inherited::DoPostCreate(itsDocument);

	// It only makes sense to do this stuff if we have a view to include.
	if (fIncludeRsrcID != kNoResource)
	{
		// initially, make me "look like" my superview
		if (fSuperView)
			SetFrame(CViewRect(gZeroVPt, fSuperView->GetSize()), kDontInvalidate);

		// get the 'View' resource
		Handle viewResource = GetResource('View', fIncludeRsrcID);
#if qDebug
		if (!viewResource)
		{
			viewResource = GetResource('view', fIncludeRsrcID);
			if (viewResource)
			{
				::ProgramBreak_AC("Use ViewPromoter. Program contains 2.0 'view' resources");
				ReleaseResource(viewResource);
				viewResource = NULL;
			}
		}
#endif

		::ThrowIfNULLResource_AC(viewResource);

		short theID;
		ResType theType;
		GetResInfo(viewResource, &theID, &theType, fResourceName);

		fResourceAttributes = GetResAttrs(viewResource);

		// create the included view
		TView * aView = TViewServer::fgViewServer->DoCreateViews(fDocument, this, fIncludeRsrcID, gZeroVPt);
		::ThrowIfNULL_AC(aView);

		// now, make me "look like" my subview
		fSizeDeterminer[vSel] = aView->GetSizeDeterminer(vSel);
		fSizeDeterminer[hSel] = aView->GetSizeDeterminer(hSel);
		SubViewChangedFrame(aView, gZeroVRect, CViewRect(fItsSubViewOffset, fItsSubViewOffset + aView->GetSize()), kDontInvalidate);
	}
}

//----------------------------------------------------------------------------------------
// TIncludeView::ReadFields:
//----------------------------------------------------------------------------------------

void TIncludeView::ReadFields(CStream_AC* aStream)
{
	Inherited::ReadFields(aStream);

	*aStream >> fIncludeRsrcID >> fItsSubViewOffset;

#if 0
	fIncludeRsrcID = aStream->ReadInteger();
	fItsSubViewOffset = aStream->ReadLongPoint();
#endif

}

//----------------------------------------------------------------------------------------
// TIncludeView::WriteFields:
//----------------------------------------------------------------------------------------

void TIncludeView::WriteFields(CStream_AC* aStream) const
{
	Inherited::WriteFields(aStream);

	*aStream << fIncludeRsrcID << fItsSubViewOffset;
#if 0
	aStream->WriteInteger(fIncludeRsrcID);

	CViewPoint itsSubViewOffset = fItsSubViewOffset;
	aStream->WriteLongPoint(itsSubViewOffset);
#endif
}

//----------------------------------------------------------------------------------------
// TIncludeView::WriteSubViews:
//----------------------------------------------------------------------------------------

void TIncludeView::WriteSubViews(CStream_AC* aStream) const
{
	// our subviews are "included" and therefore not writeable, so don't call Inherited
	aStream->WriteInteger(0);					// write out count of 0 subviews

	// Call into ViewServer to write out my subview to a 'View' resource with id=fIncludeRsrcID.
#if qDebugMsg
	if (CountSubViews() != 1)
		ProgramReport_AC("TIncludeView::WriteTo only supports writing of the first subview.\n");
#endif

	if (fIncludeRsrcID != kNoResource)
	{
		CStr255_AC name = fResourceName;
		TViewServer::fgViewServer->WriteViewsToResource(FirstSubView(), fIncludeRsrcID, name, fResourceAttributes);
	}
}

//----------------------------------------------------------------------------------------
// TIncludeView::SubViewChangedFrame:
//----------------------------------------------------------------------------------------

void TIncludeView::SubViewChangedFrame(TView* ,
									   const CViewRect& ,
									   const CViewRect& newFrame,
									   bool invalidate)
{
	// Assume that TIncludeView has only 1 subview and that this view should remain the
	// same size as this one subview. We need to duplicate most of SetFrame here except
	// that we don't want to renotify our subviews that our frame has changed, especially
	// if our subview has size determiners of sizeRelSuperView.

	CViewRect oldFrame(GetFrame());

	CViewRect computedFrame(newFrame);
	computedFrame += fLocation;					// offset newFrame by my location

	if (computedFrame != oldFrame)
	{

		// update the frame stored in the view
		fLocation = computedFrame[topLeft];
		fSize = computedFrame.GetSize();

		InvalidateFocus();						// Must re-focus because frame changed
		InvalidateCoordinates();				// Re-calc all the offsets
		gDispatcher->InvalidateMouseRegions();	// Must re-calc cursor and help regions

		if (invalidate)
			InvalidateFrameDifference(oldFrame, computedFrame);

		//  Let my adorners know that my frame changed
		NotifyAdornersOfFrameChange(oldFrame, computedFrame, invalidate);

		//  Let my superview know that my frame changed
		if (fSuperView)
			fSuperView->SubViewChangedFrame(this, oldFrame, computedFrame, invalidate);
	}
}

//========================================================================================
// static functions
//========================================================================================

//----------------------------------------------------------------------------------------
// VisibleRect:
//----------------------------------------------------------------------------------------

void TView::VisibleRect(CRect_AC& rectToRestrict)
{
	CTempRegion_AC rRgn;

	RectRgn(rRgn, &rectToRestrict);
	VisibleRgn(rRgn);

	rectToRestrict = (*(RgnHandle)rRgn)->rgnBBox;
}

//----------------------------------------------------------------------------------------
// DrawableRect:
//----------------------------------------------------------------------------------------

void TView::DrawableRect(CRect_AC& rectToRestrict)
{
	VisibleRect(rectToRestrict);

	CTempRegion_AC rRgn;

	RectRgn(rRgn, &rectToRestrict);
	DrawableRgn(rRgn);

	rectToRestrict = (*(RgnHandle)rRgn)->rgnBBox;
}

//----------------------------------------------------------------------------------------
// VisibleRgn:
//----------------------------------------------------------------------------------------

void TView::VisibleRgn(RgnHandle rgn)
{
	if (!gPrinting && !gDrawingPictScrap)
		SectRgn(rgn, TWindow::GetVisRegion(qd.thePort), rgn);
}

//----------------------------------------------------------------------------------------
// DrawableRgn:
//----------------------------------------------------------------------------------------

void TView::DrawableRgn(RgnHandle rgn)
{
	SectRgn(rgn, GetClipRegion(qd.thePort), rgn);
}

//----------------------------------------------------------------------------------------
// IsPtVisible:
//----------------------------------------------------------------------------------------

bool TView::IsPtVisible(CPoint_AC pt)
{
	return PtInRgn(pt, TWindow::GetVisRegion(qd.thePort));
}

//----------------------------------------------------------------------------------------
// IsPtDrawable:
//----------------------------------------------------------------------------------------

bool TView::IsPtDrawable(CPoint_AC pt)
{
	return IsPtVisible(pt) && PtInRgn(pt, GetClipRegion(qd.thePort));
}

//----------------------------------------------------------------------------------------
// IsRectVisible:
//----------------------------------------------------------------------------------------

bool TView::IsRectVisible(const CRect_AC& r)
{
	return RectInRgn(&r, TWindow::GetVisRegion(qd.thePort));
}

//----------------------------------------------------------------------------------------
// IsRectDrawable:
//----------------------------------------------------------------------------------------

bool TView::IsRectDrawable(const CRect_AC& r)
{
	return IsRectVisible(r) && RectInRgn(&r, GetClipRegion(qd.thePort));
}

//----------------------------------------------------------------------------------------
// GetFocus:
//----------------------------------------------------------------------------------------

void TView::GetFocus(FocusRec& theFocusRec)
{
	GetViewPortInfo(theFocusRec.itsViewPortInfo);

	theFocusRec.focusedView = gFocusedView;
	theFocusRec.printing = gPrinting;
	theFocusRec.drawingPictScrap = gDrawingPictScrap;
	theFocusRec.drawingPictScrapView = gDrawingPictScrapView;
}

//----------------------------------------------------------------------------------------
// SetFocus:
//----------------------------------------------------------------------------------------

void TView::SetFocus(const FocusRec& theFocusRec)
{
	gFocusedView = theFocusRec.focusedView;
	gPrinting = theFocusRec.printing;
	gDrawingPictScrap = theFocusRec.drawingPictScrap;
	gDrawingPictScrapView = theFocusRec.drawingPictScrapView;

	SetViewPortInfo(theFocusRec.itsViewPortInfo);
}

//----------------------------------------------------------------------------------------
// GetViewPortInfo:
//----------------------------------------------------------------------------------------

void TView::GetViewPortInfo(ViewPortInfo& theViewPortInfo)
{
	GetPort(&(theViewPortInfo.port));
	GetClip(theViewPortInfo.clip);
	theViewPortInfo.org = ((CRect_AC &)theViewPortInfo.port->portRect)[topLeft];
	theViewPortInfo.isValid = true;
}

//----------------------------------------------------------------------------------------
// SetViewPortInfo:
//----------------------------------------------------------------------------------------

void TView::SetViewPortInfo(const ViewPortInfo& theViewPortInfo)
{
	if (qd.thePort != theViewPortInfo.port)		// be nice to accelerator cards
		SetPort(theViewPortInfo.port);
	if (((CRect_AC &)qd.thePort->portRect)[topLeft] != theViewPortInfo.org)
		SetOrigin(theViewPortInfo.org.h, theViewPortInfo.org.v);
	SetClip(theViewPortInfo.clip);
}

//----------------------------------------------------------------------------------------
// WriteFocus:
//----------------------------------------------------------------------------------------
#if qDebug

void TView::WriteFocus()
{
	ProgramReport_AC(false, "  fViewToQDOffset = %s\n", (const char*)(gFocusedView->fViewToQDOffset));
	ProgramReport_AC(false, "         portRect = %s\n", (const char*)(((CRect_AC &)qd.thePort->portRect)));
	ProgramReport_AC(false, "           visRgn = %s\n", (const char*)(((CRect_AC &)(*TWindow::GetVisRegion(qd.thePort))->rgnBBox)));
	ProgramReport_AC(false, "          clipRgn = %s\n", (const char*)(((CRect_AC &)(*GetClipRegion(qd.thePort))->rgnBBox)));
}
#endif

//----------------------------------------------------------------------------------------
// GetClipRegion:
//----------------------------------------------------------------------------------------

RgnHandle TView::GetClipRegion(GrafPtr theGrafPtr)
{
	return theGrafPtr->clipRgn;
}

//----------------------------------------------------------------------------------------
// TView::InitUView
//----------------------------------------------------------------------------------------

void TView::InitUView(bool
#if qGXViews || qGXPrinting
					  initGX
#endif
					  )
{
#if qGXViews || qGXPrinting
	if (initGX && HasQDGXGraphics_AC())
	{
		static const long kGXUsegaszRsrc = 0;	// '0' indicates that GX should use the 'gasz' rsrc in your app

		gGXGraphicsClient = GXNewGraphicsClient(NULL, kGXUsegaszRsrc, 0);

		if (gGXGraphicsClient)
			GXEnterGraphics();
	}
#endif

}

//----------------------------------------------------------------------------------------
// TView::TerminateUView
//----------------------------------------------------------------------------------------

void TView::TerminateUView()
{
#if qGXViews || qGXPrinting
	if (gGXGraphicsClient)
	{

		GXDisposeGraphicsClient(gGXGraphicsClient);
		gGXGraphicsClient = NULL;

		// Only need checking if debugging the application
		GXExitGraphics();
	}
#endif

}

//----------------------------------------------------------------------------------------
// End of UView.cp

