// UWindow.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UWINDOW__
#define __UWINDOW__

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __USCRIPTABLEOBJECT__
#include "UScriptableObject.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

// Toolbox

// defines WindowRef
#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

//----------------------------------------------------------------------------------
// External and forward class declarations. 
//----------------------------------------------------------------------------------
class TDialogBehavior;
class TDocument;
#if qDrag
	class CList_AC;
#endif
class TToolboxEvent;

//----------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------
const short kFloatingWindowKind = 128;			// the windowKind of floating windows 
const DescType cPalette = 'cPal';

#if qDrag
enum EDragCallbackStatus { eWindowNotInited, eNotInstalled, eInstalled };
#endif // qDrag

struct WindowFlags
{
	private:
		unsigned short fWindowFlagBits;
		
		enum
		{
			kHasGoAwayMask = 0x8000,
			kResizableMask = 0x4000,
			kDoFirstClickMask = 0x2000,
			kFreeOnClosingMask = 0x1000,
			kDisposeOnFreeMask = 0x0800,
			kClosesDocumentMask = 0x0400,
			kOpenInitiallyMask = 0x0200,
			kMustAdaptToScreenMask = 0x0100,
			kStaggerMask = 0x0080,
			kMustForceOnScreenMask = 0x0040,
			kVertCenterMask = 0x0020,
			kHorzCenterMask = 0x0010,
			kFloatsMask = 0x0008,
			kHideOnSuspendMask = 0x0004,
			kGenerateActivatesMask = 0x0002
			// note that the low bit is used as filler
		};
		
	public:
		inline void Initialize() { fWindowFlagBits = 0x0000; }
		
		inline bool GetHasGoAway() { return(fWindowFlagBits & kHasGoAwayMask) >> 15; }
		inline bool GetResizable() { return(fWindowFlagBits & kResizableMask) >> 14; }
		inline bool GetDoFirstClick() { return(fWindowFlagBits & kDoFirstClickMask) >> 13; }
		inline bool GetFreeOnClosing() { return(fWindowFlagBits & kFreeOnClosingMask) >> 12; }
		inline bool GetDisposeOnFree() { return(fWindowFlagBits & kDisposeOnFreeMask) >> 11; }
		inline bool GetClosesDocument() { return(fWindowFlagBits & kClosesDocumentMask) >> 10; }
		inline bool GetOpenInitially() { return(fWindowFlagBits & kOpenInitiallyMask) >> 9; }
		inline bool GetMustAdaptToScreen() { return(fWindowFlagBits & kMustAdaptToScreenMask) >> 8; }
		inline bool GetStagger() { return(fWindowFlagBits & kStaggerMask) >> 7; }
		inline bool GetMustForceOnScreen() { return(fWindowFlagBits & kMustForceOnScreenMask) >> 6; }
		inline bool GetVertCenter() { return(fWindowFlagBits & kVertCenterMask) >> 5; }
		inline bool GetHorzCenter() { return(fWindowFlagBits & kHorzCenterMask) >> 4; }
		inline bool GetFloats() { return(fWindowFlagBits & kFloatsMask) >> 3; }
		inline bool GetHideOnSuspend() { return(fWindowFlagBits & kHideOnSuspendMask) >> 2; }
		inline bool GetGenerateActivates() { return(fWindowFlagBits & kGenerateActivatesMask) >> 1; }
		
		inline void SetHasGoAway(unsigned short hasGoAway) { fWindowFlagBits |= hasGoAway << 15; }
		inline void SetResizable(unsigned short resizable) { fWindowFlagBits |= resizable << 14; }
		inline void SetDoFirstClick(unsigned short doFirstClick) { fWindowFlagBits |= doFirstClick << 13; }
		inline void SetFreeOnClosing(unsigned short freeOnClosing) { fWindowFlagBits |= freeOnClosing << 12; }
		inline void SetDisposeOnFree(unsigned short disposeOnFree) { fWindowFlagBits |= disposeOnFree << 11; }
		inline void SetClosesDocument(unsigned short closesDocument) { fWindowFlagBits |= closesDocument << 10; }
		inline void SetOpenInitially(unsigned short openInitially) { fWindowFlagBits |= openInitially << 9; }
		inline void SetMustAdaptToScreen(unsigned short mustAdaptToScreen) { fWindowFlagBits |= mustAdaptToScreen << 8; }
		inline void SetStagger(unsigned short stagger) { fWindowFlagBits |= stagger << 7; }
		inline void SetMustForceOnScreen(unsigned short mustForceOnScreen) { fWindowFlagBits |= mustForceOnScreen << 6; }
		inline void SetVertCenter(unsigned short vertCenter) { fWindowFlagBits |= vertCenter << 5; }
		inline void SetHorzCenter(unsigned short horzCenter) { fWindowFlagBits |= horzCenter << 4; }
		inline void SetFloats(unsigned short floats) { fWindowFlagBits |= floats << 3; }
		inline void SetHideOnSuspend(unsigned short hideOnSuspend) { fWindowFlagBits |= hideOnSuspend << 2; }
		inline void SetGenerateActivates(unsigned short generateActivates) { fWindowFlagBits |= generateActivates << 1; }
};


//----------------------------------------------------------------------------------------
// WMgrTWindow: Used by the window mapper
//----------------------------------------------------------------------------------------
struct TWindowAndWindowRef
{
	inline TWindowAndWindowRef(TWindow* inTWindow, WindowRef inWindowRef) :
	fTWindow(inTWindow),
	fWindowRef(inWindowRef)
	{ }

	TWindow* fTWindow;
	WindowRef fWindowRef;
};

//----------------------------------------------------------------------------------------
// procedure typedefs for parameters
//----------------------------------------------------------------------------------------
typedef CompareResult_AC (*CompareWindowRefType)(WindowRef anItem, void* yourDataPtr);

class EXPORT_MA CWindowRefTable : public CSortedDynamicArray_AC
{
public:
	inline CWindowRefTable() :
	CSortedDynamicArray_AC(kEmptyIndex_AC, sizeof(TWindowAndWindowRef)) { }

	virtual ~CWindowRefTable();
		// Destructor
		
#if qRangeCheck || qDebugMsg
	TWindowAndWindowRef* At(ArrayIndex_AC index) const;
#else
	inline TWindowAndWindowRef* At(ArrayIndex_AC index) const { return reinterpret_cast<TWindowAndWindowRef*>(ComputeAddress(index)); }
#endif

#if qRangeCheck || qDebugMsg
	void AtDelete(ArrayIndex_AC index);
#else
	inline void AtDelete(ArrayIndex_AC index) { DeleteElementsAt(index, 1); }
#endif
		// Deletes via an index.

	void Delete(const WindowRef item);

	virtual ArrayIndex_AC GetEqualItemNo(const WindowRef item) const;
		// Find the first reference to item in the list. If item does not occur, return 0.
		// This is a good as GetIdentityItemNo since the WindowRefs are unique.

	virtual void Insert(const TWindowAndWindowRef& item);
		// Inserts the given item into the list in sorted order.

	virtual TWindowAndWindowRef* DoSearch(CompareWindowRefType TestItem,
								   void* yourDataPtr,
								   ArrayIndex_AC& index) const;

	virtual CompareResult_AC CompareElements(const void* item1,
												 const void* item2) const;
		// Compare TWindowAndWindowRef's.

	virtual CompareResult_AC Compare(const WindowRef element1,
												 const WindowRef element2) const;
		// Compare TWindowAndWindowRef's.
};

//----------------------------------------------------------------------------------
// TWindow: Corresponds to a desktop Window. The outermost of a nested set of views.
//----------------------------------------------------------------------------------
class EXPORT_MA TWindow : public TView, public MScriptableObject
{
	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods 
	//------------------------------------------------------------------------------------
public:
	TWindow(WindowRef itsWMgrWindow = NULL,
					  bool canResize = true,
					  bool canClose = true,
					  bool inDispose = false);
		// Constructor
		
	TWindow(const TWindow& inWindow);
		// Copy Constructor

	virtual ~TWindow();
		// Frees the window if fDisposeOnFree is true, also deletes the window from the
        // document if there is one, otherwise deletes it from the applications list of
        // free windows. 

	virtual void* Clone() const;

	virtual WindowRef GetBehindWindowPtr() const;
		// return the windowPtr in the windowlist behind which this window gets created 

	virtual void DoPostCreate(TDocument* itsDocument);	// Override
		// sets the resize limits
	
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------
	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------
	virtual void ReadFields(CStream_AC* aStream);	// override 
	
	virtual void WriteFields(CStream_AC* aStream) const;	// override 

	//------------------------------------------------------------------------------------
	// Open/Close Methods 
	//------------------------------------------------------------------------------------
	virtual void BeInDocument(TDocument* itsDocument); // override
		// Assigns fDocument itsDocument by calling Inherited and then adds
		// itself to the document's window list

	virtual void Open();
		// Makes the window visible by showing it before calling Inherited::Open. 

	virtual void DoClose(CommandNumber aCommand,
						 bool useAppleEvent = true);
		// Creates and posts a command which will close the window.  Pass a false
		// if the command is not to create an AppleEvent.

	virtual void Close();
		// Calls Inherited::Close and the hides & deactivates the window. 

	void CloseAndFree();
		// Calls and free the window if fFreeOnCompletion. 
		// This method will be removed, please call Close and then if necessary call Free

	virtual void CloseByUser(CommandNumber aCommandNumber);
		// If the window is shown and it can be closed handles the closing. 
		// Returns false if the user cancels closing. 

	virtual void GoAwayByUser(const CViewPoint& theMouse);
		// Responsible for calling CloseByUser. 

	//------------------------------------------------------------------------------------
	// Activation Methods 
	//------------------------------------------------------------------------------------
	virtual void Activate(bool entering);
		// Activate the window. 

	//------------------------------------------------------------------------------------
	// Size/ Location Methods 
	//------------------------------------------------------------------------------------
	virtual void MoveByUser(const CViewPoint& theMouse);
		// Handles the user moving the window by dragging the title bar, calls DragWindow
        // to handle movement. 

	virtual void ResizeByUser(const CViewPoint& theMouse);
		// Handles the user resizing the window by dragging the growbox, calls GrowWindow
        // to handle movement and Resize to resize the window. 

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);
		// Used to resize the window by setting its frame to the parameters
        // passed in. 

	virtual void Zoom(short partCode);
		// Responsible for actually handling the zooming of a window when the user clicks
        // in the zoombox. 

	virtual void ZoomByUser(const CViewPoint& theMouse, short partCode);
		// handles the user clicking in the zoombox to zoom the window, calls Zoom. 

	virtual CRect_AC GetDynamicMoveBounds() const;
		// returns bounds over which the window may be moved. Honors fMoveBounds when non-zero

	virtual CRect_AC GetDynamicResizeLimits() const;
		// returns limits over which the window may be resized. Honors fResizeLimits when non-zero.

	//------------------------------------------------------------------------------------
	// Focusing Methods 
	//------------------------------------------------------------------------------------
	virtual bool Focus();
		// Attempts to set the Focus to the window, returns true if it succeeds, false if
        // not. 

	virtual bool FocusOnSuperView();
		// Attempts to set the Focus to our superview, returning true if it succeeds or
        // false if not. 

	//------------------------------------------------------------------------------------
	// Drawing Methods 
	//------------------------------------------------------------------------------------
	virtual void DrawResizeIcon();
		// Draws the grow icon by calling the toolbox routine DrawGrowIcon. 

	//------------------------------------------------------------------------------------
	// Mouse Handling Methods 
	//------------------------------------------------------------------------------------
	virtual bool HandleMouseDown(const CViewPoint& theMouse,
										   TToolboxEvent* event,
										   CPoint_AC hysteresis,
										   EMouseDownType mouseDownType = kDragOrClick);
		// Handles mouse downs in the window. 

	virtual short GetPartCode(const CViewPoint& theMouse) const;
		// determines what part of the window theMouse is in 

	virtual bool HasPendingUpdate() const;
		// Returns true if there is a pending update event for this window, false if not.

	virtual void BeginUpdate();
		// Encapsulates the Toolbox BeginUpdate. 

	virtual void EndUpdate();
		// Encapsulates the Toolbox EndUpdate. 

	virtual void Update();
		// Handle the update region and drawcontents on this. 

	virtual void DoInvalidateRegion(const RgnHandle badRgn);
		// If a WMgrWindow is available calls InvalRgn 

	virtual void DoValidateRegion(const RgnHandle goodRgn);
		// If a WMgrWindow is available calls ValidRgn 

	//------------------------------------------------------------------------------------
	// Menu Handling Methods 
	//------------------------------------------------------------------------------------
	virtual bool AllowsMenuAccess();
		// By default returns true. 

	virtual void DoSetupMenus();
		// If the window is not modal calls Inherited::DoSetupMenus. 

	virtual void DoMenuCommand(CommandNumber aCommandNumber);
		// Handles closing the window. 

	//------------------------------------------------------------------------------------
	// Target and view management 
	//------------------------------------------------------------------------------------
	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);	// Override

	virtual void SetWindowTarget(TEventHandler* newTarget);
		// Sets the window's fTarget to 'newTarget'. When the window is activated, 
		// newTarget will become the target

	virtual TEventHandler* GetWindowTarget() const;
		// Return the current fTarget of this window 

	virtual void RemovedASubView(TView* theSubView);
		// Fix up the target, if the subview removed was the target and handle droppable.

	virtual void AddedASubView(TView* theSubView);
		// Added to droppable list if droppable. 

	//------------------------------------------------------------------------------------
	// Cursor Handling 
	//------------------------------------------------------------------------------------
	virtual bool HandlesCursor() const; // OVERRIDE
		// Returns true if fHandlesCursor is true, and the window is active, or
        // fDoFirstClick is true 

	virtual bool LetsSubViewsHandleCursor() const; // OVERRIDE

	virtual bool HandlesHelp() const; // OVERRIDE
		// Returns the value of fDoesHandleHelp 

	virtual bool LetsSubViewsHandleHelp() const; // OVERRIDE

	//------------------------------------------------------------------------------------
	// Misc. Methods 
	//------------------------------------------------------------------------------------
	virtual void AboutToLoseControl();
		// called when the application is about to lose control 

	virtual void RegainControl();
		// called when the application is regaining control 

	virtual void Show(bool state, bool redraw);
		// Called to make the window visible or not based on 'state'. 

	virtual bool IsShown() const;
		// Returns true if the window is currently shown. 

	virtual bool IsActive() const;
		// Returns true if the window is currently active. 

	virtual void Select();
		// Called to select the window. 

	virtual void SetTitle(const CStr255_AC& newTitle); 
		// Sets the windows title to 'newTitle'. 

	virtual CStr255_AC GetTitle() const;
	inline void GetTitle(CStr255_AC& itsTitle) const { itsTitle = GetTitle(); }
		// Gets the current title of the window. 

	virtual void SetTitleForDoc(const CStr255_AC& newDocTitle);// %%% AMB
		// Called when the documents name is known or changed 

	virtual void AdaptToScreen();
		// Resizes the window to fit the screen on which it is being displayed. 

	virtual void ForceOnScreen();
		// ForceOnScreen gaurantees that some minimal drag area is accessible to the user,
        // to be dragged to the desired location. 

	virtual void Center(bool horizontally,
							   bool vertically,
							   bool forDialog);
		// Centers the window on the screen either horizontally, vertically or both. 

	virtual void SimpleStagger(CPoint_AC delta,
								   short& counter);
		// Handles the staggering of windows as they are opened. 

	virtual GrafPtr GetGrafPort() const;
		// Returns the windows GrafPtr. 

	virtual GDHandle GetMaxIntersectedDevice(CRect_AC& screenRect) const;
		// Returns the screenRect of the most intersected device & its GDHandle, or if CQD
        // isn't available it returns GetGrayRgn intersected with screenbits.bounds and
        // NULL for the GDHandle. 

	virtual TWindow* GetWindow() const;
		// Returns this. 

	virtual void SetResizeLimits(CPoint_AC itsMinSize, CPoint_AC itsMaxSize);
		// Used to set the limits to which the window can be resized. 

	virtual bool RectInDrag(const CRect_AC& whichRect) const;
		// Returns true if any of the corner points of whichRect are draggable. 

	inline bool IsHiddenOnSuspend() const { return fHideOnSuspend; }
		// Returns true if this window gets hidden when the application is suspended. 

	virtual bool IsDismissed() const;
		// Returns true if this dialog has been dismissed. 

	virtual void Dismiss(IDType dismisser, bool validate);
		// dismiss the dialog that is currently being posed modally 

	virtual bool BuildWindowRegions(bool build) const;
		// IF build=kBuild THEN ensures that the window regions are valid; sets
        // fContentRegionInset, fContentDifference. Cleans up if necessary, when done with window
        // regions. Returns the old state of the window regions. 

	virtual TDialogBehavior* GetDialogBehavior() const;
		// Returns the windows dialog behavior, if any.
		
	virtual void SetModality(bool modal);
		// Creates a TDialogBehavior object if necessary, and sets its modality.

	virtual void SetDialogItems(IDType  itsDefaultItem,
									   IDType  itsCancelItem);
		// Creates a TDialogBehavior object if necessary, and sets its attributes.
		
	virtual IDType PoseModally();
		// poses this window in a modal state.

	virtual bool IsModal() const;
		// Returns true if this window is modal.

	virtual bool IsInModalState() const;
		// Returns true if this window is currently in a modal state.

	virtual CPoint_AC GetLocationAdjustment() const;

	virtual CViewRect GetStandardStateFrame(const CViewRect& boundingRect) const;

	virtual CViewRect GetUserStateFrame(const CViewRect& boundingRect) const;

	virtual bool CountAsOpen();
		// Returns true if this windows counts as an 'open' window.

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline WindowRef GetWindowRef() const { return fWMgrWindow; }
	inline bool GetFloats() const { return fFloats; }
	inline bool GetFreeOnClosing() const { return fFreeOnClosing; }
	inline bool GetOpenInitially() const { return fOpenInitially; }
	inline CRect_AC GetResizeLimits() const { return fResizeLimits; }
	inline IDType GetTargetID() const { return fTargetID; }
	inline bool GetDoFirstClick() const { return fDoFirstClick; }
	inline bool GetIsResizable() const { return fIsResizable; }
	inline bool GetClosesDocument() const { return fClosesDocument; }
	inline CPoint_AC GetContentRegionInset() const { return fContentRegionInset; }
	inline bool GetUpdating() const { return fUpdating; }
	inline CPoint_AC GetContentDifference() const { return fContentDifference; }
	inline short GetStrListID() const { return fStrListID; }
	inline short GetStrListIndex() const { return fIndex; }

	inline void SetHideOnSuspend(bool inHideOnSuspend) { fHideOnSuspend = inHideOnSuspend; }
	inline void SetClosesDocument(bool inClosesDocument) { fClosesDocument = inClosesDocument; }
	inline void SetFreeOnClosing(bool inFreeOnClosing) { fFreeOnClosing = inFreeOnClosing; }
	inline void SetDoFirstClick(bool inDoFirstClick) { fDoFirstClick = inDoFirstClick; }
	inline void SetMustForceOnScreen(bool inMustForceOnScreen) { fMustForceOnScreen = inMustForceOnScreen; }
	inline void SetForcedOnScreen(bool inForcedOnScreen) { fForcedOnScreen = inForcedOnScreen; }
	inline void SetResizeLimits(const CRect_AC& inResizeLimits) { fResizeLimits = inResizeLimits; }
	inline void SetIsResizable(bool isResizable) { fIsResizable = isResizable; }
	inline void SetStrListID(short strListID) { fStrListID = strListID; }
	inline void SetStrListIndex(short strListIndex) { fIndex = strListIndex; }
	
	//------------------------------------------------------------------------------------
	// Scripting Support 
	//------------------------------------------------------------------------------------
	virtual DescType GetSpecifierForm(); // override

	virtual long CountContainedObjects(DescType desiredType);

	virtual TCommandHandler* GetCommandContext(const CommandNumber aCommandNumber) const;

	virtual MScriptableObject* GetContainedObject(DescType desiredType,
											  DescType selectionForm,
											  const CAEDesc& selectionData);

	virtual bool GetObjectProperty(CAEDesc& thePropertyValue,
									DescType whichProperty,
									const CAEDesc& desiredType);

	virtual void GetSetPropertyInfo(DescType whichProperty,
								CommandNumber& cmdNum,
								bool& canUndo,
								bool& causesChange,
								TCommandHandler* &theContext);
								
	virtual void SetObjectProperty(const CAEDesc& thePropertyValue,
									DescType whichProperty);

	virtual void DoAEClose(TAppleEvent* message,
						   TAppleEvent* reply);

	virtual void DoAEMove(TAppleEvent* message,
						  TAppleEvent* reply);

	virtual void DoAEPrint(TAppleEvent* message,
						  TAppleEvent* reply);

	virtual void DoScriptCommand(CommandNumber	aCommandNumber,
								TAppleEvent* 	message,
								TAppleEvent* 	reply);
	
	void SetScriptingPrintHandler(TPrintHandler* printHandler);
	
	void ReleaseScriptingPrintHandler(TPrintHandler* printHandler);
	
#if qAttachable
	virtual bool HandleOSAEvent(CommandNumber	aCommandNumber,
									TAppleEvent* 	message,
									TAppleEvent* 	reply);
#endif
		
#if qDrag	
	//------------------------------------------------------------------------------------
	// Drag Support 
	//------------------------------------------------------------------------------------
	void RegisterDroppableView(TView* theView);
		// Droppable subviews register with the window.  A list of these views
		// is maintained in fDroppableViewList

	void UnregisterDroppableView(TView* theView);
		// Unregister a view when it is destroyed or loses its ability to receive drops
		
	void RegisterIfDroppableView(TView* theView);
		// See if theView is droppable and register it if it is then scan its
		// subviews recursively and register them if they are droppable.

	void UnregisterIfDroppableView(TView* theView);
		// See if theView is droppable and unregister it if it is then scan its
		// subviews recursively and unregister them if they are droppable.
		
	virtual TView* MouseToDropTarget(CDragItemIterator&	dragItemIterator,
									const CPoint_AC& 		globalMouse,
									CViewPoint&				localMouse);
		// Returns the TView that would accept the current drag. Returns
		// NULL if no eligible target exists.  If a target is located, localMouse is
		// set to the mouse location in the target view's coordinate system
		
	virtual void DragEnteredWindow(CDragItemIterator& dragItemIterator);
		// Perform any pre-flight necessary to handle tracking a drag
		
	virtual void DragLeftWindow();
		// Perform any clean-up after a drag has exited
		
#endif // qDrag

//----------------------------------------------------------------------------------------
// static functions
//----------------------------------------------------------------------------------------
public:

	static RgnHandle GetUpdateRegion(WindowPtr theWindow);
		// Return the correct updateRgn even if BeginUpdate has been called
	
	static RgnHandle GetVisRegion(GrafPtr theGrafPtr);
		// Return the correct visRgn even if BeginUpdate has been called
	
	static Handle GetAndLoadWDefProc(Handle windowDefProc);

	static void HighlightAndActivateWindow(WindowRef theWindow, bool activate);

	static TWindow* WMgrToWindow(const WindowRef aWMgrWindow);
		// Returns the window object that represents the given Window Manager window, or
		// NULL if there is no window object.
	
	static void DeleteWindow(TWindow* aWindow);
		// called when you have a new TWindow to track
	
	static void RegisterWindow(TWindow* aWindow);
		// called when freeing a TWindow
	
	static bool IsGhostWindow(const WindowRef window);

	static bool IsDocumentWindow(const WindowRef aWindow);
	
	static bool IsDialog(const WindowRef window);
	
	static bool IsFloatWindow(const WindowRef window);
	
	static bool IsVisibleFloatWindow(const WindowRef window);
	
	static bool IsSystemWindow(const WindowRef window);
	
	static bool IsModalWindow(const WindowRef window);
	
	static WindowRef GetLastFloatingWindowPtr();
	
	static WindowRef GetFirstFloatingWindowPtr();
	
	static void MAActivateWindow(WindowRef theWindow);
	
	static void MADeactivateWindow(WindowRef theWindow);
	
	static void MADragWindow(WindowRef windowToDrag, CPoint_AC startPoint, const CRect_AC& draggingBounds);
	
	static void MASelectToolboxWindow(WindowRef windowToSelect);
		// "Selects" windowToSelect by calling the toolbox routine SelectWindow. When any
		// window is selected, it goes through here. We never call SelectWindow directly.
	
	static void MAShowWindow(WindowRef windowToShow);
	
	static void MAHideWindow(WindowRef windowToHide);
	
	static WindowRef MAFrontWindow();
	
	static WindowRef FreeIfWMgrWindow(WindowRef w,
								bool inDispose = false);
	    // Does nothing if w is NULL.
	    // Returns NULL for convenient assignment back to the reference passed in.
		// TRUE to "DisposeWindow" the window's storage, false to "delete" it

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	CRect_AC fMoveBounds;						// bounds over which the window may be
                                                // moved. Any value that is zero will be
												// dynamically calculated. 

	CRect_AC fResizeLimits;						// limits over which the window may be
                                                // resized. Any value that is zero will be
												// dynamically calculated.  

	CPoint_AC fContentRegionInset;				// topleft inset of cont rgn in struc rgn

	CPoint_AC fContentDifference;				// total amount the content is less than
                                                // and offset into the structure (Accounts
                                                // for title bar, etc.) 
	
	TPrintHandler* fScriptingPrintHandler;		// Print handler to invoke when the window
												// receives a print apple event
																																																							
	TEventHandler* fTarget;						// when this window is activated; the
                                                // TEventHandler to SetTarget on 

	WindowRef fWMgrWindow;						// the window manager window Ptr 

	IDType fTargetID;							// corresponding ID to fTarget 

	short fProcID;								// the proc ID of the window 

	short fPreDocname;							// amount of window title that precedes
                                                // the document name 

	short fConstTitle;							// amount of the title name that is
                                                // constant 

	short fStrListID;							// window's title:	STR# rsrc id

	short fIndex;								// index into STR#

	bool fDisposeOnFree;						// What should happen to the WMgrWindow when this window is deleted. 
												// TRUE = DisposeWindow(w), false = delete w

	bool fUpdating;								// True when TWindow::Update has control.
                                                // Changes behaviour of TWindow::Focus to
                                                // clip TO the update region instead of
                                                // clipping OUT the update region 

	bool fIsActive;								// is this window active 

	bool fIsResizable;							// is this window resizeable 

	bool fIsClosable;							// is this window closable 

	bool fFreeOnClosing;						// should this window free itself when it
                                                // closes 

	bool fClosesDocument;						// does this window close the document 

	bool fOpenInitially;						// should this window be opened when its
                                                // document is asked to show its windows 

	bool fDoFirstClick;							// should a click in this window not only
                                                // select it if it is inactive but also be
                                                // sent to its contents 

	bool fFloats;								// does this window float above
                                                // non-floating windows

	bool fHideOnSuspend;						// does this window get hid on suspend? 

	bool fWasHiddenOnSuspend;					// True if this window was hid on suspend

	bool fGenerateActivates;					// a flag indicating whether or not
                                                // TWindow::Show should generate the
                                                // necessary activate events 

	bool fMustAdapt;							// does this window require adaption to
                                                // the screen 

	bool fMustHorzCenter;						// does this window require centering
                                                // horizontally 

	bool fMustVertCenter;						// does this window require centering
                                                // vertically 

	bool fMustStagger;							// does this window require staggering 

	bool fMustForceOnScreen;					// does this window require forcing on the
                                                // screen 

	bool fAdapted;								// has this window's size ever been
                                                // adapted to the screen 

	bool fHorzCentered;							// has this window been centered
                                                // horizontally 

	bool fVertCentered;							// has this window been centered
                                                // vertically 

	bool fStaggered;							// has this window been staggered 

	bool fForcedOnScreen;						// has this window been forced on the
                                                // screen 

#if qDrag	
	//------------------------------------------------------------------------------------
	// Drag and Drop Data
	//------------------------------------------------------------------------------------
	
	EDragCallbackStatus fDragCallbackStatus;	// enumerated type indicating whether or
												// not drag manager callbacks have been
												// installed for the window.

	CList_AC fDroppableViewList;				// list of subviews that are droppable
												
	CDynamicArray_AC fDropStatusCache;			// cache that maps to fDroppableViewList
												// that indicates whether views can or
												// can't receive the current drag

#endif // qDrag
												
//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
public:
	static short gStandardWindowStaggerCount;	// Used to stagger windows created from templates 

protected:
	static bool pInBeginUpdate;
	static GrafPtr pInBeginUpdateWindowPtr;
	
	static RgnHandle pSavedUpdateRegion;
	static RgnHandle pSavedVisRegion;
	static CPoint_AC pSavedVisRegionOriginalOffset;

	static CWindowRefTable fgWindows;
};

//----------------------------------------------------------------------------------------
// TCloseWindowCommand: Tells the application to close a window
//----------------------------------------------------------------------------------------

class EXPORT_MA TCloseWindowCommand : public TCommand
{
public:
	TCloseWindowCommand(CommandNumber itsCommandNumber = cNoCommand,
							 TWindow* itsWindow = NULL);

	TCloseWindowCommand(TWindow* itsWindow,
							 TAppleEvent* message,
							 TAppleEvent* reply);
		// Constructor

	virtual ~TCloseWindowCommand();
		// Destructor
		
	virtual TAppleEvent* MakeAppleEvent();

	virtual void DoIt();
		// tell the application to close a window. 
//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
public:
	TWindow* fWindow;	// The window to close

protected:
	TAppleEvent* fMessage;
	TAppleEvent* fReply;
};

#endif


