//--------------------------------------------------------------------------------------------------
//	File:			Schedule.r
//	Description:	Resource File for the Schedule Sample Program.
//	Copyright 1991-1995 by Apple Computer, Inc.
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Include Files
//--------------------------------------------------------------------------------------------------

// Standard Includes

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

// Include conditional macros first, so that things like System7-style dialogs are used.
#ifdef MACAPP_R_13
#ifndef __ConditionalMacros_AC__
#include "ConditionalMacros_AC.h"
#endif
#else
#ifndef __MACONDITIONALMACROS__
#include "MAConditionalMacros.h"
#endif
#endif

#ifndef __TYPES.R__
#include "Types.r"
#endif

#ifndef __SYSTYPES.R__
#include "SysTypes.r"
#endif

#ifndef __BalloonTypes__
#include "BalloonTypes.r"
#endif

#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif


// Include MacApp resources
#ifdef MACAPP_R_13
include "MacApp.rsrc" not 'cfrg';

// Includes for Building Blocks

include "Printing.rsrc" not 'ckid';
include "Dialog.rsrc" 	not 'ckid';
include "Editions.rsrc" not 'ckid';

#if qDebug
include "Debug.rsrc" not 'ckid';
#endif

#else
#include "MacApp.r"

// Includes for Building Blocks

#include "Printing.r"
#include "Dialog.r"
#include "Editions.r"

#if 1	// n.b., we don't have a Rez prefix file for the 3.1.5 build; 
		// we only support qDebug for this example
#include "Debug.r"
#endif

#endif

//--------------------------------------------------------------------------------------------------

// Include Code Resources

#if qPowerPC

#include "CodeFragmentTypes.r"

resource 'cfrg' (0) {
   {
		kPowerPC,
		kFullLib,
		kNoVersionNum,
		kNoVersionNum,
		0,
		0,
		kIsApp,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		"Schedule"
   }
};

#else
	#ifndef MW_Rez
		include $$Shell("ObjApp")$$Shell("XAppName") 'CODE';
		#if qModelCFM
			include $$Shell("ObjApp")$$Shell("XAppName") 'cfrg';
			include $$Shell("ObjApp")$$Shell("XAppName") 'rseg';
		#endif
	#endif
#endif

// Include custom Icons
include "ScheduleIcons.rsrc" not 'ckid';

// Include custom printing resources
#ifdef MW_Rez
#include "SchedulePrint.r"
#else
include $$Shell("ObjApp")"SchedulePrint.rsrc";
#endif

//--------------------------------------------------------------------------------------------------
// Constants
//--------------------------------------------------------------------------------------------------

// Finder Information
// ------------------

#define kApplicationName				$$Shell("XAppName")
#define kSignature						'SDUL'				// Application signature
#define kFileType						'SKED'				// Data file type
#define kStationery						'sKED'				// Stationery file type

// Bundle Constants
// ------------------

#define kBundleID						128
#define kApplicationID					128
#define kDocumentID						129
#define kStationeryID					130

// Constants for Menu IDs
// ----------------------

#define mAppointments					4
#define mWindows						5

// Command Numbers For Menu Items
// ------------------------------

#define cAddAppointment					1001
#define cChangeAppointment				1002
#define cDeleteAppointment				1003
#define cDragAppointment				1004
#define cStretchAppointment				1005

#define cLookupAppointment				1101
#define cLaunchLookup					1102

// AE Identifiers
// ------------------------------
#define kAEScheduleSuiteID				'SKDL'
#define kAELookupID						'LKUP'

#define kScheduleAEEvents				1001

// STR# resources
// --------------

#define kScheduleBuzz					1011
#define kLookupBuzz						1012

// Text styles
// --------------

#define kMonaco9TextStyle				1002

// 'View' resources
// --------------

#define kScheduleWindowID				1101
#define kScheduleClipViewID				1102

#define phAboutBox						1201
#define phGetDescription				1202
#define phLookupDialog					1203

#define kScheduleID						'SKED'
#define kSubscriberID					'SUBS'
#define kSubscriberDayID				'SUBD'

// Memory Management Resources
// ---------------------------

#define kScheduleSeg					1000
#define kScheduleMem					1000
#define kScheduleRes					1000

// Icons
// ---------------------------

#define kAboutIcon						1001

// Cursors
// ---------------------------

#define kScheduleCrossCursor			1001
#define kSubscriberCrossCursor			1002

// Patterns
// ---------------------------

#define kBlackPattern					1001

// Pictures
// ---------------------------

#define kUpDownArrow					1001
#define kLeftRightArrow					1002

// Balloon Help
// ---------------------------

#define kHelpID							1001

// Custom error result
// -------------------

#define errAppointmentNotAvailable		-26001

// error tables
// ------------

#define errAppReason					1128
#define errAppRecovery					1129
#define errAppOperation					1130

// error strings
// -------------
#define errAppReasons					2101
#define errAppRecoveries				2111
#define errAppOperations				2121

//--------------------------------------------------------------------------------------------------
// Menu Bars
//--------------------------------------------------------------------------------------------------

resource 'MBAR' (kMBarDisplayed, purgeable) {
	{ mApple; mFile; mEditionMgrEdit; mAppointments, mWindows }
};

//--------------------------------------------------------------------------------------------------
// Menus
//--------------------------------------------------------------------------------------------------

#ifdef MW_Rez

resource 'CMNU' (mApple,
#if qNames
"mApple",
#endif
purgeable) {
	mApple,
	textMenuProc,
	0x7FFFFFFD,
	enabled,
	apple,
	{
	"About ^0", noIcon, noKey, noMark, plain, cAboutApp;		// The ^0 will be dynamically replaced with CurApName
	"-",		 noIcon, noKey, noMark, plain, nocommand
	}
};

resource 'CMNU' (mFile,
#if qNames
"mFile",
#endif
purgeable) {
	mFile,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"File",
	{
	"New",				noIcon, "N",	noMark, plain, cNew;
	"Open",			noIcon, "O",	noMark, plain, cOpen;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Close",			noIcon, "W",	noMark, plain, cClose;
	"Save",				noIcon, "S",	noMark, plain, cSave;
	"Save As",			noIcon, noKey,	noMark, plain, cSaveAs;
	"Revert",			noIcon, noKey,	noMark, plain, cRevert;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Page Setup",		noIcon, noKey,	noMark, plain, cPageSetup;
	"Print One",		noIcon, noKey,	noMark, plain, cPrintOne;
	"Print",			noIcon, "P",	noMark, plain, cPrint;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Quit",				noIcon, "Q",	noMark, plain, cQuit
	}
};

resource 'CMNU' (mEditionMgrEdit,
#if qNames
"mEditionMgrEdit",
#endif
purgeable) {
	mEditionMgrEdit,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Edit",
	{
	"Undo",				noIcon, "Z",	noMark, plain, cUndo;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Cut",				noIcon, "X",	noMark, plain, cCut;
	"Copy",				noIcon, "C",	noMark, plain, cCopy;
	"Paste",			noIcon, "V",	noMark, plain, cPaste;
	"Clear",			noIcon, noKey,	noMark, plain, cClear;
	"Select All",		noIcon, "A",	noMark, plain, cSelectAll;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Create Publisher",noIcon, noKey,	noMark, plain, cCreatePublisher;
	"Subscribe to",	noIcon, noKey,	noMark, plain, cSubscribeTo;
	"Publisher Options",noIcon, noKey,	noMark, plain, cOptions;
	"Show Borders",		noIcon, noKey,	noMark, plain, cBorders;
	"Stop All Editions",noIcon, noKey,	noMark, plain, cStopAllEditions;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Show Clipboard",	noIcon, noKey,	noMark, plain, cShowClipboard
	}
};

#else
include "Defaults.rsrc"  'CMNU' (mApple);
include "Defaults.rsrc"  'CMNU' (mFile);
include "Defaults.rsrc"  'CMNU' (mEditionMgrEdit);
#endif

resource 'CMNU' (mAppointments,
#if qNames
"mAppointments",
#endif
nonpurgeable
) {
	mAppointments,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Appointments",
	{
	"Change Appointment",	noIcon, "J", noMark, plain, cChangeAppointment;
	"-",					noIcon, noKey,	noMark, plain, nocommand;
	"Open Lookup Window",	noIcon, "L", noMark, plain, cLaunchLookup;
	"Lookup Appointment",	noIcon, "F", noMark, plain, cLookupAppointment;
	}
};

resource 'CMNU' (mWindows,
#if qNames
"mWindows",
#endif
nonpurgeable
) {
	mWindows,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Windows",
	{
	}
};

#define cCancelPublisher	606		// MacApp doesn't define these
#define cCancelSubscriber	607

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
nonpurgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
		"Cancel Publisher",		noIcon,	noKey,	noMark,	plain, cCancelPublisher;
		"Cancel Subscriber",	noIcon,	noKey,	noMark,	plain, cCancelSubscriber;
		"Add Appointment",		noIcon,	noKey,	noMark,	plain, cAddAppointment;
		"Drag Appointment",		noIcon,	noKey,	noMark,	plain, cDragAppointment;
		"Resize Appointment",	noIcon,	noKey,	noMark,	plain, cStretchAppointment;
	}
};

//--------------------------------------------------------------------------------------------------
// Balloon Help Resources for Menus
//--------------------------------------------------------------------------------------------------

#ifdef MW_Rez

resource 'hmnu' (mApple) {
	HelpMgrVersion,
	hmDefaultOptions,
	0,
	0,
	HMSkipItem { },
	{
		HMStringResItem {				// Menu title messages 
			0,0,		//  Normal 
			0,0,		//  Grayed 
			0,0,		//  Checked 
			0,0,		//  Other 
		},
		HMStringResItem {				// About YourAppName 
			kStandardHelpStrings,1,
			kStandardHelpStrings,2,
			kStandardHelpStrings,3,
			-1,0
		}
	}
};

resource 'hmnu' (mFile) {
	HelpMgrVersion,
	hmDefaultOptions,
	0,
	0,
	HMSkipItem { },
	{

		HMStringResItem {				// File 
			kStandardHelpStrings,32,
			kStandardHelpStrings,33,
			-1,0,
			-1,0
		},
		HMStringResItem {				// New 
			kStandardHelpStrings,34,
			kStandardHelpStrings,35,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Open 
			kStandardHelpStrings,36,
			kStandardHelpStrings,37,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// --------
		HMStringResItem {				// Close 
			kStandardHelpStrings,38,
			kStandardHelpStrings,39,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Save 
			kStandardHelpStrings,40,
			kStandardHelpStrings,41,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Save As 
			kStandardHelpStrings,42,
			kStandardHelpStrings,43,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Revert 
			kStandardHelpStrings,52,
			kStandardHelpStrings,53,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// --------
		HMStringResItem {				// Page Setup 
			kStandardHelpStrings,44,
			kStandardHelpStrings,45,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Print One 
			kStandardHelpStrings,50,
			kStandardHelpStrings,51,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Print 
			kStandardHelpStrings,46,
			kStandardHelpStrings,47,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// --------
		HMStringResItem {				// Quit 
			kStandardHelpStrings,48,
			kStandardHelpStrings,49,
			-1,0,
			-1,0
		}
	}
};

resource 'hmnu' (mEditionMgrEdit) {
	HelpMgrVersion,
	hmDefaultOptions,
	0,
	0,
	HMSkipItem { },
	{

		HMStringResItem {				// Edit 
			kStandardHelpStrings,4,
			kStandardHelpStrings,5,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Undo
			kStandardHelpStrings,6,
			kStandardHelpStrings,7,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// --------
		HMStringResItem {				// Cut 
			kStandardHelpStrings,8,
			kStandardHelpStrings,9,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Copy 
			kStandardHelpStrings,10,
			kStandardHelpStrings,11,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Paste 
			kStandardHelpStrings,12,
			kStandardHelpStrings,13,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Clear 
			kStandardHelpStrings,14,
			kStandardHelpStrings,15,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Select All 
			kStandardHelpStrings,16,
			kStandardHelpStrings,17,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// --------
		HMStringResItem {				// Create Publisher 
			kStandardHelpStrings,24,
			kStandardHelpStrings,25,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Subscribe to 
			kStandardHelpStrings,26,
			kStandardHelpStrings,27,
			-1,0,
			-1,0
		},
		HMCompareItem {
		"Publisher Options",
		HMStringResItem {				// a) Publisher Options 
			kStandardHelpStrings,28,
			kStandardHelpStrings,29,
			-1,0,
			-1,0
		}
		},
		HMCompareItem {
		"Subscriber Options",
		HMStringResItem {				// b) Subscriber Options 
			kStandardHelpStrings,30,
			kStandardHelpStrings,31,
			-1,0,
			-1,0
		}
		},
		HMSkipItem { },					// Hide Border
		HMSkipItem { },					// Stop All Editions
		HMSkipItem { },					// --------
		HMCompareItem {					// a) Hide Clipboard
		"Hide Clipboard",
			HMStringResItem {				   
					kStandardHelpStrings,18,
					kStandardHelpStrings,19,
					kStandardHelpStrings,20,
					-1,0
			}
		},
		HMCompareItem {					// b) Show Clipboard
		"Show Clipboard",
			HMStringResItem {				   
					kStandardHelpStrings,21,
					kStandardHelpStrings,22,
					kStandardHelpStrings,23,
					-1,0
			}
		},
	}
};
#define TARGET_APPLICATION_NAME "Schedule"
#include "HelpStrings.r"
#else
include "Defaults.rsrc"  'hmnu' (mApple);
include "Defaults.rsrc"  'hmnu' (mFile);
include "Defaults.rsrc"  'hmnu' (mEditionMgrEdit);
#include $$Shell("MALibraries")"HelpStrings.r";	// cannot be pre-compiled because it accesses shell variable XAppName.
#endif

resource 'hmnu' (mAppointments)
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, // balloon definition function
	0, // variation code

	// Missing items
	HMSkipItem
	{
	},

	// Title and items
	{
		// Title
		HMStringItem
		{
			"Appointments menu\$0D\$0DThis menu concerns appointments.",
			"",
			"Menu not available; respond to the dialog.",
			"Menu not available; respond to the dialog.",
		},
		// Change Item
		HMStringItem
		{
			"Use this item to change the description of the selected appointment.",
			"Item is disabled because no single appointment is selected.",
			"Item disabled; respond to dialog.",
			"Item disabled; respond to dialog.",
		},
		// separator item
		HMSkipItem
		{
		},
		// Open Lookup window
		HMStringItem
		{
			"This item opens a window to use to lookup appointments.",
			"",
			"Item disabled; respond to dialog.",
			"Item disabled; respond to dialog.",
		},

		// Find appointment
		HMStringItem
		{
			"Use this item to search for an opening in the schedule.",
			"To activate this item, open a lookup window.",
			"Item disabled; respond to dialog.",
			"Item disabled; respond to dialog.",
		},
	}
};

resource 'hmnu' (mWindows)
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, // balloon definition function
	0, // variation code

	// Missing items
	HMStringItem
	{
		"Select this item to bring this window to the front.",
		"",
		"This is the active window.",
		"",
	},

	// Title and items
	{
		// Title
		HMStringItem
		{
			"Windows menu\$0D\$0DThis is a list of document windows.",
			"Windows menu\$0D\$0DOpen a document, and its window will appear in this menu.",
			"Document windows are not available; respond to the dialog box.",
			"Document windows are not available; respond to the dialog box.",
		},
	}
};

//--------------------------------------------------------------------------------------------------
// Text Styles
//--------------------------------------------------------------------------------------------------

resource 'TxSt' (kMonaco9TextStyle, purgeable) {tsPlain, 9, black, "Monaco"};

//--------------------------------------------------------------------------------------------------
// Views
//--------------------------------------------------------------------------------------------------

#define kDayViewHeight			48
#define kTimeViewWidth			64

#define kBorderHeight			2
#define kBorderWidth			2

#define kRowHeight				16
#define kColumnWidth			100

#define kWindowWidth			kTimeViewWidth + kBorderWidth  + (4  * kColumnWidth) + kSBarSizeMinus1
#define kWindowHeight			kDayViewHeight + kBorderHeight + (16 * kRowHeight)   + kSBarSizeMinus1

#define kPrimaryHeight			kWindowHeight - kDayViewHeight - kBorderHeight - kSBarSizeMinus1
#define kPrimaryWidth			kWindowWidth  - kTimeViewWidth - kBorderWidth  - kSBarSizeMinus1

#define kScheduleHeight			(kNumberOfTimes * kRowHeight)
#define kScheduleWidth			(kNumberOfDays * kColumnWidth)

#define kTotalHeight			kDayViewHeight + kBorderHeight + kScheduleHeight
#define kTotalWidth				kTimeViewWidth + kBorderWidth + kScheduleWidth

#define kRowInset				2
#define kColumnInset			8

#define kNumberOfDays			7
#define kNumberOfTimes			24 * 4

#define kSubscriberNameHeight	28

resource 'STR#' (kScheduleWindowID,
#if qNames
"kScheduleWindowID",
#endif
purgeable) {
	{
	"";			// window title
	"^SubscriberName";
	}
};

resource 'View' (kScheduleWindowID,
#if qNames
"kScheduleWindowID",
#endif
purgeable) { MAThreeOh,
{

ViewSignatureAndClassname
	{'wind', 1156, "TMenuedWindow", 'WIND', enabled, noIdle, {}, MAThreeOh,
	{50, 40}, {kWindowHeight, kWindowWidth}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget,
	handlesCursor, letsSubViewsHandleCursor, noCursorID,
	handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
	NoDrawingEnvironment {},
	AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
		AdornFirst, AdornerLocalObject {EraseAdorner},
		DrawView, AdornerLocalObject {DrawAdorner},
		AdornLast, AdornerLocalObject {ResizeIconAdorner}}}, emptyUserArea,
	Window {zoomDocProc, kScheduleID, goAwayBox, resizable, ignoreFirstClick,
		freeOnClosing, disposeOnFree, closesDocument, openWithDocument, dontAdaptToScreen, stagger, forceOnScreen,
		dontCenter, doesntFloat, doesntHideOnSuspend, generateActivates, filler, kScheduleWindowID, 1},
		7},

	ViewSignatureAndClassname
		{'scrl', 267, "TPrimaryScroller", 'SCRP', enabled, noIdle, {}, MAThreeOh,
		{kDayViewHeight + kBorderHeight, kTimeViewWidth + kBorderWidth},
		{kPrimaryHeight, kPrimaryWidth},
		sizeRelSuperView, sizeRelSuperView, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		Scroller {'vCRP', 'hCRP', {0, 0}, {kRowHeight, kColumnWidth}, noVertConstrain, noHorzConstrain,
		{-(kDayViewHeight + kBorderHeight), -( kTimeViewWidth + kBorderWidth), 0, 0},
		respondsToFKeys}, 3},

		ViewSignatureAndClassname
			{'txtg', 140, "TScheduleView", kScheduleID, enabled, noIdle,
			{
				BehaviorSignatureAndClassname {kNoIdentifier, "TSectionBehavior", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TAppointmentChangeLauncher", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TAppointmentStretchLauncher", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TAppointmentSelector", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TAppointmentDragLauncher", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TAppointmentSketchLauncher", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TScheduleEditLauncher", Behavior { enabled, kMaxIdleTime }},
			},
			MAThreeOh,
			{0, 0}, {kScheduleHeight, kScheduleWidth},
			sizeFixed, sizeFixed, shown, wantsToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, kScheduleCrossCursor,
			handlesHelp, letsSubViewsHandleHelp, kHelpID, 1,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {SelectionAdorner},
				DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
			TextGridView {kNumberOfTimes, kNumberOfDays, kRowHeight, kColumnWidth, kRowInset, kColumnInset, AdornRows, AdornCols, multipleSelection, kApplFont9TextStyle, dontPreferOutline},
			NoSubviews},

		ViewSignatureAndClassname
			{'patn', 91, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{0, kScheduleWidth}, {kScheduleHeight, kBorderWidth},
			sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Pattern {mPatternHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, dontPreferColor, kBlackPattern},
			NoSubviews},

		ViewSignatureAndClassname
			{'txtg', 140, "TSubscriberView", kSubscriberID, enabled, noIdle,
			{
				BehaviorSignatureAndClassname {kNoIdentifier, "TSectionBehavior", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TAppointmentSelector", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TSubscriberSketchLauncher", Behavior { enabled, kMaxIdleTime }},
				BehaviorSignatureAndClassname {kNoIdentifier, "TSubscriberEditLauncher", Behavior { enabled, kMaxIdleTime }},
			},
			MAThreeOh,
			{0, kScheduleWidth + kBorderWidth}, {kScheduleHeight, kColumnWidth},
			sizeFixed, sizeFixed, shown, wantsToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, kSubscriberCrossCursor,
			handlesHelp, letsSubViewsHandleHelp, kHelpID, 2,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {SelectionAdorner},
				DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
			TextGridView {kNumberOfTimes, 1, kRowHeight, kColumnWidth, kRowInset, kColumnInset, AdornRows, AdornCols, multipleSelection, kApplFont9TextStyle, dontPreferOutline},
			NoSubviews},

	ViewSignatureAndClassname
		{'scrl', 264, "TSecondaryScroller", 'SCRD', notEnabled, noIdle, {}, MAThreeOh,
		{0, kTimeViewWidth + kBorderWidth}, {kDayViewHeight, kPrimaryWidth},
		sizeFixed, sizeRelSuperView, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		Scroller {noID, noID, {0, 0}, {kRowHeight, kColumnWidth}, noVertConstrain, noHorzConstrain,
		noInset, respondsToFKeys},
		3},

		ViewSignatureAndClassname
			{'txtg', 135, "TDayView", 'DAYS', enabled, noIdle, {}, MAThreeOh,
			{0, 0}, {kDayViewHeight, kScheduleWidth},
			sizeFixed, sizeFixed, shown, wantsToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, kHelpID, 3,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {SelectionAdorner},
				DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
			TextGridView {1, kNumberOfDays, kDayViewHeight, kColumnWidth, 8, 8, dontAdornRows, AdornCols, singleSelection, kSystemFontTextStyle, dontPreferOutline},
			NoSubviews},

		ViewSignatureAndClassname
			{'patn', 91, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{0, kScheduleWidth}, {kDayViewHeight, kBorderWidth},
			sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Pattern {mPatternHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, dontPreferColor, kBlackPattern},
			NoSubviews},

		ViewSignatureAndClassname
			{'txtg', 135, "TSubscriberDayView", kSubscriberDayID, enabled, noIdle, {}, MAThreeOh,
			{0, kScheduleWidth + kBorderWidth},
			{kDayViewHeight, kColumnWidth},
			sizeFixed, sizeFixed,
			shown, wantsToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, kHelpID, 4,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {SelectionAdorner},
				DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
			TextGridView {1, 1, kDayViewHeight, kColumnWidth, 8, 8, dontAdornRows, AdornCols, singleSelection, kSystemFontTextStyle, dontPreferOutline},
			1},

		ViewSignatureAndClassname
			{'stat', 96, "TSubscriberNameView", noID, notEnabled, noIdle, {}, MAThreeOh,
			{kDayViewHeight - kSubscriberNameHeight, 0}, {kSubscriberNameHeight, kColumnWidth},
			sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {2, 2, 2, 2}, kApplFont9TextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justCenter,
			kScheduleWindowID, 2},
			NoSubviews},

	ViewSignatureAndClassname
		{'scrl', 265, "TSecondaryScroller", 'SCRT', notEnabled, noIdle, {}, MAThreeOh,
		{kDayViewHeight + kBorderHeight, 0}, {kPrimaryHeight, kTimeViewWidth},
		sizeRelSuperView, sizeFixed, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		Scroller {noID, noID, {0, 0}, {kRowHeight, kColumnWidth}, noVertConstrain, noHorzConstrain, noInset, respondsToFKeys},
		1},

		ViewSignatureAndClassname
			{'txtg', 136, "TTimeView", 'TIME', notEnabled, noIdle, {}, MAThreeOh,
			{0, 0}, {kScheduleHeight, kTimeViewWidth},
			sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {SelectionAdorner},
				DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
			TextGridView {kNumberOfTimes, 1, kRowHeight, kTimeViewWidth, 4, 4, AdornRows, dontAdornCols, singleSelection, kApplFont9TextStyle, dontPreferOutline},
			NoSubviews},

	ViewSignatureAndClassname
		{'patn', 91, "", noID, notEnabled, noIdle, {}, MAThreeOh,
		{kDayViewHeight, 0}, {kBorderHeight, kWindowWidth - kSBarSizeMinus1},
		sizeFixed, sizeRelSuperView, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		Pattern {mPatternHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, dontPreferColor, kBlackPattern},
		NoSubviews},

	ViewSignatureAndClassname
		{'patn', 91, "", noID, notEnabled, noIdle, {}, MAThreeOh,
		{0, kTimeViewWidth}, {kWindowHeight - kSBarSizeMinus1, kBorderWidth},
		sizeRelSuperView, sizeFixed, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		Pattern {mPatternHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, dontPreferColor, kBlackPattern},
		NoSubviews},

	ViewSignatureAndClassname
		{'ssbr', 101, "", 'hCRP', enabled, noIdle, {}, MAThreeOh,
		{0, 0}, {kSBarSizeMinus1, 0},
		sizeVariable, sizeVariable, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		ScrollerScrollBar {mVScrollBarHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, h, 0, 0, 116},
		NoSubviews},

	ViewSignatureAndClassname
		{'ssbr', 101, "", 'vCRP', enabled, noIdle, {}, MAThreeOh,
		{0, 0}, {0, kSBarSizeMinus1},
		sizeVariable, sizeVariable, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		ScrollerScrollBar {mVScrollBarHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, v, 0, 0, 256},
		NoSubviews},
	}
};

resource 'View' (kScheduleClipViewID,
#if qNames
"kScheduleClipViewID",
#endif
purgeable) { MAThreeOh,
	{
	ViewSignatureAndClassname
		{'view', 266, "TAppointmentClipView", noID, notEnabled, noIdle, {}, MAThreeOh,
		{0, 0},
		{kDayViewHeight + kBorderHeight + (kNumberOfTimes * kRowHeight),
		kTimeViewWidth + kBorderWidth + (kNumberOfDays * kColumnWidth)},
		sizeVariable, sizeVariable, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		View {}, 5},

	ViewSignatureAndClassname
		{'txtg', 135, "TDayView", 'DAYS', notEnabled, noIdle, {}, MAThreeOh,
		{0, kTimeViewWidth + kBorderWidth}, {0, 0}, sizeSuperView, sizeVariable, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		TextGridView {1, kNumberOfDays, kDayViewHeight, kColumnWidth, 8, 8, dontAdornRows, AdornCols, singleSelection, kSystemFontTextStyle, dontPreferOutline},
		NoSubviews},

	ViewSignatureAndClassname
		{'txtg', 136, "TTimeView", 'TIME', notEnabled, noIdle, {}, MAThreeOh,
		{kDayViewHeight + kBorderHeight, 0}, {0, 0}, sizeVariable, sizeSuperView, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		TextGridView {kNumberOfTimes, 1, kRowHeight, kTimeViewWidth, 4, 4, AdornRows, dontAdornCols, singleSelection, kApplFont9TextStyle, dontPreferOutline},
		NoSubviews},

	ViewSignatureAndClassname
		{'txtg', 140, "TScheduleView", kScheduleID, notEnabled, noIdle, {}, MAThreeOh,
		{kDayViewHeight + kBorderHeight, kTimeViewWidth + kBorderWidth}, {0, 0}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		TextGridView {kNumberOfTimes, kNumberOfDays, kRowHeight, kColumnWidth, kRowInset, kColumnInset, AdornRows, AdornCols, multipleSelection, kApplFont9TextStyle, dontPreferOutline},
		NoSubviews},

	ViewSignatureAndClassname
		{'patn', 91, "", noID, notEnabled, noIdle, {}, MAThreeOh,
		{kDayViewHeight, 0}, {kBorderHeight, 0}, sizeFixed, sizeSuperView, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		Pattern {mPatternHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, dontPreferColor, kBlackPattern},
		NoSubviews},

	ViewSignatureAndClassname
		{'patn', 91, "", noID, notEnabled, noIdle, {}, MAThreeOh,
		{0, kTimeViewWidth}, {0, kBorderWidth}, sizeSuperView, sizeFixed, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		Pattern {mPatternHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, dontPreferColor, kBlackPattern},
		NoSubviews},

	}
};

resource 'STR#' (phGetDescription,
#if qNames
"phGetDescription",
#endif
purgeable) {
	{
	"";							// Window title
	"Description:";				// description label
	"";							// initial description
	"Cancel";					// Cancel button title
	"OK";						// OK button title
	}
};

resource 'View' (phGetDescription,
#if qNames
"phGetDescription",
#endif
 purgeable) { MAThreeOh,
	{
	ViewSignatureAndClassname
		{'wind', 841, "", noID, enabled, noIdle,
		{BehaviorSignatureAndClassname {noID, "TDIALOGBEHAVIOR", DialogBehavior {enabled, noIdle, modal, 'OK  ', 'CNCL'}}},
		MAThreeOh,
		{50, 40}, {150, 295}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner},
			DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
		Window {dBoxProc, 'DESC', noGoAwayBox, notResizable, ignoreFirstClick,
			freeOnClosing, disposeOnFree, doesntCloseDocument, dontOpenWithDocument, dontAdaptToScreen, dontStagger, dontForceOnScreen,
			center, doesntFloat, doesntHideOnSuspend, generateActivates, filler, phGetDescription, 1},
			4},

		ViewSignatureAndClassname
			{'stat', 96, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{5, 10}, {20, 95}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justSystem, phGetDescription, 2},
			NoSubviews},
		ViewSignatureAndClassname
			{'edit', 173, "", 'DESC', enabled, noIdle, {}, MAThreeOh,
			{30, 10}, {80, 270}, sizeFixed, sizeFixed, shown, wantsToBeTarget,
			handlesCursor, doesntLetSubViewsHandleCursor, 1,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {DrawAdorner},
				AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"0F"}}}, emptyUserArea,
			EditText {mEditTextHit, notHilited, notDimmed, sizeable, {3, 3, 3, 3}, kApplFont9TextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justSystem, phGetDescription, 3, unlimited, LeftRightUpDownBackspace},
			NoSubviews},
		ViewSignatureAndClassname
			{'butn', 92, "", 'CNCL', enabled, noIdle, {}, MAThreeOh,
			{120, 80}, {20, 95}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Button {mDismiss, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, phGetDescription, 4},
			NoSubviews},
		ViewSignatureAndClassname
			{'butn', 204, "", 'OK  ', enabled, noIdle, {}, MAThreeOh,
			{115, 185}, {28, 95}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			DrawingEnvironmentClassIDAndClassname {74, DrawingEnvironment, {3, 3}, 8, $"FFFF FFFF FFFF FFFF", {0, 0, 0}, {65535, 65535, 65535}},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {DrawAdorner},
				AdornAfter, AdornerSignatureAndClassname {'rrct', "TRRECTADORNER", NoIdentifier, freeOnDeletion, $""}}}, emptyUserArea,
			Button {mDismiss, notHilited, notDimmed, sizeable, {4, 4, 4, 4}, kSystemFontTextStyle, dontPreferOutline, phGetDescription, 5},
			NoSubviews}
	}
};

resource 'STR#' (phLookupDialog,
#if qNames
"phLookupDialog",
#endif
purgeable) {
	{
	"";							// Window title
	"Day:";						// Day cluster title
	"";							// Initial Day string
	"Time:";					// Time cluster title
	"";							// Initial time string
	"Response:";				// Response cluster title
	"";							// Initial response string
	"Lookup";					// Lookup button title
	}
};

resource 'View' (phLookupDialog,
#if qNames
"phLookupDialog",
#endif
 purgeable) { MAThreeOh,
{
ViewSignatureAndClassname
	{'wind', 1125, "", noID, enabled, noIdle, {}, MAThreeOh,
	{50, 40}, {208, 448}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget,
	handlesCursor, letsSubViewsHandleCursor, noCursorID,
	handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
	NoDrawingEnvironment {},
	AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
		AdornFirst, AdornerLocalObject {EraseAdorner},
		DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
	Window {documentProc, 'LKUP', goAwayBox, notResizable, ignoreFirstClick,
		freeOnClosing, disposeOnFree, doesntCloseDocument, dontOpenWithDocument, dontAdaptToScreen, stagger, forceOnScreen,
		dontCenter, doesntFloat, doesntHideOnSuspend, generateActivates, filler, phLookupDialog, 1},
		1},

	ViewSignatureAndClassname
		{'dlog', 670, "TLookupDialog", 'DLOG', enabled, noIdle, {}, MAThreeOh,
		{0, 0}, {208, 448}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, emptyUserArea,
		DialogView {'LKUP', noID},
		4},

		ViewSignatureAndClassname
			{'clus', 291, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{16, 32}, {48, 224}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Cluster {mClusterHit, notHilited, notDimmed, sizeable, noInset, kApplFont9TextStyle,
			dontPreferOutline, phLookupDialog, 2},
			2},

			ViewSignatureAndClassname
				{'stat', 96, "", 'DAYT', notEnabled, noIdle, {}, MAThreeOh,
				{16, 16}, {16, 160}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
				handlesCursor, letsSubViewsHandleCursor, noCursorID,
				handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
				NoDrawingEnvironment {},
				AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
					DrawView, AdornerLocalObject {DrawAdorner},
					AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"0F"}}}, emptyUserArea,
				StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {2, 2, 2, 2}, kApplFont9TextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justSystem,
				phLookupDialog, 3},
				NoSubviews},

			ViewSignatureAndClassname
				{'pict', 104, "TArrowsControl", 'DAYA', enabled, noIdle, {}, MAThreeOh,
				{18, 186}, {13, 20}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
				handlesCursor, letsSubViewsHandleCursor, noCursorID,
				handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
				NoDrawingEnvironment {},
				NoAdorners {}, 0,
				Picture {mPictureHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, kLeftRightArrow},
				NoSubviews},

		ViewSignatureAndClassname
			{'clus', 92, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{16, 272}, {48, 160}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Cluster {mClusterHit, notHilited, notDimmed, sizeable, noInset, kApplFont9TextStyle, dontPreferOutline,
			phLookupDialog, 4},
			2},

			ViewSignatureAndClassname
				{'stat', 96, "", 'TIMT', notEnabled, noIdle, {}, MAThreeOh,
				{16, 16}, {16, 96}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
				handlesCursor, letsSubViewsHandleCursor, noCursorID,
				handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
				NoDrawingEnvironment {},
				AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
					DrawView, AdornerLocalObject {DrawAdorner},
					AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"0F"}}}, emptyUserArea,
				StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {2, 2, 2, 2}, kApplFont9TextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justRight,
				phLookupDialog, 5},
				NoSubviews},

			ViewSignatureAndClassname
				{'pict', 104, "TArrowsControl", 'TIMA', enabled, noIdle, {}, MAThreeOh,
				{16, 128}, {18, 11}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
				handlesCursor, letsSubViewsHandleCursor, noCursorID,
				handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
				NoDrawingEnvironment {},
				NoAdorners {}, 0,
				Picture {mPictureHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, kUpDownArrow},
				NoSubviews},

		ViewSignatureAndClassname
			{'clus', 194, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{112, 32}, {80, 400}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Cluster {mClusterHit, notHilited, notDimmed, sizeable, noInset, kApplFont9TextStyle, dontPreferOutline,
			phLookupDialog, 6},
			1},

			ViewSignatureAndClassname
				{'stat', 96, "", 'RESP', notEnabled, noIdle, {}, MAThreeOh,
				{16, 16}, {48, 368}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
				handlesCursor, letsSubViewsHandleCursor, noCursorID,
				handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
				NoDrawingEnvironment {},
				NoAdorners {}, emptyUserArea,
				StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, noInset, kApplFont9TextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justSystem,
				phLookupDialog, 7},
				NoSubviews},

		ViewSignatureAndClassname
			{'butn', 204, "", 'LKUP', enabled, noIdle, {}, MAThreeOh,
			{75, 288}, {30, 128}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			DrawingEnvironmentClassIDAndClassname {74, DrawingEnvironment, {3, 3}, 8, $"FFFF FFFF FFFF FFFF", {0, 0, 0}, {65535, 65535, 65535}},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {DrawAdorner},
				AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"40"}}}, emptyUserArea,
			Button {mButtonHit, notHilited, notDimmed, sizeable, {4, 4, 4, 4}, kSystemFontTextStyle, dontPreferOutline,
			phLookupDialog, 8},
			NoSubviews}
	}
};

//--------------------------------------------------------------------------------------------------
// About Box
//--------------------------------------------------------------------------------------------------

resource 'STR#' (phAboutBox,
#if qNames
"phAboutBox",
#endif
purgeable) {
	{
	"";							// Window title
	"OK";						// OK button title
	"^programName - ^verStr";
	"Description:";
	"Developer University Laboratory Application";
	"This program was written with MacApp 3.1/3.5\n1985-1995 Apple Computer, Inc.";
	}
};

resource 'View' (phAboutBox,
#if qNames
"phAboutBox",
#endif
 purgeable) { MAThreeOh,
	{
	ViewSignatureAndClassname
		{'wind', 1104, "", 'wind', enabled, noIdle,
		{BehaviorSignatureAndClassname {noID, "TDIALOGBEHAVIOR", DialogBehavior {enabled, noIdle, modal, 'ok  ', 'ok  '}}},
		MAThreeOh,
		{45, 50}, {220, 315}, sizeVariable, sizeVariable, notShown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner},
			DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea,
		Window {dBoxProc, 'ok  ', noGoAwayBox, notResizable, ignoreFirstClick,
			freeOnClosing, disposeOnFree, doesntCloseDocument, dontOpenWithDocument, dontAdaptToScreen, dontStagger, dontForceOnScreen,
			center, doesntFloat, doesntHideOnSuspend, generateActivates, filler, phAboutBox, 1},
			5},

		ViewSignatureAndClassname
			{'icon', 91, "", 'ICON', notEnabled, noIdle, {}, MAThreeOh,
			{15, 25}, {32, 32}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Icon {mIconHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, preferColor, kAboutIcon},
			NoSubviews},
		ViewSignatureAndClassname
			{'butn', 204, "", 'ok  ', enabled, noIdle, {}, MAThreeOh,
			{15, 220}, {28, 80}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			DrawingEnvironmentClassIDAndClassname {74, DrawingEnvironment, {3, 3}, 8, $"FFFF FFFF FFFF FFFF", {0, 0, 0}, {65535, 65535, 65535}},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {DrawAdorner},
				AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"40"}}}, emptyUserArea,
			Button {mDismiss, notHilited, notDimmed, sizeable, {4, 4, 4, 4}, kSystemFontTextStyle, dontPreferOutline, phAboutBox, 2},
			NoSubviews},
		ViewSignatureAndClassname
			{'stat', 96, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{10, 75}, {50, 135}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justSystem, phAboutBox, 3},
			NoSubviews},
		ViewSignatureAndClassname
			{'clus', 194, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{65, 20}, {85, 275}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			Cluster {mClusterHit, notHilited, notDimmed, sizeable, noInset, kMonaco9TextStyle, dontPreferOutline, phAboutBox, 4}, 1},
		ViewSignatureAndClassname
			{'stat', 96, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{3, 1}, {82, 274}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {15, 15, 5, 5}, kSystemFontTextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justSystem, phAboutBox, 5},
			NoSubviews},

		ViewSignatureAndClassname
			{'cntl', 248, "", 'xxx1', notEnabled, noIdle, {}, MAThreeOh,
			{160, 55}, {50, 215}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
				DrawView, AdornerLocalObject {DrawAdorner},
				AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"8F"}}}, emptyUserArea,
			Control {mOKHit, notHilited, notDimmed, sizeable, noInset, kSystemFontTextStyle, dontPreferOutline}, 1},
		ViewSignatureAndClassname
			{'stat', 96, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{1, 1}, {49, 214}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {3, 3, 3, 3}, kApplFont9TextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justCenter, phAboutBox, 6},
			NoSubviews}
	}
};

//--------------------------------------------------------------------------------------------------
//  AppleEvents
//--------------------------------------------------------------------------------------------------

resource 'aedt' (kScheduleAEEvents) {
	{
	kAEScheduleSuiteID, kAELookupID, cLookupAppointment;
	}
};

//--------------------------------------------------------------------------------------------------
// MultiFinder information
//--------------------------------------------------------------------------------------------------

resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	isStationeryAware,
	reserved,
	reserved,
	reserved,
	reserved,
#if qDebug
	1500 * 1024,
	1500 * 1024
#else
	1000 * 1024,
	1000 * 1024
#endif
};

//--------------------------------------------------------------------------------------------------
// Arrows picture
//--------------------------------------------------------------------------------------------------

resource 'PICT' (kUpDownArrow, purgeable) {
	89,
	{134, 272, 152, 283},
	$"1101 0100 0A8A D08A D075 3075 3090 0002 00FE 0160 0110 0170 00FE 0160 0110 016B"
	$"0086 0110 0098 011B 0000 3F80 4040 8420 8E20 9F20 BFA0 8E20 8E20 8020 8020 8E20"
	$"8E20 BFA0 9F20 8E20 8420 4040 3F80 FF"
};

resource 'PICT' (kLeftRightArrow, purgeable) {
	326,
	{-1, -1, 12, 19},
	$"1101 A000 8201 000A FFFF FFFF 000C 0013"
	$"0900 0000 0000 0000 0031 0000 0000 000B"
	$"0012 A100 D800 0400 0040 00A1 00D6 0004"
	$"0000 4000 A000 D707 0000 0000 0800 1723"
	$"0906 A100 CA00 0800 0000 0000 0000 00A1"
	$"00C8 0008 0000 005A 005A 0000 9100 0200"
	$"FE01 6001 1001 7000 FE01 6001 1001 6BFF"
	$"FD00 0400 0F00 0F00 0100 0A00 0000 0000"
	$"0000 003F 8040 4084 208E 209F 20BF A08E"
	$"208E 2080 2080 208E 208E 20BF A09F 208E"
	$"2084 2040 403F 80A0 00C9 A100 D600 0400"
	$"0040 00A0 00D7 A000 D9A1 00D6 0004 0000"
	$"4000 A100 D800 0400 0040 00A0 00BE A000"
	$"D990 0004 0000 0000 000E 0020 0000 0000"
	$"000E 0020 0000 FFFE 000E 001E 0001 0FFF"
	$"C000 1000 2000 2102 1000 2303 1000 27CF"
	$"9000 2FCF D000 27CF 9000 2303 1000 2102"
	$"1000 1000 2000 0FFF C000 0000 0000 0000"
	$"0000 0000 0000 A100 D800 0400 0040 00A0"
	$"00BF A000 D9A0 00D7 A000 83FF"
};

//--------------------------------------------------------------------------------------------------
// Memory usage information for MacApp
//--------------------------------------------------------------------------------------------------

// See UMemory.h
// -------------

#ifndef MACAPP_R_13

#if !qPowerPC
resource 'seg!' (kDefaultSegResource,
#if qNames
	"Default seg!",
#endif
	purgeable) {
	{	"GNonRes";
		"GClipboard";
		"GClose";
		"GFile";
		"GOpen";
		"GSelCommand";
		"GDoCommand";
		"BBNonRes";
		"BBOpen";
		"GNonRes2";
		"GPrint";
		"GReadResource";
	};
};
#endif qPowerPC

resource 'seg!' (kScheduleSeg,
#if qNames
kApplicationName,
#endif
	purgeable) {
	{
		"AAppointmentRes"
	}
};

// Additional Resident Segments
// ----------------------------

resource 'res!' (kScheduleRes,
#if qNames
	kApplicationName,
#endif
	purgeable) {
	{
		"ARes";
		"AAppointmentRes";
	};
};

#endif // !MACAPP_R_13


// Additional Memory Requirements
// ------------------------------

resource 'mem!' (kScheduleMem,
#if qNames
	"Additional Memory Requirements",
#endif
	purgeable) {
	0,
	0,
	0,				// Add to temporary reserve
	0,				// Add to permanent reserve
	0				// Add to stack space
};

//--------------------------------------------------------------------------------------------------
// Icons, Bundles and FRefsOh my! (don't forget the Signature)
//--------------------------------------------------------------------------------------------------

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"Schedule 1991-1995 Apple Computer, Inc."
};

resource 'FREF' (kApplicationID,
#if qNames
"Schedule Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (kDocumentID,
#if qNames
"Schedule Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'FREF' (kStationeryID,
#if qNames
"Schedule Stationery",
#endif
	purgeable) {
	kStationery,
	2,
	""
};

resource 'BNDL' (kBundleID,
#if qNames
"Schedule",
#endif
	purgeable) {
	kSignature,
	0,
	{ /* array TypeArray: 2 elements */
		/* [1] */
		'ICN#',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		},
		/* [2] */
		'FREF',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		}
	}
};

//--------------------------------------------------------------------------------------------------
//  Cursors
//--------------------------------------------------------------------------------------------------

resource 'CURS' (kScheduleCrossCursor,
#if qNames
"kScheduleCrossCursor",
#endif
 purgeable) {
	$"0000 0380 0380 0380 0380 0100 783C 7D7C"
	$"783C 0100 0380 0380 0380 0380",
	$"07C0 07C0 07C0 07C0 07C0 FFFE FFFE FFFE"
	$"FFFE FFFE 07C0 07C0 07C0 07C0 07C0",
	{7, 7}
};

resource 'CURS' (kSubscriberCrossCursor,
#if qNames
"kSubscriberCrossCursor",
#endif
 purgeable) {
	$"0000 0380 0280 0280 0280 0100 783C 4544"
	$"783C 0100 0280 0280 0280 0380",
	$"07C0 07C0 07C0 07C0 07C0 FFFE FFFE FFFE"
	$"FFFE FFFE 07C0 07C0 07C0 07C0 07C0",
	{7, 7}
};

//--------------------------------------------------------------------------------------------------
//  Patterns
//--------------------------------------------------------------------------------------------------

resource 'PAT ' (kBlackPattern,
#if qNames
"kBlackPattern",
#endif
 purgeable) {
	$"FFFF FFFF FFFF FFFF"
};

//--------------------------------------------------------------------------------------------------
//  Version resources
//--------------------------------------------------------------------------------------------------

// The revision of this particular file

resource 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x03,
	0x01,
	release,
	0x00,
	verUs,
	"3.1",
	"Schedule 3.1, 1991-1995 Apple Computer, Inc."
};

resource 'vers' (2,
#if qNames
	"Package Version",
#endif
	purgeable) {
	0x01,
	0x00,
	release,
	0x00,
	verUs,
	"DU MacApp Course 1.0",
	"DU MacApp Course 1.0"
};

//--------------------------------------------------------------------------------------------------
//  String Lists
//--------------------------------------------------------------------------------------------------

resource 'STR#' (kScheduleBuzz,
#if qNames
"kScheduleBuzz",
#endif
purgeable) {
	{
	"^Day, ^Time (^Duration)\n^Description\n";
	"^Hours hr, ^Minutes min";
	}
};

resource 'STR#' (kLookupBuzz,
#if qNames
"kLookupBuzz",
#endif
purgeable) {
	{
	"^LookupDay @ ^LookupTime can be scheduled in ^LookupDocument for ^LookupDuration.";
	"Select a Schedule Application:";
	"Schedules";
	}
};

//--------------------------------------------------------------------------------------------------
//  Balloon Help Resources for Views
//--------------------------------------------------------------------------------------------------

resource 'STR#' (kHelpID,
#if qNames
"kHelpID",
#endif
purgeable) {
	{
	"Sketch, drag, stretch or select appointments here";		// Help for TScheduleView
	"Subscribed appointments appear here.";						// Help for TSubscriberView
	"Select a day to publish from here.";						// Help for TDayView
	"To subscribe, select here.";								// Help for TSubscriberDayView
	}
};

resource 'hdlg' (kHelpID)
{
	HelpMgrVersion,
	0,
	hmDefaultOptions,
	0, /* balloon definition function */
	5, /* variation code */
	HMSkipItem {}, /* No missing item help */
	{
		HMStringResItem
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			kHelpID,1,
			kHelpID,1,
			kHelpID,1,
			kHelpID,1,
		},
		HMStringResItem
		{
			{ -1,0 }, /* tip location */
			{ 0,0,10,10 }, /* hot rectangle */
			kHelpID,2,
			kHelpID,2,
			kHelpID,2,
			kHelpID,2,
		},
		HMStringResItem
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			kHelpID,3,
			kHelpID,3,
			kHelpID,3,
			kHelpID,3,
		},
		HMStringResItem
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			kHelpID,4,
			kHelpID,4,
			kHelpID,4,
			kHelpID,4,
		}
	}
};

/*--------------------------------------------------------------------------------
	 Application Error Alert Strings and Tables
--------------------------------------------------------------------------------*/

resource 'errs' (errAppOperation,
#if qNames
"Appl Error Operation Table",
#endif
purgeable) { /* application's operation table */
	{

	whichList, 0, errAppOperations,
	errAppointmentNotAvailable, errAppointmentNotAvailable, 1;
	}
};

resource 'STR#' (errAppOperations,
#if qNames
"Appl Error Operation Strings",
#endif
purgeable) { /* application's operation strings */
	{
	"schedule the appointment",
	}
};

resource 'errs' (errAppReason,
#if qNames
"Appl Error Reason Table",
#endif
purgeable) { /* application's reason table */
	{
		whichList, 0, errAppReasons;
		errAppointmentNotAvailable, errAppointmentNotAvailable, 1,
	}
};

resource 'STR#' (errAppReasons,
#if qNames
"Appl Error Reason Strings",
#endif
 purgeable) { /* application's error reason strings */
	{
	"it is not available";
	}
};

resource 'errs' (errAppRecovery,
#if qNames
"Appl Error Recovery Table",
#endif
purgeable) { /* application's recovery table */
	{	whichList, 0, errAppRecoveries;
		errAppointmentNotAvailable, errAppointmentNotAvailable, 1,
	}
};

resource 'STR#' (errAppRecoveries,
#if qNames
"Appl Error Recovery Strings",
#endif
purgeable) { /* application's error recovery strings */
	{
	"Select a new day and time and try again.";
	}
};

