//--------------------------------------------------------------------------------------------------
//
//	File:			SchedulePrint.r
//
//	Description:	Resource File for the Schedule Sample Program Printing.
//
//	Copyright 1991-1995 by Apple Computer, Inc.  All rights reserved.
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Include Files
//--------------------------------------------------------------------------------------------------

// Standard Includes

#ifndef __TYPES.R__
#include "Types.r"
#endif

#ifndef __SYSTYPES.R__
#include "SysTypes.r"
#endif

#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

//--------------------------------------------------------------------------------------------------
// Constants
//--------------------------------------------------------------------------------------------------

// Text styles
// --------------

#define kPrintTextStyle					2001
#define kHeaderTextStyle				2002
#define kFooterTextStyle				2003

// 'View' resources
// --------------

#define kSchedulePrintID				2001
#define kScheduleHeaderID				2002
#define kScheduleFooterID				2003

// String lists
// --------------

#define kHeadingBuzzString				2001
#define kAppointmentData				2002

// Custom column list resource
// ----------------------------

#define kColumnListType					'COLW'
#define kColumnListID					2001

//--------------------------------------------------------------------------------------------------
// Text Styles
//--------------------------------------------------------------------------------------------------

resource 'TxSt' (kPrintTextStyle, purgeable) {tsPlain, 12, black, "Helvetica"};
resource 'TxSt' (kHeaderTextStyle, purgeable) {bold + italic, 14, black, "Helvetica"};
resource 'TxSt' (kFooterTextStyle, purgeable) {tsPlain, 9, black, "Helvetica"};

//--------------------------------------------------------------------------------------------------
// Views
//--------------------------------------------------------------------------------------------------

resource 'View' (kScheduleHeaderID,
#if qNames
"kScheduleHeaderID",
#endif
purgeable) { MAThreeOh,
	{
	ViewSignatureAndClassname
		{'view', 62, "", noID, notEnabled, noIdle, {}, MAThreeOh,
		{0, 0}, {60, 100}, sizeFixed, sizeVariable, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			DrawView, AdornerLocalObject {DrawAdorner},
			AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"04"}}},
		emptyUserArea,
		View {},
		2},

		ViewSignatureAndClassname
			{'stat', 96, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{0, 0}, {32, 100}, sizeFixed, sizeSuperView, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {4, 4, 4, 16}, kHeaderTextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justCenter, kHeadingBuzzString, 5},
			NoSubviews},

		ViewSignatureAndClassname
			{'txtg', 142, "TStringListView", noID, notEnabled, noIdle, {}, MAThreeOh,
			{32, 0}, {20, 100}, sizeFixed, sizeSuperView, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, kHeadingBuzzString,
			TextGridView {1, 4, 20, 20, 4, 4, dontAdornRows, dontAdornCols, singleSelection, kPrintTextStyle, dontPreferOutline},
			NoSubviews},
	}
};

resource 'View' (kSchedulePrintID,
#if qNames
"kSchedulePrintID",
#endif
purgeable) { MAThreeOh,
	{
	ViewSignatureAndClassname
		{'txtg', 145, "TSchedulePrintView", noID, notEnabled, noIdle, {}, MAThreeOh,
		{0, 0}, {100, 80}, sizeVariable, sizePage, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		NoAdorners {}, kAppointmentData,
		TextGridView {5, 4, 20, 20, 4, 4, dontAdornRows, dontAdornCols, singleSelection, kPrintTextStyle, dontPreferOutline},
		NoSubviews},
	}
};

resource 'View' (kScheduleFooterID,
#if qNames
"kScheduleFooterID",
#endif
purgeable) { MAThreeOh,
	{
	ViewSignatureAndClassname
		{'view', 62, "", noID, notEnabled, noIdle, {}, MAThreeOh,
		{0, 0}, {48, 100}, sizeFixed, sizeVariable, shown, doesntWantToBeTarget,
		handlesCursor, letsSubViewsHandleCursor, noCursorID,
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
		NoDrawingEnvironment {},
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			DrawView, AdornerLocalObject {DrawAdorner},
			AdornAfter, AdornerSignatureAndClassname {'comp', "", 'comp', freeOnDeletion, $"01"}}},
		emptyUserArea,
		View {},
		1},

		ViewSignatureAndClassname
			{'stat', 154, "", noID, notEnabled, noIdle, {}, MAThreeOh,
			{12, 0}, {36, 100}, sizeSuperView, sizeSuperView, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			NoAdorners {}, emptyUserArea,
			StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {4, 4, 4, 4}, kFooterTextStyle, dontPreferOutline, autoWrap, dontEraseFirst, justCenter, kHeadingBuzzString, 6},
			NoSubviews}
	}
};

//--------------------------------------------------------------------------------------------------
// Column widths
//--------------------------------------------------------------------------------------------------

type kColumnListType
{
	integer = $$Countof(ColumnWidths);
	array ColumnWidths { integer; };
};

resource kColumnListType (kColumnListID,
#if qNames
"Printing Column widths",
#endif
purgeable) {
	{72, 72, 72, 5 * 72 };
};

//--------------------------------------------------------------------------------------------------
// Column titles
//--------------------------------------------------------------------------------------------------

resource 'STR#' (kHeadingBuzzString,
#if qNames
"kHeadingBuzzString",
#endif
purgeable) {
	{
	"Day";
	"Time";
	"Duration";
	"Description";
	"^DocumentTitle";
	"- ^PageNumber -";
	}
};

resource 'STR#' (kAppointmentData,
#if qNames
"kAppointmentData",
#endif
purgeable) {
	{
	"^PrintDay";
	"^PrintTime";
	"^PrintDuration";
	"^PrintDescription";
	}
};

