//----------------------------------------------------------------------------------------
// TemplateCompatibility.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// This file contains compatibility typedefs, defines, and classes to ease conversion
// between earlier versions of MacApp and the current version.  You may modify its contents
// as your conversion proceeds, to remove items that have been fully converted, or to add 
// items to make your code more compatible with the current version.
//
// If your project source code must span multiple versions of MacApp, you may want to
// augment this file with other compatibility macros to allow you code to work with
// multiple versions of MacApp (test qMacAppVersion), or with other frameworks (test qMacApp).
//
// This file is a companion to the compile flag qBackwardCompatible, which also makes member
// data public in MacApp classes, and defines inlined versions of obsolete MacApp class methods.

#ifndef __TemplateCompatibility__
#define __TemplateCompatibility__

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifdef MACAPP_R_13

#if defined(qBackwardCompatible) && !qBackwardCompatible
#error "Add '#define qBackwardCompatible 0' to AutoSwitches_AC.h to make data members public and to define obsolete methods"
#endif

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __FileTypes_AC__
#include "FileTypes_AC.h"
#endif

#ifndef __CRGBColor_AC__
#include "CRGBColor_AC.h"
#endif

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif

#ifndef __CFSSpec_AC__
#include "CFSSpec_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

#ifndef __CFile_AC__
#include "CFile_AC.h"
#endif

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

#ifndef __CReporter_AC__
#include "CReporter_AC.h"
#endif

#ifndef __MSTREAMABLE_AC__
#include "MStreamable_AC.h"
#endif

#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

#ifndef __CStreamContext_AC__
#include "CStreamContext_AC.h"
#endif

#ifndef __PlatformMemory__
#include "PlatformMemory.h"
#endif

#ifndef __ResFileUtils_AC__
#include "ResFileUtils_AC.h"
#endif

#ifndef __UDIALOGVIEW__
#include "UDialogView.h"
#endif

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UGRIDVIEW__
#include "UGridView.h"
#endif

// Stuff that got moved gets automagically included in here.

#ifndef __USUBSTITUTION__
#include "USubstitution.h"
#endif

#ifndef __UFILESTREAM__
#include "UFileStream.h"
#endif

#include <Types.h>

//========================================================================================
// The following old MacApp types are defined for backward compatibility.
//========================================================================================

// Obsolete header files: 
// defines are provided here so the obsolete includes can be skipped over in your code.

#ifndef __GEOMETRY__
#define __GEOMETRY__		// use CPoint_AC and CRect_AC in ACS imaging
#endif

#ifndef __PASCALSTRING__
#define __PASCALSTRING__	// use CPascalString_AC in ACS core
#endif

#ifndef __UDEPENDENCIES__
#define __UDEPENDENCIES__	// use CDependencies_AC in ACS patterns
#endif

#ifndef __UFAILURE__
#define __UFAILURE__		// use CException_AC in ACS core
#endif

#ifndef __UITERATOR__
#define __UITERATOR__		// use CIterator_AC in ACS containers
#endif

#ifndef __ULIST__
#define __ULIST__			// use CList_AC in ACS containers
#endif

#ifndef __ULISTITERATOR__
#define __ULISTITERATOR__	// use CListIterator_AC in ACS containers
#endif

#ifndef __UOBJECT__
#define __UOBJECT__			// remove all includes of UObject.h.
#endif


// Defines for iterators.  
// N.b., FirstDocument and NextDocument can only be defined if using precompiled headers for MacApp,
// otherwise there will be a conflict with method names in a MacApp header file for an iterator.

#define More							Current
#define FirstDocument					First
#define NextDocument					Next
#define FirstObject						First
#define NextObject						Next
#define FirstSection					First
#define NextSection						Next
#define FirstAdorner					First
#define NextAdorner						Next

// Defines for casts and exception handling

#define MAstatic_cast(TYPE, EXPR)		static_cast<TYPE>(EXPR)
#define MAconst_cast(TYPE, EXPR)		const_cast<TYPE>(EXPR)
#define MAreinterpret_cast(TYPE, EXPR)	reinterpret_cast_AC(TYPE, EXPR)

#define Assertion(COND, DESC)			Assertion_AC(COND, DESC)
#define FailMemError()					ThrowIfMemError_AC()
#define FailResError()					ThrowIfResError_AC()
#define FailNewMessage(EXPR, MSG1, MSG2) ThrowNewMessage_AC(EXPR, MSG1, MSG2)
#define FailNIL(EXPR)					ThrowIfNULL_AC(EXPR)
#define FailNILResource(EXPR)			ThrowIfNULLResource_AC(EXPR)
#define FailOSErr(EXPR)					ThrowIfOSErr_AC(EXPR)
#define Failure(EXPR, MSG)				ThrowException_AC(EXPR, MSG)
#define ProgramBreak(EXPR)				ProgramBreak_AC(EXPR)
#define ProgramReport(EXPR1, EXPR2)		ProgramReport_AC(EXPR1, EXPR2)

// Defines for adorners

#define gDimAdorner						&TAdorner::fgDimAdorner
#define gDrawAdorner					&TAdorner::fgDrawAdorner
#define gEraseAdorner					&TAdorner::fgEraseAdorner
#define gHiliteAdorner					&TAdorner::fgHiliteAdorner
#define gPrintAdorner					&TAdorner::fgPrintAdorner
#define gResizeIconAdorner				&TAdorner::fgResizeIconAdorner
#define gSelectionAdorner				&TAdorner::fgSelectionAdorner

// Defines for scoped globals

#define gMenuBarManager					TMenuBarManager::fgMenuBarManager
#define gMacAppDependencies				CDependencySpace_AC::fgDefaultDependencySpace

// Note: these macros are no longer necessary for exception handling, but are
// 		 provided for backward compatibility

#define MAVolatile(type, id)				type id;
#define MAVolatileInit(type, id, value)		type id = value;
					
// typedefs

typedef class CRect_AC 					CRect;
typedef class CPoint_AC					CPoint;
typedef CViewPoint						VPoint;
typedef CViewRect  						VRect;
typedef CViewPoint* 					VPointPtr;
typedef CViewRect*  					VRectPtr;
typedef ViewCoordinate 					VCoordinate;
typedef CTempRegion_AC					CTempRegion;
typedef CTempRegion_AC 					CTemporaryRegion;			// for compatibility
typedef class CRGBColor_AC 				CRGBColor;
typedef class CTextStyle_AC 			CTextStyle;
typedef class CWhileOutlinePreferred_AC	CWhileOutlinePreferred;
typedef class CFSSpec_AC 				CFSSpec;
typedef enum EMoveHiState_AC 			EMoveHiState;
typedef enum EReleaseState_AC 			EReleaseState;
typedef enum ELockState_AC 				ELockState;
typedef CTempHLock_AC 					CTempHLock;
typedef CTempHandleLock_AC 				CTempHandleLock;
typedef CTempHandle_AC 					CTempHandle;
typedef CHandleIterator_AC				CHandleIterator;
typedef CTempPtr_AC 					CTempPtr;
typedef CTempRoutineDescriptor_AC		CTempRoutineDescriptor;
typedef CTempUseResFile_AC 				CTempUseResFile;
typedef CList_AC						TList;
typedef ArrayIndex_AC					ArrayIndex;
typedef CompareResult_AC				CompareResult;
typedef CSortedHandleList_AC			TSortedHandleList;
typedef ChangeID_AC						ChangeID;
typedef ClassID_AC						ClassID;
typedef class CCharStr_AC 				CCharStr,
										*CCharStrPtr, 
										**CCharStrHandle;
typedef class CPascalStr_AC 			CPascalStr,
										*CPascalStrPtr, 
										**CPascalStrHandle;
typedef class CPascalStr_AC 			CString, 
										*CStringPtr, 
										**CStringHandle;
typedef unsigned char 					Str2[3];
typedef unsigned char 					Str8[9];
typedef class CChar255_AC 				CChar255;
typedef class CStr255_AC 				CStr255;
typedef TemplateCharStr<Str63>			CChar63;
typedef TemplateCharStr<Str32>			CChar32;
typedef TemplateCharStr<Str31>			CChar31;
typedef TemplateCharStr<Str27>			CChar27;
typedef TemplateCharStr<Str15>			CChar15;
typedef TemplateCharStr<Str8>			CChar8;
typedef TemplateCharStr<Str2>			CChar2;
typedef TemplatePascalStr<Str63> 		CStr63;
typedef TemplatePascalStr<Str32> 		CStr32;
typedef TemplatePascalStr<Str31> 		CStr31;
typedef TemplatePascalStr<Str27> 		CStr27;
typedef TemplatePascalStr<Str15> 		CStr15;
typedef TemplatePascalStr<Str8> 		CStr8;
typedef TemplatePascalStr<Str2> 		CStr2;
typedef CDependencySpace_AC				TDependencySpace;
typedef CStreamContext_AC				TContext;

typedef CArrayIterator_AC				CArrayIterator;
typedef CDependencyIterator_AC			CDependencyIterator;
typedef CNotificationQueue_AC			TNotificationQueue;
typedef CHandleIterator_AC				CHandleIterator;
typedef CLongintIterator_AC				CLongintIterator;
typedef CNotifierIterator_AC			CNotifierIterator;

typedef CDynamicArray_AC				TDynamicArray;
typedef CHandleList_AC					THandleList;
typedef CLongintList_AC					TLongintList;
typedef CSimpleDependencySpace_AC		TSimpleDependencySpace;	
typedef CSortedDynamicArray_AC			TSortedDynamicArray;
typedef CSortedHandleList_AC			TSortedHandleList;
typedef CSortedList_AC					TSortedList;
typedef CSortedLongintList_AC			TSortedLongintList;
typedef CSortedVoidList_AC				TSortedVoidList;
typedef CStandardDependencySpace_AC		TStandardDependencySpace;

// enumerations and constants

const OSType kAllQuestionsType 			= kAllQuestionsType_AC;
const OSType kTextType 					= kTextType_AC;
const OSType kPictType 					= kPictType_AC;
const OSType kSoundType 				= kSoundType_AC;
const OSType kStyleType 				= kStyleType_AC;
const short kNoFileRefNum 				= kNoFileRefNum_AC;
const short kNoFileRefnum				= kNoFileRefNum_AC;
const OSType kFolderType 				= kFolderType_AC;
const OSType kFolderCreator 			= kFolderCreator_AC;
const OSType kGenericType 				= kGenericType_AC;
const OSType kGenericCreator 			= kGenericCreator_AC;

#define kItem1GreaterThanItem2			kItem1GreaterThanItem2_AC
#define kItem1LessThanItem2				kItem1GreaterThanItem2_AC
#define kItem1EqualItem2				kItem1EqualItem2_AC
#define kEmptyIndex						kEmptyIndex_AC
#define kIterateForward					kIterateForward_AC
#define kIterateBackward				kIterateBackward_AC

#if qDebug
const SignedByte kNewInitVal 			= kNewInitVal_AC;			// debugging constant, odd at all byte boundaries 
const SignedByte kResizedPtrInitVal		= kResizedPtrInitVal_AC;		// debugging constant, odd at all byte boundaries 
const SignedByte kResizedInitVal 		= kResizedInitVal_AC;		// debugging constant, odd at all byte boundaries 
const SignedByte kDisposedInitVal 		= kDisposedInitVal_AC;		// debugging constant, odd at all byte boundaries 
#endif

// global functions
#define MABlockMove 					BlockMove_AC
#define MABlockMoveOverlap 				BlockMoveOverlap_AC

#define GetEnvironsVersion() 			GetEnvironsVersion_AC()
#define GetMachineType()				GetMachineType_AC()
#define GetSystemVersion()				GetSystemVersion_AC()
#define HasColorQD()					HasColorQD_AC()
#define Has32BitQD()					Has32BitQD_AC()
#define KeyboardType()					KeyboardType_AC()
#define GetATDrvrVersNum()				GetATDrvrVersNum_AC()
#define HasSCSI()						HasSCSI_AC()
#define HasAUX()						HasAUX_AC()
#define HasTempMem()					HasTempMem_AC()
#define GetTEVersion()					GetTEVersion_AC()
#define HasDesktopBus()					HasDesktopBus_AC()
#define HasTrueType()					HasTrueType_AC()
#define HasThreadManager()				HasThreadManager_AC()
#define HasQDGXGraphics()				HasQDGXGraphics_AC()
#define HasQDGXPrinting()				HasQDGXPrinting_AC()
#define HasCFM()						HasCFM_AC()
#define HasTranslationManager()			HasTranslationManager_AC()
#define HasSpeechManager()				HasSpeechManager_AC()
#define HasCustomFile()					HasCustomFile_AC()
#define HasQuickTime()					HasQuickTime_AC()
#define HasTSM()						HasTSM_AC()
#define HasAppleScript()				HasAppleScript_AC()
#define HasAppleGuide()					HasAppleGuide_AC()
#define HasQuickDraw3D()				HasQuickDraw3D_AC()
#define HasQuickDraw3DViewer()			HasQuickDraw3DViewer_AC()
#define HasDragManager()				HasDragManager_AC()
#define HasAOCEToolBox()				HasAOCEToolBox_AC()
#define NewList()						NewList_AC()

//	// Removed due to ASLM removal vs. deinstall problems.
//	bool HasASLM();

#define Min(a,b)    					Min_AC((a), (b))   
#define Max(a,b)     					Max_AC((a), (b))   
#define MinMax(	minVal, \
				expression, maxVal)     MinMax_AC((minVal), (expression), (maxVal))   
 
#define DisposeIfHandle(aHandle) 		DisposeIfHandle_AC((aHandle))
#define DisposeIfPtr(aPtr)			DisposeIfPtr_AC((aPtr))
#define BlockSet(destPtr,byteCount,\
				setVal)					BlockSet_AC((destPtr), (byteCount), (setVal))
#define IsAResource(h) 					IsAResource_AC((h))


#define IsHandleLocked(h) 				IsHandleLocked_AC((h))
#define CopyHandle(inHandle, outHandle) CopyHandle_AC((inHandle),( outHandle))
#define MACurResFile() 					CurResFile_AC()
#define MAUseResFile(refNum) 			UseResFile_AC((refNum))

inline SignedByte LockHandle(Handle h, EMoveHiState high = eDontMoveHi)
{	return LockHandle_AC(h, high); } // can't have default params with a macro

inline void MATextBox(Ptr text,
			   short itsLength,
			   const CRect_AC& box,
			   short itsJust,
			   bool autoWrap = kAutoWrap,
			   ProcPtr wordBreak = NULL,
			   bool eraseFirst = kNoEraseFirst,
			   bool spaceForCaret = kSpaceForCaret,
			   bool preferOutline = kDontPreferOutlineFonts)	
{ TextBox_AC(text, itsLength, box, itsJust, autoWrap, wordBreak, eraseFirst, spaceForCaret, preferOutline); } 
// can't have default params with a macro

#if qDebug
#define IsHandle(h)						IsHandle_AC((h))
#endif

#define DependencyLabels				DependencyLabels_AC

#define	NewAllocatedList()				NewAllocatedList_AC()
#define NewSortedList()					NewSortedList_AC()

//========================================================================================
// The following global routines were originally for Pascal compatibility. They are basically
// wrappers that call back into the methods in VRect and VPoint. C++ programs should use
// the methods in VRect and VPoint to avoid the additional function call overhead.
//========================================================================================

EXPORT_MA void PtToVPt(const CPoint_AC thePt, VPoint& theVPt);
EXPORT_MA CPoint_AC VPtToPt(const VPoint& theVPt);
EXPORT_MA void RectToVRect(const CRect_AC& theRect, VRect& theVRect);
EXPORT_MA void VRectToRect(const VRect& theVRect, CRect_AC& theRect);
EXPORT_MA void AddVPt(const VPoint& srcVPt, VPoint& dstVPt);
EXPORT_MA void SubVPt(const VPoint& srcVPt, VPoint& dstVPt);
EXPORT_MA void SetVPt(VPoint& vPt, VCoordinate h, VCoordinate v);
EXPORT_MA bool EqualVPt(const VPoint& aVPt, const VPoint& bVPt);
EXPORT_MA void SetVRect(VRect& vRt,
			  VCoordinate left, VCoordinate top,
			  VCoordinate right, VCoordinate bottom);
EXPORT_MA void OffsetVRect(VRect& vRt, VCoordinate dh, VCoordinate dv);
EXPORT_MA void InsetVRect(VRect& vRt, VCoordinate dh, VCoordinate dv);
EXPORT_MA void Pt2VRect(const VPoint& theTopLeft, const VPoint& theBotRight, VRect& vRt);
EXPORT_MA bool PtInVRect(const VPoint& vPt, const VRect& vRt);
EXPORT_MA bool EmptyVRect(const VRect& vRt);
EXPORT_MA bool EqualVRect(const VRect& aVRt, const VRect& bVRt);
EXPORT_MA VCoordinate LengthVRect(const VRect& vRt, VHSelect whichDim);
EXPORT_MA void PinVRect(const VRect& vRt, VPoint& vPt);
EXPORT_MA bool SectVRect(const VRect& src1, const VRect& src2, VRect& dst);
EXPORT_MA void UnionVRect(const VRect& src1, const VRect& src2, VRect& dst);

// These two were moved from UMacAppUtilities.h

inline bool RectsNest(const CRect_AC& outer, const CRect_AC& inner)
{ return outer.Contains(inner); }

inline bool VRectsNest(const VRect& outer, const VRect& inner)
{ return outer.Contains(inner); }


// R13 replaces this class with a temporary allocation class for consistency with CTempAllocation
// and only uses the word Perm with Toolbox Memory Manager wrappers.

class EXPORT_MA CTempAllocateObjectsFromPerm : CTempObjectAllocation
{
public:
	inline CTempAllocateObjectsFromPerm(bool perm) : CTempObjectAllocation(!perm) { }
};

// Freeing an object, a list, and a list's contents

#define FreeIfObject(obj) (delete obj, NULL)
#define FreeListIfObject(obj) (delete obj, NULL)

// would be (list->FreeAll())

#define FREEALL(listRef, baseClass) FreeAll(listRef, static_cast<baseClass*> (NULL) )

template <class LIST, class BASE_CLASS>
void FreeAll(LIST& inList, BASE_CLASS* /*ignoreParam*/)
{
	TemplateArrayIterator_AC<BASE_CLASS> iter(inList);
	for (iter.First(); iter; ++iter)
		delete *iter;
	inList->DeleteAll();
}

#define ConstCStr255Param const CStr255_AC&

// Array iterator functions

#define Advance		Next

// Compatible TObject base class.  
// Note: you can't cast an MStreamable_AC (such as a TView descendant) into a TObject.

class TObject : public MStreamable_AC, public MDependable_AC
{
public:
	MA_DECLARE_CLASS;
	TObject() {}
	virtual ~TObject() {}
	virtual void Initialize() {}
	virtual void Free() { delete this; }
};

// Depracated FailInfo: does not function, merely allows embedded FailInfos to still compile & run.
// Note: because of native C++ exceptions, FailInfo is not needed.

typedef void (*FailureCleanupProc)(void* context);

class FailInfo 
{
public:
	void Reset() {}
	void SetCleanupProc(FailureCleanupProc /*ignore*/, void* /*ignore*/) {}
	void Sucess() {}

};
inline void Success(FailInfo& /*fi*/) {}

// Transitional macros: native C++ versions.
// Versions for MacApp prior to R13 are provided in comments.

#define TRY 		try							// would be 'Try(fi)'
#define CATCH_ALL 	catch(CException_AC& fi)	// would be 'else'
#define ENDTRY									// would be 'fi.Success();'
#define THROW 		throw 						// would be 'fi.ReSignal()'

#define CONST		const 						// would be empty (for functions that became const)

#define MA_DYNAMIC_CAST(castType, castInstance) dynamic_cast_AC(castType*, castInstance)

#define GRIDVIEWDESIGNATORPTR TGridViewDesignator*

inline GridRect GRIDVIEWDESIGNATOR_GETBOUNDS(const TGridViewDesignator* designator) 
	{ return designator->GetBounds(); }				// would be (*designator)->rgnBBox
inline GridRect GRIDVIEWDESIGNATOR_GETBOUNDS(const TGridViewDesignator& designator) 
	{ return designator.GetBounds(); }				// would be (*designator)->rgnBBox
inline void GRIDVIEWDESIGNATOR_SETEMPTY(TGridViewDesignator* designator)
	{ designator->Set(); }						// would be ::SetEmptyRgn(designator)
inline bool GRIDVIEWDESIGNATOR_ISEMPTY(const TGridViewDesignator* designator)
	{ return designator->IsEmpty(); }						// would be ::EmptyRgn(designator)
inline void GRIDVIEWDESIGNATOR_SET(TGridViewDesignator* srcdesignator, TGridViewDesignator& targdesignator) // would be ::CopyRgn(srcdesignator, targdesignator)
	{ targdesignator.Set(srcdesignator); }
	
#define PAGE_INTERIOR() (GetInteriorRect())	// would be fPageAreas.interior

#define COMPAREOBJ	void
#define APPLEEVENTPTR TAppleEvent*

#else
//========================================================================================
// MacApp 3.1/3.3 compatible definitions and types
//========================================================================================

// This defines the keyword bool in a manner compatible with Boolean,
// which requires that the compiler bool support be turned off.
// If you do not wish to have this compatibility and prefer compiler bool support, 
// set qWantsBool to 1.

#ifndef qWantsBool
#define qWantsBool 0
#endif

#if !qWantsBool
#	if __option(bool) || defined(bool)
#		error "bool support must be turned off and bool undefined to enable compatibility with Boolean"
#	else
// We define bool as Boolean
#		undef bool
#		define bool Boolean
#	endif
#endif

// Terms back-ported from R13 for use here
#ifndef TH_new
#define TH_new new
#endif

// GridView coordinate abstractions added to R12 in anticipation for new definitions in R13

typedef CPoint GridCell;
typedef CRect GridRect;
typedef short GridCoordinate;

// transition macros

#define FREEALL(listRef, baseClass) listRef->FreeAll()

// NOTE: these vary from R12's version in that TRY and ENDTRY are not scoped with { and }.
// This is because in MacApp 3.1 and 3.3 it is common to call ENDTRY inside the TRY block.
// The workaround is to add { and } manually in your code where you have more than one
// Try block at the same level of scope.

#if defined(MACAPP_3_1) || defined(MACAPP_3_3)
#define TRY 		FailInfo fi; Try(fi)
#define CATCH_ALL 	else
#define ENDTRY		fi.Success();
#define THROW 		fi.ReSignal()
#endif


#define CONST		

// "ImagingStreaming" forward-compatible definitions

inline TStream& operator << (TStream& inStream, const CPoint& inData)
{
	inStream.WritePoint(inData);
	return inStream;
}
inline TStream& operator << (TStream& inStream, const VPoint& inData)
{
	inStream.WriteVPoint(inData);
	return inStream;
}
inline TStream& operator << (TStream& inStream, const CRect& inData)
{
	inStream.WriteRect(inData);
	return inStream;
}
inline TStream& operator << (TStream& inStream, const VRect& inData)
{
	inStream.WriteVRect(inData);
	return inStream;
}
inline TStream& operator << (TStream& inStream, const CRGBColor& inData)
{
	inStream.WriteRGBColor(inData);
	return inStream;
}


// readers

inline TStream& operator >> (TStream& inStream, CPoint& inData)
{
	inData = inStream.ReadPoint();
	return inStream;
}
inline TStream& operator >> (TStream& inStream, VPoint& inData)
{
	inStream.ReadVPoint(inData);
	return inStream;
}
inline TStream& operator >> (TStream& inStream, CRect& inData)
{
	inStream.WriteRect(inData);;
	return inStream;
}
inline TStream& operator >> (TStream& inStream, VRect& inData)
{
	inStream.WriteVRect(inData);
	return inStream;
}
inline TStream& operator >> (TStream& inStream, CRGBColor& inData)
{
	inStream.WriteRGBColor(inData);
	return inStream;
}

#define GRIDVIEWDESIGNATORPTR RgnHandle

inline GridRect GRIDVIEWDESIGNATOR_GETBOUNDS(RgnHandle designator) 
	{ return (*designator)->rgnBBox; }
inline void GRIDVIEWDESIGNATOR_SETEMPTY(RgnHandle designator)
	{ ::SetEmptyRgn(designator); }
inline bool GRIDVIEWDESIGNATOR_ISEMPTY(RgnHandle designator)
	{ return ::EmptyRgn(designator); }
inline void GRIDVIEWDESIGNATOR_SET(RgnHandle srcdesignator, RgnHandle targdesignator)
	{ ::CopyRgn(srcdesignator, targdesignator); }
	
#define PAGE_INTERIOR() (fPageAreas.theInterior)

#define COMPAREOBJ TObject

#ifdef MACAPP_3_3
#define APPLEEVENTPTR TAppleEvent*
#else 
// MacApp 3.1
#define DoScriptCommand	DoAppleCommand
#define APPLEEVENTPTR const AppleEvent&
#endif

#endif

//========================================================================================


#endif // __TemplateCompatibility__
