//----------------------------------------------------------------------------------------
// UScheduleApplication.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------


#ifndef __USCHEDULEAPPLICATION__
#include "UScheduleApplication.h"
#endif

#ifndef __USCHEDULECOMMAND__
#include "UScheduleCommand.h"
#endif

#ifndef __UTEMPORARYOBJECTS__
#include "UTemporaryObjects.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __UDIALOG__
#include <UDialog.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __UDIALOGBEHAVIOR__
#include <UDialogBehavior.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

#ifndef __UMENUMGR__
#include <UMenuMgr.h>
#endif

#ifndef __UWINDOW__
#include <UWindow.h>
#endif

#ifndef __UDEPENDENCIES__
#include "UDependencies.h"
#endif


//----------------------------------------------------------------------------------------

/* Version resources for custom about box */
const ResType 	kVersInfoType		= 'vers';
const ResNumber kVers1InfoID		= 1;

//----------------------------------------------------------------------------------------
#pragma segment MAAboutApp

#undef Inherited
#define Inherited TApplication
DefineClass(TScheduleApplication, Inherited);

void TScheduleApplication::DoAboutBox()	// OVERRIDE
{
	// Substitutes application name and version number in custom about box.

	TWindow* aWindow = NULL;

	{
		CPermAllocator alloc(false);

		CStr255 aString;

		this->GetApplicationName(aString);
		MAParamText("^programName", aString);			// register it

		VersRecHndl vers1Info = (VersRecHndl)::GetResource(kVersInfoType, kVers1InfoID);	// get version

		if (vers1Info)
			aString = (**vers1Info).shortVersion;		// get version number from version 1
		else
			aString = gEmptyString;

		MAParamText("^verStr", aString);				// register it

		aWindow = gViewServer->NewTemplateWindow(phAboutBox, NULL);		// get dialog
	}

	FailNIL(aWindow);								// maybe the resource was missing?
	aWindow->PoseModally();							// run it
#ifdef MACAPP_R_13
	aWindow->Close();						// close it.
	delete aWindow;
#else
	aWindow->CloseAndFree();
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

void TScheduleApplication::DoScriptCommand(CommandNumber aCommandNumber,
								   APPLEEVENTPTR message,
								   APPLEEVENTPTR reply)		// OVERRIDE
{
	// Launches server command in response to an AppleEvent request.

	switch (aCommandNumber)
	{
	case cLookupAppointment:
		{
			PostCommand( TH_new TScheduleServerCommand(aCommandNumber, message, reply));
		}
		break;

	default:
		Inherited::DoScriptCommand(aCommandNumber, message, reply);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AInit

void TScheduleApplication::DoMakeDependencySpace()		// OVERRIDE
{
	// Since we rely on dependents of dependents being notified of changes,
	// we need a standard dependency space.
#ifdef MACAPP_R_13
	CDependencySpace_AC::fgDefaultDependencySpace = TH_new CStandardDependencySpace_AC;
#else
	TStandardDependencySpace* aSpace = new TStandardDependencySpace;
	aSpace->IStandardDependencySpace();
	gMacAppDependencies = aSpace;
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

TDocument* TScheduleApplication::DoMakeDocument(CommandNumber /*itsCommandNumber*/,
					TFile* itsFile)		// OVERRIDE
{
	// Creates the document for this application.

	return TH_new TScheduleDocument(itsFile);
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

void TScheduleApplication::DoMenuCommand(CommandNumber aCommandNumber)	// OVERRIDE
{
	// Handles global "Lookup" menu item

	switch (aCommandNumber)
	{
	case cLaunchLookup:
		{
			this->PostCommand(TH_new TLookupLauncher(aCommandNumber));
		}
		break;

	default:
		Inherited::DoMenuCommand(aCommandNumber);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleApplication::DoSetupMenus()		// OVERRIDE
{
	// Enables global "Lookup" menu item

	Inherited::DoSetupMenus();
	Enable(cLaunchLookup, !MemSpaceIsLow());
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TScheduleApplication::IsAppointmentAvailable(DayIndex theDay, TimeIndex theTime,
									TimeIndex& theDuration, CStr255& documentTitle)
{
	// Finds first document which can handle the appointment. If none can, returns
	// duration = 0 and empty document title.

	theDuration		= 0;				// init the return values, in case no document handles it.
	documentTitle	= gEmptyString;

	CDocumentIterator iter(this);		// lookup the appointment in all documents
	for (TDocument* aDocument = iter.FirstDocument(); iter.More(); aDocument = iter.NextDocument())
	{
		if (((TScheduleDocument*)aDocument)->IsAppointmentAvailable(theDay, theTime, theDuration))
		{
			aDocument->GetTitle(documentTitle);
			return;
		}
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AClipboard

TView* TScheduleApplication::MakeViewForAlienClipboard()	// OVERRIDE
{
	// override to create custom clipboard for this application's scrap data.

	long offset;

	if (::GetScrap(NULL, kAppointmentScrapType, &offset) >= 0)
	{
		MAVolatileInit(TScheduleDocument*, clipDoc, TH_new TScheduleDocument(NULL));
			// create the clip document
			// initialize it.

		TView* clipView = NULL;								// the result.

		TRY
		{

			// establish a context for the existance of the CTemprorayHandleStream
			{
				CTemporaryHandleStream theScrap(kStr255Len + kLengthByte);

				{
					CPermAllocator alloc(true);

					// get the scrap data using permanent memory.
					long err = ::GetScrap(theScrap, kAppointmentScrapType, &offset);
					if (err < 0)								// error only if negative.
						FailOSErr((OSErr)err);					// report the error
				}

				theScrap.AdjustStreamSize();					// synch stream size to handle size.

				clipDoc->ReadClipboardData(theScrap);			// read the scrap into the clip document
			}

			clipView = clipDoc->MakeClipboardView();			// create the clipboard view.
		}
		CATCH_ALL
		{
			clipDoc = (TScheduleDocument*)FreeIfObject(clipDoc);
			THROW;
		}
		ENDTRY;
		
		return clipView;
	}
	else
		return Inherited::MakeViewForAlienClipboard();
}
