/*
	UScheduleBehavior.h

	Copyright 1991-1995 Apple Computer, Inc.
	All rights reserved

	Change History:

	10/5/91		LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
 */

#ifndef __USCHEDULEBEHAVIOR__
#define __USCHEDULEBEHAVIOR__

#ifndef __UBEHAVIOR__
#include <UBehavior.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TScheduleView;

//----------------------------------------------------------------------------------------
// CLASS TScheduleViewBehavior
//----------------------------------------------------------------------------------------

class TScheduleViewBehavior : public TBehavior
{
	DeclareClass(TScheduleViewBehavior);

public:
	TScheduleViewBehavior();
	virtual TScheduleView* GetScheduleView();
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentSelector
//----------------------------------------------------------------------------------------

class TAppointmentSelector : public TScheduleViewBehavior
{
	DeclareClass(TAppointmentSelector);

public:

	// Mouse handling
	virtual bool DoMouseCommand(VPoint& theMouse,
									  TToolboxEvent* event,
									  CPoint hysteresis);		// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentDragLauncher
//----------------------------------------------------------------------------------------

class TAppointmentDragLauncher : public TScheduleViewBehavior
{
	DeclareClass(TAppointmentDragLauncher);

public:

	// Mouse handling
	virtual bool DoMouseCommand(VPoint& theMouse,
									  TToolboxEvent* event,
									  CPoint hysteresis);		// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentStretchLauncher
//----------------------------------------------------------------------------------------

class TAppointmentStretchLauncher : public TScheduleViewBehavior
{
	DeclareClass(TAppointmentStretchLauncher);

public:

	// Mouse handling
	virtual bool DoMouseCommand(VPoint& theMouse,
									  TToolboxEvent* event,
									  CPoint hysteresis);		// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentChangeLauncher
//----------------------------------------------------------------------------------------

class TAppointmentChangeLauncher : public TScheduleViewBehavior
{
	DeclareClass(TAppointmentChangeLauncher);

public:

	// Mouse handling
	virtual bool DoMouseCommand(VPoint& theMouse,
									  TToolboxEvent* event,
									  CPoint hysteresis);		// OVERRIDE
	// Command operations
	virtual void DoMenuCommand(CommandNumber aCommandNumber); 							// OVERRIDE
	virtual void DoSetupMenus(); 														// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentSketchLauncher
//----------------------------------------------------------------------------------------

class TAppointmentSketchLauncher : public TScheduleViewBehavior
{
	DeclareClass(TAppointmentSketchLauncher);

public:

	// Mouse handling
	virtual bool DoMouseCommand(VPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint hysteresis);		// OVERRIDE
	// Command launching
	virtual void LaunchSketcher(TScheduleView* itsView, const VPoint theMouse);
};

//----------------------------------------------------------------------------------------
// CLASS TSubscriberSketchLauncher
//----------------------------------------------------------------------------------------

class TSubscriberSketchLauncher : public TAppointmentSketchLauncher
{
	DeclareClass(TSubscriberSketchLauncher);

public:

	// Command launching
	virtual void LaunchSketcher(TScheduleView* itsView,
										const VPoint theMouse);		// OVERRIDE

};

//----------------------------------------------------------------------------------------
// CLASS TScheduleEditLauncher
//----------------------------------------------------------------------------------------

class TScheduleEditLauncher : public TScheduleViewBehavior
{
	DeclareClass(TScheduleEditLauncher);

public:

	// Command operations
	virtual void DoKeyEvent(TToolboxEvent* event);				// OVERRIDE
	virtual void DoMenuCommand(CommandNumber aCommandNumber);	// OVERRIDE
	virtual void DoSetupMenus(); 								// OVERRIDE

	// Menu operations
	virtual void SetupEditMenu();
};

//----------------------------------------------------------------------------------------
// CLASS TSubscriberEditLauncher
//----------------------------------------------------------------------------------------

class TSubscriberEditLauncher : public TScheduleEditLauncher
{
	DeclareClass(TSubscriberEditLauncher);

public:

	// Menu operations
	virtual void SetupEditMenu(); 								// OVERRIDE
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

extern void InitUScheduleBehavior();
	// Call this routine at initialization time

#endif
