/*
	UScheduleCommand.h

	Copyright 1991-1995 Apple Computer, Inc.
	All rights reserved

	Change History:

	9/23/91		LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
*/

#ifndef __USCHEDULECOMMAND__
#define __USCHEDULECOMMAND__

#ifndef __USCHEDULEOBJECTS__
#include "UScheduleObjects.h"
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TScheduleView;
class TScheduleDocument;

//----------------------------------------------------------------------------------------
// Global constants
//----------------------------------------------------------------------------------------

/* Command numbers */

const CommandNumber cAddAppointment			= 1001;
const CommandNumber cChangeAppointment		= 1002;
const CommandNumber cDeleteAppointment		= 1003;
const CommandNumber cDragAppointment		= 1004;
const CommandNumber cStretchAppointment		= 1005;

const CommandNumber	cPreTrack				= 1051;
const CommandNumber cPostTrack				= 1052;

const CommandNumber cLookupAppointment		= 1101;
const CommandNumber cLaunchLookup			= 1102;

// Dialog 'Views'

const ResNumber phAboutBox					= 1201;
const ResNumber phGetDescription			= 1202;
const ResNumber phLookupDialog				= 1203;

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TScheduleDocument;
class TScheduleView;
class TSubscriber;
class TLookupDialog;
class TAppleEvent;

//----------------------------------------------------------------------------------------
// CLASS TAppointmentEditCommand
//----------------------------------------------------------------------------------------

// This is the base class for all appointment editing commands.

class TAppointmentEditCommand : public TCommand
{

protected:

	TScheduleView*			fScheduleView;
	TScheduleDocument*		fScheduleDocument;
	TAppointmentList*		fClipAppointments;

public:

	// Construction/Destruction
	virtual ~TAppointmentEditCommand();

	TAppointmentEditCommand(CommandNumber aCommandNumber, TScheduleView* itsView,
				bool causesChange, bool changesClipboard);

	// Helper methods
	virtual void GetClipAppointments();
	virtual void RemoveClipAppointments();
	virtual void RestoreClipAppointments();
	virtual bool ShouldFreeClipAppointments();
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentCutCopyClearCommand
//----------------------------------------------------------------------------------------

class TAppointmentCutCopyClearCommand : public TAppointmentEditCommand
{

public:

	// Construction/Destruction
	TAppointmentCutCopyClearCommand(CommandNumber aCommandNumber, TScheduleView* itsView);

	// Action methods
	virtual void DoIt();							// OVERRIDE
	virtual void RedoIt();							// OVERRIDE
	virtual void UndoIt();							// OVERRIDE

	// Helper methods
	virtual void GetClipAppointments();				// OVERRIDE
	virtual void PutAppointmentsToClip();
	virtual bool ShouldFreeClipAppointments();	// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentEditCommand
//----------------------------------------------------------------------------------------

class TAppointmentPasteCommand : public TAppointmentEditCommand
{

public:

	// Construction/Destruction
	TAppointmentPasteCommand(CommandNumber aCommandNumber, TScheduleView* itsView);

	// Action methods
	virtual void DoIt();							// OVERRIDE
	virtual void RedoIt();							// OVERRIDE
	virtual void UndoIt();							// OVERRIDE

	// Helper methods
	virtual void GetClipAppointments();				// OVERRIDE
	virtual bool ShouldFreeClipAppointments();	// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentAdder: Adds a appointment to the document.
//----------------------------------------------------------------------------------------

class TAppointmentAdder : public TCommand
{

private:
	TScheduleView*		fScheduleView;
	TScheduleDocument*	fScheduleDocument;
	TAppointment*		fAppointment;

	DayIndex			fDay;
	TimeIndex			fTime;
	TimeIndex			fDuration;
	CStr255				fDescription;

public:
	// Construction/Destruction
	virtual ~TAppointmentAdder();										// OVERRIDE
	TAppointmentAdder(CommandNumber aCommandNumber, TScheduleView* itsView,
					DayIndex itsDay, TimeIndex itsTime, TimeIndex itsDuration,
					CStr255& itsDescription);

	// Command operations
	virtual void DoIt();										// OVERRIDE
	virtual void RedoIt();										// OVERRIDE
	virtual void UndoIt();										// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentChanger: Changes the description of a single appointment.
//----------------------------------------------------------------------------------------

class TAppointmentChanger : public TCommand
{

private:
	TAppointment*	fAppointment;

	CStr255			fOriginalDescription;
	CStr255			fNewDescription;

public:
	// Construction/Destruction
	TAppointmentChanger(TScheduleView* itsView, TAppointment* anAppointment);

	// Command operations
	virtual void DoIt();										// OVERRIDE
	virtual void RedoIt();										// OVERRIDE
	virtual void UndoIt();										// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TLookupLauncher: Launches the Lookup dialog.
//----------------------------------------------------------------------------------------

class TLookupLauncher	: public TCommand
{

public:
	// Construction/Destruction
	TLookupLauncher(CommandNumber aCommandNumber);

	// Command operations
	virtual void DoIt();										// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS Client command looks up appointment in remote dictionary.
//----------------------------------------------------------------------------------------

class TScheduleClientCommand : public TClientCommand
{

private:
	DayIndex	fDay;
	TimeIndex	fTime;
	TimeIndex	fDuration;
	CStr255		fDocumentTitle;

public:

	// Construction
	TScheduleClientCommand(CommandNumber itsCommandNumber,
										TLookupDialog* itsView,
										const DayIndex theDay, const TimeIndex theTime,
										const AEAddressDesc& itsAddress);

	// Command execution
	virtual void DoIt();									// OVERRIDE
	virtual void ProcessReply(TAppleEvent* theReply);		// OVERRIDE
};

//----------------------------------------------------------------------------------------
//  Server command passes the request to the application and returns response.
//----------------------------------------------------------------------------------------

class TScheduleServerCommand : public TServerCommand
{

public:
	TScheduleServerCommand(CommandNumber itsCommandNumber,
						APPLEEVENTPTR itsMessage,
						APPLEEVENTPTR itsReply);

	// Command execution
	virtual void	DoIt();										//	OVERRIDE
};

#endif
