/*
	UScheduleDocument.h

	Copyright 1991-1995 Apple Computer, Inc.
	All rights reserved

	Change History:

	9/23/91		LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
 */

#ifndef __USCHEDULEDOCUMENT__
#define __USCHEDULEDOCUMENT__

#ifndef __USCHEDULEOBJECTS__
#include "UScheduleObjects.h"
#endif

#ifndef __UEDITIONDOCUMENT__
#include <UEditionDocument.h>
#endif

//----------------------------------------------------------------------------------------
// Gloabal Constants
//----------------------------------------------------------------------------------------

/* Finder constants */

const OSType	kSignature				= 'SDUL';

const OSType	kDocumentFileType		= 'SKED';
const OSType	kStationeryFileType		= 'sKED';
const OSType	kAppointmentScrapType	= 'APPT';

const OSType 	kTextScrapType 			= 'TEXT';

// Booleans for GetAppointmentList

const bool	kSubscribers			= true;
const bool 	kAppointments			= !kSubscribers;

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TScheduleView;
class TDayView;
class TSubscriberDayView;
class TPrimaryScroller;

//----------------------------------------------------------------------------------------
// CLASS TScheduleDocument
//----------------------------------------------------------------------------------------

class TScheduleDocument : public TEditionDocument
{
private:
	TAppointmentList*		fAppointments;
	TAppointmentList*		fSubscribedAppointments;

	TScheduleView*  		fScheduleView;
	TScheduleView*			fSubscriberView;

	// for saving document state.
	TWindow*				fScheduleWindow;
	TPrimaryScroller*		fPrimaryScroller;
	bool					fReopening;				// indicates whether to position info is valid.
	VRect					fSavedWindowPosition;
	VPoint					fSavedScrollPosition;

	TDayView*				fDayView;				// view that changes user selection
	TSubscriberDayView*		fSubscriberDayView;

	CStr255					fSubscriberName;

public:

	// Construction/Destruction operations
	virtual void DoInitialState(); 								// OVERRIDE
	virtual ~TScheduleDocument();										// OVERRIDE
	virtual void FreeData();									// OVERRIDE

	TScheduleDocument(TFile* itsFile);

	// File operations
	virtual void DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes);	// OVERRIDE
	virtual void DoRead(TFile* aFile, bool forPrinting);		// OVERRIDE
	virtual void DoWrite(TFile* aFile, bool makingCopy);		// OVERRIDE

	// Stream operations

	virtual void DoReadData(const OSType aScrapType,
						TDesignator* aDesignator,
						TStream* aStream, long length);					// OVERRIDE

	virtual void DoWriteData(const OSType aScrapType,
						TDesignator* aDesignator, TStream* aStream);	// OVERRIDE

	virtual bool ReadAppointment(TStream* aStream, TAppointment*& anAppointment);
	virtual void ReadDocumentData(TStream* aStream);
	virtual void ReadDocumentState(TStream* aStream);
	virtual void WriteDocumentData(TStream* aStream);
	virtual void WriteDocumentState(TStream* aStream);

	// Appointment operations
	virtual void	AddAppointment(TAppointment* anAppointment);
	virtual void DeleteAppointment(TAppointment* anAppointment);

	// AppleEvent support
	virtual bool IsAppointmentAvailable(DayIndex theDay, TimeIndex theTime, TimeIndex& theDuration);

	// View operations
	virtual void DoMakeViews(bool forPrinting);				// OVERRIDE
	virtual TView* MakeClipboardView();
	virtual void SetupTheWindow();
	virtual void ShowReverted();								// OVERRIDE

	// Clipboard operations
	virtual void ReadClipboardData(TStream* aStream);
	virtual void WriteDocumentScrap();
	virtual void WriteTextScrap();
	virtual void WriteToDeskScrap();							// OVERRIDE

	// User Selection
	virtual void UserSelectionChanged(TView* changedView);		// OVERRIDE

	// Edition Manager Support
	virtual void	AddSubscribedAppointment(TAppointment* anAppointment);
	virtual void DeleteSubscribedAppointment(TAppointment* anAppointment);
	virtual void DeleteSubscribedAppointments();
	virtual TSection* GetSelectedSection();						// OVERRIDE

	virtual void RevealSelection(TDesignator* aSelection);		// OVERRIDE

	virtual bool CanPublishSelection();						// OVERRIDE
	virtual bool CanSubscribe();								// OVERRIDE

	virtual void DoAddBorder(TSection* aSection);				// OVERRIDE
	virtual void DoDeleteBorder(TSection* aSection);			// OVERRIDE

	virtual TSubscriber* GetTheSubscriber();

	virtual void AddSection(TSection* aSection);				// OVERRIDE
	virtual void RemoveSection(TSection* aSection);				// OVERRIDE

	// Accessors
	virtual TAppointmentList* GetAppointmentList(bool isSubscriber);
	virtual TScheduleView* GetScheduleView();
	virtual void GetSubscriberName(CStr255& itsName);
	virtual void SetSubscriberName(ConstCStr255Param itsName);
	
	TDayDesignator* GetDayUserSelection();
};

extern TDayDesignator* GetDayDesignator(TSection* aSection);

#endif
