//----------------------------------------------------------------------------------------
// UScheduleObjects.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __USCHEDULEOBJECTS__
#include "UScheduleObjects.h"
#endif

#ifndef __USCHEDULEDOCUMENT__
#include "UScheduleDocument.h"
#endif

#ifndef __USCHEDULEVIEW__
#include "UScheduleView.h"
#endif

#ifndef __USTREAM__
#include <UStream.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

//----------------------------------------------------------------------------------------

// Buzz string list

const ResNumber kScheduleBuzz 		= 1011;
const short		kAppointmentAsText	= 1;
const short		kDuration			= 2;

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

CStr255 GetDay(const DayIndex theDayIndex)
{
	// Obtains name of day by consulting international resources.
	CStr255 dayString;
	Assertion(theDayIndex >= 1 && theDayIndex <= kMaxNumberOfDays, "Range check in GetDay");

	Intl1Hndl aHandle = (Intl1Hndl)::GetIntlResource(1);
	if (aHandle)
		::BlockMove(&(*aHandle)->days[theDayIndex - 1], &dayString, (*aHandle)->days[theDayIndex - 1][0] + kLengthByte);
	else
		dayString = gEmptyString;
	return dayString;
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

CStr255 GetDuration(const TimeIndex theDuration)
{
	long hours			= (long)(theDuration / kIntervalsPerHour);
	long minutes		= (theDuration - hours * kIntervalsPerHour) * kMinutesPerInterval;

	CStr255 durationString;
	CStr255 aString;

	::NumToString(hours, aString);
	MAParamText("^Hours", aString);

	::NumToString(minutes, aString);
	MAParamText("^Minutes", aString);

	::GetIndString(durationString, kScheduleBuzz, kDuration);
	MAReplaceText(durationString);

	return durationString;
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

CStr255 GetTime(const TimeIndex theTimeIndex)
{
	// Obtains time by consulting international resources.
	CStr255 timeString;
	::IUTimeString((theTimeIndex - 1) * kSecondsPerInterval, false /*wantSeconds*/, timeString);
	return timeString;
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

CStr255 GetUserName()
{
	// Gets Chooser name of user by examining a System resource.

	const ResNumber kChooserNameID = -16096;
	CStr255 userName;
	
	ResNumber oldResFile = MAUseResFile(0);
	String255Handle chooserName = (String255Handle)::GetString(kChooserNameID);
	MAUseResFile(oldResFile);

	if (chooserName && *chooserName)
		CopyStr255(**chooserName, (Ptr) & userName);
	else
		userName = gEmptyString;
	return userName;
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

TAppointmentList* NewAppointmentList()
{
	// Convenience method that returns a fresh appointment list.

	TAppointmentList* aList = new TAppointmentList;
	aList->IAppointmentList();
#if qDebug && !defined(MACAPP_R_13)
		aList->SetEltType("TAppointment");
#endif
	return aList;
}

//----------------------------------------------------------------------------------------
// CLASS TAppointment
//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

#undef Inherited
#define Inherited TObject
DefineClass(TAppointment, Inherited);

void TAppointment::ChangeTo(TAppointment* anAppointment)
{
	// Sets data members to match anAppointment.

	fDay 			= anAppointment->fDay;
	fTime			= anAppointment->fTime;
	fDuration		= anAppointment->fDuration;
	this->SetDescription(anAppointment->fDescription);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CompareResult TAppointment::CompareTo(CONST TAppointment* testItem) CONST
{
	// Compares myself to testItem.

	if (this->fDay == testItem->fDay)
	{
		if (this->fTime == testItem->fTime)
		{
			if (this->fDuration == testItem->fDuration)
				return kItem1EqualItem2;
			else
				return (this->fDuration > testItem->fDuration ? kItem1GreaterThanItem2 : kItem1LessThanItem2);
		}
		else
			return (this->fTime > testItem->fTime ? kItem1GreaterThanItem2 : kItem1LessThanItem2);
	}
	else
		return (this->fDay > testItem->fDay ? kItem1GreaterThanItem2 : kItem1LessThanItem2);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TAppointment::GetAppointmentInfo(DayIndex& theDay, TimeIndex& theTime,
												TimeIndex& theDuration)
{
	// Accessor function.

	theDay		= fDay;
	theTime		= fTime;
	theDuration	= fDuration;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CStr255 TAppointment::GetDescription() const
{
	// Accessor function.
	return fDescription;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment::TAppointment() :
	fDay(0),
	fTime(0),
	fDuration(0)
{
#ifndef MACAPP_R_13
	this->IObject();
#endif	
	this->SetDescription(gEmptyString);
	fIsSelected = false;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment::TAppointment(const DayIndex itsDay,
		const TimeIndex itsTime, const TimeIndex itsDuration,
		ConstCStr255Param itsDescription) :
	fDay(itsDay),
	fTime(itsTime),
	fDuration(itsDuration)
{
#ifndef MACAPP_R_13
	this->IObject();
#endif	
	
	this->SetDescription(itsDescription);
	fIsSelected = false;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

bool TAppointment::InvertSelect()
{
	// Accessor function.

	return fIsSelected = !fIsSelected;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

bool TAppointment::IsSelected()
{
	// Accessor function.

	return fIsSelected;
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

void TAppointment::ReadFrom(TStream* aStream) // OVERRIDE
{
	// Reconstructs data members from stream.

	Inherited::ReadFrom(aStream);

	fDay			= aStream->ReadInteger();
	fTime			= aStream->ReadInteger();
	fDuration		= aStream->ReadInteger();
	
	CStr255 description;
	aStream->ReadString(description, sizeof(CStr255));
	this->SetDescription(description);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TAppointment::SetAppointmentInfo(const DayIndex& theDay, const TimeIndex& theTime,
												const TimeIndex& theDuration)
{
	// Accessor function.

	fDay		= theDay;
	fTime		= theTime;
	fDuration	= theDuration;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TAppointment::SetDescription(ConstCStr255Param itsDescription)
{
	// Accessor function.

	fDescription = itsDescription;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

bool TAppointment::SetSelect(bool select)
{
	// Accessor function.

	return fIsSelected = select;
}

//----------------------------------------------------------------------------------------
#pragma segment AWriteFile

void TAppointment::WriteAsTextTo(TStream* aStream) const
{
	// Writes appointment information in text format to aStream.

	CStr255 aString(GetDay(fDay));
	MAParamText("^Day", aString);

	aString = GetTime(fTime);
	MAParamText("^Time", aString);

	aString = GetDuration(fDuration);
	MAParamText("^Duration", aString);

	MAParamText("^Description", fDescription);

	::GetIndString(aString, kScheduleBuzz, kAppointmentAsText);
	MAReplaceText(aString);

	aStream->WriteBytes(&aString[1], aString.Length());
}

//----------------------------------------------------------------------------------------
#pragma segment AWriteFile

void TAppointment::WriteTo(TStream* aStream) CONST // OVERRIDE
{
	// Writes self to stream.

	Inherited::WriteTo(aStream);

	aStream->WriteInteger(fDay);
	aStream->WriteInteger(fTime);
	aStream->WriteInteger(fDuration);

	aStream->WriteString(this->GetDescription());
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentList
//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

#undef Inherited
#define Inherited TSortedList
DefineClass(TAppointmentList, Inherited);

TAppointment* TAppointmentList::AppointmentAt(const ArrayIndex index)
{
	// Convenient accessor method.

	return (TAppointment*)this->At(index);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CompareResult TAppointmentList::Compare(CONST COMPAREOBJ* item1, CONST COMPAREOBJ* item2)	CONST // OVERRIDE
{
	// Sorted lists must override the compare method.
	// Passes compare call to the appointment's envolved.

	return (reinterpret_cast<CONST TAppointment*> (item1)->CompareTo(reinterpret_cast<CONST TAppointment*> (item2)));	 // pass the call onto the first appointment
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TAppointmentList::IAppointmentList()
{
#ifndef MACAPP_R_13
	this->ISortedList();
#endif
}

//----------------------------------------------------------------------------------------
// CLASS CAppointmentIterator
//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CAppointmentIterator::CAppointmentIterator(TScheduleDocument* theScheduleDocument) :
	CArrayIterator(theScheduleDocument->GetAppointmentList(kAppointments))
{
	// This constructor passes the appropriate parameter to its base class's constructor.
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CAppointmentIterator::CAppointmentIterator(TAppointmentList* theList) :
	CArrayIterator(theList)
{
	// This constructor passes the appropriate parameter to its base class's constructor.
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CAppointmentIterator::CAppointmentIterator(TScheduleView* theSchedule) :
	CArrayIterator(theSchedule->GetScheduleDocument()->
			GetAppointmentList(theSchedule->fIdentifier == kSubscriberID))
{
	// This constructor passes the appropriate parameter to its base class's constructor,
	// based on the view passed as a parameter. Subscriber views will return subscriber appointment list.
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CAppointmentIterator::CAppointmentIterator(TScheduleView* theSchedule, bool itsForward) :
	CArrayIterator(theSchedule->GetScheduleDocument()->
			GetAppointmentList(theSchedule->fIdentifier == kSubscriberID), itsForward)
{
	// This constructor passes the appropriate parameter to its base class's constructor,
	// based on the view passed as a parameter. Subscriber views will return subscriber appointment list.
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment* CAppointmentIterator::CurrentAppointment()
{
	// Convenient accessor method.

	return ((TAppointmentList*)fDynamicArray)->AppointmentAt(fCurrentIndex);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment* CAppointmentIterator::FirstAppointment()
{
	// Uses the accessor method to return the appointment.

#ifdef MACAPP_R_13
	this->FirstIndex();
#else
	this->Reset();
#endif
	if (this->More())
		return this->CurrentAppointment();
	else
		return NULL;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment* CAppointmentIterator::NextAppointment()
{
	// Uses the accessor method to return the appointment.

	this->Advance();
	if (this->More())
		return this->CurrentAppointment();
	else
		return NULL;
}

//----------------------------------------------------------------------------------------
// CLASS CSubscribedAppointmentIterator
//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CSubscribedAppointmentIterator::CSubscribedAppointmentIterator(TScheduleDocument* theScheduleDocument) :
	CAppointmentIterator(theScheduleDocument->GetAppointmentList(kSubscribers))
{
	// Passes the document's subscriber list to the base class's constructor.
}

//----------------------------------------------------------------------------------------
// CLASS CSelectedAppointmentIterator
//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CSelectedAppointmentIterator::CSelectedAppointmentIterator(TScheduleView* theScheduleView) :
	CAppointmentIterator(theScheduleView)
{
	// uses the view's appointment list.
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CSelectedAppointmentIterator::CSelectedAppointmentIterator(TScheduleView* theScheduleView, bool itsForward) :
	CAppointmentIterator(theScheduleView, itsForward)
{
	// uses the view's appointment list.
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

CSelectedAppointmentIterator::CSelectedAppointmentIterator(TScheduleDocument* theScheduleDocument) :
	CAppointmentIterator(theScheduleDocument)
{
	// uses the main appointment list.
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void CSelectedAppointmentIterator::Advance()		// OVERRIDE
{
	// Advances until a selected appointment is found, if any.

	do
		CAppointmentIterator::Advance();
	while (this->More() && !this->CurrentAppointment()->IsSelected());
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void CSelectedAppointmentIterator::Reset()		// OVERRIDE
{
	// Resets. If the first appointment, if any, is not selected, advances to the
	// first selected appointment.
#ifdef MACAPP_R_13
	CAppointmentIterator::FirstIndex();
#else
	CAppointmentIterator::Reset();
#endif
	if (this->More() && !this->CurrentAppointment()->IsSelected())
		this->Advance();
}

//----------------------------------------------------------------------------------------
// CLASS TDayDesignator
//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

#undef Inherited
#define Inherited TRegionDesignator
DefineClass(TDayDesignator, Inherited);

bool TDayDesignator::Contains(TAppointment* anAppointment)
{
	// Checks to see if anAppointment is contained in this designator.
	// The fDesignation is a region whose points correspond to grid cells.

	DayIndex 	itsDay;
	TimeIndex	itsTime;
	TimeIndex	itsDuration;
	anAppointment->GetAppointmentInfo(itsDay, itsTime, itsDuration);

	CPoint topLeftPt(itsDay, itsTime);
	CPoint botRightPt = topLeftPt + CPoint(1, itsDuration);

	return ::RectInRgn(CRect(topLeftPt, botRightPt), fDesignation);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

DayIndex TDayDesignator::GetDay()
{
	// Returns the day of this designation.

	return (*fDesignation)->rgnBBox.left;
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

TDayDesignator::TDayDesignator()
#ifdef MACAPP_R_13
	: TRegionDesignator(NULL, true /* freeRegionOnFree */)
#endif
{
#ifndef MACAPP_R_13
	IRegionDesignator(NULL, true /* freeRegionOnFree */);
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TDayDesignator::SelectDay(TDayView* itsView)
{
	// Sets up fDesignation based on itsView (a TDayView).

	if (itsView->IsAnyCellSelected())
	{
		DayIndex theDay = GRIDVIEWDESIGNATOR_GETBOUNDS(itsView->fSelections).left;

		::SetRectRgn(fDesignation, theDay, 1, theDay + 1,
			itsView->GetScheduleDocument()->GetScheduleView()->fNumOfRows + 1);
	}
	else
		::SetEmptyRgn(fDesignation);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TDayDesignator::SetDay(const DayIndex theDay)
{
	// Sets the designation to the day. Called when reading a subscriber.

	::SetRectRgn(fDesignation, theDay, 1, theDay + 1, kMaxNumberOfIntervals);
}

