/*
	UScheduleObjects.h

	Copyright 1991-1995 Apple Computer, Inc.
	All rights reserved

	Change History:

	9/23/91		LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
 */

#ifndef __USCHEDULEOBJECTS__
#define __USCHEDULEOBJECTS__

#ifndef __UITERATOR__
#include <UIterator.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif

#ifndef __PASCALSTRING__
#include "PascalString.h"
#endif

#ifndef __ULIST__
#include <UList.h>
#endif

#ifndef __ULISTITERATOR__
#include <UListIterator.h>
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

//----------------------------------------------------------------------------------------
// Global Typedefs
//----------------------------------------------------------------------------------------

typedef short DayIndex;				// 1..7 = Sunday..Saturday
typedef short TimeIndex;			// 1..(24 * 4) = quarter hour segments

//----------------------------------------------------------------------------------------
// Global Constansts
//----------------------------------------------------------------------------------------

enum {kSun = 1, kMon, kTue, kWed, kThu, kFri, kSat};

const short	kMinutesPerInterval		= 15;
const short	kSecondsPerInterval		= kMinutesPerInterval * 60;
const short kIntervalsPerHour		= 60 / kMinutesPerInterval;

const short kHoursInADay			= 24;
const short kMaxNumberOfIntervals 	= kHoursInADay * kIntervalsPerHour;

const short kMaxNumberOfDays		= 7;

const short kInitialDay				= kMon;								// Monday
const short kInitialTime			= (8 * kIntervalsPerHour) + 1;		// 8 AM.

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TScheduleDocument;
class TScheduleView;
class TSubscriberView;
//class TStream;
class TDayView;

//----------------------------------------------------------------------------------------
// CLASS TAppointment
//----------------------------------------------------------------------------------------

class TAppointment : public TObject
{
	DeclareClass(TAppointment);

private:
	DayIndex			fDay;
	TimeIndex			fTime;
	TimeIndex			fDuration;
	CStr255				fDescription;

	bool				fIsSelected;

public:
	// Construction/Destruction
	TAppointment();
	
	TAppointment(const DayIndex itsDay,
		const TimeIndex itsTime, const TimeIndex itsDuration,
		ConstCStr255Param itsDescription);

	// Accessors
	virtual void ChangeTo(TAppointment* anAppointment);

	virtual CStr255 GetDescription() const;
	virtual void SetDescription(ConstCStr255Param itsDescription);

	virtual void GetAppointmentInfo(DayIndex& theDay, TimeIndex& theTime,
												TimeIndex& theDuration);
	virtual void SetAppointmentInfo(const DayIndex& theDay, const TimeIndex& theTime,
												const TimeIndex& theDuration);

	// Selection accessors
	virtual bool InvertSelect();				// returns new setting
	virtual bool IsSelected();				// returns current setting
	virtual bool SetSelect(bool select);	// returns new setting

	// Stream operations
	virtual void ReadFrom(TStream* aStream);							// OVERRIDE

	virtual void WriteAsTextTo(TStream* aStream) const;
	virtual void WriteTo(TStream* aStream) CONST;							// OVERRIDE

	// Comparison
	virtual CompareResult CompareTo(CONST TAppointment* testItem) CONST;
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentList: knows how to compare appointments.
//----------------------------------------------------------------------------------------

class TAppointmentList : public TSortedList
#ifdef MACAPP_R_13
, public MDependable_AC
#endif
{
	DeclareClass(TAppointmentList);

public:
	// Construction/Destruction
	virtual void IAppointmentList();

	// TSortedList required override
	virtual CompareResult Compare(CONST COMPAREOBJ* item1, CONST COMPAREOBJ* item2) CONST;	// OVERRIDE

	// Convenient accessor method
	virtual TAppointment* AppointmentAt(const ArrayIndex index);
};

// Convenient allocation method

extern TAppointmentList* NewAppointmentList();

//----------------------------------------------------------------------------------------
// CAppointmentIterator
//----------------------------------------------------------------------------------------

class CAppointmentIterator : public CArrayIterator
{
public:
	CAppointmentIterator(TScheduleDocument* theScheduleDocument);
	CAppointmentIterator(TAppointmentList* theList);

	CAppointmentIterator(TScheduleView* theSchedule);
	CAppointmentIterator(TScheduleView* theSchedule, bool itsForward);

	virtual TAppointment* CurrentAppointment();
	virtual TAppointment* FirstAppointment();
	virtual TAppointment* NextAppointment();
};

//----------------------------------------------------------------------------------------
// CSubscribedAppointmentIterator
//----------------------------------------------------------------------------------------

class CSubscribedAppointmentIterator : public CAppointmentIterator
{
public:
	CSubscribedAppointmentIterator(TScheduleDocument* theScheduleDocument);
};

//----------------------------------------------------------------------------------------
// CSelectedAppointmentIterator
//----------------------------------------------------------------------------------------

class CSelectedAppointmentIterator : public CAppointmentIterator
{
public:
	CSelectedAppointmentIterator(TScheduleView* theScheduleView);
	CSelectedAppointmentIterator(TScheduleView* theScheduleView, bool itsForward);
	CSelectedAppointmentIterator(TScheduleDocument* theScheduleDocument);

	virtual void Advance();		// OVERRIDE
	virtual void Reset();		// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TDayDesignator
//----------------------------------------------------------------------------------------

class TDayDesignator : public TRegionDesignator
{
	DeclareClass(TDayDesignator);

public:

	// Construction/Destruction
	TDayDesignator();

	// Containment
	virtual bool Contains(TAppointment* anAppointment);	// OVERRIDE

	// Accessors
	virtual void SelectDay(TDayView* itsView);

	virtual DayIndex GetDay();
	virtual void SetDay(const DayIndex theDay);
};

//----------------------------------------------------------------------------------------
// Date/Time utilities
//----------------------------------------------------------------------------------------

extern CStr255 GetDay(const DayIndex theDayIndex);
extern CStr255 GetDuration(const TimeIndex theDuration);
extern CStr255 GetTime(const TimeIndex theTimeIndex);

//----------------------------------------------------------------------------------------
// User Name
//----------------------------------------------------------------------------------------

extern CStr255 GetUserName();

#endif
