//----------------------------------------------------------------------------------------
// USchedulePrinting.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __USCHEDULEPRINTING__
#include "USchedulePrinting.h"
#endif

#ifndef __USCHEDULEDOCUMENT__
#include "UScheduleDocument.h"
#endif

#ifndef __USCHEDULEOBJECTS__
#include "UScheduleObjects.h"
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __UDIALOG__
#include <UDialog.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __UDEBUG__
#include <UDebug.h>
#endif

#ifndef __UFAILURE__
#include <UFailure.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

#ifndef __PRINTING__
#include <Printing.h>
#endif

#ifdef MACAPP_R_13
#define GETNUMROWHEIGHTS() (fRowHeights.GetSize())	
#define GETNUMCOLWIDTHS() (fColWidths.GetSize())	
#define DELETECOLWIDTHS() (fColWidths.DeleteItems(1, fNumOfCols)) 
#define INSERTCOLWIDTHS() (fColWidths.InsertItems(fNumOfCols + 1, 1, (*columnListHdl)[index]))
#else
#define GETNUMROWHEIGHTS() (fRowHeights->fNoOfChunks)	
#define GETNUMCOLWIDTHS() (fColWidths->fNoOfChunks)	
#define DELETECOLWIDTHS() (fColWidths->DeleteItems(1, fNumOfCols)) 
#define INSERTCOLWIDTHS() (fColWidths->InsertItems(fNumOfCols + 1, 1, (*columnListHdl)[index])) 
#endif

//----------------------------------------------------------------------------------------

const ResNumber kHeadingBuzzString			= 2001;

const ResType	kColumnListType				= 'COLW';
const ResNumber kColumnListID				= 2001;

//----------------------------------------------------------------------------------------
#pragma segment AInit

void InitUSchedulePrinting()
{
	// So the linker doesn't dead strip class info
	macroDontDeadStrip(TStringListView);
	macroDontDeadStrip(TSchedulePrintView);
}

//----------------------------------------------------------------------------------------
// CLASS TStringListView
//----------------------------------------------------------------------------------------
#pragma segment ANonRes

#undef Inherited
#define Inherited TTextGridView
DefineClass(TStringListView, Inherited);

VCoordinate TStringListView::DoBreakFollowing(VHSelect	vhs,
											  VCoordinate	previousBreak,
											  bool&	/*automatic*/ )	// OVERRIDE

{
	// Determines where page breaks occur for printing.
	// It is confusing because hSel does vertical break,
	// but that's the way MacApp's printing stuff is written.

	VCoordinate		thisBreak;
	VRect			extentRect;

	this->GetExtent(extentRect);
	switch (vhs)
	{
	case hSel:
		{
			// Break after a complete text line.
			short rowsPerPage = (short)(this->GetPrintHandler()->fViewPerPage.v / fLineHeight);
			thisBreak = previousBreak + (rowsPerPage * fLineHeight);
			break;
		}
	case vSel:
		{
			// Break on a complete column.
			short firstCol = 0;

			if (previousBreak == 0)
				firstCol = 1;
			else
			{
				VCoordinate	width = 0;
				do
				{
					firstCol++;
					width += this->GetColWidth(firstCol);
				} while (width < previousBreak);
			}

			VCoordinate totalWidth = 0;
			VCoordinate pageWidth = this->GetPrintHandler()->fViewPerPage.h;

			for (short col = firstCol; col <= fNumOfCols; col++)
			{
				VCoordinate	width	 = 0;
				width = this->GetColWidth(col);
				if (totalWidth + width <= pageWidth)
					totalWidth += width;
				else
				{
					thisBreak = previousBreak + totalWidth;
					break;
				}
			}
			if (thisBreak == previousBreak)
				thisBreak = extentRect.right;
			break;
		}
	}
	thisBreak = Min(thisBreak, extentRect[botRight][gOrthogonal[vhs]]);

	return thisBreak;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

CStr255 TStringListView::GetText(const GridCell& aCell) const
{
	// All TTextGridViews must override GetText. This version obtains
	// the text from the 'STR#' resource whose ID was passed in the user area
	// field of the 'View' template.
	CStr255 aString;
	::GetIndString(aString, (short)fUserArea, aCell.h);
	return aString;
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

void TStringListView::ReadFields(TStream* aStream)	// OVERRIDE
{
	// Allows the TTextGridView to have columns of different sizes.
	// The sizes are stored in a custom resource containing a list of shorts.
	// The first item in the list is the number of items in the list.
	// The type and ID of the list is hard-wired so all subclasses have the same column setup.
	// Someday, to be _really_ fancy, the column widths would be specified in a custom
	// 'View' template.

	typedef short ColumnList[8000], *ColumnListPtr, **ColumnListHandle;

	Inherited::ReadFields(aStream);

	MAVolatileInit(ColumnListHandle, columnListHdl, (ColumnListHandle)::GetResource(kColumnListType, kColumnListID));
	FailNILResource((Handle)columnListHdl);

	MAVolatileInit(SignedByte, saveState, LockHandleHigh((Handle)columnListHdl));

	TRY
	{
		DELETECOLWIDTHS();
		fNumOfCols = 0;

		short columnsNeeded = (*columnListHdl)[0];

		// work on fColWidths instead of InsCol, to save re-adjusting view each time.
		for (short index = 1; index <= columnsNeeded ; index++, fNumOfCols++)
			INSERTCOLWIDTHS();//fColWidths.InsertItems(fNumOfCols + 1, 1, (*columnListHdl)[index]);

		::HSetState((Handle)columnListHdl, saveState);
	}
	CATCH_ALL
	{
		::HSetState((Handle)columnListHdl, saveState);
		THROW;
	}
	ENDTRY;
}

//----------------------------------------------------------------------------------------
// TSchedulePrintView
//----------------------------------------------------------------------------------------
#pragma segment AOpen

#undef Inherited
#define Inherited TStringListView
DefineClass(TSchedulePrintView, Inherited);

void TSchedulePrintView::DoPostCreate(TDocument* itsDocument)			// OVERRIDE
{
	// Fills in data members.

	Inherited::DoPostCreate(itsDocument);

	fNextHandler = itsDocument;		// I'm not in a window, so my next handler is the document.

	fAppointments = ((TScheduleDocument*)itsDocument)->GetAppointmentList(kAppointments);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

#ifdef MACAPP_R_13
void TSchedulePrintView::DrawRangeOfCells(const GridCell& startCell,
										const GridCell& stopCell,
										const VRect& aRect)
#else
void TSchedulePrintView::DrawRangeOfCells(GridCell startCell,
										GridCell stopCell,
										const VRect& aRect)
#endif
{
	// only differences between this and original are 1) the view is given a chance to
	// set things up before each row is drawn; 2) stopCell.v is optimized.

	GridCell aStopCell(stopCell.h, (short)Min(stopCell.v, fAppointments->GetSize()));	// no need to draw beyond end of appointment list.

	VCoordinate colWidth;
	VCoordinate rowHeight;

	VRect localVRect(aRect);					// added because argument is const
	localVRect.left += fColInset / 2;
	localVRect.top += fRowInset / 2;

	VCoordinate left = localVRect.left;			// save it for each iteration of for loop

	if (GETNUMCOLWIDTHS() == 1)			// only one width
		colWidth = this->GetColWidth(1);
	if (GETNUMROWHEIGHTS() == 1)			// only one height
		rowHeight = this->GetRowHeight(1);

	CTemporaryRegion drawableRegion;
	this->GetDrawableRegion(drawableRegion);

	for (short j = startCell.v; j <= aStopCell.v; ++j)
	{
		localVRect.bottom = localVRect.top - fRowInset;
		if (GETNUMROWHEIGHTS() == 1)		// only one height
			localVRect.bottom += rowHeight;
		else
			localVRect.bottom += this->GetRowHeight(j);

		localVRect.left = left;					// start back at the left for the next row

		this->GetAppointmentParams(j);			// *** The point of all of this copying. ***

		for (short i = startCell.h; i <= aStopCell.h; ++i)
		{
			localVRect.right = localVRect.left - fColInset;
			if (GETNUMCOLWIDTHS() == 1)	// only one height
				localVRect.right += colWidth;
			else
				localVRect.right += this->GetColWidth(i);

			// check if the rect is drawable
			CRect qdRect;
			this->ViewToQDRect(localVRect, qdRect);
			if (::RectInRgn(qdRect, drawableRegion))
				this->DrawCell(GridCell(i, j), localVRect);

			localVRect.left = localVRect.right + fColInset;
		}
		localVRect.top = localVRect.bottom + fRowInset;
	}
}

//----------------------------------------------------------------------------------------
#pragma segment Printing

void TSchedulePrintView::GetAppointmentParams(short theRow)
{
	// Sets up text replacements based on appointment information for the row being printed.

	TAppointment* anAppointment = fAppointments->AppointmentAt(theRow);

	DayIndex		itsDay;
	TimeIndex		itsTime;
	TimeIndex		itsDuration;
	anAppointment->GetAppointmentInfo(itsDay, itsTime, itsDuration);

	CStr255 aString(GetDay(itsDay));
	MAParamText("^PrintDay", aString);

	aString = GetTime(itsTime);
	MAParamText("^PrintTime", aString);

	aString = GetDuration(itsDuration);
	MAParamText("^PrintDuration", aString);

	aString = anAppointment->GetDescription();
	MAParamText("^PrintDescription", aString);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

CStr255 TSchedulePrintView::GetText(const GridCell& aCell) const
{
	// Gets the text from the 'STR#' resource, then substitutes the replacement.
	CStr255 aString(Inherited::GetText(aCell));
	MAReplaceText(aString);
	return aString;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

bool TSchedulePrintView::IsActive() CONST		// OVERRIDE
{
	return true;				// Fool the print menu behavior into printing me.
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TSchedulePrintView::PrepareToPrint()
{
	// Matches the number of items in the view with the word count.
	short rowsNeeded = (short)fAppointments->GetSize();

	if (fNumOfRows > rowsNeeded)
		this->DelRowLast(fNumOfRows - rowsNeeded);
	else if (fNumOfRows < rowsNeeded)
		this->InsRowLast(rowsNeeded - fNumOfRows, fLineHeight + fRowInset);
}

//----------------------------------------------------------------------------------------
// CLASS TSchedulePrintHandler
//----------------------------------------------------------------------------------------
#pragma segment PrintImage

#undef Inherited
#define Inherited TStdPrintHandler

void TSchedulePrintHandler::AdornPage()						// OVERRIDE
{
	// Sets up the page number replacement, then draws the header and footer.

	CStr255 aString;
	::NumToString(fFocusedPage, aString);
	MAParamText("^PageNumber", aString);

	if (fHeader && fPrintHeader)
		this->DrawPageAdornment(fHeader);

	if (fFooter && fPrintFooter)
		this->DrawPageAdornment(fFooter);
}

//----------------------------------------------------------------------------------------
#pragma segment PrintNonRes

VPoint TSchedulePrintHandler::CalcViewPerPage()	// OVERRIDE
{
	const kMinAmtPerPage	= 20;	// so at least _something_ besides header and footer prints
#ifdef MACAPP_R_13
	VPoint amtPerPage(Inherited::CalcViewPerPage());
#else
	VPoint amtPerPage;
	Inherited::CalcViewPerPage(amtPerPage);
#endif
	// The view per page is reduced by the vertical size of the header and footer.

#if qDebug
	if (gUserFlag1)
		return gZeroVPt;
#endif

	if (fHeader && (amtPerPage.v - fHeader->fSize.v > kMinAmtPerPage))
	{
		amtPerPage.v -= fHeader->fSize.v;
		fPrintHeader = true;
	}
	else
		fPrintHeader = false;

	if (fFooter && (amtPerPage.v - fFooter->fSize.v > kMinAmtPerPage))
	{
		amtPerPage.v -= fFooter->fSize.v;
		fPrintFooter = true;
	}
	else
		fPrintFooter = false;
	return amtPerPage;
}

//----------------------------------------------------------------------------------------
#pragma segment PrintImage

void TSchedulePrintHandler::DrawPageAdornment(TView* theView)
{
	// Draws the header or footer.
	// This routine took a week to figure out.

	MAVolatileInit(TView*, holdView, fView);
	MAVolatileInit(VRect, holdRect, fViewedRect);

	fView = theView;										// so we can use all the built-in interior stuff

	TRY
	{
		if (fView->Focus())
		{
			gPageOffset = gZeroVPt - fView->fLocation;		// for correct placement

			VRect itsExtent;								// for correct clipping
			fView->GetExtent(itsExtent);

			fViewedRect = itsExtent;

			this->FocusOnInterior();
			this->DrawPageInterior();
		}

		fView		= holdView;
		fViewedRect	= holdRect;
	}
	CATCH_ALL
	{
		fView		= holdView;
		fViewedRect	= holdRect;
		THROW;
	}
	ENDTRY;

	FailOSErr(::PrError());
}

//----------------------------------------------------------------------------------------
#pragma segment PrintClose

TSchedulePrintHandler::~TSchedulePrintHandler()								// OVERRIDE
{
	fHeader = (TView*)FreeIfObject(fHeader);
	fFooter = (TView*)FreeIfObject(fFooter);
}

//----------------------------------------------------------------------------------------
#pragma segment PrintOpen

TSchedulePrintHandler::TSchedulePrintHandler(TDocument* itsDocument, TView* itsView) :
#ifdef MACAPP_R_13
	TStdPrintHandler(itsDocument, itsView, kFixedSize, !kFixedSize),
#endif
	fHeader(NULL),
	fFooter(NULL),
	fPrintHeader(false),
	fPrintFooter(false)
{
	// Load header and footer views _before_ calling IStdPrintHandler, so that they are available
	// when the margins are set. Sure hope I was initialized properly at allocation time

	fHeader = gViewServer->DoCreateViews(NULL, NULL, kScheduleHeaderID, gZeroVPt);
	fFooter = gViewServer->DoCreateViews(NULL, NULL, kScheduleFooterID, gZeroVPt);

#ifndef MACAPP_R_13
	IStdPrintHandler(itsDocument, itsView, !kSquareDots, kFixedSize, !kFixedSize);
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment PrintNonRes

VPoint TSchedulePrintHandler::LocatePageInterior(long thePageNumber)	// OVERRIDE
{
	// Start interior just below header.
#ifdef MACAPP_R_13
	VPoint loc(Inherited::LocatePageInterior(thePageNumber));
#else
	VPoint loc;
	Inherited::LocatePageInterior(thePageNumber, loc);
#endif
	if (fHeader && fPrintHeader)
		loc.v += fHeader->fSize.v;
	return loc;
}

//----------------------------------------------------------------------------------------
#pragma segment PrintRes

bool TSchedulePrintHandler::Print(CommandNumber itsCommandNumber)						// OVERRIDE
{
	// Prepares for printing by locating header and footer, and giving printing view a
	// chance to synch up to data being printed.

	// Prepare document title text.
	CStr255 aString;
	fDocument->GetTitle(aString);
	MAParamText("^DocumentTitle", aString);

	// Locate the header before top of shrunken interior.
	if (fHeader && fPrintHeader)
	{
		VRect headRect(PAGE_INTERIOR());
		headRect.bottom = headRect.top;
		headRect.top   -= fHeader->fSize.v;
		fHeader->SetFrame(headRect, kDontInvalidate);
	}

	// Locate footer after bottom of shrunken interior.
	if (fFooter && fPrintFooter)
	{
		VRect footRect(PAGE_INTERIOR());
		footRect.top	 = footRect.bottom;
		footRect.bottom += fFooter->fSize.v;
		fFooter->SetFrame(footRect, kDontInvalidate);
	}

	// Give printing view a chance to prepare itself.
	((TSchedulePrintView*)fView)->PrepareToPrint();

	// Print it!
#ifdef MACAPP_R_13
	return Inherited::Print(itsCommandNumber);
#else
	Boolean proceed;
	Inherited::Print(itsCommandNumber, proceed);
	return proceed;
#endif
}
