/*
	USchedulePrinting.h

	Copyright 1991-1992 Apple Computer, Inc.
	All rights reserved

	Change History:

	9/23/91		LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
 */

#ifndef __USCHEDULEPRINTING__
#define __USCHEDULEPRINTING__

#ifndef __UGRIDVIEW__
#include <UGridView.h>
#endif

#ifndef __UPRINTING__
#include <UPrinting.h>
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const ResNumber kSchedulePrintID	= 2001;
const ResNumber kScheduleHeaderID	= 2002;
const ResNumber kScheduleFooterID	= 2003;

const ResNumber kScheduleTestID		= 2010;

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TAppointmentList;

//----------------------------------------------------------------------------------------
// CLASS TStringListView
//----------------------------------------------------------------------------------------

class TStringListView : public TTextGridView
{
	DeclareClass(TStringListView);

public:

	// Streams
	virtual void ReadFields(TStream* aStream);						// OVERRIDE

	// Drawing
	virtual CStr255 GetText(const GridCell& aCell) const;			// OVERRIDE
	virtual void GetText(GridCell aCell, CStr255& itsText) { itsText = GetText(aCell); }			// OVERRIDE

	// Page breaks
	virtual VCoordinate DoBreakFollowing(VHSelect	vhs,
										 VCoordinate	previousBreak,
										 bool&	automatic );	// OVERRIDE

};

//----------------------------------------------------------------------------------------
// CLASS TSchedulePrintView
//----------------------------------------------------------------------------------------

class TSchedulePrintView : public TStringListView
{
	DeclareClass(TSchedulePrintView);

private:
	TAppointmentList* fAppointments;

public:
	// Construction/Destruction
	virtual void DoPostCreate(TDocument* itsDocument);				// OVERRIDE

	// Printing
	virtual void PrepareToPrint();

	// Drawing
#ifdef MACAPP_R_13
	virtual void DrawRangeOfCells(const GridCell& startCell,
								const GridCell& stopCell, const VRect& aRect);		// OVERRIDE
#else
	virtual void DrawRangeOfCells(GridCell startCell,
								GridCell stopCell, const VRect& aRect);		// OVERRIDE
#endif

	virtual void GetAppointmentParams(short theRow);

	virtual CStr255 GetText(const GridCell& aCell) const;			// OVERRIDE
	virtual void GetText(GridCell aCell, CStr255& itsText) { itsText = GetText(aCell); }			// OVERRIDE
	// Misc.
	virtual bool IsActive() CONST;					// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TSchedulePrintHandler
//----------------------------------------------------------------------------------------

class TSchedulePrintHandler : public TStdPrintHandler
{

private:
	TView* 				fHeader;
	TView*				fFooter;

	bool				fPrintHeader;
	bool				fPrintFooter;

public:
	// Construction/Destruction
	TSchedulePrintHandler(TDocument* itsDocument, TView* itsView);
	virtual ~TSchedulePrintHandler();										// OVERRIDE

	// Printing
	virtual void AdornPage();									// OVERRIDE
	virtual VPoint CalcViewPerPage();			// OVERRIDE
	virtual void CalcViewPerPage(VPoint& viewPerPage) 
		{ viewPerPage = CalcViewPerPage(); }
	
	virtual void DrawPageAdornment(TView* theView);
	virtual VPoint LocatePageInterior(long thePageNumber);			// OVERRIDE
	virtual void LocatePageInterior(long thePageNumber, VPoint& pageInterior) 
		{ pageInterior = LocatePageInterior(thePageNumber); }
		
	virtual bool Print(CommandNumber itsCommandNumber);			// OVERRIDE
	virtual void Print(CommandNumber itsCommandNumber, bool& proceed)
		{ proceed = Print(itsCommandNumber); }
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

extern void InitUSchedulePrinting();
	// Call this routine at initialization time

#endif
