/*
	UScheduleTracker.h

	Copyright 1991-1995 Apple Computer, Inc.
	All rights reserved

	Change History:

	9/23/91		LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
 */

#ifndef __USCHEDULETRACKER__
#define __USCHEDULETRACKER__

#ifndef __USCHEDULEVIEW__
#include "UScheduleView.h"
#endif

#ifndef __UGRIDVIEW__
#include <UGridView.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TAppointment;

//----------------------------------------------------------------------------------------
// CLASS TAppointmentSketcher
//----------------------------------------------------------------------------------------

class TAppointmentSketcher: public TRCSelectCommand
{

public:

	// Construction/Destruction
	TAppointmentSketcher(TScheduleView* itsView, const VPoint& itsMouse);

	// Tracking
	virtual void ComputeNewSelection(CONST GridCell& clickedCell);		// OVERRIDE
	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
										VPoint& anchorPoint,
										VPoint& previousPoint,
										VPoint& nextPoint,
										bool mouseDidMove);		// OVERRIDE

	// Commands
	virtual void DoIt(); 										// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentTracker: Handles changes to appointments.
//----------------------------------------------------------------------------------------

class TAppointmentTracker : public TTracker
{

protected:
	TAppointment* 	fAppointment;
	TAppointment* 	fOldAppointment;
	TAppointment* 	fNewAppointment;

	GridCell		fAnchorCell;

	TScheduleView*	fScheduleView;

public:

	// Construction/Destruction

	TAppointmentTracker(CommandNumber itsCommandNumber,
			TScheduleView* itsView, const VPoint& theMouse,
			TAppointment* theAppointment);
	virtual ~TAppointmentTracker();				// OVERRIDE

	// Commands
	virtual void DoIt();				// OVERRIDE
	virtual void RedoIt();				// OVERRIDE
	virtual void UndoIt();				// OVERRIDE

	// Tracking
	virtual void ComputeNewAppointment(const GridCell nextCell);

	virtual void TrackConstrain(TrackPhase aTrackPhase,
								const VPoint& anchorPoint,
								const VPoint& previousPoint,
								VPoint& nextPoint,
								bool mouseDidMove);	// OVERRIDE

	virtual void TrackFeedback(TrackPhase aTrackPhase,
								 const VPoint& anchorPoint,
								 const VPoint& previousPoint,
								 const VPoint& nextPoint,
								 bool mouseDidMove,
								 bool turnItOn);				// OVERRIDE

	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
										VPoint& anchorPoint,
										VPoint& previousPoint,
										VPoint& nextPoint,
										bool mouseDidMove);	// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentDragger: Drags an appointment.
//----------------------------------------------------------------------------------------

class TAppointmentDragger : public TAppointmentTracker
{

public:
	// Construction/Destruction
	TAppointmentDragger(TScheduleView* itsView, const VPoint& theMouse,
			TAppointment* theAppointment);

	// Tracking
	virtual void ComputeNewAppointment(const GridCell nextCell);		// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentStretcher: Resizes an appointment.
//----------------------------------------------------------------------------------------

class TAppointmentStretcher : public TAppointmentTracker
{

private:
	KnobSelector fWhichKnob;

public:
	// Construction/Destruction
	TAppointmentStretcher(TScheduleView* itsView, const VPoint& theMouse,
			TAppointment* theAppointment, const KnobSelector whichKnob);

	// Tracking
	virtual void ComputeNewAppointment(const GridCell nextCell);		// OVERRIDE
};

#endif
