//----------------------------------------------------------------------------------------
// UScheduleView.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __USCHEDULEVIEW__
#include "UScheduleView.h"
#endif

#ifndef __USCHEDULECOMMAND__
#include "UScheduleCommand.h"
#endif

#ifndef __USCHEDULEDOCUMENT__
#include "UScheduleDocument.h"
#endif

#ifndef __USECTION__
#include <USection.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __UMENUMGR__
#include <UMenuMgr.h>
#endif

//----------------------------------------------------------------------------------------

const short mUpArrow	= 100;
const short mDownArrow	= 101;

// Global Initialization Procedure

#pragma segment AInit

void InitUScheduleView()
{
	// So the linker doesn't dead strip class info
	macroDontDeadStrip(TScheduleView);
	macroDontDeadStrip(TDayView);
	macroDontDeadStrip(TTimeView);
	macroDontDeadStrip(TAppointmentClipView);

	macroDontDeadStrip(TLookupDialog);
	macroDontDeadStrip(TArrowsControl);

	macroDontDeadStrip(TSubscriberView);
	macroDontDeadStrip(TSubscriberDayView);
	macroDontDeadStrip(TSubscriberNameView);
}

//----------------------------------------------------------------------------------------
// CLASS TCommonView
//----------------------------------------------------------------------------------------
#pragma segment AOpen

#undef Inherited
#define Inherited TTextGridView
DefineClass(TCommonView, Inherited);

TCommonView::TCommonView()
{
	this->Initialize();
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TCommonView::AdornCol(GridCoordinate /* aCol */, const VRect& area)				// OVERRIDE
{
	// All subclasses adorn their columns with a line on the inside right side of the cell.

	::PenNormal();
	::PenPat(&qd.gray);

	CRect qdRect;
	this->ViewToQDRect(area, qdRect);

	qdRect[botRight] -= CPoint(1,1);		// draw adornment inside cell.

	::MoveTo(qdRect.right, qdRect.top);
	::Line(0, qdRect.GetLength(vSel));
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TCommonView::AdornRow(GridCoordinate /* aRow */, const VRect& area)	// OVERRIDE
{
	// All subclasses adorn their rows with a line on the inside bottom of the cell.

	::PenNormal();
	::PenPat(&qd.dkGray);

	CRect qdRect;
	this->ViewToQDRect(area, qdRect);

	qdRect[botRight] -= CPoint(1,1);		// draw adornment inside cell.

	::MoveTo(qdRect.left, qdRect.bottom);
	::Line(qdRect.GetLength(hSel), 0);
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TCommonView::DoPostCreate(TDocument* itsDocument) // OVERRIDE
{
	// Fill in the convenient memory reference to the view's schedule document.

	Inherited::DoPostCreate(itsDocument);
	fScheduleDocument = (TScheduleDocument*) itsDocument;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TScheduleDocument* TCommonView::GetScheduleDocument()
{
	// Accessor function
	return fScheduleDocument;
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TCommonView::Initialize() 						// OVERRIDE
{
	// Put view into "safe" state.
	fScheduleDocument = NULL;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TCommonView::ResignedWindowTarget()				// OVERRIDE
{
	// This method deselects the view's selection when it resign's the window target.
	// It is called when another view becomes the window target.

	this->SetEmptySelection(kHighlight);
	Inherited::ResignedWindowTarget();
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TCommonView::SetTargetSelection(bool /* redraw */)
{
	// Usually, this method scrolls the selection into view when it becomes the target.
	// But it is overridden to do nothing, so it doesn't interfere with scroll restoring.
}

//----------------------------------------------------------------------------------------
// CLASS TScheduleView
//----------------------------------------------------------------------------------------
#pragma segment ANonRes

#undef Inherited
#define Inherited TCommonView
DefineClass(TScheduleView, Inherited);

void TScheduleView::ComputeBorderRegion(TSectionAdorner* anAdorner)
{
	// Creates the "doughnut" section border region.

	DayIndex theDay = GetDayDesignator(anAdorner->fSection)->GetDay();
	this->GetDayRegion(theDay, anAdorner->fBorderRegion);
	CTemporaryRegion aRgn;
	::CopyRgn(anAdorner->fBorderRegion, aRgn);
	::InsetRgn(aRgn, 3, 3);
	::DiffRgn(anAdorner->fBorderRegion, aRgn, anAdorner->fBorderRegion);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::DoHighlightAppointment(TAppointment* anAppointment, HLState fromHL, HLState toHL)
{
	// Appointments are highlighted by drawing their knobs. This method handles dim highlighting
	// as well.

	this->SetHLPenState(fromHL, toHL);

	short start = short(inUpperKnob);
	short stop = short(notInKnob);
	for (short i = start; i < stop; i++)
	{
		VRect knobRect;
		this->GetAppointmentKnob(anAppointment, KnobSelector(i), knobRect);

		CRect qdKnobRect;
		this->ViewToQDRect(knobRect, qdKnobRect);
		::PaintRect(qdKnobRect);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::DoHighlightAppointments(HLState fromHL, HLState toHL)
{
	// This method highlights all selected appointments for this view.

	if (fromHL != toHL && this->Focus())
	{
		CSelectedAppointmentIterator iter(this);
		for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
				anAppointment = iter.NextAppointment())
			this->DoHighlightAppointment(anAppointment, fromHL, toHL);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::DoHighlightSelection(HLState fromHL, HLState toHL)	// OVERRIDE
{
	// This method is called at update time to highlight the selection.
	// It passes control to a method that can be overridden easily.

	Inherited::DoHighlightSelection(fromHL, toHL);

	this->DoHighlightAppointments(fromHL, toHL);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

#ifdef MACAPP_R_13
void TScheduleView::DoUpdate(ChangeID_AC theChange, MDependable_AC* changedObject, void* changeData, CDependencySpace_AC* dependencySpace)		// OVERRIDE
#else
void TScheduleView::DoUpdate(ChangeID theChange, TObject* changedObject, TObject* changeData, TDependencySpace* dependencySpace)		// OVERRIDE
#endif
{
	// This method is called when appointments it depends on are changed.
	// It invalidates the appropriate part of the screen.

	Inherited::DoUpdate(theChange, changedObject, changeData, dependencySpace);
	switch (theChange)
	{
	case cAddAppointment:
	case cDeleteAppointment:
	case cPreTrack:
	case cPostTrack:
		this->InvalidateAppointment((TAppointment*)changedObject);
		break;
	case cChangeAppointment:
		this->InvalidateDescription((TAppointment*)changedObject);
		break;
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::Draw(const VRect& area)	// OVERRIDE
{
	// Draws the appointments after the gridview background is drawn.

	Inherited::Draw(area);
	this->DrawAppointments(area);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::DrawAppointment(TAppointment* anAppointment, const VRect& area)
{
	// Draws an appointment by erasing the background, imaging the text, then framing the
	// appointment's extent.

	VRect itsExtent;
	this->GetAppointmentExtent(anAppointment, itsExtent);

	if ((area & itsExtent).Empty())
		return;							// Nothing to draw.

	CRect qdExtent;
	this->ViewToQDRect(itsExtent, qdExtent);

	CRect textRect;
	this->InsetForText(itsExtent);
	this->ViewToQDRect(itsExtent, textRect);

	CStr255 itsDescription(anAppointment->GetDescription());

	// MATextBox's "erase first" option only erases the lines of text. We need to erase the
	// background from the entire appointment.

	::EraseRect(qdExtent);

	MATextBox((Ptr)&itsDescription[1], itsDescription.Length(), textRect, fJustification,
		kAutoWrap, NULL, kNoEraseFirst, kNoSpaceForCaret, fPreferOutline);

	::PenNormal();
	::PenSize(2,2);
	::FrameRect(qdExtent);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::DrawAppointments(const VRect& area)
{
	// Iterates to draw all appointments for this view.

	CAppointmentIterator iter(this);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		this->DrawAppointment(anAppointment, area);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::GetAppointmentExtent(TAppointment* anAppointment, VRect& itsExtent)
{
	// Returns an appointment's extent. The operations are broken into several methods
	// so that they can be overridden by the TSubscriberView subclass.

	DayIndex	itsDay;
	TimeIndex	itsTime;
	TimeIndex	itsDuration;
	anAppointment->GetAppointmentInfo(itsDay, itsTime, itsDuration);

	this->GetExtentFromInfo(itsDay, itsTime, itsDuration, itsExtent);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::GetAppointmentKnob(TAppointment* anAppointment,
								const KnobSelector whichKnob, VRect& knobRect)
{
	// Calculates a VRect for each appointment knob.

	const VPoint kKnobSize(8, 8);
	const VRect	 kKnobRect(gZeroVPt, kKnobSize);

	VRect itsExtent;
	this->GetAppointmentExtent(anAppointment, itsExtent);

	VCoordinate knobLeft = itsExtent.left + (itsExtent.GetLength(hSel) >> 1) - (kKnobSize.h >> 1);
	VPoint upperLoc( knobLeft, itsExtent.top	- (kKnobSize.v >> 1) );
	VPoint lowerLoc( knobLeft, itsExtent.bottom	- (kKnobSize.v >> 1) );

	switch (whichKnob)
	{
	case inUpperKnob:
		knobRect = kKnobRect + upperLoc;
		break;
	case inLowerKnob:
		knobRect = kKnobRect + lowerLoc;
		break;
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleView::GetDayRegion(DayIndex theDay, RgnHandle theRegion)
{
	// This method supplies the region for the given day to the routine that calculates
	// section border methods. It uses the GridView methods, and accounts for the adornment.
	// It is overridden by TSubscriberView.

	VRect dayRect;
	this->ColToVRect(theDay, 1, dayRect);

	CRect aRect;
	this->ViewToQDRect(dayRect, aRect);

	aRect[botRight] -= CPoint(1, 1);			// Account for adornment

	::RectRgn(theRegion, aRect);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::GetExtentFromInfo(DayIndex theDay, TimeIndex theTime, TimeIndex theDuration, VRect& itsExtent)
{
	// Returns a VRect that represents the info passed as parameters. Uses GridView methods.

	CPoint	upperCell(theDay, theTime);
	CPoint 	lowerCell = upperCell + CPoint(0, theDuration - 1);

	VRect	upperCellRect;
	VRect	lowerCellRect;

	this->CellToVRect(upperCell, upperCellRect);
	this->CellToVRect(lowerCell, lowerCellRect);

	itsExtent = upperCellRect | lowerCellRect;
	itsExtent[botRight] -= VPoint(1,1);			// account for adornment

	if (qDebug && itsExtent.Empty())
		ProgramBreak("Invisible appointment extent!");
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment* TScheduleView::GetSelectedAppointment()
{
	// Returns the first selected appointment for this view.

	CSelectedAppointmentIterator iter(this);
	return iter.FirstAppointment();
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment* TScheduleView::GetTopAppointment(const VPoint& theMouse)
{
	// Returns the topmost appointment under the mouse.

	CAppointmentIterator iter(this, kIterateBackward);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		if (this->MouseInAppointment(anAppointment, theMouse))
			return anAppointment;

	return NULL;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

TAppointment* TScheduleView::GetTopSelectedAppointment(const VPoint& theMouse)
{
	// Returns the topmost selected appointment under the mouse.

	CSelectedAppointmentIterator iter(this, kIterateBackward);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		if (this->MouseInAppointment(anAppointment, theMouse))
			return anAppointment;

	return NULL;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

bool TScheduleView::HaveAnyAppointments()
{
	// Indicates whether this view has any appointments at all.

	CAppointmentIterator iter(this);
	return (iter.FirstAppointment() != NULL);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

bool TScheduleView::HaveAnySelectedAppointments()
{
	// Indicates whether this view has any selected appointments.

	return (this->GetSelectedAppointment() != NULL);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

bool TScheduleView::HaveASelectedAppointment()
{
	// Indicates whether this view has a single selected appointment.

	TAppointment* theAppointment = NULL;

	CSelectedAppointmentIterator iter(this);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		if (theAppointment)
			return false;		// more than one is selected
		else
			theAppointment = anAppointment;

	return (theAppointment != NULL);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::HighlightAppointment(TAppointment* anAppointment, bool highlight)
{
	// Called to highlight an appointment immediately. Used by clipboard support routines.

	if (this->Focus())
	{
		if (highlight)
			this->DoHighlightAppointment(anAppointment, hlOff, hlOn);
		else
			this->DoHighlightAppointment(anAppointment, hlOn, hlOff);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::InsetForText(VRect& itsExtent)
{
	// Adjusts itsExtent to account for text inset.

	itsExtent.Inset(VPoint(fColInset/2, fRowInset/2));
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::InvalidateAppointment(TAppointment* anAppointment)
{
	// Invalidates an appointment's extent, and if selected, its knobs.

	VRect itsExtent;
	this->GetAppointmentExtent(anAppointment, itsExtent);
	this->InvalidateVRect(itsExtent);
	if (anAppointment->IsSelected()) {
		short start = short(inUpperKnob);
		short stop = short(notInKnob);
		for (short i = start; i < stop; i++)
		{
			VRect knobRect;
			this->GetAppointmentKnob(anAppointment, KnobSelector(i), knobRect);
			this->InvalidateVRect(knobRect);
		}
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleView::InvalidateDescription(TAppointment* anAppointment)
{
	// Invalidates an appointment's text area.

	VRect itsExtent;
	this->GetAppointmentExtent(anAppointment, itsExtent);
	this->InsetForText(itsExtent);
	this->InvalidateVRect(itsExtent);
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

bool TScheduleView::MouseInAppointment(TAppointment* anAppointment,
					const VPoint& theMouse)
{
	// Determines whether a point is contained by an appointment. Used for hit-testing
	// mouse-down's.

	VRect itsExtent;
	this->GetAppointmentExtent(anAppointment, itsExtent);
	return itsExtent.Contains(theMouse);
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

bool TScheduleView::MouseInKnob(TAppointment* anAppointment,
								   const VPoint& theMouse, KnobSelector& whichKnob)
{
	// Determines which appointment knob contains a point.
	// Used for hit-testing mouse-down's.

	short start = short(inUpperKnob);
	short stop = short(notInKnob);
	for (short i = start; i < stop; i++)
	{
		VRect knobRect;
		this->GetAppointmentKnob(anAppointment, KnobSelector(i), knobRect);
		if (knobRect.Contains(theMouse)) {
			whichKnob = KnobSelector(i);
			return true;
		}
	}

	return false;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::RevealSelection(TDesignator* aSelection)
{
	// Scrolls the current selection into view. Used to show the publisher.

	VRect selectionRect;

	this->ColToVRect( ((TDayDesignator*)aSelection)->GetDay(), 1, selectionRect);

	VPoint minToSee = selectionRect.GetSize();

	this->RevealRect(selectionRect, minToSee, kRedraw);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::ScrollSelectionIntoView(bool redraw)			// OVERRIDE
{

	VRect selectionRect(gZeroVRect);

	CSelectedAppointmentIterator iter(this);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
	{
		VRect itsExtent;
		this->GetAppointmentExtent(anAppointment, itsExtent);
		if (selectionRect.Empty())			// a more perfect union.
			selectionRect = itsExtent;
		else
			selectionRect = selectionRect | itsExtent;
	}

	if (!selectionRect.Empty())
	{
		selectionRect.Inset(VPoint(- this->GetColWidth(1), - this->GetRowHeight(1)));	// provide a margin

		// Precalculate these since Min is an inline method.
		long hCoord = Min(selectionRect.GetLength(hSel), static_cast<long> (kMaxCoord));
		long vCoord = Min(selectionRect.GetLength(vSel), static_cast<long> (kMaxCoord));

		VPoint minToSee(hCoord, vCoord);
		this->RevealRect(selectionRect, minToSee, redraw);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleView::SetEmptySelection(bool highlight)			// OVERRIDE
{
	// Deselects all selected appointments, and highlight's them if desired.

	Inherited::SetEmptySelection(highlight);
	CSelectedAppointmentIterator iter(this);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		if (highlight)
			this->HighlightAppointment(anAppointment, anAppointment->SetSelect(kDeSelect));
		else
			anAppointment->SetSelect(kDeSelect);
}

//----------------------------------------------------------------------------------------
// CLASS TSubscriberView
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TScheduleView
DefineClass(TSubscriberView, Inherited);

void TSubscriberView::DoHighlightAppointments(HLState fromHL, HLState toHL)	// OVERRIDE
{
	// If the subscriber is not present or is canceled, don't highlight the appointments.

	TSection* subscriberSection = fScheduleDocument->GetTheSubscriber();
	if (subscriberSection && !subscriberSection->IsCanceled())
		Inherited::DoHighlightAppointments(fromHL, toHL);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TSubscriberView::DrawAppointments(const VRect& area)	// OVERRIDE
{
	// If the subscriber is not present or is canceled, don't draw the appointments.

	TSection* subscriberSection = fScheduleDocument->GetTheSubscriber();
	if (subscriberSection && !subscriberSection->IsCanceled())
		Inherited::DrawAppointments(area);
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TSubscriberView::GetDayRegion(DayIndex /* theDay */ , RgnHandle theRegion)		// OVERRIDE
{
	// Returned region is always the first column of the subscriber view.

	Inherited::GetDayRegion(1, theRegion);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TSubscriberView::GetExtentFromInfo(DayIndex /* theDay */, TimeIndex theTime, TimeIndex theDuration, VRect& itsExtent)	// OVERRIDE
{
	// Returned extent is always the first column of the subscriber view.

	Inherited::GetExtentFromInfo(1, theTime, theDuration, itsExtent);
}

//----------------------------------------------------------------------------------------
// CLASS TDayView
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TCommonView
DefineClass(TDayView, Inherited);

CStr255 TDayView::GetText(const GridCell& aCell) const
{
	// All TTextGridView's must override GetText to supply the text to draw.
	// This supplies the day.
	return GetDay(aCell.h);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TDayView::HighlightCells(CONST GRIDVIEWDESIGNATORPTR theCells,
									  HLState fromHL,
									  HLState toHL)			// OVERRIDE
{
	// This method replaces TGridView's cell inversion highlighting with
	// fancy rectangle highlighting.

	if ((fromHL != toHL) && this->Focus())
	{
		GridCell theCell = GRIDVIEWDESIGNATOR_GETBOUNDS(theCells)[topLeft];

		if (theCell == gZeroPt)
			return;

		VRect aVRect;
		this->CellToVRect(theCell, aVRect);
		CRect aQDRect;
		this->ViewToQDRect(aVRect, aQDRect);

		aQDRect[botRight] -= CPoint(1, 0);	// account for adornment (col's only.)

		switch (fromHL + toHL)
		{
		case hlOffDim:
			::PenNormal();
			::PenMode(patXor);
			::PenSize(1,1);
			UseSelectionColor();
			::FrameRect(aQDRect);
			break;

		case hlOnDim:
			::PenNormal();
			::PenMode(patXor);
			aQDRect.Inset(CPoint(1, 1));
			::PenSize(2,2);
			UseSelectionColor();
			::FrameRect(aQDRect);
			break;

		case hlOffOn:
			::PenNormal();
			::PenMode(patXor);
			::PenSize(3,3);
			UseSelectionColor();
			::FrameRect(aQDRect);
			break;
		}
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

void TDayView::ReadFields(TStream* aStream)		// OVERRIDE
{
	// MacApp doesn't have a fJustification field in view resources, so we set it by hand.

	Inherited::ReadFields(aStream);
	fJustification = teJustCenter;
}

//----------------------------------------------------------------------------------------
// CLASS TSubscriberDayView
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TDayView
DefineClass(TSubscriberDayView, Inherited);

CStr255 TSubscriberDayView::GetText(const GridCell& aCell) const
{
	// This method fools the base class into returning the correct day text, even though
	// only the first column of the subscriber view is ever drawn.

	CStr255 aString;
	TSubscriber* aSubscriber = fScheduleDocument->GetTheSubscriber();
	if (aSubscriber)
	{
		GridCell cell(GetDayDesignator(aSubscriber)->GetDay(), aCell.v);
		aString = Inherited::GetText(cell);
	}
	else
		aString = gEmptyString;
	return aString;
}

//----------------------------------------------------------------------------------------
// CLASS TSubscriberNameView
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TStaticText
DefineClass(TSubscriberNameView, Inherited);

void TSubscriberNameView::Draw(const VRect& area)		// OVERRIDE
{
	// This view's only role in life is to draw the subscriber's name, if any.
	// This method does that using MacApp's built-in text substitution facility.

	if (((TScheduleDocument*)fDocument)->GetTheSubscriber())
	{
		CStr255 itsName;
		((TScheduleDocument*)fDocument)->GetSubscriberName(itsName);
		MAParamText("^SubscriberName", itsName);
		Inherited::Draw(area);
	}
}

//----------------------------------------------------------------------------------------
// CLASS TTimeView
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TCommonView
DefineClass(TTimeView, Inherited);

CStr255 TTimeView::GetText(const GridCell& aCell) const
{
	// All TTextGridView's must override GetText to supply the text to draw.
	// This supplies the time.
	return GetTime(aCell.v);
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

void TTimeView::ReadFields(TStream* aStream)		// OVERRIDE
{
	// MacApp doesn't have a fJustification field in view resources, so we set it by hand.

	Inherited::ReadFields(aStream);
	fJustification = teJustRight;
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentClipView
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TView
DefineClass(TAppointmentClipView, Inherited);

bool TAppointmentClipView::ContainsClipType(ResType aType)	// OVERRIDE
{
	// This view is used to display the private scrap. This method is used by the
	// clipboard manager to enable the "Paste" menu item.

	return aType == ((TScheduleDocument*)fDocument)->fScrapType;
}

//----------------------------------------------------------------------------------------
#pragma segment AClipboard

void TAppointmentClipView::WriteToDeskScrap()			// OVERRIDE
{
	// This method is used by the clipboard manager to write the private scrap
	// to the desk scrap. It passes the call onto its document, because the document
	// knows how to read and write appointments.

	((TScheduleDocument*)fDocument)->WriteToDeskScrap();		// pass the call on to the document.
}

//----------------------------------------------------------------------------------------
// CLASS TLookupDialog
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TDialogView
DefineClass(TLookupDialog, Inherited);

void TLookupDialog::DoEvent(EventNumber eventNumber,
								 TEventHandler* source,
								 TEvent* event)					// OVERRIDE
{
	// This method responds to events in the dialog's controls.

	switch (eventNumber)
	{
	case mButtonHit:
		if (source->fIdentifier == 'LKUP')		// if the lookup button was hit,
			this->HandleMenuCommand(cLookupAppointment); // do the lookup command.
		else
			Inherited::DoEvent(eventNumber, source, event);
		break;

	case mDownArrow:
		if (source->fIdentifier == 'DAYA')
			this->IncrementDay(1);
		else
			this->IncrementTime(1);
		break;

	case mUpArrow:
		if (source->fIdentifier == 'DAYA')
			this->IncrementDay(-1);
		else
			this->IncrementTime(-1);
		break;

	default:
		Inherited::DoEvent(eventNumber, source, event);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

void TLookupDialog::DoMenuCommand(CommandNumber aCommandNumber)	// OVERRIDE
{
	// This method handles the lookup command by launching a client command.

	switch (aCommandNumber)
	{
		case cLookupAppointment:
		{
			PostCommand(TH_new TScheduleClientCommand(aCommandNumber, this, fDay, fTime, fTargetAddress));

			this->InProgress(true);					// we're working now.
			break;
		}
		default:
			Inherited::DoMenuCommand(aCommandNumber);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TLookupDialog::DoPostCreate(TDocument* itsDocument)	// OVERRIDE
{
	// Fill in data member references to dialog items. Set up initial values.

	Inherited::DoPostCreate(itsDocument);
	fDoItButton		= (TButton*)(this->FindSubView(fDefaultItem));
	fDayText		= (TStaticText*)(this->FindSubView('DAYT'));
	fTimeText		= (TStaticText*)(this->FindSubView('TIMT'));
	fResponseText	= (TStaticText*)(this->FindSubView('RESP'));

	CStr255 aString(GetDay(fDay));
	fDayText->SetText(aString, kDontRedraw);
	aString = GetTime(fTime);
	fTimeText->SetText(aString, kDontRedraw);

}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TLookupDialog::DoSetupMenus()		// OVERRIDE
{
	// Enable the lookup item.

	Inherited::DoSetupMenus();
	Enable(cLookupAppointment, !fInProgress);
}

//----------------------------------------------------------------------------------------
#pragma segment AClose

TLookupDialog::~TLookupDialog()		// OVERRIDE
{
	// Deallocate members that use memory.

	AEAddressDesc aDesc = fTargetAddress;
	::AEDisposeDesc(&aDesc);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TLookupDialog::HaveAddress(TargetID& theTarget)
{
	// Accepts result of PPCBrowser call. May fail.

	AEAddressDesc	targetAddress;

	FailOSErr(::AECreateDesc(typeTargetID, Ptr(&theTarget), sizeof(theTarget), &targetAddress));
	fTargetAddress = targetAddress;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TLookupDialog::HaveResponse(ConstCStr255Param theResponse)
{
	// Called to display the response in the dialog.

	this->InProgress(false);						// no longer working
	fResponseText->SetText(theResponse, kRedraw);	// display the response
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TLookupDialog::IncrementDay(DayIndex offset)
{
	// Bumps the day by offset, and updates the text.

	fDay += offset;
	if (fDay > kMaxNumberOfDays)
		fDay = 1;
	else if (fDay < 1)
		fDay = kMaxNumberOfDays;

	fDayText->SetText(GetDay(fDay), kRedraw);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TLookupDialog::IncrementTime(TimeIndex offset)
{
	// Bumps the time by offset, and updates the text.

	fTime += offset;
	if (fTime > kMaxNumberOfIntervals)
		fTime = 1;
	else if (fTime < 1)
		fTime = kMaxNumberOfIntervals;

	fTimeText->SetText(GetTime(fTime), kRedraw);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TLookupDialog::Initialize()		// OVERRIDE
{
	//	Set up default address
	fTargetAddress.descriptorType	= typeNull;
	fTargetAddress.dataHandle		= NULL;

	fDoItButton						= NULL;
	fDayText						= NULL;
	fTimeText						= NULL;
	fResponseText					= NULL;

	fInProgress						= false;

	fDay							= kInitialDay;
	fTime							= kInitialTime;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TLookupDialog::InProgress(const bool lockItUp)
{
	// If we're in progress, disable and highlight the doit button and set the flag
	// so the menu item is disabled. Otherwise, undo all of that.

	fInProgress = lockItUp;

	fDoItButton->HiliteState(lockItUp, kRedraw);
	fDoItButton->SetEnable(!lockItUp);
}

//----------------------------------------------------------------------------------------
// CLASS TArrowsControl
//----------------------------------------------------------------------------------------
#pragma segment AOpen

#undef Inherited
#define Inherited TPicture
DefineClass(TArrowsControl, Inherited);

void TArrowsControl::Initialize()		// override
{
	// Set view to "safe" state.
	fLastChange = 0;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TArrowsControl::TrackMouse(TrackPhase	aTrackPhase,
								VPoint&		/*anchorPoint*/,
								VPoint&		/*previousPoint*/,
								VPoint&		nextPoint,
								bool		/*mouseDidMove*/)	// OVERRIDE

{
	// Handles auto-repeat action of control, and determination of whether upper or lower
	// arrow was hit.

	const long kInitialDelay		= 10;
	const long kRepeatRate			= 5;

	if (::TickCount() >= fLastChange + kRepeatRate)
	{
		if (ContainsMouse(nextPoint))
		{
			VRect myFrame;
			this->GetFrame(myFrame);
			VHSelect whichAxis = LongerVSide(myFrame);

			if (nextPoint[whichAxis] <= fSize[whichAxis] >> 1)
				this->HandleEvent(mUpArrow, this, NULL);
			else
				this->HandleEvent(mDownArrow, this, NULL);
		}
		fLastChange = ::TickCount();

		if (aTrackPhase == trackBegin)
			fLastChange += kInitialDelay;

	}
}

