/*
	UScheduleView.h

	Copyright 1991-1995 Apple Computer, Inc.
	All rights reserved

	Change History:

	9/23/91		LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
 */

#ifndef __USCHEDULEVIEW__
#define __USCHEDULEVIEW__

#ifndef __USCHEDULEOBJECTS__
#include "UScheduleObjects.h"
#endif

#ifndef __UDIALOG__
#include <UDialog.h>
#endif

#ifndef __UGRIDVIEW__
#include <UGridView.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const ResNumber kScheduleWindowID 	= 1101;
const ResNumber kScheduleClipViewID	= 1102;

const IDType	kScheduleID			= 'SKED';
const IDType 	kSubscriberID		= 'SUBS';
const IDType 	kSubscriberDayID	= 'SUBD';

enum KnobSelector { inUpperKnob, inLowerKnob, notInKnob };

//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TScheduleDocument;
class TSectionAdorner;
class TButton;

//----------------------------------------------------------------------------------------
// CLASS TCommonView
//----------------------------------------------------------------------------------------

class TCommonView: public TTextGridView
{
	DeclareClass(TCommonView);

protected:
	TScheduleDocument* fScheduleDocument;

public:
	// Construction/Destruction

	TCommonView();

	virtual void DoPostCreate(TDocument* itsDocument); 		// OVERRIDE

	virtual void Initialize();								// OVERRIDE

	// Drawing
	virtual void AdornCol(GridCoordinate aCol, const VRect& area);	// OVERRIDE
	virtual void AdornRow(GridCoordinate aRow, const VRect& area);	// OVERRIDE

	// Target management
	virtual void ResignedWindowTarget();					// OVERRIDE
	virtual void SetTargetSelection(bool redraw);		// OVERRIDE

	// Accessor
	virtual TScheduleDocument* GetScheduleDocument();
};

//----------------------------------------------------------------------------------------
// CLASS TScheduleView
//----------------------------------------------------------------------------------------

class TScheduleView: public TCommonView
{
	DeclareClass(TScheduleView);

public:

	// Drawing
	virtual void DoHighlightAppointments(HLState fromHL, HLState toHL);
	virtual void DrawAppointments(const VRect& area);

	// Mouse Commands
	virtual TAppointment* GetTopAppointment(const VPoint& theMouse);
	virtual TAppointment* GetTopSelectedAppointment(const VPoint& theMouse);
	virtual bool MouseInAppointment(TAppointment* anAppointment,
					const VPoint& theMouse);

	virtual bool MouseInKnob(TAppointment* anAppointment,
					const VPoint& theMouse, KnobSelector& whichKnob);

	// Appointment selections
	virtual void DoHighlightAppointment(TAppointment* anAppointment, HLState fromHL, HLState toHL);
	virtual void DoHighlightSelection(HLState fromHL, HLState toHL);	// OVERRIDE

	virtual void GetAppointmentKnob(TAppointment* anAppointment,
								const KnobSelector whichKnob, VRect& knobRect);

	virtual TAppointment* GetSelectedAppointment();
	virtual bool HaveAnyAppointments();
	virtual bool HaveAnySelectedAppointments();
	virtual bool HaveASelectedAppointment();

	virtual void HighlightAppointment(TAppointment* anAppointment, bool highlight);

	virtual void ScrollSelectionIntoView(bool redraw);			// OVERRIDE
	virtual void SetEmptySelection(bool highlight);				// OVERRIDE

	// Appointment viewing
	virtual void Draw(const VRect& area);							// OVERRIDE
	virtual void DrawAppointment(TAppointment* anAppointment, const VRect& area);
	virtual void GetAppointmentExtent(TAppointment* anAppointment, VRect& itsExtent);
	virtual void GetExtentFromInfo(DayIndex theDay,
							TimeIndex theTime, TimeIndex theDuration, VRect& itsExtent);
	virtual void InsetForText(VRect& itsExtent);
	virtual void InvalidateAppointment(TAppointment* anAppointment);
	virtual void InvalidateDescription(TAppointment* anAppointment);

	// Edition Manager Support
	virtual void ComputeBorderRegion(TSectionAdorner* anAdorner);
	virtual void GetDayRegion(DayIndex theDay, RgnHandle theRegion);
	virtual void RevealSelection(TDesignator* aSelection);

	// Dependency operations
#ifdef MACAPP_R_13
	virtual void DoUpdate(ChangeID_AC theChange, MDependable_AC* changedObject, void* changeData, CDependencySpace_AC* dependencySpace);			// OVERRIDE
#else
	virtual void DoUpdate(ChangeID theChange, TObject* changedObject, TObject* changeData, TDependencySpace* dependencySpace);			// OVERRIDE
#endif
};

//----------------------------------------------------------------------------------------
// CLASS TSubscriberView
//----------------------------------------------------------------------------------------

class TSubscriberView: public TScheduleView
{
	DeclareClass(TSubscriberView);

public:

	// Appointment viewing
	virtual void GetExtentFromInfo(DayIndex theDay,
				TimeIndex theTime, TimeIndex theDuration, VRect& itsExtent);	// OVERRIDE

	// Edition Manager Support
	virtual void GetDayRegion(DayIndex theDay, RgnHandle theRegion);	// OVERRIDE

	// Drawing
	virtual void DoHighlightAppointments(HLState fromHL, HLState toHL);	// OVERRIDE
	virtual void DrawAppointments(const VRect& area);					// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TDayView
//----------------------------------------------------------------------------------------

class TDayView: public TCommonView
{
	DeclareClass(TDayView);

public:

	// Initialization
	virtual void ReadFields(TStream* aStream);		// OVERRIDE
		// Sets fJustification. (Work around MacApp deficiency.)

	// Drawing
	virtual CStr255 GetText(const GridCell& aCell) const;
	virtual void GetText(GridCell aCell, CStr255& itsText) { itsText = GetText(aCell); }
	
	virtual void HighlightCells(CONST GRIDVIEWDESIGNATORPTR theCells,
									  HLState fromHL,
									  HLState toHL);			// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TSubscriberDayView
//----------------------------------------------------------------------------------------

class TSubscriberDayView: public TDayView
{
	DeclareClass(TSubscriberDayView);

public:

	// Drawing
	virtual CStr255 GetText(const GridCell& aCell) const;	// OVERRIDE
	virtual void GetText(GridCell aCell, CStr255& itsText) { itsText = GetText(aCell); }
};

//----------------------------------------------------------------------------------------
// CLASS TSubscriberNameView
//----------------------------------------------------------------------------------------

class TSubscriberNameView: public TStaticText
{
	DeclareClass(TSubscriberNameView);

public:

	// Drawing
	virtual void Draw(const VRect& area);				// OVERRIDE
};

//----------------------------------------------------------------------------------------
// CLASS TTimeView
//----------------------------------------------------------------------------------------

class TTimeView: public TCommonView
{
	DeclareClass(TTimeView);

public:

	// Initialization
	virtual void ReadFields(TStream* aStream);		// OVERRIDE
		// Sets fJustification. (Work around MacApp deficiency.)

	// Drawing
	virtual CStr255 GetText(const GridCell& aCell) const;
	virtual void GetText(GridCell aCell, CStr255& itsText) { itsText = GetText(aCell); }
		// This routine gets the text to be draw in a given cell. IT MUST BE OVERRIDDEN!

};

//----------------------------------------------------------------------------------------
// CLASS TAppointmentClipView
//----------------------------------------------------------------------------------------

class TAppointmentClipView: public TView
{
	DeclareClass(TAppointmentClipView);

public:

	// Clipboard operations
	virtual bool ContainsClipType(ResType aType);					// OVERRIDE
	virtual void WriteToDeskScrap();									// OVERRIDE

};

//----------------------------------------------------------------------------------------
// CLASS TLookupDialog: Looks up appointments remotely.
//----------------------------------------------------------------------------------------

class TLookupDialog : public TDialogView
{
	DeclareClass(TLookupDialog);

private:
	TButton*		fDoItButton;
	TStaticText*	fDayText;
	TStaticText*	fTimeText;
	TStaticText*	fResponseText;

	AEAddressDesc	fTargetAddress;
	DayIndex		fDay;
	TimeIndex		fTime;

	bool			fInProgress;

public:

	// Construction/Destruction
	~TLookupDialog();											// OVERRIDE
	virtual void HaveAddress(TargetID& theTarget);
	TLookupDialog() {this->Initialize();}
	virtual void Initialize();										// OVERRIDE

	// View operations
	virtual void DoPostCreate(TDocument* itsDocument);				// OVERRIDE

	// Command operations
	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);						// OVERRIDE
	virtual void DoMenuCommand(CommandNumber aCommandNumber);		// OVERRIDE
	virtual void DoSetupMenus();									// OVERRIDE

	// Miscelleanous routines

	virtual void HaveResponse(ConstCStr255Param theResponse);
	virtual void InProgress(const bool lockItUp);

	virtual void IncrementDay(DayIndex offset);
	virtual void IncrementTime(TimeIndex offset);
};

//----------------------------------------------------------------------------------------
// CLASS TArrowsControl: (From DemoDialogs example.)
//----------------------------------------------------------------------------------------

class TArrowsControl : public TPicture
{
	DeclareClass(TArrowsControl);

private:
	long fLastChange;							// tick count of last call to DoChoice

public:
	TArrowsControl() {this->Initialize();}
	virtual void Initialize();						// OVERRIDE

	virtual void TrackMouse(TrackPhase aTrackPhase,
											 VPoint&		/*anchorPoint*/,
											 VPoint&		/*previousPoint*/,
											 VPoint&		nextPoint,
											 bool		/*mouseDidMove*/);	// OVERRIDE
};


//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

extern void InitUScheduleView();
	// Call this routine at initialization time

#endif
