//----------------------------------------------------------------------------------------
// UTemporaryObjects.cp
// Copyright 1991-1995 Apple Computer, Inc.
// 3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5). Fixed embedded FailInfo's
//----------------------------------------------------------------------------------------

#ifndef __UTEMPORARYOBJECTS__
#include "UTemporaryObjects.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

//----------------------------------------------------------------------------------------
// CTemporaryFileStream
//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CTemporaryFileStream::CTemporaryFileStream(TFile* aFile)
{
	fStream = NULL;

	// Setup the FailInfo to catch any failures while this object is in scope
	fFailInfo.Reset();		// ensure that fFailInfo is in an initial state
	fFailInfo.SetCleanupProc(CTemporaryFileStream::CallCleanup, this);

#ifdef MACAPP_R_13
	fStream = TH_new TFileStream(aFile);
#else
	TFileStream* stream = TH_new TFileStream;
	stream->IFileStream(aFile);
	fStream = stream;
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryFileStream::Cleanup()
{
	fStream = (TFileStream*)FreeIfObject(fStream);
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryFileStream::CallCleanup(void* context)
{
	((CTemporaryFileStream*) context)->Cleanup();
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CTemporaryFileStream::~CTemporaryFileStream()
{
	this->Cleanup();
	Success(fFailInfo);
}

//----------------------------------------------------------------------------------------
// CTemporaryCountingStream
//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CTemporaryCountingStream::CTemporaryCountingStream()
{
	fStream = NULL;

	// Setup the FailInfo to catch any failures while this object is in scope
	fFailInfo.Reset();		// ensure that fFailInfo is in an initial state
	fFailInfo.SetCleanupProc(CTemporaryCountingStream::CallCleanup, this);

#ifdef MACAPP_R_13
	fStream = TH_new TCountingStream;
#else
	TCountingStream* stream = TH_new TCountingStream;
	stream->ICountingStream();
	fStream = stream;
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryCountingStream::Cleanup()
{
	fStream = (TCountingStream*)FreeIfObject(fStream);
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryCountingStream::CallCleanup(void* context)
{
	((CTemporaryCountingStream*) context)->Cleanup();
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CTemporaryCountingStream::~CTemporaryCountingStream()
{
	this->Cleanup();
	Success(fFailInfo);
}

//----------------------------------------------------------------------------------------
// CTemporaryHandleStream
//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CTemporaryHandleStream::CTemporaryHandleStream(long growth) :
	fHandle(NULL),
	fStream(NULL)
{
	Set(NewPermHandle(0), growth);
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CTemporaryHandleStream::CTemporaryHandleStream(Handle aHandle, long growth) :
	fHandle(NULL),
	fStream(NULL)
{
	Set(aHandle, growth);
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryHandleStream::Set(Handle aHandle, long growth)
{
	// Setup the FailInfo to catch any failures while this object is in scope
	fFailInfo.Reset();		// ensure that fFailInfo is in an initial state
	fFailInfo.SetCleanupProc(CTemporaryHandleStream::CallCleanup, this);

#ifdef MACAPP_R_13
	fStream = TH_new THandleStream(aHandle, growth);
#else
	THandleStream* stream = TH_new THandleStream;
	stream->IHandleStream(aHandle, growth);
	fStream = stream;
#endif
	fHandle = aHandle; // set member _after_ stream initializes successfully.
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryHandleStream::Cleanup()
{
	// an failure has occurred while we were in scope - handle the failure
	fStream = (THandleStream*)FreeIfObject(fStream);
		// free stream _before_ handle, because it attempts to set the size.
	if (fHandle && !IsAResource(fHandle))
		fHandle = DisposeIfHandle(fHandle);
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryHandleStream::CallCleanup(void* context)
{
	((CTemporaryHandleStream*) context)->Cleanup();
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CTemporaryHandleStream::~CTemporaryHandleStream()
{
	this->Cleanup();
	Success(fFailInfo);
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryHandleStream::AdjustHandleSize()
{
	if (fStream)
		fStream->SetSize(fStream->GetPosition());
		// Trim the handle to its current position.
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CTemporaryHandleStream::AdjustStreamSize()
{
	if (fStream && fHandle)
		fStream->SetSize(::GetHandleSize(fHandle));
		// set the streams's position to cover all of the handle.
}

//----------------------------------------------------------------------------------------
// CPermAllocator
//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CPermAllocator::CPermAllocator(bool newPerm)
{
	fOldPerm = PermAllocation(newPerm);

	// Setup the FailInfo to catch any failures while this object is in scope
	fFailInfo.Reset();		// ensure that fFailInfo is in an initial state
	fFailInfo.SetCleanupProc(CPermAllocator::CallCleanup, this);
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CPermAllocator::Cleanup()
{
	PermAllocation(fOldPerm);	// ignore result
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

void CPermAllocator::CallCleanup(void* context)
{
	((CPermAllocator*) context)->Cleanup();
}

//----------------------------------------------------------------------------------------
#pragma segment MAGlobalRes

CPermAllocator::~CPermAllocator()
{
	this->Cleanup();
	Success(fFailInfo);
}
