/*
	UTemporaryObjects.h

	Copyright 1991-1995 Apple Computer, Inc.
	All rights reserved

	These objects facilitate using streams in C++ code. They manage their own
	internal failure handler and use overloaded operators to return appropriate
	data members.

	They are based on MacApp's CTemporaryRegion object.

	Change History:

	10/28/91	LPG	Created file
	12/20/91	LPG	Version 1.0	(uses MacApp 3.0b3)
	2/18/92		LPG	Version 1.1	(uses MacApp 3.0 Final)
	3/5/95		JS 3.1.3 (uses MacApp 3.1.3/3.5)
 */

#ifndef __UTEMPORARYOBJECTS__
#define __UTEMPORARYOBJECTS__

#ifndef __UFAILURE__
#include <UFailure.h>
#endif

#ifndef __USTREAM__
#include <UStream.h>
#endif

#ifndef __UMEMORY__
#include <UMemory.h>
#endif


class TFile;

//----------------------------------------------------------------------------------------
// CTemporaryFileStream
//----------------------------------------------------------------------------------------

class CTemporaryFileStream
{
protected:
	TFileStream* 	fStream;
	FailInfo		fFailInfo;

public:
	CTemporaryFileStream(TFile* aFile);
	~CTemporaryFileStream();

	operator TStream*();

	void Cleanup();
	static void CallCleanup(void* context);
};

//----------------------------------------------------------------------------------------
// CTemporaryCountingStream
//----------------------------------------------------------------------------------------

class CTemporaryCountingStream
{
protected:
	TCountingStream* 	fStream;
	FailInfo			fFailInfo;

public:
	CTemporaryCountingStream();
	~CTemporaryCountingStream();

	long GetBytesNeeded();

	operator TStream*();

	void Cleanup();
	static void CallCleanup(void* context);
};

//----------------------------------------------------------------------------------------
// CTemporaryHandleStream
//----------------------------------------------------------------------------------------

class CTemporaryHandleStream
{
protected:
	Handle			fHandle;
	THandleStream*	fStream;
	FailInfo		fFailInfo;

public:
	CTemporaryHandleStream(long growth = 1);
	CTemporaryHandleStream(Handle aHandle, long growth = 1);
	~CTemporaryHandleStream();
	void Set(Handle aHandle, long growth = 1);
	void AdjustHandleSize();				// adjust handle to actual size of stream
	void AdjustStreamSize();				// sets stream size to actual handle size.

	TStream* GetStream();					// accessor function

	operator Handle();						// coersion operator
	operator TStream*();					// coersion operator

	void Cleanup();
	static void CallCleanup(void* context);
};

//----------------------------------------------------------------------------------------
// CPermAllocator
//----------------------------------------------------------------------------------------

class CPermAllocator
{
protected:
	bool		 	fOldPerm;
	FailInfo		fFailInfo;

public:
	CPermAllocator(bool newPerm);
	~CPermAllocator();

	void Cleanup();
	static void CallCleanup(void* context);
};

//----------------------------------------------------------------------------------------
// inline implementations
//----------------------------------------------------------------------------------------

inline CTemporaryFileStream::operator TStream*()
{
	return fStream;
}

//----------------------------------------------------------------------------------------

inline long CTemporaryCountingStream::GetBytesNeeded()
{
	return fStream->GetPosition();
}

//----------------------------------------------------------------------------------------

inline CTemporaryCountingStream::operator TStream*()
{
	return fStream;
}

//----------------------------------------------------------------------------------------

inline TStream* CTemporaryHandleStream::GetStream()
{
	return fStream;
}

//----------------------------------------------------------------------------------------

inline CTemporaryHandleStream::operator Handle()
{
	return fHandle;
}

//----------------------------------------------------------------------------------------

inline CTemporaryHandleStream::operator TStream*()
{
	return fStream;
}

#endif
