/*
 	File:		JManager.h
 
 	Version:	Technology:	Mac OS Runtime For Java
 				Release:	MRJ SDK 1.0
 
 	Copyright:	 1984-1996 by Apple Computer, Inc.  All rights reserved.
 
 	Bugs?:		If you find a problem with this file, send the file and version
 				information (from above) and the problem description to:
 
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/
#ifndef __JMANAGER__
#define __JMANAGER__

#ifndef __CONDITIONALMACROS__
#include <ConditionalMacros.h>
#endif
#ifndef __TYPES__
#include <Types.h>
#endif
#ifndef __FILES__
#include <Files.h>
#endif
#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif
#ifndef __MENUS__
#include <Menus.h>
#endif
#ifndef __JRI__
#include <JRI.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif


enum {
	kJMVersion					= 0x10200003,					/* using Sun's 1.0.2 APIs, our current APIs. */
	kDefaultJMTime				= 0x00000400					/* how much time to give the JM library on "empty" events, in milliseconds. */
};


enum {
	kJMVersionError				= -60000L,
	kJMExceptionOccurred		= -60001L
};

typedef void *JMClientData;
/*
 * JMFrame - Java applets require a graphics context to draw themselves in. JM provides
 * a mechanism that allows clients to embed a graphics contexts anywhere. The JMFrame
 * interface provides this mechanism. At various times, JM will call back to the client to
 * register a new JMFrame, indicating whether the frame is for an applet viewer, for a modeless
 * window controlled by an applet, or for a modal window. The client should note the new frame
 * by storing a UInt32 value in it, and should also fill in the frame's callbacks.
 * If the frame is being registered for an applet viewer, nothing else is required. If the
 * frame is for a window, the appropriate kind of window should be created, and associated
 * with the frame using the client data.
*/
typedef struct OpaqueJMFrame* JMFrame;
typedef JMFrame JMFrameRef;
/*
 JMAWTContext -
 * To create a top level frame, you must use a JMAWTContext object.
 * The JMAWTContext provides a context for the AWT to request frames.
 * A AWTContext has a threadgroup associated with it - all events and processing occurs
 * there.  When you create one, it is quiescent, you must call resume before it begins executing.
*/
typedef struct OpaqueJMAWTContext* JMAWTContext;
typedef JMAWTContext JMAWTContextRef;
typedef struct OpaqueJMAppletLocator* JMAppletLocator;
typedef JMAppletLocator JMAppletLocatorRef;
typedef struct OpaqueJMAppletViewer* JMAppletViewer;
typedef JMAppletViewer JMAppletViewerRef;
/*
 * Opaque reference to abstract JM session container
*/
typedef struct OpaqueJMSession* JMSession;
typedef JMSession JMSessionRef;

enum {
	eDontCheckCode				= 0,
	eCheckRemoteCode			= 1,
	eCheckAllCode				= 2
};


enum {
	eNoNetworkAccess			= 0,
	eAppletHostAccess			= 1,
	eUnrestrictedAccess			= 2
};


enum {
	eNoFSAccess					= 0,
	eLocalAppletAccess			= 1,
	eAllFSAccess				= 2
};

/*
 * JMSecurityOptions - To control Java security, you must provide a JMSecurityOptions object when opening
 * a new JM session. fVerifyMode should be set to one of the CodeVerifierOptions values. If you set fUseHttpProxy,
 * the host name should be copied to fHttpProxy, along with the port to use in fHttpPort. Similarly for the firewall
 * proxy. fNetworkAccess should be set to one of the NetworkSecurityOptions values. Finally, fRestrictClassAccess,
 * if set to true, will prevent an applet from accessing files other than class files.
*/
struct JMSecurityOptions {
	UInt32 							fVersion;					/* should be set to kJMVersion */


																/* Byte Code Verification*/

	UInt16 							fVerifyMode;				/* eDontCheckCode, eCheckRemoteCode, eCheckAllCode */


																/* HTTP Proxy */

	Boolean 						fUseHttpProxy;
	char 							fHttpProxy[255];
	UInt32 							fHttpProxyPort;


																/* FTP Proxy */

	Boolean 						fUseFTPProxy;
	char 							fFTPProxy[255];
	UInt32 							fFTPProxyPort;


																/* Firewall Proxy*/

	Boolean 						fUseFirewallProxy;
	char 							fFirewallProxy[255];
	UInt32 							fFirewallProxyPort;


																/* Applet access to the Network*/

	UInt16 							fNetworkAccess;				/* eNoNetworkAccess, eAppletHostAccess, eUnrestrictedAccess */


																/* Applet access to the Filesystem*/

	UInt16 							fFileSystemAccess;			/* eNoFSAccess, eLocalAppletAccess, eAllFSAccess */


																/* Applet access to non-core class files*/

	Boolean 						fRestrictClassAccess;


																/* Restrict Applet definition in core class files*/

	Boolean 						fRestrictClassDefine;
};
typedef struct JMSecurityOptions JMSecurityOptions;

typedef void (*JMConsoleProcPtr)(JMSessionRef session, const char *message, UInt32 messageLen);
/* return zero when no more can be read.*/
typedef SInt32 (*JMConsoleReadProcPtr)(JMSessionRef session, char *buffer, SInt32 maxBufferLength);
/*
 callee owns the handle and will dispose it.  Only called when class not found
 in class path, but before going to the net.
*/
typedef Handle (*JMFindClassProcPtr)(JMSessionRef session, const char *className);
struct JMSessionCallbacks {
	UInt32 							fVersion;					/* should be set to kJMVersion */
	JMConsoleProcPtr 				fStandardOutput;			/* JM will route "stdout" to this function. */
	JMConsoleProcPtr 				fStandardError;				/* JM will route "stderr" to this function. */
	JMConsoleReadProcPtr 			fStandardIn;				/* read from console - can be nil for default behavior (no console IO) */
};
typedef struct JMSessionCallbacks JMSessionCallbacks;

/*
 * JMGetVersion returns the version of the currently installed JManager library
 */
 
extern unsigned long JMGetVersion();

/*
 * JMOpenSession creates a new Java runtime. If you pass nil for security, "good" defaults are
 * used. The security can be updated at any time, but it's a good idea to call JMGetSecurityOptions
 * to make a copy of the default options. The client data parameter lets a client associate an arbitrary
 * tagged pointer with the session.
*/
extern OSStatus JMOpenSession(JMSessionRef *session, const JMSecurityOptions *security, const JMSessionCallbacks *callbacks, JMClientData data);

extern OSStatus JMCloseSession(JMSessionRef session);

/*
 * Client data getter/setter functions.
*/
extern OSStatus JMGetSessionData(JMSessionRef session, JMClientData *data);

extern OSStatus JMSetSessionData(JMSessionRef session, JMClientData data);

/*
 * Security Options getter/setter functions.
*/
extern OSStatus JMGetSecurityOptions(JMSessionRef session, JMSecurityOptions *options);

extern OSStatus JMSetSecurityOptions(JMSessionRef session, const JMSecurityOptions *options);

/*
 * Java defines system-wide properties that applets can use to make queries about the
 * host system. Many of these properties correspond to defaults provided by "Internet Config."
 * JMPutSessionProperty can be used by a client program to modify various system-wide properties.
*/
extern OSStatus JMGetSessionProperty(JMSessionRef session, const char *propertyName, char *propertyValue, UInt32 valueLength);

extern OSStatus JMPutSessionProperty(JMSessionRef session, const char *propertyName, const char *propertyValue);

/*
 * Append the target of the FSSpec to the class path.
 * If a file, .zip or other known archive file - not a .class file
*/
extern OSStatus JMAddToClassPath(JMSessionRef session, const FSSpec *spec);

/*
 * Utility returns (client owned) null terminated handle containing "file://xxxx", or nil if fnfErr
*/
extern Handle JMFSSToURL(JMSessionRef session, const FSSpec *spec);

/*
 * Turns "file:///disk/file" into an FSSpec.  other handlers return paramErr
*/
extern OSStatus JMURLToFSS(JMSessionRef session, const char *urlString, FSSpec *spec);

/*
 * JMIdle gives time to all Java threads. Giving more time makes Java programs run faster,
 * but can reduce overall system responsiveness. JMIdle will return sooner if low-level (user)
 * events appear in the event queue.
*/
extern OSStatus JMIdle(JMSessionRef session, UInt32 jmTimeMillis);

/*
 * JMGetCurrenvEnv gives access to the underlying JRI interface, if available.
 * This gives the client more control over the underlying Java runtime, without
 * exposing the implementation too much.
*/
extern JRIRuntimeInstance *JMGetRuntimeInstance(JMSessionRef session);

extern JRIEnv *JMGetCurrentEnv(JMSessionRef session);

/*
 * This struct holds the callbacks for an JMFrame - things like set the port, restore it, etc.
 * Note that the contents of this structure are copied into the destination JMFrameRef.
*/
/*
 * the ProcPtr types:
 *		JMFrameSetupPortProcPtr - SetPort(), SetClip, SetOrigin() - return <something>
 *		JMFrameRestorePortProcPtr - SetPort(<something>), etc.  See Example
 *	<b>	JMFrameResizeRequest - On a resize request on the frame, return true to take result
 *		false to ignore resize request. (Feel free to modify desired)
 *		JMFrameInvalRectProcPtr - just call InvalRect, if you're a window
 *		JMFrameShowHideProcPtr - whatever's appropriate to hide the frame.
 *		JMSetTitleProcPtr - if the frame can have its title changed.
 *		JMCheckUpdateProcPtr - check for update - call JMFrameUpdate if needed
*/
typedef void *(*JMFrameSetupPortProcPtr)(JMFrameRef frame);
typedef void (*JMFrameRestorePortProcPtr)(JMFrameRef frame, void *param);
typedef Boolean (*JMFrameResizeRequestProcPtr)(JMFrameRef frame, Rect *desired);
typedef void (*JMFrameInvalRectProcPtr)(JMFrameRef frame, const Rect *r);
typedef void (*JMFrameShowHideProcPtr)(JMFrameRef frame, Boolean showFrameRequested);
typedef void (*JMSetTitleProcPtr)(JMFrameRef frame, Str255 title);
typedef void (*JMCheckUpdateProcPtr)(JMFrameRef frame);
struct JMFrameCallbacks {
	UInt32 							fVersion;					/* should be set to kJMVersion */

	JMFrameSetupPortProcPtr 		fSetupPort;
	JMFrameRestorePortProcPtr 		fRestorePort;
	JMFrameResizeRequestProcPtr 	fResizeRequest;
	JMFrameInvalRectProcPtr 		fInvalRect;
	JMFrameShowHideProcPtr 			fShowHide;
	JMSetTitleProcPtr 				fSetTitle;
	JMCheckUpdateProcPtr 			fCheckUpdate;
};
typedef struct JMFrameCallbacks JMFrameCallbacks;

extern OSStatus JMGetFrameData(JMFrameRef frame, JMClientData *data);

extern OSStatus JMSetFrameData(JMFrameRef frame, JMClientData data);

extern OSStatus JMGetFrameSize(JMFrameRef frame, Rect *result);

extern OSStatus JMSetFrameSize(JMFrameRef frame, const Rect *newSize);

/* note that only the _size_ (width, height) are used from this Rect*/
/*
 * Dispatch a particular event to an embedded frame
*/
extern OSStatus JMFrameClick(JMFrameRef frame, Point localPos, short modifiers);

extern OSStatus JMFrameKey(JMFrameRef frame, char asciiChar, char keyCode, short modifiers);

extern OSStatus JMFrameKeyRelease(JMFrameRef frame, char asciiChar, char keyCode, short modifiers);

extern OSStatus JMFrameUpdate(JMFrameRef frame, RgnHandle updateRgn);

extern OSStatus JMFrameActivate(JMFrameRef frame, Boolean activate);

extern OSStatus JMFrameResume(JMFrameRef frame, Boolean resume);

extern OSStatus JMFrameMouseOver(JMFrameRef frame, Point localPos, short modifiers);

extern OSStatus JMFrameShowHide(JMFrameRef frame, Boolean showFrame);

extern OSStatus JMFrameGoAway(JMFrameRef frame);

extern JMAWTContextRef JMGetFrameContext(JMFrameRef frame);

/*
 * Forward enum declares
*/

enum {
	eBorderlessModelessWindowFrame = 0,
	eModelessWindowFrame		= 1,
	eModalWindowFrame			= 2
};

typedef short JMFrameKind;
typedef OSStatus (*JMRequestFrameProcPtr)(JMAWTContextRef context, JMFrameRef newFrame, JMFrameKind kind, UInt32 width, UInt32 height, Boolean resizeable, JMFrameCallbacks *callbacks);
typedef OSStatus (*JMReleaseFrameProcPtr)(JMAWTContextRef context, JMFrameRef oldFrame);
typedef SInt16 (*JMUniqueMenuIDProcPtr)(JMAWTContextRef context, Boolean isSubmenu);
typedef void (*JMExceptionOccurredProcPtr)(JMAWTContextRef context, const char *exceptionName, const char *exceptionMsg, const char *stackTrace);
struct JMAWTContextCallbacks {
	UInt32 							fVersion;					/* should be set to kJMVersion */
	JMRequestFrameProcPtr 			fRequestFrame;				/* a new frame is being created. */
	JMReleaseFrameProcPtr 			fReleaseFrame;				/* an existing frame is being destroyed. */
	JMUniqueMenuIDProcPtr 			fUniqueMenuID;				/* a new menu will be created with this id. */
	JMExceptionOccurredProcPtr 		fExceptionOccurred;			/* just some notification that some recent operation caused an exception.  You can't do anything really from here. */
};
typedef struct JMAWTContextCallbacks JMAWTContextCallbacks;

extern OSStatus JMNewAWTContext(JMAWTContextRef *context, JMSessionRef session, const JMAWTContextCallbacks *callbacks, JMClientData data);

extern OSStatus JMDisposeAWTContext(JMAWTContextRef context);

extern OSStatus JMGetAWTContextData(JMAWTContextRef context, JMClientData *data);

extern OSStatus JMSetAWTContextData(JMAWTContextRef context, JMClientData data);

extern OSStatus JMResumeAWTContext(JMAWTContextRef context);

extern OSStatus JMSuspendAWTContext(JMAWTContextRef context);

extern OSStatus JMCountAWTContextFrames(JMAWTContextRef context, UInt32 *frameCount);

extern OSStatus JMGetAWTContextFrame(JMAWTContextRef context, UInt32 frameIndex, JMFrameRef *frame);

extern OSStatus JMMenuSelected(JMAWTContextRef context, MenuHandle hMenu, short menuItem);

/*
*
 * JRI Access APIs
*/
extern OSStatus JMExecMethodInContext(JMAWTContextRef context, jref objref, JRIMethodID methodID, UInt32 argCount, JRIValue args[]);

extern OSStatus JMExecStaticMethodInContext(JMAWTContextRef context, JRIClassID classID, JRIMethodID methodID, UInt32 argCount, JRIValue args[]);

/*
*
 * JMAppletLocator - Since Java applets are always referenced by a Uniform Resource Locator
 * (see RFC 1737, http://www.w3.org/pub/WWW/Addressing/rfc1738.txt), we provide an object
 * that encapsulates the information about a set of applets. A JMAppletLocator is built
 * by providing a base URL, which must point at a valid HTML document containing applet
 * tags. To save a network transaction, the contents of the document may be passed optionally. 
 *
 * You can also use a JMLocatorInfoBlock for a synchronous resolution of the applet,
 * assuming that you already have the info for the tag.
*/

enum {
	eLocatorNoErr				= 0,							/* the html was retrieved successfully*/
	eHostNotFound				= 1,							/* the host specified by the url could not be found*/
	eFileNotFound				= 2,							/* the file could not be found on the host*/
	eLocatorTimeout				= 3,							/* a timeout occurred retrieving the html text*/
	eLocatorKilled				= 4								/* in response to a JMDisposeAppletLocator before it has completed*/
};

typedef short JMLocatorErrors;
typedef void (*JMFetchCompleted)(JMAppletLocatorRef ref, JMLocatorErrors status);
struct JMAppletLocatorCallbacks {
	UInt32 							fVersion;					/* should be set to kJMVersion */
	JMFetchCompleted 				fCompleted;					/* called when the html has been completely fetched */
};
typedef struct JMAppletLocatorCallbacks JMAppletLocatorCallbacks;

/*
 * These structures are used to pass pre-parsed parameter
 * tags to the AppletLocator.  Implies synchronous semantics.
*/
struct JMLIBOptionalParams {
	char *							fParamName;					/* could be from a <parameter name=foo value=bar> or "zipbase", etc*/
	char *							fParamValue;				/* the value of this optional tag*/
};
typedef struct JMLIBOptionalParams JMLIBOptionalParams;

struct JMLocatorInfoBlock {
	UInt32 							fVersion;					/* should be set to kJMVersion */

																/* These are required to be present and not nil*/

	char *							fBaseURL;					/* the URL of this applet's host page*/
	char *							fAppletCode;				/* code= parameter*/
	short 							fWidth;						/* width= parameter*/
	short 							fHeight;					/* height= parameter*/


																/* These are optional parameters*/

	int 							fOptionalParameterCount;	/* how many in this array*/
	JMLIBOptionalParams *			fParams;					/* pointer to an array of these (points to first element)*/
};
typedef struct JMLocatorInfoBlock JMLocatorInfoBlock;

extern OSStatus JMNewAppletLocator(JMAppletLocatorRef *locatorRef, JMSessionRef session, const JMAppletLocatorCallbacks *callbacks, const char *url, const char *htmlText, JMClientData data);

extern OSStatus JMNewAppletLocatorFromInfo(JMAppletLocatorRef *locatorRef, JMSessionRef session, const JMLocatorInfoBlock *info, JMClientData data);

extern OSStatus JMDisposeAppletLocator(JMAppletLocatorRef locatorRef);

extern OSStatus JMGetAppletLocatorData(JMAppletLocatorRef locatorRef, JMClientData *data);

extern OSStatus JMSetAppletLocatorData(JMAppletLocatorRef locatorRef, JMClientData data);

extern OSStatus JMCountApplets(JMAppletLocatorRef locatorRef, UInt32 *appletCount);

extern OSStatus JMGetAppletDimensions(JMAppletLocatorRef locatorRef, UInt32 appletIndex, UInt32 *width, UInt32 *height);

extern OSStatus JMGetAppletTag(JMAppletLocatorRef locatorRef, UInt32 appletIndex, Handle *hBuffer);

extern OSStatus JMGetAppletName(JMAppletLocatorRef locatorRef, UInt32 appletIndex, char *buffer, UInt32 bufferLength);

/*
 * JMAppletViewer - Applets are instantiated, one by one, by specifying a JMAppletLocator and
 * a zero-based index (Macintosh API's usually use one-based indexing, the Java language
 * uses zero, however.). The resulting applet is encapsulated in a JMAppletViewer object. 
 * Since applets can have one or more visible areas to draw in, one or more JMFrame objects
 * may be requested while the viewer is being created, or at a later time, thus the client
 * must provide callbacks to satisfy these requests.
*/
typedef void (*JMShowDocumentProcPtr)(JMAppletViewerRef viewer, const char *urlString, Boolean inNewWindow);
typedef void (*JMSetStatusMsgProcPtr)(JMAppletViewerRef viewer, const char *statusMsg);
struct JMAppletViewerCallbacks {
	UInt32 							fVersion;					/* should be set to kJMVersion */
	JMShowDocumentProcPtr 			fShowDocument;				/* go to a url, optionally in a new window */
	JMSetStatusMsgProcPtr 			fSetStatusMsg;				/* applet changed status message */
};
typedef struct JMAppletViewerCallbacks JMAppletViewerCallbacks;

extern OSStatus JMNewAppletViewer(JMAppletViewerRef *viewer, JMAWTContextRef context, JMAppletLocatorRef locatorRef, UInt32 appletIndex, const JMAppletViewerCallbacks *callbacks, JMClientData data);

extern OSStatus JMDisposeAppletViewer(JMAppletViewerRef viewer);

extern OSStatus JMGetAppletViewerData(JMAppletViewerRef viewer, JMClientData *data);

extern OSStatus JMSetAppletViewerData(JMAppletViewerRef viewer, JMClientData data);

/*
 * JMReloadApplet reloads viewer's applet from the source.
 * JMRestartApplet reinstantiates the applet without reloading.
*/
extern OSStatus JMReloadApplet(JMAppletViewerRef viewer);

extern OSStatus JMRestartApplet(JMAppletViewerRef viewer);

/*
 * JMSuspendApplet tells the Java thread scheduler to stop executing the viewer's applet.
 * JMResumeApplet resumes execution of the viewer's applet.
*/
extern OSStatus JMSuspendApplet(JMAppletViewerRef viewer);

extern OSStatus JMResumeApplet(JMAppletViewerRef viewer);

/*
 * To get back to the JMAppletViewerRef instance from whence a frame came,
 * as well as the ultimate frame parent (the one created _for_ the applet viewer)
*/
extern OSStatus JMGetFrameViewer(JMFrameRef frame, JMAppletViewerRef *viewer, JMFrameRef *parentFrame);

/*
 * To get a ref back to the Frame that was created for this JMAppletViewerRef
*/
extern OSStatus JMGetViewerFrame(JMAppletViewerRef viewer, JMFrameRef *frame);

/*
 * The properties dialog is an easy way to let the
 * user change security properties.  Use
 * JMGetSecurityOptions to retrive them if
 * you want to store them in a file.  (This is
 * a modeless dialog, so you should probably
 * just check when shutting down.)
 * A context is required, not an AppletViewer, so
 * since a lot of these are global (proxies)
*/
extern OSStatus JMShowPropsDialog(JMAWTContextRef context);


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#ifdef __cplusplus
}
#endif

#endif /* __JMANAGER__ */

