/*
 	File:		JRI.h
 
 	Version:	Technology:	Mac OS Runtime For Java
 				Release:	MRJ SDK 1.0
 
 	Copyright:	 1984-1996 by Apple Computer, Inc.  All rights reserved.
 
 	Bugs?:		If you find a problem with this file, send the file and version
 				information (from above) and the problem description to:
 
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/
#ifndef __JRI__
#define __JRI__

#ifndef __CONDITIONALMACROS__
#include <ConditionalMacros.h>
#endif
#include <assert.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdarg.h>
#include <JRICMacros.h>

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

#if defined(__MWERKS__)
#if defined(GENERATING68K) && !GENERATINGCFM
#if !__option(fourbyteints) 
#error You need to define 'Struct Alignment: 68k' for your project.
#endif
#endif
#elif defined(__SC__)
#elif macintosh && applec
#else
#endif
#define JRI_PUBLIC_API(ResultType)		pascal ResultType
#define JRI_CALLBACK
#ifndef FAR		/* for non-Win16 */
#define FAR
#endif
typedef char *va_list;
/******************************************************************************/
/* Java Scalar Types */
typedef unsigned char jbool;
typedef char jbyte;
typedef short jchar;
typedef short jshort;
#ifdef IS_64
typedef unsigned int juint;
typedef int jint;
#else
typedef unsigned long juint;
typedef long jint;
#endif
typedef float jfloat;
typedef double jdouble;
typedef juint jsize;
/*
******************************************************************************
 * jlong : long long (64-bit signed integer type) support.
 *****************************************************************************
*/
#ifdef HAVE_LONG_LONG
typedef long long			jlong;
typedef unsigned long long	julong;
#define jlong_MAXINT		0x7fffffffffffffffLL
#define jlong_MININT		0x8000000000000000LL
#define jlong_ZERO			0x0LL
#else
#ifdef IS_LITTLE_ENDIAN
struct jlong {
	juint 							lo;
	juint 							hi;
};
typedef struct jlong jlong;

typedef jlong julong;
#else
struct jlong {
	juint 							hi;
	juint 							lo;
};
typedef struct jlong jlong;

typedef jlong julong;
#endif
extern jlong jlong_MAXINT, jlong_MININT, jlong_ZERO;
#endif
/******************************************************************************/
/*
** JDK Stuff -- This stuff is still needed while we're using the JDK
** dynamic linking strategy to call native methods.
*/
#ifdef IS_64
union JRI_JDK_stack_item {
																/* Non pointer items */
	jint 							i;
	jfloat 							f;
	jint 							o;
																/* Pointer items */
	void *							h;
	void *							p;
	unsigned char *					addr;

	double 							d;
	long 							l;							/* == 64bits! */

};
typedef union JRI_JDK_stack_item JRI_JDK_stack_item;

typedef JRI_JDK_stack_item JRI_JDK_stack_item;
#else
union JRI_JDK_stack_item {
																/* Non pointer items */
	jint 							i;
	jfloat 							f;
	jint 							o;
																/* Pointer items */
	void *							h;
	void *							p;
	unsigned char *					addr;
};
typedef union JRI_JDK_stack_item JRI_JDK_stack_item;

typedef JRI_JDK_stack_item JRI_JDK_stack_item;
#endif
union JRI_JDK_Java8Str {
	jint 							x[2];
	jdouble 						d;
	jlong 							l;
	void *							p;
	float 							f;
};
typedef union JRI_JDK_Java8Str JRI_JDK_Java8Str;

typedef JRI_JDK_Java8Str JRI_JDK_Java8;
/*
******************************************************************************
 * Types
 *****************************************************************************
*/
typedef void *JRIRef;
typedef void *JRIGlobalRef;
typedef jint JRIInterfaceID[4];
typedef jint JRIFieldID;
typedef jint JRIMethodID;
/* synonyms: */
typedef JRIGlobalRef jglobal;
typedef JRIRef jref;
union JRIValue {
	jbool 							z;
	jbyte 							b;
	jchar 							c;
	jshort 							s;
	jint 							i;
	jlong 							l;
	jfloat 							f;
	jdouble 						d;
	jref 							r;
};
typedef union JRIValue JRIValue;

typedef JRIValue JRIValue;
typedef JRIValue jvalue;

enum {
	JRIFalse					= 0,
	JRITrue						= 1
};

typedef short JRIBoolean;

enum {
	JRIUninitialized			= -1
};

typedef short JRIConstant;
/* convenience types: */
typedef JRIRef jbooleanArray;
typedef JRIRef jbyteArray;
typedef JRIRef jcharArray;
typedef JRIRef jshortArray;
typedef JRIRef jintArray;
typedef JRIRef jlongArray;
typedef JRIRef jfloatArray;
typedef JRIRef jdoubleArray;
typedef JRIRef jobjectArray;
typedef JRIRef jstringArray;
typedef JRIRef jarrayArray;
#define JRIConstructorMethodName	"<init>"
/*
******************************************************************************
 * Signature Construction Macros
 *****************************************************************************
*/
/*
** These macros can be used to construct signature strings. Hopefully their names
** are a little easier to remember than the single character they correspond to.
** For example, to specify the signature of the method:
**
**	public int read(byte b[], int off, int len);
**
** you could write something like this in C:
**
**	char* readSig = JRISigMethod(JRISigArray(JRISigByte)
**								 JRISigInt
**								 JRISigInt) JRISigInt;
**
** Of course, don't put commas between the types.
*/
#define JRISigArray(T)		"[" T
#define JRISigByte			"B"
#define JRISigChar			"C"
#define JRISigClass(name)	"L" name ";"
#define JRISigFloat		"F"
#define JRISigDouble		"D"
#define JRISigMethod(args)	"(" args ")"
#define JRISigNoArgs		""
#define JRISigInt			"I"
#define JRISigLong			"J"
#define JRISigShort		"S"
#define JRISigVoid			"V"
#define JRISigBoolean		"Z"
/******************************************************************************/
/* Current API Version Constants. */

enum {
	JRIMajorVersion				= 0x0001,
	JRIMinorVersion				= 0x0000,
	JRICurrentVersion			= (JRIMajorVersion << 16) | JRIMinorVersion
};

/*
******************************************************************************
 * JRIEnv Interface
 *****************************************************************************
*/
/* The type of a JRIEnv instance. */
typedef struct JRIEnv JRIEnv;
struct java_lang_Class;
struct java_lang_Throwable;
struct java_lang_String;
typedef struct java_lang_Class* JRIClassID;
typedef struct java_lang_Throwable* JRIThrowableID;
typedef struct java_lang_String* JRIStringID;
/*
 * JRIEnvInterface Proc Definitions
*/
typedef JRIClassID (*LoadClassProc)(JRIEnv *env, jbyte *classData, jsize dataLen);
typedef JRIClassID (*FindClassProc)(JRIEnv *env, const char *className);
typedef void (*ThrowProc)(JRIEnv *env, JRIThrowableID throwableID);
typedef void (*ThrowNewProc)(JRIEnv *env, JRIClassID classID, const char *message);
typedef JRIThrowableID (*ExceptionOccurredProc)(JRIEnv *env);
typedef void (*ExceptionDescribeProc)(JRIEnv *env);
typedef void (*ExceptionClearProc)(JRIEnv *env);
typedef jglobal (*NewGlobalRefProc)(JRIEnv *env, jref localRef);
typedef void (*DisposeGlobalRefProc)(JRIEnv *env, jglobal globalRef);
typedef jref (*GetGlobalRefProc)(JRIEnv *env, jglobal globalRef);
typedef void (*SetGlobalRefProc)(JRIEnv *env, jglobal globalRef, jref localRef);
typedef jbool (*IsSameObjectProc)(JRIEnv *env, jref refa, jref refb);
typedef jref (*NewObjectProc)(JRIEnv *env, JRIClassID jclass, JRIMethodID constructorID, ...);
typedef jref (*NewObjectVProc)(JRIEnv *env, JRIClassID jclass, JRIMethodID constructorID, va_list args);
typedef jref (*NewObjectAProc)(JRIEnv *env, JRIClassID jclass, JRIMethodID constructorID, JRIValue *args);
typedef JRIClassID (*GetObjectClassProc)(JRIEnv *env, jref objref);
typedef jbool (*IsInstanceOfProc)(JRIEnv *env, jref objref, JRIClassID classID);
typedef JRIMethodID (*GetMethodIDProc)(JRIEnv *env, JRIClassID classID, const char *methodName, const char *methodSignature);
typedef jref (*CallMethodProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jref (*CallMethodVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jref (*CallMethodAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jbool (*CallMethodBooleanProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jbool (*CallMethodBooleanVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jbool (*CallMethodBooleanAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jbyte (*CallMethodByteProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jbyte (*CallMethodByteVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jbyte (*CallMethodByteAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jchar (*CallMethodCharProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jchar (*CallMethodCharVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jchar (*CallMethodCharAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jshort (*CallMethodShortProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jshort (*CallMethodShortVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jshort (*CallMethodShortAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jint (*CallMethodIntProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jint (*CallMethodIntVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jint (*CallMethodIntAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jlong (*CallMethodLongProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jlong (*CallMethodLongVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jlong (*CallMethodLongAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jfloat (*CallMethodFloatProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jfloat (*CallMethodFloatVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jfloat (*CallMethodFloatAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jdouble (*CallMethodDoubleProc)(JRIEnv *env, jref objref, JRIMethodID methodID, ...);
typedef jdouble (*CallMethodDoubleVProc)(JRIEnv *env, jref objref, JRIMethodID methodID, va_list args);
typedef jdouble (*CallMethodDoubleAProc)(JRIEnv *env, jref objref, JRIMethodID methodID, JRIValue *args);
typedef jint (*GetFieldIDProc)(JRIEnv *env, JRIClassID classID, const char *fieldName, const char *fieldSignature);
typedef jref (*GetFieldProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jbool (*GetFieldBooleanProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jbyte (*GetFieldByteProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jchar (*GetFieldCharProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jshort (*GetFieldShortProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jint (*GetFieldIntProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jlong (*GetFieldLongProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jfloat (*GetFieldFloatProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef jdouble (*GetFieldDoubleProc)(JRIEnv *env, jref objref, JRIFieldID fieldID);
typedef void (*SetFieldProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jref value);
typedef void (*SetFieldBooleanProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jbool value);
typedef void (*SetFieldByteProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jbyte value);
typedef void (*SetFieldCharProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jchar value);
typedef void (*SetFieldShortProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jshort value);
typedef void (*SetFieldIntProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jint value);
typedef void (*SetFieldLongProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jlong value);
typedef void (*SetFieldFloatProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jfloat value);
typedef void (*SetFieldDoubleProc)(JRIEnv *env, jref objref, JRIFieldID fieldID, jdouble value);
typedef jbool (*IsSubclassOfProc)(JRIEnv *env, JRIClassID classID, JRIClassID superID);
typedef jint (*GetStaticMethodIDProc)(JRIEnv *env, JRIClassID classID, const char *methodName, const char *methodSignature);
typedef jref (*CallStaticMethodProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jref (*CallStaticMethodVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jref (*CallStaticMethodAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jbool (*CallStaticMethodBooleanProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jbool (*CallStaticMethodBooleanVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jbool (*CallStaticMethodBooleanAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jbyte (*CallStaticMethodByteProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jbyte (*CallStaticMethodByteVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jbyte (*CallStaticMethodByteAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jchar (*CallStaticMethodCharProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jchar (*CallStaticMethodCharVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jchar (*CallStaticMethodCharAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jshort (*CallStaticMethodShortProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jshort (*CallStaticMethodShortVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jshort (*CallStaticMethodShortAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jint (*CallStaticMethodIntProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jint (*CallStaticMethodIntVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jint (*CallStaticMethodIntAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jlong (*CallStaticMethodLongProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jlong (*CallStaticMethodLongVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jlong (*CallStaticMethodLongAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jfloat (*CallStaticMethodFloatProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jfloat (*CallStaticMethodFloatVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jfloat (*CallStaticMethodFloatAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jdouble (*CallStaticMethodDoubleProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, ...);
typedef jdouble (*CallStaticMethodDoubleVProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, va_list args);
typedef jdouble (*CallStaticMethodDoubleAProc)(JRIEnv *env, JRIClassID classID, JRIMethodID methodID, JRIValue *args);
typedef jint (*GetStaticFieldIDProc)(JRIEnv *env, JRIClassID classID, const char *fieldName, const char *fieldSignature);
typedef jref (*GetStaticFieldProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jbool (*GetStaticFieldBooleanProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jbyte (*GetStaticFieldByteProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jchar (*GetStaticFieldCharProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jshort (*GetStaticFieldShortProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jint (*GetStaticFieldIntProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jlong (*GetStaticFieldLongProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jfloat (*GetStaticFieldFloatProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef jdouble (*GetStaticFieldDoubleProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID);
typedef void (*SetStaticFieldProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jref value);
typedef void (*SetStaticFieldBooleanProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jbool value);
typedef void (*SetStaticFieldByteProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jbyte value);
typedef void (*SetStaticFieldCharProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jchar value);
typedef void (*SetStaticFieldShortProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jshort value);
typedef void (*SetStaticFieldIntProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jint value);
typedef void (*SetStaticFieldLongProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jlong value);
typedef void (*SetStaticFieldFloatProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jfloat value);
typedef void (*SetStaticFieldDoubleProc)(JRIEnv *env, JRIClassID classID, JRIFieldID fieldID, jdouble value);
typedef JRIStringID (*NewStringProc)(JRIEnv *env, const jchar *unicodeChars, jint length);
typedef jint (*GetStringLengthProc)(JRIEnv *env, JRIStringID stringID);
typedef const jchar *(*GetStringCharsProc)(JRIEnv *env, JRIStringID stringID);
typedef JRIStringID (*NewStringUTFProc)(JRIEnv *env, const jbyte *utfString, jint utfLength);
typedef jint (*GetStringUTFLengthProc)(JRIEnv *env, JRIStringID stringID);
typedef const jbyte *(*GetStringUTFCharsProc)(JRIEnv *env, JRIStringID stringID);
typedef jref (*NewScalarArrayProc)(JRIEnv *env, jint elementCount, const char *elementSignature, const jbyte *initialElements);
typedef jint (*GetScalarArrayLengthProc)(JRIEnv *env, jref arrayref);
typedef jbyte *(*GetScalarArrayElementsProc)(JRIEnv *env, jref arrayref);
typedef jref (*NewObjectArrayProc)(JRIEnv *env, jint elementCount, JRIClassID elementClassID, jref initialElement);
typedef jint (*GetObjectArrayLengthProc)(JRIEnv *env, jref arrayref);
typedef jref (*GetObjectArrayElementProc)(JRIEnv *env, jref arrayref, jint index);
typedef void (*SetObjectArrayElementProc)(JRIEnv *env, jref arrayref, jint index, jref value);
typedef void (*RegisterNativesProc)(JRIEnv *env, JRIClassID classID, char **nativeMethodNames, void **nativeMethodProcs);
typedef void (*UnregisterNativesProc)(JRIEnv *env, JRIClassID classID);
/*
 * JRIEnvInterface Structure Definition
*/
struct JRIEnvInterface {
	void *							reserved0;
	void *							reserved1;
	void *							reserved2;

	LoadClassProc 					LoadClass;
	FindClassProc 					FindClass;

	ThrowProc 						ThrowProcPtr;
	ThrowNewProc 					ThrowNew;
	ExceptionOccurredProc 			ExceptionOccurred;
	ExceptionDescribeProc 			ExceptionDescribe;
	ExceptionClearProc 				ExceptionClear;

	NewGlobalRefProc 				NewGlobalRef;
	DisposeGlobalRefProc 			DisposeGlobalRef;
	GetGlobalRefProc 				GetGlobalRef;
	SetGlobalRefProc 				SetGlobalRef;

	IsSameObjectProc 				IsSameObject;
	NewObjectProc 					NewObject;
	NewObjectVProc 					NewObjectV;
	NewObjectAProc 					NewObjectA;

	GetObjectClassProc 				GetObjectClass;
	IsInstanceOfProc 				IsInstanceOf;

	GetMethodIDProc 				GetMethodID;
	CallMethodProc 					CallMethod;
	CallMethodVProc 				CallMethodV;
	CallMethodAProc 				CallMethodA;
	CallMethodBooleanProc 			CallMethodBoolean;
	CallMethodBooleanVProc 			CallMethodBooleanV;
	CallMethodBooleanAProc 			CallMethodBooleanA;
	CallMethodByteProc 				CallMethodByte;
	CallMethodByteVProc 			CallMethodByteV;
	CallMethodByteAProc 			CallMethodByteA;
	CallMethodCharProc 				CallMethodChar;
	CallMethodCharVProc 			CallMethodCharV;
	CallMethodCharAProc 			CallMethodCharA;
	CallMethodShortProc 			CallMethodShort;
	CallMethodShortVProc 			CallMethodShortV;
	CallMethodShortAProc 			CallMethodShortA;
	CallMethodIntProc 				CallMethodInt;
	CallMethodIntVProc 				CallMethodIntV;
	CallMethodIntAProc 				CallMethodIntA;
	CallMethodLongProc 				CallMethodLong;
	CallMethodLongVProc 			CallMethodLongV;
	CallMethodLongAProc 			CallMethodLongA;
	CallMethodFloatProc 			CallMethodFloat;
	CallMethodFloatVProc 			CallMethodFloatV;
	CallMethodFloatAProc 			CallMethodFloatA;
	CallMethodDoubleProc 			CallMethodDouble;
	CallMethodDoubleVProc 			CallMethodDoubleV;
	CallMethodDoubleAProc 			CallMethodDoubleA;

	GetFieldIDProc 					GetFieldID;
	GetFieldProc 					GetField;
	GetFieldBooleanProc 			GetFieldBoolean;
	GetFieldByteProc 				GetFieldByte;
	GetFieldCharProc 				GetFieldChar;
	GetFieldShortProc 				GetFieldShort;
	GetFieldIntProc 				GetFieldInt;
	GetFieldLongProc 				GetFieldLong;
	GetFieldFloatProc 				GetFieldFloat;
	GetFieldDoubleProc 				GetFieldDouble;

	SetFieldProc 					SetField;
	SetFieldBooleanProc 			SetFieldBoolean;
	SetFieldByteProc 				SetFieldByte;
	SetFieldCharProc 				SetFieldChar;
	SetFieldShortProc 				SetFieldShort;
	SetFieldIntProc 				SetFieldInt;
	SetFieldLongProc 				SetFieldLong;
	SetFieldFloatProc 				SetFieldFloat;
	SetFieldDoubleProc 				SetFieldDouble;

	IsSubclassOfProc 				IsSubclassOf;

	GetStaticMethodIDProc 			GetStaticMethodID;
	CallStaticMethodProc 			CallStaticMethod;
	CallStaticMethodVProc 			CallStaticMethodV;
	CallStaticMethodAProc 			CallStaticMethodA;
	CallStaticMethodBooleanProc 	CallStaticMethodBoolean;
	CallStaticMethodBooleanVProc 	CallStaticMethodBooleanV;
	CallStaticMethodBooleanAProc 	CallStaticMethodBooleanA;
	CallStaticMethodByteProc 		CallStaticMethodByte;
	CallStaticMethodByteVProc 		CallStaticMethodByteV;
	CallStaticMethodByteAProc 		CallStaticMethodByteA;
	CallStaticMethodCharProc 		CallStaticMethodChar;
	CallStaticMethodCharVProc 		CallStaticMethodCharV;
	CallStaticMethodCharAProc 		CallStaticMethodCharA;
	CallStaticMethodShortProc 		CallStaticMethodShort;
	CallStaticMethodShortVProc 		CallStaticMethodShortV;
	CallStaticMethodShortAProc 		CallStaticMethodShortA;
	CallStaticMethodIntProc 		CallStaticMethodInt;
	CallStaticMethodIntVProc 		CallStaticMethodIntV;
	CallStaticMethodIntAProc 		CallStaticMethodIntA;
	CallStaticMethodLongProc 		CallStaticMethodLong;
	CallStaticMethodLongVProc 		CallStaticMethodLongV;
	CallStaticMethodLongAProc 		CallStaticMethodLongA;
	CallStaticMethodFloatProc 		CallStaticMethodFloat;
	CallStaticMethodFloatVProc 		CallStaticMethodFloatV;
	CallStaticMethodFloatAProc 		CallStaticMethodFloatA;
	CallStaticMethodDoubleProc 		CallStaticMethodDouble;
	CallStaticMethodDoubleVProc 	CallStaticMethodDoubleV;
	CallStaticMethodDoubleAProc 	CallStaticMethodDoubleA;

	GetStaticFieldIDProc 			GetStaticFieldID;
	GetStaticFieldProc 				GetStaticField;
	GetStaticFieldBooleanProc 		GetStaticFieldBoolean;
	GetStaticFieldByteProc 			GetStaticFieldByte;
	GetStaticFieldCharProc 			GetStaticFieldChar;
	GetStaticFieldShortProc 		GetStaticFieldShort;
	GetStaticFieldIntProc 			GetStaticFieldInt;
	GetStaticFieldLongProc 			GetStaticFieldLong;
	GetStaticFieldFloatProc 		GetStaticFieldFloat;
	GetStaticFieldDoubleProc 		GetStaticFieldDouble;
	SetStaticFieldProc 				SetStaticField;
	SetStaticFieldBooleanProc 		SetStaticFieldBoolean;
	SetStaticFieldByteProc 			SetStaticFieldByte;
	SetStaticFieldCharProc 			SetStaticFieldChar;
	SetStaticFieldShortProc 		SetStaticFieldShort;
	SetStaticFieldIntProc 			SetStaticFieldInt;
	SetStaticFieldLongProc 			SetStaticFieldLong;
	SetStaticFieldFloatProc 		SetStaticFieldFloat;
	SetStaticFieldDoubleProc 		SetStaticFieldDouble;

	NewStringProc 					NewString;
	GetStringLengthProc 			GetStringLength;
	GetStringCharsProc 				GetStringChars;
	NewStringUTFProc 				NewStringUTF;
	GetStringUTFLengthProc 			GetStringUTFLength;
	GetStringUTFCharsProc 			GetStringUTFChars;

	NewScalarArrayProc 				NewScalarArray;
	GetScalarArrayLengthProc 		GetScalarArrayLength;
	GetScalarArrayElementsProc 		GetScalarArrayElements;

	NewObjectArrayProc 				NewObjectArray;
	GetObjectArrayLengthProc 		GetObjectArrayLength;
	GetObjectArrayElementProc 		GetObjectArrayElement;
	SetObjectArrayElementProc 		SetObjectArrayElement;

	RegisterNativesProc 			RegisterNatives;
	UnregisterNativesProc 			UnregisterNatives;
};
typedef struct JRIEnvInterface JRIEnvInterface;

typedef JRIEnvInterface JRIEnvInterface;
/*
******************************************************************************
 * JRIEnv
 *****************************************************************************
*/
struct JRIEnv {
	const JRIEnvInterface* interface;
#ifdef __cplusplus
	// C++ bindings
	JRIClassID LoadClass(jbyte* classData, jsize dataLen) { return interface->LoadClass(this, classData, dataLen); }
	JRIClassID FindClass(const char* className) { return interface->FindClass(this, className); }
	void Throw(JRIThrowableID throwableID) { interface->ThrowProcPtr(this, throwableID); }
	void ThrowNew(JRIClassID classID, const char* message) { interface->ThrowNew(this, classID, message); }
	JRIThrowableID ExceptionOccurred() { return interface->ExceptionOccurred(this); }
	void ExceptionDescribe() { interface->ExceptionDescribe(this); }
	void ExceptionClear() { interface->ExceptionClear(this); }
	jglobal	NewGlobalRef(jref localRef) { return interface->NewGlobalRef(this, localRef); }
	void DisposeGlobalRef(jglobal globalRef) { interface->DisposeGlobalRef(this, globalRef); }
	jref GetGlobalRef(jglobal globalRef) { return interface->GetGlobalRef(this, globalRef); }
	void SetGlobalRef(jglobal globalRef, jref localRef) { interface->SetGlobalRef(this, globalRef, localRef); }
	jbool IsSameObject(jref refa, jref refb) { return interface->IsSameObject(this, refa, refb); }
	jref NewObject(JRIClassID classID, JRIMethodID constructorID, ...) { va_list args; va_start(args, constructorID); jref result = interface->NewObjectV(this, classID, constructorID, args); va_end(args); return result; }
	jref NewObjectV(JRIClassID classID, JRIMethodID constructorID, va_list args) { return interface->NewObjectV(this, classID, constructorID, args); }
	jref NewObjectA(JRIClassID classID, JRIMethodID constructorID, JRIValue* args) { return interface->NewObjectA(this, classID, constructorID, args); }
	JRIClassID GetObjectClass(jref objref) { return interface->GetObjectClass(this, objref); }
	jbool IsInstanceOf(jref objref, JRIClassID classID) { return interface->IsInstanceOf(this, objref, classID); }
	JRIMethodID GetMethodID(JRIClassID classID, const char* methodName, const char* methodSignature) { return interface->GetMethodID(this, classID, methodName, methodSignature); }
	jref CallMethod(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jref result = interface->CallMethodV(this, objref, methodID, args); va_end(args); return result; }
	jref CallMethodV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodV(this, objref, methodID, args); }
	jref CallMethodA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodA(this, objref, methodID, args); }
	jbool CallMethodBoolean(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jbool result = interface->CallMethodBooleanV(this, objref, methodID, args); va_end(args); return result; }
	jbool CallMethodBooleanV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodBooleanV(this, objref, methodID, args); }
	jbool CallMethodBooleanA(jref objref, JRIMethodID methodID, JRIValue* args){ return interface->CallMethodBooleanA(this, objref, methodID, args); }
	jbyte CallMethodByte(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jbyte result = interface->CallMethodByteV(this, objref, methodID, args); va_end(args); return result; }
	jbyte CallMethodByteV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodByteV(this, objref, methodID, args); }
	jbyte CallMethodByteA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodByteA(this, objref, methodID, args); }
	jchar CallMethodChar(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jchar result = interface->CallMethodCharV(this, objref, methodID, args); va_end(args); return result; }
	jchar CallMethodCharV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodCharV(this, objref, methodID, args); }
	jchar CallMethodCharA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodCharA(this, objref, methodID, args); }
	jshort CallMethodShort(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jchar result = interface->CallMethodCharV(this, objref, methodID, args); va_end(args); return result; }
	jshort CallMethodShortV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodShortV(this, objref, methodID, args); }
	jshort CallMethodShortA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodShortA(this, objref, methodID, args); }
	jint CallMethodInt(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jchar result = interface->CallMethodIntV(this, objref, methodID, args); va_end(args); return result; }
	jint CallMethodIntV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodIntV(this, objref, methodID, args); }
	jint CallMethodIntA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodIntA(this, objref, methodID, args); }
	jlong CallMethodLong(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jlong result = interface->CallMethodLongV(this, objref, methodID, args); va_end(args); return result; }
	jlong CallMethodLongV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodLongV(this, objref, methodID, args); }
	jlong CallMethodLongA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodLongA(this, objref, methodID, args); }
	jfloat CallMethodFloat(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jchar result = interface->CallMethodFloatV(this, objref, methodID, args); va_end(args); return result; }
	jfloat CallMethodFloatV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodFloatV(this, objref, methodID, args); }
	jfloat CallMethodFloatA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodFloatA(this, objref, methodID, args); }
	jdouble CallMethodDouble(jref objref, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jchar result = interface->CallMethodDoubleV(this, objref, methodID, args); va_end(args); return result; }
	jdouble CallMethodDoubleV(jref objref, JRIMethodID methodID, va_list args) { return interface->CallMethodDoubleV(this, objref, methodID, args); }
	jdouble CallMethodDoubleA(jref objref, JRIMethodID methodID, JRIValue* args) { return interface->CallMethodDoubleA(this, objref, methodID, args); }
	jint GetFieldID(JRIClassID classID, const char* fieldName, const char* fieldSignature) { return interface->GetFieldID(this, classID, fieldName, fieldSignature); }
	jref GetField(jref objref, JRIFieldID fieldID) { return interface->GetField(this, objref, fieldID); }
	jbool GetFieldBoolean(jref objref, JRIFieldID fieldID) { return interface->GetFieldBoolean(this, objref, fieldID); }
	jbyte GetFieldByte(jref objref, JRIFieldID fieldID) { return interface->GetFieldByte(this, objref, fieldID); }
	jchar GetFieldChar(jref objref, JRIFieldID fieldID) { return interface->GetFieldChar(this, objref, fieldID); }
	jshort GetFieldShort(jref objref, JRIFieldID fieldID) { return interface->GetFieldShort(this, objref, fieldID); }
	jint GetFieldInt(jref objref, JRIFieldID fieldID) { return interface->GetFieldInt(this, objref, fieldID); }
	jlong GetFieldLong(jref objref, JRIFieldID fieldID) { return interface->GetFieldLong(this, objref, fieldID); }
	jfloat GetFieldFloat(jref objref, JRIFieldID fieldID) { return interface->GetFieldFloat(this, objref, fieldID); }
	jdouble GetFieldDouble(jref objref, JRIFieldID fieldID) { return interface->GetFieldDouble(this, objref, fieldID); }
	void SetField(jref objref, JRIFieldID fieldID, jref value) { interface->SetField(this, objref, fieldID, value); }
	void SetFieldBoolean(jref objref, JRIFieldID fieldID, jbool value) { interface->SetFieldBoolean(this, objref, fieldID, value); }
	void SetFieldByte(jref objref, JRIFieldID fieldID, jbyte value) { interface->SetFieldByte(this, objref, fieldID, value); }
	void SetFieldChar(jref objref, JRIFieldID fieldID, jchar value) { interface->SetFieldChar(this, objref, fieldID, value); }
	void SetFieldShort(jref objref, JRIFieldID fieldID, jshort value) { interface->SetFieldShort(this, objref, fieldID, value); }
	void SetFieldInt(jref objref, JRIFieldID fieldID, jint value) { interface->SetFieldInt(this, objref, fieldID, value); }
	void SetFieldLong(jref objref, JRIFieldID fieldID, jlong value) { interface->SetFieldLong(this, objref, fieldID, value); }
	void SetFieldFloat(jref objref, JRIFieldID fieldID, jfloat value) { interface->SetFieldFloat(this, objref, fieldID, value); }
	void SetFieldDouble(jref objref, JRIFieldID fieldID, jdouble value) { interface->SetFieldDouble(this, objref, fieldID, value); }
	jbool IsSubclassOf(JRIClassID classID, JRIClassID superID) { return interface->IsSubclassOf(this, classID, superID); }
	jint GetStaticMethodID(JRIClassID classID, const char* methodName, const char* methodSignature) { return interface->GetStaticMethodID(this, classID, methodName, methodSignature); }
	jref CallStaticMethod(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jref result = interface->CallStaticMethodV(this, classID, methodID, args); va_end(args); return result; }
	jref CallStaticMethodV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodV(this, classID, methodID, args); }
	jref CallStaticMethodA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodA(this, classID, methodID, args); }
	jbool CallStaticMethodBoolean(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jbool result = interface->CallStaticMethodBooleanV(this, classID, methodID, args); va_end(args); return result; }
	jbool CallStaticMethodBooleanV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodBooleanV(this, classID, methodID, args); }
	jbool CallStaticMethodBooleanA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodBooleanA(this, classID, methodID, args); }
	jbyte CallStaticMethodByte(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jbyte result = interface->CallStaticMethodByteV(this, classID, methodID, args); va_end(args); return result; }
	jbyte CallStaticMethodByteV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodByteV(this, classID, methodID, args); }
	jbyte CallStaticMethodByteA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodByteA(this, classID, methodID, args); }
	jchar CallStaticMethodChar(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jchar result = interface->CallStaticMethodCharV(this, classID, methodID, args); va_end(args); return result; }
	jchar CallStaticMethodCharV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodCharV(this, classID, methodID, args); }
	jchar CallStaticMethodCharA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodCharA(this, classID, methodID, args); }
	jshort CallStaticMethodShort(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jshort result = interface->CallStaticMethodShortV(this, classID, methodID, args); va_end(args); return result; }
	jshort CallStaticMethodShortV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodShortV(this, classID, methodID, args); }
	jshort CallStaticMethodShortA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodShortA(this, classID, methodID, args); }
	jint CallStaticMethodInt(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jint result = interface->CallStaticMethodIntV(this, classID, methodID, args); va_end(args); return result; }
	jint CallStaticMethodIntV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodIntV(this, classID, methodID, args); }
	jint CallStaticMethodIntA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodIntA(this, classID, methodID, args); }
	jlong CallStaticMethodLong(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jlong result = interface->CallStaticMethodLongV(this, classID, methodID, args); va_end(args); return result; }
	jlong CallStaticMethodLongV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodLongV(this, classID, methodID, args); }
	jlong CallStaticMethodLongA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodLongA(this, classID, methodID, args); }
	jfloat CallStaticMethodFloat(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jfloat result = interface->CallStaticMethodFloatV(this, classID, methodID, args); va_end(args); return result; }
	jfloat CallStaticMethodFloatV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodFloatV(this, classID, methodID, args); }
	jfloat CallStaticMethodFloatA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodFloatA(this, classID, methodID, args); }
	jdouble CallStaticMethodDouble(JRIClassID classID, JRIMethodID methodID, ...) { va_list args; va_start(args, methodID); jdouble result = interface->CallStaticMethodDoubleV(this, classID, methodID, args); va_end(args); return result; }
	jdouble CallStaticMethodDoubleV(JRIClassID classID, JRIMethodID methodID, va_list args) { return interface->CallStaticMethodDoubleV(this, classID, methodID, args); }
	jdouble CallStaticMethodDoubleA(JRIClassID classID, JRIMethodID methodID, JRIValue* args) { return interface->CallStaticMethodDoubleA(this, classID, methodID, args); }
	jint GetStaticFieldID(JRIClassID classID, const char* fieldName, const char* fieldSignature) { return interface->GetStaticFieldID(this, classID, fieldName, fieldSignature); }
	jref GetStaticField(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticField(this, classID, fieldID); }
	jbool GetStaticFieldBoolean(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldBoolean(this, classID, fieldID); }
	jbyte GetStaticFieldByte(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldByte(this, classID, fieldID); }
	jchar GetStaticFieldChar(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldChar(this, classID, fieldID); }
	jshort GetStaticFieldShort(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldShort(this, classID, fieldID); }
	jint GetStaticFieldInt(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldInt(this, classID, fieldID); }
	jlong GetStaticFieldLong(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldLong(this, classID, fieldID); }
	jfloat GetStaticFieldFloat(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldFloat(this, classID, fieldID); }
	jdouble GetStaticFieldDouble(JRIClassID classID, JRIFieldID fieldID) { return interface->GetStaticFieldDouble(this, classID, fieldID); }
	void SetStaticField(JRIClassID classID, JRIFieldID fieldID, jref value) { interface->SetStaticField(this, classID, fieldID, value); }
	void SetStaticFieldBoolean(JRIClassID classID, JRIFieldID fieldID, jbool value) { interface->SetStaticFieldBoolean(this, classID, fieldID, value); }
	void SetStaticFieldByte(JRIClassID classID, JRIFieldID fieldID, jbyte value) { interface->SetStaticFieldByte(this, classID, fieldID, value); }
	void SetStaticFieldChar(JRIClassID classID, JRIFieldID fieldID, jchar value) { interface->SetStaticFieldChar(this, classID, fieldID, value); }
	void SetStaticFieldShort(JRIClassID classID, JRIFieldID fieldID, jshort value) { interface->SetStaticFieldShort(this, classID, fieldID, value); }
	void SetStaticFieldInt(JRIClassID classID, JRIFieldID fieldID, jint value) { interface->SetStaticFieldInt(this, classID, fieldID, value); }
	void SetStaticFieldLong(JRIClassID classID, JRIFieldID fieldID, jlong value) { interface->SetStaticFieldLong(this, classID, fieldID, value); }
	void SetStaticFieldFloat(JRIClassID classID, JRIFieldID fieldID, jfloat value) { interface->SetStaticFieldFloat(this, classID, fieldID, value); }
	void SetStaticFieldDouble(JRIClassID classID, JRIFieldID fieldID, jdouble value) { interface->SetStaticFieldDouble(this, classID, fieldID, value); }
	JRIStringID NewString(const jchar* unicodeChars, jint length) { return interface->NewString(this, unicodeChars, length); }
	jint GetStringLength(JRIStringID stringID) { return interface->GetStringLength(this, stringID); }
	const jchar* GetStringChars(JRIStringID stringID) { return interface->GetStringChars(this, stringID); }
	JRIStringID NewStringUTF(const jbyte* utfString, jint utfLength) { return interface->NewStringUTF(this, utfString, utfLength); }
	jint GetStringUTFLength(JRIStringID stringID) { return interface->GetStringUTFLength(this, stringID); }
	const jbyte* GetStringUTFChars(JRIStringID stringID) { return interface->GetStringUTFChars(this, stringID); }
	jref NewScalarArray(jint elementCount, const char* elementSignature, const jbyte* initialElements) { return interface->NewScalarArray(this, elementCount, elementSignature, initialElements); }
	jint GetScalarArrayLength(jref arrayref) { return interface->GetScalarArrayLength(this, arrayref); }
	jbyte* GetScalarArrayElements(jref arrayref) { return interface->GetScalarArrayElements(this, arrayref); }
	jref NewObjectArray(jint elementCount, JRIClassID elementClassID, jref initialElement) { return interface->NewObjectArray(this, elementCount, elementClassID, initialElement); }
	jint GetObjectArrayLength(jref arrayref) { return interface->GetObjectArrayLength(this, arrayref); }
	jref GetObjectArrayElement(jref arrayref, jint index) { return interface->GetObjectArrayElement(this, arrayref, index); }
	void SetObjectArrayElement(jref arrayref, jint index, jref value) { interface->SetObjectArrayElement(this, arrayref, index, value); }
	void RegisterNatives(JRIClassID classID, char** nativeMethodNames, void** nativeMethodProcs) { interface->RegisterNatives(this, classID, nativeMethodNames, nativeMethodProcs); }
	void UnregisterNatives(JRIClassID classID) { interface->UnregisterNatives(this, classID); }
#endif /* __cplusplus */
};
/*
******************************************************************************
 * Environments
 *****************************************************************************
*/
extern pascal JRIEnv *JRI_GetCurrentEnv(void );

/*
******************************************************************************
 * JRIRuntimeInstance / JRIRuntimeInterface
 *****************************************************************************
*/

enum {
	JRIIOMode_Unrestricted		= -1,
	JRIIOMode_None				= 0,
	JRIIOMode_AllowStdin		= 0x01,
	JRIIOMode_AllowStdout		= 0x02,
	JRIIOMode_AllowSocket		= 0x04,
	JRIIOMode_AllowFileInput	= 0x08,
	JRIIOMode_AllowFileOutput	= 0x10
};


enum {
	JRIFSMode_Unrestricted		= 0,							/* no C level filesystem checks */
	JRIFSMode_None				= 1								/* no filesystem access allowed */
};


enum {
	JRIRTMode_Unrestricted		= 0,							/* no C level runtime checks */
	JRIRTMode_None				= 1								/* no runtime access allowed */
};


enum {
	JRICollectionStarted		= 0,
	JRICollectionEnded			= 1
};


enum {
	JRIVerifyNone				= 0,
	JRIVerifyRemote				= 1,
	JRIVerifyAll				= 2
};

typedef struct JRIRuntimeInterface JRIRuntimeInterface;
typedef struct JRIRuntimeInstance JRIRuntimeInstance;
typedef short JRIIOModeFlags;
typedef short JRIFSModeFlags;
typedef short JRIRTModeFlags;
typedef short JRIRuntimeEvent;
typedef short JRIVerifyMode;
typedef void (*JRIRuntimeEventEventProc)(JRIRuntimeInstance *runtime, JRIRuntimeEvent event);
typedef struct OpaqueJavaLibParams* JavaLibParams;
typedef struct OpaqueJavaInitParams* JavaInitParams;
struct JRIRuntimeInitargs {
																/* Required arguments */
	short 							majorVersion;				/* must be JRIMajorVersion. */
	short 							minorVersion;				/* must be JRIMinorVersion. */
	jsize 							initialHeapSize;
	jsize 							maxHeapSize;
	JRIRuntimeEventEventProc 		eventProc;
	JRIVerifyMode 					verifyMode;
																/* Implementation-dependent arguments follow... */
	JavaLibParams 					libParams;
	JavaInitParams 					initParams;
};
typedef struct JRIRuntimeInitargs JRIRuntimeInitargs;

typedef JRIRuntimeInitargs JRIRuntimeInitargs;
typedef JRIRuntimeInitargs *JRIRuntimeInitargsPtr;
typedef void (*DisposeRuntimeProc)(JRIRuntimeInstance *runtime);
typedef void (*SetIOModeProc)(JRIRuntimeInstance *runtime, JRIIOModeFlags mode);
typedef void (*SetFSModeProc)(JRIRuntimeInstance *runtime, JRIFSModeFlags mode);
typedef void (*SetRTModeProc)(JRIRuntimeInstance *runtime, JRIRTModeFlags mode);
typedef JRIEnv *(*NewEnvProc)(JRIRuntimeInstance *runtime, jref thread);
typedef void (*DisposeEnvProc)(JRIEnv *env);
typedef JRIRuntimeInstance *(*GetRuntimeProc)(JRIEnv *env);
typedef jref (*GetThreadProc)(JRIEnv *env);
typedef void (*SetClassLoaderProc)(JRIEnv *env, jref classLoader);
struct JRIRuntimeInterface {
	void *							reserved0;
	void *							reserved1;
	void *							reserved2;

	DisposeRuntimeProc 				DisposeRuntime;
	SetIOModeProc 					SetIOMode;
	SetFSModeProc 					SetFSMode;
	SetRTModeProc 					SetRTMode;
	NewEnvProc 						NewEnv;
	DisposeEnvProc 					DisposeEnv;
	GetRuntimeProc 					GetRuntime;
	GetThreadProc 					GetThread;
	SetClassLoaderProc 				SetClassLoader;
};

struct JRIRuntimeInstance {
	const JRIRuntimeInterface* interface;
#ifdef __cplusplus
	// C++ bindings
	void DisposeRuntime() { interface->DisposeRuntime(this); }
	void SetIOMode(JRIIOModeFlags mode) { interface->SetIOMode(this, mode); }
	void SetFSMode(JRIFSModeFlags mode) { interface->SetFSMode(this, mode); }
	void SetRTMode(JRIRTModeFlags mode) { interface->SetRTMode(this, mode); }
	JRIEnv* NewEnv(jref thread) { return interface->NewEnv(this, thread); }
	void DisposeEnv(JRIEnv* env) { interface->DisposeEnv(env); }
	JRIRuntimeInstance* GetRuntime(JRIEnv* env) { return interface->GetRuntime(env); }
	jref GetThread(JRIEnv* env) { return interface->GetThread(env); }
	void SetClassLoader(JRIEnv* env, jref classLoader) { interface->SetClassLoader(env, classLoader); }
#endif
};

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#ifdef __cplusplus
}
#endif

#endif /* __JRI__ */

