/*
 	File:		JRICMacros.h
 
 	Version:	Technology:	Mac OS Runtime For Java
 				Release:	MRJ SDK 1.0
 
 	Copyright:	 1984-1997 by Apple Computer, Inc.  All rights reserved.
 
 	Bugs?:		If you find a problem with this file, send the file and version
 				information (from above) and the problem description to:
 
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/

/*
 * JRI Math Macros
 */

/*
** Bit masking macros.  (n must be <= 31 to be portable)
*/
#define JRI_BIT(n)			((juint)1 << (n))
#define JRI_BITMASK(n)		(JRI_BIT(n) - 1)


#ifdef HAVE_LONG_LONG

#define jlong_IS_ZERO(a)	((a) == 0)
#define jlong_EQ(a, b)		((a) == (b))
#define jlong_NE(a, b)		((a) != (b))
#define jlong_GE_ZERO(a)	((a) >= 0)
#define jlong_CMP(a, op, b)	((a) op (b))

#define jlong_AND(r, a, b)	((r) = (a) & (b))
#define jlong_OR(r, a, b)	((r) = (a) | (b))
#define jlong_XOR(r, a, b)	((r) = (a) ^ (b))
#define jlong_OR2(r, a)		((r) = (r) | (a))
#define jlong_NOT(r, a)		((r) = ~(a))

#define jlong_NEG(r, a)		((r) = -(a))
#define jlong_ADD(r, a, b)	((r) = (a) + (b))
#define jlong_SUB(r, a, b)	((r) = (a) - (b))

#define jlong_MUL(r, a, b)	((r) = (a) * (b))
#define jlong_DIV(r, a, b)	((r) = (a) / (b))
#define jlong_MOD(r, a, b)	((r) = (a) % (b))

#define jlong_SHL(r, a, b)	((r) = (a) << (b))
#define jlong_SHR(r, a, b)	((r) = (a) >> (b))
#define jlong_USHR(r, a, b)	((r) = (julong)(a) >> (b))
#define jlong_ISHL(r, a, b)	((r) = ((jlong)(a)) << (b))

#define jlong_L2I(i, l)		((i) = (int)(l))
#define jlong_L2UI(ui, l)	((ui) =(unsigned int)(l))
#define jlong_L2F(f, l)		((f) = (l))
#define jlong_L2D(d, l)		((d) = (l))

#define jlong_I2L(l, i)		((l) = (i))
#define jlong_UI2L(l, ui)	((l) = (ui))
#define jlong_F2L(l, f)		((l) = (f))
#define jlong_D2L(l, d)		((l) = (d))

#define jlong_UDIVMOD(qp, rp, a, b)  \
    (*(qp) = ((julong)(a) / (b)), \
     *(rp) = ((julong)(a) % (b)))

#else /* HAVE_LONG_LONG */

#define jlong_IS_ZERO(a)	(((a).hi == 0) && ((a).lo == 0))
#define jlong_EQ(a, b)		(((a).hi == (b).hi) && ((a).lo == (b).lo))
#define jlong_NE(a, b)		(((a).hi != (b).hi) || ((a).lo != (b).lo))
#define jlong_GE_ZERO(a)	(((a).hi >> 31) == 0)

/*
 * NB: jlong_CMP and jlong_UCMP work only for strict relationals (<, >).
 */
#define jlong_CMP(a, op, b)	(((int32)(a).hi op (int32)(b).hi) ||          \
				 (((a).hi == (b).hi) && ((a).lo op (b).lo)))
#define jlong_UCMP(a, op, b)	(((a).hi op (b).hi) ||                    \
				 (((a).hi == (b).hi) && ((a).lo op (b).lo)))

#define jlong_AND(r, a, b)	((r).lo = (a).lo & (b).lo,                    \
				 (r).hi = (a).hi & (b).hi)
#define jlong_OR(r, a, b)	((r).lo = (a).lo | (b).lo,                    \
				 (r).hi = (a).hi | (b).hi)
#define jlong_XOR(r, a, b)	((r).lo = (a).lo ^ (b).lo,                    \
				 (r).hi = (a).hi ^ (b).hi)
#define jlong_OR2(r, a)		((r).lo = (r).lo | (a).lo,                    \
				 (r).hi = (r).hi | (a).hi)
#define jlong_NOT(r, a)		((r).lo = ~(a).lo,	                          \
				 (r).hi = ~(a).hi)

#define jlong_NEG(r, a)		((r).lo = -(int32)(a).lo,                     \
				 (r).hi = -(int32)(a).hi - ((r).lo != 0))
#define jlong_ADD(r, a, b) {                                              \
    jlong _a, _b;                                                         \
    _a = a; _b = b;                                                       \
    (r).lo = _a.lo + _b.lo;                                               \
    (r).hi = _a.hi + _b.hi + ((r).lo < _b.lo);                            \
}

#define jlong_SUB(r, a, b) {                                              \
    jlong _a, _b;                                                         \
    _a = a; _b = b;                                                       \
    (r).lo = _a.lo - _b.lo;                                               \
    (r).hi = _a.hi - _b.hi - (_a.lo < _b.lo);                             \
}                                                                         \

/*
 * Multiply 64-bit operands a and b to get 64-bit result r.
 * First multiply the low 32 bits of a and b to get a 64-bit result in r.
 * Then add the outer and inner products to r.hi.
 */
#define jlong_MUL(r, a, b) {                                              \
    jlong _a, _b;                                                         \
    _a = a; _b = b;                                                       \
    jlong_MUL32(r, _a.lo, _b.lo);                                         \
    (r).hi += _a.hi * _b.lo + _a.lo * _b.hi;                              \
}

/* XXX _jlong_lo16(a) = ((a) << 16 >> 16) is better on some archs (not on mips) */
#define _jlong_lo16(a)		((a) & JRI_BITMASK(16))
#define _jlong_hi16(a)		((a) >> 16)

/*
 * Multiply 32-bit operands a and b to get 64-bit result r.
 * Use polynomial expansion based on primitive field element (1 << 16).
 */
#define jlong_MUL32(r, a, b) {                                            \
     juint _a1, _a0, _b1, _b0, _y0, _y1, _y2, _y3;                        \
     _a1 = _jlong_hi16(a), _a0 = _jlong_lo16(a);                          \
     _b1 = _jlong_hi16(b), _b0 = _jlong_lo16(b);                          \
     _y0 = _a0 * _b0;                                                     \
     _y1 = _a0 * _b1;                                                     \
     _y2 = _a1 * _b0;                                                     \
     _y3 = _a1 * _b1;                                                     \
     _y1 += _jlong_hi16(_y0);                   /* can't carry */         \
     _y1 += _y2;                                /* might carry */         \
     if (_y1 < _y2) _y3 += 1 << 16;             /* propagate */           \
     (r).lo = (_jlong_lo16(_y1) << 16) + _jlong_lo16(_y0);                \
     (r).hi = _y3 + _jlong_hi16(_y1);                                     \
}

/*
 * Divide 64-bit unsigned operand a by 64-bit unsigned operand b, setting *qp
 * to the 64-bit unsigned quotient, and *rp to the 64-bit unsigned remainder.
 * Minimize effort if one of qp and rp is null.
 */
#define jlong_UDIVMOD(qp, rp, a, b)	jlong_udivmod(qp, rp, a, b)

//extern JRI_PUBLIC_API(void)
//jlong_udivmod(julong *qp, julong *rp, julong a, julong b);

#define jlong_DIV(r, a, b) {                                              \
    jlong _a, _b;                                                         \
    juint _negative = (int32)(a).hi < 0;                                  \
    if (_negative) {                                                      \
	jlong_NEG(_a, a);                                                     \
    } else {                                                              \
	_a = a;                                                               \
    }                                                                     \
    if ((int32)(b).hi < 0) {                                              \
	_negative ^= 1;                                                       \
	jlong_NEG(_b, b);                                                     \
    } else {                                                              \
	_b = b;                                                               \
    }                                                                     \
    jlong_UDIVMOD(&(r), 0, _a, _b);                                       \
    if (_negative)                                                        \
	jlong_NEG(r, r);                                                      \
}

#define jlong_MOD(r, a, b) {                                              \
    jlong _a, _b;                                                         \
    juint _negative = (int32)(a).hi < 0;                                  \
    if (_negative) {                                                      \
	jlong_NEG(_a, a);                                                     \
    } else {                                                              \
	_a = a;                                                               \
    }                                                                     \
    if ((int32)(b).hi < 0) {                                              \
	jlong_NEG(_b, b);                                                     \
    } else {                                                              \
	_b = b;                                                               \
    }                                                                     \
    jlong_UDIVMOD(0, &(r), _a, _b);                                       \
    if (_negative)                                                        \
	jlong_NEG(r, r);                                                      \
}

/*
 * NB: b is a juint, not jlong or julong, for the shift ops.
 */
#define jlong_SHL(r, a, b) {                                              \
    if (b) {                                                              \
	jlong _a;                                                             \
        _a = a;                                                           \
        if ((b) < 32) {                                                   \
	    (r).lo = _a.lo << (b);                                            \
	    (r).hi = (_a.hi << (b)) | (_a.lo >> (32 - (b)));                  \
	} else {                                                              \
	    (r).lo = 0;                                                       \
	    (r).hi = _a.lo << ((b) & 31);                                     \
	}                                                                     \
    } else {                                                              \
	(r) = (a);                                                            \
    }                                                                     \
}

/* a is an int32, b is int32, r is jlong */
#define jlong_ISHL(r, a, b) {                                             \
    if (b) {                                                              \
	jlong _a;                                                             \
	_a.lo = (a);                                                          \
	_a.hi = 0;                                                            \
        if ((b) < 32) {                                                   \
	    (r).lo = (a) << (b);                                              \
	    (r).hi = ((a) >> (32 - (b)));                                     \
	} else {                                                              \
	    (r).lo = 0;                                                       \
	    (r).hi = (a) << ((b) & 31);                                       \
	}                                                                     \
    } else {                                                              \
	(r).lo = (a);                                                         \
	(r).hi = 0;                                                           \
    }                                                                     \
}

#define jlong_SHR(r, a, b) {                                              \
    if (b) {                                                              \
	jlong _a;                                                             \
        _a = a;                                                           \
	if ((b) < 32) {                                                       \
	    (r).lo = (_a.hi << (32 - (b))) | (_a.lo >> (b));                  \
	    (r).hi = (int32)_a.hi >> (b);                                     \
	} else {                                                              \
	    (r).lo = (int32)_a.hi >> ((b) & 31);                              \
	    (r).hi = (int32)_a.hi >> 31;                                      \
	}                                                                     \
    } else {                                                              \
	(r) = (a);                                                            \
    }                                                                     \
}

#define jlong_USHR(r, a, b) {                                             \
    if (b) {                                                              \
	jlong _a;                                                             \
        _a = a;                                                           \
	if ((b) < 32) {                                                       \
	    (r).lo = (_a.hi << (32 - (b))) | (_a.lo >> (b));                  \
	    (r).hi = _a.hi >> (b);                                            \
	} else {                                                              \
	    (r).lo = _a.hi >> ((b) & 31);                                     \
	    (r).hi = 0;                                                       \
	}                                                                     \
    } else {                                                              \
	(r) = (a);                                                            \
    }                                                                     \
}

#define jlong_L2I(i, l)		((i) = (l).lo)
#define jlong_L2UI(ui, l)	((ui) = (l).lo)
#define jlong_L2F(f, l)		{ double _d; jlong_L2D(_d, l); (f) = (float) _d; }

#define jlong_L2D(d, l) {                                                 \
    int32 _negative;                                                      \
    jlong _absval;                                                        \
                                                                          \
    _negative = (l).hi >> 31;                                             \
    if (_negative) {                                                      \
	jlong_NEG(_absval, l);                                                \
    } else {                                                              \
	_absval = l;                                                          \
    }                                                                     \
    (d) = (double)_absval.hi * 4.294967296e9 + _absval.lo;                \
    if (_negative)                                                        \
	(d) = -(d);                                                           \
}

#define jlong_I2L(l, i)		((l).hi = (i) >> 31, (l).lo = (i))
#define jlong_UI2L(l, ui)	((l).hi = 0, (l).lo = (ui))
#define jlong_F2L(l, f)		{ double _d = (double) f; jlong_D2L(l, _d); }

#define jlong_D2L(l, d) {                                                 \
    int _negative;                                                        \
    double _absval, _d_hi;                                                \
    jlong _lo_d;                                                          \
                                                                          \
    _negative = ((d) < 0);                                                \
    _absval = _negative ? -(d) : (d);                                     \
                                                                          \
    (l).hi = (juint)(_absval / 4.294967296e9);                            \
    (l).lo = 0;                                                           \
    jlong_L2D(_d_hi, l);                                                  \
    _absval -= _d_hi;                                                     \
    _lo_d.hi = 0;                                                         \
    if (_absval < 0) {                                                    \
	_lo_d.lo = (juint) -_absval;                                          \
	jlong_SUB(l, l, _lo_d);                                               \
    } else {                                                              \
	_lo_d.lo = (juint) _absval;                                           \
	jlong_ADD(l, l, _lo_d);                                               \
    }                                                                     \
                                                                          \
    if (_negative)                                                        \
	jlong_NEG(l, l);                                                      \
}

#endif /* HAVE_LONG_LONG */

#ifdef HAVE_ALIGNED_LONGLONGS
#define JRI_GET_INT64(_t,_addr) ( ((_t).x[0] = ((jint*)(_addr))[0]), \
                              ((_t).x[1] = ((jint*)(_addr))[1]),      \
                              (_t).l )
#define JRI_SET_INT64(_t, _addr, _v) ( (_t).l = (_v),                \
                                   ((jint*)(_addr))[0] = (_t).x[0], \
                                   ((jint*)(_addr))[1] = (_t).x[1] )
#else
#define JRI_GET_INT64(_t,_addr) (*(jlong*)(_addr))
#define JRI_SET_INT64(_t, _addr, _v) (*(jlong*)(_addr) = (_v))
#endif

/* If double's must be aligned on doubleword boundaries then define this */
#ifdef HAVE_ALIGNED_DOUBLES
#define JRI_GET_DOUBLE(_t,_addr) ( ((_t).x[0] = ((jint*)(_addr))[0]), \
                               ((_t).x[1] = ((jint*)(_addr))[1]),      \
                               (_t).d )
#define JRI_SET_DOUBLE(_t, _addr, _v) ( (_t).d = (_v),                \
                                    ((jint*)(_addr))[0] = (_t).x[0], \
                                    ((jint*)(_addr))[1] = (_t).x[1] )
#else
#define JRI_GET_DOUBLE(_t,_addr) (*(jdouble*)(_addr))
#define JRI_SET_DOUBLE(_t, _addr, _v) (*(jdouble*)(_addr) = (_v))
#endif

/******************************************************************************/


/*******************************************************************************
 * Specific Scalar Array Types
 ******************************************************************************/

/*
** The JRI Native Method Interface does not support boolean arrays. This
** is to allow Java runtime implementations to optimize boolean array
** storage. Using the ScalarArray operations on boolean arrays is bound
** to fail, so convert any boolean arrays to byte arrays in Java before
** passing them to a native method.
*/

#define JRI_NewByteArray(env, length, initialValues)	\
	JRI_NewScalarArray(env, length, JRISigByte, (jbyte*)(initialValues))
#define JRI_GetByteArrayLength(env, array)	\
	JRI_GetScalarArrayLength(env, array)
#define JRI_GetByteArrayElements(env, array)	\
	JRI_GetScalarArrayElements(env, array)

#define JRI_NewCharArray(env, length, initialValues)	\
	JRI_NewScalarArray(env, ((length) * sizeof(jchar)), JRISigChar, (jbyte*)(initialValues))
#define JRI_GetCharArrayLength(env, array)	\
	JRI_GetScalarArrayLength(env, array)
#define JRI_GetCharArrayElements(env, array)		   \
	((jchar*)JRI_GetScalarArrayElements(env, array))

#define JRI_NewShortArray(env, length, initialValues)	\
	JRI_NewScalarArray(env, ((length) * sizeof(jshort)), JRISigShort, (jbyte*)(initialValues))
#define JRI_GetShortArrayLength(env, array)	\
	JRI_GetScalarArrayLength(env, array)
#define JRI_GetShortArrayElements(env, array)		   \
	((jshort*)JRI_GetScalarArrayElements(env, array))

#define JRI_NewIntArray(env, length, initialValues)	\
	JRI_NewScalarArray(env, ((length) * sizeof(jint)), JRISigInt, (jbyte*)(initialValues))
#define JRI_GetIntArrayLength(env, array)	\
	JRI_GetScalarArrayLength(env, array)
#define JRI_GetIntArrayElements(env, array)		   \
	((jint*)JRI_GetScalarArrayElements(env, array))

#define JRI_NewLongArray(env, length, initialValues)	\
	JRI_NewScalarArray(env, ((length) * sizeof(jlong)), JRISigLong, (jbyte*)(initialValues))
#define JRI_GetLongArrayLength(env, array)	\
	JRI_GetScalarArrayLength(env, array)
#define JRI_GetLongArrayElements(env, array)		   \
	((jlong*)JRI_GetScalarArrayElements(env, array))

#define JRI_NewFloatArray(env, length, initialValues)	\
	JRI_NewScalarArray(env, ((length) * sizeof(jfloat)), JRISigFloat, (jbyte*)(initialValues))
#define JRI_GetFloatArrayLength(env, array)	\
	JRI_GetScalarArrayLength(env, array)
#define JRI_GetFloatArrayElements(env, array)		   \
	((jfloat*)JRI_GetScalarArrayElements(env, array))

#define JRI_NewDoubleArray(env, length, initialValues)	\
	JRI_NewScalarArray(env, ((length) * sizeof(jdouble)), JRISigDouble, (jbyte*)(initialValues))
#define JRI_GetDoubleArrayLength(env, array)	\
	JRI_GetScalarArrayLength(env, array)
#define JRI_GetDoubleArrayElements(env, array)		   \
	((jdouble*)JRI_GetScalarArrayElements(env, array))

/******************************************************************************/

/*******************************************************************************
 * JRIEnv Operations
 ******************************************************************************/

/* C bindings. */
#define JRI_LoadClass(env, buf, bufLen)	\
	((((env)->interface)->LoadClass)(env, buf, bufLen))

#define JRI_FindClass(env, name)	\
	((((env)->interface)->FindClass)(env, name))

#define JRI_Throw(env, throwableID)	\
	((((env)->interface)->Throw)(env, throwableID))

#define JRI_ThrowNew(env, classID, message)	\
	((((env)->interface)->ThrowNew)(env, classID, message))

#define JRI_ExceptionOccurred(env)	\
	((((env)->interface)->ExceptionOccurred)(env))

#define JRI_ExceptionDescribe(env)	\
	((((env)->interface)->ExceptionDescribe)(env))

#define JRI_ExceptionClear(env)	\
	((((env)->interface)->ExceptionClear)(env))

#define JRI_NewGlobalRef(env, ref)	\
	((((env)->interface)->NewGlobalRef)(env, ref))

#define JRI_DisposeGlobalRef(env, gref)	\
	((((env)->interface)->DisposeGlobalRef)(env, gref))

#define JRI_GetGlobalRef(env, gref)	\
	((((env)->interface)->GetGlobalRef)(env, gref))

#define JRI_SetGlobalRef(env, gref, ref)	\
	((((env)->interface)->SetGlobalRef)(env, gref, ref))

#define JRI_IsSameObject(env, a, b)	\
	((((env)->interface)->IsSameObject)(env, a, b))

#define JRI_NewObject(env)	(((env)->interface)->NewObject)
#define JRI_NewObjectV(env, classID, methodID, args)	\
	((((env)->interface)->NewObjectV)(env, classID, methodID, args))
#define JRI_NewObjectA(env, classID, methodID, args)	\
	((((env)->interface)->NewObjectA)(env, classID, methodID, args))

#define JRI_GetObjectClass(env, obj)	\
	((((env)->interface)->GetObjectClass)(env, obj))

#define JRI_IsInstanceOf(env, obj, classID)	\
	((((env)->interface)->IsInstanceOf)(env, obj, classID))

#define JRI_GetMethodID(env, classID, name, sig)	\
	((((env)->interface)->GetMethodID)(env, classID, name, sig))

#define JRI_CallMethod(env)	(((env)->interface)->CallMethod)
#define JRI_CallMethodV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodV)(env, obj, methodID, args))
#define JRI_CallMethodA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodA)(env, obj, methodID, args))

#define JRI_CallMethodBoolean(env)	(((env)->interface)->CallMethodBoolean)
#define JRI_CallMethodBooleanV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodBooleanV)(env, obj, methodID, args))
#define JRI_CallMethodBooleanA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodBooleanA)(env, obj, methodID, args))

#define JRI_CallMethodByte(env)	(((env)->interface)->CallMethodByte)
#define JRI_CallMethodByteV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodByteV)(env, obj, methodID, args))
#define JRI_CallMethodByteA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodByteA)(env, obj, methodID, args))

#define JRI_CallMethodChar(env)	(((env)->interface)->CallMethodChar)
#define JRI_CallMethodCharV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodCharV)(env, obj, methodID, args))
#define JRI_CallMethodCharA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodCharA)(env, obj, methodID, args))

#define JRI_CallMethodShort(env)	(((env)->interface)->CallMethodShort)
#define JRI_CallMethodShortV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodShortV)(env, obj, methodID, args))
#define JRI_CallMethodShortA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodShortA)(env, obj, methodID, args))

#define JRI_CallMethodInt(env)	(((env)->interface)->CallMethodInt)
#define JRI_CallMethodIntV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodIntV)(env, obj, methodID, args))
#define JRI_CallMethodIntA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodIntA)(env, obj, methodID, args))

#define JRI_CallMethodLong(env)	(((env)->interface)->CallMethodLong)
#define JRI_CallMethodLongV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodLongV)(env, obj, methodID, args))
#define JRI_CallMethodLongA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodLongA)(env, obj, methodID, args))

#define JRI_CallMethodFloat(env)	(((env)->interface)->CallMethodFloat)
#define JRI_CallMethodFloatV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodFloatV)(env, obj, methodID, args))
#define JRI_CallMethodFloatA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodFloatA)(env, obj, methodID, args))

#define JRI_CallMethodDouble(env)	(((env)->interface)->CallMethodDouble)
#define JRI_CallMethodDoubleV(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodDoubleV)(env, obj, methodID, args))
#define JRI_CallMethodDoubleA(env, obj, methodID, args)	\
	((((env)->interface)->CallMethodDoubleA)(env, obj, methodID, args))

#define JRI_GetFieldID(env, classID, name, sig)	\
	((((env)->interface)->GetFieldID)(env, classID, name, sig))

#define JRI_GetField(env, obj, fieldID)	\
	((((env)->interface)->GetField)(env, obj, fieldID))

#define JRI_GetFieldBoolean(env, obj, fieldID)	\
	((((env)->interface)->GetFieldBoolean)(env, obj, fieldID))

#define JRI_GetFieldByte(env, obj, fieldID)	\
	((((env)->interface)->GetFieldByte)(env, obj, fieldID))

#define JRI_GetFieldChar(env, obj, fieldID)	\
	((((env)->interface)->GetFieldChar)(env, obj, fieldID))

#define JRI_GetFieldShort(env, obj, fieldID)	\
	((((env)->interface)->GetFieldShort)(env, obj, fieldID))

#define JRI_GetFieldInt(env, obj, fieldID)	\
	((((env)->interface)->GetFieldInt)(env, obj, fieldID))

#define JRI_GetFieldLong(env, obj, fieldID)	\
	((((env)->interface)->GetFieldLong)(env, obj, fieldID))

#define JRI_GetFieldFloat(env, obj, fieldID)	\
	((((env)->interface)->GetFieldFloat)(env, obj, fieldID))

#define JRI_GetFieldDouble(env, obj, fieldID)	\
	((((env)->interface)->GetFieldDouble)(env, obj, fieldID))

#define JRI_SetField(env, obj, fieldID, value)	\
	((((env)->interface)->SetField)(env, obj, fieldID, value))

#define JRI_SetFieldBoolean(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldBoolean)(env, obj, fieldID, value))

#define JRI_SetFieldByte(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldByte)(env, obj, fieldID, value))

#define JRI_SetFieldChar(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldChar)(env, obj, fieldID, value))

#define JRI_SetFieldShort(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldShort)(env, obj, fieldID, value))

#define JRI_SetFieldInt(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldInt)(env, obj, fieldID, value))

#define JRI_SetFieldLong(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldLong)(env, obj, fieldID, value))

#define JRI_SetFieldFloat(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldFloat)(env, obj, fieldID, value))

#define JRI_SetFieldDouble(env, obj, fieldID, value)	\
	((((env)->interface)->SetFieldDouble)(env, obj, fieldID, value))

#define JRI_IsSubclassOf(env, classID, superID)	\
	((((env)->interface)->IsSubclassOf)(env, classID, superID))

#define JRI_GetStaticMethodID(env, classID, name, sig)	\
	((((env)->interface)->GetStaticMethodID)(env, classID, name, sig))

#define JRI_CallStaticMethod(env)	(((env)->interface)->CallStaticMethod)
#define JRI_CallStaticMethodV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodV)(env, classID, methodID, args))
#define JRI_CallStaticMethodA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodA)(env, classID, methodID, args))

#define JRI_CallStaticMethodBoolean(env)	(((env)->interface)->CallStaticMethodBoolean)
#define JRI_CallStaticMethodBooleanV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodBooleanV)(env, classID, methodID, args))
#define JRI_CallStaticMethodBooleanA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodBooleanA)(env, classID, methodID, args))

#define JRI_CallStaticMethodByte(env)	(((env)->interface)->CallStaticMethodByte)
#define JRI_CallStaticMethodByteV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodByteV)(env, classID, methodID, args))
#define JRI_CallStaticMethodByteA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodByteA)(env, classID, methodID, args))

#define JRI_CallStaticMethodChar(env)	(((env)->interface)->CallStaticMethodChar)
#define JRI_CallStaticMethodCharV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodCharV)(env, classID, methodID, args))
#define JRI_CallStaticMethodCharA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodCharA)(env, classID, methodID, args))

#define JRI_CallStaticMethodShort(env)	(((env)->interface)->CallStaticMethodShort)
#define JRI_CallStaticMethodShortV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodShortV)(env, classID, methodID, args))
#define JRI_CallStaticMethodShortA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodShortA)(env, classID, methodID, args))

#define JRI_CallStaticMethodInt(env)	(((env)->interface)->CallStaticMethodInt)
#define JRI_CallStaticMethodIntV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodIntV)(env, classID, methodID, args))
#define JRI_CallStaticMethodIntA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodIntA)(env, classID, methodID, args))

#define JRI_CallStaticMethodLong(env)	(((env)->interface)->CallStaticMethodLong)
#define JRI_CallStaticMethodLongV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodLongV)(env, classID, methodID, args))
#define JRI_CallStaticMethodLongA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodLongA)(env, classID, methodID, args))

#define JRI_CallStaticMethodFloat(env)	(((env)->interface)->CallStaticMethodFloat)
#define JRI_CallStaticMethodFloatV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodFloatV)(env, classID, methodID, args))
#define JRI_CallStaticMethodFloatA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodFloatA)(env, classID, methodID, args))

#define JRI_CallStaticMethodDouble(env)	(((env)->interface)->CallStaticMethodDouble)
#define JRI_CallStaticMethodDoubleV(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodDoubleV)(env, classID, methodID, args))
#define JRI_CallStaticMethodDoubleA(env, classID, methodID, args)	\
	((((env)->interface)->CallStaticMethodDoubleA)(env, classID, methodID, args))

#define JRI_GetStaticFieldID(env, classID, name, sig)	\
	((((env)->interface)->GetStaticFieldID)(env, classID, name, sig))

#define JRI_GetStaticField(env, classID, fieldID)	\
	((((env)->interface)->GetStaticField)(env, classID, fieldID))

#define JRI_GetStaticFieldBoolean(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldBoolean)(env, classID, fieldID))

#define JRI_GetStaticFieldByte(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldByte)(env, classID, fieldID))

#define JRI_GetStaticFieldChar(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldChar)(env, classID, fieldID))

#define JRI_GetStaticFieldShort(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldShort)(env, classID, fieldID))

#define JRI_GetStaticFieldInt(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldInt)(env, classID, fieldID))

#define JRI_GetStaticFieldLong(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldLong)(env, classID, fieldID))

#define JRI_GetStaticFieldFloat(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldFloat)(env, classID, fieldID))

#define JRI_GetStaticFieldDouble(env, classID, fieldID)	\
	((((env)->interface)->GetStaticFieldDouble)(env, classID, fieldID))

#define JRI_SetStaticField(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticField)(env, classID, fieldID, value))

#define JRI_SetStaticFieldBoolean(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldBoolean)(env, classID, fieldID, value))

#define JRI_SetStaticFieldByte(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldByte)(env, classID, fieldID, value))

#define JRI_SetStaticFieldChar(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldChar)(env, classID, fieldID, value))

#define JRI_SetStaticFieldShort(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldShort)(env, classID, fieldID, value))

#define JRI_SetStaticFieldInt(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldInt)(env, classID, fieldID, value))

#define JRI_SetStaticFieldLong(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldLong)(env, classID, fieldID, value))

#define JRI_SetStaticFieldFloat(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldFloat)(env, classID, fieldID, value))

#define JRI_SetStaticFieldDouble(env, classID, fieldID, value)	\
	((((env)->interface)->SetStaticFieldDouble)(env, classID, fieldID, value))

#define JRI_NewString(env, unicode, len)	\
	((((env)->interface)->NewString)(env, unicode, len))

#define JRI_GetStringLength(env, stringID)	\
	((((env)->interface)->GetStringLength)(env, stringID))

#define JRI_GetStringChars(env, stringID)	\
	((((env)->interface)->GetStringChars)(env, stringID))

#define JRI_NewStringUTF(env, utf, len)	\
	((((env)->interface)->NewStringUTF)(env, utf, len))

#define JRI_GetStringUTFLength(env, stringID)	\
	((((env)->interface)->GetStringUTFLength)(env, stringID))

#define JRI_GetStringUTFChars(env, stringID)	\
	((((env)->interface)->GetStringUTFChars)(env, stringID))

#define JRI_NewScalarArray(env, length, elementSig, initialElements)	\
	((((env)->interface)->NewScalarArray)(env, length, elementSig, initialElements))

#define JRI_GetScalarArrayLength(env, array)	\
	((((env)->interface)->GetScalarArrayLength)(env, array))

#define JRI_GetScalarArrayElements(env, array)	\
	((((env)->interface)->GetScalarArrayElements)(env, array))

#define JRI_NewObjectArray(env, length, elementClass, initialElement)	\
	((((env)->interface)->NewObjectArray)(env, length, elementClass, initialElement))

#define JRI_GetObjectArrayLength(env, array)	\
	((((env)->interface)->GetObjectArrayLength)(env, array))

#define JRI_GetObjectArrayElement(env, array, index)	\
	((((env)->interface)->GetObjectArrayElement)(env, array, index))

#define JRI_SetObjectArrayElement(env, array, index, value)	\
	((((env)->interface)->SetObjectArrayElement)(env, array, index, value))

#define JRI_RegisterNatives(env, classID, nameAndSigArray, nativeProcArray)	\
	((((env)->interface)->RegisterNatives)(env, classID, nameAndSigArray, nativeProcArray))

#define JRI_UnregisterNatives(env, classID)	\
	((((env)->interface)->UnregisterNatives)(env, classID))

/*******************************************************************************
 * JRIRuntimeInstance / JRIRuntimeInterface
 ******************************************************************************/

#define JRI_DisposeRuntime(runtime) \
	(((runtime)->interface)->DisposeRuntime(runtime))

#define JRI_SetIOMode(runtime, mode) \
	(((runtime)->interface)->SetIOMode(runtime, mode))

#define JRI_SetFSMode(runtime, mode) \
	(((runtime)->interface)->SetFSMode(runtime, mode))

#define JRI_SetRTMode(runtime, mode) \
	(((runtime)->interface)->SetRTMode(runtime, mode))

#define JRI_NewEnv(runtime, thread) \
	(((runtime)->interface)->NewEnv(runtime, thread))

#define JRI_DisposeEnv(runtime, env) \
	(((runtime)->interface)->DisposeEnv(env))

#define JRI_GetRuntime(runtime, env) \
	(((runtime)->interface)->GetRuntime(env))

#define JRI_GetThread(runtime, env) \
	(((runtime)->interface)->GetThread(env))

#define JRI_SetClassLoader(runtime, env, classLoader) \
	(((runtime)->interface)->SetClassLoader(env, classLoader))
