/*
 	File:		NativeLibSupport.h
 
 	Version:	Technology:	Mac OS Runtime For Java
 				Release:	MRJ SDK 1.0
 
 	Copyright:	 1984-1996 by Apple Computer, Inc.  All rights reserved.
 
 	Bugs?:		If you find a problem with this file, send the file and version
 				information (from above) and the problem description to:
 
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/
#ifndef __NATIVELIBSUPPORT__
#define __NATIVELIBSUPPORT__

#ifndef __CODEFRAGMENTS__
#include <CodeFragments.h>
#endif
#ifndef __JRI__
#include <JRI.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

/*
 * Forward JRI types
*/
/*
 * All JRI based native libraries must implement and export at least this one method:
 * 
 * runtimeInstance: interface to the JRI
 * javaLibFragment: the connection to the calling library
 *
*/
extern OSStatus InitJavaLibrary(JRIRuntimeInstance *runtimeInstance, CFragConnectionID javaLibFragment);

/*
 * If you're using the NativeLibSupport (static) library, you must initialize it from
 * your InitJavaLibrary method;.
*/
extern OSStatus InitNativeLibSupport(JRIRuntimeInstance *runtimeInstance, CFragConnectionID javaLibFragment);

/*
 * Once initialized, the following calls are availiable for native libraries:
*/
/*
 * M e m o r y   A l l o c a t i o n
*/
/*
 * Similar to malloc(); "clear" indicates that the memory is to be
 * initialized to zero.
*/
extern void *javaAllocMemory(unsigned long memorySize, Boolean clear);

/*
 * Similar to free()
*/
extern void javaFreeMemory(void *memory);

/*
 * Returns a (possibly disk resident) handle.  Note that
 * this is _not_ a mac handle and cannot be passed to the toolbox.
 * "clear" means the memory is to be initialized to zero.
 * When you access the handle, there must be enough memory for the
 * entire thing to exist in memory.
*/
typedef void *JavaMemoryHandle;
extern JavaMemoryHandle javaAllocHandle(unsigned long memorySize, Boolean clear);

/*
 * Disposes of a JavaMemoryHandle
*/
extern void javaDisposeHandle(JavaMemoryHandle hMemory);

/*
 * Accesses the underlying memory of a JavaMemoryHandle.
 * Note that this can fail and return nil - in which case you don't
 * need to release the memory again.  (Because these blocks
 * are disk resident, what might fail at one point may not fail another.)
*/
extern void *javaAccessHandleMemory(JavaMemoryHandle hMemory);

/*
 * Return the handle to the purgeable state
*/
extern void javaReleaseHandleMemory(JavaMemoryHandle hMemory);

/*
 * For native libraries that need C++ allocations,
 * these use the above allocators.
*/
/*
 extern void *operator new(size_t size);
 extern void operator delete(void *ptr);
*/
/*
 * A N S I - I s m s
*/
/*
 * Similar to sprintf, but takes a buffer size and terminates there.
*/
extern int javaSprintf(char *buffer, unsigned long bufferLength, const char *format, ...);

/*
 * Similar, but works with 255 buffer length Pascal style strings (returns length)
*/
extern int javaPSprintf(Str255 buffer, const char *format, ...);

/*
 * J R I - I s m s
*/
extern JRIEnv *jriGetCurrentEnv(void);


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#ifdef __cplusplus
}
#endif

#endif /* __NATIVELIBSUPPORT__ */

