//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		09/03/97	bpa		Fixed up non-precompiled includes.
		08/26/97	bpa		Created.
*/

#ifndef __CJavaAppletContext_AC__
#define __CJavaAppletContext_AC__

// Suite

#ifndef __Java_AC__
#include "Java_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CLongGeometry_AC__
#include "CLongGeometry_AC.h"
#endif

// Toolbox

#ifndef __JMANAGER__
#include <JManager.h>
#endif

//----------------------------------------------------------------------------------------

class CJavaAppletContainer_AC;
class MJavaView_AC;
class CList_AC;

//----------------------------------------------------------------------------------------

class EXPORT_AC CJavaAppletContext_AC 
{	
public:
	CJavaAppletContext_AC(CJavaAppletContainer_AC* appletContainer, UInt32 appletIndex);
	
	virtual ~CJavaAppletContext_AC ();

	virtual void AddView(MJavaView_AC* aJavaView);

	virtual void RemoveView(MJavaView_AC* aJavaView);

	CJavaAppletContainer_AC* GetAppletContainer();
	
	void SetAppletContainer(CJavaAppletContainer_AC* appletContainer);
	
	UInt32 GetAppletIndex();
	
	void SetAppletIndex(UInt32 appletIndex);
	
	virtual Handle GetAppletTag();

	JMAppletViewerRef GetAppletViewer();
	
	JMAWTContextRef GetAWTContext();

	void SetNextJavaView(MJavaView_AC* aJavaView);
	
	MJavaView_AC* GetNextJavaView();
		
	
	virtual void ResumeThreads();
	
	virtual void SuspendThreads();
	
	virtual void QueueExecution(const char* className,
										const char* methodName,
										const char* methodSig,
										JRIValue* args);
	
	virtual void QueueExecution(jref object,
										const char* methodName,
										const char* methodSig,
										JRIValue* args);
	
	virtual UInt32 CountContextFrames();
	
	virtual JMFrameRef GetIndFrame (UInt32 frameIndex);
	
	virtual void ReloadApplet();
	
	virtual void RestartApplet();
	
	virtual void SuspendApplet();
	
	virtual void ResumeApplet();

	virtual void RequestFrame(JMAWTContextRef context,
										JMFrameRef newFrame,
										JMFrameKind kind,
										const CLongPoint_AC& initialSize,
										bool resizeable,
										JMFrameCallbacks* callbacks);
		// Obtain a new applet frame.  Forwards the request to the session by default.
	
	static CJavaAppletContext_AC* GetContext(JMAWTContextRef context);

	//------------------------------------------------------------------------------------
	// CALLBACKS
	//------------------------------------------------------------------------------------

	static	short NewMenuIDProc( JMAWTContextRef ref, Boolean isHier);
	
	static	void ExceptionOccurredProc(JMAWTContextRef context,
										const char* exceptionName,
										const char* exceptionMsg,
										const char* stackTrace);

	static OSStatus RequestFrameProc(JMAWTContextRef context,
										JMFrameRef newFrame,
										JMFrameKind kind,
										UInt32 initialWidth,
										UInt32 initialHeight,
										Boolean resizeable,
										JMFrameCallbacks* callbacks);										
	
	static OSStatus ReleaseFrameProc(JMAWTContextRef context,
										JMFrameRef oldFrame);

	static void AVShowDocumentProc(JMAppletViewerRef viewer,
										const char* urlString,
										Boolean inNewWindow);
	
	static void AVSetStatusMsgProc(JMAppletViewerRef viewer,
										const char* statusMsg);
	
private:
	CJavaAppletContainer_AC* fAppletContainer;
	JMAppletViewerRef fAppletViewer;
	JMAWTContextRef fAWTContext;
	Handle fAppletTag;
	UInt32 fAppletIndex;
	MJavaView_AC* fNextJavaView;
	CAutoPtr_AC<CList_AC> fJavaViewList;
	
	static JMAWTContextCallbacks fgContextCallbacks;
	static JMAppletViewerCallbacks fgAppletViewerCallbacks;

};

//----------------------------------------------------------------------------------------

#endif
