//----------------------------------------------------------------------------------------
// CJavaSession_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		09/03/97	bpa		Fixed up non-precompiled includes.
		08/26/97	bpa		Created.
*/

#ifndef __CJavaSession_AC__
#define __CJavaSession_AC__

// Suite

#ifndef __Java_AC__
#include "Java_AC.h"
#endif

// ACS

#ifndef __CLongGeometry_AC__
#include "CLongGeometry_AC.h"
#endif

// Toolbox

#ifndef __JMANAGER__
#include <JManager.h>
#endif

//----------------------------------------------------------------------------------------
// Forward references
//----------------------------------------------------------------------------------------

class MJavaView_AC;
class CFSSpec_AC;
class CJavaAppletContext_AC;

//----------------------------------------------------------------------------------------
// CLASS CJavaSession
//----------------------------------------------------------------------------------------

class EXPORT_AC CJavaSession_AC
{
public:
	CJavaSession_AC();
		// Constructs this object.
	
	virtual ~CJavaSession_AC();
		// Closes the Java session and destructs.
		
	virtual JMSecurityOptions GetSecurityOptions();
		// Returns default security options.
		
	virtual void JavaOpen();
		// Opens the Java session.
		
	void JavaIdle();
		// Call from your application's Idle function so that Java applets get idle time.
	
	void JavaMenuSelect(short theMenuNumber, short theItemNumber);
		// Call from your application's MenuSelect function so that Java applets 
		// can handle menu commands.
		
	virtual short NewMenuIDRequest(JMAWTContextRef ref, bool isHier);
		// Called from CJavaAppletContainer_AC, to get a unique menu ID.
		// Return statically incremented default menu IDs.
		
	virtual void RequestFrame(JMAWTContextRef context,
										JMFrameRef newFrame,
										JMFrameKind kind,
										const CLongPoint_AC& initialSize,
										bool resizeable,
										JMFrameCallbacks* callbacks) = 0;
		// Called from CJavaAppletContext_AC, to obtain a new applet frame.
		// You must override this to assign a view, or open a window containing a view.

	//------------------------------------------------------------------------------------
	// Managing the active view and the view under the mouse.
	//------------------------------------------------------------------------------------

	static MJavaView_AC* GetActiveJavaView();

	static void SetActiveJavaView(MJavaView_AC* activeJavaView);

	static CJavaAppletContext_AC* GetActiveJavaContext();

	static MJavaView_AC* GetMouseOverJavaView();

	static void SetMouseOverJavaView(MJavaView_AC* mouseOverJavaView);

	//------------------------------------------------------------------------------------
	// Console standard output, error, and input
	//------------------------------------------------------------------------------------
		
	virtual void StdOut(const char* message, UInt32 messageLen);

	virtual void StdErr(const char* message, UInt32 messageLen);

	virtual SInt32 StdIn(char* buffer, SInt32 maxBufferLength);

	//------------------------------------------------------------------------------------
	// Convenience methods
	//------------------------------------------------------------------------------------

	static void AddToClassPath(const CFSSpec_AC& spec);

	static Handle FSSpecToURLHandle(const CFSSpec_AC& spec);

	static CStr255_AC FSSpecToURL(const CFSSpec_AC& spec);
	
	static void URLToFSSpec(const char* url, CFSSpec_AC& spec);

	static void FilterEndOfLineChars(char* text, UInt32 length);
	
	//------------------------------------------------------------------------------------
	// CALLBACKS
	//------------------------------------------------------------------------------------
	
	static void ConsoleWriteProc(JMSessionRef session, const char* message, UInt32 messageLen);

	static void ConsoleErrorProc(JMSessionRef session, const char* message, UInt32 messageLen);

	static SInt32 ConsoleReadProc(JMSessionRef session, char* buffer, SInt32 maxBufferLength);

	//------------------------------------------------------------------------------------
	// Useful references to the JM session and its global instance.
	//------------------------------------------------------------------------------------
	
	static JMSessionRef GetSessionRef() { return fgJavaSession; }
		
	static CJavaSession_AC* Instance() { return fgInstance; }
	
private:
	OSStatus fStatus;
	static CJavaSession_AC* fgInstance;	// the global instance; _not_ an auto pointer, to allow mixins.
	static JMSessionRef fgJavaSession;
	static MJavaView_AC* fgActiveJavaView;
	static MJavaView_AC* fgMouseOverJavaView;
	static JMSecurityOptions fgStandardSecurityOptions;
	static JMSessionCallbacks fgSessionCallbacks;
};

//----------------------------------------------------------------------------------------

#endif
