//----------------------------------------------------------------------------------------
// UJavaView.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef _H_LJavaView
#define _H_LJavaView

// MacApp

#ifndef _H_LView
#include "LView.h"
#endif

#ifndef __LSTATICTEXT__
#include "LStaticText.h"
#endif

// ACS

#ifndef __MJavaView_AC__
#include "MJavaView_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const IDType 	kMessageViewIdentifier		= 'MESG';
const IDType 	kJavaViewIdentifier			= 'JAVA';


//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CJavaAppletContainer_AC;
class CTempJavaAppletLocator_AC;

//----------------------------------------------------------------------------------------
// LJavaView
//----------------------------------------------------------------------------------------

class EXPORT_MA LJavaView : public TView, public MJavaView_AC
{
	MA_DECLARE_CLASS;
	
public:
	LView();
	LView(const LView& inOriginal);
	LView(const SPaneInfo& inPaneInfo,
		  const SViewInfo& inViewInfo);
	LView(LStream* inStream);

	virtual ~LJavaView();
	
	//virtual void Close();
		
	// PowerPlant overrides
	
	virtual void ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
	
	virtual void DrawSelf();
	
	virtual void ClickSelf(const SMouseDownEvent &inMouseDown);
	
	//virtual void DoKeyEvent(TToolboxEvent* event);

	//virtual bool WantsToBeTarget();
	
	//virtual void BecameTarget();
	
	//virtual void ResignedTarget();
		
	virtual void AdjustCursorSelf(Point inPortPt, const EventRecord &inMacEvent);
	
	// Java overrides
	
	virtual void JavaIdle();
	
	virtual void* SetupPortRequest();
	
	virtual void RestorePortRequest();
	
	virtual bool ResizeRequest(Rect* desired);
	
	virtual void ShowHideRequest(bool showIt);
	
	virtual void CheckUpdateRequest();
	
	virtual void InvalRectRequest(const CRect_AC& invalidRect);
	
};

//----------------------------------------------------------------------------------------
// LJavaURLView
//----------------------------------------------------------------------------------------

class EXPORT_MA LJavaURLView : public LJavaView
// This view is designed to stream in a URL from a view resource
{
	MA_DECLARE_CLASS;
	
public:
	LJavaURLView();
	
	virtual ~LJavaURLView();
	
	// Streaming
	
	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;
	
	virtual void DoPostCreate(TDocument* itsDocument);
	
	// New API
	
	virtual CJavaAppletContainer_AC* DoMakeJavaAppletContainer(const CTempJavaAppletLocator_AC& tempLocator);
		// create an applet container, given fURL and frame

protected:
	ResNumber	fURLStrListID;						// url:	STR# rsrc id
	short		fURLIndex;							// index into STR#
	ResNumber	fAppletCodeStrListID;				// applet code:	STR# rsrc id
	short		fAppletCodeIndex;					// index into STR#
};

//----------------------------------------------------------------------------------------
// LJavaRequestedView
//----------------------------------------------------------------------------------------

class EXPORT_MA LJavaRequestedView : public LJavaView
// This view is designed to correspond to a standalone window which is created via Java. 
{
	MA_DECLARE_CLASS;
	
public:
	LJavaRequestedView(const CViewRect& itsFrame = gZeroRect, JMAWTContextRef context = NULL,
															JMFrameRef frame = NULL);
	
	virtual ~LJavaRequestedView();

	virtual bool ResizeRequest(Rect* desired);

	virtual void StatusMessageRequest(const char* statusMsg);

	virtual void ShowHideRequest(bool showIt);

	virtual void SetTitleRequest(Str255 title);

	virtual void ReleaseFrameRequest();
};

//----------------------------------------------------------------------------------------
// LJavaMessageView
//----------------------------------------------------------------------------------------

class EXPORT_MA LJavaMessageView : public TStaticText
// This is a companion utility class used by LJavaRequestedView.
{
	MA_DECLARE_CLASS;
	
public:
	LJavaMessageView();

	virtual ~LJavaMessageView();
	
	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void SuperViewChangedFrame(const CViewRect& oldFrame, const CViewRect& newFrame,
															bool invalidate);

	void AlignToSuperView();

};

//----------------------------------------------------------------------------------------

EXPORT_MA void InitUJavaView();

//----------------------------------------------------------------------------------------

#endif
