//----------------------------------------------------------------------------------------
// MJavaView_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __MJavaView_AC__
#define __MJavaView_AC__

// Suite

#ifndef __Java_AC__
#include "Java_AC.h"
#endif

// ACS

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

// Toolbox

#ifndef __JMANAGER__
#include <JManager.h>
#endif

//----------------------------------------------------------------------------------------

class CRegion_AC;
class CJavaAppletContext_AC;

//----------------------------------------------------------------------------------------

class MJavaView_AC 
{	
public:
	MJavaView_AC(JMAWTContextRef context = NULL, JMFrameRef frame = NULL);
		// Constructor: call AdoptFrame.
		
	virtual ~MJavaView_AC();
		// Destructor: calls JavaClose
			
	virtual void JavaClose();
		// Delete our frame and context.
			  			
	virtual void AdoptFrame(JMAWTContextRef context, JMFrameRef frame);
		// Take ownership of this context and frame.
		
	JMAWTContextRef	 GetAWTContext();
		// returns my AWT context.
		
	CJavaAppletContext_AC* GetJavaAppletContext();
		// Returns my context.
		
	JMFrameRef JavaGetFrame();
		// Returns my frame.
		
	void JavaSetFrame(const CRect_AC& newFrame);
		// Sets my frame.

	//------------------------------------------------------------------------------------
	// Java handling of frame, drawing, mouse, key, target, and idling
	//------------------------------------------------------------------------------------

	void JavaDraw(CRegion_AC visibleRegion);
	
	void JavaMouseEvent(CPoint_AC& theMouse, short modifiers);
	
	void JavaKeyEvent(char asciiChar, char keyCode, short modifiers);
	
	void JavaBecameTarget();
	
	void JavaResignedTarget();
	
	void JavaIdle();

	//------------------------------------------------------------------------------------
	// Handling requests polymorphically from callbacks
	//------------------------------------------------------------------------------------
		
	virtual void* SetupPortRequest();
	
	virtual void RestorePortRequest(void* refCon);
	
	virtual bool ResizeRequest(CRect_AC& desiredRect);
	
	virtual void ShowHideRequest(bool showIt);
	
	virtual void SetTitleRequest(const CStr255_AC& title);
	
	virtual void CheckUpdateRequest();
	
	virtual void InvalRectRequest(const CRect_AC& invalidRect);
	
	virtual void StatusMessageRequest(const CStr255_AC& statusMsg);
	
	virtual void ReleaseFrameRequest();

	//------------------------------------------------------------------------------------
	// Getting a reference to Java view, given a frame ref or context ref.
	//------------------------------------------------------------------------------------

	static MJavaView_AC* JavaViewFromFrameRef(JMFrameRef ref);
	
	static MJavaView_AC* JavaViewFromContextRef(JMAWTContextRef ref);

	//------------------------------------------------------------------------------------
	// CALLBACKS
	//------------------------------------------------------------------------------------

	virtual void SetupCallbacks(JMFrameCallbacks* callbacks);

	static void* WinSetupPortRequestProc(JMFrameRef ref);
	
	static void WinRestorePortRequestProc(JMFrameRef ref, void* refcon);
	
	static Boolean WinResizeRequestProc(JMFrameRef ref, Rect* desired);
	
	static void WinInvalRectRequestProc(JMFrameRef ref, const Rect* r);
	
	static void WinShowHideRequestProc(JMFrameRef ref, Boolean showIt);
	
	static void WinSetTitleRequestProc(JMFrameRef ref, Str255 title);
	
	static void WinCheckUpdateRequestProc(JMFrameRef ref);
	
private:
	JMAWTContextRef fContext;
	JMFrameRef fFrame;
	bool fDeferredActivate;
};

//----------------------------------------------------------------------------------------

#endif
