//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __CJavaAppletContainer_AC__
#include "CJavaAppletContainer_AC.h"
#endif

// ACS

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

#ifndef __CJavaSession_AC__
#include "CJavaSession_AC.h"
#endif

#ifndef __CJavaAppletLocator_AC__
#include "CJavaAppletLocator_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

//========================================================================================
// CLASS CJavaAppletContainer_AC
//========================================================================================

JMAppletLocatorCallbacks	CJavaAppletContainer_AC::fgAppletCallbacks =	{	kJMVersion,
														CJavaAppletContainer_AC::LocatorCompletedProc };

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::CJavaAppletContainer_AC: 
//----------------------------------------------------------------------------------------

CJavaAppletContainer_AC::CJavaAppletContainer_AC(const CStr255_AC& url, const CStr255_AC& htmlText) :
	fURL(url),
	fAppletLocator(NULL),
	fContextList(::NewList_AC())
{
	// JM will test NULL-ness of the htmlText parameter, not emptiness.
	char* htmlStr = NULL;
	if (htmlText.Length() > 0)
		htmlStr = CChar255_AC(htmlText);
		
	ThrowIfOSErr_AC(::JMNewAppletLocator(&fAppletLocator, CJavaSession_AC::GetSessionRef(), &fgAppletCallbacks, 
										 CChar255_AC(fURL), htmlStr, (JMClientData) this));
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::CJavaAppletContainer_AC: 
//----------------------------------------------------------------------------------------

CJavaAppletContainer_AC::CJavaAppletContainer_AC(const CTempJavaAppletLocator_AC& locator) :
	fURL(locator.fBaseURL),
	fAppletLocator(NULL),
	fContextList(::NewList_AC())
{
	ThrowIfOSErr_AC(::JMNewAppletLocatorFromInfo(&fAppletLocator, CJavaSession_AC::GetSessionRef(), 
												 &locator, (JMClientData) this));
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC destructor
//----------------------------------------------------------------------------------------

CJavaAppletContainer_AC::~CJavaAppletContainer_AC()
{
	if (fAppletLocator)
	{
		::JMDisposeAppletLocator(fAppletLocator);
		fAppletLocator = NULL;
	}
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::GetURL: 
//----------------------------------------------------------------------------------------

CStr255_AC CJavaAppletContainer_AC::GetURL() const
{
	return fURL;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::GetAppletLocator: 
//----------------------------------------------------------------------------------------

JMAppletLocatorRef CJavaAppletContainer_AC::GetAppletLocator()
{
	return fAppletLocator;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::CountApplets: 
//----------------------------------------------------------------------------------------

UInt32 CJavaAppletContainer_AC::CountApplets()
{
	unsigned long actualCount;
	
	ThrowIfOSErr_AC(::JMCountApplets(fAppletLocator, &actualCount));
	
	return actualCount;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::GetAppletBounds: 
//----------------------------------------------------------------------------------------

CLongRect_AC CJavaAppletContainer_AC::GetAppletBounds(UInt32 appletIndex)
{
	UInt32 width;
	UInt32 height;
	
	ThrowIfOSErr_AC(::JMGetAppletDimensions(fAppletLocator, appletIndex, &width, &height));
	
	return CLongRect_AC(0, 0, width, height);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::GetAppletName: 
//----------------------------------------------------------------------------------------

CStr255_AC CJavaAppletContainer_AC::GetAppletName(UInt32 appletIndex)
{
	CStr255_AC appletName;
	ThrowIfOSErr_AC (::JMGetAppletName(fAppletLocator, appletIndex, (char*) &appletName[1], CStr255_AC::kMaxLength));
	
	appletName[0] = strlen((char*) &appletName[1]);
	return appletName;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::GetAppletTag: 
//----------------------------------------------------------------------------------------

Handle CJavaAppletContainer_AC::GetAppletTag(UInt32 appletIndex)
{
	Handle appletTag = NULL;
	
	ThrowIfOSErr_AC(::JMGetAppletTag(fAppletLocator, appletIndex, &appletTag));
	
	// Filter the tag so that newlines (\r) are converted to returns (\n)
	CJavaSession_AC::FilterEndOfLineChars(*appletTag, GetHandleSize(appletTag));
	return appletTag;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::LocatorCompleted: 
//----------------------------------------------------------------------------------------

void CJavaAppletContainer_AC::LocatorCompleted(JMLocatorErrors error)
{
#if qCoreDebug
	if (error != eLocatorNoErr)
	{
		switch (error)
		{
			case eHostNotFound:
				ReportError("Host not found: ", fURL);
				break;
				
			case eFileNotFound:
				ReportError("File not found: ", fURL);
				break;
				
			case eLocatorTimeout:
				ReportError("Timeout accessing: ", fURL);
				break;
			
			default:
				ReportError("Bad URL: ", fURL);
				break;
		}
	}
#endif
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::AddContext: 
//----------------------------------------------------------------------------------------

void CJavaAppletContainer_AC::AddContext(CJavaAppletContext_AC* aJavaAppletContext)
{
	fContextList->Insert(aJavaAppletContext);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::RemoveContext: 
//----------------------------------------------------------------------------------------

void CJavaAppletContainer_AC::RemoveContext(CJavaAppletContext_AC* aJavaAppletContext)
{
	fContextList->Delete(aJavaAppletContext);
	ArrayIndex_AC count = fContextList->GetSize();
	
	if (count == 0)
		delete this;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::GetLastContext: 
//----------------------------------------------------------------------------------------

CJavaAppletContext_AC* CJavaAppletContainer_AC::GetLastContext()
{
	return reinterpret_cast<CJavaAppletContext_AC*> (fContextList->Last());
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::StartAllApplets: 
//----------------------------------------------------------------------------------------

void CJavaAppletContainer_AC::StartAllApplets()
{
	UInt32 countApplets = CountApplets();
	
	if (countApplets > 0)
	{
		CJavaAppletContext_AC* aJavaAppletContext;
		for (UInt32 appletIndex = kEmptyIndex_AC;  appletIndex < countApplets;  ++appletIndex)
		{
			aJavaAppletContext = DoMakeContext(appletIndex);
		}
	}
#if qCoreDebug
	else
	{
		ReportError("No applets on page: ", fURL);
	}
#endif
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::DoMakeContext: 
//----------------------------------------------------------------------------------------

CJavaAppletContext_AC* CJavaAppletContainer_AC::DoMakeContext(UInt32 appletIndex)
{
	return TH_new CJavaAppletContext_AC(this, appletIndex);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::ReportError: 
//----------------------------------------------------------------------------------------

void CJavaAppletContainer_AC::ReportError(const CStr255_AC& errorMsg,
										const CStr255_AC& urlText)
{
	ProgramReport_AC(errorMsg + urlText);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContainer_AC::LocatorCompletedProc: 
//----------------------------------------------------------------------------------------

void CJavaAppletContainer_AC::LocatorCompletedProc(JMAppletLocatorRef ref, JMLocatorErrors status)
{
	if (ref)
	{
		CJavaAppletContainer_AC* appletContainer = NULL;
	
		::JMGetAppletLocatorData(ref, (JMClientData*) &appletContainer);
		
		if (appletContainer)
			appletContainer->LocatorCompleted(status);
	}
}

//========================================================================================
// CLASS CJavaAutoAppletContainer_AC
//========================================================================================

#undef Inherited
#define Inherited CJavaAppletContainer_AC

//----------------------------------------------------------------------------------------
// CJavaAutoAppletContainer_AC::CJavaAutoAppletContainer_AC: 
//----------------------------------------------------------------------------------------

CJavaAutoAppletContainer_AC::CJavaAutoAppletContainer_AC(const CStr255_AC& url, const CStr255_AC& htmlText) :
	CJavaAppletContainer_AC(url, htmlText)
{
}

//----------------------------------------------------------------------------------------
// CJavaAutoAppletContainer_AC::CJavaAutoAppletContainer_AC: 
//----------------------------------------------------------------------------------------

CJavaAutoAppletContainer_AC::CJavaAutoAppletContainer_AC(const CTempJavaAppletLocator_AC& locator) :
	CJavaAppletContainer_AC(locator)
{
}

//----------------------------------------------------------------------------------------
// CJavaAutoAppletContainer_AC destructor
//----------------------------------------------------------------------------------------

CJavaAutoAppletContainer_AC::~CJavaAutoAppletContainer_AC()
{
}

//----------------------------------------------------------------------------------------
// CJavaAutoAppletContainer_AC::LocatorCompleted: 
//----------------------------------------------------------------------------------------

void CJavaAutoAppletContainer_AC::LocatorCompleted(JMLocatorErrors error)
{
	if (error != eLocatorNoErr)
	{
		Inherited::LocatorCompleted(error);
	}
	else
	{
		StartAllApplets();
	}
}


//----------------------------------------------------------------------------------------
// end of file