//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		09/03/97	bpa		Fixed up non-precompiled includes.
		08/26/97	bpa		Created.
*/

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

// ACS
#ifndef __CJavaAppletContainer_AC__
#include "CJavaAppletContainer_AC.h"
#endif

#ifndef __CJavaSession_AC__
#include "CJavaSession_AC.h"
#endif

#ifndef __MJavaView_AC__
#include "MJavaView_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

//========================================================================================
// CLASS CJavaAppletContext_AC
//========================================================================================

JMAWTContextCallbacks CJavaAppletContext_AC::fgContextCallbacks = { kJMVersion,
													CJavaAppletContext_AC::RequestFrameProc,
													CJavaAppletContext_AC::ReleaseFrameProc,
													CJavaAppletContext_AC::NewMenuIDProc,
													CJavaAppletContext_AC::ExceptionOccurredProc };

JMAppletViewerCallbacks CJavaAppletContext_AC::fgAppletViewerCallbacks = { kJMVersion,
													CJavaAppletContext_AC::AVShowDocumentProc,
													CJavaAppletContext_AC::AVSetStatusMsgProc };

//----------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------

CJavaAppletContext_AC::CJavaAppletContext_AC(CJavaAppletContainer_AC* appletContainer, UInt32 appletIndex) :
	fAppletContainer(appletContainer),
	fAppletViewer(NULL),
	fAWTContext(NULL),
	fAppletIndex(appletIndex),
	fNextJavaView(NULL),
	fJavaViewList(::NewList_AC())
{
	SetAppletContainer(appletContainer);
	SetAppletIndex(appletIndex);
	appletContainer->AddContext(this);
	
	// Context
	
	ThrowIfOSErr_AC(::JMNewAWTContext(&fAWTContext, CJavaSession_AC::GetSessionRef(), &fgContextCallbacks, JMClientData(this)));

	ResumeThreads(); // non-virtual call

	// now create an applet viewer

	ThrowIfOSErr_AC(::JMNewAppletViewer(&fAppletViewer, fAWTContext, appletContainer->GetAppletLocator(), appletIndex, &fgAppletViewerCallbacks, (JMClientData) this));
	
	ReloadApplet(); // non-virtual call
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC destructor
//----------------------------------------------------------------------------------------

CJavaAppletContext_AC::~CJavaAppletContext_AC()
{
	if (fAppletViewer)
	{
		::JMDisposeAppletViewer(fAppletViewer);
		fAppletViewer = NULL;
	}

	if (fAWTContext)
	{
		::JMDisposeAWTContext(fAWTContext);
		fAWTContext = NULL;
	}

	if (fAppletTag)
	{
		DisposeHandle(fAppletTag);
		fAppletTag = NULL;
	}
	
	CJavaAppletContainer_AC* aJavaAppletContainer = GetAppletContainer();	
	if (aJavaAppletContainer)
		aJavaAppletContainer->RemoveContext(this);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::AddView: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::AddView(MJavaView_AC* aJavaView)
{
	fJavaViewList->Insert(aJavaView);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::RemoveView: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::RemoveView(MJavaView_AC* aJavaView)
{
	fJavaViewList->Delete(aJavaView);
	ArrayIndex_AC count = fJavaViewList->GetSize();
	
	if (count == 0)
		delete this;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::GetAppletContainer: 
//----------------------------------------------------------------------------------------

CJavaAppletContainer_AC* CJavaAppletContext_AC::GetAppletContainer()
{
	return fAppletContainer;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::SetAppletContainer: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::SetAppletContainer(CJavaAppletContainer_AC* appletContainer)
{
	fAppletContainer = appletContainer;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::GetAppletIndex: 
//----------------------------------------------------------------------------------------

UInt32 CJavaAppletContext_AC::GetAppletIndex()
{
	return fAppletIndex;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::SetAppletIndex: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::SetAppletIndex(UInt32 appletIndex)
{
	fAppletIndex = appletIndex;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::GetAppletTag: 
//----------------------------------------------------------------------------------------

Handle CJavaAppletContext_AC::GetAppletTag()
{
	Handle appletTag = NULL;
	
	UInt32 appletIndex	= GetAppletIndex();
	CJavaAppletContainer_AC* aJavaAppletContainer = GetAppletContainer();
	
	if (aJavaAppletContainer)
		appletTag = aJavaAppletContainer->GetAppletTag(appletIndex);
	
	return appletTag;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::GetAppletViewer: 
//----------------------------------------------------------------------------------------

JMAppletViewerRef CJavaAppletContext_AC::GetAppletViewer()
{
	return fAppletViewer;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::GetAWTContext: 
//----------------------------------------------------------------------------------------

JMAWTContextRef CJavaAppletContext_AC::GetAWTContext()
{
	return fAWTContext;
}

//----------------------------------------------------------------------------------------
// TJavaAppletContext::SetNextJavaView: 
//----------------------------------------------------------------------------------------
void CJavaAppletContext_AC::SetNextJavaView(MJavaView_AC* aJavaView)
{
	fNextJavaView = aJavaView;
}

//----------------------------------------------------------------------------------------
// TJavaAppletContext::GetNextJavaView: 
//----------------------------------------------------------------------------------------
MJavaView_AC* CJavaAppletContext_AC::GetNextJavaView()
{
	return fNextJavaView;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::ResumeThreads: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::ResumeThreads()
{
	ThrowIfOSErr_AC (::JMResumeAWTContext(fAWTContext));
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::SuspendThreads: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::SuspendThreads()
{
	ThrowIfOSErr_AC (::JMSuspendAWTContext(fAWTContext));
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::QueueExecution: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::QueueExecution(const char* className,
											const char* methodName,
											const char* methodSig,
											JRIValue* args)
{
	JRIEnv* env = ::JMGetCurrentEnv(CJavaSession_AC::GetSessionRef());
	
	if (env)
	{
		// find the class
		
		JRIClassID classID = JRI_FindClass(env, className);
		
		if (classID)
		{
			JRIMethodID methodID = JRI_GetStaticMethodID(env, classID, methodName, methodSig);
			
			if (methodID)
				::JMExecStaticMethodInContext(fAWTContext, classID, methodID, 1, args);
		}
	}
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::QueueExecution: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::QueueExecution(jref object,
											const char* methodName,
											const char* methodSig,
											JRIValue* args)
{
	JRIEnv* env = ::JMGetCurrentEnv(CJavaSession_AC::GetSessionRef());
	
	if (env)
	{
		// find the class
		
		JRIClassID classID = JRI_GetObjectClass(env, object);
		
		if (classID)
		{
			JRIMethodID methodID = JRI_GetMethodID(env, classID, methodName, methodSig);
			
			if (methodID)
				::JMExecMethodInContext(fAWTContext, object, methodID, 1, args);
		}
	}
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::CountContextFrames: 
//----------------------------------------------------------------------------------------

UInt32 CJavaAppletContext_AC::CountContextFrames()
{
	UInt32 countFrames = kEmptyIndex_AC;
	
	ThrowIfOSErr_AC(::JMCountAWTContextFrames(fAWTContext, &countFrames));
	
	return countFrames;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::GetIndFrame: 
//----------------------------------------------------------------------------------------

JMFrameRef CJavaAppletContext_AC::GetIndFrame(UInt32 frameIndex)
{
	JMFrameRef frame = NULL;
	
	ThrowIfOSErr_AC(::JMGetAWTContextFrame(fAWTContext, frameIndex, &frame));
	
	return frame;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::ReloadApplet: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::ReloadApplet()
{
	if (fAppletViewer)
		::JMReloadApplet(fAppletViewer);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::RestartApplet: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::RestartApplet()
{
	JMFrameRef frame = NULL;
	MJavaView_AC* aJavaView = NULL;
	OSStatus err;
	CRect_AC frameRect;
	bool resizeOK;
	
	if (fAppletViewer)
	{
		::JMRestartApplet(fAppletViewer);	
		UInt32 countFrames = CountContextFrames();
		
		for (UInt32 frameIndex = 0; frameIndex < countFrames; frameIndex++)
		{
			frame = GetIndFrame(frameIndex);
			
			if (frame)
			{
				err = ::JMGetFrameSize(frame, &frameRect);		
				if (err == noErr)
				{
					aJavaView = MJavaView_AC::JavaViewFromFrameRef(frame);
					resizeOK = aJavaView->ResizeRequest(frameRect);
					
					if (!resizeOK)
						err = ::JMSetFrameSize(frame, &frameRect);
				}
			}
		}
	}
}
	
//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::SuspendApplet: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::SuspendApplet()
{
	if (fAppletViewer)
		::JMSuspendApplet(fAppletViewer);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::ReloadApplet: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::ResumeApplet()
{
	if (fAppletViewer)
		::JMResumeApplet(fAppletViewer);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::GetContext: 
//----------------------------------------------------------------------------------------

CJavaAppletContext_AC* CJavaAppletContext_AC::GetContext(JMAWTContextRef context) // static
{
	CJavaAppletContext_AC* aJavaAppletContext;
	::JMGetAWTContextData(context, &aJavaAppletContext);
	return aJavaAppletContext;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::NewMenuIDProc: 
//----------------------------------------------------------------------------------------

short CJavaAppletContext_AC::NewMenuIDProc(JMAWTContextRef ref, Boolean isHier)
{
	return CJavaSession_AC::Instance()->NewMenuIDRequest(ref, isHier);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::ExceptionOccurredProc:
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::ExceptionOccurredProc(JMAWTContextRef /* context */,
												const char* exceptionName,
												const char* exceptionMsg,
												const char* stackTrace)
{
	fprintf(stderr, "_exceptionOccurred: %s (%s)\r", exceptionName, exceptionMsg == nil? "" : exceptionMsg);
	if (stackTrace) {
		int messageLen = strlen(stackTrace);
		Handle hText = NewHandle(messageLen);
		if (hText == nil)
			fwrite(stackTrace, messageLen, 1, stderr);
		else {
			BlockMoveData(stackTrace, *hText, messageLen);
			HLock(hText);
			CJavaSession_AC::FilterEndOfLineChars(*hText, messageLen);
			fwrite(*hText, messageLen, 1, stderr);
			DisposeHandle(hText);
		}
	}
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::RequestFrame: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::RequestFrame(JMAWTContextRef context,
										JMFrameRef newFrame,
										JMFrameKind kind,
										const CLongPoint_AC& initialSize,
										bool resizeable,
										JMFrameCallbacks* callbacks)
{
	CJavaSession_AC::Instance()->RequestFrame(context, newFrame, kind, initialSize, resizeable, callbacks);
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::RequestFrameProc: 
//----------------------------------------------------------------------------------------

OSStatus CJavaAppletContext_AC::RequestFrameProc(
											JMAWTContextRef		context,
											JMFrameRef			newFrame,
											JMFrameKind			kind,
											UInt32				initialWidth,
											UInt32				initialHeight,
											Boolean				resizeable,
											JMFrameCallbacks*	callbacks)
{
	OSStatus err = noErr;
	try
	{
		CJavaAppletContext_AC* appletContext = GetContext(context);
		ThrowIfNULL_AC(appletContext);// ??? Is this appropriate under all circumstances?
		appletContext->RequestFrame(context, newFrame, kind, CLongPoint_AC(initialWidth, initialHeight), resizeable, callbacks);
	}
	catch (CException_AC& theException)
	{
		err = theException.GetError();
	}	
	return err;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::ReleaseFrameProc: 
//----------------------------------------------------------------------------------------

OSStatus CJavaAppletContext_AC::ReleaseFrameProc(JMAWTContextRef /* context */, JMFrameRef oldFrame)
{
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(oldFrame);
	
	if (aJavaView)
		aJavaView->ReleaseFrameRequest();
	
	return noErr;
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::AVShowDocumentProc: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::AVShowDocumentProc(JMAppletViewerRef		viewer,
											const char*				/* urlString */,
											Boolean					/* inNewWindow */)
{
	CJavaAppletContext_AC* appletContext = NULL;
	
	ThrowIfOSErr_AC (::JMGetAppletViewerData(viewer, &appletContext));
}

//----------------------------------------------------------------------------------------
// CJavaAppletContext_AC::AVSetStatusMsgProc: 
//----------------------------------------------------------------------------------------

void CJavaAppletContext_AC::AVSetStatusMsgProc(JMAppletViewerRef viewer, const char* statusMsg)
{
	CJavaAppletContext_AC* appletContext = NULL;
	
	ThrowIfOSErr_AC (::JMGetAppletViewerData(viewer, &appletContext));
	
	if (appletContext)
	{
		JMAWTContextRef context = appletContext->GetAWTContext();
		MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromContextRef(context);
		
		if (aJavaView)
			aJavaView->StatusMessageRequest(statusMsg);
	}
}

//----------------------------------------------------------------------------------------
// end of file