//----------------------------------------------------------------------------------------
// CJavaSession_AC.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __CJavaSession_AC__
#include "CJavaSession_AC.h"
#endif

#ifndef __CTempGrafPort_AC__
#include "CTempGrafPort_AC.h"
#endif

#ifndef __MJavaView_AC__
#include "MJavaView_AC.h"
#endif

#ifndef __CJavaAppletContainer_AC__
#include "CJavaAppletContainer_AC.h"
#endif

#ifndef __CFSSpec_AC__
#include "CFSSpec_AC.h"
#endif

//========================================================================================
// CLASS CJavaSession_AC
//========================================================================================

CJavaSession_AC* CJavaSession_AC::fgInstance = NULL;
JMSessionRef CJavaSession_AC::fgJavaSession = NULL;
MJavaView_AC* CJavaSession_AC::fgActiveJavaView = NULL;
MJavaView_AC* CJavaSession_AC::fgMouseOverJavaView = NULL;

JMSecurityOptions CJavaSession_AC::fgStandardSecurityOptions = { kJMVersion,
																eDontCheckCode,
																false, { 0 }, 0,
																false, { 0 }, 0,
																false, { 0 }, 0,
																eUnrestrictedAccess,
																eAllFSAccess,
																false,
																false };
JMSessionCallbacks CJavaSession_AC::fgSessionCallbacks = { kJMVersion,
																CJavaSession_AC::ConsoleWriteProc,
																CJavaSession_AC::ConsoleErrorProc,
																CJavaSession_AC::ConsoleReadProc };


//----------------------------------------------------------------------------------------
// CJavaSession_AC constructor
//----------------------------------------------------------------------------------------

CJavaSession_AC::CJavaSession_AC() :
	fStatus(noErr)
{
	fgInstance = this;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC destructor
//----------------------------------------------------------------------------------------

CJavaSession_AC::~CJavaSession_AC()
{
	if (fgJavaSession)
	{
		::JMCloseSession(fgJavaSession);
		fgJavaSession = NULL;
	}
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::GetSecurityOptions: 
//----------------------------------------------------------------------------------------

JMSecurityOptions CJavaSession_AC::GetSecurityOptions()
{
	return fgStandardSecurityOptions;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::Open:
//----------------------------------------------------------------------------------------

void CJavaSession_AC::JavaOpen()
{
	JMSecurityOptions securityOptions = GetSecurityOptions();
	fStatus = ::JMOpenSession(&fgJavaSession, &securityOptions, &fgSessionCallbacks, this);
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::JavaIdle:
//----------------------------------------------------------------------------------------

void CJavaSession_AC::JavaIdle()
{
	{
		CTempGrafPort_AC	saveThePort();	
	
		::JMIdle(fgJavaSession, kDefaultJMTime);	// This sometimes changes the GrafPort
	}
	
	if (fgMouseOverJavaView)
		fgMouseOverJavaView->JavaIdle();
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::JavaMenuSelect: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::JavaMenuSelect(short theMenuNumber, short theItemNumber)
{
	if (fgActiveJavaView)
	{
		JMAWTContextRef context = fgActiveJavaView->GetAWTContext();
		
		::JMMenuSelected(context, GetMenuHandle(theMenuNumber), theItemNumber);
	}
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::NewMenuIDRequest: 
//----------------------------------------------------------------------------------------

short CJavaSession_AC::NewMenuIDRequest(JMAWTContextRef /* ref */, bool isHier)
{
	static short theNormalMenus	= 20000;
	static short theHierMenus	= 128;
	
	return isHier? theHierMenus++ : theNormalMenus++;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::GetActiveJavaView: 
//----------------------------------------------------------------------------------------

MJavaView_AC* CJavaSession_AC::GetActiveJavaView()
{
	return fgActiveJavaView;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::SetActiveJavaView: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::SetActiveJavaView(MJavaView_AC* activeJavaView)
{
	fgActiveJavaView = activeJavaView;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::GetActiveJavaContext: 
//----------------------------------------------------------------------------------------

CJavaAppletContext_AC* CJavaSession_AC::GetActiveJavaContext()
{
	CJavaAppletContext_AC* aJavaAppletContext = NULL;
	
	MJavaView_AC* aJavaView = GetActiveJavaView();
	
	if (aJavaView)
		aJavaAppletContext = aJavaView->GetJavaAppletContext();
	
	return aJavaAppletContext;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::GetMouseOverJavaView: 
//----------------------------------------------------------------------------------------

MJavaView_AC* CJavaSession_AC::GetMouseOverJavaView()
{
	return fgMouseOverJavaView;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::SetMouseOverJavaView: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::SetMouseOverJavaView(MJavaView_AC* mouseOverJavaView)
{
	fgMouseOverJavaView = mouseOverJavaView;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::AddToClassPath: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::AddToClassPath(const CFSSpec_AC& spec)
{
	// JMAddToClassPath requires a fully qualified FSSpec - no partial paths (this is a known bug)
	
	CFSSpec_AC fullSpec;
	
	ThrowIfOSErr_AC(::FSMakeFSSpec(spec.vRefNum, spec.parID, spec.name, &fullSpec));	
	ThrowIfOSErr_AC(::JMAddToClassPath(fgJavaSession, &fullSpec));
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::FSSpecToURL: 
//----------------------------------------------------------------------------------------

Handle CJavaSession_AC::FSSpecToURLHandle(const CFSSpec_AC& spec)
{
	return ::JMFSSToURL(fgJavaSession, &spec);
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::FSSpecToURL: 
//----------------------------------------------------------------------------------------

CStr255_AC CJavaSession_AC::FSSpecToURL(const CFSSpec_AC& spec)
{
	CStr255_AC url;
	Handle handle = FSSpecToURLHandle(spec);
	if (handle)
	{
		url = *handle;
		DisposeHandle(handle);
	}
	return url;
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::URLToFSSpec: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::URLToFSSpec(const char* url, CFSSpec_AC& spec)
{
	ThrowIfOSErr_AC(::JMURLToFSS(fgJavaSession, url, &spec));
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::FilterEndOfLineChars: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::FilterEndOfLineChars(char* text, UInt32 length)
{
	if (text)
	{
		char* p = text;
		char* pEnd = p + length;
		while (p < pEnd) {
			if (*p++ == '\r')
				p[-1] = '\n';
		}
	}
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::StdOut: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::StdOut(const char* message, UInt32 messageLen)
{
	fwrite(message, messageLen, 1, stdout);
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::StdErr: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::StdErr(const char* message, UInt32 messageLen)
{
	fwrite(message, messageLen, 1, stderr);
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::StdIn: 
//----------------------------------------------------------------------------------------

SInt32 CJavaSession_AC::StdIn(char* buffer, SInt32 maxBufferLength)
{
	return fread(buffer, maxBufferLength, 1, stdin);
}


//----------------------------------------------------------------------------------------
// CJavaSession_AC::ConsoleWriteProc: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::ConsoleWriteProc(JMSessionRef /*session*/, const char* message, UInt32 messageLen)
{
	Instance()->StdOut(message, messageLen);
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::ConsoleErrorProc: 
//----------------------------------------------------------------------------------------

void CJavaSession_AC::ConsoleErrorProc(JMSessionRef /*session*/, const char* message, UInt32 messageLen)
{
	Instance()->StdErr(message, messageLen);
}

//----------------------------------------------------------------------------------------
// CJavaSession_AC::ConsoleReadProc: 
//----------------------------------------------------------------------------------------

SInt32 CJavaSession_AC::ConsoleReadProc(JMSessionRef /*session*/, char* buffer, SInt32 maxBufferLength)
{
	return Instance()->StdIn(buffer, maxBufferLength);
}

//----------------------------------------------------------------------------------------
// end of file