//----------------------------------------------------------------------------------------
// LJavaApplication.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef _H_LJavaApplication
#include "LJavaApplication.h"
#endif

#ifndef _H_LJavaView
#include "LJavaView.h"
#endif

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

//========================================================================================
// LJavaApplication
//========================================================================================

#undef Inherited
#define Inherited LDocApplication

//----------------------------------------------------------------------------------------
// LJavaApplication constructor
//----------------------------------------------------------------------------------------

LJavaApplication::LJavaApplication() :
	LDocApplication()
{
	InitLJavaView();
}

//----------------------------------------------------------------------------------------
// LJavaApplication destructor
//----------------------------------------------------------------------------------------

LJavaApplication::~LJavaApplication()
{
}

//----------------------------------------------------------------------------------------
// TJavaApplication::DoInitialState: 
//----------------------------------------------------------------------------------------

void LJavaApplication::StartUp()
{
	Inherited::StartUp();

	OpenSession();
}

//----------------------------------------------------------------------------------------
// LJavaApplication::GetEvent: 
//----------------------------------------------------------------------------------------

void LJavaApplication::ProcessNextEvent()
{
	JavaIdle();

	Inherited::ProcessNextEvent();
}

//----------------------------------------------------------------------------------------
// LJavaApplication::RequestFrame: 
//----------------------------------------------------------------------------------------

void LJavaApplication:: RequestFrame(JMAWTContextRef context,
										JMFrameRef newFrame,
										JMFrameKind kind,
										UInt32 initialWidth,
										UInt32 initialHeight,
										Boolean resizeable,
										JMFrameCallbacks* callbacks)
{
	CJavaAppletContext_AC* aJavaAppletContext = CJavaAppletContext_AC::GetContext(context);
	CViewPoint viewSize(initialWidth, initialHeight);

	TJavaView* aJavaView = NULL;
	if (aJavaAppletContext)
	 	aJavaView = dynamic_cast<TJavaView*> (aJavaAppletContext->GetNextJavaView());
	
	if (aJavaView)
	{
		// Embedded frame
		
		aJavaView->AdoptFrame(context, newFrame);
		if (aJavaAppletContext)
			aJavaAppletContext->SetNextJavaView(NULL);
		aJavaView->SetupCallbacks(callbacks);
		aJavaView->Resize(viewSize, kRedraw);
	}
	else
	{
		LDocument* aDocument = NULL;
		//  bpa - need to resolve this GetDocument issue here
		//if (aJavaAppletContext)
		//	aDocument = aJavaAppletContext->GetDocument();
	
		// Create new window for frame
			
		LWindow* aJavaWindow = DoMakeJavaWindow(context, newFrame, initialWidth, initialHeight, resizeable, kind, callbacks);
		
		// Attach a printhandler
		
		aJavaView = dynamic_cast_AC(LJavaView*, aJavaWindow->FindSubView(kJavaViewIdentifier));
		TStdPrintHandler* aStdPrintHandler = TH_new TStdPrintHandler(aDocument, aJavaView, kFixedSize, kFixedSize);
		aJavaWindow->HandlePostCreate(aDocument);
		aJavaWindow->Open();
	}
}

//----------------------------------------------------------------------------------------
// DoMakeJavaWindow
//----------------------------------------------------------------------------------------

LWindow* LJavaApplication::DoMakeJavaWindow(JMAWTContextRef		context,
								JMFrameRef			newFrame,
								ViewCoordinate		width,
								ViewCoordinate		height,
								bool				resizeable,
								JMFrameKind			kind,
								JMFrameCallbacks*	callbacks)
{
	CViewRect itsViewFrame(0, 0, width, height);
	
	bool theGrowboxIntrudes	= false; // ??? Where do I get this from?

	// offset windows

	static int rows = 1;
	static int xPos = 10, yPos = 44;
	
	Rect* rBig = &(**GetMainDevice()).gdRect;
	if (xPos + width > rBig->right)
		xPos = 10 * ++rows;
	if (yPos + height > rBig->bottom)
		yPos = 44;
	
	// make a place

	Rect rBounds;
	short windowProc;
	bool goAway;
	int resizeHeight = (resizeable && ! theGrowboxIntrudes)? 15 : 0;
	
	switch (kind)
	{
		case eBorderlessModelessWindowFrame:
			goAway = false;
			windowProc = plainDBox;
			SetRect(&rBounds, xPos, yPos, xPos + width, yPos + height + resizeHeight);
			xPos += 10, yPos += 10;
			break;
		
		case eModelessWindowFrame:
			goAway = true;
			windowProc = resizeable? zoomDocProc : documentProc;
			SetRect(&rBounds, xPos, yPos, xPos + width, yPos + height + resizeHeight);
			xPos += 10, yPos += 10;
			break;
		
		case eModalWindowFrame:
			goAway = true;
			windowProc = movableDBoxProc;
			resizeable = false;
			Rect* mainDevice = &(**GetMainDevice()).gdRect;
			rBounds.left = mainDevice->left + ((mainDevice->right - mainDevice->left) / 2) - (width / 2);
			rBounds.top = mainDevice->top + ((mainDevice->bottom - mainDevice->top) / 2) - (height / 2);
			rBounds.right = rBounds.left + width;
			rBounds.bottom = rBounds.top + height;
			break;
	}
	
	if (rBounds.right < rBounds.left + 30)
		rBounds.right = rBounds.left + 30;
	
	if (rBounds.bottom < rBounds.top + 30)
		rBounds.bottom = rBounds.top + 30;
	
	WindowPtr theWMgrWindow = NewCWindow(nil, &rBounds, gEmptyString, false, windowProc, WindowPtr(-1), goAway, NULL /* nb theWindow */);
	
	/*
	if (theWMgrWindow)
	{
		WCTabHandle hWCTB = theWindowColors;
		
		if (hWCTB && HandToHand(&Handle(hWCTB)) == noErr && hWCTB)
			SetWinColor(theWMgrWindow, hWCTB);
	}
	*/
		
	LWindow* theWindow = TH_new LWindow(theWMgrWindow, resizeable, goAway, true);
	// NB, set the refcon to be this window, and set free on close flag.
	SetWRefCon(theWindow->GetWindowRef(), (long) theWindow);
	theWindow->SetFreeOnClosing(true);	

	// Create Message view
	
	LJavaMessageView* aJavaMessageView = TH_new LJavaMessageView;
	theWindow->AddSubView(aJavaMessageView);
	
	// Create default JavaView embedded into window
	
	LJavaRequestedView* aJavaRequestedView = TH_new LJavaRequestedView(itsViewFrame, context, newFrame);
	theWindow->AddSubView(aJavaRequestedView);	
	
	aJavaRequestedView->SetupCallbacks(callbacks);
	
	theWindow->SetWindowTarget(aJavaRequestedView);
	
	return theWindow;
}

//========================================================================================
// LJavaMenuBar
//========================================================================================

#undef Inherited
#define Inherited LMenuBar

CommandT LJavaMenuBar::MenuCommandSelection(const EventRecord &inMouseEvent, Int32 &outMenuChoice) const
{
	CommandT menuCmd = Inherited::MenuCommandSelection(inMouseEvent, outMenuChoice);
	
	CJavaSession::Instance()->JavaMenuSelect(HiWord(outMenuChoice), LoWord(outMenuChoice));

	return menuCmd;
}

//----------------------------------------------------------------------------------------
// end of file