//----------------------------------------------------------------------------------------
// LJavaView.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef _H_LJavaView
#include "LJavaView.h"
#endif

#ifndef __CJavaSession_AC__
#include "CJavaSession_AC.h"
#endif

#ifndef __CJavaAppletLocator_AC__
#include "CJavaAppletLocator_AC.h"
#endif

#ifndef __CJavaAppletContainer_AC__
#include "CJavaAppletContainer_AC.h"
#endif

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

//========================================================================================
// CLASS LJavaView
//========================================================================================

#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS(LJavaView);

//----------------------------------------------------------------------------------------
// LJavaView::LJavaView: 
//----------------------------------------------------------------------------------------

LJavaView::LJavaView(const CViewRect&	itsFrame,
					JMAWTContextRef		context,
					JMFrameRef			frame,
					SizeDeterminer		itsHSizeDet,
		  			SizeDeterminer		itsVSizeDet) :
	LView(itsFrame[topLeft], itsFrame[botRight], itsHSizeDet, itsVSizeDet),
	MJavaView_AC(context, frame)
{
}

//----------------------------------------------------------------------------------------
// LJavaView::~LJavaView: 
//----------------------------------------------------------------------------------------

LJavaView::~LJavaView()
{
}

//----------------------------------------------------------------------------------------
// LJavaView::Close
//----------------------------------------------------------------------------------------

void LJavaView::Close()
{
	Inherited::Close();
	
	MJavaView_AC* aJavaView = CJavaSession_AC::GetActiveJavaView();	
	if (aJavaView == this)
		CJavaSession_AC::SetActiveJavaView(NULL);

	aJavaView = CJavaSession_AC::GetMouseOverJavaView();	
	if (aJavaView == this)
		CJavaSession_AC::SetMouseOverJavaView(NULL);
}

//----------------------------------------------------------------------------------------
// LJavaView::SetFrame: 
//----------------------------------------------------------------------------------------

void LJavaView::SetFrame(const CViewRect& newFrame, bool invalidate)
{	
	Inherited::SetFrame(newFrame, invalidate);
	
	JavaSetFrame(ViewToQD(newFrame));
}

//----------------------------------------------------------------------------------------
// LJavaView::Draw: 
//----------------------------------------------------------------------------------------

void LJavaView::Draw(const CViewRect& area)
{	
	Inherited::Draw(area);
	
	CTempRegion_AC	visibleRegion;
	GetVisibleRegion(visibleRegion);	
	JavaDraw(visibleRegion);

}

//----------------------------------------------------------------------------------------
// LJavaView::DoMouseCommand: 
//----------------------------------------------------------------------------------------

void LJavaView::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC /* hysteresis */)
{
	JavaMouseEvent(ViewToQD(theMouse), event->GetEventRecord().modifiers);
}

//----------------------------------------------------------------------------------------
// LJavaView::DoKeyEvent: 
//----------------------------------------------------------------------------------------

void LJavaView::DoKeyEvent(TToolboxEvent* event)
{
	JavaKeyEvent(event->GetText()[1], event->GetKeyCode(), event->GetEventRecord().modifiers);
}

//----------------------------------------------------------------------------------------
// LJavaView::WantsToBeTarget: 
//----------------------------------------------------------------------------------------

bool LJavaView::WantsToBeTarget()
{
	return true;
}

//----------------------------------------------------------------------------------------
// LJavaView::BecameTarget: 
//----------------------------------------------------------------------------------------

void LJavaView::BecameTarget()
{
	Inherited::BecameTarget();

	JavaBecameTarget();
}

//----------------------------------------------------------------------------------------
// LJavaView::ResignedTarget: 
//----------------------------------------------------------------------------------------

void LJavaView::ResignedTarget()
{
	Inherited::ResignedTarget();

	JavaResignedTarget();
}

//----------------------------------------------------------------------------------------
// LJavaView::DoSetCursor: 
//----------------------------------------------------------------------------------------

void LJavaView::DoSetCursor(const CViewPoint& localPoint, RgnHandle cursorRegion)
{
	Inherited::DoSetCursor(localPoint, cursorRegion);

	CJavaSession_AC::SetMouseOverJavaView(this);
}

//----------------------------------------------------------------------------------------
// LJavaView::JavaIdle: 
//----------------------------------------------------------------------------------------

void LJavaView::JavaIdle()
{
	if (Focus())
	{
		MJavaView_AC::JavaIdle();
	}
}

//----------------------------------------------------------------------------------------
// LJavaView::SetupPortRequest: 
//----------------------------------------------------------------------------------------

void* LJavaView::SetupPortRequest()
{
	// Funny, but MRJ odes not work as advertised, when it comes to setportrequest and restoreportrequest
	// We have to invalidate the focus instead of saving and restoring the focus, because somehow, somewhere
	
	InvalidateFocus();
	
	Focus();
	
	return NULL;
}

//----------------------------------------------------------------------------------------
// LJavaView::RestorePortRequest: 
//----------------------------------------------------------------------------------------

void LJavaView::RestorePortRequest()
{
	// See LJavaView::SetupPortRequest for comments

	InvalidateFocus();
}

//----------------------------------------------------------------------------------------
// LJavaView::ResizeRequest: 
//----------------------------------------------------------------------------------------

bool LJavaView::ResizeRequest(Rect* desired)
{
	CRect_AC desiredRect(*desired);
	bool requestOK = true;
	
	CViewPoint newSize(desiredRect.GetSize());
	
	if (newSize != fSize)
	{
		Resize(newSize, kInvalidate);
		
		if (newSize != fSize)
		{
			// Tell the applet that we refused to change the size
			
			desired->right	= fSize.h;
			desired->bottom	= fSize.v;
			requestOK		= false;
		}
	}
	
	return requestOK;
}

//----------------------------------------------------------------------------------------
// LJavaView::ShowHideRequest: 
//----------------------------------------------------------------------------------------

void LJavaView::ShowHideRequest(bool showIt)
{
	Show(showIt, kRedraw);
	
	MJavaView_AC::ShowHideRequest(showIt);	
}

//----------------------------------------------------------------------------------------
// LJavaView::CheckUpdateRequest: 
//----------------------------------------------------------------------------------------

void LJavaView::CheckUpdateRequest()
{
	Update();
}

//----------------------------------------------------------------------------------------
// LJavaView::InvalRectRequest: 
//----------------------------------------------------------------------------------------

void LJavaView::InvalRectRequest(const CRect_AC& invalidRect)
{
	Invalidate(invalidRect);
}

//========================================================================================
// LJavaURLView
//========================================================================================

#undef Inherited
#define Inherited LJavaView

MA_DEFINE_CLASS(LJavaURLView);

//----------------------------------------------------------------------------------------
// LJavaURLView constructor:
//----------------------------------------------------------------------------------------

LJavaURLView::LJavaURLView(ResNumber urlStrListID, short urlIndex, 
				 ResNumber appletCodeStrListID, short appletCodeIndex,
				 const CViewRect& itsFrame,
				 SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet) :
	LJavaView(itsFrame, NULL, NULL, itsHSizeDet, itsVSizeDet),
	fURLStrListID(urlStrListID),
	fURLIndex(urlIndex),
	fAppletCodeStrListID(appletCodeStrListID),
	fAppletCodeIndex(appletCodeIndex)
{
}

//----------------------------------------------------------------------------------------
// LJavaURLView destructor:
//----------------------------------------------------------------------------------------

LJavaURLView::~LJavaURLView()
{
}

//----------------------------------------------------------------------------------------
// LJavaURLView::ReadFields:
//----------------------------------------------------------------------------------------

void LJavaURLView::ReadFields(CStream_AC* aStream)		// override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fURLStrListID;
	*aStream >> fURLIndex;
	*aStream >> fAppletCodeStrListID;
	*aStream >> fAppletCodeIndex;
}

//----------------------------------------------------------------------------------------
// LJavaURLView::WriteFields:
//----------------------------------------------------------------------------------------

void LJavaURLView::WriteFields(CStream_AC* aStream) const		// override 
{
	Inherited::WriteFields(aStream);
	
	*aStream << fURLStrListID;
	*aStream << fURLIndex;
	*aStream << fAppletCodeStrListID;
	*aStream << fAppletCodeIndex;
}

//----------------------------------------------------------------------------------------
// LJavaURLView::DoPostCreate:
//----------------------------------------------------------------------------------------

void LJavaURLView::DoPostCreate(TDocument* itsDocument)		// override 
{
	Inherited::DoPostCreate(itsDocument);
	
	CStr255_AC url;
	CStr255_AC appletCode;
	GetIndString(url, fURLStrListID, fURLIndex);
	GetIndString(appletCode, fAppletCodeStrListID, fAppletCodeIndex);
	if (url.Length() > 0)
	{
		CTempJavaAppletLocator_AC tempLocator(CChar255_AC(url), CChar255_AC(appletCode), fSize.h, fSize.v);
		CJavaAppletContainer_AC* container = DoMakeJavaAppletContainer(tempLocator);

		// start all the applets
		container->StartAllApplets();
		
		// attach ourselves to the last context
		CJavaAppletContext_AC* lastContext = container->GetLastContext();
		lastContext->SetNexLJavaView(this);	
		
		// Focus goes gonzo, fix it up here.
		InvalidateFocus();
		ForceRedraw();	
	}	
}	

//----------------------------------------------------------------------------------------
// LJavaURLView::DoMakeJavaAppletContainer: 
//----------------------------------------------------------------------------------------

CJavaAppletContainer_AC* LJavaURLView::DoMakeJavaAppletContainer(const CTempJavaAppletLocator_AC& tempLocator)
{
	return TH_new CJavaAppletContainer_AC(tempLocator);
}

//========================================================================================
// LJavaRequestedView
//========================================================================================

#undef Inherited
#define Inherited LJavaView

MA_DEFINE_CLASS(LJavaRequestedView);

//----------------------------------------------------------------------------------------
// LJavaRequestedView::LJavaRequestedView: 
//----------------------------------------------------------------------------------------

LJavaRequestedView::LJavaRequestedView(const CViewRect& itsFrame, JMAWTContextRef context,
												JMFrameRef frame) :
	LJavaView(itsFrame, context, frame, sizeRelSuperView, sizeRelSuperView)
{
	fIdentifier = kJavaViewIdentifier;
}

//----------------------------------------------------------------------------------------
// LJavaRequestedView destructor
//----------------------------------------------------------------------------------------

LJavaRequestedView::~LJavaRequestedView()
{
}

//----------------------------------------------------------------------------------------
// LJavaRequestedView::ResizeRequest: 
//----------------------------------------------------------------------------------------

bool LJavaRequestedView::ResizeRequest(Rect* desired)
{
	CRect_AC		desiredRect(*desired);
	bool			requestOK	= true;
	
	CViewPoint newSize(desiredRect.GetSize());
	
	if (newSize != fSize)
	{
		TWindow* aWindow = GetWindow();
	
		if (aWindow)
		{
			requestOK = true;
			
			// SPECIAL CASE - AWT Workaround.
			// If we're a modal dialog, and we haven't been shown yet,
			// we take this opportunity to place ourselves on the screen.
			// This is because when initially created, we don't know our real size
			
			if (aWindow->IsModal() && ! aWindow->IsVisible())
			{
				Rect* mainDevice = &(**GetMainDevice()).gdRect;
				int hPos = mainDevice->left + ((mainDevice->right - mainDevice->left) / 2) - ((desired->right - desired->left) / 2);
				int vPos = mainDevice->top + ((mainDevice->bottom - mainDevice->top) / 2) - ((desired->bottom - desired->top) / 2);
				MoveWindow(aWindow->GetWindowRef(), hPos, vPos, false);
			}

			newSize.v += 15;	// ??? What about modal dialogs without status view
			
			aWindow->Resize(newSize, kInvalidate);
		}
	}
	
	return requestOK;
}

//----------------------------------------------------------------------------------------
// LJavaRequestedView::StatusMessageRequest: 
//----------------------------------------------------------------------------------------

void LJavaRequestedView::StatusMessageRequest(const char* statusMsg)
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
	{
		LJavaMessageView* aJavaMessageView = (LJavaMessageView*) aWindow->FindSubView(kMessageViewIdentifier);
		
		if (aJavaMessageView)
			aJavaMessageView->SetText(statusMsg, kRedraw);
	}
}

//----------------------------------------------------------------------------------------
// LJavaRequestedView::ShowHideRequest: 
//----------------------------------------------------------------------------------------

void LJavaRequestedView::ShowHideRequest(bool showIt)
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
		aWindow->Show(showIt, kRedraw);
}

//----------------------------------------------------------------------------------------
// LJavaRequestedView::SetTitleRequest: 
//----------------------------------------------------------------------------------------

void LJavaRequestedView::SetTitleRequest(Str255 title)
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
		aWindow->SetTitle(title);
}

//----------------------------------------------------------------------------------------
// LJavaView::ReleaseFrameRequest: 
//----------------------------------------------------------------------------------------

void LJavaRequestedView::ReleaseFrameRequest()
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
		aWindow->CloseAndFree();
}

//========================================================================================
// LJavaMessageView
//========================================================================================

#undef Inherited
#define Inherited TStaticText

MA_DEFINE_CLASS(LJavaMessageView);

//----------------------------------------------------------------------------------------
// LJavaMessageView::LJavaMessageView: 
//----------------------------------------------------------------------------------------

LJavaMessageView::LJavaMessageView() :
	TStaticText(gZeroVPt, gZeroVPt, sizeRelSuperView, sizeVariable, kNoResource)
{	
	fIdentifier = kMessageViewIdentifier;
	fTextStyle = gApplicationStyle9;
}

//----------------------------------------------------------------------------------------
// LJavaMessageView::LJavaMessageView: 
//----------------------------------------------------------------------------------------

void LJavaMessageView::DoPostCreate(TDocument* itsDocument)
{	
	Inherited::DoPostCreate(itsDocument);
	
	AlignToSuperView();
}

//----------------------------------------------------------------------------------------
// LJavaMessageView destructor
//----------------------------------------------------------------------------------------

LJavaMessageView::~LJavaMessageView()
{
}

//----------------------------------------------------------------------------------------
// LJavaMessageView::SuperViewChangedFrame: 
//----------------------------------------------------------------------------------------

void LJavaMessageView::SuperViewChangedFrame(const CViewRect& oldFrame, const CViewRect& newFrame,
												bool invalidate)
{
	Inherited::SuperViewChangedFrame(oldFrame, newFrame, invalidate);
	
	AlignToSuperView();
}

//----------------------------------------------------------------------------------------
// LJavaMessageView::AlignToSuperView: 
//----------------------------------------------------------------------------------------

void LJavaMessageView::AlignToSuperView()
{
	CViewRect viewExtent(fSuperView->GetExtent());
	CViewRect newFrame;
	
	newFrame.left	= 2;
	newFrame.top	= viewExtent.bottom - 15;
	newFrame.right	= viewExtent.GetLength(hSel) - 17;
	newFrame.bottom	= newFrame.top + 15;
	
	SetFrame(newFrame, kInvalidate);
}

//----------------------------------------------------------------------------------------
// InitUJavaView:
//----------------------------------------------------------------------------------------

void InitUJavaView()
{
	MA_REGISTER_CLASS(LJavaView);
	MA_REGISTER_CLASS(LJavaURLView);
	MA_REGISTER_CLASS(LJavaRequestedView);
	MA_REGISTER_CLASS(LJavaMessageView);
}

//----------------------------------------------------------------------------------------
// end of file