//----------------------------------------------------------------------------------------
// MJavaView_AC.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		09/03/97	bpa		Fixed up non-precompiled includes.
		08/26/97	bpa		Created.
*/

#ifndef __MJavaView_AC__
#include "MJavaView_AC.h"
#endif

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

#ifndef __CTempGrafPort_AC__
#include "CTempGrafPort_AC.h"
#endif

#ifndef __CJavaSession_AC__
#include "CJavaSession_AC.h"
#endif

#ifndef __CRegion_AC__
#include "CRegion_AC.h"
#endif

//========================================================================================
// CLASS MJavaView_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// MJavaView_AC constructor
//----------------------------------------------------------------------------------------

MJavaView_AC::MJavaView_AC(JMAWTContextRef context,
					JMFrameRef frame) :
	fContext(context),
	fFrame(frame),
	fDeferredActivate(false)
{
	AdoptFrame(fContext, fFrame);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC destructor
//----------------------------------------------------------------------------------------

MJavaView_AC::~MJavaView_AC()
{
	JavaClose();
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::JavaClose: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaClose()
{
	if (fFrame)
	{		
		::JMSetFrameData(fFrame, (JMClientData) NULL);
		::JMFrameGoAway(fFrame);
		fFrame = NULL;
		CJavaAppletContext_AC* aJavaAppletContext = GetJavaAppletContext();		
		if (aJavaAppletContext)
			aJavaAppletContext->RemoveView(this);
	}

}

//----------------------------------------------------------------------------------------
// MJavaView_AC::AdoptFrame: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::AdoptFrame(JMAWTContextRef context, JMFrameRef frame)
{
	if (context && frame)
	{
		fContext = context;
		fFrame = frame;
		
		::JMSetFrameData(frame, (JMClientData) this);

		CJavaAppletContext_AC* aJavaAppletContext = GetJavaAppletContext();
		
		if (aJavaAppletContext)
			aJavaAppletContext->AddView(this);
	}
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::GetAWTContext: 
//----------------------------------------------------------------------------------------

JMAWTContextRef MJavaView_AC::GetAWTContext()
{
	return fContext;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::GetJavaAppletContext: 
//----------------------------------------------------------------------------------------

CJavaAppletContext_AC* MJavaView_AC::GetJavaAppletContext()
{
	CJavaAppletContext_AC* aJavaAppletContext = NULL;
	
	::JMGetAWTContextData(fContext, &aJavaAppletContext);
	
	return aJavaAppletContext;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::GetFrame: 
//----------------------------------------------------------------------------------------

JMFrameRef MJavaView_AC::JavaGetFrame()
{
	return fFrame;
}


//----------------------------------------------------------------------------------------
// MJavaView_AC::SetFrame: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaSetFrame(const CRect_AC& newFrame)
{
	CRect_AC oldFrame;
		
	::JMGetFrameSize(fFrame, oldFrame);
	
	if (oldFrame != newFrame)
		::JMSetFrameSize(fFrame, newFrame);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::Draw: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaDraw(CRegion_AC visibleRegion)
{
	if (fFrame)
	{
		CTempGrafPort_AC saveThePort();	
		
		::JMFrameUpdate(fFrame, visibleRegion);
		
		// Need to call JMIdle in order to make sure the imaging gets drawn on-screen
		// This might to be a bug in MRJ
		
		::JMIdle(CJavaSession_AC::GetSessionRef(), kDefaultJMTime);
	}
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::JavaMouseEvent: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaMouseEvent(CPoint_AC& theMouse, short modifiers)
{
	if (fFrame)
		::JMFrameClick(fFrame, theMouse, modifiers);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::JavaKeyEvent: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaKeyEvent(char asciiChar, char keyCode, short modifiers)
{
	if (fFrame)
		::JMFrameKey(fFrame, asciiChar, keyCode, modifiers);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::BecameTarget: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaBecameTarget()
{
//  bpa, bombs at window open time
//	if (fFrame)
//		::JMFrameActivate(fFrame, true);
//	else
		fDeferredActivate = true;
	
	CJavaSession_AC::SetActiveJavaView(this);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::ResignedTarget: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaResignedTarget()
{
	if (fFrame)
		::JMFrameActivate(fFrame, false);
	
	CJavaSession_AC::SetActiveJavaView(NULL);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::JavaIdle: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::JavaIdle()
{
	Point localPos;
	EventRecord event;

	if (fFrame)
	{
		CTempGrafPort_AC saveThePort();	

		OSEventAvail(0, &event);
		GetMouse(&localPos);
		
		::JMFrameMouseOver(fFrame, localPos, event.modifiers);
	}
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::SetupPortRequest: 
//----------------------------------------------------------------------------------------

void* MJavaView_AC::SetupPortRequest()
{	
	return NULL;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::RestorePortRequest: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::RestorePortRequest(void* /*refCon*/)
{
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::ResizeRequest: 
//----------------------------------------------------------------------------------------

bool MJavaView_AC::ResizeRequest(CRect_AC& /*desiredRect*/)
{
	bool requestProcOK = true;
	
	return requestProcOK;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::ShowHideRequest: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::ShowHideRequest(bool showIt)
{
	if (fDeferredActivate && showIt) 
	{
		::JMFrameActivate(fFrame, true);
		fDeferredActivate = false;
	}
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::SetTitleRequest: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::SetTitleRequest(const CStr255_AC& /* title */)
{
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::CheckUpdateRequest: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::CheckUpdateRequest()
{

}

//----------------------------------------------------------------------------------------
// MJavaView_AC::InvalRectRequest: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::InvalRectRequest(const CRect_AC& /*invalidRect*/)
{
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::StatusMessageRequest: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::StatusMessageRequest(const CStr255_AC& /* statusMsg */)
{
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::ReleaseFrameRequest: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::ReleaseFrameRequest()
{
	delete this;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::JavaViewFromFrameRef: 
//----------------------------------------------------------------------------------------

MJavaView_AC* MJavaView_AC::JavaViewFromFrameRef(JMFrameRef ref)
{
	MJavaView_AC* aJavaView = NULL;
	OSStatus err = ::JMGetFrameData(ref, &aJavaView);
	
	if (err != noErr)
		aJavaView = NULL;
	
	return aJavaView;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::JavaViewFromContextRef: 
//----------------------------------------------------------------------------------------

MJavaView_AC* MJavaView_AC::JavaViewFromContextRef(JMAWTContextRef ref)
{
	MJavaView_AC* aJavaView = NULL;
	JMFrameRef frame;
	OSStatus err = ::JMGetAWTContextFrame(ref, 0, &frame);
	
	if (err == noErr)
		aJavaView = MJavaView_AC::JavaViewFromFrameRef(frame);

	return aJavaView;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::WinSetupPortRequestProc: 
//----------------------------------------------------------------------------------------

void* MJavaView_AC::WinSetupPortRequestProc(JMFrameRef ref)
{
	void* result = NULL;
	
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(ref);
	
	if (aJavaView)
		result = aJavaView->SetupPortRequest();
	
	return result;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::WinRestorePortRequestProc: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::WinRestorePortRequestProc(JMFrameRef ref, void* refCon)
{
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(ref);
	
	if (aJavaView)
		aJavaView->RestorePortRequest(refCon);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::WinResizeRequestProc: 
//----------------------------------------------------------------------------------------

Boolean MJavaView_AC::WinResizeRequestProc(JMFrameRef ref, Rect* desired)
{
	bool result = false;
	
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(ref);
	
	if (aJavaView)
	{
		CRect_AC desiredRect(*desired);
		result = aJavaView->ResizeRequest(desiredRect);
		*desired = desiredRect;
	}
	return result;
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::WinInvalRectRequestProc: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::WinInvalRectRequestProc(JMFrameRef ref, const Rect* r)
{
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(ref);
	
	if (aJavaView)
	{
		CRect_AC invalidRect(*r);
		aJavaView->InvalRectRequest(invalidRect);
	}
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::WinShowHideRequestProc: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::WinShowHideRequestProc(JMFrameRef ref, Boolean showIt)
{
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(ref);
	
	if (aJavaView)
		aJavaView->ShowHideRequest(showIt);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::WinSetTitleRequestProc: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::WinSetTitleRequestProc(JMFrameRef ref, Str255 title)
{
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(ref);
	
	if (aJavaView)
		aJavaView->SetTitleRequest(title);
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::WinCheckUpdateRequestProc: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::WinCheckUpdateRequestProc(JMFrameRef ref)
{
	MJavaView_AC* aJavaView = MJavaView_AC::JavaViewFromFrameRef(ref);
	
	if (aJavaView)
		aJavaView->CheckUpdateRequest();
}

//----------------------------------------------------------------------------------------
// MJavaView_AC::SetupCallbacks: 
//----------------------------------------------------------------------------------------

void MJavaView_AC::SetupCallbacks(JMFrameCallbacks* callbacks)
{
	callbacks->fVersion			= kJMVersion;
	callbacks->fSetupPort		= MJavaView_AC::WinSetupPortRequestProc;
	callbacks->fRestorePort		= MJavaView_AC::WinRestorePortRequestProc;
	callbacks->fResizeRequest	= MJavaView_AC::WinResizeRequestProc;
	callbacks->fInvalRect		= MJavaView_AC::WinInvalRectRequestProc;
	callbacks->fShowHide		= MJavaView_AC::WinShowHideRequestProc;
	callbacks->fSetTitle 		= MJavaView_AC::WinSetTitleRequestProc;
	callbacks->fCheckUpdate		= MJavaView_AC::WinCheckUpdateRequestProc;
}

//----------------------------------------------------------------------------------------
// end of file