//----------------------------------------------------------------------------------------
// UJavaApplication.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __UJAVAAPPLICATION__
#include "UJavaApplication.h"
#endif

#ifndef __UJAVAVIEW__
#include "UJavaView.h"
#endif

#ifndef __UJAVAAUTOVIEW__
#include "UJavaAutoView.h"
#endif

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

//========================================================================================
// TJavaApplication
//========================================================================================

#undef Inherited
#define Inherited TApplication

//----------------------------------------------------------------------------------------
// TJavaApplication::TJavaApplication: 
//----------------------------------------------------------------------------------------

TJavaApplication::TJavaApplication(OSType itsMainFileType, OSType itsCreator) :
	TApplication(itsMainFileType, itsCreator)
{
}

//----------------------------------------------------------------------------------------
// TJavaApplication destructor
//----------------------------------------------------------------------------------------

TJavaApplication::~TJavaApplication()
{
}

//----------------------------------------------------------------------------------------
// TJavaApplication::DoInitialState: 
//----------------------------------------------------------------------------------------

void TJavaApplication::DoInitialState()
{
	Inherited::DoInitialState();

	JavaOpen();
}

//----------------------------------------------------------------------------------------
// TJavaApplication::GetEvent: 
//----------------------------------------------------------------------------------------

TToolboxEvent* TJavaApplication::GetEvent(short eventMask, unsigned long sleep,
									 		RgnHandle sleepRegion)
{
	JavaIdle();

	return Inherited::GetEvent(eventMask, sleep, sleepRegion);
}

//----------------------------------------------------------------------------------------
// TJavaApplication::MenuEvent: 
//----------------------------------------------------------------------------------------

void TJavaApplication::MenuEvent(short theMenuNumber, short theItemNumber)
{
	JavaMenuSelect(theMenuNumber, theItemNumber);
	
	Inherited::MenuEvent(theMenuNumber, theItemNumber);
}

//----------------------------------------------------------------------------------------
// TJavaApplication::RequestFrame: 
//----------------------------------------------------------------------------------------

void TJavaApplication::RequestFrame(JMAWTContextRef context,
										JMFrameRef newFrame,
										JMFrameKind kind,
										const CLongPoint_AC& initialSize,
										bool resizeable,
										JMFrameCallbacks* callbacks)
{
	RequestDocumentFrame(NULL, context, newFrame, kind, initialSize, resizeable, callbacks);
}

//----------------------------------------------------------------------------------------
// TJavaApplication::RequestFrame: 
//----------------------------------------------------------------------------------------

void TJavaApplication::RequestDocumentFrame(TDocument* itsDocument,
										JMAWTContextRef context,
										JMFrameRef newFrame,
										JMFrameKind kind,
										const CViewPoint& initialSize,
										bool resizeable,
										JMFrameCallbacks* callbacks)
{
	CJavaAppletContext_AC* aJavaAppletContext = CJavaAppletContext_AC::GetContext(context);

	TJavaView* aJavaView = NULL;
	if (aJavaAppletContext)
	 	aJavaView = dynamic_cast<TJavaView*> (aJavaAppletContext->GetNextJavaView());
	
	if (aJavaView)
	{
		// Embedded frame
		
		aJavaView->AdoptFrame(context, newFrame);
		if (aJavaAppletContext)
			aJavaAppletContext->SetNextJavaView(NULL);
		aJavaView->SetupCallbacks(callbacks);
		aJavaView->Resize(initialSize, kRedraw);
	}
	else
	{
		// Create new window for frame			
		TWindow* aJavaWindow = DoMakeJavaWindow(context, newFrame, kind, initialSize, resizeable, callbacks);
		
		// Attach a printhandler		
		aJavaView = dynamic_cast_AC(TJavaView*, aJavaWindow->FindSubView(kJavaViewIdentifier));
		TStdPrintHandler* aStdPrintHandler = TH_new TStdPrintHandler(itsDocument, aJavaView, kFixedSize, kFixedSize);
		aJavaWindow->HandlePostCreate(itsDocument);
		
		// Don't open the window yet, the window will receive Show/Hide requests separately.
		// aJavaWindow->Open();
	}
}

//----------------------------------------------------------------------------------------
// DoMakeJavaWindow
//----------------------------------------------------------------------------------------

TWindow* TJavaApplication::DoMakeJavaWindow(
								JMAWTContextRef		context,
								JMFrameRef			newFrame,
								JMFrameKind			kind,
								const CViewPoint&	initialSize,
								bool				resizeable,
								JMFrameCallbacks*	callbacks)
{
	CViewRect itsViewFrame(CViewPoint(0, 0), initialSize);
	
	bool theGrowboxIntrudes	= false; // ??? Where do I get this from?

	// offset windows

	static int rows = 1;
	static int xPos = 10, yPos = 44;
	
	Rect* rBig = &(**GetMainDevice()).gdRect;
	if (xPos + initialSize.h > rBig->right)
		xPos = 10 * ++rows;
	if (yPos + initialSize.v > rBig->bottom)
		yPos = 44;
	
	// make a place

	Rect rBounds;
	short windowProc;
	bool goAway;
	int resizeHeight = (resizeable && ! theGrowboxIntrudes)? 15 : 0;
	
	switch (kind)
	{
		case eBorderlessModelessWindowFrame:
			goAway = false;
			windowProc = plainDBox;
			SetRect(&rBounds, xPos, yPos, xPos + initialSize.h, yPos + initialSize.v + resizeHeight);
			xPos += 10, yPos += 10;
			break;
		
		case eModelessWindowFrame:
			goAway = true;
			windowProc = resizeable? zoomDocProc : documentProc;
			SetRect(&rBounds, xPos, yPos, xPos + initialSize.h, yPos + initialSize.v + resizeHeight);
			xPos += 10, yPos += 10;
			break;
		
		case eModalWindowFrame:
			goAway = true;
			windowProc = movableDBoxProc;
			resizeable = false;
			Rect* mainDevice = &(**GetMainDevice()).gdRect;
			rBounds.left = mainDevice->left + ((mainDevice->right - mainDevice->left) / 2) - (initialSize.h / 2);
			rBounds.top = mainDevice->top + ((mainDevice->bottom - mainDevice->top) / 2) - (initialSize.v / 2);
			rBounds.right = rBounds.left + initialSize.h;
			rBounds.bottom = rBounds.top + initialSize.v;
			break;
	}
	
	if (rBounds.right < rBounds.left + 30)
		rBounds.right = rBounds.left + 30;
	
	if (rBounds.bottom < rBounds.top + 30)
		rBounds.bottom = rBounds.top + 30;
	
	WindowPtr theWMgrWindow = NewCWindow(nil, &rBounds, gEmptyString, false, windowProc, WindowPtr(-1), goAway, NULL /* nb theWindow */);
	
	/*
	if (theWMgrWindow)
	{
		WCTabHandle hWCTB = theWindowColors;
		
		if (hWCTB && HandToHand(&Handle(hWCTB)) == noErr && hWCTB)
			SetWinColor(theWMgrWindow, hWCTB);
	}
	*/
		
	TWindow* theWindow = TH_new TWindow(theWMgrWindow, resizeable, goAway, true);
	// NB, set the refcon to be this window, and set free on close flag.
	SetWRefCon(theWindow->GetWindowRef(), (long) theWindow);
	theWindow->SetFreeOnClosing(true);	

	// Create Message view
	
	TJavaAutoMessageView* aJavaAutoMessageView = TH_new TJavaAutoMessageView;
	theWindow->AddSubView(aJavaAutoMessageView);
	
	// Create default JavaView embedded into window
	
	TJavaAutoView* aJavaAutoView = TH_new TJavaAutoView(itsViewFrame, context, newFrame);
	theWindow->AddSubView(aJavaAutoView);	
	
	aJavaAutoView->SetupCallbacks(callbacks);
	
	theWindow->SetWindowTarget(aJavaAutoView);
	
	return theWindow;
}

//----------------------------------------------------------------------------------------
// end of file