//----------------------------------------------------------------------------------------
// UJavaAutoView.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __UJAVAAUTOVIEW__
#include "UJavaAutoView.h"
#endif

#ifndef __UBACKGROUNDADORNER__
#include "UBackgroundAdorner.h"
#endif

//========================================================================================
// TJavaAutoView
//========================================================================================

#undef Inherited
#define Inherited TJavaView

MA_DEFINE_CLASS(TJavaAutoView);

//----------------------------------------------------------------------------------------
// TJavaAutoView::TJavaAutoView: 
//----------------------------------------------------------------------------------------

TJavaAutoView::TJavaAutoView(const CViewRect& itsFrame, JMAWTContextRef context,
												JMFrameRef frame) :
	TJavaView(itsFrame, context, frame, sizeRelSuperView, sizeRelSuperView)
{
	fIdentifier = kJavaViewIdentifier;
}

//----------------------------------------------------------------------------------------
// TJavaAutoView destructor
//----------------------------------------------------------------------------------------

TJavaAutoView::~TJavaAutoView()
{
}

//----------------------------------------------------------------------------------------
// TJavaAutoView::ResizeRequest: 
//----------------------------------------------------------------------------------------

bool TJavaAutoView::ResizeRequest(CRect_AC& desiredRect)
{
	bool requestOK = true;
	
	CViewPoint newSize(desiredRect.GetSize());
	
	if (newSize != fSize)
	{
		TWindow* aWindow = GetWindow();
	
		if (aWindow)
		{
			requestOK = true;
			
			// SPECIAL CASE - AWT Workaround.
			// If we're a modal dialog, and we haven't been shown yet,
			// we take this opportunity to place ourselves on the screen.
			// This is because when initially created, we don't know our real size
			
			if (aWindow->IsModal() && ! aWindow->IsVisible())
			{
				Rect* mainDevice = &(**GetMainDevice()).gdRect;
				int hPos = mainDevice->left + ((mainDevice->right - mainDevice->left) / 2) - ((desiredRect.right - desiredRect.left) / 2);
				int vPos = mainDevice->top + ((mainDevice->bottom - mainDevice->top) / 2) - ((desiredRect.bottom - desiredRect.top) / 2);
				MoveWindow(aWindow->GetWindowRef(), hPos, vPos, false);
			}

			newSize.v += 15;	// ??? What about modal dialogs without status view
			
			aWindow->Resize(newSize, kInvalidate);
		}
	}
	
	return requestOK;
}

//----------------------------------------------------------------------------------------
// TJavaAutoView::StatusMessageRequest: 
//----------------------------------------------------------------------------------------

void TJavaAutoView::StatusMessageRequest(const CStr255_AC& statusMsg)
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
	{
		TJavaAutoMessageView* aJavaAutoMessageView = dynamic_cast<TJavaAutoMessageView*> (aWindow->FindSubView(kMessageViewIdentifier));
		
		if (aJavaAutoMessageView)
			aJavaAutoMessageView->SetText(statusMsg, kRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TJavaAutoView::ShowHideRequest: 
//----------------------------------------------------------------------------------------

void TJavaAutoView::ShowHideRequest(bool showIt)
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
		aWindow->Show(showIt, kRedraw);
}

//----------------------------------------------------------------------------------------
// TJavaAutoView::SetTitleRequest: 
//----------------------------------------------------------------------------------------

void TJavaAutoView::SetTitleRequest(const CStr255_AC& title)
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
		aWindow->SetTitle(title);
}

//----------------------------------------------------------------------------------------
// TJavaView::ReleaseFrameRequest: 
//----------------------------------------------------------------------------------------

void TJavaAutoView::ReleaseFrameRequest()
{
	TWindow* aWindow = GetWindow();
	
	if (aWindow)
		aWindow->CloseAndFree();
}

//========================================================================================
// TJavaAutoMessageView
//========================================================================================

#undef Inherited
#define Inherited TStaticText

MA_DEFINE_CLASS(TJavaAutoMessageView);

//----------------------------------------------------------------------------------------
// TJavaAutoMessageView::TJavaAutoMessageView: 
//----------------------------------------------------------------------------------------

TJavaAutoMessageView::TJavaAutoMessageView() :
	TStaticText(gZeroVPt, gZeroVPt, sizeRelSuperView, sizeVariable, kNoResource)
{	
	fIdentifier = kMessageViewIdentifier;
	fTextStyle = gApplicationStyle9;
}

//----------------------------------------------------------------------------------------
// TJavaAutoMessageView destructor
//----------------------------------------------------------------------------------------

TJavaAutoMessageView::~TJavaAutoMessageView()
{
}

//----------------------------------------------------------------------------------------
// TJavaAutoMessageView::TJavaAutoMessageView: 
//----------------------------------------------------------------------------------------

void TJavaAutoMessageView::DoPostCreate(TDocument* itsDocument)
{	
	Inherited::DoPostCreate(itsDocument);
	
	AddAdorner(TH_new TBackgroundAdorner, kAdornFirst, kDontRedraw);

	Inset(CViewPoint(4, 1), kDontRedraw);
	AlignToSuperView();
}

//----------------------------------------------------------------------------------------
// TJavaAutoMessageView::SuperViewChangedFrame: 
//----------------------------------------------------------------------------------------

void TJavaAutoMessageView::SuperViewChangedFrame(const CViewRect& oldFrame, const CViewRect& newFrame,
												bool invalidate)
{
	Inherited::SuperViewChangedFrame(oldFrame, newFrame, invalidate);
	
	AlignToSuperView();
}

//----------------------------------------------------------------------------------------
// TJavaAutoMessageView::AlignToSuperView: 
//----------------------------------------------------------------------------------------

void TJavaAutoMessageView::AlignToSuperView()
{
	CViewRect viewExtent(fSuperView->GetExtent());
	CViewRect newFrame(0, viewExtent.bottom - 15, viewExtent.GetLength(hSel) - 15, viewExtent.bottom);
	
	SetFrame(newFrame, kInvalidate);
}

//----------------------------------------------------------------------------------------
// InitUJavaAutoView:
//----------------------------------------------------------------------------------------

void InitUJavaAutoView()
{
	MA_REGISTER_CLASS(TJavaAutoView);
	MA_REGISTER_CLASS(TJavaAutoMessageView);
}

//----------------------------------------------------------------------------------------
// end of file