//----------------------------------------------------------------------------------------
// UJavaView.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __UJAVAVIEW__
#include "UJavaView.h"
#endif

#ifndef __CJavaSession_AC__
#include "CJavaSession_AC.h"
#endif

#ifndef __CJavaAppletLocator_AC__
#include "CJavaAppletLocator_AC.h"
#endif

#ifndef __CJavaAppletContainer_AC__
#include "CJavaAppletContainer_AC.h"
#endif

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

//========================================================================================
// CLASS TJavaView
//========================================================================================

#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS(TJavaView);

//----------------------------------------------------------------------------------------
// TJavaView::TJavaView: 
//----------------------------------------------------------------------------------------

TJavaView::TJavaView(const CViewRect&	itsFrame,
					JMAWTContextRef		context,
					JMFrameRef			frame,
					SizeDeterminer		itsHSizeDet,
		  			SizeDeterminer		itsVSizeDet) :
	TView(itsFrame[topLeft], itsFrame[botRight], itsHSizeDet, itsVSizeDet),
	MJavaView_AC(context, frame)
{
}

//----------------------------------------------------------------------------------------
// TJavaView::~TJavaView: 
//----------------------------------------------------------------------------------------

TJavaView::~TJavaView()
{
}

//----------------------------------------------------------------------------------------
// TJavaView::Close
//----------------------------------------------------------------------------------------

void TJavaView::Close()
{
	Inherited::Close();
	
	MJavaView_AC* aJavaView = CJavaSession_AC::GetActiveJavaView();	
	if (aJavaView == this)
		CJavaSession_AC::SetActiveJavaView(NULL);

	aJavaView = CJavaSession_AC::GetMouseOverJavaView();	
	if (aJavaView == this)
		CJavaSession_AC::SetMouseOverJavaView(NULL);
		
	// Preempt the call to JavaClose, so we clean up while the wmgr window is still around.
	// If we don't, the context's fAppletViewer reference may cause a crash while being deleted.
	JavaClose();
}

//----------------------------------------------------------------------------------------
// TJavaView::SetFrame: 
//----------------------------------------------------------------------------------------

void TJavaView::SetFrame(const CViewRect& newFrame, bool invalidate)
{	
	Inherited::SetFrame(newFrame, invalidate);
	
	JavaSetFrame(ViewToQD(newFrame));
}

//----------------------------------------------------------------------------------------
// TJavaView::Draw: 
//----------------------------------------------------------------------------------------

void TJavaView::Draw(const CViewRect& area)
{	
	Inherited::Draw(area);
	
	CTempRegion_AC	visibleRegion;
	GetVisibleRegion(visibleRegion);
	JavaDraw(visibleRegion);
}

//----------------------------------------------------------------------------------------
// TJavaView::DoMouseCommand: 
//----------------------------------------------------------------------------------------

void TJavaView::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC /* hysteresis */)
{
	JavaMouseEvent(ViewToQD(theMouse), event->GetEventRecord().modifiers);
}

//----------------------------------------------------------------------------------------
// TJavaView::DoKeyEvent: 
//----------------------------------------------------------------------------------------

void TJavaView::DoKeyEvent(TToolboxEvent* event)
{
	JavaKeyEvent(event->GetText()[1], event->GetKeyCode(), event->GetEventRecord().modifiers);
}

//----------------------------------------------------------------------------------------
// TJavaView::WantsToBeTarget: 
//----------------------------------------------------------------------------------------

bool TJavaView::WantsToBeTarget()
{
	return true;
}

//----------------------------------------------------------------------------------------
// TJavaView::BecameTarget: 
//----------------------------------------------------------------------------------------

void TJavaView::BecameTarget()
{
	Inherited::BecameTarget();

	JavaBecameTarget();
}

//----------------------------------------------------------------------------------------
// TJavaView::ResignedTarget: 
//----------------------------------------------------------------------------------------

void TJavaView::ResignedTarget()
{
	Inherited::ResignedTarget();

	JavaResignedTarget();
}

//----------------------------------------------------------------------------------------
// TJavaView::DoSetCursor: 
//----------------------------------------------------------------------------------------

void TJavaView::DoSetCursor(const CViewPoint& localPoint, RgnHandle cursorRegion)
{
	Inherited::DoSetCursor(localPoint, cursorRegion);

	CJavaSession_AC::SetMouseOverJavaView(this);
}

//----------------------------------------------------------------------------------------
// TJavaView::JavaIdle: 
//----------------------------------------------------------------------------------------

void TJavaView::JavaIdle()
{
	if (Focus())
	{
		MJavaView_AC::JavaIdle();
	}
}

//----------------------------------------------------------------------------------------
// TJavaView::SetupPortRequest: 
//----------------------------------------------------------------------------------------

void* TJavaView::SetupPortRequest()
{
	// We have to invalidate the focus instead of saving and restoring the focus
	// because the port keeps changing.
	
	InvalidateFocus();
	
	Focus();
	
	return NULL;
}

//----------------------------------------------------------------------------------------
// TJavaView::RestorePortRequest: 
//----------------------------------------------------------------------------------------

void TJavaView::RestorePortRequest(void* /*refCon*/)
{
	// See TJavaView::SetupPortRequest for comments

	InvalidateFocus();
}

//----------------------------------------------------------------------------------------
// TJavaView::ResizeRequest: 
//----------------------------------------------------------------------------------------

bool TJavaView::ResizeRequest(CRect_AC& desiredRect)
{
	bool requestOK = true;
	
	CViewPoint newSize(desiredRect.GetSize());
	
	if (newSize != fSize)
	{
		Resize(newSize, kInvalidate);
		
		if (newSize != fSize)
		{
			// Tell the applet that we refused to change the size
			
			desiredRect.right = fSize.h;
			desiredRect.bottom = fSize.v;
			requestOK = false;
		}
	}
	
	return requestOK;
}

//----------------------------------------------------------------------------------------
// TJavaView::ShowHideRequest: 
//----------------------------------------------------------------------------------------

void TJavaView::ShowHideRequest(bool showIt)
{
	// The port may have changed by Java prior to the show/hide request getting called.
	InvalidateFocus();
	
	Show(showIt, kRedraw);
	
	MJavaView_AC::ShowHideRequest(showIt);	
}

//----------------------------------------------------------------------------------------
// TJavaView::CheckUpdateRequest: 
//----------------------------------------------------------------------------------------

void TJavaView::CheckUpdateRequest()
{
	Update();
}

//----------------------------------------------------------------------------------------
// TJavaView::InvalRectRequest: 
//----------------------------------------------------------------------------------------

void TJavaView::InvalRectRequest(const CRect_AC& invalidRect)
{
	Invalidate(invalidRect);
}

//========================================================================================
// TJavaURLView
//========================================================================================

#undef Inherited
#define Inherited TJavaView

MA_DEFINE_CLASS(TJavaURLView);

//----------------------------------------------------------------------------------------
// TJavaURLView constructor:
//----------------------------------------------------------------------------------------

TJavaURLView::TJavaURLView(ResNumber urlStrListID, short urlIndex, 
				 ResNumber appletCodeStrListID, short appletCodeIndex,
				 const CViewRect& itsFrame,
				 SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet) :
	TJavaView(itsFrame, NULL, NULL, itsHSizeDet, itsVSizeDet),
	fURLStrListID(urlStrListID),
	fURLIndex(urlIndex),
	fAppletCodeStrListID(appletCodeStrListID),
	fAppletCodeIndex(appletCodeIndex)
{
}

//----------------------------------------------------------------------------------------
// TJavaURLView destructor:
//----------------------------------------------------------------------------------------

TJavaURLView::~TJavaURLView()
{
}

//----------------------------------------------------------------------------------------
// TJavaURLView::ReadFields:
//----------------------------------------------------------------------------------------

void TJavaURLView::ReadFields(CStream_AC* aStream)		// override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fURLStrListID;
	*aStream >> fURLIndex;
	*aStream >> fAppletCodeStrListID;
	*aStream >> fAppletCodeIndex;
}

//----------------------------------------------------------------------------------------
// TJavaURLView::WriteFields:
//----------------------------------------------------------------------------------------

void TJavaURLView::WriteFields(CStream_AC* aStream) const		// override 
{
	Inherited::WriteFields(aStream);
	
	*aStream << fURLStrListID;
	*aStream << fURLIndex;
	*aStream << fAppletCodeStrListID;
	*aStream << fAppletCodeIndex;
}

//----------------------------------------------------------------------------------------
// TJavaURLView::DoPostCreate:
//----------------------------------------------------------------------------------------

void TJavaURLView::DoPostCreate(TDocument* itsDocument)		// override 
{
	Inherited::DoPostCreate(itsDocument);
	
	CStr255_AC url;
	CStr255_AC appletCode;
	GetIndString(url, fURLStrListID, fURLIndex);
	GetIndString(appletCode, fAppletCodeStrListID, fAppletCodeIndex);
	if (url.Length() > 0)
	{
		CTempJavaAppletLocator_AC tempLocator(CChar255_AC(url), CChar255_AC(appletCode), fSize.h, fSize.v);
		CAutoPtr_AC<CJavaAppletContainer_AC> container = DoMakeJavaAppletContainer(tempLocator);

		// start all the applets
		container->StartAllApplets();
		
		// attach ourselves to the last context
		CJavaAppletContext_AC* lastContext = container->GetLastContext();
		lastContext->SetNextJavaView(this);	

		container.release();		
		
		// Focus goes gonzo, fix it up here.
		InvalidateFocus();
		ForceRedraw();	
	}	
}	

//----------------------------------------------------------------------------------------
// TJavaURLView::DoMakeJavaAppletContainer: 
//----------------------------------------------------------------------------------------

CJavaAppletContainer_AC* TJavaURLView::DoMakeJavaAppletContainer(const CTempJavaAppletLocator_AC& tempLocator)
{
	return TH_new CJavaAppletContainer_AC(tempLocator);
}

//----------------------------------------------------------------------------------------
// InitUJavaView:
//----------------------------------------------------------------------------------------

void InitUJavaView()
{
	MA_REGISTER_CLASS(TJavaView);
	MA_REGISTER_CLASS(TJavaURLView);
}

//----------------------------------------------------------------------------------------
// end of file