//----------------------------------------------------------------------------------------
// UJavaView.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __UJAVAVIEW__
#define __UJAVAVIEW__

// MacApp

#ifndef __UVIEW__
#include "UView.h"
#endif

// ACS

#ifndef __MJavaView_AC__
#include "MJavaView_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CJavaAppletContainer_AC;
class CTempJavaAppletLocator_AC;

//----------------------------------------------------------------------------------------
// TJavaView
//----------------------------------------------------------------------------------------

class EXPORT_MA TJavaView : public TView, public MJavaView_AC
{
	MA_DECLARE_CLASS;
	
public:
	TJavaView(const CViewRect&	itsFrame = gZeroRect,
												JMAWTContextRef		context = NULL,
												JMFrameRef			frame = NULL,
												SizeDeterminer		itsHSizeDet = sizeVariable,
									  			SizeDeterminer		itsVSizeDet = sizeVariable);	
	virtual ~TJavaView();
	
	virtual void Close();
		
	// MacApp overrides
	
	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);
	
	virtual void Draw(const CViewRect& area);
	
	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);
	
	virtual void DoKeyEvent(TToolboxEvent* event);

	virtual bool WantsToBeTarget();
	
	virtual void BecameTarget();
	
	virtual void ResignedTarget();
		
	virtual void DoSetCursor(const CViewPoint& localPoint, RgnHandle cursorRegion);
	
	// Java overrides
	
	virtual void JavaIdle();
	
	virtual void* SetupPortRequest();
	
	virtual void RestorePortRequest(void* refCon);
	
	virtual bool ResizeRequest(CRect_AC& desiredRect);
	
	virtual void ShowHideRequest(bool showIt);
	
	virtual void CheckUpdateRequest();
	
	virtual void InvalRectRequest(const CRect_AC& invalidRect);
	
};

//----------------------------------------------------------------------------------------
// TJavaURLView
//----------------------------------------------------------------------------------------

class EXPORT_MA TJavaURLView : public TJavaView
// This view is designed to stream in a URL from a view resource
{
	MA_DECLARE_CLASS;
	
public:
	TJavaURLView(ResNumber urlStrListID = kNoResource_AC, short urlIndex = 0, 
				 ResNumber appletCodeStrListID = kNoResource_AC, short appletCodeIndex = 0,
				 const CViewRect& itsFrame = gZeroRect,
				 SizeDeterminer itsHSizeDet = sizeVariable, SizeDeterminer itsVSizeDet = sizeVariable);
	
	virtual ~TJavaURLView();
	
	// Streaming
	
	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;
	
	virtual void DoPostCreate(TDocument* itsDocument);
	
	// New API
	
	virtual CJavaAppletContainer_AC* DoMakeJavaAppletContainer(const CTempJavaAppletLocator_AC& tempLocator);
		// create an applet container, given fURL and frame

protected:
	ResNumber	fURLStrListID;						// url:	STR# rsrc id
	short		fURLIndex;							// index into STR#
	ResNumber	fAppletCodeStrListID;				// applet code:	STR# rsrc id
	short		fAppletCodeIndex;					// index into STR#
};

//----------------------------------------------------------------------------------------

EXPORT_MA void InitUJavaView();

//----------------------------------------------------------------------------------------

#endif
