//----------------------------------------------------------------------------------------
// UMacAppletApplication.cpp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Change History:
		08/26/97	bpa		Created.
*/

#ifndef __UMACAPPLETAPPLICATION__
#include "UMacAppletApplication.h"
#endif

#ifndef __MacAppletConstants__
#include "MacAppletConstants.h"
#endif

#ifndef __UMacAppletDocument_
#include "UMacAppletDocument.h"
#endif

// MacApp

#ifndef __UJAVAVIEW__
#include "UJavaView.h"
#endif

#ifndef __UJAVAAUTOVIEW__
#include "UJavaAutoView.h"
#endif

// ACS

#ifndef __CJavaAppletContainer_AC__
#include "CJavaAppletContainer_AC.h"
#endif

#ifndef __CJavaAppletContext_AC__
#include "CJavaAppletContext_AC.h"
#endif

#ifndef __CJavaAppletLocator_AC__
#include "CJavaAppletLocator_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short		kEmbeddedJavaViewRsrcID		= 1000;
const short		kResourceJavaViewRsrcID		= 1002;
const short		kAppletTagViewRsrcID		= 1001;
const short		kAboutBoxRsrcID				= 1000;

//========================================================================================
// TMacAppletApplication
//========================================================================================

//----------------------------------------------------------------------------------------
// TMacAppletApplication::TMacAppletApplication: 
//----------------------------------------------------------------------------------------
#undef Inherited
#define Inherited TJavaApplication

TMacAppletApplication::TMacAppletApplication(OSType itsMainFileType, OSType itsCreator) :
	TJavaApplication(itsMainFileType, itsCreator)
{
	fLaunchWithNewDocument = false;
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication destructor
//----------------------------------------------------------------------------------------

TMacAppletApplication::~TMacAppletApplication()
{
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::DoAboutBox: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::DoAboutBox()
{
	MacAppAlert(kAboutBoxRsrcID, NULL);
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::DoMakeDocument: 
//----------------------------------------------------------------------------------------

TDocument* TMacAppletApplication::DoMakeDocument(CommandNumber /*itsCommandNumber*/, TFile* itsFile)
{
	return TH_new TMacAppletDocument(itsFile);
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::DoSetupMenus: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::DoSetupMenus()
{
	Inherited::DoSetupMenus();
		
	CJavaAppletContext_AC* aJavaAppletContext = GetActiveJavaContext();
	bool hasContext = (aJavaAppletContext != NULL);	

	Enable(cLocalApplet, true);
	Enable(cRemoteApplet, true);
	Enable(cEmbeddedApplet, true);
	Enable(cResourceApplet, true);
	Enable(cReloadApplet, hasContext);
	Enable(cRestartApplet, hasContext);
	Enable(cSuspendApplet, hasContext);
	Enable(cResumeApplet, hasContext);
	Enable(cShowAppletTag, hasContext);
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::DoMenuCommand: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber)
	{
		case cLocalApplet:
			LocalApplet();
			break;
		
		case cRemoteApplet:
			RemoteApplet();
			break;
		
		case cEmbeddedApplet:
			EmbeddedApplet();
			break;

		case cResourceApplet:
			ResourceApplet();
			break;
		
		case cReloadApplet:
			ReloadApplet();
			break;
		
		case cRestartApplet:
			RestartApplet();
			break;
		
		case cSuspendApplet:
			SuspendApplet();
			break;
		
		case cResumeApplet:
			ResumeApplet();
			break;
		
		case cShowAppletTag:
			ShowAppletTag();
			break;
		
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::LocalApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::LocalApplet()
// Demonstrates the embedding of a Java applet located in a local directory.
{
	CTempJavaAppletLocator_AC tempLocator("file:///$APPLICATION/Applets/DrawTest/example1.html",
										"DrawTest.class",		/* from the applet tag */
										400, 400);
										
	// use stack-based AppletLocator - we can do this locally, since it self-deletes when it goes out of context
	CJavaAppletContainer_AC* appletContainer = TH_new CJavaAppletContainer_AC(tempLocator);
	
	// start all the applets on that page
	appletContainer->StartAllApplets();
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::RemoteApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::RemoteApplet()
// Demonstrates the embedding of a Java applet located over the network.
{
	CJavaAutoAppletContainer_AC* container = TH_new CJavaAutoAppletContainer_AC("http://www.hypno.com/javabeta/fromandy/splash/splash.html");
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::EmbeddedApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::EmbeddedApplet()
// Demonstrates the embedding of a Java applet in a subview of a scroller.
{
	CTempJavaAppletLocator_AC tempLocator("file:///$APPLICATION/Applets/ArcTest/example1.html",
											"ArcTest.class",		/* from the applet tag */
											400, 400);

	// create an AppletLocator - we can do this locally, since it self-deletes when it goes out of context
	CJavaAppletContainer_AC* appletContainer = TH_new CJavaAppletContainer_AC(tempLocator);
	
	// start all the applets on that page
	appletContainer->StartAllApplets();
	CJavaAppletContext_AC* lastContext = appletContainer->GetLastContext();
	
	TWindow* aWindow = TViewServer::fgViewServer->NewTemplateWindow(kEmbeddedJavaViewRsrcID, NULL);
	
	if (aWindow)
	{
		TJavaView* aJavaView = dynamic_cast_AC(TJavaView*, aWindow->FindSubView(kJavaViewIdentifier));	
		if (aJavaView && lastContext)
		{
			lastContext->SetNextJavaView(aJavaView);		
			TStdPrintHandler* aStdPrintHandler = TH_new TStdPrintHandler(NULL, aJavaView, kFixedSize, kFixedSize);
		}
		
		aWindow->Open();
	}
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::ResourceApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::ResourceApplet()
// Demonstrates the embedding of a Java applet specified via resources
{
	TWindow* aWindow = TViewServer::fgViewServer->NewTemplateWindow(kResourceJavaViewRsrcID, NULL);
	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::ReloadApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::ReloadApplet()
{
	CJavaAppletContext_AC* aJavaAppletContext = GetActiveJavaContext();
	
	if (aJavaAppletContext)
		aJavaAppletContext->ReloadApplet();
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::RestartApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::RestartApplet()
{
	CJavaAppletContext_AC* aJavaAppletContext = GetActiveJavaContext();
	
	if (aJavaAppletContext)
		aJavaAppletContext->RestartApplet();
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::SuspendApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::SuspendApplet()
{
	CJavaAppletContext_AC* aJavaAppletContext = GetActiveJavaContext();
	
	if (aJavaAppletContext)
		aJavaAppletContext->SuspendApplet();
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::ResumeApplet: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::ResumeApplet()
{
	CJavaAppletContext_AC* aJavaAppletContext = GetActiveJavaContext();
	
	if (aJavaAppletContext)
		aJavaAppletContext->ResumeApplet();
}

//----------------------------------------------------------------------------------------
// TMacAppletApplication::ShowAppletTag: 
//----------------------------------------------------------------------------------------

void TMacAppletApplication::ShowAppletTag()
{
	CJavaAppletContext_AC* aJavaAppletContext = GetActiveJavaContext();	
	if (aJavaAppletContext)
	{
		Handle appletTag = aJavaAppletContext->GetAppletTag();		
		if (appletTag)
		{
			TWindow* aWindow = gViewServer->NewTemplateWindow(kAppletTagViewRsrcID, NULL);			
			if (aWindow)
			{
				TTEView* aTTEView = dynamic_cast_AC(TTEView*, aWindow->FindSubView('TEXT'));
				aTTEView->StuffText(appletTag);
				aTTEView->ShowReverted();	// synch extent

				aWindow->Open();
			}
		}
	}
}
