//--------------------------------------------------------------------------------------------------
// SimpleSizer.r
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//
//	Resource File for the Skeleton Sample Program. 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// INCLUDE FILES 
//--------------------------------------------------------------------------------------------------

// Standard Includes 

// MacAppTypes.r must be first
#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif

#ifndef __BalloonTypes__
#include "BalloonTypes.r"
#endif

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

#ifndef THINK_Rez
#if qDebug | qPerform
include "Debug.rsrc" not 'ckid';
#endif

include "Dialog.rsrc" not 'ckid';
include "MacApp.rsrc" not 'ckid';

// Includes for Building Blocks

include "Printing.rsrc" not 'ckid';

// Include Code Resources

#ifndef MW_Rez
#if !qNoCode
include $$Shell("ObjApp")$$Shell("XAppName") 'CODE';
#if qModelCFM
include $$Shell("ObjApp")$$Shell("XAppName") 'cfrg';
include $$Shell("ObjApp")$$Shell("XAppName") 'rseg';
#endif
#endif
#endif // MW_Rez 
#endif //!THINK_Rez

//--------------------------------------------------------------------------------------------------
// CONSTANTS 
//--------------------------------------------------------------------------------------------------

// Finder Information
// ------------------

#if !defined(THINK_Rez) && !defined(MW_Rez)
#define kApplicationName	$$Shell("XAppName")
#else
#define kApplicationName	TARGET_APPLICATION_NAME
#endif // !THINK_Rez

#define kSignature			'SS02'			// Application signature 
#define kFileType			'SF02'			// Data file type 
#define kStationery			'sF02'			// Stationery file type 

// Bundle Constants
// ------------------

#define kBundleID						128
#define kApplicationID					128

// Constants for Menu IDs
// ----------------------

#define mSkeletonMenu	4

// Command Numbers For Menu Items
// ------------------------------

#define cFirstWindow				1001
#define cLastWindow					1004

// STR# resources
// --------------

#define kSizerMenuHelp				1000
#define kSizerWindowTitles			1001

// Memory Management Resources
// ---------------------------

#define kSizerSeg	1000
#define kSizerMem	1000
#define kSizerRes	1000

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR MENUS 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Menu Bars 
//--------------------------------------------------------------------------------------------------

resource 'MBAR' (kMBarDisplayed,
#if qNames
"Sizer MBAR",
#endif
	purgeable)
{
	{ mApple; mFile; mEdit }
};

//--------------------------------------------------------------------------------------------------
// Menus 
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc" 'CMNU' (mApple);	// Grab the default Apple menus
include "Defaults.rsrc" 'CMNU' (mEdit);		// Grab the default Edit menu

resource 'CMNU' (mFile,
#if qNames
"mFile",
#endif
purgeable) {
	mFile,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"File",
	{
	"Show Introduction",				noIcon, "N",	noMark, plain, cFirstWindow;
	"Show Sticky Views",				noIcon, "1",	noMark, plain, cFirstWindow + 1;
	"Show Views Sticking to Window",	noIcon, "2",	noMark, plain, cFirstWindow + 2;
	"Show Proportional Views",			noIcon, "3",	noMark, plain, cFirstWindow + 3;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Close",			noIcon, "W",	noMark, plain, cClose;
	"-",				noIcon, noKey,	noMark, plain, nocommand;
	"Quit",				noIcon, "Q",	noMark, plain, cQuit
	}
};

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
purgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {	/* array: 13 elements */
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
	}
};

//--------------------------------------------------------------------------------------------------
// Balloon Help Resources for Menus 
//--------------------------------------------------------------------------------------------------

// Includes resource 'STR#' (kStandardHelpStrings, "Balloon Help String 1")
// with our application name substituted in strings
#include "HelpStrings.r"

include "Defaults.rsrc"  'hmnu' (mFile);	
include "Defaults.rsrc"  'hmnu' (mEdit);	

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR VIEWS, DIALOGS, ALERTS etc. 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// About Box
//--------------------------------------------------------------------------------------------------

resource 'DITL' (phAboutApp,
#if qNames
"phAboutApp",
#endif
	purgeable) {
	 {	/* array DITLarray: 3 elements */
		/* [1] */
		{167, 288, 187, 348},
		Button {
			enabled,
			"OK"
		};
		/* [2] */
		{10, 75, 154, 348},
		StaticText {
			disabled,
			"This sample program demonstrates large views and mouse tracking."
			"\n"
			WRITTEN_WITH_MACAPP
		};
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			noteIcon
		}
	}
};

include "Defaults.rsrc"  'ALRT' (phAboutApp);		// Grab the default about box
include "Defaults.rsrc"	'aete' (0);					// Grab the default AppleEvent terminology
include "Defaults.rsrc"  'STR#' (kDefaultCredits);	// Grab the default credits


//--------------------------------------------------------------------------------------------------
// MultiFinder information
//--------------------------------------------------------------------------------------------------

/* Additional size deltas to be added to partition size */

#if qNames
#define kNamesSizeDelta 75
#else
#define kNamesSizeDelta 0
#endif

#if qDebug
#define kDebugSizeDelta 0
#else
#define kDebugSizeDelta 0
#endif

#define kModelFarSizeDelta 0

#ifndef MW_Rez					// MW projects supply size resource for us
resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	notStationeryAware,				
	reserved,
	reserved,
	reserved,
	reserved,
	
	/* Suggested partition size */
	(1000 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024,
	
	/* Minimum partition size */
	(800 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024
};
#endif // !MW_Rez

//--------------------------------------------------------------------------------------------------
// Icons
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc"  'ICN#' (kApplicationID);	// MacApp Family large black & white	
include "Defaults.rsrc"  'ics#' (kApplicationID);	// MacApp Family small black & white	
include "Defaults.rsrc"  'ics4' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'ics8' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'icl4' (kApplicationID);	// MacApp Family large 4 bit
include "Defaults.rsrc"  'icl8' (kApplicationID);	// MacApp Family large 8 bit

//--------------------------------------------------------------------------------------------------
// Memory usage information for Sizer
//--------------------------------------------------------------------------------------------------

// See UMemory.h
// -------------

// Code fragment resource for PowerPC native apps
// ----------------------------------------------
#ifndef MW_Rez 	// Metrowerks will supply it's own cfrg
#if qPowerPC
#include "CodeFragmentTypes.r"

resource 'cfrg' (0, "Configuration Info") {
	{
		kPowerPC,
		kFullLib,
		kNoVersionNum,
		kNoVersionNum,
		0,
		0,
		kIsApp,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		"Sizer"
	}
};
#endif
#endif // MW_Rez

//--------------------------------------------------------------------------------------------------
// Icons, Bundles and FRefsOh my! (don't forget the Signature)
//--------------------------------------------------------------------------------------------------

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable)
{
	"Sizer Apple Computer, Inc. 1988-95"
};

resource 'FREF' (kApplicationID,
#if qNames
"Sizer Application",
#endif
	purgeable)
{
	'APPL', 0, ""
};

resource 'BNDL' (kBundleID,
#if qNames
"Sizer",
#endif
	purgeable) { kSignature, 0,
	{
		'ICN#', { 0, kApplicationID },
		'FREF', { 0, kApplicationID }
	}
};

//--------------------------------------------------------------------------------------------------
//  Version resources
//--------------------------------------------------------------------------------------------------

// The revision of this particular file

RESOURCE 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x01,
	0x01,
	development,
	0x01,
	verUs,
	"1.1d1",
	"Sizer 1.1d1, Apple Computer, Inc. 1997"
};
include "Defaults.rsrc"  'vers' (2);		// Overall package

//--------------------------------------------------------------------------------------------------
// Views 
//--------------------------------------------------------------------------------------------------
#ifndef MW_Rez	
	// This Adlib file is included in IconEdit.prj for MW Builds
	include "SimpleSizer.view"	
#endif
