//----------------------------------------------------------------------------------------
// CODAdaptorMgr_AC.h
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODAdaptorMgr_AC__
#define __CODAdaptorMgr_AC__

// ALOE

#ifndef __ALOE_AC__
#include "ALOE_AC.h"
#endif

// Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif


//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CODAdaptor_AC;
class CODAdaptorView_AC;

template <class T, class ALLOCATOR> class list;

typedef list<CODAdaptor_AC*, allocator<CODAdaptor_AC*> > CODAdaptorList_AC;

//----------------------------------------------------------------------------------------
// CODAdaptorMgr_AC represents the requirements for ALOE Adaptor Manager
//----------------------------------------------------------------------------------------

class EXPORT_AC CODAdaptorMgr_AC {

public:
	CODAdaptorMgr_AC();
	// Construct adaptor mgr
	
	CODAdaptorMgr_AC(long dataRefNum);
	// Construct adaptor mgr from file
	
	virtual ~CODAdaptorMgr_AC();

	// --------------------------------------------------------------------------------
	// Factory methods for creating adaptors
	// --------------------------------------------------------------------------------

	virtual CODAdaptor_AC* NewAdaptor(
						WindowPtr		window,
						ALOEISOString	kind,
						RgnHandle		shape,
						Ptr				dataPtr = NULL,
						long			dataSize = 0);
	// Create an adaptor
	
	inline CODAdaptor_AC* NewAdaptor(
						WindowPtr		window,
						ResType			type,
						RgnHandle		shape,
						Ptr				dataPtr = NULL,
						long			dataSize = 0)
		{ return NewAdaptor(window, ::ALOEGetKindFromResType(type), shape, dataPtr, dataSize); }
	// Create an adaptor with automatic conversion of ResType 
	
	virtual CODAdaptor_AC* NewAdaptorFromStorage(ALOEPersistentID id, WindowPtr window);
	// Create an adaptor from OD storage	
	
	virtual CODAdaptor_AC* NewAdaptorFromDrag(
						WindowPtr		window,
						DragReference 	theDragRef, 
						long			index);
	// Create an adaptor from a drag drop session
	
	// --------------------------------------------------------------------------------
	// Storage
	// --------------------------------------------------------------------------------

	bool HasChanged(bool& isSameSize);
	// return whether adaptors have changes since last save, and whether storage size changed

	inline bool HasChanged()
		{ bool isSameSize; return HasChanged(isSameSize); }		
	// return whether adaptors have changes since last save

	inline long Save(long dataRefNum)
		{ long bytesWritten; ThrowIfError_AC(::ALOEWriteAdaptorMgrToFile(GetALOEManager(), dataRefNum, &bytesWritten)); return bytesWritten; }
	// Save adaptor manager 
	
	// --------------------------------------------------------------------------------
	// Activation
	// --------------------------------------------------------------------------------

	virtual void DoActivateApp(bool activate);
	// Set this adaptor manager's activation state
	
	inline bool ApplicationHasFocus() 
		{ return fApplicationHasFocus; }
	// return this adaptor manager's activation state
	
	// --------------------------------------------------------------------------------
	// Selection
	// --------------------------------------------------------------------------------
	
	inline CODAdaptorView_AC* GetSelectedAdaptorView()
		{ return fSelectedAdaptorView; }
	
	void ToggleAdaptorViewSelectState(CODAdaptorView_AC* adaptorView);
	// toggle view's select state and draw
	
	inline bool HasSelection()
		{ return fSelectedAdaptorView != NULL; }
		
	void DrawSelection(bool isSelected);

	// --------------------------------------------------------------------------------
	// Accessors
	// --------------------------------------------------------------------------------
	
	inline ALOEAdaptorMgr GetALOEManager()
		{ return fManager; }

	virtual void AddAdaptor(CODAdaptor_AC* adaptor);
	// add adaptor to my list
	
	virtual void RemoveAdaptor(CODAdaptor_AC* adaptor);
	// remove adaptor from my list
		
protected:
	
	CODAdaptorList_AC* GetAdaptorList();
	// return adaptor list, initializing it if not created
	
	virtual void DeleteAdaptors();
	// delete list of adaptors
	
public:
	virtual CODAdaptor_AC* FindAdaptor(ALOEPersistentID id);
	// Find adaptor that has this ID, or NULL if none found
	
	virtual CODAdaptor_AC* FindAdaptor(ALOEAdaptor adaptor);
	// Find adaptor instance that wraps this adaptor, or NULL if none found
	
private:
	ALOEAdaptorMgr fManager;
	bool fApplicationHasFocus; // when we are frontmost, this determines whether the application is active
	CODAdaptorView_AC* fSelectedAdaptorView;
	CAutoPtr_AC<CODAdaptorList_AC> fAdaptors;
};


#endif // __CODAdaptorMgr_AC__
