//----------------------------------------------------------------------------------------
// CODAdaptorView_AC.h
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODAdaptorView_AC__
#define __CODAdaptorView_AC__

// ALOE

#ifndef __ALOE_AC__
#include "ALOE_AC.h"
#endif

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CODAdaptorView_AC wraps ALOE ALOEAdaptorView
//----------------------------------------------------------------------------------------

class EXPORT_AC CODAdaptorView_AC {
public:

	CODAdaptorView_AC(CODAdaptor_AC* adaptor, 
						GrafPtr			port, 
						Point			offset, 
						THPrint			printJob = NULL);
	// Construct an adaptor view from an adaptor
	
	virtual ~CODAdaptorView_AC();
	// Destructor
	
	// Inlined calls to ALOE to handle drawing, offset, scale factor, Part Info	
	
	inline Point GetOffset()
		{ Point offset; ThrowIfError_AC(::ALOEGetAdaptorViewOffset(GetALOEAdaptorView(), &offset)); return offset; }
	
	inline void SetOffset(Point& offset)
		{ ThrowIfError_AC(::ALOESetAdaptorViewOffset(GetALOEAdaptorView(), offset)); }
	
	inline void Draw(RgnHandle viewClip)
		{ ThrowIfError_AC(::ALOEDrawAdaptorView(GetALOEAdaptorView(), viewClip)); }
	
	inline void SetScaleFactor(Fixed horzScaleFactor, Fixed vertScaleFactor)
		{ ThrowIfError_AC(::ALOESetAdaptorViewScaleFactor(GetALOEAdaptorView(), horzScaleFactor, vertScaleFactor)); }

	inline PicHandle GetPicture()
		{ PicHandle image; ThrowIfError_AC(::ALOEGetAdaptorViewPicture(GetALOEAdaptorView(), &image)); return image; }
		
	// OpenDoc states
	
	inline bool GetSelectState()
		{ return ::ALOEGetAdaptorViewSelectState(GetALOEAdaptorView()); }

	inline void SetSelectState(bool isSelected)
		{ ThrowIfError_AC(::ALOESetAdaptorViewSelectState(GetALOEAdaptorView(), isSelected)); }

	inline ALOEHighlightState GetHighlightState()
		{ return ::ALOEGetAdaptorViewHighlightState(GetALOEAdaptorView()); }

	inline void SetHighlightState(ALOEHighlightState highlightState)
		{ ThrowIfError_AC(::ALOESetAdaptorViewHighlightState(GetALOEAdaptorView(), highlightState)); }
	
	// Drag and drop
	
	inline bool CanAcceptDrop()
		{ return ::ALOEAdaptorViewCanAcceptDrop(GetALOEAdaptorView()); }

	virtual void DoStartDrag(DragReference theDrag, WindowPtr window);
		// This implements a recipe for dragging a single adaptor.

	// Part Info dialog
	
	inline void ShowInfo()
		{ OSErr ignored = ::ALOEShowAdaptorViewInfo(GetALOEAdaptorView()); }
		// !!! This sometimes returns an error code, do we ever care about it?
		
	// accessors
	
	inline CODAdaptor_AC* GetAdaptor()
		{ return fAdaptor; }
		
	inline ALOEAdaptorView GetALOEAdaptorView()
		{ return fView; }
		
protected:
	CODAdaptor_AC* fAdaptor;
	ALOEAdaptorView fView;
};

#endif // __CODAdaptorView_AC__