//----------------------------------------------------------------------------------------
// CODAdaptor_AC.h
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODAdaptor_AC__
#define __CODAdaptor_AC__

// Core

#ifndef __CAssertionFailure_AC__
#include "CAssertionFailure_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

// ALOE

#ifndef __ALOE_AC__
#include "ALOE_AC.h"
#endif

#ifndef __ODUtils_AC__
#include "ODUtils_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CODAdaptorMgr_AC;
class CODAdaptorView_AC;

template <class T, class ALLOCATOR> class list;

typedef list<CODAdaptorView_AC*, allocator<CODAdaptorView_AC*> > CODViewList_AC;

//----------------------------------------------------------------------------------------
// ODAdaptor is an encapsulation of an OpenDoc part and a single OpenDoc frame.
//----------------------------------------------------------------------------------------

class EXPORT_AC CODAdaptor_AC {

public:
						
	CODAdaptor_AC(CODAdaptorMgr_AC* manager,
						WindowPtr		window,
						ALOEISOString	kind,
						RgnHandle		shape,
						Ptr				dataPtr = NULL,
						long			dataSize = 0);
		// Construct from an ISOString.
		
	CODAdaptor_AC(CODAdaptorMgr_AC* manager, 
						ALOEPersistentID id,
						WindowPtr		window);
		// Construct from OpenDoc storage.

	CODAdaptor_AC(CODAdaptorMgr_AC* manager,
						 WindowPtr		window,
						 DragReference 	theDragRef, 
						 long			index);
		// Construct from a drag drop session.
	
	CODAdaptor_AC(const CODAdaptor_AC& fromAdaptor, WindowPtr window, bool isSynchronized = false);
		// Construct from another adaptor.  The isSynchronized parameter indicates whether the
		// new adaptor should really a duplicate of the original, or whether it should
		// be the same OpenDoc part with a different OpenDoc frame.
	
	// The default copy constructor and assignment are not implemented.

private:

	CODAdaptor_AC(const CODAdaptor_AC& adaptor);	
	CODAdaptor_AC& operator = (const CODAdaptor_AC& that);

public:

	virtual ~CODAdaptor_AC();
		// destructor
		
	// --------------------------------------------------------------------------------
	// Events
	// --------------------------------------------------------------------------------

	inline void SetDoesPropagateEvents(bool doesPropagate)
		{ ThrowIfError_AC(::ALOESetDoesPropagateEvents(GetALOEAdaptor(), doesPropagate)); }
	
	inline void Activate()
		{ ThrowIfError_AC(::ALOEActivateAdaptor(GetALOEAdaptor())); }

	// --------------------------------------------------------------------------------
	// Layout and state
	// --------------------------------------------------------------------------------
			
	inline RgnHandle GetShape()
		{ RgnHandle adaptorShape = ::NewRgn(); GetShape(adaptorShape); return adaptorShape; }

	inline void GetShape(RgnHandle adaptorShape)
		{ AssertNotNULL_AC(adaptorShape); ThrowIfError_AC(::ALOEGetAdaptorShape(GetALOEAdaptor(), adaptorShape)); }
		
	inline void SetShape(RgnHandle adaptorShape)
		{ ThrowIfError_AC(::ALOESetAdaptorShape(GetALOEAdaptor(), adaptorShape)); }

	inline void SetRemovedState(bool removed)
		{ ThrowIfError_AC(::ALOESetAdaptorRemovedState(GetALOEAdaptor(), removed)); }

	inline bool GetBundleState()
		{ return ::ALOEGetAdaptorBundleState(GetALOEAdaptor()); }

	inline void SetBundleState(bool bundled)
		{ ThrowIfError_AC(::ALOESetAdaptorBundleState(GetALOEAdaptor(), bundled)); }
	
	
	// --------------------------------------------------------------------------------
	// This is the factory function for creating an adaptor view.
	// --------------------------------------------------------------------------------

	virtual CODAdaptorView_AC* NewAdaptorView(GrafPtr port, Point offset, THPrint printJob = NULL);
	// Create an adaptor view.  If for printing, the print job is attached.
		
	// --------------------------------------------------------------------------------
	// Callbacks from ALOE
	// --------------------------------------------------------------------------------
	
	virtual void DoAdaptorShapeRequest(RgnHandle theRequestedRegion);
	// Adaptor shape request callback: how does it work?  Magic!
	
	// --------------------------------------------------------------------------------
	// Accessors
	// --------------------------------------------------------------------------------

	inline ALOEAdaptor GetALOEAdaptor() const
		{ return fAdaptor; }
		
	inline CODAdaptorMgr_AC* GetManager() const
		{ return fManager; }

	inline void GetPersistentID(ALOEPersistentID id)
		{ ThrowIfError_AC(::ALOEGetPersistentID(GetALOEAdaptor(), id)); }
	
	inline bool IsSamePersistentID(const ALOEPersistentID id)
		{ ALOEPersistentID myID; GetPersistentID(myID); return CompareALOEPersistentID(id, myID); }
	
	inline bool IsSameAdaptor(ALOEAdaptor adaptor)
		{ return adaptor == fAdaptor; } // ??? will this be guaranteed to work?  Or should we compare persistent IDs?

	virtual void AddAdaptorView(CODAdaptorView_AC* adaptorView);
	// add adaptor view to my list
	
	virtual void RemoveAdaptorView(CODAdaptorView_AC* adaptorView);
	// remove adaptor view from my list
		
protected:		
	CODViewList_AC* GetViewList();
	
	virtual void DeleteAdaptorViews();
	
private:
	CODAdaptorMgr_AC* fManager;
	ALOEAdaptor fAdaptor;
	//CAutoPtr_AC<CODViewList_AC> fViews;
};


#endif  //  __CODAdaptor_AC__
