//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC.h
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODClipboardPromise_AC__
#define __CODClipboardPromise_AC__

// ALOE

#ifndef __ALOE_AC__
#include "ALOE_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC provides a facility for promising data to the scrap using a callback
// It calls ALOE directly instead of using SClipboard so that the dependency on ALOE is
// explicit and required at compile and link time.  However, SClipboard must be compiled
// for ALOE support turned on for clipboard support to behave properly.
//----------------------------------------------------------------------------------------

class EXPORT_AC CODClipboardPromise_AC {

public:
	CODClipboardPromise_AC(ResType theType);
	// Installs the WriteScrap callback and promises the data type to the scrap.
		
	virtual void GetPromisedScrapData(long* length, Ptr* source) = 0;
	// You must create a subclass that overrides this function to provide the data
	// when ALOE needs it.

	static inline CODClipboardPromise_AC* Instance()
		{ return fgCurrentPromise; }
	// There can only be one clipboard promise at a time.
	
	virtual void DoWriteScrap(ResType theType);
	// The default implementation calls GetPromisedScrapData followed by PutScrap.
	// You may want to create a subclass that overrides this function.

protected:

	virtual ~CODClipboardPromise_AC();
	// This removes the callback. It is protected because it is normally invoked
	// automatically by DisposeOnWriteScrap.

	void SetWriteScrapCallback();
	// This installs the WriteScrap callback.

	void DisposeWriteScrapCallback();
	// This removes the WriteScrap callback.
			
	virtual void DisposeOnWriteScrap();
	// This is called automatically by DoWriteScrap to delete the clipboard promise.
		
private:
	ALOEWriteScrapUPP fWriteScrapUPP;
	static CODClipboardPromise_AC* fgCurrentPromise;
};

#endif