//----------------------------------------------------------------------------------------
// CODWindow_AC.h
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODWindow_AC__
#define __CODWindow_AC__

// ALOE

#ifndef __ALOE_AC__
#include "ALOE_AC.h"
#endif

// Core

#ifndef __CAssertionFailure_AC__
#include "CAssertionFailure_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CODWindow_AC is a lightweight encapsulation of utilities for ALOE windows
//----------------------------------------------------------------------------------------

class EXPORT_AC CODWindow_AC {
public:
	CODWindow_AC(WindowPtr window) :
		fWindow(window)
		{ Assertion_AC(window != NULL, NULL); }
		// It is a programmer error to use an CODWindow_AC with a NULL window reference

	static bool HandleEvent(EventRecord* event);
		// See if we have an event for an ALOE window, and handle what ALOE doesn't
		// Called automatically by CODSession_AC
		
	static inline bool IsODWindow(WindowPtr window)
		{ return ::IsALOEWindow(window); }
	
	static inline bool IsFloatingWindow(WindowPtr window)
		{ return ::IsALOEFloatingWindow(window); }
		// Call if you implement floating window management
	
	static inline void Deactivate(WindowPtr window)
		{ ThrowIfError_AC(::ALOEDeactivateWindow(window)); }
		// Call if you implement floating window management

	static void SetSize(WindowPtr window, Point newSize, bool update)
		{ ::ALOESizeWindow(window, newSize.h, newSize.v, update); }
		// Equivalent to SizeWindow, but !!! important !!! updates OpenDoc's 
		// cache of the window's clippable extent.  Call instead of SizeWindow.
		
	static WindowPtr GetFrontNonFloatingWindow();
		// Return frontmost non-floating window
			
public: 

	inline void Select() const
		{ ThrowIfError_AC(::ALOESelectWindow(fWindow)); }
		// This should be sufficient even for apps that have window callbacks
		// since their SelectWindow callback will get called by ALOE

	virtual void MoveByUser(Point theMouse) const;
		// call ALOEDragWindow by default

private:
	WindowPtr fWindow;
};

#endif // __CODWindow_AC__