//----------------------------------------------------------------------------------------
// CODAdaptorView_AC.h
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODAdaptorView_AC__
#include "CODAdaptorView_AC.h"
#endif

#ifndef __CODAdaptor_AC__
#include "CODAdaptor_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Construct an adaptor view from an adaptor
//----------------------------------------------------------------------------------------

CODAdaptorView_AC::CODAdaptorView_AC(CODAdaptor_AC* adaptor, 
						GrafPtr			port, 
						Point			offset,
						THPrint			printJob) :
		fAdaptor(adaptor),
		fView(NULL)
{
	fAdaptor->AddAdaptorView(this);
	ALOEAdaptorView view;
	ThrowIfError_AC(::ALOENewAdaptorView(adaptor->GetALOEAdaptor(), port, offset, printJob, &view));
	fView = view;
}


//----------------------------------------------------------------------------------------
// CODAdaptorView_AC wraps ALOEAdaptorView
//----------------------------------------------------------------------------------------

CODAdaptorView_AC::~CODAdaptorView_AC()
{
	fAdaptor->RemoveAdaptorView(this);
	if (fView != NULL)
	{
		ThrowIfError_AC(::ALOEDisposeAdaptorView(fView));
	}
}

//----------------------------------------------------------------------------------------
// Start a drag
//----------------------------------------------------------------------------------------

void CODAdaptorView_AC::DoStartDrag(DragReference theDrag, WindowPtr window)
{
	try {
		ALOEClonedID theID;
		ThrowIfError_AC(::ALOEBeginDragClone(GetAdaptor()->GetManager()->GetALOEManager(), window, theDrag, kALOEDrag));
		Point offset = GetOffset();
		ThrowIfError_AC(::ALOECloneAdaptorFromContent(GetAdaptor()->GetALOEAdaptor(), offset, theID));
		ThrowIfError_AC(::ALOEEndClone());
	}
	catch (CException_AC& exception)
	{
		OSErr err  =::ALOEAbortClone();
		// !!! We don't want to re-throw an exception during a drag session, so we report an abort error directly
#if qDebug
		if (err != noErr)
			::DebugStr("\pALOE encountered an error aborting the clone while handling an exception.");
#endif
	}
}