//----------------------------------------------------------------------------------------
// CODAdaptor_AC.cp
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODAdaptor_AC__
#include "CODAdaptor_AC.h"
#endif

// ALOE Sprockets

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

#ifndef __CODAdaptorView_AC__
#include "CODAdaptorView_AC.h"
#endif

#ifndef __CODSession_AC__
#include "CODSession_AC.h"
#endif

// STL

#ifndef LIST_H
#include <list.h>
#endif

//----------------------------------------------------------------------------------------
// constructor
//----------------------------------------------------------------------------------------

CODAdaptor_AC::CODAdaptor_AC(	CODAdaptorMgr_AC* manager,
						WindowPtr		window,
						ALOEISOString	kind,
						RgnHandle		shape,
						Ptr				dataPtr,
						long			dataSize) :
		fManager(manager),
		fAdaptor(NULL)
{
	fManager->AddAdaptor(this);
	ALOEAdaptor adaptor;
	ThrowIfError_AC(::ALOENewAdaptor(fManager->GetALOEManager(), window, kind, dataPtr, dataSize, shape, &adaptor));
	fAdaptor = adaptor;
}
	
//----------------------------------------------------------------------------------------
// Constructor from storage
//----------------------------------------------------------------------------------------

CODAdaptor_AC::CODAdaptor_AC(CODAdaptorMgr_AC* manager, ALOEPersistentID id, WindowPtr window) :
		fManager(manager),
		fAdaptor(NULL)
{
	fManager->AddAdaptor(this);
	ALOEAdaptor adaptor;
	ThrowIfError_AC(::ALOEGetAdaptor(fManager->GetALOEManager(), id, window, &adaptor));
	fAdaptor = adaptor;
}

//----------------------------------------------------------------------------------------
// Constructor from drag
//----------------------------------------------------------------------------------------

CODAdaptor_AC::CODAdaptor_AC(CODAdaptorMgr_AC* manager,
						 WindowPtr		window,
						 DragReference 	theDragRef, 
						 long			index) :
		fManager(manager),
		fAdaptor(NULL)
{
	fManager->AddAdaptor(this);
	ALOEAdaptor adaptor;
	ThrowIfError_AC(::ALOEEmbedDrag(fManager->GetALOEManager(), window, theDragRef, index, &adaptor));
	fAdaptor = adaptor;
}	

//----------------------------------------------------------------------------------------
// Copy constructor
//----------------------------------------------------------------------------------------

CODAdaptor_AC::CODAdaptor_AC(const CODAdaptor_AC& fromAdaptor, WindowPtr window, bool isSynchronized) :
	fManager(fromAdaptor.GetManager()),
	fAdaptor(NULL)
{
	fManager->AddAdaptor(this);
	ALOEAdaptor adaptor;
	if (isSynchronized)
		ThrowIfError_AC(::ALOENewSynchronizedAdaptor(fromAdaptor.GetALOEAdaptor(), window, &adaptor));
	else
		ThrowIfError_AC(::ALOEDuplicateAdaptor(fromAdaptor.GetALOEAdaptor(), &adaptor));
	fAdaptor = adaptor;
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CODAdaptor_AC::~CODAdaptor_AC()
{
	fManager->RemoveAdaptor(this);
	DeleteAdaptorViews();
	if (fAdaptor != NULL)
	{	
		ThrowIfError_AC(::ALOEDisposeAdaptor(fAdaptor));
	}
}


//----------------------------------------------------------------------------------------
// Factory: given a window and offset, instantiate and show
//----------------------------------------------------------------------------------------

CODAdaptorView_AC* CODAdaptor_AC::NewAdaptorView(GrafPtr port, Point offset, THPrint printJob)
{
	return TH_new CODAdaptorView_AC(this, port, offset, printJob);
}
	
//----------------------------------------------------------------------------------------
// Adaptor has a shape request, accept it or give it a preferred shape
//----------------------------------------------------------------------------------------

void CODAdaptor_AC::DoAdaptorShapeRequest(RgnHandle theRequestedRegion)
{
	CODSession_AC::Instance()->DefaultAdaptorShapeRequest(theRequestedRegion);
}

//----------------------------------------------------------------------------------------
// add adaptor view to my list
//----------------------------------------------------------------------------------------

void CODAdaptor_AC::AddAdaptorView(CODAdaptorView_AC* /*adaptorView*/)
{
	//GetViewList()->push_front(adaptorView);
}

//----------------------------------------------------------------------------------------
// remove adaptor view from my list
//----------------------------------------------------------------------------------------

void CODAdaptor_AC::RemoveAdaptorView(CODAdaptorView_AC* /*adaptorView*/)
{
	//GetViewList()->remove(adaptorView);
}

//----------------------------------------------------------------------------------------
// return view list, creating it if needed
//----------------------------------------------------------------------------------------

CODViewList_AC* CODAdaptor_AC::GetViewList()
{
	//if (!fViews)
	//{
	//	fViews = new CODViewList_AC;
	//}
	//return fViews;
	return NULL;
}

//----------------------------------------------------------------------------------------
// delete list of adaptor views
//----------------------------------------------------------------------------------------

void CODAdaptor_AC::DeleteAdaptorViews()
{
	// !!! goes kablooie, doesn't work yet
	//for (CODViewList_AC::iterator iter = GetViewList()->begin(); iter != GetViewList()->end(); ++iter)
	//{	
	//	delete *iter;
	//}
}
