//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC.cp
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODClipboardPromise_AC__
#include "CODClipboardPromise_AC.h"
#endif

//----------------------------------------------------------------------------------------
// There can only be one clipboard promise installed at a time with ALOE.
//----------------------------------------------------------------------------------------

CODClipboardPromise_AC* CODClipboardPromise_AC::fgCurrentPromise = NULL;

//----------------------------------------------------------------------------------------
// The following callback function is installed with ALOE that
// in turn calls a virtual function of the current ClipboardPromise.
//----------------------------------------------------------------------------------------

static OSErr pascal WriteScrapProc(ResType theType, void* /*contextPtr*/)
{
	try 
	{
		Assertion_AC(CODClipboardPromise_AC::Instance() != NULL, NULL);
		CODClipboardPromise_AC::Instance()->DoWriteScrap(theType);
	}
	catch (CException_AC& exception)
	{
		return exception.GetMessage();
	}
	
	return noErr;
}

//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC constructor
//----------------------------------------------------------------------------------------

CODClipboardPromise_AC::CODClipboardPromise_AC(ResType theType) :
	fWriteScrapUPP(NULL)
{
	Assertion_AC(fgCurrentPromise == NULL, NULL);	
	SetWriteScrapCallback();	
	ThrowIfError_AC(::ALOEPutScrap(0, theType, NULL));
	fgCurrentPromise = this;
}

//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC destructor
//----------------------------------------------------------------------------------------

CODClipboardPromise_AC::~CODClipboardPromise_AC()
{
	// Un-install the callback, dispose of it, and reset current promise
	ThrowIfError_AC(::ALOESetWriteScrapCallback(NULL, NULL));
	DisposeWriteScrapCallback();	
	fgCurrentPromise = NULL;
}

//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC SetWriteScrapCallback: This installs the WriteScrap callback.
//----------------------------------------------------------------------------------------

void CODClipboardPromise_AC::SetWriteScrapCallback()
{
	fWriteScrapUPP = NewALOEWriteScrapProc(&WriteScrapProc);
	ThrowIfNULL_AC(fWriteScrapUPP);
	ThrowIfError_AC(::ALOESetWriteScrapCallback(fWriteScrapUPP, NULL));
}

//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC DisposeWriteScrapCallback: This removes the WriteScrap callback.
//----------------------------------------------------------------------------------------

void CODClipboardPromise_AC::DisposeWriteScrapCallback()
{
	if (fWriteScrapUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fWriteScrapUPP);
		fWriteScrapUPP = NULL;
	}
}	
		
//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC DoWriteScrap:
// The default implementation calls GetPromisedScrapData followed by PutScrap.
// You may want to create a subclass that overrides this function.
//----------------------------------------------------------------------------------------

void CODClipboardPromise_AC::DoWriteScrap(ResType theType)
{
	long length;
	Ptr source;
	try 
	{
		GetPromisedScrapData(&length, &source);
		Assertion_AC(source != NULL, NULL);
		ThrowIfError_AC(::ALOEPutScrap(length, theType, source));
		DisposeOnWriteScrap();
	}
	catch (CException_AC& exception)
	{
		DisposeOnWriteScrap();
		throw;
	}

}

//----------------------------------------------------------------------------------------
// CODClipboardPromise_AC DisposeOnWriteScrap: Delete self when WriteScrap is called.
//----------------------------------------------------------------------------------------

void CODClipboardPromise_AC::DisposeOnWriteScrap()
{
	delete this;
}
