// AGAImaging_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1996 by Trygve Isaacson.

#pragma once

#ifndef __AGAImaging_AC__
#define __AGAImaging_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

// ACS

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Global functions and variables
//----------------------------------------------------------------------------------------

// AGAStringOut is used by the AGA classes to draw their text.
#define kNoTruncation -1	// Toolbox defines smTruncEnd, smTruncMiddle, etc.
enum AGAStringOutColor { kNormalOutput, kDisabledOutput, kInverseOutput, kUseCurrentColor };
EXPORT_AC void AGAStringOut(const CStr255_AC& aString,
						const CRect_AC& centeringFrame,
						SInt32 truncation,
						SInt32 justification,
						AGAStringOutColor outColor,
						bool deep,
						const CTextStyle_AC& textStyle);

// AGATextBox is a replacement for TETextBox, with the same initial parameters, plus
// a couple of extra ones for AGA control over the text. This is used by CStaticText_AC.
EXPORT_AC void AGATextBox(const void *textPtr,
						SInt32 textLength,
						const CRect_AC& textFrame,
						SInt16 justification,
						AGAStringOutColor outColor,
						bool deep,
						const CTextStyle_AC& textStyle,
						const CRGBColor_AC& teBackgroundColor,
						bool eraseFirst);

// AGABackgroundPaint is used to paint window backgrounds.
enum AGABackgroundKind { kRaisedModelessBackground, kRaisedModalBackground, kFlatWindowBackground, kNoBackgroundPaint };
EXPORT_AC void AGABackgroundPaint(const CRect_AC& backgroundBounds,
								bool drawFill,
								AGABackgroundKind backgroundKind,
								bool isActive,
								bool hasGrowBox);

// AGADrawSeparator is used to draw separator lines.
enum AGASeparatorLayout { kSeparatorAutomatic, kSeparatorLeft, kSeparatorTop, kSeparatorRight, kSeparatorBottom };
EXPORT_AC void AGADrawSeparator(const CRect_AC& bounds, bool isEnabled, const CRGBColor_AC& windowBackgroundColor, AGASeparatorLayout layout = kSeparatorAutomatic);

// AGADrawBevelButton is used to draw an empty bevel button, such as an icon button.
enum ButtonFrameType { kAutoFrame, kLargeFrame, kSmallFrame, kMiniFrame };
EXPORT_AC ButtonFrameType AGAGetDefaultFrameType(SInt16 buttonWidth);
EXPORT_AC void AGADrawBevelButton(const CRect_AC& bounds, ButtonFrameType frameType, bool isOn, bool isPressed, bool isEnabled);
EXPORT_AC void AGADrawBevelButtonAtDepth(bool deep, const CRect_AC& bounds, ButtonFrameType frameType, bool isOn, bool isPressed, bool isEnabled);

// AGAInstallGrayWCTB is used to install gray into the window's 'wctb' background.				
EXPORT_AC OSErr AGAInstallGrayWCTB(WindowPtr aWindow);
EXPORT_AC OSErr AGAInstallWCTB(WindowPtr aWindow, const CRGBColor_AC& newColor, bool* hasOldColor, CRGBColor_AC& oldColor);

// AGADrawBorderFrame is used to draw the "3D" sunken frame around an object.
// It supports correct dimming and "notched" shape (contoured to scroll bars).
EXPORT_AC void AGADrawBorderFrame(const CRect_AC& bounds, bool isEnabled, bool isNotched, bool drawFrame, const CRGBColor_AC& windowBackgroundColor);

// AGAComputeTargetBorderRegion is used to build a region that defines the
// target border, aka "focus" frame.
// It supports "notched" shape (contoured to scroll bars).
EXPORT_AC void AGAComputeTargetBorderRegion(RgnHandle borderRegion, const CRect_AC& bounds, bool isNotched);


// These inlines replace PenPat(xxx) where x is the qd.gray or qd.black global, so
// that a ConstPatternParam is used instead of a global. This makes it possible to
// use the code from a code resource.
inline void PenPatBlack() { ::PenPat((ConstPatternParam)"\xff\xff\xff\xff\xff\xff\xff\xff"); }
inline void PenPatGray() { ::PenPat((ConstPatternParam)"\xAA\x55\xAA\x55\xAA\x55\xAA\x55"); }

#endif // __AGAImaging_AC__
