// AGA_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1996 by Trygve Isaacson.

#ifndef __AGA_AC__
#define __AGA_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

// ACS

#ifndef __Configuration_AC__
#include "Configuration_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

//----------------------------------------------------------------------------------------
// Utility functions for working with the Apple Grayscale Appearance
//----------------------------------------------------------------------------------------

EXPORT_AC void InitAGA_AC(bool appearanceMgrSavvy = true, bool aaronOrKaleidoscopeSavvy = true);
// Initialize AGA globals and defaults.
// If either parameter is true and the corresponding extension is installed,
// the Appearance classes will allow their controls to be drawn by the system.
// This routine is called automatically if you call the framework-specific initializer.

EXPORT_AC bool HasAppearanceMgr_AC();
// Return whether appearance manager is installed.

EXPORT_AC bool EmulateControls_AC();
// Return whether to emulate controls (false means use system controls)

EXPORT_AC void SetEmulateControls_AC(bool emulate);
// Set whether emulating controls (only works for creating new controls after this call) 



// Turn default appearance manager behavioral attributes off or on with these calls.
// These affect what default attributes are used for certain objects
// at the time they are instantiated, in the case where the attribute
// is not explicitly set as a constructor parameter or adapter class
// function.

// These are the attributes you can turn off or on with SetAGAFeatureFlag.
// They only affect new objects, not existing ones.
#define kAGALiveScrolling_AC			0x00000001
#define kAGAProportionalScrolling_AC	0x00000002
#define kAGALiveRSliders_AC				0x00000004
#define kAGAProportionalRSliders_AC		0x00000008
#define kAGALivePSliders_AC				0x00000010
#define kAGAProportionalPSliders_AC		0x00000020

#define kAGALiveScrolling_AC			0x00000001	// causes scroll bars to init with live scrolling ON
#define kAGAProportionalScrolling_AC	0x00000002	// causes scroll bars to init with proportional thumb ON
#define kAGALiveRSliders_AC				0x00000004	// causes rectangle sliders to init with live tracking ON
#define kAGAProportionalRSliders_AC		0x00000008	// causes rectangle sliders to init with proportional thumb ON
#define kAGALivePSliders_AC				0x00000010	// causes pointy sliders to init with live tracking ON
#define kAGAProportionalPSliders_AC		0x00000020	// causes pointy sliders to init with proportional thumb ON
#define kAGAInverseHSliderValues_AC		0x00000040	// causes horizontal sliders to init as min=right max=left
#define kAGAInverseVSliderValues_AC		0x00000080	// causes vertical sliders to init as min=bottom max=top
#define kAGADimDeactivation_AC			0x00000100	// causes adapters to dim AGAObject when deactivated (PowerPlant-only for now)

EXPORT_AC void SetAGAFeatureFlag_AC(UInt32 itemMask, bool turnOn);
// Set AGA feature flag

EXPORT_AC bool GetAGAFeatureFlag_AC(UInt32 itemMask);
// Get AGA feature flag

EXPORT_AC void DelayFuture_AC(SInt32 delayTicks);
// Utility: mouse testing

EXPORT_AC void DelayFutureWhileStillDown_AC(SInt32 delayTicks);
// Utility: mouse testing

#endif // __AGA_AC__