// CCluster_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CCluster_AC__
#define __CCluster_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CCluster_AC
// CCluster_AC implements the standard AGA group box. A group box can
// either be primary or secondary. Secondary group boxes should be used
// inside primary group boxes, sez the AGA spec. You can supply a title
// string for the group box, or you can specify a "gap" width that is to
// be left blank to make room for a control or title string that will
// be drawn elsewhere.
//----------------------------------------------------------------------------------------

class EXPORT_AC CCluster_AC : public CControl_AC
	{
	public:

		enum { kPrimaryGroupBox = true, kSecondaryGroupBox = false };

		CCluster_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt32 justification, bool groupBoxType, const CStr255_AC& titleString = kEmptyString_AC, SInt16 titleGap = 0);
		CCluster_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt32 justification, bool groupBoxType, SInt16 titleStringListResourceID, SInt16 titleStringIndex, SInt16 titleGap = 0);
		virtual ~CCluster_AC();
		
		virtual void	SetTitleGap(SInt16 titleGap);
		virtual void	SetTitle(const CStr255_AC& newTitle, bool redraw);
		
		virtual void	DrawObject();

	protected:

		virtual void	DrawPrimaryFrame(SInt16 topIndent, SInt16 gapSize);
		virtual void	DrawSecondaryFrame(SInt16 topIndent, SInt16 gapSize);
		virtual SInt16	GetGapLeftInset(SInt16 gapSize);

		bool			fIsPrimaryBox;
		SInt16			fTitleGap;	// horizontal gap for other object in title, no top indent
		SInt32			fJustification;
		CStr255_AC		fTitle;		// zero-length string indicates no title, no top indent
		CTextStyle_AC	fTextStyle;
	};


#endif // __CCluster_AC__

