// CControl_AC.h
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1996 by Trygve Isaacson.

#ifndef __CControl_AC__
#define __CControl_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

// ACS

#ifndef __CRGBColor_AC__
#include "CRGBColor_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CControl_AC
{
public:
	CControl_AC(const CRect_AC& bounds);
	virtual ~CControl_AC();
	
	virtual CRect_AC	GetBounds() const;
	virtual void		SetBounds(const CRect_AC& bounds, bool redraw);

	virtual void		DrawObject();

	virtual bool		ContainsMouse(const CPoint_AC& mouseLocation);
	virtual bool		TrackMouse(const CPoint_AC& mouseLocation);

	virtual bool		IsEnabled() const;
	virtual void		SetEnable(bool isEnabled, bool redraw);
	virtual bool 		IsActive() const;
	virtual void		SetActive(bool isActive, bool redraw);

	virtual SInt32		GetValue() const;
	virtual void		SetValue(SInt32 newValue, bool redraw);

	virtual SInt32		GetMinimum() const;
	virtual SInt32		GetMaximum() const;
	virtual void		SetRange(SInt32 newMinimum, SInt32 newMaximum, bool redraw);

	virtual CStr255_AC	GetTitle() const;
	virtual void		SetTitle(const CStr255_AC& itsTitle, bool redraw);
	
	
	virtual void		SetBackgroundColors(const CRGBColor_AC& enabledColor, const CRGBColor_AC& disabledColor);
	
	enum { kRedraw = true, kDontRedraw = false };
	enum { kEnabled = true, kDisabled = false };
	enum { kActive = true, kInactive = false };
	enum { kPressed = true, kNotPressed = false };
	enum { kFrameInside = true, kFrameOutside = false };
	enum { kIsDefault = true, kIsNotDefault = false };
	enum { kAutomaticState = true, kNoAutomaticState = false };
	enum { kLiveTracking = true, kNoLiveTracking = false };
	enum { kProportional = true, kNotProportional = false };
	enum { kHorizontal = true, kVertical = false };
	enum { kTrackingIn = true, kTrackingOut = false };
	enum { kInstallOnlyIfCustom = false, kInstallAlways = true };

	void InstallBackgroundColors(bool installIfNotCustom);
	static bool DetermineBackgroundColors(CRGBColor_AC& enabledColor, CRGBColor_AC& disabledColor, bool installIfNotCustom);
	static void TurnOnCustomBackgroundColors(const CRGBColor_AC& enabledColor, const CRGBColor_AC& disabledColor);
	static void TurnOffCustomBackgroundColors();

protected:

	// HitTest and SetTrackingState are called repeatedly by the default
	// mouse tracker.
	virtual bool	HitTest(const CPoint_AC& mouseLocation);
	virtual void	SetTrackingState(bool isIn);
	virtual void	ApplyBackgroundColor();

	inline SInt16 MinSInt16(SInt16 a, SInt16 b) { return (a < b) ? a : b; }
	inline SInt16 MaxSInt16(SInt16 a, SInt16 b) { return (a > b) ? a : b; }
	inline SInt32 MinSInt32(SInt32 a, SInt32 b) { return (a < b) ? a : b; }
	inline SInt32 MaxSInt32(SInt32 a, SInt32 b) { return (a > b) ? a : b; }
	
	CRect_AC		fBounds;
	bool			fEnabled;	// if false, ignores mouse clicks etc.
	bool			fActive;	// if false, draws with dim appearance
	CRGBColor_AC	fEnabledBackgroundColor;
	CRGBColor_AC	fDisabledBackgroundColor;
	
	static bool			fgUseSpecialBackgroundColors;
	static CRGBColor_AC fgEnabledBackgroundColor;
	static CRGBColor_AC fgDisabledBackgroundColor;
};


#endif
