// CDisclosureTriangle_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CDisclosureTriangle_AC__
#define __CDisclosureTriangle_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC
// CDisclosureTriangle_AC implements the standard AGA disclosure triangle.
// You can think of it as a check box that is either off (closed) or on
// (disclosed).
//----------------------------------------------------------------------------------------

class EXPORT_AC CDisclosureTriangle_AC : public CControl_AC
{
public:

	enum { kDisclosedState = true, kClosedState = false };

	CDisclosureTriangle_AC(const CRect_AC& bounds, bool automaticState);
	virtual ~CDisclosureTriangle_AC();
	
	virtual void	DrawObject();
	virtual bool	TrackMouse(const CPoint_AC& mouseLocation);
	
	virtual bool	GetState();
	virtual void	SetState(bool isDisclosed, bool redraw);
	virtual void	SetStateAnimate(bool isDisclosed);

protected:

	virtual void	SetTrackingState(bool isIn);

	enum DTAnimationState { kDTClosed, kDTPressedClosed, kDTIntermediate, kDTPressedDisclosed, kDTDisclosed };

	virtual void	DrawTriangle(DTAnimationState state);
	virtual void	EraseBackground();

	bool		fAutomaticState;	// if true, triangle is toggled on click w/o caller needing to call SetValue
	bool		fIsDisclosed;
};

#endif // __CDisclosureTriangle_AC__

