// CGroupsContainer.h
//  Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//  Portions Copyright  1996 by Trygve Isaacson.

#ifndef __RadioGroup_AC__
#define __RadioGroup_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

//----------------------------------------------------------------------------------------
// MExclusiveObject_AC
// MExclusiveObject_AC is a mixin class for automatically keeping several
// objects in a group mutually exclusive. The radio button class below
// uses this to allow automatic radio button group behavior. MExclusiveObject_AC
// maintains knowledge of "groups" of objects. When one member of the group is
// "hit", the other members of the group are sent the TurnOff() message. To
// identify each object uniquely, the object has a pair of 32-bit identifiers.
// Typically these can set to the group's ID and GrafPtr. This allows having
// multiple windows that use the same group IDs. If you set the IDs to kNoGroupID,
// then this automatic maintenance is bypassed.
// If you are using the adaptor classes, this is not used.
//----------------------------------------------------------------------------------------


class EXPORT_AC MExclusiveObject_AC
{
public:

	MExclusiveObject_AC(UInt32 groupIDPart1, UInt32 groupIDPart2);
	virtual ~MExclusiveObject_AC();
	
	virtual void	SetGroupID(UInt32 groupIDPart1, UInt32 groupIDPart2);

	virtual void	TurnOff();

	UInt32	fGroupIDPart1;
	UInt32	fGroupIDPart2;
	
	bool NoGroupIDs() const { return (fGroupIDPart1 == kNoGroupID) && (fGroupIDPart2 == kNoGroupID); }
	bool ValidGroupIDs() const { return (fGroupIDPart1 != kNoGroupID) && (fGroupIDPart2 != kNoGroupID); }
	
	static const UInt32 kNoGroupID;
};



//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
// This class is used internally to maintain the MExclusiveObject_AC
// behavior that allows radio buttons to automatically maintain
// standard radio button group XOR behavior.
// It just maintains an array of AGAGroupMember structs, each of
// which identifies the group ID and contains a pointer to a
// group member.
// If you're using the adapter classes, this is not used.
//----------------------------------------------------------------------------------------

typedef struct
{
	UInt32						sGroupIDPart1;
	UInt32						sGroupIDPart2;
	MExclusiveObject_AC*		sGroupMember;
} SGroupMember_AC;

class EXPORT_AC CGroupsContainer_AC
{
public:

	CGroupsContainer_AC();
	virtual ~CGroupsContainer_AC();

	void	AddGroupMember(MExclusiveObject_AC* groupMember);
	void	RemoveGroupMember(MExclusiveObject_AC* groupMember);
	void	HitGroupMember(MExclusiveObject_AC* groupMember);

	static CGroupsContainer_AC*	Instance();
	
protected:

	void	AddGroupMember(MExclusiveObject_AC* groupMember, bool allowExpand);
	void	ExpandArray();

	enum { kGroupArrayChunkSize = 16 };

	UInt32				fGroupsArraySize;
	SGroupMember_AC**	fGroups;	// Handle to array of AGAGroupMember
	static CAutoPtr_AC<CGroupsContainer_AC> fgInstance;
};

#endif
