// CControl_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1997 by Trygve Isaacson.

#ifndef __CIndicator_AC__
#define __CIndicator_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif


// ACS

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CIndicator_AC
//
// CIndicator_AC is the abstract superclass for things like
// sliders and scroll bars that have a moving indicator to reflect
// the object's position. It has a minimum/maximum range, a current
// value, and optionally a page size for proportional indicator
// display. You can turn live tracking on or off; if live tracking
// is off, a "ghost" indicator is dragged and no notification occurs
// until the tracking completes.
//----------------------------------------------------------------------------------------

class CIndicator_AC;

// This is the live tracking notification prototype for CIndicator_AC.
typedef void (*CIndicatorCallback_AC)(CIndicator_AC* theIndicator, SInt32 itsNewValue, void* userData);

class EXPORT_AC CIndicator_AC : public CControl_AC
{
public:

	CIndicator_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, SInt32 pageSize = 1, bool liveTracking = true, bool proportional = false);

	virtual ~CIndicator_AC();
	
	virtual void	InstallNotificationRoutine(CIndicatorCallback_AC notificationRoutine, void* userData);
	
	virtual void	SetAttributes(bool isHorizontal, bool liveTracking, bool isProportional);
	virtual void	SetLiveTracking(bool liveTracking);

	virtual void	DrawObject();
	virtual bool	TrackMouse(const CPoint_AC& mouseLocation);
	
	virtual SInt32 GetValue() const;
	virtual void	SetValue(SInt32 newValue, bool redraw);

	virtual SInt32	GetMinimum() const;
	virtual SInt32	GetMaximum() const;
	virtual void	SetRange(SInt32 newMinimum, SInt32 newMaximum, bool redraw);
	
	virtual SInt32	GetPageSize();
	virtual void	SetPageSize(SInt32 newPageSize, bool redraw);

protected:

	enum { kNoTrackPress, kPageMinusPress, kArrowMinusPress, kIndicatorPress, kArrowPlusPress, kPagePlusPress };

	virtual bool	TrackPart(const CPoint_AC& mouseLocation, SInt32 partHit);
	virtual SInt32	TrackTestPart(const CPoint_AC& mouseLocation);
	virtual bool	IsValidIndicatorTrackMouse(const CPoint_AC& mouseLocation);
	virtual SInt32	GetValueFromMouseDelta(SInt32 originalValue, const CPoint_AC& originalMouseLocation, const CPoint_AC& newMouseLocation);
	virtual void	SetGhostValue(SInt32 newValue, bool redraw);

	virtual void	DrawBackground();
	virtual void	DrawIndicatorTrack();
	virtual void	DrawTrackEnds();
	virtual void	DrawIndicator();
	virtual void	RemoveIndicator(SInt32 newValue);
	virtual void	DrawGhost();
	virtual void	RemoveGhost(SInt32 newValue);
	
	virtual void	SetRawValue(SInt32 newValue, bool redraw);
	virtual void	NotifyValue();
	
	inline SInt32	GetGhostValue() { return fGhostValue; }
	inline bool		GetLiveTracking() { return fLiveTracking; }
	inline bool 	IsProportional() { return fIsProportional; }
	inline void 	SetIsProportional(bool isProportional) { fIsProportional = isProportional; }
	inline bool		IsHorizontal() { return fIsHorizontal; }
	inline bool		IsPressed() { return fIsPressed; }
	
	SInt32	fMinimum;
	SInt32	fMaximum;
	SInt32	fValue;
	SInt32	fPageSize;
	SInt32	fGhostValue;
	bool	fLiveTracking;
	bool	fIsProportional;
	bool	fIsHorizontal;
	bool	fIsPressed;
	
	CIndicatorCallback_AC	fNotificationRoutine;
	void*					fUserData;
};

#endif
