// CPopup_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CPopup_AC__
#define __CPopup_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

// ACS

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif

// Toolbox

#ifndef __MENUS__
#include <Menus.h>
#endif


//----------------------------------------------------------------------------------------
// CPopup_AC
// CPopup_AC implements the standard AGA pop-up menu. You can specify
// whether the supplied boundary is fixed or whether the pop-up button
// should adapt its width to the width of the associated menu. You can
// also supply a title to be drawn next to the pop-up button.
//----------------------------------------------------------------------------------------

class EXPORT_AC CPopup_AC : public CControl_AC
{
public:

	enum WidthAdjust { kFixedWidth = -1, kUseMenuWidth = 0, kSystemMDEFAdjustment = 19 };
	enum DisposalKind { kDontDispose, kDispose, kRelease };
	enum RefHandling { kNothing, kDetach, kDuplicate, kReload };

	CPopup_AC(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, MenuRef itsMenuRef = NULL, DisposalKind menuRefDisposalKind = kDontDispose, RefHandling refHandling = kNothing);
	CPopup_AC(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, SInt16 itemsStringListID, SInt16 newMenuID);
	CPopup_AC(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, ResType appendResourceType, SInt16 newMenuID);
	virtual ~CPopup_AC();
	
	virtual void	SetMenuRef(MenuRef itsMenuRef, DisposalKind menuRefDisposalKind, RefHandling refHandling);
	virtual void	SetWidthAdjustment(WidthAdjust adjustment);
	
	virtual void	DrawObject();
	virtual bool	TrackMouse(const CPoint_AC& mouseLocation);

	virtual SInt16 GetCurrentItemNo() const;
	virtual void	SetCurrentItemNo(SInt16 newItemNo, bool redraw);
	
	virtual void GetCurrentItemText(CStr255_AC& itemText) const;
	virtual bool	SetCurrentItemText(const CStr255_AC& itemTextToMatch, bool redraw);	// returns false if no match found
	virtual void	SetTitle(const CStr255_AC& newTitle, bool redraw);

	virtual void	DrawButton(bool pressed);

	inline MenuRef GetMenuRef() const { return fMenuRef; }
		// This returns fMenuRef.

protected:

	virtual void	DrawButtonNormal(bool deep);
	virtual void	DrawButtonPressed(bool deep);
	virtual void	DrawButtonDisabled(bool deep);
	virtual void	GetButtonBounds(CRect_AC& buttonBounds);
	virtual void	GetTitleBounds(CRect_AC& titleBounds);
	
	void			ReloadMenu();
		// Checks fMenuRefHandling and if kReload, calls LoadMenuRef to reload the menu.
		
	virtual MenuRef	LoadMenuRef(SInt16 menuID);
		// This calls ::GetMenu; called by ReloadMenu.
				
	virtual void	DisposeExistingMenuRef();
	
	enum { kPopupArrowSectionWidth = 22 };

	SInt16			fMenuID;
	MenuRef			fMenuRef;
	DisposalKind	fMenuRefDisposalKind;
	RefHandling		fMenuRefHandling;
	SInt16			fCurrentItemNo;
	WidthAdjust		fWidthAdjust;
	CTextStyle_AC	fTitleStyle;
	CTextStyle_AC	fTextStyle;
	CStr255_AC		fTitle;
	SInt16			fTitleOffset;
	SInt32			fTitleJustification;
};

//----------------------------------------------------------------------------------------
// CTempPopupMenuSizer_AC
// CTempPopupMenuSizer_AC is a stack-based object that will prepare the
// drawing environment for a custom popup menu font/size. This is a
// low-level toolbox hack capability.
// This is used to wrap calls to PopupMenuSelect and CalcMenuSize.
//----------------------------------------------------------------------------------------

class EXPORT_AC CTempPopupMenuSizer_AC
{
public:

	CTempPopupMenuSizer_AC(SInt16 fontNum, SInt16 fontSize);
	virtual ~CTempPopupMenuSizer_AC();
	
	void	SaveEnvironment();			// called by constructor
	void	RestoreEnvironment();		// called by destructor
	
	void	InstallCustomEnvironment();	// called by constructor after saving

	bool	fIsCustom;

protected:

	SInt16	fSavedFontNum;
	SInt16	fSavedFontSize;
	SInt16	fCustomFontNum;
	SInt16	fCustomFontSize;
};

#endif // __CPopup_AC__
