// CProgressIndicator_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CProgressIndicator_AC__
#define __CProgressIndicator_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

#ifndef __CSimpleOffscreenImage_AC__
#include "CSimpleOffscreenImage_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC
// CProgressIndicator_AC implements the standard AGA progress indicator.
// It can be determinate (a moving gauge) or indeterminate (a rotating
// barber pole). In addition, it supports a "moving origin", which is
// where, in addition to moving the gauge end point, the gauge start
// point also moves.
// This class emulates the Finder 7 progress indicator.
//----------------------------------------------------------------------------------------

class EXPORT_AC CProgressIndicator_AC : public CControl_AC
{
public:


	CProgressIndicator_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum);
	virtual ~CProgressIndicator_AC();
	
	virtual void	DrawObject();
	
	virtual SInt32 GetValue() const;
	virtual void	SetValue(SInt32 newValue, bool redraw);
	virtual void	Increment(SInt32 delta, bool redraw);
	
	virtual void	Animate();			// moves indeterminate indicator by 1
	virtual void	QuantizedAnimate();	// moves it only if kAnimationDelay has elapsed
	
	virtual SInt32	GetMinimum() const;
	virtual SInt32	GetMaximum() const;
	virtual void	SetRange(SInt32 newMinimum, SInt32 newMaximum, bool redraw);
	
	virtual SInt32	GetOriginValue();
	virtual void	SetOriginValue(SInt32 newValue, bool redraw);

protected:

	virtual void	DrawFrame();
	virtual void	DrawAnimationStep();
	virtual void	DrawAnimationPattern(const CRect_AC& clipRect);
	virtual void	DrawGauge();
	virtual void	DrawOriginArea();
	virtual void	DrawProgressArea();
	virtual void	DrawRemainderArea();
	
	virtual bool	GetOriginFrame(CRect_AC& originFrame);
	virtual bool	GetProgressFrame(CRect_AC& progressFrame);
	virtual bool	GetRemainderFrame(CRect_AC& remainderFrame);
	virtual void	GetRangeFrame(SInt32 startValue, SInt32 endValue, CRect_AC& rangeFrame);

	enum { kProgressHeight = 14 };
	enum { kProgressImageHeight = 10, kProgressImageWidth = 16 };
	enum { k1BitPattern, kDeepBluePattern, kDeepGrayscalePattern, kNumProgressPatterns };
	enum { kNumProgressAnimationSteps = 4 };	// 2, 4, 8, or 16 will work
	enum { kAnimationDelay = 5 };

	SInt32		fMinimum;
	SInt32		fMaximum;
	SInt32		fValue;
	SInt32		fOriginValue;		// usually == fMinimum
	UInt32		fAnimationIndex;
	SInt32		fLastAnimationTime;	// used for quantized animation

	static SInt8				kIndeterminateProgressImage[kNumProgressPatterns]
															[kProgressImageHeight]
															[kProgressImageWidth];

public:
	static bool fgImagesAllocated;
	static void AllocateProgressImages();
	static void DisposeProgressImages();

protected:
	static CAutoPtr_AC<CSimpleOffscreenImage_AC>	fgOffscreenIndeterminateProgressImage[kNumProgressPatterns];
};

//----------------------------------------------------------------------------------------
// CGrayscaleProgressIndicator_AC
// This subclass emulates the Apple Grayscale Appearance.
//----------------------------------------------------------------------------------------

class EXPORT_AC CGrayscaleProgressIndicator_AC : public CProgressIndicator_AC
{
public:


	CGrayscaleProgressIndicator_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum);
	virtual ~CGrayscaleProgressIndicator_AC();
	
protected:

	virtual void	DrawAnimationPattern(const CRect_AC& clipRect);
	virtual void	DrawOriginArea();
	virtual void	DrawProgressArea();
	virtual void	DrawRemainderArea();
	
};

#endif // __CProgressIndicator_AC__

