// CRadioButton_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CRadioButton_AC__
#define __CRadioButton_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

#ifndef __CGroupsContainer_AC__
#include "CGroupsContainer_AC.h"
#endif

#ifndef __CSimpleOffscreenImage_AC__
#include "CSimpleOffscreenImage_AC.h"
#endif

// ACS

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CRadioButton_AC
// CRadioButton_AC implements the standard AGA radio button. It also supports
// a mixed state that is neither on nor off, which is drawn as a dash.
// You can also assign an ID pair that will be used to automatically keep
// the radio button mutually exclusive with others with the same ID pair.
// Pass kNoGroupID for the IDs to bypass this. The adapter classes bypass it
// since the frameworks manage radio button groups.
//----------------------------------------------------------------------------------------

class EXPORT_AC CRadioButton_AC : public CControl_AC, public MExclusiveObject_AC
	{
	public:

		enum { kRadioButtonOff = 0, kRadioButtonOn = 1, kRadioButtonMixed = 2 };

		CRadioButton_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, const CStr255_AC& buttonTitle = kEmptyString_AC, UInt32 groupIDPart1 = kNoGroupID, UInt32 groupIDPart2 = kNoGroupID, bool automaticState = kNoAutomaticState);
		CRadioButton_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt16 stringListResourceID, SInt16 stringIndex, UInt32 groupIDPart1 = kNoGroupID, UInt32 groupIDPart2 = kNoGroupID, bool automaticState = kNoAutomaticState);
		virtual ~CRadioButton_AC();
		
		virtual void	DrawObject();
		virtual bool	TrackMouse(const CPoint_AC&  mouseLocation);

		virtual void	SetTitle(const CStr255_AC& buttonTitle, bool redraw);
		
		virtual SInt32 GetValue() const;
		virtual void	SetValue(SInt32 newValue, bool redraw);

		virtual void	SetGroupID(UInt32 groupIDPart1, UInt32 groupIDPart2);	// MExclusiveObject mixin override
		virtual void	TurnOff();	// MExclusiveObject mixin override

		virtual void	DrawButton(bool pressed);

		static void		AllocateRadioImages();
		static void		DisposeRadioImages();

	protected:

		virtual void	SetTrackingState(bool isIn);

		virtual void	DrawButtonNormal(bool deep);
		virtual void	DrawButtonPressed(bool deep);
		virtual void	DrawButtonDisabled(bool deep);
		
		virtual void	CopyImage(UInt32 index);
		
		SInt32			fValue;
		bool			fAutomaticState;	// if true, radio is set on click w/o caller needing to call SetValue
		UInt32			fGroupID;
		CStr255_AC		fTitle;
		CTextStyle_AC	fTextStyle;
		
		// Drawing the 3D radio button manually is just too damn tedious.
		// So we use offscreen PixMap structures and blast them onto the screen.
		// The mask (clip) region for the images is a round rectangle.

		enum { kRadioImageWidth = 12, kRadioImageHeight = 12 };
		enum { kRadioNormalOff, kRadioNormalOn, kRadioNormalMixed,
				kRadioPressedOff, kRadioPressedOn, kRadioPressedMixed,
				kRadioDisabledOff, kRadioDisabledOn, kRadioDisabledMixed,
				kNumRadioImages };
		
		static bool					fgImagesAllocated;
		static SInt8				kRadioImageValues[kNumRadioImages]
													[kRadioImageHeight]
													[kRadioImageWidth];

		static CAutoPtr_AC<CSimpleOffscreenImage_AC>	fgOffscreenRadioImages[kNumRadioImages];
	};


#endif // __CRadioButton_AC__
