// CScrollBar_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CScrollBar_AC__
#define __CScrollBar_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CIndicator_AC__
#include "CIndicator_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CScrollBar_AC
// CScrollBar_AC implements the standard AGA scroll bar. It also supports
// live indicator tracking and a proportional indicator as options.
//----------------------------------------------------------------------------------------

class EXPORT_AC CScrollBar_AC : public CIndicator_AC
{
public:

	CScrollBar_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue);
	CScrollBar_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, SInt32 singleStepSize, SInt32 pageStepSize, SInt32 pageSize, bool liveTracking, bool proportional);
	virtual ~CScrollBar_AC();

	virtual void	SetRange(SInt32 newMinimum, SInt32 newMaximum, bool redraw);

	virtual void	SetStepSizes(SInt32 singleStepSize, SInt32 pageStepSize);

protected:

	virtual bool	TrackPart(const CPoint_AC& mouseLocation, SInt32 partHit);
	virtual SInt32	TrackTestPart(const CPoint_AC& mouseLocation);
	virtual bool	IsValidIndicatorTrackMouse(const CPoint_AC& mouseLocation);
	virtual SInt32	GetValueFromMouseDelta(SInt32 originalValue, const CPoint_AC& originalMouseLocation, const CPoint_AC& newMouseLocation);

	virtual void	DrawIndicatorTrack();
	virtual void	DrawTrackEnds();
	virtual void	DrawIndicator();
	virtual void	RemoveIndicator(SInt32 newValue);
	virtual void	DrawGhost();
	virtual void	RemoveGhost(SInt32 newValue);
	
	virtual void	TrackDelta(SInt32 partHit);

	virtual void	DrawArrow(SInt32 arrowPart, bool pressed);
	
	enum { kIndicatorPixelInset = 8 };
	enum { kArrowSize = 16 };

	virtual void	GetIndicatorBox(SInt32 value, CRect_AC& indicatorBox);
	virtual SInt32	GetIndicatorPixelRange(SInt32* proportionalIndicatorPixels);
	virtual void	GetIndicatorPixelEnds(SInt32* startPt, SInt32* stopPt);
	virtual void	GetIndicatorSpan(SInt32 value, SInt32* startPt, SInt32* valuePt, SInt32* stopPt);
	
	SInt32	fSingleStepSize;	// amount to delta for arrow tracking
	SInt32	fPageStepSize;		// amount to delta for page tracking
	
	// These regions are used by RemoveIndicator and RemoveGhost.
	// They are tiny but we allocate them statically to avoid
	// repeated allocation, which could conceivably fail.
	static RgnHandle	fgSavedIndicatorClip;
	static RgnHandle	fgOldIndicatorClip;
	static RgnHandle	fgNewIndicatorClip;
};

#endif // __CScrollBar_AC__
