// CSlider_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CSlider_AC__
#define __CSlider_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CIndicator_AC__
#include "CIndicator_AC.h"
#endif

// ACS

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CSlider_AC
// CSlider_AC implements the standard AGA slider. You specify whether it
// is horizontal or vertical, labeled or unlabeled, rectangular or pointy.
// By default, if you specify a labeled slider, it is pointy; otherwise it
// is rectangular. You can change this by calling SetSliderKind, and you
// can set the labels up after construction by calling the label functions.
// It also supports live indicator tracking and a proportional indicator
// as options.
//----------------------------------------------------------------------------------------

class EXPORT_AC CSlider_AC : public CIndicator_AC
	{
	public:

		enum SliderKind { kRectSlider, kPointerSlider };
		enum { kInverseDirection = true, kNormalDirection = false };

		CSlider_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, const CTextStyle_AC& textStyle, SInt16 labelsID);
		CSlider_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, const CTextStyle_AC& textStyle, SInt16 labelsID, SliderKind kind, bool liveTracking, bool proportional);
		virtual ~CSlider_AC();
		
		virtual void	SetSliderKind(SliderKind kind);
		virtual void	SetLabelsFromResource(SInt16 stringListResourceID);
		virtual void	SetNumLabels(UInt32 numLabels);
		virtual void	SetLabel(UInt32 labelIndex, const CStr255_AC& itsLabel);
		virtual void	GetLabel(UInt32 labelIndex, CStr255_AC& itsLabel);
		virtual void	SetLabelsStyle(const CTextStyle_AC& textStyle);
		virtual void	SetJustification(SInt32 justification);
		virtual void	SetInverseDirection(bool inverseDirection);
		virtual SInt32 GetValue() const;
		virtual void	SetValue(SInt32 newValue, bool redraw);

	protected:

		virtual SInt32	TrackTestPart(const CPoint_AC& mouseLocation);
		virtual bool	IsValidIndicatorTrackMouse(const CPoint_AC& mouseLocation);
		virtual SInt32	GetValueFromMouseDelta(SInt32 originalValue, const CPoint_AC& originalMouseLocation, const CPoint_AC& newMouseLocation);

		virtual void	DrawBackground();
		virtual void	DrawIndicatorTrack();
		virtual void	DrawIndicator();
		virtual void	RemoveIndicator(SInt32 newValue);
		virtual void	DrawGhost();
		virtual void	RemoveGhost(SInt32 newValue);
		
		enum { kRectIndicatorPixelInset = 6 };
		enum { kPointerIndicatorPixelInset = 7 };
		enum { kIndicatorWidth = 16 };
		enum { kEndGapSize = 5 };
		enum { kLabelTickEndInset = 11 };
		enum { kLabelTickOffset = 8 };
		enum { kLabelPosTextOffset = 14 };
		enum { kLabelNegTextOffset = 8 };
		enum { kLabelTickLength = 6 };
		enum { kGhostIndicator = true, kNormalIndicator = false };
		enum { kIncludeBackground = true, kTrackOnly = false };

		virtual void	GetTrackFrame(CRect_AC& trackFrame, bool includeBackground);
		virtual void	GetIndicatorBox(SInt32 value, CRect_AC& indicatorBox);
		virtual SInt32	GetIndicatorPixelRange(SInt32* proportionalIndicatorPixels);
		virtual void	GetIndicatorPixelEnds(SInt32* startPt, SInt32* stopPt);
		virtual void	GetIndicatorSpan(SInt32 value, SInt32* startPt, SInt32* valuePt, SInt32* stopPt);
		
		virtual void	DrawLabels();
		virtual void	DrawRectIndicator(const CRect_AC& indicatorRect, bool isGhost);
		virtual void	DrawPointerIndicator(const CRect_AC& indicatorRect, bool isGhost);
		virtual void	BuildPointerIndicator(const CRect_AC& indicatorRect);
		
		virtual void	DisposeLabels();
		
		Handle			fLabelStringHandles;	// an array of StringHandle, may be NULL
		SInt32			fJustification;
		SliderKind		fSliderKind;
		CTextStyle_AC	fTextStyle;
		bool			fInverseDirection;
		
		// These regions are used by RemoveIndicator and RemoveGhost.
		// They are tiny but we allocate them statically to avoid
		// repeated allocation, which could conceivably fail.
		static RgnHandle	fgSavedIndicatorClip;
		static RgnHandle	fgOldIndicatorClip;
		static RgnHandle	fgNewIndicatorClip;
	};



#endif // __CSlider_AC__

