//----------------------------------------------------------------------------------------
// AGAImaging_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright  1996 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif

// Toolbox

// Use Windows_AC.h instead of <windows.h> for XPlatform compatability.
#ifndef __Windows_AC__
#include "Windows_AC.h"
#endif

//----------------------------------------------------------------------------------------
// AGABackgroundPaint
//----------------------------------------------------------------------------------------

void AGABackgroundPaint(const CRect_AC& backgroundBounds,
						bool drawFill,
						AGABackgroundKind backgroundKind,
						bool isActive,
						bool hasGrowBox)
{
	if (backgroundKind == kNoBackgroundPaint)
		return;

	CTempDrawingEnvironment_AC	env;

	// Draw in the specified active/inactive state.

	enum { kBGFill, kBGActiveLight, kBGActiveShadow, kBGActiveCorner, kBGInactiveLight, kBGInactiveShadow, kBGInactiveCorner, kBGNumIndexes };

	UInt8	colorIndexes[kBGNumIndexes];

	switch (backgroundKind)
	{
		case kRaisedModelessBackground:
			colorIndexes[kBGFill] = r2;
			colorIndexes[kBGActiveLight] = rW;
			colorIndexes[kBGActiveShadow] = r6;
			colorIndexes[kBGActiveCorner] = r2;
			colorIndexes[kBGInactiveLight] = r2;
			colorIndexes[kBGInactiveShadow] = r2;
			colorIndexes[kBGInactiveCorner] = r2;
			break;

		case kRaisedModalBackground:
			colorIndexes[kBGFill] = r2;
			colorIndexes[kBGActiveLight] = rW;
			colorIndexes[kBGActiveShadow] = r6;
			colorIndexes[kBGActiveCorner] = r4;
			colorIndexes[kBGInactiveLight] = r2;
			colorIndexes[kBGInactiveShadow] = r2;
			colorIndexes[kBGInactiveCorner] = r2;
			break;

		case kFlatWindowBackground:
			colorIndexes[kBGFill] = r2;
			colorIndexes[kBGActiveLight] = r2;
			colorIndexes[kBGActiveShadow] = r2;
			colorIndexes[kBGActiveCorner] = r2;
			colorIndexes[kBGInactiveLight] = r2;
			colorIndexes[kBGInactiveShadow] = r2;
			colorIndexes[kBGInactiveCorner] = r2;
			break;
	}

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		CRect_AC	r(backgroundBounds);

		if (deep)
		{
			::InsetRect(&r, 1, 1);
			
			if (drawFill)
			{
				kRGBColorRamp[colorIndexes[kBGFill]].SetForeColor();
				::PaintRect(&r);
			}
			
			if (isActive)
				kRGBColorRamp[colorIndexes[kBGActiveCorner]].SetForeColor();
			else
				kRGBColorRamp[colorIndexes[kBGInactiveCorner]].SetForeColor();

			::MoveTo(r.left - 1, r.bottom);
			::Line(0, 0);
			::MoveTo(r.right, r.top - 1);
			::Line(0, 0);
			
			if (isActive)
				kRGBColorRamp[colorIndexes[kBGActiveLight]].SetForeColor();
			else
				kRGBColorRamp[colorIndexes[kBGInactiveLight]].SetForeColor();

			::MoveTo(r.left - 1, r.bottom - 1);
			::LineTo(r.left - 1, r.top - 1);
			::LineTo(r.right - 1, r.top - 1);
			
			if (isActive)
				kRGBColorRamp[colorIndexes[kBGActiveShadow]].SetForeColor();
			else
				kRGBColorRamp[colorIndexes[kBGInactiveShadow]].SetForeColor();

			::MoveTo(r.left, r.bottom);
			
			if (hasGrowBox)
			{
				::LineTo(r.right - 15, r.bottom);
				::Line(0, -15);
				::Line(15, 0);
			}
			else
				::LineTo(r.right, r.bottom);

			::LineTo(r.right, r.top);
		}
		else	// 1-bit
		{
			if (drawFill)
			{
				kRGBColorRamp[rW].SetBackColor();
				::EraseRect(&r);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// AGADrawSeparator
//----------------------------------------------------------------------------------------

void AGADrawSeparator(const CRect_AC& bounds, bool isEnabled, const CRGBColor_AC& windowBackgroundColor, AGASeparatorLayout layout)
{
	Point	startPt;
	Point	endPt;
	bool	isHorizontal;
	
	if (layout == kSeparatorAutomatic)
	{
		if ((bounds.right - bounds.left) > (bounds.bottom - bounds.top))
			layout = kSeparatorTop;
		else
			layout = kSeparatorLeft;
		}
	
	switch (layout)
	{
		case kSeparatorLeft:
		case kSeparatorTop:

			startPt.h = bounds.left;
			startPt.v = bounds.top;
			
			break;

		case kSeparatorRight:

			startPt.h = bounds.right;
			startPt.v = bounds.top;
			
			break;

		case kSeparatorBottom:

			startPt.h = bounds.left;
			startPt.v = bounds.bottom;
			
			break;
	}

	switch (layout)
	{
		case kSeparatorLeft:

			endPt.h = bounds.left;
			endPt.v = bounds.bottom;

			break;

		case kSeparatorTop:

			endPt.h = bounds.right;
			endPt.v = bounds.top;

			break;

		case kSeparatorRight:
		case kSeparatorBottom:

			endPt.h = bounds.right;
			endPt.v = bounds.bottom;
			
			break;
	}
	
	isHorizontal = (layout == kSeparatorTop) || (layout == kSeparatorBottom);

	CTempDrawingEnvironment_AC	env;
	CGDIterator_AC				iter;
	bool						deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
			kRGBColorRamp[r7].SetForeColor();
		else
		{
			kRGBColorRamp[rB].SetForeColor();
			PenPatGray();
		}

		::MoveTo(startPt.h, startPt.v);
		
		if (isHorizontal)
			::LineTo(endPt.h - 2, endPt.v);
		else
			::LineTo(endPt.h, endPt.v - 2);

		if (deep)
		{
			if (isEnabled)
				kRGBColorRamp[rW].SetForeColor();
			else
				windowBackgroundColor.SetForeColor();

			::MoveTo(startPt.h + 1, startPt.v + 1);
			
			if (isHorizontal)
				::LineTo(endPt.h - 1, endPt.v + 1);
			else
				::LineTo(endPt.h + 1, endPt.v - 1);
		}
		else
			PenPatBlack();
	}
}

//----------------------------------------------------------------------------------------
// AGADrawBevelButton
//----------------------------------------------------------------------------------------

static void DrawMiniBevelEdges(const CRect_AC& bounds, bool isOn, bool isPressed, bool isEnabled)
{
	// Draw the button edges as specified for a
	// "mini" sized button.

	enum { TLFrame, TLInner, BRFrame, BRInner, CornerFrame, CornerInner, Fill, kNumEdgeColors };

	UInt8	colorIndexes[kNumEdgeColors];

	CRect_AC	r = bounds;
	
	if (isPressed)
	{
		colorIndexes[TLFrame] = rA2;
		colorIndexes[BRFrame] = r11;
		colorIndexes[CornerFrame] = r12;
		colorIndexes[TLInner] = r10;
		colorIndexes[BRInner] = r5;
		colorIndexes[CornerInner] = r8;
		colorIndexes[Fill] = r7;
	}
	else if (isEnabled)
	{
		colorIndexes[TLFrame] = isOn ? rA2 : r9;
		colorIndexes[BRFrame] = isOn ? r11 : rA1;
		colorIndexes[CornerFrame] = isOn ? r12 : r10;
		colorIndexes[TLInner] = isOn ? r10 : rW;
		colorIndexes[BRInner] = isOn ? r5 : r7;
		colorIndexes[CornerInner] = isOn ? r8 : r3;
		colorIndexes[Fill] = isOn ? r7 : r3;
	}
	else	// disabled
	{
		colorIndexes[TLFrame] = isOn ? r8 : r5;
		colorIndexes[BRFrame] = r6;
		colorIndexes[CornerFrame] = isOn ? r7 : r5;
		colorIndexes[TLInner] = isOn ? r4 : r2;
		colorIndexes[BRInner] = isOn ? r4 : r2;
		colorIndexes[CornerInner] = isOn ? r4 : r2;
		colorIndexes[Fill] = isOn ? r4 : r2;
	}
	
	// Draw the outer frame.

	kRGBColorRamp[colorIndexes[TLFrame]].SetForeColor();
	::MoveTo(r.left, r.bottom - 2);
	::LineTo(r.left, r.top);
	::LineTo(r.right - 1, r.top);

	kRGBColorRamp[colorIndexes[CornerFrame]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRFrame]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 1, r.bottom - 1);
	::LineTo(r.left, r.bottom - 1);

	kRGBColorRamp[colorIndexes[CornerFrame]].SetForeColor();
	::Line(0, 0);

	// Draw the 1st pixel of shading.

	kRGBColorRamp[colorIndexes[TLInner]].SetForeColor();
	::MoveTo(r.left + 1, r.bottom - 3);
	::LineTo(r.left + 1, r.top + 1);
	::LineTo(r.right - 2, r.top + 1);

	kRGBColorRamp[colorIndexes[CornerInner]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRInner]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 2, r.bottom - 2);
	::LineTo(r.left + 1, r.bottom - 2);

	kRGBColorRamp[colorIndexes[CornerInner]].SetForeColor();
	::Line(0, 0);

	// Fill the interior.

	kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
	::InsetRect(&r, 2, 2);
	::PaintRect(&r);
	::InsetRect(&r, -2, -2);
}
	
//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

static void DrawSmallBevelEdges(const CRect_AC& bounds, bool isOn, bool isPressed, bool isEnabled)
{
	// Draw the button edges as specified for a
	// "small" sized button.

	enum { TLFrame, TLInner1, TLInner2, BRFrame, BRInner1, BRInner2, CornerFrame, CornerInner1, CornerInner2, Fill, kNumEdgeColors };

	UInt8	colorIndexes[kNumEdgeColors];

	CRect_AC	r = bounds;
	
	if (isPressed)
	{
		colorIndexes[TLFrame] = rA2;
		colorIndexes[BRFrame] = r11;
		colorIndexes[CornerFrame] = rA2;
		colorIndexes[TLInner1] = r11;
		colorIndexes[BRInner1] = r4;
		colorIndexes[CornerInner1] = r8;
		colorIndexes[TLInner2] = r9;
		colorIndexes[BRInner2] = r6;
		colorIndexes[CornerInner2] = r7;
		colorIndexes[Fill] = r7;
	}
	else if (isEnabled)
	{
		colorIndexes[TLFrame] = isOn ? rA2 : r9;
		colorIndexes[BRFrame] = isOn ? r11 : rA1;
		colorIndexes[CornerFrame] = isOn ? rA2 : r9;
		colorIndexes[TLInner1] = isOn ? r11 : r3;
		colorIndexes[BRInner1] = isOn ? r4 : r8;
		colorIndexes[CornerInner1] = isOn ? r8 : r5;
		colorIndexes[TLInner2] = isOn ? r9 : rW;
		colorIndexes[BRInner2] = r6;
		colorIndexes[CornerInner2] = isOn ? r7 : r3;
		colorIndexes[Fill] = isOn ? r7 : r3;
	}
	else	// disabled
	{
		colorIndexes[TLFrame] = isOn ? r8 : r5;
		colorIndexes[BRFrame] = r6;
		colorIndexes[CornerFrame] = isOn ? r8 : r5;
		colorIndexes[TLInner1] = isOn ? r4 : r2;
		colorIndexes[BRInner1] = isOn ? r4 : r2;
		colorIndexes[CornerInner1] = isOn ? r4 : r2;
		colorIndexes[TLInner2] = isOn ? r4 : r2;
		colorIndexes[BRInner2] = isOn ? r4 : r2;
		colorIndexes[CornerInner2] = isOn ? r4 : r2;
		colorIndexes[Fill] = isOn ? r4 : r2;
	}
	
	// Draw the outer frame.

	kRGBColorRamp[colorIndexes[TLFrame]].SetForeColor();
	::MoveTo(r.left, r.bottom - 2);
	::LineTo(r.left, r.top);
	::LineTo(r.right - 1, r.top);

	kRGBColorRamp[colorIndexes[CornerFrame]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRFrame]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 1, r.bottom - 1);
	::LineTo(r.left, r.bottom - 1);

	kRGBColorRamp[colorIndexes[CornerFrame]].SetForeColor();
	::Line(0, 0);

	// Draw the 1st pixel of shading.

	kRGBColorRamp[colorIndexes[TLInner1]].SetForeColor();
	::MoveTo(r.left + 1, r.bottom - 3);
	::LineTo(r.left + 1, r.top + 1);
	::LineTo(r.right - 2, r.top + 1);

	kRGBColorRamp[colorIndexes[CornerInner1]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRInner1]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 2, r.bottom - 2);
	::LineTo(r.left + 1, r.bottom - 2);

	kRGBColorRamp[colorIndexes[CornerInner1]].SetForeColor();
	::Line(0, 0);

	// Draw the 2nd pixel of shading.

	kRGBColorRamp[colorIndexes[TLInner2]].SetForeColor();
	::MoveTo(r.left + 2, r.bottom - 4);
	::LineTo(r.left + 2, r.top + 2);
	::LineTo(r.right - 3, r.top + 2);

	kRGBColorRamp[colorIndexes[CornerInner2]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRInner2]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 3, r.bottom - 3);
	::LineTo(r.left + 2, r.bottom - 3);

	kRGBColorRamp[colorIndexes[CornerInner2]].SetForeColor();
	::Line(0, 0);

	// Fill the interior.

	kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
	::InsetRect(&r, 3, 3);
	::PaintRect(&r);
	::InsetRect(&r, -3, -3);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

static void DrawLargeBevelEdges(const CRect_AC& bounds, bool isOn, bool isPressed, bool isEnabled)
{
	// Draw the button edges as specified for a
	// "large" sized button.

	enum { TLFrame, TLInner1, TLInner2, TLInner3, BRFrame, BRInner1, BRInner2, BRInner3, CornerFrame, CornerInner1, CornerInner2, CornerInner3, Fill, kNumEdgeColors };

	UInt8	colorIndexes[kNumEdgeColors];

	CRect_AC	r = bounds;
	
	if (isPressed)
	{
		colorIndexes[TLFrame] = rA2;
		colorIndexes[BRFrame] = r11;
		colorIndexes[CornerFrame] = r12;
		colorIndexes[TLInner1] = rA1;
		colorIndexes[BRInner1] = r4;
		colorIndexes[CornerInner1] = r11;
		colorIndexes[TLInner2] = r11;
		colorIndexes[BRInner2] = r5;
		colorIndexes[CornerInner2] = r8;
		colorIndexes[TLInner3] = r9;
		colorIndexes[BRInner3] = r6;
		colorIndexes[CornerInner3] = r7;
		colorIndexes[Fill] = r7;
	}
	else if (isEnabled)
	{
		colorIndexes[TLFrame] = isOn ? rA2 : r9;
		colorIndexes[BRFrame] = isOn ? r11 : rA1;
		colorIndexes[CornerFrame] = isOn ? r12 : r10;
		colorIndexes[TLInner1] = isOn ? rA1 : r3;
		colorIndexes[BRInner1] = isOn ? r4 : r10;
		colorIndexes[CornerInner1] = isOn ? r11 : r5;
		colorIndexes[TLInner2] = isOn ? r11 : r1;
		colorIndexes[BRInner2] = isOn ? r5 : r8;
		colorIndexes[CornerInner2] = isOn ? r8 : r4;
		colorIndexes[TLInner3] = isOn ? r9 : rW;
		colorIndexes[BRInner3] = r6;
		colorIndexes[CornerInner3] = isOn ? r7 : r3;
		colorIndexes[Fill] = isOn ? r7 : r3;
	}
	else	// disabled
	{
		colorIndexes[TLFrame] = isOn ? r8 : r5;
		colorIndexes[BRFrame] = isOn ? r6 : r7;
		colorIndexes[CornerFrame] = isOn ? r7 : r6;
		colorIndexes[TLInner1] = isOn ? r4 : r2;
		colorIndexes[BRInner1] = isOn ? r4 : r2;
		colorIndexes[CornerInner1] = isOn ? r4 : r2;
		colorIndexes[TLInner2] = isOn ? r4 : r2;
		colorIndexes[BRInner2] = isOn ? r4 : r2;
		colorIndexes[CornerInner2] = isOn ? r4 : r2;
		colorIndexes[TLInner3] = isOn ? r4 : r2;
		colorIndexes[BRInner3] = isOn ? r4 : r2;
		colorIndexes[CornerInner3] = isOn ? r4 : r2;
		colorIndexes[Fill] = isOn ? r4 : r2;
	}
	
	// Draw the outer frame.

	kRGBColorRamp[colorIndexes[TLFrame]].SetForeColor();
	::MoveTo(r.left, r.bottom - 2);
	::LineTo(r.left, r.top);
	::LineTo(r.right - 1, r.top);

	kRGBColorRamp[colorIndexes[CornerFrame]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRFrame]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 1, r.bottom - 1);
	::LineTo(r.left, r.bottom - 1);

	kRGBColorRamp[colorIndexes[CornerFrame]].SetForeColor();
	::Line(0, 0);

	// Draw the 1st pixel of shading.

	kRGBColorRamp[colorIndexes[TLInner1]].SetForeColor();
	::MoveTo(r.left + 1, r.bottom - 3);
	::LineTo(r.left + 1, r.top + 1);
	::LineTo(r.right - 2, r.top + 1);

	kRGBColorRamp[colorIndexes[CornerInner1]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRInner1]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 2, r.bottom - 2);
	::LineTo(r.left + 1, r.bottom - 2);

	kRGBColorRamp[colorIndexes[CornerInner1]].SetForeColor();
	::Line(0, 0);

	// Draw the 2nd pixel of shading.

	kRGBColorRamp[colorIndexes[TLInner2]].SetForeColor();
	::MoveTo(r.left + 2, r.bottom - 4);
	::LineTo(r.left + 2, r.top + 2);
	::LineTo(r.right - 3, r.top + 2);

	kRGBColorRamp[colorIndexes[CornerInner2]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRInner2]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 3, r.bottom - 3);
	::LineTo(r.left + 2, r.bottom - 3);

	kRGBColorRamp[colorIndexes[CornerInner2]].SetForeColor();
	::Line(0, 0);

	// Draw the 3rd pixel of shading.

	kRGBColorRamp[colorIndexes[TLInner3]].SetForeColor();
	::MoveTo(r.left + 3, r.bottom - 5);
	::LineTo(r.left + 3, r.top + 3);
	::LineTo(r.right - 4, r.top + 3);

	kRGBColorRamp[colorIndexes[CornerInner3]].SetForeColor();
	::Line(0, 0);

	kRGBColorRamp[colorIndexes[BRInner3]].SetForeColor();
	::Move(0, 1);
	::LineTo(r.right - 4, r.bottom - 4);
	::LineTo(r.left + 3, r.bottom - 4);

	kRGBColorRamp[colorIndexes[CornerInner3]].SetForeColor();
	::Line(0, 0);

	// Fill the interior.

	kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
	::InsetRect(&r, 4, 4);
	::PaintRect(&r);
	::InsetRect(&r, -4, -4);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void AGADrawBevelButton(const CRect_AC& bounds, ButtonFrameType frameType, bool isOn, bool isPressed, bool isEnabled)
{
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
		AGADrawBevelButtonAtDepth(deep, bounds, frameType, isOn, isPressed, isEnabled);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void AGADrawBevelButtonAtDepth(bool deep, const CRect_AC& bounds, ButtonFrameType frameType, bool isOn, bool isPressed, bool isEnabled)
{
	CRect_AC	r = bounds;
	
	if (frameType == kAutoFrame)
		frameType = AGAGetDefaultFrameType(bounds.right - bounds.left);

	if (deep)
	{
		if (frameType == kMiniFrame)
			DrawMiniBevelEdges(bounds, isOn, isPressed, isEnabled);
		else if (frameType == kSmallFrame)
			DrawSmallBevelEdges(bounds, isOn, isPressed, isEnabled);
		else
			DrawLargeBevelEdges(bounds, isOn, isPressed, isEnabled);
	}
	else
	{
		if (isPressed || isOn)
			kRGBColorRamp[rB].SetForeColor();
		else
			kRGBColorRamp[rW].SetForeColor();

		::PaintRect(&r);
		
		if (! isEnabled)
			PenPatGray();

		kRGBColorRamp[rB].SetForeColor();
		::FrameRect(&r);
		::PenNormal();
	}
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

ButtonFrameType AGAGetDefaultFrameType(SInt16 buttonWidth)
{
	if (buttonWidth > 39)
		return kLargeFrame;
	else if (buttonWidth > 21)
		return kSmallFrame;
	else
		return kMiniFrame;
}

//----------------------------------------------------------------------------------------
// AGAInstallGrayWCTB
//----------------------------------------------------------------------------------------

OSErr AGAInstallGrayWCTB(WindowPtr aWindow)
{
	CRGBColor_AC	dontCareColor;
	bool			dontCareBoolean;

	return AGAInstallWCTB(aWindow, kRGBColorRamp[r2], &dontCareBoolean, dontCareColor);
}

//----------------------------------------------------------------------------------------
// AGAInstallWCTB
//----------------------------------------------------------------------------------------

OSErr AGAInstallWCTB(WindowPtr aWindow, const CRGBColor_AC& newColor, bool* hasOldColor, CRGBColor_AC& oldColor)
{
	//
	// If it's a color window, make a copy of the default
	// color table, set the background to the specified gray,
	// and install it into the window.
	//

	OSErr			err = noErr;
    AuxWinHandle	auxWinHandle;
    CTabHandle		colorTableHandle;
    
    *hasOldColor = false;

    ::GetAuxWin(NULL, &auxWinHandle);

    if (auxWinHandle != NULL)
   	{
	    colorTableHandle = (**auxWinHandle).awCTable;
	    
	    *hasOldColor = true;
	    oldColor = (**colorTableHandle).ctTable->rgb;
	    
	    err = ::HandToHand((Handle*) &colorTableHandle);

		if (err == noErr)
		{
		    (**colorTableHandle).ctTable->rgb = newColor;    
		    ::SetWinColor(aWindow, (WCTabHandle) colorTableHandle);
		 }
	}
    
    return err;
}

//----------------------------------------------------------------------------------------
// AGADrawBorderFrame
//----------------------------------------------------------------------------------------


void AGADrawBorderFrame(const CRect_AC& bounds, bool isEnabled, bool isNotched, bool drawFrame, const CRGBColor_AC& windowBackgroundColor)
{
	// Draw the frame with shading outside.
	
	enum { Shadow, Frame, Light, kNumColors };

	CRect_AC	r = bounds;
	
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		CTempDrawingEnvironment_AC	env;

		if (drawFrame)
		{
			if (deep)
			{
				if (isEnabled)
					kRGBColorRamp[rB].SetForeColor();
				else
					kRGBColorRamp[r10].SetForeColor();
			}
			else
			{
				kRGBColorRamp[rB].SetForeColor();

				if (isEnabled)
					PenPatBlack();
				else
					PenPatGray();
			}

			::InsetRect(&r, 1, 1);
			
			if (isNotched)
			{
				::MoveTo(r.right - 1, r.top);
				::LineTo(r.left, r.top);
				::LineTo(r.left, r.bottom - 1);
				::LineTo(r.right - 16, r.bottom - 1);
				::Line(0, -15);
				::Line(15, 0);
				::LineTo(r.right - 1, r.top);
			}
			else
				::FrameRect(&r);

			::InsetRect(&r, -1, -1);
		}
		
		if (deep)
		{
			if (isEnabled)
				kRGBColorRamp[r5].SetForeColor();
			else
				windowBackgroundColor.SetForeColor();

			::MoveTo(r.left, r.bottom - 2);
			::LineTo(r.left, r.top);
			::LineTo(r.right - 2, r.top);
			
			if (isEnabled)
				kRGBColorRamp[rW].SetForeColor();
			else
				windowBackgroundColor.SetForeColor();

			::MoveTo(r.left + 1, r.bottom - 1);

			if (isNotched)
			{
				::LineTo(r.right - 16, r.bottom - 1);
				::Line(0, -15);
				::Line(15, 0);
			}
			else
				::LineTo(r.right - 1, r.bottom - 1);

			::LineTo(r.right - 1, r.top + 1);
		}
	}
}

//----------------------------------------------------------------------------------------
// AGAComputeTargetBorderRegion
//----------------------------------------------------------------------------------------

void AGAComputeTargetBorderRegion(RgnHandle borderRegion, const CRect_AC& bounds, bool isNotched)
{
	// Compute the border region to be painted.

	RgnHandle	innerRegion = ::NewRgn();
	CRect_AC		r(bounds);

	// First create a region framing the outer edge of the border.
	::OpenRgn();
	::FrameRoundRect(&r, 4, 4);
	::CloseRgn(borderRegion);

	// Now subtract away the hollow inside area.
	::InsetRect(&r, 2, 2);
	::RectRgn(innerRegion, &r);
	::DiffRgn(borderRegion, innerRegion, borderRegion);
	
	if (isNotched)
	{
		// OK, region madness as we add the top/left sides
		// of a new "notch" rect frame, and also remove the
		// bottom/right corner of our existing non-notched frame.
		
		RgnHandle	notchRegion = ::NewRgn();
		CRect_AC	notchRect;
		
		// Create a new round rect for the corner area and
		// union it with the existing region.
		notchRect = bounds;
		notchRect.left = notchRect.right - 17;
		notchRect.top = notchRect.bottom - 17;

		::RectRgn(notchRegion, &notchRect);
		::UnionRgn(borderRegion, notchRegion, borderRegion);
		
		// Subtract away the corner of the existing frame.
		notchRect.left += 2;
		notchRect.top += 2;

		::RectRgn(notchRegion, &notchRect);
		::DiffRgn(borderRegion, notchRegion, borderRegion);
		
		// Almost done. There's an extra pixel at each notch
		// corner that needs to be removed to make round corners.
		
		CRect_AC	pixelRect;
		
		// The top/right corner.
		pixelRect = notchRect;
		pixelRect.left = pixelRect.right - 1;
		--pixelRect.top;
		pixelRect.bottom = pixelRect.top + 1;
		::RectRgn(notchRegion, &pixelRect);
		::DiffRgn(borderRegion, notchRegion, borderRegion);
		
		// The bottom/left corner.
		pixelRect = notchRect;
		pixelRect.top = pixelRect.bottom - 1;
		--pixelRect.left;
		pixelRect.right = pixelRect.left + 1;
		::RectRgn(notchRegion, &pixelRect);
		::DiffRgn(borderRegion, notchRegion, borderRegion);
		
		// The top/left corner.
		pixelRect = notchRect;
		pixelRect.top -= 2;
		pixelRect.left -= 2;
		pixelRect.bottom = pixelRect.top + 1;
		pixelRect.right = pixelRect.left + 1;
		::RectRgn(notchRegion, &pixelRect);
		::DiffRgn(borderRegion, notchRegion, borderRegion);
		
		::DisposeRgn(notchRegion);
	}
	
	::DisposeRgn(innerRegion);
}

//----------------------------------------------------------------------------------------
// AGAStringOut
// This is the routine we use for all of our one-line text output.
//----------------------------------------------------------------------------------------

void AGAStringOut(const CStr255_AC& aString,
					const CRect_AC& centeringFrame,
					SInt32 truncation,
					SInt32 justification,
					AGAStringOutColor outColor,
					bool deep,
					const CTextStyle_AC& textStyle)
{
	CStr255_AC	outputString;
	FontInfo	fontInfo;
	SInt16		frameWidth = centeringFrame.right - centeringFrame.left;
	SInt16		frameHeight = centeringFrame.bottom - centeringFrame.top;
	Point		penOrigin;
	bool		setColor = true;
	UInt8		colorIndex;
	
	textStyle.SetPortTextStyle();
	::GetFontInfo(&fontInfo);

	switch (outColor)
	{
		case kNormalOutput:
			colorIndex = rB;
			break;
		case kDisabledOutput:
			if (deep)
				colorIndex = r7;
			else
				colorIndex = rB;
			break;
		case kInverseOutput:
			colorIndex = rW;
			break;
		case kUseCurrentColor:
			setColor = false;
			break;
	}

	if (setColor)
		kRGBColorRamp[colorIndex].SetForeColor();
	
	outputString = aString;
	
	// Truncate the string if it does not fit the width of the frame.
	if (truncation != kNoTruncation)
	{
		// If the string would be truncated at the current
		// font settings, try again with condensed text.
		if (::TruncString(frameWidth, outputString, truncation) == truncated)
		{
			outputString = aString;
			::TextFace(condense);
			(void) ::TruncString(frameWidth, outputString, truncation);
		}
	}
	
	// Center the text vertically. Pen goes at baseline of
	// text, i.e. between the ascent and descent.
	penOrigin.v = centeringFrame.top + (frameHeight / 2) + ((fontInfo.ascent + fontInfo.descent) / 2) - fontInfo.descent;
	
	// Position the text horizontally left, centered, or right, based
	// on the specified justification and the system script direction.
	switch (CTextStyle_AC::RuntimeJustify(justification))
	{
		case teCenter:
			penOrigin.h = centeringFrame.left + (frameWidth / 2) - (::StringWidth(outputString) / 2);
			break;
		case teFlushRight:
			penOrigin.h = centeringFrame.right - ::StringWidth(outputString);
			break;
		default: //case teFlushLeft:
			penOrigin.h = centeringFrame.left;
			break;
	}

	::MoveTo(penOrigin.h, penOrigin.v);

	GrafPtr	currentPort;
	::GetPort(&currentPort);
	SInt16	savedTxMode = currentPort->txMode;
	::TextMode(srcOr);
	::DrawString(outputString);
	::TextMode(savedTxMode);
	
	if ((outColor == kDisabledOutput) && ! deep)
	{
		PenPatGray();
		::PenMode(patBic);
		::PaintRect(centeringFrame);

		::PenNormal();
	}
}

//----------------------------------------------------------------------------------------
// AGATextBox
// This is a replacement for TETextBox.
//----------------------------------------------------------------------------------------

void AGATextBox(const void *textPtr,
				SInt32 textLength,
				const CRect_AC& textFrame,
				SInt16 justification,
				AGAStringOutColor outColor,
				bool deep,
				const CTextStyle_AC& textStyle,
				const CRGBColor_AC& teBackgroundColor,
				bool eraseFirst)
{
	FontInfo	fontInfo;
	bool		setColor = true;
	UInt8		colorIndex;
	
	CTempBackColor_AC tempBackColor(teBackgroundColor);
	
	textStyle.SetPortTextStyle();
	::GetFontInfo(&fontInfo);

	switch (outColor)
	{
		case kNormalOutput:
			colorIndex = rB;
			break;
		case kDisabledOutput:
			if (deep)
				colorIndex = r7;
			else
				colorIndex = rB;
			break;
		case kInverseOutput:
			colorIndex = rW;
			break;
		case kUseCurrentColor:
			setColor = false;
			break;
	}

	if (setColor)
		kRGBColorRamp[colorIndex].SetForeColor();
	
	TextBox_AC((Ptr) textPtr, textLength, textFrame, justification, kAutoWrap, NULL, eraseFirst);

	if ((outColor == kDisabledOutput) && ! deep)
	{
		PenPatGray();
		::PenMode(patBic);
		::PaintRect(textFrame);

		::PenNormal();
	}
}

