//----------------------------------------------------------------------------------------
// AGA_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright  1996 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

// Toolbox

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#if qWantsAppearance
#	ifndef __APPEARANCE__
#	include <Appearance.h>
#	endif
#endif
//----------------------------------------------------------------------------------------
// Global for whether to emulate controls
//----------------------------------------------------------------------------------------

static bool pEmulateControls = true;
static bool pHasAppearanceMgr = false;
static UInt32 pAGAFeatures = 0;

//----------------------------------------------------------------------------------------
// InitAGA_AC
// Initialize Apple Grayscale Appearance globals and defaults.
// If either parameter is true and the corresponding extension is installed,
// the AGA classes will allow their controls to be drawn by the system.
//----------------------------------------------------------------------------------------

void InitAGA_AC(bool appearanceMgrSavvy, bool aaronOrKaleidoscopeSavvy)
{
	SInt32	gestaltResponse;
	bool 	hasAppearanceMgr = false;
	
#if qWantsAppearance
	// Check for presence of the Appearance Manager.
	// Note that we also test for compatibility mode and honor that.
	OSErr result = ::Gestalt('appr', &gestaltResponse);
	hasAppearanceMgr = (result == noErr) && 
						((gestaltResponse & 0x00000001) != 0) &&
						((gestaltResponse & 0x00000002) == 0);
#endif
	
	// Check for presence of the Aaron or Kaleidoscope.
	bool hasAaronOrKaleidoscope = (
		(::Gestalt('Aarn', &gestaltResponse) == noErr) ||
		(::Gestalt('Copl', &gestaltResponse) == noErr) ||
		(::Gestalt('Acid', &gestaltResponse) == noErr)
		);

	// Now init our global switches based on these things and the
	// client's wishes.
	
	if ((hasAppearanceMgr && appearanceMgrSavvy) ||
		(hasAaronOrKaleidoscope && aaronOrKaleidoscopeSavvy))
	{
		pEmulateControls = false;
	}
		
	// Defaults.  Only live scrolling and dim deactivation are on by default.
	SetAGAFeatureFlag_AC(kAGALiveScrolling_AC, true);
	SetAGAFeatureFlag_AC(kAGADimDeactivation_AC, true);

#if qWantsAppearance
	if (hasAppearanceMgr)
	{
		OSStatus err = ::RegisterAppearanceClient();
		::ThrowIfOSErr_AC((OSErr) err);
	}
#endif
	pHasAppearanceMgr = hasAppearanceMgr;
}

//----------------------------------------------------------------------------------------
// HasAppearanceMgr_AC
// Return whether to emulate controls (false means use system controls)
//----------------------------------------------------------------------------------------

bool HasAppearanceMgr_AC()
{
	return pHasAppearanceMgr;
}

//----------------------------------------------------------------------------------------
// EmulateControls_AC
// Return whether to emulate controls (false means use system controls)
//----------------------------------------------------------------------------------------

bool EmulateControls_AC()
{
	return pEmulateControls;
}

//----------------------------------------------------------------------------------------
// SetEmulateControls_AC
// Set whether emulating controls (only works for creating new controls after this call) 
//----------------------------------------------------------------------------------------

void SetEmulateControls_AC(bool emulate)
{
	pEmulateControls = emulate;
}

//----------------------------------------------------------------------------------------
// SetAGAFeatureFlag_AC
// Set AGA feature flag
//----------------------------------------------------------------------------------------

extern void SetAGAFeatureFlag_AC(UInt32 itemMask, bool turnOn)
{
	if (turnOn)
		pAGAFeatures |= itemMask;
	else
		pAGAFeatures &= ~itemMask;
}

//----------------------------------------------------------------------------------------
// GetAGAFeatureFlag_AC
// Get AGA feature flag
//----------------------------------------------------------------------------------------

extern bool GetAGAFeatureFlag_AC(UInt32 itemMask)
{
	return ((pAGAFeatures & itemMask) != 0);
}

//----------------------------------------------------------------------------------------
// DelayFuture_AC
// Delay for the specified number of ticks.
//----------------------------------------------------------------------------------------

void DelayFuture_AC(SInt32 delayTicks)
{
	SInt32	dontCare;
	::Delay(delayTicks, &dontCare);
}

//----------------------------------------------------------------------------------------
// DelayFutureWhileStillDown_AC
// Delay for the specified number of ticks, but bail
// out if the user lets go of the mouse first.
//----------------------------------------------------------------------------------------

void DelayFutureWhileStillDown_AC(SInt32 delayTicks)
{
	SInt32	future = delayTicks + ::LMGetTicks();

	while (::StillDown() && (::LMGetTicks() < future))
		;
}
