// CButton_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1996 by Trygve Isaacson.

#ifndef __CButton_AC__
#include "CButton_AC.h"
#endif

// Appearance

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif


//========================================================================================
// CButton_AC
// This class implements a standard text pushbutton object.
//========================================================================================

//----------------------------------------------------------------------------------------
// CButton_AC::CButton_AC
//----------------------------------------------------------------------------------------

CButton_AC::CButton_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, const CStr255_AC& title) : 
	CControl_AC(bounds),
	fIsDefault(false),
	fFrameInside(true),
	fTitle(title),
	fTextStyle(textStyle)
{
	// Construct with specified button title.
}

//----------------------------------------------------------------------------------------
// CButton_AC::CButton_AC
//----------------------------------------------------------------------------------------

CButton_AC::CButton_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt16 stringListResourceID, SInt16 stringIndex) :
	CControl_AC(bounds),
	fIsDefault(false),
	fFrameInside(true),
	fTitle(),
	fTextStyle(textStyle)
{
	// Construct by getting button title from specified resource.
	::GetIndString(fTitle, stringListResourceID, stringIndex);
}

//----------------------------------------------------------------------------------------
// CButton_AC::~CButton_AC
//----------------------------------------------------------------------------------------

CButton_AC::~CButton_AC()
{
}

//----------------------------------------------------------------------------------------
// CButton_AC::DrawObject
//----------------------------------------------------------------------------------------

void CButton_AC::DrawObject()
{
	// Draw button in normal unpressed state.

	this->DrawButton(kNotPressed);
}

//----------------------------------------------------------------------------------------
// CButton_AC::SetTitle
//----------------------------------------------------------------------------------------

void CButton_AC::SetTitle(const CStr255_AC& title, bool redraw)
{
	// Change button title, redraw if specified.

	fTitle = title;
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CButton_AC::SetDefault
//----------------------------------------------------------------------------------------

void CButton_AC::SetDefault(bool isDefault, bool frameInside)
{
	// Set default state. If fIsDefault is true, the button will
	// be drawn with a default button outline. If fFrameInside
	// is true, the outline will be inside GetBounds() and the button
	// will be drawn inset from there; if fFrameInside is false,
	// the outline will be outside GetBounds() and the button will
	// be drawn at GetBounds().

	fIsDefault = isDefault;
	fFrameInside = frameInside;
}

//----------------------------------------------------------------------------------------
// CButton_AC::IsDefault
//----------------------------------------------------------------------------------------

bool CButton_AC::IsDefault()
{
	// Return true if this button has a default outline.

	return fIsDefault;
}

//----------------------------------------------------------------------------------------
// CButton_AC::DrawButton
//----------------------------------------------------------------------------------------

void CButton_AC::DrawButton(bool pressed)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the button in the appropriate state.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (! fEnabled || ! fActive)
			this->DrawButtonDisabled(deep);
		else if (pressed)
			this->DrawButtonPressed(deep);
		else
			this->DrawButtonNormal(deep);
	}
}

//----------------------------------------------------------------------------------------
// CButton_AC::HitTest
//----------------------------------------------------------------------------------------

bool CButton_AC::HitTest(const CPoint_AC& mouseLocation)
{
	// Return true if the mouse is in the button.
	// Account for case where default button outline
	// is inside GetBounds().

	CRect_AC	r(GetBounds());
	
	if (fIsDefault && fFrameInside)
		r.Inset(3, 3);

	return r.Contains(mouseLocation);
}

//----------------------------------------------------------------------------------------
// CButton_AC::SetTrackingState
//----------------------------------------------------------------------------------------

void CButton_AC::SetTrackingState(bool isIn)
{
	// Draw the button pressed or unpressed as specified.

	this->DrawButton(isIn);
}

//----------------------------------------------------------------------------------------
// CButton_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CButton_AC::DrawButtonNormal(bool deep)
{
	// Draw the button in enabled/unpressed state.

	CRect_AC	r(GetBounds());
	
	if (fIsDefault && fFrameInside)
		r.Inset(3, 3);
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r2].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(r.right - r.left - 6, 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, r.bottom - r.top - 6);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(- (r.right - r.left - 6), 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, -(r.bottom - r.top - 6));
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		
		// Add 1st inner pixels.
		::MoveTo(r.left + 1, r.top + 2);
		kRGBColorRamp[r4].SetForeColor();
		::Line(0, 0);
		::Move(1, -1);
		::Line(0, 0);
		::Move(r.right - r.left - 5, 0);
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[r5].SetForeColor();
		::Line(0, 0);
		::Move(0, 1);
		kRGBColorRamp[r8].SetForeColor();
		::Line(0, r.bottom - r.top - 6);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(- (r.right - r.left - 6), 0);
		kRGBColorRamp[r4].SetForeColor();
		::Move(-1, 0);
		::Line(-1, -1);

		// Add 2nd inner pixels.
		::MoveTo(r.left + 2, r.bottom - 4);
		kRGBColorRamp[rW].SetForeColor();
		::Line(0, -(r.bottom - r.top - 6));
		::Line(1, 1);
		::Line(0, -1);
		::Line(r.right - r.left - 7, 0);
		::Move(1, 1);
		kRGBColorRamp[r5].SetForeColor();
		::Line(0, r.bottom - r.top - 7);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(- (r.right - r.left - 7), 0);
	}
	else	// 1-bit
	{
		kRGBColorRamp[rW].SetForeColor();
		::PaintRoundRect(&r, 7, 7);
		kRGBColorRamp[rB].SetForeColor();
		::FrameRoundRect(&r, 7, 7);
	}

	// Draw the title.
	AGAStringOut(fTitle, r, truncMiddle, teJustCenter, kNormalOutput, deep, fTextStyle);
	
	// Draw the default button outline.
	this->DrawOutlineNormal(deep);
}

//----------------------------------------------------------------------------------------
// CButton_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CButton_AC::DrawButtonPressed(bool deep)
{
	// Draw the button in enabled/pressed state.

	CRect_AC	r(GetBounds());
	
	if (fIsDefault && fFrameInside)
		r.Inset(3, 3);
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r9].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(r.right - r.left - 6, 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, r.bottom - r.top - 6);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(- (r.right - r.left - 6), 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, -(r.bottom - r.top - 6));
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		
		// Add 1st inner pixels.
		::MoveTo(r.left + 1, r.bottom - 3);
		kRGBColorRamp[r11].SetForeColor();
		::Line(0, -(r.bottom - r.top - 5));
		::Line(1, 0);
		::Line(0, -1);
		::Line(r.right - r.left - 5, 0);
		::Move(1, 1);
		kRGBColorRamp[r8].SetForeColor();
		::Line(0, 0);
		::Move(0, 1);
		kRGBColorRamp[r7].SetForeColor();
		::Line(0, r.bottom - r.top - 6);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(- (r.right - r.left - 5), 0);
		kRGBColorRamp[r8].SetForeColor();
		::Line(0, 0);

		// Add 2nd inner pixels.
		::MoveTo(r.left + 2, r.bottom - 4);
		kRGBColorRamp[r10].SetForeColor();
		::Line(0, -(r.bottom - r.top - 7));
		::Line(1, 0);
		::Line(0, -1);
		::Line(r.right - r.left - 7, 0);
		::Move(1, 1);
		kRGBColorRamp[r8].SetForeColor();
		::Line(0, r.bottom - r.top - 7);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(- (r.right - r.left - 7), 0);
	}
	else	// 1-bit
	{
		kRGBColorRamp[rB].SetForeColor();
		::PaintRoundRect(&r, 7, 7);
	}

	// Draw the title.
	AGAStringOut(fTitle, r, truncMiddle, teJustCenter, kInverseOutput, deep, fTextStyle);
	
	// Draw the default button outline.
	this->DrawOutlineNormal(deep);
}

//----------------------------------------------------------------------------------------
// CButton_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CButton_AC::DrawButtonDisabled(bool deep)
{
	// Draw the button in disabled/unpressed state.

	CRect_AC	r(GetBounds());
	
	if (fIsDefault && fFrameInside)
		r.Inset(3, 3);
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r2].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 1);
		kRGBColorRamp[r7].SetForeColor();
		::Line(0, 0);
		::Move(1, -1);
		::Line(0, 0);
		::Move(1, 0);
		::Line(r.right - r.left - 6, 0);
		::Line(0, 0);
		::Move(1, 1);
		::Line(0, 0);
		::Move(1, 1);
		::Line(0, 0);
		::Move(0, 1);
		::Line(0, r.bottom - r.top - 6);
		::Line(0, 0);
		::Move(-1, 1);
		::Line(0, 0);
		::Move(-1, 1);
		::Line(0, 0);
		::Move(-1, 0);
		::Line(- (r.right - r.left - 6), 0);
		::Line(0, 0);
		::Move(-1, -1);
		::Line(0, 0);
		::Move(-1, -1);
		::Line(0, 0);
		::Move(0, -1);
		::Line(0, -(r.bottom - r.top - 6));
		::Line(0, 0);
	}
	else	// 1-bit
	{
		kRGBColorRamp[rW].SetForeColor();
		::PaintRoundRect(&r, 7, 7);
		kRGBColorRamp[rB].SetForeColor();
		::FrameRoundRect(&r, 7, 7);
	}

	// Draw the title.
	AGAStringOut(fTitle, r, truncMiddle, teJustCenter, kDisabledOutput, deep, fTextStyle);
	
	// Draw the default button outline.
	this->DrawOutlineDisabled(deep);
	
	if (! deep)
	{
		if (fIsDefault)
			r.Inset(-3, -3);

		kRGBColorRamp[rB].SetForeColor();
		PenPatGray();
		::PenMode(patBic);
		::PaintRect(&r);

		::PenNormal();
	}
}

//----------------------------------------------------------------------------------------
// CButton_AC::DrawOutlineNormal
//----------------------------------------------------------------------------------------

void CButton_AC::DrawOutlineNormal(bool deep)
	{
	// Draw the default button outline in normal
	// state if the button has one. It may be either
	// inside or outside GetBounds().
	
	// If we are drawing the outline outside the normal
	// bounds, we should force the clipping to include
	// that outside area, because the framework or whoever
	// may have reduced clipping to our bounds.

	if (! fIsDefault)
		return;

	CRect_AC	r(GetBounds());
	
	if (! fFrameInside)
		r.Inset(-3, -3);

	if (deep)
	{
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 2);
		kRGBColorRamp[rB].SetForeColor();
		::Line(1, -1);
		::Move(1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(r.right - r.left - 8, 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(1, 1);
		::Move(1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, r.bottom - r.top - 8);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(-1, 1);
		::Move(-1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(- (r.right - r.left - 8), 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(-1, -1);
		::Move(-1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, -(r.bottom - r.top - 8));
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		
		// Draw the interior.
		::MoveTo(r.left + 1, r.bottom - 4);
		kRGBColorRamp[r2].SetForeColor();
		::Line(0, -(r.bottom - r.top - 7));
		::Line(1, 0);
		::Line(0, -1);
		::Line(1, 0);
		::Line(0, -1);
		::Line(r.right - r.left - 8, 0);
		::Move(1, 0);
		kRGBColorRamp[r3].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[r4].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[r7].SetForeColor();
		::Line(0, 1);
		kRGBColorRamp[r8].SetForeColor();
		::Line(0, r.bottom - r.top - 8);
		::Line(-2, 2);
		::Line(-(r.right - r.left - 8), 0);
		::Move(-1, 0);
		kRGBColorRamp[r7].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[r4].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[r3].SetForeColor();
		::Line(0, 0);
		::Move(1, 0);
		kRGBColorRamp[r5].SetForeColor();
		::Line(0, -(r.bottom - r.top - 8));
		::Line(2, -2);
		::Line(r.right - r.left - 8, 0);
		::Line(0, 1);
		::Line(1, 0);
		::Line(0, r.bottom - r.top - 8);
		::Line(-2, 2);
		::Line(-(r.right - r.left - 8), 0);
		::Line(0, -1);
		::Move(0, -1);
		kRGBColorRamp[r8].SetForeColor();
		::Line(1, 1);
		::MoveTo(r.left + 3, r.top + 4);
		::Line(1, -1);
		::MoveTo(r.right -5, r.top + 3);
		::Line(1, 1);
		::MoveTo(r.right -5, r.bottom - 4);
		::Line(1, -1);
		::Move(1, 1);
		kRGBColorRamp[r7].SetForeColor();
		::Line(-1, 1);
	}
	else
	{
		kRGBColorRamp[rB].SetForeColor();
		::PenSize(2, 2);
		::FrameRoundRect(&r, 11, 11);
		::PenSize(1, 1);
	}
}

//----------------------------------------------------------------------------------------
// CButton_AC::DrawOutlineDisabled
//----------------------------------------------------------------------------------------

void CButton_AC::DrawOutlineDisabled(bool deep)
{
	// Draw the default button outline in disabled
	// state if the button has one. It may be either
	// inside or outside GetBounds().

	if (! fIsDefault)
		return;

	CRect_AC	r(GetBounds());

	if (! fFrameInside)
		r.Inset(-3, -3);

	if (deep)
	{
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 2);
		kRGBColorRamp[r7].SetForeColor();
		::Line(1, -1);
		::Move(1, -1);
		::Line(0, 0);
		::Move(1, 0);
		::Line(r.right - r.left - 8, 0);
		::Line(0, 0);
		::Move(1, 1);
		::Line(1, 1);
		::Move(1, 1);
		::Line(0, 0);
		::Move(0, 1);
		::Line(0, r.bottom - r.top - 8);
		::Line(0, 0);
		::Move(-1, 1);
		::Line(-1, 1);
		::Move(-1, 1);
		::Line(0, 0);
		::Move(-1, 0);
		::Line(- (r.right - r.left - 8), 0);
		::Line(0, 0);
		::Move(-1, -1);
		::Line(-1, -1);
		::Move(-1, -1);
		::Line(0, 0);
		::Move(0, -1);
		::Line(0, -(r.bottom - r.top - 8));
		::Line(0, 0);
		
		// Draw the interior.
		::MoveTo(r.left + 1, r.bottom - 4);
		kRGBColorRamp[r4].SetForeColor();
		::Line(0, -(r.bottom - r.top - 7));
		::Line(1, 0);
		::Line(0, -1);
		::Line(1, 0);
		::Line(0, -1);
		::Line(r.right - r.left - 8, 0);
		::Move(3, 2);
		::Line(0, r.bottom - r.top - 7);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(-(r.right - r.left - 7), 0);
		::Move(-2, -2);
		::Line(1, 1);
		::Line(0, -(r.bottom - r.top - 7));
		::Line(1, 0);
		::Line(0, -1);
		::Line(1, 0);
		::Line(0, -1);
		::Line(r.right - r.left - 7, 0);
		::Line(-1, -1);
		::Move(-1, 2);
		::Line(2, 0);
		::Move(-1, 1);
		::Line(1, 0);
		::Line(0, r.bottom - r.top - 9);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(-1, 0);
		::Line(0, 1);
		::Line(-(r.right - r.left - 8), 0);
		::Line(0, -2);
		::Line(1, 1);
	}
	else
	{
		kRGBColorRamp[rB].SetForeColor();
		::PenSize(2, 2);
		::FrameRoundRect(&r, 11, 11);
		::PenSize(1, 1);
	}
}
	
