// CCheckBox_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CCheckBox_AC__
#include "CCheckBox_AC.h"
#endif

// Appearance

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

//========================================================================================
// CCheckBox_AC
// This class implements a standard checkbox object with the additional
// capability of supporting mixed state.
//========================================================================================


//----------------------------------------------------------------------------------------
// CCheckBox_AC::CCheckBox_AC
//----------------------------------------------------------------------------------------

CCheckBox_AC::CCheckBox_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, const CStr255_AC& title, bool automaticState) : 
	CControl_AC(bounds),
	fValue(kCheckBoxOff),
	fAutomaticState(automaticState),
	fTitle(title),
	fTextStyle(textStyle)
{
	// Construct with specified button title.
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::CCheckBox_AC
//----------------------------------------------------------------------------------------

CCheckBox_AC::CCheckBox_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt16 stringListResourceID, SInt16 stringIndex, bool automaticState) : 
	CControl_AC(bounds),
	fValue(kCheckBoxOff),
	fAutomaticState(automaticState),
	fTitle(),
	fTextStyle(textStyle)
{
	// Construct by getting button title from specified resource.
	::GetIndString(fTitle, stringListResourceID, stringIndex);
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::~CCheckBox_AC
//----------------------------------------------------------------------------------------

CCheckBox_AC::~CCheckBox_AC()
{
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::DrawObject
//----------------------------------------------------------------------------------------

void CCheckBox_AC::DrawObject()
{
	// Draw button in normal unpressed state.

	this->DrawButton(kNotPressed);
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::TrackMouse
//----------------------------------------------------------------------------------------

bool CCheckBox_AC::TrackMouse(const CPoint_AC& mouseLocation)
{
	// If inherited tracking succeeded and we are set to
	// automatically change our state, toggle it.

	bool	wasItHit = CControl_AC::TrackMouse(mouseLocation);

	if (wasItHit && fAutomaticState)
		this->SetValue((GetValue() == kCheckBoxOn) ? kCheckBoxOff : kCheckBoxOn, kRedraw);

	return wasItHit;
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::SetTitle
//----------------------------------------------------------------------------------------

void CCheckBox_AC::SetTitle(const CStr255_AC& title, bool redraw)
{
	// Change button title, redraw if specified.

	fTitle = title;
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::GetValue
//----------------------------------------------------------------------------------------

SInt32 CCheckBox_AC::GetValue() const
{
	// Return current state.

	return fValue;
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::SetValue
//----------------------------------------------------------------------------------------

void CCheckBox_AC::SetValue(SInt32 newValue, bool redraw)
{
	// Set current state, redraw if specified and necessary.

	if (newValue != GetValue())
	{
		fValue = newValue;
		
		if (redraw)
		{
			this->DrawButton(kNotPressed);
		}
	}
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::DrawButton
//----------------------------------------------------------------------------------------

void CCheckBox_AC::DrawButton(bool pressed)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the button in the appropriate state.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (! fEnabled || ! fActive)
			this->DrawButtonDisabled(deep);
		else if (pressed)
			this->DrawButtonPressed(deep);
		else
			this->DrawButtonNormal(deep);
	}
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::SetTrackingState
//----------------------------------------------------------------------------------------

void CCheckBox_AC::SetTrackingState(bool isIn)
{
	// Draw the button pressed or unpressed as specified.

	this->DrawButton(isIn);
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CCheckBox_AC::DrawButtonNormal(bool deep)
	{
	// Draw the button in enabled/unpressed state.

	CRect_AC	r(GetBounds());
	
	r.Offset(2, 2);	// visually match the system CDEF

	r.bottom = r.top + kCheckBoxSize;
	r.right = r.left + kCheckBoxSize;
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r2].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		kRGBColorRamp[rB].SetForeColor();
		::FrameRect(&r);
		
		// Draw the edges.
		::MoveTo(r.left + 1, r.bottom - 3);
		kRGBColorRamp[rW].SetForeColor();
		::Line(0, -8);
		::Line(8, 0);
		::Move(1, 1);
		kRGBColorRamp[r7].SetForeColor();
		::Line(0, 8);
		::Line(-8, 0);
		
		if (GetValue() == kCheckBoxOn)
		{
			// Draw the X and its shadow.
			::MoveTo(r.left + 3, r.top + 2);
			kRGBColorRamp[rB].SetForeColor();
			::Line(5, 5);
			::Move(0, 1);
			::Line(-5, -5);
			::Move(0, 4);
			::Line(5, -5);
			::Move(0, 1);
			::Line(-5, 5);
			::Move(1, 0);
			kRGBColorRamp[r8].SetForeColor();
			::Line(1, -1);
			::Move(2, -2);
			::Line(2, -2);
			::Move(0, 1);
			kRGBColorRamp[r5].SetForeColor();
			::Line(-1, 1);
			::Move(-2, 2);
			::Line(-2, 2);
			::Move(5, 0);
			::Line(0, 0);
			::Move(0, -1);
			kRGBColorRamp[r8].SetForeColor();
			::Line(0, 0);
		}
		else if (GetValue() == kCheckBoxMixed)
		{
			// Draw the dash and its shadow.
			::MoveTo(r.left + 3, r.top + 5);
			::PenSize(2, 2);
			kRGBColorRamp[rB].SetForeColor();
			::Line(4, 0);
			
			::MoveTo(r.left + 4, r.top + 7);
			::PenSize(1, 1);
			kRGBColorRamp[r5].SetForeColor();
			::Line(5, 0);
			::Line(-2, 0);
			}
		}
	else
	{
		// Paint white over previous junk, if any.
		kRGBColorRamp[rW].SetForeColor();
		::PaintRect(&r);
		// Paint frame.
		kRGBColorRamp[rB].SetForeColor();
		::FrameRect(&r);

		// Draw check or dash.
		if (GetValue() == kCheckBoxOn)
		{
			::PenSize(1, 1);
			::MoveTo(r.left, r.top);
			::Line(kCheckBoxSize-1, kCheckBoxSize-1);
			::Move(-(kCheckBoxSize-1), 0);
			::Line(kCheckBoxSize-1, -(kCheckBoxSize-1));
		}
		else if (GetValue() == kCheckBoxMixed)
		{
			::PenSize(1, 2);
			::MoveTo(r.left + 3, r.top + 5);
			::Line(5, 0);
		}
		
		::PenNormal();
	}

	// Draw the title.
	r = GetBounds();

	r.Offset(2, 2);	// visually match the system CDEF

	r.left += (kCheckBoxSize + 4);
	r.bottom = r.top + kCheckBoxSize;
	AGAStringOut(fTitle, r, truncMiddle, teJustLeft, kNormalOutput, deep, fTextStyle);
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CCheckBox_AC::DrawButtonPressed(bool deep)
{
	// Draw the button in enabled/pressed state.

	CRect_AC	r(GetBounds());
	
	r.Offset(2, 2);	// visually match the system CDEF

	r.bottom = r.top + kCheckBoxSize;
	r.right = r.left + kCheckBoxSize;
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r8].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		kRGBColorRamp[rB].SetForeColor();
		::FrameRect(&r);
		
		// Draw the edges.
		::MoveTo(r.left + 1, r.bottom - 3);
		kRGBColorRamp[r10].SetForeColor();
		::Line(0, -8);
		::Line(8, 0);
		::Move(1, 1);
		kRGBColorRamp[r6].SetForeColor();
		::Line(0, 8);
		::Line(-8, 0);
		
		if (GetValue() == kCheckBoxOn)
		{
			// Draw the X and its shadow.
			::MoveTo(r.left + 3, r.top + 2);
			kRGBColorRamp[rB].SetForeColor();
			::Line(5, 5);
			::Move(0, 1);
			::Line(-5, -5);
			::Move(0, 4);
			::Line(5, -5);
			::Move(0, 1);
			::Line(-5, 5);
			::Move(1, 0);
			kRGBColorRamp[r11].SetForeColor();
			::Line(1, -1);
			::Move(2, -2);
			::Line(2, -2);
			::Move(0, 1);
			kRGBColorRamp[r10].SetForeColor();
			::Line(-1, 1);
			::Move(-2, 2);
			::Line(-2, 2);
			::Move(5, 0);
			::Line(0, 0);
			::Move(0, -1);
			kRGBColorRamp[r11].SetForeColor();
			::Line(0, 0);
		}
		else if (GetValue() == kCheckBoxMixed)
		{
			// Draw the dash and its shadow.
			::MoveTo(r.left + 3, r.top + 5);
			::PenSize(2, 2);
			kRGBColorRamp[rB].SetForeColor();
			::Line(4, 0);
			
			::MoveTo(r.left + 4, r.top + 7);
			::PenSize(1, 1);
			kRGBColorRamp[r10].SetForeColor();
			::Line(5, 0);
			::Line(-2, 0);
		}
	}
	else
	{
		// Paint white over previous junk, if any.
		kRGBColorRamp[rW].SetForeColor();
		::PaintRect(&r);
		// Paint frame.
		kRGBColorRamp[rB].SetForeColor();
		::PenSize(2, 2);
		::FrameRect(&r);

		// Draw check or dash.
		if (GetValue() == kCheckBoxOn)
		{
			::PenSize(1, 1);
			::MoveTo(r.left, r.top);
			::Line(kCheckBoxSize-1, kCheckBoxSize-1);
			::Move(-(kCheckBoxSize-1), 0);
			::Line(kCheckBoxSize-1, -(kCheckBoxSize-1));
		}
		else if (GetValue() == kCheckBoxMixed)
		{
			::PenSize(1, 2);
			::MoveTo(r.left + 3, r.top + 5);
			::Line(5, 0);
		}
		
		::PenNormal();
	}

	// Draw the title.
	r = GetBounds();

	r.Offset(2, 2);	// visually match the system CDEF

	r.left += (kCheckBoxSize + 4);
	r.bottom = r.top + kCheckBoxSize;
	AGAStringOut(fTitle, r, truncMiddle, teJustLeft, kNormalOutput, deep, fTextStyle);
}

//----------------------------------------------------------------------------------------
// CCheckBox_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CCheckBox_AC::DrawButtonDisabled(bool deep)
{
	// Draw the button in disabled/unpressed state.

	CRect_AC	r(GetBounds());
	
	r.Offset(2, 2);	// visually match the system CDEF

	r.bottom = r.top + kCheckBoxSize;
	r.right = r.left + kCheckBoxSize;
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r2].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		kRGBColorRamp[r7].SetForeColor();
		::FrameRect(&r);

		if (GetValue() == kCheckBoxOn)
		{
			// Draw the X.
			::PenSize(1, 2);
			::MoveTo(r.left + 3, r.top + 2);
			kRGBColorRamp[r7].SetForeColor();
			::Line(5, 5);
			::Move(-5, 0);
			::Line(5, -5);
			::Move(0, 1);
		}
		else if (GetValue() == kCheckBoxMixed)
		{
			// Draw the dash.
			::MoveTo(r.left + 3, r.top + 5);
			::PenSize(2, 2);
			kRGBColorRamp[r7].SetForeColor();
			::Line(4, 0);
			}
		}
	else
	{
		// Paint white over previous junk, if any.
		kRGBColorRamp[rW].SetForeColor();
		::PaintRect(&r);

		// Paint frame.
		PenPatGray();
		kRGBColorRamp[rB].SetForeColor();
		::FrameRect(&r);

		// Draw check or dash.
		if (GetValue() == kCheckBoxOn)
		{
			PenPatBlack();	// 1-pixel diagonals can disappear when qd.gray
			::MoveTo(r.left + 2, r.top + 2);
			::Line(kCheckBoxSize-5, kCheckBoxSize-5);
			::Move(-(kCheckBoxSize-5), 0);
			::Line(kCheckBoxSize-5, -(kCheckBoxSize-5));
		}
		else if (GetValue() == kCheckBoxMixed)
		{
			::PenSize(1, 2);
			::MoveTo(r.left + 3, r.top + 5);
			::Line(5, 0);
		}
		
		::PenNormal();
	}

	// Draw the title.
	r = GetBounds();

	r.Offset(2, 2);	// visually match the system CDEF

	r.left += (kCheckBoxSize + 4);
	r.bottom = r.top + kCheckBoxSize;
	AGAStringOut(fTitle, r, truncMiddle, teJustLeft, kDisabledOutput, deep, fTextStyle);
}

