// CCluster_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CCluster_AC__
#include "CCluster_AC.h"
#endif

// Appearance

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

//========================================================================================
// CCluster_AC
// This class draws primary and secondary group boxes, with optional
// title label or title gap for a separate control.
//========================================================================================


//----------------------------------------------------------------------------------------
// CCluster_AC::CCluster_AC
//----------------------------------------------------------------------------------------

CCluster_AC::CCluster_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt32 justification, bool groupBoxType, const CStr255_AC& titleString, SInt16 titleGap) :
	CControl_AC(bounds),
	fIsPrimaryBox(groupBoxType),
	fTitleGap(titleGap),
	fJustification(justification),
	fTitle(titleString),
	fTextStyle(textStyle)
{
	// Construct with specified title.
}

//----------------------------------------------------------------------------------------
// CCluster_AC::CCluster_AC
//----------------------------------------------------------------------------------------

CCluster_AC::CCluster_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt32 justification, bool groupBoxType, SInt16 titleStringListResourceID, SInt16 titleStringIndex, SInt16 titleGap) :
	CControl_AC(bounds),
	fIsPrimaryBox(groupBoxType),
	fTitleGap(titleGap),
	fJustification(justification),
	fTextStyle(textStyle)
{
	// Construct by getting title from specified resource.
	::GetIndString(fTitle, titleStringListResourceID, titleStringIndex);
}

//----------------------------------------------------------------------------------------
// CCluster_AC::~CCluster_AC
//----------------------------------------------------------------------------------------

CCluster_AC::~CCluster_AC()
{
}

//----------------------------------------------------------------------------------------
// CCluster_AC::SetTitleGap
//----------------------------------------------------------------------------------------

void CCluster_AC::SetTitleGap(SInt16 titleGap)
{
	// Set the width of the title gap that will be left blank.

	fTitleGap = titleGap;
}

//----------------------------------------------------------------------------------------
// CCluster_AC::SetTitle
//----------------------------------------------------------------------------------------

void CCluster_AC::SetTitle(const CStr255_AC& newTitle, bool redraw)
{
	// Set the title, optionally redraw.

	fTitle = newTitle;

	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CCluster_AC::DrawObject
//----------------------------------------------------------------------------------------

void CCluster_AC::DrawObject()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the group box.

	CRect_AC	r(GetBounds());
	SInt16		titleGap = 0;
	SInt16		topIndent = 0;
	FontInfo	fontInfo;

	if (!fTitle.IsEmpty())
	{
		fTextStyle.SetPortTextStyle();	// calculate gap for title's text style
		::GetFontInfo(&fontInfo);
		
		topIndent = fontInfo.ascent;	// text baseline should be at frame top
		
		titleGap = 5 + ::StringWidth(fTitle);
	}
	else
		titleGap = fTitleGap;

	if (fIsPrimaryBox)
		this->DrawPrimaryFrame(topIndent, titleGap);
	else
		this->DrawSecondaryFrame(topIndent, titleGap);

	if (!fTitle.IsEmpty())
	{
		CRect_AC	titleRect;
		
		titleRect.left = r.left + 4 + this->GetGapLeftInset(titleGap);
		titleRect.top = r.top + 2;
		titleRect.right = titleRect.left + titleGap - 5;
		titleRect.bottom = titleRect.top + fontInfo.ascent + fontInfo.descent;

		CGDIterator_AC	iter;
		bool			deep;

		while (iter.NextDevice(deep))
			AGAStringOut(fTitle, titleRect, kNoTruncation, fJustification, (fEnabled && fActive) ? kNormalOutput : kDisabledOutput, deep, fTextStyle);
	}
}

//----------------------------------------------------------------------------------------
// CCluster_AC::DrawPrimaryFrame
//----------------------------------------------------------------------------------------

void CCluster_AC::DrawPrimaryFrame(SInt16 topIndent, SInt16 gapSize)
{
	// Draw the group box as a primary group.

	CRect_AC	r(GetBounds());
	
	r.top += topIndent;

	CGDIterator_AC	iter;
	bool			deep;
	bool			drawEnabled = fEnabled && fActive;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			if (drawEnabled)
				kRGBColorRamp[r7].SetForeColor();
			else
				kRGBColorRamp[r4].SetForeColor();
		}
		else
		{
			kRGBColorRamp[rB].SetForeColor();
			
			if (! drawEnabled)
				PenPatGray();
		}

		::MoveTo(r.right - 2, r.top);
		::LineTo(r.right - 2, r.bottom - 2);
		::LineTo(r.left, r.bottom - 2);
		::LineTo(r.left, r.top);
		::Line(10, 0);
		::Move(gapSize, 0);
		::LineTo(r.right - 2, r.top);
		
		if (deep)
		{
			if (drawEnabled)
				kRGBColorRamp[rW].SetForeColor();
			else
				this->ApplyBackgroundColor();

			::MoveTo(r.right - 1, r.top);
			::LineTo(r.right - 1, r.bottom - 1);
			::LineTo(r.left, r.bottom - 1);
			::MoveTo(r.left + 1, r.bottom - 3);
			::LineTo(r.left + 1, r.top + 1);
			::Line(9, 0);
			::Move(gapSize, 0);
			::LineTo(r.right - 3, r.top + 1);
		}
		else
			PenPatBlack();
	}
}

//----------------------------------------------------------------------------------------
// CCluster_AC::DrawSecondaryFrame
//----------------------------------------------------------------------------------------

void CCluster_AC::DrawSecondaryFrame(SInt16 topIndent, SInt16 gapSize)
{
	// Draw the group box as a secondary group.

	CRect_AC	r(GetBounds());
	
	r.top += topIndent;

	CGDIterator_AC	iter;
	bool			deep;
	bool			drawEnabled = fEnabled && fActive;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			::MoveTo(r.right - 1, r.top);

			if (drawEnabled)
			{
				kRGBColorRamp[rW].SetForeColor();
				::Move(0, 1);
			}
			else
				kRGBColorRamp[r4].SetForeColor();

			::LineTo(r.right - 1, r.bottom - 1);
			::LineTo(r.left + 1, r.bottom - 1);

			if (drawEnabled)
			{
				::Move(-1, -1);
				kRGBColorRamp[r7].SetForeColor();
			}
			else
				::Line(-1, 0);

			::LineTo(r.left, r.top);
			::Line(10, 0);
			::Move(gapSize, 0);
			::LineTo(r.right - 2, r.top);
		}
		else
		{
			kRGBColorRamp[rB].SetForeColor();
			
			if (! IsEnabled())
				PenPatGray();

			::MoveTo(r.right - 2, r.top);
			::LineTo(r.right - 2, r.bottom - 2);
			::LineTo(r.left, r.bottom - 2);
			::LineTo(r.left, r.top);
			::Line(10, 0);
			::Move(gapSize, 0);
			::LineTo(r.right - 2, r.top);
			
			if (! IsEnabled())
				PenPatBlack();
		}
	}
}

//----------------------------------------------------------------------------------------
// CCluster_AC::DrawSecondaryFrame
//----------------------------------------------------------------------------------------

SInt16 CCluster_AC::GetGapLeftInset(SInt16 gapSize)
{
	// Return the number of pixels from the left edge of the group box
	// to the start of the gap. This depends on the title justification,
	// because the gap is not necessarily a fixed number of pixels from
	// the left.
	
	SInt16	inset;
	
	switch (CTextStyle_AC::RuntimeJustify(fJustification))
	{
		case teCenter:
			inset = (fBounds.right - fBounds.left - gapSize - 12) / 2;
			break;
		case teFlushRight:
			inset = fBounds.right - fBounds.left - gapSize - 12;
			break;
		default: //case teFlushLeft:
			inset = 10;
			break;
	}
	
	return inset;
}


