//----------------------------------------------------------------------------------------
// CControl_AC.cp 
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1996 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

// ACS

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

//========================================================================================
// GLOBALS 
//========================================================================================

bool CControl_AC::fgUseSpecialBackgroundColors = false;	// toggled by tab panel installer
CRGBColor_AC CControl_AC::fgEnabledBackgroundColor;
CRGBColor_AC CControl_AC::fgDisabledBackgroundColor;

//========================================================================================
// CLASS CControl_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

CControl_AC::CControl_AC(const CRect_AC& bounds) :
	fBounds(bounds),
	fEnabled(true),
	fActive(true),
	fEnabledBackgroundColor(kRGBColorRamp[r2]),
	fDisabledBackgroundColor(kRGBColorRamp[r2])
{
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

CControl_AC::~CControl_AC()
{
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

CRect_AC CControl_AC::GetBounds() const
{
	// Return current bounds.

	return fBounds;
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetBounds(const CRect_AC& bounds, bool redraw)
{
	// Change bounds, redraw if specified.

	fBounds = bounds;
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::DrawObject()
{
	// Override to draw particular type of object.
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

bool CControl_AC::ContainsMouse(const CPoint_AC& mouseLocation)
{
	// Return true if mouse will hit object.

	return fBounds.Contains(mouseLocation);
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

bool CControl_AC::TrackMouse(const CPoint_AC& /*mouseLocation*/)
{
	if (!fEnabled)
		return false;

	// Process mouse tracking, return true if tracking
	// eventually "did something". Return false if
	// tracking had no ultimate effect, e.g. mouse
	// up outside a push button.

	bool		finishedIn = false;
	bool		wasIn = false;
	bool		isIn;
	bool		isButtonDown = true;	// ensure at least one time through
	CPoint_AC	newMouseLocation;
	
	while (isButtonDown)
	{
		::GetMouse(&newMouseLocation);	// gives local coordinates
		
		isIn = this->HitTest(newMouseLocation);
		
		if (isIn != wasIn)
			this->SetTrackingState(isIn);
		
		wasIn = isIn;

		isButtonDown = ::StillDown();
	}
	
	if (isIn)	// if finished IN, restore normal appearance
		this->SetTrackingState(kTrackingOut);
	
	return isIn;
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

bool CControl_AC::IsEnabled() const
{ 
	return fEnabled; 
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetEnable(bool isEnabled, bool redraw)
{
	// Set object enable state, redraw if specified.

	fEnabled = isEnabled;
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

bool CControl_AC::IsActive() const
{ 
	return fActive; 
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetActive(bool isActive, bool redraw)
{
	// Set object enable state, redraw if specified.
	bool	wasActive = fActive;

	fActive = isActive;
	
	if (redraw && (isActive != wasActive))
		this->DrawObject();
}


//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetBackgroundColors(const CRGBColor_AC& enabledColor, const CRGBColor_AC& disabledColor)
{
	// Set colors to be used when erasing background.
	// Most views don't need this. Default is AGA ramp r2.
	// CStaticText_AC and CDisclosureTriangle_AC need to
	// erase their background, so if your background is
	// not r2 (white, for example), call this to set it.
	// Also, objects contained in tab panel views need
	// r1 when enabled, and r2 when disabled, so that's
	// an even wackier case; the adapter classes handle
	// it for you.

	fEnabledBackgroundColor = enabledColor;
	fDisabledBackgroundColor = disabledColor;
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

SInt32 CControl_AC::GetValue() const
{
	return 0;	// overriden for objects that have a useful value
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetValue(SInt32 /*newValue*/, bool /*redraw*/)
{
	// overriden for objects that have a useful value
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

SInt32 CControl_AC::GetMinimum() const
{
	return 0;	// overriden for objects that have a useful value
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

SInt32 CControl_AC::GetMaximum() const
{
	return 0;	// overriden for objects that have a useful value
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetRange(SInt32 /*newMinimum*/, SInt32 /*newMaximum*/, bool /*redraw*/)
{
	// overriden for objects that have a useful value
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

CStr255_AC CControl_AC::GetTitle() const
{
	return kEmptyString_AC;
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetTitle(const CStr255_AC& /*itsTitle*/, bool /*redraw*/)
{
	// overriden for objects that have a useful value
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

bool CControl_AC::HitTest(const CPoint_AC& mouseLocation)
{
	// Return true if mouse is inside tracking area of
	// object. This function may be overridden for
	// classes that do not have simple rectangular
	// boundary hit testing.

	return fBounds.Contains(mouseLocation);
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::SetTrackingState(bool /*isIn*/)
{
	// Override to draw the object in the appropriate
	// state during tracking. If isIn is true, it means
	// that the mouse is now in the positive hit test
	// area of the object.
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::ApplyBackgroundColor()
{
	// Use the proper background color and apply it as
	// the current foreground color suitable for a
	// subsequent PaintRect or similar call.

	if (fEnabled && fActive)
		fEnabledBackgroundColor.SetForeColor();
	else
		fDisabledBackgroundColor.SetForeColor();
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::InstallBackgroundColors(bool installIfNotCustom)
{
	CRGBColor_AC	enabledColor;
	CRGBColor_AC	disabledColor;

	if (this->DetermineBackgroundColors(enabledColor, disabledColor, installIfNotCustom))
		this->SetBackgroundColors(enabledColor, disabledColor);
}

//----------------------------------------------------------------------------------------
// Variable-color background support
//
// Tab panels provide us a more complicated background color situation
// than other situations. While the normal AGA background is gray ramp
// value r2, the tab panels are lighter, namely r1. However, when a tab
// panel is dimmed, it becomes r2. Thus the panel subpanes can't just
// blindly use the window background color, and if the tab panel becomes
// dimmed they can't just blindly use r1 either.
//
// So what we do is maintain static color values that are picked up by
// any adapter classes that need to erase their background. There are
// separate color values for enabled and disabled states. When a panel
// view hierarchy is installed into a tab panel pane, the tab panel class
// temporarily sets these color values so that any subpanes of the panel
// will pick them up as they are instantiated.
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

bool CControl_AC::DetermineBackgroundColors(CRGBColor_AC& enabledColor, CRGBColor_AC& disabledColor, bool installIfNotCustom)
{
	bool	wereColorsSet = false;

	if (fgUseSpecialBackgroundColors)
	{
		enabledColor = fgEnabledBackgroundColor;
		disabledColor = fgDisabledBackgroundColor;
		wereColorsSet = true;
	}
	else if (installIfNotCustom)
	{
		// The calling view must have focused and set the port bg color.
		enabledColor.GetBackColor();
		disabledColor = enabledColor;
		wereColorsSet = true;
	}

	return wereColorsSet;
}
 
//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::TurnOnCustomBackgroundColors(const CRGBColor_AC& enabledColor, const CRGBColor_AC& disabledColor)
{
	fgUseSpecialBackgroundColors = true;
	fgEnabledBackgroundColor = enabledColor;
	fgDisabledBackgroundColor = disabledColor;
}

//----------------------------------------------------------------------------------------
// CControl_AC
//----------------------------------------------------------------------------------------

void CControl_AC::TurnOffCustomBackgroundColors()
{
	fgUseSpecialBackgroundColors = false;
}


