// CDialog_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1996 by Trygve Isaacson.

#ifndef __CDialog_AC__
#include "CDialog_AC.h"
#endif

// ACS

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

#ifndef __MemoryUtils_AC__
#include "MemoryUtils_AC.h"
#endif

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __ICONS__
#include <Icons.h>
#endif

//========================================================================================
// GLOBALS
//========================================================================================

static UserItemUPP gUserItemUPP = NewUserItemProc(CDialog_AC::UserItem);
static ModalFilterUPP gModalFilterUPP = NewModalFilterProc(CDialog_AC::ModalFilter);

//----------------------------------------------------------------------------------------
// DoAlert_AC
//----------------------------------------------------------------------------------------

static SInt16 DoAlert_AC(SInt16 alertID, ModalFilterUPP modalFilter, SInt16 iconID)
{
	// Run alert using the specified icon ID.

	CDialog_AC	CDialog_AC;

	return CDialog_AC.Alert(alertID, modalFilter, iconID);
}

//----------------------------------------------------------------------------------------
// Alert_AC
//----------------------------------------------------------------------------------------

SInt16 Alert_AC(SInt16 alertID, ModalFilterUPP modalFilter)
{
	// Run the alert without an icon.

	return DoAlert_AC(alertID, modalFilter, -1);
}

//----------------------------------------------------------------------------------------
// StopAlert_AC
//----------------------------------------------------------------------------------------

SInt16 StopAlert_AC(SInt16 alertID, ModalFilterUPP modalFilter)
{
	// Run the alert using the Stop icon.

	return DoAlert_AC(alertID, modalFilter, stopIcon);
}

//----------------------------------------------------------------------------------------
// NoteAlert_AC
//----------------------------------------------------------------------------------------

SInt16 NoteAlert_AC(SInt16 alertID, ModalFilterUPP modalFilter)
{
	// Run the alert using the Note icon.

	return DoAlert_AC(alertID, modalFilter, noteIcon);
}

//----------------------------------------------------------------------------------------
// CautionAlert_AC
//----------------------------------------------------------------------------------------

SInt16 CautionAlert_AC(SInt16 alertID, ModalFilterUPP modalFilter)
{
	// Run the alert using the Caution icon.

	return DoAlert_AC(alertID, modalFilter, cautionIcon);
}

//========================================================================================
// CLASS CDialog_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CDialog_AC
//----------------------------------------------------------------------------------------

CDialog_AC::CDialog_AC() :
	fMarker(kAGADialogMarker),
	fIsAlert(false),
	fDialogPtr(NULL),
	fNumAGAControls(0),
	fAGAControls((CustomObjectInfoHandle) ::NewHandle(0)),
	fUserFilter(NULL),
	fDefaultItemNo(-1),
	fCancelItemNo(-1),
	fDefaultButton(NULL),
	fCancelButton(NULL)
{
	// Construct with defaults.
}

//----------------------------------------------------------------------------------------
// ~CDialog_AC
//----------------------------------------------------------------------------------------

CDialog_AC::~CDialog_AC()
{
	// Destruct by freeing extra memory we allocated.

	this->DestroyAllCustomObjects();
	
	if (fAGAControls != NULL)
		::DisposeHandle((Handle) fAGAControls);
}

//----------------------------------------------------------------------------------------
// GetNewDialog
//----------------------------------------------------------------------------------------

DialogPtr CDialog_AC::GetNewDialog(SInt16 dialogID, Ptr dStorage, WindowPtr behind)
{
	// Wrapper for GetNewDialog.
	//
	// We get the DLOG resource to get the DITL ID and positioning info.
	// We make our own copy of the DITL data to give to the dialog,
	// create a color dialog, position it, and then install our custom
	// AGA objects.
	
	UInt8		savedHandleState;
	DialogTHndl	dlogHandle;
	Handle		ditlHandle = NULL;
	OSErr		err = memFullErr;	// assume the worst
	Rect		bounds;
	
	dlogHandle = (DialogTHndl) ::GetResource('DLOG', dialogID);

	if (dlogHandle != NULL)
	{
		// Release the DLOG and re-load it, to make sure we get a fresh copy.
		// If previously loaded by the DM, the bounds may have been wacked.
		::ReleaseResource((Handle) dlogHandle);
		dlogHandle = (DialogTHndl) ::GetResource('DLOG', dialogID);
	}

	if (dlogHandle != NULL)
	{
		savedHandleState = ::HGetState((Handle) dlogHandle);
		::HNoPurge((Handle) dlogHandle);
		ditlHandle = ::GetResource('DITL', (**dlogHandle).itemsID);
	}

	if (ditlHandle != NULL)
	{
		::HandToHand(&ditlHandle);
		err = ::MemError();
	}

	if (err != noErr)	// punt if DLOG or DITL not gotten or DITL not duplicated
		fDialogPtr = ::GetNewDialog(dialogID, dStorage, behind);
	else
	{
		CStr255_AC	title((**dlogHandle).title);
		bool	visible = (**dlogHandle).visible;
		SInt16	procID = (**dlogHandle).procID;
		bool	goAwayFlag = (**dlogHandle).goAwayFlag;
		SInt32	refCon = (**dlogHandle).refCon;
		
		bounds = (**dlogHandle).boundsRect;
		fDialogPtr = ::NewColorDialog(dStorage, &bounds, title, visible, procID, behind, goAwayFlag, refCon, ditlHandle);
	}

	if (dlogHandle != NULL)
		::HSetState((Handle) dlogHandle, savedHandleState);

	::ThrowIfNULL_AC(fDialogPtr);
	
	this->AdjustDialogPosition(dlogHandle, &bounds);
	this->BeginAGADialog();
	
	return fDialogPtr;
}

//----------------------------------------------------------------------------------------
// Alert
//----------------------------------------------------------------------------------------

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

typedef struct
{
	UInt32	reserved1;
	Rect	itemRect;
	UInt8	itemType;
	UInt8	reserved2;
	SInt16	iconID;
} IconItemData, *IconItemPtr;

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

SInt16 CDialog_AC::Alert(SInt16 alertID, ModalFilterUPP modalFilter, SInt16 iconID)
{
	// Wrapper for Alert and its variants.
	//
	// We get the ALRT resource to get the DITL ID and positioning info.
	// We make our own copy of the DITL data to give to the dialog,
	// create a color dialog, position it, and then install our custom
	// AGA objects.
	// Then we run the modal dialog right here before returning.
	
	SInt16		itemHit = -1;
	UInt8		savedHandleState;
	AlertTHndl	alrtHandle;
	Handle		ditlHandle = NULL;
	OSErr		err = memFullErr;	// assume the worst
	
	fIsAlert = true;

	alrtHandle = (AlertTHndl) ::GetResource('ALRT', alertID);

	if (alrtHandle != NULL)
	{
		// Release the ALRT and re-load it, to make sure we get a fresh copy.
		// If previously loaded by the DM, the bounds may have been wacked.
		::ReleaseResource((Handle) alrtHandle);
		alrtHandle = (AlertTHndl) ::GetResource('ALRT', alertID);
	}

	if (alrtHandle != NULL)
	{
		savedHandleState = ::HGetState((Handle) alrtHandle);
		::HNoPurge((Handle) alrtHandle);
		
		// !!! On my machine the Dialog Manager seems to ignore the CANCEL
		// flag in the alert stage info. To get identical behavior, I guess
		// I'll do the same. Need to investigate further, this is weird...
		
		// Peek into the ALRT stage info, which is 4 bytes of bit settings.
		//if (((**alrtHandle).stages & 0x0008) == 0)
			fDefaultItemNo = ok;
		//else
		//	fDefaultItemNo = cancel;
	
		// Read the DITL resource.
		ditlHandle = ::GetResource('DITL', (**alrtHandle).itemsID);
	}

	if (ditlHandle != NULL)
	{
		::HandToHand(&ditlHandle);
		err = ::MemError();
	}

	if ((err == noErr) && (iconID != -1))
	{
		// We need to expand the DITL handle and append a
		// disabled icon item to it.
		
		SInt32	oldHandleSize = ::GetHandleSize(ditlHandle);
		SInt32	newHandleSize = oldHandleSize + sizeof(IconItemData);
		
		::SetHandleSize(ditlHandle, newHandleSize);
		
		err = ::MemError();
		
		if (err == noErr)
		{
			(*((SInt16*) *ditlHandle))++;

			IconItemPtr	iconItemPtr = (IconItemPtr) ((*ditlHandle) + oldHandleSize);
			
			iconItemPtr->reserved1 = 0;
			iconItemPtr->itemRect.left = 18;
			iconItemPtr->itemRect.top = 9;
			iconItemPtr->itemRect.right = 50;
			iconItemPtr->itemRect.bottom = 41;
			iconItemPtr->itemType = kIconDialogItem | itemDisable;
			iconItemPtr->reserved2 = 2;
			iconItemPtr->iconID = iconID;
		}
	}

	if (err != noErr)	// punt if ALRT or DITL not gotten or DITL not duplicated
	{
		switch (iconID)
		{
			case -1:
				itemHit = ::Alert(alertID, modalFilter); 
				break;
			case stopIcon:
				itemHit = ::StopAlert(alertID, modalFilter); 
				break;
			case noteIcon:
				itemHit = ::NoteAlert(alertID, modalFilter); 
				break;
			case cautionIcon:
				itemHit = ::CautionAlert(alertID, modalFilter); 
				break;
		}
	}
	else
	{
		Rect	bounds = (**alrtHandle).boundsRect;

		fDialogPtr = ::NewColorDialog(NULL, &bounds, "\p", false /*visible*/, dBoxProc, (WindowPtr) -1, false /*goAwayFlag*/, 0 /*refCon*/, ditlHandle);
		::ThrowIfNULL_AC(fDialogPtr);
		
		this->AdjustAlertPosition(alrtHandle, &bounds);
		this->BeginAGADialog();
		
		::ShowWindow(fDialogPtr);
		
		while (itemHit == -1)
		{
			this->ModalDialog(modalFilter, &itemHit);
			
			if (! this->IsDITLItemEnabled(itemHit))
				itemHit = -1;
		}
		
		this->DisposeDialog();
	}

	if (alrtHandle != NULL)
		::HSetState((Handle) alrtHandle, savedHandleState);
	
	return itemHit;
}

//----------------------------------------------------------------------------------------
// BeginAGADialog
//----------------------------------------------------------------------------------------

void CDialog_AC::BeginAGADialog()
{
	// Called after the dialog and the standard items have been created
	// in memory, but before it has been altered by us and run.
	// We install a gray background and then create our objects.

	::SetWRefCon(fDialogPtr, (SInt32) this);

	(void) AGAInstallGrayWCTB(fDialogPtr);

	this->CreateAllCustomObjects();
}

//----------------------------------------------------------------------------------------
// CloseDialog
//----------------------------------------------------------------------------------------

void CDialog_AC::CloseDialog()
{
	// Clean up after ourselves before closing the dialog.

	this->DestroyAllCustomObjects();

	::CloseDialog(fDialogPtr);
	
	fDialogPtr = NULL;
}

//----------------------------------------------------------------------------------------
// DisposeDialog
//----------------------------------------------------------------------------------------

void CDialog_AC::DisposeDialog()
{
	// Clean up after ourselves before disposing the dialog.

	this->DestroyAllCustomObjects();

	::DisposeDialog(fDialogPtr);
	
	fDialogPtr = NULL;
}

//----------------------------------------------------------------------------------------
// GetDialogItem
//----------------------------------------------------------------------------------------

void CDialog_AC::GetDialogItem(SInt16 itemNo, SInt16* itemType, Handle* item, Rect* box)
{
	// This wrapper is not strictly needed. It just calls through.

	::GetDialogItem(fDialogPtr, itemNo, itemType, item, box);
}

//----------------------------------------------------------------------------------------
// SetDialogItem
//----------------------------------------------------------------------------------------

void CDialog_AC::SetDialogItem(SInt16 itemNo, SInt16 itemType, Handle item, Rect* box)
{
	// Need to find out how people are using this. About all we can
	// do is set the corresponding custom object's bounds to match.
	// Not sure whether we should call though for replaced items --
	// might this un-hide the original item?

	::SetDialogItem(fDialogPtr, itemNo, itemType, item, box);

	CControl_AC*	anAGAObject = this->GetAGAControl(itemNo);
	
	if (anAGAObject != NULL)
		anAGAObject->SetBounds(CRect_AC(*box), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// HideDialogItem
//----------------------------------------------------------------------------------------

void CDialog_AC::HideDialogItem(SInt16 itemNo)
{
	// The base item is already hidden. Hide the custom item.

	if (this->IsCustomObject(itemNo))
	{
		::HideDialogItem(fDialogPtr, itemNo + fNumAGAControls);
		this->SetCustomObjectVisibility(itemNo, false);
	}
	else
		::HideDialogItem(fDialogPtr, itemNo);
}

//----------------------------------------------------------------------------------------
// ShowDialogItem
//----------------------------------------------------------------------------------------

void CDialog_AC::ShowDialogItem(SInt16 itemNo)
{
	// The base item must remain hidden. Show the custom item.

	if (this->IsCustomObject(itemNo))
	{
		::ShowDialogItem(fDialogPtr, itemNo + fNumAGAControls);
		this->SetCustomObjectVisibility(itemNo, true);
	}
	else
		::ShowDialogItem(fDialogPtr, itemNo);
}

//----------------------------------------------------------------------------------------
// FindDialogItem
//----------------------------------------------------------------------------------------

SInt16 CDialog_AC::FindDialogItem(Point thePt)
{
	// Note: FindDialogItem is zero-based, unlike all other dialog itemNo usage.
	SInt16	result = ::FindDialogItem(fDialogPtr, thePt);
	
	if (result > fNumAGAControls)
		result -= fNumAGAControls;

	return result;
}

//----------------------------------------------------------------------------------------
// AppendDITL
//----------------------------------------------------------------------------------------

void CDialog_AC::AppendDITL(Handle theDITL, DITLMethod theMethod)
{
	// If the client is using AppendDITL to mess with the item list,
	// we'll need to respect their order. All we can do is try to
	// pretend we're not here, and then rebuild ourself to fit the
	// new items. If this doesn't work for you, let me know so we
	// can see if there are ways to make this work transparently.

	this->DestroyAllCustomObjects();
	
	::AppendDITL(fDialogPtr, theDITL, theMethod);
	
	this->CreateAllCustomObjects();
}

//----------------------------------------------------------------------------------------
// ShortenDITL
//----------------------------------------------------------------------------------------

void CDialog_AC::ShortenDITL(SInt16 numberOfItems)
{
	// If the client is using ShortenDITL to mess with the item list,
	// we'll need to respect their order. All we can do is try to
	// pretend we're not here, and then rebuild ourself to fit the
	// new items. If this doesn't work for you, let me know so we
	// can see if there are ways to make this work transparently.

	this->DestroyAllCustomObjects();
	
	::ShortenDITL(fDialogPtr, numberOfItems);
	
	this->CreateAllCustomObjects();
}

//----------------------------------------------------------------------------------------
// CountDITL
//----------------------------------------------------------------------------------------

SInt16 CDialog_AC::CountDITL()
{
	// Adjust the count so we are invisible.

	SInt16	result = ::CountDITL(fDialogPtr);
	
	if (result > fNumAGAControls)
		result -= fNumAGAControls;

	return result;
}

//----------------------------------------------------------------------------------------
// SetDialogDefaultItem
//----------------------------------------------------------------------------------------

OSErr CDialog_AC::SetDialogDefaultItem(SInt16 newItem)
{
	// Remove default outline from current default button, if any.
	
	if (fDefaultButton != NULL)
	{
		fDefaultButton->SetDefault(CControl_AC::kIsNotDefault, CControl_AC::kFrameOutside);
		fDefaultButton = NULL;
	}

	// Add default outline to new default button, if any.

	fDefaultItemNo = newItem;
	
	fDefaultButton = this->GetVerifiedPushButton(fDefaultItemNo);
	
	if (fDefaultButton != NULL)
		fDefaultButton->SetDefault(CControl_AC::kIsDefault, CControl_AC::kFrameOutside);

	return ::SetDialogDefaultItem(fDialogPtr, newItem);
}

//----------------------------------------------------------------------------------------
// SetDialogCancelItem
//----------------------------------------------------------------------------------------

OSErr CDialog_AC::SetDialogCancelItem(SInt16 newItem)
{
	// Set the cancel item so we can catch the escape key in our filter.

	fCancelItemNo = newItem;
	
	fCancelButton = this->GetVerifiedPushButton(fCancelItemNo);

	return ::SetDialogCancelItem(fDialogPtr, newItem);
}

//----------------------------------------------------------------------------------------
// GetDialogItemText
//----------------------------------------------------------------------------------------

void CDialog_AC::GetDialogItemText(Handle item, CStr255_AC& text)
{
	// This wrapper is not strictly needed. It just calls through.

	::GetDialogItemText(item, text);
}

//----------------------------------------------------------------------------------------
// SetDialogItemText
//----------------------------------------------------------------------------------------

void CDialog_AC::SetDialogItemText(Handle item, const CStr255_AC& text)
{
	// If it's an editText item, the original object handles it.
	// If it's a statText item, the original is hidden (but should
	// get the assigned text, so that GetDialogItemText will just
	// work) and ours must handle it.

	::SetDialogItemText(item, text);
	
	if (this->GetDITLItemType(this->GetDITLItemNoFromHandle(item)) == statText)
	{
		CStaticText_AC*	theStaticText = (CStaticText_AC*) this->GetAGAControlFromHandle(item);
	
		if (theStaticText != NULL)
			theStaticText->SetTitle(text, CControl_AC::kRedraw);
	}
}

//----------------------------------------------------------------------------------------
// SelectDialogItemText
//----------------------------------------------------------------------------------------

void CDialog_AC::SelectDialogItemText(SInt16 itemNo, SInt16 strtSel, SInt16 endSel)
{
	// This wrapper is not strictly needed. It just calls through.

	::SelectDialogItemText(fDialogPtr, itemNo, strtSel, endSel);
}

//----------------------------------------------------------------------------------------
// ModalDialog
//----------------------------------------------------------------------------------------

void CDialog_AC::ModalDialog(ModalFilterUPP filterProc, SInt16* itemHit)
{
	// Wrap ModalDialog by saving aside the client filterProc
	// (which will be used by our filter) and afterwards adjusting
	// the itemHit if one of our custom objects was hit.

	fUserFilter = filterProc;

	::ModalDialog(gModalFilterUPP, itemHit);

	fUserFilter = NULL;
	
	if (*itemHit > fNumAGAControls)
		(*itemHit) -= fNumAGAControls;
}

//----------------------------------------------------------------------------------------
// DialogSelect
//----------------------------------------------------------------------------------------

bool CDialog_AC::DialogSelect(EventRecord* event, SInt16* itemHit)
{
	// Wrapper for modeless dialog DialogSelect.
	// Unlike modal dialogs and alerts, we have to handle update
	// events by calling DrawDialog.
	// Note that the case of multiple modeless dialogs has not been
	// totally worked out. Normally DialogSelect is called without
	// regard to which dialog is going to get the event; in fact, you
	// don't know which window is going to get the event! But this
	// is a method of an object that wraps a particular dialog.
	//

	bool	wasEventHandled = false;

	switch (event->what)
	{
		case activateEvt:
			{
				CDialog_AC*	dialogToActivate = CDialog_AC::ObjectFromDialogPtr((WindowPtr) (event->message));

				if (dialogToActivate != NULL)
					dialogToActivate->HandleActivateEvent(((event->modifiers) & activeFlag) != 0);
			}
			break;
		
		case updateEvt:
		
			if (CDialog_AC::ObjectFromDialogPtr((WindowPtr) (event->message)) == this)
			{
				::BeginUpdate(fDialogPtr);
				this->DrawDialog();
				::EndUpdate(fDialogPtr);

				wasEventHandled = true;
				*itemHit = -1;
			}

			break;
	}

	if (! wasEventHandled)
		wasEventHandled = this->HandleModalFilter(event, itemHit);
	
	return wasEventHandled;
}

//----------------------------------------------------------------------------------------
// DrawDialog
//----------------------------------------------------------------------------------------

void CDialog_AC::DrawDialog()
{
	// This wrapper is not strictly needed. It just calls through.

	::DrawDialog(fDialogPtr);
}

//----------------------------------------------------------------------------------------
// UpdateDialog
//----------------------------------------------------------------------------------------

void CDialog_AC::UpdateDialog(RgnHandle updateRgn)
{
	// This wrapper is not strictly needed. It just calls through.

	::UpdateDialog(fDialogPtr, updateRgn);
}

//----------------------------------------------------------------------------------------
// SetControlValue
//----------------------------------------------------------------------------------------

void CDialog_AC::SetControlValue(ControlHandle theControl, SInt16 theValue)
{
	// We set the real object's value, but also have to see if we have
	// an AGA replacement object to set.

	::SetControlValue(theControl, theValue);

	CControl_AC*		theObject = this->GetAGAControlFromControlHandle(theControl);
	CTempPort_AC	port(fDialogPtr);

	if (theObject != NULL)
		theObject->SetValue(theValue, CControl_AC::kRedraw);
}

//----------------------------------------------------------------------------------------
// SetControlMinimum
//----------------------------------------------------------------------------------------

void CDialog_AC::SetControlMinimum(ControlHandle theControl, SInt16 minValue)
{
	// We don't yet support AGA scroll bars in the CDialog_AC.

	::SetControlMinimum(theControl, minValue);

	//CControl_AC*	theObject = this->GetAGAControlFromControlHandle(theControl);
	
	//	if (theObject != NULL)
	//		theObject->SetMinValue(minValue, CControl_AC::kRedraw);
}

//----------------------------------------------------------------------------------------
// SetControlMaximum
//----------------------------------------------------------------------------------------

void CDialog_AC::SetControlMaximum(ControlHandle theControl, SInt16 maxValue)
{
	// We don't yet support AGA scroll bars in the CDialog_AC.

	::SetControlMaximum(theControl, maxValue);

	CControl_AC*	theObject = this->GetAGAControlFromControlHandle(theControl);
	
	//	if (theObject != NULL)
	//		theObject->SetMaxValue(maxValue, CControl_AC::kRedraw);
}

//----------------------------------------------------------------------------------------
// SetControlTitle
//----------------------------------------------------------------------------------------

void CDialog_AC::SetControlTitle(ControlHandle theControl, const CStr255_AC& title)
{
	// Set the toolbox and AGA control title.

	::SetControlTitle(theControl, title);

	CControl_AC*	theObject = this->GetAGAControlFromControlHandle(theControl);
	
	if (theObject != NULL)
		theObject->SetTitle(title, CControl_AC::kRedraw);
}

//----------------------------------------------------------------------------------------
// HideControl
//----------------------------------------------------------------------------------------

void CDialog_AC::HideControl(ControlHandle theControl)
{
	// Need to add a visibility flag to the core CControl_AC.

	::HideControl(theControl);

	//CControl_AC*	theObject = this->GetAGAControlFromControlHandle(theControl);
	
	//	if (theObject != NULL)
	//		::HideDialogItem(itemNo + fNumAGAControls);
}

//----------------------------------------------------------------------------------------
// MoveControl
//----------------------------------------------------------------------------------------

void CDialog_AC::MoveControl(ControlHandle theControl, SInt16 h, SInt16 v)
{
	// Move the real control, and our replacement, if any.

	// Only move the control if it is visible, else it may become visible due to how the
	// dialog manager hides items (thanks T. Pedone).
	if ((**theControl).contrlVis == 255)
	{
		::MoveControl(theControl, h, v);
	}

	CControl_AC*	theObject = this->GetAGAControlFromControlHandle(theControl);
	
	if (theObject != NULL)
	{
		CRect_AC	bounds(theObject->GetBounds());
		
		bounds.Offset(-bounds.left + h, -bounds.top + v);
		
		theObject->SetBounds(bounds, CControl_AC::kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// SizeControl
//----------------------------------------------------------------------------------------

void CDialog_AC::SizeControl(ControlHandle theControl, SInt16 width, SInt16 height)
{
	// Resize the real control, and our replacement, if any.

	::SizeControl(theControl, width, height);

	CControl_AC*	theObject = this->GetAGAControlFromControlHandle(theControl);
	
	if (theObject != NULL)
	{
		CRect_AC	bounds(theObject->GetBounds());
		
		bounds.right = bounds.left + width;
		bounds.bottom = bounds.top + height;
		
		theObject->SetBounds(bounds, CControl_AC::kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// HiliteControl
//----------------------------------------------------------------------------------------

void CDialog_AC::HiliteControl(ControlHandle theControl, SInt16 hiliteState)
{
	// If the control being highlighted is one we are replacing,
	// this routine is problematic. Ideally we would just highlight
	// the replacement, but we need to find out the object class
	// and do the appropriate action. Usually HiliteControl is just
	// called as part of trying to flash a button, and we have a
	// FlashButton method.

	::HiliteControl(theControl, hiliteState);

	//CControl_AC*	theObject = this->GetAGAControlFromControlHandle(theControl);
	
	//	if (theObject != NULL)
	//		theObject->Enable/Dim/Flash/Etc.(...);
}

//----------------------------------------------------------------------------------------
// AdjustAlertPosition
//----------------------------------------------------------------------------------------

// These constants are not defined in the Universal Headers!
// Had to swipe from Types.r and make an enum.

enum { noAutoCenter = 0x0000,
		centerMainScreen = 0x280a,
		alertPositionMainScreen = 0x300a,
		staggerMainScreen = 0x380a,
		centerParentWindow = 0xa80a,
		alertPositionParentWindow = 0xb00a,
		staggerParentWindow = 0xb80a,
		centerParentWindowScreen = 0x680a,
		alertPositionParentWindowScreen = 0x700a,
		staggerParentWindowScreen = 0x780a };

void CDialog_AC::AdjustAlertPosition(AlertTHndl alrtHandle, const Rect* origBounds)
{
	SInt16	positionSpecifier = noAutoCenter;
	SInt32	alrtSize = ::GetHandleSize((Handle) alrtHandle);
	
	if (alrtSize > sizeof(AlertTemplate))
	{
		// This alert has the optional position specifier at
		// the end of the normal ALRT structure. We do some
		// skanky pointer arithmetic and typecasting to get
		// a pointer to that extra short past the standard
		// struct, and then dereference it.
		
		positionSpecifier = *((SInt16*) ((*alrtHandle) + 1));
	}

	this->AdjustWindowPosition(positionSpecifier, origBounds);
}

//----------------------------------------------------------------------------------------
// AdjustDialogPosition
//----------------------------------------------------------------------------------------

void CDialog_AC::AdjustDialogPosition(DialogTHndl dlogHandle, const Rect* origBounds)
{
	SInt16	positionSpecifier = noAutoCenter;
	SInt32	dlogActualSize = ::GetHandleSize((Handle) dlogHandle);
	SInt32	dlogStdDataSize;
	
	// Calculate how big this DLOG would really be if didn't have
	// the positioner at the end. This depends on the title string
	// length, and an optional alignment byte.
	dlogStdDataSize = sizeof(DialogTemplate) - sizeof(Str255) + 1;
	dlogStdDataSize += (**dlogHandle).title[0];
	
	if (dlogStdDataSize & 0x00000001)	// odd size, add alignment byte
		++dlogStdDataSize;
	
	if (dlogActualSize > dlogStdDataSize)
		{
		// This dialog has the optional position specifier at
		// the end of the normal DLOG data. We do some
		// skanky pointer arithmetic and typecasting to get
		// a pointer to that extra short past the standard
		// data, and then dereference it.
		
		positionSpecifier = *((SInt16*) (((Ptr) (*dlogHandle)) + dlogStdDataSize));
	}

	this->AdjustWindowPosition(positionSpecifier, origBounds);
}

//----------------------------------------------------------------------------------------
// CenterRectOnPoint
//----------------------------------------------------------------------------------------

static void CenterRectOnPoint(Rect* rect, Point itsCenter)
{
	Point	rectSize;

	::SetPt(&rectSize, rect->right - rect->left, rect->bottom - rect->top);

	rect->left = itsCenter.h - (rectSize.h / 2);
	rect->top = itsCenter.v - (rectSize.v / 2);
	rect->right = rect->left + rectSize.h;
	rect->bottom = rect->top + rectSize.v;
}

//----------------------------------------------------------------------------------------
// AdjustWindowPosition
//----------------------------------------------------------------------------------------

void CDialog_AC::AdjustWindowPosition(SInt16 positionSpecifier, const Rect* origBounds)
{
	// Man, why doesn't the Dialog Manager have a function call to do this
	// so we don't have to try and approximate its behavior?
	
	bool		adjustIt = true;
	Rect		targetRect;	// the bounds in which we must center the window
	Rect		windowRect = *origBounds;
	GDHandle	targetDevice;
	WindowPeek	parentWindow;
	Point		targetCenter;
	SInt16		targetExcessV;
	
	switch (positionSpecifier)
		{
		case centerMainScreen:
		case alertPositionMainScreen:
		case staggerMainScreen:
			// Use the main monitor as the general target area.

			targetDevice = ::GetMainDevice();
			targetRect = (**targetDevice).gdRect;
			break;
		case centerParentWindow:
		case alertPositionParentWindow:
		case staggerParentWindow:
			// Use the front window as the general target area.

			parentWindow = (WindowPeek) ::FrontWindow();
			if (parentWindow == NULL)
				adjustIt = false;
			else
				targetRect = (**(parentWindow->contRgn)).rgnBBox;
			break;
		case centerParentWindowScreen:
		case alertPositionParentWindowScreen:
		case staggerParentWindowScreen:
			// Use the front window's monitor as the general target area.

			parentWindow = (WindowPeek) ::FrontWindow();
			if (parentWindow == NULL)
				adjustIt = false;
			else
			{
				bool	found = false;
				Rect	parentWindowBounds = (**(parentWindow->contRgn)).rgnBBox;
				Point	parentWindowCenter;
				
				targetRect = (**(parentWindow->contRgn)).rgnBBox;
			
				::SetPt(&parentWindowCenter,
					(parentWindowBounds.left + parentWindowBounds.right) / 2,
					(parentWindowBounds.top + parentWindowBounds.bottom) / 2);

				// Find the device that contains the window's center.
				// Not as precise as calculating the max intersected device,
				// but good enough.
				
				targetDevice = ::GetDeviceList();
				
				while (targetDevice != NULL)
				{
					if (::PtInRect(parentWindowCenter, &(**targetDevice).gdRect))
					{
						targetRect = (**targetDevice).gdRect;
						
						if (targetDevice == ::GetMainDevice())
							targetRect.top += LMGetMBarHeight();

						break;
					}
					
					targetDevice = ::GetNextDevice(targetDevice);
				}
			}

			break;
	}

	if (adjustIt)
	{
		::SetPt(&targetCenter,
			(targetRect.left + targetRect.right) / 2,
			(targetRect.top + targetRect.bottom) / 2);

		switch (positionSpecifier)
		{
			case centerMainScreen:
			case centerParentWindow:
			case centerParentWindowScreen:
			
				CenterRectOnPoint(&windowRect, targetCenter);

				break;

			case alertPositionMainScreen:
			case alertPositionParentWindow:
			case alertPositionParentWindowScreen:
			
				// If target is smaller than window (no "excess"),
				// just center. Otherwise, center on upper area.

				targetExcessV = (targetRect.bottom - targetRect.top) -
								(windowRect.bottom - windowRect.top);

				if (targetExcessV > 0)
					targetCenter.v -= (targetExcessV / 4);	// center at top 1/4

				CenterRectOnPoint(&windowRect, targetCenter);

				break;

			case staggerMainScreen:
			case staggerParentWindow:
			case staggerParentWindowScreen:
				adjustIt = false;
				// Hmmm... the system's staggering info is not documented.
				break;
		}
	}

	if (adjustIt)
	{
		// Before moving the window, make sure all 4 corners are somewhere
		// on screen. If not, center it on the main screen.

		RgnHandle	grayRegion = ::GetGrayRgn();
		Point		wTopLeft;
		Point		wTopRight;
		Point		wBotLeft;
		Point		wBotRight;

		::SetPt(&wTopLeft, windowRect.left, windowRect.top);
		::SetPt(&wTopRight, windowRect.right, windowRect.top);
		::SetPt(&wBotLeft, windowRect.left, windowRect.bottom);
		::SetPt(&wBotRight, windowRect.right, windowRect.bottom);

		if ((! ::PtInRgn(wTopLeft, grayRegion)) || (! ::PtInRgn(wTopRight, grayRegion)) ||
			(! ::PtInRgn(wBotLeft, grayRegion)) || (! ::PtInRgn(wBotRight, grayRegion)))
		{
			targetRect = (**::GetMainDevice()).gdRect;

			::SetPt(&targetCenter,
				(targetRect.left + targetRect.right) / 2,
				(targetRect.top + targetRect.bottom) / 2);

			CenterRectOnPoint(&windowRect, targetCenter);
		}

		::MoveWindow(fDialogPtr, windowRect.left, windowRect.top, false);
	}
}

//----------------------------------------------------------------------------------------
// CreateAllCustomObjects
//----------------------------------------------------------------------------------------

void CDialog_AC::CreateAllCustomObjects()
{
	// This and BuildCustomObject are the two core functions that
	// perform most of our Patented Dialog Manager Grayscale Magic.
	//
	// We need to construct an n item DITL manually in memory.
	// It starts with a 2-byte value that is the n-1.
	// It is followed by the n variable-length items.
	// In our case every item is in fact a userItem (14 bytes).
	// See IM:TBE p. 6-152 and p. 6-155 for details.
	//
	// Once we've built this DITL and the CControl_AC objects that
	// will replace certain of the original items (the originals
	// are simply hidden), we append it to the user's standard DITL
	// that came from the DITL resource.

	OSErr	err = noErr;
	Handle	customItemsDITL;
	Ptr		ditlDataPtr;
	SInt16	numOriginalItems = ::CountDITL(fDialogPtr);
	SInt16	i = 1;
	
	//
	// Allocate a handle to contain our duplicate DITL items.
	// Each one is a userItem (14 bytes).
	//

	customItemsDITL = ::NewHandleClear(2 + (numOriginalItems * 14));

	if (customItemsDITL == NULL)
		err = memFullErr;

	//
	// Size our fAGAControls handle to make room for n CControl_AC pointers.
	//

	if (err == noErr)
	{
		::SetHandleSize((Handle) fAGAControls, numOriginalItems * sizeof(CustomObjectInfo));
		err = ::MemError();

		// Controls haven't been created yet, so initialize in case failure requires clean up	
		BlockSet_AC(*fAGAControls, numOriginalItems * sizeof(CustomObjectInfo), 0);
	}
	
	if (err == noErr)
		fNumAGAControls = numOriginalItems;
	else
		fNumAGAControls = 0;

	//
	// Create the replacement objects and store their pointers in the handle.
	//

	{
		CTempHandleLock_AC	locker(((DialogPeek) fDialogPtr)->items);
		Ptr				ditlItemPtr = *(((DialogPeek) fDialogPtr)->items);
		
		ditlItemPtr += 2;	// skip past "count minus 1" integer

			
		for (i = 1; i <= fNumAGAControls; ++i)
			this->CreateCustomObject(i, &ditlItemPtr);
	}
	
	if (customItemsDITL != NULL)
	{		
		::HLock(customItemsDITL);

		ditlDataPtr = *customItemsDITL;
		*((SInt16*) ditlDataPtr) = fNumAGAControls - 1;	// set the item count field (n-1 actually)
		ditlDataPtr += 2;								// move past item count field

		for (i = 1; i <= fNumAGAControls; ++i)
		{
			SInt16		itemType;
			Handle		item;
			Rect		box;
			UInt8		ditlItemType = userItem;

			::GetDialogItem(fDialogPtr, i, &itemType, &item, &box);
			
			if ((itemType & itemDisable) != 0)
				ditlItemType |= itemDisable;
			
			// Since these items are all "hidden", and since the Dialog Manager
			// hides items by moving them, we need to adjust to get the real
			// box where our item is to be located. See IM:TBE p. 6-123.
			if (box.left > 8192)
				::OffsetRect(&box, -16384, 0);
			
			// OK, we're touching the DITL byte layout directly here!
			// We know this is not nice, but it's the only way!

			ditlDataPtr += 4;						// move past reserved field
			
			*((Rect*) ditlDataPtr) = box;			// set the display rectangle field
			ditlDataPtr += 8;						// move past display rectangle field
			
			*((UInt8*) ditlDataPtr) = ditlItemType;	// set the item type / enable field
			ditlDataPtr += 2;						// move past item type and reserved fields
		}

		::HUnlock(customItemsDITL);
		
		::AppendDITL(fDialogPtr, customItemsDITL, overlayDITL);
		::DisposeHandle(customItemsDITL);

		// Now install our userItem proc for each of our objects.
		for (i = 1; i <= fNumAGAControls; ++i)
		{
			SInt16		itemType;
			Handle		item;
			Rect		box;

			::GetDialogItem(fDialogPtr, i + fNumAGAControls, &itemType, &item, &box);
			::SetDialogItem(fDialogPtr, i + fNumAGAControls, itemType, (Handle) gUserItemUPP, &box);
		}
	}
}

//----------------------------------------------------------------------------------------
// DestroyAllCustomObjects
//----------------------------------------------------------------------------------------

void CDialog_AC::DestroyAllCustomObjects()
{
	// Free the CControl_AC associated with each custom userItem.

	for (SInt16 i = 1; i <= fNumAGAControls; ++i)
		this->DestroyCustomObject(i);

	fNumAGAControls = 0;
}

//----------------------------------------------------------------------------------------
// CreateCustomObject
//----------------------------------------------------------------------------------------

void CDialog_AC::CreateCustomObject(SInt16 itemIndex, Ptr* ditlItemPtr)
{
	// Create an CControl_AC of the appropriate class, if any, for the
	// specified original DITL item. Hide the original if we created
	// an CControl_AC.

	bool				hideOriginal;
	CControl_AC*		theAGAObject = this->BuildCustomObject(itemIndex, &hideOriginal, ditlItemPtr);
	CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemIndex);
	
	if (infoPtr != NULL)
	{
		infoPtr->fAGAControl = theAGAObject;
		infoPtr->fTracksMouse = hideOriginal;
		infoPtr->fVisible = true;	
	}

	if (hideOriginal)
		::HideDialogItem(fDialogPtr, itemIndex);
}

//----------------------------------------------------------------------------------------
// DestroyCustomObject
//----------------------------------------------------------------------------------------

void CDialog_AC::DestroyCustomObject(SInt16 itemIndex)
{
	// Just delete the CControl_AC if there is one for this custom userItem.

	CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemIndex);
	
	if (infoPtr->fAGAControl != NULL)
	{
		delete infoPtr->fAGAControl;
		infoPtr->fAGAControl = NULL;
	}
}

//----------------------------------------------------------------------------------------
// IsCustomObject
//----------------------------------------------------------------------------------------

bool CDialog_AC::IsCustomObject(SInt16 itemIndex)
{
	// Return true if there's an CControl_AC for this custom userItem.

	CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemIndex);
	
	return (infoPtr->fAGAControl != NULL);
}

//----------------------------------------------------------------------------------------
// IsCustomObjectVisible
//----------------------------------------------------------------------------------------

bool CDialog_AC::IsCustomObjectVisible(SInt16 itemIndex)
{
	bool returnValue = false;
	CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemIndex);
	if (infoPtr != NULL)
	{
		returnValue = infoPtr->fVisible;
	}
	return returnValue;
}

//----------------------------------------------------------------------------------------
// SetCustomObjectVisibility
//----------------------------------------------------------------------------------------

void CDialog_AC::SetCustomObjectVisibility(SInt16 itemIndex, bool isVisible)
{
	CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemIndex);
	if (infoPtr != NULL)
	{
		infoPtr->fVisible = isVisible;	
	}
}

//----------------------------------------------------------------------------------------
// GetCustomObjectInfoPtr
//----------------------------------------------------------------------------------------

CustomObjectInfoPtr CDialog_AC::GetCustomObjectInfoPtr(SInt16 itemIndex)
{
	// Return a pointer to the CustomObjectInfo entry for this userItem.
	// Warning: this is a pointer into a relocatable block; use it briefly.

	if ((itemIndex > 0) && (itemIndex <= fNumAGAControls))
		return &((*fAGAControls)[itemIndex - 1]);
	else
		return NULL;
}

//----------------------------------------------------------------------------------------
// GetAGAControl
//----------------------------------------------------------------------------------------

CControl_AC* CDialog_AC::GetAGAControl(SInt16 itemIndex)
{
	// Return the CControl_AC pointer for this custom item, or NULL if
	// it has no custom object.

	CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemIndex);
	
	if (infoPtr == NULL)
		return NULL;
	else
		return infoPtr->fAGAControl;
}

//----------------------------------------------------------------------------------------
// GetAGAControlFromHandle
//----------------------------------------------------------------------------------------

CControl_AC* CDialog_AC::GetAGAControlFromHandle(Handle itemHandle)
{
	// Get the CControl_AC, if any, that corresponds to the supplied
	// original item handle.

	CControl_AC*	result = NULL;
	SInt16			itemType;
	Handle			item;
	Rect			box;

	for (SInt16 i = 1; i <= fNumAGAControls; ++i)
	{
		::GetDialogItem(fDialogPtr, i, &itemType, &item, &box);
		
		if (item == itemHandle)
		{
			result = this->GetAGAControl(i);
			break;
		}
	}

	return result;
}

//----------------------------------------------------------------------------------------
// GetDITLItemNoFromHandle
//----------------------------------------------------------------------------------------

SInt16 CDialog_AC::GetDITLItemNoFromHandle(Handle itemHandle)
{
	// Get the original DITL item number for the supplied item handle.

	SInt16	result = 0;
	SInt16	itemType;
	Handle	item;
	Rect	box;

	for (SInt16 i = 1; i <= fNumAGAControls; ++i)
	{
		::GetDialogItem(fDialogPtr, i, &itemType, &item, &box);
		
		if (item == itemHandle)
		{
			result = i;
			break;
		}
	}

	return result;
}

//----------------------------------------------------------------------------------------
// GetAGAControlFromControlHandle
//----------------------------------------------------------------------------------------

CControl_AC* CDialog_AC::GetAGAControlFromControlHandle(ControlHandle theControl)
{
	// Get the CControl_AC, if any, that corresponds to the supplied
	// control handle.

	return this->GetAGAControlFromHandle((Handle) theControl);
}

//----------------------------------------------------------------------------------------
// BuildCustomObject
//----------------------------------------------------------------------------------------

CControl_AC* CDialog_AC::BuildCustomObject(SInt16 itemIndex, bool* hideOriginal, Ptr* ditlItemPtr)
{
	// This and CreateAllCustomObjects are the two core functions that
	// perform most of our Patented Dialog Manager Grayscale Magic.
	//
	// We use the original item's itemType to determine whether and which
	// type of CControl_AC will replace the original. In certain cases
	// (such as editText items) we keep the original but add an object
	// to enhance it (an editText gets a framing object).

	CControl_AC*	anAGAObject = NULL;
	SInt16		itemType;
	Handle		item;
	CRect_AC	box;
	CStr255_AC	title;
	bool		enabled = true;
	Ptr			aPtr;
	
	*hideOriginal = true;

	::GetDialogItem(fDialogPtr, itemIndex, &itemType, &item, &box);
	
	if ((itemType & itemDisable) != 0)
	{
		enabled = false;
		itemType &= ~itemDisable;
	}
	
	// Get any item-type-specific information ready.
	switch (itemType)
	{
		case kButtonDialogItem:
		case kCheckBoxDialogItem:
		case kRadioButtonDialogItem:
			::GetControlTitle((ControlHandle) item, title);
			break;
		case kStaticTextDialogItem:
		case kEditTextDialogItem:
			::GetDialogItemText(item, title);
			break;
	}

	// Create the appropriate custom object.
	switch (itemType)
	{
		case kButtonDialogItem:

			if (EmulateControls_AC())
			{
				anAGAObject = TH_new CButton_AC(box, CTextStyle_AC::kSystemStyle, title);
				
				if (itemIndex == fDefaultItemNo)
				{
					((CButton_AC*) anAGAObject)->SetDefault(CControl_AC::kIsDefault, CControl_AC::kFrameOutside);
					fDefaultButton = (CButton_AC*) anAGAObject;
				}
				else if (itemIndex == fCancelItemNo)
					fCancelButton = (CButton_AC*) anAGAObject;
			}
			else
				*hideOriginal = false;

			break;
		case kCheckBoxDialogItem:

			if (EmulateControls_AC())
				anAGAObject = TH_new CCheckBox_AC(box, CTextStyle_AC::kSystemStyle, title, CControl_AC::kAutomaticState);
			else
				*hideOriginal = false;

			break;
		case kRadioButtonDialogItem:

			if (EmulateControls_AC())
				anAGAObject = TH_new CRadioButton_AC(box, CTextStyle_AC::kSystemStyle, title, MExclusiveObject_AC::kNoGroupID, MExclusiveObject_AC::kNoGroupID, CControl_AC::kNoAutomaticState);
			else
				*hideOriginal = false;

			break;
		case kStaticTextDialogItem:
			this->DoParamTextSubstitution(title);
			anAGAObject = TH_new CStaticText_AC(box, CTextStyle_AC::kSystemStyle, teFlushDefault, CStaticText_AC::kEraseFirst, title);
			break;
		case kEditTextDialogItem:
			::InsetRect(&box, -4, -4);
			anAGAObject = TH_new CDialogEditTextFrame_AC(CRect_AC(box));
			*hideOriginal = false;
			break;
		case kIconDialogItem:
			aPtr = (*ditlItemPtr) + 14;	// point to the icon id field of the item
			anAGAObject = TH_new CDialogIcon_AC(box, *((SInt16*) aPtr));
			break;
		default:
			*hideOriginal = false;
			break;
	}

	// Increment the ditlItemPtr to the next item, based on the size of
	// this item element.
	this->GetNextDITLItemPtr(itemType, ditlItemPtr);

	return anAGAObject;
}

//----------------------------------------------------------------------------------------
// GetNextDITLItemPtr
//----------------------------------------------------------------------------------------

void CDialog_AC::GetNextDITLItemPtr(SInt16 itemType, Ptr* ditlItemPtr)
{
	UInt32	offset;
	UInt8	titleLength;

	// Increment the ditl item pointer past the current item data.
	// See Inside Macintosh: Toolbox Essentials pp. 6-153 thru 6-155
	// for the ditl item layouts used here.

	switch (itemType)
	{
		case kButtonDialogItem:
		case kCheckBoxDialogItem:
		case kRadioButtonDialogItem:
		case kStaticTextDialogItem:
		case kEditTextDialogItem:
		
			titleLength = *((*ditlItemPtr) + 13);
		
			offset = 14 + titleLength;
			
			if ((titleLength % 2) != 0)	// odd string length
				++offset;		// adjust for alignment byte in DITL

			(*ditlItemPtr) += offset;

			break;
		case kResourceControlDialogItem:
		case kIconDialogItem:
		case kPictureDialogItem:
			(*ditlItemPtr) += 16;
			break;
		case kUserDialogItem:
			(*ditlItemPtr) += 14;
			break;
		case 1:	// no constant available such as kHelpDialogItem
			(*ditlItemPtr) += 20;
			break;
	}
}

//----------------------------------------------------------------------------------------
// HandleUserItem
//----------------------------------------------------------------------------------------

void CDialog_AC::HandleUserItem(SInt16 itemNo)
{
	// Handle a userItem proc call for one of our custom items.
	// We just tell the CControl_AC to draw if there is one.

	CControl_AC*	anAGAObject = this->GetAGAControl(itemNo - fNumAGAControls);
	
	if (anAGAObject != NULL && IsCustomObjectVisible(itemNo - fNumAGAControls))
		anAGAObject->DrawObject();
}

//----------------------------------------------------------------------------------------
// IsDITLItemEnabled
//----------------------------------------------------------------------------------------

bool CDialog_AC::IsDITLItemEnabled(SInt16 itemNo)
{
	// Convenience function to test the enable flag of the specified item.

	SInt16	itemType;
	Handle	itemHandle;
	Rect	itemRect;

	::GetDialogItem(fDialogPtr, itemNo, &itemType, &itemHandle, &itemRect);
	
	return ((itemType & itemDisable) == 0);
}

//----------------------------------------------------------------------------------------
// GetDITLItemType
//----------------------------------------------------------------------------------------

SInt16 CDialog_AC::GetDITLItemType(SInt16 itemNo)
{
	// Convenience function to get the specified item's type, without
	// regard to its enable flag.

	SInt16	itemType;
	Handle	itemHandle;
	Rect	itemRect;

	::GetDialogItem(fDialogPtr, itemNo, &itemType, &itemHandle, &itemRect);
	
	return itemType & ~itemDisable;	// clear the disable bit; the caller doesn't want it
}

//----------------------------------------------------------------------------------------
// GetVerifiedPushButton
//----------------------------------------------------------------------------------------

CButton_AC* CDialog_AC::GetVerifiedPushButton(SInt16 itemNo)
{
	// Return the pointer to the CButton_AC after checking the
	// original item's type to make sure it's really a Control Manager
	// pushbutton that we are replacing. By doing the verification
	// via the itemType, we don't need to do RTTI on our CControl_AC.

	if ((this->GetDITLItemType(itemNo) & (ctrlItem + btnCtrl)) == 0)
		return NULL;
	else
		return (CButton_AC*) this->GetAGAControl(itemNo);
}

//----------------------------------------------------------------------------------------
// ParamSubst
//----------------------------------------------------------------------------------------

static void ParamSubst(CStr255_AC& destString,
						const CStr255_AC& param1,
						const CStr255_AC& param2,
						const CStr255_AC& param3,
						const CStr255_AC& param4)
{
	StringHandle	h = NULL;
	SInt32			result;
	UInt8			targetText[2];
	CStr255_AC			tempString;

	targetText[0] = '^';

	h = ::NewString(destString);
	
	if (h != NULL)
	{
		targetText[1] = '0';
		tempString = param1;
		result = Munger((Handle) h, 1, &targetText[0], 2, &tempString[1], tempString[0]);
		
		targetText[1] = '1';
		tempString = param2;
		result = Munger((Handle) h, 1, &targetText[0], 2, &tempString[1], tempString[0]);
		
		targetText[1] = '2';
		tempString = param3;
		result = Munger((Handle) h, 1, &targetText[0], 2, &tempString[1], tempString[0]);
		
		targetText[1] = '3';
		tempString = param4;
		result = Munger((Handle) h, 1, &targetText[0], 2, &tempString[1], tempString[0]);
		
		//
		// Update the length byte of the string in the handle, because Munger just
		// munged the string data but didnt change the length byte.
		//
		**((UInt8 **) h) = (UInt8) (GetHandleSize((Handle) h) - 1);
		
		destString = *h;
		
		::DisposeHandle((Handle) h);
	}
}

//----------------------------------------------------------------------------------------
// DoParamTextSubstitution
//----------------------------------------------------------------------------------------

void CDialog_AC::DoParamTextSubstitution(CStr255_AC& dialogString)
{
	CStr255_AC		s1;
	CStr255_AC		s2;
	CStr255_AC		s3;
	CStr255_AC		s4;
	StringHandle	aStringHandle;
	
	aStringHandle = LMGetDAStrings(0);
	if ((aStringHandle == NULL) || (*aStringHandle == NULL))
		s1.Empty();
	else
		s1 = *aStringHandle;

	aStringHandle = LMGetDAStrings(1);
	if ((aStringHandle == NULL) || (*aStringHandle == NULL))
		s2.Empty();
	else
		s2 = *aStringHandle;

	aStringHandle = LMGetDAStrings(2);
	if ((aStringHandle == NULL) || (*aStringHandle == NULL))
		s3.Empty();
	else
		s3 = *aStringHandle;

	aStringHandle = LMGetDAStrings(3);
	if ((aStringHandle == NULL) || (*aStringHandle == NULL))
		s4.Empty();
	else
		s4 = *aStringHandle;

	ParamSubst(dialogString, s1, s2, s3, s4);
}

//----------------------------------------------------------------------------------------
// HandleModalFilter
//----------------------------------------------------------------------------------------

bool CDialog_AC::HandleModalFilter(EventRecord* theEvent, SInt16* itemHit)
{
	// This is the filterProc that we always pass to ModalDialog.
	// If the caller supplied a filterProc, we saved it in fUserFilter,
	// and we call it first.

	bool	wasEventHandled = false;
	GrafPtr	savedPort;
	
	::GetPort(&savedPort);
	::SetPort(fDialogPtr);
	
	::RGBForeColor(&kRGBColorRamp[rB]);
	::RGBBackColor(&kRGBColorRamp[rW]);	// make any editText items paint a white bg

	if (fUserFilter != NULL)
		wasEventHandled = CallModalFilterProc(fUserFilter, fDialogPtr, theEvent, itemHit);

	if (! wasEventHandled)
	{
		switch (theEvent->what)
		{
			case mouseDown:
				wasEventHandled = this->FilterMouseDown(theEvent, itemHit);
				break;

			case keyDown:
			case autoKey:
				wasEventHandled = this->FilterKeyDown(theEvent, itemHit);
				break;

			case activateEvt:
				{
					CDialog_AC*	dialogToActivate = CDialog_AC::ObjectFromDialogPtr((WindowPtr) (theEvent->message));

					if (dialogToActivate != NULL)
						dialogToActivate->HandleActivateEvent(((theEvent->modifiers) & activeFlag) != 0);
				}
				break;
			

			case updateEvt:
			
				if (fIsAlert &&
					(! EmulateControls_AC()) &&
					(fDefaultItemNo != -1) &&
					(CDialog_AC::ObjectFromDialogPtr((WindowPtr) (theEvent->message)) == this))
				{
					// If we're an Alert, not a ModalDialog, and the AGA
					// controls are turned off, we need to draw the default
					// button's outline, just like the standard Alert()
					// call does. (If AGA controls are on, the default
					// button draws its own AGA outline.)
					
					SInt16	itemType;
					Handle	itemHandle;
					Rect	itemRect;

					::GetDialogItem(fDialogPtr, fDefaultItemNo, &itemType, &itemHandle, &itemRect);
					
					if ((itemType & (ctrlItem + btnCtrl)) != 0)
					{
						::InsetRect(&itemRect, -4, -4);

						::PenNormal();
						::PenSize(3, 3);
						::FrameRoundRect(&itemRect, 16, 16);
						::PenNormal();
					}
				}

				break;
		}
	}

	::SetPort(savedPort);

	return wasEventHandled;
}

//----------------------------------------------------------------------------------------
// FilterMouseDown
//----------------------------------------------------------------------------------------

bool CDialog_AC::FilterMouseDown(EventRecord* theEvent, SInt16* itemHit)
{
	bool	wasEventHandled = false;
	Point	localMouse = theEvent->where;
	
	::GlobalToLocal(&localMouse);
	
	//
	// FindDialogItem is unwieldy in cases where things overlap, because
	// our custom items are always appended later in the item list, so
	// non-clickable things in the original list will always be found by
	// FindDialogItem. So we have to check the item locations ourself,
	// and walk the list backwards so that our items get first crack.
	//

	SInt16	foundItemNo = -1;
	SInt16	numItems = ::CountDITL(fDialogPtr);
	
	for (SInt16 i = numItems; (i > fNumAGAControls) && ! wasEventHandled; --i)
	{
		SInt16		itemType;
		Handle		itemHandle;
		Rect		itemRect;

		::GetDialogItem(fDialogPtr, i, &itemType, &itemHandle, &itemRect);
		
		if (::PtInRect(localMouse, &itemRect))
		{
			CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(i - fNumAGAControls);

			if ((infoPtr != NULL) && (infoPtr->fAGAControl != NULL) && infoPtr->fTracksMouse)
			{
				// It's an CControl_AC that wants to track the mouse.
				wasEventHandled = true;

				if (infoPtr->fAGAControl->TrackMouse(localMouse))
					*itemHit = i - fNumAGAControls;
			}
			else
			{
				// See if the original object can track the mouse.
				// This may be a placeholder for a trackable control. For
				// example, we may have had AGA controls turned off when
				// the dialog was initialized.

				::GetDialogItem(fDialogPtr, i - fNumAGAControls, &itemType, &itemHandle, &itemRect);
		
				if (::PtInRect(localMouse, &itemRect) &&
					((itemType & itemDisable) == 0) &&
					((itemType & ctrlItem) != 0))
				{
					wasEventHandled = true;
					
					SInt16	variant = ::GetControlVariant((ControlHandle) itemHandle);

//					if ((variant & popupMenuProc) == popupMenuProc)

					if ((itemType & (ctrlItem + resCtrl)) == (ctrlItem + resCtrl))
					{
						// Once again, the goofball popup CDEF is non-standard.
						// Pass -1 for the UPP, and always return the itemNo.
						(void) ::TrackControl((ControlHandle) itemHandle, localMouse, (ControlActionUPP) -1);
						*itemHit = i - fNumAGAControls;
					}
					else
					{
						if (::TrackControl((ControlHandle) itemHandle, localMouse, NULL) != kControlNoPart)
							*itemHit = i - fNumAGAControls;
					}
				}
			}
		}
	}

	return wasEventHandled;
}

//----------------------------------------------------------------------------------------
// FilterKeyDown
//----------------------------------------------------------------------------------------

bool CDialog_AC::FilterKeyDown(EventRecord* theEvent, SInt16* itemHit)
{
	// We catch command-period, escape, enter, and return.
	// If you're concerned about multi-byte keys, international
	// cancelling, etc., your existing filter will probably
	// be handling this already.

	bool	wasEventHandled = false;
	UInt16	key = (theEvent->message) & charCodeMask;
	bool	isCommandKey = (((theEvent->modifiers) & cmdKey) != 0);

	switch (key)
	{
		case '.':
			if (isCommandKey && (fCancelItemNo != -1))
			{
				this->FlashButton(fCancelItemNo);
				wasEventHandled = true;
				*itemHit = fCancelItemNo;
			}
			break;

		case 0x1B:	// escape
			if (fCancelItemNo != -1)
			{
				this->FlashButton(fCancelItemNo);
				wasEventHandled = true;
				*itemHit = fCancelItemNo;
			}
			break;

		case 0x03:	// enter
		case 0x0D:	// return
			if (fDefaultItemNo != -1)
			{
				this->FlashButton(fDefaultItemNo);
				wasEventHandled = true;
				*itemHit = fDefaultItemNo;
			}
			break;
	}

	return wasEventHandled;
}

//----------------------------------------------------------------------------------------
// InstallAGAUserItem
//----------------------------------------------------------------------------------------

void CDialog_AC::HandleActivateEvent(bool becomeActive)
{
	if (! GetAGAFeatureFlag_AC(kAGADimDeactivation_AC))
		return;

	GrafPtr	savedPort;
	
	::GetPort(&savedPort);
	::SetPort(fDialogPtr);
		
	for (SInt16 i = 1; i <= fNumAGAControls; ++i)
	{
		CControl_AC*	anObject = this->GetAGAControl(i);
		
		if (anObject != NULL)
			anObject->SetActive(becomeActive, CControl_AC::kRedraw);
	}

	::SetPort(savedPort);
}


//----------------------------------------------------------------------------------------
// InstallAGAUserItem
//----------------------------------------------------------------------------------------

void CDialog_AC::InstallAGAUserItem(CControl_AC* anObject, SInt16 itemNo, bool tracksMouse)
{
	// This handy routine lets you install any CControl_AC that you've instantiated
	// as a userItem in the dialog. You just make a userItem for it in your
	// original DITL, and then use this routine to install the CControl_AC.
	// The sample app shows how to do this. It has a Big Dialog Box o Stuff that
	// uses a whole bunch of such userItem CControl_AC controls.

	SInt16	itemType;
	Handle	itemHandle;
	Rect	itemRect;
	
	this->GetDialogItem(itemNo + fNumAGAControls, &itemType, &itemHandle, &itemRect);
	
	anObject->SetBounds(CRect_AC(itemRect), CControl_AC::kDontRedraw);

	CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemNo);
	
	if (infoPtr != NULL)
	{
		infoPtr->fAGAControl = anObject;
		infoPtr->fVisible = true;
		infoPtr->fTracksMouse = tracksMouse;
	}
}

//----------------------------------------------------------------------------------------
// HintPopupControl
//----------------------------------------------------------------------------------------

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

typedef struct
{
	Rect	controlRect;
	SInt16	titlePosition;	// was CNTL value
	bool	visible;
	bool	filler;
	SInt16	titleWidth;		// was CNTL max
	SInt16	menuID;			// was CNTL min
	SInt16	procID;			// variants matter!
	SInt32	refCon;
	Str255	title;			// only used string bytes are in the handle
} PopupControlData, *PopupControlDataPtr, **PopupControlDataHandle;

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

void CDialog_AC::HintPopupControl(SInt16 itemNo, SInt16 cntlRsrcID)
{
	if (EmulateControls_AC())
	{
		PopupControlDataHandle	cntlHandle = (PopupControlDataHandle) ::GetResource('CNTL', cntlRsrcID);
		
		if (cntlHandle != NULL)
		{
			SInt16			itemType;
			Handle			itemHandle;
			Rect			itemRect;
			bool			enabled = true;
			CTempHandleLock_AC	locker((Handle) cntlHandle);

			::GetDialogItem(fDialogPtr, itemNo, &itemType, &itemHandle, &itemRect);
	
			CPopup_AC*	aPopupMenu = TH_new CPopup_AC(CRect_AC(itemRect),//&((**cntlHandle).controlRect),
														(**cntlHandle).titleWidth,
														(**cntlHandle).title,
														(**cntlHandle).titlePosition,
														CTextStyle_AC::kSystemStyle /*check procID*/,
														CTextStyle_AC::kSystemStyle /*check procID*/,
														CPopup_AC::kSystemMDEFAdjustment /*check procID*/,
														(**((PopupPrivateDataHandle) (**((ControlHandle) itemHandle)).contrlData)).mHandle,
														CPopup_AC::kDontDispose,
														CPopup_AC::kReload);

			if ((itemType & itemDisable) != 0)
				aPopupMenu->SetEnable(false, CControl_AC::kDontRedraw);

			CustomObjectInfoPtr	infoPtr = this->GetCustomObjectInfoPtr(itemNo);
			
			if (infoPtr != NULL)
			{
				infoPtr->fAGAControl = aPopupMenu;
				infoPtr->fVisible = (**cntlHandle).visible;
				infoPtr->fTracksMouse = CDialog_AC::kUserItemTracksMouse;

				::HideDialogItem(fDialogPtr, itemNo);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// FlashButton
//----------------------------------------------------------------------------------------

void CDialog_AC::FlashButton(SInt16 itemNo)
{
	// Flash the specified button. If it's an CButton_AC, use the
	// CButton_AC methods. Otherwise, use the Control Manager
	// highlighting functions, assuming it's a button.

	SInt32			dontCare;
	CButton_AC*	buttonToFlash = GetVerifiedPushButton(itemNo);
	
	if (buttonToFlash != NULL)
	{
		buttonToFlash->DrawButton(CControl_AC::kPressed);
		::Delay(8, &dontCare);
		buttonToFlash->DrawButton(CControl_AC::kNotPressed);
	}
	else
	{
		SInt16	itemType;
		Handle	itemHandle;
		Rect	itemRect;

		::GetDialogItem(fDialogPtr, itemNo, &itemType, &itemHandle, &itemRect);
		
		if ((itemType & (ctrlItem + btnCtrl)) != 0)
		{
			// It is indeed a Control Manager button. It is safe to
			// call HiliteControl.
			
			::HiliteControl((ControlHandle) itemHandle, kControlButtonPart);
			::Delay(8, &dontCare);
			::HiliteControl((ControlHandle) itemHandle, kControlNoPart);
		}
	}
}

//----------------------------------------------------------------------------------------
// MarkDefaultOutlineUserItem
//----------------------------------------------------------------------------------------

void CDialog_AC::MarkDefaultOutlineUserItem(SInt16 itemNo)
{
	if (EmulateControls_AC())
	{
		// Remove the userItem proc that is intended to draw the default
		// button outline. The AGA pushbutton control will draw its own.
		
		SInt16	itemType;
		Handle	itemHandle;
		Rect	itemRect;

		::GetDialogItem(fDialogPtr, itemNo, &itemType, &itemHandle, &itemRect);
		::SetDialogItem(fDialogPtr, itemNo, itemType, NULL, &itemRect);
	}
}

//----------------------------------------------------------------------------------------
// UserItem
//----------------------------------------------------------------------------------------

pascal void CDialog_AC::UserItem(WindowPtr theWindow, SInt16 itemNo)
{
	// Static class function.
	// Call the dialog object's user item handling method.

	CDialog_AC*	theAGADialog = CDialog_AC::ObjectFromDialogPtr(theWindow);
	
	if (theAGADialog != NULL)
		theAGADialog->HandleUserItem(itemNo);
}

//----------------------------------------------------------------------------------------
// ModalFilter
//----------------------------------------------------------------------------------------

pascal bool CDialog_AC::ModalFilter(DialogPtr theDialog, EventRecord* theEvent, SInt16* itemHit)
{
	// Static class function.
	// Call the dialog object's filter proc method.

	CDialog_AC*	theAGADialog = CDialog_AC::ObjectFromDialogPtr(theDialog);
	
	if (theAGADialog != NULL)
		return theAGADialog->HandleModalFilter(theEvent, itemHit);
	else
		return false;	// we didn't handle the event
}

//----------------------------------------------------------------------------------------
// ObjectFromDialogPtr
//----------------------------------------------------------------------------------------

CDialog_AC* CDialog_AC::ObjectFromDialogPtr(WindowPtr theWindow)
{
	// Static class function.
	// Return the CDialog_AC pointer, if any, that is wrapping the
	// specified window.

	CDialog_AC*	itsDialog = (CDialog_AC*) ::GetWRefCon(theWindow);
	
	if (itsDialog != NULL)
		if (itsDialog->fMarker != kAGADialogMarker)
			itsDialog = NULL;	// It doesn't seem to actually be an CDialog_AC!
	
	return itsDialog;
}

//----------------------------------------------------------------------------------------
// CTempPort_AC
// CTempPort_AC is a stack-based object that sets the port, and
// restores it upon destruction.
//----------------------------------------------------------------------------------------

CTempPort_AC::CTempPort_AC(GrafPtr itsPort)
{
	this->Save();

	::SetPort(itsPort);
}

//----------------------------------------------------------------------------------------
// CTempPort_AC
//----------------------------------------------------------------------------------------

CTempPort_AC::~CTempPort_AC()
{
	this->Restore();
}

//----------------------------------------------------------------------------------------
// CTempPort_AC
//----------------------------------------------------------------------------------------

void CTempPort_AC::Save()
{
	::GetPort(&fSavedPort);
}

//----------------------------------------------------------------------------------------
// CTempPort_AC
//----------------------------------------------------------------------------------------

void CTempPort_AC::Restore()
{
	::SetPort(fSavedPort);
}

//----------------------------------------------------------------------------------------
// CDialogEditTextFrame_AC 
// CDialogEditTextFrame_AC draws a frame around an editText item in a dialog box.
//----------------------------------------------------------------------------------------

CDialogEditTextFrame_AC::CDialogEditTextFrame_AC(const CRect_AC& bounds)
: CControl_AC(bounds)
{
}

//----------------------------------------------------------------------------------------
// CDialogEditTextFrame_AC
//----------------------------------------------------------------------------------------

CDialogEditTextFrame_AC::~CDialogEditTextFrame_AC()
{
}

//----------------------------------------------------------------------------------------
// CDialogEditTextFrame_AC
//----------------------------------------------------------------------------------------

void CDialogEditTextFrame_AC::DrawObject()
{
	// Draw the frame and erase the interior to white.

	CTempDrawingEnvironment_AC	env;
	CRect_AC					r(GetBounds());
	
	AGADrawBorderFrame(r, true, false, true, kRGBColorRamp[r2]);
	
	kRGBColorRamp[rW].SetForeColor();
	::InsetRect(&r, 2, 2);
	::PenSize(2, 2);
	::FrameRect(&r);
}

//----------------------------------------------------------------------------------------
// CDialogIcon_AC
// CDialogIcon_AC draws a DITL Icon object (a 1-bit 'ICON').
// If an 'icl8' color icon family resource is available, it
// is used instead of the black & white ICON.
//----------------------------------------------------------------------------------------

CDialogIcon_AC::CDialogIcon_AC(const CRect_AC& bounds, SInt16 iconID) : 
	CControl_AC(bounds),
	fIconID(iconID)
{
}

//----------------------------------------------------------------------------------------
// CDialogIcon_AC
//----------------------------------------------------------------------------------------

CDialogIcon_AC::~CDialogIcon_AC()
{
}

//----------------------------------------------------------------------------------------
// CDialogIcon_AC
//----------------------------------------------------------------------------------------

void CDialogIcon_AC::DrawObject()
{
	// If there's an icon family for this ID around, use it.
	// Otherwise, use the black & white ICON.

	Handle	icl8Handle;

	::SetResLoad(false);
	icl8Handle = GetResource('icl8', fIconID);
	::SetResLoad(true);

	kRGBColorRamp[rB].SetForeColor();
	kRGBColorRamp[r2].SetForeColor();
	
	if (icl8Handle != NULL)
		(void) ::PlotIconID(&GetBounds(), atNone, ttNone, fIconID);
	else
		::PlotIcon(&GetBounds(), ::GetIcon(fIconID));
}

