// CPopup_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CPopup_AC__
#include "CPopup_AC.h"
#endif

// Appearance

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

//========================================================================================
// CPopup_AC
// This class implements a standard popup menu look, and handles the
// mouse by calling PopUpMenuSelect such that the visible menu is
// located where the AGA says it should.
//========================================================================================

//----------------------------------------------------------------------------------------
// CPopup_AC::CPopup_AC
//----------------------------------------------------------------------------------------

CPopup_AC::CPopup_AC(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, MenuRef itsMenuRef, DisposalKind menuRefDisposalKind, RefHandling refHandling) : 
	CControl_AC(bounds),
	fMenuID((itsMenuRef == NULL) ? 0 : (**itsMenuRef).menuID),
	fMenuRef(itsMenuRef),
	fMenuRefDisposalKind(menuRefDisposalKind),
	fMenuRefHandling(refHandling),
	fCurrentItemNo(1),
	fWidthAdjust(adjustment),
	fTitleStyle(titleStyle),
	fTextStyle(textStyle),
	fTitle(title),
	fTitleOffset(titleOffset),
	fTitleJustification(titleJustification)
{
	// Construct with specified MenuRef (pass NULL to defer until later).
}

//----------------------------------------------------------------------------------------
// CPopup_AC::CPopup_AC
//----------------------------------------------------------------------------------------

CPopup_AC::CPopup_AC(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, SInt16 itemsStringListID, SInt16 newMenuID) : 
	CControl_AC(bounds),
	fMenuID(newMenuID),
	fMenuRef(::NewMenu(newMenuID, CStr8_AC("AGA"))),
	fMenuRefDisposalKind(kDispose),
	fMenuRefHandling(kNothing),
	fCurrentItemNo(1),
	fWidthAdjust(adjustment),
	fTitleStyle(titleStyle),
	fTextStyle(textStyle),
	fTitle(title),
	fTitleOffset(titleOffset),
	fTitleJustification(titleJustification)
{
	// Construct with menu built from strings in a 'STR#' resource.

	::ThrowIfNULL_AC(fMenuRef);
	
	Handle	stringListHandle = ::GetResource('STR#', itemsStringListID);
	if (stringListHandle != NULL)
	{
		SInt16	numStrings = **((SInt16 **) stringListHandle);
		
		for (SInt16 stringIndex = 1; stringIndex <= numStrings; ++stringIndex)
		{
			Str255	aString;
			
			::GetIndString(aString, itemsStringListID, stringIndex);
			
			// Use SetMenuItemText to avoid metacharacter interpretation.
			::AppendMenu(fMenuRef, CStr8_AC("XXX"));
			::SetMenuItemText(fMenuRef, stringIndex, aString);
		}
	}
}

//----------------------------------------------------------------------------------------
// CPopup_AC::CPopup_AC
//----------------------------------------------------------------------------------------

CPopup_AC::CPopup_AC(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, ResType appendResourceType, SInt16 newMenuID) : 
	CControl_AC(bounds),
	fMenuID(newMenuID),
	fMenuRef(::NewMenu(newMenuID, CStr8_AC("AGA"))),
	fMenuRefDisposalKind(kDispose),
	fMenuRefHandling(kNothing),
	fCurrentItemNo(1),
	fWidthAdjust(adjustment),
	fTitleStyle(titleStyle),
	fTextStyle(textStyle),
	fTitle(title),
	fTitleOffset(titleOffset),
	fTitleJustification(titleJustification)
{
	// Construct with menu built from specified appended resource type.
	// Typical use is for a Fonts menu, using resource type 'FOND'.

	::ThrowIfNULL_AC(fMenuRef);
	
	::AppendResMenu(fMenuRef, appendResourceType);
}

//----------------------------------------------------------------------------------------
// CPopup_AC::~CPopup_AC
//----------------------------------------------------------------------------------------

CPopup_AC::~CPopup_AC()
{
	// Destruct by cleaning up the menu data.
	// This depends on how the menu was created.

	this->DisposeExistingMenuRef();
}

//----------------------------------------------------------------------------------------
// CPopup_AC::DisposeExistingMenuRef
//----------------------------------------------------------------------------------------

void CPopup_AC::DisposeExistingMenuRef()
{
	// Clean up the menu data based on how it was created
	// or how the client code specified.

	if (fMenuRef != NULL)
	{
		if (fMenuRefDisposalKind == kDispose)
			::DisposeMenu(fMenuRef);
		else if (fMenuRefDisposalKind == kRelease)
			::ReleaseResource((Handle) fMenuRef);

		fMenuRef = NULL;
	}

	fMenuRefDisposalKind = kDontDispose;
	fMenuRefHandling = kNothing;
}

//----------------------------------------------------------------------------------------
// CPopup_AC::ReloadMenu
//----------------------------------------------------------------------------------------

void CPopup_AC::ReloadMenu()
{
	if (fMenuRefHandling == kReload)
		fMenuRef = LoadMenuRef(fMenuID);
}

//----------------------------------------------------------------------------------------
// CPopup_AC::GetMenuRef
//----------------------------------------------------------------------------------------

MenuRef	CPopup_AC::LoadMenuRef(SInt16 menuID)
{
	return ::GetMenu(menuID);
}

//----------------------------------------------------------------------------------------
// CPopup_AC::SetMenuRef
//----------------------------------------------------------------------------------------

void CPopup_AC::SetMenuRef(MenuRef itsMenuRef, DisposalKind menuRefDisposalKind, RefHandling refHandling)
{
	// Install the specified MenuRef.

	this->DisposeExistingMenuRef();

	if (itsMenuRef == NULL)
		fMenuID = 0;
	else
		fMenuID = (**itsMenuRef).menuID;

	fMenuRef = itsMenuRef;
	fMenuRefDisposalKind = menuRefDisposalKind;
	fMenuRefHandling = refHandling;
	
	if (refHandling == kDetach)
		::DetachResource((Handle) fMenuRef);
	else if (refHandling == kDuplicate)
		::HandToHand((Handle*) &fMenuRef);

	// It is a no-no to try to release a non-resource handle!
	// If we detached or duplicated, it is no longer a resource.
	// It should be disposed, not released.
	if (((refHandling == kDetach) || (refHandling == kDuplicate))
		&& (menuRefDisposalKind == kRelease))
		fMenuRefDisposalKind = kDispose;

	fCurrentItemNo = 1;
}

//----------------------------------------------------------------------------------------
// CPopup_AC::SetWidthAdjustment
//----------------------------------------------------------------------------------------

void CPopup_AC::SetWidthAdjustment(WidthAdjust adjustment)
{
	// Set the width adjustment, which determines how wide
	// the closed menu appears.

	fWidthAdjust = adjustment;
}

//----------------------------------------------------------------------------------------
// CPopup_AC::DrawObject
//----------------------------------------------------------------------------------------

void CPopup_AC::DrawObject()
{
	// Draw the unpressed popup menu "button".

	this->DrawButton(kNotPressed);
}

//----------------------------------------------------------------------------------------
// CPopup_AC::TrackMouse
//----------------------------------------------------------------------------------------

bool CPopup_AC::TrackMouse(const CPoint_AC& mouseLocation)
{
	if (! IsEnabled())
		return false;

	ReloadMenu();

	// Track the mouse in the popup, and return true
	// if it caused the current popup selection to change.

	bool		selectionChanged;
	SInt32		menuResult;
	CPoint_AC	globalOrigin;
	
	CRect_AC	buttonBounds;
	GetButtonBounds(buttonBounds);
	
	if (buttonBounds.Contains(mouseLocation))
	{
		this->DrawButton(kPressed);
		
		globalOrigin.Set(GetBounds().left + fTitleOffset, GetBounds().bottom);
		::LocalToGlobal(&globalOrigin);

		// Check if the menu is already in the menu bar
		bool inMenuBar = (GetMenuHandle((**fMenuRef).menuID) != NULL); 
		if (!inMenuBar)
			::InsertMenu(fMenuRef, -1);			// put it in the hierarchical list
		
		if ((fTextStyle.tsFont != 0) || ((fTextStyle.tsSize != 0) && (fTextStyle.tsSize != 12)))
			::SetItemMark(fMenuRef, fCurrentItemNo, '');
		else
			::CheckItem(fMenuRef, fCurrentItemNo, true);

		{	// create scope so CTempPopupMenuSizer_AC is destructed right away
			CTempPopupMenuSizer_AC	sizer(fTextStyle.tsFont, fTextStyle.tsSize);
			
			menuResult = ::PopUpMenuSelect(fMenuRef, globalOrigin.v, globalOrigin.h + 1, 1);
		}

		::CheckItem(fMenuRef, fCurrentItemNo, false);

		if (!inMenuBar)
			::DeleteMenu((**fMenuRef).menuID);	// get it out of the hierarchical list

		selectionChanged = (menuResult != 0);

		if (selectionChanged)
			this->SetCurrentItemNo(menuResult & 0x0000FFFF, kDontRedraw);
		
		this->DrawButton(kNotPressed);
	}
	
	return selectionChanged;
}

//----------------------------------------------------------------------------------------
// CPopup_AC::GetCurrentItemNo
//----------------------------------------------------------------------------------------

SInt16 CPopup_AC::GetCurrentItemNo() const
{
	// Return the currently selected item index (1-based).

	return fCurrentItemNo;
}

//----------------------------------------------------------------------------------------
// CPopup_AC::SetCurrentItemNo
//----------------------------------------------------------------------------------------

void CPopup_AC::SetCurrentItemNo(SInt16 newItemNo, bool redraw)
{
	// Set the current item index (1-based), optionally redraw.

	fCurrentItemNo = newItemNo;
	
	if (redraw)
		this->DrawButton(kNotPressed);
}

//----------------------------------------------------------------------------------------
// CPopup_AC::GetCurrentItemText
//----------------------------------------------------------------------------------------

void CPopup_AC::GetCurrentItemText(CStr255_AC& itemText) const
{
	// Return the menu item text of the currently selected item.

	itemText.Empty();
	
	const_cast<CPopup_AC*>(this)->ReloadMenu();
	
	if (fMenuRef != NULL)
		::GetMenuItemText(fMenuRef, this->GetCurrentItemNo(), itemText);
}

//----------------------------------------------------------------------------------------
// CPopup_AC::SetCurrentItemText
//----------------------------------------------------------------------------------------

bool CPopup_AC::SetCurrentItemText(const CStr255_AC& itemTextToMatch, bool redraw)
{
	// Set the current item index to the item whose text
	// matches the specified string. Optionally redraw.

	bool	wasMatchFound = false;
	SInt16	numItems = 0;

	ReloadMenu();
	
	if (fMenuRef != NULL)
		numItems = ::CountMItems(fMenuRef);

	for (SInt16 itemIndex = 1; itemIndex <= numItems; ++itemIndex)
	{
		Str255	itemText;

		::GetMenuItemText(fMenuRef, itemIndex, itemText);

		if (::EqualString(itemText, itemTextToMatch, true, true)) 
		{
			wasMatchFound = true;
			this->SetCurrentItemNo(itemIndex, redraw);
			break;
		}
	}

	return wasMatchFound;
}

//----------------------------------------------------------------------------------------
// CPopup_AC::SetTitle
//----------------------------------------------------------------------------------------

void CPopup_AC::SetTitle(const CStr255_AC& newTitle, bool redraw)
{
	// Install the new title. Optionally redraw.

	fTitle = newTitle;
	
	if (redraw)
		this->DrawButton(kNotPressed);
}

//----------------------------------------------------------------------------------------
// CPopup_AC::DrawButton
//----------------------------------------------------------------------------------------

void CPopup_AC::DrawButton(bool pressed)
{
	// Draw the popup button in the specified state.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (! fEnabled || ! fActive)
			this->DrawButtonDisabled(deep);
		else if (pressed)
			this->DrawButtonPressed(deep);
		else
			this->DrawButtonNormal(deep);
	}
}

//----------------------------------------------------------------------------------------
// CPopup_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CPopup_AC::DrawButtonNormal(bool deep)
	{
	CTempDrawingEnvironment_AC	env;

	// Draw the popup button in enabled/unpressed state.

	CRect_AC	r;
	CStr255_AC	currentItemText;
	
	this->GetCurrentItemText(currentItemText);
	this->GetButtonBounds(r);
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r2].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(r.right - r.left - 6, 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, r.bottom - r.top - 6);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(- (r.right - r.left - 6), 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, -(r.bottom - r.top - 6));
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		
		if (r.right - r.left > kPopupArrowSectionWidth)
		{
			// Add flat section inner pixels.
			kRGBColorRamp[rW].SetForeColor();
			::MoveTo(r.left + 1, r.bottom - 3);
			::LineTo(r.left + 1, r.top + 2);
			::Move(1, -1);
			::LineTo(r.right - kPopupArrowSectionWidth, r.top + 1);
			kRGBColorRamp[r1].SetForeColor();
			::Line(0, 0);
			kRGBColorRamp[r5].SetForeColor();
			::MoveTo(r.left + 2, r.bottom - 2);
			::LineTo(r.right - kPopupArrowSectionWidth, r.bottom - 2);
			::LineTo(r.right - kPopupArrowSectionWidth, r.top + 2);
			
			// Add arrow section left corners
			kRGBColorRamp[r4].SetForeColor();
			::MoveTo(r.right - kPopupArrowSectionWidth + 1, r.bottom - 2);
			::Line(0, 0);
			kRGBColorRamp[r2].SetForeColor();
			::MoveTo(r.right - kPopupArrowSectionWidth + 1, r.top + 1);
			::Line(0, 0);
		}

		// Add arrow section shading.
		kRGBColorRamp[r8].SetForeColor();
		::MoveTo(r.right - kPopupArrowSectionWidth + 2, r.bottom - 2);
		::LineTo(r.right - 3, r.bottom - 2);
		::Line(0, -1);
		::Line(1, 0);
		::LineTo(r.right - 2, r.top + 3);
		::Move(-1, -2);
		kRGBColorRamp[r4].SetForeColor();
		::Line(1, 1);
		kRGBColorRamp[r5].SetForeColor();
		::Line(-1, 1);
		::LineTo(r.right - 3, r.bottom - 4);
		::Move(-1, 1);
		::LineTo(r.right - kPopupArrowSectionWidth + 3, r.bottom - 3);
		kRGBColorRamp[rW].SetForeColor();
		::MoveTo(r.right - kPopupArrowSectionWidth + 2, r.bottom - 4);
		::LineTo(r.right - kPopupArrowSectionWidth + 2, r.top + 2);
		::LineTo(r.right - 4, r.top + 2);
	}
	else	// 1-bit
	{
		kRGBColorRamp[rW].SetForeColor();
		::PaintRoundRect(&r, 7, 7);
		kRGBColorRamp[rB].SetForeColor();
		::FrameRoundRect(&r, 7, 7);
	}
	
	// Add arrow.
	kRGBColorRamp[rB].SetForeColor();
	::MoveTo(r.right - kPopupArrowSectionWidth + 6, r.top + ((r.bottom - r.top) / 2) - 2);
	::Line(8, 0);
	::Move(-1, 1);
	::Line(-6, 0);
	::Move(1, 1);
	::Line(4, 0);
	::Move(-1, 1);
	::Line(-2, 0);
	::Line(1, 1);

	if (r.right - r.left > kPopupArrowSectionWidth)
	{
		// Draw the current item text.
		r.left += 7;
		r.right -= kPopupArrowSectionWidth;
		AGAStringOut(currentItemText, r, truncEnd, teFlushDefault, kNormalOutput, deep, fTextStyle);
	}

	// If the popup has a title, draw it next to the button.	
	if (!fTitle.IsEmpty())
	{
		this->GetTitleBounds(r);
		AGAStringOut(fTitle, r, truncEnd, fTitleJustification, kNormalOutput, deep, fTitleStyle);
	}
}

//----------------------------------------------------------------------------------------
// CPopup_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CPopup_AC::DrawButtonPressed(bool deep)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the popup button in enabled/pressed state.

	CRect_AC	r;
	CStr255_AC	currentItemText;
	
	this->GetCurrentItemText(currentItemText);
	this->GetButtonBounds(r);
	
	::PenSize(1, 1);

	if (deep)
		{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r9].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(r.right - r.left - 6, 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, r.bottom - r.top - 6);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, 1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, 0);
		kRGBColorRamp[rB].SetForeColor();
		::Line(- (r.right - r.left - 6), 0);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, 0);
		::Move(-1, -1);
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		::Move(0, -1);
		kRGBColorRamp[rB].SetForeColor();
		::Line(0, -(r.bottom - r.top - 6));
		kRGBColorRamp[r12].SetForeColor();
		::Line(0, 0);
		
		if (r.right - r.left > kPopupArrowSectionWidth)
		{
			// Add flat section inner pixels.
			kRGBColorRamp[r10].SetForeColor();
			::MoveTo(r.left + 1, r.bottom - 3);
			::LineTo(r.left + 1, r.top + 2);
			::Line(1, 0);
			::Line(0, -1);
			::LineTo(r.right - kPopupArrowSectionWidth, r.top + 1);
			kRGBColorRamp[r8].SetForeColor();
			::MoveTo(r.left + 2, r.bottom - 2);
			::LineTo(r.right - kPopupArrowSectionWidth, r.bottom - 2);
			::LineTo(r.right - kPopupArrowSectionWidth, r.top + 2);
			
			// Add arrow section left corners
			kRGBColorRamp[r11].SetForeColor();
			::MoveTo(r.right - kPopupArrowSectionWidth + 1, r.bottom - 2);
			::Line(0, 0);
			::MoveTo(r.right - kPopupArrowSectionWidth + 1, r.top + 1);
			::Line(0, 0);
		}

		// Add arrow section shading.
		kRGBColorRamp[r7].SetForeColor();
		::MoveTo(r.right - kPopupArrowSectionWidth + 2, r.bottom - 2);
		::LineTo(r.right - 3, r.bottom - 2);
		::Line(0, -1);
		::Line(1, 0);
		::LineTo(r.right - 2, r.top + 2);
		::Move(-1, 1);
		kRGBColorRamp[r8].SetForeColor();
		::LineTo(r.right - 3, r.bottom - 4);
		::Move(-1, 1);
		::LineTo(r.right - kPopupArrowSectionWidth + 3, r.bottom - 3);
		kRGBColorRamp[r10].SetForeColor();
		::MoveTo(r.right - kPopupArrowSectionWidth + 2, r.bottom - 4);
		::LineTo(r.right - kPopupArrowSectionWidth + 2, r.top + 2);
		::LineTo(r.right - 4, r.top + 2);
		kRGBColorRamp[r11].SetForeColor();
		::MoveTo(r.right - kPopupArrowSectionWidth + 1, r.bottom - 3);
		::LineTo(r.right - kPopupArrowSectionWidth + 1, r.top + 2);
		::Move(1, -1);
		::LineTo(r.right - 3, r.top + 1);
	}
	else	// 1-bit
	{
		kRGBColorRamp[rB].SetForeColor();
		::PaintRoundRect(&r, 7, 7);
	}

	// Add arrow.
	
	kRGBColorRamp[rW].SetForeColor();
	::MoveTo(r.right - kPopupArrowSectionWidth + 6, r.top + ((r.bottom - r.top) / 2) - 2);
	::Line(8, 0);
	::Move(-1, 1);
	::Line(-6, 0);
	::Move(1, 1);
	::Line(4, 0);
	::Move(-1, 1);
	::Line(-2, 0);
	::Line(1, 1);

	if (r.right - r.left > kPopupArrowSectionWidth)
	{
		// Draw the current item text.
		r.left += 7;
		r.right -= kPopupArrowSectionWidth;
		AGAStringOut(currentItemText, r, truncEnd, teFlushDefault, kInverseOutput, deep, fTextStyle);
	}

	// (No need to draw the title, since it does not change
	//  upon pressing the button.)
}

//----------------------------------------------------------------------------------------
// CPopup_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CPopup_AC::DrawButtonDisabled(bool deep)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the popup button in disabled/unpressed state.

	CRect_AC	r;
	CStr255_AC	currentItemText;
	
	this->GetCurrentItemText(currentItemText);
	this->GetButtonBounds(r);
	
	::PenSize(1, 1);

	if (deep)
	{
		// Fill the interior.
		r.Inset(1, 1);
		kRGBColorRamp[r2].SetForeColor();
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		// Draw the frame.
		::MoveTo(r.left + 1, r.top + 1);
		kRGBColorRamp[r7].SetForeColor();
		::Line(0, 0);
		::Move(1, -1);
		::Line(0, 0);
		::Move(1, 0);
		::Line(r.right - r.left - 6, 0);
		::Line(0, 0);
		::Move(1, 1);
		::Line(0, 0);
		::Move(1, 1);
		::Line(0, 0);
		::Move(0, 1);
		::Line(0, r.bottom - r.top - 6);
		::Line(0, 0);
		::Move(-1, 1);
		::Line(0, 0);
		::Move(-1, 1);
		::Line(0, 0);
		::Move(-1, 0);
		::Line(- (r.right - r.left - 6), 0);
		::Line(0, 0);
		::Move(-1, -1);
		::Line(0, 0);
		::Move(-1, -1);
		::Line(0, 0);
		::Move(0, -1);
		::Line(0, -(r.bottom - r.top - 6));
		::Line(0, 0);

		if (r.right - r.left > kPopupArrowSectionWidth)
		{
			kRGBColorRamp[r5].SetForeColor();
			::MoveTo(r.right - kPopupArrowSectionWidth, r.bottom - 2);
			::LineTo(r.right - kPopupArrowSectionWidth, r.top + 2);
		}
	}
	else	// 1-bit
	{
		kRGBColorRamp[rW].SetForeColor();
		::PaintRoundRect(&r, 7, 7);
		kRGBColorRamp[rB].SetForeColor();
		::FrameRoundRect(&r, 7, 7);
	}

	// Add arrow.
	kRGBColorRamp[r7].SetForeColor();
	::MoveTo(r.right - kPopupArrowSectionWidth + 6, r.top + ((r.bottom - r.top) / 2) - 2);
	::Line(8, 0);
	::Move(-1, 1);
	::Line(-6, 0);
	::Move(1, 1);
	::Line(4, 0);
	::Move(-1, 1);
	::Line(-2, 0);
	::Line(1, 1);

	if (r.right - r.left > kPopupArrowSectionWidth)
	{
		// Draw the current item text.
		r.left += 7;
		r.right -= kPopupArrowSectionWidth;
		AGAStringOut(currentItemText, r, truncEnd, teFlushDefault, kDisabledOutput, deep, fTextStyle);
	}
	
	// If the popup has a title, draw it next to the button.	
	if (!fTitle.IsEmpty())
	{
		this->GetTitleBounds(r);
		AGAStringOut(fTitle, r, truncEnd, fTitleJustification, kDisabledOutput, deep, fTitleStyle);
	}
	
	if (! deep)
	{
		kRGBColorRamp[rB].SetForeColor();
		PenPatGray();
		::PenMode(patBic);
		::PaintRect(GetBounds());

		::PenNormal();
	}
}

//----------------------------------------------------------------------------------------
// CPopup_AC::GetButtonBounds
//----------------------------------------------------------------------------------------

void CPopup_AC::GetButtonBounds(CRect_AC& buttonBounds)
{
	// Return the bounds of the popup button. We may need
	// to check the actual menu's width, depending on the
	// width adjustment specifier.

	buttonBounds = GetBounds();
	buttonBounds.left += fTitleOffset;

	ReloadMenu();
		
	if ((fMenuRef != NULL) && (fWidthAdjust != kFixedWidth))
	{
		CTempPopupMenuSizer_AC	sizer(fTextStyle.tsFont, fTextStyle.tsSize);

		::CalcMenuSize(fMenuRef);
		buttonBounds.right = MinSInt16(buttonBounds.right, buttonBounds.left + 3 + (**fMenuRef).menuWidth + fWidthAdjust);
	}
}

//----------------------------------------------------------------------------------------
// CPopup_AC::GetTitleBounds
//----------------------------------------------------------------------------------------

void CPopup_AC::GetTitleBounds(CRect_AC& titleBounds)
{
	// Return the bounds of the title area.

	titleBounds = GetBounds();
	
	titleBounds.right = titleBounds.left + fTitleOffset - 3;
}

//========================================================================================
// CTempPopupMenuSizer_AC
// Stack-based object to hack in a custom popup menu font/size
// around a call to PopUpMenuSelect or CalcMenuSize.
//========================================================================================

//----------------------------------------------------------------------------------------
// CTempPopupMenuSizer_AC::CTempPopupMenuSizer_AC
//----------------------------------------------------------------------------------------

CTempPopupMenuSizer_AC::CTempPopupMenuSizer_AC(SInt16 fontNum, SInt16 fontSize) :
	fCustomFontNum(fontNum),
	fCustomFontSize(fontSize),
	fIsCustom((fontNum != 0) || ((fontSize != 0) && (fontSize != 12)))
{
	this->SaveEnvironment();
	this->InstallCustomEnvironment();
}

//----------------------------------------------------------------------------------------
// CTempPopupMenuSizer_AC::~CTempPopupMenuSizer_AC
//----------------------------------------------------------------------------------------

CTempPopupMenuSizer_AC::~CTempPopupMenuSizer_AC()
{
	this->RestoreEnvironment();
}

//----------------------------------------------------------------------------------------
// CTempPopupMenuSizer_AC::SaveEnvironment
//----------------------------------------------------------------------------------------

void CTempPopupMenuSizer_AC::SaveEnvironment()
{
	if (fIsCustom)
	{
		fSavedFontNum = ::LMGetSysFontFam();
		fSavedFontSize = ::LMGetSysFontSize();
	}
}

//----------------------------------------------------------------------------------------
// CTempPopupMenuSizer_AC::RestoreEnvironment
//----------------------------------------------------------------------------------------

void CTempPopupMenuSizer_AC::RestoreEnvironment()
{
	if (fIsCustom)
	{
		::LMSetSysFontFam(fSavedFontNum);
		::LMSetSysFontSize(fSavedFontSize);
		::LMSetLastSPExtra(-1);
	}
}

//----------------------------------------------------------------------------------------
// CTempPopupMenuSizer_AC::InstallCustomEnvironment
//----------------------------------------------------------------------------------------

void CTempPopupMenuSizer_AC::InstallCustomEnvironment()
{
	if (fIsCustom)
	{
		::LMSetSysFontFam(fCustomFontNum);
		::LMSetSysFontSize(fCustomFontSize);
		::LMSetLastSPExtra(-1);
	}
}
