// CProgressIndicator_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CProgressIndicator_AC__
#include "CProgressIndicator_AC.h"
#endif

// Appearance

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __ImagingUtils_AC__
#include "ImagingUtils_AC.h"
#endif

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

//========================================================================================
// CProgressIndicator_AC
// This class implements a progress indicator that can be either
// determinate or indeterminate, and also can have a moving start
// value.
//========================================================================================

bool CProgressIndicator_AC::fgImagesAllocated = false;

// We use an offscreen image for the indeterminate progress pattern.
CAutoPtr_AC<CSimpleOffscreenImage_AC>	CProgressIndicator_AC::fgOffscreenIndeterminateProgressImage[kNumProgressPatterns];

// These are the color ramp indexes that are used to build the offscreen image.
SInt8				CProgressIndicator_AC::kIndeterminateProgressImage[kNumProgressPatterns]
																[kProgressImageHeight]
																	[kProgressImageWidth] =
		{
		{	// k1BitPattern
			{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
			{  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
			{  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
			{  rW,  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW,  rW,  rW,  rW,  rW },
			{  rW,  rW,  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW,  rW,  rW,  rW },
			{  rW,  rW,  rW,  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW,  rW,  rW },
			{  rW,  rW,  rW,  rW,  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW,  rW },
			{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rW },
			{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
			{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		},
		{	// kDeepBluePattern
			{ rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP, reP, reP, reP, reP, reP, reP, reP },
			{ reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP, reP, reP, reP, reP, reP, reP },
			{ reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP, reP, reP, reP, reP, reP },
			{ reP, reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP, reP, reP, reP, reP },
			{ reP, reP, reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP, reP, reP, reP },
			{ reP, reP, reP, reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP, reP, reP },
			{ reP, reP, reP, reP, reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP, reP },
			{ reP, reP, reP, reP, reP, reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, reP },
			{ reP, reP, reP, reP, reP, reP, reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP, rFP },
			{ rFP, reP, reP, reP, reP, reP, reP, reP, reP, rFP, rFP, rFP, rFP, rFP, rFP, rFP },
		},
		{	// kDeepGrayscalePattern
			{ r11, r11, r11, r11, r11, r11, r11, r11, r10, r10, r10, r10, r10, r10, r10, r10 },
			{  r8, r10, r10, r10, r10, r10, r10, r10, r10,  r8,  r8,  r8,  r8,  r8,  r8,  r8 },
			{  r5,  r5,  r8,  r8,  r8,  r8,  r8,  r8,  r8,  r8,  r5,  r5,  r5,  r5,  r5,  r5 },
			{  r2,  r2,  r2,  r5,  r5,  r5,  r5,  r5,  r5,  r5,  r5,  r2,  r2,  r2,  r2,  r2 },
			{  rW,  rW,  rW,  rW,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  rW,  rW,  rW,  rW },
			{  r2,  r2,  r2,  r2,  r2,  r5,  r5,  r5,  r5,  r5,  r5,  r5,  r5,  r2,  r2,  r2 },
			{  r4,  r4,  r4,  r4,  r4,  r4,  r8,  r8,  r8,  r8,  r8,  r8,  r8,  r8,  r4,  r4 },
			{  r6,  r6,  r6,  r6,  r6,  r6,  r6, r10, r10, r10, r10, r10, r10, r10, r10,  r6 },
			{  r8,  r8,  r8,  r8,  r8,  r8,  r8,  r8, r11, r11, r11, r11, r11, r11, r11, r11 },
			{ r12, r10, r10, r10, r10, r10, r10, r10, r10, r12, r12, r12, r12, r12, r12, r12 },
		},
	};


//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::CProgressIndicator_AC
//----------------------------------------------------------------------------------------

CProgressIndicator_AC::CProgressIndicator_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum) :
	CControl_AC(bounds),
	fMinimum(minimum),
	fMaximum(maximum),
	fValue(minimum),
	fOriginValue(minimum),
	fAnimationIndex(0),
	fLastAnimationTime(0)
{
	// Construct with default options.
	// (minimum == maximum) means indeterminate indicator.
	if (! fgImagesAllocated)
		AllocateProgressImages();
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::~CProgressIndicator_AC
//----------------------------------------------------------------------------------------

CProgressIndicator_AC::~CProgressIndicator_AC()
{
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawObject
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawObject()
{
	// Draw at current gauge value or next animation step.

	this->DrawFrame();
	
	if (fMinimum == fMaximum)
		this->DrawAnimationStep();
	else
		this->DrawGauge();
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::GetValue
//----------------------------------------------------------------------------------------

SInt32 CProgressIndicator_AC::GetValue() const
{
	// Return current value.

	return fValue;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::SetValue
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::SetValue(SInt32 newValue, bool redraw)
{
	// Set gauge value, optionally redraw.

	// Constrain: fMinimum <= fValue <= fMaximum
	newValue = MaxSInt32(fMinimum, MinSInt32(newValue, fMaximum));
	
	if (newValue != fValue)
	{
		// Modest optimization based on Ken Victor's suggestion:
		// if there is no user-visible change, don't draw the gauge		
		CRect_AC frame;
		GetRangeFrame(fValue, newValue, frame);
		if (frame.right == frame.left)
			redraw = false;
		
		fValue = newValue;
		
		if (redraw)
			this->DrawGauge();
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::Increment
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::Increment(SInt32 delta, bool redraw)
{
	// Increment gauge value, optionally redraw.

	this->SetValue(delta + this->GetValue(), redraw);
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::Animate
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::Animate()
{
	// Draw the next animation step.

	fAnimationIndex = (fAnimationIndex + 1) % kNumProgressAnimationSteps;
	
	this->DrawAnimationStep();
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::QuantizedAnimate
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::QuantizedAnimate()
{
	// Draw the next animation step if enough time has elapsed since the
	// last animation step.
	
	if (fMinimum == fMaximum)
	{
		SInt32	now = ::TickCount();
		
		if (now > fLastAnimationTime + kAnimationDelay)
		{
			fLastAnimationTime = now;
			this->Animate();
		}
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CProgressIndicator_AC::GetMinimum() const
{
	return fMinimum;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CProgressIndicator_AC::GetMaximum() const
{
	return fMaximum;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::SetRange
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::SetRange(SInt32 newMinimum, SInt32 newMaximum, bool redraw)
{
	// Set the gauge value range, optionally redraw.
	// (minimum == maximum) means indeterminate indicator.

	if (fOriginValue == fMinimum)
		fOriginValue = newMinimum;

	fMinimum = newMinimum;
	fMaximum = MaxSInt32(newMinimum, newMaximum);
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::GetOriginValue
//----------------------------------------------------------------------------------------

SInt32 CProgressIndicator_AC::GetOriginValue()
{
	// Return the optional gauge origin value.

	return fOriginValue;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::SetOriginValue
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::SetOriginValue(SInt32 newValue, bool redraw)
{
	// Set the optional gauge origin value.

	// Constrain: fMinimum <= fOriginValue <= fValue
	newValue = MaxSInt32(fMinimum, MinSInt32(newValue, fValue));

	if (newValue != fOriginValue)
	{
		fOriginValue = newValue;
		
		if (redraw)
			this->DrawObject();
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawFrame
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawFrame()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the constant frame around the indicator.
	CRect_AC	r(GetBounds());
	
	r.bottom = r.top + kProgressHeight;
	

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			kRGBColorRamp[r5].SetForeColor();
			::MoveTo(r.left, r.bottom - 2);
			::LineTo(r.left, r.top);
			::LineTo(r.right - 2, r.top);
			
			kRGBColorRamp[rW].SetForeColor();
			::MoveTo(r.left + 1, r.bottom - 1);
			::LineTo(r.right - 1, r.bottom - 1);
			::LineTo(r.right - 1, r.top + 1);
		}
		
		r.Inset(1, 1);
		kRGBColorRamp[rB].SetForeColor();
		::FrameRect(&r);
		r.Inset(-1, -1);
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawAnimationStep
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawAnimationStep()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the indeterminate progress indicator at the
	// current animation step.

	const UInt32 kNumRows = 10;

	CRect_AC	r(GetBounds());
	SInt16	patternHOrigin;
	
	r.bottom = r.top + kProgressHeight;
	r.Inset(2, 2);
	
	//
	// Move the pen leftward so the pattern is offset for
	// the current animation step. This will mean we need
	// to explicitly clip to our gauge area, since our pattern
	// origin is outside it.
	//

	patternHOrigin = r.left - 32 + ((16 / kNumProgressAnimationSteps) * fAnimationIndex);
	
	RgnHandle	savedClip = ::NewRgn();
	::GetClip(savedClip);
	
	ClipFurtherTo_AC(r);
	
	r.left = patternHOrigin;

	DrawAnimationPattern(r);

	::SetClip(savedClip);
	::DisposeRgn(savedClip);
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawAnimationPattern
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawAnimationPattern(const CRect_AC& clipRect)
{
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
			fgOffscreenIndeterminateProgressImage[kDeepBluePattern]->DrawPattern(clipRect);
		else
			fgOffscreenIndeterminateProgressImage[k1BitPattern]->DrawPattern(clipRect);
	}
}	// force CGDIterator_AC clip to clean up now, not after we set savedClip!

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawGauge
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawGauge()
{
	// Draw the determinate progress indicator.

	if (fOriginValue != fMinimum)
		this->DrawOriginArea();
	
	this->DrawProgressArea();
	this->DrawRemainderArea();
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawOriginArea
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawOriginArea()
{
	// Draw the empty track between the minimum value
	// and the origin value.

	CRect_AC	frame;
	
	if (this->GetOriginFrame(frame))
	{
		CGDIterator_AC	iter;
		bool			deep;

		while (iter.NextDevice(deep))
		{
			CRect_AC	r(frame);

			if (deep)
			{
				kRGBColorRamp[reP].SetForeColor();
				::PaintRect(&r);

			}
			else	// 1-bit
			{
				kRGBColorRamp[rW].SetForeColor();
				::PaintRect(&r);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawProgressArea
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawProgressArea()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the filled progress gauge area between
	// the origin and the current value.

	CRect_AC	frame;
	
	if (this->GetProgressFrame(frame))
	{
		CGDIterator_AC	iter;
		bool			deep;

		while (iter.NextDevice(deep))
		{
			CRect_AC	r(frame);

			if (deep)
			{
				kRGBColorRamp[rFP].SetForeColor();
				::PaintRect(&r);
			}
			else	// 1-bit
			{
				kRGBColorRamp[rB].SetForeColor();
				::PaintRect(&r);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DrawRemainderArea
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DrawRemainderArea()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the empty track area between the current
	// gauge value and the maximum.

	CRect_AC	frame;
	
	if (this->GetRemainderFrame(frame))
	{
		CGDIterator_AC	iter;
		bool			deep;

		while (iter.NextDevice(deep))
		{
			CRect_AC	r(frame);

			if (deep)
			{
				kRGBColorRamp[reP].SetForeColor();
				::PaintRect(&r);
			}
			else	// 1-bit
			{
				kRGBColorRamp[rW].SetForeColor();
				::PaintRect(&r);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::GetOriginFrame
//----------------------------------------------------------------------------------------

bool CProgressIndicator_AC::GetOriginFrame(CRect_AC& originFrame)
{
	// Return true (and the area) if there is an area
	// to be drawn for the empty origin part of the track.

	bool	isNeeded = false;
	
	if (fOriginValue != fMinimum)
	{
		isNeeded = true;

		this->GetRangeFrame(fMinimum, fOriginValue, originFrame);
		
		if (fOriginValue != fMaximum)
			--originFrame.right;
	}

	return isNeeded;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::GetProgressFrame
//----------------------------------------------------------------------------------------

bool CProgressIndicator_AC::GetProgressFrame(CRect_AC& progressFrame)
{
	// Return true (and the area) if there is an area
	// to be drawn for the progress section of the gauge.

	bool	isNeeded = false;
	
	if (fValue != fOriginValue)
	{
		isNeeded = true;

		this->GetRangeFrame(fOriginValue, fValue, progressFrame);
		
		if (fValue == fMaximum)
			++progressFrame.right;
	}

	return isNeeded;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::GetRemainderFrame
//----------------------------------------------------------------------------------------

bool CProgressIndicator_AC::GetRemainderFrame(CRect_AC& remainderFrame)
{
	// Return true (and the area) if there is an area
	// to be drawn for the uncompleted gauge remainder area.

	bool	isNeeded = false;
	
	if (fMaximum != fValue)
	{
		isNeeded = true;

		this->GetRangeFrame(fValue, fMaximum, remainderFrame);
	}

	return isNeeded;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::GetRangeFrame
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::GetRangeFrame(SInt32 startValue, SInt32 endValue, CRect_AC& rangeFrame)
{
	// Return the rectangle that the indicator draws in,
	// which does not include the constant border frame area.

	rangeFrame = GetBounds();
	rangeFrame.bottom = rangeFrame.top + kProgressHeight;
	
	rangeFrame.Inset(2, 2);

	SInt32	divisor = fMaximum - fMinimum;
	
	if (divisor != 0)	// avoid divide by zero
	{
		SInt32	totalWidth = rangeFrame.right - rangeFrame.left;
		float	startFraction = ((float) (startValue - fMinimum)) / divisor;
		float	endFraction = ((float) (endValue - fMinimum)) / divisor;

		rangeFrame.right = rangeFrame.left + (endFraction * totalWidth);
		rangeFrame.left = rangeFrame.left + (startFraction * totalWidth);
	}
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::AllocateProgressImages
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::AllocateProgressImages()
{
	// Allocate the offscreen image for progress patterns

	CPoint_AC	imageSize;
	
	imageSize.h = kProgressImageWidth;
	imageSize.v = kProgressImageHeight;
	
	for (UInt32 imageIndex = 0; imageIndex < kNumProgressPatterns; ++imageIndex)
	{
		{
			fgOffscreenIndeterminateProgressImage[imageIndex] = TH_new CSimpleOffscreenImage_AC;
			
			fgOffscreenIndeterminateProgressImage[imageIndex]->CreateImageData(
						imageSize,
						&kIndeterminateProgressImage[imageIndex][0][0],
						NULL, 
						kRGBColorRamp[rB]);
		}
	}
		
	fgImagesAllocated = true;
}

//----------------------------------------------------------------------------------------
// CProgressIndicator_AC::DisposeProgressImages
//----------------------------------------------------------------------------------------

void CProgressIndicator_AC::DisposeProgressImages()
{
	for (UInt32 imageIndex = 0; imageIndex < kNumProgressPatterns; ++imageIndex)
		if (!fgOffscreenIndeterminateProgressImage[imageIndex].IsNULL())
			fgOffscreenIndeterminateProgressImage[imageIndex].reset();
}

//----------------------------------------------------------------------------------------
// CGrayscaleProgressIndicator_AC constructor
//----------------------------------------------------------------------------------------

CGrayscaleProgressIndicator_AC::CGrayscaleProgressIndicator_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum) :
	CProgressIndicator_AC(bounds, minimum, maximum)
{
}

//----------------------------------------------------------------------------------------
// CGrayscaleProgressIndicator_AC destructor
//----------------------------------------------------------------------------------------

CGrayscaleProgressIndicator_AC::~CGrayscaleProgressIndicator_AC()
{
}

//----------------------------------------------------------------------------------------
// CGrayscaleProgressIndicator_AC::DrawAnimationPattern
//----------------------------------------------------------------------------------------

void CGrayscaleProgressIndicator_AC::DrawAnimationPattern(const CRect_AC& clipRect)
{
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
			fgOffscreenIndeterminateProgressImage[kDeepGrayscalePattern]->DrawPattern(clipRect);
		else
			fgOffscreenIndeterminateProgressImage[k1BitPattern]->DrawPattern(clipRect);
	}
}	// force CGDIterator_AC clip to clean up now, not after we set savedClip!

//----------------------------------------------------------------------------------------
// CGrayscaleProgressIndicator_AC::DrawOriginArea
//----------------------------------------------------------------------------------------

void CGrayscaleProgressIndicator_AC::DrawOriginArea()
{
	// Draw the empty track between the minimum value
	// and the origin value.

	CRect_AC	frame;
	
	if (this->GetOriginFrame(frame))
	{
		CGDIterator_AC	iter;
		bool			deep;

		while (iter.NextDevice(deep))
		{
			CRect_AC	r(frame);

			if (deep)
			{
				SInt16	width = r.right - r.left;
				
				if (width >= 0)
				{
					kRGBColorRamp[rB].SetForeColor();
					::MoveTo(r.right, r.top);
					::LineTo(r.right, r.bottom - 1);
				}

				if (width > 0)
				{
					kRGBColorRamp[r10].SetForeColor();
					::MoveTo(r.left, r.top);
					::LineTo(r.left, r.bottom - 1);
				}
				
				if (width > 1)
				{
					kRGBColorRamp[r4].SetForeColor();
					::MoveTo(r.right - 1, r.top);
					::Line(0, 1);
					kRGBColorRamp[r2].SetForeColor();
					::LineTo(r.right - 1, r.bottom - 1);
				}
				
				if (width > 2)
				{
					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(r.left + 1, r.top);
					::LineTo(r.left + 1, r.bottom - 1);
				}
				
				if (width > 3)
				{
					kRGBColorRamp[r2].SetForeColor();
					::MoveTo(r.left + 2, r.bottom - 1);
					::LineTo(r.right - 1, r.bottom - 1);

					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(r.left + 2, r.top);
					::LineTo(r.right - 2, r.top);

					kRGBColorRamp[r4].SetForeColor();
					r.left += 2;
					++r.top;
					--r.bottom;
					--r.right;
					::PaintRect(&r);
				}
			}
			else	// 1-bit
			{
				kRGBColorRamp[rW].SetForeColor();
				::PaintRect(&r);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CGrayscaleProgressIndicator_AC::DrawProgressArea
//----------------------------------------------------------------------------------------

void CGrayscaleProgressIndicator_AC::DrawProgressArea()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the filled progress gauge area between
	// the origin and the current value.

	CRect_AC	frame;
	
	if (this->GetProgressFrame(frame))
	{
		CGDIterator_AC	iter;
		bool			deep;

		while (iter.NextDevice(deep))
		{
			CRect_AC	r(frame);

			if (deep)
			{
				SInt16	width = r.right - r.left;

				//
				// Because the endcaps of the gauge are shaded,
				// we need to carefully draw the ends horizontal
				// pixel by pixel, bailing out if there is not
				// enough width to accomodate the endcaps and the
				// midsection.
				//

				if (width > 0)
				{
					kRGBColorRamp[rB].SetForeColor();
					::MoveTo(r.right - 1, r.top);
					::LineTo(r.right - 1, r.bottom - 1);
				}

				if (width > 1)
				{
					kRGBColorRamp[r8].SetForeColor();
					::MoveTo(r.left, r.top);
					::LineTo(r.left, r.bottom - 1);
				}

				if (width > 2)
				{
					kRGBColorRamp[r10].SetForeColor();
					::MoveTo(r.right - 2, r.top);
					::Line(0, 1);
					kRGBColorRamp[r12].SetForeColor();
					::LineTo(r.right - 2, r.bottom - 1);
				}

				if (width > 3)
				{
					kRGBColorRamp[r8].SetForeColor();
					::MoveTo(r.left + 1, r.top);
					::Line(0, 1);
					kRGBColorRamp[r5].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r3].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r1].SetForeColor();
					::Line(0, 3);
					kRGBColorRamp[r3].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r5].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r8].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r10].SetForeColor();
					::Line(0, 0);
				}

				if (width > 4)
				{
					kRGBColorRamp[r10].SetForeColor();
					::MoveTo(r.right - 3, r.top);
					::Line(0, 1);
					kRGBColorRamp[r8].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r10].SetForeColor();
					::LineTo(r.right - 3, r.bottom - 1);
					kRGBColorRamp[r12].SetForeColor();
					::Line(0, 0);
				}

				if (width > 5)
				{
					kRGBColorRamp[r10].SetForeColor();
					::MoveTo(r.right - 4, r.top);
					::Line(0, 1);
					kRGBColorRamp[r8].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r5].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r3].SetForeColor();
					::Line(0, 3);
					kRGBColorRamp[r5].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r8].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r10].SetForeColor();
					::Line(0, 1);
					kRGBColorRamp[r12].SetForeColor();
					::Line(0, 0);
				}

				if (width > 6)
				{
					SInt16	distance = width - 7;
					
					kRGBColorRamp[r10].SetForeColor();
					::MoveTo(r.left + 2, r.top);
					::Line(distance, 0);
					kRGBColorRamp[r8].SetForeColor();
					::MoveTo(r.left + 2, r.top + 1);
					::Line(distance, 0);
					kRGBColorRamp[r5].SetForeColor();
					::MoveTo(r.left + 2, r.top + 2);
					::Line(distance, 0);
					kRGBColorRamp[r3].SetForeColor();
					::MoveTo(r.left + 2, r.top + 3);
					::Line(distance, 0);
					kRGBColorRamp[r1].SetForeColor();
					::MoveTo(r.left + 2, r.top + 4);
					::Line(distance, 0);
					kRGBColorRamp[r3].SetForeColor();
					::MoveTo(r.left + 2, r.top + 5);
					::Line(distance, 0);
					kRGBColorRamp[r5].SetForeColor();
					::MoveTo(r.left + 2, r.top + 6);
					::Line(distance, 0);
					kRGBColorRamp[r8].SetForeColor();
					::MoveTo(r.left + 2, r.top + 7);
					::Line(distance, 0);
					kRGBColorRamp[r10].SetForeColor();
					::MoveTo(r.left + 2, r.top + 8);
					::Line(distance, 0);
					kRGBColorRamp[r12].SetForeColor();
					::MoveTo(r.left + 2, r.top + 9);
					::Line(distance, 0);
				}
			}
			else	// 1-bit
			{
				kRGBColorRamp[rB].SetForeColor();
				::PaintRect(&r);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CGrayscaleProgressIndicator_AC::DrawRemainderArea
//----------------------------------------------------------------------------------------

void CGrayscaleProgressIndicator_AC::DrawRemainderArea()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the empty track area between the current
	// gauge value and the maximum.

	CRect_AC	frame;
	
	if (this->GetRemainderFrame(frame))
	{
		CGDIterator_AC	iter;
		bool			deep;

		while (iter.NextDevice(deep))
		{
			CRect_AC	r(frame);

			if (deep)
			{
				SInt16	width = r.right - r.left;
				
				if (width > 0)
				{
					kRGBColorRamp[r10].SetForeColor();
					::MoveTo(r.left, r.top);
					::LineTo(r.left, r.bottom - 1);
				}
				
				if (width > 1)
				{
					kRGBColorRamp[r4].SetForeColor();
					::MoveTo(r.right - 1, r.top);
					::Line(0, 1);
					kRGBColorRamp[r2].SetForeColor();
					::LineTo(r.right - 1, r.bottom - 1);
				}
				
				if (width > 2)
				{
					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(r.left + 1, r.top);
					::LineTo(r.left + 1, r.bottom - 1);
				}
				
				if (width > 3)
				{
					kRGBColorRamp[r2].SetForeColor();
					::MoveTo(r.left + 2, r.bottom - 1);
					::LineTo(r.right - 1, r.bottom - 1);

					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(r.left + 2, r.top);
					::LineTo(r.right - 2, r.top);

					kRGBColorRamp[r4].SetForeColor();
					r.left += 2;
					++r.top;
					--r.bottom;
					--r.right;
					::PaintRect(&r);
				}
			}
			else	// 1-bit
			{
				kRGBColorRamp[rW].SetForeColor();
				::PaintRect(&r);
			}
		}
	}
}
