// CRadioButton_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CRadioButton_AC__
#include "CRadioButton_AC.h"
#endif

// Appearance

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

#ifndef __CRegion_AC__
#include "CRegion_AC.h"
#endif


//========================================================================================
// CRadioButton_AC
// This class implements a standard radio button object,
// with the additional capabilities of automatic handling of
// radio group mutual exclusivity, and mixed state.
//========================================================================================


bool CRadioButton_AC::fgImagesAllocated = false;

// Each radio button drawing state has an offscreen image.
CAutoPtr_AC<CSimpleOffscreenImage_AC>	CRadioButton_AC::fgOffscreenRadioImages[kNumRadioImages];

// These are the color ramp indexes that are used to build the offscreen images.
SInt8				CRadioButton_AC::kRadioImageValues[kNumRadioImages][kRadioImageHeight][kRadioImageWidth] =
	{
		{	// kRadioNormalOff
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		{ OUT, OUT,  rB, r10,  r2,  r2,  r2,  r4, r11,  rB, OUT, OUT },
		{ OUT,  rB,  r4,  r2,  r1,  rW,  rW,  rW,  r2,  r7,  rB, OUT },
		{  r5, r10,  r2,  r1,  rW,  rW,  r1,  r1,  r2,  r4, r11,  r5 },
		{ r11,  r2,  r1,  rW,  rW,  r1,  r1,  r2,  r2,  r4,  r7, r11 },
		{  rB,  r2,  rW,  rW,  r1,  r1,  r2,  r2,  r4,  r4,  r7,  rB },
		{  rB,  r2,  rW,  r1,  r1,  r2,  r2,  r4,  r4,  r5,  r7,  rB },
		{ r11,  r4,  rW,  r1,  r2,  r2,  r4,  r4,  r5,  r5,  r7, r11 },
		{  r5, r10,  r2,  r2,  r2,  r4,  r4,  r5,  r5,  r7, r11,  r5 },
		{ OUT,  rB,  r7,  r4,  r4,  r4,  r5,  r5,  r7,  r7,  rB, OUT },
		{ OUT, OUT,  rB, r10,  r7,  r7,  r7,  r7, r11,  rB, OUT, OUT },
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		},

		{	// kRadioNormalOn
		{ OUT, OUT, OUT,  r5, r12,  rB,  rB, r12,  r5, OUT, OUT, OUT },
		{ OUT, OUT, r12,  rB, r11, r10, r10, r10, r11,  rB, OUT, OUT },
		{ OUT, r12, r11, r10,  r8,  r8,  r8,  r7,  r7,  r6,  rB, OUT },
		{  r5,  rB, r10,  r8,  rB,  rB,  rB,  rB,  r6,  r6, r11,  r5 },
		{ r12, r11,  r8,  rB,  rB,  rB,  rB,  rB,  rB,  r6,  r4, r11 },
		{  rB, r10,  r8,  rB,  rB,  rB,  rB,  rB,  rB,  r4,  r4,  rB },
		{  rB, r10,  r8,  rB,  rB,  rB,  rB,  rB,  rB,  r4,  r2,  rB },
		{ r12, r10,  r7,  rB,  rB,  rB,  rB,  rB,  rB,  r2,  rW, r11 },
		{  r5, r12,  r7,  r6,  rB,  rB,  rB,  rB,  r2,  rW, r11,  r5 },
		{ OUT, r12,  r6,  r6,  r6,  r4,  r4,  r2,  rW,  rW,  rB, OUT },
		{ OUT, OUT, r12, r11,  r4,  r4,  r2,  rW, r11,  rB, OUT, OUT },
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		},

		{	// kRadioNormalMixed
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		{ OUT, OUT,  rB, r10,  r2,  r2,  r2,  r4, r11,  rB, OUT, OUT },
		{ OUT,  rB,  r4,  r2,  r1,  rW,  rW,  rW,  r2,  r7,  rB, OUT },
		{  r5, r10,  r2,  r1,  rW,  rW,  r1,  r1,  r2,  r4, r11,  r5 },
		{ r11,  r2,  r1,  rW,  rW,  r1,  r1,  r2,  r2,  r4,  r7, r11 },
		{  rB,  r2,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  r4,  r7,  rB },
		{  rB,  r2,  rW,  rB,  rB,  rB,  rB,  rB,  rB,  r5,  r7,  rB },
		{ r11,  r4,  rW,  r1,  r2,  r2,  r4,  r4,  r5,  r5,  r7, r11 },
		{  r5, r10,  r2,  r2,  r2,  r4,  r4,  r5,  r5,  r7, r11,  r5 },
		{ OUT,  rB,  r7,  r4,  r4,  r4,  r5,  r5,  r7,  r7,  rB, OUT },
		{ OUT, OUT,  rB, r10,  r7,  r7,  r7,  r7, r11,  rB, OUT, OUT },
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		},

		{	// kRadioPressedOff
		{ OUT, OUT, OUT,  r5, r12,  rB,  rB, r12,  r5, OUT, OUT, OUT },
		{ OUT, OUT, r12,  rB, r11, r11, r11, r11, r11,  rB, OUT, OUT },
		{ OUT, r12, r11, r11, r10, r10,  r9,  r9,  r9,  r8,  rB, OUT },
		{  r5,  rB, r11, r10, r10,  r9,  r9,  r8,  r8,  r8, r11,  r5 },
		{ r12, r11, r10, r10,  r9,  r9,  r8,  r8,  r8,  r7,  r6, r11 },
		{  rB, r11, r10,  r9,  r9,  r8,  r8,  r8,  r7,  r7,  r6,  rB },
		{  rB, r11,  r9,  r9,  r8,  r8,  r8,  r7,  r7,  r6,  r6,  rB },
		{ r12, r11,  r9,  r8,  r8,  r8,  r7,  r7,  r6,  r6,  r4, r11 },
		{  r5, r12,  r9,  r8,  r8,  r7,  r7,  r6,  r6,  r4, r10,  r5 },
		{ OUT, r11,  r8,  r8,  r7,  r7,  r6,  r6,  r4,  r4,  rB, OUT },
		{ OUT, OUT, r12, r11,  r6,  r6,  r6,  r4, r10,  rB, OUT, OUT },
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		},

		{	// kRadioPressedOn
		{ OUT, OUT, OUT,  r5, r12,  rB,  rB, r12,  r5, OUT, OUT, OUT },
		{ OUT, OUT, r12,  rB, r11, r11, r11, r11, r11,  rB, OUT, OUT },
		{ OUT, r12, r11, r11, r10, r10,  r9,  r9,  r9,  r8,  rB, OUT },
		{  r5,  rB, r11, r10,  rB,  rB,  rB,  rB,  r8,  r8, r11,  r5 },
		{ r12, r11, r10,  rB,  rB,  rB,  rB,  rB,  rB,  r7,  r6, r11 },
		{  rB, r11, r10,  rB,  rB,  rB,  rB,  rB,  rB,  r7,  r6,  rB },
		{  rB, r11,  r9,  rB,  rB,  rB,  rB,  rB,  rB,  r6,  r6,  rB },
		{ r12, r11,  r9,  rB,  rB,  rB,  rB,  rB,  rB,  r6,  r4, r11 },
		{  r5, r12,  r9,  r8,  rB,  rB,  rB,  rB,  r6,  r4, r10,  r5 },
		{ OUT, r11,  r8,  r8,  r7,  r7,  r6,  r6,  r4,  r4,  rB, OUT },
		{ OUT, OUT, r12, r11,  r6,  r6,  r6,  r4, r10,  rB, OUT, OUT },
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		},

		{	// kRadioPressedMixed
		{ OUT, OUT, OUT,  r5, r12,  rB,  rB, r12,  r5, OUT, OUT, OUT },
		{ OUT, OUT, r12,  rB, r11, r11, r11, r11, r11,  rB, OUT, OUT },
		{ OUT, r12, r11, r11, r10, r10,  r9,  r9,  r9,  r8,  rB, OUT },
		{  r5,  rB, r11, r10, r10,  r9,  r9,  r8,  r8,  r8, r11,  r5 },
		{ r12, r11, r10, r10,  r9,  r9,  r8,  r8,  r8,  r7,  r6, r11 },
		{  rB, r11, r10,  rB,  rB,  rB,  rB,  rB,  rB,  r7,  r6,  rB },
		{  rB, r11,  r9,  rB,  rB,  rB,  rB,  rB,  rB,  r6,  r6,  rB },
		{ r12, r11,  r9,  r8,  r8,  r8,  r7,  r7,  r6,  r6,  r4, r11 },
		{  r5, r12,  r9,  r8,  r8,  r7,  r7,  r6,  r6,  r4, r10,  r5 },
		{ OUT, r11,  r8,  r8,  r7,  r7,  r6,  r6,  r4,  r4,  rB, OUT },
		{ OUT, OUT, r12, r11,  r6,  r6,  r6,  r4, r10,  rB, OUT, OUT },
		{ OUT, OUT, OUT,  r5, r11,  rB,  rB, r11,  r5, OUT, OUT, OUT },
		},

		{	// kRadioDisabledOff
		{ OUT, OUT, OUT,  r4,  r7,  r7,  r7,  r7,  r4, OUT, OUT, OUT },
		{ OUT, OUT,  r7,  r7,  r3,  r2,  r2,  r3,  r7,  r7, OUT, OUT },
		{ OUT,  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7, OUT },
		{  r4,  r7,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r7,  r4 },
		{  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7 },
		{  r7,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r7 },
		{  r7,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r7 },
		{  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7 },
		{  r4,  r7,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r7,  r4 },
		{ OUT,  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7, OUT },
		{ OUT, OUT,  r7,  r7,  r3,  r2,  r2,  r3,  r7,  r7, OUT, OUT },
		{ OUT, OUT, OUT,  r4,  r7,  r7,  r7,  r7,  r4, OUT, OUT, OUT },
		},

		{	// kRadioDisabledOn
		{ OUT, OUT, OUT,  r4,  r7,  r7,  r7,  r7,  r4, OUT, OUT, OUT },
		{ OUT, OUT,  r7,  r7,  r4,  r4,  r4,  r4,  r7,  r7, OUT, OUT },
		{ OUT,  r7,  r5,  r4,  r4,  r4,  r4,  r4,  r4,  r5,  r7, OUT },
		{  r4,  r7,  r4,  r4,  r8,  r8,  r8,  r8,  r4,  r4,  r7,  r4 },
		{  r7,  r4,  r4,  r8,  r8,  r8,  r8,  r8,  r8,  r4,  r4,  r7 },
		{  r7,  r4,  r4,  r8,  r8,  r8,  r8,  r8,  r8,  r4,  r4,  r7 },
		{  r7,  r4,  r4,  r8,  r8,  r8,  r8,  r8,  r8,  r4,  r4,  r7 },
		{  r7,  r4,  r4,  r8,  r8,  r8,  r8,  r8,  r8,  r4,  r4,  r7 },
		{  r4,  r7,  r4,  r4,  r8,  r8,  r8,  r8,  r4,  r4,  r7,  r4 },
		{ OUT,  r7,  r5,  r4,  r4,  r4,  r4,  r4,  r4,  r5,  r7, OUT },
		{ OUT, OUT,  r7,  r7,  r4,  r4,  r4,  r4,  r7,  r7, OUT, OUT },
		{ OUT, OUT, OUT,  r4,  r7,  r7,  r7,  r7,  r4, OUT, OUT, OUT },
		},

		{	// kRadioDisabledMixed
		{ OUT, OUT, OUT,  r4,  r7,  r7,  r7,  r7,  r4, OUT, OUT, OUT },
		{ OUT, OUT,  r7,  r7,  r3,  r2,  r2,  r3,  r7,  r7, OUT, OUT },
		{ OUT,  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7, OUT },
		{  r4,  r7,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r7,  r4 },
		{  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7 },
		{  r7,  r2,  r2,  r8,  r8,  r8,  r8,  r8,  r8,  r2,  r2,  r7 },
		{  r7,  r2,  r2,  r8,  r8,  r8,  r8,  r8,  r8,  r2,  r2,  r7 },
		{  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7 },
		{  r4,  r7,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r7,  r4 },
		{ OUT,  r7,  r3,  r2,  r2,  r2,  r2,  r2,  r2,  r3,  r7, OUT },
		{ OUT, OUT,  r7,  r7,  r3,  r2,  r2,  r3,  r7,  r7, OUT, OUT },
		{ OUT, OUT, OUT,  r4,  r7,  r7,  r7,  r7,  r4, OUT, OUT, OUT },
		}

	};


//----------------------------------------------------------------------------------------
// CRadioButton_AC::CRadioButton_AC
//----------------------------------------------------------------------------------------

CRadioButton_AC::CRadioButton_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, const CStr255_AC& title, UInt32 groupIDPart1, UInt32 groupIDPart2, bool automaticState) : 
	CControl_AC(bounds), 
	MExclusiveObject_AC(groupIDPart1, groupIDPart2),
	fValue(kRadioButtonOff),
	fAutomaticState(automaticState),
	fTitle(title),
	fTextStyle(textStyle)
{
	// Construct with specified button title.
	if (! fgImagesAllocated)
		AllocateRadioImages();
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::CRadioButton_AC
//----------------------------------------------------------------------------------------

CRadioButton_AC::CRadioButton_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt16 stringListResourceID, SInt16 stringIndex, UInt32 groupIDPart1, UInt32 groupIDPart2, bool automaticState) : 
	CControl_AC(bounds), 
	MExclusiveObject_AC(groupIDPart1, groupIDPart2),
	fValue(kRadioButtonOff),
	fAutomaticState(automaticState),
	fTitle(),
	fTextStyle(textStyle)
{
	// Construct by getting button title from specified resource.
	::GetIndString(fTitle, stringListResourceID, stringIndex);
	
	if (! fgImagesAllocated)
		AllocateRadioImages();
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::~CRadioButton_AC
//----------------------------------------------------------------------------------------

CRadioButton_AC::~CRadioButton_AC()
{
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::DrawObject
//----------------------------------------------------------------------------------------

void CRadioButton_AC::DrawObject()
{
	// Draw button in normal unpressed state.

	this->DrawButton(kNotPressed);
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::TrackMouse
//----------------------------------------------------------------------------------------

bool CRadioButton_AC::TrackMouse(const CPoint_AC& mouseLocation)
{
	// If inherited tracking succeeded and we are set to
	// automatically change our state, toggle it.

	bool	wasItHit = CControl_AC::TrackMouse(mouseLocation);

	if (wasItHit && fAutomaticState)
		this->SetValue(kRadioButtonOn, kRedraw);

	return wasItHit;
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::SetTitle
//----------------------------------------------------------------------------------------

void CRadioButton_AC::SetTitle(const CStr255_AC& title, bool redraw)
{
	// Change button title, redraw if specified.

	fTitle = title;
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::GetValue
//----------------------------------------------------------------------------------------

SInt32 CRadioButton_AC::GetValue() const
{
	// Return current state.

	return fValue;
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::SetValue
//----------------------------------------------------------------------------------------

void CRadioButton_AC::SetValue(SInt32 newValue, bool redraw)
{
	// Set current state, redraw if specified and necessary.

	if (newValue == kRadioButtonOn)
		CGroupsContainer_AC::Instance()->HitGroupMember(this);

	if (newValue != GetValue())
	{
		fValue = newValue;
		
		if (redraw)
		{
			this->DrawButton(kNotPressed);
		}
	}
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::SetGroupID
//----------------------------------------------------------------------------------------

void CRadioButton_AC::SetGroupID(UInt32 groupIDPart1, UInt32 groupIDPart2)
{
	MExclusiveObject_AC::SetGroupID(groupIDPart1, groupIDPart2);

	// If we are being installed into a group, we should turn automatic
	// mode on.
	
	if ((groupIDPart1 != kNoGroupID) && (groupIDPart2 != kNoGroupID))
		fAutomaticState = true;
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::TurnOff
//----------------------------------------------------------------------------------------

void CRadioButton_AC::TurnOff()
{
	// Turn off the radio button, presumably in response
	// to another button in the group being turned on.

	this->SetValue(kRadioButtonOff, kRedraw);
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::DrawButton
//----------------------------------------------------------------------------------------

void CRadioButton_AC::DrawButton(bool pressed)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the button in the appropriate state.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (! fEnabled || ! fActive)
			this->DrawButtonDisabled(deep);
		else if (pressed)
			this->DrawButtonPressed(deep);
		else
			this->DrawButtonNormal(deep);
	}
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::SetTrackingState
//----------------------------------------------------------------------------------------

void CRadioButton_AC::SetTrackingState(bool isIn)
{
	// Draw the button pressed or unpressed as specified.

	this->DrawButton(isIn);
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CRadioButton_AC::DrawButtonNormal(bool deep)
{
	CRect_AC	r(GetBounds());

	r.Offset(2, 2);	// visually match the system CDEF

	// Draw the button in enabled/unpressed state.

	if (deep)
	{
		if (GetValue() == kRadioButtonMixed)
			this->CopyImage(kRadioNormalMixed);
		else if (GetValue() == kRadioButtonOn)
			this->CopyImage(kRadioNormalOn);
		else
			this->CopyImage(kRadioNormalOff);
	}
	else
	{
		r.right = r.left + kRadioImageWidth;
		r.bottom = r.top + kRadioImageHeight;

		// Paint white over previous junk, if any.
		kRGBColorRamp[rW].SetForeColor();
		::PaintRect(&r);
		// Paint frame.
		kRGBColorRamp[rB].SetForeColor();
		::FrameOval(&r);

		// Draw dot or dash.
		if (GetValue() == kRadioButtonOn)
		{
			r.Inset(3, 3);
			::PaintRoundRect(&r, 4, 4);
		}
		else if (GetValue() == kRadioButtonMixed)
		{
			::PenSize(1, 2);
			::MoveTo(r.left + 3, r.top + 5);
			::Line(5, 0);
		}
		
		::PenNormal();
	}

	// Draw the title.
	r = GetBounds();

	r.Offset(2, 2);	// visually match the system CDEF

	r.left += (kRadioImageWidth + 4);
	r.bottom = r.top + kRadioImageHeight;
	AGAStringOut(fTitle, r, truncMiddle, teJustLeft, kNormalOutput, deep, fTextStyle);
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CRadioButton_AC::DrawButtonPressed(bool deep)
{
	CRect_AC	r(GetBounds());

	r.Offset(2, 2);	// visually match the system CDEF

	// Draw the button in enabled/pressed state.

	if (deep)
	{
		if (GetValue() == kRadioButtonMixed)
			this->CopyImage(kRadioPressedMixed);
		else if (GetValue() == kRadioButtonOn)
			this->CopyImage(kRadioPressedOn);
		else
			this->CopyImage(kRadioPressedOff);
	}
	else
	{
		r.right = r.left + kRadioImageWidth;
		r.bottom = r.top + kRadioImageHeight;

		// Paint white over previous junk, if any.
		kRGBColorRamp[rW].SetForeColor();
		::PaintRect(&r);
		// Paint frame.
		kRGBColorRamp[rB].SetForeColor();
		::PenSize(2, 2);
		::FrameOval(&r);

		// Draw dot or dash.
		if (GetValue() == kRadioButtonOn)
		{
			r.Inset(3, 3);
			::PaintRoundRect(&r, 4, 4);
		}
		else if (GetValue() == kRadioButtonMixed)
		{
			::PenSize(1, 2);
			::MoveTo(r.left + 3, r.top + 5);
			::Line(5, 0);
		}
		
		::PenNormal();
	}

	// Draw the title.
	r = GetBounds();

	r.Offset(2, 2);	// visually match the system CDEF

	r.left += (kRadioImageWidth + 4);
	r.bottom = r.top + kRadioImageHeight;
	AGAStringOut(fTitle, r, truncMiddle, teJustLeft, kNormalOutput, deep, fTextStyle);
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CRadioButton_AC::DrawButtonDisabled(bool deep)
{
	CRect_AC	r(GetBounds());

	r.Offset(2, 2);	// visually match the system CDEF

	// Draw the button in disabled/unpressed state.

	if (deep)
	{
		if (GetValue() == kRadioButtonMixed)
			this->CopyImage(kRadioDisabledMixed);
		else if (GetValue() == kRadioButtonOn)
			this->CopyImage(kRadioDisabledOn);
		else
			this->CopyImage(kRadioDisabledOff);
	}
	else
	{
		r.right = r.left + kRadioImageWidth;
		r.bottom = r.top + kRadioImageHeight;

		// Paint white over previous junk, if any.
		kRGBColorRamp[rW].SetForeColor();
		::PaintRect(&r);

		// Paint frame.
		PenPatGray();
		kRGBColorRamp[rB].SetForeColor();
		::FrameOval(&r);

		// Draw check or dash.
		if (GetValue() == kRadioButtonOn)
		{
			r.Inset(3, 3);
			::PaintRoundRect(&r, 4, 4);
		}
		else if (GetValue() == kRadioButtonMixed)
		{
			::PenSize(1, 2);
			::MoveTo(r.left + 3, r.top + 5);
			::Line(5, 0);
		}
		
		::PenNormal();
	}

	// Draw the title.
	r = GetBounds();

	r.Offset(2, 2);	// visually match the system CDEF

	r.left += (kRadioImageWidth + 4);
	r.bottom = r.top + kRadioImageHeight;
	AGAStringOut(fTitle, r, truncMiddle, teJustLeft, kDisabledOutput, deep, fTextStyle);
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::CopyImage
//----------------------------------------------------------------------------------------

void CRadioButton_AC::CopyImage(UInt32 index)
{
	// Blast the specified offscreen image into the
	// radio button's button area. We offset by (2, 2)
	// to match the positioning of the system CDEF.

	fgOffscreenRadioImages[index]->DrawImage(GetBounds().left + 2, GetBounds().top + 2);
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::AllocateRadioImages
//----------------------------------------------------------------------------------------

void CRadioButton_AC::AllocateRadioImages()
{
	// Allocate the offscreen image for each radio button state.

	CPoint_AC	imageSize;
	
	imageSize.h = kRadioImageWidth;
	imageSize.v = kRadioImageHeight;
	
	// A simple round rect defines the radio image clipping region.

	RgnHandle radioImageClippingRegion = CRegion_AC::MakeNewRgn();

	CRect_AC	imageClippingRect(0, 0, kRadioImageWidth, kRadioImageHeight);

	::OpenRgn();
	::FrameRoundRect(&imageClippingRect, 11, 11);
	//::FrameOval(&imageClippingRect);	ugly AGA 7/96 specs use just the chunky circle -- we'll stick with earlier specs
	::CloseRgn(radioImageClippingRegion);


	for (UInt32 imageIndex = 0; imageIndex < kNumRadioImages; ++imageIndex)
	{
		fgOffscreenRadioImages[imageIndex] = TH_new CSimpleOffscreenImage_AC;
		
		 fgOffscreenRadioImages[imageIndex]->CreateImageData(
					imageSize,
					&kRadioImageValues[imageIndex][0][0],
					radioImageClippingRegion, 
					kRGBColorRamp[rB]);
	}
			
	::DisposeRgn(radioImageClippingRegion);
		
	fgImagesAllocated = true;
}

//----------------------------------------------------------------------------------------
// CRadioButton_AC::DisposeRadioImages
//----------------------------------------------------------------------------------------

void CRadioButton_AC::DisposeRadioImages()
{
	for (UInt32 imageIndex = 0; imageIndex < kNumRadioImages; ++imageIndex)
		if (!fgOffscreenRadioImages[imageIndex].IsNULL())
			fgOffscreenRadioImages[imageIndex].reset();
}

