// CStaticText_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CStaticText_AC__
#include "CStaticText_AC.h"
#endif

// Appearance

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

// Toolbox


//========================================================================================
// CStaticText_AC
// This class draws a static text label.
//========================================================================================

//----------------------------------------------------------------------------------------
// CStaticText_AC::CStaticText_AC
//----------------------------------------------------------------------------------------

CStaticText_AC::CStaticText_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt32 justification, bool eraseFirst, const CStr255_AC& title) : 
	CControl_AC(bounds),
	fJustification(justification),
	fTitle(title),
	fTextStyle(textStyle),
	fEraseFirst(eraseFirst)
{
	// Construct with supplied string.
}

//----------------------------------------------------------------------------------------
// CStaticText_AC::CStaticText_AC
//----------------------------------------------------------------------------------------

CStaticText_AC::CStaticText_AC(const CRect_AC& bounds, const CTextStyle_AC& textStyle, SInt32 justification, bool eraseFirst, SInt16 stringListResourceID, SInt16 stringIndex) : 
	CControl_AC(bounds),
	fJustification(justification),
	fTextStyle(textStyle),
	fEraseFirst(eraseFirst)
{
	// Construct by getting string from specified resource.
	::GetIndString(fTitle, stringListResourceID, stringIndex);
}

//----------------------------------------------------------------------------------------
// CStaticText_AC::~CStaticText_AC
//----------------------------------------------------------------------------------------

CStaticText_AC::~CStaticText_AC()
{
}

//----------------------------------------------------------------------------------------
// CStaticText_AC::DrawObject
//----------------------------------------------------------------------------------------

void CStaticText_AC::DrawObject()
{
	CTempDrawingEnvironment_AC	env;

	// Paint over possible obsolete text, draw current text.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
			this->ApplyBackgroundColor();
		else
			kRGBColorRamp[rW].SetForeColor();

		//::PaintRect(GetBounds()); no erase rect, always use the color underneath us

		CRGBColor_AC	teBackgroundColor;
		teBackgroundColor.GetForeColor();	// fg color right now is what the TE must leave in the bg

		AGATextBox(&fTitle[1], fTitle.Length(), GetBounds(), fJustification, IsEnabled() && fActive ? kNormalOutput : kDisabledOutput, deep, fTextStyle, teBackgroundColor, fEraseFirst);
	}
}

//----------------------------------------------------------------------------------------
// CStaticText_AC::SetTitle
//----------------------------------------------------------------------------------------

void CStaticText_AC::SetTitle(const CStr255_AC& title, bool redraw)
{
	// Change text, redraw if specified.

	fTitle = title;
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CStaticText_AC::SetTextStyle
//----------------------------------------------------------------------------------------

void CStaticText_AC::SetTextStyle(const CTextStyle_AC& textStyle, bool redraw)
{
	// Change text, redraw if specified.

	fTextStyle = textStyle;
	
	if (redraw)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CStaticText_AC::SetJustification
//----------------------------------------------------------------------------------------

void CStaticText_AC::SetJustification(SInt32 justification, bool redraw)
{
	// Change text, redraw if specified.

	fJustification = justification;
	
	if (redraw)
		this->DrawObject();
}

