// CUpDownControl_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CUpDownControl_AC__
#include "CUpDownControl_AC.h"
#endif

// Appearance

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

//========================================================================================
// CUpDownControl_AC
// This class implements a little arrows control.
//========================================================================================

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::CUpDownControl_AC
//----------------------------------------------------------------------------------------

CUpDownControl_AC::CUpDownControl_AC(const CRect_AC& bounds) : 
	CControl_AC(bounds),
	fNotificationRoutine(NULL),
	fUserData(0)
{
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::~CUpDownControl_AC
//----------------------------------------------------------------------------------------

CUpDownControl_AC::~CUpDownControl_AC()
{
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::InstallNotificationRoutine
//----------------------------------------------------------------------------------------

void CUpDownControl_AC::InstallNotificationRoutine(CUpDownControlCallback_AC notificationRoutine, void* userData)
{
	// Install the supplied function pointer and user data to
	// be called during notification.

	fNotificationRoutine = notificationRoutine;
	fUserData = userData;
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::DrawObject
//----------------------------------------------------------------------------------------

void CUpDownControl_AC::DrawObject()
{
	// Draw the arrows in normal unpressed state.

	this->DrawButton(kNoArrowPressed);
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::TrackMouse
//----------------------------------------------------------------------------------------

bool CUpDownControl_AC::TrackMouse(const CPoint_AC& mouseLocation)
{
	if (! IsEnabled())
		return false;

	// Track the mouse in the appropriate arrow. Little arrows
	// always return true because the tracking is always successful.

	return this->TrackPart(mouseLocation, this->TrackTestPart(mouseLocation));
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::DrawButton
//----------------------------------------------------------------------------------------

void CUpDownControl_AC::DrawButton(SInt32 pressedPart)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the arrows in the specified press state. The
	// state may indicate that either or no arrow is pressed.

	enum { Frame, Fill, TL, BR, Arrow, kNumArrowColors };

	UInt8	colorIndexes[kNumArrowColors];
	
	CGDIterator_AC	iter;
	bool			deep;
	bool			drawEnabled = fEnabled && fActive;

	while (iter.NextDevice(deep))
	{
		CRect_AC	r(GetBounds());
		
		r.right = r.left + kLittleArrowWidth;
		r.bottom = r.top + kLittleArrowHeight;
		
		for (UInt32 i = 0; i < 2; ++i)
		{
			if (deep)
			{
				if (drawEnabled)
				{
					if (((pressedPart == kUpArrowPressed) && (i == 0)) ||
						((pressedPart == kDownArrowPressed) && (i == 1)))
					{
						colorIndexes[Frame] = rB;
						colorIndexes[Fill] = r8;
						colorIndexes[TL] = r10;
						colorIndexes[BR] = r6;
						colorIndexes[Arrow] = rW;
					}
					else if (((pressedPart != kUpArrowPressed) && (i == 0)) ||
						((pressedPart != kDownArrowPressed) && (i == 1)))
					{
						colorIndexes[Frame] = rB;
						colorIndexes[Fill] = r2;
						colorIndexes[TL] = rW;
						colorIndexes[BR] = r5;
						colorIndexes[Arrow] = rB;
					}
				}
				else	// disabled
				{
					colorIndexes[Frame] = r7;
					colorIndexes[Fill] = r2;
					colorIndexes[TL] = r2;
					colorIndexes[BR] = r2;
					colorIndexes[Arrow] = r7;
				}
				
				// These (1-i) and i uses here are to round off the top corners
				// of the top frame (i=0) and round off the bottom corners of
				// the bottom frame (i=1).
				kRGBColorRamp[colorIndexes[Frame]].SetForeColor();
				::MoveTo(r.left + (1-i), r.top);
				::LineTo(r.right - 1 - (1-i), r.top);
				::Move(1-i, 1-i);
				::LineTo(r.right -1, r.bottom - 1 - i);
				::Move(-i, i);
				::LineTo(r.left + i, r.bottom - 1);
				::Move(-i, -i);
				::LineTo(r.left, r.top + (1-i));
				
				r.Inset(1, 1);
				kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
				::PaintRect(&r);
				r.Inset(-1, -1);
				
				kRGBColorRamp[colorIndexes[TL]].SetForeColor();
				::MoveTo(r.left + 1, r.bottom - 3);
				::LineTo(r.left + 1, r.top + 1);
				::LineTo(r.right - 3, r.top + 1);
				
				kRGBColorRamp[colorIndexes[BR]].SetForeColor();
				::MoveTo(r.left + 2, r.bottom - 2);
				::LineTo(r.right - 2, r.bottom - 2);
				::LineTo(r.right - 2, r.top + 2);
				
				kRGBColorRamp[colorIndexes[Arrow]].SetForeColor();
			}
			else	// 1-bit
			{
				if (((pressedPart == kUpArrowPressed) && (i == 0)) ||
					((pressedPart == kDownArrowPressed) && (i == 1)))
				{
					kRGBColorRamp[rB].SetForeColor();
					r.Inset(1, 1);
					::PaintRect(&r);
					r.Inset(-1, -1);
					kRGBColorRamp[rW].SetForeColor();
					}
				else
				{
					kRGBColorRamp[rW].SetForeColor();
					::PaintRect(&r);
					kRGBColorRamp[rB].SetForeColor();

					::MoveTo(r.left + (1-i), r.top);
					::LineTo(r.right - 1 - (1-i), r.top);
					::Move(1-i, 1-i);
					::LineTo(r.right -1, r.bottom - 1 - i);
					::Move(-i, i);
					::LineTo(r.left + i, r.bottom - 1);
					::Move(-i, -i);
					::LineTo(r.left, r.top + (1-i));
				}
			}

			::MoveTo(r.left + 3, r.top + 7 - (i*3));
			::Line(6, 0);
			::Move(-1, -1 + (2*i));
			::Line(-4, 0);
			::Move(1, -1 + (2*i));
			::Line(2, 0);
			::Move(-1, -1 + (2*i));
			::Line(0, 0);
			
			r.Offset(0, kLittleArrowHeight - 1);
		}

		if ((! drawEnabled) && ! deep)
		{
			kRGBColorRamp[rB].SetForeColor();
			PenPatGray();
			::PenMode(patBic);
			::PaintRect(GetBounds());

			::PenNormal();
		}
	}
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::TrackPart
//----------------------------------------------------------------------------------------

bool CUpDownControl_AC::TrackPart(const CPoint_AC& /*mouseLocation*/, SInt32 partHit)
{
	// Track the mouse in the specified arrow. Little arrows always
	// return true because the tracking is always successful.

	bool		wasIn = false;
	bool		isIn = true;
	bool		isButtonDown = true;	// ensure at least one time through
	CPoint_AC	newMouseLocation;
	UInt32		accelerationDelay = 18;	// will divide by 2 each time
	
	while (isButtonDown)
	{
		if (isIn)
			this->NotifyDelta(partHit);

		::GetMouse(&newMouseLocation);	// gives local coordinates
		
		isIn = (this->TrackTestPart(newMouseLocation) == partHit);
		
		if (isIn != wasIn)
			this->DrawButton(isIn ? partHit : kNoArrowPressed);
		
		wasIn = isIn;
		
		if (isButtonDown && (accelerationDelay != 0))
		{
			DelayFutureWhileStillDown_AC(accelerationDelay);
			accelerationDelay = accelerationDelay >> 1;	// divide by 2
		}

		isButtonDown = ::StillDown();
	}
	
	if (isIn)	// if finished IN, restore normal appearance
		this->DrawButton(kNoArrowPressed);
	
	return true;
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::TrackTestPart
//----------------------------------------------------------------------------------------

SInt32 CUpDownControl_AC::TrackTestPart(const CPoint_AC& mouseLocation)
{
	// Return the part value where the mouse would hit.

	if ((mouseLocation.h >= GetBounds().left + kLittleArrowWidth) ||
		(mouseLocation.h < GetBounds().left) ||
		(mouseLocation.v >= GetBounds().top + kLittleArrowTotalHeight) ||
		(mouseLocation.v < GetBounds().top))
		return kNoArrowPressed;
	else if (mouseLocation.v < GetBounds().top + kLittleArrowHeight)
		return kUpArrowPressed;
	else
		return kDownArrowPressed;
}

//----------------------------------------------------------------------------------------
// CUpDownControl_AC::NotifyDelta
//----------------------------------------------------------------------------------------

void CUpDownControl_AC::NotifyDelta(SInt32 partHit)
{
	// Call the installed notification routine, if any, with
	// the installed user data and our part hit, which is the
	// value delta (-1 or 1).

	if (fNotificationRoutine != NULL)
		(*(fNotificationRoutine))(this, partHit, fUserData);
}
