//----------------------------------------------------------------------------------------
// CInternetAddress_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CInternetAddress_AC__
#define __CInternetAddress_AC__

// Net includes

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CNetworkAddress_AC__
#include "CNetworkAddress_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CInternetAddress have as data a host id and port id and is used in most of the
// internet suites API.
//----------------------------------------------------------------------------------------

class EXPORT_AC CInternetAddress_AC : public CNetworkAddress_AC
{
public:
	enum ePort
	{
		kNullPortNum,
		kHTTP = 80,
		kFTP = 21
	};
	enum eAddress
	{
		kNullIPAddress 
	};
	
	CInternetAddress_AC(const CStr255_AC& inAddressString, CInternetPortID_AC inPort = kNullPortNum);
		// Specify address with a string like "12.34.55.22".

	CInternetAddress_AC(CInternetHostID_AC inHostId, CInternetPortID_AC inPort = kNullPortNum);

	CInternetAddress_AC(const CInternetAddress_AC& inAddress);

	CInternetAddress_AC();

	virtual ~CInternetAddress_AC();
	
// assignment operators
	
	CInternetAddress_AC &operator = (const CInternetAddress_AC &inAddress);

	void operator = (CInternetHostID_AC inAddress);

	void operator = (const char* inAddressString);

	void operator = (const CStr255_AC& inAddressString);
	
// getters/setters
	
	void SetHost(CInternetHostID_AC inHostID);	

	CInternetHostID_AC GetHost() const;
		
	void SetPort( CInternetPortID_AC inPortID);

	CInternetPortID_AC GetPort() const;
	
	void Set(CInternetHostID_AC inHostID, CInternetPortID_AC inPort);	

	bool operator == (const CInternetAddress_AC& inAddress) const;
	bool operator != (const CInternetAddress_AC& inAddress) const;
	
	bool operator == (CInternetHostID_AC inID) const;
	bool operator != (CInternetHostID_AC inID) const;
			
	operator CInternetHostID_AC() const;

	virtual CNetworkAddress_AC* Clone() const;
	
// helpers
	
	static bool IsIPAddress(const char* inString);

	static const CInternetAddress_AC fgNullAddress;

	static CInternetAddress_AC GetLocalAddress();
	
protected:
	virtual void ConvertFromString(	const CStr255_AC& inString );
	virtual void ConvertToString( CStr255_AC& inString ) const;

private:
	CInternetHostID_AC fHost;
	CInternetPortID_AC fPort;
};

// Inline methods

inline CInternetAddress_AC &CInternetAddress_AC::operator = (const CInternetAddress_AC &inAddress)
{
	fHost = inAddress.fHost;
	fPort = inAddress.fPort;
	return *this;	
}
inline void CInternetAddress_AC::operator = (CInternetHostID_AC inAddress) 
{ 
	fHost = inAddress; 
	fPort = kNullPortNum;
}
inline void CInternetAddress_AC::operator = (const char* inStr)
{
	ConvertFromString(inStr);
}
inline CInternetAddress_AC::CInternetAddress_AC(
							const CStr255_AC& inStr, 
							CInternetPortID_AC inPort) 
: 	fHost(kNullIPAddress),
	fPort(inPort)
{
	ConvertFromString(inStr);
}
inline CInternetAddress_AC::CInternetAddress_AC(
							CInternetHostID_AC inHost, 
							CInternetPortID_AC inPort)
{
	fHost = inHost;
	fPort = inPort;
}
inline CInternetAddress_AC::CInternetAddress_AC(const CInternetAddress_AC& inAddress)
{
	*this = inAddress;
}

inline CInternetAddress_AC::CInternetAddress_AC() 
: 	fHost(kNullPortNum),
	fPort(kNullIPAddress)
{
}
inline CInternetAddress_AC::~CInternetAddress_AC()
{
}
inline CInternetHostID_AC CInternetAddress_AC::GetHost() const 
{ 
	return fHost; 
}
inline void CInternetAddress_AC::SetHost(CInternetHostID_AC inAddress)
{
	fHost = inAddress; 
}
inline void CInternetAddress_AC::SetPort( CInternetPortID_AC inPort)
{
	fPort = inPort;
}
inline CInternetPortID_AC CInternetAddress_AC::GetPort() const
{
	return fPort;
}
inline bool CInternetAddress_AC::operator == (const CInternetAddress_AC& in) const
{
	return fHost == in.fHost && fPort == in.fPort;
}
inline bool CInternetAddress_AC::operator != (const CInternetAddress_AC& in) const
{
	return fHost != in.fHost || fPort != in.fPort;
}
inline bool CInternetAddress_AC::operator == (CInternetHostID_AC inID) const
{
	return fHost == inID;
}
inline bool CInternetAddress_AC::operator != (CInternetHostID_AC inID) const
{
	return fHost != inID;
}
inline CInternetAddress_AC::operator CInternetHostID_AC() const
{
	return fHost;
}
inline void CInternetAddress_AC::operator = (const CStr255_AC& inAddressString)
{
	ConvertFromString(inAddressString);
}

inline CNetworkAddress_AC* CInternetAddress_AC::Clone() const
{
	return TH_new CInternetAddress_AC(*this);
}
inline void CInternetAddress_AC::Set(CInternetHostID_AC inHostID, CInternetPortID_AC inPort)
{
	fHost = inHostID;
	fPort = inPort;
}


#endif

		
	