//----------------------------------------------------------------------------------------
// CInternetHostRep_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CInternetHostRep_AC__
#define __CInternetHostRep_AC__

// Net includes

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

// patterns

#ifndef __MIdler_AC__
#include "MIdler_AC.h"
#endif

// core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// forward references

class CReactor_AC;
class CInternetHost_AC;
class CInternetAddress_AC;
class CEventTransceiverTimer_AC;

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC is the base class for the provider specific implementations
// for internet hosts.
//----------------------------------------------------------------------------------------

class EXPORT_AC CInternetHostRep_AC : public MIdler_AC
{
public:

	CInternetHostRep_AC(CInternetHost_AC* host);
	
	virtual ~CInternetHostRep_AC();

	// These are called by other classes.	

	virtual void SetAddress(const CInternetAddress_AC&, long id) = 0;
	
	virtual void SetAddresses(CInternetHostID_AC* inAddresses, long numAddresses) = 0;
		// Will make a copy of inAddresses. 

	virtual void AddAddress(const CInternetAddress_AC& inAddress) = 0;
	
	virtual long GetNumAddresses() const = 0;
	
	virtual CInternetAddress_AC GetAddress(long id = 0) = 0;

	virtual const char* GetHostName() const = 0;
	
	virtual void SetHostName(const char* inHostName) = 0;

	virtual long GetMaxNumHosts() const = 0;
	
	virtual void ResolveNameToAddressAsync() = 0;
		// asynchronous
	
	virtual void ResolveAddressToNameAsync() = 0;
		// asynchronous
		
	virtual CError_AC GetError() const
	{	return fError; }

	void SetTimeOut(long interval);
	
	long GetTimeOut() const;
	
	void StartTimer();
	
	void StopTimer();
	
protected:
	void CompleteNameToAddress(CError_AC inError, bool timedOut = false);
	
	void CompleteAddressToName(CError_AC inError, bool timedOut = false);

protected:	
	CInternetHost_AC* fHost;
	CRefCountingPtr_AC<CEventTransceiverTimer_AC>	fTimer;	
	
private:
	CError_AC fError;
};


#endif