//----------------------------------------------------------------------------------------
// CInternetHost_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	There are two internet host classes here, use CInternetHost_AC if you want to dynamically allocated an
	internet host, and point at it with a CRefCountingPtr_AC. If you wish to make one on the stack for 
	use CTempInternetHost_AC.
*/


#ifndef __CInternetHost_AC__
#define __CInternetHost_AC__

// Net

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __CInternetHostRep_AC__
#include "CInternetHostRep_AC.h"
#endif

// patterns

#ifndef __MRefCountable_AC__
#include "MRefCountable_AC.h"
#endif

#ifndef __CStateMachine_AC__
#include "CStateMachine_AC.h"
#endif

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

// core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// fwd ref

class CEvent_AC;
class CInternetHost_AC;
class CInternetHostRep_AC;

// typedef 

typedef CRefCountingPtr_AC<CInternetHost_AC> CInternetHostPtr_AC;


//----------------------------------------------------------------------------------------
// CInternetHost_AC is what you use to resolve internet addresses : both address to
// name and name to address. Blocking syncronous calls and asynchronous calls are 
// supported. 
//
// Warning: Do not make a CInternetHost_AC on the stack - only dynamically allocate it.
//----------------------------------------------------------------------------------------

class EXPORT_AC CInternetHost_AC : 	public CStateMachine_AC,
									public MEventTransceiver_AC
{
public:
	enum eState
	{
	// free to resolve
		kUnresolved = kNullState,

	// starting up
		kStartingNameToAddress,
		kStartingAddressToName,

	// resolution in progress
		kPendingNameToAddress,
		kPendingAddressToName,

	// these mean success
		kResolvedAddressToName,
		kResolvedNameToAddress,
	
	// these mean the resolution failed
		kTimedOut,
		kError
	};	
	
	CInternetHost_AC(CReactor_AC* inReactor = NULL);
	
	CInternetHost_AC(const CInternetAddress_AC& inAddress, CReactor_AC* inReactor = NULL);
	
	CInternetHost_AC(const char* inName, CReactor_AC* inReactor = NULL);
	
	virtual ~CInternetHost_AC();
		
	const char* GetHostName() const
	{	return fHostRep->GetHostName(); }
	
	void SetHostName(const char* inName)
	{	fHostRep->SetHostName(inName); }
	
	void SetAddress(const CInternetAddress_AC& inAddress, long id = 0)
	{	fHostRep->SetAddress(inAddress, id); }
	
	void SetAddresses(CInternetHostID_AC* inAddresses, long numAddresses)
	{	fHostRep->SetAddresses(inAddresses, numAddresses); }
	
		// Will make a copy of inAddresses. 
	void AddAddress(const CInternetAddress_AC& inAddress)
	{	fHostRep->AddAddress(inAddress); }
	
	long GetNumAddresses() const
	{	return fHostRep->GetNumAddresses(); }
	
	CInternetAddress_AC GetAddress(long id = 0) const
	{	return fHostRep->GetAddress(id); }

	// equality helpers
		
	bool IsEqualTo(const CInternetHost_AC& inHost) const;
	
	bool IsEqualTo(CInternetHostID_AC inHost) const;
	
	bool IsEqualTo(const char* inAddressString) const;
	
	// dnr services
			
	void ResolveNameToAddressAsync();
	void ResolveAddressToNameAsync();
		// These returns an event to subscribed event receivers.
	
	void ResolveNameToAddressSync();
	void ResolveAddressToNameSync();
		// These block until done. If ACS built with threading, they will yield.

	CError_AC GetError() const
	{	return fHostRep->GetError(); }
		
	bool IsMultiHomedHost() const
	{	return GetNumAddresses() > 1; }
	
	void SetTimerInterval(long inInterval)
	{	fHostRep->SetTimeOut(inInterval); }
	
	long GetTimerInterval() const
	{	return fHostRep->GetTimeOut(); }

protected:
	static CInternetHostRep_AC* CreateHostRep(CInternetHost_AC* inHostPtr);
		
private:
	CRefCountingPtr_AC<CInternetHostRep_AC> 		fHostRep;
	CError_AC 										fError;

	static const char 								fEmptyName[1];

	friend class CInternetHostRep_AC;
};

//----------------------------------------------------------------------------------------
// CTempInternetHost_AC is for making CInternetHost_AC object on the stack/
//----------------------------------------------------------------------------------------

class CTempInternetHost_AC : public CInternetHost_AC
{
public:
	CTempInternetHost_AC(CReactor_AC* inReactor = NULL)
		: CInternetHost_AC(inReactor) 
	{ aquire(); }
	CTempInternetHost_AC(const CInternetAddress_AC& inAddress, CReactor_AC* inReactor = NULL)
		: CInternetHost_AC(inAddress, inReactor) 
	{ aquire(); }
	CTempInternetHost_AC(const char* inName, CReactor_AC* inReactor = NULL)
		: CInternetHost_AC(inName, inReactor) 
	{ aquire(); }
	
	virtual ~CTempInternetHost_AC()
	{}

};

#endif