//----------------------------------------------------------------------------------------
// CInternetUtils_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CInternetUtils_AC__
#define __CInternetUtils_AC__

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

class CInternetUtilsRep_AC;

//----------------------------------------------------------------------------------------
// CInternetUtils_AC are utils that are provider specific that don't fit anywhere else.
// Note that most if not all the methods are called by other classes so you probably
// won't need to call the API directly.
//----------------------------------------------------------------------------------------

class EXPORT_AC CInternetUtils_AC 
{
public:

	static void ConvertAddressToString(char* str, CInternetHostID_AC host );
		// String conversion only.

	static void ConvertStringToAddress(CInternetHostID_AC& host, const char* str);
		// String conversion only.

	static CInternetAddress_AC GetLocalAddress();

	static void SetMaxNumTCPConnections(unsigned long max);
		// defaults to 0xFFFFFFFF

	static unsigned long GetMaxNumTCPConnections();
	
	static unsigned long GetTCPConnectionCount();
	
	static void Open();
	
	static bool IsOpen();

protected:
	static CInternetUtilsRep_AC* CreateUtilsRep();

private:
	static CAutoPtr_AC<CInternetUtilsRep_AC> fgUtilsRep;
};

//----------------------------------------------------------------------------------------
// CInternetUtilsRep_AC provides the API for provider specific implementation.
//----------------------------------------------------------------------------------------

class EXPORT_AC CInternetUtilsRep_AC
{
protected:
	CInternetUtilsRep_AC() {}
	
public:
	virtual ~CInternetUtilsRep_AC() {}

	virtual void ConvertAddressToString(char* str, CInternetHostID_AC host ) = 0;
	virtual void ConvertStringToAddress(CInternetHostID_AC& host, const char* str) = 0;
	virtual CInternetAddress_AC GetLocalAddress() = 0;
	virtual void SetMaxNumTCPConnections(unsigned long max) = 0;
	virtual unsigned long GetMaxNumTCPConnections() const = 0;
	virtual unsigned long GetTCPConnectionCount() const = 0;
	virtual bool IsOpen() const = 0;

};

bool InitInternetSuite_AC();
	// returns true when it's done.

#endif