//----------------------------------------------------------------------------------------
// CTCPConnector.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTCPConnector__
#define __CTCPConnector__

// network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CConnector_AC__
#include "CConnector_AC.h"
#endif

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

// core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// patterns

#ifndef __CPair_AC__
#include "CPair_AC.h"
#endif

// forward references

template <class T> class CLinkedList_AC;
class CInternetHost_AC;
class CTCPConnection_AC;
class CServiceConnector_AC;

//----------------------------------------------------------------------------------------
// CTCPConnector_AC connects to remote servers listening for TCP connections.
//----------------------------------------------------------------------------------------

class EXPORT_AC CTCPConnector_AC : public CConnector_AC
{
public:
	CTCPConnector_AC(CReactor_AC* inReactor, CServiceHandlerFactory_AC* inFactory = NULL);

	virtual ~CTCPConnector_AC();

	enum { kCreateConnection = NULL };

// 		The returned CTCPConnection_AC* is for convienience, you'll get a kConnectionInitiated event
//		when the connection is completed.

// 	Use these two ConnectTo's if you don't care what the local port of the connection is 
//	(most people won't care).

	virtual CTCPConnection_AC* ConnectTo(	const CInternetAddress_AC& remoteHost, 
											CTCPConnection_AC* inConnection = kCreateConnection,
											bool payAttentionToMaxConnectionsValue = false );

	virtual CTCPConnection_AC* ConnectTo( 	const char* inHostNameOrIPAddressStr, 
											long remotePort,
											CTCPConnection_AC* inConnection = kCreateConnection,
											bool payAttentionToMaxConnectionsValue = false);


// In the unlikely event that you need to specify the local port for your connection use these:
// These may be removed if they're deemed not necessary.

	virtual CTCPConnection_AC* ConnectTo(	const CInternetAddress_AC& remoteHost,
											long localPort,
											CTCPConnection_AC* inConnection = kCreateConnection,
											bool payAttentionToMaxConnectionsValue = false);
	
	virtual CTCPConnection_AC* ConnectTo( 	const char* inHostNameOrIPAddressStr, 
											long inPort,
											long localPort,
											CTCPConnection_AC* inConnection = kCreateConnection,
											bool payAttentionToMaxConnectionsValue = false);

protected:
	
	bool CanConnect(bool payAttentionToMaxConnectionsValue);
	
	CTCPConnection_AC* DoResolvingConnect(	const char* inDomainName, 
											long inPort,
											const CInternetAddress_AC& localAddress,
											CTCPConnection_AC* inConnection,
											bool payAttentionToMaxConnectionsValue);
	
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);

	virtual CServiceHandler_AC* CreateConnection(CReactor_AC* inReactor);

 	CServiceConnector_AC* CreateServiceConnector(	CReactor_AC* inReactor, 
 													CServiceHandler_AC* inConnection);	
 	
 	void HandleDomainNameResolved(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);
	
protected:	
 	
 	typedef CPair_AC<	CRefCountingPtr_AC<CInternetHost_AC> , 
 						CRefCountingPtr_AC<CServiceConnector_AC> > 	CPendingPair;
 	typedef CLinkedList_AC<CPendingPair> 							CList;
 	CAutoPtr_AC<CList>												fPendingConnectors;
 	
 };


#endif
