//----------------------------------------------------------------------------------------
// CInternetHostRep_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Network

#ifndef __CInternetHostRep_AC__
#include "CInternetHostRep_AC.h"
#endif

#ifndef __CInternetHost_AC__
#include "CInternetHost_AC.h"
#endif

// Patterns

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif



//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::CInternetHostRep_AC
//----------------------------------------------------------------------------------------

CInternetHostRep_AC::CInternetHostRep_AC(CInternetHost_AC* host)
	: fHost(host),
	fError(0)
{
	if(!fHost->HasReactor())
		fHost->UseDefaultReactor();
}

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::~CInternetHostRep_A
//----------------------------------------------------------------------------------------

CInternetHostRep_AC::~CInternetHostRep_AC()
{
	if(fTimer) fTimer->Stop();
}

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::CompleteAddressToName
//----------------------------------------------------------------------------------------

void CInternetHostRep_AC::CompleteAddressToName(CError_AC inError, bool timedOut)
{
	fTimer->Stop();

	::AssertNotNULL_AC(fHost);
	
	fError = inError;
	
	if(fError)
		fHost->ChangeStateTo( CInternetHost_AC::kError );
	else if (timedOut)
		fHost->ChangeStateTo( CInternetHost_AC::kTimedOut );	
	else	
		fHost->ChangeStateTo( CInternetHost_AC::kResolvedAddressToName );

	fHost->PostEvent(
		CreateEventWithData_AC( CNetEvent_AC::kInternetAddressToNameResolved, fHost, fError));
}

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::CompleteNameToAddress
//----------------------------------------------------------------------------------------

void CInternetHostRep_AC::CompleteNameToAddress(CError_AC inError, bool timedOut)
{
	fTimer->Stop();
	
	::AssertNotNULL_AC(fHost);

	fError = inError;
	
	if(fError)
		fHost->ChangeStateTo( CInternetHost_AC::kError );
	else if (timedOut)
		fHost->ChangeStateTo( CInternetHost_AC::kTimedOut );	
	else	
		fHost->ChangeStateTo( CInternetHost_AC::kResolvedNameToAddress );
	
	fHost->PostEvent(
		CreateEventWithData_AC( CNetEvent_AC::kInternetNameToAddressResolved, fHost, fError));
}

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::StartTimer
//----------------------------------------------------------------------------------------

void CInternetHostRep_AC::StartTimer()
{
	if(!fTimer)
	{
		fTimer = TH_new CEventTransceiverTimer_AC(NULL);
		fTimer->SetDuration(CNetGlobals_AC::GetDefaultTimeOut());
	}
	fTimer->GetIdleTimeFrom(fHost->GetReactor());
	fTimer->Start();
}

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::SetTimeOut
//----------------------------------------------------------------------------------------

void CInternetHostRep_AC::SetTimeOut(long interval)
{
	if(!fTimer)
		fTimer = TH_new CEventTransceiverTimer_AC(NULL);
	
	fTimer->SetDuration(interval);
}

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::StopTimer
//----------------------------------------------------------------------------------------

void CInternetHostRep_AC::StopTimer()
{
	if(fTimer) 
		fTimer->Stop();
}

//----------------------------------------------------------------------------------------
// CInternetHostRep_AC::GetTimeOut
//----------------------------------------------------------------------------------------

long CInternetHostRep_AC::GetTimeOut() const
{
	return fTimer.IsNULL() ? CNetGlobals_AC::GetDefaultTimeOut() : fTimer->GetDuration() ;
}


