//----------------------------------------------------------------------------------------
// CInternetHost_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// net

#ifndef __CInternetHost_AC__
#include "CInternetHost_AC.h"
#endif

#ifndef __CInternetHostRep_AC__
#include "CInternetHostRep_AC.h"
#endif

#ifndef __CProvider_AC__
#include "CProvider_AC.h"
#endif

#ifndef __CNetGlobals_AC__
#include "CNetGlobals_AC.h"
#endif

// patterns

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CReactorFactory_AC__
#include "CReactorFactory_AC.h"
#endif


//----------------------------------------------------------------------------------------
// CInternetHost_AC::CInternetHost_AC
//----------------------------------------------------------------------------------------

CInternetHost_AC::CInternetHost_AC(CReactor_AC* inReactor)
	: MEventTransceiver_AC(inReactor),
	fError(0)
{
	fHostRep = CreateHostRep(this);
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::CInternetHost_AC
//----------------------------------------------------------------------------------------

CInternetHost_AC::CInternetHost_AC(const CInternetAddress_AC& inAddress, CReactor_AC* inReactor)
	: MEventTransceiver_AC(inReactor),
	fError(0)
{
	fHostRep = CreateHostRep(this);
	SetAddress(inAddress);
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::CInternetHost_AC
//----------------------------------------------------------------------------------------

CInternetHost_AC::CInternetHost_AC(const char* inName, CReactor_AC* inReactor)
	: MEventTransceiver_AC(inReactor),
	fError(0)
{
	fHostRep = CreateHostRep(this);
	SetHostName(inName);
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::~CInternetHost_AC
//----------------------------------------------------------------------------------------

CInternetHost_AC::~CInternetHost_AC()
{
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::IsEqualTo
//----------------------------------------------------------------------------------------

bool CInternetHost_AC::IsEqualTo(const CInternetHost_AC& in) const
{
	bool equal = in.GetNumAddresses() == GetNumAddresses();
		
	for(long i = 0; equal && i < GetNumAddresses(); ++i)
	{
		if(GetAddress(i) != in.GetAddress(i))
			equal = false;
	}

	if(equal)
		equal = ::strcmp(GetHostName(), in.GetHostName());
	
	return equal;
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::IsEqualTo
//----------------------------------------------------------------------------------------

bool CInternetHost_AC::IsEqualTo(CInternetHostID_AC inHost) const
{
	bool equal = false;
	
	for(long i = 0; !equal && i < GetNumAddresses(); ++i)
	{
		if(GetAddress(i) == inHost)
			equal = true;
	}

	return equal;

}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::IsEqualTo
//----------------------------------------------------------------------------------------

bool CInternetHost_AC::IsEqualTo(const char* inAddressString) const
{
	return ::strcmp(GetHostName(), inAddressString) == 0;
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::ResolveNameToAddressAsync
//----------------------------------------------------------------------------------------

void CInternetHost_AC::ResolveNameToAddressAsync()
{
	if(GetState() > kUnresolved && GetState() < kResolvedAddressToName)
		ThrowError_AC(paramErr);
	
	fHostRep->ResolveNameToAddressAsync();
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::ResolveAddressToNameAsync
//----------------------------------------------------------------------------------------
	
void CInternetHost_AC::ResolveAddressToNameAsync()
{
	if(GetState() > kUnresolved && GetState() < kResolvedAddressToName)
		ThrowError_AC(paramErr);

	fHostRep->ResolveAddressToNameAsync();
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::ResolveNameToAddressSync
//----------------------------------------------------------------------------------------
	
void CInternetHost_AC::ResolveNameToAddressSync()
{
	ResolveNameToAddressAsync();
			
	while(GetState() < kResolvedAddressToName)
	{
		GetReactor()->Idle();
	}
}

//----------------------------------------------------------------------------------------
// CInternetHost_AC::ResolveAddressToNameSync
//----------------------------------------------------------------------------------------

void CInternetHost_AC::ResolveAddressToNameSync()
{
	ResolveAddressToNameAsync();
	
	while(GetState() < kResolvedAddressToName)
	{
		GetReactor()->Idle();
	}
}

