//----------------------------------------------------------------------------------------
// CTCPConnector.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Net

#ifndef __CTCPConnector_AC__
#include "CTCPConnector_AC.h"
#endif

#ifndef __CServiceConnector_AC__
#include "CServiceConnector_AC.h"
#endif

#ifndef __CInternetHost_AC__
#include "CInternetHost_AC.h"
#endif

#ifndef __CTCPConnection_AC__
#include "CTCPConnection_AC.h"
#endif
#ifndef __CServiceConnector_AC__
#include "CServiceConnector_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

// Patterns

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CPair_AC__
#include "CPair_AC.h"
#endif

// Containers

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::CTCPConnector_AC
//----------------------------------------------------------------------------------------

CTCPConnector_AC::CTCPConnector_AC(CReactor_AC* inReactor, CServiceHandlerFactory_AC* inFactory)
	:  CConnector_AC(inReactor, inFactory)
{
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::~CTCPConnector_AC
//----------------------------------------------------------------------------------------

CTCPConnector_AC::~CTCPConnector_AC()
{
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::CreateConnection
//----------------------------------------------------------------------------------------

CServiceHandler_AC* CTCPConnector_AC::CreateConnection(CReactor_AC* inReactor)
{
	return TH_new CTCPConnection_AC(inReactor);
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::ConnectTo
//----------------------------------------------------------------------------------------

CTCPConnection_AC* CTCPConnector_AC::ConnectTo(	const CInternetAddress_AC& remoteHost, 
												CTCPConnection_AC* inConnection,
												bool payAttentionToMaxConnectionsValue)
{
	return CanConnect(payAttentionToMaxConnectionsValue) ? 
		dynamic_cast_AC(CTCPConnection_AC*, CConnector_AC::ConnectTo( inConnection, remoteHost, CInternetAddress_AC::GetLocalAddress()))
		: NULL;
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::ConnectTo
//----------------------------------------------------------------------------------------

CTCPConnection_AC* CTCPConnector_AC::ConnectTo(	const CInternetAddress_AC& remoteHost, 
												long localPort,
												CTCPConnection_AC* inConnection,
												bool payAttentionToMaxConnectionsValue)
{
	CInternetAddress_AC localAddress = CInternetAddress_AC::GetLocalAddress();
	localAddress.SetPort(localPort);

	return CanConnect(payAttentionToMaxConnectionsValue) ? 
		dynamic_cast_AC(CTCPConnection_AC*, CConnector_AC::ConnectTo( inConnection, remoteHost, localAddress)) :
		NULL;
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::ConnectTo
//----------------------------------------------------------------------------------------

CTCPConnection_AC* CTCPConnector_AC::ConnectTo( 	const char* inHostNameOrIPAddressStr, 
													long inPort,
													long inLocalPort,
													CTCPConnection_AC* inConnection,
													bool payAttentionToMaxConnectionsValue)
{
	CInternetAddress_AC localAddress = CInternetAddress_AC::GetLocalAddress();
	localAddress.SetPort(inLocalPort);
	return DoResolvingConnect(inHostNameOrIPAddressStr, inPort, localAddress, inConnection, payAttentionToMaxConnectionsValue);
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::ConnectTo
//----------------------------------------------------------------------------------------

CTCPConnection_AC* CTCPConnector_AC::ConnectTo( 	const char* inDomainName, 
													long inPort,
													CTCPConnection_AC* inConnection,
													bool payAttentionToMaxConnectionsValue)
{
	return DoResolvingConnect(inDomainName, inPort, CInternetAddress_AC::GetLocalAddress(), inConnection, payAttentionToMaxConnectionsValue);
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::DoResolvingConnect
//----------------------------------------------------------------------------------------

CTCPConnection_AC* CTCPConnector_AC::DoResolvingConnect(const char* inHostNameOrIPAddressStr, 
														long inPort,
														const CInternetAddress_AC& localAddress,
														CTCPConnection_AC* inConnection,
														bool payAttentionToMaxConnectionsValue)
{
	CTCPConnection_AC* theConnection = inConnection;

	if(CanConnect(payAttentionToMaxConnectionsValue))
	{
		// First create the the CTCPConnection_AC object if we don't have one.
		if(!theConnection)
			theConnection = dynamic_cast_AC(CTCPConnection_AC*, CreateConnectionObject());
			
		if(CInternetAddress_AC::IsIPAddress(inHostNameOrIPAddressStr))
		{
			// If it's an IP address, do a normal connect.
		
			CConnector_AC::ConnectTo( 	theConnection, 
										CInternetAddress_AC(inHostNameOrIPAddressStr, inPort), 
										localAddress);
		}
		else
		{	
			// Make the list if we don't have one.
			if(!fPendingConnectors)
				fPendingConnectors = TH_new CList;

			// Create the service connector
			CServiceConnectorPtr_AC connector = CreateServiceConnector(GetReactor(), theConnection);

			// Set up the address
			connector->SetLocalAddress(localAddress);
			// Set the port
			connector->SetRemoteAddress(CInternetAddress_AC(0, inPort));

			// Make a host ptr
			CRefCountingPtr_AC<CInternetHost_AC> hostPtr = TH_new CInternetHost_AC;
			
			RegisterForEventsFrom(hostPtr);
			
			// Put the host and the connector into our list
			fPendingConnectors->Push(CPendingPair(hostPtr, connector));
				
			// begin the resolution
			hostPtr->SetHostName(inHostNameOrIPAddressStr);
			hostPtr->ResolveNameToAddressAsync();
		}
	}
	
	return theConnection;
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::ReceiveEvent
//----------------------------------------------------------------------------------------

void CTCPConnector_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kInternetNameToAddressResolved:
			HandleDomainNameResolved(inEvent, inTransceiver);
		break;
		
		default:
			CConnector_AC::ReceiveEvent(inEvent,inTransceiver);
		break;	
	}
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::HandleDomainNameResolved
//----------------------------------------------------------------------------------------

void CTCPConnector_AC::HandleDomainNameResolved(CEvent_AC* /*inEvent*/, MEventTransceiver_AC* inTransceiver)
{
	AssertNotNULL_AC(fPendingConnectors);
	
	CInternetHost_AC* inHostPtr = dynamic_cast_AC(CInternetHost_AC*,inTransceiver);

	CList::CIterator iterator = fPendingConnectors->First();
	
	while(iterator.IsValid())
	{
		CServiceConnector_AC* connector = (*iterator).fSecond;
		CInternetHost_AC* savedHostPtr	= (*iterator).fFirst;
			// for readability
			
		if(savedHostPtr == inHostPtr)
		{
		
			switch(	inHostPtr->GetState())
			{
				case CInternetHost_AC::kResolvedNameToAddress:
				{
					RegisterForEventsFrom(connector);
						// We need to Receive events from it.

					CInternetAddress_AC address;
					
					connector->GetRemoteAddress(address);
						// get the saved address. Only the port is important at this point.
					
					address.SetHost( inHostPtr->GetAddress() );
						// set the host address, not the port.
					
					connector->SetRemoteAddress(address);
						// reset the address
						
					fConnectors->Push(connector);
						// save the connector for later.
						
					connector->DoConnect(fTimeout);
						// Tell it to connect. The service connector will complete the connection
						// and open the connection.
				}
				break;
				
				case CInternetHost_AC::kTimedOut:
					PostEvent(
						CreateEventWithData_AC(	CNetEvent_AC::kConnectTimedOut, 
							CServiceHandlerPtr_AC( connector->GetPendingConnection())));				
				break;
				
				case CInternetHost_AC::kError:
					PostEvent(
						CreateEventWithData_AC(	CNetEvent_AC::kConnectFailed, 
							CServiceHandlerPtr_AC( connector->GetPendingConnection()), inHostPtr->GetError()));				
				break;
			}				
				
			fPendingConnectors->Remove(iterator);
				// remove the connector and host from the list, this will delete the host.
						
			break;
				// stop looping
		}
	
		iterator.Next();
	}	
}

//----------------------------------------------------------------------------------------
// CTCPConnector_AC::CanConnect
//----------------------------------------------------------------------------------------

bool CTCPConnector_AC::CanConnect(bool payAttentionToMaxConnectionsValue)
{
	bool canConnect = 	!payAttentionToMaxConnectionsValue ||
						CTCPConnection_AC::GetConnectionCount() < CTCPConnection_AC::GetMaxNumConnections();
	
	if(!canConnect)
		PostEvent(TH_new CEvent_AC(CNetEvent_AC::kConnectRejected));
		
	return canConnect;
}
