//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CMacTCPConnection_AC__
#define __CMacTCPConnection_AC__

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#if qWantsMacTCP_AC

// Net

#ifndef __CService_AC__
#include "CService_AC.h"
#endif

// Patterns

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

// fwd refs

class CMacTCPStream_AC;
class CReactor_AC;
class CServiceState_AC;
class CMacTCPDisconnector_AC;
class CMacTCPDataWriter;
class CMacTCPDataReader;
class CMacTCPConnection_AC;

typedef CRefCountingPtr_AC<CMacTCPConnection_AC> CMacTCPConnectionPtr_AC;

//----------------------------------------------------------------------------------------
// class CMacTCPConnection_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CMacTCPConnection_AC : public CService_AC
{
public:
	CMacTCPConnection_AC(CReactor_AC* inReactor);
	
	virtual ~CMacTCPConnection_AC();

	void Open(CMacTCPStream_AC* inStream);	
		// we'll get our reactor from our stream when we're opened.

	virtual void Hangup();
	
	virtual void Disconnect();
	
	virtual void Rcv(char* inPtr, long amt, long &flags);	
		// will generate kReceivedData when received amt or all available data
	virtual void Snd(const char* inPtr, long amt, long flags);
		// will generate kSentData event with amount of data as event data.
	
	virtual	CServiceState_AC* GetState() const;

	virtual long GetUnreceivedDataAmount() const;

	static void SetMaxConnectionCount(unsigned long max);
		
	static unsigned long GetMaxConnectionCount();
	
	static unsigned long GetConnectionCount();

protected:
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inFrom);

	void CompleteDataReceive();
	void CompleteDataSend();

protected:
	CRefCountingPtr_AC<CMacTCPStream_AC> 		fStream;
	CRefCountingPtr_AC<CMacTCPDisconnector_AC> 	fDisconnector;

	CMacTCPDataWriter* 							fWriters;
	short 										fNumWriters;
	bool										fPendingSend;
	bool										fPendingReceive;
	long										fReceiveAmt;
	short										fGotReceiveEvent;	

	static unsigned long						fgMaxConnections;
	static unsigned long						fgConnectionCount;
};

#endif

#endif