//----------------------------------------------------------------------------------------
// CMacTCPStream_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CMacTCPStream_AC__
#define __CMacTCPStream_AC__

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#if qWantsMacTCP_AC

// net

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

// pattern includes

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif
#ifndef __MIdler_AC__
#include "MIdler_AC.h"
#endif

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

// streams

#ifndef __CAsyncStream_AC__
#include "CAsyncStream_AC.h"
#endif

// MacOS 

#ifndef __MacTCP__
#include <MacTCP.h>
#endif

// fwd references
class CReactor_AC;
class CEventTransceiverTimer_AC;
class CServiceState_AC;
class CBuffer_AC;
class CMacTCPParamBlockTCP_AC;
class CMacTCPStream_AC;

template <class T> class CLinkedList_AC;

typedef CRefCountingPtr_AC<CMacTCPStream_AC> CMacTCPStreamPtr_AC;


//----------------------------------------------------------------------------------------
// class CMacTCPStream_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CMacTCPStream_AC  : public MEventTransceiver_AC
{
public:

	CMacTCPStream_AC(CReactor_AC* inReactor);
	
	virtual ~CMacTCPStream_AC();

	StreamPtr GetMacTCPStream() const 
	{ 	return fStream; }
	
	void SetMacTCPStream(StreamPtr inEndpoint)
	{ 	fStream = inEndpoint; }
	
	virtual void Idle();
	
	void Close();
	
	bool IsOpen() const 
	{ 	return fStream != NULL; }

	CInternetAddress_AC& GetRemoteHost() 
	{ 	return fRemoteHost; }
	
	CInternetAddress_AC& GetLocalHost()  
	{ 	return fLocalHost; }

	void SetRemoteHost(const CInternetAddress_AC& inAddress)
	{	fRemoteHost = inAddress; }
	
	void SetLocalHost(const CInternetAddress_AC& inAddress)
	{	fLocalHost = inAddress; }
	
	long GetID() const
	{	return fID; }
		
	void StartTimer(long interval);
	
	void StopTimer();
		
	virtual CServiceState_AC* GetState();	
		
	TCPNotifyUPP 		GetTCPNotifyUPP();
	TCPIOCompletionUPP	GetTCPCompletionUPP();

	CBuffer_AC*			GetBuffer();

	CMacTCPParamBlockTCP_AC* 	StartTCPCommand( short csCode );
	CMacTCPParamBlockTCP_AC*	GetParameterBlockPtr() const
	{	return fTCPParamBlockPtr; }
	
	bool IsReturnCode(short value)
	{
		return (-23000 >= value) && (value <= -23049);
	}
	
	enum kEventType
	{	
		kCompletion = 1,
		kNotify 
	};
	
	static void GetEventData(	CEvent_AC* inEvent,
								short& eventType, short& eventCode, short& terminateReason );
	
	static void CloseAllStreams();
		
protected:

	Ptr	CoerceToPtr()
	{	return reinterpret_cast<Ptr>(this); }

	static pascal void TCPCompletionProc(struct TCPiopb *iopb);
	static pascal void TCPNotifyProc(StreamPtr inStream, unsigned short eventCode, CMacTCPStream_AC* inTCPStream, 
		unsigned short terminReason, struct ICMPReport *icmpMsg);
		
		
protected:

	typedef CLinkedList_AC<CMacTCPStream_AC*> 			CList;
		
	enum 
	{	kBufferSize = 10024 };	
		
	CRefCountingPtr_AC<CEventTransceiverTimer_AC>		fTimer;	
	CAutoPtr_AC<CServiceState_AC>						fState;
				
	StreamPtr 											fStream;
	
	TCPNotifyUPP 										fTCPNotifyUPP;
	TCPIOCompletionUPP									fTCPCompletionProc;
	
	CRefCountingPtr_AC<CBuffer_AC>						fBuffer;
	CAsyncStream_AC	 									fEventStream;
	
	CInternetAddress_AC									fLocalHost;
	CInternetAddress_AC									fRemoteHost;
	
	long												fID;
	
	static long											fgCounter;
	static CList*										fgStreams;
	
	CAutoPtr_AC<CMacTCPParamBlockTCP_AC>				fTCPParamBlockPtr;
	
private:
	CMacTCPStream_AC(const CMacTCPStream_AC& /*in*/) {}
	CMacTCPStream_AC& operator = (const CMacTCPStream_AC& /*in*/) { return *this; }

};



#endif
#endif
