//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


// mactcp
#ifndef __CMacTCPAcceptor_AC__
#include "CMacTCPAcceptor_AC.h"
#endif


#if qWantsMacTCP_AC

#ifndef __CMacTCPStream_AC__
#include "CMacTCPStream_AC.h"
#endif

#ifndef __CMacTCPConnection_AC__
#include "CMacTCPConnection_AC.h"
#endif

#ifndef __CMacTCPOpenTCPStream_AC__
#include "CMacTCPOpenTCPStream_AC.h"
#endif

#ifndef __MacTCPPrivate_AC__
#include "MacTCPPrivate_AC.h"
#endif


// network
#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif


// patterns
#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif


// containers
#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif


// MacTCP
#ifndef __CMacTCPDisconnector_AC__
#include "CMacTCPDisconnector_AC.h"
#endif


#define kListeningIncrement 3


//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC::CMacTCPAcceptor_AC
//----------------------------------------------------------------------------------------

CMacTCPAcceptor_AC::CMacTCPAcceptor_AC(	CReactor_AC* inReactor,  
									long timeOut)
	: CServiceAcceptor_AC(inReactor, timeOut),
	fListeningCount(0)
{
}

//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC::~CMacTCPAcceptor_AC
//----------------------------------------------------------------------------------------

CMacTCPAcceptor_AC::~CMacTCPAcceptor_AC()
{
}

//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC::ListenTo
//----------------------------------------------------------------------------------------

void CMacTCPAcceptor_AC::ListenTo(	const CNetworkAddress_AC& inLocalAddress, 
									const CNetworkAddress_AC& inRemoteAddress)
{
	fOpeningStreams = TH_new CList;
	
	fOpener = TH_new CMacTCPOpenTCPStream_AC(GetReactor());
	RegisterForEventsFrom(fOpener);
	
	fRemoteAddress = dynamic_cast<const CInternetAddress_AC&>(inRemoteAddress);
	fLocalAddress = dynamic_cast<const CInternetAddress_AC&>(inLocalAddress);
			
	for( short i = 0; i < Min_AC( CMacTCPConnection_AC::GetMaxConnectionCount(), kListeningIncrement ); i++)
	{
		// open an appropriate number of streams to listen for connections.
	
		fOpener->Open();
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC::Accept
//----------------------------------------------------------------------------------------

void CMacTCPAcceptor_AC::Accept(CServiceHandler_AC* inConnection)
{
	CMacTCPConnection_AC* connection = dynamic_cast_AC(CMacTCPConnection_AC*,inConnection->GetService());
		
	::Assertion_AC( fOpeningStreams->GetLength() > 0, "No open streams?");
	
	CRefCountingPtr_AC<CMacTCPStream_AC> stream = fOpeningStreams->GetFirstElement();
	fOpeningStreams->Remove(stream);	
	connection->Open(stream);
		// Give the connection the open stream.
}

//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC::ReceiveEvent
//----------------------------------------------------------------------------------------

void CMacTCPAcceptor_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender)
{
	switch( inEvent->GetEventID() )
	{
		// Begin listening for connection request.
		case CNetEvent_AC::kProviderOpen:
		{	
			ReceiveEvent_AC(inEvent);
			
			CMacTCPStreamPtr_AC ptr = 
				GetDataFromEvent_AC(inEvent, CMacTCPStreamPtr_AC);
			
			if(!fStreams)
				fStreams = TH_new CList;
				
			fStreams->Push(ptr);
				// we own the streams we opened. If the acceptor is deleted, then
				// only the streams that only we have pointers to will be deleted.

			RegisterForEventsFrom(ptr);
			DoPassiveOpen(ptr);
		}
		break;
		
		case CNetEvent_AC::kProvider:
		{	
			// We can get two types of events from the stream, a completion
			// event or incoming event. A completion event means that our 
			// passiveopen complete which signifies that we have a connection
			// request.
			
			short eventType = 0, eventCode = 0, termReason = 0;
			
			CMacTCPStream_AC* stream = dynamic_cast_AC(CMacTCPStream_AC*,inSender);
				// get the stream from the sender variable
				
			stream->GetEventData(inEvent, eventType, eventCode, termReason);
				// get event data
				
			if(	eventType == CMacTCPStream_AC::kCompletion)
			{
				// Handle completion events.
			
				switch(eventCode)
				{
					case TCPPassiveOpen:
					{
						ReceiveEvent_AC(inEvent);
						HandleConnectionRequest(stream);
					}
					break;
				}
			}
			else
			{
				// Handle incoming events.
			
				switch(eventCode)
				{
					case TCPTerminate:
					{
						// one of our streams terminated, but is still open, re-open
						// it for conections.
			
						ReceiveEvent_AC(inEvent);
				
						CMacTCPStreamPtr_AC ptr = dynamic_cast_AC(CMacTCPStream_AC*,inSender);
				
						DoPassiveOpen(ptr);
					}
					break;
				}
			}
		}
		break;
		
		
		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inSender);
		break;
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC::HandleConnectionRequest
//----------------------------------------------------------------------------------------

void CMacTCPAcceptor_AC::HandleConnectionRequest(CMacTCPStream_AC* inOnStream)
{
	if(	CMacTCPConnection_AC::GetConnectionCount() >=
		CMacTCPConnection_AC::GetMaxConnectionCount() )
	{
		CRefCountingPtr_AC<CMacTCPDisconnector_AC> disconnector = TH_new CMacTCPDisconnector_AC(GetReactor());
		disconnector->Open(inOnStream);
		disconnector->Hangup();
			// Disconnect the connection, we've accepted one too many.
		
		PostEvent(
			TH_new CEvent_AC(CNetEvent_AC::kAcceptRejected));
	}
	else
	{
		// Accept the connection.
	
		fOpeningStreams->Push(CMacTCPStreamPtr_AC(inOnStream));
			// save the stream for later. We'll need to give it to the 
			// connection when Accept is called.
						
		PostEvent(
			TH_new CEvent_AC(CNetEvent_AC::kAcceptSucceeded));
			
		fListeningCount--;
		
		if(fListeningCount <= 1)
		{
			// we need to open some more streams if all of our streams are used. Of course we
			// don't wan't to exceed the maximum number of connections.
		
			long toOpen = Min_AC( CMacTCPConnection_AC::GetMaxConnectionCount() - CMacTCPConnection_AC::GetConnectionCount(), kListeningIncrement );
		
			for( short i = 0; i < toOpen ; i++)
			{
				// open an appropriate number of streams to listen for connections.
			
				fOpener->Open();
			}
		}		
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPAcceptor_AC::DoPassiveOpen
//----------------------------------------------------------------------------------------

void CMacTCPAcceptor_AC::DoPassiveOpen(CMacTCPStream_AC* inStream)
{
	::AssertNotNULL_AC(inStream);
	
	inStream->SetRemoteHost(fRemoteAddress);
	inStream->SetLocalHost(fLocalAddress);

	CMacTCPParamBlockTCP_AC* pb = inStream->StartTCPCommand(TCPPassiveOpen);

	pb->csParam.open.remoteHost				= inStream->GetRemoteHost();	
	pb->csParam.open.remotePort				= inStream->GetRemoteHost().GetPort();	
	pb->csParam.open.localPort				= inStream->GetLocalHost().GetPort();	

	pb->DoDriverCall();
	
	fListeningCount++;
}

#endif