//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CMacTCPConnection_AC__
#include "CMacTCPConnection_AC.h"
#endif


#if qWantsMacTCP_AC

#ifndef __CMacTCPStream_AC__
#include "CMacTCPStream_AC.h"
#endif

#ifndef __MacTCPPrivate_AC__
#include "MacTCPPrivate_AC.h"
#endif

#ifndef __CMacTCPDisconnector_AC__
#include "CMacTCPDisconnector_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif


#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif


static const long kMaxBlockSize = 0xFFFF;
unsigned long CMacTCPConnection_AC::fgMaxConnections = 0xFFFFFFFF;
unsigned long CMacTCPConnection_AC::fgConnectionCount = 0;


//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::CMacTCPConnection_AC
//----------------------------------------------------------------------------------------

CMacTCPConnection_AC::CMacTCPConnection_AC(CReactor_AC* inReactor)
	: CService_AC(inReactor),
	fWriters(NULL),
	fNumWriters(0),
	fPendingSend(false),
	fPendingReceive(false),
	fReceiveAmt(0),
	fGotReceiveEvent(0)
{
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::~CMacTCPConnection_AC
//----------------------------------------------------------------------------------------

CMacTCPConnection_AC::~CMacTCPConnection_AC()
{
	::DeleteIfArray_AC(fWriters);
	fgConnectionCount--;
	if(fTimer) 
		fTimer->Stop();
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::Open
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::Open(CMacTCPStream_AC* inStream)
{
	SetReactor(inStream->GetReactor());
	fStream = inStream; 
	RegisterForEventsFrom(fStream);
	
	fDisconnector = TH_new CMacTCPDisconnector_AC(GetReactor());
	RegisterForEventsFrom(fDisconnector);

	fDisconnector->Open(fStream);
	
	fgConnectionCount++;
}	

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::Hangup
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::Hangup()
{
	fDisconnector->Hangup(); 
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::Disconnect
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::Disconnect()
{
	fDisconnector->Disconnect(); 
}

//----------------------------------------------------------------------------------------
//  CMacTCPConnection_AC::Rcv
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::Rcv(char* inPtr, long inAmt, long &flags)
{
	flags; 

	Assertion_AC( fPendingReceive == false, "Receive already pending!");
	
	fPendingReceive = true;

		
	CMacTCPParamBlockTCP_AC* pb = fStream->StartTCPCommand(TCPRcv);
	
//	pb->csParam.receive.commandTimeoutValue		= 30;	
	pb->csParam.receive.rcvBuff					= inPtr;	
	pb->csParam.receive.rcvBuffLen				= inAmt;	

	pb->DoDriverCall();

	fReceiveAmt = inAmt;
}	

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::CompleteDataReceive
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::CompleteDataReceive()
{

	::Assertion_AC( fPendingReceive == true, "no receive pending!");
	fPendingReceive = false;

	long rcvAmt = fStream->GetParameterBlockPtr()->csParam.receive.rcvBuffLen;
	
	Assertion_AC(rcvAmt <= fReceiveAmt, "Receive amt exceeded buffer size. This is very bad!");

#if qNetTrace_AC
	TRACE << LowTracePriority_AC << "Received " << rcvAmt << " of " << fReceiveAmt << endl_AC;
#endif
	
	PostEvent( CreateEventWithData_AC( CNetEvent_AC::kReceivedData, rcvAmt));

	fReceiveAmt = 0;

}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::Snd
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::Snd(const char* inPtr, long inLen, long flags)
{
	Assertion_AC(fPendingSend == false, "Send pending!");
	
	flags; 
	
	CMacTCPParamBlockTCP_AC* pb = fStream->StartTCPCommand(TCPSend);

	short numWriters = (inLen / (unsigned long) kMaxBlockSize) + 2;
		// always at least two, one for data less the kMaxBlockSize, one
		// for the terminating writer.
		
	if (fNumWriters < numWriters)
	{
		::DeleteIfArray_AC(fWriters);
		fNumWriters = 0;
		
		fWriters = TH_new CMacTCPDataWriter[numWriters];
		fNumWriters = numWriters;
	}
	else
		::Erase_AC(fWriters, fNumWriters);

	char *outChunkPtr = const_cast<char*>(inPtr);
	long sizeTally = inLen;
	
	for(short i = 0; i < (fNumWriters-1); ++i) 
		// it is fNumWriters-1 cause last writer is always NULL
	{
		long chunkSize = ::Min_AC( kMaxBlockSize, sizeTally );
			// get size of data to have the writer point to
			
		fWriters[i].Set(outChunkPtr, chunkSize);
			// set the data writer up with a pointer to the data to send
			// and the amount of data the chunk is pointing at.
			
		outChunkPtr += chunkSize;
			// increment the pointer into the data by how much we've assigned
			// to the current data writer.
			
		sizeTally -= chunkSize;
			// decrement the size tally.
	}
	
	pb->csParam.send.wdsPtr = reinterpret_cast<Ptr>(fWriters);	
		// give the writer array to the paramblock
		
	pb->DoDriverCall();
		// call asyncronously.
	
	if(fTiming) fTimer->Start();
	
	fPendingSend = true;	
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::GetState
//----------------------------------------------------------------------------------------

CServiceState_AC* CMacTCPConnection_AC::GetState() const
{
	return fStream->GetState();
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::ReceiveEvent
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inFrom)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kError:
			PostEvent(inEvent);
		break;

		case CNetEvent_AC::kConnectionTerminated:
			PostEvent(inEvent);
			DisableServiceHandlerTimer();
		break;
		
		case CNetEvent_AC::kProvider: 
		{
			short eventType, eventCode, termReason;
			fStream->GetEventData(inEvent, eventType, eventCode, termReason);
			
			if(	eventType == CMacTCPStream_AC::kNotify)
			{
				switch(eventCode)
				{
					case TCPDataArrival:
					case TCPUrgent:
					{
						ReceiveEvent_AC(inEvent);
						
						if(fPendingReceive)
							fGotReceiveEvent++;
						else
							PostEvent( 
								TH_new CEvent_AC(CNetEvent_AC::kIncomingData));

						if(fTiming) fTimer->Start();	
					}	
					break;
				}
			}
			else
			{
				// receive completion
			
				ReceiveEvent_AC(inEvent);	
				
				switch(eventCode)
				{
					case TCPSend:
						CompleteDataSend();
					break;
					
					case TCPRcv:
						CompleteDataReceive();
						
						if(fGotReceiveEvent)
						{
							fGotReceiveEvent--;
							PostEvent( 
								TH_new CEvent_AC(CNetEvent_AC::kIncomingData));
						}
						
					break;
				}
			}
		}
		break;
		
		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inFrom);
		break;
	}

}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::CompleteDataSend
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::CompleteDataSend()
{
	::Assertion_AC(fPendingSend == true, "No pending Send!");
	fPendingSend = false;
	
	long amt = 0;
						
	for(short i = 0; i < fNumWriters; ++i)
	{
		amt += fWriters[i].length;
	}

	PostEvent( CreateEventWithData_AC( CNetEvent_AC::kSentData, amt ));
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::GetUnreceivedDataAmount
//----------------------------------------------------------------------------------------

long CMacTCPConnection_AC::GetUnreceivedDataAmount() const
{
	CMacTCPParamBlockTCP_AC* pb = fStream->StartTCPCommand(TCPStatus);
	pb->DoDriverCall(false);
		// call syncronously
	return pb->csParam.status.amtUnreadData;
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::SetMaxConnectionCount
//----------------------------------------------------------------------------------------

void CMacTCPConnection_AC::SetMaxConnectionCount(unsigned long max)
{
	fgMaxConnections = max;
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::GetMaxConnectionCount
//----------------------------------------------------------------------------------------

unsigned long CMacTCPConnection_AC::GetMaxConnectionCount()
{
	return fgMaxConnections;
}

//----------------------------------------------------------------------------------------
// CMacTCPConnection_AC::GetConnectionCount
//----------------------------------------------------------------------------------------

unsigned long CMacTCPConnection_AC::GetConnectionCount()
{
	return fgConnectionCount;
}
#endif