//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CMacTCPConnector_AC__
#include "CMacTCPConnector_AC.h"
#endif


#if qWantsMacTCP_AC

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CMacTCPConnection_AC__
#include "CMacTCPConnection_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

#ifndef __CMacTCPStream_AC__
#include "CMacTCPStream_AC.h"
#endif

#ifndef __CMacTCPOpenTCPStream_AC__
#include "CMacTCPOpenTCPStream_AC.h"
#endif

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __MacTCPPrivate_AC__
#include "MacTCPPrivate_AC.h"
#endif

#ifndef __CMacTCPDisconnector_AC__
#include "CMacTCPDisconnector_AC.h"
#endif

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif


//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::CMacTCPConnector_AC
//----------------------------------------------------------------------------------------

CMacTCPConnector_AC::CMacTCPConnector_AC(CReactor_AC* inReactor, CServiceHandler_AC* inConnection)
	: CServiceConnector_AC(inReactor, inConnection)
{
	fConnections = TH_new CList;
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::~CMacTCPConnector_AC
//----------------------------------------------------------------------------------------

CMacTCPConnector_AC::~CMacTCPConnector_AC()
{
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::SetRemoteAddress
//----------------------------------------------------------------------------------------

void CMacTCPConnector_AC::SetRemoteAddress(const CNetworkAddress_AC& inAddress)
{
	fRemoteAddress = dynamic_cast<const CInternetAddress_AC&>(inAddress);
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::SetLocalAddress
//----------------------------------------------------------------------------------------

void CMacTCPConnector_AC::SetLocalAddress(const CNetworkAddress_AC& inAddress)
{
	fLocalAddress =  dynamic_cast<const CInternetAddress_AC&>(inAddress);
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::GetRemoteAddress
//----------------------------------------------------------------------------------------

void CMacTCPConnector_AC::GetRemoteAddress(CNetworkAddress_AC& inAddress) const
{
	CInternetAddress_AC& ref = dynamic_cast<CInternetAddress_AC&>(inAddress);
	ref = fRemoteAddress;
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::GetLocalAddress
//----------------------------------------------------------------------------------------

void CMacTCPConnector_AC::GetLocalAddress(CNetworkAddress_AC& inAddress) const
{
	CInternetAddress_AC& ref = dynamic_cast<CInternetAddress_AC&>(inAddress);
	ref = fLocalAddress;
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::DoConnect
//----------------------------------------------------------------------------------------

void CMacTCPConnector_AC::DoConnect(long inTimeOut)
{
	if(!fOpener)
	{
		fOpener = TH_new CMacTCPOpenTCPStream_AC(GetReactor());
		RegisterForEventsFrom(fOpener);
	}

	CMacTCPStream_AC* stream = fOpener->Open();
	
	if(inTimeOut)
		stream->StartTimer(inTimeOut);	
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::ReceiveEvent
//----------------------------------------------------------------------------------------

void CMacTCPConnector_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender)
{
	CMacTCPStreamPtr_AC stream;

	switch( inEvent->GetEventID() )
	{
		case CNetEvent_AC::kTimedOut:
			stream = dynamic_cast_AC(CMacTCPStream_AC*,inSender);
			fConnections->Remove(stream);
			PostEvent(inEvent);
		break;
		
		case CNetEvent_AC::kError:
			stream = dynamic_cast_AC(CMacTCPStream_AC*,inSender);
			fConnections->Remove(stream);
			PostEvent(TH_new CEvent_AC(CNetEvent_AC::kConnectFailed, inEvent->GetError()));
		break;
			
		case CNetEvent_AC::kProviderOpen:
			ReceiveEvent_AC(inEvent);
			stream = GetDataFromEvent_AC(inEvent, CMacTCPStreamPtr_AC);
			fConnections->Append(stream);
			DoActiveOpen(stream);
		break;

		case CNetEvent_AC::kProvider:
		{	
			short eventType = 0, eventCode = 0, termReason = 0;
			
			stream = dynamic_cast_AC(CMacTCPStream_AC*,inSender);
			fConnections->Remove(stream);
				
			stream->GetEventData(inEvent, eventType, eventCode, termReason);
			
			if(	eventType == CMacTCPStream_AC::kCompletion)
			{
				switch(eventCode)
				{
					case TCPActiveOpen:
						ReceiveEvent_AC(inEvent);
						
						UnregisterForEventsFrom(stream);
						
						stream->SetRemoteHost(fRemoteAddress);
						
						CServiceHandler_AC* pendingConnection = GetPendingConnection();
						
						pendingConnection->Open();
							// open the connection, this creates the MacTCPConnection.
						
						CService_AC* service = pendingConnection->GetService();
							// Get the macTCPConnection
							
						::AssertNotNULL_AC(service);
						
						CMacTCPConnectionPtr_AC connection = dynamic_cast_AC(CMacTCPConnection_AC*,service);
							// convert it to our subclass
		
						::AssertNotNULL_AC(connection);	
							// make sure our cast worked.
	
						connection->Open(stream);	
							// open the MacTCPConnection, hand it the open stream.
	
						PostEvent( 
							TH_new CEvent_AC(CNetEvent_AC::kConnectSucceeded));
							// Notify our handlers that the connection is ready to rock!
							// This sends an event to the CConnector_AC, which pulls our
							// connection from this.
					break;
				}
			}
		}
		break;

		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inSender);
		break;
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPConnector_AC::DoActiveOpen
//----------------------------------------------------------------------------------------

void CMacTCPConnector_AC::DoActiveOpen(CMacTCPStream_AC* inStream)
{
	AssertNotNULL_AC(inStream);
	
	inStream->SetRemoteHost(fRemoteAddress);
	inStream->SetLocalHost(fLocalAddress);
	RegisterForEventsFrom(inStream);

	CMacTCPParamBlockTCP_AC* pb = inStream->StartTCPCommand(TCPActiveOpen);

	pb->csParam.open.ulpTimeoutValue		= 30;
	pb->csParam.open.ulpTimeoutAction		= 1;
	pb->csParam.open.validityFlags			= 0xC0;	

	pb->csParam.open.remoteHost				= inStream->GetRemoteHost();	
	pb->csParam.open.remotePort				= inStream->GetRemoteHost().GetPort();	
	pb->csParam.open.localPort				= inStream->GetLocalHost().GetPort();	
	pb->csParam.open.localHost				= 0;	

	pb->DoDriverCall();
}
#endif