//----------------------------------------------------------------------------------------
// CMacTCPDisconnector_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------



#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsMacTCP_AC
#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __MacTCPPrivate_AC__
#include "MacTCPPrivate_AC.h"
#endif


#ifndef __CMacTCPDisconnector_AC__
#include "CMacTCPDisconnector_AC.h"
#endif

#ifndef __CMacTCPStream_AC__
#include "CMacTCPStream_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CMacTCPDisconnector_AC::CMacTCPDisconnector_AC
//----------------------------------------------------------------------------------------

CMacTCPDisconnector_AC::CMacTCPDisconnector_AC(CReactor_AC* inReactor)
:	MEventTransceiver_AC(inReactor)
{
}

//----------------------------------------------------------------------------------------
// CMacTCPDisconnector_AC::~CMacTCPDisconnector_AC
//----------------------------------------------------------------------------------------

CMacTCPDisconnector_AC::~CMacTCPDisconnector_AC()
{
}

//----------------------------------------------------------------------------------------
// CMacTCPDisconnector_AC::Open
//----------------------------------------------------------------------------------------

void CMacTCPDisconnector_AC::Open(CMacTCPStream_AC* inStream)
{
	::AssertNotNULL_AC(inStream);
	SetReactor(inStream->GetReactor());
	fStream = inStream;
	RegisterForEventsFrom(fStream);
}

//----------------------------------------------------------------------------------------
// CMacTCPDisconnector_AC::ReceiveEvent
//----------------------------------------------------------------------------------------

void CMacTCPDisconnector_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kError:
			PostEvent(inEvent);
		break;

		case CNetEvent_AC::kProvider:
		{	
			short eventType, eventCode, termReason;
			fStream->GetEventData(inEvent, eventType, eventCode, termReason);
		
			if(	eventType == CMacTCPStream_AC::kCompletion)
			{
				switch(eventCode)
				{	
					case TCPClose:
						ReceiveEvent_AC(inEvent);	
						// not closed yet
					break;
					
					case TCPAbort:
						ReceiveEvent_AC(inEvent);	
						PostEvent(
							CreateEventWithData_AC(
										CNetEvent_AC::kConnectionTerminated,
										(long) CNetEvent_AC::kHangupComplete));
					break;
				}
			}
			else
			{
				switch(eventCode)
				{
					case TCPClosing:
						ReceiveEvent_AC(inEvent);	
						// all pending data has been recieved and delivered.
						// This doesn't mean the connection is terminated.
					break;
					
					case TCPTerminate:
						switch(termReason)
						{
							case TCPULPClose:
								ReceiveEvent_AC(inEvent);	
								PostEvent(
									CreateEventWithData_AC(
										CNetEvent_AC::kConnectionTerminated,
										(long) CNetEvent_AC::kDisconnectComplete));
							break;
					
								
								
							case TCPNetworkFailure:
							case TCPSecPrecMismatch:
							case TCPULPTimeoutTerminate:
							case TCPULPAbort:
						 	case TCPServiceError:
								
							case TCPRemoteAbort:
								ReceiveEvent_AC(inEvent);	
								PostEvent( 
									CreateEventWithData_AC(
										CNetEvent_AC::kConnectionTerminated,
										(long) CNetEvent_AC::kRcvdHangup));
							break;
						}
					break;
				}
			}	
		}
		break;

		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
		break;

	}
}

//----------------------------------------------------------------------------------------
// CMacTCPDisconnector_AC::Disconnect
//----------------------------------------------------------------------------------------

void CMacTCPDisconnector_AC::Disconnect()
{

	CMacTCPParamBlockTCP_AC* pb = fStream->StartTCPCommand(TCPClose);

	pb->csParam.close.ulpTimeoutValue	= 30;	
	pb->csParam.close.ulpTimeoutAction	= 1;	
	pb->csParam.close.validityFlags		= 0xC0;	
	
	pb->DoDriverCall();
}

//----------------------------------------------------------------------------------------
// CMacTCPDisconnector_AC::Hangup
//----------------------------------------------------------------------------------------

void CMacTCPDisconnector_AC::Hangup()
{
	CMacTCPParamBlockTCP_AC* pb = fStream->StartTCPCommand(TCPAbort);
	pb->DoDriverCall();
}
#endif