//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CMacTCPInternetHostRep_AC__
#include "CMacTCPInternetHostRep_AC.h"
#endif


#if qWantsMacTCP_AC

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __CInternetHost_AC__
#include "CInternetHost_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

class CMacTCPDNRCloser
{
public:
	CMacTCPDNRCloser() {}
	~CMacTCPDNRCloser()
	{	CMacTCPInternetHostRep_AC::Close(); }
};

class CMacTCPHostList_AC
{
public:
	CMacTCPHostList_AC() : fHosts(NULL) {}
	
	CMacTCPInternetHostRep_AC* GetFirstHost() const
	{	return fHosts; }

	void Insert(CMacTCPInternetHostRep_AC* inHost)
	{	fHosts = inHost->Append(fHosts); }
	
	void Remove(CMacTCPInternetHostRep_AC* inHost)
	{	fHosts = inHost->Remove(fHosts); }
	
private:
	CMacTCPInternetHostRep_AC* fHosts;
};


inline CAsyncStream_AC& operator << (CAsyncStream_AC& inStream, struct hostInfo* hostInfoPtr)
{
	inStream.Write(&hostInfoPtr, sizeof(hostInfoPtr));
	return inStream;
}
inline CAsyncStream_AC& operator >> (CAsyncStream_AC& inStream, struct hostInfo*& hostInfoPtr)
{
	inStream.Read(&hostInfoPtr, sizeof(hostInfoPtr));
	return inStream;
}


// global data (local to this file)
static CMacTCPHostList_AC 	gHostList;
static CMacTCPDNRCloser		gCloseMe;

// static data declarations
ResultUPP				CMacTCPInternetHostRep_AC::fgResultUPP = NULL;
Handle 					CMacTCPInternetHostRep_AC::fgDNRCodeHndl = NULL;
ProcPtr					CMacTCPInternetHostRep_AC::fgDNRCodePtr = NULL;


//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::CMacTCPInternetHostRep_AC
//----------------------------------------------------------------------------------------

CMacTCPInternetHostRep_AC::CMacTCPInternetHostRep_AC(CInternetHost_AC* host)
	: CInternetHostRep_AC(host)
{
	Open();
	
	::Erase_AC(&fHostInfo);
	gHostList.Insert(this);
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::~CMacTCPInternetHostRep_AC
//----------------------------------------------------------------------------------------

CMacTCPInternetHostRep_AC::~CMacTCPInternetHostRep_AC()
{
	gHostList.Remove(this);
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::SetAddress
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::SetAddress(const CInternetAddress_AC& inAddr, long id)
{
	if(id >= NUM_ALT_ADDRS)
		ThrowError_AC(paramErr);

	if(id >= fNumHosts) 
		fNumHosts = id + 1;
		
	fHostInfo.addr[id] = inAddr;
	
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::SetAddresses
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::SetAddresses(CInternetHostID_AC* inAddr, long numAddrs)
{	
	if(numAddrs > NUM_ALT_ADDRS)
		ThrowError_AC(paramErr);

	for(short i = 0; i < numAddrs; i++)
	{
		fHostInfo.addr[i] = inAddr[i];
	}

	fNumHosts = numAddrs;
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::AddAddress
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::AddAddress(const CInternetAddress_AC& inAddress)
{
	SetAddress(inAddress, fNumHosts);
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::GetNumAddresses
//----------------------------------------------------------------------------------------

long CMacTCPInternetHostRep_AC::GetNumAddresses() const
{
	return fNumHosts;
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::GetAddress
//----------------------------------------------------------------------------------------

CInternetAddress_AC CMacTCPInternetHostRep_AC::GetAddress(long id)
{
	if(id < 0 || id >= fNumHosts)
		ThrowError_AC(paramErr);

	return CInternetAddress_AC(fHostInfo.addr[id], 0); 
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::GetHostName
//----------------------------------------------------------------------------------------

const char* CMacTCPInternetHostRep_AC::GetHostName() const
{
	return fHostInfo.cname;
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::SetHostName
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::SetHostName(const char* inName)
{
	::strcpy(fHostInfo.cname, inName);
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::GetMaxNumHosts
//----------------------------------------------------------------------------------------

long CMacTCPInternetHostRep_AC::GetMaxNumHosts() const
{
	return NUM_ALT_ADDRS;
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::ResolveNameToAddressAsync
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::ResolveNameToAddressAsync()
{
	fHost->UseDefaultReactor();
	GetIdleTimeFrom(fHost->GetReactor());

	StartCall();
	
	if(!fgResultUPP) 
		fgResultUPP = NewResultProc(ResultProc);

	fHost->ChangeStateTo( CInternetHost_AC::kStartingNameToAddress );

	::strcpy(fPendingHostName, fHostInfo.cname);

	short err = CallStrToAddrProc(	reinterpret_cast<StrToAddrUPP>(fgDNRCodePtr), 
							STRTOADDR, 
							fPendingHostName, 
							&fHostInfo, 
							fgResultUPP, 
							reinterpret_cast<Ptr>(&gHostList));	
							
	if(err && err != cacheFault)
		ThrowError_AC(err);

	fHost->ChangeStateTo( CInternetHost_AC::kPendingNameToAddress );

}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::ResolveAddressToNameAsync
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::ResolveAddressToNameAsync()
{
	fHost->UseDefaultReactor();
	GetIdleTimeFrom(fHost->GetReactor());

	StartCall();
	
	if(!fgResultUPP) 
		fgResultUPP = NewResultProc(ResultProc);

	fHost->ChangeStateTo( CInternetHost_AC::kStartingAddressToName );
	
	fPendingAddress = fHostInfo.addr[0];

	short err = CallAddrToNameProc(	reinterpret_cast<AddrToNameUPP>(fgDNRCodePtr), 	// Ptr to DNR code resource
							ADDRTONAME, 											// MacTCP Selector
							fPendingAddress, 										// the address to resolve
							&fHostInfo,		 										// ptr to a hostInfo
							fgResultUPP, 											// result proc
							reinterpret_cast<Ptr>(&gHostList)); 					// userdata ptr

	if(err && err != cacheFault)
		ThrowError_AC(err);

	fHost->ChangeStateTo( CInternetHost_AC::kPendingAddressToName );

}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::ConvertAddressToString
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::ConvertAddressToString(char* str, CInternetHostID_AC host )
{
	Open();
	
	short err =	CallAddrToStrProc(reinterpret_cast<AddrToStrProcPtr>(fgDNRCodePtr), ADDRTOSTR, host, str);

	if(err && err != cacheFault)
		ThrowError_AC(err);
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::ConvertStringToAddress
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::ConvertStringToAddress(CInternetHostID_AC& host, const char* str)
{
	Open();
	
	hostInfo retHostInfo;
	::Erase_AC(&retHostInfo);	

	short err = CallStrToAddrProc(	reinterpret_cast<StrToAddrProcPtr>(fgDNRCodePtr), 
							STRTOADDR, 
							const_cast<char*>(str), 
							&retHostInfo, 
							NULL, 
							NULL);

	if(err && err != cacheFault)
		ThrowError_AC(err);
	
	host = retHostInfo.addr[0];
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::Open
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::Open()
{
	if (!IsOpen())
	{	
		char fileName[256] = {'\0'};	
		short refnum;
		
		/* open the MacTCP driver to get DNR resources. Search for it based on
		   creator & type rather than simply file name */	
		refnum = OpenOurRF();

		/* ignore failures since the resource may have been installed in the 
		   System file if running on a Mac 512Ke */
		   
		/* load in the DNR resource package */
		fgDNRCodeHndl = GetIndResource('dnrp', 1);
		if (fgDNRCodeHndl == nil)
		{
			/* can't open DNR */
			ThrowError_AC(ResError());
		}
		
		DetachResource(fgDNRCodeHndl);
		if (refnum != -1) 
		{
			CloseResFile(refnum);
		}
			
		/* lock the DNR resource since it cannot be reloated while opened */
		MoveHHi(fgDNRCodeHndl);
		HLock(fgDNRCodeHndl);
		
		fgDNRCodePtr = (ProcPtr)*fgDNRCodeHndl;
		
		/* call open resolver */
		// RRK modification 1/95 use CallOpenResolverProc define to call UPP
		
		OSErr err = CallOpenResolverProc((OpenResolverUPP)fgDNRCodePtr, OPENRESOLVER, fileName);
		if (err != noErr) 
		{
			/* problem with open resolver, flush it */
			HUnlock(fgDNRCodeHndl);
			DisposeHandle(fgDNRCodeHndl);
			fgDNRCodePtr = nil;
			
			ThrowError_AC(err);
		}
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::OpenOurRF
//----------------------------------------------------------------------------------------

short CMacTCPInternetHostRep_AC::OpenOurRF()
{
	short refnum;
	short vRefNum;
	long dirID;
	
	/* first search Control Panels for MacTCP 1.1 */
	::ThrowIfError_AC(
		::FindFolder(	
					kOnSystemDisk, 
					kControlPanelFolderType, 
					kDontCreateFolder, 
					&vRefNum, 
					&dirID));
	
	refnum = SearchFolderForDNRP('cdev', 'ztcp', vRefNum, dirID);
	if (refnum != -1) return(refnum);
		
	/* next search System Folder for MacTCP 1.0.x */
	
	::ThrowIfError_AC(
		::FindFolder(	
					kOnSystemDisk, 
					kSystemFolderType, 
					kDontCreateFolder, 
					&vRefNum, 
					&dirID));
	
	refnum = SearchFolderForDNRP('cdev', 'mtcp', vRefNum, dirID);
	if (refnum != -1) return(refnum);
		
	/* finally, search Control Panels for MacTCP 1.0.x */
	::ThrowIfError_AC(
		::FindFolder(	
					kOnSystemDisk, 
					kControlPanelFolderType, 
					kDontCreateFolder, 
					&vRefNum, 
					&dirID));

	refnum = SearchFolderForDNRP('cdev', 'mtcp', vRefNum, dirID);
	if (refnum != -1) return(refnum);
		
	return -1;
}	

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::SearchFolderForDNRP
//----------------------------------------------------------------------------------------

short CMacTCPInternetHostRep_AC::SearchFolderForDNRP(long targetType, long targetCreator, short vRefNum, long dirID)
{

/* SearchFolderForDNRP is called to search a folder for files that might 
	contain the 'dnrp' resource */

	HParamBlockRec fi;
	Str255 filename;
	short refnum;
	
	fi.fileParam.ioCompletion = nil;
	fi.fileParam.ioNamePtr = filename;
	fi.fileParam.ioVRefNum = vRefNum;
	fi.fileParam.ioDirID = dirID;
	fi.fileParam.ioFDirIndex = 1;

	while (PBHGetFInfoSync(&fi) == noErr) 
	{
		/* scan system folder for driver resource files of specific type & creator */
		if (fi.fileParam.ioFlFndrInfo.fdType == targetType &&
			fi.fileParam.ioFlFndrInfo.fdCreator == targetCreator) 
		{
			/* found the MacTCP driver file? */
			refnum = HOpenResFile(vRefNum, dirID, filename, fsRdPerm);
			if (GetIndResource('dnrp', 1) == NULL)
				CloseResFile(refnum);
			else
				return refnum;
		}
		/* check next file in system folder */
		++fi.fileParam.ioFDirIndex;
		fi.fileParam.ioDirID = dirID;	/* PBHGetFInfo() clobbers ioDirID */
	}
	return(-1);
}	

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::Close
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::Close()
{
	if (fgDNRCodePtr)
	{		
		CallCloseResolverProc((CloseResolverUPP)fgDNRCodePtr, CLOSERESOLVER);
	
		/* release the DNR resource package */
		HUnlock(fgDNRCodeHndl);
		DisposeHandle(fgDNRCodeHndl);
		fgDNRCodePtr = NULL;
		fgDNRCodeHndl = NULL;
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::ResultProc
//----------------------------------------------------------------------------------------

pascal void CMacTCPInternetHostRep_AC::ResultProc (
		struct hostInfo *hostInfoPtr, CMacTCPHostList_AC* hostList)
{
	CMacTCPInternetHostRep_AC* hostTarget = hostList->GetFirstHost();

	while(hostTarget)
	{
		if(&(hostTarget->fHostInfo) == hostInfoPtr)
		{
			hostTarget->CompleteCall();
			break;	
		}

		hostTarget = hostTarget->GetNextLink();	
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPInternetHostRep_AC::Idle
//----------------------------------------------------------------------------------------

void CMacTCPInternetHostRep_AC::Idle()
{
	if(fPending)
	{
		if(fTimer->TimedOut())
		{
			fPending = false;
			
			if(fHost->GetState() == CInternetHost_AC::kPendingNameToAddress)
			{
				::strcpy(fHostInfo.cname, fPendingHostName);
				
				CompleteNameToAddress(0, true);
			}
			else
			{
				fHostInfo.addr[0] = fPendingAddress;
				
				CompleteAddressToName(0, true);
			}
		}
		else
		if(fComplete)
		{
			if(fHost->GetState() == CInternetHost_AC::kPendingNameToAddress)
			{	
				// update the hosts count.
				
				fNumHosts = 0;
					
				for(short i = 0; i < NUM_ALT_ADDRS; i++)
				{
					if(fHostInfo.addr[i] != 0)
						fNumHosts++;
				}
				::strcpy(fHostInfo.cname, fPendingHostName);
				
				CompleteNameToAddress(fHostInfo.rtnCode);
			}
			else
			{
				fHostInfo.addr[0] = fPendingAddress;

				CompleteAddressToName(fHostInfo.rtnCode);
			}
			
			fComplete = false;
			fPending = false;
		}
	}
}

#endif