//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------



#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsMacTCP_AC

#ifndef __CMacTCPServiceState_AC__
#include "CMacTCPServiceState_AC.h"
#endif

#ifndef __CMacTCPStream_AC__
#include "CMacTCPStream_AC.h"
#endif

#ifndef __MacTCPPrivate_AC__
#include "MacTCPPrivate_AC.h"
#endif


//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::CMacTCPServiceState_AC
//----------------------------------------------------------------------------------------

CMacTCPServiceState_AC::CMacTCPServiceState_AC(CMacTCPStream_AC* inStream)
	: fStream(inStream)
{
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::~CMacTCPServiceState_AC
//----------------------------------------------------------------------------------------

CMacTCPServiceState_AC::~CMacTCPServiceState_AC()
{
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetServiceState
//----------------------------------------------------------------------------------------

long CMacTCPServiceState_AC::GetServiceState()
{
	::AssertNotNULL_AC(fStream);

	CMacTCPParamBlockTCP_AC* pb = fStream->StartTCPCommand(TCPStatus);
	pb->DoDriverCall(false);

	// states as per MacTCP manual.
	enum eState
	{
		kClosed			= 0,
		kListen 		= 2,
		kSYNReceived	= 4,
		kSYNSent		= 8,
		kEstablished 	= 10,
		kFINWait1		= 12,
		kFINWait2		= 14,
		kCloseWait		= 16,
		kLastAck		= 18,
		kTimeWait		= 20
	};

	SInt8 state = pb->csParam.status.remoteHost;

	long outState = 0;

	if(state & kListen)
		outState = kIdle; 
	else
	if(state & kSYNReceived)
		outState = kIncomingConnection;
	else
	if(state & kSYNSent)
		outState = kOutgoingConnection;
	else
	if(state & kEstablished)
		outState = kDataTransfer;
	else
	if(state & kCloseWait)
		outState = kIncomingRelease;
	else
		outState = kOutgoingRelease;
	
	return outState;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::IsConnectionOriented
//----------------------------------------------------------------------------------------

CServiceState_AC::eStateBool CMacTCPServiceState_AC::IsConnectionOriented()
{
	return kTrue;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::IsTransactionBased
//----------------------------------------------------------------------------------------

CServiceState_AC::eStateBool CMacTCPServiceState_AC::IsTransactionBased()
{
	return kUnknown;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::SupportsOrderlyDisconnect
//----------------------------------------------------------------------------------------

CServiceState_AC::eStateBool CMacTCPServiceState_AC::SupportsOrderlyDisconnect()
{
	::AssertNotNULL_AC(fStream);
	return kUnknown;
}

//----------------------------------------------------------------------------------------
//  CMacTCPServiceState_AC::SupportsZeroLengthTSDU
//----------------------------------------------------------------------------------------

CServiceState_AC::eStateBool CMacTCPServiceState_AC::SupportsZeroLengthTSDU()
{
	::AssertNotNULL_AC(fStream);
	return kUnknown;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetMaxSizeOfOptions
//----------------------------------------------------------------------------------------

long CMacTCPServiceState_AC::GetMaxSizeOfOptions()
{
	::AssertNotNULL_AC(fStream);
	return kUnknown;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetTSDUSize
//----------------------------------------------------------------------------------------

long CMacTCPServiceState_AC::GetTSDUSize()
{
	::AssertNotNULL_AC(fStream);
	return kUnknown;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetExpeditedTSDUSize
//----------------------------------------------------------------------------------------

long CMacTCPServiceState_AC::GetExpeditedTSDUSize()
{
	::AssertNotNULL_AC(fStream);
	return kUnknown;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetMaxDataSizeOnConnect
//----------------------------------------------------------------------------------------

long CMacTCPServiceState_AC::GetMaxDataSizeOnConnect() 
{
	::AssertNotNULL_AC(fStream);
	return kUnknown;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetMaxDataSizeOnDisconnect
//----------------------------------------------------------------------------------------

long CMacTCPServiceState_AC::GetMaxDataSizeOnDisconnect()
{
	::AssertNotNULL_AC(fStream);
	return kUnknown;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetRemoteAddress
//----------------------------------------------------------------------------------------

void CMacTCPServiceState_AC::GetRemoteAddress(CNetworkAddress_AC& inAddress)
{
	inAddress = fStream->GetRemoteHost();
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetStream
//----------------------------------------------------------------------------------------

CMacTCPStream_AC* CMacTCPServiceState_AC::GetStream()
{
	return fStream;
}

//----------------------------------------------------------------------------------------
// CMacTCPServiceState_AC::GetAddressType
//----------------------------------------------------------------------------------------

unsigned long CMacTCPServiceState_AC::GetAddressType()
{
	return kInternet;
}
#endif