//----------------------------------------------------------------------------------------
// CMacTCPStream_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsMacTCP_AC

#ifndef __CMacTCPStream_AC__
#include "CMacTCPStream_AC.h"
#endif

// patterns
#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif


// net 
#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CProvider_AC__
#include "CProvider_AC.h"
#endif

#ifndef __CServiceState_AC__
#include "CServiceState_AC.h"
#endif

#ifndef __CMacTCPProvider_AC__
#include "CMacTCPProvider_AC.h"
#endif


// text
#ifndef __CBuffer_AC__
#include "CBuffer_AC.h"
#endif


// containers
#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif


// MacOS
#ifndef __Devices__
#include <Devices.h>
#endif


#if qNetDebug_AC
#ifndef __CMacTCPDebug_AC__
#include "CMacTCPDebug_AC.h"
#endif

#endif

// MacTCP
#ifndef __MacTCPPrivate_AC__
#include "MacTCPPrivate_AC.h"
#endif

#ifndef __CMacTCPServiceState_AC__
#include "CMacTCPServiceState_AC.h"
#endif



//----------------------------------------------------------------------------------------
// class CMacTCPStreamDestructor_AC
//----------------------------------------------------------------------------------------

class CMacTCPStreamDestructor_AC
{
public:
	CMacTCPStreamDestructor_AC() {}
	~CMacTCPStreamDestructor_AC()
	{	CMacTCPStream_AC::CloseAllStreams(); }
};

CMacTCPStreamDestructor_AC gDestructor;

long CMacTCPStream_AC::fgCounter = 0;

CMacTCPStream_AC::CList* CMacTCPStream_AC::fgStreams = NULL;


//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::CMacTCPStream_AC
//----------------------------------------------------------------------------------------

CMacTCPStream_AC::CMacTCPStream_AC(CReactor_AC* inReactor)
:	MEventTransceiver_AC(inReactor),
	fStream(0),
	fID(++fgCounter),
	fTCPNotifyUPP(NULL),
	fTCPCompletionProc(NULL)
{	
	GetIdleTimeFrom(GetReactor());
	CProvider_AC::Open();
	
	if(!fgStreams)
		fgStreams = TH_new CList;
	
	fgStreams->Push(this);
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::~CMacTCPStream_AC
//----------------------------------------------------------------------------------------

CMacTCPStream_AC::~CMacTCPStream_AC()
{
	if(fTimer) fTimer->Stop();
	Close();
	
	fgStreams->Remove(this);
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::Close
//----------------------------------------------------------------------------------------

void CMacTCPStream_AC::Close()
{
	if(fStream) 
	{
		TCPiopb pb;
		::Erase_AC(&pb);
	
		pb.ioCompletion	 = GetTCPCompletionUPP();
		pb.ioCRefNum = CMacTCPProvider_AC::GetRefNum();
		pb.csCode = TCPRelease;
		pb.tcpStream = fStream;
		
		OSErr osErr = PBControlSync((ParmBlkPtr) &pb);
		::ThrowIfError_AC(osErr);
	
		fStream = 0;
	}
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::Idle
//----------------------------------------------------------------------------------------

void CMacTCPStream_AC::Idle()
{	

	if(fEventStream.HasData())
	{			
		short eventFrom, eventCode, terminateReason = 0;
		
		fEventStream >> eventFrom >> eventCode;
		
		if(eventFrom == kNotify) 
			fEventStream >> terminateReason;

#if qNetTrace_AC
	
		if(eventFrom == kCompletion )
			fTCPParamBlockPtr->csCode = 0;
				

		TRACE 	<< MediumTracePriority_AC 
				<< "\n CMacTCPStream_AC #"
				<< GetID() 
				<< " Received Async Event: " 
				<< CMacTCPDebug_AC::GetTCPEventCode(eventCode)
				<< "/"
				<< CMacTCPDebug_AC::GetTCPTerminationReason(terminateReason)
				<< "/"
				<< (eventFrom == kNotify ? "Notify" : "Completion")
				<< "/"
				<< CMacTCPDebug_AC::GetTCPEventCode(fTCPParamBlockPtr->csCode)
				<< endl_AC;
#endif

		unsigned long eventData = (eventFrom << 16) | eventCode;

		PostEvent( CreateEventWithData_AC(		CNetEvent_AC::kProvider, 
												eventData, 
												terminateReason));
			// Send the event.
	}			
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::StartTimer
//----------------------------------------------------------------------------------------

void CMacTCPStream_AC::StartTimer(long interval)
{
	if(!fTimer)
	{
		fTimer = TH_new CEventTransceiverTimer_AC(this);
		fTimer->GetIdleTimeFrom(GetReactor());
	}
	fTimer->SetDuration(interval);
	fTimer->Start();
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::StopTimer
//----------------------------------------------------------------------------------------

void CMacTCPStream_AC::StopTimer()
{
	if(fTimer) 
		fTimer->Stop();
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::GetState
//----------------------------------------------------------------------------------------

CServiceState_AC* CMacTCPStream_AC::GetState()
{
	if(!fState)
	{
		fState = TH_new CMacTCPServiceState_AC(this);
	}
	
	AssertNotNULL_AC(fState);
	
	return fState;
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::GetTCPNotifyUPP
//----------------------------------------------------------------------------------------

TCPNotifyUPP CMacTCPStream_AC::GetTCPNotifyUPP()
{
	if(!fTCPNotifyUPP) 
	{
		fTCPNotifyUPP = NewTCPNotifyProc(TCPNotifyProc);
		::ThrowIfNULL_AC(fTCPNotifyUPP);
	}
	
	return fTCPNotifyUPP;
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::GetTCPCompletionUPP
//----------------------------------------------------------------------------------------

TCPIOCompletionUPP CMacTCPStream_AC::GetTCPCompletionUPP()
{
	if(!fTCPCompletionProc)
	{
		fTCPCompletionProc = NewTCPIOCompletionProc(TCPCompletionProc);
		::ThrowIfNULL_AC(fTCPCompletionProc);	
	}
	
	return fTCPCompletionProc;
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::GetBuffer
//----------------------------------------------------------------------------------------

CBuffer_AC* CMacTCPStream_AC::GetBuffer()
{
	if(!fBuffer)
	{
		fBuffer = TH_new CBuffer_AC(kBufferSize);
	}
	
	return fBuffer;
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::TCPCompletionProc
//----------------------------------------------------------------------------------------

pascal void CMacTCPStream_AC::TCPCompletionProc(struct TCPiopb *pb)
{
	void *ptr = NULL;

	switch(pb->csCode)
	{
		case TCPRelease:
		case TCPCreate:
			ptr = pb->csParam.create.userDataPtr;
			break;
		case TCPPassiveOpen:
		case TCPActiveOpen:
			ptr = pb->csParam.open.userDataPtr;
			break;
		case TCPSend:
			ptr = pb->csParam.send.userDataPtr;
			break;
		case TCPNoCopyRcv:
		case TCPRcvBfrReturn:
		case TCPRcv:
			ptr = pb->csParam.receive.userDataPtr;
			break;
		case TCPClose:
			ptr = pb->csParam.close.userDataPtr;
			break;
		case TCPAbort:
			ptr = pb->csParam.abort.userDataPtr;
			break;
		case TCPStatus:
			ptr = pb->csParam.status.userDataPtr;
			break;
		case TCPGlobalInfo:
			ptr = pb->csParam.globalInfo.userDataPtr;
			break;
	}

	if(ptr)
	{
		try
		{
			CMacTCPStream_AC* stream = reinterpret_cast<CMacTCPStream_AC*>(ptr);
			stream->fEventStream << (short) kCompletion << pb->csCode;
		}
		catch(...)
		{
		}
	}
#if qNetDebug_AC
	else
	{
		DebugStr("\pCompletion proc failed to get stream;g");
	}
#endif
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::TCPNotifyProc
//----------------------------------------------------------------------------------------

pascal void CMacTCPStream_AC::TCPNotifyProc(	StreamPtr inStream, 
										unsigned short inCode, 
										CMacTCPStream_AC* streamPtr, 
										unsigned short terminReason, 
										struct ICMPReport *icmpMsg)
{
	inStream;
		// don't need this info
	icmpMsg;
		

	try
	{
		if(streamPtr)
		{
			streamPtr->fEventStream << (short) kNotify << inCode << terminReason;
		}
#if qNetDebug_AC
		else 
			DebugStr("\pCMacTCPStream_AC:Event not handled!;g");
#endif
	}
	catch(...)
	{
#if qNetDebug_AC
		DebugStr("\pCMacTCPStream_AC::Unknown exception;g");
#endif	
	}
	
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::StartTCPCommand
//----------------------------------------------------------------------------------------

CMacTCPParamBlockTCP_AC* CMacTCPStream_AC::StartTCPCommand( short inCode )
{
	if(!fTCPParamBlockPtr)
		fTCPParamBlockPtr = TH_new CMacTCPParamBlockTCP_AC;
	else
		fTCPParamBlockPtr->Clean();
		
	fTCPParamBlockPtr->csCode 		= inCode;
	fTCPParamBlockPtr->tcpStream 	= fStream;
	fTCPParamBlockPtr->ioCRefNum 	= CMacTCPProvider_AC::GetRefNum();
	fTCPParamBlockPtr->ioCompletion = GetTCPCompletionUPP();

	// this is for TCPCompletionProc, mainly.
	switch(inCode)
	{
		case TCPRelease:
		case TCPCreate:
			fTCPParamBlockPtr->csParam.create.userDataPtr = CoerceToPtr();
			break;
		case TCPPassiveOpen:
		case TCPActiveOpen:
			fTCPParamBlockPtr->csParam.open.userDataPtr = CoerceToPtr();
			break;
		case TCPSend:
			fTCPParamBlockPtr->csParam.send.userDataPtr = CoerceToPtr();
			break;
		case TCPNoCopyRcv:
		case TCPRcvBfrReturn:
		case TCPRcv:
			fTCPParamBlockPtr->csParam.receive.userDataPtr = CoerceToPtr();
			break;
		case TCPClose:
			fTCPParamBlockPtr->csParam.close.userDataPtr = CoerceToPtr();
			break;
		case TCPAbort:
			fTCPParamBlockPtr->csParam.abort.userDataPtr = CoerceToPtr();
			break;
		case TCPStatus:
			fTCPParamBlockPtr->csParam.status.userDataPtr = CoerceToPtr();
			break;
		case TCPGlobalInfo:
			fTCPParamBlockPtr->csParam.globalInfo.userDataPtr = CoerceToPtr();
			break;	
	}
	
	return fTCPParamBlockPtr;
}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::GetEventData
//----------------------------------------------------------------------------------------

void CMacTCPStream_AC::GetEventData(	CEvent_AC* inEvent,
								short& eventType, short& eventCode, short& terminateReason )
{
	unsigned long data = inEvent->GetULongData();
	
	eventType = data >> 16;
	eventCode = data & 0xFFFF;
	terminateReason = inEvent->GetError();

}

//----------------------------------------------------------------------------------------
// CMacTCPStream_AC::CloseAllStreams
//----------------------------------------------------------------------------------------

void CMacTCPStream_AC::CloseAllStreams()
{
	if(fgStreams)
	{
		{
			CList::CIterator walker = fgStreams->First();
			
			while(walker.IsValid())
			{
				(*walker)->Close();
				walker.Next();
			}
		}

		delete fgStreams;
		fgStreams = NULL;
	}
}

#endif