//----------------------------------------------------------------------------------------
// COTAcceptor_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __COTAcceptor_AC__
#define __COTAcceptor_AC__

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#if qWantsOpenTransport_AC

// Network

#ifndef __COTOpener_AC__
#include "COTOpener_AC.h"
#endif

#ifndef __CServiceAcceptor_AC__
#include "CServiceAcceptor_AC.h"
#endif

#ifndef __CTCPConnection_AC__
#include "CTCPConnection_AC.h"
#endif

#ifndef __COTConfig_AC__
#include "COTConfig_AC.h"
#endif

// Core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

// forward references

template <class T> class CLinkedList_AC;
template <class ADDRESS> class COTDisconnector_AC;
template <class ADDRESS> class COTEndpoint_AC;
template <class ADDRESS> class COTPendingConnection_AC;
class COTAcceptorStrategy_AC;

//----------------------------------------------------------------------------------------
// class COTPendingConnection_AC
// This is a OT implementation class.
//----------------------------------------------------------------------------------------

template <class ADDRESS>
class COTPendingConnection_AC : public CStateMachine_AC
{
public:

	enum eState
	{
		kOpening = kFirstUserState,
		kDoListen,
		kAccept,
		kConnected
	};
	
	COTPendingConnection_AC()
	{
		::Erase_AC(&fConnectionInfo);
	}
	
	COTPendingConnection_AC(const COTPendingConnection_AC& inConnection)
	{
		*this = inConnection;
	}
	COTPendingConnection_AC(const CRefCountingPtr_AC<COTOpener_AC<ADDRESS > >& opener)
		: fOpener(opener)
	{
		::Erase_AC(&fConnectionInfo);
	}
	COTPendingConnection_AC& operator = (const COTPendingConnection_AC& inConnection)
	{
		fOpener = inConnection.fOpener;
		::Copy_AC(&fConnectionInfo, &inConnection.fConnectionInfo);
		return *this;
	}
	
	CRefCountingPtr_AC<COTOpener_AC<ADDRESS > > fOpener;
	TCall fConnectionInfo;
};

//----------------------------------------------------------------------------------------
// class COTAcceptor_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
class EXPORT_AC COTAcceptor_AC : public CServiceAcceptor_AC
{ 
		
public:
	// interface
	COTAcceptor_AC(CReactor_AC* inReactor, long timeOut);

	virtual ~COTAcceptor_AC();
	
	virtual void ListenTo(	const CNetworkAddress_AC& inLocalAddress,
							const CNetworkAddress_AC& inRemoteAddress);
	
	virtual void Accept(CServiceHandler_AC* inHandler);
	
protected:
	// implementation
	
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);
	
	virtual void HandleConnectionRequest();

	virtual long HandleListen(COTPendingConnection_AC<ADDRESS>& inPCon);
	virtual long HandleAccept(COTPendingConnection_AC<ADDRESS>& inPCon);
		// if either of these get an otLook err, they'll return
		// the event, else they'll return zero.
			
	virtual void Idle();
	
	void HandleEndpointOpen(COTPendingConnection_AC<ADDRESS>& inPCon);
	
	void HandleConnectionComplete(COTEndpoint_AC<ADDRESS>* inEndpoint);
		
protected:	

	typedef CLinkedList_AC< COTPendingConnection_AC<ADDRESS > >	CList;
	
	CAutoPtr_AC<CList> 									fConnections;
	CRefCountingPtr_AC<COTOpener_AC<ADDRESS> >			fListener;
	CAutoPtr_AC<COTAcceptorStrategy_AC> 				fStrategy;
	CRefCountingPtr_AC<COTEndpoint_AC<ADDRESS> >		fEndpoint;
	CRefCountingPtr_AC<COTDisconnector_AC<ADDRESS> > 	fDisconnector;
	
	ADDRESS												fRemoteAddress;

//	friend COTAcceptor_AC<ADDRESS>;
};
#endif
#endif