//----------------------------------------------------------------------------------------
// COTAcceptor_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTEndpoint_AC__
#include "COTEndpoint_AC.h"
#endif

#ifndef __COTAcceptor_AC__
#include "COTAcceptor_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __COTAcceptor_AC__
#include "COTAcceptor_AC.h"
#endif

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

#ifndef __COTStrategies_AC__
#include "COTStrategies_AC.h"
#endif

#ifndef __COTConfig_AC__
#include "COTConfig_AC.h"
#endif

#ifndef __COTUtils_AC__
#include "COTUtils_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __COTConnection_AC__
#include "COTConnection_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

#ifndef __CStateMachine_AC__
#include "CStateMachine_AC.h"
#endif


//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::COTAcceptor_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTAcceptor_AC<ADDRESS>::COTAcceptor_AC(
	CReactor_AC* inReactor, long timeOut)
:	CServiceAcceptor_AC(inReactor, timeOut)
{
	fListener = TH_new COTOpener_AC<ADDRESS>(inReactor);
	fEndpoint = fListener->GetEndpoint();
	fDisconnector = TH_new COTDisconnector_AC<ADDRESS>(inReactor);
	fDisconnector->Open(fEndpoint);

	RegisterForEventsFrom(fEndpoint);
	RegisterForEventsFrom(fListener);
	RegisterForEventsFrom(fDisconnector);
		
	fStrategy = COTAcceptorStrategy_AC::Create();
	
	fConnections = TH_new CList;
	
	GetIdleTimeFrom(inReactor);
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::~COTAcceptor_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTAcceptor_AC<ADDRESS>::~COTAcceptor_AC()
{
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::ReceiveEvent
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTAcceptor_AC<ADDRESS>::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kTimedOut:
		case CNetEvent_AC::kError:
		{
			ReceiveEvent_AC(inEvent);
			
			CRefCountingPtr_AC<COTOpener_AC<ADDRESS> > opener = 
				dynamic_cast<COTOpener_AC<ADDRESS>*>(inTransceiver);
			
			if(opener)
			{
				CList::CIterator finder = fConnections->First();
				
				while(finder.IsValid())
				{
					if((*finder).fOpener == opener)
					{
						fConnections->Remove(finder);
						break;
					}
				
					finder.Next();
				}
			}				
			
			if(inEvent->GetEventID() == CNetEvent_AC::kTimedOut)
				PostEvent(inEvent);
			else	
				PostEvent(TH_new CEvent_AC(CNetEvent_AC::kAcceptFailed, inEvent->GetError()));
		}
		break;

		case CNetEvent_AC::kProviderOpen:
		{
			
			
			ReceiveEvent_AC(inEvent);
			
			CRefCountingPtr_AC<COTOpener_AC<ADDRESS> > opener = 
				dynamic_cast_AC(COTOpener_AC<ADDRESS>*, inTransceiver);
			
			CList::CIterator finder = fConnections->First();
			
			while(finder.IsValid())
			{
				if((*finder).fOpener == opener)
				{
					(*finder).ChangeStateTo(COTPendingConnection_AC<ADDRESS>::kDoListen);
					HandleEndpointOpen(finder.Current());
				}
			
				finder.Next();
			}
		}
		break;
		
		case CNetEvent_AC::kProvider:
		{
			switch(inEvent->GetLongData())
			{
				case T_LISTEN:
					ReceiveEvent_AC(inEvent);
					::ThrowIfError_AC(inEvent->GetError());
					HandleConnectionRequest();
				break;
				
				case T_ACCEPTCOMPLETE:
					ReceiveEvent_AC(inEvent);
					::ThrowIfError_AC(inEvent->GetError());
				break;
				
				case T_PASSCON:
					ReceiveEvent_AC(inEvent);
					::ThrowIfError_AC(inEvent->GetError());
					HandleConnectionComplete( dynamic_cast_AC(COTEndpoint_AC<ADDRESS>*, inTransceiver));
					
				break;
			}	
		}
		break;

		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
		break;

	}
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::HandleEndpointOpen
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTAcceptor_AC<ADDRESS>::HandleEndpointOpen(COTPendingConnection_AC<ADDRESS>& inPCon)
{
	long returnEvent = 0;
	
	if(inPCon.GetState() == COTPendingConnection_AC<ADDRESS>::kDoListen)
	{
		returnEvent = HandleListen(inPCon);
		if(!returnEvent)
		{
			inPCon.ChangeStateTo(COTPendingConnection_AC<ADDRESS>::kAccept);
		}

	}
	
	if(inPCon.GetState() == COTPendingConnection_AC<ADDRESS>::kAccept)
	{
		returnEvent = HandleAccept(inPCon);
		
		if(!returnEvent)
		{
			inPCon.ChangeStateTo(COTPendingConnection_AC<ADDRESS>::kNullState);
		}

	}
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::Idle
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTAcceptor_AC<ADDRESS>::Idle()
{
	CList::CIterator walker = fConnections->First();
		
	while(walker.IsValid())
	{
		if(	(*walker).GetState() == COTPendingConnection_AC<ADDRESS>::kDoListen || (*walker).GetState() == COTPendingConnection_AC<ADDRESS>::kAccept )
		{
			HandleEndpointOpen( walker.Current() );
		}
	
		walker.Next();
	}
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::HandleListen
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTAcceptor_AC<ADDRESS>::HandleListen(COTPendingConnection_AC<ADDRESS>& inPCon)
{
	::Erase_AC(&inPCon.fConnectionInfo);
	
	inPCon.fConnectionInfo.addr.maxlen = sizeof(ADDRESS);
	inPCon.fConnectionInfo.addr.len = sizeof(ADDRESS);
	inPCon.fConnectionInfo.addr.buf =  (unsigned char*) &fEndpoint->GetRemoteAddress();

	inPCon.fConnectionInfo.opt.maxlen = 0;
	inPCon.fConnectionInfo.opt.buf = NULL;
	inPCon.fConnectionInfo.udata.maxlen = 0;

	CError_AC err = fEndpoint->GetOTEndpoint()->Listen( &inPCon.fConnectionInfo );
	
	long evt = 0;
	
	if(err == kOTLookErr)
	{
		evt = fEndpoint->GetOTEndpoint()->Look();
	}
	else
		::ThrowIfError_AC(err);
		
	return evt;
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::HandleAccept
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTAcceptor_AC<ADDRESS>::HandleAccept(COTPendingConnection_AC<ADDRESS>& inPCon)
{
	::AssertNotNULL_AC(inPCon.fOpener);
	::AssertNotNULL_AC(inPCon.fOpener->GetEndpoint());
	::AssertNotNULL_AC(inPCon.fOpener->GetEndpoint()->GetOTEndpoint());

	CError_AC err = fEndpoint->GetOTEndpoint()->Accept(
		inPCon.fOpener->GetEndpoint()->GetOTEndpoint(), &inPCon.fConnectionInfo);	

	
	long evt = 0;
	
	if(err == kOTLookErr)
	{
		evt = fEndpoint->GetOTEndpoint()->Look();
	}
	else
	{
		::ThrowIfError_AC(err);
		
		ADDRESS remoteAddress;
		
		BlockMove_AC(inPCon.fConnectionInfo.addr.buf, &remoteAddress, inPCon.fConnectionInfo.addr.len);

		inPCon.fOpener->GetEndpoint()->SetRemoteAddress(remoteAddress);
	}
		
	return evt;
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::ListenTo
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTAcceptor_AC<ADDRESS>::ListenTo(	const CNetworkAddress_AC& inLocalAddress,
										const CNetworkAddress_AC& inRemoteAddress)
{
	
	ADDRESS localAddress;
	ADDRESS remoteAddress;
	
	// if this fails, it is CAcceptor_AC's responsibility to handle the error	
	COTUtils_AC::Convert(localAddress, inLocalAddress);
	COTUtils_AC::Convert(remoteAddress, inRemoteAddress);
	
	fEndpoint->SetRemoteAddress(remoteAddress);
	
	fListener->Open(fStrategy->GetConfig(), localAddress);
		// we want to bind to this address
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::Accept
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTAcceptor_AC<ADDRESS>::Accept(CServiceHandler_AC* inConnection)
{
	::AssertNotNULL_AC(inConnection);
	
	
	try
	{
		COTPendingConnection_AC<ADDRESS> pCon;
	
		CList::CIterator finder = fConnections->First();
		
		while(finder.IsValid())
		{
			if((*finder).GetState() == COTPendingConnection_AC<ADDRESS>::kConnected)
			{
				pCon = finder.Current();
				fConnections->Remove(finder);
				break;
			}
		
			finder.Next();
		}
	
		CRefCountingPtr_AC<COTEndpoint_AC<ADDRESS> > endpoint = pCon.fOpener->GetEndpoint();
		
		COTConnection_AC<ADDRESS>* stream = dynamic_cast<COTConnection_AC<ADDRESS>*>(inConnection->GetService());
		::AssertNotNULL_AC(stream);
		
		stream->Open(endpoint);	
			// if this fails the opener will catch the problem and we'll okay 
			// because we've release the opener already.
	}
	catch(CException_AC& inEx)
	{
		inConnection->Hangup();

		PostEvent( 
			TH_new CEvent_AC(CNetEvent_AC::kError, inEx.GetError()));
	}
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::HandleConnectionRequest
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTAcceptor_AC<ADDRESS>::HandleConnectionRequest()
{
	
	fEndpoint->StopTimer();

	if(	COTConnection_AC<ADDRESS>::GetConnectionCount() >=
		COTConnection_AC<ADDRESS>::GetMaxConnectionCount() )
	{
		fDisconnector->Hangup();
	//	fEndpoint->StartTimer(fTimeOut);
		PostEvent(
			TH_new CEvent_AC(CNetEvent_AC::kAcceptRejected));
	}
	else
	{
		CRefCountingPtr_AC<COTOpener_AC<ADDRESS > > opener;

		try
		{
			if( COTConnection_AC<ADDRESS>::GetMaxConnectionCount() == 1)
				opener = fListener;
			else
			{
				
				opener = TH_new COTOpener_AC<ADDRESS >(GetReactor());
				ADDRESS empty;
				::Erase_AC(&empty);
				opener->Open( fStrategy->GetConfig() , empty, false);

				RegisterForEventsFrom(opener);
				RegisterForEventsFrom(opener->GetEndpoint());
				
			//	fEndpoint->StartTimer(fTimeOut);
					// start time out on next connection request
			}
			
			COTPendingConnection_AC<ADDRESS> pCon(opener);
			pCon.ChangeStateTo(COTPendingConnection_AC<ADDRESS>::kOpening);
			
			fConnections->Push(pCon);
			
		}
		catch(CException_AC& inEx)
		{
			fDisconnector->Hangup();
			PostEvent(
				TH_new CEvent_AC(CNetEvent_AC::kAcceptFailed, inEx.GetError()));
		}
	}
}

//----------------------------------------------------------------------------------------
// COTAcceptor_AC<ADDRESS>::HandleConnectionComplete
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTAcceptor_AC<ADDRESS>::HandleConnectionComplete(COTEndpoint_AC<ADDRESS>* inEndpoint)
{
	CList::CIterator walker = fConnections->First();
	
	while(walker.IsValid())
	{
		if((*walker).fOpener->GetEndpoint() == inEndpoint)
		{
			(*walker).ChangeStateTo(COTPendingConnection_AC<ADDRESS>::kConnected);
			break;
		}
		walker.Next();
	}
	

	PostEvent(
		TH_new CEvent_AC(CNetEvent_AC::kAcceptSucceeded));
}

#endif