//----------------------------------------------------------------------------------------
// COTConfig_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTConfig_AC__
#include "COTConfig_AC.h"
#endif


#ifndef __OPENTRANSPORT__
#	ifndef __OpenTransport__
#	include <OpenTransport.h>
#	endif
#endif

#ifndef __OPENTPTINTERNET__
#	ifndef __OpenTptInternet__
#	include <OpenTptInternet.h>
#	endif
#endif


//----------------------------------------------------------------------------------------
// COTConfig_AC::COTConfig_AC
//----------------------------------------------------------------------------------------

COTConfig_AC::COTConfig_AC(ePredefinedConfig inType)
: fFlags(0), fConfig(kOTInvalidConfigurationPtr) 
{ 
	switch(inType)
	{
		case kTCP:
			CreateTCPConfig();
			break;
		case kUDP:
			CreateUDPConfig();
			break;
		case kRawIP:
			CreateRawIPConfig();
			break;
	} 
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::COTConfig_AC
//----------------------------------------------------------------------------------------

COTConfig_AC::COTConfig_AC(const char* inConfig)
: fFlags(0), fConfig(NULL) 
{ 
	*this = inConfig; 
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::COTConfig_AC
//----------------------------------------------------------------------------------------

COTConfig_AC::COTConfig_AC(const COTConfig_AC& inConfig) 
: fFlags(0), fConfig(NULL) 
{ 
	*this = inConfig; 
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::COTConfig_AC
//----------------------------------------------------------------------------------------

COTConfig_AC::COTConfig_AC() 
: fFlags(0), fConfig(NULL) 
{
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::~COTConfig_AC
//----------------------------------------------------------------------------------------

COTConfig_AC::~COTConfig_AC() 
{ 
	if(fConfig) OTDestroyConfiguration(fConfig);
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::GetClonedOTConfig
//----------------------------------------------------------------------------------------

OTConfiguration* COTConfig_AC::GetClonedOTConfig() const
{ 
	return fConfig? 
		::OTCloneConfiguration(fConfig) : fConfig;
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::operator =
//----------------------------------------------------------------------------------------

void COTConfig_AC::operator = (const char* inConfig)
{
	if(fConfig) ::OTDestroyConfiguration(fConfig);
	
	if(inConfig)
		fConfig = ::OTCreateConfiguration(inConfig);
	else
		fConfig = NULL;
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::operator =
//----------------------------------------------------------------------------------------

COTConfig_AC& COTConfig_AC::operator = (const COTConfig_AC& inConfig) 
{
	*this = inConfig.fConfig;
	fFlags = inConfig.fFlags;
	
	return *this;
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::operator =
//----------------------------------------------------------------------------------------

void COTConfig_AC::operator = (const OTConfiguration* inConfig)
{
	if(fConfig) ::OTDestroyConfiguration(fConfig);
	fConfig = ::OTCloneConfiguration( const_cast<OTConfiguration*>(inConfig));
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::SetOpenFlags
//----------------------------------------------------------------------------------------

void COTConfig_AC::SetOpenFlags(OTOpenFlags inFlags) 
{ 
	fFlags = inFlags; 
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::GetOpenFlags
//----------------------------------------------------------------------------------------

OTOpenFlags COTConfig_AC::GetOpenFlags() 
{ 
	return fFlags; 
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::CreateTCPConfig
//----------------------------------------------------------------------------------------

void COTConfig_AC::CreateTCPConfig() 
{ 
	*this = ::OTCreateConfiguration(kTCPName); 
	fFlags = 0; 
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::CreateUDPConfig
//----------------------------------------------------------------------------------------

void COTConfig_AC::CreateUDPConfig() 
{ 
	*this = ::OTCreateConfiguration(kUDPName); 
	fFlags = 0; 
}

//----------------------------------------------------------------------------------------
// COTConfig_AC::CreateRawIPConfig
//----------------------------------------------------------------------------------------

void COTConfig_AC::CreateRawIPConfig() 
{ 
	*this = ::OTCreateConfiguration(kRawIPName); 
	fFlags = 0; 
}
#endif