//----------------------------------------------------------------------------------------
// COTConnection_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTConnection_AC__
#include "COTConnection_AC.h"
#endif


#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif


#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __COTUtils_AC__
#include "COTUtils_AC.h"
#endif
 
#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __OTDebug_AC__
#include "OTDebug_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif

#ifndef __COTEndpoint_AC__
#include "COTEndpoint_AC.h"
#endif


template <class ADDRESS>
unsigned long COTConnection_AC<ADDRESS>::fgMaxConnections = 0xFFFFFFFF;
template <class ADDRESS>
unsigned long COTConnection_AC<ADDRESS>::fgConnectionCount = 0;

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::COTConnection_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTConnection_AC<ADDRESS>::COTConnection_AC(CReactor_AC* inReactor)
	: CService_AC(inReactor),
	fConnected(false),
	fPendingSndData(NULL),
	fPendingSndAmt(0),
	fPendingFlags(0)
{	
	fDisconnector = TH_new COTDisconnector_AC<ADDRESS>(inReactor);
	RegisterForEventsFrom(fDisconnector);
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::~COTConnection_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTConnection_AC<ADDRESS>::~COTConnection_AC()
{
	if(fTimer) 
		fTimer->Stop();
	fgConnectionCount--;
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::Open
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnection_AC<ADDRESS>::Open(COTEndpoint_AC<ADDRESS>* inPoint) 
{ 
#if qNetTrace_AC
	TRACE << LowTracePriority_AC << "Opened COTConnection_AC" << endl_AC;
#endif
	
	fDisconnector->Open(inPoint);
	fEndpoint = inPoint; 
	SetReactor(inPoint->GetReactor());
	
	RegisterForEventsFrom(fEndpoint);
	
	fConnected = true;
	
	fgConnectionCount++;
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::ReceiveEvent
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnection_AC<ADDRESS>::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kError:
			PostEvent(inEvent);
		break;

		case CNetEvent_AC::kConnectionTerminated:
			ReceiveEvent_AC(inEvent);
			PostEvent(inEvent);
			fConnected = false;
			DisableServiceHandlerTimer();
		break;
		
		case CNetEvent_AC::kProvider: 
		{
			long otEvent = inEvent->GetLongData();

			switch(otEvent)
			{
				case T_EXDATA:
				case T_DATA:
				{
					
					ReceiveEvent_AC(inEvent);
					
					unsigned long bytes;
					::OTCountDataBytes(fEndpoint->GetOTEndpoint(), &bytes);
					
					if(bytes > 0)	
						PostEvent( 
							TH_new CEvent_AC(CNetEvent_AC::kIncomingData));
#if qNetTrace_AC
					else
						TRACE << LowTracePriority_AC << "Filtered out T_DATA with 0k Data" << endl_AC;
#endif					
					if(fTiming) fTimer->Start();	
				}	
				break;

				case T_GODATA:
				case T_GOEXDATA:

					ReceiveEvent_AC(inEvent);
				
					if(fPendingSndData && fPendingSndAmt)
					{
						const char* tempBufPtr = fPendingSndData;
						long tempAmt = fPendingSndAmt;
						long tempFlags = fPendingFlags; 
						
						fPendingSndData = NULL;
						fPendingSndAmt = 0;
						fPendingFlags = 0;
						
						Snd(tempBufPtr, tempAmt, tempFlags);
									
						if(fTiming) fTimer->Start();	
					}
				break;
				
			}
		}
		break;
		
		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
		break;

	}
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::Rcv
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnection_AC<ADDRESS>::Rcv(char* inPtr, long inAmount, long &inFlags)
{
	OTFlags flags = 0;
	
	long amt = fEndpoint->GetOTEndpoint()->Rcv(inPtr,inAmount, &flags);
	
	::SetBits_AC( inFlags, kMore, ::TestBits_AC(flags, T_MORE));
	::SetBits_AC( inFlags, kPriority, ::TestBits_AC(flags, T_PRIORITY));
	
	if(amt < 0)
	{
		if(amt == kOTLookErr)
		{
			long evt = fEndpoint->GetOTEndpoint()->Look();
			amt = 0;
			
#if qNetTrace_AC
			TRACE << LowTracePriority_AC << "OTLook Event: " << COTDebug_AC::GetEventStr(evt) << endl_AC;
#endif
		}
		else
		if(amt == kOTNoDataErr) 
			amt = 0;
		else ::ThrowError_AC(amt);
	}
	
	PostEvent( CreateEventWithData_AC( CNetEvent_AC::kReceivedData, amt ));

}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::Snd
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnection_AC<ADDRESS>::Snd(const char*inPtr, long inAmount, long inFlags)
{
		
	OTFlags flags = 0;
	
	::SetBits_AC( flags, T_MORE, 		::TestBits_AC(inFlags, kMore));
	::SetBits_AC( flags, T_EXPEDITED, 	::TestBits_AC(inFlags, kPriority));
	
	::AssertNotNULL_AC(fEndpoint);
	
	TEndpoint* endpoint = fEndpoint->GetOTEndpoint();
	
	::AssertNotNULL_AC(endpoint);
	
	OTResult state = endpoint->GetEndpointState();
	
	::Assertion_AC(state >= T_DATAXFER, NULL);
	
	char* outData = const_cast_AC(char*,inPtr);
	
	long amt = endpoint->Snd( outData, inAmount, flags);
	
	if(amt < 0)
	{
		switch(amt)
		{
			case kOTLookErr:
			{
				long evt = fEndpoint->GetOTEndpoint()->Look();
				
#if qNetTrace_AC
				TRACE << LowTracePriority_AC << "OTLook Event: " << COTDebug_AC::GetEventStr(evt) << endl_AC;
#endif
			}
			break;
		
			case kOTFlowErr:
				
				// This means we've filled up the buffers and there ain't any 
				// room for anymore data. We've got to wait for a T_GODATA event.
			
				fPendingSndData = inPtr;
				fPendingSndAmt = inAmount;
				fPendingFlags = inFlags;
				
#if qNetTrace_AC
				TRACE << LowTracePriority_AC << "OT Send Buffers full. Now Blocking." << endl_AC;
#endif
			break;
		
			default:
				::ThrowError_AC(amt);
			break;
		}
	}
	else
	{
		if(amt) PostEvent( CreateEventWithData_AC( CNetEvent_AC::kSentData, amt ));
	}
		
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::Hangup
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnection_AC<ADDRESS>::Hangup()
{	
	fDisconnector->Hangup(); 
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::Disconnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnection_AC<ADDRESS>::Disconnect()	
{	
	fDisconnector->Disconnect(); 
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::GetState
//----------------------------------------------------------------------------------------

template <class ADDRESS>
CServiceState_AC* COTConnection_AC<ADDRESS>::GetState() const
{
	return fEndpoint->GetState();
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::GetUnreceivedDataAmount
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long  COTConnection_AC<ADDRESS>::GetUnreceivedDataAmount() const
{
	unsigned long bytes;
	::OTCountDataBytes(fEndpoint->GetOTEndpoint(), &bytes);
	return bytes;				
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::SetMaxConnectionCount
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnection_AC<ADDRESS>::SetMaxConnectionCount(unsigned long max)
{
	fgMaxConnections = max;
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::GetMaxConnectionCount
//----------------------------------------------------------------------------------------

template <class ADDRESS>
unsigned long COTConnection_AC<ADDRESS>::GetMaxConnectionCount()
{
	return fgMaxConnections;
}

//----------------------------------------------------------------------------------------
// COTConnection_AC<ADDRESS>::GetConnectionCount
//----------------------------------------------------------------------------------------

template <class ADDRESS>
unsigned long COTConnection_AC<ADDRESS>::GetConnectionCount()
{
	return fgConnectionCount;
}

#endif