//----------------------------------------------------------------------------------------
// COTConnector_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTConnector_AC__
#include "COTConnector_AC.h"
#endif

#ifndef __COTEndpoint_AC__
#include "COTEndpoint_AC.h"
#endif

#ifndef __COTConnection_AC__
#include "COTConnection_AC.h"
#endif

#ifndef __COTUtils_AC__
#include "COTUtils_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __COTConfig_AC__
#include "COTConfig_AC.h"
#endif


#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif


#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif


//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::COTConnector_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTConnector_AC<ADDRESS>::COTConnector_AC(CReactor_AC* inReactor,CServiceHandler_AC* inServiceHandler)
: 	CServiceConnector_AC(inReactor, inServiceHandler)
{
	Erase_AC(&fLocalAddress);
	Erase_AC(&fRemoteAddress);
	
	fOpener = TH_new COTOpener_AC<ADDRESS>(inReactor);
	fDisconnector = TH_new COTDisconnector_AC<ADDRESS>(inReactor);

	fEndpoint = fOpener->GetEndpoint();
	
	RegisterForEventsFrom(fDisconnector);
	RegisterForEventsFrom(fOpener);
	RegisterForEventsFrom(fEndpoint);
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::~COTConnector_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTConnector_AC<ADDRESS>::~COTConnector_AC() 
{
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::SetRemoteAddress
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnector_AC<ADDRESS>::SetRemoteAddress(const CNetworkAddress_AC& inAddress)
{
	COTUtils_AC::Convert(fRemoteAddress,inAddress);
}
template <class ADDRESS>

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::SetLocalAddress
//----------------------------------------------------------------------------------------

void COTConnector_AC<ADDRESS>::SetLocalAddress(const CNetworkAddress_AC& inAddress)
{
	COTUtils_AC::Convert(fLocalAddress,inAddress);
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::GetRemoteAddress
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnector_AC<ADDRESS>::GetRemoteAddress(CNetworkAddress_AC& inAddress) const
{
	COTUtils_AC::Convert(inAddress, fRemoteAddress);
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::GetLocalAddress
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnector_AC<ADDRESS>::GetLocalAddress(CNetworkAddress_AC& inAddress) const
{
	COTUtils_AC::Convert(inAddress, fLocalAddress);
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::DoConnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnector_AC<ADDRESS>::DoConnect(long inTimeOut)
{

// Set Remote Address	
	fEndpoint->SetRemoteAddress(fRemoteAddress);

// open the disconnector	
	fDisconnector->Open(fEndpoint);
		// listen for unexpected disconnects.
			

	fOpener->Open(COTConfig_AC(COTConfig_AC::kTCP), fLocalAddress);
		// if this throws we're still in an acceptable state.

	if(inTimeOut)
		fEndpoint->StartTimer(inTimeOut);
	
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::ReceiveEvent
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnector_AC<ADDRESS>::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	try
	{
		switch(inEvent->GetEventID())
		{
			case CNetEvent_AC::kTimedOut:
			case CNetEvent_AC::kError:
				PostEvent(TH_new CEvent_AC(CNetEvent_AC::kConnectFailed, inEvent->GetError()));
			break;

			case CNetEvent_AC::kProvider:
				if(inEvent->GetLongData() == T_CONNECT )
				{
					ReceiveEvent_AC(inEvent);
					::ThrowIfError_AC(inEvent->GetError());
					HandleConnect();
				}
			break;
			
			case CNetEvent_AC::kConnectionTerminated:
				PostEvent(inEvent);
			break;
		
			case CNetEvent_AC::kProviderOpen:
				// now we wants events from the endpoint.
				ReceiveEvent_AC(inEvent);
				DoConnect();
			break;
		
			default:
				MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
			break;
		}
	}
	catch(CException_AC& inEx)
	{
		if(fEndpoint) fEndpoint->Close();
		PostEvent(
			TH_new CEvent_AC(CNetEvent_AC::kConnectFailed, inEx.GetError()));
	}	
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::HandleConnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnector_AC<ADDRESS>::HandleConnect()
{
	// any errors thrown from here will be caught in ReceiveEvent
	
	TCall connectInfo;
	::Erase_AC(&connectInfo);

	connectInfo.addr.buf = (unsigned char*) &fEndpoint->GetRemoteAddress();
	connectInfo.addr.len = sizeof(ADDRESS);

	connectInfo.addr.maxlen = sizeof(ADDRESS);

	CError_AC err = fEndpoint->GetOTEndpoint()->RcvConnect(&connectInfo);
	
	
	
	::ThrowIfError_AC(err);
	
	fEndpoint->SetRemoteAddress(fRemoteAddress);
	
	CServiceHandler_AC* connection = GetPendingConnection();
	::AssertNotNULL_AC(connection);
	
	connection->Open();
		// this will create the OT connection
		
	CService_AC* service  =  GetPendingConnection()->GetService();
		// Get the service pointer
		
	COTConnection_AC<ADDRESS>* stream = dynamic_cast_AC(COTConnection_AC<ADDRESS>*,service);
		// convert the service pointer to our OT subclass
			
	stream->Open(fEndpoint);	
		// open the OT service, hand it the endpoints Reactor so they operate in 
		// the same thread of execution.	
	
	PostEvent( 
		TH_new CEvent_AC(CNetEvent_AC::kConnectSucceeded));
		// Notify our handlers that the connection is ready to rock!
		// This sends an event to the CConnector_AC, which pulls our
		// connection from this.
}

//----------------------------------------------------------------------------------------
// COTConnector_AC<ADDRESS>::DoConnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTConnector_AC<ADDRESS>::DoConnect()
{
	// any errors thrown from here will be caught in ReceiveEvent

	TCall inCall, outCall;

	::Erase_AC(&inCall);
	::Erase_AC(&outCall);
	
	inCall.addr.len = sizeof(ADDRESS);
	inCall.addr.buf = (unsigned char*) &fEndpoint->GetRemoteAddress();

	ADDRESS out;
		
		
	::Erase_AC(&out);

	outCall.addr.maxlen = sizeof(ADDRESS);
	outCall.addr.buf = (unsigned char*) &out;
	
	CError_AC err = fEndpoint->GetOTEndpoint()->Connect(&inCall, &outCall);
	
	
	
	if(err == kOTNoDataErr) err = 0;
		
	::ThrowIfError_AC(err);
		// propagate the error
}

#endif