//----------------------------------------------------------------------------------------
// COTDisconnector_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTDisconnector_AC__
#include "COTDisconnector_AC.h"
#endif

#ifndef __COTEndpoint_AC__
#include "COTEndpoint_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif


//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::COTDisconnector_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTDisconnector_AC<ADDRESS>::COTDisconnector_AC(CReactor_AC* inReactor)
:	MEventTransceiver_AC(inReactor),
	fSentOrdrel(false)
{
}

//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::~COTDisconnector_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTDisconnector_AC<ADDRESS>::~COTDisconnector_AC()
{
}

//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::Open
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTDisconnector_AC<ADDRESS>::Open(COTEndpoint_AC<ADDRESS>* inEndpoint)
{
	::AssertNotNULL_AC(inEndpoint);
	SetReactor(inEndpoint->GetReactor());
	fEndpoint = inEndpoint;
	RegisterForEventsFrom(inEndpoint);
}

//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::ReceiveEvent
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTDisconnector_AC<ADDRESS>::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kError:
			PostEvent(inEvent);
		break;

		case CNetEvent_AC::kProvider:
		{
			switch(inEvent->GetLongData())
			{
				case T_DISCONNECT:
					ReceiveEvent_AC(inEvent);
					::ThrowIfError_AC(inEvent->GetError());
					HandleDISCONNECT();
				break;
				
				case T_ORDREL:
					ReceiveEvent_AC(inEvent);
					::ThrowIfError_AC(inEvent->GetError());
					HandleORDREL();
				break;
			}
		}
		break;

		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
		break;

	}
}

//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::HandleDISCONNECT
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTDisconnector_AC<ADDRESS>::HandleDISCONNECT()
{
	PostEvent( 
		CreateEventWithData_AC(CNetEvent_AC::kConnectionTerminated, (long) CNetEvent_AC::kRcvdHangup));
}

//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::HandleORDREL
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTDisconnector_AC<ADDRESS>::HandleORDREL()
{
	CError_AC err = fEndpoint->GetOTEndpoint()->RcvOrderlyDisconnect();

	::ThrowIfError_AC(err);

	if( err == kOTLookErr)
	{
		// if we get a look error here we'll soon
		// be getting a T_DISCONNECT in our event handler.
	}
	else
	{
		if(!fSentOrdrel)
		{
			err = fEndpoint->GetOTEndpoint()->SndOrderlyDisconnect();
			::ThrowIfError_AC(err);
			
			PostEvent(
				CreateEventWithData_AC(CNetEvent_AC::kConnectionTerminated, (long) CNetEvent_AC::kRcvdDisconnect ));
		}
		else
		{
			fSentOrdrel = false;
			
			PostEvent(
				CreateEventWithData_AC(CNetEvent_AC::kConnectionTerminated, (long) CNetEvent_AC::kDisconnectComplete));

		}

		if( err == kOTLookErr)
		{
			// if we get a look error here we'll soon
			// be getting a T_DISCONNECT in our event handler.
		}
		else
		{				
			
			
	//		if(!GetState()->GetKeepEndpointOpen())
	//			DoClose();
		}
	}
}


//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::Disconnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTDisconnector_AC<ADDRESS>::Disconnect()
{
	if(fEndpoint->GetOTEndpoint()->GetEndpointState() > T_IDLE)
	{
		fSentOrdrel = true;
		CError_AC err = fEndpoint->GetOTEndpoint()->SndOrderlyDisconnect();
		::ThrowIfError_AC(err);
	}
	else
	{
		PostEvent(
			CreateEventWithData_AC( CNetEvent_AC::kConnectionTerminated, (long) CNetEvent_AC::kDisconnectComplete));
	}	
}

//----------------------------------------------------------------------------------------
// COTDisconnector_AC<ADDRESS>::Hangup
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTDisconnector_AC<ADDRESS>::Hangup()
{
	if(fEndpoint->GetOTEndpoint()->GetEndpointState() > T_IDLE)
	{
		TCall info;
		::Erase_AC(&info);
		
		info.addr.len = sizeof(ADDRESS);
		info.addr.buf = (unsigned char*) &fEndpoint->GetRemoteAddress();
		
		CError_AC err = fEndpoint->GetOTEndpoint()->SndDisconnect(&info);
		
		::ThrowIfError_AC(err);
	}	
	
	PostEvent(
		CreateEventWithData_AC((long) CNetEvent_AC::kHangupComplete, CNetEvent_AC::kConnectionTerminated));
}
#endif