//----------------------------------------------------------------------------------------
// COTEndpoint_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTEndpoint_AC__
#include "COTEndpoint_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif


#ifndef __OTDebug_AC__
#include "OTDebug_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CProvider_AC__
#include "CProvider_AC.h"
#endif

#ifndef __COTServiceState_AC__
#include "COTServiceState_AC.h"
#endif


template <class  ADDRESS>
long COTEndpoint_AC<ADDRESS>::fgCounter = 0;

//----------------------------------------------------------------------------------------
// COTEndpoint_AC<ADDRESS>::COTEndpoint_AC
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
COTEndpoint_AC<ADDRESS>::COTEndpoint_AC(CReactor_AC* inReactor)
:	MEventTransceiver_AC(inReactor),
	fEndpoint(NULL),
	fSavedCookie(NULL),
	fID(++fgCounter)
{
	::Erase_AC(&fInfo);
	::Erase_AC(&fLocalAddress);
	::Erase_AC(&fRemoteAddress);
	
	GetIdleTimeFrom(GetReactor());
}

//----------------------------------------------------------------------------------------
// COTEndpoint_AC<ADDRESS>::~COTEndpoint_AC
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
COTEndpoint_AC<ADDRESS>::~COTEndpoint_AC()
{
	if(fTimer)
		fTimer->Stop();
	Close();
}

//----------------------------------------------------------------------------------------
// COTEndpoint_AC<ADDRESS>::Close
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
void COTEndpoint_AC<ADDRESS>::Close()
{
	if(fEndpoint) 
	{
		fEndpoint->Close();
		fEndpoint = NULL;
	}
}

//----------------------------------------------------------------------------------------
// COTEndpoint_AC<ADDRESS>::Idle
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
void COTEndpoint_AC<ADDRESS>::Idle()
{	
	if(fEventStream.HasData())
	{			
		long result, error, cookie;
		
		fEventStream >> result;
		fEventStream >> error;
		fEventStream >> cookie;

		fSavedCookie = reinterpret_cast<void*>(cookie);

		
		// Caching the cookie is lame.

#if qNetTrace_AC

		TRACE 	<< LowTracePriority_AC 
				<< "\n OT CEndpoint #"
				<< GetID() 
				<< " Received Async Event: " 
				<< COTDebug_AC::GetEventStr(result)
				<< "\n"
				<< endl_AC;

		if(error)
		{
			TRACE 	<< LowTracePriority_AC 
					<< "**** "
					<< this
					<< ": Open Transport Error From Callback: "
					<< COTDebug_AC::GetOTErrorStr(error)
					<< " ("
					<< error
					<< ")"
					<< endl_AC;
		}

#endif
		

		PostEvent( 
			CreateEventWithData_AC(CNetEvent_AC::kProvider, result, error));
			// Send the event.
			
		fSavedCookie = NULL;
	}
}

//----------------------------------------------------------------------------------------
// COTEndpoint_AC<ADDRESS>::OTCallback
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
pascal void COTEndpoint_AC<ADDRESS>::OTCallback(
								COTEndpoint_AC* endpoint, 
								unsigned long inResult, 
								CError_AC inError, 
								void* inCookie)
{
	try
	{
		if(endpoint)
		{
			endpoint->fEventStream << inResult;
			endpoint->fEventStream << inError;
			endpoint->fEventStream << static_cast_AC(long, inCookie);
		}
		else 
			DebugStr("\pSOTEndpoint:Event not handled!;g");
	}
	catch( CException_AC& ex)
	{
#if qNetDebug_AC
		DebugStr("\pSOTEndpoint:Streaming error.;g");
#endif	
	}
	catch(...)
	{
#if qNetDebug_AC
		DebugStr("\pSOTEndpoint:Unknown exception;g");
#endif	
	}

}

//----------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
OTNotifyProcPtr COTEndpoint_AC<ADDRESS>::GetNotifyProcPtr() const
{
	return reinterpret_cast<OTNotifyProcPtr>(OTCallback);
}

//----------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
void COTEndpoint_AC<ADDRESS>::SetRemoteAddress(const ADDRESS& inAddress)
{
	::Copy_AC(&fRemoteAddress, &inAddress);
}

//----------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
void COTEndpoint_AC<ADDRESS>::SetLocalAddress(const ADDRESS& inAddress)
{
	::Copy_AC(&fLocalAddress, &inAddress);
}

//----------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
void COTEndpoint_AC<ADDRESS>::StartTimer(long interval)
{
	if(!fTimer)
	{
		fTimer = TH_new CEventTransceiverTimer_AC(this);
		fTimer->GetIdleTimeFrom(GetReactor());
	}
	fTimer->SetDuration(interval);
	fTimer->Start();
}

//----------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------

template <class  ADDRESS>
void COTEndpoint_AC<ADDRESS>::StopTimer()
{
	if(fTimer) 
		fTimer->Stop();
}

//----------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------

template <class ADDRESS>
CServiceState_AC* COTEndpoint_AC<ADDRESS>::GetState()
{
	if(!fState)
	{
		fState = TH_new COTServiceState_AC<ADDRESS>(this);
	}
	
	return fState;
}


#endif