//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __COTInternetHostRep_AC__
#include "COTInternetHostRep_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __CInternetHost_AC__
#include "CInternetHost_AC.h"
#endif

#ifndef __CProvider_AC__
#include "CProvider_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif


#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

#ifndef __OTDebug_AC__
#include "OTDebug_AC.h"
#endif

#ifndef __Errors__
#include <Errors.h>
#endif


#define kStreamSize 48
	// this is room for four 12 byte events. Should be more than enough. 

//----------------------------------------------------------------------------------------
// class COTHostlist. 
// This is just something that holds the list of hosts
// so that we can iterate over the list when we get an event. We need to 
// send a pointer to something that'll always be around into OTAsyncOpenInternetServices.
// We'll get this pointer back into the callback when we get an event.
//----------------------------------------------------------------------------------------

class COTHostList_AC
{
public:
	COTHostList_AC() : fHosts(NULL) {}
	
	COTInternetHostRep_AC* GetFirstHost() const
	{	return fHosts; }

	void Insert(COTInternetHostRep_AC* inHost)
	{	fHosts = inHost->Append(fHosts); }
	
	void Remove(COTInternetHostRep_AC* inHost)
	{	fHosts = inHost->Remove(fHosts); }
	
private:
	COTInternetHostRep_AC* fHosts;
};

static COTHostList_AC gHostList;

TInternetServices*	COTInternetHostRep_AC::fgRef = NULL;
CError_AC			COTInternetHostRep_AC::fgStartupError = 0;


//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::COTInternetHostRep_AC
//----------------------------------------------------------------------------------------

COTInternetHostRep_AC::COTInternetHostRep_AC(CInternetHost_AC* host)
	: CInternetHostRep_AC(host),
	fNumHosts(0),
	fEventStream(kStreamSize)
{
	::Erase_AC(&fHostInfo);
	
	gHostList.Insert(this);
	
	OpenInternetServices();
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::~COTInternetHostRep_AC
//----------------------------------------------------------------------------------------

COTInternetHostRep_AC::~COTInternetHostRep_AC()
{
	if(fTimer) 
		fTimer->Stop();
	gHostList.Remove(this);
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::OpenInternetServices
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::OpenInternetServices()
{
	static bool isOpen = false;
	
	if(!isOpen)
	{
		CProvider_AC::Open();
	
		ThrowIfError_AC(
			::OTAsyncOpenInternetServices( 	kDefaultInternetServicesPath,
											0,
											reinterpret_cast<OTNotifyProcPtr>(OTCallback),
											&gHostList));
	
		isOpen = true;
	}
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::SetAddress
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::SetAddress(const CInternetAddress_AC& inAddr, long id)
{
	if(id >= kMaxHostAddrs)
		ThrowError_AC(paramErr);

	if(id >= fNumHosts) 
		fNumHosts = id + 1;
		
	fHostInfo.addrs[id] = inAddr;
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::SetAddresses
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::SetAddresses(CInternetHostID_AC* inAddr, long numAddrs)
{
	if(numAddrs > kMaxHostAddrs)
		ThrowError_AC(paramErr);

	for(short i = 0; i < numAddrs; i++)
	{
		fHostInfo.addrs[i] = inAddr[i];
	}

	fNumHosts = numAddrs;
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::AddAddress
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::AddAddress(const CInternetAddress_AC& inAddress)
{
	SetAddress(inAddress, fNumHosts);
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::GetNumAddresses
//----------------------------------------------------------------------------------------

long COTInternetHostRep_AC::GetNumAddresses() const
{
	return fNumHosts;
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::GetAddress
//----------------------------------------------------------------------------------------

CInternetAddress_AC COTInternetHostRep_AC::GetAddress(long id)
{
	if(id < 0 || id >= fNumHosts)
		ThrowError_AC(paramErr);

	return CInternetAddress_AC(fHostInfo.addrs[id], 0); 
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::OTCallback
//----------------------------------------------------------------------------------------

pascal void COTInternetHostRep_AC::OTCallback(
	COTHostList_AC* inList, 
	CEventID_AC inEventID, CError_AC inError, void* inCookie)
{
	try
	{
		if(inEventID == T_OPENCOMPLETE)
		{
			fgStartupError = inError;
			if(!fgStartupError)
			{
				fgRef = reinterpret_cast<TInternetServices*>(inCookie);
			}
#if qNetDebug			
			else
				DebugStr("\pCOTInternetHostRep_AC::OTCallback Got error on startup;g");
#endif
		}
	
		COTInternetHostRep_AC* targetHost = NULL;
		COTInternetHostRep_AC* hostWalker = inList->GetFirstHost();
		while(hostWalker && !targetHost)
		{
			switch(inEventID)
			{
				case T_OPENCOMPLETE:
					targetHost = hostWalker;
				break;
			
				case T_DNRADDRTONAMECOMPLETE:
					if( hostWalker->fHostInfo.name == reinterpret_cast<char*>(inCookie))
						targetHost = hostWalker;
				break;
			
				case T_DNRSTRINGTOADDRCOMPLETE:
					if(	&(hostWalker->fHostInfo) == reinterpret_cast<InetHostInfo*>(inCookie))
						targetHost = hostWalker;
				break;
			}
			
			hostWalker = hostWalker->GetNextLink();
		}
	
		if(targetHost)
		{
			targetHost->fEventStream << inEventID;
			targetHost->fEventStream << inError;
			targetHost->fEventStream << (long) inCookie;
		}
	}
	catch(...)
	{
#if qNetDebug_AC
		DebugStr("\pCOTInternetHostRep_AC::OTCallback -> Unknown exception");
#endif	
	}
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::ResolveNameToAddressAsync
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::ResolveNameToAddressAsync()
{
	fHost->UseDefaultReactor();
	GetIdleTimeFrom(fHost->GetReactor());
	
	if(!fgRef)
	{
		fHost->ChangeStateTo( CInternetHost_AC::kStartingNameToAddress );
	}
	else
	{
	
		fNumHosts = 0;
		::Erase_AC(fHostInfo.addrs, kMaxHostAddrs);
	
		StartTimer();
	
		::ThrowIfError_AC( 
			fgRef->StringToAddress( fHostInfo.name, &fHostInfo));
		
		fHost->ChangeStateTo( CInternetHost_AC::kPendingNameToAddress );	
	
	}
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::ResolveAddressToNameAsync
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::ResolveAddressToNameAsync()
{
	fHost->UseDefaultReactor();
	GetIdleTimeFrom(fHost->GetReactor());

	if(!fgRef)
	{
		fHost->ChangeStateTo( CInternetHost_AC::kStartingAddressToName );
	}
	else
	{
		fHostInfo.name[0] = '\0';
	
		StartTimer();
	
		::ThrowIfError_AC( 
			fgRef->AddressToName(fHostInfo.addrs[0], fHostInfo.name ));
			
		fHost->ChangeStateTo( CInternetHost_AC::kPendingAddressToName );
	}
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::Idle
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::Idle()
{	
	if(fTimer && fTimer->TimedOut())
	{
		if(fHost->GetState() == CInternetHost_AC::kPendingNameToAddress)
		{
			CompleteNameToAddress(0, true);
		}
		else
		{
			CompleteAddressToName(0, true);
		}
	}
	else
	if(fEventStream.HasData())
	{			
		long result, error, cookie;
		
		fEventStream >> result;
		fEventStream >> error;
		fEventStream >> cookie;		
		
#if qNetTrace_AC

		TRACE 	<< LowTracePriority_AC 
				<< "\n COTInternetService_AC:"
				<< " Received Async Event: " 
				<< COTDebug_AC::GetEventStr(result)
				<< "\n"
				<< endl_AC;

		if(error)
		{
			TRACE 	<< LowTracePriority_AC 
					<< "**** COTInternetService_AC"
					<< ": Open Transport Error From Callback: "
					<< COTDebug_AC::GetOTErrorStr(error)
					<< " ("
					<< error
					<< ")"
					<< endl_AC;
		}

#endif

		switch(result)
		{
		
			case T_OPENCOMPLETE:
				COTInternetHostRep_AC* hostWalker = gHostList.GetFirstHost();
				while(hostWalker)
				{
					switch(hostWalker->fHost->GetState())
					{	
						case CInternetHost_AC::kStartingNameToAddress:
							hostWalker->ResolveNameToAddressAsync();
							break;
						case CInternetHost_AC::kStartingAddressToName:
							hostWalker->ResolveAddressToNameAsync();
							break;
					}

					hostWalker = hostWalker->GetNextLink();
				}
			break;
		
			case T_DNRADDRTONAMECOMPLETE:
				if(!fTimer->TimedOut())
					CompleteAddressToName(error);
			break;
		
			case T_DNRSTRINGTOADDRCOMPLETE:
				if(!fTimer->TimedOut())
				{
					fNumHosts = 0;
					
					for(short i = 0; i < kMaxHostAddrs; i++)
					{
						if(fHostInfo.addrs[i] != 0)
							fNumHosts++;
					}
					CompleteNameToAddress(error);
				}
			break;
		
		}
	}
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::GetHostName
//----------------------------------------------------------------------------------------

const char* COTInternetHostRep_AC::GetHostName() const
{	
	return fHostInfo.name; 
}

//----------------------------------------------------------------------------------------
// COTInternetHostRep_AC::SetHostName
//----------------------------------------------------------------------------------------

void COTInternetHostRep_AC::SetHostName(const char* inHostName)
{	
	Assertion_AC( ::strlen(inHostName) < kMaxHostNameLen + 1, "Hostname too long");
	::strcpy(fHostInfo.name, inHostName);
}

#endif