
//----------------------------------------------------------------------------------------
// COTOpener_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

/*
	Theory of operation:
	
	It this fails during endpoint opening, endpoint is closed.
*/

#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTOpener_AC__
#include "COTOpener_AC.h"
#endif

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __COTEndpoint_AC__
#include "COTEndpoint_AC.h"
#endif

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

#ifndef __COTConfig_AC__
#include "COTConfig_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#if qNetDebug
	#ifndef __dout_AC__
	#include "dout_AC.h"
	#endif
#endif


//----------------------------------------------------------------------------------------
// COTOpener_AC<ADDRESS>::COTOpener_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTOpener_AC<ADDRESS>::COTOpener_AC(CReactor_AC* inReactor)
:	MEventTransceiver_AC(inReactor),
	fDoBind(true)
{
	::Erase_AC(&fBindTo);
	fEndpoint = TH_new COTEndpoint_AC<ADDRESS>(inReactor);
	RegisterForEventsFrom(fEndpoint);
}

//----------------------------------------------------------------------------------------
// COTOpener_AC<ADDRESS>::~COTOpener_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTOpener_AC<ADDRESS>::~COTOpener_AC()
{
}

//----------------------------------------------------------------------------------------
// COTOpener_AC<ADDRESS>::Open
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTOpener_AC<ADDRESS>::Open(
	const COTConfig_AC& inConfig,
	const ADDRESS& toBindTo, 
	bool doBind)
{
	fDoBind = doBind;
	
	::Copy_AC(&fBindTo, &toBindTo);
		// save address - assuming this is a raw data type not an
		// object.
	
	COTConfig_AC nonConstConfig = inConfig;
		
	CError_AC err = ::OTAsyncOpenEndpoint( 	
								nonConstConfig.GetClonedOTConfig(),
								nonConstConfig.GetOpenFlags(),
				 				fEndpoint->GetOTEndpointInfo(),
				 				fEndpoint->GetNotifyProcPtr(), 						
								reinterpret_cast<void*>(fEndpoint.get()));
								
	::ThrowIfError_AC( err);
}

//----------------------------------------------------------------------------------------
// COTOpener_AC<ADDRESS>::ReceiveEvent
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTOpener_AC<ADDRESS>::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kError:
			HandleOpenFailed(inEvent);
		break;
	
		case CNetEvent_AC::kProvider:
			try
			{
				switch(inEvent->GetLongData())
				{
					case T_OPENCOMPLETE:
					{
						ReceiveEvent_AC(inEvent);
						inEvent->Consume();
						::ThrowIfError_AC(inEvent->GetError());
						
						TEndpoint* otEndpoint = reinterpret_cast<TEndpoint*>(fEndpoint->GetSavedCookie());
							// Get Saved cookie.
							
						fEndpoint->SetOTEndpoint( otEndpoint );
						
						StartBind();
					}
					break;
					
					case T_BINDCOMPLETE:
						ReceiveEvent_AC(inEvent);

						inEvent->Consume();
						::ThrowIfError_AC(inEvent->GetError());
					
						HandleOpenComplete();
					break;
				}
			}
			catch(CException_AC& inEx)
			{
				HandleOpenFailed(TH_new CEvent_AC(CNetEvent_AC::kError, inEx.GetError()));
			}		
		break;

		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
		break;

	}
}

//----------------------------------------------------------------------------------------
//  COTOpener_AC<ADDRESS>::HandleOpenFailed
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTOpener_AC<ADDRESS>::HandleOpenFailed(CEvent_AC* inEvent)
{
	if(fEndpoint && fEndpoint->IsOpen()) fEndpoint->Close();

#if qNetTrace_AC
	TRACE << LowTracePriority_AC << "Opened Failed:" << inEvent->GetError() << endl_AC;
#endif

	PostEvent( inEvent );
}

//----------------------------------------------------------------------------------------
// COTOpener_AC<ADDRESS>::GetEndpoint
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTEndpoint_AC<ADDRESS>* COTOpener_AC<ADDRESS>::GetEndpoint() const 
{ 
	return fEndpoint; 
}

//----------------------------------------------------------------------------------------
// COTOpener_AC<ADDRESS>::StartBind
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTOpener_AC<ADDRESS>::StartBind()
{
	::Erase_AC(&fEndpoint->GetLocalAddress());
		// don't want an old address hanging around
	
	if(!fDoBind)
	{
		HandleOpenComplete();
		// local address will be zero.
	}
	else
	{
	
		TBind inBind, outBind; 

		::Erase_AC(&inBind);
		::Erase_AC(&outBind);
		
		inBind.addr.len = sizeof(struct ADDRESS);
		inBind.addr.buf = (unsigned char *) &fBindTo;
		inBind.qlen = 0xFFFFFFFF; 
			// queue up the max num connection requests OT can handle
		
		outBind.addr.maxlen = sizeof(struct ADDRESS);
		outBind.addr.buf = (unsigned char *) &fEndpoint->GetLocalAddress();

		short err = fEndpoint->GetOTEndpoint()->Bind(&inBind, &outBind);
			// when bind completes, the local address will reflect the 
			// address the endpoint is bound to.

		::ThrowIfError_AC(err);
	}
}

//----------------------------------------------------------------------------------------
// COTOpener_AC<ADDRESS>::HandleOpenComplete
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTOpener_AC<ADDRESS>::HandleOpenComplete()
{
//	UnregisterForAllEvents();
		// don't need to get events any longer.
		
	PostEvent( 
		TH_new CEvent_AC(CNetEvent_AC::kProviderOpen));
}



//-----------------------


#endif