//----------------------------------------------------------------------------------------
// COTInteState_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#ifndef __COTServiceState_AC__
#include "COTServiceState_AC.h"
#endif

#ifndef __COTUtils_AC__
#include "COTUtils_AC.h"
#endif

#ifndef __COTEndpoint_AC__
#include "COTEndpoint_AC.h"
#endif


//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::COTServiceState_AC(
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTServiceState_AC<ADDRESS>::COTServiceState_AC(COTEndpoint_AC<ADDRESS>* inEndpoint)
	: fEndpoint(inEndpoint)
{
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::~COTServiceState_AC
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTServiceState_AC<ADDRESS>::~COTServiceState_AC()
{
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetServiceState
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTServiceState_AC<ADDRESS>::GetServiceState()
{
	// for now OT state maps to our generic state.

	::AssertNotNULL_AC(fEndpoint);

	return fEndpoint->GetOTEndpoint()->GetEndpointState();
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::IsConnectionOriented
//----------------------------------------------------------------------------------------

template <class ADDRESS>
CServiceState_AC::eStateBool COTServiceState_AC<ADDRESS>::IsConnectionOriented()
{
	::AssertNotNULL_AC(fEndpoint);
	long servType = fEndpoint->GetOTEndpointInfo()->servtype;
	
	return (servType != T_CLTS && servType != T_TRANS_CLTS) ? kTrue : kFalse;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::IsTransactionBased
//----------------------------------------------------------------------------------------

template <class ADDRESS>
CServiceState_AC::eStateBool COTServiceState_AC<ADDRESS>::IsTransactionBased()
{
	::AssertNotNULL_AC(fEndpoint);
	long servType = fEndpoint->GetOTEndpointInfo()->servtype;
	
	return (servType == T_TRANS || servType == T_TRANS_CLTS) ? kTrue : kFalse;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::SupportsOrderlyDisconnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
CServiceState_AC::eStateBool COTServiceState_AC<ADDRESS>::SupportsOrderlyDisconnect()
{
	::AssertNotNULL_AC(fEndpoint);
	long servType = fEndpoint->GetOTEndpointInfo()->servtype;
	
	return (servType == T_COTS_ORD || servType == T_TRANS_ORD) ? kTrue : kFalse;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::SupportsZeroLengthTSDU
//----------------------------------------------------------------------------------------

template <class ADDRESS>
CServiceState_AC::eStateBool COTServiceState_AC<ADDRESS>::SupportsZeroLengthTSDU()
{
	::AssertNotNULL_AC(fEndpoint);
	long flags = fEndpoint->GetOTEndpointInfo()->flags;
	
	return (flags & T_SENDZERO) ? kTrue : kFalse;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetMaxSizeOfOptions
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTServiceState_AC<ADDRESS>::GetMaxSizeOfOptions()
{
	::AssertNotNULL_AC(fEndpoint);
	return fEndpoint->GetOTEndpointInfo()->options;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetTSDUSize
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTServiceState_AC<ADDRESS>::GetTSDUSize()
{
	::AssertNotNULL_AC(fEndpoint);
	return fEndpoint->GetOTEndpointInfo()->tsdu;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetExpeditedTSDUSize
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTServiceState_AC<ADDRESS>::GetExpeditedTSDUSize()
{
	::AssertNotNULL_AC(fEndpoint);
	return fEndpoint->GetOTEndpointInfo()->etsdu;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetMaxDataSizeOnConnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTServiceState_AC<ADDRESS>::GetMaxDataSizeOnConnect() 
{
	::AssertNotNULL_AC(fEndpoint);
	return fEndpoint->GetOTEndpointInfo()->connect;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetMaxDataSizeOnDisconnect
//----------------------------------------------------------------------------------------

template <class ADDRESS>
long COTServiceState_AC<ADDRESS>::GetMaxDataSizeOnDisconnect()
{
	::AssertNotNULL_AC(fEndpoint);
	return fEndpoint->GetOTEndpointInfo()->discon;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetRemoteAddress
//----------------------------------------------------------------------------------------

template <class ADDRESS>
void COTServiceState_AC<ADDRESS>::GetRemoteAddress(CNetworkAddress_AC& inAddress)
{
	CInternetAddress_AC out;
	COTUtils_AC::Convert( out, fEndpoint->GetRemoteAddress());
	inAddress.SetWithString(out);
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetEndpoint
//----------------------------------------------------------------------------------------

template <class ADDRESS>
COTEndpoint_AC<InetAddress>* COTServiceState_AC<ADDRESS>::GetEndpoint()
{
	return fEndpoint;
}

//----------------------------------------------------------------------------------------
// COTServiceState_AC<ADDRESS>::GetAddressType
//----------------------------------------------------------------------------------------

template <class ADDRESS>
unsigned long COTServiceState_AC<ADDRESS>::GetAddressType()
{
	// for now just internet.

	unsigned long out = '????';
	
	switch(fEndpoint->GetLocalAddress().fAddressType)
	{
		case AF_INET:
			out = kInternet;
		break;
	
	}

	return out;
}
#endif