//----------------------------------------------------------------------------------------
// COTDebug_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __Net_AC__
#include "Net_AC.h"
#endif


#if qWantsOpenTransport_AC

#if qNetDebug_AC

#ifndef __stdio__
#include <stdio.h>
#endif


#ifndef __OPENTRANSPORT__
#	ifndef __OpenTransport__
#	include <OpenTransport.h>
#	endif

#endif

#ifndef __OPENTPTINTERNET__
#	ifndef __OpenTptInternet__
#	include <OpenTptInternet.h>
#	endif
#endif

#ifndef __OTDebug_AC__
#include "OTDebug_AC.h"
#endif


//----------------------------------------------------------------------------------------
// COTDebug_AC::GetEventStr
//----------------------------------------------------------------------------------------

const char* COTDebug_AC::GetEventStr(long inID)
{
	static char str[256];
	static const TLookupTable kTable[] = 
	{
		{ "T_OPENCOMPLETE", T_OPENCOMPLETE},
		{ "T_BINDCOMPLETE", T_BINDCOMPLETE},
		{ "T_UNBINDCOMPLETE", T_UNBINDCOMPLETE},
		{ "T_CONNECT", T_CONNECT},
		{ "T_DISCONNECT", T_DISCONNECT},
		{ "T_DISCONNECTCOMPLETE", T_DISCONNECTCOMPLETE},
		{ "T_LISTEN", T_LISTEN},
		{ "T_PASSCON", T_PASSCON},
		{ "T_ACCEPTCOMPLETE", T_ACCEPTCOMPLETE},
		{ "T_ORDREL", T_ORDREL},
		{ "T_EXDATA", T_EXDATA},
		{ "T_DATA", T_DATA},
		{ "T_GODATA",T_GODATA},
		{ "T_GOEXDATA", T_GOEXDATA},
		{ "T_DNRADDRTONAMECOMPLETE", T_DNRADDRTONAMECOMPLETE},
		{ "T_DNRSTRINGTOADDRCOMPLETE", T_DNRSTRINGTOADDRCOMPLETE},
		{ 0, 0 },
	};	

	const char* lookup = TableLookup(kTable, inID );
	
	if(lookup[0] == '?')
	{
		char lilStr[5];
		lilStr[4] = '\0';
		
		sprintf(str, "%s/0x%p/%ld", OSTypeToStr(inID, lilStr), inID, inID);
		lookup = str;
	}

	return lookup;
}

//----------------------------------------------------------------------------------------
// COTDebug_AC::GetOTErrorStr(
//----------------------------------------------------------------------------------------

const char *COTDebug_AC::GetOTErrorStr(long inErr)
{
	static const TLookupTable kTable[] = 
	{
		{"kOTOutOfMemoryErr", kOTOutOfMemoryErr},
		{"kOTNotFoundErr", kOTNotFoundErr},
		{"kOTDuplicateFoundErr", kOTDuplicateFoundErr},
		/*
		* Remapped XTI error codes
		*/
		{"kOTBadAddressErr", kOTBadAddressErr},		/* A Bad address was specified				*/
		{"kOTBadOptionErr", kOTBadOptionErr},		/* A Bad option was specified				*/
		{"kOTAccessErr", kOTAccessErr},			/* Missing access permission				*/
		{"kOTBadReferenceErr", kOTBadReferenceErr},			/* Bad provider reference					*/
		{"kOTNoAddressErr", kOTNoAddressErr},		/* No address was specified					*/
		{"kOTOutStateErr", kOTOutStateErr},		/* Call issued in wrong state				*/
		{"kOTBadSequenceErr", kOTBadSequenceErr},		/* Sequence specified does not exist		*/
		{"kOTSysErrorErr", kOTSysErrorErr},		/* A system error occurred					*/
		{"kOTLookErr", kOTLookErr},			/* An event occurred - call Look{}			*/
		{"kOTBadDataErr", kOTBadDataErr},			/* An illegal amount of data was specified	*/
		{"kOTBufferOverflowErr", kOTBufferOverflowErr},		/* Passed buffer not big enough				*/
		{"kOTFlowErr", kOTFlowErr},			/* Provider is flow-controlled				*/
		{"kOTNoDataErr", kOTNoDataErr},		/* No data available for reading			*/
		{"kOTNoDisconnectErr", kOTNoDisconnectErr},			/* No disconnect indication available		*/
		{"kOTNoUDErrErr", kOTNoUDErrErr},		/* No Unit Data Error indication available	*/
		{"kOTBadFlagErr", kOTBadFlagErr},		/* A Bad flag value was supplied			*/
		{"kOTNoReleaseErr", kOTNoReleaseErr},			/* No orderly release indication available	*/
		{"kOTNotSupportedErr", kOTNotSupportedErr},	/* Command is not supported					*/
		{"kOTStateChangeErr", kOTStateChangeErr},		/* State is changing - try again later		*/
		{"kOTNoStructureTypeErr", kOTNoStructureTypeErr},	/* Bad structure type requested for OTAlloc	*/
		{"kOTBadNameErr", kOTBadNameErr},		/* A bad endpoint name was supplied			*/
		{"kOTBadQLenErr", kOTBadQLenErr},		/* A Bind to an in-use addr with qlen > 0	*/
		{"kOTAddressBusyErr", kOTAddressBusyErr},		/* CAddress requested is already in use		*/
		{"kOTIndOutErr", kOTIndOutErr},		/* Accept failed because of pending listen	*/
		{"kOTProviderMismatchErr", kOTProviderMismatchErr},	/* Tried to accept on incompatible endpoint	*/
		{"kOTResQLenErr", kOTResQLenErr},
		{"kOTResAddressErr", kOTResAddressErr},
		{"kOTQFullErr", kOTQFullErr},
		{"kOTProtocolErr", kOTProtocolErr},			/* An unspecified provider error occurred	*/
		{"kOTBadSyncErr", kOTBadSyncErr},		/* A synchronous call at interrupt time		*/
		{"kOTCanceledErr", kOTCanceledErr},		/* The command was cancelled				*/
		/*
		* Remapped StdCLib error codes. %%% Remove ones we don't actually return.
		*/
		{"kEPERMErr", kEPERMErr},			/* Permission denied					*/
		{"kENOENTErr", kENOENTErr},			/* No such file or directory			*/
		{"kENORSRCErr", kENORSRCErr},			/* No such resource						*/
		{"kEINTRErr", kEINTRErr},			/* Interrupted system service			*/
		{"kEIOErr", kEIOErr},				/* I/O error							*/
		{"kENXIOErr", kENXIOErr},			/* No such device or address			*/
		{"kEBADFErr", kEBADFErr},			/* Bad file number						*/
		{"kEAGAINErr", kEAGAINErr},			/* Try operation again later			*/
		{"kENOMEMErr", kENOMEMErr},			/* Not enough space						*/
		{"kEACCESErr", kEACCESErr},			/* Permission denied					*/
		{"kEFAULTErr", kEFAULTErr},			/* Bad address							*/
		{"kEBUSYErr", kEBUSYErr},			/* Device or resource busy				*/
		{"kEEXISTErr", kEEXISTErr},			/* File exists							*/
		{"kENODEVErr", kENODEVErr},			/* No such device						*/
		{"kEINVALErr", kEINVALErr},			/* Invalid argument						*/
		{"kENOTTYErr", kENOTTYErr},			/* Not a character device				*/
		{"kEPIPEErr", kEPIPEErr},			/* Broken pipe							*/
		{"kERANGEErr", kERANGEErr},			/* Message size too large for STREAM	*/
		{"kEWOULDBLOCKErr", kEWOULDBLOCKErr},		/* Call would block, so was aborted		*/
		{"kEDEADLKErr", kEDEADLKErr},			/* or a deadlock would occur			*/
		{"kEALREADYErr", kEALREADYErr},
		{"kENOTSOCKErr", kENOTSOCKErr},			/* Socket operation on non-socket		*/
		{"kEDESTADDRREQErr", kEDESTADDRREQErr},		/* Destination address required			*/
		{"kEMSGSIZEErr", kEMSGSIZEErr},			/* Message too long						*/
		{"kEPROTOTYPEErr", kEPROTOTYPEErr},		/* Protocol wrong type for socket		*/
		{"kENOPROTOOPTErr", kENOPROTOOPTErr},		/* Protocol not available				*/
		{"kEPROTONOSUPPORTErr ", kEPROTONOSUPPORTErr},	/* Protocol not supported				*/
		{"kESOCKTNOSUPPORTErr ", kESOCKTNOSUPPORTErr}, /* Socket type not supported			*/
		{"kEOPNOTSUPPErr", kEOPNOTSUPPErr},		/* Operation not supported on socket	*/
		{"kEADDRINUSEErr", kEADDRINUSEErr},		/* CAddress already in use				*/
		{"kEADDRNOTAVAILErr", kEADDRNOTAVAILErr},	/* Can't assign requested address		*/
		{"kENETDOWNErr", kENETDOWNErr},			/* Network is down						*/
		{"kENETUNREACHErr", kENETUNREACHErr},		/* Network is unreachable				*/
		{"kENETRESETErr", kENETRESETErr},		/* Network dropped connection on reset	*/
		{"kECONNABORTEDErr", kECONNABORTEDErr},		/* Software caused connection abort		*/
		{"kECONNRESETErr", kECONNRESETErr},		/* Connection reset by peer				*/
		{"kENOBUFSErr", kENOBUFSErr},			/* No buffer space available			*/
		{"kEISCONNErr", kEISCONNErr},			/* Socket is already connected			*/
		{"kENOTCONNErr", kENOTCONNErr},			/* Socket is not connected				*/
		{"kESHUTDOWNErr", kESHUTDOWNErr},		/* Can't send after socket shutdown		*/
		{"kETOOMANYREFSErr", kETOOMANYREFSErr},		/* Too many references: can't splice	*/
		{"kETIMEDOUTErr", kETIMEDOUTErr},		/* Connection timed out					*/
		{"kECONNREFUSEDErr", kECONNREFUSEDErr},		/* Connection refused					*/
		{"kEHOSTDOWNErr", kEHOSTDOWNErr},		/* Host is down							*/
		{"kEHOSTUNREACHErr", kEHOSTUNREACHErr},		/* No route to host						*/
		{"kEPROTOErr", kEPROTOErr},
		{"kETIMEErr", kETIMEErr},
		{"kENOSRErr", kENOSRErr},
		{"kEBADMSGErr", kEBADMSGErr },
		{"kECANCELErr", kECANCELErr},
		{"kENOSTRErr", kENOSTRErr },
		{"kENODATAErr", kENODATAErr},
		{"kEINPROGRESSErr", kEINPROGRESSErr},
		{"kESRCHErr", kESRCHErr},
		{"kENOMSGErr", kENOMSGErr},

		{"kOTClientNotInittedErr", kOTClientNotInittedErr},
		{"kOTPortHasDiedErr", kOTPortHasDiedErr},
		{"kOTPortWasEjectedErr", kOTPortWasEjectedErr},
		{"kOTBadConfigurationErr",kOTBadConfigurationErr },
		{"kOTConfigurationChangedErr", kOTConfigurationChangedErr},
		{"kOTUserRequestedErr", kOTUserRequestedErr},
		{"kOTPortLostConnection", kOTPortLostConnection},
		{0, 0 }
	};

	return TableLookup(kTable, inErr);
}
#endif
#endif