//----------------------------------------------------------------------------------------
// CAbsServiceHandlerMgr_AC.h
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CAbsServiceHandlerMgr_AC__
#define __CAbsServiceHandlerMgr_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CServiceHandlerFactory_AC__
#include "CServiceHandlerFactory_AC.h"
#endif

// patterns

#ifndef __CEventRouter_AC__
#include "MEventRouter_AC.h"
#endif

#ifndef __CReactorStrategy_AC__
#include "CReactorFactory_AC.h"
#endif

// core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// forward references

template <class T> class CSimpleLinkedList;
class CServiceHandler_AC;
class CNetworkStrategy_AC;

//----------------------------------------------------------------------------------------
// CAbstractServiceHandlerManager_AC is a base class for managing CServiceHandlers_ACs.
//----------------------------------------------------------------------------------------

class EXPORT_AC CAbstractServiceHandlerManager_AC : public MEventRouter_AC
{
public:
	CAbstractServiceHandlerManager_AC();
		// ctor
	
	virtual ~CAbstractServiceHandlerManager_AC() = 0;
		// dtor

// Timers
	virtual void EnableServiceHandlerTimer(long timeOut = CNetGlobals_AC::GetDefaultTimeOut() );
		// This timer will time out if no data is sent or received for the time out durations.
		// This timer defaults to off.
	
	virtual void DisableServiceHandlerTimer();
		// Turn off the timer. 
	
// Strategies
	virtual void AddStrategy(CNetworkStrategy_AC* inStrategy);
		// MEventTransceiver_ACs are ref counted.
	
	virtual void RemoveStrategy(CNetworkStrategy_AC* inStrategy);
		// MEventTransceiver_ACs are ref counted.

// Reactor Factory
	void SetReactorFactory(CReactorFactory_AC* inFactory);
		// Set a factory for creating the Reactors.
	
	CReactorFactory_AC* GetReactorFactory() const;
	
	bool HasReactorFactory() const;

// Service Handler Factory
	void SetServiceHandlerFactory(CServiceHandlerFactory_AC* inFactory);
		// Set a factory for creating the service handler.
	
	CServiceHandlerFactory_AC* GetServiceHandlerFactory() const;
	
	bool HasServiceHandlerFactory() const;


protected:
	typedef CLinkedList_AC<CRefCountingPtr_AC<CNetworkStrategy_AC> > CList;
	
	CAutoPtr_AC<CList> 								fStrategies;
	CRefCountingPtr_AC<CReactorFactory_AC>			fReactorFactory;
	CRefCountingPtr_AC<CServiceHandlerFactory_AC> 	fServiceHandlerFactory;

	bool fServiceHandlerTimer;
	long fTimeOut;

};

// inline methods

inline void CAbstractServiceHandlerManager_AC::SetServiceHandlerFactory(CServiceHandlerFactory_AC* inFactory)
{
	fServiceHandlerFactory = inFactory;
}

inline CServiceHandlerFactory_AC* CAbstractServiceHandlerManager_AC::GetServiceHandlerFactory() const
{
	return fServiceHandlerFactory;
}

inline bool CAbstractServiceHandlerManager_AC::HasServiceHandlerFactory() const
{
	return !fServiceHandlerFactory.IsNULL();
}


inline void CAbstractServiceHandlerManager_AC::SetReactorFactory(CReactorFactory_AC* inFactory)
{
	fReactorFactory = inFactory;
}

inline CReactorFactory_AC* CAbstractServiceHandlerManager_AC::GetReactorFactory() const
{
	return fReactorFactory;
}

inline bool CAbstractServiceHandlerManager_AC::HasReactorFactory() const
{
	return !fReactorFactory.IsNULL();
}
#endif