//----------------------------------------------------------------------------------------
// CAcceptor_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __CAcceptor_AC__
#define __CAcceptor_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CServiceHandlerFactory_AC__
#include "CServiceHandlerFactory_AC.h"
#endif

// Patterns

#ifndef __MEventTransceiver__AC__
#include "MEventTransceiver_AC.h"
#endif

// Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// fwd references

class CReactor_AC;
class CServiceHandler_AC;
class CServiceAcceptor_AC;
class CNetworkAddress_AC;

//----------------------------------------------------------------------------------------
// CAcceptor_AC is a base class for accepting connections. When the connection is complete
// a CServiceHandler_AC (or subclass) is created and opened, for sending and receiving 
// of data.
//----------------------------------------------------------------------------------------

class EXPORT_AC CAcceptor_AC : public MEventTransceiver_AC
{
public:
	
	CAcceptor_AC(CReactor_AC* inReactor = NULL, CServiceHandlerFactory_AC* inFactory = NULL);
		// Set reactor for sending event to registered event receivers. Set
		// ServiceHandlerFactory for creating service handlers. Both params are optional.
	
	virtual ~CAcceptor_AC();
		// dtor

	void ListenTo(const CNetworkAddress_AC& localAddress, const CNetworkAddress_AC& remoteAddress);
		// Specify the local address to listen to and the remote address to listen for connections from.
		// The meaning of this will be further specified by the type of connection.
	
	void SetTimeOut( long inTimeOut = CNetGlobals_AC::GetDefaultTimeOut());
		// Times the accepting of a connection.
	
	long GetTimeOut() const;

	void SetServiceHandlerFactory(CServiceHandlerFactory_AC* inFactory);
		// Set the factory for creating the service handlers.
	
	CServiceHandlerFactory_AC* GetServiceHandlerFactory() const;
	
	bool HasServiceHandlerFactory() const;

protected:
	// implementation

// required ovveride

	virtual CServiceAcceptor_AC* CreateServiceAcceptor(CReactor_AC* inReactor,
														long timeOut) = 0;
		
	virtual CServiceHandler_AC* CreateConnection(CReactor_AC* inReactor);	
		// Override this if your subclass makes a service handler.
		
	CServiceHandler_AC* CreateConnectionObject();	
		// This first tries to make a service handler using the factory
		// then it calls CreateConnection.

protected:
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);
		// optional override.

protected:
	CRefCountingPtr_AC<CServiceAcceptor_AC> 		fAcceptor;
	CRefCountingPtr_AC<CServiceHandlerFactory_AC> 	fServiceHandlerFactory;
private:
	CAcceptor_AC(const CAcceptor_AC& /*inAcceptor*/) : fAcceptor(NULL) {}
	CAcceptor_AC& operator = (const CAcceptor_AC& /*inAcceptor*/) { return *this; }
};

// inline functions

inline void CAcceptor_AC::SetServiceHandlerFactory(CServiceHandlerFactory_AC* inFactory)
{
	fServiceHandlerFactory = inFactory;
}

inline CServiceHandlerFactory_AC* CAcceptor_AC::GetServiceHandlerFactory() const
{
	return fServiceHandlerFactory;
}

inline bool CAcceptor_AC::HasServiceHandlerFactory() const
{
	return !fServiceHandlerFactory.IsNULL();
}


#endif
