//----------------------------------------------------------------------------------------
// CConnector_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CConnector_AC__
#define __CConnector_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CNetworkAddress_AC__
#include "CNetworkAddress_AC.h"
#endif

#ifndef __CServiceHandlerFactory_AC__
#include "CServiceHandlerFactory_AC.h"
#endif

// Patterns

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

// fwd refs
class CEvent_AC;
class CServiceConnector_AC;
class CServiceHandler_AC;
template <class T> class CLinkedList_AC;

//--------------------------------------------------------------------------------------------
// CConnector_AC is a base class for classes the initiate a connection. When the connection
// is completed, the ServiceHandler is opened for reading and writing of data.
//--------------------------------------------------------------------------------------------

class EXPORT_AC CConnector_AC : public MEventTransceiver_AC
{
protected:
	// interface
	
	CConnector_AC(	CReactor_AC* inReactor = NULL, 
					CServiceHandlerFactory_AC* inFactory = NULL);
		// Set reactor for sending event to registered event receivers. Set
		// ServiceHandlerFactory for creating service handlers. Both params are optional.

public:
	virtual ~CConnector_AC() = 0;
		// dtor
	
	CServiceHandler_AC* ConnectTo(	CServiceHandler_AC* inServiceHandler, 
							const CNetworkAddress_AC& remote,
							const CNetworkAddress_AC& local);
		// connect to remote address. Bind to local address. Open inServiceHandler
		// when connection is open.
		
	void SetTimeOut(long inTicks);
		// Times amount of time it is taking to connect to an address. 

	void SetServiceHandlerFactory(CServiceHandlerFactory_AC* inFactory);
		// If you don't specify a ServiceHandler in ConnectTo, you can give
		// this a factory to create a new service handler to represent 
		// the open connection.
	
	CServiceHandlerFactory_AC* GetServiceHandlerFactory() const;
	
	bool HasServiceHandlerFactory() const;

protected:

	virtual CServiceConnector_AC* 
		CreateServiceConnector(CReactor_AC* inReactor, CServiceHandler_AC* inServiceHandler) = 0;

	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);

	virtual CRefCountingPtr_AC<CServiceHandler_AC> RemovePendingConnection(CServiceConnector_AC* inConnector);
		
	virtual CServiceHandler_AC* CreateConnection(CReactor_AC* inReactor);	
		// Override this if your subclass makes a service handler.
		
	CServiceHandler_AC* CreateConnectionObject();	
		// This first tries to make a service handler using the factory
		// then it calls CreateConnection.
		
protected:
	
	typedef CLinkedList_AC<CRefCountingPtr_AC<CServiceConnector_AC> >  CList;
	
	CRefCountingPtr_AC<CServiceHandlerFactory_AC> 	fServiceHandlerFactory;
	CAutoPtr_AC<CList> 								fConnectors;
	long fTimeout;
};

// inline methods

inline void CConnector_AC::SetServiceHandlerFactory(CServiceHandlerFactory_AC* inFactory)
{
	fServiceHandlerFactory = inFactory;
}

inline CServiceHandlerFactory_AC* CConnector_AC::GetServiceHandlerFactory() const
{
	return fServiceHandlerFactory;
}

inline bool CConnector_AC::HasServiceHandlerFactory() const
{
	return !fServiceHandlerFactory.IsNULL();
}

#endif