//----------------------------------------------------------------------------------------
// CFileReceiver_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CFileReceiver_AC__
#define __CFileReceiver_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CFileTransaction_AC__
#include "CFileTransaction_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CFileReceiver_AC is a class for receiving files.
//----------------------------------------------------------------------------------------

class EXPORT_AC CFileReceiver_AC : public CFileTransaction_AC
{
public:
	CFileReceiver_AC(const CFileReceiver_AC& inSender);
		// copy ctor

	CFileReceiver_AC( 	const CFSSpec_AC& inSpec, 
						long fileUsage,
						CBuffer_AC* inBuffer, 
						bool doExpedite 		= false);
		// specify file usage, the spec of where to write the file, and a buffer
		// to use.

	CFileReceiver_AC( 	const CFSSpec_AC& inSpec, 
						long fileUsage,
						long bufferSize 		= kDefaultBufferSize, 
						char* inPtr				= NULL,
						bool doExpedite 		= false);
		// specify file usage, the spec of where to write the file, and a buffer
		// to use.

	virtual ~CFileReceiver_AC();
		// dtor
	
	CFileReceiver_AC& operator = (const CFileReceiver_AC& inSender);
		// assignment operator
	
	virtual CNetworkTransaction_AC *Clone();
	
	void SetDataForkSize(long inSize);
		// Only use this if you're not recieving the file info. This size will
		// be added to fExpectedAmt.
			
	void SetResourceForkSize(long inSize);
		// Only use this if you're not recieving the file info. This size will
		// be added to fExpectedAmt.
	
	bool GetDidReceiveFile() const;
		// returns whether receive got the whole file successfully.
	
	void SetDoDeleteOnError(bool inDelete);
		// defaults to true. If an error occurs with the transfer and if we
		// created the file, if this is set to true, we'll delete the file.
	
	bool GetDoDeleteOnError() const;
		// get the current value for fDeleteOnError

	void DoEmergencyCleanup();
		// This will close the file, and delete it if we created the file
		// and if fDeleteOnError is true.
	
protected:
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);

	virtual void Process( CServiceHandler_AC* inStream);

	bool WriteNextChunk(short& fileRef, long inForkSize);

	void WriteChunkOfFile();
	
	void ReadFinderInfo();

	void CreateFile();
		
	virtual void MakeSpecUnique();	
		
private:
	enum eSubState
	{
		kStarting,
		kCreateFile,
		kReceiveDataFork,
		kRecievingDataFork,
		kReceiveResFork,
		kRecievingResFork,
		kUnknownState
	};

	long fSubState;
	
	short fFileForkRef;
	long fForkTally;
	long fReceiveAmt;
	bool fDeleteOnError;
	bool fDidReceiveFile;
	bool fCreatedFile;
	bool fDataFork;
};

// inline methods

inline bool CFileReceiver_AC::GetDidReceiveFile() const
{	
	return fDidReceiveFile; 
}

inline bool CFileReceiver_AC::GetDoDeleteOnError() const
{	
	return fDeleteOnError; 
}


#endif