//----------------------------------------------------------------------------------------
// CFileTransaction_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CFileTransaction_AC__
#define __CFileTransaction_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CNetworkTransaction_AC__
#include "CNetworkTransaction_AC.h"
#endif

// Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

// Files

#ifndef __CFSSpec_AC__
#include "CFSSpec_AC.h"
#endif

// forward references

class CFile_AC;
class CFSSpec_AC;
class CBufferStream_AC;

//----------------------------------------------------------------------------------------
// Struct SFileInfo_AC is what get sent over the wire when the file is sent
// with finder info
//----------------------------------------------------------------------------------------

typedef struct SFileInfo_AC
{
	char 	fName[32];
	OSType 	fType;
	OSType	fCreator;
	short	fFinderFlags;
	long	fDataForkSize;
	long	fResForkSize;

	long GetStreamedSize()
	{
		return 	sizeof(fName) +
				sizeof(fType) +
				sizeof(fCreator) +
				sizeof(fFinderFlags) +
				sizeof(fDataForkSize) +
				sizeof(fResForkSize);
	}
	
	SFileInfo_AC() 
		: fType(0), 
		fCreator(0),
		fFinderFlags(0), 
		fDataForkSize(0), 
		fResForkSize(0)
	{ 	fName[0] = '\0'; }
} SFileInfo_AC;

// Streaming operators to easily stream the file info in and out.

CBufferStream_AC& operator << (CBufferStream_AC& inStream, SFileInfo_AC& inFo);

CBufferStream_AC& operator >> (CBufferStream_AC& inStream, SFileInfo_AC& inFo);


//----------------------------------------------------------------------------------------
// CFileTransaction_AC is a base class for net transactions that can send
// or receive files. It is important to note the the FileTransaction can send
// or receive either fork of the file, with or without the finder info.
//----------------------------------------------------------------------------------------

class EXPORT_AC CFileTransaction_AC : public CNetworkTransaction_AC
{
public:

	// use these usage enums to specify which parts of the file
	// to send or receive.
	enum eUsage
	{
		kFinderInfo			= 1,
		kDataFork			= 1 << 1,
		kResourceFork		= 1 << 2,
		kEntireFile			= kFinderInfo | kDataFork | kResourceFork
	};
	
	// helpful enums
	enum eMisc
	{
		kTextType 			= 'TEXT',
		kUnknownType		= '????'
	};
	
	CFileTransaction_AC();
		// ctor
	
	CFileTransaction_AC( 	const CFSSpec_AC& inSpec, 
							long fileUsage,
							OSType inType, 
							OSType inCreator,
							CBuffer_AC* inBuffer, 
							bool doExpedite);
		// specify the file location, usage, type, creater, and buffer.

	CFileTransaction_AC( 	const CFSSpec_AC& inSpec, 
							long fileUsage,
							OSType inType, 
							OSType inCreator,
							long bufferSize, 
							char* inPtr,
							bool doExpedite);
		// specify the file location, usage, type, creater, and buffer.
	
	virtual ~CFileTransaction_AC();
		// dtor

	CFileTransaction_AC& operator = (const CFileTransaction_AC& inTransaction);
		// assignment operator copies data, if any.
			
	void SetFileSpec(const CFSSpec_AC& inSpec);
		// set the file location and name
	
	CFSSpec_AC& GetFileSpec();
																		
protected:
	long 					fUsage;
	CFSSpec_AC 				fFileSpec;
	SFileInfo_AC			fInfo;
};

#endif