//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CMultiConnectionManager_AC__
#define __CMultiConnectionManager_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CAbsServiceHandlerMgr_AC__
#include "CAbsServiceHandlerMgr_AC.h"
#endif

// Core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif


// forward references

template <class T> class CSimpleLinkedList;
class CReactor_AC;
class CNetworkTransaction_AC;
class CNetworkAddress_AC;
class CServiceHandler_AC;
class CConnector_AC;
class CAcceptor_AC;
class CReactorFactory_AC;

//----------------------------------------------------------------------------------------
// CMultiServiceHandlerManager_AC mananages muliple connections.
//
// Current limitations
// 		can only listen or connect on one port
//----------------------------------------------------------------------------------------

class EXPORT_AC CMultiConnectionManager_AC : public CAbstractServiceHandlerManager_AC
{
public:
	CMultiConnectionManager_AC();
		// ctor			
	
	virtual ~CMultiConnectionManager_AC();
		// dtor
	
	virtual long GetNumConnections();
		// return number of current connections

// Utils

	virtual CServiceHandler_AC* FindConnectionBy(const CNetworkAddress_AC& inAddress);
		// This searches list of open connections for connections to address.
		
	virtual void SendTransactionTo(const CNetworkAddress_AC& inAddress, CNetworkTransaction_AC* inTransaction);
		// This searches list of open connections and sends the block to it.
	
	virtual void SendTransaction(CNetworkTransaction_AC* inTransaction);
		// This send block to all connections.
		// WARNING: This clones the incoming transaction for each connection!
		
	virtual void DisconnnectFrom(const CNetworkAddress_AC& inAddress);
		// This searches for specific connection and disconnects from it.
	
	virtual void Disconnect();
		// Disconnects from all connections
	
	virtual void HangupOn(const CNetworkAddress_AC& inAddress);
		// This hangs up on connections to inAddress.
	
	virtual void Hangup();
		// Hangs up on all current connections

protected:
	void Idle();

	virtual void GetConnector(CConnector_AC*& inConnectorPtr);
	
	virtual void SetConnector(CConnector_AC* inConnector)
	{	fConnector = inConnector; }

	virtual void GetAcceptor(CAcceptor_AC*& inAcceptor);
	
	virtual void SetAcceptor(CAcceptor_AC* inAcceptor)
	{	fAcceptor = inAcceptor; }

//--- Networking Event Handlers
	
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender );

	virtual CConnector_AC*		CreateConnector(CReactor_AC* inReactor, CServiceHandlerFactory_AC* inFactory) = 0;
	virtual CAcceptor_AC* 		CreateAcceptor(CReactor_AC* inReactor, CServiceHandlerFactory_AC* inFactory) = 0;

protected:
	typedef CLinkedList_AC<CRefCountingPtr_AC<CServiceHandler_AC> > 	CConnectionList;
	
	// data
	CRefCountingPtr_AC<CConnector_AC> 	fConnector;
	CRefCountingPtr_AC<CAcceptor_AC>	fAcceptor;
	CAutoPtr_AC<CConnectionList> 		fConnections;
		// for keeping track of all our connections

};

#endif