//----------------------------------------------------------------------------------------
// CNetworkAddress_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CNetworkAddress_AC__
#define __CNetworkAddress_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

// Core

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Class CNetworkAddress_AC abstracts the notion of a network address to the lowest
// common denominator, which is a text string.
//----------------------------------------------------------------------------------------

class EXPORT_AC CNetworkAddress_AC
{
protected:
	CNetworkAddress_AC();

public:	
	virtual ~CNetworkAddress_AC() = 0;
	
	void SetWithString(const char* inString);
	void SetWithString(const CStr255_AC& inString);
		// Set the address with a string
			
	void GetAsString(CStr255_AC& inString);
	const char* GetAsString() const;
		// Get a string representation of the address
		
	bool operator == (const char* inAddressString) const;
	bool operator != (const char* inAddressString) const;
	bool operator == (const CStr255_AC& inAddressString) const;
	bool operator != (const CStr255_AC& inAddressString) const;
		// compare against a string
		
	operator const char*() const;
		// convert address to a string
	
	virtual CNetworkAddress_AC* Clone() const = 0;
	
protected:
	virtual void ConvertFromString(	const CStr255_AC& inString ) = 0;
	virtual void ConvertToString( CStr255_AC& inString ) const = 0;
		// override thes for each network address subclass.
};

// inline methods

inline CNetworkAddress_AC::CNetworkAddress_AC()
{
}
inline CNetworkAddress_AC::~CNetworkAddress_AC()
{
}
inline void CNetworkAddress_AC::SetWithString(const char* inString)
{	
	ConvertFromString(CStr255_AC(inString)); 
}
inline void CNetworkAddress_AC::SetWithString(const CStr255_AC& inString)
{	
	ConvertFromString(inString); 
}
inline void CNetworkAddress_AC::GetAsString(CStr255_AC& inString)
{	
	ConvertToString(inString); 
}
inline const char* CNetworkAddress_AC::GetAsString() const
{	
	CStr255_AC toConvert; 
	ConvertToString(toConvert); 
	return toConvert;
}
inline CNetworkAddress_AC::operator const char*() const
{	
	return GetAsString(); 
}



#endif