//----------------------------------------------------------------------------------------
// CProvider_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CProvider_AC__
#define __CProvider_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

// Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

bool InitNetworkSuite_AC();
	// This returns true when finished.
	// This utility function initializes the provider in an non-blocking asynchronous manner
	// i.e. you can the following:
	// 
	// while(!InitNetworkSuite())
	// {
	//		SpinCursor();
	//		Yield();
	// 		UpdateProgressWindow();
	// }

//----------------------------------------------------------------------------------------
// Class CProvider represents the OS specific network API, i.e. Open Transport, WinSock,
// or MacTCP. You shouldn't need to open it yourself, as most of the network classes 
// will open it before doing anything.
//----------------------------------------------------------------------------------------

class EXPORT_AC CProvider_AC
{
private:
	CProvider_AC();
	
public:
	virtual ~CProvider_AC();
		
	static CProvider_AC* Instance();
	
	static void SetProvider(CProvider_AC* inProvider);
	
	static void Open();
	
	static bool IsOpen();
	
	static bool IsProviderID(OSType inType);
		// check the provider specific headers for the different provider ID's.
	
	static OSType GetProviderID();
		
protected:

	static CProvider_AC* CreateProvider();

	virtual OSType GetProviderID() const = 0;
		
protected:
	virtual bool IsAvailable() = 0;
	
	virtual void InitProvider() = 0;
	
	virtual bool IsOpen() const = 0;
	
private:	
	static CAutoPtr_AC<CProvider_AC> fgSelf;

};

// inline methods

inline CProvider_AC* CProvider_AC::Instance()	
{	
	if(!fgSelf) 
		Open(); 
		
	return fgSelf; 
}
inline void CProvider_AC::SetProvider(CProvider_AC* inProvider)
{	
	fgSelf = inProvider; 
}
inline bool CProvider_AC::IsProviderID(OSType inType)
{	
	return Instance()->GetProviderID() == inType; 
}
inline OSType CProvider_AC::GetProviderID()
{	
	return Instance()->GetProviderID(); 
}


#endif

