//----------------------------------------------------------------------------------------
// CServiceHandler_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __CServiceHandler_AC__
#define __CServiceHandler_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

#ifndef __CService_AC__
#include "CService_AC.h"
#endif

// Patterns

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

// Core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif


// forward references
template <class T> class CLinkedList_AC;

class CEventTransceiverTimer_AC;
class CNetworkTransaction_AC;
class CService_AC;
class CServiceHandler_AC;
class CServiceState_AC;

// Typedef

typedef CRefCountingPtr_AC<CServiceHandler_AC> CServiceHandlerPtr_AC;

//----------------------------------------------------------------------------------------
// class CServiceHandler_AC is the meat and potatoes class of the networking suite. This
// is the class with which you send and receive data across the network. Generally this 
// is done using net transactions which can be queued up in the CServiceHandler_AC object.
// The queue of net transactions will not be touched until the ServiceHandler is opened. 
//
// The "Service" is the provider specific implementation class, like CMacTCPConnection_AC.
//----------------------------------------------------------------------------------------

class EXPORT_AC CServiceHandler_AC : public MEventTransceiver_AC
{
public:
	CServiceHandler_AC(CReactor_AC* inReactor);
		// Set the event sending reactor
	
	virtual ~CServiceHandler_AC();
	
// sending and receiving	
	virtual void Send(CNetworkTransaction_AC* inTransaction);
		// Queue up a 'sending' net transaction
	
	virtual void Receive(CNetworkTransaction_AC* inTransaction);
		// Queue up a 'receiving' net transaction

	void SendData(const char* inData, long amt, long flags);
		// returns kSentData event when complete. Data is expected to be unchanged
		// until kSentData event received.
		
	void ReceiveData(char* inData, long amt, long &flags);
		// returns kRecievedData event when complete. Data is expected to be unchanged
		// until kRecievedData event received.
	
// Misc transaction related methods.
	void AbortAllTransactions();

	void Hangup();

	void Disconnect();

// Misc
	void SetUserData(void* inData);
	
	void* GetUserData() const;

	CService_AC* GetService();

	long GetUnreceivedDataAmount() const;
	
	virtual	CServiceState_AC* GetState() const;	
		// the state returns information like the remote host address, etc..

// timer	
	virtual void EnableServiceHandlerTimer(long timeOut = CNetGlobals_AC::GetDefaultTimeOut() );
		// This timer will time out if nothing happens on the service for the time out period.
		
	virtual void DisableServiceHandlerTimer();

	virtual bool IsTimingServiceHandler() const;

protected:
	
		
	virtual CService_AC* CreateService(CReactor_AC* inReactor) = 0;
	
// implementation
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);

	virtual void Idle();	
			
	void QueueTransaction(CNetworkTransaction_AC* inTransaction);

// transaction managment
	void ProcessTarget(CRefCountingPtr_AC<CNetworkTransaction_AC>& target);			

	void FindNextTransaction(CRefCountingPtr_AC<CNetworkTransaction_AC>&  inTransaction, bool IsSender);

	bool OpenTransaction(CRefCountingPtr_AC<CNetworkTransaction_AC>&  inTransaction);
		// returns true if opened it.
	
	void HandleTargetError(CRefCountingPtr_AC<CNetworkTransaction_AC>& inTarget, CError_AC inError);

public:

	virtual void Open();
		// don't call this
	
private:
	typedef CLinkedList_AC<CRefCountingPtr_AC<CNetworkTransaction_AC> > CList;
	
	CRefCountingPtr_AC<CService_AC>				fService;
	CRefCountingPtr_AC<CNetworkTransaction_AC>	fSender;
	CRefCountingPtr_AC<CNetworkTransaction_AC>	fReceiver;
	CAutoPtr_AC<CList>							fTransactions;
	bool										fOpen;
	void* 										fUserData;
};

// inline methods

inline CService_AC* CServiceHandler_AC::GetService() 
{ 
	return fService; 
}
inline void CServiceHandler_AC::SendData(const char* inData, long amt, long flags)
{	
	::AssertNotNULL_AC(fService); fService->Snd(inData, amt, flags); 
}
inline void CServiceHandler_AC::ReceiveData(char* inData, long amt, long &flags)
{	
	::AssertNotNULL_AC(fService); fService->Rcv(inData, amt, flags); 
}
inline void CServiceHandler_AC::Hangup()
{
	::AssertNotNULL_AC(fService); fService->Hangup();
}
inline void CServiceHandler_AC::Disconnect()
{
	::AssertNotNULL_AC(fService); fService->Disconnect();
}
inline long CServiceHandler_AC::GetUnreceivedDataAmount() const
{
	::AssertNotNULL_AC(fService); return fService->GetUnreceivedDataAmount();
}


#endif