//----------------------------------------------------------------------------------------
// CService_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CService_AC__
#define __CService_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

// Patterns

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// forward references

class CEventTransceiverTimer_AC;
class CServiceState_AC;

//----------------------------------------------------------------------------------------
// CService_AC defines the interface for provider specific implemenation classes, such
// COTConnection_AC.
//----------------------------------------------------------------------------------------

class EXPORT_AC CService_AC : public MEventTransceiver_AC
{
public:
	enum eFlags
	{
		kMore 		= 1,
		kPriority 	= 1 << 1
	};

	CService_AC(CReactor_AC* inReactor);
	
	virtual ~CService_AC();

	virtual void Rcv(char*, long amt, long &flags) = 0;	
		// will generate kReceivedData when received amt or all available data
		
	virtual void Snd(const char*, long amt, long flags) = 0;
		// will generate kSentData event with amount of data as event data.
	
	virtual void Hangup();
	
	virtual void Disconnect();
		
	virtual void EnableServiceHandlerTimer(long timeOut = CNetGlobals_AC::GetDefaultTimeOut(),
									bool startTimer = true );
	
	virtual void DisableServiceHandlerTimer();
		
	bool IsTimingServiceHandler() const;
	
	CEventTransceiverTimer_AC* GetTimer();
		
	virtual	CServiceState_AC* GetState() const = 0;
	
	virtual long GetUnreceivedDataAmount() const = 0;


protected:
	typedef CRefCountingPtr_AC<CEventTransceiverTimer_AC> CTimerPtr;

	bool 			fTiming;
	CTimerPtr		fTimer;
};

// Inline methods

inline bool CService_AC::IsTimingServiceHandler() const
{
	return fTiming;
}
inline 	CEventTransceiverTimer_AC* CService_AC::GetTimer()
{
	return fTimer; 
}

#endif