//----------------------------------------------------------------------------------------
// CAcceptor_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

// Network

#ifndef __CAcceptor_AC__
#include "CAcceptor_AC.h"
#endif


#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __CServiceAcceptor_AC__
#include "CServiceAcceptor_AC.h"
#endif


#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

// patterns

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

// streams

#ifdef qNetTrace_AC 
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif


//----------------------------------------------------------------------------------------
// CAcceptor_AC::CAcceptor_AC
//----------------------------------------------------------------------------------------
	
CAcceptor_AC::CAcceptor_AC(CReactor_AC* inReactor, CServiceHandlerFactory_AC* inFactory)
	: MEventTransceiver_AC(inReactor),
	fServiceHandlerFactory(inFactory)
{
}

//----------------------------------------------------------------------------------------
// CAcceptor_AC::~CAcceptor_AC
//----------------------------------------------------------------------------------------
	
CAcceptor_AC::~CAcceptor_AC()
{
}

//----------------------------------------------------------------------------------------
// CAcceptor_AC::ListenTo
//----------------------------------------------------------------------------------------

void CAcceptor_AC::ListenTo(const CNetworkAddress_AC &localAddress, const CNetworkAddress_AC &remoteAddress) 
{ 
	try
	{
		fAcceptor = CreateServiceAcceptor(GetReactor(), CNetGlobals_AC::GetDefaultTimeOut());
	
		RegisterForEventsFrom(fAcceptor);

		fAcceptor->ListenTo(localAddress, remoteAddress); 
			// if this fails and throws an exception the CServiceHandler acceptor will
			// clean up after itself, and close any partially open
			// service providers.
	}
	catch(CException_AC& inEx)
	{
		PostEvent(
			TH_new CEvent_AC(CNetEvent_AC::kError, inEx.GetError()));
	}
}

//----------------------------------------------------------------------------------------
// CAcceptor_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CAcceptor_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	CServiceHandlerPtr_AC connection = NULL;
		// this is a counting ptr
		
	try
	{
		switch(inEvent->GetEventID())
		{
			
			case CNetEvent_AC::kError:
				PostEvent(TH_new CEvent_AC( CNetEvent_AC::kAcceptFailed, inEvent->GetEventID() ));
				break;
			
			case CNetEvent_AC::kAcceptRejected:
			case CNetEvent_AC::kAcceptFailed:
				ReceiveEvent_AC(inEvent);
				PostEvent(inEvent);
			break;		
	
	
			case CNetEvent_AC::kAcceptSucceeded:
			
				// okay, we got an event from the service acceptor. The connection
				// has been accepted.
				ReceiveEvent_AC(inEvent);
							
#if qNetTrace_AC
				TRACE << LowTracePriority_AC << "CAcceptor: Creating CServiceHandler Handler." << endl_AC;
#endif			
				connection = CreateConnectionObject();
	
					// make a new service handler object - i.e. the connection class
					
				connection->Open();
					// tell the connection we're ready to rock!

				fAcceptor->Accept(connection);
					// wire up the new connection object to the service acceptor
					
				PostEvent( CreateEventWithData_AC(CNetEvent_AC::kAcceptSucceeded, connection));
					// pass the connection to the subscribers. Note that "connection" is a countingPtr. 
					// If nobody Receives the message, the connection will be deleted.
			break;

			default:
				MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
			break;

		}
	}
	catch(CException_AC& inEx)
	{
		PostEvent(
			TH_new CEvent_AC(CNetEvent_AC::kError, inEx.GetError()));
	}
}

//----------------------------------------------------------------------------------------
// CAcceptor_AC::SetTimeOut
//----------------------------------------------------------------------------------------
	
void CAcceptor_AC::SetTimeOut( long inTimeOut)
{
	fAcceptor->SetTimeOut(inTimeOut);
}

//----------------------------------------------------------------------------------------
// CAcceptor_AC::GetTimeOut
//----------------------------------------------------------------------------------------
	
long CAcceptor_AC::GetTimeOut() const
{
	return fAcceptor->GetTimeOut();
}

//----------------------------------------------------------------------------------------
// CAcceptor_AC::CreateConnection
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC* CAcceptor_AC::CreateConnection(CReactor_AC* /*inReactor*/)
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// CAcceptor_AC::CreateConnectionObject
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC* CAcceptor_AC::CreateConnectionObject()
{
	return HasServiceHandlerFactory() ? 
		fServiceHandlerFactory->CreateServiceHandler(GetReactor()) :
		CreateConnection(GetReactor());
}	
