
//----------------------------------------------------------------------------------------
// CConnector_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Network

#ifndef __CConnector_AC__
#include "CConnector_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __CServiceConnector_AC__
#include "CServiceConnector_AC.h"
#endif

#ifndef __CNetGlobals_AC__
#include "CNetGlobals_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

// patterns

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

// streams

#if qNetTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif

// containers

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CConnector_AC::CConnector_AC
//----------------------------------------------------------------------------------------
	
CConnector_AC::CConnector_AC(CReactor_AC* inReactor, CServiceHandlerFactory_AC* inFactory)
	: MEventTransceiver_AC(inReactor),
	fServiceHandlerFactory(inFactory),
	fTimeout(CNetGlobals_AC::GetDefaultTimeOut())
{
	fConnectors = TH_new CList;
}

//----------------------------------------------------------------------------------------
// CConnector_AC::~CConnector_AC
//----------------------------------------------------------------------------------------
	
CConnector_AC::~CConnector_AC()
{
}

//----------------------------------------------------------------------------------------
// CConnector_AC::SetTimeOut
//----------------------------------------------------------------------------------------
	
void CConnector_AC::SetTimeOut(long inTicks)
{
	fTimeout = inTicks;
}

//----------------------------------------------------------------------------------------
// CConnector_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CConnector_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver )
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kTimedOut:
		
			PostEvent( 
				TH_new CEventWithData_AC<CServiceHandlerPtr_AC>(
							CNetEvent_AC::kConnectTimedOut, 
							RemovePendingConnection(dynamic_cast_AC(CServiceConnector_AC*,inTransceiver))));

		break;
	
		case CNetEvent_AC::kConnectSucceeded:
			ReceiveEvent_AC(inEvent);

			PostEvent( 
				TH_new CEventWithData_AC<CServiceHandlerPtr_AC>(
							CNetEvent_AC::kConnectSucceeded, 
							RemovePendingConnection(dynamic_cast_AC(CServiceConnector_AC*,inTransceiver))));

			// send event to handlers. This message also contains a countingPtr
			// so if nobody accepts the connection, the countingptrs will all destruct
			// and kill off the connection.

		break;

		case CNetEvent_AC::kError:
		case CNetEvent_AC::kConnectFailed:
			ReceiveEvent_AC(inEvent);

			PostEvent( 
				TH_new CEventWithData_AC<CServiceHandlerPtr_AC>(
							CNetEvent_AC::kConnectFailed, 
							RemovePendingConnection(dynamic_cast_AC(CServiceConnector_AC*,inTransceiver)),
							inEvent->GetError()));
		break;
				
		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
		break;
	}
}

//----------------------------------------------------------------------------------------
// CConnector_AC::RemovePendingConnection
//----------------------------------------------------------------------------------------
	
CServiceHandlerPtr_AC CConnector_AC::RemovePendingConnection(CServiceConnector_AC* inConnector)
{
	::AssertNotNULL_AC(inConnector);

	CServiceHandlerPtr_AC connection = inConnector->GetPendingConnection();
	::AssertNotNULL_AC(connection);

	fConnectors->Remove(CServiceConnectorPtr_AC(inConnector));

	return connection;
}

//----------------------------------------------------------------------------------------
// CConnector_AC::ConnectTo
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC* CConnector_AC::ConnectTo(	CServiceHandler_AC* inConnection, 
								const CNetworkAddress_AC& inRemoteAddress, 
								const CNetworkAddress_AC& inLocalAddress)
{
	CServiceConnectorPtr_AC connector = NULL;
	CServiceHandler_AC* outConnection = inConnection;
	
	try
	{
		if(!outConnection)
			outConnection = CreateConnectionObject();
		
		connector = CreateServiceConnector(GetReactor(), outConnection);
			// Make a new service connector. We're going to store the connection handler
			// in the service connector for later retrieval. The connection handler is 
			// ref counted so it will get deleted if something goes wrong if nobody else
			// owns it.

		RegisterForEventsFrom(connector);
			// We need to Receive events from it.

		connector->SetRemoteAddress(inRemoteAddress);
		
		connector->SetLocalAddress(inLocalAddress);

		fConnectors->Push(connector);
		
		connector->DoConnect(fTimeout);
			// Tell it to connect. The service connector will complete the connection
			// and open the connection.
			
	}
	catch(CException_AC& inEx)
	{
		PostEvent(
			::CreateEventWithData_AC(
				CNetEvent_AC::kConnectFailed, 
				outConnection, 
				inEx.GetError()));
	}

	return outConnection;
}

//----------------------------------------------------------------------------------------
// CConnector_AC::CreateConnection
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC* CConnector_AC::CreateConnection(CReactor_AC* /*inReactor*/)
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// CConnector_AC::CreateConnectionObject
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC* CConnector_AC::CreateConnectionObject()
{
	return HasServiceHandlerFactory() ? 
		fServiceHandlerFactory->CreateServiceHandler(GetReactor()) :
		CreateConnection(GetReactor());
}	
