//----------------------------------------------------------------------------------------
// DataTransactions_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

// Network

#ifndef __CDataReceiver_AC__
#include "CDataReceiver_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __CService_AC__
#include "CService_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

// streams

#if qTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif

// patterns

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CDataReceiver_AC::CDataReceiver_AC
//----------------------------------------------------------------------------------------
	
CDataReceiver_AC::CDataReceiver_AC(unsigned long size, char* inPtr, bool priority)
: 	CNetworkTransaction_AC(size, inPtr, priority)
{
	SetPriority(kDataPriority);
	SetIsData(true);
}

//----------------------------------------------------------------------------------------
// CDataReceiver_AC::CDataReceiver_AC
//----------------------------------------------------------------------------------------
	
CDataReceiver_AC::CDataReceiver_AC(const CDataReceiver_AC& inSender)
{
	*this = inSender;
}

//----------------------------------------------------------------------------------------
// CDataReceiver_AC::~CDataReceiver_AC
//----------------------------------------------------------------------------------------
	
CDataReceiver_AC::~CDataReceiver_AC()
{
}

//----------------------------------------------------------------------------------------
// CDataReceiver_AC::operator =
//----------------------------------------------------------------------------------------
	
CDataReceiver_AC& CDataReceiver_AC::operator = (const CDataReceiver_AC& inSender)
{
	CNetworkTransaction_AC::operator = (inSender);
	return *this;
}

//----------------------------------------------------------------------------------------
// CDataReceiver_AC::Clone
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC *CDataReceiver_AC::Clone()
{
	return TH_new CDataReceiver_AC(*this);
}

//----------------------------------------------------------------------------------------
// CDataReceiver_AC::Process
//----------------------------------------------------------------------------------------
	
void CDataReceiver_AC::Process( CServiceHandler_AC* inServiceHandler )
{
	switch(GetState())
	{
		case kOpening:
			GetStream().RewindPositions();
			// fall through
		case kProcessing:
		{	
			
			long flags;
			
			ChangeStateTo(kBlocking);
			
			long pendingData = inServiceHandler->GetUnreceivedDataAmount();
			if(!fExpectedAmt) 
				fExpectedAmt = pendingData;
			
			if(pendingData)
			{
				long amtToGet = ::Min_AC(pendingData, GetStream().GetAmountFree());
			
				inServiceHandler->ReceiveData(GetStream().GetWritePositionPtr(), amtToGet, flags);
			}
		}
		break;
	
	
		case kResuming:
			
			if(fTransferedAmt < fExpectedAmt)
				ChangeStateTo(kBlocking);
			else
				Complete();
				
		break;
	}
}

//----------------------------------------------------------------------------------------
// CDataReceiver_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CDataReceiver_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* /*inSender*/)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kReceivedData:
		{	
			long rcvAmt = inEvent->GetLongData();
			
			GetStream().OffsetWritePosition(rcvAmt);
			
			fTransferedAmt += rcvAmt;

#if qNetTrace_AC			
			TRACE << LowTracePriority_AC << "****Received " << rcvAmt << "k of data." << endl_AC;
#endif

			if(fTransferedAmt < fExpectedAmt || fExpectedAmt == 0)
				ChangeStateTo(kBlocking);
			else
				Complete();
		}	
		break;
		
		case CNetEvent_AC::kIncomingData:
			if(IsAtState(kBlocking))
					ChangeStateTo(kProcessing);
		break;
	}
}
