//----------------------------------------------------------------------------------------
// DataTransactions_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

// network

#ifndef __CDataSender_AC__
#include "CDataSender_AC.h"
#endif

#ifndef __CService_AC__
#include "CService_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

// streams

#if qNetTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif

// patterns

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CDataSender_AC::CDataSender_AC
//----------------------------------------------------------------------------------------
	
CDataSender_AC::CDataSender_AC(const CDataSender_AC& inTransceiver)
{
	*this = inTransceiver;
}

//----------------------------------------------------------------------------------------
// CDataSender_AC::CDataSender_AC
//----------------------------------------------------------------------------------------
	
CDataSender_AC::CDataSender_AC(unsigned long size, char* inPtr, bool priority)
: 	CNetworkTransaction_AC(size, inPtr, priority)
{
	SetPriority(kDataPriority);
	SetIsData(true);
}

//----------------------------------------------------------------------------------------
// CDataSender_AC::~CDataSender_AC
//----------------------------------------------------------------------------------------
	
CDataSender_AC::~CDataSender_AC()
{
}

//----------------------------------------------------------------------------------------
// CDataSender_AC::operator =
//----------------------------------------------------------------------------------------
	
CDataSender_AC& CDataSender_AC::operator = (const CDataSender_AC& inTransceiver)
{
	CNetworkTransaction_AC::operator = (inTransceiver);
	return *this;
}

//----------------------------------------------------------------------------------------
// CDataSender_AC::Clone
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC *CDataSender_AC::Clone()
{
	return TH_new CDataSender_AC(*this);
}

//----------------------------------------------------------------------------------------
// CDataSender_AC::Process
//----------------------------------------------------------------------------------------
	
void CDataSender_AC::Process( CServiceHandler_AC* inStream)
{
	switch(GetState())
	{
		case kOpening:
			GetStream().SetReadPosition(0);
		
			if(fExpectedAmt == 0) 
				fExpectedAmt = GetStream().GetWritePosition();

		case kProcessing:
		{
			long flags = IsAtState(kProcessing) ? CService_AC::kMore : 0;
				// only flip kMore flag if it's the first time through.
			
			::SetBits_AC(flags,CService_AC::kPriority, IsExpedited());  
			
			ChangeStateTo(kBlocking);
		
			inStream->SendData(
				GetStream().GetReadPositionPtr(), 
				GetStream().GetAmountAvailable(), flags);
				// if sentAmt == 0, this means try again.
		}
		break;
		
		case kResuming:
			
			
			// What happens if we are suspended in the middle
			// of  a send, i.e. there's a kMore pending? What
			// happens when we resume? Hmm.
			
			if(fTransferedAmt < fExpectedAmt)
				ChangeStateTo(kBlocking);
			else
				Complete();
				
		break;
	}
	
	
	// fTransferedAmt < fExpectedAmt
}

//----------------------------------------------------------------------------------------
// CDataSender_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CDataSender_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* /*inSender*/)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kSentData:
		{
			long sentAmt = inEvent->GetLongData();
			if(sentAmt)
			{
				fTransferedAmt += sentAmt;
				
				GetStream().OffsetReadPosition(sentAmt);
				
				if(GetStream().GetAmountAvailable())
				{
					// Still more data in the buffer to send.
					ChangeStateTo(kProcessing);
				}
				else
				if(fTransferedAmt < fExpectedAmt)
				{
					
					// set the state to processing then the buffer needs to be
					// refilled.
					
					Assertion_AC(false, "Entered unsupported state.");
				
					GetStream().RewindPositions();

					ChangeStateTo(kProcessing);
						// wait for stream to tell us it can accept more data.
				}
				else
					Complete();
						// else we're done.		
#if qNetTrace_AC						
				TRACE << LowTracePriority_AC << "****Sent " << sentAmt << "k of data." << endl_AC;
#endif
			}

		}
		
		break;
	}
}

