//----------------------------------------------------------------------------------------
// CFileReceiver_AC.h
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

// Net includes

#ifndef __CFileReceiver_AC__
#include "CFileReceiver_AC.h"
#endif

#ifndef __CService_AC__
#include "CService_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

// pattern includes


#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

// streams

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

// core

#if qNetDebug_AC
	#ifndef __dout_AC__
	#include "dout_AC.h"
	#endif
#endif

// toolbox

#ifndef __Resources__
#include <Resources.h>
#endif
//----------------------------------------------------------------------------------------
// CFileReceiver_AC::CFileReceiver_AC
//----------------------------------------------------------------------------------------
	
CFileReceiver_AC::CFileReceiver_AC( const CFileReceiver_AC& inSender )
	: fSubState(kStarting),
	fFileForkRef(0),
	fForkTally(0),
	fReceiveAmt(0),
	fDeleteOnError(true),
	fDidReceiveFile(false),
	fCreatedFile(false)
{
	SetPriority(kDataPriority);
	SetIsData(true);

	*this = inSender;
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::CFileReceiver_AC
//----------------------------------------------------------------------------------------
	
CFileReceiver_AC::CFileReceiver_AC( const CFSSpec_AC& inSpec, 
								long fileUsage,
								CBuffer_AC* inBuffer, 
								bool doExpedite)
	: CFileTransaction_AC(inSpec, fileUsage, kTextType, kUnknownType, inBuffer, doExpedite),
	fSubState(kStarting),
	fFileForkRef(0),
	fForkTally(0),
	fReceiveAmt(0),
	fDeleteOnError(true),
	fDidReceiveFile(false),
	fCreatedFile(false)
{
	SetPriority(kDataPriority);
	SetIsData(true);
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::CFileReceiver_AC
//----------------------------------------------------------------------------------------
	
CFileReceiver_AC::CFileReceiver_AC( const CFSSpec_AC& inSpec, 
								long fileUsage,
								long bufferSize, 
								char* inPtr,
								bool doExpedite)
	: CFileTransaction_AC(inSpec, fileUsage, kTextType, kUnknownType, bufferSize, inPtr,doExpedite),
	fSubState(kStarting),
	fFileForkRef(0),
	fForkTally(0),
	fReceiveAmt(0),
	fDeleteOnError(true),
	fDidReceiveFile(false),
	fCreatedFile(false)
{
	SetPriority(kDataPriority);
	SetIsData(true);
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::~CFileReceiver_AC
//----------------------------------------------------------------------------------------
	
CFileReceiver_AC::~CFileReceiver_AC()
{
	DoEmergencyCleanup();
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::operator =
//----------------------------------------------------------------------------------------
	
CFileReceiver_AC& CFileReceiver_AC::operator = (const CFileReceiver_AC& inSender)
{
	CFileTransaction_AC::operator = (inSender);
	return *this;
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::Clone
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC *CFileReceiver_AC::Clone()
{
	return TH_new CFileReceiver_AC(*this);
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::ReadFinderInfo
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::ReadFinderInfo()
{
	if( fUsage & kFinderInfo ) 
	{ 
		GetStream() >> fInfo;	
			// blast info into struct using our custom extraction operator

		fExpectedAmt  = fInfo.GetStreamedSize() +
						fInfo.fDataForkSize +
						fInfo.fResForkSize;
	
		fFileSpec.SetName( CStr63_AC( fInfo.fName) );
	
		fTransferedAmt += fInfo.GetStreamedSize();
		
#if qNetTrace_AC					
		TRACE 	<< LowTracePriority_AC 
				<< "Recieving File: << " 
				<< fInfo.fName
				<< "\n    DataForkSize = " 
				<< fInfo.fDataForkSize
				<< "\n    ResForkSize  = " 
				<< fInfo.fResForkSize
				<< endl_AC;
#endif			
	}
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::CreateFile
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::CreateFile()
{
	
	MakeSpecUnique();
	
	if( fUsage & kDataFork )
	{
		::ThrowIfError_AC(
		 	::FSpCreate(fFileSpec, fInfo.fCreator, fInfo.fType, 0));
	}
	if( fUsage & kResourceFork )
	{
		::FSpCreateResFile(fFileSpec, fInfo.fCreator, fInfo.fType, 0);
		::ThrowIfError_AC(
			 ::ResError());
	}
	
	if( fUsage & kFinderInfo )
	{
		FInfo finderInfo;
		::ThrowIfError_AC(
			::FSpGetFInfo(fFileSpec, &finderInfo));
			
		enum masks
		{
			kAliasMask 		= 0x8000, 
			kCustomIconMask	= 0x0400,
			kStationaryMask	= 0x0800
		};
		
		finderInfo.fdFlags |= (fInfo.fFinderFlags & (kAliasMask | kCustomIconMask | kStationaryMask | fHasBundle));
		
		::ThrowIfError_AC(
			 ::FSpSetFInfo(fFileSpec, &finderInfo));
	}
	
	fCreatedFile = true;
	fSubState = kReceiveDataFork;
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::MakeSpecUnique
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::MakeSpecUnique()
{
	fFileSpec.FindUniqueSpec();
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::WriteChunkOfFile
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::WriteChunkOfFile()
{
	bool rcvData = true;	
	
	ChangeStateTo(kProcessing);
	
	do
	{
		switch( fSubState )
		{
			case kStarting:
				ReadFinderInfo();
				fSubState = kCreateFile;
			break;
			
			case kCreateFile:
				CreateFile();
			break;
			
			case kReceiveDataFork:
				
				if( fUsage & kDataFork ) 
				{
					::Assertion_AC( fFileForkRef == 0 , NULL);
					
					if(fInfo.fDataForkSize > 0)
					{
						::ThrowIfError_AC(
							::FSpOpenDF(fFileSpec,fsRdWrPerm,&fFileForkRef));
					
						fSubState = kRecievingDataFork;
					}
					else
						fSubState = kReceiveResFork;
				
				}
				else
				{
					fSubState = kReceiveResFork;
				}
					
				// should be some data in the buffer at this point
				// because we've only snagged the fInfo, if there is any.	
					
			break;
				
			case kRecievingDataFork:
				if( WriteNextChunk( fFileForkRef, fInfo.fDataForkSize ))
				{
					// WriteNextChunk will write the remaining data to the
					// file and clear the buffers when done.
				
					fSubState = kReceiveResFork;
				}
				
				// else buffer is empty
			
			break;
		
			case kReceiveResFork:
				
				if( fUsage & kResourceFork ) 
				{
					::Assertion_AC( fFileForkRef == 0, NULL );
					
					if( fInfo.fResForkSize > 0 )
					{
						::ThrowIfError_AC(
							::FSpOpenRF(fFileSpec,fsRdWrPerm,&fFileForkRef));
				
						fSubState = kRecievingResFork;	
					}
					else
						fSubState = kUnknownState;
				}
			break;
		
			case kRecievingResFork:

				// WriteNextChunk will write the remaining data to the
				// file and clear the buffers when done.

				if(WriteNextChunk( fFileForkRef, fInfo.fResForkSize ))
				{
					fSubState = kUnknownState;
				}
				
				// else buffer is empty
			break;
			
			case kUnknownState:
				::Assertion_AC(false, NULL);
			break;
		}
		
		// logic to end the looping
		
		if(fExpectedAmt != 0 && fTransferedAmt == fExpectedAmt)
		{
			fDidReceiveFile = true;
			Complete();
		}
		else
		if( GetStream().GetAmountAvailable() == 0)
		{
			ChangeStateTo(kBlocking);		
			
			GetStream().RewindPositions();
				// set up buffer to read in next chunk
		}

	} while( IsAtState(kProcessing));
	
	if(GetState() == kBlocking)
		ChangeStateTo(kProcessing);

}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::Process
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::Process( CServiceHandler_AC* inServiceHandler)
{
	OSErr err = 0;

	switch(GetState())
	{
		case kOpening:
			fDidReceiveFile = false;
			fCreatedFile = false;
			
			fExpectedAmt = 0;
			fSubState = kStarting;

			GetStream().RewindPositions();
				// set up buffer to read in next chunk
			
			ChangeStateTo(kProcessing);
		// fall through
		
		case kProcessing:
		{
			ChangeStateTo(kBlocking);
			
			fReceiveAmt = inServiceHandler->GetUnreceivedDataAmount();

			if(fReceiveAmt)
			{
				fReceiveAmt = ::Min_AC(fReceiveAmt, GetStream().GetAmountFree());
				long flags; 
			
				inServiceHandler->ReceiveData(GetStream().GetWritePositionPtr(), fReceiveAmt, flags);
			}
			// else wait for data arrival.
		}
		break;
		
		case kError:
			DoEmergencyCleanup();
		break;
	}
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::DoEmergencyCleanup
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::DoEmergencyCleanup()
{
	if(fFileForkRef)
	{
		::FSClose(fFileForkRef);
		fFileForkRef = 0;
	}

	if(!fDidReceiveFile && fDeleteOnError && fCreatedFile)
	{
		::FSpDelete(fFileSpec);
	}
}

//----------------------------------------------------------------------------------------
//  CFileReceiver_AC::WriteNextChunk
//----------------------------------------------------------------------------------------
	
bool CFileReceiver_AC::WriteNextChunk(short& fileRef, long inForkSize)
{
	bool done = false;
	
	long amtToWrite = ::Min_AC(	GetStream().GetAmountAvailable(), inForkSize - fForkTally );
		// figure out how much we want to write.
		
	long amtWritten = amtToWrite;
		// save how much is in the buffer for later comparison
		
	::Assertion_AC( GetStream().GetAmountAvailable() >= amtToWrite, NULL);	

	OSErr err = ::FSWrite( fileRef, &amtWritten, GetStream().GetReadPositionPtr() );
		// write into the file from the current pos in the fStream.
	::ThrowIfError_AC(err);
		
	fForkTally += amtWritten; 
	
	fTransferedAmt += amtWritten;

	GetStream().OffsetReadPosition(amtWritten);
			// Increment the out stream so we know where to start writing to the disk 
			// from next time around.

#if qNetTrace_AC		
	TRACE << LowTracePriority_AC << "****Received " << fTransferedAmt << " of " << fExpectedAmt << " bytes." << endl_AC;
	TRACE << LowTracePriority_AC << "****Received " << fForkTally << " of " << inForkSize << " fork bytes." << endl_AC;
#endif		
	
	if(fForkTally == inForkSize)
	{
		fForkTally = 0;
	
#if qNetTrace_AC		
		TRACE << LowTracePriority_AC << "Completed Fork: " << inForkSize << endl_AC;
#endif		
		
		// If we know how big the file is and if the fork is filled up according to
		// where the file was plus how much we've written, then we must be done 
		// writing the fork, so close the file, and return true - we've finished.
	
		done = true;

		err = ::FSClose(fileRef);
		fileRef = 0;
		
		::ThrowIfError_AC(err);	
	}
	return done;
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* /*inSender*/)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kReceivedData:
		{
#if qNetTrace_AC
			if(inEvent->GetLongData() != fReceiveAmt)
				Debugger();

			TRACE << "Recieved " << inEvent->GetLongData() << " of " << fReceiveAmt << " bytes." << endl_AC;
#endif		
			GetStream().OffsetWritePosition( inEvent->GetLongData() );
				
			WriteChunkOfFile();
				// This will write out of the contents of the buffer.
		}
		break;
		
		case CNetEvent_AC::kIncomingData:
			if(IsAtState(kBlocking))
				ChangeStateTo(kProcessing);
		break;
	}
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::SetDataForkSize
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::SetDataForkSize(long inSize)
{
	if(!(fUsage & kFinderInfo))
	{
		fInfo.fDataForkSize = inSize;
		fExpectedAmt += inSize;
	}
}

//----------------------------------------------------------------------------------------
// CFileReceiver_AC::SetResourceForkSize
//----------------------------------------------------------------------------------------
	
void CFileReceiver_AC::SetResourceForkSize(long inSize)
{
	if(!(fUsage & kFinderInfo))
	{
		fInfo.fResForkSize = inSize;
		fExpectedAmt += inSize;
	}

}
