//----------------------------------------------------------------------------------------
// CFileSender_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

// Net includes

#ifndef __CFileSender_AC__
#include "CFileSender_AC.h"
#endif

#ifndef __CService_AC__
#include "CService_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

// stream includes

#if qNetTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif

// Patterns includes

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

// Standard Lib

#ifndef __cstring__
#include <string.h>
#endif

//----------------------------------------------------------------------------------------
// CFileSender_AC::CFileSender_AC
//----------------------------------------------------------------------------------------
	
CFileSender_AC::CFileSender_AC( const CFileSender_AC& inSender )
	: fSentDataFork(false),
	fReadNextChunkFromFile(false),
	fResForkRef(0),
	fDataForkRef(0)
{
	SetPriority(kDataPriority);
	SetIsData(true);

	*this = inSender;
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::CFileSender_AC
//----------------------------------------------------------------------------------------
	
CFileSender_AC::CFileSender_AC( const CFSSpec_AC& inSpec, 
								long fileUsage,
								CBuffer_AC* inBuffer, 
								bool doExpedite)
	: CFileTransaction_AC(inSpec, fileUsage, kTextType, kUnknownType, inBuffer, doExpedite),
	fSentDataFork(false),
	fReadNextChunkFromFile(false),
	fResForkRef(0),
	fDataForkRef(0)
{
	SetPriority(kDataPriority);
	SetIsData(true);
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::CFileSender_AC
//----------------------------------------------------------------------------------------
	
CFileSender_AC::CFileSender_AC( const CFSSpec_AC& inSpec, 
								long fileUsage,
								long bufferSize, 
								char* inPtr,
								bool doExpedite)
	: CFileTransaction_AC(inSpec, fileUsage, kTextType, kUnknownType, bufferSize, inPtr,doExpedite),
	fSentDataFork(false),
	fReadNextChunkFromFile(false),
	fResForkRef(0),
	fDataForkRef(0)
{
	SetPriority(kDataPriority);
	SetIsData(true);
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::~CFileSender_AC
//----------------------------------------------------------------------------------------
	
CFileSender_AC::~CFileSender_AC()
{
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::operator =
//----------------------------------------------------------------------------------------
	
CFileSender_AC& CFileSender_AC::operator = (const CFileSender_AC& inSender)
{
	CFileTransaction_AC::operator = (inSender);
	return *this;
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::Clone
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC *CFileSender_AC::Clone()
{
	return TH_new CFileSender_AC(*this);
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::Process
//----------------------------------------------------------------------------------------
	
void CFileSender_AC::Process( CServiceHandler_AC* inStream)
{

	OSErr err = 0;

	switch(GetState())
	{
		case kOpening:
			GetStream().RewindPositions();
		
			fExpectedAmt = 0;
			
			if( fUsage & kDataFork )
			{
				::Assertion_AC( fDataForkRef == 0 , NULL);
				
				err = ::FSpOpenDF(fFileSpec,fsRdPerm,&fDataForkRef);
				::ThrowIfError_AC(err);
					// open the file
			
				err = ::GetEOF(fDataForkRef, &fInfo.fDataForkSize);
				::ThrowIfError_AC(err);
				
				fSentDataFork = false;
				
				fExpectedAmt += fInfo.fDataForkSize;
			}
			else
				fSentDataFork = true;
				
			if( fUsage & kResourceFork )
			{
				::Assertion_AC( fResForkRef == 0 , NULL);
				
				err = ::FSpOpenRF(fFileSpec, fsRdPerm, &fResForkRef);
				::ThrowIfError_AC(err);
				
				err = ::GetEOF(fResForkRef, &fInfo.fResForkSize);
				::ThrowIfError_AC(err);
				
				fExpectedAmt += fInfo.fResForkSize;
			}
			if( fUsage & kFinderInfo ) 
			{
				FInfo finderInfo;
				err = ::FSpGetFInfo(fFileSpec,&finderInfo);
				::ThrowIfError_AC(err);
				
				fInfo.fFinderFlags = finderInfo.fdFlags;
					// save finder flags

				CStr63_AC name;
				fFileSpec.GetName(name);	
				::strcpy(fInfo.fName, name);
					// kinda kludgy cause TFile expects a CStr63 and thats not what were using.
					
				fInfo.fType = finderInfo.fdType;	
				fInfo.fCreator = finderInfo.fdCreator;
					
				GetStream() << fInfo;
					// stream our file information into the stream
			
				fExpectedAmt += fInfo.GetStreamedSize();
					// The amount streamed into the stream may not be the same
					// as the sizeof(fInfo) because of struct alignment issues,
					// so get the amount incremented from the stream. 
			}
			
			fReadNextChunkFromFile = true;
				// tell ourselves to read the next chunk
			
			ChangeStateTo(kProcessing);
			// fall through
				
		case kProcessing:
		{
			do
			{
				if(fReadNextChunkFromFile)
				{
					if(!fSentDataFork && (fUsage & kDataFork))
					{
						fSentDataFork = LoadNextChunk( fDataForkRef, fInfo.fDataForkSize );
					}
					if(fSentDataFork && (fUsage & kResourceFork))
					{
						LoadNextChunk( fResForkRef , fInfo.fResForkSize);
					}
					
					fReadNextChunkFromFile = false;
				}
			
				long flags = IsAtState(kProcessing) ? CService_AC::kMore : 0;
				
				::SetBits_AC(flags,CService_AC::kPriority, IsExpedited());  
				
				ChangeStateTo(kBlocking);
				
				inStream->SendData(GetStream().GetReadPositionPtr(), GetStream().GetAmountAvailable(), flags);
					// this may cause event which will cause the state to be changed.
					
			} while( IsAtState(kProcessing) ) ;
		}
		break;
		
		case kResuming:
			
			
			// What happens if we are suspended in the middle
			// of  a send, i.e. there's a kMore pending? What
			// happens when we resume? Hmm.
			
			if(fTransferedAmt < fExpectedAmt)
				ChangeStateTo(kBlocking);
			else
				Complete();
				
		break;
	}
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::LoadNextChunk
//----------------------------------------------------------------------------------------
	
bool CFileSender_AC::LoadNextChunk(short& fileRef, long inForkSize)
// This sends a chunk of data from either fork of a file.
{
	bool done = false;
	long mark;
	OSErr err = ::GetFPos(fileRef, &mark);

	long amtLeft = inForkSize - mark;
	
	if(amtLeft)
	{
		long amount = ::Min_AC(GetStream().GetAmountFree(), amtLeft);

		err = ::FSRead( fileRef, &amount, GetStream().GetWritePositionPtr() );
		::ThrowIfError_AC(err);
		
		amtLeft -= amount;
		
		GetStream().OffsetWritePosition(amount);
	}
	
	if(!amtLeft)
	{
		done = true;
	
		err = ::FSClose(fileRef);
		fileRef = 0;
		
		::ThrowIfError_AC(err);
	}
	
	return done;
}

//----------------------------------------------------------------------------------------
// CFileSender_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CFileSender_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* /*inSender*/)
{
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kResendData:
			fReadNextChunkFromFile = false; 
			ChangeStateTo(kProcessing);
		break;
	
		case CNetEvent_AC::kSentData:
		{
			long sentAmt = inEvent->GetLongData();
			
			if(sentAmt)
			{
				long wantedAmt = GetStream().GetAmountAvailable();
					// This is how much data we wanted to send,
					// we haven't incremented the position yet.
				
				Assertion_AC(sentAmt > 0, "Sent Zero amount of data?");
			
				fTransferedAmt += sentAmt;
					// increment by how much we actually sent
			
				if( fTransferedAmt == fExpectedAmt )
					Complete();
				else
				{
					ChangeStateTo(kProcessing);

					GetStream().OffsetReadPosition(sentAmt);
			
					if(GetStream().GetAmountAvailable() == 0)
					{
						Assertion_AC(wantedAmt == sentAmt, NULL);

						GetStream().RewindPositions();
							// reset the stream for the next read
							
						fReadNextChunkFromFile = true;
							// tell ourselves to read the next chunk of
							// file in Process.
					}
				}					
#if qNetTrace_AC
				TRACE << LowTracePriority_AC << "****Sent " << fTransferedAmt << " of " << fExpectedAmt << " bytes." << endl_AC;
#endif
			}
		}
		break;
	}
}
