//----------------------------------------------------------------------------------------
// CFileTransaction_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// net includes

#ifndef __CFileTransaction_AC__
#include "CFileTransaction_AC.h"
#endif

// stream includes

#ifndef __CBufferStream_AC__
#include "CBufferStream_AC.h"
#endif


//----------------------------------------------------------------------------------------
// operator >>
//----------------------------------------------------------------------------------------
	
CBufferStream_AC& operator >> (CBufferStream_AC& inStream, SFileInfo_AC& inFo)
{
	inStream.ReadBytes(inFo.fName, 32);
	inStream 	>> inFo.fType 
				>> inFo.fCreator 
				>> inFo.fFinderFlags 
				>> inFo.fDataForkSize 
				>> inFo.fResForkSize;					
	return inStream;
}

//----------------------------------------------------------------------------------------
// operator << 
//----------------------------------------------------------------------------------------
	
CBufferStream_AC& operator << (CBufferStream_AC& inStream, SFileInfo_AC& inFo)
{
	inStream.WriteBytes(inFo.fName, 32);
	inStream 	<< inFo.fType 
				<< inFo.fCreator 
				<< inFo.fFinderFlags 
				<< inFo.fDataForkSize 
				<< inFo.fResForkSize;					
	return inStream;
}

//----------------------------------------------------------------------------------------
// CFileTransaction_AC::CFileTransaction_AC
//----------------------------------------------------------------------------------------
	
CFileTransaction_AC::CFileTransaction_AC()
	: fUsage(kDataFork)
{
}

//----------------------------------------------------------------------------------------
// CFileTransaction_AC::CFileTransaction_AC
//----------------------------------------------------------------------------------------
	
CFileTransaction_AC::CFileTransaction_AC( 	const CFSSpec_AC& inSpec, 
							long fileUsage,
							OSType inType, 
							OSType inCreator,
							long bufferSize, 
							char* inPtr,
							bool doExpedite)
	: CNetworkTransaction_AC(bufferSize, inPtr, doExpedite),
	fUsage(fileUsage),
	fFileSpec(inSpec)
{
	fInfo.fCreator = inCreator;
	fInfo.fType = inType;
}

//----------------------------------------------------------------------------------------
// CFileTransaction_AC::CFileTransaction_AC
//----------------------------------------------------------------------------------------
	
CFileTransaction_AC::CFileTransaction_AC( 	const CFSSpec_AC& inSpec, 
											long inUsage,
											OSType inType,
											OSType inCreator,
											CBuffer_AC* inBuffer, 
											bool doExpedite )
	: CNetworkTransaction_AC(inBuffer, doExpedite),
	fUsage(inUsage),
	fFileSpec(inSpec)
{
	fInfo.fCreator = inCreator;
	fInfo.fType = inType;
}

//----------------------------------------------------------------------------------------
// CFileTransaction_AC::~CFileTransaction_AC
//----------------------------------------------------------------------------------------
	
CFileTransaction_AC::~CFileTransaction_AC()
{
}

//----------------------------------------------------------------------------------------
// CFileTransaction_AC::operator =
//----------------------------------------------------------------------------------------
	
CFileTransaction_AC& CFileTransaction_AC::operator = (const CFileTransaction_AC& inTransaction)
{
	CNetworkTransaction_AC::operator = (inTransaction);
	
	fFileSpec = inTransaction.fFileSpec;
	
	return *this;
}

//----------------------------------------------------------------------------------------
// CFileTransaction_AC::SetFileSpec
//----------------------------------------------------------------------------------------
	
void CFileTransaction_AC::SetFileSpec(const CFSSpec_AC& inSpec)
{
	fFileSpec = inSpec;
}

//----------------------------------------------------------------------------------------
// CFileTransaction_AC::GetFileSpec
//----------------------------------------------------------------------------------------
	
CFSSpec_AC& CFileTransaction_AC::GetFileSpec()
{
	return fFileSpec;

}
		
