//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// network includes

#ifndef __CMultiConnectionManager_AC__
#include "CMultiConnectionManager_AC.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __CServiceState_AC__
#include "CServiceState_AC.h"
#endif


#ifndef __CConnector_AC__
#include "CConnector_AC.h"
#endif

#ifndef __CAcceptor_AC__
#include "CAcceptor_AC.h"
#endif

#ifndef __CNetworkTransaction_AC__
#include "CNetworkTransaction_AC.h"
#endif

#ifndef __DisconnectTransactions_AC__
#include "DisconnectTransactions_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CNetworkAddress_AC__
#include "CNetworkAddress_AC.h"
#endif


// pattern includes

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

#ifndef __CReactorFactory_AC__
#include "CReactorFactory_AC.h"
#endif

// stream includes

#if qNetTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif

// container includes

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif


//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::CMultiConnectionManager_AC
//----------------------------------------------------------------------------------------
	
CMultiConnectionManager_AC::CMultiConnectionManager_AC()
{
	fConnections = TH_new CConnectionList;
	fReactorFactory = CReactorFactory_AC::GetDefaultFactory();
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::~CMultiConnectionManager_AC
//----------------------------------------------------------------------------------------
	
CMultiConnectionManager_AC::~CMultiConnectionManager_AC()
{
}

//----------------------------------------------------------------------------------------
//  CMultiConnectionManager_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	// Handle the events important for this class.
	// 1. A connection is made. Save a copy of the connection in the list.
	// 2. A connection is terminated. Remove the connection from the list.
	// 3. A timeout occured. Hangup up the connection. Remember timeouts are optional.
	
	switch(inEvent->GetEventID())
	{
		case CNetEvent_AC::kAcceptSucceeded:
		case CNetEvent_AC::kConnectSucceeded:
		{	
			ReceiveEvent_AC(inEvent);	
			CServiceHandlerPtr_AC connection = GetDataFromEvent_AC(inEvent,CServiceHandlerPtr_AC);

			fConnections->Push(connection); 
	
			RegisterForEventsFrom(connection);

			if(fServiceHandlerTimer)
				connection->EnableServiceHandlerTimer(fTimeOut);

#if qNetTrace_AC
			TRACE << LowTracePriority_AC << "New CServiceHandler_AC!" << endl_AC;	
#endif
		}
		break;
	
 		case CNetEvent_AC::kConnectionTerminated:
		{
			CServiceHandlerPtr_AC connection = dynamic_cast_AC( CServiceHandler_AC*, inTransceiver );
			fConnections->Remove(connection);
						
#if qNetTrace_AC
			TRACE << LowTracePriority_AC << "Received Disconnect: " << fConnections->GetLength() << " connections active." << endl_AC;
#endif
		}
		break;	
	
 		case CNetEvent_AC::kTimedOut:
		{
			CServiceHandlerPtr_AC connection = dynamic_cast<CServiceHandler_AC*>(inTransceiver );
			// Could be a timeout from another source.
			if(connection)
				connection->Send(TH_new CDisconnect_AC);
		}
		break;
	}
	
	MEventRouter_AC::ReceiveEvent(inEvent, inTransceiver);
		// this will send all the events to the subscribers.
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::Idle
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::Idle()
{
	try
	{
		static long num = 0;

		if(GetReactor())
		{
			if(num != fConnections->GetLength())
			{
				num = fConnections->GetLength();
#if qNetTrace_AC
				TRACE << LowTracePriority_AC << "Num connections: " << fConnections->GetLength() << endl_AC;
#endif
			}
		
			GetReactor()->Idle();
		}
	}
	catch(...)
	{
#if qNetTrace_AC
		TRACE << LowTracePriority_AC << "Caught Exception in CMultiConnectionManager_AC::Idle" << endl_AC;
#endif
	}
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::GetNumConnections
//----------------------------------------------------------------------------------------
	
long CMultiConnectionManager_AC::GetNumConnections()
{
	return fConnections->GetLength();
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::Hangup
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::Hangup()
{
	CConnectionList::CIterator iter = fConnections->First();

	while(iter.IsValid())
	{
		(*iter)->Send(TH_new CHangup_AC);
			// this tells the connection to hangup. We'll get a message in HandleDisconnectio
			// which will remove the connection from the list.
	
		iter.Next();
	}
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::GetConnector
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::GetConnector(CConnector_AC*& inConnectorPtr)
{
	if(	!fConnector)
	{
		fConnector = CreateConnector(fReactorFactory->CreateReactor(), fServiceHandlerFactory);
			// make our CConnector
			
		RegisterForEventsFrom(fConnector);
	}
	
	inConnectorPtr = fConnector;
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::GetAcceptor
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::GetAcceptor(CAcceptor_AC*& inAcceptor)
{
	if(	!fConnector)
	{
		fAcceptor = CreateAcceptor(fReactorFactory->CreateReactor(), fServiceHandlerFactory);
			// make our CConnector
			
		RegisterForEventsFrom(fAcceptor);
	}
	
	inAcceptor = fAcceptor;
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::FindConnectionBy
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC* CMultiConnectionManager_AC::FindConnectionBy(const CNetworkAddress_AC& inAddress)
{
	CConnectionList::CIterator iter = fConnections->First();
	
	CServiceHandler_AC* out = NULL;

	CAutoPtr_AC<CNetworkAddress_AC> address = inAddress.Clone();

	while(iter.IsValid())
	{
		CServiceState_AC* state = (*iter)->GetState();
		::AssertNotNULL_AC(state);

		state->GetRemoteAddress(*address);

		if(*address == inAddress)
		{
			out = *iter;
			break;
		}
		
		iter.Next();
	}

	return out;
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::SendTransactionTo
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::SendTransactionTo(const CNetworkAddress_AC& inAddress, CNetworkTransaction_AC* inBlock)
{
	::AssertNotNULL_AC(inBlock);
	
	CServiceHandler_AC* connection = FindConnectionBy(inAddress);

	::ThrowIfNULL_AC(connection);
	
	connection->Send(inBlock);
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::SendTransaction
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::SendTransaction(CNetworkTransaction_AC* inBlock)
{
	::AssertNotNULL_AC(inBlock);
	
	if(fConnections->GetLength() > 0)
	{
		
		long count = fConnections->GetLength();
		long i = 0;
		
		CConnectionList::CIterator iter = fConnections->First();
	
		CNetworkTransaction_AC* blkToSend = inBlock;
		
		for(long i =0; i < count; ++i)
		{
			if(i < (count-1))
				blkToSend = inBlock->Clone();
			else
				blkToSend = inBlock;
				
			::AssertNotNULL_AC(blkToSend);
		
			(*iter)->Send(blkToSend);
			
			iter.Next();
		}	
	}
	

}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::DisconnnectFrom
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::DisconnnectFrom(const CNetworkAddress_AC& inAddress)
{
	CServiceHandler_AC* connection = FindConnectionBy(inAddress);

	::ThrowIfNULL_AC(connection);
	
	connection->Send(TH_new CDisconnect_AC);
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::Disconnect
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::Disconnect()
{
	
	CConnectionList::CIterator iter = fConnections->First();

	CServiceHandler_AC* out = NULL;
	
	while(iter.IsValid())
	{
		(*iter)->Send(TH_new CDisconnect_AC);
		iter.Next();
	}
}

//----------------------------------------------------------------------------------------
// CMultiConnectionManager_AC::HangupOn
//----------------------------------------------------------------------------------------
	
void CMultiConnectionManager_AC::HangupOn(const CNetworkAddress_AC& inAddress)
{
	CServiceHandler_AC* connection = FindConnectionBy(inAddress);

	::ThrowIfNULL_AC(connection);
	
	connection->Send(TH_new CHangup_AC);
}



