//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Network includes

#ifndef __CNetworkTransaction_AC__
#include "CNetworkTransaction_AC.h"
#endif

// pattern includes

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

// core includes

#if qNetDebug_AC
#include "dout_AC.h"
#endif


//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::CNetworkTransaction_AC
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC::CNetworkTransaction_AC(bool doExpedite)
	: fFlags(0),
	fTransferedAmt(0),
	fExpectedAmt(0),
	fPriority(0)
{
	Expedite(doExpedite);
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::CNetworkTransaction_AC
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC::CNetworkTransaction_AC(const CNetworkTransaction_AC& inBlock)
	: fFlags(0),
	fTransferedAmt(0),
	fExpectedAmt(0),
	fPriority(0)
{
	*this = inBlock;
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::CNetworkTransaction_AC
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC::CNetworkTransaction_AC(CBuffer_AC* inBuffer, bool doExpedite)
	: fFlags(0),
	fTransferedAmt(0),
	fExpectedAmt(0),
	fPriority(0)
{
	Expedite(doExpedite);
	SetBuffer(inBuffer);
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::CNetworkTransaction_AC
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC::CNetworkTransaction_AC(unsigned long inSize, char* inPtr, bool doExpedite)
	: fFlags(0),
	fTransferedAmt(0),
	fExpectedAmt(0),
	fPriority(0)
{
	Expedite(doExpedite);
	SetBuffer( TH_new CBuffer_AC(inSize, inPtr));
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::~CNetworkTransaction_AC
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC::~CNetworkTransaction_AC()
{
	ReleaseBuffer();
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::SetBuffer
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::SetBuffer(CBuffer_AC* inBuffer)
{
	fBuffer = inBuffer;
	fTransferedAmt = 0;
	SyncStreamWithBuffer();
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::SetBuffer
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::SetBuffer(unsigned long inSize, char* inPtr)
{
	SetBuffer( TH_new CBuffer_AC(inSize, inPtr));
		// This will:
		//	 	release old buffer
		//	 	reset iterator
		// 		aquire inPtr if not NULL
		// 		make a new buffer of inSize of inPtr != NULL
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::operator =
//----------------------------------------------------------------------------------------
	
CNetworkTransaction_AC& CNetworkTransaction_AC::operator = (const CNetworkTransaction_AC& inBlock)
{
	CopyBuffer(inBlock.fBuffer);
		// first copy the buffers.

	if(inBlock.fBuffer)
	{
		
		if(fStream && inBlock.fStream)
		{
			fStream->SetWritePosition( inBlock.fStream->GetWritePosition() );
			fStream->SetReadPosition( inBlock.fStream->GetReadPosition() );
		}
	}

	fExpectedAmt = inBlock.fExpectedAmt;
	fTransferedAmt = inBlock.fTransferedAmt;
	fFlags = inBlock.fFlags;
	fPriority = inBlock.fPriority;
	
	

	return *this;
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::GetBuffer
//----------------------------------------------------------------------------------------
	
CBuffer_AC* CNetworkTransaction_AC::GetBuffer() 
{ 
	return fBuffer; 
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::ReleaseBuffer
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::ReleaseBuffer()
{	
	fBuffer = NULL; 
	SyncStreamWithBuffer();
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::Expedite
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::Expedite(bool isHighPriority)
{		
	::SetBits_AC(fFlags, kExpedited, isHighPriority ); 
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::Open
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::Open()
{
	ChangeStateTo(kOpening);
	
	fTransferedAmt = 0;
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::ReceiveEvent(CEvent_AC* /*inEvent*/, MEventTransceiver_AC* /*inSender*/)
{
}	

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::CopyBuffer
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::CopyBuffer(CBuffer_AC* inBuffer)
{
	if(!inBuffer)
		ReleaseBuffer();
	else
	{
		if(!fBuffer)
			fBuffer = TH_new CBuffer_AC(*inBuffer);
		else
			*fBuffer = *inBuffer;

		SyncStreamWithBuffer();
	}
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::SetIsData
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::SetIsData(bool inBool)
{
	::SetBits_AC(fFlags, kData, inBool);
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::SyncStreamWithBuffer
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::SyncStreamWithBuffer()
{
	GetStream().SetBuffer(fBuffer);
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::Complete
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::Complete()
{
	ChangeStateTo(kComplete);

	if(fStream) fStream->ResetBufferSize();
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::SetStream
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::SetStream(CBufferStream_AC* inStream)
{
	fStream = inStream;
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::ReleaseStream
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::ReleaseStream()
{
	fStream = NULL;
}

//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC::MakeStream
//----------------------------------------------------------------------------------------
	
void CNetworkTransaction_AC::MakeStream()
{
	fStream = TH_new CBufferStream_AC(fBuffer);
}
