//----------------------------------------------------------------------------------------
// CServiceHandler_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Network include

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __CNetworkTransaction_AC__
#include "CNetworkTransaction_AC.h"
#endif

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CService_AC__
#include "CService_AC.h"
#endif

// Pattern Includes

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEventTransceiverTimer_AC__
#include "CEventTransceiverTimer_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

// container includes

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

// streams includes

#if qNetTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::CServiceHandler_AC
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC::CServiceHandler_AC(CReactor_AC* inReactor)
	: MEventTransceiver_AC(inReactor),
	fOpen(false),
	fUserData(0)
{
	fTransactions = TH_new CList;
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::~CServiceHandler_AC
//----------------------------------------------------------------------------------------
	
CServiceHandler_AC::~CServiceHandler_AC()
{
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::Send
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::Send(CNetworkTransaction_AC* inMessage)
{
	inMessage->SetIsSender(true);
	QueueTransaction(inMessage);
	Idle();
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::Receive
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::Receive(CNetworkTransaction_AC* inMessage)
{
	inMessage->SetIsSender(false);
	QueueTransaction(inMessage);
	Idle();
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::QueueTransaction
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::QueueTransaction(CNetworkTransaction_AC* inTransaction)
{
	if(inTransaction->IsExpedited())
	{
		CRefCountingPtr_AC<CNetworkTransaction_AC>& currentTransaction = 
			inTransaction->IsSender() ? fSender : fReceiver ;
			
		if(currentTransaction && !currentTransaction->IsExpedited())
		{
			// if the currently executing transaction is not expidited, assume that
			// there are no expidited transactions in the queue.
		
				// preempt the old one
			currentTransaction->ChangeStateTo(CNetworkTransaction_AC::kSuspending);
			currentTransaction->Process(this);
			currentTransaction.reset();
			
				// now push new transaction on head of queue
			fTransactions->Push(currentTransaction);
			fTransactions->Push(inTransaction);
		}
		else
		{
			// We need to insert the transaction at the end of any currently queue
			// expidited transactions and before the first nonexpidited transation.
	
			bool didInsert = false;
		
			CList::CIterator iter = fTransactions->First();
	
			while (iter.IsValid())
			{
				// look for the first non expidited transaction
				
				if(!(*iter)->IsExpedited())
				{
					fTransactions->InsertAfter(iter, CRefCountingPtr_AC<CNetworkTransaction_AC>(inTransaction));
					didInsert = true;
					break;
				}
				
				iter.Next();
			}
			
			if(!didInsert)
				fTransactions->Append(CRefCountingPtr_AC<CNetworkTransaction_AC>(inTransaction));
		}
	}
	else
	{
		fTransactions->Append(CRefCountingPtr_AC<CNetworkTransaction_AC>(inTransaction));
	}
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::ReceiveEvent
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inTransceiver)
{
	switch(inEvent->GetEventID())
	{	
		case CNetEvent_AC::kConnectionTerminated:
			PostEvent(inEvent);
			break;
			
		case CNetEvent_AC::kIncomingData:
		{
			ReceiveEvent_AC(inEvent);

			if(	!fReceiver || 
				!fReceiver->IsData() ||	
				(fReceiver->GetTransferedAmount() == fReceiver->GetExpectedTransferAmount()) )
			{		
				PostEvent(inEvent);
					// propagate kIncomingData event to handlers - we fully expect handlers
					// to hand us a Transaction to Receive the data
			}
			else
			if(fReceiver) 
			{
				try
				{
					fReceiver->ReceiveEvent(inEvent, inTransceiver);
				}
				catch(CException_AC& inException)
				{				
					HandleTargetError(fReceiver, inException.GetError());
				}
			}			
		}
		break;	
		
		case CNetEvent_AC::kSentData:
			ReceiveEvent_AC(inEvent);
			
			if(fSender)
			{	
				try
				{
					fSender->ReceiveEvent(inEvent, inTransceiver);
					PostEvent( CreateEventWithData_AC( CNetEvent_AC::kSentData, fSender) );
				}
				catch(CException_AC& inException)
				{				
					HandleTargetError(fSender, inException.GetError());
				}			
			}
			else
				PostEvent( inEvent );
		break;
		
		case CNetEvent_AC::kReceivedData:
			
			ReceiveEvent_AC(inEvent);
			
			if(fReceiver)
			{
				try
				{
					fReceiver->ReceiveEvent(inEvent, inTransceiver);
					PostEvent( CreateEventWithData_AC( CNetEvent_AC::kReceivedData, fReceiver) );
				}
				catch(CException_AC& inException)
				{				
					HandleTargetError(fReceiver, inException.GetError());
				}			
			}
			else
				PostEvent( inEvent );

		break;
		
		case CNetEvent_AC::kError:
		case CNetEvent_AC::kTimedOut:
			PostEvent(inEvent);
		break;
		
		default:
			MEventTransceiver_AC::ReceiveEvent(inEvent, inTransceiver);
		break;

	}


}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::FindNextTransaction
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::FindNextTransaction(CRefCountingPtr_AC<CNetworkTransaction_AC>& inPtr, bool inIsSender)
{
	CList::CIterator iter = fTransactions->First();
	
	while(iter.IsValid())		
	{
		if(inIsSender == (*iter)->IsSender())
		{
			inPtr = iter;
			break;
		}

		iter.Next();
	}
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::OpenTransaction
//----------------------------------------------------------------------------------------
	
bool CServiceHandler_AC::OpenTransaction(CRefCountingPtr_AC<CNetworkTransaction_AC>& inTransaction)
{
	if(inTransaction)
	{
		CRefCountingPtr_AC<CNetworkTransaction_AC> toCompareAgainst = inTransaction->IsSender() ? fReceiver : fSender;
		
		if(	!toCompareAgainst)
		{
			// If we get here, we don't have a current executing transaction to compare
			// against, so let's look to see if there is one in the queue. We'll get 
			// the topmost transaction that is the counterpart to inTransaction, i.e. 
			// if inTransaction is a sender, we need to get a Receiver.
		
			FindNextTransaction(toCompareAgainst, !inTransaction->IsSender());
		}
		
		if(	!toCompareAgainst || 
			(toCompareAgainst->GetPriority() == inTransaction->GetPriority()))
		{
			// If we get here and toCompareAgainst == NULL, either there is no
			// transaction executing or we coudn't find one in the queue.
			// Otherwise, the priority of the currently executing transaction 
			// or the next counterpart transaction that is going to be 
			// excuted, is the same as inTransaction.

			fTransactions->Remove(inTransaction);
				// remove the transaction from the queue, cause inTransaction
				// now owns it and we've cleared it for takeoff.
		
			switch(inTransaction->GetState())
			{
				case  CNetworkTransaction_AC::kComplete:
					inTransaction->Open();
						// Start the transaction
				break;
				
				case  CNetworkTransaction_AC::kSuspending:
					inTransaction->ChangeStateTo(CNetworkTransaction_AC::kResuming);
						// Resume the transaction
				break;
			}
			
		}
		else
		{
			inTransaction = NULL;
			// Nope, can't do it, one of the above conditions failed.
		}
	}
	
	return inTransaction;
}

//----------------------------------------------------------------------------------------
//  CServiceHandler_AC::Idle
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::Idle()
{		
	if(fOpen)
	{
		bool doLoop;
			
		do
		{
			doLoop = false;
			
			ProcessTarget(fSender);

			ProcessTarget(fReceiver);
				// process both first, in arbitrary order.
		
			if(!fSender)
			{
				FindNextTransaction(fSender, true);
				if(OpenTransaction(fSender))
					doLoop = true; // don't turn off doLoop
			}
			if(!fReceiver)
			{
				FindNextTransaction(fReceiver, false);
				if(OpenTransaction(fReceiver))
					doLoop = true; // don't turn off doLoop
			}
			
		
		} while(doLoop);
	}
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::ProcessTarget
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::ProcessTarget(CRefCountingPtr_AC<CNetworkTransaction_AC>& inTarget)
{
	if(	inTarget &&
		!inTarget->IsAtState(CNetworkTransaction_AC::kBlocking)) 
	{
		try
		{
			if( inTarget->IsAtState(CNetworkTransaction_AC::kComplete) ||
				inTarget->IsAtState(CNetworkTransaction_AC::kAbort))
			{
				inTarget.reset();
			}
			else
			{
				inTarget->Process(this);
				
				if(inTarget && inTarget->IsAtState(CNetworkTransaction_AC::kError))
				{
					inTarget->ChangeStateTo( CNetworkTransaction_AC::kAbort );
				}
			}
		}
		catch(CException_AC& inException)
		{				
			HandleTargetError(inTarget, inException.GetError());
		}
	}
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::HandleTargetError
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::HandleTargetError(CRefCountingPtr_AC<CNetworkTransaction_AC>& inTarget, CError_AC inError)
{
	CError_AC err = inTarget->ErrorOccurred(inError);
			
	if(err)
	{
		inTarget->SetError(err);
		inTarget->ChangeStateTo( CNetworkTransaction_AC::kError );
	}
}

//----------------------------------------------------------------------------------------
//  CServiceHandler_AC::Open
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::Open()
{
	if(!fOpen)
	{
		::AssertNotNULL_AC(GetReactor());
		fService = CreateService(GetReactor());

		RegisterForEventsFrom(fService);

		GetIdleTimeFrom(GetReactor());
		
		fOpen = true;
		
		if(IsTimingServiceHandler())
			fService->GetTimer()->Start();
	}
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::EnableServiceHandlerTimer
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::EnableServiceHandlerTimer(long timeOut)
{
	fService->EnableServiceHandlerTimer(timeOut, fOpen);
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::DisableServiceHandlerTimer
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::DisableServiceHandlerTimer()
{
	fService->DisableServiceHandlerTimer();
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::IsTimingServiceHandler
//----------------------------------------------------------------------------------------
	
bool CServiceHandler_AC::IsTimingServiceHandler() const
{
	return fService->IsTimingServiceHandler();
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::GetState
//----------------------------------------------------------------------------------------
	
CServiceState_AC* CServiceHandler_AC::GetState() const
{
	return fService->GetState();
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::SetUserData
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::SetUserData(void* inData)
{
	fUserData = inData;
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::GetUserData
//----------------------------------------------------------------------------------------
	
void* CServiceHandler_AC::GetUserData() const
{
	return fUserData;
}

//----------------------------------------------------------------------------------------
// CServiceHandler_AC::AbortAllTransactions
//----------------------------------------------------------------------------------------
	
void CServiceHandler_AC::AbortAllTransactions()
{
	
}