// CTextStyle_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.

#ifndef __CTextStyle_AC__
#define __CTextStyle_AC__


#ifndef __Toolbox_AC__
#include "Toolbox_AC.h"
#endif

// ACS

#ifndef __CRGBColor_AC__
#include "CRGBColor_AC.h"
#endif

// Toolbox
 
#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

//----------------------------------------------------------------------------------------
// Enumeration (boolean) for preferring outline fonts
//----------------------------------------------------------------------------------------

enum { kDontPreferOutlineFonts, kPreferOutlineFonts };

//----------------------------------------------------------------------------------------
// CTextStyle_AC
//----------------------------------------------------------------------------------------
class EXPORT_AC CTextStyle_AC : public TextStyle
{
public:
	// CONSTRUCTORS:

	inline CTextStyle_AC() { Clear(); }
	inline CTextStyle_AC(short theFont, short theFace, short theSize, const CRGBColor_AC& theColor) { Set(theFont, theFace, theSize, theColor); }
	inline CTextStyle_AC(const CTextStyle_AC& textStyle) { Set(textStyle); }
	inline CTextStyle_AC(const TextStyle& textStyle) { Set(textStyle); }
	inline CTextStyle_AC(GrafPtr grafPort) { Set(grafPort); }

	inline CTextStyle_AC& operator=(const CTextStyle_AC& textStyle) { Set(textStyle); return *this; }
		
	// CONVERSION:
	
	inline operator TextStyle*() { return this; }
	inline operator const TextStyle*() const { return this; }
	inline operator TextStyle&() { return *this; }
	inline operator const TextStyle&() const { return *this; }
	
	// COMPARISON:

	bool operator ==(const CTextStyle_AC& textStyle) const;
	inline bool operator !=(const CTextStyle_AC& textStyle) const { return !(*this == textStyle); }
	
	// UTILITY:

	inline void Clear()
#if qPowerPC
	{ *(double*)this = 0; ((CRGBColor_AC*)&tsColor)->Clear(); }
#else
	{ *(long*)this = 0; tsSize = 0; ((CRGBColor_AC*)&tsColor)->Clear(); }
#endif
	// cheat

	void Set(const TextStyle& textStyle);
	void Set(short theFont, short theFace, short theSize, const CRGBColor_AC& theColor);
	void Set(GrafPtr grafPort);
	
	// QuickDraw extensions
	
	void Normalize(SInt16 systemFontNum, SInt16 applFontNum);
		// Convert magic font number/size into actual font number/size used by the system.
		
	void SetPortTextStyle() const;
		// Set the current qd port text style to our text style information, if it is different.

	void GetPortTextStyle();
		// Set our text style information to the current port.
		
	inline void PrepareForDrawing() const
		{ SetPortTextStyle(); }
		// Alias for developers who prefer this function name
		
	short SetKeyScript() const;
		// Set the key script to correspond to our font, returning the previous script.
	
	static short RuntimeJustify(short userJust);
		// Return the actual runtime justification value for
		// the supplied value.
	
	static const CTextStyle_AC kSystemStyle;
	static const CTextStyle_AC kApplicationStyle;
	static const CTextStyle_AC kApplicationSmallStyle;
};

//----------------------------------------------------------------------------------------
// CLASS CWhileOutlinePreferred_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CWhileOutlinePreferred_AC
{
	bool fPreferOutline;
	bool fOutlinePreferredChanged;

public:
	// Constructor/destructor
	CWhileOutlinePreferred_AC(bool preferOutline);
	~CWhileOutlinePreferred_AC();
	
};

#endif // __CTextStyle_AC__
