//----------------------------------------------------------------------------------------
// CTextStyle_AC.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif

// ACS

// Toolbox

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif



//========================================================================================
// GLOBAL STextStyles
//========================================================================================

const CTextStyle_AC CTextStyle_AC::kSystemStyle(::GetSysFont(), normal, 0, CRGBColor_AC::kRGBBlack);
const CTextStyle_AC CTextStyle_AC::kApplicationStyle(::GetAppFont(), normal, 0, CRGBColor_AC::kRGBBlack);
const CTextStyle_AC CTextStyle_AC::kApplicationSmallStyle(::GetAppFont(), normal, 9, CRGBColor_AC::kRGBBlack);

//========================================================================================
// CLASS CTextStyle_AC
//========================================================================================

//----------------------------------------------------------------------------------------
//		 Set
//----------------------------------------------------------------------------------------
	
void CTextStyle_AC::Set(const TextStyle& textStyle)
{
	*static_cast<TextStyle*>(this) = *static_cast<const TextStyle*>(&textStyle);
}

//----------------------------------------------------------------------------------------
//		 Set
//----------------------------------------------------------------------------------------
	
void CTextStyle_AC::Set(short theFont,
					  short theFace,
					  short theSize,
					  const CRGBColor_AC& theColor)
{
	tsFont = theFont;
	tsFace = theFace;
	tsSize = theSize;
	tsColor = theColor;
}

//----------------------------------------------------------------------------------------
//		 Set
//----------------------------------------------------------------------------------------

void CTextStyle_AC::Set(GrafPtr grafPort)
{
	tsFont = grafPort->txFont;
	tsFace = grafPort->txFace;
	tsSize = grafPort->txSize;
	tsColor = (CRGBColor_AC &) ((CGrafPtr) grafPort)->rgbFgColor; // ??? need colorport test?
}

//----------------------------------------------------------------------------------------
// Comparison operators for CTextStyle_AC
//----------------------------------------------------------------------------------------
	
bool CTextStyle_AC::operator ==(const CTextStyle_AC& textStyle) const
{
	return tsFont == textStyle.tsFont && tsFace == textStyle.tsFace && tsSize == textStyle.tsSize && ((CRGBColor_AC)tsColor) == ((CRGBColor_AC)textStyle.tsColor);
}

//----------------------------------------------------------------------------------------
// Normalize: 
//----------------------------------------------------------------------------------------

void CTextStyle_AC::Normalize(SInt16 systemFontNum, SInt16 applFontNum)
{
	if (tsFont == systemFontNum)
		tsFont = ::GetSysFont();//systemFont;
	else if (tsFont == applFontNum)
		tsFont = ::GetAppFont();//applFont;

}

//----------------------------------------------------------------------------------------
// SetPortTextStyle: 
//----------------------------------------------------------------------------------------

void CTextStyle_AC::SetPortTextStyle() const
{
	// Used to try to keep from setting the font all the time but the script may have changed
	// so if the font is applFont or sysFont we always force the change since they can
	// be different fonts for different scripts.
	if (tsFont == applFont || tsFont == systemFont || qd.thePort->txFont != tsFont)
		TextFont(tsFont);

	if (qd.thePort->txFace != tsFace)
		::TextFace(tsFace);
	if (qd.thePort->txSize != tsSize)
		::TextSize(tsSize);
	
	CRGBColor_AC(tsColor).SetForeColor();
}

//----------------------------------------------------------------------------------------
// GetPortTextStyle: 
//----------------------------------------------------------------------------------------
	
void CTextStyle_AC::GetPortTextStyle()
{
	// Set our text style information to the current port.
	GrafPtr	currentPort;

	::GetPort(&currentPort);
	Set(currentPort);
}

//----------------------------------------------------------------------------------------
// SetKeyScript: 
//----------------------------------------------------------------------------------------

short CTextStyle_AC::SetKeyScript() const
{
	short newKeyScript = FontToScript(tsFont);
	short currentKeyScript = static_cast<short>(GetScriptManagerVariable(smKeyScript));
	if (currentKeyScript != newKeyScript)		// Don't unnecessarily annoy the script mgr 
		::KeyScript(newKeyScript);

	return currentKeyScript;
}

//----------------------------------------------------------------------------------------
// RuntimeJustify: 
//----------------------------------------------------------------------------------------

short CTextStyle_AC::RuntimeJustify(short userJust)
{
	// Return the actual runtime justification value for
	// the supplied value. The caller can use the result
	// alone to know determine drawing direction.
	// If the supplied value specifies default, we need
	// to check the current system script direction.
	// Otherwise, whatever was specified is OK as is.

	if (userJust == teFlushDefault)
	{
		if (::GetSysDirection() == 0)
			return teFlushLeft;
		else
			return teFlushRight;
	}
	else
		return userJust;
}

//========================================================================================
// CLASS CWhileOutlinePreferred_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CWhileOutlinePreferred_AC::CWhileOutlinePreferred_AC: 
//----------------------------------------------------------------------------------------

CWhileOutlinePreferred_AC::CWhileOutlinePreferred_AC(bool preferOutline) :
	fPreferOutline(preferOutline),
	fOutlinePreferredChanged((GetOutlinePreferred() != preferOutline))
{
	// change outline preferred, if necessary
	if (fOutlinePreferredChanged)
		SetOutlinePreferred(fPreferOutline);
}

//----------------------------------------------------------------------------------------
// CWhileOutlinePreferred_AC::~CWhileOutlinePreferred_AC: 
//----------------------------------------------------------------------------------------

CWhileOutlinePreferred_AC::~CWhileOutlinePreferred_AC()
{
	// restore outline preferred, if necessary
	if (fOutlinePreferredChanged)
		SetOutlinePreferred(!fPreferOutline);
}
