//----------------------------------------------------------------------------------------
// CAbstractThreadSchedule_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------


#ifndef __CAbstractThreadSchedule_AC__
#define __CAbstractThreadSchedule_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CAbstractMonitorLock_AC;

//----------------------------------------------------------------------------------------

typedef int RelativePriority;
typedef int AbsolutePriority;

typedef unsigned long SchedulingPolicy;
typedef unsigned long PriorityLevels;

//----------------------------------------------------------------------------------------
// Class CAbstractThreadSchedule_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CAbstractThreadSchedule_AC 
{
public:
	virtual ~CAbstractThreadSchedule_AC();

protected:

	CAbstractThreadSchedule_AC();
	
	CAbstractThreadSchedule_AC( const CAbstractThreadSchedule_AC& aThreadSchedule );
		// Copy Constructor

	CAbstractThreadSchedule_AC& operator = ( const CAbstractThreadSchedule_AC& aThreadSchedule );

public:
	
	virtual CAbstractThreadContext_AC* CreateThreadContext( size_t stackSize, SimpleTaskProcPtr entry, void* parameter,
			CThread_AC* adoptedThread = NULL) = 0;
		// Create the appropriate thread context.
	
	virtual CAbstractMonitorLock_AC* CreateMonitorLock() = 0;
		// Create the appropriate monitor lock.

public:
	static CAbstractThreadSchedule_AC* CreateThreadSchedule();
		// Create a platform-specific default thread schedule

	// The following methods are for internal use only, they are really private
	// CThreadHandle_AC::SetThreadSchedule needs access to these members

	virtual AbsolutePriority GetPriority() const = 0 ;
	virtual SchedulingPolicy GetPolicy() const = 0;
	virtual AbsolutePriority GetMaximumPriority() const = 0;
	virtual AbsolutePriority GetMinimumPriority() const = 0;
	virtual void SetPriority (const AbsolutePriority thePriority) = 0;
};

#endif