//----------------------------------------------------------------------------------------
// CMPAllocator_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CMPAllocator_AC__
#define __CMPAllocator_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#ifndef MSIPL_DEFMEMORY_H
#include <mmemory.h>
#endif

#if qWantsMP

#ifndef MP_h
#include <MP.h>
#endif

//========================================================================================
//	Class CMPAllocator_AC provides an STL allocator class using th MP allocation APIs.
//========================================================================================

template <class T>
class CMPAllocator_AC {
public:
    typedef T value_type;
    typedef T* pointer;
    typedef const T* const_pointer;
    typedef T& reference;
    typedef const T& const_reference;
    typedef size_t size_type;
    typedef ptrdiff_t difference_type;
//	pointer allocate(size_type n) { 
//	return ::allocate((difference_type)n, (pointer)0);
//	}
    pointer allocate(size_type n) { 
	return (::HasMP_AC() ? (pointer)::MPAllocate((ByteCount)(n * sizeof(T))) :
									::allocate((difference_type)n, (pointer)0));
    }
//	void deallocate(pointer p) { ::deallocate(p); }
    void deallocate(pointer p) { (::HasMP_AC() ? ::MPFree((LogicalAddress)p) : ::deallocate(p)); }
    pointer address(reference x) { return (pointer)&x; }
    const_pointer const_address(const_reference x) { 
	return (const_pointer)&x; 
    }
    size_type init_page_size() { 
	return Max_AC(size_type(1), size_type(4096/sizeof(T))); 
    }
    size_type max_size() const { 
	return Max_AC(size_type(1), size_type(ULONG_MAX/sizeof(T))); 
    }
};

class EXPORT_AC CMPAllocator_AC<void> {
public:
    typedef void* pointer;
};

#endif

#endif