//----------------------------------------------------------------------------------------
// CMPSchedule_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------


#ifndef __CMPSchedule_AC__
#define __CMPSchedule_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#ifndef __CAbstractThreadSchedule_AC__
#include "CAbstractThreadSchedule_AC.h"
#endif

#if qWantsMP

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CThread_AC;

//----------------------------------------------------------------------------------------
// CMPSchedule implements a specialized schedule for MP tasks
//----------------------------------------------------------------------------------------

class EXPORT_AC CMPSchedule_AC : public CAbstractThreadSchedule_AC 
{
public:

	CMPSchedule_AC();
	// Characterstic: responsive, they handle the
	// application's human interface. For application
	// processing that takes an appreciable amount of
	// time, helper tasks with CApplicationSchedule_AC
	// should be used. See CAbstractThreadSchedule_AC before using this class.

	virtual ~CMPSchedule_AC();

	CMPSchedule_AC(const CMPSchedule_AC& anMPSchedule);

	CMPSchedule_AC& operator = (const CMPSchedule_AC& anMPSchedule);

	virtual CAbstractThreadContext_AC* CreateThreadContext(size_t stackSize, SimpleTaskProcPtr entry, void* parameter,
			CThread_AC* adoptedThread = NULL);

	virtual CAbstractMonitorLock_AC* CreateMonitorLock();

private:

	static const PriorityLevels		kNumberOfLevels;
	static const AbsolutePriority	kMaximumPriority;
	static const AbsolutePriority	kMinimumPriority;
	static const SchedulingPolicy	kPolicy;
	static const AbsolutePriority	kDefaultPriority;
	AbsolutePriority				fPriority;

public:
	// Noose methods
	virtual AbsolutePriority 		GetPriority() const;
	virtual SchedulingPolicy  		GetPolicy() const;
	virtual AbsolutePriority 		GetMaximumPriority() const;
	virtual AbsolutePriority 		GetMinimumPriority() const;
	virtual void 					SetPriority(const AbsolutePriority thePriority);

};

#endif // qWantsMP

#endif // CMPSchedule_AC
