//----------------------------------------------------------------------------------------
// CMPThreadContext_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CMPThreadContext_AC__
#define __CMPThreadContext_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#if qWantsMP

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

#ifndef __CMPAllocator_AC__
#include "CMPAllocator_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CAbstractMonitorLock_AC__
#include "CAbstractMonitorLock_AC.h"
#endif



class EXPORT_AC CMPThreadContext_AC : public CAbstractThreadContext_AC
// Concrete class providing context functionality for tasks using the MP API.
{
public:
	static CMPThreadContext_AC* GetCurrentMPContext();
		// Returns a pointer to the context object for the MP task currently executing.
		// If the current execution context is not an MP task, a special context value
		// stored in fMainContext will be returned.

	CMPThreadContext_AC(
			TaskProc		entryPoint,
			void			*parameter,
			ByteCount		stackSize,
			MPQueueID		notifyQueue,
			void 			*terminationParameter1,
			void 			*terminationParameter2,
			MPTaskOptions	options,
			CThread_AC* adoptedThread = NULL);
			
	CMPThreadContext_AC(
			SimpleTaskProcPtr	entryPoint,
			void			*parameter,
			ByteCount		stackSize,
			MPQueueID		notifyQueue,
			void 			*terminationParameter1,
			void 			*terminationParameter2,
			MPTaskOptions	options,
			CThread_AC* adoptedThread = NULL);
			
	CMPThreadContext_AC(
			MPTaskID		taskID);
		// Constructor used on a task that is already running.  Used to create the
		// fMainContext object for the main context, which is not truly an MP task.
		
	virtual ~CMPThreadContext_AC();
	
	virtual void	Suspend();	// Override

	virtual void  	Resume();	// Override
	
	virtual bool	IsAlive();	// Override
	
	virtual void 	Yield();	// Override
	
	MPSemaphoreID 	GetLock();	// Getter returns a lock used for suspending the context.
	
	MPTaskID		GetTaskID();// Getter for the MPTaskID for the task represented by
								// this context.
								
	static long		GetCurrentTaskCount();
								// Returns the number of tasks for which contexts 
								// currently exist, including tasks that have completed
								// execution but for which there are still outstanding
								// thread handles.  The main context is not included in
								// this count, as it is not technically an MP task.

protected:

	static OSStatus MPWrapperProc(CMPThreadContext_AC* context);
		// Routine used for interface translation for calling the functions to execute in
		// the task.

private:
	MPTaskID			fTaskID;	// ID for the MP task represented by this context.
	MPSemaphoreID		fLock;		// Lock used for suspending the context.
	SimpleTaskProcPtr 	fEntry;		// Entry point to be called in the task.
	void*				fParameter;	// Parameter to be passed to the entry point.
	bool				fDone;		// Flag indicating if the task has completed execution.

	static CAutoPtr_AC<map<MPTaskID,CMPThreadContext_AC* ,less<MPTaskID> ,CMPAllocator_AC<CMPThreadContext_AC*> > > fMap;
		// Dictionary of all outstanding MP contexts.

	static CAutoPtr_AC<CAbstractMonitorLock_AC>	fMapLock;
		// Lock for controlling access to fMap, the dictionary of outstanding MP contexts.

public:
	static CAutoPtr_AC<CMPThreadContext_AC> fMainContext;
		// Context object representing the main, non-MP, context.
};

#endif


#endif
