//----------------------------------------------------------------------------------------
// CMonitorCondition_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------

#ifndef __CMonitorCondition_AC__
#define __CMonitorCondition_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#ifndef __CAbstractMonitorLock_AC__
#include "CAbstractMonitorLock_AC.h"
#endif

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

#ifndef __CTime_AC__
#include "CTime_AC.h"
#endif

//----------------------------------------------------------------------------------------
//      Forward declarations and includes
//----------------------------------------------------------------------------------------

class   CAbstractMonitorLock_AC;
class	CAbstractThreadContext_AC;
class	CTime_AC;

template <class T>
class allocator;
#ifdef MSIPL_DEF_TEMPARG
template <class T, class Allocator = allocator<T> > 
#else
template <class T, class Allocator> 
#endif
class deque;


//----------------------------------------------------------------------------------------
//      CMonitorCondition_AC
//
//      Class to implement condition variables associated with a monitor.
//----------------------------------------------------------------------------------------

class EXPORT_AC   CMonitorCondition_AC 
{

public:

    CMonitorCondition_AC(CAbstractMonitorLock_AC*  theLock);

    CMonitorCondition_AC(CAbstractMonitorLock_AC*  theLock, const CTime_AC& maximumDelay);

    virtual ~CMonitorCondition_AC();

    // Note:  can all methods below be made non-virtual for performance?

    virtual void Wait();
            // wait on the condition

    virtual void Wait(const CTime_AC& maximumTimeToWait);
            // wait on the condition (up to a maximum time specified in maximumTimeToWait)
            // throws and exception on timeout.

    virtual void Notify();
            // notify the condition - unblocks a single thread

    virtual void Broadcast();
            // broadcast the condition - unblocks all waiting threads

    void GetDefaultDelay(CTime_AC& theTime) const;
            // get current value for timeout duration on a condition
            // Returns the value in theTime

    virtual void SetDefaultDelay(const CTime_AC& maximumDelay);
            // set default timeout duration for the condition
            // does not change timeout for threads currently waiting

private:
	CMonitorCondition_AC ( const CMonitorCondition_AC& );
	CMonitorCondition_AC& operator = ( const CMonitorCondition_AC& ) { return *this; };

	CAbstractMonitorLock_AC*   	fLock;  
	CTime_AC					fDefaultDelay;
	
	deque<CAbstractThreadContext_AC*, allocator<CAbstractThreadContext_AC*> >* fWaiters;

#if qWantsMP
	MPQueueID fQueue;
	bool fUsingQueue;
#endif	
};


#endif // CMonitorCondition_AC
