//----------------------------------------------------------------------------------------
// CThreadMgrSchedule_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------


#ifndef __CThreadMgrSchedule_AC__
#define __CThreadMgrSchedule_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

// ACS

#ifndef __CAbstractThreadSchedule_AC__
#include "CAbstractThreadSchedule_AC.h"
#endif

// Toolbox 

#ifndef __THREADS__
#include <Threads.h>
#endif

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CThread_AC;

//----------------------------------------------------------------------------------------
// CThreadManagerSchedule_AC is a specialization for Thread Manager tasks.
//----------------------------------------------------------------------------------------

class EXPORT_AC CThreadManagerSchedule_AC : public CAbstractThreadSchedule_AC 
{
public:

	CThreadManagerSchedule_AC (ThreadStyle style = kCooperativeThread);
	// Characterstic: responsive, they handle the
	// application's human interface. For application
	// processing that takes an appreciable amount of
	// time, helper tasks with CApplicationSchedule_AC
	// should be used. See CAbstractThreadSchedule_AC before using this class.

	virtual ~CThreadManagerSchedule_AC ( );

	CThreadManagerSchedule_AC ( const CThreadManagerSchedule_AC& aThreadManagerSchedule ) ;

	CThreadManagerSchedule_AC& operator = ( const CThreadManagerSchedule_AC& aThreadManagerSchedule ) ;

	virtual CAbstractThreadContext_AC* CreateThreadContext( size_t stackSize, SimpleTaskProcPtr entry, void* parameter,
			CThread_AC* adoptedThread = NULL);
	
	virtual CAbstractMonitorLock_AC* CreateMonitorLock();

private:
	ThreadStyle fStyle;
	static const PriorityLevels		kNumberOfLevels;
	static const AbsolutePriority	kMaximumPriority;
	static const AbsolutePriority	kMinimumPriority;
	static const SchedulingPolicy	kPolicy;
	static const AbsolutePriority	kDefaultPriority;
	AbsolutePriority				fPriority;

public:
	// Noose methods
	virtual AbsolutePriority 		GetPriority () const ;
	virtual SchedulingPolicy  		GetPolicy   () const ;
	virtual AbsolutePriority 		GetMaximumPriority () const ;
	virtual AbsolutePriority 		GetMinimumPriority () const ;
	virtual void 					SetPriority (const AbsolutePriority thePriority) ;

};


#endif // CThreadMgrSchedule_AC
