//----------------------------------------------------------------------------------------
// CThreadMgrThreadContext_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CThreadMgrThreadContext_AC__
#define __CThreadMgrThreadContext_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

// Toolbox 

#ifndef __THREADS__
#include <Threads.h>
#endif

class EXPORT_AC CThreadManagerThreadContext_AC : public CAbstractThreadContext_AC
// Concrete class providing context functionality for thread manager threads.
{
public:
	static CThreadManagerThreadContext_AC* GetCurrentThreadManagerContext();
		// Returns a pointer to the context object for the thread currently executing.

	CThreadManagerThreadContext_AC(
			ThreadStyle threadStyle, ThreadEntryProcPtr threadEntry,
			void *threadParam, Size stackSize, ThreadOptions options,
			void **threadResult,
			CThread_AC* adoptedThread = NULL);

	CThreadManagerThreadContext_AC(
			ThreadStyle threadStyle, SimpleTaskProcPtr threadEntry,
			void *threadParam, Size stackSize, ThreadOptions options,
			void **threadResult,
			CThread_AC* adoptedThread = NULL);

	CThreadManagerThreadContext_AC();

	virtual ~CThreadManagerThreadContext_AC();

	virtual void	Suspend();	// Override

	virtual	void	Resume();	// Override

	virtual bool	IsAlive();	// Override

	virtual void 	Yield();	// Override

	ThreadID		GetThreadID();

	static long		GetCurrentThreadCount();
								// Returns the number of threads for which contexts 
								// currently exist, including threads that have completed
								// execution but for which there are still outstanding
								// thread handles.  The main thread is included in this
								// count.
protected:
	static PASCAL_AC void* ThreadMgrWrapperProc(CThreadManagerThreadContext_AC* context);
		// Routine used for interface translation for calling the functions to execute in
		// the thread.

private:
	ThreadTaskRef		fTaskRef;	// Reference to the process in which this thread executes.
	ThreadID			fThreadID;	// ID for the thread represented by this context.
	SimpleTaskProcPtr 	fEntry;		// Entry point to be called in the thread
	void*				fParameter;	// Parameter to be passed to the entry point.
	bool				fResume;	// Flag used in restarting suspended thread.
	bool				fDone;		// Flag indicating if the thread has completed execution.

	static CAutoPtr_AC<map<ThreadID, CThreadManagerThreadContext_AC*, less<ThreadID>, allocator<CThreadManagerThreadContext_AC*> > > fMap;
		// Dictionary of all outstanding thread contexts.

	static ThreadEntryProcPtr fWrapperPtr;
		// Pointer to routine called for execution in thread.

public:
	static CAutoPtr_AC<CThreadManagerThreadContext_AC> fMainContext;
		// Context object representing the main thread.
};

#endif
