//----------------------------------------------------------------------------------------
// CTime_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1995 Taligent, Inc.
//----------------------------------------------------------------------------------------

#ifndef __CTime_AC__
#define __CTime_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CLASS CTime_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CTime_AC 
{
public:
	//------------------------------------------------------------------------
	//	CTime_AC Constants
	//------------------------------------------------------------------------
	static const CTime_AC& 	kPositiveInfinity;
	static const CTime_AC& 	kNegativeInfinity;
	static const CTime_AC& 	kZero;
	
	static const double kPicosecondsToNanoseconds; //	= 1.E3;
	static const double kPicosecondsToMicroseconds; //	= 1.E6;
	static const double kPicosecondsToMilliseconds; //	= 1.E9;
	static const double kPicosecondsToSeconds; //		= 1.E12;
	static const double kPicosecondsToMinutes; //		= 6.E13;
	static const double kPicosecondsToHours; //			= 3.6E15;
	static const double kPicosecondsToDays; //			= 8.64E16;

							CTime_AC ();
							CTime_AC (const CTime_AC& theTime);
	virtual 				~CTime_AC();

	friend CTime_AC operator- (const CTime_AC& howMuch, const CTime_AC& theTime);
	friend CTime_AC operator+ (const CTime_AC& howMuch, const CTime_AC& theTime);
	friend CTime_AC operator* (double howMuch, const CTime_AC& theTime);
	friend CTime_AC operator* (const CTime_AC& theTime, double howMuch);
	friend CTime_AC operator/ (const CTime_AC& theTime, double howMuch);
	friend double operator/ (const CTime_AC& theTime, const CTime_AC& howMuch);
	friend CTime_AC operator- (const CTime_AC& theTime);
	
			CTime_AC&			operator= (const CTime_AC& theTime);
	const   CTime_AC&			operator+= (const CTime_AC& theTime);
	const   CTime_AC&			operator-= (const CTime_AC& theTime);
	const   CTime_AC&			operator*= (double howMuch);
	const   CTime_AC&			operator/= (double howMuch);
								
			bool			operator< (const CTime_AC& theTime) const;
			bool			operator<= (const CTime_AC& theTime) const;
			bool			operator>= (const CTime_AC& theTime) const;
			bool			operator> (const CTime_AC& theTime) const;
			bool			operator== (const CTime_AC& theTime) const;
			bool			operator!= (const CTime_AC& theTime) const;


protected:
			void			SetPicoseconds (double thePicoseconds);
			double			GetPicoseconds() const;

private:
	double					fTime;
};

//----------------------------------------------------------------------------------------
// CLASS CPicoseconds_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CPicoseconds_AC: public CTime_AC 
{
public:
					CPicoseconds_AC ();
					CPicoseconds_AC (double numberOfPicoseconds);
					CPicoseconds_AC (const CPicoseconds_AC& thePicoseconds);
					CPicoseconds_AC (const CTime_AC& theTime);
	virtual			~CPicoseconds_AC ();
	double			GetPicoseconds() const;
	CPicoseconds_AC&	operator= (const CPicoseconds_AC& thePicoseconds);
};

//----------------------------------------------------------------------------------------
// CLASS CNanoseconds_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CNanoseconds_AC: public CTime_AC 
{
public:
					CNanoseconds_AC ();
					CNanoseconds_AC (double numberOfNanoseconds);
					CNanoseconds_AC (const CNanoseconds_AC& theNanoseconds);
					CNanoseconds_AC (const CTime_AC& theTime);
	virtual			~CNanoseconds_AC ();
			double	GetNanoseconds() const;
	CNanoseconds_AC&	operator= (const CNanoseconds_AC& theNanoseconds);
};

//----------------------------------------------------------------------------------------
// CLASS CMicroseconds_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CMicroseconds_AC: public CTime_AC 
{
public:
					CMicroseconds_AC ();
					CMicroseconds_AC (double numberOfMicroseconds);
					CMicroseconds_AC (const CMicroseconds_AC& theMicroseconds);
					CMicroseconds_AC (const CTime_AC& theTime);
	virtual			~CMicroseconds_AC ();
			double	GetMicroseconds() const;
	CMicroseconds_AC&	operator= (const CMicroseconds_AC& theMicroseconds);
};

//----------------------------------------------------------------------------------------
// CLASS CMilliseconds_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CMilliseconds_AC: public CTime_AC 
{
public:
					CMilliseconds_AC ();
					CMilliseconds_AC (double numberOfMilliseconds);
					CMilliseconds_AC (const CTime_AC& theTime);
					CMilliseconds_AC (const CMilliseconds_AC& theMilliseconds);
	virtual			~CMilliseconds_AC ();
			double	GetMilliseconds() const;
	CMilliseconds_AC&	operator= (const CMilliseconds_AC& theMilliseconds);
};

//----------------------------------------------------------------------------------------
// CLASS CSeconds_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CSeconds_AC: public CTime_AC 
{
public:
					CSeconds_AC ();
					CSeconds_AC (double numberOfSeconds);
					CSeconds_AC (const CTime_AC& theTime);
					CSeconds_AC (const CSeconds_AC& theSeconds);
	virtual			~CSeconds_AC ();
			double	GetSeconds() const;
		CSeconds_AC&	operator= (const CSeconds_AC& theSeconds);
};

//----------------------------------------------------------------------------------------
// CLASS CMinutes_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CMinutes_AC: public CTime_AC 
{
public:
					CMinutes_AC ();
					CMinutes_AC (double numberOfMinutes);
					CMinutes_AC (const CTime_AC& theTime);
					CMinutes_AC (const CMinutes_AC& theMinutes);
	virtual			~CMinutes_AC ();
			double	GetMinutes() const;
		CMinutes_AC&	operator= (const CMinutes_AC& theMinutes);
};

//----------------------------------------------------------------------------------------
// CLASS CHours_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CHours_AC: public CTime_AC 
{
public:
					CHours_AC ();
					CHours_AC (double numberOfHours);
					CHours_AC (const CTime_AC& theTime);
					CHours_AC (const CHours_AC& theHours);
	virtual			~CHours_AC ();
			double	GetHours() const;
			CHours_AC&	operator= (const CHours_AC& theHours);
};

//----------------------------------------------------------------------------------------
// CLASS CDays_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CDays_AC: public CTime_AC 
{
public:
					CDays_AC ();
					CDays_AC (double numberOfDays);
					CDays_AC (const CTime_AC& theTime);
					CDays_AC (const CDays_AC& theDays);
	virtual			~CDays_AC ();
			double	GetDays() const;
			CDays_AC&	operator= (const CDays_AC& theDays);
};


//----------------------------------------------------------------------------------------
// INLINES
//----------------------------------------------------------------------------------------

inline bool CTime_AC::operator< (const CTime_AC& theTime) const
{
	return (fTime < theTime.fTime);
}

//----------------------------------------------------------------------------------------

inline bool CTime_AC::operator<= (const CTime_AC& theTime) const
{
	return (fTime <= theTime.fTime);
}

//----------------------------------------------------------------------------------------

inline bool CTime_AC::operator> (const CTime_AC& theTime) const
{
	return (fTime > theTime.fTime);
}

//----------------------------------------------------------------------------------------

inline bool CTime_AC::operator>= (const CTime_AC& theTime) const
{
	return (fTime >= theTime.fTime);
}

//----------------------------------------------------------------------------------------

inline bool CTime_AC::operator== (const CTime_AC& theTime) const
{
	return (fTime == theTime.fTime);
}

//----------------------------------------------------------------------------------------

inline bool CTime_AC::operator!= (const CTime_AC& theTime) const
{
	return (fTime != theTime.fTime);
}
	
//----------------------------------------------------------------------------------------

inline CTime_AC operator- (const CTime_AC& howMuch, const CTime_AC& theTime)
{
	CTime_AC time;
	time.SetPicoseconds (howMuch.fTime - theTime.fTime);
	return time;
};

//----------------------------------------------------------------------------------------

inline CTime_AC operator+ (const CTime_AC& howMuch, const CTime_AC& theTime)
{
	CTime_AC time;
	time.SetPicoseconds (howMuch.fTime + theTime.fTime);
	return time;
};

//----------------------------------------------------------------------------------------

inline CTime_AC operator* (double howMuch, const CTime_AC& theTime)
{
	CTime_AC time;
	time.SetPicoseconds (howMuch * theTime.fTime);
	return time;
};

//----------------------------------------------------------------------------------------

inline CTime_AC operator* (const CTime_AC& theTime, double howMuch)
{
	CTime_AC time;
	time.SetPicoseconds (howMuch * theTime.fTime);
	return time;
};

//----------------------------------------------------------------------------------------

inline CTime_AC operator/ (const CTime_AC& theTime, double howMuch)
{
	CTime_AC time;
	time.SetPicoseconds (theTime.fTime / howMuch);
	return time;
};

//----------------------------------------------------------------------------------------

inline double operator/ (const CTime_AC& theTime, const CTime_AC& howMuch)
{
	return (theTime.fTime / howMuch.fTime);
};

//----------------------------------------------------------------------------------------

inline CTime_AC operator- (const CTime_AC& theTime)
{
	CTime_AC time;
	time.fTime = -theTime.fTime;
	return time;
};

//----------------------------------------------------------------------------------------

inline			CTime_AC&	CTime_AC::operator= (const CTime_AC& theTime) {fTime = theTime.fTime; return *this;};
inline	const   CTime_AC&	CTime_AC::operator+= (const CTime_AC& theTime) {fTime += theTime.fTime; return *this;}
inline	const   CTime_AC&	CTime_AC::operator-= (const CTime_AC& theTime) {fTime -= theTime.fTime; return *this;}
inline	const   CTime_AC&	CTime_AC::operator*= (double howMuch)	{fTime *= howMuch; return *this;}
inline	const   CTime_AC&	CTime_AC::operator/= (double howMuch) {fTime /= howMuch; return *this;}

//----------------------------------------------------------------------------------------

inline	double			CTime_AC::GetPicoseconds() const
{
	return fTime;
}

//----------------------------------------------------------------------------------------

inline	double			CPicoseconds_AC::GetPicoseconds() const 
{
	return CTime_AC::GetPicoseconds();
}

//----------------------------------------------------------------------------------------

inline	CPicoseconds_AC&	CPicoseconds_AC::operator= (const CPicoseconds_AC& thePicoseconds)
{
	CTime_AC::operator= (thePicoseconds); return *this;
}

//----------------------------------------------------------------------------------------

inline double CNanoseconds_AC::GetNanoseconds () const
{
	return ( GetPicoseconds() / kPicosecondsToNanoseconds);
}

//----------------------------------------------------------------------------------------

inline CNanoseconds_AC& CNanoseconds_AC::operator= (const CNanoseconds_AC& theNanoseconds)
{
	CTime_AC::operator= (theNanoseconds); return *this;
}

//----------------------------------------------------------------------------------------

inline double CMicroseconds_AC::GetMicroseconds () const
{
	return ( GetPicoseconds() / kPicosecondsToMicroseconds);
}

//----------------------------------------------------------------------------------------

inline CMicroseconds_AC& CMicroseconds_AC::operator= (const CMicroseconds_AC& theMicroseconds)
{
	CTime_AC::operator= (theMicroseconds); return *this;
}

//----------------------------------------------------------------------------------------

inline double CMilliseconds_AC::GetMilliseconds () const
{
	return ( GetPicoseconds() / kPicosecondsToMilliseconds);
}

//----------------------------------------------------------------------------------------

inline CMilliseconds_AC& CMilliseconds_AC::operator= (const CMilliseconds_AC& theTime)
{
	CTime_AC::operator= (theTime); return *this;
}

//----------------------------------------------------------------------------------------

inline double CSeconds_AC::GetSeconds () const
{
	return ( GetPicoseconds() / kPicosecondsToSeconds);
}

//----------------------------------------------------------------------------------------

inline CSeconds_AC& CSeconds_AC::operator= (const CSeconds_AC& theTime)
{
	CTime_AC::operator= (theTime); return *this;
}

//----------------------------------------------------------------------------------------

inline double CMinutes_AC::GetMinutes () const
{
	return ( GetPicoseconds() / kPicosecondsToMinutes);
}

//----------------------------------------------------------------------------------------

inline CMinutes_AC& CMinutes_AC::operator= (const CMinutes_AC& theTime)
{
	CTime_AC::operator= (theTime); return *this;
}

//----------------------------------------------------------------------------------------

inline double CHours_AC::GetHours () const
{
	return ( GetPicoseconds() / kPicosecondsToHours);
}

//----------------------------------------------------------------------------------------

inline CHours_AC& CHours_AC::operator= (const CHours_AC& theTime)
{
	CTime_AC::operator= (theTime); return *this;
}

//----------------------------------------------------------------------------------------

inline double CDays_AC::GetDays () const
{
	return (GetPicoseconds() / kPicosecondsToDays);
}

//----------------------------------------------------------------------------------------

inline CDays_AC& CDays_AC::operator= (const CDays_AC& theTime)
{
	CTime_AC::operator= (theTime); return *this;
}


#endif



