//----------------------------------------------------------------------------------------
// CWinThreadContext_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CWinThreadContext_AC__
#define __CWinThreadContext_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#if 0 

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

// MSWindows

#ifndef __CWinThreads_AC__
#include "CWinThreads_AC.h"
#endif

//----------------------------------------------------------------------------------------

class EXPORT_AC CWinThreadContext_AC : public CAbstractThreadContext_AC
// Concrete class providing context functionality for thread manager threads.
{
public:
	static CWinThreadContext_AC* GetCurrentWinContext();
		// Returns a pointer to the context object for the thread currently executing.

	CWinThreadContext_AC(
			LPSECURITY_ATTRIBUTES threadAttributes, DWORD stackSize,
			SimpleTaskProcPtr threadEntry, void* threadParam,
			DWORD creationFlags,
			CThread_AC* adoptedThread = NULL);

	CWinThreadContext_AC();

	virtual ~CWinThreadContext_AC();

	virtual void	Suspend();	// Override

	virtual	void	Resume();	// Override

	virtual bool	IsAlive();	// Override

	virtual void 	Yield();	// Override

	DWORD			GetThreadID();
	
	HANDLE			GetThreadRef();

	static long		GetCurrentThreadCount();
								// Returns the number of threads for which contexts 
								// currently exist, including threads that have completed
								// execution but for which there are still outstanding
								// thread handles.  The main thread is included in this
								// count.
protected:

	static void* WinWrapperProc(CWinThreadContext_AC* context);
		// Routine used for interface translation for calling the functions to execute in
		// the thread.

private:
	HANDLE					fThreadRef;	// Reference handle to this thread.
	DWORD					fThreadID;	// ID for the thread represented by this context.
	LPTHREAD_START_ROUTINE 	fEntry;		// Entry point to be called in the thread
	void*					fParameter;	// Parameter to be passed to the entry point.
	bool					fDone;		// Flag indicating if the thread has completed execution.

	static CAutoPtr_AC<map<DWORD, CWinThreadContext_AC*, less<DWORD>, allocator<CWinThreadContext_AC*> > > fMap;
		// Dictionary of all outstanding thread contexts.

	static LPTHREAD_START_ROUTINE fWrapperPtr;
		// Pointer to routine called for execution in thread.

public:
	static CAutoPtr_AC<CWinThreadContext_AC> fMainContext;
		// Context object representing the main thread.
};

#endif // qMSWindows

#endif
