//----------------------------------------------------------------------------------------
// CYieldThread_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CYieldThread_AC__
#define __CYieldThread_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#if qYieldToOtherProcesses || qWantsMP

#ifndef __CThread_AC__
#include "CThread_AC.h"
#endif

#ifndef __CThreadRunner_AC__
#include "CThreadRunner_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __MP__
#include <MP.h>
#endif

//----------------------------------------------------------------------------------------
// CYieldThread_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CYieldThread_AC : public CThread_AC
//
// This class provides a special thread that manages yielding so that threads
// and tasks get appropriate opportunities to run.  A single thread running this thread
// is launched before the first developer specified thread or task is run.
//
{
public:
	CYieldThread_AC();
	
	virtual ~CYieldThread_AC();
	
	virtual void Run();
		// Run the yield thread.

	static void Initialize();
		// Starts up a thread manager thread which yields time to, and processes requests
		// for, MP tasks.  This is essential on a single processor machine when using
		// MP tasks, since MP tasks do not preempt the main context, and would otherwise
		// only get limited time at certain system calls, such as WaitNextEvent.

#if qWantsMP

	static MPQueueID	GetCooperativeRequestQueue() { return fCooperativeRequestQueue; }
		// This MP queue is used to receive requests for execution in a cooperative 
		// (toolbox safe) context from MP tasks.  Requests are executed in the order
		// received by the yield thread each time it gets scheduled to execute.

#endif

private:
	
	static CAutoPtr_AC<CThreadRunner_AC> fgYieldThread;
		// Handle to the thread manager thread which yields time to MP tasks.
		
	static bool fgInitialized;
		// Flag to keep track of whether the yield thread has been initialized.

#if qWantsMP
	static void	SetCooperativeRequestQueue(MPQueueID theID) { fCooperativeRequestQueue = theID; }
		// This MP queue is used to receive requests for execution in a cooperative 
		// (toolbox safe) context from MP tasks.  Requests are executed in the order
		// received by the yield thread each time it gets scheduled to execute.

	static MPQueueID		fCooperativeRequestQueue;
		// This MP queue is used to receive requests for execution in a cooperative 
		// (toolbox safe) context from MP tasks.  Requests are executed in the order
		// received by the yield thread each time it gets scheduled to execute.

#endif

};

#endif

#endif