//----------------------------------------------------------------------------------------
// ThreadSwitches_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	This file defines the following switches:
		qThreadDebug
			Controls whether debugging code is compiled.		
			If not defined and qCoreDebug is defined, it is defined as qCoreDebug, otherwise 1.

		qInstrumentation	
			Controls wether instrumentation debugging code is compiled.
			If building for Windows, it is defined as 0; otherwise if not defined, it is defined as 0.

		qWantsMP		
			Controls whether MP support code is compiled.	
			If not building for PowerPC, it is defined as 0; otherwise if not defined, it is defined as 1.

		qWantsThreadMgr
			Controls whether ThreadMgr support code is compiled.
			If not defined, it is defined as 1.
		
		qYieldToOtherProcesses
			determines whether calls to yield to another thread should also 
			provide other processes opportunities to run, using SystemTask.

		qMonitorCancel	
			When supported, this flag will cause the thread supporting yeilds to MP tasks and other
			processes to take on the additional role of checking for command-period.  If the main 
			thread is blocked waiting on a lock, it will be resumed with an exception.  If the main
			thread is otherwise busy, the exception will be thown the next time YieldCPU is called
			or an attempt is made to aquire a lock.

*/

#ifndef __ThreadSwitches_AC__
#define __ThreadSwitches_AC__

//----------------------------------------------------------------------------------------
// qCoreDebug
//----------------------------------------------------------------------------------------

#if defined(qCoreDebug) && !defined (qThreadDebug)
#	define qThreadDebug qCoreDebug 
#else
#	ifndef qThreadDebug
#		define qThreadDebug 1
#	endif
#endif

//----------------------------------------------------------------------------------------
// qInstrumentation
//----------------------------------------------------------------------------------------

#if !qMSWindows
#	ifndef qInstrumentation
#		define qInstrumentation 0
#	endif
#else
// Can't have instrumentation on Win32
#	undef qInstrumentation
#	define qInstrumentation 0
#endif

//----------------------------------------------------------------------------------------
// qWantsMP
//----------------------------------------------------------------------------------------

#if __POWERPC__
#	ifndef qWantsMP
#		define qWantsMP 1
#	endif
#else
// Can't have MP on 68K
#	undef qWantsMP
#	define qWantsMP 0
#endif

//----------------------------------------------------------------------------------------
// qWantsThreadMgr
//----------------------------------------------------------------------------------------

#ifndef qWantsThreadManager
#	define qWantsThreadManager 1
#endif

//----------------------------------------------------------------------------------------
// qYieldToOtherProcesses
//----------------------------------------------------------------------------------------

#ifndef qYieldToOtherProcesses
#	define qYieldToOtherProcesses 0
#endif

//----------------------------------------------------------------------------------------
// qMonitorCancel
//----------------------------------------------------------------------------------------

#ifndef qMonitorCancel	// Not yet supported.
#	define qMonitorCancel 0
#endif

#endif
