//----------------------------------------------------------------------------------------
// CAbstractThreadContext_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

#ifndef	__CMonitorLock_AC__
#include "CMonitorLock_AC.h"
#endif

#ifndef __CThreadMgrThreadContext_AC__
#include "CThreadMgrThreadContext_AC.h"
#endif

#ifndef __CMPThreadContext_AC__
#include "CMPThreadContext_AC.h"
#endif

#ifndef __CWhileSynchronized_AC__
#include "CWhileSynchronized_AC.h"
#endif

#ifndef __CThread_AC__
#include "CThread_AC.h"
#endif

#if qYieldToOtherProcesses || qWantsMP
#	ifndef __CYieldThread_AC__
#	include "CYieldThread_AC.h"
#	endif
#endif

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CAbstractThreadContext_AC::CAbstractThreadContext_AC(CThread_AC* adoptedThread) :
	fLock(TH_new CMonitorLock_AC()),
	fAdoptedThread(adoptedThread),
	fRefCount(1)
{
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CAbstractThreadContext_AC::~CAbstractThreadContext_AC()
{
}

//----------------------------------------------------------------------------------------
// GetCurrentContext returns the current thread's context.
//----------------------------------------------------------------------------------------

CAbstractThreadContext_AC* CAbstractThreadContext_AC::GetCurrentContext()
{
	CAbstractThreadContext_AC* currentContext = NULL;
#if qWantsMP
	currentContext = CMPThreadContext_AC::GetCurrentMPContext();
	// Calling GetCurrentMPContext guarantees CMPThreadContext_AC::fMainContext.Get() is not NULL
	if ((currentContext == NULL) 
		|| (currentContext == (CAbstractThreadContext_AC*)CMPThreadContext_AC::fMainContext.get()))
#endif
			currentContext = CThreadManagerThreadContext_AC::GetCurrentThreadManagerContext();
	return currentContext;
}

//----------------------------------------------------------------------------------------
// SuspendCurrentContext
//----------------------------------------------------------------------------------------

void CAbstractThreadContext_AC::SuspendCurrentContext() // static
//
// Suspends the current thread context.
//
{
	CAbstractThreadContext_AC* context = CAbstractThreadContext_AC::GetCurrentContext();
	if (context)
		context->Suspend();
}

//----------------------------------------------------------------------------------------
// YieldCurrentContext
//----------------------------------------------------------------------------------------

void CAbstractThreadContext_AC::YieldCurrentContext() // static
//
// Yields this thread, allowing other (cooperatively run) threads time to execute.
//
{
	CAbstractThreadContext_AC* context = CAbstractThreadContext_AC::GetCurrentContext();
	if (context)
		context->Yield();
}

//----------------------------------------------------------------------------------------
// Acquire
//----------------------------------------------------------------------------------------

void CAbstractThreadContext_AC::Acquire()
{
	CWhileSynchronized_AC entry(fLock);
	++fRefCount;
}

//----------------------------------------------------------------------------------------
// Release
//----------------------------------------------------------------------------------------

void CAbstractThreadContext_AC::Release()
{
	bool orphaned = false;
	{
		CWhileSynchronized_AC entry(fLock);
		fRefCount--;
		orphaned = (fRefCount <= 0);
	}
	if (orphaned) 
		delete this;
}

//----------------------------------------------------------------------------------------
// GetRefCount
//----------------------------------------------------------------------------------------

long CAbstractThreadContext_AC::GetRefCount()
{
	CWhileSynchronized_AC entry(fLock);
	return fRefCount;
}

//----------------------------------------------------------------------------------------
// GetThread
//----------------------------------------------------------------------------------------

CThread_AC* CAbstractThreadContext_AC::GetThread()
{
	return fAdoptedThread;
}

//----------------------------------------------------------------------------------------
// CYieldThread::InitializeYieldThread
//----------------------------------------------------------------------------------------

#if qYieldToOtherProcesses || qWantsMP

void CAbstractThreadContext_AC::InitializeYieldThread()
{
	CYieldThread_AC::Initialize();
}

#endif

//----------------------------------------------------------------------------------------
// CYieldThread::GetCooperativeRequestQueue
//----------------------------------------------------------------------------------------

#if qWantsMP

MPQueueID CAbstractThreadContext_AC::GetCooperativeRequestQueue()
{
	return CYieldThread_AC::GetCooperativeRequestQueue();
}

#endif

//----------------------------------------------------------------------------------------
// GetSP
//----------------------------------------------------------------------------------------

#if qPowerPC && qThreadDebug

asm void* GetSP()
// Returns the current stack pointer for debugging purposes.
{
	mr	r3,SP
	blr
}

#endif


