//----------------------------------------------------------------------------------------
// CMPSchedule_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------


#ifndef __CMPSchedule_AC__
#include "CMPSchedule_AC.h"
#endif

#ifndef	__CMPMonitorLock_AC__
#include "CMPMonitorLock_AC.h"
#endif

#ifndef __CMPThreadContext_AC__
#include "CMPThreadContext_AC.h"
#endif

#ifndef	__CThreadMgrThreadContext_AC__
#include "CThreadMgrThreadContext_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Dummy pthread constants.
//----------------------------------------------------------------------------------------

#define PRI_OTHER_MIN 1
#define PRI_OTHER_MAX 1
#define SCHED_OTHER 1

#if qWantsMP

//----------------------------------------------------------------------------------------
// CMPSchedule_AC class requires some constants to be used
//----------------------------------------------------------------------------------------

const int				kMPNumberOfLevels		= (PRI_OTHER_MAX - PRI_OTHER_MIN + 1) / 4;
const int				kMPDefaultLevel			= kMPNumberOfLevels / 2;
const AbsolutePriority 	kMPMaximumPriority		= PRI_OTHER_MAX - kMPNumberOfLevels;
const AbsolutePriority 	kMPMinimumPriority		= kMPMaximumPriority - kMPNumberOfLevels + 1;
const SchedulingPolicy	kMPPolicy				= SCHED_OTHER;

const PriorityLevels 	CMPSchedule_AC::kNumberOfLevels		= kMPNumberOfLevels;
const AbsolutePriority	CMPSchedule_AC::kMaximumPriority	= kMPMaximumPriority;
const AbsolutePriority	CMPSchedule_AC::kMinimumPriority	= kMPMinimumPriority;
const SchedulingPolicy	CMPSchedule_AC::kPolicy				= kMPPolicy;
const AbsolutePriority	CMPSchedule_AC::kDefaultPriority	= kMPMaximumPriority - kMPDefaultLevel;

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CMPSchedule_AC::CMPSchedule_AC () :
	fPriority(kDefaultPriority)
{
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CMPSchedule_AC::~CMPSchedule_AC ()
{
}

//----------------------------------------------------------------------------------------
// Copy constructor
//----------------------------------------------------------------------------------------

CMPSchedule_AC::CMPSchedule_AC (const CMPSchedule_AC& anMPSchedule) :
	CAbstractThreadSchedule_AC(anMPSchedule),
	fPriority(anMPSchedule.fPriority)
{
}

//----------------------------------------------------------------------------------------
// Copy assignment
//----------------------------------------------------------------------------------------

CMPSchedule_AC& CMPSchedule_AC::operator =(const CMPSchedule_AC& anMPSchedule)
{
	fPriority = anMPSchedule.fPriority;
	CAbstractThreadSchedule_AC::operator=(anMPSchedule);
	return (*this);
}

//----------------------------------------------------------------------------------------
// CreateThreadContext
//----------------------------------------------------------------------------------------
CAbstractThreadContext_AC* CMPSchedule_AC::CreateThreadContext( size_t stackSize, 
			SimpleTaskProcPtr entry, void* parameter,
			CThread_AC* adoptedThread)
{
	return TH_new CMPThreadContext_AC(entry, parameter, stackSize, 0, 0, 0, kMPNormalTaskOptions, adoptedThread);
}

//----------------------------------------------------------------------------------------
// CreateMonitorLock
//----------------------------------------------------------------------------------------

CAbstractMonitorLock_AC* CMPSchedule_AC::CreateMonitorLock()
{
	return TH_new CMPMonitorLock_AC();
}

//----------------------------------------------------------------------------------------
// Getter methods
//----------------------------------------------------------------------------------------

AbsolutePriority CMPSchedule_AC::GetPriority() const
{
	return 	( fPriority );
}

//----------------------------------------------------------------------------------------

SchedulingPolicy CMPSchedule_AC::GetPolicy() const
{
	return ( kPolicy );
}

//----------------------------------------------------------------------------------------

AbsolutePriority CMPSchedule_AC::GetMaximumPriority() const
{
	return ( kMaximumPriority );
}

//----------------------------------------------------------------------------------------

AbsolutePriority CMPSchedule_AC::GetMinimumPriority() const
{
	return ( kMinimumPriority );
}

//----------------------------------------------------------------------------------------
// Setter methods
//----------------------------------------------------------------------------------------

void CMPSchedule_AC::SetPriority( const AbsolutePriority aPriority )
{
	fPriority = aPriority;
}

#endif // qWantsMP
