//----------------------------------------------------------------------------------------
// CMonitorLock_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef	__CMonitorLock_AC__
#include "CMonitorLock_AC.h"
#endif

#if qWantsMP

#ifndef __Instrumentation_AC__
#include "Instrumentation_AC.h"
#endif

#ifndef	__CMPMonitorLock_AC__
#include "CMPMonitorLock_AC.h"
#endif

#ifndef __CWhileSynchronized_AC__
#include "CWhileSynchronized_AC.h"
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

//========================================================================================
//	Class CMonitorLock_AC synchronizes between MP and cooperative threads.
//========================================================================================

CMonitorLock_AC::CMonitorLock_AC () :
	fInternalMonitor(NULL),
	fCondition(NULL),
	fHolder(NULL)
{
	TRACE_SETUP;
	LOG_ENTRY( "CMonitorLock_AC:Constructor" );
	if (::HasMP_AC()) 
	{
		fInternalMonitor = TH_new CMPMonitorLock_AC();
		fCondition = TH_new CMonitorCondition_AC(fInternalMonitor);
	}
	else 
	{
		fInternalMonitor = TH_new CThreadManagerMonitorLock_AC();
	}
	LOG_EXIT;
}

//----------------------------------------------------------------------------------------
// CMonitorLock_AC
//----------------------------------------------------------------------------------------

CMonitorLock_AC::~CMonitorLock_AC ()
{
	TRACE_SETUP;
	LOG_ENTRY( "CMonitorLock_AC:Destructor" );
	// Danger: Not safe to delete CMonitorLock_AC while there are waiters!
	delete fCondition;
	fCondition = NULL;
	delete fInternalMonitor;
	LOG_EXIT;
}

//----------------------------------------------------------------------------------------
// CMonitorLock_AC
//----------------------------------------------------------------------------------------

void CMonitorLock_AC::Acquire()
{
	TRACE_SETUP;
	LOG_ENTRY( "CMonitorLock_AC:Acquire" );
	if (::HasMP_AC()) 
	{
		CWhileSynchronized_AC entry(fInternalMonitor);
#if qThreadDebug
//		int i = 0;
//		::_MPDebugStr((StringPtr)"\pTMonitorLock::Acquire;g");
#endif
		while (fHolder != NULL) {
#if qThreadDebug
//			if (i > 0) 
//				::_MPDebugStr((StringPtr)"\pTMonitorLock::Acquire Double Wait!");
//			++i;
#endif
			fCondition->Wait();
		}
		fHolder = CAbstractThreadContext_AC::GetCurrentContext();
	}
	else 
	{
		fInternalMonitor->Acquire();
	}
	LOG_EXIT;
}

//----------------------------------------------------------------------------------------
// CMonitorLock_AC
//----------------------------------------------------------------------------------------

void CMonitorLock_AC::Release()
{
	TRACE_SETUP;
	LOG_ENTRY( "CMonitorLock_AC:Release" );
	if (::HasMP_AC()) 
	{
		CWhileSynchronized_AC entry(fInternalMonitor);
#if qThreadDebug
//		::_MPDebugStr((StringPtr)"\pTMonitorLock::Release;g");
#endif
		fHolder = NULL;
		::__sync();
		fCondition->Notify();
	}
	else 
	{
		fInternalMonitor->Release();
	}
	LOG_EXIT;
}

#endif
