//----------------------------------------------------------------------------------------
// CThreadMgrMonitorLock_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------




#ifndef	__CThreadMgrMonitorLock_AC__
#include "CThreadMgrMonitorLock_AC.h"
#endif

#ifndef DEQUE_H
#include <Deque.h>
#endif

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

#ifndef __Instrumentation_AC__
#include "Instrumentation_AC.h"
#endif

#if qMSWindows
#ifndef __QTMLMAPNAMES__
#include <QTMLMapNames.h>
#endif
#endif

//----------------------------------------------------------------------------------------
//	CThreadManagerMonitorLock_AC provides a monitor specialized for ThreadManager threads.
//----------------------------------------------------------------------------------------

CThreadManagerMonitorLock_AC::CThreadManagerMonitorLock_AC() :
	fWaiters(TH_new deque<ThreadID, allocator<ThreadID> >()),
	fHolder(kNoThreadID)
{
	TRACE_SETUP;
	LOG_ENTRY( "CThreadManagerMonitorLock_AC:Constructor" );
	LOG_EXIT;
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CThreadManagerMonitorLock_AC::~CThreadManagerMonitorLock_AC()
{
	TRACE_SETUP;
	LOG_ENTRY( "CThreadManagerMonitorLock_AC:Destructor" );
	OSErr err;
	err = ::ThreadBeginCritical();
	ThreadID waiter = kNoThreadID;
	while (!fWaiters->empty()) {	// Release all waiters.
		waiter = (*fWaiters->begin());
		err = ::SetThreadState(waiter, kReadyThreadState, kNoThreadID);
		fWaiters->pop_front();
	}
	fHolder = kNoThreadID;
	delete fWaiters;
	err = ::ThreadEndCritical();
	LOG_EXIT;
}

//----------------------------------------------------------------------------------------
// Acquire
//----------------------------------------------------------------------------------------

void CThreadManagerMonitorLock_AC::Acquire()
{
	TRACE_SETUP;
	LOG_ENTRY( "CThreadManagerMonitorLock_AC:Acquire" );

	DEBUG_SETUP_STACK_FRAME();

	OSErr err = ::ThreadBeginCritical();
	ThreadID current = kNoThreadID;
	::GetCurrentThread(&current);
	while ( fHolder != kNoThreadID ) 
	{
		fWaiters->push_back(current);
		err = ::SetThreadStateEndCritical(kCurrentThreadID, kStoppedThreadState, fHolder);
		err = ::ThreadBeginCritical();
	}
	fHolder = current;
	err = ::ThreadEndCritical();
	LOG_EXIT;
}

//----------------------------------------------------------------------------------------
// Release
//----------------------------------------------------------------------------------------

void CThreadManagerMonitorLock_AC::Release()
{
	TRACE_SETUP;
	LOG_ENTRY( "CThreadManagerMonitorLock_AC:Release" );
	OSErr err;
	err = ::ThreadBeginCritical();
	ThreadID waiter = kNoThreadID;
	fHolder = kNoThreadID;
	bool waiterFound = false;
	while (!fWaiters->empty() && !waiterFound) 
	{
		waiter = (*fWaiters->begin());
		err = ::SetThreadState(waiter, kReadyThreadState, kNoThreadID);
		waiterFound = (err != threadNotFoundErr);	// Don't release to a dead thread.
		fWaiters->pop_front();
	}
	err = ::SetThreadStateEndCritical(kCurrentThreadID, kReadyThreadState, waiter);
	LOG_EXIT;
}
