//----------------------------------------------------------------------------------------
// CThreadMgrSchedule_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------


#ifndef __CThreadMgrSchedule_AC__
#include "CThreadMgrSchedule_AC.h"
#endif

#ifndef	__CThreadMgrMonitorLock_AC__
#include "CThreadMgrMonitorLock_AC.h"
#endif

#ifndef __CThreadMgrThreadContext_AC__
#include "CThreadMgrThreadContext_AC.h"
#endif


//----------------------------------------------------------------------------------------
// Dummy pthread constants.
//----------------------------------------------------------------------------------------

#define PRI_OTHER_MIN 1
#define PRI_OTHER_MAX 1
#define SCHED_OTHER 1

//----------------------------------------------------------------------------------------
// CThreadManagerSchedule_AC class requires some constants to be used
//----------------------------------------------------------------------------------------

const int				kThreadManagerNumberOfLevels		= (PRI_OTHER_MAX - PRI_OTHER_MIN + 1) / 4;
const int				kThreadManagerDefaultLevel			= kThreadManagerNumberOfLevels / 2;
const AbsolutePriority 	kThreadManagerMaximumPriority		= PRI_OTHER_MAX - kThreadManagerNumberOfLevels;
const AbsolutePriority 	kThreadManagerMinimumPriority		= kThreadManagerMaximumPriority - kThreadManagerNumberOfLevels + 1;
const SchedulingPolicy	kThreadManagerPolicy				= SCHED_OTHER;

const PriorityLevels 	CThreadManagerSchedule_AC::kNumberOfLevels	= kThreadManagerNumberOfLevels;
const AbsolutePriority	CThreadManagerSchedule_AC::kMaximumPriority	= kThreadManagerMaximumPriority;
const AbsolutePriority	CThreadManagerSchedule_AC::kMinimumPriority	= kThreadManagerMinimumPriority;
const SchedulingPolicy	CThreadManagerSchedule_AC::kPolicy			= kThreadManagerPolicy;
const AbsolutePriority	CThreadManagerSchedule_AC::kDefaultPriority	= kThreadManagerMaximumPriority - kThreadManagerDefaultLevel;

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CThreadManagerSchedule_AC::CThreadManagerSchedule_AC(ThreadStyle style) :
	fStyle(style),
	fPriority(kDefaultPriority)
{

}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CThreadManagerSchedule_AC::~CThreadManagerSchedule_AC()
{
}

//----------------------------------------------------------------------------------------
// Copy constructor
//----------------------------------------------------------------------------------------

CThreadManagerSchedule_AC::CThreadManagerSchedule_AC (const CThreadManagerSchedule_AC& aThreadManagerSchedule) :
	CAbstractThreadSchedule_AC(aThreadManagerSchedule),
	fPriority(aThreadManagerSchedule.fPriority)
{
}

//----------------------------------------------------------------------------------------
// Copy assignment
//----------------------------------------------------------------------------------------

CThreadManagerSchedule_AC& CThreadManagerSchedule_AC::operator =(const CThreadManagerSchedule_AC& aThreadManagerSchedule)
{
	fPriority = aThreadManagerSchedule.fPriority;
	CAbstractThreadSchedule_AC::operator=(aThreadManagerSchedule);
	return (*this);
}

//----------------------------------------------------------------------------------------
// CreateThreadContext
//----------------------------------------------------------------------------------------

CAbstractThreadContext_AC* CThreadManagerSchedule_AC::CreateThreadContext(size_t stackSize, 
			SimpleTaskProcPtr entry, void* parameter,
			CThread_AC* adoptedThread)
{
	return TH_new CThreadManagerThreadContext_AC(fStyle, entry, parameter, stackSize, 0, NULL, adoptedThread);
}

//----------------------------------------------------------------------------------------
// CreateMonitorLock
//----------------------------------------------------------------------------------------

CAbstractMonitorLock_AC* CThreadManagerSchedule_AC::CreateMonitorLock()
{
	return TH_new CThreadManagerMonitorLock_AC();
}

//----------------------------------------------------------------------------------------
// Getter methods
//----------------------------------------------------------------------------------------

AbsolutePriority CThreadManagerSchedule_AC::GetPriority() const
{
	return ( fPriority );
}

//----------------------------------------------------------------------------------------

SchedulingPolicy  CThreadManagerSchedule_AC::GetPolicy() const
{
	return ( kPolicy );
}

//----------------------------------------------------------------------------------------

AbsolutePriority CThreadManagerSchedule_AC::GetMaximumPriority() const
{
	return ( kMaximumPriority );
}

//----------------------------------------------------------------------------------------

AbsolutePriority CThreadManagerSchedule_AC::GetMinimumPriority() const
{
	return ( kMinimumPriority );
}

//----------------------------------------------------------------------------------------
// Setter methods
//----------------------------------------------------------------------------------------

void CThreadManagerSchedule_AC::SetPriority(const AbsolutePriority aPriority)
{
	fPriority = aPriority;
}
