//----------------------------------------------------------------------------------------
// ThreadManagerThreadContext.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CThreadMgrThreadContext_AC__
#include "CThreadMgrThreadContext_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

#ifndef MAP_H
#include <map.h>
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CMPThreadContext_AC__
#include "CMPThreadContext_AC.h"
#endif

#if qMSWindows
#ifndef __QTMLMAPNAMES__
#include <QTMLMapNames.h>
#endif
#endif

//----------------------------------------------------------------------------------------
// CThreadManagerThreadContext_AC provides a wrapper for creating Thread Manager threads.
//----------------------------------------------------------------------------------------

CAutoPtr_AC<map<ThreadID,CThreadManagerThreadContext_AC* ,less<ThreadID>, allocator<CThreadManagerThreadContext_AC*> > > CThreadManagerThreadContext_AC::fMap;
CAutoPtr_AC<CThreadManagerThreadContext_AC> CThreadManagerThreadContext_AC::fMainContext;

//----------------------------------------------------------------------------------------
// GetCurrentThreadManagerContext
//----------------------------------------------------------------------------------------

CThreadManagerThreadContext_AC* CThreadManagerThreadContext_AC::GetCurrentThreadManagerContext()
{
	CThreadManagerThreadContext_AC* result = NULL;
	if (fMap.get() != NULL)
	{
		ThreadID currentThread = kNoThreadID;
		::GetCurrentThread(&currentThread);
		// Lookup context in static list using ID as key.
		if (!fMap->empty())
		{
			map<ThreadID,CThreadManagerThreadContext_AC* ,less<ThreadID>, allocator<CThreadManagerThreadContext_AC*> >::iterator threadMapIterator = fMap->find(currentThread);
			if ( (*threadMapIterator).first == currentThread)
			{
				result = (*threadMapIterator).second;
			}
		}
	}
	if (result == NULL)
	{
		// Main context still needs to be created!
		if (fMainContext.get() == NULL)
		{
			fMainContext = TH_new CThreadManagerThreadContext_AC();
			result = fMainContext.get();
		}
	}
	return result;
}

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CThreadManagerThreadContext_AC::CThreadManagerThreadContext_AC(
			ThreadStyle threadStyle, ThreadEntryProcPtr threadEntry,
			void *threadParam, Size stackSize, ThreadOptions options,
			void **threadResult,
			CThread_AC* adoptedThread) :
	CAbstractThreadContext_AC(adoptedThread),
	fThreadID(kNoThreadID),
	fEntry(NULL),
	fParameter(NULL),
	fResume(false),
	fDone(false)
{
	OSErr err;
	if (fMap.get() == NULL)
	{
		fMap = TH_new map<ThreadID,CThreadManagerThreadContext_AC* ,less<ThreadID>, allocator<CThreadManagerThreadContext_AC*> >();
#if qYieldToOtherProcesses || qWantsMP
		InitializeYieldThread();	// Make sure the yielding thread is initialized.
#endif
	}
	err = ::GetThreadCurrentTaskRef(&fTaskRef);
	err = ::NewThread(threadStyle, threadEntry,
			threadParam, stackSize, options,
			threadResult, &fThreadID);

	::ThrowIfError_AC(err);
	// Add context to static list keyed by thread ID.  List must be thread safe :-)
	pair<const ThreadID, CThreadManagerThreadContext_AC* > threadMapPair(fThreadID, this);
	fMap->insert(threadMapPair);
}

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CThreadManagerThreadContext_AC::CThreadManagerThreadContext_AC() :
	CAbstractThreadContext_AC(NULL),
	fThreadID(kNoThreadID),
	fEntry(NULL),
	fParameter(NULL),
	fResume(false),
	fDone(false)
{
	OSErr err = ::GetThreadCurrentTaskRef(&fTaskRef);
	if (fMap.get() == NULL)
	{
		fMap = TH_new map<ThreadID,CThreadManagerThreadContext_AC* ,less<ThreadID>, allocator<CThreadManagerThreadContext_AC*> >();
#if qYieldToOtherProcesses || qWantsMP
		InitializeYieldThread();	// Make sure the yielding thread is initialized.
#endif
	}
	::GetCurrentThread(&fThreadID);
	pair<const ThreadID, CThreadManagerThreadContext_AC* > threadMapPair(fThreadID, this);
	fMap->insert(threadMapPair);
}

//----------------------------------------------------------------------------------------
// This is the callback proc
//----------------------------------------------------------------------------------------

PASCAL_AC void* CThreadManagerThreadContext_AC::ThreadMgrWrapperProc(CThreadManagerThreadContext_AC* context)
{
	(context->fEntry)(context->fParameter);
	context->fDone = true;
	return NULL;
}

//----------------------------------------------------------------------------------------
// This is a static pointer to the callback proc
//----------------------------------------------------------------------------------------

#if GENERATING68K && GENERATINGCFM
ThreadEntryProcPtr CThreadManagerThreadContext_AC::fWrapperPtr = 
	::NewThreadEntryProc(CThreadManagerThreadContext_AC::ThreadMgrWrapperProc);
#else
ThreadEntryProcPtr CThreadManagerThreadContext_AC::fWrapperPtr = 
	(ThreadEntryProcPtr)CThreadManagerThreadContext_AC::ThreadMgrWrapperProc;
#endif

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CThreadManagerThreadContext_AC::CThreadManagerThreadContext_AC(
			ThreadStyle threadStyle, SimpleTaskProcPtr threadEntry,
			void *threadParam, Size stackSize, ThreadOptions options,
			void **threadResult,
			CThread_AC* adoptedThread) :
	CAbstractThreadContext_AC(adoptedThread),
	fThreadID(kNoThreadID),
	fEntry(threadEntry),
	fParameter(threadParam),
	fResume(false),
	fDone(false)
{
	OSErr err;
	if (fMap.get() == NULL)
	{
		fMap = TH_new map<ThreadID,CThreadManagerThreadContext_AC* ,less<ThreadID>, allocator<CThreadManagerThreadContext_AC*> >();

#if qYieldToOtherProcesses || qWantsMP
		InitializeYieldThread();	// Make sure the yielding thread is initialized.
#endif
	}
	err = ::GetThreadCurrentTaskRef(&fTaskRef);
	err = ::NewThread(threadStyle, fWrapperPtr,
			this, stackSize, options,
			threadResult, &fThreadID);

	::ThrowIfError_AC(err);
	// Add context to static list keyed by thread ID.  List must be thread safe :-)
	pair<const ThreadID, CThreadManagerThreadContext_AC* > threadMapPair(fThreadID, this);
	fMap->insert(threadMapPair);
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CThreadManagerThreadContext_AC::~CThreadManagerThreadContext_AC()
{
	fMap->erase(fThreadID);
	
	// Do not call DisposeThread here because this destructor may get called from 
	// within the thread (by the CThreadHandle in the ThreadPrologue), and if that
	// happens, we would leak our schedule, its monitor lock, and our thread.
	// When the thread completes, the ThreadMgr will cleanup after itself.
	// ::DisposeThread(fThreadID, 0, true);
}

//----------------------------------------------------------------------------------------
// Suspend
//----------------------------------------------------------------------------------------

void CThreadManagerThreadContext_AC::Suspend()
{
	DEBUG_SETUP_STACK_FRAME();

	OSErr err = ::ThreadBeginCritical();
	while (!fResume)
	{
		err = ::SetThreadStateEndCritical(fThreadID, kStoppedThreadState, kNoThreadID);
		err = ::ThreadBeginCritical();
	}
	fResume = false;
	err = ::ThreadEndCritical();
}

//----------------------------------------------------------------------------------------
// Resume
//----------------------------------------------------------------------------------------

void CThreadManagerThreadContext_AC::Resume()
{
	DEBUG_SETUP_STACK_FRAME();

#if !qPowerPC
	OSErr err = ::ThreadBeginCritical();
#endif

#if qWantsMP
	if (::HasMP_AC() && (CMPThreadContext_AC::fMainContext.get() != NULL)
			&& CMPThreadContext_AC::GetCurrentMPContext() != CMPThreadContext_AC::fMainContext)
	{
		// Queue this context to be waken by the cooperative context.
		::MPNotifyQueue(GetCooperativeRequestQueue(), this, NULL, NULL);
	}
	else
#endif
	{
		fResume = true;
		
		// Use of this call from MP contexts would need to be validated by system software.
		::SetThreadReadyGivenTaskRef(fTaskRef, fThreadID);
	}

#if !qPowerPC
	err = ::ThreadEndCritical();
#endif
}

//----------------------------------------------------------------------------------------
// IsAlive
//----------------------------------------------------------------------------------------

bool CThreadManagerThreadContext_AC::IsAlive()
{
	ThreadState state;
	return ((fDone == false) && (GetThreadState(fThreadID, &state) != threadNotFoundErr));
}

//----------------------------------------------------------------------------------------
// Yield
//----------------------------------------------------------------------------------------

void CThreadManagerThreadContext_AC::Yield()
{
	::YieldToAnyThread();
}

//----------------------------------------------------------------------------------------
// GetThreadID
//----------------------------------------------------------------------------------------

ThreadID CThreadManagerThreadContext_AC::GetThreadID()
{
	return fThreadID;
}

//----------------------------------------------------------------------------------------
// GetCurrentThreadCount
//----------------------------------------------------------------------------------------

long CThreadManagerThreadContext_AC::GetCurrentThreadCount()
{
	long result = 1;	// Count the main thread.
	if (fMap.get() != NULL)
	{
		result += fMap->size();
	}
	return result;
}
