//----------------------------------------------------------------------------------------
// CThread_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------

#ifndef __CThread_AC__
#include "CThread_AC.h"
#endif

#ifndef __CAbstractThreadSchedule_AC__
#include "CAbstractThreadSchedule_AC.h"
#endif

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

//----------------------------------------------------------------------------------------

CAutoPtr_AC<CAbstractThreadSchedule_AC> CThread_AC::fSchedule;

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

CThread_AC::CThread_AC(const CThread_AC& /*source*/) :
	fShutdownRequested(false)
{
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

CThread_AC::CThread_AC() :
	fShutdownRequested(false)
{
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

CThread_AC::~CThread_AC()
{
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

CThread_AC& CThread_AC::operator=(const CThread_AC& /*sourceThread*/)
{
	return *this;
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

CAbstractThreadSchedule_AC* CThread_AC::GetThreadSchedule()
//
//	Returns the schedule object to be used in the thread, creating one if not created yet.
//	Override this method if you dont want the default thread schedule.
//
{
	if (fSchedule.IsNULL()) {
		fSchedule = CAbstractThreadSchedule_AC::CreateThreadSchedule();
	}
	return fSchedule;
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

size_t CThread_AC::GetStackSize() const
//
//	Returns the size of the stack to be set up for the thread.
//	Override this method if you dont want the default stack size.
//
{
	return 65536;	// 64Kbyte stack.
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

void CThread_AC::Prepare()
//
//	Prepare() is called in the new thread before the CThreadRunner_AC constructor completes.
//	It should not be called by clients.
//
{
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

void CThread_AC::RequestShutdown()
//
// RequestShutdown() may be called either directly or through the Thread runner to
// request that the thread gracefully terminate itself.  It will have no effect
// unless the CThread_AC subclass periodically checks the value returned by
// ShutdownRequested in its Run method.
// Override this method only if immediate processing other than setting the
// fShudownRequested flag is required.
//
{
	fShutdownRequested = true;
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

bool CThread_AC::ShutdownRequested()
//
// Returns true if RequestShutdown has been called.  Subclasses should gracefully
// terminate their execution when this call returns true.
//
{
	return fShutdownRequested;
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

void CThread_AC::Yield() // static
//
// Yields this thread, allowing other (cooperatively run) threads time to execute.
//
{
	CAbstractThreadContext_AC::YieldCurrentContext();
}

//----------------------------------------------------------------------------------------
// CThread_AC
//----------------------------------------------------------------------------------------

void CThread_AC::Suspend() // static
//
// Suspends this thread.  Must be called from within the thread (e.g., Run).
//
{
	CAbstractThreadContext_AC::SuspendCurrentContext();
}
