//----------------------------------------------------------------------------------------
// CThreadedIdleEngine_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CThreadedIdleEngine_AC__
#include "CThreadedIdleEngine_AC.h"
#endif

// ACS patterns

#ifndef __CIdleTimeDispenser_AC__
#include "CIdleTimeDispenser_AC.h"
#endif

// ACS threads

#ifndef __CThread_AC__
#include "CThread_AC.h"
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CProperty_AC__
#include "CProperty_AC.h"
#endif

#ifndef __CAbstractTask_AC__
#include "CAbstractTask_AC.h"
#endif


//----------------------------------------------------------------------------------------
// CYieldTask_AC is used internally to provide for flexible yielding
//----------------------------------------------------------------------------------------

class CYieldTask_AC : public CAbstractTask_AC
{
public:
	CYieldTask_AC() {}
	
	virtual ~CYieldTask_AC() {}
	
	virtual void DoTask()
	{
		CThread_AC::Yield();
		DidYield(true);
	}
	
	CProperty_AC DidYield;
};

//----------------------------------------------------------------------------------------
// CIdleEngineThread is used internally by CThreadedIdleEngine to perform threading.
//----------------------------------------------------------------------------------------

class CIdleEngineThread_AC : public CThread_AC
{
public:
	CIdleEngineThread_AC(CThreadedIdleEngine_AC* reactor);
		
	virtual ~CIdleEngineThread_AC();

	virtual void Run();
	
private:
	CThreadedIdleEngine_AC*	fIdleEngine;
	CRefCountingPtr_AC<CYieldTask_AC> fYieldTask;	
};

//========================================================================================
//	CThreadedIdleEngine_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CThreadedIdleEngine_AC::CThreadedIdleEngine_AC()
{
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CThreadedIdleEngine_AC::~CThreadedIdleEngine_AC()
{
	Stop();
}

//----------------------------------------------------------------------------------------
// CThreadedIdleEngine_AC::Run
//----------------------------------------------------------------------------------------

void CThreadedIdleEngine_AC::Run()
{
	if (fThreadRunner.IsNULL()) 
	{
		fThreadRunner = TH_new CThreadRunner_AC(TH_new CIdleEngineThread_AC(this));
	}
}

//----------------------------------------------------------------------------------------
// CThreadedIdleEngine_AC::Stop
//----------------------------------------------------------------------------------------

void CThreadedIdleEngine_AC::Stop()
{
	
	
	
	if (fThreadRunner)
	{	
		fThreadRunner.reset();	// deletes it, which Joins and cleans up the thread
	}
}

//----------------------------------------------------------------------------------------
// CThreadedIdleEngine_AC::IsRunning
//----------------------------------------------------------------------------------------

bool CThreadedIdleEngine_AC::IsRunning() const
{
	return !fThreadRunner.IsNULL();
}



//========================================================================================
//	CIdleEngineThread_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CIdleEngineThread_AC::CIdleEngineThread_AC(CThreadedIdleEngine_AC* reactor) :
	fIdleEngine(reactor)
{
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------
CIdleEngineThread_AC::~CIdleEngineThread_AC()
{
}

//----------------------------------------------------------------------------------------
// CIdleEngineThread_AC::Run
//----------------------------------------------------------------------------------------

#include "CTraceStream_AC.h"

void CIdleEngineThread_AC::Run()
{
	while (!ShutdownRequested())
	{
		if(fYieldTask.IsNULL())
		{
			fYieldTask = TH_new CYieldTask_AC;
			fIdleEngine->GetDispenser()->SetYieldTask(fYieldTask);	
			fYieldTask->DidYield(false);
		}
		else
		if(fIdleEngine->GetDispenser())
		{
			fYieldTask->DidYield(false);
			fIdleEngine->GetDispenser()->Idle();
		}
		else
		{
			RequestShutdown();
		}

		if(fYieldTask.IsNULL() || fYieldTask->DidYield() == false)
			Yield();
		// Yield is called by the yield task.
	}
}