//----------------------------------------------------------------------------------------
// CTime_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------


#ifndef __CTime_AC__
#include "CTime_AC.h"
#endif

#include <fp.h>

// ---------------------------------------------------------------------------
//	CTime_AC
// ---------------------------------------------------------------------------

// Conversion constants.
const double CTime_AC::kPicosecondsToNanoseconds	= 1.E3;
const double CTime_AC::kPicosecondsToMicroseconds	= 1.E6;
const double CTime_AC::kPicosecondsToMilliseconds	= 1.E9;
const double CTime_AC::kPicosecondsToSeconds		= 1.E12;
const double CTime_AC::kPicosecondsToMinutes		= 6.E13;
const double CTime_AC::kPicosecondsToHours			= 3.6E15;
const double CTime_AC::kPicosecondsToDays			= 8.64E16;

// positive infinity
const CPicoseconds_AC posInfinity ((double)HUGE_VAL);	
const CTime_AC& CTime_AC::kPositiveInfinity = posInfinity;

// negative infinity
const CPicoseconds_AC negInfinity ( -(double)HUGE_VAL);
const CTime_AC& CTime_AC::kNegativeInfinity = negInfinity;

// zero time
const CPicoseconds_AC zeroTime (0.0);
const CTime_AC& CTime_AC::kZero = zeroTime;

// ---------------------------------------------------------------------------
//	CTime
// ---------------------------------------------------------------------------

CTime_AC::CTime_AC ()
{
}
CTime_AC::CTime_AC (const CTime_AC& theTime)
	: fTime (theTime.fTime)
{
}

CTime_AC::~CTime_AC()
{
}

void CTime_AC::SetPicoseconds(double thePicoseconds)
{
	fTime = thePicoseconds;
}

// ---------------------------------------------------------------------------
//	CPicoseconds_AC
// ---------------------------------------------------------------------------

CPicoseconds_AC::CPicoseconds_AC ()
	: CTime_AC ()
{
}

CPicoseconds_AC::CPicoseconds_AC (double thePicoseconds)
//	: CTime_AC (thePicoseconds)
{
	SetPicoseconds(thePicoseconds);
}

CPicoseconds_AC::CPicoseconds_AC (const CPicoseconds_AC& thePicoseconds)
	: CTime_AC (thePicoseconds)
{
}

CPicoseconds_AC::CPicoseconds_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CPicoseconds_AC::~CPicoseconds_AC ()
{
}

// ---------------------------------------------------------------------------
//	CNanoseconds_AC
// ---------------------------------------------------------------------------

CNanoseconds_AC::CNanoseconds_AC ()
	: CTime_AC ()
{
}

CNanoseconds_AC::CNanoseconds_AC (double theNanoseconds)
{
	SetPicoseconds(theNanoseconds * kPicosecondsToNanoseconds);
}

CNanoseconds_AC::CNanoseconds_AC (const CNanoseconds_AC& theNanoseconds)
	: CTime_AC (theNanoseconds)
{
}

CNanoseconds_AC::CNanoseconds_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CNanoseconds_AC::~CNanoseconds_AC ()
{
}

// ---------------------------------------------------------------------------
//	CMicroseconds_AC
// ---------------------------------------------------------------------------

CMicroseconds_AC::CMicroseconds_AC ()
	: CTime_AC ()
{
}

CMicroseconds_AC::CMicroseconds_AC (double theMicroseconds)
{
	SetPicoseconds(theMicroseconds * kPicosecondsToMicroseconds);
}

CMicroseconds_AC::CMicroseconds_AC (const CMicroseconds_AC& theMicroseconds)
	: CTime_AC (theMicroseconds)
{
}

CMicroseconds_AC::CMicroseconds_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CMicroseconds_AC::~CMicroseconds_AC ()
{
}

// ---------------------------------------------------------------------------
//	CMilliseconds_AC
// ---------------------------------------------------------------------------

CMilliseconds_AC::CMilliseconds_AC ()
	: CTime_AC ()
{
}

CMilliseconds_AC::CMilliseconds_AC (double theMilliseconds)
{
	SetPicoseconds(theMilliseconds * kPicosecondsToMilliseconds);
}

CMilliseconds_AC::CMilliseconds_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CMilliseconds_AC::CMilliseconds_AC (const CMilliseconds_AC& theTime)
	: CTime_AC (theTime)
{
}

CMilliseconds_AC::~CMilliseconds_AC ()
{
}

// ---------------------------------------------------------------------------
//	CSeconds_AC
// ---------------------------------------------------------------------------

CSeconds_AC::CSeconds_AC ()
	: CTime_AC ()
{
}

CSeconds_AC::CSeconds_AC (double theSeconds)
{
	SetPicoseconds(theSeconds * kPicosecondsToSeconds);
}

CSeconds_AC::CSeconds_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CSeconds_AC::CSeconds_AC (const CSeconds_AC& theTime)
	: CTime_AC (theTime)
{
}


CSeconds_AC::~CSeconds_AC ()
{
}

// ---------------------------------------------------------------------------
//	CMinutes_AC
// ---------------------------------------------------------------------------

CMinutes_AC::CMinutes_AC (double theMinutes)
{
	SetPicoseconds(theMinutes * kPicosecondsToMinutes);
}

CMinutes_AC::CMinutes_AC ()
	: CTime_AC ()
{
}

CMinutes_AC::CMinutes_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CMinutes_AC::CMinutes_AC (const CMinutes_AC& theTime)
	: CTime_AC (theTime)
{
}

CMinutes_AC::~CMinutes_AC ()
{
}

// ---------------------------------------------------------------------------
//	CHours_AC
// ---------------------------------------------------------------------------
CHours_AC::CHours_AC (double theHours)
//	: CTime_AC (theHours * kHoursToPico)
{
	SetPicoseconds(theHours * kPicosecondsToHours);
}

CHours_AC::CHours_AC ()
	: CTime_AC ()
{
}

CHours_AC::CHours_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CHours_AC::CHours_AC (const CHours_AC& theTime)
	: CTime_AC (theTime)
{
}

CHours_AC::~CHours_AC ()
{
}

// ---------------------------------------------------------------------------
//	CDays_AC
// ---------------------------------------------------------------------------

CDays_AC::CDays_AC (double theDays)
{
	SetPicoseconds(theDays * kPicosecondsToDays);
}

CDays_AC::CDays_AC ()
	: CTime_AC ()
{
}

CDays_AC::CDays_AC (const CTime_AC& theTime)
	: CTime_AC (theTime)
{
}

CDays_AC::CDays_AC (const CDays_AC& theTime)
	: CTime_AC (theTime)
{
}

CDays_AC::~CDays_AC ()
{
}

// ---------------------------------------------------------------------------
//	End of File CTime_AC.cp
