//----------------------------------------------------------------------------------------
// CWinSchedule_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CWinSchedule_AC__
#include "CWinSchedule_AC.h"
#endif

#if 0 

#ifndef	__CWinMonitorLock_AC__
#include "CWinMonitorLock_AC.h"
#endif

#ifndef __CWinThreadContext_AC__
#include "CWinThreadContext_AC.h"
#endif


//----------------------------------------------------------------------------------------
// Dummy pthread constants.
//----------------------------------------------------------------------------------------

#define PRI_OTHER_MIN 1
#define PRI_OTHER_MAX 1
#define SCHED_OTHER 1

//----------------------------------------------------------------------------------------
// CWinSchedule_AC class requires some constants to be used
//----------------------------------------------------------------------------------------

const int				kWinNumberOfLevels		= (PRI_OTHER_MAX - PRI_OTHER_MIN + 1) / 4;
const int				kWinDefaultLevel			= kWinNumberOfLevels / 2;
const AbsolutePriority 	kWinMaximumPriority		= PRI_OTHER_MAX - kWinNumberOfLevels;
const AbsolutePriority 	kWinMinimumPriority		= kWinMaximumPriority - kWinNumberOfLevels + 1;
const SchedulingPolicy	kWinPolicy				= SCHED_OTHER;

const PriorityLevels 	CWinSchedule_AC::kNumberOfLevels	= kWinNumberOfLevels;
const AbsolutePriority	CWinSchedule_AC::kMaximumPriority	= kWinMaximumPriority;
const AbsolutePriority	CWinSchedule_AC::kMinimumPriority	= kWinMinimumPriority;
const SchedulingPolicy	CWinSchedule_AC::kPolicy			= kWinPolicy;
const AbsolutePriority	CWinSchedule_AC::kDefaultPriority	= kWinMaximumPriority - kWinDefaultLevel;

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CWinSchedule_AC::CWinSchedule_AC(ThreadStyle style) :
	fStyle(style),
	fPriority(kDefaultPriority)
{

}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CWinSchedule_AC::~CWinSchedule_AC()
{
}

//----------------------------------------------------------------------------------------
// Copy constructor
//----------------------------------------------------------------------------------------

CWinSchedule_AC::CWinSchedule_AC (const CWinSchedule_AC& aWinSchedule) :
	CAbstractThreadSchedule_AC(aWinSchedule),
	fPriority(aWinSchedule.fPriority)
{
}

//----------------------------------------------------------------------------------------
// Copy assignment
//----------------------------------------------------------------------------------------

CWinSchedule_AC& CWinSchedule_AC::operator =(const CWinSchedule_AC& aWinSchedule)
{
	fPriority = aWinSchedule.fPriority;
	CAbstractThreadSchedule_AC::operator=(aWinSchedule);
	return (*this);
}

//----------------------------------------------------------------------------------------
// CreateThreadContext
//----------------------------------------------------------------------------------------

CAbstractThreadContext_AC* CWinSchedule_AC::CreateThreadContext(size_t stackSize, 
			SimpleTaskProcPtr entry, void* parameter,
			CThread_AC* adoptedThread)
{
	return TH_new CWinThreadContext_AC(0/*fStyle*/, stackSize, entry, parameter, 0, NULL, adoptedThread);
}

//----------------------------------------------------------------------------------------
// CreateMonitorLock
//----------------------------------------------------------------------------------------

CAbstractMonitorLock_AC* CWinSchedule_AC::CreateMonitorLock()
{
	return TH_new CWinMonitorLock_AC();
}

//----------------------------------------------------------------------------------------
// Getter methods
//----------------------------------------------------------------------------------------

AbsolutePriority CWinSchedule_AC::GetPriority() const
{
	return ( fPriority );
}

//----------------------------------------------------------------------------------------

SchedulingPolicy  CWinSchedule_AC::GetPolicy() const
{
	return ( kPolicy );
}

//----------------------------------------------------------------------------------------

AbsolutePriority CWinSchedule_AC::GetMaximumPriority() const
{
	return ( kMaximumPriority );
}

//----------------------------------------------------------------------------------------

AbsolutePriority CWinSchedule_AC::GetMinimumPriority() const
{
	return ( kMinimumPriority );
}

//----------------------------------------------------------------------------------------
// Setter methods
//----------------------------------------------------------------------------------------

void CWinSchedule_AC::SetPriority(const AbsolutePriority aPriority)
{
	fPriority = aPriority;
}

#endif // qMSWindows