//----------------------------------------------------------------------------------------
// WinThreadContext.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CWinThreadContext_AC__
#include "CWinThreadContext_AC.h"
#endif

#if 0 

#ifndef MAP_H
#include <map.h>
#endif

//----------------------------------------------------------------------------------------
//	Class CWinThreadContext_AC provides a wrapper for creating MSWindows threads.
//----------------------------------------------------------------------------------------

CAutoPtr_AC<map<DWORD,CWinThreadContext_AC* ,less<DWORD>, allocator<CWinThreadContext_AC*> > > CWinThreadContext_AC::fMap;
CAutoPtr_AC<CWinThreadContext_AC> CWinThreadContext_AC::fMainContext;

//----------------------------------------------------------------------------------------

CWinThreadContext_AC* CWinThreadContext_AC::GetCurrentWinContext()
{
	CWinThreadContext_AC* result = NULL;
	if (fMap.get() != NULL)
	{
		DWORD currentThread = ::GetCurrentThreadId();
		// Lookup context in static list using ID as key.
		if (!fMap->empty())
		{
			map<DWORD,CWinThreadContext_AC* ,less<DWORD>, allocator<CWinThreadContext_AC*> >::iterator threadMapIterator = fMap->find(currentThread);
			if ( (*threadMapIterator).first == currentThread)
			{
				result = (*threadMapIterator).second;
			}
		}
	}
	if (result == NULL)
	{
		// Main context still needs to be created!
		if (fMainContext.get() == NULL)
		{
			fMainContext = TH_new CWinThreadContext_AC();
			result = fMainContext.get();
		}
	}
	return result;
}

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CWinThreadContext_AC::CWinThreadContext_AC(
			LPSECURITY_ATTRIBUTES threadAttributes, DWORD stackSize,
			SimpleTaskProcPtr threadEntry, void* threadParam,
			DWORD creationFlags,
			CThread_AC* adoptedThread) :
	CAbstractThreadContext_AC(adoptedThread),
	fThreadID(kNoWinThreadID),
	fEntry(NULL),
	fParameter(NULL),
	fDone(false)
{
	if (fMap.get() == NULL)
	{
		fMap = TH_new map<DWORD,CWinThreadContext_AC* ,less<DWORD>, allocator<CWinThreadContext_AC*> >();
#if qYieldToOtherProcesses || qWantsMP
		InitializeYieldThread();	// Make sure the yielding thread is initialized.
#endif
	}

	fThreadRef = ::CreateThread(threadAttributes, stackSize, threadEntry, threadParam, creationFlags, &fThreadID);

	::ThrowIfNULL_AC(fThreadRef);
	// Add context to static list keyed by thread ID.  List must be thread safe :-)
	pair<const DWORD, CWinThreadContext_AC* > threadMapPair(fThreadID, this);
	fMap->insert(threadMapPair);
}

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

CWinThreadContext_AC::CWinThreadContext_AC() :
	CAbstractThreadContext_AC(NULL),
	fThreadID(kNoWinThreadID),
	fEntry(NULL),
	fParameter(NULL),
	fDone(false)
{
	if (fMap.get() == NULL)
	{
		fMap = TH_new map<DWORD,CWinThreadContext_AC* ,less<DWORD>, allocator<CWinThreadContext_AC*> >();
#if qYieldToOtherProcesses || qWantsMP
		InitializeYieldThread();	// Make sure the yielding thread is initialized.
#endif
	}

	pair<const DWORD, CWinThreadContext_AC* > threadMapPair(fThreadID, this);
	fMap->insert(threadMapPair);
}

//----------------------------------------------------------------------------------------
// This is the callback proc
//----------------------------------------------------------------------------------------

void* CWinThreadContext_AC::WinWrapperProc(CWinThreadContext_AC* context)
{
	(context->fEntry)(context->fParameter);
	context->fDone = true;
	return NULL;
}

//----------------------------------------------------------------------------------------
// This is a static pointer to the callback proc
//----------------------------------------------------------------------------------------

LPTHREAD_START_ROUTINE CWinThreadContext_AC::fWrapperPtr = 
	(LPTHREAD_START_ROUTINE) CWinThreadContext_AC::WinWrapperProc;

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CWinThreadContext_AC::~CWinThreadContext_AC()
{
	fMap->erase(fThreadID);

	::TerminateThread(fThreadRef, 0);
}

//----------------------------------------------------------------------------------------
// Suspend
//----------------------------------------------------------------------------------------

void CWinThreadContext_AC::Suspend()
{
	
	DWORD result = ::SuspendThread(fThreadRef);
}

//----------------------------------------------------------------------------------------
// Resume
//----------------------------------------------------------------------------------------

void CWinThreadContext_AC::Resume()
{
	
	DWORD result = ::ResumeThread(fThreadRef);
}

//----------------------------------------------------------------------------------------
// IsAlive
//----------------------------------------------------------------------------------------

bool CWinThreadContext_AC::IsAlive()
{
	
	return (fDone == false);
}

//----------------------------------------------------------------------------------------
// Yield
//----------------------------------------------------------------------------------------

void CWinThreadContext_AC::Yield()
{
	
	::Sleep(0);
}

//----------------------------------------------------------------------------------------
// GetThreadID
//----------------------------------------------------------------------------------------

DWORD CWinThreadContext_AC::GetThreadID()
{
	return fThreadID;
}

//----------------------------------------------------------------------------------------
// GetThreadID
//----------------------------------------------------------------------------------------

HANDLE CWinThreadContext_AC::GetThreadRef()
{
	return fThreadRef;
}

//----------------------------------------------------------------------------------------
// GetCurrentThreadCount
//----------------------------------------------------------------------------------------

long CWinThreadContext_AC::GetCurrentThreadCount()
{
	long result = 1;	// Includes the main thread.
	if (fMap.get() != NULL)
	{
		result = fMap->size();
		if (fMainContext.get() == NULL)
			result += 1;	// Add one for main thread if its context is not yet in the collection.
	}
	return result;
}

#endif // qMSWindows
