//----------------------------------------------------------------------------------------
// CYieldThread_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CYieldThread_AC__
#include "CYieldThread_AC.h"
#endif

#if qYieldToOtherProcesses || qWantsMP

#ifndef __CAbstractThreadContext_AC__
#include "CAbstractThreadContext_AC.h"
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CThreadMgrThreadContext_AC__
#include "CThreadMgrThreadContext_AC.h"
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

//----------------------------------------------------------------------------------------
// CYieldThread_AC::CYieldThread_AC
//----------------------------------------------------------------------------------------

CYieldThread_AC::CYieldThread_AC()
{
}

//----------------------------------------------------------------------------------------
// CYieldThread_AC::~CYieldThread_AC
//----------------------------------------------------------------------------------------

CYieldThread_AC::~CYieldThread_AC()
{
}

//----------------------------------------------------------------------------------------
// CYieldThread_AC::Run
//----------------------------------------------------------------------------------------

void CYieldThread_AC::Run()
{
	CAbstractThreadContext_AC* context = CAbstractThreadContext_AC::GetCurrentContext();
#if qWantsMP
	OSStatus result;
	if (::HasMP_AC()) 
	{
		MPQueueID theID;
		result = ::MPCreateQueue(&theID);
		SetCooperativeRequestQueue(theID);
	}
#endif

	while (!ShutdownRequested() && (context->GetRefCount() > 1)) 
	{	// Yield while static handle exists.
		// This is where support for qMonitorCancel will eventually be added.
#if qYieldToOtherProcesses
		::SystemTask();	// Yields to other processes, as well as MP tasks on a uniprocessor.
#elif qWantsMP
		if (::HasMP_AC()) 
		{
			::MPYield();	// MP tasks do not preempt the cooperative context on a uniprocessor.
			void* request = NULL;
			result = ::MPWaitOnQueue(GetCooperativeRequestQueue(),
										&request, NULL, NULL, kDurationImmediate);
			if (result == noErr) 
			{
				// Currently, queue is only used for resuming cooperative threads.
				CThreadManagerThreadContext_AC* thread = (CThreadManagerThreadContext_AC*) request;
				if (thread)
					thread->Resume();
			}
		}
#endif
		::YieldToAnyThread();
#if !qYieldToOtherProcesses 
		if (::HasMP_AC()) // Cooperative yielding is only necessary when there are MP tasks
		{
			// Yield occasionally to avoid locking the system when the main task is blocked.
			static int i = 0;
			if (i >= 100) 
			{
				i = 0;
				// Yields to other processes, as well as MP tasks on a uniprocessor.
				EventRecord ignored;
				bool dontCare = ::EventAvail(everyEvent, &ignored);
			}
			else 
			{
				++i;
			}
		}
#endif
	}
#if qWantsMP
	if (::HasMP_AC()) 
		result = ::MPDeleteQueue(GetCooperativeRequestQueue());
#endif
}

//----------------------------------------------------------------------------------------
// CYieldThread::Initialize
//----------------------------------------------------------------------------------------

void CYieldThread_AC::Initialize()
// Starts the thread that yields to MP or other tasks.
{
	if (!fgInitialized)
	{
		fgInitialized = true; // set immediately so we don't recur
		fgYieldThread = TH_new CThreadRunner_AC(TH_new CYieldThread_AC());
	}
}

//----------------------------------------------------------------------------------------
// Static thread handle
//----------------------------------------------------------------------------------------

CAutoPtr_AC<CThreadRunner_AC>	CYieldThread_AC::fgYieldThread;
bool							CYieldThread_AC::fgInitialized = false;

//----------------------------------------------------------------------------------------
// Static global initialization
//----------------------------------------------------------------------------------------

#if qWantsMP
MPQueueID CYieldThread_AC::fCooperativeRequestQueue = NullMPQueueID;
#endif

#endif // qYieldToOtherProcesses || qWantsMP

