//----------------------------------------------------------------------------------------
// ThreadUtilities_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CThreadMgrThreadContext_AC__
#include "CThreadMgrThreadContext_AC.h"
#endif

#if qWantsMP

#ifndef __CMPThreadContext_AC__
#include "CMPThreadContext_AC.h"
#endif

#ifndef MP_h
#include <MP.h>
#endif

#endif

#ifndef __CThreadedReactorFactory_AC__
#include "CThreadedReactorFactory_AC.h"
#endif


#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __THREADS__
#include <Threads.h>
#endif

#ifndef __CODEFRAGMENTS__
#include <CodeFragments.h>
#endif

//----------------------------------------------------------------------------------------
// HasThreadManager_AC:
//----------------------------------------------------------------------------------------


bool HasThreadManager_AC()
{
	static bool defined = false;
	static bool hasThreadManager;
	if (!defined)
	{
		defined = true;
		long response;
		hasThreadManager = Gestalt('thds', &response) == noErr && response & 1;
#if GENERATINGPOWERPC || GENERATINGCFM
		// verify presence of shared library
		hasThreadManager = hasThreadManager && ((Ptr)NewThread != (Ptr)kUnresolvedCFragSymbolAddress);
#endif
	}
	return hasThreadManager;
}

//----------------------------------------------------------------------------------------
// HasMP_AC:
//----------------------------------------------------------------------------------------


bool HasMP_AC()
{
	static bool defined = false;
	static bool hasMP;
	if (!defined)
	{
		defined = true;
#if GENERATINGPOWERPC  && qWantsMP
		hasMP = MPLibraryIsLoaded();
#else
		hasMP = false;
#endif
	}
	return hasMP;
}

//----------------------------------------------------------------------------------------
// IdleThreads_AC:
//----------------------------------------------------------------------------------------


void IdleThreads_AC(bool /*frontProcess*/, long /*idleFrequency*/)
{
#if qWantsThreadManager || qWantsMP
	long threadCount = CThreadManagerThreadContext_AC::GetCurrentThreadCount();
#if qYieldToOtherProcesses || qWantsMP
	if (threadCount > 0) 
		threadCount -= 1; // Disregard the yield thread.
#endif
	// If there are any threads, yield to them.  The Thread manager will handle time management.
	if (threadCount > 0) 
	{
		::YieldToAnyThread();
	}
#endif
}

//----------------------------------------------------------------------------------------
// AssertToolboxSafeMPTask_AC:
//----------------------------------------------------------------------------------------


#if qWantsMP

void AssertToolboxSafeMPTask_AC()
{
	if (::HasMP_AC())
	{
		MPTaskID currentTask = ::MPCurrentTaskID();
		// _MP* debug interfaces require Daystar MP headers and library.
		if (!_MPTaskIsToolboxSafe(currentTask))
			::_MPDebugStr("\pInvalid call to new or delete from an unsafe MP task");
	}
}

#endif

//----------------------------------------------------------------------------------------
// InitThreads_AC
//----------------------------------------------------------------------------------------

void InitThreads_AC()
{
	
	// to be dead stripped and this line of code will haul all that in.

	CReactorFactory_AC::SetDefaultFactory(TH_new CThreadedReactorFactory_AC);
}
