//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeapBlock_AC__
#define __CTidyHeapBlock_AC__

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

#if qTidyHeap

// TidyHeap
#ifndef __TidyHeapTypes_AC__
#include "TidyHeapTypes_AC.h"
#endif

#ifndef __CTidyHeapListItem_AC__
#include "CTidyHeapListItem_AC.h"
#endif
#ifndef __CTidyHeapFlag_AC__
#include "CTidyHeapFlag_AC.h"
#endif


// ANSI
#ifndef __STDDEF__
#include <stddef.h>
#endif

//----------------------------------------------------------------------------------------
// class CTidyHeapBlock_AC
//----------------------------------------------------------------------------------------

struct BlockHeader;
class EXPORT_AC CTidyHeapBlock_AC : public CTidyHeapListItem_AC
{
friend class CTidyHeap_AC;

protected:

	CTidyHeapBlock_AC();

	virtual ~CTidyHeapBlock_AC();

public:
//*** Info Getters ****//

	long GetID() const;

	long GetSize() const;

	void* GetAddress() const;
	
	const char* GetFileName() const;

	short GetLineNum() const;	
		// Note: file name and line num may not be valid. If they're not,
		// filename will be an empty string and linenum will be zero, i.e.
		// GetFileName will always return a valid ptr.
	
	CTidyHeapMarker_AC GetMarker( short markerNum );
		// valid markers are 1-3 only!
					
//*** Getters/Setters ****//
			
	void SetRefCon(void* inWhatever);
		// Use this for whatever you wish.

	void* GetRefCon() const;
		
	void AquireDeleteAction( CTidyHeapAction_AC* action );
		// Send in nil to remove delete action.

	CTidyHeapAction_AC* GetDeleteAction() const;
		// Blocks don't have a delete action by default.
	
//** Utils **//	
	
	CTidyHeapError_AC Verify();
		// This verifys the block. Returns an error if there is a prob.

	void LogBlockData();
		// This logs the block's data to the currently installed logger, if there
		// is one.

	CTidyHeapFlag_AC WasDeleted;
	CTidyHeapFlag_AC DontDelete;
	
	bool IsExempt() const;

	void SetExempt(bool inBool);

	// Advanced users only!

	static CTidyHeapBlock_AC* GetHeadOfList();
		// for walking the tidy heap list. This gets the head of the list

	CTidyHeapBlock_AC* Next();
		// Gets the next block header in the list.

private:
	CTidyHeapFlag_AC 	IsExemptFlag;

	long 				fId;
	long 				fSize;
	void*  				fRefCon;
	void*  				fAddress;
	
	CTidyHeapAction_AC* fDeleteAction;
	
	char 				fFile[32];
	short 				fLineNum;
	
	BlockHeader* 		fHeader;

};

//----------------------------------------------------------------------------------------
// inlines
//----------------------------------------------------------------------------------------

inline CTidyHeapBlock_AC* CTidyHeapBlock_AC::Next() 	
{	
	return (CTidyHeapBlock_AC*) CTidyHeapListItem_AC::Next();  
}


#endif
#endif