//----------------------------------------------------------------------------------------
// CTidyHeapFile_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeapFile_AC__
#define __CTidyHeapFile_AC__

// Toolbox

#ifndef __FILES__
#include <Files.h>
#endif

// TidyHeap

#ifndef __MTidyHeapAllocator_AC__
#include "MTidyHeapAllocator_AC.h"
#endif

//========================================================================================
// class CTidyHeapFile_AC
//========================================================================================

class MTidyHeapPersistantObject_AC;
class EXPORT_AC CTidyHeapFile_AC : public MTidyHeapAllocator_AC
{
public:
	
	// constructors/destructors
	
	CTidyHeapFile_AC();
	
	CTidyHeapFile_AC(const CTidyHeapFile_AC & inFile);
	
	CTidyHeapFile_AC(const FSSpec & inSpec);
	
	virtual ~CTidyHeapFile_AC();

	// Assignment operator
	
	CTidyHeapFile_AC &operator = (const CTidyHeapFile_AC & inFile);

	// Opening and Closing
	
	enum ePerms
	{
		kRead,
		kWrite,
		kAppend
	};
	
	void Open(	ePerms inPerms = kRead );
	
	void Close();

	bool IsOpen() const { return fRef != 0; }
	
	// Reading and Writing
	
	long Read( void *inBuf, long size);
	
	long Write( const void *inBuf, long size);

	void ReadObject(MTidyHeapPersistantObject_AC &inPObj);
	
	void WriteObject(MTidyHeapPersistantObject_AC &inPObj);

	// Specifying

	void SetSpec( const FSSpec &inSpec );
	
	void SetName( const unsigned char* inPtr );

	void AppendExtension(const char* inPtr);	
	
	void SetVolRefNum( short inShort);
	
	void SetDirID( long inDirID);
	
	const FSSpec &GetSpec() const { return fSpec; }
	
	void SetTypeAndCreator( OSType inType, OSType inCreator );
	
	// Misc Info "getters"
	
	long GetSize();
	
	OSErr GetErr() const { return fErr; }
	
	void SetErr(OSErr err) { fErr = err; }
	
	void ReportErr(const char* inMessage);

protected:

	static void CopySpec( FSSpec *dest, const FSSpec *src ) 
		{ ::BlockMove( src, dest, sizeof(FSSpec)); }
	
protected:
	FSSpec 	fSpec;
	short 	fRef;
	OSErr 	fErr;
	OSType 	fType;
	OSType 	fCreator;
};

#endif
