//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeapSettings_AC__
#define __CTidyHeapSettings_AC__

#ifndef __CTidyHeapPersistantObject_AC__
#include "CTidyHeapPersistantObject_AC.h"
#endif

#ifndef __MTidyHeapAllocator_AC__
#include "MTidyHeapAllocator_AC.h"
#endif

#ifndef __CTidyHeapFlag_AC__
#include "CTidyHeapFlag_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CTidyHeapSettings_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CTidyHeapSettings_AC : public MTidyHeapAllocator_AC, public MTidyHeapPersistantObject_AC
{
public:

	enum { kNameSize = 32 };
	
		
public:

	CTidyHeapSettings_AC();

	CTidyHeapSettings_AC(const CTidyHeapSettings_AC & inSettings);

	virtual ~CTidyHeapSettings_AC();

	CTidyHeapSettings_AC &operator = (const CTidyHeapSettings_AC & inSettings);
	
	virtual void RevertToDefault();
		
	virtual const char* GetSessionName() const;
	
	virtual void SetSessionName(const char* inPtr);
	
	virtual void SetNewFailCount( long inCount );
		// this value is only used for the NewFailerAction	

	virtual long GetNewFailCount() const;
				
	virtual unsigned long GetLastWriteSeconds() const
	{	return fLastWriteSeconds; }
	
	bool CheckSettingsVersion(CTidyHeapFile_AC* inFile);
		// returns true if same version, false if it aint.
		
		// feature prefs
	CTidyHeapFlag_AC IsEnabled;
	CTidyHeapFlag_AC WantsWarnings;
	CTidyHeapFlag_AC WantsUndeletedBlockWarning;
	CTidyHeapFlag_AC DoLogUndeletedBlocks;
	CTidyHeapFlag_AC PointersLivesInAppHeap;
	
	// action prefs
		
		// Global New Actions
	CTidyHeapFlag_AC InstallMakeNewFail;				
	CTidyHeapFlag_AC InstallIncrementalFailer;
	CTidyHeapFlag_AC InstallNewVerifyAction;
	CTidyHeapFlag_AC InstallNewVerifyAllAction;
	CTidyHeapFlag_AC InstallExemptUnknownBlocks;
	CTidyHeapFlag_AC InstallGarbageIn;
	CTidyHeapFlag_AC InstallNewBreakOnUnknownBlock;
	
		 // Global Delete Actions
	CTidyHeapFlag_AC InstallDeleteBreakOnUnknownBlock;
	CTidyHeapFlag_AC InstallDeleteVerifyAction;
	CTidyHeapFlag_AC InstallGarbageOut;
	CTidyHeapFlag_AC InstallDeleteVerifyAllAction;
	CTidyHeapFlag_AC InstallDoubleDeleteWatcher;
	CTidyHeapFlag_AC SettingsModifiedExternally;
	
protected:

	virtual void SetSessionNameToDefault();		
		
private:
	virtual void Read( CTidyHeapFile_AC * inFile);

	virtual void Write( CTidyHeapFile_AC * inFile);

private:	
	char 			fSessionName[kNameSize];	
	long 			fNewFailCount;
	unsigned long	fLastWriteSeconds;
	
};

//----------------------------------------------------------------------------------------
// Inlines
//----------------------------------------------------------------------------------------

inline const char* CTidyHeapSettings_AC::GetSessionName() const 
{ 
	return fSessionName; 
}
inline void CTidyHeapSettings_AC::SetNewFailCount( long inCount ) 
{ 
	fNewFailCount = inCount; 
}
inline long CTidyHeapSettings_AC::GetNewFailCount() const 
{ 
	return fNewFailCount; 
}

#endif
