//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC.cp
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

#if qTidyHeap

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __cstring__
#include <string.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

// TidyHeap

#ifndef __CTidyHeapSettings_AC__
#include "CTidyHeapSettings_AC.h"
#endif
#ifndef __CTidyHeapFile_AC__
#include "CTidyHeapFile_AC.h"
#endif

//========================================================================================
// class CTidyHeapSettings_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC default constructor
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC::CTidyHeapSettings_AC()
{
	RevertToDefault();
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC copy constructor
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC::CTidyHeapSettings_AC(const CTidyHeapSettings_AC &inSettings)
{
	*this = inSettings;
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC destructor
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC::~CTidyHeapSettings_AC()
{
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC assignment operator
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC &CTidyHeapSettings_AC::operator = (const CTidyHeapSettings_AC &inSettings)
{
	IsEnabled = inSettings.IsEnabled;
	WantsWarnings = inSettings.WantsWarnings;
	WantsUndeletedBlockWarning = inSettings.WantsUndeletedBlockWarning;
	DoLogUndeletedBlocks = inSettings.DoLogUndeletedBlocks;
	PointersLivesInAppHeap = inSettings.PointersLivesInAppHeap;

	InstallMakeNewFail = inSettings.InstallMakeNewFail;				
	InstallIncrementalFailer = inSettings.InstallIncrementalFailer;
	InstallNewVerifyAction = inSettings.InstallNewVerifyAction;
	InstallNewVerifyAllAction = inSettings.InstallNewVerifyAllAction;
	InstallExemptUnknownBlocks = inSettings.InstallExemptUnknownBlocks;
	InstallGarbageIn = inSettings.InstallGarbageIn;
	InstallNewBreakOnUnknownBlock = inSettings.InstallNewBreakOnUnknownBlock;
	
		 // Global Delete Actions
	InstallDeleteBreakOnUnknownBlock = inSettings.InstallDeleteBreakOnUnknownBlock;
	InstallDeleteVerifyAction = inSettings.InstallDeleteVerifyAction;
	InstallGarbageOut = inSettings.InstallGarbageOut;
	InstallDeleteVerifyAllAction = inSettings.InstallDeleteVerifyAllAction;
	InstallDoubleDeleteWatcher = inSettings.InstallDoubleDeleteWatcher;
	SettingsModifiedExternally = inSettings.SettingsModifiedExternally;
	
	SetSessionName( inSettings.GetSessionName() );
	
	fNewFailCount = inSettings.fNewFailCount;
	fLastWriteSeconds = inSettings.fLastWriteSeconds;
	
	return *this;
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::RevertToDefault
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::RevertToDefault()
{
	SetSessionNameToDefault();
	
	IsEnabled(true);
	WantsWarnings(true);
	WantsUndeletedBlockWarning(true);
	DoLogUndeletedBlocks(true);
	PointersLivesInAppHeap(true);

	// trues
	InstallGarbageIn(true);
	InstallGarbageOut(true);
	InstallDeleteVerifyAction(true);
	
	// falses
	InstallMakeNewFail(false);				
	InstallIncrementalFailer(false);	
	InstallNewVerifyAction(false);	
	InstallNewVerifyAllAction(false);
	InstallExemptUnknownBlocks(false);
	InstallNewBreakOnUnknownBlock(false);
	InstallDeleteBreakOnUnknownBlock(false);
	InstallDoubleDeleteWatcher(false);
	InstallDeleteVerifyAllAction(false);
	
	SettingsModifiedExternally(false);
		
	fNewFailCount = 0;
	fLastWriteSeconds = 0;
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::Read
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::Read( CTidyHeapFile_AC *inFile )
{
	
	// prefs
	if(!inFile->GetErr()) inFile->Read(&IsEnabled, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&WantsWarnings, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&WantsUndeletedBlockWarning, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&DoLogUndeletedBlocks, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&PointersLivesInAppHeap, sizeof(bool));

	// new actions
	if(!inFile->GetErr()) inFile->Read(&InstallGarbageIn, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&InstallMakeNewFail, sizeof(bool));				
	if(!inFile->GetErr()) inFile->Read(&InstallNewVerifyAction, sizeof(bool));	
	if(!inFile->GetErr()) inFile->Read(&InstallNewVerifyAllAction, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&InstallNewBreakOnUnknownBlock, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&InstallIncrementalFailer, sizeof(bool));	
	if(!inFile->GetErr()) inFile->Read(&InstallExemptUnknownBlocks, sizeof(bool));
	
	// delete actions
	if(!inFile->GetErr()) inFile->Read(&InstallGarbageOut, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&InstallDeleteBreakOnUnknownBlock, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&InstallDoubleDeleteWatcher, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&InstallDeleteVerifyAllAction, sizeof(bool));
	if(!inFile->GetErr()) inFile->Read(&InstallDeleteVerifyAction, sizeof(bool));

	// session name	
	if(!inFile->GetErr()) inFile->Read( fSessionName, kNameSize );

	// new fail count
	if(!inFile->GetErr()) inFile->Read( &fNewFailCount, sizeof(long) );

	if(!inFile->GetErr())
	{
		// We're going to allow this to fail so we remain compatible with
		// old formats.
	
		inFile->Read(&SettingsModifiedExternally, sizeof(bool) );
		
		if(!inFile->GetErr())
			inFile->Read(&fLastWriteSeconds, sizeof(unsigned long) );
		
		if(inFile->GetErr())
			inFile->SetErr(noErr);
	}

	if(inFile->GetErr()) 
	{
		RevertToDefault();
	}
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::Write
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::Write( CTidyHeapFile_AC *inFile )
{
	// prefs
	if(!inFile->GetErr()) inFile->Write(&IsEnabled, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&WantsWarnings, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&WantsUndeletedBlockWarning, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&DoLogUndeletedBlocks, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&PointersLivesInAppHeap, sizeof(bool));

	// new actions
	if(!inFile->GetErr()) inFile->Write(&InstallGarbageIn, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&InstallMakeNewFail, sizeof(bool));				
	if(!inFile->GetErr()) inFile->Write(&InstallNewVerifyAction, sizeof(bool));	
	if(!inFile->GetErr()) inFile->Write(&InstallNewVerifyAllAction, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&InstallNewBreakOnUnknownBlock, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&InstallIncrementalFailer, sizeof(bool));	
	if(!inFile->GetErr()) inFile->Write(&InstallExemptUnknownBlocks, sizeof(bool));
	
	// delete actions
	if(!inFile->GetErr()) inFile->Write(&InstallGarbageOut, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&InstallDeleteBreakOnUnknownBlock, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&InstallDoubleDeleteWatcher, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&InstallDeleteVerifyAllAction, sizeof(bool));
	if(!inFile->GetErr()) inFile->Write(&InstallDeleteVerifyAction, sizeof(bool));	

	if(!inFile->GetErr())
		inFile->Write(fSessionName, kNameSize );

	if(!inFile->GetErr())
		inFile->Write(&fNewFailCount, sizeof(long) );
		
	if(!inFile->GetErr())
	{
		SettingsModifiedExternally(false);
		inFile->Write(&SettingsModifiedExternally, sizeof(bool) );
	}
	
	if(!inFile->GetErr())
	{
		GetDateTime(&fLastWriteSeconds);
		inFile->Write(&fLastWriteSeconds, sizeof(unsigned long) );
	}
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::SetSessionName
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::SetSessionName(const char* inStr)
{
	::strcpy( fSessionName, inStr );
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::SetSessionNameToDefault
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::SetSessionNameToDefault()
{
	Str255 temp;
	pstrcpy(temp, ::LMGetCurApName());
	
	p2cstr(temp);

	SetSessionName((char*) temp);
}

bool CTidyHeapSettings_AC::CheckSettingsVersion(CTidyHeapFile_AC* inFile)
{
	bool out = true;
	long prefsSize = (18 * sizeof(bool)) + (sizeof(long) * 2) + kNameSize;
	
	inFile->Open();
	if(!inFile->GetErr())
	{
		if(inFile->GetSize() != prefsSize)
		{
			out = false;
			DebugStr("\pTidyHeap Error:.prf file incompatible format - resetting;g");
			RevertToDefault();
		}
		inFile->Close();
	}
	return out;
}


#endif