// UDynamicArray.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __CDynamicArray_AC__
#define __CDynamicArray_AC__

// ACS

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __MStreamable_AC__
#include "MStreamable_AC.h"
#endif

// ANSI

#ifndef __STDDEF__
#include <stddef.h>
#endif

//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------

enum {
	kIterateBackward_AC = false,
	kIterateForward_AC = true
};

enum {
	kEmptyIndex_AC = 0								// Value to use when no valid index is
                                                // available. Indexes are always positive
};

typedef long ArrayIndex_AC;						// At least always positive ( in this
                                                // universe ) !!! kEmptyIndex_AC..MaxLong
                                                // would be nice

//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class CArrayIterator_AC;

#if qTheDebugger
class ClassDesc;
#endif

enum
{
	kAllocationIncrement_AC = 6					// Initial Allocation increment. Six
};                                              // elements of slop shouldn't break
                                                // anybody and provides a _nice_ cushion
                                                // from the memory manager.

//----------------------------------------------------------------------------------------
// Constants for CDynamicArray_AC::CompareElements and CSortedxxx::Compare
//----------------------------------------------------------------------------------------
enum {
	kItem1LessThanItem2_AC = -1,
	kItem1EqualItem2_AC = 0,
	kItem1GreaterThanItem2_AC = 1
};


//----------------------------------------------------------------------------------------
// Constants for CSortedList_AC::Search.
//----------------------------------------------------------------------------------------
enum {
	kItemGreaterThanCriteria_AC = -1,
	kItemEqualCriteria_AC = 0,
	kItemLessThanCriteria_AC = 1
};

typedef short CompareResult_AC;
	// Negative, zero, and positive results are all meaningful even though we have the
    // nice constants above.


//----------------------------------------------------------------------------------------
// procedure typedefs for parameters
//----------------------------------------------------------------------------------------
typedef CompareResult_AC (*CompareIndexType_AC)(ArrayIndex_AC anItem, void* yourDataPtr);

typedef CompareResult_AC (* CompareElementsType_AC)(void* element1,
											void* Element2,
											void* yourDataPtr);

class CStream_AC;
//----------------------------------------------------------------------------------------
// CDynamicArray_AC
//----------------------------------------------------------------------------------------
class EXPORT_AC CDynamicArray_AC : public MStreamable_AC
{
friend CArrayIterator_AC;
#if qTheDebugger
friend ClassDesc;
#endif

public:
	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------

	CDynamicArray_AC(ArrayIndex_AC initialSize = 0,
									  short elementSize = sizeof(void*));
		// Initialize a new array with initialSize elements

	CDynamicArray_AC(const CDynamicArray_AC& inDynamicArray);
		// Copy Constructor

	virtual ~CDynamicArray_AC();
		// Destructor
		
	virtual void* Clone() const;
	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFrom(CStream_AC* aStream);
	
	virtual void WriteTo(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// Array manipulation primitives
	//------------------------------------------------------------------------------------

	void* operator[](ArrayIndex_AC index) const;
		// Returns a pointer to the index-th element in the array.
		// the index is "zero" based.
		
	void* ComputeAddress(ArrayIndex_AC index) const;
		// Returns a pointer to of the index-th element in the array.
		// the index is "one" based.

	ArrayIndex_AC GetSize() const { return fSize; }
		// Returns the ACTUAL number of elements in the array.

	ArrayIndex_AC GetElementSize() const { return fElementSize; }
		// Returns the size of an element.

	void SetArraySize(ArrayIndex_AC theSize, bool canShrink = true);
		// Sets the array allocation to handle up to theSize elements
		// canShrink controls whether the fData can be shrunk (if possible) when sizing.

	void InsertElementsBefore(ArrayIndex_AC index,
										 const void* ElementPtr,
										 ArrayIndex_AC count);
		// Insert Elements before the indicated Element. The index of the new element will
        // be index. If index = 1 this inserts at the start of the array. If index = fSize
        // + 1 this inserts at the end of the array. Signals Failure if unable to change
        // the size of the array. !!! NOTE MULTIPLE ( >1 ) element moves for non-power of
        // 2 element sizes are NOT yet supported!

	void ReplaceElementsAt(ArrayIndex_AC index,
									  const void* ElementPtr,
									  ArrayIndex_AC count);
		// Replaces the Elements at the indicated index. !!! NOTE MULTIPLE ( >1 ) element
        // moves for non-power of 2 element sizes are NOT yet supported!

	void DeleteElementsAt(ArrayIndex_AC index,
										 ArrayIndex_AC count);
		// Deletes the Element at the indicated index. Compresses the array !!! NOTE
        // MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT yet
        // supported!

	void GetElementsAt(ArrayIndex_AC index,
									  void* ElementPtr,
									  ArrayIndex_AC count) const;
		// copies count elements to the location specified by ptr. !!! NOTE MULTIPLE ( >1
        // ) element moves for non-power of 2 element sizes are NOT yet supported!

	void DeleteAll();
		// Delete every element from the list Leave fSize at 0.
		
	//------------------------------------------------------------------------------------
	// Misc. functions
	//------------------------------------------------------------------------------------

	inline bool IsEmpty() const { return fSize == kEmptyIndex_AC; }
		// Test if this array is empty or not.

	void Merge(const CDynamicArray_AC* aDynamicArray);
		// merges aDynamicArray with itself. leaves aDynamicArray unchanged. !!! NOTE
        // MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT yet
        // supported!

	inline ArrayIndex_AC GetAllocationIncrement() const { return fAllocationIncrement; }
	inline void SetAllocationIncrement(ArrayIndex_AC inNewIncrement)  { fAllocationIncrement = inNewIncrement; }

	//------------------------------------------------------------------------------------
	// Public Sorting access
	//------------------------------------------------------------------------------------
	virtual CompareResult_AC CompareElements(const void* Element1, const void* Element2) const;
		// Returns the sort order between elements of the array. Default considers
		// all elements equivalent.

	virtual void Sort();
		// Sorts the list usingthe Compare and SortBy methods.

	virtual void SortElementsBy(CompareElementsType_AC CompareItems, void* yourDataPtr);
		// Sorts the list using the supplied CompareItems function.

	//------------------------------------------------------------------------------------
	// Sorting
	//------------------------------------------------------------------------------------
protected:
	void* GetSwapBuffer();
		// Returns a buffer to use in element exchanges

	void ReleaseSwapBuffer(void* swapBuffer);
		// Releases a buffer returned by GetSwapBuffer

	void SwapElements(void* element1, void* element2, void* swapBuffer);
		// Swaps two elements using the supplied swapbuffer

	void QuickSort(ArrayIndex_AC beginIndex,
								  ArrayIndex_AC endIndex,
								  CompareElementsType_AC CompareItems,
								  void* yourDataPtr,
								  void* swapBuffer);
		// Uses QuickSort Algorithm, adapted from:
		//	Cormen, Leiserson, and Rivest, _Introductions_To_Algorithms_, pp 153-167.

	ArrayIndex_AC QSPartition(ArrayIndex_AC beginIndex,
										ArrayIndex_AC endIndex,
										CompareElementsType_AC CompareItems,
										void* yourDataPtr,
										void* swapBuffer);
		// Called by RandomPartition. Partitions the array A[beginIndex..endIndex] in place

	ArrayIndex_AC QSRandomPartition(ArrayIndex_AC beginIndex,
										ArrayIndex_AC endIndex,
										CompareElementsType_AC CompareItems,
										void* yourDataPtr,
										void* swapBuffer);
		// Called by QuickSort. Partitions the array A[beginIndex..endIndex] in place

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	Ptr fArraySpace;							// space for actually storing the elements
												// of this CDynamicArray_AC.
		
	CArrayIterator_AC* fIteratorPtr;			// pointer to the Iterators in use

	ArrayIndex_AC fSize;						// number of elements ACTUALLY in the
                                                // array, from 0 to the limit of memory

	ArrayIndex_AC fAllocationIncrement;			// Number of elements by which to increase
                                                // or decrease the allocated size of the
                                                // array when it needs to grow or shrink.
                                                // Thus reducing memory manager
                                                // aggravation.

	ArrayIndex_AC fAllocatedSize;					// Number of elements for which storage is
                                                // ALLOCATED

	unsigned short fElementSize;				// Size in bytes of an element. MUST be a
                                                // power of 2 ie. 1, 2, 4, 8, 16, etc.

	unsigned short fElementSizeShift;			// the power of 2 for the element size.
                                                // Will be used to avoid DIV and MUL
};

//----------------------------------------------------------------------------------------
// CSortedDynamicArray_AC: A CDynamicArray_AC with ordering.
//----------------------------------------------------------------------------------------
class EXPORT_AC CSortedDynamicArray_AC : public CDynamicArray_AC
{
public:

	inline CSortedDynamicArray_AC(ArrayIndex_AC initialSize = 0, short elementSize = sizeof(void*)) : CDynamicArray_AC(initialSize, elementSize) { }
		// Initialize a new array with initialSize elements

	virtual ~CSortedDynamicArray_AC();
		// Destructor
		
	virtual void* Clone() const;

	virtual void InsertElementInOrder(const void* ElementPtr);
		// Insert an element in its correct position according to the ordering defined by
        // the Compare method

	virtual bool DoSearchElement(CompareIndexType_AC TestItem,
										   void* yourDataPtr,
										   ArrayIndex_AC& index) const;
		// !!! Adapted from CSortedList_AC. Performs a binary search for an element for which
        // TestItem returns true. "index" returns the position of a found item, or where
        // it should be inserted if not found
};

#endif
