//  UListIterator.h
//  Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __CListIterator_AC__
#define __CListIterator_AC__

// ACS

#ifndef __CIterator_AC__
#include "CIterator_AC.h"
#endif

#ifndef __CDynamicArray_AC__
#include "CDynamicArray_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Forward class declarations.
//----------------------------------------------------------------------------------------
class CSortedList_AC;
class CSortedList_AC;
class CSortedHandleList_AC;
class CSortedLongintList_AC;

//----------------------------------------------------------------------------------------
// CArrayIterator_AC
//----------------------------------------------------------------------------------------
class EXPORT_AC CArrayIterator_AC : public CIterator_AC
{
public:
	CArrayIterator_AC(const CDynamicArray_AC* itsDynamicArray,
				   ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
				   bool itsForward = kIterateForward_AC);

	CArrayIterator_AC(const CDynamicArray_AC* itsDynamicArray, bool itsForward = kIterateForward_AC);
	// Iterate over the entire available array. kMaxLong is ok here it will get bounded in IArrayIterator

	~CArrayIterator_AC();
	// Remove me from the linked list of iterators
	
	//
	// Required functions 
	//
	ArrayIndex_AC First();
	inline ArrayIndex_AC Current() const { return fCurrentIndex; }
	ArrayIndex_AC Next();

	//
	// Convenience functions for C++meisters
	//

	inline ArrayIndex_AC operator++() { return Next(); }
	inline ArrayIndex_AC operator++(int) { ArrayIndex_AC returnVal = Current(); Next(); return returnVal; }

	//
	// Compatibility functions
	//
	inline ArrayIndex_AC FirstIndex() { return First(); }
	inline ArrayIndex_AC CurrentIndex() const { return Current(); }
	inline ArrayIndex_AC NextIndex() { return Next(); }
	
	//
	// Helper functions
	//
	void DeleteElementAt(ArrayIndex_AC theIndex, ArrayIndex_AC theCount);
		// Bends the index in the Iteration node to account for the deleted elements

	void InsertElementBefore(ArrayIndex_AC theIndex, ArrayIndex_AC theCount);
		// Bends the index to account for the inserted elements 

	void AbortIteration();
		// Called when iterations are in progress and the CDynamicArray_AC is deleted

protected:
	void IArrayIterator(const CDynamicArray_AC* itsDynamicArray ,
						ArrayIndex_AC itsLowBound,
						ArrayIndex_AC itsHighBound,
						bool itsForward);
		// Called by the constructors to do common initialization and reset the iteration

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------

public:
	// These items are currently public for searching
	ArrayIndex_AC fCurrentIndex;					// current index of this iteration
	ArrayIndex_AC fLowBound;						// lower bound of iteration in progress
	ArrayIndex_AC fHighBound;						// upper bound of iteration in progress
													// through the list

private:
	CArrayIterator_AC* fNextLink;					// link to next iterator
	
protected:
	const CDynamicArray_AC* fDynamicArray;			// the associated dynamic array
	bool fIterateForward;							// if iteration is forward or backward
};

//----------------------------------------------------------------------------------------
// CommonArrayIterator_AC
// Returns pointers TO array elements (not pointers stored IN array elements)
// Returns NULL pointers for no element
//----------------------------------------------------------------------------------------
class EXPORT_AC CCommonArrayIterator_AC : public CArrayIterator_AC
{
public:
	CCommonArrayIterator_AC(const CDynamicArray_AC* inList,
					ArrayIndex_AC inLowBound, ArrayIndex_AC inHighBound,
					bool inForward = kIterateForward_AC);

	CCommonArrayIterator_AC(const CDynamicArray_AC* inList, bool inForward = kIterateForward_AC);

	//
	// Required functions 
	//
	void* First();
	inline void* Current() const { return fCurrentElement; }
	void* Next();

protected:
	void* fCurrentElement;									// current object address of this iteration
};

//----------------------------------------------------------------------------------------
// CCommonVoidIterator_AC
// Returns pointers stored IN elements (not pointers TO elements)
// Returns NULL pointers for no element
//----------------------------------------------------------------------------------------
class EXPORT_AC CCommonVoidIterator_AC : public CCommonArrayIterator_AC
{
public:
	CCommonVoidIterator_AC(const CDynamicArray_AC* itsList,
					ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
					bool itsForward = kIterateForward_AC);

	CCommonVoidIterator_AC(const CDynamicArray_AC* itsList, bool itsForward = kIterateForward_AC);

	//
	// Required functions 
	//
	void* First();
	inline void* Current() const { return fCurrent; }
	void* Next();

protected:
	void* fCurrent;									// current object of this iteration
};

//----------------------------------------------------------------------------------------
// TemplateArrayIterator_AC
// Returns pointers TO items stored in elements
// Returns NULL pointers for no element
//----------------------------------------------------------------------------------------
template <class T>
class TemplateArrayIterator_AC : public CCommonArrayIterator_AC
{
public:
	TemplateArrayIterator_AC(const CDynamicArray_AC* inList,
					ArrayIndex_AC inLowBound, ArrayIndex_AC inHighBound,
					bool inForward = kIterateForward_AC) :
	CCommonArrayIterator_AC(inList, inLowBound, inHighBound, inForward)
	{ }

	TemplateArrayIterator_AC(const CDynamicArray_AC* inList, bool inForward = kIterateForward_AC) :
	CCommonArrayIterator_AC(inList, inForward)
	{ }

	//
	// Required functions 
	//
	inline T* First() { return reinterpret_cast<T*>(CCommonArrayIterator_AC::First()); }
	inline T* Current() const { return reinterpret_cast<T*>(CCommonArrayIterator_AC::Current()); }
	inline T* Next() { return reinterpret_cast<T*>(CCommonArrayIterator_AC::Next()); }

	//
	// Convenience functions for C++meisters
	//
	inline T* operator++() { return Next(); }
	inline T* operator++(int) { T* returnVal = Current(); Next(); return returnVal; }
	inline operator T*() const { return Current(); }

	inline T* operator->() const { return Current(); } // this is only valid for non NULL
	inline T& operator*() const { return *Current(); } // this is only valid for non NULL
};

//----------------------------------------------------------------------------------------
// TemplateScalarIterator_AC
// Returns items stored IN elements that are the sizeof(void*) but the items are
// scalars (they can't be dereferenced)
//----------------------------------------------------------------------------------------
template <class T>
class TemplateScalarIterator_AC : public CCommonVoidIterator_AC
{
public:
	TemplateScalarIterator_AC(const CDynamicArray_AC* inList,
					ArrayIndex_AC inLowBound, ArrayIndex_AC inHighBound,
					bool inForward = kIterateForward_AC) :
	CCommonVoidIterator_AC(inList, inLowBound, inHighBound, inForward)
	{ }

	TemplateScalarIterator_AC(const CDynamicArray_AC* inList, bool inForward = kIterateForward_AC) :
	CCommonVoidIterator_AC(inList, inForward)
	{ }

	//
	// Required functions 
	//
	inline T First() { return reinterpret_cast<T>(CCommonVoidIterator_AC::First()); }
	inline T Current() const { return reinterpret_cast<T>(CCommonVoidIterator_AC::Current()); }
	inline T Next() { return reinterpret_cast<T>(CCommonVoidIterator_AC::Next()); }

	//
	// Convenience functions for C++meisters
	//
	inline T operator++() { return Next(); }
	inline T operator++(int) { T returnVal = Current(); Next(); return returnVal; }
	inline operator T() const { return Current(); }
};

//----------------------------------------------------------------------------------------
// TemplateIterator_AC
// Returns items stored IN elements that are the sizeof(void*) and are pointers
// of some kind
//----------------------------------------------------------------------------------------
template <class T>
class TemplateIterator_AC : public TemplateScalarIterator_AC<T>
{
public:
	TemplateIterator_AC(const CDynamicArray_AC* inList,
					ArrayIndex_AC inLowBound, ArrayIndex_AC inHighBound,
					bool inForward = kIterateForward_AC) :
	TemplateScalarIterator_AC<T>(inList, inLowBound, inHighBound, inForward)
	{ }

	TemplateIterator_AC(const CDynamicArray_AC* inList, bool inForward = kIterateForward_AC) :
	TemplateScalarIterator_AC<T>(inList, inForward)
	{ }

	inline T operator->() const { return Current(); }
};

typedef TemplateScalarIterator_AC<Handle> CHandleIterator_AC;
typedef TemplateScalarIterator_AC<long> CLongintIterator_AC;

#endif
