//----------------------------------------------------------------------------------------
// CMapIterator_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CMapIterator_AC__
#define __CMapIterator_AC__

#ifndef __CMap_AC__
#include "CMap_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CMapIterator_AC
//----------------------------------------------------------------------------------------

template <class MAP>
class EXPORT_AC CMapIterator_AC 
{
public:
	CMapIterator_AC( MAP* inMap );

	~CMapIterator_AC();
	
	void Begin();
	
	void More();
	
	bool HasMore();
	
	long GetNumDependencies() const;

	MAP::Key GetKey() { return (*fWalker).first; }
	
	MAP::Type GetType() { return (*fWalker).second; }

	operator MAP::iterator() { return fWalker; }

private:	
	bool fIterating;

	MAP* fMap;
	MAP::iterator fWalker;
	MAP::iterator fEnd;
};

template <class MAP>
inline bool CMapIterator_AC<MAP>::HasMore()
{
	return fIterating && (fWalker != fEnd);
}

template <class MAP>
inline long CMapIterator_AC<MAP>::GetNumDependencies() const
{
	::AssertNotNULL_AC(fMap);
	
	return fMap->size();
}

template <class MAP>
inline CMapIterator_AC<MAP>::CMapIterator_AC(MAP* inMap)
	: fMap(inMap),
	fIterating(false)
{
	Begin();
}

template <class MAP>
inline CMapIterator_AC<MAP>::~CMapIterator_AC()
{
	if(fIterating)
	{
		::AssertNotNULL_AC(fMap);

		fIterating = false;
		fMap->stop_iterating();
	}
}

template <class MAP>
void CMapIterator_AC<MAP>::Begin()
{
	::AssertNotNULL_AC(fMap);
	
	if(!fIterating)
	{
		fIterating = true;
		fWalker  = 	fMap->begin_iterating();
	}
	else
	{
		fWalker = fMap->begin(); 
			// if already iterating don't increment maps counter
	}

	fEnd = fMap->end();

	if (fMap->is_marked((*fWalker).second))
	{
		More();
	}
}

template <class MAP>
void CMapIterator_AC<MAP>::More()
{
	::Assertion_AC(fIterating == true, NULL);
	::AssertNotNULL_AC(fMap);

	++fWalker;
	
	if(!HasMore())
	{		
		fMap->stop_iterating();
		fIterating = false;
	}
	else 
	if (fMap->is_marked((*fWalker).second))
	{
		More();
	}	
}

#endif
