//----------------------------------------------------------------------------------------
// MLinkable_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __T__
#define __T__

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

template <class T>
class EXPORT_AC MLinkable_AC
{
public:
	MLinkable_AC() 
		: fNext(NULL) 
	{
	}
	virtual ~MLinkable_AC() 
	{
		Assertion_AC(fNext == NULL, "Deleting element still in list");
	}

	void Push(T* pushInFrontOf);
		// Push this MLinkable on the head of the list. This MLinkable becomes
		// the head of the list.

	T* Append(T* first);
		// Append this MLinkable to list. Send in head of list, which can
		// be NULL. Returns new head of list.

	T* Remove(T* first);
		// Remove this MLinkable from list. Send in head of list, may return
		// new head of list.

	bool InList(T* first) const;
		// Returns whether this MLinkable_AC is in list.

	static void DeleteList(	T*& first );
		// Delete each MLinkable in list. 
		// Send in first element in list, which will be set to NULL.

	T* GetNextLink() const
	{	return fNext; }
		
protected:
	T* fNext;
};

template <class T>
inline void MLinkable_AC<T>::Push(T* pushInFrontOf)
{	
	Assertion_AC(fNext == NULL, "Inserting element already in list");

	fNext = pushInFrontOf;
}

template <class T>
T* MLinkable_AC<T>::Append(T* first)
{
	Assertion_AC(fNext == NULL, "Inserting element already in list");
	
	T* walker = first;
	T* prev = NULL;
	while(walker)
	{
		prev = walker;
		walker = walker->fNext;
	}
	
	if(prev)
	{
	 	prev->fNext = dynamic_cast_AC(T*,this);
	}
	else
	{
		first = dynamic_cast_AC(T*,this);
	}
	
	return first;
}

template <class T>
T* MLinkable_AC<T>::Remove(T* first)
{	 
	AssertNotNULL_AC(first);
	
	T* walker = first;
	T* prev = NULL;
	
	while(walker)
	{
		if(walker == this)
		{
			if(walker == first)
			{
				first = walker->fNext;
			}
			else
			{
				AssertNotNULL_AC(prev); // don't think this can happen
				prev->fNext = walker->fNext;
			}
			
			fNext = NULL;
			break;	
		}
		prev = walker;
		walker = walker->fNext;
	}	
	
	return first;
}

template <class T>
inline void MLinkable_AC<T>::DeleteList(T*& first )
{
	T* walker = first;
	T* deleter = NULL;
	
	while(walker)
	{
		deleter = walker;
		walker = walker->fNext;
		
		delete deleter;
	}

	first = NULL;
}

template <class T>
bool MLinkable_AC<T>::InList(T* first) const
{
	bool inList = false;
	T* walker = first;
	while(walker)
	{
		if(walker == this)
		{
			inList = true;
			break;
		}
		
		walker = walker->fNext;
	}
}
		



#endif