//----------------------------------------------------------------------------------------
// UDynamicArray.cp 
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CDynamicArray_AC__
#include "CDynamicArray_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CStream_AC__
#include "CStream_AC.h"
#endif

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

#if 0 //SRF
#ifndef __USTREAM__
#include "UStream.h"
#endif
#endif

// Toolbox

#ifndef __PACKAGES__
#include <Packages.h>
#endif

// ANSI

#ifndef __STDLIB__
#include <stdlib.h>
#endif

#ifndef __STRIND__
#include <string.h>
#endif


//========================================================================================
// GLOBAL Procedures
//========================================================================================
static CompareResult_AC TestItemForInsert_AC(ArrayIndex_AC anItem,
									   void* yourDataPtr);

static CompareResult_AC CompareElements_AC(void* element1,
									void* element2,
									void* yourDataPtr);

static ArrayIndex_AC RandomArrayIndex_AC(ArrayIndex_AC beginIndex,
								   ArrayIndex_AC endIndex);

//========================================================================================
// CLASS CDynamicArray_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CDynamicArray_AC::CDynamicArray_AC: 
//----------------------------------------------------------------------------------------
CDynamicArray_AC::CDynamicArray_AC(ArrayIndex_AC initialSize,
								  short elementSize) :
	fIteratorPtr(NULL),

	fAllocatedSize(kEmptyIndex_AC),
	fAllocationIncrement(kAllocationIncrement_AC),
	fElementSize(elementSize),
	fElementSizeShift(0),
	fSize(kEmptyIndex_AC),
	fArraySpace(NULL)
{
#if qDebug
	if (initialSize < kEmptyIndex_AC)
	{
		::ProgramBreak_AC("initialSize must be non-negative!");
		initialSize = kEmptyIndex_AC;
	}

	if (elementSize <= 0)
	{
		::ProgramBreak_AC("In CDynamicArray_AC.CDynamicArray_AC: preposterous element size. (zero or negative)");
		::ThrowException_AC(minErr, 0);
	}
#endif

	// calculate the elementsizeshift that represents the nearest power of 2 
	while (((elementSize - 1) >> fElementSizeShift) > 0)
		++fElementSizeShift;

	SetArraySize(initialSize);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC destructor
//----------------------------------------------------------------------------------------
CDynamicArray_AC::~CDynamicArray_AC()
{
	DeleteAll();

	if (fIteratorPtr)
		fIteratorPtr->AbortIteration();

	delete[] fArraySpace;
	fArraySpace = NULL;
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC copy constructor
//----------------------------------------------------------------------------------------
CDynamicArray_AC::CDynamicArray_AC(const CDynamicArray_AC& inDynamicArray) :
	MStreamable_AC(inDynamicArray),

	fArraySpace(NULL),
	fIteratorPtr(NULL),
	fSize(inDynamicArray.fSize),
	fAllocationIncrement(inDynamicArray.fAllocationIncrement),
	fAllocatedSize(inDynamicArray.fAllocatedSize),
	fElementSize(inDynamicArray.fElementSize),
	fElementSizeShift(inDynamicArray.fElementSizeShift)
{
	if (inDynamicArray.fArraySpace)
	{
		long itsSize = inDynamicArray.fAllocatedSize * inDynamicArray.fElementSize;

		fArraySpace = TH_new char[itsSize];
			
		BlockMove_AC(inDynamicArray.fArraySpace, fArraySpace, itsSize);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC Clone
//----------------------------------------------------------------------------------------
void* CDynamicArray_AC::Clone() const
{
	return TH_new CDynamicArray_AC(*this);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::ReadFrom: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::ReadFrom(CStream_AC* aStream)
{
	MStreamable_AC::ReadFrom(aStream);

	fElementSize = aStream->ReadInteger();
	fElementSizeShift = aStream->ReadInteger();
	fAllocationIncrement = aStream->ReadLong();

	SetArraySize(0);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::WriteTo: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::WriteTo(CStream_AC* aStream) const
{
	MStreamable_AC::WriteTo(aStream);

	aStream->WriteInteger(fElementSize);
	aStream->WriteInteger(fElementSizeShift);
	aStream->WriteLong(fAllocationIncrement);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::operator[]: 
//----------------------------------------------------------------------------------------
void* CDynamicArray_AC::operator[](ArrayIndex_AC index) const
{
#if qRangeCheck
	::Assertion_AC(fArraySpace != NULL, "Whoops! you called CDynamicArray_AC::operator[] for an empty array");
#endif
	return fArraySpace + (index << fElementSizeShift);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::ComputeAddress: 
//----------------------------------------------------------------------------------------
void* CDynamicArray_AC::ComputeAddress(ArrayIndex_AC index) const
{
#if qRangeCheck
	::Assertion_AC(fArraySpace != NULL, "Whoops! you called CDynamicArray_AC::ComputeAddress for an empty array");
#endif
	return fArraySpace + ((index - 1) << fElementSizeShift);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::DeleteElementsAt: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::DeleteElementsAt(ArrayIndex_AC index,
										ArrayIndex_AC count)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CDynamicArray_AC.DeleteElementsAt");
	}
#endif

	if (count > kEmptyIndex_AC)
	{
		ArrayIndex_AC countBytes = count << fElementSizeShift;
	
		void* indexPtr = ComputeAddress(index);
		void* nextElementPtr = ComputeAddress(index + count);
		void* lastElementPtr = ComputeAddress(fSize + 1);
	
		if (nextElementPtr < lastElementPtr)		// deleted from middle? Compress the array 
			BlockMoveOverlap_AC(nextElementPtr, indexPtr, reinterpret_cast<ptrdiff_t>(lastElementPtr) - reinterpret_cast<ptrdiff_t>(nextElementPtr));
	
#if qDebug
		BlockSet_AC(static_cast<Ptr>( lastElementPtr) - countBytes, countBytes, kResizedInitVal_AC);
#endif
	
		SetArraySize(fSize - count, true);				// take up slack if necessary. Should never
		// Fail when shrinking array.
		fSize -= count;
	
		if (fIteratorPtr)							// Keep all iterations apprised
			fIteratorPtr->DeleteElementAt(index, count);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::DeleteAll: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::DeleteAll()
{
	if (fSize)
		DeleteElementsAt(1, fSize);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::GetElementsAt: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::GetElementsAt(ArrayIndex_AC index,
									 void* ElementPtr,
									 ArrayIndex_AC count) const
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CDynamicArray_AC::GetElementsAt");
	}
#endif

	// The count computation for this move makes sure that if the element size is not
	// a power of 2 that we don't copy more bytes back to the caller than required for a
	// given number of elements sending the caller into orbit and heading straight for the
	// Excedrin.

	// !!! NOTE MULTIPLE (> 1) element moves for non-power of 2 element sizes are not yet
	// supported!

	if (count > kEmptyIndex_AC)
		BlockMove_AC(ComputeAddress(index), (Ptr)ElementPtr, ((count - 1) << fElementSizeShift) + fElementSize);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::InsertElementsBefore: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::InsertElementsBefore(ArrayIndex_AC index,
											const void* ElementPtr,
											ArrayIndex_AC count)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize + 1))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CDynamicArray_AC.InsertElementsBefore");
	}
#endif

	if (count > kEmptyIndex_AC)
	{
		if (fSize + count > fAllocatedSize)
			SetArraySize(fSize + count, false);			// make sure there's room if needed 
	
		ArrayIndex_AC countBytes = count << fElementSizeShift;
	
		void* indexPtr = ComputeAddress(index);
		void* nextIndexPtr = ComputeAddress(index + count);
		void* lastElementPtr = ComputeAddress(fSize + 1);
	
		if (index <= fSize)							// clear out a hole? 
			BlockMoveOverlap_AC(indexPtr, nextIndexPtr, reinterpret_cast<ptrdiff_t>(lastElementPtr) - reinterpret_cast<ptrdiff_t>(indexPtr));
	
		// !!! we still don't account for multiple element moves with non power of 2 element sizes.
		// Would it be best to create a MoveElements method and put the smarts in it?
	
		if ((countBytes == sizeof(long))
#if !(qPowerPC || qMC68020)
		&& !Odd_AC((Ptr)ElementPtr) && !Odd_AC(indexPtr) /* test for processor that can't stand odd pointers */
#endif
		)
			*static_cast<long*>(indexPtr) = *const_cast<const long*>(ElementPtr);// shortcut for longs 
		else
			BlockMove_AC((Ptr)ElementPtr, indexPtr, countBytes);// longcut for shorts (and other sizes) 
	
		fSize += count;
	
		if (fIteratorPtr)							// Keep all iterations apprised
			fIteratorPtr->InsertElementBefore(index, count);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::Merge: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::Merge(const CDynamicArray_AC* aDynamicArray)
{
#if qDebug
	if ((fElementSize != aDynamicArray->fElementSize) || (fElementSizeShift != aDynamicArray->fElementSizeShift))
		::ProgramBreak_AC("In CDynamicArray_AC.Merge: fElementSize or fElementSizeShift don't match with the CDynamicArray_AC to merge!");
#endif

	ArrayIndex_AC arraySize = aDynamicArray->GetSize();
	if (arraySize != kEmptyIndex_AC)
	{
		// Force the memory move before the call to ComputeAddress.
		SetArraySize(GetSize() + arraySize, true);
		InsertElementsBefore(GetSize() + 1, aDynamicArray->ComputeAddress(1), arraySize);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::ReplaceElementsAt: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::ReplaceElementsAt(ArrayIndex_AC index,
										 const void* ElementPtr,
										 ArrayIndex_AC count)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CDynamicArray_AC.ReplaceElementsAt");
	}
#endif

	if (count > kEmptyIndex_AC)
		BlockMove_AC((Ptr)ElementPtr, ComputeAddress(index), count << fElementSizeShift);
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::SetArraySize: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::SetArraySize(ArrayIndex_AC theSize, bool canShrink)
{
	if ((theSize > fAllocatedSize) || (canShrink && (fAllocatedSize - theSize >= fAllocationIncrement)))
	{
#if qDebug
		if (fAllocationIncrement < 0)
			::ProgramBreak_AC("fAllocationIncrement < 0 !  You have serious problems.");
#endif
		// Set the # of allocated elements to the nearest multiple of
		// fAllocationIncrement.
		ArrayIndex_AC newAllocatedSize = theSize ? (fAllocationIncrement ? (theSize + fAllocationIncrement) - (theSize + fAllocationIncrement) % fAllocationIncrement : theSize) : theSize;

		if (newAllocatedSize != fAllocatedSize)
		{
			if (newAllocatedSize)
			{
				try
				{
					void* newPtr = TH_new char[ newAllocatedSize << fElementSizeShift ];
				
					if (fArraySpace)
					{
						Size		moveSize = Min_AC(fSize, theSize);
						moveSize = moveSize << fElementSizeShift;
						BlockMove_AC(fArraySpace, newPtr, moveSize);

						delete[] fArraySpace;
					}
					fArraySpace = (Ptr)newPtr;
				}
				catch(CException_AC& theException)
				{
					DoCatchMessage_AC(theException);
					if (theSize <= fSize) // shrinking the space, just leave the old pointer, don't fail
						;
					else
						throw; // Growing the space, just leave the old pointer, but fail
				}
				

			}
			else
			{
				delete[] fArraySpace;
				fArraySpace = NULL;
			}
		}
		fAllocatedSize = newAllocatedSize;
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::CompareElements_AC: 
//----------------------------------------------------------------------------------------
CompareResult_AC CDynamicArray_AC::CompareElements(const void* /* Element1 */, const void* /* Element2 */) const
{
	// By Default consider all items equal 
	return kItem1EqualItem2_AC;
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::Sort: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::Sort()
{
	if (GetSize() > 0)
	{
		void* swapBuffer = GetSwapBuffer();
		QuickSort(1, fSize, &::CompareElements_AC, this, swapBuffer);
		ReleaseSwapBuffer(swapBuffer);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::SortBy: NOTE: This doesn't work with a CompareItems Function that inserts
// or deletes elements.
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::SortElementsBy(CompareElementsType_AC CompareItems,
						 void* yourDataPtr)
{
	if (GetSize() > 0)
	{
		void* swapBuffer = GetSwapBuffer();
		QuickSort(1, fSize, CompareItems, yourDataPtr, swapBuffer);
		ReleaseSwapBuffer(swapBuffer);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::QuickSort: 
//----------------------------------------------------------------------------------------
void* CDynamicArray_AC::GetSwapBuffer()
{
	return (fElementSize == sizeof(long)) ? NULL : TH_new char[fElementSize];
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::QuickSort: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::ReleaseSwapBuffer(void* swapBuffer)
{
	if (fElementSize != sizeof(long))
		delete[] swapBuffer;
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::QuickSort: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::SwapElements(void* element1, void* element2, void* swapBuffer)
{
	if (fElementSize == sizeof(long)) // shortcut for longs
	{
		register long swapLong = *static_cast<long*>(element1);
		*static_cast<long*>(element1) = *static_cast<long*>(element2);
		*static_cast<long*>(element2) = swapLong;
	}
	else
	{
		BlockMove_AC(element1, swapBuffer, fElementSize);
		BlockMove_AC(element2, element1, fElementSize);
		BlockMove_AC(swapBuffer, element2, fElementSize);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::QuickSort: 
//----------------------------------------------------------------------------------------
void CDynamicArray_AC::QuickSort(ArrayIndex_AC beginIndex,
							// p
							ArrayIndex_AC endIndex,
							// r
							CompareElementsType_AC CompareItems,
							void* yourDataPtr,
							void* swapBuffer)
{
	if (beginIndex < endIndex)
	{
		ArrayIndex_AC left = QSRandomPartition(beginIndex, endIndex, CompareItems, yourDataPtr, swapBuffer);
		QuickSort(beginIndex, left, CompareItems, yourDataPtr, swapBuffer);
		QuickSort(left + 1, endIndex, CompareItems, yourDataPtr, swapBuffer);
	}
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::QSPartition: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CDynamicArray_AC::QSPartition(ArrayIndex_AC beginIndex, // p
									ArrayIndex_AC endIndex, // r
									CompareElementsType_AC CompareItems,
									void* yourDataPtr,
									void* swapBuffer)
{
	if (beginIndex >= endIndex)
		return endIndex;
	{
		void* pivot = ComputeAddress(beginIndex);		// x
		ArrayIndex_AC i = beginIndex - 1;			// p - 1
		ArrayIndex_AC j = endIndex + 1;			// r + 1
		for(;;)
		{
			void* secondKey;
			do
			{
				--j;
				secondKey = ComputeAddress(j);				// A[j]
			} while (CompareItems(pivot, secondKey, yourDataPtr) <= kItem1LessThanItem2_AC);

			do
			{
				++i;
				secondKey = ComputeAddress(i);				// A[i]
			} while (CompareItems(pivot, secondKey, yourDataPtr) >= kItem1GreaterThanItem2_AC);

			if (i < j)
			{
				// Swap the elements
				SwapElements(ComputeAddress(i), ComputeAddress(j), swapBuffer);
			}
			else
				return j;
		}
	}

	return 0;									// this return statement is never executed, but exists to defeat an xlC warning
}

//----------------------------------------------------------------------------------------
// CDynamicArray_AC::QSRandomPartition: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CDynamicArray_AC::QSRandomPartition(ArrayIndex_AC beginIndex,
										  // p
										  ArrayIndex_AC endIndex,
										  // r
										  CompareElementsType_AC CompareItems,
										  void* yourDataPtr,
											void* swapBuffer)
{
	ArrayIndex_AC i = RandomArrayIndex_AC(beginIndex, endIndex);

	// exchange
	SwapElements(ComputeAddress(beginIndex), ComputeAddress(i), swapBuffer);

	return QSPartition(beginIndex, endIndex, CompareItems, yourDataPtr, swapBuffer);
}

//----------------------------------------------------------------------------------------
// RandomArrayIndex_AC: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC RandomArrayIndex_AC(ArrayIndex_AC beginIndex,
							ArrayIndex_AC endIndex)
{
	ArrayIndex_AC returnVal = beginIndex;
	if (beginIndex != endIndex)
		returnVal += rand() % labs(endIndex - beginIndex);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CompareObjects: 
//----------------------------------------------------------------------------------------
CompareResult_AC CompareElements_AC(void* element1,
							 void* element2,
							 void* yourDataPtr)
{
	return static_cast<CDynamicArray_AC*>(yourDataPtr)->CompareElements(element1, element2);
}


//========================================================================================
// struct CElementInserter
//========================================================================================
struct CElementInserter
{
public:
	CElementInserter(const void*& theElementPtr,
					 const CSortedDynamicArray_AC* theDynamicArray) :
		fElementPtr(theElementPtr),
		fDynamicArray(theDynamicArray)
	{ }

	const void*& fElementPtr;
	const CSortedDynamicArray_AC* fDynamicArray;
};


//----------------------------------------------------------------------------------------
// TestItemForInsert_AC: 
//----------------------------------------------------------------------------------------
CompareResult_AC TestItemForInsert_AC(ArrayIndex_AC anItem,
								void* yourDataPtr)
{
	const CElementInserter * insertInfo = (CElementInserter *)yourDataPtr;
	return insertInfo->fDynamicArray->CompareElements(insertInfo->fElementPtr, insertInfo->fDynamicArray->ComputeAddress(anItem));
}


//========================================================================================
// CLASS CSortedDynamicArray_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CSortedDynamicArray_AC destructor
//----------------------------------------------------------------------------------------
CSortedDynamicArray_AC::~CSortedDynamicArray_AC()
{
}

//----------------------------------------------------------------------------------------
// CSortedDynamicArray_AC Clone
//----------------------------------------------------------------------------------------
void* CSortedDynamicArray_AC::Clone() const
{
	return TH_new CSortedDynamicArray_AC(*this);
}

//----------------------------------------------------------------------------------------
// CSortedDynamicArray_AC::DoSearchElement: DON'T use exit to get out of this routine from
// your TestItem function or you will be really sad! (our debugger will check for you)
// That's why you can return true to stop enumerating. Signaling Failure is OK too.
//----------------------------------------------------------------------------------------
bool CSortedDynamicArray_AC::DoSearchElement(CompareIndexType_AC TestItem,
											 void* yourDataPtr,
											 ArrayIndex_AC& index) const
{
	bool found = false;

	if (fSize == kEmptyIndex_AC)
		index = 1;
	else
	{
		CompareResult_AC aCompareResult;
		CArrayIterator_AC iter(this);

		do
		{
			// (fLowBound + fHighBound) / 2
			iter.fCurrentIndex = (iter.fLowBound + iter.fHighBound) >> 1;

			aCompareResult = TestItem(iter.fCurrentIndex, yourDataPtr);

			if (aCompareResult <= kItemGreaterThanCriteria_AC)
				iter.fHighBound = iter.fCurrentIndex - 1;
			else
				iter.fLowBound = iter.fCurrentIndex + 1;

		} while (!((aCompareResult == kItemEqualCriteria_AC) || (iter.fLowBound > iter.fHighBound)));

		if (aCompareResult == kItemEqualCriteria_AC)
			found = true;
		else if (aCompareResult >= kItemLessThanCriteria_AC)
			++iter.fCurrentIndex;

		// keep index in range 
		index = ((iter.fCurrentIndex < 1) || (iter.fCurrentIndex > fSize + 1)) ? kEmptyIndex_AC : iter.fCurrentIndex;

	}

	return found;
}

//----------------------------------------------------------------------------------------
// CSortedDynamicArray_AC::InsertElementInOrder: 
//----------------------------------------------------------------------------------------
void CSortedDynamicArray_AC::InsertElementInOrder(const void* elementPtr)
{
	ArrayIndex_AC index;
	CElementInserter anElementInserter(elementPtr, this);

	DoSearchElement(&TestItemForInsert_AC, &anElementInserter, index);
	InsertElementsBefore(index, elementPtr, 1);
}

//----------------------------------------------------------------------------------------
// End of UDynamicArray.cp
