//----------------------------------------------------------------------------------------
// UList.cp 
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// ACS

#ifndef __Core_AC__
#include "core_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

#if 0 //SRF
#ifndef __USTREAM__
#include "UStream.h"
#endif
#endif

// Toolbox

#ifndef __PACKAGES__
#include <Packages.h>
#endif

// ANSI

#ifndef __STDLIB__
#include <stdlib.h>
#endif


//========================================================================================
// GLOBAL typedefs
//========================================================================================
struct ElementToVoidComparerStruct
{
	CompareVoidsType callersCompareItems;
	void* callersYourDataPtr;
};

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static CompareResult_AC TestElementForSearch(ArrayIndex_AC anItem,
										  void* yourDataPtr);
static CompareResult_AC TestElementForHandleSearch(ArrayIndex_AC anItem,
												void* yourDataPtr);
static CompareResult_AC CompareHandles(Handle anItem,
									void* yourDataPtr);
static CompareResult_AC TestElementForLongintSearch(ArrayIndex_AC anItem,
												 void* yourDataPtr);
static CompareResult_AC CompareLongs(long anItem,
								  void* yourDataPtr);
static CompareResult_AC TestElementForVoidSearch(ArrayIndex_AC anItem,
												 void* yourDataPtr);
static CompareResult_AC CompareVoids(void* anItem,
								  void* yourDataPtr);

CompareResult_AC ElementToVoidComparer(void* element1, void* element2, void* yourDataPtr);

//----------------------------------------------------------------------------------------
// NewList_AC: 
//----------------------------------------------------------------------------------------
CList_AC* NewList_AC()
{
	return TH_new CList_AC;
}

//----------------------------------------------------------------------------------------
// NewSortedList_AC: 
//----------------------------------------------------------------------------------------
CSortedList_AC* NewSortedList_AC()
{
	return TH_new CSortedList_AC;
}

//----------------------------------------------------------------------------------------
// NewAllocatedList_AC: 
//----------------------------------------------------------------------------------------
CList_AC* NewAllocatedList_AC(ArrayIndex_AC iSize)
{
	return TH_new CList_AC(iSize);
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
//========================================================================================
// struct CSearchHandleIndex
//========================================================================================

struct CSearchHandleIndex
{
public:
	// Constructor
	inline CSearchHandleIndex(CompareHandleType& TestItem,
							  void*& yourDataPtr,
							  Handle& theHandle,
							  const CSortedHandleList_AC* theHandleList) :
		fTestItem(TestItem),
		fStaticLink(yourDataPtr),
		fHandle(theHandle),
		fSortedHandleList(theHandleList)
	{ }

	CompareHandleType& fTestItem;
	void*& fStaticLink;
	Handle& fHandle;
	const CSortedHandleList_AC* fSortedHandleList;
};

//----------------------------------------------------------------------------------------
// TestElementForHandleSearch: 
//----------------------------------------------------------------------------------------
CompareResult_AC TestElementForHandleSearch(ArrayIndex_AC anItem,
										 void* yourDataPtr)
{
	CSearchHandleIndex * searchInfo = (CSearchHandleIndex *)yourDataPtr;
	searchInfo->fHandle = searchInfo->fSortedHandleList->At(anItem);

	return searchInfo->fTestItem(searchInfo->fHandle, searchInfo->fStaticLink);
}


//========================================================================================
// struct CHandleComparer
//========================================================================================

struct CHandleComparer
{
public:
	// Constructor
	inline CHandleComparer(const Handle& theHandle,
						   const CSortedHandleList_AC* theHandleList) :
		fHandle(theHandle),
		fSortedHandleList(theHandleList)
	{ }

	const Handle& fHandle;
	const CSortedHandleList_AC* fSortedHandleList;
};


//----------------------------------------------------------------------------------------
// CompareHandles: 
//----------------------------------------------------------------------------------------
CompareResult_AC CompareHandles(Handle anItem,
							 void* yourDataPtr)
{
	CHandleComparer * comparisonInfo = (CHandleComparer *)yourDataPtr;

	return comparisonInfo->fSortedHandleList->Compare(comparisonInfo->fHandle, anItem);
}


//========================================================================================
// CLASS CSortedHandleList_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CSortedHandleList_AC destructor
//----------------------------------------------------------------------------------------
CSortedHandleList_AC::~CSortedHandleList_AC()
{
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC Clone
//----------------------------------------------------------------------------------------
void* CSortedHandleList_AC::Clone() const
{
	return TH_new CSortedHandleList_AC(*this);
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::FreeAll: 
//----------------------------------------------------------------------------------------
void CSortedHandleList_AC::FreeAll()
{
	for (CHandleIterator_AC iter(this); iter; ++iter)
		::DisposeHandle(iter); //SRF should use DisposeIfHandle_AC

	DeleteAll();
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::FreeList: 
//----------------------------------------------------------------------------------------
void CSortedHandleList_AC::FreeList()
{
#if qDebug
	::ProgramBreak_AC("CSortedHandleList_AC::FreeList is deprecated. Please call FreeAll and then if necessary free the object");
#endif
	FreeAll();

	delete this;
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::At: 
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
// Return the index'th element of the list. 
Handle CSortedHandleList_AC::At(ArrayIndex_AC index) const
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CSortedHandleList_AC.At");
		::ThrowException_AC(minErr, 0);
	}
#endif

	return *((Handle*)ComputeAddress(index));
}
#endif

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::AtDelete: 
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
// Deletes via an index. 
void CSortedHandleList_AC::AtDelete(ArrayIndex_AC index)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CSortedHandleList_AC.AtDelete");
	}
#endif

	DeleteElementsAt(index, 1);
}
#endif

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::Delete: 
//----------------------------------------------------------------------------------------
void CSortedHandleList_AC::Delete(Handle item)
{

	if (item)
	{
#if qDebug
		VerboseIsHandle_AC(item);
#endif

		ArrayIndex_AC index = GetIdentityItemNo(item);
		if (index != kEmptyIndex_AC)
			AtDelete(index);
	}
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::First: 
//----------------------------------------------------------------------------------------
Handle CSortedHandleList_AC::First() const
{
	return fSize ? At(1) : NULL;
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::Last: 
//----------------------------------------------------------------------------------------
Handle CSortedHandleList_AC::Last() const
{
	return fSize ? At(fSize) : NULL;
}


//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::DoSearch: DON'T use exit to get out of this routine from your
// TestItem function or you will be really sad! (our debugger will check for you).
//----------------------------------------------------------------------------------------
Handle CSortedHandleList_AC::DoSearch(CompareHandleType TestItem,
								   void* yourDataPtr,
								   ArrayIndex_AC& index) const
{
	Handle aHandle;
	CSearchHandleIndex aSearcher(TestItem, yourDataPtr, aHandle, this);

	return (DoSearchElement(&::TestElementForHandleSearch, &aSearcher, index)) ? aHandle : NULL;
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::GetIdentityItemNo: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedHandleList_AC::GetIdentityItemNo(const Handle item) const
{
	ArrayIndex_AC returnVal = kEmptyIndex_AC;

	// the equality test should not change the list's size or the
	//  resulting index can be invalid

	if (item)
		for (TemplateArrayIterator_AC<Handle>iter(this); iter; ++iter)
			if (*iter == item)
			{
				returnVal = iter.CurrentIndex();
				break;
			}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::GetEqualItemNo: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedHandleList_AC::GetEqualItemNo(const Handle item) const
{
	ArrayIndex_AC returnVal = kEmptyIndex_AC;

	if (item)
	{
#if qDebug
		VerboseIsHandle_AC(item);
#endif

		CHandleComparer aHandleComparer(item, this);

		// Find the first reference to item in the list.
		// If item does not occur, return kEmptyIndex_AC (0).

		ArrayIndex_AC index;
		if (DoSearch(&::CompareHandles, &aHandleComparer, index))
			returnVal = index;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::Insert: 
//----------------------------------------------------------------------------------------
void CSortedHandleList_AC::Insert(const Handle item)
{
#if qDebug
	VerboseIsHandle_AC(item);
#endif

	InsertElementInOrder(&item);
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::Compare: 
//----------------------------------------------------------------------------------------
CompareResult_AC CSortedHandleList_AC::Compare(const Handle item1,
										 const Handle item2) const
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;
	// By default just compare the ordinal value of the items. Subclasses that want to use
	// the items as pointers or other types should override if the comparison is based on
	// the pointed at value
	if (item1 > item2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (item1 < item2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC::CompareElements: 
//----------------------------------------------------------------------------------------
CompareResult_AC CSortedHandleList_AC::CompareElements(const void* element1,
												 const void* element2) const// override 
{
	return Compare(*static_cast<const Handle*>(element1), *static_cast<const Handle*>(element2));
}

//========================================================================================
// CLASS CHandleList_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CHandleList_AC destructor
//----------------------------------------------------------------------------------------
CHandleList_AC::~CHandleList_AC()
{
}

//----------------------------------------------------------------------------------------
// CHandleList_AC Clone
//----------------------------------------------------------------------------------------
void* CHandleList_AC::Clone() const
{
	return TH_new CHandleList_AC(*this);
}

//----------------------------------------------------------------------------------------
// CHandleList_AC::InsertElementInOrder: Overridden to do arrival sequence insertion
//----------------------------------------------------------------------------------------
void CHandleList_AC::InsertElementInOrder(const void* ElementPtr)// override 
{
	InsertElementsBefore(fSize + 1, ElementPtr, 1);
}


//========================================================================================
// struct CSearchLongIndex
//========================================================================================
struct CSearchLongIndex
{
public:
	// Constructor
	inline CSearchLongIndex(CompareLongType& TestItem,
							void*& yourDataPtr,
							long& theLong,
							const CSortedLongintList_AC* theLongintList) :
		fTestItem(TestItem),
		fStaticLink(yourDataPtr),
		fLong(theLong),
		fLongintList(theLongintList)
	{ }

	CompareLongType& fTestItem;
	void*& fStaticLink;
	long& fLong;
	const CSortedLongintList_AC* fLongintList;
};


//----------------------------------------------------------------------------------------
// TestElementForLongintSearch: 
//----------------------------------------------------------------------------------------
CompareResult_AC TestElementForLongintSearch(ArrayIndex_AC anItem,
										  void* yourDataPtr)
{
	CSearchLongIndex * searchInfo = (CSearchLongIndex *)yourDataPtr;
	searchInfo->fLong = searchInfo->fLongintList->At(anItem);

	return searchInfo->fTestItem(searchInfo->fLong, searchInfo->fStaticLink);
}


//========================================================================================
// struct CLongComparer
//========================================================================================

struct CLongComparer
{
public:
	// Constructor
	inline CLongComparer(long& theLong,
						 const CSortedLongintList_AC* theSortedLongintList) :
		fLong(theLong),
		fSortedLongintList(theSortedLongintList)
	{ }

	long& fLong;
	const CSortedLongintList_AC* fSortedLongintList;
};


//----------------------------------------------------------------------------------------
// CompareLongs: 
//----------------------------------------------------------------------------------------
CompareResult_AC CompareLongs(long anItem,
						   void* yourDataPtr)
{
	CLongComparer * comparisonInfo = (CLongComparer *)yourDataPtr;

	return comparisonInfo->fSortedLongintList->Compare(comparisonInfo->fLong, anItem);
}


//========================================================================================
// GLOBAL Procedures
//========================================================================================
//========================================================================================
// CLASS CSortedLongintList_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CSortedLongintList_AC destructor
//----------------------------------------------------------------------------------------
CSortedLongintList_AC::~CSortedLongintList_AC()
{
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC Clone
//----------------------------------------------------------------------------------------
void* CSortedLongintList_AC::Clone() const
{
	return TH_new CSortedLongintList_AC(*this);
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::At: Return the index'th element of the list.
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
long CSortedLongintList_AC::At(ArrayIndex_AC index) const
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CSortedLongintList_AC.At");
		::ThrowException_AC(minErr, 0);
	}
#endif

	return *((long*)ComputeAddress(index));
}
#endif

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::AtDelete: Deletes via an index.
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
void CSortedLongintList_AC::AtDelete(ArrayIndex_AC index)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CSortedLongintList_AC.AtDelete");
	}
#endif

	DeleteElementsAt(index, 1);
}
#endif

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::First: 
//----------------------------------------------------------------------------------------
long CSortedLongintList_AC::First() const
{
	return fSize ? At(1) : NULL;
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::Last: 
//----------------------------------------------------------------------------------------
long CSortedLongintList_AC::Last() const
{
	return fSize ? At(fSize) : NULL;
}


//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::Delete: 
//----------------------------------------------------------------------------------------
void CSortedLongintList_AC::Delete(long item)
{
	ArrayIndex_AC index = GetIdentityItemNo(item);
	if (index != kEmptyIndex_AC)
		AtDelete(index);
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::GetIdentityItemNo: Find the first reference to the IDENTITY item in
// the list. If item does not occur, return 0.
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedLongintList_AC::GetIdentityItemNo(long item) const
{
	ArrayIndex_AC returnVal = 0;
	
	for (TemplateArrayIterator_AC<long>iter(this); iter; ++iter)
		if (*iter == item)
		{
			returnVal = iter.CurrentIndex();
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::Compare: 
//----------------------------------------------------------------------------------------
CompareResult_AC CSortedLongintList_AC::Compare(long item1,
										  long item2) const
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;
	// By default just compare the ordinal value of the items. Subclasses that want to use
	// the items as pointers or other types should override if the comparison is based on
	// the pointed at value
	if (item1 > item2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (item1 < item2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::CompareElements: 
//----------------------------------------------------------------------------------------
CompareResult_AC CSortedLongintList_AC::CompareElements(const void* element1,
												  const void* element2) const// override 
{
	return Compare(*static_cast<const long*>(element1), *static_cast<const long*>(element2));
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::DoSearch: DON'T use exit to get out of this routine from your
// TestItem function or you will be really sad! (our debugger will check for you).
//----------------------------------------------------------------------------------------
bool CSortedLongintList_AC::DoSearch(CompareLongType TestItem,
									 void* yourDataPtr,
									 ArrayIndex_AC& index) const
{
	long aLong = 0;
	CSearchLongIndex aSearcher(TestItem, yourDataPtr, aLong, this);

	return DoSearchElement(&TestElementForLongintSearch, &aSearcher, index);
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::GetEqualItemNo: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedLongintList_AC::GetEqualItemNo(long item) const
{
	ArrayIndex_AC returnVal = kEmptyIndex_AC;
	CLongComparer aLongComparer(item, this);

	// Find the first reference to item in the list.
	// If item does not occur, return kEmptyIndex_AC (0).
	ArrayIndex_AC index;
	if (DoSearch(&::CompareLongs, &aLongComparer, index))
		returnVal = index;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::Insert: 
//----------------------------------------------------------------------------------------
void CSortedLongintList_AC::Insert(long item)
{
	InsertElementInOrder(&item);
}

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC::Search: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedLongintList_AC::Search(CompareLongType TestItem,
									  void* yourDataPtr)
{
	ArrayIndex_AC returnVal = kEmptyIndex_AC;

	ArrayIndex_AC index;
	if (DoSearch(TestItem, yourDataPtr, index))
		returnVal = index;

	return returnVal;
}


//========================================================================================
// CLASS CLongintList_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CLongintList_AC destructor
//----------------------------------------------------------------------------------------
CLongintList_AC::~CLongintList_AC()
{
}

//----------------------------------------------------------------------------------------
// CLongintList_AC Clone
//----------------------------------------------------------------------------------------
void* CLongintList_AC::Clone() const
{
	return TH_new CLongintList_AC(*this);
}

//----------------------------------------------------------------------------------------
// CLongintList_AC::DoSearchElement: DON'T use exit to get out of this routine from your
// TestItem function or you will be really sad! (our debugger will check for you) That's
// why you can return true to stop enumerating. Signaling Failure is OK too.
//----------------------------------------------------------------------------------------
bool CLongintList_AC::DoSearchElement(CompareIndexType_AC TestItem,
									  void* yourDataPtr,
									  ArrayIndex_AC& index) const// override 
{
	bool returnVal = false;

	for (TemplateArrayIterator_AC<long>iter(this); iter; ++iter)
		if (TestItem(iter.CurrentIndex(), yourDataPtr) == kItem1EqualItem2_AC)
		{
			index = iter.CurrentIndex();
			returnVal = true;
			break;
		}

	if (!returnVal)
		index = fSize + 1;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CLongintList_AC::InsertElementInOrder: Overridden to do arrival sequence insertion
//----------------------------------------------------------------------------------------
void CLongintList_AC::InsertElementInOrder(const void* ElementPtr)// override 
{
	InsertElementsBefore(fSize + 1, ElementPtr, 1);
}

//----------------------------------------------------------------------------------------
// CLongintList_AC::AtPut: 
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
void CLongintList_AC::AtPut(ArrayIndex_AC index,
						 long newItem)
{
	// Replace the index'th element of the list.
	// Range check only if the compile-flag qRangeCheck is true.
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CLongintList_AC.AtPut");
		::ThrowException_AC(minErr, 0);
	}
#endif

	*((long*)ComputeAddress(index)) = newItem;
}
#endif

//----------------------------------------------------------------------------------------
// CLongintList_AC::InsertBefore: Insert a reference to item at the indicated index.
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
void CLongintList_AC::InsertBefore(ArrayIndex_AC index,
								long item)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize + 1))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CLongintList_AC.InsertBefore");
	}
#endif

	InsertElementsBefore(index, &item, 1);
}
#endif

//----------------------------------------------------------------------------------------
// CLongintList_AC::InsertLast
//----------------------------------------------------------------------------------------
void CLongintList_AC::InsertLast(long item)
{
	InsertBefore(fSize + 1, item);
}

//----------------------------------------------------------------------------------------
// CLongintList_AC::Pop: LIFO stack pop
//----------------------------------------------------------------------------------------
long CLongintList_AC::Pop()
{
	long topOfStack = 0;

	if (fSize)
	{
		topOfStack = At(fSize);
		AtDelete(fSize);
	}

	return topOfStack;
}


//========================================================================================
// struct CSearchVoidIndex
//========================================================================================
struct CSearchVoidIndex
{
public:
	// Constructor
	inline CSearchVoidIndex(CompareVoidType& TestItem,
							void*& yourDataPtr,
							void*& theVoid,
							const CSortedList_AC* theList) :
		fTestItem(TestItem),
		fStaticLink(yourDataPtr),
		fVoid(theVoid),
		fList(theList)
	{ }

	CompareVoidType& fTestItem;
	void*& fStaticLink;
	void*& fVoid;
	const CSortedList_AC* fList;
};


//----------------------------------------------------------------------------------------
// TestElementForVoidSearch: 
//----------------------------------------------------------------------------------------
CompareResult_AC TestElementForVoidSearch(ArrayIndex_AC anItem,
										  void* yourDataPtr)
{
	CSearchVoidIndex * searchInfo = (CSearchVoidIndex *)yourDataPtr;
	searchInfo->fVoid = searchInfo->fList->At(anItem);

	return searchInfo->fTestItem(searchInfo->fVoid, searchInfo->fStaticLink);
}


//========================================================================================
// struct CVoidComparer
//========================================================================================

struct CVoidComparer
{
public:
	// Constructor
	inline CVoidComparer(const void*& theVoid,
						 const CSortedList_AC* theSortedList) :
		fVoid(theVoid),
		fSortedList(theSortedList)
	{ }

	const void*& fVoid;
	const CSortedList_AC* fSortedList;
};


//----------------------------------------------------------------------------------------
// CompareVoids: 
//----------------------------------------------------------------------------------------
CompareResult_AC CompareVoids(void* anItem,
						   void* yourDataPtr)
{
	CVoidComparer * comparisonInfo = (CVoidComparer *)yourDataPtr;

	return comparisonInfo->fSortedList->Compare(comparisonInfo->fVoid, anItem);
}


//========================================================================================
// CLASS CSortedList_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CSortedList_AC destructor
//----------------------------------------------------------------------------------------
CSortedList_AC::~CSortedList_AC()
{
}

//----------------------------------------------------------------------------------------
// CSortedList_AC Clone
//----------------------------------------------------------------------------------------
void* CSortedList_AC::Clone() const
{
	return TH_new CSortedList_AC(*this);
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::At: Return the index'th element of the list.
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
void* CSortedList_AC::At(ArrayIndex_AC index) const
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CSortedList_AC.At");
		::ThrowException_AC(minErr, 0);
	}
#endif

	return *static_cast<void**>(ComputeAddress(index));
}
#endif

//----------------------------------------------------------------------------------------
// CSortedList_AC::AtDelete: Deletes via an index.
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
void CSortedList_AC::AtDelete(ArrayIndex_AC index)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CSortedList_AC.AtDelete");
	}
#endif

	DeleteElementsAt(index, 1);
}
#endif

//----------------------------------------------------------------------------------------
// CSortedList_AC::First: 
//----------------------------------------------------------------------------------------
void* CSortedList_AC::First() const
{
	return fSize ? At(1) : NULL;
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::Last: 
//----------------------------------------------------------------------------------------
void* CSortedList_AC::Last() const
{
	return fSize ? At(fSize) : NULL;
}


//----------------------------------------------------------------------------------------
// CSortedList_AC::Delete: 
//----------------------------------------------------------------------------------------
void CSortedList_AC::Delete(void* item)
{
	ArrayIndex_AC index = GetIdentityItemNo(item);
	if (index != kEmptyIndex_AC)
		AtDelete(index);
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::GetIdentityItemNo: Find the first reference to the IDENTITY item in
// the list. If item does not occur, return 0.
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedList_AC::GetIdentityItemNo(const void* item) const
{
	ArrayIndex_AC returnVal = 0;

	for (TemplateArrayIterator_AC<void*>iter(this); iter; ++iter)
		if (*iter == item)
		{
			returnVal = iter.CurrentIndex();
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::Compare: 
//----------------------------------------------------------------------------------------
CompareResult_AC CSortedList_AC::Compare(const void* item1,
										  const void* item2) const
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;
	// By default just compare the ordinal value of the items. Subclasses that want to use
	// the items as pointers or other types should override if the comparison is based on
	// the pointed at value
	if (item1 > item2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (item1 < item2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::CompareElements: 
//----------------------------------------------------------------------------------------
CompareResult_AC CSortedList_AC::CompareElements(const void* element1,
												  const void* element2) const // override 
{
	return Compare(*static_cast<void const * const*>(element1), *static_cast<void const * const*>(element2));
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::DoSearch: DON'T use exit to get out of this routine from your
// TestItem function or you will be really sad! (our debugger will check for you).
//----------------------------------------------------------------------------------------
void* CSortedList_AC::DoSearch(CompareVoidType TestItem,
									 void* yourDataPtr,
									 ArrayIndex_AC& index) const
{
	void* aVoid = NULL;
	CSearchVoidIndex aSearcher(TestItem, yourDataPtr, aVoid, this);

	return DoSearchElement(&TestElementForVoidSearch, &aSearcher, index) ? aVoid : NULL;
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::GetEqualItemNo: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedList_AC::GetEqualItemNo(const void* item) const
{
	ArrayIndex_AC returnVal = kEmptyIndex_AC;
	CVoidComparer aVoidComparer(item, this);

	// Find the first reference to item in the list.
	// If item does not occur, return kEmptyIndex_AC (0).
	ArrayIndex_AC index;
	if (DoSearch(&::CompareVoids, &aVoidComparer, index))
		returnVal = index;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::Insert: 
//----------------------------------------------------------------------------------------
void CSortedList_AC::Insert(const void* item)
{
	InsertElementInOrder(&item);
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::Search: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CSortedList_AC::Search(CompareVoidType TestItem,
									  void* yourDataPtr)
{
	ArrayIndex_AC returnVal = kEmptyIndex_AC;

	ArrayIndex_AC index;
	if (DoSearch(TestItem, yourDataPtr, index))
		returnVal = index;

	return returnVal;
}


//----------------------------------------------------------------------------------------
// Little wrapper function that forward the function call with a dereference of the element pointers
//----------------------------------------------------------------------------------------
CompareResult_AC ElementToVoidComparer(void* element1, void* element2, void* yourDataPtr)
{
	ElementToVoidComparerStruct& callerData = *(ElementToVoidComparerStruct*)yourDataPtr;

	return callerData.callersCompareItems(*static_cast<void**>(element1), *static_cast<void**>(element2), callerData.callersYourDataPtr);
}

//----------------------------------------------------------------------------------------
// CSortedList_AC::SortBy: NOTE: This doesn't work with a CompareItems Function that inserts
// or deletes elements.
//----------------------------------------------------------------------------------------
void CSortedList_AC::SortBy(CompareVoidsType CompareItems,
						 void* yourDataPtr)
{
	ElementToVoidComparerStruct callerData;
	callerData.callersCompareItems = CompareItems;
	callerData.callersYourDataPtr = yourDataPtr;
	
	SortElementsBy(::ElementToVoidComparer, &callerData);
}


//========================================================================================
// CLASS CList_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CList_AC destructor
//----------------------------------------------------------------------------------------
CList_AC::~CList_AC()
{
}

//----------------------------------------------------------------------------------------
// CList_AC Clone
//----------------------------------------------------------------------------------------
void* CList_AC::Clone() const
{
	return TH_new CList_AC(*this);
}

//----------------------------------------------------------------------------------------
// CList_AC::DoSearchElement: DON'T use exit to get out of this routine from your
// TestItem function or you will be really sad! (our debugger will check for you) That's
// why you can return true to stop enumerating. Signaling Failure is OK too.
//----------------------------------------------------------------------------------------
bool CList_AC::DoSearchElement(CompareIndexType_AC TestItem,
									  void* yourDataPtr,
									  ArrayIndex_AC& index) const// override 
{
	bool returnVal = false;

	for (CArrayIterator_AC iter(this); iter.Current(); ++iter)
		if (TestItem(iter.Current(), yourDataPtr) == kItem1EqualItem2_AC)
		{
			index = iter.Current();
			returnVal = true;
			break;
		}

	if (!returnVal)
		index = fSize + 1;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CList_AC::InsertElementInOrder: Overridden to do arrival sequence insertion
//----------------------------------------------------------------------------------------
void CList_AC::InsertElementInOrder(const void* ElementPtr)// override 
{
	InsertElementsBefore(fSize + 1, ElementPtr, 1);
}

//----------------------------------------------------------------------------------------
// CList_AC::AtPut: 
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
void CList_AC::AtPut(ArrayIndex_AC index,
						 void* newItem)
{
	// Replace the index'th element of the list.
	// Range check only if the compile-flag qRangeCheck is true.
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CList_AC.AtPut");
		::ThrowException_AC(minErr, 0);
	}
#endif

	*static_cast<void**>(ComputeAddress(index)) = newItem;
}
#endif

//----------------------------------------------------------------------------------------
// CList_AC::InsertBefore: Insert a reference to item at the indicated index.
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
void CList_AC::InsertBefore(ArrayIndex_AC index,
								void* item)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize + 1))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CList_AC.InsertBefore");
	}
#endif

	InsertElementsBefore(index, &item, 1);
}
#endif

//----------------------------------------------------------------------------------------
// CList_AC::InsertLast
//----------------------------------------------------------------------------------------
void CList_AC::InsertLast(void* item)
{
	InsertBefore(fSize + 1, item);
}

//----------------------------------------------------------------------------------------
// CList_AC::Pop: LIFO stack pop
//----------------------------------------------------------------------------------------
void* CList_AC::Pop()
{
	void* topOfStack = 0;

	if (fSize)
	{
		topOfStack = At(fSize);
		AtDelete(fSize);
	}

	return topOfStack;
}

//----------------------------------------------------------------------------------------
// End of UList.cp
