//----------------------------------------------------------------------------------------
// Assertions_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CAssertionFailure_AC__
#define __CAssertionFailure_AC__

#ifndef __Core_AC__
#error "Include CoreAC.h to get Assertions_AC.h"
#endif

#	if qAssertions

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CAssertionFailure_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CAssertionFailure_AC : public CException_AC
{
public:
	CAssertionFailure_AC();

	CAssertionFailure_AC(const CAssertionFailure_AC& inEx);

	CAssertionFailure_AC(CError_AC inError, Message inMessage = 0, const char* inString = NULL);

	virtual ~CAssertionFailure_AC();
	
	CAssertionFailure_AC& operator = (const CAssertionFailure_AC& inEx);

	const char* GetAssertionMsg() const;
	
	void SetAssertionMsg(const char* inString);

	virtual const char* what() const throw();
		// exception override.
	
private:
	char fMsgString[256];
};

// inline functions

inline const char* CAssertionFailure_AC::GetAssertionMsg() const
{	
	return fMsgString; 
}
inline CAssertionFailure_AC& CAssertionFailure_AC::operator = (const CAssertionFailure_AC& inEx)
{ 	
	SetAssertionMsg(inEx.GetAssertionMsg()); 
		// copy msg
	CException_AC::operator = (inEx); 
	return *this; 
} 
inline CAssertionFailure_AC::CAssertionFailure_AC() 
{ 
	*fMsgString = '\0';
}
inline CAssertionFailure_AC::CAssertionFailure_AC(const CAssertionFailure_AC& inEx) 
{ 	
	*this = inEx; 
}
inline CAssertionFailure_AC::CAssertionFailure_AC(CError_AC inError, Message inMessage, const char* inString) 
	: CException_AC(inError, inMessage) 
{
	SetAssertionMsg(inString);
} 
inline CAssertionFailure_AC::~CAssertionFailure_AC() 
{
}
inline const char* CAssertionFailure_AC::what() const throw() 
{ 
	return "CAssertionFailure_AC"; 
}

#define Assertion_AC(condition, msg) 	DebugAssert_AC((condition), msg, __FILE__, __LINE__ )

#define AssertNotNULL_AC( ptr ) 		DebugAssertNotNULL_AC((ptr), __FILE__, __LINE__ )

#define AssertGoodCast( ptr )			DebugAssertGoodCast_AC((ptr), __FILE__, __LINE__ )

#define Warning_AC(condition,msg)		DebugWarning_AC((condition), msg,  __FILE__, __LINE__ )

	// ok to send in NULL for msg.

EXPORT_AC void DebugAssert_AC( bool condition, const char* optionalMsg, char* inFile, short inLine);
EXPORT_AC void DebugWarning_AC( bool condition, char* message, char* inFile, short inLine);
EXPORT_AC void DebugAssertNotNULL_AC( void* inPtr, char* inFile, short inLine);
EXPORT_AC void DebugAssertNotNULL_AC( const void* inPtr, char* inFile, short inLine);
EXPORT_AC void DebugAssertGoodCast_AC( const void* inPtr, char* inFile, short inLine);

inline void DebugAssertNotNULL_AC( void* inPtr, char* inFile, short inLine)
{	DebugAssertNotNULL_AC((const void* )inPtr, inFile, inLine); }

#	else // qAssertions

inline void AssertNotNULL_AC(const void*) {}
inline void Assertion_AC(bool, const char*) {}
inline void Warning_AC(bool, const char*) {}
inline void AssertGoodCast_AC( const void* ) {}

#	endif


#endif

