//----------------------------------------------------------------------------------------
// CHandle_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CHandle_AC__
#define __CHandle_AC__

// ACS Toolbox

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Global constants declarations.
//----------------------------------------------------------------------------------------

enum EMoveHiState_AC
{
	eDontMoveHi,
	eMoveHi
};

enum EReleaseState_AC
{
	eDontReleaseIt,
	eReleaseIt
};

enum ELockState_AC
{
	eLockIt,
	eDontLockIt
};

#if qCoreDebug
const SignedByte kNewInitVal_AC = 0xF3;				// debugging constant, odd at all byte boundaries 
const SignedByte kResizedPtrInitVal_AC = 0xF5;		// debugging constant, odd at all byte boundaries 
const SignedByte kResizedInitVal_AC = 0xF7;			// debugging constant, odd at all byte boundaries 
const SignedByte kDisposedInitVal_AC = 0xD3;		// debugging constant, odd at all byte boundaries 
#endif

//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

inline bool odd_AC(void* value)
{ return (reinterpret_cast<long>(value) & 0x00000001); }

EXPORT_AC Handle DisposeIfHandle_AC(Handle aHandle);

EXPORT_AC Ptr DisposeIfPtr_AC(Ptr aPtr);


EXPORT_AC Handle NewPermHandle_AC(Size size);
// Creates a handle by calling NewHandle, or if a PermHandleHandler is installed, calls that
EXPORT_AC void SetPermHandleSize_AC(Handle h, Size newSize);

class EXPORT_AC CPermHandleHandler_AC
{
public:
	CPermHandleHandler_AC();
	virtual ~CPermHandleHandler_AC();
	
	virtual Handle DoNewHandle(Size size);
		// A subclass may perform pre or post-processing by overriding this & calling inherited.
		
	virtual void DoSetHandleSize(Handle h, Size newSize);
		// A subclass may perform pre or post-processing by overriding this & calling inherited.
	
};

EXPORT_AC void SetPermHandleHandler_AC(CPermHandleHandler_AC* newHandler);
// Installs a handle allocator which will be used by NewPermHandle and SetPermHandleSize

EXPORT_AC
#if GENERATINGCFM
#define DisposeIfRoutineDescriptor_AC(p) ((p) ? (DisposeRoutineDescriptor(p), (UniversalProcPtr)NULL) : (UniversalProcPtr)NULL)
#else
#define DisposeIfRoutineDescriptor_AC(p) NULL
#endif

//----------------------------------------------------------------------------------------
// Handle helpers
//----------------------------------------------------------------------------------------

template <class T>
inline void Lock_AC(T inHandle)
	{ return ::HLock(reinterpret_cast<Handle>(inHandle)); }

template <class T>
inline void Unlock_AC(T inHandle)
	{ ::HUnlock(reinterpret_cast<Handle>(inHandle)); }

template <class T>
inline T DisposeIfNotNULL_AC(T& inHandle)
	{ return reinterpret_cast<T>(DisposeIfHandle_AC(reinterpret_cast<Handle>(inHandle))); }


EXPORT_AC bool IsAResource_AC(Handle h);

#if qCoreDebug
EXPORT_AC bool IsHandle_AC(Handle h);
EXPORT_AC bool VerboseIsHandle_AC(Handle h);
#endif

EXPORT_AC bool IsHandleLocked_AC(Handle h);

#if qCoreDebug
EXPORT_AC 	bool IsHandlePurged_AC(Handle h);
#else
	inline bool IsHandlePurged_AC(Handle h)
	{ return (!*h); }
#endif

EXPORT_AC OSErr CopyHandle_AC(const Handle inHandle, Handle& outHandle);
	// A nice HandToHand replacement that makes the temp for you.

//----------------------------------------------------------------------------------------
// Lock a handle, optionally moving it in memory
//----------------------------------------------------------------------------------------

EXPORT_AC SignedByte LockHandle_AC(Handle h, EMoveHiState_AC high = eDontMoveHi);
inline SignedByte LockHandleHigh_AC(Handle h) { return LockHandle_AC(h, eDontMoveHi); }

//----------------------------------------------------------------------------------------
// Store a handle's state temporarily, optionally locking it
//----------------------------------------------------------------------------------------
class EXPORT_AC CTempHLock_AC
{
public:
	CTempHLock_AC(Handle itsData = NULL);
	// Sets the pointee to itsData
	// and locks the handle.

	CTempHLock_AC(Handle itsData, EMoveHiState_AC high, EReleaseState_AC releaseIt = eDontReleaseIt, ELockState_AC lockIt = eLockIt);
	// Sets the pointee to itsData
	// lockit locks the handle, high moves it high first, releaseit releases (as resource) when done.

	inline ~CTempHLock_AC() { RestoreState(); }
	// restores the old state

	void RestoreState();

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	Handle	pointee;
	short fHandleBits;
	bool fRelease;
};

//----------------------------------------------------------------------------------------
// Store a handle's state temporarily, optionally locking it
//----------------------------------------------------------------------------------------
#define DontUnlock release
template <class T>
class TemplateTempHandleLock : public CTempHLock_AC
{
public:
	inline TemplateTempHandleLock(T inData = NULL) :
	CTempHLock_AC(reinterpret_cast<Handle>(inData))
	{ }
	// Sets the pointee to itsData
	// and locks the handle.

	inline TemplateTempHandleLock(T inData, EMoveHiState_AC high, EReleaseState_AC releaseIt = eDontReleaseIt, ELockState_AC lockIt = eLockIt) :
	CTempHLock_AC(reinterpret_cast<Handle>(inData), high, releaseIt, lockIt)
	{ }
	// Sets the pointee to itsData
	// lockit locks the handle, high moves it high first, releaseit releases (as resource) when done.



// DXC	inline T operator->() const { return get(); }

	inline T get() const { return reinterpret_cast<T>(pointee); }

	inline T release() { T oldPointee = get(); pointee = NULL; return oldPointee; }

	inline void reset(T p = NULL) { if (reinterpret_cast<T>(pointee) != p) { RestoreState(); pointee = reinterpret_cast<Handle>(p); } }

	inline TemplateTempHandleLock& operator=(T rhs) { pointee = reinterpret_cast<Handle>(rhs); return *this; }

	inline operator T() { return *reinterpret_cast<T*>(&pointee); }
		// Conversion operator, for converting data to desired return type.
};

typedef TemplateTempHandleLock<Handle> CTempHandleLock_AC;

//----------------------------------------------------------------------------------------
// Manage a temporary Handle
//----------------------------------------------------------------------------------------
template <class T>
class TemplateTempHandle
{
public:
	inline TemplateTempHandle(T inData = NULL) : pointee(inData) { }
	// Sets the pointee to itsData

	inline ~TemplateTempHandle() { DisposeIfHandle_AC(reinterpret_cast<Handle>(get())); }
	// Deallocates the data



//	inline T operator->() const { return get(); }

	inline T get() const { return reinterpret_cast<T>(pointee); }

	inline T release() { T returnVal = get(); pointee = NULL; return returnVal; }
		// Clears the owned reference to the data when returning it so it won't get released.
	
	inline void reset(T p = NULL) { if (pointee != p) { DisposeIfHandle_AC(reinterpret_cast<Handle>(get())); pointee = p; } }

	inline TemplateTempHandle& operator=(T rhs) { pointee = rhs; return *this; }

	inline operator T() const { return get(); }
		// Conversion operator, for converting data to desired return type.
	
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	T pointee;
};

typedef TemplateTempHandle<Handle> CTempHandle_AC;

//----------------------------------------------------------------------------------------
// Manage a temporary Ptr (from the toolbox)
//----------------------------------------------------------------------------------------
template <class T>
class TemplateTempPtr
{
public:
	inline TemplateTempPtr(T inData = NULL) : pointee(inData) { }
	// Sets the pointee to itsData

	inline ~TemplateTempPtr() { DisposeIfPtr_AC(reinterpret_cast<Ptr>(get())); }
	// Deallocates the data



// DXC	inline T operator->() const { return get(); }

	inline T get() const { return pointee; }

	inline T release() { T returnVal = get(); pointee = NULL; return returnVal; }
		// Clears the owned reference to the data when returning it so it won't get released.

	inline void reset(T p = NULL) { if (pointee != p) { DisposeIfPtr_AC(reinterpret_cast<Ptr>(get())); pointee = p; } }

	inline TemplateTempPtr& operator=(T rhs) { pointee = rhs; return *this; }

	inline operator T() const { return get(); }
		// Conversion operator, for converting data to desired return type.
	
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	T pointee;
};

typedef TemplateTempPtr<Ptr> CTempPtr_AC;

//----------------------------------------------------------------------------------------
// Manage a temporary RoutineDescriptor
//----------------------------------------------------------------------------------------
template <class T>
class TemplateTempRoutineDescriptor
{
public:
	inline TemplateTempRoutineDescriptor(T inData = NULL) : pointee(inData) { }
	// Sets the pointee to itsData

	inline ~TemplateTempRoutineDescriptor() { DisposeIfRoutineDescriptor_AC(static_cast<UniversalProcPtr>(get())); }
	// Deallocates the data

	inline T get() const { return pointee; }

	inline T release() { T returnVal = pointee; pointee = NULL; return returnVal; }
		// Clears the owned reference to the data when returning it so it won't get released.

	inline void reset(T p = NULL) { if (pointee != p) { DisposeIfRoutineDescriptor_AC(static_cast<UniversalProcPtr>(get())); pointee = p; } }

	inline TemplateTempRoutineDescriptor& operator=(T rhs) { pointee = rhs; return *this; }

	inline operator T() const { return get(); }
		// Conversion operator, for converting data to desired return type.
	
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	T pointee;
};

typedef TemplateTempRoutineDescriptor<UniversalProcPtr> CTempRoutineDescriptor_AC;

#endif // __CHandle_AC__
