// CPascalString_AC.h 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.


#ifndef __CPascalString_AC__
#define __CPascalString_AC__

// ACS Core

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef __LIMITS__
#include <Limits.h>
#endif

// If Length() is defined as a macro in Types.h, undefine it
// so we can have member functions with the same name
#ifdef Length
#undef Length
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

typedef unsigned char Str2[3];
typedef unsigned char Str8[9];

// Forward declaration for all the CCharStr_AC classes.
class CCharStr_AC;
class CChar255_AC;
typedef class CCharStr_AC *CCharStrPtr_AC, **CCharStrHandle_AC;

// Forward declaration for all the CPascalStr_AC classes.
class CPascalStr_AC;
class CStr255_AC;

typedef class CPascalStr_AC *CPascalStrPtr_AC, **CPascalStrHandle_AC;

// Some constants defining the length of each of the CPascalStr_AC types.

const unsigned char kStr255Len = 255;
const unsigned char kStr63Len = 63;
const unsigned char kStr32Len = 32;
const unsigned char kStr31Len = 31;
const unsigned char kStr27Len = 27;
const unsigned char kStr15Len = 15;
const unsigned char kStr2Len = 2;
const unsigned char kLengthByte = 1;

//----------------------------------------------------------------------------------------
// CCharStr_AC: simple class to enclose a null terminated c style string.
//----------------------------------------------------------------------------------------
class EXPORT_AC CCharStr_AC
{
public:
	//
	// No Constructors provided. We don't want anyone constructing this class
	// directly since it has no storage. Besides, constructors and assignment operators
	// are not inherited anyways.
	//
	void CopyFrom(const unsigned char* str, unsigned char maxLength);
	// From Pascal style string with length byte at beginning

	void CopyFrom(const char* str, size_t maxLength);
	// From C style string with null terminator at end

	void CopyFrom(const void* str, size_t forLength);
	// From raw memory for a given length

	size_t Length() const;
	
	inline bool IsEmpty() const
	{ return !*reinterpret_cast<const char*>(this); }

	// Character selector operator.

	inline char& operator[](size_t index)
	{ return reinterpret_cast<char*>(this)[index]; }

	inline char operator[](size_t index) const
	{ return reinterpret_cast<const char*>(this)[index]; }
	
	inline operator const char*() const
	{ return reinterpret_cast<const char*>(this); }

	inline operator char*()
	{ return reinterpret_cast<char*>(this); }
};

//----------------------------------------------------------------------------------------
// CChar255_AC:
//----------------------------------------------------------------------------------------
class EXPORT_AC CChar255_AC : public CCharStr_AC
{
	enum { kMaxLength = kStr255Len };
	char fStr[kMaxLength + 1];

public:
	inline CChar255_AC()
	{ *reinterpret_cast<char*>(this) = 0; }
	
	inline CChar255_AC(const char* str)
	{ CopyFrom(str, kMaxLength); }
	
	inline CChar255_AC(const unsigned char* str)
	{ CopyFrom(str, kMaxLength); }

	inline CChar255_AC(const void* str, size_t maxLength)
	{ CopyFrom(str, maxLength); }

	inline CChar255_AC(const CPascalStr_AC& str)
	{ CopyFrom(reinterpret_cast<const unsigned char*>(&str), kMaxLength); }

	inline CChar255_AC(const CStr255_AC& str)
	{ CopyFrom(reinterpret_cast<const unsigned char*>(&str), kMaxLength); }
};

//----------------------------------------------------------------------------------------
// TemplateCharStr:
//----------------------------------------------------------------------------------------
template <class T>
class TemplateCharStr : public CCharStr_AC
{
public:
	enum { kMaxLength = sizeof(T) - 1 };
	T fStr;

public:
	// Constructors

	inline TemplateCharStr()
	{ *reinterpret_cast<char*>(this) = 0; }
	
	inline TemplateCharStr(const char* str)
	{ CopyFrom(str, kMaxLength); }
	
	inline TemplateCharStr(const unsigned char* str)
	{ CopyFrom(str, kMaxLength); }

	inline TemplateCharStr(const void* str, size_t maxLength)
	{ CopyFrom(str, maxLength); }

	inline TemplateCharStr(const CPascalStr_AC& str)
	{ CopyFrom(reinterpret_cast<const unsigned char*>(&str), kMaxLength); }
};

//----------------------------------------------------------------------------------------
// CPascalStr_AC: Superclass of all Pascal string compatible string classes.
//----------------------------------------------------------------------------------------
class EXPORT_AC CPascalStr_AC
{
public:
	//
	// No Constructors provided. We don't want anyone constructing this class
	// directly since it has no storage. Besides, constructors and assignment operators
	// are not inherited anyways.
	//
	void CopyFrom(const unsigned char* str, unsigned char maxLength);
	// From Pascal style string with length byte at beginning

	void CopyFrom(const char* str, unsigned char maxLength);
	// From C style string with null terminator at end

	void CopyFrom(const void* str, unsigned char forLength);
	// From raw memory for a given length

	void CopyFrom(const unsigned char* str);
	// From Pascal style string with length byte at beginning
	// Copies up to kStr255Len bytes, be careful

	void CopyFrom(const char* str);
	// From C style string with null terminator at end
	// Copies up to kStr255Len bytes, be careful

#if qPowerPC || qNeedsMC68020
	inline void CopyFrom(unsigned long id)
	{ *reinterpret_cast<unsigned char*>(this) = sizeof(id); *reinterpret_cast<unsigned long*>(&reinterpret_cast<unsigned char*>(this)[1]) = id; }
#else
	void CopyFrom(unsigned long id);
#endif
	// From a long value directly into the string

#if qPowerPC || qNeedsMC68020
	inline void CopyFrom(unsigned short num)
	{ *reinterpret_cast<unsigned char*>(this) = sizeof(num); *reinterpret_cast<unsigned short*>(&reinterpret_cast<unsigned char*>(this)[1]) = num; }
#else
	void CopyFrom(unsigned short num);
#endif
	// From a short value directly into the string

	inline void CopyFrom(unsigned char ch)
	{ *reinterpret_cast<unsigned char*>(this) = sizeof(ch); reinterpret_cast<unsigned char*>(this)[1] = ch; }
	// From a short value into the string

	inline void CopyFrom(const CPascalStr_AC& str, unsigned char maxLength)
	{ CopyFrom(reinterpret_cast<const unsigned char*>(&str), maxLength); }
	// From CPascalStr_AC (Pascal) style string with length byte at beginning

	inline void CopyFrom(const CPascalStr_AC& str)
	{ CopyFrom(reinterpret_cast<const unsigned char*>(&str)); }
	// From CPascalStr_AC (Pascal) style string with length byte at beginning
	// Copies up to kStr255Len bytes, be careful

	void CopyFrom(short strListID, short index, unsigned char maxLength);
	// From toolbox Str# resource using GetIndString

	void CopyFrom(const Handle inHandle, unsigned char maxLength);
	// Copies up to kStr255Len bytes, be careful and use on a string that's big enough
	// as indicated in maxLength

	void CopyTo(unsigned char* str) const;
	// Copies this string TO str with a length byte

	void CopyTo(char* str) const;
	// Copies this string TO str with a null termination

	void CopyTo(void* str) const;
	// Copies this string TO str without a null termination

	unsigned char GetMaxLength(unsigned char pos, unsigned char length) const;
	// returns the maximum length to copy starting from pos
	// for length characters OR the length of the string whichever comes first.

	//
	// Character selection operator.
	//
	inline unsigned char& operator[](int pos)
	{ return reinterpret_cast<unsigned char*>(this)[pos]; }

	inline const unsigned char& operator[](int pos) const
	{ return reinterpret_cast<const unsigned char*>(this)[pos]; }
	
	//
	// Basic length method, inherited by all derived classes. Returns a reference
	//
	inline unsigned char& Length()
	{ return *reinterpret_cast<unsigned char*>(this); }

	inline const unsigned char& Length() const
	{ return *reinterpret_cast<const unsigned char*>(this); }

	inline bool IsEmpty() const
	{ return !*reinterpret_cast<const unsigned char*>(this); }

	inline void Empty()
	{ *reinterpret_cast<unsigned char*>(this) = 0; }

#if defined(qBackwardCompatible) && qBackwardCompatible
	// Returns a copy of the string on the stack. (as a null terminated c style string)
	// Note that this will not work when inlining is turned off. MacApp makes no use of this
	// operator internally. However, it is still provided as a convenience for compatibility.
	// Ideally you should write code that constructs a CChar255_AC in place from your string
	inline operator const char*() const
	{ return CChar255_AC(*this); }
#endif

	// Used to create a toolbox type Str255, etc. from our CPascalStr_AC. This is simply a type
	// coercion! Both CPascalStr_AC and Str255, etc. are expected to be pascal-style strings.
	inline operator unsigned char*()
	{ return reinterpret_cast<unsigned char*>(this); }
	
	inline operator const unsigned char*() const
	{ return reinterpret_cast<const unsigned char *>(this); }

	//------------------------------------------------------------------------------------
	
	// Return an ID represented as a CPascalStr_AC to the actual ID (an unsigned long).
	
	unsigned long ToULong() const;
	unsigned short ToUShort() const;

	// Relational operators that are inherited by all the derived CPascalStr_AC types. Five of
	// each so that literal C Strings and characters can be conveniently used for one of the
	// operators as well as two of the derived classes as operators.
	static short CPascalStrCompare(const CPascalStr_AC& s1, const CPascalStr_AC& s2);
	static short CPascalStrCompare(const CPascalStr_AC& s1, const char* s2);
	static short CPascalStrCompare(const CPascalStr_AC& s1, unsigned char ch);
	static short CPascalStrCompare(const char* s1, const CPascalStr_AC& s2);
	static short CPascalStrCompare(unsigned char ch, const CPascalStr_AC& s2);

	friend bool operator==(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) == 0; }
	friend bool operator==(const CPascalStr_AC& s1, const char* s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) == 0; }
	friend bool operator==(const CPascalStr_AC& s1, unsigned char ch)
	{ return CPascalStr_AC::CPascalStrCompare(s1, ch) == 0; }
	friend bool operator==(const char* s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) == 0; }
	friend bool operator==(unsigned char ch, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(ch, s2) == 0; }

	friend bool operator!=(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) != 0; }
	friend bool operator!=(const CPascalStr_AC& s1, const char* s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) != 0; }
	friend bool operator!=(const CPascalStr_AC& s1, unsigned char ch)
	{ return CPascalStr_AC::CPascalStrCompare(s1, ch) != 0; }
	friend bool operator!=(const char* s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) != 0; }
	friend bool operator!=(unsigned char ch, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(ch, s2) != 0; }

	friend bool operator>(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) > 0; }
	friend bool operator>(const CPascalStr_AC& s1, const char* s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) > 0; }
	friend bool operator>(const CPascalStr_AC& s1, unsigned char ch)
	{ return CPascalStr_AC::CPascalStrCompare(s1, ch) > 0; }
	friend bool operator>(const char* s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) > 0; }
	friend bool operator>(unsigned char ch, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(ch, s2) > 0; }

	friend bool operator<(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) < 0; }
	friend bool operator<(const CPascalStr_AC& s1, const char* s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) < 0; }
	friend bool operator<(const CPascalStr_AC& s1, unsigned char ch)
	{ return CPascalStr_AC::CPascalStrCompare(s1, ch) < 0; }
	friend bool operator<(const char* s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) < 0; }
	friend bool operator<(unsigned char ch, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(ch, s2) < 0; }

	friend bool operator>=(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) >= 0; }
	friend bool operator>=(const CPascalStr_AC& s1, const char* s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) >= 0; }
	friend bool operator>=(const CPascalStr_AC& s1, unsigned char ch)
	{ return CPascalStr_AC::CPascalStrCompare(s1, ch) >= 0; }
	friend bool operator>=(const char* s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) >= 0; }
	friend bool operator>=(unsigned char ch, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(ch, s2) >= 0; }

	friend bool operator<=(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) <= 0; }
	friend bool operator<=(const CPascalStr_AC& s1, const char* s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) <= 0; }
	friend bool operator<=(const CPascalStr_AC& s1, unsigned char ch)
	{ return CPascalStr_AC::CPascalStrCompare(s1, ch) <= 0; }
	friend bool operator<=(const char* s1, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(s1, s2) <= 0; }
	friend bool operator<=(unsigned char ch, const CPascalStr_AC& s2)
	{ return CPascalStr_AC::CPascalStrCompare(ch, s2) <= 0; }

	// Concatenation operator that are inherited by all the derived CPascalStr types. Three
	// of each so that literal C Strings can be conveniently used for one of the operators
	// as well as using any two classes derived from CPascalStr.
	friend CStr255_AC operator+(const CPascalStr_AC& s1, const CPascalStr_AC& s2);
	friend CStr255_AC operator+(const CPascalStr_AC& s1, const char* s2);
	friend CStr255_AC operator+(const CPascalStr_AC& s1, unsigned char ch);
	friend CStr255_AC operator+(const char* s1, const CPascalStr_AC& s2);
	friend CStr255_AC operator+(unsigned char ch, const CPascalStr_AC& s2);

	// Methods that mimic the Pascal builtin CPascalStr functions for Pos, Insert and Delete.
	// Note that insert and copy is implemented in the derived classes.
	unsigned char Pos(const char* subStr, unsigned char startPos = 1) const;
	unsigned char Pos(const CPascalStr_AC& subStr, unsigned char startPos = 1) const;
	unsigned char Pos(char c, unsigned char startPos = 1) const;
	void Delete(unsigned char pos, unsigned char length);

	void Insert(const char* insStr, unsigned char insStrLength, unsigned char pos, unsigned char maxLength);
	void Insert(const char* insStr, unsigned char pos, unsigned char maxLength);
	void Insert(const char* insStr, unsigned char pos);
	void Insert(const CPascalStr_AC& insStr, unsigned char pos, unsigned char maxLength);
	void Insert(const CPascalStr_AC& insStr, unsigned char pos);

	
//----------------------------------------------------------------------------------------
// static member functions
//----------------------------------------------------------------------------------------
	static Handle SetItl2Handle(Handle itsItl2Handle);

//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
	static Handle fgItl2Handle;			// Handle to the international rsrc to be used
										// in string compares. Normally nil, MacApp depends on
										// The system value for proper operation of it's internal
										// Strings, However you may replace and restore this
										// during certain of your own operations.
};

#if 1
//----------------------------------------------------------------------------------------
// CStr255_AC:
//----------------------------------------------------------------------------------------
class EXPORT_AC CStr255_AC : public CPascalStr_AC
{
public:
	enum { kMaxLength = kStr255Len };
	Str255 fStr;

public:
	inline CStr255_AC()
	{ *reinterpret_cast<unsigned char*>(this) = 0; }
	
	inline CStr255_AC(const CPascalStr_AC& str)
	{ CopyFrom(str); }
	// Generic string constructor
	
	inline CStr255_AC(const CStr255_AC& str)
	{ CopyFrom(str); }
	// copy constructor

	//	
	// Don't have any optimized constructors for string smaller than me.
	// Because all other strings are smaller than me, so we don't need any.
	//

	inline CStr255_AC(const unsigned char* str)
	{ CopyFrom(str); }
	
	inline CStr255_AC(const char* str)
	{ CopyFrom(str); }

	inline CStr255_AC(const char* str, unsigned char maxLength)
	{ CopyFrom(str, maxLength); }

	inline CStr255_AC(const void* str, unsigned char forLength)
	{ CopyFrom(str, forLength); }

	inline CStr255_AC(unsigned long id)
	{ CopyFrom(id); }

	inline CStr255_AC(unsigned short num)
	{ CopyFrom(num); }

	inline CStr255_AC(unsigned char ch)
	{ CopyFrom(ch); }
	
	inline CStr255_AC(short strListID, short index)
	{ CopyFrom(strListID, index, kMaxLength); }

	// Returns a copy of the string on the stack. (as a null terminated c style string)
	inline operator const char*() const
	{ return CChar255_AC(*this); }

	// Copy and Insert roughly equivalent to the Pascal Insert and Copy functions.
	
	inline CStr255_AC& Copy(unsigned char pos, unsigned char length) const
	{ return CStr255_AC(reinterpret_cast<const char*>(&(*this)[static_cast<int>(pos)]), GetMaxLength(pos, length)); }

	inline void Insert(const CPascalStr_AC& str, unsigned char pos)
	{ CPascalStr_AC::Insert(str, pos); }

	inline void Insert(const char* str, unsigned char pos)
	{ CPascalStr_AC::Insert(str, pos); }
					   
	// Concatenation operator
	
	inline CStr255_AC& operator +=(const CPascalStr_AC& str)
	{ Insert(str, Length() + 1); return *this; }

	inline CStr255_AC& operator +=(const char* str)
	{ Insert(str, Length() + 1); return *this; }

	inline CStr255_AC& operator +=(unsigned char ch)
	{ CPascalStr_AC::Insert(reinterpret_cast<const char*>(&ch), 1, Length() + 1, kMaxLength); return *this; }

	// Assignment operators
	inline CStr255_AC& operator=(const CStr255_AC& str)
	{ CopyFrom(str); return *this; }

	inline CStr255_AC& operator=(const CPascalStr_AC& str)
	{ CopyFrom(str); return *this; }

	inline CStr255_AC& operator=(const unsigned char* str)
	{ CopyFrom(str); return *this; }

	inline CStr255_AC& operator=(const char* str)
	{ CopyFrom(str); return *this; }

	inline CStr255_AC& operator=(unsigned char ch)
	{ CopyFrom(ch); return *this; }
};
#endif

//----------------------------------------------------------------------------------------
// TemplatePascalStr:
//----------------------------------------------------------------------------------------
template <class T>
class TemplatePascalStr : public CPascalStr_AC
{
public:
	enum { kMaxLength = sizeof(T) - 1 };
	T fStr;

public:
	// Constructors

	inline TemplatePascalStr()
	{ *reinterpret_cast<unsigned char*>(this) = 0; }

	inline TemplatePascalStr(const CPascalStr_AC& str)
	{ CopyFrom(str, kMaxLength); }
	// Generic string constructor

	inline TemplatePascalStr(const TemplatePascalStr& str)
	{ CopyFrom(str); }
	// copy constructor
	
	inline TemplatePascalStr(const unsigned char* str)
	{ CopyFrom(str, kMaxLength); }
	
	inline TemplatePascalStr(const char* str)
	{ CopyFrom(str, kMaxLength); }

	inline TemplatePascalStr(const char* str, unsigned char maxLength)
	{ CopyFrom(str, maxLength); }

	inline TemplatePascalStr(const void* str, unsigned char forLength)
	{ CopyFrom(str, forLength); }

	inline TemplatePascalStr(unsigned long id)
	{ CopyFrom(id); }

	inline TemplatePascalStr(unsigned short num)
	{ CopyFrom(num); }

	inline TemplatePascalStr(unsigned char ch)
	{ CopyFrom(ch); }
	
	inline TemplatePascalStr(short strListID, short index)
	{ CopyFrom(strListID, index, kMaxLength); }

	// Returns a copy of the string on the stack. (as a null terminated c style string)
	inline operator const char*() const
	{ return CChar255_AC(*this); }

	// Copy and Insert roughly equivalent to the Pascal Insert and Copy functions.
#if 0
	inline TemplatePascalStr Copy(unsigned char pos, unsigned char length) const
	{ return TemplatePascalStr(static_cast<const char*>(&(*this)[pos]), GetMaxLength(pos, length)); }
#endif

	inline void Insert(const CPascalStr_AC& str, unsigned char pos)
	{ CPascalStr_AC::Insert(str, pos, kMaxLength); }

	inline void Insert(const char* str, unsigned char pos)
	{ CPascalStr_AC::Insert(str, pos, kMaxLength); }

	// Concatenation operator
	
	inline TemplatePascalStr& operator +=(const CPascalStr_AC& str)
	{ Insert(str, Length() + 1); return *this; }

	inline TemplatePascalStr& operator +=(const char* str)
	{ Insert(str, Length() + 1); return *this; }

	inline TemplatePascalStr& operator +=(unsigned char ch)
	{ CPascalStr_AC::Insert(reinterpret_cast<char*>(&ch), 1, Length() + 1, kMaxLength); return *this; }

	// Assignment operators

	inline TemplatePascalStr& operator=(const TemplatePascalStr& str)
	{ CopyFrom(str); return *this; }

	inline TemplatePascalStr& operator=(const CPascalStr_AC& str)
	{ CopyFrom(str, kMaxLength); return *this; }

	inline TemplatePascalStr& operator=(const unsigned char* str)
	{ CopyFrom(str, kMaxLength); return *this; }

	inline TemplatePascalStr& operator=(const char* str)
	{ CopyFrom(str, kMaxLength); return *this; }

	inline TemplatePascalStr& operator=(unsigned char ch)
	{ CopyFrom(ch); return *this; }
};

// typedef 'em so that existing code still works
//typedef TemplateCharStr<Str255> CChar255_AC; // Actually we use a CChar255_AC that has optimizations instead.
typedef TemplateCharStr<Str63> CChar63_AC;
typedef TemplateCharStr<Str32> CChar32_AC;
typedef TemplateCharStr<Str31> CChar31_AC;
typedef TemplateCharStr<Str27> CChar27_AC;
typedef TemplateCharStr<Str15> CChar15_AC;
typedef TemplateCharStr<Str8> CChar8_AC;
typedef TemplateCharStr<Str2> CChar2_AC;

//typedef TemplatePascalStr<Str255> CStr255_AC; // Actually we use a CStr255_AC that has optimizations instead.
typedef TemplatePascalStr<Str63> CStr63_AC;
typedef TemplatePascalStr<Str32> CStr32_AC;
typedef TemplatePascalStr<Str31> CStr31_AC;
typedef TemplatePascalStr<Str27> CStr27_AC;
typedef TemplatePascalStr<Str15> CStr15_AC;
typedef TemplatePascalStr<Str8> CStr8_AC;
typedef TemplatePascalStr<Str2> CStr2_AC;

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

extern const CStr2_AC kEmptyString_AC;

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#endif
