//----------------------------------------------------------------------------------------
// CRefCountingPtr_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __CRefCountingPtr_AC__
#define __CRefCountingPtr_AC__

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __MRefCountable_AC__
#include "MRefCountable_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CRefCountingPtr_AC
//----------------------------------------------------------------------------------------

template <class T>
class CRefCountingPtr_AC  
{
public:	
	CRefCountingPtr_AC(T* ptr);

	CRefCountingPtr_AC(const CRefCountingPtr_AC<T> &toCopy);

	CRefCountingPtr_AC();

	~CRefCountingPtr_AC();
	
	T* get() const;
	
	T* operator ->() const;
		
	operator T* () const;
	
	T & operator *() const;
	
	void operator = (T* ptr);
	
	CRefCountingPtr_AC<T> &operator = ( const CRefCountingPtr_AC<T> &ptr );
	
	bool operator == (const CRefCountingPtr_AC<T> &p) const;
	
	bool operator != (const CRefCountingPtr_AC<T> &p) const;
		
	bool operator == (const T* p) const;
	
	bool operator != (const T* p) const;

	T* release();
	
	void reset();

	bool IsNULL() const;

private:

#if qCoreDebug	
	bool operator == (const void* ) const; // Use IsNULL
	bool operator != (const void* ) const; // Use IsNULL
	bool operator == (int ) const; // Use IsNULL
	bool operator != (int ) const; // Use IsNULL
#endif

private:
	T* fPointee;
};

//----------------------------------------------------------------------------------------
// Inline Implementation 
//----------------------------------------------------------------------------------------

template <class T>
inline	CRefCountingPtr_AC<T>::CRefCountingPtr_AC(T* ptr) 
: 	fPointee(ptr)			
{ 
	if(fPointee) fPointee->aquire();
}
template <class T>
inline	CRefCountingPtr_AC<T>::CRefCountingPtr_AC(const CRefCountingPtr_AC<T> &toCopy) 
:	fPointee(toCopy.fPointee)	
{ 
	if(fPointee) fPointee->aquire();
}
template <class T>
inline	CRefCountingPtr_AC<T>::CRefCountingPtr_AC() 
: 	fPointee(NULL) 
{
}
template <class T>
inline	CRefCountingPtr_AC<T>::~CRefCountingPtr_AC() 
{ 
	reset();
}
template <class T>
inline	T* CRefCountingPtr_AC<T>::get() const 
{ 
	return fPointee; 
}
template <class T>
inline	T* CRefCountingPtr_AC<T>::operator ->() const
{ 
	::AssertNotNULL_AC(fPointee);
	return fPointee; 
}
template <class T>
inline	CRefCountingPtr_AC<T>::operator T* () const
{ 
	return fPointee; 
}
		
template <class T>
inline	void CRefCountingPtr_AC<T>::operator = ( T* ptr) 
{ 
	reset();
	fPointee = ptr; 
	if(fPointee) fPointee->aquire();
}

template <class T>
inline	T & CRefCountingPtr_AC<T>::operator *() const
{
	::AssertNotNULL_AC(fPointee);
	return *fPointee; 
} 
template <class T>
inline	CRefCountingPtr_AC<T> &CRefCountingPtr_AC<T>::operator = ( const CRefCountingPtr_AC<T> &ptr )
{ 
	if(this != &ptr)
	{
		reset();
		fPointee = ptr.fPointee; 
		if(fPointee) fPointee->aquire();
	}
	return *this; 
}
template <class T>
inline	bool CRefCountingPtr_AC<T>::operator == (const CRefCountingPtr_AC<T> &p) const 
{	
	return fPointee == p.fPointee; 
}
template <class T>
inline	bool CRefCountingPtr_AC<T>::operator != (const CRefCountingPtr_AC<T> &p) const 
{	
	return fPointee != p.fPointee;	
}
template <class T>
inline T* CRefCountingPtr_AC<T>::release() 
{ 	
	T* old = fPointee;
	fPointee = NULL;
	if(old) old->release();
	return old;
}
template <class T>
inline void CRefCountingPtr_AC<T>::reset() 
{	
	ReleaseCountedObject_AC(fPointee);
}
template <class T>
inline bool CRefCountingPtr_AC<T>::operator == (const T* inPtr) const
{
	return fPointee == inPtr;
}
	
template <class T>
inline bool CRefCountingPtr_AC<T>::operator != (const T* inPtr) const
{
	return fPointee != inPtr;
}

template <class T>
inline bool CRefCountingPtr_AC<T>::IsNULL() const
{
	return fPointee == NULL;
}


#endif