//----------------------------------------------------------------------------------------
// CReporter_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CReporter_AC__
#define __CReporter_AC__

#ifndef __Core_AC__
#error "Include CoreAC.h to get CReporter_AC.h"
#endif

#ifndef __CoreSwitches_AC__
#include "CoreSwitches_AC.h"
#endif
#ifndef __ConditionalMacros_AC__
#include "ConditionalMacros_AC.h"
#endif
#ifndef __Types_AC__
#include "Types_AC.h"
#endif
#ifndef __CException_AC__
#include "CException_AC.h"
#endif
#ifndef __CAssertionFailure_AC__
#include "CAssertionFailure_AC.h"
#endif


template <class T> class CLinkedList_AC;

class CStr255_AC;

//----------------------------------------------------------------------------------------
// class CReporter_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CReporter_AC
{
public:
	CReporter_AC(bool makeCurrentInstance = true);
	virtual ~CReporter_AC();
	
	virtual void OutputMessage(	const char* inStr, 	
								bool continueIfPossible = true);

	CReporter_AC* SetIsCurrentInstance(bool is = true);
		// Returns previous when is = true, and new current when is == false
		// calling at static construction/destruction time is
		// supported and will work independently of static construction
		// order.
	
	static CReporter_AC* Instance();
		// This returns the current instance, or if there ain't one,
		// returns a pointer to the default CReporter_AC which just 
		// outputs to dout. This will never return an invalid
		// pointer. Calling Instance at static contruction time is
		// not supported.

	static CReporter_AC* GetDefaultInstance();
		// returns pointer to the default CReporter_AC, which just calls
		// dout.

protected:	

	typedef CLinkedList_AC<CReporter_AC*> 	CList;
	typedef CList*							CListPtr;

	static CListPtr fgReporters;
};

// MacApp Compatible Wrappers 
void ProgramBreak_AC(const CStr255_AC& grievance);
void ProgramBreak_AC(const char* grievance, ...);
	// this accepts printf style arguments.
	
void ProgramReport_AC(const CStr255_AC& grievance, bool stopInDebugger = false);
void ProgramReport_AC(const char* grievance, bool stopInDebugger = false );

void ProgramReport_AC(bool stopInDebugger, const char* grievance, ...);
	// this accepts printf style arguments. Note: changed order of params
	// to make more sense for formatting.


#endif
