//----------------------------------------------------------------------------------------
// MemoryUtils_AC.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __MemoryUtils_AC__
#define __MemoryUtils_AC__

// acs core

#ifndef __Core_AC__
#error "Include CoreAC.h to get MemoryUtils_AC.h"
#endif

#ifndef __CoreSwitches_AC__
#include "CoreSwitches_AC.h"
#endif

#ifndef __ConditionalMacros_AC__
#include "ConditionalMacros_AC.h"
#endif

#ifndef __Types_AC__
#include "Types_AC.h"
#endif

// Toolbox

#if qPowerPC 
// Include Memory.h so that BlockMove_AC and BlockMoveOverlap_AC can be #defined using BlockMoveData
// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif
#endif

#ifndef __LIMITS__
#include <Limits.h>
#endif

#ifndef __cstring__
#include <string.h>
#endif

//----------------------------------------------------------------------------------------
// BlockMove_AC: BlockMoveData() is fastest on PowerPC, memcpy() on 68K
//----------------------------------------------------------------------------------------

#if qPowerPC
	#define BlockMove_AC(srcPtr, destPtr, byteCount) \
		BlockMoveData(srcPtr, destPtr, Size(byteCount))
#elif (UINT_MAX >= ULONG_MAX)
	#define BlockMove_AC(srcPtr, destPtr, byteCount) \
		memcpy(destPtr, srcPtr, size_t(byteCount))
#elif (UINT_MAX < ULONG_MAX)
	// size_t is (usually) defined as an unsigned int. If we are compiled with 2-byte ints,
	// it will be smaller than a long. We want to use memcpy because it is faster,
	// but it only takes a size_t for the byte count. So we use memcpy for short
	// moves, and BlockMove for the long ones. 
	EXPORT_AC void BlockMove_AC(const void* srcPtr, void* destPtr, long byteCount);
#endif

//----------------------------------------------------------------------------------------
// BlockMoveOverlap_AC: BlockMoveData() is fastest on PowerPC, memmove() on 68K
//----------------------------------------------------------------------------------------

#if qPowerPC
	#define BlockMoveOverlap_AC(srcPtr, destPtr, byteCount) \
		BlockMoveData(srcPtr, destPtr, Size(byteCount))
#elif (UINT_MAX >= ULONG_MAX)
	// moves, and BlockMove for the long ones. 
	#define BlockMoveOverlap_AC(srcPtr, destPtr, byteCount) \
		memmove(destPtr, srcPtr, size_t(byteCount));
#elif (UINT_MAX < ULONG_MAX)
	// size_t is (usually) defined as an unsigned int. If we are compiled with 2-byte ints,
	// it will be smaller than a long. We want to use memcpy because it is faster,
	// but it only takes a size_t for the byte count. So we use memcpy for short
	// moves, and BlockMove for the long ones. 
	EXPORT_AC void BlockMoveOverlap_AC(const void* srcPtr, void* destPtr, long byteCount);
#endif

//----------------------------------------------------------------------------------------
// BlockSet_AC
//----------------------------------------------------------------------------------------

EXPORT_AC void BlockSet_AC(void* destPtr,long byteCount,unsigned char setVal);

//----------------------------------------------------------------------------------------
// StripLong_AC
//----------------------------------------------------------------------------------------
#if q68K && !qNeedsSystem7_6
EXPORT_AC long StripLong_AC(void* address);
#else
inline long StripLong_AC(void* address) { return reinterpret_cast<long>(address); }
#endif

//----------------------------------------------------------------------------------------
// CopyPString_AC:
//----------------------------------------------------------------------------------------

inline unsigned char* CopyPString_AC(unsigned char* out, const unsigned char* in)
{
	::BlockMove_AC(in, out, *in + 1);
	return out;
}

//----------------------------------------------------------------------------------------
// Erase_AC:
//----------------------------------------------------------------------------------------

template <class T>
inline T* Erase_AC(T* inData, unsigned long numItems = 1)
{
	if(inData != NULL)
		::BlockSet_AC(static_cast_AC(void*, inData), sizeof(T) * numItems, 0);
	return inData;
}

//----------------------------------------------------------------------------------------
// Copy_AC:
//----------------------------------------------------------------------------------------

template <class T>
inline T* Copy_AC(T* out, const T*in)
{
	::BlockMove_AC(in, out, sizeof(T));
	return out;
}

//----------------------------------------------------------------------------------------
// DeleteIfObject_AC:
//----------------------------------------------------------------------------------------

template <class T>
inline void DeleteIfObject_AC(T*& inType)
{
	if(inType)
	{
		T* temp = inType;
		inType = NULL;
		delete temp;
	}
}

//----------------------------------------------------------------------------------------
// DeleteIfArray_AC:
//----------------------------------------------------------------------------------------

template <class T>
inline void DeleteIfArray_AC(T*& inArray)
{
	if(inArray)
	{
		T* temp = inArray;
		inArray = NULL;
		delete[] temp;
	}
}

// Look in CHandle_AC.h for equivelent disposeif... for handles.

#endif
