//----------------------------------------------------------------------------------------
// Utils_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Utils_AC__
#define __Utils_AC__

#ifndef __Core_AC__
#error "Include CoreAC.h to get Utils_AC.h"
#endif

#ifndef __CoreSwitches_AC__
#include "CoreSwitches_AC.h"
#endif

#ifndef __ConditionalMacros_AC__
#include "ConditionalMacros_AC.h"
#endif

#ifndef __Types_AC__
#include "Types_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

#ifndef __CAssertionFailure_AC__
#include "CAssertionFailure_AC.h"
#endif

#ifndef __typeinfo__
#include <typeinfo.h>
#endif


#if 1 // we won't need these but some folks may...
#	define static_cast_AC(TYPE, EXPR)			static_cast<TYPE>(EXPR)
#	define const_cast_AC(TYPE, EXPR)			const_cast<TYPE>(EXPR)
#	define reinterpret_cast_AC(TYPE, EXPR)		reinterpret_cast<TYPE>(EXPR)
#endif
// min/max 

#if qCoreDebug


#define dynamic_cast_AC(TYPE, EXPR)	DYNAMIC_CAST_AC(static_cast<TYPE>(NULL), EXPR, #TYPE, __FILE__, __LINE__)

template <class TO, class FROM>
inline TO* DYNAMIC_CAST_AC(TO* /*ignored param*/, FROM* inFrom, const char* toClassName,  const char*fileName, short lineNum)
{
	// the parameter is ignored because all it is doing is telling the compiler what
	// type to cast to when expanding the template.
	::AssertNotNULL_AC(inFrom);

	TO* outPtr = dynamic_cast<TO*>(inFrom);

	if(!outPtr)
		::DebugThrowBadCast_AC(toClassName, typeid(*inFrom).name(), fileName, lineNum);
		
	return outPtr;
}

#else

#define dynamic_cast_AC(TYPE, EXPR)	DYNAMIC_CAST_AC(static_cast<TYPE>(NULL), EXPR)

template <class TO, class FROM>
inline TO* DYNAMIC_CAST_AC(TO* /*ignored param*/, FROM* inFrom)
{
	TO* outPtr = dynamic_cast<TO*>(inFrom);
	
	if(!outPtr)
		throw bad_cast("bad_cast");
	
	return outPtr;
}

#endif

inline short Min_AC(short a, short b) 
{ 
	return a < b ? a : b; 
}
inline short Max_AC(short a, short b) 
{ 
	return a > b ? a : b; 
}
inline short MinMax_AC(short minVal, short expression, short maxVal) 
{ 
	return Min_AC(Max_AC(expression, minVal), maxVal);
}
inline unsigned short Min_AC(unsigned short a, unsigned short b) 
{ 
	return a < b ? a : b;
}
inline unsigned short Max_AC(unsigned short a, unsigned short b) 
{ 
	return a > b ? a : b; 
}
inline unsigned short MinMax_AC(unsigned short minVal, unsigned short expression, unsigned short maxVal) 
{ 
	return Min_AC(Max_AC(expression, minVal), maxVal); 
}
inline long Min_AC(long a, long b) 
{ 
	return a < b ? a : b; 
}
inline long Max_AC(long a, long b) 
{ 	
	return a > b ? a : b; 
}
inline long MinMax_AC(long minVal, long expression, long maxVal) 
{ 
	return Min_AC(Max_AC(expression, minVal), maxVal); 
}
inline unsigned long Min_AC(unsigned long a, unsigned long b) 
{ 
	return a < b ? a : b; 
}
inline unsigned long Max_AC(unsigned long a, unsigned long b) 
{ 
	return a > b ? a : b; 
}
inline unsigned long MinMax_AC(unsigned long minVal, unsigned long expression, unsigned long maxVal) 
{ 
	return Min_AC(Max_AC(expression, minVal), maxVal); 
}
inline double Min_AC(double a, double b) 
{ 	
	return a < b ? a : b; 
}
inline double Max_AC(double a, double b) 
{ 
	return a > b ? a : b; 
}
inline double MinMax_AC(double minVal, double expression, double maxVal) 
{ 	
	return Min_AC(Max_AC(expression, minVal), maxVal); 
}

// bit flippers

template <class T, class M>
inline void EnableBits_AC(T& flags, M mask)
{	
	flags |= mask; 
}
	
template <class T, class M>
inline void DisableBits_AC(T& flags, M mask)
{	
	flags &= ~mask; 
}
	
template <class T, class M>
inline void SetBits_AC(T& flags, M mask, bool toTrue)
{	
	if(toTrue) EnableBits_AC(flags, mask);
	else DisableBits_AC(flags, mask);
}
	
template <class T, class M>
inline bool TestBits_AC(T flags, M mask)
{
	return (flags & mask) == mask;
}


template <class T>
inline bool Odd_AC(const T& value)
{ 
	return ((int)(value) & 1); 
}


#endif
