//----------------------------------------------------------------------------------------
// Assertions_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Core_AC__
#include "Core_AC.h"
#endif


// ACS

#if qAssertions

#ifndef __dout_AC__
#include "dout_AC.h"
#endif

#ifndef __stdio__
#include <stdio.h>
#endif

#ifndef __string__
#include <string.h>
#endif

#ifndef __errors__
#include <errors.h>
#endif

//----------------------------------------------------------------------------------------
// DebugAssertNotNULL_AC
//----------------------------------------------------------------------------------------

void DebugAssertNotNULL_AC( const void* inPtr, char* inFile, short inLine)
{ 	
	static bool alreadyFailed = false;
	static char str[256];
				
				
	if(inPtr == NULL)
	{
		try
		{
			if(alreadyFailed)
			{
				dout 	<< "Warning: Encountered Nested Failures of AssertNotNULL_AC" 
						<< endl_AC;
			}			

			alreadyFailed = true;
			
			CAssertionFailure_AC ex = memFullErr;
			ex.SetThrownFrom(inFile, inLine);
			ex.SetAssertionMsg("AssertNotNULL Failed");
			
			if(!alreadyFailed)
			{
				::ProgramReport_AC(true, "Assertion failed: [%s]", ex.GetAssertionMsg());
			}
			
			DoThrowMessage_AC(ex);
			throw ex;
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			alreadyFailed = false;
			throw;
		}
	}
}

//----------------------------------------------------------------------------------------
// CAssertionFailure_AC
//----------------------------------------------------------------------------------------

void CAssertionFailure_AC::SetAssertionMsg(const char* inString)
{	
	if(inString)
		::strncpy(fMsgString, inString, sizeof(fMsgString)); 
	else
		::strcpy(fMsgString, "Unknown");
}

//----------------------------------------------------------------------------------------
// DebugAssert_AC
//----------------------------------------------------------------------------------------

void DebugAssert_AC( bool condition, const char* msg, char* inFile, short inLine  )
{
	if(condition == false)
	{
		CAssertionFailure_AC ex;
		ex.SetThrownFrom(inFile, inLine);
		ex.SetAssertionMsg(msg);
		
		::ProgramReport_AC(true, "Assertion failed: [%s]", ex.GetAssertionMsg());

		DoThrowMessage_AC(ex);
		throw ex;
	}
}

//----------------------------------------------------------------------------------------
// DebugWarning_AC
//----------------------------------------------------------------------------------------

void DebugWarning_AC( bool condition, char* optMsg, char* inFile, short inLine)
{
	if(condition != true)
	{
		const char* message = optMsg ? optMsg : "";
	
		::ProgramReport_AC( 
				true, 
				"Warning failed: [%s], File:%s, Line:%d",
				message,
				inFile,
				inLine);
	}
}

//----------------------------------------------------------------------------------------
// DebugAssertGoodCast_AC
//----------------------------------------------------------------------------------------

void DebugAssertGoodCast_AC( const void* inPtr, char* inFile, short inLine)
{
	if(inPtr == NULL)
	{
		ProgramBreak_AC("Assertion Failed: BadCast. File: %s, Line: %d",  inFile, inLine);
		throw bad_cast("bad_cast");
	}
}


#else	// qAssertions

	// inlined in header file

#endif	// qAssertions



