//----------------------------------------------------------------------------------------
// CHandle_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

// Toolbox

#ifndef __RESOURCES__
#include <Resources.h>
#endif


//----------------------------------------------------------------------------------------
// Static globals:
//----------------------------------------------------------------------------------------

static CPermHandleHandler_AC pDefaultPermHandleHandler;
static CAutoPtr_AC<CPermHandleHandler_AC> pPermHandleHandler;

//----------------------------------------------------------------------------------------
// CPermHandleHandler_AC constructor: 
//----------------------------------------------------------------------------------------

CPermHandleHandler_AC::CPermHandleHandler_AC()
{
}

//----------------------------------------------------------------------------------------
// CPermHandleHandler_AC destructor: 
//----------------------------------------------------------------------------------------

CPermHandleHandler_AC::~CPermHandleHandler_AC()
{
}

//----------------------------------------------------------------------------------------
// CPermHandleHandler_AC::DoNewHandle: 
//----------------------------------------------------------------------------------------
	
Handle CPermHandleHandler_AC::DoNewHandle(Size size)
{
	Handle returnHandle = ::NewHandle(size);
	::ThrowIfNULL_AC(returnHandle);
	return returnHandle;
}

//----------------------------------------------------------------------------------------
// CPermHandleHandler_AC::DoSetHandleSize: 
//----------------------------------------------------------------------------------------

void CPermHandleHandler_AC::DoSetHandleSize(Handle h, Size newSize)
{
	::SetHandleSize(h, newSize);
	::ThrowIfMemError_AC();
}

//----------------------------------------------------------------------------------------
// SetPermHandleHandler_AC
//----------------------------------------------------------------------------------------

EXPORT_AC void SetPermHandleHandler_AC(CPermHandleHandler_AC* newHandler)
{
	pPermHandleHandler = newHandler;
}

//----------------------------------------------------------------------------------------
// DisposeIfHandle_AC
//----------------------------------------------------------------------------------------

Handle DisposeIfHandle_AC(Handle aHandle)
{
	if (aHandle)
	{
#if !qCoreDebug
		::DisposeHandle(aHandle);
#else
		if (IsHandle_AC(aHandle))				// Test handlehood 
		{
			HGetState(aHandle);				// force subsequent MemError, if any
			if (IsHandlePurged_AC(aHandle))	// h might have been purged 
				DisposeHandle(aHandle);
			else if (MemError() != noErr)
			{
				::ProgramBreak_AC("Handle was so bad I couldn't even get the handle bits!\nBad Handle: %p\n", aHandle);
			}
			else if (IsAResource_AC(aHandle))
			{
				::ProgramBreak_AC("Trying to dispose a resource handle\nBad Handle: %p\n", aHandle);
			}
			else
			{
				// Set the handle contents to a real nice value for any dangling pointerciples 
				BlockSet_AC((*aHandle), GetHandleSize(aHandle), kDisposedInitVal_AC);
				DisposeHandle(aHandle);
			}
		}
		else
		{
			VerboseIsHandle_AC(aHandle);		// Get the diagnosis printed 
			::ProgramBreak_AC("Trying to dispose an invalid handle\nBad Handle: %p\n", aHandle);
		}
#endif
	}

	return NULL;
}

//----------------------------------------------------------------------------------------
// DisposeIfPtr_AC 
//----------------------------------------------------------------------------------------

Ptr DisposeIfPtr_AC(Ptr aPtr)
{
	if (aPtr)
	{
#if !qCoreDebug
		::DisposePtr(aPtr);
#else
		
		// Test pointerhood
		if (!odd_AC(aPtr))
		{
			BlockSet_AC(aPtr, GetPtrSize(aPtr), kDisposedInitVal_AC);
			DisposePtr(aPtr);
		}
		else
		{
			::ProgramBreak_AC("Trying to dispose an invalid pointer\nBad Pointer: %p\n", aPtr);
		}
#endif
	}

	return NULL;
}

//----------------------------------------------------------------------------------------
// CopyHandle_AC
//----------------------------------------------------------------------------------------

OSErr CopyHandle_AC(const Handle inHandle, Handle& outHandle)
{
	outHandle = NULL;
	Handle temp = inHandle;

	OSErr returnVal = HandToHand(&temp);
	if (returnVal == noErr)
		outHandle = temp;
		
	return returnVal;
}

//----------------------------------------------------------------------------------------
// LockHandle_AC
//----------------------------------------------------------------------------------------

SignedByte LockHandle_AC(Handle h, EMoveHiState_AC high)
{
	const short lockBit = 7;

	SignedByte savedState = 0;
	if (h)
	{
#if qCoreDebug
		if (!IsHandle_AC(h))
		{
			VerboseIsHandle_AC(h);					// Get the diagnosis printed 
			::ProgramBreak_AC("In LockHandle: not handed a handle.");
		}
		else
#endif
		{
			savedState = HGetState(h);
			
			// skip if already locked
			if (!(savedState & (0x1 << lockBit)))
				if (high == eMoveHi)
					::HLockHi(h);
				else
					::HLock(h);
		}
	}

	return savedState;
}

//----------------------------------------------------------------------------------------
// IsFreeHandle_AC: Walk the free-list looking for the given handle
//----------------------------------------------------------------------------------------
#if qCoreDebug

static bool IsFreeHandle_AC(Handle aHandle)
{
	bool returnVal = false;

	THz applZone = ApplicationZone();
	Handle currHandle = reinterpret_cast<Handle>(applZone->hFstFree);

	while (currHandle)
	{
		if (currHandle == aHandle)
		{
			returnVal = true;
			break;
		}
		currHandle = reinterpret_cast<Handle>(*currHandle);
	}

	return returnVal;
}
#endif

//----------------------------------------------------------------------------------------
// IsAResource_AC: 
//----------------------------------------------------------------------------------------

bool IsAResource_AC(Handle h)
{
	static const short kResNotFound = -1;

	return HomeResFile(h) != kResNotFound;
}

//----------------------------------------------------------------------------------------
// IsHandle_AC:	Returns true if handle appears valid. Must run on any processor since
//				it is called (indirectly) from MacAppAlert.
//----------------------------------------------------------------------------------------
#if qCoreDebug

bool IsHandle_AC(Handle h)
{
	// Test handle NULLness, Test handle Oddness, Test master pointer Oddness

	return h && !odd_AC(h) && !odd_AC(*h);
}
#endif

//----------------------------------------------------------------------------------------
// IsHandleLocked_AC: Returns lockState of h.
//----------------------------------------------------------------------------------------

bool IsHandleLocked_AC(Handle h)
{
	static const short lockBit = 7;

	SignedByte handleBits = ::HGetState(h);		// h might have been purged 

	return MemError() == noErr ? handleBits & (0x1 << lockBit) : false;
}

//----------------------------------------------------------------------------------------
// IsHandlePurged_AC: Returns purgeState of h.
//----------------------------------------------------------------------------------------
#if qCoreDebug

bool IsHandlePurged_AC(Handle h)
{
	bool returnVal;

	if (!IsHandle_AC(h))
	{
		VerboseIsHandle_AC(h);						// Get the diagnosis printed 
		::ProgramBreak_AC("IsHandlePurged was not handed a handle, pretty handy, eh?");

		returnVal = true;
	}
	else
		returnVal = *h == NULL;
	
	return returnVal;
}

#endif

#if qCoreDebug
//----------------------------------------------------------------------------------------
// VerboseIsHandle_AC
//----------------------------------------------------------------------------------------

bool VerboseIsHandle_AC(Handle h)
{
	static const long kUnInitStorage1 = 0x72677267;		// Pascal provided uninited storage 
	static const long kUnInitStorage2 = 0x67726772;		// odd byte boundary of above 
	static const long kDebugHandleInit = 0xF3F3F3F3;	// Handles are inited to this in MacApp
	static const long kDebugPtrInit = 0xF5F5F5F5;		// Pointers are inited to this in MacApp
	static const long kDebugObjInit = 0xF1F1F1F1;		// Objects are inited to this in MacApp

	if (::odd_AC(h))
	{
		if (static_cast<long>(h) == kUnInitStorage1)
			ProgramReport_AC("  That handle appears to be from uninitialized storage.\n");
		else if (static_cast<long>(h) == kDebugHandleInit)
			ProgramReport_AC("  That handle appears to be from a handle initialized by debugging.\n");
		else if (static_cast<long>(h) == kDebugPtrInit)
			ProgramReport_AC("  That handle appears to be from a pointer initialized by debugging.\n");
		else if (static_cast<long>(h) == kDebugObjInit)
			ProgramReport_AC("  That handle appears to be an uninitialized instance variable.\n");
		else
			ProgramReport_AC("  That handle is odd.\n");
	}
	else if (static_cast<long>(h) == kUnInitStorage2)
		ProgramReport_AC("  That handle appears to be from uninitialized storage.\n");
	else if (!h)
		ProgramReport_AC("  That handle is nil.\n");
	else
	{
		Ptr masterPointer = (*h);
		if (::odd_AC(masterPointer))
			ProgramReport_AC("  The master pointer is odd.\n");
		else if (IsFreeHandle_AC(h))
			ProgramReport_AC("  The handle has been freed.\n");
		else
			return true;
	}
	return false;
}
#endif

//========================================================================================
// CLASS CTempHLock_AC
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// CTempHLock_AC constructor
//----------------------------------------------------------------------------------------

CTempHLock_AC::CTempHLock_AC(Handle inData) :
pointee(inData),
fHandleBits(0),
fRelease(eDontReleaseIt)
{
	if (pointee)
		fHandleBits = LockHandle_AC(pointee);
}

//----------------------------------------------------------------------------------------
// CTempHLock_AC constructor 
//----------------------------------------------------------------------------------------

CTempHLock_AC::CTempHLock_AC(Handle inData, EMoveHiState_AC high, EReleaseState_AC releaseIt, ELockState_AC lockIt) :
pointee(inData),
fHandleBits(0),
fRelease(releaseIt == eReleaseIt)
{
	if (pointee)
	{
		if (lockIt == eLockIt)
		{
			fHandleBits = ::LockHandle_AC(pointee, high);
		}
		else if (pointee && high == eMoveHi)
		{
			fHandleBits = HGetState(pointee);
			MoveHHi(pointee);
		}
	}
}

//----------------------------------------------------------------------------------------
// CTempHandleLock::RestoreState: 
//----------------------------------------------------------------------------------------

void CTempHLock_AC::RestoreState()
{
	if (pointee)
	{
		::HSetState(pointee, fHandleBits);
		if (fRelease)
			::ReleaseResource(pointee);
	}
}

//----------------------------------------------------------------------------------------
// NewPermHandle_AC
//----------------------------------------------------------------------------------------

Handle NewPermHandle_AC(Size size)
{
	if (!pPermHandleHandler.IsNULL())
		return pPermHandleHandler->DoNewHandle(size);
	else
		return pDefaultPermHandleHandler.DoNewHandle(size);
}

//----------------------------------------------------------------------------------------
// SetPermHandleSize
//----------------------------------------------------------------------------------------

void SetPermHandleSize_AC(Handle h, Size newSize)
{
	if (!pPermHandleHandler.IsNULL())
		pPermHandleHandler->DoSetHandleSize(h, newSize);
	else
		pDefaultPermHandleHandler.DoSetHandleSize(h, newSize);
}

//----------------------------------------------------------------------------------------
// End of CHandle_AC.cp


