//----------------------------------------------------------------------------------------
// CPascalString_AC.cp 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

// ACS

#ifndef __Utils_AC__
#include "Utils_AC.h"
#endif

// ANSI

#ifndef __STRING__
#include <string.h>
#endif



//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
Handle CPascalStr_AC::fgItl2Handle;

//----------------------------------------------------------------------------------------
// Template Instantiations
//----------------------------------------------------------------------------------------

// Instantiate Template classes for all String types declared in <Types.h>
template class TemplatePascalStr<Str255>;
template class TemplatePascalStr<Str63>;
template class TemplatePascalStr<Str32>;
template class TemplatePascalStr<Str31>;
template class TemplatePascalStr<Str27>;
template class TemplatePascalStr<Str15>;

// Extras that also get commonly used
template class TemplatePascalStr<Str8>;
template class TemplatePascalStr<Str2>;

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const CStr2_AC kEmptyString_AC;

//========================================================================================
// CLASS CCharStr_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CCharStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CCharStr_AC::CopyFrom(const unsigned char* str, unsigned char maxLength)
{
	if (!str)
		*(char*)this = 0;
	else
	{
		size_t possibleLength = str[0];
		possibleLength = possibleLength < maxLength ? possibleLength : maxLength;

		BlockMove_AC(&str[1], this, possibleLength);
		reinterpret_cast<char*>(this)[possibleLength] = 0;
	}
}

//----------------------------------------------------------------------------------------
// CCharStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CCharStr_AC::CopyFrom(const char* str, size_t maxLength)
{
	if (!str)
		*(char*)this = 0;
	else
	{
		size_t possibleLength = strlen(str);
		possibleLength = possibleLength < maxLength ? possibleLength : maxLength;

		BlockMove_AC(str, this, possibleLength);
		reinterpret_cast<char*>(this)[possibleLength] = 0;
	}
}

//----------------------------------------------------------------------------------------
// CCharStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CCharStr_AC::CopyFrom(const void* str, size_t forLength)
{
	if (!str)
		*(char*)this = 0;
	else
	{
		//
		// NOTE: there is _NO_ maxlength protection here, be careful!
		// However, since Length() is an unsigned char
		// its maximum value is kStr255Len anyways.
		//
		BlockMove_AC(str, this, forLength);
		reinterpret_cast<char*>(this)[forLength] = 0;
	}
}

//----------------------------------------------------------------------------------------
// CCharStr_AC::Length
//----------------------------------------------------------------------------------------

size_t CCharStr_AC::Length() const
{ 
	return strlen(reinterpret_cast<const char*>(this)); 
}

//========================================================================================
// CLASS CPascalStr_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(const unsigned char* str, unsigned char maxLength)
{
	if (!str)
		Length() = 0;
	else
	{
		unsigned char possibleLength = str[0];
		possibleLength = possibleLength < maxLength ? possibleLength : maxLength;

		Length() = possibleLength;
		BlockMove_AC(&str[1], &(*this)[1], Length());
	}
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(const char* str, unsigned char maxLength)
{
	if (!str)
		Length() = 0;
	else
	{
		unsigned char possibleLength = static_cast<unsigned char>(strlen(str));
		possibleLength = possibleLength < maxLength ? possibleLength : maxLength;

		Length() = possibleLength;
		BlockMove_AC(str, &(*this)[1], Length());
	}	
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(const void* str, unsigned char forLength)
{
	//
	// NOTE: there is _NO_ maxlength protection here, be careful!
	// However, since Length() is an unsigned char
	// its maximum value is kStr255Len anyways.
	//
	if (!str)
		Length() = 0;
	else
	{
		Length() = forLength;
		BlockMove_AC(str, &(*this)[1], Length());
	}
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(const unsigned char* str)
{
	//
	// NOTE: there is _NO_ maxlength protection here, be careful!
	// However, since Length() is an unsigned char
	// its maximum value is kStr255Len anyways.
	//
	if (!str)
		Length() = 0;
	else
	{
		Length() = str[0];
		BlockMove_AC(&str[1], &(*this)[1], Length());
	}
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(const char* str)
{
	//
	// NOTE: there is _NO_ maxlength protection here, be careful!
	// However, since Length() is an unsigned char
	// its maximum value is kStr255Len anyways.
	//
	if (!str)
		Length() = 0;
	else
	{
		Length() = strlen(str);
		BlockMove_AC(str, &(*this)[1], Length());
	}
}

#if !(qPowerPC || qNeedsMC68020)
//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(unsigned long id)
{
	//
	// NOTE: there is _NO_ maxlength protection here, be careful!
	// However, since Length() is an unsigned char
	// its maximum value is kStr255Len anyways.
	//
	Length() = sizeof(id);
	BlockMove_AC(&id, &(*this)[1], Length());
}
#endif


#if !(qPowerPC || qNeedsMC68020)
//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(unsigned short num)
{
	//
	// NOTE: there is _NO_ maxlength protection here, be careful!
	// However, since Length() is an unsigned char
	// its maximum value is kStr255Len anyways.
	//
	Length() = sizeof(num);
	BlockMove_AC(&num, &(*this)[1], Length());
}
#endif

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(short strListID, short index, unsigned char maxLength)
{
	CStr255_AC toolBoxString;
	GetIndString(toolBoxString, strListID, index);

	CopyFrom(toolBoxString, maxLength);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyFrom
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyFrom(const Handle inHandle, unsigned char maxLength)
{
	if (inHandle)
	{
		Size handleSize = GetHandleSize(inHandle);
		Size sizeToCopy = handleSize > maxLength ? maxLength : handleSize;
		CopyFrom(reinterpret_cast<void*>(*inHandle), static_cast<unsigned char>(sizeToCopy));
	}
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyTo
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyTo(unsigned char* str) const
{
	if (str)
		BlockMove_AC(&(*this)[0], str, Length() + 1);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyTo
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyTo(char* str) const
{
	if (str)
	{
		BlockMove_AC(&(*this)[1], str, Length());
		str[Length()] = 0;	// Add null terminations
	}
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::CopyTo
//----------------------------------------------------------------------------------------

void CPascalStr_AC::CopyTo(void* str) const
{
	if (str)
		BlockMove_AC(&(*this)[1], str, Length());
}
//----------------------------------------------------------------------------------------
// CPascalStr_AC::GetMaxLength:
//----------------------------------------------------------------------------------------

unsigned char CPascalStr_AC::GetMaxLength(unsigned char pos, unsigned char length) const
{
	short testLength = Length() - pos + kLengthByte;

	testLength = (length > testLength) ? testLength : length;

	if (testLength < 0)
		testLength = 0;
		
	return static_cast<unsigned char>(testLength);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::ToULong
//----------------------------------------------------------------------------------------

unsigned long CPascalStr_AC::ToULong() const
{
	unsigned long	returnVal = 0;
	unsigned long	myLength = Length();
	if (myLength)
		BlockMove_AC(&(*this)[1], &returnVal, (sizeof(returnVal) < myLength) ? sizeof(returnVal) : myLength);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::ToULong
//----------------------------------------------------------------------------------------

unsigned short CPascalStr_AC::ToUShort() const
{
	unsigned short	returnVal = 0;
	unsigned long	myLength = Length();
	if (myLength)
		BlockMove_AC(&(*this)[1], &returnVal, (sizeof(returnVal) < myLength) ? sizeof(returnVal) : myLength);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// Definitions for relational string operators helpers
//----------------------------------------------------------------------------------------

short CPascalStr_AC::CPascalStrCompare(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
{
	return ::CompareText(&s1[1], &s2[1], s1.Length(), s2.Length(), CPascalStr_AC::fgItl2Handle);
}

short CPascalStr_AC::CPascalStrCompare(const CPascalStr_AC& s1, const char* s2)
{
	return ::CompareText(&s1[1], s2, s1.Length(), s2 ? strlen(s2) : 0, CPascalStr_AC::fgItl2Handle);
}

short CPascalStr_AC::CPascalStrCompare(const CPascalStr_AC& s1, unsigned char ch)
{
	return ::CompareText(&s1[1], &ch, s1.Length(), 1, CPascalStr_AC::fgItl2Handle);
}

short CPascalStr_AC::CPascalStrCompare(const char* s1, const CPascalStr_AC& s2)
{
	return ::CompareText(s1, &s2[1], s1 ? strlen(s1) : 0, s2.Length(), CPascalStr_AC::fgItl2Handle);
}

short CPascalStr_AC::CPascalStrCompare(unsigned char ch, const CPascalStr_AC& s2)
{
	return ::CompareText(&ch, &s2[1], 1, s2.Length(), CPascalStr_AC::fgItl2Handle);
}


//----------------------------------------------------------------------------------------
// CPascalStr_AC::operator+:
//----------------------------------------------------------------------------------------

CStr255_AC operator+(const CPascalStr_AC& s1, const CPascalStr_AC& s2)
{
	CStr255_AC newStr;

	if (s1.Length() + s2.Length() > kStr255Len)
		newStr.Length() = kStr255Len;
	else
		newStr.Length() = s1.Length() + s2.Length();
		
	BlockMove_AC(&s1[1], &newStr[1], s1.Length());
	BlockMove_AC(&s2[1], &newStr[s1.Length() + kLengthByte], newStr.Length() - s1.Length());

	return newStr;
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::operator+:
//----------------------------------------------------------------------------------------

CStr255_AC operator+(const CPascalStr_AC& s1, const char* s2)
{
	CStr255_AC newStr;
	short s2Len = (s2) ? static_cast<short>(strlen(s2)) : 0;

	if (s1.Length() + s2Len > kStr255Len)
		newStr.Length() = kStr255Len;
	else
		newStr.Length() = s1.Length() + s2Len;
		
	BlockMove_AC(&s1[1], &newStr[1], s1.Length());
	BlockMove_AC(s2, &newStr[s1.Length() + kLengthByte], newStr.Length() - s1.Length());

	return newStr;
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::operator+:
//----------------------------------------------------------------------------------------

CStr255_AC operator+(const CPascalStr_AC& s1, unsigned char ch)
{
	CStr255_AC newStr;
	short s2Len = 1;

	if (s1.Length() + s2Len > kStr255Len)
		newStr.Length() = kStr255Len;
	else
		newStr.Length() = s1.Length() + s2Len;
		
	BlockMove_AC(&s1[1], &newStr[1], s1.Length());
	BlockMove_AC(&ch, &newStr[s1.Length() + kLengthByte], newStr.Length() - s1.Length());

	return newStr;
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::operator+:
//----------------------------------------------------------------------------------------

CStr255_AC operator+(const char* s1, const CPascalStr_AC& s2)
{
	CStr255_AC newStr;
	short s1Len = (s1) ? static_cast<short>(strlen(s1)) : 0;

	if (s1Len + s2.Length() > kStr255Len)
		newStr.Length() = kStr255Len;
	else
		newStr.Length() = s1Len + s2.Length();
		
	BlockMove_AC(s1, &newStr[1], s1Len);
	BlockMove_AC(&s2[1], &newStr[s1Len + kLengthByte], newStr.Length() - s1Len);

	return newStr;
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::operator+:
//----------------------------------------------------------------------------------------

CStr255_AC operator+(unsigned char ch, const CPascalStr_AC& s2)
{
	CStr255_AC newStr;
	short s1Len = 1;

	if (s1Len + s2.Length() > kStr255Len)
		newStr.Length() = kStr255Len;
	else
		newStr.Length() = s1Len + s2.Length();
		
	BlockMove_AC(&ch, &newStr[1], s1Len);
	BlockMove_AC(&s2[1], &newStr[s1Len + kLengthByte], newStr.Length() - s1Len);

	return newStr;
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Insert
//----------------------------------------------------------------------------------------

void CPascalStr_AC::Insert(const char* insStr, unsigned char insStrLength, unsigned char pos, unsigned char maxLength)
{
	if (insStr)
	{
		int shortPos = pos;		// Was: short.
		if (shortPos > Length() + 1)
		{
#if qDebugMsg
			ProgramReport_AC("###CPascalStr_AC::Insert: Insert position greater than length of CPascalStr_AC.\n");
#endif
			if (Length() < maxLength)	
				shortPos = Length() + 1;
		}
	
#if qDebugMsg
		if (static_cast<short>(Length()) + insStrLength > maxLength)
			ProgramReport_AC("### CPascalStr_AC::Insert: CPascalStr_AC truncated during insert call.\n");	
#endif
	
		short usableLengthOfInsertString;
		short endPosOfInsertString;
		short usableLengthOfShiftedString;
		
		if (shortPos + insStrLength > maxLength)
			usableLengthOfInsertString = maxLength - shortPos + 1;
		else
			usableLengthOfInsertString = insStrLength;
		endPosOfInsertString = shortPos + usableLengthOfInsertString - 1;
		
		if ((endPosOfInsertString + 1) + (Length() - shortPos + 1) > maxLength)
			usableLengthOfShiftedString = maxLength - endPosOfInsertString;
		else
			usableLengthOfShiftedString = Length() - shortPos + 1;
			
		BlockMoveOverlap_AC(&(*this)[shortPos], &(*this)[endPosOfInsertString + 1], usableLengthOfShiftedString);
		BlockMove_AC(insStr, &(*this)[shortPos], usableLengthOfInsertString);
		Length() = usableLengthOfShiftedString + endPosOfInsertString;
	}
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Insert
//----------------------------------------------------------------------------------------

void CPascalStr_AC::Insert(const char* insStr, unsigned char pos, unsigned char maxLength)
{
	if (insStr)
		Insert(insStr, strlen(insStr), pos, maxLength);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Insert
//----------------------------------------------------------------------------------------

void CPascalStr_AC::Insert(const char* insStr, unsigned char pos)
{
	if (insStr)
		Insert(insStr, strlen(insStr), pos, kStr255Len);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Insert
//----------------------------------------------------------------------------------------

void CPascalStr_AC::Insert(const CPascalStr_AC& insStr, unsigned char pos, unsigned char maxLength)
{
	Insert(reinterpret_cast<const char*>(&insStr[1]), insStr.Length(), pos, maxLength);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Insert
//----------------------------------------------------------------------------------------

void CPascalStr_AC::Insert(const CPascalStr_AC& insStr, unsigned char pos)
{
	Insert(reinterpret_cast<const char*>(&insStr[1]), insStr.Length(), pos, kStr255Len);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Pos
//----------------------------------------------------------------------------------------

unsigned char CPascalStr_AC::Pos(const char* subStr, unsigned char startPos) const
{
	char cStr[kStr255Len + 1];

	BlockMove_AC(&(*this)[1], cStr, Length());
	cStr[Length()] = 0;
	char* ptr = strstr(&cStr[startPos - 1], subStr);
	
	return ptr ? ptr - &cStr[0] + 1 : 0;
}


//----------------------------------------------------------------------------------------
// CPascalStr_AC::Pos(CPascalStr_AC):
//----------------------------------------------------------------------------------------

unsigned char CPascalStr_AC::Pos(const CPascalStr_AC& subStr, unsigned char startPos) const
{
	char cStr[kStr255Len + 1];

	BlockMove_AC(&subStr[1], cStr, subStr.Length());
	cStr[subStr.Length()] = 0;

	return Pos(cStr, startPos);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Pos(CPascalStr_AC):
//----------------------------------------------------------------------------------------

unsigned char CPascalStr_AC::Pos(char c, unsigned char startPos) const
{
	char cStr[2];

	cStr[0] = c;
	cStr[1] = 0;

	return Pos(cStr, startPos);
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::Delete
//----------------------------------------------------------------------------------------
void CPascalStr_AC::Delete(unsigned char pos, unsigned char length)
{
	if ((pos > 0) && (length > 0) && (pos <= Length()))	// should also check that pos <= kMaxLength
	{
		if (pos + length > Length())
			Length() = pos - 1;
		else
		{
			BlockMoveOverlap_AC(&(*this)[static_cast<int>(pos) + static_cast<int>(length)],
								&(*this)[static_cast<int>(pos)], Length() - (pos + length) + kLengthByte);
			Length() -= length;
		}
	}
}

//----------------------------------------------------------------------------------------
// CPascalStr_AC::SetItl2Handle
//----------------------------------------------------------------------------------------

Handle CPascalStr_AC::SetItl2Handle(Handle itsItl2Handle)
{
	Handle returnVal = fgItl2Handle;
	fgItl2Handle = itsItl2Handle;
	
	return returnVal;
}


//----------------------------------------------------------------------------------------
// End of CPascalString_AC.cp


