//----------------------------------------------------------------------------------------
// CReporter_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Core_AC__
#include "Core_AC.h"
#endif


#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif


#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif


#ifndef __stdarg__
#include <stdarg.h>
#endif

#ifndef __stdio__
#include <stdio.h>
#endif


#ifndef __dout_AC__
#include "dout_AC.h"
#endif

#ifndef __string__
#include <string.h>
#endif


#define kEndOfBufferCookie 0x012345678
#define kBufferSize BUFSIZ

static char gWorkBuffer[kBufferSize + sizeof(long)];
// this is okay because we're not allocating any memory.

CReporter_AC::CListPtr CReporter_AC::fgReporters;
static CReporter_AC gDefaultReporter(false);



//----------------------------------------------------------------------------------------
// PrepareBuffer
//----------------------------------------------------------------------------------------

inline void PrepareBuffer()
{
	long* endOData = reinterpret_cast<long*>(&gWorkBuffer[kBufferSize]);
	*endOData = kEndOfBufferCookie;
}

//----------------------------------------------------------------------------------------
// VerifyBuffer
//----------------------------------------------------------------------------------------

inline void VerifyBuffer()
{
	long* endOData = reinterpret_cast<long*>(&gWorkBuffer[kBufferSize]);
	Assertion_AC( *endOData == kEndOfBufferCookie, "ProgramBreak buffer overwritten!");
}

//----------------------------------------------------------------------------------------
// CReporter_AC::CReporter_AC
//----------------------------------------------------------------------------------------

CReporter_AC::CReporter_AC(bool becomeDefault)
{
	SetIsCurrentInstance(becomeDefault);
}

//----------------------------------------------------------------------------------------
// CReporter_AC::~CReporter_AC
//----------------------------------------------------------------------------------------

CReporter_AC::~CReporter_AC()
{
	SetIsCurrentInstance(false);
}

#if qCoreDebug

//----------------------------------------------------------------------------------------
// CReporter_AC::OutputMessage
//----------------------------------------------------------------------------------------

void CReporter_AC::OutputMessage(const char* inOutput, bool continueIfPossible)
{
	static bool cantNest = false;
	
	if(!cantNest)
	{
		cantNest = true;
		dout << inOutput;
		
		if(continueIfPossible)
			dout << endl_AC;
		else
			dout << flush_AC;
		
		cantNest = false;
	}
	else
	{
		DebugStr("\pCReporter_AC Encounter nested calls to dout.");
	}
}
#else

//----------------------------------------------------------------------------------------
// CReporter_AC::OutputMessage
//----------------------------------------------------------------------------------------

void CReporter_AC::OutputMessage(const char* /*inOutput*/, bool /*continueIfPossible*/)
{
}
#endif


//----------------------------------------------------------------------------------------
// CReporter_AC::Instance
//----------------------------------------------------------------------------------------

CReporter_AC* CReporter_AC::Instance()
{
	return (fgReporters && (fgReporters->GetLength() > 0)) ? fgReporters->GetFirstElement() : GetDefaultInstance();
}


//----------------------------------------------------------------------------------------
// CReporter_AC::SetIsCurrentInstance
//----------------------------------------------------------------------------------------

CReporter_AC* CReporter_AC::SetIsCurrentInstance(bool makeCurrent)
{	
	static bool firstTime = true;
	
	if(firstTime)
	{
		firstTime = false;
		fgReporters = NULL;
	}
	
	CReporter_AC* returnVal = NULL; 
	
	if(	makeCurrent )
	{
		if(!fgReporters)
		{
			fgReporters = TH_new CList;
		}		
		if(!fgReporters->InList(this))
		{
			if(fgReporters->GetLength() > 0)
				returnVal = fgReporters->GetFirstElement();
		
			fgReporters->Push(this);
		}
	}
	else
	if( !makeCurrent )
	{
		if(fgReporters && fgReporters->InList(this))
		{
			fgReporters->Remove(this);
			
			if(fgReporters->GetLength() == 0)
			{
				delete fgReporters;
				fgReporters = NULL;	
			}
			
			returnVal = Instance();
		}
	}
	
	return returnVal;
}


//----------------------------------------------------------------------------------------
// CReporter_AC::GetDefaultInstance
//----------------------------------------------------------------------------------------

CReporter_AC* CReporter_AC::GetDefaultInstance()
{
	return &gDefaultReporter;
}

//----------------------------------------------------------------------------------------
// ProgramBreak
//----------------------------------------------------------------------------------------


void ProgramBreak_AC(const CStr255_AC& grievance)
{
	if(grievance.Length() > 0)
		CReporter_AC::Instance()->OutputMessage(grievance, false);
}

//----------------------------------------------------------------------------------------
// ProgramBreak_AC
//----------------------------------------------------------------------------------------

void ProgramBreak_AC(const char* grievance, ...)
{
	static va_list args;

	PrepareBuffer();

	va_start(args, grievance);
	vsprintf(gWorkBuffer, grievance, args);
	va_end(args);

	VerifyBuffer();

	if(::strlen(gWorkBuffer) > 0)
		CReporter_AC::Instance()->OutputMessage(gWorkBuffer, false);
		// true means stop in debugger.
}

//----------------------------------------------------------------------------------------
// ProgramReport
//----------------------------------------------------------------------------------------

void ProgramReport_AC(const CStr255_AC& grievance, bool breakInDebugger)
{
	if(grievance.Length() > 0)
		CReporter_AC::Instance()->OutputMessage(grievance, !breakInDebugger);
}

//----------------------------------------------------------------------------------------
// ProgramReport_AC
//----------------------------------------------------------------------------------------

void ProgramReport_AC(const char* grievance, bool stopInDebugger)
{
	if(::strlen(grievance) > 0)
		CReporter_AC::Instance()->OutputMessage(grievance, !stopInDebugger);
}

//----------------------------------------------------------------------------------------
// ProgramReport_AC
//----------------------------------------------------------------------------------------

void ProgramReport_AC(bool stopInDebugger, const char* grievance, ...)
{
	static va_list args;

	PrepareBuffer();
	
	va_start(args, grievance);
	vsprintf(gWorkBuffer, grievance, args);
	va_end(args);

	VerifyBuffer();
	
	if(::strlen(gWorkBuffer) > 0)
		CReporter_AC::Instance()->OutputMessage(gWorkBuffer, !stopInDebugger);
}