//----------------------------------------------------------------------------------------
// CDebugStrStream_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __dout_AC__
#include "dout_AC.h"
#endif

#if qCoreDebug

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef _STRING
#include <string.h>
#endif

#ifndef __CBuffer_AC__
#include "CBuffer_AC.h"
#endif

#ifndef __CBufferedStreamRep_AC__
#include "CBufferedStreamRep_AC.h"
#endif


#define kLineSize_AC 		85
#define kBufferSize_AC 		kLineSize_AC * 3


//----------------------------------------------------------------------------------------
// class CDebugStrStreamRep_AC
//----------------------------------------------------------------------------------------

class CDebugStrStreamRep_AC : public CBufferedStreamRep_AC
{
public:
	CDebugStrStreamRep_AC()
		:CBufferedStreamRep_AC(kBufferSize_AC)
	{	fWorkBuffer = TH_new CBuffer_AC(kBufferSize_AC+3); }
	virtual ~CDebugStrStreamRep_AC()
	{}
	
protected:
	virtual void WriteBufferedBytes(const void*, long amt);
	virtual void ReadBufferedBytes(void*, long) {}
	
protected:
	CBufferPtr_AC		fWorkBuffer;

};

CDebugStrStream_AC dout;


//----------------------------------------------------------------------------------------
// IsReturnChar
//----------------------------------------------------------------------------------------

inline bool IsReturnChar(char inChar)
{
	return inChar == '\n' || inChar == '\r';
}


//----------------------------------------------------------------------------------------
// CDebugStrStreamRep_AC
//----------------------------------------------------------------------------------------

void CDebugStrStreamRep_AC::WriteBufferedBytes(const void* bytes, long inAmt)
{	
	char* str = fWorkBuffer->GetBuffer();
	
#if qCoreDebug
	Erase_AC(str, kLineSize_AC*2);
#endif
	
	const char* buffer = static_cast<const char*>(bytes);
		// the string to send to DebugStr
		
	short bufferIndex = 0;
		// for copying into our work string
	
	short strIndex = -1;
		// the index into our tempStr
	short linePos = -1;
	
	long tally = 0;
	
	while(tally < inAmt)
	{
		bool insertedReturn = false;
	
		if( (++linePos == kLineSize_AC) && (bufferIndex != inAmt-1))
		{
			str[++strIndex] = '\r';
			linePos = -1;
			insertedReturn = true;
		}
	
		if( IsReturnChar(buffer[bufferIndex]) || bufferIndex == (fWorkBuffer->GetSize() - 1))
		{
			// We've found a '\n' in the buffer or our workbuffer is full.
			
			if(bufferIndex != inAmt-1) 
			{
				// if not the last char and didn't already insert one above
				if(!insertedReturn) str[++strIndex] = '\r';
				linePos = -1;
			}
			else
			{
				str[++strIndex] = ';';
				str[++strIndex] = 'g';
				str[++strIndex] = '\0'; 
				
				::c2pstr(str);
				DebugStr(reinterpret_cast<StringPtr>(str)); 
		
				strIndex = -1;
				// reset the strIndex to the start
				linePos = -1;
				
				bufferIndex = 0;
			
			}

		}
		else
		{
			str[++strIndex] = buffer[bufferIndex];
				// copy char into or work buffer.
		}
		
		++bufferIndex;
			// increment index into buffer.
		++tally;
			// keep track of how much we've snagged.
	}

	if(strIndex > -1)
	{
		// if we have chars in string ready to output, then
		// we need to do a DebugStr.
	
		str[++strIndex] = '\0'; 

		::c2pstr(str);
		DebugStr((StringPtr) str); 
	}
}


//----------------------------------------------------------------------------------------
// CDebugStrStream_AC::CDebugStrStream_AC
//----------------------------------------------------------------------------------------

CDebugStrStream_AC::CDebugStrStream_AC()
	: CCharOutputSink_AC(TH_new CDebugStrStreamRep_AC)
{
}	

//----------------------------------------------------------------------------------------
// CDebugStrStream_AC::~CDebugStrStream_AC
//----------------------------------------------------------------------------------------

CDebugStrStream_AC::~CDebugStrStream_AC()
{
}
#endif
	// qCoreDebug