//----------------------------------------------------------------------------------------
// CFSSpec_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CFSSpec_AC__
#define __CFSSpec_AC__

#ifndef __Files_AC__
#include "Files_AC.h"
#endif

// ACS

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

// Toolbox

#ifndef __FILES__
#include <Files.h>
#endif

//----------------------------------------------------------------------------------------
// CFSSpec_AC
//----------------------------------------------------------------------------------------
class EXPORT_AC CFSSpec_AC : public FSSpec
{
public:
	// CONSTRUCTORS:

	inline CFSSpec_AC() { Clear(); }
	inline CFSSpec_AC(const CFSSpec_AC& inFSSpec) { Set(inFSSpec); }
	inline CFSSpec_AC(const FSSpec& inFSSpec) { Set(inFSSpec); }

	inline CFSSpec_AC& operator=(const CFSSpec_AC& inFSSpec) { Set(inFSSpec); return *this; }
		
	// CONVERSION:
	
	inline operator FSSpec*() { return this; }
	inline operator const FSSpec*() const { return this; }
	inline operator FSSpec&() { return *this; }
	inline operator const FSSpec&() const { return *this; }
	
	// COMPARISON:

	bool operator ==(const CFSSpec_AC& rhs) const;
	inline bool operator !=(const CFSSpec_AC& rhs) const { return !(*this == rhs); }
	
	// QUICK-LOOKS:
	// The following calls can be used for quick-looks at a file's state.
	// If you need more functionality than that provided by these calls (e.g.,
	// aliases, file access, persistence, directory information), use CFile instead.
	// In the following functions, if Exists() returns false, the others will return a default 
	// value without indicating an error. If you prefer error checking, call GetFinderInfo 
	// directly or use the CFile-equivalent functions which do propagate errors.
	
	OSErr GetFinderInfo(FInfo& fndrInfo) const;
		// Return finder information about the file.

	OSErr GetFinderInfo(FInfo& fndrInfo, FXInfo& itsFXInfo, bool& isDirectory) const;
		// Return finder information about the file.
		// IF isDirectory returns true (indicating that it's a directory)
		// 		returns DInfo in fndrInfo and DXInfo in itsFXInfo
		// ELSE
		// 		returns FInfo in fndrInfo and FXInfo in itsFXInfo
	
	bool Exists() const;

	bool IsDirectory() const;
	
	bool IsFile() const;
	
	bool IsStationery() const;
	
	OSType GetFileType() const;

	OSType GetFileCreator() const;
	
	// UTILITY:

	void GetName(CStr63_AC& fileName) const;
		// Return the name of the file

	void SetName(const CStr63_AC& newName);
		// Sets the name field of the filespec but doesn't rename the file
		
	inline void Clear()
#if qPowerPC
	{ *(double*)this = 0; }
#else
	{ *(long*)this = *(((long*)this) + 1) = 0; }
#endif
	// cheat

	void Set(const FSSpec& inFSSpec);
	
	bool FindUniqueSpec();
		// Makes the spec unique by changing the name of the file. It adds ' x' (space num)
		// to the end of the file name if file exists. If lengh of file name exceeds 31 characters,
		// then the name is truncated, and elipses '' is appended, and then the ' x' is appended.
};

#endif // __CFSSpec__