//----------------------------------------------------------------------------------------
// CFile_AC.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CFile_AC__
#define __CFile_AC__

// This is required first in order to turn off old routine names, otherwise there is a conflict.

#ifndef __Files_AC__
#include "Files_AC.h"
#endif

// ACS

#ifndef __CFSSpec_AC__
#include "CFSSpec_AC.h"
#endif

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

#ifndef __FileTypes_AC__
#include "FileTypes_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __ALIASES__
#include <Aliases.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

//----------------------------------------------------------------------------------------
// CFile_AC
//----------------------------------------------------------------------------------------

enum ResourceUsage
{
	needResourceFork,
	preferResourceFork,
	noResourceFork
};

const bool kUsesDataFork = true;
const ResourceUsage kUsesRsrcFork = preferResourceFork;	
const bool kDataOpen = true;	
const bool kRsrcOpen = true;	



class EXPORT_AC CFile_AC
{

public:

	CFile_AC(OSType itsFileType = kGenericType_AC,
			  OSType itsCreator = kGenericCreator_AC,
			  bool usesDataFork = false,
			  ResourceUsage usesRsrcFork = noResourceFork,
			  bool keepsDataOpen = false,
			  bool keepsRsrcOpen = false);
		// Constructor

	CFile_AC(const CFile_AC& inFile);
		// Copy Constructor

	virtual ~CFile_AC();

	//------------------------------------------------------------------------------------
	// File Specification
	//------------------------------------------------------------------------------------

	virtual void Specify(const CFSSpec_AC& theFile);
		// Specify the file by its CFSSpec_AC.If the file has already opened a file, Specify
        // will attempt to close the file before re-identifying the file

	virtual void SpecifyWithStandardFileReply(const StandardFileReply& itsReply);
		// Specify a file with the information returned from CustomGetFile/CustomPutFile
		// This will also set the filetype and scripttag, and indicates if the
		// file is a stationery pad.
		
	virtual OSErr SpecifyWithSFReply(const SFReply& itsReply);
		// Specify a file with the information returned from SFPGetFile/SFPPutFile
		// This will also set the filetype.
		
	virtual OSErr SpecifyWithAlias(AliasHandle alias);
		// Similar to CFile_AC::Specify except the file is specified with an alias.  Also queries
		// the finder information to determine the real filetype, creator, and
		// whether or not the file pointed to by the alias is a stationery pad.

	virtual void SpecifyWithFile(CFile_AC* aFile);
		// Make this file reference the same file as aFile.

	virtual OSErr SpecifyWithTrio(short volRefNum, long dirID, const CStr63_AC& name);
		// Specify a file by volrefnum, dir ID, and filename. If dirID is zero, it then
        // volRefNum is assumed to be a working directory and will be resolved to a
        // directory ID

	//------------------------------------------------------------------------------------
	// Creating/Opening/Closing/Deleting
	//------------------------------------------------------------------------------------

	virtual OSErr CloseFile();
		// Close the forks of the file irrespective of the fKeepOpen fields

	virtual OSErr CloseFileIfNotKeptOpen();
		// Close the file respecting whether or not the data and resource fork should
		// be left open.
		
	virtual OSErr CloseDataFork();
		// Close the data fork. Called by CloseFile

	virtual OSErr CloseRsrcFork();
		// Close the rsrc fork. Called by CloseFile

	virtual OSErr CreateFile();
		// Create the file

	virtual OSErr CreateDataFork();
		// Create the data fork of the file. Called by CreateFile

	virtual OSErr CreateRsrcFork();
		// Create the rsrc fork of the file. Called by CreateFile

	virtual OSErr DeleteFile();
		// Close the file and delete it.

	virtual OSErr OpenFile();
		// Open the forks of the file irrespective of the fKeepOpen fields

	virtual OSErr OpenFileIfKeptOpen();
		// Open the file with the fDataPermission and fRsrcPermission respecting the
		// fKeepOpen fields

	virtual OSErr OpenDataFork(SignedByte permission = fsRdPerm);
		// Open the data fork of the file. Called by OpenFile

	virtual OSErr OpenRsrcFork(SignedByte permission);
		// Open the rsrc fork of the file. Called by OpenFile


	//------------------------------------------------------------------------------------
	// file attributes
	//------------------------------------------------------------------------------------

	virtual OSErr GetAlias(AliasHandle& alias);
		// Return the file specification as an alias handle

	inline OSType GetCachedFileType() const { return fFileType; }
		// Return the cached type of the file 

	inline OSType GetCachedFileCreator() const { return fCreator; }
		// Return the cached creator of the file

	virtual OSErr GetCatInfo(CInfoPBRec& cInfo);
		// Return the catalog information about the this file

	virtual long GetCreationDate();
		// Date the file was created

	virtual OSErr GetDataLength(long& length) const;
		// Return the length of the data fork

	virtual OSErr GetDataMark(long& mark) const;
		// Return the position of the mark in the data fork

	inline short GetDataRefNum() const { return fDataRefNum; }
		// Return the reference for reading/writing to the data fork.

	virtual long GetDirID();
		// Return the directory ID stored in the CFSSpec_AC

	virtual OSErr GetFileCreator(OSType& creator);
		// Return the creator of the file as found in the finder info

	virtual OSErr GetFileInfo(HParamBlockRec& pb);
		// Return all the information the file manager provides

	inline CFSSpec_AC GetFileSpec() const { return fFileSpec; }
	inline void GetFileSpec(CFSSpec_AC& theFileSpec) const { theFileSpec = GetFileSpec(); } // synonym for compatibility
		// Return its file specification record	
	
	virtual OSErr GetFileType(OSType& fileType);
		// Return the type of the file as found in the finder info
		
	virtual OSErr GetFinderInfo(FInfo& fndrInfo);
		// Return finder information about the file

	virtual OSErr GetFinderInfo(FInfo& fndrInfo, FXInfo& itsFXInfo, bool& isDirectory);
		// Return finder information about the file
		// IF isDirectory returns true (indicating that it's a directory)
		// 		returns DInfo in fndrInfo and DXInfo in itsFXInfo
		// ELSE
		// 		returns FInfo in fndrInfo and FXInfo in itsFXInfo

	virtual long GetModificationDate();
		// Return the date the file was last modified

	virtual void GetName(CStr63_AC& name);
		// Return the name of the file

	virtual OSErr GetPathName(CStr255_AC& pathName);
		// Return the MPW style path name of the file
		
	virtual OSErr GetPhysicalSize(long& dataSize, long& rsrcSize);
		// Returns the physical size in bytes of the data and resource fork

	long GetPosition() const;
		// Retrieve the current writing position.

	inline short GetRsrcRefNum() const { return fRsrcRefNum; }
		// Return the reference for reading/writing to resource fork.

	virtual OSErr GetScript(ScriptCode& theScript);
		// Returns the script code as stored in the extended finder info

	inline void SetFileType(const OSType& fileType) { fFileType = fileType; }
		// Set the cached file type
		
	virtual OSErr SetModificationDate(long modificationDate);
		// Sets the modification date of the file.
		
	virtual OSErr SetCreationDate(long creationDate);
		// Sets the creation date of the file.
		
	virtual OSErr SetFinderInfo(const FInfo& fndrInfo);
		// Sets the finder information for the file.
	
	void SetPosition(const long positionOrOffset, const short aMode = fsFromStart) const;
		// Set the current writing position.

	inline void SetUsesDataFork(bool usesDataFork) { fUsesDataFork = usesDataFork; }
		// Set if the file uses its data fork

	inline void SetUsesRsrcFork(ResourceUsage usesRsrcFork) { fUsesRsrcFork = usesRsrcFork; }
		// Set need or prefer the file uses its resource fork

	//----------------------------------------------------------------------------------------
	// Volume attributes
	//----------------------------------------------------------------------------------------

	virtual OSErr FlushVolume();
		// Flush the volume to force any changes to take place

	virtual OSErr GetBlockSize(long& blockSize);
		// Return the size of allocated blocks on the volume

	virtual OSErr GetFreeBlocks(long& freeBlocks);
		// Return the number of free blocks on the volume

	virtual OSErr GetVolumeInfo(HParamBlockRec& pb);
		// Return all information provided by the file manager on the volume

	virtual OSErr GetVolumeName(CStr63_AC& name);
		// Return the name of the volume

	virtual short GetVolRefNum();
		// Return the volume reference number stored in the CFSSpec_AC

	virtual OSErr ExchangeFiles(CFile_AC* aFile);
		// Exchange the contents of this with the contents of another file maintaining the
        // file id

	virtual bool IsDataForkOpen();
		// True if the data fork of the file is open

	virtual bool IsModified();
		// Checks fModDate to determine if file changed

	virtual bool IsRsrcForkOpen();
		// True if the resource fork of the file is open

	virtual bool IsSameFile(CFile_AC* aFile);
		// Does this file refer to the same file as aFile?

	virtual bool IsStationery();
		// True if this file is a stationery pad

	virtual OSErr IsVolumeLocked(bool& locked);
		// Returns true if the volume is hardware or software locked

	virtual bool HasDataFork();
		// Returns true if file has a data fork

	virtual bool HasRsrcFork();
		// Returns true if file has a resource fork

	virtual bool HasValidFileSpec();
		// Returns true if the filespec has been set to some value other than
		// the initialized value.
		
	virtual void Modified();
		// Update fModDate to the value stored by the finder

	virtual OSErr MoveAndRename(const CFSSpec_AC& dest);
		// Move the file to the new destination and give it a new name

	virtual OSErr ReadData(void* buffer, long& count);
		// Read count bytes from the data fork into buffer

	virtual OSErr ReadUntil(void* buffer, long& count, char untilChar);
		// Read bytes from the supplied buffer until the stop character untilChar
		// is found.  Returns the number of bytes actually read in count.
		
	virtual OSErr RenameFile(const CStr63_AC& newName);
		// Rename the file to the specified name

	virtual OSErr SetCatInfo(CInfoPBRec& cInfo);
		// Set the catalog information about this file. The file reference portion of the
        // record is supplied by fFileSpec.

	virtual OSErr SetDataLength(long length);
		// Set the length of the data fork

	virtual OSErr SetDataMark(long mark, short fromWhere = fsFromStart);
		// Set the position of the mark in the data fork

	virtual OSErr SetFileInfo( HParamBlockRec& pb);
		// Set the file information about this file. The file reference portion of the
        // record is supplied by fFileSpec.

	virtual void SetName(const CStr63_AC& newName);
		// Sets the name field of the filespec but doesn't rename the file

	virtual void SetPermissions(SignedByte dataPermission,
									   SignedByte rsrcPermission);
		// Set the fDataPermission and fRsrcPermission fields of the file
		
	virtual OSErr SetScript(ScriptCode theScript);
		// Sets the script code stored in the extended finder info

	bool UpdateFileSpec();
		// Update the file's CFSSpec_AC and notify if a change was required.

	virtual OSErr InteractWithUser();
		// Returns noErr.  A subclass may want to implement or call their own Notification-Manager-based routine.
		
	virtual OSErr UpdateResource();
		// Update the resource referenced by fRsrcRefNum

	virtual short UseResource();
		// Returns the current resource file and set itself as the current file

	virtual OSErr WriteData(const void* buffer, long& count);
		// Write count bytes from buffer to the data fork

	//----------------------------------------------------------------------------------------
	// simple accessors
	//----------------------------------------------------------------------------------------
	inline void SetKeepDataOpen(bool inKeepDataOpen) { fKeepDataOpen = inKeepDataOpen; }
	inline void SetKeepRsrcOpen(bool inKeepRsrcOpen) { fKeepRsrcOpen = inKeepRsrcOpen; }
	inline bool GetKeepDataOpen() const { return fKeepDataOpen; }
	inline bool GetKeepRsrcOpen() const { return fKeepRsrcOpen; }

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	CFSSpec_AC fFileSpec;						// volume/ directory/filename

	OSType fFileType;							// file type
	OSType fCreator;							// creator ID

	long fModDate;								// file mod date when last read/ saved

	short fDataRefNum;							// Reference for reading/ writing to data fork
	short fRsrcRefNum;							// Reference for reading/ writing to resource fork

	ScriptCode fScriptTag;						// The script the file was created with

	ResourceUsage fUsesRsrcFork;				// A value of need or prefer indicates
												// the file uses its resource fork
												
	SignedByte fDataPermission;					// Permissions used when opening the data fork
	SignedByte fRsrcPermission;					// Permissions used when opening the resource fork

	bool fStationery;							// True if this is stationery
	bool fUsesDataFork;							// True if the file uses its data fork
	bool fKeepDataOpen;							// Will the data fork be left open?
	bool fKeepRsrcOpen;							// Will the rsrc fork be left open?

protected:
	TemplateTempHandle<AliasHandle> fAlias;
};


#endif


