//----------------------------------------------------------------------------------------
// CFSSpec_AC.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CFSSpec_AC__
#include "CFSSpec_AC.h"
#endif

#include "FileTypes_AC.h"

// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __Errors__
#include <Errors.h>
#endif

//========================================================================================
// CLASS CFSSpec_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CFSSpec_AC::GetName: 
//----------------------------------------------------------------------------------------


void CFSSpec_AC::GetName(CStr63_AC& fileName) const
{
	fileName = name;
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::SetName: 
//----------------------------------------------------------------------------------------


void CFSSpec_AC::SetName(const CStr63_AC& newName)
{
	newName.CopyTo(name);
}

//----------------------------------------------------------------------------------------
//		 Set
//----------------------------------------------------------------------------------------
	
void CFSSpec_AC::Set(const FSSpec& inFSSpec)
{
	*(FSSpec*)this = *(FSSpec*)&inFSSpec;
}

//----------------------------------------------------------------------------------------
// Comparison operators for CFSSpec_AC
//----------------------------------------------------------------------------------------
	
bool CFSSpec_AC::operator ==(const CFSSpec_AC& inFSSpec) const
{
	return vRefNum == inFSSpec.vRefNum && parID == inFSSpec.parID && *(CStr63_AC*)&name == *(CStr63_AC*)&inFSSpec.name;
}

//----------------------------------------------------------------------------------------
// GetFinderInfo: This is the only File Manager abstraction provided by CFSSpec,
// which can be used for quick looks at a file's existence and type.  All other
// queries or operations on a file should go through the CFile class.
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// CFSSpec_AC::GetFinderInfo: 
//----------------------------------------------------------------------------------------


OSErr CFSSpec_AC::GetFinderInfo(FInfo& fndrInfo) const
{
	FXInfo itsFXInfo;
	bool isDirectory;

	return GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::GetFinderInfo: 
//----------------------------------------------------------------------------------------


OSErr CFSSpec_AC::GetFinderInfo(FInfo& fndrInfo, FXInfo& itsFXInfo, bool& isDirectory) const
{
	CInfoPBRec cInfo;

	BlockSet_AC(&cInfo, sizeof(CInfoPBRec), 0x00);// zero-out our storage 
	cInfo.hFileInfo.ioCompletion = NULL;

	cInfo.hFileInfo.ioNamePtr = const_cast<StringPtr>(name);

	cInfo.hFileInfo.ioVRefNum = vRefNum;
	cInfo.hFileInfo.ioFDirIndex = 0;
	cInfo.hFileInfo.ioDirID = parID;

	PBGetCatInfoSync(&cInfo);
	
	if (cInfo.hFileInfo.ioResult == noErr)
	{
		isDirectory = cInfo.hFileInfo.ioFlAttrib & ioDirMask;
		if (isDirectory)
		{
			fndrInfo = *(FInfo*)&cInfo.dirInfo.ioDrUsrWds;
			itsFXInfo = *(FXInfo*)&cInfo.dirInfo.ioDrFndrInfo;
		}
		else
		{
			fndrInfo = cInfo.hFileInfo.ioFlFndrInfo;
			itsFXInfo = cInfo.hFileInfo.ioFlXFndrInfo;
		}
	}

	return cInfo.hFileInfo.ioResult;
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::Exists: Returns whether the specified file exists on disk.
//----------------------------------------------------------------------------------------

bool CFSSpec_AC::Exists() const
{
	FInfo fndrInfo;
	FXInfo itsFXInfo;
	bool isDirectory;
	
	OSErr err = GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
	return (err == noErr);
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::IsDirectory: Returns whether the specified file is a directory.
//----------------------------------------------------------------------------------------

bool CFSSpec_AC::IsDirectory() const
{
	FInfo fndrInfo;
	FXInfo itsFXInfo;
	bool isDirectory;
	
	OSErr err = GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
	return isDirectory;
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::IsFile: Returns whether the specified file is a file (not a directory).
//----------------------------------------------------------------------------------------

bool CFSSpec_AC::IsFile() const
{
	FInfo fndrInfo;
	FXInfo itsFXInfo;
	bool isDirectory;
	
	OSErr err = GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
	return (err == noErr) && !isDirectory;
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::IsStationery: Returns whether the specified file is stationery.
//----------------------------------------------------------------------------------------

bool CFSSpec_AC::IsStationery() const
{
	FInfo fndrInfo;
	FXInfo itsFXInfo;
	bool isDirectory;
		
	OSErr err = GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
	return !isDirectory && ((fndrInfo.fdFlags & kIsStationary) != 0);	
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::GetFileType: Returns the file's type.
//----------------------------------------------------------------------------------------


OSType CFSSpec_AC::GetFileType() const
{
	FInfo finderInfo;
	FXInfo xFinderInfo;
	bool isDirectory;
	
	OSErr theErr = GetFinderInfo(finderInfo, xFinderInfo, isDirectory);
	return theErr == noErr ? (isDirectory ? kFolderType_AC : finderInfo.fdType) : kAllQuestionsType_AC;
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::GetFileCreator: Returns the file's creator.
//----------------------------------------------------------------------------------------


OSType CFSSpec_AC::GetFileCreator() const
{
	FInfo finderInfo;
	FXInfo xFinderInfo;
	bool isDirectory;
	
	OSErr theErr = GetFinderInfo(finderInfo, xFinderInfo, isDirectory);
	return theErr == noErr ? (isDirectory ? kFolderCreator_AC : finderInfo.fdCreator) : kGenericCreator_AC;
}

//----------------------------------------------------------------------------------------
// CFSSpec_AC::MakeUnique: Makes the spec unique by changing the name of the file.
//----------------------------------------------------------------------------------------

bool CFSSpec_AC::FindUniqueSpec()
{
	FInfo fndrInfo;
	FXInfo itsFXInfo;
	bool isDirectory;
	bool foundOne = false;
	
	OSErr err = GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
	CFSSpec_AC savedSpec = *this;

	long counter = 0;
	Str255 counterStr;
		
	while(err == noErr)
	{
		*this = savedSpec;
		
		NumToString(++counter, counterStr);
			
		short numLen = *counterStr;
		short insertIndex = *name;
				
		if(insertIndex + numLen + 1 > 31)
		{
			insertIndex = 31 - numLen - 1;
			*name = insertIndex;
			name[insertIndex] = '';
		}

		name[++insertIndex] = ' ';	
		(*name)++;
		
		for(short i = 1; i <= numLen; i++)
		{
			name[insertIndex+i] = counterStr[i];
			(*name)++;
		}
		
		err = GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
	}	
	
	if(err == fnfErr)
		foundOne = true;
	
	if(!foundOne)
		*this = savedSpec;
		
	return foundOne;
}

//----------------------------------------------------------------------------------------
// End of CFSSpec_AC.cp


