//----------------------------------------------------------------------------------------
// CFileStream_AC.cp 
// Copyright  1990-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein.  Used by permission 
//----------------------------------------------------------------------------------------

#ifndef __CFileStream_AC__
#include "CFileStream_AC.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

#ifndef __CFile_AC__
#include "CFile_AC.h"
#endif

//========================================================================================
// CLASS CFileStreamRep_AC
//========================================================================================

class CFileStreamRep_AC : public CRandomAccessStreamRep_AC
{
public:
	CFileStreamRep_AC(CFile_AC* itsFile);
	
	virtual ~CFileStreamRep_AC();
	
	virtual long GetPosition() const; // Override

	virtual void SetPosition(long newPosition );  // Override

	virtual long GetSize() const;  // Override

	virtual void SetSize(long newSize); // Override

	virtual void ReadBytes(void* p, long count); // Override

	virtual void WriteBytes(const void* p, long count); // Override

public:
	CFile_AC* fFile; 
};

//========================================================================================
// CLASS CFileStreamRep_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC constructor
//----------------------------------------------------------------------------------------

CFileStreamRep_AC::CFileStreamRep_AC(CFile_AC* itsFile) :
	fFile(itsFile)
{
}

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC destructor
//----------------------------------------------------------------------------------------

CFileStreamRep_AC::~CFileStreamRep_AC()
{
}

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC::GetPosition: 
//----------------------------------------------------------------------------------------

long CFileStreamRep_AC::GetPosition() const		// override 
{
	long itsDataMark;

	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->GetDataMark(itsDataMark));	// Call the CFile_AC method. 
	return itsDataMark;							// Return the result. 
}

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC::GetSize: 
//----------------------------------------------------------------------------------------

long CFileStreamRep_AC::GetSize() const			// override 
{
	long itsDataLength;

	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->GetDataLength(itsDataLength));		// Call the CFile_AC method. 
	return itsDataLength;						// Return the result. 
}

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC::SetPosition: 
//----------------------------------------------------------------------------------------


void CFileStreamRep_AC::SetPosition(long newPosition )// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->SetDataMark(newPosition,fsFromStart));
}

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC::SetSize: 
//----------------------------------------------------------------------------------------

void CFileStreamRep_AC::SetSize(long newSize)	// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->SetDataLength(newSize));
}

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC::ReadBytes: 
//----------------------------------------------------------------------------------------

void CFileStreamRep_AC::ReadBytes(void* p,
								   long count)	// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->ReadData(p,count));	
}

//----------------------------------------------------------------------------------------
// CFileStreamRep_AC::WriteBytes: 
//----------------------------------------------------------------------------------------

void CFileStreamRep_AC::WriteBytes(const void* p,
									long count)	// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->WriteData(p,count));
}

//========================================================================================
// CFileStream_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CFileStream_AC constructor
//----------------------------------------------------------------------------------------

CFileStream_AC::CFileStream_AC(CFile_AC* itsFile)
	: CRandomAccessStream_AC(TH_new CFileStreamRep_AC(itsFile))
{
}

//----------------------------------------------------------------------------------------
// CFileStream_AC destructor
//----------------------------------------------------------------------------------------

CFileStream_AC::~CFileStream_AC()
{
}
