//----------------------------------------------------------------------------------------
// CFile_AC.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CFile_AC__
#include "CFile_AC.h"
#endif

// ACS

#ifndef __Configuration_AC__
#include "Configuration_AC.h"
#endif

#ifndef __CProcess_AC__
#include "CProcess_AC.h"
#endif

#ifndef __ResFileUtils_AC__
#include "ResFileUtils_AC.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FINDER__
#include <Finder.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================


//========================================================================================
// CLASS CFile_AC
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// CFile_AC::IFile: 
//----------------------------------------------------------------------------------------

CFile_AC::CFile_AC(OSType itsFileType,
						 OSType itsCreator,
						 bool usesDataFork,
						 ResourceUsage usesRsrcFork,
						 bool keepsDataOpen,
						 bool keepsRsrcOpen) :
	fFileType(itsFileType),
	fCreator(itsCreator),
	fModDate(0),
	fDataRefNum(kNoFileRefNum_AC),
	fRsrcRefNum(kNoFileRefNum_AC),
	fScriptTag(smSystemScript),
	fUsesRsrcFork(usesRsrcFork),
	fDataPermission(fsRdPerm),
	fRsrcPermission(fsRdPerm),

	fStationery(false),
	fUsesDataFork(usesDataFork),
	fKeepDataOpen(keepsDataOpen),
	fKeepRsrcOpen(keepsRsrcOpen)
{
#if qDebugMsg
	if (keepsDataOpen &&!usesDataFork)
	{
		ProgramReport_AC("In CFile_AC::CFile_AC:  keepsDataOpen && !usesDataFork;\n");
		ProgramReport_AC("In CFile_AC::CFile_AC:  usesDataFork = true;\n");
		usesDataFork = true;
	}
	if (keepsRsrcOpen && (usesRsrcFork == noResourceFork))
	{
		ProgramReport_AC("In CFile_AC::CFile_AC:  keepsRsrcOpen && (usesRsrcFork == noResourceFork);\n");
		ProgramReport_AC("In CFile_AC::CFile_AC:  usesRsrcFork = preferResourceFork;\n");
		usesRsrcFork = preferResourceFork;
	}
#endif
}

//----------------------------------------------------------------------------------------
// CFile_AC::Free: 
//----------------------------------------------------------------------------------------

CFile_AC::~CFile_AC()
{
	CloseFile();
}

//----------------------------------------------------------------------------------------
// CFile_AC copy constructor 
//----------------------------------------------------------------------------------------

CFile_AC::CFile_AC(const CFile_AC& inFile) :
	fFileSpec(inFile.fFileSpec),
	fFileType(inFile.fFileType),
	fCreator(inFile.fCreator),
	fModDate(inFile.fModDate),
	fDataRefNum(inFile.fDataRefNum),
	fRsrcRefNum(inFile.fRsrcRefNum),
	fScriptTag(inFile.fScriptTag),
	fUsesRsrcFork(inFile.fUsesRsrcFork),
	fDataPermission(inFile.fDataPermission),
	fRsrcPermission(inFile.fRsrcPermission),

	fStationery(inFile.fStationery),
	fUsesDataFork(inFile.fUsesDataFork),
	fKeepDataOpen(inFile.fKeepDataOpen),
	fKeepRsrcOpen(inFile.fKeepRsrcOpen)
{
	if (inFile.fAlias)
	{
		AliasHandle newAliasHandle = NULL;
		::ThrowIfError_AC(GetAlias(newAliasHandle));
		fAlias = newAliasHandle;
	}
}

//----------------------------------------------------------------------------------------
// CFile_AC::Specify: 
//----------------------------------------------------------------------------------------

void CFile_AC::Specify(const CFSSpec_AC& theFile)
{
#if qDebugMsg
	// Warn if the file is open.
	if (IsDataForkOpen())
		ProgramReport_AC("CFile_AC::Specify: Called with data fork of file open.\n");
	if (IsRsrcForkOpen())
		ProgramReport_AC("CFile_AC::Specify: Called with resource fork of file open.\n");
#endif

	// If we are re-specifying an open file, close the file 
	CloseFile();
	fFileSpec = theFile;

	fAlias.reset();

	FInfo info;
	OSErr anErr = GetFinderInfo(info);
	if (anErr == noErr)
	{	AliasHandle newAliasHandle = NULL;
		::ThrowIfError_AC(NewAliasMinimal(&theFile, &newAliasHandle));
		fAlias = newAliasHandle;
	}
	else if (anErr != fnfErr && anErr != nsvErr)
		::ThrowIfError_AC(anErr);
}

//----------------------------------------------------------------------------------------
// CFile_AC::SpecifyWithStandardFileReply: 
//----------------------------------------------------------------------------------------

void CFile_AC::SpecifyWithStandardFileReply(const StandardFileReply& itsReply)
{
	Specify(itsReply.sfFile);
	
	fFileType = itsReply.sfType;			// only valid for StandardGetFile & CustomGetFile
	fScriptTag = itsReply.sfScript;
	if (itsReply.sfFlags & kIsStationary)	// only valid for StandardGetFile & CustomGetFile
		fStationery = true;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SpecifyWithSFReply: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SpecifyWithSFReply(const SFReply& itsReply)
{
	OSErr theErr = SpecifyWithTrio(itsReply.vRefNum, 0, itsReply.fName);
	
	fFileType = itsReply.fType;

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SpecifyWithAlias: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SpecifyWithAlias(AliasHandle alias)
{
	CFSSpec_AC theFileSpec;
	Boolean dontCare;

	OSErr theErr = ResolveAlias(NULL, alias, &theFileSpec, &dontCare);
	if (theErr == noErr)
	{
		FInfo finderInfo;
		FXInfo xFinderInfo;
		bool isDirectory;
		
		fFileType = kGenericType_AC;		// Since we don't know, set to original values
		fCreator = kGenericCreator_AC;
		fStationery = false;

		Specify(theFileSpec);
		if (GetFinderInfo(finderInfo, xFinderInfo, isDirectory) == noErr)
		{
			if (isDirectory)
			{
				fFileType = kFolderType_AC;
				fCreator = kFolderCreator_AC;
			}
			else
			{
				fFileType = finderInfo.fdType;
				fCreator = finderInfo.fdCreator;
				fStationery = (finderInfo.fdFlags & kIsStationary) != 0;
			}
		}
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SpecifyWithFile: 
//----------------------------------------------------------------------------------------

void CFile_AC::SpecifyWithFile(CFile_AC* aFile)
{
	aFile->UpdateFileSpec();		// Insure its specs are up-to-date.
	CFSSpec_AC theFile = aFile->fFileSpec;
	
	Specify(theFile);
	
	fFileType = aFile->fFileType;
	fCreator = aFile->fCreator;
	fScriptTag = aFile->fScriptTag;
	fStationery = aFile->fStationery;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SpecifyWithTrio: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SpecifyWithTrio(short volRefNum,
								   long dirID,
								   const CStr63_AC& name)
{
	CFSSpec_AC theFileSpec;
	OSErr theErr = noErr;

	if ((theErr = FSMakeFSSpec(volRefNum, dirID, name, &theFileSpec)) == fnfErr)
		theErr = noErr;						// The file may not yet exist

	if (theErr == noErr)
		Specify(theFileSpec);

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::CloseFile: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::CloseFile()
{
	OSErr dataErr = noErr;
	OSErr rsrcErr = noErr;

	if (fUsesDataFork)
		dataErr = CloseDataFork();
	if (fUsesRsrcFork != noResourceFork)
	{
		rsrcErr = CloseRsrcFork();
		if ((fUsesRsrcFork == preferResourceFork) && (rsrcErr == resFNotFound))
			rsrcErr = noErr;
	}

	return dataErr != noErr ? dataErr : rsrcErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::CloseFileIfNotKeptOpen: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::CloseFileIfNotKeptOpen()
{
	OSErr dataErr = noErr;
	OSErr rsrcErr = noErr;

	if (fUsesDataFork && !fKeepDataOpen)
		dataErr = CloseDataFork();
	if ((fUsesRsrcFork != noResourceFork) && !fKeepRsrcOpen)
	{
		rsrcErr = CloseRsrcFork();
		if ((fUsesRsrcFork == preferResourceFork) && (rsrcErr == resFNotFound))
			rsrcErr = noErr;
	}

	return dataErr != noErr ? dataErr : rsrcErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::CloseDataFork: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::CloseDataFork()
{
	OSErr theErr = noErr;

	if (fDataRefNum != kNoFileRefNum_AC)
	{
		theErr = FSClose(fDataRefNum);
		fDataRefNum = kNoFileRefNum_AC;
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::CloseRsrcFork: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::CloseRsrcFork()
{
	OSErr theErr = noErr;

	if (fRsrcRefNum != kNoFileRefNum_AC)
	{
		CloseResFile(fRsrcRefNum);
		theErr = ResError();
		fRsrcRefNum = kNoFileRefNum_AC;
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::CreateFile: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::CreateFile()
{
	OSErr dataErr = noErr;
	OSErr rsrcErr = noErr;

	if (fUsesDataFork)
		dataErr = CreateDataFork();

	if (fUsesRsrcFork != noResourceFork)
		rsrcErr = CreateRsrcFork();

	return dataErr != noErr ? dataErr : rsrcErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::CreateDataFork: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::CreateDataFork()
{
	return FSpCreate(&fFileSpec, fCreator, fFileType, fScriptTag);
}

//----------------------------------------------------------------------------------------
// CFile_AC::CreateRsrcFork: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::CreateRsrcFork()
{
	FSpCreateResFile(&fFileSpec, fCreator, fFileType, fScriptTag);

	return ResError();
}

//----------------------------------------------------------------------------------------
// CFile_AC::DeleteFile: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::DeleteFile()
{
	fModDate = 0;
	fAlias.reset();
	
	return FSpDelete(&fFileSpec);
}

//----------------------------------------------------------------------------------------
// CFile_AC::OpenFile: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::OpenFile()
{
	OSErr dataErr = noErr;
	OSErr rsrcErr = noErr;

	if (fUsesDataFork)
		dataErr = OpenDataFork(fDataPermission);

	if (fUsesRsrcFork != noResourceFork)
	{
		rsrcErr = OpenRsrcFork(fRsrcPermission);
		if ((fUsesRsrcFork == preferResourceFork) && (rsrcErr != noErr))
			rsrcErr = noErr;
	}

	return dataErr != noErr ? dataErr : rsrcErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::OpenFileIfKeptOpen: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::OpenFileIfKeptOpen()
{
	OSErr dataErr = noErr;
	OSErr rsrcErr = noErr;

	if (fUsesDataFork && fKeepDataOpen)
		dataErr = OpenDataFork(fDataPermission);

	if ((fUsesRsrcFork != noResourceFork) && fKeepRsrcOpen)
	{
		rsrcErr = OpenRsrcFork(fRsrcPermission);
		if ((fUsesRsrcFork == preferResourceFork) && (rsrcErr != noErr))
			rsrcErr = noErr;
	}

	return dataErr != noErr ? dataErr : rsrcErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::OpenDataFork: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::OpenDataFork(SignedByte permission)
{
	// Here's a short explanation of the permission codes (so you don't
	// have to look them up).  Note that they are NOT the same as
	// the 'permission' parameter. The bits (and decimal value) are:
	//	0 (1):    Request read permission.
	//	1 (2):    Request write permission.
	//	2-3:      Reserved; must be 0.
	//	4 (16):   Deny other readers access to this file.
	//	5 (32):   Deny other writers access to this file.
	//	6-15:     Reserved; must be 0.
	// If you do not use one of the standard permissions (fsRdPerm,
	// fsWrPerm, fsRdWrPerm, fsCurPerm or fsRdWrShPerm) then make sure
	// you've set the permission as ioDenyModes is going to expect.
	OSErr theErr = noErr;

	if (fDataRefNum == kNoFileRefNum_AC)			// Don't open if already open 
	{
		HParamBlockRec pb;

		BlockSet_AC(&pb, sizeof(HParamBlockRec), 0x00);
		pb.accessParam.ioNamePtr = fFileSpec.name;
		pb.accessParam.ioVRefNum = fFileSpec.vRefNum;
		pb.fileParam.ioDirID = fFileSpec.parID;
		
		switch (permission)
		{
			// read/deny write
			case fsRdPerm:
				pb.accessParam.ioDenyModes = fsRdPerm | 32;
				break;

			// read/write/deny read/deny write			
			case fsWrPerm:
			case fsRdWrPerm:
			case fsCurPerm:
				pb.accessParam.ioDenyModes = fsRdWrPerm | 16 | 32;
				break;
			
			// read/write/deny none
			case fsRdWrShPerm:
				pb.accessParam.ioDenyModes = fsRdWrPerm;
				break;
	
			// read/deny write			
			default:
				pb.accessParam.ioDenyModes = permission;
		}
				
		theErr = PBHOpenDenySync(&pb);			// Try the shared volume open. 

		if ((theErr == paramErr) || (theErr == wPrErr))	// Not on a shared volume, try HFS open. 
		{
			pb.ioParam.ioPermssn = permission & 3;
			theErr = PBHOpenSync(&pb);
		}

		if (theErr == noErr)
			fDataRefNum = pb.ioParam.ioRefNum;
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::OpenRsrcFork: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::OpenRsrcFork(SignedByte permission)
{
	OSErr theErr = noErr;

	if (fRsrcRefNum == kNoFileRefNum_AC)			// Don't open if already open 
	{
		fRsrcRefNum = FSpOpenResFile(&fFileSpec, (permission & 7));
		theErr = ResError();
	}
	if (theErr != noErr)
		fRsrcRefNum = kNoFileRefNum_AC;

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::ExchangeFiles: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::ExchangeFiles(CFile_AC* aFile)
{
	return FSpExchangeFiles(&fFileSpec, &aFile->fFileSpec);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetAlias: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetAlias(AliasHandle& alias)
{
	return NewAliasMinimal(&fFileSpec, &alias);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetCatInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetCatInfo(CInfoPBRec& cInfo)
{
	cInfo.hFileInfo.ioNamePtr = fFileSpec.name;
	cInfo.hFileInfo.ioVRefNum = fFileSpec.vRefNum;
	cInfo.hFileInfo.ioFDirIndex = 0;
	cInfo.hFileInfo.ioDirID = fFileSpec.parID;

	OSErr err = PBGetCatInfoSync(&cInfo);

	return err;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetCreationDate: 
//----------------------------------------------------------------------------------------

long CFile_AC::GetCreationDate()
{
	long returnVal = 0;

	CInfoPBRec cInfo;

	BlockSet_AC(&cInfo, sizeof(CInfoPBRec), 0x00);// zero-out our storage 
	if (GetCatInfo(cInfo) == noErr)
		returnVal = cInfo.hFileInfo.ioFlAttrib & ioDirMask ? cInfo.dirInfo.ioDrCrDat : cInfo.hFileInfo.ioFlCrDat;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetDataLength: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetDataLength(long& length) const
{
	return ::GetEOF(fDataRefNum, &length);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetDataMark: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetDataMark(long& mark) const
{
	return ::GetFPos(fDataRefNum, &mark);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetDirID: 
//----------------------------------------------------------------------------------------

long CFile_AC::GetDirID()
{
	return fFileSpec.parID;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetFileCreator: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetFileCreator(OSType& creator)
{
	FInfo finderInfo;
	FXInfo xFinderInfo;
	bool isDirectory;
	
	OSErr theErr = GetFinderInfo(finderInfo, xFinderInfo, isDirectory);
	creator = theErr == noErr ? (creator = isDirectory ? kFolderCreator_AC : finderInfo.fdCreator) : kGenericCreator_AC;
	
	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetFileInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetFileInfo(HParamBlockRec& pb)
{
	pb.fileParam.ioNamePtr = fFileSpec.name;
	pb.fileParam.ioVRefNum = fFileSpec.vRefNum;
	pb.fileParam.ioDirID = fFileSpec.parID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = 0;

	OSErr err = ::PBHGetFInfoSync(&pb);

	return err;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetFileType: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetFileType(OSType& fileType)
{
	FInfo finderInfo;
	FXInfo xFinderInfo;
	bool isDirectory;
	
	OSErr theErr = GetFinderInfo(finderInfo, xFinderInfo, isDirectory);
	fileType = theErr == noErr ? (fileType = isDirectory ? kFolderType_AC : finderInfo.fdType) : kAllQuestionsType_AC;
	
	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetFinderInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetFinderInfo(FInfo& fndrInfo)
{
	FXInfo itsFXInfo;
	bool isDirectory;

	return GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetFinderInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetFinderInfo(FInfo& fndrInfo, FXInfo& itsFXInfo, bool& isDirectory)
{
	return fFileSpec.GetFinderInfo(fndrInfo, itsFXInfo, isDirectory);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetModificationDate: 
//----------------------------------------------------------------------------------------

long CFile_AC::GetModificationDate()
{
	long returnVal = 0;

	CInfoPBRec cInfo;

	BlockSet_AC(&cInfo, sizeof(CInfoPBRec), 0x00);// zero-out our storage 
	if (GetCatInfo(cInfo) == noErr)
		returnVal = cInfo.hFileInfo.ioFlAttrib & ioDirMask ? cInfo.dirInfo.ioDrMdDat : cInfo.hFileInfo.ioFlMdDat;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetName: 
//----------------------------------------------------------------------------------------

void CFile_AC::GetName(CStr63_AC& name)
{
	name = fFileSpec.name;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetPathName: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetPathName(CStr255_AC& pathName)
{
	CStr63_AC dirName;
	
	pathName.Empty();
	
	CStr2_AC delimiter = HasAUX_AC() ? "/" : ":";
	
	CInfoPBRec cInfoPB;
	cInfoPB.dirInfo.ioFDirIndex = -1;
	cInfoPB.dirInfo.ioVRefNum = fFileSpec.vRefNum;
	cInfoPB.dirInfo.ioDrDirID = fFileSpec.parID;
	cInfoPB.dirInfo.ioNamePtr = dirName;
	
	OSErr err = noErr;
	while (err == noErr)
	{
		err = PBGetCatInfoSync(&cInfoPB);
		pathName.Insert(dirName + delimiter, 1);
		if (cInfoPB.dirInfo.ioDrDirID == fsRtDirID)
			break;

		cInfoPB.dirInfo.ioDrDirID = cInfoPB.dirInfo.ioDrParID;
	}
	
	if (HasAUX_AC())
		pathName.Insert("/", 1);
	
	return err;
	
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetPhysicalSize: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetPhysicalSize(long& dataSize,
									long& rsrcSize)
{
	HParamBlockRec pb;
	OSErr theErr;

	if ((theErr = GetFileInfo(pb)) == noErr)
	{
		dataSize = pb.fileParam.ioFlPyLen;
		rsrcSize = pb.fileParam.ioFlRPyLen;
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetPosition: 
//----------------------------------------------------------------------------------------

long CFile_AC::GetPosition() const
{
	// Retrieve the current writing position.
	long	position;
	::ThrowIfError_AC(::GetFPos(fDataRefNum, &position));
	return position;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetPosition: 
//----------------------------------------------------------------------------------------

void CFile_AC::SetPosition(const long positionOrOffset, const short aMode) const
{
	// Set the current writing position according to aMode.  The possible
	// values for aMode are defined in Files.h but are shown here for
	// your convenience:
	//	fsAtMark	= 0		Ignore positionOrOffset.  Do nothing.
	//	fsFromStart	= 1		Set position relative to beginning of file.  Default.
	//	fsFromLEOF	= 2		Set position relative to logical end-of-file.
	//	fsFromMark	= 3		Set position relative to current position.

	::ThrowIfError_AC(::SetFPos(fDataRefNum, aMode, positionOrOffset));
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetScript: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetScript(ScriptCode& theScript)
{
	OSErr theErr;
	CInfoPBRec cInfo;

	BlockSet_AC(&cInfo, sizeof(CInfoPBRec), 0x00);// zero-out our storage 
	theErr = GetCatInfo(cInfo);
	if (theErr == noErr)
	{
		theScript = cInfo.hFileInfo.ioFlXFndrInfo.fdScript;
		fScriptTag = theScript;
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetModificationDate: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetModificationDate(long modificationDate)
{
	// Get the current information and just touch the modification date
	CInfoPBRec cInfo;
	OSErr theErr = noErr;

	BlockSet_AC(&cInfo, sizeof(CInfoPBRec), 0x00);// zero-out our storage 
	if (GetCatInfo(cInfo) == noErr)
	{
		if (cInfo.hFileInfo.ioFlAttrib & ioDirMask)
			cInfo.dirInfo.ioDrMdDat = modificationDate;
		else
			cInfo.hFileInfo.ioFlMdDat = modificationDate;

		SetCatInfo(cInfo);
	}

	return theErr;
}
		
//----------------------------------------------------------------------------------------
// CFile_AC::SetCreationDate: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetCreationDate(long creationDate)
{
	// Get the current information and just touch the modification date
	CInfoPBRec cInfo;
	OSErr theErr = noErr;

	BlockSet_AC(&cInfo, sizeof(CInfoPBRec), 0x00);// zero-out our storage 
	if (GetCatInfo(cInfo) == noErr)
	{
		if (cInfo.hFileInfo.ioFlAttrib & ioDirMask)
			cInfo.dirInfo.ioDrCrDat = creationDate;
		else
			cInfo.hFileInfo.ioFlCrDat = creationDate;

		SetCatInfo(cInfo);
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetFinderInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetFinderInfo(const FInfo& fndrInfo)
{
	HParamBlockRec pb;
	OSErr theErr = noErr;

	// Get the current information and just touch the finder info
	theErr = GetFileInfo(pb);
	if (theErr == noErr)
	{
		pb.fileParam.ioFlFndrInfo = fndrInfo;
		theErr = SetFileInfo(pb);
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::FlushVolume: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::FlushVolume()
{
	return FlushVol(NULL, fFileSpec.vRefNum);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetBlockSize: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetBlockSize(long& blockSize)
{
	HParamBlockRec pb;
	OSErr theErr;

	BlockSet_AC(&pb, sizeof(HParamBlockRec), 0x00);// zero-out our storage 
	theErr = GetVolumeInfo(pb);
	if (theErr == noErr)
		blockSize = pb.volumeParam.ioVAlBlkSiz;
	else
		blockSize = 0;

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetFreeBlocks: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetFreeBlocks(long& freeBlocks)
{
	HParamBlockRec pb;
	BlockSet_AC(&pb, sizeof(HParamBlockRec), 0x00);// zero-out our storage 

	OSErr theErr = GetVolumeInfo(pb);
	freeBlocks = theErr == noErr ? pb.volumeParam.ioVFrBlk : 0;

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetVolumeInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetVolumeInfo(HParamBlockRec& pb)
{
	pb.volumeParam.ioCompletion = NULL;
	pb.volumeParam.ioNamePtr = NULL;
	pb.volumeParam.ioVRefNum = fFileSpec.vRefNum;
	pb.volumeParam.ioVolIndex = -1;				// Use the vRefNum since it is all we have 

	return PBHGetVInfoSync(&pb);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetVolumeName: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::GetVolumeName(CStr63_AC& name)
{
	HParamBlockRec pb;

	BlockSet_AC(&pb, sizeof(HParamBlockRec), 0x00);
	pb.volumeParam.ioNamePtr = name;			// Fill in the name 
	pb.volumeParam.ioVRefNum = fFileSpec.vRefNum;
	pb.volumeParam.ioVolIndex = 0;				// Use the vRefNum since it is all we have 

	return PBHGetVInfoSync(&pb);
}

//----------------------------------------------------------------------------------------
// CFile_AC::GetVolRefNum: 
//----------------------------------------------------------------------------------------

short CFile_AC::GetVolRefNum()
{
	return fFileSpec.vRefNum;
}

//----------------------------------------------------------------------------------------
// CFile_AC::IsDataForkOpen: 
//----------------------------------------------------------------------------------------

bool CFile_AC::IsDataForkOpen()
{
	return fDataRefNum != kNoFileRefNum_AC;
}

//----------------------------------------------------------------------------------------
// CFile_AC::IsRsrcForkOpen: 
//----------------------------------------------------------------------------------------

bool CFile_AC::IsRsrcForkOpen()
{
	return fRsrcRefNum != kNoFileRefNum_AC;
}

//----------------------------------------------------------------------------------------
// CFile_AC::IsSameFile: 
//----------------------------------------------------------------------------------------

bool CFile_AC::IsSameFile(CFile_AC* aFile)
{
	// See if aFile represents the same file as this.
	bool returnVal = false;

	if (aFile == this)
	{
		returnVal = true;		// Short circuit test for identity.
	}
	else
	{
		UpdateFileSpec();		// Make sure we're up-to-date.
	
		if ((fFileSpec.vRefNum == aFile->GetVolRefNum())
			&& (fFileSpec.parID == aFile->GetDirID()))
		{
			CStr63_AC name;
			aFile->GetName(name);
			returnVal = EqualString(fFileSpec.name, name, false, true);
		}
	}
	return returnVal;
}

//----------------------------------------------------------------------------------------
// CFile_AC::IsStationery: 
//----------------------------------------------------------------------------------------

bool CFile_AC::IsStationery()
{
	return fStationery;
}

//----------------------------------------------------------------------------------------
// CFile_AC::IsModified: 
//----------------------------------------------------------------------------------------

bool CFile_AC::IsModified()
{
	return GetModificationDate() != fModDate;
}

//----------------------------------------------------------------------------------------
// CFile_AC::IsVolumeLocked: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::IsVolumeLocked(bool& locked)
{
	HParamBlockRec pb;
	OSErr theErr;

	if ((theErr = GetVolumeInfo(pb)) == noErr)
		locked = (pb.volumeParam.ioVAtrb & 0x8080) != 0;

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::HasDataFork: 
//----------------------------------------------------------------------------------------

bool CFile_AC::HasDataFork()
{
	return fUsesDataFork;
}

//----------------------------------------------------------------------------------------
// CFile_AC::HasRsrcFork: 
//----------------------------------------------------------------------------------------

bool CFile_AC::HasRsrcFork()
{
	return fUsesRsrcFork != noResourceFork;
}

//----------------------------------------------------------------------------------------
// CFile_AC::HasValidFileSpec: 
//----------------------------------------------------------------------------------------

bool CFile_AC::HasValidFileSpec()
{
	return fFileSpec.vRefNum || fFileSpec.parID;
}

//----------------------------------------------------------------------------------------
// CFile_AC::Modified: 
//----------------------------------------------------------------------------------------

void CFile_AC::Modified()
{
	fModDate = GetModificationDate();
	
	FInfo info;
	::ThrowIfError_AC(GetFinderInfo(info));

	fAlias.reset();

	AliasHandle newAliasHandle = NULL;
	::ThrowIfError_AC(GetAlias(newAliasHandle));
	fAlias = newAliasHandle;
}

//----------------------------------------------------------------------------------------
// CFile_AC::MoveAndRename: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::MoveAndRename(const CFSSpec_AC& dest)
{
	OSErr returnVal = noErr;

	CInfoPBRec cInfoPB;
	CStr63_AC destName;

	cInfoPB.dirInfo.ioVRefNum = dest.vRefNum;
	cInfoPB.dirInfo.ioDrDirID = dest.parID;
	cInfoPB.dirInfo.ioNamePtr = destName;		// Will be filled in 
	cInfoPB.dirInfo.ioFDirIndex = -1;
	if ((returnVal = PBGetCatInfoSync(&cInfoPB)) == noErr)
	{
		CMovePBRec cMovePB;

		long parentDirID = cInfoPB.dirInfo.ioDrParID;

		cMovePB.ioVRefNum = fFileSpec.vRefNum;
		cMovePB.ioDirID = fFileSpec.parID;
		cMovePB.ioNamePtr = fFileSpec.name;
		cMovePB.ioNewDirID = parentDirID;
		cMovePB.ioNewName = destName;
		if ((returnVal = PBCatMoveSync(&cMovePB)) == noErr)
			returnVal = HRename(dest.vRefNum, dest.parID, fFileSpec.name, dest.name);
	}
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// CFile_AC::ReadData: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::ReadData(void* buffer,
							 long& count)
{
	return FSRead(fDataRefNum, &count, (Ptr)buffer);
}

//----------------------------------------------------------------------------------------
// CFile_AC::ReadUntil: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::ReadUntil(void* buffer,
   							  long& count,
   							  char untilChar)
{
   OSErr theResult = noErr;
 
 	if (fDataRefNum != kNoFileRefNum_AC)
	{
		ParamBlockRec theParamBlock;
		theParamBlock.ioParam.ioCompletion = NULL;
		theParamBlock.ioParam.ioRefNum = fDataRefNum;
		theParamBlock.ioParam.ioBuffer = (Ptr) buffer;
		theParamBlock.ioParam.ioReqCount = count;
		theParamBlock.ioParam.ioPosMode = fsAtMark | 0x0080 | (untilChar * 0x100);
		theResult = ::PBReadSync(&theParamBlock);

		count = theParamBlock.ioParam.ioActCount;
	}

	return theResult;
 }

//----------------------------------------------------------------------------------------
// CFile_AC::RenameFile: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::RenameFile(const CStr63_AC& newName)
{
	return FSpRename(&fFileSpec, newName);
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetCatInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetCatInfo(CInfoPBRec& cInfo)
{
	cInfo.hFileInfo.ioCompletion = NULL;
	cInfo.hFileInfo.ioNamePtr = fFileSpec.name;
	cInfo.hFileInfo.ioVRefNum = fFileSpec.vRefNum;
	cInfo.hFileInfo.ioDirID = fFileSpec.parID;

	OSErr err = PBSetCatInfoSync(&cInfo);

	return err;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetDataLength: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetDataLength(long length)
{
	return ::SetEOF(fDataRefNum, length);
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetDataMark: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetDataMark(long mark,
							short fromWhere)
{
	return ::SetFPos(fDataRefNum, fromWhere, mark);
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetFileInfo: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetFileInfo(HParamBlockRec& pb)
{
	pb.fileParam.ioNamePtr = fFileSpec.name;
	pb.fileParam.ioVRefNum = fFileSpec.vRefNum;
	pb.fileParam.ioDirID = fFileSpec.parID;

	OSErr err = PBHSetFInfoSync(&pb);

	return err;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetName: 
//----------------------------------------------------------------------------------------

void CFile_AC::SetName(const CStr63_AC& newName)
{
	newName.CopyTo(fFileSpec.name);
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetScript: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::SetScript(ScriptCode theScript)
{
	OSErr theErr;
	CInfoPBRec cInfo;

	// Get the current info so we don't have to set up the entire record 
	if ((theErr = GetCatInfo(cInfo)) == noErr)
	{
		cInfo.hFileInfo.ioFlXFndrInfo.fdScript = theScript;
		theErr = SetCatInfo(cInfo);
	}

	return theErr;
}

//----------------------------------------------------------------------------------------
// CFile_AC::SetPermissions: 
//----------------------------------------------------------------------------------------

void CFile_AC::SetPermissions(SignedByte dataPermission,
							  SignedByte rsrcPermission)
{
	// The values the 'permission' parameters can take are:
	//	fsCurPerm      =  0;    {whatever permission is allowed}
	//	fsRdPerm       =  1;    {read permission}
	//	fsWrPerm       =  2;    {write permission}
	//	fsRdWrPerm     =  3;    {exclusive read/write permission}
	//	fsRdWrShPerm   =  4;    {shared read/write permission}

	fDataPermission = dataPermission;
	fRsrcPermission = rsrcPermission;
}

//----------------------------------------------------------------------------------------
// CFile_AC::UpdateFileSpec: 
//----------------------------------------------------------------------------------------

bool CFile_AC::UpdateFileSpec()
{
	// Before we go using the aFile's information we must make sure it's
	// up-to-date (things might have change in the Finder, for example, such
	// as moving the file to another folder on the same volume or renaming
	// the file.
	
	Boolean		wasChanged = false;
	
	if (fAlias)
	{
		// We can't just fail if no interaction is allowed before resolving the
		// alias because there may be no need for user interaction.  If we
		// are in the background then immediately assume interaction is not
		// allowed.
		OSErr		anErr = errAENoUserInteraction;  // Assumes we're backgrounded.

		if (CProcess_AC::fgProcess && CProcess_AC::fgProcess->IsFrontProcess())
			anErr = CProcess_AC::DoInteractWithUser();

		if (anErr == errAENoUserInteraction)
		{
			// No interaction is allowed so just attempt to resolve the alias without UI.
			CFSSpec_AC	theFileSpec;			// Resulting file spec goes here.
			Boolean		updateAlias = false;
			short		myMatchCount = 1;
	
			// First try the absolute search method.
			anErr = ::MatchAlias(NULL,					// fromFile
								kARMMountVol
									+ kARMSearch
									+ kARMNoUI,			// rulesMask
								fAlias,					// alias
								&myMatchCount,			// aliasCount
								&theFileSpec,			// aliasList (FSSpecArrayPtr)
								&updateAlias,			// needsUpdate
								NULL,					// aliasFilter
								NULL);					// yourDataPtr);
		}
		else
			anErr = ::ResolveAlias(NULL, fAlias, &fFileSpec, &wasChanged);

		if (anErr != userCanceledErr)
			::ThrowIfError_AC(anErr);
	}

	return wasChanged;
}

//----------------------------------------------------------------------------------------
// CFile_AC::InteractWithUser: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::InteractWithUser()
{
	return noErr;
}
//----------------------------------------------------------------------------------------
// CFile_AC::UpdateResource: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::UpdateResource()
{
	UpdateResFile(fRsrcRefNum);

	return ResError();
}

//----------------------------------------------------------------------------------------
// CFile_AC::UseResource: 
//----------------------------------------------------------------------------------------

short CFile_AC::UseResource()
{
	return UseResFile_AC(fRsrcRefNum);
}

//----------------------------------------------------------------------------------------
// CFile_AC::WriteData: 
//----------------------------------------------------------------------------------------

OSErr CFile_AC::WriteData(const void* buffer,
							  long& count)
{
	return FSWrite(fDataRefNum, &count, buffer);
}


//----------------------------------------------------------------------------------------
// End of CFile_AC.cp


