// CResourceStream_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein. Used by permission

#ifndef __CResourceStream_AC__
#include "CResourceStream_AC.h"
#endif

// Toolbox

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

//========================================================================================
// CLASS CResourceStreamRep_AC
//========================================================================================

class CResourceStreamRep_AC : public CRandomAccessStreamRep_AC
{
public:
	CResourceStreamRep_AC(ResType theType, ResNumber theID);
	
	virtual ~CResourceStreamRep_AC();

	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------
	virtual long GetPosition() const;
		// Returns fPosition.

	virtual void SetPosition(long newPosition );
		// Sets fPosition.
	
	virtual long GetSize() const;
		// Returns fSize.

	virtual void SetSize(long newSize);
		// Sets fSize.
	
	virtual void ReadBytes(void* p, long count);
		// Reads bytes from the resource calling ReadPartialResource.

	virtual void WriteBytes(const void* p, long count);
		// Write bytes to the resource calling WritePartialResource.

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	Handle fResource;							// the resource to write to/ read from
	long fPosition;								// offset into the resource
	long fSize;									// offset into the resource
};

//----------------------------------------------------------------------------------------
// CResourceStream_AC::IResourceStream: 
//----------------------------------------------------------------------------------------

CResourceStreamRep_AC::CResourceStreamRep_AC(ResType theType,
									  ResNumber theID) :
	fResource(NULL),
	fPosition(0),
	fSize(0)
{
	if (theType != kNoIdentifier_AC)
	{
		short oldResLoad = LMGetResLoad();
		SetResLoad(false);
		fResource = GetResource(theType, theID);
		SetResLoad(oldResLoad);
		::ThrowIfNULLResource_AC(fResource);
	
		fSize = GetResourceSizeOnDisk(fResource);
		::ThrowIfResError_AC();
	}
}

//----------------------------------------------------------------------------------------
// CResourceStreamRep_AC destructor
//----------------------------------------------------------------------------------------

CResourceStreamRep_AC::~CResourceStreamRep_AC()
{
}

//----------------------------------------------------------------------------------------
// CResourceStreamRep_AC::GetPosition: 
//----------------------------------------------------------------------------------------

long CResourceStreamRep_AC::GetPosition() const				// override 
{
	return fPosition;
}

//----------------------------------------------------------------------------------------
// CResourceStreamRep_AC::SetPosition: 
//----------------------------------------------------------------------------------------

void CResourceStreamRep_AC::SetPosition(long newPosition )// override 
{
	fPosition = newPosition;
}

//----------------------------------------------------------------------------------------
// CResourceStreamRep_AC::GetSize: 
//----------------------------------------------------------------------------------------

long CResourceStreamRep_AC::GetSize() const
{
	return fSize;
}

//----------------------------------------------------------------------------------------
// CResourceStreamRep_AC::SetSize: 
//----------------------------------------------------------------------------------------

void CResourceStreamRep_AC::SetSize(long newSize)		// override 
{
	SetResourceSize(fResource, newSize);
	::ThrowIfResError_AC();

	fSize = newSize;							// update if successfull

	if (newSize < fPosition)					// If we shrunk the size, then we have to adjust the position.
		fPosition = newSize;
}

//----------------------------------------------------------------------------------------
// CResourceStreamRep_AC::WriteBytes: 
//----------------------------------------------------------------------------------------

void CResourceStreamRep_AC::WriteBytes(const void* p,
								 long count)
{
	register long minSize = GetPosition() + count;
	if (minSize > GetSize())
		SetSize(minSize);						// increase the size before writing
	WritePartialResource(fResource, GetPosition(), p, count);
	::ThrowIfResError_AC();

	SetPosition(minSize);						// Advance the position. 
}

//----------------------------------------------------------------------------------------
// CResourceStreamRep_AC::ReadBytes: 
//----------------------------------------------------------------------------------------

void CResourceStreamRep_AC::ReadBytes(void* p,
								long count)		// override 
{
	register long minToRead = Min_AC(count, GetSize() - GetPosition());
	ReadPartialResource(fResource, GetPosition(), p, minToRead);
	::ThrowIfResError_AC();

	SetPosition(GetPosition() + minToRead);		// Advance the position. 
}

//========================================================================================
// CLASS CResourceStream_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CResourceStream_AC constructor
//----------------------------------------------------------------------------------------


CResourceStream_AC::CResourceStream_AC(ResType theType,ResNumber theID)
	: CRandomAccessStream_AC(TH_new CResourceStreamRep_AC(theType, theID))
{
}

//----------------------------------------------------------------------------------------
// CResourceStream_AC destructor
//----------------------------------------------------------------------------------------

CResourceStream_AC::~CResourceStream_AC()
{
}