// CGDIterator_AC.h
//  Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//  Portions Copyright  1996 by Trygve Isaacson.

#ifndef __CGDIterator_AC__
#define __CGDIterator_AC__

// ACS

#ifndef __Imaging_AC__
#include "Imaging_AC.h"
#endif

// Toolbox

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

//----------------------------------------------------------------------------------------
// CGDIterator_AC
//
// To properly handle drawing in grayscale with multiple monitors
// and differing color depths, we must draw while iterating over
// the GD handles. This iterator makes a for loop easy while also
// reducing the clipping area to the appropriate device and setting
// a boolean to indicate whether the device is "deep" or not. "Deep"
// meaning suitable for grayscale drawing as opposed to black & white.
//----------------------------------------------------------------------------------------

class EXPORT_AC CGDIterator_AC
{
public:

	CGDIterator_AC();
	virtual ~CGDIterator_AC();
	
	void	Setup();			// called by constructor; can be called again
	bool	NextDevice(bool& deep);// returns true while you should draw again
	void	Cleanup();			// called by destructor; can be called earlier
	
	void	ClipFurtherToCurrentDevice();	// can be called for tricky clipping situations (see slider labels)

protected:

	// To look good in gray we need 4 bits in monochrome, or 8 bits in color.
	// This is because 2 bits monochrome is only black, white, plus 2 shades;
	// and 4 bits in color is just that plus 12 irrelevant colors.
	enum { kDeepGrayscaleBits = 4, kDeepColorBits = 8 };

	GDHandle	FindNextValidDevice(GDHandle currentDevice, bool& deep);
	void		ClipFurtherToDevice(GDHandle aDevice);

	bool fIteratedYet;
	GDHandle fCurrentGDHandle;	// inited by Setup, changed by NextDevice

	static RgnHandle fgSavedClipping;		// saved by Setup, restored by Cleanup
};


#endif
