// CPoint_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.

#ifndef __CPoint_AC__
#define __CPoint_AC__

// ACS

#ifndef __Imaging_AC__
#include "Imaging_AC.h"
#endif

#if defined(qCoreDebug) && !defined(__CPascalString_AC__)
#include "CPascalString_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

// ANSI

#if defined(qCoreDebug) && !defined(_STDIO)
#include <stdio.h>
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kMaxCoord = SHRT_MAX - 1;	// -1 to minimize signed overflow problems

//----------------------------------------------------------------------------------------
// Types
//----------------------------------------------------------------------------------------

class CPoint_AC;
class CRect_AC;

//--------------------------------------------------------------------------------
// Selector operator indicator enumeration. Which coordinate do we want
// to select?, v or h?
//--------------------------------------------------------------------------------
#if qMSWindows
	const VHSelect vSel = 0, hSel = 1;
#else
	enum
	{
		vSel,
		hSel
	};
#endif

//--------------------------------------------------------------------------------
// CPoint_AC: is a toolbox compatible class for the toolbox struct Point. It is bitwise
// compatible with the toolbox struct because it contains no virtual functions.
//--------------------------------------------------------------------------------
class EXPORT_AC CPoint_AC : public Point
{
public:
	// CONSTRUCTORS:

	inline CPoint_AC() { Clear(); }
	inline CPoint_AC(short iH, short iV) { v = iV; h = iH; }
	inline CPoint_AC(const Point& pt) { *this = *(const CPoint_AC*)&pt; }

	// SELECTION:
	
	inline short& operator[](VHSelect sel) { return (sel == vSel) ? v : h; }
	inline const short& operator[](VHSelect sel) const { return (sel == vSel) ? v : h; }

	// CONVERSION:
	
#if qCoreDebug
	inline operator const char*() const
	{
		char textPoint[64];
		sprintf (textPoint, "CPoint_AC(%d, %d)", h, v);
		return CChar63_AC(textPoint);
	}
	// For converting CPoint_AC to a textual representation of a CPoint_AC. "CPoint_AC(h,v)".
#endif
	
	inline operator Point*() { return this; }
	inline operator const Point*() const { return this; }		

	inline operator long&() { return *reinterpret_cast<long*>(this); }
	inline operator const long&() const { return *reinterpret_cast<const long*>(this); }

	// ARITHMETIC:

	CPoint_AC& operator+=(const CPoint_AC& pt);
	CPoint_AC& operator-=(const CPoint_AC& pt);
	CPoint_AC& operator*=(const CPoint_AC& pt);
	CPoint_AC& operator/=(const CPoint_AC& pt);
	CPoint_AC operator+(const CPoint_AC& pt) const;
	CPoint_AC operator-(const CPoint_AC& pt) const;
	CPoint_AC operator*(const CPoint_AC& pt) const;
	CPoint_AC operator/(const CPoint_AC& pt) const;

	CPoint_AC operator-() const;

	CPoint_AC& operator++();
	CPoint_AC& operator--();
	// Prefix;

	inline const CPoint_AC operator++(int) { const CPoint_AC returnVal(*this); ++*this; return returnVal; }
	inline const CPoint_AC operator--(int) { const CPoint_AC returnVal(*this); --*this; return returnVal; }
	// Postfix;

	// COMPARISON:

	bool operator>=(const CPoint_AC& pt) const;
	bool operator<=(const CPoint_AC& pt) const;
	bool operator>(const CPoint_AC& pt) const;
	bool operator<(const CPoint_AC& pt) const;
	inline bool operator==(const CPoint_AC& pt) const { return (*reinterpret_cast<const long*>(this) == *reinterpret_cast<const long*>(&pt)); }
	inline bool operator!=(const CPoint_AC& pt) const { return !(*this == pt); }

	// UTILITY:

	inline CPoint_AC AsPoint() const { return CPoint_AC(*this); }
		// Creates a CPoint_AC by casting the VCoordinate values to short values. In other
		// words, it throws away the top 16 bits of each long word value. If the values
		// don't fit within 16 bits you will get strange results (i.e. 32769 -> 1). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDPt()!
		// ADDED TO CPoint_AC FOR ORTHOGONALITY
	
	inline CPoint_AC ToPoint() const { return CPoint_AC(*this); }
		// Creates a CPoint_AC by limiting the VCoordinate values to +/- 32K and casting
		// them to short values. This produces reasonable results if the values were
		// more than 16 bits (i.e. 32769 -> 32767). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDPt()!
		// ADDED TO CPoint_AC FOR ORTHOGONALITY
	
	inline void	Clear( ) { *(long*)this = 0; }
	inline void	Set(short iH, short iV) { v = iV; h = iH; }
	void Offset(short iH, short iV);

	void ConstrainTo(const CRect_AC& rt);
		// If the CPoint_AC is not already inside the rectangle move the CPoint_AC to the nearest
        // edge.
    
    // CONSTANTS:
    
public:    
    static const CPoint_AC kZeroPt;
};


#endif
