//----------------------------------------------------------------------------------------
// CRect_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CRect_AC__
#define __CRect_AC__

// ACS

#ifndef __Imaging_AC__
#include "Imaging_AC.h"
#endif

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

// ANSI

#if defined(qCoreDebug) && !defined(_STDIO)
#include <stdio.h>
#endif

//----------------------------------------------------------------------------------------
// Types
//----------------------------------------------------------------------------------------

class CPoint_AC;
class CRect_AC;

//--------------------------------------------------------------------------------
// Selector operator indicator enumeration. Which CPoint_AC do we want
// to select, top-left or bottom-right?
//--------------------------------------------------------------------------------
enum PointSelector
{
	topLeft,
	botRight
};


//--------------------------------------------------------------------------------
// CRect_AC: is a toolbox compatible class for the toolbox struct Rect. It is bitwise
// compatible with the toolbox struct because it contains no virtual functions.
//--------------------------------------------------------------------------------
class EXPORT_AC CRect_AC : public Rect
{
public:
	// CONSTRUCTORS:

	inline CRect_AC() { Clear(); }
	inline CRect_AC(short iLeft, short iTop, short iRight, short iBottom) { top = iTop; left = iLeft; bottom = iBottom; right = iRight; }
	inline CRect_AC(const CPoint_AC& topLeftPt, const CPoint_AC& botRightPt) { *(CPoint_AC*)&top = topLeftPt; *(CPoint_AC*)&bottom = botRightPt; }	
#if qPowerPC
	inline CRect_AC(const CRect_AC& rect) { *(double*)this = *(const double*)&rect; }
		// cheat, we know that we're eight bytes long
#endif
	inline CRect_AC(const Rect& rect) { *this = *(const CRect_AC*)&rect; }

#if qPowerPC
	inline CRect_AC& operator=(const CRect_AC& rect) { *(double*)this = *(const double*)&rect; return *this; }
		// cheat, we know that we're eight bytes long
#endif
		
	// SELECTION:
	
	inline CPoint_AC& operator[](PointSelector sel) { return (sel == topLeft) ? (*((CPoint_AC *) &top)) : (*((CPoint_AC *) &bottom)); }
	inline const CPoint_AC& operator[](PointSelector sel) const { return (sel == topLeft) ? (*((const CPoint_AC *) &top)) : (*((const CPoint_AC *) &bottom)); }

	// CONVERSION:
	
#if qCoreDebug
	inline operator const char*() const
	{
		char textRect[64];	
		sprintf (textRect, "CRect_AC(%d, %d, %d, %d)", left, top, right, bottom);	
		return CChar63_AC(textRect);
	}
	// For converting CRect_AC to a textual representation of a CRect_AC. "CRect_AC(left,top,right,bottom)".
	
#endif
	
	inline operator Rect*() { return this; }
	inline operator const Rect*() const { return this; }

	// ARITHMETIC:

	CRect_AC& operator+=(const CRect_AC& rt);
	CRect_AC& operator-=(const CRect_AC& rt);
	CRect_AC& operator*=(const CRect_AC& rt);
	CRect_AC& operator/=(const CRect_AC& rt);
	CRect_AC operator+(const CRect_AC& rt) const;
	CRect_AC operator-(const CRect_AC& rt) const;
	CRect_AC operator*(const CRect_AC& rt) const;
	CRect_AC operator/(const CRect_AC& rt) const;

	CRect_AC& operator+=(const CPoint_AC& pt);
	CRect_AC& operator-=(const CPoint_AC& pt);
	CRect_AC& operator*=(const CPoint_AC& pt);
	CRect_AC& operator/=(const CPoint_AC& pt);
	CRect_AC operator+(const CPoint_AC& pt) const;
	CRect_AC operator-(const CPoint_AC& pt) const;
	CRect_AC operator*(const CPoint_AC& pt) const;
	CRect_AC operator/(const CPoint_AC& pt) const;
	
	CRect_AC operator-() const;

	// COMPARISON:

	bool operator==(const CRect_AC& rt) const;
	inline bool operator!=(const CRect_AC& rt) const { return !(*this == rt); }

	// UTILITY:

	CRect_AC& operator&=(const CRect_AC& rt);
	CRect_AC& operator|=(const CRect_AC& rt);
	CRect_AC operator&(const CRect_AC& rt) const;
	CRect_AC operator|(const CRect_AC& rt) const;
	// Two simple area operators, the intersection & (bitwise and in C++) and the
	// union | (bitwise or in C++). The definition of union here is to return a CRect_AC
	// that exactly encloses its operands.
		
	inline CRect_AC AsRect() const { return CRect_AC(*this); }
		// Creates a CRect_AC by casting the VCoordinate values to short values. In other
		// words, it throws away the top 16 bits of each long word value. If the values
		// don't fit within 16 bits you will get strange results (i.e. 32769 -> 1). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDRect()!
		// ADDED TO CRect_AC FOR ORTHOGONALITY

	inline CRect_AC ToRect() const { return CRect_AC(*this); }
		// Creates a CRect_AC by limiting the VCoordinate values to +/- 32K and casting
		// them to short values. This produces reasonable results if the values were
		// more than 16 bits (i.e. 32769 -> 32767). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDRect()!
		// ADDED TO CRect_AC FOR ORTHOGONALITY
	
#if qPowerPC
	inline void Clear() { *(double*)this = 0; }
		// cheat, we know that we're eight bytes long
#else
	inline void Clear() { *(long*)&top = *(long*)&bottom = 0; }
#endif

	inline void Set(short iLeft, short iTop, short iRight, short iBottom) { top = iTop; left = iLeft; bottom = iBottom; right = iRight; }
	void Set(const CPoint_AC& topLeftPt, short width, short height);
	inline void Set(const CPoint_AC& topLeftPt, const CPoint_AC& botRightPt) { *(CPoint_AC*)&top = topLeftPt; *(CPoint_AC*)&bottom = botRightPt; }	

	void Offset(short iH, short iV);
	inline void Offset(const CPoint_AC& delta) { *this+= delta; }

	void Inset(short iH, short iV);
	void Inset(const CPoint_AC& delta);
	void Inset(const CRect_AC& delta);

	bool IsEmpty() const;
	inline bool Empty() const { return IsEmpty(); }
	// Return true if the VRect is empty (right - left = 0 and bottom - top = 0)
	
	bool IsValid() const;
	inline bool Valid() const { return IsValid(); }
	// Returns true if a valid rectangle (left < right and top < bottom).
		
	void Validate();
	// Validate rectagle by switching the points if right-bottom is above or to the
    // left of left-top.
		
	short GetLength(VHSelect sel) const;
	// Returns the length of a CRect_AC in a given dimension.
	
	CPoint_AC GetSize() const;
	// Returns the size of a CRect_AC in both dimensions. (a CPoint_AC)
	
	bool Contains(const CPoint_AC& pt) const;
	bool Contains(const CRect_AC& rt) const;
	// The Contains method takes either a CPoint_AC or a CRect_AC and returns true if the operand
	// is inside of the CRect_AC the method is applied to.

protected:
	inline short Min(const short a, const short b) const { return a < b ? a : b; }
	inline short Max(const short a, const short b) const { return a > b ? a : b; }
	inline short MinMax(short MinVal, short expression, short MaxVal) const { return Min(Max(expression, MinVal), MaxVal); }
		
	// CONSTANTS:
	
public:
	static const CRect_AC kZeroRect;

};

#endif
