// CRegion_AC.h
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.

#ifndef __CRegion_AC__
#define __CRegion_AC__

// ACS

#ifndef __Imaging_AC__
#include "Imaging_AC.h"
#endif

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __CLongGeometry_AC__
#include "CLongGeometry_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

// ANSI

#if defined(qCoreDebug) && !defined(_STDIO)
#include <stdio.h>
#endif

//----------------------------------------------------------------------------------------
// Types
//----------------------------------------------------------------------------------------

class CPoint_AC;
class CRect_AC;

//--------------------------------------------------------------------------------
// CRegion_AC: is a toolbox compatible class for the toolbox struct RgnHandle.
// 				They don't "own" the region handle, though they'll make one for you
//				If you want a version that "owns" the handle, use a CTempRegion_AC
//--------------------------------------------------------------------------------
class EXPORT_AC CRegion_AC
{
public:
	// CONSTRUCTORS:

	inline CRegion_AC() { MakeRgnHandle(); }
	// Allocates an empty region

	inline CRegion_AC(RgnHandle itsData) : fData(itsData) {}
	// adopts itsData

	CRegion_AC(short iLeft, short iTop, short iRight, short iBottom);
	CRegion_AC(const CPoint_AC& topLeftPt, const CPoint_AC& botRightPt);
	CRegion_AC(const CRect_AC& rect);
	CRegion_AC(const CRegion_AC& rgn);
	~CRegion_AC() { } // do nothing, just abandon the region to fate
	
	CRegion_AC& operator=(const CRegion_AC& rgn);
	inline CRegion_AC& operator=(const RgnHandle rgn) { *this = CRegion_AC(rgn); return *this; }
	// assignment of the fData or RgnHandle, NOT copying the region!
	
	inline void MakeRgnHandle() { fData = MakeNewRgn(); }

	// SELECTION:
	
	// CONVERSION:
	
	inline operator RgnHandle() const { return fData; }

	// ARITHMETIC:

	CRegion_AC& operator+=(const CRegion_AC& rt);
	CRegion_AC& operator-=(const CRegion_AC& rt);
	CRegion_AC operator+(const CRegion_AC& rt) const;
	CRegion_AC operator-(const CRegion_AC& rt) const;
	
	CRegion_AC& operator+=(const CPoint_AC& rt);
	CRegion_AC& operator-=(const CPoint_AC& rt);
	CRegion_AC operator+(const CPoint_AC& rt) const;
	CRegion_AC operator-(const CPoint_AC& rt) const;

	
private:
	// The following have no semantic meaning, and are therefore unavailable
	CRegion_AC& operator*=(const CRegion_AC& rt);
	CRegion_AC& operator/=(const CRegion_AC& rt);
	CRegion_AC operator*(const CRegion_AC& rt) const;
	CRegion_AC operator/(const CRegion_AC& rt) const;
	CRegion_AC operator-() const;

public:
	// COMPARISON:

	inline bool operator==(const CRegion_AC& rt) const { return ::EqualRgn(fData, rt); }
	inline bool operator!=(const CRegion_AC& rt) const { return !(*this == rt); }

	// UTILITY:
	CRegion_AC& operator^=(const CRegion_AC& rt);
	CRegion_AC operator^(const CRegion_AC& rt) const;
	// Exclusive or (XorRgn)

	CRegion_AC& operator&=(const CRegion_AC& rt);
	CRegion_AC& operator|=(const CRegion_AC& rt);
	CRegion_AC& operator|=(const CPoint_AC& rt);
	CRegion_AC& operator|=(const CLongRect_AC& rt);
	CRegion_AC operator&(const CRegion_AC& rt) const;
	CRegion_AC operator|(const CRegion_AC& rt) const;
	CRegion_AC operator|(const CPoint_AC& rt) const;
	CRegion_AC operator|(const CLongRect_AC& rt) const;
	// Two simple area operators, the intersection & (bitwise and in C++) and the
	// union | (bitwise or in C++). The definition of union here is to return a CRegion_AC
	// that exactly encloses its operands.
	
	inline CRect_AC Bounds() const { return CRect_AC((*fData)->rgnBBox); }
	
	inline void Clear() { ::SetEmptyRgn(fData); }

	inline void Set(const CRect_AC& bounds) { ::RectRgn(fData, &bounds); }
	inline void Set(short iLeft, short iTop, short iRight, short iBottom) { Set(CRect_AC(iLeft, iTop, iRight, iBottom)); }
	inline void Set(const CPoint_AC& topLeftPt, const CPoint_AC& botRightPt) { Set(CRect_AC(topLeftPt, botRightPt)); }	
	inline void Set(const RgnHandle rhs);
	
	inline void Offset(const CPoint_AC& delta) { ::OffsetRgn(fData, delta.h, delta.v); }
	inline void Offset(short iH, short iV) { Offset(CPoint_AC(iH, iV)); }

	inline void Inset(const CPoint_AC& delta);
	inline void Inset(short iH, short iV) { Inset(CPoint_AC(iH, iV)); }
	inline void Inset(const CRect_AC& delta);

	inline bool IsEmpty() const { return ::EmptyRgn(fData); }
	// Return true if the Region is empty
	
	bool IsValid() const;
		
	void Validate();
	// Validate Region boundsrect by switching the points if right-bottom is above or to the
    // left of left-top.
		
	short GetLength(VHSelect sel) const;
	// Returns the length of the region boundsrect in a given dimension.
	
	CPoint_AC GetSize() const;
	// Returns the size of a CRect_AC in both dimensions. (a CPoint_AC)
	
	bool Contains(const CPoint_AC& pt) const;
	bool Contains(const CRect_AC& rt) const;
	bool Contains(const CRegion_AC& rt) const;
	inline bool Contains(const RgnHandle& rt) const { return Contains(RgnHandle(rt)); }
	// The Contains method takes either a CPoint_AC or a CRect_AC and returns true if the operand
	// is inside of the Region the method is applied to.
	
	//------------------------------------------------------------------------------------
	// Static scoped functions
	//------------------------------------------------------------------------------------
	static RgnHandle MakeNewRgn();
	// returns a new empty region handle either from the cache or freshly made

	static RgnHandle DisposeIfRgnHandle(RgnHandle itsData);
	// if non-NULL, either disposes itsData or returns it to the cache as appropriate
	
	//------------------------------------------------------------------------------------
	// data member[s]
	//------------------------------------------------------------------------------------
protected:
	RgnHandle	fData;
	
	//------------------------------------------------------------------------------------
	// static data members
	//------------------------------------------------------------------------------------
protected:
	static RgnHandle fCachedRgn;				// the single level region cache
};


#endif
