//----------------------------------------------------------------------------------------
// CTempRegion_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTempRegion_AC__
#define __CTempRegion_AC__

// ACS

#ifndef __Imaging_AC__
#include "Imaging_AC.h"
#endif

#ifndef __CRegion_AC__
#include "CRegion_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Manage a temporary region
//----------------------------------------------------------------------------------------

class EXPORT_AC CTempRegion_AC : public CRegion_AC
{
public:
	CTempRegion_AC() { }
	// Allocates an empty region

	inline CTempRegion_AC(RgnHandle itsData) : CRegion_AC(itsData) { }
	// adopts itsData

	inline CTempRegion_AC(short iLeft, short iTop, short iRight, short iBottom) : CRegion_AC(iLeft, iTop, iRight, iBottom) { }
	inline CTempRegion_AC(const CPoint_AC& topLeftPt, const CPoint_AC& botRightPt) : CRegion_AC(topLeftPt, botRightPt) { }
	inline CTempRegion_AC(const CRect_AC& rect) : CRegion_AC(rect) { }
	inline CTempRegion_AC(const CTempRegion_AC& rgn) : CRegion_AC(rgn) { }

	inline ~CTempRegion_AC() { DisposeIfRgnHandle(fData); }
	// Automatically Deallocates the fRgnHandle

	CTempRegion_AC& operator=(const CTempRegion_AC& rgn);
	CTempRegion_AC& operator=(const CRegion_AC& rgn);
	inline CTempRegion_AC& operator=(const RgnHandle rgn) { *this = CRegion_AC(rgn); return *this; }
	// assignment of the fData or RgnHandle, NOT copying the region!

	inline RgnHandle get() const { return fData; }

	inline RgnHandle release() { RgnHandle returnVal = fData; fData = NULL; return returnVal; }
		// Clears the owned reference to the data when returning it so it won't get released.

	inline void reset(RgnHandle p = NULL) { if (fData != p) { DisposeIfRgnHandle(get()); fData = p; } }

	
};

#endif // __SRegion__