//----------------------------------------------------------------------------------------
// CTemplatePoint_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


// Notes: places optimizations removed:
// copy constuctor
// assignment operator
// clear

#ifndef __CTemplatePoint_AC__
#define __CTemplatePoint_AC__

// ACS Imaging

#ifndef __Imaging_AC__
#include "Imaging_AC.h"
#endif

// ACS

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------
enum
{
	kMaxQDCoord = SHRT_MAX - 1	// -1 to minimize signed overflow problems
};

//----------------------------------------------------------------------------------------
// Types
//----------------------------------------------------------------------------------------
template <class T> class CTemplatePoint_AC;
template <class T> class CTemplateRect_AC;

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC: is a toolbox compatible class for the old struct CTemplatePoint_AC. It is bitwise
// compatible with the old struct because it contains no virtual functions.
//----------------------------------------------------------------------------------------

template <class T>
class CTemplatePoint_AC
{
public:

	typedef T Coordinate;

	Coordinate v;
	Coordinate h;

	// CONSTRUCTORS:

	CTemplatePoint_AC() 
	{ Clear(); }
	
	CTemplatePoint_AC(Coordinate iH, Coordinate iV) 
		: v(iV), h(iH) 
	{ }  
	
	CTemplatePoint_AC(const CPoint_AC& pt) 
	: v(pt.v), h(pt.h) 
	{ }
	
#if 0 // qPowerPC 

	// copy constructor
	CTemplatePoint_AC(const CTemplatePoint_AC<T>& pt) 
	{ *(double*)this = *(const double*)&pt; }

	CTemplatePoint_AC& operator=(const CTemplatePoint_AC<T>& pt) 
	{ *(double*)this = *(const double*)&pt; return *this; }
		// cheat, we know that we're eight bytes long
#else

	CTemplatePoint_AC(const CTemplatePoint_AC<T>& pt) 
		: v(pt.v), h(pt.h)
	{}

	CTemplatePoint_AC<T>& operator=(const CTemplatePoint_AC<T>& pt) 
	{ v = pt.v; h = pt.h; return *this; }

#endif
		
	// SELECTION:
	
	Coordinate& operator[](VHSelect sel) 
	{ return (sel == vSel) ? v : h; }
	
	const Coordinate& operator[](VHSelect sel) const 
	{ return (sel == vSel) ? v : h; }

	// CONVERSION:

#if qCoreDebug
		// Conversion operator, for converting CTemplatePoint_AC to a textual representation of a
        // CTemplatePoint_AC. "CTemplatePoint_AC(h,v)".
	operator const char*() const
	{
		char textPoint[64];
#if 0 
		sprintf (textPoint, "CTemplatePoint_AC(%d, %d)", h, v);
#endif
		return CChar63_AC(textPoint);
	}
#endif
	
	// ARITHMETIC:

	CTemplatePoint_AC<T>& operator+=(const CTemplatePoint_AC<T>& pt);
	CTemplatePoint_AC<T>& operator-=(const CTemplatePoint_AC<T>& pt);
	CTemplatePoint_AC<T>& operator*=(const CTemplatePoint_AC<T>& pt);
	CTemplatePoint_AC<T>& operator/=(const CTemplatePoint_AC<T>& pt);
	CTemplatePoint_AC<T> operator+(const CTemplatePoint_AC<T>& pt) const;
	CTemplatePoint_AC<T> operator-(const CTemplatePoint_AC<T>& pt) const;
	CTemplatePoint_AC<T> operator*(const CTemplatePoint_AC<T>& pt) const;
	CTemplatePoint_AC<T> operator/(const CTemplatePoint_AC<T>& pt) const;

	CTemplatePoint_AC<T> operator-() const;

	CTemplatePoint_AC<T>& operator++();
	CTemplatePoint_AC<T>& operator--();
	// Prefix;

	const CTemplatePoint_AC<T> operator++(int) 
	{ const CTemplatePoint_AC<T> returnVal(*this); ++*this; return returnVal; }
	const CTemplatePoint_AC<T> operator--(int) 
	{ const CTemplatePoint_AC<T> returnVal(*this); --*this; return returnVal; }
	// Postfix;

	// COMPARISON:

	bool operator>=(const CTemplatePoint_AC<T>& pt) const;
	bool operator<=(const CTemplatePoint_AC<T>& pt) const;
	bool operator>(const CTemplatePoint_AC<T>& pt) const;
	bool operator<(const CTemplatePoint_AC<T>& pt) const;
	bool operator==(const CTemplatePoint_AC<T>& pt) const;
	bool operator!=(const CTemplatePoint_AC<T>& pt) const { return !(*this == pt); }

	// UTILITY:

	inline CPoint_AC AsPoint() const { return CPoint_AC(static_cast<short>(h), static_cast<short>(v)); }
		// Creates a CPoint_AC by casting the Coordinate values to short values. In other
		// words, it throws away the top 16 bits of each long word value. If the values
		// don't fit within 16 bits you will get strange results (i.e. 32769 -> 1). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDPt()!
	
	CPoint_AC ToPoint() const;
		// Creates a CPoint_AC by limiting the Coordinate values to +/- 32K and casting
		// them to short values. This produces reasonable results if the values were
		// more than 16 bits (i.e. 32769 -> 32767). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDPt()!
	
#if 0 // qPowerPC

	void Clear() 
	{ *(double*)this = 0; }
		// cheat, we know that we're eight bytes long
#else
	void Clear( ) 
	{ v = h = 0; }
#endif

	void Set(Coordinate iH, Coordinate iV) 
	{ v = iV; h = iH; }
	
	void Offset(Coordinate iH, Coordinate iV);

	void ConstrainTo(const CTemplateRect_AC<T>& rt);
		// If the CTemplatePoint_AC is not already inside the rectangle move the CTemplatePoint_AC to the nearest
        // edge.
		
protected:

	Coordinate Min(const Coordinate a, const Coordinate b) const 
	{ return a < b ? a : b; }
	Coordinate Max(const Coordinate a, const Coordinate b) const
	{ return a > b ? a : b; }

	Coordinate MinMax(Coordinate MinVal, Coordinate expression, Coordinate MaxVal) const 
	{ return Min(Max(expression, MinVal), MaxVal); }
	// Returns the bounded minimum and maximum 

public:    
    static const CTemplatePoint_AC<T> kZeroPt;
};


#endif
