// CTemplateRect_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved.

#ifndef __CTemplateRect_AC__
#define __CTemplateRect_AC__

// ACS Imaging

#ifndef __Imaging_AC__
#include "Imaging_AC.h"
#endif

// ACS

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __CTemplatePoint_AC__
#include "CTemplatePoint_AC.h"
#endif

#if defined(qCoreDebug) && !defined(__CPascalString_AC__)
#include "CPascalString_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __LIMITS__
#include <Limits.h>
#endif


// Notes: places optimizations removed:
// 1. Clear.

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

// const short kMaxTemplate = SHRT_MAX - 1;	// -1 to minimize signed overflow problems
	
	
//----------------------------------------------------------------------------------------
// Types
//----------------------------------------------------------------------------------------
template <class T> class CTemplatePoint_AC;
template <class T> class CTemplateRect_AC;

//----------------------------------------------------------------------------------------
// CTemplateRect_AC is a toolbox compatible class for the old struct CTemplateRect_AC. It is bitwise compatible
// with the old struct because it contains no virtual functions.
//----------------------------------------------------------------------------------------

template <class T>
class CTemplateRect_AC
{
public:

	typedef T Coordinate;

	// CONSTRUCTORS:

	CTemplateRect_AC() 
	{ Clear(); }
	
	CTemplateRect_AC(	Coordinate iLeft, 
						Coordinate iTop, 
						Coordinate iRight, 
						Coordinate iBottom) 
						
		: top(iTop), left(iLeft), bottom(iBottom), right(iRight) 
	{}
	
	CTemplateRect_AC(	const CTemplatePoint_AC<T>& topLeftPt, 
						const CTemplatePoint_AC<T>& botRightPt) 
	{ 	*(CTemplatePoint_AC<T>*)&top = *(const CTemplatePoint_AC<T>*)&topLeftPt; 
		*(CTemplatePoint_AC<T>*)&bottom = *(const CTemplatePoint_AC<T>*)&botRightPt; }
	
	CTemplateRect_AC(const CRect_AC& aRect) 
	: top(aRect.top), left(aRect.left), bottom(aRect.bottom), right(aRect.right) 
	{ }
	
	CTemplateRect_AC(const Rect& aRect) 
	: top(aRect.top), left(aRect.left), bottom(aRect.bottom), right(aRect.right) 
	{ }

#if qPowerPC
	CTemplateRect_AC(const CTemplateRect_AC<T>& inRect) 
	{ 	*(CTemplatePoint_AC<T>*)&top = *(const CTemplatePoint_AC<T>*)&inRect.top; 
		*(CTemplatePoint_AC<T>*)&bottom = *(const CTemplatePoint_AC<T>*)&inRect.bottom; }

	CTemplateRect_AC& operator=(const CTemplateRect_AC<T>& inRect) 
	{ 	*(CTemplatePoint_AC<T>*)&top = *(const CTemplatePoint_AC<T>*)&inRect.top; 
		*(CTemplatePoint_AC<T>*)&bottom = *(const CTemplatePoint_AC<T>*)&inRect.bottom; 
		return *this; }

#else

	CTemplateRect_AC(const CTemplateRect_AC<T>& inRect) 
		: top(inRect.top), left(inRect.left), right(inRect.right), bottom(inRect.bottom)
	{}

	CTemplateRect_AC<T>& operator=(const CTemplateRect_AC<T>& inRect) 
	{ 	top = inRect.top; left = inRect.left; right = inRect.right; bottom = inRect.bottom; 
		return *this; }

#endif		
	
	// SELECTION:
	
	CTemplatePoint_AC<T>& operator[](PointSelector sel) 
	{ return (sel == topLeft) ? (*((CTemplatePoint_AC<T> *) &top)) : (*((CTemplatePoint_AC<T> *) &bottom)); }
	
	const CTemplatePoint_AC<T>& operator[](PointSelector sel) const 
	{ return (sel == topLeft) ? (*((const CTemplatePoint_AC<T> *) &top)) : (*((const CTemplatePoint_AC<T> *) &bottom)); }

	// CONVERSION:
	
#if qCoreDebug
		// Conversion operator, for converting CTemplateRect_AC to a textual representation of a
        // CTemplateRect_AC. "CTemplateRect_AC(top,righ,bottom,left)".
	operator const char*() const
	{
		char textRect[64];
#if 0 
		sprintf (textRect, "CTemplateRect_AC(%ld, %ld, %ld, %ld)", left, top, right, bottom);
#endif						
		return CChar63_AC(textRect);
	}
#endif	

	// ARITHMETIC:

	CTemplateRect_AC<T>& operator+=(const CTemplateRect_AC<T>& rt);
	CTemplateRect_AC<T>& operator-=(const CTemplateRect_AC<T>& rt);
	CTemplateRect_AC<T>& operator*=(const CTemplateRect_AC<T>& rt);
	CTemplateRect_AC<T>& operator/=(const CTemplateRect_AC<T>& rt);
	CTemplateRect_AC<T> operator+(const CTemplateRect_AC<T>& rt) const;
	CTemplateRect_AC<T> operator-(const CTemplateRect_AC<T>& rt) const;
	CTemplateRect_AC<T> operator*(const CTemplateRect_AC<T>& rt) const;
	CTemplateRect_AC<T> operator/(const CTemplateRect_AC<T>& rt) const;

	CTemplateRect_AC<T>& operator+=(const CTemplatePoint_AC<T>& pt);
	CTemplateRect_AC<T>& operator-=(const CTemplatePoint_AC<T>& pt);
	CTemplateRect_AC<T>& operator*=(const CTemplatePoint_AC<T>& pt);
	CTemplateRect_AC<T>& operator/=(const CTemplatePoint_AC<T>& pt);
	CTemplateRect_AC<T> operator+(const CTemplatePoint_AC<T>& pt) const;
	CTemplateRect_AC<T> operator-(const CTemplatePoint_AC<T>& pt) const;
	CTemplateRect_AC<T> operator*(const CTemplatePoint_AC<T>& pt) const;
	CTemplateRect_AC<T> operator/(const CTemplatePoint_AC<T>& pt) const;
	
	CTemplateRect_AC<T> operator-() const;

	// COMPARISON:

	bool operator==(const CTemplateRect_AC<T>& rt) const;
	
	bool operator!=(const CTemplateRect_AC<T>& rt) const 
	{ return !(*this == rt); }

	// UTILITY:

	CTemplateRect_AC<T>& operator &=(const CTemplateRect_AC<T>& rt);
	CTemplateRect_AC<T>& operator |=(const CTemplateRect_AC<T>& rt);
	CTemplateRect_AC<T> operator&(const CTemplateRect_AC<T>& rt) const;
	CTemplateRect_AC<T> operator|(const CTemplateRect_AC<T>& rt) const;
	// Two simple area operators, the intersection & (bitwise and in C++) and the
	// union | (bitwise or in C++). The definition of union here is to return a CRect_AC
	// that exactly encloses its operands.

	inline CRect_AC AsRect() const 
	{ return CRect_AC(static_cast<short>(left), 
						static_cast<short>(top), 
						static_cast<short>(right), 
						static_cast<short>(bottom)); }
		// Creates a CRect_AC by casting the Coordinate values to short values. In other
		// words, it throws away the top 16 bits of each long word value. If the values
		// don't fit within 16 bits you will get strange results (i.e. 32769 -> 1). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDRect()!

	CRect_AC ToRect() const;
		// Creates a CRect_AC by limiting the Coordinate values to +/- 32K and casting
		// them to short values. This produces reasonable results if the values were
		// more than 16 bits (i.e. 32769 -> 32767). 
		// Danger: Don't use this in TView methods for converting from view space to QD
		// space. Instead, use TView::ViewToQDRect()!
	
#if 0 // qPowerPC

	void Clear() 
	{ *(double*)&top = *(double*)&bottom = 0; }
#else
	void Clear() 
	{ top = left = bottom = right = 0; }
#endif

	void Set(Coordinate iLeft, Coordinate iTop, Coordinate iRight, Coordinate iBottom) 
	{ top = iTop; left = iLeft; bottom = iBottom; right = iRight; }
	
	void Set( const CTemplatePoint_AC<T>&, Coordinate width, Coordinate height );
	
	void Set(const CTemplatePoint_AC<T>& topLeftPt, const CTemplatePoint_AC<T>& botRightPt) 
	{ *(CTemplatePoint_AC<T>*)&top = topLeftPt; *(CTemplatePoint_AC<T>*)&bottom = botRightPt; }	

	void Offset( Coordinate iH, Coordinate iV );
	
	void Offset(const CPoint_AC& delta) 
	{ *this += delta; }

	void Inset( Coordinate iH, Coordinate iV );
	void Inset(const CTemplatePoint_AC<T>& delta);
	void Inset(const CTemplateRect_AC<T>& delta);

	bool IsEmpty() const;
	bool Empty() const 
	{ return IsEmpty(); }
	// Return true if the CTemplateRect_AC is empty (right - left = 0 and bottom - top = 0)
	
	bool IsValid() const;
	bool Valid() const 
	{ return IsValid(); }
	// Returns true if a valid rectangle (left < right and top < bottom).
		
	void Validate();
	// Validate rectagle by switching the points if right-bottom is above or to the
    // left of left-top.
		
	Coordinate GetLength(VHSelect sel) const;
	// Returns the length of a CTemplateRect_AC in a given dimension.
		
	CTemplatePoint_AC<T> GetSize() const;
	// Returns the size of a CTemplateRect_AC in a CTemplatePoint_AC<T>.
	
	bool Contains(const CTemplatePoint_AC<T>& pt) const;
	bool Contains(const CTemplateRect_AC<T>& rt) const;

protected:
	Coordinate Min(const Coordinate a, const Coordinate b) const 
	{ return a < b ? a : b; }
	Coordinate Max(const Coordinate a, const Coordinate b) const 
	{ return a > b ? a : b; }
	Coordinate MinMax(Coordinate MinVal, Coordinate expression, Coordinate MaxVal) const 
	{ return Min(Max(expression, MinVal), MaxVal); }
	// Returns the bounded minimum and maximum 

	// CONSTANTS:
	
public:
	static const CTemplateRect_AC<T> kZeroRect;

	Coordinate top;
	Coordinate left;
	Coordinate bottom;
	Coordinate right;
};

#endif
