//----------------------------------------------------------------------------------------
// CGDIterator_AC.cp 
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1996 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

// ACS

#ifndef __ImagingUtils_AC__
#include "ImagingUtils_AC.h"
#endif

#ifndef __Configuration_AC__
#include "Configuration_AC.h"
#endif

//========================================================================================
// CLASS CGDIterator_AC
//========================================================================================

RgnHandle CGDIterator_AC::fgSavedClipping = ::NewRgn();

//----------------------------------------------------------------------------------------
// CGDIterator_AC::CGDIterator_AC
//----------------------------------------------------------------------------------------

CGDIterator_AC::CGDIterator_AC() :
	fIteratedYet(false),
	fCurrentGDHandle(NULL)
{
	Setup();
}

//----------------------------------------------------------------------------------------
// CGDIterator_AC::~CGDIterator_AC
//----------------------------------------------------------------------------------------

CGDIterator_AC::~CGDIterator_AC()
{
	Cleanup();
}

//----------------------------------------------------------------------------------------
// CGDIterator_AC::Setup
//----------------------------------------------------------------------------------------

void CGDIterator_AC::Setup()
{
	// Save the current clipping so we can put it back in Cleanup
	// and use it as we iterate over each device.

	::GetClip(fgSavedClipping);
	
	fIteratedYet = false;
	fCurrentGDHandle = NULL;
}

//----------------------------------------------------------------------------------------
// CGDIterator_AC::NextDevice
//----------------------------------------------------------------------------------------

bool CGDIterator_AC::NextDevice(bool& deep)
{
	// If there's another device to draw on, get its GDHandle and deepness,
	// and return true. Otherwise, return false.
	
	bool	gotNextDevice;

	if (::HasColorQD_AC())
	{
		fCurrentGDHandle = FindNextValidDevice(fCurrentGDHandle, deep);
		gotNextDevice = (fCurrentGDHandle != NULL);
	}
	else
	{
		deep = false;
		gotNextDevice = ! fIteratedYet;
		
		fIteratedYet = true;
	}
	
	return gotNextDevice;
}

//----------------------------------------------------------------------------------------
// CGDIterator_AC::Cleanup
//----------------------------------------------------------------------------------------

void CGDIterator_AC::Cleanup()
{
	// Restore clipping to what it was when we were constructed.

	::SetClip(fgSavedClipping);
}

//----------------------------------------------------------------------------------------
// CGDIterator_AC::ClipFurtherToCurrentDevice
//----------------------------------------------------------------------------------------

void CGDIterator_AC::ClipFurtherToCurrentDevice()
{
	// Clip further to prevent drawing outside the current device.

	ClipFurtherToDevice(fCurrentGDHandle);
}

//----------------------------------------------------------------------------------------
// CGDIterator_AC::FindNextValidDevice
//----------------------------------------------------------------------------------------

GDHandle CGDIterator_AC::FindNextValidDevice(GDHandle currentDevice, bool& deep)
{
	// Return the GDHandle of the next valid device, if any, and indicate
	// whether it is "deep".
	// As a designed side-effect, when we find a next device, we restore
	// the original clipping and then re-clip further to this next device
	// so no drawing occurs on other devices.

	GDHandle	deviceToTest;
	GDHandle	deviceFound = NULL;
	
	deep = false;
	
	if (currentDevice == NULL)
		deviceToTest = ::GetDeviceList();
	else
		deviceToTest = ::GetNextDevice(currentDevice);

	while ((deviceToTest != NULL) && (deviceFound == NULL))
	{
		bool	isValidDevice = false;
		
		if (::TestDeviceAttribute(deviceToTest, screenDevice))
			if (::TestDeviceAttribute(deviceToTest, screenActive))
			{
				isValidDevice = true;
				
				SInt16	pixelDepth = (**((**deviceToTest).gdPMap)).pixelSize;
				bool	isColor = ::TestDeviceAttribute(deviceToTest, gdDevType);
				
				deep = (pixelDepth >= kDeepColorBits) || ((pixelDepth >= kDeepGrayscaleBits) && ! isColor);
				
				deviceFound = deviceToTest;
			}

		if (! isValidDevice)
			deviceToTest = ::GetNextDevice(deviceToTest);
	}
	
	if (deviceFound != NULL)
	{
		::SetClip(fgSavedClipping);
		ClipFurtherToDevice(deviceFound);
	}

	return deviceFound;
}

//----------------------------------------------------------------------------------------
// CGDIterator_AC::ClipFurtherToDevice
//----------------------------------------------------------------------------------------

void CGDIterator_AC::ClipFurtherToDevice(GDHandle aDevice)
{
	// Clip further to the specified device so no drawing will occur
	// on other devices.

	if (aDevice != NULL)
	{
		CRect_AC	deviceRect((**aDevice).gdRect);
		
		::GlobalToLocal(&deviceRect[topLeft]);
		::GlobalToLocal(&deviceRect[botRight]);
		
		::ClipFurtherTo_AC(deviceRect);
	}
}
