//----------------------------------------------------------------------------------------
// CRGBColor_AC.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CRGBColor_AC__
#include "CRGBColor_AC.h"
#endif

// Toolbox

#ifndef __LIMITS__
#include <Limits.h>
#endif



//========================================================================================
// CLASS CRGBColor_AC
//========================================================================================

const CRGBColor_AC CRGBColor_AC::kRGBBlack(0, 0, 0);
const CRGBColor_AC CRGBColor_AC::kRGBWhite(USHRT_MAX, USHRT_MAX, USHRT_MAX);
const CRGBColor_AC CRGBColor_AC::kRGBRed(USHRT_MAX, 0, 0);
const CRGBColor_AC CRGBColor_AC::kRGBGreen(0, USHRT_MAX, 0);
const CRGBColor_AC CRGBColor_AC::kRGBBlue(0, 0, USHRT_MAX);

//----------------------------------------------------------------------------------------
//		 Set
//----------------------------------------------------------------------------------------
	
void CRGBColor_AC::Set(const RGBColor& color)
{
	*(RGBColor*)this = *(RGBColor*)&color;
}

//----------------------------------------------------------------------------------------
// Comparison operators for CRGBColor_AC
//----------------------------------------------------------------------------------------
	
bool CRGBColor_AC::operator ==(const CRGBColor_AC& color) const
{
	return red == color.red && green == color.green && blue == color.blue;
}

CRGBColor_AC& CRGBColor_AC::operator +=(const CRGBColor_AC& color)
{
	red += color.red;
	green += color.green;
	blue += color.blue;
	
	return *this;
}

CRGBColor_AC& CRGBColor_AC::operator -=(const CRGBColor_AC& color)
{
	red -= color.red;
	green -= color.green;
	blue -= color.blue;
	
	return *this;
}

CRGBColor_AC& CRGBColor_AC::operator *=(const CRGBColor_AC& color)
{
	red *= color.red;
	green *= color.green;
	blue *= color.blue;
	
	return *this;
}

CRGBColor_AC& CRGBColor_AC::operator /=(const CRGBColor_AC& color)
{
	red /= color.red;
	green /= color.green;
	blue /= color.blue;
	
	return *this;
}
	
//----------------------------------------------------------------------------------------
// Arithmetic, by another color for CRGBColor_AC
//----------------------------------------------------------------------------------------
	
CRGBColor_AC CRGBColor_AC::operator +(const CRGBColor_AC& color) const
{
	return CRGBColor_AC(*this) += color;
}

CRGBColor_AC CRGBColor_AC::operator -(const CRGBColor_AC& color) const
{
	return CRGBColor_AC(*this) -= color;
}

CRGBColor_AC CRGBColor_AC::operator *(const CRGBColor_AC& color) const
{
	return CRGBColor_AC(*this) *= color;
}

CRGBColor_AC CRGBColor_AC::operator /(const CRGBColor_AC& color) const
{
	return CRGBColor_AC(*this) /= color;
}


//----------------------------------------------------------------------------------------
// Arithmatic, by scalar for CRGBColor_AC
//----------------------------------------------------------------------------------------

CRGBColor_AC& CRGBColor_AC::operator +=(const unsigned short aScalar)
{
	red += aScalar;
	green += aScalar;
	blue += aScalar;
	
	return *this;
}

CRGBColor_AC& CRGBColor_AC::operator -=(const unsigned short aScalar)
{
	red -= aScalar;
	green -= aScalar;
	blue -= aScalar;
	
	return *this;
}

CRGBColor_AC& CRGBColor_AC::operator *=(const unsigned short aScalar)
{
	red *= aScalar;
	green *= aScalar;
	blue *= aScalar;
	
	return *this;
}

CRGBColor_AC& CRGBColor_AC::operator /=(const unsigned short aScalar)
{
	red /= aScalar;
	green /= aScalar;
	blue /= aScalar;
	
	return *this;
}

CRGBColor_AC CRGBColor_AC::operator +(const unsigned short aScalar) const
{
	return CRGBColor_AC(*this) += aScalar;
}

CRGBColor_AC CRGBColor_AC::operator -(const unsigned short aScalar) const
{
	return CRGBColor_AC(*this) -= aScalar;
}

CRGBColor_AC CRGBColor_AC::operator *(const unsigned short aScalar) const
{
	return CRGBColor_AC(*this) *= aScalar;
}

CRGBColor_AC CRGBColor_AC::operator /(const unsigned short aScalar) const
{
	return CRGBColor_AC(*this) /= aScalar;
}

//----------------------------------------------------------------------------------------
// IsColorPort: Returns true if the GrafPort is a Color QD GrafPort test is:
// IsColorPort = CGrafPtr(port)->portversion == 0xC000
//----------------------------------------------------------------------------------------


bool CRGBColor_AC::IsColorPort()
{
	return qd.thePort ? ((((CGrafPtr)qd.thePort)->portVersion & static_cast<short>(0xC000))) == static_cast<short>(0xC000) : false;
}

void CRGBColor_AC::GetForeColor()
{
	::GetForeColor(this);
}

void CRGBColor_AC::SetForeColor() const
{
	// Try to avoid calling RGBFore/BackColor unnecessarally
	if (!IsColorPort() || (((CRGBColor_AC &) ((CGrafPtr) qd.thePort)->rgbFgColor) != *this))
		::RGBForeColor(this);
}

void CRGBColor_AC::GetBackColor()
{
	::GetBackColor(this);
}

void CRGBColor_AC::SetBackColor() const
{
	// Try to avoid calling RGBFore/BackColor unnecessarally
	if (!IsColorPort() || (((CRGBColor_AC &) ((CGrafPtr) qd.thePort)->rgbBkColor) != *this))
		::RGBBackColor(this);
}