//----------------------------------------------------------------------------------------
// CRect_AC.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif



//========================================================================================
// CLASS CRect_AC
//========================================================================================

const CRect_AC CRect_AC::kZeroRect(0, 0, 0, 0);	// Constructor keeps this from getting dead-stripped

//----------------------------------------------------------------------------------------
// CRect_AC::operator+=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator+=(const CRect_AC& rt)
{
	top += rt.top;
	left += rt.left;
	bottom += rt.bottom;
	right += rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator-=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator-=(const CRect_AC& rt)
{
	top -= rt.top;
	left -= rt.left;
	bottom -= rt.bottom;
	right -= rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator*=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator*=(const CRect_AC& rt)
{
	top *= rt.top;
	left *= rt.left;
	bottom *= rt.bottom;
	right *= rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator-=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator/=(const CRect_AC& rt)
{
	top /= rt.top;
	left /= rt.left;
	bottom /= rt.bottom;
	right /= rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator+:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator+(const CRect_AC& rt) const
{ return CRect_AC(*this) += rt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator-:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator-(const CRect_AC& rt) const
{ return CRect_AC(*this) -= rt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator*:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator*(const CRect_AC& rt) const
{ return CRect_AC(*this) *= rt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator-:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator/(const CRect_AC& rt) const
{ return CRect_AC(*this) /= rt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator+=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator+=(const CPoint_AC& pt)
{
	top += pt.v;
	bottom += pt.v;
	left += pt.h;
	right += pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator-=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator-=(const CPoint_AC& pt)
{
	top -= pt.v;
	bottom -= pt.v;
	left -= pt.h;
	right -= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator*=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator*=(const CPoint_AC& pt)
{
	top *= pt.v;
	bottom *= pt.v;
	left *= pt.h;
	right *= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator/=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator/=(const CPoint_AC& pt)
{
	top /= pt.v;
	bottom /= pt.v;
	left /= pt.h;
	right /= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator+:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator+(const CPoint_AC& pt) const
{ return CRect_AC(*this) += pt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator-:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator-(const CPoint_AC& pt) const
{ return CRect_AC(*this) -= pt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator*:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator*(const CPoint_AC& pt) const
{ return CRect_AC(*this) *= pt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator/:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator/(const CPoint_AC& pt) const
{ return CRect_AC(*this) /= pt; }

//----------------------------------------------------------------------------------------
// CPoint_AC::operator-:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator-() const
{
	return CRect_AC(-left, -top, -right, -bottom);
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator==
//----------------------------------------------------------------------------------------
bool CRect_AC::operator==(const CRect_AC& rhs) const
{
	return *reinterpret_cast<const long*>(&top) == *reinterpret_cast<const long*>(&rhs.top) && *reinterpret_cast<const long*>(&bottom) == *reinterpret_cast<const long*>(&rhs.bottom);
	// cheat, we know that we're two longs
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator&=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator&=(const CRect_AC& rt)
{
	left = Max(left, rt.left);
	top = Max(top, rt.top);
	right = Min(right, rt.right);
	bottom = Min(bottom, rt.bottom);

	if (!Valid())
		top = left = bottom = right = 0;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator|=:
//----------------------------------------------------------------------------------------
CRect_AC& CRect_AC::operator|=(const CRect_AC& rt)
{
	left = Min(left, rt.left);
	top = Min(top, rt.top);
	right = Max(right, rt.right);
	bottom = Max(bottom, rt.bottom);

	return *this;
}

//----------------------------------------------------------------------------------------
// CRect_AC::operator&:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator&(const CRect_AC& rt) const
{ return CRect_AC(*this) &= rt; }

//----------------------------------------------------------------------------------------
// CRect_AC::operator|:
//----------------------------------------------------------------------------------------
CRect_AC CRect_AC::operator|(const CRect_AC& rt) const
{ return CRect_AC(*this) |= rt; }

//----------------------------------------------------------------------------------------
// CRect_AC::Set
//----------------------------------------------------------------------------------------
void CRect_AC::Set(const CPoint_AC& topLeftPt, short width, short height)
{
	(*this)[topLeft] = topLeftPt;
	bottom = top + height;
	right = left + width;
}

//----------------------------------------------------------------------------------------
// CRect_AC::Offset
//----------------------------------------------------------------------------------------
void CRect_AC::Offset(short iH, short iV)
{
	top += iV;
	left += iH;
	bottom += iV;
	right += iH;
}

//----------------------------------------------------------------------------------------
// CRect_AC::Inset
//----------------------------------------------------------------------------------------
void CRect_AC::Inset(short iH, short iV)
{
	top += iV;
	left += iH;
	bottom -= iV;
	right -= iH;
}

//----------------------------------------------------------------------------------------
// CRect_AC::Inset
//----------------------------------------------------------------------------------------
void CRect_AC::Inset(const CPoint_AC& delta)
{
	top += delta.v;
	left += delta.h;
	bottom -= delta.v;
	right -= delta.h;
}

//----------------------------------------------------------------------------------------
// CRect_AC::Inset
//----------------------------------------------------------------------------------------
void CRect_AC::Inset(const CRect_AC& delta)
{
	top += delta.top;
	left += delta.left;
	bottom -= delta.bottom;
	right -= delta.right;
}

//----------------------------------------------------------------------------------------
// CRect_AC::IsEmpty
//----------------------------------------------------------------------------------------
bool CRect_AC::IsEmpty() const
{
	return !(right > left && bottom > top);
}

//----------------------------------------------------------------------------------------
// CRect_AC::IsValid
//----------------------------------------------------------------------------------------
bool CRect_AC::IsValid() const
{
	return left <= right && top <= bottom;
}

//----------------------------------------------------------------------------------------
// CRect_AC::Validate.
//----------------------------------------------------------------------------------------
void CRect_AC::Validate()
{
	if (top > bottom)
	{
		short tmp = top;
		top = bottom;
		bottom = tmp;
	}
	if (left > right)
	{
		short tmp = left;
		left = right;
		right = tmp;
	}
}

//----------------------------------------------------------------------------------------
// CRect_AC::GetLength
//----------------------------------------------------------------------------------------
short CRect_AC::GetLength(VHSelect sel) const
{
	return sel == vSel ? bottom - top : right - left;
}

//----------------------------------------------------------------------------------------
// CRect_AC::GetSize: Returns the size of a CRect_AC as a CPoint_AC.
//----------------------------------------------------------------------------------------
CPoint_AC CRect_AC::GetSize() const
{
	return CPoint_AC(right - left, bottom - top);
}

//----------------------------------------------------------------------------------------
// CRect_AC::Contains
//----------------------------------------------------------------------------------------
bool CRect_AC::Contains(const CPoint_AC& pt) const
{
	return pt.v >= top && pt.v < bottom && pt.h >= left && pt.h < right;
}

//----------------------------------------------------------------------------------------
// CRect_AC::Contains
//----------------------------------------------------------------------------------------
bool CRect_AC::Contains(const CRect_AC& rt) const
{
	return rt.top >= top && rt.bottom <= bottom && rt.left >= left && rt.right <= right;
}

//----------------------------------------------------------------------------------------
// End of Geometry.cp


