//----------------------------------------------------------------------------------------
// CRegion_AC.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------



#ifndef __CRegion_AC__
#include "CRegion_AC.h"
#endif

// ACS

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
RgnHandle CRegion_AC::fCachedRgn;

//----------------------------------------------------------------------------------------
// Constructors
//----------------------------------------------------------------------------------------
CRegion_AC::CRegion_AC(short iLeft,
					   short iTop,
					   short iRight,
					   short iBottom)
{
	MakeRgnHandle();
	Set(iLeft, iTop, iRight, iBottom);
}


CRegion_AC::CRegion_AC(const CPoint_AC& topLeftPt,
					   const CPoint_AC& botRightPt)
{
	MakeRgnHandle();
	Set(topLeftPt, botRightPt);
}


CRegion_AC::CRegion_AC(const CRect_AC& rect)
{
	MakeRgnHandle();
	Set(rect);
}


CRegion_AC::CRegion_AC(const CRegion_AC& rgn)
{
	MakeRgnHandle();
	Set(rgn);
}


//----------------------------------------------------------------------------------------
// CRegion_A::operator=
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator=(const CRegion_AC& rgn)
{
	if (fData != rgn.fData)
		DisposeIfRgnHandle(fData);
	fData = rgn.fData;

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator+=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator+=(const CRegion_AC& rt)
{
	::UnionRgn(fData, rt, fData);

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator-=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator-=(const CRegion_AC& rt)
{
	::DiffRgn(fData, rt, fData);

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator^=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator^=(const CRegion_AC& rt)
{
	::XorRgn(fData, rt, fData);

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator^:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator^(const CRegion_AC& rt) const
{
	return CRegion_AC(*this) ^= rt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator+:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator+(const CRegion_AC& rt) const
{
	return CRegion_AC(*this) += rt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator-:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator-(const CRegion_AC& rt) const
{
	return CRegion_AC(*this) -= rt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator+=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator+=(const CPoint_AC& pt)
{
	Offset(pt);

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator-=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator-=(const CPoint_AC& pt)
{
	Offset(-pt);

	return *this;
}
//----------------------------------------------------------------------------------------
// CRegion_AC::operator+:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator+(const CPoint_AC& pt) const
{
	return CRegion_AC(*this) += pt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator-:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator-(const CPoint_AC& pt) const
{
	return CRegion_AC(*this) -= pt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator|=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator|=(const CRegion_AC& rt)
{
	::UnionRgn(fData, rt, fData);

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator|=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator|=(const CPoint_AC& rt)
{
	*this |= CTempRegion_AC(rt.h, rt.v, rt.h + 1, rt.v + 1);

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator|=:
//----------------------------------------------------------------------------------------
CRegion_AC& CRegion_AC::operator|=(const CLongRect_AC& rt)
{
	*this |= CTempRegion_AC(rt.ToRect());

	return *this;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator&:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator&(const CRegion_AC& rt) const
{
	return CRegion_AC(*this) &= rt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator|:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator|(const CRegion_AC& rt) const
{
	return CRegion_AC(*this) |= rt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator|:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator|(const CPoint_AC& rt) const
{
	return CRegion_AC(*this) |= rt;
}

//----------------------------------------------------------------------------------------
// CRegion_AC::operator|:
//----------------------------------------------------------------------------------------
CRegion_AC CRegion_AC::operator|(const CLongRect_AC& rt) const
{
	return CRegion_AC(*this) |= rt;
}

//----------------------------------------------------------------------------------------
// CRegion_A::Set
//----------------------------------------------------------------------------------------
void CRegion_AC::Set(const RgnHandle rhs)
{
	::CopyRgn(rhs, fData);
	::ThrowIfError_AC(::MemError());
}

//----------------------------------------------------------------------------------------
// CRegion_A::Inset
//----------------------------------------------------------------------------------------
void CRegion_AC::Inset(const CPoint_AC& delta)
{
	Inset(delta.h, delta.v);
	::ThrowIfError_AC(::MemError());
}
//----------------------------------------------------------------------------------------
// CRegion_A::Inset
//----------------------------------------------------------------------------------------
void CRegion_AC::Inset(const CRect_AC& delta)
{
	::MapRgn(fData, &Bounds(), &delta);
	::ThrowIfError_AC(::MemError());
}

//----------------------------------------------------------------------------------------
// CRegion_AC::IsValid
//----------------------------------------------------------------------------------------
bool CRegion_AC::IsValid() const
{
	return Bounds().IsValid();
}

//----------------------------------------------------------------------------------------
// CRegion_AC::Validate.
//----------------------------------------------------------------------------------------
void CRegion_AC::Validate()
{
	// ???
}

//----------------------------------------------------------------------------------------
// CRegion_AC::GetLength
//----------------------------------------------------------------------------------------
short CRegion_AC::GetLength(VHSelect sel) const
{
	return Bounds().GetLength(sel);
}

//----------------------------------------------------------------------------------------
// CRegion_AC::GetSize: Returns the size of a CRegion_AC as a CPoint_AC.
//----------------------------------------------------------------------------------------
CPoint_AC CRegion_AC::GetSize() const
{
	return Bounds().GetSize();
}

//----------------------------------------------------------------------------------------
// CRegion_AC::Contains
//----------------------------------------------------------------------------------------
bool CRegion_AC::Contains(const CPoint_AC& pt) const
{
	return ::PtInRgn(pt, fData);
}

//----------------------------------------------------------------------------------------
// CRegion_AC::Contains
//----------------------------------------------------------------------------------------
bool CRegion_AC::Contains(const CRect_AC& rt) const
{
	return ::RectInRgn(rt, fData);
}

//----------------------------------------------------------------------------------------
// CRegion_AC::Contains
//----------------------------------------------------------------------------------------
bool CRegion_AC::Contains(const CRegion_AC& rt) const
{
	CTempRegion_AC diff(*this);
	diff &= rt;
	return ::EmptyRgn(diff);
}

//----------------------------------------------------------------------------------------
// CRegion_AC::MakeRgnHandle
//----------------------------------------------------------------------------------------
RgnHandle CRegion_AC::MakeNewRgn()
{
	RgnHandle returnVal = NULL;

	// Simple single level cache.
	// take the cached region if available otherwise make one up to give out
	if (fCachedRgn)
	{
		returnVal = fCachedRgn;
		fCachedRgn = NULL;
	}
	else
	{
		returnVal = ::NewRgn();
		ThrowIfNULL_AC(returnVal);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// AboutToDisposeHandle: !!! Needs to be in ToolboxUtilities or somesuch
//----------------------------------------------------------------------------------------

#if qCoreDebug
static bool AboutToDisposeHandle(Handle aHandle)
{
	bool returnVal = false;

	// Test handlehood 
	if (IsHandle_AC(aHandle))
		returnVal = true;
	else
	{
		VerboseIsHandle_AC(aHandle);				// Get the diagnosis printed 
		::ProgramBreak_AC("Trying to dispose an invalid handle.\nBad handle: %p\n", aHandle);
	}

	return returnVal;
}
#endif
//----------------------------------------------------------------------------------------
// DisposeIfRgnHandle: 
//----------------------------------------------------------------------------------------
RgnHandle CRegion_AC::DisposeIfRgnHandle(RgnHandle aRgnHandle)
{
	if (aRgnHandle 
#if qCoreDebug
		&& AboutToDisposeHandle((Handle)aRgnHandle)
#endif		
		)
	{
		// return to the cache if not already full
		if (fCachedRgn)
		{
#if 0 // qCoreDebug !!! Need ToolboxUtilities
			// Set the handle contents to a real nice value for any dangling pointerciples 
			BlockSet_AC(*((Handle)aRgnHandle), GetHandleSize((Handle)aRgnHandle), kDisposedInitVal_AC);
#endif
			::DisposeRgn(aRgnHandle);			// already full
		}
		else
		{
			fCachedRgn = aRgnHandle;
			::SetEmptyRgn(aRgnHandle);
		}
	}

	return NULL;
}

//----------------------------------------------------------------------------------------
// End of CRegion_AC.cp
