//----------------------------------------------------------------------------------------
// CTemplatePoint_AC.cp 
// Copyright  1996 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTemplatePoint_AC__
#include "CTemplatePoint_AC.h"
#endif

// ACS

#ifndef __CTemplateRect_AC__
#include "CTemplateRect_AC.h"
#endif

// Toolbox

#ifndef __LIMITS__
#include <limits.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif


//========================================================================================
// CTemplatePoint_AC<T> method definitions
//========================================================================================

template <class T>
const CTemplatePoint_AC<T> CTemplatePoint_AC<T>::kZeroPt;			// Constructor keeps this from getting dead-stripped

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator+=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T>& CTemplatePoint_AC<T>::operator+=(const CTemplatePoint_AC<T>& pt)
{
	v += pt.v;
	h += pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator-=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T>& CTemplatePoint_AC<T>::operator-=(const CTemplatePoint_AC<T>& pt)
{
	v -= pt.v;
	h -= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator*=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T>& CTemplatePoint_AC<T>::operator*=(const CTemplatePoint_AC<T>& pt)
{
	v *= pt.v;
	h *= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator/=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T>& CTemplatePoint_AC<T>::operator/=(const CTemplatePoint_AC<T>& pt)
{
	v /= pt.v;
	h /= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator+:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T> CTemplatePoint_AC<T>::operator+(const CTemplatePoint_AC<T>& pt) const
{ 	
	return CTemplatePoint_AC<T>(*this) += pt; 
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator-:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T> CTemplatePoint_AC<T>::operator-(const CTemplatePoint_AC<T>& pt) const
{ 	
	return CTemplatePoint_AC<T>(*this) -= pt; 
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator*:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T> CTemplatePoint_AC<T>::operator*(const CTemplatePoint_AC<T>& pt) const
{ 
	return CTemplatePoint_AC<T>(*this) *= pt; 
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator/:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T> CTemplatePoint_AC<T>::operator/(const CTemplatePoint_AC<T>& pt) const
{ 
	return CTemplatePoint_AC<T>(*this) /= pt; 
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator-:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T> CTemplatePoint_AC<T>::operator-() const
{
	return CTemplatePoint_AC<T>(-h, -v);
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator++:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T>& CTemplatePoint_AC<T>::operator++()
{
	++v;
	++h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator--:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T>& CTemplatePoint_AC<T>::operator--()
{
	--v;
	--h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator==:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplatePoint_AC<T>::operator==(const CTemplatePoint_AC<T>& rhs) const
{
	return v == rhs.v && h == rhs.h;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator>=:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplatePoint_AC<T>::operator>=(const CTemplatePoint_AC<T>& pt) const
{
	return v >= pt.v && h >= pt.h;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator<=:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplatePoint_AC<T>::operator<=(const CTemplatePoint_AC<T>& pt) const
{
	return v <= pt.v && h <= pt.h;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator>:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplatePoint_AC<T>::operator>(const CTemplatePoint_AC<T>& pt) const
{
	return v > pt.v && h > pt.h;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::operator<=:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplatePoint_AC<T>::operator<(const CTemplatePoint_AC<T>& pt) const
{
	return v < pt.v && h < pt.h;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::ToPoint:
//----------------------------------------------------------------------------------------

template <class T>
CPoint_AC CTemplatePoint_AC<T>::ToPoint() const
{
	return CPoint_AC(static_cast<short>(MinMax(SHRT_MIN, h, SHRT_MAX)), static_cast<short>(MinMax(SHRT_MIN, v, SHRT_MAX)));
}

//----------------------------------------------------------------------------------------
// CPoint::Offset:
//----------------------------------------------------------------------------------------

template <class T>
void CTemplatePoint_AC<T>::Offset(Coordinate iH, Coordinate iV)
{
	v += iV;
	h += iH;
}

//----------------------------------------------------------------------------------------
// CTemplatePoint_AC<T>::ConstrainTo:
//----------------------------------------------------------------------------------------

template <class T>
void CTemplatePoint_AC<T>::ConstrainTo(const CTemplateRect_AC<T>& rt)
{
	if (v < rt.top)
		v = rt.top;
	if (v > rt.bottom)
		v = rt.bottom;
	if (h < rt.left)
		h = rt.left;
	if (h > rt.right)
		h = rt.right;
}



//----------------------------------------------------------------------------------------
// End of CTemplatePoint_AC.cp


