//----------------------------------------------------------------------------------------
// CTemplateRect_AC.cp 
// Copyright  1996 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTemplateRect_AC__
#include "CTemplateRect_AC.h"
#endif

// Toolbox

#ifndef __LIMITS__
#include <Limits.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif



//========================================================================================
// CTemplateRect_AC<T> method definitions
//========================================================================================


template <class T>
const CTemplateRect_AC<T> CTemplateRect_AC<T>::kZeroRect;	
	// Constructor keeps this from getting dead-stripped

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator+=:
//----------------------------------------------------------------------------------------
template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator+=(const CTemplateRect_AC<T>& rt)
{
	top += rt.top;
	left += rt.left;
	bottom += rt.bottom;
	right += rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator-=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator-=(const CTemplateRect_AC<T>& rt)
{
	top -= rt.top;
	left -= rt.left;
	bottom -= rt.bottom;
	right -= rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator*=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator*=(const CTemplateRect_AC<T>& rt)
{
	top *= rt.top;
	left *= rt.left;
	bottom *= rt.bottom;
	right *= rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator/=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator/=(const CTemplateRect_AC<T>& rt)
{
	top /= rt.top;
	left /= rt.left;
	bottom /= rt.bottom;
	right /= rt.right;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator+:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator+(const CTemplateRect_AC<T>& rt) const
{ 
	return CTemplateRect_AC<T>(*this) += rt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator-:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator-(const CTemplateRect_AC<T>& rt) const
{ 
	return CTemplateRect_AC<T>(*this) -= rt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator*:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator*(const CTemplateRect_AC<T>& rt) const
{ 
	return CTemplateRect_AC<T>(*this) *= rt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator/:
//----------------------------------------------------------------------------------------


template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator/(const CTemplateRect_AC<T>& rt) const
{ 	
	return CTemplateRect_AC<T>(*this) /= rt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator+=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator+=(const CTemplatePoint_AC<T>& pt)
{
	top += pt.v;
	left += pt.h;
	bottom += pt.v;
	right += pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator-=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator-=(const CTemplatePoint_AC<T>& pt)
{
	top -= pt.v;
	left -= pt.h;
	bottom -= pt.v;
	right -= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator*=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator*=(const CTemplatePoint_AC<T>& pt)
{
	top *= pt.v;
	left *= pt.h;
	bottom *= pt.v;
	right *= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator/=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator/=(const CTemplatePoint_AC<T>& pt)
{
	top /= pt.v;
	left /= pt.h;
	bottom /= pt.v;
	right /= pt.h;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator+:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator+(const CTemplatePoint_AC<T>& pt) const
{ 	
	return CTemplateRect_AC<T>(*this) += pt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::operator-:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator-(const CTemplatePoint_AC<T>& pt) const
{ 
	return CTemplateRect_AC<T>(*this) -= pt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator*:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator*(const CTemplatePoint_AC<T>& pt) const
{ 
	return CTemplateRect_AC<T>(*this) *= pt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator/:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator/(const CTemplatePoint_AC<T>& pt) const
{ 	
	return CTemplateRect_AC<T>(*this) /= pt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator-:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator-() const
{
	return CTemplateRect_AC<T>(-left, -top, -right, -bottom);
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator==:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplateRect_AC<T>::operator==(const CTemplateRect_AC<T>& rhs) const
{
	return top == rhs.top && left == rhs.left && bottom == rhs.bottom && right == rhs.right;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator&=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator&=(const CTemplateRect_AC<T>& rt)
{
	left = Max(left, rt.left);
	top = Max(top, rt.top);
	right = Min(right, rt.right);
	bottom = Min(bottom, rt.bottom);

	if (!Valid())
		top = left = bottom = right = 0;

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator|=:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T>& CTemplateRect_AC<T>::operator|=(const CTemplateRect_AC<T>& rt)
{
	left = Min(left, rt.left);
	top = Min(top, rt.top);
	right = Max(right, rt.right);
	bottom = Max(bottom, rt.bottom);

	return *this;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator&:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator&(const CTemplateRect_AC<T>& rt) const
{ 
	return CTemplateRect_AC<T>(*this) &= rt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::operator|:
//----------------------------------------------------------------------------------------

template <class T>
CTemplateRect_AC<T> CTemplateRect_AC<T>::operator|(const CTemplateRect_AC<T>& rt) const
{ 
	return CTemplateRect_AC<T>(*this) |= rt; 
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::ToRect:
//----------------------------------------------------------------------------------------

template <class T>
CRect_AC CTemplateRect_AC<T>::ToRect() const
{
	return CRect_AC(	static_cast<short>(MinMax(SHRT_MIN, left, SHRT_MAX)), 
						static_cast<short>(MinMax(SHRT_MIN, top, SHRT_MAX)), 
						static_cast<short>(MinMax(SHRT_MIN, right, SHRT_MAX)), 
						static_cast<short>(MinMax(SHRT_MIN, bottom, SHRT_MAX)));
}

//----------------------------------------------------------------------------------------
// CRCTemplateRectect_AC::Set
//----------------------------------------------------------------------------------------

template <class T>
void CTemplateRect_AC<T>::Set(const CTemplatePoint_AC<T>& topLeftPt, Coordinate width, Coordinate height)
{
	(*this)[topLeft] = topLeftPt; 
	bottom = top + height;
	right = left + width;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::Offset
//----------------------------------------------------------------------------------------

template <class T>
void CTemplateRect_AC<T>::Offset(Coordinate iH, Coordinate iV)
{
	top += iV;
	left += iH;
	bottom += iV;
	right += iH;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::Inset
//----------------------------------------------------------------------------------------

template <class T>
void CTemplateRect_AC<T>::Inset(Coordinate iH, Coordinate iV)
{
	top += iV;
	left += iH;
	bottom -= iV;
	right -= iH;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::Inset:
//----------------------------------------------------------------------------------------

template <class T>
void CTemplateRect_AC<T>::Inset(const CTemplatePoint_AC<T>& delta)
{
	top += delta.v;
	left += delta.h;
	bottom -= delta.v;
	right -= delta.h;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC<T>::Inset
//----------------------------------------------------------------------------------------

template <class T>
void CTemplateRect_AC<T>::Inset(const CTemplateRect_AC<T>& delta)
{
	top += delta.top;
	left += delta.left;
	bottom -= delta.bottom;
	right -= delta.right;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::IsEmpty:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplateRect_AC<T>::IsEmpty() const
{
	return !(right > left && bottom > top);
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::IsValid:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplateRect_AC<T>::IsValid() const
{
	return left <= right && top <= bottom;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::Validate:
//----------------------------------------------------------------------------------------

template <class T>
void CTemplateRect_AC<T>::Validate()
{
	if (top > bottom)
	{
		Coordinate tmp = top;
		top = bottom;
		bottom = tmp;
	}
	if (left > right)
	{
		Coordinate tmp = left;
		left = right;
		right = tmp;
	}
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::GetLength:
//----------------------------------------------------------------------------------------

template <class T>
 CTemplateRect_AC<T>::Coordinate CTemplateRect_AC<T>::GetLength(VHSelect sel) const
{
	return sel == vSel ? bottom - top : right - left;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::GetSize:
//----------------------------------------------------------------------------------------

template <class T>
CTemplatePoint_AC<T> CTemplateRect_AC<T>::GetSize() const
{
	return CTemplatePoint_AC<T>(right - left, bottom - top);
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::Contains:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplateRect_AC<T>::Contains(const CTemplatePoint_AC<T>& pt) const
{
	return pt.v >= top && pt.v < bottom && pt.h >= left && pt.h < right;
}

//----------------------------------------------------------------------------------------
// CTemplateRect_AC::Contains:
//----------------------------------------------------------------------------------------

template <class T>
bool CTemplateRect_AC<T>::Contains(const CTemplateRect_AC<T>& rt) const
{
	return rt.top >= top && rt.bottom <= bottom && rt.left >= left && rt.right <= right;
}



//----------------------------------------------------------------------------------------
// End of CTemplateRect_AC.cp


