//----------------------------------------------------------------------------------------
// TextBox_AC.cp
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif

#ifndef __Core_AC__
#include "Core_AC.h"
#endif


// Toolbox

#ifndef __QUICKDRAWTEXT__
#include "QuickDrawText.h"
#endif

static TEHandle 	pTextBoxTE;
static WordBreakUPP	pTEDefaultWordBreak;

//----------------------------------------------------------------------------------------
// StdNoRect: StdNoRect filters out the Rect drawing calls.
//----------------------------------------------------------------------------------------


static pascal void StdNoRect(GrafVerb			/* verb */,
							 const CRect_AC&		/* r */ )
{
}

//----------------------------------------------------------------------------------------
// TextBox_AC: 
//----------------------------------------------------------------------------------------
void TextBox_AC(Ptr text,
			   short itsLength,
			   const CRect_AC& box,
			   short itsJust,
			   bool autoWrap,
			   ProcPtr wordBreak,
			   bool eraseFirst,
			   bool spaceForCaret,
			   bool preferOutline)
{
	union QuickDrawProcs
	{
		QDProcs theQDProcs;
		CQDProcs theCQDProcs;
	};

	static const short kTextBoxCaretSlopSize = 1;// Since TETextBox uses TE to image the
												// text, we may need to adjust by 1 pixel.
												// Reason: TE draws beginning 1 pixel to
												// the right to allow for the insertion
												// point (which we won't have since this
												// is drawn text, not editable text).
	static const short kMaxTEChars = 32000;		// Actually TE suffers some other
												// limitations as well. Such as
												// misbehaviour and or bombing when the
												// sum of the lineheights > 32k or a
												// linewidth > 32k (overflows QuickDraw
												// space) But these are _MUCH_ more
												// difficult to test for in a quick way

	static QDRectUPP theRectProc;				// This can hang around all the time

	CWhileOutlinePreferred_AC setOP(preferOutline);

	// Create my goodies if necessary 
	if (!pTextBoxTE)
	{
		pTextBoxTE = TENew(box, box);
		if (pTextBoxTE)							// can allocate space for our terecord 
		{
			//  save off several items of interest
			pTEDefaultWordBreak = (*pTextBoxTE)->wordBreak;
		}

		if (!theRectProc)
		{
			theRectProc = NewQDRectProc(&StdNoRect);
			::ThrowIfNULL_AC(theRectProc);
		}
	}

	if (!pTextBoxTE)				// couldn't allocate the TE handle 
		TETextBox(text, itsLength, box, itsJust);// default to TextBox in low memory 
	else
	{
		WordBreakUPP wbUPP = NULL;

		// Setup the work TE with the necessary parameters 
		FontInfo fInfo;
		::GetFontInfo(&fInfo);	// Need to get font's height and ascent. 

		TERec& workTE = **pTextBoxTE;

		// Intersect the clip and the requested box
		((CRect_AC &)workTE.viewRect) = ((CRect_AC &)(*qd.thePort->clipRgn)->rgnBBox) & box;
		((CRect_AC &)workTE.destRect) = box;
		if (!spaceForCaret)
		{
			// Widen the destrect but not the visrect. This lets the 1 pixel wide area to the
			// left of all text and the right of all text go unshown.
			workTE.destRect.left -= kTextBoxCaretSlopSize;
			workTE.destRect.right += kTextBoxCaretSlopSize;
		}

		workTE.inPort = qd.thePort;					// Current port and its characteristics 

		workTE.txSize = qd.thePort->txSize;
		workTE.txFont = qd.thePort->txFont;
		workTE.txFace = qd.thePort->txFace;
		workTE.fontAscent = fInfo.ascent;
		workTE.lineHeight = fInfo.ascent + fInfo.descent + fInfo.leading;// returns the font height
		workTE.just = itsJust;

		workTE.crOnly = autoWrap ? 0 : -1;			//if >=0, word wrap
													//if <0, new line at Return only

		workTE.wordBreak = pTEDefaultWordBreak;

		if (wordBreak)
		{
			::ThrowIfNULL_AC(wbUPP = NewWordBreakProc(wordBreak));
			workTE = **pTextBoxTE;				// re-dereference because memory may have moved
			workTE.wordBreak = wbUPP;
		}

		TESetText(text, Min_AC(itsLength, kMaxTEChars), pTextBoxTE);

		// if called with eraseFirst true, then let TEUpdate image with its built-in EraseRect 
		if (eraseFirst)
			TEUpdate(box, pTextBoxTE);
		else
		{
			/* replace the existing QD procs ( standard or externally supplied )
			  so that the EraseRect in TEUpdate is ignored */
			QDProcsPtr saveQDProcsPtr = qd.thePort->grafProcs;
			QuickDrawProcs myQDProcs;
			QDRectUPP saveRectProc;

			if (!saveQDProcsPtr)
			{
				if (CRGBColor_AC::IsColorPort())
				{
					SetStdCProcs(&(myQDProcs.theCQDProcs));
					myQDProcs.theCQDProcs.rectProc = theRectProc;
					qd.thePort->grafProcs = (QDProcsPtr) & myQDProcs.theCQDProcs;
				}
				else
				{
					SetStdProcs(&(myQDProcs.theQDProcs));
					myQDProcs.theQDProcs.rectProc = theRectProc;
					qd.thePort->grafProcs = &myQDProcs.theQDProcs;
				}
			}
			else
			{
				saveRectProc = qd.thePort->grafProcs->rectProc;
				qd.thePort->grafProcs->rectProc = theRectProc;
			}

			// Now do the imaging 
			TEUpdate(box, pTextBoxTE);

			// Restore the QDProcs 
			if (!saveQDProcsPtr)
				qd.thePort->grafProcs = NULL;
			else
				qd.thePort->grafProcs->rectProc = saveRectProc;
		}

		if (wbUPP)
		{
			DisposeRoutineDescriptor(wbUPP);
			wbUPP = NULL;
		}
	}
}
