//----------------------------------------------------------------------------------------
// CAbstractTask_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __CAbstractTask_AC__
#define __CAbstractTask_AC__

// Patterns
#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CAbstractTask_AC is a class for performing an action, like a TCommand. 
// Install in into a CIdleTimeDispenser_AC.
//----------------------------------------------------------------------------------------

class EXPORT_AC CAbstractTask_AC : public MEventTransceiver_AC
{
public:
	
	CAbstractTask_AC();
		// default ctor
	
	CAbstractTask_AC(CReactor_AC* inReactor);
		// Event Sending Reactor
	
	virtual ~CAbstractTask_AC();
		// dtor
	
	virtual void DoTask() = 0;
		// override this
	
	void SetIsRepeater(bool isRepeater, long inFrequency = 0);
		// Is DoTask to be called repeatedly? If so how often?

	bool GetIsRepeater()
	{	return fRepeater; }
			
	void SetDelay(long inDelay);
		// Delay between asking for idle time and first call to DoTask.
		
	bool IsActive() const
	{	return fActive; }
		// Active means DoTask gets called.
		
	void SetIsActive(bool inActive)
	{	fActive = inActive; }
		// Changed active flag
	
	void SetFrequency(long inFrequency)
	{ 	fFrequency = inFrequency; }
		// How often DoTask is called.
	
	long GetFrequency() const
	{	return fFrequency; }
		
	virtual void StopIdling()
	{	SetIsRepeater(false); MIdler_AC::StopIdling(); }
		// Remove from the CIdleTimeDispenser.
	
protected:
	virtual void Idle();	

	bool fActive;
	long fFrequency;
	long fFireAt;
	bool fRepeater;
};


#endif
