//----------------------------------------------------------------------------------------
// CAbstractTimer_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CAbstractTimer_AC__
#define __CAbstractTimer_AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __CAbstractTask_AC__
#include "CAbstractTask_AC.h"
#endif

#ifndef __PatternEvents_AC__
#include "PatternEvents_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CAbstractTimer_AC is a base class for timing things specified by the subclass.
//----------------------------------------------------------------------------------------

class EXPORT_AC CAbstractTimer_AC : public CAbstractTask_AC
{
public:

	// Duration in Ticks
	
	enum eDuration
	{
		kSecond = 60,
		kMinute = kSecond * 60
	};

	CAbstractTimer_AC(	CReactor_AC* inReactor, 
						long inDuration = 0, 
						CEventID_AC inEvent = CPatternEvent_AC::kTimedOut);
		// Specify Reactor to send with, and duration and event timer will send.

	CAbstractTimer_AC(	long inDuration = 0, 
						CEventID_AC inEvent = CPatternEvent_AC::kTimedOut);
		// Specify duration and event timer will send.

	virtual ~CAbstractTimer_AC();
	
	virtual void Start();
		// Start timing using stored duration, i.e. set duration first.
	
	virtual void Stop();
		// Stop timing.
		
	virtual void SetDuration(long inDuration);
		// Duration is how long to time in Ticks.

	long GetDuration() const;
	
	virtual void SetEventID( CEventID_AC inEventId );
		// This is the event the timer will send when it times out. By default it
		// sends a CPatternEvent_AC::kTimedOut
	
	CEventID_AC GetEventID() const;	
	
	bool TimedOut() const;
		// Has this timer timed out yet?

	virtual void GetIdleTimeFrom(CIdleTimeDispenser_AC* inEngine);
		// Be sure and plug the timer into a dispenser.
						
protected:
	
	virtual void DoTask();
	
	virtual void HandleTimerEvent() = 0;

	CEventID_AC 			fEvent;
	long 					fDuration;
	bool 					fTimedOut;
	CIdleTimeDispenser_AC*	fDispenser;

};

// Inlines

inline long CAbstractTimer_AC::GetDuration() const
{	
	return fDuration; 
}
inline CEventID_AC CAbstractTimer_AC::GetEventID() const
{	
	return fEvent; 
}		
inline bool CAbstractTimer_AC::TimedOut() const
{
	return fTimedOut;
}
#endif