//----------------------------------------------------------------------------------------
// CErrorHandler_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __CErrorHandler_AC__
#define __CErrorHandler_AC__

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

class MErrorHandler_AC;
class MErrorProne_AC;

typedef CRefCountingPtr_AC<MErrorHandler_AC> CErrorHandlerPtr_AC;

//----------------------------------------------------------------------------------------
// MErrorHandler_AC is a mixin for responding to errors from an MErrorProne_AC class.
//----------------------------------------------------------------------------------------

class EXPORT_AC MErrorHandler_AC 
{
public:
	MErrorHandler_AC() {}
	virtual ~MErrorHandler_AC() {}

	virtual CError_AC RespondToError(CError_AC /*error*/, MErrorProne_AC* /*offendingObject*/)
	{	return 0; }
		// Override this to handle an error. Offending object is the object that had the error.
};

//----------------------------------------------------------------------------------------
// MErrorHandler_AC is a mixin for classes that may have errors occur that need to be
// handled by other means that throwing an exception.
//----------------------------------------------------------------------------------------

class EXPORT_AC MErrorProne_AC
{
public:
	MErrorProne_AC();
		// default ctor
	
	virtual ~MErrorProne_AC();
		// dtor
	
	MErrorHandler_AC* GetErrorHandler() const;
		// the currently installed error handler
	
	void SetErrorHandler(	MErrorHandler_AC* inHandler, bool deleteErrorHandler = false );		
		// set the MErrorHandler_AC to handle errors. You can install only one. DeleteErrorHandler
		// means that the MErrorHandler_AC owns the MErrorHandler_AC, i.e. it will delete it.
		
	CError_AC ErrorOccurred(CError_AC error);
		// Call this when an error occurs, and it will propagate the error into the MErrorProne_AC.
	
	bool HasErrorHandler() const;
		// Does this class have an MErrorProne_AC installed.

protected:
	MErrorHandler_AC*		fErrorHandler;
	bool					fOwner;
};

// Inline functions
	
inline MErrorProne_AC::MErrorProne_AC() 
	: fErrorHandler(NULL),
	fOwner(false)
{
}

inline MErrorProne_AC::~MErrorProne_AC() 
{  
	if(fOwner && fErrorHandler) 
		delete fErrorHandler; 
}

inline MErrorHandler_AC* MErrorProne_AC::GetErrorHandler() const
{ 	
	return fErrorHandler; 
}

inline void MErrorProne_AC::SetErrorHandler(	MErrorHandler_AC* inHandler, bool deleteErrorHandler )
{	
	fErrorHandler = inHandler; fOwner = deleteErrorHandler; 
}			

inline CError_AC MErrorProne_AC::ErrorOccurred(CError_AC error)
{	
	return fErrorHandler == NULL ? error : fErrorHandler->RespondToError(error, this); 
}

inline bool MErrorProne_AC::HasErrorHandler() const
{	
	return fErrorHandler != NULL; 
}

	
#endif