//----------------------------------------------------------------------------------------
// CEventWithData_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CEventWithData_AC__
#define __CEventWithData_AC__

// Pattern includes

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CEventWithData_AC is a subclass of CEvent that carries data with it. The data is the
// templated type.
//----------------------------------------------------------------------------------------

template <class DATA_TYPE>
class CEventWithData_AC : public CEvent_AC
{
public:
	CEventWithData_AC();
		// default ctor

	CEventWithData_AC(CEventID_AC inEvent, const DATA_TYPE& inData, CError_AC inErr = kNoError);
		// Set the CEventID_AC, the data type, and error if any.
	
	CEventWithData_AC(const CEventWithData_AC& inEvent);
		// copy ctor

	virtual ~CEventWithData_AC();
		// dtor

// operators

	CEventWithData_AC& operator = (const CEventWithData_AC& inEvent);
		// assignment operator
	
	void operator = (CEventID_AC inEvent);
		// Set the event id.
	
	bool operator == (const CEventWithData_AC& inEvent) const;
		// Compare against another event object
	
	bool operator != (const CEventWithData_AC& inEvent) const;

// getters/setters
	
	void SetData(const DATA_TYPE& inData);
		// Set the data that the class will carry with it.
	
	DATA_TYPE& GetData();
		// Get the data that the class will carry with it.

// Slightly gross helpers:

	static DATA_TYPE& GetData(CEvent_AC* inEvent);
		// Helper function for getting data out of a CEvent.

	static CEventWithData_AC<DATA_TYPE>* GetEvent(CEvent_AC* inEvent)
	// this returns NULL if can't coerce the event
	{
		CEventWithData_AC<DATA_TYPE>* testPtr = dynamic_cast<CEventWithData_AC<DATA_TYPE>*>(inEvent);
		return testPtr;
	}	
	
private:
	DATA_TYPE 	fData;
};

// Helpers

template <class DATA_TYPE>
inline CEvent_AC* CreateEventWithData_AC(CEventID_AC inEvent, const DATA_TYPE& inData, CError_AC inErr = kNoError)
	// This method makes creating a event with data a little easier. Just call it with it with the event
	// Id, the data to have the event carry, and error if any.
{
	return TH_new CEventWithData_AC<DATA_TYPE >(inEvent, inData, inErr);
}

// Macros for even more helping.

#define GetDataFromEvent_AC(EVENT_PTR, TYPE_NAME ) CEventWithData_AC<TYPE_NAME>::GetData((EVENT_PTR))
	// use this like this:
	// long myData = GetDataFromEvent(theEventPtr, long);

#define GetEventWithData_AC(EVENT_PTR, TYPE_NAME) CEventWithData_AC<TYPE_NAME>::GetEvent((EVENT_PTR))
	// use this like this:
	// CEventWithData<FOO>* fooEventPtr = GetEventWithData(theEventPtr, FOO);
	// if(fooEventPtr) blah...
	
// inline implementation

template <class DATA_TYPE>
inline DATA_TYPE& CEventWithData_AC<DATA_TYPE>::GetData(CEvent_AC* inEvent)
{
	CEventWithData_AC<DATA_TYPE>* event = dynamic_cast_AC(CEventWithData_AC<DATA_TYPE>*, inEvent);
	return event->GetData();
}

template < class DATA_TYPE>
inline CEventWithData_AC<DATA_TYPE>::CEventWithData_AC() 
{
}

template < class DATA_TYPE>
inline CEventWithData_AC<DATA_TYPE>::CEventWithData_AC(	CEventID_AC inEvent, 
														const DATA_TYPE& inData, 
														CError_AC inErr)
: 	CEvent_AC(inEvent, inErr),
	fData(inData)
{
}

template < class DATA_TYPE>
inline CEventWithData_AC<DATA_TYPE>& CEventWithData_AC<DATA_TYPE>::operator = (const CEventWithData_AC<DATA_TYPE>& inEvent) 
{ 	
	CEvent_AC::operator = (inEvent);
	fData = inEvent.fData; 
	return *this; 
}

template < class DATA_TYPE>
inline CEventWithData_AC<DATA_TYPE>::CEventWithData_AC(const CEventWithData_AC<DATA_TYPE>& inEvent) 
{ 
	*this = inEvent; 
}

template < class DATA_TYPE>
inline CEventWithData_AC<DATA_TYPE>::~CEventWithData_AC() 
{
}

template < class DATA_TYPE>
inline void CEventWithData_AC<DATA_TYPE>::operator = (CEventID_AC inEvent) 
{ 
	fEvent = inEvent; 
}

template < class DATA_TYPE>
inline bool CEventWithData_AC<DATA_TYPE>::operator == (const CEventWithData_AC<DATA_TYPE>& inEvent) const
{ 
	return (	CEvent_AC::operator == (inEvent) &&
				fData == inEvent.fData);
}

template < class DATA_TYPE>
inline bool CEventWithData_AC<DATA_TYPE>::operator != (const CEventWithData_AC<DATA_TYPE>& inEvent) const
{ 
	return (	CEvent_AC::operator != (inEvent) &&
				fData != inEvent.fData); 
}


template < class DATA_TYPE>
inline void CEventWithData_AC<DATA_TYPE>::SetData(const DATA_TYPE& inData)
{
	fData = inData;
}
	
template < class DATA_TYPE>
inline DATA_TYPE& CEventWithData_AC<DATA_TYPE>::GetData()
{
	return fData;
}

#endif