//----------------------------------------------------------------------------------------
// CEvent_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CEvent_AC__
#define __CEvent_AC__

// Pattern includes

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

#ifndef __PatternEvents_AC__
#include "PatternEvents_AC.h"
#endif

// core includes

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

#if qPatternsTrace_AC

#ifndef __typeinfo__
#include <typeinfo.h>
#endif

#endif

class CEvent_AC;

typedef CRefCountingPtr_AC<CEvent_AC> CEventPtr_AC;

//----------------------------------------------------------------------------------------
// CEvent_AC is the class that carries the event data for the event processing subsystem
// in the patterns suite. CEvent has an id value, and an error value. If the event is
// consumed it will not be propagated any further.
//----------------------------------------------------------------------------------------

class EXPORT_AC CEvent_AC : public MRefCountable_AC
{
public:

	enum { 	kNoError = 0 };

public:
	CEvent_AC();
		// default ctor

	CEvent_AC(CEventID_AC inEvent, CError_AC inErr = kNoError);
		// Set the event id and error if any.
	
	CEvent_AC(const CEvent_AC& inEvent);
		// copy ctor

	virtual ~CEvent_AC();
		// dtor

	// operators

	CEvent_AC& operator = (const CEvent_AC& inEvent);
		// assignment operator
	
	void operator = (CEventID_AC inEvent);
		// set the event ID
	
	// comparisons
	bool operator == (const CEvent_AC& inEvent) const;
	
	bool operator != (const CEvent_AC& inEvent) const;

	bool operator == (CEventID_AC inEvent) const;

	bool operator != (CEventID_AC inEvent) const;

	operator CEventID_AC() const;
		// convert to the event id value
		
//
// getters/setters
//	

// event type
	void SetEventID( CEventID_AC inEvent );

	CEventID_AC GetEventID() const;
	
// error	
	void SetError( CError_AC inErr);

	CError_AC GetError() const;

// data helpers for common data types, these do a downcast to CEventWithData_AC
// templated on the specified types.

	long GetLongData();

	unsigned long GetULongData();
		
	short GetShortData();
	
	unsigned short GetUShortData();
	
	char* GetCStringData();
	
	StringPtr GetPStringData();
	
	void* GetVoidPtrData();
	
// event handling utils
			
	void Consume();
		// Consuming the event will cause it to not be propagated
		// to any other EventHandlers
		
	bool WasConsumed() const;

// debug helpers
	
#ifdef qPatternsDebug_AC
	inline void Receive();
		// To Receive the event just means that a Receiver did something
		// with the event. Each time Receive is called, a counter is 
		// incremented.
	
	inline long WasReceived() const;
		// Get how many times the event was Received.				
#endif
						
protected:
	CEventID_AC 		fEvent;
	CError_AC 			fError;
	bool 				fConsumed;

#if qPatternsDebug_AC
	long				fReceiveCnt;
	CStr255_AC			fTypeName;
	const char* 		GetTypeName();
#endif
};

// This is debugging code.

#if qPatternsDebug_AC

#	if qPatternsTrace_AC

EXPORT_AC void DoDebugEventReceivedTraceMsg_AC(CEvent_AC* inEvent, const char* receiverClassName, const char* file, short line);
	// This function will write a string to TRACE.
		
#		define ReceiveEvent_AC(EVENT)	\
			DoDebugEventReceivedTraceMsg_AC((EVENT), typeid(*this).name(), __FILE__, __LINE__)
	// Use this macro to 'receive' events because it will be compiled out for non-debug builds.		
			
#	else 
	
inline void DoDebugReceiveEvent_AC(CEvent_AC*inEvent) 
	// This function receives the event for debug builds if TRACE is disabled.
{
	inEvent->Receive();
}

#	define ReceiveEvent_AC(EVENT) DoDebugReceiveEvent_AC((EVENT))
	// Use this macro to 'receive' events because it will be compiled out for non-debug builds.		
	
#	endif

#else

	inline void DoStubOutReceiveEvent_AC() {}
	#	define ReceiveEvent_AC(EVENT) DoStubOutReceiveEvent_AC

	// This is what gets compiled for non-debug builds.

#endif

// inline functions

inline CEvent_AC::CEvent_AC() 
: 	fEvent(CPatternEvent_AC::kNullEvent), 
	fError(kNoError),
	fConsumed(false)
{
#if qPatternsDebug_AC
	fReceiveCnt = 0;
	fTypeName = GetTypeName();
#endif
}
inline CEvent_AC::CEvent_AC(CEventID_AC inEvent, CError_AC inErr )
: 	fEvent(inEvent), 
	fError(inErr),
	fConsumed(false)
{
#if qPatternsDebug_AC
	fReceiveCnt = 0;
	fTypeName = GetTypeName();
#endif
}
inline CEvent_AC& CEvent_AC::operator = (const CEvent_AC& inEvent) 
{ 	
	fEvent = inEvent.fEvent; 
	fConsumed = inEvent.fConsumed; 
	fError = inEvent.fError;
#if qPatternsDebug_AC
	fReceiveCnt = inEvent.fReceiveCnt;
#endif
	return *this; 
}
inline CEvent_AC::CEvent_AC(const CEvent_AC& inEvent) 
{ 
#if qPatternsDebug_AC
	fReceiveCnt = 0;
	fTypeName = GetTypeName();
#endif
	*this = inEvent; 
}
inline CEvent_AC::~CEvent_AC() 
{
}
inline CEvent_AC::operator CEventID_AC() const 
{ 
	return fEvent; 
}
inline CEventID_AC CEvent_AC::GetEventID() const 
{ 
	return fEvent; 
}
inline void CEvent_AC::SetEventID( CEventID_AC inEvent ) 
{ 
	fEvent = inEvent; 
}
inline void CEvent_AC::operator = (CEventID_AC inEvent) 
{ 
	fEvent = inEvent; 
}
inline bool CEvent_AC::operator == (const CEvent_AC& inEvent) const
{ 
	return (	fEvent == inEvent.fEvent 	&& 
				fError == inEvent.fError 	);
}
inline bool CEvent_AC::operator != (const CEvent_AC& inEvent) const
{ 
	return (	fEvent != inEvent.fEvent || 
				fError != inEvent.fError 	); 
}
inline bool CEvent_AC::operator == (CEventID_AC inEvent) const 
{ 
	return fEvent == inEvent; 
}	
inline bool CEvent_AC::operator != (CEventID_AC inEvent) const 
{ 
	return fEvent != inEvent; 
}
inline void  CEvent_AC::Consume()
{
	fConsumed = true;
}
inline bool  CEvent_AC::WasConsumed() const
{
	return fConsumed; 
}
inline CError_AC CEvent_AC::GetError() const 
{ 
	return fError; 
}
inline void CEvent_AC::SetError( CError_AC inErr)
{ 
	fError = inErr; 
}
#if qPatternsDebug_AC

inline void CEvent_AC::Receive()
{
	++fReceiveCnt;		
}
inline long CEvent_AC::WasReceived() const
{
	return fReceiveCnt;
}

#endif 

#endif