//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC.h
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CIdleTimeDispenser_AC__
#define __CIdleTimeDispenser_AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __CIdleTimeEngine_AC__
#include "CIdleTimeEngine_AC.h"
#endif

#ifndef __MIdler_AC__
#include "MIdler_AC.h"
#endif

// core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// forward references
template <class T> class CLinkedList_AC;
class CAbstractTask_AC;

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC dispenses idle time to a list of MIdler objects. Idle time is
// give to the CIdleTimeDispenser_AC by a CIdleTimeEngine_AC, which the dispenser owns.
// The yield task is an abstract task whose DoTask method is called after all the MIdlers 
// in the dispenser's list get idle time.
//----------------------------------------------------------------------------------------

class EXPORT_AC CIdleTimeDispenser_AC : public MIdler_AC
{
public:
	CIdleTimeDispenser_AC();
		// default ctor
	
	virtual ~CIdleTimeDispenser_AC();
		// dtor

// Idlers	
	void AttachIdler(MIdler_AC* inIdler);
		// Add idler to list of idlers who get idle time. You can also call 
		// MIdler_AC::GetIdleTimeFrom. Note, if the Dispenser has an idle engine,
		// idling only occurs if there are idlers in the MIdler list.

	void DetachIdler(MIdler_AC* inIdler);
		// Remove idler from list of idlers who get idle time. You can also call 
		// MIdler_AC::StopIdliing

// Idle Time Engine
	void AquireIdleTimeEngine(CIdleTimeEngine_AC* inEngine, CAbstractTask_AC* yieldTask = NULL);
		// Set the idle time engine, which is what calls idle on this despenser. Install optional
		// YieldTask.
	
	CIdleTimeEngine_AC* GetIdleTimeEngine() const;
		// Get current engine.
	
	void ReleaseIdleTimeEngine();
		// Delete the engine.
	
	bool HasIdleTimeEngine() const;
		// Does this dispenser have an engine?

// Yield Task	
	void SetYieldTask(CAbstractTask_AC* yieldTask);
		// Set the optional yield task. The yield task's DoTask method is called after all the
		// MIlders get idle time.

	CAbstractTask_AC* GetYieldTask() const;
		// Get the Yield task.

	void ReleaseYieldTask();
		// Release yield task, which is ref counted so it may or may not be deleted.

// Misc
	virtual void Idle();
		// Call this to drive all the idlers	

private:
	typedef CLinkedList_AC<MIdler_AC*> 			CList;
	CAutoPtr_AC<CList >							fIdlers;
	CAutoPtr_AC<CIdleTimeEngine_AC> 			fEngine;
	CRefCountingPtr_AC<CAbstractTask_AC>		fYieldTask;
};

#endif