//----------------------------------------------------------------------------------------
// CReactorFactory_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CReactorStrategy_AC__
#define __CReactorStrategy_AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

// Core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// forward references

class CReactor_AC;
class CAbstractTask_AC;
class CIdleTimeDispenser_AC;

//----------------------------------------------------------------------------------------
// CReactorFactory_AC is a abstract class for creating a CReactor_AC. This allows for
// factories to be define that the Patterns suite doesn't know about, for a example
// the Threads suite defines a Reactor factory that makes a thread savvy CReactor.
//----------------------------------------------------------------------------------------

class EXPORT_AC CReactorFactory_AC : public MRefCountable_AC
{
public:
	CReactorFactory_AC();
		// ctor
	
	virtual ~CReactorFactory_AC();
		// dtor
	
	virtual CReactor_AC* CreateReactor() = 0;
		// override this to make the reactor
		
// Static creators
	
	// The Default factory is one that can be installed by anybody, and all 
	// reactors made with 'CreateDefaultReactor' will use the the factory
	// installed.
	
	static CReactor_AC* CreateDefaultReactor();
		// Create the default factory.
	
	static CReactorFactory_AC* GetDefaultFactory();
		// Get the default CReactorFactory_AC.
		
	static void SetDefaultFactory(CReactorFactory_AC* inFactory);
		// Set the default CReactorFactory_AC.
	
private:
	static CRefCountingPtr_AC<CReactorFactory_AC> fgDefault;
	
};



#endif