//----------------------------------------------------------------------------------------
// CStateMachine_AC.h
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CStateMachine_AC__
#define __CStateMachine_AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CStateMachine_AC is a simple object that keeps track of explicit state.
//----------------------------------------------------------------------------------------

class EXPORT_AC CStateMachine_AC
{
public:
	
	enum { 	kNullState,	
			kFirstUserState };

	CStateMachine_AC();
		// ctor

	CStateMachine_AC(const CStateMachine_AC& inState);
		// copy ctor

	virtual ~CStateMachine_AC();
		// dtor

	CStateMachine_AC& operator = (const CStateMachine_AC& inState);
		// assignment operator
		
	long GetState() const;
		// Get current state.
	
	long GetPreviousState() const;
		// Get the previous state.
	
	long ChangeToNextState();
		// This adds 1 to the value of the current state. This also changes the
		// previous state to the current state
	
	long ChangeStateTo(long newState);
		// Change state to a explicit state. This also changes the
		// previous state to the current state.

	bool IsAtState(long inState);
		// Is inState the same as the current state?

	bool IsInStateRange(long low, long hi);
		// Is the current state >= low and <= hi?

private:
	long fState; 
	long fPrevState;
};

// inline functions

inline long CStateMachine_AC::ChangeStateTo(long newState) 
{ 
	if(newState != fState) 
	{ 
		fPrevState = fState; 
		fState = newState; 
	}
	
	return fPrevState;
}

inline CStateMachine_AC::CStateMachine_AC() 
	: fState(kNullState), 
	fPrevState(kNullState) 
{
}

inline CStateMachine_AC::CStateMachine_AC(
	const CStateMachine_AC& inState) 
	: fState( inState.fState), 
	fPrevState(inState.fPrevState) 
{
}

inline CStateMachine_AC::~CStateMachine_AC() 
{
}

inline CStateMachine_AC& CStateMachine_AC::operator = 
	(const CStateMachine_AC& inState)
{
	fState = inState.fState; 
	fPrevState = inState.fPrevState; 
	return *this; 
}

inline long CStateMachine_AC::GetState() const 
{ 
	return fState; 
}

inline long CStateMachine_AC::GetPreviousState() const 
{ 
	return fPrevState; 
}

inline long CStateMachine_AC::ChangeToNextState() 
{ 
	ChangeStateTo(fState+1); 
	return fPrevState; 
}

inline bool CStateMachine_AC::IsAtState(long inState)
{
	return inState == fState; 
}

inline bool CStateMachine_AC::IsInStateRange(long low, long hi)
{
	return (fState >= low && fState <= hi );
}

#endif

