// ClassRegistration_AC.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __ClassRegistration_AC__
#define __ClassRegistration_AC__

// ACS

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

// ANSI

#ifndef __STDDEF__
#include <stddef.h>
#endif

#ifndef __typeinfo__
#include <typeinfo.h>
#endif

//========================================================================================
// Macros for generating and accessing class information by name or by signature
//========================================================================================
#define DEFINE_REGISTERED_AC(argument)													\
	extern const char _ClassName_##argument[] = #argument;								\
	extern MStreamable_AC* _DefaultConstructor_##argument();								\
	extern MStreamable_AC* _DefaultConstructor_##argument() { return TH_new argument; }
// Defines methods and fields of application class <argument> necessary to create by name or signature.

//
// NOTE: the "REGISTER" macros register the names using text so that they can potentially be
//       generated and inserted into a compilable file that doesn't need any includes for the
//       classes referenced by registration.
//
#define MAP_SIGNATURE_TO_CLASS_AC(argument, signature)										\
	{																						\
		extern const char _ClassName_##argument[];											\
		extern MStreamable_AC* _DefaultConstructor_##argument();								\
		CClassRegistration_AC::RegisterSignature(_ClassName_##argument, signature);						\
	}
// DOES NOT Force the class <argument> to be marked active so that the linker does not dead strip it.
// It just, makes it possible for the class to be created by name OR by signature if the class
// is later dead strip suppressed by MA_REGISTER_CLASS or MA_REGISTER_SIGNATURE. This can be used
// to provide the default mappings but not keep the classes from being dead stripped unless you want them

#define REGISTER_CLASS_AC(argument) 														\
	{																						\
		extern const char _ClassName_##argument[];											\
		extern MStreamable_AC* _DefaultConstructor_##argument();								\
		CClassRegistration_AC::RegisterClass(_ClassName_##argument, &_DefaultConstructor_##argument);	\
	}
// Forces the class <argument> to be marked active so that the linker does not dead strip it.
// Also, makes it possible for the class to be created by name

#define REGISTER_SIGNATURE_AC(argument, signature)											\
	{																						\
		extern const char _ClassName_##argument[];											\
		extern MStreamable_AC* _DefaultConstructor_##argument();								\
		CClassRegistration_AC::RegisterClass(_ClassName_##argument, &_DefaultConstructor_##argument);	\
		CClassRegistration_AC::RegisterSignature(_ClassName_##argument, signature);						\
	}
// Forces the class <argument> to be marked active so that the linker does not dead strip it.
// Also, makes it possible for the class to be created by name OR by signature

//----------------------------------------------------------------------------------------
// Forward class declarations
//----------------------------------------------------------------------------------------
class CClassRegistration_AC;
class MStreamable_AC;

//----------------------------------------------------------------------------------------
// Types
//----------------------------------------------------------------------------------------
typedef OSType IDType;
typedef unsigned short ClassID_AC;
typedef MStreamable_AC* (* ConstructorProcPtr_AC)();
typedef char* ClassName_AC;					// c-style null terminated

//----------------------------------------------------------------------------------------
// Some constants
//----------------------------------------------------------------------------------------
#define kNilClass_AC NULL

//----------------------------------------------------------------------------------------
// CLASS CClassRegistration_AC
//----------------------------------------------------------------------------------------
class EXPORT_AC CClassRegistration_AC
{
public:
	CClassRegistration_AC(const char* inTypeName, ConstructorProcPtr_AC constructor, ClassID_AC inClassID);
			  			  
	inline ~CClassRegistration_AC() { }
			  			  
	inline MStreamable_AC* Call_DefaultConstructor() { return fDefaultConstructor(); } const
		// Return a new instance of the class. 
	
	inline char* const GetName() const { return fTypeName; }
		// Return a pointer to the class name. 
	
	inline ClassID_AC GetClassID() const { return fClassID; }
		// Return the signature used for creating this class. 
	
	//----------------------------------------------------------------------------------------
	// static member functions
	//----------------------------------------------------------------------------------------
public:
	static void RegisterClass(const char* inTypeName, ConstructorProcPtr_AC constructor);
	static void RegisterSignature(const char* inTypeName, IDType signature);

	static CClassRegistration_AC* GetClassDescFromClassID(const ClassID_AC classID);
	static CClassRegistration_AC* GetClassDescFromClassName(const char* className);
	static CClassRegistration_AC* GetClassDescFromSignature(IDType signature);
	static IDType GetSignatureFromClassName(const char* className);
	
	inline static MStreamable_AC* NewByClassDesc(CClassRegistration_AC* inClassDesc) { return inClassDesc->Call_DefaultConstructor(); }

	static MStreamable_AC* NewByClassID(const ClassID_AC classID);
	static MStreamable_AC* NewByClassName(const char* className);
	static MStreamable_AC* NewBySignature(IDType signature);
	
	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
private:
	const char* fTypeName;								// reference to the type information for this class
	const ConstructorProcPtr_AC fDefaultConstructor;		// Default constructor for this class
	ClassID_AC fClassID;									// an arbitrary number that can be used to identify this class
};

//----------------------------------------------------------------------------------------
// Object Creation Utilities
//----------------------------------------------------------------------------------------
inline MStreamable_AC* NewStdObject_AC(IDType signature) { return CClassRegistration_AC::NewBySignature(signature); }
	// Returns the prototype object for the given signature.

EXPORT_AC MStreamable_AC* NewObjectBySignature_AC(IDType signature,
							  const char* className);
	// Returns an instance of class "className" if className is non-empty. Returns the
    // prototype object for the given signature if className is empty

#endif // __ClassRegistration_AC__
