// MDependable.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __MDependable_AC__
#define __MDependable_AC__

#ifndef __ConditionalMacros_AC__
#include "ConditionalMacros_AC.h"
#endif

// ACS Core

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

typedef long ChangeID_AC;

enum
{
	kNoChange_AC = -1
};

class CDependencySpace_AC;

//----------------------------------------------------------------------------------------
// MDependable_AC: Definition an object that can be or have dependents
//----------------------------------------------------------------------------------------

class EXPORT_AC MDependable_AC
{
public:
	//------------------------------------------------------------------------------------
	// Constructor, destructor
	//------------------------------------------------------------------------------------
	inline MDependable_AC() { }
		// Constructor
		
	virtual ~MDependable_AC();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Common stuff
	//------------------------------------------------------------------------------------
	void RemoveAllDependencies();
		// Remove all dependency relations in the GetDependencySpace() dependency space
		// that involve this notifier or dependent.

	virtual CDependencySpace_AC* GetDependencySpace();
		// Returns the default dependency space for storing dependents of this object.
		// Returns fgDefaultDependencySpace.

	//------------------------------------------------------------------------------------
	// Notifier stuff
	//------------------------------------------------------------------------------------
	void AddDependent(MDependable_AC* dependent);
		// Registers a dependent of this notifier so that it will be notified of changes 
		// in this notifier.

	void RemoveDependent(MDependable_AC* dependent);
		// Remove the specified dependent from the list of dependents of this notifier

	virtual void Changed(ChangeID_AC theChange, void* changeData);
		// Call this to inform dependents of this notifier that it has been changed. 
		// "theChange" will often contain a command number.
		// "changeData" will often contain a command object. 
		// Directly called often, overridden rarely.

	//------------------------------------------------------------------------------------
	// Dependent stuff
	//------------------------------------------------------------------------------------
	virtual void DoUpdate(ChangeID_AC theChange, 
								MDependable_AC* changedObject,
								void* changeData,
								CDependencySpace_AC* dependencySpace);
		// Informs this dependent that a notifier on which it is dependent has been changed.
		// "theChange" will often contain a command number.
		// "changeData" will often contain a command object. 
        // Directly called rarely, overridden often.

	virtual void SetMark(bool state);
		// Intended to mark (state=true) or unmark (state=false) this object. Can be used
		// by dependency spaces that require marking, and overridden by objects that store the
		// mark internally.
		// The ACS dependency spaces do not use this method. 

	virtual bool IsMarked();
		// Intended to return true if this object is marked. Can be used
		// by dependency spaces that require marking, and overridden by objects that store the
		// mark internally.
		// The ACS dependency spaces do not use this method. 
};

#endif
