//----------------------------------------------------------------------------------------
// MEventReceiver_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __MEventReceiver_AC__
#define __MEventReceiver_AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

// Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// forward references

class MEventTransceiver_AC;
class CEventAdaptor_AC;
class CEvent_AC;

//----------------------------------------------------------------------------------------
// MEventReceiver_AC is a mixin for recieving events from a MEventTransceiver_AC. This
// class is not ref counted, which is why it exists, so that you can mix it into any
// object. 
//
// The MEventReceiver_AC uses a CEventAdaptor_AC to convert the events from an 
// MEventTransceiver_AC event to a MEventReceiver_AC event.
//----------------------------------------------------------------------------------------

class EXPORT_AC MEventReceiver_AC
{
public:
	MEventReceiver_AC();
		// will not make a new Adaptor.
	
	MEventReceiver_AC(CEventAdaptor_AC* inAdaptor);
		// use this to share a Adaptor.
		
	virtual ~MEventReceiver_AC();
		// ctor

	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender) = 0;
		// override this to respond to an event.
			
	CEventAdaptor_AC* GetAdaptor() const;
		// Get the currently installed adaptor.
		
	virtual void SetAdaptor(CEventAdaptor_AC* inAdaptor);
		// Set the Adaptor
	
	void RegisterForEventsFrom(MEventTransceiver_AC* inTransceiver);
		// Call this to register for events from a source. This will also
		// make an adaptor if none is assigned.

	void UnregisterForEventsFrom(MEventTransceiver_AC* inTransceiver);
		// Unregister for events.
	
protected:
	virtual void Open();
		// open will make an adaptor if none is assigned.
			
protected:
	CRefCountingPtr_AC<CEventAdaptor_AC> fAdaptor;
};

// inline functions

inline CEventAdaptor_AC* MEventReceiver_AC::GetAdaptor() const
{	
	return fAdaptor; 
}


#endif