//----------------------------------------------------------------------------------------
// MEventTransceiver_AC.h
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __MEventTransceiver__AC__
#define __MEventTransceiver__AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __MIdler__
#include "MIdler_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

// Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// forward reference

template <class T> class CLinkedList_AC;

class CEvent_AC;

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC is the class that you actually send and receive events with.
// To receive events, registering with a source explicitely is required. Events are
// actually sent through a CReactor_AC, which you can specify, or the MEventTransceiver_AC
// will create one for you using CReactorFactory_AC::CreateDefaultReactor().
//----------------------------------------------------------------------------------------

class EXPORT_AC MEventTransceiver_AC : public MIdler_AC
{
public:
	MEventTransceiver_AC();
		// ctor
	
	MEventTransceiver_AC(CReactor_AC* inReactor);
		// Send in the reactor you're going to send events through.
	
	virtual ~MEventTransceiver_AC();
		// dtor

// Registering for events

	// Registering for events means that you're going to receive events 
	// from the EventTransceiver you're registering with.

	void RegisterForEventsFrom(MEventTransceiver_AC* inEventSender);
		// Register for events from inEventSender.
			
	void UnregisterForEventsFrom(MEventTransceiver_AC* inEventSender);
		// Register for events from inEventSender.

	// StartSendingEventsTo and StopSendingEventsTo are equivelant in functionality
	// to RegisterForEventsFrom and UnregisterForEventsFrom. They are provided as
	// an alternative API and more importantly you may want to override them
	// to change how registration occurs (this is very specialized case most
	// developers won't need to do).

	virtual void StartSendingEventsTo(MEventTransceiver_AC* inEventReceiver);
	
	virtual void StopSendingEventsTo(MEventTransceiver_AC* inEventReceiver);

	virtual void StopSendingAllEvents();
		// Stop sending all events.

	virtual void StopReceivingAllEvents();
		// Stop Receiving all events.
		
	virtual void StopTransceivingAllEvents();
		// Stop Sending and Receiving.
	
// Recieving Events

	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inFrom);
		// Receive an event from a subscribed source. This event doesn't come from
		// this objects reactor, it comes from the senders reactor.

// Sending Events

	virtual void PostEvent( CEvent_AC* inEventToSend);
		// This sends an event through this objects Reactor.

// Reactor management

	virtual void SetReactor(CReactor_AC* inReactor);
		// This is the reactor used to _SEND_ events only. Events recieved are
		// routed through the sender's Reactor, i.e. you can be receiving events
		// through an unlimited number of Reactors.
	
	virtual void UseDefaultReactor(bool replaceCurrentReactor = false);
		// Tell this MEventTransceiver_AC to use a Reactor created by the 
		// default CReactorFactory_AC.
		
	virtual CReactor_AC* GetReactor();
		// Get the Reactor this object uses to _SEND_ events.
	
	virtual bool HasReactor() const;
		// Does this object have a Reactor with which to send events?

protected:
	virtual void RegistrationChanged(CReactor_AC* inReactor, bool registering); 

protected:
	typedef CLinkedList_AC<CReactor_AC*> 	CReactorList;
	
	CRefCountingPtr_AC<CReactor_AC> 		fReactor;
	CAutoPtr_AC<CReactorList>				fRegisteredWith;
	
	friend class CReactor_AC;	
	
};

// inline functions

inline bool MEventTransceiver_AC::HasReactor() const
{
	return !fReactor.IsNULL();
}

inline void MEventTransceiver_AC::RegisterForEventsFrom(MEventTransceiver_AC* inEventSender)
{
	::AssertNotNULL_AC(inEventSender);
	inEventSender->StartSendingEventsTo(this);
}

inline void MEventTransceiver_AC::UnregisterForEventsFrom(MEventTransceiver_AC* inEventSender)
{
	::AssertNotNULL_AC(inEventSender);
	inEventSender->StopSendingEventsTo(this);
}

// typedef

typedef CRefCountingPtr_AC<MEventTransceiver_AC> MEventTransceiverPtr_AC;


#endif