//----------------------------------------------------------------------------------------
// MIdler_AC.h
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __MIdler_AC__
#define __MIdler_AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __MRefCountable_AC__
#include "MRefCountable_AC.h"
#endif

// forward ref

class CIdleTimeDispenser_AC;

//----------------------------------------------------------------------------------------
// MIdler_AC is a mixin object for objects desiring to receive idle time from a 
// CIdleTimeDispenser. Override Idle to get idle time. Calling GetIdleTimeFrom with
// a valid CIdleTimeDispenser_AC is required to receive idle time.
//----------------------------------------------------------------------------------------

class EXPORT_AC MIdler_AC : public MRefCountable_AC
{
public:
	MIdler_AC();
		// ctor
	
	virtual ~MIdler_AC();
		// dtor
		
	virtual void Idle();
		// Override this to receive idle time.
	
	CIdleTimeDispenser_AC* GetIdleTimeDispenser();
		// Get the CIdleTimeDispenser_AC this object is getting idle time from, if 
		// any.
	
	virtual void GetIdleTimeFrom(CIdleTimeDispenser_AC* inEngine);
		// Start getting idle time from inEngine.
		
	virtual void StopIdling();
		// Stop receiving idle time. Calling this sets fEngine to NULL.
			
protected:
	virtual void SetEngine(	CIdleTimeDispenser_AC* inEngine);
			
private:
	CIdleTimeDispenser_AC* fEngine;

	friend class CIdleTimeDispenser_AC;
};

// inline functions

inline CIdleTimeDispenser_AC* MIdler_AC::GetIdleTimeDispenser()
{	
	return fEngine; 
}

#endif