//----------------------------------------------------------------------------------------
// CBuffer_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// patterns

#ifndef __CBuffer_AC__
#include "CBuffer_AC.h"
#endif

// toolbox

#ifndef __Memory__
#include <Memory.h>
#endif
#ifndef __Errors__
#include <Errors.h>
#endif

#define kGrowChunkPercent_AC 10

//----------------------------------------------------------------------------------------
// copy ctor
//----------------------------------------------------------------------------------------

CBuffer_AC::CBuffer_AC( const CBuffer_AC& in )
:	fSize(0),
	fOwner(false),
	fBuffer(NULL),
	fGrownAmt(0),
	fGrowChunkSize(0),
	fGrowChunkPercent(kDefaultGrowChunkPercent)
{
	*this = in;
}

//----------------------------------------------------------------------------------------
// ctor
//----------------------------------------------------------------------------------------

CBuffer_AC::CBuffer_AC(long size, void* buf, eOwnership ownerFlags)
:	fSize(0),
	fOwner(false),
	fBuffer(NULL),
	fGrowChunkSize(0),
	fGrownAmt(0),
	fGrowChunkPercent(kDefaultGrowChunkPercent)
{
	CalcGrowChunkSize();
	SetBuffer(size, buf, ownerFlags);
}

//----------------------------------------------------------------------------------------
// default ctor
//----------------------------------------------------------------------------------------

CBuffer_AC::CBuffer_AC()
:	fSize(0),
	fOwner(false),
	fBuffer(NULL),
	fGrownAmt(0),
	fGrowChunkSize(0),
	fGrowChunkPercent(kDefaultGrowChunkPercent)
{
}

//----------------------------------------------------------------------------------------
// dtor
//----------------------------------------------------------------------------------------

CBuffer_AC::~CBuffer_AC()
{
	ReleaseBuffer();
}

//----------------------------------------------------------------------------------------
// CBuffer_AC::ReleaseBuffer
//----------------------------------------------------------------------------------------

void CBuffer_AC::ReleaseBuffer()
{	
	if(fOwner) 
		::DeleteIfArray_AC(fBuffer); 
	else 
		fBuffer = NULL; 
	
	fSize = 0; 
	fGrownAmt = 0;
	fGrowChunkSize = 0;
}

//----------------------------------------------------------------------------------------
// assignment operator
//----------------------------------------------------------------------------------------

CBuffer_AC& CBuffer_AC::operator = (const CBuffer_AC& inBlock)
// This copies the buffer?
{
	if( &inBlock != this )
	{
		if(inBlock.fOwner)
		{
			// let's make a copy of the buffer

			SetBuffer(inBlock.fSize, NULL);
				// set the buffer 				
			::BlockMove_AC(inBlock.fBuffer, fBuffer, inBlock.fSize);
				// copy the data into the buffer
				
			fGrownAmt = inBlock.fGrownAmt;
			fGrowChunkSize = inBlock.fGrowChunkSize;
			fGrowChunkPercent = inBlock.fGrowChunkPercent;
				// copy over the other data
		}
		else
		{
			BlockMove_AC(&inBlock, this, sizeof(CBuffer_AC));
				// copy the whole thing since we're pointing at the same data. 
		}
	}
	
	
	return *this;
}

//----------------------------------------------------------------------------------------
// CBuffer_AC::SetBuffer
//----------------------------------------------------------------------------------------

void CBuffer_AC::SetBuffer(long inSize, void* inPtr, eOwnership inFlag)
{
	if(inFlag == kAutoSet)
	{
		if(inPtr == NULL) 
			inFlag = kOwnData;
		else
			inFlag = kBorrowData;
	}

	if(inPtr != NULL)
	{
		ReleaseBuffer();

		fBuffer = static_cast<char*>(inPtr);
		fSize = inSize;

		if(inFlag == kBorrowData)
			fOwner = false;
		else
			fOwner = true;
	}
	else
	{
		if( inFlag == kOwnData )
		{
			if(fBuffer && !fOwner)
				ReleaseBuffer();
			
			fOwner = true;
			
			SetSize(inSize, false);
			
			CalcGrowChunkSize();
		}
		else
			ThrowException_AC(paramErr, 0);
	}
}

//----------------------------------------------------------------------------------------
// CBuffer_AC::SetSize
//----------------------------------------------------------------------------------------

void CBuffer_AC::SetSize(long newSize, bool preserveData)
{
	if(!fOwner)
		ThrowException_AC(paramErr,0);
	
	if(newSize == 0)
	{
		ReleaseBuffer();
	}
	else
	if(newSize != fSize)
	{	
		char* newBuffer = TH_new char[newSize];

		fGrownAmt += (newSize - fSize);

		if(preserveData && fBuffer)
			BlockMove_AC(fBuffer, newBuffer, Min_AC( fSize, newSize ));

		ReleaseBuffer();
		
		fBuffer = newBuffer;
		fSize = newSize;
	}
}



