//----------------------------------------------------------------------------------------
// CDependencies.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

// ACS

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

// Toolbox

#ifndef __PACKAGES__
#include <Packages.h>
#endif


//========================================================================================
// struct CDependencyComparer_AC
//========================================================================================
struct CDependencyComparer_AC
{
public:
	// Constructor
	inline CDependencyComparer_AC(SDependency_AC& targetDependency,
						const CDependencies_AC* theDependencies) :
		fTargetDependency(targetDependency),
		fDependencies(theDependencies)
	{ }

	SDependency_AC& fTargetDependency;
	const CDependencies_AC* fDependencies;
};


//========================================================================================
// GLOBAL Variables
//========================================================================================
CAutoPtr_AC<CDependencySpace_AC> CDependencySpace_AC::fgDefaultDependencySpace;

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static CompareResult_AC CompareDependency_AC(ArrayIndex_AC index,
									   void* yourDataPtr);

//========================================================================================
// CLASS CNotificationQueue_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CNotificationQueue_AC destructor
//----------------------------------------------------------------------------------------
CNotificationQueue_AC::~CNotificationQueue_AC()
{
}

//----------------------------------------------------------------------------------------
// CNotificationQueue_AC::Dequeue: 
//----------------------------------------------------------------------------------------
bool CNotificationQueue_AC::Dequeue(SNotification_AC& item)
{
	bool available = (fSize > 0);
	if (available)
	{
		GetElementsAt(1, &item, 1);
		DeleteElementsAt(1, 1);
	}

	return available;
}

//========================================================================================
// CLASS CDependencies_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CDependencies_AC destructor
//----------------------------------------------------------------------------------------
CDependencies_AC::~CDependencies_AC()
{
}

//----------------------------------------------------------------------------------------
// CDependencies_AC::AddDependency: 
//----------------------------------------------------------------------------------------
void CDependencies_AC::AddDependency(MDependable_AC* notifier,
								  MDependable_AC* dependent,
								  DependencyLabels_AC dependencyLabel)
{
	ArrayIndex_AC index;

	if (!FindDependency(notifier, dependent, dependencyLabel, index))
		InsertDependencyBefore(index, notifier, dependent, dependencyLabel);
}

//----------------------------------------------------------------------------------------
// CDependencies_AC::InsertDependencyBefore: 
//----------------------------------------------------------------------------------------
void CDependencies_AC::InsertDependencyBefore(ArrayIndex_AC index,
										   MDependable_AC* notifier,
										   MDependable_AC* dependent,
										   DependencyLabels_AC dependencyLabel)
{
	SDependency_AC newDependency(notifier, dependent, dependencyLabel);

	InsertElementsBefore(index, &newDependency, 1);
}


//----------------------------------------------------------------------------------------
// CDependencies_AC::FindDependency: 
//----------------------------------------------------------------------------------------
bool CDependencies_AC::FindDependency(MDependable_AC* notifier,
									  MDependable_AC* dependent,
									  DependencyLabels_AC dependencyLabel,
									  ArrayIndex_AC& index) const
{
	SDependency_AC targetDependency(notifier, dependent, dependencyLabel);
	CDependencyComparer_AC aDependencyComparer(targetDependency, this);

	return DoSearchElement(CompareDependency_AC, &aDependencyComparer, index);
}

//----------------------------------------------------------------------------------------
// CDependencies_AC::FindPrimaryObject: 
//----------------------------------------------------------------------------------------
bool CDependencies_AC::FindPrimaryObject(const MDependable_AC* notifier,
										 ArrayIndex_AC& index) const
{
	SDependency_AC targetDependency(notifier, NULL, kNoLabel_AC);
	CDependencyComparer_AC aDependencyComparer(targetDependency, this);

	return DoSearchElement(CompareDependency_AC, &aDependencyComparer, index);
}

//----------------------------------------------------------------------------------------
// CDependencies_AC::RemoveDependency: 
//----------------------------------------------------------------------------------------
void CDependencies_AC::RemoveDependency(MDependable_AC* notifier,
									 MDependable_AC* dependent,
									 DependencyLabels_AC dependencyLabel)
{
	ArrayIndex_AC index;

	if (FindDependency(notifier, dependent, dependencyLabel, index))
		DeleteElementsAt(index, 1);
}

//----------------------------------------------------------------------------------------
// CDependencies_AC::CompareElements: 
//----------------------------------------------------------------------------------------
CompareResult_AC CDependencies_AC::CompareElements(const void* Element1,
											 const void* Element2) const// Override 

{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	const SDependency_AC& Dependency1 = *static_cast<const SDependency_AC*>(Element1);
	const SDependency_AC& Dependency2 = *static_cast<const SDependency_AC*>(Element2);

	if (Dependency1.fPrimaryObject > Dependency2.fPrimaryObject)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (Dependency1.fPrimaryObject < Dependency2.fPrimaryObject)
		returnVal = kItem1LessThanItem2_AC;
	else
	{
		if (!Dependency1.fSecondaryObject || !Dependency2.fSecondaryObject || (Dependency1.fSecondaryObject == Dependency2.fSecondaryObject))
			returnVal = kItem1EqualItem2_AC;
		else if (Dependency1.fSecondaryObject < Dependency2.fSecondaryObject)
			returnVal = kItem1LessThanItem2_AC;
		else
			returnVal = kItem1GreaterThanItem2_AC;
	}

	return returnVal;
}


//========================================================================================
// CLASS CDependencySpace_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CDependencySpace_AC destructor
//----------------------------------------------------------------------------------------
CDependencySpace_AC::~CDependencySpace_AC()
{
}

//----------------------------------------------------------------------------------------
// CDependencySpace_AC::AddDependency: 
//----------------------------------------------------------------------------------------
void CDependencySpace_AC::AddDependency(MDependable_AC*	/*notifier*/ ,
									 MDependable_AC*	/*dependent*/ ,
									 DependencyLabels_AC/*dependencyLabel*/ )
{
	SubClassResponsibility_AC();
}

//----------------------------------------------------------------------------------------
// CDependencySpace_AC::RemoveDependency: 
//----------------------------------------------------------------------------------------
void CDependencySpace_AC::RemoveDependency(MDependable_AC*	/*notifier*/  ,
										MDependable_AC*		/*dependent*/  ,
										DependencyLabels_AC	/*dependencyLabel*/ )
{
	SubClassResponsibility_AC();
}

//----------------------------------------------------------------------------------------
// CDependencySpace_AC::NotifierChanged: 
//----------------------------------------------------------------------------------------
void CDependencySpace_AC::NotifierChanged(MDependable_AC*	/*notifier*/ ,
									   ChangeID_AC	/*theChange*/,
									   void*	/*changeData*/ )// Override
{
	SubClassResponsibility_AC();
}

//----------------------------------------------------------------------------------------
// CDependencySpace_AC::RemoveDependencies: 
//----------------------------------------------------------------------------------------
void CDependencySpace_AC::RemoveDependencies(MDependable_AC*	/*anObject*/  )
{
	SubClassResponsibility_AC();
}


//========================================================================================
// CLASS CSimpleDependencySpace_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CSimpleDependencySpace_AC destructor: 
//----------------------------------------------------------------------------------------
CSimpleDependencySpace_AC::~CSimpleDependencySpace_AC()
{
}

//----------------------------------------------------------------------------------------
// CSimpleDependencySpace_AC::AddDependency: 
//----------------------------------------------------------------------------------------
void CSimpleDependencySpace_AC::AddDependency(MDependable_AC* notifier,
										   MDependable_AC* dependent,
										   DependencyLabels_AC dependencyLabel)// Override
{
	fDependents.AddDependency(notifier, dependent, dependencyLabel);
	fNotifiers.AddDependency(dependent, notifier, dependencyLabel);
}

//----------------------------------------------------------------------------------------
// CSimpleDependencySpace_AC::RemoveDependency: 
//----------------------------------------------------------------------------------------
void CSimpleDependencySpace_AC::RemoveDependency(MDependable_AC* notifier,
											  MDependable_AC* dependent,
											  DependencyLabels_AC dependencyLabel)// Override
{
	fDependents.RemoveDependency(notifier, dependent, dependencyLabel);
	fNotifiers.RemoveDependency(dependent, notifier, dependencyLabel);
}

//----------------------------------------------------------------------------------------
// CSimpleDependencySpace_AC::NotifierChanged: 
//----------------------------------------------------------------------------------------
void CSimpleDependencySpace_AC::NotifierChanged(MDependable_AC* notifier,
											 ChangeID_AC theChange,
											 void* changeData)// Override
{
	for (CDependentIterator iter(notifier, this); iter; ++iter)
		iter->DoUpdate(theChange, notifier, changeData, this);
}

//----------------------------------------------------------------------------------------
// CSimpleDependencySpace_AC::RemoveDependencies: 
//----------------------------------------------------------------------------------------
void CSimpleDependencySpace_AC::RemoveDependencies(MDependable_AC* anObject)// Override
{
	{
		for (CDependentIterator iter(anObject, this); iter; ++iter)
		{
			fDependents.DeleteElementsAt(iter.CurrentIndex(), 1);
			fNotifiers.RemoveDependency(iter, anObject, kNoLabel_AC);
		}
	}
	{
		for (CNotifierIterator_AC iter(anObject, this); iter; ++iter)
		{
			fNotifiers.DeleteElementsAt(iter.CurrentIndex(), 1);
			fDependents.RemoveDependency(iter, anObject, kNoLabel_AC);
		}
	}
}

//========================================================================================
// CLASS CStandardDependencySpace_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC constructor
//----------------------------------------------------------------------------------------
CStandardDependencySpace_AC::CStandardDependencySpace_AC() :
	fUpdating(false)
{
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC destructor
//----------------------------------------------------------------------------------------
CStandardDependencySpace_AC::~CStandardDependencySpace_AC()
{
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::NotifierChanged: 
//----------------------------------------------------------------------------------------
void CStandardDependencySpace_AC::NotifierChanged(MDependable_AC* notifier,
											   ChangeID_AC theChange,
											   void* changeData)// Override
{
	if (fUpdating)
		fNestedUpdates.Queue(SNotification_AC(notifier, theChange, changeData));
	else
	{
		NotifyDependents(notifier, theChange, changeData);

		while (fNestedUpdates.GetSize() > 0)
		{
			SNotification_AC n;
			if (fNestedUpdates.Dequeue(n))
				NotifyDependents(n.fNotifier, n.fChange, n.fChangeData);
		}
	}
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::NotifyDependents: 
//----------------------------------------------------------------------------------------
void CStandardDependencySpace_AC::NotifyDependents(MDependable_AC* notifier,
												ChangeID_AC theChange,
												void* changeData)
{
	if (notifier && (theChange != kNoChange_AC))
	{
		fUpdating = true;
		MarkAllDependents(notifier, true);
		UpdateMarkedObjects(theChange, notifier, changeData);
		fUpdating = false;
	}
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::MarkObject: 
//----------------------------------------------------------------------------------------
void CStandardDependencySpace_AC::MarkObject(MDependable_AC* anObject,
										  bool state)
{
	if (anObject)
	{
		ArrayIndex_AC index;
		if (fNotifiers.FindPrimaryObject(anObject, index))
		{
			SDependency_AC* p = static_cast<SDependency_AC*>(fNotifiers.ComputeAddress(index));
			if (state != (p->fLabel < 0))
				p->fLabel = -(p->fLabel);
		}
	}
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::IsObjectMarked: 
//----------------------------------------------------------------------------------------
bool CStandardDependencySpace_AC::IsObjectMarked(MDependable_AC* anObject)
{
	bool returnVal = false;

	ArrayIndex_AC index;
	if (anObject && fNotifiers.FindPrimaryObject(anObject, index))
		returnVal = static_cast<SDependency_AC*>(fNotifiers.ComputeAddress(index))->fLabel < 0;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::MarkAllDependents: 
//----------------------------------------------------------------------------------------
void CStandardDependencySpace_AC::MarkAllDependents(MDependable_AC* notifier,
												 bool state)
{
	if (notifier)
		for (CDependentIterator iter(notifier, this); iter; ++iter)
			if (state != IsObjectMarked(iter))
			{
				MarkObject(iter, state);
				MarkAllDependents(iter, state);
			}
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::UpdateMarkedObjects: 
//----------------------------------------------------------------------------------------
void CStandardDependencySpace_AC::UpdateMarkedObjects(ChangeID_AC theChange,
												   MDependable_AC* changedObject,
												   void* changeData)
{
	for (TemplateArrayIterator_AC<SDependency_AC>iter(&fNotifiers); iter; ++iter)
		if (iter->fLabel < 0)
			HandleUpdate(const_cast<MDependable_AC*>(iter->fPrimaryObject), theChange, changedObject, changeData);
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::HandleUpdate: 
//----------------------------------------------------------------------------------------
void CStandardDependencySpace_AC::HandleUpdate(MDependable_AC* anObject,
											ChangeID_AC theChange,
											MDependable_AC* changedObject,
											void* changeData)
{
	if (IsObjectMarked(anObject))
	{
		MarkObject(anObject, false);

		
		UpdateNotifiers(anObject, theChange, changedObject, changeData);

		anObject->DoUpdate(theChange, changedObject, changeData, this);
	}
}

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC::UpdateNotifiers: 
//----------------------------------------------------------------------------------------
void CStandardDependencySpace_AC::UpdateNotifiers(MDependable_AC* anObject,
											   ChangeID_AC theChange,
											   MDependable_AC* changedObject,
											   void* changeData)
{
	for (CNotifierIterator_AC iter(anObject, this); iter; ++iter)
		HandleUpdate(iter, theChange, changedObject, changeData);
}


//----------------------------------------------------------------------------------------
// CompareDependency_AC: 
//----------------------------------------------------------------------------------------
CompareResult_AC CompareDependency_AC(ArrayIndex_AC index,
								void* yourDataPtr)
{
	const CDependencyComparer_AC* dependencyInfo = static_cast<const CDependencyComparer_AC*>(yourDataPtr);

	return const_cast<CDependencies_AC*>(dependencyInfo->fDependencies)->CompareElements(&dependencyInfo->fTargetDependency, dependencyInfo->fDependencies->ComputeAddress(index));
}


//========================================================================================
// CLASS CDependencyIterator_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CDependencyIterator_AC::CDependencyIterator_AC: CDependentIterator
//----------------------------------------------------------------------------------------
CDependencyIterator_AC::CDependencyIterator_AC(const CDependencies_AC* theArray,
										 const MDependable_AC* theObject) :
	CArrayIterator_AC(theArray, kIterateForward_AC),

	fObject(theObject)
{
	First();
}

//----------------------------------------------------------------------------------------
// CDependencyIterator_AC::First: 
//----------------------------------------------------------------------------------------
MDependable_AC* CDependencyIterator_AC::First()
{
	if (fDynamicArray)
	{
		ArrayIndex_AC foundIndex;

		// Although the lists are ordered the search is a binary search and can return
		// a random entry from the set of matching entries.  First back up the the _real_
		// first matching entry and then we will march forward applying the DoToDependency

		if (dynamic_cast_AC(const CDependencies_AC*,fDynamicArray)->FindPrimaryObject(fObject, foundIndex))
		{
			{									// extra block to ensure iterators are destroyed in fifo order
				for (CArrayIterator_AC iter(fDynamicArray, 1, foundIndex, kIterateBackward_AC); iter.Current(); ++iter)
				{
					if (dynamic_cast_AC(const CDependencies_AC*,fDynamicArray)->PrimaryObjectAt(iter.Current()) == fObject)
						foundIndex = iter.Current();
					else
						break;
				}
				fLowBound = foundIndex;
			}
		}
		else
			fLowBound = kEmptyIndex_AC;
	}

	return fCurrent = CArrayIterator_AC::First() ? dynamic_cast_AC(const CDependencies_AC*,fDynamicArray)->SecondaryObjectAt(CurrentIndex()) : NULL;
}

//----------------------------------------------------------------------------------------
// CDependencyIterator_AC::Next: 
//----------------------------------------------------------------------------------------
MDependable_AC* CDependencyIterator_AC::Next()
{
	return fCurrent = CArrayIterator_AC::Next() && (dynamic_cast_AC(const CDependencies_AC*,fDynamicArray)->PrimaryObjectAt(CurrentIndex()) == fObject) ? 
		dynamic_cast_AC(const CDependencies_AC*,fDynamicArray)->SecondaryObjectAt(CurrentIndex()) : NULL;
}

//========================================================================================
// CLASS CDependentIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CDependentIterator::CDependentIterator: 
//----------------------------------------------------------------------------------------
CDependentIterator::CDependentIterator(const MDependable_AC* theObject,
									   CSimpleDependencySpace_AC* dependencySpace) :
	CDependencyIterator_AC(dependencySpace ? &dependencySpace->fDependents : NULL, theObject)
{
}

//========================================================================================
// CLASS CNotifierIterator_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CNotifierIterator_AC::CNotifierIterator_AC: 
//----------------------------------------------------------------------------------------
CNotifierIterator_AC::CNotifierIterator_AC(const MDependable_AC* theObject,
									 CSimpleDependencySpace_AC* dependencySpace) :
	CDependencyIterator_AC(dependencySpace ? &dependencySpace->fNotifiers : NULL, theObject)
{
}

//----------------------------------------------------------------------------------------
// End of UDependencies.cp
