//----------------------------------------------------------------------------------------
// CIdleTimeDispenser.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Patterns

#ifndef __CIdleTimeDispenser_AC__
#include "CIdleTimeDispenser_AC.h"
#endif

#ifndef __CIdleTimeEngine_AC__
#include "CIdleTimeEngine_AC.h"
#endif

#ifndef __MIdler_AC__
#include "MIdler_AC.h"
#endif

#ifndef __CAbstractTask_AC__
#include "CAbstractTask_AC.h"
#endif

// Streams

#if qTrace_AC
#	ifndef __CTraceStream_AC__
#	include "CTraceStream_AC.h"
#	endif
#endif

// Core

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// Containers

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

//----------------------------------------------------------------------------------------
// ctor
//----------------------------------------------------------------------------------------

CIdleTimeDispenser_AC::CIdleTimeDispenser_AC()
{
}

//----------------------------------------------------------------------------------------
// dtor
//----------------------------------------------------------------------------------------

CIdleTimeDispenser_AC::~CIdleTimeDispenser_AC()
{
	ReleaseYieldTask();

	if(fIdlers)
	{
		MIdler_AC* current;
		
		while(fIdlers->GetLength() > 0)
		{
			current = fIdlers->GetFirstElement();
			fIdlers->Remove(current);
			if(current) 
				current->SetEngine(NULL);
		}
	}
	
}

//----------------------------------------------------------------------------------------
//  CIdleTimeDispenser_AC::Idle
//----------------------------------------------------------------------------------------

void CIdleTimeDispenser_AC::Idle()
{
	if(fIdlers && fIdlers->GetLength())
	{
		CRefCountingPtr_AC<CIdleTimeDispenser_AC> selfPtr = this;

		{
			// make the iterator in it's own scope
			CList::CIterator iter = fIdlers->First();
					
			while(iter.IsValid())
			{
				CRefCountingPtr_AC<MIdler_AC> currentIdler = *iter;
		
				currentIdler->Idle();
			
				iter.Next();
			}
		}	

		if(fYieldTask)
			fYieldTask->DoTask();
		
	}
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::AttachIdler
//----------------------------------------------------------------------------------------

void CIdleTimeDispenser_AC::AttachIdler(MIdler_AC* inIdler)
{
	::AssertNotNULL_AC(inIdler);
	
	::Assertion_AC(HasIdleTimeEngine(), "Attaching Idler to Engineless IdleTime dispenser");
	
	if(inIdler != this)
		// Can't give idle time to self if we want the machine to like, not hang.
	{
		if(!fIdlers)
			fIdlers = TH_new CList;
		
		if( !fIdlers->InList(inIdler))
		{
			// only save if not in list.
		
			fIdlers->Append(inIdler);
			inIdler->SetEngine(this);
		}
	}
	
	if(fEngine && !fEngine->IsRunning())
		fEngine->Run();
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::DetachIdler
//----------------------------------------------------------------------------------------

void CIdleTimeDispenser_AC::DetachIdler(MIdler_AC* inIdler)
{
	// mark removed idler to NULL. Delete link in list 
	// later under controlled situations.

	::AssertNotNULL_AC(inIdler);

	if(fIdlers)
	{
		inIdler->SetEngine(NULL);
		fIdlers->Remove(inIdler);
	}
	
	if((!fIdlers || fIdlers->GetLength() == 0) && 
		(fEngine && fEngine->IsRunning()))
	{
		fEngine->Stop();
	}	 
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::SetYieldTask
//----------------------------------------------------------------------------------------

void CIdleTimeDispenser_AC::SetYieldTask(CAbstractTask_AC* yieldTask)
{
	ReleaseYieldTask();
	
	fYieldTask = yieldTask;
	if(fYieldTask) 
	{
		fYieldTask->SetIsRepeater(true);
		fYieldTask->SetEngine(this);
	}
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::ReleaseYieldTask
//----------------------------------------------------------------------------------------

void CIdleTimeDispenser_AC::ReleaseYieldTask()
{
	if(fYieldTask)
	{
		fYieldTask->StopIdling();
		fYieldTask.reset();
	}
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::GetYieldTask
//----------------------------------------------------------------------------------------

CAbstractTask_AC* CIdleTimeDispenser_AC::GetYieldTask() const
{
	return fYieldTask;
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::AquireIdleTimeEngine
//----------------------------------------------------------------------------------------

void CIdleTimeDispenser_AC::AquireIdleTimeEngine(CIdleTimeEngine_AC* inEngine, CAbstractTask_AC* yieldTask)
{
	ReleaseIdleTimeEngine();
	fEngine = inEngine;
	fEngine->SetDispenser(this);
	
	if(yieldTask)
		SetYieldTask(yieldTask);
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::ReleaseIdleTimeEngine
//----------------------------------------------------------------------------------------

void CIdleTimeDispenser_AC::ReleaseIdleTimeEngine()
{
	if(fEngine)
	{
		fEngine->SetDispenser(NULL);
		fEngine->Stop();
		fEngine.reset();
	}
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::GetIdleTimeEngine
//----------------------------------------------------------------------------------------

CIdleTimeEngine_AC* CIdleTimeDispenser_AC::GetIdleTimeEngine() const
{
	return fEngine;
}

//----------------------------------------------------------------------------------------
// CIdleTimeDispenser_AC::HasIdleTimeEngine
//----------------------------------------------------------------------------------------

bool CIdleTimeDispenser_AC::HasIdleTimeEngine() const
{
	return !fEngine.IsNULL();
}