//----------------------------------------------------------------------------------------
// CReactor_AC.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

// Patterns

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

// containers

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif
	

// streams

#if qPatternsTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif

	unsigned long gEventCounter = 0;
	#include <typeinfo.h>
	#define kTab "    "
#endif

//----------------------------------------------------------------------------------------
// constructor
//----------------------------------------------------------------------------------------

CReactor_AC::CReactor_AC()
	: fFiltering(true)
{
	fDependencies = TH_new CDependencyList;
}

//----------------------------------------------------------------------------------------
// destructor
//----------------------------------------------------------------------------------------

CReactor_AC::~CReactor_AC()
{
	UnregisterAll();
}

//----------------------------------------------------------------------------------------
// Post Event
//----------------------------------------------------------------------------------------

void CReactor_AC::PostEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender)
{	
#if qPatternsTrace_AC	

	long eventNumber = ++gEventCounter;

	char lilStr[8];

	TRACE 	<< MediumTracePriority_AC 
			<< "\n[-POSTING-] event #"
			<< eventNumber
			<< "\n";
		
		// event	
	
	TRACE	<< kTab
			<< "Event: " 
			<< CDebug_AC::OSTypeToStr(inEvent->GetEventID(), lilStr)
//			<< "/" 
//			<< CNetEvent_AC::What(inEvent->GetEventID())
			<< "\n";
	
	TRACE	<< kTab 
			<< "From:  " 
			<< typeid(*inSender).name()
			<< endl_AC;

#endif

	::AssertNotNULL_AC(inEvent);
	::AssertNotNULL_AC(inSender);
	
	CRefCountingPtr_AC<MEventTransceiver_AC> senderPtr = inSender;

	CRefCountingPtr_AC<CEvent_AC> eventPtr = inEvent;
		// Aquire a lock on the event and sender, which may be deleted later.
		
	CDependencyList::CIterator listWalker = fDependencies->First();
		// Make an iterator to iterate over the map. We need to make this on 
		// the stack here because the Reactor supports nested iterations, i.e.
		// the call to ReceiveEvent may cause another event to be posted.
						
	while(listWalker.IsValid())
	{
		if(	!fFiltering || (*listWalker).fSender == inSender ) 
		{
			try
			{
				CRefCountingPtr_AC<MEventTransceiver_AC> curReceiver = (*listWalker).fReceiver;
			
				curReceiver->ReceiveEvent(inEvent, inSender);
					// send the event to the receiver
				if(inEvent->WasConsumed())
					break;
					// if the event was consumed, this means don't propagate it.

			}
			catch(CException_AC& inEx)
			{
				PostEvent( TH_new CEvent_AC( CPatternEvent_AC::kError, inEx.GetError()), inSender);		
			}		
		}
		
		listWalker.Next();
			// increment the iterator.
	}
}

//----------------------------------------------------------------------------------------
// Unregister All
//----------------------------------------------------------------------------------------

void CReactor_AC::UnregisterAll()
{
	CDependencyList::CIterator listWalker = fDependencies->First();
	
	while(listWalker.IsValid())
	{
		MEventTransceiver_AC* receiver = (*listWalker).fReceiver;
		fDependencies->Remove(listWalker);
		receiver->RegistrationChanged(this, false);
	
		listWalker.Next();
	}
}

//----------------------------------------------------------------------------------------
// RegisterForEvents
//----------------------------------------------------------------------------------------

void CReactor_AC::RegisterForEvents(  MEventTransceiver_AC *inReceiver, MEventTransceiver_AC* inSender )
{			
	::AssertNotNULL_AC(inReceiver);
	::AssertNotNULL_AC(inSender);

	CEventDependency toInsert(inReceiver, inSender);

	if(	!fDependencies->InList(toInsert))
	{
		// only insert the dependency if its not in the list already.
		
		fDependencies->Append(toInsert);	
		
		if(inReceiver) inReceiver->RegistrationChanged(this, true);
	}
}

//----------------------------------------------------------------------------------------
// IsRegisteredDependency
//----------------------------------------------------------------------------------------

bool CReactor_AC::IsRegisteredDependency(MEventTransceiver_AC* inReceiver, MEventTransceiver_AC* inSender)
{
	return fDependencies->InList(CEventDependency(inReceiver, inSender));
}

//----------------------------------------------------------------------------------------
// Unregister
//----------------------------------------------------------------------------------------

void CReactor_AC::Unregister(  MEventTransceiver_AC *inReceiver, MEventTransceiver_AC* inSender )
{
	::AssertNotNULL_AC(inReceiver);
	fDependencies->Remove(CEventDependency(inReceiver, inSender));
	inReceiver->RegistrationChanged(this, false);
}

//----------------------------------------------------------------------------------------
// UnregisterSender
//----------------------------------------------------------------------------------------

void CReactor_AC::UnregisterSender(MEventTransceiver_AC* inSender)
{
	::AssertNotNULL_AC(inSender);
	
	CDependencyList::CIterator listWalker = fDependencies->First();
	
	while(listWalker.IsValid())
	{
		if((*listWalker).fSender == inSender)
		{
			MEventTransceiver_AC* receiver = (*listWalker).fReceiver;
			fDependencies->Remove(listWalker);
			receiver->RegistrationChanged(this, false);
			break;
		}

		listWalker.Next();
	}
}

//----------------------------------------------------------------------------------------
// UnregisterReceiver
//----------------------------------------------------------------------------------------

void CReactor_AC::UnregisterReceiver(MEventTransceiver_AC* inReceiver)
{
	::AssertNotNULL_AC(inReceiver);
	
	CDependencyList::CIterator listWalker = fDependencies->First();
	
	while(listWalker.IsValid())
	{
		if((*listWalker).fReceiver == inReceiver)
		{
			MEventTransceiver_AC* receiver = (*listWalker).fReceiver;
			fDependencies->Remove(listWalker);
			receiver->RegistrationChanged(this, false);
			break;
		}

		listWalker.Next();
	}
}

//----------------------------------------------------------------------------------------
// Unregister
//----------------------------------------------------------------------------------------
	
void CReactor_AC::Unregister(MEventTransceiver_AC* inTransceiver)
{
	::AssertNotNULL_AC(inTransceiver);
	
	CDependencyList::CIterator listWalker = fDependencies->First();
	
	while(listWalker.IsValid())
	{
		if((*listWalker).fReceiver == inTransceiver)
		{
			MEventTransceiver_AC* receiver = (*listWalker).fReceiver;
			fDependencies->Remove(listWalker);
			receiver->RegistrationChanged(this, false);
		}
		else
		if((*listWalker).fSender == inTransceiver)
		{
			MEventTransceiver_AC* receiver = (*listWalker).fReceiver;
			fDependencies->Remove(listWalker);
			receiver->RegistrationChanged(this, false);
		}
		
		listWalker.Next();
	}
}


