//----------------------------------------------------------------------------------------
// ClassRegistration_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __ClassRegistration_AC__
#include "ClassRegistration_AC.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

#ifndef __MStreamable_AC__
#include "MStreamable_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

// ANSI

#ifndef __STRING__
#include <string.h>
#endif

//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC : Stores a list of NameRecords by Name
//----------------------------------------------------------------------------------------
class CRegisteredClasses_AC : public CSortedDynamicArray_AC
{
public:
	CRegisteredClasses_AC();

	virtual ~CRegisteredClasses_AC();

	virtual CompareResult_AC CompareElements(const void* item1,
													const void* item2) const;// override

	CClassRegistration_AC* FindByClassName(const char* className);
	CClassRegistration_AC* FindByClassID(ClassID_AC inClassID);

	void RegisterClass(const char* inTypeName, ConstructorProcPtr_AC constructor);

protected:
	static CompareResult_AC TestClassName(ArrayIndex_AC index,
									   void* yourDataPtr);
};

struct SignatureRec
{
	const char* fTypeName;
	IDType fSignature;

	SignatureRec(const char* inTypeName, IDType inSignature) :
	fTypeName(inTypeName),
	fSignature(inSignature) { }
};

//----------------------------------------------------------------------------------------
// CRegisteredSignatures_AC : Stores a list of SignatureRecords by Signature
//----------------------------------------------------------------------------------------
class CRegisteredSignatures_AC : public CSortedDynamicArray_AC
{
public:
	CRegisteredSignatures_AC();

	virtual ~CRegisteredSignatures_AC();

	virtual CompareResult_AC CompareElements(const void* item1,
													   const void* item2) const;// override

	const char* FindBySignature(IDType signature);

	void RegisterSignature(const char* inTypeName, IDType signature);

protected:
	static CompareResult_AC TestSignature(ArrayIndex_AC index,
									   void* yourDataPtr);
};


//========================================================================================
// GLOBALS
//========================================================================================
CRegisteredClasses_AC pRegisteredClasses;
CRegisteredSignatures_AC pRegisteredSignatures;

//----------------------------------------------------------------------------------------
// NewObjectBySignature: 
//----------------------------------------------------------------------------------------
MStreamable_AC* NewObjectBySignature_AC(IDType signature,
							  const char* className)
{
	return *className ? CClassRegistration_AC::NewByClassName(className): CClassRegistration_AC::NewBySignature(signature);
}

//========================================================================================
// CLASS CClassRegistration_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// Static Member Functions:
//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------
// CClassRegistration_AC::GetClassDescFromClassID:
//----------------------------------------------------------------------------------------
CClassRegistration_AC* CClassRegistration_AC::GetClassDescFromClassID(const ClassID_AC classID)
{
	return pRegisteredClasses.FindByClassID(classID);
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::GetClassDescFromClassName:
//----------------------------------------------------------------------------------------
CClassRegistration_AC* CClassRegistration_AC::GetClassDescFromClassName(const char* className)
{
	return pRegisteredClasses.FindByClassName(className);
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::GetClassDescFromSignature:
//----------------------------------------------------------------------------------------
CClassRegistration_AC* CClassRegistration_AC::GetClassDescFromSignature(IDType signature)
{
	return pRegisteredClasses.FindByClassName(pRegisteredSignatures.FindBySignature(signature));
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::GetSignatureFromClassName:
//----------------------------------------------------------------------------------------
IDType CClassRegistration_AC::GetSignatureFromClassName(const char* className)
{
	IDType returnVal = NULL;
	CStr255_AC localClassName = className;

	for (TemplateArrayIterator_AC<SignatureRec>iter(&pRegisteredSignatures); iter.Current(); ++iter)
		if (::EqualString(CStr255_AC(iter->fTypeName), localClassName, false, true))
		{
			returnVal = iter->fSignature;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::NewByClassID:
//----------------------------------------------------------------------------------------
MStreamable_AC* CClassRegistration_AC::NewByClassID(const ClassID_AC classID)
{
	MStreamable_AC* returnVal = NULL;

	CClassRegistration_AC* classDesc = CClassRegistration_AC::GetClassDescFromClassID(classID);

	if (classDesc)
		returnVal = NewByClassDesc(classDesc);
	else
	{
#if qDebug
		::ProgramBreak_AC("The application doesn't contain the class with class ID " + CStr255_AC(static_cast<unsigned long>(classID)) + ".");
#endif

//SRF		gErrorParm3 = CStr255_AC(static_cast<unsigned long>(classID));			// show ID of the class 
//SRF		::ThrowException_AC(errMissingClass, 0);
		::ThrowException_AC(minErr, 0);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::NewByClassName:
//----------------------------------------------------------------------------------------
MStreamable_AC* CClassRegistration_AC::NewByClassName(const char* className)
{
	MStreamable_AC* returnVal = NULL;

	CClassRegistration_AC* classDesc = CClassRegistration_AC::GetClassDescFromClassName(className);

	if (classDesc)
		returnVal = NewByClassDesc(classDesc);
	else
	{
#if qDebug
		::ProgramBreak_AC("The application doesn't contain the class " + CStr255_AC(className) + ".");
#endif

//SRF		gErrorParm3 = className;				// show name of the class 
//SRF		::ThrowException_AC(errMissingClass, 0);
		::ThrowException_AC(minErr, 0);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::NewBySignature:
//----------------------------------------------------------------------------------------
MStreamable_AC* CClassRegistration_AC::NewBySignature(IDType signature)
{
	MStreamable_AC* returnVal = NULL;

	CClassRegistration_AC* classDesc = CClassRegistration_AC::GetClassDescFromSignature(signature);

	if (classDesc)
		returnVal = NewByClassDesc(classDesc);
	else
	{
#if qDebugMsg
		ProgramReport_AC(false, "signature='%4s'\n", &signature);
		::ProgramBreak_AC("Unable to find class for the given signature");
#endif

//SRF		::ThrowException_AC(errMissingSignature, 0);
		::ThrowException_AC(minErr, 0);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::CClassRegistration_AC:
//----------------------------------------------------------------------------------------
CClassRegistration_AC::CClassRegistration_AC(const char* inTypeName,
					ConstructorProcPtr_AC constructor,
					ClassID_AC inClassID) :
	fTypeName(inTypeName),
	fDefaultConstructor(constructor),
	fClassID(inClassID)
{
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::RegisterClass:
//----------------------------------------------------------------------------------------
void CClassRegistration_AC::RegisterClass(const char* inTypeName, ConstructorProcPtr_AC constructor)
{
	pRegisteredClasses.RegisterClass(inTypeName, constructor);
}

//----------------------------------------------------------------------------------------
// CClassRegistration_AC::RegisterSignature:
//----------------------------------------------------------------------------------------
void CClassRegistration_AC::RegisterSignature(const char* inTypeName, IDType signature)
{
	pRegisteredSignatures.RegisterSignature(inTypeName, signature);
}

//========================================================================================
// CLASS CTestClassNameParams
//========================================================================================
class CTestClassNameParams
{
public:
	const CStr255_AC fClassName;
	CSortedDynamicArray_AC* fDynamicArray;

	inline CTestClassNameParams(const char* theClassName,
						 CSortedDynamicArray_AC* theDynamicArray) :
		fClassName(theClassName),
		fDynamicArray(theDynamicArray)
		{ }
};

//========================================================================================
// CLASS CTestSignatureParams
//========================================================================================
class CTestSignatureParams
{
public:
	IDType fSignature;
	CSortedDynamicArray_AC* fDynamicArray;

	inline CTestSignatureParams(IDType signature,
						 CSortedDynamicArray_AC* theDynamicArray) :
	fSignature(signature),
	fDynamicArray(theDynamicArray)
	{ }

};

//========================================================================================
// CLASS CRegisteredClasses_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC constructor
//----------------------------------------------------------------------------------------
CRegisteredClasses_AC:: CRegisteredClasses_AC() :
CSortedDynamicArray_AC(0, sizeof(CClassRegistration_AC))
{
}

//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC destructor
//----------------------------------------------------------------------------------------
CRegisteredClasses_AC::~CRegisteredClasses_AC()
{
}

//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC::CompareElements:
//----------------------------------------------------------------------------------------
CompareResult_AC CRegisteredClasses_AC::CompareElements(const void* item1,
													const void* item2) const
{
	const CClassRegistration_AC* classDesc1 = *(const CClassRegistration_AC**)item1;
	const CClassRegistration_AC* classDesc2 = *(const CClassRegistration_AC**)item2;

	// Classnames are not international strings!
	return ::RelString(CStr255_AC(classDesc1->GetName()), CStr255_AC(classDesc2->GetName()), false, true);
}

//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC::FindByClassName:
//----------------------------------------------------------------------------------------
CClassRegistration_AC* CRegisteredClasses_AC::FindByClassName(const char* className)
{
	ArrayIndex_AC index;
	CClassRegistration_AC * classDesc = NULL;
	CTestClassNameParams params(className, this);

	if (DoSearchElement(TestClassName, &params, index))
		classDesc = (CClassRegistration_AC*)ComputeAddress(index);

	return classDesc;
}

//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC::FindByClassID:
//----------------------------------------------------------------------------------------
CClassRegistration_AC* CRegisteredClasses_AC::FindByClassID(ClassID_AC inClassID)
{
	CClassRegistration_AC* returnVal = NULL;

	for (TemplateArrayIterator_AC<CClassRegistration_AC>iter(this); iter.Current(); ++iter)
		if (iter->GetClassID() == inClassID)
		{
			returnVal = iter;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC::RegisterClass: Register or re-register a type and a class
//----------------------------------------------------------------------------------------
void CRegisteredClasses_AC::RegisterClass(const char* inTypeName, ConstructorProcPtr_AC constructor)
{
	static ClassID_AC pNextClassID;

	CTestClassNameParams params(inTypeName, this);
	ArrayIndex_AC index;
	if (!DoSearchElement(TestClassName, &params, index))
		InsertElementsBefore(index, &CClassRegistration_AC(inTypeName, constructor, ++pNextClassID), 1);
}

//----------------------------------------------------------------------------------------
// CRegisteredClasses_AC::TestClassName: 
//----------------------------------------------------------------------------------------
CompareResult_AC CRegisteredClasses_AC::TestClassName(ArrayIndex_AC index,
												  void* yourDataPtr)
{
	CTestClassNameParams& params = *static_cast<CTestClassNameParams*>(yourDataPtr);

	CClassRegistration_AC* item = static_cast<CClassRegistration_AC*>(params.fDynamicArray->ComputeAddress(index));

	// Classnames are not international strings!
	return ::RelString(params.fClassName, CStr255_AC(item->GetName()), false, true);
}


//========================================================================================
// CLASS CRegisteredSignatures_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CRegisteredSignatures_AC constructor
//----------------------------------------------------------------------------------------
CRegisteredSignatures_AC::CRegisteredSignatures_AC() :
CSortedDynamicArray_AC(0, sizeof(SignatureRec))
{
}

//----------------------------------------------------------------------------------------
// CRegisteredSignatures_AC destructor
//----------------------------------------------------------------------------------------
CRegisteredSignatures_AC::~CRegisteredSignatures_AC()
{
}

//----------------------------------------------------------------------------------------
// CRegisteredSignatures_AC::CompareElements:
//----------------------------------------------------------------------------------------
CompareResult_AC CRegisteredSignatures_AC::CompareElements(const void* item1,
													   const void* item2) const
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	const SignatureRec* rec1 = *(const SignatureRec**)item1;
	const SignatureRec* rec2 = *(const SignatureRec**)item2;

	if (rec1->fSignature > rec2->fSignature)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (rec1->fSignature < rec2->fSignature)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CRegisteredSignatures_AC::FindBySignature:
//----------------------------------------------------------------------------------------
const char* CRegisteredSignatures_AC::FindBySignature(IDType signature)
{
	SignatureRec* item = NULL;

	CTestSignatureParams params(signature, this);

	ArrayIndex_AC index;
	if (DoSearchElement(TestSignature, &params, index))
		item = static_cast<SignatureRec*>(ComputeAddress(index));

	return item ? item->fTypeName : NULL;
}


//----------------------------------------------------------------------------------------
// CRegisteredSignatures_AC::RegisterSignature: Register or re-register a type and a class
//----------------------------------------------------------------------------------------
void CRegisteredSignatures_AC::RegisterSignature(const char* inTypeName, IDType signature)
{
	// Replace or add signature as necessary 

	CTestSignatureParams params(signature, this);
	ArrayIndex_AC index;
	if (DoSearchElement(TestSignature, &params, index))
	{
		// replace
		ReplaceElementsAt(index, &SignatureRec(inTypeName, signature), 1);
	}
	else
		InsertElementsBefore(index, &SignatureRec(inTypeName, signature), 1);
}

//----------------------------------------------------------------------------------------
// CRegisteredSignatures_AC::TestSignature: 
//----------------------------------------------------------------------------------------
CompareResult_AC CRegisteredSignatures_AC::TestSignature(ArrayIndex_AC index,
													 void* yourDataPtr)
{
	CompareResult_AC returnVal = kItemEqualCriteria_AC;

	CTestSignatureParams& params = *static_cast<CTestSignatureParams*>(yourDataPtr);

	SignatureRec* item = static_cast<SignatureRec*>(params.fDynamicArray->ComputeAddress(index));

	if (item->fSignature > params.fSignature)
		returnVal = kItemGreaterThanCriteria_AC;
	else if (item->fSignature < params.fSignature)
		returnVal = kItemLessThanCriteria_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// End of ClassRegistration_AC.cp


