//----------------------------------------------------------------------------------------
// MEventTransceiver_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Patterns

#ifndef __MEventTransceiver_AC__
#include "MEventTransceiver_AC.h"
#endif

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __PatternEvents_AC__
#include "PatternEvents_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif

#ifndef __CReactorFactory_AC__
#include "CReactorFactory_AC.h"
#endif

// containers

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

//----------------------------------------------------------------------------------------
// ctor
//----------------------------------------------------------------------------------------

MEventTransceiver_AC::MEventTransceiver_AC()
{
}

//----------------------------------------------------------------------------------------
// ctor
//----------------------------------------------------------------------------------------

MEventTransceiver_AC::MEventTransceiver_AC(CReactor_AC* inReactor)
	: fReactor(inReactor)
{
}

//----------------------------------------------------------------------------------------
// dtor
//----------------------------------------------------------------------------------------

MEventTransceiver_AC::~MEventTransceiver_AC()
{
	StopTransceivingAllEvents();
		// Stop sending and receiving events.
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::StartSendingEventsTo
//----------------------------------------------------------------------------------------

void MEventTransceiver_AC::StartSendingEventsTo(MEventTransceiver_AC* inReceiver)
{
	::AssertNotNULL_AC(inReceiver);

	if(!HasReactor())
		UseDefaultReactor();

	GetReactor()->RegisterForEvents(inReceiver, this);
}

//----------------------------------------------------------------------------------------
//  MEventTransceiver_AC::StopSendingEventsTo
//----------------------------------------------------------------------------------------
	
void MEventTransceiver_AC::StopSendingEventsTo(MEventTransceiver_AC* inReceiver)
{
	::AssertNotNULL_AC(inReceiver);

	if(!HasReactor())
		UseDefaultReactor();
	
	GetReactor()->Unregister(inReceiver, this);
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::StopSendingAllEvents
//----------------------------------------------------------------------------------------

void MEventTransceiver_AC::StopSendingAllEvents()
{
	if(HasReactor())
		GetReactor()->Unregister(this);
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::ReceiveEvent
//----------------------------------------------------------------------------------------


void MEventTransceiver_AC::ReceiveEvent(	CEvent_AC* /*inEvent*/, 
											MEventTransceiver_AC* /*inTransceiver*/)
{
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::SetReactor
//----------------------------------------------------------------------------------------

void MEventTransceiver_AC::SetReactor(CReactor_AC* inReactor)
{
	if(!fReactor)
		fReactor = inReactor;
	else	
	if(fReactor != inReactor)
	{
		fReactor->Unregister(this);
			// this requires registering
		fReactor = inReactor; 
	}
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::GetReactor
//----------------------------------------------------------------------------------------

CReactor_AC* MEventTransceiver_AC::GetReactor()
{
	return fReactor; 
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::PostEvent
//----------------------------------------------------------------------------------------

void MEventTransceiver_AC::PostEvent( CEvent_AC* inEvent)
{
	::AssertNotNULL_AC(inEvent);
	
	// Don't create a Reactor here because if there's no reactor, there's no
	// event receivers, and there ain't any point!
		
	if(HasReactor())
	{
		GetReactor()->PostEvent(inEvent, this); 
	}
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::RegistrationChanged
//----------------------------------------------------------------------------------------

void MEventTransceiver_AC::RegistrationChanged(CReactor_AC* inReactor, bool registering)
{
	if(registering)
	{
		if(!fRegisteredWith)
			fRegisteredWith = TH_new CReactorList;
		
		fRegisteredWith->Push(inReactor);
	}
	else
	if(fRegisteredWith)
	{
		fRegisteredWith->Remove(inReactor);
	}
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::StopReceivingAllEvents
//----------------------------------------------------------------------------------------

void MEventTransceiver_AC::StopReceivingAllEvents()
{
	if(fRegisteredWith)
	{
		CReactorList::CIterator iterator = fRegisteredWith->First();
		
		while(iterator.IsValid())
		{
			(*iterator)->UnregisterReceiver(this);
		
			iterator.Next();
		}
	}
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::StopTransceivingAllEvents
//----------------------------------------------------------------------------------------
	
void MEventTransceiver_AC::StopTransceivingAllEvents()
{
	StopSendingAllEvents();
	StopReceivingAllEvents();
}

//----------------------------------------------------------------------------------------
// MEventTransceiver_AC::UseDefaultReactor
//----------------------------------------------------------------------------------------

void MEventTransceiver_AC::UseDefaultReactor(bool replaceCurrentReactor)
{
	if(replaceCurrentReactor || !HasReactor())
		SetReactor( CReactorFactory_AC::CreateDefaultReactor() );
}

