//----------------------------------------------------------------------------------------
// CAsyncStream_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CAsyncStream_AC__
#define __CAsyncStream_AC__

#ifndef __Streams_AC__
#include "Streams_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------------------------

// Defaultable parameter for Read
enum {
	kAllAvailableData = -1
};

//----------------------------------------------------------------------------------------
// CLASS CAsyncStream
//----------------------------------------------------------------------------------------
class EXPORT_AC CAsyncStream_AC
{
public:
	CAsyncStream_AC();
	CAsyncStream_AC(long size);
	~CAsyncStream_AC();
	
			// specific data types
	CAsyncStream_AC& operator << (const char* inPtr); // null terminated cstring

	CAsyncStream_AC& operator << (long n)
	{	Write(&n, sizeof(n)); return *this; }
	CAsyncStream_AC& operator << (unsigned long n )
	{	Write(&n, sizeof(n)); return *this; }
	CAsyncStream_AC& operator << (short n)
	{	Write(&n, sizeof(n)); return *this; }
	CAsyncStream_AC& operator << (unsigned short n)
	{	Write(&n, sizeof(n)); return *this; }
	
	CAsyncStream_AC& operator >> (long& n)
	{	Read(&n, sizeof(n)); return *this; }
	CAsyncStream_AC& operator >> (unsigned long& n)
	{	Read(&n, sizeof(n)); return *this; }
	CAsyncStream_AC& operator >> (short& n )
	{	Read(&n, sizeof(n)); return *this; }
	CAsyncStream_AC& operator >> (unsigned short& n)
	{	Read(&n, sizeof(n)); return *this; }

	// raw data streaming
	
	void Write(const void*, long inSize);

	void Read(void* outData, long inAmount = kAllAvailableData);
		// defaultable argument => read in all available data
	
	
	bool HasData() const;

		// note that these can change at interrupt time so the number you
		// get back may not be accurate immediately following the call.
		// however, if you operatate on the assumption that the the number
		// is correct until you call it again, or extract data, you'll
		// be ok.
	
	long GetDataSize();

	long GetFreeSpaceSize();
	
	bool IsFull();
	
private:
	long CalculateFreeSpace( char *writerCopy );	
	
private:
	
	char* fWriter;	
		// if fWriter == 0 then the queue is full.
	char* fSavedWriter; 
		// writer only changes during a write	
	char* fReader;
		// reader only changes during a read
		
	char* fBuffer;
	char* fEndAddress;
	
	long fSize;
};

//----------------------------------------------------------------------------------------

inline long CAsyncStream_AC::GetDataSize()
{
	return fSize - CalculateFreeSpace(fWriter);
}

//----------------------------------------------------------------------------------------

inline long CAsyncStream_AC::GetFreeSpaceSize()
{
	return CalculateFreeSpace(fWriter);
}

#endif