//----------------------------------------------------------------------------------------
// CTraceStream_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTraceStream_AC__
#define __CTraceStream_AC__

#ifndef __Streams_AC__
#include "Streams_AC.h"
#endif

#if qTrace_AC

// IMPORTANT: Tracing is not supported during static construction or destruction.

#ifndef __CCharOutputSink_AC__
#include "CCharOutputSink_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CTraceStream_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CTraceStream_AC : public CCharOutputSink_AC
{
public:

	enum { 	
			kOff = 0,
			kLowPriority  = 1, 
			kHighPriority  = 10, 
			kMedPriority  = kHighPriority/2,
			kDefaultPriority = kHighPriority
			};
	
	CTraceStream_AC();
	virtual ~CTraceStream_AC();

	static CTraceStream_AC& GetStreamRef();
	
	static void AddStream(CStream_AC& in); 
		
	static void RemoveStream(CStream_AC& stream);

	void SetPriorityFilter(short level)
	{	fPrevPriorityFilter = fPriorityFilter;
		fPriorityFilter = level; }
	
	short GetPriorityFilter() const
	{	return fPriorityFilter; }
	
	void ResetPriorityFilter()
	{	short temp = fPriorityFilter;
		fPriorityFilter = fPrevPriorityFilter;
		fPrevPriorityFilter = temp; }
	
	void SetPriority(short level)
	{	fPrevPriority = fPriority; 
		fPriority = level; }
	
	void ResetPriority()
	{	short temp = fPriority;
		fPriority = fPrevPriority;
		fPrevPriority = temp; }
	
	short GetPriority() const
	{	return fPriority; }
		
	virtual void WriteCharBytes(const char* inBytes, long inAmt);
	
	static CTraceStream_AC* Instance()
	{	return fgSelf; }
	
private:
	short fPriorityFilter, fPrevPriorityFilter, fPriority, fPrevPriority;
	
	static CAutoPtr_AC<CTraceStream_AC> fgSelf;
};

//----------------------------------------------------------------------------------------
// Convenience macro for tracing

#define TRACE CTraceStream_AC::GetStreamRef()

//----------------------------------------------------------------------------------------
// class SetTracePriorityFilter_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC SetTracePriorityFilter_AC
{
public:
	SetTracePriorityFilter_AC(short level) : fLevel(level) {}
	short fLevel;
};

//----------------------------------------------------------------------------------------
// class SetTracePriority_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC SetTracePriority_AC
{
public:
	SetTracePriority_AC(short level) : fLevel(level) {}
	short fLevel;
};

//----------------------------------------------------------------------------------------
// inlines
//----------------------------------------------------------------------------------------

inline CTraceStream_AC& operator << (CTraceStream_AC& inStream, SetTracePriorityFilter_AC& inSetter)
{	inStream.SetPriorityFilter(inSetter.fLevel); return inStream; }

//----------------------------------------------------------------------------------------

inline CTraceStream_AC& operator << (CTraceStream_AC& inStream, SetTracePriority_AC& inSetter)
{	inStream.SetPriority(inSetter.fLevel); return inStream; }

//----------------------------------------------------------------------------------------

inline  CStream_AC& LowTracePriority_AC (CStream_AC& stream)
{
	CTraceStream_AC& trace = dynamic_cast<CTraceStream_AC&>(stream);
   	trace.SetPriority(CTraceStream_AC::kLowPriority);
    return stream;
}

//----------------------------------------------------------------------------------------

inline  CStream_AC& MediumTracePriority_AC (CStream_AC& stream)
{
	CTraceStream_AC& trace = dynamic_cast<CTraceStream_AC&>(stream);
   	trace.SetPriority(CTraceStream_AC::kMedPriority);
    return stream;
}

//----------------------------------------------------------------------------------------

inline  CStream_AC& HighTracePriority_AC (CStream_AC& stream)
{
	CTraceStream_AC& trace = dynamic_cast<CTraceStream_AC&>(stream);
   	trace.SetPriority(CTraceStream_AC::kHighPriority);
    return stream;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& ResetTracePriority_AC (CStream_AC& stream)
{
	CTraceStream_AC& trace = dynamic_cast<CTraceStream_AC&>(stream);
   	trace.ResetPriority();
    return stream;
}

//----------------------------------------------------------------------------------------

inline CStream_AC& ResetTracePriorityFilter_AC (CStream_AC& stream)
{
	CTraceStream_AC& trace = dynamic_cast<CTraceStream_AC&>(stream);
   	trace.ResetPriorityFilter();
    return stream;
}

//----------------------------------------------------------------------------------------

#endif // qTrace_AC

#endif // __CTraceStream_AC__