//----------------------------------------------------------------------------------------
// CAsyncStream_AC.cp
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CAsyncStream_AC__
#include "CAsyncStream_AC.h"
#endif

//----------------------------------------------------------------------------------------

static const long kDefaultSize = 256;

//========================================================================================
// CLASS CAsyncStream
//========================================================================================

//----------------------------------------------------------------------------------------
// CAsyncStream_AC::CAsyncStream_AC
//----------------------------------------------------------------------------------------

CAsyncStream_AC::CAsyncStream_AC()
	: 	fSize(kDefaultSize),
		fSavedWriter(NULL),
		fBuffer(NULL)
{
	fBuffer = TH_new char[fSize];

	fReader = fWriter = fBuffer;	
	fEndAddress = fBuffer + fSize;
}

//----------------------------------------------------------------------------------------
// CAsyncStream_AC::CAsyncStream_AC
//----------------------------------------------------------------------------------------

CAsyncStream_AC::CAsyncStream_AC(long inSize)
	:	fSize(inSize),
		fSavedWriter(NULL),
		fBuffer(NULL)
{
	fBuffer = TH_new char[fSize];

	fReader = fWriter = fBuffer;	
	fEndAddress = fBuffer + fSize;
}

//----------------------------------------------------------------------------------------
// CAsyncStream_AC::~CAsyncStream_AC
//----------------------------------------------------------------------------------------

CAsyncStream_AC::~CAsyncStream_AC()
{
	DeleteIfArray_AC(fBuffer);
}

//----------------------------------------------------------------------------------------
// CAsyncStream_AC::Read
//----------------------------------------------------------------------------------------

void CAsyncStream_AC::Read( void* outDataVoidPtr, long inAmount )
{	
	char*	outData = (char*) outDataVoidPtr;
	char*	workWriter = fWriter;
	char*	workReader = fReader;
	long	freeSpace = CalculateFreeSpace(workWriter);
	long	amountOfData = fSize - freeSpace;

	if (inAmount == kAllAvailableData)
	{
		inAmount = amountOfData;
	}
	else if (inAmount > amountOfData)
	{
		// don't want partial reads.
	//	throw CException_AC(CNetSink_AC::kUnderflow);
	}	
	
	if (workReader < workWriter)
	{
		// simple matter of reading up to workWriter
		::BlockMove_AC(workReader, outData, inAmount);
		fReader += inAmount;

		if(fReader == fEndAddress)
			fReader = fBuffer; // wrap read around to the front
	}
	else if (workReader > workWriter)
	{
		// read to end
		// read from top to workWriter
		
		long offsetAmount = (fEndAddress - workReader);		
		long amountRead = 0;
		long chunkSize = offsetAmount < inAmount ? offsetAmount : inAmount;

		::BlockMove_AC(workReader, outData, chunkSize );
		
		fReader += chunkSize;
		
		if (fReader == fEndAddress)
			fReader = fBuffer; // wrap read around to the front
			
		if (chunkSize < inAmount)
		{
			char *outPtr = outData+chunkSize;
			
			chunkSize = inAmount - chunkSize;
		
			::BlockMove_AC(fBuffer, outPtr, chunkSize);
			
			fReader += chunkSize;
		}
	}
}

//----------------------------------------------------------------------------------------
// CAsyncStream_AC::Write
//----------------------------------------------------------------------------------------

void CAsyncStream_AC::Write(const void* inStreamVoidPtr, long inSize)
{
	const char* inStream = (const char*) inStreamVoidPtr;
	
	// if stream was previously full, but we read some data out
	// of it, and it's no longer full, then reset fWriter to 
	// previous mark.
	if (!fWriter && (fSavedWriter != fReader))
	{
		fWriter = fSavedWriter;
		fSavedWriter = NULL;
	}
	
	char *workWriter = fWriter;

	long freeSpace = CalculateFreeSpace(workWriter);

	if (inSize > freeSpace)
	{
//		throw CException_AC(CNetSink_AC::kOverflow);
	}

	long count = inSize;
	char *inStreamWalker = (char*) inStream;
		
	while(count--)
	{
		if (workWriter == fEndAddress)
		{
			// wrap around to beginning of data.
			workWriter = fBuffer;
		}
#if qNetDebug_AC 
		if ((fWriter < fReader)
			&& ((workWriter < fWriter) || (workWriter > fReader)))
		{
			throw CException_AC();
		}
#endif
	
		*workWriter++ = *inStreamWalker++;
	}
	
	if (workWriter == fEndAddress)
	{
		// wrap around to beginning of data.
		workWriter = fBuffer;
	}
	
	if (workWriter != fReader)
	{
		fWriter = workWriter;
	}
	else
	{
		// we've filled it up
		fWriter = NULL;
		fSavedWriter = workWriter;
	}
}

//----------------------------------------------------------------------------------------
// CAsyncStream_AC::HasData
//----------------------------------------------------------------------------------------

bool CAsyncStream_AC::HasData() const
{
	return fReader != fWriter;
}

//----------------------------------------------------------------------------------------
// CAsyncStream_AC::CalculateFreeSpace
//----------------------------------------------------------------------------------------

long CAsyncStream_AC::CalculateFreeSpace( char *inWriter )
{
	long freeSpace = (inWriter == NULL) ? 0 : fSize;
	
	if (inWriter)
	{
		if (inWriter > fReader)
		{
			freeSpace = (fEndAddress - inWriter) + (fReader - fBuffer);
		}
		else if (inWriter < fReader)
		{
			freeSpace = fReader - inWriter;
		}
	}
	return freeSpace;
}


