//----------------------------------------------------------------------------------------
// CBufferStream_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CBufferStream_AC__
#include "CBufferStream_AC.h"
#endif

#ifndef __CBuffer_AC__
#include "CBuffer_AC.h"
#endif

//========================================================================================
// CLASS CBufferStreamRep_AC
//========================================================================================

class CBufferStreamRep_AC : public CStreamRep_AC
{
public:
	CBufferStreamRep_AC()
		: fSize(0),
		fWritePos(0),
		fReadPos(0)
		{
		}
		
	virtual ~CBufferStreamRep_AC()
		{
		}

	virtual long GetWritePosition() const
		{ return fWritePos; }

	virtual void SetWritePosition(long newPosition)
		{ fWritePos = newPosition; }
			
	virtual void SetReadPosition(long pos)
		{ fReadPos = pos; }
	
	virtual long GetReadPosition() const
		{ return fReadPos; }

	virtual void WriteBytes(const void* inBytes, long inAmt)
		{
			long spaceNeeded = GetWritePosition() + inAmt;
			if(spaceNeeded > fBuffer->GetSize())
			{
				fBuffer->SetSize(spaceNeeded);
			}

			BlockMove_AC( inBytes, fBuffer->GetBuffer() + GetWritePosition(), inAmt);
			
			fWritePos += inAmt;
		}

	virtual void ReadBytes(void* inBytes, long inAmt)
		{
			if( GetReadPosition() + inAmt > fBuffer->GetSize())
				ThrowError_AC(paramErr);

			BlockMove_AC( fBuffer->GetBuffer() + GetReadPosition(), inBytes, inAmt);

			fReadPos += inAmt;
		}

	void SetBuffer(CBuffer_AC* inBuffer)
		{ fBuffer = inBuffer; }
	
	CBuffer_AC* GetBuffer() const
		{ return fBuffer; }
	
	
	char* GetWritePositionPtr()
		{	return fBuffer->GetBuffer() + fWritePos; }
	
	char* GetReadPositionPtr()
		{	return fBuffer->GetBuffer() + fReadPos; }

	void ResetBufferSize()
		{ fBuffer->ResetBufferSize(false); }
	
public:	
	CRefCountingPtr_AC<CBuffer_AC> 	fBuffer;
	
	long 	fSize,
			fWritePos, 
		 	fReadPos;
};

//========================================================================================
// CLASS CBufferStream_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CBufferStream_AC::CBufferStream_AC
//----------------------------------------------------------------------------------------

CBufferStream_AC::CBufferStream_AC()
	: CStream_AC(TH_new CBufferStreamRep_AC),
	fStreamRep( dynamic_cast<CBufferStreamRep_AC*>(GetStreamRep()))
{
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::CBufferStream_AC
//----------------------------------------------------------------------------------------

CBufferStream_AC::CBufferStream_AC(CBuffer_AC* inBuffer)
	: CStream_AC(TH_new CBufferStreamRep_AC),
	fStreamRep( dynamic_cast<CBufferStreamRep_AC*>(GetStreamRep()))
{
	SetBuffer(inBuffer);
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::CBufferStream_AC
//----------------------------------------------------------------------------------------

CBufferStream_AC::CBufferStream_AC(long size, void* inBuffer)
	: CStream_AC(TH_new CBufferStreamRep_AC),
	fStreamRep( dynamic_cast<CBufferStreamRep_AC*>(GetStreamRep()))
{
	SetBuffer(size, inBuffer);
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::~CBufferStream_AC
//----------------------------------------------------------------------------------------

CBufferStream_AC::~CBufferStream_AC()
{
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::SetBuffer
//----------------------------------------------------------------------------------------

void CBufferStream_AC::SetBuffer(CBuffer_AC* inBuffer)
{
	fStreamRep->SetBuffer(inBuffer);
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::SetBuffer
//----------------------------------------------------------------------------------------

void CBufferStream_AC::SetBuffer(long inSize, void* inBuff)
{
	fStreamRep->SetBuffer(TH_new CBuffer_AC(inSize, inBuff));
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::SetBufferSize
//----------------------------------------------------------------------------------------

void CBufferStream_AC::SetBufferSize(long inSize)
{
	SetBuffer(inSize, NULL);
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::GetBufferSize
//----------------------------------------------------------------------------------------

long CBufferStream_AC::GetBufferSize() const
{
	return fStreamRep->GetBuffer() ? 
		fStreamRep->GetBuffer()->GetSize() : 0;
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::GetWritePositionPtr
//----------------------------------------------------------------------------------------

char* CBufferStream_AC::GetWritePositionPtr()
{
	return fStreamRep->GetWritePositionPtr();
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::GetReadPositionPtr
//----------------------------------------------------------------------------------------

char* CBufferStream_AC::GetReadPositionPtr()
{
	return fStreamRep->GetReadPositionPtr();
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::GetBuffer
//----------------------------------------------------------------------------------------

CBuffer_AC* CBufferStream_AC::GetBuffer()
{		
	return fStreamRep->GetBuffer(); 
}

//----------------------------------------------------------------------------------------
// CBufferStream_AC::ResetBufferSize
//----------------------------------------------------------------------------------------

void CBufferStream_AC::ResetBufferSize()
{
	fStreamRep->ResetBufferSize();
}