//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CBufferedStreamRep_AC__
#include "CBufferedStreamRep_AC.h"
#endif

//========================================================================================
// CLASS CAsyncStream
//========================================================================================

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::CBufferedStreamRep_AC
//----------------------------------------------------------------------------------------

CBufferedStreamRep_AC::CBufferedStreamRep_AC() 
{ 
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::CBufferedStreamRep_AC
//----------------------------------------------------------------------------------------

CBufferedStreamRep_AC::CBufferedStreamRep_AC(CBuffer_AC* inBuffer)
{
	SetBuffer(inBuffer);
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::CBufferedStreamRep_AC
//----------------------------------------------------------------------------------------

CBufferedStreamRep_AC::CBufferedStreamRep_AC(long inBufferSize, void* inBuffer)
{
	SetBuffer(inBufferSize, inBuffer);
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::~CBufferedStreamRep_AC
//----------------------------------------------------------------------------------------

CBufferedStreamRep_AC::~CBufferedStreamRep_AC() 
{
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::GetWritePosition
//----------------------------------------------------------------------------------------

long CBufferedStreamRep_AC::GetWritePosition() const 
{
	return fWritePos.GetPosition();
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::SetWritePosition
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::SetWritePosition(long newPosition)
{
	fWritePos.SetPosition(newPosition);
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::SetBuffer
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::SetBuffer(CBuffer_AC* inBuffer)
{	
	fBuffer = inBuffer;
	if (fBuffer)
	{
		fWritePos = fBuffer->GetIterator();
		fReadPos = fBuffer->GetIterator();
	}
	else
	{ 
//		fWritePos = NULL;
//		fReadPos = NULL;			
	}
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::SetBuffer
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::SetBuffer(long inSize, void* inBuff)
{
	SetBuffer( TH_new CBuffer_AC(inSize, inBuff));
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::SetBufferSize
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::SetBufferSize(long inSize)
{
	SetBuffer(inSize, NULL);
}
	
//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::GetBufferSize
//----------------------------------------------------------------------------------------

long CBufferedStreamRep_AC::GetBufferSize() const
{
	return fBuffer.IsNULL() ? 0 : fBuffer->GetSize();
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::GetReadPosition
//----------------------------------------------------------------------------------------

long CBufferedStreamRep_AC::GetReadPosition() const
{
	return fReadPos.GetPosition();
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::SetReadPosition
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::SetReadPosition(long newPosition)
{
	fReadPos.SetPosition(newPosition);

#if 0 
	// if tied
	if (fWritePos && fWritePos.GetPosition() <= fReadPos.GetPosition())
	{
		fReadPos.Rewind();
		fWritePos.Rewind();
	}	
#endif
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::WriteBytes
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::WriteBytes(const void* inBytes, long inAmt)
{
	if (!fBuffer)
	{
		WriteBufferedBytes(inBytes, inAmt);
		fWritePos += inAmt;	
	}
	else
	{	
		char*	walker = static_cast<char*>(const_cast<void*>(inBytes));				
		long amtToWrite = inAmt;
			
		while (amtToWrite)
		{
			long toWrite = ::Min_AC(amtToWrite, fWritePos.GetOffsetToEnd());
		
			::BlockMove_AC( walker, fWritePos.GetPositionPtr(), toWrite);
			
			fWritePos.Offset(toWrite);
			
			amtToWrite -= toWrite;
			walker += toWrite;
			
			if (amtToWrite > 0)
			{
				WriteBufferedBytes( fBuffer->GetBuffer(), fWritePos.GetPosition() );
				
				SetWritePosition(0);
			}
		}		
	
	}
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::ReadBytes
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::ReadBytes(void* inBytes, long inAmt)
{
	if (!fBuffer || GetReadAheadAmount() == 0)
	{
		ReadBufferedBytes(inBytes, inAmt);
		fReadPos += inAmt;	
	}
	else
	{
		char *walker = static_cast<char*>(inBytes);			
		long amtToRead = inAmt;
			
		while (amtToRead)
		{
			long toRead = ::Min_AC(amtToRead, GetBufferSize());
				// find out the max amount we can copy in this chunk
		
			::BlockMove_AC(fReadPos.GetPositionPtr(), walker, toRead);
				// copy data from buffer into client pointer
			
			amtToRead -= toRead;
				// note how much we've read
				
			walker += toRead;
				// move pointer into client data up

			fReadPos.Offset(toRead);
			
			if (amtToRead > 0)
			{
				if (fReadPos.IsAtEnd())
				{
					long toGet = Max_AC( GetBufferSize(), GetReadAheadAmount() );
				
					ReadBufferedBytes(fBuffer->GetBuffer(), toGet);
			
					fReadPos.Rewind();
				}
			}
		}		
	}
}

//----------------------------------------------------------------------------------------
// CBufferedStreamRep_AC::Flush
//----------------------------------------------------------------------------------------

void CBufferedStreamRep_AC::Flush()
{
	if(fBuffer) 
	{
		WriteBufferedBytes( fBuffer->GetBuffer(), fWritePos.GetPosition() );
		SetWritePosition(0);	
	}
}
