//----------------------------------------------------------------------------------------
// CCountingStream_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein. Used by permission
//----------------------------------------------------------------------------------------


#ifndef __CCountingStream_AC__
#include "CCountingStream_AC.h"
#endif

//========================================================================================
// CLASS CCountingStreamRep_AC
//========================================================================================

class CCountingStreamRep_AC : public CRandomAccessStreamRep_AC
{
public:
	CCountingStreamRep_AC();

	virtual ~CCountingStreamRep_AC();

	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------

	virtual long GetPosition() const;					// Override
	virtual void SetPosition(long newPosition);	// Override
	virtual long GetSize() const;						// Override
	virtual void SetSize(long newSize);			// Override

	virtual void WriteBytes(const void* p, long count); 	// Override

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------

protected:
	virtual void ReadBytes(void*, long);

public:

	long fPosition;								// current position in the stream 
	long fSize;									// current size of stream 
};

//========================================================================================
// CLASS CCountingStream_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CCountingStream_AC constructor
//----------------------------------------------------------------------------------------

CCountingStreamRep_AC::CCountingStreamRep_AC() :
	fPosition(0),
	fSize(0)
{
}

//----------------------------------------------------------------------------------------
// CCountingStreamRep_AC destructor
//----------------------------------------------------------------------------------------

CCountingStreamRep_AC::~CCountingStreamRep_AC()
{
}

//----------------------------------------------------------------------------------------
// CCountingStreamRep_AC::GetPosition: 
//----------------------------------------------------------------------------------------

long CCountingStreamRep_AC::GetPosition() const				// override 
{
	return fPosition;
}

//----------------------------------------------------------------------------------------
// CCountingStreamRep_AC::SetPosition: 
//----------------------------------------------------------------------------------------

void CCountingStreamRep_AC::SetPosition(long newPosition )// Override
{
	OSErr err;

	if (newPosition < 0)						// Same error returned by File Manager. 
		::ThrowIfOSErr_AC(posErr);
	else if (newPosition > fSize)				// Hit the end of the stream. 
	{
		newPosition = fSize;					// We still want to position to end of the stream. 
		err = eofErr;
	}
	else
		err = noErr;

	fPosition = newPosition;
	::ThrowIfOSErr_AC(err);
}

//----------------------------------------------------------------------------------------
// CCountingStreamRep_AC::GetSize: 
//----------------------------------------------------------------------------------------

long CCountingStreamRep_AC::GetSize() const					// override 
{
	return fSize;
}

//----------------------------------------------------------------------------------------
// CCountingStreamRep_AC::SetSize: 
//----------------------------------------------------------------------------------------

void CCountingStreamRep_AC::SetSize(long newSize)		// Override
{
	if (newSize < fPosition)					// If we shrunk the size, then we have to adjust the position. 
		fPosition = newSize;
	fSize = newSize;
}

//----------------------------------------------------------------------------------------
// CCountingStreamRep_AC::WriteBytes: 
//----------------------------------------------------------------------------------------

void CCountingStreamRep_AC::WriteBytes(const void*	/* p */ ,
								 long count)	// override 
{
	if (count < 0)								// Can't write negative number of bytes. 
		::ThrowIfOSErr_AC(paramErr);

	fPosition += count;							// Advance the position. 

	if (fPosition > fSize)						// Adjust the size if necessary. 
		fSize = fPosition;
}

//----------------------------------------------------------------------------------------
// CCountingStreamRep_AC::ReadBytes: 
//----------------------------------------------------------------------------------------

void CCountingStreamRep_AC::ReadBytes(void*, long)
{
}

//========================================================================================
// CLASS CCountingStream_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CCountingStream_AC constructor
//----------------------------------------------------------------------------------------

CCountingStream_AC::CCountingStream_AC()
	: CRandomAccessStream_AC(TH_new CCountingStreamRep_AC)
{
}

//----------------------------------------------------------------------------------------
// CCountingStream_AC destructor
//----------------------------------------------------------------------------------------

CCountingStream_AC::~CCountingStream_AC()
{
}

