//----------------------------------------------------------------------------------------
// CHandleStream_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein. Used by permission
//----------------------------------------------------------------------------------------

#ifndef __CHandleStream_AC__
#include "CHandleStream_AC.h"
#endif

//========================================================================================
// CLASS CHandleStreamRep_AC
//========================================================================================

class CHandleStreamRep_AC : public CRandomAccessStreamRep_AC
{
public:
	CHandleStreamRep_AC(Handle itsHandle,  long growth);
	
	virtual ~CHandleStreamRep_AC();

	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------
	virtual long GetPosition() const;

	virtual void SetPosition(long newPosition);

	virtual long GetSize() const;

	virtual void SetSize(long newSize);

	virtual void ReadBytes(void* p, long count); 

	virtual void WriteBytes(const void* p, long count); 

	Handle GetHandle() const
	{	return fHandle; }
	
	void SetHandle(Handle itsHandle)
	{	fHandle = itsHandle; }

	//------------------------------------------------------------------------------------
	// Growing 
	//------------------------------------------------------------------------------------
	virtual long GrowthSize(long needed);
		// Returns amount to grow by, given that we need 'needed' bytes.Default is to grow
        // by Max(fGrowth, needed). 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	Handle fHandle;								// the actual handle 
	long fPosition;								// offset into the handle 
	long fSize;									// size of the stream (may be smaller than handle) 
	long fGrowthSize;							// how much to grow the handle when needed
	char fSavedState;							// handle's saved state
};



//========================================================================================
// CLASS CHandleStreamRep_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::CHandleStreamRep_AC: 
//----------------------------------------------------------------------------------------

CHandleStreamRep_AC::CHandleStreamRep_AC(Handle itsHandle,
								  long growth) :
	fHandle(itsHandle),
	fPosition(0),
	fSize(0),
	fGrowthSize(growth),
	fSavedState(0)
{
	if (itsHandle)
	{
		if (IsAResource_AC(itsHandle))
		{
			::LoadResource(itsHandle);		// ensure it's loaded in memory
			::ThrowIfResError_AC();

		}
		fSize = ::GetHandleSize(itsHandle);
		fSavedState = ::HGetState(itsHandle);
		::HNoPurge(itsHandle);					// ensure it's not purgeable
		fHandle = itsHandle;
	}
}

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::Free: 
//----------------------------------------------------------------------------------------

CHandleStreamRep_AC::~CHandleStreamRep_AC()
{
	if (fHandle)
	{
		SetSize(GetPosition());					// Trim the handle to its current position. 
		::HSetState(fHandle, fSavedState);
	}
}

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::GetPosition: 
//----------------------------------------------------------------------------------------

long CHandleStreamRep_AC::GetPosition() const				// override 
{
	return fPosition;
}

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::GetSize: 
//----------------------------------------------------------------------------------------

long CHandleStreamRep_AC::GetSize() const					// override 
{
	return fSize;
}

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::GrowthSize: 
//----------------------------------------------------------------------------------------

long CHandleStreamRep_AC::GrowthSize(long needed)
{
	return Max_AC(fGrowthSize, needed);
}


//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::SetPosition: 
//----------------------------------------------------------------------------------------

void CHandleStreamRep_AC::SetPosition(long newPosition )// override 
{
	OSErr err;

	if (newPosition < 0)						// Same error returned by File Manager. 
		::ThrowIfOSErr_AC(posErr);
	else if (newPosition > fSize)				// Hit the end of the stream. 
	{
		newPosition = fSize;					// We still want to position to end of the stream.
		err = eofErr;
	}
	else
		err = noErr;

	fPosition = newPosition;
	::ThrowIfOSErr_AC(err);
}

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::SetSize: 
//----------------------------------------------------------------------------------------

void CHandleStreamRep_AC::SetSize(long newSize)		// override 
{
	SetPermHandleSize_AC(fHandle, newSize);

	if (newSize < fPosition)					// If we shrunk the size, then adjust position and size
		fPosition = newSize;

	fSize = newSize;
}

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::ReadBytes: 
//----------------------------------------------------------------------------------------

void CHandleStreamRep_AC::ReadBytes(void* p,
							  long count)		// override 
{
	OSErr err = noErr;

	register long available = fSize - fPosition;// Bytes available in the stream. 

	if (available < count)						// About to read past end of stream. 
	{
		count = available;
		err = eofErr;
	}

	if (count > 0)								// Move the bytes. 
	{
		BlockMove_AC(*fHandle + fPosition, p, count);
		fPosition += count;
	}
	else if (count < 0)
		err = paramErr;							// Negative request count. 

	::ThrowIfOSErr_AC(err);
}

//----------------------------------------------------------------------------------------
// CHandleStreamRep_AC::WriteBytes: 
//----------------------------------------------------------------------------------------

void CHandleStreamRep_AC::WriteBytes(const void* p,
							   long count)		// override 
{
	if (count < 0)
		::ThrowIfOSErr_AC(paramErr);					// Negtive request count. 

	long available = fSize - fPosition;			// Bytes available in the stream. 

	if (available < count)						// Grow the handle to accommodate request. 
	{
		// code as 2 statements to avoid interference of memory move in call to GrowthSize
		long itsGrowthSize = GrowthSize(count - available);
		SetSize(fSize + itsGrowthSize);
	}

	BlockMove_AC(p, *fHandle + fPosition, count);// Move the bytes. 

	fPosition += count;							// Advance the position. 

	if (fPosition > fSize)						// Adjust the size if necessary. 
		fSize = fPosition;
}

//========================================================================================
// CLASS CHandletream_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CHandleStream_AC::CHandleStream_AC
//----------------------------------------------------------------------------------------

CHandleStream_AC::CHandleStream_AC(Handle itsHandle,  long growth)
	: CRandomAccessStream_AC(TH_new CHandleStreamRep_AC(itsHandle, growth))
{
}

//----------------------------------------------------------------------------------------
// CHandleStream_AC::~CHandleStream_AC
//----------------------------------------------------------------------------------------

CHandleStream_AC::~CHandleStream_AC()
{
}

//----------------------------------------------------------------------------------------
// CHandleStream_AC::SetHandle
//----------------------------------------------------------------------------------------

void CHandleStream_AC::SetHandle(Handle itsHandle)
{
	dynamic_cast_AC(CHandleStreamRep_AC*,GetStreamRep())->SetHandle(itsHandle);
}

//----------------------------------------------------------------------------------------
// CHandleStream_AC::GetHandle
//----------------------------------------------------------------------------------------

Handle CHandleStream_AC::GetHandle()
{
	return dynamic_cast_AC(CHandleStreamRep_AC*,GetStreamRep())->GetHandle();
}

//----------------------------------------------------------------------------------------
// CHandleStream_AC::GrowthSize
//----------------------------------------------------------------------------------------

long CHandleStream_AC::GrowthSize(long needed)
{
	return dynamic_cast_AC(CHandleStreamRep_AC*,GetStreamRep())->GrowthSize(needed);
}
