//----------------------------------------------------------------------------------------
// CTraceStream_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

#if qTrace_AC

#ifndef __CBufferedStreamRep_AC__
#include "CBufferedStreamRep_AC.h"
#endif

#ifndef __CLinkedList_AC__
#include "CLinkedList_AC.h"
#endif

//----------------------------------------------------------------------------------------
// local definitions
//----------------------------------------------------------------------------------------

#define kDefaultSize 256

//----------------------------------------------------------------------------------------
// static data
//----------------------------------------------------------------------------------------

CAutoPtr_AC<CTraceStream_AC> CTraceStream_AC::fgSelf;

//----------------------------------------------------------------------------------------
// CTraceStream_AC::CTraceStream_AC
//----------------------------------------------------------------------------------------
CTraceStream_AC::CTraceStream_AC() 
	: CCharOutputSink_AC(TH_new CBufferedStreamRep_AC(kDefaultSize)),
	fPriorityFilter(kLowPriority), 
	fPriority(kDefaultPriority),
	fPrevPriorityFilter(0),
	fPrevPriority(0)
{
}

//----------------------------------------------------------------------------------------
// CTraceStream_AC::~CTraceStream_AC
//----------------------------------------------------------------------------------------

CTraceStream_AC::~CTraceStream_AC() 
{
}

//----------------------------------------------------------------------------------------
// CTraceStream_AC::GetStreamRef
//----------------------------------------------------------------------------------------

CTraceStream_AC& CTraceStream_AC::GetStreamRef()
{
	if(!fgSelf) fgSelf = TH_new CTraceStream_AC;
	return *fgSelf;
}

//----------------------------------------------------------------------------------------
// CTraceStream_AC::AddStream
//----------------------------------------------------------------------------------------

void CTraceStream_AC::AddStream(CStream_AC& in)
{
	GetStreamRef().PipeOutputTo(&in);
}

//----------------------------------------------------------------------------------------
// CTraceStream_AC::RemoveStream
//----------------------------------------------------------------------------------------

void CTraceStream_AC::RemoveStream(CStream_AC& stream)
{
	GetStreamRef().BreakPipeWith(&stream);
}

//----------------------------------------------------------------------------------------
// CTraceStream_AC::WriteCharBytes
//----------------------------------------------------------------------------------------

void CTraceStream_AC::WriteCharBytes(const char* inBytes, long inAmt)
{
	if((fPriority >= fPriorityFilter) && (fPriorityFilter > 0))
	{
		CCharOutputSink_AC::WriteCharBytes(inBytes, inAmt);
		
		if(!fOutputPipes.IsNULL())
		{
			CCharOutputSink_AC* pipe = NULL;	
			CStreamList::CIterator iter = fOutputPipes->First();
		
			while(iter.IsValid())
			{
				pipe = dynamic_cast<CCharOutputSink_AC*>(iter.Current());
				if(pipe)
					pipe->WriteCharBytes(inBytes, inAmt);
				else
					(*iter)->WriteBytes(inBytes, inAmt);
				
				iter.Next();
			}
		}
	}		
}

#endif  // qTrace_AC