//----------------------------------------------------------------------------------------
// CNetParser_AC.h
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTextParser_AC__
#define __CTextParser_AC__

#ifndef __Text_AC__
#include "Text_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#define kStandardDelimeters " \t\r\n"

//----------------------------------------------------------------------------------------
// class CTextParser_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CTextParser_AC
{
public:
	
	class CToken
	{
	public:
		CToken(const char* inToken) : fToken(inToken) {}
		CToken(const CToken& inToken) : fToken(inToken.fToken) {}
		~CToken() {}
	
		CToken& operator = (const CToken& inToken) 
			{ fToken = inToken.fToken; return *this; }
			
		operator const char* () const { return fToken; }
		
		bool operator == (const char*) const;
		bool operator != (const char*) const;
		
	private:
		const char* fToken;
	};

	CTextParser_AC(long bufferSize = 0xFFF); // 4k
		
	virtual ~CTextParser_AC();

	enum eDelimeters
	{
		kSpace = ' ',
		kLineFeed = 0xA,
		kReturn = 0xD,
		kTab = '\t'
	};
	
	inline bool IsDelimeter(char inChar, char* inDelimeters)
	{
		while( *inDelimeters != '\0' )
		{
			if(*inDelimeters++ == inChar) return true;
		}
		
		return false;
	}

	CToken Start(const char* unParsedText);

	CToken Start(const char* unParsedText, long size);

	CToken GetNextToken();
	
	CToken GetCurToken() { return fCurToken; }
	
	void AddDelimeter(char);
		
	void SetDelimeters(const char* inPtr);
		// expects null terminated string
	
protected:
	CToken GetPrevToken();
		// not implemented yet.
	
	class EXPORT_AC TBuffer
	{
	public:
		TBuffer(bool doDelete ) 
			: fWalker(0), fBuffer(0), fSize(0), fTop(0), fDelete(doDelete) {}

		~TBuffer() { if(fDelete && fBuffer) delete[] const_cast<char *>(fBuffer); } 
			
		void Clean() { fWalker = 0; fBuffer = 0; fTop = 0; fSize = 0; }

		void Set(const char* inBuffer, long inSize) 
			{ fBuffer = (char*) inBuffer; fSize = inSize; fTop = fBuffer + fSize; }

		bool AtEnd() { return fWalker >= fTop; }
		
		void ResetWalker() { fWalker = (char*) fBuffer; }
		
		char* fWalker;	
	
	private:
		const char* fBuffer;
		const char* fTop;
		long  fSize;
		bool fDelete;		
	};

private:
	char				fDelims[25];
	char*				fCurToken;

	TBuffer	fTokenBuffer;
	TBuffer fRawBuffer;
};

#endif