//----------------------------------------------------------------------------------------
// UNetParser.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __CTextParser_AC__
#include "CTextParser_AC.h"
#endif

#ifndef __string__
#include <string.h>
#endif

//----------------------------------------------------------------------------------------
// CTextParser_AC::CToken::operator ==
//----------------------------------------------------------------------------------------

bool CTextParser_AC::CToken::operator == (const char* inStr) const
{
	return ::strcmp(fToken, inStr) == 0;
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::CToken::operator !=
//----------------------------------------------------------------------------------------

bool CTextParser_AC::CToken::operator != (const char* inStr) const
{
	return ::strcmp(fToken, inStr) != 0;
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::CTextParser_AC
//----------------------------------------------------------------------------------------

CTextParser_AC::CTextParser_AC(long bufferSize)
:	fTokenBuffer(true),
	fRawBuffer(false),
	fCurToken(0)
{
	fDelims[0] = '\0';
	fTokenBuffer.Set(TH_new char[bufferSize], bufferSize);
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::~CTextParser_AC
//----------------------------------------------------------------------------------------

CTextParser_AC::~CTextParser_AC()
{
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::Start
//----------------------------------------------------------------------------------------

CTextParser_AC::CToken CTextParser_AC::Start(const char* unParsedText)
{ 
	return Start( unParsedText, ::strlen(unParsedText) ); 
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::Start
//----------------------------------------------------------------------------------------

CTextParser_AC::CToken CTextParser_AC::Start(const char* unParsedText, long size)
{
	fRawBuffer.Set(unParsedText, size);
	
	fRawBuffer.ResetWalker();
	
	fTokenBuffer.ResetWalker();
	
	return GetNextToken();
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::GetNextToken
//----------------------------------------------------------------------------------------

CTextParser_AC::CToken CTextParser_AC::GetNextToken()
{

	// eat up the delimeters
	while( !fRawBuffer.AtEnd() && 
			IsDelimeter(*(fRawBuffer.fWalker), fDelims))
	{
		++fRawBuffer.fWalker;
	}

	fCurToken = fRawBuffer.AtEnd() ? 0 : fTokenBuffer.fWalker ;
		// get next token pointer. This points into the tokenbuffer
	
	if( fCurToken )
	{	
		// Now copy the token out of the rawbuffer into the tokenbuffer
		// while being careful not to overrun the buffer.	
		while( 	!fRawBuffer.AtEnd() && 
				!IsDelimeter(*(fRawBuffer.fWalker), fDelims))
		{
			if(!fTokenBuffer.AtEnd())
				*(fTokenBuffer.fWalker)++ = *(fRawBuffer.fWalker)++;
			else
			{
				fCurToken = 0; 
				// can't send out a string without a '\0';
				DebugStr("\pBuffer overflow");
				break;
			}
		}
	}
		
	if( fCurToken )
	{
		// tack a '\0' onto the token, which is really the point of all this.
		if(!fTokenBuffer.AtEnd())
			*(fTokenBuffer.fWalker)++ = '\0';
		else
		{
			fCurToken = 0; 
				// can't send out a string without a '\0';
			DebugStr("\pBuffer overflow");
		}
	}
	
	// eat up the delimeters
	while( !fRawBuffer.AtEnd() && 
			IsDelimeter(*(fRawBuffer.fWalker), fDelims))
	{
		++fRawBuffer.fWalker;
	}

	
	// return the token
	return CToken(fCurToken);
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::GetPrevToken
//----------------------------------------------------------------------------------------

CTextParser_AC::CToken CTextParser_AC::GetPrevToken()
{
	return CToken(0);
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::AddDelimeter
//----------------------------------------------------------------------------------------

void CTextParser_AC::AddDelimeter(char inDelim)
{
	char* walker = fDelims;
	while(1)
	{
		if(*walker == '\0')
		{
			*walker++ = inDelim;
			*walker = '\0';
			break;
		}
		++walker;	
	}
}

//----------------------------------------------------------------------------------------
// CTextParser_AC::SetDelimeters
//----------------------------------------------------------------------------------------

void CTextParser_AC::SetDelimeters(const char* inStr)
{
	::strcpy( fDelims, inStr );
}