//----------------------------------------------------------------------------------------
// CClipboard_AC.h
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CClipboard_AC__
#define __CClipboard_AC__

// ACS Toolbox

#ifndef __Toolbox_AC__
#include "Toolbox_AC.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#if qALOE && !defined(__ALOE__)
#include "ALOE.h"
#endif

//----------------------------------------------------------------------------------------
// CClipboard is a lightweight encapsulation of the Scrap manager routines
// based on core functionality of MacApp's UClipboardMgr, and modified for ALOE support.
// It is designed to be conditionally compiled rather than using virtual functions
// so that the Scrap manager calls may be made static.
//----------------------------------------------------------------------------------------


class EXPORT_AC CClipboard_AC {

public:
	CClipboard_AC();
	// Construct and cache the state of the scrap
	
	virtual ~CClipboard_AC();
	// Free data associated with this class
		
	void AbsorbScrapStuff();
	// Cache the state of the scrap
			
	short GetScrapCount() const;
	// This returns the current cached scrap count
	
	short GetOldScrapCount() const;
	// This returns the previous cached scrap count
	
	short GetScrapCountMinusOne() const;
	// This returns the InfoScrap scrap count minus one
	
	bool ScrapCountStillValid(short oldScrapCount) const;
	// Returns difference between current scrap count and previous cached scrap count

	bool IsScrapInMemory() const;
	// Return whether the scrap data is in memory
	
	bool IsScrapEmpty() const;
	// Return whether the scrap is empty
	
	static long StandardZeroScrap() { return ::ZeroScrap(); }
	// Clients must call this when using TE or other indirect scrap calls
	// especially when qALOE is turned on; otherwise call the ZeroScrap method.

	// These functions encapsulate the Scrap manager or ALOE scrap.

#if !qALOE // Use the scrap manager calls directly

	static ScrapStuffPtr InfoScrap()
		{ return ::InfoScrap(); }
	
	static long ZeroScrap()
		{ return ::ZeroScrap(); }
	
	static long PutScrap(long length, ResType theType, Ptr source)
		{ return ::PutScrap(length, theType, source); }
	
	static long GetScrap(Handle hDest, ResType theType, long* offset)
		{ return ::GetScrap(hDest, theType, offset); }
	
	static long UnloadScrap()
		{ return ::UnloadScrap(); }
	
	static long LoadScrap()
		{ return ::LoadScrap(); }

#else // qALOE, use the ALOE (OpenDoc) scrap calls

	static ScrapStuffPtr InfoScrap()
		{ return ::ALOEInfoScrap(); }
	
	static long ZeroScrap()
		{ return ::ALOEZeroScrap(); }
	
	static long PutScrap(long length, ResType theType, Ptr source)
		{ return ::ALOEPutScrap(length, theType, source); }
	
	static long GetScrap(Handle hDest, ResType theType, long* offset)
		{ return ::ALOEGetScrap(hDest, theType, offset); }
	
	static long UnloadScrap()
		{ return ::ALOEUnloadScrap(); }
	
	static long LoadScrap()
		{ return ::ALOELoadScrap(); }

#endif // qALOE

private:
	ScrapStuff fOldScrapStuff;					// This is the last ScrapStuff record used in
                                                // tracking the scrap.

	ScrapStuff fNewScrapStuff;					// This is the current ScrapStuff record used in
                                                // tracking the scrap.
};



#endif