//----------------------------------------------------------------------------------------
// CProcess_AC.cp
// Copyright  1997 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CProcess_AC__
#include "CProcess_AC.h"
#endif

// Core

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

// Toolbox

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

#ifndef __PROCESSES__
#include <Processes.h>
#endif

//----------------------------------------------------------------------------------------
// Static initialization
//----------------------------------------------------------------------------------------

CProcess_AC* CProcess_AC::fgProcess = NULL;

//----------------------------------------------------------------------------------------
// CProcess_AC::DoInteractWithUser - static
//----------------------------------------------------------------------------------------
OSErr CProcess_AC::DoInteractWithUser()
{
	return fgProcess
		? fgProcess->InteractWithUser()
		: errAENoUserInteraction;
}

//----------------------------------------------------------------------------------------
// CProcess_AC constructor
//----------------------------------------------------------------------------------------
CProcess_AC::CProcess_AC()
{
	fProcessNumber.highLongOfPSN = kNoProcess;
	fProcessNumber.lowLongOfPSN = kNoProcess;

#if !qMSWindows
	::ThrowIfOSErr_AC(::GetCurrentProcess(&fProcessNumber));
#else
	::ThrowIfOSErr_AC(QTMLGetCurrentProcess(&fProcessNumber));
#endif

	fgProcess = this;
}

//----------------------------------------------------------------------------------------
// CProcess_AC destructor
//----------------------------------------------------------------------------------------
CProcess_AC::~CProcess_AC()
{
}

//----------------------------------------------------------------------------------------
// CProcess_AC::InteractWithUser
//----------------------------------------------------------------------------------------
OSErr CProcess_AC::InteractWithUser()
{
	return AEInteractWithUser(kAEDefaultTimeout, NULL, NULL);
}

OSErr CProcess_AC::InteractWithUser(long timeOutInTicks,
									NMRecPtr nmReqPtr,
									AEIdleUPP idleProc)
{
	return AEInteractWithUser(timeOutInTicks, nmReqPtr, idleProc);
}

//----------------------------------------------------------------------------------------
// CProcess_AC::IsFrontProcess: 
//----------------------------------------------------------------------------------------
bool CProcess_AC::IsFrontProcess()
{
	ProcessSerialNumber aPSN;
	ProcessSerialNumber applicationPSN;
	Boolean result;

	::ThrowIfOSErr_AC(::GetFrontProcess(&aPSN));
	applicationPSN = fProcessNumber;
	::ThrowIfOSErr_AC(::SameProcess(&aPSN, &applicationPSN, &result));

	return result;
}

//----------------------------------------------------------------------------------------
// CProcess_AC::MakeFrontProcess: 
//----------------------------------------------------------------------------------------
void CProcess_AC::MakeFrontProcess()
{
	if (!IsFrontProcess())
	{
		ProcessSerialNumber aPSN = fProcessNumber;
		::ThrowIfOSErr_AC(::SetFrontProcess(&aPSN));
	}
}

//----------------------------------------------------------------------------------------
// CProcess_AC::WakeProcess: 
//----------------------------------------------------------------------------------------
void CProcess_AC::WakeProcess()
{
	ProcessSerialNumber aPSN = fProcessNumber;
	::ThrowIfOSErr_AC(::WakeUpProcess(&aPSN));
}

//----------------------------------------------------------------------------------------
// End of CProcess_AC.cp



