//----------------------------------------------------------------------------------------
// Configuration_AC.cp
// Copyright  1984-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Configuration_AC__
#include "Configuration_AC.h"
#endif

// Toolbox

#ifndef __CODEFRAGMENTS__
#include <CodeFragments.h>
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __APPLEGUIDE__
#include <AppleGuide.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#if qWantsThreadManager
#ifndef __THREADS__
#include <Threads.h>
#endif
#endif

//----------------------------------------------------------------------------------------
// GestaltAttribute_AC: 
//----------------------------------------------------------------------------------------


bool GestaltAttribute_AC(OSType itsAttr, short itsBit)
{
	long response;

	return Gestalt(itsAttr, &response) == noErr && (response >> itsBit) & 1;
	
}

//----------------------------------------------------------------------------------------
// HasGestalt_AC: 
//----------------------------------------------------------------------------------------


bool HasGestalt_AC(OSType itsGestalt)
{
	long response;

	return Gestalt(itsGestalt, &response) == noErr;

}

//----------------------------------------------------------------------------------------
// System configuration.
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// GetEnvironsVersion: 
//----------------------------------------------------------------------------------------


short GetEnvironsVersion_AC()
{
	static bool defined;
	static short environsVersion;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltVersion, &response);
		environsVersion = static_cast<short>(response);
	}
	return environsVersion;
}

//----------------------------------------------------------------------------------------
// GetMachineType:
//----------------------------------------------------------------------------------------


short GetMachineType_AC()
{
	static bool defined;
	static short machineType;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltMachineType, &response);
		machineType = static_cast<short>(response);
	}
	return machineType;
}

//----------------------------------------------------------------------------------------
// GetSystemVersion:
//----------------------------------------------------------------------------------------


short GetSystemVersion_AC()
{
	static bool defined;
	static short systemVersion;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltSystemVersion, &response);
		systemVersion = static_cast<short>(response);
	}
	return systemVersion;
}

//----------------------------------------------------------------------------------------
// HasColorQD:
//----------------------------------------------------------------------------------------


bool HasColorQD_AC()
{
	static bool defined;
	static bool hasColorQD;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltQuickdrawVersion, &response);
		hasColorQD = response != gestaltOriginalQD;
	}
	return hasColorQD;
}

//----------------------------------------------------------------------------------------
// Has32BitQD:
//----------------------------------------------------------------------------------------


bool Has32BitQD_AC()
{
	static bool defined;
	static bool has32BitQD;
	if (!defined)
	{
		defined = true;
		has32BitQD = HasColorQD_AC();
		if (has32BitQD)
		{
			long response;
			OSErr err = ::Gestalt(gestaltQuickdrawVersion, &response);
			has32BitQD = has32BitQD && (response != gestalt8BitQD);
		}
	}
	return has32BitQD;
}

//----------------------------------------------------------------------------------------
// KeyboardType:
//----------------------------------------------------------------------------------------


short KeyboardType_AC()
{
	static bool defined;
	static short keyboardType;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltKeyboardType, &response);
		keyboardType = static_cast<short>(response);
	}
	return keyboardType;
}

//----------------------------------------------------------------------------------------
// GetATDrvrVersNum:
//----------------------------------------------------------------------------------------


short GetATDrvrVersNum_AC()
{
	static bool defined;
	static short atDrvrVersNum;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltAppleTalkVersion, &response);
		atDrvrVersNum = static_cast<short>(response);
	}
	return atDrvrVersNum;
}

//----------------------------------------------------------------------------------------
// HasAUX:
//----------------------------------------------------------------------------------------


bool HasAUX_AC()
{
	static bool defined;
	static bool hasAUX;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltAUXVersion, &response);
		hasAUX = response != 0;
	}
	return hasAUX;
}

//----------------------------------------------------------------------------------------
// GetTEVersion:
//----------------------------------------------------------------------------------------


long GetTEVersion_AC()
{
	static bool defined;
	static long teVersion;
	if (!defined)
	{
		defined = true;
		long response;
		OSErr err = ::Gestalt(gestaltTextEditVersion, &response);
		teVersion = response;
	}
	return teVersion;
}

//----------------------------------------------------------------------------------------
// HasSCSI:
//----------------------------------------------------------------------------------------


bool HasSCSI_AC()
{
	static bool defined;
	static bool hasSCSI;
	if (!defined)
	{
		defined = true;
		hasSCSI = GestaltAttribute_AC(gestaltHardwareAttr, gestaltHasSCSI);
	}
	return hasSCSI;
}

//----------------------------------------------------------------------------------------
// HasTempMem:
//----------------------------------------------------------------------------------------


bool HasTempMem_AC()
{
	static bool defined;
	static bool hasTempMem;
	if (!defined)
	{
		defined = true;
		hasTempMem = GestaltAttribute_AC(gestaltOSAttr, gestaltTempMemSupport);
	}
	return hasTempMem;
}

//----------------------------------------------------------------------------------------
// HasDesktopBus:
//----------------------------------------------------------------------------------------


bool HasDesktopBus_AC()
{

#if qMSWindows
	return false;
#else

	static bool defined;
	static bool hasDesktopBus;
	if (!defined)
	{
		defined = true;
		const short mDesktopBus = 0x0400;
		hasDesktopBus = (LMGetHWCfgFlags() & mDesktopBus) > 0;
	}
	return hasDesktopBus;

#endif 
}

//----------------------------------------------------------------------------------------
// HasTrueType:
//----------------------------------------------------------------------------------------


bool HasTrueType_AC()
{
	static bool defined;
	static bool hasTrueType;
	if (!defined)
	{
		defined = true;
		hasTrueType = GestaltAttribute_AC(gestaltFontMgrAttr, gestaltOutlineFonts);
	}
	return hasTrueType;
}

/* Provided by ThreadUtilities_AC:
//----------------------------------------------------------------------------------------
// HasThreadManager:
//----------------------------------------------------------------------------------------


bool HasThreadManager_AC()
{
	static bool defined;
	static bool hasThreadManager;
	if (!defined)
	{
		defined = true;

		hasThreadManager = GestaltAttribute_AC('thds', 0);
#if qPowerPC || qModelCFM
		// verify presence of shared library
		hasThreadManager = hasThreadManager && ((Ptr)NewThread != (Ptr)kUnresolvedCFragSymbolAddress);
#endif
	}
	return hasThreadManager;
}
*/

//----------------------------------------------------------------------------------------
// HasQDGXGraphics:
//----------------------------------------------------------------------------------------


bool HasQDGXGraphics_AC()
{
	static bool defined;
	static bool hasQDGXGraphics;
	if (!defined)
	{
		defined = true;
		hasQDGXGraphics = HasGestalt_AC('grfx');
	}
	return hasQDGXGraphics;
}

//----------------------------------------------------------------------------------------
// HasQDGXPrinting:
//----------------------------------------------------------------------------------------


bool HasQDGXPrinting_AC()
{
	static bool defined;
	static bool hasQDGXPrinting;
	if (!defined)
	{
		defined = true;
		hasQDGXPrinting = HasGestalt_AC('pmgr');
	}
	return hasQDGXPrinting;
}

#if 0
	// Removed due to ASLM removal vs. deinstall problems.
	//----------------------------------------------------------------------------------------
	// HasASLM:
	//----------------------------------------------------------------------------------------
	
	
	bool HasASLM_AC()
	{
		static bool defined;
		static bool hasASLM;
		if (!defined)
		{
			defined = true;
			hasASLM = GestaltAttribute_AC('aslm', 0);
		}
		return hasASLM;
	}
#endif

//----------------------------------------------------------------------------------------
// HasCFM:
//----------------------------------------------------------------------------------------


bool HasCFM_AC()
{
	static bool defined;
	static bool hasCFM;
	if (!defined)
	{
		defined = true;
		hasCFM = GestaltAttribute_AC(gestaltCFMAttr, gestaltCFMPresent);
	}
	return hasCFM;
}

//----------------------------------------------------------------------------------------
// HasTranslationManager:
//----------------------------------------------------------------------------------------


bool HasTranslationManager_AC()
{
	static bool defined;
	static bool hasTranslationManager;
	if (!defined)
	{
		defined = true;
		hasTranslationManager = GestaltAttribute_AC(gestaltTranslationAttr, gestaltTranslationMgrExists);
	}
	return hasTranslationManager;
}

//----------------------------------------------------------------------------------------
// HasSpeechManager:
//----------------------------------------------------------------------------------------


bool HasSpeechManager_AC()
{
	static bool defined;
	static bool hasSpeechManager;
	if (!defined)
	{
		defined = true;
		hasSpeechManager = GestaltAttribute_AC(gestaltSpeechAttr, gestaltSpeechMgrPresent);
	}
	return hasSpeechManager;
}

//----------------------------------------------------------------------------------------
// HasCustomFile:
//----------------------------------------------------------------------------------------


bool HasCustomFile_AC()
{
	static bool defined;
	static bool hasCustomFile;
	if (!defined)
	{
		defined = true;
		hasCustomFile = GestaltAttribute_AC(gestaltStandardFileAttr, gestaltStandardFile58);
	}
	return hasCustomFile;
}

//----------------------------------------------------------------------------------------
// HasQuickTime:
//----------------------------------------------------------------------------------------


bool HasQuickTime_AC()
{
	static bool defined;
	static bool hasQuickTime;
	if (!defined)
	{
		defined = true;
		hasQuickTime = HasGestalt_AC(gestaltQuickTime);
	}
	return hasQuickTime;
}

//----------------------------------------------------------------------------------------
// HasTSM:
//----------------------------------------------------------------------------------------


bool HasTSM_AC()
{
	static bool defined;
	static bool hasTSM;
	if (!defined)
	{
		defined = true;
		hasTSM = HasGestalt_AC(gestaltTSMgrVersion);
	}
	return hasTSM;
}

//----------------------------------------------------------------------------------------
// HasAppleScript:
//----------------------------------------------------------------------------------------


bool HasAppleScript_AC()
{
	static bool defined;
	static bool hasAppleScript;
	if (!defined)
	{
		defined = true;
		hasAppleScript = GestaltAttribute_AC('ascr', 0);
	}
	return hasAppleScript;
}
//----------------------------------------------------------------------------------------
// HasQuickDraw3D:
//----------------------------------------------------------------------------------------

bool HasQuickDraw3D_AC() 
{

	static bool defined;
	static bool hasQD3D;
	if (!defined)
	{
		defined = true;
		hasQD3D = HasGestalt_AC('qd3d');
	}
	return hasQD3D;
}

//----------------------------------------------------------------------------------------
// HasQuickDraw3DViewer:
//----------------------------------------------------------------------------------------
bool HasQuickDraw3DViewer_AC() 
{
	static bool defined;
	static bool hasQD3DViewer;
	if (!defined)
	{
		defined = true;
		hasQD3DViewer = HasGestalt_AC('q3vc');
	}
	return hasQD3DViewer;
}

//----------------------------------------------------------------------------------------
// HasAppleGuide:
//----------------------------------------------------------------------------------------
bool HasAppleGuide_AC() 
{
	static bool defined;
	static bool hasAppleGuide;
	if (!defined)
	{
		defined = true;
		hasAppleGuide = GestaltAttribute_AC(gestaltHelpMgrAttr, gestaltAppleGuidePresent);
	}
	return hasAppleGuide;

}
bool HasDragManager_AC()
{
	static bool defined;
	static bool hasDragManager;
	if (!defined)
	{
		defined = true;

		hasDragManager = GestaltAttribute_AC('drag', 0);
#if qPowerPC || qModelCFM
		// verify presence of shared library
		hasDragManager = hasDragManager && ((void*)NewDrag != (void*)kUnresolvedCFragSymbolAddress);
#endif
	}
	return hasDragManager;
}

//----------------------------------------------------------------------------------------
// End of Configuration_AC.cp


