//----------------------------------------------------------------------------------------
// UWindow.cp 
// Copyright  1987-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	Change History:
		07/25/97	When updating with platinum appearance we were not respecting the window's content
					region color. We needed to set the background color for the window when focusing.
					However, to allow drawing environments to provide alternate coloration we also needed
					to call TView::SetupDrawingEnvironment() when focusing. TWhiteBackgroundAdorner installs
					a DrawingEnvironment to cause the color environment to change to the desired one.
*/

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// ACS

#if qDrag
#ifndef __CList_AC__
#include "CList_AC.h"
#endif
#endif

// MacApp

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UDIALOGBEHAVIOR__
#include "UDialogBehavior.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#if qDrag
#ifndef __UDRAGDROP__
#include "UDragDrop.h"
#endif

#endif

#if qDrag
#ifndef __UDRAGDROPBEHAVIOR__
#include "UDragDropBehavior.h"
#endif

#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __FLOATWINDOW__
#include "FloatWindow.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __FIXMATH__
#include <FixMath.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Use Windows_AC.h instead of <windows.h> for XPlatform compatability.
#ifndef __Windows_AC__
#include "Windows_AC.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
// Get rid of the CloseWindow macro since it will interfere with CloseWindow method calls
//#	undef CloseWindow
#endif

// ACS ALOE

#if qALOE
#	ifndef __CODWindow_AC__
#	include "CODWindow_AC.h"
#	endif
#endif // qALOE

enum
{
	kActivateWindow = true, kDeactivateWindow = false
};

//========================================================================================
// static data members
//========================================================================================
short TWindow::gStandardWindowStaggerCount;

bool TWindow::pInBeginUpdate;
GrafPtr TWindow::pInBeginUpdateWindowPtr;
RgnHandle TWindow::pSavedUpdateRegion;
RgnHandle TWindow::pSavedVisRegion;
CPoint_AC TWindow::pSavedVisRegionOriginalOffset;
CWindowRefTable TWindow::fgWindows;

//========================================================================================
// Prototypes
//========================================================================================
CompareResult_AC TestElementForWindowRefSearch(ArrayIndex_AC anItem,
										 void* yourDataPtr);
CompareResult_AC CompareWindowRefs(WindowRef anItem,
							 void* yourDataPtr);
//========================================================================================
// struct CSearchWindowRefIndex
//========================================================================================
struct CSearchWindowRefIndex
{
public:
	// Constructor
	inline CSearchWindowRefIndex(CompareWindowRefType& TestItem,
							  void*& yourDataPtr,
							  WindowRef& theWindowRef,
							  const CWindowRefTable* theWindowRefList) :
		fTestItem(TestItem),
		fStaticLink(yourDataPtr),
		fWindowRef(theWindowRef),
		fSortedWindowRefList(theWindowRefList)
	{ }

	CompareWindowRefType& fTestItem;
	void*& fStaticLink;
	WindowRef& fWindowRef;
	const CWindowRefTable* fSortedWindowRefList;
};
//----------------------------------------------------------------------------------------
// TestElementForWindowRefSearch: 
//----------------------------------------------------------------------------------------
CompareResult_AC TestElementForWindowRefSearch(ArrayIndex_AC anItem,
										 void* yourDataPtr)
{
	CSearchWindowRefIndex * searchInfo = reinterpret_cast<CSearchWindowRefIndex*>(yourDataPtr);
	searchInfo->fWindowRef = searchInfo->fSortedWindowRefList->At(anItem)->fWindowRef;

	return searchInfo->fTestItem(searchInfo->fWindowRef, searchInfo->fStaticLink);
}


//========================================================================================
// struct CWindowRefComparer
//========================================================================================

struct CWindowRefComparer
{
public:
	// Constructor
	inline CWindowRefComparer(const WindowRef& theWindowRef,
						   const CWindowRefTable* theWindowRefList) :
		fWindowRef(theWindowRef),
		fSortedWindowRefList(theWindowRefList)
	{ }

	const WindowRef& fWindowRef;
	const CWindowRefTable* fSortedWindowRefList;
};


//----------------------------------------------------------------------------------------
// CompareWindowRefs: 
//----------------------------------------------------------------------------------------
CompareResult_AC CompareWindowRefs(WindowRef anItem,
							 void* yourDataPtr)
{
	CWindowRefComparer * comparisonInfo = (CWindowRefComparer *)yourDataPtr;

	return comparisonInfo->fSortedWindowRefList->Compare(comparisonInfo->fWindowRef, anItem);
}


//========================================================================================
// CLASS CWindowRefTable
//========================================================================================
//----------------------------------------------------------------------------------------
// CWindowRefTable destructor
//----------------------------------------------------------------------------------------
CWindowRefTable::~CWindowRefTable()
{
}

//----------------------------------------------------------------------------------------
// CWindowRefTable::At: 
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
// Return the index'th element of the list. 
TWindowAndWindowRef* CWindowRefTable::At(ArrayIndex_AC index) const
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CWindowRefTable.At");
		::ThrowException_AC(minErr, 0);
	}
#endif

	return (TWindowAndWindowRef*)ComputeAddress(index);
}
#endif

//----------------------------------------------------------------------------------------
// CWindowRefTable::AtDelete: 
//----------------------------------------------------------------------------------------
#if qRangeCheck || qDebugMsg
// Deletes via an index. 
void CWindowRefTable::AtDelete(ArrayIndex_AC index)
{
#if qRangeCheck
	if ((index < 1) || (index > fSize))
	{
#if qDebugMsg
		ProgramReport_AC(false, "fSize == %1d  index == %1d\n", fSize, index);
#endif

		::ProgramBreak_AC("Range Check in CWindowRefTable.AtDelete");
	}
#endif

	DeleteElementsAt(index, 1);
}
#endif

//----------------------------------------------------------------------------------------
// CWindowRefTable::Delete: 
//----------------------------------------------------------------------------------------
void CWindowRefTable::Delete(const WindowRef item)
{

	if (item)
	{
		ArrayIndex_AC index = GetEqualItemNo(item);
		if (index != kEmptyIndex_AC)
			AtDelete(index);
	}
}

//----------------------------------------------------------------------------------------
// CWindowRefTable::DoSearch: DON'T use exit to get out of this routine from your
// TestItem function or you will be really sad! (our debugger will check for you).
//----------------------------------------------------------------------------------------
TWindowAndWindowRef* CWindowRefTable::DoSearch(CompareWindowRefType TestItem,
								   void* yourDataPtr,
								   ArrayIndex_AC& index) const
{
	WindowRef aWindowRef = NULL;
	CSearchWindowRefIndex aSearcher(TestItem, yourDataPtr, aWindowRef, this);

	return DoSearchElement(::TestElementForWindowRefSearch, &aSearcher, index) ? At(index) : NULL;
}

//----------------------------------------------------------------------------------------
// CWindowRefTable::GetEqualItemNo: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CWindowRefTable::GetEqualItemNo(const WindowRef item) const
{
	ArrayIndex_AC returnVal = kEmptyIndex_AC;

	if (item)
	{
		CWindowRefComparer aWindowRefComparer(item, this);

		// Find the first reference to item in the list.
		// If item does not occur, return kEmptyIndex_AC (0).

		ArrayIndex_AC index;
		if (DoSearch(::CompareWindowRefs, &aWindowRefComparer, index))
			returnVal = index;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CWindowRefTable::Insert: 
//----------------------------------------------------------------------------------------
void CWindowRefTable::Insert(const TWindowAndWindowRef& item)
{
	InsertElementInOrder(&item);
}

//----------------------------------------------------------------------------------------
// CWindowRefTable::Compare: 
//----------------------------------------------------------------------------------------
CompareResult_AC CWindowRefTable::Compare(const WindowRef item1,
										 const WindowRef item2) const
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;
	// By default just compare the ordinal value of the items. Subclasses that want to use
	// the items as pointers or other types should override if the comparison is based on
	// the pointed at value
	if (item1 > item2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (item1 < item2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CWindowRefTable::CompareElements: 
//----------------------------------------------------------------------------------------
CompareResult_AC CWindowRefTable::CompareElements(const void* element1,
												 const void* element2) const// override 
{
	return Compare(reinterpret_cast<const TWindowAndWindowRef*>(element1)->fWindowRef, reinterpret_cast<const TWindowAndWindowRef*>(element2)->fWindowRef);
}


//========================================================================================
// CLASS TCloseWindowCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TCloseWindowCommand constructor
//----------------------------------------------------------------------------------------
TCloseWindowCommand::TCloseWindowCommand(CommandNumber itsCommandNumber,
											  TWindow* itsWindow) :
	TCommand(itsCommandNumber, itsWindow->GetCommandContext(itsCommandNumber), kCantUndo, kDoesNotCauseChange, NULL),

	fWindow(itsWindow),
	fMessage(NULL),
	fReply(NULL)
{
	fUseAppleEvent = true;		// The default will be to record.

#if qDebug
	// We should never get a window which has a document attached because we
	// intercept that situation in TWindow's CloseByUser and DoAEClose.
	if (fWindow->GetDocument() && fWindow->GetClosesDocument())
		::ProgramBreak_AC("Attempt to close window which has a document");
#endif
}

//----------------------------------------------------------------------------------------
// TCloseWindowCommand constructor
//----------------------------------------------------------------------------------------
TCloseWindowCommand::TCloseWindowCommand(TWindow* itsWindow,
											  TAppleEvent* message,
											  TAppleEvent* reply) :
	TCommand(cClose, itsWindow->GetCommandContext(cClose), kCantUndo, kDoesNotCauseChange, NULL),

	fWindow(itsWindow),
	fMessage(message),
	fReply(reply)
{
#if qDebug
	// We should never get a window which has a document attached because we
	// intercept that situation in TWindow's CloseByUser and DoAEClose.
	if (fWindow->GetDocument() && fWindow->GetClosesDocument())
		::ProgramBreak_AC("Attempt to close window which has a document");
#endif
}

//----------------------------------------------------------------------------------------
// TCloseWindowCommand destructor
//----------------------------------------------------------------------------------------
TCloseWindowCommand::~TCloseWindowCommand()
{
	// NOTE: We deliberately do not dispose of fMessage and fReply.
}

//----------------------------------------------------------------------------------------
// TCloseWindowCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------
TAppleEvent* TCloseWindowCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> returnVal(TH_new TAppleEvent(kAECoreSuite, kAEClose, gServerAddress, kAEWaitReply));

	CTempDesc theWindowDesc;
	fWindow->MakeObjectSpecifier(theWindowDesc, fWindow->GetSpecifierForm());
	returnVal->WriteParameter(keyDirectObject, theWindowDesc);

	// We don't save any of the saving options here because that is done
	// in the TCloseDocument command.

	return returnVal.release();
}

//----------------------------------------------------------------------------------------
// TCloseWindowCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TCloseWindowCommand::DoIt()
{
	if (fWindow)
	{
		fWindow->Close();
		if (fWindow->GetFreeOnClosing())
			delete fWindow;
	}
}


//========================================================================================
// CLASS TWindow
//========================================================================================
MA_DEFINE_CLASS(TWindow);

//----------------------------------------------------------------------------------------
// TWindow Constructor.
//----------------------------------------------------------------------------------------
TWindow::TWindow(WindowRef itsWMgrWindow,
					  bool canResize,
					  bool canClose,
					  bool inDispose) :
	MScriptableObject(cWindow),
	TView(gZeroPt, gZeroPt, sizeVariable, sizeVariable),

	// We can't tell if any changing will be required so we just the fMustxxx these false here. If
	// the developer sets them true then we respect them or if the developer calls the
	// corresponding functions we set them.
	fScriptingPrintHandler(NULL),
	fWMgrWindow(NULL),
	fTargetID(fIdentifier),
	fProcID(0),
	fPreDocname(0),
	fConstTitle(0),
	fStrListID(kNoResource),					// window's title:	STR# rsrc id
	fIndex(kNoResource),						//					index into STR#
	fDisposeOnFree(false),						// delete the storage
	fUpdating(false),
	fIsActive(false),
	fIsResizable(true),
	fIsClosable(true),
	fFreeOnClosing(false),
	fClosesDocument(true),
	fOpenInitially(true),
	fDoFirstClick(false),
	fFloats(false),
	fHideOnSuspend(false),
	fWasHiddenOnSuspend(false),
	fGenerateActivates(true),

	fMustAdapt(false),
	fMustHorzCenter(false),
	fMustVertCenter(false),
	fMustStagger(false),
	fMustForceOnScreen(false),

	fAdapted(false),							// Set by AdaptToScreen
	fHorzCentered(false),						// Set by Center 
	fVertCentered(false),						// Set by Center 
	fStaggered(false),							// Set by SimpleStagger 
	fForcedOnScreen(false)						// Set by ForceOnScreen 

#if qDrag
	,
	fDropStatusCache(0, sizeof(short)),
	fDragCallbackStatus(eWindowNotInited)
#endif // qDrag		

{
	fTarget = this;
	
	if (itsWMgrWindow)
	{
		GrafPtr savePort;							// can't be volatile because it is passed by reference
		GetPort(&savePort);

		SetPortWindowPort(itsWMgrWindow);	// So LocalToGlobal works 
		fLocation = CViewPoint(LocalToGlobal(((CRect_AC &)GetWindowPort(itsWMgrWindow)->portRect)[topLeft]));

		fSize = CViewPoint(((CRect_AC &)GetWindowPort(itsWMgrWindow)->portRect).GetSize());
		try
		{
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			SetPort(savePort);
			itsWMgrWindow = FreeIfWMgrWindow(itsWMgrWindow, inDispose);
			throw;
		}
		
		// Initialize instance variables.
		fWMgrWindow = itsWMgrWindow;
		fDisposeOnFree = inDispose;

		ResNumber itsID = 0;
		Handle theWDEFProc = WindowPeek(itsWMgrWindow)->windowDefProc;
		if (IsAResource_AC(theWDEFProc))
		{
			ResType itsType;
			CStr255_AC itsName;

			GetResInfo(theWDEFProc, &itsID, &itsType, itsName);
		}
		fProcID = (itsID << 4) | GetWVariant(fWMgrWindow);

		fIsResizable = canResize;

		try
		{
			if (canResize)
				AddAdorner(&TAdorner::fgResizeIconAdorner, kAdornLast, false);
			fIsClosable = canClose;
			BuildWindowRegions(BuildWindowRegions(kBuild));// sets fContentRegionInset && fContentDifference 

			// don't use the GetResizeLimits accessor b/c we want to preserve zero values for dynamics
			SetResizeLimits(fResizeLimits[topLeft], fResizeLimits[botRight]);

			CStr255_AC aString(GetTitle());

			short preDocname;
			short constTitle;
			if (ParseTitleTemplate(aString, preDocname, constTitle))
				SetWTitle(fWMgrWindow, aString);
			fPreDocname = preDocname;
			fConstTitle = constTitle;

			AddAdorner(&TAdorner::fgEraseAdorner, kAdornFirst, false);

			RegisterWindow(this);					// add to the lookup table
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			SetPort(savePort);
			throw;
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow copy constructor
//----------------------------------------------------------------------------------------
TWindow::TWindow(const TWindow& inWindow) :
	TView(inWindow),
	MScriptableObject(inWindow),

	fScriptingPrintHandler(inWindow.fScriptingPrintHandler),
	fTarget(FindSubView(inWindow.fTargetID)),
	fWMgrWindow(NULL), // don't copy the incoming one it doesn't belong to us
	fTargetID(inWindow.fTargetID),
	fProcID(inWindow.fProcID),
	fPreDocname(inWindow.fPreDocname),
	fConstTitle(inWindow.fConstTitle),
	fStrListID(inWindow.fStrListID),
	fIndex(inWindow.fIndex),
	fDisposeOnFree(inWindow.fDisposeOnFree),
	fUpdating(inWindow.fUpdating),
	fIsActive(inWindow.fIsActive),
	fIsResizable(inWindow.fIsResizable),
	fIsClosable(inWindow.fIsClosable),
	fFreeOnClosing(inWindow.fFreeOnClosing),
	fClosesDocument(inWindow.fClosesDocument),
	fOpenInitially(inWindow.fOpenInitially),
	fDoFirstClick(inWindow.fDoFirstClick),
	fFloats(inWindow.fFloats),
	fHideOnSuspend(inWindow.fHideOnSuspend),
	fWasHiddenOnSuspend(inWindow.fWasHiddenOnSuspend),
	fGenerateActivates(inWindow.fGenerateActivates),

	fMustAdapt(inWindow.fMustAdapt),
	fMustHorzCenter(inWindow.fMustHorzCenter),
	fMustVertCenter(inWindow.fMustVertCenter),
	fMustStagger(inWindow.fMustStagger),
	fMustForceOnScreen(inWindow.fMustForceOnScreen),

	fAdapted(inWindow.fAdapted),
	fHorzCentered(inWindow.fHorzCentered),
	fVertCentered(inWindow.fVertCentered),
	fStaggered(inWindow.fStaggered),
	fForcedOnScreen(inWindow.fForcedOnScreen)

#if qDrag
	,
	fDropStatusCache(0, sizeof(short)),
	fDragCallbackStatus(inWindow.fDragCallbackStatus)
#endif // qDrag		

{
	if (inWindow.fWMgrWindow)
	{
		CTempAllocation tempAllocation(true);

		CRect_AC boundsRect(GetWindowPort(inWindow.fWMgrWindow)->portRect);

		WindowRef behind = inWindow.IsModal() ? (WindowRef) - 1 : inWindow.GetBehindWindowPtr();

		CStr255_AC windowTitle(inWindow.GetTitle());

		long windowRefCon = GetWRefCon(inWindow.fWMgrWindow);

		WindowRef storage = NULL;
		if (!fDisposeOnFree)
			storage = (WindowRef)TH_new CWindowRecord;

		WindowRef aWMgrWindow;
		if (qNeedsColorQD || HasColorQD_AC())
			aWMgrWindow = NewCWindow(storage, &boundsRect, windowTitle, false, inWindow.fProcID, behind, inWindow.fIsClosable, windowRefCon);
		else
			aWMgrWindow = NewWindow(storage, &boundsRect, windowTitle, false, inWindow.fProcID, behind, inWindow.fIsClosable, windowRefCon);

		if (!aWMgrWindow)
		{
			delete storage;
	
			::ThrowIfNULL_AC(aWMgrWindow);
		}

		fWMgrWindow = aWMgrWindow;

		RegisterWindow(this);					// add to the lookup table
	}
}

//----------------------------------------------------------------------------------------
// TWindow::DoPostCreate: 
//----------------------------------------------------------------------------------------
void TWindow::DoPostCreate(TDocument* itsDocument)// Override
{
	TView::DoPostCreate(itsDocument);

	// don't use the GetResizeLimits accessor b/c we want to preserve zero values for dynamics
	SetResizeLimits(fResizeLimits[topLeft], fResizeLimits[botRight]);
	BuildWindowRegions(BuildWindowRegions(kBuild));// sets fContentRegionInset && fContentDifference

#if qDrag
	fDragCallbackStatus = eNotInstalled;

	if (HasDragManager_AC() && !fDroppableViewList.IsEmpty())// If subviews have registered then register the window with the drag session
	{
		TDragDropSession::fgDragDropSession->RegisterDroppableWindow(this, fWMgrWindow);
		fDragCallbackStatus = eInstalled;
	}
#endif // qDrag
}

//----------------------------------------------------------------------------------------
// TWindow::GetBehindWindowPtr: 
//----------------------------------------------------------------------------------------
WindowRef TWindow::GetBehindWindowPtr() const
{
	WindowRef behind = GetLastFloatingWindowPtr();
	if (!behind)
		behind = (WindowRef)(-1);				// bring the window up in front of all other windows

	return behind;
}

//----------------------------------------------------------------------------------------
// TWindow::GetDialogBehavior: 
//----------------------------------------------------------------------------------------
TDialogBehavior* TWindow::GetDialogBehavior() const
{
	// This can return NULL.

	return dynamic_cast<TDialogBehavior*>(GetBehaviorWithIdentifier(kDialogBehavior));
}

//----------------------------------------------------------------------------------------
// TWindow::SetModality: 
//----------------------------------------------------------------------------------------
void TWindow::SetModality(bool modal)
{
	TDialogBehavior * itsDialogBehavior = GetDialogBehavior();

	if (itsDialogBehavior)
		itsDialogBehavior->fModal = modal;
	else
	{
		if (modal)
		{
			// Allocate the behavior from temporary memory in case we are trying to warn
			// the user about something.  We shouldn't fail here.
			CTempObjectAllocation tempObjectAllocation(true);
			itsDialogBehavior = TH_new TDialogBehavior(modal, kNoIdentifier, kNoIdentifier);

			AddBehavior(itsDialogBehavior);
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow::SetDialogItems: 
//----------------------------------------------------------------------------------------
void TWindow::SetDialogItems(IDType itsDefaultItem,
							 IDType itsCancelItem)
{
	TDialogBehavior * itsDialogBehavior = GetDialogBehavior();

	if (itsDialogBehavior)
	{
		itsDialogBehavior->fDefaultItem = itsDefaultItem;
		itsDialogBehavior->fCancelItem = itsCancelItem;
	}
	else
	{
		if ((itsDefaultItem != kNoIdentifier) || (itsCancelItem != kNoIdentifier))
		{
			// Allocate the behavior from temporary memory in case we are trying to warn
			// the user about something.  We shouldn't fail here.
			CTempObjectAllocation tempObjectAllocation(true);
			itsDialogBehavior = TH_new TDialogBehavior(false, itsDefaultItem, itsCancelItem);

			AddBehavior(itsDialogBehavior);
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow::Free: 
//----------------------------------------------------------------------------------------
TWindow::~TWindow()
{
	bool disposeOnFree = fDisposeOnFree;
	WindowRef wmgrWindow = fWMgrWindow;

#if qDrag
	if (HasDragManager_AC() && fDragCallbackStatus == eInstalled)// remove drag manager callbacks
	{
		TDragDropSession::fgDragDropSession->UnregisterDroppableWindow(fWMgrWindow);
		fDragCallbackStatus = eNotInstalled;
	}
#endif // qDrag

	DeleteWindow(this);							// remove from the lookup table
	fWMgrWindow = NULL;
	BeInPort(NULL);								// make sure subviews don't depend on the window manager window

	if (fDocument)
		fDocument->DeleteWindow(this);
	else if (gDispatcher)
		gDispatcher->DeleteWindow(this);

	wmgrWindow = FreeIfWMgrWindow(wmgrWindow, disposeOnFree);
}

//----------------------------------------------------------------------------------------
// TWindow::GetStandardSignature: 
//----------------------------------------------------------------------------------------
IDType TWindow::GetStandardSignature() const			// override 
{
	return kStdWindow;
}

//----------------------------------------------------------------------------------------
// TWindow::ReadFields: 
//----------------------------------------------------------------------------------------
void TWindow::ReadFields(CStream_AC* aStream)		// override 
{
	TView::ReadFields(aStream);

	fProcID = aStream->ReadInteger();
	fTargetID = aStream->ReadIDType();

	WindowFlags itsFlags;
	aStream->ReadBytes(&itsFlags, sizeof(WindowFlags));
	fIsResizable = itsFlags.GetResizable();
	fIsClosable = itsFlags.GetHasGoAway();
	fDoFirstClick = itsFlags.GetDoFirstClick();
	fFreeOnClosing = itsFlags.GetFreeOnClosing();

	fDisposeOnFree = itsFlags.GetDisposeOnFree();
	fDisposeOnFree = false; // ignore the stream value, template windows allocate as they wish

	fClosesDocument = itsFlags.GetClosesDocument();
	fOpenInitially = itsFlags.GetOpenInitially();

	fMustAdapt = itsFlags.GetMustAdaptToScreen();
	fMustStagger = itsFlags.GetStagger();
	fMustForceOnScreen = itsFlags.GetMustForceOnScreen();
	fMustHorzCenter = itsFlags.GetHorzCenter();
	fMustVertCenter = itsFlags.GetVertCenter();
	fFloats = itsFlags.GetFloats();

	fHideOnSuspend = itsFlags.GetHideOnSuspend();
	fGenerateActivates = itsFlags.GetGenerateActivates();

	fStrListID = aStream->ReadInteger();
	fIndex = aStream->ReadInteger();
	CStr255_AC title;
	if (fStrListID != kNoResource)			// retrieve the window's title from the resource
		GetIndString(title, fStrListID, fIndex);

	// If we already have a window it is because someone wanted it. Don't lose it
	if (!fWMgrWindow)
	{
		WindowRef behind = IsModal() ? (WindowRef) - 1 : GetBehindWindowPtr();
	
		// Compatibility Floating WDEF support. We will
		// use the WDEF specified by this ID if we are not
		// running on System 7.5 or better. If we are, however,
		// we will change the ID dynamically in the window
		// initialization sequence. This ensures that we use
		// the WDEF stored in your program on pre System 7.5
		// but, use the WDEF stored in the system thereafter.		
	
		// top 12 bits identify the WDEF, bottom 4 bits are the style
		short procID = fProcID;					// default
		if (qNeedsSystem7_5 || GetSystemVersion_AC() >= 0x750)
			if ((fProcID & 0xFFF0) == kSystem75AndLess_kWindoidWDEF)
				procID = kSystem75_kWindoidWDEF | (fProcID & 0x000F);
	
		CRect_AC theBounds = GetFrame().ToRect();
	
		WindowRef storage = NULL;
		if (!fDisposeOnFree)
			storage = (WindowRef)TH_new CWindowRecord;
	
		WindowRef aWMgrWindow;
		if (qNeedsColorQD || HasColorQD_AC())
			aWMgrWindow = (WindowRef)NewCWindow(storage, &theBounds, title, false, procID, behind, fIsClosable, reinterpret_cast<long>(this));
		else
			aWMgrWindow = NewWindow(storage, &theBounds, title, false, procID, behind, fIsClosable, reinterpret_cast<long>(this));
		
		if (!aWMgrWindow)
		{
			delete storage;
	
			::ThrowIfNULL_AC(aWMgrWindow);
		}
	
		fWMgrWindow = aWMgrWindow;
	}

	title = GetTitle();

	short preDocname;
	short constTitle;
	if (ParseTitleTemplate(title, preDocname, constTitle))
		SetWTitle(fWMgrWindow, title);
	fPreDocname = preDocname;
	fConstTitle = constTitle;

	RegisterWindow(this);					// add to the lookup table
}

//----------------------------------------------------------------------------------------
// TWindow::WriteFields: 
//----------------------------------------------------------------------------------------
void TWindow::WriteFields(CStream_AC* aStream) const		// override 
{
	TView::WriteFields(aStream);

	aStream->WriteInteger(fProcID);
	aStream->WriteIDType(fTargetID);

	WindowFlags itsFlags;

	itsFlags.Initialize();
	itsFlags.SetHasGoAway(fIsClosable);
	itsFlags.SetResizable(fIsResizable);
	itsFlags.SetDoFirstClick(fDoFirstClick);
	itsFlags.SetFreeOnClosing(fFreeOnClosing);
	itsFlags.SetDisposeOnFree(fDisposeOnFree);
	itsFlags.SetClosesDocument(fClosesDocument);
	itsFlags.SetOpenInitially(fOpenInitially);
	itsFlags.SetMustAdaptToScreen(fMustAdapt);
	itsFlags.SetStagger(fMustStagger);
	itsFlags.SetMustForceOnScreen(fMustForceOnScreen);
	itsFlags.SetVertCenter(fMustVertCenter);
	itsFlags.SetHorzCenter(fMustHorzCenter);
	itsFlags.SetFloats(fFloats);
	itsFlags.SetHideOnSuspend(fHideOnSuspend);
	itsFlags.SetGenerateActivates(fGenerateActivates);

	// write the flags
	aStream->WriteBytes(&itsFlags, sizeof(WindowFlags));

	// write the window title
	aStream->WriteInteger(fStrListID);
	aStream->WriteInteger(fIndex);
}

//----------------------------------------------------------------------------------------
// TWindow::Activate: 
//----------------------------------------------------------------------------------------
void TWindow::Activate(bool entering)
{
	bool different = (entering != IsActive());
	if (different)
	{
		// Change to allow floating windows to be activated even if there is an
		// active document window around.
		bool someOneElseActive = false;
		if (entering && !fFloats)
			// make sure that there is not already an active window - we
			// can't have two active ones at the same time (note that code outside
			// MacApp might call SelectWindow and generate activate events for windows
			// that really are not active, so we have to catch this possibility)
			for (CWMgrIterator iter; iter; ++iter)
			{
				TWindow * theMAWindow = WMgrToWindow(iter);
				if (theMAWindow && theMAWindow->IsActive() && !IsVisibleFloatWindow(iter))
				{
					someOneElseActive = true;					// someone else is active - bail out!
					break;
				}
			}

		if (!someOneElseActive)
		{
			TView::Activate(entering);
	
			fIsActive = entering;
	
			if (!(fFloats && (fTarget == this)))
				SetTarget(entering ? fTarget : gDispatcher);
	
			// When we de/activate a window, we need to invalidate the menus manually here.
			// Because we no longer receive activate/deactivates regularly from the Event
			// Manager, we cannot count on TApplication::DidEvent to handle this for us
			InvalidateMenus();
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow::AdaptToScreen: 
//----------------------------------------------------------------------------------------
void TWindow::AdaptToScreen()
{
	const CViewPoint stdScreen(512, 342);

	fAdapted = true;							// We adapted to the screen 

	// Compute pixel difference between current screen and std Mac screen Note:
	// Calculation written as two separate expressions so that CPoint_AC's "-" operator can
	// be inline.
	CViewPoint diff(((CRect_AC &)qd.screenBits.bounds).GetSize());
	diff -= stdScreen;

	CViewRect theFrame(GetFrame());

	// If screen is larger, enlarge the window. If the window is too large, shrink it 
	if (diff != gZeroVPt || !((CRect_AC &)qd.screenBits.bounds).Contains(theFrame[botRight].ToPoint()))
	{
		// do an adjustment for larger screen
		// the window adapted to the new screen size by proportion. 
		CViewPoint newSize(Fix2Long(FixMul(Long2Fix(fSize.h), FixDiv(Long2Fix(stdScreen.h + diff.h), Long2Fix(stdScreen.h)))), Fix2Long(FixMul(Long2Fix(fSize.v), FixDiv(Long2Fix(stdScreen.v + diff.v), Long2Fix(stdScreen.v)))));

		// Consider the location of the window, bound it by the resize limits 
		newSize.ConstrainTo(GetDynamicResizeLimits());
		Resize(newSize, kInvalidate);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::AllowsMenuAccess: 
//----------------------------------------------------------------------------------------
bool TWindow::AllowsMenuAccess()
{
	// Default is to always allow menu access, even if modal 
	return true;
}


typedef WindowDefProcPtr* WDefProcTypeHandle;

//----------------------------------------------------------------------------------------
// TWindow::Center: 
//----------------------------------------------------------------------------------------
void TWindow::Center(bool horizontally,
					 bool vertically,
					 bool forDialog)
{
	if (fWMgrWindow)
	{
		fHorzCentered = horizontally;
		fVertCentered = vertically;

		bool rgnsWereBuilt = BuildWindowRegions(kBuild);
		CTempRegion_AC strucRgn;
		GetWindowStructureRgn(fWMgrWindow, strucRgn);
		CPoint_AC windowSize(((CRect_AC &)(*strucRgn)->rgnBBox).GetSize());
		BuildWindowRegions(rgnsWereBuilt);

		CViewRect theFrame(GetFrame());

		// GetMaxIntersectedDevice accounts for menubar so don't subtract GetMBarHeight
		CRect_AC screenRect;
		GetMaxIntersectedDevice(screenRect);
		CPoint_AC screenSize = screenRect.GetSize();

		if (horizontally)
			theFrame.left = screenRect.left + fContentRegionInset.h + (screenSize.h - windowSize.h) / 2;

		if (vertically)
		{
			if (forDialog)
			// Put it in the top third of the screen 
				theFrame.top = screenRect.top + fContentRegionInset.v + (screenSize.v - windowSize.v) / 3;
			else
				theFrame.top = screenRect.top + fContentRegionInset.v + (screenSize.v - windowSize.v) / 2;
		}

		Locate(theFrame[topLeft], kDontInvalidate);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::Close: 
//----------------------------------------------------------------------------------------
void TWindow::Close()							// override 
{
	TView::Close();

	Show(false, kRedraw);
}

//----------------------------------------------------------------------------------------
// TWindow::CloseAndFree: 
//----------------------------------------------------------------------------------------
void TWindow::CloseAndFree()
{
#if qDebug
	::ProgramBreak_AC("TWindow::CloseAndFree is deprecated: please call Close() and then if necessary free the object");
#endif

	Close();
	if (fFreeOnClosing)
		delete this;
}

//----------------------------------------------------------------------------------------
// TWindow::CloseByUser: 
//----------------------------------------------------------------------------------------
void TWindow::CloseByUser(CommandNumber aCommandNumber)
{
	// First see if the document is to be closed.  This will happen if there
	// is a document and we're not the clipboard window.
	::Assertion_AC(gClipboardMgr != NULL, "gClipboardMgr is unexpectedly NULL");

	if (fDocument && gClipboardMgr->fClipWindow != this)
	{
		// Allow the document to determine the proper response to
		// closing one of its windows.
		fDocument->CloseWindow(this);
	}
	else
	{
		// Close the window directly since it wasn't attached to a document.
		DoClose(aCommandNumber);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::DoClose: 
//----------------------------------------------------------------------------------------
void TWindow::DoClose(CommandNumber aCommand,
					  bool useAppleEvent)
{
	// Close the window if it wasn't attached to a document.
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	TCloseWindowCommand * aCloseWindowCommand = TH_new TCloseWindowCommand(aCommand, this);
	aCloseWindowCommand->SetUseAppleEvent(useAppleEvent);

	if (useAppleEvent)
		PostCommand(aCloseWindowCommand);
	else
		aCloseWindowCommand->Process();

}

//----------------------------------------------------------------------------------------
// TWindow::DoMenuCommand: 
//----------------------------------------------------------------------------------------
void TWindow::DoMenuCommand(CommandNumber aCommandNumber)// override 
{
	switch (aCommandNumber)
	{
		case cClose:
			CloseByUser(aCommandNumber);
			break;

		default:
			TView::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::DoSetupMenus: 
//----------------------------------------------------------------------------------------
void TWindow::DoSetupMenus()					// override 
{
	if (!IsInModalState())						// Don't enable menu/app commands if modal 
	{
		Enable(cClose, fIsClosable);			// window objects take care of themselves! 

		TView::DoSetupMenus();
	}
}

//----------------------------------------------------------------------------------------
// TWindow::HandlesCursor: 
//----------------------------------------------------------------------------------------
bool TWindow::HandlesCursor() const				// override 
{
	return fHandlesCursor && (IsActive() || fDoFirstClick);
}

//----------------------------------------------------------------------------------------
// TWindow::LetsSubViewsHandleCursor: 
//----------------------------------------------------------------------------------------
bool TWindow::LetsSubViewsHandleCursor() const		// override 
{
	return fLetsSubViewsHandleCursor && (IsActive() || fDoFirstClick);
}

//----------------------------------------------------------------------------------------
// TWindow::HandlesHelp: 
//----------------------------------------------------------------------------------------
bool TWindow::HandlesHelp() const					// override 
{
	return fHandlesHelp && IsActive();
}

//----------------------------------------------------------------------------------------
// TWindow::LetsSubViewsHandleHelp: 
//----------------------------------------------------------------------------------------
bool TWindow::LetsSubViewsHandleHelp() const		// override 
{
	return fLetsSubViewsHandleHelp && IsActive();
}

//----------------------------------------------------------------------------------------
// TWindow::DrawResizeIcon: 
//----------------------------------------------------------------------------------------
void TWindow::DrawResizeIcon()
{
	if (fIsResizable && Focus())
	{
		CRect_AC r(GetQDExtent());
		r[topLeft] = r[botRight] - CPoint_AC(kSBarSizeMinus1, kSBarSizeMinus1);

		// clip down and avoid drawing the lines that delimit scroll bars

		CTempRegion_AC tempRgn;
		CTempRegion_AC tempRgn2;

		// Avoid drawing the GrowIcon if it would have been clipped
		GetClip(tempRgn);
		RectRgn(tempRgn2, &r);
		SectRgn(tempRgn2, tempRgn, tempRgn2);
		if (!EmptyRgn(tempRgn2))
		{
			SetClip(tempRgn2);
			PenNormal();
			DrawGrowIcon(fWMgrWindow);
			SetClip(tempRgn);
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow::Focus: 
//----------------------------------------------------------------------------------------
bool TWindow::Focus()						// override 
{
	bool returnVal = false;

	if (IsFocused())							// Already focused 
	{
#if qDebug
		GrafPtr currentPort;

		GetPort(&currentPort);
		if (GetGrafPort() != currentPort)
			::ProgramBreak_AC("TWindow.Focus: Port is incorrect");
#endif
		returnVal = true;
	}
	else if (fWMgrWindow)						// do my own focus 
	{
		// Set the port 
		GrafPtr currentPort;
		GrafPtr theWindowPort = GetGrafPort();

		GetPort(&currentPort);
		if (currentPort != theWindowPort)
			SetPort(theWindowPort);

		CPoint_AC qdOrigin(GetQDOrigin());
		SetOrigin(qdOrigin.h, qdOrigin.v);
		gFocusedView = this;

		// Add the clipping.  Clip either to the visrgn or the update region depending on whether
		// there is an invalid area.
		RgnHandle theUpdateRgn = GetUpdateRegion(fWMgrWindow);
		if (!EmptyRgn(theUpdateRgn))
		{
			// The update region is in global coords but the clip is in local coords.
			// Offset the region to make it in local coords here and restore it there
			// to save copying it
			SuperViewToLocal(theUpdateRgn);
			CTempRegion_AC tempRgn;

			if (fUpdating)						// clip TO the update region 
				SectRgn(GetVisRegion(theWindowPort), theUpdateRgn, tempRgn);
			else								// clip OUT the update region 
				DiffRgn(GetVisRegion(theWindowPort), theUpdateRgn, tempRgn);
			SetClip(tempRgn);
			LocalToSuperView(theUpdateRgn);
		}
		else
			SetClip(GetVisRegion(theWindowPort));

		// now setup the default content region color. This didn't use to show up as
		// a problem, but with widespread use of platinum appearance we find that the
		// window content region color is much more important and can't just be set
		// to white anymore. If this is not done here then any erasure in the window
		// will not have the colors setup right.
		if (qNeedsColorQD || HasColorQD_AC())
		{
			AuxWinHandle awHndl = NULL;
			WCTabHandle ctabHandle = GetAuxWin(fWMgrWindow, &awHndl) ? (WCTabHandle)(*awHndl)->awCTable : (WCTabHandle)awHndl;
			
			SetIfBkColor((*ctabHandle)->ctTable->rgb); // content background color is first entry
		}
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TWindow::FocusOnSuperView: 
//----------------------------------------------------------------------------------------
bool TWindow::FocusOnSuperView()				// override 
{
	return false;
}

//----------------------------------------------------------------------------------------
// TWindow::GetLocationAdjustment: 
//----------------------------------------------------------------------------------------
CPoint_AC TWindow::GetLocationAdjustment() const
{
	CPoint_AC delta;

	const short kMinDragArea = 4;
	CViewRect theVFrame(GetFrame());
	CRect_AC theFrame(theVFrame.ToRect());

	// since we don't _really_ know what the drag rgn is, we'll assume that moving the
	// topleft pt of the window on screen is sufficient to make it draggable, so calculate
	// the deltas necessary to move the topleft pt into visible screen CRect_AC.

	CRect_AC visScreenRect;
	GetMaxIntersectedDevice(visScreenRect);		// NOTE: uses tempRgn 
	visScreenRect.Inset(kMinDragArea, kMinDragArea);

	if (theFrame.top < visScreenRect.top)
		delta.v = visScreenRect.top - theFrame.top + fContentRegionInset.v;
	else if (theFrame.top > visScreenRect.bottom)
		delta.v = visScreenRect.bottom - theFrame.top - fContentRegionInset.v;

	if (theFrame.left < visScreenRect.left)
		delta.h = visScreenRect.left - theFrame.left + fContentRegionInset.h;
	else if (theFrame.left > visScreenRect.right)
		delta.h = visScreenRect.right - theFrame.right - fContentRegionInset.h;

	return delta;
}


//----------------------------------------------------------------------------------------
// TWindow::ForceOnScreen: ForceOnScreen guarantees that some minimal drag area is
// accessible to the user, to be dragged to the desired location.
//----------------------------------------------------------------------------------------
void TWindow::ForceOnScreen()
{
	const short kMinDragArea = 4;

	fForcedOnScreen = true;

	// On many systems (including Color QD && the Radius FPD), it's possible to have a
	// non-rectangular desktop. Try to be nice to people who saved windows on secondary
	// screens. GrayRgn is the true indicator of the shape of the
	// desktop--screenBits.bounds is the size of the screen with the menu bar on it.

	CTempRegion_AC tempRgn;
	CTempRegion_AC strucRgn;
	CTempRegion_AC contRgn;

	bool rgnsWereBuilt = BuildWindowRegions(kBuild);

	GetWindowStructureRgn(fWMgrWindow, strucRgn);
	GetWindowContentRgn(fWMgrWindow, contRgn);
	{
		DiffRgn(strucRgn, contRgn, tempRgn);	// strucRgn less contRgn  drag rgn
		if (EmptyRgn(tempRgn))
			CopyRgn(strucRgn, tempRgn);			// at least get the strucRgn 
	}
	BuildWindowRegions(rgnsWereBuilt);

	// get the desktop rgn, inset by a minimal drag area 
	CTempRegion_AC aTempRgn;

	CopyRgn(GetGrayRgn(), aTempRgn);			// aTempRgn = desktop rgn 
	InsetRgn(aTempRgn, kMinDragArea, kMinDragArea);// inset aTempRgn 
	SectRgn(tempRgn, aTempRgn, aTempRgn);		// do drag rgn && desktop rgn instersect ? 

	CPoint_AC delta;

	if (EmptyRgn(aTempRgn) || !RectInDrag((*(RgnHandle)aTempRgn)->rgnBBox))
		delta = GetLocationAdjustment();			// no => adjust the window's location 

	Locate(fLocation.ToPoint() + delta, kDontInvalidate);
}

//----------------------------------------------------------------------------------------
// TWindow::GetMaxIntersectedDevice: 
//----------------------------------------------------------------------------------------
GDHandle TWindow::GetMaxIntersectedDevice(CRect_AC& screenRect) const
{
	GDHandle returnVal = NULL;
	if (qNeedsColorQD || HasColorQD_AC())
	{
		long maxSectArea = 0;
		CRect_AC moveBounds(GetDynamicMoveBounds());

		// set fContentRegionInset && fContentDifference, ensure contrgn and strucrgn are available
		bool rgnsWereBuilt = BuildWindowRegions(kBuild);
		CTempRegion_AC strucRgn;
		GetWindowStructureRgn(fWMgrWindow, strucRgn);
		CRect_AC globalStrucRect((*strucRgn)->rgnBBox);
		BuildWindowRegions(rgnsWereBuilt);

		GDHandle aGDHandle = GetDeviceList();
		GDHandle maxSectGD = GetMainDevice();	// set as best choice default 
		while (aGDHandle)
		{										// calc which scrn intersects largest part of window
			if (TestDeviceAttribute(aGDHandle, screenDevice) && TestDeviceAttribute(aGDHandle, screenActive))
			{
				CRect_AC aGDScreenRect((*aGDHandle)->gdRect);
				CRect_AC gdSectRect(globalStrucRect & aGDScreenRect);
				if (!(aGDScreenRect & moveBounds).Empty() && !gdSectRect.Empty())
				{
					long sectArea = gdSectRect.GetLength(vSel) * gdSectRect.GetLength(hSel);
					if (sectArea > maxSectArea)	// do we have a new winner? 
					{
						maxSectArea = sectArea;
						maxSectGD = aGDHandle;
					}
				}
			}
			aGDHandle = GetNextDevice(aGDHandle);
		}

		if (maxSectGD != GetMainDevice())
			screenRect = (*maxSectGD)->gdRect;
		else
		{										// Account for menu bar on the main screen.
			// Don't just assume that its at the top of
			// the screen!
			CTempRegion_AC tempRgn;
			CRect_AC gdRect = (*maxSectGD)->gdRect;

			RectRgn(tempRgn, &gdRect);			// main screen with menubar 
			SectRgn(tempRgn, GetGrayRgn(), tempRgn);// GetGrayRgn == desktop rgn w/o menubar 
			screenRect = (*(RgnHandle)tempRgn)->rgnBBox;// => main screen w/o menubar 
		}
		returnVal = maxSectGD;
	}
	else
	{											// Account for menu bar on the main
		// screen. Don't just assume that its at
		// the top of the screen!
		CTempRegion_AC tempRgn;

		RectRgn(tempRgn, &(qd.screenBits.bounds));// main screen with menubar 
		SectRgn(tempRgn, GetGrayRgn(), tempRgn);// GetGrayRgn == desktop rgn w/o menubar 
		screenRect = (*(RgnHandle)tempRgn)->rgnBBox;// => main screen w/o menubar 
	}
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TWindow::GetGrafPort: 
//----------------------------------------------------------------------------------------
GrafPtr TWindow::GetGrafPort() const					// override 
{
	return gPrinting || gDrawingPictScrap ? qd.thePort : ((GrafPtr)fWMgrWindow);
}

//----------------------------------------------------------------------------------------
// TWindow::GetWindowTarget: 
//----------------------------------------------------------------------------------------
TEventHandler* TWindow::GetWindowTarget() const
{
	return fTarget;
}

//----------------------------------------------------------------------------------------
// TWindow::GetTitle: 
//----------------------------------------------------------------------------------------
CStr255_AC TWindow::GetTitle() const
{
	CStr255_AC returnVal;

	if (fWMgrWindow)
		GetWTitle(fWMgrWindow, returnVal);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TWindow::GetWindow: 
//----------------------------------------------------------------------------------------
TWindow* TWindow::GetWindow() const				// override 
{
	return const_cast<TWindow*>(this);
}

//----------------------------------------------------------------------------------------
// TWindow::GoAwayByUser: 
//----------------------------------------------------------------------------------------
void TWindow::GoAwayByUser(const CViewPoint& theMouse)
{
	if (fIsClosable && TrackGoAway(fWMgrWindow, LocalToSuperView(theMouse).ToPoint()))
		CloseByUser(cClose);
}

//----------------------------------------------------------------------------------------
// TWindow::HandleMouseDown: 
//----------------------------------------------------------------------------------------
bool TWindow::HandleMouseDown(const CViewPoint& theMouse,
								 TToolboxEvent* event,
								 CPoint_AC hysteresis,
								 EMouseDownType	/* mouseDownType */)// override 
{
	bool wasHandled = false;

	// Find out where the mouse was pressed 
	short aPartCode = GetPartCode(theMouse);

	// Test to determine if the window should be selected. To be selected,
	//   this window must not be the active window; and
	//   this window must not float, or if it does, it must not be front-most.
	EMouseDownType mouseDownType = kDragOrClick;
	if ((gDispatcher->GetActiveWindow(kNoFloaters) != this) && (!fFloats || (fWMgrWindow != FrontWindow())))
	{
		mouseDownType = fDoFirstClick ? kFirstClick_dragOrClick : kFirstClick_dragOnly;

		if (aPartCode != inContent && (aPartCode != inDrag || !event->IsCommandKeyPressed()))
		{
			Select();
			gDispatcher->UpdateAllWindows();
		}
	}

	// dispatch according to where the mouse was pressed 
	switch (aPartCode)
	{
		case inContent:
			wasHandled = TView::HandleMouseDown(theMouse, event, hysteresis, mouseDownType);
			break;

		case inDrag:
			MoveByUser(theMouse);
			wasHandled = true;
			break;

		case inGrow:
			
			if(GetIsResizable())
			{
				ResizeByUser(theMouse);
				wasHandled = true;
			}
			else
				wasHandled = TView::HandleMouseDown(theMouse, event, hysteresis, mouseDownType);
			
			break;

		case inGoAway:
			GoAwayByUser(theMouse);
			wasHandled = true;
			break;

		case inZoomIn:
		case inZoomOut:
			ZoomByUser(theMouse, aPartCode);
			wasHandled = true;
			break;

		case inDesk:
			wasHandled = true;
			break;

		default:
			break;
	}

	return wasHandled;
}

//----------------------------------------------------------------------------------------
// TWindow::HasPendingUpdate: 
//----------------------------------------------------------------------------------------
bool TWindow::HasPendingUpdate() const
{
	return !EmptyRgn(GetUpdateRegion(fWMgrWindow));
}

//----------------------------------------------------------------------------------------
// TWindow::GetPartCode: 
//----------------------------------------------------------------------------------------
short TWindow::GetPartCode(const CViewPoint& theMouse) const// override 
{
	short returnVal = inDesk;

	// Call FindWindow to determine the partcode 
	WindowRef aWMgrWindow;
	short partCode = FindWindow(LocalToSuperView(theMouse).ToPoint(), &aWMgrWindow);
	if (aWMgrWindow == fWMgrWindow)				// if we fail this sanity check, then 
		returnVal = partCode;						// passed the sanity check so return the
													// part code we got back from FindWindow
#if qDebug
	else
		::ProgramBreak_AC("in TWindow.GetPartCode: passed a CViewPoint that didn't belong to the window");
#endif

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TWindow::BeInDocument: 
//----------------------------------------------------------------------------------------
void TWindow::BeInDocument(TDocument* itsDocument)
{
	TDocument* oldDocument = fDocument;
	
	TView::BeInDocument(itsDocument);

	if (itsDocument && itsDocument != oldDocument)
	{
		gDispatcher->DeleteWindow(this);		// Just in case 

		CStr255_AC aString(itsDocument->GetTitle());
		if (!aString.IsEmpty())					// not an untitled document 
			SetTitleForDoc(aString);
		fNextHandler = itsDocument;
	}
	else
	{
		gDispatcher->AddWindow(this);
		fNextHandler = gDispatcher;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::IsDismissed: 
//----------------------------------------------------------------------------------------
bool TWindow::IsDismissed() const
{
	TDialogBehavior * theDialogBehavior = GetDialogBehavior();

	return theDialogBehavior ? theDialogBehavior->fDismissed : true;
}

//----------------------------------------------------------------------------------------
// TWindow::Dismiss: 
//----------------------------------------------------------------------------------------
void TWindow::Dismiss(IDType dismisser,
					  bool validate)
{
	TDialogBehavior * theDialogBehavior = GetDialogBehavior();

	if (theDialogBehavior)
		theDialogBehavior->Dismiss(dismisser, validate);
}

//----------------------------------------------------------------------------------------
// TWindow::RectInDrag: 
//----------------------------------------------------------------------------------------
bool TWindow::RectInDrag(const CRect_AC& whichRect) const
// Returns true if any of the corner points of whichRect are draggable. 
{
	bool result = false;						// assume window isn't draggable

	WDefProcTypeHandle wDefProc = (WDefProcTypeHandle)GetAndLoadWDefProc(((WindowPeek)fWMgrWindow)->windowDefProc);
	if (wDefProc)
	{
		CTempHandleLock_AC tempHandleLock((Handle)wDefProc);
		WindowDefProcPtr wdef = *wDefProc;

		bool rgnsWereBuilt = BuildWindowRegions(kBuild);// regions needed for hit testing 
		short variant = GetWVariant(fWMgrWindow);

		//
		// The static_casts are because SCpp (a certain ZorTech derived compiler) can't figure out how
		// to apply the proper coercion operator from CPoint_AC to long. They can be removed whenever
		// that compiler is feeling better. 4/1/96
		//
		result = (CallWindowDefProc(wdef, variant, fWMgrWindow, wHit, (whichRect[topLeft])) == wInDrag
		|| CallWindowDefProc(wdef, variant, fWMgrWindow, wHit, (whichRect[botRight])) == wInDrag
		|| CallWindowDefProc(wdef, variant, fWMgrWindow, wHit, CPoint_AC(whichRect.left, whichRect.bottom)) == wInDrag
		|| CallWindowDefProc(wdef, variant, fWMgrWindow, wHit, CPoint_AC(whichRect.right, whichRect.top)) == wInDrag);

		BuildWindowRegions(rgnsWereBuilt);		// restore state of window rgns
	}
#if qDebugMsg
	else
		::ProgramBreak_AC("###TWindow.RectInDrag: can't load wdef");
#endif

	return result;
}

//----------------------------------------------------------------------------------------
// TWindow::IsShown: 
//----------------------------------------------------------------------------------------
bool TWindow::IsShown() const						// override 
{
	return fWMgrWindow ? IsWindowVisible(fWMgrWindow) : false;
}

//----------------------------------------------------------------------------------------
// TWindow::IsActive: 
//----------------------------------------------------------------------------------------
bool TWindow::IsActive() const						// override 
{
	return fIsActive;
}

//----------------------------------------------------------------------------------------
// TWindow::DoInvalidateRegion: 
//----------------------------------------------------------------------------------------
void TWindow::DoInvalidateRegion(const RgnHandle badRgn)
{
	if ((qd.thePort == (GrafPtr)GetWindowPort(fWMgrWindow)) && IsShown() && !EmptyRgn(badRgn))
	{
		::InvalRgn(badRgn);
		if (gShowInvalidations)
			FillRgn(badRgn, &qd.ltGray);

		// Maintain the Focus
		if (fUpdating)						// clip should INCLUDE the update region
			UnionRgn(GetClipRegion(qd.thePort), badRgn, qd.thePort->clipRgn);
		else								// clip should EXCLUDE the update region
			DiffRgn(GetClipRegion(qd.thePort), badRgn, qd.thePort->clipRgn);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::DoValidateRegion: 
//----------------------------------------------------------------------------------------
void TWindow::DoValidateRegion(const RgnHandle goodRgn)
{
	if ((qd.thePort == (GrafPtr)GetWindowPort(fWMgrWindow)) && IsShown())
		if (!EmptyRgn(goodRgn))
		{
			ValidRgn(goodRgn);

			// Maintain the Focus
			if (fUpdating)						// clip should INCLUDE the update region and exclude valid bits
				DiffRgn(GetClipRegion(qd.thePort), goodRgn, qd.thePort->clipRgn);
			else								// clip should EXCLUDE the update region and include valid bits
				UnionRgn(GetClipRegion(qd.thePort), goodRgn, qd.thePort->clipRgn);
		}
}

//----------------------------------------------------------------------------------------
// TWindow::BuildWindowRegions: Calculate window size including structure region (i.e.
// title bar). To do this we need to force the window to compute its structure region by
// calling its defproc, if the window isn't shown. If build is false, set the regions back
// to empty regions, so as not to confuse the window manager. Return the previous state of
// the regions.
//----------------------------------------------------------------------------------------
bool TWindow::BuildWindowRegions(bool build) const
{
	bool returnVal = kBuild;

	WindowRecord & theWindowRecord = *((WindowPeek)fWMgrWindow);

	// The regions are considered to be built if either:
	//  a) the window is shown; or
	//  b) the structure rgn is not empty.

	if (IsShown() || !EmptyRgn(theWindowRecord.strucRgn))
	{
		if ((build != kBuild) && !IsShown())
		{
			SetEmptyRgn(theWindowRecord.strucRgn);
			SetEmptyRgn(theWindowRecord.contRgn);
		}
	}
	else
	{
		if (build == kBuild)
		{
			WDefProcTypeHandle wDefProc = (WDefProcTypeHandle)GetAndLoadWDefProc(theWindowRecord.windowDefProc);
			if (wDefProc)
			{
				{
					CTempHandleLock_AC tempHandleLock((Handle)wDefProc);
					WindowDefProcPtr wdef = *wDefProc;
					CallWindowDefProc(wdef, GetWVariant(fWMgrWindow), fWMgrWindow, wCalcRgns, 0);
				}

				// Calculate offset from top-left of window structure to top-left of window content
				const_cast<TWindow*>(this)->fContentRegionInset = ((CRect_AC &)(*theWindowRecord.contRgn)->rgnBBox)[topLeft] - ((CRect_AC &)(*theWindowRecord.strucRgn)->rgnBBox)[topLeft];

				CRect_AC globalStrucRect = (*theWindowRecord.strucRgn)->rgnBBox;
				CRect_AC globalContRect = (*theWindowRecord.contRgn)->rgnBBox;
				const_cast<TWindow*>(this)->fContentDifference = globalStrucRect.GetSize() - globalContRect.GetSize();
			}
		}
		returnVal = !kBuild;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TWindow::Select: 
//----------------------------------------------------------------------------------------
void TWindow::Select()
{
	MASelectToolboxWindow(fWMgrWindow);			// 3.5
}

//----------------------------------------------------------------------------------------
// TWindow::MoveByUser: 
//----------------------------------------------------------------------------------------
void TWindow::MoveByUser(const CViewPoint& theMouse)
{
	CViewRect oldFrame(GetFrame());

	MADragWindow(fWMgrWindow, LocalToSuperView(theMouse).ToPoint(), GetDynamicMoveBounds());

	// Don't forget to tell the window object 
	if (Focus())
		Locate(LocalToGlobal(((CRect_AC &)GetWindowPort(fWMgrWindow)->portRect)[topLeft]), kDontInvalidate);	// can't use localtosuper since fLocation not updated yet

	CViewRect newFrame(GetFrame());
	if (TOSADispatcher::fgDispatcher->GetDefaultTarget()->IsRecordingOn() && (newFrame != oldFrame))
	{
		TSetPropertyEvent theEvent(gServerAddress, kAENoReply + kAEDontExecute, this, pBounds);
		theEvent.WriteRect(keyAEData, newFrame.ToRect());
		CAutoPtr_AC<TAppleEvent> theReply(theEvent.Send());
	}
}

//----------------------------------------------------------------------------------------
// TWindow::Open: 
//----------------------------------------------------------------------------------------
void TWindow::Open()							// override 
{
	if (!IsShown())
	{
		// Keep us matching the window since we are parallel structures
		// Be sure all views are sized right
		Resize(((CRect_AC &)GetWindowPort(fWMgrWindow)->portRect).GetSize(), kDontInvalidate);

		if (fMustAdapt && !fAdapted)
			AdaptToScreen();
		if ((fMustHorzCenter && !fHorzCentered) || (fMustVertCenter && !fVertCentered))
			Center(fMustHorzCenter, fMustVertCenter, IsModal());

		if (fMustStagger && !fStaggered)
		{
			// If both staggering and forcing on screen are specified then we must ensure
			// that the window is forced on screen FIRST so that the staggering will occur
			// in a visible (and usable) location. BUT, after staggering we must STILL
			// ensure that the window is forced on screen so, we reset the forced flag so
			// that forcing will occur on schedule later.
			if (fMustForceOnScreen && !fForcedOnScreen)
			{
				ForceOnScreen();
				fForcedOnScreen = false;		// Make sure we can be forced again 
			}
			SimpleStagger(CPoint_AC(kStdStaggerAmount, kStdStaggerAmount), gStandardWindowStaggerCount);
		}
		if (fMustForceOnScreen && !fForcedOnScreen)
			ForceOnScreen();

		// If fVisibleExtent is empty, InvalidateCoordinates so our contents will be drawn (b/c of caching)
		if (GetVisibleExtent().Empty())
			InvalidateCoordinates();

		if (fTarget)
			fTarget->SetTargetSelection(kDontRedraw);
		Show(true, kRedraw);					// Make me visible 

	}
	TView::Open();							// Tell subviews to open in case they care 
}

//----------------------------------------------------------------------------------------
// TWindow::PoseModally: 
//----------------------------------------------------------------------------------------
IDType TWindow::PoseModally()
{
	TDialogBehavior * itsDialogBehavior = GetDialogBehavior();
	IDType result = kNoIdentifier;

	if (itsDialogBehavior)
	{
		bool wasModal = itsDialogBehavior->fModal;

		if (!wasModal)
			SetModality(true);

		try
		{
			Open();
			Select();							// Bring it to the front 

			itsDialogBehavior->PoseModally();
			result = itsDialogBehavior->fDismisser;
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			Close();
			if (fFreeOnClosing)
				delete this;

			throw;
		}
		
		
		if (!wasModal)
			SetModality(wasModal);
	}
	else
	{
		Close();
		if (fFreeOnClosing)
			delete this;
#if qDebug
		::ProgramBreak_AC("Can't PoseModally because the window doesn't contain a TDialogBehavior.");
#endif

		gErrorParm3 = "TDialogBehavior";		// show name of class 
		::ThrowException_AC(errMissingClass, 0);
	}

	return result;
}

//----------------------------------------------------------------------------------------
// TWindow::IsModal: 
//----------------------------------------------------------------------------------------
bool TWindow::IsModal() const
{
	TDialogBehavior * theDialogBehavior = GetDialogBehavior();

	return theDialogBehavior ? theDialogBehavior->fModal : false;
}

//----------------------------------------------------------------------------------------
// TWindow::IsInModalState: 
//----------------------------------------------------------------------------------------
bool TWindow::IsInModalState() const
{
	TDialogBehavior * theDialogBehavior = GetDialogBehavior();

	return theDialogBehavior ? (theDialogBehavior->fModal) && (!theDialogBehavior->fDismissed) : false;
}

//----------------------------------------------------------------------------------------
// TWindow::AddedASubView: 
//----------------------------------------------------------------------------------------
void TWindow::AddedASubView(TView* theSubView)
{
#if qDrag
	// Check this view and all of its subviews to see if any of them
	// are droppable.
	RegisterIfDroppableView(theSubView);
#endif
}

//----------------------------------------------------------------------------------------
// TWindow::RemovedASubView: 
//----------------------------------------------------------------------------------------
void TWindow::RemovedASubView(TView* theSubView)
{
	// if theSubView (or any subView) is the target, we need to 
	// set the target to this window. It might be fatal for the target to
	// point to a view no longer in the window.

	TEventHandler * handler;
	for (handler = GetTarget(); handler; handler = handler->GetNextHandler())
		if (theSubView == handler)
		{
			SetTarget(this);
			break;
		}

	for (handler = fTarget; handler; handler = handler->GetNextHandler())
		if (theSubView == handler)
		{
			fTarget = this;
			break;
		}

#if qDrag
	// If the view being removed is droppable then we need to unregister it.
	if (theSubView->GetDroppable())
		UnregisterIfDroppableView(theSubView);
#endif
}

//----------------------------------------------------------------------------------------
// TWindow::SetFrame: 
//----------------------------------------------------------------------------------------
void TWindow::SetFrame(const CViewRect& newFrame,
					   bool invalidate)		// override 
{
	if (fWMgrWindow)
	{
		CRect_AC newQDFrame(newFrame.ToRect());
		if (!(newFrame.GetSize() < fSize))
		{
			if (newFrame[topLeft] != fLocation)
				MoveWindow(fWMgrWindow, newQDFrame[topLeft].h, newQDFrame[topLeft].v, false);

			if (newFrame.GetSize() != fSize)
#if qALOE
				CODWindow_AC::SetSize(fWMgrWindow, newQDFrame.GetSize(), invalidate);
#else
				SizeWindow(fWMgrWindow, newQDFrame.GetSize().h, newQDFrame.GetSize().v, invalidate);
#endif
		}
		else
		{
			if (newFrame.GetSize() != fSize)
#if qALOE
				CODWindow_AC::SetSize(fWMgrWindow, newQDFrame.GetSize(), invalidate);
#else
				SizeWindow(fWMgrWindow, newQDFrame.GetSize().h, newQDFrame.GetSize().v, invalidate);
#endif
			if (newFrame[topLeft] != fLocation)
				MoveWindow(fWMgrWindow, newQDFrame[topLeft].h, newQDFrame[topLeft].v, false);
		}
	}

	CViewPoint oldSize = fSize;
	CViewRect vr(-kSBarSizeMinus1, -kSBarSizeMinus1, 0, 0);// standard size of grow box 

	if (fIsResizable && invalidate && newFrame.GetSize() != oldSize)
		Invalidate(vr + fSize);			// old location

	TView::SetFrame(newFrame, invalidate);

	if (fIsResizable && invalidate && fSize != oldSize)
		Invalidate(vr + fSize);			// new location
}

//----------------------------------------------------------------------------------------
// TWindow::ResizeByUser: 
//----------------------------------------------------------------------------------------
void TWindow::ResizeByUser(const CViewPoint& theMouse)
{
	if (fIsResizable)
	{
		long growResult = GrowWindow(fWMgrWindow, LocalToSuperView(theMouse).ToPoint(), &GetDynamicResizeLimits());
		if (growResult)
			Resize(CViewPoint(LoWord(growResult), HiWord(growResult)), kInvalidate);

		if (growResult && TOSADispatcher::fgDispatcher->GetDefaultTarget()->IsRecordingOn())
		{
			TSetPropertyEvent theEvent(gServerAddress, kAENoReply + kAEDontExecute, this, pBounds);
			theEvent.WriteRect(keyAEData, GetFrame().ToRect());
			CAutoPtr_AC<TAppleEvent> theReply(theEvent.Send());
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow::SetResizeLimits: 
//----------------------------------------------------------------------------------------
void TWindow::SetResizeLimits(CPoint_AC itsMinSize,
							  CPoint_AC itsMaxSize)
{
	fResizeLimits.Set(itsMinSize, itsMaxSize);

	CRect_AC resizeLimits(GetDynamicResizeLimits());		// get the dynamic values

	// If the window is zoomable, keep the window's state data current 
	if (GetWindowZoomFlag(fWMgrWindow))
	{
		CRect_AC theStdState;
		GetWindowStandardState(fWMgrWindow, &theStdState);
		theStdState.right = Min_AC(theStdState.right, static_cast<short>(fLocation.h + resizeLimits.right - 1));
		theStdState.bottom = Min_AC(theStdState.bottom, static_cast<short>(fLocation.v + resizeLimits.bottom - 1));
		SetWindowStandardState(fWMgrWindow, &theStdState);
	}

	// If the window is not sized within the new limits, resize it to fit
	CViewPoint newSize(fSize);
	newSize.ConstrainTo(resizeLimits);

	Resize(newSize, IsVisible());
}

//----------------------------------------------------------------------------------------
// TWindow::DoEvent: 
//----------------------------------------------------------------------------------------
void TWindow::DoEvent(EventNumber eventNumber,
					  TEventHandler* source,
					  TEvent* event)			// Override

{
	switch (eventNumber)
	{
		case mBecameTarget:
			if (IsActive())
				SetWindowTarget(source);
			break;

		default:
			TView::DoEvent(eventNumber, source, event);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::SetWindowTarget: 
//----------------------------------------------------------------------------------------
void TWindow::SetWindowTarget(TEventHandler* newTarget)
{
	if (!newTarget)								// Would be annoying if a NULL got in here! 
		newTarget = this;
	if (newTarget != fTarget)
	{
		fTarget->ResignedWindowTarget();
		fTarget = newTarget;
		newTarget->BecameWindowTarget();
	}
}

//----------------------------------------------------------------------------------------
// TWindow::SetTitle: 
//----------------------------------------------------------------------------------------
void TWindow::SetTitle(const CStr255_AC& newTitle)
{
	CStr255_AC oldTitle;

	GetWTitle(fWMgrWindow, oldTitle);			// to minimize flash, we only set the
												// title if it's different from the
												// current title
	if (oldTitle != newTitle)
		::SetWTitle(fWMgrWindow, newTitle);
}

//----------------------------------------------------------------------------------------
// TWindow::SetTitleForDoc: 
//----------------------------------------------------------------------------------------
void TWindow::SetTitleForDoc(const CStr255_AC& newDocTitle)
{
	if (fPreDocname > 0)						// optimize for case of nothing to do 
	{
		CStr255_AC title(GetTitle());

		if (SubstituteInTitle(title, newDocTitle, fPreDocname, fConstTitle))
			SetTitle(title);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::AboutToLoseControl: 
//----------------------------------------------------------------------------------------
void TWindow::AboutToLoseControl()
{
	if (IsShown() && IsHiddenOnSuspend())
	{
		fWasHiddenOnSuspend = true;
		bool oldManageActivation = fGenerateActivates;
		fGenerateActivates = false;
		Show(false, kRedraw);
		fGenerateActivates = oldManageActivation;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::RegainControl: 
//----------------------------------------------------------------------------------------
void TWindow::RegainControl()
{
	if (fWasHiddenOnSuspend)
	{
		fWasHiddenOnSuspend = false;
		// Any necessary activation is handled in TApplication.HandleSystemEvent 
		bool oldManageActivation = fGenerateActivates;
		fGenerateActivates = false;
		Show(true, kRedraw);
		fGenerateActivates = oldManageActivation;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::Show: 
//----------------------------------------------------------------------------------------
void TWindow::Show(bool state,
				   bool redraw)				// override 
{
	if (state)
	{
		// Be sure all views are sized right 
		Resize(((CRect_AC &)GetWindowPort(fWMgrWindow)->portRect).GetSize(), redraw);
	}

	if (fGenerateActivates && !fFloats)
	{
		if (state)
			MAShowWindow(fWMgrWindow);
		else
			MAHideWindow(fWMgrWindow);
	}
	else
	{
		if (fFloats)
			HiliteWindow(fWMgrWindow, state);

		ShowHide(fWMgrWindow, state);
		if (!state)
			Activate(false);					// at least de-activate ourselves when
		// hiding
	}

	// manage floaters window list position
	if (state && fFloats)
	{
		// if we are behind any of the normal document windows, we move
		// back into the floating layer
		bool foundNormalDocWindow = false;
		bool foundOurself = false;
		bool needToMove = false;

		for (CWMgrIterator iter; iter; ++iter)
			if (iter.Current() == fWMgrWindow)
			{
				foundOurself = true;
				needToMove = foundNormalDocWindow;
			}
			else if (!IsModalWindow(iter) && !IsVisibleFloatWindow(iter))
				foundNormalDocWindow = true;

		if (needToMove)
			BringToFront(fWMgrWindow);
	}

	// manage activation:
	if (fFloats)
		Activate(state);						// for floaters, activation == visibility 
	else if ((!state) && IsActive())
		Activate(false);						// if active, de-activate ourselves when hiding

	TView::Show(state, redraw);
}

//----------------------------------------------------------------------------------------
// TWindow::SimpleStagger: 
//----------------------------------------------------------------------------------------
void TWindow::SimpleStagger(CPoint_AC delta,
							short& counter)
{
	fStaggered = true;
	CViewRect theFrame(GetFrame());

	theFrame[botRight] = (theFrame[topLeft] + CViewPoint(GetDynamicMoveBounds()[botRight])) - theFrame[botRight];

	// This covers the case of delta.h && delta.v both >= 0; if either is less than 0, the
	// right (bottom) limit is set to the left (top) edge of the screen -- allowing for
	// some margin. This makes the right (bottom) edge less than the left (top) edge, but
	// this is OK since the/ statement below will be dividing 2 negative numbers.

	if (delta.h < 0)
		theFrame.right = fMoveBounds.left;
	if (delta.v < 0)
		theFrame.bottom = fMoveBounds.top;

	// This code avoids divide by zero problems 
	long nSlots = (!delta.h || !delta.v) ? 0 : Min_AC((theFrame.GetLength(hSel) + delta.h - 1) / delta.h, (theFrame.GetLength(vSel) + delta.v - 1) / delta.v);
	long slot = nSlots ? counter % nSlots : 0;

	if (slot)									// move the window 
	{
		// The place to position the window 
		Locate(theFrame[topLeft] + CViewPoint((slot * delta.h), (slot * delta.v)), kDontInvalidate);
	}

	++counter;
}

//----------------------------------------------------------------------------------------
// TWindow::BeginUpdate: 
//----------------------------------------------------------------------------------------
void TWindow::BeginUpdate()
{
	fUpdating = true;							// So TWindow.Focus will know to focus TO
	// the update rgn

	//
	// Set the flags that let the GetVisibleRegion and GetUpdateRegion functions
	// return the correct regions.
	//
	pInBeginUpdate = true;
	pInBeginUpdateWindowPtr = fWMgrWindow;

	//
	// Save the update region for reference, since ::BeginUpdate smashes it
	//
	if (!pSavedUpdateRegion)
		pSavedUpdateRegion = NewRgn();
	CopyRgn(((WindowPeek)fWMgrWindow)->updateRgn, pSavedUpdateRegion);

	// Yes, I know that there is an LMGetSaveVisRgn call available. The problem is that the Process Manager
	// doesn't [always?] switch the low memory global that stores the saved visRgn and that makes it
	// d*mn near impossible to debug-step through any framework updating code as the region
	// referenced there keeps getting modified. Copying the region to pSavedVisRegion is somewhat
	// less efficient... but actually works correctly.
	if (!pSavedVisRegion)
		pSavedVisRegion = NewRgn();
	CopyRgn(fWMgrWindow->visRgn, pSavedVisRegion);

	//
	// Save the offset of the visRgn since we'll need it in the GetVisRegion accessor function
	//
	RgnHandle savedVisRgn = pSavedVisRegion;
	pSavedVisRegionOriginalOffset = ((CRect_AC &)(*savedVisRgn)->rgnBBox)[topLeft] - ((CRect_AC &)fWMgrWindow->portRect)[topLeft];

	::BeginUpdate(fWMgrWindow);
}

//----------------------------------------------------------------------------------------
// TWindow::EndUpdate: 
//----------------------------------------------------------------------------------------
void TWindow::EndUpdate()
{
	fUpdating = false;

	pInBeginUpdate = false;
	pInBeginUpdateWindowPtr = NULL;
	SetEmptyRgn(pSavedUpdateRegion);

	SetEmptyRgn(pSavedVisRegion);

	::EndUpdate(fWMgrWindow);

}

//----------------------------------------------------------------------------------------
// TWindow::Update: 
//----------------------------------------------------------------------------------------
void TWindow::Update()							// override 
{
	// if we are already in the process of updating, don't do anything.
	if (!fUpdating)
	{
		if (HasPendingUpdate())
		{
			try
			{
				InvalidateFocus();
				BeginUpdate();
#if qDebug
				if (gShowInvalidations && Focus())
				{
					FillRect(&GetQDExtent(), &qd.ltGray);// we'll be clipped appropriately
					long finalTicks;
					Delay(15, &finalTicks);		// so you can see it
				}
#endif

#if 1	// Experimental: Fix up fLocation so that SuperViewToLocal can't fail when dynamically 
		// changing screen resolution.
				if (Focus()) ;
				CPoint_AC windowLocation(LocalToGlobal(((CRect_AC &)GetWindowPort(fWMgrWindow)->portRect)[topLeft]));
				if (fLocation != windowLocation)
				{
#if qDebug
					ProgramBreak_AC("Window appears to have moved, captain, by a change to a smaller screen resolution.");
#endif
					fLocation = windowLocation;
				}
#endif // Experimental end.
		
				HandleDraw(SuperViewToLocal((*GetUpdateRegion(fWMgrWindow))->rgnBBox));
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);

				// Or we get into an infinite loop 
				// trying to update the window when 
				// displaying an alert 
				EndUpdate();
				InvalidateFocus();

				throw;
			}
			

			EndUpdate();
			InvalidateFocus();
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow::GetStandardStateFrame:  Former nested routine, now a private method
//----------------------------------------------------------------------------------------
CViewRect TWindow::GetStandardStateFrame(const CViewRect& boundingRect) const
{
	CViewRect stdFrame;

	const short edge = 2;						// leave space around window structure area, settable?
	CViewRect localBoundingRect(boundingRect);

	localBoundingRect.Inset(edge, edge);

	// calculate the maximum structure size 
	CRect_AC resizeLimits(GetDynamicResizeLimits());

	CViewPoint delta;
	delta.h = Min_AC((localBoundingRect.GetLength(hSel)), resizeLimits[botRight][hSel] + fContentDifference[hSel]);//!!! long-> short
	delta.v = Min_AC((localBoundingRect.GetLength(vSel)), resizeLimits[botRight][vSel] + fContentDifference[vSel]);//!!! long-> short

	// Relocate the view only if necessary, and then move it as little as possible
	if ((fLocation.v >= localBoundingRect.top) && ((fLocation.v + delta.v - fContentDifference.v - 1) <= localBoundingRect.bottom))
		stdFrame.top = fLocation.v;
	else if (fLocation.v < localBoundingRect.top)
		stdFrame.top = localBoundingRect.top;
	else
		stdFrame.top = localBoundingRect.bottom - delta.v + fContentRegionInset.v;
	// stdFrame.top = localBoundingRect.top + fContentRegionInset.v + ((localBoundingRect.GetLength(vSel)) - delta.v) / 2;

	if ((fLocation.h >= localBoundingRect.left) && ((fLocation.h + delta.h - fContentDifference.h - 1) <= localBoundingRect.right))
		stdFrame.left = fLocation.h;
	else if (fLocation.h < localBoundingRect.left)
		stdFrame.left = localBoundingRect.left;
	else
		stdFrame.left = localBoundingRect.right - delta.h + fContentRegionInset.h;
	// stdFrame.left = localBoundingRect.left + fContentRegionInset.h + ((localBoundingRect.GetLength(hSel)) - delta.h) / 2;

	stdFrame[botRight] = stdFrame[topLeft] + delta - CViewPoint(fContentDifference) - CViewPoint(1, 1);

	return stdFrame;
}

//----------------------------------------------------------------------------------------
// TWindow::GetUserStateFrame: 
//----------------------------------------------------------------------------------------
CViewRect TWindow::GetUserStateFrame(const CViewRect&	/* boundingRect */) const
{
	CViewRect userFrame;

	if (GetWindowZoomFlag(fWMgrWindow))			// has zoom box
	{
		CRect_AC userQDState;
		GetWindowUserState(fWMgrWindow, &userQDState);
		userFrame = userQDState;
	}
#if qDebug
	else
		::ProgramBreak_AC("Whoops");
#endif

	return userFrame;
}
//----------------------------------------------------------------------------------------
// TWindow::CountAsOpen: 
//----------------------------------------------------------------------------------------
bool TWindow::CountAsOpen()
{
	// Override this if you prefer some other indicator of whether a window should
	// be considered as being open.
	return IsShown() && !fFloats;
}

//----------------------------------------------------------------------------------------
// TWindow::Zoom: 
//----------------------------------------------------------------------------------------
void TWindow::Zoom(short partCode)
{
	CRect_AC aGDScreenRect;
	CViewRect itsNewFrame;

	// Find out where the window would like to be zoomed out to 
	GetMaxIntersectedDevice(aGDScreenRect);
	CViewRect aGDScreenVRect(aGDScreenRect);

	if (partCode == inZoomOut)
	{
		// new zoom-out CRect_AC
		itsNewFrame = GetStandardStateFrame(aGDScreenVRect);
		if (GetWindowZoomFlag(fWMgrWindow))		// has zoom box
		{
			CRect_AC newFrame = itsNewFrame.ToRect();
			SetWindowStandardState(fWMgrWindow, &newFrame);
		}
	}
	else
	{											// new zoom-in CRect_AC
		itsNewFrame = GetUserStateFrame(aGDScreenVRect);
		if (GetWindowZoomFlag(fWMgrWindow))		// has zoom box
		{
			CRect_AC newFrame = itsNewFrame.ToRect();
			SetWindowUserState(fWMgrWindow, &newFrame);
		}
	}

	if (Focus())								// The ROM requires that thePort be the window being zoomed.
	{
		// if the window both moves and resizes then call the ZoomWindow trap to get the right visual
		// effect.
		// If it only moves OR resizes then just call SetFrame since more bits
		// may be preserved.
		if (itsNewFrame[topLeft] != fLocation && itsNewFrame.GetSize() != fSize)
		{
			EraseRect(&GetQDExtent());

			if (GetWindowZoomFlag(fWMgrWindow))	// check for a zoomable defproc 
				ZoomWindow(fWMgrWindow, partCode, false);
		}

		// let the View know what we've done 
		SetFrame(itsNewFrame, kInvalidate);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::ZoomByUser: 
//----------------------------------------------------------------------------------------
void TWindow::ZoomByUser(const CViewPoint& theMouse,
						 short partCode)
{
	if (TrackBox(fWMgrWindow, LocalToSuperView(theMouse).ToPoint(), partCode))
	{
		Zoom(partCode);

		if (TOSADispatcher::fgDispatcher->GetDefaultTarget()->IsRecordingOn())
		{
			TSetPropertyEvent theEvent(gServerAddress, kAENoReply + kAEDontExecute, this, pIsZoomed);
			theEvent.WriteBoolean(keyAEData, (partCode == inZoomOut));
			CAutoPtr_AC<TAppleEvent> theReply(theEvent.Send());
		}
	}
}

//----------------------------------------------------------------------------------------
// TWindow::GetDynamicMoveBounds:
//----------------------------------------------------------------------------------------
CRect_AC TWindow::GetDynamicMoveBounds() const
{
	//
	// Any value is made dynamic by leaving it at zero. Any non-zero values
	// are honored.
	//
	CRect_AC moveBounds(fMoveBounds);

	CRect_AC grayBox((*GetGrayRgn())->rgnBBox);
	grayBox.Inset(4, 4);

	if (!fMoveBounds.top)						// dynamic
		moveBounds.top = grayBox.top;

	if (!fMoveBounds.left)						// dynamic
		moveBounds.left = grayBox.left;

	if (!fMoveBounds.bottom)					// dynamic
		moveBounds.bottom = grayBox.bottom;

	if (!fMoveBounds.right)						// dynamic
		moveBounds.right = grayBox.right;

	return moveBounds;
}

//----------------------------------------------------------------------------------------
// TWindow::GetDynamicResizeLimits:
//----------------------------------------------------------------------------------------
CRect_AC TWindow::GetDynamicResizeLimits() const
{
	//
	// Any value in fResizeLimits is made dynamic by leaving it at zero. Any
	// non-zero values are honored.
	//
	CRect_AC resizeLimits = fResizeLimits;

	// maximum size vertically
	static const short kHalfTitleBar = 8;
	CPoint_AC grayRgnBBoxSize(CRect_AC((*GetGrayRgn())->rgnBBox).GetSize());

	if (!fResizeLimits.bottom)					// dynamic
		resizeLimits.bottom = grayRgnBBoxSize.v - kHalfTitleBar;

	// maximum size horizontally
	if (!fResizeLimits.right)					// dynamic
		resizeLimits.right = grayRgnBBoxSize.h;

	//
	// Ideally the window's settings should come from the window template and
	// the default settings should come from a resource (with a constant for a
	// backstop if the resource is not present). However, the template format
	// is not changing (for MA 3.3) and I'd have to check with the MacApp team
	// about defining a resource for the default.
	//
	// I am able to fix one persistent problem here, though. User interface
	// elements have been shrinking over the past several years and very small
	// status windows have become more common. MacApp has supplied a default
	// minimum resize limit of 80, 80 pixels in previous releases. This is too
	// large for these small windows and yet is still close to reasonable. I
	// propose to reduce the constant minimum to a slightly smaller value than
	// before (approximately two scroll arrows plus a thumb plus a resize icon)
	// as well as modifying the calculation of the default value to accept the
	// current size of the window as the minimum if it is smaller than the
	// constant values.
	//
	static const short kMinWindowWidth = kStdSzSBar * 4;
	static const short kMinWindowHeight = kStdSzSBar * 4;

	// minimum size vertically
	if (!fResizeLimits.top)						// dynamic
	{
		resizeLimits.top = Min_AC(kMinWindowHeight, resizeLimits.bottom);
		// capture the minimum if it will be smaller than the default
		if (fSize.v < resizeLimits.top)
			resizeLimits.top = const_cast<TWindow*>(this)->fResizeLimits.top = fSize.v;
	}

	// minimum size horizontally
	if (!fResizeLimits.left)					// dynamic
	{
		resizeLimits.left = Min_AC(kMinWindowWidth, resizeLimits.right);
		// capture the minimum if it will be smaller than the default
		if (fSize.v < resizeLimits.left)
			resizeLimits.left = const_cast<TWindow*>(this)->fResizeLimits.left = fSize.h;
	}
	
	return resizeLimits;
}

//----------------------------------------------------------------------------------------
// TWindow::GetSpecifierForm:
//----------------------------------------------------------------------------------------
DescType TWindow::GetSpecifierForm()
{
	return (!GetTitle().IsEmpty()) ? formName : MScriptableObject::GetSpecifierForm();
}

//----------------------------------------------------------------------------------------
// TWindow::CountContainedObjects:
//----------------------------------------------------------------------------------------
long TWindow::CountContainedObjects(DescType desiredType)
{
	long result = 0;

	if (IsShown() && !fFloats)					// don't support floaters and invisible windows 
	{
		if (gDispatcher->IsDocumentClass(desiredType))
			result = 1;
		else if (fDocument)
			result = fDocument->CountContainedObjects(desiredType);
	}
	else

		if (!result)
			result = MScriptableObject::CountContainedObjects(desiredType);

	return result;
}

//----------------------------------------------------------------------------------------
// TWindow::GetCommandContext
//----------------------------------------------------------------------------------------
TCommandHandler* TWindow::GetCommandContext(const CommandNumber/* aCommandNumber */ ) const
{
	return fDocument ? dynamic_cast<TCommandHandler*>(fDocument) : gDispatcher;
}

//----------------------------------------------------------------------------------------
// TWindow::GetContainedObject:
//----------------------------------------------------------------------------------------
MScriptableObject* TWindow::GetContainedObject(DescType desiredType,
											   DescType selectionForm,
											   const CAEDesc& selectionData)
{
	MScriptableObject * result = NULL;
	if (IsShown() && !fFloats)
	{
		if (gDispatcher->IsDocumentClass(desiredType))
			result = fDocument;
		else if (fDocument && (desiredType != cProperty) && (desiredType != GetOMClass()))
			result = fDocument->GetContainedObject(desiredType, selectionForm, selectionData);
	}

	if (!result)
		result = MScriptableObject::GetContainedObject(desiredType, selectionForm, selectionData);

	return result;
}

//----------------------------------------------------------------------------------------
// TWindow::GetObjectProperty:
//----------------------------------------------------------------------------------------
bool TWindow::GetObjectProperty(CAEDesc& thePropertyValue,
								   DescType whichProperty,
								   const CAEDesc& desiredType)
{
	bool hasProperty = true;

	try
	{
		switch (whichProperty)
		{
			case pClass:
				thePropertyValue.PutType(GetOMClass());
				break;

			case pBounds:
				thePropertyValue.PutRect(ViewToQD(GetFrame()));
				break;

			case pName:
				thePropertyValue.PutString(GetTitle());
				break;

			case pHasCloseBox:
				thePropertyValue.PutBoolean(fIsClosable);
				break;

			case pHasTitleBar:
				thePropertyValue.PutBoolean(fContentDifference != gZeroPt);
				break;

			case pIndex:
				{
					long index = 0;
					hasProperty = false;

					for (CWMgrIterator iter; iter; ++iter)
					{
						TWindow* aWindow = WMgrToWindow(iter);
						// count its index either as a floater or non-floater
						if (aWindow && aWindow->IsShown() && (fFloats == aWindow->fFloats))
						{
							++index;
							if (aWindow == this)
							{
								thePropertyValue.PutLong(index);
								hasProperty = true;
								break;
							}
						}
					}
					break;
				}

			case pIsModal:
				thePropertyValue.PutBoolean(IsModal());
				break;

			case pIsResizable:
				thePropertyValue.PutBoolean(fIsResizable);
				break;

			case pIsZoomable:
				thePropertyValue.PutBoolean((fProcID) & zoomDocProc);
				break;

			case pIsZoomed:
				{
					CRect_AC aGDScreenRect;
					GetMaxIntersectedDevice(aGDScreenRect);
					CViewRect aGDScreenVRect(aGDScreenRect);
					CViewRect zoomFrame(GetUserStateFrame(aGDScreenVRect));
					thePropertyValue.PutBoolean((GetFrame() != zoomFrame));
					break;
				}

			case pSelection:
				{
					TDocument * theDoc = fDocument;
					if (theDoc)
					{
						TDesignator * theUserSelection = theDoc->GetUserSelection();
						if (theUserSelection)
						{
							CTempDesc propData;
							theUserSelection->MakeObjectSpecifier(propData, theUserSelection->GetSpecifierForm());
							thePropertyValue = propData;
						}
						else
							::ThrowIfOSErr_AC(errAENoSuchObject);
					}
					break;
				}

			default:
				{
					// Users will frequently refer to the window when they mean the document. If this window
					// has a document we'll give it a crack at supplying the property.
					if (fDocument)
						hasProperty = fDocument->GetObjectProperty(thePropertyValue, whichProperty, desiredType);
					else
						hasProperty = MScriptableObject::GetObjectProperty(thePropertyValue, whichProperty, desiredType);
				}
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		hasProperty = false;
	}
	

	return hasProperty;
}

//----------------------------------------------------------------------------------------
// TWindow::GetSetPropertyInfo:
//----------------------------------------------------------------------------------------
void TWindow::GetSetPropertyInfo(DescType whichProperty,
								 CommandNumber& cmdNum,
								 bool& canUndo,
								 bool& causesChange,
								 TCommandHandler*& theContext)
{
	MScriptableObject::GetSetPropertyInfo(whichProperty, cmdNum, canUndo, causesChange, theContext);

	if (whichProperty == pBounds || whichProperty == pIsZoomed || whichProperty == pName)
	{
		canUndo = false;
		causesChange = false;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::SetObjectProperty:
//----------------------------------------------------------------------------------------
void TWindow::SetObjectProperty(const CAEDesc& thePropertyValue,
								DescType whichProperty)
{
	switch (whichProperty)
	{
		case pBounds:
			{
				CRect_AC theWindowRect;
				thePropertyValue.GetRect(theWindowRect);
				SetFrame(CViewRect(theWindowRect), true);
			}
			break;

		case pName:
			{
				CStr255_AC theWindowName;
				thePropertyValue.GetString(theWindowName);
				SetTitle(theWindowName);
			}
			break;

		case pIsZoomed:
			{
				CRect_AC aGDScreenRect;
				GetMaxIntersectedDevice(aGDScreenRect);
				CViewRect aGDScreenVRect(aGDScreenRect);
				CViewRect zoomFrame(GetUserStateFrame(aGDScreenVRect));
				bool isZoomed = (GetFrame() != zoomFrame);
				bool shouldZoom = thePropertyValue.GetBoolean();
				short partCode = 0;
				if (shouldZoom && !isZoomed)
					partCode = inZoomOut;
				else if (!shouldZoom && isZoomed)
					partCode = inZoomIn;
				if (partCode)
					Zoom(partCode);
				break;
			}

		case pHasCloseBox:
		case pHasTitleBar:
		case pIsModal:
		case pIsResizable:
		case pIsZoomable:
			::ThrowIfOSErr_AC(errAECantSetReadOnly);
			break;

		default:
			// Users will frequently refer to the window when they mean the document. If this window
			// has a document we'll give it a crack at setting the property.
			if (fDocument)
				fDocument->SetObjectProperty(thePropertyValue, whichProperty);
			else
				MScriptableObject::SetObjectProperty(thePropertyValue, whichProperty);
			break;
	}
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TWindow::HandleOSAEvent:
//----------------------------------------------------------------------------------------
bool TWindow::HandleOSAEvent(CommandNumber aCommandNumber,
								TAppleEvent* message,
								TAppleEvent* reply)
{
	// Overridden to also give the document's script a shot at the event
	return MScriptableObject::HandleOSAEvent(aCommandNumber, message, reply) || (fDocument && fDocument->HandleOSAEvent(aCommandNumber, message, reply));
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TWindow::DoAEMove:
//----------------------------------------------------------------------------------------
void TWindow::DoAEMove(TAppleEvent* message,
					   TAppleEvent* reply)
{
	CTempDesc theInsertionLoc, windowObj;
	CAEDesc insertionRec;
	DescType theType, position;
	Size paramSize;
	message->ReadParameter(keyAEInsertHere, typeInsertionLoc, theInsertionLoc);
	::ThrowIfOSErr_AC(AECoerceDesc(theInsertionLoc, typeAERecord, insertionRec));
	insertionRec.GetKeyDesc(keyAEObject, typeWildCard, windowObj);
	::ThrowIfOSErr_AC(AEGetKeyPtr(insertionRec, keyAEPosition, typeEnumeration, &theType, &position, sizeof(DescType), &paramSize));

	if (position == kAEBeginning)
		Select();								// don't support the other keys yet

	::ThrowIfOSErr_AC(AEDisposeDesc(insertionRec));

	// Return an object specifier for the window
	CTempDesc theWindowSpec;
	MakeObjectSpecifier(theWindowSpec, formName);
	reply->WriteParameter(keyAEResult, theWindowSpec);
}

//----------------------------------------------------------------------------------------
// TWindow::DoAEClose:
//----------------------------------------------------------------------------------------
void TWindow::DoAEClose(TAppleEvent* message,
						TAppleEvent* reply)
{
	// First see if a document is to be closed.
	if (fDocument && fClosesDocument)
	// Closing the document will close any associated windows.
		fDocument->DoAEClose(message, reply);
	else
		// Close the window if it wasn't attached to a document.
		DoClose(cClose, false);
}

//----------------------------------------------------------------------------------------
// TWindow::DoAEPrint
//----------------------------------------------------------------------------------------
void TWindow::DoAEPrint(TAppleEvent* message,
						TAppleEvent* reply)
{
	bool wasHandled = false;

	if (fScriptingPrintHandler)
		wasHandled = fScriptingPrintHandler->DoScriptCommand(cPrint, message, reply);

	if (!wasHandled)
	{
		if (fDocument)
			fDocument->HandleScriptCommand(cPrint, message, reply);
		else
			MScriptableObject::DoAEPrint(message, reply);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::DoScriptCommand
//----------------------------------------------------------------------------------------
void TWindow::DoScriptCommand(CommandNumber aCommandNumber,
							  TAppleEvent* message,
							  TAppleEvent* reply)
{
	switch (aCommandNumber)
	{
		case cAEDelete:
			DoAEClose(message, reply);
			break;

		case cAEOpen:
		case cAESave:
		case cAECountElements:
		case cAECreateElement:
		case cAEDoObjectsExist:
			if (fDocument)
			{
				fDocument->HandleScriptCommand(aCommandNumber, message, reply);
				break;
			}
		// no break - drop thru!

		default:
			MScriptableObject::DoScriptCommand(aCommandNumber, message, reply);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::SetScriptingPrintHandler
//----------------------------------------------------------------------------------------
void TWindow::SetScriptingPrintHandler(TPrintHandler* printHandler)
{
	fScriptingPrintHandler = printHandler;
}

//----------------------------------------------------------------------------------------
// TWindow::ReleaseScriptingPrintHandler
//----------------------------------------------------------------------------------------
void TWindow::ReleaseScriptingPrintHandler(TPrintHandler* printHandler)
{
	if (fScriptingPrintHandler == printHandler)
		fScriptingPrintHandler = NULL;
}

#if qDrag

//----------------------------------------------------------------------------------------
// Drag Support
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TWindow::RegisterDroppableView
//----------------------------------------------------------------------------------------
void TWindow::RegisterDroppableView(TView* theView)
{

#if qDebug
	if (!HasDragManager_AC())						// should never get called if no Drag Manager
	{
		::ProgramBreak_AC("A TDragDropBehavior registered but the Drag Manager isn't present.");
		return;
	}
#endif

#if qDebug
	// 
	ArrayIndex_AC index = fDroppableViewList.GetIdentityItemNo(theView);
	if (index)
	{
		// ::ProgramBreak_AC("### A drag/drop view tried to register itself twice");
		return;
	}
#endif

	// Insert the view into the list if it's not already there.
	if (fDroppableViewList.GetEqualItemNo(&theView) == kEmptyIndex_AC)
		fDroppableViewList.InsertElementInOrder(&theView);	// Add the view to the list

	// Register this window with the global drag session if appropriate
	if (fDragCallbackStatus == eNotInstalled)
	{
		TDragDropSession::fgDragDropSession->RegisterDroppableWindow(this, fWMgrWindow);
		fDragCallbackStatus = eInstalled;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::RegisterIfDroppableView
//----------------------------------------------------------------------------------------
void TWindow::RegisterIfDroppableView(TView* theView)
{
	if (HasDragManager_AC())
	{
		// If the view being removed is droppable then we need to register it.
		if (theView->GetDroppable())
			RegisterDroppableView(theView);
	
		// Now we have to check to see if any of our subviews are droppable.
		for (CSubViewIterator iter(theView); iter; ++iter)
			RegisterIfDroppableView(iter);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::UnregisterDroppableView
//----------------------------------------------------------------------------------------
void TWindow::UnregisterDroppableView(TView* theView)
{
#if qDebug
	if (!HasDragManager_AC())						// should never get called if the Drag Mgr isn't present
	{
		::ProgramBreak_AC("###A drag/drop view unregistered but the Drag Manager is not present.");
		return;
	}
#endif

	fDroppableViewList.Delete(theView);

	// if the behavior count went to zero, unregister the window callbacks
	if (fDroppableViewList.IsEmpty())
	{
		TDragDropSession::fgDragDropSession->UnregisterDroppableWindow(fWMgrWindow);
		fDragCallbackStatus = eNotInstalled;
	}
}

//----------------------------------------------------------------------------------------
// TWindow::UnregisterIfDroppableView
//----------------------------------------------------------------------------------------
void TWindow::UnregisterIfDroppableView(TView* theView)
{
	if (HasDragManager_AC())
	{
		// If the view being removed is droppable then we need to unregister it.
		if (theView->GetDroppable())
			UnregisterDroppableView(theView);
	
		// Now we have to check to see if any of our subviews are droppable.
		for (CSubViewIterator iter(theView); iter; ++iter)
			UnregisterIfDroppableView(iter);
	}
}

//----------------------------------------------------------------------------------------
// TWindow::MouseToDropTarget
//----------------------------------------------------------------------------------------
TView* TWindow::MouseToDropTarget(CDragItemIterator& dragItemIterator,
								  const CPoint_AC& globalMouse,
								  CViewPoint& localMouse)
{
	TView * targetView = NULL;
	TView * aView = NULL;
	CViewPoint aPoint;
	ArrayIndex_AC maxIndex = fDropStatusCache.GetSize();
	short oldStatus,
	newStatus;

	CViewPoint pointInWindow = SuperViewToLocal(globalMouse);

	// Loop through the status list. The status list is a cache of values that
	// indicate whether or not a specific view can accept the drag.
	for (ArrayIndex_AC index = 1; index <= maxIndex; ++index)
	{
		fDropStatusCache.GetElementsAt(index, &oldStatus, 1);
		if (oldStatus != kCantReceiveDrop)
		{
			newStatus = oldStatus;
			fDroppableViewList.GetElementsAt(index, &aView, 1);
			if (newStatus == kDropStatusUnknown)
			{
				if (!aView->GetDroppable() || !aView->IsShown() || !aView->IsEnabled())
					newStatus = kCantReceiveDrop;
			}

			if (newStatus != kCantReceiveDrop)
			{
				aPoint = aView->RootViewToLocal(pointInWindow);
				if (aView->GetVisibleExtent().Contains(aPoint))
				{
					if (newStatus == kDropStatusUnknown)
						newStatus = aView->WillAcceptDrop(dragItemIterator) ? kCanReceiveDrop : kCantReceiveDrop;

					if ((newStatus == kCanReceiveDrop) && (!targetView || targetView->ContainsSubView(aView)))// we've got a winner!!
					{
						targetView = aView;
						localMouse = aPoint;
					}
				}

			}
			if (oldStatus != newStatus)
				fDropStatusCache.ReplaceElementsAt(index, &newStatus, 1);// cache the new status
		}
	}

	return targetView;
}

//----------------------------------------------------------------------------------------
// TWindow::DragEnteredWindow
//----------------------------------------------------------------------------------------
void TWindow::DragEnteredWindow(CDragItemIterator&/* dragItemIterator */  )
{
	short viewStatus = kDropStatusUnknown;
	ArrayIndex_AC viewListSize = fDroppableViewList.GetSize();

	fDropStatusCache.SetArraySize(viewListSize);

	// initialize the status list with kDropStatusUnknown
	for (ArrayIndex_AC i = 0; i < viewListSize; ++i)
		fDropStatusCache.InsertElementsBefore(i + 1, &viewStatus, 1);
}

//----------------------------------------------------------------------------------------
// TWindow::DragLeftWindow
//----------------------------------------------------------------------------------------
void TWindow::DragLeftWindow()
{
	fDropStatusCache.DeleteAll();
}

#endif // qDrag

//----------------------------------------------------------------------------------------
// GetVisRegion: 
//----------------------------------------------------------------------------------------
RgnHandle TWindow::GetVisRegion(GrafPtr theGrafPtr)
{
	RgnHandle returnVal;

	if (pInBeginUpdate && theGrafPtr == pInBeginUpdateWindowPtr)
	{
		returnVal = pSavedVisRegion;

		CPoint_AC currentOffset(((CRect_AC &)(*returnVal)->rgnBBox)[topLeft]);
		OffsetRgn(returnVal, -currentOffset.h + theGrafPtr->portRect.left + pSavedVisRegionOriginalOffset.h, -currentOffset.v + theGrafPtr->portRect.top + pSavedVisRegionOriginalOffset.v);
	}
	else
		returnVal = theGrafPtr->visRgn;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// GetUpdateRegion: 
//----------------------------------------------------------------------------------------
RgnHandle TWindow::GetUpdateRegion(WindowPtr theWindow)
{
	RgnHandle returnVal;

	if (TWindow::pInBeginUpdate && theWindow == pInBeginUpdateWindowPtr)
		returnVal = pSavedUpdateRegion;
	else
		returnVal = ((WindowPeek)theWindow)->updateRgn;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// GetAndLoadWDefProc: 
//----------------------------------------------------------------------------------------
Handle TWindow::GetAndLoadWDefProc(Handle windowDefProc)
{
	Handle returnVal = NULL;
	Handle wDefProc = (Handle)(StripLong(windowDefProc));// strip the variant code 

	if ((*wDefProc))							// if Master Ptr is NULL => resource is purged
		returnVal = wDefProc;
	else
	{
		LoadResource(wDefProc);
		if (ResError() == noErr)				// only return it if the LoadResource worked
			returnVal = wDefProc;
	}
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// HighlightAndActivateWindow: Common code for ActivateWindow and DeactivateWindow.  Does
// actual highlighting and calling of the activate handler.
//----------------------------------------------------------------------------------------
void TWindow::HighlightAndActivateWindow(WindowRef theWindow,
										 bool activate)
{
	if (theWindow)
	{
		if ((activate && !IsWindowHilited(theWindow)) || (!activate && IsWindowHilited(theWindow)))
			HiliteWindow(theWindow, activate);	// make the window "look" right 

		TWindow * theMAWindow = WMgrToWindow(theWindow);
		if (theMAWindow)
			theMAWindow->Activate(activate);
	}
}

//----------------------------------------------------------------------------------------
// IsGhostWindow: 
//----------------------------------------------------------------------------------------
bool TWindow::IsGhostWindow(WindowRef window)
{
	return window ? window == (WindowRef)LMGetGhostWindow() : false;
}


//----------------------------------------------------------------------------------------
// WMgrToWindow: 
//----------------------------------------------------------------------------------------
TWindow* TWindow::WMgrToWindow(WindowRef aWindowPtr)
{
	ArrayIndex_AC item = 0;
	if (aWindowPtr)
		item = fgWindows.GetEqualItemNo(aWindowPtr);

	return item ? fgWindows.At(item)->fTWindow : NULL;
}

//----------------------------------------------------------------------------------------
// DeleteWindow: 
//----------------------------------------------------------------------------------------
void TWindow::DeleteWindow(TWindow* aWindow)
{
	if (aWindow)
		fgWindows.Delete(aWindow->GetWindowRef());
}

//----------------------------------------------------------------------------------------
// RegisterWindow: 
//----------------------------------------------------------------------------------------
void TWindow::RegisterWindow(TWindow* aWindow)
{
	fgWindows.Insert(TWindowAndWindowRef(aWindow, aWindow->GetWindowRef()));
}

//----------------------------------------------------------------------------------------
// IsDocumentWindow: 
//----------------------------------------------------------------------------------------
bool TWindow::IsDocumentWindow(WindowRef aWindow)
// If a window is not a floater window, and is not a system window, and is not the special
// THINK environment window, then it is a document window.
{
	const short kTHINKWindowKind = 32700;
	short itsWindowKind = GetWindowKind(aWindow);

	return !IsFloatWindow(aWindow) && !IsSystemWindow(aWindow) && itsWindowKind < kTHINKWindowKind;
}

//----------------------------------------------------------------------------------------
// IsDialog: 
//----------------------------------------------------------------------------------------
bool TWindow::IsDialog(WindowRef window)
{
	bool returnVal = false;

	if (window)
	{
		short windowVariant = GetWVariant(window);
		returnVal = windowVariant == dBoxProc || windowVariant == plainDBox || windowVariant == altDBoxProc || windowVariant == movableDBoxProc;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// IsFloatWindow: 
//----------------------------------------------------------------------------------------
bool TWindow::IsFloatWindow(WindowRef window)
{
	bool returnVal = false;

	if (window)
	{
		TWindow * aMAWindow = WMgrToWindow(window);
		if (aMAWindow)
			returnVal = aMAWindow->fFloats;
		else
			returnVal = GetWindowKind(window) == kFloatingWindowKind;
#if qALOE
			returnVal |= CODWindow_AC::IsFloatingWindow(window);
#endif
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// IsVisibleFloatWindow: 
//----------------------------------------------------------------------------------------
bool TWindow::IsVisibleFloatWindow(WindowRef window)
{
	// A note on IsVisibleFloatWindow:
	// We only consider a window to be a floating window if it is visible.  
	// The motivation around the check for visibility is that, given the Toolbox
	// Window Manager's tendency to move visible windows to the head of WindowList,
	// there is no clean way for us to guarantee that hidden floaters will stay in
	// our floating window "layer".  Instead, the approach we take is to allow 
	// hidden floaters to be moved wherever in the window list, and we ensure that
	// they move back to the floating window layer when they are shown.  The check
	// for visibility ensures that functions that depend on the first or last
	// floating window (such as TWindow::GetBehindWindowPtr) perform as expected.
	bool returnVal = false;

	if (window && IsWindowVisible(window))
	{
		returnVal = IsFloatWindow(window);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// IsSystemWindow: 
//----------------------------------------------------------------------------------------
bool TWindow::IsSystemWindow(WindowRef window)
{
	bool returnVal = false;

	const short kWorldScriptWDEF = 125;
	if (window)
	{
		short theID;
		ResType theType;
		CStr255_AC theName;
		GetResInfo(((WindowPeek)window)->windowDefProc, &theID, &theType, theName);
		short windowKind = GetWindowKind(window);
		returnVal = (windowKind < 0) || (theID == kWorldScriptWDEF) || (theID == kBalloonWDEFID);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// IsModalWindow: 
//----------------------------------------------------------------------------------------
bool TWindow::IsModalWindow(WindowRef window)
{
	bool result = false;

	if (window)
	{
		TWindow * aMAWindow = WMgrToWindow(window);
		if (aMAWindow)
			result = aMAWindow->IsModal();
		else
		{
			// according to IM 4-9 (new version), these are
			// the possible window variants for modal dialogs:
			short windowVariant = GetWVariant(window);
			result = windowVariant == dBoxProc || windowVariant == plainDBox || windowVariant == altDBoxProc || windowVariant == movableDBoxProc;
		}
	}

	return result;
}

//----------------------------------------------------------------------------------------
// GetLastFloatingWindowPtr: 
//----------------------------------------------------------------------------------------
// returns the last floating window in the window list
WindowRef TWindow::GetLastFloatingWindowPtr()
{
	WindowRef lastFloatWindowPtr = NULL;

	for (CWMgrIterator iter; iter; ++iter)
		if (IsVisibleFloatWindow(iter))
			lastFloatWindowPtr = iter;

	return lastFloatWindowPtr;
}

//----------------------------------------------------------------------------------------
// GetFirstFloatingWindowPtr: 
//----------------------------------------------------------------------------------------
// returns the first floating window in the window list
WindowRef TWindow::GetFirstFloatingWindowPtr()
{
	WindowRef firstFloatWindowPtr = NULL;

	for (CWMgrIterator iter; iter; ++iter)
		if (IsVisibleFloatWindow(iter))
		{
			firstFloatWindowPtr = iter;
			break;
		}

	return firstFloatWindowPtr;
}

//----------------------------------------------------------------------------------------
// ActivateWindow: Activates the window by highlighting it, and calling its
// activate handler.
//----------------------------------------------------------------------------------------
void TWindow::MAActivateWindow(WindowRef theWindow)
{
	HighlightAndActivateWindow(theWindow, kActivateWindow);
}

//----------------------------------------------------------------------------------------
// DeactivateWindow: Deactivates the window by unhighlighting it and calling its
// activate handler.
//----------------------------------------------------------------------------------------
void TWindow::MADeactivateWindow(WindowRef theWindow)
{
	HighlightAndActivateWindow(theWindow, kDeactivateWindow);
}

//----------------------------------------------------------------------------------------
// MADragWindow: Our replacement for DrawWindow to handle floating windows
//----------------------------------------------------------------------------------------
void TWindow::MADragWindow(WindowRef windowToDrag,
						   CPoint_AC startPoint,
						   const CRect_AC& draggingBounds)
{
	CRect_AC dragRect;
	GrafPtr savePort;
	GrafPtr windowManagerPort;
	long dragResult;
	short topLimit;
	short newHorizontalWindowPosition;
	short newVerticalWindowPosition;
	short horizontalOffset;
	short verticalOffset;
	bool commandKeyDown = false;

	if (WaitMouseUp())
	{
		// Adjust the top of the dragging rectangle so that its below the menu bar
		topLimit = GetMBarHeight() + 4;
		dragRect = draggingBounds;
		if (dragRect.top < topLimit)
			dragRect.top = topLimit;

		// Set up the Window Manager port.
		GetPort(&savePort);
		GetWMgrPort(&windowManagerPort);
		SetPort(windowManagerPort);
		SetClip(GetGrayRgn());

		commandKeyDown = IsCommandKeyDown();

		if (commandKeyDown || !(IsFloatWindow(windowToDrag) || IsModalWindow(windowToDrag)))
		{
			if (!commandKeyDown)
			{
				// If there are floating windows, clip the dragging outline to draw
				// behind the floaters.
				ClipAbove(MAFrontWindow());
			}
			else
			{
				// If the command key was down, clip the outline to draw behind any
				// windows above the window being dragged.
				ClipAbove(windowToDrag);
			}
		}

		// Create a region to drag
		CTempRegion_AC dragRegion;

		GetWindowStructureRgn(windowToDrag, dragRegion);

		// Drag the window around
		dragResult = DragGrayRgn(dragRegion, startPoint, &dragRect, &dragRect, noConstraint, NULL);

		// Restore the port for coordinate conversion.
		SetPort(savePort);

		if (dragResult)
		{
			horizontalOffset = static_cast<short>(dragResult & 0xFFFF);
			verticalOffset = static_cast<short>(dragResult >> 16);

			// Only move it if it stayed inside the dragging box.
			if (verticalOffset != -32768)
			{
				CTempRegion_AC contRgn;
				GetWindowContentRgn(windowToDrag, contRgn);

				newHorizontalWindowPosition = (*contRgn)->rgnBBox.left + horizontalOffset;
				newVerticalWindowPosition = (*contRgn)->rgnBBox.top + verticalOffset;

				MoveWindow(windowToDrag, newHorizontalWindowPosition, newVerticalWindowPosition, false);
			}
		}

		// Bring the window forward if the command key wasnt down
		if (!commandKeyDown)
			MASelectToolboxWindow(windowToDrag);// 3.5
	}
}

//----------------------------------------------------------------------------------------
// MASelectToolboxWindow: 
//----------------------------------------------------------------------------------------
void TWindow::MASelectToolboxWindow(WindowRef windowToSelect)
{
	WindowRef currentFrontWindow = NULL;
	WindowRef lastFloatingWindow = NULL;
	bool isFloatingWindow = IsFloatWindow(windowToSelect);

	if (isFloatingWindow)
	{
		currentFrontWindow = GetFirstFloatingWindowPtr();
	}
	else
	{
		currentFrontWindow = MAFrontWindow();
		lastFloatingWindow = GetLastFloatingWindowPtr();
	}

	// Be fast (and lazy) and do nothing if we dont have to.
	if (currentFrontWindow != windowToSelect)
	{
		// Selecting floating windows are easy, since theyre always active
		if (isFloatingWindow)
			BringToFront(windowToSelect);
		else
		{
			// If there are no floating windows, we can call SelectWindow like the good ol days
			if (!lastFloatingWindow)
			{
				// Keep control off Window's control list b/c the Window Mgr adds the regions of controls in
				// the control list to the update region (see NOTE under DrawControls in Inside Mac volume I)
				// generating an update event and causing all controls in a window to redraw unnecessarily.

				ControlHandle oldFrontControlList;
				ControlHandle newFrontControlList;

				bool currentFrontWindowWasMacAppWindow = false;
				if (currentFrontWindow)
				{
					currentFrontWindowWasMacAppWindow = TWindow::WMgrToWindow(currentFrontWindow) != NULL;
					if (currentFrontWindowWasMacAppWindow)
					{
						oldFrontControlList = ((WindowPeek)currentFrontWindow)->controlList;
						((WindowPeek)currentFrontWindow)->controlList = NULL;
					}
				}

				bool newFrontWindowWasMacAppWindow = false;
				if (windowToSelect)
				{
					newFrontWindowWasMacAppWindow = TWindow::WMgrToWindow(windowToSelect) != NULL;
					if (newFrontWindowWasMacAppWindow)
					{
						newFrontControlList = ((WindowPeek)windowToSelect)->controlList;
						((WindowPeek)windowToSelect)->controlList = NULL;
					}
				}

				::SelectWindow(windowToSelect);

				if (currentFrontWindowWasMacAppWindow)
					((WindowPeek)currentFrontWindow)->controlList = oldFrontControlList;

				if (newFrontWindowWasMacAppWindow)
					((WindowPeek)windowToSelect)->controlList = newFrontControlList;
			}
			else
			{
				// Deactivate the window currently in front.
				MADeactivateWindow(currentFrontWindow);

				// Bring it behind the last floating window if it is not modal and activate it.
				// Note that Inside Mac 1 states that you need to call PaintOne() and CalcVis() on a
				// window if you are using SendBehind() to bring it closer to the front.  With System 7,
				// this is no longer necessary.

				if (!(IsModalWindow(windowToSelect)))
					SendBehind(windowToSelect, lastFloatingWindow);
				else
					BringToFront(windowToSelect);

				MAActivateWindow(windowToSelect);
			}
		}
	}

	// Make sure previous mouse clicks are not considered part of a multi-click.
	gDispatcher->SetLastClickPart(inDesk);
}
// End added for 3.5

//----------------------------------------------------------------------------------------
// MAShowWindow: Show the specified window.  If the window is the frontmost document
// window, unhighlight the window behind it, deactivate it, and activate this one.
//----------------------------------------------------------------------------------------
void TWindow::MAShowWindow(WindowRef windowToShow)
{
	enum WindowFoundState
	{
		eWindowNotFound, eWindowIsFrontmost, eWindowIsNotFrontmost
	};


	WindowRef frontNonFloatingNonModalWindow = NULL;
	bool windowIsInFront = false;

	if (!IsWindowVisible(windowToShow))
	{
		frontNonFloatingNonModalWindow = MAFrontWindow();

		if (IsFloatWindow(windowToShow))
		{
			// A floating window is being shown.  Check to see if a modal window is up before
			// allowing the floater to receive activates.
			windowIsInFront = !IsModalWindow(FrontWindow());
		}
		else
		{
			if (IsModalWindow(windowToShow))
			{
				// We are bringing up a modal window! If we are bringing it up in front of
				// another modal window, we need to deactivate that one.  Otherwise we need
				// to make sure that the frontNonFloatingNonModalWindow has been deactivated
				WindowFoundState windowToShowFoundState = eWindowNotFound;

				for (CWMgrIterator iter; iter; ++iter)
				{
					if (iter.Current() == windowToShow)
						windowToShowFoundState = eWindowIsFrontmost;
					else if (IsWindowVisible(iter))
						windowToShowFoundState = (IsModalWindow(iter) ? eWindowIsNotFrontmost : eWindowIsFrontmost);
						
					if (windowToShowFoundState != eWindowNotFound)
						break;
				}

				if (windowToShowFoundState == eWindowIsFrontmost)
				{
					windowIsInFront = true;

					// now deactivate the frontmost modal window or nonfloater, whichever comes first
					WindowRef theWnd = FrontWindow();
					if (theWnd)
					{
						if (IsModalWindow(theWnd))
							MADeactivateWindow(theWnd);
						else
							MADeactivateWindow(frontNonFloatingNonModalWindow);
					}
				}
			}
			else
			{
				// We are showing a non-floater, non-modal window.  If it is in front of the
				// frontNonFloatingNonModalWindow, we need a deactivate

				if (frontNonFloatingNonModalWindow)
				{
					WindowFoundState windowToShowFoundState = eWindowNotFound;

					for (CWMgrIterator iter; iter; ++iter)
					{
						if (iter == windowToShow)
							windowToShowFoundState = eWindowIsFrontmost;
						else if (iter == frontNonFloatingNonModalWindow)
							windowToShowFoundState = eWindowIsNotFrontmost;
							
						if (windowToShowFoundState != eWindowNotFound)
							break;
					}

					if (windowToShowFoundState == eWindowIsFrontmost)
					{
						windowIsInFront = true;
						MADeactivateWindow(frontNonFloatingNonModalWindow);
					}
				}
				else
					windowIsInFront = true;
			}
		}
		
		if (windowIsInFront)
		{
			// Determine whether this is the foreground app.
			ProcessSerialNumber current, foreground;
			GetCurrentProcess(&current);
			OSErr anErr = GetFrontProcess(&foreground);
			Boolean isSameProcess = false;
			if (!anErr)
				anErr = SameProcess(&current, &foreground, &isSameProcess);

			windowIsInFront = anErr ? false : isSameProcess;
		}

		if (windowIsInFront || IsFloatWindow(windowToShow))
		{
			// Set the highlight state so the window appears highlighted from the start.
			HiliteWindow(windowToShow, true);
		}

		// Show the window
		ShowHide(windowToShow, true);

		// If this is the new frontmost document window or a floating window, send it an activate event
		if (windowIsInFront)
		{
			TWindow * theMAWindow = WMgrToWindow(windowToShow);
			if (theMAWindow)
				theMAWindow->Activate(true);
		}
	}
}

//----------------------------------------------------------------------------------------
// MAHideWindow: Hide the specified window.
//----------------------------------------------------------------------------------------
void TWindow::MAHideWindow(WindowRef windowToHide)
{
	bool needToActivateAnotherWindow = false;

	// Dont do anything if the window is already invisible.
	if (IsWindowVisible(windowToHide))
	{
		needToActivateAnotherWindow = IsWindowHilited(windowToHide) && !IsFloatWindow(windowToHide);

		// Give the window a change of deactivating in case it is presently active
		MADeactivateWindow(windowToHide);

		// Hide the window.
		ShowHide(windowToHide, false);

		// if we were hiding the active window (it was either the frontmost nonfloater or the
		// frontmost modal window), we need to find and activate another window
		if (needToActivateAnotherWindow)
		{
			WindowRef windowToActivate = FrontWindow();
			while (windowToActivate && (IsFloatWindow(windowToActivate) || !IsWindowVisible(windowToActivate)))
				windowToActivate = GetNextWindow(windowToActivate);

			// if we found one, activate it!
			if (windowToActivate)
				MAActivateWindow(windowToActivate);
		}
	}
}


//----------------------------------------------------------------------------------------
// MAFrontWindow: 
//----------------------------------------------------------------------------------------
// Returns the first visible, non-floater window in the window list. Ignores the ghost window (if any). 
// Note that if a modal window has been brought up in front of the floaters, we bypass it too.
WindowRef TWindow::MAFrontWindow()
{
	WindowRef returnVal = NULL;

	for (CWMgrIterator iter; iter; ++iter)
		if (!(IsVisibleFloatWindow(iter) || IsGhostWindow(iter) || IsSystemWindow(iter) || IsModalWindow(iter)) && IsWindowVisible(iter))
		{
			returnVal = iter;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// FreeIfWMgrWindow: 
//----------------------------------------------------------------------------------------
WindowRef TWindow::FreeIfWMgrWindow(WindowRef w,
								bool inDispose)
{
	if (w)
	{
		// If the window is visible, hide it first so
		// that the proper activate and deactivate events are sent.
		if (IsWindowVisible(w))
			MAHideWindow(w);

		if (GetWindowPort(w) == (CGrafPtr)qd.thePort)// Only need to invalidate focus if freed window is the current port
		{
			if (gDispatcher)
				gDispatcher->InvalidateFocus();
			::SetPortWindowPort(gWorkPort);
		}

		if(inDispose)
			::DisposeWindow(w);
		else
		{
#if qMSWindows
			::QTMLCloseWindow(w);
#else
			::CloseWindow(w);
#endif
			delete w;
		}
	}

	return NULL;								// convenience to caller 
}

//----------------------------------------------------------------------------------------
// End of UWindow.cp
