//--------------------------------------------------------------------------------------------------
// SimpleContainer.r
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//
//	Resource File for the SimpleContainer Sample Program. 
//--------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------
// INCLUDE FILES 
//--------------------------------------------------------------------------------------------------

// Standard Includes 

// MacAppTypes.r must be first
#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif

#ifndef __BalloonTypes__
#include "BalloonTypes.r"
#endif

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

// ALOE

#ifndef __UODDefaults__
#include "UODDefaults.h"
#endif


#ifndef THINK_Rez
#if qDebug | qPerform
include "Debug.rsrc" not 'ckid';

include "Dialog.rsrc" not 'ckid';
include "MacApp.rsrc" not 'ckid';

// Includes for Building Blocks

include "Printing.rsrc" not 'ckid';

#endif

// Include Code Resources

#ifndef MW_Rez 
#if !qNoCode
include $$Shell("ObjApp")$$Shell("XAppName") 'CODE';
#if qModelCFM
include $$Shell("ObjApp")$$Shell("XAppName") 'cfrg';
include $$Shell("ObjApp")$$Shell("XAppName") 'rseg';
#endif
#endif
#endif
#endif //!THINK_Rez

//--------------------------------------------------------------------------------------------------
// CONSTANTS 
//--------------------------------------------------------------------------------------------------

// Finder Information
// ------------------

#if !defined(THINK_Rez) && !defined(MW_Rez)
#define kApplicationName	$$Shell("XAppName")
#else
#define kApplicationName	TARGET_APPLICATION_NAME
#endif // !THINK_Rez

#define kSignature			'SS01'			// Application signature 
#define kFileType			'SF01'			// Data file type 
#define kStationery			'sF01'			// Stationery file type 

// Bundle Constants
// ------------------

#define kBundleID						128
#define kApplicationID					128
#define kDocumentID						129
#define kStationeryID					130

// Constants for Menu IDs
// ----------------------

#define mSimpleContainerMenu	4

// Command Numbers For Menu Items
// ------------------------------

#define cNewSamplePart					400
#define cNewDrawEditor					401
#define cNewCyberDocument				402
#define cShowPartExtent					410

// STR# resources
// --------------

#define kSimpleContainerMenuHelp				1000
#define kSimpleContainerWindowTitles			1001

// Text styles
// --------------

#define kSystemTextStyle				1000

// 'View' resources
// --------------

#define kSimpleContainerWindowID				kDefaultWindowID

#define kODContainerView	'ODOC'

// Memory Management Resources
// ---------------------------

#define kSimpleContainerSeg	1000
#define kSimpleContainerMem	1000
#define kSimpleContainerRes	1000

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR MENUS 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Menu Bars 
//--------------------------------------------------------------------------------------------------

resource 'MBAR' (kMBarDisplayed,
	purgeable) {
	{ mApple; mODDocument; mODEdit; mSimpleContainerMenu }
};

//--------------------------------------------------------------------------------------------------
// Menus 
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc" 'CMNU' (mApple);	// Grab the default Apple menus
include "Defaults.rsrc" 'CMNU' (mFile);		// Grab the default File menus
include "Defaults.rsrc" 'CMNU' (mEdit);		// Grab the default Edit menu

resource 'CMNU' (mSimpleContainerMenu,
#if qNames
"mSimpleContainerMenu",
#endif
nonpurgeable) {
	mSimpleContainerMenu,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Container",
	 {	
		"New Clock by ISOString",	noIcon, noKey, noMark, plain, cNewSamplePart;
		"New Draw Editor by Kind", noIcon, noKey, noMark, plain, cNewDrawEditor;
		"New CyberDocument", noIcon, noKey, noMark, plain, cNewCyberDocument;
		"-", noIcon, noKey, noMark, plain, nocommand;
		"Show Part Extents", noIcon, noKey, noMark, plain, cShowPartExtent;
	  }
	};

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
purgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {	/* array: 13 elements */
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
	}
};

//--------------------------------------------------------------------------------------------------
// Balloon Help Resources for Menus 
//--------------------------------------------------------------------------------------------------

#if defined(THINK_Rez) || defined(MW_Rez)
#include "HelpStrings.r"
#else
#include $$Shell("MALibraries") "HelpStrings.r";
#endif

include "Defaults.rsrc"  'hmnu' (mFile);	
include "Defaults.rsrc"  'hmnu' (mEdit);	

resource 'STR#' (kSimpleContainerMenuHelp, purgeable) { 
	{
		"No help yet for this  item",
		"Help For Menu Title",
		"Help For First Menu Item",
		"Help For Second Menu Item",
		"Help For Third Menu Item",
	}
};

resource 'hmnu' (mSimpleContainerMenu) 
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, // balloon definition function 
	0, // variation code 
	
	// Missing items 
	HMStringResItem 
	{
		kSimpleContainerMenuHelp,1,
		kSimpleContainerMenuHelp,1,
		kSimpleContainerMenuHelp,1,
		kSimpleContainerMenuHelp,1,
	},
	
	// Title and items 
	{	
		// Title 
		HMStringResItem 
		{
			kSimpleContainerMenuHelp,2,
			kSimpleContainerMenuHelp,2,
			kSimpleContainerMenuHelp,2,
			kSimpleContainerMenuHelp,2,
		},
		// First Item 
		HMStringResItem 
		{
			kSimpleContainerMenuHelp,3,
			kSimpleContainerMenuHelp,3,
			kSimpleContainerMenuHelp,3,
			kSimpleContainerMenuHelp,3,
		},
		// Second Item
		HMStringResItem 
		{
			kSimpleContainerMenuHelp,4,
			kSimpleContainerMenuHelp,4,
			kSimpleContainerMenuHelp,4,
			kSimpleContainerMenuHelp,4,
		},
		// Third Item
		HMStringResItem 
		{
			kSimpleContainerMenuHelp,5,
			kSimpleContainerMenuHelp,5,
			kSimpleContainerMenuHelp,5,
			kSimpleContainerMenuHelp,5,
		},
	}
};

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR VIEWS, DIALOGS, ALERTS etc. 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Views 
//--------------------------------------------------------------------------------------------------

#ifndef MW_Rez
#include "SimpleContainer.view"	// This file is included in SimpleContainer.prj for MW Builds
#endif

#if 0	// The following has been moved into SimpleContainer.view AdLib file

resource 'STR#' (kSimpleContainerWindowTitles,
#if qNames
"Window titles",
#endif
purgeable) {
	{
	""
	}
};

resource 'TxSt' (kSystemTextStyle, purgeable) {tsPlain, 0, black, ""};

resource 'View' (kSimpleContainerWindowID,
#if qNames
"kSimpleContainerWindowID",
#endif
purgeable) { MAThreeOh, 
	{
	ViewSignatureAndClassname
		{'wind', 534, "", 'WIND', enabled, noIdle, {}, MAThreeOh /* try: MARelease13 */, 
		{50, 20}, {260, 430}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		DrawingEnvironmentClassIDAndClassname {1, "TDrawingEnvironment", {1, 1}, 8, $"FFFF FFFF FFFF FFFF", {0, 0, 0}, {61166, 61166, 61166}}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}, 
			AdornLast, AdornerLocalObject {ResizeIconAdorner}}}, emptyUserArea, 
		Window {zoomDocProc, kODContainerView, goAwayBox, resizable, ignoreFirstClick, 
			freeOnClosing, disposeOnFree, closesDocument, openWithDocument, dontAdaptToScreen, stagger, forceOnScreen, 
			dontCenter, doesntFloat, doesntHideOnSuspend, generateActivates, filler, kSimpleContainerWindowTitles, 1}, 3}, 
	ViewSignatureAndClassname
		{'scrl', 186, "", 'SCLR', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {245, 415}, sizeRelSuperView, sizeRelSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Scroller {'vCLR', 'hCLR', {0, 0}, {16, 16}, VertConstrain, HorzConstrain, noInset, respondsToFKeys}, 1}, 
	ViewSignatureAndClassname
		{'view', 75, "TODContainerView", kODContainerView, enabled, noIdle, 
		{BehaviorSignatureAndClassname {noID, "TDragDropBehavior", DragDropBehavior {enabled, noIdle, notDraggable, droppable, moveWithinApplication, noID}}}, MAThreeOh, 
		{0, 0}, {72 * 15, 72 * 12}, sizeFixed, sizeFixed, shown, wantsToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews},
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'hCLR', enabled, noIdle, {}, MAThreeOh, 
		{245, -1}, {16, 417}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mHScrollBarHit, notHilited, notDimmed, sizeable, noInset, kSystemTextStyle, dontPreferOutline, h, 0, 0, 39585}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'vCLR', enabled, noIdle, {}, MAThreeOh, 
		{-1, 415}, {247, 16}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mVScrollBarHit, notHilited, notDimmed, sizeable, noInset, kSystemTextStyle, dontPreferOutline, v, 0, 0, 49755}, 
		NoSubviews}
	}
};
#endif // Views moved into SimpleContainer.view AdLib file

//--------------------------------------------------------------------------------------------------
// About Box
//--------------------------------------------------------------------------------------------------

resource 'DITL' (phAboutApp,
#if qNames
"phAboutApp",
#endif
	purgeable) {
	 {	/* array DITLarray: 3 elements */
		/* [1] */
		{160, 182, 180, 262},
		Button {
			enabled,
			"OK"
		};
		/* [2] */
		{10, 75, 150, 316},
		StaticText {
			disabled,
			"SimpleContainer demonstrates how to embed OpenDoc content into your existing MacApp application."
			"\n"
			WRITTEN_WITH_MACAPP
		};
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		}
	}
};

include "Defaults.rsrc"  'ALRT' (phAboutApp);		// Grab the default about box
include "Defaults.rsrc"	'aete' (0);					// Grab the default AppleEvent terminology
include "Defaults.rsrc"  'STR#' (kDefaultCredits);	// Grab the default credits


//--------------------------------------------------------------------------------------------------
// MultiFinder information
//--------------------------------------------------------------------------------------------------

/* Additional size deltas to be added to partition size */

#if qNames
#define kNamesSizeDelta 75
#else
#define kNamesSizeDelta 0
#endif

#if qDebug
#define kDebugSizeDelta 0
#else
#define kDebugSizeDelta 0
#endif

#define kModelFarSizeDelta 0

#ifndef MW_Rez
resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	notStationeryAware,				
	reserved,
	reserved,
	reserved,
	reserved,
	
	/* Suggested partition size */
	(4000 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024,
	
	/* Minimum partition size */
	(1200 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024
};
#endif
//--------------------------------------------------------------------------------------------------
// Icons
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc"  'ICN#' (kApplicationID);	// MacApp Family large black & white	
include "Defaults.rsrc"  'ICN#' (kDocumentID);		// MacApp Document large black & white
include "Defaults.rsrc"  'ICN#' (kStationeryID);	// MacApp Stationery large black & white	
include "Defaults.rsrc"  'ics#' (kApplicationID);	// MacApp Family small black & white	
include "Defaults.rsrc"  'ics#' (kDocumentID);		// MacApp Document small black & white
include "Defaults.rsrc"  'ics#' (kStationeryID);	// MacApp Stationery small black & white
include "Defaults.rsrc"  'ics4' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'ics4' (kDocumentID);		// MacApp Document small 4 bit
include "Defaults.rsrc"  'ics4' (kStationeryID);	// MacApp Stationery small 4 bit
include "Defaults.rsrc"  'ics8' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'ics8' (kDocumentID);		// MacApp Document small 4 bit
include "Defaults.rsrc"  'ics8' (kStationeryID);	// MacApp Stationery small 4 bit
include "Defaults.rsrc"  'icl4' (kApplicationID);	// MacApp Family large 4 bit
include "Defaults.rsrc"  'icl4' (kDocumentID);		// MacApp Document large 4 bit
include "Defaults.rsrc"  'icl4' (kStationeryID);	// MacApp Stationery large 4 bit
include "Defaults.rsrc"  'icl8' (kApplicationID);	// MacApp Family large 8 bit
include "Defaults.rsrc"  'icl8' (kDocumentID);		// MacApp Document large 8 bit
include "Defaults.rsrc"  'icl8' (kStationeryID);	// MacApp Stationery large 8 bit

//--------------------------------------------------------------------------------------------------
// Memory usage information for SimpleContainer
//--------------------------------------------------------------------------------------------------

// See UMemory.h
// -------------

// Code fragment resource for PowerPC native apps
// ----------------------------------------------
#ifndef MW_Rez 	// Metrowerks will supply it's own cfrg
#if qPowerPC
#include "CodeFragmentTypes.r"

resource 'cfrg' (0, "Configuration Info") {
	{
		kPowerPC,
		kFullLib,
		kNoVersionNum,
		kNoVersionNum,
		0,
		0,
		kIsApp,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		"SimpleContainer"
	}
};
#endif
#endif // MW_Rez

//--------------------------------------------------------------------------------------------------
// Icons, Bundles and FRefsOh my! (don't forget the Signature)
//--------------------------------------------------------------------------------------------------

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"SimpleContainer Apple Computer, Inc. 1996"
};

resource 'FREF' (kApplicationID,
#if qNames
"SimpleContainer Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (kDocumentID,
#if qNames
"SimpleContainer Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'FREF' (kStationeryID,
#if qNames
"SimpleContainer Stationery",
#endif
	purgeable) {
	kStationery,
	2,
	""
};

resource 'BNDL' (kBundleID,
#if qNames
"SimpleContainer",
#endif
	purgeable) {
	kSignature,
	0,
	{ /* array TypeArray: 2 elements */
		/* [1] */
		'ICN#',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		},
		/* [2] */
		'FREF',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		}
	}
};

//--------------------------------------------------------------------------------------------------
//  Version resources
//--------------------------------------------------------------------------------------------------

// The revision of this particular file

RESOURCE 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x01,
	0x00,
	development,
	0x00,
	verUs,
	"1.0",
	"SimpleContainer 1.0, Apple Computer, Inc. 1996"
};

include "Defaults.rsrc"  'vers' (2);		// Overall package
