//----------------------------------------------------------------------------------------
// UDocumentSimpleContainer.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UDocumentSimpleContainer__
#include "UDocumentSimpleContainer.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// ALOE

#ifndef __CODSession_AC__
#include "CODSession_AC.h"
#endif

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

#ifndef __UODAdaptorViewProxy__
#include "UODAdaptorViewProxy.h"
#endif

#ifndef __UODPartView__
#include "UODPartView.h"
#endif

#ifndef __UFILESTREAM__
#include "UFileStream.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#include <list.h>

//----------------------------------------------------------------------------------------
// Constants:

#define cNewSamplePart					400
#define cNewDrawEditor					401
#define cNewCyberDocument				402
#define cShowPartExtent					410

const ChangeID_AC cSubViewAdded = 1000;
	
const ResNumber kSimpleContainerWindowID = kDefaultWindowID;
const ResNumber kContainerWindoidID = 1003;

const IDType kODContainerView = 'ODOC';


const CViewRect kStandardInset = CViewRect(0, 0, 0, 0);


//========================================================================================
// CLASS TDocumentSimpleContainer
//========================================================================================
#undef Inherited
#if qALOE
#define Inherited TODFileBasedDocument
#else
#define Inherited TFileBasedDocument
#endif


MA_DEFINE_CLASS(TDocumentSimpleContainer);

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer constructor 
//----------------------------------------------------------------------------------------


TDocumentSimpleContainer::TDocumentSimpleContainer(TFile* itsFile,
										  OSType itsCreator) :
		Inherited(itsFile, itsCreator),
		fContainerView(NULL),
		fShowViewExtents(false),
		fStaggerOffset(0, 0),
		fProxyList(NULL)
{
} // TDocumentSimpleContainer::TDocumentSimpleContainer 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer destructor
//----------------------------------------------------------------------------------------


TDocumentSimpleContainer::~TDocumentSimpleContainer()
{
}

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::FreeData: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::FreeData()
{
	Inherited::FreeData();
} // TDocumentSimpleContainer::FreeData 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoInitialState: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoInitialState()
{
	Inherited::DoInitialState();
} // TDocumentSimpleContainer::DoInitialState 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoMakeViews(bool /*forPrinting*/)
{
	// Open a windoid that shows status information !!! Only for testing
	//TWindow* windoid = gViewServer->NewTemplateWindow(kContainerWindoidID, this);
	//::ThrowIfNULL_AC(windoid);

	TWindow* aWindow = gViewServer->NewTemplateWindow(kSimpleContainerWindowID, this);
	::ThrowIfNULL_AC(aWindow);

	/* Store a reference to our container view (we use it as a proxy for content) */
	fContainerView = aWindow->FindSubView(kODContainerView);

	TStdPrintHandler* aHandler = TH_new TStdPrintHandler(this,			// its document 
							   fContainerView,			// its view 
							   kFixedSize,		// horzontal page size is fixed 
							   kFixedSize);		// vertical page size is fixed 

#if qALOE
	// Add our adaptor views now, if any and create views
	if (fProxyList)
	{
		for (TProxyList::iterator iter = GetProxyList()->begin(); iter != GetProxyList()->end(); ++iter)
		{
			AddAdaptorViewFromProxy(*iter);
		}
	}
#endif // qALOE

} // TDocumentSimpleContainer::DoMakeViews 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoSetupMenus: This method is overridden to enable menu items which
// should be enabled when this object is in the target chain. MacApp initially disables
// all menu items, then lets the objects in the target chain enable those items they
// handle.
//
// A document object is in the target chain when its window is the active window.
//
// The inherited method is called so that TDocument can enable document-level menu items
// like "Save ". This also ensures that objects further up the target chain (the
// application) can set up THEIR menus.
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoSetupMenus()
{
	Inherited::DoSetupMenus();
	
	Enable(cNewSamplePart, true);
	Enable(cNewDrawEditor, true);
	Enable(cNewCyberDocument, true);
	EnableCheck(cShowPartExtent, true, fShowViewExtents);
} // TDocumentSimpleContainer::DoSetupMenus 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoMenuCommand: This method is overridden to handle menu items which
// are enabled when this document is in the target chain. In this example, this is true
// when the document is open and its window is the active window. The inherited method
// should always be called so that MacApp can allow successor objects in the target chain
// (i.e. the application) to handle THEIR menu items.
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoMenuCommand(CommandNumber aCommandNumber)
{
	#define kODISOPrefix 					 "+//ISO 9070/ANSI::113722::US::CI LABS::"
	#define kClockPartISO 		kODISOPrefix "Apple:ODFExamples:Kind:ODFClock"

	const ResType kDrawEditorKind = 'drwd';		// !!! not working
	const ResType kCyberEditorKind = 'cdog';	// !!! not working
	
	switch (aCommandNumber) {
#if qALOE
		case cNewSamplePart:
			AddAdaptorViewByISOString(kClockPartISO);
			break;
		
		case cNewDrawEditor:
			AddAdaptorViewByKind(kDrawEditorKind);
			break;

		case cNewCyberDocument:
			AddAdaptorViewByKind(kCyberEditorKind);
			break;
			
		case cShowPartExtent:
			ToggleShowViewExtents();
			break;
#endif // qALOE
			
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
} // TDocumentSimpleContainer::DoMenuCommand 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoEvent: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event)
{
	switch (eventNumber) {
#if qALOE
		case cDrop:
			// Our container view sends us this event when it receives a drop
			if (source == fContainerView)
				AddAdaptorViewsFromDrag();
			break;
#endif // qALOE
			
		default:
			Inherited::DoEvent(eventNumber, source, event);
			break;
	}
}
		
//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes)
{
	Inherited::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);
} // TDocumentSimpleContainer::DoNeedDiskSpace 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoRead: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoRead(TFile* aFile,
						   	   bool forPrinting)
{
	// Call inherited first so AdaptorMgr is instantiated
	Inherited::DoRead(aFile, forPrinting);	
	
	// Now read our native content

#if qALOE
	// Since we are a simple container, all we care about is where we are locating our
	// embedded adaptor views
	TFileStream aFileStream(aFile);
	short viewCount = aFileStream.ReadInteger();
	for (short i = 0; i < viewCount; ++i)
	{
		CAutoPtr_AC<TODAdaptorViewProxy> proxy(TH_new TODAdaptorViewProxy);
		proxy->ReadFrom(&aFileStream);
		if (proxy->IsGood())
			AddProxy(proxy);
		proxy.release();
	}
#endif
} // TDocumentSimpleContainer::DoRead 

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::DoWrite: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::DoWrite(TFile* aFile,
									bool makingCopy)
{
	// Save our native content first
#if qALOE	
	// Since we are a simple container, all we care about is where we want to
	// locate our embedded adaptor views
	TFileStream aFileStream(aFile);
	aFileStream.WriteInteger(fContainerView->CountSubViews());
	for (CSubViewIterator iter(fContainerView); iter; ++iter)
	{
		// Saving container's views requires a cast
		TODPartView* aView = dynamic_cast<TODPartView*>(iter.Current());
		if (aView) {
			TODAdaptorViewProxy proxy(aView->GetAdaptorView());
			proxy.WriteTo(&aFileStream);
		}
#if qDebugMsg
		else
			::DebugStr("\pThere is a subview that isn't an ODPartView");
#endif
	}
#endif
	
	// Call inherited last to save adaptor manager
	Inherited::DoWrite(aFile,makingCopy);
} // TDocumentSimpleContainer::DoWrite 

#if qALOE

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::ToggleShowViewExtents
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::ToggleShowViewExtents()
{
	fShowViewExtents = !fShowViewExtents;
	for (CSubViewIterator iter(fContainerView); iter; ++iter)
	{
		if (fShowViewExtents)
			iter->AddAdorner(&TAdorner::fgHiliteAdorner, kAdornLast, kRedraw);
		else
			iter->DeleteAdorner(&TAdorner::fgHiliteAdorner, kRedraw);
	}
	
	fContainerView->ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::AddAdaptorView: Common code for adding a new adaptor
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::AddAdaptorView(TODPartView* maView)
{
	if (fShowViewExtents)
		maView->AddAdorner(&TAdorner::fgHiliteAdorner, kAdornLast, kRedraw);			
	fContainerView->AddSubView(maView);
	gApplication->InvalidateFocus();
	maView->ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::AddAdaptorViewFromProxy
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::AddAdaptorViewFromProxy(TODAdaptorViewProxy* proxy)
{
	// Create the adaptor
	ALOEPersistentID id;
	proxy->GetPersistentID(id);	
	WindowPtr theWindow = fContainerView->GetWindow()->GetWindowRef();
	CODAdaptor_AC* odAdaptor = GetAdaptorManager()->NewAdaptorFromStorage(id, theWindow);
	
	// Create the view in the appropriate offset (nb view needs to adjust to adaptor)
	CPoint_AC offset = proxy->GetOffset();
	const CViewPoint itsLocation(offset);
	const CViewPoint itsSize(180, 80);
	
	// Create an ODPartView to display the ALOEAdaptorView
	AddAdaptorView(TH_new TODPartView(itsLocation, itsSize, sizeVariable, sizeVariable, odAdaptor, kStandardInset));

	// Creation of view by proxy occurs as a result of file reading,
	// so we don't mark the document as changed
}

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::AddAdaptorViewByISOString
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::AddAdaptorViewByISOString(ALOEISOString kind)
{
	const CViewPoint itsSize(180, 80);
	const short kStandardOffset = 20;
	
	// Simple-stagger sample part	
	fStaggerOffset.Offset(kStandardOffset, kStandardOffset);
	
	WindowPtr theWindow = fContainerView->GetWindow()->GetWindowRef();
			
	// Create an ALOEAdaptorView from ALOE and give it default extent properties
	RgnHandle shape = ::NewRgn();
	::SetRectRgn(shape, 0, 0, itsSize.h, itsSize.v);
	
	CODAdaptor_AC* odAdaptor = GetAdaptorManager()->NewAdaptor(theWindow, kind, shape);
			
	// Create an ODPartView to display the ALOEAdaptorView
	AddAdaptorView(TH_new TODPartView(fStaggerOffset, itsSize, sizeVariable, sizeVariable, odAdaptor, kStandardInset));

	// creation by kind means document has changed
	Changed(cSubViewAdded, this);
}
//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::AddAdaptorViewByKind
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::AddAdaptorViewByKind(ResType kind)
{
	const CViewPoint itsSize(180, 80);
	const short kStandardOffset = 20;
	
	// Simple-stagger sample part	
	fStaggerOffset.Offset(kStandardOffset, kStandardOffset);
	
	WindowPtr theWindow = fContainerView->GetWindow()->GetWindowRef();
			
	// Create an ALOEAdaptorView from ALOE and give it default extent properties
	RgnHandle shape = ::NewRgn();
	::SetRectRgn(shape, 0, 0, itsSize.h, itsSize.v);
	
	CODAdaptor_AC* odAdaptor = GetAdaptorManager()->NewAdaptor(theWindow, kind, shape);
			
	// Create an ODPartView to display the ALOEAdaptorView
	AddAdaptorView(TH_new TODPartView(fStaggerOffset, itsSize, sizeVariable, sizeVariable, odAdaptor, kStandardInset));

	// creation by kind means document has changed
	Changed(cSubViewAdded, this);
}

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer::AddAdaptorViewsFromDrag:
//----------------------------------------------------------------------------------------


void TDocumentSimpleContainer::AddAdaptorViewsFromDrag()
{
	// ALOE accepts the drag reference and item numbers, so we use these instead of
	// the dragItemIterator
	DragReference theReference = TDragDropSession::fgDragDropSession->GetDragReference();
	WindowPtr theWindow = fContainerView->GetWindow()->GetWindowRef();

	// Get drag offset in our view for use in frame negotiation
	// --- (recipe clip here) ---
	Point thePinnedMouse;
	CPoint_AC theMouse;
	::ThrowIfOSErr_AC( ::GetDragMouse(theReference, &theMouse, &thePinnedMouse));
	::GlobalToLocal(&theMouse);
	CODSession_AC::Instance()->SetDropLocation(theMouse);
	// --- (recipe clip here) ---
	unsigned short itemCount = TDragDropSession::fgDragDropSession->GetItemCount();
	for (unsigned short thisItem = 1; thisItem <= itemCount; ++thisItem)
	{
		// Get item bounds
		Rect dragBounds;
		ItemReference theItemRef;
		GetDragItemReferenceNumber(theReference, thisItem, &theItemRef);
		::ThrowIfOSErr_AC(::GetDragItemBounds(theReference, theItemRef, &dragBounds));
		
		// Create an ALOEAdaptor
		CODAdaptor_AC* odAdaptor = GetAdaptorManager()->NewAdaptorFromDrag(theWindow, theReference, thisItem);

		// Use drop size and coordinates that make sense to the container view
		CViewPoint itsSize(0, 0); // will get negotiated
				
		// Create an ODPartView to display the ALOEAdaptorView
		AddAdaptorView(TH_new TODPartView(fContainerView->RootViewToLocal(CViewPoint(theMouse.h, theMouse.v)), itsSize, sizeVariable, sizeVariable, odAdaptor, kStandardInset));

		// A drop means document changed
		Changed(cSubViewAdded, this);
	}
}

//----------------------------------------------------------------------------------------
// Gets list, creating an empty one if not initialized yet
//----------------------------------------------------------------------------------------

TProxyList* TDocumentSimpleContainer::GetProxyList()
{
	if (fProxyList.IsNULL())
	{
		fProxyList = TH_new TProxyList;
	}
	return fProxyList;
}
	
//----------------------------------------------------------------------------------------
// Add proxy to proxy list
//----------------------------------------------------------------------------------------
	
void TDocumentSimpleContainer::AddProxy(TODAdaptorViewProxy* proxy)
{
	GetProxyList()->push_front(proxy);
}
	
//----------------------------------------------------------------------------------------
// Delete proxy list
//----------------------------------------------------------------------------------------

void TDocumentSimpleContainer::DeleteProxyList()
{
	// This deletes and resets the proxy list pointer
	fProxyList = NULL;
}

#endif // qALOE

//----------------------------------------------------------------------------------------
// End of UDocumentSimpleContainer.cp


