// UDocumentSimpleContainer.h
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 

#ifndef __UDocumentSimpleContainer__
#define __UDocumentSimpleContainer__

#ifndef __UODFILEBASEDDOCUMENT__
#include "UODFileBasedDocument.h"
#endif

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

// Commands
const CommandNumber cCommandHandledByDocument = 401;

// Files
const OSType kFileType = 'SF01';	// File-type code used for document files 

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class TODPartView;

class TODAdaptorViewProxy;

#include <list.h>

template <class T, class ALLOCATOR> class list;

typedef list<TODAdaptorViewProxy*, allocator<TODAdaptorViewProxy*> > TProxyList;

//----------------------------------------------------------------------------------------
// TDocumentSimpleContainer
//----------------------------------------------------------------------------------------

#if qALOE
class TDocumentSimpleContainer: public TODFileBasedDocument
#else
class TDocumentSimpleContainer: public TFileBasedDocument
#endif
{
	MA_DECLARE_CLASS;
	
  public:
	TDocumentSimpleContainer(TFile* itsFile = NULL, OSType itsCreator = kNoIdentifier); 
	virtual ~TDocumentSimpleContainer();
	
	virtual void  DoInitialState();	// Override
	virtual void  FreeData();	// Override
			
	// Views
	virtual void  DoMakeViews(bool forPrinting);	// Override

	// Commands
	virtual void  DoSetupMenus();	// Override
	virtual void  DoMenuCommand(CommandNumber aCommandNumber);	// Override

	virtual void DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event); // Override
		
		
	// Saving and Restoring
	virtual void  DoNeedDiskSpace(TFile* itsFile,
								  long& dataForkBytes,
								  long& rsrcForkBytes);	// Override
								  
	virtual void  DoRead(TFile* aFile,
						 bool forPrinting);	// Override
	virtual void  DoWrite(TFile* aFile,
						  bool makingCopy);	// Override

#if qALOE
	// Extensions for SimpleContainer
	
	void	ToggleShowViewExtents();
	// Adds or removes adorners for debugging purposes
	
	void	AddAdaptorView(TODPartView* aView);
	// Common code for adding a new adaptor view
	
	void	AddAdaptorViewFromProxy(TODAdaptorViewProxy* proxy);
	// Add new adaptor view from proxy (e.g., from file storage)
	
	void	AddAdaptorViewByISOString(ALOEISOString kind);
	// Add new adaptor view by embedding it
	
	void	AddAdaptorViewByKind(OSType kind);
	// Add new adaptor view by embedding it
	
	void	AddAdaptorViewsFromDrag();
	// Add new adaptor views from a drag
	
	TProxyList*	GetProxyList();
	// Gets list, creating an empty one if not initialized yet
	
	void	AddProxy(TODAdaptorViewProxy* proxy);
	// Add proxy to proxy list
	
	void	DeleteProxyList();
	// Delete proxy list
	
#endif

protected:
	TView*		fContainerView;
	bool		fShowViewExtents;
	CViewPoint		fStaggerOffset;
	CAutoPtr_AC<TProxyList>	fProxyList;
};

#endif
