//----------------------------------------------------------------------------------------
// UODContainerView.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UODContainerView__
#include "UODContainerView.h"
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

//----------------------------------------------------------------------------------------
// InitUODContainerView: 
//----------------------------------------------------------------------------------------


void InitUODContainerView()
{
	MA_REGISTER_CLASS(TODContainerView);
} 

//========================================================================================
// CLASS TODContainerView
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TODContainerView);

TODContainerView::TODContainerView(const CViewPoint& itsLocation, const CViewPoint& itsSize,
				SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet):
			TView(itsLocation, itsSize,itsHSizeDet,itsVSizeDet)
{
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------


TODContainerView::~TODContainerView()
{
}

#if qDrag
//----------------------------------------------------------------------------------------
// DoPostCreate: register our window with the Drag manager
//----------------------------------------------------------------------------------------

void TODContainerView::DoPostCreate(TDocument* itsDocument)
{
	fDroppable = true;

	Inherited::DoPostCreate(itsDocument);	
}
		
//----------------------------------------------------------------------------------------
// TODContainerView::WillAcceptDrop
//----------------------------------------------------------------------------------------


bool TODContainerView::WillAcceptDrop(CDragItemIterator& /*dragItemIterator*/)
{
	// We will accept anything dropped in by default
	// which lets OpenDoc cope with document translation, etc.
	
	return true;
}

//----------------------------------------------------------------------------------------
// TODContainerView::DoMakeDragDropCommand
//----------------------------------------------------------------------------------------


TCommand* TODContainerView::DoMakeDragDropCommand(CommandNumber itsCommandNumber,
										CDragItemIterator&	dragItemIterator)
{
	TCommand 		*returnCommand = NULL;
	TCommandHandler *itsContext = GetContext(itsCommandNumber);
	
	switch(itsCommandNumber)
	{
		case cDrag:	// ??? handled by the part itself
			break;
			
		case cDrop:
			{			
				// return a command to ensure proper drag drop feedback
				returnCommand = TH_new TODDropCommand(this,
				 								itsCommandNumber,
												itsContext, kCantUndo, kDoesNotCauseChange,
												itsContext);
			}
			break;
		
		case cDragMove: // doesn't do anything...will cause return value to be NULL
			break;
			
		default:
			returnCommand = Inherited::DoMakeDragDropCommand(itsCommandNumber, dragItemIterator);
			break;
	}
	
	return returnCommand;
}

//========================================================================================
// CLASS TODContainerViewCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TODDropCommand::TODDropCommand
//----------------------------------------------------------------------------------------


TODDropCommand::TODDropCommand(			TODContainerView*	itsContainerView,
										CommandNumber		itsCommandNumber,
										TCommandHandler*	itsContext,
										bool				canUndo,
										bool				causesChange,
										MDependable_AC*		objectToNotify) :
	TCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify)
{	
	// Context should be the document	
	itsContainerView->GetDocument()->DoEvent(cDrop, itsContainerView, NULL);
}

//----------------------------------------------------------------------------------------
// TODDropCommand::~TODDropCommand
//----------------------------------------------------------------------------------------


TODDropCommand::~TODDropCommand()
{
}

//----------------------------------------------------------------------------------------
// TODDropCommand::DoIt
//----------------------------------------------------------------------------------------


void TODDropCommand::DoIt()
{
// ??? Drag session seems to go stale if we call DoEvent from here
}

#endif // qDrag

//----------------------------------------------------------------------------------------
// End of UODContainerView.cp


