//----------------------------------------------------------------------------------------
// TTempoGridView.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __TTempoGridView__
#include "TTempoGridView.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

static const kMinColumnWidth = 10;
static const kMaxColumnWidth = 256;
static const ResNumber kColumnSizingCursor = 256;

#undef Inherited

//========================================================================================
// Global Initialization Procedure
//========================================================================================
//----------------------------------------------------------------------------------------
// InitTTempoGridView: 
//----------------------------------------------------------------------------------------


void InitTTempoGridView()
{
	MA_REGISTER_CLASS(TTempoGridView);
	MA_REGISTER_CLASS(TTempoGridViewAdorner);
	MA_REGISTER_CLASS(TTempoColumnsView);
	MA_REGISTER_CLASS(TTempoColumnsViewAdorner);
}

//========================================================================================
// CLASS TTempoGridView
//========================================================================================
#undef Inherited
#define Inherited TTextGridView


MA_DEFINE_CLASS(TTempoGridView);

TTempoGridView::TTempoGridView()
{
}

TTempoGridView::~TTempoGridView()
{
}

//----------------------------------------------------------------------------------------
// TTempoGridView::GetText
//----------------------------------------------------------------------------------------


CStr255_AC TTempoGridView::GetText(const GridCell& aCell) const
{
	CStr255_AC aString;
	CStr255_AC bString;
	NumToString(aCell.h, aString);
	NumToString(aCell.v, bString);

	return aString + ", " + bString;
}


//----------------------------------------------------------------------------------------
// TTempoGridView::DoMouseCommand
//----------------------------------------------------------------------------------------


void TTempoGridView::DoMouseCommand(CViewPoint& theMouse,
								  TToolboxEvent* event,
								  CPoint_AC /* hysteresis */)	// Override
{
	GridCell aCell;

	if (IdentifyPoint(theMouse, aCell) != badChoice && CanSelectCell(aCell))
		PostCommand(new TRowSelectCommand(this, theMouse, event->IsShiftKeyPressed(), event->IsCommandKeyPressed()));
}

//----------------------------------------------------------------------------------------
// TTempoGridView::HighlightCells
//----------------------------------------------------------------------------------------


void TTempoGridView::HighlightCells(const TGridViewDesignator* theCells, HLState fromHL, HLState toHL)
{	
	// This example works for contiguous rectangular selections.
	CViewRect cellBounds(theCells->GetBounds());
	
	TGridView* itsColumnsView = dynamic_cast_AC(TGridView*, GetWindow()->FindSubView('COLS')); 
	if (itsColumnsView)
	{
		// get the column selection and hilite the columns in the main grid view
		CViewRect columnSelection(itsColumnsView->GetSelections()->GetBounds()); 
		if (columnSelection != gZeroVRect)
		{
			CViewRect hilitedRect(columnSelection.left, cellBounds.top, columnSelection.right, cellBounds.bottom);
			TGridViewDesignator hilitedCells(hilitedRect);
			
			// This only highlights the intersected cell, which avoids unfriendly highlighting.
			Inherited::HighlightCells(&hilitedCells, fromHL, toHL);
		}
	}
}

//========================================================================================
// CLASS TTempoGridViewAdorner
//========================================================================================
#undef Inherited
#define Inherited TAdorner


MA_DEFINE_CLASS(TTempoGridViewAdorner);

TTempoGridViewAdorner::TTempoGridViewAdorner()
{
}

TTempoGridViewAdorner::~TTempoGridViewAdorner()
{
}

//----------------------------------------------------------------------------------------
// TTempoGridViewAdorner::
//----------------------------------------------------------------------------------------


void TTempoGridViewAdorner::Draw(TView* itsView, const CViewRect& area)
{

	TGridView* itsGridView = dynamic_cast<TGridView*>(itsView);
	if (itsGridView && itsGridView->Focus())
	{
		GridCell startCell(itsGridView->VPointToLastCell(area[topLeft]));
		GridCell stopCell(itsGridView->VPointToLastCell(area[botRight]));

		CViewRect localArea = area;

		CViewRect aRect(itsGridView->CellToVRect(startCell));
		CViewRect bRect(itsGridView->CellToVRect(stopCell));
		aRect[botRight] = bRect[botRight];

		CTempForeColor_AC tempForeColor(kRGBColorRamp[rW]); // white
		CTempBackColor_AC tempBackColor(kRGBColorRamp[r1]); // light gray
		
		// Adorn area with light gray
		CRect_AC aQDRect = itsGridView->ViewToQD(aRect);		
		EraseRect(aQDRect);

		
		// the columns view's selection has changed, and cache only the column selection.
		// For the purpose of illustration (er, a quick hack), assume there is a real
		// columnsView and use FindSubView instead.
		
		// Get hilited row(s) and adorn darker gray
		kRGBColorRamp[r2].SetBackColor(); // medium gray
		TGridView* itsColumnsView = dynamic_cast_AC(TGridView*,itsView->GetWindow()->FindSubView('COLS')); 
		if (itsColumnsView)
		{
			// get the column selection and hilite the columns in the main grid view
			CViewRect columnSelection(itsColumnsView->GetSelections()->GetBounds()); 
			if (columnSelection != gZeroVRect)
			{
				CViewRect hilitedRect(itsGridView->ColToVRect(columnSelection.left, columnSelection.right - columnSelection.left));
				EraseRect(itsGridView->ViewToQD(hilitedRect));
			}
		}
		
		for (long i = startCell.v; i <= stopCell.v; ++i)
		{
			aRect.bottom = aRect.top + itsGridView->GetRowHeight(i);

			// Adorn row bottom with white line
			MoveTo(aRect.left, aRect.bottom - 1);
			LineTo(aRect.right, aRect.bottom - 1);
						
			aRect.top = aRect.bottom;
		}
		
	}
}

//========================================================================================
// CLASS TTempoColumnsView
//========================================================================================
#undef Inherited
#define Inherited TTextGridView


MA_DEFINE_CLASS(TTempoColumnsView);

TTempoColumnsView::TTempoColumnsView() :
	fGridView(NULL)
{
}

TTempoColumnsView::~TTempoColumnsView()
{
}

//----------------------------------------------------------------------------------------
// TTempoColumnsView::GetText
//----------------------------------------------------------------------------------------


CStr255_AC TTempoColumnsView::GetText(const GridCell& aCell) const
{
	CStr255_AC aString;
	CStr255_AC bString;
	NumToString(aCell.h, aString);
	NumToString(aCell.v, bString);

	return aString + ", " + bString;
}


//----------------------------------------------------------------------------------------
// TTempoColumnsView::DoMouseCommand
//----------------------------------------------------------------------------------------


void TTempoColumnsView::DoMouseCommand(CViewPoint& theMouse,
								  TToolboxEvent* event,
								  CPoint_AC /* hysteresis */)	// Override
{
	GridCell aCell;
	GridViewPart whichPart = IdentifyPoint(theMouse, aCell);	//since almost 
	switch (whichPart)
	{
		case inCell:
			PostCommand(new TColumnSelectCommand(this, theMouse, event->IsShiftKeyPressed(), event->IsCommandKeyPressed()));
			break;
			
		case inColumn:
		case inVertex:
			if (aCell.h > 1)
				PostCommand(new TColumnSizer(fGridView, this, aCell.h - 1, theMouse));
			break;
	}
}

//----------------------------------------------------------------------------------------
// TTempoColumnsView::Activate
//----------------------------------------------------------------------------------------


void TTempoColumnsView::Activate(bool entering)
{
	Inherited::Activate(entering);

	if (Focus())
	{
		ForceRedraw();
		DrawContents();				// Draw me. This will account for IsCellSelected and IsActive etc.
	}
}

//----------------------------------------------------------------------------------------
// TTempoColumnsView::HighlightCells
//----------------------------------------------------------------------------------------


void TTempoColumnsView::HighlightCells(const TGridViewDesignator* theCells, HLState /*fromHL*/, HLState toHL)
{
	// Highlights the cells in theCells according to the given highlight state.
	if (Focus())
		;
	
	// Undo any wierd pen state	
	PenNormal();
	CViewRect 	insetRect(fColInset / 2, fRowInset / 2,
					  -(fColInset / 2), -(fRowInset / 2));
	for (CCellInRegionIterator iter(this, theCells); iter; ++iter)	
	{
		GridCell curCell(iter);
		CViewRect localVRect(CellToVRect(curCell));
		CRect_AC aRect(ViewToQD(localVRect));
		localVRect += insetRect;
		
		// Adorn cell like a button
		AGADrawBevelButton(aRect, kMiniFrame, false /*isOn*/, toHL==hlOn /*isSelected*/, IsEnabled() && IsActive());
		// And draw its contents as usual
		if (Focus())
		{
			PenNormal();
			DrawCell(curCell, localVRect);
		}
	}

}

//----------------------------------------------------------------------------------------
// TTempoColumnsView::DoEvent
//----------------------------------------------------------------------------------------


void TTempoColumnsView::DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event)
{
	if (eventNumber == mListItemHit && source == this)
	{
		
		// to the main grid view's adorner, e.g., Changed(eventNumber, this);
		// For the purpose of illustration (er, a quick hack), use FindSubView instead.
		TView* aView = GetWindow()->FindSubView('GRID');
		TGridView* aGridView = dynamic_cast<TGridView*>(aView);
		if (aGridView)
		{
			aGridView->ForceRedraw();
		}
	} else
		Inherited::DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TCellsView::DoSetCursor: 
//----------------------------------------------------------------------------------------
void TTempoColumnsView::DoSetCursor(const CViewPoint& globalMouse,
							 RgnHandle cursorRegion)	// override 
{

	CViewRect cellsExtent;

	GridCell aCell(-1, -1);						// This value should never be used
	GridViewPart aGridViewPart = IdentifyPoint(globalMouse, aCell);

	switch (aGridViewPart)					//since almost 
	{
		case inColumn:
		case inVertex:
			{
				if (aCell.h > 1)
				{
					SetCursor(*GetCursor(kColumnSizingCursor));
					ColToVRect(Min_AC(aCell.h, fNumOfCols), 1, cellsExtent);
					CRect_AC columnQDExtent(ViewToQD(cellsExtent));

					// Which edge is the mouse closer to? 
					if (labs(columnQDExtent.right - globalMouse.h) < labs(columnQDExtent.left - globalMouse.h))
						columnQDExtent.left = columnQDExtent.right;

					columnQDExtent.left = columnQDExtent.left - fColInset / 2;
					columnQDExtent.right = columnQDExtent.left + fColInset;
					RectRgn(cursorRegion, columnQDExtent);
				}
				break;
			}
		default:
			break;
	}
}

//========================================================================================
// CLASS TTempoColumnsViewAdorner
//========================================================================================
#undef Inherited
#define Inherited TAdorner


MA_DEFINE_CLASS(TTempoColumnsViewAdorner);

TTempoColumnsViewAdorner::TTempoColumnsViewAdorner()
{
}

TTempoColumnsViewAdorner::~TTempoColumnsViewAdorner()
{
}

//----------------------------------------------------------------------------------------
// TTempoGridViewAdorner::
//----------------------------------------------------------------------------------------


void TTempoColumnsViewAdorner::Draw(TView* itsView, const CViewRect& area)
{
	TGridView* itsGridView = dynamic_cast<TGridView*>(itsView);
	if (itsGridView)
	{
		GridCell startCell(itsGridView->VPointToLastCell(area[topLeft]));
		GridCell stopCell(itsGridView->VPointToLastCell(area[botRight]));

		CRect_AC aRect;
		
		for (long i = startCell.h; i <= stopCell.h; ++i)
		{
			aRect = itsGridView->ViewToQD(itsGridView->CellToVRect(GridCell(i, 1)));
			
			// Adorn cell like a button
			AGADrawBevelButton(aRect, kMiniFrame, false /*isOn*/, itsGridView->IsCellSelected(GridCell(i, 1)),  itsGridView->IsEnabled() && itsGridView->IsActive());
		}
		if (itsGridView->Focus())
			;
	}
}

//========================================================================================
// CLASS TColumnSizer
//========================================================================================
#undef Inherited
#define Inherited TTracker

//----------------------------------------------------------------------------------------
// TColumnSizer::IColumnSizer: 
//----------------------------------------------------------------------------------------


TColumnSizer::TColumnSizer(TGridView* itsGridView, TGridView* itsColumnsView,
									   GridCoordinate c, const CViewPoint& theMouse) :
	TTracker(cNoCommand, itsGridView->GetDocument(), kCanUndo, kCausesChange, itsGridView->GetDocument(),
				   itsColumnsView,
				   itsColumnsView->GetScroller(true), theMouse),
	fGridView(itsGridView),
	fColumnsView(itsColumnsView),
	fColumn(c),

	fLeftEdge(itsColumnsView->ColToVRect(c, 1).left),
	fNewWidth(itsColumnsView->GetColWidth(c)),
	fOldWidth(fNewWidth)
{
	fConstrainsMouse = true;
	fViewConstrain = false;						// So that rightmost column can grow 
}

//----------------------------------------------------------------------------------------
// TColumnSizer destructor
//----------------------------------------------------------------------------------------


TColumnSizer::~TColumnSizer()
{
}

//----------------------------------------------------------------------------------------
// TColumnSizer::TrackMouse: 
//----------------------------------------------------------------------------------------


TTracker* TColumnSizer::TrackMouse(TrackPhase	/*aTrackPhase*/,
										  CViewPoint&		/*anchorPoint*/ ,
										  CViewPoint&		/*previousPoint*/ ,
										  CViewPoint&		nextPoint,
										  bool		/*mouseDidMove*/)// override 
{
	fNewWidth = Min_AC(static_cast<short>(nextPoint.h - fLeftEdge), kMaxColumnWidth);
	fNewWidth = Max_AC(fNewWidth, kMinColumnWidth);
	nextPoint.h = Min_AC(nextPoint.h, fLeftEdge + kMaxColumnWidth);

	return this;
} 


//----------------------------------------------------------------------------------------
// TColumnSizer::TrackFeedback: 
//----------------------------------------------------------------------------------------


void TColumnSizer::TrackFeedback(TrackPhase		/* aTrackPhase */,
										const CViewPoint&	/* anchorPoint */,
										const CViewPoint&	/* previousPoint */,
										const CViewPoint&	nextPoint,
										bool			mouseDidMove,
										bool			/* turnItOn */)// override 
{
	PenState	pState;

	// if there was any horizontal movement
	if (mouseDidMove)
	{
		GetPenState(&pState);				// save pen state
		PenPat(&qd.black);

		// draw in the columns view
		CRect_AC viewedRect = fView->GetQDExtent();
		MoveTo(Min_AC(static_cast<short>(nextPoint.h), static_cast<short>(fLeftEdge + kMaxColumnWidth)), viewedRect.top);
		Line(0, viewedRect.bottom - viewedRect.top);

		// draw in the cells view
		short savePenMode = qd.thePort->pnMode;
		fGridView->Focus();
		PenPat(&qd.black);
		PenMode(savePenMode);
		viewedRect = fGridView->GetQDExtent();
		MoveTo(Min_AC(static_cast<short>(nextPoint.h), static_cast<short>(fLeftEdge + kMaxColumnWidth)), viewedRect.top);
		Line(0, viewedRect.bottom - viewedRect.top);

		fView->Focus();
		
		SetPenState(&pState);				// restore pen state
	}
}


//----------------------------------------------------------------------------------------
// TColumnSizer::TrackConstrain: 
//----------------------------------------------------------------------------------------


void TColumnSizer::TrackConstrain(TrackPhase		/*aTrackPhase*/,
										 const CViewPoint&	/*anchorPoint*/ ,
										 const CViewPoint&	/*previousPoint*/ ,
										 CViewPoint&		nextPoint,
										 bool		/*mouseDidMove*/)// override 
{
	if (nextPoint.h < fLeftEdge + 10)
		nextPoint.h = fLeftEdge + 10;
	else
		nextPoint.h = Min_AC(nextPoint.h, fLeftEdge + kMaxColumnWidth);
}


//----------------------------------------------------------------------------------------
// TColumnSizer::DoIt: 
//----------------------------------------------------------------------------------------


void TColumnSizer::DoIt()			// override 
{
	SetColumnWidth(fNewWidth);

	CViewRect aRect(fGridView->ColToVRect(fColumn, 1));
	fGridView->RevealRect(aRect, aRect.GetSize(), true);

	CViewRect bRect(fColumnsView->ColToVRect(fColumn, 1));
	fColumnsView->RevealRect(bRect, bRect.GetSize(), true);
}


//----------------------------------------------------------------------------------------
// TColumnSizer::UndoIt: 
//----------------------------------------------------------------------------------------


void TColumnSizer::UndoIt()			// override 
{
	SetColumnWidth(fOldWidth);
}


//----------------------------------------------------------------------------------------
// TColumnSizer::SetColumnWidth: 
//----------------------------------------------------------------------------------------


void TColumnSizer::SetColumnWidth(ViewCoordinate newWidth)
{
	if (newWidth > kMaxColumnWidth)
	{
		newWidth = kMaxColumnWidth;
		fNewWidth = kMaxColumnWidth;
	}
	else if (newWidth < kMinColumnWidth)
		newWidth = kMinColumnWidth;

	fGridView->SetColWidth(fColumn, 1, newWidth);
	fColumnsView->SetColWidth(fColumn, 1, newWidth);
}

//----------------------------------------------------------------------------------------
// End of TTempoGridView.cp


