// TTempoGridView.h
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 


#ifndef __TTempoGridView__
#define __TTempoGridView__

#ifndef __UGRIDVIEW__
#include "UGridView.h"
#endif

//----------------------------------------------------------------------------------------
// TTempoGridView
//----------------------------------------------------------------------------------------

class TTempoGridView : public TTextGridView
{
	MA_DECLARE_CLASS;
	
public:
	TTempoGridView();
	
	virtual ~TTempoGridView();
	
	virtual void DoMouseCommand(CViewPoint&			theMouse,
								TToolboxEvent*	event,
								CPoint_AC			hysteresis);
								
	virtual CStr255_AC GetText(const GridCell& aCell) const;
	
	virtual void HighlightCells(const TGridViewDesignator* theCells, HLState fromHL, HLState toHL);
		// Highlights the cells in a Tempo-savvy way	
};

//----------------------------------------------------------------------------------------
// TTempoGridViewAdorner
//----------------------------------------------------------------------------------------

class TTempoGridViewAdorner : public TAdorner
{
	MA_DECLARE_CLASS;
	
public:
	TTempoGridViewAdorner();
	
	virtual ~TTempoGridViewAdorner();
	
	virtual void Draw(TView* itsView, const CViewRect& area);
		// Adorn GridView cells a la Tempo
};

//----------------------------------------------------------------------------------------
// TTempoColumnsView
//----------------------------------------------------------------------------------------

class TTempoColumnsView : public TTextGridView
{
	MA_DECLARE_CLASS;
public:
	TTempoColumnsView();
	virtual ~TTempoColumnsView();
	
	virtual void DoMouseCommand(CViewPoint&			theMouse,
								TToolboxEvent*	event,
								CPoint_AC			hysteresis);
	virtual CStr255_AC GetText(const GridCell& aCell) const;

	virtual void Activate(bool entering);
		// Handles activation highlighting

	virtual void HighlightCells(const TGridViewDesignator* theCells, HLState fromHL, HLState toHL);
		// Highlights the cells in a Tempo-savvy way

	virtual void DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event);
		// check for item hit

	void SetPrimaryGridView(TGridView* itsPrimaryGridView) 
		{ fGridView = itsPrimaryGridView; }
		// Set at view initialization time so we can call back to the primary grid view.

	virtual void DoSetCursor(const CViewPoint& globalMouse,
							 RgnHandle cursorRegion);

protected:
	TGridView*	fGridView; // Our primary grid view
};

//----------------------------------------------------------------------------------------
// TTempoColumnsViewAdorner
//----------------------------------------------------------------------------------------

class TTempoColumnsViewAdorner : public TAdorner
{
	MA_DECLARE_CLASS;
	
public:
	TTempoColumnsViewAdorner();
	
	virtual ~TTempoColumnsViewAdorner();
	
	virtual void Draw(TView* itsView, const CViewRect& area);
		// Adorn GridView cells a la Tempo
};

//--------------------------------------------------------------------------------------------------
// TColumnSizer: TColumnSizer is a command object created to handle mouse movement when
// you click on the boundary between two columns.
//--------------------------------------------------------------------------------------------------

class TColumnSizer : public TTracker
{
public:
	TColumnSizer(TGridView* itsGridView = NULL, TGridView* itsColumnsView = NULL,
					GridCoordinate	c = 0, const CViewPoint& theMouse = gZeroVPt);

	virtual ~TColumnSizer();
		// Destructor
	
	virtual TTracker* TrackMouse(TrackPhase	/*aTrackPhase*/,
										CViewPoint&		/*anchorPoint*/,
										CViewPoint&		/*previousPoint*/,
										CViewPoint&		nextPoint,
										bool		/*mouseDidMove*/);

	virtual void TrackFeedback(TrackPhase	/*aTrackPhase*/,
									  const CViewPoint&	/*anchorPoint*/ ,
									  const CViewPoint&	/*previousPoint*/ ,
									  const CViewPoint&	nextPoint,
									  bool		mouseDidMove,
									  bool		/*turnItOn*/);// override 

	virtual void TrackConstrain(TrackPhase		/*aTrackPhase*/,
									   const CViewPoint&	/*anchorPoint*/ ,
									   const CViewPoint&	/*previousPoint*/ ,
									   CViewPoint&			nextPoint,
									   bool			/*mouseDidMove*/);// override 

	virtual void DoIt();				// override 

	virtual void UndoIt();			// override 

	virtual void SetColumnWidth(ViewCoordinate newWidth);

public:
	TGridView*			fGridView;
	TGridView*			fColumnsView;
	GridCoordinate		fColumn;
	ViewCoordinate			fLeftEdge;
	ViewCoordinate			fNewWidth;
	ViewCoordinate			fOldWidth;				// remember previous width for Undo 
 };

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

extern void InitTTempoGridView();
	// Call this routine at initialization time

#endif
