//----------------------------------------------------------------------------------------
// UDocumentTempoGridView.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UDocumentTempoGridView__
#include "UDocumentTempoGridView.h"
#endif

#ifndef __TTempoGridView__
#include "TTempoGridView.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//----------------------------------------------------------------------------------------
// Constants:

const ResNumber kTempoGridViewWindowID = kDefaultWindowID;

//========================================================================================
// CLASS TDocumentTempoGridView
//========================================================================================
#undef Inherited
#define Inherited TFileBasedDocument


MA_DEFINE_CLASS_M1(TDocumentTempoGridView, Inherited);

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView destructor
//----------------------------------------------------------------------------------------


TDocumentTempoGridView::~TDocumentTempoGridView()
{
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::FreeData: 
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::FreeData() // Override 
{
	Inherited::FreeData();
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::DoInitialState: 
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::DoInitialState() // Override 
{
	Inherited::DoInitialState();
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::DoMakeViews(bool /*forPrinting*/) // Override 
{
	TWindow* aWindow = gViewServer->NewTemplateWindow(kTempoGridViewWindowID, this);
	::ThrowIfNULL_AC(aWindow);

	const IDType kGridView = 'GRID';
	const IDType kColumnsView = 'COLS';
	TView* gridView = aWindow->FindSubView(kGridView);
	TView* columnsView = aWindow->FindSubView(kColumnsView);

	TPrimaryScroller* aGridScroller = dynamic_cast_AC(TPrimaryScroller*,gridView->GetScroller(true));
	TSecondaryScroller* aColumnScroller = dynamic_cast_AC(TSecondaryScroller*,columnsView->GetScroller(true));
	aGridScroller->AddSecondaryScroller(aColumnScroller, kHDependent, kNotVDependent);

	TStdPrintHandler* aHandler = new TStdPrintHandler(this,			// its document 
							   gridView,			// its view 
							   kFixedSize,		// horzontal page size is fixed 
							   kFixedSize);		// vertical page size is fixed
	 
	 // Set an initial selection for the columns view and give it a reference to the main grid view
	 TGridView* primaryView = dynamic_cast_AC(TGridView*,gridView);
	 TTempoColumnsView* secondaryView = dynamic_cast_AC(TTempoColumnsView*,columnsView);
	 if (primaryView && secondaryView)
	 {
	 	secondaryView->SetPrimaryGridView(primaryView);
	 	secondaryView->SelectCell(GridCell(1, 1), kDontExtend, kHighlight, kSelect);
	 }
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes) // Override 
{
	Inherited::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::DoMenuCommand: This method is overridden to handle menu items which
// are enabled when this document is in the target chain. In this example, this is true
// when the document is open and its window is the active window. The inherited method
// should always be called so that MacApp can allow successor objects in the target chain
// (i.e. the application) to handle THEIR menu items.
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		/*	Typically, you will have dispatch your own menu commands here:
		case cCommandHandledByDocument: 
			{
				TCommandTempoGridView* aCommand = new TCommandTempoGridView;
				aCommand->ICommandTempoGridView(aCommandNumber, this);
				PostCommand(aCommand);
			}
			break;
		*/
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::DoRead: 
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::DoRead(TFile* aFile,
						   			bool forPrinting) // Override 
{
	Inherited::DoRead(aFile,forPrinting);
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::DoSetupMenus: This method is overridden to enable menu items which
// should be enabled when this object is in the target chain. MacApp initially disables
// all menu items, then lets the objects in the target chain enable those items they
// handle.
//
// A document object is in the target chain when its window is the active window.
//
// The inherited method is called so that TDocument can enable document-level menu items
// like "Save ". This also ensures that objects further up the target chain (the
// application) can set up THEIR menus.
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::DoSetupMenus() // Override 
{
	Inherited::DoSetupMenus();
	
	/*	Typically, you will have enable your own menu commands here:
	Enable(cCommandHandledByDocument,true);
	*/
}

//----------------------------------------------------------------------------------------
// TDocumentTempoGridView::DoWrite: 
//----------------------------------------------------------------------------------------


void TDocumentTempoGridView::DoWrite(TFile* aFile,
									bool makingCopy) // Override 
{
	Inherited::DoWrite(aFile,makingCopy);
}

//----------------------------------------------------------------------------------------
// End of UDocumentTempoGridView.cp


