//--------------------------------------------------------------------------------------------------
// UMyApplication.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright  1997 by Trygve Isaacson.
//--------------------------------------------------------------------------------------------------


#include "UMyApplication.h"

#include "MyConstants.h"

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __CDialog_AC__
#include "CDialog_AC.h"
#endif

#ifndef __UBEVELEDTARGETBORDERVIEW__
#include "UBeveledTargetBorderView.h"
#endif

#ifndef __UBACKGROUNDADORNER__
#include "UBackgroundAdorner.h"
#endif

#ifndef __USCROLLBARPROPORTIONAL__
#include "UScrollBarProportional.h"
#endif

#ifndef __UDISCLOSUREBEHAVIOR__
#include "UDisclosureBehavior.h"
#endif

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

#undef Inherited
#define Inherited TApplication

TMyApplication::TMyApplication() :
	TApplication('????', '????'),
	// Init the pointers & stuff used during idling.
	fProgressIndicator1(NULL),
	fProgressIndicator2(NULL),
	fMovingProgressOrigin(false),
	fSkipNext(false),
	fMainWindow(NULL)
{

	//
	// Initialize views used by our example
	//
	InitUBeveledTargetBorderView();
	InitUBackgroundAdorner();
	InitUNewControls();
#if qGrayscaleAppearance
	InitUScrollBarProportional();
#endif
	// Tell MacApp we don't want an Untitled document created at launch.
	fLaunchWithNewDocument = false;	
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::DoInitialState()
{
	Inherited::DoInitialState();
	this->SetIdleFreq(2);
	
	// Create our main window.
	this->SwapMainWindow();
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

TMyApplication::~TMyApplication()
{
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::SwapMainWindow()
{
	// Close existing window and toggle flags if window exists.
	
	if (fMainWindow != NULL)
	{
		fMainWindow->Close();
		delete fMainWindow;
		SetEmulateControls_AC(!EmulateControls_AC());
	}

	// Create our main window.
	
	::ThrowIfNULL_AC(fMainWindow = gViewServer->NewTemplateWindow(kMainWindowViewID, NULL));
	
	// Set the checkboxes that show/control the GC global flags.

	TCheckBox*	aCheckBox;
	
	aCheckBox = dynamic_cast_AC(TCheckBox*,fMainWindow->FindSubView('StdX'));
	aCheckBox->SetState(EmulateControls_AC(), kDontRedraw);	// not yet visible so don't redraw yet
	
	// Allow the window to be resized, but no smaller than
	// its initial size.
	
	CPoint_AC	minWindowSize;
	CPoint_AC	maxWindowSize(kMaxCoord, kMaxCoord);
	
	CRect_AC	windowFrame(fMainWindow->GetQDExtent());
	minWindowSize = windowFrame[botRight] - windowFrame[topLeft];

	fMainWindow->SetResizeLimits(minWindowSize, maxWindowSize);

#if !qGrayscaleAppearance
	// Proportional scroll bars are not available when the grayscale appearance feature is disabled
	// so we disable the button that shows proportional scroll bars.
	TButton* aButton = dynamic_cast_AC(TButton*,fMainWindow->FindSubView(kMyViewBase+kScrollBarDlgIndex));
	aButton->ViewEnable(false, kDontRedraw);	// not yet visible so don't redraw yet
	aButton->DimState(true, kDontRedraw);		

	// Cluster dialog requires qGrayscaleAppearance
	aButton = dynamic_cast_AC(TButton*,fMainWindow->FindSubView(kMyViewBase + kGroupBoxDlgIndex));
	aButton->ViewEnable(false, kDontRedraw);	// not yet visible so don't redraw yet
	aButton->DimState(true, kDontRedraw);
	
	// Icon buttons are only available in qGrayscaleAppearance is on (depends on mods to TCtlMgr)
	aButton = dynamic_cast_AC(TButton*,fMainWindow->FindSubView(kMyViewBase + kIconButtonDlgIndex));
	aButton->ViewEnable(false, kDontRedraw);	// not yet visible so don't redraw yet
	aButton->DimState(true, kDontRedraw);

	// Progress indicators are only available if qGrayscaleAppearance is on (depends on mods to TCtlMgr)
	aButton = dynamic_cast_AC(TButton*,fMainWindow->FindSubView(kMyViewBase + kProgressDlgIndex));
	aButton->ViewEnable(false, kDontRedraw);	// not yet visible so don't redraw yet
	aButton->DimState(true, kDontRedraw);
#endif
	fMainWindow->Open();
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::DoSetupMenus()
{
	Inherited::DoSetupMenus();
	
	// Enable all of the Test menu commands.
	for (UInt32 i = 0; i < kNumDialogs; ++i)
		Enable(kMyCommandBase + i, true);

#if !qGrayscaleAppearance
	// disable the menu items that can't be opened if not compiling for qGrayscaleAppearance
	Enable(kMyCommandBase + kScrollBarDlgIndex, false);
	Enable(kMyCommandBase + kGroupBoxDlgIndex, false);
	Enable(kMyCommandBase + kIconButtonDlgIndex, false);
	Enable(kMyCommandBase + kProgressDlgIndex, false);
#endif

	Enable(kMADlgMenuCmdBase + phAboutApp, true);
	Enable(kMADlgMenuCmdBase + phRevert, true);
	Enable(kMADlgMenuCmdBase + phSaveChanges, true);
	Enable(kMADlgMenuCmdBase + phNewPublisherAlert, true);
	Enable(kMADlgMenuCmdBase + phMultPublisherWrn, true);
	Enable(kMADlgMenuCmdBase + phSavingMultPublisherWrn, true);
	Enable(kMADlgMenuCmdBase + phFileChanged, true);
	Enable(kMADlgMenuCmdBase + phPurgeOld, true);
	Enable(kMADlgMenuCmdBase + phInvalidValue, true);
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::DoMenuCommand(CommandNumber aCommand)
{
	// Handle the Test menu commands.

	if ((aCommand - kMyCommandBase >= 0) &&
		(aCommand - kMyCommandBase < kNumDialogs))
		this->PoseExampleDialog(aCommand - kMyCommandBase);
	else if (aCommand == kMADlgMenuCmdBase + phAboutApp)
		this->DoStdMAAboutBox();
	else if (aCommand > kMADlgMenuCmdBase)
	{
		::ParamText("\pParam1", "\pParam2", "\pParam3", "\pParam4");
		(void) MacAppAlert(aCommand - kMADlgMenuCmdBase, NULL);
	}
	else
		Inherited::DoMenuCommand(aCommand);
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------
void TMyApplication::DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event)
{
	IDType		sourceID = source->GetIdentifier();
	UInt32		buttonDialogIndex = sourceID;	// main window buttons are ID'd as kMyViewBase + dialogIndex

	if ((buttonDialogIndex >= kMyViewBase) && (buttonDialogIndex < (kMyViewBase + kNumDialogs)))
		this->PoseExampleDialog(buttonDialogIndex - kMyViewBase);
	else
	{
		switch (sourceID)
		{
			case 'StdX':
			
				{
					gApplication->PostCommand(TH_new TSwapMainWindowCmd(20000, gApplication, kCantUndo, kDoesNotCauseChange, NULL));
				}

				break;
			
			case 'ENAB':
				{
					// Enable or disable everything in the button's dialog.

					TCheckBox*	aCheckBox = dynamic_cast_AC(TCheckBox*,source);
					TWindow*	aWindow = aCheckBox->GetWindow();
					
					this->EnableDisableDialog(fCurrentDialogIndex, aWindow, aCheckBox->IsOn());
				}
				break;

			case 'SBAR':
				{
					// The scroll bar dialog's TScrollBar was hit.
					// Update the text that displays the TScrollBar value.

					TScrollBar*		aScrollBar = dynamic_cast_AC(TScrollBar*,source);
					TWindow*		aWindow = aScrollBar->GetWindow();
					TStaticText*	valueText = dynamic_cast_AC(TStaticText*,aWindow->FindSubView('SBTX'));
					SInt32			scrollBarValue = aScrollBar->GetLongVal();
					CStr255_AC			valueString;
					
					::NumToString(scrollBarValue, valueString);
					
					valueText->SetText(valueString, kRedraw);
				}
				break;

#if qGrayscaleAppearance
			case 'LIVE':
			case 'PROP':
				{
					// The scroll bar dialog had one of its checkboxes hit.
					// Toggle the appropriate attribute of the 3 scroll bars
					// in the dialog.
					if (EmulateControls_AC())
					{
						TScrollerScrollBarProportional*	aScrollerScrollBar;
						TWindow*	aWindow = dynamic_cast_AC(TCheckBox*,source)->GetWindow();
						TCheckBox*	liveCheckBox = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('LIVE'));
						TCheckBox*	propCheckBox = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('PROP'));
		
						aScrollerScrollBar = dynamic_cast_AC(TScrollerScrollBarProportional*,aWindow->FindSubView('HSBR'));
						aScrollerScrollBar->GetAGAScrollBar()->SetAttributes(CControl_AC::kHorizontal, liveCheckBox->IsOn(), propCheckBox->IsOn());
						aScrollerScrollBar->UpdateScrollAmounts();
						
						if (sourceID == 'PROP')
							aScrollerScrollBar->ForceRedraw();
		
						aScrollerScrollBar = dynamic_cast_AC(TScrollerScrollBarProportional*,aWindow->FindSubView('VSBR'));
						aScrollerScrollBar->GetAGAScrollBar()->SetAttributes(CControl_AC::kVertical, liveCheckBox->IsOn(), propCheckBox->IsOn());
						aScrollerScrollBar->UpdateScrollAmounts();
						
						if (sourceID == 'PROP')
							aScrollerScrollBar->ForceRedraw();
		
						TScrollBarProportional*	aScrollBar = dynamic_cast_AC(TScrollBarProportional*,aWindow->FindSubView('SBAR'));
						aScrollBar->GetAGAScrollBar()->SetAttributes(CControl_AC::kHorizontal, liveCheckBox->IsOn(), propCheckBox->IsOn());
						
						if (sourceID == 'PROP')
							aScrollBar->ForceRedraw();
					}
				}
				break;
#endif // qGrayscaleAppearance

			case 'LVSL':
			case 'PRSL':
				{
					// The slider dialog had one of its checkboxes hit.
					// Toggle the appropriate attribute of all sliders
					// in the dialog.

					TSlider*	aSlider;
					TWindow*		aWindow = dynamic_cast_AC(TCheckBox*,source)->GetWindow();
					TCheckBox*		liveCheckBox = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('LVSL'));
					TCheckBox*		propCheckBox = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('PRSL'));

					aSlider = dynamic_cast_AC(TSlider*,aWindow->FindSubView('VSPL'));
					aSlider->GetAGAControl()->SetAttributes(CControl_AC::kVertical, liveCheckBox->IsOn(), propCheckBox->IsOn());
					if (sourceID == 'PRSL') aSlider->ForceRedraw();

					aSlider = dynamic_cast_AC(TSlider*,aWindow->FindSubView('VSPR'));
					aSlider->GetAGAControl()->SetAttributes(CControl_AC::kVertical, liveCheckBox->IsOn(), propCheckBox->IsOn());
					if (sourceID == 'PRSL') aSlider->ForceRedraw();

					aSlider = dynamic_cast_AC(TSlider*,aWindow->FindSubView('VSRC'));
					aSlider->GetAGAControl()->SetAttributes(CControl_AC::kVertical, liveCheckBox->IsOn(), propCheckBox->IsOn());
					if (sourceID == 'PRSL') aSlider->ForceRedraw();

					aSlider = dynamic_cast_AC(TSlider*,aWindow->FindSubView('HSPL'));
					aSlider->GetAGAControl()->SetAttributes(CControl_AC::kHorizontal, liveCheckBox->IsOn(), propCheckBox->IsOn());
					if (sourceID == 'PRSL') aSlider->ForceRedraw();

					aSlider = dynamic_cast_AC(TSlider*,aWindow->FindSubView('HSPR'));
					aSlider->GetAGAControl()->SetAttributes(CControl_AC::kHorizontal, liveCheckBox->IsOn(), propCheckBox->IsOn());
					if (sourceID == 'PRSL') aSlider->ForceRedraw();

					aSlider = dynamic_cast_AC(TSlider*,aWindow->FindSubView('HSRC'));
					aSlider->GetAGAControl()->SetAttributes(CControl_AC::kHorizontal, liveCheckBox->IsOn(), propCheckBox->IsOn());
					if (sourceID == 'PRSL') aSlider->ForceRedraw();
				}
				break;

			case 'BIGP':
			case 'LILP':
				{
					TPicture*	thePicture = dynamic_cast_AC(TPicture*,dynamic_cast_AC(TView*,source)->GetWindow()->FindSubView('PICT'));
					
					thePicture->SetPictureRsrcID((sourceID == 'BIGP') ? kBigPictureID : kLittlePictureID, kRedraw);

					CRect_AC	pictFrame = (**(thePicture->fDataHandle)).picFrame;
					CViewPoint	pictSize(pictFrame[botRight] - pictFrame[topLeft]);
					
					thePicture->Resize(pictSize, kInvalidate);
				}
				break;

			case 'LRGT':
			case 'SMLT':
				{
					TTabbedView*		tabPanel;
					bool				small = (sourceID == 'SMLT');
					
					tabPanel = dynamic_cast_AC(TTabbedView*,dynamic_cast_AC(TView*,source)->GetWindow()->FindSubView('TAB1'));
					if ((tabPanel != NULL) && (tabPanel->GetAGAControl() != NULL))
					{
						tabPanel->GetAGAControl()->SetTabSize(small ? CTabbedView_AC::kSmallTabs : CTabbedView_AC::kLargeTabs);
						tabPanel->GetAGAControl()->SetLabelsStyle(small ? CTabbedView_AC::kSmallTabPanelStyle : gSystemStyle);
						tabPanel->ForceRedraw();
						
						tabPanel = dynamic_cast_AC(TTabbedView*,dynamic_cast_AC(TView*,source)->GetWindow()->FindSubView('TAB2'));
						tabPanel->GetAGAControl()->SetTabSize(small ? CTabbedView_AC::kSmallTabs : CTabbedView_AC::kLargeTabs);
						tabPanel->GetAGAControl()->SetLabelsStyle(small ? CTabbedView_AC::kSmallTabPanelStyle : gSystemStyle);
						tabPanel->ForceRedraw();
					}
				}
				break;

			case 'GO_1':
				{
					// Save the pointer to the indeterminate progress indicator,
					// so we'll increment it at idle. Swap the enable states
					// of its Go and Stop buttons.

					TButton*	buttonToDisable = dynamic_cast_AC(TButton*,source);
					TWindow*	aWindow = buttonToDisable->GetWindow();
					TButton*	buttonToEnable = dynamic_cast_AC(TButton*,aWindow->FindSubView('STP1'));
					
					this->ToggleButtons(buttonToDisable, buttonToEnable);
					
					fProgressIndicator1 = dynamic_cast_AC(TProgressIndicator*,aWindow->FindSubView('PRG1'));
					
					fProgressIndicator1->StartAutoAnimate();
				}
				break;
			
			case 'GO_2':
				{
					// Null the pointer to the determinate progress indicator,
					// so we'll ignore it at idle. Swap the enable states
					// of its Go and Stop buttons.

					TButton*	buttonToDisable = dynamic_cast_AC(TButton*,source);
					TWindow*	aWindow = buttonToDisable->GetWindow();
					TButton*	buttonToEnable = dynamic_cast_AC(TButton*,aWindow->FindSubView('STP2'));
					
					this->ToggleButtons(buttonToDisable, buttonToEnable);
					
					fProgressIndicator2 = dynamic_cast_AC(TProgressIndicator*,aWindow->FindSubView('PRG2'));
				}
				break;
			
			case 'STP1':
				{
					// Save the pointer to the indeterminate progress indicator,
					// so we'll increment it at idle. Swap the enable states
					// of its Go and Stop buttons.

					TButton*	buttonToDisable = dynamic_cast_AC(TButton*,source);
					TWindow*	aWindow = buttonToDisable->GetWindow();
					TButton*	buttonToEnable = dynamic_cast_AC(TButton*,aWindow->FindSubView('GO_1'));
					
					this->ToggleButtons(buttonToDisable, buttonToEnable);
					
					fProgressIndicator1 = dynamic_cast_AC(TProgressIndicator*,aWindow->FindSubView('PRG1'));
					fProgressIndicator1->StopAutoAnimate();
					fProgressIndicator1 = NULL;
				}
				break;
			
			case 'STP2':
				{
					// Null the pointer to the indeterminate progress indicator,
					// so we'll ignore it at idle. Swap the enable states
					// of its Go and Stop buttons.

					TButton*	buttonToDisable = dynamic_cast_AC(TButton*,source);
					TWindow*	aWindow = buttonToDisable->GetWindow();
					TButton*	buttonToEnable = dynamic_cast_AC(TButton*,aWindow->FindSubView('GO_2'));
					
					this->ToggleButtons(buttonToDisable, buttonToEnable);
			
					fProgressIndicator2 = NULL;
				}
				break;
			
			case 'CORG':
				{
					// The progress dialog's "Moving Origin" checkbox was hit.
					// Enable or disable the edit text and little arrows, and
					// set our flag that we'll check at idle.

					TCheckBox*	aCheckBox = dynamic_cast_AC(TCheckBox*,source);
					TWindow*	aWindow = aCheckBox->GetWindow();
					bool		enabled = aCheckBox->IsOn();
					TControl*	aControl;
					
					aControl = dynamic_cast_AC(TControl*,aWindow->FindSubView('NORG'));
					dynamic_cast_AC(TEditText*,aControl)->StopEdit();
					aControl->SetEnable(enabled);
					aControl->DimState(! enabled, kRedraw);
					
					aControl = dynamic_cast_AC(TControl*,aWindow->FindSubView('AORG'));
					aControl->SetEnable(enabled);
					aControl->DimState(! enabled, kRedraw);
					
					aControl = dynamic_cast_AC(TControl*,aWindow->FindSubView('TORG'));
					aControl->SetEnable(enabled);
					aControl->DimState(! enabled, kRedraw);
					
					fMovingProgressOrigin = enabled;
				}
				break;
			
			case 'SETV':
				{
					// The "Set Values" button in the progress dialog was hit.
					// Poke the determinate progress indicator with the current
					// values that have been entered in the dialog.

					TWindow*		aWindow = dynamic_cast_AC(TButton*,source)->GetWindow();
					TNumberText*	aNumberText;
					TCheckBox*		originCheckBox = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('CORG'));
					SInt32			newMin;
					SInt32			newMax;
					SInt32			newValue;
					SInt32			newOrigin;

					TProgressIndicator*	indicator = dynamic_cast_AC(TProgressIndicator*,aWindow->FindSubView('PRG2'));
					
					aNumberText = dynamic_cast_AC(TNumberText*,aWindow->FindSubView('NMIN'));
					newMin = aNumberText->GetValue();
					
					aNumberText = dynamic_cast_AC(TNumberText*,aWindow->FindSubView('NMAX'));
					newMax = aNumberText->GetValue();
					
					//indicator->GetAGAControl()->SetRange(newMin, newMax, kDontRedraw);
					indicator->SetMin(newMin, kDontRedraw);
					indicator->SetMax(newMax, kDontRedraw);
					
					aNumberText = dynamic_cast_AC(TNumberText*,aWindow->FindSubView('NVAL'));
					newValue = aNumberText->GetValue();
					//indicator->GetAGAControl()->SetValue(newValue, kDontRedraw);
					indicator->SetVal(newValue, kDontRedraw);
					if (! originCheckBox->IsOn())
						newOrigin = newMin;
					else
					{
						aNumberText = dynamic_cast_AC(TNumberText*,aWindow->FindSubView('NORG'));
						newOrigin = Max_AC(newMin, aNumberText->GetValue());
					}

	
	//#if !qWantsAppearance
					//indicator->GetAGAControl()->SetOriginValue(newOrigin, kDontRedraw);
	//#endif				
					indicator->ForceRedraw();
				}
				break;

			case 'EDCB':
				{
					// Enable or disable the first field of the edit text dialog.

					TCheckBox*	aCheckBox = dynamic_cast_AC(TCheckBox*,source);
					TWindow*	aWindow = aCheckBox->GetWindow();
					bool		enabled = aCheckBox->IsOn();
					TEditText*	anEditText = dynamic_cast_AC(TEditText*,aWindow->FindSubView('EDT1'));

					anEditText->StopEdit();
					anEditText->SetEnable(enabled);
					anEditText->DimState(! enabled, kRedraw);
				}
				break;

		}
	}
	
	Inherited::DoEvent(eventNumber, source, event);
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::ToggleButtons(TButton* buttonToDisable, TButton* buttonToEnable)
{
	buttonToDisable->SetEnable(false);
	buttonToDisable->DimState(true, kRedraw);

	buttonToEnable->SetEnable(true);
	buttonToEnable->DimState(false, kRedraw);
}

bool TMyApplication::DoIdle(IdlePhase /*phase*/)
{
	// If we have a pointer to the determinate progress indicator,
	// increment its value.
	if (fProgressIndicator2 != NULL)
		if (fProgressIndicator2->Focus())
		{
			fProgressIndicator2->SetVal(fProgressIndicator2->GetVal() + 1, kRedraw);
			
			// Increment the origin every other time, if it's moving.

//#if !qWantsAppearance
			//if (fMovingProgressOrigin && ! fSkipNext)
			//	fProgressIndicator2->GetAGAControl()->SetOriginValue(1 + fProgressIndicator2->GetAGAControl()->GetOriginValue(), kRedraw);
//#endif			
			fSkipNext = ! fSkipNext;	// make moving origin go slower than value
		}

	return false;	// did not free self
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::PoseExampleDialog(UInt32 dialogIndex)
{
	// Run the specified dialog.
	if (dialogIndex == kLaunchGridViewIndex)
		LaunchGridViewExample();
	else
	{

		TWindow	*aWindow;

		::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(kMyViewBase + dialogIndex, NULL));
		
		fCurrentDialogIndex = dialogIndex;
		
		this->SetupExampleDialog(dialogIndex, aWindow);
		
		(void) aWindow->PoseModally();
		
		this->CleanupExampleDialog(dialogIndex, aWindow);
		
		aWindow->Close();
		delete aWindow;
	}
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::LaunchGridViewExample()
{
	const OSType gridViewAppSignature = 'SSgv';
	ProcessSerialNumber psn;
							
	::ThrowIfOSErr_AC(::LaunchBySignature(gridViewAppSignature, psn, NULL, NULL, true, launchContinue));
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::SetupExampleDialog(UInt32 dialogIndex, TWindow* itsWindow)
{
	// Note that when we set CControl_AC states & values here, we tell it
	// not to redraw right now, because the dialog isn't visible yet.
	// If we *did* tell it to redraw, we would have to Focus() it first
	// so that the CControl_AC would be drawing into the correct GrafPort.

	switch (dialogIndex)
	{
		case kPushButtonDlgIndex:
			{
				TCheckBox*	mixedCheckBox;
				mixedCheckBox = dynamic_cast_AC(TCheckBox*,itsWindow->FindSubView('BTN5'));
				mixedCheckBox->SetState(kControlCheckboxMixedValue, kRedraw);

				TRadio*	mixedRadioButton;
				mixedRadioButton = dynamic_cast_AC(TRadio*,itsWindow->FindSubView('BTN8'));
				mixedRadioButton->SetState(kControlRadioButtonMixedValue, kRedraw);
			}
			break;

#if qGrayscaleAppearance
		case kScrollBarDlgIndex:
			if (EmulateControls_AC())
			{
				TScrollBarProportional	*aScrollBar = dynamic_cast_AC(TScrollBarProportional*,itsWindow->FindSubView('SBAR'));
				aScrollBar->GetAGAScrollBar()->SetPageSize(25, CControl_AC::kDontRedraw);
				aScrollBar->GetAGAScrollBar()->SetStepSizes(1, 25);
			}
			break;
#endif // qGrayscaleAppearance

		case kSliderDlgIndex:
			break;

		case kDisclosureDlgIndex:
			{
				// Disclosure Behavior example.
				itsWindow->AddBehavior(TH_new TDisclosureBehavior('OKAY', 'DIS1', 20));
			}
			break;
		case kProgressDlgIndex:
			{	
				// Make a call to the Set Values button so the progress indicator values match the fields.
				DoEvent(mButtonHit, itsWindow->FindSubView('SETV'), NULL);
			}
			break;
		case kFolderTabsDlgIndex:
			{
				TTabbedView*	tabPanel;
				
				tabPanel = dynamic_cast_AC(TTabbedView*,itsWindow->FindSubView('TAB1'));
				
				tabPanel->InstallPanel(0, 3000, NULL);
				tabPanel->InstallPanel(1, 3001, NULL);
				tabPanel->InstallPanel(2, 3002, NULL);
			}
			break;
		}
	}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::CleanupExampleDialog(UInt32 dialogIndex, TWindow* /*itsWindow*/)
{
	switch (dialogIndex)
	{
		case kProgressDlgIndex:
			{
				// Null the pointers so we don't go haywire at next DoIdle !

				fProgressIndicator1 = NULL;
				fProgressIndicator2 = NULL;
				fMovingProgressOrigin = false;
				fSkipNext = false;
			}
			break;
	}
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::EnableDisableDialog(UInt32 dialogIndex, TWindow* itsWindow, bool enable)
	{
	// Find all of the appropriate items of the specified dialog,
	// and set their enable/disable/dim state as requested.

	TControl*	aControl;
	TView*		aView;
	
	switch (dialogIndex)
	{
		case kPushButtonDlgIndex:

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN1'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN2'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			// Check boxes
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN3'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN4'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN5'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			// Radio buttons
			

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN6'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN7'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('BTN8'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			// Popup menus
						
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('POP1'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('POP2'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('POP3'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			break;

		case kIconButtonDlgIndex:			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN1'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN2'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN3'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
	
			// Icon Checkboxes
				
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN4'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN5'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN6'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			// Icon Radio buttons
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN7'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN8'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('ICN9'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('IC10'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('IC11'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('IC12'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('IC13'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			// Icon popups
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('POP1'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('POP2'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('POP3'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			break;
			
		case kScrollBarDlgIndex:

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('SBAR'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('HSBR'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('VSBR'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			break;

		case kSliderDlgIndex:

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('VSPL'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('VSPR'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('VSRC'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('HSPL'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('HSPR'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);
			
			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('HSRC'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			break;

		case kPopupMenuDlgIndex:
			break;

		case kGroupBoxDlgIndex:

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GRP1'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GR1A'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R1A1'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R1A2'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GR1B'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R1B1'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R1B2'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GRP2'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GR2A'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R2A1'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R2A2'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GR2B'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R2B1'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R2B2'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('CBX1'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GRP3'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('CBX2'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GR3A'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R3A1'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R3A2'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('CBX3'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

				aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('GR3B'));
				aControl->SetEnable(enable);
				aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R3B1'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

					aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('R3B2'));
					aControl->SetEnable(enable);
					aControl->DimState(! enable, kRedraw);

			break;

		case kFolderTabsDlgIndex:

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('TAB1'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			aControl = dynamic_cast_AC(TControl*,itsWindow->FindSubView('TAB2'));
			aControl->SetEnable(enable);
			aControl->DimState(! enable, kRedraw);

			break;

		case kFocusBorderDlgIndex:

			::ThrowIfNULL_AC(aView = itsWindow->FindSubView('BRD1'));
			aView->SetEnable(enable);
			aView->ForceRedraw();

			::ThrowIfNULL_AC(aView = itsWindow->FindSubView('BRD2'));
			aView->SetEnable(enable);
			aView->ForceRedraw();

			::ThrowIfNULL_AC(aView = itsWindow->FindSubView('BRD3'));
			aView->SetEnable(enable);
			aView->ForceRedraw();

			::ThrowIfNULL_AC(aView = itsWindow->FindSubView('TVW1'));
			aView->SetEnable(enable);
			aView->ForceRedraw();

			::ThrowIfNULL_AC(aView = itsWindow->FindSubView('TVW2'));
			aView->SetEnable(enable);
			aView->ForceRedraw();

			::ThrowIfNULL_AC(aView = itsWindow->FindSubView('TVW3'));
			aView->SetEnable(enable);
			aView->ForceRedraw();

			break;

	}
}

//--------------------------------------------------------------------------------------------------
// TMyApplication
//--------------------------------------------------------------------------------------------------

void TMyApplication::DoAboutBox()
{
	TWindow*		aWindow;

	::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(kAboutBoxViewID, NULL));
	
	(void) aWindow->PoseModally();
	
	aWindow->Close();
	delete aWindow;
}

void TSwapMainWindowCmd::DoIt()
{
	dynamic_cast_AC(TMyApplication*,gApplication)->SwapMainWindow();
}

//--------------------------------------------------------------------------------------------------
//
// To demonstrate the use of a more complex filter, I have swiped the
// method and filter for the MacApp scrolling credits about box,
// and tweaked the filter to comply with the AGA Dialog Manager
// needs.
//
// Changes to the filter (marked with ! n !).
//
// 1. Get the AGA dialog object pointer from the dialog refcon.
// 2 & 3. Use the AGA method call, rather than the raw Dialog Mgr call,
//	to install the new text.
//--------------------------------------------------------------------------------------------------

extern bool hadCreditsStringList;		// does the rsrc 'STR#' == kDefaultCredits exist ?
extern short lastCreditsStringIndex;		// the last CPascalStr_AC in the STR# to be displayed
extern long lastCreditsShownTicks;			// the tickcount when the last Credit was Shown
extern CPascalStrHandle_AC originalText;			// the about box's original text (prior to credits)
extern short waitTicks;					// how long to wait between credits 

static pascal Boolean DoShowAboutAppFilter(DialogRef theDialog,
										   EventRecord* theEvent,
										   short* itemHit);

void TMyApplication::DoStdMAAboutBox()
{
	FailSpaceIsLow();
	::ThrowIfOSErr_AC(MAInteractWithUser());

	CStr255_AC apName;
	GetApplicationName(apName);

	ParamText(apName, gEmptyString, gEmptyString, gEmptyString);// Put Application name in the about box 

	if (GetResource('STR#', kDefaultCredits))
	{
		lastCreditsStringIndex = 1;
		lastCreditsShownTicks = TickCount();
		waitTicks = 5 * 60;
		originalText = (CPascalStrHandle_AC)NewString(gEmptyString);
		::ThrowIfNULL_AC(originalText);
		MacAppAlert(phAboutApp, DoShowAboutAppFilter);
		originalText = (CPascalStrHandle_AC)::DisposeIfHandle_AC((Handle)originalText);
	}
	else
		StdAlert(phAboutApp);
}

#undef Inherited

//----------------------------------------------------------------------------------------
// DoShowAboutAppFilter: 
//----------------------------------------------------------------------------------------


pascal Boolean DoShowAboutAppFilter(DialogRef theDialog,
									EventRecord* theEvent,
									short* itemHit)
{
	
	CStr255_AC originalStr;
	Handle item;
	bool returnValue = false;
	short itemType;
	CRect_AC box;

	CDialog_AC*	theAGADialog = (CDialog_AC*) ::GetWRefCon(theDialog);	// ! 1 !

	switch (theEvent->what)
	{
		case keyDown:
			switch (static_cast<unsigned char>(theEvent->message & charCodeMask))
			{
				case chEnter:
				case chReturn:
					DoAlertKeyDown(theDialog, ok);
					break;
			}
			break;

		case nullEvent:
			if ((TickCount() - lastCreditsShownTicks) > waitTicks)
			{
				short itemNo = 1;

				do
				{
					item = NULL;
					GetDialogItem(theDialog, itemNo, &itemType, &item, box);
					if (((itemType) & 0x7F) == statText)// we don't care if its enabled or not 
						break;
					else
						++itemNo;
				} while (item);

				CStr255_AC s(kDefaultCredits, lastCreditsStringIndex);

				if (!s.IsEmpty())
				{
					// save the original text 
					if ((lastCreditsStringIndex == 1) && ((*originalText)->IsEmpty() && item))
					{
						GetDialogItemText(item, originalStr);
						SetString((StringHandle)originalText, (ConstStr255Param) & originalStr);
					}
					++lastCreditsStringIndex;
					lastCreditsShownTicks = TickCount();
					if (item)
						//SetDialogItemText(item, s);
						theAGADialog->SetDialogItemText(item, s);	// ! 2 !
					waitTicks = Min_AC(static_cast<short>(s.Length() * 6), 60);
				}
				else							// no more items 
					{
						lastCreditsStringIndex = 1;
						lastCreditsShownTicks = TickCount();
						if (item)
						{
							originalStr = **originalText;
							//SetDialogItemText(item, originalStr);
							theAGADialog->SetDialogItemText(item, originalStr);	// ! 3 !
						}
						waitTicks = 6 * 60;
					}
			}
			break;
	}

	// Forward on to the standard filter 
	if (gMacAppAlertFilter)
		returnValue = gMacAppAlertFilter(theDialog, theEvent, itemHit);

	return returnValue;
}
