//----------------------------------------------------------------------------------------
// UDocumentTidyHeap.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------



#ifndef __UDocumentTidyHeap__
#include "UDocumentTidyHeap.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#ifndef __CFileStream_AC__
#include "CFileStream_AC.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif


enum eCommand
{
	cSetToDefault = 400
};


//----------------------------------------------------------------------------------------
// operator <<
//----------------------------------------------------------------------------------------

inline CStream_AC& operator << (CStream_AC& inStream, const TPrefsFormat& inFormat)
{
	inStream 	<< inFormat.IsEnabled
				<< inFormat.WantsWarnings
				<< inFormat.WantsUndeletedBlockWarning
				<< inFormat.DoLogUndeletedBlocks
				<< inFormat.PointersLivesInAppHeap
				
				<< inFormat.InstallGarbageIn
				<< inFormat.InstallMakeNewFail
				<< inFormat.InstallNewVerifyAction
				<< inFormat.InstallNewVerifyAllAction				
				<< inFormat.InstallNewBreakOnUnknownBlock
				<< inFormat.InstallIncrementalFailer
				<< inFormat.InstallExemptUnknownBlocks
				
				<< inFormat.InstallGarbageOut
				<< inFormat.InstallDeleteBreakOnUnknownBlock
				<< inFormat.InstallDoubleDeleteWatcher
				<< inFormat.InstallDeleteVerifyAllAction
				<< inFormat.InstallDeleteVerifyAction;

	inStream.WriteBytes(inFormat.fSessionName, TPrefsFormat::kStringLen);

	inStream 	<< inFormat.fFailCount
				<< inFormat.SettingsModifiedExternally
				<< inFormat.fLastWriteSeconds;

	return inStream;
}

//----------------------------------------------------------------------------------------
// operator >>
//----------------------------------------------------------------------------------------

inline CStream_AC& operator >> (CStream_AC& inStream, TPrefsFormat& inFormat)
{
	inStream 	>> inFormat.IsEnabled
				>> inFormat.WantsWarnings
				>> inFormat.WantsUndeletedBlockWarning
				>> inFormat.DoLogUndeletedBlocks
				>> inFormat.PointersLivesInAppHeap
				
				>> inFormat.InstallGarbageIn
				>> inFormat.InstallMakeNewFail
				>> inFormat.InstallNewVerifyAction
				>> inFormat.InstallNewVerifyAllAction				
				>> inFormat.InstallNewBreakOnUnknownBlock
				>> inFormat.InstallIncrementalFailer
				>> inFormat.InstallExemptUnknownBlocks
				
				>> inFormat.InstallGarbageOut
				>> inFormat.InstallDeleteBreakOnUnknownBlock
				>> inFormat.InstallDoubleDeleteWatcher
				>> inFormat.InstallDeleteVerifyAllAction
				>> inFormat.InstallDeleteVerifyAction;
	
	inStream.ReadBytes(inFormat.fSessionName, TPrefsFormat::kStringLen);

	inStream 	>> inFormat.fFailCount
				>> inFormat.SettingsModifiedExternally
				>> inFormat.fLastWriteSeconds;

	return inStream;
}


//----------------------------------------------------------------------------------------
// Constants:

const ResNumber kTidyHeapWindowID = kDefaultWindowID;

//========================================================================================
// CLASS TDocumentTidyHeap
//========================================================================================

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap constructor 
//----------------------------------------------------------------------------------------

TDocumentTidyHeap::TDocumentTidyHeap(TFile* inFile) 
	: TFileBasedDocument(inFile),
	fFailCountText(NULL)
{
	Erase_AC(fBoxes, kNumCheckBoxes);

	fSavePrintInfo = false;
		
} // TDocumentTidyHeap::TDocumentTidyHeap 

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap destructor
//----------------------------------------------------------------------------------------

TDocumentTidyHeap::~TDocumentTidyHeap()
{
}

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::DoNeedDiskSpace
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::DoNeedDiskSpace(TFile* /*itsFile*/,
								 long& dataForkBytes,
								 long& /*rsrcForkBytes*/)
{
	dataForkBytes = TPrefsFormat::kStringLen + 
					(sizeof(CProperty_AC) * TPrefsFormat::kNumProperties) +
					(sizeof(long) * 2);
}

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::DoRead
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::DoRead(TFile* aFile, bool /*forPrinting*/)
{
	CFileStream_AC fileStream(aFile);
	fileStream 	>> fSettings;
}

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::DoWrite
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::DoWrite(TFile* aFile, bool /*makingCopy*/)
{
// sync checkboxes up with data	
	fSettings.IsEnabled(fBoxes[kEnabled]->IsOn());
	fSettings.WantsWarnings(fBoxes[kWarnings]->IsOn());
	fSettings.WantsUndeletedBlockWarning(fBoxes[kUndeletedBlocks]->IsOn());
	fSettings.DoLogUndeletedBlocks(fBoxes[kLogBlocks]->IsOn());
	fSettings.PointersLivesInAppHeap(fBoxes[kPointersInAppHeap]->IsOn());
	
	fSettings.InstallGarbageIn(fBoxes[kInstallGarbageIn]->IsOn());
	fSettings.InstallGarbageOut(fBoxes[kInstallGarbageOut]->IsOn());
	fSettings.InstallDeleteVerifyAction(fBoxes[kInstallDeleteVerify]->IsOn());

	fSettings.InstallMakeNewFail(fBoxes[kInstallMakeNewFail]->IsOn());				
	fSettings.InstallIncrementalFailer(fBoxes[kInstallIncrementalNewFailer]->IsOn());
	fSettings.InstallNewVerifyAction(fBoxes[kInstallNewVerifyAction]->IsOn());
	fSettings.InstallNewVerifyAllAction(fBoxes[kInstallNewVerifyAllAction]->IsOn());
	fSettings.InstallExemptUnknownBlocks(fBoxes[kInstallExemptUnknownBlocks]->IsOn());

	fSettings.InstallNewBreakOnUnknownBlock(fBoxes[kInstallNewBreakOnUnknownBlock]->IsOn());
	
		 // Global Delete Actions
	fSettings.InstallDeleteBreakOnUnknownBlock(fBoxes[kInstallDeleteBreakOnUnknownBlock]->IsOn());
	fSettings.InstallDeleteVerifyAllAction(fBoxes[kInstallDeleteVerifyAllAction]->IsOn());
	fSettings.InstallDoubleDeleteWatcher(fBoxes[kInstallDoubleDeleteWatcher]->IsOn());
	
	fSettings.fFailCount = fFailCountText->GetValue();

	fSettings.SettingsModifiedExternally(true);
	GetDateTime(&fSettings.fLastWriteSeconds);	
	// stream data to the file

	CFileStream_AC fileStream(aFile);
	
	fileStream << fSettings;

}

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::DoMakeViews: 
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::DoMakeViews(bool /*forPrinting*/)
{
	TWindow* aWindow = gViewServer->NewTemplateWindow(kTidyHeapWindowID, this);
	::ThrowIfNULL_AC(aWindow);
	
	CStr255_AC theText = fSettings.fSessionName;

	TStaticText *staticText =  (TStaticText*) aWindow->FindSubView('targ');
	::ThrowIfNULL_AC(staticText);
	
	staticText->SetText( theText, false );
	
	for(short i = 0; i < kNumCheckBoxes; i++)
	{
		fBoxes[i] = dynamic_cast_AC(TCheckBox*, aWindow->FindSubView(i));
		::ThrowIfNULL_AC(fBoxes[i]);
	}
	
	fFailCountText = (TNumberText*) aWindow->FindSubView('NUM1');
	::ThrowIfNULL_AC(fFailCountText);
	
	SyncViewsWithData(false);
	
	aWindow->Open();
} // TDocumentTidyHeap::DoMakeViews 



//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::DoMenuCommand
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber) 
	{
		case cSetToDefault: 
			SetToDefault();
			break;
		default:
			TFileBasedDocument::DoMenuCommand(aCommandNumber);
			break;
	}
} // TDocumentTidyHeap::DoMenuCommand 


//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::DoSetupMenus
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::DoSetupMenus()
{
	TFileBasedDocument::DoSetupMenus();
	
	Enable(cSetToDefault,TRUE);
	Enable(cSaveAs, FALSE);
} // TDocumentTidyHeap::DoSetupMenus 

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::DoEvent
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event)
{
	if(	eventNumber == mCheckBoxHit/* || eventNumber == 27*/ )
	{
		fChangeCount++;
	}

	TFileBasedDocument::DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap::SyncViewsWithData
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::SyncViewsWithData(bool drawNow)
{
	fBoxes[kEnabled]->SetState( fSettings.IsEnabled() , drawNow);
	fBoxes[kWarnings]->SetState( fSettings.WantsWarnings() , drawNow);
	fBoxes[kUndeletedBlocks]->SetState( fSettings.WantsUndeletedBlockWarning() , drawNow);
	fBoxes[kLogBlocks]->SetState( fSettings.DoLogUndeletedBlocks() , drawNow);
	fBoxes[kPointersInAppHeap]->SetState( fSettings.PointersLivesInAppHeap() , drawNow);
	
	fBoxes[kInstallGarbageIn]->SetState( fSettings.InstallGarbageIn() , drawNow);
	fBoxes[kInstallGarbageOut]->SetState( fSettings.InstallGarbageOut() , drawNow);
	fBoxes[kInstallDeleteVerify]->SetState( fSettings.InstallDeleteVerifyAction() , drawNow);
	fBoxes[kInstallMakeNewFail]->SetState( fSettings.InstallMakeNewFail() , drawNow);
	fBoxes[kInstallIncrementalNewFailer]->SetState( fSettings.InstallIncrementalFailer() , drawNow);
	fBoxes[kInstallNewVerifyAction]->SetState( fSettings.InstallNewVerifyAction() , drawNow);
	fBoxes[kInstallNewVerifyAllAction]->SetState( fSettings.InstallNewVerifyAllAction() , drawNow);
	fBoxes[kInstallExemptUnknownBlocks]->SetState( fSettings.InstallExemptUnknownBlocks() , drawNow);
	fBoxes[kInstallNewBreakOnUnknownBlock]->SetState( fSettings.InstallNewBreakOnUnknownBlock() , drawNow);
	fBoxes[kInstallDeleteBreakOnUnknownBlock]->SetState( fSettings.InstallDeleteBreakOnUnknownBlock() , drawNow);
	fBoxes[kInstallDeleteVerifyAllAction]->SetState( fSettings.InstallDeleteVerifyAllAction() , drawNow);
	fBoxes[kInstallDoubleDeleteWatcher]->SetState( fSettings.InstallDoubleDeleteWatcher() , drawNow);

	fFailCountText->SetValue( fSettings.fFailCount, drawNow );
}

//----------------------------------------------------------------------------------------
//  TDocumentTidyHeap::SetToDefault
//----------------------------------------------------------------------------------------

void TDocumentTidyHeap::SetToDefault()
{	
	fChangeCount++;
		// dirty the document
	
	fSettings.fFailCount = 0;

	fSettings.IsEnabled(true);
	fSettings.WantsWarnings(true);
	fSettings.WantsUndeletedBlockWarning(true);
	fSettings.DoLogUndeletedBlocks(true);
	fSettings.PointersLivesInAppHeap(true);

	fSettings.InstallGarbageIn(true);
	fSettings.InstallGarbageOut(true);
	fSettings.InstallDeleteVerifyAllAction(true);
		// trues

	fSettings.InstallMakeNewFail(false);				
	fSettings.InstallIncrementalFailer(false);	
	fSettings.InstallNewVerifyAction(false);	
	fSettings.InstallNewVerifyAllAction(false);
	fSettings.InstallExemptUnknownBlocks(false);
	fSettings.InstallNewBreakOnUnknownBlock(false);
	fSettings.InstallDeleteBreakOnUnknownBlock(false);
	fSettings.InstallDoubleDeleteWatcher(false);
		// falses
	
	fSettings.SettingsModifiedExternally(false);

	SyncViewsWithData(true);
		// update the checkboxes
}


