//----------------------------------------------------------------------------------------
// UDocumentTidyHeap.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __UDocumentTidyHeap__
#define __UDocumentTidyHeap__

#ifndef __UFILEBASEDDOCUMENT__
#include "UFileBasedDocument.h"
#endif

#ifndef __CProperty_AC__
#include "CProperty_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

// Commands
const CommandNumber cCommandHandledByDocument = 401;

// Files
//const OSType kFileType = 'SF01';	// File-type code used for document files 

//----------------------------------------------------------------------------------------
// TDocumentTidyHeap
//----------------------------------------------------------------------------------------

class TFile;

const OSType kFileType 	=  'thpf';	// File-type code used for document files 
const OSType kSignature	 = 'thap';	// Application signature


//----------------------------------------------------------------------------------------
// class TPrefsFormat
//----------------------------------------------------------------------------------------

class TPrefsFormat
{
public:
	enum { 	kStringLen = 32,
			kNumProperties = 18 };
	
	TPrefsFormat(): fFailCount(0)
	{ fSessionName[0] = '\0'; }
	
	CProperty_AC IsEnabled;							// 0
	CProperty_AC WantsWarnings;						// 1
	CProperty_AC WantsUndeletedBlockWarning;		// 2
	CProperty_AC DoLogUndeletedBlocks;				// 3
	CProperty_AC PointersLivesInAppHeap;			// 4
	
	CProperty_AC InstallGarbageIn; 					// 5
	CProperty_AC InstallMakeNewFail;				// 8	
	CProperty_AC InstallNewVerifyAction; 			// 10
	CProperty_AC InstallNewVerifyAllAction;			// 12
	CProperty_AC InstallNewBreakOnUnknownBlock;		// 14
	CProperty_AC InstallIncrementalFailer; 			// 9
	CProperty_AC InstallExemptUnknownBlocks;		// 13
	
	CProperty_AC InstallGarbageOut;					// 6
	CProperty_AC InstallDeleteBreakOnUnknownBlock;	// 15
	CProperty_AC InstallDoubleDeleteWatcher;		// 17
	CProperty_AC InstallDeleteVerifyAllAction;		// 16
	CProperty_AC InstallDeleteVerifyAction; 		// 7
	
		 // Global Delete Actions

	char fSessionName[kStringLen];
	long fFailCount;

	CProperty_AC SettingsModifiedExternally;		// 18
	unsigned long fLastWriteSeconds;
};


//----------------------------------------------------------------------------------------
// class TDocumentTidyHeap
//----------------------------------------------------------------------------------------

class TDocumentTidyHeap: public TFileBasedDocument
{
public:
	TDocumentTidyHeap(TFile* itsFile); 
	virtual ~TDocumentTidyHeap();
	
	// Views
	virtual void  DoMakeViews(bool forPrinting);
	
	
	virtual void DoNeedDiskSpace(TFile* itsFile,
								 long& dataForkBytes,
								 long& rsrcForkBytes);
	virtual void DoRead(TFile* aFile, bool forPrinting);
	virtual void DoWrite(TFile* aFile, bool makingCopy);
	
	virtual void DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event);
	
	// Commands
	virtual void  DoSetupMenus();
	virtual void  DoMenuCommand(CommandNumber aCommandNumber);

private:
	void SyncViewsWithData(bool drawNow);
	void SetToDefault();
		
private:
	
		
	enum 
	{
		// column 1
		kEnabled,
		kWarnings,
		kUndeletedBlocks,
		kLogBlocks,
		kPointersInAppHeap,
		
		// column 2
		kInstallIncrementalNewFailer,
		kInstallNewVerifyAction,
		kInstallNewVerifyAllAction,
		kInstallMakeNewFail,
		kInstallExemptUnknownBlocks,
		kInstallGarbageIn,
		kInstallNewBreakOnUnknownBlock,
		
		// column 3
		kInstallDoubleDeleteWatcher,
		kInstallDeleteVerify,
		kInstallDeleteVerifyAllAction,
		kInstallGarbageOut,
		kInstallDeleteBreakOnUnknownBlock,
		kNumCheckBoxes
	};
	
	TCheckBox* fBoxes[kNumCheckBoxes];
	
	TPrefsFormat fSettings;
	TNumberText* fFailCountText;
};

#endif
