//----------------------------------------------------------------------------------------
// CDNRSampleApp.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CDNRSampleApp__
#include "CDNRSampleApp.h"
#endif

// MacApp

#ifndef __ULogWindow__
#include "ULogWindow.h"
#endif

#ifndef __Utils__
#include "Utils.h"
#endif

// patterns

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEventWithData_AC__
#include "CEventWithData_AC.h"
#endif

// networking

#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CInternetUtils_AC__
#include "CInternetUtils_AC.h"
#endif

#ifndef __CInternetHost_AC__
#include "CInternetHost_AC.h"
#endif

#ifndef __NetGlobals_AC__
#include "CNetGlobals_AC.h"
#endif

// streams

#ifndef __CBufferStream_AC__
#include "CBufferStream_AC.h"
#endif

#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif

// MacApp Includes

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

//----------------------------------------------------------------------------------------
// Constants and globals
//----------------------------------------------------------------------------------------

enum eCommands
{
	cResolveAddress = 400
};

//----------------------------------------------------------------------------------------
// TStarter
//----------------------------------------------------------------------------------------

class TStarter : public TCommand
{
public:
	TStarter(CDNRSampleApp* app) 
		: 	TCommand(0, NULL, false, false, NULL), 
			fApp(app) 
		{}

	virtual void DoIt() { fApp->Start(); }

private:
	CDNRSampleApp* fApp;
};

//----------------------------------------------------------------------------------------
// CDNRSampleApp constructor
//----------------------------------------------------------------------------------------

CDNRSampleApp::CDNRSampleApp()
	:	TApplication('????', '????')
{
	fLaunchWithNewDocument = false;
	
	CViewRect frame(10,50,450, 300);
	
	fTrace = TH_new TLogWindow;
	fTrace->ReadViewsFromResource();
	fTrace->GetWindow()->SetFrame(frame, false);
	fTrace->GetWindow()->SetTitle(CStr255_AC("Event Log"));
	fTrace->Show();

	frame.Offset(0,250);

	fLog = TH_new TLogWindow;
	
	fLog->ReadViewsFromResource();
	
	TWindow* window = fLog->GetWindow();
	
	window->SetFrame(frame, false);
	window->SetTitle(CStr255_AC("Results"));
	window->Open();
		
	PostCommand(TH_new TStarter(this));
	
	CNetGlobals_AC::SetDefaultTimeOut(900);
		// timeout after 15 seconds.
}

//----------------------------------------------------------------------------------------
// CDNRSampleApp destructor
//----------------------------------------------------------------------------------------

CDNRSampleApp::~CDNRSampleApp()
{
	CTraceStream_AC::RemoveStream(*fTrace);
		// turn off trace
}

//----------------------------------------------------------------------------------------
// CDNRSampleApp::Start:
//----------------------------------------------------------------------------------------

void CDNRSampleApp::Start()
{
	CTraceStream_AC::AddStream( *fTrace );
		// hook our log window into the trace mechanism.
	
	while (!InitInternetSuite_AC()) 
	{
	}
	
	CTempInternetHost_AC machineInfo = CInternetAddress_AC::GetLocalAddress();
	machineInfo.ResolveAddressToNameSync();
	
	*fLog 	<< "Host Name: " 
			<< machineInfo.GetHostName() 
			<< "\nHost Address: "
			<< CInternetAddress_AC::GetLocalAddress().GetAsString() 
			<< endl_AC;
}

//----------------------------------------------------------------------------------------
// CDNRSampleApp::DoSetupMenus
//----------------------------------------------------------------------------------------

void CDNRSampleApp::DoSetupMenus() 
{
	TApplication::DoSetupMenus();		

	Enable(cNew, false);
	Enable(cOpen,false);
	Enable(cResolveAddress, true);
}

//----------------------------------------------------------------------------------------
// CDNRSampleApp::DoMenuCommand
//----------------------------------------------------------------------------------------

void CDNRSampleApp::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		case cResolveAddress:
			ResolveAddress();
		break;
	
		default:
			TApplication::DoMenuCommand(aCommandNumber);
		break;
	}

} 

//----------------------------------------------------------------------------------------
// CDNRSampleApp::ResolveAddress
//----------------------------------------------------------------------------------------

void CDNRSampleApp::ResolveAddress()
{
	CStr255_AC str;
	
	if(GetSomeTextFromUser("Enter Address or Name to Resolve:", str, "www.apple.com"))
	{
		CInternetHost_AC* host = CreateInternetHost();
				
		if(CInternetAddress_AC::IsIPAddress(str))
		{	
			CInternetAddress_AC address(str);
			
			host->SetAddress(address);
			host->ResolveAddressToNameAsync();
		}
		else
		{
			host->SetHostName(str);
			host->ResolveNameToAddressAsync();
		}	
	}
}

//----------------------------------------------------------------------------------------
// CDNRSampleApp::AddressToNameResolved
//----------------------------------------------------------------------------------------

void CDNRSampleApp::AddressToNameResolved(CInternetHost_AC* inHost)
{
	*fLog << endl_AC;
	
	switch(	inHost->GetState())
	{
		case CInternetHost_AC::kResolvedAddressToName:
			*fLog 	<< inHost->GetAddress().GetAsString() 
					<< " resolves to: \n" 
					<< "    "
					<< inHost->GetHostName()
					<< endl_AC;
		break;
		
		case CInternetHost_AC::kTimedOut:
			*fLog 	<< inHost->GetAddress().GetAsString() 
					<< ": Address To Name Timed Out." 
					<< endl_AC;
		break;
		
		case CInternetHost_AC::kError:
			*fLog 	<< inHost->GetAddress().GetAsString() 
					<< " failed to resolve. Error #" 
					<< inHost->GetError()
					<< endl_AC;
		break;
	}
}

//----------------------------------------------------------------------------------------
// CDNRSampleApp::NameToAddressResolved
//----------------------------------------------------------------------------------------

void CDNRSampleApp::NameToAddressResolved(CInternetHost_AC* inHost)
{
	*fLog << endl_AC;
		
	switch(	inHost->GetState())
	{
		case CInternetHost_AC::kResolvedNameToAddress:
			*fLog << inHost->GetHostName() << " resolves to:" << endl_AC;
		
			for(short i = 0; i < inHost->GetNumAddresses(); ++i)
			{
				CInternetAddress_AC address = inHost->GetAddress(i);
				const char* addStr = address.GetAsString();
				*fLog << "    " << addStr << endl_AC;
			}	
		break;
		
		case CInternetHost_AC::kTimedOut:
			*fLog 	<< inHost->GetHostName() 
					<< ":Name to Address Timed Out." 
					<< endl_AC;
		break;
		
		case CInternetHost_AC::kError:
			*fLog 	<< inHost->GetHostName() 
					<< " failed to resolve. Error #" 
					<< inHost->GetError()
					<< endl_AC;
		break;
	}
}

