//----------------------------------------------------------------------------------------
// CDropenerApp.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------



#define qTestDroppedFile 0
	// define this to one to test what happens when a file is dropped on this app
#define qDoQD3D 0
	// define this to one to enable dropener to handle qd3d files.

#ifndef __CDropenerApp__
#include "CDropenerApp.h"
#endif

// Shared Net Example files.
#ifndef __ULogWindow__
#include "ULogWindow.h"
#endif

#ifndef __Utils__
#include "Utils.h"
#endif


// pattern includes
#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif


// text includes
#ifndef __CBufferStream_AC__
#include "CBufferStream_AC.h"
#endif


// network includes
#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __CFileSender_AC__
#include "CFileSender_AC.h"
#endif

#ifndef __CFileReceiver_AC__
#include "CFileReceiver_AC.h"
#endif

#ifndef __DisconnectTransactions_AC__
#include "DisconnectTransactions_AC.h"
#endif

#ifndef __CMultiTCPConnectionMgr_AC__
#include "CMultiTCPConnectionMgr_AC.h"
#endif

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __CTCPConnection_AC__
#include "CTCPConnection_AC.h"
#endif

#ifndef __CDataReceiver_AC__
#include "CDataReceiver_AC.h"
#endif

#ifndef __CInternetUtils_AC__
#include "CInternetUtils_AC.h"
#endif


// streams
#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif


// MacApp Includes

#ifndef __UQuickTime__
#include "UQuickTime.h"
#endif

#ifndef __UFile__
#include "UFile.h"
#endif

#ifndef __UProgressIndicator__
#include "UProgressIndicator.h"
#endif


#if qDoQD3D
#include "UQD3DViewer.h"
#include "UQD3DCommon.h"
#endif

// dropener files

#ifndef __CMyProgressWindow__
#include "CMyProgressWindow.h"
#endif



// command id's from the TRACE menu.
enum 
{
	cLow = 400,
	cMed,
	cHigh,
	cNone
};

const long kDropenerPort = 5555;
	// This is the port Dropener will listen on for connection requests.

//----------------------------------------------------------------------------------------
// class TStarter
//----------------------------------------------------------------------------------------

// TStarter just calls the App's Start method because the logging windows
// needs to be fully created before anything is logged to it.

class TStarter : public TCommand
{
public:
	TStarter(CDropenerApp* app) 
		: 	TCommand(0, NULL, false, false, NULL), 
			fApp(app) 
		{}

	virtual void DoIt() { fApp->Start(); }

private:
	CDropenerApp* fApp;
};

//----------------------------------------------------------------------------------------
// CDropenerApp ctor
//----------------------------------------------------------------------------------------

CDropenerApp::CDropenerApp()
	:	TApplication('????', '????'),
	fDefaultIPAddress("0.0.0.0")
{
	fLaunchWithNewDocument = false;
		// This app has no documents.
		
		
#if qTrace_AC	
	CViewRect frame(10,50,400, 500);
	fLog = TH_new TLogWindow;
	fLog->ReadViewsFromResource();
	fLog->GetWindow()->SetFrame(frame, false);
	fLog->GetWindow()->SetTitle(CStr255_AC("Event Log"));
	fLog->Show();

	frame.Offset(410, 0);
#endif
		
	TStarter* starter = TH_new TStarter(this);
	PostCommand(starter);
	
	// This is where we make the connection manager. It will automatically
	// get idle time from a thread so each connection will run in its 
	// own thread.	
	fConnections = TH_new CMultiTCPConnectionManager_AC;
	RegisterForEventsFrom(fConnections);

// init misc
	InitUProgressIndicator();
	
// init quicktime
	InitUQuickTime();
	MA_REGISTER_CLASS(TMovieView);
	MA_REGISTER_CLASS(TMovieController);
		
// init qd3d	

#if qDoQD3D
	InitUQD3DViewer();
#endif
}

//----------------------------------------------------------------------------------------
// CDropenerApp dtor
//----------------------------------------------------------------------------------------

CDropenerApp::~CDropenerApp()
{
#if qTrace_AC
	CTraceStream_AC::RemoveStream(*fLog);
		// turn off trace
#endif
}

//----------------------------------------------------------------------------------------
// CDropenerApp::Start()
//----------------------------------------------------------------------------------------

void CDropenerApp::Start()
{
	while(!InitInternetSuite_AC())
	{}
	
#if qTrace_AC	
	CTraceStream_AC::AddStream( *fLog );
		// hook our log window into the trace mechanism.

	TRACE.SetPriorityFilter(CTraceStream_AC::kHighPriority);

	TRACE << "Listening to Port #" << kDropenerPort << endl_AC;
#endif
							
	
	TWindow* window = gViewServer->NewTemplateWindow(1007, NULL);
	::ThrowIfNULL_AC(window);

	TStaticText* staticText =  dynamic_cast_AC(TStaticText*,window->FindSubView('addr'));

	CStr255_AC str = CInternetAddress_AC::GetLocalAddress().GetAsString();

	staticText->SetText(str, false);
	window->Open();

	fConnections->ListenTo(kDropenerPort);
}

//----------------------------------------------------------------------------------------
// CDropenerApp::ReceiveData()
//----------------------------------------------------------------------------------------

void CDropenerApp::ReceiveData(	CServiceHandler_AC* inServiceHandler, 
									CNetworkTransaction_AC* inTransaction)
{
	// Update our progress window. When the transfer is complete this will kill the progress window.

	CMyProgressWindow::Update(inServiceHandler, inTransaction->GetTransferedAmount(), inTransaction->GetExpectedTransferAmount());

	// If the transaction is complete, make a new TFile for the application to open if it can.
	if( inTransaction->GetTransferedAmount() == inTransaction->GetExpectedTransferAmount())
	{
		CFileReceiver_AC* fileReceiver = dynamic_cast_AC(CFileReceiver_AC*,inTransaction);
		
		TFile* file = TH_new TFile;
		file->Specify(fileReceiver->GetFileSpec());
		HandleReceivedFile(file);
	}
}

//----------------------------------------------------------------------------------------
// CDropenerApp::ReceiveData()
//----------------------------------------------------------------------------------------

void CDropenerApp::RespondToDataSent(	CServiceHandler_AC* inServiceHandler, 
										CNetworkTransaction_AC* inTransaction)
{
	// All we need to do is update our progress window.
	CMyProgressWindow::Update(inServiceHandler, inTransaction->GetTransferedAmount(), inTransaction->GetExpectedTransferAmount());

}

//----------------------------------------------------------------------------------------
// CDropenerApp::AcceptConnection()
//----------------------------------------------------------------------------------------
			
void CDropenerApp::RespondToAcceptorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler)
{
	// Start a progress window when we accept a connection.
	if(inEvent->GetEventID() == CNetEvent_AC::kAcceptSucceeded)
	{
		TH_new CMyProgressWindow(inHandler, false);
	}
}

//----------------------------------------------------------------------------------------
// CDropenerApp::RespondToConnectorEvent()
//----------------------------------------------------------------------------------------

void CDropenerApp::RespondToConnectorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler)
{
	if(inEvent->GetEventID() != CNetEvent_AC::kConnectSucceeded)
	{
		// Kill the progress window
		CMyProgressWindow::Update(inHandler, 0, 0);
		// Let our user know that the connection attempt failed
		StdAlert(500);
	}
}


//----------------------------------------------------------------------------------------
// CDropenerApp::PrepareToReceiveData()
//----------------------------------------------------------------------------------------

void  CDropenerApp::PrepareToReceiveData(CServiceHandler_AC* inHandler)
{
	
	CFSSpec_AC spec;
	
	// we're going to make our received file on the desktop so find the desktop folder info.
	OSErr err = FindFolder(	kOnSystemDisk, 
							kDesktopFolderType, 
							kDontCreateFolder, 
							&spec.vRefNum, 
							&spec.parID);
	::ThrowIfError_AC(err);
	
	// Make the file receiver, notice that we're receving the entire file.
	CFileReceiver_AC* receiver = TH_new CFileReceiver_AC(spec, CFileReceiver_AC::kEntireFile);
	
	// Queue up the net transaction for processing
	inHandler->Receive(receiver);

	// When all the files are recieved, do an orderly disconnect.
	// Queue up a disconnect transaction. This will execute once the file receiver has
	// completed.
	inHandler->Send(TH_new CDisconnect_AC);
}

//----------------------------------------------------------------------------------------
// CDropenerApp::DoMenuCommand()
//----------------------------------------------------------------------------------------

void CDropenerApp::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	
	switch (aCommandNumber) 
	{

#if qTrace_AC
		case cLow:	
			TRACE.SetPriorityFilter(CTraceStream_AC::kLowPriority);
		break;
		case cMed:
			TRACE.SetPriorityFilter(CTraceStream_AC::kMedPriority);
		break;			
		case cHigh:
			TRACE.SetPriorityFilter(CTraceStream_AC::kHighPriority);
		break;			
		
		case cNone:
			TRACE.SetPriorityFilter(CTraceStream_AC::kOff);
		break;	
#else

		case cLow:	
		case cMed:
		case cHigh:
		case cNone:
		break;	

#endif			
		default:
			TApplication::DoMenuCommand(aCommandNumber);
			break;
	}
} 

//----------------------------------------------------------------------------------------
// CDropenerApp::DoSetupMenus()
//----------------------------------------------------------------------------------------

void CDropenerApp::DoSetupMenus() 
{
	TApplication::DoSetupMenus();		

	Enable(cNew, false);
//	Enable(cOpen,false);

#if qTrace_AC
	EnableCheck(cLow, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kLowPriority);
	EnableCheck(cMed, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kMedPriority);
	EnableCheck(cHigh, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kHighPriority);
	EnableCheck(cNone, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kOff );
#else
	Enable(cLow, false);
	Enable(cMed, false);
	Enable(cHigh, false);
	Enable(cNone, false);
#endif
}

//----------------------------------------------------------------------------------------
// CDropenerApp::OpenNew()
//----------------------------------------------------------------------------------------

TDocument* CDropenerApp::OpenNew(CommandNumber /*itsCommandNumber*/)
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// CDropenerApp::OpenOld()
//----------------------------------------------------------------------------------------

TDocument* CDropenerApp::OpenOld(CommandNumber /*itsOpenCommand*/, CList_AC* aFileList)
{
	if(!gDispatcher->GetDone())
	{		

#if !qTestDroppedFile
		long localPort = kDropenerPort+1; // GetANumberFromUser("Local port?");
		long remotePort = kDropenerPort; // GetANumberFromUser("Remote port?");
		CStr255_AC remoteAddressStr;
		 
		// get the ip address or host name to connect to 
		if(GetSomeTextFromUser("Connect to? [No response or zero connects to self]", remoteAddressStr, fDefaultIPAddress))
		{
			if(remoteAddressStr == "" || remoteAddressStr == "0")
			{
				remoteAddressStr = "0.0.0.0";
			}
			
			// save the address or host name
			fDefaultIPAddress = remoteAddressStr;
			
			// iterate through the files
			TemplateIterator_AC<TFile*> iter(aFileList);
			TFile * aFile = iter.First();
			
			while (aFile)
			{
			
				// start a new connection. If remoteAddressStr is a host name it will attempt 
				// to resolve the connection before connecting.
				CRefCountingPtr_AC<CServiceHandler_AC> newConnection = 
					fConnections->ConnectTo(remoteAddressStr, remotePort );
				
				// Make a new sender. Give it the file spec of the file we're dealiing with
				// in the iteration.
				CFileSender_AC* sender = TH_new CFileSender_AC(
					aFile->GetFileSpec(), 	
					CFileSender_AC::kEntireFile);
				
				// Install a file sender. This will start sending the file once the connection
				// is made and opened.
				newConnection->Send(sender);
				
				// Make the progress window
				TH_new CMyProgressWindow(newConnection, true);

				aFile = iter.Next();
			}
		}	
#else
			TemplateIterator_AC<TFile*> iter(aFileList);
			TFile * aFile = iter.First();
			
			while (aFile)
			{
				TFile* fileCopy = TH_new TFile;
				fileCopy->Specify(aFile->GetFileSpec());
				HandleReceivedFile(fileCopy);

				aFile = iter.Next();
			}

#endif					
	}

	
	return NULL;
}

//----------------------------------------------------------------------------------------
// CDropenerApp::HandleReceivedFile()
//----------------------------------------------------------------------------------------

void CDropenerApp::HandleReceivedFile(TFile* file)
{
	OSType type = 0;
	ThrowIfError_AC(file->GetFileType(type));

	file->SetUsesDataFork(true);
	file->SetUsesRsrcFork(preferResourceFork);
	
	switch(type)
	{
		case 'MooV':
			PlayMovie(file);
		break;

#if qDoQD3D		
		case kMac3DMFFileType:
			OpenQD3DObject(file);
		break;
#endif
		default:
			delete file;
		break;	
	}
	
}

//----------------------------------------------------------------------------------------
// CDropenerApp::OpenQD3DObject()
//----------------------------------------------------------------------------------------


void CDropenerApp::OpenQD3DObject(TFile* inFile)
{
#if qDoQD3D
	const IDType	kQD3DViewer = 'qd3d';

	TWindow* aWindow = aWindow = gViewServer->NewTemplateWindow(1005, NULL);
	::ThrowIfNULL_AC(aWindow);
	
	TQD3DViewer* viewer = dynamic_cast_AC(TQD3DViewer*,aWindow->FindSubView(kQD3DViewer));

	viewer->UseFile(inFile->GetFileSpec());
	
	aWindow->Open();

#else
	inFile;
#endif
}

//----------------------------------------------------------------------------------------
// CDropenerApp::PlayMovie()
//----------------------------------------------------------------------------------------

void CDropenerApp::PlayMovie(TFile* inFile)
{
	CAutoPtr_AC<TMovie> movie = TH_new TMovie(inFile);
	
	TWindow* window = gViewServer->NewTemplateWindow(1004, NULL);
	::ThrowIfNULL_AC(window);
	
	TMovieView* movieView = dynamic_cast_AC(TMovieView*,window->FindSubView('MooV'));

	movieView->HaveMovie(movie);
	movie.release();
	
	CViewRect frame = movieView->GetFrame();
	
	TMovieController* controllerView = dynamic_cast_AC(TMovieController*,window->FindSubView('MCtl'));
	
	frame.bottom += controllerView->GetFrame().bottom - controllerView->GetFrame().top;
	
	window->SetFrame(frame, false);
	window->Open();
	
	movieView->SetPlayWhenInactive(true);
	movieView->GetMovie()->Start();
	movieView->SetIdleIfMoviePlaying();
}

