// CDropenerAppApp.h
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 

#ifndef __CDropenerApp__
#define __CDropenerApp__

// MacApp

#ifndef __UApplication__
#include "UApplication.h"
#endif

// ACS Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif
#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// ACS Network 

#ifndef __MServiceEventReceiver_AC__
#include "MServiceEventReceiver_AC.h"
	// Note we could have chosen to subclass MTCPConnectionEventReceiver,
	// but this shows off the polymorphic nature of the network suite.
#endif

#ifndef __CMultiTCPConnectionMgr_AC__
#include "CMultiTCPConnectionMgr_AC.h"
#endif

// fwd refs
class TLogWindow;
class CNetworkTransaction_AC;
class CFSSpec_AC;

//----------------------------------------------------------------------------------------
// class CDropenerApp
//----------------------------------------------------------------------------------------

class CDropenerApp:		public TApplication,
						public MServiceEventReceiver_AC
						
{
public:
	// public interface

	CDropenerApp();
		// Constructor
	virtual ~CDropenerApp();
		// Destructor
#if qTrace_AC	
	TLogWindow& Log() { return *fLog; }
		// This is where the networking activity is logged.
#endif
		
protected:
	virtual void Start();
		// This is called via a command to start up the networking
		// and the logging objects once the application is fully
		// constructed.	


// Network Event Handlers
	virtual void PrepareToReceiveData(CServiceHandler_AC* inHandler);

	virtual void ReceiveData(CServiceHandler_AC* inHandler, CNetworkTransaction_AC*);

	virtual void RespondToDataSent(CServiceHandler_AC* inHandler, CNetworkTransaction_AC*);
		
	virtual void RespondToAcceptorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler);

	virtual void RespondToConnectorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler);
										
// MacApp overrides
		
	virtual void DoSetupMenus(); 

	virtual void DoMenuCommand(CommandNumber aCommandNumber); 

	virtual TDocument* OpenNew(CommandNumber itsCommandNumber);
		// This doesn't do anything.
		
	virtual TDocument* OpenOld(CommandNumber itsOpenCommand, CList_AC* aFileList);
		// get list of files dropped on Dropener. By overriding this we're bypassing
		// the document support in MacApp.

// Dropener methods

	void HandleReceivedFile(TFile* inFile);
		// This is where Dropener opens the file dropped on it, like a Movie or a
		// long sound.

	void PlayMovie(TFile* inFile);
	
	void OpenQD3DObject(TFile* inFile);
	
private:

	CStr255_AC 											fDefaultIPAddress;
		// keep track of the last ip address the user connected to
		
	CRefCountingPtr_AC<CMultiTCPConnectionManager_AC>	fConnections;

#if qTrace_AC
	CAutoPtr_AC<TLogWindow> 							fLog;
#endif

	// friends
	friend class TStarter;
};	


#endif
