//----------------------------------------------------------------------------------------
// CMyProgressWindow.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CMyProgressWindow__
#include "CMyProgressWindow.h"
#endif


#ifndef __UStaticText__
#include "UStaticText.h"
#endif

#ifndef __UProgressIndicator__
#include "UProgressIndicator.h"
#endif

#ifndef __CServiceHandler_AC__
#include "CServiceHandler_AC.h"
#endif

#ifndef __cstring__
#include <String.h>
#endif


//----------------------------------------------------------------------------------------
// CMyProgressWindow::CMyProgressWindow
//----------------------------------------------------------------------------------------

CMyProgressWindow::CMyProgressWindow(CServiceHandler_AC* inHandler, bool isSender)
	: fProgressIndicator(NULL),
	fByteProgress(0),
	fBytesPerSecond(0),
	fStartTick(0),
	fMaxAmt(0),
	fStartAmt(0)
{

	fWindow = gViewServer->NewTemplateWindow(1001, NULL);
	::ThrowIfNULL_AC(fWindow);

	fProgressIndicator = dynamic_cast_AC(TProgressIndicator*,fWindow->FindSubView('prog'));

	fProgressIndicator->StartAutoAnimate();
	
	if(isSender) fWindow->SetTitle(CStr255_AC("Sending File"));
	else fWindow->SetTitle(CStr255_AC("Receiving File"));
	
	fWindow->Open();
	
	inHandler->SetUserData( reinterpret_cast<void*>(this) );
	
	fByteProgress =  dynamic_cast_AC(TStaticText*,fWindow->FindSubView('TEXT'));

	fBytesPerSecond =  dynamic_cast_AC(TStaticText*,fWindow->FindSubView('BYTE'));

}

//----------------------------------------------------------------------------------------
// CMyProgressWindow::~CMyProgressWindow
//----------------------------------------------------------------------------------------

CMyProgressWindow::~CMyProgressWindow()
{
	fWindow->Close();
}

//----------------------------------------------------------------------------------------
// CMyProgressWindow::Update
//----------------------------------------------------------------------------------------

void CMyProgressWindow::Update(CServiceHandler_AC* inServiceHandler, long cur, long max)
{
	void* userData = inServiceHandler->GetUserData();
	if(userData)
	{
		CMyProgressWindow* window = reinterpret_cast<CMyProgressWindow*>(userData);
		::ThrowIfNULL_AC(window);
		
		window->DoUpdate(cur,max);
		
		if(cur == max)
		{
			inServiceHandler->SetUserData(NULL);
			delete window;
		}
	}
}

//----------------------------------------------------------------------------------------
// CMyProgressWindow::DoUpdate
//----------------------------------------------------------------------------------------

void CMyProgressWindow::DoUpdate(long cur, long max)
{
	if(!fMaxAmt)
	{
		fStartTick = TickCount();
		fMaxAmt = max;
		fStartAmt = cur;
	}

	fProgressIndicator->SetLongValues( cur, 0, max, fProgressIndicator->Focus());
	
	static char str[256];
	
	sprintf(str, "Transfered %ldk of %ldk", cur/1024, max/1024 );
	
	CStr255_AC text;
	
	text = str;
	
	fByteProgress->ForceRedraw();
	fByteProgress->SetText(text, kDontRedraw);
	
	if(TickCount() - fStartTick > 60)
	{
		long bytesPerSecond = ((cur - fStartAmt) / ((TickCount() -fStartTick) / 60)) / 1024;
	
		sprintf(str, "%ldk per second", bytesPerSecond );
	
		text = str;
	
		fBytesPerSecond->ForceRedraw();
		fBytesPerSecond->SetText(text, kDontRedraw);
		
		fStartAmt = cur;
		fStartTick = TickCount();
	}
	
	gDispatcher->UpdateAllWindows();
}

