//----------------------------------------------------------------------------------------
// CPartyLineApp.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CPartyLineApp__
#include "CPartyLineApp.h"
#endif

// macApp
#ifndef __ULogWindow__
#include "ULogWindow.h"
#endif

#ifndef __Utils__
#include "Utils.h"
#endif


// pattern includes

#ifndef __CReactor_AC__
#include "CReactor_AC.h"
#endif

#ifndef __CEvent_AC__
#include "CEvent_AC.h"
#endif


// text includes
#ifndef __CBufferStream_AC__
#include "CBufferStream_AC.h"
#endif


// network includes
#ifndef __NetEvents_AC__
#include "NetEvents_AC.h"
#endif

#ifndef __DisconnectTransactions_AC__
#include "DisconnectTransactions_AC.h"
#endif

#ifndef __CMultiTCPConnectionMgr_AC__
#include "CMultiTCPConnectionMgr_AC.h"
#endif

#ifndef __CDataReceiver_AC__
#include "CDataReceiver_AC.h"
#endif

#ifndef __CDataSender_AC__
#include "CDataSender_AC.h"
#endif

#ifndef __CInternetAddress_AC__
#include "CInternetAddress_AC.h"
#endif

#ifndef __CServiceState_AC__
#include "CServiceState_AC.h"
#endif

#ifndef __CInternetUtils_AC__
#include "CInternetUtils_AC.h"
#endif


// stream
#ifndef __CTraceStream_AC__
#include "CTraceStream_AC.h"
#endif


// MacApp Includes

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

enum eCommands
{
	cConnect = 400,
	cTalk,
	cLogOut
};

const long kListeningPort = 50000;
const long kChunkSize = 512;

//----------------------------------------------------------------------------------------
// class TStarter
//----------------------------------------------------------------------------------------

class TStarter : public TCommand
{
public:
	TStarter(CPartyLineApp* app) 
		: 	TCommand(0, NULL, false, false, NULL), 
			fApp(app) 
		{}

	virtual void DoIt() { fApp->Start(); }

private:
	CPartyLineApp* fApp;
};


//----------------------------------------------------------------------------------------
// CPartyLineApp::CPartyLineApp
//----------------------------------------------------------------------------------------

CPartyLineApp::CPartyLineApp()
	:	TApplication('????', '????')
{
	fLaunchWithNewDocument = false;
			
	TStarter* starter = TH_new TStarter(this);
	PostCommand(starter);
	
	fConnections = TH_new CMultiTCPConnectionManager_AC;

	RegisterForEventsFrom(fConnections);
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::DoInitialState
//----------------------------------------------------------------------------------------

void CPartyLineApp::DoInitialState()
{
	TApplication::DoInitialState();

	CViewRect frame(10,50,500, 500);
	
	fLog = TH_new TLogWindow;
	fLog->ReadViewsFromResource();
	fLog->GetWindow()->SetFrame(frame, false);
	fLog->GetWindow()->SetTitle(CStr255_AC("Event Log"));
	fLog->Show();

	frame.Offset(10, 10);

	fChatLog = 	TH_new TLogWindow;
	fChatLog->ReadViewsFromResource();
	fChatLog->GetWindow()->SetFrame(frame, false);
	fChatLog->Show();
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::~CPartyLineApp
//----------------------------------------------------------------------------------------

CPartyLineApp::~CPartyLineApp()
{
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::Start
//----------------------------------------------------------------------------------------

void CPartyLineApp::Start()
{
	CTraceStream_AC::AddStream( *fLog );
		// hook our log window into the trace mechanism.

	while(!InitInternetSuite_AC())
	{}
	
	*fChatLog 	<< "\n"
				<< CInternetAddress_AC::GetLocalAddress().GetAsString() 
				<< ":"
				<< kListeningPort
				<< " open for chat fun!"  << endl_AC;

	fConnections->ListenTo(kListeningPort);

	if(!GetSomeTextFromUser("Please Enter Your Name: ", fName, "MacApp Developer" ))
		fName = "Spoilsport";
	
	*fChatLog << "\nWelcome, " << (const char*) fName << ", please connect to a chat.\n" << endl_AC;		
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::ReceiveData
//----------------------------------------------------------------------------------------

void CPartyLineApp::ReceiveData(	CServiceHandler_AC* /*inServiceHandler*/, 
											CNetworkTransaction_AC* inReceiver)
{

	inReceiver->GetStream() << '\0';
		// we need to add a terminator onto the string we received.

	*fChatLog << inReceiver->GetStream().GetReadPositionPtr() << endl_AC;
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::PrepareToReceiveData
//----------------------------------------------------------------------------------------

void  CPartyLineApp::PrepareToReceiveData(CServiceHandler_AC* inHandler)
{
	inHandler->Receive( TH_new CDataReceiver_AC(kChunkSize) );
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::DoMenuCommand
//----------------------------------------------------------------------------------------

void CPartyLineApp::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		case cConnect:
			StartNewChat();
			Talk();
			break;
		
		case cTalk:
			Talk();
			break;
		
		case cLogOut:
			LogOut();
			break;
		
		case cQuit:
			CTraceStream_AC::RemoveStream(*fLog);
			LogOut();

		// fall through
		
		default:
			TApplication::DoMenuCommand(aCommandNumber);
			break;
	}
} 

//----------------------------------------------------------------------------------------
// CPartyLineApp::DoSetupMenus
//----------------------------------------------------------------------------------------

void CPartyLineApp::DoSetupMenus() 
{
	TApplication::DoSetupMenus();		

	Enable(cNew, false);
	Enable(cOpen,false);
	Enable(cConnect, true);
	Enable(cTalk, true);
	Enable(cLogOut, true);
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::LogOut
//----------------------------------------------------------------------------------------

void CPartyLineApp::LogOut()
{
	*fChatLog  << (const char*) fName << ", you've left the chat. " << endl_AC;
	
	CRefCountingPtr_AC<CDataSender_AC> sender = TH_new CDataSender_AC(kChunkSize);
	sender->GetStream() << "Net Msg: " << (const char*) fName << " has left the chat. ";
	fConnections->SendTransaction(sender);	
	fConnections->Disconnect();	
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::RespondToAcceptorEvent
//----------------------------------------------------------------------------------------

void CPartyLineApp::RespondToAcceptorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler)
{
	if(inEvent->GetEventID() == CNetEvent_AC::kAcceptSucceeded)
	{
		CInternetAddress_AC remoteAddress;
		inHandler->GetState()->GetRemoteAddress(remoteAddress);
		
		*fChatLog << (const char*) fName << ", you've accepted connection from: " << remoteAddress.GetAsString() << " " << endl_AC;
	}
	else
	{
		*fChatLog << (const char*) fName << ", you've rejected a connection attempt " << endl_AC;
	}

}

//----------------------------------------------------------------------------------------
// CPartyLineApp::RespondToConnectorEvent
//----------------------------------------------------------------------------------------

void CPartyLineApp::RespondToConnectorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler)
{
	if(inEvent->GetEventID() == CNetEvent_AC::kConnectSucceeded)
	{
		CRefCountingPtr_AC<CDataSender_AC> sender = TH_new CDataSender_AC(kChunkSize);

		CInternetAddress_AC remoteAddress;
		inHandler->GetState()->GetRemoteAddress(remoteAddress);
		
		*fChatLog  << (const char*) fName << ", you've joined the chat at " << remoteAddress.GetAsString() << endl_AC;
		sender->GetStream() << "Net Msg: " << (const char*) fName << " has joined the chat from " << CInternetAddress_AC::GetLocalAddress().GetAsString();
		
		fConnections->SendTransaction(sender);	
	}
	else
	{
		*fChatLog  << (const char*) fName << ", the connect attempt failed. " << endl_AC;
	}
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::RespondToDisconnect
//----------------------------------------------------------------------------------------

void CPartyLineApp::RespondToDisconnect(CServiceHandler_AC* /*inHandler*/, CNetEvent_AC::eEvent /*inReason*/)
{
	*fChatLog  << (const char*) fName << ", you have been disconnected." << endl_AC;
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::StartNewChat
//----------------------------------------------------------------------------------------

void CPartyLineApp::StartNewChat()
{
	long localPort = kListeningPort+1;
	long remotePort = kListeningPort; 
	CStr255_AC remoteAddressStr;
	 
	if(GetSomeTextFromUser("Connect to? [Enter Zero to connect to Self]", remoteAddressStr, "0.0.0.0"))
	{
		if(remoteAddressStr == "" || remoteAddressStr == "0")
		{
			remoteAddressStr = "0.0.0.0";
		}
		
		fConnections->ConnectTo(remoteAddressStr,remotePort);
	}			
}

//----------------------------------------------------------------------------------------
// CPartyLineApp::Talk
//----------------------------------------------------------------------------------------

void CPartyLineApp::Talk()
{
	static CStr255_AC str;
	 
	if(GetSomeTextFromUser("Send some text:", str))
	{
		*fChatLog << (const char*) fName << ": " << (const char*) str << endl_AC;
	
		long len = ::strlen(str);
	
		if(len)
		{
			CRefCountingPtr_AC<CDataSender_AC> sender = TH_new CDataSender_AC(256);
			
			sender->GetStream() << (const char*) fName << ": " << (const char*) str; 
			fConnections->SendTransaction(sender);	
		}
	}			
}



