// CPartyLineAppApp.h
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 

#ifndef __CPartyLineApp__
#define __CPartyLineApp__

// MacApp

#ifndef __UApplication__
#include "UApplication.h"
#endif

// ACS Core

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// ACS Network

#ifndef __MServiceEventReceiver_AC__
#include "MServiceEventReceiver_AC.h"
#endif

#ifndef __CMultiTCPConnectionMgr_AC__
#include "CMultiTCPConnectionMgr_AC.h"
#endif

// Forward Ref

class TLogWindow;
class CNetworkTransaction_AC;

//----------------------------------------------------------------------------------------
// CPartyLineApp
//----------------------------------------------------------------------------------------

class CPartyLineApp:	public TApplication,
						public MServiceEventReceiver_AC
						
{
public:
	// public interface

	CPartyLineApp();
		// Constructor
	virtual ~CPartyLineApp();
		// Destructor
	
	TLogWindow& Log() { return *fLog; }
		// This is where the networking activity is logged.
		
	virtual void DoInitialState();
		// Override to create global log window
		
protected:

	virtual void Start();
		// This is called via a command to start up the networking
		// and the logging objects once the application is fully
		// constructed.	
		
	void StartNewChat();

	void Talk();
	
	void LogOut();


// MServiceEventReceiver_AC overrides

	virtual void PrepareToReceiveData(CServiceHandler_AC*);

	virtual void ReceiveData(	CServiceHandler_AC*, 
								CNetworkTransaction_AC*);

	virtual void RespondToConnectorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler);
	
	virtual void RespondToAcceptorEvent(CEvent_AC* inEvent, CServiceHandler_AC* inHandler);
		
	virtual void RespondToDisconnect(CServiceHandler_AC* inHandler, CNetEvent_AC::eEvent inReason);
										
// MacApp overrides

	virtual TDocument* DoMakeDocument(CommandNumber , TFile* )	{ return NULL; }
		// This application doesn't have a 'document'.
		
	virtual void DoSetupMenus(); 

	virtual void DoMenuCommand(CommandNumber aCommandNumber); 

private:
	CRefCountingPtr_AC<CMultiTCPConnectionManager_AC>		fConnections;
	CAutoPtr_AC<TLogWindow> 								fLog;
	CAutoPtr_AC<TLogWindow> 								fChatLog;
	CStr255_AC												fName;
	
private:
	// friends
	friend class TStarter;
};	


#endif
