//----------------------------------------------------------------------------------------
// UTEViewStream.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#include "UTEViewStream.h"

#include "UWindow.h"
#include "CBufferedStreamRep_AC.h"

#define kDefaultSize 256


//----------------------------------------------------------------------------------------
// class TViewStreamRep
//----------------------------------------------------------------------------------------

class TViewStreamRep : public CBufferedStreamRep_AC
{
public:
	TViewStreamRep(TTEView* inTEView) 
		: CBufferedStreamRep_AC(kDefaultSize),
	fTEView(inTEView)
	{}
	
	virtual ~TViewStreamRep()
	{}

	void SetTEView(TTEView* inView)
	{	
		fTEView = inView; 
		fTEView->SetEnable(false);
	}	

	TTEView* GetTEView() const
	{	return fTEView; }

	void ReadTEViewFromResource(TWindow* inWindow, OSType viewID);

protected:
	virtual void WriteBufferedBytes(const void* inBytes, long inAmt);
		
protected:
	TTEView* fTEView;


};
	

//----------------------------------------------------------------------------------------
// TViewStreamRep::WriteBufferedBytes
//----------------------------------------------------------------------------------------

void TViewStreamRep::WriteBufferedBytes(const void* inBytes, long inAmt)
{
	::AssertNotNULL_AC(fTEView);
	
	long textlen = inAmt;
	
	char* walker = const_cast<char*>( static_cast<const char*>(inBytes) );

	fTEView->SetActive(false);
	
	while(textlen > 0)
	{
	
		static char text[kDefaultSize+1];
		
		long amtToCopy = Min_AC(kDefaultSize, textlen );
		
		::BlockMove( walker, text, amtToCopy) ;
		walker += amtToCopy;
		textlen -= amtToCopy;
		text[amtToCopy] = '\0';
		
		if((*(fTEView->fHTE))->teLength + textlen < 32000)
		{
			// In the normal situation we have enough space to add the text, 
			// set the selection to the end.
			fTEView->SetSelection(	(*(fTEView->fHTE))->teLength, 
									(*(fTEView->fHTE))->teLength, true);

		
		}
		else
		{
			// Else there ain't enough room to insert the text. The new length would be longer than
			// 32k. So chop off the first half of the text. It's not pretty, I know. :-)
		
			fTEView->SetSelection(0, (*(fTEView->fHTE))->teLength / 2, true);
				// Set the selection to delete.

			fTEView->SynchView(true);
			fTEView->ScrollSelectionIntoView(true);

			::TEDelete(fTEView->fHTE);
				// this deletes the selection

			fTEView->RecalcText();
				// fix up the view
			
			fTEView->SetSelection(	(*(fTEView->fHTE))->teLength, 
									(*(fTEView->fHTE))->teLength, true);
				// Set the selection to the end.

			fTEView->SynchView(true);
			fTEView->ScrollSelectionIntoView(true);
				// scroll the end into view.
		}
		
	
	/*
		There seems to be some inconsistancy with what '\n' and '\r' mean. 
	*/
		char* kludge = (char*) text;
		for( short i = 0; i < amtToCopy; ++i)
			if( kludge[i] == 0xA )
				kludge[i] = 0xD;

		::TEInsert(text, amtToCopy, fTEView->fHTE );
	}
	
	fTEView->SetSelection((*(fTEView->fHTE))->teLength, (*(fTEView->fHTE))->teLength, true);
	fTEView->SynchView(true);
	fTEView->SetActive(true);

}

//----------------------------------------------------------------------------------------
// TViewStreamRep::ReadTEViewFromResource
//----------------------------------------------------------------------------------------

void TViewStreamRep::ReadTEViewFromResource(TWindow* inWindow, OSType inID)
{
	::AssertNotNULL_AC (inWindow);

	fTEView = dynamic_cast_AC(TTEView*,inWindow->FindSubView(inID));
	
	fTEView->SetEnable(false);
}


//----------------------------------------------------------------------------------------
// TTEViewStream::TTEViewStream
//----------------------------------------------------------------------------------------

TTEViewStream::TTEViewStream(TTEView* inTEView)
	: CCharOutputSink_AC( TH_new TViewStreamRep(inTEView))
{
}

//----------------------------------------------------------------------------------------
// TTEViewStream::~TTEViewStream
//----------------------------------------------------------------------------------------

TTEViewStream::~TTEViewStream()
{
}

//----------------------------------------------------------------------------------------
// TTEViewStream::SetTEView
//----------------------------------------------------------------------------------------

void TTEViewStream::SetTEView(TTEView* inView)
{	
	TViewStreamRep* rep = dynamic_cast_AC(TViewStreamRep*,GetStreamRep());
	rep->SetTEView(inView);
}

//----------------------------------------------------------------------------------------
// TTEViewStream::GetTEView
//----------------------------------------------------------------------------------------

TTEView* TTEViewStream::GetTEView() const
{
	TViewStreamRep* rep = dynamic_cast_AC(TViewStreamRep*,GetStreamRep());
	return rep->GetTEView();
}

//----------------------------------------------------------------------------------------
// TTEViewStream::ReadTEViewFromResource
//----------------------------------------------------------------------------------------

void TTEViewStream::ReadTEViewFromResource(TWindow* inWindow, OSType inID)
{
	TViewStreamRep* rep = dynamic_cast_AC(TViewStreamRep*,GetStreamRep());
	rep->ReadTEViewFromResource(inWindow, inID);
}

