//----------------------------------------------------------------------------------------
// CWebinatorApp.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CWebinatorApp__
#include "CWebinatorApp.h"
#endif

// macApp
#ifndef __ULOGWINDOW__
#include "ULogWindow.h"
#endif


#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

// Net includes
#ifndef __CSimpleHTTPServer_AC__
#include "CSimpleHTTPServer_AC.h"
#endif

#ifndef __CInternetHost_AC__
#include "CInternetHost_AC.h"
#endif

#ifndef __CMultiTCPConnectionMgr_AC__
#include "CMultiTCPConnectionMgr_AC.h"
#endif

#ifndef __CInternetUtils_AC__
#include "CInternetUtils_AC.h"
#endif

// streams
#if qTrace_AC
	#ifndef __CTraceStream_AC__
	#include "CTraceStream_AC.h"
	#endif
#endif


enum eMenuCommands
{
	cLow = 400,
	cMed,
	cHigh,
	cNone
};

//----------------------------------------------------------------------------------------
// class TStarter
//----------------------------------------------------------------------------------------

class TStarter : public TCommand
{
public:
	TStarter(CWebinatorApp* app) 
		: 	TCommand(0, NULL, false, false, NULL), 
			fApp(app) 
		{}

	virtual void DoIt() { fApp->Start(); }

private:
	CWebinatorApp* fApp;
};


//----------------------------------------------------------------------------------------
// CWebinatorApp::CWebinatorApp
//----------------------------------------------------------------------------------------

CWebinatorApp::CWebinatorApp()
	:	TApplication('????', '????')
{
	fLaunchWithNewDocument = false;

	CViewRect frame(10,50,400, 250);	
	
	fLog = TH_new TLogWindow;
	fLog->ReadViewsFromResource();
	fLog->GetWindow()->SetFrame(frame, false);
	fLog->GetWindow()->SetTitle(CStr255_AC("Event Log"));
	fLog->Show();

#if qTrace_AC

	frame.Offset(0,245);

	fTrace = TH_new TLogWindow;
	
	fTrace->ReadViewsFromResource();
	
	TWindow* window = fTrace->GetWindow();
	
	window->SetFrame(frame, false);
	window->SetTitle(CStr255_AC("Trace"));
	window->Open();
#endif

	TStarter* starter = TH_new TStarter(this);
	PostCommand(starter);
	}

CWebinatorApp::~CWebinatorApp()
{
#if qTrace_AC
	CTraceStream_AC::RemoveStream(*fTrace);
		// turn off trace
#endif
	LogOut();
}

//----------------------------------------------------------------------------------------
// CWebinatorApp::Start
//----------------------------------------------------------------------------------------

void CWebinatorApp::Start()
{
#if qTrace_AC
	CTraceStream_AC::AddStream( *fTrace );
		// hook our log window into the trace mechanism.
	CTraceStream_AC::Instance()->SetPriorityFilter(CTraceStream_AC::kLowPriority);
#endif		
	
	while(!InitInternetSuite_AC())
	{
	}

	CTempInternetHost_AC machineName  = CInternetAddress_AC::GetLocalAddress();
	machineName.ResolveAddressToNameSync();
	
	*fLog 	<< "Host Name: " 
			<< machineName.GetHostName() 
			<< "\nHost Address: "
			<< CInternetAddress_AC::GetLocalAddress().GetAsString() 
			<< endl_AC;
	
	fConnections = TH_new CMultiTCPConnectionManager_AC;
	fConnections->AddStrategy(TH_new CSimpleHTTPServer_AC);
	fConnections->ListenTo(CInternetAddress_AC::kHTTP);
	
	*fLog << "Webinator is now open for Bidness!!\n" << endl_AC;
}

//----------------------------------------------------------------------------------------
// CWebinatorApp::DoMenuCommand
//----------------------------------------------------------------------------------------

void CWebinatorApp::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{

#if qTrace_AC	
		case cLow:
			TRACE.SetPriorityFilter(CTraceStream_AC::kLowPriority);
		break;
		case cMed:
			TRACE.SetPriorityFilter(CTraceStream_AC::kMedPriority);
		break;			
		case cHigh:
			TRACE.SetPriorityFilter(CTraceStream_AC::kHighPriority);
		break;			
		
		case cNone:
			TRACE.SetPriorityFilter(CTraceStream_AC::kOff);
		break;	
#else
		case cLow:
		break;
		case cMed:
		break;			
		case cHigh:
		break;			
		
		case cNone:
		break;	
#endif		
		default:
			TApplication::DoMenuCommand(aCommandNumber);
			break;
	}
} 

//----------------------------------------------------------------------------------------
// CWebinatorApp::DoSetupMenus
//----------------------------------------------------------------------------------------

void CWebinatorApp::DoSetupMenus() 
{
	TApplication::DoSetupMenus();		

	Enable(cNew, false);
	Enable(cOpen,false);

#if qTrace_AC
	EnableCheck(cLow, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kLowPriority);
	EnableCheck(cMed, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kMedPriority);
	EnableCheck(cHigh, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kHighPriority);
	EnableCheck(cNone, true, TRACE.GetPriorityFilter() == CTraceStream_AC::kOff );
#else
	Enable(cLow, false);
	Enable(cMed, false);
	Enable(cHigh, false);
	Enable(cNone, false);
#endif
}

//----------------------------------------------------------------------------------------
// CWebinatorApp::LogOut
//----------------------------------------------------------------------------------------

void CWebinatorApp::LogOut()
{
	fConnections->Disconnect();	
}


