//--------------------------------------------------------------------------------------------------
// DynamicPopups.r
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved. 
//
//	Resource File for the DynamicPopups Sample Program. 
//--------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------
// INCLUDE FILES 
//--------------------------------------------------------------------------------------------------

// Standard Includes 
#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif

#ifndef __BalloonTypes__
#include "BalloonTypes.r"
#endif

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef THINK_Rez
#if qDebug | qPerform
include "Debug.rsrc" not 'ckid';
#endif

include "Dialog.rsrc" not 'ckid';
include "MacApp.rsrc" not 'ckid';

// Includes for Building Blocks

include "Printing.rsrc" not 'ckid';

// Include Code Resources

#ifndef MW_Rez 
#if !qNoCode
include $$Shell("ObjApp")$$Shell("XAppName") 'CODE';
#if qModelCFM
include $$Shell("ObjApp")$$Shell("XAppName") 'cfrg';
include $$Shell("ObjApp")$$Shell("XAppName") 'rseg';
#endif	// qModelCFM
#endif	// !qNoCode
#endif	// !MW_Rez
#endif	// !THINK_Rez

//--------------------------------------------------------------------------------------------------
// CONSTANTS 
//--------------------------------------------------------------------------------------------------

// Finder Information
// ------------------

#if !defined(THINK_Rez) && !defined(MW_Rez)
#define kApplicationName	$$Shell("XAppName")
#else
#define kApplicationName	TARGET_APPLICATION_NAME
#endif // !THINK_Rez

#define kSignature			'SS01'			// Application signature 
#define kFileType			'SF01'			// Data file type 
#define kStationery			'sF01'			// Stationery file type 

// Bundle Constants
// ------------------

#define kBundleID						128
#define kApplicationID					128
#define kDocumentID						129
#define kStationeryID					130

// Constants for Menu IDs
// ----------------------

#define mDynamicPopupsMenu	4

// Command Numbers For Menu Items
// ------------------------------

#define cCommandHandledByApplication	400
#define cCommandHandledByDocument		401
#define cCommandHandledByView			402
#define cTrackerCommand					403

// STR# resources
// --------------

#define kDynamicPopupsMenuHelp				1000
#define kDynamicPopupsWindowTitles			1001

// Text styles
// --------------

#define kSystemTextStyle				1000

// 'View' resources
// --------------

#define kDynamicPopupsWindowID				kDefaultWindowID

// Memory Management Resources
// ---------------------------

#define kDynamicPopupsSeg	1000
#define kDynamicPopupsMem	1000
#define kDynamicPopupsRes	1000

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR MENUS 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Menu Bars 
//--------------------------------------------------------------------------------------------------

resource 'MBAR' (kMBarDisplayed,
#if qNames
"DynamicPopup",
#endif
	purgeable) {
	{ mApple; mFile; mEdit }
};

//--------------------------------------------------------------------------------------------------
// Menus 
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc" 'CMNU' (mApple);	// Grab the default Apple menus
include "Defaults.rsrc" 'CMNU' (mFile);		// Grab the default File menus
include "Defaults.rsrc" 'CMNU' (mEdit);		// Grab the default Edit menu

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
purgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {	/* array: 13 elements */
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
		"Tracker Command",		noIcon,	noKey,	noMark, plain, cTrackerCommand;
	}
};

//--------------------------------------------------------------------------------------------------
// Balloon Help Resources for Menus 
//--------------------------------------------------------------------------------------------------

#if defined(THINK_Rez) || defined(MW_Rez)
#include "HelpStrings.r"
#else
#include $$Shell("MALibraries") "HelpStrings.r";
#endif

include "Defaults.rsrc"  'hmnu' (mFile);	
include "Defaults.rsrc"  'hmnu' (mEdit);	

resource 'STR#' (kDynamicPopupsMenuHelp, purgeable) { 
	{
		"No help yet for this  item",
		"Help For Menu Title",
		"Help For First Menu Item",
		"Help For Second Menu Item",
		"Help For Third Menu Item",
	}
};

resource 'hmnu' (mDynamicPopupsMenu) 
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, // balloon definition function 
	0, // variation code 
	
	// Missing items 
	HMStringResItem 
	{
		kDynamicPopupsMenuHelp,1,
		kDynamicPopupsMenuHelp,1,
		kDynamicPopupsMenuHelp,1,
		kDynamicPopupsMenuHelp,1,
	},
	
	// Title and items 
	{	
		// Title 
		HMStringResItem 
		{
			kDynamicPopupsMenuHelp,2,
			kDynamicPopupsMenuHelp,2,
			kDynamicPopupsMenuHelp,2,
			kDynamicPopupsMenuHelp,2,
		},
		// First Item 
		HMStringResItem 
		{
			kDynamicPopupsMenuHelp,3,
			kDynamicPopupsMenuHelp,3,
			kDynamicPopupsMenuHelp,3,
			kDynamicPopupsMenuHelp,3,
		},
		// Second Item
		HMStringResItem 
		{
			kDynamicPopupsMenuHelp,4,
			kDynamicPopupsMenuHelp,4,
			kDynamicPopupsMenuHelp,4,
			kDynamicPopupsMenuHelp,4,
		},
		// Third Item
		HMStringResItem 
		{
			kDynamicPopupsMenuHelp,5,
			kDynamicPopupsMenuHelp,5,
			kDynamicPopupsMenuHelp,5,
			kDynamicPopupsMenuHelp,5,
		},
	}
};

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR VIEWS, DIALOGS, ALERTS etc. 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// About Box
//--------------------------------------------------------------------------------------------------

resource 'DITL' (phAboutApp,
#if qNames
"phAboutApp",
#endif
	purgeable) {
	 {	/* array DITLarray: 3 elements */
		/* [1] */
		{160, 182, 180, 262},
		Button {
			enabled,
			"OK"
		};
		/* [2] */
		{10, 75, 150, 316},
		StaticText {
			disabled,
			"This application demonstrates the use of dynamic popup menus."
			"\n\n"
			WRITTEN_WITH_MACAPP
		};
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		}
	}
};

include "Defaults.rsrc"  'ALRT' (phAboutApp);		// Grab the default about box
include "Defaults.rsrc"	'aete' (0);					// Grab the default AppleEvent terminology
include "Defaults.rsrc"  'STR#' (kDefaultCredits);	// Grab the default credits

//--------------------------------------------------------------------------------------------------
// MultiFinder information
//--------------------------------------------------------------------------------------------------

/* Additional size deltas to be added to partition size */

#if qNames
#define kNamesSizeDelta 75
#else
#define kNamesSizeDelta 0
#endif

#if qDebug
#define kDebugSizeDelta 0
#else
#define kDebugSizeDelta 0
#endif

#define kModelFarSizeDelta 0

#ifndef MW_Rez					// MW projects supply size resource for us
resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	notStationeryAware,				
	reserved,
	reserved,
	reserved,
	reserved,
	
	/* Suggested partition size */
	(1000 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024,
	
	/* Minimum partition size */
	(800 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024
};
#endif // !MW_Rez
//--------------------------------------------------------------------------------------------------
// Icons
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc"  'ICN#' (kApplicationID);	// MacApp Family large black & white	
include "Defaults.rsrc"  'ICN#' (kDocumentID);		// MacApp Document large black & white
include "Defaults.rsrc"  'ICN#' (kStationeryID);	// MacApp Stationery large black & white	
include "Defaults.rsrc"  'ics#' (kApplicationID);	// MacApp Family small black & white	
include "Defaults.rsrc"  'ics#' (kDocumentID);		// MacApp Document small black & white
include "Defaults.rsrc"  'ics#' (kStationeryID);	// MacApp Stationery small black & white
include "Defaults.rsrc"  'ics4' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'ics4' (kDocumentID);		// MacApp Document small 4 bit
include "Defaults.rsrc"  'ics4' (kStationeryID);	// MacApp Stationery small 4 bit
include "Defaults.rsrc"  'ics8' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'ics8' (kDocumentID);		// MacApp Document small 4 bit
include "Defaults.rsrc"  'ics8' (kStationeryID);	// MacApp Stationery small 4 bit
include "Defaults.rsrc"  'icl4' (kApplicationID);	// MacApp Family large 4 bit
include "Defaults.rsrc"  'icl4' (kDocumentID);		// MacApp Document large 4 bit
include "Defaults.rsrc"  'icl4' (kStationeryID);	// MacApp Stationery large 4 bit
include "Defaults.rsrc"  'icl8' (kApplicationID);	// MacApp Family large 8 bit
include "Defaults.rsrc"  'icl8' (kDocumentID);		// MacApp Document large 8 bit
include "Defaults.rsrc"  'icl8' (kStationeryID);	// MacApp Stationery large 8 bit

//--------------------------------------------------------------------------------------------------
// Memory usage information for DynamicPopups
//--------------------------------------------------------------------------------------------------

// See UMemory.h
// -------------

// Code fragment resource for PowerPC native apps
// ----------------------------------------------
#ifndef MW_Rez 	// Metrowerks will supply it's own cfrg
#if qPowerPC
#include "CodeFragmentTypes.r"

resource 'cfrg' (0, "Configuration Info") {
	{
		kPowerPC,
		kFullLib,
		kNoVersionNum,
		kNoVersionNum,
		0,
		0,
		kIsApp,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		"DynamicPopups"
	}
};
#endif // qPowerPC
#endif // MW_Rez
//--------------------------------------------------------------------------------------------------
// Icons, Bundles and FRefsOh my! (don't forget the Signature)
//--------------------------------------------------------------------------------------------------

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"DynamicPopups Apple Computer, Inc. 1996"
};

resource 'FREF' (kApplicationID,
#if qNames
"DynamicPopups Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (kDocumentID,
#if qNames
"DynamicPopups Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'FREF' (kStationeryID,
#if qNames
"DynamicPopups Stationery",
#endif
	purgeable) {
	kStationery,
	2,
	""
};

resource 'BNDL' (kBundleID,
#if qNames
"DynamicPopups",
#endif
	purgeable) {
	kSignature,
	0,
	{ /* array TypeArray: 2 elements */
		/* [1] */
		'ICN#',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		},
		/* [2] */
		'FREF',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		}
	}
};

//--------------------------------------------------------------------------------------------------
//  Version resources
//--------------------------------------------------------------------------------------------------

// The revision of this particular file

RESOURCE 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x01,
	0x00,
	development,
	0x00,
	verUs,
	"1.0",
	"DynamicPopups 1.0, Apple Computer, Inc. 1996-97"
};

include "Defaults.rsrc"  'vers' (2);		// Overall package
