//----------------------------------------------------------------------------------------
// UDocumentDynamicPopups.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UDocumentDynamicPopups__
#include "UDocumentDynamicPopups.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//----------------------------------------------------------------------------------------
// Constants:

const ResNumber kDynamicPopupsWindowID = kDefaultWindowID;

//========================================================================================
// CLASS TDocumentDynamicPopups
//========================================================================================
#undef Inherited
#define Inherited TFileBasedDocument


MA_DEFINE_CLASS(TDocumentDynamicPopups);

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups constructor 
//----------------------------------------------------------------------------------------


TDocumentDynamicPopups::TDocumentDynamicPopups(TFile* itsFile,
										  OSType itsCreator) :
TFileBasedDocument(itsFile,itsCreator)
{
} // TDocumentDynamicPopups::TDocumentDynamicPopups 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups destructor
//----------------------------------------------------------------------------------------


TDocumentDynamicPopups::~TDocumentDynamicPopups()
{
}

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::FreeData: 
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::FreeData()
{
	Inherited::FreeData();
} // TDocumentDynamicPopups::FreeData 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoInitialState: 
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::DoInitialState()
{
	Inherited::DoInitialState();
} // TDocumentDynamicPopups::DoInitialState 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::DoMakeViews(bool /*forPrinting*/)
{
	TWindow* aWindow = gViewServer->NewTemplateWindow(kDynamicPopupsWindowID, this);
	::ThrowIfNULL_AC(aWindow);

	TView* aView = aWindow->FindSubView('MENU');
	TPopup* aPopup = dynamic_cast<TPopup*> (aView);
	::ThrowIfNULL_AC(aPopup);

	aPopup->CreateMenu();
	aPopup->AddItem("Hello there");
	aPopup->AddItem("All righty then");
	aPopup->AddItem("Toggle Menu Bar");	// 3rd item will actually do stuff
	aPopup->AddSeparator();

	// This is the dynamic part: we add the document's name to the end 
	// so it's unique per instance of the popup
	aPopup->AddItem(aWindow->GetTitle());
	
	aPopup->SetCurrentItem(5, kRedraw);
} // TDocumentDynamicPopups::DoMakeViews 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------

void TDocumentDynamicPopups::DoEvent(EventNumber eventNumber,
							TEventHandler* source,
							TEvent* event)
{
	if (eventNumber == mPopupHit)
	{
		TPopup* aPopup = dynamic_cast<TPopup*> (source);
		if (aPopup)
		{
			switch (aPopup->GetCurrentItem())
			{
				case 3:
					if (TMenuBarManager::fgMenuBarManager->IsMenuBarShown())
						TMenuBarManager::fgMenuBarManager->HideMenuBar();
					else
						TMenuBarManager::fgMenuBarManager->ShowMenuBar();
					break;
					
				default: ;
			}
		}
	}
	else
		Inherited::DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes)
{
	Inherited::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);
} // TDocumentDynamicPopups::DoNeedDiskSpace 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoMenuCommand: This method is overridden to handle menu items which
// are enabled when this document is in the target chain. In this example, this is true
// when the document is open and its window is the active window. The inherited method
// should always be called so that MacApp can allow successor objects in the target chain
// (i.e. the application) to handle THEIR menu items.
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber) 
	{
		/*	Typically, you will have dispatch your own menu commands here:
		case cCommandHandledByDocument: 
			{
				TCommandDynamicPopups* aCommand = new TCommandDynamicPopups;
				aCommand->ICommandDynamicPopups(aCommandNumber, this);
				this->PostCommand(aCommand);
			}
			break;
		*/
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
} // TDocumentDynamicPopups::DoMenuCommand 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoRead: 
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::DoRead(TFile* aFile,
						   	   bool forPrinting)
{
	Inherited::DoRead(aFile,forPrinting);
} // TDocumentDynamicPopups::DoRead 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoSetupMenus: This method is overridden to enable menu items which
// should be enabled when this object is in the target chain. MacApp initially disables
// all menu items, then lets the objects in the target chain enable those items they
// handle.
//
// A document object is in the target chain when its window is the active window.
//
// The inherited method is called so that TDocument can enable document-level menu items
// like "Save ". This also ensures that objects further up the target chain (the
// application) can set up THEIR menus.
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::DoSetupMenus()
{
	Inherited::DoSetupMenus();
	
	/*	Typically, you will have enable your own menu commands here:
	Enable(cCommandHandledByDocument,true);
	*/
} // TDocumentDynamicPopups::DoSetupMenus 

//----------------------------------------------------------------------------------------
// TDocumentDynamicPopups::DoWrite: 
//----------------------------------------------------------------------------------------


void TDocumentDynamicPopups::DoWrite(TFile* aFile,
									bool makingCopy)
{
	Inherited::DoWrite(aFile,makingCopy);
} // TDocumentDynamicPopups::DoWrite 

//----------------------------------------------------------------------------------------
// End of UDocumentDynamicPopups.cp


