//----------------------------------------------------------------------------------------
// UMyQD3DBoxView.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

//QD3D
#ifndef  QD3D_h
#include "QD3D.h"
#endif

#ifndef __UMYQD3DBOXVIEW__
#include "UMyQD3DBoxView.h"
#endif

#ifndef __UQD3DBOX__
#include "UQD3DBox.h"
#endif

#ifndef __UQD3DView__
#include "UQD3DView.h"
#endif

//QD3D
#ifndef   QD3DMath_h
#include "QD3DMath.h" 
#endif


//========================================================================================
// CLASS TMyQD3DBoxView
//========================================================================================
#undef Inherited
#define Inherited TQD3DView


MA_DEFINE_CLASS_M1(TMyQD3DBoxView, Inherited);

//----------------------------------------------------------------------------------------
// TMyQD3DBoxView constructor 
//----------------------------------------------------------------------------------------


TMyQD3DBoxView::TMyQD3DBoxView()
{
} 

//----------------------------------------------------------------------------------------
// TMyQD3DBoxView destructor 
//----------------------------------------------------------------------------------------


TMyQD3DBoxView::~TMyQD3DBoxView()
{
	if (fBox)
		delete fBox;
}

//----------------------------------------------------------------------------------------
// TMyQD3DBoxView::DoPostCreate: 
//----------------------------------------------------------------------------------------

	
void TMyQD3DBoxView::DoPostCreate(TDocument* itsDocument) 
{
	Inherited::DoPostCreate(itsDocument); 			
	
	
	TQ3ColorARGB theBackgroundColor = {1.0,1.0,1.0,1.0};

	
	
	this->SetDrawingContext(theBackgroundColor,kQ3True);
	
	TQ3Point3D	cameraLocation = {0.0,0.0,12.0}; 	
	TQ3Point3D	pointOfInterest = {0.0,0.0,0.0};  
	TQ3Vector3D upVector = {0.0,1.0,0.0}; 
	float		hither = 0.1;
	float 		yon = 100.0;
	float 		fieldOfView = 30 * kQ3Pi / 180; 	// approx 30 degrees
	
	//Set Rendering
	this->SetRenderer(kQ3RendererTypeInteractive, false, false);

	//Lights
	float brightness = 1.0;
	TQ3ColorRGB	color = { 1.0, 1.0, 1.0 }; 			//whitelight
	TQ3Vector3D	fillDirection = { -0.5, -1.0, 0.3 };
	TQ3Point3D theLocation = {1.0,0,0};

	//DirectionalLight
	this->CreateLightObject(kDirectionalLight,kQ3True,brightness,color,kQ3False,fillDirection);
	this->SetLightGroup();

	//Set Camera
	this->SetCameraObject(cameraLocation,pointOfInterest,upVector,hither,yon,fieldOfView);
	
	//Set Styles
	this->SetInterpolation(kQ3InterpolationStyleVertex);
	this->SetBackfacing(kQ3BackfacingStyleBoth);
	this->SetFillStyle(kQ3FillStyleFilled);
	
	//Set Illumination Shading
	this->SetIllumination(kLambertIllumination, false);
	
	//Set Texture
	this->SetTextureFromPICTResource(128, false);
			
	//BoxData	
	float			ambience 	= 0.8;
	TQ3ColorRGB 	theColor 	= {0.0,0.0,0.0};
	TQ3Point3D		origin 		= {0,0,0};
	TQ3Vector3D 	orientation = {0,1.0,0}; 	// was 0,1,0
	TQ3Vector3D 	majorAxis	= {0,0,2.0};
	TQ3Vector3D		minorAxis 	= {2.0,0,0};
		
	fBox = TH_new TQD3DBox(ambience,theColor,origin,orientation,majorAxis,minorAxis);
}

//----------------------------------------------------------------------------------------
// TMyQD3DBoxView::RenderGeometries: 
//----------------------------------------------------------------------------------------
	
void TMyQD3DBoxView::RenderGeometries() 
{
	Fail3DErr(Q3Geometry_Submit(fBox->GetQD3DBox(),fQD3DView));
}

/*



//----------------------------------------------------------------------------------------
// TMyQD3DBoxView::WillAcceptDrop
//----------------------------------------------------------------------------------------


bool TMyQD3DBoxView::WillAcceptDrop(CDragItemIterator& dragItemIterator)
{
	bool returnVal = false;

	if (dragItemIterator.GetItemCount() == 1)
	{		
		returnVal = dragItemIterator.First()->FlavorExists(kPictType_AC);
		
		MSpeakable SpeechObj;
		
		if(returnVal)
			SpeechObj.Say("Drop it on me!",true);
		else
			SpeechObj.Say("Yuck - I can't use that.",true);
	}
	
	return returnVal;
}
*/