// ----------------------------------------------------------------------------------------
// UQD3DBox.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// ----------------------------------------------------------------------------------------

#ifndef  QD3D_h
#include "QD3D.h"
#endif

#ifndef __UQD3DBOX__
#include "UQD3DBox.h"
#endif

#ifndef  __UQD3DCOMMON__
#include "UQD3DCommon.h"
#endif

// ========================================================================================
// Global Initialization Procedure
// ========================================================================================
#undef Inherited

// ========================================================================================
// CLASS TQD3DBox
// ========================================================================================



// ----------------------------------------------------------------------------------------
// TQD3DBox constructor 
// ----------------------------------------------------------------------------------------


TQD3DBox::TQD3DBox(float ambience,TQ3ColorRGB theColor,TQ3Point3D theOrigin,
						TQ3Vector3D theOrientation, TQ3Vector3D majorAxis, TQ3Vector3D minorAxis):
	fQD3DBox(NULL)
{
	//NO GOOD, Need to set diffuse and attribute types.
	TQ3AttributeSet 	theAttributes;
	TQ3BoxData			theBoxData;
	
	//Setup the Attributes for this box (Color, Ambience)
	theAttributes = Q3AttributeSet_New();
	::ThrowIfNULL_AC(theAttributes);
	Fail3DErr(Q3AttributeSet_Add(theAttributes, kQ3AttributeTypeDiffuseColor, &theColor));
	Fail3DErr(Q3AttributeSet_Add(theAttributes, kQ3AttributeTypeAmbientCoefficient, &ambience));
	
	//Set Box Data
	theBoxData.origin 			= theOrigin;
	theBoxData.orientation 		= theOrientation;
	theBoxData.majorAxis 		= majorAxis;
	theBoxData.minorAxis 		= minorAxis;
	theBoxData.faceAttributeSet = NULL;
	theBoxData.boxAttributeSet 	= theAttributes;
	
	fQD3DBox = Q3Box_New(&theBoxData);
	::ThrowIfNULL_AC(fQD3DBox);
	
	Q3Object_Dispose(theAttributes);	
} 

// ----------------------------------------------------------------------------------------
// TQD3DBox destructor 
// ----------------------------------------------------------------------------------------


TQD3DBox::~TQD3DBox()
{
	if(fQD3DBox)
		Q3Object_Dispose(fQD3DBox);	
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::SetData 
// ----------------------------------------------------------------------------------------
void TQD3DBox::SetData(TQ3BoxData theBoxData)
{
	Fail3DErr(Q3Box_SetData(fQD3DBox,&theBoxData));	
} 

// ----------------------------------------------------------------------------------------
// TQD3DBox::GetData 
// ----------------------------------------------------------------------------------------
TQ3BoxData TQD3DBox::GetData()
{
	TQ3BoxData theBoxData;
	
	Fail3DErr(Q3Box_GetData(fQD3DBox,&theBoxData));
	
	return theBoxData;
} 

/*
// ----------------------------------------------------------------------------------------
// TQD3DBox EmptyData 
// ----------------------------------------------------------------------------------------

TQD3DBox::EmptyData()
{
	
} 
*/

// ----------------------------------------------------------------------------------------
// TQD3DBox SetOrigin 
// ----------------------------------------------------------------------------------------

void TQD3DBox::SetOrigin(TQ3Point3D theOrigin)
{
	Fail3DErr(Q3Box_SetOrigin(fQD3DBox,&theOrigin));
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::GetOrigin: 
// ----------------------------------------------------------------------------------------

TQ3Point3D TQD3DBox::GetOrigin()
{
	TQ3Point3D theOrigin;
	
	Fail3DErr(Q3Box_GetOrigin(fQD3DBox,&theOrigin));
	
	return theOrigin;
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::SetOrientation 
// ----------------------------------------------------------------------------------------

void TQD3DBox::SetOrientation(TQ3Vector3D theOrientation)
{
	Fail3DErr(Q3Box_SetOrientation(fQD3DBox,&theOrientation));
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::GetOrientation: 
// ----------------------------------------------------------------------------------------

TQ3Vector3D TQD3DBox::GetOrientation()
{
	TQ3Vector3D theOrientation;
	
	Fail3DErr(Q3Box_GetOrientation(fQD3DBox,&theOrientation));
	
	return theOrientation;
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::SetMajorAxis 
// ----------------------------------------------------------------------------------------

void TQD3DBox::SetMajorAxis(TQ3Vector3D theMajorAxis)
{
	Fail3DErr(Q3Box_SetMajorAxis(fQD3DBox,&theMajorAxis));
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::GetMajorAxis: 
// ----------------------------------------------------------------------------------------

TQ3Vector3D TQD3DBox::GetMajorAxis()
{
	TQ3Vector3D theMajorAxis;
	
	Fail3DErr(Q3Box_GetMajorAxis(fQD3DBox,&theMajorAxis));
	
	return theMajorAxis;
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::SetMinorAxis 
// ----------------------------------------------------------------------------------------

void TQD3DBox::SetMinorAxis(TQ3Vector3D theMinorAxis)
{
	Fail3DErr(Q3Box_SetMinorAxis(fQD3DBox,&theMinorAxis));
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::GetMinorAxis: 
// ----------------------------------------------------------------------------------------

TQ3Vector3D TQD3DBox::GetMinorAxis()
{
	TQ3Vector3D theMinorAxis;
	
	Fail3DErr(Q3Box_GetMinorAxis(fQD3DBox,&theMinorAxis));
	
	return theMinorAxis;
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::SetFaceAttributeSet 
// ----------------------------------------------------------------------------------------

void TQD3DBox::SetFaceAttributeSet(unsigned long faceIndex, TQ3AttributeSet theAttributeSet)
{
	Fail3DErr(Q3Box_SetFaceAttributeSet(fQD3DBox,faceIndex,theAttributeSet));
}

// ----------------------------------------------------------------------------------------
// TQD3DBox::GetFaceAttributeSet: 
// ----------------------------------------------------------------------------------------

TQ3AttributeSet TQD3DBox::GetFaceAttributeSet(unsigned long faceIndex)
{
	TQ3AttributeSet theAttributeSet;
	
	Fail3DErr(Q3Box_GetFaceAttributeSet(fQD3DBox,faceIndex,&theAttributeSet));
	
	return theAttributeSet;
}

