// ----------------------------------------------------------------------------------------
// UQD3DLight.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// ----------------------------------------------------------------------------------------


#ifndef __UQD3DLight__
#include "UQD3DLight.h"
#endif

#ifndef  __UQD3DCOMMON__
#include "UQD3DCommon.h"
#endif

// ========================================================================================
// CLASS TQD3DLight
// ========================================================================================



// ----------------------------------------------------------------------------------------
// TQD3DLight::IQD3DLight: 
// ----------------------------------------------------------------------------------------


TQD3DLight::TQD3DLight(lightingType theLightType, TQ3Boolean isOn,
							float brightness,TQ3ColorRGB lightColor, TQ3Boolean castShadows,
							TQ3Vector3D direction, TQ3AttenuationType attenuationType, TQ3Point3D location,
							float hotAngle, float outerAngle, TQ3FallOffType fallOff) :
		f3DLightObject(NULL),
		fLightType(theLightType),
		fIsOn(isOn),
		fBrightness(0.0),
		fLightColor(),
		fCastsShadows(kQ3False),
		fDirection(zeroedVector),
		fAttenuation(kQ3AttenuationTypeNone),
		fLocation(zeroedOrigin),
		fHotAngle(0.0),
		fOuterAngle(0.0),
		fFallOff(kQ3FallOffTypeNone)
{
	//WILL BE MAKING A LIGHT OBJECT
	
	//Problems: 1.) Need to empty the light group to change, sort of
	//          2.) How do you edit the light group
	
	
	TQ3LightData			theLightData;

	//General Information for all lights...
	theLightData.isOn 		= isOn;									
	theLightData.color 		= lightColor;									
	theLightData.brightness = brightness;

	switch(theLightType)
	{
		case kAmbientLight:
			{
				f3DLightObject = Q3AmbientLight_New(&theLightData);
				::ThrowIfNULL_AC(f3DLightObject);
				break;
			}
			
		case kDirectionalLight:
			{
				TQ3DirectionalLightData	theDirectionalLightData;
				
				theDirectionalLightData.lightData 	 = theLightData;						
				theDirectionalLightData.castsShadows = castShadows;						
				theDirectionalLightData.direction 	 = direction;					
				
				f3DLightObject = Q3DirectionalLight_New(&theDirectionalLightData);
				::ThrowIfNULL_AC(f3DLightObject);
				break;
			}
		
		case kPointLight:
			{
				TQ3PointLightData	thePointLightData;
				
				thePointLightData.lightData 	= theLightData;						
				thePointLightData.castsShadows 	= castShadows;	
				thePointLightData.attenuation	= attenuationType;
				thePointLightData.location		= location;	
	
				f3DLightObject = Q3PointLight_New(&thePointLightData);
				::ThrowIfNULL_AC(f3DLightObject);
				break;
			}
					
		case kSpotLight:
			{
				TQ3SpotLightData	theSpotLightData;
				
				theSpotLightData.lightData 		= theLightData;						
				theSpotLightData.castsShadows 	= castShadows;	
				theSpotLightData.attenuation	= attenuationType;
				theSpotLightData.location		= location;
				theSpotLightData.direction 		= direction;
				theSpotLightData.hotAngle		= hotAngle;
				theSpotLightData.outerAngle		= outerAngle;
				theSpotLightData.fallOff		= fallOff;					

				f3DLightObject = Q3SpotLight_New(&theSpotLightData);
				::ThrowIfNULL_AC(f3DLightObject);
				break;
			}		
	
	}
		//Q3Object_Dispose(f3DLightObject);
	
}

// ----------------------------------------------------------------------------------------
// TQD3DLight destructor 
// ----------------------------------------------------------------------------------------


TQD3DLight::~TQD3DLight()
{
	Q3Object_Dispose(f3DLightObject);
}

/*
// ----------------------------------------------------------------------------------------
// TQD3DLight::SetData 
// ----------------------------------------------------------------------------------------
void TQD3DLight::SetData(TQ3LightData theLightData)
{
	Fail3DErr(Q3Light_SetData(fQD3DLight,&theLightData));	
} 
*/

