//----------------------------------------------------------------------------------------
// UQD3DView.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

//QD3D - Must be first....
#ifndef  QD3D_h
#include "QD3D.h"
#endif

//This file
#ifndef __UQD3DVIEW__
#include "UQD3DView.h"
#endif

//MacApp

#ifndef __UROTATIONBEHAVIOR__
#include "URotationBehavior.h"
#endif

#ifndef __UQD3DLight__
#include "UQD3DLight.h"
#endif

#ifndef __UQD3DUTILS__
#include "UQD3DView.h"
#endif

//QD3D
#ifndef   QD3DCamera_h
#include "QD3DCamera.h" 
#endif

#ifndef   QD3DDrawContext_h
#include "QD3DDrawContext.h" 
#endif

#ifndef   QD3DGeometry_h
#include "QD3DGeometry.h" 
#endif

#ifndef   QD3DGroups_h
#include "QD3DGroup.h" 
#endif

#ifndef   QD3DRenderer_h
#include "QD3DRenderer.h"
#endif

#ifndef   QD3DShader_h
#include "QD3DShader.h" 
#endif

#ifndef   QD3DStorage_h
#include "QD3DStorage.h"
#endif

//for Rotation
#ifndef   QD3DTransform_h
#include "QD3DTransform.h" 
#endif

#ifndef   QD3DView_h
#include "QD3DView.h" 
#endif

//========================================================================================
// Global Initialization Procedure
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// InitUQD3DView: 
//----------------------------------------------------------------------------------------

void InitUQD3DView()
{
	// So the linker doesn't dead strip class info 
	MA_REGISTER_CLASS(TQD3DView);
	
	TQ3Status theStatus = kQ3Failure;

	if(HasQuickDraw3D_AC())
		theStatus = Q3Initialize();

#if qDebug
	if (!HasQuickDraw3D_AC())
		DebugStr("\p InitUQD3DView called but QD3D isn't present.");
	else if(theStatus == kQ3Failure)
		DebugStr("\p QD3D Failed to load and initialize. This is bad.");
#endif
} // InitUQD3DView 

//========================================================================================
// CLASS TQD3DView
//========================================================================================
#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS_M1(TQD3DView, Inherited);

//----------------------------------------------------------------------------------------
// TQD3DView constructor 
//----------------------------------------------------------------------------------------

TQD3DView::TQD3DView(const CViewPoint& itsLocation,
						 	const CViewPoint& itsSize) :
	TView(itsLocation,itsSize, sizeFixed, sizeFixed),
	
	fQD3DView(Q3View_New()),
	fDrawContext(NULL),
	fInterpolation(NULL),
	fBackfacing(NULL),
	fFillStyle(NULL),
	fIlluminationShader(NULL),
	fTexture(NULL),
	fLightGroup(NULL),
	fGeometeryObject(NULL)
{
	::ThrowIfNULL_AC(fQD3DView);
} 

//----------------------------------------------------------------------------------------
// TQD3DView destructor 
//----------------------------------------------------------------------------------------

TQD3DView::~TQD3DView()
{
	//Cleanup after yourself....
	Q3Object_Dispose(fQD3DView);
	
	if (fDrawContext)
		Q3Object_Dispose(fDrawContext);
	if (fInterpolation)
		Q3Object_Dispose(fInterpolation);
	if (fBackfacing)
		Q3Object_Dispose(fBackfacing);
	if (fFillStyle)
		Q3Object_Dispose(fFillStyle);
	if (fIlluminationShader)
		Q3Object_Dispose(fIlluminationShader);
	if (fTexture)
		Q3Object_Dispose(fTexture);
}

//----------------------------------------------------------------------------------------
// TQD3DView::ReadFields 
//----------------------------------------------------------------------------------------

void TQD3DView::ReadFields(CStream_AC* aStream)
{
	 Inherited::ReadFields(aStream);
}


//----------------------------------------------------------------------------------------
// TQD3DView::SetDrawingContext: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetDrawingContext(TQ3ColorARGB backgroundColor, TQ3Boolean useDoubleBuffering) 
{
	
	TQ3DrawContextData			theDrawContextData;
	TQ3MacDrawContextData		theMacDrawContextData;
	//TQ3DrawContextObject 		theDrawContext;
	
	//background
	theDrawContextData.clearImageMethod = kQ3ClearMethodWithColor;
	theDrawContextData.clearImageColor = backgroundColor;
	
	//pane
	theDrawContextData.paneState = kQ3True; //use bounds? if false, use whole window
	
	if (theDrawContextData.paneState == kQ3True)
	{
		CViewRect theFrame = this->GetFrame();
		theDrawContextData.pane.min.x = theFrame.left - 1 ; 
		theDrawContextData.pane.max.x = theFrame.right - 1;
		theDrawContextData.pane.min.y = theFrame.top - 1;
		theDrawContextData.pane.max.y = theFrame.bottom - 1;
	}
	
	theDrawContextData.maskState = kQ3False;
	theDrawContextData.doubleBufferState = useDoubleBuffering;
	
	//Mac Specific (will need windows version later... ;->)
	theMacDrawContextData.drawContextData = theDrawContextData;
	theMacDrawContextData.window = (CWindowPtr)this->GetWindow()->GetWindowRef();
	theMacDrawContextData.library = kQ3Mac2DLibraryNone;
	theMacDrawContextData.viewPort = NULL; //GX ONLY!!!
	theMacDrawContextData.grafPort = NULL; 
	
	//Create the draw context
	fDrawContext = Q3MacDrawContext_New(&theMacDrawContextData);
	::ThrowIfNULL_AC(fDrawContext);
	
	//Assign new context to view
	Fail3DErr(Q3View_SetDrawContext(fQD3DView,fDrawContext));
	
}

//----------------------------------------------------------------------------------------
// TQD3DView::SetLightGroup: 
//----------------------------------------------------------------------------------------

void TQD3DView::SetLightGroup()
{
	Fail3DErr(Q3View_SetLightGroup(fQD3DView, fLightGroup));
}

//----------------------------------------------------------------------------------------
// TQD3DView::CreateLightObject: 
//----------------------------------------------------------------------------------------

TQ3GroupPosition TQD3DView::CreateLightObject(lightingType theLightType, TQ3Boolean isOn,float brightness,
											 TQ3ColorRGB lightColor, TQ3Boolean castShadows, TQ3Vector3D direction,
											TQ3AttenuationType attenuationType, TQ3Point3D location, float hotAngle,
											float outerAngle, TQ3FallOffType fallOff)
{
	TQ3GroupPosition theGroupPosition;

	//Logic: If this group does not have a light group yet, we need to add one before we can assign a light
	//		 to it.
	if(!fLightGroup)
	{
		fLightGroup = Q3LightGroup_New();
		::ThrowIfNULL_AC(fLightGroup);
	}
	
	//Create the LightObject !!! This call creates a memory leak
	TQD3DLight* theLightObject = TH_new TQD3DLight(theLightType,isOn,brightness,lightColor,castShadows,direction,attenuationType,location,
								 hotAngle,outerAngle,fallOff);

	// Add Light to Group
	theGroupPosition = Q3Group_AddObject(fLightGroup, theLightObject->f3DLightObject);
	
		
	return theGroupPosition; 
}

//----------------------------------------------------------------------------------------
// TQD3DView::SetCameraObject: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetCameraObject(TQ3Point3D	cameraLocation,
								TQ3Point3D	pointOfInterest,
								TQ3Vector3D	upVector,
								float		hither,
								float		yon,
								float		fieldOfView) 
{
	
	TQ3CameraObject		theCamera;
	TQ3CameraData		theCameraData;
	TQ3Area				thePane;
	TQ3ViewAngleAspectCameraData	theViewAngleAspectCameraData;

	//Set the Camera placement physics
	theCameraData.placement.cameraLocation  = cameraLocation;
	theCameraData.placement.pointOfInterest = pointOfInterest; 
	theCameraData.placement.upVector = upVector; // 99.9% of the time, this should be the
												 // Y axis
												 
	//Hither and Yon (how far/near can I see on the Z axis)
	//add if hither = 0 debug code)
	theCameraData.range.hither = hither;
	theCameraData.range.yon = yon;

	//View port information
	theCameraData.viewPort.origin.x = -1.0;
	theCameraData.viewPort.origin.y = 1.0;
	theCameraData.viewPort.width = 2.0;
	theCameraData.viewPort.height = 2.0;
	
	//camera specific
	theViewAngleAspectCameraData.cameraData = theCameraData;
	theViewAngleAspectCameraData.fov		= fieldOfView;

	Fail3DErr(Q3DrawContext_GetPane(fDrawContext,&thePane));
	//Chances are, this view is not a perfect square, so don't squash...
	theViewAngleAspectCameraData.aspectRatioXToY = (float)(thePane.max.x-thePane.min.x)/(float)(thePane.max.y-thePane.min.y);
	//theViewAngleAspectCameraData.aspectRatioXToY = 1.0;
	
	//Set the Camera 
	theCamera = Q3ViewAngleAspectCamera_New(&theViewAngleAspectCameraData);
	//assign to the view
	Fail3DErr(Q3View_SetCamera(fQD3DView,theCamera));
}

//----------------------------------------------------------------------------------------
// TQD3DView::SetInterpolation: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetInterpolation(TQ3InterpolationStyle theInterpolation) 
{
	fInterpolation = Q3InterpolationStyle_New(theInterpolation);
}
 
//----------------------------------------------------------------------------------------
// TQD3DView::SetBackfacing: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetBackfacing(TQ3BackfacingStyle theBackfacing) 
{
	fBackfacing = Q3BackfacingStyle_New(theBackfacing);
}

//----------------------------------------------------------------------------------------
// TQD3DView::SetFillStyle: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetFillStyle(TQ3FillStyle theFillStyle) 
{
	fFillStyle = Q3FillStyle_New(theFillStyle);
}

//----------------------------------------------------------------------------------------
// TQD3DView::SetIllumination: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetIllumination(illuminationType theIlluminationType, bool redraw) 
{
	switch(theIlluminationType)
	{
		case kNoIllumination:
			{
				fIlluminationShader = Q3NULLIllumination_New();
				break;
			}
			
		case kLambertIllumination:
			{
				fIlluminationShader = Q3LambertIllumination_New();
				break;
			}
		
		case kPhongIllumination:
			{
				fIlluminationShader = Q3PhongIllumination_New();
				break;
			}
	
	}
	
	if(redraw)
		this->ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TQD3DView::SetRenderer: 
//----------------------------------------------------------------------------------------

void TQD3DView::SetRenderer(TQ3ObjectType renderType, bool redraw, bool useHardwareBypass)
{
	Fail3DErr(Q3View_SetRendererByType(fQD3DView, renderType));
	
	if(useHardwareBypass && (renderType == kQ3RendererTypeInteractive))
		Q3InteractiveRenderer_SetDoubleBufferBypass(GetRenderer(), kQ3True);
		
	
	if(redraw)
		this->ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TQD3DView::GetRenderer: 
//----------------------------------------------------------------------------------------

TQ3RendererObject TQD3DView::GetRenderer()
{
	TQ3RendererObject  theRenderer;
	
	Fail3DErr(Q3View_GetRenderer(fQD3DView,&theRenderer));
	
	return theRenderer;
} 

//----------------------------------------------------------------------------------------
// TQD3DView::GetRendererType: 
//----------------------------------------------------------------------------------------

TQ3ObjectType TQD3DView::GetRendererType()
{
	
	TQ3RendererObject  theRenderer = GetRenderer();
	
	TQ3ObjectType theObjectType = Q3Renderer_GetType(theRenderer);
	
	return theObjectType;
} 

//----------------------------------------------------------------------------------------
// TQD3DView::SetBackgroundColor: 
/*
	Note: QuickDraw3D uses it's own color structures. When calling this method, if you have
		  an RGB value handy that you want to use (maybe you called GetColor), the best thing
		  to do is to get each value from your RGB.x, divide it by 65535, and stuff it back in
		  the TQ3ColorARGB structure.
		  
		  Or, you can just use the easy overloaded version of this function...
	  
*/
//----------------------------------------------------------------------------------------

void TQD3DView::SetBackgroundColor(TQ3ColorARGB* newColor,bool redraw)
{
	Fail3DErr(Q3DrawContext_SetClearImageColor(fDrawContext,newColor));

	if(redraw)
		this->Update();

} 

//----------------------------------------------------------------------------------------
// TQD3DView::SetBackgroundColor: 
//----------------------------------------------------------------------------------------

void TQD3DView::SetBackgroundColor(CRGBColor_AC newColor, float alpha, bool redraw)
{
	TQ3ColorARGB theQ3Color;
	float colorOffset = 65535.0;
	 
	theQ3Color.r = newColor.red/colorOffset;
	theQ3Color.g = newColor.green/colorOffset;
	theQ3Color.b = newColor.blue/colorOffset;
	
	theQ3Color.a = alpha;

	this->SetBackgroundColor(&theQ3Color,redraw);
	
}

//----------------------------------------------------------------------------------------
// TQD3DView::GetRGBBackgroundColor: 
//----------------------------------------------------------------------------------------

CRGBColor_AC TQD3DView::GetRGBBackgroundColor()
{
	CRGBColor_AC 	  theRGBColor;
	TQ3ColorARGB  theQ3Color;
	
	Fail3DErr(Q3DrawContext_GetClearImageColor(fDrawContext,&theQ3Color));
	
	float colorOffset = 65535.0;
	 
	theRGBColor.red 	= theQ3Color.r * colorOffset;
	theRGBColor.green 	= theQ3Color.g * colorOffset;
	theRGBColor.blue 	= theQ3Color.b * colorOffset;
	
	return theRGBColor;
	
} 

//----------------------------------------------------------------------------------------
// TQD3DView::GetBackgroundColor: 
//----------------------------------------------------------------------------------------

TQ3ColorARGB* TQD3DView::GetBackgroundColor()
{
	TQ3ColorARGB* theQ3Color = NULL;

	Fail3DErr(Q3DrawContext_GetClearImageColor(fDrawContext,theQ3Color));
	
	return theQ3Color;
	
} 

//----------------------------------------------------------------------------------------
// TQD3DView::Draw: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::Draw(const CViewRect&  /*area*/) 
{
	
	if(fDrawContext)
	{
		this->SubmitDrawingData();
	}
}

//----------------------------------------------------------------------------------------
// TQD3DView::RenderObjects: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SubmitDrawingData() 
{
	TQ3ViewStatus theViewStatus;
	
	this->GetWindow()->Focus();
	Fail3DErr(Q3View_StartRendering(fQD3DView));
	
	do {
			if(fInterpolation)
				Fail3DErr(Q3Style_Submit(fInterpolation,fQD3DView));
			if(fBackfacing)
				Fail3DErr(Q3Style_Submit(fBackfacing,fQD3DView));
			if(fFillStyle)
				Fail3DErr(Q3Style_Submit(fFillStyle,fQD3DView));
			if(fIlluminationShader)
				Fail3DErr(Q3Shader_Submit(fIlluminationShader, fQD3DView));
			if(fTexture)
				Fail3DErr(Q3Shader_Submit(fTexture, fQD3DView));
			if(IsRotating())
			{
				TRotationBehavior* theRotationBehavior = GetRotationBehavior();
				TQ3Matrix4x4 theRotationMatrix = theRotationBehavior->GetRotationMatrix();
				Fail3DErr(Q3MatrixTransform_Submit(&theRotationMatrix,fQD3DView));
			}
			
			this->RenderGeometries();
			
			theViewStatus = Q3View_EndRendering(fQD3DView);
		
		}while (theViewStatus == kQ3ViewStatusRetraverse);
	
	this->Focus();
}

//----------------------------------------------------------------------------------------
// TQD3DView::SetTextureFromPICTResource: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetTextureFromPICTResource(short resourceID, bool redraw) 
{
	PicHandle			thePicture;

	//Get the pict from the resource			
	thePicture = GetPicture(resourceID);
	::ThrowIfNULL_AC(thePicture); 
	
	this->SetTextureFromPICT(thePicture, redraw); 

	ReleaseResource((Handle)thePicture);
}

//----------------------------------------------------------------------------------------
// TQD3DView::GetTextureFromPICT: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::SetTextureFromPICT(PicHandle thePicture, bool redraw) 
{
	TQ3StoragePixmap 	theStoragePixmap;
	TQ3TextureObject	theTexture;

	//Create Pixmap data
	CreatePixmapData(thePicture,&theStoragePixmap);
	
	
	//Make the Pixmap texture
	theTexture = Q3PixmapTexture_New(&theStoragePixmap);
	::ThrowIfNULL_AC(theTexture);
		
	fTexture = Q3TextureShader_New(theTexture);
	::ThrowIfNULL_AC(fTexture);

	//Cleanup
	Q3Object_Dispose (theTexture);
	
	Q3Object_Dispose (theStoragePixmap.image);	// Disposes of extra reference to storage obj
												// from CreateMyTexturePixmapData
	if(redraw)
		this->SubmitDrawingData();
} 

//----------------------------------------------------------------------------------------
// TQD3DView::CreatePixmapData: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::CreatePixmapData(PicHandle	thePicture,TQ3StoragePixmap* theStoragePixmap)
{
	Ptr 					pixAddr;
	Rect 					rectGW;
	GWorldPtr 				pGWorld,oldGW;
	PixMapHandle 			hPixMap;
	GDHandle				oldGD;
	unsigned long			size,sizeX,sizeY,rowBytes;
	TQ3StorageObject		theStorageObject;

	//How big is this PICT?					
	sizeX = 	(**thePicture).picFrame.right  - (**thePicture).picFrame.left;
	sizeY = 	(**thePicture).picFrame.bottom - (**thePicture).picFrame.top,

	//GWorld to Hold the PICT
	::SetRect(&rectGW, 0, 0, sizeX, sizeY);					// set dimensions
	::ThrowIfOSErr_AC(NewGWorld(&pGWorld, 32, &rectGW, 0, 0, 0L));	// make 32 bit gworld (must be 32bit!)

	hPixMap = GetGWorldPixMap(pGWorld);							
	pixAddr = GetPixBaseAddr(hPixMap);							// get addr of pixels
	rowBytes = (unsigned long)(**hPixMap).rowBytes & 0x3fff;	

	::GetGWorld(&oldGW, &oldGD);								// save current port
	::SetGWorld(pGWorld, nil);									// set port to GWorld
	::LockPixels(hPixMap);										// lock pixels
	::EraseRect(&rectGW);										// clear the GWorld
	::DrawPicture(thePicture, &rectGW);							// draw PICT
	::SetGWorld(oldGW, oldGD);									// restore port 

	//store in memory
	size = rowBytes * sizeY;									// memory = rowBytes * # of rows
	theStorageObject = Q3MemoryStorage_New((unsigned char *)pixAddr,size);
	

	//Finally, set the data....
					
	theStoragePixmap->image		= theStorageObject;			// image = storage object
	theStoragePixmap->width 	= sizeX;					// set pixel width
	theStoragePixmap->height	= sizeY;					// set pixel height
	theStoragePixmap->rowBytes 	= rowBytes;					// set rowBytes
	
	theStoragePixmap->pixelSize = 32;						// 32 bit depth
	theStoragePixmap->pixelType	= kQ3PixelTypeRGB32;		
	theStoragePixmap->bitOrder	= kQ3EndianBig;				// It's a Mac, after all....
	theStoragePixmap->byteOrder	= kQ3EndianBig;
}

//----------------------------------------------------------------------------------------
// TQD3DView::CreateBox: 
//----------------------------------------------------------------------------------------
	
TQ3GeometryObject TQD3DView::CreateBox(float ambience,TQ3ColorRGB theColor,TQ3Point3D  theOrigin,
										TQ3Vector3D theOrientation,TQ3Vector3D majorAxis,TQ3Vector3D minorAxis) 
{
	TQ3AttributeSet 	theAttributes;
	TQ3BoxData			theBoxData;
	TQ3GeometryObject	theBox;

	//Setup the Attributes for this box (Color, Ambience)
	theAttributes = Q3AttributeSet_New();
	::ThrowIfNULL_AC(theAttributes);
	Fail3DErr(Q3AttributeSet_Add(theAttributes, kQ3AttributeTypeDiffuseColor, &theColor));
	Fail3DErr(Q3AttributeSet_Add(theAttributes, kQ3AttributeTypeAmbientCoefficient, &ambience));
	
	//Set Box Data
	theBoxData.origin 			= theOrigin;
	theBoxData.orientation 		= theOrientation;
	theBoxData.majorAxis 		= majorAxis;
	theBoxData.minorAxis 		= minorAxis;
	theBoxData.faceAttributeSet = NULL;
	theBoxData.boxAttributeSet 	= theAttributes;
	
	theBox = Q3Box_New(&theBoxData);
	::ThrowIfNULL_AC(theBox);
	
	Q3Object_Dispose(theAttributes);
	
	return theBox;
}

//----------------------------------------------------------------------------------------
// TQD3DView::RenderGeometries: 
//----------------------------------------------------------------------------------------
	
void TQD3DView::RenderGeometries() 
{
	//Subclass responsiblity...for now.
}

//----------------------------------------------------------------------------------------
// TQD3DView::IsRotating: 
//----------------------------------------------------------------------------------------
	
bool TQD3DView::IsRotating() 
{
	bool theBehaviorExists = false;
	
	if(GetRotationBehavior())
		theBehaviorExists = true;
		
	return	theBehaviorExists;
} 

//----------------------------------------------------------------------------------------
// TQD3DView::GetRotationBehavior: 
//----------------------------------------------------------------------------------------
	
TRotationBehavior* TQD3DView::GetRotationBehavior() 
{
	TRotationBehavior* theRotationBehavior = NULL;
	
	theRotationBehavior = dynamic_cast<TRotationBehavior*> (GetBehaviorWithIdentifier(kRotationBehavior));
		
	return	theRotationBehavior;
} 

#if qDrag

//----------------------------------------------------------------------------------------
// TQD3DView::WillAcceptDrop
//----------------------------------------------------------------------------------------

bool TQD3DView::WillAcceptDrop(CDragItemIterator& dragItemIterator)
{
	bool returnVal = false;

	if (dragItemIterator.GetItemCount() == 1)
		returnVal = dragItemIterator.First()->FlavorExists(kPictType_AC);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TQD3DView::DoMakeDragDropCommand
//----------------------------------------------------------------------------------------

TCommand* TQD3DView::DoMakeDragDropCommand(CommandNumber 		itsCommandNumber,
										CDragItemIterator&	dragItemIterator)
{
	TCommand 		*returnCommand = NULL;
	TCommandHandler *itsContext = GetContext(itsCommandNumber);
	
	switch(itsCommandNumber)
	{
		case cDrop:
			{
				Handle		pictureHandle = NULL;
				TDragItem	*firstItem = dragItemIterator.First();
				
				firstItem->FocusOnFlavor(kPictType_AC);
				pictureHandle = firstItem->GetDataAsHandle();
					
				returnCommand = TH_new TQD3DDropCommand((PicHandle)pictureHandle, this, itsCommandNumber,
											 itsContext, kCanUndo, kCausesChange, itsContext);
			}
			break;
			
		default:
			returnCommand = Inherited::DoMakeDragDropCommand(itsCommandNumber, dragItemIterator);
			break;
	}
	
	return returnCommand;
}

#endif // qDrag

//========================================================================================
// CLASS TQD3DDropCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TQD3DDropCommand::TQD3DDropCommand
//----------------------------------------------------------------------------------------

TQD3DDropCommand::TQD3DDropCommand(PicHandle newPicHandle,
							 	TQD3DView* theView,
								CommandNumber itsCommandNumber,
								TCommandHandler* itsContext,
								bool canUndo,
								bool causesChange,
								MDependable_AC* objectToNotify) :
	TCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify),

	fSavedDataHandle(newPicHandle),
	fQD3DView(theView)
{
}

//----------------------------------------------------------------------------------------
// TQD3DDropCommand::Free
//----------------------------------------------------------------------------------------

TQD3DDropCommand::~TQD3DDropCommand()
{
	if (fSavedDataHandle)
	{
		if (IsAResource_AC((Handle)fSavedDataHandle))
		{
			ReleaseResource((Handle)fSavedDataHandle);
			fSavedDataHandle = NULL;
		}
		else
			fSavedDataHandle = (PicHandle)DisposeIfHandle_AC((Handle)fSavedDataHandle);
	}
}

//----------------------------------------------------------------------------------------
// TQD3DDropCommand::DoIt
//----------------------------------------------------------------------------------------

void TQD3DDropCommand::DoIt()
{
	//PicHandle tempDataHandle = NULL;

	//if (fSavedRsrcID != kNoResource)
	//	fQD3DView->SetTextureFromPICTResource(fSavedRsrcID, true);
	//else
	
	fQD3DView->SetTextureFromPICT(fSavedDataHandle, true);
		
	//fSavedDataHandle = tempDataHandle;
}

//----------------------------------------------------------------------------------------
// TQD3DDropCommand::UndoIt
//----------------------------------------------------------------------------------------

void TQD3DDropCommand::UndoIt()
{
	//DoIt();
}

//----------------------------------------------------------------------------------------
