//----------------------------------------------------------------------------------------
// UQD3DView.h
// Copyright  1997 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UQD3DVIEW__
#define __UQD3DVIEW__

#ifndef __UROTATIONBEHAVIOR__
#include "URotationBehavior.h"
#endif

#ifndef  __UQD3DCOMMON__
#include "UQD3DCommon.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

//QuickDraw3D
#ifndef  QD3DView_h
#include "QD3DView.h"
#endif

#ifndef   QD3DGroups_h
#include "QD3DGroup.h" 
#endif

#ifndef   QD3DLights_h
#include "QD3DLight.h" 
#endif


class TRotationBehavior;
//----------------------------------------------------------------------------------------
// TQD3DView
//----------------------------------------------------------------------------------------

class TQD3DView: public TView
{
	MA_DECLARE_CLASS;

  public:	
  
	TQD3DView(const CViewPoint& itsLocation = gZeroVPt,
					const CViewPoint& itsSize = gZeroVPt); 

	virtual ~TQD3DView();
	
	virtual void ReadFields(CStream_AC* aStream);

	virtual void Draw(const CViewRect&  area); 

	virtual void SubmitDrawingData(); 

	virtual void SetDrawingContext(TQ3ColorARGB backgroundColor, TQ3Boolean useDoubleBuffering);
	
	virtual	void SetCameraObject(TQ3Point3D		cameraLocation,
								 TQ3Point3D		pointOfInterest,
								 TQ3Vector3D	upVector,
								 float		    hither,
								 float			yon,
								 float			fieldOfView);
	
	// ---------------------------------------------------------
	// Lighting
	// ---------------------------------------------------------
	
	virtual void SetLightGroup();
								
	virtual TQ3GroupPosition CreateLightObject(lightingType theLightType,
												TQ3Boolean isOn,
												float brightness,
												TQ3ColorRGB lightColor,
												TQ3Boolean castShadows = kQ3False,
												TQ3Vector3D direction = zeroedVector,
												TQ3AttenuationType AttenuationType = kQ3AttenuationTypeNone,
												TQ3Point3D location = zeroedOrigin,
												float hotAngle = 0.0,
												float outerAngle = 0.0,
												TQ3FallOffType fallOff = kQ3FallOffTypeNone);

	// ---------------------------------------------------------
	// Rendering Services
	// ---------------------------------------------------------
						  					 
	virtual void SetRenderer(TQ3ObjectType renderType, bool redraw, bool useHardwareBypass);
	//UseHardwareBypass tells QD3D to use harware if available and is fast enough 
	//not to use DoubleBuffering
		
	virtual TQ3RendererObject	GetRenderer();
	
	virtual TQ3ObjectType		GetRendererType();
	
	// ---------------------------------------------------------
	// Sytles and Shading
	// ---------------------------------------------------------

	virtual void SetInterpolation(TQ3InterpolationStyle theInterpolation);
	
	inline TQ3StyleObject GetInterpolation() 	{return fInterpolation;}

	virtual void SetBackfacing(TQ3BackfacingStyle theBackfacing);
	
	inline TQ3StyleObject GetBackfacing() 		{return fBackfacing;}

	virtual void SetFillStyle(TQ3FillStyle theFillStyle);
	
	inline TQ3StyleObject GetFillStyle() 		{return fFillStyle;}

	virtual void SetIllumination(illuminationType theIlluminationType, bool redraw);
			
	//inline illuminationType GetIllumination() 	{return fIlluminationShader};

		
	// virtual void SetShading(TQ3ShaderObject shaderType);
	
	// ---------------------------------------------------------
	// Background Color accessors and setters.
	// ---------------------------------------------------------

	virtual void SetBackgroundColor(TQ3ColorARGB* newColor, bool redraw);
	
	virtual void SetBackgroundColor(CRGBColor_AC newColor, float alpha, bool redraw);	
		
	virtual CRGBColor_AC	 	GetRGBBackgroundColor();
	
	virtual TQ3ColorARGB* 	GetBackgroundColor();
	
	// ---------------------------------------------------------
	// Texture Mapping
	// ---------------------------------------------------------
	
	virtual void SetTextureFromPICTResource(short resourceID, bool redraw);
	
	virtual void SetTextureFromPICT(PicHandle thePicture, bool redraw);
	
	virtual void CreatePixmapData(PicHandle	thePicture,TQ3StoragePixmap* theStoragePixmap);

	// ---------------------------------------------------------
	// Drag and Drop
	// ---------------------------------------------------------

#if qDrag	
	virtual bool  WillAcceptDrop(CDragItemIterator& dragItemIterator);

	virtual TCommand* DoMakeDragDropCommand(CommandNumber 		itsCommandNumber,
										CDragItemIterator&	dragItemIterator);
#endif	
	// ---------------------------------------------------------
	// Cursor Handling
	// ---------------------------------------------------------
		
	//virtual	void DoSetCursor(const CViewPoint& localPoint, RgnHandle cursorRegion);
	
	// ---------------------------------------------------------
	// Rotations
	// ---------------------------------------------------------
	
	virtual bool IsRotating();
		
	virtual TRotationBehavior* GetRotationBehavior();
	
	
	// Temporary - For Testing Only (?)
	
	
	virtual TQ3GeometryObject CreateBox(float ambience,TQ3ColorRGB theColor,TQ3Point3D theOrigin,
										TQ3Vector3D theOrientation,TQ3Vector3D majorAxis,
										TQ3Vector3D minorAxis);
	
	virtual void RenderGeometries();


	// ---------------------------------------------------------
	// Data Members
	// ---------------------------------------------------------
	
  protected:
  
  	TQ3ViewObject		 	fQD3DView;
	TQ3DrawContextObject  	fDrawContext;
	TQ3StyleObject			fInterpolation;
	TQ3StyleObject			fBackfacing;
	TQ3StyleObject			fFillStyle;
	TQ3ShaderObject			fIlluminationShader;
	TQ3ShaderObject			fTexture;
	TQ3GroupObject			fLightGroup;
	TQ3GeometryObject*		fGeometeryObject;
};


//========================================================================================
// TQD3DDropCommand
//========================================================================================

class TQD3DDropCommand : public TCommand
{
public:
		TQD3DDropCommand(PicHandle newPicHandle = NULL,
							 	TQD3DView* theView = NULL,
								CommandNumber itsCommandNumber = cNoCommand,
								TCommandHandler* itsContext = NULL,
								bool canUndo = true,
								bool causesChange = true,
								MDependable_AC* objectToNotify = NULL);
			
		virtual ~TQD3DDropCommand();	

		virtual void DoIt();
		
		virtual void UndoIt();

protected:
	PicHandle	fSavedDataHandle;
	TQD3DView*	fQD3DView;
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

extern void InitUQD3DView();
	// Call this routine at initialization time

#endif
