//----------------------------------------------------------------------------------------
// UQD3DViewWindow.cp 
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef  QD3D_h
#include "QD3D.h"
#endif

#ifndef __UQD3DViewWindow__
#include "UQD3DViewWindow.h"
#endif


#ifndef __UMYQD3DBOXVIEW__
#include "UMyQD3DBoxView.h"
#endif

static const float kOneHundred = 100.0;

//========================================================================================
// CLASS TQD3DViewWindow
//========================================================================================
#undef Inherited
#define Inherited TWindow


MA_DEFINE_CLASS_M1(TQD3DViewWindow, Inherited);

//----------------------------------------------------------------------------------------
// TQD3DViewWindow Constructor.
//----------------------------------------------------------------------------------------


TQD3DViewWindow::TQD3DViewWindow()
	:	fQD3DView(NULL),
		fRotationPopup(NULL),
		fRotationCheckbox(NULL),
		fShadingCluster(NULL),
		fDragTextureCheckbox(NULL),
		fXOrigin(NULL),
		fYOrigin(NULL),
		fZOrigin(NULL),
		fXOrientation(NULL),
		fYOrientation(NULL),
		fZOrientation(NULL),
		fXMajor(NULL),
		fYMajor(NULL),
		fZMajor(NULL),
		fXMinor(NULL),
		fYMinor(NULL),
		fZMinor(NULL),
		fXRotView(NULL),
		fYRotView(NULL),
		fZRotView(NULL)
{

}


//----------------------------------------------------------------------------------------
// TQD3DViewWindow Destructor.
//----------------------------------------------------------------------------------------


TQD3DViewWindow::~TQD3DViewWindow()
{
	
}

//----------------------------------------------------------------------------------------
// TQD3DViewWindow DoPostCreate.
//----------------------------------------------------------------------------------------


void TQD3DViewWindow::DoPostCreate(TDocument* itsDocument)
{
	// Do these FindSubViews first, since inherited DoPostCreate will call DoEvent.
	fQD3DView = dynamic_cast_AC(TQD3DView*,FindSubView(kQD3DView));
	fShadingCluster = dynamic_cast_AC(TCluster*,FindSubView(kShadingCluster));
	
	SetupRotationData();
	
	fDragTextureCheckbox = dynamic_cast_AC(TCheckBox*,FindSubView(kDragTextureCheckbox));
	
	if(fQD3DView->GetDroppable())
		fDragTextureCheckbox->SetState(true,false);
	
	//Geometry Data	
	fXOrigin = dynamic_cast_AC(TNumberText*,FindSubView(kXOrigin));
	fYOrigin = dynamic_cast_AC(TNumberText*,FindSubView(kYOrigin));
	fZOrigin = dynamic_cast_AC(TNumberText*,FindSubView(kZOrigin));
	fXOrientation = dynamic_cast_AC(TNumberText*,FindSubView(kXOrientation));
	fYOrientation = dynamic_cast_AC(TNumberText*,FindSubView(kYOrientation));
	fZOrientation = dynamic_cast_AC(TNumberText*,FindSubView(kZOrientation));
	fXMajor = dynamic_cast_AC(TNumberText*,FindSubView(kXMajorAxis));
	fYMajor = dynamic_cast_AC(TNumberText*,FindSubView(kYMajorAxis));
	fZMajor = dynamic_cast_AC(TNumberText*,FindSubView(kZMajorAxis));
	fXMinor = dynamic_cast_AC(TNumberText*,FindSubView(kXMinorAxis));
	fYMinor = dynamic_cast_AC(TNumberText*,FindSubView(kYMinorAxis));
	fZMinor = dynamic_cast_AC(TNumberText*,FindSubView(kZMinorAxis));
	
	SetupBoxData(dynamic_cast_AC(TMyQD3DBoxView*,fQD3DView)->fBox);

	// Set the rendering and shading
	dynamic_cast_AC(TRadio*, FindSubView(kInteractiveRadio))->SetState(true, kRedraw);
	dynamic_cast_AC(TRadio*, FindSubView(kLambertRadio))->SetState(true, kRedraw);
	
	Inherited::DoPostCreate(itsDocument);

}

//----------------------------------------------------------------------------------------
// TQD3DViewWindow::DoEvent: 
//----------------------------------------------------------------------------------------


void TQD3DViewWindow::DoEvent(EventNumber eventNumber,
					  			TEventHandler* source,
					  			TEvent* event)	// Override
				
{
	IDType origID = dynamic_cast_AC(TView*,source)->GetIdentifier();

	if (fQD3DView)
	switch (eventNumber)
	{
		case mRadioHit:							
			switch (origID)
			{
				//rendering radio's
				case kInteractiveRadio:
				{
					fQD3DView->SetRenderer(kQ3RendererTypeInteractive, true, false);
					fShadingCluster->SetEnable(true);
					fShadingCluster->DimState(false,true);
				}
				break;
				
				case kWireFrameRadio:
				{
					fQD3DView->SetRenderer(kQ3RendererTypeWireFrame, true, false);
					fShadingCluster->SetEnable(false);
					fShadingCluster->DimState(true,true);
				}
				break;
				
				// Illumination Shaders
				case kNoIlluminationRadio:
					fQD3DView->SetIllumination(kNoIllumination,true);
				break;
				
				case kPhongRadio:
					fQD3DView->SetIllumination(kPhongIllumination,true);
				break;
				
				case kLambertRadio:
					fQD3DView->SetIllumination(kLambertIllumination,true);
				break;
			}
			break;
			
		case mCheckBoxHit:							
			switch (origID)
			{
				case kRotationCheckbox: 
					TRotationBehavior* theRotationBehavior = fQD3DView->GetRotationBehavior();
					if(fRotationCheckbox->IsOn())
					{
						if(theRotationBehavior)
						{
							theRotationBehavior->Rotate();
						}
						else
						{
							theRotationBehavior = TH_new TRotationBehavior(kRotationBehavior,4,0.5,0.5,0.5); 
																									 // values from the class, if it has them,
																									 // in the beta.
							fQD3DView->AddBehavior(theRotationBehavior);
						}
					}
					else if(theRotationBehavior) //checkbox is off, but there is a rotating behavior
						theRotationBehavior->StopRotate();							
					
					if(fRotationCheckbox->IsOn())
					{
						fRotationPopup->SetEnable(true);
						fRotationPopup->DimState(false,true);

					}
					else
					{
						fRotationPopup->SetEnable(false);
						fRotationPopup->DimState(true,true);
					}
					
				break;
				
				case kDragTextureCheckbox: 
					if(fDragTextureCheckbox->IsOn())
					{
						//Add DragDrop Behavior, if it doesn't have one already
						if(fQD3DView->GetBehaviorWithIdentifier(kDragDropBehavior))
						{
							if (!fQD3DView->GetDroppable())
								fQD3DView->SetDroppable(true);
						}
						else
						{
							TDragDropBehavior* theDragDropBehavior = TH_new TDragDropBehavior(kDragDropBehavior,false,true,kNeverMove,kNoIdentifier);
							theDragDropBehavior->SetOwner(fQD3DView);
						}
					}
					else if(fQD3DView->GetDroppable())
					{
						//Just set the droppable to false, but don't remove the behavior
						fQD3DView->SetDroppable(false);
					}
				break;
			}
			break;

		case mButtonHit:							
			switch (origID)
			{
				case 'setB':
				{
					CRGBColor_AC	newColor;
					CStr255_AC		thePrompt = "Pick a TH_new color";
					CRGBColor_AC theCurrentColor = fQD3DView->GetRGBBackgroundColor();
					
					if (GetColor(kBestSystemLocation, thePrompt, theCurrentColor, newColor))
					{
						fQD3DView->SetBackgroundColor(newColor,1.0,true);
					}
				}
				break;
			}
			break;
			
		case mPopupHit:							
			switch (origID)
			{
				case kTexturePopup:
				{
					short theResourceID = 0;
					TPopup* theTextMapPopup = NULL;
					theTextMapPopup = dynamic_cast_AC(TPopup*,FindSubView(kTexturePopup));
					short theItem  = theTextMapPopup->GetCurrentItem();
					
					switch (theItem)
					{
						case 1:
							theResourceID = kMacAppLogoID;
						break;
						
						case 2:
							theResourceID  = kMacOSLogoID;
						break;
						
						case 3:
							theResourceID = kMapID;
						break;
						
						case 4:
							theResourceID = kOpenDocLogoID;
						break;
						
						default:
							theResourceID = kNoResource_AC;
					}
					
					if (theResourceID != kNoResource_AC)
						fQD3DView->SetTextureFromPICTResource(theResourceID,true);
				}
				break;


				case kRotationPopup:
				{
					short theItem  = fRotationPopup->GetCurrentItem();
					TRotationBehavior* theRotationBehavior = fQD3DView->GetRotationBehavior();
					if(theRotationBehavior)
					{
						if(theItem > 3) 
						{
							float XValue = (float) fXRotView->GetValue()/ kOneHundred;
							float YValue = (float) fYRotView->GetValue()/ kOneHundred;
							float ZValue = (float) fZRotView->GetValue()/ kOneHundred;
					
							theRotationBehavior->SetRotationData(theItem,XValue,YValue,ZValue);

						}
					}
#if qDebug
					else
						DebugStr("\p Send it to QA, it's a bug!");
#endif
				}
				break;
			}
			break;
			
		default:
			Inherited::DoEvent(eventNumber, source, event);
			break;
	}
		
}


//----------------------------------------------------------------------------------------
// TQD3DViewWindow :: SetupRotationData.
//----------------------------------------------------------------------------------------
void TQD3DViewWindow::SetupRotationData()
{
	fRotationPopup = dynamic_cast_AC(TPopup*,FindSubView(kRotationPopup));
	fRotationCheckbox = dynamic_cast_AC(TCheckBox*,FindSubView(kRotationCheckbox));
	

	
	fXRotView = dynamic_cast_AC(TNumberText*,FindSubView(kXRotation));
	fYRotView = dynamic_cast_AC(TNumberText*,FindSubView(kYRotation));
	fZRotView = dynamic_cast_AC(TNumberText*,FindSubView(kZRotation));

	TRotationBehavior* itsRotationBehavior = fQD3DView->GetRotationBehavior();
	
	if(itsRotationBehavior)
	{
		fRotationCheckbox->SetState(true,false); 								//turnon the checkbox
		short itsRotationType = (short)itsRotationBehavior->GetRotationType();	
		fRotationPopup->SetCurrentItem(itsRotationType,true);					//set the popup
		fXRotView->SetValue((int)(itsRotationBehavior->GetXAxisRotation()*kOneHundred),false);
		fYRotView->SetValue((int)(itsRotationBehavior->GetYAxisRotation()*kOneHundred),false);
		fZRotView->SetValue((int)(itsRotationBehavior->GetZAxisRotation()*kOneHundred),false);
	}
	
	if(fRotationCheckbox->IsOn())
	{
		fRotationPopup->SetEnable(true);
		fRotationPopup->DimState(false,false);
	}	// Don't need else, b/c I know it is set in the view, and I am lazy...it's a test app
		// for crying out loud! Stop looking at me like that - it's 10:15 at night!
	
}

//----------------------------------------------------------------------------------------
// TQD3DViewWindow :: SetupBoxData.
//----------------------------------------------------------------------------------------
void TQD3DViewWindow::SetupBoxData(TQD3DBox* aBox)
{
	
	TQ3Point3D theOrigin = aBox->GetOrigin();
	
	fXOrigin->SetValue(theOrigin.x * kOneHundred,true);
	fYOrigin->SetValue(theOrigin.y * kOneHundred,true);
	fZOrigin->SetValue(theOrigin.z * kOneHundred,true);

	TQ3Vector3D theOrientation = aBox->GetOrientation();
	
	fXOrientation->SetValue(theOrientation.x * kOneHundred,true);
	fYOrientation->SetValue(theOrientation.y * kOneHundred,true);
	fZOrientation->SetValue(theOrientation.z * kOneHundred,true);

	TQ3Vector3D theMajorAxis = aBox->GetMajorAxis();
	
	fXMajor->SetValue(theMajorAxis.x * kOneHundred,true);
	fYMajor->SetValue(theMajorAxis.y * kOneHundred,true);
	fZMajor->SetValue(theMajorAxis.z * kOneHundred,true);

	TQ3Vector3D theMinorAxis = aBox->GetMinorAxis();
	
	fXMinor->SetValue(theMinorAxis.x * kOneHundred,true);
	fYMinor->SetValue(theMinorAxis.y * kOneHundred,true);
	fZMinor->SetValue(theMinorAxis.z * kOneHundred,true);
}


//----------------------------------------------------------------------------------------
// TQD3DViewWindow :: CalculateOrigin.
//----------------------------------------------------------------------------------------
void TQD3DViewWindow::CalculateOrigin()
{
	TQ3Point3D theNewOrigin;
	TQD3DBox* aBox = dynamic_cast_AC(TMyQD3DBoxView*,fQD3DView)->fBox;	
	
	theNewOrigin.x = (float)fXOrigin->GetValue()/kOneHundred;
	theNewOrigin.y = (float)fYOrigin->GetValue()/kOneHundred;
	theNewOrigin.z = (float)fZOrigin->GetValue()/kOneHundred;

	aBox->SetOrigin(theNewOrigin);
	fQD3DView->ForceRedraw();
	
}

//----------------------------------------------------------------------------------------
// TQD3DViewWindow :: CalculateOrientation.
//----------------------------------------------------------------------------------------
void TQD3DViewWindow::CalculateOrientation()
{
	TQ3Vector3D theNewOrientation;
	
	theNewOrientation.x = (float)fXOrientation->GetValue()/kOneHundred;
	theNewOrientation.y = (float)fYOrientation->GetValue()/kOneHundred;
	theNewOrientation.z = (float)fZOrientation->GetValue()/kOneHundred;

	dynamic_cast_AC(TMyQD3DBoxView*,fQD3DView)->fBox->SetOrientation(theNewOrientation);
		fQD3DView->ForceRedraw();
	
}

//----------------------------------------------------------------------------------------
// TQD3DViewWindow :: CalculateMajorAxis.
//----------------------------------------------------------------------------------------
void TQD3DViewWindow::CalculateMajorAxis()
{
	TQ3Vector3D theNewMajorAxis;
	
	theNewMajorAxis.x = (float)fXMajor->GetValue()/kOneHundred;
	theNewMajorAxis.y = (float)fYMajor->GetValue()/kOneHundred;
	theNewMajorAxis.z = (float)fZMajor->GetValue()/kOneHundred;

	dynamic_cast_AC(TMyQD3DBoxView*,fQD3DView)->fBox->SetMajorAxis(theNewMajorAxis);	
		fQD3DView->ForceRedraw();

}

//----------------------------------------------------------------------------------------
// TQD3DViewWindow :: CalculateOrigin.
//----------------------------------------------------------------------------------------
void TQD3DViewWindow::CalculateMinorAxis()
{
	TQ3Vector3D theNewMinorAxis;
	
	theNewMinorAxis.x = (float)fXMinor->GetValue()/kOneHundred;
	theNewMinorAxis.y = (float)fYMinor->GetValue()/kOneHundred;
	theNewMinorAxis.z = (float)fZMinor->GetValue()/kOneHundred;

	dynamic_cast_AC(TMyQD3DBoxView*,fQD3DView)->fBox->SetMinorAxis(theNewMinorAxis);		
		fQD3DView->ForceRedraw();

}
