//----------------------------------------------------------------------------------------
// URotationBehavior.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UROTATIONBEHAVIOR__
#include "URotationBehavior.h"
#endif

#ifndef   QD3DMath_h
#include "QD3DMath.h" 
#endif

#ifndef  __UQD3DCOMMON__
#include "UQD3DCommon.h"
#endif


//========================================================================================
// CLASS TRotationBehavior
//========================================================================================

#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TRotationBehavior);

//----------------------------------------------------------------------------------------
// TRotationBehavior constructor
//----------------------------------------------------------------------------------------


TRotationBehavior::TRotationBehavior(IDType itsIdentifier,
										  rotationType itsRotationType,
										  float	itsXAxisAngle,
										  float	itsYAxisAngle,
										  float	itsZAxisAngle) :
	TBehavior(itsIdentifier),

	fRotationType(itsRotationType),
	fXAxisRotation(itsXAxisAngle),
	fYAxisRotation(itsYAxisAngle),
	fZAxisRotation(itsZAxisAngle)
{
	SetRotationData(itsRotationType,itsXAxisAngle,itsYAxisAngle,itsZAxisAngle);
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::Free:
//----------------------------------------------------------------------------------------


TRotationBehavior::~TRotationBehavior()
{
	gApplication->InstallCohandler(fRotationIdler, false);
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::ReadFrom:
//----------------------------------------------------------------------------------------


void TRotationBehavior::ReadFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);
	
	fRotationType 	= (rotationType) aStream->ReadInteger();
	fXAxisRotation 	= ((float)aStream->ReadInteger()) /100.0;
	fYAxisRotation	= ((float)aStream->ReadInteger()) /100.0;
	fZAxisRotation	= ((float)aStream->ReadInteger()) /100.0;
	//aStream->ReadStreamObject(fRotationIdler);
	

}


//----------------------------------------------------------------------------------------
// TRotationBehavior::WriteTo
//----------------------------------------------------------------------------------------


void TRotationBehavior::WriteTo(CStream_AC* aStream) const
{
	Inherited::WriteTo(aStream);

	aStream->WriteInteger((int)fRotationType);
	aStream->WriteInteger((int)fXAxisRotation*100);
	aStream->WriteInteger((int)fZAxisRotation*100);
	aStream->WriteInteger((int)fYAxisRotation*100);
	//aStream->WriteStreamObject(fRotationIdler);
	
}


//----------------------------------------------------------------------------------------
// TRotationBehavior::SetOwner
//----------------------------------------------------------------------------------------


void TRotationBehavior::SetOwner(TEventHandler* itsOwner)
{
	Inherited::SetOwner(itsOwner);
	

#if qDebug	 // Is this attached to a QD3D view?
	if (itsOwner && !dynamic_cast<TQD3DView*>(itsOwner))
	{
		ProgramBreak_AC("###A TRotationBehavior is attached to a non-3D view object");
		//return;
	}
#endif
	if(itsOwner)
		Rotate(); 
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::GetStandardSignature:
//----------------------------------------------------------------------------------------


IDType TRotationBehavior::GetStandardSignature() const
{
	return kRotationBehavior;
}


//----------------------------------------------------------------------------------------
// TRotationBehavior::Rotate
//----------------------------------------------------------------------------------------


void TRotationBehavior::Rotate()
{
	
	TQD3DView* itsView = dynamic_cast<TQD3DView*>(fOwner);
	
#if qDebug	
	if (!itsView)
	{
		ProgramBreak_AC("###A TRotationBehavior is attached to a non-3D view object and is about to cause a crash");
	}
#endif
	
	TQ3Matrix4x4 theRotationMatrix = GetRotationMatrix();
	Q3Matrix4x4_SetIdentity(&theRotationMatrix);
	SetRotationMatrix(theRotationMatrix);
	
	InstallRotationIdler();
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::StopRotate
//----------------------------------------------------------------------------------------


void TRotationBehavior::StopRotate()
{	
	fOwner->RemoveBehavior(this);
	delete this;
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::SetRotationData
//----------------------------------------------------------------------------------------


void TRotationBehavior::SetRotationData(rotationType itsRotateType,float itsXAxisAngle,
											float itsYAxisAngle,float itsZAxisAngle)
{
	SetRotationType(itsRotateType);
	SetAxisData(itsXAxisAngle,itsYAxisAngle,itsZAxisAngle);
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::SetRotationType
//----------------------------------------------------------------------------------------


void TRotationBehavior::SetRotationType(rotationType itsRotateType)
{
	fRotationType 	= itsRotateType;
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::SetAxisData
//----------------------------------------------------------------------------------------


void TRotationBehavior::SetAxisData(float itsXAxisAngle,float itsYAxisAngle,float itsZAxisAngle)
{
	fXAxisRotation = itsXAxisAngle;
	fYAxisRotation = itsYAxisAngle;
	fZAxisRotation = itsZAxisAngle;
}


//----------------------------------------------------------------------------------------
// TRotationBehavior::InstallRotationIdler
//----------------------------------------------------------------------------------------

void TRotationBehavior::InstallRotationIdler()
{	
	fRotationIdler = TH_new TRotationIdler(this);
	gApplication->InstallCohandler(fRotationIdler,true);
	fRotationIdler->SetIdleFreq(1);
}

//----------------------------------------------------------------------------------------
// TRotationBehavior::SubmitDrawingData
//----------------------------------------------------------------------------------------

void TRotationBehavior::SubmitDrawingData()
{	
	TQD3DView* itsView = dynamic_cast<TQD3DView*>(fOwner);
	
#if qDebug	
	if (!itsView)
	{
		ProgramBreak_AC("###A TRotationBehavior is attached to a non-3D view object and is about to cause a crash");
	}
#endif

	itsView->SubmitDrawingData();	
}


//========================================================================================
// CLASS TRotationIdler
//========================================================================================
#undef Inherited
#define Inherited TEventHandler


MA_DEFINE_CLASS(TRotationIdler);

//----------------------------------------------------------------------------------------
// TRotationIdler Constructor
//----------------------------------------------------------------------------------------


TRotationIdler::TRotationIdler(TRotationBehavior* theRotationBehavior) :
	TEventHandler(NULL),

	fRotationBehavior(theRotationBehavior)
{
}

//----------------------------------------------------------------------------------------
// ~TRotationIdler
//----------------------------------------------------------------------------------------


TRotationIdler::~TRotationIdler()
{
}

//----------------------------------------------------------------------------------------
// TRotationIdler::DoIdle
//----------------------------------------------------------------------------------------


bool TRotationIdler::DoIdle(IdlePhase phase)
{
	bool returnVal = false;

	if (phase == idleContinue)
		{
			TQ3Matrix4x4	tempMatrix;
			Q3Matrix4x4_SetIdentity(&tempMatrix);

			switch(fRotationBehavior->GetRotationType())
			{

				case kAboutAxis:
				/*
					Q3Matrix4x4_SetRotateAboutAxis(&tempMatrix,
				*/
					break;
					
 				case kAboutPoint:
 				/*
					Q3Matrix4x4_SetRotateAboutPoint(&tempMatrix,fOriginPoint,fXAxisAngle,fYAxisAngle,fZAxisAngle);
				*/
					break;
					
				case kVectorToVector:
				/*
					Q3Matrix4x4_SetRotateVectorToVector(&tempMatrix,fRotationVector1,fRotationVector2);
				*/
						break;
					
				case kXYZ:
					Q3Matrix4x4_SetRotate_XYZ(&tempMatrix,fRotationBehavior->GetXAxisRotation(),fRotationBehavior->GetYAxisRotation(),
												fRotationBehavior->GetZAxisRotation());
					break;
					
				case kXAxis:
					Q3Matrix4x4_SetRotate_X(&tempMatrix,fRotationBehavior->GetXAxisRotation());
					break;
					
				case kYAxis:
					Q3Matrix4x4_SetRotate_Y(&tempMatrix,fRotationBehavior->GetYAxisRotation());
					break;
					
				case kZAxis:
					Q3Matrix4x4_SetRotate_Z(&tempMatrix,fRotationBehavior->GetZAxisRotation());
					break;
					
				default:
				
				break;	
			}	
			TQ3Matrix4x4 theRotationMatrix = fRotationBehavior->GetRotationMatrix();
			Q3Matrix4x4_Multiply(&theRotationMatrix, &tempMatrix,&theRotationMatrix);
			fRotationBehavior->SetRotationMatrix(theRotationMatrix);
			fRotationBehavior->SubmitDrawingData();	
		}
	else
		returnVal = Inherited::DoIdle(phase); 

	return returnVal;
}

/*
//----------------------------------------------------------------------------------------
// TRotationIdler::SetRotationType
//----------------------------------------------------------------------------------------


void TRotationIdler::SetRotationType(rotationType theRotationType)
{
	fRotationType = theRotationType;
}

//----------------------------------------------------------------------------------------
// TRotationIdler::GetRotationType
//----------------------------------------------------------------------------------------


rotationType TRotationIdler::GetRotationType()
{
	return fRotationType;
}

//----------------------------------------------------------------------------------------
// TRotationIdler::SetAxisData
//----------------------------------------------------------------------------------------


void TRotationIdler::SetAxisData(float XAxis, float YAxis, float ZAxis)
{
	fXAxisAngle = XAxis;
	fYAxisAngle = YAxis;
	fZAxisAngle = ZAxis;
}
//----------------------------------------------------------------------------------------
*/

