//----------------------------------------------------------------------------------------
// URotationBehavior.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UROTATIONBEHAVIOR__
#define __UROTATIONBEHAVIOR__

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef  __UQD3DCOMMON__
#include "UQD3DCommon.h"
#endif

#ifndef __UQD3DVIEW__
#include "UQD3DView.h"
#endif


//----------------------------------------------------------------------------------------
//Forward Declarations
class TQD3DView;
class TRotationIdler;
class TRotationBehavior;

//----------------------------------------------------------------------------------------
// Constants
const IDType kRotationBehavior		= 'rotB';

//========================================================================================
// CLASS TRotationIdler
//========================================================================================

class TRotationIdler : public TEventHandler
{
	MA_DECLARE_CLASS;

public:
	TRotationIdler(TRotationBehavior* theRotationBehavior = NULL);
	
	virtual ~TRotationIdler();

	virtual bool DoIdle(IdlePhase phase);
	
private:
	TRotationBehavior*  fRotationBehavior;

};

//===================================================================================
// Class TQD3DViewWindow
//===================================================================================

class TRotationBehavior : public TBehavior
{
	MA_DECLARE_CLASS;
	
public:
	
	TRotationBehavior(IDType itsIdentifier = kNoIdentifier,
								  rotationType itsRotationType = kXYZ,
								  float	itsXAxisAngle = 0.0,
								  float	itsYAxisAngle = 0.0,
								  float	itsZAxisAngle = 0.0);
	virtual ~TRotationBehavior();

	virtual void ReadFrom(CStream_AC* aStream);
	virtual void WriteTo(CStream_AC* aStream) const;
	virtual void SetOwner(TEventHandler* itsOwner);
	virtual void Rotate();
	virtual void StopRotate();
	virtual	void SetRotationData(rotationType itsRotateType,float itsXAxisAngle,
									float itsYAxisAngle,float itsZAxisAngle);
	virtual void SetRotationType(rotationType itsRotateType);
	virtual	rotationType GetRotationType() {return fRotationType;};
	inline	float GetXAxisRotation() 	{return fXAxisRotation;}
	inline	float GetYAxisRotation()	{return fYAxisRotation;}
	inline	float GetZAxisRotation()	{return fZAxisRotation;};

	virtual	void SetAxisData(float itsXAxisAngle,float itsYAxisAngle,float itsZAxisAngle);
	virtual void InstallRotationIdler();
	inline	TQ3Matrix4x4 GetRotationMatrix() {return fRotationMatrix;}
	inline	void SetRotationMatrix(TQ3Matrix4x4 itsRotationMatrix) { fRotationMatrix = itsRotationMatrix;}
	virtual void SubmitDrawingData();
	virtual IDType GetStandardSignature() const;

private:
	rotationType 	fRotationType;
	float		 	fXAxisRotation;
	float		 	fYAxisRotation;
	float 			fZAxisRotation;
	CAutoPtr_AC<TRotationIdler> fRotationIdler;
	TQ3Matrix4x4	fRotationMatrix;

};

#endif