//----------------------------------------------------------------------------------------
// UViewerDocument.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UViewerDocument__
#include "UViewerDocument.h"
#endif

// Sample QD3DViewer

#ifndef __UQD3DCommon__
#include "UQD3DCommon.h"
#endif

#ifndef __UQD3DViewer__
#include "UQD3DViewer.h"
#endif

// MacApp

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//QD3D
#ifndef QD3D_h
#include <QD3D.h>
#endif

#ifndef   QD3DViewer_h
#include <QD3DViewer.h>
#endif

const ResNumber kCustomWindowID = 1000;


//========================================================================================
// CLASS TViewerDocument
//========================================================================================
#undef Inherited
#define Inherited TFileBasedDocument


MA_DEFINE_CLASS(TViewerDocument);

//----------------------------------------------------------------------------------------
// TViewerDocument constructor 
//----------------------------------------------------------------------------------------


TViewerDocument::TViewerDocument(TFile* itsFile) :
	TFileBasedDocument(itsFile)
{
}

//----------------------------------------------------------------------------------------
// TViewerDocument destructor
//----------------------------------------------------------------------------------------


TViewerDocument::~TViewerDocument()
{
}

//----------------------------------------------------------------------------------------
// TViewerDocument::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TViewerDocument::DoMakeViews(bool /*forPrinting*/) 
{
	const IDType	kQD3DViewer = 'q3d2';

	TWindow* aWindow = NULL;
	::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(kCustomWindowID, this));
	fViewer = dynamic_cast_AC(TQD3DViewer*,aWindow->FindSubView(kQD3DViewer));

	// ??? Can this move to the viewer?  e.g., viewer asks doc if file exists,
	// and if so, tells QD3D to use it.	
	if (this->FileExists())
		fViewer->UseFile(fFile->GetFileSpec());
}


//----------------------------------------------------------------------------------------
// TViewerDocument::DoMenuCommand: 
//----------------------------------------------------------------------------------------


void TViewerDocument::DoMenuCommand(CommandNumber aCommandNumber)
{

	switch (aCommandNumber)
	{
		//For this sample, we are just going to write a 3DMF File.
		case cSave:
		case cSaveAs:
		case cSaveCopy:
			Write3DMF();
			break;

		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TViewerDocument::DoSetupMenus: 
//----------------------------------------------------------------------------------------
void TViewerDocument::DoSetupMenus()
{
	Inherited::DoSetupMenus();

	Enable(cSave, true); // This should only be enabled if the Viewer actually has an
						 // image to save. Right now you could save an
						 // empty 3DMF - fix for alpha, R12. gjc 
}

//----------------------------------------------------------------------------------------
// TViewerDocument::DoMenuCommand: 
//----------------------------------------------------------------------------------------
void TViewerDocument::Write3DMF()
{
	TFile* theFile  = NewFile(kMac3DMFFileType, kAllQuestionsType_AC, true, noResourceFork, true, false);
	
	// Ask the user for the destination file location and name.
	::ThrowIfOSErr_AC(MAInteractWithUser());
	try
	{
		RequestFileName(cSave, false, theFile);
	}
	catch(CException_AC& exception)
	{
		if (exception.GetError() == noErr && 
			exception.GetMessage() == messageCancelled)
			::ThrowIfOSErr_AC(userCanceledErr);
		throw;
	}

	theFile->CreateFile();
	fViewer->Write3DMFFile(theFile); // ??? why is this a method of the viewer?
	
	// ??? who owns theFile now?  Should it be deleted here?
}

//----------------------------------------------------------------------------------------
// End of UViewerDocument.cp

