//----------------------------------------------------------------------------------------
// UViewerWindow.cp 
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UViewerWindow__
#include "UViewerWindow.h"
#endif


//========================================================================================
// CLASS TViewerWindow
//========================================================================================
#undef Inherited
#define Inherited TWindow


MA_DEFINE_CLASS(TViewerWindow);

//----------------------------------------------------------------------------------------
// TViewerWindow Constructor.
//----------------------------------------------------------------------------------------


TViewerWindow::TViewerWindow():
	fViewerDocument(NULL),
	fQD3DViewer(NULL)
{
	
} // TViewerWindow::TViewerWindow


//----------------------------------------------------------------------------------------
// TViewerWindow Destructor.
//----------------------------------------------------------------------------------------


TViewerWindow::~TViewerWindow()
{
	
} // TViewerWindow Destructor

//----------------------------------------------------------------------------------------
// TViewerWindow:::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TViewerWindow::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	fQD3DViewer = dynamic_cast_AC(TQD3DViewer*,(this->FindSubView(kQD3DViewer)));
}	

//----------------------------------------------------------------------------------------
// TViewerWindow::DoEvent: 
//----------------------------------------------------------------------------------------


void TViewerWindow::DoEvent(EventNumber eventNumber,
					  			TEventHandler* source,
					  			TEvent* event)	// Override
				
{
	const IDType kSetViewerFlags		= 'setF';
	const IDType kSetBackgroundColor	= 'setB';
	
	IDType origID = dynamic_cast_AC(TView*,source)->GetIdentifier();

	switch (eventNumber)
	{
		case mButtonHit:							
			switch (origID)
			{
				case kSetViewerFlags:
					SetViewerFlags();
				break;
					
				case kSetBackgroundColor:
				{
					CRGBColor_AC	newColor;
					CStr255_AC		thePrompt = "Pick a new color";
					CRGBColor_AC theCurrentColor = fQD3DViewer->GetRGBBackgroundColor();
					
					if (GetColor(kBestSystemLocation, thePrompt, theCurrentColor, newColor))
						fQD3DViewer->SetBackgroundColor(newColor,1.0,true);
				}
				break;
			}
			break;
			
		default:
			Inherited::DoEvent(eventNumber, source, event);
			break;
	}
		
}

//----------------------------------------------------------------------------------------
// TViewerWindow::SetViewerFlags: 
//----------------------------------------------------------------------------------------

void TViewerWindow::SetViewerFlags()
{
	unsigned long theViewerFlags = 0;
	TCheckBox* theShowBadgeCB;
	TCheckBox* theViewerActiveCB;
	TCheckBox* theControllerVisCB;
	//TCheckBox* theDraggingOffCB;
	TCheckBox* theCameraButtonCB;
	TCheckBox* theTruckButtonCB;
	TCheckBox* theOrbitButtonCB;
	TCheckBox* theZoomButtonCB;
	TCheckBox* theDollyButtonCB;

	//Check the  rendering method - NOT a viewer flag
	TCheckBox* theWireFrameCB;
	theWireFrameCB = dynamic_cast_AC(TCheckBox*,FindSubView('rend'));
	if (theWireFrameCB->IsOn())
		fQD3DViewer->SetRenderer(kQ3RendererTypeWireFrame, true);
	else
		fQD3DViewer->SetRenderer(kQ3RendererTypeInteractive, true);

	theShowBadgeCB 		= dynamic_cast_AC(TCheckBox*,FindSubView('shBG'));
	theViewerActiveCB 	= dynamic_cast_AC(TCheckBox*,FindSubView('vwAC'));
	theControllerVisCB 	= dynamic_cast_AC(TCheckBox*,FindSubView('cVIS'));
	//::ThrowIfNULL_AC(theDraggingOffCB 	= dynamic_cast_AC(TCheckBox*,FindSubView('****')));
	theCameraButtonCB 	= dynamic_cast_AC(TCheckBox*,FindSubView('camB'));
	theTruckButtonCB 	= dynamic_cast_AC(TCheckBox*,FindSubView('truB'));
	theOrbitButtonCB 	= dynamic_cast_AC(TCheckBox*,FindSubView('orbB'));
	theZoomButtonCB 	= dynamic_cast_AC(TCheckBox*,FindSubView('zooB'));
	theDollyButtonCB	= dynamic_cast_AC(TCheckBox*,FindSubView('dolB'));
	
	if (theShowBadgeCB->IsOn())
		theViewerFlags |= kQ3ViewerShowBadge;

	if (theViewerActiveCB->IsOn())
		theViewerFlags |= kQ3ViewerActive;
		
	if (theControllerVisCB->IsOn())
		theViewerFlags |= kQ3ViewerControllerVisible;
		
	if (theCameraButtonCB->IsOn())
		theViewerFlags |= kQ3ViewerButtonCamera;
		
	if(theTruckButtonCB->IsOn())
		theViewerFlags |= kQ3ViewerButtonTruck;
		
	if (theOrbitButtonCB->IsOn())
		theViewerFlags |= kQ3ViewerButtonOrbit;
		
	if (theZoomButtonCB->IsOn())
		theViewerFlags |= kQ3ViewerButtonZoom;
		
	if (theDollyButtonCB->IsOn())
		theViewerFlags |= kQ3ViewerButtonDolly;
		
	fQD3DViewer->SetViewerFlags(theViewerFlags);
}
