//----------------------------------------------------------------------------------------
// UMovieViewer.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UMovieViewer__
#include "UMovieViewer.h"
#endif


//----------------------------------------------------------------------------------------
// Constants:

const OSType kSignature	= 'MS01';			// Application signature
const OSType kFileType	= 'MooV';			// open Movie files only 

const cPlayEveryFrame = 1001;		

//========================================================================================
// CLASS TMovieViewerApplication
//========================================================================================


#undef Inherited
#define Inherited TApplication


MA_DEFINE_CLASS(TMovieViewerApplication);

//----------------------------------------------------------------------------------------
// TMovieViewerApplication::TMovieViewerApplication: 
//----------------------------------------------------------------------------------------


TMovieViewerApplication::TMovieViewerApplication() :
	TApplication(kFileType, kSignature)
{
	fLaunchWithNewDocument = false;
	
	// So the linker doesn't dead strip class info 
	
	MA_REGISTER_CLASS(TMovieView);
	MA_REGISTER_CLASS(TMovieController);

	// So my view will be substituted when MacApp creates the "default view"
	
	MA_REGISTER_SIGNATURE(TMovieView, 'dflt');
}

//----------------------------------------------------------------------------------------
// TMovieViewerApplication destructor
//----------------------------------------------------------------------------------------


TMovieViewerApplication::~TMovieViewerApplication()
{
}

//----------------------------------------------------------------------------------------
// TMovieViewerApplication::DoMakeDocument:
//----------------------------------------------------------------------------------------

			
TDocument* TMovieViewerApplication::DoMakeDocument(	CommandNumber /*itsCommandNumber*/,
													TFile* itsFile) // Override 
{
	return TH_new TMovieViewerDocument(itsFile, kSignature);
}

//========================================================================================
// CLASS TMovieViewerDocument
//========================================================================================


#undef Inherited
#define Inherited TFileBasedDocument


MA_DEFINE_CLASS(TMovieViewerDocument);

//----------------------------------------------------------------------------------------
// TMovieViewerDocument::TMovieViewerDocument: 
//----------------------------------------------------------------------------------------

		
TMovieViewerDocument::TMovieViewerDocument(TFile* itsFile, OSType itsCreator) :
	TFileBasedDocument(itsFile, itsCreator),
	
	fMovie(itsFile ? TH_new TMovie(itsFile) : NULL)
{
}

//----------------------------------------------------------------------------------------
// TMovieViewerDocument::Free: 
//----------------------------------------------------------------------------------------

		
TMovieViewerDocument::~TMovieViewerDocument()
{
}

//----------------------------------------------------------------------------------------
// TMovieViewerDocument::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TMovieViewerDocument::DoMakeViews(bool /*forPrinting*/) // Override 
{
	/*
	**	get the default MacApp window
	**	get the movie's frame
	**	and set its size to that of the movie plus room for the scrollers
	*/
	TView* aView = gViewServer->NewTemplateWindow(kDefaultWindowID, this);
	
	/*
	**	get the default MacApp view in the window
	**	and let it know about the movie
	*/	
	TMovieView* movieView =dynamic_cast_AC(TMovieView*,aView->FindSubView(kIDDefaultView));
	if(fMovie)
		movieView->HaveMovie(fMovie);
	
}

//----------------------------------------------------------------------------------------
// End of UMovieViewer.cp


