//----------------------------------------------------------------------------------------
// UQuickTime.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
// Based on a contribution by Steven Friedrich
//----------------------------------------------------------------------------------------


#ifndef __UQUICKTIME__
#define __UQUICKTIME__

#ifndef __MOVIES__
#include <Movies.h>
#endif

//Forward Declarations
class TMovieView;	
class TMovieController;	



enum VariableMovieSize
{
	normalSize, halfSize, doubleSize
};


//----------------------------------------------------------------------------------------
// TMovie
//----------------------------------------------------------------------------------------

class TMovie
{
//	MA_DECLARE_CLASS;

public:

	TMovie(TFile* file = NULL);

	virtual ~TMovie();

	virtual void SetBox(const CRect_AC& itsBox);

	virtual OSErr PutOnScrap(long movieScrapFlags);

	virtual void Start();

	virtual void Stop();

	virtual void SetPlayEveryFrame(bool playEveryFrame);

	virtual PicHandle GetPoster() const;

	virtual TimeValue GetDuration() const;

	CRect_AC GetBox() const;

	inline CRect_AC GetOriginalBox() const { return fOriginalBox; }

	inline Movie GetMovie() const
	{ return fMovie; }

	virtual bool IsPlaying() const;

	virtual bool IsDone() const;

protected:
	CRect_AC fOriginalBox;
	Movie fMovie;
	bool fPlayEveryFrame;
};

//----------------------------------------------------------------------------------------
// TMovieViewIdler
//----------------------------------------------------------------------------------------
class TMovieViewIdler : public TEventHandler
{
	MA_DECLARE_CLASS;

public:
	TMovieViewIdler(TMovieView* itsMovieView = NULL);

	virtual ~TMovieViewIdler();

	virtual bool DoIdle(IdlePhase phase);// Override

protected:
	TMovieView* fMovieView;
};

//----------------------------------------------------------------------------------------
// TMovieView
//----------------------------------------------------------------------------------------
class TMovieView : public TView
{
	MA_DECLARE_CLASS;

public:

	TMovieView();

	virtual ~TMovieView();

	virtual void DoPostCreate(TDocument* itsDocument);// Override

	virtual void HaveMovie(TMovie* movie);

	virtual void DoMouseCommand(CViewPoint& theMouse,
									   TToolboxEvent* event,
									   CPoint_AC hysteresis);// Override

	virtual void BeInPort(GrafPtr itsPort);// Override

	virtual void Draw(const CViewRect& area);// Override

	virtual bool Focus();				// Override

	virtual void Activate(bool entering);// Override

	virtual void HaveMovieController(TMovieController* itsMovieController);

	virtual void SetFrame(const CViewRect& newFrame,bool invalidate);// Override

	virtual void InvalidateMovieClip();

	virtual void EnsureMovieClipIsValid();

	virtual bool SetIdleIfMoviePlaying();

	virtual bool IdleMovies();

	virtual bool IsActive() const;			// Override

	virtual void UpdateCoordinates();	// Override

	virtual void ComputeFrame(CViewRect& newFrame);	// Override

	virtual void AdjustOffsetsForController();

	virtual void SetLoop(bool loop);

	virtual void SetMovieSize(VariableMovieSize newSizeH, VariableMovieSize newSizeV);

	virtual bool SetPlayWhenInactive(bool shouldPlay);

	virtual void DoMenuCommand(CommandNumber aCommandNumber);// override 

	inline TMovie* GetMovie() const
	{ return fMovie; }
	
	virtual bool HasMovie() const;

	virtual bool IsPlaying() const;

public:
	VariableMovieSize fMovieSize[2];			// How this movie's size is to be
												// determined when the SizeDeterminer
												// is sizeVariable
	bool fPlayWhenInactive;
	bool fLoop;
	bool fAutoRewind;

protected:
	IDType fMovieControllerIdentifier;
	TMovie* fMovie;
	TMovieController* fMovieController;
	CAutoPtr_AC<TMovieViewIdler> fIdler;
	bool fWasPlaying;
	bool fIsActive;
	bool fMovieClipIsValid;					// set to false whenever the view's geometry changes

};

//----------------------------------------------------------------------------------------
// TMovieController
//----------------------------------------------------------------------------------------
class TMovieController : public TView
{
	MA_DECLARE_CLASS;

public:

	TMovieController();

	virtual ~TMovieController();

	virtual void DoPostCreate(TDocument* itsDocument);// Override

	virtual void SetFrame(const CViewRect& newFrame,
								 bool invalidate);// Override
	virtual void Show(bool state,
							 bool redraw);// Override

	virtual void BeInPort(GrafPtr itsPort);// Override

	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);	// Override

	virtual void DoMouseCommand(CViewPoint& theMouse,
									   TToolboxEvent* event,
									   CPoint_AC hysteresis);// Override

	virtual void DoKeyEvent(TToolboxEvent* event);// Override

	virtual void DoCommandKeyEvent(TToolboxEvent* event);// Override

	virtual void Draw(const CViewRect& area);// Override

	virtual bool Focus();				// Override

	virtual void Activate(bool entering);// Override

	virtual void SetActive(bool state);// Override

	virtual void AttachMovieView(TMovieView* itsMovieView);

	virtual bool SetIdleIfMoviePlaying();

	virtual void HaveMovie(TMovie* movie);

	inline MovieController GetController()
	{ return fMovieController; }

	virtual void IdleMovies();

	virtual bool IsActive() const;			// Override

	virtual void MovieChangedFrame(const CViewRect& newFrame,
								 bool invalidate);

	virtual void UpdateCoordinates();	// Override

	virtual void SetAutoSizeToMovie(bool autoSize);
	virtual void SetPlayEveryFrame(bool playEveryFrame);
	virtual void ComputeFrame(CViewRect& newFrame);	// Override
	virtual void DoSetupMenus();			// override 
	virtual void DoMenuCommand(CommandNumber aCommandNumber);// override 

	virtual bool IsPlaying() const;

protected:
	MovieController fMovieController;
	TMovieView* fMovieView;
	bool fIsActive;
	bool fPlayEveryFrame;					// true to force every frame to play
	bool fAutoSizeToMovie;					// true to make the controller stick to the bottom of the movie
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------
void InitUQuickTime();
	// Call this routine at initialization time


#endif //quicktime


