//----------------------------------------------------------------------------------------
// CReporterExampleApp.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CReporterExampleApp__
#include "CReporterExampleApp.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __CReporter_AC__
#include "CReporter_AC.h"
#endif

#ifndef __cstring__
#include <stdio.h>
#endif

enum eMenuCommands
{
	cDefault 	= 400,
	cAlert 		= 401,
	cLog		= 402
};


//----------------------------------------------------------------------------------------
// class TStarter
//----------------------------------------------------------------------------------------

class TStarter : public TCommand
{
public:
	TStarter(CReporterExampleApp* app) 
		: 	TCommand(0, NULL, false, false, NULL), 
			fApp(app) 
		{}

	virtual void DoIt() { fApp->Start(); }

private:
	CReporterExampleApp* fApp;
};


//----------------------------------------------------------------------------------------
// class CLogReporter
//----------------------------------------------------------------------------------------


class CLogReporter : public CReporter_AC
{
public:
	CLogReporter(TLogWindow& stream)
		: fLog(stream)
	{}
	virtual ~CLogReporter()
	{}
	
	virtual void OutputMessage(const char* inStr, bool /*continueIfPossible*/)
	{
		fLog << inStr << endl_AC;
	}
	
	TLogWindow& fLog;
};

//----------------------------------------------------------------------------------------
// class CAlertReporter
//----------------------------------------------------------------------------------------

class CAlertReporter : public CReporter_AC
{
public:
	CAlertReporter()
	{}
	virtual ~CAlertReporter()
	{}
	
	virtual void OutputMessage(const char* inStr, bool /*continueIfPossible*/)
	{
		CAutoPtr_AC<TWindow> window = gViewServer->NewTemplateWindow(1001, NULL);
		::ThrowIfNULL_AC(window);

		CAutoPtr_AC<TStaticText> p = dynamic_cast_AC(TStaticText*,window->FindSubView('TEXT'));

		p->SetText(inStr, false);

		IDType dismisser = window->PoseModally();

		window->Close();

		switch(dismisser)
		{
			case 'gogo':
				break;
			case 'dbug':
				Debugger();
				break;
			case 'quit':
				ExitToShell();
				break;
		}
	}
};

//----------------------------------------------------------------------------------------
// CReporterExampleApp::CReporterExampleApp
//----------------------------------------------------------------------------------------

CReporterExampleApp::CReporterExampleApp()
	:	TApplication('????', '????'),
	fCurReporter(kDefault)
{
	fLaunchWithNewDocument = false;
	
	TStarter* startCmd = TH_new TStarter(this);
	PostCommand(startCmd);
}

//----------------------------------------------------------------------------------------
// CReporterExampleApp::~CReporterExampleApp
//----------------------------------------------------------------------------------------

CReporterExampleApp::~CReporterExampleApp()
{
}

//----------------------------------------------------------------------------------------
// CReporterExampleApp::Start
//----------------------------------------------------------------------------------------

void CReporterExampleApp::Start()
{
	
	fLog.ReadViewsFromResource();
		// using default settings for loading in TEView and Window
	fLog.Show();

	fLog << "Choose New to see example of reporting." << endl_AC;;

}

//----------------------------------------------------------------------------------------
// CReporterExampleApp::DoMenuCommand
//----------------------------------------------------------------------------------------

void CReporterExampleApp::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		case cLog:
			fReporter = TH_new CLogReporter(fLog);
			fCurReporter = kLog;
			break;
	
		case cAlert:
			fReporter = TH_new CAlertReporter;
			fCurReporter = kAlert;
			break;
	
		case cDefault:
			fReporter = NULL;
			fCurReporter = kDefault;
			break;
	
		case cQuit:
			fReporter = NULL;
			fCurReporter = kDefault;
		default:
			TApplication::DoMenuCommand(aCommandNumber);
			break;
	}
} 

//----------------------------------------------------------------------------------------
// CReporterExampleApp::DoSetupMenus
//----------------------------------------------------------------------------------------

void CReporterExampleApp::DoSetupMenus() 
{
	TApplication::DoSetupMenus();		

	Enable(cNew, true);
	EnableCheck(cDefault, true, fCurReporter == kDefault);
	EnableCheck(cAlert, true, fCurReporter == kAlert);
	EnableCheck(cLog, true, fCurReporter == kLog);
}

