//----------------------------------------------------------------------------------------
// Extended.cp 
// Copyright  1985-1994 by Apple Computer, Inc.  All rights reserved.
// Jeroen Schalk - DTS.
//----------------------------------------------------------------------------------------

#ifndef __USTREAM__
#include <UStream.h>
#endif

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __FP__
#include <fp.h>
#endif

#ifndef __CEXTENDED__
#include <CExtended.h>
#endif

#ifndef __MEMORY__
#include <UMemory.h>
#endif

#ifndef __STDIO__
#include <stdio.h>
#endif

//----------------------------------------------------------------------------------------
// Constructors for CExtended. 
//----------------------------------------------------------------------------------------


CExtended::CExtended(const extended80 value)
{
#if qPowerPC
#if size(long double) == 16
	x80told(&value, &fExtended);
#else
	fExtended = x80tod(&value);
#endif
#else
#if GENERATING68881
	_extended96* _Extended = (_extended96 *)&fExtended;
	_Extended->exp[0] = value.exp;
	_Extended->exp[1] = 0;
	_Extended->man = value.man;
#else
	fExtended = value;
#endif
#endif
}

//----------------------------------------------------------------------------------------


CExtended::CExtended(const extended96 value)
{
#if qPowerPC
	extended80 value80;
	value80.exp = value.exp[0];
	::BlockMove((Ptr)value.man, (Ptr)value80.man, 8);
#if size(long double) == 16
	x80told(&value80, &fExtended);
#else
	fExtended = x80tod(&value80);
#endif
#else
#if GENERATING68881
	fExtended = value;
#else
	_extended80* _Extended = (_extended80 *)&fExtended;
	_Extended->exp = value.exp[0];
	::BlockMove((Ptr)value.man, (Ptr)_Extended->man, 8);
#endif
#endif
}

//----------------------------------------------------------------------------------------


CExtended::CExtended(const extended128 value)
{
#if qPowerPC
#if size(long double) == 16
	::BlockMove((Ptr)&value, (Ptr)&fExtended, 16);
#else
	::BlockMove((Ptr)&value.tail, (Ptr)&fExtended, 8);
#endif
#else
	
	double d0;
	double d1;
	::BlockMove((Ptr)&value.head, (Ptr)&d0, 8);
	::BlockMove((Ptr)&value.tail, (Ptr)&d1, 8);
	fExtended = d1;
#endif
}

//----------------------------------------------------------------------------------------
// Cast operators for CExtended. 
//----------------------------------------------------------------------------------------


CExtended::operator char*() const
{
	static char textExtended[40];
	sprintf (textExtended, "%g", fExtended);
	return textExtended;
}
//----------------------------------------------------------------------------------------


CExtended::operator extended80*() const
{
	static extended80 x80;
#if qPowerPC
#if size(long double) == 16
	ldtox80(&fExtended, &x80);
#else
	dtox80((double*)&fExtended, &x80);
#endif
#else
#if GENERATING68881
	_extended96* _Extended = (_extended96 *)&fExtended;
	x80.exp = _Extended->exp[0];
	x80.man = _Extended->man;
#else
	x80 = fExtended;
#endif
#endif
	return &x80;
}
//----------------------------------------------------------------------------------------


CExtended::operator extended96*() const
{
	static extended96 x96;
#if qPowerPC
	extended80 x80;
#if size(long double) == 16
	ldtox80(&fExtended, &x80);
#else
	dtox80((double*)&fExtended, &x80);
#endif
	x96.exp[0] = x80.exp;
	x96.exp[1] = 0;
	::BlockMove((Ptr)x80.man, (Ptr)x96.man, 8);
#else
#if GENERATING68881
	x96 = fExtended;
#else
	_extended80* _Extended = (_extended80 *)&fExtended;
	x96.exp[0] = _Extended->exp;
	x96.exp[1] = 0;
	::BlockMove((Ptr)_Extended->man, (Ptr)x96.man, 8);
#endif
#endif
	return &x96;
}
//----------------------------------------------------------------------------------------


CExtended::operator extended128*() const
{
static extended128 x128 = {{0,0,0,0},{0,0,0,0}};
#if qPowerPC
#if size(long double) == 16
	::BlockMove((Ptr)&fExtended, (Ptr)&x128, 16);
#else
	::BlockMove((Ptr)&fExtended, (Ptr)&x128.tail, 8);
#endif
	return &x128;
#else
	
	double d0 = 0;
	double d1 = fExtended;
	::BlockMove((Ptr)&d0, (Ptr)&x128.head, 8);
	::BlockMove((Ptr)&d1, (Ptr)&x128.tail, 8);
#endif
	return &x128;
}
//----------------------------------------------------------------------------------------


CExtended::operator extended80() const
{
	static extended80 x80;
#if qPowerPC
#if size(long double) == 16
	ldtox80(&fExtended, &x80);
#else
	dtox80((double*)&fExtended, &x80);
#endif
#else
#if GENERATING68881
	_extended96* _Extended = (_extended96 *)&fExtended;
	x80.exp = _Extended->exp[0];
	x80.man = _Extended->man;
#else
	x80 = fExtended;
#endif
#endif
	return x80;
}
//----------------------------------------------------------------------------------------


CExtended::operator extended96() const
{
	static extended96 x96;
#if qPowerPC
	extended80 x80;
#if size(long double) == 16
	ldtox80(&fExtended, &x80);
#else
	dtox80((double*)&fExtended, &x80);
#endif
	x96.exp[0] = x80.exp;
	x96.exp[1] = 0;
	::BlockMove((Ptr)x80.man, (Ptr)x96.man, 8);
#else
#if GENERATING68881
	x96 = fExtended;
#else
	_extended80* _Extended = (_extended80 *)&fExtended;
	x96.exp[0] = _Extended->exp;
	x96.exp[1] = 0;
	::BlockMove((Ptr)_Extended->man, (Ptr)x96.man, 8);
#endif
#endif
	return x96;
}
//----------------------------------------------------------------------------------------


CExtended::operator extended128() const
{
	static extended128 x128;
#if qPowerPC
	::BlockMove((Ptr)&fExtended, (Ptr)&x128, 16);
#else
	
	double d0 = 0;
	double d1 = fExtended;
	::BlockMove((Ptr)&d0, (Ptr)&x128.head, 8);
	::BlockMove((Ptr)&d1, (Ptr)&x128.tail, 8);
#endif
	return x128;
}

//----------------------------------------------------------------------------------------
// Relational operators for CExtended. 
//----------------------------------------------------------------------------------------


bool CExtended::operator==(const CExtended& ext) const
{
#if qPowerPC
	return relationl(*this, ext) == EQUALTO;
#else
	long double first = this->fExtended;
	long double second = ext.fExtended;
	return relation(first, second) == EQUALTO;
#endif
} // CExtended::operator==

bool CExtended::operator>(const CExtended& ext) const
{
#if qPowerPC
	return relationl(*this, ext) == GREATERTHAN;
#else
	long double first = this->fExtended;
	long double second = ext.fExtended;
	return relation(first, second) == GREATERTHAN;
#endif
} // CExtended::operator>

bool CExtended::operator<(const CExtended& ext) const
{
#if qPowerPC
	return relationl(*this, ext) == LESSTHAN;
#else
	long double first = this->fExtended;
	long double second = ext.fExtended;
	return relation(first, second) == LESSTHAN;
#endif
} // CExtended::operator<

bool CExtended::operator!=(const CExtended& ext) const
{
	return ! this->operator==(ext);
} // CExtended::operator!=

bool CExtended::operator>=(const CExtended& ext) const
{
	return ! this->operator<(ext);
} // CExtended::operator>=

bool CExtended::operator<=(const CExtended& ext) const
{
	return ! this->operator>(ext);
}  // CExtended::operator<=

//----------------------------------------------------------------------------------------


CExtended CExtended::operator+(const CExtended& ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended + ext.fExtended;
	return returnRect;
}

CExtended CExtended::operator-(const CExtended& ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended - ext.fExtended;
	return returnRect;
}

CExtended CExtended::operator*(const CExtended& ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended * ext.fExtended;
	return returnRect;
}

CExtended CExtended::operator/(const CExtended& ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended / ext.fExtended;
	return returnRect;
}

CExtended& CExtended::operator+=(const CExtended& ext)
{
	fExtended += ext.fExtended;
	return *this;
}

CExtended& CExtended::operator-=(const CExtended& ext)
{
	fExtended -= ext.fExtended;
	return *this;
}

CExtended& CExtended::operator*=(const CExtended& ext)
{
	fExtended *= ext.fExtended;
	return *this;
}

CExtended& CExtended::operator/=(const CExtended& ext)
{
	fExtended /= ext.fExtended;
	return *this;
}

//----------------------------------------------------------------------------------------

CExtended CExtended::operator+(const long double ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended + ext;
	return returnRect;
}

CExtended CExtended::operator-(const long double ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended - ext;
	return returnRect;
}

CExtended CExtended::operator*(const long double ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended * ext;
	return returnRect;
}

CExtended CExtended::operator/(const long double ext) const
{
	CExtended returnRect;
	returnRect.fExtended = fExtended / ext;
	return returnRect;
}

CExtended& CExtended::operator+=(const long double ext)
{
	fExtended += ext;
	return *this;
}

CExtended& CExtended::operator-=(const long double ext)
{
	fExtended -= ext;
	return *this;
}

CExtended& CExtended::operator*=(const long double ext)
{
	fExtended *= ext;
	return *this;
}

CExtended& CExtended::operator/=(const long double ext)
{
	fExtended /= ext;
	return *this;
}

//----------------------------------------------------------------------------------------


void CExtended::Write(TStream *aStream) const
{
	aStream->WriteBytes(&fExtended, sizeof(fExtended));
}

//----------------------------------------------------------------------------------------


void CExtended::Read(TStream *aStream)
{
	aStream->ReadBytes(&fExtended, sizeof(fExtended));
}
//----------------------------------------------------------------------------------------


void CExtended::Write64(TStream *aStream) const
{
	double x64 = fExtended; // (double)this; does not work???
	aStream->WriteBytes(&x64, sizeof(x64));
}

//----------------------------------------------------------------------------------------


void CExtended::Read64(TStream *aStream)
{
	double x64;
	aStream->ReadBytes(&x64, sizeof(x64));
	CExtended tmp(x64);
	fExtended = tmp.fExtended;
}
//----------------------------------------------------------------------------------------


void CExtended::Write80(TStream *aStream) const
{
	extended80 x80 = *(extended80*)this;
	aStream->WriteBytes(&x80, sizeof(x80));
}

//----------------------------------------------------------------------------------------


void CExtended::Read80(TStream *aStream)
{
	extended80 x80;
	aStream->ReadBytes(&x80, sizeof(x80));
	CExtended tmp(x80);
	fExtended = tmp.fExtended;
}
//----------------------------------------------------------------------------------------


void CExtended::Write96(TStream *aStream) const
{
	extended96 x96 = *(extended96*)this;
	aStream->WriteBytes(&x96, sizeof(x96));
}

//----------------------------------------------------------------------------------------


void CExtended::Read96(TStream *aStream)
{
	extended96 x96;
	aStream->ReadBytes(&x96, sizeof(x96));
	CExtended tmp(x96);
	fExtended = tmp.fExtended;
}
//----------------------------------------------------------------------------------------


void CExtended::Write128(TStream *aStream) const
{
	extended128 x128 = *(extended128*)this;
	aStream->WriteBytes(&x128, sizeof(x128));
}

//----------------------------------------------------------------------------------------


void CExtended::Read128(TStream *aStream)
{
	extended128 x128;
	aStream->ReadBytes(&x128, sizeof(x128));
	CExtended tmp(x128);
	fExtended = tmp.fExtended;
}
