//----------------------------------------------------------------------------------------
// CExtended.h 
// Copyright  1985-1994 by Apple Computer, Inc.  All rights reserved.
// Jeroen Schalk - DTS.
//----------------------------------------------------------------------------------------

// THEORY OF OPERATION
// This code allows you to convert between the various extended formats on PowerPC and 68K.
// A CExtended has an embedded fExtended field, which will assume the "natural" long double
// format for the architecture. This is long double (128 bit) on PowerPC and SANE extended
// on 68K, either 80 or 96 bits.  However, Metrowerks on PPC uses 64 bits for long double,
// as for double.

#ifndef __CEXTENDED__

#define __CEXTENDED__

#ifndef __TYPES__
#include <Types.h>
#endif

//----------------------------------------------------------------------------------------

class TStream;

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct _extended128 {
	short	head[4];
	short	tail[4];
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif

typedef struct _extended128 extended128;

//----------------------------------------------------------------------------------------

class CExtended 
{
protected:
	long double fExtended;	// 128 bits on PowerPC, extended (80, 96) on 68K, 
							// 64 bits Metrowerks PPC
public:
	CExtended();
	CExtended(const long value);
	CExtended(const float value);
	CExtended(const double value);
	
	CExtended(const extended80 value);
	CExtended(const extended96 value);
	CExtended(const extended128 value);
	
	operator char*() const;

	operator long() const {return fExtended;}
	operator float() const {return fExtended;}
	operator double() const {return fExtended;}
#if qPowerPC
	operator long double() const {return fExtended;}
#endif

	operator extended80*() const;
	operator extended80() const;
	operator extended96*() const;
	operator extended96() const;
	operator extended128*() const;
	operator extended128() const;
	
	bool operator!=(const CExtended& ext) const;	// a != b
	bool operator==(const CExtended& ext) const;	// a == b
	bool operator>(const CExtended& ext) const;	// a > b
	bool operator<(const CExtended& ext) const;	// a < b
	bool operator>=(const CExtended& ext) const;	// a >= b
	bool operator<=(const CExtended& ext) const;	// a <= b

	CExtended operator+(const CExtended& ext) const;
	CExtended operator-(const CExtended& ext) const;
	CExtended operator*(const CExtended& ext) const;
	CExtended operator/(const CExtended& ext) const;

	CExtended& operator+=(const CExtended& ext);
	CExtended& operator-=(const CExtended& ext);
	CExtended& operator*=(const CExtended& ext);
	CExtended& operator/=(const CExtended& ext);

	CExtended operator+(const long double ext) const;
	CExtended operator-(const long double ext) const;
	CExtended operator*(const long double ext) const;
	CExtended operator/(const long double ext) const;
	
	CExtended& operator+=(const long double ext);
	CExtended& operator-=(const long double ext);
	CExtended& operator*=(const long double ext);
	CExtended& operator/=(const long double ext);

	virtual void Write(TStream *aStream) const;
	virtual void Read(TStream *aStream);
		// read/write natural size
		
	virtual void Write64(TStream *aStream) const;
	virtual void Read64(TStream *aStream);
		// read/write as 64 bits (double)
		
	virtual void Write80(TStream *aStream) const;
	virtual void Read80(TStream *aStream);
		// read/write as 80 bits (68K long double)
		
	virtual void Write96(TStream *aStream) const;
	virtual void Read96(TStream *aStream);
		// read/write as 96 bits (68K FPU long double)
		
	virtual void Write128(TStream *aStream) const;
	virtual void Read128(TStream *aStream);
		// read/write as 128 bits (PowerPC long double)
};

//--------------------------------------------------------------------------------------------------

inline CExtended::CExtended() : fExtended(0)
{
}

inline CExtended::CExtended(const long value) : fExtended(value)
{
}

inline CExtended::CExtended(const float value) : fExtended(value)
{
}

inline CExtended::CExtended(const double value) : fExtended(value)
{
}

#endif // of #ifndef __CEXTENDED__
