/*-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker
PowerPC version by Jeroen Schalk

Copyright  1988-1994 Apple Computer, Inc.
All rights reserved.

--------------------------------------------------------------------------------------------

This is the Main part of the program. Its job is to make sure that were running on
an OK machine and, if so, initialize all the parts of MacApp we need, make an
application object, and run it. When TFracAppApplication.Run returns, that means that
the user has selected Quit, that everything is shut down, and that all we have to
do is get out of here. If the machine type is not OK, then we put up an alert saying
what is wrong.

--------------------------------------------------------------------------------------------*/

#ifndef __UPRINTING__
#include <UPrinting.h>
#endif

#ifndef __UDIALOG__
#include <UDialog.h>
#endif

#ifndef __UTEVIEW__
#include <UTEView.h>
#endif

#ifndef __UERRORMGR__
#include <UErrorMgr.h>
#endif

#ifndef __FRACAPP__
#include "UFracApp.h"
#endif

#ifndef __UFRACDOC__
#include "UFracDoc.h"
#endif

#if qUseThreads
	#ifndef __THREADS__
	#include <Threads.h>
	#endif
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __UMACAPPGLOBALS__
#include <UMacAppGlobals.h>
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

//-------------------------------------------------------------------------------------------

enum
{
	kUnsupportedMac 	= -25001,
	kCompiledWrong		= -25003,
	kNoThreadManager	= -25004
};

//-------------------------------------------------------------------------------------------
void main()
{
	InitUMacApp(8);

	/* Check to see that we were compiled with the right options. FracApp absolutely
	  positively needs to be compiled for Color QuickDraw and an FPU. We take
	  advantage of 32-bit Color QuickDraw if its around, but we dont require it.
	  If FracApp was compiled with the wrong options, put up a dialog saying so,
	  and quit the application. */

#if qPowerPC
	if (!HasColorQD_AC())
		StdAlert(kCompiledWrong);
	else
#else
	if (!HasColorQD_AC() || !HasFPU_AC())
		StdAlert(kCompiledWrong);
	else
#endif

#if qUseThreads
	if (!HasThreadManager_AC())
		StdAlert(kNoThreadManager);
	else
#endif
	{
		/* InitToolBox has filled in a global variable call gConfiguration. This
		  record contains a lot of information on what is available on the Macintosh we
		  are running on. What we do next is call ValidateConfiguration to make sure
		  that the machine features this program requires are actually available.
		  ValidateConfiguration is self-configuring based on the compiler options you
		  used to compile this program. If a feature is missing, ValidateConfiguration
		  returns false. For instance, we compile FracApp with the -NeedsColorQD
		  option. If we try to run FracApp on a machine that doesnt have Color
		  QuickDraw, then ValidateConfiguration returns false. Otherwise, it returns
		  true, and we can safely run on this machine.
	
		  We also test here to see if we are running on a machine that has an FPU. We
		  dont require that you actually COMPILE FracApp with the -NeedsFPU option,
		  but there are some assembly routines that get compiled with 68881 code, and
		  those pieces need to be run on a machine with an FPU. If you dont put
		  -NeedsFPU on the MABuild command line, then ValidateConfiguration wont check
		  for an FPU. Therefore, we do it ourself by hand. */
	
		/* We now continue with the rest of MacApps initialization. Calling InitUMacApp
		  is required. It takes a single parameter which is the number of times that
		  MoreMasters should be called. InitUPrinting, InitUDialog, and InitUTEView
		  setup their respective units, and only need to be called if you will be using
		  any of their services. Forgetting to call InitUDialog is a particularly
		  common problem that is tricky to detect. Many times, youll introduce a
		  dialog to your program, and forget to call InitUDialog. Youll also need to
		  call InitUTEView if you use any TEditText items, not just TTEViews. If you
		  forget to do any of this, youll find that the various views that appear in
		  dialogs will get stripped out by the Linker, and your program will crash
		  mysteriously. If you have a program that works in Debug mode, but not in
		  Non-Debug, then this is most likely the problem. */
	
		InitUPrinting();
		InitUTEView();
		InitUDialog();
	
		/* Finally, we create our application object, initialize it, and run it. From
		  this point on, MacApp is in control. When the user selects Quit from the
		  menu, we come back to here, where all we have to do exit. */
	
		TFracAppApplication gFracAppApplication(kFileType);
		gFracAppApplication.Run();
	}
}
