// UFracApp.cp
// Copyright  1991-97 by Apple Computer, Inc., all rights reserved.

//	Contains:	FracApp source

#ifndef __FRACAPP__
#include "UFracApp.h"
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#if qUseThreads
	#ifndef __THREADS__
	#include <Threads.h>
	#endif
#endif

#ifndef __UFRACGLOBALS__
#include "UFracGlobals.h"
#endif

#ifndef __UFRACDOC__
#include "UFracDoc.h"
#endif

#ifndef __UFRACVIEWS__
#include "UFracViews.h"
#endif

#ifndef __UMEMORY__
#include <UMemory.h>
#endif

#ifndef __UMENUMGR__
#include <UMenuMgr.h>
#endif

#ifndef __UERRORMGR__
#include <UErrorMgr.h>
#endif

#ifndef __UCLIPBOARDMGR__
#include <UClipBoardMgr.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

#ifndef __UWINDOWMENUBEHAVIOR__
#include "UWindowMenuBehavior.h"
#endif

//-------------------------------------------------------------------------------------------

#if qPerform
#define qUseTimeManager		0
#else
#define qUseTimeManager		0
#endif

/* We define two different palettes for two different situations. Wed like to be able to
  animate when we can; in that case, we attach the palette identified with
  kAnimatingPaletteID to each window we make. If we cant animate our colors (which occurs
  when we dont have 32-bit Color QuickDraw), then we fall back on a palette that at least
  guarantees we have the colors we need (kTolerantPalette). */

const short kTolerantPalette	= 1001;				// palette ID for not animating
const short kAnimatingPalette	= 1002;				// palette ID for animating
const short kClut				= 501;				/* ID for the clut resource that we use
													 for offscreen blinging. */

/* When calling SetPalette, we need to pass a bool that tells the Palette manager whether
  or not we want updates when our window is activated. These two constants help clarify what
  we are passing to SetPalette. */

const bool kWantUpdates		= true;
const bool kDontWantUpdates	= false;

const short kNo32BCQD			= - 25002;			/* error code if 32bCQD is not around.
													 Used to display a warning that some
													 menu items will be disabled. */

//-------------------------------------------------------------------------------------------
#if qUseTimeManager

/* This is the record type we use when installing a time manager task. It includes a field
  for an A5 so that our task can get to our global variables. In addition to a normal time
  manager task record, we define two other fields that are used by the double no drift
  time manager. */

	typedef struct MyTimeTask
	{
		long 		A5;
		TMTask		taskRecord;
		long		tmWakeUp;				/* In case the Extended time manager is
											 around */
		long 		tmReserved;
	};
#endif

//-------------------------------------------------------------------------------------------

/* For determining how long it takes to calculate a document, we use the Time Manager. But we
  can only use the Time Manager if we are not using the performance tools, which use the
  VIAs directly and interfere with the Time Manager. Everything would be fine if the the
  performance tools used the Time Manager, but they dont. At least, not as of MPW 3.2.
  Therefore, we can use the Time Manager only if we dont use the performance tools, so we
  define this Time Manager task record only under that circumstance. */

#if qUseTimeManager
	MyTimeTask	gTMTask;
#endif

/* We have two palettes to attach to windows, depending on whether or not we have 32-Bit
  Color QuickDraw. We figure out which one to use when our application initializes itself,
  and store its resource ID here. Then, whenever we need a palette, we just use the value
  here, rather than refiguring the whole thing out every time. */

short		gPaletteIDToUse;

/* When using system 6.0.2 or later, we can associate a palette with more than one window. If
  we are running under those conditions, we get the palette we need just once, save it in
  this variable, and attach it to any new windows we create. Otherwise, we just do a
  GetNewPalette every time, and this variable goes unused. */

PaletteHandle	gPalette;

/* Just because we HAVE 32-Bit CQD doesnt mean that we want to use it. This variable is used
  to control whether or not we want to use it if its available. Its initially set to
  gConfiguration.has32BitCQD. */

bool		gUse32BitCQD;

/* The following variables describe the location of the dynamic parts of our Windows menu.
  They are determined by looking for the dummy menu item with the command number
  cFirstWindowBase. That menu item is removed, and any window names replace it. */

//short		gFirstWindowBase;
//short		gWindowMenuNumber;

//-------------------------------------------------------------------------------------------

#if qUseTimeManager

	// Functions implemented in "GoFigger.a"

	void TimeCounterThatFetchesItsOwnTaskPtr();

	void TimeCounter();

	void InitCounter(Ptr A5);

	void InsTimeNoDrift(QElemPtr tmTaskPtr);

#endif

//-------------------------------------------------------------------------------------------
//-----------------------------------Global Procedures---------------------------------------
//-------------------------------------------------------------------------------------------


/* Called whenever we create a window. This routine is an intelligent version of SetPalette.
  It determines if we are running on a system that allows default application palettes or
  not. If not, it gets a new palette, and attaches it to the window. Otherwise, it does
  nothing, allowing the system to attach the correct palette to it. For more information
  on default application palettes, see Technote #211. */

void MySetPalette(TWindow* theWindow)
{
	if (!gHasDefaultApplicationPalettes && (theWindow))
	{
		Boolean oldPerm = PermAllocation(true);
		PaletteHandle palette = ::GetNewPalette(gPaletteIDToUse);
		oldPerm = PermAllocation(oldPerm);
		::ThrowIfNULL_AC(palette);
		::SetPalette(theWindow->GetWindowRef(), palette, kDontWantUpdates);
	}
}

//-------------------------------------------------------------------------------------------
//------------------------------TFracAppApplication Methods----------------------------------
//-------------------------------------------------------------------------------------------


/* Initializes the application and globals. Called by the main application when the
  applicaton object is created. */

#undef Inherited
#define Inherited TApplication
DefineClass(TFracAppApplication, TApplication);

TFracAppApplication::TFracAppApplication(OSType itsMainFileType) :
	TApplication(itsMainFileType, kSignature)
{
	
	fLastCalculated = NULL;

	//fWindowList = NewList_AC(); TWindowMenuBehavior

	/* Set gRebuildWindowsMenu to true to force the Windows menu to be rebuilt for
	  the first time. This is important, so that our dummy menu item gets removed.
	  Also, call CommandToMenuItem to find the menu item of our first window entry in
	  the menu. */

	//gRebuildWindowsMenu = true;
	//CommandToMenuItem(cFirstWindowBase, gWindowMenuNumber, gFirstWindowBase);

	/* If 32-bit Color QuickDraw is around, we make a note that we could like to use
	  it for our offscreen routines as a default. However, this can be turned off
	  if wed like to offscreen things the old fashioned way with a flick of a menu
	  item. Also, decide which palette we are going to use for our windows,
	  depending on if we have 32-bit Color QuickDraw. If  32-bit Color QuickDraw is
	  not around, then show an alert that tells the user that some features will
	  not be available. */

	gUse32BitCQD = Has32BitQD_AC();

	if (gUse32BitCQD)
	{
		gPaletteIDToUse = kAnimatingPalette;
	}
	else
	{
		gPaletteIDToUse = kTolerantPalette;
		StdAlert(kNo32BCQD);
	}

	/* Wed like to take advantage to default application palettes if the system
	  supports them (see technote #211). To condition our program in the places
	  necessary to support default application palettes, we set this global
	  variable when running on the appropriate systems. */

	gHasDefaultApplicationPalettes = (GetSystemVersion_AC() >= 0x0602);

	/* We can calculate our fractals in one of two different ways. There is a
	  pixel-by-pixel method, which is good for complicated fractals, and there is
	  another method that incurs some additional overhead, but more than makes this
	  up in optimizations on documents that have some large areas that are all the
	  same color. By default, we use the latter, faster method. */

	gUseFastAlgorithm = true;

	/* Set the default print resolution to 72 dpi. This in anticipation of the day
	  when this program will be flexible enough to calculate at other resolutions.
	  However, for now, its unsupported. */

	gPrintResolution = 72;

	/* Start off not animating, initialize the palettes to normal (not jumbled) and
	  set the last time that we did animate to sometime way in the past (Jan 1,
	  1904) */

	gPaletteIsJumbled = false;
	gAnimate = false;
	gLastAnimated = 0;

	/* If we are not using the Performance tools, we can use the Time Manager for
	  apportioning our time, and for calculating the elapsed time to calculate a
	  fractal. If so, install a time manager task. If we are running under
	  6.0.3 or later, we can take advantage of the fact that the Time Manager
	  gives our time task procedure a pointer to our task record. Otherwise, we
	  have to fall back on method of remembering a reference to our global by hand.
	  See Technote #180, sub-section Time Manager Tasks, for more information on
	  this.

	  Also note that when I install the time manager task, that I call my own glue
	  routine rather than just calling InsTime. This is because I have some special
	  Assembly language glue that calls InsXTime. This is a new time manager call
	  that offers a no-drift feature. In other words, the difference between
	  successive calls to my time manager procedure is exactly what I specified,
	  rather than what I specified, plus the overhead involved in the time manager,
	  plus the time it takes my time task procedure to execute. InsTimeNoDrift
	  works in such a way that it will automatically take advantage of this feature
	  if it is there. */

	#if qUseTimeManager
		gTMTask.A5 = ::SetCurrentA5();
		if (::GetSystemVersion_AC() < 0x0603)
		{
			gTMTask.taskRecord.tmAddr = NewTimerProc(&TimeCounterThatFetchesItsOwnTaskPtr);
			InitCounter((Ptr) &gTMTask.taskRecord);
		}
		else
		{
			gTMTask.taskRecord.tmAddr = NewTimerProc(&TimeCounter);
		}

		gCounter = 0;
		InsTimeNoDrift((QElemPtr) &gTMTask.taskRecord);
		::PrimeTime((QElemPtr) &gTMTask.taskRecord, 1);
	#endif

	/* Initialize so we can use TFracAppDocument and its associated views */
	InitUFracDoc();

	this->CreateManyColors();

	/* Associate one of our palettes with the clipboard window. This is so that we
	  can see our fractal properly if we copy them to the desk scrap. */

	MySetPalette(gClipboardMgr->fClipWindow);

	// We want a windows menu, add the behavior
	this->AddBehavior(TH_new TWindowMenuBehavior);
	
} // TFracAppApplication::IFracAppApplication

//-------------------------------------------------------------------------------------------


/* This is just a simple routine that gets called when we are done calculating a fractal
  document that is part of a multipaging process. It creates a file name based on what
  part of the overall fractal it represents, and saves everything out to that file. It then
  saves some information so that we can know where to create any subsequent documents, and
  closes the document. */

void TFracAppApplication::AutoSaveThisGuy(TFracAppDocument* doneDoc)
{
	const bool kDontAskForFileName = false;
	const bool kNotMakingCopy	  = false;

	/* First, save off and close the previous document. We title the thing Set- x,y, and
	  tell MacApp to jam it to whatever directory we happen to be set to. */

	{
		CStr255_AC	title;
		{
			CStr255_AC	page;
			title = "Set- ";
			::NumToString(gPageRecord.currentH, page);
			title += page;
			title += ",";
			::NumToString(gPageRecord.currentV, page);
			title += page;
		}
		doneDoc->SetTitle(title);
	}

	/* Before we close the document, we have to leave some information laying around
	  so we know how to create the next one. All of this information is
	  conveniently kept in the pages field of the FracHeader. So we just save it
	  in a global PageRecord variable. */

	{
		FracRecord f = doneDoc->GetFracHeader();
		gPageRecord = f.pages;
	}

	/* Save the document with no questions asked, using the name of the document
	  that was set before.	If we have a problem with the file, an alert will come
	  up freezing the operation for user input, but that is OK.  We at least try to
	  be automatic, but if something freaks out it is OK to pause. */

	// Satisfy a bug in TFileBasedDocument where it doesn't mark fFileExists
	// unless _it_ saves to a file, but we are hardcoding an autosave file name,
	// which may have previously been saved.
	doneDoc->SetFileExists( (doneDoc->GetFile())->GetFileSpec().Exists());
	
	doneDoc->SaveDocument(cSave);
	doneDoc->Close();
}

//-------------------------------------------------------------------------------------------


/* We have a special kind of window that notifies our application when it opens or closes
  itself. Our application then either adds or removes that window from a list it keeps.
  Also, if a window renames itself (say, the owning document was saved), it notifies the
  application. In both cases, this method gets called to totally rebuild the Windows menu.
  It also matches up the top window with a menu item, and puts a check mark by it. */

#if 0
void TFracAppApplication::BuildWindowsMenu()
{
	short curItem;

	if (gRebuildWindowsMenu)
	{
		MenuRef windowsMenu = MAGetMenuRef(gWindowMenuNumber);
		::ThrowIfNULL_AC(windowsMenu);
		short nItems = ::CountMItems(windowsMenu);

		if (nItems >= gFirstWindowBase)
		{
			for (curItem = nItems; curItem >= gFirstWindowBase; curItem--)
			{
				::DeleteMenuItem(windowsMenu, curItem);

				/* The Menu Manager has this feature (yes, its really a feature), where it sets
				  the top bit of enableFlags when you call ::DeleteMenuItem. It does this so that
				  any items that are being scrolled into the enableFlags range will
				  automatically be enabled. This is being consistant with the fact that all
				  menu items beyond 32 are enabled. Anyway, it messes us up BuildWindowsMenu,
				  because we call it in the middle of DoSetupMenus. If we delete all menu items
				  here, we would like to have the Windows menu title greyed out. However, that
				  wont happen, because ::DeleteMenuItem set some bits in enableFlags, and MacApp
				  wont gray out the menu title. So we compensate for that by disabling the
				  32nd menu item by hand. */

				::DisableItem(windowsMenu, 31);
			}
		}

		/* This sub-procedure is responsible for adding the name of a window to the
		  menu. It is called by the CList_AC.Each method for each item in the list. We
		  coerce the object to a TWindow, get the windows title, and append it to the
		  windows menu. Note how we do this: we first add an item with a dummy name,
		  and then rename it with SetMenuItemText. This is to avoid the Menu Managers
		  interpreting things like /, <, !, and other meta characters. */

		curItem = gFirstWindowBase;
		for (CArrayIterator_AC iter(fWindowList); iter.Current(); ++iter)
		{
			
			TWindow* aWindow = (TWindow*)(iter.Current());
			CStr255_AC	itsTitle(aWindow->GetTitle());
			::AppendMenu(windowsMenu, (ConstStr255Param)"\px");
			::SetMenuItemText(windowsMenu, curItem++, itsTitle);
		}

		gRebuildWindowsMenu = false;
	}
}
#endif

//-------------------------------------------------------------------------------------------


/* The Close method to allow us to clean up before the application quits. We override this
  method rather than TFracAppApplication::Free, because .Free rarely gets called. The
  only time it can get called is when TApplication.Run returns to the main program. At that
  point, it is the main procedures responsibility to call .Free. However, hardly anyone
  ever does that, so we override TFracAppApplication::Close instead, which gets called when
  the user selects Quit.

  Our Quit item just removes the time manager task. Failure to do so will leave a pointer to
  a no-longer existant task record and task procedure. Once another application is launched,
  these will get overridden in memory, and the time manager will crash your machine with
  glee. */

void TFracAppApplication::Close()
{
	Inherited::Close();
	#if qUseTimeManager
		::RmvTime((QElemPtr) &gTMTask.taskRecord);
	#endif

	
	// we are getting a serious bomb in the palette manager around static destruction time,
	// so for the time being, do an ES abruptly.
	ExitToShell();
} // TFracAppApplication::Close

//-------------------------------------------------------------------------------------------


/* Called during the initialization of our application to set up some color stuff. First of
  all, if we can take advantage of the default application palette, we get the palette,
  and set set it as the default. Note that its documented in technote #211 that we can just
  define a pltt resource of ID 0, and that will automatically be used as the default
  application palette. However, through the wonders of System Software Engineering, that no
  longer works on a Mac IIci and System 6.0.4. Explicitly setting the application palette is
  the workaround. But I digress...After we set the palette, we read in and save a reference
  to the color table used for our offscreen devices and palette manager animation stuff. */

void TFracAppApplication::CreateManyColors()
{
	Boolean oldPerm;

	/* We can take advantage of default application palettes under System 6.0.2 or
	  later. So check for this before trying to set such a palette. If we dont
	  support application palettes, MySetPalette will kick in when we create a
	  window and attach an individual palette to each window. */

	if (gHasDefaultApplicationPalettes)
	{
		oldPerm = PermAllocation(true);
		gPalette = ::GetNewPalette(gPaletteIDToUse);
		oldPerm = PermAllocation(oldPerm);
		::ThrowIfNULL_AC(gPalette);
		::SetPalette(WindowPtr( - 1), gPalette, kDontWantUpdates);
	}

	/* Now allocate a color table that we will use whenever we create a new
	  document. This is so we have the same color table for each document. */

	oldPerm = PermAllocation(true);
	gOurColors = ::GetCTable(kClut);
	oldPerm = PermAllocation(oldPerm);
	::ThrowIfNULLResource_AC(Handle(gOurColors));
}

//-------------------------------------------------------------------------------------------


/* Looks for a document that needs calculation time. Round-robins among open
  documents. It does this by calling EachWMgrWindowDoTil with a procedure that
  looks at the document attached to that window, and asks if it needs calculation
  time. If it does, then we return that document to the caller. If we dont find
  one, we return NULL. If 'lastCalculated' is NULL, it returns the first qualifying
  document it finds. Otherwise, it returns the first qualifying document (if any)
  after it finds 'lastCalculated'. */

TFracAppDocument* TFracAppApplication::GetDocumentToCalculate(TFracAppDocument* lastCalculated)
{
	TFracAppDocument* returnVal = NULL;

	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow *aWindow = TWindow::WMgrToWindow(iter);
		if (aWindow && dynamic_cast<TFracAppDocument*> (aWindow->GetDocument()))
		{
			TFracAppDocument* doc = dynamic_cast_AC(TFracAppDocument*,aWindow->GetDocument());
			if (lastCalculated == NULL && !doc->GetDone())
			{
				returnVal = doc;
				break;
			}
			if (lastCalculated == doc)
				lastCalculated = NULL;
		}
	}

	return returnVal;
}

//-------------------------------------------------------------------------------------------


/* Finds a document that will do some animation for us. This routine calls
  EachWMgrWindowDoTil to find the first TFracAppWindow it can, and returns the
  attached document. If there are no such windows, then return NULL. */

TFracAppDocument* TFracAppApplication::GetDocumentToAnimate()
{
	TFracAppDocument* returnVal = NULL;
	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow *aWindow = TWindow::WMgrToWindow(iter);
		if (aWindow && dynamic_cast<TFracAppDocument*> (aWindow->GetDocument()))
		{
			returnVal = dynamic_cast_AC(TFracAppDocument*,aWindow->GetDocument());
			break;
		}
	}

	return returnVal;
}

//-------------------------------------------------------------------------------------------


/* Performs Idle time processing for the application. This will do the fractal calculation
  during idle time. It will allow each open document a chance to calculate.  The way we
  do this is in a front to back order. We get a reference to the top document. If it is not
  done, we call its engine to calculate a little bit of the fractal, and then get out
  of here. If the top document IS done, we find the next document behind it, and give
  it some time, if needed. This process is carried all the way back for as many documents as
  we have.

  This routine is also responsible for doing the Palette Manager animation, if the user has
  selected that option. Finally, if a document signals that its done, and that document is
  part of a multipage set, we save that document to disk and start a new one. */

bool TFracAppApplication::DoIdle(IdlePhase phase)
{
	/* MacApp calls us before and after every event, with phase codes of
	  idleBegin and idleEnd.  Phase idleContinue is the 'true' idle.
	  In order to give decent performance to the rest of the application
	  (mouse clicks, menu commands), we want to do real work only on idleContinue. */

	if (phase != idleContinue)
		return false;

#if qUseThreads
	YieldToAnyThread();
#endif

	/* round-robin on documents */
	TFracAppDocument* documentToCalculate = this->GetDocumentToCalculate(fLastCalculated);
	if ((documentToCalculate == NULL) && fLastCalculated) {
		documentToCalculate = this->GetDocumentToCalculate(NULL);
	}

	fLastCalculated = documentToCalculate;

	TFracAppDocument* documentToAnimate = GetDocumentToAnimate();

	if (documentToAnimate)
		documentToAnimate->AnimateColors();

	bool done;
	if (documentToCalculate)
		done = documentToCalculate->CalcTown();

	if ((documentToCalculate == NULL) && (documentToAnimate == NULL))
		this->SetIdleFreq(kMaxIdleTime);

	/* See if we are doing multipage operation, and if so, save the document, close
	  the window and start a new one. */

	if (documentToCalculate && done && documentToCalculate->IsMultiPaging())
	{
		this->AutoSaveThisGuy(documentToCalculate);
		this->MakeNewGibbleyFromGPageRecord();
	}

	/* Idle Time handler have to return a boolean value that states whether or not
	  they have removed themselves from memory. This was something that was
	  supported transparently in MacApp 1.x, but things have changed since then.
	  Now that MacApp 2.0 is MultiFinder aware, it tries to be more intelligent
	  about giving time up to background apps. Because of this, it manages the time
	  given to idle time handlers more thoroughly. It maintains fields that say how
	  often they should be called, and when the last time one was called. Normally,
	  it tries to set this latter field when the TEvtHandler.DoIdle method is done.
	  However, what happens if the object removed itself from the chain and freed
	  itself from memory? The reference to the object is no longer valid, and
	  MacApp would be writing to random memory. To prevent this from happening, we
	  tell MacApp if we are still around. If we return true, MacApp knows that we
	  are history. */

	return false;
} // TFracAppApplication::DoIdle

//-------------------------------------------------------------------------------------------


/* Launches a TFracAppDocument; called when applications icon is opened, or when New, New
  From Selection, New MultiPage, or Open is requested by the user. After it is done, the
  view and window can be created, relying upon the data in the document. Every application
  which uses documents MUST override this method */

TDocument* TFracAppApplication::DoMakeDocument(CommandNumber itsCommandNumber,
													  TFile* itsFile)
{
	// Allocate and initialize the document.

	/* Now initialize the document fields, and set up the global state of the
	  fractal to a default set of the starting fractal. We specify here what kind
	  of COffscreen object to create in IFracAppDocument. In order to use
	  CNewCoolOffscreen (ie, the new 32-bit CQD stuff), we both have to have 32-bit
	  CQD and want to use it. This could be specified by a menu item, for example. */
	  
	TFracAppDocument* aFracAppDocument = TH_new TFracAppDocument(itsFile,
							Has32BitQD_AC() && gUse32BitCQD, itsCommandNumber);
	::ThrowIfNULL_AC(aFracAppDocument);


	// Set the idle time so that we start calculating now!

	this->SetIdleFreq(0);

	/* We successfully created a document so we can return the document object for
	  use by the application. */

	return aFracAppDocument;

} // TFracAppApplication::DoMakeDocument

//-------------------------------------------------------------------------------------------


/* Handles the menu selections that have global thermonuclear impact. In this case, we handle
  the menu items that determine the characteristics of the next fractal document we create
  (dpi settings, calculation method, or use of 32 Bit CQD), and the Windows menu. Note that
  we have to do some special handling of the Windows menu, as MacApp knows nothing about it;
  we created that puppy ourselves. */

void TFracAppApplication::DoMenuCommand(CommandNumber aCommandNumber)
{
	/* Check to see if the user selected one of the window names that is dynamically
	  added to the Windows menu. It does this in two ways. First, it checks to see
	  if this is a menu command number that MacApp doesnt know about. In other
	  words, is this a menu that didnt start off life in a cmnu resource? If
	  not, then we added this menu item by hand with Menu Manager calls, and it
	  could be a Windows menu item. We also need to check to see if the command
	  number passed to us is the same as the dummy menu item we installed to
	  determine the location of the window menu items. This is because the first
	  window menu item will replace that dummy item, and assume its command number.
	  If either of these cases is true, then call DoWindowsMenu to deal with
	  bringing the window forward. */

	
	//if ((aCommandNumber < 0) || (aCommandNumber == cFirstWindowBase))
	//{
	//	this->DoWindowsCommand(aCommandNumber);
	//}
	//else
	{
		// A menu item that MacApp knows about was selected.

		switch (aCommandNumber)
		{
			/* Determine whether or not to use the 32 Bit CQD routines for our offscreen
			  creation and management. If not, well use some home grown routines. */

			case cUse32CQD:
				gUse32BitCQD = true;
				break;

			case cUseHomebrew:
				gUse32BitCQD = false;
				break;

			/* Set the printing/calculation resolution. This would be handy for generating
			  some 300 dpi pictures for laserprinters and such. */

			case cMake72dpi:
				gPrintResolution = 72;
				break;

			case cMake300dpi:
				gPrintResolution = 300;
				break;

			/* Which algorithm to use. Either calculate everything a pixel at a time, or use
			  the Mariani/Silver algorithm to preflight certain solid areas. */

			case cSlowAlgorithm:
				gUseFastAlgorithm = false;
				break;

			case cFastAlgorithm:
				gUseFastAlgorithm = true;
				break;

			// E. None of the above.

			default:
				Inherited::DoMenuCommand(aCommandNumber);
		}
	}
}

//-------------------------------------------------------------------------------------------


/* Determines the state of our menus. Before MacApp calls DoSetupMenus, it goes through and
  disables all the menu items, and removes any check marks. Then, starting with gTarget, it
  calls DoSetupMenus. Each DoSetupMenus should call its Inherited::method first so that they
  can set up the world the way the like it, and then we have a chance to override that
  (theres that word again...). In this case, TFracAppApplication deals with all menu items
  that have a global aspect (like the options for creating a new document), but arent
  handled by TApplication. */

void TFracAppApplication::DoSetupMenus(void)
{
	Inherited::DoSetupMenus(); 					// Do mainline stuff first.

	bool has32BitQD = Has32BitQD_AC();
	EnableCheck(cUse32CQD, has32BitQD, has32BitQD && gUse32BitCQD);
	EnableCheck(cUseHomebrew, true, ! gUse32BitCQD);
	EnableCheck(cMake72dpi, false, (gPrintResolution == 72));
	EnableCheck(cMake300dpi, false, (gPrintResolution == 300));
	EnableCheck(cSlowAlgorithm, true, ! gUseFastAlgorithm);
	EnableCheck(cFastAlgorithm, true, gUseFastAlgorithm);

	/* Make sure that our Windows menu is up to date. BuildWindowsMenu is our own
	  routine the will see if the Windows menu needs updating. If so, it will tear
	  it down, and rebuild it with the current list of windows. */
	
	//this->BuildWindowsMenu();

	/* Enable all of the items in the Window Menu. Also, match up the top window
	  with its menu item, and put a check mark by the menu item. */
}

//-------------------------------------------------------------------------------------------


/* Show our own About box rather than MacApps fine but rather standard one. We simply use a
  MacApp view and window, and call PoseModally on its TDialogView. */

void TFracAppApplication::DoAboutBox(void)
{
	TWindow* theWindow = dynamic_cast_AC(TWindow*,(gViewServer->NewTemplateWindow(kAbout, NULL)));
	IDType dismisser;
	do
	{
		dismisser = theWindow->PoseModally();
		if (dismisser == 'icon')
		{
			TWindow* theWindow2 = dynamic_cast_AC(TWindow*,gViewServer->NewTemplateWindow(kAbout2, NULL));
			IDType dismisser2 = theWindow2->PoseModally();
			theWindow2->Close();
		}
	} while (dismisser != 'okok');
	theWindow->Close();
} // TFracAppApplication::DoAboutBox

//-------------------------------------------------------------------------------------------


/* Sub-method to handle the Windows menu. Called by TFracAppApplication::DoMenuCommand to map
  the menu item selected to a window, and to bring that window to front. It does this by
  converting the menu item number into an index into the applications fWindowList, and
  extracting that TWindow. When we get it, we bring it to front with the TWindow.Select
  method. */

#if 0 

void TFracAppApplication::DoWindowsCommand(CommandNumber aCommandNumber)
{
	short menu;
	short item;
	CommandToMenuItem(aCommandNumber, menu, item);
	
	TWindow* chosenWindow = (TWindow*)((fWindowList->At(item - gFirstWindowBase + 1)));
	#if qDebug
		::ThrowIfNULL_AC(chosenWindow);
	#endif
	chosenWindow->Select();
}
#endif

//-------------------------------------------------------------------------------------------


/* This method is called by TFracAppWindows when they either show or hide themselves. They
  pass in a reference to themselves (this), and whether they are showing or hiding
  themselves. This routine then either adds that window to or deletes it from a list it
  maintains for the purpose. After its done that, it flags the menus as dirty. */

#if 0 

void TFracAppApplication::InstallWindowMenuItem(TWindow* window, bool install)
{
	if (install)
		fWindowList->InsertLast(window);
	else
		fWindowList->AtDelete(fWindowList->GetIdentityItemNo(window));

	InvalidateMenus();
	gRebuildWindowsMenu = true;
} // InstallWindowMenuItem

#endif

//-------------------------------------------------------------------------------------------


/* Handy routine used when multi-paging. After the previous document has been closed, we want
  to make a new one. We figure out the coordinates of the next page to be done, and check to
  see if it is in range of our maximum page range. If not, we are all done, and dont do
  anything. However, if we need to continue, we call OpenNew to kick off a new document. */

void TFracAppApplication::MakeNewGibbleyFromGPageRecord(void)
{
	/* Work on creating the new document. We base this on the values of realMin,
	  imagMin, realMax, and imagMax that we save from the last document. These
	  values are jammed into gPageRecord before the old document is erased from
	  existence. */

	bool	morePagesToMake = true;

	CExtended realMax(gPageRecord.realMax);
	CExtended realMin(gPageRecord.realMin);
	CExtended imagMax(gPageRecord.imagMax);
	CExtended imagMin(gPageRecord.imagMin);
	
	long double deltaReal = realMax - realMin;
	long double deltaImag = imagMax - imagMin;

	if (gPageRecord.currentH < gPageRecord.maxH)
	{
		gPageRecord.currentH++;			// We just move one page to the right.

		imagMin = imagMax;
		imagMax = imagMin + deltaImag;
	}
	else
	{
		/* If we are off the end of the maximum number of pages vertically,
		  we are done, so we can just mark it as done and skip it. */

		if (gPageRecord.currentV < gPageRecord.maxV)
		{
			gPageRecord.currentH = 1;		// Move back to the left hand side.
			gPageRecord.currentV++;			// and down a page.

			imagMin -= (gPageRecord.maxH - 1) * deltaImag;
			imagMax = imagMin + deltaImag;

			// Move down a page vertically, based on old position.
			realMin = realMax;
			realMax = realMin + deltaReal;
		}
		else
		{
			morePagesToMake = false;
		}								// if done with page
	}									// if done with this row


#if qPowerPC
	gPageRecord.realMax = realMax;
	gPageRecord.realMin = realMin;
	gPageRecord.imagMax = imagMax;
	gPageRecord.imagMin = imagMin;
#else	// SCpp can't figure this out
	gPageRecord.realMax = *(extended80 *)realMax;
	gPageRecord.realMin = *(extended80 *)realMin;
	gPageRecord.imagMax = *(extended80 *)imagMax;
	gPageRecord.imagMin = *(extended80 *)imagMin;
#endif

	if (morePagesToMake)
	{
		this->OpenNew(cContinuingMultiPage);
	}
	
} // MakeNewGibbleyFromGPageRecord
