/*-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker
PowerPC version by Jeroen Schalk

Copyright  1988-1994 Apple Computer, Inc.
All rights reserved.

--------------------------------------------------------------------------------------------

Main unit for FracApp 3.1. It defines the following classes:

TFracAppApplication - Handles creation of documents, Windows menu, idle time
	munging, and multipaging.

--------------------------------------------------------------------------------------------*/

#ifndef __FRACAPP__
#define __FRACAPP__

#ifndef __UAPPLICATION__
#include <UApplication.h>
#endif

#ifndef __FRACRECORD__
#include "FracRecord.h"
#endif

class TFracAppDocument; // external

//-------------------------------  Application  -------------------------------

DeclareClassDesc(TFracAppApplication);

class TFracAppApplication : public TApplication 
{
	DeclareClass(TFracAppApplication);

  public:
	//CList_AC*		fWindowList;	 /* Private - Holds the list of windows
	//								in the order in which they were
	//								created. Maintained by InstallWindowMenuItem */
	TFracAppDocument*	fLastCalculated; /* Keep track of last document calculated. */

	TFracAppApplication(OSType itsMainFileType = kTextType_AC);
	/* Initializes the application and globals. Called by the main application when
	   the applicaton object is created. */

	virtual void AutoSaveThisGuy(TFracAppDocument* doneDoc);
	/* Handy dude for saving and closing a document without human accompaniament.
	   We use this when creating a series of documents, possibly at night when
	   no one is around. Called by DoIdle when a document
	   signals that its done. */

	//virtual void BuildWindowsMenu(void);
	/* Called by DoSetupMenus whenever our Windows menu changes
	   and needs to be rebuilt. */

	virtual void Close(void);
	/* Called by MacApp when the user selects Quit. Tear down stuff when the program
	   is about to quit. Most importantly, we have to turn off the time manager
	   routines. Done here rather than in TApplication.Free, as .Free rarely gets
	   called. Its up to the main application to do that, and weve never
	   recommended that that be done. */

	virtual void CreateManyColors(void);
	// Called by IFracAppApplication to initialize our color Palettes.

	virtual TFracAppDocument* GetDocumentToCalculate(TFracAppDocument* lastCalculated);
	/* Looks for a document that needs calculation time. Round-robins among open
	  documents. */

	virtual TFracAppDocument* GetDocumentToAnimate(void);
	/* Finds a document that will do some animation for us. */

	virtual bool DoIdle(IdlePhase Phase);
	/* Called by MacApp to perform Idle time processing for the application.  This
	   will do the fractal calculation during the idle times. Also allows the
	   topmost document a chance to to color animation. */

	virtual TDocument* DoMakeDocument(CommandNumber itsCommandNumber, TFile* itsFile);
	/* Launches a TFracAppDocument; called when applications icon is opened, or
	   when New, New From Selection, New MultiPage, or Open is requested by the
	   user. Every application which uses documents MUST override this method. */

	virtual void DoMenuCommand(CommandNumber aCommandNumber);
	/* Called by MacApp when a menu item has been chosen. Handles executing Animate,
	   32-Bit Color QuickDraw, and 300 dpi options. */

	virtual void DoSetupMenus(void);
	/* Called by MacApp when the user clicks on the menu. Handles the en/disabling
	   of Animate, 32-Bit Color QuickDraw, and 300 dpi options. */

	virtual void DoAboutBox(void);
	/* Shows the About Box. With MacApp 2.0, this method is called from a
	   TAboutCommand when the user selects the About item. */

	//virtual void DoWindowsCommand(CommandNumber aCommandNumber);
	/* Sub-method to handle selections in the Windows menu. Called by
	   DoMenuCommand. */

	//virtual void InstallWindowMenuItem(TWindow* window, bool install);
	/* Called by our TFracAppWindow object when it shows or hides itself. This
	   method keeps track of the window in a list, which is used for building
	   our Windows menu. */

	virtual void MakeNewGibbleyFromGPageRecord(void);
	/* Handy routine for when we are multipaging. It is called from our DoIdle
	   method to create a new document based on the state of the last one. */
};								// TFracAppApplication

//-------------------------------------------------------------------------------------------

void MySetPalette(TWindow* theWindow);

//-------------------------------------------------------------------------------------------

#endif
