//	UFracGlobals.cp

#ifndef __UFRACGLOBALS__
#include "UFracGlobals.h"
#endif

/* This is a handle to the color table that we use throughout the program. It is loaded in
  from a clut resource early on in the program. It is used when we create an offscreen
  world for the offscreen gDevices colortable, and also for some of the whizzy animation
  effects that we do (see MakeOffWorlder, JumblePalette and RestorePalette) */

CTabHandle 	gOurColors;

// Set to true if we have selected the menu item for the Mariani/Silver algorithm.

bool		gUseFastAlgorithm;

/* Normally set to 72. Its here so that we can implement other resolutions (like 300 dpi) in
  the future. */

short		gPrintResolution;

/* We set this global variable to true if we want to do palette animation at idle time. We
  also check it when we set up the menus so we know if we need to check the menu item or
  not. */

bool		gAnimate;

/* This bool is set to true when we have jumbled up the palettes. We use it when we are
  setting up the menus so we know if we need to check the menu item or not. */

bool		gPaletteIsJumbled;

/* We need some sort of clutch when animating. I initially animated whenever my DoIdle method
  was called. However, on faster machines, we could get called more times in a second than
  we care for. So we keep track of when we last animated, and make sure that a decent number
  of ticks go by before animating again. This makes sure that our effects dont go whizzing
  by too fast on that 100MHz Mac III. */

long		gLastAnimated;

/* Wed like to take advantage to default application palettes if the system supports them.
  (see technote #211). To condition our program in the places necessary to support default
  application palettes, we set this global variable when running on the those systems. */

bool		gHasDefaultApplicationPalettes;

/* When we are creating a new document, we have some preset values that define the number
  range applied to the Mandelbrot calculations. On the other hand, if we are creating a new
  document based on the selection, we need to initialize the document with that information.
  The following boolean says which method of initialization we are using, and the PageRecord
  holds the necessary information for when gCreateFromPageRecord is true. These are accessed
  in TFracAppDocument::DoInitialState. */

PageRecord	gPageRecord;

/* gCounter is like a running stopwatch with millisecond resolution.
   Whenever I need to timestamp something, I just look at gCounter. */

long gCounter;

/* FracApp makes large windows, which tend to hide each other. We have a Windows menu that
  allows easy access to all windows. The way it works is all this documents are shown in a
  TFracAppWindow. This windows Show method is overridden to call
  TFracAppApplication::InstallWindowMenuItem when it is shown or hidden, and hence needs to
  be added to or deleted from the Windows menu. InstallWindowMenuItem does what it needs in
  order to remember what windows are visible, and then sets gRebuildWindowsMenu to true.
  Also, when a documents name changes (say, we just saved it to disk), we need to update
  its name in the menubar, so we again set gRebuildWindowsMenu to true. The next time our
  applications DoSetupMenus is called, it calls BuildWindowsMenu. BuildWindowsMenu checks
  this flag to see if it needs to do anything. If so, it rebuilds the menu, and clears the
  flag. */

bool		gRebuildWindowsMenu;

/* So that we can intelligently move the items in our infobar around, we need to know how
  wide the items are. Finding the width of the labels can be done by doing a StringWidth on
  the text for those labels. In order to find the length of the time items, we create a
  prototype string at application initialization time. This string is created by calling
  IUTimeString with a time of zero. This will represent the width of a time string, as all
  numerical digits are guaranteed to be the same width in all fonts. By creating this
  string, we can use it for determining the spacing for the time elements. */

CStr255_AC		gMaxWidthTimeString;
