//	UFracGlobals.h

#ifndef __UFRACGLOBALS__
#define __UFRACGLOBALS__

#ifndef __UMACAPPGLOBALS__
#include <UMacAppGlobals.h>
#endif

#ifndef __PALETTES__
#include <Palettes.h>
#endif

#ifndef __TIMER__
#include <Timer.h>
#endif

#ifndef __FRACRECORD__
#include "FracRecord.h"
#endif

//-------------------------------------------------------------------------------------------

// Constants used for identification

// Window and view resource numbers

const short kFracAppWindowID	= kDefaultWindowID;	// ID of the view resource used to create our windows
const short kAbout				= 4000;				// ID for our about dialog
const short kAbout2 			= 4001;				// Ditto
const short kMultiDialog		= 2002;				// Dialog Id for multipage dialog

// Constants for our own menu items.

const CommandNumber cNewFromSelection	= 1000;		// New Fractal from selection.
const CommandNumber cNewMultiPage		= 1001; 	// New Multipage
const CommandNumber cAnimate			= 1002; 	// Start the palettes moving
const CommandNumber cJumble 			= 1003; 	// Mix up the palette
const CommandNumber cNormal 			= 1004; 	// Make the palette normal again
const CommandNumber cUse32CQD			= 1005; 	// Use 32CQD for offscreen management.
const CommandNumber cUseHomebrew		= 1006; 	// Use old method for offscreen stuff.
const CommandNumber cMake72dpi			= 1007; 	// Generate at 72 dpi for the screen.
const CommandNumber cMake300dpi 		= 1008; 	// Generate a 300 dpi image for the printer
const CommandNumber cSlowAlgorithm		= 1009; 	// Use left/right, up/down method
const CommandNumber cFastAlgorithm		= 1010; 	// Use divide & conquer method
const CommandNumber cFirstWindowBase	= 2000; 	// Dummy item to find where windows go
const CommandNumber cContinuingMultiPage = 2001;	// Passed to OpenNew when making addl docs

// Other Thangs...

const short kNumColors			= 195;				/* number of colors we animate, and use in
													 calculation. */

//-------------------------------------------------------------------------------------------

/* This is a handle to the color table that we use throughout the program. It is loaded in
  from a clut resource early on in the program. It is used when we create an offscreen
  world for the offscreen gDevices colortable, and also for some of the whizzy animation
  effects that we do (see MakeOffWorlder, JumblePalette and RestorePalette) */

extern CTabHandle 	gOurColors;

// Set to true if we have selected the menu item for the Mariani/Silver algorithm.

extern bool		gUseFastAlgorithm;

/* Normally set to 72. Its here so that we can implement other resolutions (like 300 dpi) in
  the future. */

extern short		gPrintResolution;

/* We set this global variable to true if we want to do palette animation at idle time. We
  also check it when we set up the menus so we know if we need to check the menu item or
  not. */

extern bool		gAnimate;

/* This bool is set to true when we have jumbled up the palettes. We use it when we are
  setting up the menus so we know if we need to check the menu item or not. */

extern bool		gPaletteIsJumbled;

/* We need some sort of clutch when animating. I initially animated whenever my DoIdle method
  was called. However, on faster machines, we could get called more times in a second than
  we care for. So we keep track of when we last animated, and make sure that a decent number
  of ticks go by before animating again. This makes sure that our effects dont go whizzing
  by too fast on that 100MHz Mac III. */

extern long		gLastAnimated;

/* Wed like to take advantage to default application palettes if the system supports them.
  (see technote #211). To condition our program in the places necessary to support default
  application palettes, we set this global variable when running on the those systems. */

extern bool		gHasDefaultApplicationPalettes;

/* When we are creating a new document, we have some preset values that define the number
  range applied to the Mandelbrot calculations. On the other hand, if we are creating a new
  document based on the selection, we need to initialize the document with that information.
  The following boolean says which method of initialization we are using, and the PageRecord
  holds the necessary information for when gCreateFromPageRecord is true. These are accessed
  in TFracAppDocument::DoInitialState. */

extern PageRecord	gPageRecord;

/* gCounter is like a running stopwatch with millisecond resolution.
   Whenever I need to timestamp something, I just look at gCounter. */

extern long gCounter;

/* FracApp makes large windows, which tend to hide each other. We have a Windows menu that
  allows easy access to all windows. The way it works is all this documents are shown in a
  TFracAppWindow. This windows Show method is overridden to call
  TFracAppApplication::InstallWindowMenuItem when it is shown or hidden, and hence needs to
  be added to or deleted from the Windows menu. InstallWindowMenuItem does what it needs in
  order to remember what windows are visible, and then sets gRebuildWindowsMenu to true.
  Also, when a documents name changes (say, we just saved it to disk), we need to update
  its name in the menubar, so we again set gRebuildWindowsMenu to true. The next time our
  applications DoSetupMenus is called, it calls BuildWindowsMenu. BuildWindowsMenu checks
  this flag to see if it needs to do anything. If so, it rebuilds the menu, and clears the
  flag. */

extern bool		gRebuildWindowsMenu;

/* So that we can intelligently move the items in our infobar around, we need to know how
  wide the items are. Finding the width of the labels can be done by doing a StringWidth on
  the text for those labels. In order to find the length of the time items, we create a
  prototype string at application initialization time. This string is created by calling
  IUTimeString with a time of zero. This will represent the width of a time string, as all
  numerical digits are guaranteed to be the same width in all fonts. By creating this
  string, we can use it for determining the spacing for the time elements. */

extern CStr255_AC		gMaxWidthTimeString;

//-------------------------------------------------------------------------------------------

#endif
