//----------------------------------------------------------------------------------------
// UFracThread.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __FRACRECORD__
#include "FracRecord.h"
#endif

#if qUseThreads

#ifndef __CMPSchedule_AC__
#include "CMPSchedule_AC.h"
#endif

#ifndef __CThreadMgrSchedule_AC__
#include "CThreadMgrSchedule_AC.h"
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CWhileSynchronized_AC__
#include "CWhileSynchronized_AC.h"
#endif

#ifndef _STDLIB
#include <stdlib.h>
#endif

#include "UFracThread.h"

#ifndef __UENGINE__
#include "UEngine.h"
#endif

CAutoPtr_AC<CAbstractThreadSchedule_AC> TFracThread::fSchedule;


TFracThread::TFracThread(CFracAppEngine* theEngine) :
	fEngine(theEngine)
{
}

TFracThread::~TFracThread()
{
}

void TFracThread::Run()
{
	bool done = ShutdownRequested();
	while (!done){
	   done = fEngine->CalcCity();
	   Yield();
	   done = done || ShutdownRequested();
	}
	CWhileSynchronized_AC(&fEngine->fLock);
	if (--fEngine->fActiveThreadCounter == 0) {
		fEngine->fDocumentIsFinished = TRUE;
	}
}

CAbstractThreadSchedule_AC *TFracThread::GetThreadSchedule()
{
	if (fSchedule.IsNULL()) {
#if qPowerPC && qWantsMP
		if (fEngine->fThreadStyle == kPreemptiveThread ) {
			fSchedule = TH_new CMPSchedule_AC();
		}
		else {
			fSchedule = TH_new CThreadManagerSchedule_AC();
		}
#else
		fSchedule = TH_new CThreadManagerSchedule_AC(fEngine->fThreadStyle);
#endif
	}
	return fSchedule;
}

#endif
