//----------------------------------------------------------------------------------------
// UFracViews.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UFRACVIEWS__
#include "UFracViews.h"
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __UAREASELECTOR__
#include "UAreaSelector.h"
#endif

#ifndef __UFRACGLOBALS__
#include "UFracGlobals.h"
#endif

#ifndef __FRACAPP__
#include "UFracApp.h"
#endif

#ifndef __UFRACDOC__
#include "UFracDoc.h"
#endif

#ifndef __UFRACVIEWS__
#include "UFracViews.h"
#endif

#ifndef __USCROLLER__
#include <UScroller.h>
#endif

#ifndef __UMENUMGR__
#include <UMenuMgr.h>
#endif

#ifndef __UCLIPBOARDMGR__
#include <UClipBoardMgr.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

//-------------------------------------------------------------------------------------------

const short kAnimationDelay 	= 2;				// Ticks between palette animation

//-------------------------------------------------------------------------------------------
//---------------------------------TFracAppWindow Methods------------------------------------
//-------------------------------------------------------------------------------------------


// Initialize our window object. Overridden to set our instance variables to NULL.

#undef Inherited
#define Inherited TWindow
DefineClass(TFracAppWindow, TWindow);

void TFracAppWindow::Initialize()
{
	fCTimeLabelView = NULL;
	fCTimeView = NULL;
	fETimeLabelView = NULL;
	fETimeView = NULL;
	fFracAppView = NULL;
	fMethodNameView = NULL;
	fPercentView = NULL;
	fPercentLabelView = NULL;
	fSingleBarView = NULL;
}

//-------------------------------------------------------------------------------------------


/* Initialize a lot of the windows fields with references to its subviews. */

void TFracAppWindow::DoPostCreate(TDocument *itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	fFracAppView = (TFracAppView*)(this->FindSubView('FRAC'));
	fCTimeLabelView = (TStaticText*)(this->FindSubView('tCTL'));
	fCTimeView = (TStaticText*)(this->FindSubView('tCTm'));
	fETimeLabelView = (TStaticText*)(this->FindSubView('tETL'));
	fETimeView = (TStaticText*)(this->FindSubView('tETm'));
	fMethodNameView = (TStaticText*)(this->FindSubView('tTYP'));
	fPercentView = (TStaticText*)(this->FindSubView('tPCT'));
	fPercentLabelView = (TStaticText*)(this->FindSubView('tPCL'));
	fSingleBarView = (TControl*)(this->FindSubView('sBAR'));
}

//-------------------------------------------------------------------------------------------


/* Handy subroutine that takes a string and a style that it will be imaged with.
  It returns the width of that string in that style, adding in kStaticTextSlop. */

short GetTextWidth(const CStr255_AC& itsText, const TextStyle& itsTextStyle);
short GetTextWidth(const CStr255_AC& itsText, const TextStyle& itsTextStyle)
{
	const short kStaticTextSlop 	= 1;	/* Slop to add in GetTextWidth. The width of our
	TStaticText items cant just be the width of the string. We also have to add a 1 pixel
	slop. This is because imaging is done with TextEdit, which inserts a 1 pixel margin
	at the left so that it can put an insertion point there. We have to take this into account
	when sizing our fields. */

	GrafPtr			oldPort;

	::GetPort(&oldPort);
	::SetPort(gWorkPort);
	SetPortTextStyle(itsTextStyle);
	short width = ::StringWidth(itsText) + kStaticTextSlop;
	::SetPort(oldPort);

	return width;
}

//-------------------------------------------------------------------------------------------


/* Called by TFracAppWindow::Show and Resize to make sure the items in the infobar and the
  percentage complete label are all in the right places. */

/* We define this record to keep track of the new locations of our fields. Each
  one needs a newLocation specification, as well as a horizontal size (we never
  change an items height). We also keep a newEnd field here, which is
  newLocation.h + newSize, for convenience. */

typedef struct NewSpecs
{
	Point	newLocation;
	short	newEnd;
	short	newSize;
};

// Subroutine that moves and sizes theView according to theSpecs

static void MoveView(TView* theView, const NewSpecs& theSpecs, bool invalidate)
{
	//	theView->Locate(theSpecs.newLocation.h, theSpecs.newLocation.v, invalidate);
	//	theView->Resize(theSpecs.newSize, theView->GetSize().v, invalidate);
	CViewRect frame(theSpecs.newLocation.h, theSpecs.newLocation.v,
				theSpecs.newLocation.h + theSpecs.newSize, theSpecs.newLocation.v + theView->GetSize().v);
	theView->SetFrame(frame, invalidate);
}

void TFracAppWindow::AdjustInfoBar(ViewCoordinate width, ViewCoordinate height,
										  bool invalidate)
{
	const short kInfoBarMargin		= 5;		// Margin to use on the left and right sides of the infobar.
	const short kLabelMargin		= 5;		// Margin to use between a time display and its label.
	const short kMinimumSpacing 	= 15;		// Minimum space to maintain between fields.
	const short kTimeSlop			= 0;		// Slop to add to time elements.

	CStr255_AC		itsText;
	TextStyle	itsStyle;
	NewSpecs 	specCTimeLabel;
	NewSpecs	specCTime;
	NewSpecs 	specETimeLabel;
	NewSpecs	specETime;
	NewSpecs 	specMethodName;
	NewSpecs	specPercentLabel;
	NewSpecs	specPercent;
	NewSpecs	specSingleBar;
	short		rightEdge;
	short		topEdge;
	TScrollBar*	bottomScrollBar;

	/* We dont want to do anything if none of our subviews exists yet. It is
	  possible for this routine to get called before the window is completely
	  built, in which case our subviews might not be created yet. Check to see if
	  they exists, and exit if they dont. */

	if (fCTimeLabelView == NULL)
		return;

	/* Do the leftmost item. This is the fields that says Calculation Time:
	  xx/xx/xx. It is in two parts: a TStaticText for the Calculation Time: 
	  (the label) and another for the time itself. The label is positioned
	  kInfoBarMargin pixels from the left. The time field is positioned
	  kLabelMargin pixels past the label. */

	itsText = fCTimeLabelView->GetText();
	itsStyle = fCTimeLabelView->fTextStyle;
	specCTimeLabel.newSize = GetTextWidth(itsText, itsStyle) + kLabelMargin;
	specCTimeLabel.newLocation.h = kInfoBarMargin;
	specCTimeLabel.newLocation.v = short(fCTimeLabelView->GetLocation().v);
	specCTimeLabel.newEnd = specCTimeLabel.newLocation.h + specCTimeLabel.newSize;

	itsText = gMaxWidthTimeString;
	itsStyle = fCTimeView->fTextStyle;
	specCTime.newSize = GetTextWidth(itsText, itsStyle) + kTimeSlop;
	specCTime.newLocation.h = specCTimeLabel.newEnd;
	specCTime.newLocation.v = short(fCTimeView->GetLocation().v);
	specCTime.newEnd = specCTime.newLocation.h + specCTime.newSize;

	/* Do the rightmost item. This is the fields that says Elapsed Time: xx/xx/xx.
	  It is in two parts: a TStaticText for the Elapsed Time:  (the label) and
	  another for the time itself.The time field is positioned kInfoBarMargin
	  pixels from the right edge of the window. The label is positioned
	  kLabelMargin pixels to the right of the time field. */

	specETime.newSize = specCTime.newSize;
	specETime.newEnd = short(width - kInfoBarMargin);
	specETime.newLocation.h = specETime.newEnd - specETime.newSize;
	specETime.newLocation.v = short(fETimeView->GetLocation().v);

	itsText = fETimeLabelView->GetText();
	itsStyle = fETimeLabelView->fTextStyle;
	specETimeLabel.newSize = GetTextWidth(itsText, itsStyle) + kLabelMargin;
	specETimeLabel.newEnd = specETime.newLocation.h;
	specETimeLabel.newLocation.h = specETimeLabel.newEnd - specETimeLabel.newSize;
	specETimeLabel.newLocation.v = short(fETimeLabelView->GetLocation().v);

	/* Center the middle item. This is the TStaticText item that holds the string
	  displaying what algorithms we are using. Its position is determined by
	  centering it between the  left and right edges of the window. */

	itsText = fMethodNameView->GetText();
	itsStyle = fMethodNameView->fTextStyle;
	specMethodName.newSize = GetTextWidth(itsText, itsStyle);
	specMethodName.newLocation.h = short((width - specMethodName.newSize) / 2);
	specMethodName.newLocation.v = short(fMethodNameView->GetLocation().v);
	specMethodName.newEnd = specMethodName.newLocation.h + specMethodName.newSize;

	/* Make sure we maintain minimum spacing. Examine the locations of the three
	  items weve just calculated. See if they are all at least kMinimumSpacing
	  pixels from each other. If not, then shift everything over to the right to
	  maintain this spacing. Note that this means that the Elapsed time field can
	  get shoved off the right edge of the window if the window is too small. */

	specMethodName.newLocation.h = short(Max_AC(specMethodName.newLocation.h,
											 specCTime.newEnd + kMinimumSpacing));
	specETimeLabel.newLocation.h = short(Max_AC(specETimeLabel.newLocation.h,
											 specMethodName.newLocation.h +
											 specMethodName.newSize + kMinimumSpacing));
	specETime.newLocation.h = specETimeLabel.newLocation.h + specETimeLabel.newSize;

	/* Do the percent complete item at the bottom. This guy is centered between the
	  left edge of the window, and the right edge of the bottom scrollbar. Note
	  that this is the only item here that I move around vertically as well. Moving
	  the Percent Complete item also means having to move the little line used to
	  seperate it from the fractal view above it. */

	bottomScrollBar = fFracAppView->GetScroller(false)->fScrollBars[hSel];
	rightEdge = short(bottomScrollBar->GetLocation().h);
	topEdge = short(height - bottomScrollBar->GetSize().v);

	specSingleBar.newSize = rightEdge;
	specSingleBar.newLocation.h = 0;
	specSingleBar.newLocation.v = topEdge + 1;

	itsText = fPercentLabelView->GetText();
	itsStyle = fPercentLabelView->fTextStyle;
	specPercentLabel.newSize = GetTextWidth(itsText, itsStyle);

	itsText = "000";
	itsStyle = fPercentView->fTextStyle;
	specPercent.newSize = GetTextWidth(itsText, itsStyle);
	specPercent.newLocation.h = (rightEdge - specPercent.newSize - specPercentLabel.newSize) / 2;
	specPercent.newLocation.v = topEdge + 2;
	specPercent.newEnd = specPercent.newLocation.h + specPercent.newSize;

	specPercentLabel.newLocation.h = specPercent.newEnd;
	specPercentLabel.newLocation.v = topEdge + 2;

	//--- Move them all into position ---

	MoveView(fCTimeLabelView, specCTimeLabel, invalidate);
	MoveView(fCTimeView, specCTime, invalidate);
	MoveView(fETimeLabelView, specETimeLabel, invalidate);
	MoveView(fETimeView, specETime, invalidate);
	MoveView(fMethodNameView, specMethodName, invalidate);
	MoveView(fPercentLabelView, specPercentLabel, invalidate);
	MoveView(fPercentView, specPercent, invalidate);
	MoveView(fSingleBarView, specSingleBar, invalidate);
}

//-------------------------------------------------------------------------------------------


/* Called during Idle time to do any palette manager animation that may be needed. Three
  conditions must be satisfied before we animate: 1) the user must select the right menu
  item, 2) we must be in the foreground, and 3) a decent amount of time has to have elapsed
  since we last animated (we dont want things to go whizzing by too fast on those faster
  Macs).

  If all of those conditions are satisfied, we animate. We first get a copy of the windows
  current palette and put it into a color table with the Palette2Ctab call. We then shift up
  all the color entries, moving the one at the beginning to the end. Finally, we call
  AnimatePalette with our modified color table. */

void TFracAppWindow::AnimateColors(void)
{
	if (gAnimate && ! gDispatcher->IsFrontProcess() &&
		(::TickCount() > (gLastAnimated + kAnimationDelay)))
	{
		/* Get the palette for our window. First attempt to get any palette that is
		  directly attached to the window. If that doesnt work, try to get the default
		  application palette. Once we got one, convert it to a color table. */

		PaletteHandle currPalette = ::GetPalette(GetWindowRef());
		if ((currPalette == NULL) && gHasDefaultApplicationPalettes)
			currPalette = ::GetPalette(WindowPtr( - 1));
		::ThrowIfNULL_AC(currPalette);

		CTabHandle destCTab = NULL;

		try
		{
			destCTab = CTabHandle(::NewHandle(sizeof(ColorTable) +
													   ((kNumColors + 16) * sizeof(ColorSpec))));
			::ThrowIfNULL_AC(destCTab);
			::Palette2CTab(currPalette, destCTab);

		}
		catch(...)
		{
			gAnimate = false;
			throw;
		}

		/* Move the colors around in the color table, skipping the first 16, and moving
		  all the elements down by one, and copying the element at 16 back to the end
		  of the table. The effect is to rotate the colors in the table. */

		ColorSpec lastCSpec = (**destCTab).ctTable[16];	// pull first one off.
		::BlockMove(&(**destCTab).ctTable[17], &(**destCTab).ctTable[16],
				  (kNumColors - 1) * sizeof(ColorSpec));		// copy all one entry down.
		(**destCTab).ctTable[kNumColors + 16 - 1] = lastCSpec; // put last color back on front.
		::AnimatePalette(GetWindowRef(), destCTab, 16, 16, kNumColors);

		::DisposeHandle(Handle(destCTab));
		gLastAnimated = ::TickCount();
	}
}

//-------------------------------------------------------------------------------------------


/* Called in response to the user selecting the Jumble Palette menu item. This takes our
  color table, rotates every third color 65 position up, another every third color 65
  positions down, and leaves the rest alone. It then calls AnimatePalette with this rotated
  color table. */

void TFracAppWindow::JumblePalette(void)
{
	CTabHandle newcolors = gOurColors;
	::ThrowIfError_AC(::HandToHand((Handle *)&newcolors));
	::ThrowIfNULL_AC(newcolors);
	{
		ColorSpecPtr ct = (**newcolors).ctTable;
		CRGBColor_AC tempRGB;
		for (short i = 0; i <= (63 / 3); ++i)
		{
			tempRGB			= ct[i*3+1].rgb;
			ct[i*3+1].rgb	= ct[i*3+66].rgb;
			ct[i*3+66].rgb	= ct[i*3+131].rgb;
			ct[i*3+131].rgb	= (RGBColor)tempRGB;

			tempRGB 		= ct[i*3+132].rgb;
			ct[i*3+132].rgb	= ct[i*3+67].rgb;
			ct[i*3+67].rgb	= ct[i*3+2].rgb;
			ct[i*3+2].rgb	= (RGBColor)tempRGB;
		}
	}
	::AnimatePalette(GetWindowRef(), newcolors, 1, 16, kNumColors);
	::DisposeHandle(Handle(newcolors));
}

//-------------------------------------------------------------------------------------------


/* Called by MacApp when it needs to open a window. We override it here so that we can also
  call AdjustInfoBar. */

void TFracAppWindow::Open(void)
{
	this->AdjustInfoBar(GetSize().h, GetSize().v, kDontInvalidate);
	Inherited::Open();
}

//-------------------------------------------------------------------------------------------


/* Called by MacApp when it needs to resize a window. We override it here so that we can also
  call AdjustInfoBar. */

void TFracAppWindow::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	this->AdjustInfoBar(GetSize().h, GetSize().v, invalidate);
}

//-------------------------------------------------------------------------------------------


/* This effectively nullifies the effects of AnimateColors or JumblePalette. It makes sure
  that the color tables for our offscreen world and our gDevices match again. */

void TFracAppWindow::RestorePalette(void)
{
	::AnimatePalette(GetWindowRef(), gOurColors, 1, 16, kNumColors);
}

//-------------------------------------------------------------------------------------------


/* We support a Windows menu by having smarter than average windows. These windows tell
  MacApp when they are showing and hiding themselves. They do this by calling
  TFracAppApplication::InstallWindowMenuItem. That method notes what is going on, and
  rebuilds the Windows menu appropriately. */

void TFracAppWindow::Show(bool state, bool redraw)
{
	Inherited::Show(state, redraw);
	//((TFracAppApplication*)(gApplication))->InstallWindowMenuItem(this, state);
}

//-------------------------------------------------------------------------------------------


/* If our window changes its name, we need to tell the application so that it can rebuild the
  Windows menu. Otherwise, well have a menu item that refers to a window whose title no
  longer matches that of the menu item. */

void TFracAppWindow::SetTitle(const CStr255_AC& newTitle)
{
	Inherited::SetTitle(newTitle);
	//InvalidateMenus();
	//gRebuildWindowsMenu = true;
}

//-------------------------------------------------------------------------------------------
//-------------------------------TNoFlashStaticText Methods----------------------------------
//-------------------------------------------------------------------------------------------
/* This object is used to display the elapsed time items, and the percentage complete item. I
  found that with a normal TStaticText item, I got too much flashing when I updated the
  strings. There was too much of a time gap between when the old string got erased, and the
  new one got drawn. Ive tried to minimize that by having MATextbox (which is what
  ultimately draws the string) erase the old text when it draws the new. There is still SOME
  flicker, but not as much as there used to be. */
//-------------------------------------------------------------------------------------------


/* Called by TStaticText in its Draw method to do the actual imaging of text. Overridden so
  that it passes the kEraseText parameter to MATextbox. */

#undef Inherited
#define Inherited TStaticText
DefineClass(TNoFlashStaticText, TStaticText);

void TNoFlashStaticText::Initialize()
{
	// fEraseFirst = kNoEraseFirst;
}

//-------------------------------------------------------------------------------------------


/* This method is called to set the text that the TStaticText item displays. It draws the
  text immediately, without invalidating the screen and simply responding to the update
  event. In doing this, it calls EraseRect, and then calls Draw. Overridden to inhibit
  erasing the old text before drawing the new, as we now do that in ImageText. */

void TNoFlashStaticText::SetText(const CStr255_AC& theText, bool redraw)
{
	if ((fDataHandle == NULL) || (theText != **fDataHandle))
	{
		this->ReleaseText();
		fDataHandle = (CPascalStrHandle_AC)::NewString(theText);
		if (MemError() != noErr)
			fDataHandle = NULL;

		if (redraw && this->Focus() && this->IsVisible())
		{
			CViewRect area;

			this->ControlArea(area);
			// ::EraseRect(area);				// Removed from original code.
			this->Draw(area);
		}
	}
}

//-------------------------------------------------------------------------------------------
//----------------------------------TFracAppView Methods-------------------------------------
//-------------------------------------------------------------------------------------------
/* This is the object that displays the fractal. Besides having its .Draw method overridden
  so that it can copybits the offscreen image, this View also keeps track of the current
  selection rectangle. Because of this, it also handles the menu items that would only be
  enabled when there is a selection. This includes Copy, New From Selection, and New
  Multi-Page. */
//-------------------------------------------------------------------------------------------


/* Init our FracAppView. This sets the current selection to an empty rect. */

#undef Inherited
#define Inherited TView
DefineClass(TFracAppView, TView);

void TFracAppView::Initialize(void)
{
	fSelectionRect = gZeroRect;
	fFracAppDocument = NULL;
}

//-------------------------------------------------------------------------------------------


/* Init our FracAppView. This keeps a reference to our owning document. This is so we can
  call some methods in that document that return information about the picture we have to
  display. This reference is kept in addition to the fDocument field that TViews already
  have. Our own reference is stored as a TFracAppDocument, and not just a plain TDocument.
  This is because we need a reference to a TFracAppDocument in order to access the methods
  that it has that a normal TDocument doesnt. We could just coerce fDocument into a
  TFracAppDocument whenever we accessed it, but thats a pain, and this is inexpensive. */

void TFracAppView::DoPostCreate(TDocument* itsDocument)
{
	fFracAppDocument = (TFracAppDocument*)(itsDocument);

	Inherited::DoPostCreate(itsDocument);
}

//-------------------------------------------------------------------------------------------


/* Called by MacApps updating routines to highlight the current selection rectangle if there
  is one. This is drawn in srcCopy mode to make it stand out better when it is a final
  selection.  XOR is used for the rubberband, until mouseUp. */

void TFracAppView::DoHighlightSelection(HLState fromHL, HLState toHL)
{
	if (this->HasSelection())
	{		
		CRect_AC tempRect;
		this->GetSelection(tempRect);
		if ((fromHL == hlOff) && (toHL == hlOn))
		{

			PatHandle selPatHandle = ::GetPattern(kSelPattern); // get the pattern we use.
			if (selPatHandle)
			{
				// If pattern available, use it.
				Pattern thePat = **selPatHandle;
				::PenPat(&thePat); 		// set pen pattern to our selection kind.
			}
			::PenMode(srcCopy);			// copy mode on pattern selection.
			::FrameRect(tempRect);		// outline the frame of selection.
		} 								// highlight turned on.
		else if ((fromHL == hlOn) && (toHL == hlOff))
		{
			/* Turning off the highlight. We need to remove the traces of the selection.
			  To do this, redraw that rectangle. */
			CViewRect area;
			area = QDToView(tempRect);
			SetSelection(gZeroRect, kDontRedraw);
			this->Draw(area);			// Redraw it to clear selection.
			SetSelection(tempRect, kDontRedraw);
		}
	}
} // TFracAppView::DoHighlightSelection

//-------------------------------------------------------------------------------------------


/* Called by MacApp when a menu item has been chosen. Handle the menu choices for New From
  Selection and New MultiPage out of the File Menu. These make new Fractals based on the
  current selection. It does it by calling on the application object to make a new document.
  The communication to the DoInitialState is through the global variable gPageRecord */

void TFracAppView::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber)
	{
		case cCopy:
			::ThrowIfError_AC(short(::ZeroScrap()));
			this->WriteToDeskScrap();
			gClipboardMgr->CheckDeskScrap();		// Force MacApp to notice the change
			break;

		case cSelectAll:
			{
				CRect_AC qdExtent(this->GetQDExtent());
				this->SetSelection(qdExtent, kRedraw);
			}
			break;

		case cNewFromSelection:
			fFracAppDocument->StashSelectedRange();
			gApplication->OpenNew(cNewFromSelection);
			break;

		case cNewMultiPage:
			{
				/* When they choose the MultiPage option, we have to put up the dialog to find
				  out what they want to do, get the values back, store them into the
				  gPageRecord global, and start up the first document based on the current
				  selection and the number of pages to do. */

				// Run the dialog to get the number of pages desired.

				TWindow* pageDialog = (TWindow*)(gViewServer->NewTemplateWindow(kMultiDialog, NULL));
				IDType dismisser = pageDialog->PoseModally();
				long horPages = ((TNumberText*)(pageDialog->FindSubView('HORZ')))->GetValue();
				long verPages = ((TNumberText*)(pageDialog->FindSubView('VERT')))->GetValue();
				pageDialog->Close();

				if (dismisser == 'OKOK')
				{

					fFracAppDocument->StashSelectedRange();

					/* Now that we have a Extended rectangle defining the area to calculate, we
					  need to get the next documents Extended rectangle by dividing that large
					  rectangle by the number of pages desired. */

					CExtended realMax(gPageRecord.realMax);
					CExtended realMin(gPageRecord.realMin);
					CExtended imagMax(gPageRecord.imagMax);
					CExtended imagMin(gPageRecord.imagMin);
					
#if qPowerPC
					gPageRecord.realMax = (realMin + (realMax - realMin) / verPages);
					gPageRecord.imagMax = (imagMin + (imagMax - imagMin) / horPages);
#else	// SCpp can't figure this out
					gPageRecord.realMax = *(extended80 *)(realMin + (realMax - realMin) / verPages);
					gPageRecord.imagMax = *(extended80 *)(imagMin + (imagMax - imagMin) / horPages);
#endif
				   // Now we have the page count desired, save it off in the global gPageRecord

					gPageRecord.maxH = short(horPages);
					gPageRecord.maxV = short(verPages);
					gPageRecord.currentH = 1;				// Start at page 1 on both axes.
					gPageRecord.currentV = 1;

					/* Now we have the actual area to use as the rectangle to calculate, we need to
					  turn the real numbers into the page rectangle that we will use to print and
					  save and so on. This is done by creating the full document in DoMakeDocument,
					  and the global variables set here will be used there. */

					gApplication->OpenNew(cNewMultiPage);
				}								// if (dismisser
			}
			break;

		default:
			Inherited::DoMenuCommand(aCommandNumber);		// next guy in chain.

	}										// switch on aCommandNumber
	/* Assume that we have no command to return, since none of our commands currently
	  change the document. */
} // TFracAppView::DoMenuCommand

//-------------------------------------------------------------------------------------------


/* Called by MacApp to handle the mouse events in the view. This will pass back the command
  object to handle tracking the mouse and creating a new selection in preparation for making
  a new fractal. */

void TFracAppView::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* /*event*/,
										 CPoint_AC /*hysteresis*/)
{
	TAreaSelector* tracker = TH_new TAreaSelector(this, fFracAppDocument, theMouse);
													// make a new command object.
	::ThrowIfNULL_AC(tracker);						// no memory, trash out.
	PostCommand(tracker);						// post it for later execution.
} // TFracAppView::DoMouseCommand

//-------------------------------------------------------------------------------------------


/* Called by MacApp when the user clicks on the menu. Set up the New Fractal menus choice in
  Fractal Menu, based on selection. */

void TFracAppView::DoSetupMenus(void)
{
	Inherited::DoSetupMenus(); 					// Do mainline stuff first.

	/* If we have a non-zero selection, we can enable the menu item to use it as the
	  new fractal dimensions for this document. */

	bool hasSelection = this->HasSelection();
	Enable(cCopy, hasSelection);
	Enable(cSelectAll, true);
	Enable(cNewFromSelection, hasSelection);
	Enable(cNewMultiPage, hasSelection);
} // TFracAppView::DoSetupMenus

//-------------------------------------------------------------------------------------------


/* Called by MacApp to draw the view seen in the window. Every nonblank view MUST override
  this method. This is the display routine to take the data out of the offscreen buffer and
  whip it up to the window, as the current view. The fractal is full page size, clips
  without scaling into the window. */

void TFracAppView::Draw(const CViewRect& area)
{
	/* Lock down our bits so that they dont move when we try to copy them to the
	  screen. */

	fFracAppDocument->LockThePixels(kLock);

	/* Set the fore and background colors. If we just leave them set to any old
	  values, Copybits will attempt to colorize the bitmap that we transfer. See
	  technote #163 for the gory details. */

	::RGBForeColor(gRGBBlack);
	::RGBBackColor(gRGBWhite);

	// Copy the bits to the screen, allowing CopyBits to sort out the colors.

	CRect_AC qdArea;
	qdArea = ViewToQD(area);
	::CopyBits(&GrafPtr(fFracAppDocument->GetFracPort())->portBits,
			   &(qd.thePort->portBits),
			   qdArea,
			   qdArea, srcCopy, NULL);

	// Were done with the pixels, so let em float.
	fFracAppDocument->LockThePixels(kUnlock);

	if (this->HasSelection())
		DoHighlightSelection(hlOff, hlOn);
		
} // TFracAppView::Draw

//-------------------------------------------------------------------------------------------


/* Overridden so that we can redefine the extent of the view when pasting to the scrap. One
  of the things that GetQDExtent is used for is by TView.WriteToDeskScrap. When this method
  is called, it is so that it will image the view into a PICT, and then put this PICT on the
  desk scrap. GetQDExtent is called to find out the boundaries of this PICT. We special case
  GetQDExtent here for when we are drawing to that PICT. We know this by the setting of the
  global variable, gDrawingPICTScrap. In that case, we only want to draw the current
  selection to the scrap. So we just return the fSelectionRect. */



//void TFracAppView::GetQDExtent(CRect_AC& qdExtent)
//{
//	if (! gDrawingPictScrap) {
//		Inherited::GetQDExtent(qdExtent);
//	} else {
//		CRect_AC aRect;
//		this->GetSelection(aRect);
//		qdExtent = aRect;
//	}
//}

//-------------------------------------------------------------------------------------------


/* Sets and redraws the selection. Redraws the selection only if the redraw parameter is
  true, we can focus (which just means that we have a port to draw into), and if any part of
  our view is visible. If any of those assertions are false, then we just remember the
  selection in fSelectionRect. */

void TFracAppView::SetSelection(const CRect_AC& theSelectionRect, bool redraw)
{
	if (redraw && this->Focus() && this->IsVisible())
	{
		this->DoHighlightSelection(hlOn, hlOff);
		fSelectionRect = theSelectionRect;
		this->DoHighlightSelection(hlOff, hlOn);
	}
	else
		fSelectionRect = theSelectionRect;
}
