/*-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker
PowerPC version by Jeroen Schalk

Copyright  1988-1994 Apple Computer, Inc.
All rights reserved.

--------------------------------------------------------------------------------------------

TFracAppWindow - Calls the application to tell it when windows are being added
	or removed. This is so we can have a Windows menu (very necessary when you
	have an application that makes all its windows as large as your screen.) Also
	repositions items in the info bars when the window changes size. This
	wouldnt be necessary if MacApp had location determiners to go along with its
	size determiners.
TFracAppView - CopyBits stuff from our offscreen pixmap to the window. Handles
	creation of new documents based on the selection, as the View keeps track of
	the selection rectangle. Also deals with highlighting the selection, and
	writing that selection to scrap on a Copy.

--------------------------------------------------------------------------------------------*/

#ifndef __UFRACVIEWS__
#define __UFRACVIEWS__

#ifndef  __UWINDOW__
#include <UWindow.h>
#endif

#ifndef  __UDIALOG__
#include <UDialog.h>
#endif

class TFracAppDocument; // external
class TFracAppView; // forward

	// Constants used for identification

const short kSelPattern		= 128;			/* pattern resource Id. This is in the public section
											as both UFracViews and UAreaSelector access the pattern. */

//-------------------------------  Window  -------------------------------

DeclareClassDesc(TFracAppWindow);

class TFracAppWindow : public TWindow 
{
	DeclareClass(TFracAppWindow);

  public:
	TFracAppView*	fFracAppView;
	TStaticText*	fCTimeLabelView;
	TStaticText*	fCTimeView;
	TStaticText*	fETimeLabelView;
	TStaticText*	fETimeView;
	TStaticText*	fMethodNameView;
	TStaticText*	fPercentView;
	TStaticText*	fPercentLabelView;
	TControl*		fSingleBarView;

	TFracAppWindow() {this->Initialize();};
	virtual void Initialize();
	/* Initialize our window object. Overridden to set our instance variables to NULL. */

	virtual void DoPostCreate(TDocument* itsDocument);
	/* Initialize a lot of the windows fields with references to its subviews. */

	virtual void AdjustInfoBar(ViewCoordinate width, ViewCoordinate height,
									  bool invalidate);
	/* Called by Show and Resize to make sure the items in the
	   infobar and the percentage complete label are all in the right places. */

	virtual void AnimateColors(void);
	// Called by the application at Idle time to do the palette animation.

	TStaticText* GetCTimeView(void)	{ return fCTimeView; }
	/* Return the Calculation Time text item. Called by TFracAppDocument when it
	   needs to update that part of the display. */

	TStaticText* GetETimeView(void)	{ return fETimeView; }
	/* Return the Elapsed Time text item. Called by TFracAppDocument when it needs
	   to update that part of the display. */

	TFracAppView* GetFracAppView(void)	{ return fFracAppView; }
	/* Return the main view that displays the fractal. Called by TFracAppDocument
	   when it needs to update that part of the display. */

	TStaticText* GetMethodView(void)	{ return fMethodNameView; }
	/* Return the text item that shows the algorithm being used. Called by
	   TFracAppDocument when it needs to initialize that part of the display. */

	TStaticText* GetPercentView(void)	{ return fPercentView; }
	/* Return the Percentage complete text item. Called by TFracAppDocument when
	   it needs to update that part of the display. */

	virtual void JumblePalette(void);
	/* Jumbles up the palette so that we get a nice banding effect. Called by
	   DoMenuCommand when the user selects the Jump Palette menu
	   item. */

	virtual void Open(void);
	/* Called by MacApp when it needs to open a window. We override it here so that
	   we can also call AdjustInfoBar. */

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);
	/* Called by MacApp when it needs to resize a window. We override it here so
	   that we can also call AdjustInfoBar. */

	virtual void RestorePalette(void);
	/* Recovers from the effects of Animating or Jumbling. Called by
	   DoMenuCommand when the user selects Everything back to
	   Normal, or turns off Jumble Palette. */

	virtual void Show(bool state, bool redraw);
	/* Called by MacApp when it shows or hides a window. Overridden so that we can
	   add or delete ourselves from the Windows menu. */

	virtual void SetTitle(const CStr255_AC& newTitle);
	/* Overridden so that we can force the Windows menu to be rebuilt if this
	   windows name changes. This can happen when a document is saved. */
};

//-------------------------------  View	-------------------------------

/* This object is used to display the elapsed time items, and the percentage
  complete item. I found that with a normal TStaticText item, I got too much
  flashing when I updated the strings. There was too much of a time gap between
  when the old string got erased, and the new one got drawn. Ive tried to
  minimize that by having MATextbox (which is what ultimately draws the string)
  erase the old text when it draws the new. There is still SOME flicker, but
  not as much as there used to be. */

DeclareClassDesc(TNoFlashStaticText);

class TNoFlashStaticText : public TStaticText 
{
	DeclareClass(TNoFlashStaticText);

	TNoFlashStaticText() {this->Initialize();};
	virtual void Initialize();
	// Overridden so that it passes the kEraseText parameter to MATextbox.

	virtual void SetText(const CStr255_AC& theText, bool redraw);
	// Overridden to inhibit erasing the old text before drawing the new.
};


/* This is the object that displays the fractal. Besides having its .Draw method
  overridden so that it can copybits the offscreen image, this View also keeps
  track of the current selection rectangle. Because of this, it also handles
  the menu items that would only be enabled when there is a selection. This
  includes Copy, New From Selection, and New Multi-Page. */

DeclareClassDesc(TFracAppView);

class TFracAppView : public TView 
{
	DeclareClass(TFracAppView);

  public:
	TFracAppDocument*	fFracAppDocument;	 /* Private - Used to tell the document
												to do whatever is necessary to
												prepare the offscreen for updating of
												the window. Also used to tell the
												document to stash the selected range
												into the global page record. Although
												the view has the selected range, the
												document contains the necessary
												scalinging information. */
	CRect_AC			 	fSelectionRect;		 /* Rectangle that is current selection.
												You can get/set this rectangle with
												Get/SetSelection. */

	virtual void Initialize(void);
	/* Inits the view object itself. Keeps track of the document, and initializes
	   the selection rect to be empty. */

	virtual void DoPostCreate(TDocument* itsDocument);
	/* Inits the view object itself. Keeps track of the document, and initializes
	   the selection rect to be empty. */

	virtual void DoHighlightSelection(HLState fromHL, HLState toHL);
	/* Called by MacApps updating routines to highlight the current selection
	   rectangle if there is one.  */

	virtual void DoMenuCommand(CommandNumber aCommandNumber);
	/* Called by MacApp when a menu item has been chosen. Handle the menu choices
	  for New Fractal out of the File Menu, and the Copy and SelectAll commands
	  from the Edit menu. */

	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event,
									   CPoint_AC hysteresis);
	/* Called by MacApp to handle the mouse events in the view. Needs to do the
	   selection of a new range for the next fractal to be calculated. */

	virtual void DoSetupMenus(void);
	/* Called by MacApp when the user clicks on the menu. Set up the New Fractal
	   menus choice in Fractal Menu, based on selection. */

	virtual void Draw(const CViewRect& area);
	/* Called by MacApp to draw the view seen in the window. Every nonblank view
	   MUST override this method. */

	//virtual void GetQDExtent(CRect_AC& qdExtent);
	
	/* Overridden so that we can redefine the extent of the view when pasting to
	   the scrap.  Paul Snively Memorial Brace ---> */

	void GetSelection(CRect_AC& selection)	{ selection = fSelectionRect; }
	/* Returns the current selection rectangle. Among other routines, this is called by
	   TFracAppDocument.StashSelectedRange to get the information it needs to to
	   help create a new document based on the current selection. */

	bool HasSelection(void)	{ return ! fSelectionRect.Empty(); }
	/* Returns true if there is a selection on the screen. Handy when we are
	   enabling menu items based on whether or not there is a selection. */

	virtual void SetSelection(const CRect_AC& theSelectionRect, bool redraw);
	// Sets and redraws the selection.
};								// TFracAppView

#endif
