#ifndef __UMEMORY__
#include <UMemory.h>
#endif

#ifndef __UOFFSCREEN__
#include "UOffscreen.h"
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

//--------------------------------------------------------------------------------------------------


// Initialize the offscreen world. This NULL out our instance variables and calls IObject.

COffscreen::COffscreen(void)
{
	fOldPort = NULL;
	fOldDevice = NULL;
}

//--------------------------------------------------------------------------------------------------


COffscreen::~COffscreen(void)
{
}

//--------------------------------------------------------------------------------------------------


/* Sets the state of a flag in the ctFlags field of our offscreen world. This is needed in at
  least one case, where we have to clear bit 14 before saving the color table to disk. This
  routine will either clear or set the bit. */

void COffscreen::ChangeCTFlag(short whichBit, bool whichWay)
{
	CGrafPtr offPort = GetOffPort();
	PixMapHandle portPixMap = offPort->portPixMap;
	CTabHandle pmTable = (**portPixMap).pmTable;
	short fred = (**pmTable).ctFlags;

	short mask = (1 << whichBit);
	if (whichWay == kClearFlag)
	{
		// BClr(fred, whichBit)
		fred &= ~mask;
	}
	else
	{
		// BSet(fred, whichBit);
		fred |= mask;
	}

	(**pmTable).ctFlags = fred;
}

//--------------------------------------------------------------------------------------------------


/* For 32BCQD support. Does nothing by default. Should be overridden if you are using the
  32-bit Color QuickDraw routines for offscreen support, and need to lock down your pixels
  before accessing them. */

void COffscreen::LockThePixels(void)
{
}

//--------------------------------------------------------------------------------------------------


/* Call this method to save off the current drawing world and swap in the offscreen drawing
  world. It calls SaveOldWord and SetUpOffWorld to do all the dirty work. Since these
  methods do nothing in TOffScreen, you must override them in your sub-class. */

void COffscreen::PreDraw(void)
{
	this->SaveOldWorld();
	this->SetupOffWorld();
}

//--------------------------------------------------------------------------------------------------


// Reverses the effects of PreDraw by restoring the previously save port and device.

void COffscreen::PostDraw(void)
{
	this->RestoreOldWorld();
}

//--------------------------------------------------------------------------------------------------


void COffscreen::RestoreOldWorld(void)
{
	::SetGDevice(fOldDevice);
	::SetPort(fOldPort);
	fOldDevice = NULL;							/* Clear these out so that if anyone calls
												 GetOldDevice or GetOldPort theyll get
												 NULL, meaning that we arent
												 offscreening right now. */
	fOldPort = NULL;
}

//--------------------------------------------------------------------------------------------------


void COffscreen::SaveOldWorld(void)
{
	fOldDevice = ::GetGDevice();
	::GetPort(&fOldPort);
}

//--------------------------------------------------------------------------------------------------


/* For 32BCQD support. Does nothing by default. Should be overridden if you are using the
  32-bit Color QuickDraw routines for offscreen support, and need to unlock your pixels
  after accessing them. */

void COffscreen::UnlockThePixels(void)
{
}

//--------------------------------------------------------------------------------------------------


/* Constructor - initializes data members to safe values. */

COldGrossOffscreen::COldGrossOffscreen(void)
{
	/* Set all these to NULL. If there is an error, then we need to free ourselves.
	  In that case, we dont want these variables holding garbage. If they do, that
	  garbage may look enough like the real thing, and the Dispose routines will
	  try to de-allocate memory that was never allocated to begin with */

	fOffPort = NULL;
	fOffDevice = NULL;
	fBigBuff = NULL;
}

//--------------------------------------------------------------------------------------------------


/* Create and initialize an offscreen drawing environment using the routines described in
  Inside Mac V. The offscreen world size is specified by  offBound, and uses the colors in
  itsColors. It makes a copy of the table. */

// These are the flags we pass to NewGDevice when creating our offscreen device.

const short kGDFlags		= (1 << screenActive) + (1 << noDriver) +
							  (1 << ramInit) + (1 << gdDevType);
const short kNoDriver		= 0;				// passed to NewGDevice.
const long kModeForNoDriver	= -1;				// passed to NewGDevice.

void COldGrossOffscreen::IOldGrossOffscreen(const CRect_AC& offBounds, CTabHandle itsColors)
{

	/* Prepare for failure. The memory used creating the view must be out of permanent
	  memory so call PermAllocation to make it so. Set up an error handler that will
	  set it back in case we cant get any of that memory. We also try to restore the
	  drawing world to something useful, so make sure we save a consistant world by
	  calling SaveOldWorld. */

	this->SaveOldWorld();
	Boolean oldPerm = PermAllocation(true);

	try
	{

		/* Lets set up the size of the rectangle we are using for the document. Dont
		  forget that the width must be an even number, so we round it up. */

		long docW = (((offBounds.right - offBounds.left) + 1) / 2) * 2;
		long docH = offBounds.bottom - offBounds.top;

		/* Now try to set up the offscreen pixMap (color).  If we fail we have to split,
		  and we might since we may not have 500K or more for the pixMap. Each document
		  on screen will have a full pixMap for it. Allocate a full buffer in 8 bit
		  depth, its size based on the bounding rectangle passed in to us. Also make it
		  into a color port so we can draw into it normally and use it as a source for
		  CopyBits. Requires 8 bits deep for the number of colors, and sets up a buffer
		  with that in mind, that is full docRect size with one byte per pixel as 8 bit
		  mode. This is width * height. 8 bits/byte. */

		Ptr aBuffPointer = ::NewPtr(docW * docH);
		::ThrowIfNULL_AC(aBuffPointer);						// couldnt get it we die.
		fBigBuff = aBuffPointer;

		/* OK, now we get wacko. We need to create our own gDevice, since we want to
		  have an offscreen device. This needs to be done so that we have full control
		  over the color table used, in order to save full 8 bit documents, even if we
		  arent in 8 bit mode when we save. So...	We will start by creating a
		  NewGDevice, that will allocate a temporary ITable, and PixMap with partial
		  colorTable; change the fields of the devices pixMap to our bitMap, with
		  right size, depth, and rowbytes; init the fields of that device, including
		  changing the color table to our color table created from our clut; set that
		  gDevice as the current one; then do the OpenCPort which will use the current
		  gDevice to make its PixMap and color table. When we go to draw or save the
		  data in the offscreen buffer, we need to set the current device so we use our
		  color table, making all the colors come out right. */

		/* Now we need to do the piece to make an offscreen gDevice that is not
		  connected to the screen. Allocate a new one, with stub pixMap. */

		GDHandle anOffDevice = ::NewGDevice(kNoDriver, kModeForNoDriver);
		::ThrowIfNULL_AC(anOffDevice);						// If we failed, error out.
		fOffDevice = anOffDevice;

		/* Now init all the fields we can in the gDevice Record, since it comes
		  uninitialized. */

		LockHandleHigh(Handle(fOffDevice));
		{
			GDPtr gd = *fOffDevice;

			gd->gdID = 0;							// no ID for search & complement procs
			gd->gdType = clutType;					// color table type fer sure.

			/* Get the color table for the offscreen gDevice.  This is a copy of the global
			  color table we created early on. */

			::DisposeCTable((**gd->gdPMap).pmTable);	// kill the stub that is there.
			(**gd->gdPMap).pmTable = NULL;				// it's gone
			CTabHandle pmTable = itsColors;				// make a safe copy
			::ThrowIfError_AC(::HandToHand((Handle *)&pmTable));// and stick it into this gDevice too.
			(**gd->gdPMap).pmTable = pmTable;			// make a copy of our global color table.

			/* Build a new iTable for this device, based on the new color table.  3 bit res
			  to save on memory since we dont need a big iTable for our stuff. */

			::MakeITable(pmTable, gd->gdITable, 3);
			::ThrowIfError_AC(::QDError()); 					// no memory, we can leave here.

			gd->gdResPref = 3;				// preferred resolution in table.
			gd->gdSearchProc = NULL;		// no search proc.
			gd->gdCompProc = NULL;			// no complement proc.

			// Set the gdFlags to be: color, ramInit, noDriver, screenActive

			gd->gdFlags = kGDFlags;

			{
				PixMapPtr pm = *gd->gdPMap;

				// Now set up the fields in the offscreen PixMap correctly.

				pm->baseAddr = fBigBuff;		// The base address is our buffer.
				pm->bounds = offBounds;	// bounding rectangle to our device.

				// One byte per pixel horizontally is rowBytes.  + 0x8000 to make it color port.

				pm->rowBytes = short(docW | 0x8000);
				pm->pixelSize = 8;
				pm->cmpCount = 1;
				pm->cmpSize = 8;
			}									// WITH gdPMap^^

			gd->gdRect = offBounds;		/* the bounding rectangle for gDevice, too. */
		}

		/* Now unlock the gDevice handle.  The system can use it unlocked as well as
		  locked so we try to help avoid fragmentation. */

		::HUnlock(Handle(fOffDevice));

		/* Yow, that was rough. Now we have a fully initialized gDevice offscreen with
		  its own color table. All color mapping should be done using that color table,
		  and the drawing we do to it should make the saved pictures save that color
		  table too. Set to our new device so we OpenCPort with all new parameters. This
		  is safe, because we saved the old device when we entered this routine. */

		::SetGDevice(fOffDevice);

		/* After all of that, we have a gDevice which is complete.  It has the color
		  table we want associated with it from the CLUT. It has the right
		  portBits.baseAddr, and the right size. It is complete, except that we cant
		  draw into it using normal calls. We thus need to make a port that we can use.
		  We have set the gDevice to be the one we just created, and when we OpenCPort
		  we will get a copy of the fields we just set up in our new gDevice. The port
		  is simply an interface into our gDevice for drawing. Allocate a port record
		  on the heap as a pointer (permanent memory usage). After it lives, though we
		  want to check if we have no more reserve, and if so we must bag this
		  document. */

		fOffPort = CGrafPtr(::NewPtr(sizeof(CGrafPort)));
		::ThrowIfNULL_AC(fOffPort);							// didnt get it, means we die.

		::OpenCPort(fOffPort);						// make a new port offscreen.
		FailNoReserve();							// Make reserve, die if we cant

		/* QuickDraw is most obnoxious about making a port that is bigger than the
		  screen, so we need to modify the visRgn to make it as big as our full page
		  document. It is OK to change this ports visRgn since we own it offscreen.
		  This is in case we are opening a document made on a different computer with a
		  bigger screen. Also, you may have seen in Inside Mac the notice that the
		  visRgn has no effect on images that arent displayed on the screen. Dont
		  believe it, because it DOES affect offscreen images. */

		::RectRgn(fOffPort->visRgn, offBounds);

		// Go whap on the other pieces of the port record to set it up to be offscreen.

		fOffPort->portRect = offBounds;

		/* OK, we have a nice new color port that is offscreen. It has a fancy color
		  table that came from the clut that will be used for the owning window.  It is
		  8 bits deep, has 256 colors in its color table and has a rect the size passed
		  in. It has no pieces that are related to the main gDevice, so we shouldnt
		  alter that by drawing in this port.  */

		/* Clear the error handler chain, we dont make any more dangerous requests. Set
		  the memory allocation to what we started with. */
	}
	catch(...)
	{
		/* This is the error handler for when we get errors while making a new GWorld,
		  typically like running out of memory.  Since we call the Free method for the
		  COffscreen, we dont have to clean things up ourselves here. Just set
		  allocation back to normal (for the error message itself), set the drawing
		  environment back to normal, call Free, and return. */

		oldPerm = PermAllocation(oldPerm);
		this->RestoreOldWorld();
		// delete this;

		throw;
	}
	oldPerm = PermAllocation(oldPerm);

	/* Swap over to our offscreen world so that we can clear it out. We swap over to
	  it with a simple SetupOffWorld call, which is OK because we saved the
	  original drawing environment at the beginning on this procedure with
	  SaveOldWorld. */

	this->SetupOffWorld();
	::EraseRect(offBounds);						// clear the bits.
	this->RestoreOldWorld();

} // IOldGrossOffscreen

//--------------------------------------------------------------------------------------------------


COldGrossOffscreen::~COldGrossOffscreen(void)
{
	// Kill the bits for the offscreen bitMap if they were allocated.

	fBigBuff = Ptr(DisposeIfPtr_AC(fBigBuff));

	/* Close the port: remove from portList, kill visRgn and clipRgn, kill the
	  penPixPat and fill PixPat and back PixPat, kill PixMap handle, kill grafVars
	  handle. */

	if (fOffPort)
	{
		::CloseCPort(fOffPort);
		fOffPort = CGrafPtr(DisposeIfPtr_AC(Ptr(fOffPort)));
	}

	/* DisposeGDevice does: kills the ITable, kills Cursor expanded data and mask if
	  nonzero, calls DisposePixMap if gdPMap is nonzero, then disposes the gDevice
	  handle itself. DisposePixMap kills the colorTable and the pixMap record. */

	if (fOffDevice)
	{
		::DisposeGDevice(fOffDevice);
		fOffDevice = NULL;
	}
}

//--------------------------------------------------------------------------------------------------


GDHandle COldGrossOffscreen::GetOffDevice(void)
{
	return fOffDevice;
}

//--------------------------------------------------------------------------------------------------


Ptr COldGrossOffscreen::GetOffPixBase(void)
{
	return fBigBuff;
}

//--------------------------------------------------------------------------------------------------


CGrafPtr COldGrossOffscreen::GetOffPort(void)
{
	return CGrafPtr(fOffPort);
}

//--------------------------------------------------------------------------------------------------


/* The default SetupOffWorld doesnt do anything because it doesnt know what form our
  offscreen management takes. So we have to override it here. We set up our offscreen world
  with simple SetGDevice and SetPort calls. */

void COldGrossOffscreen::SetupOffWorld(void)
{
	::SetGDevice(fOffDevice);
	::SetPort(GrafPtr(fOffPort));
}

//--------------------------------------------------------------------------------------------------


/* Constructor - initializes data members to safe values. */

CNewCoolOffscreen::CNewCoolOffscreen(void)
{
	/* Set this to NULL. If there is an error, then we need to free ourselves.
	  In that case, we dont want our variables holding garbage. If they do, that
	  garbage may look enough like the real thing, and the Dispose routines will
	  try to de-allocate memory that was never allocated to begin with */

	fOffWorld = NULL;
}

//--------------------------------------------------------------------------------------------------


/* Create and initialize an offscreen drawing environment using the 32-bit Color QuickDraw
  routines. The offscreen world size is specified by offBound, and uses the colors in
  itsColors. It makes a copy of the table. */

void CNewCoolOffscreen::INewCoolOffscreen(const CRect_AC& offBounds, CTabHandle itsColors)
{
	/* Prepare for failure. The memory used creating the view must be out of permanent
	  memory so call PermAllocation to make it so. Set up an error handler that will
	  set it back in case we cant get any of that memory. We also try to restore the
	  drawing world to something useful, so make sure we save a consistant world by
	  calling SaveOldWorld. */

	this->SaveOldWorld();
	Boolean oldPerm = PermAllocation(true);

	try
	{
		/* Time to make the big, bad NewGWorld call. This does everything:

		   anOffWorld: a VAR parameter. NewGWorld returns a reference to our
			GWorld in this.
		   k8Bits: desired depth of our offscreen GWorld
		   offBounds: a CRect_AC used for the bounds, the portRect, the gdRect
		   itsColors: a CTable used for color indexing
		   NULL: a GDHandle if we wanted to provid one of our own
		   []: there are two flags that we could have passed here. By passing the
			empty set, we are saying that we dont want the PixMap purgable, and
			that we want NewGWorld to make a GDevice for us. Since were basically
			lazy (all programmers are), thats fine with us.

		  NewGWorld returns a QDErr if it cant cut the mustard. */

		const short kBitDepth			= 8;

		GWorldPtr anOffWorld;
		::ThrowIfError_AC(::NewGWorld(&anOffWorld, kBitDepth, offBounds, itsColors, NULL, 0));
		fOffWorld = anOffWorld;

		GDHandle auxDev = ::GetGWorldDevice(fOffWorld);
		::MakeITable(itsColors, (**auxDev).gdITable, 3); // 3 bits to save room
		::ThrowIfError_AC(::QDError()); 						// sad, he says
		oldPerm = PermAllocation(oldPerm);
	}
	catch(...)
	{
		oldPerm = PermAllocation(oldPerm);
		this->RestoreOldWorld();
		// delete this;
		throw;
	}

	/* Swap over to our offscreen world so that we can clear it out. We swap over to
	  it with a simple SetupOffWorld call, which is OK because we saved the
	  original drawing environment at the beginning on this procedure with
	  SaveOldWorld. */

	this->SetupOffWorld();
	::EraseRect(&(fOffWorld->portRect)); 			// clear the bits.
	this->UnlockThePixels();
	this->RestoreOldWorld();
}

//--------------------------------------------------------------------------------------------------


CNewCoolOffscreen::~CNewCoolOffscreen(void)
{
	if (fOffWorld)
	{
		::DisposeGWorld(fOffWorld);
		fOffWorld = NULL;
	}
}

//--------------------------------------------------------------------------------------------------


GDHandle CNewCoolOffscreen::GetOffDevice(void)
{
	return ::GetGWorldDevice(fOffWorld);
}

//--------------------------------------------------------------------------------------------------


Ptr CNewCoolOffscreen::GetOffPixBase(void)
{
	return ::GetPixBaseAddr(fOffWorld->portPixMap);
}

//--------------------------------------------------------------------------------------------------


CGrafPtr CNewCoolOffscreen::GetOffPort(void)
{
	return CGrafPtr(fOffWorld);
}

//--------------------------------------------------------------------------------------------------


void CNewCoolOffscreen::PostDraw(void)
{
	this->UnlockThePixels();

	COffscreen::PostDraw();
}

//--------------------------------------------------------------------------------------------------


void CNewCoolOffscreen::SetupOffWorld(void)
{
	this->LockThePixels();
	::SetGWorld(fOffWorld, NULL);
}

//--------------------------------------------------------------------------------------------------


/* Call 32-bit Color QuickDraws LockPixels routine to lock down the offscreen data
  structures. We ignore the result, because the only meaningful result (false), means that
  LockPixels tried to re-allocate some purged structures, and failed. We dont allow those
  structures to be purged (we specified that when we called NewGWorld), so it will never
  return false for us. */

void CNewCoolOffscreen::LockThePixels(void)
{
	if (fOffWorld)
		::LockPixels(fOffWorld->portPixMap);
}

//--------------------------------------------------------------------------------------------------


/* Reverse the effect of LockThePixels. When we are drawing to the offscreen PixMap, we need
  its bit locked down so they dont move around when we try to access them. But when our
  attention is elsewhere and we arent drawing to the offscreen world, wed like them to
  float around in the heap, so as not to fragment it. This is OK with 32-bit Color QuickDraw
  it the data is unlocked when its not looking. */

void CNewCoolOffscreen::UnlockThePixels(void)
{
	if (fOffWorld)
		::UnlockPixels(fOffWorld->portPixMap);
}
