//	UPictDoc.cp
#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __UFILE__
#include <UFile.h>
#endif

#ifndef __UPICTDOC__
#include "UPictDoc.h"
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __UFRACGLOBALS__
#include "UFracGlobals.h"
#endif

//-------------------------------------------------------------------------------------------

const long kPICTHeaderSize 	= 512;					/* 512 bytes off the file are used for our
													 info and print info. */

/* The next globals are used for the QuickDraw bottlenecks when reading or writing a picture
  to disk. These are needed, since the bottlenecks cannot be owned procedures, and we cant
  just use local variables. */

long		gPictSize;					// number of bytes used for saving a PICT.
OSErr 		gPictError;					// do some error handling in bottleneck.
TFile*		gPictFile;					// Need the refnum of the open file too.
PicHandle	gPictHandle;				// for reading/writing a picture.

//-------------------------------------------------------------------------------------------
//---------------------------------TPICTDocument Methods-------------------------------------
//-------------------------------------------------------------------------------------------


/* Init for the TPICTDocument. We dont have any variables to initialize, so just set up the
  TDocument object itself. */

#undef Inherited
#define Inherited TFileBasedDocument
DefineClass(TPICTDocument, TFileBasedDocument);

TPICTDocument::TPICTDocument(TFile* itsFile, OSType itsScrapType) :
	TFileBasedDocument(itsFile, itsScrapType)
{
/*	if (itsFile && (itsFile->fUsesDataFork != kUsesDataFork))
	{
		#if qDebug
			ProgramBreak_AC((StringPtr)"\pTPICTDocument must use data fork!");
		#endif
	} */
} // TPICTDocument::IPICTDocument

//-------------------------------------------------------------------------------------------


/* Abstract method here. Because our TPICTDocument supports writing to the disk, we need some
  way to generate the drawing commands. However, the generic TPICTDocument knows nothing
  about how the data is stored or should be imaged. Therefore, we create the name of a
  method that should be called, and require that this method be overridden in order for it
  to have any effect. If this method is not overridden, the programmer gets a message in the
  debugger saying so. */

void TPICTDocument::DoDrawPicture(void)
{
#if qDebug
	ProgramBreak_AC((StringPtr)"\pTPICTDocument::DoDrawPicture: Gotta override me!");
#endif
} // TPICTDocument::DoDrawPicture

//-------------------------------------------------------------------------------------------


/* This routine will size the current image as it goes to the disk. It wont actually save
  any data or anything, but will merely watch the bytes go by keeping track of how many go
  by. The size is used by DoNeedDiskSpace. */

static pascal void PictSizer(void* /*dataPtr*/, short byteCount)
{
	gPictSize += byteCount;

} // PictSizer

//-------------------------------------------------------------------------------------------


/* Routine to find out how much disk space will be required to save the data. This does not
  call the Inherited DoNeedDiskSpace since we dont support printing info here.  The routine
  will replace the PutPicProc of the port with our PictSizer routine. When the picture is
  created here, no bytes will actually be allocated or saved, we will just watch it go by
  and save off the size in the global variable. That value is returned as the expected
  document size, along with the size of the PICT user data area of 512 bytes. */

void TPICTDocument::DoNeedDiskSpace(TFile* /*itsFile*/, long& dataForkBytes,
										   long& /*rsrcForkBytes*/)
{
	CQDProcs			newProcs;
	CQDProcsPtr			oldProcs;

	/* Save off the old GrafProcs, and install a new set. This new set includes a
	  pointer to PictSizer, which will be used to measure the size of the final
	  PICT. When we are all done, well re-install the normal set of GrafProcs. */

	QDPutPicUPP putPictUPP = NewQDPutPicProc(PictSizer);
	this->SetRWGrafProcs(oldProcs, newProcs, NULL, putPictUPP);

	// Init the size of the pict we are going to save.  Start with picture header.

	gPictSize = sizeof(Picture);

	/* Now go ahead and open the picture and build it in RAM. We would have done
	  this by slices before, but the newer systems have a patch for playing back
	  pictures that minimize the RAM hit, so we dont have to worry about the full
	  screen CopyBits here. */

	gPictHandle = ::OpenPicture(&(qd.thePort->portRect));
	this->DoDrawPicture();
	::ClosePicture();							// the picture is created, and packed.

	// Done saving the size of the picture itself.  Now set the GrafProcs back to normal.

	CGrafPtr(qd.thePort)->grafProcs = oldProcs;
	DisposeRoutineDescriptor(putPictUPP);

	// Dispose the pict handle, we didnt actually make anything there.

	::KillPicture(gPictHandle);
	gPictHandle = NULL;

	/* The picture has been sized.  Now add that in to the total size the file will use on
	  disk, include the header for the file, plus the number of bytes in actual PICT. */

	dataForkBytes += gPictSize + kPICTHeaderSize;

} // TPICTDocument::DoNeedDiskSpace

//-------------------------------------------------------------------------------------------


/* Bottleneck routine to read the picture from the disk. This will read the data required,
  and pass it along to the unpacker. This makes it possible to avoid using any RAM for the
  actual reading part, as it is being played back into the offscreen device. Error handling
  is somewhat tricky, since we need to force the picture to finish, and there isnt a really
  good way to do this. The desired attempt here is to pass back a picture is finished
  opcode (0x00FF) so we can get back to our code to handle the error. This is better than no
  error recovery, but is not guaranteed to work. */

static pascal void PictReader(void* dataPtr, short byteCount)
{
	if (gPictError == noErr)
	{
		long longHunk = byteCount;
		gPictError = gPictFile->ReadData(dataPtr, longHunk);
	}

	// Handle the error situation by passing back 0x00FF as the data.

	if (gPictError != noErr)
	{
		Ptr stomper = Ptr(dataPtr);
		for (short i = 1; i <= byteCount; ++i)
		{
			*stomper++ = Odd_AC(i) ? 0x00 : 0xFF;
		}
	}
} // PictReader

//-------------------------------------------------------------------------------------------


/* Routine to read the data from the data fork of the file into our document so it can be
  displayed.  The quickdraw bottleneck will be replaced with the PictReader routine, making
  it read the data from the disk as the picture requests more data.  This obviates the need
  for an extra handle that is used to play back the picture. This is done since that extra
  handle can be on the order of 100K, memory we may not have available. */

void TPICTDocument::DoRead(TFile* aFile, bool /*forPrinting*/)
{
	CQDProcs	newProcs;
	CQDProcsPtr	oldProcs;

	// Make sure the file position is right at the start of the picture in the file.

	short filePos = short(kPICTHeaderSize);
	::ThrowIfError_AC(aFile->SetDataMark(filePos, fsFromStart));

	/* Allocate a small handle that will be used as the Pict handle for drawing from
	  the disk.  This is just the picture header. */

	gPictHandle = PicHandle(::NewHandle(sizeof(Picture)));
	::ThrowIfNULL_AC(gPictHandle);

	// If the read of the picture header fails, we want to dispose the handle allocated.
	gPictFile = aFile;
	gPictError = noErr;


	try
	{
		// Now fill in the picture header itself, using the data from the disk.

		long recsize = sizeof(Picture);
		gPictError = aFile->ReadData(*gPictHandle, recsize);
		::ThrowIfError_AC(gPictError);

	}
	catch (...)
	{
		if (gPictHandle)
			::KillPicture(gPictHandle);
		gPictHandle = NULL;
		throw;
	}

	/* That is the only call we cant recover from immediately, the rest of the
	  routine is not easy to recover from, so we wont go through DeathRead. */

	/* The file position is right at the beginning of the picture data, so we can just
	  install the bottleneck and call DrawPicture to fill our offscreen gDevice
	  with the data that was saved.  Set to that port and gDevice for playback. */

	/* Save the pointer to the current CGrafProcs, and
	  the install our our Pict proc in the grafProcs. */

	QDGetPicUPP getPicUPP = NewQDGetPicProc(PictReader);
	this->SetRWGrafProcs(oldProcs, newProcs, getPicUPP, NULL);

	/* We can now draw the picture that will be read out of the file into this port
	  in order to init the port for later use in updating the window.  We are already
	  set to draw in the offscreen port.  Do the DrawPicture to have PictReader read
	  the data out of the file while it is being played into the offscreen Port. */

	::DrawPicture(gPictHandle, &((**gPictHandle)).picFrame);

	// Done reading the data of the picture itself.  Now set the GrafProcs back to normal.

	CGrafPtr(qd.thePort)->grafProcs = oldProcs;
	DisposeRoutineDescriptor(getPicUPP);

	// Bag the handle we made for playing back the picture.

	::KillPicture(gPictHandle);
	gPictHandle = NULL;

	// If we had an error while reading the data, we must error out.

	::ThrowIfError_AC(gPictError);

} // TPICTDocument::DoRead

//-------------------------------------------------------------------------------------------


/* This routine will save the current image as it is created.  As the data requests go by
  that data will be written to the file. The data is being created by the OpenPicture/
  CopyBits in DoWrite. This is the bottleneck for that operation. Any errors found while
  doing this will make us skip any further requests to write data to the disk. No memory is
  allocated. Communication with DoWrite is done through globals, since bottlenecks must be
  at the main level. The bottleneck must also keep track of how many bytes are written, so
  that the header on the picture can be fixed up to be correct. This must be done to avoid
  creating bogus pictures.	The picSize field of the handle must be updated continuously so
  that when the picture is done, the ClosePicture can create a valid picture. The check for
  the NULL handle is to handle the problem of when the OpenPicture is called.  The proc gets
  called before the handle is valid. Be very careful of these bottleneck things; it is easy
  to run into problems that are very hard to figure out. QuickDraw has no facilities to give
  you info when things go wrong so it makes it a bit tougher. */

static pascal void PictWriter(void* dataPtr, short byteCount)
{
	if (gPictError == noErr)
	{
		long longHunk = byteCount;
		gPictError = gPictFile->WriteData(dataPtr, longHunk);
		gPictSize += longHunk;
		if (gPictHandle)
			(**gPictHandle).picSize = LoWord(gPictSize);
	}										// if no error so far
} // PictWriter

//-------------------------------------------------------------------------------------------


/* Write the data calculated into the document to the file. This will make it a real PICT
  file. The file will be saved using QuickDraw Bottlenecks for the PutPicProc. As the data
  requests go by, they will be written to the file, using the PictWriter routine. */

void TPICTDocument::DoWrite(TFile* aFile, bool /*makingCopy*/)
{
	CQDProcs	newProcs;
	CQDProcsPtr	oldProcs = NULL;

	/* We need to write the picture data itself out to the file, after we set the
	  mark to be after the entire header.  Make sure the file is that big before we
	  do it. Included in this set is the header of the picture itself, the 10 bytes
	  that include the rectangle. Those bytes will be updated after the picture is
	  written. */

	short filePos = short(kPICTHeaderSize + sizeof(Picture));
	::ThrowIfError_AC(aFile->SetDataLength(filePos));
	::ThrowIfError_AC(aFile->SetDataMark(filePos, fsFromStart));

	// If the write of the picture header fails, we want to dispose the handle allocated.
	gPictFile = aFile;
	gPictSize = sizeof(Picture);
	gPictError = noErr;
	gPictHandle = NULL;
	QDPutPicUPP putPictUPP = NULL;

	try
	{

		/* The file is all set up to go.  We now want to replace the QuickDraw
		  bottleneck and create the actual Picture data. */

		putPictUPP = NewQDPutPicProc(PictWriter);
		this->SetRWGrafProcs(oldProcs, newProcs, NULL, putPictUPP);

		/* Tell PictWriter what file to write to, and start the pic size including the
		  picture header.  Start all the pieces off the right way. */

		/* Actually open the picture and do the CopyBits in order to process the picture.
		  The data will be written by PictWriter as it is called by QuickDraw. */

		gPictHandle = ::OpenPicture(&(qd.thePort->portRect));
		::ClipRect(&(qd.thePort->portRect));		// Make it a happier picture.

		// copy all of the image to itself, in an open picture it saves the bits.

		this->DoDrawPicture();
		::ClosePicture();							// the picture is created, and packed.

		/* Now check for errors during the write operation. The gPictError field will be
		  nonzero if we failed during the operation. */

		::ThrowIfError_AC(gPictError);

		// Move back to front of file and write the valid picture info to file.

		filePos = short(kPICTHeaderSize);
		::ThrowIfError_AC(aFile->SetDataMark(filePos,fsFromStart));
		long recsize = sizeof(Picture);
		::ThrowIfError_AC(aFile->WriteData(*gPictHandle, recsize));

		/* Done saving the data of the picture itself.  Now set the GrafProcs back to
		  normal. */

		CGrafPtr(qd.thePort)->grafProcs = oldProcs;
		DisposeRoutineDescriptor(putPictUPP);

		// Dispose the pict handle, we didnt actually make anything there.

		::KillPicture(gPictHandle);
		gPictHandle = NULL;

	}
	catch(...)
	{
		if (gPictHandle)
			::KillPicture(gPictHandle);
		gPictHandle = NULL;

		CGrafPtr(qd.thePort)->grafProcs = oldProcs;
		if (putPictUPP)
			DisposeRoutineDescriptor(putPictUPP);

		throw;
	}

} // TPICTDocument::DoWrite

//-------------------------------------------------------------------------------------------


/* Common routine to set the current ports grafprocs for reading and writing. It gets called
  by our DoRead and DoWrite overrides to that we can read and write PICTs without causing a
  big memory hit. */

void TPICTDocument::SetRWGrafProcs(CQDProcsPtr& oldProcs, CQDProcs& newProcs,
										  QDGetPicUPP readProc, QDPutPicUPP writeProc)
{
	QDGetPicUPP		picProc;
	
	oldProcs = CGrafPtr(qd.thePort)->grafProcs;
	::SetStdCProcs(&newProcs);
	CGrafPtr(qd.thePort)->grafProcs = &newProcs;

	if (readProc) {
		picProc = DisposeIfRoutineDescriptor((UniversalProcPtr)newProcs.getPicProc);
		newProcs.getPicProc = readProc;
	}

	if (writeProc) {
		picProc = DisposeIfRoutineDescriptor((UniversalProcPtr)newProcs.putPicProc);
		newProcs.putPicProc = writeProc;
	}
}
