/*-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker
PowerPC version by Jeroen Schalk

Copyright  1988-1994 Apple Computer, Inc.
All rights reserved.

--------------------------------------------------------------------------------------------

TPICTDocument - Sort of a generic PICT handling document. Its supposed to be
	all-purpose, but I admit that it was created with FracApp in mind, so you
	might not be able to just drop it into your application without some changes.
	Anyway, it knows how to read and write a PICT file. All you have to do is
	sub-class it and override the DoDrawPicture method to do the actual imaging.
	You might also want to override DoRead and DoWrite to deal with any PICT
	header information you may have.

--------------------------------------------------------------------------------------------*/

#ifndef __UPICTDOC__
#define __UPICTDOC__

#ifndef __UFILEBASEDDOCUMENT__
#include <UFileBasedDocument.h>
#endif

//-------------------------------  Document	-------------------------------

DeclareClassDesc(TPICTDocument);

class TPICTDocument : public TFileBasedDocument 
{
	DeclareClass(TPICTDocument);

  public:
	TPICTDocument(TFile* itsFile = NULL, OSType itsScrapType = kNoIdentifier);
	/* Init routine for the document. For TPICTDocument, it does nothing except call
	   IDocument. */

	virtual void DoDrawPicture(void);
	/* DoNeedDiskSpace and DoWrite need some way to image the picture. A
	   TPICTDocument doesnt have any idea how the data is being kept, so it doesnt
	   know how to do the imaging. Therefore, we have this DoDrawPicture method that
	   should be overridden by subclasses. It is the subclasses that know how the
	   data is stored, and so they can image it. By default, this routine does
	   nothing but complain. */

	virtual void DoNeedDiskSpace(TFile* itsFile, long& dataForkBytes,
								 long& rsrcForkBytes);
	/* Called by MacApp when preparing to write out a disk file. Finds out the
	   entire size of the object to be saved into the file so that the correct
	   amount of disk space can be used. For a TPICTDocument, this is the size of
	   the PICT, plus 512 bytes for the header. */

	virtual void DoRead(TFile* aFile, bool forPrinting);
	/* Called by MacApp to read the data from the data fork of the file. Stows it
	   into the documents bitMap. This bitmap should be setup by the subclass in
	   its DoRead method before calling this INHERITED method. In other words, this
	   just draws into the current port and device. */

	virtual void DoWrite(TFile* aFile, bool makingCopy);
	/* Called by MacApp to write out the documents data. Converts the data in the
	   documents port into a PICT, then writes that block out to the file, making
	   it into a standard PICT file. It does this with the help of its humbler
 	   servant, DoDrawPicture. */

	virtual void SetRWGrafProcs(CQDProcsPtr& oldProcs, CQDProcs& newProcs,
								QDGetPicUPP readProc, QDPutPicUPP writeProc);
	/* Common routine to set the current ports grafprocs for reading and writing.
	   It gets called by our DoRead and DoWriter overrides to that we can read
	   and write PICTs without causing a big memory hit. */
};

#endif
