//	URectStack
#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __URECTSTACK__
#include "URectStack.h"
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CWhileSynchronized_AC__
#include "CWhileSynchronized_AC.h"
#endif

//-------------------------------------------------------------------------------------------
//----------------------------------CRectStack Methods---------------------------------------
//-------------------------------------------------------------------------------------------


// Initialize a new array with numberOfRects rects.

CRectStack::CRectStack(long numberOfRects)
{
	fSize = 0;
	fAllocatedSize = 0;
#if qTuneRectStack
	fHighWaterMark = 0;
#endif
	fData = NULL;
	fData = TH_new CRect_AC[numberOfRects];
	fAllocatedSize = numberOfRects;
}

//-------------------------------------------------------------------------------------------


// Free the array.

CRectStack::~CRectStack(void)
{
	if (fData)
	{
		delete[] fData;
		fData = NULL;
		fSize = 0;
		fAllocatedSize = 0;
	}
}

//------------------------------------------------------------------------------------
// Array manipulation primitives
//-------------------------------------------------------------------------------------------


// Takes the CRect_AC you pass it, and pushes it on the stack.

void CRectStack::PushRect(const CRect_AC& theRect)
{
#if qUseThreads
	CWhileSynchronized_AC entry(&fLock);
#endif
	register long newSize = fSize + 1;
#if qTuneRectStack
	if (fHighWaterMark < newSize)
		fHighWaterMark = newSize;
#endif
	if (newSize >= fAllocatedSize)
#if qWantsMP
		if (::HasMP_AC()) {
			_MPDebugStr((StringPtr)"\pCRectStack overflow!");
		}
		else {
			ProgramBreak_AC((StringPtr)"\pCRectStack overflow!");
		}
#else
		ProgramBreak_AC((StringPtr)"\pCRectStack overflow!");
#endif
	else
		fData[fSize++] = theRect;
}

//-------------------------------------------------------------------------------------------


// Returns the first CRect_AC on the stack. That CRect_AC is then removed from
// the stack.

bool CRectStack::PopRect(CRect_AC& theRect)
{
#if qUseThreads
	CWhileSynchronized_AC entry(&fLock);
#endif
	bool result = false;
	if (fSize > 0) {
		theRect = fData[--fSize];
		result = true;
	}
	return result;
}

//-------------------------------------------------------------------------------------------
// Input/Output functions
//-------------------------------------------------------------------------------------------


void CRectStack::DoRead(short aRefNum, bool /*forPrinting*/)
{
	DeleteAll();

	long newSize;
	long recSize = sizeof(fSize);
	::ThrowIfError_AC(::FSRead(aRefNum, &recSize, &newSize));
	if (fSize > 0)
	{
#if qTuneRectStack
		if (fHighWaterMark < newSize)
			fHighWaterMark = newSize;
#endif
		if (newSize >= fAllocatedSize)
#if qWantsMP
			if (::HasMP_AC()) {
				_MPDebugStr((StringPtr)"\pCRectStack overflow!");
			}
			else {
				ProgramBreak_AC((StringPtr)"\pCRectStack overflow!");
			}
#else
			ProgramBreak_AC((StringPtr)"\pCRectStack overflow!");
#endif
		else
		{
			recSize = sizeof(CRect_AC) * newSize;
			::ThrowIfError_AC(::FSRead(aRefNum, &recSize, fData));
			fSize = newSize;
		}
	}
}

//-------------------------------------------------------------------------------------------


void CRectStack::DoWrite(short aRefNum, bool /*makingCopy*/)
{
	long recSize = sizeof(fSize);
	::ThrowIfError_AC(::FSWrite(aRefNum, &recSize, &fSize));
	if (fSize > 0)
	{
		recSize = sizeof(CRect_AC) * fSize;
		::ThrowIfError_AC(::FSWrite(aRefNum, &recSize, fData));
	}
}

//-------------------------------------------------------------------------------------------
//----------------------------------CRectQueue Methods---------------------------------------
//-------------------------------------------------------------------------------------------


// Initialize a new array with numberOfRects rects.

CRectQueue::CRectQueue(long numberOfRects)
#if qUseThreads
	: fNotFull(&fLock),
	  fFilled(false)
#endif
{
	fSize = fHead = fTail = 0;
	fAllocatedSize = 0;
#if qTuneRectStack
	fHighWaterMark = 0;
#endif
	fData = NULL;
	fData = TH_new CRect_AC[numberOfRects];
	fAllocatedSize = numberOfRects;
}

//-------------------------------------------------------------------------------------------


// Free the array.

CRectQueue::~CRectQueue(void)
{
	if (fData)
	{
		delete fData;
		fData = NULL;
		fSize = fHead = fTail = 0;
		fAllocatedSize = 0;
	}
}

//------------------------------------------------------------------------------------
// Array manipulation primitives
//-------------------------------------------------------------------------------------------


// Takes the CRect_AC you pass it, and pushes it on the queue.

void CRectQueue::PushRect(const CRect_AC& theRect)
{
	register long qLen = fSize + 1;
#if qTuneRectStack
	if (fHighWaterMark < qLen)
		fHighWaterMark = qLen;
#endif
	if (qLen > fAllocatedSize)
#if qWantsMP
		if (::HasMP_AC()) {
			_MPDebugStr((StringPtr)"\pCRectQueue overflow!");
		}
		else {
			ProgramBreak_AC((StringPtr)"\pCRectQueue overflow!");
		}
#else
		ProgramBreak_AC((StringPtr)"\pCRectQueue overflow!");
#endif
	else
	{
		fSize = qLen;
		fData[fTail] = theRect;
		if (++fTail >= fAllocatedSize)
			fTail = 0;
	}
}

//-------------------------------------------------------------------------------------------


// Wait if the queue is full, then push the CRect_AC into it.

void CRectQueue::PushRectSync(const CRect_AC& theRect)
{
#if qUseThreads
	CWhileSynchronized_AC entry(&fLock);
	while (IsFull()) fNotFull.Wait();
	this->PushRect(theRect);
#else
	this->PushRect(theRect);
#endif
}

//-------------------------------------------------------------------------------------------


// Takes the CRect_AC you pass it, and pushes it on the queue.
// If possible, add the CRect_AC to the last one in the queue.

void CRectQueue::AddRect(const CRect_AC& theRect)
{
	bool skip = false;

	long qLen = fSize;
	if (qLen > 0)
	{
		// Try to consolidate with the last CRect_AC
		CRect_AC* last = (fTail > 0) ? &fData[fTail - 1] : &fData[fAllocatedSize - 1];
		if ((theRect.left == last->left) && (theRect.right == last->right))
		{
			if (theRect.top == last->bottom)
			{
				last->bottom = theRect.bottom;
				skip = true;
			}
			else if (last->top == theRect.bottom)
			{
				last->top = theRect.top;
				skip = true;
			}
		}
		else if ((theRect.top == last->top) && (theRect.bottom == last->bottom))
		{
			if (theRect.left == last->right)
			{
				last->right = theRect.right;
				skip = true;
			}
			else if (last->left == theRect.right)
			{
				last->left = theRect.left;
				skip = true;
			}
		}
	}

	if (!skip)
	{
		// Push a new CRect_AC
		++qLen;
#if qTuneRectStack
		if (fHighWaterMark < qLen)
			fHighWaterMark = qLen;
#endif
		if (qLen > fAllocatedSize)
#if qWantsMP
			if (::HasMP_AC()) {
				_MPDebugStr((StringPtr)"\pCRectQueue overflow!");
			}
			else {
				ProgramBreak_AC((StringPtr)"\pCRectQueue overflow!");
			}
#else
			ProgramBreak_AC((StringPtr)"\pCRectQueue overflow!");
#endif
		else
		{
			fSize = qLen;
			fData[fTail] = theRect;
			if (++fTail >= fAllocatedSize)
				fTail = 0;
		}
	}
}

//-------------------------------------------------------------------------------------------


// Wait if the queue is full, then add the CRect_AC into it.

void CRectQueue::AddRectSync(const CRect_AC& theRect)
{
#if qUseThreads
	CWhileSynchronized_AC entry(&fLock);
	while (IsFull()) fNotFull.Wait();
	this->AddRect(theRect);
#else
	this->AddRect(theRect);
#endif
}

//-------------------------------------------------------------------------------------------


// Returns the first CRect_AC on the queue. That CRect_AC is then removed from the queue.

void CRectQueue::PopRect(CRect_AC& theRect)
{
#if qUseThreads
	CWhileSynchronized_AC entry(&fLock);
	if (!fFilled) fFilled = IsFull(); // Set when full
#endif
	if (fSize <= 0)
#if qWantsMP
		if (::HasMP_AC()) {
			_MPDebugStr((StringPtr)"\pCRectQueue underflow!");
		}
		else {
			ProgramBreak_AC((StringPtr)"\pCRectQueue underflow!");
		}
#else
		ProgramBreak_AC((StringPtr)"\pCRectQueue underflow!");
#endif
	else
	{
		--fSize;
		theRect = fData[fHead];
		if (++fHead >= fAllocatedSize)
			fHead = 0;
	}
#if qUseThreads
	if (fFilled && IsEmpty()) fNotFull.Broadcast();  // Clear when empty.
#endif
}
