//--------------------------------------------------------------------------------------------------
// SimpleThreads.r
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved. 
//
//	Resource File for the SimpleThreads Sample Program. 
//--------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------
// INCLUDE FILES 
//--------------------------------------------------------------------------------------------------

// Standard Includes 

#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif

#ifndef __BalloonTypes__
#include "BalloonTypes.r"
#endif

#ifndef THINK_Rez
#if qDebug | qPerform
include "Debug.rsrc" not 'ckid';
#endif

include "Dialog.rsrc" not 'ckid';
include "MacApp.rsrc" not 'ckid';

// Includes for Building Blocks

include "Printing.rsrc" not 'ckid';

// Include Code Resources

#ifndef MW_Rez 
#if !qNoCode
include $$Shell("ObjApp")$$Shell("XAppName") 'CODE';
#if qModelCFM
include $$Shell("ObjApp")$$Shell("XAppName") 'cfrg';
include $$Shell("ObjApp")$$Shell("XAppName") 'rseg';
#endif
#endif
#endif
#endif //!THINK_Rez

//--------------------------------------------------------------------------------------------------
// CONSTANTS 
//--------------------------------------------------------------------------------------------------

// Finder Information
// ------------------

#if !defined(THINK_Rez) && !defined(MW_Rez)
#define kApplicationName	$$Shell("XAppName")
#else
#define kApplicationName	TARGET_APPLICATION_NAME
#endif // !THINK_Rez

#define kSignature			'SS01'			// Application signature 
#define kFileType			'SF01'			// Data file type 
#define kStationery			'sF01'			// Stationery file type 

// Bundle Constants
// ------------------

#define kBundleID						128
#define kApplicationID					128
#define kDocumentID						129
#define kStationeryID					130

// Constants for Menu IDs
// ----------------------

#define mSimpleThreadsMenu	4

#define mRunningThread		1000
#define mStoppedThread		1001

// Command Numbers For Menu Items
// ------------------------------

#define cMPThreads				402
#define	cThread1				403
#define	cThread2				404
#define	cThread3				405
#define	cThread4				406
#define	cThread5				407
#define	cThread6				408
#define cStartThread			409
#define cStopThread				410

// STR# resources
// --------------

#define kSimpleThreadsMenuHelp				1000
//#define kSimpleThreadsWindowTitles			1001

// Text styles
// --------------

//#define kSystemTextStyle				1000

// 'View' resources
// --------------

//#define kSimpleThreadsWindowID				kDefaultWindowID

// Memory Management Resources
// ---------------------------

#define kSimpleThreadsSeg	1000
#define kSimpleThreadsMem	1000
#define kSimpleThreadsRes	1000

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR MENUS 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Menu Bars 
//--------------------------------------------------------------------------------------------------

resource 'MBAR' (kMBarDisplayed,
	purgeable) {
	{ mApple; mFile; mEdit; mSimpleThreadsMenu }
};

resource 'MBAR' (kMBarHierarchical,
	purgeable) {
	{ mRunningThread, mStoppedThread }
};

//--------------------------------------------------------------------------------------------------
// Menus 
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc" 'CMNU' (mApple);	// Grab the default Apple menus
include "Defaults.rsrc" 'CMNU' (mFile);		// Grab the default File menus
include "Defaults.rsrc" 'CMNU' (mEdit);		// Grab the default Edit menu

resource 'CMNU' (mSimpleThreadsMenu,
#if qNames
"SimpleThreads Menu",
#endif
nonpurgeable) {
	mSimpleThreadsMenu,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Run",
	 {
	 /* TBD - add runtime switch for using MP threads
		"Use MP Threads", noIcon, "1", noMark, plain, cMPThreads;
		"-", noIcon, noKey, noMark, plain, nocommand;
	*/
		"Thread 1", noIcon, "1", noMark, plain, cThread1;
		"Thread 2", noIcon, "2", noMark, plain, cThread2;
		"Thread 3", noIcon, "3", noMark, plain, cThread3;
		"Thread 4", noIcon, "4", noMark, plain, cThread4;
		"Thread 5", noIcon, "5", noMark, plain, cThread5;
		"Thread 6", noIcon, "6", noMark, plain, cThread6;
	  }
	};

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
purgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {	/* array: 13 elements */
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
	}
};

//--------------------------------------------------------------------------------------------------
// Contextual menus
//--------------------------------------------------------------------------------------------------

resource 'CMNU' (mRunningThread,
nonpurgeable) {
	mRunningThread,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Run",
	 {
		"Stop Thread", noIcon, noKey, noMark, plain, cStopThread;
	  }
	};

//--------------------------------------------------------------------------------------------------
// Contextual menus
//--------------------------------------------------------------------------------------------------

resource 'CMNU' (mStoppedThread,
nonpurgeable) {
	mStoppedThread,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Run",
	 {
		"Start Thread", noIcon, noKey, noMark, plain, cStartThread;
	  }
	};

//--------------------------------------------------------------------------------------------------
// Balloon Help Resources for Menus 
//--------------------------------------------------------------------------------------------------

#if defined(THINK_Rez) || defined(MW_Rez)
#include "HelpStrings.r"
#else
#include $$Shell("MALibraries") "HelpStrings.r";
#endif

include "Defaults.rsrc"  'hmnu' (mFile);	
include "Defaults.rsrc"  'hmnu' (mEdit);	

resource 'STR#' (kSimpleThreadsMenuHelp, purgeable) { 
	{
		"No help yet for this  item",
		"Help For Menu Title",
		"Help For First Menu Item",
		"Help For Second Menu Item",
		"Help For Third Menu Item",
	}
};

resource 'hmnu' (mSimpleThreadsMenu) 
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, // balloon definition function 
	0, // variation code 
	
	// Missing items 
	HMStringResItem 
	{
		kSimpleThreadsMenuHelp,1,
		kSimpleThreadsMenuHelp,1,
		kSimpleThreadsMenuHelp,1,
		kSimpleThreadsMenuHelp,1,
	},
	
	// Title and items 
	{	
		// Title 
		HMStringResItem 
		{
			kSimpleThreadsMenuHelp,2,
			kSimpleThreadsMenuHelp,2,
			kSimpleThreadsMenuHelp,2,
			kSimpleThreadsMenuHelp,2,
		},
		// First Item 
		HMStringResItem 
		{
			kSimpleThreadsMenuHelp,3,
			kSimpleThreadsMenuHelp,3,
			kSimpleThreadsMenuHelp,3,
			kSimpleThreadsMenuHelp,3,
		},
		// Second Item
		HMStringResItem 
		{
			kSimpleThreadsMenuHelp,4,
			kSimpleThreadsMenuHelp,4,
			kSimpleThreadsMenuHelp,4,
			kSimpleThreadsMenuHelp,4,
		},
		// Third Item
		HMStringResItem 
		{
			kSimpleThreadsMenuHelp,5,
			kSimpleThreadsMenuHelp,5,
			kSimpleThreadsMenuHelp,5,
			kSimpleThreadsMenuHelp,5,
		},
	}
};

//--------------------------------------------------------------------------------------------------
// RESOURCES FOR VIEWS, DIALOGS, ALERTS etc. 
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Views 
//--------------------------------------------------------------------------------------------------

#ifndef MW_Rez
include "SimpleThreads.view" not 'ckid';
#endif

/*
resource 'STR#' (kSimpleThreadsWindowTitles,
#if qNames
"Window titles",
#endif
purgeable) {
	{
	""
	}
};

resource 'STR#' (1200,
purgeable) {
	{
	""
	}
};

resource 'TxSt' (1200, purgeable) {
	tsBold,
	10,
	black,
	"Geneva"
};

resource 'TxSt' (kSystemTextStyle, purgeable) {tsPlain, 0, black, ""};
*/


/*
resource 'View' (kSimpleThreadsWindowID,
#if qNames
"kSimpleThreadsWindowID",
#endif
purgeable) { MAThreeOh, 
	{
	ViewSignatureAndClassname
		{'wind', 534, "", 'WIND', enabled, noIdle, {}, MAThreeOh, 
		{50, 20}, {335, 215}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}, 
			AdornLast, AdornerLocalObject {ResizeIconAdorner}}}, emptyUserArea, 
		Window {zoomDocProc, 'THR1', goAwayBox, resizable, ignoreFirstClick, 
			freeOnClosing, disposeOnFree, closesDocument, openWithDocument, dontAdaptToScreen, stagger, forceOnScreen, 
			dontCenter, doesntFloat, doesntHideOnSuspend, generateActivates, filler, kSimpleThreadsWindowTitles, 1}, 4}, 
	ViewSignatureAndClassname
		{'stat', 96, "", 'THRD', notEnabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {20, 150}, sizeFixed, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		StaticText {mStaticTextHit, notHilited, notDimmed, sizeable, {3, 3, 3, 3}, 1200, dontPreferOutline, autoWrap, dontEraseFirst, justCenter, 1200, 1}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'scrl', 186, "", 'SCLR', enabled, noIdle, {}, MAThreeOh, 
		{20, 0}, {300, 200}, sizeRelSuperView, sizeRelSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Scroller {'vCLR', 'hCLR', {0, 0}, {16, 16}, VertConstrain, HorzConstrain, noInset, respondsToFKeys}, 6}, 
	ViewSignatureAndClassname
		{'view', 75, "TSimpleThreadsView", 'THR1', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {100, 100}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews},
	ViewSignatureAndClassname
		{'view', 75, "TSimpleThreadsView", 'THR2', enabled, noIdle, {}, MAThreeOh, 
		{0, 100}, {100, 100}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews},
	ViewSignatureAndClassname
		{'view', 75, "TSimpleThreadsView", 'THR3', enabled, noIdle, {}, MAThreeOh, 
		{100, 0}, {100, 100}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews},
	ViewSignatureAndClassname
		{'view', 75, "TSimpleThreadsView", 'THR4', enabled, noIdle, {}, MAThreeOh, 
		{100, 100}, {100, 100}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews},
	ViewSignatureAndClassname
		{'view', 75, "TSimpleThreadsView", 'THR5', enabled, noIdle, {}, MAThreeOh, 
		{200, 0}, {100, 100}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews},
	ViewSignatureAndClassname
		{'view', 75, "TSimpleThreadsView", 'THR6', enabled, noIdle, {}, MAThreeOh, 
		{200, 100}, {100, 100}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews},
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'hCLR', enabled, noIdle, {}, MAThreeOh, 
		{245, -1}, {16, 417}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mHScrollBarHit, notHilited, notDimmed, sizeable, noInset, kSystemTextStyle, dontPreferOutline, h, 0, 0, 39585}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'vCLR', enabled, noIdle, {}, MAThreeOh, 
		{-1, 415}, {247, 16}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mVScrollBarHit, notHilited, notDimmed, sizeable, noInset, kSystemTextStyle, dontPreferOutline, v, 0, 0, 49755}, 
		NoSubviews}
	}
};
*/

//--------------------------------------------------------------------------------------------------
// About Box
//--------------------------------------------------------------------------------------------------

resource 'DITL' (phAboutApp,
#if qNames
"phAboutApp",
#endif
	purgeable) {
	 {	/* array DITLarray: 3 elements */
		/* [1] */
		{160, 182, 180, 262},
		Button {
			enabled,
			"OK"
		};
		/* [2] */
		{10, 75, 150, 316},
		StaticText {
			disabled,
			"This is a template application created by MakeNewProjectFromTemplate."
			"\n"
			WRITTEN_WITH_MACAPP
		};
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		}
	}
};

include "Defaults.rsrc"  'ALRT' (phAboutApp);		// Grab the default about box
include "Defaults.rsrc"	'aete' (0);					// Grab the default AppleEvent terminology
include "Defaults.rsrc"  'STR#' (kDefaultCredits);	// Grab the default credits

//--------------------------------------------------------------------------------------------------
// MultiFinder information
//--------------------------------------------------------------------------------------------------

/* Additional size deltas to be added to partition size */

#if qNames
#define kNamesSizeDelta 75
#else
#define kNamesSizeDelta 0
#endif

#if qDebug
#define kDebugSizeDelta 0
#else
#define kDebugSizeDelta 0
#endif

#define kModelFarSizeDelta 0

#ifndef MW_Rez
resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	notStationeryAware,				
	reserved,
	reserved,
	reserved,
	reserved,
	
	/* Suggested partition size */
	(10000 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024,
	
	/* Minimum partition size */
	(800 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024
};
#endif
//--------------------------------------------------------------------------------------------------
// Icons
//--------------------------------------------------------------------------------------------------

include "Defaults.rsrc"  'ICN#' (kApplicationID);	// MacApp Family large black & white	
include "Defaults.rsrc"  'ICN#' (kDocumentID);		// MacApp Document large black & white
include "Defaults.rsrc"  'ICN#' (kStationeryID);	// MacApp Stationery large black & white	
include "Defaults.rsrc"  'ics#' (kApplicationID);	// MacApp Family small black & white	
include "Defaults.rsrc"  'ics#' (kDocumentID);		// MacApp Document small black & white
include "Defaults.rsrc"  'ics#' (kStationeryID);	// MacApp Stationery small black & white
include "Defaults.rsrc"  'ics4' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'ics4' (kDocumentID);		// MacApp Document small 4 bit
include "Defaults.rsrc"  'ics4' (kStationeryID);	// MacApp Stationery small 4 bit
include "Defaults.rsrc"  'ics8' (kApplicationID);	// MacApp Family small 4 bit
include "Defaults.rsrc"  'ics8' (kDocumentID);		// MacApp Document small 4 bit
include "Defaults.rsrc"  'ics8' (kStationeryID);	// MacApp Stationery small 4 bit
include "Defaults.rsrc"  'icl4' (kApplicationID);	// MacApp Family large 4 bit
include "Defaults.rsrc"  'icl4' (kDocumentID);		// MacApp Document large 4 bit
include "Defaults.rsrc"  'icl4' (kStationeryID);	// MacApp Stationery large 4 bit
include "Defaults.rsrc"  'icl8' (kApplicationID);	// MacApp Family large 8 bit
include "Defaults.rsrc"  'icl8' (kDocumentID);		// MacApp Document large 8 bit
include "Defaults.rsrc"  'icl8' (kStationeryID);	// MacApp Stationery large 8 bit

//--------------------------------------------------------------------------------------------------
// Memory usage information for SimpleThreads
//--------------------------------------------------------------------------------------------------

// See UMemory.h
// -------------

// Code fragment resource for PowerPC native apps
// ----------------------------------------------
#ifndef MW_Rez 	// Metrowerks will supply it's own cfrg
#if qPowerPC
#include "CodeFragmentTypes.r"

resource 'cfrg' (0, "Configuration Info") {
	{
		kPowerPC,
		kFullLib,
		kNoVersionNum,
		kNoVersionNum,
		0,
		0,
		kIsApp,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		"SimpleThreads"
	}
};
#endif
#endif // MW_Rez
//--------------------------------------------------------------------------------------------------
// Icons, Bundles and FRefsOh my! (don't forget the Signature)
//--------------------------------------------------------------------------------------------------

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"SimpleThreads Apple Computer, Inc. 1996"
};

resource 'FREF' (kApplicationID,
#if qNames
"SimpleThreads Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (kDocumentID,
#if qNames
"SimpleThreads Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'FREF' (kStationeryID,
#if qNames
"SimpleThreads Stationery",
#endif
	purgeable) {
	kStationery,
	2,
	""
};

resource 'BNDL' (kBundleID,
#if qNames
"SimpleThreads",
#endif
	purgeable) {
	kSignature,
	0,
	{ /* array TypeArray: 2 elements */
		/* [1] */
		'ICN#',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		},
		/* [2] */
		'FREF',
		{ /* array IDArray: 3 elements */
			0, kApplicationID,
			1, kDocumentID,
			2, kStationeryID
		}
	}
};

//--------------------------------------------------------------------------------------------------
//  Version resources
//--------------------------------------------------------------------------------------------------

// The revision of this particular file

RESOURCE 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x01,
	0x00,
	development,
	0x00,
	verUs,
	"1.0",
	"SimpleThreads 1.0, Apple Computer, Inc. 1996-97"
};

include "Defaults.rsrc"  'vers' (2);		// Overall package
