//----------------------------------------------------------------------------------------
// UDocumentSimpleThreads.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UDocumentSimpleThreads__
#include "UDocumentSimpleThreads.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Threads

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif


//----------------------------------------------------------------------------------------
// Constants:

const ResNumber kSimpleThreadsWindowID = kDefaultWindowID;

const CommandNumber cNoThreads = 400;
const CommandNumber cThreadMgrThreads = 401;
const CommandNumber cMPThreads = 402;
const CommandNumber cThread1 = 403;
const CommandNumber cThread2 = 404;
const CommandNumber cThread3 = 405;
const CommandNumber cThread4 = 406;
const CommandNumber cThread5 = 407;
const CommandNumber cThread6 = 408;

const IDType kThreadStatus	= 'THRD';
const IDType kThread1		= 'THR1';
const IDType kThread2		= 'THR2';
const IDType kThread3		= 'THR3';
const IDType kThread4		= 'THR4';
const IDType kThread5		= 'THR5';
const IDType kThread6		= 'THR6';

//========================================================================================
// CLASS TDocumentSimpleThreads
//========================================================================================
#undef Inherited
#define Inherited TFileBasedDocument


MA_DEFINE_CLASS(TDocumentSimpleThreads);

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads constructor 
//----------------------------------------------------------------------------------------


TDocumentSimpleThreads::TDocumentSimpleThreads(TFile* itsFile,
										  OSType itsCreator) :
	TFileBasedDocument(itsFile,itsCreator),
	fThreadStyle(kNoThreads)
{
} // TDocumentSimpleThreads::TDocumentSimpleThreads 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads destructor
//----------------------------------------------------------------------------------------


TDocumentSimpleThreads::~TDocumentSimpleThreads()
{
}

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::FreeData: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::FreeData()
{
	Inherited::FreeData();
} // TDocumentSimpleThreads::FreeData 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::DoInitialState: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::DoInitialState()
{
	Inherited::DoInitialState();
	
} // TDocumentSimpleThreads::DoInitialState 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::DoMakeViews(bool /*forPrinting*/)
{
	TWindow* aWindow = gViewServer->NewTemplateWindow(kSimpleThreadsWindowID, this);
	::ThrowIfNULL_AC(aWindow);

	/*	You will likely have a different view to which you would like to 
		attach a print handler. */
	TView* aView = aWindow->FirstSubView();

	TStdPrintHandler* aHandler = TH_new TStdPrintHandler(this,			// its document 
							   aView,			// its view 
							   kFixedSize,		// horzontal page size is fixed 
							   kFixedSize);		// vertical page size is fixed 
							   
	fThread1 = aWindow->FindSubView(kThread1);
	fThread2 = aWindow->FindSubView(kThread2);
	fThread3 = aWindow->FindSubView(kThread3);
	fThread4 = aWindow->FindSubView(kThread4);
	fThread5 = aWindow->FindSubView(kThread5);
	fThread6 = aWindow->FindSubView(kThread6);
	
	TView* evalOnce = aWindow->FindSubView(kThreadStatus);
	TStaticText* aStatus = dynamic_cast<TStaticText*> (evalOnce);
	if (aStatus)
	{
		
		aStatus->SetText("Control-click to use contextual menus.", kRedraw);
	}
} // TDocumentSimpleThreads::DoMakeViews 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::DoSetupMenus: This method is overridden to enable menu items which
// should be enabled when this object is in the target chain. MacApp initially disables
// all menu items, then lets the objects in the target chain enable those items they
// handle.
//
// A document object is in the target chain when its window is the active window.
//
// The inherited method is called so that TDocument can enable document-level menu items
// like "Save ". This also ensures that objects further up the target chain (the
// application) can set up THEIR menus.
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::DoSetupMenus()
{
	Inherited::DoSetupMenus();
	
	// TBD - add runtime switch for using MP threads
	EnableCheck(cMPThreads, ::UsingMP_AC(),  fThreadStyle == kMPThreads);
	
	EnableCheck(cThread1, true, fThread1->GetUserArea() == true);
	EnableCheck(cThread2, true, fThread2->GetUserArea() == true);
	EnableCheck(cThread3, true, fThread3->GetUserArea() == true);
	EnableCheck(cThread4, true, fThread4->GetUserArea() == true);
	EnableCheck(cThread5, true, fThread5->GetUserArea() == true);
	EnableCheck(cThread6, true, fThread6->GetUserArea() == true);
} // TDocumentSimpleThreads::DoSetupMenus 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::DoMenuCommand: This method is overridden to handle menu items which
// are enabled when this document is in the target chain. In this example, this is true
// when the document is open and its window is the active window. The inherited method
// should always be called so that MacApp can allow successor objects in the target chain
// (i.e. the application) to handle THEIR menu items.
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber) 
	{
			
		case cMPThreads:
			// TBD - add runtime switch for using MP threads
			// toggle fThreadStyle
			// restart running threads
			break;
		
		case cThread1:
			fThread1->DoEvent(mToggleThread, this, NULL);
			break;
		case cThread2:
			fThread2->DoEvent(mToggleThread, this, NULL);
			break;
		case cThread3:
			fThread3->DoEvent(mToggleThread, this, NULL);
			break;
		case cThread4:
			fThread4->DoEvent(mToggleThread, this, NULL);
			break;
		case cThread5:
			fThread5->DoEvent(mToggleThread, this, NULL);
			break;
		case cThread6:
			fThread6->DoEvent(mToggleThread, this, NULL);
			break;
			
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
} // TDocumentSimpleThreads::DoMenuCommand 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes)
{
	Inherited::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);
} // TDocumentSimpleThreads::DoNeedDiskSpace 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::DoRead: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::DoRead(TFile* aFile,
						   	   bool forPrinting)
{
	Inherited::DoRead(aFile,forPrinting);
} // TDocumentSimpleThreads::DoRead 

//----------------------------------------------------------------------------------------
// TDocumentSimpleThreads::DoWrite: 
//----------------------------------------------------------------------------------------


void TDocumentSimpleThreads::DoWrite(TFile* aFile,
									bool makingCopy)
{
	Inherited::DoWrite(aFile,makingCopy);
} // TDocumentSimpleThreads::DoWrite 

//----------------------------------------------------------------------------------------
// End of UDocumentSimpleThreads.cp


