//----------------------------------------------------------------------------------------
// USimpleThreadsView.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __USIMPLETHREADSVIEW__
#include "USimpleThreadsView.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UDOCUMENTSKELETON__
#include "UDocumentSimpleThreads.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __FONTS__
#include "Fonts.h"
#endif

#include <fp.h>
// Threading


#ifndef __THREADS__
#include <Threads.h>
#endif

#ifndef __USimpleThread__
#include "USimpleThread.h"
#endif

#ifndef __CThreadRunner_AC__
#include "CThreadRunner_AC.h"
#endif

const mRunningThread = 1000;
const mStoppedThread = 1001;

const cStartThread = 409;
const cStopThread = 410;

//========================================================================================
// Global Initialization Procedure
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// InitUSimpleThreadsView: 
//----------------------------------------------------------------------------------------


void InitUSimpleThreadsView()
{
	// So the linker doesn't dead strip class info 
	MA_REGISTER_CLASS(TSimpleThreadsView);
}


//========================================================================================
// CLASS TSimpleThreadsView
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TSimpleThreadsView);

//----------------------------------------------------------------------------------------
// TSimpleThreadsView constructor 
//----------------------------------------------------------------------------------------


TSimpleThreadsView::TSimpleThreadsView(const CViewPoint& itsLocation,
						 				const CViewPoint& itsSize) : 
	TView(itsLocation,itsSize,sizeFixed, sizeFixed),
	fThread(NULL)
{
}

//----------------------------------------------------------------------------------------
// TSimpleThreadsView destructor
//----------------------------------------------------------------------------------------


TSimpleThreadsView::~TSimpleThreadsView()
{
	StopThreadExample();
}

//----------------------------------------------------------------------------------------
// TSimpleThreadsView::DoPostCreate: 
//----------------------------------------------------------------------------------------

	
void TSimpleThreadsView::DoPostCreate(TDocument* itsDocument) // Override 
{
	Inherited::DoPostCreate(itsDocument);
	
	// StartThreadExample();
}
	
//----------------------------------------------------------------------------------------
// TSimpleThreadsView::Draw: 
//----------------------------------------------------------------------------------------


void TSimpleThreadsView::Draw(const CViewRect& /*area*/) // Override 
{
	CRect_AC aRect(GetQDExtent());
	::PenNormal();	
	::PenPat(&qd.black);
	gRGBBlack.SetForeColor();
	::PaintRect(aRect);
}


//----------------------------------------------------------------------------------------
// TSimpleThreadsView StartThreadExample
//----------------------------------------------------------------------------------------


void TSimpleThreadsView::StartThreadExample()
{
	if (fThread == NULL)
	{
		fThread = TH_new CThreadRunner_AC(TH_new TSimpleThread(this));
		
		// set fUserArea so the document knows our thread is running
		fUserArea = true;
	}
}

//----------------------------------------------------------------------------------------
// TSimpleThreadsView StopThreadExample
//----------------------------------------------------------------------------------------


void TSimpleThreadsView::StopThreadExample()
{
	if (fThread != NULL) 
	{
		delete fThread;
		fThread = NULL;
		
		ForceRedraw();
		
		// set fUserArea so the document knows our thread has stopped
		fUserArea = false;
	}
}

//----------------------------------------------------------------------------------------
// TSimpleThreadsView DrawSpirograph
//----------------------------------------------------------------------------------------


void TSimpleThreadsView::DrawSpirograph(int& iter)
{
	if (Focus()) 
	{
		::PenNormal();
		CRGBColor_AC color;
		if (iter >= 16) 
		{	// Clear the view every so often.
			CViewRect aVRect = GetVisibleExtent();
			::PenNormal();	
			::PenPat(&qd.black);
			color.Set(0, 0, 0);
			color.SetForeColor();
			::PaintRect(&ViewToQD(aVRect));
			iter = 0;
		}
		++iter;
		color.Set(::Random(), ::Random(), ::Random());
		color.SetForeColor();
		::PenPat(&qd.black);
		// Generate three random polar coordinates on the unit circle.
		float r1, r2, r3, theta1, theta2, theta3;
		r1 = float(::Random())/float(32767);
		r2 = float(::Random())/float(32767);
		r3 = float(::Random())/float(32767);
		theta1 = float(::Random())/(2.0*pi);
		theta2 = float(::Random())/(2.0*pi);
		theta3 = float(::Random())/(2.0*pi);
		// Generate random number of increments.
		int count = 3 + ::Random()/(1024);
		// Get dimensions.
		CViewPoint center(fSize.h / 2, fSize.v / 2); 
		CViewPoint radii = center;
		--radii.h;
		--radii.v;
		// Compute starting point.
		CViewPoint start;
		start.h = center.h + (cos(theta1) * r1) * radii.h;
		start.v = center.v + (sin(theta1) * r1) * radii.v;
		CPoint_AC temp = ViewToQD(start);
		
		for (int i = 0; i < count; ++i) 
		{
			color.SetForeColor();
			::PenPat(&qd.black);
			::MoveTo(temp.h, temp.v);
			// Convert points to view coordinates.
			
			CViewPoint p1, p2, p3;
			p1.h = center.h + (cos(theta1) * r1) * radii.h;
			p1.v = center.v + (sin(theta1) * r1) * radii.v;
			p2.h = center.h + (cos(theta2) * r2) * radii.h;
			p2.v = center.v + (sin(theta2) * r2) * radii.v;
			p3.h = center.h + (cos(theta3) * r3) * radii.h;
			p3.v = center.v + (sin(theta3) * r3) * radii.v;
			// Draw the figure.
			temp = ViewToQD(p1);
			::LineTo(temp.h, temp.v);
			temp = ViewToQD(p2);
			::LineTo(temp.h, temp.v);
			temp = ViewToQD(p3);
			::LineTo(temp.h, temp.v);
			// Increment the angles.
			theta1 += 2.0*pi/float(count);
			theta2 += 2.0*pi/float(count);
			theta3 += 2.0*pi/float(count);
			::YieldToAnyThread();
			// Reestablish Focus after yield.
			Focus();
		}
		::MoveTo(temp.h, temp.v);
		temp = ViewToQD(start);
		::LineTo(temp.h, temp.v);	// Close the figure.
	}
}

//----------------------------------------------------------------------------------------
// TSimpleThreadsView::DoEvent: 
//----------------------------------------------------------------------------------------

			
void TSimpleThreadsView::DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event)
{
	switch (eventNumber)
	{
		case mStartThread:
			StartThreadExample();
			break;
			
		case mStopThread:
			StopThreadExample();
			break;
		
		case mToggleThread:
			if (fThread)
				StopThreadExample();
			else
				StartThreadExample();
			break;
	}
	Inherited::DoEvent(eventNumber, source, event);
}

#if qWantsContextualMenus

//----------------------------------------------------------------------------------------
// TSimpleThreadsView:: 
//----------------------------------------------------------------------------------------


void TSimpleThreadsView::DoSetupMenus()
{
	Inherited::DoSetupMenus();

	Enable(cStartThread, true /*(fThread == NULL)*/);
	Enable(cStopThread, true /*(fThread != NULL)*/);
} // TSimpleThreadsView::DoSetupMenus 

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------


void TSimpleThreadsView::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber) 
	{
			
		case cStartThread:
			StartThreadExample();
			break;
		
		case cStopThread:
			StopThreadExample();
			break;
			
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
} // TSimpleThreadsView::DoMenuCommand 

//----------------------------------------------------------------------------------------
// Contextual menu example
//----------------------------------------------------------------------------------------

void TSimpleThreadsView::GetContextualMenu(MenuRef& theMenuRef, CAEDesc& theSelection)
{
	// Contextual menu example code
	if (fThread == NULL)
		theMenuRef = ::MAGetMenu(mStoppedThread);
	else
		theMenuRef = ::MAGetMenu(mRunningThread);
		
	// Pass back our web site address for the selection!!!
	
}

#endif // qWantsContextualMenus

//----------------------------------------------------------------------------------------
// End of USimpleThreadsView.cp


