// Calc.r
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved. 

// Resource File for the Calc Sample Program.

/*  Auto-Include the requirements for this source */


#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

#ifndef __BalloonTypes__
#include "BalloonTypes.r"
#endif

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif


#ifndef THINK_Rez

	#if qDebug | qPerform
		include "Debug.rsrc" not 'ckid';
		include "Dialog.rsrc" not 'ckid';
	#endif

	include "MacApp.rsrc" not 'ckid';
	include "Printing.rsrc" not 'ckid';

	include "Editions.rsrc" not 'ckid';				// since Calc supports the Edition Mgr

	#ifndef MW_Rez
		#if !qNoCode
			include $$Shell("ObjApp")$$Shell("XAppName") 'CODE';
			#if qModelCFM
				include $$Shell("ObjApp")$$Shell("XAppName") 'cfrg';
				include $$Shell("ObjApp")$$Shell("XAppName") 'rseg';
			#endif
		#endif
	#endif // MW_Rez

	#ifndef MW_Rez	// We include the .r file directly in the MW Project
		/* Include the separately compiled resources */
		include "CalcUtilities.rsrc" not 'ckid';
	#endif  // MW_Rez

#endif // !THINK_Rez

#define mFormat			4
#define mCalculation	5
#define mGraphing		6
#define mWindowMenu		42

#define cRecalculate		401
#define cAutoCalc			403
#define cManualCalc			404
#define cGraphSelection		405	// Used for ALOE graphing
#define cGeneral			501
#define cNoDecimal			503
#define cDecimal			504
#define cScientific			505
#define cSystemJustify		506
#define cForceLeftJustify	507
#define cRightJustify		508
#define cCenter				509
#define cSelecting			1000
#define cSizeColumn			1001
#define cCutText			1103
#define cCopyText			1104
#define cClearText			1106
#define cCutCells			1203
#define cCopyCells			1204
#define cClearCells			1206
#define cStandardCut		1303
#define cStandardCopy		1304
#define cStandardClear		1306

/* static variables to initialize view templates */
#define	kMaxRows			120				/* Max number of rows */
#define	kMaxColumns			64				/* max number of columns */
#define	kCellWidth			80				/* default width of each cell */
#define	kCellHeight			17				/* default height of each cell */
#define	kRowInset			4				/* pixels separating cell contents from top/bottom edge */
#define	kColumnInset		4				/* pixels separating cell contents from left/right edge */
#define	kRowTitleWidth		32				/* width of row titles */
#define kRowTitleHeight		kCellHeight		/* height of the row titles */
#define	kColumnTitleWidth	kCellWidth		/* width of column titles */
#define	kColumnTitleHeight	20				/* height of column titles */
#define kFontSize			10
#define kMaxChars			255 			/* number of characters in the text edit field */

	
#define kCalcWindowType		1001 			/* ID of the main window or view resource */
#define kCellsViewHelp		128 			/* ID of the help resource for the cells, row and column views */
#define kHelpStrings		1201			/* ID of STR# resource for help strings */

#define	kCalcPlainFontTextStyle	130			// TextStyle information for Calc's Font
#define	kCalcBoldFontTextStyle	131			// TextStyle information for Calc's Font

#define kSignature			'SS12'			/* Application signature */
#define kFileType			'SF12'			/* Data file type */
#define kStationery			'sF12'			/* Stationery file type */

#define phSplash			1001			/* ID of splash screen dialog */
#define kDefaultWindowTitle 1006


/*--------------------------------------------------------------------------------
 MultiFinder info
--------------------------------------------------------------------------------*/

/* Additional size deltas to be added to partition size */

#if qNames
#define kNamesSizeDelta 100
#else
#define kNamesSizeDelta 0
#endif

#if qDebug
#define kDebugSizeDelta 512
#else
#define kDebugSizeDelta 0
#endif

#define kModelFarSizeDelta 0

#ifndef MW_Rez					// MW projects supply size resource for us
resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	isStationeryAware,				
	reserved,
	reserved,
	reserved,
	reserved,
	
	/* Suggested partition size */
	(1600 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024,
	
	/* Minimum partition size */
	(1600 + kNamesSizeDelta + kDebugSizeDelta + kModelFarSizeDelta) * 1024
};
#endif // MW_Rez

/*--------------------------------------------------------------------------------
	memory stuff
--------------------------------------------------------------------------------*/
resource 'mem!' (132,
#if qNames
	"Additional Stack Space",
#endif
	purgeable) {
	0,				// Add to object heap size
	0,				// Add to heap increment size
	0,				// Add to temporary reserve
	0,				// Add to permanent reserve
	49152			// Add to stack space
};

#ifndef MW_Rez 	// Metrowerks will supply it's own cfrg
	#if qPowerPC
	#include "CodeFragmentTypes.r"

	resource 'cfrg' (0, "Configuration Info") {
		{
			kPowerPC,
			kFullLib,
			kNoVersionNum,
			kNoVersionNum,
			49152,					// minimum application stack size
			0,
			kIsApp,
			kOnDiskFlat,
			kZeroOffset,
			kWholeFork,
			"Calc"
		}
	};
	#endif
#endif // MW_Rez

/*--------------------------------------------------------------------------------
 About Box
--------------------------------------------------------------------------------*/
resource 'DITL' (phAboutApp,
#if qNames
"phAboutApp",
#endif
	purgeable) {
	 {	/* array DITLarray: 3 elements */
		/* [1] */
		{167, 288, 187, 348},
		Button {
			enabled,
			"OK"
		};
		/* [2] */
		{10, 75, 154, 348},
		StaticText {
			disabled,
			"This sample program implements a spreadsheet with MacApp."
#if qALOE
			" Uses ALOE to create graphs."
#endif
			"\n"
			WRITTEN_WITH_MACAPP
		};
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			noteIcon
		}
	}
};

include "ResourceLib.rsrc"  'CURS' (9);			// Grab the grabber hand
include "Defaults.rsrc"  'ALRT' (phAboutApp);	// Grab the default about box
include "Defaults.rsrc"	'aete' (0);				// Grab the default AppleEvent terminology

/*--------------------------------------------------------------------------------
 Menus!
--------------------------------------------------------------------------------*/
include "Defaults.rsrc" 'CMNU' (mApple);		// Grab the default Apple menus

resource 'CMNU' (mFile,
#if qNames
"mFile",
#endif
purgeable) {
	mFile,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"File",
	 {	/* array: 14 elements */
/* [1] */	"New",				noIcon, "N",	noMark, plain, cNew;
/* [2] */	"Open",			noIcon, "O",	noMark, plain, cOpen;
/* [3] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [4] */	"Close",			noIcon, "W",	noMark, plain, cClose;
/* [5] */	"Save",				noIcon, "S",	noMark, plain, cSave;
/* [6] */	"Save As",			noIcon, noKey,	noMark, plain, cSaveAs;
/* [7] */	"Revert",			noIcon, noKey,	noMark, plain, cRevert;
/* [8] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [9] */	"Page Setup",		noIcon, noKey,	noMark, plain, cPageSetup;
/* [10] */	"Print",			noIcon, "P",	noMark, plain, cPrint;
/* [11] */	"Print One Copy",	noIcon, noKey,	noMark, plain, cPrintOne;
/* [12] */	"Print Selection",	noIcon, noKey,	noMark, plain, cPrintSelection;
/* [13] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [14] */	"Quit",				noIcon, "Q",	noMark, plain, cQuit
	}
};

include "Defaults.rsrc" 'CMNU' (mEditionMgrEdit);	// Grab the default Edition Manager Edit menu

resource 'CMNU' (mFormat,
#if qNames
"mFormat",
#endif
purgeable) {
	mFormat,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Format",
	 {	/* array: 8 elements */
/* [1] */	"General",					noIcon, noKey, noMark, plain, cGeneral;
/* [2] */	"No Decimal",				noIcon, noKey, noMark, plain, cNoDecimal;
/* [3] */	"Decimal",					noIcon, noKey, noMark, plain, cDecimal;
/* [4] */	"Scientific",				noIcon, noKey, noMark, plain, cScientific;
/* [5] */	"-",						noIcon, noKey, noMark, plain, nocommand;
/* [6] */	"System Justification",		noIcon, noKey, noMark, plain, cSystemJustify;
/* [7] */	"Force Left Justification",	noIcon, noKey, noMark, plain, cForceLeftJustify;
/* [8] */	"Center Justification",		noIcon, noKey, noMark, plain, cCenter;
/* [9] */	"Right Justification",		noIcon, noKey, noMark, plain, cRightJustify
	  }
	};

resource 'CMNU' (mCalculation,
#if qNames
"mCalculation",
#endif
purgeable) {
	mCalculation,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Calculation",
	 {	/* array: 4 elements */
/* [1] */	"Recalculate",		noIcon, "=",	noMark,	plain, cRecalculate;
/* [2] */	"-",				noIcon, noKey,	noMark,	plain, nocommand;
/* [3] */	"Automatic Calc",	noIcon, noKey,	check,	plain, cAutoCalc;
/* [4] */	"Manual Calc",		noIcon, noKey,	noMark,	plain, cManualCalc
	}
};

resource 'CMNU' (mGraphing,
#if qNames
"mGraphing",
#endif
purgeable) {
	mGraphing,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Graphing",
	 {	/* array: 4 elements */
/* [1] */	"Graph Selection",		noIcon, noKey,	noMark,	plain, cGraphSelection;
	}
};

resource 'CMNU' (mWindowMenu,
#if qNames
"mWindowMenu",
#endif
purgeable) {
	mWindowMenu,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Windows",
	{	/* array: 4 elements */
	}
};

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
purgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {	/* array: 13 elements */
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
		"Selection",			noIcon,	noKey,	noMark, plain, cSelecting;
		"Column Resize",		noIcon,	noKey,	noMark, plain, cSizeColumn;
		"Cut Text",				noIcon,	"X",	noMark, plain, cCutText;
		"Copy Text",			noIcon,	"C",	noMark, plain, cCopyText;
		"Clear Text",			noIcon,	noKey,	noMark, plain, cClearText;
		"Cut Cells",			noIcon,	"X",	noMark, plain, cCutCells;
		"Copy Cells",			noIcon,	"C",	noMark, plain, cCopyCells;
		"Clear Cells",			noIcon,	noKey,	noMark, plain, cClearCells;
		"Cut",					noIcon,	"X",	noMark, plain, cStandardCut;
		"Copy",					noIcon,	"C",	noMark, plain, cStandardCopy;
		"Clear",				noIcon,	 noKey,	noMark, plain, cStandardClear;
		"Cancel Publisher",		noIcon,  noKey, noMark, plain, cCancelPublisher;
		"Cancel Subscriber",	noIcon,  noKey, noMark, plain, cCancelSubscriber
	}
};

resource 'MBAR' (kMBarDisplayed,
#if qNames
"Calc",
#endif
	purgeable) {
	{mApple; mFile; mEditionMgrEdit; mFormat; mCalculation;
#if qALOE
	mGraphing;
#endif
	mWindowMenu
	}
};

//--------------------------------------------------------------------------------
// For building the main view from templates
//--------------------------------------------------------------------------------

#ifndef MW_Rez 
include "Calc.view";	// This file is included in Calc.prj for MW Builds
#endif

#if 0 

resource 'STR#' (kDefaultWindowTitle,
#if qNames
"Window titles",
#endif
purgeable) {
	{	/* array StringArray: 1 elements */
		/* [1] */
		"Spreadsheet <<<>>>"
	}
};

resource 'TxSt' (1000, purgeable) {
	tsPlain,
	10,
	{	/* array: 3 elements */
		/* [1] */
		0x0,
		/* [2] */
		0x0,
		/* [3] */
		0x0
	},
	"Geneva"
};

resource 'TxSt' (1001, purgeable) {
	tsBold,
	10,
	{	/* array: 3 elements */
		/* [1] */
		0x0,
		/* [2] */
		0x0,
		/* [3] */
		0x0
	},
	"Geneva"
};

/*--------------------------------------------------------------------------------
 For building the main view from templates
--------------------------------------------------------------------------------*/
resource 'View' (kCalcWindowType,
#if qNames
"Spreadsheet",
#endif
purgeable) 
{MAThreeOh, 
	{
	ViewSignatureAndClassname
		{'wind', 1668, "TCalcWindow", 'WIND', enabled, noIdle, {}, MAThreeOh, 
		{40, 10}, {289, 447}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}, 
			AdornLast, AdornerLocalObject {ResizeIconAdorner}}}, emptyUserArea, 
		Window {zoomDocProc, noID, goAwayBox, resizable, ignoreFirstClick, 
			freeOnClosing, disposeOnFree, closesDocument, openWithDocument, dontAdaptToScreen, stagger, forceOnScreen, 
			dontCenter, doesntFloat, doesntHideOnSuspend, generateActivates, filler, 1006, 1}, 6}, 
	ViewSignatureAndClassname
		{'view', 458, "", noID, notEnabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {35, 447}, sizeFixed, sizeSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		DrawingEnvironmentClassIDAndClassname {1, "TDrawingEnvironment", {1, 1}, 8, $"FFFF FFFF FFFF FFFF", {0, 0, 0}, {55570, 56502, 55979}}, 
		AdornerListSignatureAndClassname {NoIdentifier, "TAdornerList", 8, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {56, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea, 
		View {}, 2}, 
	ViewSignatureAndClassname
		{'view', 72, "TCoordView", 'CORD', enabled, noIdle, {}, MAThreeOh, 
		{11, 8}, {18, 24}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'tevw', 211, "TEntryView", 'ENTV', enabled, 0, {}, MAThreeOh, 
		{9, 64}, {18, 364}, sizeFixed, sizeFixed, shown, wantsToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, 1, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		DrawingEnvironmentClassIDAndClassname {1, "TDrawingEnvironment", {1, 1}, 8, $"FFFF FFFF FFFF FFFF", {0, 0, 0}, {65535, 65535, 65535}}, 
		AdornerListSignatureAndClassname {NoIdentifier, "TAdornerList", 8, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {56, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}, 72, AdornerLocalObject {SelectionAdorner}}}, emptyUserArea, 
		TEView {withoutStyle, crOnly, acceptChanges, dontFreeText, cTyping, 255, {2, 2, 2, 2}, justSystem, 1000, dontPreferOutline}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'scrl', 268, "TSecondaryScroller", 'SCL2', enabled, noIdle, {}, MAThreeOh, 
		{35, 32}, {18, 400}, sizeFixed, sizeRelSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Scroller {noID, noID, {18, 400}, {0, 0}, noVertConstrain, noHorzConstrain, noInset, respondsToFKeys}, 1}, 
	ViewSignatureAndClassname
		{'txtg', 139, "TColumnsView", 'COLS', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {18, 400}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			DrawView, AdornerLocalObject {DrawAdorner}, 
			DrawView, AdornerLocalObject {SelectionAdorner}}}, emptyUserArea, 
		TextGridView {1, 64, 21, 80, 0, 4, dontAdornRows, AdornCols, multipleSelection, 1001, dontPreferOutline}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'scrl', 265, "TSecondaryScroller", 'SCL3', enabled, noIdle, {}, MAThreeOh, 
		{53, 0}, {221, 32}, sizeRelSuperView, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Scroller {noID, noID, {2040, 32}, {0, 0}, noVertConstrain, noHorzConstrain, noInset, respondsToFKeys}, 1}, 
	ViewSignatureAndClassname
		{'txtg', 136, "TRowsView", 'ROWS', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {2040, 32}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			DrawView, AdornerLocalObject {DrawAdorner}, 
			DrawView, AdornerLocalObject {SelectionAdorner}}}, emptyUserArea, 
		TextGridView {120, 1, 17, 32, 4, 0, AdornRows, dontAdornCols, multipleSelection, 1001, dontPreferOutline}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'scrl', 300, "TPrimaryScroller", 'SCL1', enabled, noIdle, {}, MAThreeOh, 
		{53, 32}, {221, 400}, sizeRelSuperView, sizeRelSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Scroller {'vCL1', 'hCL1', {0, 0}, {0, 0}, VertConstrain, HorzConstrain, {-17, -31, 0, 0}, respondsToFKeys}, 1}, 
	ViewSignatureAndClassname
		{'txtg', 173, "TCellsView", 'CELL', enabled, noIdle, {BehaviorSignatureAndClassname {'sect', "TSectionBehavior", Behavior {enabled, noIdle}}}, MAThreeOh, 
		{0, 0}, {0, 0}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			DrawView, AdornerLocalObject {DrawAdorner}, 
			DrawView, AdornerLocalObject {SelectionAdorner}}}, emptyUserArea, 
		TextGridView {120, 64, 17, 80, 4, 4, AdornRows, AdornCols, multipleSelection, 1000, dontPreferOutline}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'hCL1', enabled, noIdle, {}, MAThreeOh, 
		{273, 0}, {16, 433}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mHScrollBarHit, notHilited, notDimmed, sizeable, noInset, 128, dontPreferOutline, h, 0, 0, 0}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'vCL1', enabled, noIdle, {}, MAThreeOh, 
		{35, 432}, {239, 16}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mVScrollBarHit, notHilited, notDimmed, sizeable, noInset, 128, dontPreferOutline, v, 0, 0, 0}, 
		NoSubviews}
	}
};

#endif 

//--------------------------------------------------------------------------------------------------
// Balloon Help Resources for Menus 
//--------------------------------------------------------------------------------------------------

// Includes resource 'STR#' (kStandardHelpStrings, "Balloon Help String 1")
// with our application name substituted in strings
#include "HelpStrings.r"

include "Defaults.rsrc" 'hmnu' (mApple);		// Grab the default Apple menus

resource 'hmnu' (mFile) {
	HelpMgrVersion,
	hmDefaultOptions,
	0,
	0,
	HMSkipItem { },
	{

		HMStringResItem {				// File 
			kStandardHelpStrings,32,
			kStandardHelpStrings,33,
			-1,0,
			-1,0
		},
		HMStringResItem {				// New 
			kStandardHelpStrings,34,
			kStandardHelpStrings,35,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Open 
			kStandardHelpStrings,36,
			kStandardHelpStrings,37,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// --------
		HMStringResItem {				// Close 
			kStandardHelpStrings,38,
			kStandardHelpStrings,39,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Save 
			kStandardHelpStrings,40,
			kStandardHelpStrings,41,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Save As 
			kStandardHelpStrings,42,
			kStandardHelpStrings,43,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// Save A Copy As
		HMSkipItem { },					// Revert
		HMSkipItem { },					// --------
		HMStringResItem {				// Page Setup 
			kStandardHelpStrings,44,
			kStandardHelpStrings,45,
			-1,0,
			-1,0
		},
		HMStringResItem {				// Print 
			kStandardHelpStrings,46,
			kStandardHelpStrings,47,
			-1,0,
			-1,0
		},
		HMSkipItem { },					// Print Selection
		HMSkipItem { },					// --------
		HMStringResItem {				// Quit 
			kStandardHelpStrings,48,
			kStandardHelpStrings,49,
			-1,0,
			-1,0
		}
	}
};

include "Defaults.rsrc" 'hmnu' (mEditionMgrEdit);		// Grab the edition mgr help info

//--------------------------------------------------------------------------------------------------
// Balloon Help Resources for Views 
//--------------------------------------------------------------------------------------------------

resource 'hdlg' (kCellsViewHelp, purgeable) 
{
	HelpMgrVersion,
	0,
	hmDefaultOptions,
	0, /* balloon definition function */
	5, /* variation code */
	HMSkipItem {}, /* No missing item help */
	{
		/* Help for cells view */
		HMStringResItem 
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			kHelpStrings,1,
			kHelpStrings,1,
			kHelpStrings,1,
			kHelpStrings,1,
		},
		/* Help for rows view */
		HMStringResItem 
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			kHelpStrings,2,
			kHelpStrings,2,
			kHelpStrings,2,
			kHelpStrings,2,
		},
		/* Help for columns view */
		HMStringResItem 
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			kHelpStrings,3,
			kHelpStrings,3,
			kHelpStrings,3,
			kHelpStrings,3,
		}
	}
};

resource 'STR#' (kHelpStrings, purgeable) { 
	{
		"This is the cells view.",
		"Selecting a numbered box selects the entire row of cells alongside it" ,
		"Selecting a labelled box selects the entire column of cells below it.",
	}
};

/*--------------------------------------------------------------------------------
 Splash screen
--------------------------------------------------------------------------------*/
resource 'DITL' (phSplash,
#if qNames
"phSplash",
#endif
	purgeable) {
	{	/* array DITLarray: 1 elements */
		/* [1] */
		{1, 1, 453-261+1, 490-101+1},
		Picture {
			disabled,
			phSplash
		}
	}
};

resource 'DLOG' (phSplash,
#if qNames
"phSplash",
#endif
	purgeable) {
	{76, 80, 453-261+76+2, 490-101+80+2},
	altDBoxProc,
	visible,
	noGoAway,
	0,
	phSplash,
	"",
	centerMainScreen
};

#ifndef __PICT_R__
#Include "Pict.r"
#endif

resource 'PICT' (1001, purgeable) {
	{-1, -1, 196, 393},
	VersionTwo {
		{	/* array OpCodes: 48 elements */
			/* [1] */
			HeaderOp {
				Version2,
				0xFFFF0000,
				0xFFFF0000,
				0x1890000,
				0xC40000,
				0
			},
			/* [2] */
			ShortComment {
				140
			},
			/* [3] */
			DefHilite {

			},
			/* [4] */
			Origin {
				-1,
				-1
			},
			/* [5] */
			Clip {
				{-2, -2, 195, 392},
				$""
			},
			/* [6] */
			PnSize {
				{0, 0}
			},
			/* [7] */
			PnPat {
				$"77BB EEDD 77BB EEDD"
			},
			/* [8] */
			ShortLine {
				{-3200, -3200},
				0,
				0
			},
			/* [9] */
			ShortLineFrom {
				0,
				0
			},
			/* [10] */
			LongComment {
				182,
				$"0001 0001"
			},
			/* [11] */
			ShortComment {
				130
			},
			/* [12] */
			Clip {
				{10, 13, 186, 189},
				$""
			},
			/* [13] */
			PackBitsRect {
				32,
				{0, 0, 32, 32},
				0,
				unpacked,
				0,
				0x480000,
				0x480000,
				chunky,
				8,
				1,
				8,
				0,
				23634384,
				0x8,
				0,
				{	/* array ColorSpec: 256 elements */
					/* [1] */
					0, 65535, 65535, 65535,
					/* [2] */
					1, 65535, 65535, 52428,
					/* [3] */
					2, 65535, 65535, 39321,
					/* [4] */
					3, 65535, 65535, 26214,
					/* [5] */
					4, 65535, 65535, 13107,
					/* [6] */
					5, 65535, 65535, 0,
					/* [7] */
					6, 65535, 52428, 65535,
					/* [8] */
					7, 65535, 52428, 52428,
					/* [9] */
					8, 65535, 52428, 39321,
					/* [10] */
					9, 65535, 52428, 26214,
					/* [11] */
					10, 65535, 52428, 13107,
					/* [12] */
					11, 65535, 52428, 0,
					/* [13] */
					12, 65535, 39321, 65535,
					/* [14] */
					13, 65535, 39321, 52428,
					/* [15] */
					14, 65535, 39321, 39321,
					/* [16] */
					15, 65535, 39321, 26214,
					/* [17] */
					16, 65535, 39321, 13107,
					/* [18] */
					17, 65535, 39321, 0,
					/* [19] */
					18, 65535, 26214, 65535,
					/* [20] */
					19, 65535, 26214, 52428,
					/* [21] */
					20, 65535, 26214, 39321,
					/* [22] */
					21, 65535, 26214, 26214,
					/* [23] */
					22, 65535, 26214, 13107,
					/* [24] */
					23, 65535, 26214, 0,
					/* [25] */
					24, 65535, 13107, 65535,
					/* [26] */
					25, 65535, 13107, 52428,
					/* [27] */
					26, 65535, 13107, 39321,
					/* [28] */
					27, 65535, 13107, 26214,
					/* [29] */
					28, 65535, 13107, 13107,
					/* [30] */
					29, 65535, 13107, 0,
					/* [31] */
					30, 65535, 0, 65535,
					/* [32] */
					31, 65535, 0, 52428,
					/* [33] */
					32, 65535, 0, 39321,
					/* [34] */
					33, 65535, 0, 26214,
					/* [35] */
					34, 65535, 0, 13107,
					/* [36] */
					35, 65535, 0, 0,
					/* [37] */
					36, 52428, 65535, 65535,
					/* [38] */
					37, 52428, 65535, 52428,
					/* [39] */
					38, 52428, 65535, 39321,
					/* [40] */
					39, 52428, 65535, 26214,
					/* [41] */
					40, 52428, 65535, 13107,
					/* [42] */
					41, 52428, 65535, 0,
					/* [43] */
					42, 52428, 52428, 65535,
					/* [44] */
					43, 52428, 52428, 52428,
					/* [45] */
					44, 52428, 52428, 39321,
					/* [46] */
					45, 52428, 52428, 26214,
					/* [47] */
					46, 52428, 52428, 13107,
					/* [48] */
					47, 52428, 52428, 0,
					/* [49] */
					48, 52428, 39321, 65535,
					/* [50] */
					49, 52428, 39321, 52428,
					/* [51] */
					50, 52428, 39321, 39321,
					/* [52] */
					51, 52428, 39321, 26214,
					/* [53] */
					52, 52428, 39321, 13107,
					/* [54] */
					53, 52428, 39321, 0,
					/* [55] */
					54, 52428, 26214, 65535,
					/* [56] */
					55, 52428, 26214, 52428,
					/* [57] */
					56, 52428, 26214, 39321,
					/* [58] */
					57, 52428, 26214, 26214,
					/* [59] */
					58, 52428, 26214, 13107,
					/* [60] */
					59, 52428, 26214, 0,
					/* [61] */
					60, 52428, 13107, 65535,
					/* [62] */
					61, 52428, 13107, 52428,
					/* [63] */
					62, 52428, 13107, 39321,
					/* [64] */
					63, 52428, 13107, 26214,
					/* [65] */
					64, 52428, 13107, 13107,
					/* [66] */
					65, 52428, 13107, 0,
					/* [67] */
					66, 52428, 0, 65535,
					/* [68] */
					67, 52428, 0, 52428,
					/* [69] */
					68, 52428, 0, 39321,
					/* [70] */
					69, 52428, 0, 26214,
					/* [71] */
					70, 52428, 0, 13107,
					/* [72] */
					71, 52428, 0, 0,
					/* [73] */
					72, 39321, 65535, 65535,
					/* [74] */
					73, 39321, 65535, 52428,
					/* [75] */
					74, 39321, 65535, 39321,
					/* [76] */
					75, 39321, 65535, 26214,
					/* [77] */
					76, 39321, 65535, 13107,
					/* [78] */
					77, 39321, 65535, 0,
					/* [79] */
					78, 39321, 52428, 65535,
					/* [80] */
					79, 39321, 52428, 52428,
					/* [81] */
					80, 39321, 52428, 39321,
					/* [82] */
					81, 39321, 52428, 26214,
					/* [83] */
					82, 39321, 52428, 13107,
					/* [84] */
					83, 39321, 52428, 0,
					/* [85] */
					84, 39321, 39321, 65535,
					/* [86] */
					85, 39321, 39321, 52428,
					/* [87] */
					86, 39321, 39321, 39321,
					/* [88] */
					87, 39321, 39321, 26214,
					/* [89] */
					88, 39321, 39321, 13107,
					/* [90] */
					89, 39321, 39321, 0,
					/* [91] */
					90, 39321, 26214, 65535,
					/* [92] */
					91, 39321, 26214, 52428,
					/* [93] */
					92, 39321, 26214, 39321,
					/* [94] */
					93, 39321, 26214, 26214,
					/* [95] */
					94, 39321, 26214, 13107,
					/* [96] */
					95, 39321, 26214, 0,
					/* [97] */
					96, 39321, 13107, 65535,
					/* [98] */
					97, 39321, 13107, 52428,
					/* [99] */
					98, 39321, 13107, 39321,
					/* [100] */
					99, 39321, 13107, 26214,
					/* [101] */
					100, 39321, 13107, 13107,
					/* [102] */
					101, 39321, 13107, 0,
					/* [103] */
					102, 39321, 0, 65535,
					/* [104] */
					103, 39321, 0, 52428,
					/* [105] */
					104, 39321, 0, 39321,
					/* [106] */
					105, 39321, 0, 26214,
					/* [107] */
					106, 39321, 0, 13107,
					/* [108] */
					107, 39321, 0, 0,
					/* [109] */
					108, 26214, 65535, 65535,
					/* [110] */
					109, 26214, 65535, 52428,
					/* [111] */
					110, 26214, 65535, 39321,
					/* [112] */
					111, 26214, 65535, 26214,
					/* [113] */
					112, 26214, 65535, 13107,
					/* [114] */
					113, 26214, 65535, 0,
					/* [115] */
					114, 26214, 52428, 65535,
					/* [116] */
					115, 26214, 52428, 52428,
					/* [117] */
					116, 26214, 52428, 39321,
					/* [118] */
					117, 26214, 52428, 26214,
					/* [119] */
					118, 26214, 52428, 13107,
					/* [120] */
					119, 26214, 52428, 0,
					/* [121] */
					120, 26214, 39321, 65535,
					/* [122] */
					121, 26214, 39321, 52428,
					/* [123] */
					122, 26214, 39321, 39321,
					/* [124] */
					123, 26214, 39321, 26214,
					/* [125] */
					124, 26214, 39321, 13107,
					/* [126] */
					125, 26214, 39321, 0,
					/* [127] */
					126, 26214, 26214, 65535,
					/* [128] */
					127, 26214, 26214, 52428,
					/* [129] */
					128, 26214, 26214, 39321,
					/* [130] */
					129, 26214, 26214, 26214,
					/* [131] */
					130, 26214, 26214, 13107,
					/* [132] */
					131, 26214, 26214, 0,
					/* [133] */
					132, 26214, 13107, 65535,
					/* [134] */
					133, 26214, 13107, 52428,
					/* [135] */
					134, 26214, 13107, 39321,
					/* [136] */
					135, 26214, 13107, 26214,
					/* [137] */
					136, 26214, 13107, 13107,
					/* [138] */
					137, 26214, 13107, 0,
					/* [139] */
					138, 26214, 0, 65535,
					/* [140] */
					139, 26214, 0, 52428,
					/* [141] */
					140, 26214, 0, 39321,
					/* [142] */
					141, 26214, 0, 26214,
					/* [143] */
					142, 26214, 0, 13107,
					/* [144] */
					143, 26214, 0, 0,
					/* [145] */
					144, 13107, 65535, 65535,
					/* [146] */
					145, 13107, 65535, 52428,
					/* [147] */
					146, 13107, 65535, 39321,
					/* [148] */
					147, 13107, 65535, 26214,
					/* [149] */
					148, 13107, 65535, 13107,
					/* [150] */
					149, 13107, 65535, 0,
					/* [151] */
					150, 13107, 52428, 65535,
					/* [152] */
					151, 13107, 52428, 52428,
					/* [153] */
					152, 13107, 52428, 39321,
					/* [154] */
					153, 13107, 52428, 26214,
					/* [155] */
					154, 13107, 52428, 13107,
					/* [156] */
					155, 13107, 52428, 0,
					/* [157] */
					156, 13107, 39321, 65535,
					/* [158] */
					157, 13107, 39321, 52428,
					/* [159] */
					158, 13107, 39321, 39321,
					/* [160] */
					159, 13107, 39321, 26214,
					/* [161] */
					160, 13107, 39321, 13107,
					/* [162] */
					161, 13107, 39321, 0,
					/* [163] */
					162, 13107, 26214, 65535,
					/* [164] */
					163, 13107, 26214, 52428,
					/* [165] */
					164, 13107, 26214, 39321,
					/* [166] */
					165, 13107, 26214, 26214,
					/* [167] */
					166, 13107, 26214, 13107,
					/* [168] */
					167, 13107, 26214, 0,
					/* [169] */
					168, 13107, 13107, 65535,
					/* [170] */
					169, 13107, 13107, 52428,
					/* [171] */
					170, 13107, 13107, 39321,
					/* [172] */
					171, 13107, 13107, 26214,
					/* [173] */
					172, 13107, 13107, 13107,
					/* [174] */
					173, 13107, 13107, 0,
					/* [175] */
					174, 13107, 0, 65535,
					/* [176] */
					175, 13107, 0, 52428,
					/* [177] */
					176, 13107, 0, 39321,
					/* [178] */
					177, 13107, 0, 26214,
					/* [179] */
					178, 13107, 0, 13107,
					/* [180] */
					179, 13107, 0, 0,
					/* [181] */
					180, 0, 65535, 65535,
					/* [182] */
					181, 0, 65535, 52428,
					/* [183] */
					182, 0, 65535, 39321,
					/* [184] */
					183, 0, 65535, 26214,
					/* [185] */
					184, 0, 65535, 13107,
					/* [186] */
					185, 0, 65535, 0,
					/* [187] */
					186, 0, 52428, 65535,
					/* [188] */
					187, 0, 52428, 52428,
					/* [189] */
					188, 0, 52428, 39321,
					/* [190] */
					189, 0, 52428, 26214,
					/* [191] */
					190, 0, 52428, 13107,
					/* [192] */
					191, 0, 52428, 0,
					/* [193] */
					192, 0, 39321, 65535,
					/* [194] */
					193, 0, 39321, 52428,
					/* [195] */
					194, 0, 39321, 39321,
					/* [196] */
					195, 0, 39321, 26214,
					/* [197] */
					196, 0, 39321, 13107,
					/* [198] */
					197, 0, 39321, 0,
					/* [199] */
					198, 0, 26214, 65535,
					/* [200] */
					199, 0, 26214, 52428,
					/* [201] */
					200, 0, 26214, 39321,
					/* [202] */
					201, 0, 26214, 26214,
					/* [203] */
					202, 0, 26214, 13107,
					/* [204] */
					203, 0, 26214, 0,
					/* [205] */
					204, 0, 13107, 65535,
					/* [206] */
					205, 0, 13107, 52428,
					/* [207] */
					206, 0, 13107, 39321,
					/* [208] */
					207, 0, 13107, 26214,
					/* [209] */
					208, 0, 13107, 13107,
					/* [210] */
					209, 0, 13107, 0,
					/* [211] */
					210, 0, 0, 65535,
					/* [212] */
					211, 0, 0, 52428,
					/* [213] */
					212, 0, 0, 39321,
					/* [214] */
					213, 0, 0, 26214,
					/* [215] */
					214, 0, 0, 13107,
					/* [216] */
					215, 61166, 0, 0,
					/* [217] */
					216, 56797, 0, 0,
					/* [218] */
					217, 48059, 0, 0,
					/* [219] */
					218, 43690, 0, 0,
					/* [220] */
					219, 34952, 0, 0,
					/* [221] */
					220, 30583, 0, 0,
					/* [222] */
					221, 21845, 0, 0,
					/* [223] */
					222, 17476, 0, 0,
					/* [224] */
					223, 8738, 0, 0,
					/* [225] */
					224, 4369, 0, 0,
					/* [226] */
					225, 0, 61166, 0,
					/* [227] */
					226, 0, 56797, 0,
					/* [228] */
					227, 0, 48059, 0,
					/* [229] */
					228, 0, 43690, 0,
					/* [230] */
					229, 0, 34952, 0,
					/* [231] */
					230, 0, 30583, 0,
					/* [232] */
					231, 0, 21845, 0,
					/* [233] */
					232, 0, 17476, 0,
					/* [234] */
					233, 0, 8738, 0,
					/* [235] */
					234, 0, 4369, 0,
					/* [236] */
					235, 0, 0, 61166,
					/* [237] */
					236, 0, 0, 56797,
					/* [238] */
					237, 0, 0, 48059,
					/* [239] */
					238, 0, 0, 43690,
					/* [240] */
					239, 0, 0, 34952,
					/* [241] */
					240, 0, 0, 30583,
					/* [242] */
					241, 0, 0, 21845,
					/* [243] */
					242, 0, 0, 17476,
					/* [244] */
					243, 0, 0, 8738,
					/* [245] */
					244, 0, 0, 4369,
					/* [246] */
					245, 61166, 61166, 61166,
					/* [247] */
					246, 56797, 56797, 56797,
					/* [248] */
					247, 48059, 48059, 48059,
					/* [249] */
					248, 43690, 43690, 43690,
					/* [250] */
					249, 34952, 34952, 34952,
					/* [251] */
					250, 30583, 30583, 30583,
					/* [252] */
					251, 21845, 21845, 21845,
					/* [253] */
					252, 17476, 17476, 17476,
					/* [254] */
					253, 8738, 8738, 8738,
					/* [255] */
					254, 4369, 4369, 4369,
					/* [256] */
					255, 0, 0, 0
				},
				{0, 0, 32, 32},
				{10, 13, 186, 189},
				srcCopy,
				$"02E1 000E F900 00C0 FAEC FA00 00C0 FAEC 0000 13F9 0000 ECFB C001 ECC0 FC00 01C0"
				$"ECFB C001 EC00 13FA 0001 C0EC FAC0 00EC FD00 01C0 ECFA C001 ECFA 12FA 0000 ECF9"
				$"C005 ECC0 0000 C0EC F9C0 01EC FA11 FB00 01C0 ECF8 C003 EC00 C0EC F8C0 01EC FA0F"
				$"FB00 00EC F7C0 02EC C0EC F7C0 01EC FA0E FC00 01C0 ECF6 C000 ECF6 C001 ECFA 09FC"
				$"0000 ECE9 C001 ECFA 0AFD 0001 C0EC E9C0 01EC FA09 FD00 00EC E8C0 01EC FA0A FE00"
				$"01C0 ECE8 C001 ECFA 09FE 0000 ECE7 C001 ECFA 0E05 0000 C0EC C0C0 ECFF FEC0 01EC"
				$"FA11 0200 00EC FEC0 00FF EEF7 05FF FAC0 C0EC FA11 0200 C0EC FEC0 00FF EEF7 05FF"
				$"FAC0 C0EC FA10 0100 ECFD C000 FFEE EB05 FFFA C0C0 ECFA 2001 C0EC FDC0 00FF FEF6"
				$"00EB FEF6 00EB FEF6 00EB FEF6 00EB FEF6 05FF FAC0 C0EC FA1F 00EC FCC0 04FF F6F7"
				$"F7EB FEF7 11EB F6F7 F7EB F6F7 F7EB F6F7 F7FF FAC0 C0EC FA11 FBEC 00FF FEEB FC47"
				$"F6EB 01FF FAFE EC00 FA18 0000 FCFA 00FF FEF6 FC47 FEF6 00EB FEF6 00EB FEF6 00FF"
				$"FCFA 1A00 00FC FA03 FFF6 F7F7 FC47 0BF6 F7F7 EBF6 F7F7 EBF6 F7F7 FFFC FA0F FB00"
				$"00FF FEEB FC47 F6EB 01FF FAFD 001B FB00 00FF FEF6 00EB FEF6 00EB FEF6 00EB FEF6"
				$"00EB FEF6 01FF FAFD 001B FB00 15FF F6F7 F7EB F6F7 F7EB F6F7 F7EB F6F7 F7EB F6F7"
				$"F7FF FAFD 000B FB00 00FF EEEB 01FF FAFD 001B FB00 00FF FEF6 00EB FEF6 00EB FEF6"
				$"00EB FEF6 00EB FEF6 01FF FAFD 001B FB00 15FF F6F7 F7EB F6F7 F7EB F6F7 F7EB F6F7"
				$"F7EB F6F7 F7FF FAFD 0008 FB00 ECFF 00FA FD00 06FA 00EC FAFD 0002 E100 02E1 00"
			},
			/* [14] */
			ShortComment {
				131
			},
			/* [15] */
			LongComment {
				150,
				$"0100 0000 0200 0000 0000"
			},
			/* [16] */
			Origin {
				-211,
				-15
			},
			/* [17] */
			Clip {
				{0, 0, 56, 171},
				$""
			},
			/* [18] */
			ShortLineFrom {
				0,
				0
			},
			/* [19] */
			FontName {
				20,
				"Times"
			},
			/* [20] */
			TxFont {
				20
			},
			/* [21] */
			TxFace {
				0x1
			},
			/* [22] */
			TxSize {
				56
			},
			/* [23] */
			AppleReserved002E {
				$"0000 0100"
			},
			/* [24] */
			DVText {
				42,
				"CALC"
			},
			/* [25] */
			ShortComment {
				151
			},
			/* [26] */
			LongComment {
				150,
				$"0100 0000 0200 0000 0000"
			},
			/* [27] */
			Origin {
				-9,
				-47
			},
			/* [28] */
			Clip {
				{0, 0, 72, 153},
				$""
			},
			/* [29] */
			PnPat {
				$"FFFF FFFF FFFF FFFF"
			},
			/* [30] */
			ShortLineFrom {
				0,
				0
			},
			/* [31] */
			TxFace {
				0x0
			},
			/* [32] */
			TxSize {
				24
			},
			/* [33] */
			LongText {
				{18, 0},
				"a spreadsheet "
			},
			/* [34] */
			DVText {
				24,
				"sample written "
			},
			/* [35] */
			DVText {
				24,
				"with MacApp"
			},
			/* [36] */
			ShortComment {
				151
			},
			/* [37] */
			LongComment {
				150,
				$"0100 0000 0200 0000 0000"
			},
			/* [38] */
			Origin {
				9,
				-87
			},
			/* [39] */
			Clip {
				{0, 0, 24, 153},
				$""
			},
			/* [40] */
			ShortLineFrom {
				0,
				0
			},
			/* [41] */
			FontName {
				3,
				"Geneva"
			},
			/* [42] */
			TxFont {
				3
			},
			/* [43] */
			TxSize {
				9
			},
			/* [44] */
			LongText {
				{10, 0},
				"Copyright  Apple Computer, Inc. "
			},
			/* [45] */
			DVText {
				12,
				"1985-1996. All rights reserved."
			},
			/* [46] */
			ShortComment {
				151
			},
			/* [47] */
			ShortComment {
				141
			},
			/* [48] */
			OpEndPic {

			}
		}
	}
};

include "Defaults.rsrc"  'STR#' (kDefaultCredits);		// Grab the default credits

// Icons, Bundles and FRefsOh my! (don't forget the Signature)

resource 'ICN#' (128, "Calc Family", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0000 0000 007F 00FE 0081 0102 0080 8203"
		$"0100 8403 0100 4803 0200 5003 0200 2003"
		$"0400 0003 0400 0003 0800 0003 0800 0003"
		$"1000 0003 13FF FFE3 2200 0033 2200 0033"
		$"43FF FFF3 4222 2233 8222 2233 FBFF FFF7"
		$"7A3E 2237 7A3E 2237 03FF FFF0 0222 2230"
		$"0222 2230 03FF FFF0 0222 2230 0222 2230"
		$"03FF FFF0 01FF FFF0",
		/* [2] */
		$"0000 0000 00FF 01FE 00FF 83FE 01FF 87FF"
		$"01FF CFFF 03FF DFFF 03FF FFFF 07FF FFFF"
		$"07FF FFFF 0FFF FFFF 0FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 3FFF FFFF 3FFF FFFF 7FFF FFFF"
		$"7FFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
		$"7FFF FFFF 7FFF FFFF 03FF FFF0 03FF FFF0"
		$"03FF FFF0 03FF FFF0 03FF FFF0 03FF FFF0"
		$"03FF FFF0 01FF FFF0"
	}
};

resource 'ICN#' (129, "Calc Document", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"1FFF FC00 1000 0600 1000 0500 1000 0480"
		$"1000 0440 1000 0420 1000 07F0 1000 0010"
		$"1000 0010 1000 0010 1000 0010 1000 0010"
		$"13FF FF90 1044 4410 1044 4410 13FF FF90"
		$"107C 4410 107C 4410 13FF FF90 1044 4410"
		$"1044 4410 13FF FF90 1044 4410 1044 4410"
		$"13FF FF90 1044 4410 1044 4410 1000 0010"
		$"1000 0010 1000 0010 1000 0010 1FFF FFF0",
		/* [2] */
		$"1FFF FC00 1FFF FE00 1FFF FF00 1FFF FF80"
		$"1FFF FFC0 1FFF FFE0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
	}
};

resource 'ICN#' (130, "Calc Stationery", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"3FFF FFE0 2000 0020 2000 0038 2000 0028"
		$"27FF FF28 2088 8828 2088 8828 27FF FF28"
		$"20F8 8828 20F8 8828 27FF FF28 2088 8828"
		$"2088 8828 27FF FF28 2088 8828 2088 8828"
		$"27FF FF28 2088 8828 2088 8828 2000 0028"
		$"2000 0028 2000 0028 2000 0028 2000 0FE8"
		$"2000 0848 2000 0888 2000 0908 2000 0A08"
		$"2000 0C08 3FFF F808 0800 0008 0FFF FFF8",
		/* [2] */
		$"3FFF FFE0 3FFF FFE0 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 0FFF FFF8 0FFF FFF8"
	}
};

resource 'ics#' (128, "Calc Family", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0707 0489 0851 0821 1001 1001 2FF9 280D"
		$"6FFD 6AAD EFFD 6BAC 0FFC 0AAC 0FFC 07FC",
		/* [2] */
		$"078F 0FDF 0FFF 1FFF 1FFF 3FFF 3FFF 7FFF"
		$"7FFF FFFF FFFF 7FFF 0FFC 0FFC 0FFC 07FC"
	}
};

resource 'ics#' (129, "Calc Document", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FE0 4030 4028 403C 4004 4004 5FF4 4AA4"
		$"5FF4 4EA4 5FF4 4AA4 5FF4 4AA4 4004 7FFC",
		/* [2] */
		$"7FE0 7FF0 7FF8 7FFC 7FFC 7FFC 7FFC 7FFC"
		$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
	}
};

resource 'ics#' (130, "Calc Stationery", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FF8 4008 5FCE 4A8A 5FCA 4E8A 5FCA 4A8A"
		$"5FCA 4A8A 407A 4052 4062 7FC2 1002 1FFE",
		/* [2] */
		$"7FF8 7FF8 7FFE 7FFE 7FFE 7FFE 7FFE 7FFE"
		$"7FFE 7FFE 7FFE 7FFE 7FFE 7FFE 1FFE 1FFE"
	}
};

resource 'ics4' (128, "Calc Family", purgeable) {
	$"0000 0666 7000 7666 0000 7677 6707 6776"
	$"0000 6777 7676 7776 0007 6777 7767 7776"
	$"0006 7777 7777 7776 0076 7777 7777 7776"
	$"0067 FFFF FFFF F776 0767 FCCC CCCC FD76"
	$"0677 F666 6666 FD76 7677 FC6C 6C6C FD76"
	$"6666 F633 3666 FD66 0DDD FC33 3C6C FDDD"
	$"0000 F633 3666 FD00 0000 FC6C 6C6C FD00"
	$"0000 FFFF FFFF FD00 0000 0DDD DDDD DD"
};

resource 'ics4' (129, "Calc Document", purgeable) {
	$"0FFF FFFF FFF0 0000 0F0C 0C0C 0CFF 0000"
	$"0FC0 C0C0 C0FC F000 0F0C 0C0C 0CFF FF00"
	$"0FC0 C0C0 C0C0 CF00 0F0C 0C0C 0C0C 0F00"
	$"0FC6 6666 6666 CF00 0F0C 6C6C 6C6C 0F00"
	$"0FC6 6666 6666 CF00 0F0C 6360 606C 0F00"
	$"0FC6 6666 6666 CF00 0F0C 6060 606C 0F00"
	$"0FC6 6666 6666 CF00 0F0C 6C6C 6C6C 0F00"
	$"0FC0 C0C0 C0C0 CF00 0FFF FFFF FFFF FF"
};

resource 'ics4' (130, "Calc Stationery", purgeable) {
	$"0FFF FFFF FFFF F000 0F0C 0C0C 0C0C F000"
	$"0FC6 6666 66C0 FFF0 0F0C 6060 6C0C FDF0"
	$"0FC6 6666 66C0 FDF0 0F0C 6360 6C0C FDF0"
	$"0FC6 6666 66C0 FDF0 0F0C 6060 6C0C FDF0"
	$"0FC6 6666 66C0 FDF0 0F0C 6C6C 6C0C FDF0"
	$"0FC0 C0C0 CFFF FDF0 0F0C 0C0C 0FCF DDF0"
	$"0FC0 C0C0 CFFD DCF0 0FFF FFFF FFDD C0F0"
	$"000F DDDD DDDC 0CF0 000F FFFF FFFF FFF0"
};

resource 'ics8' (128, "Calc Family", purgeable) {
	$"0000 0000 00EC ECEC C000 0000 C0EC ECEC"
	$"0000 0000 C0EC C0C0 ECC0 00C0 ECC0 C0EC"
	$"0000 0000 ECC0 C0C0 C0EC C0EC C0C0 C0EC"
	$"0000 00C0 ECC0 C0C0 C0C0 ECC0 C0C0 C0EC"
	$"0000 00EC C0C0 C0C0 C0C0 C0C0 C0C0 C0EC"
	$"0000 C0EC C0C0 C0C0 C0C0 C0C0 C0C0 C0EC"
	$"0000 ECC0 FFFF FFFF FFFF FFFF FFC0 C0EC"
	$"00C0 ECC0 FFF7 F7F7 F7F7 F7F7 FFFA C0EC"
	$"00EC C0C0 FFEC ECEC ECEC ECEC FFFA C0EC"
	$"C0EC C0C0 FFF7 ECF7 ECF7 ECF7 FFFA C0EC"
	$"ECEC ECEC FFEC 4747 47EC ECEC FFFA ECEC"
	$"00FA FAFA FFF7 4747 47F7 ECF7 FFFA FAFA"
	$"0000 0000 FFEC 4747 47EC ECEC FFFA 0000"
	$"0000 0000 FFF7 ECF7 ECF7 ECF7 FFFA 0000"
	$"0000 0000 FFFF FFFF FFFF FFFF FFFA 0000"
	$"0000 0000 00FA FAFA FAFA FAFA FAFA"
};

resource 'ics8' (129, "Calc Document", purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FF2B FF00 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF FFFF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5EC ECEC ECEC ECEC ECEC F5FF 0000"
	$"00FF F5F5 ECF5 ECF5 ECF5 ECF5 F5FF 0000"
	$"00FF F5EC ECEC ECEC ECEC ECEC F5FF 0000"
	$"00FF F5F5 ECD8 ECF5 ECF5 ECF5 F5FF 0000"
	$"00FF F5EC ECEC ECEC ECEC ECEC F5FF 0000"
	$"00FF F5F5 ECF5 ECF5 ECF5 ECF5 F5FF 0000"
	$"00FF F5EC ECEC ECEC ECEC ECEC F5FF 0000"
	$"00FF F5F5 ECF5 ECF5 ECF5 ECF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'ics8' (130, "Calc Stationery", purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF FF00 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 FF00 0000"
	$"00FF F5EC ECEC ECEC ECEC F5F5 FFFF FF00"
	$"00FF F5F5 ECF5 ECF5 ECF5 F5F5 FFF9 FF00"
	$"00FF F5EC ECEC ECEC ECEC F5F5 FFF9 FF00"
	$"00FF F5F5 ECD8 ECF5 ECF5 F5F5 FFF9 FF00"
	$"00FF F5EC ECEC ECEC ECEC F5F5 FFF9 FF00"
	$"00FF F5F5 ECF5 ECF5 ECF5 F5F5 FFF9 FF00"
	$"00FF F5EC ECEC ECEC ECEC F5F5 FFF9 FF00"
	$"00FF F5F5 EC00 EC00 ECF5 F5F5 FFF9 FF00"
	$"00FF F5F5 F5F5 F5F5 F5FF FFFF FFF9 FF00"
	$"00FF F5F5 F5F5 F5F5 F5FF 2BFF F9F9 FF00"
	$"00FF F5F5 F5F5 F5F5 F5FF FFF9 F9F5 FF00"
	$"00FF FFFF FFFF FFFF FFFF F9F9 F5F5 FF00"
	$"0000 00FF F9F9 F9F9 F9F9 F9F5 F5F5 FF00"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FF"
};

resource 'icl4' (128, "Calc Family", purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 7666 6666 0000 0007 6666 6660"
	$"0000 0000 6777 7776 7000 0076 7777 7760"
	$"0000 0007 6777 7777 6000 0767 7777 776D"
	$"0000 0006 7777 7777 6700 7677 7777 776D"
	$"0000 0076 7777 7777 7607 6777 7777 776D"
	$"0000 0067 7777 7777 7676 7777 7777 776D"
	$"0000 0767 7777 7777 7767 7777 7777 776D"
	$"0000 0677 7777 7777 7777 7777 7777 776D"
	$"0000 7677 7777 7777 7777 7777 7777 776D"
	$"0000 6777 7777 7777 7777 7777 7777 776D"
	$"0007 6777 7777 7777 7777 7777 7777 776D"
	$"0006 7777 7777 7777 7777 7777 7777 776D"
	$"0076 77FF FFFF FFFF FFFF FFFF FFF7 776D"
	$"0067 77FC CCCC CCCC CCCC CCCC CCFD 776D"
	$"0767 77FC CCCC CCCC CCCC CCCC CCFD 776D"
	$"0677 77F6 6666 6666 6666 6666 66FD 776D"
	$"7677 77FC CC6C CC6C CC6C CC6C CCFD 776D"
	$"6777 77FC CC6C CC6C CC6C CC6C CCFD 776D"
	$"6666 66F6 6633 3336 6666 6666 66FD 666D"
	$"0DDD DDFC CC33 333C CC6C CC6C CCFD DDDD"
	$"0DDD DDFC CC33 333C CC6C CC6C CCFD DDDD"
	$"0000 00F6 6633 3336 6666 6666 66FD 0000"
	$"0000 00FC CC6C CC6C CC6C CC6C CCFD 0000"
	$"0000 00FC CC6C CC6C CC6C CC6C CCFD 0000"
	$"0000 00F6 6666 6666 6666 6666 66FD 0000"
	$"0000 00FC CC6C CC6C CC6C CC6C CCFD 0000"
	$"0000 00FC CC6C CC6C CC6C CC6C CCFD 0000"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFD 0000"
	$"0000 000D DDDD DDDD DDDD DDDD DDDD"
};

resource 'icl4' (129, "Calc Document", purgeable) {
	$"000F FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0FF0 0000 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 CFCF 0000 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0FCC F000 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 CFCC CF00 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0FCC CCF0 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 CFFF FFFF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C066 6666 6666 6666 6666 60CF 0000"
	$"000F 0C0C 060C 060C 060C 060C 0C0F 0000"
	$"000F C0C0 C6C0 C6C0 C6C0 C6C0 C0CF 0000"
	$"000F 0C66 6666 6666 6666 6666 6C0F 0000"
	$"000F C0C0 C633 36C0 C6C0 C6C0 C0CF 0000"
	$"000F 0C0C 0633 360C 060C 060C 0C0F 0000"
	$"000F C066 6666 6666 6666 6666 60CF 0000"
	$"000F 0C0C 060C 060C 060C 060C 0C0F 0000"
	$"000F C0C0 C6C0 C6C0 C6C0 C6C0 C0CF 0000"
	$"000F 0C66 6666 6666 6666 6666 6C0F 0000"
	$"000F C0C0 C6C0 C6C0 C6C0 C6C0 C0CF 0000"
	$"000F 0C0C 060C 060C 060C 060C 0C0F 0000"
	$"000F C066 6666 6666 6666 6666 60CF 0000"
	$"000F 0C0C 060C 060C 060C 060C 0C0F 0000"
	$"000F C0C0 C6C0 C6C0 C6C0 C6C0 C0CF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'icl4' (130, "Calc Stationery", purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF FFF0 0000"
	$"00FC 0C0C 0C0C 0C0C 0C0C 0C0C 0CF0 0000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 C0C0 C0FF F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C 0C0C 0CFD F000"
	$"00F0 C666 6666 6666 6666 6666 C0FD F000"
	$"00FC 0C0C 6C0C 6C0C 6C0C 6C0C 0CFD F000"
	$"00F0 C0C0 60C0 60C0 60C0 60C0 C0FD F000"
	$"00FC 0666 6666 6666 6666 6666 0CFD F000"
	$"00F0 C0C0 6333 60C0 60C0 60C0 C0FD F000"
	$"00FC 0C0C 6333 6C0C 6C0C 6C0C 0CFD F000"
	$"00F0 C666 6666 6666 6666 6666 C0FD F000"
	$"00FC 0C0C 6C0C 6C0C 6C0C 6C0C 0CFD F000"
	$"00F0 C0C0 60C0 60C0 60C0 60C0 C0FD F000"
	$"00FC 0666 6666 6666 6666 6666 0CFD F000"
	$"00F0 C0C0 60C0 60C0 60C0 60C0 C0FD F000"
	$"00FC 0C0C 6C0C 6C0C 6C0C 6C0C 0CFD F000"
	$"00F0 C666 6666 6666 6666 6666 C0FD F000"
	$"00FC 0C0C 6C0C 6C0C 6C0C 6C0C 0CFD F000"
	$"00F0 C0C0 60C0 60C0 60C0 60C0 C0FD F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C 0C0C 0CFD F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 C0C0 C0FD F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C 0C0C 0CFD F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 C0C0 C0FD F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C FFFF FFFD F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 FCCC CFDD F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C FCCC FDDC F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 FCCF DDCC F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C FCFD DCCC F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 FFDD CCCC F000"
	$"00FF FFFF FFFF FFFF FFFF FDDC CCCC F000"
	$"0000 FDDD DDDD DDDD DDDD DDCC CCCC F000"
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF F0"
};

resource 'icl8' (128, "Calc Family", purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 C0EC ECEC ECEC ECEC"
	$"0000 0000 0000 00C0 ECEC ECEC ECEC EC00"
	$"0000 0000 0000 0000 ECC0 C0C0 C0C0 C0EC"
	$"C000 0000 0000 C0EC C0C0 C0C0 C0C0 EC00"
	$"0000 0000 0000 00C0 ECC0 C0C0 C0C0 C0C0"
	$"EC00 0000 00C0 ECC0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 0000 00EC C0C0 C0C0 C0C0 C0C0"
	$"ECC0 0000 C0EC C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 0000 C0EC C0C0 C0C0 C0C0 C0C0"
	$"C0EC 00C0 ECC0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 0000 ECC0 C0C0 C0C0 C0C0 C0C0"
	$"C0EC C0EC C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 00C0 ECC0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 00EC C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 C0EC C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 00C0 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 00EC C0C0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 C0EC C0C0 FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFC0 C0C0 ECFA"
	$"0000 ECC0 C0C0 FFF7 F7F7 F7F7 F7F7 F7F7"
	$"F7F7 F7F7 F7F7 F7F7 F7F7 FFFA C0C0 ECFA"
	$"00C0 ECC0 C0C0 FFF7 F7F7 F7F7 F7F7 F7F7"
	$"F7F7 F7F7 F7F7 F7F7 F7F7 FFFA C0C0 ECFA"
	$"00EC C0C0 C0C0 FFEB EBEB EBEB EBEB EBEB"
	$"EBEB EBEB EBEB EBEB EBEB FFFA C0C0 ECFA"
	$"C0EC C0C0 C0C0 FFF6 F6F6 EBF6 F6F6 EBF6"
	$"F6F6 EBF6 F6F6 EBF6 F6F6 FFFA C0C0 ECFA"
	$"ECC0 C0C0 C0C0 FFF6 F7F7 EBF7 F7F7 EBF6"
	$"F7F7 EBF6 F7F7 EBF6 F7F7 FFFA C0C0 ECFA"
	$"ECEC ECEC ECEC FFEB EBEB 4747 4747 47EB"
	$"EBEB EBEB EBEB EBEB EBEB FFFA ECEC ECFA"
	$"00FA FAFA FAFA FFF6 F6F6 4747 4747 47F6"
	$"F6F6 EBF6 F6F6 EBF6 F6F6 FFFA FAFA FAFA"
	$"00FA FAFA FAFA FFF6 F7F7 4747 4747 47F6"
	$"F7F7 EBF6 F7F7 EBF6 F7F7 FFFA FAFA FAFA"
	$"0000 0000 0000 FFEB EBEB 4747 4747 47EB"
	$"EBEB EBEB EBEB EBEB EBEB FFFA 0000 0000"
	$"0000 0000 0000 FFF6 F6F6 EBF6 F6F6 EBF6"
	$"F6F6 EBF6 F6F6 EBF6 F6F6 FFFA 0000 0000"
	$"0000 0000 0000 FFF6 F7F7 EBF6 F7F7 EBF6"
	$"F7F7 EBF6 F7F7 EBF6 F7F7 FFFA 0000 0000"
	$"0000 0000 0000 FFEB EBEB EBEB EBEB EBEB"
	$"EBEB EBEB EBEB EBEB EBEB FFFA 0000 0000"
	$"0000 0000 0000 FFF6 F6F6 EBF6 F6F6 EBF6"
	$"F6F6 EBF6 F6F6 EBF6 F6F6 FFFA 0000 0000"
	$"0000 0000 0000 FFF6 F7F7 EBF6 F7F7 EBF6"
	$"F7F7 EBF6 F7F7 EBF6 F7F7 FFFA 0000 0000"
	$"0000 0000 0000 FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFA 0000 0000"
	$"0000 0000 0000 00FA FAFA FAFA FAFA FAFA"
	$"FAFA FAFA FAFA FAFA FAFA FAFA"
};

resource 'icl8' (129, "Calc Document", purgeable) {
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF 0000 0000 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF FF00 0000 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2BFF 0000 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2B2B FF00 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2B2B 2BFF 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2B2B 2B2B FF00 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF FFFF FFFF FFFF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC ECF5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC ECF5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC D8D8 D8EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC D8D8 D8EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC ECF5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC ECF5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC ECF5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5EC F5F5 F5EC F5F5"
	$"F5EC F5F5 F5EC F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'icl8' (130, "Calc Stationery", purgeable) {
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FF00 0000 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFFF FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5EC ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5EC ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECD8 D8D8 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECD8 D8D8 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5EC ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5EC ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5EC ECEC ECEC ECEC ECEC ECEC"
	$"ECEC ECEC ECEC ECEC F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 ECF5 F5F5 ECF5 F5F5"
	$"ECF5 F5F5 ECF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFFF FFFF FFFF FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FF2B 2B2B F7FF F9F7 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FF2B 2BF7 FFF9 F72B FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FF2B F7FF F9F7 2BF6 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFF7 FFF9 F72B F6F6 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFFF F9F7 2BF6 F6F6 FF00 0000"
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFF9 F72B F6F6 F6F6 FF00 0000"
	$"0000 0000 FFF9 F9F9 F9F9 F9F9 F9F9 F9F9"
	$"F9F9 F9F9 F9F7 2BF6 F6F6 F6F6 FF00 0000"
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FF"
};

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"Calc Apple Computer, Inc. 1988-95"
};

resource 'FREF' (128,
#if qNames
"Calc Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (129,
#if qNames
"Calc Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'FREF' (130,
#if qNames
"Calc Stationery",
#endif
	purgeable) {
	kStationery,
	2,
	""
};

resource 'BNDL' (128,
#if qNames
"Calc",
#endif
	purgeable) {
	kSignature,
	0,
	{ /* array TypeArray: 2 elements */
		/* [1] */
		'ICN#',
		{ /* array IDArray: 2 elements */
			/* [1] */
			0, 128,
			/* [2] */
			1, 129,
			2, 130
		},
		/* [2] */
		'FREF',
		{ /* array IDArray: 2 elements */
			/* [1] */
			0, 128,
			/* [2] */
			1, 129,
			2, 130
		}
	}
};

// Get the default Version resources
/*--------------------------------------------------------------------------------
 The revision of this particular file
--------------------------------------------------------------------------------*/
resource 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x01,
	0x01,
	development,
	0x01,
	verUs,
	"1.1d1",
	"Calc 1.1d1, Apple Computer, Inc. 1988-97"
};
include "Defaults.rsrc"  'vers' (2);		// Overall package

/********************************************************************************/
/*	End of File: Calc.r															*/
/********************************************************************************/