//----------------------------------------------------------------------------------------
// CalcUtilities.cp
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CALCUTILITIES__
#include "CalcUtilities.h"
#endif

// MacApp

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __USYNCHSCROLLER__
#include "USynchScroller.h"
#endif

// Toolbox

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __FP__
#include <fp.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// ANSI

#ifndef __STRING__
#include <string.h>
#endif

//----------------------------------------------------------------------------------------
// FitString:  Truncates theString to fit in maxWidth pixels 
//----------------------------------------------------------------------------------------


void FitString(	CStr255_AC& theString,
				short maxWidth)
{
	TruncString(maxWidth, theString, truncEnd);
}


//----------------------------------------------------------------------------------------
// GetRgnBounds: 
//----------------------------------------------------------------------------------------


CRect_AC GetRgnBounds(RgnHandle aRgn)
{
	CRect_AC aRect((*aRgn)->rgnBBox);

	--aRect[botRight];

	return aRect;
}


//----------------------------------------------------------------------------------------
// IsDigit: 
//----------------------------------------------------------------------------------------


bool IsDigit(char Ch)
{
	return (Ch >= '0') && (Ch <= '9');
}


//----------------------------------------------------------------------------------------
// SetEditCommandName: Sets the Edit menu command name to the custom name, e.g. 'Copy
// Cells'.
//----------------------------------------------------------------------------------------


void SetEditCommandName(CommandNumber theCommand, 
						CommandNumber customCommand)
{
	SetCommandName(theCommand, CommandToName(customCommand));
}


//----------------------------------------------------------------------------------------
// SetTheFont: 
//----------------------------------------------------------------------------------------


void SetTheFont(short fontNumber,
				short fontSize,
				Style fontStyle)

{
	TextFont(fontNumber);
	TextSize(fontSize);
	TextFace(fontStyle);
}

//----------------------------------------------------------------------------------------
// SmartDrawString: Draws the CPascalStr_AC at the [h,v] coordinate, taking into account the
// width and justification.
//----------------------------------------------------------------------------------------


void SmartDrawString(CStr255_AC& theString,
					 CRect_AC area,
					 short justification)
{
	FitString(theString, area.right - area.left);
	MADrawString(theString, area, justification);
}


//----------------------------------------------------------------------------------------
// ReadScrap: Utility for extracting data from the scrap, where scrapOffset indicates the
// start of the data within the scrap handle, and dataLength is the length of the data to
// be extracted. dataLength is added to scrapOffset.
//----------------------------------------------------------------------------------------


void ReadScrap(Handle theScrap,
			   long& scrapOffset,
			   void* theData,
			   short dataLength)
{
	BlockMove_AC(*theScrap + scrapOffset, theData, dataLength);
	scrapOffset += dataLength;
}

//----------------------------------------------------------------------------------------
// WriteScrap:  Utility for appending data to the end of the scrap handle 
//----------------------------------------------------------------------------------------


void WriteScrap(Handle theScrap,
				long& scrapOffset,
				void* theData,
				short dataLength)
{
	SetHandleSize(theScrap, scrapOffset + dataLength);
	::ThrowIfMemError_AC();
	BlockMove_AC(theData, *theScrap + scrapOffset, dataLength);
	scrapOffset += dataLength;
}

//----------------------------------------------------------------------------------------
// End of CalcUtilities.cp

