//----------------------------------------------------------------------------------------
// MCalc.cp 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// Calc

#ifndef __UCALC__
#include "UCalc.h"
#endif

// MacApp

#ifndef __UBACKGROUNDADORNER__
#include "UBackgroundAdorner.h"
#endif

#ifndef __UBUSYCURSOR__
#include "UBusyCursor.h"
#endif

#ifndef __UEDITIONDOCUMENT__
#include "UEditionDocument.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UDIALOG__
#include "UDialog.h"
#endif

#ifndef __UGRIDVIEW__
#include "UGridView.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

// Toolbox

#ifndef __FP__
#include <fp.h>
#endif


//========================================================================================
// Constants
//========================================================================================
enum
{
	phSplash = 1001 						// Application's splash screen 
};

//========================================================================================
// Global Functions
//========================================================================================
static void InitWithSplashScreen();

//----------------------------------------------------------------------------------------
// InitWithSplashScreen: Display a splash screen while doing MacApp initialization.
//----------------------------------------------------------------------------------------
void InitWithSplashScreen()
{
	// To avoid heap fragmentation, we allocate space for the DialogRecord on the stack,
    // as a local variable in this procedure, so that the dialog record WON'T be allocated
    // as a non-relocatable block at the bottom of the heap. If we were to pass NULL to
    // GetNewCenteredDialog for the dStorage, then the call to GetNewDialog would allocate
    // the dialog record as a non-relocatable block at the bottom of the heap leading to
    // heap fragmentation during InitUMacApp_Step3's call to MoreMasters.

	DialogRecord		theDialogRecord;				// allocated on the stack 

	if (GetNewCenteredDialog(phSplash, (Ptr) &theDialogRecord, (WindowRef) (-1)))
		DrawDialog((GrafPort*) &theDialogRecord);		// Show splash screen 

	// Continue with remainder of initialization 
	
	InitUMacApp_Step3(16);								// Initialize memory (formerly InitUMacApp)

	TBusyCursor::InitUBusyCursor();						// Optionally, install the auto busy cursor

	InitUEditionDocument();
	InitUTEView();

	InitUGridView(); 
	InitUPrinting();

	InitUBackgroundAdorner();

	CloseDialog((DialogRef) &theDialogRecord);			// Remember to remove the splash screen
	DisposeIfHandle_AC(theDialogRecord.items);
}
		
//----------------------------------------------------------------------------------------
// main: 
//----------------------------------------------------------------------------------------
void main()
{
	InitUMacApp_Step1();								// calls InitToolBox, ValidateConfiguration

	PullApplicationToFront(); 							// Pull app to front under MultiFinder 

	InitWithSplashScreen();
	
	TCalcApplication aCalcApplication;
	aCalcApplication.Run();
}

//----------------------------------------------------------------------------------------
// End of MCalc.cp

