// UCalc.h 
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.

// This MacApp sample implements a simple spreadsheet 
// This version, created Christmas Day 1989 by Larry Tesler, uses
// classes TPrimaryScroller and TSecondaryScroller in USynchScroller
// [instead of defining its own TCalcScroller and four AutoScroll methods]
// to achieve synchronous scrolling [instead of near-synchronous scrolling]
// and to enable auto-scrolling during column resizing.

#ifndef __UCALC__
#define __UCALC__

// Calc

#ifndef __CALCUTILITIES__
#include "CalcUtilities.h"
#endif

#ifndef __UCALCCELLS__
#include "UCalcCells.h"
#endif

#ifndef __UCALCROWS__
#include "UCalcRows.h" 
#endif

#ifndef __UCALCCOLUMNS__
#include "UCalcColumns.h"
#endif

// ACS
#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __UASSOCIATION__
#include "UAssociation.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UEDITIONDOCUMENT__
#include "UEditionDocument.h"
#endif

#ifndef __UEVENT__
#include "UEvent.h"
#endif

#ifndef __UEVENTHANDLER__
#include "UEventHandler.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include "UFileBasedDocument.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UGRABBERTRACKER__
#include "UGrabberTracker.h"
#endif

#ifndef __UGRIDVIEW__
#include "UGridView.h"
#endif

#ifndef __UMACGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USECTION__
#include "USection.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __USYNCHSCROLLER__
#include "USynchScroller.h"
#endif

#ifndef __UTECOMMANDS__
#include "UTECommands.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#if qALOE
#	ifndef __UODAdaptable__
#	include "UODAdaptable.h"
#	endif
#endif

// Toolbox

#ifndef __ALIASES__
#include <Aliases.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __CONTROLS__
#include <Controls.h>
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __FP__
#include <fp.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __SEGLOAD__
#include <SegLoad.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// Use Windows_AC.h instead of <windows.h> for XPlatform compatability.
#ifndef __Windows_AC__
#include "Windows_AC.h"
#endif

class TRow;
class TRowsView;
class TRowList;
class CRowIterator;
class TColumn;
class TColumnsView;
class TColumnList;
class CColumnIterator;
class TCell;
class TCellList;

enum
{
	kMaxRows = 120,								// Maximum row number (arbitrary)
	kMaxColumns = 64,							// Maximum column number (arbitrary)
	kMaxValueLength = 24						// Max. length of a value as a CPascalStr_AC 
};

enum
{
	kSignature = 'SS12',						// Application signature. 
	kFileType = 'SF12'							// Data file type. 
};

enum
{
	kCalcWindowType = 1001,						// Resource id of the spreadsheet window 
	kGraphWindowType = 1002						// Resource id of the Graph window
};

typedef decform ValueFormat;					// Have SANE display numeric values in decimal

enum TypeOfLine
{
	kNoLine,									// for DrawLine routine
	kSolidLine,
	kBoldLine,
	kVDottedLine,
	kHDottedLine,
	kRowSeparator,
	kColumnSeparator
};

//============================================================================================
//============================================================================================
enum
{
	kTextScrapType = kTextType_AC,
	kCalcScrapType = 'CALC',

	kCellFont = applFont,
	kCellFontSize = 10,
	kRowTitleWidth = 32,						// width of row titles 
	kColumnTitleHeight = 20,					// height of column titles 
	kTitlesFont = applFont,						// font for column/row titles 
	kTitlesFontSize = 10,						// font size for column/row titles 

	kEntryFont = applFont,						// default font of cell contents 
	kEntryFontSize = 10,						// default font size of cell contents 
	kEntryHeight = 20,							// height of the cell entry view 

	kDefaultJustification = teCenter,			// default cell justification 
	kNoJustification = 2,						// constant representing no justification 

	kValuePrecision = 18,						// digits of precision in our values 
	kTELength = (80 * 5),						// number of characters in the text edit field 

	kCellsViewHelp = 128,
	kColsViewHelp = kCellsViewHelp,
	kRowsViewHelp = kCellsViewHelp

};
const bool kForceAutomatic = true;				// force automatic calculation 
const bool kUpdateDependencies = true;			// set cell's dependents 

const CalcMode kCalcMode = kAutomatic;			// default calculation mode 

// Command Constants 
enum
{
	cRecalculate = 401,
	cAutoCalc = 403,
	cManualCalc = 404,
	cGraphSelection = 405,						// Used for ALOE graphing
	cGeneral = 501,
	cNoDecimal = 503,
	cDecimal = 504,
	cScientific = 505,
	cSystemJustify = 506,
	cForceLeftJustify = 507,
	cRightJustify = 508,
	cCenter = 509,
	cSelecting = 1000,
	cSizeColumn = 1001,
	cCutText = 1103,
	cCopyText = 1104,
	cClearText = 1106,
	cCutCells = 1203,
	cCopyCells = 1204,
	cClearCells = 1206,
	cStandardCut = 1303,
	cStandardCopy = 1304,
	cStandardClear = 1306
};

enum
{
	mWidthChanged = 1002,
	mValueChanged = 1003
};

// Cursor Resource ID's 

enum
{
	kColumnSizingCursor = 256,					// cursor for resizing a column 
	kRowSizingCursor = 257						// cursor for resizing a row--UNUSED 
};

enum
{
	mWindowMenu = 42							// Window menu
};

//--------------------------------------------------------------------------------------------------
GridCell ReadCellCoordinate(TFile* aFile);


//--------------------------------------------------------------------------------------------------
// TCalcApplication:
//--------------------------------------------------------------------------------------------------
class TCalcApplication : public TApplication
{
public:
	TCalcApplication();
	// Constructor 

	virtual ~TCalcApplication();
	// Destructor

	virtual void AboutToLoseControl(bool convertClipboard);// override 
	// Override to reset Edit menu items for incoming DAs 

	virtual TDocument* DoMakeDocument(CommandNumber itsCommandNumber,
									  TFile* itsFile);// override 
	// Launches a TCalcDocument 

	virtual TView* MakeViewForAlienClipboard();	// override 
	// Handle data that was left in the Clipboard by forces outside the application 
};


//--------------------------------------------------------------------------------------------------
// Forward class declaration.
//--------------------------------------------------------------------------------------------------
class TCellsView;
class TEntryView;
class TCoordView;


//--------------------------------------------------------------------------------------------------
// TCalcDocument:
//--------------------------------------------------------------------------------------------------
class TCalcDocument : public TEditionDocument
#if qALOE
, public MODAdaptable
#endif
{
	MA_DECLARE_CLONE;

public:
	//------------------------------------------------------------------------------------
	// Initialization and termination
	//------------------------------------------------------------------------------------

	TCalcDocument(TFile* itsFile = NULL,
							   const GridRect& dimensions = GridRect(gZeroRect));
	// Put the document in a safely free-able state 

	virtual ~TCalcDocument();					// override 
	// Free the document 

	virtual void DoInitialState();				// override 
	// Initialize allocation counts 

	virtual void DoMakeViews(bool forPrinting);// override 
	// Launch the views which are seen in the document's window 

	virtual void DoToPublisher(TCell* theCell,
							   TPublisher* aPublisher);

	virtual void FreeData();					// override 
	// Delete the cells, rows, and columns 

	//------------------------------------------------------------------------------------
	// Filing OVERRIDES 
	//------------------------------------------------------------------------------------

	virtual void DoNeedDiskSpace(TFile* itsFile,
								 long& dataForkBytes,
								 long& rsrcForkBytes);// override 
	// Tells how many bytes of disk space will be required to store the
	// data for the document in a file on disk

#if qALOE
	virtual bool IsChanged(); // override
	// Returns wwhether changes have been made to our content or to embedded ALOE graphs
#endif
	
	virtual void DoRead(TFile* aFile,
						bool forPrinting);	// override 
	// Reads in the data from the disk, when a document is Opened or Reverted 

	virtual void DoWrite(TFile* aFile,
						 bool makingCopy);	// override 
	// Writes the data to the disk, when a document is Saved 

	//------------------------------------------------------------------------------------
	// Menu handling 
	//------------------------------------------------------------------------------------
	virtual void DoSetupMenus();				// override 
	// Enable menu commands 

	virtual void DoMenuCommand(CommandNumber aCommandNumber);// override 
	// Handle menu commands 

	//------------------------------------------------------------------------------------
	// Cell management 
	//------------------------------------------------------------------------------------
	virtual void AddCell(TCell* theCell,
						 GridCell itsLoc);
	// Add the cell to the document 

	virtual bool CellExists(GridCell itsLoc);
	// Is there a cell object at these coordinates? 

	virtual void DeleteCell(GridCell itsLoc);
	// Mark the cell at the given coordinates as deleted 

	virtual bool IsExistingCell(TCell* aCell);

	virtual void EditCell();
	// Change the formula of the cell being edited to the CPascalStr_AC in the entry view 

	virtual void FreeCell(TCell* theCell);
	// Free the specified cell object 

	virtual void FreeDeletedCells();
	// Free each cell object that is marked deleted 

	virtual TCell* GetCell(GridCell cellLocation);
	// Return the cell object for the given coordinates. 

	virtual TCell* GetExistingCell(GridCell itsLoc);
	// Return the cell object or NULL if no cell is allocated for the given coordinates. 

	virtual bool IsAutoCalc();
	// Evaluates fCalcMode == kAutomatic 

	virtual void SetEntry(GridCell itsLoc);
	// Set the text in the entry view to the formula in the cell at the given
	// coordinates

	virtual void UndeleteCell(GridCell itsLoc);
	// If there is a cell at the given coordinates, mark it undeleted 

	//------------------------------------------------------------------------------------
	// Row management 
	//------------------------------------------------------------------------------------
	virtual void AddRow(TRow* theRow);
	// Add the row to the document 

	virtual TRow* GetRow(GridCoordinate r);

	virtual bool RowExists(GridCoordinate r);

	//------------------------------------------------------------------------------------
	// Column management 
	//------------------------------------------------------------------------------------
	virtual void AddColumn(TColumn* theColumn);
	// Add the column to the document 

	virtual TColumn* GetColumn(GridCoordinate c);

	virtual bool ColumnExists(GridCoordinate c);

	//------------------------------------------------------------------------------------
	// Ranges of cells 
	//------------------------------------------------------------------------------------
	virtual bool CellInRange(GridCoordinate r,
								GridCoordinate c,
								const GridRect& range);
	// Do the given coordinates fall within the range? 

	virtual void ConstrainToUsedCells(GridRect& cellRange);
	// Shrink the range of cells to exclude unallocated cells, if possible 

	virtual void DoRecalculate(bool forceAutomatic,
							   bool setDependents);
	// Recalculate all cells if fCalcMode == kAutomatic, otherwise just selected cells 

	//------------------------------------------------------------------------------------
	// Edition Manager Support 
	//------------------------------------------------------------------------------------
	virtual void SetUserSelection(TDesignator* newSelection);
	// Sets the designator for the user selection (if any).

	virtual void UserSelectionChanged(TView* changedView);// override 

	virtual void DoWriteData(const OSType aScrapType,
							 TDesignator* aDesignator,
							 CStream_AC* aStream);	// override 
	// Given a designator and a stream, writes the designated data to the stream. 

	virtual void DoWritePrivateTypes(TDesignator* aDesignator,
									 CStream_AC* aStream);
	// called by DoWriteData if aScrapType == kCalcScrapType 

	virtual void DoReadData(const OSType aScrapType,
							TDesignator* aDesignator,
							CStream_AC* aStream,
							long count);		// override 
	// Given a designator and a stream, read the designated data from the stream. 

	virtual void DoReadPrivateTypes(TDesignator* aDesignator,
									CStream_AC* aStream);
	// called by DoReadData if aScrapType == kCalcScrapType 

	virtual void DoReadTEXT(TDesignator* aDesignator,
							CStream_AC* aStream,
							long length);
	// called by DoReadData if aScrapType == 'TEXT' 

	void AddPublisherDependency(TSection* aSection);
	// Create a dependency between a publisher and its published cells.

	void RemovePublisherDependency(TSection* aSection);
	// Remove the dependency between a publisher and its published cells.

	virtual void DoPostMakeViews(bool forPrinting);
	// Iterate over the sections and create the dependencies between a publishers
	// and its published cells.

	virtual void DoAddBorder(TSection* aSection);
	// creates a border for the given section

	virtual void DoDeleteBorder(TSection* aSection);
	// deletes a border for the given section

	virtual void RevealSelection(TDesignator* aSelection);

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	GridRect fDimensions;						// the document's dimensions
	GridRect fInUseBounds;						// the area of the document that is in use 

	TCellList fCells;
	TRowList fRows;
	TColumnList fColumns;

	CStandardDependencySpace_AC fDependencies;

	TCell* fEditCell;							// the cell being edited (the current selection)

	TCellsView* fCellsView;						// the view of the cells themselves 
	TRowsView* fRowsView;						// the view of the row numbers 
	TColumnsView* fColumnsView;					// the view of the column letters 
	TEntryView* fEntryView;						// the view for entering/editing cell values
	TCoordView* fCoordView;						// the view that shows the selected cell's coordinates

	GridCell fOffset;

	GridCoordinate fAllocatedCells;				// the number of allocated cells
	GridCoordinate fAllocatedRows;				// the number of allocated rows
	GridCoordinate fAllocatedColumns;			// the number of allocated columns 
	GridCoordinate fNoOfRows;					// maximum number of rows 
	GridCoordinate fNoOfColumns;				// maximum number of columns

	GridCell fEditLocation;						// the location of the cell being edited 

	CalcMode fCalcMode;							// Manual or Automatic recalculation 

	TypeOfSelection fSelectionType;

	bool fColumnIsSelected;					// a column has been selected 
};

//--------------------------------------------------------------------------------------------------
// TCalcWindow:
//--------------------------------------------------------------------------------------------------
class TCalcWindow : public TWindow
{
	MA_DECLARE_CLONE;

public:
	virtual~ TCalcWindow();
	// Destructor

	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void Draw(const CViewRect& area);		// override 
};


//--------------------------------------------------------------------------------------------------
// TCellsView is the view used to show the cell values. 
//--------------------------------------------------------------------------------------------------
class TCellsView : public TTextGridView
{
	MA_DECLARE_CLONE;

public:
	TCellsView(bool forClipboard = false);

	virtual~ TCellsView();
	// Destructor

	virtual void CreatePublisherAdorner(TPublisher* aPublisher);

	virtual void CreateSubscriberAdorner(TSubscriber* aSubscriber);

	virtual void ComputeBorderRgn(RgnHandle itsBorderRegion,
								  const TGridViewDesignator* itsDesignation);

	//--------------------------------------------------------------------------------------------------
	// Initialization and termination 
	//--------------------------------------------------------------------------------------------------
	virtual void DoPostCreate(TDocument* itsDocument);

	//--------------------------------------------------------------------------------------------------
	// Commands and menus 
	//--------------------------------------------------------------------------------------------------
	virtual void DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* event,
								CPoint_AC hysteresis);// override 

	virtual void DoSetupMenus();				// override 

	virtual void DoMenuCommand(CommandNumber aCommandNumber);// override 
	// Handle menu commands 

	virtual void DoKeyEvent(TToolboxEvent* event);// override 

	//--------------------------------------------------------------------------------------------------
	// Display and Selection 
	//--------------------------------------------------------------------------------------------------
	virtual void AdornCol(GridCoordinate aCol,
						  const CViewRect& area);	// override 
	// Draw a column delimiter ("adornment") in the given area 

	virtual void AdornRow(GridCoordinate aCol,
						  const CViewRect& area);	// override 
	// Draw a row delimiter in the given area 

	virtual Handle DesignatorAsTEXT(TDesignator* aDesignator);

	virtual bool DoIdle(IdlePhase phase);	// override 
	// Checks the state of the option key and decides whether to make the cursor
	// need to be reset.  We used to use gAlwaysTrackCursor, but this is better.

	virtual void DoSetCursor(const CViewPoint& localPoint,
							 RgnHandle cursorRegion);// override 

	virtual CStr255_AC GetText(const GridCell& aCell) const;	// override 

	virtual void DrawCell(const GridCell& aCell,
						  const CViewRect& aRect);	// override 
	// Draw the cell's value, using the proper justification 

	virtual GridRect GetVisibleCells();
	// Return a rectangle giving the cells that are visible 

	virtual bool IsCellVisible(const GridCell& aCell);
	// Tell whether the specified cell is visible or not 

	virtual void PositionAtCell(const GridCell& aCell);
	// Redraw the views so that aCell is at the top left corner of the grid 

	virtual void ReSelect(const TGridViewDesignator* cellRegion);
	// Set the current selection to the given cells 

	virtual void ReSelectCell(const GridCell& aCell);
	// Set the current selection to the given cell 

	virtual void ScrollSelectionIntoView(bool redraw);// override 
	// Scroll so that at least the top left cell of the selection is visible 

	virtual void SetCell(const GridCell& aCell);

	virtual void SetColWidth(GridCoordinate aCol,
							 GridCoordinate numOfCols,
							 ViewCoordinate aWidth);		// override 

	virtual void SetSelection(const TGridViewDesignator* cellsToSelect,
							  bool extendSelection,
							  bool highlight,
							  bool select);	// override 
	// Set the current selections to the specified cells. Adjust the entry and
	// coordinate views accordingly.

	virtual void UserSelectionChanged(TView* changedView);
	// invalidate all publisher/subscriber adorners when the user selection changes

	//--------------------------------------------------------------------------------------------------
	// Printing OVERRIDES 
	//--------------------------------------------------------------------------------------------------
	virtual ViewCoordinate DoBreakFollowing(VHSelect vhs,
										 ViewCoordinate previousBreak,
										 bool& Automatic);// override 

	virtual CViewRect GetPrintExtent();// override 
	// Return the area to print, depending on whether the command was Print Selection
	// or Print.

	//--------------------------------------------------------------------------------------------------
	// Clipboard handling 
	//--------------------------------------------------------------------------------------------------
	virtual bool ContainsClipType(ResType aType);	// override 

	virtual long GivePasteData(Handle aDataHandle, ResType dataType);	// override

	virtual void WriteCalcScrap(Handle calcScrap);
	virtual void WriteTextScrap(Handle textScrap);

	virtual void WriteToDeskScrap();			// override 

	void UpdateFromDocument();
	// update view data to match document (number of rows, columns, etc)

#if qDebug
	virtual void DoDrawPageBreak(VHSelect vhs,
								 short whichBreak,
								 ViewCoordinate loc,
								 bool Automatic);// override 
	// For debugging printing only, draw lines showing the page breaks. 
#endif

#if qALOE
	virtual void GraphSelection();
		// ALOE example!  Calc connects with ALOE here and then uses a Graph tool to graph the selection.
		// Compare with SimpleContainer which connects at startup time.
#endif

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TCalcDocument*	fCalcDocument;
	bool			fLastOptionKey;						// true if option key down at last idle.
	// helps in tracking modifier keys for views that care. 
};


//--------------------------------------------------------------------------------------------------
// TCalcPrintHandler 
//--------------------------------------------------------------------------------------------------
class TCalcPrintHandler : public TStdPrintHandler
{
	MA_DECLARE_CLONE;

public:
	inline TCalcPrintHandler(TDocument* itsDocument = NULL,
							 TView* itsView = NULL,
							 bool itsHFixedSize = kFixedSize,
							 bool itsVFixedSize = kFixedSize) : 
			TStdPrintHandler(itsDocument, itsView, itsHFixedSize, itsVFixedSize) { }

	virtual~ TCalcPrintHandler();
	// Destructor

	virtual CViewPoint CalcViewPerPage();// override 

	virtual void DoMenuCommand(CommandNumber aCommandNumber);// override 
	// Overridden to handle Print Selection. 

	virtual void DoSetupMenus();				// override 
	// Overridden to handle Print Selection. 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	CommandNumber fCmdNumber;
};


//--------------------------------------------------------------------------------------------------
// TEntryView uses TextEdit to display and edit the selected cell's formula. 
//--------------------------------------------------------------------------------------------------
class TEntryView : public TTEView
{
	MA_DECLARE_CLONE;

public:

	virtual~ TEntryView();
	// Destructor

	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void DoKeyEvent(TToolboxEvent* event);// override 

	virtual TTETypingCommand* DoMakeTypingCommand(const CStr2_AC& ch);// override 
	// Actually returns a TCalcTypingCommand 

	virtual void DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* event,
								CPoint_AC hysteresis);// override 

	virtual void DoMenuCommand(CommandNumber aCommandNumber);// override

	virtual void DoSetupMenus();				// override 

	//	virtual void SetupUndoMenu(); // override

	virtual void Draw(const CViewRect& area);		// override 

	virtual bool WantsToBeTarget();			// override 

	virtual void EditMode(bool editing);
	// Enter or leave TextEdit mode.  In TextEdit mode (fTEditing==true),
	// the Undo command causes an undo of the latest TextEdit action;
	// in non-TextEdit mode, Undo brings back the previous CPascalStr_AC (fOldString)

	virtual CStr255_AC GetAsString();
	// Return the current text as a CPascalStr_AC 

	virtual void SetActive(bool state);		// Override 

	virtual void SetEditMode();
	// Enter TextEdit mode and clear the current text 

	virtual void SetToString(const CStr255_AC& theString);
	// Set the current text to theString 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TCalcDocument* fCalcDocument;

	bool fTouched;
	bool fTEditing;								// true: TextEdit mode 
	bool fFirstEdit;							// true: undo to previous CPascalStr_AC 

};

//--------------------------------------------------------------------------------------------------
// TCoordView is the view used to display the coordinates of the cell which is currently selected.
//--------------------------------------------------------------------------------------------------
class TCoordView : public TView
{
	MA_DECLARE_CLONE;

public:

	virtual~ TCoordView();
	// Destructor

	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void Draw(const CViewRect& area);		// override 
	// Draws the view seen in the window.  Every nonblank view MUST override this method 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TCalcDocument* fCalcDocument;
};

//--------------------------------------------------------------------------------------------------
// TCalcSelectCommand is a command object created to handle mouse movement when you
// click in a TCellsView.
//--------------------------------------------------------------------------------------------------
class TCalcSelectCommand : public TCellSelectCommand
{
public:
	TCalcSelectCommand(TCalcDocument* itsDocument = NULL,
									TGridView* itsView = NULL,
									const CViewPoint& theMouse = gZeroVPt,
									bool theShiftKey = false,
									bool theCommandKey = false);

	virtual ~TCalcSelectCommand();
	// Destructor

	virtual void ComputeNewSelection(const GridCell& clickedCell);// override 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TCalcDocument* fCalcDocument;
};

//--------------------------------------------------------------------------------------------------
// TRowSelector: TRowSelector is a command object created to handle mouse movement when
// you click in a TRowsView.
//--------------------------------------------------------------------------------------------------
class TRowSelector : public TCalcSelectCommand
{
public:
	TRowSelector(TCalcDocument* itsDocument = NULL,
									TGridView* itsView = NULL,
									const CViewPoint& theMouse = gZeroVPt,
									bool theShiftKey = false,
									bool theCommandKey = false);
	// Construct a TCalcSelectCommand to handle selection in the cells view in parallel
	//  with selection in our view, the rows view


	virtual ~TRowSelector();
	// Dispose of our cell selector 

	virtual void ComputeAnchorCell(const GridCell& clickedCell);// override 

	virtual void ComputeNewSelection(const GridCell& clickedCell);// override 

	virtual void DoIt();						// override 
	// Perform these operations for the cells view as well as for the rows view 

	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
								 CViewPoint& anchorPoint,
								 CViewPoint& previousPoint,
								 CViewPoint& nextPoint,
								 bool mouseDidMove);// override 

public:
	TCalcSelectCommand fCellSelector;			// command to handle cells view selection 
};

//--------------------------------------------------------------------------------------------------
// TColumnSelector: TColumnSelector is a command object created to handle mouse movement
// when you click in a TColumnsView.
//--------------------------------------------------------------------------------------------------
class TColumnSelector : public TCalcSelectCommand
{
public:
	TColumnSelector(TCalcDocument* itsDocument = NULL,
									TGridView* itsView = NULL,
									const CViewPoint& theMouse = gZeroVPt,
									bool theShiftKey = false,
									bool theCommandKey = false);			// override 

	// Create a TCalcSelectCommand to handle selection in the cells view in parallel with
	// selection in our view, the column headers

	virtual ~TColumnSelector();				// override 
	// Dispose of our cell selector 

	virtual void ComputeAnchorCell(const GridCell& clickedCell);// override 
	virtual void ComputeNewSelection(const GridCell& clickedCell);// override 
	virtual void DoIt();				// override 
	// Perform these operations for the cells view as well as for the column headers 

	virtual TTracker* TrackMouse(TrackPhase	aTrackPhase,
										CViewPoint&		/*anchorPoint*/,
										CViewPoint&		/*previousPoint*/,
										CViewPoint&		nextPoint,
										bool		mouseDidMove);// override 

public:
	TCalcSelectCommand fCellSelector;			// command to handle cells view selection 
};												// TColumnSelector 

//--------------------------------------------------------------------------------------------------
// TCalcTypingCommand: We define our own typing command in order to remember which cell the
// typing was performed in. Then if the user makes another selection we can undo/redo the
// correct cell.
//--------------------------------------------------------------------------------------------------
class TCalcTypingCommand : public TTETypingCommand
{
public:
	TCalcTypingCommand(TEntryView* inEntryView = NULL,
											const CStr2_AC& itsFirstChar = gEmptyString);// override 
	// Construct the command object. Save the current cell selection for later Undo/Redo

	virtual ~TCalcTypingCommand();
	// Destructor

	virtual void AddCharacter(const CStr2_AC& ch);		// override 
	// Overridden to keep text visible when it overflows the entry view's box 

	virtual void UndoIt();						// override 
	// Undo the typing in the target cell 

	virtual void RedoIt();						// override 
	// Redo the typing in the target cell 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TEntryView* fEntryView;
	TCellsView* fCellsView;

	GridCell fTargetCell;						// cell selected when command was created 
};

//----------------------------------------------------------------------------------------
// Globals defined by this unit
//----------------------------------------------------------------------------------------
extern ValueFormat gGeneralFormat;				// SANE conversion formats 
extern ValueFormat gDecimalFormat;
extern ValueFormat gNoDecimalFormat;
extern ValueFormat gScientificFormat;
extern ValueFormat gNoFormat;
extern FormatRecord gDefaultFormat;				// default cell format 

extern Pattern gColumnSeparatorPattern;			// patterns for separator lines 
extern Pattern gRowSeparatorPattern;

#endif
