//----------------------------------------------------------------------------------------
// UCalcColumns.cp
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UCALCCOLUMNS__
#include "UCalcColumns.h"
#endif

// Calc

#ifndef __UCALCROWS__
#include "UCalcRows.h"
#endif

#ifndef __CALCUTILITIES__
#include "CalcUtilities.h"
#endif

// ACS

#if qGrayscaleAppearance
#	ifndef __AGAImaging_AC__
#	include "AGAImaging_AC.h"
#	endif
#endif

// MacApp

#ifndef __UCALC__
#include "UCalc.h"
#endif

// Toolbox

// ANSI

#ifndef __STDLIB__
#include <stdlib.h>
#endif


//========================================================================================
// CLASS TColumnsView
//========================================================================================
MA_DEFINE_CLASS(TColumnsView);

//----------------------------------------------------------------------------------------
// TColumnsView::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TColumnsView::DoPostCreate(TDocument*	itsDocument)	// override 

{
	fCalcDocument = (TCalcDocument *)(itsDocument);
	SetRowHeight(1, fNumOfRows, kCellHeight + 1);
	fCalcDocument->fColumnIsSelected = false;
	fHelpID = kColsViewHelp;
	fHelpIndex = 3;
}

//----------------------------------------------------------------------------------------
// TColumnsView destructor
//----------------------------------------------------------------------------------------


TColumnsView::~TColumnsView()
{
}

//----------------------------------------------------------------------------------------
// TColumnsView::AdornCol: 
//----------------------------------------------------------------------------------------


void TColumnsView::AdornCol(GridCoordinate		/*aCol*/,
								   const CViewRect& area)// override 
{
	PenPat(&qd.black);
	PenSize(1, 1);

	CRect_AC qdArea(ViewToQD(area));
	--qdArea[botRight];

	// top line 
	MoveToPt(qdArea[topLeft]);
	LineTo(qdArea.right, qdArea.top);

	// bottom line
	MoveTo(qdArea.left, qdArea.bottom);
	LineToPt(qdArea[botRight]);

	// right line 
	MoveTo(qdArea.right, qdArea.top);
	LineToPt(qdArea[botRight]);
}


//----------------------------------------------------------------------------------------
// TColumnsView::CoordToString: 
//----------------------------------------------------------------------------------------


void TColumnsView::CoordToString(GridCoordinate	coord,
										CStr255_AC& theString)
{
	theString.Empty();
	coord = coord - 1;
	if (coord < 26)
		theString += ('A' + coord);
	else
	{
		theString += ('A' + (coord / 26) - 1);
		theString += ('A' + (coord % 26));
	}
}


//----------------------------------------------------------------------------------------
// TColumnsView::DoMouseCommand: 
//----------------------------------------------------------------------------------------


void TColumnsView::DoMouseCommand(CViewPoint&		theMouse,
										 TToolboxEvent* event,
										 CPoint_AC			/*hysteresis*/)// override 
{
	GridCell			aCell(-1, -1);

	GridViewPart whichPart = IdentifyPoint(theMouse, aCell);	//since almost 

	switch (whichPart)
	{
		case inCell:
			fCalcDocument->fColumnIsSelected = true;
			PostCommand(TH_new TColumnSelector(fCalcDocument, this, theMouse, event->IsShiftKeyPressed(), event->IsCommandKeyPressed()));
			fCalcDocument->fSelectionType = kColumnSelection;
			break;

		case inColumn:
		case inVertex:
			if (aCell.h > 1)
				PostCommand(TH_new TColumnSizer(fCalcDocument, aCell.h - 1, theMouse));
			break;
	}
	fCalcDocument->fRowsView->SetEmptySelection(kHighlight);
}


//----------------------------------------------------------------------------------------
// TColumnsView::DoSetCursor: 
//----------------------------------------------------------------------------------------


void TColumnsView::DoSetCursor(const CViewPoint&	globalMouse,
									  RgnHandle		cursorRegion)// override 

{
	GridCell		aCell(-1, -1);						// This value should never be used

	GridViewPart aGridViewPart = IdentifyPoint(globalMouse, aCell);

	switch (aGridViewPart)						//since almost 
	{
		case badChoice:
			break;

		case inColumn:
		case inVertex:
			if (aCell.h > 1)
			{
				SetCursor(*GetCursor(kColumnSizingCursor));
				CViewRect cellsExtent;
				ColToVRect(Min_AC(aCell.h, fNumOfCols), 1, cellsExtent);
				CRect_AC columnQDExtent = ViewToQD(cellsExtent);

				// Which edge is the mouse closer to? 
				if (labs(columnQDExtent.right - globalMouse.h) < labs(columnQDExtent.left - globalMouse.h))
					columnQDExtent.left = columnQDExtent.right;

				columnQDExtent.left = columnQDExtent.left - fColInset / 2;
				columnQDExtent.right = columnQDExtent.left + fColInset;
				RectRgn(cursorRegion, columnQDExtent);
			}
			break;

		case inRow:
		case inCell:
			SetCursor(&(qd.arrow));
			CViewRect cellsExtent;
			ColToVRect(Min_AC(aCell.h, fNumOfCols), 1, cellsExtent);
			cellsExtent.Inset(fColInset / 2, 0);// Account for the column resizer 
			RectRgn(cursorRegion, &ViewToQD(cellsExtent));
			break;
	}
}


//----------------------------------------------------------------------------------------
// TColumnsView::DrawCell: 
//----------------------------------------------------------------------------------------


void TColumnsView::DrawCell(const GridCell&		aCell,
								   const CViewRect&	aRect)// override 

{
	CStr255_AC	theString;
	CoordToString(aCell.h, theString);

	CRect_AC qdArea(ViewToQD(aRect));

#if qGrayscaleAppearance
	CRect_AC bgRect(qdArea);
	// Undo the inset, then inset by an appropriate amount
	CRect_AC 	insetRect(fColInset / 2, fRowInset / 2,
					  -(fColInset / 2), -(fRowInset / 2));
	bgRect -= insetRect;
	bgRect -= CPoint_AC(1, 1);
	++bgRect[topLeft];
	++bgRect.top; // we have a top line adornment
	AGABackgroundPaint(bgRect, true, kRaisedModelessBackground, true, false);
#endif

	qdArea.top = qdArea.top + 3;				// aesthetic adjustment of the CRect_AC 
	MADrawString(theString, qdArea, teCenter);
}


//----------------------------------------------------------------------------------------
// TColumnsView::ReSelect: 
//----------------------------------------------------------------------------------------


void TColumnsView::ReSelect(const TGridViewDesignator* cellRegion)

{
	GridRect rgnBBox(cellRegion->GetBounds());
	
	GridCell aCell = rgnBBox[topLeft];
	if (rgnBBox != fSelections.GetBounds())
	{
		// selection has changed 
		fCalcDocument->fCellsView->SetEmptySelection(kHighlight);
		fCalcDocument->fRowsView->SetEmptySelection(kHighlight);
		fCalcDocument->fColumnIsSelected = true;
		SetSelection(cellRegion, kDontExtend, kHighlight, kSelect);
		TGridViewDesignator cellsToSelect;
		TGridViewDesignator cellsInColumn;
		{
			for (CSelectedCellIterator iter(this); iter.Current() != gZeroVPt; ++iter)
			{
				GridCell columnCell = iter;
				cellsInColumn.Set(GridRect(columnCell.h, 1, columnCell.h + 1, fCalcDocument->fCellsView->GetNumOfRows() + 1));
				cellsToSelect.Append(&cellsInColumn);
			}
		}

		fCalcDocument->fCellsView->SetSelection(&cellsToSelect, kDontExtend, kHighlight, kSelect);
	}
}


//========================================================================================
// CLASS TColumn
//========================================================================================
//----------------------------------------------------------------------------------------
// TColumn constructor
//----------------------------------------------------------------------------------------


TColumn::TColumn(GridCoordinate number) :
	fNumber(number),
	fFormat(gDefaultFormat),
	fOldFormat(gDefaultFormat),
	fWidth(kCellWidth)
{
}

//----------------------------------------------------------------------------------------
// TColumn destructor
//----------------------------------------------------------------------------------------


TColumn::~TColumn()
{
}


//----------------------------------------------------------------------------------------
// TColumn::ReadFromDisk: 
//----------------------------------------------------------------------------------------


void TColumn::ReadFromDisk(TFile* aFile)
{
	ColumnDiskInfo	columnInfo;

	long count = sizeof(ColumnDiskInfo);
	::ThrowIfOSErr_AC(aFile->ReadData(&columnInfo, count));
	fNumber = columnInfo.number;
	fFormat = columnInfo.format;
	fWidth = columnInfo.width;
}


//----------------------------------------------------------------------------------------
// TColumn::WriteToDisk: 
//----------------------------------------------------------------------------------------


void TColumn::WriteToDisk(TFile* aFile)
{
	ColumnDiskInfo	columnInfo;

	// write out the info that TCalcDocument.DoRead expects to see 
	GridCoordinate c = fNumber;
	long count = sizeof(GridCoordinate);
	::ThrowIfOSErr_AC(aFile->WriteData(&c, count));

	// now write out the info that TColumn.ReadFromDisk expects to see 
	columnInfo.number = fNumber;
	columnInfo.format = fFormat;
	columnInfo.width = fWidth;
	count = sizeof(ColumnDiskInfo);
	::ThrowIfOSErr_AC(aFile->WriteData(&columnInfo, count));
}


//----------------------------------------------------------------------------------------
// TColumn::ReadFromScrap: 
//----------------------------------------------------------------------------------------


void TColumn::ReadFromScrap(Handle	theScrap,
								   long&	scrapOffset)
{
	ColumnDiskInfo columnInfo;

	ReadScrap(theScrap, scrapOffset, &columnInfo, sizeof(columnInfo));
	fNumber = columnInfo.number;
	fFormat = columnInfo.format;
	fWidth = columnInfo.width;
}


//----------------------------------------------------------------------------------------
// TColumn::WriteToScrap: 
//----------------------------------------------------------------------------------------


void TColumn::WriteToScrap(Handle	theScrap,
								  long&		scrapOffset)
{
	ColumnDiskInfo columnInfo;

	columnInfo.number = fNumber;
	columnInfo.format = fFormat;
	columnInfo.width = fWidth;
	WriteScrap(theScrap, scrapOffset, &columnInfo, sizeof(columnInfo));
}



//========================================================================================
// CLASS TColumnList
//========================================================================================
MA_DEFINE_CLASS(TColumnList);

//----------------------------------------------------------------------------------------
// TColumnList destructor
//----------------------------------------------------------------------------------------


TColumnList::~TColumnList()
{
}

//----------------------------------------------------------------------------------------
// TColumnList::GetColumn: 
//----------------------------------------------------------------------------------------


TColumn* TColumnList::GetColumn(GridCoordinate c)
{
	TColumn* returnVal = NULL;

	for (CCalcColumnIterator iter(this); iter.Current(); ++iter)				// initialize the column iterator to find the column
		if (iter->fNumber == c)
		{
			returnVal = iter;						// as soon as we find it, leave the method call
			break;
		}

	return returnVal;
}


//----------------------------------------------------------------------------------------
// TColumnList::Compare: 
//----------------------------------------------------------------------------------------


CompareResult_AC TColumnList::Compare(const void* item1,
										  const void* item2) const// override 

{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	// we know that item1 and item2 are columns, since this is a column list, and
	// so all of these disgusting typecasts are ok.
	
	if (((const TColumn *)(item1))->fNumber > ((const TColumn *)(item2))->fNumber)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (((const TColumn *)(item1))->fNumber < ((const TColumn *)(item2))->fNumber)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TColumnList::FreeAll: 
//----------------------------------------------------------------------------------------


void TColumnList::FreeAll()
{
	for (TemplateIterator_AC<TColumnList*> iter(this); iter.Current(); ++iter)
		delete iter.Current();

	DeleteAll();
}


//========================================================================================
// CLASS TColumnFormatter
//========================================================================================
//----------------------------------------------------------------------------------------
// TColumnFormatter constructor 
//----------------------------------------------------------------------------------------


TColumnFormatter::TColumnFormatter(TCalcDocument* itsDocument,
										 CommandNumber	itsCommand) :
	TCommand(itsCommand, itsDocument, kCanUndo, kCausesChange, itsDocument),

	fCalcDocument(itsDocument),
	fSelection(*itsDocument->fColumnsView->GetSelections())
{
	for (CSelectedCellIterator iter(fCalcDocument->fColumnsView); iter.Current() != gZeroVPt; ++iter)
		SaveFormat(iter);
}


//----------------------------------------------------------------------------------------
// TColumnFormatter::Free: 
//----------------------------------------------------------------------------------------


TColumnFormatter::~TColumnFormatter()		// override 
{
}


//----------------------------------------------------------------------------------------
// TColumnFormatter::DoIt: 
//----------------------------------------------------------------------------------------


void TColumnFormatter::DoIt()		// override 
{
	for (CSelectedCellIterator iter(fCalcDocument->fColumnsView); iter.Current() != gZeroVPt; ++iter)
		SetFormat(iter);

	if ((fIdentifier >= cGeneral) && (fIdentifier <= cScientific))// Style change 
		fCalcDocument->DoRecalculate(!kForceAutomatic, !kUpdateDependencies);
	fCalcDocument->fCellsView->InvalidateSelection();
}


//----------------------------------------------------------------------------------------
// TColumnFormatter::RedoIt: 
//----------------------------------------------------------------------------------------


void TColumnFormatter::RedoIt()		// override 
{
	fCalcDocument->fColumnsView->ReSelect(&fSelection);

	DoIt();
}


//----------------------------------------------------------------------------------------
// TColumnFormatter::UndoIt: 
//----------------------------------------------------------------------------------------


void TColumnFormatter::UndoIt()		// override 
{
	fCalcDocument->fColumnsView->ReSelect(&fSelection);

	for (CSelectedCellIterator iter(fCalcDocument->fColumnsView); iter.Current() != gZeroVPt; ++iter)
		RestoreFormat(iter);

	if ((fIdentifier >= cGeneral) && (fIdentifier <= cScientific))// Style change 
		fCalcDocument->DoRecalculate(!kForceAutomatic, !kUpdateDependencies);

	fCalcDocument->fCellsView->InvalidateSelection();
}


//----------------------------------------------------------------------------------------
// TColumnFormatter::SaveFormat: 
//----------------------------------------------------------------------------------------
void TColumnFormatter::SaveFormat(const GridCell& aCell)// PROTECTED METHOD CALL
{
	TColumn * theColumn = fCalcDocument->GetColumn(aCell.h);
	theColumn->fOldFormat = theColumn->fFormat;
}


//----------------------------------------------------------------------------------------
// TColumnFormatter::SetFormat: 
//----------------------------------------------------------------------------------------
void TColumnFormatter::SetFormat(const GridCell& aCell)// PROTECTED METHOD CALL
{
	TColumn * theColumn = fCalcDocument->GetColumn(aCell.h);
	switch (fIdentifier)
	{
		case cGeneral:
			theColumn->fFormat.fStyle = kGeneral;
			break;

		case cNoDecimal:
			theColumn->fFormat.fStyle = kNoDecimal;
			break;

		case cDecimal:
			theColumn->fFormat.fStyle = kDecimalStyle;
			break;

		case cScientific:
			theColumn->fFormat.fStyle = kScientific;
			break;

		case cSystemJustify:
			theColumn->fFormat.fJustification = teFlushDefault;
			break;

		case cForceLeftJustify:
			theColumn->fFormat.fJustification = teFlushLeft;
			break;

		case cRightJustify:
			theColumn->fFormat.fJustification = teFlushRight;
			break;

		case cCenter:
			theColumn->fFormat.fJustification = teCenter;
			break;
	}
}


//----------------------------------------------------------------------------------------
// TColumnFormatter::RestoreFormat: 
//----------------------------------------------------------------------------------------
void TColumnFormatter::RestoreFormat(const GridCell& aCell)// PROTECTED METHOD CALL
{
	TColumn * theColumn = fCalcDocument->GetColumn(aCell.h);
	theColumn->fFormat = theColumn->fOldFormat;
}


//========================================================================================
// CLASS TColumnSizer
//========================================================================================
//----------------------------------------------------------------------------------------
// TColumnSizer::IColumnSizer: 
//----------------------------------------------------------------------------------------


TColumnSizer::TColumnSizer(TCalcDocument* itsDocument,
									   GridCoordinate c,
									   const CViewPoint& theMouse) :
	TTracker(cSizeColumn, itsDocument, kCanUndo, kCausesChange, itsDocument,
				   itsDocument->fColumnsView /* itsDocument->fCellsView */,
				   itsDocument->fColumnsView->GetScroller(true) /* itsDocument->fCellsView->GetScroller(true) */, theMouse),

	fCalcDocument(itsDocument),
	fCellsView(itsDocument->fCellsView),
	fColumn(itsDocument->GetColumn(c)),

	fLeftEdge(fCellsView->ColToVRect(fColumn->fNumber, 1).left),
	fNewWidth(fCellsView->GetColWidth(fColumn->fNumber)),
	fOldWidth(fNewWidth)
{
	fConstrainsMouse = true;
	fViewConstrain = false;						// So that rightmost column can grow 
}

//----------------------------------------------------------------------------------------
// TColumnSizer destructor
//----------------------------------------------------------------------------------------


TColumnSizer::~TColumnSizer()
{
}

//----------------------------------------------------------------------------------------
// TColumnSizer::TrackMouse: 
//----------------------------------------------------------------------------------------


TTracker* TColumnSizer::TrackMouse(TrackPhase	/*aTrackPhase*/,
										  CViewPoint&		/*anchorPoint*/ ,
										  CViewPoint&		/*previousPoint*/ ,
										  CViewPoint&		nextPoint,
										  bool		/*mouseDidMove*/)// override 
{
	fNewWidth = Min_AC(static_cast<short>(nextPoint.h - fLeftEdge), kTELength);
	fNewWidth = Max_AC(fNewWidth, 10);
	nextPoint.h = Min_AC(nextPoint.h, static_cast<ViewCoordinate>(fLeftEdge + kTELength));

	return this;
} 


//----------------------------------------------------------------------------------------
// TColumnSizer::TrackFeedback: 
//----------------------------------------------------------------------------------------


void TColumnSizer::TrackFeedback(TrackPhase		/* aTrackPhase */,
										const CViewPoint&	/* anchorPoint */,
										const CViewPoint&	/* previousPoint */,
										const CViewPoint&	nextPoint,
										bool			mouseDidMove,
										bool			/* turnItOn */)// override 
{
	PenState	pState;

	// if there was any horizontal movement
	if (mouseDidMove)
	{
		GetPenState(&pState);				// save pen state
		PenPat(&qd.black);

		// draw in the columns view
		CRect_AC viewedRect = fView->GetQDExtent();
		MoveTo(Min_AC(static_cast<short>(nextPoint.h), static_cast<short>(fLeftEdge + kTELength)), viewedRect.top);
		Line(0, viewedRect.bottom - viewedRect.top);

		// draw in the cells view
		short savePenMode = qd.thePort->pnMode;
		fCellsView->Focus();
		PenPat(&qd.black);
		PenMode(savePenMode);
		viewedRect = fCellsView->GetQDExtent();
		MoveTo(Min_AC(static_cast<short>(nextPoint.h), static_cast<short>(fLeftEdge + kTELength)), viewedRect.top);
		Line(0, viewedRect.bottom - viewedRect.top);

		fView->Focus();
		
		SetPenState(&pState);				// restore pen state
	}
}


//----------------------------------------------------------------------------------------
// TColumnSizer::TrackConstrain: 
//----------------------------------------------------------------------------------------


void TColumnSizer::TrackConstrain(TrackPhase		/*aTrackPhase*/,
										 const CViewPoint&	/*anchorPoint*/ ,
										 const CViewPoint&	/*previousPoint*/ ,
										 CViewPoint&		nextPoint,
										 bool		/*mouseDidMove*/)// override 
{
	if (nextPoint.h < fLeftEdge + 10)
		nextPoint.h = fLeftEdge + 10;
	else
		nextPoint.h = Min_AC(nextPoint.h, static_cast<ViewCoordinate>(fLeftEdge + kTELength));
}


//----------------------------------------------------------------------------------------
// TColumnSizer::DoIt: 
//----------------------------------------------------------------------------------------


void TColumnSizer::DoIt()			// override 
{
	SetColumnWidth(fNewWidth);

	CViewRect aRect(fCellsView->ColToVRect(fColumn->fNumber, 1));
	fCellsView->RevealRect(aRect, aRect.GetSize(), TRUE);

	CViewRect bRect(fCalcDocument->fColumnsView->ColToVRect(fColumn->fNumber, 1));
	fCalcDocument->fColumnsView->RevealRect(bRect, bRect.GetSize(), true);
}


//----------------------------------------------------------------------------------------
// TColumnSizer::UndoIt: 
//----------------------------------------------------------------------------------------


void TColumnSizer::UndoIt()			// override 
{
	SetColumnWidth(fOldWidth);
}


//----------------------------------------------------------------------------------------
// TColumnSizer::SetColumnWidth: 
//----------------------------------------------------------------------------------------


void TColumnSizer::SetColumnWidth(ViewCoordinate newWidth)
{
	if (newWidth > kTELength)
	{
		newWidth = kTELength;
		fNewWidth = kTELength;
	}
	else if (newWidth < 10)
		newWidth = 10;

	fCellsView->SetColWidth(fColumn->fNumber, 1, newWidth);
	fCalcDocument->fColumnsView->SetColWidth(fColumn->fNumber, 1, newWidth);
	fColumn->fWidth = newWidth;
}


//========================================================================================
// CLASS CCalcColumnIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CCalcColumnIterator::CCalcColumnIterator: 
//----------------------------------------------------------------------------------------


CCalcColumnIterator::CCalcColumnIterator(TCalcDocument* theCalcDocument) :
	CTColumnIterator(theCalcDocument ? &theCalcDocument->fColumns : NULL)

{
}


//----------------------------------------------------------------------------------------
// End of UCalcColumns.cp


