//----------------------------------------------------------------------------------------
// UCalcRows.cp
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCALCROWS__
#include "UCalcRows.h"
#endif

// Calc

#ifndef __UCALCCOLUMNS__
#include "UCalcColumns.h"
#endif

#ifndef __CALCUTILITIES__
#include "CalcUtilities.h"
#endif

// ACS

#if qGrayscaleAppearance
#	ifndef __AGAImaging_AC__
#	include "AGAImaging_AC.h"
#	endif
#endif

// MacApp

#ifndef __UCALC__
#include "UCalc.h"
#endif

// Toolbox

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

//========================================================================================
// CLASS TRowsView
//========================================================================================
MA_DEFINE_CLASS(TRowsView);

//----------------------------------------------------------------------------------------
// TRowsView::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TRowsView::DoPostCreate(TDocument*	itsDocument)	// override 
{
	fHelpID = kRowsViewHelp;
	fHelpIndex = 2;
	fCalcDocument = (TCalcDocument *)(itsDocument);
} 

//----------------------------------------------------------------------------------------
// TRowsView destructor
//----------------------------------------------------------------------------------------


TRowsView::~TRowsView()
{
}


//----------------------------------------------------------------------------------------
// TRowsView::AdornRow: 
//----------------------------------------------------------------------------------------


void TRowsView::AdornRow(GridCoordinate			/*aRow*/,
								const CViewRect& 	area)// override 
{
	PenSize(1, 1);
	PenPat(&qd.black);

	CRect_AC qdArea(ViewToQD(area));
	--qdArea[botRight];

	// right line 
	MoveTo(qdArea.right, qdArea.top);
	LineToPt(qdArea[botRight]);

	// bottom line 
	MoveTo(qdArea.left, qdArea.bottom);
	LineToPt(qdArea[botRight]);
}


//----------------------------------------------------------------------------------------
// TRowsView::DoMouseCommand: 
//----------------------------------------------------------------------------------------


void TRowsView::DoMouseCommand(CViewPoint&			theMouse,
									  TToolboxEvent*	event,
									  CPoint_AC			/*hysteresis*/)	// override 
{
	PostCommand(TH_new TRowSelector(fCalcDocument, this, theMouse, event->IsShiftKeyPressed(), event->IsCommandKeyPressed()));

	fCalcDocument->fSelectionType = kRowSelection;
	fCalcDocument->fColumnsView->SetEmptySelection(kHighlight);
	fCalcDocument->fColumnIsSelected = false;
}


//----------------------------------------------------------------------------------------
// TRowsView::DrawCell: 
//----------------------------------------------------------------------------------------


void TRowsView::DrawCell(const GridCell&		aCell,
								const CViewRect&	aRect)// override 
{
	CStr255_AC	theString;
	NumToString(aCell.v, theString);

	CRect_AC qdArea(ViewToQD(aRect));

#if qGrayscaleAppearance
	CRect_AC bgRect(qdArea);
	// Undo the inset, then inset to avoid Adornment
	CRect_AC 	insetRect(fColInset / 2, fRowInset / 2,
					  -(fColInset / 2), -(fRowInset / 2));
	bgRect -= insetRect;
	bgRect -= CPoint_AC(1,1);
	++bgRect[topLeft];
	
	AGABackgroundPaint(bgRect, true, kRaisedModelessBackground, true, false);
#endif

	//qdArea.top = qdArea.top + 2;				// aesthetic adjustment of the CRect_AC 
	MADrawString(theString, qdArea, teCenter);
}


//========================================================================================
// CLASS TRow
//========================================================================================
//----------------------------------------------------------------------------------------
// TRow constructor 
//----------------------------------------------------------------------------------------


TRow::TRow(GridCoordinate number) :	
	fNumber(number)
{
}

//----------------------------------------------------------------------------------------
// TRow destructor
//----------------------------------------------------------------------------------------


TRow::~TRow()
{
}

//----------------------------------------------------------------------------------------
// TRow::ReadFromDisk: 
//----------------------------------------------------------------------------------------


void TRow::ReadFromDisk(TFile* aFile)
{
	RowDiskInfo	rowInfo;
	long count = sizeof(RowDiskInfo);

	::ThrowIfOSErr_AC(aFile->ReadData(&rowInfo, count));
	fNumber = rowInfo.number;
}


//----------------------------------------------------------------------------------------
// TRow::WriteToDisk: 
//----------------------------------------------------------------------------------------


void TRow::WriteToDisk(TFile* aFile)
{
	RowDiskInfo	rowInfo;

	// need to write our row number which is read in in TCalcDocument.DoRead 
	GridCoordinate r = fNumber;
	long count = sizeof(GridCoordinate);
	::ThrowIfOSErr_AC(aFile->WriteData(&r, count));

	// now, write out the info that TRow.ReadFromDisk expects to see 
	rowInfo.number = fNumber;
	count = sizeof(RowDiskInfo);
	::ThrowIfOSErr_AC(aFile->WriteData(&rowInfo, count));
}


//----------------------------------------------------------------------------------------
// TRow::ReadFromScrap: 
//----------------------------------------------------------------------------------------


void TRow::ReadFromScrap(Handle theScrap,
								long& scrapOffset)
{
	RowDiskInfo RowInfo;

	ReadScrap(theScrap, scrapOffset, &RowInfo, sizeof(RowInfo));
	fNumber = RowInfo.number;
}


//----------------------------------------------------------------------------------------
// TRow::WriteToScrap: 
//----------------------------------------------------------------------------------------


void TRow::WriteToScrap(Handle	theScrap,
							   long&	scrapOffset)
{
	RowDiskInfo RowInfo;

	RowInfo.number = fNumber;
	WriteScrap(theScrap, scrapOffset, &RowInfo, sizeof(RowInfo));
}


//========================================================================================
// CLASS TRowList
//========================================================================================
MA_DEFINE_CLASS(TRowList);

//----------------------------------------------------------------------------------------
// TRowList destructor
//----------------------------------------------------------------------------------------


TRowList::~TRowList()
{
}

//----------------------------------------------------------------------------------------
// TRowList::Compare: 
//----------------------------------------------------------------------------------------


CompareResult_AC TRowList::Compare(const void* item1,
									   const void* item2) const// override 
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;
	// like TCellList and TColumnList, we are guaranteed a homogenous list (Rows this case)
	// so typecasting is perfectly safe.

	if (((const TRow *)(item1))->fNumber > ((const TRow *)(item2))->fNumber)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (((const TRow *)(item1))->fNumber < ((const TRow *)(item2))->fNumber)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}


//----------------------------------------------------------------------------------------
// TRowList::GetRow: 
//----------------------------------------------------------------------------------------


TRow* TRowList::GetRow(GridCoordinate r)
{
	TRow* returnVal = NULL;

	for (CCalcRowIterator iter(this); iter.Current(); ++iter)				// initialize the row iterator to find the row
		if (iter->fNumber == r)
		{
			returnVal = iter;						// as soon as we find it, leave the method call
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TRowList::FreeAll: 
//----------------------------------------------------------------------------------------


void TRowList::FreeAll()
{
	for (TemplateIterator_AC<TRow*> iter(this); iter.Current(); ++iter)
		delete iter.Current();

	DeleteAll();
}


//========================================================================================
// CLASS CCalcRowIterator
//----------------------------------------------------------------------------------------
// CCalcRowIterator::CCalcRowIterator: 
//----------------------------------------------------------------------------------------


CCalcRowIterator::CCalcRowIterator(TCalcDocument* theCalcDocument) :
	CTRowIterator(theCalcDocument ? &theCalcDocument->fRows : NULL)
{
}


//----------------------------------------------------------------------------------------
// End of UCalcRows.cp


