// UCalcRows.h
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UCALCROWS__
#define __UCALCROWS__

// Calc Includes

#ifndef __UCALCCELLS__
#include "UCalcCells.h"
#endif

// MacApp Includes

// Toolbox

// ANSI

//--------------------------------------------------------------------------------------------------
// Forward and external class declarations. 
//--------------------------------------------------------------------------------------------------

class TRow;
class TRowsView;
class TRowList;
class CRowIterator;


//--------------------------------------------------------------------------------------------------
// TRowsView:
//--------------------------------------------------------------------------------------------------

class TRowsView : public TTextGridView
{
	MA_DECLARE_CLONE;

public:
	// TRowsView is the view used to show the row numbers. Click in this view
	//  to select an entire row or group of rows.

	inline TRowsView()
	{ }
	// Empty Constructor

	virtual~ TRowsView();
	// Destructor

	virtual void DoPostCreate(TDocument* itsDocument);// override 
	// Create the view from a resource template

	virtual void AdornRow(GridCoordinate aRow,
						  const CViewRect& area);	// override 
	// Draw a row delimiter in the given area 

	virtual void DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* event,
								CPoint_AC hysteresis);// override 

	virtual void DrawCell(const GridCell& aCell,
						  const CViewRect& aRect);	// override 

public:
	TCalcDocument* fCalcDocument;
};



//--------------------------------------------------------------------------------------------------
// TRow:
//--------------------------------------------------------------------------------------------------

class TRow
{
public:
	TRow(GridCoordinate number = 0);
	// Constructor

	virtual~ TRow();
	// Destructor

	virtual void ReadFromDisk(TFile* aFile);

	virtual void WriteToDisk(TFile* aFile);

	virtual void ReadFromScrap(Handle theScrap,
							   long& scrapOffset);

	virtual void WriteToScrap(Handle theScrap,
							  long& scrapOffset);

public:
	GridCoordinate fNumber;
};


//--------------------------------------------------------------------------------------------------
// TRowList:
//--------------------------------------------------------------------------------------------------

class TRowList : public CSortedList_AC
{
	MA_DECLARE_CLONE;

public:
	TRowList() : CSortedList_AC() { }
	// Constructor

	virtual~ TRowList();
	// Destructor

	virtual CompareResult_AC Compare(const void* item1,
									   const void* item2) const;// override 
	// Compares rows based on row number 

	virtual TRow* GetRow(GridCoordinate r);
	// Returns the row whose number is r 

	void FreeAll();
		// Frees all the rows and then deletes all the elements
};

//--------------------------------------------------------------------------------------------------
typedef TemplateIterator_AC<TRow*> CTRowIterator;

class CCalcRowIterator : public CTRowIterator
{
public:
	inline CCalcRowIterator(TRowList* theRowList) :
	CTRowIterator(theRowList)
	{ }

	CCalcRowIterator(TCalcDocument* theCalcDocument);
};

#endif //define UCALCROWS
