//----------------------------------------------------------------------------------------
// UDemoDialogs.More.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDEMODIALOGS__
#include "UDemoDialogs.h"
#endif

// ACS

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif 

// MacApp

#ifndef __UCHECKBOX__
#include "UCheckbox.h"
#endif

#ifndef __UDRAWINGENVIRONMENT__
#include "UDrawingEnvironment.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UNUMBERTEXT__
#include "UNumberText.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __FP__
#include <fp.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SOUND__
#include <Sound.h>
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif


//----------------------------------------------------------------------------------------
// Constants

const short kMaxPlaces = 13;

const short mUpArrow = 100;
const short mDownArrow = 101;
const short mTemperatureChanged = 102;

//========================================================================================
// CLASS TSwallowBehavior
//========================================================================================
#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TSwallowBehavior);

//----------------------------------------------------------------------------------------
// TSwallowBehavior destructor
//----------------------------------------------------------------------------------------


TSwallowBehavior::~TSwallowBehavior()
{
}

//----------------------------------------------------------------------------------------
// TSwallowBehavior::DoToolboxEvent: 
//----------------------------------------------------------------------------------------


bool TSwallowBehavior::DoToolboxEvent(TToolboxEvent* event)
{
	if (event && (event->GetIdentifier() == mouseDown) && event->IsOptionKeyPressed())
	{
		gApplication->Beep(1);
		return true;				// consume the event
	}
	else
		return Inherited::DoToolboxEvent(event);
}




//========================================================================================
// CLASS TChangeBehavior
//========================================================================================
#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TChangeBehavior);

//----------------------------------------------------------------------------------------
// TChangeBehavior destructor
//----------------------------------------------------------------------------------------


TChangeBehavior::~TChangeBehavior()
{
}

//----------------------------------------------------------------------------------------
// TChangeBehavior::DoEvent: 
//----------------------------------------------------------------------------------------


void TChangeBehavior::DoEvent(EventNumber eventNumber,
							  TEventHandler* source,
							  TEvent* event)// override 
{
	fOwner->Changed(eventNumber, event);

	Inherited::DoEvent(eventNumber, source, event);
}



//========================================================================================
// CLASS TUpdateBehavior
//========================================================================================
#undef Inherited


MA_DEFINE_CLASS(TUpdateBehavior);

//----------------------------------------------------------------------------------------
// TUpdateBehavior destructor
//----------------------------------------------------------------------------------------


TUpdateBehavior::~TUpdateBehavior()
{
}

//----------------------------------------------------------------------------------------
// TUpdateBehavior::DoBehaviorUpdate: 
//----------------------------------------------------------------------------------------


void TUpdateBehavior::DoUpdate(ChangeID_AC theChange,
								MDependable_AC* changedObject,
								void* changeData,
								CDependencySpace_AC* dependencySpace) // override
{
	if (((theChange == mVScrollBarHit) || (theChange == mHScrollBarHit)))
		dynamic_cast_AC(TScrollBar*,(fOwner))->SetLongVal(-dynamic_cast_AC(TScrollBar*,(changedObject))->GetLongVal(), kRedraw);
	else
		MDependable_AC::DoUpdate(theChange, changedObject, changeData, dependencySpace);
}


//========================================================================================
// CLASS TArrowsControl
//========================================================================================
#undef Inherited
#define Inherited TPicture


MA_DEFINE_CLASS(TArrowsControl);

//----------------------------------------------------------------------------------------
// TArrowsControl constructor 
//----------------------------------------------------------------------------------------


TArrowsControl::TArrowsControl() :	 
	fLastChange(0)
{
}

//----------------------------------------------------------------------------------------
// TArrowsControl destructor
//----------------------------------------------------------------------------------------


TArrowsControl::~TArrowsControl()
{
}

//----------------------------------------------------------------------------------------
// TArrowsControl::TrackMouse: 
//----------------------------------------------------------------------------------------


void TArrowsControl::TrackMouse(TrackPhase	aTrackPhase,
								CViewPoint&		/*anchorPoint*/,
								CViewPoint&		/*previousPoint*/,
								CViewPoint&		nextPoint,
								bool			/*mouseDidMove*/)// override 
{
	if (TickCount() >= fLastChange + 5)
	{
		fLastChange = TickCount();
		if (aTrackPhase == trackBegin)
			fLastChange = fLastChange + 10;

		if (ContainsMouse(nextPoint))
			if (nextPoint.v <= fSize.v / 2)
				HandleEvent(mUpArrow, this, NULL);
			else
				HandleEvent(mDownArrow, this, NULL);
	}
}


//========================================================================================
// CLASS TTemperatureCluster
//========================================================================================
#undef Inherited
#define Inherited TCluster


MA_DEFINE_CLASS(TTemperatureCluster);

//----------------------------------------------------------------------------------------
// TTemperatureCluster destructor
//----------------------------------------------------------------------------------------


TTemperatureCluster::~TTemperatureCluster()
{
}

//----------------------------------------------------------------------------------------
// TTemperatureCluster::DoEvent: 
//----------------------------------------------------------------------------------------


void TTemperatureCluster::DoEvent(EventNumber eventNumber,
								  TEventHandler* source,
								  TEvent* event) // override 
{
	TNumberText*	theNumberText;
	long			theNumber;


	switch (eventNumber)
	{
		case mUpArrow:
		case mDownArrow:
			{
				theNumberText = dynamic_cast_AC(TNumberText*,(FindSubView('Numb')));
				theNumber = theNumberText->GetValue();
				if (eventNumber == mUpArrow)
					theNumber = theNumber + 1;
				else
					theNumber = theNumber - 1;
				theNumberText->SetValue(theNumber, kRedraw);

				HandleEvent(mTemperatureChanged, this, NULL);
				break;
			}
		default:
			Inherited::DoEvent(eventNumber, source, event);
	}
}

//----------------------------------------------------------------------------------------
// TTemperatureCluster::GetValue: 
//----------------------------------------------------------------------------------------


long TTemperatureCluster::GetValue() const
{
	TNumberText * numberText;

	numberText = dynamic_cast_AC(TNumberText*,FindSubView('Numb'));
	return (numberText->GetValue());
}

//----------------------------------------------------------------------------------------
// TTemperatureCluster::SetValue: 
//----------------------------------------------------------------------------------------


void TTemperatureCluster::SetValue(long		newValue,
										  bool	redraw)
{
	TNumberText * numberText;

	numberText = dynamic_cast_AC(TNumberText*,FindSubView('Numb'));
	numberText->SetValue(newValue, redraw);
}


//========================================================================================
// CLASS TTemperatureConversionCluster
//========================================================================================
#undef Inherited
#define Inherited TCluster


MA_DEFINE_CLASS(TTemperatureConversionCluster);

//----------------------------------------------------------------------------------------
// TTemperatureConversionCluster destructor
//----------------------------------------------------------------------------------------


TTemperatureConversionCluster::~TTemperatureConversionCluster()
{
}

//----------------------------------------------------------------------------------------
// TTemperatureConversionCluster::DoEvent: 
//----------------------------------------------------------------------------------------


void TTemperatureConversionCluster::DoEvent(EventNumber eventNumber,
											TEventHandler* source,
											TEvent* event)
{
	TTemperatureCluster * Celsius;
	TTemperatureCluster * Fahrenheit;

	if (eventNumber == mTemperatureChanged)
	{
		Celsius = dynamic_cast_AC(TTemperatureCluster*,(FindSubView('Cels')));
		Fahrenheit = dynamic_cast_AC(TTemperatureCluster*,(FindSubView('Fahr')));
		if (source == Celsius)
		{
			Fahrenheit->SetValue(Celsius->GetValue() * 1.8 + 32, kRedraw);
		}
		else if (source == Fahrenheit)
		{
			Celsius->SetValue((Fahrenheit->GetValue() - 32) / 1.8, kRedraw);
		}
	}
	else
		Inherited::DoEvent(eventNumber, source, event);
}


//========================================================================================
// CLASS THomeBrewSlider
//========================================================================================
#undef Inherited
#define Inherited TPicture


MA_DEFINE_CLASS(THomeBrewSlider);

//----------------------------------------------------------------------------------------
// THomeBrewSlider constructor 
//----------------------------------------------------------------------------------------


THomeBrewSlider::THomeBrewSlider() :			
	fValue(0),
	fKnobPicture(NULL),
	fMinTop(0),
	fMaxTop(0)
{
}

//----------------------------------------------------------------------------------------
// THomeBrewSlider destructor
//----------------------------------------------------------------------------------------


THomeBrewSlider::~THomeBrewSlider()
{
}

//----------------------------------------------------------------------------------------
// THomeBrewSlider::DoPostCreate: 
//----------------------------------------------------------------------------------------


void THomeBrewSlider::DoPostCreate(TDocument* itsDocument)// override 
{
	PicHandle	knobPicture;
	CViewRect	knobRect;
	CRect_AC	tmpR;


	Inherited::DoPostCreate(itsDocument);

	knobPicture = GetPicture(1002);
	fKnobPicture = knobPicture;
	tmpR = (*knobPicture)->picFrame;
	knobRect.left = fSize.h - (tmpR.right - tmpR.left);
	knobRect.top = fSize.v - 16;
	knobRect.right = knobRect.left + tmpR.right - tmpR.left;
	knobRect.bottom = knobRect.top + tmpR.bottom - tmpR.top;

	fKnobRect = knobRect.ToRect();
	fMinTop = fKnobRect.top - 84;
	fMaxTop = fKnobRect.top;
}

//----------------------------------------------------------------------------------------
// THomeBrewSlider::ContainsMouse: 
//----------------------------------------------------------------------------------------


bool THomeBrewSlider::ContainsMouse(const CViewPoint& theMouse)// override 
{
	return GetKnobRect().Contains(ViewToQD(theMouse));
}

//----------------------------------------------------------------------------------------
// THomeBrewSlider::Draw: 
//----------------------------------------------------------------------------------------


void THomeBrewSlider::Draw(const CViewRect& area)// override
{
	Inherited::Draw(area);						// This draws everything but the knob
	DrawKnob();
}

//----------------------------------------------------------------------------------------
// THomeBrewSlider::DrawKnob: 
//----------------------------------------------------------------------------------------


void THomeBrewSlider::DrawKnob()
{
	LoadResource(Handle(fKnobPicture));			// In case it was purged from memory 
	DrawPicture(fKnobPicture, GetKnobRect());
}

//----------------------------------------------------------------------------------------
// THomeBrewSlider::GetKnobRect: 
//----------------------------------------------------------------------------------------


CRect_AC THomeBrewSlider::GetKnobRect()
{
	return fKnobRect + CPoint_AC(0, -(fValue * 12));
}

//----------------------------------------------------------------------------------------
// THomeBrewSlider::TrackMouse: 
//----------------------------------------------------------------------------------------


void THomeBrewSlider::TrackMouse(TrackPhase	aTrackPhase,
								 CViewPoint& anchorPoint,
								 CViewPoint& previousPoint,
								 CViewPoint& nextPoint,
								 bool /*mouseDidMove*/)	// override 
{
	CRect_AC oldRect(GetKnobRect());
	CRect_AC newRect(oldRect);

	// Compute rectangle of knob's current position 
	short offset = static_cast<short>(MinMax_AC(fMinTop - newRect.top, previousPoint.v - anchorPoint.v, fMaxTop - newRect.top));
	oldRect.Offset(0, offset);

	// Compute rectangle of knob's new position 
	offset = static_cast<short>(MinMax_AC(fMinTop - newRect.top, nextPoint.v - anchorPoint.v, fMaxTop - newRect.top));
	newRect.Offset(0, offset);

	// If the mouse was released, pin the knob to a value and set slider's value 
	if (aTrackPhase == trackEnd)
	{
		short base = static_cast<short>(fSize.v - 16 - newRect.top);
		offset = base - ((base + 6) / 12 * 12);
		OffsetRect(newRect, 0, offset);
		fValue = static_cast<short>(fSize.v - 16 - newRect.top) / 12;
	}

	// To redraw minimal amount, compute difference between old and new positions 
	CRect_AC difference(oldRect);
	if (oldRect.top > newRect.top)
		difference.top = Max_AC(oldRect.top, newRect.bottom);
	else
		difference.bottom = Min_AC(oldRect.bottom, newRect.top);

	CTempRegion_AC savedClip;
	GetClip(savedClip);

	ClipRect(difference);
	Inherited::Draw(QDToView(difference));

	SetClip(savedClip);

	LoadResource(Handle(fKnobPicture));			// In case it was purged from memory 
	DrawPicture(fKnobPicture, newRect);
}



//========================================================================================
// CLASS TSumStaticText
//========================================================================================
#undef Inherited
#define Inherited TStaticText


MA_DEFINE_CLASS(TSumStaticText);

//----------------------------------------------------------------------------------------
// TSumStaticText destructor
//----------------------------------------------------------------------------------------


TSumStaticText::~TSumStaticText()
{
}

//----------------------------------------------------------------------------------------
// TSumStaticText::Draw: 
//----------------------------------------------------------------------------------------


void TSumStaticText::Draw(const CViewRect& area)// override 

{
	// Make sure the gray background gets completely erased 
	//InsetRect(theRect, fPenSize.h, fPenSize.v);
	EraseRect(&GetQDExtent());

	Inherited::Draw(area);
}


//========================================================================================
// CLASS TCalcDialog
//========================================================================================
#undef Inherited
#define Inherited TPicture


MA_DEFINE_CLASS(TCalcDialog);

//----------------------------------------------------------------------------------------
// TCalcDialog destructor
//----------------------------------------------------------------------------------------


TCalcDialog::~TCalcDialog()
{
}

//----------------------------------------------------------------------------------------
// TCalcDialog::DoEvent: 
//----------------------------------------------------------------------------------------


void TCalcDialog::DoEvent(EventNumber eventNumber,
						  TEventHandler* source,
						  TEvent* event)
{
	// dcs - 3.0 fix : Bad News. its either this or a mess of FindSubView calls..
	// Can we count on source being a view at this call.. Hmmm..
	//	origID = source->fIdentifier;
	IDType origID = dynamic_cast_AC(TView*,source)->GetIdentifier();
	char keyHit = static_cast<char>(origID & 0xFF);	// strip all but the last letter of the ID, which
										// we've assigned to the name of the key
	switch (eventNumber)
	{
		case mOKHit:							/*??? Should be mControlHit*/
			switch (keyHit)
			{
				case 'C':
					ClearSum();
					break;
				case '=':
				case 'E':
					TotalSum(noOperator);
					break;
				case '/':
					TotalSum(divOperator);
					break;
				case '*':
					TotalSum(mulOperator);
					break;
				case '-':
					TotalSum(subOperator);
					break;
				case '+':
					TotalSum(addOperator);
					break;
				default:
					NewDigit(keyHit);
					break;
			}
			break;

		default:
			Inherited::DoEvent(eventNumber, source, event);
			break;
	}
}


//----------------------------------------------------------------------------------------
// TCalcDialog::DoKeyEvent: 
//----------------------------------------------------------------------------------------


void TCalcDialog::DoKeyEvent(TToolboxEvent* event)// override 
{
	bool invalidKeyStroke = false;

	// Determine the ID to find the view which will be used as the originator of the keystroke 
	IDType	theID;
	if (event->GetKeyCode() == kClearVirtualCode)
		theID = 'keyC';
	else
	{
		if (event->GetText() == '0'
		|| event->GetText() == '1'
		|| event->GetText() == '2'
		|| event->GetText() == '3'
		|| event->GetText() == '4'
		|| event->GetText() == '5'
		|| event->GetText() == '6'
		|| event->GetText() == '7'
		|| event->GetText() == '8'
		|| event->GetText() == '9'
		|| event->GetText() == '.'
		|| event->GetText() == '+'
		|| event->GetText() == '-'
		|| event->GetText() == '*'
		|| event->GetText() == '/'
		|| event->GetText() == '='
		|| event->GetText() == '1')
			theID = 'key ' | event->GetText()[1];
		else if (event->GetText() == chEnter || event->GetText() == chReturn)
			theID = 'keyE';
		else
			invalidKeyStroke = true;
	}

	if (!invalidKeyStroke)
	{
		TView *	theView = FindSubView(theID);
		if (theView)
			dynamic_cast_AC(TControl*,(theView))->Flash();
		HandleEvent(mOKHit, theView, NULL);
	}
	else
		SysBeep(1);
}

//----------------------------------------------------------------------------------------
// TCalcDialog::ClearSum: 
//----------------------------------------------------------------------------------------


void TCalcDialog::ClearSum()
{
	fArgument = 0;
	fSum = 0;
	fOperator = noOperator;
	fDecimalPoint = false;
	fRestart = true;
	dynamic_cast_AC(TStaticText*,(FindSubView('SUMM')))->SetText("0", true);
}

//----------------------------------------------------------------------------------------
// TCalcDialog::FetchValue: 
//----------------------------------------------------------------------------------------


double TCalcDialog::FetchValue()
{
	return atof(dynamic_cast_AC(TStaticText*,FindSubView('SUMM'))->GetText());
}

//----------------------------------------------------------------------------------------
// TCalcDialog::SetValue: 
//----------------------------------------------------------------------------------------


void TCalcDialog::SetValue()
{
	CStr255_AC		theStr;
	double		aDouble;
	char		aCString[255];
	double		orderOfMagnitude;


	aDouble = fSum;
	orderOfMagnitude = log2(fabs(aDouble)) / log2(10);		// this says how many digits are 
															// left of the decimal point
	if ((orderOfMagnitude > kMaxPlaces - 1) || (orderOfMagnitude < -6))
	{
		short mantissaLength;
		
		orderOfMagnitude = log2(fabs(orderOfMagnitude)) / log2(10);// find the order of the exponent
		if (orderOfMagnitude > 2)
			mantissaLength = kMaxPlaces - 9;
		else
			mantissaLength = kMaxPlaces - 7;
			
		sprintf(aCString, "%.*e", mantissaLength, aDouble);	// use floating point conversion
		theStr = aCString;
	}
	else
	{
		sprintf(aCString, "%.*f", kMaxPlaces - 1, aDouble);
		theStr = aCString;
		Truncate(theStr);					// strip insignificant zeros / decimal point

		// now we must make sure it fits in the space in the dialog, which we know to be
		// kMaxPlaces wide (aren't fixed width fonts great?).  If the number is still too
		// large, we can just chop off what's off the end, because we know that the most
		// significant part of the number will still be correctly represented.
		if (theStr.Length() > kMaxPlaces)		
			theStr.Delete(kMaxPlaces, theStr.Length() - kMaxPlaces);
	}
	dynamic_cast_AC(TStaticText*,(FindSubView('SUMM')))->SetText(theStr, true);
}

//----------------------------------------------------------------------------------------
// TCalcDialog::Truncate: 
//----------------------------------------------------------------------------------------
void TCalcDialog::Truncate(CStr255_AC& theStr)
{
	int thePos = static_cast<int>(theStr.Length());
	while (theStr[thePos] == '0')
		--thePos;								// strip off 0s right of the decimal point
	if (theStr[thePos] == '.')					// there is no fractional part so
		--thePos;								// also strip the decimal point.

	if (thePos != theStr.Length())				// if we've decided to cut something
	{
		if (thePos)
			theStr.Delete(thePos + 1, theStr.Length() - thePos); // cut it.
		else
			theStr = "0";		// if everything has been stripped, we know the value is 0.
	}
}

//----------------------------------------------------------------------------------------
// TCalcDialog::TotalSum: 
//----------------------------------------------------------------------------------------


void TCalcDialog::TotalSum(CalcOperator newOperator)
{
	fArgument = FetchValue();
	switch (fOperator)
	{
		case noOperator:
			fSum = fArgument;
			break;
		case divOperator:
			fSum = fSum / fArgument;
			break;
		case mulOperator:
			fSum = fSum * fArgument;
			break;
		case subOperator:
			fSum = fSum - fArgument;
			break;
		case addOperator:
			fSum = fSum + fArgument;
			break;
	}
	SetValue();
	fRestart = true;
	fArgument = fSum;
	fOperator = newOperator;
}

//----------------------------------------------------------------------------------------
// TCalcDialog::NewDigit: 
//----------------------------------------------------------------------------------------


void TCalcDialog::NewDigit(char theNumber)

{
	TStaticText * theText = dynamic_cast_AC(TStaticText*,(FindSubView('SUMM')));
	CStr255_AC theStr;
	if (!fRestart)
		theStr = theText->GetText();

	if (theStr.Length() < kMaxPlaces - 1)	// is there room for another digit? 
	{
		fRestart = false;
		if ((theNumber == '.') &&!fDecimalPoint)
			fDecimalPoint = true;
		theStr += theNumber;	// append the char onto the end
		theText->SetText(theStr, true);
	}
}


//========================================================================================
// CLASS TAdornmentDemo
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TAdornmentDemo);

//----------------------------------------------------------------------------------------
// TAdornmentDemo constructor 
//----------------------------------------------------------------------------------------


TAdornmentDemo::TAdornmentDemo() :
	fTestView(NULL)
{
}

//----------------------------------------------------------------------------------------
// TAdornmentDemo destructor
//----------------------------------------------------------------------------------------


TAdornmentDemo::~TAdornmentDemo()
{
}

//----------------------------------------------------------------------------------------
// TAdornmentDemo::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TAdornmentDemo::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	fTestView = FindSubView('test');
}

//----------------------------------------------------------------------------------------
// TAdornmentDemo::DoEvent: 
//----------------------------------------------------------------------------------------


void TAdornmentDemo::DoEvent(EventNumber eventNumber,
							 TEventHandler* source,
							 TEvent* event)	// override 
{
	TView *			whichView;
	AdornPriority	priority;

	// Each adorner has the same class signature and ID as the ID of the corresponding checkbox 
	if (eventNumber == mCheckBoxHit)
	{

		if (source->GetIdentifier() == 'gray')
		{
			priority = kAdornFirst;
			whichView = this;
		}
		else if (source->GetIdentifier() == 'fram')
		{
			priority = kAdornLast;				// lower than the embossed frame adorner 
			whichView = fTestView;
		}
		else
		{
			priority = kAdornAfter;
			whichView = fTestView;
		}

		if (dynamic_cast_AC(TCheckBox*,source)->IsOn())
			whichView->AddAdorner(TAdorner::NewStdAdorner(source->GetIdentifier(), gEmptyString, source->GetIdentifier(), kFreeOnDeletion), priority, kRedraw);
		else
			whichView->DeleteAdornerByID(source->GetIdentifier(), kRedraw);

	}
	else
		Inherited::DoEvent(eventNumber, source, event);
}


//========================================================================================
// CLASS TEmbossedFrame
//========================================================================================
#undef Inherited
#define Inherited TAdorner


MA_DEFINE_CLASS(TEmbossedFrame);

//----------------------------------------------------------------------------------------
// TEmbossedFrame destructor
//----------------------------------------------------------------------------------------


TEmbossedFrame::~TEmbossedFrame()
{
}

//----------------------------------------------------------------------------------------
// TEmbossedFrame::Draw: 
//----------------------------------------------------------------------------------------


void TEmbossedFrame::Draw(TView* itsView, const CViewRect&	/*area*/)// override 
{
	PenNormal();

	CRect_AC QDArea(itsView->ViewToQD(itsView->GetAdornExtent()));
	CRect_AC tempRect(QDArea);

	//  Build the region to use for filling, first create the outer region
	QDArea.Inset(1, 1);
	CTempRegion_AC outerRgn = NewRgn();
	RectRgn(outerRgn, QDArea);

	//  Create the inner region
	InsetRect(QDArea, 6, 6);
	CTempRegion_AC innerRgn = NewRgn();
	RectRgn(innerRgn, QDArea);

	//  Finally subtract the inner from the outer region
	DiffRgn(outerRgn, innerRgn, outerRgn);

	//  Set the foreground color to the global face color, this allows the user to
	//	 change the color of the face																										
	SetIfColor(kRGBColorRamp[r2]);
	PaintRgn(outerRgn);

	//  Draw the inside black frame
	SetIfColor(gRGBBlack);
	QDArea.Inset(1, 1);
	FrameRect(&QDArea);

	//  Draw the shadows
	// THIS WAS "WITH tempRect"
	{
		//  Draw the outer gray shadow
		SetIfColor(kRGBColorRamp[r7]);
		MoveTo(tempRect.left + 1, tempRect.bottom - 1);		//(h, v)
		LineToPt(tempRect[botRight] - CPoint_AC(1, 1));	//(h, v)
		LineTo(tempRect.right - 1, tempRect.top + 1);		//(h, v)

		//  Draw the inner gray shadow	
		SetIfColor(kRGBColorRamp[r7]);
		MoveTo(tempRect.left + 7, tempRect.bottom - 8);		//(h, v)
		LineToPt(tempRect[topLeft] + CPoint_AC(7, 7));		//(h, v)
		LineTo(tempRect.right - 8, tempRect.top + 7);		//(h, v)
	}

	//  Restore the drawing environment
	itsView->SetupDrawingEnvironment();
}


//========================================================================================
// CLASS TGrayFill
//========================================================================================
#undef Inherited
#define Inherited TAdorner


MA_DEFINE_CLASS(TGrayFill);

//----------------------------------------------------------------------------------------
// TGrayFill destructor
//----------------------------------------------------------------------------------------


TGrayFill::~TGrayFill()
{
}

//----------------------------------------------------------------------------------------
// TGrayFill::Draw: 
//----------------------------------------------------------------------------------------


void TGrayFill::Draw(TView* itsView,
					 const CViewRect& /*area*/)	// override 
{
	//  Save off the current pen state and the foreground color
	PenState	savePenState;
	GetPenState(&savePenState);
	CRGBColor_AC saveColor(GetIfColor());
	PenNormal();

	SetIfColor(kRGBColorRamp[r7]);
	PaintRect(&itsView->ViewToQD(itsView->GetAdornExtent()));

	//  Restore the foreground color and the pen
	SetIfColor(saveColor);
	SetPenState(&savePenState);
}


//========================================================================================
// CLASS TEtchedFrame
//========================================================================================
#undef Inherited
#define Inherited TAdorner


MA_DEFINE_CLASS(TEtchedFrame);

//----------------------------------------------------------------------------------------
// TEtchedFrame constructor 
//----------------------------------------------------------------------------------------


TEtchedFrame::TEtchedFrame() :
	fInset(1, 1)
{
}

//----------------------------------------------------------------------------------------
// TEtchedFrame destructor
//----------------------------------------------------------------------------------------


TEtchedFrame::~TEtchedFrame()
{
}

//----------------------------------------------------------------------------------------
// TEtchedFrame::Draw: 
//----------------------------------------------------------------------------------------


void TEtchedFrame::Draw(TView* itsView, const CViewRect& /*area*/)	// override 
{
	//  Save off the current pen state and the foreground color
	PenState	savePenState;
	GetPenState(&savePenState);

	CRGBColor_AC saveColor(GetIfColor());
	PenNormal();

	CRect_AC QDArea(itsView->ViewToQD(itsView->GetAdornExtent()));

	SetIfColor(gRGBBlack);
	QDArea.Inset(fInset);
	FrameRect(QDArea);									// Draw the shadows 

	QDArea.Offset(1, 1);
	SetIfColor(gRGBWhite);
	FrameRect(QDArea);									// Draw the light edges 

	QDArea.Offset( -1, -1);
	SetCPixel(QDArea.left, QDArea.bottom, gRGBBlack);	// Finish the corners 
	SetCPixel(QDArea.right, QDArea.top, gRGBBlack);

	//  Restore the foreground color and the pen
	SetIfColor(saveColor);
	SetPenState(&savePenState);
}


//========================================================================================
// CLASS TSlowScroller
//========================================================================================
#undef Inherited
#define Inherited TScroller


MA_DEFINE_CLASS(TSlowScroller);

//----------------------------------------------------------------------------------------
// TSlowScroller destructor
//----------------------------------------------------------------------------------------


TSlowScroller::~TSlowScroller()
{
}

//----------------------------------------------------------------------------------------
// TSlowScroller::ScrollStep: 
//----------------------------------------------------------------------------------------


ViewCoordinate TSlowScroller::ScrollStep(VHSelect vhs, short partCode) // override
{
	ViewCoordinate scrollUnit = fScrollUnit[vhs];
	if (IsControlKeyDown())
		scrollUnit = 1;
	ViewCoordinate delta;

	switch (partCode)
	{
		case kControlUpButtonPart:
		case kControlDownButtonPart:
			delta = scrollUnit;
			break;
			
		case kControlPageUpPart:
		case kControlPageDownPart:
			if (IsControlKeyDown())
				delta = 1;
			else
				delta = fSize[vhs] - scrollUnit;
			break;
			
#if qDebug
		default:
			{
				CStr255_AC theString;
				ConcatNumber("TScroller::ScrollStep: bad part code =", partCode, theString);
				ProgramBreak_AC(theString);
			}
			break;
#endif

	}

	if ((partCode == kControlUpButtonPart) || (partCode == kControlPageUpPart))
		delta = -delta;

	// Constrain if necessary 
	if (fConstrain[vhs] && scrollUnit)
	{
		ViewCoordinate adjustment = (Max_AC(0, fTranslation[vhs] + delta)) % scrollUnit;
		if (adjustment)
			if (delta > 0)
				delta -= adjustment;
			else
				delta += (scrollUnit - adjustment);
	}

	CViewPoint deltaPt;
	deltaPt[vhs] = delta;

	DoScroll(deltaPt, kRedraw);
	return delta;
}

//----------------------------------------------------------------------------------------
// End of UDemoDialogs.More.cp


