//----------------------------------------------------------------------------------------
// UDemoDialogs.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDEMODIALOGS__
#include "UDemoDialogs.h"
#endif

// DemoDialogs

#ifndef __USTRUCTUREINSPECTORS__
#include "UStructureInspectors.h"
#endif

// MacApp

#ifndef __UBUTTON__
#include "UButton.h"
#endif

#ifndef __UCHECKBOX__
#include "UCheckbox.h"
#endif

#ifndef __UDRAWINGENVIRONMENT__
#include "UDrawingEnvironment.h"
#endif

#ifndef __UEDITTEXT__
#include "UEditText.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UKEYSELECTIONBEHAVIOR__
#include "UKeySelectionBehavior.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPattern__
#include "UPattern.h"
#endif

#ifndef __UPOPUP__
#include "UPopup.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __URADIO__
#include "URadio.h"
#endif

#ifndef __USUBSTITUTION__
#include "USubstitution.h"
#endif

#ifndef __UTABBEHAVIORS__
#include "UTabBehaviors.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __FONTS__
#include <Fonts.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

// Use Windows_AC.h instead of <windows.h> for XPlatform compatability.
#ifndef __Windows_AC__
#include "Windows_AC.h"
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif


#ifndef __STDLIB__
#include <stdlib.h>
#endif

//----------------------------------------------------------------------------------------
// Constants
const short kDynamicPopup = 300;

const short kNoOfLines = 10000;
const short kMaxScrollBar = 1000;
const short kTestIcon = 1000;
const short kTestAlert = 1000;
const short kTestPopup = 240;
const short kBaudPopup = 236;
const short kParityPopup = 237;
const short kPopup = 238;

const short kTextStylePopup = 239;
const short kFontHier = 230;
const short kStyleHier = 231;
const short kSizeHier = 232;

const short kTestPicture = 1000;
const short kTestPattern = 1000;
const short kTestStatic = 1200;

// Constants for Command Items
const short cFirstItem = 1001;
const short cProcedureViews = 1001;
const short cTemplateViews = 1002;
const short cMonthlyDialog = 1003;
const short cSaveDialog = 1004;
const short cModalBeepDialog = 1005;
const short cPageSetupDialog = 1006;
const short cHomeBrewControls = 1007;
const short cTabbingTest = 1008;
const short cCalculator = 1009;
const short cScrollingTest = 1011;
const short cFormatDialog = 1012;
const short cModelessBeepDialog = 1013;
const short cMessageWindoid = 1014;
const short cBalloons = 1015;
const short cAdornment = 1016;
const short cViewInspector = 1017;
const short cTargetInspector = 1018;
const short cLastItem = 1018;
const short cPopupExample = 1100;
const short cTextStyleChanged = 1102;
const short cResetTextStyle = 1103;

//----------------------------------------------------------------------------------------

struct FondRecord
{
	FamRec familyStuff;
	short noOfFonts;
	struct
	{
		short size;
		short style;
		short resID;
	} fontStuff[1001];
};


typedef FondRecord* FondPointer, ** FondHandle;

long gMonthlyValues[13];
IDType gMonthIDs[13];


//========================================================================================
// Global Functions
//========================================================================================
static bool GetFontNumber(CStr255_AC& fontName, short& fontNumber);

//========================================================================================
// CLASS TTestApplication
//========================================================================================
#undef Inherited
#define Inherited TApplication


MA_DEFINE_CLASS(TTestApplication);

//----------------------------------------------------------------------------------------
// TTestApplication constructor
//----------------------------------------------------------------------------------------


TTestApplication::TTestApplication() :
	TApplication(kFileType, kSignature)
{
	const char* monthIDs = "jan feb mar apr may jun jul aug sep oct nov dec ";

	fLaunchWithNewDocument = false;				// Suppress the creation of a new document at launch

	for (short i = 0; i < 12; ++i)
	{
		gMonthlyValues[i] = (i + 1) * 100;
	}

	BlockMove_AC(monthIDs, (Ptr)gMonthIDs, sizeof(gMonthIDs));

	// Suppress dead-stripping of the following classes 
	MA_REGISTER_CLASS(TDrawingEnvironment);
	MA_REGISTER_CLASS(TMonthlyDialog);
	MA_REGISTER_CLASS(TArrowsControl);
	MA_REGISTER_CLASS(TTemperatureCluster);
	MA_REGISTER_CLASS(TTemperatureConversionCluster);
	MA_REGISTER_CLASS(THomeBrewSlider);
	MA_REGISTER_CLASS(TFontListView);
	MA_REGISTER_CLASS(TSizeListView);
	MA_REGISTER_CLASS(TRadioIcon);
	MA_REGISTER_CLASS(TPageSetupDialog);
	MA_REGISTER_CLASS(TSumStaticText);
	MA_REGISTER_CLASS(TCalcDialog);
	MA_REGISTER_CLASS(TNumbersView);
	MA_REGISTER_CLASS(TNumberListView);
	MA_REGISTER_CLASS(TViewHierarchyView);
	MA_REGISTER_CLASS(TTargetChainView);
	MA_REGISTER_CLASS(TModelessBeepDialog);
	MA_REGISTER_CLASS(TScrollBar);

	MA_REGISTER_CLASS(TAdornmentDemo);
	MA_REGISTER_CLASS(TEmbossedFrame);
	MA_REGISTER_CLASS(TGrayFill);
	MA_REGISTER_CLASS(TEtchedFrame);
	MA_REGISTER_CLASS(TSlowScroller);


	MA_REGISTER_SIGNATURE(TEmbossedFrame, 'embo');
	MA_REGISTER_SIGNATURE(TGrayFill, 'gray');
	MA_REGISTER_SIGNATURE(TEtchedFrame, 'etch');
	MA_REGISTER_SIGNATURE(TRoundView, 'rndV');

	// to demonstrate support of dynamic menus with TPopup 
	CStr255_AC menuName("Fonts:");
	fMenuRef = NewMenu(kDynamicPopup, menuName);
	::ThrowIfNULL_AC(fMenuRef);
	AppendResMenu(fMenuRef, 'FONT');
	MAInsertInMenuTable(fMenuRef, kDynamicPopup);
}

//----------------------------------------------------------------------------------------
// TTestApplication destructor
//----------------------------------------------------------------------------------------


TTestApplication::~TTestApplication()
{
}

//----------------------------------------------------------------------------------------
// TTestApplication::DoMakeDocument: 
//----------------------------------------------------------------------------------------


TDocument* TTestApplication::DoMakeDocument(CommandNumber /*itsCommandNumber*/ ,
											TFile* /*itsFile*/  )	// override 
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeProcedureViews: 
//----------------------------------------------------------------------------------------


void TTestApplication::MakeProcedureViews(CommandNumber aCommandNumber)
{
	const CStr255_AC StyleName[8] = {
								  "Plain", "Bold", "Italic", "Underline", "Outline", "Shadow", "Condense", "Extend"};
	const CStr255_AC ParamSubs0 = "^0";
	const CStr255_AC ParamSubs1 = "^1";
	const CStr255_AC ExpansionSubs0 = "PARAM0";
	const CStr255_AC ExpansionSubs1 = "PARAM1";

	TView * aTestView = TH_new TView(gZeroVPt, CViewPoint(480, 1200), sizeFixed, sizeFixed);
	MAParamText(ParamSubs0, ExpansionSubs0);
	MAParamText(ParamSubs1, ExpansionSubs1);

	TWindow * aWindow = gViewServer->NewSimpleWindow(static_cast<ResNumber>(aCommandNumber), true, true, aTestView);
	::ThrowIfNULL_AC(aWindow);
	aWindow->SetFreeOnClosing(true);

	TStaticText * aStaticText = TH_new TStaticText(CViewPoint(50, 100), CViewPoint(300, 150), sizeVariable, sizeVariable, kTestStatic, 1);
	aTestView->AddSubView(aStaticText);

	aStaticText->InstallTextStyle(CTextStyle_AC(applFont, italic, 12, CRGBColor_AC(0, 0, 0xFFFF)), kDontRedraw);
	aStaticText->SetJustification(teCenter, kRedraw);

	TCluster * aCluster = TH_new TCluster(CViewPoint(100, 340), CViewPoint(150, 190), sizeVariable, sizeVariable, kNoResource, 0);
	aTestView->AddSubView(aCluster);
	aCluster->SetLabel("A Cluster!", kRedraw);

	for (short i = 0; i < 8; ++i)
	{
		TRadio * aRadio = TH_new TRadio(CViewPoint(10, (i + 1) * 20), CViewPoint(130, 20), sizeVariable, sizeVariable, gEmptyString, i == 2);
		aCluster->AddSubView(aRadio);
		aRadio->InstallTextStyle(CTextStyle_AC(systemFont, Style((1 << i) / 2), 12, gRGBBlack), kDontRedraw);
		aRadio->SetText(StyleName[i], kRedraw);
	}

	TCheckBox * aCheckBox = TH_new TCheckBox(CViewPoint(100, 540), CViewPoint(100, 20), sizeVariable, sizeVariable, "Check Box", false);
	aTestView->AddSubView(aCheckBox);
	aCheckBox->InstallTextStyle(CTextStyle_AC(applFont, 0, 12, gRGBBlack), kRedraw);

	TButton * aButton = TH_new TButton(CViewPoint(270, 540), CViewPoint(100, 28), sizeVariable, sizeVariable, "Push Button");
	aTestView->AddSubView(aButton);
	aButton->InstallTextStyle(CTextStyle_AC(applFont, NULL, 10, gRGBBlack), kDontRedraw);
	aButton->SetPenSize(CPoint_AC(3, 3));
	aButton->SetAdornment(adnRRect, kDontRedraw);
	aButton->Inset(CViewPoint(4, 4), kRedraw);

	TIcon * anIcon = TH_new TIcon(CViewPoint(111, 571), CViewPoint(34, 34), sizeFixed, sizeFixed, kTestIcon);	
	aTestView->AddSubView(anIcon);
	anIcon->SetAdornment(kFrame | adnShadow, kDontRedraw);
	anIcon->ViewEnable(true, kDontRedraw);
	anIcon->SetInset(CViewRect(1, 1, 1, 1), kDontRedraw);

	TIcon * anotherIcon = TH_new TIcon(CViewPoint(173, 570), CViewPoint(32, 32), sizeFixed, sizeFixed, kTestIcon);	
	anotherIcon->DimState(true, kDontRedraw);
	anotherIcon->ViewEnable(false, kDontRedraw);
	aTestView->AddSubView(anotherIcon);

	TPopup * aPopup = TH_new TPopup(CViewPoint(290, 589), CViewPoint(120, 22), sizeVariable, sizeVariable, kTestPopup, 2, 50, cPopupExample, 1, 0, teJustLeft, kDontUseAddResMenu, kNoIdentifier);
	aTestView->AddSubView(aPopup);

	TPopup * bPopup = TH_new TPopup(CViewPoint(290, 624), CViewPoint(120, 22), sizeVariable, sizeVariable, kBaudPopup, 1, 50, cPopupExample, 2, 0, teJustLeft, kDontUseAddResMenu, kNoIdentifier);
	aTestView->AddSubView(bPopup);

	TPopup * cPopup = TH_new TPopup(CViewPoint(290, 659), CViewPoint(120, 22), sizeVariable, sizeVariable, kParityPopup, 1, 50, cPopupExample, 3, 0, teJustLeft, kDontUseAddResMenu, kNoIdentifier);
	aTestView->AddSubView(cPopup);
	cPopup->fTextStyle.Set(applFont, NULL, 9, gRGBBlack);

	TPopup * dPopup = TH_new TPopup(CViewPoint(290, 694), CViewPoint(120, 22), sizeVariable, sizeVariable, kDynamicPopup, 1, 50, cPopupExample, 4, 0, teJustLeft, !kDontUseAddResMenu, 'FOND');
	aTestView->AddSubView(dPopup);

	TPicture * aPicture = TH_new TPicture(CViewPoint(100, 660), CViewPoint(0x18C - 0xF0 + 16, 0x12C - 0x8F + 16), sizeVariable, sizeVariable, kTestPicture);
	aTestView->AddSubView(aPicture);
	//	aPicture->fPenSize = CPoint_AC(0x00020002);
	aPicture->SetAdornment(adnRRect | adnShadow, kDontRedraw);
	aPicture->ViewEnable(true, kDontRedraw);
	aPicture->Inset(CViewPoint(8, 8), kDontRedraw);
	aPicture->SetLocalOrigin(CViewPoint(50, 50), kDontRedraw);
	aPicture->UpdateCoordinates();

	TPattern * aPattern = TH_new TPattern(CViewPoint(100, 850), CViewPoint(220, 25), sizeFixed, sizeFixed, kTestPattern, true);
	aTestView->AddSubView(aPattern);
	aPattern->ViewEnable(true, kDontRedraw);

	TEditText * anEditText = TH_new TEditText(CViewPoint(140, 620), CViewPoint(100, 22), 255);
	aTestView->AddSubView(anEditText);
	anEditText->InstallTextStyle(CTextStyle_AC(applFont, 0, 12, gRGBBlack), kDontRedraw);
	anEditText->SetJustification(teFlushRight, kDontRedraw);
	anEditText->SetText("EditText", kDontRedraw);

	aWindow->HandlePostCreate(NULL /* itsDocument */); // New requirement
	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TTestApplication::DoMenuCommand: 
//----------------------------------------------------------------------------------------


void TTestApplication::DoMenuCommand(CommandNumber aCommandNumber)// override 
{
	switch (aCommandNumber)
	{
		case cProcedureViews:
			MakeProcedureViews(aCommandNumber);
			break;
		case cTemplateViews:
			MakeTemplateViews(aCommandNumber);
			break;
		case cScrollingTest:
			MakeScrollingTest(aCommandNumber);
			break;
		case cMonthlyDialog:
			MakeMonthlyDialog(aCommandNumber);
			break;
		case cSaveDialog:
			MakeSaveDialog(aCommandNumber);
			break;
		case cModalBeepDialog:
			MakeModalBeepDialog(aCommandNumber);
			break;
		case cModelessBeepDialog:
			MakeDialog(aCommandNumber);
			break;
		case cMessageWindoid:
			MakeDialog(aCommandNumber);
			break;
		case cViewInspector:
		case cTargetInspector:
			{
				TWindow * aWindow;

				::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
				aWindow->SetResizeLimits(CPoint_AC(210, 180), CPoint_AC(210, 1000));
				aWindow->Open();
			}
			break;
		case cBalloons:
			MakeBalloonWindow(aCommandNumber);
			break;
		case cAdornment:
			MakeDialog(aCommandNumber);
			break;
		case cFormatDialog:
			MakeFormatDialog(aCommandNumber);
			break;
		case cPageSetupDialog:
			MakePageSetupDialog(aCommandNumber);
			break;
		case cHomeBrewControls:
			MakeHomeBrewDialog(aCommandNumber);
			break;
		case cTabbingTest:
			MakeDialog(aCommandNumber);
			break;
		case cCalculator:
			MakeCalculator(aCommandNumber);
			break;
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TTestApplication::DoSetupMenus: 
//----------------------------------------------------------------------------------------


void TTestApplication::DoSetupMenus()			// override 
{
	Inherited::DoSetupMenus();

	if (!InModalState())
	{
		for (short i = cFirstItem; i <= cLastItem; ++i)
			Enable(i, TRUE);
	
		Enable(cAdornment, HasColorQD_AC());
	
		// If the Dynamic menu is not in the menubar then insert it 
		if (!::GetMenuHandle((*fMenuRef)->menuID))
			MAInsertMenu(fMenuRef, (*fMenuRef)->menuID, -1);
	}
}


//----------------------------------------------------------------------------------------
// TTestApplication::MakeTemplateViews: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeTemplateViews(CommandNumber aCommandNumber)
{
	TWindow * aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL);
	::ThrowIfNULL_AC(aWindow);

	TDialogView * aDialogView = dynamic_cast_AC(TDialogView*,(aWindow->FindSubView('DLOG')));

	TStdPrintHandler * aPrintHandler = TH_new TStdPrintHandler(NULL,		// its document 
									aDialogView,// its view 
									!kFixedSize,// horizontal page size is variable 
									kFixedSize);// vertical page size is fixed 

	TView * popupContainer = dynamic_cast_AC(TView*,(gViewServer->DoCreateViews(NULL, aDialogView, cPopupExample, gZeroVPt)));
	
	MAParamText("^0", "PARAM0");
	MAParamText("^1", "PARAM1");
	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeScrollingTest: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeScrollingTest(CommandNumber aCommandNumber)
{
	TWindow* aWindow;
	CPoint_AC minSize;

	::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	SetPt(minSize, 315, 260);
	aWindow->SetResizeLimits(minSize, aWindow->GetResizeLimits()[botRight]);
	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeMonthlyDialog: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeMonthlyDialog(CommandNumber aCommandNumber)
{
	TWindow* aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL);
	::ThrowIfNULL_AC(aWindow);

	TMonthlyDialog * aMonthlyDialog = dynamic_cast<TMonthlyDialog*>((aWindow->FindSubView('DLOG')));
	if (aMonthlyDialog)
		aMonthlyDialog->StuffValues();

	// Add a tabbing behavior to the window. This shows how to prevent cross-window
	// tabbing when there are floating windows present
	aWindow->AddBehavior(TH_new TViewTabber);

	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeSaveDialog: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeSaveDialog(CommandNumber aCommandNumber)
{
	CAutoPtr_AC<TWindow> aWindow(gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	::ThrowIfNULL_AC(aWindow.get());

	IDType dismisser = aWindow->PoseModally();

#if qDebug
	if (dismisser == 'yes ')
		ProgramBreak_AC("The user said yes.\n");
	else if (dismisser == 'no  ')
		ProgramBreak_AC("The user said no.\n");
	else if (dismisser == 'cncl')
		ProgramBreak_AC("The user cancelled the dialog.\n");
	else
		ProgramBreak_AC("I don't know how the user responded\n");
#endif

	aWindow->Close();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeModalBeepDialog: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeModalBeepDialog(CommandNumber aCommandNumber)
{
	CAutoPtr_AC<TWindow> aWindow(gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	::ThrowIfNULL_AC(aWindow.get());

	IDType dismisser = aWindow->PoseModally();
	long n = dismisser == 'ok  ' ? dynamic_cast_AC(TNumberText*,(aWindow->FindSubView('numb')))->GetValue() : 0;

	for (long i = 1; i <= n; ++i)
		gApplication->Beep(2);

	aWindow->Close();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeDialog: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeDialog(CommandNumber aCommandNumber)
{
	TWindow* aWindow;

	::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeBalloonWindow: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeBalloonWindow(CommandNumber aCommandNumber)
{
	TWindow* aWindow;
	TView* aSubView;

	::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	aSubView = aWindow->FindSubView('ADN1');
	aSubView->SetHelpID(128);
	aSubView->SetHelpIndex(1);
	aSubView->SetCursorID(plusCursor);

	aSubView = aWindow->FindSubView('ADN2');
	aSubView->SetHelpID(128);
	aSubView->SetHelpIndex(2);
	aSubView->SetHandlesCursor(false);

	aSubView = aWindow->FindSubView('ADN3');
	aSubView->SetHelpID(128);
	aSubView->SetHelpIndex(3);

	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeFormatDialog: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeFormatDialog(CommandNumber aCommandNumber)
{
	CAutoPtr_AC<TWindow> aWindow(gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	::ThrowIfNULL_AC(aWindow.get());

	TDialogView * theDialog = dynamic_cast_AC(TDialogView*,(aWindow->FindSubView('DLOG')));
	SetupTheFontView(theDialog);

	IDType dismisser = aWindow->PoseModally();
#if qDebug
	if (dismisser != 'ok  ')
		ProgramBreak_AC("The dialog was cancelled.\n");
#endif

	aWindow->Close();
}

//----------------------------------------------------------------------------------------
// TTestApplication::SetupTheFontView: 
//----------------------------------------------------------------------------------------
void TTestApplication::SetupTheFontView(TDialogView* theDialog)
{
	TFontListView* theFontListView;
	TSizeListView* theSizeListView;

	theSizeListView = dynamic_cast_AC(TSizeListView*,(theDialog->FindSubView('slst')));
	theSizeListView->fSelection = 0;
	theSizeListView->fSelectedSize = 9;
	theFontListView = dynamic_cast_AC(TFontListView*,(theDialog->FindSubView('flst')));
	theFontListView->InitFontList();
	theFontListView->SelectItem(1, kDontExtend, kHighlight, kSelect);
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakePageSetupDialog: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakePageSetupDialog(CommandNumber aCommandNumber)
{
	CAutoPtr_AC<TWindow> aWindow(gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	::ThrowIfNULL_AC(aWindow.get());

	TPageSetupDialog * theDialog = dynamic_cast_AC(TPageSetupDialog*,(aWindow->FindSubView('DLOG')));

	IDType dismisser = aWindow->PoseModally();
#if qDebug
	if (dismisser == 'ok  ')
	{
	}
	else
		ProgramBreak_AC("The dialog was cancelled.\n");
#endif

	aWindow->Close();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeCalculator: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeCalculator(CommandNumber aCommandNumber)
{
	TWindow* aWindow;
	TCalcDialog* aCalcDialog;

	::ThrowIfNULL_AC(aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL));
	aCalcDialog = dynamic_cast_AC(TCalcDialog*,(aWindow->FindSubView('DLOG')));
	aCalcDialog->ClearSum();
	aWindow->Open();
}

//----------------------------------------------------------------------------------------
// TTestApplication::MakeHomeBrewDialog: 
//----------------------------------------------------------------------------------------
void TTestApplication::MakeHomeBrewDialog(CommandNumber aCommandNumber)
{
	TWindow* aWindow = aWindow = gViewServer->NewTemplateWindow(static_cast<ResNumber>(aCommandNumber), NULL);

	::ThrowIfNULL_AC(aWindow);
	TScrollBar * firstScrollBar = dynamic_cast_AC(TScrollBar*,(aWindow->FindSubView('Scr1')));
	TScrollBar * secondScrollBar = dynamic_cast_AC(TScrollBar*,(aWindow->FindSubView('Scr2')));

	firstScrollBar->AddBehavior(TH_new TChangeBehavior);
	secondScrollBar->AddBehavior(TH_new TChangeBehavior);

	TUpdateBehavior* firstUpdateBehavior = TH_new TUpdateBehavior;
	firstScrollBar->AddBehavior(firstUpdateBehavior);

	TUpdateBehavior* secondUpdateBehavior = TH_new TUpdateBehavior;
	secondScrollBar->AddBehavior(secondUpdateBehavior);

	firstScrollBar->AddDependent(secondUpdateBehavior);
	secondScrollBar->AddDependent(firstUpdateBehavior);

	aWindow->Open();
}


//========================================================================================
// CLASS TNumbersView
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TNumbersView);

//----------------------------------------------------------------------------------------
// TNumbersView destructor
//----------------------------------------------------------------------------------------


TNumbersView::~TNumbersView()
{
}

//----------------------------------------------------------------------------------------
// TNumbersView::ReadFields: 
//----------------------------------------------------------------------------------------

void TNumbersView::ReadFields(CStream_AC* aStream)	// override 
{
	Inherited::ReadFields(aStream);

	fWantsToBeTarget = true;
}

//----------------------------------------------------------------------------------------
// TNumbersView::Draw: 
//----------------------------------------------------------------------------------------


void TNumbersView::Draw(const CViewRect& area)
{
	CViewRect viewRect(area);
	--viewRect[botRight];

	long firstLine = (viewRect.top / 16) + 1;
	long lastLine = (viewRect.bottom / 16) + 1;

	SetPortTextStyle(gSystemStyle);
	for (long i = firstLine; i <= lastLine; ++i)
	{
		MoveToPt(ViewToQD(CViewPoint(0, i * 16)));

		CStr255_AC s;
		NumToString(i, s);
		DrawString(s);
	}

	Inherited::Draw(area);
}

//========================================================================================
// CLASS TNumberListView
//========================================================================================
#undef Inherited
#define Inherited TTextListView


MA_DEFINE_CLASS(TNumberListView);

//----------------------------------------------------------------------------------------
// TNumberListView destructor
//----------------------------------------------------------------------------------------


TNumberListView::~TNumberListView()
{
}

//----------------------------------------------------------------------------------------
// TNumberListView::DoPostCreate: 
//----------------------------------------------------------------------------------------

void TNumberListView::DoPostCreate(TDocument* itsDocument)// override 
{
	Inherited::DoPostCreate(itsDocument);

	fWantsToBeTarget = true;					// !!! should be set in 3.0 'View' resource
}

//----------------------------------------------------------------------------------------
// TNumberListView::BecameWindowTarget: 
//----------------------------------------------------------------------------------------

void TNumberListView::BecameWindowTarget()		// override 
{
	DoHighlightSelection(fActiveHL, hlOn);
	fActiveHL = hlOn;

	Inherited::BecameWindowTarget();
}

//----------------------------------------------------------------------------------------
// TNumberListView::ResignedWindowTarget: 
//----------------------------------------------------------------------------------------
void TNumberListView::ResignedWindowTarget()	// override 
{
	DoHighlightSelection(fActiveHL, hlDim);
	fActiveHL = hlDim;

	Inherited::ResignedWindowTarget();
}

//----------------------------------------------------------------------------------------
// TNumberListView::GetItemText: 
//----------------------------------------------------------------------------------------


CStr255_AC TNumberListView::GetItemText(GridCoordinate anItem) const// override 
{
	CStr255_AC returnVal;
	NumToString(anItem, returnVal);
	
	return returnVal;
}



//========================================================================================
// CLASS TModelessBeepDialog
//========================================================================================
#undef Inherited
#define Inherited TWindow


MA_DEFINE_CLASS(TModelessBeepDialog);

//----------------------------------------------------------------------------------------
// TModelessBeepDialog destructor
//----------------------------------------------------------------------------------------


TModelessBeepDialog::~TModelessBeepDialog()
{
}

//----------------------------------------------------------------------------------------
// TModelessBeepDialog::DoEvent: 
//----------------------------------------------------------------------------------------


void TModelessBeepDialog::DoEvent(EventNumber eventNumber,
								  TEventHandler* source,
								  TEvent* event)// override 
{
	switch (eventNumber)
	{
		case mButtonHit:						// Default button and Enter or Return key
			{
				long maxi = dynamic_cast_AC(TNumberText*,(FindSubView('numb')))->GetValue();
				if (IsHierarchyValid())
					for (long i = 1; i <= maxi; ++i)
						gApplication->Beep(2);
				break;
			}
		default:
			Inherited::DoEvent(eventNumber, source, event);
	}
}


//========================================================================================
// CLASS TMonthlyDialog
//========================================================================================
#undef Inherited
#define Inherited TDialogView


MA_DEFINE_CLASS(TMonthlyDialog);

//----------------------------------------------------------------------------------------
// TMonthlyDialog destructor
//----------------------------------------------------------------------------------------


TMonthlyDialog::~TMonthlyDialog()
{
}

//----------------------------------------------------------------------------------------
// TMonthlyDialog::Close: 
//----------------------------------------------------------------------------------------


void TMonthlyDialog::Close()					// override 
{
	if (IsHierarchyValid())
	{
		for (short i = 0; i < 12; ++i)
		{
			gMonthlyValues[i] = dynamic_cast_AC(TNumberText*,FindSubView(gMonthIDs[i]))->GetValue();
			Inherited::Close();
		}

	}
	else
		::ThrowException_AC(noErr, 0);						// Silent Failure 
}

//----------------------------------------------------------------------------------------
// TMonthlyDialog::DoKeyEvent: 
//----------------------------------------------------------------------------------------


void TMonthlyDialog::DoKeyEvent(TToolboxEvent* event)// override 
{
	// If option-tab is pressed, deselect the current edit text and leave nothing
	// selected. This is for test purposes only. It is not part of the Macintosh user
	// interface.
	if ((event->GetText() == chTab) && (event->IsOptionKeyPressed()))
		GetWindow()->BecomeTarget();			// Formerly: DoSelectEditText(NULL, false); 
	else
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TMonthlyDialog::StuffValues: 
//----------------------------------------------------------------------------------------


void TMonthlyDialog::StuffValues()
{
	for (int i = 0; i < 12; ++i)
		dynamic_cast_AC(TNumberText*,FindSubView(gMonthIDs[i]))->SetValue(gMonthlyValues[i], kDontRedraw);
}


//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// GetFontNumber: 
//----------------------------------------------------------------------------------------


// From IM-VI p. 12-16:

bool GetFontNumber(CStr255_AC& fontName, short& fontNumber)
{
	bool returnVal = TRUE;

	GetFNum(fontName, &fontNumber);
	if (!fontNumber)
	{
		CStr255_AC systemFontName;

		// Either the font was not found, or it is the system font

		GetFontName(0, systemFontName);
		returnVal = systemFontName == fontName;
	}

	return returnVal;
}


//========================================================================================
// CLASS TFontListView
//========================================================================================
#undef Inherited
#define Inherited TTextListView


MA_DEFINE_CLASS(TFontListView);

//----------------------------------------------------------------------------------------
// TFontListView destructor 
//----------------------------------------------------------------------------------------


TFontListView::~TFontListView()
{
}

//----------------------------------------------------------------------------------------
// TFontListView::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TFontListView::DoPostCreate(TDocument* itsDocument)	// override 
{
	Inherited::DoPostCreate(itsDocument);

	fWantsToBeTarget = true;
}

//----------------------------------------------------------------------------------------
// TFontListView::DrawCell 
//----------------------------------------------------------------------------------------


void TFontListView::DrawCell(const GridCell& aCell, const CViewRect& aRect)	// Override
{
	CRect_AC qdArea(ViewToQD(aRect));
	GridCoordinate index = aCell.v;
	CStr255_AC fontName(GetItemText(index));

	// Set the font before drawing
	short fontNum = (*fFontList)[index];
	::TextFont(fontNum);

	::TruncString(qdArea.right - qdArea.left, fontName, truncEnd);
	MADrawString(fontName, qdArea, fJustification, fPreferOutline);
}

//----------------------------------------------------------------------------------------
// TFontListView::InitFontList: 
//----------------------------------------------------------------------------------------


void TFontListView::InitFontList()
{
	AddBehavior(TH_new TKeySelectionBehavior('KYSL'));

	fFontList = NULL;
	short noOfFonds = CountResources('FOND');
	if (noOfFonds > kMaxFonds)
		noOfFonds = kMaxFonds;
	FontListPtr pFondIDs = FontListPtr(NewPermPtr((noOfFonds + 1) * 2));// Our indexing scheme is one-based
	// element zero is unused, so we'd better get one more item

	CStr255_AC aString(static_cast<unsigned char>(chSpace));
	for (int i = 1; i <= noOfFonds; ++i)
	{											// put each FOND's id in the list 
		(*pFondIDs)[i] = FondAfter(aString, noOfFonds);// in alphabetical order 
		if (aString.IsEmpty())				// we finished early 
		{
			noOfFonds = i - 1;
			break;
		}
	}

	fFontList = pFondIDs;
	InsRowLast(noOfFonds, 16);
}

//----------------------------------------------------------------------------------------
// TFontListView::FondAfter: 
//----------------------------------------------------------------------------------------


short TFontListView::FondAfter(CStr255_AC& fontName, short noOfFonds)
{
// Find the FOND whose name follows fontName alphabetically, and return its id and name 
	bool foundFOND = false;
	CStr255_AC lastName = "~~~~~~~~";
	short lastID = 0;
	for (short index = 1; index <= noOfFonds; ++index)
	{
		Handle theFondResource = GetIndResource('FOND', index);

		short thisResID;
		ResType itsType;
		CStr255_AC thisName;
		GetResInfo(theFondResource, &thisResID, &itsType, thisName);

		short thisID;
		if (GetFontNumber(thisName, thisID))	// Remember the font number, not its resource ID
		{
			if ((thisName > fontName) && (thisName < lastName))
			{
				lastID = thisID;
				lastName = thisName;
				foundFOND = true;
			}
		}
	}

	fontName.Empty();

	if (foundFOND)
		fontName = lastName;					// Skip duplicate FOND names 

	return lastID;
}

//----------------------------------------------------------------------------------------
// TFontListView::DoKeyEvent - handle selection keys up-arrow and down-arrow
//----------------------------------------------------------------------------------------


void TFontListView::DoKeyEvent(TToolboxEvent* event)	// Override
{
	bool handled = false;
	if (fNumOfRows > 0)	// Do nothing if there are no rows.
	{
		if (event->GetText() == chDown || event->GetText() == chUp)
		{
			GridCoordinate selectedItem = FirstSelectedItem();	// current selection
			GridCoordinate itemIndex = selectedItem;
			if (event->GetText() == chDown)
			{
				if (itemIndex < fNumOfRows)
					++itemIndex;
			}
			else	// chUp
			{
				if (itemIndex > 1)
					--itemIndex;
			}

			if (itemIndex != selectedItem)
			{
				SelectItem(itemIndex, false, true, true);
				ScrollSelectionIntoView(true);
			}
			handled = true;
		}
	}

	if (!handled)
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TFontListView::GetItemText: 
//----------------------------------------------------------------------------------------


CStr255_AC TFontListView::GetItemText(GridCoordinate anItem) const
{
	CStr255_AC returnVal;
	GetFontName((*fFontList)[anItem], returnVal);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TFontListView::SelectItem: 
//----------------------------------------------------------------------------------------


void TFontListView::SelectItem(GridCoordinate anItem,
							   bool extendSelection,
							   bool highlight,
							   bool select)
{
	Inherited::SelectItem(anItem, extendSelection, highlight, select);

	if (select)
	{
		TView * aView = dynamic_cast_AC(TView*,GetWindow())->FindSubView('slst');
		::ThrowIfNULLResource_AC((Handle)aView);
		dynamic_cast_AC(TSizeListView*,aView)->InstallFontFamily((*fFontList)[anItem]);
	}
}


//========================================================================================
// CLASS TSizeListView
//========================================================================================
#undef Inherited
#define Inherited TTextListView


MA_DEFINE_CLASS(TSizeListView);

//----------------------------------------------------------------------------------------
// TSizeListView destructor
//----------------------------------------------------------------------------------------


TSizeListView::~TSizeListView()
{
}

//----------------------------------------------------------------------------------------
// TSizeListView::GetItemText: 
//----------------------------------------------------------------------------------------


CStr255_AC TSizeListView::GetItemText(GridCoordinate anItem) const// override 
{
	CStr255_AC returnVal;
	NumToString(fFontSizeList[anItem - 1], returnVal);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TSizeListView::InstallFontFamily: 
//----------------------------------------------------------------------------------------


void TSizeListView::InstallFontFamily(short theFontNumber)
{
	short noOfSizes = 0;
	short sizeToSelect = 0;

	// We only check a specific set of Font sizes, as recommended in IM VI p. 12-15	
	for (short fontSize = 9; fontSize <= 96; ++fontSize)
	{
		// Check the same set of font sizes that MPW does: every point size from 9 to 24,
		// then every 6th point size up to size 96.
		if (fontSize <= 24 || (fontSize % 6 == 0))
		{
			// If this font size is available & there is room in the table
			if (RealFont(theFontNumber, fontSize) && (noOfSizes < kMaxSizes))
			{
				// then remember this value
				fFontSizeList[noOfSizes] = fontSize;

				noOfSizes = noOfSizes + 1;
				if (fontSize == fSelectedSize)
					sizeToSelect = noOfSizes;
			}
		}
	}

	SetNumberOfItems(noOfSizes);
	SelectItem(sizeToSelect, kDontExtend, kDontHighlight, kSelect);
	ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TSizeListView::SelectItem: 
//----------------------------------------------------------------------------------------


void TSizeListView::SelectItem(GridCoordinate anItem,
							   bool extendSelection,
							   bool highlight,
							   bool select)	// override 
{
	Inherited::SelectItem(anItem, extendSelection, highlight, select);

	if (select && anItem)
	{
		CStr255_AC aString(GetItemText(anItem));

		TNumberText * theSizeView = dynamic_cast_AC(TNumberText*,(GetWindow()->FindSubView('size')));
		theSizeView->SetText(aString, kRedraw);
		theSizeView->SetSelection(0, SHRT_MAX, kRedraw);

		long fontSize;
		StringToNum(aString, &fontSize);
		fSelectedSize = static_cast<short>(fontSize);
	}
}

//----------------------------------------------------------------------------------------
// TSizeListView::SetNumberOfItems: 
//----------------------------------------------------------------------------------------


void TSizeListView::SetNumberOfItems(GridCoordinate aNumber)
{
	SelectItem(0, kDontExtend, kHighlight, kSelect);

	if (fNumOfRows > aNumber)
		DelItemFirst(fNumOfRows - aNumber);
	else if (fNumOfRows < aNumber)
		InsItemFirst(aNumber - fNumOfRows);
}


//========================================================================================
// CLASS TPageSetupDialog
//========================================================================================
#undef Inherited
#define Inherited TDialogView


MA_DEFINE_CLASS(TPageSetupDialog);

//----------------------------------------------------------------------------------------
// TPageSetupDialog constructor 
//----------------------------------------------------------------------------------------


TPageSetupDialog::TPageSetupDialog() :
	fOrientation(vSel),
	fTallAdjusted(NULL),
	fVerticalOrientation(NULL),
	fHorizontalOrientation(NULL)
{
}

//----------------------------------------------------------------------------------------
// TPageSetupDialog destructor
//----------------------------------------------------------------------------------------


TPageSetupDialog::~TPageSetupDialog()
{
}

//----------------------------------------------------------------------------------------
// TPageSetupDialog::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TPageSetupDialog::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	fTallAdjusted = dynamic_cast_AC(TCheckBox*,(FindSubView('tall')));
	fVerticalOrientation = dynamic_cast_AC(TRadioIcon*,(FindSubView('vert')));
	fHorizontalOrientation = dynamic_cast_AC(TRadioIcon*,(FindSubView('horz')));

	fVerticalOrientation->HiliteState(fOrientation == vSel, kDontRedraw);
	fHorizontalOrientation->HiliteState(fOrientation == hSel, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TPageSetupDialog::DoEvent: 
//----------------------------------------------------------------------------------------


void TPageSetupDialog::DoEvent(EventNumber eventNumber,
							   TEventHandler* source,
							   TEvent* event)	// override 
{
	VHSelect newOrientation;

	switch (eventNumber)
	{
		case mIconHit:
			{
				if (source == fVerticalOrientation)
					newOrientation = vSel;
				else
					newOrientation = hSel;
				if (newOrientation != fOrientation)
				{
					fVerticalOrientation->HiliteState(newOrientation == vSel, kRedraw);
					fHorizontalOrientation->HiliteState(newOrientation == hSel, kRedraw);
					fTallAdjusted->DimState(newOrientation == hSel, kRedraw);
					fTallAdjusted->SetState(false, kRedraw);
					fOrientation = newOrientation;
				}
				break;
			}
		default:
			Inherited::DoEvent(eventNumber, source, event);
	}
}



//========================================================================================
// CLASS TRadioIcon
//========================================================================================
#undef Inherited
#define Inherited TIcon


MA_DEFINE_CLASS(TRadioIcon);

//----------------------------------------------------------------------------------------
// TRadioIcon destructor
//----------------------------------------------------------------------------------------


TRadioIcon::~TRadioIcon()
{
}

//----------------------------------------------------------------------------------------
// TRadioIcon::TrackMouse: 
//----------------------------------------------------------------------------------------


void TRadioIcon::TrackMouse(TrackPhase aTrackPhase,
							CViewPoint&		/*anchorPoint*/,
							CViewPoint&		/*previousPoint*/,
							CViewPoint&		/*nextPoint*/,
							bool			/*mouseDidMove*/)	// override 
{
	if (aTrackPhase == trackBegin)
		HandleEvent(mIconHit, this, NULL);
}


//========================================================================================
// CLASS TRoundView
//========================================================================================
#undef Inherited
#define Inherited TControl


MA_DEFINE_CLASS(TRoundView);

//----------------------------------------------------------------------------------------
// TRoundView constructor
//----------------------------------------------------------------------------------------
TRoundView::TRoundView()
{
	SetOpacity(true);
}


//----------------------------------------------------------------------------------------
// TRoundView destructor
//----------------------------------------------------------------------------------------
TRoundView::~TRoundView()
{
}

//----------------------------------------------------------------------------------------
// TRoundView::Draw
//----------------------------------------------------------------------------------------
void TRoundView::Draw(const CViewRect& area)
{
	EraseRect(&ViewToQD(area));

	// If we contain the mouse, draw a little differently
	if (fgSetCursorView == this)
	{
		CPoint_AC centerView(ViewToQD(GetExtent().GetSize()));
		centerView.h /= 4;
		centerView.v /= 3;
		
		::SetIfColor(gRGBBlack);
		MoveToPt(centerView);
		::DrawString(CStr255_AC("Hi Mom!"));
	}
}

//----------------------------------------------------------------------------------------
// TRoundView::DoSetCursor
//----------------------------------------------------------------------------------------
void TRoundView::DoSetCursor(const CViewPoint& globalMouse,
							 RgnHandle cursorRegion)
{
	Inherited::DoSetCursor(globalMouse, cursorRegion);
	

	// We'll simply redraw ourselves to indicate that we contain the mouse
	DrawContents();
	Focus();	// our caller expects us to still be focused.
}

//----------------------------------------------------------------------------------------
// TRoundView::DoUnsetCursor
//----------------------------------------------------------------------------------------
void TRoundView::DoUnsetCursor()
{
	// We'll simply redraw ourselves to indicate that we contain the mouse
	DrawContents();
}

//----------------------------------------------------------------------------------------
// TRoundView::GetExtentRegion: 
//----------------------------------------------------------------------------------------


void TRoundView::GetExtentRegion(RgnHandle itsExtentRgn)// override 
{
	//
	// This is a simple example of a view that has a non-rectangular region.
	// For simplicity we are actually constructing the region in this method.
	// If we wanted, we could construct the region some other time (ReadFrom/DoPostCreate/etc.)
	// and store it. Then we would simply copy the region in this method.
	//

	OpenRgn();
	FrameOval(&GetQDExtent());
	CloseRgn(itsExtentRgn);
}

//----------------------------------------------------------------------------------------
// End of UDemoDialogs.cp


