// UDemoDialogs.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved. 

#ifndef __UDEMODIALOGS__
#define __UDEMODIALOGS__

// ACS

#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCLUSTER__
#include "UCluster.h"
#endif

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#ifndef __UDIALOGVIEW__
#include "UDialogView.h"
#endif

#ifndef __UGRIDVIEW__
#include "UGridView.h"
#endif

#ifndef __UICON__
#include "UIcon.h"
#endif

#ifndef __UPICTURE__
#include "UPicture.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USTATICTEXT__
#include "UStaticText.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

// ToolBox

// for PicHandle
#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif


//========================================================================================
//	Constants
//========================================================================================

// Command numbers 
const short cMakePicture = 1000;

const OSType kSignature = 'SS11';				// Application signature 
const OSType kFileType = 'SF11';				// File-type code used for document files
												// created by this application
const short kTestWindowID = 1001;				// This is passed to NewSimpleWindow as 
												// the resource ID of the the WIND Resource
												// which defines the window for this
												// application's documents

const short kMaxFonds = 100;					// Max number of FONDs the FontList holds 
const short kMaxSizes = 30;						// Max number of Font sizes the SizeList holds

//========================================================================================
//	Types
//========================================================================================

typedef enum
{
	noOperator, divOperator, mulOperator, subOperator, addOperator
} CalcOperator;

typedef short FontList[kMaxFonds];				// FOND resource IDs 
typedef FontList* FontListPtr;


//========================================================================================
// CLASS DECLARATIONS
//========================================================================================

//----------------------------------------------------------------------------------------
// class TTestApplication
//----------------------------------------------------------------------------------------

class TTestApplication : public TApplication
{
	MA_DECLARE_CLONE;

public:
	TTestApplication();

	virtual ~TTestApplication();

	virtual TDocument* DoMakeDocument(CommandNumber/*itsCommandNumber*/ ,
									  TFile*	/*itsFile*/ );// override 
	// Overridden to do nothing since we don't have a document 

	virtual void MakeProcedureViews(CommandNumber aCommandNumber);

	virtual void DoMenuCommand(CommandNumber aCommandNumber);// override 

	virtual void DoSetupMenus();				// override

protected:										// the procedures to do the views by procedures!!
	virtual void MakeTemplateViews(CommandNumber aCommandNumber);

	virtual void MakeScrollingTest(CommandNumber aCommandNumber);

	virtual void MakeMonthlyDialog(CommandNumber aCommandNumber);

	virtual void MakeSaveDialog(CommandNumber aCommandNumber);

	virtual void MakeModalBeepDialog(CommandNumber aCommandNumber);

	virtual void MakeDialog(CommandNumber aCommandNumber);

	virtual void MakeBalloonWindow(CommandNumber aCommandNumber);

	virtual void MakeFormatDialog(CommandNumber aCommandNumber);

	virtual void SetupTheFontView(TDialogView* theDialog);

	virtual void MakePageSetupDialog(CommandNumber aCommandNumber);

	virtual void MakeCalculator(CommandNumber aCommandNumber);

	virtual void MakeHomeBrewDialog(CommandNumber aCommandNumber);

public:
	MenuRef fMenuRef;
};


//----------------------------------------------------------------------------------------
// class TNumbersView
//----------------------------------------------------------------------------------------

class TNumbersView : public TView
{
	MA_DECLARE_CLONE;

public:
	inline TNumbersView() { }

	virtual ~TNumbersView();

	virtual void ReadFields(CStream_AC* aStream);	// override 

	virtual void Draw(const CViewRect& area);
};


//----------------------------------------------------------------------------------------
// class TNumberListView
//----------------------------------------------------------------------------------------

class TNumberListView : public TTextListView
{
	MA_DECLARE_CLONE;

public:
	inline TNumberListView() { }

	virtual ~TNumberListView();

	virtual void DoPostCreate(TDocument* itsDocument);// override 

	virtual void BecameWindowTarget();			// override 

	virtual void ResignedWindowTarget();		// override 

	virtual CStr255_AC GetItemText(GridCoordinate anItem) const;	// override 
};


//----------------------------------------------------------------------------------------
// class TMonthlyDialog
//----------------------------------------------------------------------------------------

class TMonthlyDialog : public TDialogView
{
	MA_DECLARE_CLONE;

public:
	inline TMonthlyDialog() { }

	virtual ~TMonthlyDialog();

	virtual void DoKeyEvent(TToolboxEvent* event);// override 

	virtual void Close();						// override 

	virtual void StuffValues();
};


//----------------------------------------------------------------------------------------
// class TModelessBeepDialog
//----------------------------------------------------------------------------------------

class TModelessBeepDialog : public TWindow
{
	MA_DECLARE_CLONE;

public:
	inline TModelessBeepDialog() { }

	virtual ~TModelessBeepDialog();

	virtual void DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event);		// override 
};


//----------------------------------------------------------------------------------------
// class TFontListView
//----------------------------------------------------------------------------------------

class TFontListView : public TTextListView
{
	MA_DECLARE_CLONE;

public:
	inline TFontListView() { }

	virtual ~TFontListView();

	virtual void DoPostCreate(TDocument* itsDocument);	// override 

	virtual void DrawCell(const GridCell& aCell, const CViewRect& aRect);	// Override

	virtual void InitFontList();
	// Build an array of ids of font resources whose names are in alphabetical order 

	virtual void DoKeyEvent(TToolboxEvent* event);	// Override
	// Handle arrow key selection

	virtual CStr255_AC GetItemText(GridCoordinate anItem) const;	// override 

	virtual void SelectItem(GridCoordinate anItem,
							bool extendSelection,
							bool highlight,
							bool select);	// override 

protected:
	virtual short FondAfter(CStr255_AC& fontName, short noOfFonds);

public:
	TemplateTempPtr<FontListPtr> fFontList;						// font resource ids 
};


//----------------------------------------------------------------------------------------
// class TSizeListView
//----------------------------------------------------------------------------------------

class TSizeListView : public TTextListView
{
	MA_DECLARE_CLONE;

public:
	inline TSizeListView() { }

	virtual ~TSizeListView();

	virtual CStr255_AC GetItemText(GridCoordinate anItem) const;	// override 

	virtual void InstallFontFamily(short theFondID);

	virtual void SelectItem(GridCoordinate anItem,
							bool extendSelection,
							bool highlight,
							bool select);	// override 

	virtual void SetNumberOfItems(GridCoordinate aNumber);

public:
	short fFontSizeList[kMaxSizes];
	short fSelection;
	short fSelectedSize;
};


//----------------------------------------------------------------------------------------
// class TRadioIcon
//----------------------------------------------------------------------------------------

class TRadioIcon : public TIcon
{
	MA_DECLARE_CLONE;

public:											// Used in the Page Setup dialog 
	inline TRadioIcon() { }

	virtual ~TRadioIcon();

	virtual void TrackMouse(TrackPhase aTrackPhase,
							CViewPoint&			/*anchorPoint*/ ,
							CViewPoint&			/*previousPoint*/ ,
							CViewPoint&			/*nextPoint*/ ,
							bool				/*mouseDidMove*/);// override 
};


//----------------------------------------------------------------------------------------
// class TPageSetupDialog
//----------------------------------------------------------------------------------------

class TPageSetupDialog : public TDialogView
{
	MA_DECLARE_CLONE;

public:
	TPageSetupDialog();

	virtual ~TPageSetupDialog();

	virtual void DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event);		// override 

	virtual void DoPostCreate(TDocument* itsDocument); // override 

public:
	VHSelect	fOrientation;					// Vertical or Horizontal printing orientation 
	TCheckBox*	fTallAdjusted;					// Convenient reference to a subview 
	TRadioIcon*	fVerticalOrientation;			// Convenient reference to a subview 
	TRadioIcon*	fHorizontalOrientation;			// Convenient reference to a subview 
};


//----------------------------------------------------------------------------------------
// class TArrowsControl
//----------------------------------------------------------------------------------------

class TArrowsControl : public TPicture
{
	MA_DECLARE_CLONE;

public:
	TArrowsControl();

	virtual ~TArrowsControl();

	virtual void TrackMouse(TrackPhase aTrackPhase,
							CViewPoint&			/*anchorPoint*/ ,
							CViewPoint&			/*previousPoint*/ ,
							CViewPoint& nextPoint,
							bool				/*mouseDidMove*/); // override 

public:
	long fLastChange;							// tick count of last call to DoChoice 
};


//----------------------------------------------------------------------------------------
// class TSwallowBehavior
//----------------------------------------------------------------------------------------

class TSwallowBehavior : public TBehavior
{
	MA_DECLARE_CLONE;

public:
	inline TSwallowBehavior() { }

	virtual ~TSwallowBehavior();

	virtual bool DoToolboxEvent(TToolboxEvent* event);
};


//----------------------------------------------------------------------------------------
// class TChangeBehavior
//----------------------------------------------------------------------------------------

class TChangeBehavior : public TBehavior
{
	MA_DECLARE_CLONE;

public:
	inline TChangeBehavior() { }

	virtual ~TChangeBehavior();

	virtual void DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event);		// override 
};


//----------------------------------------------------------------------------------------
// class TUpdateBehavior
//----------------------------------------------------------------------------------------

class TUpdateBehavior : public TBehavior, public MDependable_AC
{
	MA_DECLARE_CLONE;

public:
	inline TUpdateBehavior(IDType itsIdentifier = kNoIdentifier) : TBehavior(itsIdentifier) { }

	virtual ~TUpdateBehavior();

	virtual void DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject,
						  void* changeData,
						  CDependencySpace_AC* dependencySpace);/* override */
};


//----------------------------------------------------------------------------------------
// class TTemperatureCluster
//----------------------------------------------------------------------------------------

class TTemperatureCluster : public TCluster
{
	MA_DECLARE_CLONE;

public:
	inline TTemperatureCluster() { }

	virtual ~TTemperatureCluster();

	virtual void DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event);		// override 

	virtual long GetValue() const;

	virtual void SetValue(long newValue,
						  bool redraw);
};


//----------------------------------------------------------------------------------------
// class TTemperatureConversionCluster
//----------------------------------------------------------------------------------------

class TTemperatureConversionCluster : public TCluster
{
	MA_DECLARE_CLONE;

public:
	inline TTemperatureConversionCluster() { }

	virtual ~TTemperatureConversionCluster();

	virtual void DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event);		// override 
};


//----------------------------------------------------------------------------------------
// class THomeBrewSlider
//----------------------------------------------------------------------------------------

class THomeBrewSlider : public TPicture
{
	MA_DECLARE_CLONE;

public:
	THomeBrewSlider();

	virtual ~THomeBrewSlider();

	virtual void DoPostCreate(TDocument* itsDocument);// override 

	virtual bool ContainsMouse(const CViewPoint& theMouse);// override 

	virtual void Draw(const CViewRect& area);		// override 

	virtual void DrawKnob();

	CRect_AC GetKnobRect();

	virtual void TrackMouse(TrackPhase aTrackPhase,
							CViewPoint& anchorPoint,
							CViewPoint& previousPoint,
							CViewPoint& nextPoint,
							bool				/*mouseDidMove*/);// override 

public:
	short		fValue;
	PicHandle	fKnobPicture;
	CRect_AC	fKnobRect;
	short		fMinTop;
	short		fMaxTop;
};


//----------------------------------------------------------------------------------------
// class TSumStaticText
//----------------------------------------------------------------------------------------

class TSumStaticText : public TStaticText
{
	MA_DECLARE_CLONE;

public:
	inline TSumStaticText() { }

	virtual ~TSumStaticText();

	virtual void Draw(const CViewRect& area);		// override 
};


//----------------------------------------------------------------------------------------
// class TCalcDialog
//----------------------------------------------------------------------------------------

class TCalcDialog : public TPicture
{
	MA_DECLARE_CLONE;

public:
	inline TCalcDialog() { }

	virtual ~TCalcDialog();

	virtual void DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event);		// override 

	virtual void DoKeyEvent(TToolboxEvent* event);// override 

	virtual void ClearSum();

	virtual double FetchValue();

	virtual void SetValue();

	virtual void TotalSum(CalcOperator newOperator);

	virtual void NewDigit(char theNumber);

protected:
	virtual void Truncate(CStr255_AC& theStr);

public:
	double			fSum;
	double			fArgument;
	CalcOperator	fOperator;
	bool			fDecimalPoint;
	bool			fRestart;
};


//----------------------------------------------------------------------------------------
// class TAdornmentDemo
//----------------------------------------------------------------------------------------

class TAdornmentDemo : public TView
{
	MA_DECLARE_CLONE;

public:
	TAdornmentDemo();

	virtual ~TAdornmentDemo();

	virtual void DoPostCreate(TDocument* itsDocument);// override 

	virtual void DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event);		// override 

public:
	TView* fTestView;
};


//----------------------------------------------------------------------------------------
// class TEmbossedFrame
//----------------------------------------------------------------------------------------

class TEmbossedFrame : public TAdorner
{
	MA_DECLARE_CLONE;

public:
	inline TEmbossedFrame() { }

	virtual ~TEmbossedFrame();

	virtual void Draw(TView* itsView,
					  const CViewRect&				/*area*/ );// override 
};


//----------------------------------------------------------------------------------------
// class TGrayFill
//----------------------------------------------------------------------------------------

class TGrayFill : public TAdorner
{
	MA_DECLARE_CLONE;

public:
	inline TGrayFill() { }

	virtual ~TGrayFill();

	virtual void Draw(TView* itsView,
					  const CViewRect&				/*area*/ );// override 
};


//----------------------------------------------------------------------------------------
// class TEtchedFrame
//----------------------------------------------------------------------------------------

class TEtchedFrame : public TAdorner
{
	MA_DECLARE_CLONE;

public:
	TEtchedFrame();

	virtual ~TEtchedFrame();

	virtual void Draw(TView* itsView,
					  const CViewRect&				/*area*/ );// override 

public:
	CPoint_AC fInset;
};


//----------------------------------------------------------------------------------------
// class TSlowScroller
//----------------------------------------------------------------------------------------

class TSlowScroller : public TScroller
{
	MA_DECLARE_CLONE;

public:
	inline TSlowScroller() { }

	virtual ~TSlowScroller();

	virtual ViewCoordinate ScrollStep(VHSelect vhs,
								   short partCode);// override
};

//----------------------------------------------------------------------------------------
// class TRoundView
//----------------------------------------------------------------------------------------

class TRoundView : public TControl
{
	MA_DECLARE_CLONE;

public:
	TRoundView();

	virtual ~TRoundView();

	virtual void Draw(const CViewRect& area);
	// Image it

	virtual void GetExtentRegion(RgnHandle itsExtentRgn);// override


	virtual void DoSetCursor(const CViewPoint& globalMouse,
							 RgnHandle cursorRegion);
	virtual void DoUnsetCursor();
};


#endif // __UDEMODIALOGS__
