// UStructureInspectors.h
//  Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __USTRUCTUREINSPECTORS__
#define __USTRUCTUREINSPECTORS__


#ifndef __UEVENTHANDLER__
#include "UEventHandler.h"
#endif

#ifndef __UGRIDVIEW__
#include "UGridView.h"
#endif

// for IdlePhase
#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class CList_AC;
class TDocument;
class TIdler;

//----------------------------------------------------------------------------------------
// class TIdler
//----------------------------------------------------------------------------------------

class TIdler : public TEventHandler
{
public:
	TIdler(TEventHandler* itsHandler = NULL);

	virtual ~TIdler();

	virtual bool DoIdle(IdlePhase phase);	// Override 

public:
	TEventHandler* fHandler;
};


//----------------------------------------------------------------------------------------
// class TObjectListView
//----------------------------------------------------------------------------------------

class TObjectListView : public TTextListView
{
public:
	TObjectListView();

	virtual ~TObjectListView();

	virtual void DoPostCreate(TDocument* itsDocument);// override 
	virtual void BuildList(CList_AC* newList);
	virtual bool IsSynchronized();
	virtual void Synchronize(bool redraw);
	virtual bool DoIdle(IdlePhase phase);	// override 
	virtual void Draw(const CViewRect& area);
	virtual void Open();						// override 
	virtual void Close();						// override 
	virtual void* GetSelectedObject();
	virtual void* GetNthObject(GridCoordinate n) const;
	virtual void SelectObject(void* anObject);

public:
	CAutoPtr_AC<CList_AC> fAList;
	CAutoPtr_AC<CList_AC> fBList;
	CAutoPtr_AC<TIdler> fIdler;

	CList_AC* fDisplayedList;
};


//----------------------------------------------------------------------------------------
// class TViewHierarchyView
//----------------------------------------------------------------------------------------

class TViewHierarchyView : public TObjectListView
{
public:
	inline TViewHierarchyView() { }

	virtual ~TViewHierarchyView();

	virtual void FlattenHierarchy(TView* root,
								  CList_AC* theList);
	virtual void BuildList(CList_AC* newList);		// Override
	virtual CStr255_AC GetItemText(GridCoordinate anItem) const;	// override 
	virtual TView* GetSelectedView();
};


//----------------------------------------------------------------------------------------
// class TTargetChainView
//----------------------------------------------------------------------------------------

class TTargetChainView : public TObjectListView
{
public:
	inline TTargetChainView() { }

	virtual ~TTargetChainView();

	virtual void BuildList(CList_AC* newList);		// Override
	virtual CStr255_AC GetItemText(GridCoordinate anItem) const;	// override 
	virtual TEventHandler* GetSelectedHandler();
};

#endif
