//----------------------------------------------------------------------------------------
// UScriptMenu.cp 
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USCRIPTMENU__
#include "UScriptMenu.h"
#endif

// MacApp

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UOSASCRIPT__
#include "UOSAScript.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

//----------------------------------------------------------------------------------------
// Global Functions: 
//----------------------------------------------------------------------------------------
OSErr GetAppLocation(short refNum,
					 short& vRefNum,
					 long& dirID,
					 CStr255_AC& fileName);
void FindFiles(OSType theType,
			   OSType theCreator,
			   CFSSpec_AC& inFSSpec,
			   long& numToFind,
			   CFSSpec_AC* theFSSpecs);

//----------------------------------------------------------------------------------------
// GetAppLocation: 
//----------------------------------------------------------------------------------------


OSErr GetAppLocation(short refNum,
					 short& vRefNum,
					 long& dirID,
					 CStr255_AC& fileName)
{
	FCBPBRec pb;
	OSErr error;

	pb.ioNamePtr = (StringPtr) & fileName;
	pb.ioVRefNum = 0;
	pb.ioRefNum = refNum;
	pb.ioFCBIndx = 0;
	error = PBGetFCBInfoSync(&pb);
	vRefNum = pb.ioFCBVRefNum;
	dirID = pb.ioFCBParID;
	return (error);
}

//----------------------------------------------------------------------------------------
// FindFiles: 
//----------------------------------------------------------------------------------------

void FindFiles(OSType theType,
			   OSType theCreator,
			   CFSSpec_AC& inFSSpec,
			   long& numToFind,
			   CFSSpec_AC* theFSSpecs)
{
	const short kBufferSize = 16384;
	CInfoPBRec spec1, spec2;
	CSParam pb;
	short err = 0;
	short bufferSize = kBufferSize;

	CTempHandle_AC buffer(TempNewHandle(kBufferSize, &err));
	::ThrowIfNULL_AC(buffer);
	HLock(buffer);
	Ptr bufferPtr = *(Handle)buffer;

	spec1.hFileInfo.ioFlFndrInfo.fdType = theType;
	spec1.hFileInfo.ioFlFndrInfo.fdCreator = theCreator;
	spec1.hFileInfo.ioNamePtr = NULL;
	spec1.hFileInfo.ioFlParID = inFSSpec.parID;

	spec2.hFileInfo.ioFlFndrInfo.fdType = (OSType)0xffffffff;
	spec2.hFileInfo.ioFlFndrInfo.fdCreator = (OSType)0xffffffff;
	spec2.hFileInfo.ioFlParID = inFSSpec.parID;
	spec2.hFileInfo.ioNamePtr = NULL;
	spec2.hFileInfo.ioFlFndrInfo.fdFlags = 0x0000;
	spec2.hFileInfo.ioFlFndrInfo.fdLocation.h = 0x0000;
	spec2.hFileInfo.ioFlFndrInfo.fdLocation.v = 0x0000;
	spec2.hFileInfo.ioFlFndrInfo.fdFldr = 0x0000;

	pb.ioCompletion = NULL;
	pb.ioNamePtr = NULL;
	pb.ioMatchPtr = (FSSpecPtr)theFSSpecs;
	pb.ioReqMatchCount = numToFind;
	pb.ioSearchBits = fsSBFlFndrInfo + fsSBFlParID;
	pb.ioSearchInfo1 = (CInfoPBPtr) & spec1;
	pb.ioSearchInfo2 = (CInfoPBPtr) & spec2;
	pb.ioSearchTime = 0;
	pb.ioOptBuffer = bufferPtr;
	pb.ioOptBufSize = bufferSize;

	pb.ioCatPosition.initialize = 0;
	pb.ioVRefNum = inFSSpec.vRefNum;
	err = PBCatSearchSync(&pb);
	if ((err != noErr) && (err != -39))
		::ThrowIfOSErr_AC(err);
	numToFind = pb.ioActMatchCount;
}

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior
//----------------------------------------------------------------------------------------

#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TScriptMenuBehavior);

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior::TScriptMenuBehavior: 
//----------------------------------------------------------------------------------------

TScriptMenuBehavior::TScriptMenuBehavior() :
	TBehavior('osas')
{	
	if (HasAppleScript_AC())
	{
		CStr63_AC scriptFolderName(kScriptMenuStrings, kFolderName);
		CStr63_AC globalsScriptName(kScriptMenuStrings, kGlobalsScript);
		CStr63_AC sharedScriptName(kScriptMenuStrings, kSharedScript);
		CStr63_AC startupScriptName(kScriptMenuStrings, kStartupScript);
		
		OSErr err = noErr;
		long stdScriptsFound = 0;
		ComponentInstance theCmp = COSAScript::GetDefaultComponent();
		
		InstallGlobalsScript(theCmp, scriptFolderName, globalsScriptName, stdScriptsFound);
		InstallSharedScript(theCmp, sharedScriptName, stdScriptsFound);
		ExecuteStartupScript(theCmp, startupScriptName, stdScriptsFound);
		AddScriptMenu(globalsScriptName, sharedScriptName, startupScriptName, stdScriptsFound);
	}
}

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior destructor
//----------------------------------------------------------------------------------------


TScriptMenuBehavior::~TScriptMenuBehavior()
{
}

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior::InstallGlobalsScript: 
//----------------------------------------------------------------------------------------


void TScriptMenuBehavior::InstallGlobalsScript(ComponentInstance theCmp,
											   const CStr63_AC& scriptFolderName,
											   const CStr63_AC& globalsScriptName,
											   long& stdScriptsFound)
{
	COSAScript globalsScript;
	OSErr err = noErr;
	
	CStr32_AC delimeter = HasAUX_AC() ? "/" : ":";

	// Look for a globals script
	CStr255_AC fileName;
	short appRef = LMGetCurApRefNum();
	CFSSpec_AC theSpec;
	::ThrowIfOSErr_AC(GetAppLocation(appRef, theSpec.vRefNum, theSpec.parID, fileName));
	fileName = delimeter + scriptFolderName + delimeter + globalsScriptName;
	err = FSMakeFSSpec(theSpec.vRefNum, theSpec.parID, fileName, &theSpec);
	fScriptFolder = theSpec;

	(CStr63_AC &)theSpec.name = globalsScriptName;
	AliasHandle theAliasHdl = NULL;
	err = NewAliasMinimal(&theSpec, &theAliasHdl);
	if (err == noErr)
	{
		CAEDesc globalsAliasDesc(typeAlias, (Handle)theAliasHdl);
		::ThrowIfOSErr_AC(globalsScript.SetScript(theCmp, globalsAliasDesc, 
										  kOSAModeCompileIntoContext, kOSAModeNull));
		COSAScript::SetGlobalScript(globalsScript);
		++stdScriptsFound;
	}
	theAliasHdl = (AliasHandle)DisposeIfHandle_AC((Handle)theAliasHdl);
}

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior::InstallSharedScript: 
//----------------------------------------------------------------------------------------


void TScriptMenuBehavior::InstallSharedScript(ComponentInstance theCmp,
											  const CStr63_AC& sharedScriptName,
											  long& stdScriptsFound)
{
	OSErr err = noErr;
	CFSSpec_AC theSpec = fScriptFolder;
	(CStr63_AC &)theSpec.name = sharedScriptName;
	AliasHandle theAliasHdl = NULL;
	err = NewAliasMinimal(&theSpec, &theAliasHdl);
	if (err == noErr)
	{
		CAEDesc sharedAliasDesc(typeAlias, (Handle)theAliasHdl);
		COSAScript sharedScript;
		::ThrowIfOSErr_AC(sharedScript.SetScript(theCmp, sharedAliasDesc, kOSAModeCompileIntoContext, kOSAModeNull));
		COSAScript::SetSharedScript(sharedScript);
		++stdScriptsFound;
	}
	theAliasHdl = (AliasHandle)DisposeIfHandle_AC((Handle)theAliasHdl);
}

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior::ExecuteStartupScript: 
//----------------------------------------------------------------------------------------


void TScriptMenuBehavior::ExecuteStartupScript(ComponentInstance theCmp,
											   const CStr63_AC& startupScriptName,
											   long& stdScriptsFound)
{
	OSErr err = noErr;
	{
		CFSSpec_AC theSpec = fScriptFolder;
		(CStr63_AC &)theSpec.name = startupScriptName;
		AliasHandle theAliasHdl = NULL;
		err = NewAliasMinimal(&theSpec, &theAliasHdl);
		if (err == noErr)
		{
			CAEDesc startupAliasDesc(typeAlias, (Handle)theAliasHdl);
			COSAScript startupScript;
			::ThrowIfOSErr_AC(startupScript.SetScript(theCmp, startupAliasDesc, kOSAModeCompileIntoContext, kOSAModeNull));
			COSAScript aResult;
			COSAScript globalsScript = COSAScript::GetGlobalScript();
			bool hasGlobals = (globalsScript != kOSANullScript);
			bool sharedScript = (globalsScript != kOSANullScript);
			startupScript.Execute(theCmp, globalsScript, kOSAModeNull, aResult.fScriptID);
			aResult.DisposeScript(theCmp);
			++stdScriptsFound;
		}
		theAliasHdl = (AliasHandle)DisposeIfHandle_AC((Handle)theAliasHdl);
	}
}

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior::AddScriptMenu: 
//----------------------------------------------------------------------------------------


void TScriptMenuBehavior::AddScriptMenu(const CStr63_AC& globalsScriptName,
										const CStr63_AC& sharedScriptName,
										const CStr63_AC& startupScriptName,
										long& stdScriptsFound)
{
	OSErr err = noErr;
	
	gApplication->SetupTheMenus();
	long numScripts = 20;
	CFSSpec_AC* theSpecs = (CFSSpec_AC *)NewPtr(sizeof(CFSSpec_AC) * numScripts);
	CFSSpec_AC theSpec = fScriptFolder;
	FindFiles('osas', 'ToyS', theSpec, numScripts, theSpecs);
	if ((numScripts - stdScriptsFound) > 0)
	{
		CStr255_AC fileName;
		MenuRef scriptMenu = MAGetMenu(mScripts);
		MAInsertMenu(scriptMenu, mScripts, 0);
		CFSSpec_AC * aScriptSpec = theSpecs;
		short theMItem = 1;
		while (numScripts--)
		{
			fileName = (CStr63_AC &)aScriptSpec->name;
			if ((fileName != globalsScriptName) &&
				(fileName != sharedScriptName) &&
				(fileName != startupScriptName))
			{
				AppendMenu(scriptMenu, (ConstStr255Param)"\ptemp");
				fileName = (CStr63_AC &)aScriptSpec->name;
				SetMenuItemText(scriptMenu, theMItem++, fileName);
			}
			++aScriptSpec;
		}
	}
	InvalidateMenuBar();
	theSpecs = (CFSSpec_AC *)DisposeIfPtr_AC((Ptr)theSpecs);
}

//----------------------------------------------------------------------------------------
// TScriptMenuBehavior::DoMenuCommand:
//----------------------------------------------------------------------------------------


void TScriptMenuBehavior::DoMenuCommand(CommandNumber aCommandNumber)
{
	CStr255_AC aName;
	short menu;
	short item;
	CommandToMenuItem(aCommandNumber, menu, item);
	if (menu == mScripts)
	{
		GetMenuItemText(MAGetMenu(menu), item, aName);
		CFSSpec_AC theSpec = fScriptFolder;
		(CStr63_AC &)theSpec.name = aName;
		AliasHandle theAliasHdl = NULL;

		OSErr err = NewAliasMinimal(&theSpec, &theAliasHdl);
		if (err == noErr)
		{
			ComponentInstance theCmp = COSAScript::GetDefaultComponent();
			CAEDesc scriptAliasDesc(typeAlias, (Handle)theAliasHdl);
			COSAScript theScript;
			::ThrowIfOSErr_AC(theScript.SetScript(theCmp, scriptAliasDesc, kOSAModeCompileIntoContext, kOSAModeNull));

			fOwner->PostCommand(TH_new TExecuteScriptCommand(cNoCommand, dynamic_cast_AC(TCommandHandler*,fOwner), false, false, fOwner, theScript, kOSANullScript, kOSAModeNull, false, theCmp));
			MAHiliteMenu(0);						// Go ahead and unhilite the menu item since this script might
			// cause a context switch and MacApp won't unhilite it until later.
		}
	}
	else
		Inherited::DoMenuCommand(aCommandNumber);
}

//----------------------------------------------------------------------------------------
// End of UScriptMenu.cp


