//----------------------------------------------------------------------------------------
// UTEDocument.cp 
// Copyright  1990-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UTEDOCUMENT__
#include "UTEDocument.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#if qPowerTalk
	#ifndef __UMAILER__
	#include "UMailer.h"
	#endif
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#if UsingDemoTextPrinting
#	ifndef __UPRINTING__
#	include "UPrinting.h"
#	endif
#endif

#ifndef __UTECOMMANDS__
#include "UTECommands.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __ALIASES__
#include <Aliases.h>
#endif

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#if qPowerTalk
	#ifndef __OCESTANDARDMAIL__
	#include <OCEStandardMail.h>
	#endif
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __COLORPICKER__
#include <ColorPicker.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
TextSpecs TTEDocument::gDefaultSpecs;


//========================================================================================
// GLOBAL Procedures
//========================================================================================


//----------------------------------------------------------------------------------------
// InitUTEDocument: 
//----------------------------------------------------------------------------------------


void InitUTEDocument()
{
	TTEDocument::gDefaultSpecs.theTextFont = "Geneva";
	TTEDocument::gDefaultSpecs.theTextFace = 0;
	TTEDocument::gDefaultSpecs.theTextSize = 12;
	TTEDocument::gDefaultSpecs.theTextColor = gRGBBlack;
	TTEDocument::gDefaultSpecs.theJustification = teFlushDefault;
	TTEDocument::gDefaultSpecs.theBackColor = gRGBWhite;
}


//========================================================================================
// CLASS TTEDocument
//========================================================================================
#undef Inherited

#if qPowerTalk
#define Inherited TMailableDocument
#else
#define Inherited TFileBasedDocument
#endif


MA_DEFINE_CLASS(TTEDocument);

//----------------------------------------------------------------------------------------
// TTEDocument::TTEDocument: 
//----------------------------------------------------------------------------------------


TTEDocument::TTEDocument(TFile* itsFile,
									 const OSType itsScrapType,
									 ResNumber itsViewResourceID) :
	TFileBasedDocument(itsFile, itsScrapType),

	fDocText(NewPermHandle(0)),
	fElements(NULL),
	fStyles(NULL),
	fTEView(NULL),
	fTextSpecs(gDefaultSpecs),
	fViewResourceID(itsViewResourceID)
{
}

//----------------------------------------------------------------------------------------
// TTEDocument destructor
//----------------------------------------------------------------------------------------


TTEDocument::~TTEDocument()
{
	fDocText = DisposeIfHandle_AC(fDocText);
}

//----------------------------------------------------------------------------------------
// TTEDocument::ChangeBackColor: 
//----------------------------------------------------------------------------------------


void TTEDocument::ChangeBackColor(const CRGBColor_AC& newColor)
{
	GrafPtr oldPort;
	TView * itsRootView;

	//??? Is this right?  Should we set the background color even if
	// the view isn't in a window yet?
	GetPort(&oldPort);
	SetPort((*fTEView->fHTE)->inPort);
	RGBBackColor(newColor);
	itsRootView = fTEView->GetRootView();
	if (itsRootView)
		itsRootView->ForceRedraw();
	SetPort(oldPort);
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoInitialState: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoInitialState()	// Override 
{
	fTextSpecs = gDefaultSpecs;
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoMakeViews(bool /*forPrinting*/ )	// Override 

{
	TWindow* aWindow = gViewServer->NewTemplateWindow(fViewResourceID, this);
	::ThrowIfNULL_AC(aWindow);

	fTEView = dynamic_cast_AC(TTEView*,aWindow->FindSubView('TEVW'));	// Must cast because FindSubView returns TView

#if UsingDemoTextPrinting
	TStdPrintHandler* aHandler = TH_new TStdPrintHandler(this,			// its document 
							   fTEView,			// its view 
							   kFixedSize,		// horzontal page size is fixed 
							   !kFixedSize);	// vertical page size is variable (could be
	// set to true on non-style TE systems)
#endif

	ShowReverted();
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoJustChange: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoJustChange(CommandNumber aCommandNumber)
{
	short newJust;

	switch (aCommandNumber)
	{
		case cJustSystem:
			newJust = teFlushDefault;
			break;
			
		case cJustForceLeft:
			newJust = teFlushLeft;
			break;
			
		case cJustCenter:
			newJust = teCenter;
			break;
			
		case cJustRight:
			newJust = teFlushRight;
			break;
	};
	TJustCommand * aJustChange = TH_new TJustCommand(fTEView, newJust);
	fTextSpecs.theJustification = newJust;
	PostCommand(aJustChange);
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoStyleChange: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoStyleChange(CommandNumber aCommandNumber)
{
	CTextStyle_AC newStyle;

	switch (aCommandNumber)
	{
		case cBold:
			newStyle.tsFace = bold;
			break;
			
		case cItalic:
			newStyle.tsFace = italic;
			break;
			
		case cUnderline:
			newStyle.tsFace = underline;
			break;
			
		case cOutline:
			newStyle.tsFace = outline;
			break;
			
		case cShadow:
			newStyle.tsFace = shadow;
			break;
			
		case cCondense:
			newStyle.tsFace = condense;
			break;
			
		case cExtend:
			newStyle.tsFace = extend;
			break;
	}

	fTEView->DoneTyping();
	
	if ((newStyle.tsFace * fTextSpecs.theTextFace) == newStyle.tsFace)
		fTextSpecs.theTextFace -= newStyle.tsFace;
	else
		fTextSpecs.theTextFace += newStyle.tsFace;
	PostCommand(fTEView->DoMakeStyleCommand(newStyle, cStyleChange, doFace + doToggle));
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoColTextChange: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoColTextChange()
{
	CTextStyle_AC newStyle;
	if (GetColor(kBestSystemLocation, CStr255_AC(kPromptsRsrcID, kColTextPrompt), fTextSpecs.theTextColor, ((CRGBColor_AC &) newStyle.tsColor)))
	{
		fTEView->DoneTyping();
	
		fTextSpecs.theTextColor = newStyle.tsColor;
		PostCommand(fTEView->DoMakeStyleCommand(newStyle, cColorChange, doColor + doToggle));
	}
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoColBackChange: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoColBackChange()
{
	CRGBColor_AC newColor;
	if (GetColor(kBestSystemLocation, CStr255_AC(kPromptsRsrcID, kColBackPrompt), fTextSpecs.theBackColor, newColor))
	{
		fTextSpecs.theBackColor = newColor;
		ChangeBackColor(newColor);
	}
}

//----------------------------------------------------------------------------------------
// TTEDocument::InstallChangedDeterminer: 
//----------------------------------------------------------------------------------------


void TTEDocument::InstallChangedDeterminer(VHSelect vhs,
										   SizeDeterminer sd)
{
	if (sd != fTEView->GetSizeDeterminer(vhs))
	{
		fTEView->SetSizeDeterminer(vhs, sd);

		// If we changed the horizontal size determiner,
		//  we must ask the TTEView to recompute the TE rectangles.
		if (vhs == hSel)
			switch (sd)
			{
				case sizeSuperView:
					{
						CViewRect itsFrame(fTEView->GetFrame());
						fTEView->SuperViewChangedFrame(itsFrame, itsFrame, true);// Don't care about deltas
						break;
					}

				case sizePage:
					fTEView->DoPagination();
					break;
			}

		fTEView->AdjustFrame();
		fTEView->ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoWidthChange: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoWidthChange(CommandNumber aCommandNumber)
{
	SizeDeterminer sd;

	if (aCommandNumber == cWidthSuperView)
		sd = sizeSuperView;
	else if (aCommandNumber == cWidthOnePage)
		sd = sizePage;
	else
		sd = sizeFixed;

	// NB: The following is not undoable in the
	// current version
	InstallChangedDeterminer(hSel, sd);
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoHeightChange: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoHeightChange(CommandNumber aCommandNumber)
{
	SizeDeterminer sd;

	switch (aCommandNumber)
	{
		case cHeightSuperView:
			sd = sizeSuperView;
			break;
			
		case cHeightPages:
			sd = sizeFillPages;
			break;
			
		case cHeightText:
			sd = sizeVariable;
			break;
			
		case cHeightConst:
			sd = sizeFixed;
			break;
	}

	// NB: The following is not undoable in the
	// current version
	InstallChangedDeterminer(vSel, sd);
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoMenuCommand: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoMenuCommand(CommandNumber aCommandNumber)// override 
{
	CStr255_AC aName;
	short menu;
	short item;
	CTextStyle_AC newStyle;

	CommandToMenuItem(aCommandNumber, menu, item);

	if (menu == mFont)
	{
		fTEView->DoneTyping();
	
		GetMenuItemText(MAGetMenu(menu), item, aName);
		GetFNum(aName, &(newStyle.tsFont));
		fTextSpecs.theTextFont = aName;
		PostCommand(fTEView->DoMakeStyleCommand(newStyle, cFontChange, doFont + doToggle));
	}
	else if ((aCommandNumber > cSizeBase) && (aCommandNumber <= cSizeMax))
	{
		fTEView->DoneTyping();
	
		newStyle.tsSize = static_cast<short>(aCommandNumber - cSizeBase);
		fTextSpecs.theTextSize = newStyle.tsSize;
		PostCommand(fTEView->DoMakeStyleCommand(newStyle, cSizeChange, doSize + doToggle));
	}
	else
		switch (aCommandNumber)
		{
			case cSizeOther:
				// as yet unsupported
				break;

			case cSizeGrow:
			case cSizeShrink:
				fTEView->DoneTyping();
	
				if (aCommandNumber == cSizeGrow)
				{
					newStyle.tsSize = kRelSizeAmount;
					fTextSpecs.theTextSize += kRelSizeAmount;
				}
				else
				{
					newStyle.tsSize = -kRelSizeAmount;
					fTextSpecs.theTextSize -= kRelSizeAmount;
				}
				PostCommand(fTEView->DoMakeStyleCommand(newStyle, cSizeChange, doSize + addSize + doToggle));
				break;
	
			case cJustForceLeft:
			case cJustCenter:
			case cJustRight:
			case cJustSystem:
				DoJustChange(aCommandNumber);
				break;
	
			case cPlainText:
				fTEView->DoneTyping();
	
				newStyle.tsFace = 0;
				fTextSpecs.theTextFace = 0;
				PostCommand(fTEView->DoMakeStyleCommand(newStyle, cStyleChange, doFace));
				break;
	
			case cBold:
			case cItalic:
			case cUnderline:
			case cOutline:
			case cShadow:
			case cCondense:
			case cExtend:
				DoStyleChange(aCommandNumber);
				break;
	
			case cColorText:
				DoColTextChange();
				break;
	
			case cColorBackground:
				DoColBackChange();
				break;
	
			case cWidthSuperView:
			case cWidthView:
			case cWidthOnePage:
				DoWidthChange(aCommandNumber);
				break;
	
			case cHeightSuperView:
			case cHeightPages:
			case cHeightText:
			case cHeightConst:
				DoHeightChange(aCommandNumber);
				break;
	
			default:
				Inherited::DoMenuCommand(aCommandNumber);
				break;
		}
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoNeedDiskSpace(TFile* itsFile,
										 long& dataForkBytes,
										 long& rsrcForkBytes)// override 
{

	dataForkBytes += GetHandleSize(fDocText);

	if (fTEView->fHTE)
	{
		TEStyleHandle styles;
		STHandle elements;
		fTEView->ExtractStyles(styles, elements);
		rsrcForkBytes += GetHandleSize((Handle)styles) + kRsrcTypeOverhead + kRsrcOverhead + GetHandleSize((Handle)elements) + kRsrcTypeOverhead + kRsrcOverhead;
	}

	rsrcForkBytes += sizeof(TextSpecs) + kRsrcTypeOverhead + kRsrcOverhead;

	// Get resource file overhead 
	Inherited::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoRead: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoRead(TFile* aFile,
								bool forPrinting)		// override 
{
	long numChars;
	TextSpecsHdl hTextSpecs;
	bool oldPermAllocation;

	::ThrowIfOSErr_AC(aFile->GetDataLength(numChars));	// Read in the text 

	if (numChars > kUnlimited)					// The file may have been created by 
	{
		// someone else - limit it to 32K! 
		gApplication->ShowError(0, messageAlert + kFileTooBig);
		numChars = kUnlimited;
	}

	SetPermHandleSize(fDocText, numChars);
	SignedByte savedState = LockHandleHigh(fDocText);	// lock handle before deref

	try
	{
		::ThrowIfOSErr_AC(aFile->ReadData(*fDocText, numChars));
		
		HSetState(fDocText, savedState);		// restore state of handle
	}
	catch(CException_AC&)
	{
		HSetState(fDocText, savedState);		// restore state of handle
		throw;
	}
	
	// Read in the TEStyleRec 
	fStyles = (TEStyleHandle)Get1Resource(kTextStyleRsrcType, kStylesRsrcID);
	if (fStyles)
		DetachResource((Handle)fStyles);

	// Read in the STElement array from permanent allocation 
	oldPermAllocation = PermAllocation(true);
	fElements = (STHandle)(Get1Resource(kTextStyleRsrcType, kElementsRsrcID));
	PermAllocation(oldPermAllocation);
	
	if (fElements)
		DetachResource((Handle)fElements);

	// Read the text specs resource 
	hTextSpecs = (TextSpecsHdl)Get1Resource(kTextSpecsRsrcType, kTextSpecsRsrcID);
	if (hTextSpecs)
	{
		fTextSpecs = **hTextSpecs;
		ReleaseResource((Handle)hTextSpecs);
	}
	else
		DoInitialState();

	Inherited::DoRead(aFile, forPrinting);
}

#if qPowerTalk
//----------------------------------------------------------------------------------------
// TTEDocument::AddStandardMailContent: 
//----------------------------------------------------------------------------------------


void TTEDocument::AddStandardMailContent(bool& okToSend)
{
	WindowRef theWindow = fTEView->GetGrafPort();
	bool appendFlag = false;
	Handle theText = fTEView->ExtractText();
	SignedByte wasTextState = LockHandleHigh(theText);
	Ptr buffer = *theText;
	unsigned long bufferSize = GetHandleSize(theText);
	StScrpHandle theStyles = (StScrpHandle)NewHandle(10);
	fTEView->GivePasteData((Handle)theStyles, kStyleType_AC);
	SignedByte wasStyleState = LockHandleHigh((Handle)theStyles);
	bool startNewScript = true;
	short scriptID = smRoman;
	::ThrowIfOSErr_AC(SMPAddContent(theWindow, kMailStyledTextSegmentType, appendFlag,
							buffer, bufferSize, *theStyles, startNewScript, scriptID));
	HSetState(theText, wasTextState);
	HSetState((Handle)theStyles, wasStyleState);
	DisposeIfHandle_AC((Handle)theStyles);
	okToSend = true;
}

//----------------------------------------------------------------------------------------
// TTEDocument::ReadStandardMailContent: 
//----------------------------------------------------------------------------------------


bool TTEDocument::ReadStandardMailContent()
{
	bool readContent = false;
	const unsigned long kBufferSize = 10000;
	MailSegmentMask segmentTypeMask = kMailStyledTextSegmentMask;
	Handle buffer				 	= NewHandle(kBufferSize);
	::ThrowIfNULL_AC(buffer);
	
	const short kNumStyles 		 = 200;
	const long  kStyleBufferSize = (sizeof(ScrpSTElement) * kNumStyles) + 4;
	StScrpHandle theStyles 		 = (StScrpHandle)NewHandleClear(kStyleBufferSize);
	::ThrowIfNULL_AC(theStyles);
	
	Boolean endOfContent 		= false;
	OSErr err 					= noErr;
	bool allContentExtracted = true;
	
	while ((err == noErr) && !endOfContent)
	{
		// skip past empty text segments
		ScriptCode 		script 			= 0;
		unsigned long	dataSize		= 0;
		long 			segmentLength 	= 0;
		long 			segmentID 		= 0;
		Boolean 		endOfScript 	= false;
		Boolean 		endOfSegment 	= false;
		MailSegmentType segmentType;
		
		endOfContent = false;

	
		unsigned long bufferSize = kBufferSize;
		SetHandleSize(buffer, bufferSize);
		SetHandleSize((Handle)theStyles, kStyleBufferSize);
		(**theStyles).scrpNStyles = kNumStyles;
		SignedByte wasStylesState = LockHandle_AC((Handle)theStyles);
		SignedByte wasBufferState = LockHandle_AC(buffer);
		err = SMPReadContent(fLetter->GetMailerWindowRef(), segmentTypeMask,
							 *buffer, bufferSize, &dataSize, *theStyles, &script,
							 &segmentType, (unsigned char*)&endOfScript, &endOfSegment, &endOfContent,
							 &segmentLength, &segmentID);
		
		if(allContentExtracted && (dataSize > 0) && !endOfContent)
			allContentExtracted = false;
		
		if ((err == noErr) && (dataSize > 0))
		{
			HSetState(buffer, wasBufferState);
			HSetState((Handle)theStyles, wasStylesState);
			SetHandleSize(buffer, dataSize);
			unsigned long textSize = dataSize;
			dataSize = (sizeof(ScrpSTElement) * (**theStyles).scrpNStyles) + 4;
			SetHandleSize((Handle)theStyles, dataSize);
			fTEView->StyleInsert(*buffer, textSize, (StScrpHandle)theStyles);
			fTEView->SynchView(false);
			readContent = true;
		}
	}
	
	DisposeIfHandle_AC(buffer);
	DisposeIfHandle_AC((Handle)theStyles);
	
	if(!allContentExtracted)
	{
		MacAppAlert(kContentNotExtracted, NULL);
	}

	return readContent;
}

//----------------------------------------------------------------------------------------
// TTEDocument::ReadNativeMailContent: 
//----------------------------------------------------------------------------------------


bool TTEDocument::ReadNativeMailContent()
{	
	bool readContent = TMailableDocument::ReadNativeMailContent();
	
	// fStyles and fElements are native to DemoText (see TTEDocument::DoRead)
	// so they can be used as a valid check for native content.
	if (fDocText && fStyles && fElements)
	{
		ShowReverted();
		readContent = true;
	}
	return readContent;
}

//----------------------------------------------------------------------------------------
// TTEDocument::ReadNativeMailContent: 
//----------------------------------------------------------------------------------------


bool TTEDocument::ReadSnapshotMailContent()
{	
	bool readContent = false;
	
	OCECreatorType	 		theBlockType;
	theBlockType.msgCreator = 'apml';
	theBlockType.msgType 	= 'imag';
	unsigned long 			blockIndex = 1;
	const unsigned long 	theBufferSize = 10000;
	Handle					buffer = NewHandle(theBufferSize);
	::ThrowIfNULL_AC(buffer);
	
	unsigned long			dataOffset 	= 0;
	unsigned long			dataSize 	= 0;
	Boolean					endOfBlock 	= false;
	unsigned long			dataRemaining;
	OSErr 					err = noErr;
	
	unsigned long bufferSize = theBufferSize;
	SetHandleSize(buffer, bufferSize);

	
	err = SMPReadBlock(fLetter->GetMailerWindowRef(),&theBlockType,blockIndex,*buffer,bufferSize,
						dataOffset,&dataSize,&endOfBlock,&dataRemaining);			   
	if(err == noErr && dataSize > 0)
	{
		MacAppAlert(kSnapshotNotExtracted, NULL);
		readContent = true;
	}
	
	return readContent;
}

//----------------------------------------------------------------------------------------
// TTEDocument::SetReplyContents: 
//----------------------------------------------------------------------------------------


void TTEDocument::SetReplyContents(TDocument* replyToDoc)
{
	DisposeIfHandle_AC(fDocText);
	fDocText = dynamic_cast_AC(TTEDocument*,replyToDoc)->fDocText;
	::ThrowIfOSErr_AC(HandToHand(&fDocText));
	
	DisposeIfHandle_AC((Handle)fElements);
	fElements = dynamic_cast_AC(TTEDocument*,replyToDoc)->fElements;
	if (fElements)
		::ThrowIfOSErr_AC(HandToHand((Handle*)&fElements));
	
	DisposeIfHandle_AC((Handle)fStyles);
	fStyles = dynamic_cast_AC(TTEDocument*,replyToDoc)->fStyles;
	if (fStyles)
		::ThrowIfOSErr_AC(HandToHand((Handle*)&fStyles));
	
	ShowReverted();
	
	if (fTEView->Focus())
	{
		(*fTEView->fHTE)->selStart = 0;
		(*fTEView->fHTE)->selEnd = 0;
		
		CStr255_AC replySpacer(kPromptsRsrcID, kReplySpacer);
		CTextStyle_AC newStyle;
		newStyle.tsFace = 0;
		TESetStyle(doFace,&newStyle,false,fTEView->fHTE);
		TEInsert((Ptr)&replySpacer.fStr[1], replySpacer.Length(),fTEView->fHTE);
		
		(*fTEView->fHTE)->selStart = 0;
		(*fTEView->fHTE)->selEnd = 0;
		fTEView->SynchView(false);
	}
}

//----------------------------------------------------------------------------------------
// TTEDocument::ImageDocumentForLetter: 
//----------------------------------------------------------------------------------------


void TTEDocument::ImageDocumentForLetter(bool /* inColor */)
{
	
}

#endif // qPowerTalk

//----------------------------------------------------------------------------------------
// TTEDocument::DoSetupMenus: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoSetupMenus()		// override 
{
	bool checkPlain;
	bool checkFont;
	short just;
	short aMode;
	Style aFace;
	SizeDeterminer sd;
	MenuRef aMenuRef;
	CStr255_AC aName;
	CTextStyle_AC aStyle;
	short theFont;
	short startOfSelection;
	short endOfSelection;
	short nItems;
	
	Inherited::DoSetupMenus();

	startOfSelection = (*fTEView->fHTE)->selStart;
	endOfSelection = (*fTEView->fHTE)->selEnd;

	aMode = doFace;
	checkPlain = fTEView->ContinuousStyle(startOfSelection, endOfSelection, aMode, aStyle) && !aStyle.tsFace;

	aMode = doAll;
	aStyle.tsFace = bold | italic | underline | outline | shadow | extend | condense;

	fTEView->ContinuousStyle(startOfSelection, endOfSelection, aMode, aStyle);
	checkFont = (aMode & doFont) != 0;

	aMenuRef = MAGetMenu(mFont);
	if (aMenuRef)
	{
		GetFontName(aStyle.tsFont, aName);			// Get real font number in case tsFont is 
		GetFNum(aName, &theFont);					// the system or application font. 
		nItems = CountMItems(aMenuRef);
		for (short item = 1; item <= nItems; ++item)
		{
			// There can be more than 31 menu entries with scrolling menus, but trying to enable
			// an item with number > 31 is bad news.  If the menu itself is enabled (which it
			// will be in MacApp if any of the first 31 items is enabled), then the extras
			// will always be enabled.
			if (item <= 31)
				EnableItem(aMenuRef, item);
			if (checkFont)
			{
				short fnt;
	
				GetMenuItemText(aMenuRef, item, aName);
				GetFNum(aName, &fnt);
				CheckItem(aMenuRef, item, (fnt == theFont));
			}
		}
	}

	sd = fTEView->GetSizeDeterminer(hSel);		// Enable size determiner related menu items
	EnableCheck(cWidthSuperView, true, (sd == sizeSuperView));
	EnableCheck(cWidthOnePage, true, (sd == sizePage));
	EnableCheck(cWidthView, true, (sd == sizeFixed));

	sd = fTEView->GetSizeDeterminer(vSel);
	EnableCheck(cHeightSuperView, true, (sd == sizeSuperView));
	EnableCheck(cHeightPages, true, (sd == sizeFillPages));
	EnableCheck(cHeightText, true, (sd == sizeVariable));
	EnableCheck(cHeightConst, true, (sd == sizeFixed));

	just = fTEView->fJustification;				// Enable justification related menu items 
	EnableCheck(cJustSystem, true, (just == teFlushDefault));
	EnableCheck(cJustForceLeft, true, (just == teFlushLeft));
	EnableCheck(cJustCenter, true, (just == teCenter));
	EnableCheck(cJustRight, true, (just == teFlushRight));

	Enable(cStyle, true);						// Enable sub-menus 
	Enable(cSize, true);
	Enable(cFont, true);
	Enable(cColor, (qNeedsColorQD || HasColorQD_AC()));

	aFace = aStyle.tsFace;
	EnableCheck(cPlainText, true, checkPlain);	// Enable normal Style menu items 
	EnableCheck(cBold, true, bold & aFace);
	EnableCheck(cItalic, true, italic & aFace);
	EnableCheck(cUnderline, true, underline & aFace);
	EnableCheck(cOutline, true, outline & aFace);
	EnableCheck(cShadow, true, shadow & aFace);
	EnableCheck(cCondense, true, condense & aFace);
	EnableCheck(cExtend, true, extend & aFace);

	for (CommandNumber c = cSizeMin; c <= cSizeMax; ++c)
	{
		bool checkSize;

		if (!(aMode & doSize))
			checkSize = false;
		else
			checkSize = ((c - cSizeBase) == aStyle.tsSize);

		EnableCheck(c, true, checkSize);
		// if it _is_ styled and font is continuous 
		// and the size is a real one
		if ((aMode & doFont) && RealFont(aStyle.tsFont, static_cast<short>(c - cSizeBase)))
			aFace = outline;				// then we outline it 
		else
			aFace = 0;
		SetStyle(c, aFace);
	}

	Enable(cSizeGrow, true);
	Enable(cSizeShrink, true);

	Enable(cColorText, (qNeedsColorQD || HasColorQD_AC()));
	Enable(cColorBackground, (qNeedsColorQD || HasColorQD_AC()));
}

//----------------------------------------------------------------------------------------
// TTEDocument::DoWrite: 
//----------------------------------------------------------------------------------------


void TTEDocument::DoWrite(TFile* aFile,
								 bool makingCopy)		// override 
{
	// Write out the text 
	long numChars = GetHandleSize(fDocText);

	{	
		CTempHandleLock_AC tempHandleLock(fDocText, eMoveHi);
		::ThrowIfOSErr_AC(aFile->WriteData(*fDocText, numChars));
	}

	TEStyleHandle styles;
	STHandle elements;

	fTEView->ExtractStyles(styles, elements);
	AddResource((Handle)PermCopyHandle((Handle)styles), kTextStyleRsrcType, kStylesRsrcID, gEmptyString);
	::ThrowIfResError_AC();						// !!! need a failure handler
	::AddResource((Handle)PermCopyHandle((Handle)elements), kTextStyleRsrcType, kElementsRsrcID, gEmptyString);
	::ThrowIfResError_AC();						// !!! need a failure handler

	// Write the text specification resource, after converting it to a handle 
	TextSpecsHdl hTextSpecs = (TextSpecsHdl)NewPermHandle(sizeof(TextSpecs));
	**hTextSpecs = fTextSpecs;
	::AddResource((Handle)hTextSpecs, kTextSpecsRsrcType, kTextSpecsRsrcID, gEmptyString);
	::ThrowIfResError_AC();							// !!! need a failure handler
	
	Inherited::DoWrite(aFile, makingCopy);
}

//----------------------------------------------------------------------------------------
// TTEDocument::FreeData: 
//----------------------------------------------------------------------------------------


void TTEDocument::FreeData()			// override 
{
	SetPermHandleSize(fDocText, 0);
}

//----------------------------------------------------------------------------------------
// TTEDocument::SetSpecStyle: 
//----------------------------------------------------------------------------------------


void TTEDocument::SetSpecStyle()
{
	short theFont;
	GetFNum(fTextSpecs.theTextFont, &theFont);

	CTextStyle_AC theTS(theFont, fTextSpecs.theTextFace, fTextSpecs.theTextSize, fTextSpecs.theTextColor);

	fTEView->SetOneStyle(0, 0, doAll, theTS, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TTEDocument::ShowReverted: 
//----------------------------------------------------------------------------------------


void TTEDocument::ShowReverted()		// override 
{
	CRGBColor_AC aColor;

	fTEView->StuffText(fDocText);				// put in the text 
	fTEView->SetSelection(0, 0, !kRedraw);
	if (fStyles && fElements)					// If we're able, stuff style info 
		fTEView->StuffStyles(fStyles, fElements);
	else
		SetSpecStyle();

	if (qNeedsColorQD || HasColorQD_AC())
	{
		aColor = fTextSpecs.theBackColor;
		ChangeBackColor(aColor);
	}
	fTEView->SetJustification(fTextSpecs.theJustification, kDontRedraw);
	Inherited::ShowReverted();
}


//========================================================================================
// CLASS TJustCommand
//========================================================================================


#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TJustCommand::TJustCommand:
//----------------------------------------------------------------------------------------

TJustCommand::TJustCommand(TTEView* itsTEView,
									   short itsNewJust) :
	TCommand(cJustChange, itsTEView ? itsTEView->GetContext(cJustChange) : NULL, kCanUndo, kCausesChange, itsTEView ? itsTEView->GetContext(cJustChange) : NULL),

	fTEView(itsTEView),
	fOldJust(itsTEView ? itsTEView->fJustification : teFlushDefault),
	fNewJust(itsNewJust)
{
}

//----------------------------------------------------------------------------------------
// TJustCommand destructor
//----------------------------------------------------------------------------------------


TJustCommand::~TJustCommand()
{
}

//----------------------------------------------------------------------------------------
// TJustCommand::DoIt: 
//----------------------------------------------------------------------------------------


void TJustCommand::DoIt()			// override 
{
	fTEView->SetJustification(fNewJust, kRedraw);
}

//----------------------------------------------------------------------------------------
// TJustCommand::UndoIt: 
//----------------------------------------------------------------------------------------


void TJustCommand::UndoIt()			// override 
{
	fTEView->SetJustification(fOldJust, kRedraw);
}

//----------------------------------------------------------------------------------------
// End of UTEDocument.cp


