// IconEdit.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UICONEDIT__
#define __UICONEDIT__

// MacApp

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include "UFileBasedDocument.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif


// Constants
const OSType kSignature = 'ICED';				// Application signature
const OSType kFileType = 'ICON';				// File-type for document files created by this application

const short kDefaultMagnification = 7;			// Default icon magnification.

// definitions for the custom apple event suite
const OSType kAEIconEditClass = 'ICED';			// event class

const OSType kAEInvertID = 'INVT';				// event IDs
const OSType kAEZoomInID = 'ZmIn';
const OSType kAEZoomOutID = 'ZmOt';
const OSType kAEDrawPoints = 'DrPt';

const AEKeyword keyPointList = 'kPtL';
const AEKeyword keyErasePoints = 'kErs';

// Forward references
class TIconBitMap;
class TIconEditView;

// Classes
// ====================================================================================

class TIconEditApplication : public TApplication
{
	MA_DECLARE_CLASS;

public:
	TIconEditApplication();

	virtual ~TIconEditApplication();
	// Destructor

	virtual TDocument* DoMakeDocument(CommandNumber itsCommandNumber,
									  TFile* itsFile);// Override

	virtual TView* MakeViewForAlienClipboard();	// Override
	//  Makes a view of type TIconEditView for the Clipboard if the public desk
	//	scrap contains data of type 'ICON'.  Otherwise it calls INHERITED
	//	MakeViewForAlienClipboard. 
};

// ====================================================================================

class TIconBitMap
{
public:
	TIconBitMap();
	// Constructor

	virtual ~TIconBitMap();

	virtual void SetIconBitMap(Handle theBitMap);

	virtual void ReadFrom(CStream_AC* aStream);	// Override

	virtual void WriteTo(CStream_AC* aStream) const;		// Override

	virtual void LoadFromScrap();

	virtual void WriteToScrap();

	virtual TIconBitMap* Copy();

	virtual void CopyDataTo(TIconBitMap* anIcon);

	virtual void Clear();

	virtual void Invert();

	virtual void Draw(const CRect_AC& area,
					  const CRGBColor_AC& drawingColor);//not Override!

	virtual bool GetBit(CViewPoint iconBit);

	virtual void SetBit(CViewPoint iconBit,
						bool turnBitOn);

	virtual void IconBitToByteBit(CViewPoint iconBit,
								  short& byte,
								  short& bit);

protected:
	CTempHandle_AC fDataHandle;							// Handle to the icon's data.
};


// ====================================================================================
//old class TIconDocument : public TDocument

class TIconDocument : public TFileBasedDocument	//new
{
	MA_DECLARE_CLASS;

public:
	TIconDocument(TFile* itsDocument = NULL);
	// Constructor

	virtual ~TIconDocument();

	virtual void DoInitialState();				// Override

	virtual void DoMakeViews(bool forPrinting);// Override

	virtual void DoMenuCommand(CommandNumber aCommandNumber);

	virtual void DoScriptCommand(CommandNumber aCommand,
								 TAppleEvent* message,
								 TAppleEvent* reply);// Override

	virtual void DoSetupMenus();				// Override

	virtual void DoNeedDiskSpace(TFile* itsDocument,
								 long& dataForkBytes,
								 long& rsrcForkBytes);// Override

	virtual void DoRead(TFile* itsDocument,
						bool forPrinting);	// Override

	virtual void DoWrite(TFile* itsDocument,
						 bool makingCopy);	// Override

	virtual void InvertIcon();

	inline TIconEditView* GetIconView() const
	{ return fIconView; }


	virtual void ClearIcon();

	virtual void SetIcon(TIconBitMap* newIcon);

	inline TIconBitMap* ReturnBitMap() const
	{ return fIconBitMap; }

	virtual void RedrawViews();

	virtual void GetSetPropertyInfo(DescType whichProperty,
									CommandNumber& cmdNum,
									bool& canUndo,
									bool& causesChange,
									TCommandHandler*& theContext);// Override

	virtual void SetObjectProperty(const CAEDesc& thePropertyValue,
								   DescType whichProperty);// Override

	virtual bool GetObjectProperty(CAEDesc& thePropertyValue,
									  DescType whichProperty,
									  const CAEDesc& desiredType); // Override

	inline CRGBColor_AC GetIconColor() const
	{ return fColor; }

	virtual void SetIconColor(CRGBColor_AC& newColor);

protected:
	CRGBColor_AC fColor;							// color property of the document

	CAutoPtr_AC<TIconBitMap> fIconBitMap;		// The icon's bit map
	TIconEditView* fIconView;					// a reference to the single view in this window
};


// ====================================================================================

class TIconEditView : public TView
{
	MA_DECLARE_CLASS;

public:
	TIconEditView(const CViewPoint& itsLocation = gZeroVPt,
							   short itsMagnification = kDefaultMagnification);
	// Constructor

	virtual ~TIconEditView();
	// Destructor

	virtual void DoPostCreate(TDocument* itsDocument);// Override

	virtual CViewRect CalcMinFrame();	// Override

	virtual bool ContainsClipType(ResType aType);// Override
	//  Called when a view of this type is installed in the Clipboard.
	//	Returns true if aType is kIconClipType (i.e. 'ICON'). 

	virtual void DoKeyEvent(TToolboxEvent* info);// Override
	//  Treats the backspace the same as the Clear command. 

	virtual void DoMenuCommand(CommandNumber aCommandNumber);// Override

	virtual void DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* event,
								CPoint_AC hysteresis);// Override

	virtual void DoSetCursor(const CViewPoint& localPoint,
							 RgnHandle cursorRegion);// Override

	virtual void DoSetupMenus();				// Override

	virtual void Draw(const CViewRect& area);		// Override

	virtual void DrawBit(CViewPoint theBit,
						 bool turnBitOn,
						 CRGBColor_AC& drawingColor);

	bool PointToBit(const CViewPoint& thePoint,
							   CViewPoint& iconBit);

	short GetMagnification() const
	{ return fMagnification; }

	virtual void SetMagnification(short magnification);

	virtual void WriteToDeskScrap();			// Override

protected:
	TIconDocument* fIconDocument;				// Reference to the view's icon document.
	short fMagnification;						// No. of times to magnify the icon.
};

// ====================================================================================

class TDrawPointsAppleEvent : public TAppleEvent
{
public:
	TDrawPointsAppleEvent(MScriptableObject* itsDirectObject = NULL,
							  TIconBitMap* iconBitMapModel = NULL,
							  bool turnBitsOn = false);

	virtual ~TDrawPointsAppleEvent();
	// Destructor
};

// ====================================================================================

class TDrawPointsCommand : public TCommand
{
public:
	TDrawPointsCommand(TIconDocument* itsIconDocument = NULL,
									TAppleEvent* theAppleEvent = NULL);

	// Constructor

	virtual ~TDrawPointsCommand();

	virtual void DoIt();						// Override

	virtual void RedoIt();						// Override

	virtual void UndoIt();						// Override

protected:
	CAutoPtr_AC<CDynamicArray_AC> fPointList;					// list of points to set or unset

	TIconBitMap* fSavedBitMap;
	TIconDocument* fIconDocument;				// The document affected by this command.
	bool fErasePoints;
};

// ====================================================================================

class TSetColorCommand : public TCommand
{
public:
	TSetColorCommand(TIconDocument* itsIconDocument = NULL,
								  const CRGBColor_AC& newColor = gRGBWhite);
	// Constructor

	virtual ~TSetColorCommand();
	// Destructor

	virtual void DoIt();						// Override

	virtual void RedoIt();						// Override

	virtual void UndoIt();						// Override

protected:
	CRGBColor_AC fOldColor;
	CRGBColor_AC fNewColor;
};


// ====================================================================================

class TInvertCommand : public TCommand
{
public:
	TInvertCommand(TIconDocument* itsIconDocument = NULL);
	// Constructor

	virtual ~TInvertCommand();
	// Destructor

	virtual TAppleEvent* MakeAppleEvent();


	virtual void DoIt();						// Override

	virtual void RedoIt();						// Override

	virtual void UndoIt();						// Override

protected:
	TIconDocument* fIconDocument;				// The document affected by this command.
};


// ====================================================================================

class TIconDrawCommand : public TTracker
{
public:
	TIconDrawCommand(TIconEditView* itsIconEditView = NULL,
								  TIconDocument* itsIconDocument = NULL,
								  const CViewPoint& theMouse = gZeroVPt);
	// Constructor

	virtual ~TIconDrawCommand();

	virtual void DoIt();						// Override
	//  Sets the fIconDocument's icon bitmap to that of fIconBitMap by calling
	//	the document's SetIcon method. Validates the edit view to avoid
	//	flashing. 	

	virtual void RedoIt();						// Override

	virtual void TrackConstrain(TrackPhase aTrackPhase,
								const CViewPoint& anchorPoint,
								const CViewPoint& previousPoint,
								CViewPoint& nextPoint,
								bool mouseDidMove);

	virtual void TrackFeedback(TrackPhase aTrackPhase,
							   const CViewPoint& anchorPoint,
							   const CViewPoint& previousPoint,
							   const CViewPoint& nextPoint,
							   bool mouseDidMove,
							   bool turnItOn);

	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
								 CViewPoint& anchorPoint,
								 CViewPoint& previousPoint,
								 CViewPoint& nextPoint,
								 bool mouseDidMove);

	virtual void UndoIt();						// Override
	//  Sets the fIconDocument's icon bitmap to that of fOriginalIcon by calling
	//	the document's SetIcon method. 

protected:
	CRGBColor_AC fColor;							// drawing color (property of fDocument)

	CAutoPtr_AC<TIconBitMap> fIconBitMap;		// The icon in which drawing takes place.
	CAutoPtr_AC<TIconBitMap> fOriginalIcon;		// A copy of the original icon.
	CAutoPtr_AC<TIconBitMap> fIconBitMapModel;	// Model used to generate apple events

	TIconDocument* fIconDocument;				// The document affected by this command.
	TIconEditView* fIconEditView;				// The view in which this command draws.
	bool fTurnBitsOn;							// Whether to draw bits black or white.
};


// ====================================================================================

class TIconEditCommand : public TCommand
{
public:
	TIconEditCommand(CommandNumber itsCommandNumber = cNoCommand,
								  TIconEditView* itsIconEditView = NULL,
								  TIconDocument* itsIconDocument = NULL);
	// Constructor

	virtual ~TIconEditCommand();

	virtual void DoIt();						// Override

	virtual void RedoIt();						// Override

	virtual void UndoIt();						// Override

protected:
	CAutoPtr_AC<TIconBitMap> fSavedBitMap;		// Saved state for undo/redo

	TIconDocument* fIconDocument;				// The document affected by this command.
	TIconEditView* fIconEditView;				// The view in which this command draws.
};


// ====================================================================================

class TIconPasteCommand : public TCommand
{
public:
	TIconPasteCommand(TIconEditView* itsIconEditView = NULL,
								   TIconDocument* itsIconDocument = NULL);
	// Constructor

	virtual ~TIconPasteCommand();

	virtual void DoIt();						// Override

	virtual void UndoIt();						// Override

protected:
	CAutoPtr_AC<TIconBitMap> fSavedIcon;			// Temporary saved icon

	TIconDocument* fIconDocument;				// The document affected by this command.
	TIconEditView* fIconEditView;				// The view in which this command draws.
};


// ====================================================================================
class TZoomOutCommand : public TCommand
{
public:
	TZoomOutCommand(TIconDocument* itsIconDocument = NULL);
	// Constructor

	virtual ~TZoomOutCommand();
	// Destructor

	virtual void DoIt();						// Override

	virtual void UndoIt();						// Override

	virtual TAppleEvent* MakeAppleEvent();

public:
	TIconDocument* fIconDocument;				// The document affected by this command.
};


// ====================================================================================
class TZoomInCommand : public TCommand
{
public:
	TZoomInCommand(TIconDocument* itsIconDocument = NULL);//, TAppleEvent* theAppleEvent);

	// Constructor

	virtual ~TZoomInCommand();
	// Destructor

	virtual void DoIt();						// Override

	virtual void UndoIt();						// Override

	virtual TAppleEvent* MakeAppleEvent();

public:
	TIconDocument* fIconDocument;				// The document affected by this command.
};

#endif


