//----------------------------------------------------------------------------------------
// Nothing.cp
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	This is a very small sample application. The application's windows each contain
	the word 'MacApp', in large type, and are framed by a large gray border. 
	
	All applications that create views procedurally need to reimplement at least
	three methods:
	
		TApplication::DoMakeDocument	Launches the appropriate type of Document object
		TDocument::DoMakeViews			Launches the appropriate type of View and window
										objects
		TView::Draw						Draws the contents of a view
	
	This application, however, consists of the reimplementation of only one method, 
	TView::Draw. This is possible since the view is created from templates; MacApp
	supplies the default 'view' resource. So in a sense this application is the smallest
	possible MacApp application.
*/

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UMACAPPGLOBALS
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

// Toolbox

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __TYPES__
#include <Types.h>
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------
enum
{
	kSignature = 'SS01',				// Application signature
	kFileType  = 'SF01'				// File-type code used for document files created by this application
};

//----------------------------------------------------------------------------------------
// TNothingApplication:
//----------------------------------------------------------------------------------------
class TNothingApplication : public TApplication
{
public:
	TNothingApplication();

	virtual ~TNothingApplication();
	
	virtual TDocument* DoMakeDocument(CommandNumber itsCommandNumber, TFile* itsFile);
		// Override
};

//----------------------------------------------------------------------------------------
// TDefaultView:
//----------------------------------------------------------------------------------------
class TDefaultView : public TView
{
	MA_DECLARE_CLONE;
	
public:
	virtual ~TDefaultView();
	
	virtual void Draw(const CViewRect& area);
		// Draws the view seen in the window. Every nonblank view MUST override this method.
};

//========================================================================================
// CLASS TNothingApplication
//========================================================================================
//----------------------------------------------------------------------------------------
// TNothingApplication constructor
//----------------------------------------------------------------------------------------

TNothingApplication::TNothingApplication() :
	TApplication(kFileType, kSignature)
{
	// So we can create the view object by name and
	// So my view will be substituted when MacApp creates the "default view"	
	REGISTER_SIGNATURE_AC(TDefaultView, kStdDefaultView);
}

//----------------------------------------------------------------------------------------
// TNothingApplication destructor
//----------------------------------------------------------------------------------------

TNothingApplication::~TNothingApplication()
{
}

//----------------------------------------------------------------------------------------
// TNothingApplication::DoMakeDocument: 
//----------------------------------------------------------------------------------------

TDocument* TNothingApplication::DoMakeDocument(CommandNumber, TFile* /*itsFile*/)
{
	return TH_new TDocument;
}

//========================================================================================
// CLASS TDefaultView
//========================================================================================
MA_DEFINE_CLASS(TDefaultView);

//----------------------------------------------------------------------------------------
// TDefaultView destructor
//----------------------------------------------------------------------------------------

TDefaultView::~TDefaultView()
{
}

//----------------------------------------------------------------------------------------
// TDefaultView::Draw: 
//----------------------------------------------------------------------------------------

void TDefaultView::Draw(const CViewRect&)
{
	::PenNormal();
	::PenSize(10, 10);
	::PenPat(&qd.dkGray);

	// Draw a dark gray frame
	::FrameRect(&GetQDExtent());

	// Set font and size for subsequent display in the window
	::TextFont(times);	// use a nice outline font
	::TextSize(72);

	::MoveTo(45, 90);

	// Draw the word 'MacApp' in large type.
	::DrawString("\pMacApp");

	// Restore the pen state.
	::PenNormal();
}


//========================================================================================
// GLOBAL Functions
//========================================================================================

//----------------------------------------------------------------------------------------
// main:
//----------------------------------------------------------------------------------------
void main()
{
	InitUMacApp(3);					// Initialize MacApp; 3 calls to MoreMasters

	TNothingApplication aNothingApplication;
	aNothingApplication.Run();
}

//----------------------------------------------------------------------------------------
// End of Nothing.cp
