//----------------------------------------------------------------------------------------
// UDocumentSkeleton.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UDOCUMENTSKELETON__
#include "UDocumentSkeleton.h"
#endif

// Skeleton

#ifndef __UCOMMANDSKELETON__
#include "UCommandSkeleton.h"
#endif

#ifndef __UVIEWSKELETON__
#include "UViewSkeleton.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//----------------------------------------------------------------------------------------
// Constants:

const ResNumber kSkeletonWindowID = kDefaultWindowID;

//========================================================================================
// CLASS TDocumentSkeleton
//========================================================================================
#undef Inherited
#define Inherited TFileBasedDocument


MA_DEFINE_CLASS(TDocumentSkeleton);

//----------------------------------------------------------------------------------------
// TDocumentSkeleton constructor 
//----------------------------------------------------------------------------------------


TDocumentSkeleton::TDocumentSkeleton(TFile* itsFile, OSType itsCreator) :
TFileBasedDocument(itsFile,itsCreator)
{
	// Initialize fields to safe values here
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton destructor
//----------------------------------------------------------------------------------------


TDocumentSkeleton::~TDocumentSkeleton()
{
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::FreeData: 
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::FreeData() // Override 
{
	Inherited::FreeData();
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::DoInitialState: 
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::DoInitialState() // Override 
{
	Inherited::DoInitialState();
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::DoMakeViews: 
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::DoMakeViews(bool /*forPrinting*/) // Override 
{
	TWindow* aWindow = gViewServer->NewTemplateWindow(kSkeletonWindowID, this);
	::ThrowIfNULL_AC(aWindow);

	TViewSkeleton* aViewSkeleton = (TViewSkeleton*) (aWindow->FindSubView('SKEL'));	// Must cast because FindSubView returns TView

	TStdPrintHandler* aHandler = TH_new TStdPrintHandler(this,			// its document 
							   aViewSkeleton,	// its view 
							   kFixedSize,		// horzontal page size is fixed 
							   kFixedSize);		// vertical page size is fixed 
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes) // Override 
{
	Inherited::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::DoMenuCommand: This method is overridden to handle menu items which
// are enabled when this document is in the target chain. In this example, this is true
// when the document is open and its window is the active window. The inherited method
// should always be called so that MacApp can allow successor objects in the target chain
// (i.e. the application) to handle THEIR menu items.
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		case cCommandHandledByDocument: 
			PostCommand(TH_new TCommandSkeleton(aCommandNumber, this));
			break;

		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::DoRead: 
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::DoRead(TFile* aFile,
						   			bool forPrinting) // Override 
{
	Inherited::DoRead(aFile,forPrinting);
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::DoSetupMenus: This method is overridden to enable menu items which
// should be enabled when this object is in the target chain. MacApp initially disables
// all menu items, then lets the objects in the target chain enable those items they
// handle.
//
// A document object is in the target chain when its window is the active window.
//
// The inherited method is called so that TDocument can enable document-level menu items
// like "Save ". This also ensures that objects further up the target chain (the
// application) can set up THEIR menus.
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::DoSetupMenus() // Override 
{
	Inherited::DoSetupMenus();
	
	Enable(cCommandHandledByDocument,true);
}

//----------------------------------------------------------------------------------------
// TDocumentSkeleton::DoWrite: 
//----------------------------------------------------------------------------------------


void TDocumentSkeleton::DoWrite(TFile* aFile,
									bool makingCopy) // Override 
{
	Inherited::DoWrite(aFile,makingCopy);
}

//----------------------------------------------------------------------------------------
// End of UDocumentSkeleton.cp


