//----------------------------------------------------------------------------------------
// UViewSkeleton.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __UVIEWSKELETON__
#include "UViewSkeleton.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UDOCUMENTSKELETON__
#include "UDocumentSkeleton.h"
#endif

#ifndef __UTRACKERSKELETON__
#include "UTrackerSkeleton.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __FONTS__
#include "Fonts.h"
#endif

//----------------------------------------------------------------------------------------
// Constants:

const CommandNumber cCommandHandledByView = 402;
	
//========================================================================================
// Global Initialization Procedure
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// InitUViewSkeleton: 
//----------------------------------------------------------------------------------------


void InitUViewSkeleton()
{
	// So the linker doesn't dead strip class info 
	MA_REGISTER_CLASS(TViewSkeleton);
}


//========================================================================================
// CLASS TViewSkeleton
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TViewSkeleton);

//----------------------------------------------------------------------------------------
// TViewSkeleton constructor 
//----------------------------------------------------------------------------------------


TViewSkeleton::TViewSkeleton(const CViewPoint& itsLocation,
						 				const CViewPoint& itsSize) : 
	TView(itsLocation,itsSize,sizeFixed, sizeFixed),

	fDocumentSkeleton(NULL)
{
}

//----------------------------------------------------------------------------------------
// TViewSkeleton destructor
//----------------------------------------------------------------------------------------


TViewSkeleton::~TViewSkeleton()
{
}
	
//----------------------------------------------------------------------------------------
// TViewSkeleton::Draw: 
//----------------------------------------------------------------------------------------


void TViewSkeleton::Draw(const CViewRect& area) // Override 
{
	const ViewCoordinate boxHeight = 50;
	const long nRows = fSize.v / boxHeight;
	const CViewPoint increment(fSize.h / nRows, boxHeight);

	CViewRect aVRect(gZeroVPt, increment);				// construct the rect at top left of view.
	
	::PenNormal();	
	::PenPat(&qd.ltGray);

	for (long row = 0; row < nRows; ++row)
	{
		if (!(area & aVRect).Empty())
			::PaintRect(&ViewToQD(aVRect));
			
		if (!row)									// draw title over first rect, regardless of rect's visibility.
		{
			CViewRect stringRect(aVRect);
			stringRect.right += 400;
			if (!(area & stringRect).Empty())
			{
				::TextFont(times);			
				::TextSize(static_cast<short>(stringRect.GetLength(vSel)));
				MADrawString("Skeleton Application",ViewToQD(stringRect),teFlushDefault,true);
			}
		}
		
		aVRect += increment;						// move rect after drawing.
	}
}

//----------------------------------------------------------------------------------------
// TViewSkeleton::DoMenuCommand: This method is overridden to handle menu items which are
// enabled when this view is in the target chain. In this example, this is true when the
// window containing this view is the active window. The inherited method should always be
// called so that MacApp can allow successor objects in the target chain (i.e. document
// and application) to handle THEIR menu items.
//----------------------------------------------------------------------------------------


void TViewSkeleton::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		case cCommandHandledByView :
			::SysBeep(2);
			break;
			
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TViewSkeleton::DoPostCreate: 
//----------------------------------------------------------------------------------------

	
void TViewSkeleton::DoPostCreate(TDocument* itsDocument) // Override 
{
	Inherited::DoPostCreate(itsDocument);
	
	fDocumentSkeleton = dynamic_cast_AC(TDocumentSkeleton*,itsDocument);
}
	
//----------------------------------------------------------------------------------------
// TViewSkeleton::DoSetupMenus: This method is overridden to enable menu items which
// should be enabled when this view is in the target chain. MacApp initially disables all
// menu items, then lets the objects in the target chain enable those items they handle.
//
// A view is generally in the target chain when its window is the active window, and the
// view or one of its subviews is designated as the target of that window. In this
// example, the window's resource specifies the skeleton view as the target.
// 
// The inherited method is called so that objects further up the target chain (the
// scroller, window, document and application) can set up THEIR menus.
//----------------------------------------------------------------------------------------

			
void TViewSkeleton::DoSetupMenus() // Override 
{
	Inherited::DoSetupMenus();

	Enable(cCommandHandledByView,true);
}

//----------------------------------------------------------------------------------------
// TViewSkeleton::DoMouseCommand: 
//----------------------------------------------------------------------------------------


void TViewSkeleton::DoMouseCommand(CViewPoint& theMouse,
								  TToolboxEvent* /* event */,
								  CPoint_AC /* hysteresis */)	// Override
{
	PostCommand(TH_new TTrackerSkeleton(fDocumentSkeleton, this, GetScroller(false), theMouse));
}

//----------------------------------------------------------------------------------------
// End of UViewSkeleton.cp


