//----------------------------------------------------------------------------------------
// UODPartView.h
// Copyright  1994-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UODPARTVIEW__
#define __UODPARTVIEW__

#if qALOE

// ACS ALOE

#ifndef __CODAdaptor_AC__
#include "CODAdaptor_AC.h"
#endif

#ifndef __CODAdaptorView_AC__
#include "CODAdaptorView_AC.h"
#endif

// MacApp

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

//----------------------------------------------------------------------------------------
// TODPartView
//----------------------------------------------------------------------------------------

class EXPORT_MA TODPartView : public TView
{
	MA_DECLARE_CLASS;
	
	public:
		TODPartView(const CViewPoint& itsLocation = gZeroVPt,
				const CViewPoint& itsSize = gZeroVPt,
				SizeDeterminer itsHSizeDet = sizeVariable,
				SizeDeterminer itsVSizeDet = sizeVariable,
				CODAdaptor_AC* adaptor = NULL,
				const CViewRect& itsInset = gZeroVRect);

		virtual 					~TODPartView();

#if qDrag

		virtual bool				WillDrag(	const CViewPoint& 	localMouse,
												const RgnHandle	dragCursorRegion);
			// We will drag if we have a valid adaptor
			
		virtual void				DoAddDragContent();
			// We will add our view flavor first, then the adaptor content second
			
		virtual bool 				WillAcceptDrop(CDragItemIterator& dragItemIterator);
			// delegate this to our AdaptorView
			
		virtual TCommand*			DoMakeDragDropCommand(CommandNumber itsCommandNumber,
													  CDragItemIterator& dragItemIterator);
			// create a dummy command to satisfy MacApp on behalf of the AdaptorView
#endif
			
		virtual void 				BeInSuperView(TView* itsSuperView);
			// create or delete our adaptor view depending on our GrafPtr's existence

		virtual void				Draw(const CViewRect& area);
			// Draw the adaptor
			
		virtual	void				SetFrame(const CViewRect& newFrame, bool invalidate);
			// OpenDoc parts must be offset relative to the window
		
		virtual void 				ComputeFrame(CViewRect& newFrame);
			// Allow for inset for some size determiners
			
		virtual void				UpdateCoordinates();
			// OpenDoc parts must be offset relative to the window
			
		virtual void				GetExtentRegion(RgnHandle itsExtentRgn);
			// OpenDoc parts have region extents: honor them

		virtual CViewRect 				CalcMinFrame();
			// return a rect that fits adaptor's shape

		virtual void DoSetCursor(const CViewPoint& localPoint, 
									RgnHandle cursorRegion);
			// Eat the cursor region so that OpenDoc can set it properly ??? is this right?
			
		CODAdaptor_AC*					GetAdaptor() const;
			// Get my adaptor
					
		virtual	void				SetAdaptor(CODAdaptor_AC* adaptor);
			// Set my adaptor
		
		CODAdaptorView_AC* 			GetAdaptorView() const;
			// Get my adaptor view
							
		virtual void				SizeAdaptorToView();
			// force the adaptor to conform to the size of the view

		virtual	void 				SizeViewToAdaptor();
			// force the view conform to the size of the adaptor
		
		virtual bool				HasAdaptorView();
			// Does this view have an adaptor attached to it?
			
		virtual void				DoKeyEvent(TToolboxEvent* event);
			// Handle the delete key. ??? belongs in TContainerView?

		virtual void				DoSetupMenus();
		
		virtual void 				DoMenuCommand(CommandNumber aCommandNumber);
				
		virtual void				NewAdaptorView();
			// Called when an adaptor is created or when the adaptor is about to be shown

		// Handy utilities for binding a view to an adaptor
		
		virtual void				NewAdaptorAndView(ALOEISOString adaptorType, Ptr dataPtr = NULL, long dataSize = 0);
			// Utility for dynamically creating an adaptor and adaptorView by ISOString

		inline void					NewAdaptorAndView(ResType adaptorType, Ptr dataPtr = NULL, long dataSize = 0)
			{ NewAdaptorAndView(::ALOEGetKindFromResType(adaptorType), dataPtr, dataSize); }
			// Utility for dynamically creating an adaptor and adaptorView by ResType
				
	protected:
		CViewPoint						GetWindowOrigin();
			// Return our 0,0 origin as Focused Window offset
																					
	protected:
		CAutoPtr_AC<CODAdaptor_AC>		fAdaptor;
		CAutoPtr_AC<CODAdaptorView_AC>	fAdaptorView;
		CViewRect						fInset;			// Amount to inset from edges of view
};


//----------------------------------------------------------------------------------------
// TODDragCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TODDragCommand : public TCommand
{
	TODDragCommand();
	
	virtual ~TODDragCommand();
	
public:
	TODDragCommand(CommandNumber itsCommandNumber, TODPartView* itsView, TCommandHandler* itsContext, 
					bool canUndo, bool causesChange, MDependable_AC* objectToNotify);

	virtual void	DoIt();
	
	virtual void	UndoIt();
	
	// Data members
	
	TODPartView*	fODPartView;
	CViewPoint			fSavedLocation;
};

//----------------------------------------------------------------------------------------
// Initialization for this file
//----------------------------------------------------------------------------------------

EXPORT_MA  void InitUODPartView();

#endif // qALOE

#endif // __UODPartView__
