//----------------------------------------------------------------------------------------
// UODAdaptable.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#if qALOE

#ifndef __UODAdaptable__
#include "UODAdaptable.h"
#endif

// ACS ALOE

#ifndef __CODSession_AC__
#include "CODSession_AC.h"
#endif

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

// MacApp

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __USTREAMABLE__
#include "UStreamable.h"
#endif

#ifndef __UFILESTREAM__
#include "UFileStream.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

//----------------------------------------------------------------------------------------
// Constants and types:

const ResType kAdaptorMgrSizeResType = 'aloe';
const short kAdaptorMgrSizeResID = 1515;

typedef long* AdaptorMgrSizePtr;
typedef AdaptorMgrSizePtr* AdaptorMgrSizeHandle;

//========================================================================================
// CLASS MODAdaptable
//========================================================================================


//----------------------------------------------------------------------------------------
// MODAdaptable constructor 
//----------------------------------------------------------------------------------------


MODAdaptable::MODAdaptable(bool createAdaptorManager)
{
	if (createAdaptorManager)
		MakeAdaptorManager();
} // MODAdaptable::MODAdaptable 

//----------------------------------------------------------------------------------------
// MODAdaptable destructor
//----------------------------------------------------------------------------------------


MODAdaptable::~MODAdaptable()
{
}

//----------------------------------------------------------------------------------------
// MODAdaptable::IsChanged
//----------------------------------------------------------------------------------------


bool MODAdaptable::IsChangedAdaptorManager()
{
	// NB may be called before adaptor manager has had a chance to be created
	return (GetAdaptorManager() != NULL && GetAdaptorManager()->HasChanged());
}

//----------------------------------------------------------------------------------------
// MODAdaptable::DoRead: 
//----------------------------------------------------------------------------------------


void MODAdaptable::DoReadAdaptorManager(TFile* aFile)
{
	// The adaptor manager itself is stuffed at the end of the file 
	// because adaptors may save at any time independently from this call.
	long adaptorMgrSize = 0;

	// Strategy for saving adaptor manager size: use resource fork,
	// where it's relatively safe from corruption in a bad data fork.
	if (aFile->HasRsrcFork() && aFile->IsRsrcForkOpen())
	{
		CTempUseResFile_AC tempUseResFile(aFile->GetRsrcRefNum());
		if (ResError() == noErr)
		{
			AdaptorMgrSizeHandle theRsrc = AdaptorMgrSizeHandle(Get1Resource(kAdaptorMgrSizeResType, kAdaptorMgrSizeResID));
			if (theRsrc)
				adaptorMgrSize = **theRsrc;
		}
	}

	if (adaptorMgrSize > 0)
	{
		// read adaptor manager relative to end of file
		TFileStream aFileStream(aFile);
		aFileStream.SetPosition(aFileStream.GetSize() - adaptorMgrSize);
		fAdaptorMgr = CODSession_AC::Instance()->NewAdaptorMgrFromFile(aFile->GetDataRefNum());
		aFileStream.SetPosition(0);
	}
} // MODAdaptable::DoRead 


//----------------------------------------------------------------------------------------
// MODAdaptable::DoWrite: 
//----------------------------------------------------------------------------------------


void MODAdaptable::DoWriteAdaptorManager(TFile* aFile)
{
	// !!! assert that file position is at end of file
	
	if (GetAdaptorManager())
	{
		// Save adaptor manager at end of file
		long adaptorMgrSize = GetAdaptorManager()->Save(aFile->GetDataRefNum());
		
		// Save adaptor manager size in resource
		if (aFile->HasRsrcFork())
		{
			// allocate a handle for the resource
			CTempHandle_AC theRsrc(NewPermHandle(sizeof(long)));
			
			// write the adaptor manager size to the handle
			AdaptorMgrSizeHandle tempRsrc = AdaptorMgrSizeHandle(Handle(theRsrc));
			**tempRsrc = adaptorMgrSize;
			
			// add the new selection resource
			MAAddResource(theRsrc, kAdaptorMgrSizeResType, kAdaptorMgrSizeResID, gEmptyString);
			::ThrowIfResError_AC();
			theRsrc.DontRelease();
		}
	}
} // MODAdaptable::DoWrite 

//----------------------------------------------------------------------------------------
// MODAdaptable::GetAdaptorManager: 
//----------------------------------------------------------------------------------------


CODAdaptorMgr_AC* MODAdaptable::GetAdaptorManager()
{ 
	return fAdaptorMgr; 
}

//----------------------------------------------------------------------------------------
// MODAdaptable::MakeAdaptorManager: 
//----------------------------------------------------------------------------------------


void MODAdaptable::MakeAdaptorManager()
{
	if (!fAdaptorMgr)
		fAdaptorMgr = CODSession_AC::Instance()->NewAdaptorMgr();
}

#endif // qALOE

//----------------------------------------------------------------------------------------
// End of MODAdaptable.cp


