//----------------------------------------------------------------------------------------
// UODDispatcher.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#if qALOE

#ifndef __UODDispatcher__
#include "UODDispatcher.h"
#endif

// ACS ALOE

#ifndef __CODSession_AC__
#include "CODSession_AC.h"
#endif

#ifndef __UODSession__
#include "UODSession.h"
#endif

// MacApp

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif


//========================================================================================
// CLASS TODDispatcher
//========================================================================================
#undef Inherited
#define Inherited TDispatcher

//----------------------------------------------------------------------------------------
// TODDispatcher constructor
//----------------------------------------------------------------------------------------


TODDispatcher::TODDispatcher(OSType itsMainFileType,
									 OSType itsCreator) :
		TDispatcher(itsMainFileType, itsCreator)
{
	// OpenDoc needs to decide for us when it's tracking the cursor,
	// so we can't leave this decision up to mouseMoved events
	fAlwaysTrackCursor = true;
}

//----------------------------------------------------------------------------------------
// TODDispatcher destructor
//----------------------------------------------------------------------------------------


TODDispatcher::~TODDispatcher()
{
	// ODSession disconnects with ALOE automatically
	delete CODSession_AC::Instance();
}

//----------------------------------------------------------------------------------------
// TODDispatcher::NewODSession
//----------------------------------------------------------------------------------------


void TODDispatcher::NewODSession()
{
	// Create session object.  This does not start a session.
	TH_new TODSession;
}

//----------------------------------------------------------------------------------------
// TODDispatcher::Run
//----------------------------------------------------------------------------------------


void TODDispatcher::Run()
{
	NewODSession();
	
	Inherited::Run();
}

//----------------------------------------------------------------------------------------
// TODDispatcher::GetEvent: 
//----------------------------------------------------------------------------------------


TToolboxEvent* TODDispatcher::GetEvent(short eventMask,
									 unsigned long sleep,
									 RgnHandle sleepRegion)
{
	 TToolboxEvent* theEvent = Inherited::GetEvent(
		CODSession_AC::GetEventMask(eventMask), 
		CODSession_AC::GetSleepTime(sleep),
		CODSession_AC::GetSleepRegion(sleepRegion));
	
	return theEvent;
}

//----------------------------------------------------------------------------------------
// TODDispatcher::DispatchEvent: 
//----------------------------------------------------------------------------------------


void TODDispatcher::DoToolboxEvent(TToolboxEvent* event)
{
	bool handledEvent = false;

	// ALOE gets first chance at responding to events
	handledEvent = CODSession_AC::Instance()->HandleEvent(&event->GetEventRecord(0));
	if (handledEvent && (event->IsDoubleEvent()))
	{
		// double-byte key events: ??? pass through or supress?
		handledEvent = CODSession_AC::Instance()->HandleEvent(&event->GetEventRecord(1));
	}
	
	// ??? Check PP sample for shadowing adaptor view selections when handledEvent
	//     is true in HandleMouseDown
	if (!handledEvent)
		Inherited::DoToolboxEvent(event);
}

//----------------------------------------------------------------------------------------
// TODDispatcher::HandleMouseDown: 
//----------------------------------------------------------------------------------------


void TODDispatcher::HandleMouseDown(TToolboxEvent* event)
{	
	// If there was a mouse event in the menu bar, ALOE will have already called 
	// MenuSelect, so we can't call it in inherited, we must dispatch the unhandled 
	// MenuSelect result directly.
	if (CODSession_AC::Instance()->IsUnhandledMenuEvent(&event->GetEventRecord(0)))
	{
		long menuItem = CODSession_AC::Instance()->GetUnhandledMenuEvent(&event->GetEventRecord(0));
		MenuEvent(HiWord(menuItem), LoWord(menuItem));
	}
	else
		Inherited::HandleMouseDown(event);
}

//----------------------------------------------------------------------------------------
// TODDispatcher::HandleActivateEvent
//----------------------------------------------------------------------------------------


void TODDispatcher::HandleActivateEvent(TToolboxEvent* event)
{
	bool activate = (event->GetModifiers() & activeFlag) != 0;
	if (activate)
		CODSession_AC::Instance()->ApplicationActivated();
	
	Inherited::HandleActivateEvent(event);
}

//----------------------------------------------------------------------------------------
// TODDispatcher::Idle: 
//----------------------------------------------------------------------------------------


void TODDispatcher::Idle(IdlePhase phase)
{
	Inherited::Idle(phase);
	
	if (phase == idleContinue)
		(void) CODSession_AC::Instance()->Idle();
}

//----------------------------------------------------------------------------------------
// TODDispatcher::SetupTheMenus: 
//----------------------------------------------------------------------------------------


void TODDispatcher::SetupTheMenus()
{
	// if we don't have the focus, we should not be messing with the menus.
	if (CODSession_AC::ApplicationHasFocus())
		Inherited::SetupTheMenus();
}

#endif // qALOE

//----------------------------------------------------------------------------------------
// End of UODDispatcher.cp


