//----------------------------------------------------------------------------------------
// UODFileBasedDocument.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#if qALOE

#ifndef __UODFileBasedDocument__
#include "UODFileBasedDocument.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// ALOE

#ifndef __CODSession_AC__
#include "CODSession_AC.h"
#endif

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

//========================================================================================
// CLASS TODFileBasedDocument
//========================================================================================
#undef Inherited
#define Inherited TFileBasedDocument


MA_DEFINE_CLASS(TODFileBasedDocument);

//----------------------------------------------------------------------------------------
// TODFileBasedDocument constructor 
//----------------------------------------------------------------------------------------


TODFileBasedDocument::TODFileBasedDocument(TFile* itsFile,
										  OSType itsCreator) :
TFileBasedDocument(itsFile,itsCreator),
MODAdaptable(!itsFile /* Don't create an adaptor manager if we have a file, it will be read in */)	
{
} // TODFileBasedDocument::TODFileBasedDocument 

//----------------------------------------------------------------------------------------
// TODFileBasedDocument destructor
//----------------------------------------------------------------------------------------


TODFileBasedDocument::~TODFileBasedDocument()
{
}

//----------------------------------------------------------------------------------------
// TODFileBasedDocument::IsChanged
//----------------------------------------------------------------------------------------


bool TODFileBasedDocument::IsChanged()
{
	// NB may be called before adaptor manager has had a chance to be created
	return Inherited::IsChanged() || MODAdaptable::IsChangedAdaptorManager();
}

//----------------------------------------------------------------------------------------
// TODFileBasedDocument::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------


void TODFileBasedDocument::DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes)
{
	Inherited::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);
	
	// ??? We would like to ask ALOE how much space the AdaptorMgr is likely to need
} // TODFileBasedDocument::DoNeedDiskSpace 

//----------------------------------------------------------------------------------------
// TODFileBasedDocument::DoRead: 
//----------------------------------------------------------------------------------------


void TODFileBasedDocument::DoRead(TFile* aFile,
						   	   bool forPrinting)
{
	// The adaptor manager itself is stuffed at the end of the file 
	// because adaptors may save at any time independently from this call.
	MODAdaptable::DoReadAdaptorManager(aFile);
	
	Inherited::DoRead(aFile, forPrinting);	
} // TODFileBasedDocument::DoRead 


//----------------------------------------------------------------------------------------
// TODFileBasedDocument::DoWrite: 
//----------------------------------------------------------------------------------------


void TODFileBasedDocument::DoWrite(TFile* aFile,
									bool makingCopy)
{
	// !!! assert that file position is at end of file
	
	// Save adaptor manager at end of file
	MODAdaptable::DoWriteAdaptorManager(aFile);
	
	Inherited::DoWrite(aFile, makingCopy);
} // TODFileBasedDocument::DoWrite 


#endif // qALOE

//----------------------------------------------------------------------------------------
// End of UODFileBasedDocument.cp


