//----------------------------------------------------------------------------------------
// UODPartView.cp
// Copyright  1994-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#if qALOE

#ifndef __UODPARTVIEW__
#include "UODPartView.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif


#ifndef __UFILEBASEDDOCUMENT__
#include "UFileBasedDocument.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif


#ifndef __UODAdaptable__
#include "UODAdaptable.h"
#endif



const CommandNumber cInsertPart 	= 6661; 
const CommandNumber cCopyODPart 	= 6662; 
const CommandNumber cCutODPart 		= 6663; 
const CommandNumber cPasteODPart 	= 6664; 

#ifndef __UODDefaults__
#include "UODDefaults.h"
#endif

//========================================================================================
// CLASS TODPartView
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TODPartView);

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------


TODPartView::TODPartView(const CViewPoint& itsLocation,
				  			  	const CViewPoint& itsSize,
				 			  	SizeDeterminer itsHSizeDet,
				  			  	SizeDeterminer itsVSizeDet,
				  			  	CODAdaptor_AC* adaptor,
				  			  	const CViewRect& itsInset) :
		TView(itsLocation,itsSize,itsHSizeDet,itsVSizeDet),
		fAdaptor(adaptor),
		fAdaptorView(NULL),
		fInset(itsInset)
{
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------


TODPartView::~TODPartView()
{
}

#if qDrag

//----------------------------------------------------------------------------------------
// TODPartView::WillDrag
//----------------------------------------------------------------------------------------


bool TODPartView::WillDrag(	const CViewPoint& 	localMouse,
							const RgnHandle	dragCursorRegion)
{
	return Inherited::WillDrag(localMouse, dragCursorRegion) ? (HasAdaptorView()) : false;
}

//----------------------------------------------------------------------------------------
// TODPartView::DoAddDragContent
//----------------------------------------------------------------------------------------


void TODPartView::DoAddDragContent()
{
}

//----------------------------------------------------------------------------------------
// TODPartView::WillAcceptDrop
//----------------------------------------------------------------------------------------


bool TODPartView::WillAcceptDrop(CDragItemIterator& /*dragItemIterator*/)
{
	// This view can accept a drop if the adaptor view says so
	// but the adaptor view will handle the acceptance.
	bool acceptDrop = false;
	
	if (HasAdaptorView())
		acceptDrop = GetAdaptorView()->CanAcceptDrop();
		
	return acceptDrop;
}

//----------------------------------------------------------------------------------------
// TODPartView::DoMakeDragDropCommand
//----------------------------------------------------------------------------------------


TCommand* TODPartView::DoMakeDragDropCommand(CommandNumber itsCommandNumber,
										CDragItemIterator&	dragItemIterator)
{
	TCommand 		*returnCommand = NULL;
	TCommandHandler *itsContext = GetContext(itsCommandNumber);
	
	switch(itsCommandNumber)
	{
		case cDrag:
			{
				// Return a command that can drag our view around or embed its content
				returnCommand = TH_new TODDragCommand(itsCommandNumber, this, itsContext, kCanUndo, 
												kCausesChange, itsContext);
			}
			break;
			
		case cDrop:
			{			
				// return a dummy command to ensure proper drag drop feedback
				returnCommand = TH_new TCommand(itsCommandNumber,
												itsContext, kCantUndo, kDoesNotCauseChange,
												itsContext);
			}
			break;
		
		case cDragMove: // doesn't do anything...will cause return value to be NULL
			break;
			
		default:
			returnCommand = Inherited::DoMakeDragDropCommand(itsCommandNumber, dragItemIterator);
			break;
	}
	
	return returnCommand;
}

#endif // qDrag
//----------------------------------------------------------------------------------------
// TODPartView::BeInSuperView: 
//----------------------------------------------------------------------------------------

	
void TODPartView::BeInSuperView(TView* itsSuperview)
{
	// call inherited first so that we can safely call GetGrafPort()	
	Inherited::BeInSuperView(itsSuperview);

	if (itsSuperview)
	{
		if (fAdaptor)
		{
			NewAdaptorView();
			
			// Need to set droppable _after_ we are added to a GrafPort()
			SetDroppable(true);
			ShowReverted();
		}
	}
	else
	{
		// ??? is it our class's responsibility to delete fAdaptorView?
		fAdaptorView = NULL;
	}
} 

//----------------------------------------------------------------------------------------
// TODPartView::Draw
//----------------------------------------------------------------------------------------


void TODPartView::Draw(const CViewRect& /*area*/)
{
	if (HasAdaptorView())
	{		
		CTempRegion_AC thisRgn = GetAdaptor()->GetShape();
		CPoint_AC itsOffset = GetAdaptorView()->GetOffset();
		::OffsetRgn(thisRgn, itsOffset.h, itsOffset.v);
		
		// ALOE: OpenDoc parts draw with origin at top left of window,
		// so we must focus on the window, draw, and then focus back again
		GetWindow()->Focus();
		GetAdaptorView()->Draw(thisRgn);		
		Focus();		
	}
}

//----------------------------------------------------------------------------------------
// TODPartView::SetFrame
//----------------------------------------------------------------------------------------


void TODPartView::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	
	if (HasAdaptorView())
	{
		// Update location
		GetAdaptorView()->SetOffset(GetWindowOrigin().AsPoint());
	}
}

//----------------------------------------------------------------------------------------
// TODPartView::ComputeFrame: 
//----------------------------------------------------------------------------------------


void TODPartView::ComputeFrame(CViewRect& newFrame)		// override
{
	Inherited::ComputeFrame(newFrame);

	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		switch (fSizeDeterminer[vhs])
		{
			case sizeFixed:
			case sizeRelSuperView:
				break;

			case sizeSuperView:
				if (fSuperView)
					break;
				// else drop through to default
			default:
				newFrame[botRight][vhs] += fInset[topLeft][vhs] + fInset[botRight][vhs];
				break;
		}
}

//----------------------------------------------------------------------------------------
// TODPartView::InvalidateCoordinates
//----------------------------------------------------------------------------------------

void TODPartView::UpdateCoordinates()
{
	bool  wasInvalid = fCoordinatesAreInvalid;

	Inherited::UpdateCoordinates();
	
	// Update adaptor view's offset
	if (wasInvalid && HasAdaptorView())
		GetAdaptorView()->SetOffset(GetWindowOrigin().AsPoint());	
}

//----------------------------------------------------------------------------------------
// TODPartView::GetExtentRegion
// OpenDoc parts have region extents: honor them
//----------------------------------------------------------------------------------------


void TODPartView::GetExtentRegion(RgnHandle itsExtentRgn)
{
	if (HasAdaptorView()) {
		GetAdaptor()->GetShape(itsExtentRgn);
		Rect srcRect = (*itsExtentRgn)->rgnBBox;
		CViewRect dstRect(gZeroVPt, fSize);
		dstRect.Inset(-fInset);
		::MapRgn(itsExtentRgn, &srcRect, &dstRect.AsRect());
	}
	else
		Inherited::GetExtentRegion(itsExtentRgn);
}			

//----------------------------------------------------------------------------------------
// TODPartView::CalcMinFrame: 
// Return a rect that fits adaptor's shape 
//----------------------------------------------------------------------------------------


CViewRect TODPartView::CalcMinFrame()
{
	if (HasAdaptorView())
	{
		CTempRegion_AC itsRgn(GetAdaptor()->GetShape());
		CViewRect itsVRect((*itsRgn)->rgnBBox);
		itsVRect.Inset(-fInset);
		return itsVRect;
	}
	else
		return Inherited::CalcMinFrame();
}

//----------------------------------------------------------------------------------------
// TODPartView::GetWindowOrigin: 
// Return our 0,0 origin as Focused Window offset
//----------------------------------------------------------------------------------------


CViewPoint TODPartView::GetWindowOrigin()
{
	if (Focus()) ; //??? why are we focusing here

	return LocalToRootView(fInset[topLeft]);		
}

//----------------------------------------------------------------------------------------
// TODPartView::GetWindowOrigin: 
// Return our 0,0 origin as Focused Window offset
//----------------------------------------------------------------------------------------


void TODPartView::DoSetCursor(const CViewPoint& /*localPoint*/, 
							RgnHandle /*cursorRegion*/)
{			
	// Eat the cursor region so that OpenDoc can set it properly ??? is this right?
	
}
//----------------------------------------------------------------------------------------
// TODPartView::GetAdaptor
//----------------------------------------------------------------------------------------


CODAdaptor_AC* TODPartView::GetAdaptor() const
{
	return fAdaptor;
}

//----------------------------------------------------------------------------------------
// TODPartView::SetAdaptorView
//----------------------------------------------------------------------------------------


void TODPartView::SetAdaptor(CODAdaptor_AC* adaptor)
{
	fAdaptor = adaptor;
	
	// Synch fAdaptorView by removing it and adding again to its superview
	TView* itsSuperView = fSuperView;
	
	itsSuperView->RemoveSubView(this);
	itsSuperView->AddSubView(this);
}

//----------------------------------------------------------------------------------------
// TODPartView::GetAdaptorView
//----------------------------------------------------------------------------------------


CODAdaptorView_AC* TODPartView::GetAdaptorView() const
{
	return fAdaptorView;
}

//----------------------------------------------------------------------------------------
// TODPartView::SizeAdaptorToView
//----------------------------------------------------------------------------------------


void TODPartView::SizeAdaptorToView()
{
	if (HasAdaptorView())
	{
		// Using adaptor's shape, map it to our extent
		
		RgnHandle bounds = GetAdaptor()->GetShape();
		Rect srcRect = (*bounds)->rgnBBox;
		Rect dstRect = GetQDExtent();
		::MapRgn(bounds, &srcRect, &dstRect);
		GetAdaptor()->SetShape(bounds);
	
		// ??? ALOE deletes the RgnHandle?
	}
}

//----------------------------------------------------------------------------------------
// TODPartView::SizeViewToAdaptor
//----------------------------------------------------------------------------------------


void TODPartView::SizeViewToAdaptor()
{
	if (HasAdaptorView())
	{
		CTempRegion_AC shape(GetAdaptor()->GetShape());

		CViewRect myFrame = GetFrame();
		short hDiff = fSize.h - ((*shape)->rgnBBox.right - (*shape)->rgnBBox.left);
		short vDiff = fSize.v - ((*shape)->rgnBBox.bottom - (*shape)->rgnBBox.top);
		myFrame.right += hDiff;
		myFrame.bottom += vDiff;
		
		SetFrame(myFrame, kRedraw);
	}	
}

//----------------------------------------------------------------------------------------
// TODPartView::HasPart()
//----------------------------------------------------------------------------------------


bool TODPartView::HasAdaptorView()
{
	return (fAdaptorView);
}

//----------------------------------------------------------------------------------------
// TODPartView::DoKeyEvent
//----------------------------------------------------------------------------------------
void TODPartView::DoKeyEvent(TToolboxEvent* event)// override 
{

	bool handledCharacter = false;

	if (IsEnabled() && HasAdaptorView())
		if (event->GetText() == chFwdDelete || event->GetText() == chBackspace)
		{
		// !!! Post a delete view command
			handledCharacter = true;
		}
			
	if (!handledCharacter)
		Inherited::DoKeyEvent(event);
}


//----------------------------------------------------------------------------------------
// TODPartView::DoSetupMenus
//----------------------------------------------------------------------------------------

			
void TODPartView::DoSetupMenus()
{
	Inherited::DoSetupMenus();
	
	if (HasAdaptorView())
	{		
 		// cODGetPartInfo should only be enabled if it has an adaptor view and the adaptor 
 		// view is not "active", since there is no sense of selection in the view....
		Enable(kODCommandGetPartInfo, true);	
	}
} 

//----------------------------------------------------------------------------------------
// TODPartView::DoMenuCommand
//----------------------------------------------------------------------------------------


void TODPartView::DoMenuCommand(CommandNumber aCommandNumber)
{

	switch (aCommandNumber) 
	{
		case kODCommandGetPartInfo :
			GetAdaptorView()->ShowInfo();
			break;
			
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
	
}

//----------------------------------------------------------------------------------------
// TODPartView::NewAdaptorAndView
// Utility for dynamically creating an adaptor and adaptorView
//----------------------------------------------------------------------------------------


void TODPartView::NewAdaptorAndView(ALOEISOString adaptorType, Ptr dataPtr, long dataSize)
{
	// Create adaptor sized to our view
	CODAdaptorMgr_AC* adaptorMgr = NULL;
	WindowRef theWindow = GetWindow()->GetWindowRef();
	RgnHandle shape = ::NewRgn();
	::SetRectRgn(shape, 0, 0, fSize.h, fSize.v);
	TCommandHandler* context = GetContext(cNoCommand);
	MODAdaptable* adaptable = dynamic_cast<MODAdaptable*>(context);
	if (adaptable)
		adaptorMgr = adaptable->GetAdaptorManager();

#if qDebug
	if (!adaptable)
		DebugStr("\pThis view's context has no adaptor manager, perhaps you forgot to subclass from TODFileBasedDocument or a mixin of MODAdaptable?");
	else if (!adaptorMgr)
		DebugStr("\pThe Adaptor Manager for the adaptable context has not been initialized.");
#endif

	::ThrowIfNULL_AC(adaptorMgr);
	fAdaptor = adaptorMgr->NewAdaptor(theWindow, adaptorType, shape, dataPtr, dataSize);

	NewAdaptorView();
}

//----------------------------------------------------------------------------------------
// TODPartView::NewAdaptorView
// Called when an adaptor is created or when the adaptor is about to be shown
//----------------------------------------------------------------------------------------


void TODPartView::NewAdaptorView()
{
	CViewPoint offset = GetWindowOrigin();
	fAdaptorView = fAdaptor->NewAdaptorView(GetGrafPort(), offset.AsPoint());
}

//========================================================================================
// CLASS TODDragCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TODDragCommand::TODDragCommand
//----------------------------------------------------------------------------------------


TODDragCommand::TODDragCommand(CommandNumber itsCommandNumber, TODPartView* itsView, TCommandHandler* itsContext, 
					bool canUndo, bool causesChange, MDependable_AC* objectToNotify) :
		TCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify),
		fODPartView(itsView),
		fSavedLocation(itsView->GetLocation())
{
	// Use ALOE to include our adaptor's content
	if (fODPartView->HasAdaptorView())
		fODPartView->GetAdaptorView()->DoStartDrag(TDragDropSession::fgDragDropSession->GetDragReference(),
										fODPartView->GetWindow()->GetWindowRef());

}

//----------------------------------------------------------------------------------------
// TODDragCommand::~TODDragCommand
//----------------------------------------------------------------------------------------


TODDragCommand::~TODDragCommand()
{
}

//----------------------------------------------------------------------------------------
// TODDragCommand::DoIt
//----------------------------------------------------------------------------------------


void TODDragCommand::DoIt()
{
	// ??? not sure what to do here, this is a placeholder
	//CViewRect itsFrame = fODPartView->GetFrame();
	//itsFrame.Offset(fODPartView->fLocation - fSavedLocation);
	//fSavedLocation = fODPartView->fLocation;
	//fODPartView->SetFrame(itsFram);
}

//----------------------------------------------------------------------------------------
// TODDragCommand::UndoIt
//----------------------------------------------------------------------------------------


void TODDragCommand::UndoIt()
{
	DoIt();
}

//----------------------------------------------------------------------------------------
// InitUODPartView: 
//----------------------------------------------------------------------------------------


void InitUODPartView()
{
	MA_REGISTER_CLASS(TODPartView);
} 

//----------------------------------------------------------------------------------------
// End of UODPartView.cp

#endif // qALOE
