//----------------------------------------------------------------------------------------
// UODPartViewCommands.cp
// Copyright  1994-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#if qContainer

#ifndef __UODPARTVIEWCOMMANDS__
#include "UODPartViewCommands.h"
#endif

#ifndef __UODPARTVIEW__
#include "UODPartView.h"
#endif


//========================================================================================
// CLASS TCopyODPartCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TCopyODPartCommand::TCopyODPartCommand
//----------------------------------------------------------------------------------------


TCopyODPartCommand::TCopyODPartCommand(CommandNumber itsCommandNumber,
											TDocument* itsDocument,
											TODPartView* thePartView) :
	TCommand(itsCommandNumber, NULL, kCantUndo, kDoesNotCauseChange, NULL),
	fDocument(itsDocument),
	fODPartView(thePartView)
{
}

//----------------------------------------------------------------------------------------
// TCopyODPartCommand::~TCopyODPartCommand
//----------------------------------------------------------------------------------------



TCopyODPartCommand::~TCopyODPartCommand()
{
}

//----------------------------------------------------------------------------------------
// TCopyODPartCommand::DoIt
//----------------------------------------------------------------------------------------



void TCopyODPartCommand::DoIt()
{
	CADocumentRef	theClipboardDocument = CAGetClipboardDocument();
	CADocumentRef	sourceDocument 	= fDocument->GetContainer();
	CAFrameRef 		sourceFrameRef 	= fODPartView->GetFrameRef();
	CACloneKey		cloneKey 	 	= CABeginClone(sourceDocument, theClipboardDocument, kCACloneCopy);
	CAFrameRef		clipFrameRef	= CACloneFrameRef(sourceFrameRef,sourceDocument,
															theClipboardDocument,true,cloneKey);
	CAEndClone(sourceDocument,theClipboardDocument,cloneKey);
}



//========================================================================================
// CLASS TCutODPartCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TCutODPartCommand::TCutODPartCommand
//----------------------------------------------------------------------------------------


TCutODPartCommand::TCutODPartCommand(CommandNumber itsCommandNumber,
										TDocument* itsDocument,
										TODPartView* thePartView) :
	TCommand(itsCommandNumber, NULL, kCantUndo, kCausesChange, NULL),

	fDocument(itsDocument),
	fODPartView(thePartView)
{
}

//----------------------------------------------------------------------------------------
// TCutODPartCommand::~TCutODPartCommand
//----------------------------------------------------------------------------------------


TCutODPartCommand::~TCutODPartCommand()
{
}

//----------------------------------------------------------------------------------------
// TCutODPartCommand::DoIt
//----------------------------------------------------------------------------------------


void TCutODPartCommand::DoIt()
{
	CADocumentRef	theClipDocument = CAGetClipboardDocument();
	CADocumentRef	sDocument 	= fDocument->GetContainer();
	CAFrameRef 		sFrameRef 	= fODPartView->GetFrameRef();
	CACloneKey		cKey 	 	= CABeginClone(sDocument, theClipDocument, kCACloneCut);
	CAFrameRef		cRef		= CACloneFrameRef(sFrameRef,sDocument,
														theClipDocument,true,cKey);
	CAEndClone(sDocument,theClipDocument,cKey);
	fODPartView->RemovePart(); //not condusive to an undo! (Save it off, remove it in committ)
}


//========================================================================================
// CLASS TPasteODPartCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TPasteODPartCommand::TPasteODPartCommand
//----------------------------------------------------------------------------------------


TPasteODPartCommand::TPasteODPartCommand(CommandNumber itsCommandNumber,
												TDocument* itsDocument,
												TODPartView* thePartView) :
	TCommand(itsCommandNumber, NULL, kCantUndo, kCausesChange, NULL),

	fDocument(itsDocument),	
	fODPartView(thePartView)
{
}

//----------------------------------------------------------------------------------------
// TPasteODPartCommand::~TPasteODPartCommand
//----------------------------------------------------------------------------------------


TPasteODPartCommand::~TPasteODPartCommand()
{
}

//----------------------------------------------------------------------------------------
// TPasteODPartCommand::DoIt
//----------------------------------------------------------------------------------------


void TPasteODPartCommand::DoIt()
{
	if(fODPartView->ContainsClipType(kCAScrapTypeODBentoContainer))
	{
		CADocumentRef	theClipboardDocument;
		long			scrapLength;
		long		    scrapOffset;
		Handle			scrapHandle;

		//If we have a part, get rid of it
		if(fODPartView->HasPart())
			fODPartView->RemovePart();
			
		// Get the container stream
		scrapHandle = NewHandle(0);
		//Lock handle?	FailOSErr!
		scrapLength = CClipboard_AC::GetScrap(scrapHandle, kCAScrapTypeODBentoContainer, &scrapOffset );

		// Open the container
		theClipboardDocument = CAOpenDocument (NULL, scrapHandle, 0, scrapLength);
		fODPartView->AquirePartFromDocument(theClipboardDocument);
		CACloseDocument(theClipboardDocument);
		DisposeHandle(scrapHandle);
	}
}

#endif //qContainer