//----------------------------------------------------------------------------------------
// UODSession.cp
// Copyright  1997 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#if qALOE

#ifndef __UODSession__
#include "UODSession.h"
#endif

// ACS ALOE

#ifndef __UODDefaults__
#include "UODDefaults.h"
#endif

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

// MacApp

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif


#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UODAdaptable__
#include "UODAdaptable.h"
#endif

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------

TODSession::TODSession()
{
}

//----------------------------------------------------------------------------------------
// Install a menu bar for ALOE
//----------------------------------------------------------------------------------------

void TODSession::DoSetBaseMenuBar()
{
	// Menu bar swapping algorithm:
	// 1. Get MacApp to setup menus so the menu bar and menus are loaded
	InvalidateMenus();
	gDispatcher->SetupTheMenus();

	// 2. Save MacApp's default menu bar handle
	SaveMenuBar();

	// 3. ??? 
		
	// 4. Specify ALOE menu bar 
	// !!! There must be no CMNUs in this menu bar
	// until the ALOE CMNU loading code is fixed.
#if qCoreDebug
	if (GetResource('MBAR', kMBarOpenDoc) == NULL)
		DebugStr("\pThe menu bar we'd like to give to ALOE/OpenDoc isn't present.");
#endif

	SetBaseMenuBar(kMBarOpenDoc);
	
	// 5. Register menu commands
	// Apple menu
	MenuHandle theAppleMenuHandle = ::GetMenu(mODMENUApple);
	::ThrowIfNULL_AC(theAppleMenuHandle);
	short theAppleMenuID = ((MenuInfo)**theAppleMenuHandle).menuID;
	RegisterMenuCommand(theAppleMenuID, mODMENUApple, kODCommandAppleMenu);
	RegisterMenuCommand(theAppleMenuID, 1, kODCommandAbout);
	
	// File menu
	MenuHandle theFileMenuHandle = ::GetMenu(mODMENUDocument);
	::ThrowIfNULL_AC(theFileMenuHandle);
	short theFileMenuID = ((MenuInfo)**theFileMenuHandle).menuID;
	RegisterMenuCommand(theFileMenuID, theFileMenuID, kODCommandDocumentMenu);
	RegisterMenuCommand(theFileMenuID, 1, kODCommandNew);
	RegisterMenuCommand(theFileMenuID, 2, kODCommandOpen);
	RegisterMenuCommand(theFileMenuID, 3, kODCommandOpenDocument); // ??? Oui?
	RegisterMenuCommand(theFileMenuID, 4, kODCommandInsert);
	RegisterMenuCommand(theFileMenuID, 5, kODCommandClose);
	// ---
	RegisterMenuCommand(theFileMenuID, 7, kODCommandSave);
	RegisterMenuCommand(theFileMenuID, 8, kODCommandSaveACopy);
	RegisterMenuCommand(theFileMenuID, 9, kODCommandRevert);
	RegisterMenuCommand(theFileMenuID, 10, kODCommandDocumentInfo);
	// ---
	RegisterMenuCommand(theFileMenuID, 12, kODCommandPageSetup);
	RegisterMenuCommand(theFileMenuID, 13, kODCommandPrint);
	// ---
	RegisterMenuCommand(theFileMenuID, 15, cQuit); // <- register MacApp Quit command
	
	// Edit menu
	MenuHandle theEditMenuHandle = ::GetMenu(mODMENUEdit);
	::ThrowIfNULL_AC(theEditMenuHandle);
	short theEditMenuID = ((MenuInfo)**theEditMenuHandle).menuID;
	
	RegisterMenuCommand(theEditMenuID, mODMENUEdit, kODCommandEditMenu);
	RegisterMenuCommand(theEditMenuID, 1, kODCommandUndo);
	RegisterMenuCommand(theEditMenuID, 2, kODCommandRedo);
	// ---
	RegisterMenuCommand(theEditMenuID, 4, kODCommandCut);
	RegisterMenuCommand(theEditMenuID, 5, kODCommandCopy);
	RegisterMenuCommand(theEditMenuID, 6, kODCommandPaste);
	RegisterMenuCommand(theEditMenuID, 7, kODCommandPasteAs);
	RegisterMenuCommand(theEditMenuID, 8, kODCommandClear);
	RegisterMenuCommand(theEditMenuID, 9, kODCommandSelectAll);
	// ---
	RegisterMenuCommand(theEditMenuID, 11, kODCommandGetPartInfo);
	RegisterMenuCommand(theEditMenuID, 12, kODCommandPreferences);
	// ---
	RegisterMenuCommand(theEditMenuID, 14, kODCommandViewAsWin);

#if 0
// Menu bar sharing algorithm
// !!! ALOE does not read in CMNUs correctly.  The auto-CommandNumber registration fails,
// and when it's disabled (in ALOE), the CMNU menus appear corrupted as soon as they're copied

	SetBaseMenuBar(kMBarDisplayed);
	
	// Register MacApp menu commands as OpenDoc menu commands so ALOE can 
	// dispatch from them when a part is active and has the menu focus.
	// Some commands are redundant because MacApp defines them specifically for ALOE.
	RegisterCommandNumber(cAboutApp, kODCommandAbout);
	
	// File menu
	RegisterCommandNumber(cNew, kODCommandNew);
	RegisterCommandNumber(kODCommandOpen, kODCommandOpen);
	RegisterCommandNumber(cOpen, kODCommandOpenDocument); // ??? Oui?
	RegisterCommandNumber(kODCommandInsert, kODCommandInsert);
	RegisterCommandNumber(cClose, kODCommandClose);
	RegisterCommandNumber(cSave, kODCommandSave);
	RegisterCommandNumber(cSaveCopy, kODCommandSaveACopy);
	RegisterCommandNumber(cRevert, kODCommandRevert);
	//RegisterCommandNumber(kODCommandDraft, kODCommandDraft); ??? not available in a container app?
	RegisterCommandNumber(kODCommandDocumentInfo, kODCommandDocumentInfo);
	RegisterCommandNumber(cPageSetup,kODCommandPageSetup);
	RegisterCommandNumber(cPrint, kODCommandPrint);
	
	// Edit menu
	RegisterCommandNumber(cUndo, kODCommandUndo);
	RegisterCommandNumber(cRedo, kODCommandRedo);
	RegisterCommandNumber(cCut, kODCommandCut);
	RegisterCommandNumber(cCopy, kODCommandCopy);
	RegisterCommandNumber(cPaste, kODCommandPaste);
	RegisterCommandNumber(kODCommandPasteAs, kODCommandPasteAs);
	RegisterCommandNumber(cClear, kODCommandClear);
	RegisterCommandNumber(cSelectAll, kODCommandSelectAll);
	RegisterCommandNumber(kODCommandGetPartInfo, kODCommandGetPartInfo);
	RegisterCommandNumber(kODCommandPreferences, kODCommandPreferences);
	RegisterCommandNumber(kODCommandViewAsWin, kODCommandViewAsWin);
#endif // shared menu algorithm
}

//----------------------------------------------------------------------------------------
// Save current menu bar before letting ALOE take over
//----------------------------------------------------------------------------------------
void TODSession::SaveMenuBar()
{
	fSavedMenuBar = ::GetMenuBar();
}
	
	
//----------------------------------------------------------------------------------------
// Restore saved menu bar when ALOE tells us it's okay
//----------------------------------------------------------------------------------------

void TODSession::RestoreMenuBar()
{
	::ClearMenuBar();
	::SetMenuBar(fSavedMenuBar);
	::InvalMenuBar();

	// ??? Tell MacApp to setup its menu bar?
	//InvalidateMenus();
	//gDispatcher->SetupTheMenus();
}
	

//----------------------------------------------------------------------------------------
// add a call to SetWindowActionCallback
//----------------------------------------------------------------------------------------

void TODSession::SetCallbacks()
{
	inherited::SetCallbacks();
	
	try 
	{
		SetWindowActionCallback();
	}
	catch (CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		DisposeCallbacks();
	}
}

//----------------------------------------------------------------------------------------
// swap menu bars if activating application
//----------------------------------------------------------------------------------------

void TODSession::DoActivateApp(bool activate)
{
	if (activate)
		RestoreMenuBar();
	//else (if !activate && ApplicationHasFocus())	
	//	gApplication->InvalidateFocus();
	
	// call inherited to get default behavior
	inherited::DoActivateApp(activate);
}

//----------------------------------------------------------------------------------------
// handle callback from ALOE to restore our menu bar
//----------------------------------------------------------------------------------------

void TODSession::DoResetMenuBar(WindowRef /*window*/)
{
	RestoreMenuBar();	
}	

//----------------------------------------------------------------------------------------
// handle callback from ALOE to do window action
//----------------------------------------------------------------------------------------


void TODSession::DoWindowAction(WindowRef window, ALOEWindowAction action, bool /*param*/)
{
	// handle callback from ALOE to perform window action
	
	TWindow* aWindow = TWindow::WMgrToWindow(window);
	
	switch (action) {
		case kALOEAddWindow:			   
		case kALOERemoveWindow:
			// ??? Not sure what is expected here,
			break;

		case kALOEShowWindow:
			// !!! Need to invalidate focus or else Deactivate will blow chunks
			gDispatcher->InvalidateFocus();
			
			if (aWindow)
				aWindow->Show(true, kRedraw);
			else 
				TWindow::MAShowWindow(window);
				
			break;

		case kALOEHideWindow:
			if (aWindow)
				aWindow->Show(false, kRedraw);
			else 
				TWindow::MAHideWindow(window);
			break;

		case kALOESelectWindow:			
			if (aWindow)
				aWindow->Select();
			else
				TWindow::MASelectToolboxWindow(window);
			break;

		default:
#if qCoreDebug
			DebugStr("\pBad selector in ALOE Window callback - DoWindowAction.");
#endif // qCoreDebug
			break;
	}
}


//----------------------------------------------------------------------------------------
// Register MacApp CommandNumber as OpenDoc command number
//----------------------------------------------------------------------------------------

void TODSession::RegisterCommandNumber(CommandNumber maCommand, long odCommand)
{
	MenuID menuNo;
	short itemNo;

	CommandToComponents(maCommand, menuNo, itemNo);
	if (menuNo) // was found
		RegisterMenuCommand(menuNo, itemNo, odCommand);
#if qDebug
	else
		ProgramBreak_AC("TODSession::RegisterCommandNumber: %1d not found in the default menu bar (check your menus)", maCommand);
#endif
}

//----------------------------------------------------------------------------------------
// Return the frontmost window's corresponding adaptor mgr
//----------------------------------------------------------------------------------------

CODAdaptorMgr_AC* TODSession::GetFrontAdaptorMgr()
{
 	CODAdaptorMgr_AC* manager = NULL;
 	
 	// MacApp keeps its adaptor mgrs hidden behind a mixin class.
 	// The mixin should be the current window's context.
 	TWindow* theWindow = gDispatcher->GetFrontWindow();
 	if (theWindow)
 	{
 		TCommandHandler* context = theWindow->GetContext(cNoCommand);
 		if (context)
 		{
	 		MODAdaptable* adaptable = dynamic_cast<MODAdaptable*>(context);
			if (adaptable)
	 			manager = adaptable->GetAdaptorManager();
 		}
 	}
 	return manager;
}

//----------------------------------------------------------------------------------------
// End of UODSession.cp

#endif // qALOE



