// UApplication.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UAPPLICATION__
#define __UAPPLICATION__

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __USCRIPTABLEOBJECT__
#include "UScriptableObject.h"
#endif

// Toolbox

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

// ALOE support

#if qALOE

#ifndef __UODDispatcher__
#include "UODDispatcher.h"
#endif

#endif // qALOE


//----------------------------------------------------------------------------------------
// Performance monitor menu command constants used by TApplication.
//----------------------------------------------------------------------------------------
enum
{
	cPerfMonInit = 901,
	cPerfMonDump = 902,
	cPerfMonToggle = 903,
	cPerfMonEnd = 904
};

//----------------------------------------------------------------------------------------
// Debugger menu command constants used by TApplication.
//----------------------------------------------------------------------------------------
enum
{
	cDebugFlags = 913,
	cDoFirstClick = 915,			// <!> Toggle. "'Do First Click' for this
	cRefreshFrontWindow = 917,		// <!> Action. "Refresh Front Window"
	cSetSysJust = 921				// <!> Action. "Switch system justification"
};

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------
struct StandardFileReply;
class TDocument;
class TFile;
class TView;
class TWindow;
class TEvent;
class TToolboxEvent;

//----------------------------------------------------------------------------------------
// TApplication: The purpose of this class is to implement the main event loop that all
// Macintosh applications must have. There is only one instance of TApplication. One thing
// that TApplication does is to interpret the raw events that are posted and convert them
// into higher-level events. For example, it converts a click in the menu bar into a 'menu
// event' that contains the menu ID/ item number that was chosen by the user.
//----------------------------------------------------------------------------------------
#if qALOE
class EXPORT_MA TApplication : public TODDispatcher
#else
class EXPORT_MA TApplication : public TDispatcher
#endif
{
public:
	//------------------------------------------------------------------------------------
	// Construction and destruction
	//------------------------------------------------------------------------------------
	TApplication(OSType itsMainFileType = kTextType_AC,
									 OSType itsCreator = kAllQuestionsType_AC);
		// itsMainFileType should be the 4-byte file type that this application reads and
		// writes.  itsCreator is the application's signiture.  They are used when creating
		// a default file with DoMakeFile and to display the proper file types in the
		// StandardFile dialogs.

	virtual ~TApplication();
		// Destructor

	virtual bool ChooseApplication(AEAddressDesc& theAddress);
		// Choose a running application using the PPCBrowser and store the address in an
        // AppleEvent address

#if qNeedsVU
	virtual void DoInitialState();
		// Overrides TDispatcher method to call DoMakeVUAssist.
		
	virtual void DoMakeVUAssist();
		// By default, creates a TVUAssist instance.  Override to create a subclass.
#endif

	//------------------------------------------------------------------------------------
	// Finder Requests
	//------------------------------------------------------------------------------------
	virtual bool CanOpenDocument(CommandNumber itsCommandNumber, TFile* aFile);
		// Simulates the filtering done by Std File; this is only called when opening/
        // printing documents from the finder; when using Std File, Std File does the
        // filtering.

	virtual void HandleFinderRequest();
		// If the AppleEvents Manager is not required or present: Gets the info from the
        // finder about what files to open/ print and opens/ prints them.
	

	//------------------------------------------------------------------------------------
	// Opening and Printing Documents
	//------------------------------------------------------------------------------------
	virtual bool ChooseDocument(CommandNumber itsCommandNumber, CList_AC** aFileList);
		// Call this to make a Std File Get call; returns true is user selected a file.

	virtual TDocument* DoMakeDocument(CommandNumber itsCommandNumber, TFile* itsFile);
		// Must be overridden. Based on itsCommandNumber create a document object of the
        // appropriate kind. If itsFile <> NULL it is attached to the document

	virtual CommandNumber KindOfDocument(CommandNumber itsCommandNumber, TFile* itsFile);
		// Given a command number and a specification of the file, return the command number to
        // pass to DoMakeDocument that indicates the type of document to make; Default is
        // to return itsCommandNumber. If you have multiple document types, a good convention
        // is to return the command numbers assigned to create new documents of each kind.
        // aFile will be NULL if we are creating a brand new document, rather than opening
        // an existing document.

	virtual TDocument* OpenNew(CommandNumber itsCommandNumber);
		// Called when the application is opened (itsCommandNumber = cFinderNew), or when NEW
        // is chosen from menu (itsCommandNumber = cNew). If you do not want to create a new
        // document when the user opens the application, override this and do nothing if
        // itsCommandNumber = cFinderNew. Returns the document that was opened. 

	virtual TDocument* OpenOld(CommandNumber itsOpenCommand, CList_AC* aFileList);
		// Called when opening an existing document from finder (itsCommandNumber =
        // cFinderOpen) or if OPEN is chosen from menu (itsCommandNumber = cOpen).
		// It can also handle stationery.

	virtual void PrintDocuments(CommandNumber itsCommandNumber, CList_AC* aFileList,
								const CFSSpec_AC* targetPrinter = NULL);
		// Called to print the documents specified from the finder.  The list of files may
		// have come from a 'pdoc' AppleEvent or from HandleFinderRequest
		// If targetPrinter is non-nil it specifies a target printer for the print job

	virtual void GetStandardFileParameters(CommandNumber itsCommandNumber,
											 FileFilterYDProcPtr& fileFilter,
											 TypeListHandle& typeList,
											 short& dlgID,
											 CPoint_AC& where,
											 DlgHookYDProcPtr& dlgHook,
											 ModalFilterYDProcPtr& modalFilter,
											 short*& activeList,
											 ActivateYDProcPtr& activateProc,
											 StandardFileReply* aReply,
											 void*& yourDataPtr);
		// Called to return all the parameters that should be passed to SFPGetFile or
        // CustomGetFile, and CanOpenDocument; Defaults to:
		// 		dlgID = getDlgID or sfGetDialogID;
		// 		where = (100, 100) or (-1, -1);
		// 		fileFilter = NULL;
		// 		dlgHook = NULL;
		// 		modalFilter = gStandardFileFilter;
		// 		activeList := NULL;
		// 		activateProc := NULL;
		// 		yourDataPtr := NULL;

	virtual void GetFileTypeList(CommandNumber itsCommandNumber,
										TypeListHandle& typeList);
		// Return an valid handle to a list of file types supported by your application.
		// and the number of types in the list. The default is to allocate enough space
		// for fMainFileType.  Called by ChooseDocument to determine the types of files
		// to display.  If no type is specified then all types are displayed.


	//------------------------------------------------------------------------------------
	// Command Handlers
	//------------------------------------------------------------------------------------
	virtual void DoMenuCommand(CommandNumber aCommandNumber);
		// Handles the application specific menu commands.

	virtual void DoSetupMenus();

	virtual void DoNew(CommandNumber theCmdNumber);
	// Handles the New menu item.

	virtual void DoOpen(CommandNumber theCmdNumber);
	// Handles the Open menu item.

	//------------------------------------------------------------------------------------
	// AppleEvent Handling
	//------------------------------------------------------------------------------------
	virtual void DoAEClose(TAppleEvent* message,
						   TAppleEvent* reply);
	// Handles the close event for the application.

	virtual void DoScriptCommand(CommandNumber aCommandNumber,
								TAppleEvent* message,
								TAppleEvent* reply);
	// Handles object model AppleEvents for the application.

	virtual void DoAECreateElement(TAppleEvent* message,
								   TAppleEvent* reply);
	// Creates a new document. If your application has more than one type of document
	// you'll need to override this method. The default simulates choosing "New" from
	// the file menu.

	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------
	virtual void DoAboutBox();
		// Called to show the "About the App" information

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	bool fLaunchWithNewDocument;				// True (the default) to create a new
                                                // untitled document when the application
                                                // is launched without a document.
	bool fUseQuickTimePreview;					// true (the default) to use QT getfile previews 
#if qDebug
	CAutoPtr_AC<TWindow> fDebugFlagsWindow;					// Window for setting the debug flags
#endif
};

//----------------------------------------------------------------------------------------
// TNewDocumentCommand: Tells the application open a new document.
//----------------------------------------------------------------------------------------
class EXPORT_MA TNewDocumentCommand : public TCommand
{
public:
	TNewDocumentCommand(CommandNumber itsCommandNumber = cNoCommand);
		// constructor

	virtual ~TNewDocumentCommand();
		// Destructor
		
	virtual void DoIt();
		// tell the application open a new document.
		
	virtual TAppleEvent* MakeAppleEvent();
};

//----------------------------------------------------------------------------------------
// TFilesCommand: Retreives a list of files from an AppleEvent Abstract class for
// TODocCommand and TPDocCommand.
//----------------------------------------------------------------------------------------
class EXPORT_MA TFilesCommand : public TServerCommand
{
public:
	TFilesCommand(CommandNumber itsCommandNumber = cNoCommand, CList_AC* theDocuments = NULL);
		// Constructor

	TFilesCommand(CommandNumber itsCommandNumber,
					   const AppleEvent& itsMessage,
					   const AppleEvent& itsReply);
		// Construct the command from the information contained in the Apple Event.
		
	virtual ~TFilesCommand();
		// Free the list of file alias handles.

	void GetFilesList();
		// Get the list of files from the Apple Event.

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	CAutoPtr_AC<CList_AC> fFileList;
};


//----------------------------------------------------------------------------------------
// TODocCommand: Accepts a list of file aliases and opens each of them
//----------------------------------------------------------------------------------------
class EXPORT_MA TODocCommand : public TFilesCommand
{
public:
	TODocCommand(CommandNumber itsCommandNumber = cNoCommand, CList_AC* theDocuments = NULL) :
		TFilesCommand(itsCommandNumber, theDocuments) { }
		// constructor.
		
	TODocCommand(CommandNumber itsCommandNumber,
					  const AppleEvent& itsMessage,
					  const AppleEvent& itsReply) :
		TFilesCommand(itsCommandNumber, itsMessage, itsReply) { }
		// construct the command from the information contained in the Apple Event.

	virtual ~TODocCommand();
		// Destructor
		
	virtual void DoIt();
		// Open each file found in fFileList.

	virtual TAppleEvent* MakeAppleEvent(); // Override
};


//----------------------------------------------------------------------------------------
// TPDocCommand: Accepts a list of file aliases and prints each of them
//----------------------------------------------------------------------------------------
class EXPORT_MA TPDocCommand : public TFilesCommand
{
public:
	TPDocCommand(CommandNumber itsCommandNumber = cNoCommand, CList_AC* theDocuments = NULL);
		// Construct the PDocCommand procedurally.

	TPDocCommand(CommandNumber itsCommandNumber,
					  const AppleEvent& itsMessage,
					  const AppleEvent& itsReply);
		// Construct the command from the information contained in the Apple Event.

	virtual ~TPDocCommand();
		// Destructor
				
	virtual void DoIt();
		// Print each file found in fFileList.

protected:
	CFSSpec_AC fTargetPrinter;			// The target printer if this is a "drag print"
};


//----------------------------------------------------------------------------------------
// TQuitCommand: Tells the application to do its pre-quitting stuff.
//----------------------------------------------------------------------------------------
class EXPORT_MA TQuitCommand : public TCommand
{
public:
	TQuitCommand(CommandNumber itsCommandNumber = cNoCommand);
		// constructor

	TQuitCommand(TAppleEvent* message,
					  TAppleEvent* reply);
		// constructor

	virtual ~TQuitCommand();
		// Destructor
	
	virtual bool IsReadyToExecute();	// Override
		// Don't let application Quit if there's a modal window open

	virtual void DoIt();
		// Tell the application to quit.
		
	void CloseAWindow(TWindow* aWindow);
	void CloseADocument(TDocument* aDocument);
	void DoCloseInWindowOrder();
	void DoCloseWindowlessDocuments();
	void DoCloseApplication();
		
protected:
	bool			fRespondingToAE;
	bool			fRecord;
	TAppleEvent*	fMessage;
	TAppleEvent*	fReply;
};


//----------------------------------------------------------------------------------------
// TQuitAppCommand: Tells the application to quit in a recordable manner.
//	 Don't issue a TQuitAppCommand to quit the application--issue a
//	 TQuitCommand instead.
//----------------------------------------------------------------------------------------
class EXPORT_MA TQuitAppCommand : public TCommand
{
public:
	TQuitAppCommand(CommandNumber itsCommandNumber = cNoCommand);
		// constructor

	virtual ~TQuitAppCommand();
		// Destructor
		
	virtual void DoIt();
		// tell the application to quit.
		
	virtual TAppleEvent* MakeAppleEvent(); // Override

	virtual bool IsReadyToExecute();	// Override
		// Don't let application Quit if there's a modal window open
};


//----------------------------------------------------------------------------------------
// TAboutBoxCommand: Tells the application to show its about box.
//----------------------------------------------------------------------------------------
class EXPORT_MA TAboutBoxCommand : public TCommand
{
public:
	TAboutBoxCommand(CommandNumber itsCommandNumber = cNoCommand);
		// constructor

	virtual ~TAboutBoxCommand();
		// Destructor
		
	virtual void DoIt();
		// tell the application to DoAboutBox.
};

//----------------------------------------------------------------------------------------
// Globals defined by this unit
//----------------------------------------------------------------------------------------
EXPORT_MA extern TApplication* gApplication;		// the application object

#endif


