// UContextMenuBehavior.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.


#ifndef __UCONTEXTMENUBEHAVIOR__
#define __UCONTEXTMENUBEHAVIOR__

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UView__
#include "UView.h"
#endif


EXPORT_MA extern void InitUContextMenuBehavior();

//----------------------------------------------------------------------------------------
// TContextMenuBehavior
// Attach this behavior to your application to get contextual menu support.
//----------------------------------------------------------------------------------------

class EXPORT_MA TContextMenuBehavior : public TBehavior
{
	MA_DECLARE_CLASS;
	
public:
	TContextMenuBehavior(IDType itsIdentifier = kNoIdentifier);
		// Constructor

	TContextMenuBehavior(const TBehavior& inBehavior);
		// Copy Constructor
		
	virtual ~TContextMenuBehavior();

	//------------------------------------------------------------------------------------
	//  Event Filtering Methods
	//------------------------------------------------------------------------------------

	//  Special hooks which are used when attached to a TApplication

	virtual bool DoToolboxEvent(TToolboxEvent* event);
		// When this is called by MacApp, theEvent is the event to be processed.
		// If the event is a contextual menu event, handle contextual menu is called to
		// get the target view set, and to show the contextual menu.
	
	void DispatchContextualMenuEvent(TToolboxEvent* event);
		// Handles the dispatch of contextual menu event.
		
	void HandleContextualMenu(TView* thisView, const CViewPoint& theMouse, MenuRef& theMenuRef, CAEDesc& theSelection);
		// Iterates recursively over subviews to set the correct target, its contextual menu, and selection descriptor.

	static bool HasContextualMenus();
		// Returns whether contextual menus are available (system-wide)
};

#endif
