//----------------------------------------------------------------------------------------
// UApplication.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

// ACS

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#if qDebug
#ifndef __UDEBUGVIEW__
#include "UDebugView.h"
#endif

#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __FLOATWINDOW__
#include "FloatWindow.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPERFORM__
#include "UPerform.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __UOSASCRIPT__
#include "UOSAScript.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USECTIONMGR__
#include "USectionMgr.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UTABBEHAVIORS__
#include "UTabBehaviors.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#if qNeedsVU
#ifndef __UVUASSIST__
#include "UVUAssist.h"
#endif
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __DISKINIT__
#include <DiskInit.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __IMAGECOMPRESSION__
#include <ImageCompression.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __TRANSLATION__
#include <Translation.h>
#endif

// For TargetIDHdl
#ifndef __EPPC__
#include <EPPC.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

// Other

#if defined(__MWERKS__) && (qDebug || qTheDebugger)
#ifndef __CWDEBUG__
#include "CWDebug.h"
#endif	

#endif

//----------------------------------------------------------------------------------------
// type definitions
//----------------------------------------------------------------------------------------
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
struct RSRCopen		// the 'open' resource format for the translation manager.
{
	OSType	signature;
	short	filler;
	short	numTypes;
	OSType	types[1];
};
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif
typedef RSRCopen *RSRCopenPtr, **RSRCopenHdl;

//----------------------------------------------------------------------------------------
// Constant definitions
//----------------------------------------------------------------------------------------

enum SystemJustification
{
	smSysJustLeft = 0, smSysJustRight = -1
};
// Constants to use with GetSysDirection() and SetSysDirection().


//----------------------------------------------------------------------------------------
// Global variable definitions.
//----------------------------------------------------------------------------------------

TApplication* gApplication;
//bool gInitialized;
//AEAddressDesc gServerAddress;


bool hadCreditsStringList;					// does the rsrc 'STR#' == kDefaultCredits exist ?
short lastCreditsStringIndex;					// the last CPascalStr_AC in the STR# to be displayed
long lastCreditsShownTicks;						// the tickcount when the last Credit was Shown
CPascalStrHandle_AC originalText;						// the about box's original text (prior to credits)
short waitTicks;								// how long to wait between credits 

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static pascal Boolean DoShowAboutAppFilter(DialogRef theDialog,
										   EventRecord* theEvent,
										   short* itemHit);
static void DoSetupTheMenus(void* yourDataPtr);

static WindowRef GetWindowToClose();

//----------------------------------------------------------------------------------------
// DoShowAboutAppFilter: 
//----------------------------------------------------------------------------------------

pascal Boolean DoShowAboutAppFilter(DialogRef theDialog,
									EventRecord* theEvent,
									short* itemHit)
{
	
	CStr255_AC originalStr;
	Handle item;
	bool returnValue = false;
	short itemType;
	CRect_AC box;

	switch (theEvent->what)
	{
		case keyDown:
			switch (static_cast<unsigned char>(theEvent->message & charCodeMask))
			{
				case chEnter:
				case chReturn:
					DoAlertKeyDown(theDialog, ok);
					break;
			}
			break;

		case nullEvent:
			if ((TickCount() - lastCreditsShownTicks) > waitTicks)
			{
				short itemNo = 1;

				do
				{
					item = NULL;
					GetDialogItem(theDialog, itemNo, &itemType, &item, box);
					if (((itemType) & 0x7F) == statText)// we don't care if its enabled or not 
						break;
					else
						++itemNo;
				} while (item);

				CStr255_AC s(kDefaultCredits, lastCreditsStringIndex);

				if (!s.IsEmpty())
				{
					// save the original text 
					if ((lastCreditsStringIndex == 1) && ((*originalText)->IsEmpty() && item))
					{
						GetDialogItemText(item, originalStr);
						SetString((StringHandle)originalText, (ConstStr255Param) & originalStr);
					}
					++lastCreditsStringIndex;
					lastCreditsShownTicks = TickCount();
					if (item)
						SetDialogItemText(item, s);
					waitTicks = Min_AC(static_cast<short>(s.Length() * 6), static_cast<short>(60));
				}
				else							// no more items 
					{
						lastCreditsStringIndex = 1;
						lastCreditsShownTicks = TickCount();
						if (item)
						{
							originalStr = **originalText;
							SetDialogItemText(item, originalStr);
						}
						waitTicks = 6 * 60;
					}
			}
			break;
	}

	// Forward on to the standard filter 
	if (gMacAppAlertFilter)
		returnValue = gMacAppAlertFilter(theDialog, theEvent, itemHit);

	return returnValue;
}

//========================================================================================
// CLASS TQuitCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TQuitCommand::TQuitCommand: 
//----------------------------------------------------------------------------------------

TQuitCommand::TQuitCommand(CommandNumber itsCommandNumber) :
	TCommand(itsCommandNumber, gApplication, kCantUndo, kDoesNotCauseChange, NULL),

	fRespondingToAE(false),
	fRecord(true),
	fMessage(NULL),
	fReply(NULL)
{
	++fPriority;								// Lower our priority.
}

//----------------------------------------------------------------------------------------
// TQuitCommand::TQuitCommand: 
//----------------------------------------------------------------------------------------

TQuitCommand::TQuitCommand(TAppleEvent* message,
								TAppleEvent* reply) :
	TCommand(cQuit, gApplication, kCantUndo, kDoesNotCauseChange, NULL),

	fRespondingToAE(true),
	fRecord(false),
	fMessage(message),
	fReply(reply)
{
	++fPriority;								// Lower our priority.
}

//----------------------------------------------------------------------------------------
// TQuitCommand destructor
//----------------------------------------------------------------------------------------

TQuitCommand::~TQuitCommand()
{
}

//----------------------------------------------------------------------------------------
// TQuitCommand::IsReadyToExecute: 
//----------------------------------------------------------------------------------------

bool TQuitCommand::IsReadyToExecute()		// Override
{
	// Don't execute Quit if application has a dialog posed modally!
	return gApplication->InModalState() ? false : fReadyToExecute;
}

//----------------------------------------------------------------------------------------
// TQuitCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TQuitCommand::DoIt()
{
	try
	{
		// Anyone who wants to know we're shutting down can ask.
		gApplication->SetDone(true);
		
		// Close all windows and documents in window order then close
		// all documentless windows.  Finally, shut down the application.
		DoCloseInWindowOrder();
		DoCloseWindowlessDocuments();
		DoCloseApplication();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		gApplication->SetDone(false);	// We're no longer shutting down.

		if (fRespondingToAE)
			throw;
		else if (theException.GetError() != userCanceledErr)
			gApplication->ShowError(theException.GetError(), 0);
	}
}

//----------------------------------------------------------------------------------------
// GetWindowToClose: 
//----------------------------------------------------------------------------------------

WindowRef GetWindowToClose()
{
	WindowRef returnVal = NULL;

	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(iter);
		if (aWindow && (aWindow->GetFreeOnClosing() || aWindow->IsShown()))
		{
			returnVal = iter;
			break;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TQuitCommand::DoCloseInWindowOrder: 
//----------------------------------------------------------------------------------------

void TQuitCommand::DoCloseInWindowOrder()
{
	// This will iterate through all open windows and will close the associated
	// document if there is one or will close the window if there is no document.
	// (Ghost documents are not closed here.  But their windows are.)
	WindowRef aWindowPtr;
	while ((aWindowPtr = GetWindowToClose()) != NULL)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(aWindowPtr);
		if (aWindow)
		{
			TDocument * windowsDocument = aWindow->GetDocument();
			if (windowsDocument && !windowsDocument->GetIsGhostDocument())
				CloseADocument(windowsDocument);
			else
				CloseAWindow(aWindow);
		}
		else
			gApplication->CloseToolboxWindow(aWindowPtr);
	}
}

//----------------------------------------------------------------------------------------
// TQuitCommand::DoCloseWindowlessDocuments: 
//----------------------------------------------------------------------------------------

void TQuitCommand::DoCloseWindowlessDocuments()
{
	// This really doesn't care if the document is windowless or not, it's
	// just that all documents with windows should have been closed by now.
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	for (CNoGhostDocsIterator iter(gApplication); iter.Current(); ++iter)
		CloseADocument(iter);
}

//----------------------------------------------------------------------------------------
// TQuitCommand::CloseAWindow: 
//----------------------------------------------------------------------------------------

void TQuitCommand::CloseAWindow(TWindow* aWindow)
{
	bool record = !fRespondingToAE && (gClipboardMgr->fClipWindow != aWindow)
					 && aWindow->IsShown() && fRecord;

	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	CAutoPtr_AC<TCloseWindowCommand> aCloseWindowCommand = TH_new TCloseWindowCommand(cClose, aWindow);

	if (!record)
		aCloseWindowCommand->SetUseAppleEvent(false);

	aCloseWindowCommand->Process();
	aCloseWindowCommand.release();
}

//----------------------------------------------------------------------------------------
// TQuitCommand::DoCloseDocuments: 
//----------------------------------------------------------------------------------------

void TQuitCommand::CloseADocument(TDocument* aDocument)
{
	// Closing the document will close any associated windows.
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	TCloseDocCommand*		aCloseCommand = aDocument->MakeCloseCommand();

	if (fRespondingToAE)
		aCloseCommand->SetUseAppleEvent(false);

	aCloseCommand->Process();
}

//----------------------------------------------------------------------------------------
// TQuitCommand::DoCloseApplication: 
//----------------------------------------------------------------------------------------

void TQuitCommand::DoCloseApplication()
{
	// Final Stage: We're ready to do the final shutdown now that 
	// all windows and documents have been closed.
	CTempAllocation tempAllocation(true);
	CTempObjectAllocation tempObjectAllocation(true);
	
	CAutoPtr_AC<TQuitAppCommand> aQuitAppCommand = TH_new TQuitAppCommand(cQuit);

	if (!fRecord || fRespondingToAE)
		aQuitAppCommand->SetUseAppleEvent(false);

	aQuitAppCommand->Process();
	aQuitAppCommand.release();
}

//========================================================================================
// CLASS TQuitAppCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TQuitAppCommand::TQuitAppCommand: 
//----------------------------------------------------------------------------------------

TQuitAppCommand::TQuitAppCommand(CommandNumber itsCommandNumber) :
	TCommand(itsCommandNumber, gApplication, kCantUndo, kDoesNotCauseChange, NULL)
{
	fUseAppleEvent = true;		// The default will be to record.
}

//----------------------------------------------------------------------------------------
// TQuitAppCommand destructor
//----------------------------------------------------------------------------------------

TQuitAppCommand::~TQuitAppCommand()
{
}

//----------------------------------------------------------------------------------------
// TQuitAppCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------

TAppleEvent* TQuitAppCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kCoreEventClass, kAEQuitApplication, gServerAddress, kAEWaitReply));

	CAEDesc mySpecifier;
	gApplication->MakeObjectSpecifier(mySpecifier, formName);
	theEvent->WriteParameter(keyDirectObject, mySpecifier);

	return theEvent.release();
}

//----------------------------------------------------------------------------------------
// TQuitAppCommand::IsReadyToExecute: 
//----------------------------------------------------------------------------------------

bool TQuitAppCommand::IsReadyToExecute()		// Override
{
	// Don't execute Quit if application has a dialog posed modally!
	return gApplication->InModalState() ? false : fReadyToExecute;
}

//----------------------------------------------------------------------------------------
// TQuitAppCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TQuitAppCommand::DoIt()
{
	try
	{
		
		gApplication->Close();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		
		throw;
	}
	
}

//========================================================================================
// CLASS TNewDocumentCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TNewDocumentCommand::TNewDocumentCommand: 
//----------------------------------------------------------------------------------------

TNewDocumentCommand::TNewDocumentCommand(CommandNumber itsCommandNumber) :
	TCommand(itsCommandNumber, gApplication, kCantUndo, kDoesNotCauseChange, NULL)
{
}

//----------------------------------------------------------------------------------------
// TNewDocumentCommand destructor
//----------------------------------------------------------------------------------------

TNewDocumentCommand::~TNewDocumentCommand()
{
}

//----------------------------------------------------------------------------------------
// TNewDocumentCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TNewDocumentCommand::DoIt()
{
	gApplication->OpenNew(fIdentifier);
}

//----------------------------------------------------------------------------------------
// TNewDocumentCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------

TAppleEvent* TNewDocumentCommand::MakeAppleEvent()
{
	// Create a recordable apple event
	CTempDesc theAppDesc;
	gApplication->MakeObjectSpecifier(theAppDesc);

	return TAppleEvent::MakeCreateElementEvent(gServerAddress, kAEWaitReply, theAppDesc, cDocument, kAEBeginning, CAEDesc::fgNullDesc, CAEDesc::fgNullDesc);
}

//========================================================================================
// CLASS TFilesCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TFilesCommand constructor
//----------------------------------------------------------------------------------------

TFilesCommand::TFilesCommand(CommandNumber itsCommandNumber,
								  CList_AC* theDocuments) :
	TServerCommand(itsCommandNumber, gApplication, kCantUndo, kDoesNotCauseChange, NULL),
	fFileList(theDocuments)
{
	fCausesChange = false;
}

//----------------------------------------------------------------------------------------
// TFilesCommand constructor
//----------------------------------------------------------------------------------------

TFilesCommand::TFilesCommand(CommandNumber itsCommandNumber,
					   const AppleEvent& itsMessage,
					   const AppleEvent& itsReply) :
	TServerCommand(itsCommandNumber, gApplication, kCantUndo, kDoesNotCauseChange, NULL, itsMessage, itsReply),
	fFileList(NULL)
{
	fCausesChange = false;

	// Check to see what is in the direct object list
	GetFilesList();
}

//----------------------------------------------------------------------------------------
// TFilesCommand::Free: 
//----------------------------------------------------------------------------------------

TFilesCommand::~TFilesCommand()
{
	for (TemplateIterator_AC<TFile*> iter(fFileList); iter; ++iter)
		delete iter.Current();
}

//----------------------------------------------------------------------------------------
// TFilesCommand::GetFilesList: 
//----------------------------------------------------------------------------------------

void TFilesCommand::GetFilesList()
{
	// Check to see what is in the direct object list
	CAEDesc directObj;
	::ThrowIfOSErr_AC(AEGetParamDesc(fMessage, keyDirectObject, typeAEList, directObj));
	long theCount = 0;
	::ThrowIfOSErr_AC(AECountItems(directObj, &theCount));
	if (theCount > 0)
	{
		// Get the first descriptor and see what type it is
		DescType theKey;
		CTempDesc theDesc;
		::ThrowIfOSErr_AC(AEGetNthDesc(directObj, 1, typeWildCard, &theKey, theDesc));
	}

	CHandleList_AC aHandleList;

	CList_AC * aFileList = NULL;
	try
	{
		aFileList = NewList_AC();

		fMessage->ReadHandleList(keyDirectObject, typeAlias, &aHandleList);

		// Block is necessary for correct failure handling
		{
			for (CHandleIterator_AC iter(&aHandleList); iter; ++iter)
			{
				CAutoPtr_AC<TFile> aFile = gApplication->DoMakeFile(fIdentifier);
				::ThrowIfOSErr_AC(aFile->SpecifyWithAlias(AliasHandle(iter.Current())));

				aFileList->InsertLast(aFile);
				aFile.release();

				aHandleList.Delete(iter);// Delete it from the list so if we fail
				// we don't try to free it twice
				::DisposeIfHandle_AC(iter.Current());
			}
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		aHandleList.FreeAll();

		for (TemplateIterator_AC<TFile*> iter(aFileList); iter; ++iter)
			delete iter.Current();
		delete aFileList;

		throw;
	}

	fFileList = aFileList;
	directObj.DisposeDesc();
}

//========================================================================================
// CLASS TODocCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TODocCommand destructor
//----------------------------------------------------------------------------------------

TODocCommand::~TODocCommand()
{
}

//----------------------------------------------------------------------------------------
// TODocCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TODocCommand::DoIt()						// override 
{
	gApplication->OpenOld(fIdentifier, fFileList);
}

//----------------------------------------------------------------------------------------
// TODocCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------

TAppleEvent* TODocCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kCoreEventClass, kAEOpenDocuments, gServerAddress, kAEWaitReply));

	CTempDesc theList;
	theList.CreateList();

	for (TemplateIterator_AC<TFile*> iter(fFileList); iter; ++iter)
	{
		AliasHandle theAlias = NULL;
		OSErr theErr = iter->GetAlias(theAlias);
		if (theErr == noErr)
		{
			CAEDesc theAliasDesc(typeAlias, (Handle)theAlias);
			theList.PutListItem(0, theAliasDesc);
		}
	}

	theEvent->WriteParameter(keyDirectObject, theList);

	return theEvent.release();
}

//========================================================================================
// CLASS TPDocCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TPDocCommand constructor
//----------------------------------------------------------------------------------------

TPDocCommand::TPDocCommand(CommandNumber itsCommandNumber, CList_AC* theDocuments) :
	TFilesCommand(itsCommandNumber, theDocuments)
{
	fRequiresUserInteraction = true;
}

//----------------------------------------------------------------------------------------
// TPDocCommand constructor 
//----------------------------------------------------------------------------------------

TPDocCommand::TPDocCommand(CommandNumber itsCommandNumber,
								const AppleEvent& itsMessage,
								const AppleEvent& itsReply) :
	TFilesCommand(itsCommandNumber, itsMessage, itsReply)
{
	fRequiresUserInteraction = true;

	bool draggedToDTP = false;				// this will let us support cDragPrint

	AEDescList dtpList;
	OSErr err = AEGetAttributeDesc(&itsMessage, keyOptionalKeywordAttr, typeAEList, &dtpList);
	if (err == noErr)
	{
		draggedToDTP = true;

		AEKeyword theKeyword;
		DescType typeCode;
		Size actualSize;

		OSErr err = AEGetNthPtr(&dtpList, 1, typeFSS, &theKeyword, &typeCode, (Ptr) & fTargetPrinter, sizeof(CFSSpec_AC), &actualSize);
		AEDisposeDesc(&dtpList);
	}

	fIdentifier = draggedToDTP ? cDragPrint : fIdentifier;
}

//----------------------------------------------------------------------------------------
// TPDocCommand destructor
//----------------------------------------------------------------------------------------

TPDocCommand::~TPDocCommand()
{
}

//----------------------------------------------------------------------------------------
// TPDocCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TPDocCommand::DoIt()						// override 
{
	gApplication->PrintDocuments(fIdentifier, fFileList, fTargetPrinter.name[0] ? &fTargetPrinter : NULL);
}


//========================================================================================
// CLASS TAboutBoxCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TAboutBoxCommand::IAboutBoxCommand: 
//----------------------------------------------------------------------------------------

TAboutBoxCommand::TAboutBoxCommand(CommandNumber itsCommandNumber) :
	TCommand(itsCommandNumber, gApplication, kCantUndo, kDoesNotCauseChange, NULL)
{
}

//----------------------------------------------------------------------------------------
// TAboutBoxCommand destructor
//----------------------------------------------------------------------------------------

TAboutBoxCommand::~TAboutBoxCommand()
{
}

//----------------------------------------------------------------------------------------
// TAboutBoxCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TAboutBoxCommand::DoIt()
{
	::ThrowIfOSErr_AC(MAInteractWithUser());

	gApplication->DoAboutBox();
}

//========================================================================================
// CLASS TApplication
//========================================================================================
#undef Inherited

#if qALOE
#define Inherited TODDispatcher
#else
#define Inherited TDispatcher
#endif

//----------------------------------------------------------------------------------------
// TApplication::TApplication: 
//----------------------------------------------------------------------------------------

TApplication::TApplication(OSType itsMainFileType,
									 OSType itsCreator) :
	Inherited(itsMainFileType, itsCreator),

	fLaunchWithNewDocument(true),
	fUseQuickTimePreview(true)
#if qDebug
	,
	fDebugFlagsWindow(NULL)
#endif
{
	// This is a special case.  It needs to be available as soon as possible.
	gApplication = this;

	try
	{
		// Install the default tab  behavior 
		AddBehavior(TH_new TMultiWindowTabber);

		// Register views so we can create them from templates
#if qDebug
		InitUDebugView();
#endif
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

#if qDebug
		::ProgramBreak_AC("Can't initialize the application object!");
#endif
		gApplication = NULL;
		throw;
	}
}

//----------------------------------------------------------------------------------------
// TApplication destructor
//----------------------------------------------------------------------------------------

TApplication::~TApplication()
{
	gApplication = NULL;
}

#if qNeedsVU

//----------------------------------------------------------------------------------------
// TApplication::DoInitialState
//----------------------------------------------------------------------------------------
void TApplication::DoInitialState()
{
	Inherited::DoInitialState();
	
	DoMakeVUAssist();
}

//----------------------------------------------------------------------------------------
// TApplication DoMakeVUAssist
//----------------------------------------------------------------------------------------
void TApplication::DoMakeVUAssist()
{
	// The TVUAssist constructor will set the global reference
	if (!TVUAssist::fgVUAssist)
		TVUAssist* aVUAssist = TH_new TVUAssist(gUGridViewInitialized);
}

#endif // qNeedsVU

//----------------------------------------------------------------------------------------
// TApplication::CanOpenDocument: 
//----------------------------------------------------------------------------------------

// This is called only when opening/printing from the finder and from standard file;
// it simulates the filtering done by Std File.

bool TApplication::CanOpenDocument(CommandNumber itsCommandNumber,
									  TFile* aFile)
{
	bool returnValue = false;

	FileFilterYDProcPtr fileFilter;
	TypeListHandle typeList = NULL;
	short dlgID;
	CPoint_AC where;
	DlgHookYDProcPtr dlgHook;
	ModalFilterYDProcPtr modalFilter;
	short* activeList;
	ActivateYDProcPtr activateProc;
	void* yourDataPtr = NULL;

	GetStandardFileParameters(itsCommandNumber, fileFilter, typeList, dlgID, where, dlgHook, modalFilter, activeList, activateProc, NULL, yourDataPtr);

	CStr63_AC fileName;
	aFile->GetName(fileName);

	CInfoPBRec paramBlock;

	short numTypes = static_cast<short>(GetHandleSize((Handle)typeList) / sizeof(ResType));
	if (!numTypes)
	{
		if (!fileFilter)
			returnValue = true;					// no file filter then want all types
		else if (aFile->GetCatInfo(paramBlock) == noErr)
		{
			paramBlock.hFileInfo.ioNamePtr = (StringPtr)fileName;
			// Call through the supplied filterProc
			returnValue = !fileFilter(&paramBlock, NULL);
		}
		else
			returnValue = false;
	}
	else
		for (short i = 0; i < numTypes; ++i)
			if (aFile->GetCachedFileType() == (*typeList)[i])
			{
				if (!fileFilter)
					returnValue = true;
				else if (aFile->GetCatInfo(paramBlock) == noErr)
				{
					paramBlock.hFileInfo.ioNamePtr = (StringPtr)fileName;
					// Call through the supplied filterProc
					returnValue = !fileFilter(&paramBlock, NULL);
				}
				else
					returnValue = false;
				break;
			}

	typeList = (TypeListHandle)::DisposeIfHandle_AC((Handle)typeList);

	return returnValue;
}

//----------------------------------------------------------------------------------------
// TApplication::ChooseApplication: 
//----------------------------------------------------------------------------------------

bool TApplication::ChooseApplication(AEAddressDesc& theAddress)
{
	bool returnValue = false;

	TargetIDHdl theTargetID;
	LocationNameRec theLoc;
	PortInfoRec thePortInfo;

	OSErr theErr = PPCBrowser(gEmptyString, gEmptyString, false, &theLoc, &thePortInfo, NULL, gEmptyString);
	if (theErr == noErr)
	{
		theTargetID = (TargetIDHdl)NewPermHandle(sizeof(TargetID));

		(*theTargetID)->location = theLoc;
		(*theTargetID)->name = thePortInfo.name;
		theAddress.descriptorType = typeTargetID;
		theAddress.dataHandle = (Handle)theTargetID;
		returnValue = true;
	}
	else if (theErr != userCanceledErr)
		::ThrowIfOSErr_AC(theErr);

	return returnValue;
}

//----------------------------------------------------------------------------------------
// TApplication::ChooseDocument: 
//----------------------------------------------------------------------------------------

bool TApplication::ChooseDocument(CommandNumber itsCommandNumber,
									 CList_AC** aFileList)
{
	typedef SFTypeList* SFTypeListPtr;
	typedef SFTypeListPtr* SFTypeListHandle;

	TFile *aFile = NULL;

	bool fileChosen;

	FileFilterYDProcPtr fileFilter;
	TypeListHandle typeList;
	short dlgID;
	CPoint_AC where;
	DlgHookYDProcPtr dlgHook;
	ModalFilterYDProcPtr modalFilter;
	short* activeList;
	ActivateYDProcPtr activateProc;
	StandardFileReply customReply;
	void* yourDataPtr = NULL;

	GetStandardFileParameters(itsCommandNumber, fileFilter, typeList, dlgID, where, dlgHook, modalFilter, activeList, activateProc, &customReply, yourDataPtr);

	SFTypeListPtr pTypeList;
	short numTypes = static_cast<short>(GetHandleSize((Handle)typeList) / sizeof(ResType));
	if (!numTypes)
	{
		numTypes = -1;						// Tell Std File to display all types.
		pTypeList = (SFTypeListPtr) & pTypeList;// arbitrary, as long as it points to 4 bytes of valid memory
	}
	else
	{
		LockHandleHigh((Handle)typeList);	// in case Std File does allocations 
		pTypeList = *((SFTypeListHandle)typeList);
	}

	// Causes TApplication::GetEvent to call CheckRsrcUsage. 
	gRsrcCheck = 0;

	try
	{
		aFile = DoMakeFile(itsCommandNumber);
		fileChosen = false;

		::ThrowIfOSErr_AC(MAInteractWithUser());

		if (!yourDataPtr)
			yourDataPtr = &itsCommandNumber;

		gClipboardMgr->AboutToLoseControl(true);// so scrap gets converted

		TemplateTempRoutineDescriptor<FileFilterYDUPP> cgfFileFilter(NewFileFilterYDProc(fileFilter));
		TemplateTempRoutineDescriptor<DlgHookYDUPP> cgfDlgHook(NewDlgHookYDProc(dlgHook));
		TemplateTempRoutineDescriptor<ModalFilterYDUPP> cgfModalFilter(NewModalFilterYDProc(modalFilter));
		TemplateTempRoutineDescriptor<ActivateYDUPP> cgfActivateProc(NewActivateYDProc(activateProc));

		if (fUseQuickTimePreview && HasQuickTime_AC())
			CustomGetFilePreview(cgfFileFilter, numTypes, (*pTypeList), &customReply, dlgID, where, cgfDlgHook, cgfModalFilter, activeList, cgfActivateProc, yourDataPtr);
		else
			CustomGetFile(cgfFileFilter, numTypes, (*pTypeList), &customReply, dlgID, where, cgfDlgHook, cgfModalFilter, activeList, cgfActivateProc, yourDataPtr);

		gClipboardMgr->RegainControl(true);	// so scrap gets converted

		fileChosen = customReply.sfGood;
		if (fileChosen)
			aFile->SpecifyWithStandardFileReply(customReply);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			typeList = (TypeListHandle)::DisposeIfHandle_AC((Handle)typeList);
			delete aFile;
			aFile = NULL;
			throw;
		}

		typeList = (TypeListHandle)::DisposeIfHandle_AC((Handle)typeList);

	if (fileChosen)
	{
		try
		{
			// Return the file(s) chosen 
			*aFileList = NewList_AC();
			(*aFileList)->InsertLast(aFile);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			if (*aFileList)
			{
				delete *aFileList;
				*aFileList = NULL;
			}
			delete aFile;
			aFile = NULL;
			throw;
		}
	}
	else										// user cancelled or something 
	{
		delete aFile;	// free the unwanted file object 
		aFile = NULL;
	}

	return fileChosen;
}

//----------------------------------------------------------------------------------------
// TApplication::DoMakeDocument: 
//
// E X A M P L E
// {
// 		TDocument*	aYOURDocument = NULL;
//
// 		aYOURDocument = new TDocument;
// 		aYOURDocument->IYOURDocument();
//		return aYOURDocument;
// }
//----------------------------------------------------------------------------------------

TDocument* TApplication::DoMakeDocument(CommandNumber,
										TFile*)
{
	SubClassResponsibility();

	return NULL;								// So that CFront doesn't complain
}

//----------------------------------------------------------------------------------------
// TApplication::DoMenuCommand: 
//----------------------------------------------------------------------------------------

void TApplication::DoMenuCommand(CommandNumber aCommandNumber)
{
	// ===================================================================================
	// Some commands will be posted to perform actions that must _ALWAYS_ be available.
	// The allocation cannot be allowed to fail. So we do a temp allocation which by
	// definition cannot be allowed to fail. This strategy is used wherever we want to use
	// command objects but don't want to leave the user twisting in the breeze. NOTE:
	// Don't forget to allow for this memory in your mem! resource if you copy this style
	// in your own code.
	// ===================================================================================

	switch (aCommandNumber)
	{
		case cQuit:
			{
				CTempAllocation tempAllocation(true);
				CTempObjectAllocation tempObjectAllocation(true);
				
				PostCommand(TH_new TQuitCommand(aCommandNumber));
			}
			break;

		case cNew:								// cNew..cNewLast:
		case 11:
		case 12:
		case 13:
		case 14:
		case 15:
		case 16:
		case 17:
		case 18:
		case cNewLast:
		case cFinderNew:
			DoNew(aCommandNumber);
			break;

		case cOpen:								// cOpen..cOpenLast:
		case 21:
		case 22:
		case 23:
		case 24:
		case 25:
		case 26:
		case 27:
		case 28:
		case cOpenLast:
			DoOpen(aCommandNumber);
			break;

		case cAboutApp:
			PostCommand(TH_new TAboutBoxCommand);
			break;

#if qDebug
		case cDebugFlags:
			{
				if (fDebugFlagsWindow.IsNULL())
				{
					fDebugFlagsWindow = TViewServer::fgViewServer->NewTemplateWindow(kDebugFlagsView, NULL);
					::ThrowIfNULL_AC(fDebugFlagsWindow.get());
				}
				fDebugFlagsWindow->Open();
				fDebugFlagsWindow->Select();
			}
			break;

		case cRefreshFrontWindow:
			{
				TWindow * aWindow = GetActiveWindow(kNoFloaters);

				if (aWindow)
					aWindow->ForceRedraw();
			}
			break;

		case cDoFirstClick:
			{
				TWindow * aWindow = GetActiveWindow(kNoFloaters);

				if (aWindow)
					aWindow->SetDoFirstClick(!aWindow->GetDoFirstClick());
			}
			break;

#endif

#if qPerform
		case cPerfMonInit:
			{
				CAutoPtr_AC<TWindow> aWindow = TViewServer::fgViewServer->NewTemplateWindow(kPerfMonInitView, NULL);
				::ThrowIfNULL_AC(aWindow);

				IDType dismisser = aWindow->PoseModally();
				if (dismisser == 'init')
					InitiatePerfMonitor(aWindow);
				aWindow->Close();
			}
			break;

		case cPerfMonDump:
			{
				StandardFileReply replyRec;
				StandardPutFile((StringPtr)"\pSave as:", (StringPtr)"\puntitled", &replyRec);
				if (replyRec.sfGood)
					DumpPerfMonitor(replyRec.sfFile);
			}
			break;

		case cPerfMonToggle:
			EnablePerfMonitor(!PerfMonitorEnabled());
			break;

		case cPerfMonEnd:
			TerminatePerfMonitor();
			break;
#endif

		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TApplication::DoSetupMenus: 
//----------------------------------------------------------------------------------------

void TApplication::DoSetupMenus()
{
	Inherited::DoSetupMenus();
	
	if (!InModalState())
	{
		bool lowSpace = MemSpaceIsLow();
	
		Enable(cQuit, fEventLevel <= 1);			// Can't enable Quit if in nested event handling
		Enable(cNew, !lowSpace);
		Enable(cOpen, !lowSpace);

#if qDebug
		{
			TWindow * aWindow = gApplication->GetActiveWindow(kNoFloaters);
	
			Enable(cRefreshFrontWindow, aWindow != NULL);
			Enable(cDoFirstClick, aWindow != NULL);
			if (aWindow)
				SetMenuState(cDoFirstClick, kDebugBuzzStrings, bzDoFirstClick, bzDontDoFirstClick, aWindow->GetDoFirstClick());
	
			Enable(cDebugFlags, true);
	
			// System Justification - see comment in Debug.r
			//	Enable(cSetSysJust, true);
			//	SetMenuState(cSetSysJust, kDebugBuzzStrings, bzSetRightSysJust, bzSetLeftSysJust, GetSysDirection() != smSysJustLeft);
		}
#endif

#if qPerform
		{
			bool initiated = PerfMonitorInitiated();
	
			Enable(cPerfMonInit, !initiated);
			Enable(cPerfMonDump, initiated);
			Enable(cPerfMonToggle, initiated);
			SetMenuState(cPerfMonToggle, kDebugBuzzStrings, bzContinuePerfMon, bzPausePerfMon, PerfMonitorEnabled());
			Enable(cPerfMonEnd, initiated);
		}
#endif
	}
}

//----------------------------------------------------------------------------------------
// TApplication::DoAboutBox: Method to display the "About" box for your application.
// Override to do interesting things. Since it is normally called from a command; the app
// usually has the maximum free space available.
//----------------------------------------------------------------------------------------

void TApplication::DoAboutBox()
{
	FailSpaceIsLow();
	::ThrowIfOSErr_AC(MAInteractWithUser());

	CStr255_AC apName;
	GetApplicationName(apName);

	ParamText(apName, gEmptyString, gEmptyString, gEmptyString);// Put Application name in the about box 

	if (GetResource('STR#', kDefaultCredits))
	{
		lastCreditsStringIndex = 1;
		lastCreditsShownTicks = TickCount();
		waitTicks = 5 * 60;
		originalText = (CPascalStrHandle_AC)NewString(gEmptyString);
		::ThrowIfNULL_AC(originalText);
		MacAppAlert(phAboutApp, DoShowAboutAppFilter);
		originalText = (CPascalStrHandle_AC)::DisposeIfHandle_AC((Handle)originalText);
	}
	else
		StdAlert(phAboutApp);
}

//----------------------------------------------------------------------------------------
// TApplication::HandleFinderRequest: 
//----------------------------------------------------------------------------------------

void TApplication::HandleFinderRequest()
{
}

//----------------------------------------------------------------------------------------
// TApplication::KindOfDocument: 
//----------------------------------------------------------------------------------------

CommandNumber TApplication::KindOfDocument(CommandNumber itsCommandNumber,
										   TFile*)
{
	return itsCommandNumber;
}

//----------------------------------------------------------------------------------------
// TApplication::OpenNew: 
//----------------------------------------------------------------------------------------

TDocument* TApplication::OpenNew(CommandNumber itsCommandNumber)
{
	CAutoPtr_AC<TDocument> aDocument = DoMakeDocument(KindOfDocument(itsCommandNumber, NULL), NULL);
	::ThrowIfNULL_AC(aDocument);

	try
	{
		CStr255_AC newTitle;
		aDocument->UntitledName(newTitle);
		aDocument->SetTitle(newTitle);
	
		aDocument->DoInitialState();
	
		aDocument->DoMakeViews(kForDisplay);
	
		FailSpaceIsLow();						// Fail if document leaves us with no room 
	
		// Don't attempt to show the windows until we're sure we won't fail 
		aDocument->DoPostMakeViews(kForDisplay);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), messageNewFailed);
	}
	
	return aDocument.release();
}

//----------------------------------------------------------------------------------------
// TApplication::OpenOld: 
//----------------------------------------------------------------------------------------

TDocument* TApplication::OpenOld(CommandNumber itsOpenCommand,
								 CList_AC* aFileList)
{
	TDocument* aDocument = NULL;
	TFile* aNewFile = NULL;
	OSErr savedError = 0;
	long savedMessage = 0;
	
	Size oldCodeReserve;						// can't be volatile because it is passed by reference
	Size oldMemReserve;							// can't be volatile because it is passed by reference

	{
		// Put iterator in a separate block so that it will go out of scope before the end of the method.
		// We don't want the iterator's failure handler to be invoked, because its stack ptr is munged.
		// If an error occurs, we'll just save it and ReSignal it after the iterator is gone.

		for (TemplateIterator_AC<TFile*> iter(aFileList); iter; ++iter)
		{
			TFile * aFile = iter;
		
			CommandNumber kindOfDocCmd = KindOfDocument(itsOpenCommand, aFile);
			// reset aDocument and aNewFile so that they no longer refer to items from the
			// last go round (important in case we fail downstream)
			aDocument = NULL;
			aNewFile = NULL;

			CStr63_AC fileName;
			aFile->GetName(fileName);			// If failure occurs the file may already be
			// freed so we will save the name just in
			// case.

			aFileList->Delete(aFile);			// ReadStationery if successful will dispose
			// of the file for us so if failure occurs
			// this had better not be in the file list.

			try
			{
				// Set reserve down a little to ensure that we can open existing documents 
				GetReserveSize(oldCodeReserve, oldMemReserve);
				SetReserveSize(oldCodeReserve, oldMemReserve / 2);

				TDocument * otherDoc = FindDocument(aFile);
				bool isStationery = aFile->IsStationery();

				if (otherDoc && !isStationery)
				{
					otherDoc->OpenAgain(kindOfDocCmd, aDocument);
					delete aFile;
					aFile = NULL;
					SetReserveSize(oldCodeReserve, oldMemReserve);
				}
				else if (CanOpenDocument(kindOfDocCmd, aFile))
				{
					TFile * fileOwnedByDoc = aFile;
					aFile = NULL;
					::ThrowIfNULL_AC(aDocument = DoMakeDocument(kindOfDocCmd, fileOwnedByDoc));

					if (!isStationery)
						aDocument->ReadDocument(kForDisplay);
					else
					{
						aNewFile = DoMakeFile(kindOfDocCmd);
						aDocument->ReadStationery(aNewFile);
						aFile = NULL;			// This file has already been freed by ReadStationery.
						aNewFile = NULL;		// Clear our reference so that if we fail
						// we don't try to free this file twice.
					}

					aDocument->DoMakeViews(kForDisplay);

					if (isStationery)
					{
						CStr255_AC newTitle;
						aDocument->UntitledName(newTitle);
						aDocument->SetTitle(newTitle);
					}

					FailSpaceIsLow();			// Fail if the document leaves us with no
					// memory
					// Set the reserve back to where it was 
					SetReserveSize(oldCodeReserve, oldMemReserve);

					// Don't attempt to show the windows until we're sure we won't fail 
					aDocument->DoPostMakeViews(kForDisplay);
				}
				else
				{
					// If we ended up without a document because we didn't understand this type
					// of file then let's get rid of the file.
					delete aFile;
					aFile = NULL;
					::ThrowException_AC(errNotMyType, 0);
				}
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				if (!theException.GetMessage())
					gErrorParm3 = fileName;

				// We may have ended up without a document because of a failure during document
				// init. b/c this can be the case we don't know what happened to aFile, whether 
				// it was freed by the document in the failure process or orphaned. Because of
				// this we will not free aFile here.
				if (aDocument)
				{
					delete aDocument;// The document will free the file
					aDocument = NULL;
				}

				delete aNewFile;		// If we successfully read the
				aNewFile = NULL;		// stationery then aNewFile will be NULL

				// Set the reserve back to where it was 
				SetReserveSize(oldCodeReserve, oldMemReserve);

				//	Don't want to fail here, because the CObjectIterator (with its embedded FailInfo) is 
				//	still in scope. Instead, save the error info until after the iterator has self-destructed.
				//	::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), messageOpenFailed);
				savedError = theException.GetError();
				savedMessage = theException.GetMessage();
				if (!savedMessage)
					savedMessage = messageOpenFailed;
			}
			
		}
	}

	if (savedError != noErr)					// Do we need to ReSignal a failure?
	{
		// signal the failure
		::ThrowException_AC(savedError, savedMessage);
	}
	return aDocument;
}

//----------------------------------------------------------------------------------------
// TApplication::PrintDocuments: 
//----------------------------------------------------------------------------------------

void TApplication::PrintDocuments(CommandNumber itsCommandNumber,
								  CList_AC* aFileList,
								  const CFSSpec_AC* targetPrinter)
{
	TPrintHandler::gFinderPrintingProceed = true;// set in TStdPrintHandler::DoPrintCommand
	TPrintHandler::gPrintHandler->PrepareForFinderPrinting(targetPrinter);

	for (TemplateIterator_AC<TFile*> iter(aFileList); iter; ++iter)
	{
		TFile * aFile = iter;
		
		if (TPrintHandler::gFinderPrintingProceed)
		{
			CAutoPtr_AC<TDocument> aDocument;

			CommandNumber kindOfDocCmd = KindOfDocument(itsCommandNumber, aFile);

			if (CanOpenDocument(kindOfDocCmd, aFile))
			{
				try
				{
					aDocument = DoMakeDocument(KindOfDocument(itsCommandNumber, aFile), aFile);
					::ThrowIfNULL_AC(aDocument);
					aDocument->ReadDocument(kForPrinting);
					aDocument->DoMakeViews(kForPrinting);
					aDocument->DoPostMakeViews(kForPrinting);
	
					aDocument->HandleMenuCommand(itsCommandNumber);
				}
				catch(CException_AC& theException)
				{
					DoCatchMessage_AC(theException);
					// If there is a document then it will have a reference to aFile and
					// will free it when it is freed. Therefore we need to remove it from
					// the list so that when the file list is freed we don't attempt to
					// free it twice.
					if (aDocument)
						aFileList->Delete(aFile);
	
					throw;
				}
			}
			else
			{
				// If we ended up without a document because we didn't understand this type
				// of file then let's get rid of the file.
				delete aFile;
				ThrowException_AC(errNotMyType, 0);
			}
			
			aFileList->Delete(aFile);
		}
		else
			break;								// The user canceled printing
	}
}

//----------------------------------------------------------------------------------------
// TApplication::GetStandardFileParameters: 
//----------------------------------------------------------------------------------------

void TApplication::GetStandardFileParameters(CommandNumber itsCommandNumber,
											 FileFilterYDProcPtr& fileFilter,
											 TypeListHandle& typeList,
											 short& dlgID,
											 CPoint_AC& where,
											 DlgHookYDProcPtr& dlgHook,
											 ModalFilterYDProcPtr& modalFilter,
											 short*& activeList,
											 ActivateYDProcPtr& activateProc,
											 StandardFileReply* /*reply*/  ,
											 void*& yourDataPtr)
{
	dlgID = 0; // Let the system decide
	where = kBestSystemLocation;

	GetFileTypeList(itsCommandNumber, typeList);

	fileFilter = NULL;
	dlgHook = NULL;
	modalFilter = gModalFilterYDProcPtr;
	activeList = NULL;
	activateProc = NULL;
	yourDataPtr = NULL;
}

//----------------------------------------------------------------------------------------
// TApplication::GetFileTypeList: 
//----------------------------------------------------------------------------------------

void TApplication::GetFileTypeList(CommandNumber,
								   TypeListHandle& typeList)
{
#if 0
	enum
	{
		kMaxTranslationTypes = 100
	};

	RSRCopenHdl	open = (RSRCopenHdl)MAGetResource('open', 128);
	
	if (open)		// if the user provided an 'open' resource, then
	{				// we can compute the file list on the fly!
		// Get native types out of resource
		short		nativeTypes = (**open).numTypes;
		Size		size	 	= nativeTypes * sizeof(OSType);
		typeList	= (TypeListHandle) NewPermHandleClear(HasTranslationManager_AC() ? kMaxTranslationTypes*sizeof(OSType) : size);
		BlockMove(&(**open).types, *typeList, size);						// copy 'em over

		if (HasTranslationManager_AC())
		{
			// if we have the translation manager, we can extend the list
		
			short newTypes = kMaxTranslationTypes - nativeTypes;
			HLock((Handle) typeList);
			OSErr err = ExtendFileTypeList(*typeList, nativeTypes, &(*typeList)[nativeTypes], &newTypes);
			HUnlock((Handle) typeList);
			SetHandleSize((Handle) typeList, (newTypes + nativeTypes) * sizeof(OSType));
			ThrowIfMemError_AC();
			ThrowIfOSErr_AC(err);
		}
	}
	else	// Do it the old way
#endif
	{
		typeList = (TypeListHandle) NewPermHandle(sizeof(OSType));
		(*typeList)[0] = fMainFileType;
	}
}

//----------------------------------------------------------------------------------------
// TApplication::DoNew: 
//----------------------------------------------------------------------------------------

void TApplication::DoNew(CommandNumber theCmdNumber)
{
	TNewDocumentCommand * aNewDocCommand = TH_new TNewDocumentCommand(theCmdNumber);
	aNewDocCommand->SetUseAppleEvent(true);
	PostCommand(aNewDocCommand);
}

//----------------------------------------------------------------------------------------
// TApplication::DoOpen: 
//----------------------------------------------------------------------------------------

void TApplication::DoOpen(CommandNumber theCmdNumber)
{
	CList_AC * aFileList = NULL;

	if (ChooseDocument(theCmdNumber, &aFileList))
	{
		TODocCommand * anODocCommand = TH_new TODocCommand(theCmdNumber, aFileList);
		anODocCommand->SetUseAppleEvent(true);
		PostCommand(anODocCommand);
	}
}

//----------------------------------------------------------------------------------------
// TApplication::DoAECreateElement: 
//----------------------------------------------------------------------------------------

void TApplication::DoAECreateElement(TAppleEvent* message,
									 TAppleEvent* reply)
{
	// Creates a new document. If your application has more than one type of document
	// you'll need to override this method. The default simulates choosing "New" from
	// the file menu.
	DescType theClass = message->ReadType(keyAEObjectClass);
	if (IsDocumentClass(theClass) || (theClass == cWindow))
	{
		if (CommandEnabled(cNew))
		{
			TDocument * theDoc = OpenNew(cNew);
			// Apply the properties if any.
			theDoc->SetPropertiesFromEvent(message);

			// Apply the initial data if any
			if (message->HasParameter(keyAEData))
				theDoc->DoAESetData(message, reply);

			// Return the document's object specifier
			CTempDesc theDocsSpecifier;
			theDoc->MakeObjectSpecifier(theDocsSpecifier, formName);
			if (reply->dataHandle)
				reply->WriteParameter(keyAEResult, theDocsSpecifier);
		}
	}
}

//----------------------------------------------------------------------------------------
// TApplication::DoAEClose: 
//----------------------------------------------------------------------------------------

void TApplication::DoAEClose(TAppleEvent* message,
							 TAppleEvent* reply)
{
	if (::CommandEnabled(cQuit))
	{
		CTempAllocation tempAllocation(true);
		CTempObjectAllocation tempObjectAllocation(true);
		
		TQuitCommand * quitCommand = TH_new TQuitCommand(message, reply);
		quitCommand->Process();
	}
}

//----------------------------------------------------------------------------------------
// TApplication::DoScriptCommand: 
//----------------------------------------------------------------------------------------

void TApplication::DoScriptCommand(CommandNumber aCommandNumber,
								   TAppleEvent* message,
								   TAppleEvent* reply)
{
	switch (aCommandNumber)
	{
		case cFinderNew:
			// Special case the old MacPaint style open if user knows about the option
			// key. Only if the application is frontmost though.
			if (IsFrontProcess() && IsOptionKeyDown())
				HandleMenuCommand(cOpen);
			else if (fLaunchWithNewDocument)
				GetTarget()->HandleMenuCommand(cFinderNew);
			break;

		case cFinderOpen:
			PostCommand(TH_new TODocCommand(aCommandNumber, *message, *reply));
			break;

		case cFinderPrint:
		case cDragPrint:
			PostCommand(TH_new TPDocCommand(aCommandNumber, *message, *reply));
			break;

		case cSectionRead:
		case cSectionWrite:
		case cSectionScroll:
		case cSectionCancel:
			{
				TSectionMgr * aSectionMgr = dynamic_cast_AC(TSectionMgr*,GetBehaviorWithIdentifier(kSectionMgrBehaviorID));
				
				aSectionMgr->DoAESectionEvent(aCommandNumber, *message, *reply);
			}
			break;

		default:
			Inherited::DoScriptCommand(aCommandNumber, message, reply);
			break;
	}
}

//----------------------------------------------------------------------------------------
// End of UApplication.cp 
