//----------------------------------------------------------------------------------------
// UContextMenuBehavior.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCONTEXTMENUBEHAVIOR__
#include "UContextMenuBehavior.h"
#endif

#if qWantsContextualMenus


// Toolbox

#ifndef __CONTEXTUALMENU__
#include <ContextualMenu.h>
#endif

//----------------------------------------------------------------------------------------

void InitUContextMenuBehavior()
{
	MA_REGISTER_CLASS(TContextMenuBehavior);
}

//========================================================================================
// CLASS TContextMenuBehavior
//========================================================================================

#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TContextMenuBehavior);

//----------------------------------------------------------------------------------------
// TContextMenuBehavior constructor
//----------------------------------------------------------------------------------------


TContextMenuBehavior::TContextMenuBehavior(IDType itsIdentifier) :
	TBehavior(itsIdentifier)
{
	if (HasContextualMenus())
	{
		OSStatus err = ::InitContextualMenus();
		::ThrowIfOSErr_AC((OSErr) err);
	}
}

//----------------------------------------------------------------------------------------
// TContextMenuBehavior copy constructor
//----------------------------------------------------------------------------------------


TContextMenuBehavior::TContextMenuBehavior(const TBehavior& inBehavior) :
	TBehavior(inBehavior)
{
}

//----------------------------------------------------------------------------------------
// TContextMenuBehavior destructor
//----------------------------------------------------------------------------------------


TContextMenuBehavior::~TContextMenuBehavior()
{
}


//----------------------------------------------------------------------------------------
// TContextMenuBehavior::DoToolboxEvent:
//----------------------------------------------------------------------------------------


bool TContextMenuBehavior::DoToolboxEvent(TToolboxEvent* event)
{
	if (HasContextualMenus() && IsShowContextualMenuClick(&event->GetEventRecord())) {
		DispatchContextualMenuEvent(event);
		return false;	// ContextualMenu handled the event.
	}
	else
	{
		return Inherited::DoToolboxEvent(event);
	}
}

//----------------------------------------------------------------------------------------
// TContextMenuBehavior::DispatchContextualMenuEvent: 
//----------------------------------------------------------------------------------------


void TContextMenuBehavior::DispatchContextualMenuEvent(TToolboxEvent* event)
{
	MenuRef inMenuRef = NULL;
	bool inBalloonAvailable = true;	// we only show contextual menus for views that support balloons
	UInt32 inHelpType = kCMHelpItemNoHelp; 
	CStr255_AC inHelpItemString;
	CAEDesc inSelection;
	UInt32 outUserSelectionType;
	SInt16 outMenuID;
	UInt16 outMenuItem;
	CPoint_AC globalMouse = event->GetEventRecord().where;
	
	// Get the relevant window and iterate to find the target subview
	WindowRef aWMgrWindow;
	if (FindWindow(globalMouse, &aWMgrWindow) == inContent)
	{
		TWindow * helpWindow = TWindow::WMgrToWindow(aWMgrWindow);
		if ((helpWindow) && helpWindow->HandlesHelp())
		{
			HandleContextualMenu(helpWindow, helpWindow->SuperViewToLocal(globalMouse), inMenuRef, inSelection);
		}
	}

	// Make sure that the menus are setup correctly before we handle the event.
	// Note: HandleContextualMenu may change the target, so it's important this call comes immediately before the menu select.
	InvalidateMenus();
	gDispatcher->SetupTheMenus(); 
	
	// Always show "some" contextual menu (NULL shows a default system menu)
	OSStatus result = ::ContextualMenuSelect(inMenuRef,
										globalMouse,
										inBalloonAvailable,
										inHelpType,
										inHelpItemString,
										NULL,
										&outUserSelectionType,
										&outMenuID,
										&outMenuItem);
	
	if (result != userCanceledErr)
	{
		::ThrowIfOSErr_AC((OSErr) result);
	
		switch (outUserSelectionType)
		{
			case kCMMenuItemSelected:
				gDispatcher->MenuEvent(outMenuID, outMenuItem);
				break;
			
			case kCMShowBalloonSelected:
				gDispatcher->TrackHelp(CPoint_AC(event->GetEventRecord().where));	
				break;
			
			case kCMShowHelpSelected:
				break;
				
			default:
				break;
		}
	}
}

//----------------------------------------------------------------------------------------
// TContextMenuBehavior::HandleContextualMenu: 
//----------------------------------------------------------------------------------------


void TContextMenuBehavior::HandleContextualMenu(TView* thisView, const CViewPoint& theMouse, MenuRef& theMenuRef, CAEDesc& theSelection)
{
	//Assume if we got here then we already know the mouse is in this view
	if (thisView->LetsSubViewsHandleHelp())
		for (CSubViewIterator iter(thisView, kIterateBackward_AC); iter; ++iter)
		{
			CViewPoint subViewPt(iter->SuperViewToLocal(theMouse));

			if (iter->ContainsMouse(subViewPt) && iter->HandlesHelp())
			{				
				HandleContextualMenu(iter, subViewPt, theMenuRef, theSelection);
				break;
			}
		}

	if (theMenuRef == NULL && thisView->HandlesHelp())
	{
		thisView->GetContextualMenu(theMenuRef, theSelection);
		if (theMenuRef != NULL)
		{
			// !!! become the target
			// !!! have the thing under the mouse become the selection
			gDispatcher->SetTarget(thisView);
		}
	}
}

//----------------------------------------------------------------------------------------
// TContextMenuBehavior::HasContextualMenus
//----------------------------------------------------------------------------------------

bool TContextMenuBehavior::HasContextualMenus()
{
	static bool defined;
	static bool hasContextMenus;
	if (!defined)
	{
		defined = true;
		hasContextMenus = MAGestaltAttribute(gestaltContextualMenuAttr, gestaltContextualMenuTrapAvailable);
	}
	return hasContextMenus;
}

#endif // qWantsContextualMenus

//----------------------------------------------------------------------------------------
// End of UContextMenuBehavior.cp


