//----------------------------------------------------------------------------------------
// UDebugView.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#if qDebug

#ifndef __UDEBUGVIEW__
#include "UDebugView.h"
#endif

// MacApp

#ifndef __UCHECKBOX__
#include "UCheckBox.h"
#endif

#ifndef __UCLUSTER__
#include "UCluster.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDIALOG__
#include "UDialog.h"
#endif

#ifndef __UDIALOGBEHAVIOR__
#include "UDialogBehavior.h"
#endif

#ifndef __USTATICTEXT__
#include "UStaticText.h"
#endif


// Toolbox

// ANSI

void InitUDebugView()
{		
	// Register views so we can create them from templates
	if (qTemplateViews)
	{
		InitUCluster();
		InitUCheckbox();
		InitUStaticText();	

		MA_REGISTER_CLASS(TDebugFlagsView);
		MA_REGISTER_SIGNATURE(TDialogBehavior, kStdDialogBehavior);
	}
} 

//========================================================================================
// CLASS TDebugFlagsView
//========================================================================================
MA_DEFINE_CLASS(TDebugFlagsView);

//----------------------------------------------------------------------------------------
// TDebugFlagsView destructor
//----------------------------------------------------------------------------------------
TDebugFlagsView::~TDebugFlagsView()
{
}

//----------------------------------------------------------------------------------------
// TDebugFlagsView::DoEvent:
//----------------------------------------------------------------------------------------
void TDebugFlagsView::DoEvent(EventNumber eventNumber,
									 TEventHandler* source,
									 TEvent* event)
{
	if (eventNumber == mCheckBoxHit)
	{
		switch (source->GetIdentifier())
		{
			case 'usr1':
				gUserFlag1 = !gUserFlag1;
				break;
				
			case 'usr2':
				gUserFlag2 = !gUserFlag2;
				break;
				
			case 'usr3':
				gUserFlag3 = !gUserFlag3;
				break;
				
			case 'focu':
				gAssumeFocused = !gAssumeFocused;
				break;

			case 'prnt':
				gDebugPrinting = !gDebugPrinting;
				break;
				
			case 'ints':
				gIntenseDebugging = !gIntenseDebugging;
				break;
				
			case 'evnt':
				gReportEvent = !gReportEvent;
				break;
				
			case 'menu':
				gReportMenuChoices = !gReportMenuChoices;
				break;
				
			case 'curs':
				gShowCursorRegion = !gShowCursorRegion;
				break;
				
			case 'help':
				gShowHelpRegion = !gShowHelpRegion;
				break;
				
			case 'invl':
				gShowInvalidations = !gShowInvalidations;
				break;
				
			case 'slee':
				gShowSleepRegion = !gShowSleepRegion;
				break;
				
			case 'memb':
				gMemMgtBreak = !gMemMgtBreak;
				break;
				
			case 'rsrc':
				gRsrcReport = !gRsrcReport;
				break;
		}
	}
	else
		TView::DoEvent(eventNumber, source, event);
}


//----------------------------------------------------------------------------------------
// TDebugFlagsView::DoPostCreate:
//----------------------------------------------------------------------------------------
void TDebugFlagsView::DoPostCreate(TDocument* itsDocument)
{
	TView::DoPostCreate(itsDocument);

	// Synchronize the global flags and the check boxes used to set toggle them.
	
	#define SyncState(id,flag) do { TCheckBox* workBox = dynamic_cast_AC(TCheckBox*,FindSubView('usr1')); \
								 	workBox->SetState(flag, false); } while(0)

	SyncState('usr1', gUserFlag1);
	SyncState('usr2', gUserFlag2);
	SyncState('usr3', gUserFlag3);
	SyncState('focu', gAssumeFocused);
	SyncState('prnt', gDebugPrinting);
	SyncState('ints', gIntenseDebugging);
	SyncState('evnt', gReportEvent);
	SyncState('menu', gReportMenuChoices);
	SyncState('curs', gShowCursorRegion);
	SyncState('help', gShowHelpRegion);
	SyncState('invl', gShowInvalidations);
	SyncState('slee', gShowSleepRegion);
	SyncState('memb', gMemMgtBreak);
	SyncState('rsrc', gRsrcReport);
	
	#undef SyncState
}

#endif // qDebug

//----------------------------------------------------------------------------------------
// End of UDebugView.cp
