//----------------------------------------------------------------------------------------
// UPerform.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPERFORM__
#include "UPerform.h"
#endif

// MacApp

#ifndef __UDIALOG__
#include "UDialog.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

// Apple Specials
	
#if qPerform
	#ifndef __PERF__
	#include "Perf.h"
	#endif
#endif

//----------------------------------------------------------------------------------------
// Static variable definitions.
//----------------------------------------------------------------------------------------

#if qPerform

static TP2PerfGlobals pTP2PerfGlobals;			// Pointer to performance globals record
                                                // Non-nil if tools are inited

static bool pPerfMonitorEnabled;				// Is the performance monitor enabled or
                                                // disabled.
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#if qPerform

//----------------------------------------------------------------------------------------
// PerfMonitorInitiated: Are we currently doing performance monitoring.
//----------------------------------------------------------------------------------------
bool PerfMonitorInitiated()
{
	return pTP2PerfGlobals != NULL;
}


//----------------------------------------------------------------------------------------
// InitiatePerfMonitor: Initiate performance monitoring from the initialize performance
// monitor dialog box.
//----------------------------------------------------------------------------------------
void InitiatePerfMonitor(TWindow* aWindow)
{
#if q68K
	bool aBool;
	
	short timerCount = static_cast<short>(dynamic_cast_AC(TNumberText*,aWindow->FindSubView('tmct'))->GetValue());
	short codeAndROMBucketSize = static_cast<short>(dynamic_cast_AC(TNumberText*,aWindow->FindSubView('cbsz'))->GetValue());
	bool doROM = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('mrcd'))->IsOn();
	bool doAppCode = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('macd'))->IsOn();
	CStr255_AC appCodeType;
	dynamic_cast_AC(TEditText*,aWindow->FindSubView('acrs'))->GetText(appCodeType);
	short romID = static_cast<short>(dynamic_cast_AC(TNumberText*,aWindow->FindSubView('rmid'))->GetValue());
	CStr255_AC romName;
	dynamic_cast_AC(TEditText*,aWindow->FindSubView('rmnm'))->GetText(romName);
	bool doRAM = dynamic_cast_AC(TCheckBox*,aWindow->FindSubView('mout'))->IsOn();
	long ramLow = dynamic_cast_AC(TNumberText*,aWindow->FindSubView('from'))->GetValue();
	long ramHigh = dynamic_cast_AC(TNumberText*,aWindow->FindSubView('to  '))->GetValue();
	short ramBucketSize = static_cast<short>(dynamic_cast_AC(TNumberText*,aWindow->FindSubView('otsz'))->GetValue());
		
	aBool = InitPerf(&pTP2PerfGlobals, timerCount, codeAndROMBucketSize, doROM, doAppCode,
					 appCodeType, romID, romName, doRAM, ramLow, ramHigh, ramBucketSize);
	
	if (!aBool)
		pTP2PerfGlobals = NULL;
	else
		EnablePerfMonitor(true);
#else
	pTP2PerfGlobals = NULL;
#endif
}


//----------------------------------------------------------------------------------------
// DumpPerfMonitor:
//----------------------------------------------------------------------------------------
void DumpPerfMonitor(const CFSSpec_AC& fileSpec)
{
#if q68K
	if (pTP2PerfGlobals)
	{
		// Create a TFile to get the pathname.
		
		TFile aFile(kTextType_AC, 'MPS ', kUsesDataFork, noResourceFork, kDataOpen, !kRsrcOpen);
		aFile.Specify(fileSpec);

		CStr255_AC pathName;
		aFile.GetPathName(pathName);

		PerfDump(pTP2PerfGlobals, pathName + (const CStr255_AC&) fileSpec.name, true, 80);
	}
#endif
}


//----------------------------------------------------------------------------------------
// EnablePerfMonitor: Enable or disable performance monitoring, assuming already its
// initiated.
//----------------------------------------------------------------------------------------
void EnablePerfMonitor(const bool enable)
{
#if q68K
	if (pTP2PerfGlobals && enable != pPerfMonitorEnabled)
	{
		PerfControl(pTP2PerfGlobals, enable);
		pPerfMonitorEnabled = enable;
	}
#endif
}


//----------------------------------------------------------------------------------------
// PerfMonitorEnabled: Is the performance monitor enabled?
//----------------------------------------------------------------------------------------
bool PerfMonitorEnabled()
{
	return pPerfMonitorEnabled;
}


//----------------------------------------------------------------------------------------
// TerminatePerfMonitor: Shut down the performance monitoring tools.
//----------------------------------------------------------------------------------------
void TerminatePerfMonitor()
{
#if q68K
	if (pTP2PerfGlobals)
	{
		TermPerf(pTP2PerfGlobals);
		pTP2PerfGlobals = NULL;
	}
#else
	pTP2PerfGlobals = NULL;
#endif
}

#endif	// qPerform

//----------------------------------------------------------------------------------------
// DebugTerminate:
//----------------------------------------------------------------------------------------
#if qDebug

void DebugTerminate()
{
#if qPerform
	// Make sure the performance tools are shut down if they are initialized 
	if (pTP2PerfGlobals)
		TerminatePerfMonitor();
#endif
}

#endif	// qDebug

//----------------------------------------------------------------------------------------
// End of UPerform.cp
