//----------------------------------------------------------------------------------------
// Compatibility.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __Compatibility__
#define __Compatibility__

#if qBackwardCompatible

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif
#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif
#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif
#ifndef __FileTypes_AC__
#include "FileTypes_AC.h"
#endif
#ifndef __CRGBColor_AC__
#include "CRGBColor_AC.h"
#endif
#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif
#ifndef __CFSSpec_AC__
#include "CFSSpec_AC.h"
#endif
#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif
#ifndef __CFile_AC__
#include "CFile_AC.h"
#endif
#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif
#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif
#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif
#ifndef __CReporter_AC__
#include "CReporter_AC.h"
#endif
#ifndef __MSTREAMABLE__
#include "MStreamable.h"
#endif
#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif
#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif
#ifndef __CStreamContext_AC__
#include "CStreamContext_AC.h"
#endif

//========================================================================================
// The following old MacApp types are defined for backward compatibility.
//========================================================================================

// Obsolete header files: 
// defines are provided here so the obsolete includes can be skipped over in your code.

#ifndef __GEOMETRY__
#define __GEOMETRY__		// use CPoint_AC and CRect_AC in ACS imaging
#endif

#ifndef __PASCALSTRING__
#define __PASCALSTRING__	// use CPascalString_AC in ACS core
#endif

#ifndef __ULIST__
#define __ULIST__			// use CList_AC in ACS containers
#endif

#ifndef __ULISTITERATOR__
#define __ULISTITERATOR__	// use CListIterator_AC in ACS containers
#endif

#ifndef __UFAILURE__
#define __UFAILURE__		// use CErrorException_AC in ACS core
#endif

#ifndef __UOBJECT__
#define __UOBJECT__			// remove all includes of UObject.h.
#endif

// defines

#define More							Current
#define FirstObject						First
#define NextObject						Next

#define MAstatic_cast(TYPE, EXPR)		static_cast<TYPE>(EXPR)
#define MAconst_cast(TYPE, EXPR)		const_cast<TYPE>(EXPR)
#define MAreinterpret_cast(TYPE, EXPR)	reinterpret_cast_AC(TYPE, EXPR)

#define Assertion(COND, DESC)			Assertion_AC(COND, DESC)
#define FailMemError()					ThrowIfMemError_AC()
#define FailResError()					ThrowIfResError_AC()
#define FailNewMessage(EXPR, MSG1, MSG2) ThrowNewMessage_AC(EXPR, MSG1, MSG2)
#define FailNIL(EXPR)					ThrowIfNULL_AC(EXPR)
#define FailNILResource(EXPR)			ThrowIfNULLResource_AC(EXPR)
#define FailOSErr(EXPR)					ThrowIfOSErr_AC(EXPR)
#define Failure(EXPR, MSG)				ThrowException_AC(EXPR, MSG)
#define ProgramBreak(EXPR)				ProgramBreak_AC(EXPR)
#define ProgramReport(EXPR1, EXPR2)		ProgramReport_AC(EXPR1, EXPR2)

#define gDimAdorner						&TAdorner::fgDimAdorner
#define gDrawAdorner					&TAdorner::fgDrawAdorner
#define gEraseAdorner					&TAdorner::fgEraseAdorner
#define gHiliteAdorner					&TAdorner::fgHiliteAdorner
#define gPrintAdorner					&TAdorner::fgPrintAdorner
#define gResizeIconAdorner				&TAdorner::fgResizeIconAdorner
#define gSelectionAdorner				&TAdorner::fgSelectionAdorner

#define gMenuBarManager					TMenuBarManager::fgMenuBarManager


// Note: these macros are no longer necessary for exception handling, but are
// 		 provided for backward compatibility

#if qHasVolatile
#define MAVolatile(type, id)				type volatile id
#define MAVolatileInit(type, id, value)		type volatile id(value)
#else
#define MAVolatile(type, id)				type id; ((void) &id)
#define MAVolatileInit(type, id, value)		type id = value; ((void) &id)
#endif

// typedefs

typedef class CRect_AC 					CRect;
typedef class CPoint_AC					CPoint;
typedef CViewPoint						VPoint;
typedef CViewRect  						VRect;
typedef CViewPoint* 					VPointPtr;
typedef CViewRect*  					VRectPtr;
typedef ViewCoordinate 					VCoordinate;
typedef CTempRegion_AC					CTempRegion;
typedef CTempRegion_AC 					CTemporaryRegion;			// for compatibility
typedef class CRGBColor_AC 				CRGBColor;
typedef class CTextStyle_AC 			CTextStyle;
typedef class CWhileOutlinePreferred_AC	CWhileOutlinePreferred;
typedef class CFSSpec_AC 				CFSSpec;
typedef enum EMoveHiState_AC 			EMoveHiState;
typedef enum EReleaseState_AC 			EReleaseState;
typedef enum ELockState_AC 				ELockState;
typedef CTempHLock_AC 					CTempHLock;
typedef CTempHandleLock_AC 				CTempHandleLock;
typedef CTempHandle_AC 					CTempHandle;
typedef CHandleIterator_AC				CHandleIterator;
typedef CTempPtr_AC 					CTempPtr;
typedef CTempRoutineDescriptor_AC		CTempRoutineDescriptor;
typedef CTempUseResFile_AC 				CTempUseResFile;
typedef CList_AC						TList;
typedef ArrayIndex_AC					ArrayIndex;
typedef CompareResult_AC				CompareResult;
typedef CSortedHandleList_AC			TSortedHandleList;
typedef ChangeID_AC						ChangeID;
typedef ClassID_AC						ClassID;
typedef class CCharStr_AC 				CCharStr,
										*CCharStrPtr, 
										**CCharStrHandle;
typedef class CPascalStr_AC 			CPascalStr,
										*CPascalStrPtr, 
										**CPascalStrHandle;
typedef class CPascalStr_AC 			CString, 
										*CStringPtr, 
										**CStringHandle;
typedef unsigned char 					Str2[3];
typedef unsigned char 					Str8[9];
typedef class CChar255_AC 				CChar255;
typedef class CStr255_AC 				CStr255;
typedef TemplateCharStr<Str63>			CChar63;
typedef TemplateCharStr<Str32>			CChar32;
typedef TemplateCharStr<Str31>			CChar31;
typedef TemplateCharStr<Str27>			CChar27;
typedef TemplateCharStr<Str15>			CChar15;
typedef TemplateCharStr<Str8>			CChar8;
typedef TemplateCharStr<Str2>			CChar2;
typedef TemplatePascalStr<Str63> 		CStr63;
typedef TemplatePascalStr<Str32> 		CStr32;
typedef TemplatePascalStr<Str31> 		CStr31;
typedef TemplatePascalStr<Str27> 		CStr27;
typedef TemplatePascalStr<Str15> 		CStr15;
typedef TemplatePascalStr<Str8> 		CStr8;
typedef TemplatePascalStr<Str2> 		CStr2;
typedef CDependencySpace_AC				TDependencySpace;
typedef CStreamContext_AC				TContext;

// enumerations and constants

const OSType kAllQuestionsType 			= kAllQuestionsType_AC;
const OSType kTextType 					= kTextType_AC;
const OSType kPictType 					= kPictType_AC;
const OSType kSoundType 				= kSoundType_AC;
const OSType kStyleType 				= kStyleType_AC;
const short kNoFileRefNum 				= kNoFileRefNum_AC;
const short kNoFileRefnum				= kNoFileRefNum_AC;
const OSType kFolderType 				= kFolderType_AC;
const OSType kFolderCreator 			= kFolderCreator_AC;
const OSType kGenericType 				= kGenericType_AC;
const OSType kGenericCreator 			= kGenericCreator_AC;

#define kItem1GreaterThanItem2			kItem1GreaterThanItem2_AC
#define kItem1LessThanItem2				kItem1GreaterThanItem2_AC
#define kItem1EqualItem2				kItem1EqualItem2_AC
#define kEmptyIndex						kEmptyIndex_AC
#define kIterateForward					kIterateForward_AC
#define kIterateBackward				kIterateBackward_AC

#if qDebug
const SignedByte kNewInitVal 			= kNewInitVal_AC;			// debugging constant, odd at all byte boundaries 
const SignedByte kResizedPtrInitVal		= kResizedPtrInitVal_AC;		// debugging constant, odd at all byte boundaries 
const SignedByte kResizedInitVal 		= kResizedInitVal_AC;		// debugging constant, odd at all byte boundaries 
const SignedByte kDisposedInitVal 		= kDisposedInitVal_AC;		// debugging constant, odd at all byte boundaries 
#endif

// global functions
#define MABlockMove 					BlockMove_AC
#define MABlockMoveOverlap 				BlockMoveOverlap_AC

#define GetEnvironsVersion() 			GetEnvironsVersion_AC()
#define GetMachineType()				GetMachineType_AC()
#define GetSystemVersion()				GetSystemVersion_AC()
#define HasColorQD()					HasColorQD_AC()
#define Has32BitQD()					Has32BitQD_AC()
#define KeyboardType()					KeyboardType_AC()
#define GetATDrvrVersNum()				GetATDrvrVersNum_AC()
#define HasSCSI()						HasSCSI_AC()
#define HasAUX()						HasAUX_AC()
#define HasTempMem()					HasTempMem_AC()
#define GetTEVersion()					GetTEVersion_AC()
#define HasDesktopBus()					HasDesktopBus_AC()
#define HasTrueType()					HasTrueType_AC()
#define HasThreadManager()				HasThreadManager_AC()
#define HasQDGXGraphics()				HasQDGXGraphics_AC()
#define HasQDGXPrinting()				HasQDGXPrinting_AC()
#define HasCFM()						HasCFM_AC()
#define HasTranslationManager()			HasTranslationManager_AC()
#define HasSpeechManager()				HasSpeechManager_AC()
#define HasCustomFile()					HasCustomFile_AC()
#define HasQuickTime()					HasQuickTime_AC()
#define HasTSM()						HasTSM_AC()
#define HasAppleScript()				HasAppleScript_AC()
#define HasAppleGuide()					HasAppleGuide_AC()
#define HasQuickDraw3D()				HasQuickDraw3D_AC()
#define HasQuickDraw3DViewer()			HasQuickDraw3DViewer_AC()
#define HasDragManager()				HasDragManager_AC()
#define HasAOCEToolBox()				HasAOCEToolBox_AC()
#define NewList()						NewList_AC()

//	// Removed due to ASLM removal vs. deinstall problems.
//	bool HasASLM();

#define Min(a,b)    					Min_AC((a), (b))   
#define Max(a,b)     					Max_AC((a), (b))   
#define MinMax(	minVal, \
				expression, maxVal)     MinMax_AC((minVal), (expression), (maxVal))   
 
#define DisposeIfHandle(aHandle) 		DisposeIfHandle_AC((aHandle))
#define DisposeIfPtr(aPtr)			DisposeIfPtr_AC((aPtr))
#define BlockSet(destPtr,byteCount,\
				setVal)					BlockSet_AC((destPtr), (byteCount), (setVal))
#define IsAResource(h) 					IsAResource_AC((h))


#define IsHandleLocked(h) 				IsHandleLocked_AC((h))
#define CopyHandle(inHandle, outHandle) CopyHandle_AC((inHandle),( outHandle))
#define MACurResFile() 					CurResFile_AC()
#define MAUseResFile(refNum) 			UseResFile_AC((refNum))

inline SignedByte LockHandle(Handle h, EMoveHiState high = eDontMoveHi)
{	return LockHandle_AC(h, high); } // can't have default params with a macro

inline void MATextBox(Ptr text,
			   short itsLength,
			   const CRect_AC& box,
			   short itsJust,
			   bool autoWrap = kAutoWrap,
			   ProcPtr wordBreak = NULL,
			   bool eraseFirst = kNoEraseFirst,
			   bool spaceForCaret = kSpaceForCaret,
			   bool preferOutline = kDontPreferOutlineFonts)	
{ TextBox_AC(text, itsLength, box, itsJust, autoWrap, wordBreak, eraseFirst, spaceForCaret, preferOutline); } 
// can't have default params with a macro

#if qDebug
#define IsHandle(h)						IsHandle_AC((h))
#endif

//========================================================================================
// The following global routines were originally for Pascal compatibility. They are basically
// wrappers that call back into the methods in VRect and VPoint. C++ programs should use
// the methods in VRect and VPoint to avoid the additional function call overhead.
//========================================================================================

EXPORT_MA void PtToVPt(const CPoint_AC thePt, VPoint& theVPt);
EXPORT_MA CPoint_AC VPtToPt(const VPoint& theVPt);
EXPORT_MA void RectToVRect(const CRect_AC& theRect, VRect& theVRect);
EXPORT_MA void VRectToRect(const VRect& theVRect, CRect_AC& theRect);
EXPORT_MA void AddVPt(const VPoint& srcVPt, VPoint& dstVPt);
EXPORT_MA void SubVPt(const VPoint& srcVPt, VPoint& dstVPt);
EXPORT_MA void SetVPt(VPoint& vPt, VCoordinate h, VCoordinate v);
EXPORT_MA bool EqualVPt(const VPoint& aVPt, const VPoint& bVPt);
EXPORT_MA void SetVRect(VRect& vRt,
			  VCoordinate left, VCoordinate top,
			  VCoordinate right, VCoordinate bottom);
EXPORT_MA void OffsetVRect(VRect& vRt, VCoordinate dh, VCoordinate dv);
EXPORT_MA void InsetVRect(VRect& vRt, VCoordinate dh, VCoordinate dv);
EXPORT_MA void Pt2VRect(const VPoint& theTopLeft, const VPoint& theBotRight, VRect& vRt);
EXPORT_MA bool PtInVRect(const VPoint& vPt, const VRect& vRt);
EXPORT_MA bool EmptyVRect(const VRect& vRt);
EXPORT_MA bool EqualVRect(const VRect& aVRt, const VRect& bVRt);
EXPORT_MA VCoordinate LengthVRect(const VRect& vRt, VHSelect whichDim);
EXPORT_MA void PinVRect(const VRect& vRt, VPoint& vPt);
EXPORT_MA bool SectVRect(const VRect& src1, const VRect& src2, VRect& dst);
EXPORT_MA void UnionVRect(const VRect& src1, const VRect& src2, VRect& dst);

// These two were moved from UMacAppUtilities.h

inline bool RectsNest(const CRect_AC& outer, const CRect_AC& inner)
{ return outer.Contains(inner); }

inline bool VRectsNest(const VRect& outer, const VRect& inner)
{ return outer.Contains(inner); }


//========================================================================================

#endif
	// qBackwardCompatible

#endif
	// __Compatibility__
	
