// MacAppTypes.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

// This unit implements a set of simple type declarations.

#ifndef __MACAPPTYPES__
#define __MACAPPTYPES__

// MacApp

#ifndef __MacAppRequired__
#include "MacAppRequired.h"
#endif

#ifndef __PASCALSTRING__
#include "CPascalString_AC.h"
#endif

#ifndef __FileTypes_AC__
#include "FileTypes_AC.h"
#endif

#ifndef __UCLASSDESC__
#include "UClassDesc.h"
#endif

// Toolbox

#ifndef __TYPES__
#include <Types.h>
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

//------------------------------------------------------------------------------------------------
// Long and short integer size limits.
//------------------------------------------------------------------------------------------------

// You should change from using the MacApp versions of these constants
// to the standard ones in <limits.h>. 

const short kMinShort = SHRT_MIN;
const short kMaxShort = SHRT_MAX;
const long kMinLong = LONG_MIN;
const long kMaxLong = LONG_MAX;


//------------------------------------------------------------------------------------------------
// The following macros make dealing with Pascal compatible sets (bitmaps basically)
// a little less painful.
//------------------------------------------------------------------------------------------------

#define macroAsSetElem(integral) (1 << (integral))

#define macroIn(set, elem) ((set) & (elem) ? true : false)

// Does A contains B?
// C = A XOR B identifies which bits are set in A or B but
//   not both i.e. identifies bits with different values.
// C AND B, out of the bits with differents values, which ones are set in C.
//   If this is true for any bits, then A does not contain B.

#define macroContains(container, containee) ((container) & ((container) ^ (containee)) ? false : true)

#define macroUnion(set1, set2) ((set1) | (set2))

#define macroIntersection(set1, set2) ((set1) & (set2))


//----------------------------------------------------------------------------------------
// Enumeration for ASCII Character Constants
//----------------------------------------------------------------------------------------

enum EAsciiControlCode {
	chBackspace = 8,					// ASCII code for Backspace character  
	chClear = 27,						// ASCII code for Clear key (aka ESC)  
	chDown = 31,						// ASCII code for down arrow  
	chEnd = 4,							// ASCII code for the End key  
	chEnter = 3,						// ASCII code for Enter character  
	chEscape = 27,						// ASCII code for Escape (aka Clear) key  
	chFunction = 16,					// ASCII code for any function key  
	chFwdDelete = 127,					// ASCII code for forward delete  
	chHelp = 5,							// ASCII code for Help key  
	chHome = 1,							// ASCII code for the Home key  
	chLeft = 28,						// ASCII code for left arrow  
	chPageDown = 12,					// ASCII code for Page Down key  
	chPageUp = 11,						// ASCII code for Page Up key  
	chReturn = 13,						// ASCII code for Return character  
	chRight = 29,						// ASCII code for right arrow  
	chSpace = 32,						// ASCII code for Space character  
	chTab = 9,							// ASCII code for Tab character  
	chUp = 30							// ASCII code for up arrow
};


//----------------------------------------------------------------------------------------
// Enumeration for Virtual Key Code Constants
//----------------------------------------------------------------------------------------

enum EVirtualKeyCode {
	kClearVirtualCode = 71,				// Clear key virtual code  
	kEscapeVirtualCode = 53,			// Escape key virtual code  
	kF10VirtualCode = 109,				// F10 virtual key code  
	kF11VirtualCode = 103,				// F11 virtual key code  
	kF12VirtualCode = 111,				// F12 virtual key code  
	kF13VirtualCode = 105,				// F13 virtual key code  
	kF14VirtualCode = 107,				// F14 virtual key code  
	kF15VirtualCode = 113,				// F15 virtual key code  
	kF1VirtualCode = 122,				// F1 virtual key code  
	kF2VirtualCode = 120,				// F2 virtual key code  
	kF3VirtualCode = 99,				// F3 virtual key code  
	kF4VirtualCode = 118,				// F4 virtual key code  
	kF5VirtualCode = 96,				// F5 virtual key code  
	kF6VirtualCode = 97,				// F6 virtual key code  
	kF7VirtualCode = 98,				// F7 virtual key code  
	kF8VirtualCode = 100,				// F8 virtual key code  
	kF9VirtualCode = 101,				// F9 virtual key code  
	kFwdDelVirtualCode = 117			// Forward Delete virtual code
};


extern const char* kSysFontName;				//  GetFontNum converts this to zero.
extern const char* kApplFontName;				//  GetFontNum converts this to one.

extern const char* kHexDigits;					//  Digits in base 16


const long kNoStaticLink = -1;
#define NOSTATICLINK	MAstatic_cast(void*, kNoStaticLink)
	// Compiler note: CW5 won't let me init this const as a void* in a pre-compiled header.
	
	// Functions and procedures written in Pascal can take "procedure" parameters which are like C++
    // function pointers with the additional capabilitly to access upscope variables. This
    // is particularly convenient for "call back" type procedures because they are
    // executed in their original scope. HOWEVER, when a static link is not used or
	// required; C++ users must insert a non-nil placeholder. When the receiving code is in
	// pascal it includes special detection of the case where nil is passed for static link. 
	// When nil is passed in, >>> NO <<< static link will be passed to the "procedure"
    // parameter when it is called! You can imagine the ensuing heartache for well meaning
    // C++ users as their stack gets tromped. The answer, my friends is to have C++ users
    // pass kNoStaticLink when they don't wish to use the static link when calling a function
    // written in Pascal with Pascal calling conventions that takes a procedure as a parameter.


const unsigned char kMANameSize = 255;			// Maximum size CPascalStr_AC supported for
                                                // an MAName

//----------------------------------------------------------------------------------------
// Highlight constants: These constants can be used to test for combinations of
// fromHL+toHL, when you do not care which is from and which is to
//----------------------------------------------------------------------------------------

enum EHighLight {
	hlOff = 1,							// the selection is to be unhighlighted
	hlDim = 2,							// the selection is to be dimly highlighted
	hlOn = 4,							// the selection is to be fully highlighted
	hlOffDim = hlOff + hlDim,
	hlDimOff = hlOffDim,
	hlDimOn = hlDim + hlOn,
	hlOnDim = hlDimOn,
	hlOffOn = hlOff + hlOn,
	hlOnOff = hlOffOn
};

//----------------------------------------------------------------------------------------
// EVENT/COMMAND NUMBERS  
//----------------------------------------------------------------------------------------
typedef long EventNumber;

typedef EventNumber CommandNumber;

typedef CommandNumber* CommandNumberPtr;

// CmdNumbers tell MacApp what verb is intended after mapping from menu/item.

// Special command codes  
const EventNumber cNoEvent = 0;					// <!> Event number representing no event
const CommandNumber cNoCommand = cNoEvent;		// <!> Command number representing no
                                                // command   
const CommandNumber cCantUndo = -1;				// <!> Passed to TApplication::SetUndoText
                                                // to indicate that the command cannot be
                                                // undone.

//----------------------------------------------------------------------------------------
// Declarations of pointers to commonly used types.  
//----------------------------------------------------------------------------------------

typedef short* IntegerPtr;
typedef IntegerPtr* IntegerHandle;

typedef long* LongIntPtr;
typedef LongIntPtr* LongIntHandle;

typedef Handle* HandlePtr;
typedef HandlePtr* HandleHandle;

typedef bool* BooleanPtr;
typedef BooleanPtr* BooleanHandle;

typedef bool* boolPtr;
typedef boolPtr* boolHandle;

typedef SignedByte* SignedBytePtr;
typedef SignedBytePtr* SignedByteHandle;

typedef CStr255_AC* String255Ptr;
typedef String255Ptr* String255Handle;

typedef char String8[9];

typedef CStr255_AC MAName;
	// A Name in MacApp. ClassNames, MethodNames, IVar Names, ProcNames, etc. ??? Should
    // we punt on stringlength and only define pointers to these strings?

typedef MAName* MANamePtr;

typedef MANamePtr* MANameHandle;

typedef SInt16 ResNumber;
typedef ResType IDType;
	// MacApp uses four byte "signatures" in several places. They are a handy way to have
    // a partially human readable quantity that only takes up a long.

typedef short MenuID;
	// Every menu has an ID number; this is the number returned by MenuSelect, for example
	// The MenuID of a menu does not have to equal its resource ID, although it often does.

typedef OSType* TypeListPtr;
typedef OSType** TypeListHandle;


//----------------------------------------------------------------------------------------
// Enumeration for specifying standard adornments. These are used to operate on Pascal
// sets, thus the setting of explicit values for each identifier in the enumeration.
//----------------------------------------------------------------------------------------

enum AdornPieces
{
	adnLineTop = 	0x01,			// Draw a line at the top of the extent
	adnLineLeft = 	0x02,			// Draw a line on the left side of the extent
	adnLineBottom =	0x04,			// Draw a line on the bottom side of the extent
	adnLineRight = 	0x08,			// Draw a line on the right side of the extent
	adnDummy = 		0x10,			// Place filler
	adnOval = 		0x20,			// Do a FrameOval using the extent
	adnRRect = 		0x40,			// Do a (16,16) FrameRoundRect using the extent
	adnShadow = 	0x80			// Draw drop shadows against framed selections
};

typedef char CntlAdornment;



enum TrackPhase
{
	trackBegin,
	trackContinue,
	trackEnd
};

//----------------------------------------------------------------------------------------
// H I G H L I G H T I N G
//----------------------------------------------------------------------------------------

typedef char HLState;

#endif // __MACAPPTYPES__
