// PlatformMemory.h 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.

#ifndef PLATFORMMEMORY_H
#define PLATFORMMEMORY_H

// MacApp

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

// ANSI

#ifndef __STDDEF__
#include <stddef.h>
#endif

//========================================================================================
// Constants
//========================================================================================

#if qDebug
#define DEBUG 1
#endif

//========================================================================================
// Debugging
//========================================================================================

EXPORT_MA void PLATFORM_ASSERT(bool condition);

inline void PLATFORM_DEBUGGER_STRING(const CStr255_AC& string)
{
	ProgramBreak_AC(string);
}

EXPORT_MA void PLATFORM_PRINT_STRING(const char* string);

//========================================================================================
// Global function definitions
//========================================================================================

//----------------------------------------------------------------------------------------
// PlatformAllocateBlock
//----------------------------------------------------------------------------------------

EXPORT_MA  Handle MAAllocateHandle(size_t size);
// Definition moved to PlatformMemory.cp

#if 0
removed as part of OD MemMgr upgrade
//----------------------------------------------------------------------------------------
// PlatformFreeBlock
//----------------------------------------------------------------------------------------

inline void PlatformFreeBlock(void *ptr)
{
	DisposePtr((Ptr) ptr);
}
#endif

//----------------------------------------------------------------------------------------
// PlatformCopyMemory
//----------------------------------------------------------------------------------------

#if 0
removed as part of OD MemMgr upgrade
inline void PlatformCopyMemory(void *source, void *destination, size_t length)
{
	BlockMove_AC(source, destination, (Size) length);
}
#endif

//----------------------------------------------------------------------------------------
// TempAllocation assistance.
// This is used by PlatformMemory, but that is not part of ACS yet, so this lives here for now.
//----------------------------------------------------------------------------------------

EXPORT_MA bool TempObjectAllocation(bool allocateFromTemp);
	// Set to true to make object allocation calls use temporary memory (if they cannot be
    // allowed to fail). Set to false to make object allocation calls use permanent memory (the default). 
    // Returns old state.
	
EXPORT_MA bool GetTempObjectAllocationState();
	// Return the state of object allocation.  False for permanent, true for temporary.

//----------------------------------------------------------------------------------------
// Manage AllocateObjectsFromTemp State
//----------------------------------------------------------------------------------------
class EXPORT_MA CTempObjectAllocation
{
public:
	inline CTempObjectAllocation(bool allocateFromTemp) : fOldState(TempObjectAllocation(allocateFromTemp)) { }
	// Sets the fData to itsData

	inline ~CTempObjectAllocation() { TempObjectAllocation(fOldState); }
	// Deallocates the data

	inline operator bool&() { return fOldState; }
		// Conversion operator, for converting data to desired return type.
	
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	bool	fOldState;
};

#endif
