// UAssociation.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UASSOCIATION__
#define __UASSOCIATION__

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TEntry;
class TEntriesList;


//----------------------------------------------------------------------------------------
// TEntry: This class associates a key CPascalStr_AC with a value CPascalStr_AC.
//----------------------------------------------------------------------------------------

class EXPORT_MA TEntry
{
public:
	TEntry(const CStr255_AC& itsKey = "", const CStr255_AC& itsValue = "");
		// Constructor
		
	virtual ~TEntry();
		//  Frees the fKey & fValue CPascalStr_AC handles then calls Inherited::Free

	virtual void SetValue(const CStr255_AC& value);
		//  Sets the fValue field to theValue

public:
	MA_operator_delete_auto_ptr<CStr255_AC> fKey;				// A pointer to the 'keyStr'
	MA_operator_delete_auto_ptr<CStr255_AC> fValue;			// A pointer to the 'valueStr'	
};


//----------------------------------------------------------------------------------------
// TEntriesList: Manages a sorted list of TEntry objects,
// used as an instance variable in TAssociation
//----------------------------------------------------------------------------------------

class EXPORT_MA TEntriesList : public CSortedList_AC
{
public:
	inline TEntriesList() : CSortedList_AC() { }
		// constructor

	virtual ~TEntriesList();
		// Destructor
		
	virtual void* Clone() const;
		// Clone

	virtual CompareResult_AC Compare(const void* item1,
									const void* item2) const;
		// Compares 'item'1 with 'item2' returning an integer indicating the results of the comparison
};


//----------------------------------------------------------------------------------------
// TAssociation: This class is used to create lists of strings that are accessed via their
// associated keys. The keys & strings are stored in TEntry objects which are in the
// TEntriesList instance variable
//----------------------------------------------------------------------------------------

class EXPORT_MA TAssociation
{
public:
	TAssociation();
		// Constructor
		
	virtual ~TAssociation();
		//  Frees the instance variable fEntries

	virtual bool ValueAt(const CStr255_AC& keyStr, CStr255_AC& valueStr);
		// Given 'keyStr' returns the associated CPascalStr_AC in 'valueStr', with the result
        // true if an associated CPascalStr_AC was found

	virtual CStr255_AC ValueAt(const CStr255_AC& keyStr);
		// Given 'keyStr' returns the associated valueStr. returns empty string
		// for not found (or for empty strings, watch out!)

	virtual bool KeyAt(const CStr255_AC& valueStr, CStr255_AC& keyStr);
		// Given 'valueStr' returns the associated key in 'keyStr', with the result true
        // if an associated key was found

	virtual CStr255_AC KeyAt(const CStr255_AC& valueStr);
		// Given 'valueStr' returns the associated key. returns empty string
		// for not found (or for empty strings, watch out!)

	virtual TEntry* EntryWithKey(const CStr255_AC& keyStr);
		//  Returns the Entry containing 'keyStr'

	virtual TEntry* EntryWithValue(const CStr255_AC& valueStr);
		//  Returns the entry containing 'valueStr'

	virtual void InsertEntry(const CStr255_AC& keyStr, const CStr255_AC& valueStr);
		// Inserts an entry into thei association, if 'keyStr' already exists then the
        // value is merely replaced with 'valueStr'. Otherwise a new Entry is created with
        // the above 'keyStr' and 'valueStr'

	virtual void RemoveValueAt(const CStr255_AC& keyStr);
		// Removes the value associated with 'keyStr'

	virtual void RemoveKeyAt(const CStr255_AC& valueStr);
		// Removes the key associated with 'valueStr'

public:
	TEntriesList fEntries;			// The list of TEntries that stores the associations
};


#endif


