// UCoreErrorMgr.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __UCOREERRORMGR__
#define __UCOREERRORMGR__

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

// Toolbox

//----------------------------------------------------------------------------------------
// Alerts
//----------------------------------------------------------------------------------------
enum
{
	errReasonID = 128,						// resource ID of errs resource describing error messages
	errRecoveryID = 129,					// resource ID of errs resource describing recovery messages
	errOperationsID = 130					// resource ID of errs resource containing operation strings
};

enum
{
	errAppTable = 1000,							// added to the MacApp resource ID to get
                                                // the ID of an application error table
	messageCommandError = 0,					// when added to a command number, gives a
                                                // message value to display the alert
                                                // "Could not complete the command "
	messageAlert = -65536,						// when added to an alert number, gives a
                                                // message value to display that alert
	messageLookup = -131072,					// when added to an INTEGER, gives a
                                                // message value that will look up the
                                                // INTEGER in table errOperationsID and
                                                // display the alert "Could not because "
	messageAlternateRecovery = -196608,			// same as messageLookup, except the INTEGER
                                                // is used instead of the error number to
                                                // lookup the recovery CPascalStr_AC
	messageStringList = 130 * 65536,			// Standard list of operation strings used
                                                // by MacApp * $10000 to plunk it into hiwrd
	messageCancelled = messageStringList,			// used with error = noErr if user cancels
                                                // Save, Quit,

	messageInitializationFailed,
	messageSaveFailed,
	messageRevertFailed,
	messagePrintFailed,
	messageNewFailed,
	messageOpenFailed,
	messageSaveAsFailed,
	messageSaveCopyFailed,
	messageDrawFailed,
	messageImportClipFailed,
	messageExportClipFailed
};

//----------------------------------------------------------------------------------------
// error codes used in MacApp, applications should use errors between -26000 and -29999
//----------------------------------------------------------------------------------------
enum
{
	errSpooling = -20000,					// print spooling failed OSErr
	errRevertFNF = -20001,					// Revert failed because file was deleted
	errFileChanged = -20002,				// disk file modification date changed
	errSaveAgain = -20003,					// Save As/ Save a Copy in an opened document
	errFTypeChanged = -20004,				// disk file type changed
	errNoPrintDrvr = -20005,				// print driver file not found
	errNotMyType = -20006,					// can't open this file type
	errNotImplemented = -20007,				// Signaled when a function is not yet implemented
	errMissingClass = -20008,				// Signaled when a class is missing.
											// Usually due to failure to dead-strip
                                            // suppress when creating objects any
											// other way than by "NEW"
	errMissingSignature = -20009,			// Signaled when a signature has no associated class
	errPrintDriverBusy = -20010,			// Print driver is in use by another application
	errGenericErrorWithNum = -20011,		// Generic error message with substitution of
											// or error code in resulting string.
	errNoPagesToPrint = -20012,				// There were no pages to print.
	errTooManyChars = -20013,				// no more characters can be added here.

//----------------------------------------------------------------------------------------
// AppleEvent related error codes used in MacApp,  [-21000 and -21999]
//----------------------------------------------------------------------------------------
	errAECantClone = -21001,
	errAECantClose = -21002,
	errAECantCount = -21003,
	errAECantCreate = -21004,
	errAECantDelete = -21005,
	errAECantExist = -21006,
	errAECantGetClassInfo = -21007,
	errAECantGetEventInfo = -21008,
	errAECantGetData = -21009,
	errAECantGetDataSize = -21010,
	errAECantMove = -21011,
	errAECantOpen = -21012,
	errAECantPrint = -21013,
	errAECantSave = -21014,
	errAECantSetData = -21015,
	errAECantRevert = -21016,
	errAECantSetReadOnly = -21017,
	errAENoSuchProperty = -21018
};

//----------------------------------------------------------------------------------------
// Alerts 0 - 249 are reserved by MacApp.
//----------------------------------------------------------------------------------------
enum
{
	phGenError = 128,							// 'Could not ^2, because ^0. ^1'
	phCommandError = 129,						// 'Could not complete the ^2 command because ^0. ^1'.
	phUnknownErr = 130,							// 'Could not complete your request
                                                // because ^0. ^1', only used if no
                                                // operation is supplied.
	phSaveChanges = 131,						// 'Save changes to the ^2 document ^0 before ^1?'
	phRevert = 132,								// 'Revert to the last saved version of ^0?'
	phFileChanged = 133,						// 'File changed since last save.'
	phPurgeOld = 134,							// 'OK to purge old version before saving new one?'
	phReopenDoc = 135,							// 'Document is already open.'
	phSpaceIsLow = 136,							// 'Space is low'
	phUnsupportedConfiguration = 137,			// 'Can't start app because your machine is too wimpy!'
	phOfferReadOnly = 138,						// 'Can't open for file for write. Open read-only?'
	phNewPublisherAlert = 140,					// Used by TEditionDocument when closing
                                                // the document if there are new
                                                // publishers but the document's DATA is unchanged.
	phMultPublisherWrn = 141,					// Used by TEditionDocument when creating
                                                // multiple publishers to the same edition
                                                // container file.
	phSavingMultPublisherWrn = 142,				// Used by TEditionDocument when closing
                                                // the document if there are multiple
                                                // publishers to the same edition
                                                // container file.
	phTooManyChars = 150,						// 'Too Many Characters', used to reject
                                                // paste or keystrokes in UDialog and
                                                // UTEView
	phStylesTooBig = 151,						// 'Too Many Styles', used to reject the
                                                // styles portion of a paste in UTEView
	phAboutApp = 201,							// Can hold the 'About <appName>' message
	phUnimplemented = 202						// 'This feature not yet implemented'
};

//----------------------------------------------------------------------------------------
// Global variables
//----------------------------------------------------------------------------------------

EXPORT_MA extern CStr255_AC gErrorParm3;
	// This is used as the last argument to ParamText in ErrorAlert, if ErrorAlert
    // displays one of the standard alerts. (This CPascalStr_AC will replace ^3 in those alerts.)
    // ErrorAlert also sets this to '' when called. You can use this to parameterize the
    // automatic alerts that MacApp displays. For example, TDocument::ReadFromFile sets
    // this to the document name.

//----------------------------------------------------------------------------------------
// Global routines
//----------------------------------------------------------------------------------------

EXPORT_MA extern bool LookupErrString(short value, ResNumber resourceID, CStr255_AC& str);
	// Looks up an error value in an 'errs' resource and returns true if found. resourceID
    // should be that of a MacApp errs resource; we do the lookup first in
    // errAppTable+resourceID, which applications can use to override or extend the MacApp
    // table. If the value is not found, str will be empty.

EXPORT_MA void NotYetImplemented(const CStr255_AC& where);
	// Signals the notYetImplemented failure which when caught puts up an alert saying
    // "Not Yet Implemented" and in debug WriteLns where.

#endif // __UCOREERRORMGR__
