// UCoreGlobals.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UCOREGLOBALS__
#define __UCOREGLOBALS__

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __TOOLBOX__
#include "Toolbox.h"
#endif

// Toolbox

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __PROCESSES__
#include <Processes.h>
#endif

//----------------------------------------------------------------------------------------
//  Menu constants
//----------------------------------------------------------------------------------------
enum
{
	kMBarDisplayed = 128,			// Default 'MBAR' id of the menus that are read in and installed in menu bar
	kMBarNotDisplayed = 129,		// Default 'MBAR' id of the menus that are read in but not installed
	kMBarHierarchical = 130,		// Default 'MBAR' id of the menus that pop up when a menu item is choosen
// Added for 3.5
	kMBarOpenDoc = 131,				// Default 'MBAR' id of the menus that are displayed running under OpenDoc
	kMBarDebug = 900				// Default 'MBAR' id of the Debug menus
};

//----------------------------------------------------------------------------------------
// Command Numbers: Negative numbers and positive numbers up to 999 are reserved by
// MacApp.
//
// Not all of these are caught by MacApp:
//
// "<!>" means MacApp catches it.
// "<&>" means TEView catches it.
// "<->" means the application must catch it if it is used as a command number in any
// 			menu.

//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// Apple-menu commands
//----------------------------------------------------------------------------------------
enum
{
	cAboutApp = 1				// <!> "About <appname>"
};


//----------------------------------------------------------------------------------------
// File-menu filing commands
//----------------------------------------------------------------------------------------
enum
{
	cNew = 10,					// <!> "New" (??? reserve a range of NEW commands ???)
	cNewLast = 19,
	cOpen = 20,					// <!> "Open " (??? reserve a range of OPEN commands ???)
	cOpenLast = 29,
// Added for 3.5
	cInsert = 2004,				// <-> "Insert"
	cSave = 30,					// <!> "Save"
	cClose = 31,				// <!> "Close"
	cSaveAs = 32,				// <!> "Save as"
	cSaveCopy = 33,				// <!> "Save a Copy in "
	cRevert = 34,				// <!> "Revert" (to previous version)
	cShowClipboard = 35,		// <!> "Show Clipboard"/ "Hide Clipboard"
	cQuit = 36					// <!> "Quit"
};

//----------------------------------------------------------------------------------------
// Edit-menu commands: For the following command numbers, we must guarantee that <command
// number> - cEditBase = <appropriate number to pass to SystemEdit>. This relationship is
// enforced in TApplication::IApplication.
//----------------------------------------------------------------------------------------

enum
{
	cEditBase = 101,			// start of standard editing commands
	cUndo = 101,				// <!> "Undo <command>"
	cRedo = 102,				// <!> "Redo <command>"
	cUndoRedo = 132,			// <!> "Undo/Redo <command>" (for single-undo)
	cCut = 103,					// <&> "Cut"
	cCopy = 104,				// <&> "Copy"
	cPaste = 105,				// <&> "Paste"
	cClear = 106,				// <-> "Clear"
	cEditLast = cClear,
	cSelectAll = 110,			// <&> "Select All"
	cTyping = 120,				// for use in a TTypingCommand
	cMouseCommand = 121,		// generic mouse command
	cStyleChange = 130,			// "Menu" command for style change
	cRememberStyle = 131		// "Menu" command for remembering present style
};

//----------------------------------------------------------------------------------------
// AppleEvent support
//----------------------------------------------------------------------------------------
enum
{
	cAppleEventReply = 40		// General command number used to identify asynchronous replies
};

//----------------------------------------------------------------------------------------
// Finder pseudo-commands
//----------------------------------------------------------------------------------------
enum
{
	cFinderNew = 41,			// <!> The user selected the tool icon in the Finder and chose "Open"
	cFinderPrint = 42,			// <!> The user selected document icons in the Finder and chose "Print"
	cFinderOpen = 43,			// <!> The user selected document icons in the Finder and chose "Open"
	cFinderQuit = 44			// <!> The user selected "ShutDown" and then the Finder sends a 'quit' AppleEvent
};

//----------------------------------------------------------------------------------------
// EDITION MANAGER AppleEvent COMMAND CONSTANTS
//----------------------------------------------------------------------------------------
enum
{
	cSectionRead = 50,			// subscribers should read themselves afresh from disk
	cSectionWrite = 51,			// publishers should write themselves to an edition file
	cSectionScroll = 52,		// the user selects Open Publisher for a subscriber
	cSectionCancel = 53		// cancel this section in a document
};

//----------------------------------------------------------------------------------------
// Object Model AppleEvent Command Numbers
//----------------------------------------------------------------------------------------
enum
{
//  Command numbers for the core suite.
	cAEClone = 200,
	cAEClose = 201,
	cAECountElements = 202,
	cAECreateElement = 203,
	cAEDelete = 204,
	cAEDoObjectsExist = 205,
	cAEGetClassInfo = 206,
	cAEGetData = 207,
	cAEGetDataSize = 208,
	cAEGetEventInfo = 209,
	cAEMove = 210,
	cAEOpen = cFinderOpen,
	cAEPrint = cFinderPrint,
	cAEQuit = cFinderQuit,
	cAESave = 214,
	cAESetData = 215,
	cAERevert = cRevert,

//  Command numbers for recording
	cAERecordingOn = 216,
	cAERecordingOff = 217,

//  Command numbers for the misc suite.
	cAEUndo = 218,
	cAERedo = 219,
	cAEPaste = 220,
	cAECut = 221,
	cAECopy = 222,

//  Command numbers for the mail suite.
	cAESend = 223,
	cAEOpenNextLetter = 224,


//  Command number for opening a letter from the mailbox.
	cOpenLetter = cOpenLast - 1
};

//----------------------------------------------------------------------------------------
// PowerTalk Mail Command Numbers
//----------------------------------------------------------------------------------------
enum
{
	cAddMailer = 650, // add a mailer to the window
	cSend = 651, // send a letter
	cReply = 652, // reply to a letter
	cForward = 653, // forward a letter
	cOpenNextLetter = 654, // open the next letter from the mailbox 

	typeLetterDesc = 'lttr',
	kStandardLetterFileType = typeLetterDesc
};

//----------------------------------------------------------------------------------------
// File-menu printing commands
//----------------------------------------------------------------------------------------
enum
{
	cPrFileBase = 176,			// Command numbers between cPrFileBase and
	cPrFileMax = 195,			// cPrFileMax are reserved for printing.
	cPageSetup = 176,			// <!> "Page Setup"
	cCustomPageSetup = 181,		// <!> "Custom Page Setup" (for GX)
	cDragPrint = 181,			// <-> Dragged document[s] to desktop printer
												// like the old cFinderPrint except that it
												// bypasses the job dialog and performs
												// like cPrintOne
	cPrint = 178,				// <!> "Print"
	cPrintOne = 177,			// <!> "Print One Copy"
	cPrintSelection = 180,		// <-> "Print Selection"
	cPrintToFile = 179,			// <!> "Print to file"
	cPrintSpoolFile = 190,		// <-> "Print spooled file"
	cPrViewBase = 201,			// Command numbers between cPrViewBase and
                                                // cPrViewMax are printing commands
                                                // applied to a displayed view in the
                                                // fTarget chain
	cPrViewMax = 250,
	cShowBorders = 199			// <!> Toggle. "Show view borders"--also usable outside Debug menu
};

//----------------------------------------------------------------------------------------
// Zooming commands
//----------------------------------------------------------------------------------------
enum
{
	cReduce50 = 301,			// <-> "Reduce 50%"
	cReduceToFit = 302,			// <-> "Reduce to Fit"
	cShowFullSize = 303		// <-> "Show Full Size"
};

//----------------------------------------------------------------------------------------
// Drag & Drop commands
//----------------------------------------------------------------------------------------
enum
{
	cDrag = 350,				// Remove content from the drag source
	cDrop = 351,				// Add content to the drop target
	cDragMove = 352,			// Move content within a view
	cDragMoveOut = 353			// Move content Out of a view
};

//----------------------------------------------------------------------------------------
// Control tracking
//----------------------------------------------------------------------------------------
enum
{
	cTrackingControl = 400,
	cDragView = 410			// Drag a view within its parent.
};

//----------------------------------------------------------------------------------------
// AOCE Strings - StringResource
//----------------------------------------------------------------------------------------
enum
{
	kAOCEStrings = 255,		// AOCE strings STR# resource
	
	kGetIdentityPrompt = 1,
	kAddMailerCmdName = 2,
	kRemoveMailerCmdName = 3,
	
	kAppleTalkRecordType = 128,
	kDirExtTypeTableID = 128
};

//----------------------------------------------------------------------------------------
// Additional AE Constants
//----------------------------------------------------------------------------------------

enum
{
keyAESaveCopy = 'svCp'	// used as a parameter for the core suite save
										// event to support saving a copy of a document
};

//----------------------------------------------------------------------------------------
// Constants for the Mail AppleEvent Suite
//----------------------------------------------------------------------------------------
enum
{
	cMailAddress	 = 'mAdd',
	cMailer = 'mhdr',
	cRecipient	 = 'mrcc',
	cSender = 'msnd',
	cReplyDocument = 'mRpl',
	enumPriorityHigh	 = 'hiGh',
	enumPriorityLow = 'loW ',
	enumPriorityNormal	 = 'NOrm',
	kAEMailSuite	 = 'MAIL',
	kAESendLetter = 'send',
	kAEOpenNextLetter = 'oLet',
	kAEToRecipient	 = ' to ',
	kAECCRecipient	 = ' cc ',
	kAEBCCRecipient = 'bcc ',
	keyAEAddress	 = 'pAdd',
	keyAEIncludeAppleMailFmt = 	'lamf',
	keyAEIncludeMainEnclFmt = 'lmef',
	keyAEIncludeSnapShotFmt = 'lssf',
	keyAEPriority = 'PRTY',
	keyAESignWhenSent = 'SiGn',
	pAddress	 = keyAEAddress,
	pFrom = 'pfrm',
	pHasContent = 'phco',
	pIdentity = 'ocid',
	pMailAddress	 = keyAEAddress,
	pReceived	 = 'prec',
	pRecipientType	 = 'RCTy',
	pSendDateTime = 'psdt',
	pSubject	 = 'psub',
	typeMailAddress = 'mAdd'
};

//----------------------------------------------------------------------------------------
// MacAppBuzz - StringResource
//----------------------------------------------------------------------------------------

enum
{
	kEmptySTRIndex = 0,					// Empty value for STR# indices.

	kIDBuzzString = 250			// std list of buzzwords stored as 'STR#'
                                                // resources. This CPascalStr_AC list is reserved
                                                // for MacApp, you should choose a
                                                // different resource ID for your strings.
};

//----------------------------------------------------------------------------------------
// Indices for strings in the buzz-CPascalStr_AC resource
//----------------------------------------------------------------------------------------
enum
{
	bzSaveAs = 1,						// 'Save This Document As:'
	bzSaveCopy = 2,						// 'Save a Copy In:'
	bzShowClip = 3,						// 'Show Clipboard'
	bzHideClip = 4,						// 'Hide Cllpboard'
	bzUndo = 5,							// 'Undo x'
	bzRedo = 6,							// 'Redo x'
	bzCantDraw = 7,						// 'Unable to draw contents of window'
	bzUntitled = 8,						// name of untitled document, if blank
                                                // then this is taken from the window
                                                // title template, if this includes the
                                                // CPascalStr_AC '<<<>>>' then MacApp will
                                                // substitute a number for the CPascalStr_AC
                                                // (e.g., if the buzz CPascalStr_AC contains
                                                // 'Untitled-<<<>>>' then untitled windows
                                                // will be named 'Untitled-1', Untitled-2,
	bzClosing = 9,						// 'closing'
	bzQuitting = 10,					// 'quitting'
	bzCantUndo = 11,					// 'Cant Undo'
	bzSaveAnyways = 12,					// 'save'
	bzRevertAnyways = 13,				// 'revert'
	bzHideBorders = 14,					// 'Hide Borders'
	bzShowBorders = 15,					// 'Show Borders'
	bzSubscriberOptions = 16,			// 'Subscriber Options'
	bzPublisherOptions = 17,			// 'Publisher Options'
	bzFirstUntitled = 18,				// 'untitled'
	bzSaveCopyFileName = 19				// Pattern for making "Filename copy" default file name.
};

//----------------------------------------------------------------------------------------
// DebugBuzz - StringResource
//----------------------------------------------------------------------------------------
enum
{
	kDebugBuzzStrings = 251,		// Debug buzzwords STR# resource
	bzDoFirstClick = 1,					// 'Do First Click For This Window'
	bzDontDoFirstClick = 2,				// 'Don't Do First Click For This Window'
	bzSetRightSysJust = 3,				// Set system to right justification
	bzSetLeftSysJust = 4,				// Set system to left justification
	bzContinuePerfMon = 5,				// 'Continue performance monitor'
	bzPausePerfMon = 6					// 'Pause performance monitor'
};

//----------------------------------------------------------------------------------------
// Resource ID's
//
// Negative numbers and numbers up to 127 are reserved by Apple
// Positive numbers from 128 up to 999 are reserved by MacApp
//----------------------------------------------------------------------------------------
enum
{
	kDefaultCredits = 999,			// People responsible for the application
	kIDClipWindow = 200,			// window displaying the Clipboard
	kAEDispatchTable = 400,			// AppleEvents dispatch table
	kSystemFontTextStyle = 128,		// TextStyle information for System Font
	kApplFont9TextStyle = 129,		// TextStyle information for Application Font, 9 pt
	kApplFont12TextStyle = 130		// TextStyle information for Application Font, 12 pt
};

//----------------------------------------------------------------------------------------
// View Identifiers: These follow the same reservation rules as resource types
//----------------------------------------------------------------------------------------
enum
{
	kDebugFlagsView = 900,			// Debug flags view.
	kPerfMonInitView = 901,			// Performance monitor initialization dialog.
	kDefaultWindowID = 1001,		// Window view resource installed by default in TDocument
	kDefaultViewID = 1002,			// View resource installed by default in TDocument
	kIDDefaultView = 'DFLT',			// View ID of default view
	kIDClipView = 'CLIP'				// identifier of the clipboard "orhanage"
};

//----------------------------------------------------------------------------------------
// Object Signatures: These follow the same reservation rules as resource types The
// signature is used to search the signature table and create an object "by signature".
// If the signature's entry in the table has been updated with a new class then an object
// of that class will be created.
//----------------------------------------------------------------------------------------
enum
{
	kStdDocument = 'docu',
	kStdTracker = 'trak',
	kStdList = 'list',

	kStdButton = 'butn',
	kStdCheckBox = 'chkb',
	kStdCluster = 'clus',
	kStdClusterBehavior = 'club',
	kStdControl = 'cntl',
	kStdDefaultView = 'dflt',
	kStdDialogView = 'dlog',
	kStdDialogTEView = 'dtev',
	kStdDrawingEnvironment = 'drwe',
	kStdEditText = 'edit',
	kStdGridView = 'grid',
	kStdIcon = 'icon',
	kStdIconSuite = 'iste',
	kStdIncludeAt = 'inc@',
	kStdNumberText = 'nmbr',
	kStdPattern = 'patn',
	kStdPicture = 'pict',
	kStdPopup = 'popp',
	kStdRadio = 'radb',
	kStdScroller = 'scrl',
	kStdScrollBar = 'sbar',
	kStdScrollerScrollBar = 'ssbr',
	kStdSmallIcon = 'sicn',
	kStdStaticText = 'stat',
	kStdTEView = 'tevw',
	kStdTextGridView = 'txtg',
	kStdTextListView = 'lstg',
	kStdView = 'view',
	kStdWindow = 'wind',
	kStdTargetBorderView = 'tbor'
};

//----------------------------------------------------------------------------------------
// Choices: Negative numbers and positive numbers up to 999 are reserved by MacApp
//----------------------------------------------------------------------------------------
enum
{
	mOKHit = 1,
	mCancelHit = 2,
	mButtonHit = 3,
	mCheckBoxHit = 4,
	mClusterHit = 5,
	mEditTextHit = 6,
	mIconHit = 7,
	mListItemHit = 8,
	mListScrollBarHit = 9,
	mPictureHit = 10,
	mPopupHit = 11,
	mRadioHit = 12,
	mStaticTextHit = 13,
	mHScrollBarHit = 14,
	mVScrollBarHit = 15,
	mEditTabKey = 16,
	mEditReturnKey = 17,
	mEditEnterKey = 18,
	mPatternHit = 19,
	mControlHit = 20,
	mCancelKey = 21,
	mDefaultKey = 22,
	mSmallIconHit = 23,
	mBecameWindowTarget = 24,
	mResignedWindowTarget = 25,
	mBecameTarget = 26,
	mResignedTarget = 27,
	mClosed = 28,
	mAboutToLoseControl = 29,
	mScrapChanged = 30,
	mTurnOn = 31,
	mTurnOff = 32,
	mToggle = 33,
	mDismiss = 34,
	mFreeClipboardView = 35,
	mClipViewChanged = 36,
	mPrinterChanged = 37,
	mRegainControl = 39,
	mPrinterOpened = 40,
	mPrinterClosed = 41,
	mPrintInfoChanged = 42,
	mFrameChanged = 43,


	kNoIdentifier = '    ',
	kNoTemplate = -1,
	kNoResource = -1
};

//----------------------------------------------------------------------------------------
// Miscellaneous
//----------------------------------------------------------------------------------------
enum
{
	kDesignatorResType = 'DESG',		// the ResType for saving a designator to a resource
	kYesButton = 1,						// the 'Yes' button in various dialogs
	kNoButton = 3,						// the 'No' button in various dialogs
	kLowSpaceInterval = 2 * 60 * 60,	// default low space msg interval (2 seconds)
	kStdScrollUnit = 16,				// default value of fScrollUnit used by scrollers
	kSBarSize = 16,
	kSBarSizeMinus1 = kSBarSize - 1,
	kStdSzSBar = 16,					// width/height of a standard vertical/horizontal scroll bar
	kStdSzMinus1SBar = kStdSzSBar - 1,
	kStdStaggerAmount = 16,				// standard # of pixels to stagger windows
	kMaxIdleTime = LONG_MAX,			// Default tick count sent to WaitNextEvent
										// It is 2147483647 2 ticks, which is 414 days. That should be enough.
	kOpenHandCursor = 370,				// Open hand 'CURS' to indicate draggable content
	kClosedHandCursor = 371				// Closed hand 'CURS' used while drag is in progress
};

//----------------------------------------------------------------------------------------
// Drag Move determiner values.
//----------------------------------------------------------------------------------------
enum
{
	kNeverMove = 0,
	kMoveWithinView = 1,
	kMoveWithinWindow = 2,
	kMoveWithinContext = 3,
	kMoveWithinApplication = 4
};

//----------------------------------------------------------------------------------------
// Numbers to pass to SystemEdit
//----------------------------------------------------------------------------------------
enum
{
	kSysUndo = 0,
	kSysCut = 2,
	kSysCopy = 3,
	kSysPaste = 4,
	kSysClear = 5,

	kRsrcCheckInterval = 100			// If checking rsrc usage then check at every n (kRsrcCheckInterval) events.
};

const bool kBuild = true;					// Pass to TWindow::BuildWindowRegions
const bool kAllowApplicationToSleep = true;	// Pass to TApplication::PollToolboxEvent
                                                // if the application is allowed to sleep
                                                // if there are no events pending.


//----------------------------------------------------------------------------------------
// Printing
//----------------------------------------------------------------------------------------

const bool kFixedSize = true;				// Fixed Page Size in Printing

//----------------------------------------------------------------------------------------
// size in bytes of the overhead for a resource file, kRsrcFileOverhead is added in
// TFileHandler::DoNeedDiskSpace if the document uses the rsrc fork. You should add
// kRsrcTypeOverhead for each unique type you use plus kRsrcOverhead for each resource you
// use plus the total bytes in all the resources plus the length of all the names for
// named resources.
//----------------------------------------------------------------------------------------
enum
{
	kRsrcFileOverhead = 256 + 30,		// fixed part of resource map
	kRsrcTypeOverhead = 8,				// overhead for each resource type
	kRsrcOverhead = 16,					// overhead for each resource
	kViewRsrcExpandAmt = 1024			// ??? Is 1K enough for expansion, or too much maybe???
};

//----------------------------------------------------------------------------------------
// argument of TDocument::DoMakeViews
//----------------------------------------------------------------------------------------
const bool kForDisplay = false;
const bool kForPrinting = !kForDisplay;

//----------------------------------------------------------------------------------------
// arguments of TScroller::IScroller and TViewServer::NewPaletteWindow
//----------------------------------------------------------------------------------------
const bool kWantHScrollBar = true;
const bool kWantVScrollBar = true;

//----------------------------------------------------------------------------------------
// arguments of TFileHandler::SaveFile
//----------------------------------------------------------------------------------------
const bool kAskForFilename = true;
const bool kMakingCopy = true;
const bool kSwitchToTarget = !kMakingCopy;

//----------------------------------------------------------------------------------------
// argument of TApplication::SetUndoText
//----------------------------------------------------------------------------------------
const bool kShowUndo = true;	
const bool kShowRedo = false;	
const bool kShowCantUndo = kShowUndo;

//----------------------------------------------------------------------------------------
// arguments to routines with a 'redraw' parameter
//----------------------------------------------------------------------------------------
const bool kRedraw = true;
const bool kDontRedraw = false;

//----------------------------------------------------------------------------------------
// arguments to routines with an 'invalidate' parameter
//----------------------------------------------------------------------------------------
const bool kInvalidate = true;
const bool kDontInvalidate = false;

const bool kVisible = true;	
const bool kInvisible = false;	

//----------------------------------------------------------------------------------------
// Initialization
//----------------------------------------------------------------------------------------

EXPORT_MA bool ValidateConfiguration();

EXPORT_MA bool MAGestaltAttribute(OSType itsAttr, short itsBit);
	// Returns true if the gestalt is present and the bit specified by itsBit is on

EXPORT_MA bool MAHasGestalt(OSType itsGestalt);
	// Returns true if the gestalt is present

EXPORT_MA void InitCoreMacApp(short callsToMoreMasters);

EXPORT_MA OSErr FindProcessBySignature(OSType sig,
							 ProcessSerialNumber& psn,
							 CFSSpec_AC* fileSpec);
	// Search process list for app with this signature

#endif // __UCOREGLOBALS__
