// UCoreUtilities.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UCOREUTILITIES__
#define __UCOREUTILITIES__

// ACS

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

#ifndef __Configuration_AC__
#include "Configuration_AC.h"
#endif

#ifndef __ResFileUtils_AC__
#include "ResFileUtils_AC.h"
#endif

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

typedef void(* DoWithResFileType)(void* scopeLink);

//----------------------------------------------------------------------------------------
// Global constants declarations. Build options.
//----------------------------------------------------------------------------------------

EXPORT_MA extern const char* kCopyright;

//----------------------------------------------------------------------------------------
// Global variable declarations.
//----------------------------------------------------------------------------------------

EXPORT_MA extern  short gApplicationRefNum;

EXPORT_MA extern  short gCodeRefNum; 
	// Reference to where to find code segments (note this is the application
	// for PowerPCs)

EXPORT_MA extern  bool gToolBoxInitialized;

EXPORT_MA extern  bool gUMemoryInitialized;

EXPORT_MA extern  Ptr gStrippedAddress;

//----------------------------------------------------------------------------------------
// Global assembly inline function definitions.
//----------------------------------------------------------------------------------------

#if !qPowerPC && !qMSWindows
#ifdef __MWERKS__
	#pragma d0_pointers on
#endif

long GetParmBlockPtr() = {0x2008};				// MOVE.L A0,D0
												// Return the value of register A0.
												// Useful for getting the pointer
												// to the parameter block from a
												// VBL task or a completion routine.

long GetA5() = {0x200D};						// MOVE.L A5,D0
												// Return the value of register A5.
												// Useful for getting the immediate
												// value of A5 which is not always
												// the same as CurrentA5.  Generally
												// a pointer to the program's global
												// area and jump table.	

#if qDebug
Ptr GetCurStackFramePtr() = {0x200E};			// MOVE.L A6,D0
												// Return the value of register A6.
												// Usually a pointer to the local
												// stack frame.  Most often used to
												// find out the caller's name when
												// invoking a debugging routine.
	
Ptr GetCurStackTop() = {0x200F};				// MOVE.L A7,D0
												// Return the value of register A7.
												// Usually the top of the stack.
												// Useful for stack sniffing (not
												// a crime).
#endif

#ifdef __MWERKS__
	#pragma d0_pointers reset
#endif
#endif

//----------------------------------------------------------------------------------------
// Global function declarations that needs to always be compiled for 68000.
//----------------------------------------------------------------------------------------

EXPORT_MA TrapType GetTrapType(short theTrap);

EXPORT_MA bool TrapExists(short theTrap);

//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

#if GENERATINGCFM
#define DisposeIfRoutineDescriptor(p) ((p) ? (DisposeRoutineDescriptor(p), (UniversalProcPtr)nil) : (UniversalProcPtr)nil)
#else
#define DisposeIfRoutineDescriptor(p) NULL
#endif

EXPORT_MA bool EqualBlocks(const void* first, const void* second, size_t theSize);

inline bool IsHandlePurged(Handle h) { return (!*h); }

inline SignedByte LockHandleHigh(Handle h) { return LockHandle_AC(h, eMoveHi); }

inline long StripLong(void* address)
{
#if qPowerPC || qNeedsSystem7_6
	return reinterpret_cast<long>(address);
#else
	return reinterpret_cast<long>(address) & static_cast<long>(gStrippedAddress);
#endif
}

EXPORT_MA unsigned char UprChar(unsigned char ch, short inScript = smCurrentScript);

EXPORT_MA void UprStr255(CStr255_AC& s, short inScript = smCurrentScript);

EXPORT_MA void UprMAName(MAName& s, short inScript = smCurrentScript);

EXPORT_MA unsigned char LowerChar(unsigned char ch, short inScript = smCurrentScript);

EXPORT_MA void LowerStr255(CStr255_AC& s, short inScript = smCurrentScript);

EXPORT_MA void UseROMMap(bool resLoad);

#if qDebug
inline bool VerboseIsHandle(Handle h) { return VerboseIsHandle_AC(h); }

EXPORT_MA void WriteHandleContents(Handle theHandle);

EXPORT_MA void WrLblHandleContents(const CStr255_AC& aLabel, Handle theHandle);
								
EXPORT_MA void WritePtr(void* val);

EXPORT_MA void WrLblPtr(const CStr255_AC& aLabel, void* val);

EXPORT_MA void WriteBoolean(bool b);

EXPORT_MA void WrLblBoolean(const CStr255_AC& aLabel, bool b);

EXPORT_MA void WriteSig(IDType theID);

EXPORT_MA void WrLblSig(const CStr255_AC& theLabel, IDType theID);

EXPORT_MA void WriteHexInt(short theInt);

EXPORT_MA void WrLblHexInt(const CStr255_AC& theLabel, short theInt);

EXPORT_MA void WriteHexLongint(long theLongint);

EXPORT_MA void WrLblHexLongint(const CStr255_AC& theLabel, long theLongint);
#endif

//	These "MAFoo" functions were originally for THINK Pascal compatibility (but useful in the larger
//	problem of multiple open resource maps in general); when running under the THINK environment,
//	CODE resources are not found in the same resource file as other application resources, so a
//	UseResFile call needs to be made to bring the project resource file into the search path.
//	"gCodeRefNum" is set up at initialization time.
//	!!! A much more general solution to "the resource problem" appears to be warranted.

EXPORT_MA void WithCodeResFileDo(DoWithResFileType DoWithResFile, void* yourDataPtr);
	// Ensure that the resource call is done against gCodeRefNum
	
EXPORT_MA void MAAddResource(Handle theData, ResType theType, short theID, const CStr255_AC& name);

EXPORT_MA Handle MAGet1Resource(ResType rType, short rID);

EXPORT_MA Handle MAGet1NamedResource(ResType rType, const CStr255_AC& name);

EXPORT_MA Handle MAGet1IndResource(ResType rType, short index);

EXPORT_MA short MACount1Resources(ResType rType);

EXPORT_MA Handle MAGetResource(ResType rType, short rID);

EXPORT_MA Handle MAGetNamedResource(ResType rType, const CStr255_AC& name);
								 
EXPORT_MA Handle MAGetIndResource(ResType rType, short index);

EXPORT_MA short MACountResources(ResType rType);

inline bool MASetResLoad(bool state)
{
	short returnVal = LMGetResLoad();
	SetResLoad(state);
	
	return returnVal;
}
// Calls SetResLoad, returns old state

//----------------------------------------------------------------------------------------
// MacApp emulation of auto_ptr that uses operator delete
//----------------------------------------------------------------------------------------
#define __MEMBER_TEMPLATES__ 0

#define DontRelease release
// for compatibility with r10

template <class T> class stl_operator_delete_auto_ptr
{
public:
	/* explicit */ stl_operator_delete_auto_ptr(T* p = NULL) : pointee(p) { }
	~stl_operator_delete_auto_ptr() { operator delete(get()); }

#if FALSE && __MEMBER_TEMPLATES__
	template <class U> stl_operator_delete_auto_ptr(const stl_operator_delete_auto_ptr<U>& rhs) : pointee(rhs.release()) { }

	template <class U> stl_operator_delete_auto_ptr &operator=(const stl_operator_delete_auto_ptr<U>& rhs) { if (pointee != rhs.pointee) reset(rhs.release()); return *this; }
#else
	stl_operator_delete_auto_ptr(const stl_operator_delete_auto_ptr& rhs) : pointee(rhs.release()) { }

	stl_operator_delete_auto_ptr& operator=(const stl_operator_delete_auto_ptr& rhs) { if (pointee != rhs.pointee) reset(rhs.release()); return *this; }
#endif

	T& operator*() const { return *get(); }

	T* operator->() const { return get(); }

	T* get() const { return pointee; }

	T* release() const { T* oldPointee = pointee; const_cast<stl_operator_delete_auto_ptr*>(this)->pointee = NULL; return oldPointee; }

	void reset(T* p = NULL) { if (pointee != p) { operator delete(get()); pointee = p; } }

protected:
	T* pointee;
};

// MA_operator_delete_auto_ptr adds assignment and coercion operators
template <class T> class MA_operator_delete_auto_ptr : public stl_operator_delete_auto_ptr<T>
{
public:
	typedef T* TPtr;

	/* explicit */ MA_operator_delete_auto_ptr(T* p = NULL) : stl_operator_delete_auto_ptr<T>(p) { }
	~MA_operator_delete_auto_ptr() { }

#if FALSE && __MEMBER_TEMPLATES__
	template <class U> MA_operator_delete_auto_ptr(const MA_operator_delete_auto_ptr<U>& rhs) : stl_operator_delete_auto_ptr<T>(rhs.release()) { }

	template <class U> MA_operator_delete_auto_ptr &operator=(const MA_operator_delete_auto_ptr<U>& rhs) { if (pointee != rhs.pointee) reset(rhs.release()); return *this; }
#else
	MA_operator_delete_auto_ptr(const MA_operator_delete_auto_ptr& rhs) : stl_operator_delete_auto_ptr<T>(rhs.release()) { }

	MA_operator_delete_auto_ptr& operator=(const MA_operator_delete_auto_ptr& rhs) { if (pointee != rhs.pointee) reset(rhs.release()); return *this; }
#endif

	MA_operator_delete_auto_ptr& operator=(T* rhs) { pointee = rhs; return *this; }

	operator T*() const { return get(); }
};

#if qDebugMsg
	EXPORT_MA void SubClassResponsibility();
#else
	inline void SubClassResponsibility() { }
#endif
	// Called from functions that MUST be overridden in a subclass.


#endif // __UCOREUTILITIES__
