// UDebug.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UDEBUG__
#define __UDEBUG__

// ACS

#ifndef __CIterator_AC__
#include "CIterator_AC.h"
#endif

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

// Toolbox

// ANSI

#ifndef __STDDEF__
#include <stddef.h>
#endif

//----------------------------------------------------------------------------------------
// Forward class declarations
//----------------------------------------------------------------------------------------

class CStackFrame;


//----------------------------------------------------------------------------------------
// Procedure pointer typedefs
//----------------------------------------------------------------------------------------

typedef void (*DoToFrameType)(const CStackFrame& frame, void* yourDataPtr);

#if qDebug

//----------------------------------------------------------------------------------------
// class CStackFrame
//----------------------------------------------------------------------------------------

class EXPORT_MA CStackFrame
{
public: // methods
	inline CStackFrame() { fFramePtr = (ptrdiff_t) 0; }
	inline CStackFrame(void* framePtr) { fFramePtr = (ptrdiff_t) framePtr; }
	inline CStackFrame(ptrdiff_t framePtr) {  fFramePtr = framePtr; }
	inline CStackFrame(const CStackFrame& rhs) {fFramePtr = rhs.fFramePtr; }

	inline CStackFrame& operator=(const CStackFrame& rhs) {fFramePtr = rhs.fFramePtr; return *this; }
	
	CStackFrame GetCallersStackFrame() const;
	
	void* GetReturnAddress() const;
	
	inline const ptrdiff_t GetStackFramePtr() const { return fFramePtr; };
	const ptrdiff_t GetParametersPtr() const;
	const short& GetParameterWord(short nthWord) const;
	const long& GetParameterDoubleWord(short nthDoubleWord) const;
	
	const short& GetLocalWord(short nthWord) const;
	const long& GetLocalDoubleWord(short nthDoubleWord) const;
	
	bool LooksValid() const;
	
	inline bool operator==(const CStackFrame& stackFrame) const { return fFramePtr == stackFrame.fFramePtr; }
	inline bool operator!=(const CStackFrame& stackFrame) const { return !(*this == stackFrame); }

	inline operator void*() { return (void*)fFramePtr; }

private: // data fields
	ptrdiff_t fFramePtr;
};


//----------------------------------------------------------------------------------------
// class CStackFrameIterator
//----------------------------------------------------------------------------------------

class EXPORT_MA CStackFrameIterator : public CIterator_AC
{
public: // methods
	CStackFrameIterator(const CStackFrame& topFrame);
	
	//
	// Required functions 
	//
	CStackFrame First();
	inline const CStackFrame& Current() const { return fCurrent; }
	CStackFrame Next();

	//
	// Convenience functions for C++meisters
	//
	inline CStackFrame operator++() { return Next(); }
	inline CStackFrame operator++(int) { CStackFrame returnVal = Current(); Next(); return returnVal; }
	inline operator CStackFrame() const { return Current(); }

private: // data fields
	CStackFrame fCurrent;
	CStackFrame fTopFrame;
};


//----------------------------------------------------------------------------------------
// Externally available global variables
//----------------------------------------------------------------------------------------

EXPORT_MA extern bool gHighLevelDebuggerRunning;

#endif	// qDebug

//----------------------------------------------------------------------------------------
// Externally available global flags for controlling printf info.
//----------------------------------------------------------------------------------------

#if qDebug || qTheDebugger

// Debuggers <= kxNoDebugger don't have the ability to show debug
// messages in a console window they use the AppleEvent hack for now
#define kxMWDebug68K	-2
#define kxMWDebugPPC	-1
#define kxNoDebugger	0
#define kxJasik 		1
#define kxSourceBug		2
#define kxSade			3
#define kxR2DbPPC		4
#define kxR2Db68K		5
#define kxVooDoo		6

EXPORT_MA extern short gCurrentDebugger;

#endif

#if qDebug
EXPORT_MA extern bool gUserFlag1;
EXPORT_MA extern bool gUserFlag2;
EXPORT_MA extern bool gUserFlag3;
EXPORT_MA extern bool gAssumeFocused;
EXPORT_MA extern bool gDebugPrinting;
EXPORT_MA extern bool gIntenseDebugging;
EXPORT_MA extern bool gReportEvent;
EXPORT_MA extern bool gReportMenuChoices;
EXPORT_MA extern bool gShowCursorRegion;
EXPORT_MA extern bool gShowHelpRegion;
EXPORT_MA extern bool gShowInvalidations;
EXPORT_MA extern bool gShowSleepRegion;
EXPORT_MA extern bool gMemMgtBreak;
EXPORT_MA extern bool gRsrcReport;
EXPORT_MA extern bool gSegReport;

#else
const bool gUserFlag1 = false;
const bool gUserFlag2 = false;
const bool gUserFlag3 = false;
const bool gAssumeFocused = false;
const bool gDebugPrinting = false;
const bool gIntenseDebugging = false;
const bool gReportEvent = false;
const bool gReportMenuChoices = false;
const bool gShowCursorRegion = false;
const bool gShowHelpRegion = false;
const bool gShowInvalidations = false;
const bool gShowSleepRegion = false;
const bool gMemMgtBreak = false;
const bool gRsrcReport = false;
const bool gSegReport = false;
#endif


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

#if qDebug || qTheDebugger

EXPORT_MA extern void InitUDebug();

#endif


#if qDebug

EXPORT_MA bool DebugCanReadLn();
	// Returns true if you can ReadLn to the user. This in turn is true when there is a
    // debug view (pDebugView), that view can currently write to its window, and this unit
    // is initialized.

EXPORT_MA bool DebugCanWriteLn();
	// Returns true if you can WriteLn to the user in the debug window. This is true if
    // there exists a debug view and this unit is initialized.

EXPORT_MA void DebugWriteLnHook(Ptr textBuf, long byteCount);
	// MacApp's bottleneck for writing debug output (not for general use). 

EXPORT_MA void EachFrameDo(const CStackFrame& baseFrame,
						DoToFrameType DoToFrame,
						void* yourDataPtr);
	// Recursive iterate up the stack frame calling DoToFrame for each frame.
	
EXPORT_MA void GetCallersMethodName(char* s);
	// Return the name of the calling method in s.

EXPORT_MA void GetMethodName(void* pc, char* s);
	// Returns in s the name of the method or procedure into which address ppc points.
    // Calls GetProcName to do the work.

EXPORT_MA void GetProcName(void* pc, char* className, char* procName);
	// Returns in procName the name of the method, procedure, or function into which
    // address ppc points. If it is a method, the method's class is returned in className.
    // If not a method, className is set to ''.

EXPORT_MA void ConfirmHighLevelDebugger();
	// Confirm that the high level debugger is still running.
	
#endif // qDebug

#endif // __UDEBUG__
