//----------------------------------------------------------------------------------------
// Compatibility.cp 
// Copyright  1987-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#include "Compatibility.h"

#if qBackwardCompatible

void PtToVPt(const CPoint thePt,
			 VPoint& theVPt)
{
	theVPt = VPoint(thePt);
}


CPoint VPtToPt(const VPoint& theVPt)
{
	return theVPt.ToPoint();
}


void RectToVRect(const CRect& theRect,
				 VRect& theVRect)
{
	theVRect = VRect(theRect);
}


void VRectToRect(const VRect& theVRect,
				 CRect& theRect)
{
	theRect = theVRect.ToRect();
}


void AddVPt(const VPoint& srcVPt,
			VPoint& dstVPt)
{
	dstVPt += srcVPt;
}


void SubVPt(const VPoint& srcVPt,
			VPoint& dstVPt)
{
	dstVPt -= srcVPt;
}


void SetVPt(VPoint& vPt,
			VCoordinate h,
			VCoordinate v)
{
	vPt.Set(h, v);
}


bool EqualVPt(const VPoint& aVPt,
				 const VPoint& bVPt)
{
	return aVPt == bVPt;
}


void SetVRect(VRect& vRt,
			  VCoordinate left,
			  VCoordinate top,
			  VCoordinate right,
			  VCoordinate bottom)
{
	vRt.Set(left, top, right, bottom);
}


void OffsetVRect(VRect& vRt,
				 VCoordinate dh,
				 VCoordinate dv)
{
	vRt.Offset(dh, dv);
}


void InsetVRect(VRect& vRt,
				VCoordinate dh,
				VCoordinate dv)
{
	vRt.Inset(dh, dv);
}


void Pt2VRect(const VPoint& theTopLeft,
			  const VPoint& theBotRight,
			  VRect& vRt)
{
	vRt.Set(theTopLeft, theBotRight);
}


bool PtInVRect(const VPoint& vPt,
				  const VRect& vRt)
{
	return vRt.Contains(vPt);
}


bool EmptyVRect(const VRect& vRt)
{
	return vRt.Empty();
}


bool EqualVRect(const VRect& aVRt,
				   const VRect& bVRt)
{
	return aVRt == bVRt;
}


VCoordinate LengthVRect(const VRect& vRt,
						VHSelect whichDim)
{
	return vRt.GetLength(whichDim);
}


void PinVRect(const VRect& vRt, VPoint& vPt)
{
	vPt.ConstrainTo(vRt);
}


bool SectVRect(const VRect& src1,
				  const VRect& src2,
				  VRect& dst)
{
	dst = src1 & src2;

	return !dst.Empty();
}


void UnionVRect(const VRect& src1,
				const VRect& src2,
				VRect& dst)
{
	dst = src1 | src2;
}

#endif
