//----------------------------------------------------------------------------------------
// UPointerObject.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

//========================================================================================
// Defines
//========================================================================================

// The default is to use MacApp's object heap for all classes and structures. 
// To revert to using calloc/malloc for structures (blech!), compile with -d qMAGlobalNew=0
// -- OR --
// Omit this file 

#ifndef qMAGlobalNew
#define qMAGlobalNew	1
#endif

// Setting the following flag true will link our replacements for the standard allocators
// if you have the MacApp librarys linking ahead of MSL. Be warned that it will generate some
// duplicate symbol warnings. Wish there was a better way...
#ifndef qMAmalloc
#define qMAmalloc	0
#endif

#if qMAGlobalNew && !qTidyHeap 

//----------------------------------------------------------------------------------------
// operator new: 
//----------------------------------------------------------------------------------------


void* operator new(size_t size)
{
   return MAOperatorNew(size);
}

//----------------------------------------------------------------------------------------
// operator delete: 
//----------------------------------------------------------------------------------------


void operator delete(void* obj)
{
	MAOperatorDelete(obj);
}

#endif


#if qMAmalloc 
//========================================================================================
// These are to be linked in ahead of the standard library in order to provice nice
// speedy performance using the MacApp allocator instead of the standard performance of the
// standard allocaters
//========================================================================================

//----------------------------------------------------------------------------------------
// malloc: 
//----------------------------------------------------------------------------------------
void* malloc(size_t size)
{
	void* returnVal = NULL;
	
	try
	{
		returnVal = MAOperatorNew(size);
	}
	catch(CException_AC& theException)
	{
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// calloc: 
//----------------------------------------------------------------------------------------
void* calloc(size_t nmemb, size_t size)
{
	long itsFullSize = nmemb * size;
	void* returnVal = malloc(itsFullSize);
	
#if qDebug
	// MacApp's global operator new will provide this service when not debugging
	if (returnVal)
		BlockSet_AC(returnVal, size, 0);
#endif

	return returnVal;
}

//----------------------------------------------------------------------------------------
// realloc: 
//----------------------------------------------------------------------------------------
void* realloc(void* ptr, size_t size)
{
	void * returnVal = malloc(size);

		// if ptr is null then the call is equivalent to malloc.
	if (ptr)
	{
		if (returnVal)
			BlockMove_AC(ptr, returnVal, size); // may copy some garbage, but it won't hurt

		free(ptr);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// free: 
//----------------------------------------------------------------------------------------
void free(void *ptr)
{
	MAOperatorDelete(ptr);
}

#endif


