//----------------------------------------------------------------------------------------
// NubInspectorHooks.cp
// Copyright  1994-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __NUBINSPECTORHOOKS__
#include "NubInspectorHooks.h"
#endif

// MacApp

// for MAxxx_cast
#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#if qDebug || qInspector

// Toolbox
													 
#ifndef __GESTALT__
#include <Gestalt.h>
#endif
													 
//--------------------------------------------------------------------------------------
// Constants
//--------------------------------------------------------------------------------------

#define gestaltNubDispatch 		'NubD'		// Entry point for NubDispatch

#define kNubNewObject			30
#define kNubObjectFreed			31


//--------------------------------------------------------------------------------------
// Globals
//--------------------------------------------------------------------------------------

UniversalProcPtr	gNubEntry;


//========================================================================================
// GLOBAL Functions
//========================================================================================

//--------------------------------------------------------------------------------------
// NubIsInstalled
//--------------------------------------------------------------------------------------
// True if the DebuggerINIT is installed.
bool NubIsInstalled ()
{
	if (!gNubEntry)
	{
		OSErr err = Gestalt(gestaltNubDispatch, reinterpret_cast<long*>(&gNubEntry));
		if (err != noErr)
			gNubEntry = reinterpret_cast<UniversalProcPtr>(-1);
	}
	
	return gNubEntry != reinterpret_cast<UniversalProcPtr>(-1);
}	


//--------------------------------------------------------------------------------------
// NubNewObject
//--------------------------------------------------------------------------------------
#if q68K
pascal void CallNubNewObject (void *theObject, const CStr255_AC& className, void *nubEntry)
 = 	{	0x205F,					// move.l 	(sp)+,a0
 		0x303C, kNubNewObject,	// move.w 	kNubNewObject,d0
 		0x4E90					// jsr		(a0)
 	}; 	
#else
enum {
	uppNubNewObjectProcInfo =  kD0DispatchedPascalStackBased 
							 | DISPATCHED_STACK_ROUTINE_PARAMETER(1, kFourByteCode)
							 | DISPATCHED_STACK_ROUTINE_PARAMETER(2, kFourByteCode)
							 | DISPATCHED_STACK_ROUTINE_SELECTOR_SIZE(kTwoByteCode)
};
#endif

void NubNewObject (void *theObject, const CStr255_AC& className)
{
#if q68K
	CallNubNewObject(theObject, className, gNubEntry);
#else
	CallUniversalProc(gNubEntry, uppNubNewObjectProcInfo,
					  kNubNewObject, theObject, &className);
#endif
}

//--------------------------------------------------------------------------------------
// NubObjectFreed
//--------------------------------------------------------------------------------------
#if q68K
pascal void CallNubObjectFreed (void *theObject, const CStr255_AC& className, void *nubEntry)
 = 	{	0x205F,						// move.l 	(sp)+,a0
 		0x303C, kNubObjectFreed,	// move.w 	#kNubObjectFreed,d0
 		0x4E90						// jsr		(a0)
 	}; 	
#else
enum {
	uppNubObjectFreedProcInfo =  kD0DispatchedPascalStackBased 
							   | DISPATCHED_STACK_ROUTINE_PARAMETER(1, kFourByteCode)
							   | DISPATCHED_STACK_ROUTINE_PARAMETER(2, kFourByteCode)
							   | DISPATCHED_STACK_ROUTINE_SELECTOR_SIZE(kTwoByteCode)
};
#endif

void NubObjectFreed (void *theObject, const CStr255_AC& className)
{
#if q68K
	CallNubObjectFreed(theObject, className, gNubEntry);
#else
	CallUniversalProc(gNubEntry, uppNubObjectFreedProcInfo,
					  kNubObjectFreed, theObject, &className);
#endif
}

#endif // qDebug || qInspector

//----------------------------------------------------------------------------------------
// End of NubInspectorHooks.cp
