// PlatformMemory.cp 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.

#ifndef PLATFORMMEMORY_H
#include "PlatformMemory.h"
#endif

// MacApp

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

// ANSI

//========================================================================================
// Global variable definitions
//========================================================================================

static bool pAllocateObjectsFromTemp = false;				// Used to track whether to
                                                            // allocate objects from
                                                            // temporary memory or not.

//----------------------------------------------------------------------------------------
// PlatformAllocateBlock
//----------------------------------------------------------------------------------------

Handle MAAllocateHandle(size_t size)
{
	bool heapTemp;
	if (gUMemoryInitialized)
		heapTemp = TemporaryAllocation(pAllocateObjectsFromTemp);

	Handle h = ::NewHandle(size);

	if (gUMemoryInitialized)
		TemporaryAllocation(heapTemp);		// Reset temp flag before possible Failure
	
	::ThrowIfNULL_AC(h);
		
	return h;
}

//----------------------------------------------------------------------------------------
// PLATFORM_ASSERT
//----------------------------------------------------------------------------------------


void PLATFORM_ASSERT(bool condition)
{
	if (!condition)
		::ProgramBreak_AC("Assertion failed.");
}

//----------------------------------------------------------------------------------------
// PLATFORM_PRINT_STRING
//----------------------------------------------------------------------------------------


void PLATFORM_PRINT_STRING(const char* string)
{
#if qDebugMsg
	ProgramReport_AC(string);
#else
	::ProgramBreak_AC(string);
#endif
}

//----------------------------------------------------------------------------------------
// GetTempObjectAllocationState: 
//----------------------------------------------------------------------------------------

bool GetTempObjectAllocationState()
{
	return pAllocateObjectsFromTemp;
}

//----------------------------------------------------------------------------------------
// TempObjectAllocation: 
//----------------------------------------------------------------------------------------

bool TempObjectAllocation(bool allocateFromTemp)
{
	bool previousState = pAllocateObjectsFromTemp;
	pAllocateObjectsFromTemp = allocateFromTemp;

	return previousState;
}

//----------------------------------------------------------------------------------------
// End of PlatformMemory.cp


