//----------------------------------------------------------------------------------------
// UAssociation.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UASSOCIATION__
#include "UAssociation.h"
#endif

// ACS

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static CompareResult_AC CompareEntryKeys(void* anItem,
									  void* yourDataPtr);

//========================================================================================
// struct CEntryWithKey
//========================================================================================

struct CEntryWithKey
{
public:
	// Constructor
	CEntryWithKey(const CStr255_AC& thekeyStr) :
		fKeyStr(thekeyStr)
	{
	}

	// Fields
	const CStr255_AC& fKeyStr;
};


//----------------------------------------------------------------------------------------
// CompareEntryKeys:
//----------------------------------------------------------------------------------------


CompareResult_AC CompareEntryKeys(void* anItem,
							   void* yourDataPtr)
{
	CEntryWithKey * comparisonInfo = (CEntryWithKey *)yourDataPtr;

	return CPascalStr_AC::CPascalStrCompare(comparisonInfo->fKeyStr, *static_cast<TEntry*>(anItem)->fKey);
}


//========================================================================================
// CLASS TEntry
//========================================================================================
//----------------------------------------------------------------------------------------
// TEntry::TEntry:
//----------------------------------------------------------------------------------------


TEntry::TEntry(const CStr255_AC& itsKey,
					const CStr255_AC& itsValue)
{
	fKey.reset((CStr255_AC *)operator new(itsKey.Length() + kLengthByte));
	::ThrowIfNULL_AC(fKey);
	*fKey = itsKey;

	fValue.reset((CStr255_AC *)operator new(itsValue.Length() + kLengthByte));
	::ThrowIfNULL_AC(fValue);
	*fValue = itsValue;
}

//----------------------------------------------------------------------------------------
// TEntry::Free:
//----------------------------------------------------------------------------------------


TEntry::~TEntry()
{
}

//----------------------------------------------------------------------------------------
// TEntry::SetValue:
//----------------------------------------------------------------------------------------


void TEntry::SetValue(const CStr255_AC& value)
{
	fValue.reset((CStr255_AC *)operator new(value.Length() + kLengthByte));
	::ThrowIfNULL_AC(fValue);
	*fValue = value;
}


//========================================================================================
// CLASS TEntriesList
//========================================================================================
MA_DEFINE_CLASS(TEntriesList);

//----------------------------------------------------------------------------------------
// TEntriesList destructor
//----------------------------------------------------------------------------------------


TEntriesList::~TEntriesList()
{
}

//----------------------------------------------------------------------------------------
// TEntriesList::Compare:
//----------------------------------------------------------------------------------------


CompareResult_AC TEntriesList::Compare(const void* item1,
									const void* item2) const
{
	return CPascalStr_AC::CPascalStrCompare(*static_cast<const TEntry*>(item1)->fKey, *static_cast<const TEntry*>(item2)->fKey);
}

//========================================================================================
// CLASS TAssociation
//========================================================================================
//----------------------------------------------------------------------------------------
// TAssociation constructor
//----------------------------------------------------------------------------------------


TAssociation::TAssociation()
{
}

//----------------------------------------------------------------------------------------
// TAssociation::Free:
//----------------------------------------------------------------------------------------


TAssociation::~TAssociation()
{
	for (TemplateIterator_AC<TEntry*> iter(&fEntries); iter; ++iter)
		delete iter.Current();
}

//----------------------------------------------------------------------------------------
// TAssociation::EntryWithKey:
//----------------------------------------------------------------------------------------


TEntry* TAssociation::EntryWithKey(const CStr255_AC& keyStr)
{
	CEntryWithKey aCEntryWithKey(keyStr);
	ArrayIndex_AC index;

	return static_cast<TEntry*>(fEntries.DoSearch(&CompareEntryKeys, &aCEntryWithKey, index));
}

//----------------------------------------------------------------------------------------
// TAssociation::EntryWithValue:
//----------------------------------------------------------------------------------------


TEntry* TAssociation::EntryWithValue(const CStr255_AC& valueStr)
{
	TEntry * returnEntry = NULL;
	for (TemplateArrayIterator_AC<TEntry>iter(&fEntries); iter; ++iter)
		if (valueStr == *iter->fValue)
		{
			returnEntry = iter;
			break;
		}

	return returnEntry;
}

//----------------------------------------------------------------------------------------
// TAssociation::InsertEntry:
//----------------------------------------------------------------------------------------


void TAssociation::InsertEntry(const CStr255_AC& keyStr,
							   const CStr255_AC& valueStr)
{
	TEntry * anEntry = EntryWithKey(keyStr);
	if (anEntry)
		anEntry->SetValue(valueStr);
	else
		fEntries.Insert(TH_new TEntry(keyStr, valueStr));
}

//----------------------------------------------------------------------------------------
// TAssociation::KeyAt:
//----------------------------------------------------------------------------------------


bool TAssociation::KeyAt(const CStr255_AC& valueStr,
							CStr255_AC& keyStr)
{
	bool returnVal = false;
	keyStr.Empty();

	for (TemplateIterator_AC<TEntry*> iter(&fEntries); iter; ++iter)
	{
		TEntry * anEntry = iter;
	
		if (valueStr == *anEntry->fValue)
		{
			keyStr = *anEntry->fKey;
			returnVal = true;
			break;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TAssociation::KeyAt:
//----------------------------------------------------------------------------------------


CStr255_AC TAssociation::KeyAt(const CStr255_AC& valueStr)
{
	CStr255_AC * returnVal = NULL;

	for (TemplateIterator_AC<TEntry*> iter(&fEntries); iter; ++iter)
	{
		TEntry * anEntry = iter;
		if (valueStr == *anEntry->fValue)
		{
			returnVal = anEntry->fValue;
			break;
		}
	}

	return (unsigned char*)returnVal; // Don't have a constructor for CPascalStr_AC*
}

//----------------------------------------------------------------------------------------
// TAssociation::RemoveValueAt:
//----------------------------------------------------------------------------------------


void TAssociation::RemoveValueAt(const CStr255_AC& keyStr)
{
	CEntryWithKey aCEntryWithKey(keyStr);
	ArrayIndex_AC index;

	TEntry * theEntry = static_cast<TEntry*>(fEntries.DoSearch(&CompareEntryKeys, &aCEntryWithKey, index));
	if (theEntry)
		fEntries.Delete(theEntry);
}

//----------------------------------------------------------------------------------------
// TAssociation::RemoveKeyAt:
//----------------------------------------------------------------------------------------


void TAssociation::RemoveKeyAt(const CStr255_AC& valueStr)
{
	for (TemplateIterator_AC<TEntry*> iter(&fEntries); iter; ++iter)
	{
		TEntry * anEntry = iter;
		if (valueStr == *anEntry->fValue)
		{
			fEntries.Delete(anEntry);
			break;
		}
	}
}

//----------------------------------------------------------------------------------------
// TAssociation::ValueAt:
//----------------------------------------------------------------------------------------


bool TAssociation::ValueAt(const CStr255_AC& keyStr,
							  CStr255_AC& valueStr)
{
	bool returnVal = false;
	valueStr.Empty();

	CEntryWithKey aCEntryWithKey(keyStr);
	ArrayIndex_AC index;

	TEntry * theEntry = static_cast<TEntry*>(fEntries.DoSearch(&CompareEntryKeys, &aCEntryWithKey, index));
	if (theEntry)
	{
		valueStr = *theEntry->fValue;
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TAssociation::ValueAt:
//----------------------------------------------------------------------------------------


CStr255_AC TAssociation::ValueAt(const CStr255_AC& keyStr)
{
	CStr255_AC * returnVal = NULL;

	CEntryWithKey aCEntryWithKey(keyStr);
	ArrayIndex_AC index;

	TEntry * theEntry = static_cast<TEntry*>(fEntries.DoSearch(&CompareEntryKeys, &aCEntryWithKey, index));
	if (theEntry)
		returnVal = theEntry->fValue;

	return (unsigned char*)returnVal; // Don't have a constructor for CPascalStr_AC*
}

//----------------------------------------------------------------------------------------
// End of UAssociation.cp


