//----------------------------------------------------------------------------------------
// UCoreErrorMgr.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

// MacApp

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

// Toolbox

#ifndef __RESOURCES__
#include <Resources.h>
#endif

// ANSI



//----------------------------------------------------------------------------------------
CStr255_AC gErrorParm3;

//----------------------------------------------------------------------------------------
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
struct ErrRecord
{
	short lowErr;
	short highErr;
	short index;
};
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef ErrRecord *ErrRecordPointer, **ErrRecordHandle;

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static bool SearchErrTable(short value,
					   ResNumber resourceID,
					   CStr255_AC& str);

//----------------------------------------------------------------------------------------
// SearchErrTable: Private function
//----------------------------------------------------------------------------------------


bool SearchErrTable(short value,
					   ResNumber resourceID,
					   CStr255_AC& str)
{
	bool found = false;

	str.Empty();
	ErrRecordHandle table = (ErrRecordHandle)GetResource('errs', resourceID);
	if (table)
	{
		ResNumber strID = 0;
		CTempHandleLock_AC tempHandleLock((Handle)table);
		ErrRecordPointer pEntry = *table;

		short lenTab = static_cast<short>(GetHandleSize((Handle)table) / sizeof(ErrRecord));
		for (short i = 1; i <= lenTab; ++i, ++pEntry)
		{
			if (!pEntry->lowErr)
				strID = pEntry->index;
			else if ((pEntry->lowErr <= value) && (value <= pEntry->highErr))
			{
				if (pEntry->index > 0)
					GetIndString(str, strID, pEntry->index);
				found = true;
				break;
			}
		}
	}

	return found;
}


//----------------------------------------------------------------------------------------
// LookupErrString: 
//----------------------------------------------------------------------------------------
bool LookupErrString(short value,
						ResNumber resourceID,
						CStr255_AC& str)
{
	return SearchErrTable(value, errAppTable + resourceID, str) ? true : SearchErrTable(value, resourceID, str);
}

//----------------------------------------------------------------------------------------
// NotYetImplemented: 
//----------------------------------------------------------------------------------------


#if qDebugMsg
void NotYetImplemented(const CStr255_AC& where)
#else
void NotYetImplemented(const CStr255_AC&		/* where */)
#endif

{
#if qDebugMsg
	ProgramBreak_AC("NotYetImplemented: %s\n", (const char*)(where));
#endif

	::ThrowException_AC(errNotImplemented, 0);
}

//----------------------------------------------------------------------------------------
// End of UCoreErrorMgr.cp 


