//----------------------------------------------------------------------------------------
// UCoreGlobals.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FOLDERS__
#include <Folders.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

//========================================================================================
// GLOBAL Functions
//========================================================================================

//----------------------------------------------------------------------------------------
// ValidateConfiguration: 
//----------------------------------------------------------------------------------------


bool ValidateConfiguration()
{
	// Run the gauntlet of support tests using the conditionally set constants. If any
    // single test fails then the app is considered unsupported on this machine.

	// Minimum system version (System 7.0)
	bool isSupported = GetSystemVersion_AC() >= 0x700;

	// Optionally required 
	if (qNeedsColorQD)
		isSupported = isSupported && HasColorQD_AC();

	// System 7.5 Support 
	if (qNeedsSystem7_5)
		isSupported = isSupported && GetSystemVersion_AC() >= 0x750;

	return isSupported;
}

//----------------------------------------------------------------------------------------
// MAGestaltAttribute: 
//----------------------------------------------------------------------------------------


bool MAGestaltAttribute(OSType itsAttr, short itsBit)
{
	long response;

	return Gestalt(itsAttr, &response) == noErr && (response >> itsBit) & 1;
	
}

//----------------------------------------------------------------------------------------
// MAHasGestalt: 
//----------------------------------------------------------------------------------------


bool MAHasGestalt(OSType itsGestalt)
{
	long response;

	return Gestalt(itsGestalt, &response) == noErr;

}


//----------------------------------------------------------------------------------------
// InitCoreMacApp
//----------------------------------------------------------------------------------------


void InitCoreMacApp(short callsToMoreMasters)
{
	// Install MacApp's Memory management system 
	InitUMemory(callsToMoreMasters);
}

//----------------------------------------------------------------------------------------
// FindProcessBySignature: 
//----------------------------------------------------------------------------------------


OSErr FindProcessBySignature(OSType sig,
							 ProcessSerialNumber& psn,
							 CFSSpec_AC* fileSpec)
{
	OSErr err;
	
	ProcessInfoRec info;

	psn.highLongOfPSN = 0;
	psn.lowLongOfPSN  = kNoProcess;
	do
	{
		err= GetNextProcess(&psn);
		if( err == noErr )
		{

			info.processInfoLength = sizeof(ProcessInfoRec);
			info.processName = NULL;
			info.processAppSpec = fileSpec;

			err= GetProcessInformation(&psn, &info);
		}
	} while( (err == noErr) && info.processSignature != sig );

	if( err == noErr )
		psn = info.processNumber;

	return err;
}


//----------------------------------------------------------------------------------------
// End of UCoreGlobals.cp


