//----------------------------------------------------------------------------------------
// UCoreUtilities.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

// MacApp

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Toolbox

#ifndef __APPLEGUIDE__
#include <AppleGuide.h>
#endif

#if qPowerPC || qModelCFM
	#ifndef __CODEFRAGMENTS__
	#include <CodeFragments.h>
	#endif
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#if qDrag
#ifndef __DRAG__
#include <Drag.h>
#endif
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __OCESTANDARDMAIL__
#include <OCEStandardMail.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// ANSI


#ifndef __STRING__
#include <string.h>
#endif

// Compatibility Hacks

#ifndef __PATCHES__
enum {
	kOSTrapType			= OSTrap,
	kToolboxTrapType	= ToolTrap
};
#endif

//----------------------------------------------------------------------------------------
// extern definitions:

// constants

const char* kCopyright = MA_COPYRIGHT;
short gApplicationRefNum;
short gCodeRefNum;

bool gToolBoxInitialized;
bool gUMemoryInitialized;

Ptr gStrippedAddress;


//========================================================================================
// GLOBAL Procedures
//========================================================================================
static short NumToolboxTraps();

#if qDebug
static bool IsFreeHandle(Handle aHandle);
#endif

//----------------------------------------------------------------------------------------
// EqualBlocks: 
//----------------------------------------------------------------------------------------
bool EqualBlocks(const void* first, const void* second, size_t theSize)
{
	return !memcmp(first, second, theSize);
}

//----------------------------------------------------------------------------------------
// GetTrapType: 
//----------------------------------------------------------------------------------------
TrapType GetTrapType(short theTrap)
{
	// OS traps start with A0, Tool with A8 or AA. per D.A.
	return !(theTrap & 0x0800) ? kOSTrapType : kToolboxTrapType;
}

//----------------------------------------------------------------------------------------
// NumToolboxTraps: InitGraf is always implemented (trap 0xA86E). If the trap table is
// big enough, trap 0xAA6E will always point to either Unimplemented or some other trap,
// but will never be the same as InitGraf. Thus, you can check the size of the trap table
// by asking if the address of trap 0xA86E is the same as 0xAA6E.
//----------------------------------------------------------------------------------------
short NumToolboxTraps()
{
#if qMSWindows
	return 0;
#else
	return ::NGetTrapAddress(_InitGraf, kToolboxTrapType) == ::NGetTrapAddress(0xAA6E, kToolboxTrapType) ? 0x200 : 0x400;
#endif
}

//----------------------------------------------------------------------------------------
// TrapExists: 
//----------------------------------------------------------------------------------------
bool TrapExists(short theTrap)
{
#if qMSWindows
	return false;
#else
	TrapType theTrapType = ::GetTrapType(theTrap);
	short localTrap = theTrap;					// since theTrap is a const
	if (theTrapType == kToolboxTrapType)
	{
		localTrap = (localTrap & 0x07FF);
		if (localTrap >= NumToolboxTraps())
			localTrap = _Unimplemented;
	}

	return ::NGetTrapAddress(_Unimplemented, kToolboxTrapType) != ::NGetTrapAddress(localTrap, theTrapType);
#endif
}

//----------------------------------------------------------------------------------------
// LowerChar: 
//----------------------------------------------------------------------------------------
unsigned char LowerChar(unsigned char ch, short inScript)
{
	unsigned char returnVal = ch;
	::LowercaseText((Ptr)&returnVal, sizeof(ch), inScript);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// LowerStr255: 
//----------------------------------------------------------------------------------------
void LowerStr255(CStr255_AC& s, short inScript)
{
	::LowercaseText(reinterpret_cast<char*>(&s[1]), s.Length(), inScript);
}

//----------------------------------------------------------------------------------------
// UprChar: 
//----------------------------------------------------------------------------------------
unsigned char UprChar(unsigned char ch, short inScript)
{
	unsigned char returnVal = ch;
	::UppercaseText(reinterpret_cast<char*>(&returnVal), sizeof(ch), inScript);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// UprStr255: 
//----------------------------------------------------------------------------------------
void UprStr255(CStr255_AC& s, short inScript)
{
	::UppercaseText(reinterpret_cast<char*>(&s[1]), s.Length(), inScript);
}

//----------------------------------------------------------------------------------------
// UprMAName: 
//----------------------------------------------------------------------------------------
void UprMAName(MAName& s, short inScript)
{
	::UppercaseText(reinterpret_cast<char*>(&s[1]), s.Length(), inScript);
}

//----------------------------------------------------------------------------------------
// UseROMMap: 
//----------------------------------------------------------------------------------------
void UseROMMap(bool resLoad)
{
	const short kLoMemMapTrue = -1;
	const short kLoMemMapFalse = 0;
	
	::LMSetROMMapInsert(resLoad ? kLoMemMapTrue : kLoMemMapFalse);
}

#if qDebug
//----------------------------------------------------------------------------------------
// WriteHandleContents: 
//----------------------------------------------------------------------------------------
void WriteHandleContents(Handle theHandle)
{
	Size Max = GetHandleSize(theHandle) - 1;
	if (Max > 0)
	{
		CTempHandleLock_AC tempHandleLock(theHandle);

		for (short index = 0; index <= Max; ++index)
			ProgramReport_AC(false, "%X", *((Ptr)(*theHandle + index)));
	}
	else
		ProgramReport_AC("**Empty**");
}

//----------------------------------------------------------------------------------------
// WrLblHandleContents: 
//----------------------------------------------------------------------------------------
void WrLblHandleContents(const CStr255_AC& aLabel,
								Handle theHandle)
{
	ProgramReport_AC(false, "%s = ", (const char*)(aLabel));
	WriteHandleContents(theHandle);
}

//----------------------------------------------------------------------------------------
// WritePtr: 
//----------------------------------------------------------------------------------------
void WritePtr(void* val)
{
	if (val)
		ProgramReport_AC(false, "%X", val);
	else
		ProgramReport_AC("nil");
}

//----------------------------------------------------------------------------------------
// WrLblPtr: 
//----------------------------------------------------------------------------------------
void WrLblPtr(const CStr255_AC& aLabel,
					 void* val)
{
	ProgramReport_AC(false, "%s = ", (const char*)(aLabel));
	WritePtr(val);
}

//----------------------------------------------------------------------------------------
// WriteBoolean: 
//----------------------------------------------------------------------------------------
void WriteBoolean(bool b)
{
	ProgramReport_AC(false, "%s", b ? "true" : "false");
}

//----------------------------------------------------------------------------------------
// WrLblBoolean: 
//----------------------------------------------------------------------------------------
void WrLblBoolean(const CStr255_AC& aLabel,
						 bool b)
{
	ProgramReport_AC(false, "%s = ", (const char*)(aLabel));
	WriteBoolean(b);
}

//----------------------------------------------------------------------------------------
// WriteSig: 
//----------------------------------------------------------------------------------------
void WriteSig(IDType theID)
{
	ProgramReport_AC(false, "%4s", &theID);
}

//----------------------------------------------------------------------------------------
// WrLblSig: 
//----------------------------------------------------------------------------------------
void WrLblSig(const CStr255_AC& theLabel,
					 IDType theID)
{
	ProgramReport_AC(false, "%s = ", (const char*)(theLabel));
	WriteSig(theID);
}

//----------------------------------------------------------------------------------------
// WriteHexInt: 
//----------------------------------------------------------------------------------------
void WriteHexInt(short theInt)
{
	ProgramReport_AC(false, "%X", theInt);	
}

//----------------------------------------------------------------------------------------
// WrLblHexInt: 
//----------------------------------------------------------------------------------------
void WrLblHexInt(const CStr255_AC& theLabel,
						short theInt)
{
	ProgramReport_AC(false, "%s = ", (const char*)(theLabel));
	WriteHexInt(theInt);
}

//----------------------------------------------------------------------------------------
// WriteHexLongint: 
//----------------------------------------------------------------------------------------
void WriteHexLongint(long theLongint)
{
	ProgramReport_AC(false, "%lx", theLongint);
}

//----------------------------------------------------------------------------------------
// WrLblHexLongint: 
//----------------------------------------------------------------------------------------
void WrLblHexLongint(const CStr255_AC& theLabel,
							long theLongint)
{
	ProgramReport_AC(false, "%s = ", (const char*)(theLabel));
	WriteHexLongint(theLongint);
}
#endif

//----------------------------------------------------------------------------------------
// These "MAFoo" functions were originally for THINK Pascal compatibility (but useful in the
// larger problem of multiple open resource maps in general); when running under the THINK
// environment, CODE resources are not found in the same resource file as other
// application resources, so a UseResFile call needs to be made to bring the project
// resource file into the search path. "gCodeRefNum" is set up at initialization time.
//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------
// WithCodeResFileDo: 
//----------------------------------------------------------------------------------------
void WithCodeResFileDo(DoWithResFileType DoWithResFile,
						void *scopeLink)
{
	CTempUseResFile_AC tempUseResFile(gCodeRefNum);

	DoWithResFile(scopeLink);
}

//----------------------------------------------------------------------------------------
// MAAddResource: 
//----------------------------------------------------------------------------------------
void MAAddResource(Handle theData, ResType theType, short theID, const CStr255_AC& name)
{
	// Checks to see if the resource already exists and if so,
	// removes it and adds the new one.
	short oldResLoad = ::LMGetResLoad();
	::SetResLoad(false);
	Handle existingResource = ::Get1Resource(theType, theID);
	::SetResLoad(oldResLoad);
	if (existingResource)
	{
		RemoveResource(existingResource);
		existingResource = ::DisposeIfHandle_AC(existingResource);
	}
	::AddResource(theData, theType, theID, name);
}

//----------------------------------------------------------------------------------------
// MAGet1Resource: 
//----------------------------------------------------------------------------------------
Handle MAGet1Resource(ResType rType, short rID)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	Handle resource = ::Get1Resource(rType, rID);
	UseResFile_AC(oldResFile);

	return resource;
}

//----------------------------------------------------------------------------------------
// MAGet1NamedResource: 
//----------------------------------------------------------------------------------------
Handle MAGet1NamedResource(ResType rType, const CStr255_AC &name)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	Handle namedResource = ::Get1NamedResource(rType, name);
	UseResFile_AC(oldResFile);

	return namedResource;
}

//----------------------------------------------------------------------------------------
// MAGet1IndResource: 
//----------------------------------------------------------------------------------------
Handle MAGet1IndResource(ResType rType, short index)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	Handle indResource = ::Get1IndResource(rType, index);
	UseResFile_AC(oldResFile);

	return indResource;
}

//----------------------------------------------------------------------------------------
// MACount1Resources: 
//----------------------------------------------------------------------------------------
short MACount1Resources(ResType rType)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	short resourceCnt = ::Count1Resources(rType);
	UseResFile_AC(oldResFile);

	return resourceCnt;
}

//----------------------------------------------------------------------------------------
// MAGetResource: 
//----------------------------------------------------------------------------------------
Handle MAGetResource(ResType rType, short rID)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	Handle h = ::GetResource(rType, rID);
	UseResFile_AC(oldResFile);

	return h;
}

//----------------------------------------------------------------------------------------
// MAGetNamedResource: 
//----------------------------------------------------------------------------------------
Handle MAGetNamedResource(ResType rType, const CStr255_AC &name)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	Handle h = ::GetNamedResource(rType, name);
	UseResFile_AC(oldResFile);

	return h;
}

//----------------------------------------------------------------------------------------
// MAGetIndResource: 
//----------------------------------------------------------------------------------------
Handle MAGetIndResource(ResType rType, short index)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	Handle h = ::GetIndResource(rType, index);
	UseResFile_AC(oldResFile);

	return h;
}

//----------------------------------------------------------------------------------------
// MACountResources: 
//----------------------------------------------------------------------------------------
short MACountResources(ResType rType)
{
	short oldResFile = UseResFile_AC(gCodeRefNum);
	short resourceCnt = ::CountResources(rType);
	UseResFile_AC(oldResFile);

	return resourceCnt;
}

//----------------------------------------------------------------------------------------
// SubClassResponsibility: 
//----------------------------------------------------------------------------------------
#if qDebugMsg
void SubClassResponsibility()
{
	char s[255];
	GetCallersMethodName(s);

	ProgramReport_AC(false, "%s: must be overridden!", s);
}
#endif
