//----------------------------------------------------------------------------------------
// UDebug.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UTHEDEBUGGER__
#include "UTheDebugger.h"
#endif

// Toolbox

#if qDebug || qTheDebugger
	#ifndef __LOWMEM__
	#include <LowMem.h>
	#endif
#endif

#if !qMSWindows && !defined(SYMANTEC_CPLUS)
	#ifndef __UNMANGLER__
	#include "Unmangler.h"
	#endif
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

// Apple Specials
	
#if qDebug && !qPowerPC
	#ifndef __DISASMLOOKUP__
	#include "DisAsmLookup.h"
	#endif
#endif

#ifndef __MWERKS__
	#ifndef __FCNTL__
	#include "fcntl.h"
	#endif
	
	#ifndef __IOCTL__
	#include "ioctl.h"
	#endif
#endif

// Voodoo Monkey

#ifndef __NUBINSPECTORHOOKS__
#include "NubInspectorHooks.h"
#endif

// Metrowerks

#ifdef __MWERKS__
	#ifndef __TRAPS__
	#include <Traps.h>
	#endif	
#endif

#if (qDebug || qTheDebugger) && defined(__MWERKS__)
	#ifndef __CWDEBUG__
	#include "CWDebug.h"
	#endif
#endif


//----------------------------------------------------------------------------------------
// Procedure pointer typedefs
//----------------------------------------------------------------------------------------

#if qDebug || qTheDebugger

typedef long(*GetProcPtr)(Ptr buffer, long count);
typedef void(*PutProcPtr)(Ptr buffer, long count);

#endif	// qDebug || qTheDebugger

//----------------------------------------------------------------------------------------
// struct IOPort: Used to hook MPW StdErr, StdIn, StdOut
//----------------------------------------------------------------------------------------

#if qDebug || qTheDebugger

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
struct IOPort
{
	unsigned char filler[12];
	long count;
	Ptr bufp;
};
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#endif	// qDebug || qTheDebugger

bool gHighLevelDebuggerRunning;					// True if we were able to launch
                                                    // SourceBug, Sade OR Jasik running


//----------------------------------------------------------------------------------------
// Global variable definitions.
//----------------------------------------------------------------------------------------

#if qDebug

bool gUserFlag1;

bool gUserFlag2;

bool gUserFlag3;

bool gAssumeFocused = true;						// make TView::AssumeFocused actually
                                                    // check focus.

bool gDebugPrinting;

bool gIntenseDebugging;

bool gReportEvent;

bool gReportMenuChoices;

bool gShowCursorRegion;

bool gShowHelpRegion;

bool gShowInvalidations;							// make TWindow::InvalidVRect
                                                    // graphically show the invalidation
                                                    // region
													
bool gShowSleepRegion;

bool gMemMgtBreak;

bool gRsrcReport;
	// Report resource maximums to the debugger window.

bool gSegReport;
	// Report segment loadings to the debugger window.

//bool gMastReport = true;

#endif	// qDebug

#if qDebug || qTheDebugger

// Support "-NoDebug -Sym -TheDebugger" builds.
short gCurrentDebugger = kxNoDebugger;	// constants defined in CWDebug.h

#endif	// qDebug

//----------------------------------------------------------------------------------------
// Static variable definitions.
//----------------------------------------------------------------------------------------

#if qDebug || qTheDebugger
#if !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)

static GetProcPtr pGetProc;						// Address of the Proc to Handle
                                                // Read requests Function
                                                // DEVGETTEXT(textBuf: Ptr, byteCount:
                                                // longint): longint;

static PutProcPtr pPutProc;						// Address of the Proc to Handle
                                                // WriteLn requests Procedure
                                                // DEVPUTTEXT(textBuf: Ptr, byteCount:
                                                // longint);
#endif // !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)

static bool pUDebugInitialized;				// True if InitUDebug has been called

#endif // (qDebug || qTheDebugger)

#if (qDebug || qTheDebugger)

static ProcessSerialNumber pDebuggerProcess;	// The process number of the high level
                                                // debugger if there is one.

#endif	// qDebug


//========================================================================================
// GLOBAL Procedures
//========================================================================================

#if qDebug || qTheDebugger

#if qDebug
static void FindHighLevelDebugger();
static bool FindDebuggerProcess(OSType sig, short debugger);
#endif

static void InstallWriteLnHook();

#if !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)
static GetProcPtr SetGetProc(GetProcPtr theGetProc);
static PutProcPtr SetPutProc(PutProcPtr thePutProc);
static long DebugReadLn(Ptr buffer, long byteCount);
#endif // !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)

#endif

//----------------------------------------------------------------------------------------
// External function definitions (no mangling).
//----------------------------------------------------------------------------------------

#if !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)
extern "C"
{
	extern long DevClose(long iop);
	extern long DevIoctl(long iop,
						 long command,
						 void* arg);
	extern long DevFAccess(char* fName,
						   long command,
						   void* arg);
	extern long DevRead(long iop);
	extern long DevWrite(long iop);

	extern long _addDevHandler(long slot,
							   long dvName,
							   void* dvFAccess,
							   void* dvClose,
							   void* dvRead,
							   void* dvWrite,
							   void* dvIoctl);
}
#endif // !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)

#if qDebug

//========================================================================================
// CLASS CStackFrame
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// CStackFrame::GetCallersStackFrame:
//----------------------------------------------------------------------------------------
CStackFrame CStackFrame::GetCallersStackFrame() const
{
	return CStackFrame(*((ptrdiff_t *) fFramePtr));
}


//----------------------------------------------------------------------------------------
// CStackFrame::GetReturnAddress:
//----------------------------------------------------------------------------------------
void* CStackFrame::GetReturnAddress() const
{
	return *(void **) (fFramePtr + 4);
}


//----------------------------------------------------------------------------------------
// CStackFrame::GetParametersPtr:
//----------------------------------------------------------------------------------------
const ptrdiff_t CStackFrame::GetParametersPtr() const
{
	return fFramePtr + 8;
}


//----------------------------------------------------------------------------------------
// CStackFrame::GetParameterWord:
//----------------------------------------------------------------------------------------
const short& CStackFrame::GetParameterWord(short nthWord) const
{
	return *reinterpret_cast<short*>(fFramePtr + 8 + (nthWord - 1) * sizeof(short));
}


//----------------------------------------------------------------------------------------
// CStackFrame::GetParameterDoubleWord:
//----------------------------------------------------------------------------------------
const long& CStackFrame::GetParameterDoubleWord(short nthDoubleWord) const
{
	return *reinterpret_cast<long*>(fFramePtr + 8 + (nthDoubleWord - 1) * sizeof(long));
}


//----------------------------------------------------------------------------------------
// CStackFrame::GetLocalWord:
//----------------------------------------------------------------------------------------
const short& CStackFrame::GetLocalWord(short nthWord) const
{
	return *reinterpret_cast<short*>(fFramePtr - nthWord * sizeof(short));
}


//----------------------------------------------------------------------------------------
// CStackFrame::GetLocalDoubleWord:
//----------------------------------------------------------------------------------------
const long& CStackFrame::GetLocalDoubleWord(short nthDoubleWord) const
{
	return *reinterpret_cast<long*>(fFramePtr - nthDoubleWord * sizeof(long));
}


//----------------------------------------------------------------------------------------
// CStackFrame::LooksValid:
//----------------------------------------------------------------------------------------
bool CStackFrame::LooksValid() const
{
#if !q68K
#define GetCurStackTop() 0
#endif

	return fFramePtr && !Odd_AC(fFramePtr) && 
				fFramePtr < (ptrdiff_t) LMGetCurStackBase() &&
					fFramePtr > (ptrdiff_t) GetCurStackTop();

}


//========================================================================================
// CLASS CStackFrameIterator
//========================================================================================
#undef Inherited
#define Inherited CIterator_AC

//----------------------------------------------------------------------------------------
// CStackFrameIterator::CStackFrameIterator:
//----------------------------------------------------------------------------------------


CStackFrameIterator::CStackFrameIterator(const CStackFrame& topFrame) :
	fCurrent(),
	fTopFrame(topFrame)
{
	First();
}


//----------------------------------------------------------------------------------------
// CStackFrameIterator::First:
//----------------------------------------------------------------------------------------
CStackFrame CStackFrameIterator::First()
{
	return fCurrent = fTopFrame;
}


//----------------------------------------------------------------------------------------
// CStackFrameIterator::Next:
//----------------------------------------------------------------------------------------
CStackFrame CStackFrameIterator::Next()
{
	return fCurrent = fCurrent.LooksValid() ? fCurrent.GetCallersStackFrame() : CStackFrame();
}


#endif // qDebug

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// InitUDebug: essential initialization (segTable, nonRes left in for compatibility (2.0)
//----------------------------------------------------------------------------------------


#if qDebug || qTheDebugger

void InitUDebug()
{
	if ( TheDbgr_IsInstalled() ) 
	{
#if qTheDebugger
		gHighLevelDebuggerRunning = true;
		InstallWriteLnHook();
		TheDbgr_Init_Ext_Dbgr();		// Initialize Hooks for Jasik
		gCurrentDebugger = kxJasik;
#endif
	}
#if qDebug
	else
	{
		gHighLevelDebuggerRunning = false;
		FindHighLevelDebugger();
		// MA_REGISTER_CLASS(TDebugFlagsView);	// Registered in IApplication
		InstallWriteLnHook();
	}
#endif
	
	pUDebugInitialized = true;
}

#endif // qDebug || qTheDebugger

//----------------------------------------------------------------------------------------
// FindHighLevelDebugger:
//----------------------------------------------------------------------------------------


#if qDebug

void FindHighLevelDebugger()
{
	// First, check to see if the NutraBug Nub exists.
	if (NubIsInstalled())
		gHighLevelDebuggerRunning = true;

	// Check to see if one of the high level debuggers are running (SourceBug or Sade). 
	// If so then we will send debug output to it. 
	bool found = FindDebuggerProcess('Objr', kxSourceBug);	// Is SourceBug running?
	if (!found)
		found = FindDebuggerProcess('sade', kxSade);			// Is Sade running?
	if (!found)
		found = FindDebuggerProcess('MWDB', kxMWDebug68K);		// Is MWDebug68K running?
	if (!found)
		found = FindDebuggerProcess('MPDB', kxMWDebugPPC);		// Is MWDebugPPC running?
	if (!found)
		found = FindDebuggerProcess('R2Db', kxR2DbPPC);			// Is Apple's R2DbPPC running?
	if (!found)
		found = FindDebuggerProcess('Alar', kxR2Db68K);			// Is Apple's R2Db68K running?
	if (!found)
		found = FindDebuggerProcess('sDeb', kxVooDoo);			// Is VooDooMonkey running?
	
	gHighLevelDebuggerRunning = found;
	if (!found)
		gCurrentDebugger = kxNoDebugger;
}

#endif // qDebug

//----------------------------------------------------------------------------------------
// FindDebuggerProcess
//----------------------------------------------------------------------------------------


#if qDebug

bool FindDebuggerProcess(OSType sig, short debugger)
{
	ProcessSerialNumber psn;
	OSErr err = FindProcessBySignature(sig, psn, NULL);
	bool found = (err == noErr);
	if (found)
	{
		gCurrentDebugger = debugger;
		pDebuggerProcess = psn;
	}

	return found;
}

#endif // qDebug

//----------------------------------------------------------------------------------------
// DebugCanReadLn: Returns true if you can readln to the user
//----------------------------------------------------------------------------------------


#if qDebug

bool DebugCanReadLn()
{
	return false;
}

#endif	// qDebug

//----------------------------------------------------------------------------------------
// DebugCanWriteLn: Returns true if you can writeln to the user
//----------------------------------------------------------------------------------------


#if qDebug

bool DebugCanWriteLn()
{
	return pUDebugInitialized;
}

#endif

//----------------------------------------------------------------------------------------
// EachFrameDo:
//----------------------------------------------------------------------------------------


#if qDebug

void EachFrameDo(const CStackFrame& baseFrame,
				 DoToFrameType DoToFrame,
				 void* yourDataPtr)
{
	for (CStackFrameIterator iter(baseFrame); (CStackFrame)iter; ++iter)
		DoToFrame(iter, yourDataPtr);
}

#endif	// qDebug

//----------------------------------------------------------------------------------------
// GetCallersMethodName:
//----------------------------------------------------------------------------------------


#if qDebug

void GetCallersMethodName(char* s)
{
#if !q68K
	s = "non-68K (PPC or other platform)";
#else
	CStackFrame stackFrame(GetCurStackFramePtr());
	GetMethodName(stackFrame.GetReturnAddress(), s); // report about our caller"s caller 
#endif
}

#endif	// qDebug

//----------------------------------------------------------------------------------------
// GetMethodName: GetMethodName returns the name of the method (or procedure) in which pc
// points.
//----------------------------------------------------------------------------------------


#if qDebug

void GetMethodName(void* pc, char* s)
{
	char discardStr[255];
	GetProcName(pc, discardStr, s);
}

#endif	// qDebug

//----------------------------------------------------------------------------------------
// GetProcName: GetProcName returns the name of the procedure or function in which pc
// points. If it is in a method, then it return"s the name of the method"s class in
// className.
//----------------------------------------------------------------------------------------


#if qDebug

void GetProcName(void* 
#if q68K
	pc
#endif
	, char* className, char* procName)
{
#if q68K
	if (pc && !Odd_AC(pc))
	{
		void* nextPC;
		CStr255_AC localProcName;

		Ptr limit = (Ptr) ((ptrdiff_t) pc + 32767);

#if !(qModelCFM && defined(__MWERKS__))
		while (!endOfModule(pc, limit, (char*)(&localProcName), &nextPC))
		{
			if (pc >= limit)
			{
				*className = 0;
				*procName = 0;
				localProcName.Empty();
				return;
			}
			else
				pc = reinterpret_cast<void*>(static_cast<ptrdiff_t> (pc) + 2);
		}
#endif
		char cProcName[256];

		if (unmangle(cProcName, (char*)(&localProcName), 255) < 1)
			localProcName.CopyTo(procName);
		else
			procName = cProcName;

		short index = CStr255_AC(procName).Pos(".");
		if (index)
			(CStr255_AC(procName).Copy(1, index - 1)).CopyTo(className);
		else
			*className = 0;
	}
	else
#endif
	{
		*className = 0;
		*procName = 0;
	}
}

#endif	// qDebug

//========================================================================================
// Debug output with Apple's runtime library
//========================================================================================

#if qDebug || qTheDebugger
#if !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)

//----------------------------------------------------------------------------------------
// DevFAccess:
//----------------------------------------------------------------------------------------


long DevFAccess(char* fName, long command, void* /* arg */)
{
	long returnVal = -1;

	const bool CaseSensitive = true;
	const bool DiacritSensitive = true;
	const CStr255_AC fileName = fName;

	const CStr255_AC kConsoleName = "dev:console";		// console device name
	if (EqualString(fileName, kConsoleName, !CaseSensitive, DiacritSensitive))
		switch (command)
		{
			case F_OPEN:
				returnVal = 0;
				break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// DevClose:
//----------------------------------------------------------------------------------------


long DevClose(long)
{
	return 0;
}

//----------------------------------------------------------------------------------------
// DevRead:
//----------------------------------------------------------------------------------------


long DevRead(long iop)
{
	IOPort * iopPtr = (IOPort *)iop;
	long bytesRead;

	bytesRead = pGetProc(iopPtr->bufp, iopPtr->count);
	iopPtr->bufp += bytesRead;
	iopPtr->count -= bytesRead;

	return 0;
}

//----------------------------------------------------------------------------------------
// DevWrite:
//----------------------------------------------------------------------------------------


long DevWrite(long iop)
{
	IOPort * iopPtr = (IOPort *)iop;

	pPutProc(iopPtr->bufp, iopPtr->count);
	iopPtr->count = 0;

	return 0;
}

//----------------------------------------------------------------------------------------
// DevIoctl:
//----------------------------------------------------------------------------------------


long DevIoctl(long /* iop */,
			  long command,
			  void* /* arg */)
{
	long returnVal = -1;

	switch (command)
	{
		case FIOINTERACTIVE:
		case TIOFLUSH:
			returnVal = 0;
			break;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// SetGetProc:
//----------------------------------------------------------------------------------------


GetProcPtr SetGetProc(GetProcPtr theGetProc)
{
	GetProcPtr returnProc = pGetProc;
	pGetProc = theGetProc;

	return returnProc;
}

//----------------------------------------------------------------------------------------
// SetPutProc:
//----------------------------------------------------------------------------------------


PutProcPtr SetPutProc(PutProcPtr thePutProc)
{
	PutProcPtr returnProc = pPutProc;
	pPutProc = thePutProc;

	return returnProc;
}

//----------------------------------------------------------------------------------------
// DebugReadLn:
//----------------------------------------------------------------------------------------


long DebugReadLn(Ptr /*buffer*/, long /*byteCount*/)
{
	return 0;
}

#endif // !(THINK_CPLUS || SYMANTEC_CPLUS) && !defined(__MWERKS__)
#endif	// qDebug || qTheDebugger

//----------------------------------------------------------------------------------------
// ConfirmHighLevelDebugger:
//----------------------------------------------------------------------------------------


#if qDebug || qTheDebugger

void ConfirmHighLevelDebugger()
{
#if qTheDebugger
	if (gExtDbgr_Info.isBeingDebugged || TheDbgr_IsInstalled())	// keeps MMU from complaining
		gHighLevelDebuggerRunning = true;
#endif // qTheDebugger
#if qDebug
#if qTheDebugger
	else 
#endif // qTheDebugger
	if (gHighLevelDebuggerRunning)
	{
		if (NubIsInstalled())
			gHighLevelDebuggerRunning = true;

		OSErr err;
		ProcessInfoRec processInfo;
		
		// Check to see if the high level debugger is still running. The user could have
		// closed the application. If it is no longer running then just quit sending
		// output to the debugger.
		
		processInfo.processInfoLength = sizeof(ProcessInfoRec);
		processInfo.processName = NULL;
		processInfo.processAppSpec = NULL;
		
		err = GetProcessInformation(&pDebuggerProcess, &processInfo);
		if (err != noErr)
		{
			gHighLevelDebuggerRunning = false;
			gCurrentDebugger = kxNoDebugger;
		}
	}
#endif // qDebug
}

#endif	// qDebug || qTheDebugger

//----------------------------------------------------------------------------------------
// DebugWriteLnHook:
//----------------------------------------------------------------------------------------


#if qDebug || qTheDebugger

void DebugWriteLnHook(Ptr textBuf, long byteCount)
{
	ConfirmHighLevelDebugger();

	CStr255_AC debugStr;
	Ptr p = textBuf;
	while (byteCount > 0) 
	{
		long len = Min_AC(byteCount, 250);
		int i = 0;
		char* d = reinterpret_cast<char*>(&debugStr[1]);
		
			// parse through text and stop if newline reached.
		while (i < len)
		{
			char c = *p++;
			*d++ = c;
			if ((c == '\r') || (c == '\n'))
			{
				len = i + 1;
				break;
			}
			++i;
		}
		debugStr.Length() = static_cast<unsigned char>(i);

		if ((gCurrentDebugger == kxSourceBug) || (gCurrentDebugger == kxVooDoo) ||
			(gCurrentDebugger == kxSade))
		{
			debugStr += '\n';
			SysBreakFunc(debugStr);
		}
		else		
#if qTheDebugger
		if (gExtDbgr_Info.isBeingDebugged || TheDbgr_IsInstalled())
		{
			debugStr += '\n';
			SysBreakFunc(debugStr);
		}
		else		
#endif
		{
			ProgramBreak_AC( debugStr );
		}
		byteCount -= len;
	
	}
}

#endif // qDebug || qTheDebugger

//----------------------------------------------------------------------------------------
// InstallWriteLnHook:
//----------------------------------------------------------------------------------------


#if qDebug || qTheDebugger

void InstallWriteLnHook()
{
#if defined(__MWERKS__)
	InitCWDebug();
#elif (THINK_CPLUS || SYMANTEC_CPLUS)
	// Don't have Symantec debug support yet
#else
	fclose(stdin);
	fclose(stdout);
	fclose(stderr);
	const short _CODEV = 1;						// console device number
	_addDevHandler(_CODEV, 0, DevFAccess, DevClose, DevRead, DevWrite, DevIoctl);
	freopen("dev:console", "r", stdin);
	freopen("dev:console", "a", stdout);
	freopen("dev:console", "a", stderr);
	SetPutProc(DebugWriteLnHook);
	setvbuf(stderr, NULL, _IOLBF, 128);
	SetGetProc(DebugReadLn);
#endif
}

#endif // qDebug || qTheDebugger

//----------------------------------------------------------------------------------------
// End of UDebug.cp


