//----------------------------------------------------------------------------------------
// UPatch.cp 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UPATCH__
#include "UPatch.h"
#endif

// MacApp

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

// Toolbox

#if qPowerPC
	#ifndef __CODEFRAGMENTS__
	#include <CodeFragments.h>
	#endif
#endif
	
#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#if qPowerPC
	#ifndef __RESOURCES__
	#include <Resources.h>
	#endif
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

//----------------------------------------------------------------------------------------

TrapPatchPtr TrapPatch::pPatchList;					// global allocation declared in my header

//----------------------------------------------------------------------------------------
// PatchTrap: 
//----------------------------------------------------------------------------------------


TrapPatch::TrapPatch()
{
	trapNum = 0;
	oldTrapAddr = NULL;
	patchRoutine = NULL;
	nextPatch = NULL;
}

//----------------------------------------------------------------------------------------
// PatchTrap: 
//----------------------------------------------------------------------------------------


short TrapPatch::PatchTrap(short theTrapNum, void* theRoutine)
{
#if !qMSWindows	
	LookupOldTrapAddress(theTrapNum);
	trapNum = theTrapNum;
	nextPatch = pPatchList;
	pPatchList = this;
	NSetTrapAddress((UniversalProcPtr)theRoutine, theTrapNum, GetTrapType(theTrapNum));

	return noErr;
#endif
}

//----------------------------------------------------------------------------------------
// LookupOldTrapAddress:
//----------------------------------------------------------------------------------------


void TrapPatch::LookupOldTrapAddress(short theTrapNum)
{
#if !qMSWindows	
	oldTrapAddr = NGetTrapAddress(theTrapNum, GetTrapType(theTrapNum));
#endif
}

//----------------------------------------------------------------------------------------
// GetPreviousPatchPtr: walks the patch list backwards to return the patch record just
// prior to (*thePatchPtr) in the patch list
//----------------------------------------------------------------------------------------


TrapPatchPtr TrapPatch::GetPreviousPatchPtr()
{
	TrapPatchPtr returnVal = NULL;

	if (this != pPatchList)
	{
		returnVal = pPatchList;
		while (returnVal && returnVal->nextPatch != this)
			returnVal = returnVal->nextPatch;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// GetNewerPatchPtr: returns a newer patch record in the patch list which has the *same*
// trapNum as thePatch
//----------------------------------------------------------------------------------------


TrapPatchPtr TrapPatch::GetNewerPatchPtr()
{
	TrapPatchPtr newerPatch = NULL;
	TrapPatchPtr tempPatchPtr = pPatchList;

	while (tempPatchPtr && tempPatchPtr != this)
	{
		if (tempPatchPtr->trapNum == trapNum)
			newerPatch = tempPatchPtr;
		tempPatchPtr = tempPatchPtr->nextPatch;
	}

	return newerPatch;
}


//----------------------------------------------------------------------------------------
// UnpatchTrap: 
//----------------------------------------------------------------------------------------


void TrapPatch::UnpatchTrap()
{
#if !qMSWindows	
// You can't patch nothing, so we use the old trap address to keep track of if we
	// have a patch installed. 
	if (oldTrapAddr)
	{	
		// If this trap has a newer patch than the patch we're removing, then we have to take
	    // some extra special precautions. We have to muck with that patch's oldTrapAddr to
	    // point to this patch record's oldTrapAddr. We can pretty well ignore the case of
		// an older patch on this same trap since the trapaddress in our patch record will
		// be correct.
		
		// only set the trap address if there *isn't* a newer patch
		TrapPatchPtr newerPatchPtr = GetNewerPatchPtr();
		if (newerPatchPtr)
			// set up newerPatchPtr patch record so that it points to thePatch's OldTrapAddr 
			newerPatchPtr->oldTrapAddr = oldTrapAddr;
		else
			NSetTrapAddress(oldTrapAddr, trapNum, GetTrapType(trapNum));

		oldTrapAddr = NULL;
		
		if (patchRoutine)
		{
			DisposeRoutineDescriptor(patchRoutine);
			patchRoutine = NULL;
		}
	
		// Unlink the patch from the linked list of patches 
		if (this == pPatchList)
			pPatchList = nextPatch;
		else
		{
			TrapPatchPtr aPatchPtr = GetPreviousPatchPtr();
			if (aPatchPtr)					// Couldn't find thePatch, don't unpatch it
				aPatchPtr->nextPatch = nextPatch;
		}
		nextPatch = NULL;
	}
#endif
}

//----------------------------------------------------------------------------------------
// FlushCache: 
//----------------------------------------------------------------------------------------


void FlushCache()
{
#if q68K
	// Flush the instruction and data cache, if traps are enabled.
	long OldA5 = SetCurrentA5();	// ***** May be called from trap patches *****
	if (TrapExists(_HWPriv))
	{
		FlushDataCache();
		FlushInstructionCache();
	}
	SetA5(OldA5);
#endif
}

#if q68K
//----------------------------------------------------------------------------------------
// PatchJmpInstruction: 
//----------------------------------------------------------------------------------------


void PatchJmpInstruction(void* patchAddress, void* jumpAddress)
{
	JmpInstructionTemplate*	aJmpInstructionPtr = (JmpInstructionTemplate *) patchAddress;
	aJmpInstructionPtr->Jmp = 0x4EF9;
	aJmpInstructionPtr->Routine = jumpAddress;

	FlushCache();
}

#endif

//----------------------------------------------------------------------------------------
// UnpatchAll: 
//----------------------------------------------------------------------------------------


void TrapPatch::UnpatchAll()
{
	while (pPatchList)
		pPatchList->UnpatchTrap();
}


#if q68K
//----------------------------------------------------------------------------------------
// SetCallBack: 
//----------------------------------------------------------------------------------------


void SetCallBack(void* targProc, long itsRefCon, CallBackPtr theCallBackPtr)
{
	theCallBackPtr->saveRtnAdd = 0x2F17;
	theCallBackPtr->moveRefCon = 0x2F7C;
	theCallBackPtr->refCon = itsRefCon;
	theCallBackPtr->targOffset = 0x0004;
	theCallBackPtr->jmpInst = 0x4EF9;
	theCallBackPtr->jmpTarg = targProc;

	FlushCache();
}

#endif

//----------------------------------------------------------------------------------------
// End of UPatch.cp


