//----------------------------------------------------------------------------------------
// UPointerObject.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

// MacApp

#ifndef __UCLASSDESC__
#include "UClassDesc.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

// OpenDoc

#ifndef _MEMMGR_
#include "MemMgr.h"
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

#if qDebug

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#endif

//========================================================================================
// Global variable definitions
//========================================================================================

MemHeap* gObjectHeap;

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// MAOperatorNew: 
//----------------------------------------------------------------------------------------


void* MAOperatorNew(size_t size)
{
#if qDebug
	// Verify we're not being called from an MP task.
	::AssertToolboxSafeMPTask_AC();
#endif

	// Create the default object heap the first time we are invoked. If the overloads
	// of operator new/delete are omitted this code will be omitted as well and the
	// development system's new/delete will be used instead.
	
	if (!gObjectHeap)
	{
		gObjectHeap = MMNewHeap( kMMAppMemory, (size_t)GetSizeObjectHeap(), (size_t)GetSizeHeapIncrement(),
				"Default MacApp Heap" );

		// Set the default heap if none has been set yet
		if (!MMGetDefaultHeap())
			MMSetDefaultHeap(gObjectHeap);
	}
	
	void* blk = MMAllocateIn( size, gObjectHeap );
	::ThrowIfNULL_AC(blk);

#if qDebug
	BlockSet_AC(blk, size, 0x33); // enhanced bug finding
#else
	BlockSet_AC(blk, size, 0); // enhanced stability
#endif

	return blk;
}

//----------------------------------------------------------------------------------------
// MAOperatorDelete: 
//----------------------------------------------------------------------------------------
void MAOperatorDelete(void* obj)
{
#if qDebug
	if (!gObjectHeap)
		::ProgramBreak_AC("operator delete invoked prior to gObjectHeap being created!");
	else
	if (obj && ::Odd_AC(obj))
		::ProgramBreak_AC("In MAOperatorDelete: Not handed a valid object. NULL is ok but odd is not");

	// Verify we're not being called from an MP task.
	::AssertToolboxSafeMPTask_AC();
#endif
		MMFree(obj);
}


//----------------------------------------------------------------------------------------
// End of UPointerObject.cp


