//----------------------------------------------------------------------------------------
// UReporter.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#include "UReporter.h"

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UTHEDEBUGGER__
#include "UTheDebugger.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __PROCESSES__
#include <Processes.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef _STDLIB
#include <stdlib.h>
#endif

//----------------------------------------------------------------------------------------
// Class TReporter
//----------------------------------------------------------------------------------------

class TReporter : public CReporter_AC
{
public:
	TReporter();
	virtual ~TReporter();
	
	virtual void OutputMessage(const char* inStr, bool continueIfPossible = true);
};

static CAutoPtr_AC<TReporter> gMacAppReporter;


void InitUReporter()
{
	gMacAppReporter = TH_new TReporter;
}


//----------------------------------------------------------------------------------------
// TReporter ctor
//----------------------------------------------------------------------------------------

TReporter::TReporter()
{
	// What this odd looking code does is make sure that if a MacApp
	// application makes a subclass of CReporter_AC, that it always
	// ends up the current instance with this TReporter as the previous
	// reporter. This will work despite unknown order of contructors getting
	// called at static constructor time.

	CReporter_AC* old = SetIsCurrentInstance();
	if(old)
	{
		// if old is true, this means theres another installed reporter, probably
		// installed by the MacApp client, so we wan't that to be the current 
		// reporter. 
	
		old->SetIsCurrentInstance();
			// this will make this TReporter the previous reporter.
	}	
		
}

//----------------------------------------------------------------------------------------
// TReporter dtor
//----------------------------------------------------------------------------------------

TReporter::~TReporter()
{
}

//----------------------------------------------------------------------------------------
//  TReporter::OutputMessage
//----------------------------------------------------------------------------------------

void TReporter::OutputMessage(const char* inStr, bool continueIfPossible)
{
	static CStr255_AC grievance = inStr;
	static CStr255_AC report = grievance + "\n";
		// make these static to take up minimal room on stack.
		
#if qDebug || qTheDebugger
#if !defined(__MWERKS__) || (TypeOfCWDebugging != kxUseSTE)
	fflush(stderr);
#endif
#endif

#if qDebug
	if ((gCurrentDebugger == kxSourceBug) || (gCurrentDebugger == kxVooDoo) ||
		(gCurrentDebugger == kxSade))
	{
		if (!continueIfPossible)
			SysBreakStr(report);
		else
			SysBreakFunc(report);
	}
	else		
#endif

#if qTheDebugger
	if (gExtDbgr_Info.isBeingDebugged || TheDbgr_IsInstalled())
	{
		if (!continueIfPossible)
			SysBreakStr(report);
		else
			SysBreakFunc(report);
	}
	else		
#endif
	{
		CReporter_AC::OutputMessage(inStr, continueIfPossible);
	}

}

