// USectionMgr.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __USECTIONMGR__
#define __USECTIONMGR__

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __USECTION__
#include "USection.h"
#endif

// Toolbox

//----------------------------------------------------------------------------------------
// External and forward class declarations. 
//----------------------------------------------------------------------------------------

class TDocument;
class TPublisher;
class TSection;
class TSectionList;
class TSubscriber;


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const IDType kSectionMgrBehaviorID = 'smgr';			// behavior id for section mgr

//----------------------------------------------------------------------------------------
// TSectionMgr: This object is responsible for interaction with the Edition manager
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionMgr : public TBehavior
{
	MA_DECLARE_CLASS;
	
public:

	TSectionMgr();
		// constructor
		
	virtual ~TSectionMgr();
		// frees this object

	virtual void DoAESectionEvent(CommandNumber aCommandNumber,
								  const AppleEvent& message,
								  const AppleEvent& reply);
		// responds to the edition manager apple events

public:
	TSectionList fSectionList;
};


//----------------------------------------------------------------------------------------
// TSectionCommand: an abstract superclass used as the base class for the edition manager
// apple event command classes
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionCommand : public TServerCommand
{
public:
	TSectionCommand(CommandNumber itsCommandNumber = NULL,
						 TDocument* itsDocument = NULL,
						 TSection* itsSection = NULL);
		// constructor

	TSectionCommand(CommandNumber itsCommandNumber,
						 const AppleEvent& itsMessage,
						 const AppleEvent& itsReply);
		// constructor

	virtual ~TSectionCommand();
		// Destructor
		
public:
	TSection* fSection;							// section object for this apple event command
};


//----------------------------------------------------------------------------------------
// TSectionReadEventCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionReadEventCommand : public TSectionCommand
{
public:
	TSectionReadEventCommand(CommandNumber itsCommandNumber = cNoCommand,
								  TDocument* itsDocument = NULL,
								  TSubscriber* itsSubscriber = NULL);

	inline TSectionReadEventCommand(CommandNumber itsCommandNumber,
						 const AppleEvent& itsMessage,
						 const AppleEvent& itsReply) :
	TSectionCommand(itsCommandNumber, itsMessage, itsReply) { }
		// constructor

	virtual ~TSectionReadEventCommand();
		// Destructor
		
	virtual void DoIt();
		// call ((TSubscriber)fSection)->Subscribe
};


//----------------------------------------------------------------------------------------
// TSectionWriteEventCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionWriteEventCommand : public TSectionCommand
{
public:
	TSectionWriteEventCommand(CommandNumber itsCommandNumber = cNoCommand,
								   TDocument* itsDocument = NULL,
								   TPublisher* itsPublisher = NULL) :
	TSectionCommand(itsCommandNumber, itsDocument, itsPublisher) { }

	inline TSectionWriteEventCommand(CommandNumber itsCommandNumber,
						 const AppleEvent& itsMessage,
						 const AppleEvent& itsReply) :
	TSectionCommand(itsCommandNumber, itsMessage, itsReply) { }
		// constructor

	virtual ~TSectionWriteEventCommand();
		// Destructor
		
	virtual void DoIt();
		// call ((TPublisher)fSection))->Publish
};


//----------------------------------------------------------------------------------------
// TSectionScrollEventCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionScrollEventCommand : public TSectionCommand
{
public:
	TSectionScrollEventCommand(CommandNumber itsCommandNumber = cNoCommand,
									TDocument* itsDocument = NULL,
									TSection* itsSection = NULL) :
	TSectionCommand(itsCommandNumber, itsDocument, itsSection) { }

	inline TSectionScrollEventCommand(CommandNumber itsCommandNumber,
						 const AppleEvent& itsMessage,
						 const AppleEvent& itsReply) :
	TSectionCommand(itsCommandNumber, itsMessage, itsReply) { }
		// constructor

	virtual ~TSectionScrollEventCommand();
		// Destructor
		
	virtual void DoIt();
		// scrolls the document to the publisher
};


//----------------------------------------------------------------------------------------
// TSectionCancelEventCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionCancelEventCommand : public TSectionCommand
{
public:

	TSectionCancelEventCommand(CommandNumber itsCommandNumber = cNoCommand,
									TDocument* itsDocument = NULL,
									TSection* itsSection = NULL);

	inline TSectionCancelEventCommand(CommandNumber itsCommandNumber,
						 const AppleEvent& itsMessage,
						 const AppleEvent& itsReply) :
	TSectionCommand(itsCommandNumber, itsMessage, itsReply) { }
		// constructor

	virtual ~TSectionCancelEventCommand();
		// Destructor
		
	virtual void DoIt();
		// cancels this section

	virtual void UndoIt();
		// un-cancels this section
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

EXPORT_MA  void InitUSectionMgr();
	// Initialize the section manager and attach it to the application. Note that the
    // application should already have been instantiated and initialized when this
    // procedure is called.

#endif // __USECTIONMGR__
