//----------------------------------------------------------------------------------------
// UEditionDocument.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UEDITIONDOCUMENT__
#include "UEditionDocument.h"
#endif

// ACS

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

// MacApp

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USECTION__
#include "USection.h"
#endif

#ifndef __USECTIONMGR__
#include "USectionMgr.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


#ifndef __STRING__
#include <string.h>
#endif

//----------------------------------------------------------------------------------------
const unsigned long kEditionDocRsrcType = 'EdSt';// rsrc type for Edition Doc private rsrc 
const short kEditionDocRsrcID = 301;			 // rsrc id for Edition Doc private rsrc 
extern const char *kEditionDocRsrcName = "Edition Doc Settings";

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
typedef struct EditionDocSettings
{												 // used for saving settings in a resource 
	Boolean showSectionBorders;
	Boolean stopAllEditions;
}    *EditionDocSettingsPtr, ** EditionDocSettingsHandle;
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

//========================================================================================
// CLASS CSectionIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CSectionIterator::CSectionIterator: 
//----------------------------------------------------------------------------------------
CSectionIterator::CSectionIterator(const TEditionDocument* itsDocument,
								   ArrayIndex_AC itsLowBound,
								   ArrayIndex_AC itsHighBound,
								   bool itsForward) :
	TemplateIterator_AC<TSection*>(itsDocument ? itsDocument->GetSectionList() : NULL, itsLowBound, itsHighBound, itsForward)

{
}

//----------------------------------------------------------------------------------------
// CSectionIterator::CSectionIterator: 
//----------------------------------------------------------------------------------------
CSectionIterator::CSectionIterator(const TEditionDocument* itsDocument,
								   bool itsForward) :
	TemplateIterator_AC<TSection*>(itsDocument ? itsDocument->GetSectionList() : NULL, itsForward)
{
}

//----------------------------------------------------------------------------------------
// CSectionIterator::CSectionIterator: 
//----------------------------------------------------------------------------------------
CSectionIterator::CSectionIterator(const TEditionDocument* itsDocument) :
	TemplateIterator_AC<TSection*>(itsDocument ? itsDocument->GetSectionList() : NULL, kIterateForward_AC)
{
}

//----------------------------------------------------------------------------------------
// CSectionIterator::CSectionIterator: 
//----------------------------------------------------------------------------------------
CSectionIterator::CSectionIterator(const TSectionList* itsSectionList) :
	TemplateIterator_AC<TSection*>(itsSectionList, kIterateForward_AC)
{
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// InitUEditionDocument: 
//----------------------------------------------------------------------------------------
void InitUEditionDocument()
{
	::ThrowIfOSErr_AC(InitEditionPack());
	
	MA_REGISTER_SIGNATURE(TSectionBehavior, kSectionBehavior);

	// debug check to ensure that the user has included Editions.r in their <program>.r file
#if qDebugMsg
const short kAEEditionsDispatchTable = 401;		// ID of our 'aedt' resource
	Handle aedtResource = GetResource('aedt', kAEEditionsDispatchTable);
	if (!aedtResource)
	{
		ProgramBreak_AC("###Missing 'aedt' resource id=%d!\n", kAEEditionsDispatchTable);
		::ProgramBreak_AC("Did you forget to include Editions.rsrc in your .r file?\n");
	}
#endif
}


//========================================================================================
// CLASS TEditionDocument
//========================================================================================
MA_DEFINE_CLASS(TEditionDocument);

//----------------------------------------------------------------------------------------
// TEditionDocument constructor
//----------------------------------------------------------------------------------------
TEditionDocument::TEditionDocument(TFile* itsFile,
										const OSType itsScrapType,
										const OSType editionCreator) :
	TFileBasedDocument(itsFile, itsScrapType),

	fEditionCreator(editionCreator),
	fNextEditionNumber(1),
	fStopAllEditions(false),
	fShowSectionBorders(true),
	fNewPublishers(false)
{
	InitUSectionMgr();							// ensure it's init'd
}

//----------------------------------------------------------------------------------------
// TEditionDocument::Free: 
//----------------------------------------------------------------------------------------
TEditionDocument::~TEditionDocument()
{
	for (TemplateIterator_AC<TSection*> iter(&fSectionList); iter; ++iter)
		delete iter.Current();
}

//----------------------------------------------------------------------------------------
// TEditionDocument::FreeData: 
//----------------------------------------------------------------------------------------
void TEditionDocument::FreeData()	// override 
{
	for (TemplateIterator_AC<TSection*> iter(&fSectionList); iter; ++iter)
		delete iter.Current();
	fSectionList.DeleteAll();

	TFileBasedDocument::FreeData();
}

//----------------------------------------------------------------------------------------
// TEditionDocument::CanPublishSelection: 
//----------------------------------------------------------------------------------------
bool TEditionDocument::CanPublishSelection()
{
	bool returnVal = false;
	// we can publish if:
	// 1. there is a user selection, and
	// 2. the user selection contains something, and
	// 3. there isn't already a non-canceled publisher EXACTLY contained in the current selection.

	TDesignator * userSelection = GetUserSelection();
	if (userSelection && !userSelection->IsEmpty())
	{
		returnVal = true;
		for (CSectionIterator iter(this); iter.Current(); ++iter)
			if (!iter->IsCanceled() && (iter->GetSectionType() == stPublisher) && userSelection->IsContained(iter->GetDesignator()) == kExactlyContained)
			{
				returnVal = false;
				break;
			}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::CanSubscribe: 
//----------------------------------------------------------------------------------------
bool TEditionDocument::CanSubscribe()
{
	bool returnVal = false;
	// we can subscribe if:
	// 1. there is a user selection, and
	// 2. the user selection contains something, and
	// 3. there isn't a non-canceled subscriber currently selected (i.e. don't subscribe OVER a subscriber).

	TDesignator * userSelection = GetUserSelection();
	if (userSelection && !userSelection->IsEmpty())
	{
		returnVal = true;
		for (CSectionIterator iter(this); iter.Current(); ++iter)
			if (!iter->IsCanceled() && (iter->GetSectionType() == stSubscriber) && userSelection->IsContained(iter->GetDesignator()) != kNotContained)
			{
				returnVal = false;
				break;
			}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoWrite: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoWrite(TFile* aFile, bool makingCopy)	// override 
{
	TFileBasedDocument::DoWrite(aFile, makingCopy);

	// 0. warn user if doc has multiple publishers to same edition
	{
		TSection* prevPublisher = NULL;
		for (CSectionIterator iter(this); iter.Current(); ++iter)
			if (iter->GetSectionType() == stPublisher)
			{
				if (prevPublisher && ((*(prevPublisher->GetSectionHandle()))->controlBlock == (*(iter->GetSectionHandle()))->controlBlock))
				{
					PoseSavingMultPublishersAlert(prevPublisher);
					break;
				}
				prevPublisher = iter;
			}
	}

	// 1. write out (as a resource) fStopAllEditions and fShowSectionBorders 
	DoWriteSettings(aFile);

	// 2. clear the new publishers flag 
	fNewPublishers = false;

	// 3. tell each section to write itself out 
	for (CSectionIterator iter(this); iter.Current(); ++iter)
		iter->DoWrite(aFile, false);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoWriteSettings: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoWriteSettings(TFile* /*aFile*/)
{
	EditionDocSettingsHandle h = (EditionDocSettingsHandle)NewPermHandle(sizeof(EditionDocSettings));

	(*h)->showSectionBorders = fShowSectionBorders;
	(*h)->stopAllEditions = fStopAllEditions;
	
	MAAddResource((Handle)h, kEditionDocRsrcType, kEditionDocRsrcID, CStr255_AC(kEditionDocRsrcName));
	::ThrowIfResError_AC();
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoRead: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoRead(TFile* aFile, bool forPrinting)	// override 
{
	CTempUseResFile_AC tempUseResFile(aFile->GetRsrcRefNum());

	TFileBasedDocument::DoRead(aFile, forPrinting);

	// 1. read in (from a resource) fStopAllEditions and fShowSectionBorders 
	DoReadSettings(aFile);

	// 2. create the section objects and tell each section to read itself in 
	for (short i = 1; i <= Count1Resources(rSectionType); ++i)
	{
		short theID;
		Handle h;

		h = Get1IndResource(rSectionType, i);	// get the resource handle 
		::ThrowIfNULLResource_AC((Handle)h);
		theID = static_cast<short>((*reinterpret_cast<SectionHandle>(h))->sectionID);
		switch ((*reinterpret_cast<SectionHandle>(h))->kind)
		{
			case stPublisher:
				DoReadPublisher(aFile, theID);
				break;
			case stSubscriber:
				DoReadSubscriber(aFile, theID);
				break;
			default:
#if qDebug
				::ProgramBreak_AC("SectionHandle appears to be corrupt");
#endif
				break;
		}
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoReadSettings: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoReadSettings(TFile* /*aFile*/)
{
	EditionDocSettingsHandle h = (EditionDocSettingsHandle) Get1Resource(kEditionDocRsrcType, kEditionDocRsrcID);
	::ThrowIfNULLResource_AC((Handle)h);
	fShowSectionBorders = (*h)->showSectionBorders;
	fStopAllEditions = (*h)->stopAllEditions;
	ReleaseResource((Handle)h);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoReadPublisher: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoReadPublisher(TFile* aFile, short theID)
{
	CAutoPtr_AC<TPublisher> aPublisher(DoMakePublisher(NULL, NULL, theID));

	aPublisher->DoRead(aFile);
	AddSection(aPublisher);

	aPublisher.release();
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoReadSubscriber: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoReadSubscriber(TFile* aFile, short theID)
{
	CAutoPtr_AC<TSubscriber> aSubscriber(DoMakeSubscriber(NULL, NULL, theID));

	aSubscriber->DoRead(aFile);
	AddSection(aSubscriber);

	aSubscriber.release();
}

//----------------------------------------------------------------------------------------
// TEditionDocument::RevertDocument: 
//----------------------------------------------------------------------------------------
void TEditionDocument::RevertDocument()
{
	Abandon();
	TFileBasedDocument::RevertDocument();
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoMenuCommand: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoMenuCommand(CommandNumber aCommandNumber)// override 

{
	switch (aCommandNumber)
	{
		case cCreatePublisher:
			DoNewPublisher();
			break;

		case cSubscribeTo:
			PostCommand(TH_new TNewSubscriberCommand(cSubscribeTo, this));
			break;

		case cOptions:
			DoSectionOptions();
			break;

		case cBorders:
			PostCommand(TH_new TSectionBorderCommand(cBorders, this));
			break;

		case cStopAllEditions:
			PostCommand(TH_new TStopAllEditionsCommand(cStopAllEditions, this));
			break;

		default:
			TFileBasedDocument::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoSectionOptions: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoSectionOptions()

{
	TSection * aSection = GetSelectedSection();
	if (aSection)
	{
		if (aSection->GetSectionType() == stPublisher)
			DoPublisherOptions(dynamic_cast_AC(TPublisher*,aSection));
		else if (aSection->GetSectionType() == stSubscriber)
			DoSubscriberOptions(dynamic_cast_AC(TSubscriber*,aSection));
#if qDebug
		else
			::ProgramBreak_AC("what kind of section is it anyway?");
#endif
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::Close: 
//----------------------------------------------------------------------------------------
void TEditionDocument::Close()		// override 
{
	short poseResult = cancel;
	long changeCount;

	changeCount = GetChangeCount();
	
	// preflight the Close for the case in which the document's DATA is unchanged (the change
	// count is 0) but there are new publishers in the document
	if (!changeCount && fNewPublishers)
	{
		poseResult = PoseNewPublishersAlert();
		if (poseResult == cancel)
			::ThrowException_AC(noErr, messageCancelled);
	}
	else if (changeCount && fAskOnClose)
	{
		poseResult = PoseSaveDialog();
		if (poseResult == cancel)
			::ThrowException_AC(noErr, messageCancelled);
	}

	Changed(mClosed,this);

	if (poseResult == kYesButton)
		SaveDocument(cClose);				// Will fail if unable to save 
	else if (poseResult == kNoButton)
		Abandon();

	// Must never be called for a document related to a view in the Clipboard.
	// Why is this???
	for (CWindowIterator iter(this); iter.Current(); ++iter)
	{
		iter->Close();
		if (iter->GetFreeOnClosing())
			delete iter.Current();
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::PoseNewPublishersAlert: 
//----------------------------------------------------------------------------------------
short TEditionDocument::PoseNewPublishersAlert()

{
	short returnVal = kNoButton;

	::ThrowIfOSErr_AC(MAInteractWithUser());
	
	CStr255_AC reason(kIDBuzzString, gDispatcher->GetDone() ? bzQuitting : bzClosing);

	ParamText(fTitle, reason, gEmptyString, gEmptyString);
	
	CTempUseResFile_AC tempUseResFile(gApplicationRefNum);
	returnVal = MacAppAlert(phNewPublisherAlert, NULL);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::PoseMultPublishersAlert: 
//----------------------------------------------------------------------------------------
short TEditionDocument::PoseMultPublishersAlert(TSection* aSection)

{
	short returnVal = kNoButton;

	::ThrowIfOSErr_AC(MAInteractWithUser());

	CStr255_AC name;
	aSection->GetEditionName(name);

	ParamText(name, gEmptyString, gEmptyString, gEmptyString);
	
	CTempUseResFile_AC tempUseResFile(gApplicationRefNum);
	returnVal = MacAppAlert(phMultPublisherWrn, NULL);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::PoseSavingMultPublishersAlert: 
//----------------------------------------------------------------------------------------
short TEditionDocument::PoseSavingMultPublishersAlert(TSection* aSection)

{
	short returnVal = kNoButton;

	::ThrowIfOSErr_AC(MAInteractWithUser());

	CStr255_AC name;
	aSection->GetEditionName(name);

	ParamText(name, fTitle, gEmptyString, gEmptyString);
	
	CTempUseResFile_AC tempUseResFile(gApplicationRefNum);
	returnVal = MacAppAlert(phSavingMultPublisherWrn, NULL);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::Abandon: 
//----------------------------------------------------------------------------------------
void TEditionDocument::Abandon()		// override 
{
	for (CSectionIterator iter(this); iter.Current(); ++iter)
		if (iter->GetSectionType() == stPublisher)
		{
			// if publisher is *new* and it hasn't been saved yet,
			// delete the edition container file 
			if (dynamic_cast_AC(TPublisher*,iter.Current())->GetNewSection())
				dynamic_cast_AC(TPublisher*,iter.Current())->DeleteEditionFile();
		}
	
	fNewPublishers = false;						// no new publishers any more

	TFileBasedDocument::Abandon();
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoNeedDiskSpace(TFile* itsFile,
									   long& dataForkBytes,
									   long& rsrcForkBytes)
{
	TFileBasedDocument::DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);

	rsrcForkBytes += kRsrcTypeOverhead + kRsrcOverhead +
						sizeof(EditionDocSettings) + strlen(kEditionDocRsrcName) + kLengthByte;
	
	// if there are any sections to write, add in type overhead for alis', 'sect', and 'DESG'
	CSectionIterator iter(this);
	if (iter.Current())
		rsrcForkBytes += (3 * kRsrcTypeOverhead);

	for (; iter.Current(); ++iter)
		iter->DoNeedDiskSpace(dataForkBytes, rsrcForkBytes);
	
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoAddSectionBehavior: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoAddSectionBehavior(TView* itsView)
// a convenience method supplied for the convenience of the user
{
	if (itsView)
	{
		TBehavior* itsBehavior = itsView->GetBehaviorWithIdentifier(kSectionBehavior);
		if (!itsBehavior)
			itsView->AddBehavior(TH_new TSectionBehavior(kSectionBehavior));
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoNewPublisher: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoNewPublisher()

{
	OSErr err;
	TDesignator * itsDesignator;
	
	NewPublisherReply reply;	// can't be volatile because it is passed by reference

	// set up the reply record 
	err = GetLastEditionContainerUsed(&(reply.container));
	if ((err != noErr) && (err != fnfErr))		// allow fnfErr 
		::ThrowIfOSErr_AC(err);
	GetNextEditionName(*((CStr63_AC *) reply.container.theFile.name));
	reply.usePart = false;						// always set this to false 

	// create the preview 
	itsDesignator = GetUserSelection();
	::ThrowIfNULL_AC(itsDesignator);

	DoMakePreview(itsDesignator, reply.previewFormat, reply.preview);

	try
	{
		DoNewPublisherDialog(reply);			// query the user 
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		DisposeOfPreview(reply);
		throw;
	}
	
	
	DisposeOfPreview(reply);

	if (!reply.canceled)
	{
		if (!reply.replacing)
			::ThrowIfOSErr_AC(CreateEditionContainerFile(&(reply.container.theFile), GetEditionCreatorSignature(), reply.container.theFileScript));

		try
		{
			SectionHandle sectionH = NULL;	// can't be volatile because it is passed by reference
			
			TPublisher* aPublisher = NULL;
			TDesignator* newDesignator = NULL;
			
			// Prepare for call to NewSection
			short newSectionID = GetUniqueSectRsrcID();
	
			CFSSpec_AC aFSSpec;
			FSSpecPtr aFSSpecPtr = NULL;						// can pass in NULL to NewSection 
			TFile* aFile = GetFile();
			if (aFile && (aFile->HasValidFileSpec()))
			{
				aFSSpec = aFile->GetFileSpec();
				aFSSpecPtr = &aFSSpec;
			}
	
			// Call NewSection (which calls RegisterSection for us)
			err = NewSection(&(reply.container), aFSSpecPtr, stPublisher, newSectionID, pumOnSave, &sectionH);
			if ((err != noErr) && (err != multiplePublisherWrn) && (err != notThePublisherWrn))
				::ThrowIfOSErr_AC(err);
	
			try
			{
				// publisher owns its designator -- so clone it
				newDesignator = static_cast<TDesignator*>(itsDesignator->Clone());
		
				// Create the publisher object, add it to our list of sections,
				// mark it as changed, and publish it to disk.
				
				aPublisher = DoMakePublisher(newDesignator, sectionH, newSectionID);
				if ((err == multiplePublisherWrn) || (err == notThePublisherWrn))
					PoseMultPublishersAlert(aPublisher);
		
				AddSectionAndBorder(aPublisher);
				aPublisher->MarkAsChanged();
				aPublisher->Publish(aFile);
				
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				if (aPublisher)
					delete aPublisher;
				else
				{
					delete newDesignator;
					if (sectionH && (IsRegisteredSection(sectionH) == noErr))
						UnRegisterSection(sectionH);	// Ignore the warnings.
				}
				throw;
			}
			
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			err = DeleteEditionContainerFile(&(reply.container.theFile));
			throw;
		}
		

		// let the edition document know that there is a new publisher 
		fNewPublishers = true;
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoMakePreview: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoMakePreview(TDesignator* itsDesignator,
									 FormatType& previewFormat,
									 Handle& preview)
{
	preview = NULL;
	previewFormat = GetPublishPreference();

	if (previewFormat == kTextType_AC || previewFormat == kPictType_AC || previewFormat == kSoundType_AC)
	{
		CTempHandle_AC tempPreview(NewPermHandle(0));

		DoWriteData(previewFormat, itsDesignator, &THandleStream(tempPreview, 6));
		
		preview = tempPreview.release();
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DisposeOfPreview: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DisposeOfPreview(NewPublisherReply& reply)
{
	if (reply.previewFormat == kPictType_AC)
		reply.preview = (Handle)DisposeIfPicHandle((PicHandle) reply.preview);
	else if (reply.previewFormat == kTextType_AC)
		reply.preview = ::DisposeIfHandle_AC(reply.preview);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::GetPublishPreference: 
//----------------------------------------------------------------------------------------
FormatType TEditionDocument::GetPublishPreference()

{
	return kTextType_AC;						// by default, prefer to publish 'TEXT'
                                                // over 'PICT'
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoMakePublisher: 
//----------------------------------------------------------------------------------------
TPublisher* TEditionDocument::DoMakePublisher(TDesignator* itsDesignator,
											  SectionHandle itsSectionHandle,
											  short itsSectionID)
{
	return TH_new TPublisher(this, itsDesignator, itsSectionHandle, itsSectionID);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoNewPublisherDialog: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoNewPublisherDialog(NewPublisherReply& reply)

{
	// override this method if you decide to use the NewPublisherExpDialog instead 
	::ThrowIfOSErr_AC(MAInteractWithUser()); 

	gClipboardMgr->AboutToLoseControl(true);	// so scrap gets converted
	::ThrowIfOSErr_AC(NewPublisherDialog(&reply));
	gClipboardMgr->RegainControl(true);			// so scrap gets converted
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoNewSubscriber: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoNewSubscriber()

{
	NewSubscriberReply reply;
	OSErr err = GetLastEditionContainerUsed(&(reply.container));// fill in the reply 

	if ((err != noErr) && (err != fnfErr))		// allow fnfErr 
		::ThrowIfOSErr_AC(err);
	reply.formatsMask = GetSubscriberFormatsMask();// set up the formatsMask 
	DoNewSubscriberDialog(reply);

	if (!reply.canceled)
	{
		short newSectionID = GetUniqueSectRsrcID();
		// create a new section handle 
		CFSSpec_AC aFSSpec;
		FSSpecPtr aFSSpecPtr = NULL;
		TFile* aFile = GetFile();
		if (aFile && (aFile->HasValidFileSpec()))
		{
			aFSSpec = aFile->GetFileSpec();
			aFSSpecPtr = &aFSSpec;
		}
		SectionHandle sectionH;
		::ThrowIfOSErr_AC(NewSection(&(reply.container), aFSSpecPtr, stSubscriber, newSectionID, sumAutomatic, &sectionH));
		// NewSection calls RegisterSection for us 

		// create a TDesignator for the TSubscriber object 
		TDesignator* aDesignator = GetUserSelection();
		if (aDesignator)
			aDesignator = static_cast<TDesignator*>(aDesignator->Clone());// the subscriber owns the designator

		// create a TSubscriber object, add it to our list of sections, and mark it as changed
		TSubscriber* aSubscriber = DoMakeSubscriber(aDesignator, sectionH, newSectionID);
		AddSectionAndBorder(aSubscriber);
		aSubscriber->MarkAsChanged();
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoMakeSubscriber: 
//----------------------------------------------------------------------------------------
TSubscriber* TEditionDocument::DoMakeSubscriber(TDesignator* itsDesignator,
												SectionHandle itsSectionHandle,
												short itsSectionID)
{
	return TH_new TSubscriber(this, itsDesignator, itsSectionHandle, itsSectionID);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoNewSubscriberDialog: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoNewSubscriberDialog(NewSubscriberReply& reply)

{
	// override this method if you decide to use the NewSubscriberExpDialog instead 
	::ThrowIfOSErr_AC(MAInteractWithUser()); 

	gClipboardMgr->AboutToLoseControl(true);		// so scrap gets converted
	::ThrowIfOSErr_AC(NewSubscriberDialog(&reply));		// pose the new subscriber dialog 
	gClipboardMgr->RegainControl(true);			// so scrap gets converted
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoPublisherOptions: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoPublisherOptions(TPublisher* aPublisher)

{
	SectionOptionsReply reply;

	reply.sectionH = aPublisher->GetSectionHandle();
	DoPublisherOptionsDialog(reply);
	if (!reply.canceled)
	{
		if (reply.changed)
			aPublisher->MarkAsChanged();

		if (reply.action == sectionWriteMsgID)
			aPublisher->Publish(GetFile());
		else if (reply.action == sectionCancelMsgID)
			PostCommand(TH_new TSectionCancelEventCommand(cCancelPublisher, this, aPublisher));
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoPublisherOptionsDialog: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoPublisherOptionsDialog(SectionOptionsReply& reply)

{
	// override this method if you decide to use the SectionOptionsExpDialog instead 
	::ThrowIfOSErr_AC(MAInteractWithUser()); 

	gClipboardMgr->AboutToLoseControl(true);		// so scrap gets converted
	::ThrowIfOSErr_AC(SectionOptionsDialog(&reply));
	gClipboardMgr->RegainControl(true);			// so scrap gets converted
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoSubscriberOptions: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoSubscriberOptions(TSubscriber* aSubscriber)

{
	SectionOptionsReply reply;

	reply.sectionH = aSubscriber->GetSectionHandle();
	DoSubscriberOptionsDialog(reply);
	if (!reply.canceled)
	{
		if (reply.changed)
			aSubscriber->MarkAsChanged();

		if (reply.action == sectionReadMsgID)
			aSubscriber->Subscribe();
		else if (reply.action == 'goto')
			aSubscriber->OpenPublisher();
		else if (reply.action == sectionCancelMsgID)
			PostCommand(TH_new TSectionCancelEventCommand(cCancelSubscriber, this, aSubscriber));
	}
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoSubscriberOptionsDialog: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoSubscriberOptionsDialog(SectionOptionsReply& reply)

{
	// override this method if you decide to use the SectionOptionsExpDialog instead 
	::ThrowIfOSErr_AC(MAInteractWithUser()); 

	gClipboardMgr->AboutToLoseControl(true);	// so scrap gets converted
	::ThrowIfOSErr_AC(SectionOptionsDialog(&reply));
	gClipboardMgr->RegainControl(true);			// so scrap gets converted
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoSetupMenus: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoSetupMenus()			// override 

{
	TFileBasedDocument::DoSetupMenus();

	if (fNewPublishers)						// there are new publishers! 
	{
		Enable(cSave, true);
		Enable(cRevert, true);
	}

	Enable(cCreatePublisher, CanPublishSelection());
	Enable(cSubscribeTo, CanSubscribe());

	TSection * aSection = GetSelectedSection();
	if (aSection)
	{
		Enable(cOptions, true);
		if (aSection->GetSectionType() == stPublisher)
			SetIndividualCommandName(cOptions, kIDBuzzString, bzPublisherOptions);
		else if (aSection->GetSectionType() == stSubscriber)
			SetIndividualCommandName(cOptions, kIDBuzzString, bzSubscriberOptions);
#if qDebug
		else
			::ProgramBreak_AC("what kind of section is it anyway?");
#endif
	}

	Enable(cBorders, true);
	SetMenuState(cBorders, kIDBuzzString, bzShowBorders, bzHideBorders, fShowSectionBorders);

	EnableCheck(cStopAllEditions, true, fStopAllEditions);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::GetNextEditionName: 
//----------------------------------------------------------------------------------------
void TEditionDocument::GetNextEditionName(CStr63_AC& editionName)

{
	CStr255_AC name(kIDBuzzString, bzUntitled);

	short preInsert;
	short constChars;
	if (ParseTitleTemplate(name, preInsert, constChars))
	{
		CStr255_AC numStr;
		NumToString(fNextEditionNumber, numStr);
		if (SubstituteInTitle(name, numStr, preInsert, constChars))
			++fNextEditionNumber;
	}

	editionName = name;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::GetSubscriberFormatsMask: 
//----------------------------------------------------------------------------------------
SignedByte TEditionDocument::GetSubscriberFormatsMask()

{
	// Return a mask corresponding to the edition format type to display within the
    // subscriber dialog box. This mask may consist of as many types as are supported in
    // the application. Construct the mask as follows:
	//  GetSubscriberFormatsMask = [kPICTformatMask] [+] [kTextformatMask [+] [ksndFormatMask]

	SignedByte returnVal = 0;

	if (GetPublishPreference() == kTextType_AC)
		returnVal = kTEXTformatMask;
	else if (GetPublishPreference() == kPictType_AC)
		returnVal = kPICTformatMask;
	else if (GetPublishPreference() == kSoundType_AC)
		returnVal = ksndFormatMask;
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::GetUniqueSectRsrcID: 
//----------------------------------------------------------------------------------------
const short kFirstRsrcID = 1000;
const short kLastRsrcID = 32767;
const short kBitsInArray = kLastRsrcID - kFirstRsrcID;

short TEditionDocument::GetUniqueSectRsrcID()
{
	short counter = 0;
	Size arraySize = (kBitsInArray / sizeof(Byte)) + 1;
	Ptr aPtr = NewPermPtr(arraySize);
	bool foundNewID;

	BlockSet_AC(aPtr, arraySize, 0x00);			// zero it out
	
	for (CSectionIterator iter(this); iter.Current(); ++iter)
	{
		if (iter->GetRsrcID() >= kFirstRsrcID)
			BitSet(aPtr, iter->GetRsrcID() - kFirstRsrcID);
		else
		{
#if qDebug
			::ProgramBreak_AC("problem with section rsrc id");
#endif
		}
	}

	while (BitTst(aPtr, counter) && (counter < kBitsInArray))
		++counter;

	foundNewID =!BitTst(aPtr, counter);			// see if we got a new rsrc id 

	aPtr = ::DisposeIfPtr_AC(aPtr);					// dispose of the array 

	if (!foundNewID)
		::ThrowException_AC(minErr, 0);

	return (counter + kFirstRsrcID);			// return the new sect rsrc id 
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoPostMakeViews: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoPostMakeViews(bool forPrinting)
{
	// The case that we're concerned with here is the TApplication::OpenOld case, in which
    // our DoRead method will get called before there are any views. This method provides
    // a way for borders to be added to views "after the fact" to handle the OpenOld case.
    // The document will have been read in (DoRead), the views created (DoMakeViews) and
    // finally the post create method called (DoPostMakeViews) at which point the borders
    // can then be safely added.

	if (!forPrinting)
		for (CSectionIterator iter(this); iter.Current(); ++iter)
			DoAddBorder(iter);

	TFileBasedDocument::DoPostMakeViews(forPrinting);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoAddBorder: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoAddBorder(TSection* /* aSection */)

{
	// subclasses should override this to create a border to adorn this section
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoAdjustBorder: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoAdjustBorder(TSection* /* aSection */)

{
	// subclasses should override this to create a border to adorn this section
}

//----------------------------------------------------------------------------------------
// TEditionDocument::DoDeleteBorder: 
//----------------------------------------------------------------------------------------
void TEditionDocument::DoDeleteBorder(TSection* /* aSection */)

{
	// subclasses should override this to remove the border adorning this section
}

//----------------------------------------------------------------------------------------
// TEditionDocument::IsBorderShown: 
//----------------------------------------------------------------------------------------
bool TEditionDocument::IsBorderShown(TSection* aSection)

{
	return !aSection->IsCanceled() ? fShowSectionBorders || GetSelectedSection() == aSection : false;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::IsSectionSelected: 
//----------------------------------------------------------------------------------------
bool TEditionDocument::IsSectionSelected(TSection* aSection)
// !!! should really be called TEditionDocument::IsSectionInSelection
{
	bool returnVal = false;

	if (aSection && !aSection->IsCanceled() && aSection->GetDesignator())
	{
		TDesignator* userSelection = GetUserSelection();
		if (userSelection)
			returnVal = userSelection->IsContained(aSection->GetDesignator()) != kNotContained;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::AddSection: 
//----------------------------------------------------------------------------------------
void TEditionDocument::AddSection(TSection* aSection)

{
	// Protect against double installation 
	if (!fSectionList.GetIdentityItemNo(aSection))// doesn't already exist in list 
			fSectionList.Insert(aSection);
	
	// add the section to the section mgr's list of registered sections
	TSectionMgr* aSectionMgr = dynamic_cast<TSectionMgr*>(gDispatcher->GetBehaviorWithIdentifier(kSectionMgrBehaviorID));
	if (aSectionMgr)
		aSectionMgr->fSectionList.Insert(aSection);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::AddSectionAndBorder: 
//----------------------------------------------------------------------------------------
void TEditionDocument::AddSectionAndBorder(TSection* aSection)

{
	AddSection(aSection);
	// notify the document to create a border for this section
	DoAddBorder(aSection);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::RemoveSection: 
//----------------------------------------------------------------------------------------
void TEditionDocument::RemoveSection(TSection* aSection)

{
		fSectionList.Delete(aSection);

	// add the section to the section mgr's list of registered sections
	TSectionMgr* aSectionMgr = dynamic_cast<TSectionMgr*>(gDispatcher->GetBehaviorWithIdentifier(kSectionMgrBehaviorID));
	if (aSectionMgr)
		aSectionMgr->fSectionList.Delete(aSection);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::CancelSection: 
//----------------------------------------------------------------------------------------
void TEditionDocument::CancelSection(TSection* aSection, bool cancel)
{
	aSection->CancelSection(cancel);
	
	// notify the document to add or delete a border for this section
	if (cancel)
		DoDeleteBorder(aSection);
	else
		DoAddBorder(aSection);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::GetSelectedSection: 
//----------------------------------------------------------------------------------------
TSection* TEditionDocument::GetSelectedSection()
{
	TSection* selectedSection = NULL;

	for (CSectionIterator iter(this); iter.Current(); ++iter)
		if (IsSectionSelected(iter))
			if (!selectedSection)
				selectedSection = iter;
			else
			{	// in this case, >1 sections are in the user selection, so need to
				// determine which section is the preferred selected section
				if (iter->GetDesignator()->GetSize() < selectedSection->GetDesignator()->GetSize())
					selectedSection = iter;
			}
	
	return selectedSection;
}

//----------------------------------------------------------------------------------------
// TEditionDocument::SetSectionBorders: 
//----------------------------------------------------------------------------------------
void TEditionDocument::SetSectionBorders(bool state, bool invalidate)
{
	if (fShowSectionBorders != state)
	{
		fShowSectionBorders = state;

		if (invalidate)
			for (CWindowIterator iter(this); iter.Current(); ++iter)
				iter->ForceRedraw();
	}
	
	// Something like the following could be done instead:
	//		CAdornerIterator iter(fViewWithBorders);
	//		for (CWindowIterator iter(this); iter.Current; ++iter)
	//			if ((iter->fIdentifier == kPublisherAdornerID) || (iter->fIdentifier == kSubscriberAdornerID))
	//				iter->InvalidateAdorner(fCellsView);
}

//----------------------------------------------------------------------------------------
// TEditionDocument::SetStopAllEditions: 
//----------------------------------------------------------------------------------------
void TEditionDocument::SetStopAllEditions(bool state)
{
	if (fStopAllEditions != state)
	{
		fStopAllEditions = state;
		if (!state)								// if turning "off" stop editions 
			for (CSectionIterator iter(this); iter.Current(); ++iter)
				if (iter->GetSectionType() == stSubscriber)
				{
					dynamic_cast_AC(TSubscriber*,iter.Current())->SubscribeIfNewer();
				}
	}
}


//========================================================================================
// CLASS TNewSubscriberCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TNewSubscriberCommand::TNewSubscriberCommand: 
//----------------------------------------------------------------------------------------
TNewSubscriberCommand::TNewSubscriberCommand(CommandNumber itsCommandNumber,
												  TDocument* itsDocument) :
	TCommand(itsCommandNumber, itsDocument, kCantUndo, kCausesChange, itsDocument)
{
}

//----------------------------------------------------------------------------------------
// TNewSubscriberCommand destructor
//----------------------------------------------------------------------------------------
TNewSubscriberCommand::~TNewSubscriberCommand()
{
}

//----------------------------------------------------------------------------------------
// TNewSubscriberCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TNewSubscriberCommand::DoIt()
{
	dynamic_cast_AC(TEditionDocument*,fContext)->DoNewSubscriber();
}


//========================================================================================
// CLASS TSectionBorderCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSectionBorderCommand::TSectionBorderCommand: 
//----------------------------------------------------------------------------------------
TSectionBorderCommand::TSectionBorderCommand(CommandNumber itsCommandNumber,
												  TDocument* itsDocument) :
	TCommand(itsCommandNumber, itsDocument->GetContext(itsCommandNumber), kCanUndo, kCausesChange, itsDocument),

	fShowSectionBorders(!dynamic_cast_AC(TEditionDocument*,itsDocument)->GetShowSectionBorders())
{
}

//----------------------------------------------------------------------------------------
// TSectionBorderCommand destructor
//----------------------------------------------------------------------------------------
TSectionBorderCommand::~TSectionBorderCommand()
{
}

//----------------------------------------------------------------------------------------
// TSectionBorderCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TSectionBorderCommand::DoIt()

{
	TEditionDocument* doc = dynamic_cast_AC(TEditionDocument*,fContext);
	
	doc->SetSectionBorders(fShowSectionBorders, kInvalidate);
}

//----------------------------------------------------------------------------------------
// TSectionBorderCommand::UndoIt: 
//----------------------------------------------------------------------------------------
void TSectionBorderCommand::UndoIt()

{
	TEditionDocument* doc = dynamic_cast_AC(TEditionDocument*,fContext);
	
	doc->SetSectionBorders(!fShowSectionBorders, kInvalidate);
}


//========================================================================================
// CLASS TStopAllEditionsCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TStopAllEditionsCommand::TStopAllEditionsCommand: 
//----------------------------------------------------------------------------------------
TStopAllEditionsCommand::TStopAllEditionsCommand(CommandNumber itsCommandNumber, TDocument* itsDocument) 
	: TCommand(itsCommandNumber, itsDocument->GetContext(itsCommandNumber), kCanUndo, kCausesChange, itsDocument),
	fStopAllEditions(!dynamic_cast_AC(TEditionDocument*,itsDocument)->GetStopAllEditions()) 
{
}

//----------------------------------------------------------------------------------------
// TStopAllEditionsCommand destructor
//----------------------------------------------------------------------------------------
TStopAllEditionsCommand::~TStopAllEditionsCommand()
{
}

//----------------------------------------------------------------------------------------
// TStopAllEditionsCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TStopAllEditionsCommand::DoIt()

{
	TEditionDocument* doc = dynamic_cast_AC(TEditionDocument*,fContext);
	doc->SetStopAllEditions(fStopAllEditions);
}

//----------------------------------------------------------------------------------------
// TStopAllEditionsCommand::UndoIt: 
//----------------------------------------------------------------------------------------
void TStopAllEditionsCommand::UndoIt()

{
	TEditionDocument* doc = dynamic_cast_AC(TEditionDocument*,fContext);
	doc->SetStopAllEditions(!fStopAllEditions);
}


//========================================================================================
// CLASS TSectionAdorner
//========================================================================================
MA_DEFINE_CLASS(TSectionAdorner);

//----------------------------------------------------------------------------------------
// TSectionAdorner::TSectionAdorner: 
//----------------------------------------------------------------------------------------
TSectionAdorner::TSectionAdorner(IDType itsID,
									  TSection* itsSection,
									  RgnHandle itsBorderRegion) :
	TAdorner(itsID, kFreeOnDeletion),

	fSection(itsSection),
	fBorderRegion(itsBorderRegion)
{
}

//----------------------------------------------------------------------------------------
// TSectionAdorner::Free: 
//----------------------------------------------------------------------------------------
TSectionAdorner::~TSectionAdorner()
{
}

//----------------------------------------------------------------------------------------
// TSectionAdorner::InvalidateAdorner: 
//----------------------------------------------------------------------------------------
void TSectionAdorner::InvalidateAdorner(TView* itsView)	// override 

{
	if (itsView && fBorderRegion)
		itsView->Invalidate(fBorderRegion);
}

//----------------------------------------------------------------------------------------
// TSectionAdorner::DrawBorder: 
//----------------------------------------------------------------------------------------
void TSectionAdorner::DrawBorder(const Pattern& whichPattern)

{
	PenNormal();
	PenMode(patXor);
	PenSize(3, 3);
	PenPat(&whichPattern);

	// Don't use FillRgn since it ignores the penMode. 
	FrameRgn(fBorderRegion);
}


//========================================================================================
// CLASS TSubscriberAdorner
//========================================================================================
MA_DEFINE_CLASS(TSubscriberAdorner);

//----------------------------------------------------------------------------------------
// TSubscriberAdorner destructor
//----------------------------------------------------------------------------------------
TSubscriberAdorner::~TSubscriberAdorner()
{
}

//----------------------------------------------------------------------------------------
// TSubscriberAdorner::DoHighlightSelection: 
//----------------------------------------------------------------------------------------
void TSubscriberAdorner::DoHighlightSelection(TView* itsView,
											  const CViewRect& ,
											  HLState fromHL,
											  HLState toHL)	// override 
{
	if (itsView)
	{
		TEditionDocument * itsDocument = dynamic_cast<TEditionDocument*>(itsView->GetDocument());
		if (itsDocument && (itsDocument->IsBorderShown(fSection)))
		{
			if ((fromHL == hlOn) && ((toHL == hlDim) || (toHL == hlOff)))
				DrawBorder(qd.dkGray);			// turning off 
			else if (((fromHL == hlOff) || (fromHL == hlDim)) && (toHL == hlOn))
				DrawBorder(qd.dkGray);			// turning on 
		}
	}
}


//========================================================================================
// CLASS TPublisherAdorner
//========================================================================================
MA_DEFINE_CLASS(TPublisherAdorner);

//----------------------------------------------------------------------------------------
// TPublisherAdorner destructor
//----------------------------------------------------------------------------------------
TPublisherAdorner::~TPublisherAdorner()
{
}

//----------------------------------------------------------------------------------------
// TPublisherAdorner::DoHighlightSelection: 
//----------------------------------------------------------------------------------------
void TPublisherAdorner::DoHighlightSelection(TView* itsView,
											 const CViewRect& ,
											 HLState fromHL,
											 HLState toHL)	// override 
{
	if (itsView)
	{
		TEditionDocument * itsDocument = dynamic_cast<TEditionDocument*>(itsView->GetDocument());
		if ((itsDocument) && (itsDocument->IsBorderShown(fSection)))
		{
			if ((fromHL == hlOn) && ((toHL == hlDim) || (toHL == hlOff)))
				DrawBorder(qd.gray);			// turning off 
			else if (((fromHL == hlOff) || (fromHL == hlDim)) && (toHL == hlOn))
				DrawBorder(qd.gray);			// turning on 
		}
	}
}


//========================================================================================
// CLASS TSectionBehavior
//========================================================================================
MA_DEFINE_CLASS(TSectionBehavior);

//----------------------------------------------------------------------------------------
// TSectionBehavior destructor
//----------------------------------------------------------------------------------------
TSectionBehavior::~TSectionBehavior()
{
}

//----------------------------------------------------------------------------------------
// TSectionBehavior::DoMouseCommand: 
//----------------------------------------------------------------------------------------
bool TSectionBehavior::DoMouseCommand(CViewPoint& theMouse,
										 TToolboxEvent* event,
										 CPoint_AC hysteresis)
{
	bool returnVal = false;

	TView* itsView = dynamic_cast_AC(TView*,fOwner);
	
	// itsView->Focus();
	CPoint_AC qdMouse(itsView->ViewToQD(theMouse));

	for (CAdornerIterator iter(itsView); iter && !returnVal; ++iter)
		if ((iter->GetIdentifier() == kPublisherAdornerID) || (iter->GetIdentifier() == kSubscriberAdornerID))
		{
			TEditionDocument* itsEditionDocument = dynamic_cast_AC(TEditionDocument*,itsView->GetDocument());
			
			TSectionAdorner* itsSectionAdorner = dynamic_cast_AC(TSectionAdorner*,iter.Current());
			
			TSection* itsSection = itsSectionAdorner->GetSection();
			if ((event->GetClickCount() == 1) && (PtInRgn(qdMouse, itsSectionAdorner->GetBorderRegion())))
			{
				TDesignator* theSelection = itsSection->GetDesignator();
				if (theSelection)
				{
					theSelection = static_cast<TDesignator*>(theSelection->Clone());
					if (theSelection)
					{
						TView* view = dynamic_cast_AC(TView*,fOwner);
						
						view->SetUserSelection(theSelection);
					}
				}
				returnVal =true;
			}
			else if ((event->GetClickCount() > 1) && (PtInRgn(qdMouse, itsSectionAdorner->GetBorderRegion())))
			{
				if (itsSection->GetSectionType() == stPublisher)
					itsEditionDocument->DoPublisherOptions(dynamic_cast_AC(TPublisher*,itsSection));
				else
				{
					if (event->IsOptionKeyPressed())
					{
						TSubscriber* subscriber = dynamic_cast_AC(TSubscriber*,itsSection);
						
						subscriber->OpenPublisher();
					}
					else
						itsEditionDocument->DoSubscriberOptions(dynamic_cast_AC(TSubscriber*,itsSection));
				}
				returnVal = true;
			}
		}
	
	return returnVal || TBehavior::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// TSectionBehavior::GetStandardSignature: 
//----------------------------------------------------------------------------------------
IDType TSectionBehavior::GetStandardSignature() const
{
	return kSectionBehavior;
}

//----------------------------------------------------------------------------------------
// End of UEditionDocument.cp
