//----------------------------------------------------------------------------------------
// UFileBasedDocument.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UFILEBASEDDOCUMENT__
#include "UFileBasedDocument.h"
#endif

// ACS

#ifndef __CIterator_AC__
#include "CIterator_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UFILESTREAM__
#include "UFileStream.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FOLDERS__
#include <Folders.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

//========================================================================================
// CLASS TSaveFileDocCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSaveFileDocCommand constructor: 
//----------------------------------------------------------------------------------------
TSaveFileDocCommand::TSaveFileDocCommand(TDocument* itsDocument)
	:	TSaveCommand(itsDocument, cSave)
{
	TFileBasedDocument*		aDocument = dynamic_cast_AC(TFileBasedDocument*,itsDocument);
	
	// By the time we get into this command we should already
	// have a file which exists.  If there is no file then
	// the Save As command should have been used before the
	// Save command is available.
	if (!aDocument->FileExists())
	{
		#if qDebug
			::ProgramBreak_AC("TSaveFileDocCommand::TSaveFileDocCommand called but no file exists!");
		#endif
		ThrowError_AC(fnfErr);		// Throw a file-not-found error.
	}
		
	fSaveFile = aDocument->DoMakeFile(cSave);
	fSaveFile->SpecifyWithFile(aDocument->GetFile());
}

//----------------------------------------------------------------------------------------
// TSaveFileDocCommand constructor: 
//----------------------------------------------------------------------------------------
TSaveFileDocCommand::TSaveFileDocCommand(TDocument* itsDocument,
											  TAppleEvent* message,
										 TAppleEvent* reply,
										 TFile* saveFile)
	:	TSaveCommand(itsDocument, message, reply, cSave),
		fSaveFile(saveFile)
{
	// Do we know to where we are to save the document?  We should
	// since the dispatcher (DoAESave) will have already converted this to
	// a cSaveAs otherwise.
	TFileBasedDocument*		aDocument = dynamic_cast_AC(TFileBasedDocument*,itsDocument);
	
	if (!aDocument->FileExists())
	{
		#if qDebug
			::ProgramBreak_AC("TSaveFileDocCommand::TSaveFileDocCommand called but no file exists!");
		#endif
		::ThrowError_AC(fnfErr);		// Throw a file-not-found error.
	}
	if (message->HasParameter(keyAEFileType))
		fSaveFile->SetFileType(message->ReadType(keyAEFileType));
}

//----------------------------------------------------------------------------------------
// TSaveFileDocCommand destructor
//----------------------------------------------------------------------------------------
TSaveFileDocCommand::~TSaveFileDocCommand()
{
}

//----------------------------------------------------------------------------------------
// TSaveFileDocCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------
TAppleEvent* TSaveFileDocCommand::MakeAppleEvent()
{
	// We have to save the file specifier in the AppleEvent.
	CAutoPtr_AC<TAppleEvent> theEvent(TSaveCommand::MakeAppleEvent());

	// 'Save' is only valid if a file already exists and something has changed.
	// But this should have already been checked in the menu setup.
#if qDebug
	if (!fSaveFile)
		::ProgramBreak_AC("TSaveFileDocCommand::MakeAppleEvent: No file to save to.");
#endif

	// We don't record the file spec because we're assuming that we're
	// simply saving right into the current file.

	return theEvent.release();
}

//----------------------------------------------------------------------------------------
// TSaveFileDocCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TSaveFileDocCommand::DoIt()
{
#if qDebug
	if (fDocument == NULL)
		::ProgramBreak_AC("TSaveFileDocCommand::DoIt called without a document!");
	else
#endif
	{
		// Set the document's TFile object so that it saves in the correct place.
		TFileBasedDocument* aFileDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);
		
		TFile*	aFile = aFileDocument->GetFile();
		::Assertion_AC(aFile != NULL, "TSaveFileDocCommand::DoIt: Document's file is bad.");
		
		// If the file has really changed then go ahead and respecify it.
		if (!fSaveFile->IsSameFile(aFile))
			aFile->SpecifyWithFile(fSaveFile);
		
		// Test to see if the destination file is there.
		
		aFileDocument->SetFileExists(fSaveFile->GetModificationDate() != 0);

		aFileDocument->SaveDocument(fIdentifier);
	}
}

//========================================================================================
// CLASS TSaveAsFileDocCommand
//========================================================================================

//----------------------------------------------------------------------------------------
// TSaveAsFileDocCommand constructor: 
//----------------------------------------------------------------------------------------
TSaveAsFileDocCommand::TSaveAsFileDocCommand(TDocument* itsDocument, EventNumber saveType)
	:	TSaveCommand(itsDocument, saveType)
{
	TFileBasedDocument* aDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);

	fSaveFile = aDocument->DoMakeFile(saveType);

	// Ask the user for the destination file location and name.
	::ThrowIfOSErr_AC(MAInteractWithUser());
	try
	{
		// If this fails (i.e. user cancels) then we want it to be caught
		// in elsewhere with a more appropriate error code.
		aDocument->RequestFileName(saveType, false, fSaveFile);
	}
	catch(CException_AC& theException)
	{
		if (theException.GetError() == noErr && theException.GetMessage() == messageCancelled)
			::ThrowIfOSErr_AC(userCanceledErr);
		throw;
	}
	
	#if 0
		// We use to check for saving over ourselves using the following code
		// but since changing the saving commands into separate commands it
		// became impossible to accomodate this.

		// Check to see if we've just been told to save on top of ourselves.
		// If so then change our cSaveAs/cSaveCopy to a cSave.
		if (aDocument->FileExists()
			&& fSaveFile->IsSameFile(aDocument->GetFile()))
		{
			fSaveFile.reset();
			fIdentifier = cSave;
			fSaveFile = aDocument->DoMakeFile(fIdentifier);
			fSaveFile->SpecifyWithFile(aDocument->GetFile());
		}
	#endif
}

//----------------------------------------------------------------------------------------
// TSaveAsFileDocCommand constructor: 
//----------------------------------------------------------------------------------------
TSaveAsFileDocCommand::TSaveAsFileDocCommand(TDocument* itsDocument,
											 TAppleEvent* message,
											 TAppleEvent* reply,
											 TFile* saveFile,
											 EventNumber saveType)
	:	TSaveCommand(itsDocument, message, reply, saveType),
		fSaveFile(saveFile)
{
	try
	{
		TFileBasedDocument* aDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);

		// Do we know to where we are to save?  BTW, we already know that if
		// we do have a file specified it is _not_ our current file because
		// the dispatcher (DoAESave) will have already caught that situation
		// and routed it off to a TSaveFileDocCommand.
		if (!saveFile)
		{
			// No file was specified so we have to ask the user for one.
			fSaveFile = aDocument->DoMakeFile(fIdentifier);
		
			// Ask the user for the destination file location and name.
			::ThrowIfOSErr_AC(MAInteractWithUser());
			try
			{
				// If this fails (i.e. user cancels) then we want it to be caught
				// in elsewhere with a more appropriate error code.
				aDocument->RequestFileName(fIdentifier, false, fSaveFile);
			}
			catch(CException_AC& theException)
			{
				if (theException.GetError() == noErr && theException.GetMessage() == messageCancelled)
					::ThrowIfOSErr_AC(userCanceledErr);
				throw;
			}
			
			// Check to see if we've just been told to save on top of ourselves.
			// If so then change our cSaveAs to a cSave.
			if (aDocument->FileExists()
				&& fSaveFile->IsSameFile(aDocument->GetFile()))
			{
				fSaveFile.reset();
				fIdentifier = cSave;
				fSaveFile = aDocument->DoMakeFile(fIdentifier);
				fSaveFile->SpecifyWithFile(aDocument->GetFile());
			}
		}
	}
	catch(CException_AC& theException)
	{
		
		// If this fails because user cancels then we want it 
		// to be caught with a more appropriate error code.
		if (theException.GetError() == noErr && theException.GetMessage() == messageCancelled)
			theException.SetError(userCanceledErr);
		throw;
	}
	
	if (message->HasParameter(keyAEFileType))
		fSaveFile->SetFileType(message->ReadType(keyAEFileType));
}

//----------------------------------------------------------------------------------------
// TSaveAsFileDocCommand destructor
//----------------------------------------------------------------------------------------
TSaveAsFileDocCommand::~TSaveAsFileDocCommand()
{
}

//----------------------------------------------------------------------------------------
// TSaveAsFileDocCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------
TAppleEvent* TSaveAsFileDocCommand::MakeAppleEvent()
{
	// We have to save the file specifier in the AppleEvent.
	CAutoPtr_AC<TAppleEvent> theEvent(TSaveCommand::MakeAppleEvent());

	// Which file to store it into.
	if (fSaveFile)
	{
		CAEDesc aSaveFileDesc;
		aSaveFileDesc.PutFSSpec(fSaveFile->GetFileSpec());

		theEvent->WriteParameter(keyAEFile, aSaveFileDesc);
	}

	return theEvent.release();
}

//----------------------------------------------------------------------------------------
// TSaveAsFileDocCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TSaveAsFileDocCommand::DoIt()
{
#if qDebug
	if (fDocument == NULL)
		::ProgramBreak_AC("TSaveAsFileDocCommand::DoIt called without a document!");
	else
#endif
	{
		// Attempt to delete the existing file.
		// ------------------------------------
			// Formerly, we did the actual deletion of the file during the 
			// initialization of this command but it now gets postponed
			// until the command is performed (here).  We assume that if we
			// make it to this point and there is a file in the way that it is 
			// okay to delete it based on prior approval.
			OSErr err = fSaveFile->DeleteFile();
			if ((err != noErr) && (err != fnfErr))
				::ThrowException_AC(err, 0);
		
		// Set the document's TFile object so that it saves in the correct place.
		
		TFileBasedDocument*	aFileDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);

		TFile* aFile = aFileDocument->GetFile();
		aFile->SpecifyWithFile(fSaveFile);
		
		// Test to see if the destination file is there.
		aFileDocument->SetFileExists(fSaveFile->GetModificationDate() != 0);

		aFileDocument->SaveDocument(fIdentifier);
	}
}

//========================================================================================
// CLASS TSaveCopyFileDocCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSaveCopyFileDocCommand constructor: 
//----------------------------------------------------------------------------------------
TSaveCopyFileDocCommand::TSaveCopyFileDocCommand(TDocument* itsDocument, EventNumber saveType)
	:	TSaveAsFileDocCommand(itsDocument, saveType)
{
}

//----------------------------------------------------------------------------------------
// TSaveCopyFileDocCommand constructor: 
//----------------------------------------------------------------------------------------
TSaveCopyFileDocCommand::TSaveCopyFileDocCommand(TDocument* itsDocument,
												 TAppleEvent* message,
												 TAppleEvent* reply,
										 		 TFile* saveFile,
											 	 EventNumber saveType)
	:	TSaveAsFileDocCommand(itsDocument, message, reply, saveFile, saveType)
{
}

//----------------------------------------------------------------------------------------
// TSaveCopyFileDocCommand destructor
//----------------------------------------------------------------------------------------
TSaveCopyFileDocCommand::~TSaveCopyFileDocCommand()
{
}

//----------------------------------------------------------------------------------------
// TSaveCopyFileDocCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------
TAppleEvent* TSaveCopyFileDocCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> theEvent(TSaveAsFileDocCommand::MakeAppleEvent());

	// Add the 'makeCopy' parameter.
	theEvent->WriteBoolean(keyAESaveCopy, true);

	return theEvent.release();
}

//========================================================================================
// CLASS TCloseFileDocCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TCloseFileDocCommand constructor
//----------------------------------------------------------------------------------------
TCloseFileDocCommand::TCloseFileDocCommand(CommandNumber itsCommandNumber,
												TDocument* itsDocument) :
	TCloseDocCommand(itsCommandNumber, itsDocument),
	
	fSaveCommand(cSave)
{
	fUseAppleEvent = true;		// We want to default here.
	
	// Are we saving?
	if (fSavingState == kDoSave)
	{
		try
		{
			::ThrowIfOSErr_AC(MAInteractWithUser());
			TFileBasedDocument*	aDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);

			bool hasFile = aDocument->FileExists();
	
			// Get where to save the file
			
			if (hasFile)
			{
				fCloseFile = aDocument->DoMakeFile(itsCommandNumber);
				fCloseFile->SpecifyWithFile(aDocument->GetFile());
			}
			else
			{
				// We have to ask for the save location.
				fSaveCommand = cSaveAs;
				
				fCloseFile = aDocument->DoMakeFile(cSaveAs);
			
				// Ask the user for the destination file location and name.
				aDocument->RequestFileName(cSaveAs, false, fCloseFile);
			}
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			// If this fails because user cancels then we want it 
			// to be caught with a more appropriate error code.
			if (theException.GetError() == noErr && theException.GetMessage() == messageCancelled)
				theException.SetError(userCanceledErr);
			//SetValidationError(theException.GetError());	// In case we are linked.
			throw;
		}
	}
}

//----------------------------------------------------------------------------------------
// TCloseFileDocCommand constructor
//----------------------------------------------------------------------------------------
TCloseFileDocCommand::TCloseFileDocCommand(TDocument* itsDocument,
												TAppleEvent* message,
												TAppleEvent* reply) :
	TCloseDocCommand(itsDocument, message, reply),

	fSaveCommand(cSave)
{
	if (fSavingState != kDontSave)
	{
		try
		{
			// Does the AppleEvent specifically say where to save the file;
			if (message->HasParameter(keyAEFile))
			{
				DescType		returnedType;
				CFSSpec_AC			theFSSpec;
				Size			actualSize;
		
				::ThrowIfOSErr_AC(AEGetParamPtr(message, keyAEFile, typeFSS, &returnedType, (Ptr) & theFSSpec, sizeof(theFSSpec), &actualSize));
				TFileBasedDocument*	aDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);

				fCloseFile = aDocument->DoMakeFile(fIdentifier);
				fCloseFile->Specify(theFSSpec);
				fSaveCommand = cSaveAs;
			}
			else
			{
				// No but do we already have a file wherein we can save?
				TFileBasedDocument*	aDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);

				bool hasFile = aDocument->FileExists();
		
				// Get where to save the file
				
				if (hasFile)
				{
					fCloseFile = aDocument->DoMakeFile(cClose);
					fCloseFile->SpecifyWithFile(aDocument->GetFile());
				}
				else
				{
					fSaveCommand = cSaveAs;
					fCloseFile = aDocument->DoMakeFile(cSaveAs);
				
					// Ask the user for the destination file location and name.
					::ThrowIfOSErr_AC(MAInteractWithUser());
					aDocument->RequestFileName(cSaveAs, false, fCloseFile);
				}
			}
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			// If this fails because user cancels then we want it 
			// to be caught with a more appropriate error code.
			if (theException.GetError() == noErr && theException.GetMessage() == messageCancelled)
				theException.SetError(userCanceledErr);
			//SetValidationError(theException.GetError());	// In case we are linked.
			throw;
		}
	}
}

//----------------------------------------------------------------------------------------
// TCloseFileDocCommand destructor
//----------------------------------------------------------------------------------------
TCloseFileDocCommand::~TCloseFileDocCommand()
{
}

//----------------------------------------------------------------------------------------
// TCloseFileDocCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------
TAppleEvent* TCloseFileDocCommand::MakeAppleEvent()
{
	// We have to save the file specifier in the AppleEvent.
	CAutoPtr_AC<TAppleEvent> theEvent(TCloseDocCommand::MakeAppleEvent());

	// Which file to store it into.
	if (fSavingState != kDontSave && !fCloseFile.IsNULL())
	{
		CAEDesc aCloseFileDesc;
		aCloseFileDesc.PutFSSpec(fCloseFile->GetFileSpec());

		theEvent->WriteParameter(keyAEFile, aCloseFileDesc);
	}

	return theEvent.release();
}

//----------------------------------------------------------------------------------------
// TCloseFileDocCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TCloseFileDocCommand::DoIt()
{
	if (fDocument)
	{
		// Some documents want us to specifically ask if they should be saved.
		if (fSavingState == kAskSave)
		{
			short poseResult = fDocument->PoseSaveDialog();
			switch (poseResult)
			{
				case kNoButton:
					fSavingState = kDontSave;
					break;
				case kYesButton:
					fSavingState = kDoSave;
					break;
				case cancel:
					::ThrowIfOSErr_AC(userCanceledErr);
					break;
			}
		}
		
		if (fSavingState == kDoSave)
		{
			fDocument->CommitLastCommand();
		
			// Set the document's TFile object so that it saves in the correct place.
			TFileBasedDocument* aFileDocument = dynamic_cast_AC(TFileBasedDocument*,fDocument);
			ThrowIfNULL_AC(aFileDocument);
			

			aFileDocument->GetFile()->SpecifyWithFile(fCloseFile);
			aFileDocument->SetFileExists(fCloseFile->GetModificationDate() != 0);
			aFileDocument->SaveDocument(cSave);
		}

		// If there is actually some saving to do it should have already been done.
		fDocument->SetChangeCount(0);
		fDocument->Close();
		delete fDocument;
		fDocument = NULL;
	}
}

//========================================================================================
// CLASS TFileBasedDocument
//========================================================================================

MA_DEFINE_CLONE(TFileBasedDocument);

//----------------------------------------------------------------------------------------
// TFileBasedDocument constructor
//----------------------------------------------------------------------------------------
TFileBasedDocument::TFileBasedDocument(TFile* itsFile, OSType itsScrapType)
	:	TDocument(),
		fFileExists(false),
		fScrapType(itsScrapType),							// need constant kNoScrapType
		fHowToSave(svtAskUser),
		fFile(itsFile ? itsFile : DoMakeFile(cNew)) 
{ }

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoMakeFile: 
//----------------------------------------------------------------------------------------
TFile* TFileBasedDocument::DoMakeFile(CommandNumber itsCommandNumber)
{
	return gDispatcher->DoMakeFile(itsCommandNumber);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::Free: 
//----------------------------------------------------------------------------------------
TFileBasedDocument::~TFileBasedDocument()
{
	if (fFile)
		CloseFile();
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoInitialState:
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoInitialState()
{
	// don't need to call TDocument's method because it is empty
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoSetupMenus: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoSetupMenus()
{
	TDocument::DoSetupMenus();

	Enable(cSaveAs, true);
	Enable(cSaveCopy, true);
	if (IsChanged())
	{
		Enable(cSave, true);
		Enable(cRevert, FileExists());
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::FindDocument: 
//----------------------------------------------------------------------------------------
bool TFileBasedDocument::FindDocument(TFile* aFile)// override 
{
	return FileAlreadyOpen(aFile);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::CheckFile: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::CheckFile(ResNumber rsrcId,
								   short rsrcIndex,
								   bool reverting)
{
	OSErr err = FileChanged(reverting);	// don't care about the file type if saving 
	if (err == errFileChanged)
	{
		::ThrowIfOSErr_AC(MAInteractWithUser());
		CStr255_AC s(rsrcId, rsrcIndex);
		ParamText(fTitle, s, gEmptyString, gEmptyString);
		if (MacAppAlert(phFileChanged, NULL) == cancel)
			::ThrowException_AC(noErr, messageCancelled);
	}
	else if ((err != noErr) && reverting)
		::ThrowException_AC(err, 0);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::Close: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::Close()
{
	long changeCount;
	short poseResult = kYesButton;
	
#if qDebug
	if (gClipboardMgr->fClipWindow->GetDocument() == this)
		::ProgramBreak_AC("Attempt to close clipboard document");
#endif

	changeCount = GetChangeCount();
	if (changeCount && fAskOnClose)
	{
		poseResult = PoseSaveDialog();
		if (poseResult == cancel)
			::ThrowException_AC(noErr, messageCancelled);
	}

	if (changeCount)
	{
		if (poseResult == kYesButton)
			SaveDocument(cClose);				// Will fail if unable to save 
		else if (poseResult == kNoButton)
			Abandon();
	}
	TDocument::Close();
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoAESave
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoAESave(TAppleEvent* message,
								  TAppleEvent* reply)
{
	CTempAllocation					tempAllocation(true);
	CTempObjectAllocation			tempObjectAllocation(true);
	CAutoPtr_AC<TFile> 				saveFile;
	EventNumber						saveType = cSaveAs;
	
	// We determine what kind of save is to be performed:
	//		1. Save - Message has no file and we do or file specified
	//			 in message is the same as ours.
	//		2. SaveAs - Message has a file, it's not the same file,
	//			 or the file has never been saved.
	//		3. SaveCopy - Message has 'makeCopy' argument and it's true.

	// It's cSaveCopy if the 'makeCopy' parameter is supplied and is true.  This
	// might get reset below if the target file is the same as our current file.
	if (message->HasParameter(keyAESaveCopy) && message->ReadBoolean(keyAESaveCopy))
		saveType = cSaveCopy;

	// If a file is provided then we stick with the default cSaveAs (unless it's
	// already be set to cSaveCopy) unless we're saving as our current file.
	if (message->HasParameter(keyAEFile))
	{
		// It has a file specifier so we know right where to save it.
		DescType		returnedType;
		CFSSpec_AC		theFSSpec;
		Size			actualSize;

		::ThrowIfOSErr_AC(AEGetParamPtr(message, keyAEFile, typeFSS, &returnedType, (Ptr) &theFSSpec, sizeof(theFSSpec), &actualSize));
		saveFile = DoMakeFile(cSave);
		saveFile->Specify(theFSSpec);

		if (FileExists() && saveFile->IsSameFile(GetFile()))
		{
			saveFile.reset();
			saveFile = DoMakeFile(cSave);
			saveType = cSave;
			saveFile->SpecifyWithFile(GetFile());
		}
	}
	else if ((saveType != cSaveCopy) && FileExists())
	{
		// And if no file was specified, we're not doing a cSaveCopy, and
		// we already have a file then we're just a simple cSave.
		saveType = cSave;
		saveFile = DoMakeFile(cSave);
		saveFile->SpecifyWithFile(GetFile());
	}
	
	// Generate a save document command.
	TSaveCommand*	aSaveCommand = NULL;
	switch (saveType)
	{
		case cSave:
			aSaveCommand = TH_new TSaveFileDocCommand(this, message, reply, saveFile);
			break;
		case cSaveAs:
			aSaveCommand = TH_new TSaveAsFileDocCommand(this, message, reply, saveFile, saveType);
			break;
		case cSaveCopy:
			aSaveCommand = TH_new TSaveCopyFileDocCommand(this, message, reply, saveFile, saveType);
			break;
		default:
			::ThrowError_AC(errAEEventNotHandled);	// Throw "can't understand" error.
			break;
	}
	// Immediately process the command rather than post it.
	saveFile.release();
	aSaveCommand->Process();
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoNeedDiskSpace: 
//----------------------------------------------------------------------------------------
#pragma segment MAWriteFile

void TFileBasedDocument::DoNeedDiskSpace(TFile* itsFile,
										 long& dataForkBytes,
										 long& rsrcForkBytes)
{
	if (itsFile->HasRsrcFork())
	{
		CStr255_AC theName;
		CStr255_AC theMessage;
		
		gDispatcher->GetApplicationName(theName);
		GetFinderMessage(theMessage);
		
		if (!theMessage.IsEmpty())
			rsrcForkBytes += kRsrcTypeOverhead + kRsrcOverhead + theMessage.Length() + 1;
		else if (!theName.IsEmpty())
			rsrcForkBytes += kRsrcTypeOverhead + kRsrcOverhead + theName.Length() + 1;

		if (fSavePrintInfo && fPrintInfo)
			rsrcForkBytes += kRsrcTypeOverhead + kRsrcOverhead + TPrintHandler::gPrintHandler->GetSavePrintInfoSize(fPrintInfo, itsFile, itsFile->HasRsrcFork());
		
		if (fSaveUserSelection)
		{
			TDesignator* theSelection = GetUserSelection();
			if (theSelection)
			{
				TCountingStream aStream;
				aStream.WriteStreamObject(theSelection);
				rsrcForkBytes += kRsrcTypeOverhead + kRsrcOverhead + aStream.GetSize();
			}
		}
		rsrcForkBytes += kRsrcFileOverhead;
	}
	else if (fSavePrintInfo && fPrintInfo)
		dataForkBytes += TPrintHandler::gPrintHandler->GetSavePrintInfoSize(fPrintInfo, itsFile, itsFile->HasRsrcFork());
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::RegainControl: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::RegainControl()
{
	if (ValidateFileSpec())
	{
		CStr63_AC fileName;
		GetFileName(fileName);
		SetTitle(fileName);
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoRead: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoRead(TFile* aFile,
								bool forPrinting)
{
	if (fSavePrintInfo)
		DoReadPrintInfo(aFile, forPrinting);
	
	if (fSaveUserSelection)
		DoReadSelection(aFile, forPrinting);

#if qAttachable
	if (fSaveAttachedScript)
		DoReadScript(aFile, forPrinting);
#endif
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoReadPrintInfo: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoReadPrintInfo(TFile* aFile,
												bool /* forPrinting */)
{
	if (FileExists())
		fPrintInfo = TPrintHandler::gPrintHandler->RestorePrintInfo(aFile, aFile->HasRsrcFork());
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoReadScript: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoReadScript(TFile* aFile, bool /* forPrinting */)
{
	if (HasAppleScript_AC() && FileExists())
	{
		if (aFile->HasRsrcFork())
		{
			if (aFile->IsRsrcForkOpen())
			{
				CTempDesc scriptDesc;
				
				scriptDesc.ReadFrom(aFile, kScriptInfoRsrcType, kScriptInfoRsrcID);
				if (scriptDesc.descriptorType != typeNull)
					SetOSAScript(scriptDesc);
			}
		}
		else
			ReadOSAScript(&TFileStream(aFile));
	}
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoReadSelection: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoReadSelection(TFile* aFile,
										 bool forPrinting)
{
	Handle theSelectionRsrc = NULL;
	if (FileExists() && aFile->HasRsrcFork() && aFile->IsRsrcForkOpen())
	{
		CTempUseResFile_AC tempUseResFile(aFile->GetRsrcRefNum());
		if (ResError() == noErr)
			theSelectionRsrc = Get1Resource(kDesignatorResType, kSelectionRsrcID);
	}
	
	if (theSelectionRsrc && !forPrinting)
	{
		CTempHandleLock_AC tempHandleLock(theSelectionRsrc, eDontMoveHi, eReleaseIt, eDontLockIt);
		// read in the associated designator 
		THandleStream aHandleStream(theSelectionRsrc, 10);
		
		bool wasValid;
		CAutoPtr_AC<TDesignator> aUserSelection = dynamic_cast_AC(TDesignator*,aHandleStream.ReadStreamObject(wasValid));
		
		if (wasValid && aUserSelection)
		{
			SetUserSelection(aUserSelection);
			aUserSelection.release();
		}
#if qDebugMsg
		else
			::ProgramBreak_AC("###Unknown TDesignator for selection.");
#endif	
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::AboutToSaveFile: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::AboutToSaveFile(TFile* /* theSaveFile */,
										 CommandNumber /* itsCmd */,
										 bool& /* makingCopy */)
{
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoWrite: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoWrite(TFile* aFile,
								 bool makingCopy)
{
	DoWriteFinderString(aFile, makingCopy);
	
	if (fSavePrintInfo && fPrintInfo)
		DoWritePrintInfo(aFile, makingCopy);
	
	if (fSaveUserSelection)
		DoWriteSelection(aFile, makingCopy);

#if qAttachable
	if (fSaveAttachedScript)
		DoWriteScript(aFile, makingCopy);
#endif
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoWriteFinderString: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoWriteFinderString(TFile* aFile,
												 bool /* makingCopy */)
{
	if (aFile->HasRsrcFork())
	{
		CStr255_AC theName;
		CStr255_AC theMessage;
		
		gDispatcher->GetApplicationName(theName);
		GetFinderMessage(theMessage);
		
		if (!theMessage.IsEmpty())
		{
			CPascalStrHandle_AC messageHandle;

			bool oldTemp = TemporaryAllocation(false);
			messageHandle = (CPascalStrHandle_AC)NewString(theMessage);
			TemporaryAllocation(oldTemp);
			::ThrowIfNULL_AC((Handle) messageHandle);
			
			try
			{
				MAAddResource((Handle) messageHandle, kFinderStringType, kFinderMessageID, gEmptyString);
				::ThrowIfResError_AC();
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				messageHandle = (CPascalStrHandle_AC) ::DisposeIfHandle_AC((Handle) messageHandle);
				throw;
			}
			
		}
		else if (!theName.IsEmpty())
		{
			CPascalStrHandle_AC nameHandle;

			bool oldTemp = TemporaryAllocation(false);
			nameHandle = (CPascalStrHandle_AC)NewString(theName);
			TemporaryAllocation(oldTemp);
			::ThrowIfNULL_AC((Handle) nameHandle);
						
			try
			{
				MAAddResource((Handle) nameHandle, kFinderStringType, kFinderApplicationNameID, gEmptyString);
				::ThrowIfResError_AC();
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				nameHandle = (CPascalStrHandle_AC) ::DisposeIfHandle_AC((Handle) nameHandle);
				throw;
			}
			
		}
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoWritePrintInfo: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoWritePrintInfo(TFile* aFile,
										  bool /* makingCopy */)
{
	if (fPrintInfo)
		TPrintHandler::gPrintHandler->SavePrintInfo(fPrintInfo, aFile, aFile->HasRsrcFork());
#if qDebug
	else
		::ProgramBreak_AC("No print record in document!");
#endif
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoWriteScript: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoWriteScript(TFile* aFile,
									   bool /* makingCopy */)
{
	if (HasOSAScript())
	{
		if (aFile->HasRsrcFork())
		{
			CStr255_AC	resourceName;
			fOSAScript->WriteOSAScript(aFile, kScriptInfoRsrcType, kScriptInfoRsrcID, resourceName);
		}
		else
			WriteOSAScript(&TFileStream(aFile), typeOSAGenericStorage);
	}
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoWriteSelection: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoWriteSelection(TFile* aFile,
												 bool /* makingCopy */)
{
	TDesignator* theSelection = GetUserSelection();
	if (theSelection && aFile->HasRsrcFork())
	{
		// allocate a handle for the resource
		CTempHandle_AC theSelectionRsrc(NewPermHandle(50)); //!!! SRF - what's this magic number?

		// write the user selection to the handle
		THandleStream aHandleStream(theSelectionRsrc, 10);

		// write out the associated designator 
		aHandleStream.WriteStreamObject(theSelection);

		// add the new selection resource
		MAAddResource(theSelectionRsrc, kDesignatorResType, kSelectionRsrcID, gEmptyString);
		::ThrowIfResError_AC();
		theSelectionRsrc.DontRelease();
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::CloseFile: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::CloseFile()
{
	if (fFile)
		fFile->CloseFile();
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::ReadDocument: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::ReadDocument(bool forPrinting) // override 
{
	CStr63_AC fileName;
	GetFileName(fileName);
	fTitle = fileName;
	ReadFile(forPrinting);
	SetChangeCount(0);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::ReadStationery: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::ReadStationery(TFile* itsNewFile) // override 
{
	ReadFile(kForDisplay);
	SetFile(itsNewFile);
	SetChangeCount(1);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::RevertDocument: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::RevertDocument()
{
	CAutoPtr_AC<TFileBasedDocument> tempObject(this);
	// Failure will Free the document since the data may not be in any state to redisplay

	try
	{
		CheckFile(kIDBuzzString, bzRevertAnyways, true);

			CommitLastCommand();

		FreeData();

		if (FileExists())
			ReadFile(kForDisplay);
		else
		{
			for (TemplateIterator_AC<TView*> iter(&fViewList); iter; ++iter)
			{
				TPrintHandler* itsPrintHandler = iter->GetPrintHandler();
		
				if (itsPrintHandler)
					itsPrintHandler->Reset();
			}

			DoInitialState();
		}

		SetChangeCount(0);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		if (theException.GetError() == fnfErr)
			theException.SetError(errRevertFNF);
			
		if (!theException.GetMessage())
			gErrorParm3 = fTitle;
			
		::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), messageRevertFailed);
	}
	tempObject.release();
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::GetSaveLocation
//----------------------------------------------------------------------------------------
void TFileBasedDocument::GetSaveLocation(CommandNumber itsCommandNumber, CAEDesc& theSaveDesc)
// Creates an CAEDesc describing where the document would be saved
{
	bool askForFilename = !FileExists() || ((itsCommandNumber != cSave) && (itsCommandNumber != cClose));
	bool makingCopy = itsCommandNumber == cSaveCopy;
	CAutoPtr_AC<TFile> theSaveFile(DoMakeFile(itsCommandNumber));

	if (askForFilename)
		RequestFileName(itsCommandNumber, makingCopy, theSaveFile);
	else
		theSaveFile->SpecifyWithFile(GetFile());

	::ThrowIfOSErr_AC(AECreateDesc(typeFSS,&theSaveFile->GetFileSpec(),sizeof(theSaveDesc), theSaveDesc));
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::IsChanged: 
//----------------------------------------------------------------------------------------
bool TFileBasedDocument::IsChanged()
{
	return TDocument::IsChanged();
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoSave
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoSave()
{
	// If the document has never been saved before then do a Save As...
	// but if it hasn't changed then don't do anything.
	if (!FileExists())
		DoSaveAs();
	else if (IsChanged())
	{
		CTempAllocation tempAllocation(true);
		CTempObjectAllocation tempObjectAllocation(true);
	
		TSaveFileDocCommand* aSaveFileDocCommand = TH_new TSaveFileDocCommand(this);
	
		aSaveFileDocCommand->SetUseAppleEvent(true);
		PostCommand(aSaveFileDocCommand);
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoSaveAs
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoSaveAs()
{
	// Generate a saveAs document command.
	CTempAllocation					tempAllocation(true);
	CTempObjectAllocation			tempObjectAllocation(true);
	TSaveAsFileDocCommand*			aSaveCommand = TH_new TSaveAsFileDocCommand(this, cSaveAs);

	aSaveCommand->SetUseAppleEvent(true);
	PostCommand(aSaveCommand);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoSaveCopy
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoSaveCopy()
{
	// Generate a saveCopy document command.
	CTempAllocation					tempAllocation(true);
	CTempObjectAllocation			tempObjectAllocation(true);
	TSaveCopyFileDocCommand*		aSaveCommand = TH_new TSaveCopyFileDocCommand(this, cSaveCopy);

	aSaveCommand->SetUseAppleEvent(true);
	PostCommand(aSaveCommand);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::DoRevert
//----------------------------------------------------------------------------------------
void TFileBasedDocument::DoRevert()
{
	CTempAllocation					tempAllocation(true);
	CTempObjectAllocation			tempObjectAllocation(true);
	TRevertCommand*					aRevertCommand = TH_new TRevertCommand(this);

	aRevertCommand->SetUseAppleEvent(true);
	PostCommand(aRevertCommand);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SaveDocument: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SaveDocument(CommandNumber itsCommandNumber)
{
	bool askForFilename = (itsCommandNumber == cClose) && !fFileExists;
	bool makingCopy = itsCommandNumber == cSaveCopy;
	bool copyFInfo = fFileExists;

	if (copyFInfo)
		CheckFile(kIDBuzzString, bzSaveAnyways, false);

	if (fCommitOnSave || (!makingCopy) )
		CommitLastCommand();

	SaveFile(itsCommandNumber, askForFilename, copyFInfo, makingCopy);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::MakeCloseCommand: 
//----------------------------------------------------------------------------------------
#pragma segment MAClose

TCloseDocCommand* TFileBasedDocument::MakeCloseCommand(CommandNumber itsCommandNumber)
{
	return TH_new TCloseFileDocCommand(itsCommandNumber, this);
}

TCloseDocCommand* TFileBasedDocument::MakeCloseCommand(TAppleEvent* message, TAppleEvent* reply)
{
	return TH_new TCloseFileDocCommand(this, message, reply);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::FileHasBeenSaved: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::FileHasBeenSaved(TFile* inFile)
{
	::AssertNotNULL_AC(inFile);
	
	CStr63_AC newName;
	
	SetChangeCount(0);
	inFile->GetName(newName);

	if (fTitle != newName)
		SetTitle(newName);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SFPutParms: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SFPutParms(CommandNumber itsCommandNumber,
									CStr255_AC& prompt,
									CStr255_AC& defaultName,
									ResNumber& dlgID,
									CPoint_AC& where,
									DlgHookYDProcPtr& dlgHook,
									ModalFilterYDProcPtr& modalFilter,
									short*& activeList,
									ActivateYDProcPtr& activateProc,
									StandardFileReply* /*reply*/,
									void*& yourDataPtr)
{
	dlgID = 0; // Let the system decide
	where = kBestSystemLocation;

	short idx = 0;
	switch (itsCommandNumber)
	{
		case cSave:
		case cSaveAs:
			idx = bzSaveAs;
			break;
			
		case cSaveCopy:
			idx = bzSaveCopy;
			break;
	}

	prompt.Empty();
	if (idx)
		GetIndString(prompt, kIDBuzzString, idx);
		
	defaultName = GetTitle();
	
	if (itsCommandNumber == cSaveCopy)
	{
		// Substitute the 'copy' into the string.
		::MAParamText("^FILENAME", defaultName);
		::GetIndString(defaultName, kIDBuzzString, bzSaveCopyFileName);
		::MAReplaceText(defaultName);
	}

	dlgHook = NULL;
	modalFilter = gModalFilterYDProcPtr;
	activeList = NULL;
	activateProc = NULL;
	yourDataPtr = NULL;
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SetTitle: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SetTitle(const CStr255_AC& aTitle)
{
	TDocument::SetTitle(aTitle);
	SetFileName(aTitle);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::GetFinderMessage: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::GetFinderMessage(CStr255_AC& theMessage)
{
	theMessage.Empty();
}

//----------------------------------------------------------------------------------------
// File Handling
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TFileBasedDocument::FileAlreadyOpen: 
//----------------------------------------------------------------------------------------
bool TFileBasedDocument::FileAlreadyOpen(TFile* aFile)
{
	return fFileExists ? fFile->IsSameFile(aFile) : false;
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::FileChanged: 
//----------------------------------------------------------------------------------------
OSErr TFileBasedDocument::FileChanged(bool checkType)
{
	HParamBlockRec pb;
	OSErr err = noErr;

	if (fFileExists)
	{
		err = fFile->GetFileInfo(pb);
	
		if (err == noErr)
		{
			if (checkType && (pb.fileParam.ioFlFndrInfo.fdType != fFile->GetCachedFileType()))
				err = errFTypeChanged;
			else if (fFile->IsModified())
				err = errFileChanged;
		}
	}

	return err;
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::FileExists: 
//----------------------------------------------------------------------------------------
bool TFileBasedDocument::FileExists()
{
	return fFileExists;
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::GetFile: 
//----------------------------------------------------------------------------------------
TFile* TFileBasedDocument::GetFile()
{
	return fFile;
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SetFile: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SetFile(TFile* itsNewFile)
{
	CloseFile();
	fFile = itsNewFile;
	fFileExists = false;			// Until we are shown otherwise
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::GetFileName: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::GetFileName(CStr63_AC& aName)
{
	if (fFile)
		fFile->GetName(aName);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::GetSaveInfo: 
//----------------------------------------------------------------------------------------
bool TFileBasedDocument::GetSaveInfo(CommandNumber /* aCommandNumber */,
									 bool copyFInfo,
									 CInfoPBRec& cInfo)
{
	bool saveInfo = false;

	if (fFileExists && copyFInfo)
	{
		OSErr err = fFile->GetCatInfo(cInfo);
		if (err == noErr)
			saveInfo = true;
	}
	// Set the type and creator in case it has changed; the file might be on
	// a file server and someone else could have changed the document 
	cInfo.hFileInfo.ioFlFndrInfo.fdType = fFile->GetCachedFileType();
	cInfo.hFileInfo.ioFlFndrInfo.fdCreator = fFile->GetCachedFileCreator();
	if (fFile->IsStationery())
		cInfo.hFileInfo.ioFlFndrInfo.fdFlags |= kIsStationery;
	else
		cInfo.hFileInfo.ioFlFndrInfo.fdFlags &= ~kIsStationery;

	return saveInfo;
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::GetTempName: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::GetTempName(CStr63_AC& filename)
{
	const short maxName = 31;					//maximum name size to generate
	const short maxNumber = 10;					//maximum # digits of the random number
	const short maxPrefix = maxName - maxNumber;

	CStr255_AC name;
	unsigned long time;

	// If the file is untitled, use the application name.
	if (fFile)
		fFile->GetName((CStr63_AC &)name);
	
	if (name.IsEmpty())
		gDispatcher->GetApplicationName(name);
		
	filename = name;

	// Check prefix length, and trim it down if too large.
	if (filename.Length() > maxPrefix)
		filename.Length() = static_cast<unsigned char>(maxPrefix);

	// Append a pseudo-random number.
	GetDateTime(&time);
	NumToString((time ^ (TickCount() >> 16)) & 0x7FFF, name);	// don't have neg number
	filename += name;

	// Check name length, and trim it down if too large.
	if (filename.Length() > maxName)
		filename.Length() = static_cast<unsigned char>(maxName);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::ReadFile: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::ReadFile(bool forPrinting)
{
	if (fFile)
	{
		OSType itsType = NULL;
		try
		{
			::ThrowIfOSErr_AC(fFile->GetFileType(itsType));
			
			if (itsType != kFolderType_AC)
			{
				::ThrowIfOSErr_AC(fFile->OpenFile());
				
				// Just in case the file was left open, reset to the beginning of the file
				fFile->SetDataMark(0, fsFromStart);
			}

			fFileExists = true;

			DoRead(fFile, forPrinting);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			if (itsType != kFolderType_AC)
				CloseFile();
			throw;
		}
		
		// We normally close the file once we have finished reading it's data
		// into memory.
		if (itsType != kFolderType_AC)
			::ThrowIfOSErr_AC(fFile->CloseFileIfNotKeptOpen());

		fFile->Modified();						// Set the modification date 
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::RequestFileName: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::RequestFileName(CommandNumber itsCommandNumber,
										 bool /*makingCopy*/,
										 TFile* aFile)
{
	CStr255_AC prompt;
	CStr255_AC filename;
	short dlgID;
	CPoint_AC dlgLoc;
	DlgHookYDProcPtr dlgHook;
	ModalFilterYDProcPtr modalFilter;
	short* activeList;
	ActivateYDProcPtr activateProc;
	void* yourDataPtr;
	StandardFileReply customReply;

	SFPutParms(itsCommandNumber, prompt, filename, dlgID, dlgLoc, dlgHook,
					 modalFilter, activeList, activateProc, &customReply, yourDataPtr);

	gRsrcCheck = 0;								//force immediate check

	::ThrowIfOSErr_AC(MAInteractWithUser());

	if (!yourDataPtr)
		yourDataPtr = &itsCommandNumber;
	
	gClipboardMgr->AboutToLoseControl(true);		// so scrap gets converted

	TemplateTempRoutineDescriptor<DlgHookYDUPP> cpfDlgHook(NewDlgHookYDProc(dlgHook));
	TemplateTempRoutineDescriptor<ModalFilterYDUPP> cpfModalFilter(NewModalFilterYDProc(modalFilter));
	TemplateTempRoutineDescriptor<ActivateYDUPP> cpfActivateProc(NewActivateYDProc(activateProc));

	CustomPutFile(prompt, filename, &customReply, dlgID, dlgLoc, cpfDlgHook, cpfModalFilter, activeList, cpfActivateProc, yourDataPtr);
	
	gClipboardMgr->RegainControl(true);			// so scrap gets converted

	bool goodReply = customReply.sfGood;
	if (goodReply)
		aFile->Specify(customReply.sfFile);		// Don't use SpecifyWithStandardFile
                                                // here since that would clobber the
                                                // filetype.
	if (goodReply)
	{
		// See if there is an open document with the same name.	If there is, tell it 
		// we're trying to save it again, which will ordinarily result in failure.
		TDocument* otherDoc = gDispatcher->FindDocument(aFile);
		if (otherDoc)
			otherDoc->SaveAgain(itsCommandNumber, this);
		
		// Formerly, we did the actual deletion of the file here but now that
		// is postponed until the command is performed.  We assume that if we
		// make it to that point and there is a file in the way that it is 
		// okay to delete it based on prior approval.
	}
	else
		::ThrowException_AC(noErr, messageCancelled);			// user cancelled 
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SaveFile: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SaveFile(CommandNumber itsCommandNumber,
								  bool askForFilename,
								  bool copyFInfo,
								  bool makingCopy)
{
	long dataBytes = 0;
	long rsrcBytes = 0;
	long neededBlks;
	long usedBlks;
	long freeBlks;
	long blkSize;
	bool canSaveInPlace;
	CStr255_AC name;
	OSErr err;
	TFile* theSaveFile = NULL;

	try
	{
		// This can't fail or we won't be able to save in low memory conditions.
		{
			CTempObjectAllocation	tempObjectAllocation(true);
			theSaveFile = DoMakeFile(itsCommandNumber);
		}
		
		// Make the file object representing where we are going to save.
		if (askForFilename || fFile.IsNULL())
			RequestFileName(itsCommandNumber, makingCopy, theSaveFile);
		else
			theSaveFile->SpecifyWithFile(fFile);

		theSaveFile->SetPermissions(fsRdWrPerm, fsRdWrPerm);// Since we intend to write to it 

		AboutToSaveFile(theSaveFile, itsCommandNumber, makingCopy);

		// Get information about the volume saving to 
		::ThrowIfOSErr_AC(theSaveFile->GetFreeBlocks(freeBlks));

		// Don't fill the disk completely.
		--freeBlks;

		// compute size needed to save document 
		::ThrowIfOSErr_AC(theSaveFile->GetBlockSize(blkSize));

		DoNeedDiskSpace(theSaveFile, dataBytes, rsrcBytes);
		neededBlks = NumBlocks(rsrcBytes, blkSize) + NumBlocks(dataBytes, blkSize);

		// If there is enough space we're done.  Save the file by the method specified 
		if (freeBlks >= neededBlks)
		{
			if ((fHowToSave == svtAskUser) || (fHowToSave == svtAlways))
				SaveViaTemp(itsCommandNumber, makingCopy, copyFInfo, theSaveFile);
			else
				SaveInPlace(itsCommandNumber, makingCopy, copyFInfo, theSaveFile);
		}
		// Is there enough space to save the file in place? 
		else
		{
			canSaveInPlace = false;				// Default value 

			// See if we can save in place be recovering the space used by the existing file 
			err = theSaveFile->GetPhysicalSize(dataBytes, rsrcBytes);
			if (err == noErr)
			{
				usedBlks = NumBlocks(dataBytes, blkSize) + NumBlocks(rsrcBytes, blkSize);

				if (neededBlks <= usedBlks + freeBlks)
				{
					// Saving in place purges the existing file.  Should we ask before going ahead? 
					if ((fHowToSave == svtAskUser) || (fHowToSave == sipAskUser))
					{
						::ThrowIfOSErr_AC(MAInteractWithUser());
						theSaveFile->GetName((CStr63_AC &)name);
						ParamText(name, gEmptyString, gEmptyString, gEmptyString);
						if (MacAppAlert(phPurgeOld, NULL) == kYesButton)
							canSaveInPlace = true;
						else
							::ThrowException_AC(noErr, messageCancelled);
					}
					else if (fHowToSave == sipAlways)
						canSaveInPlace = true;
				}
			}
			// If no file to recover space from, signal disk full error.  Otherwise just display error 
			else if (err != fnfErr)
				::ThrowException_AC(err, 0);

			if (canSaveInPlace)
				SaveInPlace(itsCommandNumber, makingCopy, copyFInfo, theSaveFile);
			else
				::ThrowException_AC(dskFulErr, 0);
		}

#if qDebugMsg
		err = theSaveFile->GetPhysicalSize(dataBytes, rsrcBytes);
		if (err == noErr)
		{
			usedBlks = NumBlocks(dataBytes, blkSize) + NumBlocks(rsrcBytes, blkSize);
			if (usedBlks != neededBlks)
			{
				ProgramReport_AC("In TFileBasedDocument::SaveFile: DoNeedDiskSpace estimated disk space incorrectly.\n");
				ProgramReport_AC(false, "estimated # disk blocks = %ld\n", neededBlks);
				ProgramReport_AC(false, "   actual # disk blocks = %ld\n", usedBlks);
			}
		}
#endif

		if (!makingCopy)
		{
			FileHasBeenSaved(theSaveFile);

			fFileExists = true;
			fFile->SpecifyWithFile(theSaveFile);
			fFile->Modified();
			::ThrowIfOSErr_AC(fFile->OpenFileIfKeptOpen());
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		if (theSaveFile)
		{
			err = theSaveFile->FlushVolume();
			if (!theException.GetMessage())
				theSaveFile->GetName((CStr63_AC &)gErrorParm3);

			if (fFile != theSaveFile)
				delete theSaveFile;
		}

		long newMsg = messageSaveAsFailed;

		if (!askForFilename)
			newMsg = messageSaveFailed;
		else if (makingCopy)
			newMsg = messageSaveCopyFailed;

		::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), newMsg);
	}

	err = theSaveFile->FlushVolume();

	if (fFile != theSaveFile)
		delete theSaveFile;
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SaveInPlace: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SaveInPlace(CommandNumber itsCommandNumber,
									 bool makingCopy,
									 bool copyFInfo,
									 TFile* itsFile)
{
#if 1
	
	// shown in the original code below.  I couldn't really see why it was
	// necessary.
	
	// When saving in place just write into the existing file after setting
	// position and then truncating unless this is a new file in which case
	// we must create the file.
#if qDebugMsg
	if (makingCopy)
		ProgramBreak_AC("SaveInPlace: Called with makingCopy true. (%d)\n", stderr);
#endif
	if (!fFileExists || !itsFile->IsSameFile(fFile))
	{
		// We need to create the file since it doesn't exist or it's
		// different from our current file.  Delete the file if it already
		// exists since the user will have already said "replace existing."
		CInfoPBRec cInfo;			// Retain finder information.
		bool validInfo = GetSaveInfo(itsCommandNumber, copyFInfo, cInfo);
		
		// Delete the file if it already exists.
		{
			OSErr err = itsFile->DeleteFile();
			if ((err != noErr) && (err != fnfErr))
				::ThrowException_AC(err, 0);
		}
		::ThrowIfOSErr_AC(itsFile->CreateFile());	// Create the file.
		if (validInfo)			// Change attributes of itsFile?
			::ThrowIfOSErr_AC(itsFile->SetCatInfo(cInfo));
		::ThrowIfOSErr_AC(itsFile->OpenFile());		// And open it.
		DoWrite(itsFile, makingCopy);
		::ThrowIfOSErr_AC(itsFile->CloseFileIfNotKeptOpen());
	}
	else
	{
		// We are apparently saving the file over itself.
		// Open the file if not already open.
		if (!itsFile->GetKeepDataOpen())
			::ThrowIfOSErr_AC(itsFile->OpenFile());
		itsFile->SetPosition(0);
		DoWrite(itsFile, makingCopy);
		::ThrowIfOSErr_AC(itsFile->SetDataLength(fFile->GetPosition()));
		::ThrowIfOSErr_AC(itsFile->CloseFileIfNotKeptOpen());
	}

#else
	CInfoPBRec cInfo;
	bool validInfo = GetSaveInfo(itsCommandNumber, copyFInfo, cInfo);

	CloseFile();								// close the file 
	{
		OSErr err = itsFile->DeleteFile();
		if ((err != noErr) && (err != fnfErr))
			::ThrowException_AC(err, 0);
	}

	try
	{
		::ThrowIfOSErr_AC(itsFile->CreateFile());

		if (validInfo)							// Change attributes of itsFile?
			::ThrowIfOSErr_AC(itsFile->SetCatInfo(cInfo));

		::ThrowIfOSErr_AC(itsFile->OpenFile());

		DoWrite(itsFile, makingCopy);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		OSErr recoverErr = itsFile->CloseFile();
#if qDebugMsg
		if (recoverErr != noErr)
			ProgramReport_AC(false, "In SaveInPlace recovery: error from CloseFile is %d\n", recoverErr);
#endif
		recoverErr = itsFile->DeleteFile();
#if qDebugMsg
		if ((recoverErr != noErr) && (recoverErr != fnfErr))
			ProgramReport_AC(false, "In SaveInPlace recovery: error from DeleteFile is %d\n", recoverErr);
#endif
		throw;
	}
	::ThrowIfOSErr_AC(itsFile->CloseFile());
#endif
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SaveViaTemp: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SaveViaTemp(CommandNumber itsCommandNumber,
									 bool makingCopy,
									 bool copyFInfo,
									 TFile* itsFile)
{
	CInfoPBRec cInfo;
	bool validInfo = GetSaveInfo(itsCommandNumber, copyFInfo, cInfo);

	CFSSpec_AC saveFileSpec(itsFile->GetFileSpec());		// Save file spec since it may be clobbered 

	CStr63_AC 	tmpName;
	GetTempName(tmpName);

	short tmpVRefNum;
	long tmpDirID;
	OSErr		anErr;
	anErr = FindFolder(saveFileSpec.vRefNum, kTemporaryFolderType, kCreateFolder, &tmpVRefNum, &tmpDirID);
	if (anErr == noErr)
		::ThrowIfOSErr_AC(itsFile->SpecifyWithTrio(tmpVRefNum, tmpDirID, tmpName));
	else
		itsFile->SetName(tmpName);

	try
	{
		::ThrowIfOSErr_AC(itsFile->CreateFile());

		if (validInfo)							// Change attributes of itsFile??
			::ThrowIfOSErr_AC(itsFile->SetCatInfo(cInfo));
		::ThrowIfOSErr_AC(itsFile->OpenFile());

		DoWrite(itsFile, makingCopy);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		OSErr recoverErr;

		recoverErr = itsFile->CloseFile();
		
#if qDebugMsg
		if (recoverErr != noErr)
			ProgramReport_AC(false, "In SaveViaTemp error recovery: error from CloseFile is %d\n", recoverErr);
#endif

		recoverErr = itsFile->DeleteFile();
		
#if qDebugMsg
		if ((recoverErr != noErr) && (recoverErr != fnfErr))
			ProgramReport_AC(false, "In SaveViaTemp error recovery: error from itsFile.DeleteFile is %d\n", recoverErr);
#endif

		throw;
	}
	

	::ThrowIfOSErr_AC(itsFile->CloseFile());

	if (!makingCopy)
		CloseFile();							// Close the documents file 

	try
	{
		if (fFileExists && copyFInfo)
		{
			// Since ExchangeFiles doesn't check that the destination file is locked
			// we need to check ourselves.
			CInfoPBRec cInfo;
		
			BlockSet_AC((Ptr) & cInfo, sizeof(CInfoPBRec), 0x00);// zero-out our storage 
			::ThrowIfOSErr_AC(fFile->GetCatInfo(cInfo));
			
			if (cInfo.hFileInfo.ioFlAttrib & 0x01)
				::ThrowIfOSErr_AC(fLckdErr);
				
			// Exchange the existing file for the one in the temporary folder 
			::ThrowIfOSErr_AC(itsFile->ExchangeFiles(fFile));

			// and delete the one left in the temporary folder 
			OSErr err = itsFile->DeleteFile();
			if ((err != noErr) && (err != fnfErr))
				::ThrowException_AC(err, 0);
		}
		else
			// The temp file was created in the temporary directory.
			// Move it back where it belongs
			::ThrowIfOSErr_AC(itsFile->MoveAndRename(saveFileSpec));

		// Specify the file again since it may have changed if it was saved in a temporary folder 
		itsFile->Specify(saveFileSpec);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		OSErr recoverErr = itsFile->DeleteFile();
		
#if qDebugMsg
		if ((recoverErr != noErr) && (recoverErr != fnfErr))
			ProgramReport_AC(false, "In SaveViaTemp error recovery: error from DeleteFile is %d\n", recoverErr);
#endif
		
		if (fFileExists && !makingCopy)
			recoverErr = fFile->OpenFileIfKeptOpen();
			
#if qDebugMsg
		if (recoverErr != noErr)
			ProgramReport_AC(false, "In SaveViaTemp error recovery: error from OpenFileIfKeptOpen is %d\n", recoverErr);
#endif

		// Set the name to something intelligent in case we have to display it to the user.
		itsFile->SetName(saveFileSpec.name);
		
		throw;
	}
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::SetFileName: 
//----------------------------------------------------------------------------------------
void TFileBasedDocument::SetFileName(const CStr63_AC& aName)
{
	if (fFile)
		fFile->SetName(aName);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::ValidateFileSpec: 
//----------------------------------------------------------------------------------------
bool TFileBasedDocument::ValidateFileSpec()
{
	return fFile ? fFile->UpdateFileSpec() : false;
}

//----------------------------------------------------------------------------------------
// Scripting
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TFileBasedDocument::CountContainedObjects: 
//----------------------------------------------------------------------------------------
long TFileBasedDocument::CountContainedObjects(DescType desiredType)
{
	return desiredType == cFile ? (FileExists() ? 1 : 0) : TDocument::CountContainedObjects(desiredType);
}

//----------------------------------------------------------------------------------------
// TFileBasedDocument::GetContainedObject: 
//----------------------------------------------------------------------------------------
MScriptableObject* TFileBasedDocument::GetContainedObject(DescType desiredType,
											  DescType selectionForm,
											  const CAEDesc& selectionData)
{
	MScriptableObject * result = NULL;
	if (desiredType == cFile)
	{
		if (FileExists())
		{
			if (selectionForm == formName)
			{
				TFile * resultFile = DoMakeFile(cOpen);
				CStr255_AC thePathName;
				selectionData.GetString(thePathName);
				resultFile->SpecifyWithTrio(0, 0, (CStr63_AC &)thePathName);
				TOSADispatcher::fgDispatcher->AddTemporaryToken(resultFile);
				result = resultFile;
			}
			else
			{
				TFile * myFile = GetFile();
				if (myFile)
				{
					TFile * resultFile = DoMakeFile(cOpen);
					resultFile->SpecifyWithFile(myFile);
					TOSADispatcher::fgDispatcher->AddTemporaryToken(resultFile);
					result = resultFile;
				}
			}
		}
	}
	else
		result = TDocument::GetContainedObject(desiredType, selectionForm, selectionData);

	return result;
}

//----------------------------------------------------------------------------------------
// End of UFileBasedDocument.cp
