//----------------------------------------------------------------------------------------
// USectionMgr.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USECTIONMGR__
#include "USectionMgr.h"
#endif

// MacApp

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDESKSCRAPVIEW__
#include "UDeskScrapView.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UEDITIONDOCUMENT__
#include "UEditionDocument.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include "UFileBasedDocument.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USECTION__
#include "USection.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// ANSI


//----------------------------------------------------------------------------------------
// private global variables
bool pSectionMgrInitialized;


//========================================================================================
// CLASS TSectionMgr
//========================================================================================
MA_DEFINE_CLASS(TSectionMgr);

//----------------------------------------------------------------------------------------
// TSectionMgr::TSectionMgr:
//----------------------------------------------------------------------------------------
TSectionMgr::TSectionMgr() :
	TBehavior(kSectionMgrBehaviorID)
{
}

//----------------------------------------------------------------------------------------
// TSectionMgr::Free: 
//----------------------------------------------------------------------------------------
TSectionMgr::~TSectionMgr()
{
}

//----------------------------------------------------------------------------------------
// TSectionMgr::DoAESectionEvent: 
//----------------------------------------------------------------------------------------
void TSectionMgr::DoAESectionEvent(CommandNumber aCommandNumber,
								 const AppleEvent& message,
								 const AppleEvent& reply)
{
	TSectionCommand* theCommand = NULL;

	switch (aCommandNumber)
	{
		case cSectionRead:
			theCommand = TH_new TSectionReadEventCommand(aCommandNumber, message, reply);
			break;

		case cSectionWrite:
			theCommand = TH_new TSectionWriteEventCommand(aCommandNumber, message, reply);
			break;

		case cSectionScroll:
			theCommand = TH_new TSectionScrollEventCommand(aCommandNumber, message, reply);
			break;

		case cSectionCancel:
			theCommand = TH_new TSectionCancelEventCommand(aCommandNumber, message, reply);
			break;
	}

	if (theCommand)
		fOwner->PostCommand(theCommand);
}


//========================================================================================
// CLASS TSectionCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSectionCommand::TSectionCommand
//----------------------------------------------------------------------------------------
TSectionCommand::TSectionCommand(CommandNumber itsCommandNumber,
									  TDocument* itsDocument,
									  TSection* itsSection) :
	TServerCommand(itsCommandNumber, itsDocument, kCantUndo, kDoesNotCauseChange, itsDocument),

	fSection(itsSection)
{
}

//----------------------------------------------------------------------------------------
// TSectionCommand::TSectionCommand
//----------------------------------------------------------------------------------------
TSectionCommand::TSectionCommand(CommandNumber itsCommandNumber,
								 const AppleEvent& itsMessage,
								 const AppleEvent& itsReply) :
	TServerCommand(itsCommandNumber, gDispatcher, kCantUndo, kDoesNotCauseChange, NULL,
				   itsMessage, itsReply),

	fSection(NULL)
{
	// get the section handle 
	SectionHandle aSectionHandle = fMessage->ReadSectionHandle(keyDirectObject);
#if qDebug
	if (gIntenseDebugging)
		ProgramReport_AC(false, "SectionHandle from AppleEvent = %ld \n", static_cast<long>(aSectionHandle));
#endif
	
	TSectionMgr* aSectionMgr =
		dynamic_cast_AC(TSectionMgr*,gDispatcher->GetBehaviorWithIdentifier(kSectionMgrBehaviorID));

	for (CSectionIterator iter(&aSectionMgr->fSectionList); iter.Current(); ++iter)
		if (iter->GetSectionHandle() == aSectionHandle)
		{
			fSection = iter;
			break;
		}

	if (!fSection)
		::ThrowException_AC(noErr, 0);	// silent failure
	
	// associate the command correctly 
	fContext = fSection->GetDocument();

#if qDebug
	::ThrowIfNonObject_AC(fContext);
#endif

	// ensure that the section is registered 
	::ThrowIfOSErr_AC(fSection->Register());
}

//----------------------------------------------------------------------------------------
// TSectionCommand destructor
//----------------------------------------------------------------------------------------
TSectionCommand::~TSectionCommand()
{
}

//========================================================================================
// CLASS TSectionReadEventCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSectionReadEventCommand::TSectionReadEventCommand: 
//----------------------------------------------------------------------------------------
TSectionReadEventCommand::TSectionReadEventCommand(CommandNumber itsCommandNumber,
													   TDocument* itsDocument,
													   TSubscriber* itsSubscriber) :

	TSectionCommand(itsCommandNumber, itsDocument, itsSubscriber)
{
	fCausesChange = kCausesChange;
}

//----------------------------------------------------------------------------------------
// TSectionReadEventCommand destructor
//----------------------------------------------------------------------------------------
TSectionReadEventCommand::~TSectionReadEventCommand()
{
}

//----------------------------------------------------------------------------------------
// TSectionReadEventCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TSectionReadEventCommand::DoIt()	// override 
{
	TSubscriber * aSubscriber = dynamic_cast_AC(TSubscriber*,fSection);

	if ((!aSubscriber->GetDocument()->GetStopAllEditions()) && (aSubscriber->GetUpdateMode() == sumAutomatic))
		aSubscriber->Subscribe();
}


//========================================================================================
// CLASS TSectionWriteEventCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSectionWriteEventCommand destructor
//----------------------------------------------------------------------------------------
TSectionWriteEventCommand::~TSectionWriteEventCommand()
{
}

//----------------------------------------------------------------------------------------
// TSectionWriteEventCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TSectionWriteEventCommand::DoIt()	// override 
{
	TFile* itsFile = NULL;
	TFileBasedDocument* aFileBasedDocument = fSection->GetDocument();
	if (aFileBasedDocument && aFileBasedDocument->FileExists())
		itsFile = aFileBasedDocument->GetFile();
	
	dynamic_cast_AC(TPublisher*,fSection)->Publish(itsFile);
}


//========================================================================================
// CLASS TSectionScrollEventCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSectionScrollEventCommand destructor
//----------------------------------------------------------------------------------------
TSectionScrollEventCommand::~TSectionScrollEventCommand()
{
}

//----------------------------------------------------------------------------------------
// TSectionScrollEventCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TSectionScrollEventCommand::DoIt()	// override 
{
	// at this point, the application has received and processed the 'odoc' AppleEvent 
	// so, the window containing the section should be frontmost

	// 1. bring the application to front (using the Process Manager) 
	gDispatcher->MakeFrontProcess();

	// 2. tell the document to reveal the selection and select it
	if (fContext)
		dynamic_cast_AC(TEditionDocument*,fContext)->RevealSelection(fSection->GetDesignator());
}


//========================================================================================
// CLASS TSectionCancelEventCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TSectionCancelEventCommand::TSectionCancelEventCommand: 
//----------------------------------------------------------------------------------------
TSectionCancelEventCommand::TSectionCancelEventCommand(CommandNumber itsCommandNumber,
														   TDocument* itsDocument,
														   TSection* itsSection) :

	TSectionCommand(itsCommandNumber, itsDocument, itsSection)
{
	fCanUndo = kCanUndo;		// can undo the cancel
	fCausesChange = kCausesChange;
}

//----------------------------------------------------------------------------------------
// TSectionCancelEventCommand destructor
//----------------------------------------------------------------------------------------
TSectionCancelEventCommand::~TSectionCancelEventCommand()
{
	if (fCommandDone && fSection)
		fSection->Delete();
	
	delete fSection;
	fSection = NULL;
}

//----------------------------------------------------------------------------------------
// TSectionCancelEventCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TSectionCancelEventCommand::DoIt()	// override 
{
	fSection->CancelSection(kCancel);
	dynamic_cast_AC(TEditionDocument*,fContext)->DoDeleteBorder(fSection);
}

//----------------------------------------------------------------------------------------
// TSectionCancelEventCommand::UndoIt: 
//----------------------------------------------------------------------------------------
void TSectionCancelEventCommand::UndoIt()	// override 
{
	fSection->CancelSection(!kCancel);
	dynamic_cast_AC(TEditionDocument*,fContext)->DoAddBorder(fSection);
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// InitUSectionMgr: 
//----------------------------------------------------------------------------------------
void InitUSectionMgr()

{
	if (!pSectionMgrInitialized)
	{
		// instantiate a section mgr and give the application this behavior
		gDispatcher->AddBehavior(TH_new TSectionMgr);

		pSectionMgrInitialized = true;
	}
}

//----------------------------------------------------------------------------------------
// End of USectionMgr.cp
