// UCommandHandler.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UCOMMANDHANDLER__
#define __UCOMMANDHANDLER__

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UEVENTHANDLER__
#include "UEventHandler.h"
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// The parent class of applications, documents and views. 
// Provides a context for undoable commands
//----------------------------------------------------------------------------------------

class EXPORT_MA TCommandHandler : public TEventHandler
{
public:
	//------------------------------------------------------------------------------------
	// Constructor/destructor.
	//------------------------------------------------------------------------------------
	TCommandHandler(TEventHandler* itsNextHandler = NULL);
		// Constructor
				
	virtual ~TCommandHandler();
	
	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Command Handling
	//------------------------------------------------------------------------------------
	virtual TCommandHandler* GetContext(CommandNumber aCommandNumber);
		// A convenient way to determine the context when initializing a command posted
		// by this CommandHandler. The default returns this. Overridden by TView.
	
	virtual void PerformCommand(TCommand* command);
		// Performs the given command and its linked command by calling
		// DoPerformCommand. This DOES NOT check to see if command is NULL.
        
    virtual bool PerformCommandAppleEvent(TCommand* command);

	virtual TCommand* GetLastCommand();
		// Returns the last undoable command. The command returned is not yet committed.
	
	virtual void CommitLastCommand();
		// Clear any commands in the undo/redo stacks. 
	
	virtual void PrepareForUndoRedo(TCommand* command);
		// Make this context ready for undo/redo. 
	
	virtual void RevealUndoRedo(TCommand* command);
		// Make this context visible after undo/redo. 

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	unsigned long GetPendingActionID() const { return fPendingActionID; }

#if qBackwardCompatible
public:
#else
protected:
#endif
	unsigned long			fPendingActionID;													

	static unsigned long	fgTransactionIDCount;
};

#endif // __UCOMMANDHANDLER__
