// UControl.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UCONTROL__
#define __UCONTROL__

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// ACS Appearance

#if qGrayscaleAppearance

#	ifndef __CControl_AC__
#	include "CControl_AC.h"
#	endif

#endif // qGrayscaleAppearance

// MacApp

// Included for gSystemStyle
#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UTRACKER__
#include "UTracker.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

// Toolbox

#ifndef __CONTROLS__
#include <Controls.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TScroller;
class TToolboxEvent;
class CList_AC;

//----------------------------------------------------------------------------------------
// TControl: An abstract class that represents a control.
//----------------------------------------------------------------------------------------

class EXPORT_MA TControl : public TView
{
	MA_DECLARE_CLASS;
	
public:
	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	TControl(const CViewPoint& itsLocation = gZeroVPt,
								 const CViewPoint& itsSize = gZeroVPt,
								 SizeDeterminer itsHSizeDet = sizeVariable,
								 SizeDeterminer itsVSizeDet = sizeVariable,
							     const CTextStyle_AC& itsTextStyle = gSystemStyle);
		// Constructor

	virtual ~TControl();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);	// override 
	
	virtual void WriteFields(CStream_AC* aStream) const;	// override 

	//------------------------------------------------------------------------------------
	// Coordinate information/transformation
	//------------------------------------------------------------------------------------
	CViewRect ControlArea();
	inline void ControlArea(CViewRect& theArea)
	{ theArea = ControlArea(); }
		// Returns in 'theArea' the controls active area.

	CRect_AC ControlQDArea();
		// Returns in 'theArea' the controls active area (in QD coordinates).

	//------------------------------------------------------------------------------------
	// View method overrides
	//------------------------------------------------------------------------------------
	virtual void ComputeFrame(CViewRect& newFrame);
		// Called to compute the control's size.

	virtual bool ContainsMouse(const CViewPoint& theMouse);
		// Returns true if 'theMouse is currently in the controls area.

	virtual void Dim();
		// dims the current image in the TControl

	virtual void DimState(bool state, bool redraw);
		// If we are not currently dimmed then set fDimmed to 'state.and if 'redraw' is
        // true then call DrawContents.

	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event,
									   CPoint_AC hysteresis);
		// Creates a control tracker command, which is posted.

	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);
		// Handles the mTurnOn, mTurnOff and mToggle messages by adjusting highlighting
		
	virtual bool Focus();
		// Calls Inherited::Focus and if its true then sets the port's text style and
        // returns true, if it can't be focused returns false.

	virtual void Flash();
		// Causes a control to flash with an 8 tick delay.

	virtual void Hilite();
		// hilites the current image in the TControl

	virtual void HiliteState(bool state, bool redraw);
		// If our fHilite is not 'state' the set it to 'state' before calling Hilite.
        // Note: hilite is only called if 'redraw is true and we can Focus.

	virtual void Inset(const CViewPoint& delta, bool redraw);
		// If we are sizable then offset fInset to 'dh' & 'dv' redrawing if 'redraw' is
        // true.

	virtual bool IsDimmed();
		// Returns true if the control is currently dimmed.

	virtual CViewRect GetAdornExtent();
		// Returns the adorn extent for adorning this view

	virtual EventNumber GetEventNumber(); // Override
		// Returns fEventNumber

	virtual void SetAdornment(/*CntlAdornment*/ short newAdornment,
									 bool redraw);
		// Set fAdornment of the 2.0-compatibility adorner.

	virtual void GetAdornment(CntlAdornment& itsAdornment);
		// Return fAdornment of the 2.0-compatibility adorner.

	virtual void SetPenSize(CPoint_AC newPenSize);
	virtual void GetPenSize(CPoint_AC& itsPenSize);
		// For 2.0-compatible adornment.

	virtual void SetInset(const CViewRect& newInset, bool redraw);
		// Set fInset to 'newInset' redrawing if 'redraw' is true.

	virtual void InstallColor(const CRGBColor_AC& theColor, bool redraw);
		// Sets the color field of our text style record to 'theColor', if 'redraw' then
        // call DrawContents.

	virtual void InstallTextStyle(const CTextStyle_AC& theTextStyle, bool redraw);
		// Sets fTextStyle to 'theTextStyle', if 'redraw' then call DrawContents.

	virtual void TrackFeedback(TrackPhase aTrackPhase,
										const CViewPoint& anchorPoint,
										const CViewPoint& previousPoint,
										const CViewPoint& nextPoint,
										bool mouseDidMove,
										bool turnItOn);
		// Doesn't use the default feedback

	virtual void TrackMouse(TrackPhase aTrackPhase,
								   CViewPoint& anchorPoint,
								   CViewPoint& previousPoint,
								   CViewPoint& nextPoint,
								   bool mouseDidMove);
		// Called back from tracking command. Handles tracking of the mouse and then calls
        // DoChoice on TrackRelease.

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
public:

	CViewRect fInset;								// amount to inset the control portion of
                                                // the view

	EventNumber fEventNumber;					// the event number to automatically pass
												// up the line via HandleEvent

	short fTextStyleRsrcID;						// Rsrc ID of 'TxSt' resource which has
												// CTextStyle_AC information.

	CTextStyle_AC fTextStyle;						// the CTextStyle_AC to use when rendering
                                                // this view

	bool fHilite;							// is this view hilited

	bool fDimmed;							// is this view dimmed

	bool fSizeable;							// is the control portion of this view
                                                // sizeable

	bool fPreferOutline;						// prefer outline fonts?
};


//----------------------------------------------------------------------------------------
// TCtlMgr: An abstract class that represents a toolbox Control Manager control.
//----------------------------------------------------------------------------------------

class EXPORT_MA TCtlMgr : public TControl
{
	MA_DECLARE_CLASS;
	
public:
	TCtlMgr(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								const CStr255_AC& itsTitle = gEmptyString,
								long itsVal = 0,
								long itsMin = 0,
								long itsMax = 0,
								short itsProcID = 0,
							    const CTextStyle_AC& itsTextStyle = gSystemStyle);
		// Constructor
		
	TCtlMgr(const TCtlMgr& inCtlMgr);
		// Copy Constructor

	virtual ~TCtlMgr();
		// Set the size of control to zero call DisposeControl, then call Inherited::Free.

	//------------------------------------------------------------------------------------
	// Rest of methods
	//------------------------------------------------------------------------------------
	virtual void DoPostCreate(TDocument* itsDocument);
	// Create the Toolbox control (if necessary)

	virtual void BeInPort(GrafPtr itsPort);
	// Sets the control to be in 'itsPort', by setting the contrlOwner field of the
    // ControlRef.

	virtual void CreateCMgrControl(const CStr255_AC& itsTitle,
										  long itsVal,
										  long itsMin,
										  long itsMax,
										  short itsProcID);
		// Used to create a new control, calls NewControl to create the control.

	virtual void InstallColor(const CRGBColor_AC& theColor, bool redraw);
		// Calls Inherited and sets the control color table.

	virtual void DimState(bool state, bool redraw);
		// Set the dim state to 'state' and redrawing it if 'redraw'.

	virtual void DoMouseCommand(CViewPoint& theMouse,
									   TToolboxEvent* event,
									   CPoint_AC hysteresis);
		// Handles mouse downs in the control by calling TestControl and DoChoice

	virtual void Draw(const CViewRect& area);
		// Draws the control by calling the toolbox routine ShowControl.

	virtual ViewCoordinate GetLongMax();
		// Returns the current fLongMax value.

	virtual ViewCoordinate GetLongMin();
		// Returns the current fLongMin value.

	virtual ViewCoordinate GetLongVal() const;
		// Returns the current fLongVal value.

	virtual void SetLongMax(ViewCoordinate itsMax,
								   bool redraw);
		// Sets fLongMax to 'itsMax', redrawing if required.

	virtual void SetLongMin(ViewCoordinate itsMin, bool redraw);
		// Sets flongMin to 'itsMin', redrawing if required.

	virtual void SetLongVal(ViewCoordinate itsVal, bool redraw);
		// Sets fLongVal to 'itsVal' redrawing if required.

	virtual void SetLongValues(ViewCoordinate itsVal,
									  ViewCoordinate itsMin,
									  ViewCoordinate itsMax,
									  bool redraw);
		// Sets all of the values at once, redrawing if required.

	virtual short GetHelpState();
		// Override to support help states for controls
	
	//------------------------------------------------------------------------------------
	// Methods to manipulate the Control Manager control values
	//------------------------------------------------------------------------------------
	
	virtual short GetMax();
		// Returns the maximum value of the Control Manager control.

	virtual short GetMin();
		// Returns the minimum value of the Control Manager control.

	virtual CStr255_AC GetText() const;
		// Returns the controls title, if any.

	virtual short GetVal() const;
		// Returns the current value of the Control Manager control.

	virtual short GetVariant();
		// Returns the variant control value for the Control Manager control.

	virtual void SetMax(short itsMax, bool redraw);
		// While focused sets the maximum value of the Control Manager control to 'itsMax'.

	virtual void SetMin(short itsMin, bool redraw);
		// While focused sets the minimum value of the Control Manager control to 'itsMin'.

	virtual void SetText(const CStr255_AC& itsText, bool redraw);
		// Sets the controls title to 'itsText', redrawing if required.

	virtual void SetVal(short newVal, bool redraw);
		// While focused sets the Control Manager controls value to 'newVal', redrawing if
        // required.

	virtual void SetValues(short itsVal,
								  short itsMin,
								  short itsMax,
								  bool redraw);
		// While focused sets all the values of the Control Manager control, at once,
        // redrawing if required.

	virtual void HiliteState(bool state, bool redraw);
		// Set the hilite state to 'state', redrawing if required.If hilite state is
        // currently 'state' then does nothing.

	virtual void Inset(const CViewPoint& delta, bool redraw);
		// If we are sizable then offset fInset to 'dh' & 'dv' redrawing if 'redraw' is
        // true.

	virtual void SetInset(const CViewRect& newInset, bool redraw);
		// Set fInset to 'newInset' redrawing if 'redraw' is true.

	virtual void AdjustCtlMgrControl(bool redraw);
		// Adjusts ctl mgr control taking into account control area.

	virtual bool IsCMgrVisible();
		// Returns true if the control is currently visible, false if not.

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);
		// If the control is resizable calls the toolbox routines MoveControl &
        // SizeControl before calling Inherited::SetFrame.

	virtual void UpdateCoordinates();// override
		// Keeps the control manager control in synch with any VSpace or QDSpace
		// origin changes.

	virtual bool SetCMgrVisibility(bool beVisible);
		// Sets the visibility of the control to 'isVisible'.
		// Returns old state

	virtual void Show(bool state, bool redraw);
		// Keep the CMgr visibility state synchronized.

	virtual void Activate(bool entering);
		// Inform control about activation state

#if qGrayscaleAppearance
	
	// Overridden Control functions

	virtual void Dim();
		// Let AGA handle dimming

	virtual void SetEnable(bool state);
		// Let AGA handle enable state
	
	// AGA functions
			
	virtual bool IsStandardControl() const;
		// Returns whether this class is able to suppress AGA drawing (true for TCtlMgr)

	virtual bool IsEmulatingControls() const;
		// Returns whether this class should emulate controls
	
	virtual void CreateAGAControl();
		// Subclasses must implement to create an AGA control
		
	virtual CControl_AC* GetAGAControl() const;
		// Subclasses must implement to return their instantiated AGA control
		
#endif // qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
public:
	MA_operator_delete_auto_ptr<CStr255_AC> fTitle;	// A pointer to storage for the title if no fCMgrControl

	ControlRef fCMgrControl;					// the Control Manager Control that is
                                                // being managed

	ViewCoordinate fLongVal;						// the current value

	ViewCoordinate fLongMin;						// the minimum setting

	ViewCoordinate fLongMax;						// the maximum setting

	short	fProcID;							// Save the procID for use when cloning

	short fBitsToShift;							// # bits shifted to convert between short
                                                // and long value.
};


//----------------------------------------------------------------------------------------
// TScrollBar: A subclass of TCtlMgr that maps between longint values for our view
// coordinates and the integers of a toolbox scrollbar.
//----------------------------------------------------------------------------------------

class EXPORT_MA TScrollBar : public TCtlMgr
{
	MA_DECLARE_CLASS;
	
public:

	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	TScrollBar(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								VHSelect itsDirection = hSel,
								long itsVal = 0,
								long itsMin = 0,
								long itsMax = 0);
		// Constructor

	virtual ~TScrollBar();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);	// override 
	
	virtual void WriteFields(CStream_AC* aStream) const;	// override 

	//------------------------------------------------------------------------------------
	// View method overrides
	//------------------------------------------------------------------------------------
	virtual void DeltaValue(ViewCoordinate delta);
		// Does range checking on 'delta' checking it against fLongMin & fLongMax for the
        // scrollbar.

	virtual void DoMouseCommand(CViewPoint& theMouse,
									   TToolboxEvent* event,
									   CPoint_AC hysteresis);
		// Handles the mouse down in the various parts of the scrollbar.

	virtual void TrackScrollBar(short partCode);
		// Tracks the scrollbar by calling DeltaValue.

	virtual void ActionProc(short partCode);
		// Forwarded to from the global routine: ActionProcForTScrollBar which was set as
        // the ScrollBar's action proc. Typically forwards again to TrackScrollBar after
        // bounds checking and Updates before returning to the ScrollBar.

#if qGrayscaleAppearance
	
	virtual bool IsEmulatingControls() const;
		// Policy decision: standard scrollbars always use system CDEF for consistency/

#endif // qGrayscaleAppearance	

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:

	VHSelect fDirection;						// is this a horizontal or vertical
                                                // scrollbar

};

//----------------------------------------------------------------------------------------
// TScrollerScrollBar: A subclass of TScrollBar that is associated with one or more
// scrollers.
//----------------------------------------------------------------------------------------

class EXPORT_MA TScrollerScrollBar : public TScrollBar
{
	MA_DECLARE_CLASS;
	
public:	

	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	TScrollerScrollBar(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								VHSelect itsDirection = hSel,
								long itsMax = 0,
								TScroller* itsScroller = NULL);
		// Constructor
		
	TScrollerScrollBar(const TScrollerScrollBar& inScrollerScrollBar);
		// Copy Constructor

	virtual ~TScrollerScrollBar();
		// If the scrollers are not NULL then first detach the scrollbars from them before
        // freeing the scrollers, then call Inherited::Free.

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);	// override 
	
	virtual void WriteFields(CStream_AC* aStream) const;	// override 

	//------------------------------------------------------------------------------------
	// Methods
	//------------------------------------------------------------------------------------

	virtual void AttachScroller(TScroller* itsScroller);
		// IF 'itsScroller' is not NULL then add it to our list and attach ourselves to it.

	virtual void BeInPort(GrafPtr itsPort);
		// Call Inherited::BeInPort and then set our visibility based on the state of the
        // window we are in.

	virtual void Activate(bool entering);
		// If 'entering' is true then show ourselves, otherwise just call Draw to frame
        // our area.

	virtual void DoMouseCommand(CViewPoint& theMouse,
									   TToolboxEvent* event,
									   CPoint_AC hysteresis);
		// Handles mouse downs in the scrollbar.

	virtual void Draw(const CViewRect& area);
		// Draws a frame around the controls area before calling Inherited::Draw.

	virtual void TrackScrollBar(short partCode);
		// Called to handle tracking of the scrollbar by calling DeltaValue.

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	CList_AC fScrollers;							// the associated scrollers if any
};


//----------------------------------------------------------------------------------------
// TControlTracker: A Tracker adapted for tracking controls.
//----------------------------------------------------------------------------------------

class EXPORT_MA TControlTracker : public TTracker
{
public:
	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------
	TControlTracker(TControl* theControl = NULL, const CViewPoint& itsMouse = gZeroVPt);
		// Constructor

	virtual ~TControlTracker();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TControl* fControl;	// the control being tracked
};

//------------------------------------------------------------------------------------
// Utility class.
//------------------------------------------------------------------------------------

class EXPORT_MA CWhileFocused
{
public:
	// Constructor
	CWhileFocused(TCtlMgr* itsCtlMgr,
				  bool redraw);

	~CWhileFocused();
	// Restores the focus, removes the failure handler.

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	CPoint_AC fOldControlLocation;
	TCtlMgr* fCtlMgr;
	bool fRedraw;
	bool fWasVisible;
	bool fDidFocus;
};

#endif

