// UFileStream.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein. Used by permission


#ifndef __UFILESTREAM__
#define __UFILESTREAM__

#include "CFileStream_AC.h"
#include "CFileResourceStream_AC.h"
// MacApp

typedef CFileStream_AC 			TFileStream;
typedef CFileResourceStream_AC 	TFileResourceStream;

#if 0

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __TOOLBOX__
#include "Toolbox.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Toolbox


//----------------------------------------------------------------------------------------
// Forward class declarations. 
//----------------------------------------------------------------------------------------

class TFile;
class CList_AC;

//----------------------------------------------------------------------------------------
// TFileStream: This implements a stream on a file. The client is responsible for opening
// and closing the file. 
//----------------------------------------------------------------------------------------

class EXPORT_MA TFileStream : public CStream_AC
{
public:
	//------------------------------------------------------------------------------------
	// Creation/Destruction 
	//------------------------------------------------------------------------------------

	TFileStream(TFile* itsFile = NULL);
		// Constructor

	virtual ~TFileStream();
		// Destructor
		
	virtual void* Clone() const;
		// Clone
		
	//------------------------------------------------------------------------------------
	// SubClass Implementations 
	//------------------------------------------------------------------------------------

	virtual long GetPosition() const;

	virtual void SetPosition(long newPosition);

	virtual long GetSize() const;

	virtual void SetSize(long newSize);

	virtual void ReadBytes(void* p, long count); 

	virtual void WriteBytes(const void* p, long count); 

public:
	TFile* fFile; 
};


//----------------------------------------------------------------------------------------
// TFileResourceStream: This implements a stream that reads and writes to partial resources.
// Since the partial resources traps are only available in System 7, this stream class can
// only be used under System 7. 
//----------------------------------------------------------------------------------------

class EXPORT_MA TFileResourceStream : public TResourceStream
{
public:
	TFileResourceStream(TFile* theFile = NULL, 
								ResType theType = kNoIdentifier_AC, ResNumber theID = kNoResource_AC);

	virtual ~TFileResourceStream();
		// Destructor
	
	virtual void* Clone() const;
		// Clone
		
	virtual void WriteBytes(const void* p, long count);

	virtual void ReadBytes(void* p, long count);

	virtual void SetSize(long newSize);

public:
	TFile*		fFile;
};

#endif

#endif // __UFILESTREAM__
