// UMacAppGlobals.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UMACAPPGLOBALS__
#define __UMACAPPGLOBALS__

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __UCLASSDESC__
#include "UClassDesc.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

// Toolbox

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __NOTIFICATION__
#include <Notification.h>
#endif

#ifndef __PROCESSES__
#include <Processes.h>
#endif

// For ExitToShell
#ifndef __SEGLOAD__
#include <SegLoad.h>
#endif

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef _STDLIB
#include <stdlib.h>
	// for exit
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class MStreamable;
class TPrintHandler;
class TView;

//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

EXPORT_MA extern CTextStyle_AC gSystemStyle;
	// System's default text style

EXPORT_MA extern CTextStyle_AC gApplicationStyle;
	// Application's default text style

EXPORT_MA extern CTextStyle_AC gApplicationStyle9;
	// Application's default text style, 9 Point

EXPORT_MA extern WindowRef gWorkPort;
	// Pointer to a window that is created during initialization, but never shown or
    // enumerated.

EXPORT_MA extern RgnHandle gTemporaryRegion;
	// gTemporaryRegion is a temporary RgnHandle created in InitUMacApp. When debugging: before
    // using gTemporaryRegion, call UseTemporaryRegion and when done call DoneWithTemporaryRegion; this ensures
    // that 2 routines do not try to use gTemporaryRegion and the same time

EXPORT_MA extern NMRecPtr gNotificationPtr;
	// Passed to all occurrences of AEInteractWithUser.  Initialized to NULL so the
	// AppleEvent manager will provide default notification behavior.  Point it at
	// your own custom notification record if you want other behavior
	
EXPORT_MA extern ModalFilterYDProcPtr gModalFilterYDProcPtr;
	// Default returnval for SFPut/GetParms.  Initialized to MacAppStandardFileFilter.
	
#if qDebug
EXPORT_MA extern bool gBusyTemporaryRegion;
	// Is gTemporaryRegion in use?

EXPORT_MA extern CStr255_AC gUsedBy;
	// The routine using gTemporaryRegion
#endif

//----------------------------------------------------------------------------------------
// Debugging
//----------------------------------------------------------------------------------------

#if qDebug
EXPORT_MA void DoneWithTemporaryRegion();
	// Indicates that gTemporaryRegion is no longer in use. Call this only if qDebug is true.

EXPORT_MA void EnterDebugger(bool entering);
	// Call to enter the MacApp Debugger

EXPORT_MA void UseTemporaryRegion(const CStr255_AC& byWhom);
	// Call this when you are about to use gTemporaryRegion and qDebug is true. Used with
    // DoneWithTemporaryRegion will prevent you from trying to use gTemporaryRegion from two places at the
    // same time.

#endif

//----------------------------------------------------------------------------------------
// Windows
//----------------------------------------------------------------------------------------

EXPORT_MA bool ParseTitleTemplate(CStr255_AC& itsTemplate,
						   short& preDocname,
						   short& constTitle,
						   short inScript = smCurrentScript);
	// Used in TWindow::IWindow to parse the template for window titles. Returns true if it
    // changed the template. preDocname is start of document name in CPascalStr_AC; constChars is
    // total # characters of window title that are constants. (These values are fixed
    // regardless of the document name.) If preDocname is 0, then the entire title is
    // constant.

EXPORT_MA bool SubstituteInTitle(CStr255_AC& title,
						  const CStr255_AC& newStuff,
						  short preDocname,
						  short constTitle);
// Substitutes newStuff into the template as parsed by ParseTitleTemplate; returns true if
// a substitution was made.


// InitUMacApp is now a function again. We don't need the macro because we have native
// exceptions now. The reason for the macro was to force the topmost failure handler
// to be in clients main. This is unnecessary now.

EXPORT_MA void InitUMacApp(short callsToMoreMasters);
	// Initialize the UMacApp unit.
	// Call this in your program after calling InitToolBox and ValidateConfiguration.

EXPORT_MA void InitUMacApp_Step1();
EXPORT_MA void InitUMacApp_Step3(short callsToMoreMasters);

//----------------------------------------------------------------------------------------
// Miscellaneous
//----------------------------------------------------------------------------------------

EXPORT_MA void DoAlertKeyDown(DialogRef theDialog, short itemNo);
	// Given a dialog pointer and button item, simulate pressing the button

EXPORT_MA void DoInitUMacApp();
	// Should be called only by InitUMacApp. It is here because this doesn't need to be
    // universal code but InitUMacApp does.

EXPORT_MA void GetAlertButtonTitle(DialogRef theDialog, short itemNo, CStr255_AC& theTitle);
	// Given a dialog pointer and a button item, return the title of that button

EXPORT_MA bool CompareAlertKeysToItem(DialogRef theDialog, const CStr2_AC& theChars, short& itemHit);
	// Compares the multi-byte char to the first character of each button title
    // 1st button in alert (by convention == "OK"). 2nd button in alert (by convention ==
    // "Cancel"). 3rd button in alert (by convention == "No")

EXPORT_MA void DrawDefaultProc(DialogRef dlog);
	// Used by the MinimalAlertFilter to draw the bold outline around the default button

EXPORT_MA pascal Boolean MinimalAlertFilter(DialogRef theDialog, EventRecord& theEvent, short& itemHit);
	// MacApp's minimal alert filter; handles return & enter, command-. and escape, and
	// draws the bold outline around the default button
	// NOTE: Left with pascal keyword so it could continue to be used with the Toolbox
	// Dialog Manager.
	
EXPORT_MA pascal Boolean MacAppAlertFilter(DialogRef theDialog,
								 EventRecord* theEvent,
								 short* itemHit);
	// Default alert filter used throughout MacApp. Lets you answer yes and no to dialogs
    // with keystrokes. Handles "command-." and escape. If the alert has not been called in a
    // failure sequence then the application also handles update and activate events and
    // idles.
	// NOTE: Requires pascal keyword for use with the Toolbox Dialog Manager.

EXPORT_MA pascal Boolean MacAppAppleEventIdleProc(EventRecord& theEventRecord,
										long& sleepTime,
								 		RgnHandle& mouseRgn);
	// Default AppleEvent idle proc used throughout MacApp. Handles update and activate events
	// NOTE: Requires pascal keyword for use with the Apple Event Manager.

EXPORT_MA pascal Boolean MacAppStandardFileFilter(DialogRef theDialog, EventRecord* theEvent,
							short* itemHit, void *yourDataPtr);
	// Default Filter proc for Standard File dialogs. Handles update and activate events
	// NOTE: Requires pascal keyword for use with Standard File.

EXPORT_MA void ApplicationBeep();
	// If gApplication is not NULL just calls gApplication->Beep else it does a SysBeep.
    // Removes a circular reference for UFailure.

EXPORT_MA DialogRef GetNewCenteredDialog(ResNumber dialogID,
							   Ptr dStorage,
							   WindowRef behind);
	// The same as GetNewDialog except it centers the dialog on the screen and sets the
    // cursor to an arrow.

EXPORT_MA void GetTextStyleFontInfo(const CTextStyle_AC& theTextStyle,
						  FontInfo& theFontInfo,
						  short& theFontHeight);
	// Returns the FontInfo record for the font/ face/ size specified in theTextStyle. Uses a
    // temporary port so you don't have to. Also returns the font's line height.

EXPORT_MA void ExitMacApp();
	// Call this if for some reason you want to immediately exit the application. It calls
    // CleanupMacApp to clean up some other internal stuff, and then calls exit()
    // (Normally, you would not call this, because MacApp takes care of
    // terminating the application.)

EXPORT_MA void InstallIfPrintHandler(TPrintHandler* aPrintHandler, TView* aView);
	// Install a clone of aPrintHandler into the view if aPrintHandler is not the same as
    // the NULL print handler. (Typically if printing is initialized.) This lets us install
    // a print handler if printing is being used in the application and do NOTHING if it
    // is not.

#ifdef __cplusplus
extern "C" {
#endif
EXPORT_MA void CleanupMacApp ();
	// Installed as an atexit procedure.
#ifdef __cplusplus
}
#endif


//==========================================================================================
// The following routines are only supported for System 7
//==========================================================================================

EXPORT_MA OSErr GetSysVolume(short& vRefNum);
	// Get the vRefNum of the system (boot) volume

EXPORT_MA OSErr GetIndVolume(short index,
				   short& vRefNum);
	// Get the vRefNum of an indexed on-line volume

EXPORT_MA OSErr VolHasDesktopDB(short vRefNum,
					  bool& hasDesktop);
	// Check if a volume supports desktop DB calls

EXPORT_MA OSErr FindAppOnVolume(OSType sig,
					  short vRefNum,
					  CFSSpec_AC& thefile);
	// Ask vol's desktop db for application

EXPORT_MA OSErr LaunchAppByFSSpec(const CFSSpec_AC& fileSpec,
						LaunchFlags launchControlFlags,
						ProcessSerialNumber& psn);
	// Launch app given a fileSpec


EXPORT_MA OSErr LaunchBySignature(OSType sig,			// Signature of app
						ProcessSerialNumber& psn,// Returns PSN of app
						CFSSpec_AC * fileSpec,	// Returns location of app (ignored if NULL)
						bool * launched,	// Returns true if I had to launch (NULL ok)
						bool allowLaunch,// OK to launch it if not running?
						LaunchFlags launchControlFlags// Launch control flags; see Process Mgr
						);
	// Main routine. Find app, launching if need be
#endif


