// UMacAppUtilities.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UMACAPPUTILITIES__
#define __UMACAPPUTILITIES__

// ACS

#ifndef __CIterator_AC__
#include "CIterator_AC.h"
#endif

#ifndef __CRegion_AC__
#include "CRegion_AC.h"
#endif

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif
#ifndef __TOOLBOX__
#include "Toolbox.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

// Toolbox

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------
class CStream_AC;
class TView;

//----------------------------------------------------------------------------------------
// Global constants declarations. Build options.
//----------------------------------------------------------------------------------------

// constants passed in as flags to MATextBox()
enum { kDontPreferOutline, kPreferOutline };

//----------------------------------------------------------------------------------------
// Some useful struct(s)
//----------------------------------------------------------------------------------------

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

struct MATextStyle
{
    Style tsFace;					// character Style
    char filler;					// tsFace is unpacked byte
    short tsSize;					// size in points
    CRGBColor_AC tsColor;				// absolute (RGB) color
    CStr255_AC tsFont;					// font (family) number
};

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef MATextStyle *MATextStylePtr, **MATextStyleHandle;


//----------------------------------------------------------------------------------------
// Global variable declarations.
//----------------------------------------------------------------------------------------

#if qDebugMsg
extern const CStr15_AC gBoolString[2];
#endif

EXPORT_MA extern bool gUGridViewInitialized;

// Note: gMATextBoxTE and gTEDefaultWordBreak are now static data members in TextBox_AC.h.

EXPORT_MA extern const VHSelect gOrthogonal[2];

// Note: utility data types previously in this file for point, rect and color are 
// now const static data members of their classes in the corresponding ACS source files 
// (e.g., gZeroRect is now CRect_AC::kZeroRect in CRect_AC.h).


#define gZeroRect	CRect_AC::kZeroRect
#define gZeroPt		CPoint_AC::kZeroPt
#define gZeroVPt	CLongPoint_AC::kZeroPt
#define gZeroVRect	CLongRect_AC::kZeroRect
#define gRGBBlack	CRGBColor_AC::kRGBBlack
#define gRGBWhite	CRGBColor_AC::kRGBWhite

EXPORT_MA extern const CPoint_AC kBestSystemLocation;	// use when system calls take -1,-1 for best window locations
EXPORT_MA extern const CStr2_AC gEmptyString;


//----------------------------------------------------------------------------------------
// CStringListRsrc_AC: A simple class for managing and accessing strings in STR# resources.
//
// Limitations: this class has no notion of which resource file to be used for accessing
// the strings from the CPascalStr_AC list resource.  This could be added as an enhancement.
//----------------------------------------------------------------------------------------

class EXPORT_MA CStringListRsrc_AC
{
public:
	//----------------------------------------------------------------------------------------
	// class-scoped constants
	//----------------------------------------------------------------------------------------
	enum { kDontAddString, kAddString };

	//----------------------------------------------------------------------------------------
	// constructors
	//----------------------------------------------------------------------------------------
	inline CStringListRsrc_AC(short strListID, const CStr255_AC& strListRsrcName) :
			fStrListID(strListID),
			fStrListRsrcName(strListRsrcName)
			{}
		// constructor

	inline CStringListRsrc_AC(short strListID) :
			fStrListID(strListID)
			{  }
		// constructor

	//----------------------------------------------------------------------------------------
	// accessors
	//----------------------------------------------------------------------------------------
	short AppendString(const CStr255_AC& theString);
		// append theString to the STR# resource whose id is fStrListID and return its index
	
	void ClearAll();
		// clears all strings in the STR# resource

	short CountStrings() const;
		// returns the number of strings in the STR# resource whose id is fStrListID

	short FindString(const CStr255_AC& theString, bool addString = kDontAddString);
		// find theString in the STR# resource whose id is fStrListID and return its index
		// if theString is not found in the STR# resource, add it if addString is kAddString
	
	void GetListName(CStr255_AC& itsName);
		//	Returns the name of the STR# list either from its field or if that is empty
		//	from the actual resource

	void GetString(short index, CStr255_AC& theString) const;
		// return in theString the "index" CPascalStr_AC in the STR# resource whose id is fStrListID

	void RemoveAt(short index);
		// removes the CPascalStr_AC at the specified index.
	
	void ReplaceAt(const CStr255_AC& theString, short index);
		// replace the CPascalStr_AC at "index" with "theString"

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	CStr255_AC		fStrListRsrcName;			// The name of the STR# rsrc, this is
											// only used when adding the STR# rsrc.
	ResNumber	fStrListID;					// The ID of the STR# rsrc.
};


//----------------------------------------------------------------------------------------
// CWMgrIterator: A simple iterator for window lists.
//----------------------------------------------------------------------------------------

class EXPORT_MA CWMgrIterator : public CIterator_AC
{
public:
	CWMgrIterator(bool itsForward = true);

	//
	// Required functions 
	//
	WindowRef First();
	inline WindowRef Current() const { return fCurrent; }
	WindowRef Next();

	//
	// Convenience functions for C++meisters
	//
	inline WindowRef operator++() { return Next(); }
	inline WindowRef operator++(int) { WindowRef returnVal = Current(); Next(); return returnVal; }
	inline operator WindowRef() const { return Current(); }
	inline WindowRef operator->() const { return Current(); }

	//
	// Compatibility functions
	//
	inline WindowRef FirstWMgrWindow() { return First(); }
	inline WindowRef CurrentWMgrWindow() const { return Current(); }
	inline WindowRef NextWMgrWindow() { return Next(); }

protected:
	WindowRef NextWindow(WindowRef aWindow);
		// returns the next window in the window list, excluding gWorkPort
		
	WindowRef PreviousWindow(WindowRef aWindow);
		// returns the previous window in the window list, excluding gWorkPort

	WindowRef FirstWindow();
		// returns the first window in the window list, excluding gWorkPort
		
	WindowRef LastWindow();
		// returns the last window in the window list, excluding gWorkPort

private:
	WindowRef fCurrent;
	
protected:
	bool fIterateForward;
};


//----------------------------------------------------------------------------------------
// Global function declarations that needs to always be compiled for 68000.
//----------------------------------------------------------------------------------------

EXPORT_MA void CenterRectOnScreen(CRect_AC& aRect,
						bool horizontally,
						bool vertically,
						bool forDialog);

EXPORT_MA void ConcatNumber(const CStr255_AC& aString,
				  long aNumber,
				  CStr255_AC& theResult);

EXPORT_MA void PullApplicationToFront();

inline void SetRGBColor(CRGBColor_AC& inColor, short inRed, short inGreen, short inBlue) { inColor.Set(inRed, inGreen, inBlue); }

EXPORT_MA OSErr MAInteractWithUser(long timeOutInTicks, NMRecPtr nmReqPtr = NULL, AEIdleUPP idleProc = NULL);

EXPORT_MA OSErr MAInteractWithUser();

EXPORT_MA OSErr MAInteractWithUserNoIdleProc();

//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

#if qNeedsSystem7_5
inline short MACharacterByteType(Ptr textBuf, short textOffset, ScriptCode script)
{ return CharacterByteType(textBuf, textOffset, script); }
#else
EXPORT_MA short MACharacterByteType(Ptr textBuf, short textOffset, ScriptCode script);
#endif

#if qNeedsSystem7_5
inline short MACharacterType(Ptr textBuf, short textOffset, ScriptCode script)
{ return CharacterType(textBuf, textOffset, script); }
#else
EXPORT_MA short MACharacterType(Ptr textBuf, short textOffset, ScriptCode script);
#endif

EXPORT_MA bool CompareMultiByteChars(const CStr31_AC& first,
							  const CStr31_AC& second,
							  bool caseSens);
	// Special case single byte characters and allow case insensitive comparisons

inline short CompareStrings(const CStr255_AC& first, const CStr255_AC& second)
#if qMSWindows
{ return ::QTMLCompareString(first, second, NULL); }
#else
{ return ::CompareString(first, second, NULL); }
#endif

inline void CopyStr255(const CStr255_AC& fmStr, Ptr toAddr)
{ fmStr.CopyTo(reinterpret_cast<unsigned char*>(toAddr)); }

EXPORT_MA WindowRef FindWindowBefore(WindowRef theWindow);

EXPORT_MA void DefaultSize(short& theSize);

EXPORT_MA PicHandle DisposeIfPicHandle(PicHandle aPicHandle);

EXPORT_MA  RgnHandle MakeNewRgn();
inline RgnHandle MakeNewRgn() { return CRegion_AC::MakeNewRgn(); }

EXPORT_MA  RgnHandle DisposeIfRgnHandle(RgnHandle aRgnHandle);
inline RgnHandle DisposeIfRgnHandle(RgnHandle aRgnHandle) { return CRegion_AC::DisposeIfRgnHandle(aRgnHandle); }

EXPORT_MA SectionHandle DisposeIfSectionHandle(SectionHandle aSectionHandle);

EXPORT_MA short GetActualJustification(short justification);

EXPORT_MA void GetDeskTopRegion(RgnHandle deskTopRgn);

EXPORT_MA void ShowGlobalRegion(RgnHandle globalRegion);

EXPORT_MA void ShowViewRegion(TView *theView, RgnHandle viewRegion);

EXPORT_MA short GetFontNum(const CStr255_AC& fontName);

EXPORT_MA CRGBColor_AC GetIfColor();
inline void GetIfColor(CRGBColor_AC& aColor)
{ aColor = GetIfColor(); }

EXPORT_MA CRGBColor_AC GetIfBkColor();
inline void GetIfBkColor(CRGBColor_AC& aColor)
{ aColor = GetIfBkColor(); }

EXPORT_MA CTextStyle_AC GetPortTextStyle();
inline void GetPortTextStyle(CTextStyle_AC& theTextStyle) { theTextStyle = GetPortTextStyle(); }

EXPORT_MA CTextStyle_AC MAGetTextStyle(ResNumber rsrcID);
inline void MAGetTextStyle(ResNumber rsrcID, CTextStyle_AC& theTextStyle) { theTextStyle = MAGetTextStyle(rsrcID); }
// Given the rsrcID of a 'TxSt' resource, this function creates and
// returns in theTextStyle the corresponding TextStyle record.

EXPORT_MA void GetPortFontInfo(short fontNum, CStr255_AC& fontName, short& fontSize);

EXPORT_MA long GetRandom(long rangeStart, long rangeEnd);

EXPORT_MA bool IsColorPort(GrafPtr port);

inline short GetWindowVariant(WindowRef theWindow)
{ return GetWVariant(theWindow); }
// For MacApp 3.0 app's, _GetWVariant is *always* available.

inline short LengthRect(const CRect_AC& r, VHSelect vhs)
{ return r.GetLength(vhs); }

EXPORT_MA VHSelect LongerSide(const CRect_AC& r);
EXPORT_MA VHSelect LongerSide(const CViewRect& r);

inline VHSelect LongerVSide(const CViewRect& r) { return LongerSide(r); }

EXPORT_MA void LIntToHex(long decNumber, CStr31_AC& hexNumber, short noOfDigits);

EXPORT_MA short MAGetFontInfo(FontInfo& theFontInfo);

EXPORT_MA void MADrawString(const CStr255_AC& s,
				  const CRect_AC& box,
				  short justification,
				  bool preferOutline = kDontPreferOutline);

EXPORT_MA void NumberToHex(long theNumber,
				 CStr255_AC& hexString,
				 short hexDigits);

EXPORT_MA bool IsThisKeyDown(const short theKey);
// Returns TRUE if the specified key is currently depressed. (Pushed down, not unhappy!)

EXPORT_MA bool IsCapsLockKeyDown();

EXPORT_MA bool IsCommandKeyDown();

EXPORT_MA bool IsControlKeyDown();

EXPORT_MA bool IsOptionKeyDown();

EXPORT_MA bool IsShiftKeyDown();

EXPORT_MA long PinOnRect(const CRect_AC& theRect, CPoint_AC thePt);

EXPORT_MA void PinOnVRect(const CViewRect& theRect, const CViewPoint& thePt, CViewPoint& thePin);

EXPORT_MA void PointerToHex(long theNumber, CStr31_AC& hexString, short hexDigits);

EXPORT_MA long RoundUp(long aNumber, short aModulus);

EXPORT_MA short SetKeyScript(short newKeyScript);

EXPORT_MA void SetIfColor(const CRGBColor_AC& aColor);

EXPORT_MA void SetIfBkColor(const CRGBColor_AC& aColor);

EXPORT_MA void SetPortTextStyle(const CTextStyle_AC& theTextStyle);

inline void SetTextStyle(CTextStyle_AC& theTextStyle,
				  short theFont,
				  short theFace,
				  short theSize,
				  const CRGBColor_AC& theColor) { theTextStyle.Set(theFont, theFace, theSize, theColor); }
// left in for compatibility

EXPORT_MA void UseSelectionColor();

EXPORT_MA void UseROMMap(bool resLoad);

EXPORT_MA long NumBlocks(long numBytes, long blkSize);

#if qDebug
EXPORT_MA bool CanReadLn();
#else
inline bool CanReadLn()
{ return false; }
#endif

#if qDebug
EXPORT_MA bool CanWriteLn();
#else
inline bool CanWriteLn()
{ return false; }
#endif

inline void CopyPat(const Pattern& pat, Pattern& toPat)
{ toPat = pat; }

EXPORT_MA void XorPat(const Pattern& patA, const Pattern& patB, Pattern& toPat);

EXPORT_MA void RotatePat(Pattern& aPattern);

// Forward declaration for StreamToHandle

class CRandomAccessStream_AC;

EXPORT_MA Handle StreamToHandle(CRandomAccessStream_AC* theStream);

//----------------------------------------------------------------------------------------
// System configuration.
//----------------------------------------------------------------------------------------

#if qOldConfigurationFlags

// The Configuration structure and gConfiguration are obsolete. 
// Use the configuration functions in UMacAppUtilities instead. 
// For example, instead of gConfiguration.hasSpeechManager, 
// use HasSpeechManager(). 

EXPORT_MA Configuration gConfiguration;
#endif

#endif // __UMACAPPUTILITIES__
