//----------------------------------------------------------------------------------------
// USpeakable.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __USPEAKABLE__
#define __USPEAKABLE__

#ifndef __MacAppRequired__
#include "MacAppRequired.h"
#endif

//Toolbox
#ifndef _SPEECH_
#include <Speech.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

//MacApp

#ifndef __UCLASSDESC__
#include "UClassDesc.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __USTREAMABLE__
#include "UStreamable.h"
#endif

//----------------------------------------------------------------------------------------
// MSpeakable : Mixin Class for Speech Technologies
//----------------------------------------------------------------------------------------

class EXPORT_MA MSpeakable 
{
	public:		
		MSpeakable(); 				//uses system defaults
		
		MSpeakable(const CStr63_AC& name); 	//uses custom voice by name
		
		MSpeakable(short index); 	//uses custom voice by index
						
		virtual ~MSpeakable();
		
		// ---------------------------------------------------------
		// Voice Utilites
		// ---------------------------------------------------------
		
		virtual void 	UseVoiceByName(const CStr63_AC& name);
		
		virtual void 	UseVoiceByIndex(short index);
		
		virtual CStr63_AC GetVoiceName(short index) const;
		
		virtual short 	GetNumberVoices() const;
		
		// ---------------------------------------------------------
		// Speech Utilites
		// ---------------------------------------------------------
		
				bool	IsSpeechBusy(bool systemWide) const;
		
		virtual void	WaitForSpeech();
		
		virtual void	StopSpeech(long  whereToStop = kImmediate);
		
		virtual void 	PauseSpeech(long whereToStop = kImmediate);
		
		virtual void 	ContinueSpeech();
	

		// ---------------------------------------------------------
		// Say It
		// ---------------------------------------------------------
		
				void 	Say(float number, short digitsDesired, bool wait); 	//float
		
				void 	Say(long number, bool wait); 						//integer
		
				void 	Say(const char* str, bool wait); 					//string
				
				void 	Say(const TEHandle TEHdl, bool wait); 				//TEHandle
		
		virtual void 	Say(const void *textBuf, short firstByte, short byteCount, bool wait);
			//text, the workhorse. The other Say methods forward into here.
		
		// ---------------------------------------------------------
		// Speech Speed
		// ---------------------------------------------------------
	
		virtual void	SetSpeechRateModifier(Fixed newModifier);
		
		virtual void	SetSpeechRate(Fixed newRate);
		
		virtual void 	SpeakFaster();
		
		virtual void 	SpeakSlower();
		
	protected:
		Fixed			fSpeechRateModifier;
		SpeechChannel	fSpeechChannel;		
		Fixed			fSpeechRate;
};

#endif