/************************************************************

    ViewTypes.r
	MacApp 'View' type template definitions.

    Copyright  1985-96 by Apple Computer, Inc. All rights reserved. 

************************************************************/

#ifndef __ViewTypes__
#define __ViewTypes__

/*------------------------------------Equates-----------------------------------*/
// some MacApp types
#define VRECT wide array [4] { longint; }
#define VPOINT wide array [2] { longint; }
#define RGBCOLOR wide array [3] { unsigned integer; }

// see comments in UMacAppUtilities about justLeft
#define justLeft justSystem

// Version number is: main release in upper byte, and dot release in lower byte.
#define	kMAThreeOhTemplateVersion 0x0300
#define kMARelease13ViewsTemplateVersion 0x1300

// some useful values
#define gZeroVPt { 0, 0 }
#define gZeroVRect { 0, 0, 0, 0 }
#define	kNoIdentifier '    '
#define kNoResource -1
#define kMaxIdleTime 2147483647

// adorn priorities
#define	kAdornLast 127
#define	kAdornAfter 95
#define	kDrawView 64
#define	kAdornBefore 32
#define	kAdornFirst 0

// signatures
#define	kSectionBehavior 'sect'
#define	kDialogBehavior 'dlog'
#define	kDragDropBehavior 'drag'
#define	kTabBehavior 'tabb'

// Delimiter
#define	kNullObject 0
#define	kLocalObjectNumber 1
#define	kClassIDAndClassname 2
#define	kSignatureAndClassname 3

/*	The following are used to set adornments.  Simply add for desired value.	*/
#define adnNone 		0b00000000
#define adnLineTop		0b00000001
#define adnLineLeft 	0b00000010
#define adnLineBottom	0b00000100
#define adnLineRight	0b00001000
#define adnFrame		0b00001111
#define adnPatFill		0b00010000
#define adnOval 		0b00100000
#define adnRRect		0b01000000
#define adnShadow		0b10000000

#define noAdornment 	adnNone, {1, 1}
#define arrowsAndBackspace	0xF0000100
#define carriageReturn	0x00002000
#define black			{0, 0, 0}
#define noInset 		{0, 0, 0, 0}
#define stdInset 		{3, 3, 3, 3}

#define systemFont		plain,	0, black, ""
#define applFont9		plain,	9, black, "a"
#define applFont12		plain, 12, black, "a"

#define IncludeViews	noID, {0, 0}, {0, 0}, 0, 0, 0, 0, Include
#define IncludeViewsAt	noID, {0, 0}, {0, 0}, 0, 0, 0, 0, IncludeAt

#define kNoResource 	-1
#define kSBarSizeMinus1 15

#define MAThreeOh		MAThreeOhView {}
#define MARelease13		MARelease13View {}

/*-------------------------------------drawing environment----------------------*/
#define TDRAWINGENVIRONMENTDATA	\
		point;							/* fPenSize 		*/					\
		integer;						/* fPenMode 		*/					\
		hex string[8];					/* fPenPattern		*/					\
		RGBCOLOR;						/* fForegroundColor */					\
		RGBCOLOR						/* fBackgroundColor */

/*-------------------------------------view-------------------------------------*/
#define TVIEWDATA_THREEOH	\
		VPOINT;							/* Location inside parent view */		\
		VPOINT;							/* Size of view */						\
																				\
		unsigned bitstring[8]			/* Vertical view determiner */			\
					sizeSuperView,		/* View is the same size as superview */\
					sizeRelSuperView,	/* Size is relative to the superview */	\
					sizePage,			/* View to be the size of one page */	\
					sizeFillPages,		/* View fills an exact # of pages */	\
					sizeVariable,		/* Size criteria is app-specific */		\
					sizeFixed;			/* No default handling of size */		\
																				\
		unsigned bitstring[8]			/* Horizontal view determiner */		\
					sizeSuperView,		/* View is the same size as superview */\
					sizeRelSuperView,	/* Size is relative to the superview */	\
					sizePage,			/* View to be the size of one page */	\
					sizeFillPages,		/* View fills an exact # of pages */	\
					sizeVariable,		/* Size criteria is app-specific */		\
					sizeFixed;			/* No default handling of size */		\
																				\
		unsigned byte 	notShown, shown;										\
		unsigned byte 	doesntWantToBeTarget, wantsToBeTarget;					\
																				\
		unsigned byte 	doesntHandleCursor, handlesCursor;						\
		unsigned byte 	doesntLetSubViewsHandleCursor, letsSubViewsHandleCursor;\
		integer			noCursorID = kNoResource;	/* cursorID */				\
																				\
		unsigned byte 	doesntHandleHelp, handlesHelp;							\
		unsigned byte 	doesntLetSubViewsHandleHelp, letsSubViewsHandleHelp;	\
		integer			noHelpID = kNoResource;		/* helpID */				\
		integer;						/* help index */						\
																				\
StartOfDrawingEnvironment:														\
		switch {						/* drawing environment */				\
			case NoDrawingEnvironment:											\
				key integer = kNullObject;										\
																				\
			case DrawingEnvironmentLocalObject:									\
				key integer = kLocalObjectNumber;								\
				longint;														\
																				\
			case DrawingEnvironmentClassIDAndClassname:							\
				key integer = kClassIDAndClassname;								\
  				longint;								/* class id */			\
				longint = ((EndOfDrawingEnvironment[$$ArrayIndex(ViewArray)] - StartOfDrawingEnvironment[$$ArrayIndex(ViewArray)]) / 8) - 6;	\
				pstring DrawingEnvironment = "TDRAWINGENVIRONMENT";				\
				TDRAWINGENVIRONMENTDATA;										\
																				\
			case DrawingEnvironmentSignatureAndClassname:						\
				key integer = kSignatureAndClassname;							\
				literal longint NoIdentifier = kNoIdentifier;/* signature */	\
				longint = ((EndOfDrawingEnvironment[$$ArrayIndex(ViewArray)] - StartOfDrawingEnvironment[$$ArrayIndex(ViewArray)]) / 8) - 6;	\
				pstring;														\
				TDRAWINGENVIRONMENTDATA;										\
		};																		\
EndOfDrawingEnvironment:														\
																				\
StartOfAdorners:																\
		switch {						/* adorner list */						\
			case NoAdorners:													\
				key integer = kNullObject;										\
																				\
			case AdornerListLocalObject:										\
				key integer = kLocalObjectNumber;								\
				longint;														\
																				\
			case AdornerListClassIDAndClassname:								\
				key integer = kClassIDAndClassname;								\
  				longint;								/* class id */			\
ClassIDAdornersSize:	longint = ((EndOfAdorners[$$ArrayIndex(ViewArray)] - StartOfAdorners[$$ArrayIndex(ViewArray)]) / 8) - 6;\
				hex string[$$Long(ClassIDAdornersSize[$$ArrayIndex(ViewArray)]) - 4];	\
																				\
			case AdornerListSignatureAndClassname:								\
				key integer = kSignatureAndClassname;							\
				literal longint NoIdentifier = kNoIdentifier;/* signature */	\
				longint = ((EndOfAdorners[$$ArrayIndex(ViewArray)] - StartOfAdorners[$$ArrayIndex(ViewArray)]) / 8) - 6;\
				pstring AdornerList = "TADORNERLIST";							\
				integer AdornerElementSize = 6;		/* element size */			\
				integer AdornerElementSizeShift = 3;/* element size shift */	\
				longint DynamicArrayAllocationIncrement = 6;	/* allocation increment */	\
				longint = $$CountOf(AdornerElementArray);	/* number of adorners */\
				wide array AdornerElementArray {								\
					unsigned byte	AdornLast = kAdornLast,						\
									AdornAfter = kAdornAfter,					\
									DrawView = kDrawView,						\
									AdornBefore = kAdornBefore,					\
									AdornFirst = kAdornFirst;					\
StartOfEachAdorner:																\
					switch {						/* each adorner */			\
						case AdornerLocalObject:								\
							key integer = kLocalObjectNumber;					\
							longint	DimAdorner = 1,								\
									DrawAdorner = 2,							\
									EraseAdorner = 3,							\
									HiliteAdorner = 4,							\
									PrintAdorner = 5,							\
									ResizeIconAdorner = 6,						\
									SelectionAdorner = 7;						\
																				\
						case AdornerClassIDAndClassname:						\
							key integer = kClassIDAndClassname;					\
							longint;				/* class id */				\
ClassIDEachAdornerSize:		longint = ((EndOfEachAdorner[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)] - StartOfEachAdorner[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)]) / 8) - 6;\
							hex string[$$Long(ClassIDEachAdornerSize[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)]) - 4];	\
																				\
						case AdornerSignatureAndClassname:						\
							key integer = kSignatureAndClassname;				\
							literal longint NoIdentifier = kNoIdentifier;/* signature */	\
SignatureEachAdornerSize:	longint = ((EndOfEachAdorner[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)] - StartOfEachAdorner[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)]) / 8) - 6;\
							pstring CompatibilityAdorner = "TCOMPATIBILITYADORNER";			\
							literal longint NoIdentifier = kNoIdentifier;/* signature */	\
							unsigned byte 	dontFreeOnDeletion, freeOnDeletion;	\
DataEachAdornerSize:		hex string[$$Long(SignatureEachAdornerSize[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)]) - ((DataEachAdornerSize[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)] - SignatureEachAdornerSize[$$ArrayIndex(ViewArray), $$ArrayIndex(AdornerElementArray)]) / 8)];	\
					};															\
EndOfEachAdorner:																\
				};																\
		};																		\
EndOfAdorners:																	\
																				\
		longint	emptyUserArea = 0			/* userArea */

/*-------------------------------------window-----------------------------------*/
#define TWINDOWDATA_THREEOH	\
		integer 		documentProc, dBoxProc, plainDBox, altDBoxProc,			\
						noGrowDocProc, movableDBoxProc, zoomDocProc = 8,		\
						zoomNoGrow = 12, rDocProc = 16;							\
		literal longint noID = kNoIdentifier;									\
		boolean 		noGoAwayBox, goAwayBox;									\
		boolean 		notResizable, resizable;								\
		boolean 		ignoreFirstClick, doFirstClick;							\
		boolean 		dontFreeOnClosing, freeOnClosing;						\
		boolean 		dontDisposeOnFree, disposeOnFree;						\
		boolean 		doesntCloseDocument, closesDocument;					\
		boolean 		dontOpenWithDocument, openWithDocument;					\
		boolean 		dontAdaptToScreen, adaptToScreen;						\
		boolean 		dontStagger, stagger;									\
		boolean 		dontForceOnScreen, forceOnScreen;						\
		unsigned bitstring[2]	dontCenter, centerHorizontally,					\
									centerVertically, center;					\
		boolean 		doesntFloat, floats;									\
		boolean 		doesntHideOnSuspend, hidesOnSuspend;					\
		boolean 		doesntGenerateActivates, generateActivates;				\
		boolean			filler;													\
		integer;		/* window title:	STR# rsrc ID			*/ 			\
		integer			/*					index into STR# rsrc ID */

/*-------------------------------------scroller-----------------------------------*/
#define TSCROLLERDATA_THREEOH	\
			literal	longint noID = kNoIdentifier;	/* ID of vert scrollbar */	\
			literal longint noID = kNoIdentifier;	/* ID of horz scrollbar */	\
			VPOINT;									/* scroll limit */			\
			VPOINT;									/* scroll units */			\
			byte		noVertConstrain, VertConstrain;							\
			byte		noHorzConstrain, HorzConstrain;							\
			VRECT;									/* Scroll bar offsets */	\
			byte		doesntRespondToFKeys, respondsToFKeys	/* Function keys */

/*-------------------------------------dialog view-----------------------------------*/
#define TDIALOGVIEW_THREEOH	\
	/* Id of default item */	literal longint noID = kNoIdentifier;		\
	/* Id of cancel item */		literal longint noID = kNoIdentifier

/*-------------------------------------target border view-----------------------------------*/
#define TTARGETBORDERVIEW_THREEOH	\
	/* Id of target view */		literal longint noID = kNoIdentifier

/*-------------------------------------control-----------------------------------*/
#define TCONTROLDATA_THREEOH	\
	/* fDefChoice		*/		unsigned longint	mNoHit, mOKHit, mCancelHit,	\
				mButtonHit, mCheckBoxHit, mClusterHit, mEditTextHit, mIconHit,	\
				mListItemHit, mListScrollBarHit, mPictureHit, mPopupHit,		\
				mRadioHit, mStaticTextHit, mHScrollBarHit, mVScrollBarHit,		\
				mEditTabKey, mEditReturnKey, mEditEnterKey, mPatternHit,		\
				mControlHit, mCancelKey, mDefaultKey, mSmallIconHit,			\
				mDismiss = 34;													\
																				\
								unsigned byte 	notHilited, hilited;			\
								unsigned byte 	notDimmed, dimmed;				\
								unsigned byte	notSizeable, sizeable;			\
	/* Control inset	*/		VRECT; 											\
	/* TextStyle RsrcID */		integer;										\
								unsigned byte dontPreferOutline, preferOutline

/*-------------------------------------button-----------------------------------*/
#define TBUTTONDATA_THREEOH 	\
	/* Button label STR# rsrc ID */ integer;									\
	/* index into STR# rsrc ID */	integer

/*-------------------------------------check box-----------------------------------*/
#define TCHECKBOXDATA_THREEOH	\
	/* Initially on 	*/		byte		off, on;							\
	/* check box label STR# rsrc ID */ integer;									\
	/* index into STR# rsrc ID */	integer

/*-------------------------------------radio button-----------------------------------*/
#define TRADIODATA_THREEOH		\
	/* Initially on 	*/		byte		off, on;							\
	/* radio's label STR# rsrc ID */ integer;									\
	/* index into STR# rsrc ID */	integer

/*-------------------------------------scrollbar-----------------------------------*/
#define TSCROLLBARDATA_THREEOH	\
	/* its Direction	*/		byte v,h;										\
	/* Initial value	*/		longint;										\
	/* Minimum value	*/		longint;										\
	/* Maximum value	*/		longint

/*-------------------------------------cluster-----------------------------------*/
#define TCLUSTERDATA_THREEOH	\
	/* Cluster's label STR# rsrc ID */ integer;									\
	/* index into STR# rsrc ID */	integer

/*-------------------------------------icon-----------------------------------*/
#define TICONDATA_THREEOH		\
	/* If 'cicn' first	*/		byte		dontPreferColor, preferColor;		\
	/* ICON or cicn id	*/		integer

/*-------------------------------------small icon-----------------------------------*/
#define TSMALLICONDATA_THREEOH		\
	/* ICON or cicn id	*/		integer

/*-------------------------------------pattern-----------------------------------*/
#define TPATTERNDATA_THREEOH	\
	/* If 'ppat' first	*/		byte		dontPreferColor, preferColor;		\
	/* PAT	or ppat id	*/		integer

/*-------------------------------------picture-----------------------------------*/
#define TPICTUREDATA_THREEOH	\
	/* PICT rsrc id 	*/		integer

/*-------------------------------------popup-----------------------------------*/
#define TPOPUPDATA_THREEOH		\
	/* MENU rsrc ID 	*/		integer;										\
	/* Current item #	*/		integer;										\
	/* Item left offset */		integer;										\
	/* style			*/		unsigned byte	plain;							\
	/* Justification	*/		integer 	forceLeft = -2, justRight,			\
											justSystem, justCenter;				\
								unsigned byte	dontUseAddResMenu, useAddResMenu;	\
								literal longint noID = kNoIdentifier;			\
	/* STR# rsrc ID 	*/		integer;										\
	/* rsrc index		*/		integer

/*-------------------------------------static text-----------------------------------*/
#define TSTATICTEXTDATA_THREEOH \
	/* autowrap 		*/		unsigned byte	dontAutoWrap, autoWrap;			\
	/* erase first 		*/		unsigned byte	dontEraseFirst, eraseFirst;		\
	/* Justification	*/		integer 	forceLeft = -2, justRight,			\
											justSystem, justCenter;				\
	/* stat text STR# rsrc ID */ integer;										\
	/* index into STR# rsrc ID */	integer

/*-------------------------------------edit text-----------------------------------*/
#define TEDITTEXTDATA_THREEOH	\
	/* Max. characters	*/		integer 	unlimited = 255;					\
	/* Good cntl chars	*/		unsigned binary longint	LeftRightUpDownBackspace = 0b11110000000000000000000100000000

/*-------------------------------------number text-----------------------------------*/
#define TNUMBERTEXTDATA_THREEOH \
	/* Initial value	*/		longint;										\
	/* Minimum value	*/		longint;										\
	/* Maximum value	*/		longint

/*-------------------------------------tteview-----------------------------------*/
#define TTEVIEWDATA_THREEOH 	\
	/* Styled flag		*/		unsigned byte	withoutStyle, withStyle;		\
	/* Auto wrapping	*/		unsigned byte	crOnly, autoWrap;				\
	/* Changes flag 	*/		unsigned byte	dontAcceptChanges, acceptChanges;	\
	/* Free text flag	*/		unsigned byte	dontFreeText, freeText; 		\
	/* Key cmd number	*/		longint 	cTyping = 120;						\
	/* Max number chars */		integer 	unlimited = 32767;					\
	/* Inset			*/		wide array [4] { unsigned longint; };			\
	/* Justification	*/		integer 	forceLeft = -2, justRight,			\
											justSystem, justCenter;				\
	/* TextStyle RsrcID */		integer;										\
								unsigned byte dontPreferOutline, preferOutline

/*-------------------------------------gridview-------------------------------------*/
#define TGRIDVIEWDATA_THREEOH	\
	/* Number of rows	*/		integer;										\
	/* Number of cols	*/		integer;										\
	/* row height		*/		integer;										\
	/* column width 	*/		integer;										\
	/* row inset		*/		integer;										\
	/* column inset 	*/		integer;										\
	/* adorn rows		*/		unsigned byte 	dontAdornRows, AdornRows;			\
	/* adorn columns	*/		unsigned byte 	dontAdornCols, AdornCols;			\
	/* single select.	*/		unsigned byte 	multipleSelection, singleSelection

/*-------------------------------------textgridview-------------------------------------*/
#define TTEXTGRIDVIEWDATA_THREEOH	\
	/* TextStyle RsrcID */		integer;										\
								unsigned byte dontPreferOutline, preferOutline


#define TBEHAVIORDATA \
	/* fEnabled			*/					byte	notEnabled, enabled;		\
  	/* fIdleFreq		*/					longint noIdle = kMaxIdleTime

#define TTABBEHAVIORDATA \
	/* fRecursive		*/					byte	notRecursive, recursive

#define TDIALOGBEHAVIORDATA \
	/* fModal			*/					byte	notModal, modal;			\
	/* fDefaultItem		*/					literal longint noID = kNoIdentifier;\
	/* fCancelItem		*/					literal longint noID = kNoIdentifier

#define TDRAGDROPBEHAVIORDATA \
	/* fDraggable			*/				byte	notDraggable, draggable;				\
	/* fDroppable			*/				byte	notDroppable, droppable;				\
	/* fDragMoveDeterminer	*/				integer neverMove, moveWithinView,				\
													moveWithinWindow, moveWithinContext,	\
													moveWithinApplication;					\
	/* fDragMoveFamilyID	*/				literal longint noID = kNoIdentifier

type 'View' {
	switch {
		case MAThreeOhView:
			key literal integer = kMAThreeOhTemplateVersion;	// Version number
	};
  	
  	
	wide array ViewArray {
StartOfView:
		switch {
			case NullObject:
				key integer = kNullObject;

			case LocalObject:
				key integer = kLocalObjectNumber;
  				longint;								// local object number

			case ViewSignatureAndClassname:
				key integer = kSignatureAndClassname;
				literal longint;
				//longint = $$Long(Signature);			// Template signature
//				longint = ((EndOfView[$$ArrayIndex(ViewArray)] - StartOfView[$$ArrayIndex(ViewArray)]) / 8) - 6;
				longint;
				pstring;								// Class name

				literal longint noID = kNoIdentifier;	/* fIdentifier		*/
				byte	notEnabled, enabled;			/* fEnabled			*/
				longint noIdle = kMaxIdleTime; 			/* fIdleFreq		*/
				integer = $$CountOf(BehaviorArray);		/* numBehaviors		*/
				wide array BehaviorArray {				/* behaviors		*/		
StartOfBehavior:
					switch {
						case BehaviorNullObject:
							key integer = kNullObject;

						case BehaviorLocalObject:
							key integer = kLocalObjectNumber;
  							longint;

						case BehaviorClassIDAndClassname:
							key integer = kClassIDAndClassname;
							longint;					/* class id			*/
							longint = ((EndOfBehavior[$$ArrayIndex(ViewArray), $$ArrayIndex(BehaviorArray)] - StartOfBehavior[$$ArrayIndex(ViewArray), $$ArrayIndex(BehaviorArray)]) / 8) - 6;
							pstring;
							switch {
								case Behavior:
									key literal longint = kNoIdentifier;
									TBEHAVIORDATA;

								case TabBehavior:
									key literal longint = kTabBehavior;
									TBEHAVIORDATA;
									TTABBEHAVIORDATA;

								case DialogBehavior:
									key literal longint = kDialogBehavior;
									TBEHAVIORDATA;
									TDIALOGBEHAVIORDATA;

								case DragDropBehavior:
									key literal longint = kDragDropBehavior;
									TBEHAVIORDATA;
									TDRAGDROPBEHAVIORDATA;
							};

						case BehaviorSignatureAndClassname:
							key integer = kSignatureAndClassname;
							literal longint noID = kNoIdentifier;	/* signature		*/
							longint = ((EndOfBehavior[$$ArrayIndex(ViewArray), $$ArrayIndex(BehaviorArray)] - StartOfBehavior[$$ArrayIndex(ViewArray), $$ArrayIndex(BehaviorArray)]) / 8) - 6;
							pstring;
							switch {
								case Behavior:
									key literal longint = kNoIdentifier;
									TBEHAVIORDATA;

								case TabBehavior:
									key literal longint = kTabBehavior;
									TBEHAVIORDATA;
									TTABBEHAVIORDATA;

								case DialogBehavior:
									key literal longint = kDialogBehavior;
									TBEHAVIORDATA;
									TDIALOGBEHAVIORDATA;

								case DragDropBehavior:
									key literal longint = kDragDropBehavior;
									TBEHAVIORDATA;
									TDRAGDROPBEHAVIORDATA;
							};

					};
EndOfBehavior:
				};

				switch {
					case MAThreeOhView:
						key literal integer = kMAThreeOhTemplateVersion;	/* Version number */


					case MARelease13View:
						key literal integer = kMARelease13ViewTemplateVersion;
StartOfViewFields:		longint = ((EndOfViewFields[$$ArrayIndex(ViewArray)] - StartOfViewFields[$$ArrayIndex(ViewArray)]) / 8) - 6;
				};



				TVIEWDATA_THREEOH;

Signature:		switch {
					case IncludeAt:
						key literal longint = 'inc@';	// Template signature

						integer;						// id of included view rsrc
						VPOINT;							// Location to offset subviews			

					case View:
						key literal longint = 'view';	// Template signature

					case Window:
						key literal longint = 'wind';	// Template signature

						TWINDOWDATA_THREEOH;

					case FloatWindow:
						key literal longint = 'fwnd';	// Template signature

						TWINDOWDATA_THREEOH;

					case Scroller:
						key literal longint = 'scrl';	// Template signature

						TSCROLLERDATA_THREEOH;

					case DialogView:
						key literal longint = 'dlog';	// Template signature

						TDIALOGVIEW_THREEOH;

					case TargetBorderView:
						key literal longint = 'tbor';	// Template signature

						TTARGETBORDERVIEW_THREEOH;

					case Control:
						key literal longint = 'cntl';	// Template signature

						TCONTROLDATA_THREEOH;

					case Button:
						key literal longint = 'butn';	// Template signature

						TCONTROLDATA_THREEOH;
						TBUTTONDATA_THREEOH;

					case CheckBox:
						key literal longint = 'chkb';	// Template signature

						TCONTROLDATA_THREEOH;
						TCHECKBOXDATA_THREEOH;

					case Radio:
						key literal longint = 'radb';	// Template signature

						TCONTROLDATA_THREEOH;
						TRADIODATA_THREEOH;

					case ScrollBar:
						key literal longint = 'sbar';	// Template signature

						TCONTROLDATA_THREEOH;
						TSCROLLBARDATA_THREEOH;

					case ScrollerScrollBar:
						key literal longint = 'ssbr';	// Template signature

						TCONTROLDATA_THREEOH;
						TSCROLLBARDATA_THREEOH;

					case Cluster:
						key literal longint = 'clus';	// Template signature

						TCONTROLDATA_THREEOH;
						TCLUSTERDATA_THREEOH;

					case Icon:
						key literal longint = 'icon';	// Template signature

						TCONTROLDATA_THREEOH;
						TICONDATA_THREEOH;

					case SmallIcon:
						key literal longint = 'sicn';	// Template signature

						TCONTROLDATA_THREEOH;
						TSMALLICONDATA_THREEOH;

					case Pattern:
						key literal longint = 'patn';	// Template signature

						TCONTROLDATA_THREEOH;
						TPATTERNDATA_THREEOH;

					case Picture:
						key literal longint = 'pict';	// Template signature

						TCONTROLDATA_THREEOH;
						TPICTUREDATA_THREEOH;

					case Popup:
						key literal longint = 'popp';	// Template signature

						TCONTROLDATA_THREEOH;
						TPOPUPDATA_THREEOH;

					case StaticText:
						key literal longint = 'stat';	// Template signature

						TCONTROLDATA_THREEOH;
						TSTATICTEXTDATA_THREEOH;

					case EditText:
						key literal longint = 'edit';	// Template signature

						TCONTROLDATA_THREEOH;
						TSTATICTEXTDATA_THREEOH;
						TEDITTEXTDATA_THREEOH;

					case NumberText:
						key literal longint = 'nmbr';	// Template signature

						TCONTROLDATA_THREEOH;
						TSTATICTEXTDATA_THREEOH;
						TEDITTEXTDATA_THREEOH;
						TNUMBERTEXTDATA_THREEOH;

					case TEView:
						key literal longint = 'tevw';	// Template signature

						TTEVIEWDATA_THREEOH;

					case GridView:
						key literal longint = 'grid';	// Template signature

						TGRIDVIEWDATA_THREEOH;

					case TextGridView:
						key literal longint = 'txtg';	// Template signature

						TGRIDVIEWDATA_THREEOH;
						TTEXTGRIDVIEWDATA_THREEOH;

					case TextListView:
						key literal longint = 'lstg';	// Template signature

						TGRIDVIEWDATA_THREEOH;
						TTEXTGRIDVIEWDATA_THREEOH;

					case OTHER:
						key literal longint = 'othr';	// Template signature

						integer = (($$CountOf(dataArray) + 5) / 2 ) * 2;
						wide array dataArray {
							unsigned hex byte;
						};

// auto-defined by script system
#if defined(XIncludeOtherViewTypes)
#IF XIncludeOtherViewTypes
#Include $$Shell("XOtherViewTypesSrc");	// let end users extend the view type
#ENDIF
#ENDIF
				};

EndOfViewFields: 
			integer NoSubviews = 0;	// count of subviews
		};
EndOfView:
	};
};
#endif
