/*
	ErrorMgr.r
	MacApp Error manager Resources
	Copyright  1990-97 by Apple Computer, Inc. All rights reserved.
	
	For an explanation of how the error tables work please refer to
	the UErrorMgr.cp source file.
*/

/*  Auto-Include the requirements for this source */
#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

// NOTE:
//	By Toolbox convention, the OK button is item 1 and the Cancel button
//		is item 2.

// One of 3 alerts for indicating errors.
resource 'DITL' (phGenError,
#if qNames
	"phGenError",
#endif
purgeable) {
	{
		{151, 288, 171, 348},
			Button { enabled, "OK" },
		{10, 75, 138, 348},
			StaticText { disabled, "Could not ^2 because ^0. ^1" },
			//	^0 = reason for error; e.g., 'the disk is locked'
			//	^1 = how to recover; e.g., 'Eject the disk and move the tab.'
			//	^2 = what operation failed; e.g., 'save the document'
		{10, 20, 42, 52},
			Icon { disabled, stopIcon }
	}
};

resource 'DITL' (phCommandError,
#if qNames
	"phCommandError",
#endif
	purgeable) {
	{
		{151, 288, 171, 348},
			Button { enabled, "OK" };
		{10, 75, 138, 348},
			StaticText { disabled, "Could not complete the '^2' command because ^0. ^1" };
			//	^0 = reason for error; e.g., 'the disk is locked'
			//	^1 = how to recover; e.g., 'Eject the disk and move the tab.'
			//	^2 = what menu command failed; e.g., 'New'
		{10, 20, 42, 52},
			Icon { disabled, stopIcon }
	}
};

//	This alert is used if the program does not provide any indication
//		of what operation failed.

resource 'DITL' (phUnknownErr,
#if qNames
	"phUnknownErr",
#endif
	purgeable) {
	{
		{151, 288, 171, 348},
			Button { enabled, "OK" };
		{10, 75, 138, 348},
			StaticText { disabled, "Could not complete your request because ^0. ^1" };
			//	^0 = reason for error; e.g., 'the disk is locked'
			//	^1 = how to recover; e.g., 'Eject the disk and move the tab.'
		{10, 20, 42, 52},
			Icon { disabled, stopIcon }
	}
};


//  Used when the space is very low.

resource 'DITL' (phSpaceIsLow,
#if qNames
	"phSpaceIsLow",
#endif
	purgeable) {
	{
		{55, 288, 75, 348},
			Button { enabled, "OK" };
		{10, 75, 42, 348},
			StaticText { disabled, "Memory space is low. Some commands may be disabled." };
		{10, 20, 42, 52},
			Icon { disabled, noteIcon }
	}
};


// Used when the user attempts to run an app on unsupported ROMs.

resource 'DITL' (phUnsupportedConfiguration,
#if qNames
	"phUnsupportedConfiguration",
#endif
	purgeable) {
	{
		{71, 288, 91, 348},
			Button { enabled, "OK" };
		{10, 75, 58, 348},
			StaticText { disabled, "Your computer lacks the required hardware or system software to run this application." };
		{10, 20, 42, 52},
			Icon { disabled, stopIcon }
	}
};

// Used to confirm saving changes to a document when closing or quitting.

resource 'DITL' (phSaveChanges,
#if qNames
	"phSaveChanges",
#endif
	purgeable) {
	{
		{71, 288, 91, 348},
			Button { enabled, "Save" },
		{71, 215, 91, 275},
			Button { enabled, "Cancel" },
		{71, 72, 91, 156},
			Button { enabled, "Don't Save" },
		{10, 75, 58, 348},
			StaticText { disabled, "Save changes to the ^2 document '^0' before ^1?" },
		{10, 20, 42, 52},
			Icon { disabled, cautionIcon }
	}
};

// Used to confirm a Revert command.

resource 'DITL' (phRevert,
#if qNames
	"phRevert",
#endif
	purgeable) {
	{
		{71, 288, 91, 348},
			Button { enabled, "Revert" },
		{71, 215, 91, 275},
			Button { enabled, "Cancel" },
		{10, 75, 58, 348},
			StaticText { disabled, "Revert to the last saved version of '^0'?" },
		{10, 20, 42, 52},
			Icon { disabled, cautionIcon }
	}
};


// Used when a document changes unexpectedly on the disk; e.g., if
//	the document in on a file server and 2 people open it.

resource 'DITL' (phFileChanged,
#if qNames
	"phFileChanged",
#endif
	purgeable) {
	{
		{103, 288, 123, 348},
			Button { enabled, "OK" };
		{103, 215, 123, 275},
			Button { enabled, "Cancel" };
		{10, 75, 90, 348},
			StaticText { disabled,
				"The disk copy of '^0' was changed since you last "
				"opened or saved it. Do you want to ^1 anyway?" };
			//	^0 = name of document
			//	^1 = the operation (e.g., 'save' or 'revert')
		{10, 20, 42, 52},
			Icon { disabled, cautionIcon }
	}
};

// Used when the document cannot be saved unless the original version is
//	deleted from the disk.	This gives the user a chance to confirm this.
	
resource 'DITL' (phPurgeOld,
#if qNames
	"phPurgeOld",
#endif
	purgeable) {
	{
		{103, 288, 123, 348},
			Button { enabled, "Delete" };
		{103, 215, 123, 275},
			Button { enabled, "Cancel" };
		{10, 75, 90, 348},
			StaticText { disabled,
				"There is not enough disk space to save the document "
				"unless the existing copy is deleted first. Delete existing copy of '^0'?" };
		{10, 20, 42, 52},
			Icon { disabled, cautionIcon }
	}
};

// Used when the user opens the same document twice.

resource 'DITL' (phReopenDoc,
#if qNames
	"phReopenDoc",
#endif
	purgeable) {
	{
		{55, 288, 75, 348},
			Button { enabled, "OK" };
		{10, 75, 42, 348},
			StaticText { disabled, "'^0' is already open." };
			//	^0 = name of document.
		{10, 20, 42, 52},
			Icon { disabled, stopIcon }
	}
};


// Used to offer a read-only copy of a file when writing isn't possible.

resource 'DITL' (phOfferReadOnly,
#if qNames
	"phOfferReadOnly",
#endif
	purgeable) {
	{
		{71, 288, 91, 348},
			Button { enabled, "OK" };
		{71, 215, 91, 275},
			Button { enabled, "Cancel" };
		{10, 75, 58, 348},
			StaticText { disabled, "'^0' can't be opened for writing. Do you want to open it anyway?" };
			//	^0 = name of document.
		{10, 20, 42, 52},
			Icon { disabled, cautionIcon }
	}
};

// Used in dialogs and TEViews when the user exceeds the # of characters.

resource 'DITL' (phTooManyChars,
#if qNames
	"phTooManyChars",
#endif
	purgeable) {
	{
		{55, 288, 75, 348},
			Button { enabled, "OK" };
		{10, 75, 42, 348},
			StaticText { disabled, "Cannot add any more characters here." };
		{10, 20, 42, 52},
			Icon { disabled, stopIcon }
	}
};

resource 'DITL' (phStylesTooBig,
#if qNames
	"phStylesTooBig",
#endif
	purgeable) {
	{
		{71, 288, 91, 348},
			Button { enabled, "OK" };
		{10, 75, 58, 348},
			StaticText { disabled, "Cannot remember the styles associated with the text. The text is intact, however." };
		{10, 20, 42, 52},
			Icon { disabled, stopIcon }
	}
};

resource 'ALRT' (phGenError,
#if qNames
	"phGenError",
#endif
	purgeable) {
	{80, 80, 261, 438},
	phGenError,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phCommandError,
#if qNames
	"phCommandError",
#endif
	purgeable) {
	{80, 80, 261, 438},
	phCommandError,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phUnknownErr,
#if qNames
	"phUnknownErr",
#endif
	purgeable) {
	{80, 80, 261, 438},
	phUnknownErr,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phSpaceIsLow,
#if qNames
	"phSpaceIsLow",
#endif
	purgeable) {
	{100, 110, 185, 468},
	phSpaceIsLow,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phUnsupportedConfiguration,
#if qNames
	"phUnsupportedConfiguration",
#endif
	purgeable) {
	{90, 110, 191, 468},
	phUnsupportedConfiguration,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionMainScreen
};

resource 'ALRT' (phSaveChanges,
#if qNames
	"phSaveChanges",
#endif
	purgeable) {
	{86, 60, 187, 418},
	phSaveChanges,
	{
		OK, visible, silent,
		OK, visible, silent,
		OK, visible, silent,
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phRevert,
#if qNames
	"phRevert",
#endif
	purgeable) {
	{86, 60, 187, 418},
	phRevert,
	{
		Cancel, visible, silent,
		Cancel, visible, silent,
		Cancel, visible, silent,
		Cancel, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phFileChanged,
#if qNames
	"phFileChanged",
#endif
	purgeable) {
	{100, 110, 233, 468},
	phFileChanged,
	{
		Cancel, visible, silent;
		Cancel, visible, silent;
		Cancel, visible, silent;
		Cancel, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phPurgeOld,
#if qNames
	"phPurgeOld",
#endif
	purgeable) {
	{100, 110, 233, 468},
	phPurgeOld,
	{
		Cancel, visible, silent;
		Cancel, visible, silent;
		Cancel, visible, silent;
		Cancel, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phReopenDoc,
#if qNames
	"phReopenDoc",
#endif
	purgeable) {
	{100, 110, 185, 468},
	phReopenDoc,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionMainScreen
};

resource 'ALRT' (phOfferReadOnly,
#if qNames
	"phOfferReadOnly",
#endif
	purgeable) {
	{100, 110, 208, 468},
	phOfferReadOnly,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionMainScreen
};

resource 'ALRT' (phTooManyChars,
#if qNames
	"phTooManyChars",
#endif
	purgeable) {
	{100, 110, 185, 468},
	phTooManyChars,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};

resource 'ALRT' (phStylesTooBig,
#if qNames
	"phStylesTooBig",
#endif
	purgeable) {
	{100, 110, 201, 468},
	phStylesTooBig,
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};

//	For an explanation of how the error tables work please refer to
//	the UErrorMgr.cp source file.

// This list maps error codes to failure reasons.

resource 'errs' (errReasonID,
#if qNames
	"errReasonID",
#endif
	purgeable) {
	{	whichList, 0, errReasonIDStrings;
		
		// Some of the following are out of order on purpose so that specific errors
		// can be specified while still having a default range of errors covering
		// those errors not specifically called out.
		-27, -27, 10;		// printer I/O error [OR edition mgr error (IM VI 4-69)]
		-34, -33, 2;		// disk full
		-35, -35, 89;		// the volume cannot be found
		-39, -39, 31;		// end of file error
		-42, -42, 21;		// too many files are open
		-43, -43, 32;		// file not found
		-45, -45, 6;		// file locked
		-46, -44, 3;		// volumes locked
		-47, -47, 7;		// files are open
		-49, -49, 7;		// can't have > 1 writer to a file
		-50, -50, 91;		// parameter error
		-51, -51, 27;		// Bad file reference number
		-53, -53, 20;		// volume offline
		-54, -54, 6;		// can't open locked file for writing
		-61, -61, 29;		// write permissions error
		-84, -35, 5;		// other File Manager / disk errors - Default Range
		-95, -91, 17;		// AppleTalk operation errors
		-98, -97, 18;		// AppleTalk could not be opened
		-102, -102, 90;		// no object of that type in scrap
		-117, -108, 4;		// Memory Manager errors
		-124, -124, 14; 	// Lost volume on AppleTalk
		-127, -120, 5;		// HFS errors - Default Range
		-128, -128, 65;		// User cancelled.
		-192, -192, 22;		// Resource not found
		-450, -450, 26;		// Edition Manager not initialized
		-453, -451, 28;		// Unknown edition format, or format not available
		-454, -454, 30;		// not a publisher
		-1700, -1700, 34,	// can't coerce data to requested type
		-1701, -1701, 35,	// a descriptor was not found
		-1702, -1702, 36,	// data was corrupt and could not be read
		-1703, -1703, 37,	// data was the wrong type
		-1704, -1704, 38,	// of an invalid descriptor record
		-1705, -1705, 39,	// of a bad list item
		-1706, -1706, 40,	// a newer version of the AppleEvent Manager is required
		-1707, -1707, 41,	// the event is not an AppleEvent
		-1708, -1708, 42,	// the event was not handled
		-1709, -1709, 43,	// the reply is not valid
		-1710, -1710, 44,	// the sending mode is unknown
		-1711, -1711, 45,	// the wait was canceled
		-1712, -1712, 46,	// the event timed out
		-1713, -1713, 47,	// no user interaction is allowed
		-1714, -1714, 48,	// the keyword is not valid for a special function
		-1715, -1715, 49,	// a required parameter is missing
		-1716, -1716, 50,	// of an unknown AppleEvent address type
		-1717, -1717, 51,	// a handler was not found
		-1718, -1718, 52,	// the reply has not yet arrived
		-1719, -1719, 53,	// of an invalid list index
		-1720, -1720, 54,	// an invalid range was specified
		-1721, -1721, 55,	// the number of arguments is wrong
		-1723, -1723, 56,	// an object accessor was not found
		-1725, -1725, 57,	// there is no such logical operator
		-1726, -1726, 58,	// of a bad test key
		-1727, -1727, 59,	// there is not an object specifier
		-1728, -1728, 60,	// that object can not be found
		-1729, -1729, 61,	// the number of objects is negative
		-1730, -1730, 62,	// the container is an empty list
		-1731, -1731, 63,	// of an unknown object type
		-1732, -1732, 64,	// recording is already on
		-1906, -1906, 87,   // there are open attachments
		-1901, -1901, 88,   // there is a copy in progress
		-4100, -4096, 10;	// printer comm error
		-4101, -4101, 9;	// printer not found (PAP)
		-5000, -5000, 15;	// AFP access error
		-5006, -5006, 7;	// AFP: deny conflict
		-5029, -5001, 14;	// AFP errors - Default Range
		-20000, -20000, 8;	// MacApp error code: could not spool print file
		-20001, -20001, 11; // MacApp error code: could not revert because file was deleted
		-20003, -20003, 12; // MacApp error code: user tried to Save As or Save a Copy in an opened document
		-20004, -20004, 13; // MacApp error code: user did Revert but file type changed
		-20005, -20005, 16; // MacApp error code: printer driver file not found
		-20006, -20006, 19; // Can't open file of given type
		-20007, -20007, 23; // Function is not yet implemented
		-20008, -20008, 24; // Class is missing. (Usually due to dead stripping)
		-20009, -20009, 25; // Signature has no corresponding class
		-20010, -20010, 33; // printer driver is in use by another application
		-20012, -20012, 85; // there are no pages to print
		-20013, -20013, 86; // no more characters can be added
		-21001, -21001, 66;	// can't clone an object
		-21002, -21002, 67;	// can't close an object
		-21003, -21003, 68;	// can't count objects
		-21004, -21004, 69;	// can't create an object
		-21005, -21005, 70;	// can't delete an object
		-21006, -21006, 71;	// can't determine if object exists
		-21007, -21007, 72;	// can't get info on class
		-21008, -21008, 73;	// can't get info on event
		-21009, -21009, 74;	// can't get data for an object
		-21010, -21010, 75;	// can't get size of an object's data
		-21011, -21011, 76;	// can't move an object
		-21012, -21012, 77;	// can't open an object
		-21013, -21013, 78;	// can't print an object
		-21014, -21014, 79;	// can't save an object
		-21015, -21015, 80;	// can't set an object data
		-21016, -21016, 81;	// can't revert an object
		-21017, -21017, 83;	// can't set a read-only property
		-21018, -21018, 84; // no such property

		minErr, maxErr, 1
	}
};

// Error explanations.

resource 'STR#' (errReasonIDStrings,
#if qNames
	"error explanations",
#endif
	purgeable) {
	{	/* [1] */	"of a program error";								// catch-all
		/* [2] */	"the disk is full";
		/* [3] */	"the disk is locked";
		/* [4] */	"there is not enough memory";
		/* [5] */	"of a disk error";
		/* [6] */	"the file is locked";
		/* [7] */	"the file is already in use or was left open";
		/* [8] */	"of a problem writing the print file";
		/* [9] */	"the printer could not be found";
		/* [10] */	"of a problem communicating with the printer";
		/* [11] */	"its disk copy was deleted";
		// used when user picks Revert and file was deleted

		/* [12] */	"that document is currently open";
		// used when user does Save As or Save a Copy In and specifies a document that is already open

		/* [13] */	"its disk copy was changed";
		// used when user does a Revert and the file type has been changed

		/* [14] */	"of a problem with the file server";				// catch-all AFP error
		/* [15] */	"you do not have the necessary access privileges";	// AFP: access denied
		/* [16] */	"the selected printing resource could not be found";// Couldn't open print driver
		/* [17] */	"of a problem with AppleTalk";						// AppleTalk operation errors
		/* [18] */	"AppleTalk is not connected";						// AppleTalk open errors
		/* [19] */	"it is not the right kind of document";				// File is the wrong type
		/* [20] */	"the disk is not available";
		// Volume is offline: asynch I/O or command-.

		/* [21] */	"there are too many files open";					// Resource not found
		/* [22] */	"the required resources were not found";
		/* [23] */	"this feature is not implemented";
		/* [24] */	"'^3', an internal component, is missing";			// Missing class
		/* [25] */	"an internal component is missing";					// Missing signature

		// Edition Manager error reasons
		/* [26] */	"the Edition Manager isn't initialized";
		/* [27] */	"of a problem with the reference number";			// File Manager error
		/* [28] */	"the specified format is not in the edition";
		/* [29] */	"of a write permissions error";						// File Manager error
		/* [30] */	"there is a problem specifying the edition";

		// Miscellaneous
		/* [31] */	"end of file";
		/* [32] */	"file not found";
		/* [33] */	"the printer file is used by another application or the Chooser";// printer driver is open

		// AppleEvent Manager error reasons
		/* [34] */	"can't coerce data to requested type";
		/* [35] */	"a descriptor was not found";
		/* [36] */	"data was corrupt and could not be read";
		/* [37] */	"data was the wrong type";
		/* [38] */	"of an invalid descriptor record";
		/* [39] */	"of a bad list item";
		/* [40] */	"a newer version of the AppleEvent Manager is required";
		/* [41] */	"the event is not an AppleEvent";
		/* [42] */	"the event was not handled";
		/* [43] */	"the reply is not valid";
		/* [44] */	"the sending mode is unknown";
		/* [45] */	"the wait was canceled";
		/* [46] */	"the event timed out";
		/* [47] */	"no user interaction is allowed";
		/* [48] */	"the keyword is not valid for a special function";
		/* [49] */	"a required parameter is missing";
		/* [50] */	"of an unknown AppleEvent address type";
		/* [51] */	"a handler was not found";
		/* [52] */	"the reply has not yet arrived";
		/* [53] */	"of an invalid list index";
		/* [54] */	"an invalid range was specified";
		/* [55] */	"the number of arguments is wrong";
		/* [56] */	"an object accessor was not found";
		/* [57] */	"there is no such logical operator";
		/* [58] */	"of a bad test key";
		/* [59] */	"there is not an object specifier";
		/* [60] */	"that object can not be found";
		/* [61] */	"the number of objects is negative";
		/* [62] */	"the container is an empty list";
		/* [63] */	"of an unknown object type";
		/* [64] */	"recording is already on";
		/* [65] */	"the user cancelled";
		/* [66] */	"cannot duplicate this object";
		/* [67] */	"cannot close this object";
		/* [68] */	"cannot count these elements";
		/* [69] */	"cannot create an object";
		/* [70] */	"cannot delete this object";
		/* [71] */	"cannot tell if this object exists";
		/* [72] */	"cannot get class info for this object";
		/* [73] */	"cannot get event info";
		/* [74] */	"cannot get data for this object";
		/* [75] */	"cannot get the size of this object's data";
		/* [76] */	"cannot move this object";
		/* [77] */	"cannot open this object";
		/* [78] */	"cannot print this object";
		/* [79] */	"cannot save this object";
		/* [80] */	"cannot set the data for this object";
		/* [81] */	"cannot revert this object";
		/* [82] */	"an error [^ENUM] occurred";
		/* [83] */	"cannot set a read-only property";
		/* [84] */	"no such property";

		/* [85] */	"there are no pages to print";
		/* [86] */	"no more characters can be added";
		/* [87] */  "the letter has an open enclosure";
		/* [88] */  "there is a copy in progress";
		/* [89] */	"the disk does not seem to be available any more";
		/* [90] */	"there is no object of that type in the scrap";
		/* [91] */	"a bad parameter was passed"
	}
};


// This list maps error codes to recovery strings.

resource 'errs' (errRecoveryID,
#if qNames
	"errRecoveryID",
#endif
	purgeable) {
	{	whichList, 0, errRecoveryIDStrings;
		-42, -42, 4;
		-45, -45, 1;
		-4101, -4101, 2;
		-20003, -20003, 3;
		-20005, -20005, 2;
		-20008, -20008, 5;
		-20009, -20009, 5;
		-20010, -20010, 6;

		// Edition Manager error recovery
		-450, -450, 7;		// Edition Manager not initialized
	}
};

// Recovery strings - gives the user advice on how to recover from the error.

resource 'STR#' (errRecoveryIDStrings,
#if qNames
	"recovery strings",
#endif
	purgeable) {
	{	/* [1] */	"Use the 'Get Info' command in the Finder to unlock the file.";
		/* [2] */	"Use 'Chooser' to select a printer and try again.";
		// this is used if (1) the chosen LaserWriter can't be found or
		// (2) the chosen printer driver file can't be found

		/* [3] */	"Save using a different name, or close the document and try again.";
		/* [4] */	"Try closing some windows and try again.";
#if qDebug
		/* [5] */	"Call Apple for a secret decoder ring.";
#else
		/* [5] */	"Please contact the developer.";
#endif
		/* [6] */	"Try closing the Chooser.";
		// Edition Manager error recovery
#if qDebug
		/* [7] */	"Call InitUEditionDocument()!"
#else
		/* [7] */	"Please contact the developer."
#endif
	}
};

// Strings describing the operation; these are used in error alerts.
resource 'STR#' (errOperationsIDStrings,
#if qNames
	"strings describing the operation",
#endif
	purgeable) {
	{	/* [1] */	"start the application";
		/* [2] */	"save '^3'";
		/* [3] */	"revert to the previous version of '^3'";
		/* [4] */	"print '^3'";
		/* [5] */	"create a new document";
		/* [6] */	"open '^3'";
		/* [7] */	"save as '^3'";
		/* [8] */	"save a copy in '^3'";
		/* [9] */	"display the contents of the window";
		/* [10] */	"make the clipboard available inside the application";
		/* [11] */	"make the clipboard available outside the application"
	}
};
