//----------------------------------------------------------------------------------------
// UAdorners.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif


//----------------------------------------------------------------------------------------
// Definition for global variables declared in interface:

// Global, pre-allocated, shared adorners
TDimAdorner TAdorner::fgDimAdorner;
TDrawAdorner TAdorner::fgDrawAdorner;
TEraseAdorner TAdorner::fgEraseAdorner;
THiliteAdorner TAdorner::fgHiliteAdorner;
TPrintAdorner TAdorner::fgPrintAdorner;
TResizeIconAdorner TAdorner::fgResizeIconAdorner;
TSelectionAdorner TAdorner::fgSelectionAdorner;

//========================================================================================
// CLASS CAdornerIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CAdornerIterator::CAdornerIterator:
//----------------------------------------------------------------------------------------
CAdornerIterator::CAdornerIterator(const TView* theView,
								   ArrayIndex_AC itsLowBound,
								   ArrayIndex_AC itsHighBound,
								   bool itsForward) :
	CArrayIterator_AC(theView ? theView->GetAdorners() : NULL, itsLowBound, itsHighBound, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CAdornerIterator::CAdornerIterator:
//----------------------------------------------------------------------------------------
CAdornerIterator::CAdornerIterator(const TView* theView,
								   bool itsForward) :
	CArrayIterator_AC(theView ? theView->GetAdorners() : NULL, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CAdornerIterator::CAdornerIterator:
//----------------------------------------------------------------------------------------
CAdornerIterator::CAdornerIterator(const TView* theView) :
	CArrayIterator_AC(theView ? theView->GetAdorners() : NULL, kIterateForward_AC)
{
	First();
}

//----------------------------------------------------------------------------------------
// CAdornerIterator::CAdornerIterator:
//----------------------------------------------------------------------------------------
CAdornerIterator::CAdornerIterator(const TAdornerList* theAdornerList) :
	CArrayIterator_AC(theAdornerList, kIterateForward_AC)
{
	First();
}

//----------------------------------------------------------------------------------------
// CAdornerIterator::operator TAdorner*:
//----------------------------------------------------------------------------------------
TAdorner* CAdornerIterator::First()
{
	return fCurrent = CArrayIterator_AC::First() ? dynamic_cast_AC(const TAdornerList*,fDynamicArray)->AdornerAt(fCurrentIndex) : NULL;
}

//----------------------------------------------------------------------------------------
// CAdornerIterator::Next:
//----------------------------------------------------------------------------------------
TAdorner* CAdornerIterator::Next()
{
	return fCurrent = CArrayIterator_AC::Next() ? dynamic_cast_AC(const TAdornerList*,fDynamicArray)->AdornerAt(fCurrentIndex) : NULL;
}


//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// InitUAdorners:
//----------------------------------------------------------------------------------------
void InitUAdorners()
{
	// Register the system-supplied adorners 
	REGISTER_CLASS_AC(TAdornerList);

	REGISTER_SIGNATURE_AC(TCompatibilityAdorner, kCompatibilityAdorner);
	REGISTER_SIGNATURE_AC(TDimAdorner, kDimAdorner);
	REGISTER_SIGNATURE_AC(TDrawAdorner, kDrawAdorner);
	REGISTER_SIGNATURE_AC(TEraseAdorner, kEraseAdorner);
	REGISTER_SIGNATURE_AC(TFrameAdorner, kFrameAdorner);
	REGISTER_SIGNATURE_AC(THiliteAdorner, kHiliteAdorner);
	REGISTER_SIGNATURE_AC(TLineBottomAdorner, kLineBottomAdorner);
	REGISTER_SIGNATURE_AC(TLineLeftAdorner, kLineLeftAdorner);
	REGISTER_SIGNATURE_AC(TLineRightAdorner, kLineRightAdorner);
	REGISTER_SIGNATURE_AC(TLineTopAdorner, kLineTopAdorner);
	REGISTER_SIGNATURE_AC(TOvalAdorner, kOvalAdorner);
	REGISTER_SIGNATURE_AC(TPrintAdorner, kPrintAdorner);
	REGISTER_SIGNATURE_AC(TResizeIconAdorner, kResizeIconAdorner);
	REGISTER_SIGNATURE_AC(TRRectAdorner, kRRectAdorner);
	REGISTER_SIGNATURE_AC(TSelectionAdorner, kSelectionAdorner);
	REGISTER_SIGNATURE_AC(TShadowedFrameAdorner, kShadowedFrameAdorner);
}

//========================================================================================
// CLASS TAdorner
//========================================================================================
MA_DEFINE_CLASS(TAdorner);

//----------------------------------------------------------------------------------------
// NewStdAdorner:
//----------------------------------------------------------------------------------------
TAdorner* TAdorner::NewStdAdorner(IDType signature,
						const char* className,
						IDType itsID,
						bool freeOnDeletion)
{
	MStreamable_AC* object = ::NewObjectBySignature_AC(signature, className);

	TAdorner * anAdorner = dynamic_cast<TAdorner*>(object);
	if(anAdorner) 
		anAdorner->Set(itsID, freeOnDeletion);
	else 
		delete object;
	
	return anAdorner;
}

//----------------------------------------------------------------------------------------
// TAdorner constructor 
//----------------------------------------------------------------------------------------
TAdorner::TAdorner(IDType itsID,
						bool freeOnDeletion, bool isOpaque) :
	fIdentifier(itsID),
	fFreeOnDeletion(freeOnDeletion),
	fIsOpaque(isOpaque)
{
}
 
//----------------------------------------------------------------------------------------
// TAdorner destructor
//----------------------------------------------------------------------------------------
TAdorner::~TAdorner()
{
}

//----------------------------------------------------------------------------------------
// TAdorner constructor 
//----------------------------------------------------------------------------------------
void TAdorner::Set(IDType itsID,
						bool freeOnDeletion)
{
	fIdentifier = itsID;
	fFreeOnDeletion = freeOnDeletion;
}

//----------------------------------------------------------------------------------------
// TAdorner::AddedToView:
//----------------------------------------------------------------------------------------
void TAdorner::AddedToView(TView*				/* itsView */ )
{
}

//----------------------------------------------------------------------------------------
// TAdorner::RemovedFromView:
//----------------------------------------------------------------------------------------
void TAdorner::RemovedFromView(TView*			/* itsView */ )
{
}

//----------------------------------------------------------------------------------------
// TAdorner::ReadFrom:
//----------------------------------------------------------------------------------------
void TAdorner::ReadFrom(CStream_AC* aStream)		// override 
{
	MStreamable::ReadFrom(aStream);

	fIdentifier = aStream->ReadIDType();
	fFreeOnDeletion = aStream->ReadBoolean();
}

//----------------------------------------------------------------------------------------
// TAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TAdorner::GetStandardSignature() const// override 
{
	return fIdentifier;
}

//----------------------------------------------------------------------------------------
// TAdorner::WriteTo:
//----------------------------------------------------------------------------------------
void TAdorner::WriteTo(CStream_AC* aStream) const		// override 
{
	MStreamable::WriteTo(aStream);

	aStream->WriteIDType(fIdentifier);
	aStream->WriteBoolean(fFreeOnDeletion);
}

//----------------------------------------------------------------------------------------
// TAdorner::Draw:
//----------------------------------------------------------------------------------------
void TAdorner::Draw(TView*						/* itsView */ ,
					const CViewRect&				/* area */ )
{
}

//----------------------------------------------------------------------------------------
// TAdorner::DrawLine:
//----------------------------------------------------------------------------------------
void TAdorner::DrawLine(VHSelect vhs,
						short lh,
						short lv,
						short lto)
{
	::MoveToPt(CPoint_AC(lh, lv));
	if (vhs == hSel)
		::Line(lto - lh, 0);
	else
		::Line(0, lto - lv);
}

//----------------------------------------------------------------------------------------
// TAdorner::DoHighlightSelection:
//----------------------------------------------------------------------------------------
void TAdorner::DoHighlightSelection(TView*		/* itsView */ ,
									const CViewRect&/* area */  ,
									HLState		/* fromHL */,
									HLState		/* toHL */)
{
}

//----------------------------------------------------------------------------------------
// TAdorner::InvalidateAdorner:
//----------------------------------------------------------------------------------------
void TAdorner::InvalidateAdorner(TView* itsView)
{
	if (itsView)
		itsView->ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TAdorner::ViewChangedFrame:
//----------------------------------------------------------------------------------------
void TAdorner::ViewChangedFrame(TView*			/* itsView */ ,
								const CViewRect&	/* oldFrame */ ,
								const CViewRect&	/* newFrame */ ,
								bool			/* invalidate */)
{
}

//----------------------------------------------------------------------------------------
// TAdorner::DoesAdorn:
//----------------------------------------------------------------------------------------
bool TAdorner::DoesAdorn(const TView*				/* itsView */ ) const
{
	return true;
}

//========================================================================================
// CLASS TCompatibilityAdorner
//========================================================================================
MA_DEFINE_CLASS(TCompatibilityAdorner);

//----------------------------------------------------------------------------------------
// TCompatibilityAdorner destructor
//----------------------------------------------------------------------------------------
TCompatibilityAdorner::~TCompatibilityAdorner()
{
}

//----------------------------------------------------------------------------------------
// TCompatibilityAdorner::TCompatibilityAdorner:
//----------------------------------------------------------------------------------------
TCompatibilityAdorner::TCompatibilityAdorner(CntlAdornment itsAdornment) :
	TAdorner(kCompatibilityAdorner, kFreeOnDeletion),

	fAdornment(itsAdornment)
{
}

//----------------------------------------------------------------------------------------
// TCompatibilityAdorner::ReadFrom:
//----------------------------------------------------------------------------------------
void TCompatibilityAdorner::ReadFrom(CStream_AC* aStream)// override 
{
	TAdorner::ReadFrom(aStream);

	aStream->ReadBytes(&fAdornment, sizeof(CntlAdornment));
}

//----------------------------------------------------------------------------------------
// TCompatibilityAdorner::WriteTo:
//----------------------------------------------------------------------------------------
void TCompatibilityAdorner::WriteTo(CStream_AC* aStream) const// override 
{
	TAdorner::WriteTo(aStream);

	CntlAdornment itsAdornment = fAdornment;
	aStream->WriteBytes(&itsAdornment, sizeof(CntlAdornment));
}

//----------------------------------------------------------------------------------------
// TCompatibilityAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TCompatibilityAdorner::GetStandardSignature() const// override 
{
	return kCompatibilityAdorner;
}

//----------------------------------------------------------------------------------------
// TCompatibilityAdorner::Draw:
//----------------------------------------------------------------------------------------
void TCompatibilityAdorner::Draw(TView* itsView,
								 const CViewRect& area)
{
	if (fAdornment)
	{
		CViewRect itsAdornExtent(itsView->GetAdornExtent());
		if (!(area & itsAdornExtent).Empty())	// is there something to draw?
		{
			CRect_AC qdArea(itsView->ViewToQD(itsAdornExtent));// convert to QD coord's 

			if (macroIn(fAdornment, adnShadow))
			{
				::OpenRgn();
				::ShowPen();						// so drawing occurs 
			}


			if ((fAdornment & kFrame) == kFrame)// adnFrame
				::FrameRect(&qdArea);
			else
			{
				if (macroIn(fAdornment, adnLineTop))
					DrawLine(hSel, qdArea.left, qdArea.top, qdArea.right);

				if (macroIn(fAdornment, adnLineLeft))
					DrawLine(vSel, qdArea.left, qdArea.top, qdArea.bottom);

				if (macroIn(fAdornment, adnLineBottom))
				{
					PenState pnState;
					::GetPenState(&pnState);
					DrawLine(hSel, qdArea.left, qdArea.bottom - pnState.pnSize.v, qdArea.right);// account for pen height 
				}

				if (macroIn(fAdornment, adnLineRight))
				{
					PenState pnState;
					::GetPenState(&pnState);
					DrawLine(vSel, qdArea.right - pnState.pnSize.h, qdArea.top, qdArea.bottom);// account for pen width 
				}
			}

			if (macroIn(fAdornment, adnOval))
				::FrameOval(qdArea);

			if (macroIn(fAdornment, adnRRect))
				::FrameRoundRect(qdArea, 16, 16);

			if (macroIn(fAdornment, adnShadow))
			{
				PenState pnState;

				CTempRegion_AC tempRgn;
				CTempRegion_AC tempRgn2;

				::HidePen();
				::CloseRgn(tempRgn);

				::CopyRgn(tempRgn, tempRgn2);
				::GetPenState(&pnState);
				::OffsetRgn(tempRgn2, pnState.pnSize.h, pnState.pnSize.v);
				::DiffRgn(tempRgn2, tempRgn, tempRgn2);
				::PaintRgn(tempRgn2);
			}
		}
	}
}

//========================================================================================
// CLASS TDimAdorner
//========================================================================================
MA_DEFINE_CLASS(TDimAdorner);

//----------------------------------------------------------------------------------------
// TDimAdorner destructor
//----------------------------------------------------------------------------------------
TDimAdorner::~TDimAdorner()
{
}

//----------------------------------------------------------------------------------------
// TDimAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TDimAdorner::GetStandardSignature() const		// override 
{
	return kDimAdorner;
}

//----------------------------------------------------------------------------------------
// TDimAdorner::Draw:
//----------------------------------------------------------------------------------------
void TDimAdorner::Draw(TView* itsView,
					   const CViewRect&)
{
	itsView->Dim();
}

//========================================================================================
// CLASS TDrawAdorner
//========================================================================================
MA_DEFINE_CLASS(TDrawAdorner);

//----------------------------------------------------------------------------------------
// TDrawAdorner destructor
//----------------------------------------------------------------------------------------
TDrawAdorner::~TDrawAdorner()
{
}

//----------------------------------------------------------------------------------------
// TDrawAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TDrawAdorner::GetStandardSignature() const		// override 
{
	return kDrawAdorner;
}

//----------------------------------------------------------------------------------------
// TDrawAdorner::Draw:
//----------------------------------------------------------------------------------------
void TDrawAdorner::Draw(TView* itsView,
						const CViewRect& area)
{
	::PenNormal();
	itsView->Draw(area);
}

//========================================================================================
// CLASS TEraseAdorner
//========================================================================================
MA_DEFINE_CLASS(TEraseAdorner);

//----------------------------------------------------------------------------------------
// TEraseAdorner destructor
//----------------------------------------------------------------------------------------
TEraseAdorner::~TEraseAdorner()
{
}

//----------------------------------------------------------------------------------------
// TEraseAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TEraseAdorner::GetStandardSignature() const	// override 
{
	return kEraseAdorner;
}

//----------------------------------------------------------------------------------------
// TEraseAdorner::Draw:
//----------------------------------------------------------------------------------------
void TEraseAdorner::Draw(TView* itsView,
						 const CViewRect& area)
{
	::EraseRect(&itsView->ViewToQD(area));
}

//========================================================================================
// CLASS THiliteAdorner:
//========================================================================================
MA_DEFINE_CLASS(THiliteAdorner);

//----------------------------------------------------------------------------------------
// THiliteAdorner destructor
//----------------------------------------------------------------------------------------
THiliteAdorner::~THiliteAdorner()
{
}

//----------------------------------------------------------------------------------------
// THiliteAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType THiliteAdorner::GetStandardSignature() const	// override 
{
	return kHiliteAdorner;
}

//----------------------------------------------------------------------------------------
// THiliteAdorner::Draw:
//----------------------------------------------------------------------------------------
void THiliteAdorner::Draw(TView* itsView,
						  const CViewRect&)
{
	itsView->Hilite();
}

//========================================================================================
// CLASS TResizeIconAdorner
//========================================================================================
MA_DEFINE_CLASS(TResizeIconAdorner);

//----------------------------------------------------------------------------------------
// TResizeIconAdorner destructor
//----------------------------------------------------------------------------------------
TResizeIconAdorner::~TResizeIconAdorner()
{
}

//----------------------------------------------------------------------------------------
// TResizeIconAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TResizeIconAdorner::GetStandardSignature() const// override 
{
	return kResizeIconAdorner;
}

//----------------------------------------------------------------------------------------
// TResizeIconAdorner::DoHighlightSelection:
//----------------------------------------------------------------------------------------
void TResizeIconAdorner::DoHighlightSelection(TView* itsView,
											  const CViewRect&/* area */  ,
											  HLState/* fromHL */ ,
											  HLState/* toHL */ )
{
	TWindow* window = dynamic_cast_AC(TWindow*,itsView);
	window->DrawResizeIcon();
}


//========================================================================================
// CLASS TSelectionAdorner
//========================================================================================
MA_DEFINE_CLASS(TSelectionAdorner);

//----------------------------------------------------------------------------------------
// TSelectionAdorner destructor
//----------------------------------------------------------------------------------------
TSelectionAdorner::~TSelectionAdorner()
{
}

//----------------------------------------------------------------------------------------
// TSelectionAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TSelectionAdorner::GetStandardSignature() const// override 
{
	return kSelectionAdorner;
}

//----------------------------------------------------------------------------------------
// TSelectionAdorner::DoHighlightSelection:
//----------------------------------------------------------------------------------------
void TSelectionAdorner::DoHighlightSelection(TView* itsView,
											 const CViewRect&/* area */  ,
											 HLState fromHL,
											 HLState toHL)
{
	::PenNormal();
	itsView->DoHighlightSelection(fromHL, toHL);
}


//========================================================================================
// CLASS TPrintAdorner
//========================================================================================
MA_DEFINE_CLASS(TPrintAdorner);

//----------------------------------------------------------------------------------------
// TPrintAdorner destructor
//----------------------------------------------------------------------------------------
TPrintAdorner::~TPrintAdorner()
{
}

//----------------------------------------------------------------------------------------
// TPrintAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TPrintAdorner::GetStandardSignature() const	// override 
{
	return kPrintAdorner;
}

//----------------------------------------------------------------------------------------
// TPrintAdorner::DoHighlightSelection:
//----------------------------------------------------------------------------------------
void TPrintAdorner::DoHighlightSelection(TView* itsView,
										 const CViewRect& area,
										 HLState fromHL,
										 HLState/* toHL */ )
{
	// only do this in the "HandleDraw" case; in which case "fromHL" is hlOff 
	if (fromHL == hlOff)
		itsView->DoDrawPrintFeedback(area);
}


//========================================================================================
// CLASS TFrameAdorner
//========================================================================================
MA_DEFINE_CLASS(TFrameAdorner);

//----------------------------------------------------------------------------------------
// TFrameAdorner destructor
//----------------------------------------------------------------------------------------
TFrameAdorner::~TFrameAdorner()
{
}

//----------------------------------------------------------------------------------------
// TFrameAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TFrameAdorner::GetStandardSignature() const	// override 
{
	return kFrameAdorner;
}

//----------------------------------------------------------------------------------------
// TFrameAdorner::Draw:
//----------------------------------------------------------------------------------------
void TFrameAdorner::Draw(TView* itsView,
						 const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
		::FrameRect(&itsView->ViewToQD(itsAdornExtent));
}


//========================================================================================
// CLASS TShadowedFrameAdorner
//========================================================================================
MA_DEFINE_CLASS(TShadowedFrameAdorner);

//----------------------------------------------------------------------------------------
// TShadowedFrameAdorner destructor
//----------------------------------------------------------------------------------------
TShadowedFrameAdorner::~TShadowedFrameAdorner()
{
}

//----------------------------------------------------------------------------------------
// TShadowedFrameAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TShadowedFrameAdorner::GetStandardSignature() const// override 
{
	return kShadowedFrameAdorner;
}

//----------------------------------------------------------------------------------------
// TShadowedFrameAdorner::Draw:
//----------------------------------------------------------------------------------------
void TShadowedFrameAdorner::Draw(TView* itsView,
								 const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
	{
		CRect_AC qdArea(itsView->ViewToQD(itsAdornExtent));// convert to QD coord's 
		--qdArea[botRight];

		::PenSize(1, 1);
		::FrameRect(qdArea);
		::MoveTo(qdArea.left + 1, qdArea.bottom);
		::LineToPt(qdArea[botRight]);
		::LineTo(qdArea.right, qdArea.top + 1);
	}
}


//========================================================================================
// CLASS TLineBottomAdorner
//========================================================================================
MA_DEFINE_CLASS(TLineBottomAdorner);

//----------------------------------------------------------------------------------------
// TLineBottomAdorner destructor
//----------------------------------------------------------------------------------------
TLineBottomAdorner::~TLineBottomAdorner()
{
}

//----------------------------------------------------------------------------------------
// TLineBottomAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TLineBottomAdorner::GetStandardSignature() const// override 
{
	return kLineBottomAdorner;
}

//----------------------------------------------------------------------------------------
// TLineBottomAdorner::Draw:
//----------------------------------------------------------------------------------------
void TLineBottomAdorner::Draw(TView* itsView,
							  const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
	{
		PenState pnState;
		::GetPenState(&pnState);
		CRect_AC qdArea = itsView->ViewToQD(itsAdornExtent);// convert to QD coord's 
		DrawLine(hSel, qdArea.left, qdArea.bottom - pnState.pnSize.v, qdArea.right);// account for pen height 
	}
}


//========================================================================================
// CLASS TLineLeftAdorner
//========================================================================================
MA_DEFINE_CLASS(TLineLeftAdorner);

//----------------------------------------------------------------------------------------
// TLineLeftAdorner destructor
//----------------------------------------------------------------------------------------
TLineLeftAdorner::~TLineLeftAdorner()
{
}

//----------------------------------------------------------------------------------------
// TLineLeftAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TLineLeftAdorner::GetStandardSignature() const	// override 
{
	return kLineLeftAdorner;
}

//----------------------------------------------------------------------------------------
// TLineLeftAdorner::Draw:
//----------------------------------------------------------------------------------------
void TLineLeftAdorner::Draw(TView* itsView,
							const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
	{
		CRect_AC qdArea(itsView->ViewToQD(itsAdornExtent));// convert to QD coord's 
		DrawLine(vSel, qdArea.left, qdArea.top, qdArea.bottom);
	}
}


//========================================================================================
// CLASS TLineRightAdorner
//========================================================================================
MA_DEFINE_CLASS(TLineRightAdorner);

//----------------------------------------------------------------------------------------
// TLineRightAdorner destructor
//----------------------------------------------------------------------------------------
TLineRightAdorner::~TLineRightAdorner()
{
}

//----------------------------------------------------------------------------------------
// TLineRightAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TLineRightAdorner::GetStandardSignature() const// override 
{
	return kLineRightAdorner;
}

//----------------------------------------------------------------------------------------
// TLineRightAdorner::Draw:
//----------------------------------------------------------------------------------------
void TLineRightAdorner::Draw(TView* itsView,
							 const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
	{
		PenState pnState;
		::GetPenState(&pnState);
		CRect_AC qdArea(itsView->ViewToQD(itsAdornExtent));// convert to QD coord's 
		DrawLine(vSel, qdArea.right - pnState.pnSize.h, qdArea.top, qdArea.bottom);// account for pen width 
	}
}


//========================================================================================
// CLASS TLineTopAdorner
//========================================================================================
MA_DEFINE_CLASS(TLineTopAdorner);

//----------------------------------------------------------------------------------------
// TLineTopAdorner destructor
//----------------------------------------------------------------------------------------
TLineTopAdorner::~TLineTopAdorner()
{
}

//----------------------------------------------------------------------------------------
// TLineTopAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TLineTopAdorner::GetStandardSignature() const	// override 
{
	return kLineTopAdorner;
}

//----------------------------------------------------------------------------------------
// TLineTopAdorner::Draw:
//----------------------------------------------------------------------------------------
void TLineTopAdorner::Draw(TView* itsView,
						   const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
	{
		CRect_AC qdArea(itsView->ViewToQD(itsAdornExtent));// convert to QD coord's 
		DrawLine(hSel, qdArea.left, qdArea.top, qdArea.right);
	}
}


//========================================================================================
// CLASS TOvalAdorner
//========================================================================================
MA_DEFINE_CLASS(TOvalAdorner);

//----------------------------------------------------------------------------------------
// TOvalAdorner destructor
//----------------------------------------------------------------------------------------
TOvalAdorner::~TOvalAdorner()
{
}

//----------------------------------------------------------------------------------------
// TOvalAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TOvalAdorner::GetStandardSignature() const	// override 
{
	return kOvalAdorner;
}

//----------------------------------------------------------------------------------------
// TOvalAdorner::Draw:
//----------------------------------------------------------------------------------------
void TOvalAdorner::Draw(TView* itsView,
						const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
		::FrameOval(&itsView->ViewToQD(itsAdornExtent));
}


//========================================================================================
// CLASS TRRectAdorner
//========================================================================================
MA_DEFINE_CLASS(TRRectAdorner);

//----------------------------------------------------------------------------------------
// TRRectAdorner destructor
//----------------------------------------------------------------------------------------
TRRectAdorner::~TRRectAdorner()
{
}

//----------------------------------------------------------------------------------------
// TRRectAdorner::GetStandardSignature:
//----------------------------------------------------------------------------------------
IDType TRRectAdorner::GetStandardSignature() const	// override 
{
	return kRRectAdorner;
}

//----------------------------------------------------------------------------------------
// TRRectAdorner::Draw:
//----------------------------------------------------------------------------------------
void TRRectAdorner::Draw(TView* itsView,
						 const CViewRect& area)
{
	CViewRect itsAdornExtent(itsView->GetAdornExtent());

	if (!(area & itsAdornExtent).Empty())		// is there something to draw?
		::FrameRoundRect(&itsView->ViewToQD(itsAdornExtent), 16, 16);
}



//========================================================================================
// CLASS TAdornerList
//========================================================================================
MA_DEFINE_CLASS(TAdornerList);

//----------------------------------------------------------------------------------------
// TAdornerList destructor
//----------------------------------------------------------------------------------------
TAdornerList::~TAdornerList()
{
}

//----------------------------------------------------------------------------------------
// TAdornerList::ReadFrom:
//----------------------------------------------------------------------------------------
void TAdornerList::ReadFrom(CStream_AC* aStream)	// override 
{
	CSortedDynamicArray_AC::ReadFrom(aStream);

	// read the size of the list
	ArrayIndex_AC itsSize;
	itsSize = aStream->ReadLong();

	// add in each element
	for (short count = 0; count < itsSize; ++count)
	{
		AdornPriority itsPriority = aStream->ReadByte();

		bool wasValid;
		TAdorner * itsAdorner = dynamic_cast_AC(TAdorner*,aStream->ReadStreamObject(wasValid));
		if (wasValid && itsAdorner)
			AddAdorner(itsAdorner, itsPriority);
#if qDebugMsg
		else
			::ProgramBreak_AC("###Unknown TAdorner in TAdornerList::ReadFrom.");
#endif

	}
}

//----------------------------------------------------------------------------------------
// TAdornerList::WriteTo:
//----------------------------------------------------------------------------------------
void TAdornerList::WriteTo(CStream_AC* aStream) const	// override 
{
	CSortedDynamicArray_AC::WriteTo(aStream);

	// write the size of the list
	ArrayIndex_AC itsSize;
	itsSize = GetSize();
	aStream->WriteLong(itsSize);

	// write out each element
	if (itsSize > 0)
		for (short count = 1; count <= itsSize; ++count)
		{
			AdornerElementPtr theAdornerElementPtr;
			theAdornerElementPtr = (AdornerElementPtr)ComputeAddress(count);

			aStream->WriteByte(theAdornerElementPtr->priority);

			aStream->WriteStreamObject(theAdornerElementPtr->adorner, kStandardObject);
		}
}

//----------------------------------------------------------------------------------------
// TAdornerList::CompareElements:
//----------------------------------------------------------------------------------------
CompareResult_AC TAdornerList::CompareElements(const void* Element1,
											const void* Element2) const// Override
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	AdornPriority priority1 = ((AdornerElementPtr)Element1)->priority;
	AdornPriority priority2 = ((AdornerElementPtr)Element2)->priority;

	if (priority1 > priority2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (priority1 < priority2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TAdornerList::AddAdorner:
//----------------------------------------------------------------------------------------
void TAdornerList::AddAdorner(TAdorner* itsAdorner,
							  AdornPriority itsPriority)
{
	// Need to avoid adding itsAdorner with itsPriority if it's already in the array, so test each
	// item in the array and then bail out if it's already in the array.
	bool wasInList = false;
	ArrayIndex_AC itsSize = GetSize();
	if (itsSize > 0)
		for (short count = 1; count <= itsSize; ++count)
		{
			AdornerElementPtr theAdornerElementPtr;
			theAdornerElementPtr = (AdornerElementPtr)ComputeAddress(count);

			if ((theAdornerElementPtr->adorner == itsAdorner) && (theAdornerElementPtr->priority == itsPriority))
			{
				wasInList = true;
				break;							// it's already in the list, so bail out.
			}
		}

	if (!wasInList)
	{
		// It's not already in the list, so insert it.
		AdornerElement anAdornerElement(itsPriority, itsAdorner);

		InsertElementInOrder(&anAdornerElement);
	}
}

//----------------------------------------------------------------------------------------
// End of UAdorners.cp
