//----------------------------------------------------------------------------------------
// UAppleEvents.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

// ACS

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __USUBSTITUTION__
#include "USubstitution.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __PROCESSES__
#include <Processes.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SEGLOAD__
#include <SegLoad.h>
#endif

// ANSI


//========================================================================================
// static allocations
//========================================================================================

TemplateTempRoutineDescriptor<AEIdleUPP> TAppleEvent::fgIdleProc;
TemplateTempRoutineDescriptor<AEFilterUPP> TAppleEvent::fgFilterProc;

CAEDesc CAEDesc::fgNullDesc;

//========================================================================================
// CLASS CAEDesc
//========================================================================================
//----------------------------------------------------------------------------------------
// CAEDesc::CAEDesc: 
//----------------------------------------------------------------------------------------


CAEDesc::CAEDesc()
{
	descriptorType = typeNull;
	dataHandle = NULL;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::CanCoerceDesc: 
//--------------------------------------------------------------------------------------------------


bool CAEDesc::CanCoerceDesc(DescType toType) const
{
	CTempDesc tempDesc;

	return AECoerceDesc(this, toType, tempDesc) == noErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::CoerceDesc: 
//--------------------------------------------------------------------------------------------------


CAEDesc CAEDesc::CoerceDesc(DescType toType) const
{
	CAEDesc result;
	::ThrowIfOSErr_AC(AECoerceDesc(this, toType, result));
	
	return result;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::CreateDesc: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::CreateDesc(DescType typeCode,
						 const void* dataPtr,
						 Size dataSize)
{
	if (dataHandle)						// optimization
		::ThrowIfOSErr_AC(DisposeDesc());

	::ThrowIfOSErr_AC(AECreateDesc(typeCode, dataPtr, dataSize, this));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::DisposeDesc: 
//--------------------------------------------------------------------------------------------------


OSErr CAEDesc::DisposeDesc()
// DisposeDesc returns a status because it is used in destructors, where we can't fail,
// and it is also used independently, as in CreateDesc. 
{
	OSErr result = noErr;

	if (dataHandle)
		result = AEDisposeDesc(this);

	if (result == noErr)
	{
		dataHandle = NULL;
		descriptorType = typeNull;
	}

	return result;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyPtr:
//--------------------------------------------------------------------------------------------------


OSErr CAEDesc::GetKeyPtr(AEKeyword theAEKeyword,
						 DescType desiredType,
						 DescType* typeCode,
						 void* dataPtr,
						 Size maximumSize,
						 Size* actualSize) const
{
	return AEGetKeyPtr(this, theAEKeyword, desiredType, typeCode, dataPtr, maximumSize, actualSize);
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyPtr:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyPtr(AEKeyword theAEKeyword,
						DescType typeCode,
						const void* dataPtr,
						Size dataSize)
{
	::ThrowIfOSErr_AC(AEPutKeyPtr(this, theAEKeyword, typeCode, dataPtr, dataSize));
}

#if !qPowerPC
//--------------------------------------------------------------------------------------------------
// CAEDesc::Set: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::Set(const AEDesc& theDesc)
{
	descriptorType = theDesc.descriptorType;
	dataHandle = theDesc.dataHandle;
}
#endif

//--------------------------------------------------------------------------------------------------
// CAEDesc::Set: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::Set(DescType theType,
					  Handle theHandle)
{
	descriptorType = theType;
	dataHandle = theHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::EqualDesc: 
//--------------------------------------------------------------------------------------------------


bool CAEDesc::EqualDesc(const CAEDesc& theDesc) const
{
	return CompareDesc(theDesc, kAEEquals);
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::CompareDesc: 
//--------------------------------------------------------------------------------------------------


bool CAEDesc::CompareDesc(const CAEDesc& theDesc,
							 DescType compareHow) const
{
	bool theResult = false;

	CAEDesc localDesc1( *this);
	CAEDesc localDesc2(theDesc);
	DescType type1 = localDesc1.GetDescriptorType();
	DescType type2 = localDesc2.GetDescriptorType();
	bool tossDesc1 = false;
	bool tossDesc2 = false;

	if (compareHow == kAEEquals && (localDesc1.GetDataHandle() == localDesc2.GetDataHandle()))
		theResult = true;
	else
	{
		bool proceed = true;

		if (type1 != type2)
		{
			// Try coercing theDesc to type1
			if (AECoerceDesc(theDesc, type1, localDesc2) == noErr)
				tossDesc2 = true;
			// Try coercing myself to type2
			else if (AECoerceDesc(*this, type2, localDesc1) == noErr)
				tossDesc1 = true;
			else
				proceed = false;
		}

		if (proceed)
		{
			type1 = localDesc1.GetDescriptorType();

			try
			{
				switch (type1)
				{
					case typeAEList:
						{
							// Compare all the elements in the list
							long items1 = localDesc1.CountListItems();
							long items2 = localDesc2.CountListItems();
							if (((compareHow == kAEEquals) && (items1 == items2)) || (((compareHow == kAEBeginsWith) || (compareHow == kAEEndsWith)) && (items1 >= items2)))
							{
								theResult = true;		// empty lists are equal
								long numToCheck = items1;
								if ((compareHow == kAEBeginsWith) || (compareHow == kAEEndsWith))
									numToCheck = items2;
								long list1Indx = 1;
								if (compareHow == kAEEndsWith)
									list1Indx = (items1 - items2) + 1;
								for (long i = 1; i <= numToCheck; ++i)
								{
									CTempDesc list1Item;
									CTempDesc list2Item;
									localDesc1.GetNthItem(list1Indx++, typeWildCard, list1Item);
									localDesc2.GetNthItem(i, typeWildCard, list2Item);
									theResult = theResult && list1Item.CompareDesc(list2Item, kAEEquals);
								}
							}
							break;
						}

					case typeChar:
						{
							Handle charsHdl1 = localDesc1.GetDataHandle();
							Handle charsHdl2 = localDesc2.GetDataHandle();

							CTempHandleLock_AC tempHandleLock(charsHdl1);
							CTempHandleLock_AC tempHandleLock2(charsHdl2);

							short size1 = static_cast<short>(localDesc1.GetDataSize());
							short size2 = static_cast<short>(localDesc2.GetDataSize());
							short magIDResult = IUMagString(*charsHdl1, *charsHdl2, size1, size2);
							switch (compareHow)
							{
								case kAEGreaterThan:
									theResult = magIDResult == 1;
									break;
								case kAEGreaterThanEquals:
									theResult = magIDResult != -1;
									break;
								case kAEEquals:
									theResult = magIDResult == 0;
									break;
								case kAELessThan:
									theResult = magIDResult == -1;
									break;
								case kAELessThanEquals:
									theResult = magIDResult != 1;
									break;
								case kAEBeginsWith:
									if (size1 >= size2)
										theResult = !IUMagString(*charsHdl1, *charsHdl2, size2, size2);
									break;
								case kAEEndsWith:
									{
										long theOffset = size1 - size2;
										if (theOffset >= 0)
											theResult = !IUMagString(*charsHdl1 + theOffset, *charsHdl2, size2, size2);
										break;
									}
								case kAEContains:
									if (size1 >= size2)
										theResult = Munger(charsHdl1, 0, *charsHdl2, size2, NULL, 0) >= 0;
									break;
							}
							break;
						}

						// Compare rectangles
					case typeQDRectangle:
						{
							CRect_AC rect1;
							localDesc1.GetRect(rect1);
							CRect_AC rect2;
							localDesc2.GetRect(rect2);
							if (compareHow == kAEEquals)
								theResult = rect1 == rect2;
							else if (compareHow == kAEContains)
								theResult = rect1.Contains(rect2);
							break;
						}

						// Compare colors
					case typeRGBColor:
						if (compareHow == kAEEquals)
						{
							CRGBColor_AC myPropColor;
							localDesc1.GetRGBColor(myPropColor);
							CRGBColor_AC compareColor;
							localDesc2.GetRGBColor(compareColor);
							theResult = myPropColor == compareColor;
							break;
						}

					case typeLongDateTime:
						{
							unsigned long num1 = static_cast<unsigned long>(localDesc1.GetDateTime());
							unsigned long num2 = static_cast<unsigned long>(localDesc2.GetDateTime());
							switch (compareHow)
							{
								case kAEGreaterThan:
									theResult = num1 > num2;
									break;
								case kAEGreaterThanEquals:
									theResult = num1 >= num2;
									break;
								case kAEEquals:
									theResult = num1 == num2;
									break;
								case kAELessThan:
									theResult = num1 < num2;
									break;
								case kAELessThanEquals:
									theResult = num1 <= num2;
									break;
							}
						}
						break;

						// Handle all the types we can convert to longs and then compare
					case typeType:
					case typeEnumerated:
					case typeBoolean:
					case typeLongInteger:
					case typeShortInteger:
						{
							long num1 = 0;
							long num2 = 0;

							switch (type1)
							{
									// There are no coercion handlers for some of these types so we'll get them by
									// the descriptor type then cast the results to longs.
								case typeEnumerated:
									{
										num1 = static_cast<long>(localDesc1.GetEnum());
										num2 = static_cast<long>(localDesc2.GetEnum());
										break;
									}
								case typeType:
									{
										num1 = static_cast<long>(localDesc1.GetType());
										num2 = static_cast<long>(localDesc2.GetType());
										break;
									}
								case typeBoolean:
									{
										num1 = static_cast<long>(localDesc1.GetBoolean());
										num2 = static_cast<long>(localDesc2.GetBoolean());
										break;
									}
								default:
									{
										num1 = localDesc1.GetLong();
										num2 = localDesc2.GetLong();
									}
							}
							switch (compareHow)
							{
								case kAEGreaterThan:
									theResult = num1 > num2;
									break;
								case kAEGreaterThanEquals:
									theResult = num1 >= num2;
									break;
								case kAEEquals:
									theResult = num1 == num2;
									break;
								case kAELessThan:
									theResult = num1 < num2;
									break;
								case kAELessThanEquals:
									theResult = num1 <= num2;
									break;
							}
							break;
						}

					default:
						{
							// Check for the same handle content
							long size1 = localDesc1.GetDataSize();
							long size2 = localDesc2.GetDataSize();
							Handle data1 = localDesc1.GetDataHandle();
							Handle data2 = localDesc2.GetDataHandle();
							theResult = ((compareHow == kAEEquals) && (size1 == size2) && (EqualBlocks(*data1, *data2, static_cast<short>(size1))));
						}
				}
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);

				// Don't resignal failure, just return false
				theResult = false;
			}
			

			if (tossDesc1)
				localDesc1.DisposeDesc();
			if (tossDesc2)
				localDesc2.DisposeDesc();
		}
	}

	return theResult;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetDataSize: 
//--------------------------------------------------------------------------------------------------


long CAEDesc::GetDataSize() const
{
	return dataHandle ? ::GetHandleSize(dataHandle) : 0;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::CreateList: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::CreateList()
{
	::ThrowIfOSErr_AC(AECreateList(NULL, 0, false, this));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::CountListItems: 
//--------------------------------------------------------------------------------------------------


long CAEDesc::CountListItems()
{
	long theCount = 0;
	::ThrowIfOSErr_AC(AECountItems(this, &theCount));

	return theCount;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetNthItem: 
//--------------------------------------------------------------------------------------------------


CAEDesc CAEDesc::GetNthItem(long index,
						 DescType desiredType)
{
	DescType theAEKeyword;
	CAEDesc theDesc;
	::ThrowIfOSErr_AC(AEGetNthDesc(this, index, desiredType, &theAEKeyword, theDesc));
	
	return theDesc;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutListItem: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutListItem(long index,
						  CAEDesc& theDesc)
{
	::ThrowIfOSErr_AC(AEPutDesc(this, index, theDesc));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::DeleteListItem: 
//--------------------------------------------------------------------------------------------------


OSErr CAEDesc::DeleteListItem(long index)
{
	return AEDeleteItem(this, index);
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::CreateRecord: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::CreateRecord()
{
	::ThrowIfOSErr_AC(AECreateList(NULL, 0, true, this));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetFSSpec: Note that this method provides transparent coercion of alias records into
// CFSSpec_AC records.
//--------------------------------------------------------------------------------------------------


CFSSpec_AC CAEDesc::GetFSSpec() const
{
	return descriptorType == typeFSS ? **(CFSSpec_AC **)dataHandle : **(CFSSpec_AC **)CTempDesc(CoerceDesc(typeFSS)).dataHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutFSSpec: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutFSSpec(const CFSSpec_AC& fileSpec)
{
	CreateDesc(typeFSS, &fileSpec, sizeof(fileSpec));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetRect: 
//--------------------------------------------------------------------------------------------------


CRect_AC CAEDesc::GetRect() const
{
	return descriptorType == typeQDRectangle ? **(CRect_AC **)dataHandle : **(CRect_AC **)CTempDesc(CoerceDesc(typeQDRectangle)).dataHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutRect: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutRect(const CRect_AC& theRect)
{
	CreateDesc(typeQDRectangle, &theRect, sizeof(theRect));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetType: 
//--------------------------------------------------------------------------------------------------


DescType CAEDesc::GetType() const
{
	return descriptorType == typeType ? **(DescType **)dataHandle : **(DescType **)CTempDesc(CoerceDesc(typeType)).dataHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutType: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutType(DescType theType)
{
	CreateDesc(typeType, &theType, sizeof(theType));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetString: 
//--------------------------------------------------------------------------------------------------


CStr255_AC CAEDesc::GetString(long maximumSize) const
{
	CStr255_AC theString;

	long theSize = Min_AC(GetDataSize(), maximumSize);
	if (theSize)
	{
		if (descriptorType == typeChar)
			theString.CopyFrom(static_cast<void*>(*dataHandle), theSize);
		else
		{
			CTempDesc myTempDesc(CoerceDesc(typeChar));

			theString.CopyFrom(static_cast<void*>(*myTempDesc.dataHandle), Min_AC(maximumSize, myTempDesc.GetDataSize()));
		}
	}
	
	return theString;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutString: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutString(const CStr255_AC& theString)
{
	CreateDesc(typeChar, &theString.fStr[1], theString.Length());
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetBoolean: 
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetBoolean() const
{
	return descriptorType == typeBoolean ? **(Boolean **)dataHandle : **(Boolean **)CTempDesc(CoerceDesc(typeBoolean)).dataHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutBoolean: 
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutBoolean(bool theBoolean)
{
	Boolean localBoolean = theBoolean;
	CreateDesc(typeBoolean, &localBoolean, sizeof(localBoolean));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetEnum: 
//--------------------------------------------------------------------------------------------------


DescType CAEDesc::GetEnum() const
{
	return descriptorType == typeEnumeration ? **(DescType **)dataHandle : **(DescType **)CTempDesc(CoerceDesc(typeEnumeration)).dataHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutEnum:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutEnum(DescType theEnum)
{
	CreateDesc(typeEnumeration, &theEnum, sizeof(theEnum));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetLong:
//--------------------------------------------------------------------------------------------------


long CAEDesc::GetLong() const
{
	return descriptorType == typeLongInteger ? **(long **)dataHandle : **(long **)CTempDesc(CoerceDesc(typeLongInteger)).dataHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutLong:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutLong(long theLong)
{
	CreateDesc(typeLongInteger, &theLong, sizeof(theLong));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetDateTime:
//--------------------------------------------------------------------------------------------------


unsigned long CAEDesc::GetDateTime() const
{
	LongDateTime date;
	((LongDateCvt *) & date)->hl.lHigh = 0;
	((LongDateCvt *) & date)->hl.lLow = 0;

	date = (descriptorType == typeLongDateTime) ? **(LongDateTime **)dataHandle : **(LongDateTime **)CTempDesc(CoerceDesc(typeLongDateTime)).dataHandle;

	return ((LongDateCvt *) & date)->hl.lLow;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutDateTime:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutDateTime(unsigned long theDateTime)
{
	LongDateTime date;
	((LongDateCvt *) & date)->hl.lHigh = 0;
	((LongDateCvt *) & date)->hl.lLow = theDateTime;
	CreateDesc(typeLongDateTime, &date, sizeof(date));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetRGBColor:
//--------------------------------------------------------------------------------------------------


CRGBColor_AC CAEDesc::GetRGBColor() const
{
	return descriptorType == typeRGBColor ? **(CRGBColor_AC **)dataHandle : **(CRGBColor_AC **)CTempDesc(CoerceDesc(typeRGBColor)).dataHandle;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutRGBColor:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutRGBColor(const CRGBColor_AC& theColor)
{
	CreateDesc(typeRGBColor, &theColor, sizeof(theColor));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetObject:
//--------------------------------------------------------------------------------------------------


MScriptableObject* CAEDesc::GetObject() const
{
	return dataHandle ? **(MScriptableObject * **)dataHandle : gDispatcher;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutObject:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutObject(MScriptableObject* theObject)
{
	CreateDesc(theObject->GetOMClass(), &theObject, sizeof(theObject));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyString:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyString(DescType theKey,
							  CStr255_AC& theStr,
							  long maximumSize) const
{
	CStr255_AC localString;
	DescType actualType;
	long actualSize = 0;
	OSErr iErr = GetKeyPtr(theKey, typeChar, &actualType, &localString[1], maximumSize, &actualSize);
	if (!iErr)
	{
		localString[0] = static_cast<unsigned char>(actualSize);
		theStr = localString;
	}

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyEnum:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyEnum(DescType theKey,
							DescType& theEnum) const
{
	DescType localEnum;
	DescType actualType;
	long actualSize = 0;
	OSErr iErr = GetKeyPtr(theKey, typeEnumeration, &actualType, &localEnum, 4, &actualSize);
	if (!iErr)
		theEnum = localEnum;

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyDesc:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyDesc(DescType theKey,
							DescType desiredType,
							CAEDesc& theAEDesc) const
{
	CAEDesc localDesc;

	OSErr iErr = AEGetKeyDesc(this, theKey, desiredType, localDesc);
	if (!iErr)
		theAEDesc = localDesc;

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyLong:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyLong(DescType theKey,
							long& theNum) const
{
	long localNum;
	DescType actualType;
	long actualSize = 0;
	OSErr iErr = GetKeyPtr(theKey, typeLongInteger, &actualType, &localNum, 4, &actualSize);
	if (!iErr)
		theNum = localNum;

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyRect:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyRect(DescType theKey,
							CRect_AC& theRect) const
{
	CRect_AC localRect;
	DescType actualType;
	long actualSize = 0;
	OSErr iErr = GetKeyPtr(theKey, typeQDRectangle, &actualType, &localRect, sizeof(localRect), &actualSize);
	if (!iErr)
		theRect = localRect;

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyDateTime:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyDateTime(DescType theKey,
								unsigned long& theDate) const
{
	LongDateTime date;
	((LongDateCvt *) & date)->hl.lHigh = 0;
	((LongDateCvt *) & date)->hl.lLow = 0;
	DescType actualType;
	long actualSize = 0;
	OSErr iErr = GetKeyPtr(theKey, typeLongDateTime, &actualType, &date, sizeof(date), &actualSize);
	if (iErr == noErr)
		theDate = ((LongDateCvt *) & date)->hl.lLow;

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyBoolean:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyBoolean(DescType theKey,
							   bool& theBoolean) const
{
	Boolean localBoolean;
	DescType actualType;
	long actualSize = 0;
	OSErr iErr = GetKeyPtr(theKey, typeBoolean, &actualType, &localBoolean, sizeof(localBoolean), &actualSize);
	if (iErr == noErr)
		theBoolean = localBoolean;

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::GetKeyColor:
//--------------------------------------------------------------------------------------------------


bool CAEDesc::GetKeyColor(DescType theKey,
							 CRGBColor_AC& theColor) const
{
	CRGBColor_AC localColor;
	DescType actualType;
	long actualSize = 0;
	OSErr iErr = GetKeyPtr(theKey, typeRGBColor, &actualType, &localColor, sizeof(localColor), &actualSize);
	if (!iErr)
		theColor = localColor;

	return !iErr;
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyString:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyString(DescType theKey,
						   const CStr255_AC& theStr)
{
	CStr255_AC localString = theStr;
	PutKeyPtr(theKey, typeChar, &localString[1], localString.Length());
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyEnum:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyEnum(DescType theKey,
						 DescType theEnum)
{
	PutKeyPtr(theKey, typeEnumeration, &theEnum, 4);
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyDateTime:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyDateTime(DescType theKey,
							 unsigned long theDate)
{
	LongDateTime date;
	((LongDateCvt *) & date)->hl.lHigh = 0;
	((LongDateCvt *) & date)->hl.lLow = theDate;
	PutKeyPtr(theKey, typeLongDateTime, &date, sizeof(date));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::DeleteKeyDesc:
//--------------------------------------------------------------------------------------------------


OSErr CAEDesc::DeleteKeyDesc(DescType theKey)
{
	return AEDeleteKeyDesc(this, theKey);
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyDesc:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyDesc(DescType theKey,
						 CAEDesc& theAEDesc)
{
	::ThrowIfOSErr_AC(AEPutKeyDesc(this, theKey, theAEDesc));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyLong:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyLong(DescType theKey,
						 long theNum)
{
	PutKeyPtr(theKey, typeLongInteger, &theNum, sizeof(theNum));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyRect:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyRect(DescType theKey,
						 const CRect_AC& theRect)
{
	PutKeyPtr(theKey, typeQDRectangle, &theRect, sizeof(theRect));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyBoolean:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyBoolean(DescType theKey,
							bool theBoolean)
{
	Boolean localBoolean = theBoolean;
	PutKeyPtr(theKey, typeBoolean, &localBoolean, sizeof(localBoolean));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::PutKeyColor:
//--------------------------------------------------------------------------------------------------


void CAEDesc::PutKeyColor(DescType theKey,
						  const CRGBColor_AC& theColor)
{
	PutKeyPtr(theKey, typeRGBColor, &theColor, sizeof(theColor));
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::ReadFrom
//--------------------------------------------------------------------------------------------------


void CAEDesc::ReadFrom(CStream_AC* aStream)
{
	::ThrowIfOSErr_AC(DisposeDesc());
	descriptorType = (DescType)aStream->ReadLong();
	bool hasHandle = aStream->ReadBoolean();
	if (hasHandle)
		dataHandle = aStream->ReadHandle();
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::ReadFrom
//--------------------------------------------------------------------------------------------------


void CAEDesc::ReadFrom(TFile* aFile,
					   ResType resourceType,
					   short resourceID)
{
#if qDebug
	::ThrowIfNonObject_AC(aFile);
#endif

	::ThrowIfOSErr_AC(DisposeDesc());
	if (aFile->HasRsrcFork())
	{
		CTempUseResFile_AC tempUseResFile(aFile->GetRsrcRefNum());
		Handle descHandle = Get1Resource(resourceType, resourceID);

		if (descHandle)
		{
			descriptorType = resourceType;
			DetachResource(descHandle);
			dataHandle = descHandle;
		}
	}
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::WriteTo
//--------------------------------------------------------------------------------------------------


void CAEDesc::WriteTo(CStream_AC* aStream) const
{
	aStream->WriteLong(static_cast<long>(GetDescriptorType()));
	Handle theDataHandle = GetDataHandle();
	aStream->WriteBoolean(theDataHandle != NULL);
	if (theDataHandle)
		aStream->WriteHandle(theDataHandle);
}

//--------------------------------------------------------------------------------------------------
// CAEDesc::WriteTo
//--------------------------------------------------------------------------------------------------


void CAEDesc::WriteTo(TFile* aFile,
					  short resourceID,
					  CStr255_AC& resourceName) const
{
#if qDebug
	::ThrowIfNonObject_AC(aFile);
#endif

	if (aFile->HasRsrcFork() && dataHandle)
	{
		Handle copiedHandle;
		::ThrowIfOSErr_AC(CopyHandle_AC(dataHandle, copiedHandle));
		CTempHandle_AC tempdataHandle(copiedHandle);

		CTempUseResFile_AC tempUseResFile(aFile->GetRsrcRefNum());
		MAAddResource(tempdataHandle, descriptorType, resourceID, resourceName);
		::ThrowIfResError_AC();
		WriteResource(tempdataHandle);
		ReleaseResource(tempdataHandle.release());
	}
}

//========================================================================================
// CLASS TAppleEvent
//========================================================================================
//----------------------------------------------------------------------------------------
// TAppleEvent::TAppleEvent:
//----------------------------------------------------------------------------------------


TAppleEvent::TAppleEvent(const AEEventClass itsEventClass,
							  const AEEventID itsEventID,
							  const AEAddressDesc& itsAddress,
							  long itsSendingMode) :
	fSendingMode(itsSendingMode),
	fTimeoutVal(kAEDefaultTimeout),
	fPriority(kAENormalPriority),
	fFreeMessage(false)
{
	if (itsEventClass != typeNull)
	{
		::ThrowIfOSErr_AC(AECreateAppleEvent(itsEventClass, itsEventID, &itsAddress, kAutoGenerateReturnID, kAnyTransactionID, this));

		fFreeMessage = true;						// We created it, we need to free it
	}
}

//----------------------------------------------------------------------------------------
// TAppleEvent::TAppleEvent:
//----------------------------------------------------------------------------------------


TAppleEvent::TAppleEvent(const AppleEvent& theMessage,
										bool freeMessage) :
	CAEDesc(theMessage),

	fSendingMode(kAENoReply),
	fTimeoutVal(kAEDefaultTimeout),
	fPriority(kAENormalPriority),
	fFreeMessage(freeMessage)
{
	// Initialize the reply mode.
	DescType actualType;
	Size actualSize;
	short interactLevel = 0;
	if (AEGetAttributePtr(this, keyInteractLevelAttr, typeShortInteger, &actualType, &interactLevel, sizeof(short), &actualSize) == noErr)
		fSendingMode = interactLevel & kAEReplyModeMask;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::~TAppleEvent
//----------------------------------------------------------------------------------------


TAppleEvent::~TAppleEvent()
{
	if (fFreeMessage)
		AEDisposeDesc(this);
}

//----------------------------------------------------------------------------------------
//		 MakeCreateElementEvent: 
//----------------------------------------------------------------------------------------


TAppleEvent* TAppleEvent::MakeCreateElementEvent(AEAddressDesc& targetAddress,
												 AESendMode sendMode,
												 CAEDesc& targetSpecifier,
												 DescType newElementType,
												 DescType newElementPosition,
												 CAEDesc& initialData,
												 CAEDesc& initialProperties)
{
	CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kAECoreSuite, kAECreateElement, targetAddress, sendMode));

	// create the insertion record
	CTempDesc insertionRecord;
	insertionRecord.CreateRecord();
	insertionRecord.PutKeyDesc(keyAEObject, targetSpecifier);
	insertionRecord.PutKeyEnum(keyAEPosition, newElementPosition);
	CTempDesc insertionLocation(insertionRecord.CoerceDesc(typeInsertionLoc));

	// add the insertion record to the apple event
	theEvent->WriteParameter(keyAEInsertHere, insertionLocation);

	// add the class of the item to be created to the event
	theEvent->WriteType(keyAEObjectClass, newElementType);

	CTempDesc emptyRecord;
	emptyRecord.CreateRecord();

	// add the initial data to the event
	if (initialData.GetDataHandle())
		theEvent->WriteParameter(keyAEData, initialData);
	else
		theEvent->WriteParameter(keyAEData, emptyRecord);

	if (initialProperties.GetDataHandle())
		theEvent->WriteParameter(keyAEPropData, initialProperties);
	else
		theEvent->WriteParameter(keyAEPropData, emptyRecord);

	return theEvent.release();
}

//----------------------------------------------------------------------------------------
// TAppleEvent::Send:
//----------------------------------------------------------------------------------------


TAppleEvent* TAppleEvent::Send()
{
	TAppleEvent * resultEvent = NULL;
	::ThrowIfOSErr_AC(SendEvent(resultEvent));

	return resultEvent;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SendEvent:
//----------------------------------------------------------------------------------------


OSErr TAppleEvent::SendEvent()
{
	TAppleEvent* resultEvent = NULL;
	OSErr err = SendEvent(resultEvent);
	delete resultEvent;

	return err;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SendEvent:
//----------------------------------------------------------------------------------------


OSErr TAppleEvent::SendEvent(TAppleEvent*& replyEvent)
{
	AppleEvent reply;

	OSErr err = AECreateDesc(typeNull, NULL, 0, &reply);
	if (err == noErr)
	{
		// Debugging note: 
		//	If the application is sending this Apple Event to itself, you can follow it
		//	through the AESend by setting breakpoints in the functions
		//	TOSADispatcher::PreDispatchHandlerGlue and TOSADispatcher::DispatchHandlerGlue
		//	in UScripting.cp. 

		bool keepBusy;
		if (gDispatcher)
			keepBusy = gDispatcher->KeepCursorBusy(true);

		err = AESend(this, &reply, fSendingMode, fPriority, fTimeoutVal, TAppleEvent::fgIdleProc, TAppleEvent::fgFilterProc);

		if (gDispatcher)
			gDispatcher->KeepCursorBusy(keepBusy);

		if (err == noErr && (GetSendingMode() & kAEReplyModeMask) == kAEWaitReply)
			replyEvent = TH_new TAppleEvent(reply, true);
		else
			AEDisposeDesc(&reply);
	}

	return err;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::GetAddress:
//----------------------------------------------------------------------------------------


void TAppleEvent::GetAddress(AEAddressDesc& theAddress)

{
	::ThrowIfOSErr_AC(AEGetAttributeDesc(this, keyAddressAttr, typeWildCard, &theAddress));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::GetProcessSerialNumber:
//----------------------------------------------------------------------------------------


void TAppleEvent::GetProcessSerialNumber(ProcessSerialNumber& thePSN)

{
	DescType actualType;
	Size actualSize;
	::ThrowIfOSErr_AC(AEGetAttributePtr(this, keyAddressAttr, typeProcessSerialNumber, &actualType, &thePSN, sizeof(ProcessSerialNumber), &actualSize));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::GetReturnID:
//----------------------------------------------------------------------------------------


long TAppleEvent::GetReturnID()
{
	DescType actualType;
	long returnID;
	long actualSize;

	::ThrowIfOSErr_AC(AEGetAttributePtr(this, keyReturnIDAttr, typeLongInteger, &actualType, &returnID, sizeof(returnID), &actualSize));

	return returnID;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::GetTransactionID:
//----------------------------------------------------------------------------------------


long TAppleEvent::GetTransactionID()
{
	long transactionID;
	DescType actualType;
	long actualSize;

	return (AEGetAttributePtr(this, keyTransactionIDAttr, typeLongInteger, &actualType, &transactionID, sizeof(transactionID), &actualSize) == noErr) ? transactionID : 0;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SetAddress:
//----------------------------------------------------------------------------------------


void TAppleEvent::SetAddress(const AEAddressDesc& theAddress)
{
	::ThrowIfOSErr_AC(AEPutAttributeDesc(this, keyAddressAttr, &theAddress));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SetPriority:
//----------------------------------------------------------------------------------------


void TAppleEvent::SetPriority(short thePriority)
{
	fPriority = thePriority;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SetReturnID:
//----------------------------------------------------------------------------------------


void TAppleEvent::SetReturnID(long returnID)
{
	::ThrowIfOSErr_AC(AEPutAttributePtr(this, keyReturnIDAttr, typeLongInteger, &returnID, sizeof(returnID)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SetSendingMode:
//----------------------------------------------------------------------------------------


void TAppleEvent::SetSendingMode(long sendingMode)
{
	fSendingMode = sendingMode;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SetTimeoutVal:
//----------------------------------------------------------------------------------------


void TAppleEvent::SetTimeoutVal(long theTimeoutVal)
{
	fTimeoutVal = theTimeoutVal;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::SetTransactionID:
//----------------------------------------------------------------------------------------


void TAppleEvent::SetTransactionID(long transactionID)
{
	::ThrowIfOSErr_AC(AEPutAttributePtr(this, keyTransactionIDAttr, typeLongInteger, &transactionID, sizeof(transactionID)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadEventClass:
//----------------------------------------------------------------------------------------


AEEventClass TAppleEvent::ReadEventClass()
{
	DescType typeCode;
	AEEventClass eventClass;
	Size actualSize;

	::ThrowIfOSErr_AC(AEGetAttributePtr(this, keyEventClassAttr, typeType, &typeCode, &eventClass, sizeof(eventClass), &actualSize));

	return eventClass;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadEventClass:
//----------------------------------------------------------------------------------------


AEEventID TAppleEvent::ReadEventID()
{
	DescType typeCode;
	AEEventID eventID;
	Size actualSize;

	::ThrowIfOSErr_AC(AEGetAttributePtr(this, keyEventIDAttr, typeType, &typeCode, &eventID, sizeof(eventID), &actualSize));

	return eventID;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadShort:
//----------------------------------------------------------------------------------------


short TAppleEvent::ReadShort(const AEKeyword theKey)
{
	DescType actualType;
	Size actualSize;
	short theData = 0;

	::ThrowIfOSErr_AC(AEGetParamPtr(this, theKey, typeShortInteger, &actualType, &theData, sizeof(theData), &actualSize));

	return theData;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadLong:
//----------------------------------------------------------------------------------------


long TAppleEvent::ReadLong(const AEKeyword theKey)
{
	DescType actualType;
	Size actualSize;
	long theData = 0;

	::ThrowIfOSErr_AC(AEGetParamPtr(this, theKey, typeLongInteger, &actualType, &theData, sizeof(theData), &actualSize));

	return theData;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadString:
//----------------------------------------------------------------------------------------


void TAppleEvent::ReadString(const AEKeyword theKey,
							 CStr255_AC& theData)
{
	DescType actualType;
	Size actualSize;

	theData.Empty();
	::ThrowIfOSErr_AC(AEGetParamPtr(this, theKey, typeChar, &actualType, &theData[1], sizeof(CStr255_AC) - 1, &actualSize));

	if (actualSize > 255)
	{
#if qDebug
		ProgramReport_AC("TAppleEvent::ReadString: String was longer than 255 bytes.  Will be truncated\n");
#endif

		actualSize = 255;
	}
	theData.Length() = static_cast<unsigned char>(actualSize);
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadSectionHandle:
//----------------------------------------------------------------------------------------


SectionHandle TAppleEvent::ReadSectionHandle(const AEKeyword theKey)
{
	DescType actualType;
	Size actualSize;
	SectionHandle theData = NULL;

	::ThrowIfOSErr_AC(AEGetParamPtr(this, theKey, typeSectionH, &actualType, &theData, sizeof(SectionHandle), &actualSize));

	return theData;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadPtrList:
//----------------------------------------------------------------------------------------


void TAppleEvent::ReadPtrList(const AEKeyword theKey,
							  const DescType theType,
							  CDynamicArray_AC* theData)
{
	AEDescList theDescList;
	::ThrowIfOSErr_AC(AEGetParamDesc(this, theKey, typeAEList, &theDescList));

	long items;

	::ThrowIfOSErr_AC(AECountItems(&theDescList, &items));// Get the number of elements in the list 
	CTempPtr_AC thePtr(NewPermPtr(theData->GetElementSize()));

	for (short index = 1; index <= items; ++index)
	{
		AEKeyword theActualKey;
		DescType theActualType;
		Size theActualSize;

		::ThrowIfOSErr_AC(AEGetNthPtr(&theDescList, index, theType, &theActualKey, &theActualType, thePtr, theData->GetElementSize(), &theActualSize));

		theData->InsertElementsBefore(theData->GetSize() + 1, thePtr, 1);
	}
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadHandleList:
//----------------------------------------------------------------------------------------


void TAppleEvent::ReadHandleList(const AEKeyword theKey,
								 const DescType theType,
								 CDynamicArray_AC* theData)
{
	AEDescList theDescList;
	::ThrowIfOSErr_AC(AEGetParamDesc(this, theKey, typeAEList, &theDescList));

	long items;
	::ThrowIfOSErr_AC(AECountItems(&theDescList, &items));// Get the number of elements in the list 
	for (short index = 1; index <= items; ++index)
	{
		CTempDesc theHandle;
		AEKeyword theActualKey;
		::ThrowIfOSErr_AC(AEGetNthDesc(&theDescList, index, theType, &theActualKey, theHandle));

		theData->InsertElementsBefore(theData->GetSize() + 1, &theHandle.dataHandle, 1);
		theHandle.DontRelease();
	}
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadParameter:
//----------------------------------------------------------------------------------------


OSErr TAppleEvent::ReadParameter(const AEKeyword theKey,
								 const DescType desiredType,
								 CAEDesc& theData)
{
	return AEGetParamDesc(this, theKey, desiredType, theData);
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadParameterPtr:
//----------------------------------------------------------------------------------------


void TAppleEvent::ReadParameterPtr(const AEKeyword theKey,
								   const DescType desiredType,
								   DescType& actualType,
								   Ptr theData,
								   long maximumSize,
								   long& actualSize)
{
	::ThrowIfOSErr_AC(AEGetParamPtr(this, theKey, desiredType, &actualType, theData, maximumSize, &actualSize));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteShort:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteShort(const AEKeyword theKey,
							 short theData)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeShortInteger, &theData, sizeof(theData)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteLong:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteLong(const AEKeyword theKey,
							long theData)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeLongInteger, &theData, sizeof(theData)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteString:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteString(const AEKeyword theKey,
							  const CStr255_AC& theData)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeChar, &theData[1], theData.Length()));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteSectionHandle:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteSectionHandle(const AEKeyword theKey,
									 SectionHandle theData)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeSectionH, &theData, sizeof(SectionHandle)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WritePtrList:
//----------------------------------------------------------------------------------------


void TAppleEvent::WritePtrList(const AEKeyword theKey,
							   const DescType theType,
							   CDynamicArray_AC* theData)
{
	CTempDesc theDescList;						// used to be VOLATILE, but since it's a structure it won't be cached into registers on OUR compilers

	theDescList.CreateList();

	{
		// Create a block for the iterator to disambiguate it from the failure handler it's nested in
		for (CArrayIterator_AC iter(theData); iter.Current(); ++iter)
			::ThrowIfOSErr_AC(AEPutPtr(theDescList, iter.Current(), theType, theData->ComputeAddress(iter.Current()), theData->GetElementSize()));
	}
	::ThrowIfOSErr_AC(AEPutParamDesc(this, theKey, theDescList));

	::ThrowIfOSErr_AC(theDescList.DisposeDesc());
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteHandleList:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteHandleList(const AEKeyword theKey,
								  const DescType theType,
								  CDynamicArray_AC* theData)
{
	CTempDesc theDescList;

	theDescList.CreateList();

	// Create a block for the iterator to disambiguate it from the failure handler it's nested in
	for (CArrayIterator_AC iter(theData); iter.Current(); ++iter)
	{
		CAEDesc theElement(theType, NULL);
		theElement.dataHandle = *((Handle *)(theData->ComputeAddress(iter.Current())));
		theDescList.PutListItem(iter.Current(), theElement);
	}

	::ThrowIfOSErr_AC(AEPutParamDesc(this, theKey, theDescList));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteParameter:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteParameter(const AEKeyword theKey,
								 const CAEDesc& theData)
{
	::ThrowIfOSErr_AC(AEPutParamDesc(this, theKey, theData));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteParameterPtr:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteParameterPtr(const AEKeyword theKey,
									const DescType typeCode,
									Ptr theData,
									long dataSize)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeCode, theData, dataSize));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::DeleteParameter:
//----------------------------------------------------------------------------------------


void TAppleEvent::DeleteParameter(const AEKeyword theKey)
{
	::ThrowIfOSErr_AC(AEDeleteKeyDesc(this, theKey));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::CheckRequiredParameters:
//----------------------------------------------------------------------------------------


void TAppleEvent::CheckRequiredParameters()
{
	// look for the keyMissedKeywordAttr
	DescType returnedType;
	Size actualSize;
	OSErr theErr = AEGetAttributePtr(this, keyMissedKeywordAttr, typeWildCard, &returnedType, NULL, 0, &actualSize);
	if (theErr == noErr)
	{
		// We missed a parameter!
		::ThrowIfOSErr_AC(errAEParamMissed);
	}
	else if (theErr != errAEDescNotFound)
	{
		// Its OK if we got errAEDescNotFound because that means we got all the params.
		::ThrowIfOSErr_AC(theErr);
	}
}

//----------------------------------------------------------------------------------------
// TAppleEvent::HasParameter:
//----------------------------------------------------------------------------------------


bool TAppleEvent::HasParameter(const AEKeyword theKey)
{
	DescType typeCode;
	long actualSize;

	return AEGetParamPtr(this, theKey, typeWildCard, &typeCode, NULL, 0, &actualSize) == noErr;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ParameterType:
//----------------------------------------------------------------------------------------


DescType TAppleEvent::ParameterType(const AEKeyword theKey)
{
	// Returns the descriptor type of an event parameter.
	// TAppleEvent::ParameterType is here courtesy of Matt Clark. Thanks Matt.
	CTempDesc paramDesc;

	OSErr theErr = AEGetParamDesc(this, theKey, typeWildCard, paramDesc);
	if ((theErr != noErr) && (theErr != errAEDescNotFound))
		::ThrowIfOSErr_AC(theErr);
	DescType result = paramDesc.GetDescriptorType();

	return result;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadType:
//----------------------------------------------------------------------------------------


DescType TAppleEvent::ReadType(const AEKeyword theKey)
{
	DescType actualType;
	long actualSize;
	DescType theData = 0;

	OSErr theErr = AEGetParamPtr(this, theKey, typeType, &actualType, &theData, sizeof(theData), &actualSize);

	if ((theErr != noErr) && (theErr != errAEDescNotFound))
		::ThrowIfOSErr_AC(theErr);

	return theData;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadEnum:
//----------------------------------------------------------------------------------------


DescType TAppleEvent::ReadEnum(const AEKeyword theKey)
{
	DescType actualType;
	long actualSize;
	DescType theData = 0;

	OSErr theErr = AEGetParamPtr(this, theKey, typeEnumeration, &actualType, &theData, sizeof(theData), &actualSize);

	if ((theErr != noErr) && (theErr != errAEDescNotFound))
		::ThrowIfOSErr_AC(theErr);

	return theData;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadBoolean:
//----------------------------------------------------------------------------------------


bool TAppleEvent::ReadBoolean(const AEKeyword theKey)
{
	DescType actualType;
	long actualSize;
	Boolean theData = false;

	OSErr theErr = AEGetParamPtr(this, theKey, typeBoolean, &actualType, &theData, sizeof(theData), &actualSize);

	if ((theErr != noErr) && (theErr != errAEDescNotFound))
		::ThrowIfOSErr_AC(theErr);

	return theData;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadRect:
//----------------------------------------------------------------------------------------


void TAppleEvent::ReadRect(const AEKeyword theKey,
						   CRect_AC& theRect)
{
	DescType actualType;
	long actualSize;
	CRect_AC localRect;

	OSErr theErr = AEGetParamPtr(this, theKey, typeQDRectangle, &actualType, &localRect, sizeof(localRect), &actualSize);

	if ((theErr != noErr) && (theErr != errAEDescNotFound))
		::ThrowIfOSErr_AC(theErr);

	theRect = localRect;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteType:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteType(const AEKeyword theKey,
							DescType theType)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeType, &theType, sizeof(theType)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteEnum:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteEnum(const AEKeyword theKey,
							DescType theEnum)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeEnumeration, &theEnum, sizeof(theEnum)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteBoolean:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteBoolean(const AEKeyword theKey,
							   bool theBoolean)
{
	Boolean localBoolean = theBoolean;
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeBoolean, &localBoolean, sizeof(localBoolean)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteRect:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteRect(const AEKeyword theKey,
							const CRect_AC& theRect)
{
	::ThrowIfOSErr_AC(AEPutParamPtr(this, theKey, typeQDRectangle, &theRect, sizeof(theRect)));
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteReplySpecifier:
//----------------------------------------------------------------------------------------


bool TAppleEvent::WriteReplySpecifier(MScriptableObject* theReplyObject)
{
	// Use this whenever you need to return the object's specifier as the result of an event.
	// Makes an object specifier for an object and puts it in the keyAEResult parameter of this reply event.
	// MScriptableObject::ReturnReplySpecifier is here courtesy of Matt Clark. Thanks Matt.
	bool result = false;

	if (dataHandle)
	{
		CTempDesc objSpecifier;
		if (theReplyObject->MakeObjectSpecifier(objSpecifier, theReplyObject->GetSpecifierForm()))
			WriteParameter(keyAEResult, objSpecifier);

		result = true;
	}

	return result;
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteError:
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteError(long errNum,
							 short errorStringResource,
							 short errorStringIndex)
{
	if (dataHandle)
	{
		WriteLong(keyErrorNumber, errNum);
		CStr255_AC errStr(errorStringResource, errorStringIndex);
		WriteString(keyErrorString, errStr);
	}
}

//----------------------------------------------------------------------------------------
// TAppleEvent::WriteOSError
//----------------------------------------------------------------------------------------


void TAppleEvent::WriteOSError(OSErr error)
{
	if (descriptorType != typeNull && dataHandle)
	{
		CStr255_AC errStr;
		WriteLong(keyErrorNumber, errAEEventFailed);
		if (!LookupErrString(error, errReasonID, errStr))
		{
			// No error string was found, substitute the generic one.
			CStr255_AC errNumStr;
			LookupErrString(errGenericErrorWithNum, errReasonID, errStr);
			NumToString(error, errNumStr);
			MAParamText("^ENUM", errNumStr);
			MAReplaceText(errStr);
		}

		WriteString(keyErrorString, errStr);
	}
}

//----------------------------------------------------------------------------------------
// TAppleEvent::ReadError:
//----------------------------------------------------------------------------------------


void TAppleEvent::ReadError(long& errorNumber,
							CStr255_AC& errorString)
{
	errorNumber = noErr;

	if (HasParameter(keyErrorNumber))
		errorNumber = (OSErr)ReadLong(keyErrorNumber);

	if (HasParameter(keyErrorString))
		ReadString(keyErrorString, errorString);
}

//----------------------------------------------------------------------------------------
// End of UAppleEvents.cp



