//----------------------------------------------------------------------------------------
// UClipboardMgr.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

// ACS

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

// MacApp

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDESKSCRAPVIEW__
#include "UDeskScrapView.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#ifndef __CClipboard_AC__
#include "CClipboard_AC.h"
#endif

// ANSI

TClipboardMgr* gClipboardMgr;					// declared in interface


//========================================================================================
// CLASS TClipboardMgr
//========================================================================================
#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TClipboardMgr);

//----------------------------------------------------------------------------------------
// TClipboardMgr constructor
//----------------------------------------------------------------------------------------


TClipboardMgr::TClipboardMgr(CClipboard_AC* itsClipboard) :
	TBehavior(kClipboardManager),	

	fClipContext(NULL), 
	fClipView(NULL),
	fClipWindow(NULL),
	fClipOrphanage(NULL),
	fGotClipType(false),
	fClipboard(itsClipboard)
{
	gClipboardMgr = this;
}

//----------------------------------------------------------------------------------------
// TClipboardMgr destructor
//----------------------------------------------------------------------------------------

TClipboardMgr::~TClipboardMgr()
{
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::AboutToLoseControl:
//----------------------------------------------------------------------------------------

void TClipboardMgr::AboutToLoseControl(bool convertClipboard)
{
	if (convertClipboard)
	{
		Changed(mAboutToLoseControl,this);

		if (fClipView && !fClipWrittenToDeskScrap)
		{
			long err = fClipboard->ZeroScrap();

			try
			{
				fClipView->WriteToDeskScrap();
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);


#if qDebugMsg
				ProgramReport_AC("Can't use clipboard data outside this app\n");
#endif
				if (!theException.GetMessage())
					theException.SetMessage( messageExportClipFailed);

				if (gDispatcher)
					gDispatcher->ShowError(theException.GetError(), theException.GetMessage());
			}
			

			fClipWrittenToDeskScrap = true;
			AbsorbScrapStuff();
		}
	}
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::AbsorbScrapStuff:
//----------------------------------------------------------------------------------------


void TClipboardMgr::AbsorbScrapStuff()
{
	fClipboard->AbsorbScrapStuff();
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::CheckDeskScrap:
//----------------------------------------------------------------------------------------


void TClipboardMgr::CheckDeskScrap()
{
	OSErr err;

	AbsorbScrapStuff();

	if (!fClipboard->ScrapCountStillValid(fClipboard->GetOldScrapCount()))
	{
		Changed(mScrapChanged,this);
		
		fClipView->FreeFromClipboard();			// AbandonCurrentClipboard 
		fClipView = NULL;						// no reason to have an Undo clipboard 

		// Reset the document of the clip window.
		if (fClipWindow)
			fClipWindow->SetDocument(NULL);

		// If the scrap is in memory and we are low on memory, then write
        // the scrap to disk.
		if (fClipboard->IsScrapInMemory() && MemSpaceIsLow())
			err = (OSErr) fClipboard->UnloadScrap();			// Write the scrap to disk.
		
		ReadFromDeskScrap();
	}
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::GetDataToPaste:
//----------------------------------------------------------------------------------------


long TClipboardMgr::GetDataToPaste(Handle aDataHandle,
										  ResType& dataType)
{
	long length = -1;

	if (fGotClipType)
	{
		dataType = fPrefClipType;
		length = fClipView->GivePasteData(aDataHandle, dataType);
		if (length < 0)
			::ThrowException_AC((OSErr)length, 0);
	}
#if qDebugMsg
	else
	{
		::ProgramBreak_AC("GetDataToPaste called when fGotClipType was false");
	}
#endif

	return length;
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::Launch:
//----------------------------------------------------------------------------------------


void TClipboardMgr::Launch()
{
	// Create the clip window and default view 
	fClipWindow = MakeClipboardWindow();
	::ThrowIfNULL_AC(fClipWindow);
	fClipOrphanage = fClipWindow->FindSubView(kIDClipView);
	::ThrowIfNonObject_AC(fClipOrphanage);

	// Add the clipboard manager as an application behavior
	gDispatcher->AddBehavior(this);

	AbsorbScrapStuff();
	ReadFromDeskScrap();
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::MakeClipboardWindow:
//----------------------------------------------------------------------------------------


TWindow* TClipboardMgr::MakeClipboardWindow()
{
	TWindow * aWindow;

	if (qTemplateViews)
		aWindow = TViewServer::fgViewServer->NewTemplateWindow(kIDClipWindow, NULL);
	else
	{
		TDeskScrapView * aDeskScrapView = TH_new TDeskScrapView;
		aDeskScrapView->SetIdentifier(kIDClipView);
		aWindow = TViewServer::fgViewServer->NewSimpleWindow(kIDClipWindow, true, true, aDeskScrapView);
	}
	
	if (aWindow)
	{
		aWindow->SetHideOnSuspend(true);
		aWindow->SetHandlesCursor(false);
		aWindow->SetLetsSubViewsHandleCursor(false);
		aWindow->SetClosesDocument(false);
	}
	return aWindow;
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::ReadFromDeskScrap:
//----------------------------------------------------------------------------------------


void TClipboardMgr::ReadFromDeskScrap()
{
	TView* aViewForClipboard;
	
	try
	{
		aViewForClipboard = MakeViewForAlienClipboard();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		aViewForClipboard = fClipOrphanage;
		if (!theException.GetMessage())
			theException.SetMessage( messageImportClipFailed );

		if (gDispatcher)
			gDispatcher->ShowError(theException.GetError(), theException.GetMessage());
	}
	

	SetClipView(aViewForClipboard,NULL);
	fClipWrittenToDeskScrap = true;	// We're in synch with the scrap
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::MakeViewForAlienClipboard:
//----------------------------------------------------------------------------------------


TView* TClipboardMgr::MakeViewForAlienClipboard()
{
	TView * aView = gDispatcher->MakeViewForAlienClipboard();
	if (!aView)
		aView = fClipOrphanage;

	::ThrowIfNonObject_AC(aView);

	return aView;
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::RegainControl:
//----------------------------------------------------------------------------------------


void TClipboardMgr::RegainControl(bool checkClipboard)
{
	if (checkClipboard)
		CheckDeskScrap();
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::SetClipView:
//----------------------------------------------------------------------------------------


void TClipboardMgr::SetClipView(TView* clipView, TCommandHandler* itsContext)
{
	if (clipView)
	{
		if (fClipWindow)
		{
			// get the old clip window scroller if possible
			TView * theSuperView = fClipWindow->HasSubViews() ? fClipWindow->FirstSubView() : fClipWindow.get();

			// Set the document of the clip window and its subview.
			fClipWindow->SetDocument(clipView->GetDocument());
			theSuperView->SetDocument(clipView->GetDocument());
				
			// throw out any views contained in the scroller (or window)
			TView* theSubView;
			while ((theSubView = theSuperView->FirstSubView()) != NULL)
			{
				theSubView->ForceRedraw();
				if (fClipWindow->IsShown())
					theSubView->Close();
				theSuperView->RemoveSubView(theSubView);
			}
	
			// pop in the new clipview
			theSuperView->AddSubView(clipView);
			if (fClipWindow->IsShown())
				clipView->Open();
	
			clipView->AdjustFrame();
			clipView->RevealTop(kDontRedraw);
			clipView->ForceRedraw();
	
			fClipWindow->SetWindowTarget(fClipWindow);
			fClipWrittenToDeskScrap = clipView == fClipOrphanage;
		}
#if qDebug
		else
		{
			::ProgramBreak_AC("SetClipView in absence of fClipWindow");
		}
#endif

		clipView->SetEnable(false);					// Ignore clicks while in clipboard views
		Changed(mClipViewChanged, this);
		fClipView = clipView;
		fClipContext = itsContext;
		
		TDocument* itsDocument = clipView->GetDocument();
		if (itsDocument)
			itsDocument->SetIsGhostDocument(true);
	}
#if qDebug
	else
	{
		::ProgramBreak_AC("SetClipView: Attempt to set NULL clipView.");
	}
#endif
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::DoSetupMenus:
//----------------------------------------------------------------------------------------


void TClipboardMgr::DoSetupMenus()
{
	Inherited::DoSetupMenus();

	Enable(cShowClipboard, !gDispatcher->InModalState());
	SetMenuState(cShowClipboard, kIDBuzzString, bzShowClip, bzHideClip, fClipWindow == gDispatcher->GetActiveWindow(kNoFloaters));
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::CanPaste:
//----------------------------------------------------------------------------------------


void TClipboardMgr::CanPaste(ResType aClipType)
{
	if (fClipView && !fGotClipType)
		if (fClipView->ContainsClipType(aClipType))
		{
			fGotClipType = true;
			fPrefClipType = aClipType;
		}

	if (!gDispatcher->GetSysWindowActive())
		Enable(cPaste, fGotClipType);
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::DoMenuCommand:
//----------------------------------------------------------------------------------------


void TClipboardMgr::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber)
	{
		case cShowClipboard:
			if (fClipWindow == gDispatcher->GetActiveWindow(kNoFloaters))
			{
				fClipWindow->Close();	// Hide the clipboard
				if (fClipWindow->GetFreeOnClosing())
					fClipWindow.reset();
			}
			else
			{
				fClipWindow->Open();
				fClipWindow->Select();
			}
			break;
		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::Close:
//----------------------------------------------------------------------------------------

void TClipboardMgr::Close()
{
	fClipboard->LoadScrap();
}

//----------------------------------------------------------------------------------------
// TClipboardMgr::PutDeskScrapData:
//----------------------------------------------------------------------------------------

OSErr TClipboardMgr::PutDeskScrapData(ResType aResType,
									  Handle aDataHandle)
{
	CTempHandleLock_AC tempHandleLock(aDataHandle);
	long err = fClipboard->PutScrap(GetHandleSize(aDataHandle), aResType, (*aDataHandle));
#if qDebugMsg
	if (err != noErr)
		ProgramReport_AC(false, "Error from PutScrap is: %d\n", err);
#endif
	return err;
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// InitUClipboardMgr:
//----------------------------------------------------------------------------------------

void InitUClipboardMgr()
{
	TClipboardMgr * aClipboardMgr = TH_new TClipboardMgr(TH_new CClipboard_AC);

#if qTemplateViews
	MA_REGISTER_CLASS(TDeskScrapView);
#endif
}

//----------------------------------------------------------------------------------------
// End of UClipboardMgr.cp


