//----------------------------------------------------------------------------------------
// UControl.cp
// Copyright  1987-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCONTROL__
#include "UControl.h"
#endif

// ACS

#ifndef __CIterator__
#include "CIterator_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// ACS Appearance

#if qGrayscaleAppearance

#	ifndef __AGA_AC__
#	include "AGA_AC.h"
#	endif

#endif // qGrayscaleAppearance

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif


// MacApp

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDRAWINGENVIRONMENT__
#include "UDrawingEnvironment.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

// Use Windows_AC.h instead of <windows.h> for XPlatform compatability.
#ifndef __Windows_AC__
#include "Windows_AC.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif


#ifndef __STDLIB__
#include <stdlib.h>
#endif

#if qWantsAppearance
#	ifndef __APPEARANCE__
#	include <Appearance.h>
#	endif
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static pascal void ActionProcForTScrollBar(ControlRef aCMgrControl,
										   short partCode);

//========================================================================================
// CLASS CWhileFocused
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// CWhileFocused::CWhileFocused:
//----------------------------------------------------------------------------------------


CWhileFocused::CWhileFocused(TCtlMgr * itsCtlMgr, bool redraw) :
	fCtlMgr(itsCtlMgr),
	fRedraw(redraw)
{
	fDidFocus = false;
	if (fCtlMgr->fCMgrControl)
	{
		if (fRedraw && fCtlMgr->Focus())
		{
			fDidFocus = true;

			CRect_AC qdArea(fCtlMgr->ControlQDArea());
		
			bool wasVisible = fCtlMgr->SetCMgrVisibility(false);
			fOldControlLocation = ((CRect_AC&)(*fCtlMgr->fCMgrControl)->contrlRect)[topLeft]; // I'd rather have an accessor!!!
			MoveControl(fCtlMgr->fCMgrControl, qdArea.left, qdArea.top); //SRF
			fCtlMgr->SetCMgrVisibility(wasVisible);
		}
		else
			fWasVisible = fCtlMgr->SetCMgrVisibility(false);
	}
}

//----------------------------------------------------------------------------------------
// CWhileFocused::~CWhileFocused: 
//----------------------------------------------------------------------------------------


CWhileFocused::~CWhileFocused()
{
	if (fCtlMgr->fCMgrControl)
		if (fDidFocus)
		{
			bool wasVisible = fCtlMgr->SetCMgrVisibility(false);
			MoveControl(fCtlMgr->fCMgrControl, fOldControlLocation.h, fOldControlLocation.v); //SRF
			fCtlMgr->SetCMgrVisibility(wasVisible); 
		}
		else
			fCtlMgr->SetCMgrVisibility(fWasVisible);	// restore visibility 
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// ActionProcForTScrollBar:
//----------------------------------------------------------------------------------------


pascal void ActionProcForTScrollBar(ControlRef aCMgrControl,
									short partCode)
{
	TScrollBar * aScrollBar = reinterpret_cast<TScrollBar*>(GetControlReference(aCMgrControl));
	::ThrowIfNULL_AC(aScrollBar);						// What else you gonna' do? 

	aScrollBar->ActionProc(partCode);

}


//========================================================================================
// CLASS TControlTracker
//========================================================================================
#undef Inherited
#define Inherited TTracker

//----------------------------------------------------------------------------------------
// TControlTracker constructor
//----------------------------------------------------------------------------------------


TControlTracker::TControlTracker(TControl* theControl,
									  const CViewPoint& itsMouse) :
	TTracker(cTrackingControl, theControl, kCantUndo, kDoesNotCauseChange, NULL,
					theControl, theControl->GetScroller(kImmediateSuperView), itsMouse),

	fControl(theControl)
{

#if qDebug
	if (!theControl)
		::ProgramBreak_AC("Control passed to IControlTracker can''t be nil.");
#endif

	fTrackNonMovement = true;
	fViewConstrain = false;
}

//----------------------------------------------------------------------------------------
// TControlTracker destructor
//----------------------------------------------------------------------------------------


TControlTracker::~TControlTracker()
{
}

//========================================================================================
// CLASS TControl
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TControl);

//----------------------------------------------------------------------------------------
// TControl constructor
//----------------------------------------------------------------------------------------


TControl::TControl(const CViewPoint& itsLocation,
						const CViewPoint& itsSize,
						SizeDeterminer itsHSizeDet,
						SizeDeterminer itsVSizeDet,
						const CTextStyle_AC& itsTextStyle) :
	TView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fEventNumber(mOKHit),
	fTextStyleRsrcID(kNoResource),
	fTextStyle(itsTextStyle),
	fHilite(false),
	fDimmed(false),
	fSizeable(true),
	fPreferOutline(kDontPreferOutline)
{
#if qDebug
	if (itsSize.h > kMaxCoord || itsSize.v > kMaxCoord)
	{
		char msg[128];
		
		sprintf(msg, "The size in pixels of a TControl cannot exceed %ld\n", kMaxCoord);
		::ProgramBreak_AC(msg);
	}
#endif
	// By default, controls are assumed to require redrawing the entire frame when
	// the frame changes size.  Call SetFrameInvalidateDeterminer to change this default.
	fFrameInvalidateDeterminer = frameInvalidateWholeFrame;
}

//----------------------------------------------------------------------------------------
// TControl destructor
//----------------------------------------------------------------------------------------


TControl::~TControl()
{
}

//----------------------------------------------------------------------------------------
// TControl::GetStandardSignature:
//----------------------------------------------------------------------------------------


IDType TControl::GetStandardSignature() const	// override 
{
	return kStdControl;
}
//----------------------------------------------------------------------------------------
// TControl::ReadFields:
//----------------------------------------------------------------------------------------


void TControl::ReadFields(CStream_AC* aStream)		// override 
{
	Inherited::ReadFields(aStream);

	fEventNumber = aStream->ReadLong();
	
	// set various flags
	HiliteState(aStream->ReadBoolean(), kDontRedraw);
	DimState(aStream->ReadBoolean(), kDontRedraw);
	fSizeable = aStream->ReadBoolean();
	
	*aStream >> fInset;
	
//	fInset = aStream->ReadLongRect();
	
	fTextStyleRsrcID = aStream->ReadInteger();
	if (fTextStyleRsrcID != kNoResource)
		fTextStyle = MAGetTextStyle(fTextStyleRsrcID);
	
	fPreferOutline = aStream->ReadBoolean();

#if qDebug											
	if ((fSize.h > kMaxCoord) || (fSize.v > kMaxCoord))
	{
		char msg[128];
		
		sprintf(msg, "The size in pixels of a TControl cannot exceed %ld\n", kMaxCoord);
		::ProgramBreak_AC(gEmptyString);
	}
#endif
}

//----------------------------------------------------------------------------------------
// TControl::WriteFields:
//----------------------------------------------------------------------------------------


void TControl::WriteFields(CStream_AC* aStream) const		// override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteLong(fEventNumber);				// write its fEventNumber

	aStream->WriteBoolean(fHilite);
	aStream->WriteBoolean(fDimmed);
	aStream->WriteBoolean(fSizeable);

	*aStream << fInset;
	
//	aStream->WriteLongRect(fInset);					// write its inset

	aStream->WriteInteger(fTextStyleRsrcID);

	aStream->WriteBoolean(fPreferOutline);
}

//----------------------------------------------------------------------------------------
// TControl::ComputeFrame:
//----------------------------------------------------------------------------------------


void TControl::ComputeFrame(CViewRect& newFrame)// override
{
	Inherited::ComputeFrame(newFrame);
	if (!fSizeable)
	{
		// we need to adjust the bot/right offset
		// If view is going to change size, then doesn't change control's actual size
		fInset[botRight] += newFrame.GetSize() - fSize;
	}
}

//----------------------------------------------------------------------------------------
// TControl::ContainsMouse:
//----------------------------------------------------------------------------------------


bool TControl::ContainsMouse(const CViewPoint& theMouse)// override
{
	return ControlArea().Contains(theMouse) && Inherited::ContainsMouse(theMouse);
}

//----------------------------------------------------------------------------------------
// TControl::ControlArea:
//----------------------------------------------------------------------------------------


CViewRect TControl::ControlArea()
{
	CViewRect theArea(GetExtent());

	theArea.Inset(fInset);

	CntlAdornment itsAdornment;
	GetAdornment(itsAdornment);
	if (macroIn(itsAdornment, adnShadow))
	{
		CPoint_AC itsPenSize;
		GetPenSize(itsPenSize);
		theArea[botRight] -= CViewPoint(itsPenSize);
	}

	return theArea;
}

//----------------------------------------------------------------------------------------
// TControl::ControlQDArea:
//----------------------------------------------------------------------------------------


CRect_AC TControl::ControlQDArea()
{
	return ViewToQD(ControlArea());
}

//----------------------------------------------------------------------------------------
// TControl::Dim:
//----------------------------------------------------------------------------------------


void TControl::Dim()
{
#if qDebug
	AssumeFocused();
#endif

	PenPat(&qd.gray);
	PenMode(patBic);
	PaintRect(&ControlQDArea());
}

//----------------------------------------------------------------------------------------
// TControl::DimState:
//----------------------------------------------------------------------------------------


void TControl::DimState(bool state, bool redraw)
{
	if (state != fDimmed)
	{
		fDimmed = state;
		if (state)								// dim adorner draws the dim state
			AddAdorner(&TAdorner::fgDimAdorner, kAdornLast - 10, kDontRedraw);
		else
			DeleteAdorner(&TAdorner::fgDimAdorner, kDontRedraw);
		if (redraw)
			DrawContents();				// Draw change immediately
	}
}

//----------------------------------------------------------------------------------------
// TControl::DoMouseCommand:
//----------------------------------------------------------------------------------------


void TControl::DoMouseCommand(CViewPoint& theMouse,
							  TToolboxEvent* ,
							  CPoint_AC)		// override 
{
	PostCommand(TH_new TControlTracker(this, theMouse));
}

//----------------------------------------------------------------------------------------
// TControl::DoEvent:
//----------------------------------------------------------------------------------------


void TControl::DoEvent(EventNumber eventNumber,
					   TEventHandler* source,
					   TEvent* event)	// override 

{
	switch (eventNumber)
	{
		case mTurnOn:
			HiliteState(true,kRedraw);
			break;
		case mTurnOff:
			HiliteState(false,kRedraw);
			break;
		case mToggle:
			HiliteState(!fHilite,kRedraw);
			break;
		default:
			Inherited::DoEvent(eventNumber, source, event);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TControl::Flash:
//----------------------------------------------------------------------------------------


void TControl::Flash()
{
	long dontCare;

	HiliteState(true, kRedraw);
	Delay(8, &dontCare);
	HiliteState(false, kRedraw);
}

//----------------------------------------------------------------------------------------
// TControl::Focus:
//----------------------------------------------------------------------------------------


bool TControl::Focus()			// override 
{
	bool returnVal = Inherited::Focus();
	if (returnVal)
	{
		PenNormal();
		SetPortTextStyle(fTextStyle);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TControl::Hilite:
//----------------------------------------------------------------------------------------


void TControl::Hilite()
{
#if qDebug
	AssumeFocused();
#endif

	InvertRect(&ControlQDArea());
}

//----------------------------------------------------------------------------------------
// TControl::HiliteState:
//----------------------------------------------------------------------------------------


void TControl::HiliteState(bool state, bool redraw)
{
	if (state != fHilite)
	{
		fHilite = state;
		if (state)								// hilite adorner draws the hilite state
			AddAdorner(&TAdorner::fgHiliteAdorner, kAdornLast - 5, kDontRedraw);
		else
			DeleteAdorner(&TAdorner::fgHiliteAdorner, kDontRedraw);
		if (redraw && IsDrawable())
			Hilite();
	}
}

//----------------------------------------------------------------------------------------
// TControl::Inset:
//----------------------------------------------------------------------------------------


void TControl::Inset(const CViewPoint& delta, bool redraw)
{
	if (fSizeable)
	{
		fInset += delta;

		if (redraw)
			ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TControl::InstallColor:
//----------------------------------------------------------------------------------------


void TControl::InstallColor(const CRGBColor_AC& theColor, bool redraw)
{
	fTextStyle.tsColor = theColor;
	if (redraw)
		DrawContents();
}


//----------------------------------------------------------------------------------------
// TControl::InstallTextStyle:
//----------------------------------------------------------------------------------------


void TControl::InstallTextStyle(const CTextStyle_AC& theTextStyle, bool redraw)
{
	fTextStyle = theTextStyle;
	if (redraw)
		DrawContents();
}

//----------------------------------------------------------------------------------------
// TControl::IsDimmed:
//----------------------------------------------------------------------------------------


bool TControl::IsDimmed()
{
	return fDimmed;
}

//----------------------------------------------------------------------------------------
// TControl::GetAdornExtent:
//----------------------------------------------------------------------------------------


CViewRect TControl::GetAdornExtent()// override
{
	CViewRect itsAdornExtent = GetExtent();

	CntlAdornment itsAdornment;
	GetAdornment(itsAdornment);
	if (macroIn(itsAdornment, adnShadow))
	{
		CPoint_AC itsPenSize;
		GetPenSize(itsPenSize);
		itsAdornExtent[botRight] -= CViewPoint(itsPenSize);
	}

	return itsAdornExtent;
}

//----------------------------------------------------------------------------------------
// TControl::GetEventNumber:
//----------------------------------------------------------------------------------------


EventNumber TControl::GetEventNumber()	// Override
{
	return fEventNumber;
}

//----------------------------------------------------------------------------------------
// TControl::SetAdornment:
//----------------------------------------------------------------------------------------


void TControl::SetAdornment(/*CntlAdornment*/ short newAdornment, bool redraw)
//!!! Using short rather than CntlAdornment because of a compiler bug
{
	TCompatibilityAdorner* itsAdorner = dynamic_cast<TCompatibilityAdorner*>(AdornerWithID(kCompatibilityAdorner));
	if (!itsAdorner) 
	{
		if (newAdornment)
		{
			itsAdorner = TH_new TCompatibilityAdorner(newAdornment);
			AddAdorner(itsAdorner, kAdornAfter, redraw);
		}
	}
	else
	{
		itsAdorner->fAdornment = newAdornment;
		if (redraw)
			ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TControl::GetAdornment:
//----------------------------------------------------------------------------------------


void TControl::GetAdornment(CntlAdornment& itsAdornment)
{
	TAdorner* itsAdorner = AdornerWithID(kCompatibilityAdorner);
	itsAdornment = 0; 
	if (itsAdorner)
		itsAdornment = dynamic_cast<TCompatibilityAdorner*>(itsAdorner)->fAdornment;
}

//----------------------------------------------------------------------------------------
// TControl::SetPenSize:
//----------------------------------------------------------------------------------------


void TControl::SetPenSize(CPoint_AC newPenSize)
{
	if (newPenSize != CPoint_AC(1,1))
	{
		CreateDrawingEnvironment();
		fDrawingEnvironment->fPenSize = newPenSize;
	}
}

//----------------------------------------------------------------------------------------
// TControl::GetPenSize:
//----------------------------------------------------------------------------------------


void TControl::GetPenSize(CPoint_AC& itsPenSize)
{
	if (fDrawingEnvironment)
		itsPenSize = fDrawingEnvironment->fPenSize;
	else
		itsPenSize.Set(1,1);
}

//----------------------------------------------------------------------------------------
// TControl::SetInset:
//----------------------------------------------------------------------------------------


void TControl::SetInset(const CViewRect& newInset, bool redraw)
{
	if (fSizeable)
	{
		fInset = newInset;

		// Resize(fSize.Copy(), redraw);
		if (redraw)
			ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TControl::TrackFeedback:
//----------------------------------------------------------------------------------------


void TControl::TrackFeedback(TrackPhase /* aTrackPhase */,
							 const CViewPoint& /* anchorPoint */,
							 const CViewPoint& /* previousPoint */,
							 const CViewPoint& /* nextPoint */,
							 bool /* mouseDidMove */,
							 bool /* turnItOn */)

{
	// do nothing
}

//----------------------------------------------------------------------------------------
// TControl::TrackMouse:
//----------------------------------------------------------------------------------------


void TControl::TrackMouse(TrackPhase aTrackPhase,
						  CViewPoint& /* anchorPoint */,
						  CViewPoint& /* previousPoint */,
						  CViewPoint& nextPoint,
						  bool /* mouseDidMove */)
{
	switch (aTrackPhase)
	{
		case trackBegin:
			HiliteState(true, kRedraw);
			break;
		case trackContinue:
			HiliteState(ContainsMouse(nextPoint), kRedraw);
			break;
		case trackEnd:
			if (ContainsMouse(nextPoint))
			{
				switch (fEventNumber)
				{
					// The defchoice determines different kinds of tracking behavior
					case mCheckBoxHit:
						// Toggle highlighting
						HandleEvent(mToggle,this,NULL);
						break;
					case mRadioHit:
						// Once it's on, it's on, until a cluster turns it off
						HandleEvent(mTurnOn,this,NULL);
						break;
					default:
						// Button action. Turn off highlighting.
						HandleEvent(mTurnOff,this,NULL);
						break;
				}
				HandleEvent(fEventNumber, this, NULL);
			}
			break;
	}
}


//========================================================================================
// CLASS TCtlMgr
//========================================================================================
#undef Inherited
#define Inherited TControl


MA_DEFINE_CLASS(TCtlMgr);

//----------------------------------------------------------------------------------------
// TCtlMgr constructor
//----------------------------------------------------------------------------------------


TCtlMgr::TCtlMgr(const CViewPoint& itsLocation,
					  const CViewPoint& itsSize,
					  SizeDeterminer itsHSizeDet,
					  SizeDeterminer itsVSizeDet,
					  const CStr255_AC& itsTitle,
					  long itsVal,
					  long itsMin,
					  long itsMax,
					  short itsProcID,
					  const CTextStyle_AC& itsTextStyle) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle),
	fTitle(itsTitle.Length() ? (CStr255_AC *)operator new(itsTitle.Length() + kLengthByte) : NULL),

	fCMgrControl(NULL),

	// These MUST be initialized because SetLongValues tests each new value against 
	// the old value before doing anything. If the fLongMax value should happen to
	// have a (bogus - uninitialized) value that matches itsMax, fBitsToShift won't 
	// be initialized and the control manager control will behave erratically.
	fBitsToShift(0),
	fLongVal(itsVal),
	fLongMin(itsMin),
	fLongMax(itsMax),
	fProcID(itsProcID)
{
	if (fTitle)
		*fTitle = itsTitle;
}

//----------------------------------------------------------------------------------------
// TCtlMgr copy constructor
//----------------------------------------------------------------------------------------


TCtlMgr::TCtlMgr(const TCtlMgr& inCtlMgr):
	fTitle(inCtlMgr.fTitle.get()),

	fCMgrControl(NULL),
	fBitsToShift(inCtlMgr.fBitsToShift),
	fLongVal(inCtlMgr.fLongVal),
	fLongMin(inCtlMgr.fLongMin),
	fLongMax(inCtlMgr.fLongMax),
	fProcID(inCtlMgr.fProcID)
{
	if (inCtlMgr.fCMgrControl)
	{
		// Copy the control values to locals and then set the values in the TCtlMgr to
        // zero. The SetLongxxx routines check to see if the control values have changed
        // before actual changing the toolbox control.
		
		fLongMin = fLongMax = fLongVal = 0;

		CStr255_AC contrlTitle;
		::GetControlTitle(inCtlMgr.fCMgrControl, contrlTitle);

		CreateCMgrControl(contrlTitle, inCtlMgr.fLongVal, inCtlMgr.fLongMin, inCtlMgr.fLongMax, fProcID);
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::Free:
//----------------------------------------------------------------------------------------


TCtlMgr::~TCtlMgr()
{
	if (fCMgrControl)
	{
		SetCMgrVisibility(false);			// This insures that HideControl, which
                                                // will be called by SizeControl, doesn't
                                                // do anything. HideControl seems to have
                                                // some problems with large (~>
                                                // 14000x14000)
		SizeControl(fCMgrControl, 0, 0);		// Prevent CMgr from erasing the control!
		DisposeControl(fCMgrControl);
		fCMgrControl = NULL;					// So BeInPort and others will be happy!
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::DoPostCreate:
//----------------------------------------------------------------------------------------


void TCtlMgr::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);
	
	// Scrollers that auto-create scroll bars have HandlePostCreate called on the scroll bars.
	// This code ensures that the controls are created, since ReadFields won't get called on them.
	CreateCMgrControl(fTitle ? *fTitle : (CStr255_AC)gEmptyString, fLongVal, fLongMin, fLongMax, fProcID);
}


//----------------------------------------------------------------------------------------
// TCtlMgr::BeInPort:
//----------------------------------------------------------------------------------------


void TCtlMgr::BeInPort(GrafPtr itsPort)	// override 
{
	Inherited::BeInPort(itsPort);
	
	if (fCMgrControl)
	{
		WindowRef itsNewControlOwner;
		if (itsPort)
		{
			SetCMgrVisibility(fShown); 
			itsNewControlOwner = (WindowRef)itsPort;
		}
		else
		{
			SetCMgrVisibility(false);
			itsNewControlOwner = (WindowRef)gWorkPort;
		}
		
		//
		// Get the control pointing at the owning window
		//
		WindowRef itsOldControlOwner = (*fCMgrControl)->contrlOwner;
		(*fCMgrControl)->contrlOwner = gWorkPort;	// in case it's not anywhere else
		
		//
		// Get the control off the control list of the old owner
		//
		if (itsOldControlOwner)
		{
			// first control on list
			if (((WindowPeek)itsOldControlOwner)->controlList == fCMgrControl)
			{
				((WindowPeek)itsOldControlOwner)->controlList = (*fCMgrControl)->nextControl;
			}
			else
			{
				ControlHandle currentControl = ((WindowPeek)itsOldControlOwner)->controlList;
				while (currentControl)
				{
					if ((*currentControl)->nextControl == fCMgrControl)
					{
						(*currentControl)->nextControl = (*fCMgrControl)->nextControl;
						break;
					}
					currentControl = (*currentControl)->nextControl;
				}
			}
		}
		
		//
		// and on to the control list of the new owner
		//
		if (itsNewControlOwner)
		{
			TWindow* itsWindow = GetWindow();
		 	if (!(itsWindow && itsWindow->GetWindowRef() == itsNewControlOwner))
		 		itsNewControlOwner = gWorkPort; // guaranteed to be a valid window!
		 	
			(*fCMgrControl)->nextControl = ((WindowPeek)itsNewControlOwner)->controlList;
			((WindowPeek)itsNewControlOwner)->controlList = fCMgrControl;

			(*fCMgrControl)->contrlOwner = itsNewControlOwner;
		}

	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::CreateCMgrControl:
//----------------------------------------------------------------------------------------


void TCtlMgr::CreateCMgrControl(const CStr255_AC& itsTitle,
								long itsVal,
								long itsMin,
								long itsMax,
								short itsProcID)
{
	if (!fCMgrControl)
	{
		TWindow* itsTWindow = GetWindow();
		WindowRef itsWindow = itsTWindow ? itsTWindow->GetWindowRef() : gWorkPort;

		fProcID = itsProcID;

#if qGrayscaleAppearance
		if (IsEmulatingControls())
		{		
			if (!GetAGAControl())
			{
				CreateAGAControl();
				
				if (GetAGAControl())
					GetAGAControl()->SetTitle(itsTitle, kDontRedraw);
			}
		}
		else
#endif
		{
			ControlRef aCMgrControl = NewControl(itsWindow, ControlQDArea(), itsTitle, false, 0, 0, 0, itsProcID, reinterpret_cast_AC(long, this));
			::ThrowIfNULL_AC(aCMgrControl);
			fCMgrControl = aCMgrControl;
		}	
		DimState(fDimmed, kDontRedraw);
		HiliteState(fHilite, kDontRedraw);

		fLongMax = fLongMin = fLongVal = 0; // so we get fresh values
		SetLongValues(itsVal, itsMin, itsMax, kDontRedraw);

		SetCMgrVisibility(itsWindow != gWorkPort);

		fTitle.reset(); // whatever stored title there was has been used so we don't need it here anymore
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::InstallColor:
//----------------------------------------------------------------------------------------


void TCtlMgr::InstallColor(const CRGBColor_AC& theColor, bool redraw)
{
	if ((qNeedsColorQD || HasColorQD_AC()) && fCMgrControl)
	{
		// retrieve the control's color table
		AuxCtlHandle itsAuxCtlHandle;
		bool hasAuxCtl = GetAuxiliaryControlRecord(fCMgrControl, &itsAuxCtlHandle);
		::ThrowIfNULL_AC(itsAuxCtlHandle);
		CCTabHandle colorTableHandle = (*itsAuxCtlHandle)->acCTable;
		::ThrowIfNULL_AC(colorTableHandle);
		
		// determine if the control color table needs to be changed
		bool changeColorTable = true;

		// The ctSize field defines the number of elements in the table, minus one.
		for (short i = 0; i <= (*colorTableHandle)->ctSize; ++i)
		{
			if ((*colorTableHandle)->ctTable[i].value == cFrameColor)
			{
				changeColorTable = CRGBColor_AC((*colorTableHandle)->ctTable[i].rgb) != theColor;
				break;
			}
		}

		// modify the color table if necessary
		if (changeColorTable)
		{
			CCTabHandle newColorTableHandle = (CCTabHandle)PermCopyHandle((Handle) colorTableHandle);
	
			// Parse the color table, and make the appropriate settings.
			// The ctSize field defines the number of elements in the table, minus one.
			for (short i = 0; i <= (*newColorTableHandle)->ctSize; ++i)
			{
				switch ((*newColorTableHandle)->ctTable[i].value)
				{
					case cFrameColor:
					case cTextColor:
						(*newColorTableHandle)->ctTable[i].rgb = theColor;
						break;

					case cThumbColor:
					case cBodyColor:
						break;	// do nothing - InstallColor only affects the "fore" color
				}
			}
	
			// set the new color table
			CWhileFocused itsWhileFocused(this, redraw);
			CWhileOutlinePreferred_AC setOP(fPreferOutline);

			SetControlColor(fCMgrControl, newColorTableHandle);
		}
	}

	Inherited::InstallColor(theColor, redraw);
}


//----------------------------------------------------------------------------------------
// TCtlMgr::DimState:
//----------------------------------------------------------------------------------------


void TCtlMgr::DimState(bool state, bool redraw)	// override 
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		if (GetAGAControl() != NULL)
			GetAGAControl()->SetEnable(!state, redraw && Focus() && IsVisible());
	}
	else
#endif // qGrayscaleAppearance
	if (fCMgrControl && (*fCMgrControl)->contrlHilite != (state ? 255 : 0))
	{
		CWhileFocused itsWhileFocused(this, redraw);
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		HiliteControl(fCMgrControl, state ? 255 : 0);
	}
	fDimmed = state;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::DoMouseCommand:
//----------------------------------------------------------------------------------------


void TCtlMgr::DoMouseCommand(CViewPoint& theMouse,
							 TToolboxEvent* /* event */,
							 CPoint_AC /* hysteresis */)		// override
{
	CPoint_AC theQDMouse(ViewToQD(theMouse));

#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		// Let the CControl_AC track the mouse; if tracking succeeds
		// handle our event number.
		if (GetAGAControl() != NULL)
			if (GetAGAControl()->TrackMouse(theQDMouse))
				HandleEvent(fEventNumber, this, NULL);
	}
	else
#endif // qGrayscaleAppearance
	if (fCMgrControl)
	{
		CWhileFocused itsWhileFocused(this, kRedraw);
		if (TestControl(fCMgrControl, theQDMouse))
		{
			CWhileOutlinePreferred_AC setOP(fPreferOutline);

			short trackResult = TrackControl(fCMgrControl, theQDMouse, (ControlActionUPP)(-1));
			if (trackResult)
			{
				long newLongValue = GetVal() == GetMax() ? fLongMax : static_cast<long>(GetVal()) << fBitsToShift;
				SetLongVal(newLongValue, kRedraw);
				
				HandleEvent(fEventNumber, this, NULL);

				gDispatcher->InvalidateMouseRegions();
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::Draw:
//----------------------------------------------------------------------------------------


void TCtlMgr::Draw(const CViewRect& area)	// override 
{
	static controlPrintingReentrancyFlag;
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		// Let the CControl_AC draw itself.
		if (GetAGAControl() != NULL)
			GetAGAControl()->DrawObject();
	}
	else 
#endif // qGrayscaleAppearance	
	if (IsCMgrVisible())
	{
#if qDebug
		AssumeFocused();
#endif
		// for controls, spool the picture and draw that to overcome font problems
		
		// HOWEVER, controls are often drawn dimmed with grayisTextOR and I quote from IM Text on TextMode:
		// "The grayishTextOr transfer mode draws dimmed text on the screen. You can use it for black-and-white
		// or color graphics ports. The grayishTextOr transfer mode is not considered a standard transfer mode
		// because currently it is not stored in pictures, and printing with it is undefined. (It does not pass
		// through the QuickDraw bottleneck routines.)"
		
		bool spooledPrintOK = false;
		PicHandle aPH = NULL;
		if (gPrinting && !controlPrintingReentrancyFlag)
		{
			GrafPtr printPort = qd.thePort;
			// spool picture, take care as this will call TCtlMgr.Draw()
			controlPrintingReentrancyFlag = true;
			aPH = AsPict();
			controlPrintingReentrancyFlag = false;
			// if success, print this picture
			if (aPH)
			{
				SetPort(printPort);
				DrawPicture(aPH, &GetQDExtent());
				DisposeIfPicHandle(aPH);
				spooledPrintOK = true;
			}
		}
		
		// if we did not spool to a pict (successfully) use Draw1Control()
		if (!spooledPrintOK)
		{
			// save/set/reset the port in case we're imaging in some other port 
			WindowRef savedOwner = (*fCMgrControl)->contrlOwner;
			(*fCMgrControl)->contrlOwner = (WindowRef)qd.thePort;	//NOTE: !!! Still needs proper fix, SRF
		
			CWhileFocused itsWhileFocused(this, kRedraw);
			CWhileOutlinePreferred_AC setOP(fPreferOutline);

			PenNormal();
			if (gPrinting && fDimmed)
			{
				// grayishTextOr is not defined for printers or for Pictures (blech!)
				DimState(false, kRedraw);
				TControl::Dim();	// let our ancestor do it the old dumb way

				// restore things
				PenNormal();
				DimState(fDimmed, kDontRedraw);
			}
			else
				Draw1Control(fCMgrControl);
	
			// restore control owner
			(*fCMgrControl)->contrlOwner = savedOwner;
		}
	}

	Inherited::Draw(area);						// to get adornment 
}

//----------------------------------------------------------------------------------------
// TCtlMgr::IsCMgrVisible:
//----------------------------------------------------------------------------------------


bool TCtlMgr::IsCMgrVisible()
{
	return (fCMgrControl && ((*fCMgrControl)->contrlVis == 255));
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetMax:
//----------------------------------------------------------------------------------------


short TCtlMgr::GetMax()
{
	return fCMgrControl ? GetControlMaximum(fCMgrControl) : 0;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetMin:
//----------------------------------------------------------------------------------------


short TCtlMgr::GetMin()
{
	return fCMgrControl ? GetControlMinimum(fCMgrControl) : 0;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetText:
//----------------------------------------------------------------------------------------


CStr255_AC TCtlMgr::GetText() const
{
	CStr255_AC returnVal;

	if (fCMgrControl)
		GetControlTitle(fCMgrControl, returnVal);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetVal:
//----------------------------------------------------------------------------------------


short TCtlMgr::GetVal() const
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		return GetAGAControl() ? GetAGAControl()->GetValue() : 0;
	}
	else
#endif
	return fCMgrControl ? GetControlValue(fCMgrControl) : 0;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetVariant:
//----------------------------------------------------------------------------------------


short TCtlMgr::GetVariant()
{
	return fCMgrControl ? GetControlVariant(fCMgrControl) : 0;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::HiliteState:
//----------------------------------------------------------------------------------------


void TCtlMgr::HiliteState(bool state, bool redraw)	// override 
{
	// if the control is dimmed, leave it alone!
	if (fCMgrControl && (*fCMgrControl)->contrlHilite != 255 && fHilite != (*fCMgrControl)->contrlHilite != (state * 10))
	{
		CWhileFocused itsWhileFocused(this, redraw);
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		HiliteControl(fCMgrControl, state * 10);
	}
	fHilite = state;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::Inset:
//----------------------------------------------------------------------------------------


void TCtlMgr::Inset(const CViewPoint& delta, bool redraw)	// override 
{
	Inherited::Inset(delta, kDontRedraw);
	
	AdjustCtlMgrControl(redraw);
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetInset:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetInset(const CViewRect& newInset, bool redraw)
{
	Inherited::SetInset(newInset, kDontRedraw);
	
	AdjustCtlMgrControl(redraw);
}

//----------------------------------------------------------------------------------------
// TCtlMgr::AdjustCtlMgrControl:
//----------------------------------------------------------------------------------------


void TCtlMgr::AdjustCtlMgrControl(bool redraw)
{
	if (fCMgrControl)
	{
		CWhileFocused itsWhileFocused(this, redraw);
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		CRect_AC qdArea(ControlQDArea());
	
		MoveControl(fCMgrControl, qdArea.left, qdArea.top);
		if (fSizeable)
			SizeControl(fCMgrControl, abs(qdArea.GetLength(hSel)), abs(qdArea.GetLength(vSel)));
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetFrame:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetFrame(const CViewRect& newFrame, bool invalidate)	// override 
{
	CViewRect oldFrame(GetFrame());

	Inherited::SetFrame(newFrame, invalidate);

	if (newFrame != oldFrame)
		AdjustCtlMgrControl(invalidate);

#if qGrayscaleAppearance
	if (GetAGAControl() != NULL)
		GetAGAControl()->SetBounds(ControlQDArea(), CControl_AC::kDontRedraw);
#endif // qGrayscaleAppearance
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetCMgrVisibility:
//----------------------------------------------------------------------------------------


bool TCtlMgr::SetCMgrVisibility(bool beVisible)
{
	bool returnVal = IsCMgrVisible();

	if (fCMgrControl)
		if (beVisible)
			(*fCMgrControl)->contrlVis = 255;
		else
			(*fCMgrControl)->contrlVis = 0;
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::Show:
//----------------------------------------------------------------------------------------


void TCtlMgr::Show(bool state, bool redraw)
{
	SetCMgrVisibility(state);
	
	Inherited::Show(state, redraw);
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetMax:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetMax(short itsMax, bool redraw)
{
	if (fCMgrControl && GetControlMaximum(fCMgrControl) != itsMax)
	{
		CWhileFocused itsWhileFocused(this, redraw);
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		SetControlMaximum(fCMgrControl, itsMax);
	}

#if qGrayscaleAppearance
	if (GetAGAControl())
	{
		if (GetAGAControl()->GetMaximum() != itsMax)
			GetAGAControl()->SetRange(GetAGAControl()->GetMinimum(), itsMax, redraw);
	}
#endif
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetMin:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetMin(short itsMin, bool redraw)
{
	if (fCMgrControl && GetControlMinimum(fCMgrControl) != itsMin)
	{
		CWhileFocused itsWhileFocused(this, redraw);
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		SetControlMinimum(fCMgrControl, itsMin);
	}

#if qGrayscaleAppearance
	if (GetAGAControl())
	{
		if (GetAGAControl()->GetMinimum() != itsMin)
			GetAGAControl()->SetRange(itsMin, GetAGAControl()->GetMaximum(), redraw);
	}
#endif
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetText:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetText(const CStr255_AC& itsText, bool redraw)
{
	if (fCMgrControl)
	{
		if (CompareStrings(GetText(), itsText))
		{
			CWhileFocused itsWhileFocused(this, redraw);
			CWhileOutlinePreferred_AC setOP(fPreferOutline);
	
			SetControlTitle(fCMgrControl, itsText);
			if (IsFocused())					// we were focused if redraw was true
			{
				CRect_AC r = (*(fCMgrControl))->contrlRect;

				// Because Control Manager invalidates it, but it was actually drawn.
				// Note: we only validate the area that is currently Drawable
				CTempRegion_AC rgnToValidate;
				RectRgn(rgnToValidate, &r);
				GrafPtr currentPort;
				GetPort(&currentPort);
				SectRgn(rgnToValidate, currentPort->clipRgn, rgnToValidate);
				Validate(rgnToValidate);
			}
		}
	}
	else
	{
		fTitle.reset(itsText.Length() ? (CStr255_AC *)operator new(itsText.Length() + kLengthByte) : NULL);

		if (fTitle)
			*fTitle = itsText;
	}

#if qGrayscaleAppearance
	if (GetAGAControl())
	{
		GetAGAControl()->SetTitle(itsText, IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
#endif
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetVal:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetVal(short newVal, bool redraw)
{
	if (fCMgrControl && GetControlValue(fCMgrControl) != newVal)
	{
		CWhileFocused itsWhileFocused(this, redraw);
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		SetControlValue(fCMgrControl, newVal);
	}
#if qGrayscaleAppearance
	if (GetAGAControl() != NULL)
		GetAGAControl()->SetValue(newVal, redraw);
#endif
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetValues:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetValues(short itsVal,
						short itsMin,
						short itsMax,
						bool redraw)
{
	if (fCMgrControl)
	{
		CWhileFocused itsWhileFocused(this, redraw);
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		SetControlMinimum(fCMgrControl, itsMin);
		SetControlMaximum(fCMgrControl, itsMax);
		SetControlValue(fCMgrControl, itsVal);
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetLongMax:
//----------------------------------------------------------------------------------------


ViewCoordinate TCtlMgr::GetLongMax()
{
	return fLongMax;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetLongMin:
//----------------------------------------------------------------------------------------


ViewCoordinate TCtlMgr::GetLongMin()
{
	return fLongMin;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetLongVal:
//----------------------------------------------------------------------------------------


ViewCoordinate TCtlMgr::GetLongVal() const
{
	return fLongVal;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetLongMax:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetLongMax(ViewCoordinate itsMax, bool redraw)
{
	if (itsMax != fLongMax)
	{
		fLongMax = itsMax;
		fBitsToShift = 0;
		while (itsMax > SHRT_MAX)
		{
			itsMax = ((itsMax) >> 1);
			++fBitsToShift;
		}
		SetMax(static_cast<short>(itsMax), redraw);
		if (fLongMax < fLongVal) {
			SetLongVal(fLongMax, redraw);
		}
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetLongMin:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetLongMin(ViewCoordinate itsMin, bool redraw)
{
	if (itsMin != fLongMin)
	{
		fLongMin = itsMin;
		SetMin(static_cast<short>(itsMin >> fBitsToShift), redraw);
		if (fLongMin > fLongVal) {
			SetLongVal(fLongMin, redraw);
		}
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetLongVal:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetLongVal(ViewCoordinate itsVal, bool redraw)
{
	itsVal = MinMax_AC(fLongMin, itsVal, fLongMax);
	if (itsVal != fLongVal)
	{
		fLongVal = itsVal;
		SetVal(static_cast<short>(itsVal >> fBitsToShift), redraw);
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetLongValues:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetLongValues(ViewCoordinate itsVal,
							ViewCoordinate itsMin,
							ViewCoordinate itsMax,
							bool redraw)
{
	SetLongMax(itsMax, redraw);			// Must be first to get fBitsToShift setup
	SetLongMin(itsMin, redraw);
	SetLongVal(itsVal, redraw);
}

//----------------------------------------------------------------------------------------
// TCtlMgr::GetHelpState:
//----------------------------------------------------------------------------------------


short TCtlMgr::GetHelpState()
{
	short helpState;
	
	if (IsEnabled())
	{
		short ctlValue = GetVal();
		if (ctlValue == 1)
			helpState = kHMCheckedItem;
		else
			if (ctlValue > 1)
				helpState = kHMOtherItem;
			else
				helpState = kHMEnabledItem;
	}
	else
		helpState = kHMDisabledItem;
	
	return helpState;
}

//----------------------------------------------------------------------------------------
// TCtlMgr::UpdateCoordinates:
//----------------------------------------------------------------------------------------


void TCtlMgr::UpdateCoordinates()// override 
{
	CPoint_AC oldQDOrigin(fQDOrigin);
	CViewPoint oldViewToQDOffset(fViewToQDOffset);

	Inherited::UpdateCoordinates();

	if (fQDOrigin != oldQDOrigin || fViewToQDOffset != oldViewToQDOffset)
		AdjustCtlMgrControl(kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TCtlMgr::Activate:
//----------------------------------------------------------------------------------------


void TCtlMgr::Activate(bool entering)
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{	
		if (GetAGAControl() != NULL)
			GetAGAControl()->SetActive(entering, Focus() && IsVisible());
	}
	else
#endif

#if qWantsAppearance
	if (::HasAppearanceMgr_AC() && fCMgrControl && !fDimmed && entering != IsControlActive(fCMgrControl))
	{
		CWhileFocused itsWhileFocused(this, IsVisible());
		if (entering)
			(void) ::ActivateControl(fCMgrControl);
		else
			(void) ::DeactivateControl(fCMgrControl);
	}
#endif

	Inherited::Activate(entering);
}

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TCtlMgr::Dim:
//----------------------------------------------------------------------------------------


void TCtlMgr::Dim()
{
	if (IsEmulatingControls())
	{
		if (GetAGAControl() != NULL)
			GetAGAControl()->SetEnable(CControl_AC::kDisabled, Focus() && IsVisible());
	}
	else
		Inherited::Dim();	
}

//----------------------------------------------------------------------------------------
// TCtlMgr::SetEnable:
//----------------------------------------------------------------------------------------


void TCtlMgr::SetEnable(bool state)
{
	Inherited::SetEnable(state);

	if (IsEmulatingControls())
	{
		if (GetAGAControl() != NULL)
			GetAGAControl()->SetEnable(state, CControl_AC::kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TCtlMgr::IsSuppressable:
//----------------------------------------------------------------------------------------


bool TCtlMgr::IsStandardControl() const
{
	return true;	// A CtlMgr control is standard, subclasses must override if otherwise
}

//----------------------------------------------------------------------------------------
// TCtlMgr::IsEmulatingControls:
//----------------------------------------------------------------------------------------


bool TCtlMgr::IsEmulatingControls() const 
{ 
	return EmulateControls_AC() || !IsStandardControl(); 
}

//----------------------------------------------------------------------------------------
// TCtlMgr::CreateAGAControl:
//----------------------------------------------------------------------------------------


void TCtlMgr::CreateAGAControl()
{
	// Subclass responsibility
}
		
//----------------------------------------------------------------------------------------
// MAGAControl::GetAGAControl:
//----------------------------------------------------------------------------------------


CControl_AC* TCtlMgr::GetAGAControl() const
{
	// Subclass responsibility
	return NULL;
}

#endif // qGrayscaleAppearance

//========================================================================================
// CLASS TScrollBar
//========================================================================================
#undef Inherited
#define Inherited TCtlMgr


MA_DEFINE_CLASS(TScrollBar);

//----------------------------------------------------------------------------------------
// TScrollBar constructor
//----------------------------------------------------------------------------------------


TScrollBar::TScrollBar(const CViewPoint& itsLocation,
							const CViewPoint& itsSize,
							SizeDeterminer itsHSizeDet,
							SizeDeterminer itsVSizeDet,
							VHSelect itsDirection,
							long itsVal,
							long itsMin,
							long itsMax) :
	TCtlMgr(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, gEmptyString, itsVal, itsMin, itsMax, scrollBarProc),

	fDirection(itsDirection)
{
	fEventNumber = itsDirection == hSel ? mHScrollBarHit : mVScrollBarHit;
	fHandlesFirstClicks = false;
	SetOpacity(true);
}

//----------------------------------------------------------------------------------------
// TScrollBar destructor
//----------------------------------------------------------------------------------------


TScrollBar::~TScrollBar()
{
}

//----------------------------------------------------------------------------------------
// TScrollBar::GetStandardSignature:
//----------------------------------------------------------------------------------------


IDType TScrollBar::GetStandardSignature() const	// override 
{
	return kStdScrollBar;
}

//----------------------------------------------------------------------------------------
// TScrollBar::ReadFields:
//----------------------------------------------------------------------------------------


void TScrollBar::ReadFields(CStream_AC* aStream)				// override 
{
	Inherited::ReadFields(aStream);
	
	fDirection = (VHSelect) aStream->ReadByte();

	long initialValue = aStream->ReadLong();
	long minimumValue = aStream->ReadLong();
	long maximumValue = aStream->ReadLong();

	CreateCMgrControl(gEmptyString, initialValue, minimumValue, maximumValue, scrollBarProc);
}

//----------------------------------------------------------------------------------------
// TScrollBar::WriteFields:
//----------------------------------------------------------------------------------------


void TScrollBar::WriteFields(CStream_AC* aStream) const		// override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteByte(fDirection);

	aStream->WriteLong(fLongVal);
	aStream->WriteLong(fLongMin);
	aStream->WriteLong(fLongMax);
}

//----------------------------------------------------------------------------------------
// TScrollBar::ActionProc:
//----------------------------------------------------------------------------------------


void TScrollBar::ActionProc(short partCode)
{
	if (partCode)
	{
		bool backwards = (partCode == kControlUpButtonPart || partCode == kControlPageUpPart);

		if ((backwards && fLongVal > fLongMin) || (!backwards && fLongVal < fLongMax))
			TrackScrollBar(partCode);

		Update();							// Make sure that we're in synch before
                                                // returning
		Focus();							// make sure i am looking at myself e.e. cummings
	}
}

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TScrollBar::IsEmulatingControls:
//----------------------------------------------------------------------------------------


bool TScrollBar::IsEmulatingControls() const
{
	return false;
}

#endif // qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TScrollBar::DeltaValue:
//----------------------------------------------------------------------------------------


void TScrollBar::DeltaValue(ViewCoordinate delta)
{
	if (delta)								// Ensure that delta does not cause an overflow (or underflow)
		SetLongVal(fLongVal + MinMax_AC(fLongMin - fLongVal, delta, fLongMax - fLongVal), fShown);
}

//----------------------------------------------------------------------------------------
// TScrollBar::DoMouseCommand:
//----------------------------------------------------------------------------------------


void TScrollBar::DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* /* event */,
								CPoint_AC /* hysteresis */)		// override
{
#if qDebug
	AssumeFocused();
#endif

	CPoint_AC theQDMouse(ViewToQD(theMouse));
	ViewCoordinate oldLongValue = fLongVal;

	CWhileFocused itsWhileFocused(this, kRedraw);

	switch (TestControl(fCMgrControl, theQDMouse))
	{
		case kControlUpButtonPart:
		case kControlDownButtonPart:
		case kControlPageUpPart:
		case kControlPageDownPart:
			// note: the braces defeat an xlC warning
			{
				TemplateTempRoutineDescriptor<ControlActionUPP> actionProc(NewControlActionProc(&ActionProcForTScrollBar));
				::ThrowIfNULL_AC(actionProc);
				TrackControl(fCMgrControl, theQDMouse, actionProc);
			}

			// This method MUST inform its superview of the change! Thanks to GLB!
			if (fLongVal != oldLongValue)
				HandleEvent(fEventNumber, this, NULL);
			break;
		case kControlIndicatorPart: // Thumb
			if (TrackControl(fCMgrControl, theQDMouse, NULL) == kControlIndicatorPart)
			{
				// If thumb is dragged to bottom of scroll bar then ensure that the new
                // long value is set to the maximum long value. We get killed by side
                // effects if we don't use a temporary variable here. This doesn't show up
                // in the TScrollerScrollBar class because this code is overridden, but it
                // will show up in derived classes that depend on this code
				ViewCoordinate newLongValue;

				if (GetVal() == GetMax())
					newLongValue = fLongMax;
				else
					newLongValue = static_cast<long>(GetVal()) << fBitsToShift;

				SetLongVal(newLongValue, kRedraw);

				// This method MUST inform its superview of the change! 
				if (fLongVal != oldLongValue)
					HandleEvent(fEventNumber, this, NULL);
			}
			break;
	}
}

//----------------------------------------------------------------------------------------
// TScrollBar::TrackScrollBar:
//----------------------------------------------------------------------------------------


void TScrollBar::TrackScrollBar(short partCode)
{
	DeltaValue((partCode == kControlPageUpPart) || (partCode == kControlUpButtonPart) ? -1 : 1);
}


//========================================================================================
// CLASS TScrollerScrollBar
//========================================================================================
#undef Inherited
#define Inherited TScrollBar


MA_DEFINE_CLASS(TScrollerScrollBar);

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::TScrollerScrollBar:
//----------------------------------------------------------------------------------------


TScrollerScrollBar::TScrollerScrollBar(const CViewPoint& itsLocation,
											const CViewPoint& itsSize,
											SizeDeterminer itsHSizeDet,
											SizeDeterminer itsVSizeDet,
											VHSelect itsDirection,
											long itsMax,
											TScroller* itsScroller) : 
	TScrollBar(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsDirection, 0, 0, itsMax)
{
	SetCMgrVisibility(IsActive());

	AttachScroller(itsScroller);
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar copy constructor
//----------------------------------------------------------------------------------------


TScrollerScrollBar::TScrollerScrollBar(const TScrollerScrollBar& inScrollerScrollBar) :
	fScrollers(inScrollerScrollBar.fScrollers)
{
#if qDebug
	if (inScrollerScrollBar.fScrollers.GetSize() != 1)
		::ProgramBreak_AC("###TScrollerScrollBar copy constructor: Unexpected number of scrollers!");
#endif
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::Free:
//----------------------------------------------------------------------------------------


TScrollerScrollBar::~TScrollerScrollBar()
{
	for (TemplateIterator_AC<TScroller*> iter(&fScrollers); iter; ++iter)
		if (iter->fScrollBars[fDirection] == this)
			iter->HaveScrollBar(NULL, fDirection);
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::GetStandardSignature:
//----------------------------------------------------------------------------------------


IDType TScrollerScrollBar::GetStandardSignature() const	// override 
{
	return kStdScrollerScrollBar;
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::ReadFields:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::ReadFields(CStream_AC* aStream)				// override 
{
	Inherited::ReadFields(aStream);

	SetCMgrVisibility(IsActive());
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::WriteFields:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::WriteFields(CStream_AC* aStream) const		// override 
{
	Inherited::WriteFields(aStream);

}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::Activate:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::Activate(bool entering)// override
{
	CWhileFocused itsWhileFocused(this, kRedraw);

	SetCMgrVisibility(entering);
	DrawContents();
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::AttachScroller:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::AttachScroller(TScroller* itsScroller)
{
	if (itsScroller)
	{
		// create a fake fIdentifier for TScrollerScrollBar, if needed
		if (fIdentifier == kNoIdentifier)
		{
			IDType tempID = itsScroller->GetIdentifier();
			Ptr hiBytePtr = (Ptr) &tempID;

			if (fDirection == vSel)
				*hiBytePtr = 'v';
			else
				*hiBytePtr = 'h';
			
			fIdentifier = tempID;
		}
	
		fScrollers.Insert(itsScroller);
		itsScroller->HaveScrollBar(this, fDirection);
	}
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::BeInPort:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::BeInPort(GrafPtr itsPort)// override
{
	Inherited::BeInPort(itsPort);

	SetCMgrVisibility(IsActive());
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::DoMouseCommand:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::DoMouseCommand(CViewPoint& theMouse,
										TToolboxEvent* event,
										CPoint_AC hysteresis)	// override 
{

#if qDebug
	AssumeFocused();
#endif

	CPoint_AC theQDMouse = ViewToQD(theMouse);

	CWhileFocused itsWhileFocused(this, kRedraw);

	if (TestControl(fCMgrControl, theQDMouse) == kControlIndicatorPart) // Thumb
	{
		if (TrackControl(fCMgrControl, theQDMouse, NULL) == kControlIndicatorPart)
		{
			// If thumb is dragged to bottom of scroll bar then ensure that
			// the new long value is set to the maximum long value.
			fLongVal = GetVal() == GetMax() ? fLongMax : GetVal() << fBitsToShift;

			long sBarDelta = 0;
			for (TemplateIterator_AC<TScroller*> iter(&fScrollers); iter; ++iter)
				sBarDelta += iter->ScrollRelative(fDirection, fLongVal);

			if (Focus() && sBarDelta)
				SetLongVal(fLongVal + sBarDelta, kRedraw);
		}
	}
	else
		Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::Draw:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::Draw(const CViewRect& area)// override
{
#if qDebug
	AssumeFocused();
#endif

	if (!IsCMgrVisible())
	{
		PenNormal();
		CRGBColor_AC savedBkColor(GetIfBkColor());
		SetIfBkColor(gRGBWhite);

		CWhileFocused itsWhileFocused(this, kRedraw);
		CRect_AC itsRect((*fCMgrControl)->contrlRect);

		EraseRect(itsRect);
		FrameRect(itsRect);
		SetIfBkColor(savedBkColor);
	}

	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBar::TrackScrollBar:
//----------------------------------------------------------------------------------------


void TScrollerScrollBar::TrackScrollBar(short partCode)
{
	long sBarDelta = 0;

	for (TemplateIterator_AC<TScroller*> iter(&fScrollers); iter; ++iter)
		sBarDelta += iter->ScrollStep(fDirection, partCode);

	if (Focus())
		DeltaValue(sBarDelta);
}

//----------------------------------------------------------------------------------------
// End of UControl.cp


