//----------------------------------------------------------------------------------------
// UDesignator.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

// ACS

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif

// MacApp

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

//========================================================================================
// CLASS TDesignator
//========================================================================================
#undef Inherited


MA_DEFINE_CLASS(TDesignator);

//----------------------------------------------------------------------------------------
// TDesignator::TDesignator
//----------------------------------------------------------------------------------------


TDesignator::TDesignator()
	: MScriptableObject(cSelection)
{
}

//----------------------------------------------------------------------------------------
// TDesignator destructor
//----------------------------------------------------------------------------------------


TDesignator::~TDesignator()
{
}

//----------------------------------------------------------------------------------------
// TDesignator::GetSize: 
//----------------------------------------------------------------------------------------


Size TDesignator::GetSize() const
{
	// SubClassResponsibility();
	return 0;
}

//----------------------------------------------------------------------------------------
// TDesignator::IsContained: 
//----------------------------------------------------------------------------------------


ContainmentType TDesignator::IsContained(TDesignator*/* aDesignator */ )
{
	// SubclassResponsibility();
	return kNotContained;
}

//----------------------------------------------------------------------------------------
// TDesignator::IsEmpty: 
//----------------------------------------------------------------------------------------


bool TDesignator::IsEmpty() const
{
	return GetSize() <= 0;				// empty if nothing designated
}


//========================================================================================
// CLASS TLinearDesignator
//========================================================================================
#undef Inherited
#define Inherited TDesignator


MA_DEFINE_CLASS(TLinearDesignator);

//----------------------------------------------------------------------------------------
// TLinearDesignator::TLinearDesignator
//----------------------------------------------------------------------------------------


TLinearDesignator::TLinearDesignator(long startPos,
												 long endPos) :
	fStartPos(startPos),
	fEndPos(endPos)
{
}

//----------------------------------------------------------------------------------------
// TLinearDesignator destructor
//----------------------------------------------------------------------------------------


TLinearDesignator::~TLinearDesignator()
{
}

//----------------------------------------------------------------------------------------
// TLinearDesignator::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TLinearDesignator::GetStandardSignature() const
{
	return kLinearDesignator;
}

//----------------------------------------------------------------------------------------
// TLinearDesignator::GetSize: 
//----------------------------------------------------------------------------------------


Size TLinearDesignator::GetSize() const
{
	return (Size)(fEndPos - fStartPos);
}

//----------------------------------------------------------------------------------------
// TLinearDesignator::ReadFrom: 
//----------------------------------------------------------------------------------------


void TLinearDesignator::ReadFrom(CStream_AC* aStream)
{
	MStreamable::ReadFrom(aStream);

	fStartPos = aStream->ReadLong();
	fEndPos = aStream->ReadLong();
}

//----------------------------------------------------------------------------------------
// TLinearDesignator::WriteTo: 
//----------------------------------------------------------------------------------------


void TLinearDesignator::WriteTo(CStream_AC* aStream) const// override 
{
	MStreamable::WriteTo(aStream);

	aStream->WriteLong(fStartPos);
	aStream->WriteLong(fEndPos);
}

//----------------------------------------------------------------------------------------
// TLinearDesignator::IsContained: 
//----------------------------------------------------------------------------------------


ContainmentType TLinearDesignator::IsContained(TDesignator* aDesignator)
{
	ContainmentType returnVal = kNotContained;

#if qDebug
	if (aDesignator && !dynamic_cast<TLinearDesignator*>(aDesignator))
		::ProgramBreak_AC("The containing designator for the TLinearDesignator is not of the same class");
#endif

	if (aDesignator)
	{
		TLinearDesignator* workDesignator = dynamic_cast_AC(TLinearDesignator*,aDesignator);
	
		long itsStartPos = workDesignator->fStartPos;
		long itsEndPos = workDesignator->fEndPos;
		
		// Are we designating the data in the other direction?  If so switch for the test.
		if (itsStartPos > itsEndPos)
		{
			itsStartPos = itsEndPos;
			itsEndPos = workDesignator->fStartPos;
		}
		
		if ((fStartPos == itsStartPos) && (fEndPos == itsEndPos))
			returnVal = kExactlyContained;
		else if ((fStartPos <= itsStartPos) && (fEndPos >= itsEndPos))
			returnVal = kFullyContained;
		else if (((fStartPos <= itsStartPos) && (fEndPos >= itsStartPos)) || ((fStartPos <= itsEndPos) && (fEndPos >= itsEndPos)))
			returnVal = kPartiallyContained;
	}

	return returnVal;
}


//========================================================================================
// CLASS TViewRectDesignator
//========================================================================================
#undef Inherited
#define Inherited TDesignator


MA_DEFINE_CLASS(TViewRectDesignator);

//----------------------------------------------------------------------------------------
// TViewRectDesignator::TViewRectDesignator: 
//----------------------------------------------------------------------------------------


TViewRectDesignator::TViewRectDesignator(const CViewRect& itsDesignation) :
	fDesignation(itsDesignation)
{
}

//----------------------------------------------------------------------------------------
// TViewRectDesignator destructor
//----------------------------------------------------------------------------------------


TViewRectDesignator::~TViewRectDesignator()
{
}

//----------------------------------------------------------------------------------------
// TViewRectDesignator::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TViewRectDesignator::GetStandardSignature() const
{
	return kVRectDesignator;
}

//----------------------------------------------------------------------------------------
// TViewRectDesignator::GetSize: 
//----------------------------------------------------------------------------------------


Size TViewRectDesignator::GetSize() const
{
	return fDesignation.GetLength(vSel) * fDesignation.GetLength(hSel);
}

//----------------------------------------------------------------------------------------
// TViewRectDesignator::ReadFrom: 
//----------------------------------------------------------------------------------------


void TViewRectDesignator::ReadFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);
	*aStream >> fDesignation;
}

//----------------------------------------------------------------------------------------
// TViewRectDesignator::WriteTo: 
//----------------------------------------------------------------------------------------


void TViewRectDesignator::WriteTo(CStream_AC* aStream) const// override 
{
	Inherited::WriteTo(aStream);
	*aStream << fDesignation;
}

//----------------------------------------------------------------------------------------
// TViewRectDesignator::IsContained: 
//----------------------------------------------------------------------------------------


ContainmentType TViewRectDesignator::IsContained(TDesignator* aDesignator)
{
	ContainmentType returnVal = kNotContained;

#if qDebug
	if (aDesignator && !dynamic_cast<TViewRectDesignator*>(aDesignator))
		::ProgramBreak_AC("The containing designator for the TViewRectDesignator is not of the same class");
#endif

	if (aDesignator)
	{
		CViewRect itsVRect(dynamic_cast_AC(TViewRectDesignator*,aDesignator)->fDesignation);
		
		if (fDesignation == itsVRect)
			returnVal = kExactlyContained;
		else
		{
			CViewRect dstRect = fDesignation & itsVRect;
			if (!dstRect.Empty())
				if (dstRect == itsVRect)
					returnVal = kFullyContained;
				else
					returnVal = kPartiallyContained;
		}
	}

	return returnVal;
}


//========================================================================================
// CLASS TRegionDesignator
//========================================================================================
#undef Inherited
#define Inherited TDesignator


MA_DEFINE_CLASS(TRegionDesignator);

//----------------------------------------------------------------------------------------
// TRegionDesignator::TRegionDesignator: 
//----------------------------------------------------------------------------------------


TRegionDesignator::TRegionDesignator(RgnHandle itsDesignation,
										   bool freeRegionOnFree) :
	fFreeRegionOnFree(freeRegionOnFree),
	fDesignation(itsDesignation ? itsDesignation :MakeNewRgn())	// Either assign the designation, or create it.
{
}

//----------------------------------------------------------------------------------------
// TRegionDesignator::~TRegionDesignator: 
//----------------------------------------------------------------------------------------


TRegionDesignator::~TRegionDesignator()
{
	if (!fFreeRegionOnFree)
		fDesignation.release();
}

//----------------------------------------------------------------------------------------
// TRegionDesignator::ReadFrom: 
//----------------------------------------------------------------------------------------


void TRegionDesignator::ReadFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);

	if (!fDesignation)
		fDesignation.reset((RgnHandle)aStream->ReadHandle());
	else
	{
		// read the size of the handle in from the stream
		Size itsSize = aStream->ReadLong();

		// increase the handle appropriately
		SetPermHandleSize((Handle) fDesignation.get(), itsSize);

		// lock the handle and read into it from the stream
		CTempHandleLock_AC tempHandleLock((Handle) fDesignation.get());
		::ThrowIfMemError_AC();
		aStream->ReadBytes(*((Handle) fDesignation.get()), itsSize);
	}
}



//----------------------------------------------------------------------------------------
// TRegionDesignator::WriteTo: 
//----------------------------------------------------------------------------------------


void TRegionDesignator::WriteTo(CStream_AC* aStream) const// override 
{
	Inherited::WriteTo(aStream);

	aStream->WriteHandle((Handle) fDesignation.get());
}

//----------------------------------------------------------------------------------------
// TRegionDesignator copy constructor
//----------------------------------------------------------------------------------------


TRegionDesignator::TRegionDesignator(const TRegionDesignator& inRegionDesignator) :
	TDesignator(inRegionDesignator),

	fFreeRegionOnFree(inRegionDesignator.fFreeRegionOnFree),
	fDesignation(MakeNewRgn())
{
	CopyRgn(inRegionDesignator.fDesignation, fDesignation);
}

//----------------------------------------------------------------------------------------
// TRegionDesignator::GetSize: 
//----------------------------------------------------------------------------------------


Size TRegionDesignator::GetSize() const
{
	Size returnVal = 0;

	if (fDesignation && !EmptyRgn(fDesignation))
	{
		CRect_AC rgnBBox((*fDesignation)->rgnBBox);
		returnVal = (rgnBBox.GetLength(vSel) * rgnBBox.GetLength(hSel));
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TRegionDesignator::IsContained: 
//----------------------------------------------------------------------------------------


ContainmentType TRegionDesignator::IsContained(TDesignator* aDesignator)
{
	ContainmentType returnVal = kNotContained;
#if qDebug
	if (aDesignator && !dynamic_cast<TRegionDesignator*>(aDesignator))
		::ProgramBreak_AC("The containing designator for the TRegionDesignator is not of the same class");
#endif

	if (aDesignator)
	{
		TRegionDesignator* designator = dynamic_cast_AC(TRegionDesignator*,aDesignator);
	
		RgnHandle itsDesignation = designator->fDesignation;
		
		if (EqualRgn(fDesignation, itsDesignation))
			returnVal = kExactlyContained;
		else
		{
			CTempRegion_AC rRgn;
			SectRgn(fDesignation, itsDesignation, rRgn);
			if (!EmptyRgn(rRgn))
				if (EqualRgn(itsDesignation, rRgn))
					returnVal = kFullyContained;
				else
					returnVal = kPartiallyContained;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TRegionDesignator::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TRegionDesignator::GetStandardSignature() const
{
	return kRegionDesignator;
}

//----------------------------------------------------------------------------------------
// TRegionDesignator::SetDesignationRect: 
//----------------------------------------------------------------------------------------


void TRegionDesignator::SetDesignationRect(const CRect_AC& theDesignation)
{
	RectRgn(fDesignation, theDesignation);
}

//----------------------------------------------------------------------------------------
// End of UDesignator.cp


