//----------------------------------------------------------------------------------------
// UDispatcher.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

// ACS

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UBUSYCURSOR__
#include "UBusyCursor.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __FLOATWINDOW__
#include "FloatWindow.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __UOSASCRIPT__
#include "UOSAScript.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UTRACKER__
#include "UTracker.h"
#endif

#ifndef __UUNDO__
#include "UUndo.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#ifndef __AEObjects_AC__
#include "AEObjects_AC.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __DISKINIT__
#include <DiskInit.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// Threads

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

// Other

#if defined(__MWERKS__) && (qDebug || qTheDebugger)
#ifndef __CWDEBUG__
#include "CWDebug.h"
#endif	

#endif

//----------------------------------------------------------------------------------------
// Constant definitions
//----------------------------------------------------------------------------------------

enum SystemJustification
{
	smSysJustLeft = 0, smSysJustRight = -1
};
// Constants to use with GetSysDirection() and SetSysDirection().


//----------------------------------------------------------------------------------------
// Global variable definitions.
//----------------------------------------------------------------------------------------

TDispatcher* gDispatcher;
bool gInitialized;
CTempDesc gServerAddress;

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static void DoSetupTheMenus(void* yourDataPtr);

static WindowRef GetWindowToClose();

//========================================================================================
// CLASS CEventListIterator
//========================================================================================
class CEventListIterator : public CIterator_AC
{
public:
	inline CEventListIterator(const TEventList* itsEventList) :
		fCurrent(NULL),
		fCurrentIndex(kEmptyIndex_AC),
		// We iterate backward over the event list
		fEventList(itsEventList)
	{ First(); }

	//
	// Required functions 
	//
	inline TEvent* First() { fCurrentIndex = fEventList->GetSize(); return fCurrent = fCurrentIndex ? static_cast<TEvent*>(fEventList->At(fCurrentIndex)) : NULL; }
	inline TEvent* Current() const { return fCurrent; }
	inline TEvent* Next() { fCurrentIndex = (fCurrentIndex > 1) ? fCurrentIndex - 1 : kEmptyIndex_AC; return fCurrent = fCurrentIndex ? static_cast<TEvent*>(fEventList->At(fCurrentIndex)) : NULL; }

	//
	// Convenience functions for C++meisters
	//
	inline TEvent* operator++() { return Next(); }
	inline TEvent* operator++(int) { TEvent* returnVal = Current(); Next(); return returnVal; }
	inline operator TEvent*() const { return Current(); }
	inline TEvent* operator->() const { return Current(); }

protected:
	TEvent* fCurrent;
	const TEventList* fEventList;				// the main event list

public:
	// These items are currently public for searching

	ArrayIndex_AC fCurrentIndex;					// current index of this iteration
};

//========================================================================================
// CLASS CNoGhostDocsIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CNoGhostDocsIterator::CNoGhostDocsIterator: 
//----------------------------------------------------------------------------------------
CNoGhostDocsIterator::CNoGhostDocsIterator(const TDispatcher* itsApplication,
				  ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
				  bool itsForward) :
CDocumentIterator(itsApplication, itsLowBound, itsHighBound, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CNoGhostDocsIterator::CNoGhostDocsIterator: 
//----------------------------------------------------------------------------------------
CNoGhostDocsIterator::CNoGhostDocsIterator(const TDispatcher* itsApplication, bool itsForward) :
CDocumentIterator(itsApplication, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CNoGhostDocsIterator::First: 
//----------------------------------------------------------------------------------------
TDocument* CNoGhostDocsIterator::First()
{
	CDocumentIterator::First();
	
	// It doesn't, so we'll get the next  one, which is guaranteed to either meet the criteria, or be NULL
	if (IsGhostDoc())
		Next();
		
	return Current();
}

//----------------------------------------------------------------------------------------
// CNoGhostDocsIterator::Next: 
//----------------------------------------------------------------------------------------
TDocument* CNoGhostDocsIterator::Next()
{
	// advances iterator to next valid document
	do
	{
		CDocumentIterator::Next();

		if (!IsGhostDoc())
			break;
	}
	while (Current());
	
	return Current();
}

//----------------------------------------------------------------------------------------
// CNoGhostDocsIterator::IsGhostDoc: 
//----------------------------------------------------------------------------------------
bool CNoGhostDocsIterator::IsGhostDoc()
{
	return Current() ? Current()->GetIsGhostDocument() : false;
}

//========================================================================================
// struct CSetupTheMenus
//========================================================================================

struct CSetupTheMenus
{
public:
	TDispatcher* fDispatcher;

	// Constructor
	inline CSetupTheMenus(TDispatcher* theDispatcher) :
		fDispatcher(theDispatcher)
	{
	}
};

//----------------------------------------------------------------------------------------
// DoSetupTheMenus: 
//----------------------------------------------------------------------------------------
void DoSetupTheMenus(void*						/* yourDataPtr */ )
{
	if (gDispatcher)
	{
		Enable(cAboutApp, true);
		Enable(-(mApple << 8), true);				// allow the whole menu

		// Set the Undo and Redo menu items
		//	// to "Can't Undo" before giving the target chain a chance
		//	// to change it.
		//		gDispatcher->SetUndoText(cCantUndo);
		//		gDispatcher->SetRedoText(cCantUndo);

		gDispatcher->SetupUndoRedoMenus();

		gClipboardMgr->fGotClipType = false;	// so CanPaste will work correctly
		TDispatcher::GetTarget()->HandleSetupMenus();// Setup menus relevant to target chain 

		// Set up the menu commands that are not dependent on the target chain 

		if (gDispatcher->GetSysWindowActive())
		{
			Enable(cUndo, true);
			Enable(cCut, true);
			Enable(cCopy, true);
			Enable(cPaste, true);
			Enable(cClear, true);

			gDispatcher->SetUndoText(cNoCommand);
			gDispatcher->SetRedoText(cNoCommand);
		}

#if qPerform
		{
			bool initiated = PerfMonitorInitiated();

			Enable(cPerfMonInit, !initiated);
			Enable(cPerfMonDump, initiated);
			Enable(cPerfMonToggle, initiated);
			SetMenuState(cPerfMonToggle, kDebugBuzzStrings, bzContinuePerfMon, bzPausePerfMon, PerfMonitorEnabled());
			Enable(cPerfMonEnd, initiated);
		}
#endif

		if (CommandEnabled(cAboutApp) && gDispatcher->InModalState())
		{
			WindowRef aWindowPtr = FrontWindow();
			if (aWindowPtr && GetWVariant(aWindowPtr) == movableDBoxProc)
				Enable(cAboutApp, false);		// just disallow the about item
			else
				Enable(-(mApple << 8), false);	// disallow the whole menu
		}
	}
}


//========================================================================================
// CLASS TEventList:
//========================================================================================
//----------------------------------------------------------------------------------------
// TEventList destructor
//----------------------------------------------------------------------------------------
TEventList::~TEventList()
{
}

//----------------------------------------------------------------------------------------
// TEventList::Compare: 
//----------------------------------------------------------------------------------------
CompareResult_AC TEventList::Compare(const void* item1,
								  const void* item2) const
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	const TEvent* event1 = static_cast<const TEvent*>(item1);
	const TEvent* event2 = static_cast<const TEvent*>(item2);

	// We want higher priority events at the end of the queue so we can delete from the
	// end of the list. This will save us a block move in DeleteElementsAt.
	if (event1->GetPriority() > event2->GetPriority())
		returnVal = kItem1LessThanItem2_AC;
	else if (event1->GetPriority() < event2->GetPriority())
		returnVal = kItem1GreaterThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEventList::Insert: 
//----------------------------------------------------------------------------------------
void TEventList::Insert(const void* item)			// override 
{
	CTempObjectAllocation tempObjectAllocation(true);

	CSortedList_AC::Insert(item);
}

//========================================================================================
// CLASS TEventRetrieverCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TEventRetrieverCommand::TEventRetrieverCommand: 
//----------------------------------------------------------------------------------------
TEventRetrieverCommand::TEventRetrieverCommand(CommandNumber itsCommandNumber) :
	TCommand(itsCommandNumber, gDispatcher, kCantUndo, kDoesNotCauseChange, NULL)
{
	// Hang around 
	fFreeOnCompletion = false;
	fRecurring = true;

	// Let more important stuff happen first 
	fPriority = kPriorityLow;
}

//----------------------------------------------------------------------------------------
// TEventRetrieverCommand destructor
//----------------------------------------------------------------------------------------
TEventRetrieverCommand::~TEventRetrieverCommand()
{
}

//----------------------------------------------------------------------------------------
// TEventRetrieverCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TEventRetrieverCommand::DoIt()
{
	gDispatcher->PollToolboxEvent(gDispatcher->GetAllowApplicationToSleep());
}


//========================================================================================
// CLASS TDispatcher
//========================================================================================
//----------------------------------------------------------------------------------------
// TDispatcher constructor 
//----------------------------------------------------------------------------------------
TDispatcher::TDispatcher(OSType itsMainFileType,
							  OSType itsCreator) :
	MDefaultScriptableObject(cApplication),
	TCommandHandler(NULL),

	fCreator(itsCreator),
	fMainFileType(itsMainFileType),
	fHeadCohandler(NULL),
	fUndoCommand(cNoCommand),
	fRedoCommand(cNoCommand),
	fUndoRedoCommand(cNoCommand),
	fUndoRedoState(kShowUndo),
	fLastUpTime(TickCount()),
	fNextSpaceMessage(TickCount()),
	fLowSpaceInterval(kLowSpaceInterval),
	fIdlePhase(idleEnd),
	fDisplayedMenus(kMBarDisplayed),
	fMBarHierarchical(kMBarHierarchical),
	fOtherMenus(kMBarNotDisplayed),
	fClickCount(0),
	fLastClickPart(inDesk),
	fMainEventMask(everyEvent),
	fEventLevel(1),
	fAllowApplicationToSleep(true),
	fAlwaysTrackCursor(false),
	fDone(false),
	fSysWindowActive(false),
	fWantKeyUpEvents(false),
	fAutoKeyCommandKeys(false)
{
	SetTarget(this);
	// This is a special case.  It needs to be available as soon as possible.
	gDispatcher = this;

	try
	{
		// Menu bar manager, view server, and default dependency space
		// are initialized from polymorphic functions called from DoInitialState.
		
		// OSADispatcher setup
		if (!TOSADispatcher::fgDispatcher)
			InitUScripting();

		TOSADispatcher::fgDispatcher->SetDefaultTarget(this);

		TPrintHandler::gPrintHandler->DispatcherIsAvailable();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		gDispatcher = NULL;
		throw;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher destructor
//----------------------------------------------------------------------------------------
TDispatcher::~TDispatcher()
{
	for (CEventListIterator iter(&fEventList); iter; ++iter)
		delete iter.Current();

	gDispatcher = NULL;
}

//----------------------------------------------------------------------------------------
// TDispatcher::AboutToLoseControl: 
//----------------------------------------------------------------------------------------
void TDispatcher::AboutToLoseControl(bool convertClipboard)
{
	gClipboardMgr->AboutToLoseControl(convertClipboard);

	// Let all windows know that we're losing control - e.g. so floaters can hide
	// themselves
	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(iter);
		if (aWindow)
			aWindow->AboutToLoseControl();
	}

	Changed(mAboutToLoseControl, this);

	ActivateBusyCursor(false);					// Don't want busy cursor while in DA
	
	TMenuBarManager::fgMenuBarManager->AboutToLoseControl();
}

//----------------------------------------------------------------------------------------
// TDispatcher::InteractWithUser: 
//----------------------------------------------------------------------------------------
OSErr TDispatcher::InteractWithUser()
{
	return InteractWithUser(kMacAppTimeout, gNotificationPtr, TAppleEvent::fgIdleProc);
}

OSErr TDispatcher::InteractWithUser(long timeOutInTicks,
									NMRecPtr nmReqPtr,
									AEIdleUPP idleProc)
{
	long interactTimeout;

	if (timeOutInTicks == kMacAppTimeout)
	{
		if (TOSADispatcher::fgDispatcher && TOSADispatcher::fgDispatcher->GetDispatchLevel() > 0)
			interactTimeout = kAEDefaultTimeout;
		else
			interactTimeout = kNoTimeOut;
	}
	else
		interactTimeout = timeOutInTicks;

	return AEInteractWithUser(interactTimeout, nmReqPtr, idleProc);
}

//----------------------------------------------------------------------------------------
// TDispatcher::ActivateBusyCursor: 
//----------------------------------------------------------------------------------------
bool TDispatcher::ActivateBusyCursor(bool entering)
{
	return gBusyCursor.get() ? gBusyCursor->Activate(entering) : false;
}

//----------------------------------------------------------------------------------------
// TDispatcher::KeepCursorBusy: 
//----------------------------------------------------------------------------------------
bool TDispatcher::KeepCursorBusy(bool keepItSpinning)
{
	return gBusyCursor.get() ? gBusyCursor->KeepCursorBusy(keepItSpinning) : false;
}

//----------------------------------------------------------------------------------------
// TDispatcher::AddDocument: 
//----------------------------------------------------------------------------------------
void TDispatcher::AddDocument(TDocument* aNewDocument)
{
	fDocumentList.Insert(aNewDocument);
}

//----------------------------------------------------------------------------------------
// TDispatcher::AddWindow: 
//----------------------------------------------------------------------------------------
void TDispatcher::AddWindow(TWindow* aWindow)
{
	// Protect against double installation
	if (!fFreeWindowList.GetIdentityItemNo(aWindow))
		fFreeWindowList.Insert(aWindow);
}

//----------------------------------------------------------------------------------------
// TDispatcher::AlertFilter: AlertFilter is a default filterProc used by MacAppAlert if
// the filterProc passed in is nil. It maps key strokes to the first character of button
// item titles. It also hands off activate and update processing to gDispatcher if we're
// not being called from an error condition or while nested.
//----------------------------------------------------------------------------------------
bool TDispatcher::AlertFilter(DialogRef theDialog,
								 EventRecord& theEvent,
								 short& itemHit)

{
	bool returnValue = false;
	bool oldInFilterState = gInFilter;

	gInFilter = true;

	// First call the minimal alert filter; if it does not handle the
	// event, do some more complex handling
	returnValue = MinimalAlertFilter(theDialog, theEvent, itemHit);
	if (!returnValue)
	{
		try
		{
			// Wouldn't want MacApp to get lied to about where the focus _Actually_ is 
			if (!gInhibitNestedHandling && !oldInFilterState)
				InvalidateFocus();

			switch (theEvent.what)
			{
					// this is the first event the alert gets, use it to initialize the global buffer
				case activateEvt:
				case updateEvt:
					if ((((DialogRef)theEvent.message) != theDialog) && (!gInhibitNestedHandling && !oldInFilterState))
					{
						GrafPtr savePort;
						GetPort(&savePort);
						TToolboxEvent * event;
						{
							CTempObjectAllocation tempObjectAllocation(true);
							event = TH_new TToolboxEvent(this, &theEvent);
						}
						event->Process();

						SetPort(savePort);
					}
					break;

					// let's determine if the key pressed corresponds to our button titles
				case keyDown:
					{
						static TToolboxEvent* bufferedKeyEvent;// Must be a global, since it's used each
						// time the alert filter is entered. (It
						// could be a field of TDispatcher...)

						bool needsSecondaryEvent = false;
						if (!bufferedKeyEvent)
						{
							CTempObjectAllocation tempObjectAllocation(true);
							bufferedKeyEvent = TH_new TToolboxEvent(this, &theEvent);
							needsSecondaryEvent = bufferedKeyEvent->IsDoubleEvent();
						}
						else
							bufferedKeyEvent->AddSecondaryKeyEvent(theEvent);

						if (!needsSecondaryEvent)
						{
							GetTarget()->KeyEventToComponents(bufferedKeyEvent);// Find out what keys were _REALLY_ pressed


							if (CompareAlertKeysToItem(theDialog, bufferedKeyEvent->GetText(), itemHit))
							{
								DoAlertKeyDown(theDialog, itemHit);
								returnValue = true;
							}
							delete bufferedKeyEvent;
							bufferedKeyEvent = NULL;
						}
					}
			}

			// Idle but only if _REALLY_ necessary 
			EventRecord anEvent;
			if (!gInhibitNestedHandling && !oldInFilterState && !EventAvail(everyEvent, &anEvent))
				Idle(fIdlePhase);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
		}
		
	}
	gInFilter = oldInFilterState;

	return returnValue;
}

//----------------------------------------------------------------------------------------
// TDispatcher::AppleEventIdleProc: 
//----------------------------------------------------------------------------------------
bool TDispatcher::AppleEventIdleProc(EventRecord& theEventRecord,
										long&	/* sleepTime */ ,
										RgnHandle&/* mouseRgn */  )
{
	if ((theEventRecord.what == activateEvt) || (theEventRecord.what == updateEvt))
	{
		TToolboxEvent * event;
		{
			CTempObjectAllocation tempObjectAllocation(true);
			event = TH_new TToolboxEvent(this, &theEventRecord);
		}
		event->Process();
	}
	return false;
}

//----------------------------------------------------------------------------------------
// TDispatcher::StandardFileFilter: 
//----------------------------------------------------------------------------------------
bool TDispatcher::StandardFileFilter(DialogRef/* theDialog */ ,
										EventRecord& theEvent,
										short&	/* itemHit */ ,
										void*	/* yourDataPtr */ )
{
	InvalidateFocus();
	switch (theEvent.what)
	{
		case activateEvt:
		case updateEvt:
			// check if intended for a MacApp window
			if (!TWindow::WMgrToWindow((WindowRef)theEvent.message))
				break;
			// drop thru to processing the event
		case osEvt:
		case kHighLevelEvent:
			{
				GrafPtr savePort;
				GetPort(&savePort);				// If the port get changed behind us
												// the dialog can't find its controls
				TToolboxEvent * event;
				{
					CTempObjectAllocation tempObjectAllocation(true);
					event = TH_new TToolboxEvent(NULL, &theEvent);
				}
				
				try
				{
					++fEventLevel;
					event->Process();
					--fEventLevel;
				}
				catch(CException_AC& theException)
				{
					DoCatchMessage_AC(theException);
					--fEventLevel;
					
					throw;
				}

				SetPort(savePort);
			}
			break;
	}

	return false;								// did not take the event
}

//----------------------------------------------------------------------------------------
// TDispatcher::FindDocument: 
//----------------------------------------------------------------------------------------
TDocument* TDispatcher::FindDocument(TFile* aFile)
{
	TDocument * returnVal = NULL;

	for (CDocumentIterator iter(this); iter.Current(); ++iter)
		if (iter->FindDocument(aFile))
		{
			returnVal = iter;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDispatcher::Beep: 
//----------------------------------------------------------------------------------------
void TDispatcher::Beep(short duration)
{
#if !qMSWindows
	::SysBeep(duration);
#endif
}

//----------------------------------------------------------------------------------------
// TDispatcher::Close: 
//----------------------------------------------------------------------------------------
void TDispatcher::Close()
{
	// Close down the cohandler chain.  Add a block for failure handling
	{
		for (CHandlerIterator iter(fHeadCohandler); iter; ++iter)
			delete iter.Current();
	}

	gClipboardMgr->Close();

#if qAttachable
	// Toss any attached script
	FreeOSAScript();
#endif

}

//----------------------------------------------------------------------------------------
// TDispatcher::CloseToolboxWindow: 
//----------------------------------------------------------------------------------------
void TDispatcher::CloseToolboxWindow(WindowRef aWMgrWindow)
{
	TWindow * aWindow = TWindow::WMgrToWindow(aWMgrWindow);
	if (aWindow)
		aWindow->CloseByUser(cClose);
	else
		HideWindow(aWMgrWindow);
}

//----------------------------------------------------------------------------------------
// TDispatcher::CountClicks: 
//----------------------------------------------------------------------------------------
short TDispatcher::CountClicks(TToolboxEvent* event,
							   short whereMouseDown)
{
	short newClickCount;

	newClickCount = 1;

	if ((whereMouseDown == fLastClickPart) &&	// clicked in the same place 
		(fClickCount > 0) &&					// not the first click 
		((event->GetWhen() < fLastUpTime) ||	// clicked again before we recorded last up time, or
		 (event->GetWhen() - fLastUpTime < LMGetDoubleTime())) && // close enough in time 
		GetTarget()->DoMultiClick(fLastMousePoint, event->GetWhere())) // close enough in space
		newClickCount = fClickCount + 1;

	fLastMousePoint = event->GetWhere();

	fLastClickPart = whereMouseDown;
	fClickCount = newClickCount;
	return newClickCount;
}

//----------------------------------------------------------------------------------------
// TDispatcher::DeleteDocument: 
//----------------------------------------------------------------------------------------
void TDispatcher::DeleteDocument(TDocument* docToDelete)
{
	fDocumentList.Delete(docToDelete);
}

//----------------------------------------------------------------------------------------
// TDispatcher::DeleteWindow: 
//----------------------------------------------------------------------------------------
void TDispatcher::DeleteWindow(TWindow* windowToDelete)
{
	fFreeWindowList.Delete(windowToDelete);
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoCommandKeyEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoCommandKeyEvent(TToolboxEvent* event)// override 
{
	if (!event->IsAutoKeyEvent() || fAutoKeyCommandKeys)
	{
		SetupTheMenus();

		// == THIS =======================================================================================
		// Optional support for case sensitive command keys.
		// Use the following line because
		// KeyEventToComponents returns the correct character for shifted keys when the
		// command key is down. That lets us test for things like command-period
		// correctly. So in order to be backward compatible (sigh) we now have to ignore
		// the _correct_ char that is passed in (and is in event->fText) and use the
		// old ToolBox supplied unPasteurized character that is left in the actual
		// EventRecord
		//
		// long menuKeyResult = MenuKey(event->fText[1]);
		// if (event->fDoubleEvent)
		// 		menuKeyResult = MenuKey(event->fText[2]);


		// == OR THIS ====================================================================================
		// Optional support for the latest version (1.2) of Mercutio MDEF which allows the
		// display and processing of Option and Shifted command keys.
		// http://www-leland.stanford.edu/~felciano/da/mercutio/
		//
		// const mMercutioMenu = xxx; // ID of any menu that uses the Mercutio MDEF
		//
		// MenuHandle anMHandle = MAGetMenu(mMercutioMenu);
		// long menuKeyResult = MDEF_MenuKey(event->fEventRecords[0].message, event->fEventRecords[0].modifiers, anMHandle);
		// if (event->fDoubleEvent)
		// 		menuKeyResult = MDEF_MenuKey(event->fEventRecords[1].message, event->fEventRecords[1].modifiers, anMHandle);


		// == OR THIS ====================================================================================
		// Ordinary case and modifier insensitive command keys
		long menuKeyResult = MenuKey(static_cast<unsigned char>(event->GetMessage() & charCodeMask));
		if (event->IsDoubleEvent())
			menuKeyResult = MenuKey(static_cast<unsigned char>(event->GetMessage(1) & charCodeMask));

		MenuEvent(HiWord(menuKeyResult), LoWord(menuKeyResult));
	}
	else
		TCommandHandler::DoCommandKeyEvent(event);

}

//----------------------------------------------------------------------------------------
// TDispatcher::DoKeyEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoKeyEvent(TToolboxEvent* event)// override 
{
	CommandNumber aCommand = cNoCommand;

	switch (event->GetKeyCode())
	{
		case kF1VirtualCode:
			aCommand = event->IsShiftKeyPressed() ? cRedo : cUndo;
			break;

		case kF2VirtualCode:
			aCommand = cCut;
			break;

		case kF3VirtualCode:
			aCommand = cCopy;
			break;

		case kF4VirtualCode:
			aCommand = cPaste;
			break;

		case kClearVirtualCode:
			aCommand = cClear;
			break;
	}

	if (aCommand != cNoCommand)
	{
		SetupTheMenus();
		if (CommandEnabled(aCommand))
			GetTarget()->HandleMenuCommand(aCommand);
	}
	else
		TCommandHandler::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoMakeFile: 
//----------------------------------------------------------------------------------------
TFile* TDispatcher::DoMakeFile(CommandNumber)
{
	return NewFile(fMainFileType, fCreator, kUsesDataFork, kUsesRsrcFork, !kDataOpen, !kRsrcOpen);
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoMenuCommand: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoMenuCommand(CommandNumber aCommandNumber)
{
	// ===================================================================================
	// Some commands will be posted to perform actions that must _ALWAYS_ be available.
	// The allocation cannot be allowed to fail. So we do a temp allocation which by
	// definition cannot be allowed to fail. This strategy is used wherever we want to use
	// command objects but don't want to leave the user twisting in the breeze. NOTE:
	// Don't forget to allow for this memory in your mem! resource if you copy this style
	// in your own code.
	// ===================================================================================

	switch (aCommandNumber)
	{
		case cUndo:
		case cRedo:
		case cUndoRedo:
			{
				CTempAllocation  tempAllocation(true);
				CTempObjectAllocation tempObjectAllocation(true);
				
				TUndoRedoCommand * undoRedoCommand = TH_new TUndoRedoCommand(aCommandNumber);
				undoRedoCommand->SetUseAppleEvent(true);
				PostCommand(undoRedoCommand);
			}
			break;

		case cClose:
#if qDebug
			if (TWindow::WMgrToWindow(FrontWindow()))
				::ProgramBreak_AC("The frontmost window is a window object but didn't handle the cClose CommandNumber, your TWindow subclass probably forgot to call Inherited DoMenuCommand!");
#endif

			CloseToolboxWindow(FrontWindow());	// TWindow would have handled the command
			// before we get here so the window is
			break;								// probably a DA or something

		default:
			TCommandHandler::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoSetupMenus: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoSetupMenus()
{
	if (!gDispatcher->InModalState())
		TCommandHandler::DoSetupMenus();

	TMenuBarManager::fgMenuBarManager->SetPreferredMenuBarID(fDisplayedMenus);
	TMenuBarManager::fgMenuBarManager->SetPreferredHierMenuBarID(fMBarHierarchical);

	bool lowSpace = MemSpaceIsLow();

	Enable(-(mApple << 8), true);				// allow the whole menu

	WindowRef aWindowPtr = FrontWindow();
	if (aWindowPtr && !TWindow::WMgrToWindow(aWindowPtr))
	// window objects will take care of themselves, but we take care of the indigent.
		Enable(cClose, GetWindowGoAwayFlag(aWindowPtr));
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoMakeDependencySpace: 
//----------------------------------------------------------------------------------------

void TDispatcher::DoMakeDependencySpace()
{
	// Set the default dependency space
	if (!CDependencySpace_AC::fgDefaultDependencySpace)
		CDependencySpace_AC::fgDefaultDependencySpace = TH_new CSimpleDependencySpace_AC;
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoMakeMenuBarManager: 
//----------------------------------------------------------------------------------------

void TDispatcher::DoMakeMenuBarManager()
{
	// The TMeuBarManager constructor will set the global reference
	if (!TMenuBarManager::fgMenuBarManager)
		TMenuBarManager* aMenuBarManager = TH_new TMenuBarManager(fOtherMenus);
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoMakeViewServer: 
//----------------------------------------------------------------------------------------

void TDispatcher::DoMakeViewServer()
{
	// The TViewServer constructor will set the global reference
	if (!TViewServer::fgViewServer)
		TViewServer * aViewServer = TH_new TViewServer;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetDefaultCursorRegion: Make the region the desktop less the active
// window, and any first click windows. The current mouse location is added in
// TDispatcher::TrackCursor.
//----------------------------------------------------------------------------------------
void TDispatcher::GetDefaultCursorRegion(CPoint_AC	/* globalMouse */,
										 RgnHandle cursorRegion)
{
	// Start off with the "desktop region" 
	GetDeskTopRegion(cursorRegion);

	// Remove the content of all windows for which TestWindow returns true 
	CTempRegion_AC windowContent;

	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(iter);

		if (aWindow && aWindow->IsShown()
#if !qDrag
			&&aWindow->HandlesCursor()
#endif

			&&aWindow->Focus())
		{
			aWindow->GetVisibleRegion(windowContent);

			// Convert the content to global coordinates. We focused on the window because
			// the content is in the coordinates of the current port
			aWindow->LocalToSuperView(windowContent);

			// Remove the visRgn from the cursor region 
			DiffRgn(cursorRegion, windowContent, cursorRegion);
		}
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::InstallHelpMenuItems: 
//----------------------------------------------------------------------------------------
void TDispatcher::InstallHelpMenuItems()
{
	// abstract method; override to install your help menu item(s).  Use
	// TMenuBarManager::fgMenuBarManager->AddHelpMenuItem( "^0 Help", kYourHelpCommandNumber );
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetDefaultHelpRegion: Make the region the desktop less the active window,
// and any first click windows. The current mouse location is added in
// TDispatcher::TrackHelp.
//----------------------------------------------------------------------------------------
void TDispatcher::GetDefaultHelpRegion(CPoint_AC	/* globalMouse */,
									   RgnHandle helpRegion)
{
	// Start off with the "desktop region" 
	GetDeskTopRegion(helpRegion);

	// Remove the content of all windows for which TestWindow returns true 
	CTempRegion_AC windowContent;

	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(iter);

		if (aWindow && (aWindow->IsShown() && aWindow->HandlesHelp()) && aWindow->Focus())
		{
			aWindow->GetVisibleRegion(windowContent);

			// Convert the content to global coordinates. We focused on the window because
			// the content is in the coordinates of the current port
			aWindow->LocalToSuperView(windowContent);

			// Remove the visRgn from the cursor region 
			DiffRgn(helpRegion, windowContent, helpRegion);
		}
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetHelpParameters: 
//----------------------------------------------------------------------------------------
void TDispatcher::GetHelpParameters(ResNumber helpResource,
									short helpIndex,
									short helpState,
									HMMessageRecord& helpMessage,
									CPoint_AC& localQDTip,
									CRect_AC& localQDRect,
									short& balloonVariant)
{
	CTempUseResFile_AC tempUseResFile(gApplicationRefNum);

	unsigned long options;
	short theProc;
	short count;
	::ThrowIfOSErr_AC(HMGetIndHelpMsg(kHMDialogResType, helpResource, helpIndex, helpState, &options, localQDTip, localQDRect, &theProc, &balloonVariant, &helpMessage, &count));
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetSleepRegion: 
//----------------------------------------------------------------------------------------
RgnHandle TDispatcher::GetSleepRegion(const CPoint_AC* forGlobalMouse)
{
	RgnHandle returnVal = NULL;

	if (IsFrontProcess())
	{
		// Compute based on where the mouse is right NOW 
		CPoint_AC globalMouse;
		if (forGlobalMouse)
			globalMouse = *forGlobalMouse;
		else
		{			
			::GetMouse(globalMouse);
			::LocalToGlobal(globalMouse);
		}
		bool isSleepInvalid = false;

		// The help region is computed before the cursor rgn so that
		// the cursor region excludes the balloon shape
		if ((IsHelpRgnInvalid()) && (IsHelpEnabled()))
		{
			TrackHelp(globalMouse);
			isSleepInvalid = true;
		}

		if (IsCursorRgnInvalid())
		{
			TrackCursor(globalMouse);
			isSleepInvalid = true;
		}

		if (isSleepInvalid)
		{
			if (IsHelpEnabled())
				SectRgn(fCursorRegion, fHelpRegion, fSleepRegion);
			else
				CopyRgn(fCursorRegion, fSleepRegion);
		}

		returnVal = fSleepRegion;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetEvent: 
//----------------------------------------------------------------------------------------
TToolboxEvent* TDispatcher::GetEvent(short eventMask,
									 unsigned long sleep,
									 RgnHandle sleepRegion)
{
	TToolboxEvent * returnEvent = NULL;

	if (--gRsrcCheck <= 0)
	{
		CheckRsrcUsage();
		gRsrcCheck = kRsrcCheckInterval;
	}

#if qDebug
	if (gIntenseDebugging && gReportEvent)
	{
		ProgramReport_AC(false, "WaitNextEvent: sleep=0%d", sleep);
		// faceless driver bug fixed in MF 7.0 
		if (!sleepRegion)
			ProgramReport_AC(", sleep region=nil");
		else
		{
			CRect_AC bBox((*sleepRegion)->rgnBBox);
			ProgramReport_AC(false, ", sleep region = %s", (const char*)(bBox));
		}
		ProgramReport_AC("\n");

	}

	if (gShowCursorRegion || gShowHelpRegion || gShowSleepRegion)
	{
		if (gShowCursorRegion)
			ShowGlobalRegion(fCursorRegion);

		if (gShowHelpRegion)
			ShowGlobalRegion(fHelpRegion);

		if (gShowSleepRegion)
			ShowGlobalRegion(fSleepRegion);

		sleep = 60;
	}

#endif // qDebug

	// If we are in a nested event level than don't turn off the busy cursor since we are
	// probably in the middle of an idle or filter proc and don't want to turn on and off
	// the busy cursor each time it gets to service an event.
	bool oldActivateBusyCursor;
	if (fEventLevel <= 1)
		oldActivateBusyCursor = ActivateBusyCursor(false);				// Turn off busy cursor while we're away.

#if qPerform
	bool oldSetting = PerfMonitorEnabled();
#endif

	// Foreground applications don't really "sleep" so we have a code here to just hang around
	// til we get an event OR the sleep time expires
    unsigned long sleepTil = TickCount() + sleep;
    unsigned long snooze;
    do
    {
    	unsigned long now = TickCount();
    	
    	// don't use Max_AC with a subtraction instead of the comparison b/c the unsigned value
    	// won't go negative for overdue sleep and we'll sleep forever.
		snooze = (sleepTil > now) ? sleepTil - now : 0;

        // SystemEvents aren't queued and will be lost if not retrieved when available. So we
        // ensure here that they are always retrieved by adding osMask.
        EventRecord anEvent;
        if (WaitNextEvent(eventMask | osMask, &anEvent, snooze, sleepRegion))
        {
            // MWERKS you can remove the following 3 lines if you always send debug messages to your debugger
            // instead of to SIOUX
#if defined(__MWERKS__) && (qDebug || qTheDebugger)
            if (!CWDebugHandleOneEvent(&anEvent))
#endif
            {
                {
	                CTempObjectAllocation tempObjectAllocation(true);
                	returnEvent = TH_new TToolboxEvent(this, &anEvent);
                }
                if (returnEvent->IsDoubleEvent())
                {
                    // The event was part of a pair used to input 16 bit characters
                    // Retrieve the second half of the pair immediately.
                    // NOTE: although you would think that the second event can be retrieved with GetOSEvent
                    // The two byte sytems (KanjiTalk, etc.) seem to only return the second byte thru Wait/GetNextEvent.
                    EventRecord secondaryEvent;
                    if (WaitNextEvent(keyDownMask | keyUpMask | autoKeyMask, &secondaryEvent, 0, NULL))
                        returnEvent->AddSecondaryKeyEvent(secondaryEvent);
                }
            }

            // We have an event, let's go
           break;
        }
    } while (snooze && fAllowApplicationToSleep);


#if qPerform
	EnablePerfMonitor(oldSetting);
#endif

	if (fEventLevel <= 1 && IsFrontProcess())						// If we're not in the background, then 
		ActivateBusyCursor(oldActivateBusyCursor);					// restore the busy cursor mechanism. 

	return returnEvent;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetFrontWindow: 
//----------------------------------------------------------------------------------------
TWindow* TDispatcher::GetFrontWindow()
{
	TWindow * returnVal = NULL;

	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(iter);

		if ((aWindow) && (aWindow->IsShown()) && (!aWindow->GetFloats()))
		{
			returnVal = aWindow;
			break;
		}
	}
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetActiveWindow: 
//----------------------------------------------------------------------------------------
TWindow* TDispatcher::GetActiveWindow(bool floatersAreOkay)
{
	TWindow * returnVal = NULL;

	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(iter);

		if (aWindow && aWindow->IsShown() && aWindow->IsActive() && (!aWindow->GetFloats() || (aWindow->GetFloats() && floatersAreOkay)))
		{
			returnVal = aWindow;
			break;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDispatcher::RetrieveAnEvent: 
//----------------------------------------------------------------------------------------
TEvent* TDispatcher::RetrieveAnEvent()			// override
{
	TEvent * returnEvent = NULL;

	// Higher priority events are at the end of the list.
	for (CEventListIterator iter(&fEventList); iter; ++iter)
	{
		if (iter->IsReadyToExecute())
		{
			returnEvent = iter;
			break;
		}
	}

	RemoveEvent(returnEvent);

	return returnEvent;
}

//----------------------------------------------------------------------------------------
// TDispatcher::RemoveEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::RemoveEvent(TEvent* event)
{
	if (event && !event->IsRecurring())
		fEventList.Delete(event);
}

//----------------------------------------------------------------------------------------
// TDispatcher::NewToolboxWindow: 
//----------------------------------------------------------------------------------------
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

struct WINDTemplate
{
	CRect_AC bounds;
	short procID;
	Boolean visible;
	Boolean filler1;
	Boolean goAway;
	Boolean filler2;
	long refcon;
	short itemsID;								// only for DLOG resource 
};
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef struct WINDTemplate WINDTemplate;
typedef WINDTemplate* WINDTemplatePtr, ** WINDTemplateHandle;

WindowRef TDispatcher::NewToolboxWindow(short rsrcId,
										bool& isResizable,
										bool& isClosable)
{
	// We force INVISIBLE in the WIND definition so the screen won't flash. 
	WindowRef aWMgrWindow;
	WINDTemplateHandle templateHandle;

	// Even though the window is permanent, we allocate it under a temporary flag so that the
	// maximum memory is available. Quickdraw can blow up if it can't allocate a grafPort.
	{
		CTempAllocation tempAllocation(true);
		CTempObjectAllocation tempObjectAllocation(true);
		
		templateHandle = (WINDTemplateHandle)GetResource('WIND', rsrcId);
		::ThrowIfNULLResource_AC((Handle)templateHandle);
		MoveHHi((Handle)templateHandle);		// in case it is locked by the ROM 

		WINDTemplate & templateData = **templateHandle;
		templateData.visible = false;
		isClosable = templateData.goAway;
		isResizable = ((templateData.procID == documentProc) || (templateData.procID == zoomDocProc) || (((templateData.procID & 0xFFF0) == kSystem75_kWindoidWDEF) && ((templateData.procID & 0x000F) & kSystem75_hasGrow)) || (((templateData.procID & 0xFFF0) == kSystem75AndLess_kWindoidWDEF) && ((templateData.procID & 0x000F) & kSystem75_hasGrow))
#if qUseOldFloatingWDEF
					   ||(((templateData.procID & 0xFFF0) == kWindoidWDEF) && !((templateData.procID & 0x000F) & kWindoidNotResizable))
#endif

					   );
		// If your own defProc is resizable, too, then after the call on NewToolboxWindow, set
		// isResizable true 

		WindowRef putWindowBehind = TWindow::GetLastFloatingWindowPtr();
		if (!putWindowBehind)
			putWindowBehind = (WindowRef) - 1;

		WindowRef storage = (WindowRef)TH_new CWindowRecord;	// allocate enough space for either color or B/W

		if (qNeedsColorQD || HasColorQD_AC())
			aWMgrWindow = (WindowRef)(GetNewCWindow(rsrcId, storage, putWindowBehind));
		else
			aWMgrWindow = GetNewWindow(rsrcId, storage, putWindowBehind);

		::ThrowIfNULL_AC(aWMgrWindow);
	}

	// Now we must make sure that the code reserve is still intact.
	if (!CheckReserve())
	{
		aWMgrWindow = TWindow::FreeIfWMgrWindow(aWMgrWindow);

		::ThrowException_AC(memFullErr, 0);
	}

	return aWMgrWindow;
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleActivateEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleActivateEvent(TToolboxEvent* event)
{
	TWindow * aWindow = TWindow::WMgrToWindow((WindowRef)(event->GetMessage()));
	if (aWindow)
	{
		bool activate = ((event->GetModifiers()) & activeFlag) != 0;
		if (aWindow->GetFloats())					// ignore floaters
		{
			TWindow * frontWindow = GetFrontWindow();// find frontmost non-floating window
			if (frontWindow)
			{
				aWindow = frontWindow;			// (de)activate it instead of the floater
				HiliteWindow(aWindow->GetWindowRef(), activate);
			}
		}
		aWindow->Activate(activate);
	}
	else
		HandleAlienEvent(event);
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleAlienEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleAlienEvent(TToolboxEvent* event)
{
	for (CHandlerIterator iter(fHeadCohandler); iter; ++iter)
		if (iter->DoCoHandlerEvent(event))
			break;
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleDiskEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleDiskEvent(TToolboxEvent* event)
{
	if (HiWord(event->GetMessage()) != noErr)
	{
		short err = DIBadMount(CPoint_AC(112, 80), event->GetMessage());
#if qDebugMsg
		if (err != noErr)
			ProgramReport_AC(false, "error from DIBadMount is %d\n", err);
#endif
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoToolboxEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoToolboxEvent(TToolboxEvent* event)
{
	// For Toolbox events, fIdentifier == fEventRecord.what 
	if (event)
	{
		switch (event->GetIdentifier())
		{
			case mouseUp:
				HandleMouseUp(event);
				break;

			case mouseDown:
				HandleMouseDown(event);
				break;

			case activateEvt:
				HandleActivateEvent(event);
				break;

			case updateEvt:
				HandleUpdateEvent(event);
				break;

			case keyDown:
			case autoKey:
				HandleKeyDownEvent(event);
				break;

			case keyUp:
				HandleKeyUpEvent(event);
				break;

			case diskEvt:
				HandleDiskEvent(event);
				break;

			case osEvt:
				HandleSystemEvent(event);
				break;

			case kHighLevelEvent:
				HandleHighLevelEvent(event);
				break;

			default:
				HandleAlienEvent(event);
				break;
		}
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleHighLevelEvent: Assume that the high level event received was an
// Apple Event and process it. This will dispatch to us via MAGeneralDispatch.
//----------------------------------------------------------------------------------------
void TDispatcher::HandleHighLevelEvent(TToolboxEvent* event)
{
	SetupTheMenus();							// Make sure that the menus are setup
	// correctly before we handle the event

	AEProcessAppleEvent(&event->GetEventRecord(0));
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleKeyDownEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleKeyDownEvent(TToolboxEvent* event)
{
	GetTarget()->KeyEventToComponents(event);	// Find out what keys were _REALLY_ pressed 

	if (event->IsCommandKeyPressed())
		GetTarget()->HandleCommandKey(event);
	else
		GetTarget()->HandleKeyCommand(event);
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleKeyUpEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleKeyUpEvent(TToolboxEvent* event)
{
	// We don't think KeyUp events should affect the menus.
	event->SetAffectsMenus(false);

	GetTarget()->KeyEventToComponents(event);	// Find out what keys were _REALLY_ released 

	GetTarget()->HandleKeyUp(event);
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleMouseDown: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleMouseDown(TToolboxEvent* event)
{
	WindowRef aWMgrWindow;

	short whereMouseDown = FindWindow(event->GetWhere(), &aWMgrWindow);
	event->SetClickCount(CountClicks(event, whereMouseDown));

	TWindow * aWindow = TWindow::WMgrToWindow(aWMgrWindow);

	bool proceed = true;

	if ((whereMouseDown != inMenuBar) && InModalState() && (aWindow != (GetActiveWindow(kNoFloaters))))
	{
		//
		// Allow command drag of behind windows
		//
		if ((whereMouseDown != inDrag) || !event->IsCommandKeyPressed())
		{
			Beep(2);
			proceed = false;
		}
	}

	if (proceed)
		switch (whereMouseDown)
		{
			case inMenuBar:
				{								// 3.5
					SetupTheMenus();			// gives application a chance to setup individual menu items
					SetCursor(&(qd.arrow));
					long menuItem = MenuSelect(event->GetWhere());
					MenuEvent(HiWord(menuItem), LoWord(menuItem));
				}
				break;

			case inSysWindow:
				SystemClick(&event->GetEventRecord(0), (WindowPtr)aWMgrWindow);
				break;

			default:							// if a MacApp window was associated with the WindowRef then let the window object decide
				// what to do with the mouse click
				if (aWindow)
				{
					if (aWindow->Focus())		// if we can't focus, we're in trouble 
						aWindow->HandleMouseDown(aWindow->SuperViewToLocal(event->GetWhere()), event, GetStdHysteresis());
#if qDebug
					else
						::ProgramBreak_AC("In TDispatcher::HandleMouseDown: couldn't focus on a window object!");
#endif

				}
				else
					HandleAlienEvent(event);
				break;

		}

	fLastUpTime = TickCount();					// Because the toolbox often eats mouseUpEvents remember the time on the way out for
	// double/ triple detection.
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleMouseUp: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleMouseUp(TToolboxEvent* event)
{
	// now we _Really_ know what the last mouse up time is.
	// Remember time of last mouse up, in order to detect double/triple (multiple) clicks 
	fLastUpTime = event->GetWhen();

	WindowRef aWMgrWindow;
	short whereMouseDown = FindWindow(event->GetWhere(), &aWMgrWindow);

	switch (whereMouseDown)
	{
		case inMenuBar:
			break;

		case inSysWindow:
			break;

		default:								// if a MacApp window was associated with the WindowRef then let the window object decide
			{
				TWindow * aWindow = TWindow::WMgrToWindow(aWMgrWindow);

				// what to do with the mouse click
				if (aWindow)
				{
					if (aWindow->Focus())		// if we can't focus, we're in trouble 
						aWindow->HandleMouseUp(aWindow->SuperViewToLocal(event->GetWhere()), event, GetStdHysteresis());
#if qDebug
					else
						::ProgramBreak_AC("In TDispatcher::HandleMouseUp: couldn't focus on a window object!");
#endif

				}
				else
					HandleAlienEvent(event);
			}
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleSystemEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleSystemEvent(TToolboxEvent* event)
{
	switch (static_cast<unsigned long>(event->GetMessage() & osEvtMessageMask) >> 24)
	{
		case suspendResumeMessage:
			{
				bool switchingIn = ((event->GetMessage()) & resumeFlag) != 0;
				bool convertClipboard = ((event->GetMessage()) & convertClipboardFlag) != 0;

				if (switchingIn)
				{
					// The Process Manager may have activated the wrong window (if, when
					// we suspended earlier, we had non-floating window(s) that hide on suspend
					// such as the clipboard) so we augment what it did by activating the
					// right one!
					if (TWindow::MAFrontWindow())
						TWindow::MADeactivateWindow(TWindow::MAFrontWindow());

					RegainControl(convertClipboard);

					TWindow * aWindow = GetFrontWindow();
					if (aWindow)
						TWindow::MAActivateWindow(aWindow->GetWindowRef());
				}
				else
				{
					// When switching out, after having hid windows that get hid on suspend in
					// AboutToLoseControl, we need to ensure that our idea about the front
					// window and the Process Mgr's idea about the front window are the same.
					// (note that we do the second half of the work when we get the resume
					// event).
					WindowRef theFrontWindow = TWindow::MAFrontWindow();// remember the front window

					AboutToLoseControl(convertClipboard);// might hide windows

					TWindow * aWindow = GetActiveWindow(false);
					if (aWindow)
						TWindow::MADeactivateWindow(aWindow->GetWindowRef());
				}

				InvalidateMouseRegions();
			}
			break;

		case mouseMovedMessage:
			event->SetAffectsMenus(false);
			// We don't think mouse tracking usually
			// bothers the menus. We got the mouse
			// moved event because the mouse strayed
			// outside of fSleepRegion. It may or may
			// not have strayed outside of the other
			// mouse regions. We'll only invalidate
			// them here and force recomputation to get
			// the most accurate tracking.
			if (!fCursorRegion.Contains(event->GetWhere()))
				InvalidateCursorRgn();
			if (!fHelpRegion.Contains(event->GetWhere()))
				InvalidateHelpRgn();
			GetSleepRegion(&event->GetWhere()); // compute it now so it is most accurate
			break;

		default:
#if qDebugMsg
			if (gIntenseDebugging)
				ProgramReport_AC("in TDispatcher.HandleSystemEvent: got unrecognized event\n");
#endif

			break;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleToolboxEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleToolboxEvent(TToolboxEvent* event)
{
	try
	{
#if qDebug
		if (gReportEvent && event)
			event->ReportEvent();
#endif
		TBehavior * aBehavior = GetFirstEnabledBehavior();

		if (aBehavior && !aBehavior->DoToolboxEvent(event))
			DoToolboxEvent(event);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		DidEvent(event);
		throw;
	}	

	DidEvent(event);
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleUpdateEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleUpdateEvent(TToolboxEvent* event)
{
	// We don't think that update events should affect menus.
	event->SetAffectsMenus(false);

	TWindow * aWindow = TWindow::WMgrToWindow((WindowRef)event->GetMessage());
	if (aWindow)
		aWindow->Update();
	else
		HandleAlienEvent(event);
}

//----------------------------------------------------------------------------------------
// TDispatcher::Idle: 
//----------------------------------------------------------------------------------------
void TDispatcher::Idle(IdlePhase phase)
{
	try
	{
		if (phase == idleBegin)
		{
			if (!gInFilter)
			{
				if (MemSpaceIsLow())
					SpaceIsLowAlert();
				else
					fNextSpaceMessage = TickCount();
			}
		}

		else if (phase == idleContinue)
		{
#if qAttachable
			// Give an attached script some idle time
			IdleOSAScript();
#endif
		}

#if qWantsThreadManager
		// Threads live off of the idle time we give them.
		IdleThreads_AC(IsFrontProcess(), fIdleFreq);
#endif

		for (CHandlerIterator iter(fHeadCohandler); iter; ++iter)
			iter->HandleIdle(phase);
		
#if qDebug
		::Assertion_AC(GetTarget() != NULL, "GetTarget != nil");
#endif

		for (CHandlerIterator iter(GetTarget()); iter; ++iter)
			iter->HandleIdle(phase);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

#if qDebugMsg
		ProgramBreak_AC("Idle failed\n");
#endif
		throw;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetWaitTicks: 
//----------------------------------------------------------------------------------------
unsigned long TDispatcher::GetWaitTicks(bool allowApplicationToSleep)
{
	unsigned long returnValue = 0;

	if (allowApplicationToSleep)
	{
		unsigned long now = TickCount();
		unsigned long minNextIdleTime = now + kMaxIdleTime;

		// Check the cohandler chain first.  Add new block for failure handling
		{
			for (CHandlerIterator iter(fHeadCohandler); iter; ++iter)
				minNextIdleTime = Min_AC(iter->NextIdle(), minNextIdleTime);
		}

#if qDebug
		::Assertion_AC(GetTarget() != NULL, "GetTarget != nil");
#endif

		// now run through the target chain.  Add new block for failure handling
		{
			for (CHandlerIterator iter(GetTarget()); iter; ++iter)
				minNextIdleTime = Min_AC(iter->NextIdle(), minNextIdleTime);
		}

		unsigned long newTickCount = TickCount();
		if (minNextIdleTime > newTickCount)
			returnValue = Min_AC(minNextIdleTime - newTickCount, kMaxIdleTime);
	}

	return returnValue;
}

//----------------------------------------------------------------------------------------
// TDispatcher::InModalState: 
//----------------------------------------------------------------------------------------
bool TDispatcher::InModalState()
{
	bool returnVal = false;

	WindowRef aWindowPtr = FrontWindow();

	// in case the front window is an alert or something 
	if (aWindowPtr && !TWindow::WMgrToWindow(aWindowPtr))
		switch (GetWVariant(aWindowPtr))
		{
			case dBoxProc:
			case plainDBox:
			case altDBoxProc:
			case movableDBoxProc:
				returnVal = true;
		}
	else
	{
		TWindow * aWindow = GetActiveWindow(kNoFloaters);
		returnVal = (aWindow && aWindow->IsInModalState());
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDispatcher::InstallCohandler: 
//----------------------------------------------------------------------------------------
void TDispatcher::InstallCohandler(TEventHandler* aCohandler,
								   bool addIt)
{
	fHeadCohandler = addIt ? aCohandler->AddHandler(fHeadCohandler) : aCohandler->RemoveHandler(fHeadCohandler);
}

//----------------------------------------------------------------------------------------
// TDispatcher::IsHelpEnabled: 
//----------------------------------------------------------------------------------------
bool TDispatcher::IsHelpEnabled()
{
	return HMGetBalloons();
}

//----------------------------------------------------------------------------------------
// TDispatcher::InvalidateMouseRegions: 
//----------------------------------------------------------------------------------------
void TDispatcher::InvalidateMouseRegions()
{
	InvalidateCursorRgn();
	InvalidateHelpRgn();
}

//----------------------------------------------------------------------------------------
// TDispatcher::InvalidateCursorRgn: 
//----------------------------------------------------------------------------------------
void TDispatcher::InvalidateCursorRgn()
{
	if (fCursorRegion)
		SetEmptyRgn(fCursorRegion);				// Make sure it gets changed back 
}

//----------------------------------------------------------------------------------------
// TDispatcher::InvalidateHelpRgn: 
//----------------------------------------------------------------------------------------
void TDispatcher::InvalidateHelpRgn()
{
	if (fHelpRegion)
		SetEmptyRgn(fHelpRegion);				// Make sure it gets changed back 

	if (IsFrontProcess() && HMIsBalloon())
	{
		OSErr err = HMRemoveBalloon();
		if (err != hmNoBalloonUp)
			::ThrowIfOSErr_AC(err);
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::InvalidateFocus: 
//----------------------------------------------------------------------------------------
void TDispatcher::InvalidateFocus()
{
	if (TView::gFocusedView)
		TView::gFocusedView->InvalidateFocus();
}

//----------------------------------------------------------------------------------------
// TDispatcher::IsCursorRgnInvalid: 
//----------------------------------------------------------------------------------------
bool TDispatcher::IsCursorRgnInvalid()
{
	// The cursor is normally tracked via MouseMoved events. But for those with special
	// needs fAlwaysTrackCursor can come to the rescue.

	return (EmptyRgn(fCursorRegion) || fAlwaysTrackCursor);
}

//----------------------------------------------------------------------------------------
// TDispatcher::IsHelpRgnInvalid: 
//----------------------------------------------------------------------------------------
bool TDispatcher::IsHelpRgnInvalid()
{
	return EmptyRgn(fHelpRegion);
}

//----------------------------------------------------------------------------------------
// TDispatcher::KeyEventToComponents: 
//----------------------------------------------------------------------------------------
void TDispatcher::KeyEventToComponents(TToolboxEvent* event)
// See Tech Note #263 for the reason for this abomination 
{
	TCommandHandler::KeyEventToComponents(event);		// Get default translation, if any 

	if ((event->GetWhat() == keyDown) || (event->GetWhat() == keyUp) || (event->GetWhat() == autoKey))
	{
		// Now see if the command key is down. If it is, get the correct ASCII translation by
		// masking the command key out and re-translating because the command key will
		// mask the shift modifier.

		if (event->IsCommandKeyPressed())
		{
			enum
			{
				kMaskModifier = 0xFE00,	// need to strip command key from Modifiers 
				kMaskASCII1 = 0x000000FF,// get key from KeyTranslate return 
				kMaskASCII2 = 0x00FF0000,// get key from KeyTranslate return 
				kUpKeyMask = 0x0080
			};

			int maxCount = event->IsDoubleEvent() ? 2 : 1;
			CStr2_AC newText;

			for (int i = 0; i < maxCount; ++i)
			{
				// set the upkey bit so KeyTranslate doesn't do special deadkey processing
				// See IM-V pp. 195 
				short keyCodeParameter = ((((event->GetModifiers(i)) & kMaskModifier) | event->GetKeyCode()) | kUpKeyMask);

				// Get the correct keytable pointer. We don't want to grope the system unnecessarily so
				// use the script managers improvements if they're there.
				Ptr keyTransTable = (Ptr)(GetScriptManagerVariable(smKCHRCache));

				UInt32 state = 0;
				UInt32 keyInfo = KeyTranslate(keyTransTable, keyCodeParameter, &state);

				newText[i + 1] = static_cast<unsigned char>(keyInfo & kMaskASCII1);
				// Remember fText is a string, the text starts at [1]
				if (!newText[i + 1])
					newText[i + 1] = static_cast<unsigned char>((keyInfo & kMaskASCII2) >> 16);
				// Adjust the length if necessary (thanks to Ken V)
				if (newText[i+1] != 0)
					newText[0] = i+1;
			}
			
			event->SetText(newText);
		}
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::MainEventLoop: 
//----------------------------------------------------------------------------------------
void TDispatcher::MainEventLoop()
{
	fIdlePhase = idleBegin;
	while (!fDone)
		PollEvent(kAllowApplicationToSleep);
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoLaunchClipboard: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoLaunchClipboard()
{
	if (gClipboardMgr)
		gClipboardMgr->Launch();
}

//----------------------------------------------------------------------------------------
// TDispatcher::MakeViewForAlienClipboard: 
//----------------------------------------------------------------------------------------
TView* TDispatcher::MakeViewForAlienClipboard()
{
	return NULL;								// use defaults 
}

//----------------------------------------------------------------------------------------
// TDispatcher::MenuEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::MenuEvent(short theMenuNumber, short theItemNumber)
{
	if (theMenuNumber)
	{
		CommandNumber command = CommandFromMenuItem(theMenuNumber, theItemNumber);

#if qDebugMsg
		if (command == cCantUndo)
		{
			ProgramReport_AC(false, "Command number %d is reserved for MacApp.\n", cCantUndo);
			::ProgramBreak_AC("Use of reserved command number.");
		}

		if (gReportMenuChoices && (command > 0))
			ProgramReport_AC(false, "Menu Choice Command Number == %d\n", command);
#endif

		if ((command < 0) && (theMenuNumber == mApple))
		{
			CStr255_AC deskAccName;

			GetMenuItemText(MAGetMenuRef(mApple), theItemNumber, deskAccName);
			OpenDeskAccessory(deskAccName);
		}
		else if ((command < cEditBase) || (command > cEditLast) || (!SystemEdit(static_cast<short>(command - cEditBase))))
		{
			try
			{
				bool oldActivateBusyCursor;
				if (fSysWindowActive)
					oldActivateBusyCursor = ActivateBusyCursor(true);

				GetTarget()->HandleMenuCommand(command);

				if (fSysWindowActive)
					ActivateBusyCursor(oldActivateBusyCursor);
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				if (fSysWindowActive)
					ActivateBusyCursor(false);

				::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), CException_AC::BuildMessage(static_cast<short>(command), messageCommandError));
			}
			
		}
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::OpenDeskAccessory: 
//----------------------------------------------------------------------------------------
void TDispatcher::OpenDeskAccessory(const CStr255_AC& deskAccName)
{
	GrafPtr savedPort;

	GetPort(&savedPort);
	OpenDeskAcc(deskAccName);
	SetPort(savedPort);
}

//----------------------------------------------------------------------------------------
// TDispatcher::ProcessEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::ProcessEvent(TEvent* event)
{
#if qDebug
	::ThrowIfNonObject_AC(event);

	if (gIntenseDebugging)
		ProgramReport_AC(false, "The Event to process: %s\n", typeid(*event).name());
#endif

	if (event)
		event->Process();
}

//----------------------------------------------------------------------------------------
// TDispatcher::PollEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::PollEvent(bool allowApplicationToSleep)
{
	bool didAllowApplicationToSleep = fAllowApplicationToSleep;

	fAllowApplicationToSleep = allowApplicationToSleep;

	++fEventLevel;

#if qDebugMsg
	if (!GetTarget())
		::ProgramBreak_AC("Serious Error!! in TDispatcher.PollEvent: target == nil");
#endif

	try
	{
		TEvent * retrievedEvent = RetrieveAnEvent();
		if (retrievedEvent)
			ProcessEvent(retrievedEvent);

		// The desk scrap may have been changed by use of command-X or command-C in desk accessories.
		if (fSysWindowActive)
		{
			gClipboardMgr->CheckDeskScrap();
			InvalidateFocus();
		}

		--fEventLevel;

		if (!fEventLevel)
			gInhibitNestedHandling = false;		// All clear 
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

#if qDebugMsg
		if (theException.GetError() != noErr)
			ProgramReport_AC("\n");				// add a blank line after all the messages from Failure
#endif

		fAllowApplicationToSleep = didAllowApplicationToSleep;

		--fEventLevel;
		if (!fEventLevel)
		{
			if (theException.GetError() != noErr && theException.GetError() != userCanceledErr)
				ShowError(theException.GetError(), theException.GetMessage());

			InvalidateMenus();
		}
		else
			throw;
	}
	

	fAllowApplicationToSleep = didAllowApplicationToSleep;
}

//----------------------------------------------------------------------------------------
// TDispatcher::PollToolboxEvent: 
//----------------------------------------------------------------------------------------
bool TDispatcher::PollToolboxEvent(bool allowApplicationToSleep)
{
	unsigned long waitTicks = 0;
	RgnHandle sleepRegion = NULL;

	EventRecord theEvent;
	if (!::OSEventAvail(fMainEventMask, &theEvent))// We'll only sleep if there aren't any
	// waiting Events (keystrokes, mousedowns)
	{
		SetupTheMenus();

		// ensure that a mouse click or key stroke didn't come in during SetupTheMenus
		if (!::OSEventAvail(fMainEventMask, &theEvent))
		{
			sleepRegion = GetSleepRegion();

			// ensure that a mouse click or key stroke didn't come in during GetSleepRegion
			if (!::OSEventAvail(fMainEventMask, &theEvent))
				waitTicks = GetWaitTicks(allowApplicationToSleep);// calc last so it's most accurate 
			else
				sleepRegion = NULL;

		}
	}

	MAHiliteMenu(0);							// Cleanup any menu titles left hilited.
	// Done here so titles stay hilited until
	// synchronous actions are performed.
	TToolboxEvent * event = GetEvent(fMainEventMask, waitTicks, sleepRegion);
	bool gotAnEvent = event != NULL;
	if (gotAnEvent)
	{
		// GetEvent returned an event (work to do). If we were idling before then we must
		// keep the calls balanced with an IdleEnd to because we are no longer idling
		if (fIdlePhase == idleContinue)
		{
			Idle(idleEnd);
			fIdlePhase = idleBegin;
		}

		// Rather than posting the event where it would have to be sorted and compete with
		// other commands we will process the command directly.
		ProcessEvent(event);

		// The desk scrap may have been changed by use of command-X or command-C in desk accessories.
		if (fSysWindowActive)
		{
			gClipboardMgr->CheckDeskScrap();
			InvalidateFocus();
		}
	}
	else										// Woke up with no event available. Truly idle
		{
			Idle(fIdlePhase);
			fIdlePhase = idleContinue;
		}

#if qDebug
	gErrorParm3 = "?????";						// to prevent anyone from using old values 
#endif

	return gotAnEvent;
}

//----------------------------------------------------------------------------------------
// TDispatcher::PostAnEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::PostAnEvent(TEvent* event)	// Override
{
	try
	{
		if (event->IsReadyToPost())
			fEventList.Insert(event);			// inserts event ordered the list
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		if (event->ShouldFreeOnCompletion())
			delete event;
		throw;
	}
	
}

//----------------------------------------------------------------------------------------
// TDispatcher::PostCommand: 
//----------------------------------------------------------------------------------------
void TDispatcher::PostCommand(TCommand* command)// Override
{
	if (command)
	{
		if (command->CanBeUndone())
			TUndoHandler::fgUndoHandler.AddActionToHistory(command, command->GetIdentifier());

		PostAnEvent(command);
	}
}


//----------------------------------------------------------------------------------------
// TDispatcher::DidEvent: 
//----------------------------------------------------------------------------------------
void TDispatcher::DidEvent(TToolboxEvent* event)
{
	if (event && event->GetAffectsMenus())
		InvalidateMenus();

	bool temp = TemporaryAllocation(true);
#if qDebug
	if (!temp)
		::ProgramBreak_AC("The temporary allocation flag was left false.");
#endif

	if (event && event->ShouldFreeOnCompletion())
		delete event;
}

//----------------------------------------------------------------------------------------
// TDispatcher::RegainControl: 
//----------------------------------------------------------------------------------------
void TDispatcher::RegainControl(bool checkClipboard)
{
	Changed(mRegainControl, this);

	ActivateBusyCursor(true);

	gClipboardMgr->RegainControl(checkClipboard);

	// Let all windows know that we're regaining control - e.g. so floaters can show themselves 
	for (CWMgrIterator iter; iter; ++iter)
	{
		TWindow * aWindow = TWindow::WMgrToWindow(iter);

		if (aWindow)
			aWindow->RegainControl();
	}

	for (CDocumentIterator docIter(this); docIter.Current(); ++docIter)
		docIter->RegainControl();
	
	TMenuBarManager::fgMenuBarManager->RegainControl();
}

//----------------------------------------------------------------------------------------
// TDispatcher::InitialState: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoInitialState()
{
	try 
	{
		// Allocate the global dependency space gMacAppDependencySpace
		DoMakeDependencySpace();

		// Make the command that operates the main Event Loop 
		PostCommand(TH_new TEventRetrieverCommand);

		// Create the MenuBar manager object
		DoMakeMenuBarManager();

		// Create the view server
		DoMakeViewServer();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

#if qDebug
		::ProgramBreak_AC("Can't initialize a required dispatcher helper object!");
#endif
		gDispatcher = NULL;
		throw;
	}
}
	
//----------------------------------------------------------------------------------------
// TDispatcher::Run: 
//----------------------------------------------------------------------------------------
void TDispatcher::Run()
{
	FailSpaceIsLow();							// make sure we have enough memory to continue

	gInitialized = true;						// was set false at static init time 

	DoInitialState();

	DoLaunchClipboard();

	if (fWantKeyUpEvents)
		::SetEventMask(everyEvent);				// allow us to get keyup events too.

#if qWantsThreadManager
	// Something needs to idle and not sleep, it had better be us since we potentially have threads.
	if (fIdleFreq == kMaxIdleTime)
		SetIdleFreq(fIdleFreq - 1);
#endif

	fEventLevel = 0;							// Indicate outermost level 
	MainEventLoop();							// runs until a quit command 

	AboutToLoseControl(true);

	// See if previous max. resource usage has been exceeded by the termination code and
	// resources.
	CheckRsrcUsage();

	if (TPrintHandler::gPrintHandler)			// shut down the printing system
		TPrintHandler::gPrintHandler->TerminateUPrinting();

	TView::TerminateUView();					// shut down the view system

	// We must call CleanupMacApp here; if we wait to fall thru to the end of the main
	// program, A5 has been invalidated and we can't refer to any globals.
	CleanupMacApp();
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetMainEventMask: 
//----------------------------------------------------------------------------------------
short TDispatcher::GetMainEventMask()
{
	return fMainEventMask;
}

//----------------------------------------------------------------------------------------
// TDispatcher::SetMainEventMask: 
//----------------------------------------------------------------------------------------
short TDispatcher::SetMainEventMask(short newMask)
{
	short oldMask = fMainEventMask;
	fMainEventMask = newMask;

	return oldMask;
}

//----------------------------------------------------------------------------------------
// TDispatcher::SetupUndoRedoMenus: 
//----------------------------------------------------------------------------------------
void TDispatcher::SetupUndoRedoMenus()
{
	// setup the undo menu item
	CommandNumber undoCommand = TUndoHandler::fgUndoHandler.GetUndoID();
	Enable(cUndo, undoCommand != cNoCommand);
	SetUndoText(undoCommand);

	// setup the undo menu item
	CommandNumber redoCommand = TUndoHandler::fgUndoHandler.GetRedoID();
	Enable(cRedo, redoCommand != cNoCommand);
	SetRedoText(redoCommand);

	// setup the undo/redo menu item
	CommandNumber undoRedoCommand = undoCommand != cNoCommand ? undoCommand : redoCommand;
	bool cmdDone = undoCommand != cNoCommand; 
	Enable(cUndoRedo, undoRedoCommand != cNoCommand);
	SetUndoRedoText(cmdDone, undoRedoCommand != cNoCommand ? undoRedoCommand : cCantUndo);
}

//----------------------------------------------------------------------------------------
// TDispatcher::SetUndoText: 
//----------------------------------------------------------------------------------------
void TDispatcher::SetUndoText(CommandNumber id)
{
	if (fUndoCommand != id)
	{
		SetCommandName(cUndo, GetUndoRedoText(kShowUndo, id));
		fUndoCommand = id;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::SetRedoText: 
//----------------------------------------------------------------------------------------
void TDispatcher::SetRedoText(CommandNumber id)
{
	if (fRedoCommand != id)
	{
		SetCommandName(cRedo, GetUndoRedoText(kShowRedo, id));
		fRedoCommand = id;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::SetUndoRedoText: 
//----------------------------------------------------------------------------------------
void TDispatcher::SetUndoRedoText(bool cmdDone, CommandNumber id)
{
	if (fUndoRedoCommand != id || fUndoRedoState != cmdDone)
	{
		SetCommandName(cUndoRedo, GetUndoRedoText(cmdDone, id));
		fUndoRedoCommand = id;
		fUndoRedoState = cmdDone;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::SetupTheMenus: 
//----------------------------------------------------------------------------------------
void TDispatcher::SetupTheMenus()
{
		if (IsFrontProcess() && MenusHavePendingUpdate())
		{
			TMenuBarManager::fgMenuBarManager->Reset();			// reset the preferred menubar
	
			PerformMenuSetup(&DoSetupTheMenus, NULL);
	
			TMenuBarManager::fgMenuBarManager->InstallPreferredMenus();
			if (MenusHavePendingUpdate())
			{
				// Add the debugger menu
#if qDebug || qPerform
				AddMenuBar(kMBarDebug, false);
#endif
	
				// Recurse to get the menus correctly setup
				SetupTheMenus();
			}
		}
}

//----------------------------------------------------------------------------------------
// TDispatcher::ShowError: 
//----------------------------------------------------------------------------------------
void TDispatcher::ShowError(OSErr error,
							long message)
{
	ErrorAlert(error, message);
}

//----------------------------------------------------------------------------------------
// TDispatcher::SpaceIsLowAlert: 
//----------------------------------------------------------------------------------------
void TDispatcher::SpaceIsLowAlert()
{
	// Show 'space is low' alert only after ever fLowSpaceInterval ticks. 
	if (fLowSpaceInterval > 0 && IsFrontProcess() && TickCount() > fNextSpaceMessage)
	{
		gInhibitNestedHandling = true;		// Don't tell em again from the alert 
		StdAlert(phSpaceIsLow);
		fNextSpaceMessage = TickCount() + fLowSpaceInterval;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoSetCursor: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoSetCursor(CPoint_AC globalMouse,
							  RgnHandle cursorRegion)
{
	GetDefaultCursorRegion(globalMouse, cursorRegion);
	SetCursor(&(qd.arrow));
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleCursor: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleCursor(CPoint_AC globalMouse,
							   RgnHandle cursorRegion)
{
	WindowRef aWMgrWindow;
	if (FindWindow(globalMouse, &aWMgrWindow) == inContent)
	{
		TWindow * cursorWindow = TWindow::WMgrToWindow(aWMgrWindow);
		if (cursorWindow)
		{
			if (gDispatcher->GetFrontWindow() == cursorWindow || !gDispatcher->InModalState())
			{
				if (cursorWindow->HandlesCursor())
				{
					cursorWindow->HandleCursor(cursorWindow->SuperViewToLocal(globalMouse), cursorRegion);

					// Convert cursor region from window coords to global coords 
					cursorWindow->LocalToSuperView(cursorRegion);
				}
#if qDrag
				else if (HasDragManager_AC())
				{
					cursorWindow->HandleDragCursor(cursorWindow->SuperViewToLocal(globalMouse), cursorRegion);

					cursorWindow->LocalToSuperView(cursorRegion);
				}
#endif

				cursorWindow->Focus();			// leave the window origin and clip set
				// so that external utilities such as
				// English Speech Recognition will find
				// controls unclipped.
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::TrackCursor: 
//----------------------------------------------------------------------------------------
void TDispatcher::TrackCursor(CPoint_AC globalMouse)
{
	try
	{
#if qDebugMsg
		if (PtInRgn(globalMouse, fCursorRegion))
		{
			if (gIntenseDebugging)
				ProgramReport_AC("cursor is in cursor region\n");
		}
#endif

		InvalidateCursorRgn();

		// Find out if the cursor is in a window which handles the cursor, and if so, call
		// HandleCursor to allow a view to claim the cursor, and compute a region
		TView* oldSetCursorView = TView::fgSetCursorView;
		TView::fgSetCursorView = NULL;

		HandleCursor(globalMouse, fCursorRegion);

		if (EmptyRgn(fCursorRegion))
			DoSetCursor(globalMouse, fCursorRegion);

		if (TView::fgSetCursorView != oldSetCursorView)
		{
			if (oldSetCursorView)
				oldSetCursorView->HandleUnCursor();
		}

#if qDebugMsg
		if (gIntenseDebugging)
			if (!fCursorRegion)
				ProgramReport_AC("fCursorRegion is nil\n");
#endif

#if qDebug
		if (gIntenseDebugging && !PtInRgn(globalMouse, fCursorRegion))
		{
			CRect_AC bBox = (*fCursorRegion)->rgnBBox;
			ProgramReport_AC("Whoops, cursor region was not correctly calculated.\n");
			ProgramReport_AC(false, "global cursor = ", (const char*)(globalMouse));
			ProgramReport_AC(false, "  (*fCursorRegion)->rgnBBox = ", (const char*)(bBox));
			ProgramReport_AC("\n");
			::ProgramBreak_AC("The cursor is not in the cursor region at end of TDispatcher.TrackCursor!");
		}
#endif
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

#if qDebugMsg
		ProgramReport_AC("TrackCursor failed\n");
#endif

		SetEmptyRgn(fCursorRegion);
		// don't ReSignal
	}
	
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoShowHelp: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoShowHelp(CPoint_AC globalMouse,
							 RgnHandle helpRegion)
{
	if (HMIsBalloon())
	{
		OSErr err = HMRemoveBalloon();
		if (err != hmNoBalloonUp)
			::ThrowIfOSErr_AC(err);
	}
	GetDefaultHelpRegion(globalMouse, helpRegion);
}

//----------------------------------------------------------------------------------------
// TDispatcher::HandleHelp: 
//----------------------------------------------------------------------------------------
void TDispatcher::HandleHelp(CPoint_AC globalMouse,
							 RgnHandle helpRegion)
{
	WindowRef aWMgrWindow;
	if (FindWindow(globalMouse, &aWMgrWindow) == inContent)
	{
		TWindow * helpWindow = TWindow::WMgrToWindow(aWMgrWindow);
		if ((helpWindow) && helpWindow->HandlesHelp())
		{
			helpWindow->HandleHelp(helpWindow->SuperViewToLocal(globalMouse), helpRegion);

			// Convert helpRegion region from window coords to global coords 
			helpWindow->LocalToSuperView(helpRegion);
		}
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::TrackHelp: 
//----------------------------------------------------------------------------------------
void TDispatcher::TrackHelp(CPoint_AC globalMouse)
{

#if qDebugMsg
	if (PtInRgn(globalMouse, fHelpRegion))
	{

		if (gIntenseDebugging)
			ProgramReport_AC("cursor is in help region\n");
	}
#endif

	InvalidateHelpRgn();

	// Find out if the cursor is in a window which handles help, and if so, call
	// HandleHelp to allow some subview to show help, and compute a region
	HandleHelp(globalMouse, fHelpRegion);

	if (EmptyRgn(fHelpRegion))
		DoShowHelp(globalMouse, fHelpRegion);

#if qDebugMsg
	if (gIntenseDebugging)
		if (!fHelpRegion)
			ProgramReport_AC("fHelpRegion is nil\n");
#endif

#if qDebug
	if (gIntenseDebugging && !PtInRgn(globalMouse, fHelpRegion))
	{
		CRect_AC bBox = (*fHelpRegion)->rgnBBox;
		ProgramReport_AC("Whoops, help region was not correctly calculated.\n");
		ProgramReport_AC(false, "global cursor = ", (const char*)(globalMouse));
		ProgramReport_AC(false, "  (*fHelpRegion)->rgnBBox = ", (const char*)(bBox));
		ProgramReport_AC("\n");
		::ProgramBreak_AC("The cursor is not in the help region at end of TDispatcher.TrackHelp!");
	}
#endif

}

//----------------------------------------------------------------------------------------
// TDispatcher::TrackMouse: 
//----------------------------------------------------------------------------------------
TTracker* TDispatcher::TrackMouse(const CViewPoint& theMouse,
								  CPoint_AC hysteresis,
								  TTracker* theCommand)
{
	TTracker* currentTracker = NULL;

	try
	{
		currentTracker = theCommand->HandleTrackBegin(theMouse, hysteresis);
		theCommand = NULL;					// so failure handler works

		while (currentTracker && !currentTracker->IsDoneTracking())
			if (IsFrontProcess())
				currentTracker = currentTracker->HandleTrackContinue();

		if (currentTracker)
			currentTracker = currentTracker->HandleTrackEnd();

	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		if (theCommand)
		{
			theCommand->Completed();

			if (theCommand->ShouldFreeOnCompletion())
				delete theCommand;
		}

		if (currentTracker)
		{
			currentTracker->Completed();

			if (currentTracker->ShouldFreeOnCompletion())
				delete currentTracker;
		}

		throw;
	}
	

	return currentTracker;
}

//----------------------------------------------------------------------------------------
// TDispatcher::UpdateAllWindows: 
//----------------------------------------------------------------------------------------
void TDispatcher::UpdateAllWindows()
{
	// Something bad could happen here if for some reason the event queue was blocked (constipated?)
	// or something, causing us to loop forever. Let's put an unreasonable upper bound on the number
	// of events we'll handle in this loop.
	try
	{
		int maxEvents = 500;
		TToolboxEvent * event;
	
		++fEventLevel;
		while ((event = GetEvent(updateMask | activMask, 0, NULL)) != NULL && --maxEvents)
			event->Process();
		--fEventLevel;
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		--fEventLevel;
		
		throw;
	}
}

#if qBackwardCompatible
//----------------------------------------------------------------------------------------
// TDispatcher::WMgrToWindow: 
//----------------------------------------------------------------------------------------
TWindow* TDispatcher::WMgrToWindow(WindowRef aWMgrWindow)
{
	// just uses the global routine in UWindow
	return TWindow::WMgrToWindow(aWMgrWindow);
}
#endif

//----------------------------------------------------------------------------------------
// TDispatcher::GetApplicationName: 
//----------------------------------------------------------------------------------------
void TDispatcher::GetApplicationName(CStr255_AC& theName)
{
	try
	{
		ProcessInfoRec info;
		info.processInfoLength = sizeof(ProcessInfoRec);
		info.processName = theName;
		info.processAppSpec = NULL;
		::ThrowIfOSErr_AC(GetProcessInformation(&fProcessNumber, &info));
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

#if qDebug
		::ProgramBreak_AC("TDispatcher::GetApplicationName() failed!");
#endif

		// no need to fail completely, just return generic string
		theName = "<Application>";

		// Don't ReSignal
	}
	
}

//----------------------------------------------------------------------------------------
// TDispatcher::DoUndoRedo: 
//----------------------------------------------------------------------------------------
bool TDispatcher::DoUndoRedo(TCommandHandler* theContext,
								CommandNumber aCommandNumber,
								MScriptableObject* theTarget)
// Handles the undo choice from the menu. If you want undo/redo to be recordable
// your DoMenuCommand method should call this on cUndo.
{
	bool returnVal = false;

	TCommand * lastCommand = theContext->GetLastCommand();
	if (lastCommand && (lastCommand->GetContext() == theContext))
	{
		TAppleEvent * theEvent = TH_new TAppleEvent(kAEMiscStandards, lastCommand->GetCommandDone() ? static_cast<AEEventID>(kAEUndo) : static_cast<AEEventID>(kAERedo), gServerAddress, kAEWaitReply);

		CTempDesc targetDesc;
		theTarget->MakeObjectSpecifier(targetDesc, theTarget->GetSpecifierForm());
		theEvent->WriteParameter(keyDirectObject, targetDesc);

		PostCommand(TH_new TClientCommand(aCommandNumber, theContext, false, false, theContext, theEvent));
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDispatcher::UndoRedoInContext: 
//----------------------------------------------------------------------------------------
bool TDispatcher::UndoRedoInContext(TCommandHandler* /*theContext*/,
									CommandNumber aCommandNumber)
// Handles the undo event. TDispatcher will handle this for itself, but if
// you handle undo/redo in another context you should call this from that object's
// DoScriptCommand method.
{
	bool completed = false;
	if (aCommandNumber == cAEUndo || aCommandNumber == cAERedo)
	{
		CTempAllocation tempAllocation(true);
		CTempObjectAllocation tempObjectAllocation(true);
		
		// Generate the proper undo or redo command.
		TUndoRedoCommand * anUndoOrRedoCommand = TH_new TUndoRedoCommand(aCommandNumber);
		anUndoOrRedoCommand->Process();

		completed = true;
	}

	return completed;
}

//----------------------------------------------------------------------------------------
// TDispatcher::CountContainedObjects: 
//----------------------------------------------------------------------------------------
long TDispatcher::CountContainedObjects(DescType desiredType)
{
	// Counts the number of windows or documents the application has open.

	long result = 0;

	switch (desiredType)
	{
		case cMenu:
			result = CountMenus(TMenuBarManager::fgMenuBarManager->fDisplayedMenus);
			break;

		case cPalette:
		case cWindow:
		{
			// Go through the window list and count the visible windows.
			for (CWMgrIterator iter; iter; ++iter)
			{
				TWindow * aWindow = TWindow::WMgrToWindow(iter);
				if ((aWindow) && aWindow->IsShown() && (aWindow->GetOMClass() == desiredType))
					++result;
			}
			break;
		}

		default:
		{
			// Go through the document list and count the documents of this type.
			for (CNoGhostDocsIterator iter(this); iter.Current(); ++iter)
				if ((iter->GetOMClass() == desiredType) || (desiredType == cDocument))
					++result;
			break;
		}
	}

	return result;
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TDispatcher::HandleOSAEvent:
//----------------------------------------------------------------------------------------
bool TDispatcher::HandleOSAEvent(CommandNumber aCommandNumber,
									TAppleEvent* message,
									TAppleEvent* reply)
{
	// Don't give OSA the run event so that scripts won't have to continue it.
	return (aCommandNumber != cFinderNew) && MScriptableObject::HandleOSAEvent(aCommandNumber, message, reply);
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TDispatcher::DoScriptCommand: 
//----------------------------------------------------------------------------------------
void TDispatcher::DoScriptCommand(CommandNumber aCommandNumber,
								  TAppleEvent* message,
								  TAppleEvent* reply)
{
	TEventHandler * theTarget = GetTarget();

	switch (aCommandNumber)
	{
		case cAEUndo:
			if (CommandEnabled(cUndo))
				UndoRedoInContext(this, aCommandNumber);
			break;
		
		case cAERedo:
			if (CommandEnabled(cRedo))
				UndoRedoInContext(this, aCommandNumber);
			break;

		case cAEPaste:
			if (CommandEnabled(cPaste))
				theTarget->HandleMenuCommand(cPaste);
			break;

		case cAECut:
			if (CommandEnabled(cCut))
				theTarget->HandleMenuCommand(cCut);
			break;

		case cAECopy:
			if (CommandEnabled(cCopy))
				theTarget->HandleMenuCommand(cCopy);
			break;

		default:
			MDefaultScriptableObject::DoScriptCommand(aCommandNumber, message, reply);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::SetObjectProperty: 
//----------------------------------------------------------------------------------------
void TDispatcher::SetObjectProperty(const CAEDesc& thePropertyValue,
									DescType whichProperty)
{
	CAEDesc localPropValue = thePropertyValue;
	switch (whichProperty)
	{
		case pClipboard:
			{
				gClipboardMgr->AboutToLoseControl(true);// so scrap gets converted
				::ThrowIfOSErr_AC(static_cast<OSErr>(CClipboard_AC::ZeroScrap()));
				if (localPropValue.GetDescriptorType() == typeAEList)
				{
					// Put all the elements of the list on the scrap
					long count = 0;
					::ThrowIfOSErr_AC(AECountItems(localPropValue, &count));
					while (count)
					{
						CTempDesc theScrapData;
						DescType theAEKeyword;
						::ThrowIfOSErr_AC(AEGetNthDesc(localPropValue, count--, typeWildCard, &theAEKeyword, theScrapData));
						gClipboardMgr->PutDeskScrapData(theScrapData.GetDescriptorType(), theScrapData.GetDataHandle());
					}
				}
				else
				{
					gClipboardMgr->PutDeskScrapData(localPropValue.GetDescriptorType(), localPropValue.GetDataHandle());
				}
				gClipboardMgr->RegainControl(true);// so scrap gets converted
			}
			break;

		case pName:
		case pIsFrontProcess:
		case pVersion:
		case pUserSelection:
			::ThrowIfOSErr_AC(errAECantSetReadOnly);
			break;

		default:
			MScriptableObject::SetObjectProperty(thePropertyValue, whichProperty);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetObjectProperty: 
//----------------------------------------------------------------------------------------
bool TDispatcher::GetObjectProperty(CAEDesc& thePropertyValue,
									   DescType whichProperty,
									   const CAEDesc& desiredType)
{
	// Return properties for the application.
	bool hasProperty = true;

	try
	{
		switch (whichProperty)
		{
			case pClipboard:
				{
					// Since we don't know what's on the scrap and we're trying to return
					// its contents in a generic way, we'll get the desk scrap handle
					// and walk down it grabbing each scrap type we come upon.
					bool inFront = IsFrontProcess();
					if (inFront)
					{
						gClipboardMgr->AboutToLoseControl(true);// so scrap gets converted
						gClipboardMgr->RegainControl(true);// so scrap gets converted
					}
					::ThrowIfOSErr_AC(static_cast<OSErr>(CClipboard_AC::LoadScrap()));
					PScrapStuff pScrpInf = CClipboard_AC::InfoScrap();
					Handle theScrap = pScrpInf->scrapHandle;

					CTempHandle_AC theScrapData(NewPermHandle(0));

					THandleStream theScrapStream(theScrap, 0);

					thePropertyValue.CreateList();

					while ((theScrapStream.GetPosition() + 8) < pScrpInf->scrapSize)
					{
						DescType theScrapType = (DescType)theScrapStream.ReadLong();
						long theLength = theScrapStream.ReadLong();
						SetPermHandleSize(theScrapData, theLength);
						::ThrowIfMemError_AC();

						{
							// scoping for temphandlelock
							CTempHandleLock_AC tempHandleLock(theScrapData);
							theScrapStream.ReadBytes(*(Handle)theScrapData, theLength);
						}
						CAEDesc theScrapDesc(theScrapType, theScrapData);
						::ThrowIfOSErr_AC(AEPutDesc(thePropertyValue, 0, theScrapDesc));
					}
				}
				break;

			case pName:
				{
					CStr255_AC theAppName;
					GetApplicationName(theAppName);
					thePropertyValue.PutString(theAppName);
				}
				break;

			case pIsFrontProcess:
				thePropertyValue.PutBoolean(IsFrontProcess());
				break;

			case pVersion:
				{
					CTempUseResFile_AC tempUseResFile(gApplicationRefNum);// set this resource to be current
					Handle theVersResource = (Handle)Get1Resource((ResType)'vers', 1);
					::ThrowIfResError_AC();
					CTempHandleLock_AC tempHandleLock(theVersResource, eDontMoveHi, eReleaseIt, eLockIt);
					CAEDesc propData;
					::ThrowIfOSErr_AC(AECreateDesc(typeVersion, *theVersResource, GetHandleSize(theVersResource), propData));
					thePropertyValue = propData;
				}
				break;

			case pUserSelection:
				{
					TWindow * theFrontWindow = GetFrontWindow();
					if (theFrontWindow)
					{
						TDocument * theDoc = theFrontWindow->GetDocument();
						if (theDoc)
						{
							TDesignator * theUserSelection = theDoc->GetUserSelection();
							::ThrowIfNonObject_AC(theUserSelection);

							CAEDesc propData;
							theUserSelection->MakeObjectSpecifier(propData, formPropertyID);
							thePropertyValue = propData;
						}
					}
				}
				break;

			default:
				hasProperty = MScriptableObject::GetObjectProperty(thePropertyValue, whichProperty, desiredType);
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		// we failed while trying to get the property
		hasProperty = false;
	}
	

	return hasProperty;
}

//----------------------------------------------------------------------------------------
// TDispatcher::MakeObjectSpecifier: 
//----------------------------------------------------------------------------------------
bool TDispatcher::MakeObjectSpecifier(CAEDesc& theObjectSpecifier,
										 DescType/*preferredForm*/ )
{
	//return (AECreateDesc(typeNull, NULL, 0, theObjectSpecifier) == noErr);
	::ThrowIfOSErr_AC(AECreateDesc(typeNull, NULL, 0, theObjectSpecifier));
	return true;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetContainedObject: 
//----------------------------------------------------------------------------------------
MScriptableObject* TDispatcher::GetContainedObject(DescType desiredType,
												   DescType selectionForm,
												   const CAEDesc& selectionData)
{
	MScriptableObject * result = NULL;

	if (desiredType == cFile)
	{
		// When you script 'file "Macintosh HD:My File"' AppleScript will
		// give the app a pathname and expect a file object in return.
		TFile * resultFile = DoMakeFile(cOpen);
		CStr255_AC thePathName;
		selectionData.GetString(thePathName);
		resultFile->SpecifyWithTrio(0, 0, (CStr63_AC &)thePathName);
		TOSADispatcher::fgDispatcher->AddTemporaryToken(resultFile);

		result = resultFile;
	}
	else if (IsDocumentClass(desiredType) && selectionForm == formName)
	{
		// We know we're going after a document with a 
		// specific name so just scan the document list.
		CStr255_AC theDocName;
		selectionData.GetString(theDocName);

		for (CNoGhostDocsIterator iter(this); iter.Current(); ++iter)
			if (iter->GetOMClass() == desiredType && iter->GetTitle() == theDocName)
			{
				result = iter;
				break;
			}
	}
	else
		result = MScriptableObject::GetContainedObject(desiredType, selectionForm, selectionData);

#if qALOE
	if (result != NULL)
	{
		if (desiredType == cWindow)
		{
			TWindow* window = dynamic_cast<TWindow*>(result);
			if (window)
				SetCurrentResolutionContext_AC(window->GetWindowRef());
		}
		else if (IsDocumentClass(desiredType))
		{
			TDocument* document = dynamic_cast<TDocument*>(result);
			if (document)
			{
				// get toolbox window of first window of doc
				// and call ALOESetCurrentResolutionContext
				// TWindow* window = document->
			}
		}
	}
#endif // qALOE
	return result;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetIndContainedObject: 
//----------------------------------------------------------------------------------------
MScriptableObject* TDispatcher::GetIndContainedObject(DescType desiredType,
													  long index)
{
	MScriptableObject * indexedObject = NULL;	// no such menu

	switch (desiredType)
	{
		case cWindow:
			indexedObject = GetWindowByIndex(static_cast<short>(index));
			break;

		case cPalette:
			indexedObject = GetPaletteByIndex(static_cast<short>(index));
			break;

		case cMenu:
			{
				// Create a menu accessor to handle events for this menu
				MenuRef theMenuRef = GetIndMenu(TMenuBarManager::fgMenuBarManager->fDisplayedMenus, static_cast<short>(index));
				if (theMenuRef)
				{
					TMenuAccessor * theAccessor = TH_new TMenuAccessor(theMenuRef, static_cast<short>(index));
					TOSADispatcher::fgDispatcher->AddTemporaryToken(theAccessor);
					indexedObject = theAccessor;
				}
			}
			break;

		default:
			if (IsDocumentClass(desiredType))
				indexedObject = GetDocumentByIndex(desiredType, static_cast<short>(index));
			else
				indexedObject = MScriptableObject::GetIndContainedObject(desiredType, index);
			break;
	}

	return indexedObject;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetWindowByIndex: 
//----------------------------------------------------------------------------------------
TWindow* TDispatcher::GetWindowByIndex(short wIndex)
{
	TWindow * indexedWindow = NULL;

	short windowCount = 0;

	for (CWMgrIterator iter; iter; ++iter)
	{
		if (!TWindow::IsFloatWindow(iter))
		{
			TWindow * tempWindow = TWindow::WMgrToWindow(iter);
			if (tempWindow && tempWindow->IsShown())
			{
				++windowCount;
				if (windowCount == wIndex)
				{
					indexedWindow = tempWindow;
					break;
				}
			}
		}
	}

	return indexedWindow;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetPaletteByIndex: 
//----------------------------------------------------------------------------------------
TWindow* TDispatcher::GetPaletteByIndex(short index)
{
	TWindow * indexedPalette = NULL;

	short paletteCount = 0;

	for (CWMgrIterator iter; iter; ++iter)
	{
		if (TWindow::IsFloatWindow(iter))
		{
			TWindow * tempWindow = TWindow::WMgrToWindow(iter);
			if (tempWindow && tempWindow->IsShown())
			{
				++paletteCount;
				if (paletteCount == index)
				{
					indexedPalette = tempWindow;
					break;
				}
			}
		}
	}

	return indexedPalette;
}

//----------------------------------------------------------------------------------------
// TDispatcher::GetDocumentByIndex: 
//----------------------------------------------------------------------------------------
TDocument* TDispatcher::GetDocumentByIndex(DescType desiredType,
										   short desiredIndex)
{
	// Go through the window list and count to the Nth window associated with a document
	// of the desired type.  Only count the first window for each document just
	// in case a document has more than one window.

	TDocument*	aDocument = NULL;

	TWindow*	aWindow = GetFrontWindow();
	short		documentsSeen = 0;
	if (aWindow)
	{
		// Make a list for all of the documents we find until we find the one we want.
		CList_AC aDocList;
		WindowRef	aWindowRef = aWindow->GetWindowRef();

		while (aWindowRef && (documentsSeen < desiredIndex))
		{
			aWindow = TWindow::WMgrToWindow(aWindowRef);
			if (aWindow)
			{
				aDocument = aWindow->GetDocument();
				if (aDocument && (aDocument->GetOMClass() == desiredType))
				{
					// See if this document is already in the list.
					if (!aDocList.GetIdentityItemNo(aDocument))
					{
						++documentsSeen;
						aDocList.InsertLast(aDocument);
					}
				}
			}
			aWindowRef = GetNextWindow(aWindowRef);
		}
	}
	return (documentsSeen == desiredIndex) ? aDocument : NULL;
}

//----------------------------------------------------------------------------------------
// TDispatcher::IsDocumentClass: 
//----------------------------------------------------------------------------------------
bool TDispatcher::IsDocumentClass(DescType desiredType)
{
	return desiredType == cDocument;
}

//----------------------------------------------------------------------------------------
// TDispatcher::ReportReplyError: 
//----------------------------------------------------------------------------------------
void TDispatcher::ReportReplyError(CommandNumber/*fromCmdNum*/ ,
								   OSErr theError,
								   const CStr255_AC& theErrStr)
{
	// When a TClientCommand gets an error back from an AppleEvent it calls this
	// method so the application can report the error on the client side.
	if (theErrStr.Length())
	{
		ParamText(theErrStr, gEmptyString, gEmptyString, gEmptyString);
		StdAlert(phUnknownErr);
		ResetAlertStage();
	}
	else if (theError != noErr)
		ErrorAlert(theError, 0);
}

//----------------------------------------------------------------------------------------
// End of UDispatcher.cp 
