//----------------------------------------------------------------------------------------
// UDragDrop.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//---------------------------------------------------------------------------------------- 

// These includes needed for uses outside of qDrag.
#ifndef __UDRAGDROP__
#include "UDragDrop.h"
#endif

#ifndef __UCLASSDESC__
#include "UClassDesc.h"
#endif

#ifndef __UDRAGDROPBEHAVIOR__
#include "UDragDropBehavior.h"
#endif

#if qDrag

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

#ifndef __CStreamContext_AC__
#include "CStreamContext_AC.h"
#endif


#include "CBuffer_AC.h"
#include "CLinkedList_AC.h"

// MacApp

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __UUNDO__
#include "UUndo.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#if qPowerPC || qModelCFM
#ifndef __CODEFRAGMENTS__
#include <CodeFragments.h>
#endif

#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FOLDERS__
#include <Folders.h>
#endif


class TDragFlavorStreamRep : public CRandomAccessStreamRep_AC
{
public:
	TDragFlavorStreamRep(FlavorType theFlavor, ItemReference theReference);
	
	virtual ~TDragFlavorStreamRep();
	
	//------------------------------------------------------------------------------------
	// Access:  
	//------------------------------------------------------------------------------------

	virtual long GetPosition() const;
	
	virtual void SetPosition(long newPosition);
		
	virtual long GetSize() const;
		
	virtual void ReadBytes(void* p, long count);
	
	virtual void WriteBytes(const void* p, long count);
	
protected:
	ItemReference	fItemReference;
	FlavorType		fFlavorType;
	long 			fPosition;	// offset into stream
	
};

//========================================================================================
// Globals
//========================================================================================

// public globals
CAutoPtr_AC<TDragDropSession> TDragDropSession::fgDragDropSession;

// private globals
TemplateTempRoutineDescriptor<DragTrackingHandlerUPP> TDragDropSession::fDragTrackingHandlerUPP;
TemplateTempRoutineDescriptor<DragReceiveHandlerUPP> TDragDropSession::fDragReceiveHandlerUPP;
TemplateTempRoutineDescriptor<DragSendDataUPP> TDragDropSession::fDragSendDataUPP;

#if qDebug
short TDragFlavorStream::fInstanceCount;		// initialize static refcount of TDragFlavorStreams
#endif // qDebug

#endif // qDrag

//----------------------------------------------------------------------------------------
// InitUDragManager
//----------------------------------------------------------------------------------------


void InitUDragManager()
{
#if qDrag
	if (HasDragManager_AC())						// balk if drag manager isn't present
	{
		TDragDropSession::fgDragDropSession = TH_new TDragDropSession;
	}
#endif
	MA_REGISTER_SIGNATURE(TDragDropBehavior, kDragDropBehavior);
}

#if qDrag

//========================================================================================
// CLASS TDragDropSession
//========================================================================================
//----------------------------------------------------------------------------------------
// TDragDropSession::TDragDropSession
//----------------------------------------------------------------------------------------


TDragDropSession::TDragDropSession() :
	fDropTarget(NULL),
	fDragSource(NULL),
	fScroller(NULL),
	fSourceWindow(NULL),
	fTargetWindow(NULL),
	fDragReference(0L),
	fDragLeftSourceView(false),
	fTargetIsHilited(false),
	fUserRequestedCopy(false)
{
	// create universal proc pointers for the callbacks
	fDragTrackingHandlerUPP = NewDragTrackingHandlerProc(TDragDropSession::DragTrackingHandlerGlue);
	::ThrowIfNULL_AC(fDragTrackingHandlerUPP);

	fDragReceiveHandlerUPP = NewDragReceiveHandlerProc(TDragDropSession::DragReceiveHandlerGlue);
	::ThrowIfNULL_AC(fDragReceiveHandlerUPP);

	fDragSendDataUPP = NewDragSendDataProc(TDragDropSession::DragSendDataProcGlue);
	::ThrowIfNULL_AC(fDragSendDataUPP);
}

//----------------------------------------------------------------------------------------
// TDragDropSession::~DragDropSession
//----------------------------------------------------------------------------------------


TDragDropSession::~TDragDropSession()
{
}

//----------------------------------------------------------------------------------------
// TDragDropSession::DragTrackingHandlerGlue
//----------------------------------------------------------------------------------------


pascal OSErr TDragDropSession::DragTrackingHandlerGlue(DragTrackingMessage message,
													   WindowRef theWindowMgrWindow,
													   void* handlerRefCon,
													   DragReference theDragRef)
{
#if q68K
	// Set and restore A5 for compatibility with AOCE
	long A5RegisterOnEntry = SetCurrentA5();
#endif

	OSErr err = noErr;

	// scoped for destruction of Metrowerks destruction chain before restoring A5
	{

		// it is necessary for a failure handler to exist at this level to prevent
		// blowing away the failure handler stack while in drag manager callbacks
		try
		{
			fgDragDropSession->DragTrackingHandler(message, theWindowMgrWindow, handlerRefCon, theDragRef);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			err = theException.GetError();					// don't resignal - we're in a callback
		}
		
	}
#if q68K
	SetA5(A5RegisterOnEntry);
#endif

	return err;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::DragReceiveHandlerGlue
//----------------------------------------------------------------------------------------


pascal OSErr TDragDropSession::DragReceiveHandlerGlue(WindowRef theWindowMgrWindow,
													  void* handlerRefCon,
													  DragReference theDragRef)
{
#if q68K
	// Set and restore A5 for compatibility with AOCE
	long A5RegisterOnEntry = SetCurrentA5();
#endif

	OSErr err = noErr;

	// scoped for destruction of Metrowerks destruction chain before restoring A5
	{
		// it is necessary for a failure handler to exist at this level to prevent
		// blowing away the failure handler stack while in drag manager callbacks
		try
		{	
			err = fgDragDropSession->DragReceiveHandler(theWindowMgrWindow, handlerRefCon, theDragRef);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			err = theException.GetError();					// don't resignal - we're in a callback
		}
		

#if qDebug
		TDragFlavorStream::CheckInstanceCount();// test the TDragFlavorStream instance count
#endif

	}
#if q68K
	SetA5(A5RegisterOnEntry);
#endif

	return err;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::DragSendDataProcGlue
//----------------------------------------------------------------------------------------


pascal OSErr TDragDropSession::DragSendDataProcGlue(FlavorType theType,
													void* dragSendDataRefCon,
													ItemReference theItemRef,
													DragReference theDragRef)
{
#if q68K
	// Set and restore A5 for compatibility with AOCE
	long A5RegisterOnEntry = SetCurrentA5();
#endif

	OSErr err = noErr;

	// scoped for destruction of Metrowerks destruction chain before restoring A5
	{

		// it is necessary for a failure handler to exist at this level to prevent
		// blowing away the failure handler stack while in drag manager callbacks
		try
		{
			fgDragDropSession->DragSendDataProc(theType, dragSendDataRefCon, theItemRef, theDragRef);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			err = theException.GetError();					// don't resignal
		}
	}
#if q68K
	SetA5(A5RegisterOnEntry);
#endif

	return err;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::DragTrackingHandler
//----------------------------------------------------------------------------------------


void TDragDropSession::DragTrackingHandler(DragTrackingMessage message,
										   WindowRef/*theWindowMgrWindow*/ ,
										   void* handlerRefCon,
										   DragReference theDragRef)
{
	fDragReference = theDragRef;

	switch (message)
	{
		case dragTrackingEnterHandler:
			HandleEnterHandler();
			break;

		case dragTrackingLeaveHandler:
			HandleLeaveHandler();
			break;

		case dragTrackingEnterWindow:
			HandleEnterWindow(static_cast<TWindow*>(handlerRefCon));
			break;

		case dragTrackingLeaveWindow:
			HandleLeaveWindow();
			break;

		case dragTrackingInWindow:
			HandleTrackInWindow();
			break;

		default:
			break;
	}
}

//----------------------------------------------------------------------------------------
// TDragDropSession::DragReceiveHandler
//----------------------------------------------------------------------------------------


OSErr TDragDropSession::DragReceiveHandler(WindowRef/*theWindowMgrWindow*/ ,
										   void*/*handlerRefCon*/  ,
										   DragReference theDragRef)
{
	OSErr returnValue = noErr;

	if (fDropTarget)
	{
		CTempDesc dropLocationDesc;
		TCommand * sourceCommand = NULL;
		TCommand * targetCommand = NULL;

		if (fDropTarget->DoMakeDropLocationDescriptor(dropLocationDesc))
			SetDropLocation(theDragRef, dropLocationDesc);

		CDragItemIterator dragItemIterator(&fDragItemList);

		if (fDragSource && !UserRequestedCopy() && fDragSource->WillDragMove(fDropTarget))
		{
			if (fDragSource == fDropTarget)
				targetCommand = fDragSource->DoMakeDragDropCommand(cDragMove, dragItemIterator);
			else
			{
				sourceCommand = fDragSource->DoMakeDragDropCommand(cDrag, dragItemIterator);
				sourceCommand->SetIdentifier(cDragMoveOut);

				dragItemIterator.First();			// reset the iterator
				targetCommand = fDropTarget->DoMakeDragDropCommand(cDrop, dragItemIterator);
			}
		}
		else
			targetCommand = fDropTarget->DoMakeDragDropCommand(cDrop, dragItemIterator);

		if (targetCommand)							// should always have a target command
		{
			if (sourceCommand)
				sourceCommand->GetContext()->PostCommand(sourceCommand);// source command is primary

			targetCommand->GetContext()->PostCommand(targetCommand);// target command is primary

			// if this process is in the background, it is possible for
			// it to be sitting in WaitNextEvent with an infinite sleep value.
			// waking the process will cause the command(s) that were just posted
			// to be processed immediately.
			if (!gDispatcher->IsFrontProcess())
				gDispatcher->WakeProcess();

			if (!fDragSource)
				gDispatcher->PostCommand(TH_new TInvalCursorCommand);
		}
		else
		{
			returnValue = dragNotAcceptedErr;
#if qDebug
			// its an error if a source command was provided without a target
			if (sourceCommand)
			{
				::ProgramBreak_AC("###Drag source provided a target, drop target did not.");
				delete sourceCommand;
			}
#endif
		}

		SetDropTarget(NULL);					// allow the view to clean up
		fUserRequestedCopy = false;					// reset user request for copy
	}
	else
		returnValue = dragNotAcceptedErr;

	return returnValue;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::DragSendDataProc
//----------------------------------------------------------------------------------------


void TDragDropSession::DragSendDataProc(FlavorType theType,
										void*	/*dragSendDataRefCon*/ ,
										ItemReference theItemRef,
										DragReference
#if qDebug
										theDragRef
#endif

										)
{
#if qDebug
	if (theDragRef != fDragReference)
		::ProgramBreak_AC("###DragSendDataProc called with mismatched Drag Reference");
#endif

	TDragItem promisedItem(theItemRef);
	promisedItem.FocusOnFlavor(theType);

	fDragSource->DoFulfillPromise(&promisedItem);
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HandleEnterHandler
//----------------------------------------------------------------------------------------


void TDragDropSession::HandleEnterHandler()
{
	// initialize the flavor list	
	fDragItemList.BuildListFromDrag();

	fDropTarget = NULL;
	fTargetWindow = NULL;
	fTargetIsHilited = false;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HandleLeaveHandler
//----------------------------------------------------------------------------------------


void TDragDropSession::HandleLeaveHandler()
{
	for (TemplateIterator_AC<TDragItem *> iter(&fDragItemList); iter; ++iter)
		delete iter.Current();
	fDragItemList.DeleteAll();
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HandleEnterWindow
//----------------------------------------------------------------------------------------


void TDragDropSession::HandleEnterWindow(TWindow* targetWindow)
{
	if (!(gDispatcher->InModalState() && (gDispatcher->GetFrontWindow() != targetWindow)))
	{
		CDragItemIterator dragItemIterator(&fDragItemList);
		targetWindow->DragEnteredWindow(dragItemIterator);

		fTargetWindow = targetWindow;
	}
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HandleLeaveWindow
//----------------------------------------------------------------------------------------


void TDragDropSession::HandleLeaveWindow()
{
	SetDropTarget(NULL);					// clear the current drop target

	if (fTargetWindow)
	{
		fTargetWindow->DragLeftWindow();		// notify the window
		fTargetWindow = NULL;					// reset data
	}
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HandleTrackInWindow
//----------------------------------------------------------------------------------------


void TDragDropSession::HandleTrackInWindow()
{
	if (fTargetWindow && !HandleAutoScroll())
	{
		CPoint_AC theMouse;
		CPoint_AC pinnedMouse;

		GetDragMouse(fDragReference, theMouse, pinnedMouse);

		CDragItemIterator dragItemIterator(&fDragItemList);
		CViewPoint localPoint;
		TView * newTarget = fTargetWindow->MouseToDropTarget(dragItemIterator, pinnedMouse, localPoint);

		SetDropTarget(newTarget);

		if (fDropTarget)
		{
			fDropTarget->Focus();
			fDropTarget->DoDragWithin(localPoint);
		}
	}
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HandleAutoScroll
//----------------------------------------------------------------------------------------


bool TDragDropSession::HandleAutoScroll()
{
	bool didScroll = false;

	if (fScroller && fTargetWindow == fSourceWindow && fSourceWindow->IsActive() && fScroller->Focus())
	{
		CPoint_AC pinnedMouse;
		CPoint_AC theMouse;

		GetDragMouse(fDragReference, theMouse, pinnedMouse);

		CViewPoint delta(fScroller->GetDragScrollDelta(fSourceWindow->SuperViewToLocal(pinnedMouse)));
		if (delta != gZeroPt)
		{
			SetDropTarget(NULL);			// clear the drop target

			fScroller->ScrollBy(delta, kRedraw);// scroll and inval
			fScroller->Update();				// redraw content
			didScroll = true;
		}
	}

	return didScroll;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HandleDragToTrash
//----------------------------------------------------------------------------------------


bool TDragDropSession::HandleDragToTrash()
{
	bool draggedToTrash = false;

	if (!UserRequestedCopy()
		&& IsDropLocationFinderTrash()
		&& fDragSource
		&& fDragSource->GetDragMoveDeterminer() != kNeverMove)
	{
		CDragItemIterator dragItemIterator(&fDragItemList);
		TCommand * dragCommand = fDragSource->DoMakeDragDropCommand(cDrag, dragItemIterator);
		if (dragCommand)
			dragCommand->GetContext()->PostCommand(dragCommand);

		if (!gDispatcher->IsFrontProcess())
			gDispatcher->WakeProcess();			// time to wake up...

		draggedToTrash = true;
	}

	return draggedToTrash;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::GetGlobalDragMouse
//----------------------------------------------------------------------------------------


void TDragDropSession::GetGlobalDragMouse(CPoint_AC& mouse,
										  CPoint_AC& pinnedMouse)
{
	::ThrowIfOSErr_AC(GetDragMouse(fDragReference, mouse, pinnedMouse));
}

//----------------------------------------------------------------------------------------
// TDragDropSession::SetGlobalPinnedDragMouse
//----------------------------------------------------------------------------------------


void TDragDropSession::SetGlobalPinnedDragMouse(CPoint_AC& pinnedMouse)
{
	::ThrowIfOSErr_AC(SetDragMouse(fDragReference, pinnedMouse));
}

//----------------------------------------------------------------------------------------
// TDragDropSession::SetDropTarget
//----------------------------------------------------------------------------------------


void TDragDropSession::SetDropTarget(TView* newDropTarget)
{
	// track whether drag has left the source view
	if (!fDragLeftSourceView && (newDropTarget != fDragSource))
		fDragLeftSourceView = true;

	if (fDropTarget != newDropTarget)
	{
		if (fDropTarget)
		{
			fDropTarget->Focus();
			fDropTarget->DoDragLeave();			// resign the current target
			HideDropTargetHilite();		// unhilite the current target
		}

		fDropTarget = newDropTarget;

		if (fDropTarget)
		{
			fDropTarget->Focus();
			fDropTarget->DoDragEnter();			// target the new target
			ShowDropTargetHilite();
		}
	}
}

//----------------------------------------------------------------------------------------
// TDragDropSession::RegisterDroppableWindow
//----------------------------------------------------------------------------------------


void TDragDropSession::RegisterDroppableWindow(TWindow* theWindow,
											   WindowRef theWindowMgrWindow)
{
	// register a window with the Drag Manager. 

#if qDebug
	if (!HasDragManager_AC())
	{
		::ProgramBreak_AC("Window tried to register w/DragSession when drag isn't available.");
		return;
	}
#endif

	try
	{
		::ThrowIfOSErr_AC(InstallTrackingHandler(fDragTrackingHandlerUPP, theWindowMgrWindow, theWindow));

		::ThrowIfOSErr_AC(InstallReceiveHandler(fDragReceiveHandlerUPP, theWindowMgrWindow, theWindow));
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		RemoveTrackingHandler(fDragTrackingHandlerUPP, theWindowMgrWindow);
		RemoveReceiveHandler(fDragReceiveHandlerUPP, theWindowMgrWindow);
	}
	
}

//----------------------------------------------------------------------------------------
// TDragDropSession::UnregisterDroppableWindow
//----------------------------------------------------------------------------------------


void TDragDropSession::UnregisterDroppableWindow(WindowRef theWindowMgrWindow)
{
#if qDebug
	if (!HasDragManager_AC())
	{
		::ProgramBreak_AC("Window tried to unregister w/DragSession when drag isn't available.");
		return;
	}
#endif

		RemoveTrackingHandler(fDragTrackingHandlerUPP, theWindowMgrWindow);
		RemoveReceiveHandler(fDragReceiveHandlerUPP, theWindowMgrWindow);
}


//----------------------------------------------------------------------------------------
// TDragDropSession::HasDragLeftSenderWindow
//----------------------------------------------------------------------------------------


bool TDragDropSession::HasDragLeftSenderWindow() const
{
	DragAttributes attributes;
	GetDragAttributes(fDragReference, &attributes);

	return (attributes & dragHasLeftSenderWindow) == dragHasLeftSenderWindow;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::ShowDropTargetHilite
//----------------------------------------------------------------------------------------


void TDragDropSession::ShowDropTargetHilite()
{
	if (fDragLeftSourceView && fDropTarget)
	{
		CTempRegion_AC hiliteRegion(fDropTarget->DoMakeDropHiliteRegion());

		fDropTarget->LocalToRootView(hiliteRegion);
		fTargetWindow->Focus();

		ShowDragHilite(fDragReference, hiliteRegion, true);
		fTargetIsHilited = true;
	}
}

//----------------------------------------------------------------------------------------
// TDragDropSession::HideDropTargetHilite
//----------------------------------------------------------------------------------------


void TDragDropSession::HideDropTargetHilite()
{
	if (fTargetIsHilited)
	{
		fTargetWindow->Focus();
		HideDragHilite(fDragReference);
		fTargetIsHilited = false;
	}
}

//----------------------------------------------------------------------------------------
// TDragDropSession::GetItemCount
//----------------------------------------------------------------------------------------
unsigned short TDragDropSession::GetItemCount()
{
	return fDragItemList.GetSize();
}

//----------------------------------------------------------------------------------------
// TDragDropSession::UserIsDragging
//----------------------------------------------------------------------------------------


bool TDragDropSession::UserIsDragging(CPoint_AC& mousePoint)
{

	return WaitMouseMoved(mousePoint);
}

//----------------------------------------------------------------------------------------
// TDragDropSession::UserRequestedCopy
//----------------------------------------------------------------------------------------


bool TDragDropSession::UserRequestedCopy()
{
	if (!fUserRequestedCopy && IsOptionKeyDown())
		fUserRequestedCopy = true;

	return fUserRequestedCopy;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::ClearDrag
//----------------------------------------------------------------------------------------


void TDragDropSession::ClearDrag()
{
	if (fDragReference)
		DisposeDrag(fDragReference);

	fDropTarget = NULL;
	fDragSource = NULL;
	fDragReference = 0;
	fScroller = NULL;
	fSourceWindow = NULL;
	fTargetWindow = NULL;

	for (TemplateIterator_AC<TDragItem *> iter(&fDragItemList); iter; ++iter)
		delete iter.Current();
	fDragItemList.DeleteAll();

	fDragReference = 0L;
	fDragLeftSourceView = false;
	fTargetIsHilited = false;
	fUserRequestedCopy = false;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::StartDrag
//----------------------------------------------------------------------------------------


void TDragDropSession::StartDrag(TView* sourceView,
								 TToolboxEvent* event,
								 RgnHandle dragRegion)
{
	OSErr dragErr;

	fDragSource = sourceView;

	UserRequestedCopy();					// test for copy override
	fScroller = fDragSource->GetScroller(false);
	fSourceWindow = sourceView->GetWindow();

	bool oldBusyCursorState = gDispatcher->ActivateBusyCursor(false);

	TUndoHandler::fgUndoHandler.BeginAction(cDragMove);

	dragErr = TrackDrag(fDragReference, &event->GetEventRecord(0), dragRegion);

	// Must refocus on the source. Otherwise the port and the source
	// will get out of synch
	sourceView->Focus();

	if (dragErr == noErr)
		HandleDragToTrash();

	TUndoHandler::fgUndoHandler.EndAction();

	gDispatcher->PostCommand(TH_new TInvalCursorCommand);

	gDispatcher->ActivateBusyCursor(oldBusyCursorState);

	ClearDrag();
}

//----------------------------------------------------------------------------------------
// TDragDropSession::AddDragItem
//----------------------------------------------------------------------------------------


TDragItem* TDragDropSession::AddDragItem(ItemReference theItemRef)
{
	if (fDragItemList.IsEmpty())				// create a new drag manager drag
	{
		::ThrowIfOSErr_AC(NewDrag(&fDragReference));
		::ThrowIfOSErr_AC(SetDragSendProc(fDragReference, fDragSendDataUPP, NULL));
	}

	TDragItem * newItem = TH_new TDragItem(theItemRef);
	fDragItemList.InsertElementInOrder(&newItem);

	return newItem;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::GetDragItemByIndex
//----------------------------------------------------------------------------------------


TDragItem* TDragDropSession::GetDragItemByIndex(unsigned short index)
{
	TDragItem * indexedItem = static_cast<TDragItem*>(fDragItemList.At(index));
	indexedItem->SetPosition(0);

	return indexedItem;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::GetDragItemByReference
//----------------------------------------------------------------------------------------


TDragItem* TDragDropSession::GetDragItemByReference(ItemReference itemReference)
{
	TDragItem * returnVal = NULL;
	for (CDragItemIterator iter(&fDragItemList); iter.Current(); ++iter)
		if (iter->GetItemReference() == itemReference)
		{
			iter->SetPosition(0);
			returnVal = iter;
			break;
		}

	if (!returnVal)
		::ThrowException_AC(minErr, 0);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TDragDropSession::GetDropLocationDescriptor
//----------------------------------------------------------------------------------------


void TDragDropSession::GetDropLocationDescriptor(CAEDesc& dropLocationDesc) const
{
	::ThrowIfOSErr_AC(GetDropLocation(fDragReference, dropLocationDesc));
}

//----------------------------------------------------------------------------------------
// TDragDropSession::IsDropLocationFinderTrash
//----------------------------------------------------------------------------------------


bool TDragDropSession::IsDropLocationFinderTrash() const
{
	bool result = false;
	
	CTempDesc dropLocation;

	if (!fDropTarget)
	{
		try
		{
			CInfoPBRec cpb;

			GetDropLocationDescriptor(dropLocation);

			if (!dropLocation.CanCoerceDesc(typeFSS))
				result = false;
			else
			{
				CFSSpec_AC dropLocationSpec(dropLocation.GetFSSpec());

				cpb.dirInfo.ioNamePtr = dropLocationSpec.name;// get destination's info
				cpb.dirInfo.ioVRefNum = dropLocationSpec.vRefNum;
				cpb.dirInfo.ioFDirIndex = 0;
				cpb.dirInfo.ioDrDirID = dropLocationSpec.parID;

				::ThrowIfOSErr_AC(PBGetCatInfoSync(&cpb));

				if ((cpb.dirInfo.ioFlAttrib & (1 << 4)) == (1 << 4))// it's a directory
				{
					short trashVRefNum;
					long trashDirID;

					::ThrowIfOSErr_AC(FindFolder(cpb.dirInfo.ioVRefNum, kTrashFolderType, kDontCreateFolder, &trashVRefNum, &trashDirID));

					if (cpb.dirInfo.ioDrDirID == trashDirID)
						result = true;
				}
			}
		}
		catch(CException_AC& theException)
		{	
			DoCatchMessage_AC(theException);
			result = false;
		}
		
	}

	return result;
}

//========================================================================================
// CLASS TDragItemList
//========================================================================================
MA_DEFINE_CLONE(TDragItemList);

//----------------------------------------------------------------------------------------
// TDragItemList::~TDragItemList
//----------------------------------------------------------------------------------------


TDragItemList::~TDragItemList()
{
}

//----------------------------------------------------------------------------------------
// TDragItemList::BuildListFromDrag
//----------------------------------------------------------------------------------------


void TDragItemList::BuildListFromDrag()
{
	DragReference dragReference = TDragDropSession::fgDragDropSession->GetDragReference();

	for (TemplateIterator_AC<TDragItem *> iter(this); iter; ++iter)
		delete iter.Current();
	DeleteAll();

	unsigned short itemCount;
	::ThrowIfOSErr_AC(CountDragItems(dragReference, &itemCount));

	for (unsigned short itemIndex = 1; itemIndex <= itemCount; ++itemIndex)
	{
		ItemReference itemReference;
		::ThrowIfOSErr_AC(GetDragItemReferenceNumber(dragReference, itemIndex, &itemReference));

		TDragItem * dragItem = TH_new TDragItem(itemReference);
		InsertLast(dragItem);
	}
}

//========================================================================================
// CLASS TDragItem
//========================================================================================
//----------------------------------------------------------------------------------------
// TDragItem::TDragItem
//----------------------------------------------------------------------------------------


TDragItem::TDragItem(ItemReference itemReference) :
	fItemReference(itemReference),
	fFlavorOffset(0),
	fFlavorType(kAllQuestionsType_AC)
{
}

//----------------------------------------------------------------------------------------
// TDragItem::~TDragItem
//----------------------------------------------------------------------------------------


TDragItem::~TDragItem()
{
}

//----------------------------------------------------------------------------------------
// TDragItem::AddFlavor
//----------------------------------------------------------------------------------------


void TDragItem::AddFlavor(FlavorType newFlavor,
						  CFlavorFlags& flavorFlags,
						  void* buffer,
						  long count)
{
	::ThrowIfOSErr_AC(AddDragItemFlavor(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, newFlavor, buffer, count, flavorFlags));
	FocusOnFlavor(newFlavor);
	fFlavorOffset = count;
}

//----------------------------------------------------------------------------------------
// TDragItem::PromiseFlavor
//----------------------------------------------------------------------------------------


void TDragItem::PromiseFlavor(FlavorType newFlavor,
							  CFlavorFlags& flavorFlags)
{
	::ThrowIfOSErr_AC(AddDragItemFlavor(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, newFlavor, NULL, 0L, flavorFlags));
	FocusOnFlavor(newFlavor);
}

//----------------------------------------------------------------------------------------
// TDragItem::CountFlavors
//----------------------------------------------------------------------------------------


unsigned short TDragItem::CountFlavors()
{
	unsigned short flavorCount;

	::ThrowIfOSErr_AC(CountDragItemFlavors(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, &flavorCount));

	return flavorCount;
}

//----------------------------------------------------------------------------------------
// TDragItem::FlavorExists
//----------------------------------------------------------------------------------------


bool TDragItem::FlavorExists(FlavorType requestedFlavor)
{
	FlavorFlags flags;

	return (GetFlavorFlags(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, requestedFlavor, &flags) == noErr) ? true : false;
}

//----------------------------------------------------------------------------------------
// TDragItem::FocusOnFlavor
//----------------------------------------------------------------------------------------


void TDragItem::FocusOnFlavor(FlavorType focusFlavor)
{
	fFlavorOffset = 0;

	if (FlavorExists(focusFlavor))
		fFlavorType = focusFlavor;
	else
		::ThrowIfOSErr_AC(badDragFlavorErr);
}

//----------------------------------------------------------------------------------------
// TDragItem::GetPosition
//----------------------------------------------------------------------------------------


long TDragItem::GetPosition()
{
	return fFlavorOffset;
}

//----------------------------------------------------------------------------------------
// TDragItem::SetPosition
//----------------------------------------------------------------------------------------


void TDragItem::SetPosition(long newPosition)
{
	fFlavorOffset = newPosition;
}

//----------------------------------------------------------------------------------------
// TDragItem::GetBounds
//----------------------------------------------------------------------------------------


CRect_AC TDragItem::GetBounds()
{
	CRect_AC itemBounds;

	::ThrowIfOSErr_AC(GetDragItemBounds(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, itemBounds));

	return itemBounds;
}

//----------------------------------------------------------------------------------------
// TDragItem::SetBounds
//----------------------------------------------------------------------------------------


void TDragItem::SetBounds(CRect_AC& itemBounds)
{
	if (!itemBounds.IsEmpty())
		::ThrowIfOSErr_AC(SetDragItemBounds(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, itemBounds));
}

//----------------------------------------------------------------------------------------
// TDragItem::GetSize
//----------------------------------------------------------------------------------------


long TDragItem::GetSize() const
{
	long dataSize;

	::ThrowIfOSErr_AC(GetFlavorDataSize(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, fFlavorType, &dataSize));

	return dataSize;
}

//----------------------------------------------------------------------------------------
// TDragItem::GetFlags
//----------------------------------------------------------------------------------------


CFlavorFlags TDragItem::GetFlags()
{
	CFlavorFlags flags;

	::ThrowIfOSErr_AC(GetFlavorFlags(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, fFlavorType, flags));

	return flags;
}

//----------------------------------------------------------------------------------------
// TDragItem::GetFlavorType
//----------------------------------------------------------------------------------------


FlavorType TDragItem::GetFlavorType()
{
	return fFlavorType;
}

//----------------------------------------------------------------------------------------
// TDragItem::GetData
//----------------------------------------------------------------------------------------


void TDragItem::GetData(void* buffer,
						long& count)
{
	::ThrowIfOSErr_AC(GetFlavorData(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, fFlavorType, buffer, &count, fFlavorOffset));
	fFlavorOffset += count;
}

//----------------------------------------------------------------------------------------
// TDragItem::GetDataAsHandle
//----------------------------------------------------------------------------------------


Handle TDragItem::GetDataAsHandle()
{
	TDragFlavorStream dragFlavorStream(fFlavorType, fItemReference);
	dragFlavorStream.SetPosition(fFlavorOffset);

	Handle dataHandle = StreamToHandle(&dragFlavorStream);

	fFlavorOffset = dragFlavorStream.GetPosition();

	return dataHandle;
}

//----------------------------------------------------------------------------------------
// TDragItem::SetData
//----------------------------------------------------------------------------------------


void TDragItem::SetData(void* buffer,
						long count)
{
	::ThrowIfOSErr_AC(SetDragItemFlavorData(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, fFlavorType, buffer, count, fFlavorOffset));
	fFlavorOffset += count;
}

//----------------------------------------------------------------------------------------
// TDragItem::SetDataFromHandle
//----------------------------------------------------------------------------------------


void TDragItem::SetDataFromHandle(Handle dataHandle)
{
	if (!dataHandle)
		::ThrowException_AC(minErr, 0);

	CTempHandleLock_AC tempHandleLock(dataHandle);
	Size dataSize = GetHandleSize(dataHandle);
	::ThrowIfMemError_AC();

	SetData(*dataHandle, dataSize);
}

//----------------------------------------------------------------------------------------
// TDragItem::GetDataStream
//----------------------------------------------------------------------------------------


TDragFlavorStream* TDragItem::GetDataStream()
{
	return TH_new TDragFlavorStream(fFlavorType, fItemReference);
}

//========================================================================================
// CLASS ~TDragFlavorStreamRep
//========================================================================================
MA_DEFINE_CLONE(TDragFlavorStream);

//----------------------------------------------------------------------------------------
// TDragFlavorStream::TDragFlavorStream
//----------------------------------------------------------------------------------------


TDragFlavorStreamRep::TDragFlavorStreamRep(FlavorType theFlavor,
										  ItemReference theReference) :
	fItemReference(theReference),
	fFlavorType(theFlavor),
	fPosition(0)
{
}

//----------------------------------------------------------------------------------------
// TDragFlavorStreamRep::~TDragFlavorStreamRep
//----------------------------------------------------------------------------------------


TDragFlavorStreamRep::~TDragFlavorStreamRep()
{
}


//----------------------------------------------------------------------------------------
// TDragFlavorStream::GetPosition
//----------------------------------------------------------------------------------------


long TDragFlavorStreamRep::GetPosition() const
{
	return fPosition;
}

//----------------------------------------------------------------------------------------
// TDragFlavorStreamRep::SetPosition
//----------------------------------------------------------------------------------------


void TDragFlavorStreamRep::SetPosition(long newPosition)
{
	OSErr err = noErr;
	long dataSize;

	dataSize = GetSize();

	if (newPosition < 0)						// Same error returned by File Manager
		::ThrowIfOSErr_AC(posErr);
	else if (newPosition > dataSize)			// Hit the end of the stream
	{
		newPosition = dataSize;					// Position to end of stream
		err = eofErr;
	}

	fPosition = newPosition;
	::ThrowIfOSErr_AC(err);
}

//----------------------------------------------------------------------------------------
// TDragFlavorStreamRep::GetSize
//----------------------------------------------------------------------------------------


long TDragFlavorStreamRep::GetSize() const
{
	long dataSize;
	OSErr err = GetFlavorDataSize(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, fFlavorType, &dataSize);
	if (err != noErr)
		dataSize = 0L;

	return dataSize;
}

//----------------------------------------------------------------------------------------
// TDragFlavorStreamRep::ReadBytes
//----------------------------------------------------------------------------------------


void TDragFlavorStreamRep::ReadBytes(void* p,
								  long count)
{
	if (count < 0)
		::ThrowIfOSErr_AC(paramErr);					// Negative count request

	if (GetSize() - fPosition < count)
		::ThrowIfOSErr_AC(eofErr);						// about to read past end of stream

	// read the bytes
	long bytesRead = count;
	::ThrowIfOSErr_AC(GetFlavorData(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, fFlavorType, p, &bytesRead, fPosition));
	fPosition += bytesRead;
}

//----------------------------------------------------------------------------------------
// TDragFlavorStreamRep::WriteBytes
//----------------------------------------------------------------------------------------


void TDragFlavorStreamRep::WriteBytes(const void* p,
								   long count)
{
	::ThrowIfOSErr_AC(SetDragItemFlavorData(TDragDropSession::fgDragDropSession->GetDragReference(), fItemReference, fFlavorType, p, count, fPosition));
	fPosition += count;
}

//========================================================================================
// CLASS TDragFlavorStream
//========================================================================================

//----------------------------------------------------------------------------------------
// TDragFlavorStream::TDragFlavorStream
//----------------------------------------------------------------------------------------

TDragFlavorStream::TDragFlavorStream(ItemReference theReference, FlavorType theFlavor)
	: CRandomAccessStream_AC(TH_new TDragFlavorStreamRep(theReference, theFlavor))
{
#if qDebug
	++fInstanceCount;							// increment the instance counter
#endif
}

//----------------------------------------------------------------------------------------
// TDragFlavorStream::~TDragFlavorStream
//----------------------------------------------------------------------------------------

TDragFlavorStream::~TDragFlavorStream()
{
#if qDebug
	--fInstanceCount;							// decrement the instance counter
#endif
}

#if qDebug
//----------------------------------------------------------------------------------------
// TDragFlavorStreamRep::CheckReferenceCount
//----------------------------------------------------------------------------------------


void TDragFlavorStream::CheckInstanceCount()
{
	// This method is called by TDragSession before returning control to the drag manager
	// after receiving a drop.  Any existing instances of TDragFlavorStream are about to
	// become invalid. If a positive instance count exists, a program break is
	// generated.
	if (fInstanceCount)
	{
		::ProgramBreak_AC("###TDragFlavorStream failed instance count check");
		fInstanceCount = 0;						// reset the count
	}
}

#endif // qDebug

//========================================================================================
// CLASS TInvalCursorCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TInvalCursorCommand::TInvalCursorCommand
//----------------------------------------------------------------------------------------


TInvalCursorCommand::TInvalCursorCommand() :
	TCommand(cNoCommand, NULL, kCantUndo, kDoesNotCauseChange, NULL)
{
}

//----------------------------------------------------------------------------------------
// TInvalCursorCommand::DoIt
//----------------------------------------------------------------------------------------


void TInvalCursorCommand::DoIt()
{
	// Compute based on where the mouse is right now 
	CPoint_AC globalMouse;

	::GetMouse(globalMouse);
	::LocalToGlobal(globalMouse);

	gDispatcher->UpdateAllWindows();			// necessary for calculation of visible regions
	gDispatcher->InvalidateMouseRegions();
	gDispatcher->TrackCursor(globalMouse);
}

#endif // qDrag

//----------------------------------------------------------------------------------------
// End of UDragDrop.cp


