// UDragDropBehavior.cp
// Copyright  1994-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UDRAGDROPBEHAVIOR__
#include "UDragDropBehavior.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//========================================================================================
// CLASS TDragDropBehavior
//========================================================================================
#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TDragDropBehavior);

//----------------------------------------------------------------------------------------
// TDragDropBehavior constructor
//----------------------------------------------------------------------------------------


TDragDropBehavior::TDragDropBehavior(IDType itsIdentifier, 
											bool draggable, 
											bool droppable,
											short	dragMoveDeterminer,
											IDType	dragMoveFamily) :
	TBehavior(itsIdentifier),

	fDraggable(draggable),
	fDroppable(droppable),
	fDragMoveDeterminer(dragMoveDeterminer),
	fDragMoveFamily(dragMoveFamily)
{
}

//----------------------------------------------------------------------------------------
// TDragDropBehavior destructor
//----------------------------------------------------------------------------------------


TDragDropBehavior::~TDragDropBehavior()
{
}

//----------------------------------------------------------------------------------------
// TDragDropBehavior::ReadFrom 
//----------------------------------------------------------------------------------------


void TDragDropBehavior::ReadFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);

	fDraggable = aStream->ReadBoolean();
	fDroppable = aStream->ReadBoolean();
	fDragMoveDeterminer = aStream->ReadInteger();
	fDragMoveFamily = aStream->ReadIDType();
}


//----------------------------------------------------------------------------------------
// TDragDropBehavior::WriteTo
//----------------------------------------------------------------------------------------


void TDragDropBehavior::WriteTo(CStream_AC* aStream) const
{
	Inherited::WriteTo(aStream);
	
#if qDrag
	// read drag data out of the owner, since the values in the behavior may not be
	// current
	{
		TView *ownerView = dynamic_cast_AC(TView*,fOwner);
	
		const_cast<TDragDropBehavior*>(this)->fDraggable = ownerView->GetDraggable();
		const_cast<TDragDropBehavior*>(this)->fDroppable = ownerView->GetDroppable();
		const_cast<TDragDropBehavior*>(this)->fDragMoveDeterminer = ownerView->GetDragMoveDeterminer();
		const_cast<TDragDropBehavior*>(this)->fDragMoveFamily = ownerView->GetDragMoveFamily();
	}
#endif // qDrag

	aStream->WriteBoolean(fDraggable);
	aStream->WriteBoolean(fDroppable);
	aStream->WriteInteger(fDragMoveDeterminer);
	aStream->WriteIDType(fDragMoveFamily);
}

//----------------------------------------------------------------------------------------
// TDragDropBehavior::SetOwner
//----------------------------------------------------------------------------------------


void TDragDropBehavior::SetOwner(TEventHandler* itsOwner)
{
	Inherited::SetOwner(itsOwner);
	
#if qDrag
#if qDebug
	// verify that the owner is a TView
	
	if (itsOwner && !dynamic_cast<TView*>(itsOwner))
	{
		::ProgramBreak_AC("###A TDragDropBehavior is attached to a non-view object");
		return;
	}
#endif // qDebug

	// set data in the owner view
	if (itsOwner)
	{
		TView *ownerView = dynamic_cast_AC(TView*,itsOwner);
		
		ownerView->SetDraggable(fDraggable);
		ownerView->SetDroppable(fDroppable);
		ownerView->SetDragMoveDeterminer(fDragMoveDeterminer);
		ownerView->SetDragMoveFamily(fDragMoveFamily);
	}
#endif // qDrag

}

//----------------------------------------------------------------------------------------
// TDragDropBehavior::GetStandardSignature:
//----------------------------------------------------------------------------------------


IDType TDragDropBehavior::GetStandardSignature() const
{
	return kDragDropBehavior;
}

//----------------------------------------------------------------------------------------
// End of UDragDropBehavior.cp


