//----------------------------------------------------------------------------------------
// UDrawingEnvironment.cp 
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDRAWINGENVIRONMENT__
#include "UDrawingEnvironment.h"
#endif

// ACS

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif

// MacApp

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// ANSI

#ifndef __STRING__
#include <string.h>
#endif


//========================================================================================
// CLASS TDrawingEnvironment
//========================================================================================
#undef Inherited


MA_DEFINE_CLASS(TDrawingEnvironment);

//----------------------------------------------------------------------------------------
// TDrawingEnvironment constructor
//----------------------------------------------------------------------------------------


TDrawingEnvironment::TDrawingEnvironment() :
	fPenSize(1,1),
	fPenMode(patCopy),
	
	fPenPattern(qd.black),
	fForegroundColor(gRGBBlack),
	fBackgroundColor(gRGBWhite)
{
}

//----------------------------------------------------------------------------------------
// TDrawingEnvironment destructor
//----------------------------------------------------------------------------------------


TDrawingEnvironment::~TDrawingEnvironment()
{
}

//----------------------------------------------------------------------------------------
// TDrawingEnvironment::ReadFrom: 
//----------------------------------------------------------------------------------------


void TDrawingEnvironment::ReadFrom(CStream_AC* aStream)	 
{
	MStreamable::ReadFrom(aStream);
	
	*aStream >> fPenSize >> fPenMode;
	
	aStream->ReadBytes(&fPenPattern, sizeof(Pattern));
	
	*aStream >> fForegroundColor >> fBackgroundColor;
}

//----------------------------------------------------------------------------------------
// TDrawingEnvironment::WriteTo: 
//----------------------------------------------------------------------------------------


void TDrawingEnvironment::WriteTo(CStream_AC* aStream) const	 
{
	MStreamable::WriteTo(aStream);

	*aStream << fPenSize << fPenMode;
	
	aStream->WriteBytes(&fPenPattern, sizeof(Pattern));
	
	*aStream << fForegroundColor << fBackgroundColor;
}

//----------------------------------------------------------------------------------------
// TDrawingEnvironment::GetGrafPort: 
//----------------------------------------------------------------------------------------


GrafPtr TDrawingEnvironment::GetGrafPort() const
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// TDrawingEnvironment::Prepare: 
//----------------------------------------------------------------------------------------


void TDrawingEnvironment::Prepare()	 
{
	GetPenState(&fSavedPenState);
	fSavedForegroundColor = GetIfColor();
	fSavedBackgroundColor = GetIfBkColor();
}

//----------------------------------------------------------------------------------------
// TDrawingEnvironment::Setup: 
//----------------------------------------------------------------------------------------


void TDrawingEnvironment::Setup()	 
{
	PenNormal();
	PenSize(fPenSize.h,fPenSize.v);
	PenMode(fPenMode);
	PenPat(&fPenPattern);
	SetIfColor(fForegroundColor);
	SetIfBkColor(fBackgroundColor);
}

//----------------------------------------------------------------------------------------
// TDrawingEnvironment::Complete: 
//----------------------------------------------------------------------------------------


void TDrawingEnvironment::Complete()	 
{
	SetPenState(&fSavedPenState);
	SetIfColor(fSavedForegroundColor);
	SetIfBkColor(fSavedBackgroundColor);	
}

//----------------------------------------------------------------------------------------
// End of UDrawingEnvironment.cp


