//----------------------------------------------------------------------------------------
// UEvent.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UEVENT__
#include "UEvent.h"
#endif

// MacApp

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UEVENTHANDLER__
#include "UEventHandler.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

// Toolbox

#ifndef __EPPC__
#include <EPPC.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Use Windows_AC.h instead of <windows.h> for XPlatform compatability.
#ifndef __Windows_AC__
#include "Windows_AC.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI



//========================================================================================
// CLASS TEvent
//========================================================================================
//----------------------------------------------------------------------------------------
// TEvent constructor
//----------------------------------------------------------------------------------------


TEvent::TEvent(EventNumber itsEventNumber,
					TEventHandler* itsSource,
					TEventHandler* itsHandler) :
	fIdentifier(itsEventNumber),
	fSource(itsSource),
	fHandler(itsHandler),
	fAffectsMenus(false),
	fPriority(kPriorityNormal)				// Don't worry, be happy! 
{
}

//----------------------------------------------------------------------------------------
// TEvent::Free: 
//----------------------------------------------------------------------------------------


TEvent::~TEvent()
{
#if qDebug
	if (gDispatcher)
	{
		TEventList* eventList = gDispatcher->GetEventList();
		if (eventList && eventList->GetIdentityItemNo(this))
			::ProgramBreak_AC("You are trying to free an event that is still on the event queue.");
	}
#endif
}

//----------------------------------------------------------------------------------------
// TEvent::GetHandler: 
//----------------------------------------------------------------------------------------


TEventHandler* TEvent::GetHandler()
{
	return fHandler ? fHandler : gDispatcher;
}

//----------------------------------------------------------------------------------------
// TEvent::ShouldFreeOnCompletion: 
//----------------------------------------------------------------------------------------


bool TEvent::ShouldFreeOnCompletion()
{
	return true;
}

//----------------------------------------------------------------------------------------
// TEvent::IsReadyToPost: 
//----------------------------------------------------------------------------------------


bool TEvent::IsReadyToPost()
{
	return true;
}

//----------------------------------------------------------------------------------------
// TEvent::IsReadyToExecute: 
//----------------------------------------------------------------------------------------


bool TEvent::IsReadyToExecute()
{
	return true;
}

//----------------------------------------------------------------------------------------
// TEvent::IsRecurring: 
//----------------------------------------------------------------------------------------


bool TEvent::IsRecurring()
{
	return false;
}

//----------------------------------------------------------------------------------------
// TEvent::Process: 
//----------------------------------------------------------------------------------------


void TEvent::Process()	

{
	TEventHandler* handler = GetHandler();
	
	if (handler)
		handler->HandleEvent(fIdentifier,fSource,this);
}

#if qDebug
//----------------------------------------------------------------------------------------
// TEvent::ReportEvent: 
//----------------------------------------------------------------------------------------


void TEvent::ReportEvent()
{
}
#endif


//========================================================================================
// CLASS TToolboxEvent
//========================================================================================
//----------------------------------------------------------------------------------------
// TToolboxEvent::TToolboxEvent:
//----------------------------------------------------------------------------------------


TToolboxEvent::TToolboxEvent(TEventHandler* itsHandler,
								  const EventRecord* theEvent) :
	TEvent(cNoCommand, NULL, itsHandler),

	fKeyScript(static_cast<short>(GetScriptManagerVariable(smKeyScript))), 
	fKeyCode(0),					// Default, we don't know if this is a keystroke yet
	fClickCount(0),
	fDoubleEvent(false)
#if qBackwardCompatible
	,fEventRecord(fEventRecords[0]),
	fCharacter(fText[1])
#endif
{
	fAffectsMenus = true;			// assume going in that this event affects the menus

	if (theEvent)
	{
		fIdentifier = static_cast<unsigned short>(theEvent->what);
		fEventRecords[0] = *theEvent;
	}
}

//----------------------------------------------------------------------------------------
// TToolboxEvent destructor
//----------------------------------------------------------------------------------------


TToolboxEvent::~TToolboxEvent()
{
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsDoubleEvent: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsDoubleEvent()

{
	bool returnVal = false;

	if ((fEventRecords[0].what == keyDown) || (fEventRecords[0].what == keyUp) || (fEventRecords[0].what == autoKey))
	{
		unsigned char theCharacter = static_cast<unsigned char>(fEventRecords[0].message & charCodeMask);
		returnVal = MACharacterByteType((Ptr)&theCharacter, 0, fKeyScript) == smFirstByte;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::AddSecondaryKeyEvent: 
//----------------------------------------------------------------------------------------


void TToolboxEvent::AddSecondaryKeyEvent(const EventRecord& theEvent)
{
	fDoubleEvent = true;
	fEventRecords[1] = theEvent;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsButtonPressed: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsButtonPressed() const
{
	return ((fEventRecords[0].modifiers) & btnState) != 0;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsCommandKeyPressed: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsCommandKeyPressed() const
{
	return ((fEventRecords[0].modifiers) & cmdKey) != 0;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsShiftKeyPressed: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsShiftKeyPressed() const
{
	return ((fEventRecords[0].modifiers) & shiftKey) != 0;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsAlphaLock: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsAlphaLock() const
{
	return ((fEventRecords[0].modifiers) & alphaLock) != 0;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsOptionKeyPressed: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsOptionKeyPressed() const
{
	return ((fEventRecords[0].modifiers) & optionKey) != 0;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsControlKeyPressed: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsControlKeyPressed() const
{
	return ((fEventRecords[0].modifiers) & controlKey) != 0;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::IsAutoKeyEvent: 
//----------------------------------------------------------------------------------------


bool TToolboxEvent::IsAutoKeyEvent() const
{
	return fEventRecords[0].what == autoKey;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::Process: 
//----------------------------------------------------------------------------------------


void TToolboxEvent::Process()	
{
	gDispatcher->HandleToolboxEvent(this);
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::GetWhat: 
//----------------------------------------------------------------------------------------
short TToolboxEvent::GetWhat() const
{
	return fEventRecords[0].what;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::GetWhen: 
//----------------------------------------------------------------------------------------
long TToolboxEvent::GetWhen() const
{
	return fEventRecords[0].when;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::GetWhere: 
//----------------------------------------------------------------------------------------
CPoint_AC TToolboxEvent::GetWhere() const
{
	return fEventRecords[0].where;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::GetMessage: 
//----------------------------------------------------------------------------------------
long TToolboxEvent::GetMessage(short zeroOrOne) const
{
	return fEventRecords[zeroOrOne].message;
}

//----------------------------------------------------------------------------------------
// TToolboxEvent::GetModifiers: 
//----------------------------------------------------------------------------------------
short TToolboxEvent::GetModifiers(short zeroOrOne) const
{
	return fEventRecords[zeroOrOne].modifiers;
}

#if qDebug
//----------------------------------------------------------------------------------------
// TToolboxEvent::ReportEvent: 
//----------------------------------------------------------------------------------------

void TToolboxEvent::ReportEvent()		// override 
{
	ProgramReport_AC(false, "t = %d", fEventRecords[0].when);
	CStr63_AC mods = " ---------";
	//		1234567890 

	if (IsControlKeyPressed())
		mods[2] = 'C';
	if (IsOptionKeyPressed())
		mods[3] = 'O';
	if (IsAlphaLock())
		mods[4] = 'L';
	if (IsShiftKeyPressed())
		mods[5] = 'S';
	if (IsCommandKeyPressed())
		mods[6] = 'C';
	if (IsButtonPressed())
		mods[7] = 'M';
	if (fEventRecords[0].what == activateEvt)
		mods[8] = fEventRecords[0].modifiers & activeFlag ? 'A' : 'D';
	ProgramReport_AC(false, "%s", (const char*)(mods));

	switch (fEventRecords[0].what)
	{
		case nullEvent:
			ProgramReport_AC("nullEvent\n");
			break;
		case mouseDown:
		case mouseUp:
		{
			if (fEventRecords[0].what == mouseDown)
				ProgramReport_AC("mouseDown   ");
			else
				ProgramReport_AC("mouseUp     ");
			ProgramReport_AC(false, "%s", (const char*)(CPoint_AC(fEventRecords[0].where)));
			CStr255_AC aString;
			WindowRef aWMgrWindow;
			switch (FindWindow(fEventRecords[0].where, &aWMgrWindow))
			{
				case inMenuBar:
					aString = "inMenuBar";
					break;
					
				case inSysWindow:
					aString = "inSysWindow";
					break;
					
				case inDrag:
					aString = "inDrag";
					break;
					
				case inGrow:
					aString = "inGrow";
					break;
					
				case inGoAway:
					aString = "inGoAway";
					break;
					
				case inContent:
					aString = "inContent";
					break;
					
				case inZoomIn:
					aString = "inZoomIn";
					break;
					
				case inZoomOut:
					aString = "inZoomOut";
					break;
					
				default:
					aString = "Mouse clicked in an unknown place.";
					break;
			}

			ProgramReport_AC(false, "%5s\n", (const char*)(aString));
			break;
		}
		case keyDown:
		case autoKey:
		case keyUp:
			if (fEventRecords[0].what == keyDown)
				ProgramReport_AC("keyDown     ");
			else if (fEventRecords[0].what == autoKey)
				ProgramReport_AC("autoKey     ");
			else
				ProgramReport_AC("keyUp       ");

			short ch = static_cast<short>(fEventRecords[0].message & charCodeMask);
			short cap = static_cast<short>(fEventRecords[0].message >> 8);

			if (ch >= chSpace && ch <= 0xD8 && ch != 0x7F)
				ProgramReport_AC(false, "\"%c\"", ch);
			else
				ProgramReport_AC("   ");

			ProgramReport_AC(false, "( %1d / %1d )\n", ch, cap);
			break;

		case updateEvt:
		case activateEvt:
		{
			if (fEventRecords[0].what == updateEvt)
				ProgramReport_AC("updateEvt   ");
			else
				ProgramReport_AC("activateEvt ");

			CStr255_AC aString;
			GetWTitle((WindowRef)(fEventRecords[0].message), aString);
			ProgramReport_AC(false, "\"%s\"\n", (const char*)(aString));
			break;
		}
		case diskEvt:
			ProgramReport_AC(false, "diskEvt     d = %1d e = %1d \n", LoWord(fEventRecords[0].message), HiWord(fEventRecords[0].message));
			break;

		case osEvt:
			switch ((((fEventRecords[0].message) & osEvtMessageMask)) >> 24)
			{
				case suspendResumeMessage:
					if (fEventRecords[0].message & 1)
						ProgramReport_AC("resume      ");
					else
						ProgramReport_AC("suspend     ");
					break;
					
				case mouseMovedMessage:
				{
					ProgramReport_AC("mouse moved ");
					ProgramReport_AC(false, "%s", (const char*)(CPoint_AC(fEventRecords[0].where)));
					break;
				}
					
				default:
					ProgramReport_AC("osEvt     ");
					break;
			}
			WrLblHexLongint(", message", fEventRecords[0].message);
			ProgramReport_AC("\n");
			break;
			
		case kHighLevelEvent:
			WrLblHexLongint("kHighLevelEvent     , message", fEventRecords[0].message);
			ProgramReport_AC("\n");
			break;
			
		default:
			ProgramReport_AC(false, "??? unknown = %1d   ", fEventRecords[0].what);
			WriteHexLongint(fEventRecords[0].message);
			break;
	}
}

#endif

//----------------------------------------------------------------------------------------
// End of UEvent.cp
