//----------------------------------------------------------------------------------------
// UFile.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UFILE__
#include "UFile.h"
#endif

// MacApp

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

//	#ifndef __UDISPATCHER__
//	#include "UDispatcher.h"
//	#endif

//	#ifndef __UERRORMGR__
//	#include "UErrorMgr.h"
//	#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __MSTREAMABLE__
#include "UStreamable.h"
#endif




#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __FINDER__
#include <Finder.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// NewFile: 
//----------------------------------------------------------------------------------------


TFile* NewFile(OSType itsFileType,
				  OSType itsCreator,
				  bool usesDataFork,
				  ResourceUsage usesRsrcFork,
				  bool keepsDataOpen,
				  bool keepsRsrcOpen)
{
	return TH_new TFile(itsFileType, itsCreator, usesDataFork, usesRsrcFork, keepsDataOpen, keepsRsrcOpen);
}


//========================================================================================
// CLASS TFile
//========================================================================================
MA_DEFINE_CLONE(TFile);

//----------------------------------------------------------------------------------------
// TFile::IFile: 
//----------------------------------------------------------------------------------------
TFile::TFile(OSType itsFileType,
						 OSType itsCreator,
						 bool usesDataFork,
						 ResourceUsage usesRsrcFork,
						 bool keepsDataOpen,
						 bool keepsRsrcOpen) :
	CFile_AC(itsFileType, itsCreator, usesDataFork, usesRsrcFork, keepsDataOpen, keepsRsrcOpen),
	MScriptableObject(cFile)
{
}

//----------------------------------------------------------------------------------------
// TFile::Free: 
//----------------------------------------------------------------------------------------
TFile::~TFile()
{
}

//----------------------------------------------------------------------------------------
// TFile::GetObjectProperty: 
//----------------------------------------------------------------------------------------
bool TFile::GetObjectProperty(CAEDesc& thePropertyValue,
								 DescType whichProperty,
								 const CAEDesc& desiredType)
{
	bool hasProperty = true;
	try
	{
		switch (whichProperty)
		{
			case pName:
				{
					CStr255_AC thePathName;
					if (GetPathName(thePathName) == noErr)
					{
						CStr63_AC theFileName;
						GetName(theFileName);
						thePropertyValue.PutString(thePathName + theFileName);
					}
					break;
				}
	
			case pIsStationeryPad:
				thePropertyValue.PutBoolean(IsStationery());
				break;
	
			default:
				hasProperty = MScriptableObject::GetObjectProperty(thePropertyValue, whichProperty, desiredType);
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		hasProperty = false;
	}
	
	return hasProperty;
}

//----------------------------------------------------------------------------------------
// TFile::SetObjectProperty: 
//----------------------------------------------------------------------------------------
void TFile::SetObjectProperty(const CAEDesc& thePropertyValue,
							  DescType whichProperty)
{
	switch (whichProperty)
	{
		case pName:
		case pIsStationeryPad:
			::ThrowIfOSErr_AC(errAECantSetReadOnly);
			break;

		default:
			MScriptableObject::SetObjectProperty(thePropertyValue, whichProperty);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TFile::InteractWithUser: 
//----------------------------------------------------------------------------------------
OSErr TFile::InteractWithUser()
{
	return MAInteractWithUser();
}

//----------------------------------------------------------------------------------------
// TFile::DoAEOpen
//----------------------------------------------------------------------------------------
void TFile::DoAEOpen(TAppleEvent* message, TAppleEvent* reply)
{
	// Post a command to handle the Open AppleEvent
	gApplication->PostCommand(TH_new TODocCommand(cAEOpen, *message, *reply));
}

//----------------------------------------------------------------------------------------
// End of UFile.cp
