//----------------------------------------------------------------------------------------
// UFileStream.cp 
// Copyright  1990-97 by Apple Computer, Inc. All rights reserved.
//
// Originally written by Larry S. Rosenstein.  Used by permission 
//----------------------------------------------------------------------------------------


#if 0
#ifndef __UFILESTREAM__
#include "UFileStream.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif


//========================================================================================
// CLASS TFileStream
//========================================================================================
#undef Inherited
#define Inherited TStream


MA_DEFINE_CLONE(TFileStream);

//----------------------------------------------------------------------------------------
// TFileStream constructor
//----------------------------------------------------------------------------------------


TFileStream::TFileStream(TFile* itsFile) :
	fFile(itsFile)
{
}

//----------------------------------------------------------------------------------------
// TFileStream destructor
//----------------------------------------------------------------------------------------


TFileStream::~TFileStream()
{
}

//----------------------------------------------------------------------------------------
// TFileStream::GetPosition: 
//----------------------------------------------------------------------------------------


long TFileStream::GetPosition() const		// override 
{
	long itsDataMark;

	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->GetDataMark(itsDataMark));	// Call the TFile method. 
	return itsDataMark;							// Return the result. 
}

//----------------------------------------------------------------------------------------
// TFileStream::GetSize: 
//----------------------------------------------------------------------------------------


long TFileStream::GetSize() const			// override 
{
	long itsDataLength;

	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->GetDataLength(itsDataLength));		// Call the TFile method. 
	return itsDataLength;						// Return the result. 
}


//----------------------------------------------------------------------------------------
// TFileStream::SetPosition: 
//----------------------------------------------------------------------------------------


void TFileStream::SetPosition(long newPosition)// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->SetDataMark(newPosition,fsFromStart));
}

//----------------------------------------------------------------------------------------
// TFileStream::SetSize: 
//----------------------------------------------------------------------------------------


void TFileStream::SetSize(long newSize)	// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->SetDataLength(newSize));
}

//----------------------------------------------------------------------------------------
// TFileStream::ReadBytes: 
//----------------------------------------------------------------------------------------


void TFileStream::ReadBytes(void* p,
								   long count)	// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->ReadData(p,count));	
}

//----------------------------------------------------------------------------------------
// TFileStream::WriteBytes: 
//----------------------------------------------------------------------------------------


void TFileStream::WriteBytes(const void* p,
									long count)	// override 
{
	::ThrowIfNULL_AC(fFile);
	::ThrowIfOSErr_AC(fFile->WriteData(p,count));
}


//========================================================================================
// CLASS TFileResourceStream
//========================================================================================
#undef Inherited
#define Inherited TResourceStream


MA_DEFINE_CLONE(TFileResourceStream);

//----------------------------------------------------------------------------------------
// TFileResourceStream::TFileResourceStream: 
//----------------------------------------------------------------------------------------


TFileResourceStream::TFileResourceStream( 	TFile* theFile, 
												ResType theType, 
												ResNumber theID) :
	TResourceStream(theType, theID),
	
	fFile(NULL)
{
	if (fFile)
	{
		CTempUseResFile_AC tempUseResFile(theFile->GetRsrcRefNum());
		
		fFile = theFile;
	}
}

//----------------------------------------------------------------------------------------
// TFileResourceStream destructor
//----------------------------------------------------------------------------------------


TFileResourceStream::~TFileResourceStream()
{
}

//----------------------------------------------------------------------------------------
// TFileResourceStream::WriteBytes: 
//----------------------------------------------------------------------------------------


void TFileResourceStream::WriteBytes(const void* p, long count)
{
	CTempUseResFile_AC tempUseResFile(fFile->GetRsrcRefNum());
	
	Inherited::WriteBytes (p, count);
}

//----------------------------------------------------------------------------------------
// TFileResourceStream::ReadBytes: 
//----------------------------------------------------------------------------------------


void TFileResourceStream::ReadBytes(void* p, long count)
{
	CTempUseResFile_AC tempUseResFile(fFile->GetRsrcRefNum());
	
	Inherited::ReadBytes (p, count);
}

//----------------------------------------------------------------------------------------
// TFileResourceStream::SetSize: 
//----------------------------------------------------------------------------------------


void TFileResourceStream::SetSize(long newSize)
{
	CTempUseResFile_AC tempUseResFile(fFile->GetRsrcRefNum());
	
	Inherited::SetSize (newSize);
}

//----------------------------------------------------------------------------------------
// End of UFileStream.cp


#endif