//----------------------------------------------------------------------------------------
// UFloatWindow.cp 
// Copyright  1987-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UFLOATWINDOW__
#include "UFloatWindow.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

// Toolbox

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

//----------------------------------------------------------------------------------------
bool gUFloatWindowInitialized;		// declared in header

//----------------------------------------------------------------------------------------

//========================================================================================
// CLASS TFloatWindow
//========================================================================================
#undef Inherited
#define Inherited TWindow


MA_DEFINE_CLASS(TFloatWindow);

//----------------------------------------------------------------------------------------
// TFloatWindow constructor
//----------------------------------------------------------------------------------------


TFloatWindow::TFloatWindow(WindowRef itsWMgrWindow,
									   bool canResize,
									   bool canClose,
									   bool inDispose) :
	TWindow(itsWMgrWindow, canResize, canClose, inDispose)
{
	// Override default value for fResizeLimits
	fResizeLimits[topLeft].Set(1, 1);
	
	fHideOnSuspend = true;
	fDoFirstClick = true;
	fFloats = true;
	fGenerateActivates = false;
	fOMClassID = cPalette;			// Establish AEOM class ID

	SetWindowKind(itsWMgrWindow,kFloatingWindowKind);// mark it as a floating window 
}

//----------------------------------------------------------------------------------------
// TFloatWindow destructor
//----------------------------------------------------------------------------------------


TFloatWindow::~TFloatWindow()
{
}

//----------------------------------------------------------------------------------------
// TFloatWindow::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TFloatWindow::GetStandardSignature() const// override 
{
	return kStdFloatWindow;
}

//----------------------------------------------------------------------------------------
// TFloatWindow::ReadFields: 
//----------------------------------------------------------------------------------------


void TFloatWindow::ReadFields(CStream_AC* aStream)// override 
{
	Inherited::ReadFields(aStream);

	SetWindowKind(fWMgrWindow, kFloatingWindowKind);// mark it as a floating window 
}

//----------------------------------------------------------------------------------------
// TFloatWindow::GetBehindWindowPtr: 
//----------------------------------------------------------------------------------------


WindowRef TFloatWindow::GetBehindWindowPtr() const

{
	// bring the float window up in front of all other windows
	return (WindowRef) - 1;						
}

//----------------------------------------------------------------------------------------
// TFloatWindow::Close: 
//----------------------------------------------------------------------------------------


void TFloatWindow::Close()	// Override 

{
	Inherited::Close();
	
	TEventHandler* activeTarget = NULL;
	
	TWindow* activeWindow = gDispatcher->GetActiveWindow(!kFloatersOK);
	
	// Non-floater is always targettable.
	if (activeWindow)
		activeTarget = activeWindow->GetWindowTarget();
	
	if (activeTarget)
	{
		SetTarget(activeTarget);
		activeTarget->SetTargetSelection(kRedraw);
	}
	else
		SetTarget(gDispatcher);
}

//----------------------------------------------------------------------------------------
// TFloatWindow::DoSetupMenus: 
//----------------------------------------------------------------------------------------


void TFloatWindow::DoSetupMenus()
{
		Inherited::DoSetupMenus();
		Enable(cClose, false);		// Floaters cannot be closed via cmd-W! 
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// InitUFloatWindow:
//----------------------------------------------------------------------------------------


void InitUFloatWindow()
{
	if (!gUFloatWindowInitialized)
	{
		gUFloatWindowInitialized = true;

#if qTemplateViews
	MA_REGISTER_SIGNATURE(TFloatWindow, kStdFloatWindow);
#endif
	}
}

//----------------------------------------------------------------------------------------
// End of UFloatWindow.cp


