//----------------------------------------------------------------------------------------
// UInsertPartCommand.cp
// Copyright  1994-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UINSERTPARTCOMMAND__
#include "UInsertPartCommand.h"
#endif

#if qContainer

#ifndef __UODPARTVIEW__
#include "UODPartView.h"
#endif

// MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// CALib

#if qContainer
	#ifndef _CALIB_
	#include "CALib.h"
	#endif
#endif

// Toolbox

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

//========================================================================================
// CLASS TInsertPartCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------


TInsertPartCommand::TInsertPartCommand(CommandNumber itsCommandNumber,
											   TDocument* itsDocument,
											   TODPartView* itsPartView) :
		TCommand(itsCommandNumber, itsDocument,
						 kCanUndo, kCausesChange, itsDocument),

	fODPartView(NULL),
	fDocument(NULL)
{
	bool  fileChosen = ChoosePartFile();
	if (fileChosen)
	{
//		ICommand(itsCommandNumber, itsDocument,
//						 kCanUndo, kCausesChange, itsDocument);
		fDocument = itsDocument;
		fODPartView = itsPartView;
	}
	

}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------


TInsertPartCommand::~TInsertPartCommand()
{
}

//----------------------------------------------------------------------------------------
// TInsertPartCommand::DoIt()
//----------------------------------------------------------------------------------------


void TInsertPartCommand::DoIt()
{
	
	if (fODPartView)
	{
		
		//Remove the old part.
		if(fODPartView->HasPart())
		{
			fODPartView->RemovePart();
		}
		
		
		//if(!fODPartView)
		//fODPartView = new
		
		fODPartView->AquirePartFromFile(fFileSpec);
	}
}

//----------------------------------------------------------------------------------------
// TInsertPartCommand::UndoIt()
//----------------------------------------------------------------------------------------


void TInsertPartCommand::UndoIt()
{
	if(fODPartView->HasPart())
		fODPartView->RemovePart();
}

//----------------------------------------------------------------------------------------
// TInsertPartCommand::ChoosePartFile
//----------------------------------------------------------------------------------------


bool TInsertPartCommand::ChoosePartFile()
{
	::ThrowIfOSErr_AC(MAInteractWithUser()); 
	gClipboardMgr->AboutToLoseControl(true);			// so scrap gets converted

	bool				done = false;

	// Insure we can go modal at this time.
	WindowPtr		frontWindow = CAGetFrontDocWindow();
	if (CARequestModalFocus(frontWindow))
	{
		StandardFileReply	customReply;
		FileFilterYDUPP		cgfFileFilter = NewFileFilterYDProc(TInsertPartCommand::PartFileFilter);
		ModalFilterYDUPP 	cgfModalFilter = NewModalFilterYDProc(gModalFilterYDProcPtr);

		CustomGetFile(cgfFileFilter,
						-1,				// Pass all file types
						NULL,			// No type list
						&customReply,
						sfGetDialogID,
						kBestSystemLocation,
						NULL,			// Dialog hook function
						cgfModalFilter,
						NULL,			// Activate list
						NULL,			// Activate proc
						NULL);

		CARelinquishModalFocus(frontWindow);
	
		if (customReply.sfGood)
		{
			fFileSpec = customReply.sfFile;
			done = true;
		}
	}
	return done;	
}

//----------------------------------------------------------------------------------------
// TInsertPartCommand::PartFileFilter
//----------------------------------------------------------------------------------------


pascal Boolean TInsertPartCommand::PartFileFilter(CInfoPBPtr pb)
{
	HFileInfo*	fpb = (HFileInfo*)pb;

	if (!(fpb->ioFlAttrib & 16))
	{
		return (!(fpb->ioFlFndrInfo.fdCreator == kODShellSignature) &&
			(fpb->ioFlFndrInfo.fdType != 'APPL') &&
			(fpb->ioFlFndrInfo.fdType != 'shlb'));
	}
	return false;
}

//----------------------------------------------------------------------------------------
// TInsertPartCommand::Commit
//----------------------------------------------------------------------------------------


void TInsertPartCommand::Commit()
{
	if (!fCommandDone && fODPartView)
		delete fODPartView;
}

#endif // qContainer

//----------------------------------------------------------------------------------------
// End of UInsertPartCommand.cp


