//----------------------------------------------------------------------------------------
// UMacAppGlobals.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#include "MacAppRequired.h"

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

// MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

// always include for TDragDropBehavior stub
#ifndef __UDRAGDROBEHAVIOR__
#include "UDragDropBehavior.h"
#endif

#ifndef __UDRAWINGENVIRONMENT__
#include "UDrawingEnvironment.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __UReporter__
#include "UReporter.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __UUNDO__
#include "UUndo.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#	ifndef __AGA_AC__
#	include "AGA_AC.h"
#	endif

#if qGrayscaleAppearance
#	ifndef __CDialog_AC__
#	include "CDialog_AC.h"
#	endif
#endif

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#if qWantsThreadManager	
#	ifndef __ThreadUtilities_AC__
#	include "ThreadUtilities_AC.h"
#	endif
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FOLDERS__
#include <Folders.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


#ifndef __STDLIB__
#include <stdlib.h>
#endif

//----------------------------------------------------------------------------------------
#if !defined(__MWERKS__)
QDGlobals qd;
#endif

CTextStyle_AC gSystemStyle;
CTextStyle_AC gApplicationStyle;
CTextStyle_AC gApplicationStyle9;

WindowRef gWorkPort;
static CAutoPtr_AC<CWindowRecord> pWorkPortStorage;

RgnHandle gTemporaryRegion;

NMRecPtr gNotificationPtr;

ModalFilterYDProcPtr gModalFilterYDProcPtr = MacAppStandardFileFilter;

#if qDebug
bool gBusyTemporaryRegion;
CStr255_AC gUsedBy;
#endif


//========================================================================================
// GLOBAL Procedures
//========================================================================================
static short FindPos(const CStr255_AC& pattern,
					short patternScript,
					 CStr255_AC& source,
					 short sourceScript);

//========================================================================================
// GLOBAL Functions
//========================================================================================

#if qOldConfigurationFlags
static void DefineConfiguration(Configuration& theConfiguration);
#endif

//----------------------------------------------------------------------------------------
// GetAlertButtonTitle: Retrieve the title of the button control. If itemNo isn't a
// button, then return ''.
//----------------------------------------------------------------------------------------


void GetAlertButtonTitle(DialogRef theDialog,
						 short itemNo,
						 CStr255_AC& theTitle)
{
	short itemType;
	Handle item;
	CRect_AC box;
	GetDialogItem(theDialog, itemNo, &itemType, &item, &box);

	if (itemType == (ctrlItem + btnCtrl))
		GetControlTitle((ControlRef)item, theTitle);
	else
		theTitle.Empty();
}

//----------------------------------------------------------------------------------------
// DoAlertKeyDown: Handle a keypress that has been mapped to one of the button controls.
//----------------------------------------------------------------------------------------


void DoAlertKeyDown(DialogRef theDialog,
					short itemNo)
{
#if qGrayscaleAppearance
	CDialog_AC*	agaDialog = CDialog_AC::ObjectFromDialogPtr(theDialog);
	
	if (agaDialog != NULL)
		agaDialog->FlashButton(itemNo);
	else
#endif
		{
		short itemType;
		Handle item;
		CRect_AC box;

		GetDialogItem(theDialog, itemNo, &itemType, &item, &box);
		if (itemType == (ctrlItem + btnCtrl))
		{
			// this code gives visual feedback 
			HiliteControl((ControlRef)item, kControlUpButtonPart);// hilite the button 

			long finalTicks;
			Delay(8, &finalTicks);					// delay for 8 ticks 

			HiliteControl((ControlRef)item, kControlNoPart);// stop hiliting the button 
		}
	}
}

//----------------------------------------------------------------------------------------
// CompareAlertKeysToItem: Compares the buffered multi-byte chars to the first character
// of each button title 1st button in alert (by convention == "OK"). 2nd button in alert
// (by convention == "Cancel"). 3rd button in alert (by convention == "No")
//----------------------------------------------------------------------------------------


bool CompareAlertKeysToItem(DialogRef theDialog,
							   const CStr2_AC& theChars,
							   short& itemHit)
{
	bool returnVal = false;

	static const short itemtests[3] = { ok, cancel, kNoButton };

	for (short i = 0; i < 4; ++i)
	{
		CStr255_AC title;
		GetAlertButtonTitle(theDialog, itemtests[i], title);
		if (CompareMultiByteChars(theChars, title, false))
		{
			itemHit = itemtests[i];
			returnVal = true;
			break;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// DrawDefaultProc:  Draw the thick rounded rectangle around the default button
// (Imported to MacApp from Ponnuki by GA)
//
// This routine uses Keith Rollin's algorithm, as presented in the USENET Guide to
// Programming the Macintosh. I have modified the basic algorithm only slightly--I add
// two to the calculated 'buttonOval' value. This gets better results, particularly for
// buttons of the default size (18 points).
//----------------------------------------------------------------------------------------


void DrawDefaultProc(DialogRef dlog)
{
	// Don't call GetDialogItem if the default button # has a strange value
	short defaultButton = GetDialogDefaultItem(dlog);
	if (defaultButton > 0)
	{
		GrafPtr savePort;
		GetPort(&savePort);
		SetPortWindowPort(dlog);

		short type = 0;
		ControlRef buttonHandle = NULL;
		CRect_AC outlineBox;

		GetDialogItem(dlog, defaultButton, &type, (Handle *) & buttonHandle, outlineBox);
		outlineBox.Inset(-4, -4);

		// Don't draw the bold outline around the default button
		// unless it really is a button
		if (type == ctrlItem + btnCtrl)
		{
			PenState saveState;
			short buttonOval;

			GetPenState(&saveState);

			// We want to draw the thick line with a normal
			// pen pattern that is 3 pixels wide
			PenNormal();
			PenSize(3, 3);

			// If the button we are outlining is disabled,
			// draw the outline with a gray pattern.
			
			if ((*buttonHandle)->contrlHilite == 255)
				PenPat(&qd.gray);

			// Calculate the curvature to use and draw the thick line
			buttonOval = 2 + (outlineBox.bottom - outlineBox.top) / 2;
			FrameRoundRect(outlineBox, buttonOval, buttonOval);

			SetPenState(&saveState);
		}
		SetPort(savePort);
	}
}

//----------------------------------------------------------------------------------------
// MinimalAlertFilter: This is the minimally-functional alert filter that MacApp uses
// in cases where gDispatcher has not yet been initialized.  TApplication.AlertFilter
// calls this filter as well.
//----------------------------------------------------------------------------------------


pascal Boolean MinimalAlertFilter(DialogRef theDialog,
								  EventRecord& theEvent,
								  short& itemHit)
{
	bool returnValue = false;

	switch (theEvent.what)
	{
		case updateEvt:
			if (((DialogRef)theEvent.message) == theDialog)
				DrawDefaultProc(theDialog);
			break;

		case keyDown:
			{
				static CStr2_AC bufferedCharacter;	// hangs around in between keystrokes
				unsigned char theChar = static_cast<unsigned char>(theEvent.message & charCodeMask);

				bool processCharacter = true;
				switch (MACharacterByteType((Ptr) & theChar, 0, smCurrentScript))
				{
					case smFirstByte:
						processCharacter = false;
						// no break - just drop through

					case smSingleByte:
						bufferedCharacter = theChar;
						break;

					case smLastByte:
						bufferedCharacter += theChar;
						break;
				}

				if (processCharacter)
				{
					if ((bufferedCharacter == chEnter) || (bufferedCharacter == chReturn))
					{
						itemHit = GetDialogDefaultItem(theDialog);
						DoAlertKeyDown(theDialog, itemHit);
						returnValue = true;
					}
					else if ((bufferedCharacter == '`') || (bufferedCharacter == chEscape) || ((bufferedCharacter == '.') && (theEvent.modifiers & cmdKey)))
					{
						DoAlertKeyDown(theDialog, cancel);
						itemHit = cancel;
						returnValue = true;
					}
				}

				break;
			}
	}

	return returnValue;
}

//----------------------------------------------------------------------------------------
// MacAppAlertFilter: 
//----------------------------------------------------------------------------------------


pascal Boolean MacAppAlertFilter(DialogRef theDialog,
								 EventRecord* theEvent,
								 short* itemHit)
{
	return gDispatcher ? gDispatcher->AlertFilter(theDialog, *theEvent, *itemHit) : MinimalAlertFilter(theDialog, *theEvent, *itemHit);
}

//----------------------------------------------------------------------------------------
// MacAppAppleEventIdleProc: 
//----------------------------------------------------------------------------------------


pascal Boolean MacAppAppleEventIdleProc(EventRecord& theEventRecord,
										long& sleepTime,
										RgnHandle& mouseRgn)
{
	return gDispatcher ? gDispatcher->AppleEventIdleProc(theEventRecord, sleepTime, mouseRgn) : false;
}

//----------------------------------------------------------------------------------------
// MacAppStandardFileFilter: 
//----------------------------------------------------------------------------------------


pascal Boolean MacAppStandardFileFilter(DialogRef theDialog,
										EventRecord* theEvent,
										short* itemHit,
										void* yourDataPtr)
{
	return gDispatcher ? gDispatcher->StandardFileFilter(theDialog, *theEvent, *itemHit, yourDataPtr) : false;
}

//----------------------------------------------------------------------------------------
// ApplicationBeep: 
//----------------------------------------------------------------------------------------


void ApplicationBeep()
{
#if qMSWindows
	if (gDispatcher)
		gDispatcher->Beep(2);
#else
	gDispatcher ? gDispatcher->Beep(2) : ::SysBeep(2);	// At least give some indication 
#endif
}

//----------------------------------------------------------------------------------------
// CleanupMacApp: 
//----------------------------------------------------------------------------------------


void CleanupMacApp()
{
	static bool inCleanupMacApp;					// flag ensures one and only one call
	
	if (!inCleanupMacApp)
	{
		inCleanupMacApp = true;						// dont' re-enter

		::SetResLoad(true);							// Make sure segments can load 


		//
		// Override Terminate if you need any special shutdown handling.
		//
		if (gDispatcher)
			gDispatcher->Terminate();

		UnpatchAll();

		// Clear gDispatcher so nobody can refer to the object
		gDispatcher = NULL;
	}
}

//----------------------------------------------------------------------------------------
// DoneWithTemporaryRegion: Indicates that gTemporaryRegion is no longer in use. Call this
// only if qDebug is true.
//----------------------------------------------------------------------------------------
#if qDebug


void DoneWithTemporaryRegion()
{
	if (!gBusyTemporaryRegion)
		::ProgramBreak_AC("DoneWithTemporaryRegion called, but gTemporaryRegion is not locked");

	gBusyTemporaryRegion = false;
	gUsedBy.Empty();
	SetEmptyRgn(gTemporaryRegion);
}

#endif

//----------------------------------------------------------------------------------------
// EnterDebugger: 
//----------------------------------------------------------------------------------------
#if qDebug


void EnterDebugger(bool entering)
{
	if (gDispatcher)
		gDispatcher->ActivateBusyCursor(!entering);
}

#endif

//----------------------------------------------------------------------------------------
// ExitMacApp: 
//----------------------------------------------------------------------------------------


void ExitMacApp()
{
	CleanupMacApp();

	exit(EXIT_FAILURE);
}

//----------------------------------------------------------------------------------------
// GetNewCenteredDialog: 
//----------------------------------------------------------------------------------------


DialogRef GetNewCenteredDialog(ResNumber dialogID,
							   Ptr dStorage,
							   WindowRef behind)
{
	DialogRef returnVal = NULL;

	DialogTHndl dlogTemplate;

	SetCursor(&(qd.arrow));

	if (gDispatcher)
		gDispatcher->InvalidateMouseRegions();

	dlogTemplate = (DialogTHndl)GetResource('DLOG', dialogID);
	if (dlogTemplate)
	{
		CenterRectOnScreen(((CRect_AC &)(*dlogTemplate)->boundsRect), true, true, true);
		returnVal = GetNewDialog(dialogID, dStorage, behind);
	}
	else
	{
		ApplicationBeep();
#if qDebug
		CStr255_AC theString;
		ConcatNumber("Unable to find 'DLOG' resource ", dialogID, theString);
		::ProgramBreak_AC(theString);
#endif

	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// GetTextStyleFontInfo: Really a utility, but the gWorkPort isn't reachable from
// UMacAppUtilities
//----------------------------------------------------------------------------------------


void GetTextStyleFontInfo(const CTextStyle_AC& theTextStyle,
						  FontInfo& theFontInfo,
						  short& theFontHeight)
{
	GrafPtr savedPort;

	GetPort(&savedPort);
	SetPortWindowPort(gWorkPort);
	SetPortTextStyle(theTextStyle);
	theFontHeight = MAGetFontInfo(theFontInfo);
	SetPort(savedPort);
}

#if qOldConfigurationFlags

//----------------------------------------------------------------------------------------
// DefineConfiguration: 
//----------------------------------------------------------------------------------------


void DefineConfiguration(Configuration& theConfiguration)
{
	theConfiguration.environsVersion 		= GetEnvironsVersion_AC();
	theConfiguration.machineType 			= GetMachineType_AC();
	theConfiguration.systemVersion 			= GetSystemVersion_AC();
	theConfiguration.processor 				= GetProcessor_AC();
	theConfiguration.hasFPU 				= HasFPU_AC();
	theConfiguration.hasColorQD 			= HasColorQD_AC();
	theConfiguration.has32BitQD 			= Has32BitQD_AC();
	theConfiguration.keyboardType	 		= KeyboardType_AC();
	theConfiguration.atDrvrVersNum 			= GetATDrvrVersNum_AC();
	theConfiguration.hasSCSI				= HasSCSI_AC();
	theConfiguration.hasAUX 				= HasAUX_AC();
	theConfiguration.hasTempMem 			= HasTempMem_AC();
	theConfiguration.teVersion 				= GetTEVersion_AC();
	theConfiguration.hasDesktopBus 			= HasDesktopBus_AC();
	theConfiguration.hasTrueType 			= HasTrueType_AC();
	theConfiguration.hasDragManager 		= HasDragManager_AC();
	theConfiguration.hasThreadManager 		= HasThreadManager_AC();
	theConfiguration.hasAOCEToolBox 		= HasAOCEToolBox_AC();
	theConfiguration.hasQDGXGraphics 		= HasQDGXGraphics_AC();
	theConfiguration.hasQDGXPrinting 		= HasQDGXPrinting_AC();
	theConfiguration.hasCFM 				= HasCFM_AC();
	theConfiguration.hasTranslationManager 	= HasTranslationManager_AC();
	theConfiguration.hasSpeechManager 		= HasSpeechManager_AC();
	theConfiguration.hasCustomFile 			= HasCustomFile_AC();
	theConfiguration.hasQuickTime 			= HasQuickTime_AC();
	theConfiguration.hasTSM 				= HasTSM_AC();
	theConfiguration.hasAppleScript 		= HasAppleScript_AC();

	theConfiguration.isOnlyBackground 		= IsOnlyBackground();
	theConfiguration.isHighLevelEventAware 	= IsHighLevelEventAware();

#if 0
	// Removed due to ASLM removal vs. deinstall problems.
	theConfiguration.hasASLM 				= HasASLM();
#endif
}

#endif // qOldConfigurationFlags

//----------------------------------------------------------------------------------------
// InitUMacApp
//----------------------------------------------------------------------------------------
void InitUMacApp(short callsToMoreMasters)	
{
	InitUMacApp_Step1();									
	InitUMacApp_Step3(callsToMoreMasters);
}

//----------------------------------------------------------------------------------------
// InitUMacApp_Step1
//----------------------------------------------------------------------------------------
void InitUMacApp_Step1()
{
#if qTidyHeap
	CTidyHeap_AC::Instance()->StopTrackingUnknownBlocks();
#endif

//	CException_AC::SetTerminateHandler();
		// Install default termination handler.
	
	InitUReporter(); 
		
	UniversalStartup();

#if qOldConfigurationFlags
	DefineConfiguration(gConfiguration);
#endif

	if (!ValidateConfiguration())
		AlertUnsupportedConfiguration();

	// Init the basics used by the ErrorMgr
	gMacAppAlertFilter = MacAppAlertFilter;	
	InitAGA_AC();
	

#if qWantsThreadManager
	InitThreads_AC();
#endif	
	
}

//----------------------------------------------------------------------------------------
// InitUMacApp_Step3
//----------------------------------------------------------------------------------------

void InitUMacApp_Step3(short callsToMoreMasters)
{
	// Install MacApp's Memory management systems
	InitCoreMacApp(callsToMoreMasters);

	DoInitUMacApp();							// do rest of initialization 
}

//----------------------------------------------------------------------------------------
// DoInitUMacApp: Must be in the init segment
//----------------------------------------------------------------------------------------


void DoInitUMacApp()
{
#if qDebug || qTheDebugger
	InitUDebug();
#endif

	// assign the global text style records from resources
	gSystemStyle = CTextStyle_AC(::GetSysFont(), normal, 0, gRGBBlack);
	gApplicationStyle = CTextStyle_AC(::GetAppFont(), normal, 0, gRGBBlack);
	gApplicationStyle9 = CTextStyle_AC(::GetAppFont(), normal, 9, gRGBBlack);

	atexit(CleanupMacApp);

	// Initialize the view system
	bool initGX = false;
#if qGXViews || qGXPrinting
	initGX = true;
#endif

	TView::InitUView(initGX);

	// Other 1-time initialization 
	gTemporaryRegion = MakeNewRgn();

	// Create a work port for our convenience
	pWorkPortStorage = TH_new CWindowRecord;
	if (qNeedsColorQD || HasColorQD_AC())
		gWorkPort = NewCWindow(pWorkPortStorage, gZeroRect, gEmptyString, false, documentProc, NULL, false, 0);
	else
		gWorkPort = NewWindow((WindowRef)pWorkPortStorage.get(), gZeroRect, gEmptyString, false, documentProc, NULL, false, 0);

	::ThrowIfNULL_AC(gWorkPort);

	TPrintHandler::gPrintHandler = TH_new TPrintHandler;

#if qTemplateViews
	{
		// =============================================== 
		// Suppress Linker dead stripping of these classes 
#if qDrag
		MA_REGISTER_CLASS(TDragDropBehavior);
#endif
		// =============================================== 

		MA_REGISTER_SIGNATURE(TView, kStdView);
		MA_REGISTER_SIGNATURE(TView, kStdDefaultView);
		MA_REGISTER_SIGNATURE(TIncludeView, kStdIncludeAt);
		MA_REGISTER_SIGNATURE(TWindow, kStdWindow);
		MA_REGISTER_SIGNATURE(TScrollBar, kStdScrollBar);
		MA_REGISTER_SIGNATURE(TScrollerScrollBar, kStdScrollerScrollBar);
		MA_REGISTER_SIGNATURE(TScroller, kStdScroller);

		MA_REGISTER_SIGNATURE(TDrawingEnvironment, kStdDrawingEnvironment);
	}
#endif

	InitUAdorners();							// Temporarily needed for debug window 

	InitUMenuMgr();

	// check for special case of single-undo
	short menu;
	short item;
	CommandToMenuItem(cUndoRedo, menu, item);
	if (menu)
		TUndoHandler::fgUndoHandler.SetMaxDesiredItems(1);

	InitUClipboardMgr();

	// create gServerAddress
	ProcessSerialNumber psn;
	psn.highLongOfPSN = 0;
	psn.lowLongOfPSN = kCurrentProcess;
	::ThrowIfOSErr_AC(AECreateDesc(typeProcessSerialNumber, (Ptr) & psn, sizeof(ProcessSerialNumber), &gServerAddress));

	// initialize the pseudo-random sequence generation function
	unsigned long time;
	GetDateTime(&time);
	srand((unsigned int)time);

#if qDebug
	//InitUDialog(); //Necessary for debug dialog.
#if qPerform
	InitUTEView();
#endif

#endif


}

//----------------------------------------------------------------------------------------
// InstallIfPrintHandler: 
//----------------------------------------------------------------------------------------


void InstallIfPrintHandler(TPrintHandler* aPrintHandler,
						   TView* aView)

{
	// Install a copy of aPrintHandler into the view.
	if (aPrintHandler && aView)
	{
		TPrintHandler * aNewPrintHandler = static_cast<TPrintHandler*>(aPrintHandler->Clone());// signals failure if can't clone

		aNewPrintHandler->fView = aView;
		TDocument* itsDocument = aView->GetDocument();
		if (itsDocument)
			aNewPrintHandler->fDocument = itsDocument;// so the printinfo can be shared if necessary

		aNewPrintHandler->SetDefaultPrintInfo();// do this after we have a document (if any)

		//
		// Now do the attachments
		//
		if (itsDocument)
			itsDocument->AttachPrintHandler(aNewPrintHandler);
		aView->AttachPrintHandler(aNewPrintHandler);
	}
}

//----------------------------------------------------------------------------------------
// FindPos: 
//----------------------------------------------------------------------------------------


short FindPos(const CStr255_AC& pattern,
					short patternScript,
					 CStr255_AC& source,
					 short sourceScript)
{
	int i = 0;
	int j = 0;
	int position = 0;

	do
	{
		++i;
		position = i;
		for (j = 1; j <= pattern.Length(); ++j)
			if (!((source[i + j - 1] == pattern[j]) && (MACharacterByteType((Ptr) & source[1], i + j - 2, sourceScript) == MACharacterByteType((Ptr) & pattern[1], j - 1, patternScript))))
			{
				position = 0;
				break;
			}
	} while (!((position > 0) || (i >= source.Length() - pattern.Length() + 1)));

	return position;
}


//----------------------------------------------------------------------------------------
// ParseTitleTemplate: 
//----------------------------------------------------------------------------------------
bool ParseTitleTemplate(CStr255_AC& itsTemplate,
						   short& preDocname,
						   short& constTitle,
						   short inScript)

{
	const CStr255_AC kPreDocname = "<<<";
	const short kPreSize = 3;
	const CStr255_AC kPostDocname = ">>>";
	const short kPostSize = 3;

	if (itsTemplate.IsEmpty())
	{
		preDocname = 1;
		constTitle = 0;
	}
	else
	{
		preDocname = FindPos(kPreDocname, smRoman, itsTemplate, inScript);
		if (preDocname > 0)
		{
			itsTemplate.Delete(preDocname, kPreSize);
			short x = FindPos(kPostDocname, smRoman, itsTemplate, inScript);
			if (x)
			{
				itsTemplate.Delete(x, kPostSize);
				constTitle = itsTemplate.Length() - x + preDocname;
			}
			else
				constTitle = preDocname - 1;
		}
	}

	return preDocname > 0;
}

//----------------------------------------------------------------------------------------
// SubstituteInTitle: 
//----------------------------------------------------------------------------------------


bool SubstituteInTitle(CStr255_AC& title,
						  const CStr255_AC& newStuff,
						  short preDocname,
						  short constTitle)
{
	bool returnVal = false;

	if (preDocname > 0)
	{
		if (constTitle)
		{
			title.Delete(preDocname, title.Length() - constTitle);
			title.Insert(newStuff, preDocname);
		}
		else
			title = newStuff;
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// UseTemporaryRegion: Call this when you are about to use gTemporaryRegion and qDebug is
// true. Used with DoneWithTemporaryRegion will prevent you from trying to use
// gTemporaryRegion from two places at the same time.
//----------------------------------------------------------------------------------------
#if qDebug


void UseTemporaryRegion(const CStr255_AC& byWhom)
{
	if (gBusyTemporaryRegion)
	{
		ProgramReport_AC(false, "'%s' is trying to lock gTemporaryRegion,", (const char*)(byWhom));
		ProgramReport_AC(false, "but it is already locked by '%s'\n", (const char*)(gUsedBy));
		::ProgramBreak_AC("Error in UseTemporaryRegion");
	}
	else
	{
		gBusyTemporaryRegion = true;
		gUsedBy = byWhom;
	}
}

#endif


//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// GetSysVolume: 
//----------------------------------------------------------------------------------------


OSErr GetSysVolume(short& vRefNum)
{
	long dir;

	return FindFolder(kOnSystemDisk, kSystemFolderType, false, &vRefNum, &dir);
}


//----------------------------------------------------------------------------------------
// GetIndVolume: 
//----------------------------------------------------------------------------------------


OSErr GetIndVolume(short index,
				   short& vRefNum)
{
	ParamBlockRec pb;
	pb.volumeParam.ioCompletion = NULL;
	pb.volumeParam.ioNamePtr = NULL;
	pb.volumeParam.ioVolIndex = index;

	OSErr err = PBGetVInfoSync(&pb);

	vRefNum = pb.volumeParam.ioVRefNum;

	return err;
}


//----------------------------------------------------------------------------------------
// VolHasDesktopDB: 
//----------------------------------------------------------------------------------------


OSErr VolHasDesktopDB(short vRefNum,
					  bool& hasDesktop)
{
	GetVolParmsInfoBuffer info;

	HParamBlockRec pb;
	pb.ioParam.ioCompletion = NULL;
	pb.ioParam.ioVRefNum = vRefNum;
	pb.ioParam.ioNamePtr = NULL;
	pb.ioParam.ioBuffer = (Ptr) & info;
	pb.ioParam.ioReqCount = sizeof(GetVolParmsInfoBuffer);

	OSErr err = ::PBHGetVolParmsSync(&pb);

	hasDesktop = err == noErr && info.vMAttrib & (1L << bHasDesktopMgr);

	return err;
}


//----------------------------------------------------------------------------------------
// FindAppOnVolume: 
//----------------------------------------------------------------------------------------


OSErr FindAppOnVolume(OSType sig,
					  short vRefNum,
					  CFSSpec_AC& thefile)
{
	OSErr err;

	DTPBRec pb;
	pb.ioCompletion = NULL;
	pb.ioVRefNum = vRefNum;
	pb.ioNamePtr = NULL;
	if ((err = PBDTGetPath(&pb)) == noErr)
	{
		short refNum = pb.ioDTRefNum;

		pb.ioCompletion = NULL;
		pb.ioDTRefNum = refNum;
		pb.ioIndex = 0;
		pb.ioFileCreator = sig;
		pb.ioNamePtr = (StringPtr)thefile.name;
		err = PBDTGetAPPLSync(&pb);					// Find it!

		if (err == fnfErr)
			err = afpItemNotFound;					// Bug in PBDTGetAPPL
		if (err == noErr)
		{
			thefile.vRefNum = vRefNum;
			thefile.parID = pb.ioAPPLParID;
		}
	}

	return err;
}

//----------------------------------------------------------------------------------------
// LaunchAppByFSSpec: 
//----------------------------------------------------------------------------------------


OSErr LaunchAppByFSSpec(const CFSSpec_AC& fileSpec,
						LaunchFlags launchControlFlags,
						ProcessSerialNumber& psn)
{
	CFSSpec_AC localFSSpec = fileSpec;				// so we can take its address below

	LaunchParamBlockRec pb;
	pb.launchBlockID = extendedBlock;
	pb.launchEPBLength = extendedBlockLen;
	pb.launchFileFlags = launchNoFileFlags;
	pb.launchControlFlags = launchControlFlags | launchNoFileFlags;
	pb.launchAppSpec = &localFSSpec;
	pb.launchAppParameters = NULL;

	OSErr err = LaunchApplication(&pb);
	if (err == noErr)
		psn = pb.launchProcessSN;

	return err;
}


//----------------------------------------------------------------------------------------
// LaunchBySignature: 
//----------------------------------------------------------------------------------------


OSErr LaunchBySignature(OSType sig,
						ProcessSerialNumber& psn,
						CFSSpec_AC* fileSpec,
						bool* launched,
						bool allowLaunch,
						LaunchFlags launchControlFlags)
{
	// First see if it's already running:
	OSErr err = FindProcessBySignature(sig, psn, fileSpec);

	if (launched)
		*launched = false;

	if (err == noErr && !(launchControlFlags & launchDontSwitch))
		err = SetFrontProcess(&psn);			// They wanted to switch to it
	else if (err == procNotFound && allowLaunch)
	{
		// Well, it's not running but it's okay to launch it. Let's have a look around:

		short sysVRefNum;
		err = GetSysVolume(sysVRefNum); // Find boot volume

		short vRefNum = sysVRefNum;			// Start search with boot volume
		short index = 0;
		while (err == noErr)				// Keep going until we run out of vols
		{
			if (!index || vRefNum != sysVRefNum)
			{
				bool hasDesktopDB;

				if ((err = VolHasDesktopDB(vRefNum, hasDesktopDB)) != noErr)
					break;
				else if (hasDesktopDB)
				{
					// If volume has a desktop DB,
					CFSSpec_AC file;
					err = FindAppOnVolume(sig, vRefNum, file);// ask it to find app
					if (err == noErr)
					{
						// If found,
						if (fileSpec)
							*fileSpec = file;
						if (launched)
							*launched = true;
						err = LaunchAppByFSSpec(file, launchControlFlags, psn);// Launch it!
						break;
					}
					else if (err != afpItemNotFound && err != paramErr/* in case of Volume with no files (audio CD) */ )
						break;
				}
			}
			err = GetIndVolume(++index, vRefNum);// Else go to next volume
		} 

		if (err == nsvErr || err == afpItemNotFound)
			err = fnfErr;					// File not found on any volume
	}

	return err;
}

//----------------------------------------------------------------------------------------
// End of UMacAppGlobals.cp


