//----------------------------------------------------------------------------------------
// UMacAppUtilities.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

// ACS

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

// MacApp

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#if qPowerPC || qModelCFM
	#ifndef __CODEFRAGMENTS__
	#include <CodeFragments.h>
	#endif
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


#ifndef __STRING__
#include <string.h>
#endif

//----------------------------------------------------------------------------------------
// extern definitions:
extern WindowRef gWorkPort;

// constants

#if qOldConfigurationFlags
Configuration gConfiguration;
#endif

#if qDebugMsg
const CStr15_AC gBoolString[2] = { "false", "true"};
#endif

bool gUGridViewInitialized;

const VHSelect gOrthogonal[2] = { hSel, vSel};

const CPoint_AC kBestSystemLocation(-1, -1);

const CStr2_AC gEmptyString(""); 		// Constructor keeps this from getting dead-stripped

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static pascal void StdNoRect(GrafVerb			/* verb */,
							 const CRect_AC&		/* r */ );
static void CheckAdornment(CntlAdornment p,
						   CntlAdornment alias,
						   CStr255_AC theString,
						   char* name);
static void CheckStyleItem(unsigned char s,
						   unsigned char alias,
						   CStr255_AC theString,
						   char* name);

//========================================================================================
// CLASS CStringListRsrc_AC
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::GetString: 
//----------------------------------------------------------------------------------------

void CStringListRsrc_AC::GetString(short index, CStr255_AC& theString) const
{
	if (fStrListID != kNoResource)
		GetIndString(theString, fStrListID, index);
}

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::FindString: 
//----------------------------------------------------------------------------------------

short CStringListRsrc_AC::FindString(const CStr255_AC& theString, bool addString)
{
	short returnVal = kNoResource;

	short numStrings = CountStrings();

	for (short i = 1; i <= numStrings; ++i)
	{
		CStr255_AC testString;
		GetString(i, testString);

		if (testString == theString)
		{
			returnVal = i;							// found it! return the index
			break;
		}
	}

	// CPascalStr_AC is not found, should we add it?
	if (returnVal == kNoResource && addString == kAddString)
		returnVal = AppendString(theString);	// added it! return the index

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::AppendString: 
//----------------------------------------------------------------------------------------

short CStringListRsrc_AC::AppendString(const CStr255_AC& theString)
{
	short returnVal = kNoResource;

	short totalStringLength = theString.Length() + kLengthByte;
	Handle aHandle = GetResource('STR#', fStrListID);
	if (aHandle)
	{
		Size itsSize = GetHandleSize(aHandle);
		if (itsSize > 0)
		{
			short numStrings = CountStrings();
			SetPermHandleSize(aHandle, itsSize + totalStringLength);
			(**((IntegerHandle)aHandle)) =++numStrings;// increment the count of strings in the STR# resource

			// needs a failure handler
			long result = Munger(aHandle, itsSize, NULL, totalStringLength, &theString, totalStringLength);
			if (result > 0)
			{
				// mark it as changed
				ChangedResource(aHandle);

				returnVal = numStrings;
			}
		}
	}
	else
	{
		// must create the resource...
		aHandle = NewPermHandle(sizeof(short) + totalStringLength);
		(**((IntegerHandle)aHandle)) = 1;		// set the count of strings in the STR# resource
		long result = Munger(aHandle, sizeof(short), NULL, totalStringLength, &theString, totalStringLength);
		if (result > 0)
		{
			AddResource(aHandle, 'STR#', fStrListID, fStrListRsrcName);
			::ThrowIfResError_AC();						// if fails, it's most likely because the file is read-only

			SetResAttrs(aHandle, resPurgeable);

			// Need the following ChangedResource call because the SetResAttrs call on the previous
			// line cleared the "resChanged" attribute of the resource.
			ChangedResource(aHandle);

			returnVal = 1;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::ClearAll: 
//----------------------------------------------------------------------------------------

void CStringListRsrc_AC::ClearAll()
{
	Handle aHandle = GetResource('STR#', fStrListID);
	if (aHandle)
	{
		SetPermHandleSize(aHandle, sizeof(short));
		(**((IntegerHandle)aHandle)) = 0;		// set the count of strings in the STR# resource

		// mark it as changed
		ChangedResource(aHandle);
	}
}

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::CountStrings: 
//----------------------------------------------------------------------------------------

short CStringListRsrc_AC::CountStrings() const
{
	// return the leading integer in the STR# resource:
	//
	// type 'STR#' {
	//      integer = $$Countof(StringArray);
	//      array StringArray {
	//              pstring;
	//      };
	// };

	Handle aHandle = GetResource('STR#', fStrListID);

	return aHandle ? **((IntegerHandle)aHandle) : kNoResource;
}

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::GetListName: 
//----------------------------------------------------------------------------------------

void CStringListRsrc_AC::GetListName(CStr255_AC& theString)
{
	// Do we have the name in our field?
	if (fStrListRsrcName.Length() > 0)
		theString = fStrListRsrcName;
	else										//	We don't have the name so try and get it from the resource
		{
			// Get the resource's handle
			Handle aHandle = GetResource('STR#', fStrListID);
			if (aHandle)
			{
				short itsRsrcID;
				CStr255_AC itsName;
				ResType itsType;

				// Extract the resource info from the handle
				GetResInfo(aHandle, &itsRsrcID, &itsType, itsName);
				if (itsName.Length() > 0)
				{
					theString = itsName;

					// Store the name in our field
					fStrListRsrcName = theString;
				}

				ReleaseResource(aHandle);
			}
			else								// Sorry we don't have anything so return an empty CPascalStr_AC
				theString.Empty();
		}
}

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::RemoveAt: 
//----------------------------------------------------------------------------------------

void CStringListRsrc_AC::RemoveAt(short index)
{
	if ((index > 0) && (index <= CountStrings()))
	{
		Handle aHandle = GetResource('STR#', fStrListID);
		if (aHandle)
		{
			Size originalSize = GetHandleSize(aHandle);

			// determine the accumulated size
			Size accumulatedSize = sizeof(short);// the accumulated size is the size of the count
			for (short i = 1; i < index; ++i)
				accumulatedSize += (*((CPascalStrPtr_AC)(*aHandle + accumulatedSize))).Length() + kLengthByte;

			// determine the actual CPascalStr_AC size
			Size stringSize = (*((CPascalStrPtr_AC)(*aHandle + accumulatedSize))).Length() + kLengthByte;

			// move the the data in the handle
			BlockMove_AC(*aHandle + accumulatedSize + stringSize, *aHandle + accumulatedSize, originalSize - accumulatedSize - stringSize);

			// trim the handle down to its new size
			SetPermHandleSize(aHandle, originalSize - stringSize);

			// decrement count of strings in STR# resource
			--(**((IntegerHandle)aHandle));

			// mark it as changed
			ChangedResource(aHandle);
		}
#if qDebugMsg
		else
			ProgramReport_AC(false, "CStringListRsrc_AC::RemoveAt can't find STR# = %d.\n", fStrListID);
#endif

	}
#if qDebugMsg
	else
		ProgramReport_AC(false, "CStringListRsrc_AC::RemoveAt there's no CPascalStr_AC at %d to remove!\n", index);
#endif
}

//----------------------------------------------------------------------------------------
// CStringListRsrc_AC::ReplaceAt: 
//----------------------------------------------------------------------------------------

void CStringListRsrc_AC::ReplaceAt(const CStr255_AC& theString, short index)
{
	if ((index > 0) && (index <= CountStrings()))
	{
		Handle aHandle = GetResource('STR#', fStrListID);
		if (aHandle)
		{
			Size newStringSize = theString.Length() + kLengthByte;// the actual CPascalStr_AC size
			Size originalSize = GetHandleSize(aHandle);

			// determine the accumulated size of the valid portion of the resource
			Size accumulatedSize = sizeof(short);// the accumulated size is the size of the count
			for (short i = 1; i < index; ++i)
				accumulatedSize += (*((CPascalStrPtr_AC)(*aHandle + accumulatedSize))).Length() + kLengthByte;

			// determine the actual CPascalStr_AC size
			Size oldStringSize = (*((CPascalStrPtr_AC)(*aHandle + accumulatedSize))).Length() + kLengthByte;

			// determine the new handle size
			Size newHandleSize = originalSize + (newStringSize - oldStringSize);

			// determine whether to grow or shrink the handle?
			if (newStringSize > oldStringSize)
			{
				// grow the handle
				SetPermHandleSize(aHandle, newHandleSize);

				// move the bytes
				BlockMove_AC(*aHandle + accumulatedSize + oldStringSize, *aHandle + accumulatedSize + newStringSize, originalSize - accumulatedSize - oldStringSize);
			}
			else if (oldStringSize > newStringSize)
			{
				// move the bytes
				BlockMove_AC(*aHandle + accumulatedSize + oldStringSize, *aHandle + accumulatedSize + newStringSize, originalSize - accumulatedSize - oldStringSize);

				// shrink the handle
				SetPermHandleSize(aHandle, newHandleSize);
			}

			// assign the CPascalStr_AC
			*((CStr255_AC *)(*aHandle + accumulatedSize)) = theString;

			// mark it as changed
			ChangedResource(aHandle);
		}
#if qDebugMsg
		else
			ProgramReport_AC(false, "CStringListRsrc_AC::ReplaceAt can't find STR# = %d.\n", fStrListID);
#endif

	}
#if qDebugMsg
	else
		ProgramReport_AC(false, "CStringListRsrc_AC::ReplaceAt there's no CPascalStr_AC at %d to replace!\n", index);
#endif
}

//========================================================================================
// CLASS CWMgrIterator
//========================================================================================
#undef Inherited
#define Inherited CIterator_AC

//----------------------------------------------------------------------------------------
// CWMgrIterator::CWMgrIterator: 
//----------------------------------------------------------------------------------------

CWMgrIterator::CWMgrIterator(bool itsForward) :
	fIterateForward(itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CWMgrIterator::First: 
//----------------------------------------------------------------------------------------

WindowRef CWMgrIterator::First()
{
	return fCurrent = fIterateForward ? FirstWindow() : LastWindow();
}

//----------------------------------------------------------------------------------------
// CWMgrIterator::Next: 
//----------------------------------------------------------------------------------------

WindowRef CWMgrIterator::Next()
{
	if (fCurrent)
		fCurrent = fIterateForward ? NextWindow(fCurrent) : PreviousWindow(fCurrent);

	return Current();
}

//----------------------------------------------------------------------------------------
// CWMgrIterator::FirstWindow: 
//----------------------------------------------------------------------------------------

WindowRef CWMgrIterator::FirstWindow()
{
	WindowRef currentWindow = (WindowRef)LMGetWindowList();
	if (currentWindow && (currentWindow == gWorkPort))
		currentWindow = NextWindow(currentWindow);

	return currentWindow;
}

//----------------------------------------------------------------------------------------
// CWMgrIterator::NextWindow: 
//----------------------------------------------------------------------------------------

WindowRef CWMgrIterator::NextWindow(WindowRef aWindow)
{
	WindowRef currentWindow = GetNextWindow(aWindow);

	while (currentWindow && (currentWindow == gWorkPort))
		// ignore the work window
		currentWindow = GetNextWindow(currentWindow);

	return currentWindow;
}

//----------------------------------------------------------------------------------------
// CWMgrIterator::PreviousWindow: 
//----------------------------------------------------------------------------------------

WindowRef CWMgrIterator::PreviousWindow(WindowRef aWindow)
{
	WindowRef currentWindow = FirstWindow();
	while (currentWindow && (NextWindow(currentWindow) != aWindow))
		currentWindow = NextWindow(currentWindow);

	return currentWindow;
}

//----------------------------------------------------------------------------------------
// CWMgrIterator::LastWindow: 
//----------------------------------------------------------------------------------------

WindowRef CWMgrIterator::LastWindow()
{
	WindowRef currentWindow = FirstWindow();
	while (currentWindow && NextWindow(currentWindow))
		currentWindow = NextWindow(currentWindow);

	return currentWindow;
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// CanWriteLn: 
//----------------------------------------------------------------------------------------

#if qDebug
bool CanWriteLn()
{
	return DebugCanWriteLn();
}
#endif

//----------------------------------------------------------------------------------------
// CanReadLn: 
//----------------------------------------------------------------------------------------

#if qDebug
bool CanReadLn()
{
	return DebugCanReadLn();
}
#endif

//----------------------------------------------------------------------------------------
// CenterRectOnScreen: 
//----------------------------------------------------------------------------------------
// This must always be in a resident segment as aRect may be within a handle

void CenterRectOnScreen(CRect_AC& aRect,
						bool horizontally,
						bool vertically,
						bool forDialog)
{
	short newSize = 0;
	CPoint_AC rectSize(aRect.GetSize());

	CPoint_AC screenSize(((CRect_AC &)qd.screenBits.bounds).GetSize());
	screenSize.v -= GetMBarHeight();

	// Calculate screen size minus menu bar 
	if (horizontally)
		aRect.left = (screenSize.h - rectSize.h) / 2;
	if (vertically)
		if (forDialog)
		{
			newSize = (screenSize.v - rectSize.v) / 5;
			aRect.top = Max_AC(newSize, static_cast<short>(10)) + GetMBarHeight();
		}
		else
			aRect.top = (screenSize.v - rectSize.v) / 2;

	aRect[botRight] = aRect[topLeft] + rectSize;
}

//----------------------------------------------------------------------------------------
// ConcatNumber: 
//----------------------------------------------------------------------------------------

void ConcatNumber(const CStr255_AC& aString,
				  long aNumber,
				  CStr255_AC& theResult)
{
	NumToString(aNumber, theResult);
	theResult = aString + theResult;
}

//----------------------------------------------------------------------------------------
// PullApplicationToFront: 
//----------------------------------------------------------------------------------------
// Must be in main segment as it is called in early initialization && in MacAppAlert

void PullApplicationToFront()
{
	EventRecord theEvent;

	// The "Programmer's guide to MultiFinder says make an event call several times. I
	// guess 3 calls counts as several. Also, it says call GetNextEvent but we don't want
	// to lose events on the floor so we use EventAvail since it seems to work OK
	for (short i = 1; i <= 3; ++i)
		EventAvail(everyEvent, &theEvent);
}

//----------------------------------------------------------------------------------------
// MAInteractWithUser: 
//----------------------------------------------------------------------------------------

OSErr MAInteractWithUser(long timeOutInTicks,
						 NMRecPtr nmReqPtr,
						 AEIdleUPP idleProc)
{
	return gDispatcher ? gDispatcher->InteractWithUser(timeOutInTicks, nmReqPtr, idleProc) : AEInteractWithUser(kNoTimeOut, nmReqPtr, idleProc);
}

//----------------------------------------------------------------------------------------
// MAInteractWithUser: 
//----------------------------------------------------------------------------------------

OSErr MAInteractWithUser()
{
	return MAInteractWithUser(kMacAppTimeout, gNotificationPtr, TAppleEvent::fgIdleProc);
}

//----------------------------------------------------------------------------------------
// MAInteractWithUserNoIdleProc: 
//----------------------------------------------------------------------------------------

OSErr MAInteractWithUserNoIdleProc()
{
	return MAInteractWithUser(kMacAppTimeout, gNotificationPtr);
}

#if !qNeedsSystem7_5
//----------------------------------------------------------------------------------------
// MACharacterByteType: 
//----------------------------------------------------------------------------------------

short MACharacterByteType(Ptr textBuf,
						  short textOffset,
						  ScriptCode script)
{
	// CharacterByteType only avail in sys 7.1+
	short byteType;
	if (GetSystemVersion_AC() >= 0x0710)
		byteType = CharacterByteType(textBuf, textOffset, script);// textOffset is zero-based 
	else
	{
		short oldScript = SetKeyScript(script);
		byteType = CharByte(textBuf, textOffset);// textOffset is zero-based 
		SetKeyScript(oldScript);
	}

#if qDebug
	::Assertion_AC(byteType != smMiddleByte, "byteType != smMiddleByte. We should never receive smMiddleByte");
#endif

	return byteType;
}

//----------------------------------------------------------------------------------------
// MACharacterByte: 
//----------------------------------------------------------------------------------------

short MACharacterType(Ptr textBuf,
					  short textOffset,
					  ScriptCode script)
{
	// CharacterType only avail in sys 7.1+
	short byteType;
	if (GetSystemVersion_AC() >= 0x0710)
		byteType = CharacterType(textBuf, textOffset, script);// textOffset is zero-based 
	else
	{
		short oldScript = SetKeyScript(script);
		byteType = CharType(textBuf, textOffset);// textOffset is zero-based 
		SetKeyScript(oldScript);
	}

#if qDebug
	::Assertion_AC(byteType != smMiddleByte, "byteType != smMiddleByte. We should never receive smMiddleByte");
#endif

	return byteType;
}
#endif

//----------------------------------------------------------------------------------------
// CompareMultiByteChars: 
//----------------------------------------------------------------------------------------

bool CompareMultiByteChars(const CStr31_AC& first,
							  const CStr31_AC& second,
							  bool caseSens)
{
	bool areEqual = false;

	if (caseSens)
		areEqual = first == second;
	else
	{
		int textOffset = 0;
		bool done = false;

		do
		{
			++textOffset;
			char c = first[textOffset];

			short byteType = MACharacterByteType((Ptr) & c, 0, smCurrentScript);

			areEqual = first[textOffset] == second[textOffset];
			switch (byteType)
			{
					// special case single byte characters to allow lower case characters to map to
					// upper case characters
				case smSingleByte:
					areEqual = (LowerChar(first[1]) == LowerChar(second[1]));
					done = true;
					break;

				case smFirstByte:
					done = !areEqual;			// we're done if they don't match 
					break;

				case smLastByte:
					done = true;
					break;
			}
		} while (!done);
	}

	return areEqual;
}

//----------------------------------------------------------------------------------------
// DefaultSize: 
//----------------------------------------------------------------------------------------

void DefaultSize(short& theSize)
{
	if (theSize == GetDefFontSize())
		theSize = 0;
}

//----------------------------------------------------------------------------------------
// DisposeIfPicHandle: 
//----------------------------------------------------------------------------------------

PicHandle DisposeIfPicHandle(PicHandle aPicHandle)
{
	if (aPicHandle)
	{
#if !qDebug
		KillPicture(aPicHandle);
#else
		char msg[128];

		if (IsHandle_AC((Handle)aPicHandle))		// Test handlehood 
		{
			// Set the handle contents to a real nice value for any dangling pointerciples 
			BlockSet_AC(*((Handle)aPicHandle), GetHandleSize((Handle)aPicHandle), kDisposedInitVal_AC);
			KillPicture(aPicHandle);
		}
		else
		{
			VerboseIsHandle((Handle)aPicHandle);// Get the diagnosis printed 
			sprintf(msg, "Trying to dispose an invalid PicHandle)\nBad PicHandle: %p\n", aPicHandle);
			::ProgramBreak_AC(msg);
		}
#endif

	}

	return NULL;
}

//----------------------------------------------------------------------------------------
// DisposeIfSectionHandle: 
//----------------------------------------------------------------------------------------

SectionHandle DisposeIfSectionHandle(SectionHandle aSectionHandle)
{
	if (aSectionHandle)
	{
#if !qDebug
		::DisposeIfHandle_AC((Handle)(*aSectionHandle)->alias);// dispose of the alias
		::DisposeIfHandle_AC((Handle)aSectionHandle);// dispose of the SectionHandle
#else
		char msg[128];

		// Test handlehood 
		if (IsHandle_AC((Handle)aSectionHandle))
		{
			::DisposeIfHandle_AC((Handle)(*aSectionHandle)->alias);// dispose of the alias

			// Set the handle contents to a real nice value for any dangling pointerciples 
			BlockSet_AC(*((Handle)aSectionHandle), GetHandleSize((Handle)aSectionHandle), kDisposedInitVal_AC);
			::DisposeIfHandle_AC((Handle)aSectionHandle);// dispose of the SectionHandle
		}
		else
		{
			VerboseIsHandle((Handle)aSectionHandle);// Get the diagnosis printed 
			sprintf(msg, "Trying to dispose an invalid SectionHandle.\nBad SectionHandle: %p\n", aSectionHandle);
			::ProgramBreak_AC(msg);
		}
#endif

	}

	return NULL;
}

//----------------------------------------------------------------------------------------
// FindWindowBefore: returns the window just before a given window. Returns NULL if the
// given window is frontmost or not found.
//----------------------------------------------------------------------------------------

WindowRef FindWindowBefore(WindowRef theWindow)
{
	WindowRef returnVal = NULL;

	for (CWMgrIterator iter; iter; ++iter)
		if (GetNextWindow(iter) == theWindow)
		{
			returnVal = iter;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// GetActualJustification: 
//----------------------------------------------------------------------------------------

short GetActualJustification(short justification)
{
	return justification == teFlushDefault ? GetSysDirection() : justification;
}

//----------------------------------------------------------------------------------------
// GetDeskTopRegion: 
//----------------------------------------------------------------------------------------

void GetDeskTopRegion(RgnHandle deskTopRgn)
{
	if (qNeedsColorQD || HasColorQD_AC())			// deskTopRgn = main screen CRect_AC 
		RectRgn(deskTopRgn, &((*GetMainDevice())->gdRect));
	else
		RectRgn(deskTopRgn, &(qd.screenBits.bounds));
	UnionRgn(GetGrayRgn(), deskTopRgn, deskTopRgn);// deskTopRgn = grayRgn + deskTopRgn 
}

//----------------------------------------------------------------------------------------
// ShowGlobalRegion: 
//----------------------------------------------------------------------------------------

void ShowGlobalRegion(RgnHandle globalRegion)
{
	if (globalRegion)
	{
		GrafPtr savedPort;
		CGrafPort desktopPort;

		// Cursor region is in global coords. Need to create desktop port 
		GetPort(&savedPort);
		if (qNeedsColorQD || HasColorQD_AC())
			OpenCPort(&desktopPort);
		else
			OpenPort((GrafPtr) & desktopPort);

		CopyRgn(GetGrayRgn(), TWindow::GetVisRegion((GrafPtr) & desktopPort));
		desktopPort.portRect = (*TWindow::GetVisRegion((GrafPtr) & desktopPort))->rgnBBox;

		PenNormal();
		PenMode(patXor);

		PaintRgn(globalRegion);
		unsigned long endCount = TickCount() + 30;
		EventRecord eventRecord;
		CPoint_AC globalMouse;

		GetMouse(&globalMouse);
		LocalToGlobal(&globalMouse);
		// delay up to thirty ticks
		while (TickCount() < endCount && !EventAvail(mouseDown | keyDown, &eventRecord) && CRegion_AC(globalRegion).Contains(globalMouse))
		{
			GetMouse(&globalMouse);
			LocalToGlobal(&globalMouse);
		}

		PaintRgn(globalRegion);

		if (qNeedsColorQD || HasColorQD_AC())
			CloseCPort(&desktopPort);
		else
			ClosePort((GrafPtr) & desktopPort);
		SetPort(savedPort);
	}
}

//----------------------------------------------------------------------------------------
// ShowViewRegion: 
//----------------------------------------------------------------------------------------

void ShowViewRegion(TView* theView,
					RgnHandle viewRegion)
{
	if (viewRegion && theView)
	{
		CTempRegion_AC tempRegion;

		CopyRgn(viewRegion, tempRegion);
		theView->LocalToGlobal(tempRegion);
		ShowGlobalRegion(tempRegion);
	}
}

//----------------------------------------------------------------------------------------
// GetFontNum: 
//----------------------------------------------------------------------------------------

short GetFontNum(const CStr255_AC& fontName)
{
	short fontNum;
	CStr255_AC localFontName(fontName);

	UprStr255(localFontName);
	if (localFontName == CStr255_AC(kSysFontName))
		fontNum = GetSysFont();
	else if (localFontName == CStr255_AC(kApplFontName))
		fontNum = GetAppFont();
	else
		GetFNum(fontName, &fontNum);

	return fontNum;
}

//----------------------------------------------------------------------------------------
// GetIfBkColor: 
//----------------------------------------------------------------------------------------

CRGBColor_AC GetIfBkColor()
{
	// Since we require system 7 we can always use the GetFore/BackColor calls
	// because QD will do the color mapping for us

	CRGBColor_AC aColor;

	GetBackColor(aColor);
	
	return aColor;
}

//----------------------------------------------------------------------------------------
// GetIfColor: 
//----------------------------------------------------------------------------------------

CRGBColor_AC GetIfColor()
{
	// Since we require system 7 we can always use the GetFore/BackColor calls
	// because QD will do the color mapping for us

	CRGBColor_AC aColor;

	GetForeColor(aColor);
	
	return aColor;
}

//----------------------------------------------------------------------------------------
// MAGetFontInfo: 
//----------------------------------------------------------------------------------------

short MAGetFontInfo(FontInfo& theFontInfo)
// MAGetFontInfo may be used in place of GetFontInfo since it also returns the font height. 
{
	GetFontInfo(&theFontInfo);					// get the current port's font info 

	return theFontInfo.ascent + theFontInfo.descent + theFontInfo.leading;// returns the font height 
}

//----------------------------------------------------------------------------------------
// GetPortFontInfo: 
//----------------------------------------------------------------------------------------

void GetPortFontInfo(short fontNum,
					 CStr255_AC& fontName,
					 short& fontSize)
{
	if ((fontNum == systemFont) || (fontNum == GetSysFont()))
	{
		fontName = kSysFontName;
		DefaultSize(fontSize);
	}
	else if ((fontNum == applFont) || (fontNum == GetAppFont()))
	{
		fontName = kApplFontName;
		DefaultSize(fontSize);
	}
	else
		GetFontName(fontNum, fontName);
}

//----------------------------------------------------------------------------------------
// GetRandom: 
//----------------------------------------------------------------------------------------

long GetRandom(long rangeStart,
			   long rangeEnd)
{
	unsigned short rndNum = Random();

	return rangeStart < rangeEnd ? rangeStart + rndNum % (rangeEnd - rangeStart + 1) : rangeStart;
}

//----------------------------------------------------------------------------------------
// IsColorPort: Returns true if the GrafPort is a Color QD GrafPort test is:
// IsColorPort = CGrafPtr(port)->portversion == 0xC000
//----------------------------------------------------------------------------------------

bool IsColorPort(GrafPtr port)
{
	return port ? ((((CGrafPtr)port)->portVersion & (0xC000))) == (0xC000) : false;
}

//----------------------------------------------------------------------------------------
// IsThisKeyDown: 
//----------------------------------------------------------------------------------------

bool IsThisKeyDown(const short theKey)
{
	KeyMap map;
	GetKeys(map);

	return ((Byte *) & map)[theKey >> 3] & (1 << (theKey & 0x07)) ? true : false;
}

//----------------------------------------------------------------------------------------
// IsCapsLockKeyDown: 
//----------------------------------------------------------------------------------------

bool IsCapsLockKeyDown()
{
	static const short kCapsLockKey = 57;

	return IsThisKeyDown(kCapsLockKey);
}

//----------------------------------------------------------------------------------------
// IsCommandKeyDown: 
//----------------------------------------------------------------------------------------

bool IsCommandKeyDown()
{
	static const short kCommandKey = 55;

	return IsThisKeyDown(kCommandKey);
}

//----------------------------------------------------------------------------------------
// IsControlKeyDown: 
//----------------------------------------------------------------------------------------

bool IsControlKeyDown()
{
	static const short kCtlKey = 0x3B;

	return IsThisKeyDown(kCtlKey);
}

//----------------------------------------------------------------------------------------
// IsOptionKeyDown: 
//----------------------------------------------------------------------------------------

bool IsOptionKeyDown()
{
	static const short kOptionKey = 58;

	return IsThisKeyDown(kOptionKey);
}

//----------------------------------------------------------------------------------------
// IsShiftKeyDown: 
//----------------------------------------------------------------------------------------

bool IsShiftKeyDown()
{
	static const short kShiftKey = 56;

	return IsThisKeyDown(kShiftKey);
}

//----------------------------------------------------------------------------------------
// LongerSide: 
//----------------------------------------------------------------------------------------

VHSelect LongerSide(const CRect_AC& r)
{
	return r.GetLength(vSel) >= r.GetLength(hSel) ? vSel : hSel;
}

//----------------------------------------------------------------------------------------
// LongerVSide: 
//----------------------------------------------------------------------------------------

VHSelect LongerSide(const CViewRect& r)
{
	return r.GetLength(vSel) >= r.GetLength(hSel) ? vSel : hSel;
}

//----------------------------------------------------------------------------------------
// LIntToHex: 
//----------------------------------------------------------------------------------------

void LIntToHex(long decNumber,
			   CStr31_AC& hexNumber,
			   short noOfDigits)
{
	short localNoOfDigits = Min_AC(noOfDigits, static_cast<short>(8));
	long localDecNumber = decNumber;
	hexNumber.Length() = static_cast<unsigned char>(localNoOfDigits);
	for (int i = localNoOfDigits; i >= 1; --i, localDecNumber = localDecNumber >> 4)
		hexNumber[i] = kHexDigits[(localDecNumber & 15)];
}

//----------------------------------------------------------------------------------------
// NumberToHex: 
//----------------------------------------------------------------------------------------

void NumberToHex(long theNumber,
				 CStr255_AC& hexString,
				 short hexDigits)
{
	CStr31_AC tempString;

	LIntToHex(theNumber, tempString, hexDigits);
	hexString = "$" + tempString;
}

//----------------------------------------------------------------------------------------
// PointerToHex: 
//----------------------------------------------------------------------------------------

void PointerToHex(long theNumber,
				  CStr31_AC& hexString,
				  short hexDigits)
{
	CStr31_AC tempString;

	if (!theNumber)
		hexString = "nil";
	else
	{
		LIntToHex(theNumber, tempString, hexDigits);
		hexString = "$" + tempString;
	}
}

//----------------------------------------------------------------------------------------
// NumBlocks: 
//----------------------------------------------------------------------------------------

long NumBlocks(long numBytes,
			   long blkSize)
{
	return (numBytes + blkSize - 1) / blkSize;
}

//----------------------------------------------------------------------------------------
// MADrawString: 
//----------------------------------------------------------------------------------------

void MADrawString(const CStr255_AC& s,
				  const CRect_AC& box,
				  short justification,
				  bool preferOutline)
{
	CWhileOutlinePreferred_AC setOP(preferOutline);

	short widthOfString = ::StringWidth(s);
	CRect_AC localBox(box);
	short boxWidth = localBox.GetLength(hSel);

	if (widthOfString < boxWidth)
		switch (GetActualJustification(justification))
		{
			case teCenter:
				localBox.left += (boxWidth - widthOfString) / 2;
				break;

			case teFlushRight:
				localBox.left += boxWidth - widthOfString;
				break;
		}

	FontInfo theFontInfo;
	::GetFontInfo(&theFontInfo);
	::MoveTo(localBox.left, localBox.top + theFontInfo.ascent);

	if (widthOfString > boxWidth)	// string is too long for box
	{
		CTempRegion_AC tempRgn;
		::GetClip(tempRgn);
		::ClipRect(box);
		::DrawString(s);
		::SetClip(tempRgn);
	}
	else
		::DrawString(s);
}

//----------------------------------------------------------------------------------------
// PinOnRect: 
//----------------------------------------------------------------------------------------

long PinOnRect(const CRect_AC& theRect,
			   CPoint_AC thePt)
{
	CPoint_AC localPt = thePt;
	if (localPt.h < theRect.left)
		localPt.h = theRect.left;
	if (localPt.h > theRect.right)
		localPt.h = theRect.right;
	if (localPt.v < theRect.top)
		localPt.v = theRect.top;
	if (localPt.v > theRect.bottom)
		localPt.v = theRect.bottom;

	return *reinterpret_cast<long*>(&localPt);
}

//----------------------------------------------------------------------------------------
// PinOnVRect: 
//----------------------------------------------------------------------------------------

void PinOnVRect(const CViewRect& theRect,
				const CViewPoint& thePt,
				CViewPoint& thePin)
{
	long hCoord = MinMax_AC(theRect.left, thePt.h, theRect.right);
	long vCoord = MinMax_AC(theRect.top, thePt.v, theRect.bottom);
	thePin.Set(hCoord, vCoord);
}

//----------------------------------------------------------------------------------------
// RoundUp: 
//----------------------------------------------------------------------------------------

long RoundUp(long aNumber,
			 short aModulus)
{
	return ((aNumber + aModulus - 1) / aModulus) * aModulus;
}

//----------------------------------------------------------------------------------------
// SetKeyScript: 
//----------------------------------------------------------------------------------------

short SetKeyScript(short newKeyScript)
{
	short currentKeyScript = static_cast<short>(GetScriptManagerVariable(smKeyScript));
	if (currentKeyScript != newKeyScript)		// Don't unnecessarily annoy the script mgr 
		::KeyScript(newKeyScript);

	return currentKeyScript;
}

//----------------------------------------------------------------------------------------
// SetIfBkColor: 
//----------------------------------------------------------------------------------------

void SetIfBkColor(const CRGBColor_AC& aColor)
{
	// Since we require system 7 we can always use the RGBFore/BackColor calls
	// because QD will do the color mapping for us

	// Try to avoid calling RGBFore/BackColor unnecessarally
	if (!IsColorPort(qd.thePort) || (((CRGBColor_AC &) ((CGrafPtr) qd.thePort)->rgbBkColor) != aColor))
		::RGBBackColor(aColor);
}

//----------------------------------------------------------------------------------------
// SetIfColor: 
//----------------------------------------------------------------------------------------

void SetIfColor(const CRGBColor_AC& aColor)
{
	// Since we require system 7 we can always use the RGBFore/BackColor calls
	// because QD will do the color mapping for us

	// Try to avoid calling RGBFore/BackColor unnecessarally
	if (!IsColorPort(qd.thePort) || (((CRGBColor_AC &) ((CGrafPtr) qd.thePort)->rgbFgColor) != aColor))
		::RGBForeColor(aColor);
}

//----------------------------------------------------------------------------------------
// GetPortTextStyle: 
//----------------------------------------------------------------------------------------

CTextStyle_AC GetPortTextStyle()
{
	return CTextStyle_AC(qd.thePort->txFont, qd.thePort->txFace, qd.thePort->txSize, GetIfColor());
}

//----------------------------------------------------------------------------------------
// MAGetTextStyle: 
//----------------------------------------------------------------------------------------

CTextStyle_AC MAGetTextStyle(ResNumber rsrcID)
{
	TemplateTempHandleLock<MATextStyleHandle> theTextStyleHdl(reinterpret_cast<MATextStyleHandle>(GetResource('TxSt', rsrcID)));
	::ThrowIfNULLResource_AC(theTextStyleHdl);

	MATextStyle & resourceStyle = **theTextStyleHdl;
	
	return CTextStyle_AC(GetFontNum(resourceStyle.tsFont), resourceStyle.tsFace, resourceStyle.tsSize, resourceStyle.tsColor);
}

//----------------------------------------------------------------------------------------
// SetPortTextStyle: 
//----------------------------------------------------------------------------------------

void SetPortTextStyle(const CTextStyle_AC& theTextStyle)
{
	theTextStyle.SetPortTextStyle();
}

//----------------------------------------------------------------------------------------
// UseSelectionColor: 
//----------------------------------------------------------------------------------------

void UseSelectionColor()
{
	unsigned char hiliteModeValue = LMGetHiliteMode();
	BitClr((Ptr) & hiliteModeValue, pHiliteBit);
	LMSetHiliteMode(hiliteModeValue);
}

//----------------------------------------------------------------------------------------
// CheckAdornment: 
//----------------------------------------------------------------------------------------

void CheckAdornment(CntlAdornment p,
					CntlAdornment alias,
					CStr255_AC theString,
					char* name)
{
	// "set1 <= set2" means set1 is wholly contained in set2 
	if (p <= alias)
		if (theString == "[")
			theString += name;
		else
			theString += CStr2_AC(",") + name;
}

//----------------------------------------------------------------------------------------
// CheckStyleItem: 
//----------------------------------------------------------------------------------------
void CheckStyleItem(unsigned char s,
					unsigned char alias,
					CStr255_AC theString,
					char* name)
{
	if (s & alias)
		if (theString == "[")
			theString += name;
		else
			theString += CStr2_AC(",") + name;
}

//----------------------------------------------------------------------------------------
// RotatePat: 
//----------------------------------------------------------------------------------------

void RotatePat(Pattern& aPattern)
{
	unsigned char temp = aPattern.pat[0];
	for (short index = 0; index < (sizeof(Pattern) - 1); ++index)
		aPattern.pat[index] = aPattern.pat[index + 1];
	aPattern.pat[sizeof(Pattern) - 1] = temp;
}

//----------------------------------------------------------------------------------------
// XorPat: 
//----------------------------------------------------------------------------------------

void XorPat(const Pattern& patA,
			const Pattern& patB,
			Pattern& toPat)
{
	for (short index = 0; index < sizeof(Pattern); ++index)
		toPat.pat[index] = patA.pat[index] ^ patB.pat[index];// xor
}

//----------------------------------------------------------------------------------------
// StreamToHandle: 
//----------------------------------------------------------------------------------------

Handle StreamToHandle(CRandomAccessStream_AC* theStream)
{
	Size dataSize = theStream->GetSize() - theStream->GetPosition();

	CTempHandle_AC dataHandle(NewPermHandle(dataSize));	// fails if error
	CTempHandleLock_AC tempHandleLock(dataHandle);

	theStream->ReadBytes(*dataHandle, dataSize);

	return dataHandle.release();
}

//----------------------------------------------------------------------------------------
// End of UMacAppUtilities.cp



