//----------------------------------------------------------------------------------------
// UOSAScript.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UOSASCRIPT__
#include "UOSAScript.h"
#endif

// MacApp

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

//	#ifndef __UMACAPPUTILITIES__
//	#include "UMacAppUtilities.h"
//	#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

// Toolbox

#ifndef __AEPACKOBJECT__
#include <AEPackObject.h>
#endif

#ifndef __ASDEBUGGING__
#include <ASDebugging.h>
#endif

#ifndef __ASREGISTRY__
#include <ASRegistry.h>
#endif

#ifndef __OSAGeneric__
#include <OSAGeneric.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

//----------------------------------------------------------------------------------------
// Static Allocations
//----------------------------------------------------------------------------------------

ComponentInstance COSAScript::fgGenericScriptingComponent;

COSAScript COSAScript::fgGlobalScript;
COSAScript COSAScript::fgSharedScript;
COSAScript COSAScript::fgEmptyScript;

//========================================================================================
// CLASS COSAScript
//========================================================================================

//----------------------------------------------------------------------------------------
// COSAScript::GetDefaultComponent: 
//----------------------------------------------------------------------------------------


ComponentInstance COSAScript::GetDefaultComponent()
{
	if (HasAppleScript_AC() && !fgGenericScriptingComponent)
		fgGenericScriptingComponent = OpenDefaultComponent(kOSAComponentType,
														   kOSAGenericScriptingComponentSubtype);

	return fgGenericScriptingComponent;
}

//----------------------------------------------------------------------------------------
// COSAScript::GetEmptyScript: 
//----------------------------------------------------------------------------------------


COSAScript COSAScript::GetEmptyScript()
{
	if (HasAppleScript_AC() && (fgEmptyScript == kOSANullScript))
	{
		ComponentInstance defaultComponent = GetDefaultComponent();
		Handle scriptRes = MAGet1Resource('scpt', 128);
		::ThrowIfNULLResource_AC(scriptRes);
		DetachResource(scriptRes);
		CTempDesc emptyScriptDesc('scpt', scriptRes);
		fgEmptyScript.SetScript(defaultComponent, emptyScriptDesc,
								kOSAModeCompileIntoContext, kOSAModeNull);
	}
	return fgEmptyScript;
}

//----------------------------------------------------------------------------------------
// COSAScript::COSAScript
//----------------------------------------------------------------------------------------


COSAScript::COSAScript()
{
	fScriptID = kOSANullScript;
}

//----------------------------------------------------------------------------------------
// COSAScript::COSAScript
//----------------------------------------------------------------------------------------


COSAScript::COSAScript(OSAID theScript)
{
	fScriptID = theScript;
}

//----------------------------------------------------------------------------------------
// COSAScript::IsEmpty
//----------------------------------------------------------------------------------------


bool COSAScript::IsEmpty()
{
	return (fScriptID == kOSANullScript);
}

//----------------------------------------------------------------------------------------
// COSAScript::DisposeScript
//----------------------------------------------------------------------------------------


OSAError COSAScript::DisposeScript(ComponentInstance theComponent)
{
	OSAError err = noErr;
	if (!IsEmpty())
	{
		CTempUseResFile_AC tempUseResFile(CurResFile_AC());  // preserve the current resource file
		err = OSADispose(theComponent, fScriptID);
		fScriptID = kOSANullScript;
	}
	return err;
}

//----------------------------------------------------------------------------------------
// COSAScript::Execute
//----------------------------------------------------------------------------------------


OSAError COSAScript::Execute(ComponentInstance theComponent,
						  OSAID theContext,
						  long executeFlags,
						  OSAID& theResult)
{
	OSAError err = noErr;

	CTempUseResFile_AC tempUseResFile(CurResFile_AC());  // preserve the current resource file
	err = OSAExecute(theComponent, fScriptID, theContext, executeFlags, &theResult);

	return err;
}

//----------------------------------------------------------------------------------------
// COSAScript::GetScript
//----------------------------------------------------------------------------------------


OSAError COSAScript::GetScript(ComponentInstance theComponent,
							DescType whichType,
							CAEDesc& scriptDesc,
							long storeFlags)
{
	OSAError err = noErr;
	
	CTempUseResFile_AC tempUseResFile(CurResFile_AC());  // preserve the current resource file
	// Check to see if they want a text version of the script
	if ((whichType == typeIntlText) || (whichType == typeChar))
		err = OSAGetSource(theComponent, fScriptID, whichType, scriptDesc);
	else
		err = OSAStore(theComponent, fScriptID, whichType, storeFlags, scriptDesc);
	
	return err;
}

//----------------------------------------------------------------------------------------
// COSAScript::SetScript
//----------------------------------------------------------------------------------------


OSAError COSAScript::SetScript(ComponentInstance theComponent,
							   const CAEDesc& scriptDesc,
							   long compileFlags,
							   long loadFlags)
{
	OSAError err = noErr;
	CTempUseResFile_AC tempUseResFile(CurResFile_AC()); // setting the script may toss the resource chain

	try
	{
		OSAID newScriptID = kOSANullScript;
		DescType scriptType = scriptDesc.GetDescriptorType();
		if (scriptType != typeNull)
		{
			if ((scriptType == typeIntlText) || (scriptType == typeChar))
			{
				if (scriptDesc.GetDataSize()) // lets you empty out an objects script with ""
					::ThrowIfError_AC(OSACompile(theComponent, scriptDesc, compileFlags, &newScriptID));
			}
			else if ((scriptType == typeFSS) || (scriptType == typeAlias))
			{
				TFile scriptFile(kAllQuestionsType_AC, kAllQuestionsType_AC, kUsesDataFork, preferResourceFork, !kDataOpen, !kRsrcOpen);

				if (scriptType == typeAlias)
					scriptFile.SpecifyWithAlias((AliasHandle)scriptDesc.GetDataHandle());
				if (scriptType == typeFSS)
				{
					CFSSpec_AC theSpec = **(CFSSpec_AC**)scriptDesc.GetDataHandle();
					scriptFile.Specify(theSpec);
				}
				OSType fileType = kAllQuestionsType_AC;
				::ThrowIfOSErr_AC(scriptFile.GetFileType(fileType));
				if (fileType == kTextType_AC)
				{
					scriptFile.SetUsesRsrcFork(noResourceFork);
					::ThrowIfOSErr_AC(scriptFile.OpenFile());
					long textSize = 0;
					::ThrowIfOSErr_AC(scriptFile.GetDataLength(textSize));
					Handle textHdl = NewHandle(textSize);
					::ThrowIfNULL_AC(textHdl);
					{
						CTempHandleLock_AC tempHandleLock(textHdl);
						::ThrowIfOSErr_AC(scriptFile.ReadData(*textHdl, textSize));
					}
					::ThrowIfOSErr_AC(scriptFile.CloseFile());
					CTempDesc textDesc(typeChar, textHdl);
					::ThrowIfError_AC(OSACompile(theComponent, textDesc, compileFlags, &newScriptID));
				}
				else if (fileType == kOSAFileType)
				{
					scriptFile.SetUsesDataFork(false);
					scriptFile.SetUsesRsrcFork(needResourceFork);
					::ThrowIfOSErr_AC(scriptFile.OpenFile());
					Handle scriptHdl = Get1Resource(kOSAScriptResourceType, 128);
					::ThrowIfNULLResource_AC(scriptHdl);
					CAEDesc scriptRsrcDesc(typeOSAGenericStorage, scriptHdl);
					::ThrowIfError_AC(OSALoad(theComponent, scriptRsrcDesc, loadFlags, &newScriptID));
					::ThrowIfOSErr_AC(scriptFile.CloseFile());
				}
			}
			else
				::ThrowIfError_AC(OSALoad(theComponent, scriptDesc, loadFlags, &newScriptID));
		}
		if (!IsEmpty())
			::ThrowIfOSErr_AC(DisposeScript(theComponent));
		fScriptID = newScriptID;
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		err = theException.GetError();
	}
	

	return err;
}

//----------------------------------------------------------------------------------------
// COSAScript::SetParentScript
//----------------------------------------------------------------------------------------


OSAError COSAScript::SetParentScript(ComponentInstance genericComponent,
									 OSAID itsParent)
{
	OSAError err = noErr;
	CTempUseResFile_AC tempUseResFile(CurResFile_AC()); // preserve the current resource file

	try
	{
		if (!IsEmpty()) 
		{
			ComponentInstance exactComponent = NULL;
			CTempDesc parentProp;
			DescType parentCode = pASParent;
			::ThrowIfOSErr_AC(AECreateDesc(typeProperty, (Ptr)&parentCode, sizeof(parentCode), parentProp));
			OSAID myScriptID = fScriptID;
			OSAID parentScriptID = itsParent;
			if (itsParent == kOSANullScript)
				parentScriptID = GetEmptyScript();
			::ThrowIfError_AC(OSAGenericToRealID(genericComponent, &myScriptID, &exactComponent));
			::ThrowIfError_AC(OSAGenericToRealID(genericComponent, &parentScriptID, &exactComponent));
			::ThrowIfError_AC(OSASetProperty(exactComponent, kOSAModeNull, myScriptID, parentProp, parentScriptID));
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		err = theException.GetError();
	}
	

	return err;
}

//----------------------------------------------------------------------------------------
// COSAScript::GetParentScript
//----------------------------------------------------------------------------------------


OSAError COSAScript::GetParentScript(ComponentInstance genericComponent,
									 OSAID& itsParent)
{
	OSAError err = noErr;
	CTempUseResFile_AC tempUseResFile(CurResFile_AC()); // preserve the current resource file

	try
	{
		if (!IsEmpty())
		{
			CTempDesc parentProp;
			ComponentInstance exactComponent = NULL;
			DescType parentCode = pASParent;
			::ThrowIfOSErr_AC(AECreateDesc(typeProperty, (Ptr)&parentCode, sizeof(parentCode), parentProp));
			OSAID myScriptID = fScriptID;
			OSAID parentScriptID = fScriptID;
			
			if (genericComponent != exactComponent)
				::ThrowIfError_AC(OSAGenericToRealID(genericComponent, &myScriptID, &exactComponent));

			::ThrowIfError_AC(OSAGetProperty(exactComponent, kOSAModeNull, myScriptID, parentProp, &parentScriptID));
			if ((err == noErr) && (genericComponent != exactComponent))
				::ThrowIfError_AC(OSARealToGenericID(genericComponent, &parentScriptID, exactComponent));

			itsParent = parentScriptID;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		err = theException.GetError();
	}
	

	return err;
}

//----------------------------------------------------------------------------------------
// COSAScript::DoOSAEvent
//----------------------------------------------------------------------------------------


OSAError COSAScript::DoOSAEvent(ComponentInstance theComponent,
								TAppleEvent* message,
								TAppleEvent* reply,
								long modeFlags)
{
	OSAError err = noErr;

	CTempUseResFile_AC tempUseResFile(CurResFile_AC());  // preserve the current resource file
	err = OSADoEvent(theComponent, message, fScriptID, modeFlags, reply);

	return err;
}

//========================================================================================
// CLASS COSAScriptCntPtr
//========================================================================================

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::COSAScriptCntPtr
//----------------------------------------------------------------------------------------

COSAScriptCntPtr::COSAScriptCntPtr() :
	fRep(NULL)
{
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::~COSAScriptCntPtr
//----------------------------------------------------------------------------------------

COSAScriptCntPtr::~COSAScriptCntPtr()
{
	DownCount();
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::COSAScriptCntPtr
//----------------------------------------------------------------------------------------

COSAScriptCntPtr::COSAScriptCntPtr(const COSAScriptCntPtr& other) :
	fRep(other.fRep)
{
	UpCount();
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::COSAScriptCntPtr
//----------------------------------------------------------------------------------------

COSAScriptCntPtr::COSAScriptCntPtr(TOSAScriptRep* rep) :
	fRep(rep)
{
	UpCount();
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::operator=
//----------------------------------------------------------------------------------------

COSAScriptCntPtr& COSAScriptCntPtr::operator=(const COSAScriptCntPtr& other)
{
	SetRep(other.fRep);
	return *this;
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::operator=
//----------------------------------------------------------------------------------------

COSAScriptCntPtr& COSAScriptCntPtr::operator=(TOSAScriptRep* rep)
{
	SetRep(rep);
	return *this;
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::SetRep
//----------------------------------------------------------------------------------------

void COSAScriptCntPtr::SetRep(TOSAScriptRep* rep)
{
	if (fRep != rep)
	{
		DownCount();
		fRep = rep;
		UpCount();
	}
}
		
//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::UpCount
//----------------------------------------------------------------------------------------

void COSAScriptCntPtr::UpCount()
{
	if (fRep)
		fRep->IncrementReferenceCount();
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::DownCount
//----------------------------------------------------------------------------------------

void COSAScriptCntPtr::DownCount()
{
	if (fRep && !fRep->DecrementReferenceCount())
		delete fRep;
	// It is not necessary to set fRep to 0. To see why, see how DownCount is used above.
	// In all cases, fRep is either immediately reset, or the instance is being destroyed.
	// Note too that DownCount is private, so the usages above all all possible usages.
}

//----------------------------------------------------------------------------------------
// COSAScriptCntPtr::NewOSAScript:
//----------------------------------------------------------------------------------------


TOSAScriptRep* COSAScriptCntPtr::NewOSAScript()
{
	return TH_new TOSAScriptRep;
}

//========================================================================================
// CLASS TOSAScriptRep
//========================================================================================
//----------------------------------------------------------------------------------------
// TOSAScriptRep constructor
//----------------------------------------------------------------------------------------


TOSAScriptRep::TOSAScriptRep()
	: fScript(kOSANullScript),
	  fNextScriptIdle(0),
	  fCompileFlags(kOSAModeCompileIntoContext),
	  fLoadFlags(kOSAModeNull),
	  fStoreFlags(kOSAModeDontStoreParent),
	  fExecuteFlags(kOSAModeNull),
	  fDoEventFlags(kOSAModeNull),
	  fRefCount(0)
{
  fScriptingComponent = COSAScript::GetDefaultComponent();
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep destructor
//----------------------------------------------------------------------------------------


TOSAScriptRep::~TOSAScriptRep()
{
	FreeOSAScript();
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::HasOSAScript:
//----------------------------------------------------------------------------------------


bool TOSAScriptRep::HasOSAScript()
{
	return HasAppleScript_AC() && (!fScript.IsEmpty());
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::FreeOSAScript:
//----------------------------------------------------------------------------------------

 
void TOSAScriptRep::FreeOSAScript()
{
	if (HasOSAScript())
		::ThrowIfError_AC(fScript.DisposeScript(fScriptingComponent));
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::SetOSAScript:
//----------------------------------------------------------------------------------------


void TOSAScriptRep::SetOSAScript(const CAEDesc& scriptDesc)
{
	if (HasAppleScript_AC())
	{
		FreeOSAScript();	// dispose the current script
		::ThrowIfError_AC(fScript.SetScript(fScriptingComponent, scriptDesc, fCompileFlags, fLoadFlags));
		OSAID parentScript = GetOSAParent();
		fScript.SetParentScript(fScriptingComponent, parentScript);
	}
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::GetOSAScript:
//----------------------------------------------------------------------------------------


void TOSAScriptRep::GetOSAScript(CAEDesc& scriptDesc,
								const CAEDesc& desiredType)
{
	if (HasAppleScript_AC())
	{
		if (HasOSAScript())
		{
			DescType whichType = typeOSAGenericStorage;
			if (desiredType.GetDescriptorType() == typeType)
				whichType = desiredType.GetType();
			::ThrowIfError_AC(fScript.GetScript(fScriptingComponent, whichType, scriptDesc, fStoreFlags));
		}
	}
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::ExecuteOSAScript:
//----------------------------------------------------------------------------------------


OSAID TOSAScriptRep::ExecuteOSAScript()
{
	OSAID resultScript = kOSANullScript;
	if (HasOSAScript())
		::ThrowIfError_AC(fScript.Execute(fScriptingComponent, kOSANullScript, fExecuteFlags, resultScript));
	
	return resultScript;
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::HandleOSAEvent:
//----------------------------------------------------------------------------------------


bool TOSAScriptRep::HandleOSAEvent(CommandNumber /*aCommandNumber*/,
								   TAppleEvent* message,
								   TAppleEvent* reply)
{
	bool handled = false;
	if (HasAppleScript_AC())
	{
		OSErr err = errAEEventNotHandled;
		COSAScript globalScript = COSAScript::GetGlobalScript();
		COSAScript parentScript = GetOSAParent();
		bool hasParentScript = !(parentScript == kOSANullScript);
		bool hasLocalScript = HasOSAScript();
		bool useGlobalScript = !(globalScript == kOSANullScript) &&
								  (hasLocalScript || hasParentScript);
		
		if (useGlobalScript || hasLocalScript || hasParentScript)
		{
			long refCon = 0;						// passed by address
			AEEventHandlerUPP currProc = NULL;		// passed by address

			try
			{
				::ThrowIfError_AC(OSAGetResumeDispatchProc(fScriptingComponent, &currProc, &refCon));
				::ThrowIfError_AC(OSASetResumeDispatchProc(fScriptingComponent, (AEEventHandlerUPP)kOSAUseStandardDispatch, kOSADontUsePhac));
				
				if (useGlobalScript)
				{
					if (hasLocalScript)
						globalScript.SetParentScript(fScriptingComponent, fScript);
					else if (hasParentScript)
						globalScript.SetParentScript(fScriptingComponent, parentScript);
					err = globalScript.DoOSAEvent(fScriptingComponent, message, reply, fDoEventFlags);
				}
				else if (hasLocalScript)
				{
					err = fScript.DoOSAEvent(fScriptingComponent, message, reply, fDoEventFlags);
				}
				else if (hasParentScript)
				{
					err = parentScript.DoOSAEvent(fScriptingComponent, message, reply, fDoEventFlags);
				}
				handled = (err == noErr);
				::ThrowIfError_AC(OSASetResumeDispatchProc(fScriptingComponent, currProc, refCon));
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				::ThrowIfError_AC(OSASetResumeDispatchProc(fScriptingComponent, currProc, refCon));
				throw;
			}
			
			
		}
	}
	return handled;
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::ReadOSAScript:
//----------------------------------------------------------------------------------------


void TOSAScriptRep::ReadOSAScript(CStream_AC* aStream)
{
	CTempDesc theScriptDesc;
	theScriptDesc.ReadFrom(aStream);
	SetOSAScript(theScriptDesc);
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::ReadOSAScript:
//----------------------------------------------------------------------------------------


void TOSAScriptRep::ReadOSAScript(	TFile* 	aFile,
									ResType	resourceType,
									short	resourceID)
{
	CTempDesc theScriptDesc;
	
	theScriptDesc.ReadFrom(aFile, resourceType, resourceID);
	SetOSAScript(theScriptDesc);
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::WriteOSAScript:
//----------------------------------------------------------------------------------------


void TOSAScriptRep::WriteOSAScript(	CStream_AC* aStream,
									DescType asType)
{
	CTempDesc theScriptDesc;
	CTempDesc theTypeDesc;
	theTypeDesc.PutType(asType);
	GetOSAScript(theScriptDesc, theTypeDesc);
	theScriptDesc.WriteTo(aStream);
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::WriteOSAScript:
//----------------------------------------------------------------------------------------


void TOSAScriptRep::WriteOSAScript(	TFile* 		aFile,
									DescType 	asType,
									short		resourceID,
									CStr255_AC&	resourceName)
{
	CTempDesc scriptDesc;
	CTempDesc typeDesc;
	
	typeDesc.PutType(asType);
	GetOSAScript(scriptDesc, typeDesc);
	scriptDesc.WriteTo(aFile, resourceID, resourceName);
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::GetOSAParent:
//----------------------------------------------------------------------------------------


OSAID TOSAScriptRep::GetOSAParent()
{
	return COSAScript::GetSharedScript();
}

//----------------------------------------------------------------------------------------
// TOSAScriptRep::IdleOSAScript:
//----------------------------------------------------------------------------------------


void TOSAScriptRep::IdleOSAScript()
{
	// Gives an attached script idle time so its idle event handler can do some work.
	if (HasAppleScript_AC() && (fNextScriptIdle != kMaxIdleTime) && (TickCount() > fNextScriptIdle))
	{
		CAEDesc myAddress;
		TAppleEvent* message = NULL;
		TAppleEvent* reply = NULL;
		TClientCommand* theCmd = NULL;

		try
		{
			message = TH_new TAppleEvent('misc', 'idle', myAddress, kAEWaitReply);
			reply = TH_new TAppleEvent('misc', 'idle', myAddress, kAENoReply);
			
			fNextScriptIdle = TickCount() + 3600; // Default to one minute in case the idle proc doesn't return anything
			try
			{
				if (HandleOSAEvent(0, message, reply) && reply->HasParameter(keyAEResult))
				{
					unsigned long idleSeconds = reply->ReadLong(keyAEResult);
					fNextScriptIdle = TickCount() + (idleSeconds * 60);
				}
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				// fall through without resignalling
			}
			
			
			theCmd = TH_new TClientCommand(0, NULL, false, false, NULL, message);
			theCmd->ProcessReply(reply);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			// fall through without resignalling
		}
		
		
		if (theCmd)
			delete theCmd;
		else
		{	// Freeing the command will free the message and reply
			delete message;
			delete reply;
		}
	}
}

//========================================================================================
// CLASS TExecuteScriptCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TExecuteScriptCommand constructor
//----------------------------------------------------------------------------------------


TExecuteScriptCommand::TExecuteScriptCommand(CommandNumber itsCommandNumber,
												  TCommandHandler* itsContext,
											      bool canUndo,
											      bool causesChange,
											      MDependable_AC* objectToNotify,
											      OSAID scriptToExecute,
											      OSAID theScriptContext,
											      long executeFlags,
											      bool disposeOfScript,
											      ComponentInstance whichComponent) :
	TCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify),

	fScript (scriptToExecute),
	fScriptContext(theScriptContext),
	fDisposeScript(disposeOfScript),
	fComponent(whichComponent),
	fExecuteFlags(executeFlags)
{
}

//----------------------------------------------------------------------------------------
// TExecuteScriptCommand::~TExecuteScriptCommand:
//----------------------------------------------------------------------------------------


TExecuteScriptCommand::~TExecuteScriptCommand()
{
	if (fDisposeScript)
		fScript.DisposeScript(fComponent);
}

//----------------------------------------------------------------------------------------
// TExecuteScriptCommand::DoIt:
//----------------------------------------------------------------------------------------


void TExecuteScriptCommand::DoIt()	// override
{
	COSAScript theScript(fScript);
	OSAID aResult;
	if (theScript != kOSANullScript)
	{
		// Run it through the globals script if there is one
		COSAScript globalsScript = COSAScript::GetGlobalScript();
		if (globalsScript != kOSANullScript)
		{
			globalsScript.SetParentScript(fComponent, theScript);		
			globalsScript.Execute(fComponent, fScriptContext, fExecuteFlags, aResult);
		}
		else
			theScript.Execute(fComponent, fScriptContext, fExecuteFlags, aResult);
	}
}

//----------------------------------------------------------------------------------------
// End of UOSAScript.cp


