//----------------------------------------------------------------------------------------
// UScriptableObject.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USCRIPTABLEOBJECT__
#include "UScriptableObject.h"
#endif

// ACS

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#if qAttachable
	#ifndef __UOSASCRIPT__
	#include "UOSAScript.h"
	#endif
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

// Toolbox

#ifndef __AEPACKOBJECT__
#include <AEPackObject.h>
#endif

#ifndef __ASDEBUGGING__
#include <ASDebugging.h>
#endif

#ifndef __ASREGISTRY__
#include <ASRegistry.h>
#endif

#ifndef __OSAGeneric__
#include <OSAGeneric.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

//========================================================================================
// CLASS MScriptableObject
//========================================================================================
//----------------------------------------------------------------------------------------
// MScriptableObject constructor
//----------------------------------------------------------------------------------------


MScriptableObject::MScriptableObject()
	: fOMClassID(typeNull)
{
}

MScriptableObject::MScriptableObject(DescType myClassID)
	: fOMClassID(myClassID)
{
}

#if qAttachable

//----------------------------------------------------------------------------------------
// MScriptableObject destructor
//----------------------------------------------------------------------------------------


MScriptableObject::~MScriptableObject()
{
	FreeOSAScript();
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// MScriptableObject::GetOMClass
//----------------------------------------------------------------------------------------


DescType MScriptableObject::GetOMClass()
{
	return fOMClassID;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetSpecifierForm
//----------------------------------------------------------------------------------------


DescType MScriptableObject::GetSpecifierForm()
{
	return formAbsolutePosition;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::CountContainedObjects
//----------------------------------------------------------------------------------------


long MScriptableObject::CountContainedObjects(DescType/* desiredType */ )
{
	// Returns the number of objects ofType that this object contains.
	return 0;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::CompareScriptableObjects
//----------------------------------------------------------------------------------------


bool MScriptableObject::CompareScriptableObjects(DescType operation,
													const CAEDesc& thingToCompare)
{
	// Compares this object to something else. This default version compares to another
	// object for direct object equality. Override this for more detailed comparisons.
	MScriptableObject * compareWith = thingToCompare.GetObject();

	return operation == kAEEquals ? compareWith == this : false;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::CompareProperties
//----------------------------------------------------------------------------------------


bool MScriptableObject::CompareProperties(DescType whichProperty,
											 DescType operation,
											 const CAEDesc& thingToCompare)
{
	// Compares a property of this object to something else. This is a generic comparison routine
	// Designed to handle the most common comparison requests.

	bool result = false;
	CTempDesc theProperty;
	if (GetObjectProperty(theProperty, whichProperty, CAEDesc::fgNullDesc))
		result = theProperty.CompareDesc(thingToCompare, operation);
	return result;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::MakeObjectSpecifier
//----------------------------------------------------------------------------------------


bool MScriptableObject::MakeObjectSpecifier(CAEDesc& theObjectSpecifier,
											   DescType preferredForm)
// Creates an object specifier describing this object. This calls GetObjectProperty and
// GetObjectsContainer to support formName, formAbsolutePosition, and formUniqueID.
// You'll only need to override this method if you want to make other forms of
// object specifiers.
{
	CAEDesc containerSpec;
	DescType containerForm = preferredForm;
	MScriptableObject * theContainer = GetObjectsContainer();
	theContainer->MakeObjectSpecifier(containerSpec, theContainer->GetSpecifierForm());
	DescType myClass = GetOMClass();
	CAEDesc formDesc;
	bool createSpecifier = false;
	if (preferredForm == formName)
	{
		if (GetObjectProperty(formDesc, pName, CAEDesc::fgNullDesc))
			createSpecifier = true;
		else
			preferredForm = formAbsolutePosition; // no name, try absolute position.
	}
	if (preferredForm == formUniqueID)
	{
		if (GetObjectProperty(formDesc, pID, CAEDesc::fgNullDesc))
			createSpecifier = true;
		else
			preferredForm = formAbsolutePosition; // no unique ID, try absolute position.
	}
	if (preferredForm == formAbsolutePosition)
	{
		createSpecifier = GetObjectProperty(formDesc, pIndex, CAEDesc::fgNullDesc);
	}
	if (createSpecifier)
		::ThrowIfOSErr_AC(CreateObjSpecifier(myClass, containerSpec, preferredForm, formDesc, true, theObjectSpecifier));

	return createSpecifier;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetCommandContext
//----------------------------------------------------------------------------------------


TCommandHandler* MScriptableObject::GetCommandContext(const CommandNumber aCommandNumber) const
{
	MScriptableObject* castFrom = const_cast<MScriptableObject*>(this);
	TCommandHandler *commandHandler = dynamic_cast<TCommandHandler*>(castFrom);
	if (commandHandler)
		commandHandler = commandHandler->GetContext(aCommandNumber);
	
	if (!commandHandler)
	{
		MDefaultScriptableObject *defaultTarget = TOSADispatcher::fgDispatcher->GetDefaultTarget();
		
		if (defaultTarget)
		{
			commandHandler = dynamic_cast<TCommandHandler*>(defaultTarget);
			if (commandHandler)
				commandHandler = commandHandler->GetContext(aCommandNumber);
		}
	}
	
	if (!commandHandler)	// getting desperate...
		commandHandler = gDispatcher;
		
	return commandHandler;
}

#if qOptimizeSelfSendAevt		

//----------------------------------------------------------------------------------------
// MScriptableObject::IsPendingAction
//----------------------------------------------------------------------------------------


bool MScriptableObject::IsPendingAction(	const CommandNumber aCommandNumber,
											const unsigned long actionID) const
{
	bool pending = false;
	
	if (actionID)
	{
		TCommandHandler *commandHandler = GetCommandContext(aCommandNumber);
		if (commandHandler)
			pending = (actionID == commandHandler->GetPendingActionID());
	}
	return pending;
}

#endif

//----------------------------------------------------------------------------------------
// MScriptableObject::HandleScriptCommand:
//----------------------------------------------------------------------------------------


void MScriptableObject::HandleScriptCommand(CommandNumber	aCommandNumber,
											TAppleEvent* 	message,
											TAppleEvent*	reply)
{
	bool wasHandled = false;
	
	TCommandHandler* commandHandler = dynamic_cast<TCommandHandler*>(this);
	if (commandHandler)
	{
		TBehavior *aBehavior = commandHandler->GetFirstEnabledBehavior();
		if (aBehavior)
			wasHandled = aBehavior->DoScriptCommand(aCommandNumber, message, reply);
	}
	
	if (!wasHandled)
		DoScriptCommand(aCommandNumber, message, reply);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoScriptCommand
//----------------------------------------------------------------------------------------


void MScriptableObject::DoScriptCommand(CommandNumber 	aCommandNumber,
										TAppleEvent* 	message,
										TAppleEvent* 	reply)
// Aftering resolving an object specifier into an object this method is called.
// Based on the command number in the refCon it dispatches the event to
// one of the Core event support routines defined below.
// Override this method if this object supports events beyond the core suite.
{

	switch (aCommandNumber)
	{
		case cAEClone:
			DoAEClone(message, reply);
			break;

		case cAEQuit:
		case cAEClose:
			DoAEClose(message, reply);
			break;

		case cAECountElements:
			DoAECountElements(message, reply);
			break;

		case cAECreateElement:
			DoAECreateElement(message, reply);
			break;

		case cAEDoObjectsExist:
			DoAEDoObjectsExist(message, reply);
			break;

		case cAEGetClassInfo:
			DoAEGetClassInfo(message, reply);
			break;

		case cAEGetEventInfo:
			DoAEGetEventInfo(message, reply);
			break;

		case cAEMove:
			DoAEMove(message, reply);
			break;

		case cAESetData:
			DoAESetData(message, reply);
			break;

		case cAEGetData:
			DoAEGetData(message, reply);
			break;

		case cAEGetDataSize:
			DoAEGetDataSize(message, reply);
			break;

		case cAEDelete:
			DoAEDelete(message, reply);
			break;

		case cAESave:
			DoAESave(message, reply);
			break;

		case cAEOpen:
			DoAEOpen(message, reply);
			break;

		case cAEPrint:
		case cPrint:
			DoAEPrint(message, reply);
			break;

		default:
			::ThrowIfOSErr_AC(errAEEventNotHandled);
			break;
	}
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEOnContainedObjects
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEOnContainedObjects(TScriptableObjectList* theObjectList,
											   CommandNumber aCommandNumber,
											   TAppleEvent* message,
											   TAppleEvent* reply)
{
	// If the AppleEvent's object specifier resolves into a list of objects, MacApp first gives
	// their container object a shot at handling the event.

	// If this is a list of TPropertyAccessors and its a SetData AppleEvent, create a TSetPropertyCommand
	// To do the work on all the objects
	if ((aCommandNumber == cAESetData) && theObjectList->IsPropertyList())
	{
		CList_AC theCmdObjectList;

		DescType theProperty;
		for (TemplateIterator_AC<TPropertyAccessor*> iter(theObjectList); iter; ++iter)
		{
			theProperty = iter->fWhichProperty;
			theCmdObjectList.InsertLast(iter);
		}

		CAutoPtr_AC<TSetPropertyCommand> theCmd(TH_new TSetPropertyCommand(theProperty, &theCmdObjectList, message, reply));
		theCmd->Process();
		theCmd.release();
	}
	else
		theObjectList->DoScriptCommand(aCommandNumber, message, reply);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEClone
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEClone(TAppleEvent*	/* message */ ,
								  TAppleEvent* /* reply */)
{
	// Handle the Clone AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantClone);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEClose
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEClose(TAppleEvent*	/* message */ ,
								  TAppleEvent* /* reply */)
{
	// Handle the Close AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantClose);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAECountElements
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAECountElements(TAppleEvent* message,
										  TAppleEvent* reply)
{
	// Handle the Count Elements AppleEvent from the Core Suite.
	DescType theClass = dynamic_cast_AC(TAppleEvent*,message)->ReadType(keyAEObjectClass);
	long numObjects = CountContainedObjects(theClass);
	reply->WriteLong(keyAEResult, numObjects);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAECreateElement
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAECreateElement(TAppleEvent*/* message */  ,
										  TAppleEvent* /* reply */)
{
	// Handle the Create Element AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantCreate);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEDelete
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEDelete(TAppleEvent*	/* message */ ,
								   TAppleEvent* /* reply */)
{
	// Handle the Delete AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantDelete);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEDoObjectsExist
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEDoObjectsExist(TAppleEvent* /* message */  ,
										   TAppleEvent* reply)
{
	// Handle the DoObjectsExist AppleEvent from the Core Suite.
	// Of course I exist, otherwise I couldn't do this:
	dynamic_cast_AC(TAppleEvent*,reply)->WriteBoolean(keyAEResult, true);
	// If I didn't exist AccessContainedObjects would fail and that's handled 
	// in TScriptEventDispatcher:DispatchAppleEvent.
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEGetClassInfo
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEGetClassInfo(TAppleEvent* /* message */  ,
										 TAppleEvent* /* reply */)
{
	// Handle the DoGetClassInfo AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantGetClassInfo);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEGetEventInfo
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEGetEventInfo(TAppleEvent* /* message */  ,
										 TAppleEvent* /* reply */)
{
	// Handle the DoGetEventInfo AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantGetEventInfo);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEGetData
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEGetData(TAppleEvent*/* message */  ,
									TAppleEvent* reply)
{
	// Handle the GetData AppleEvent from the Core Suite.
	CTempDesc theSpecifier;
	MakeObjectSpecifier(theSpecifier, GetSpecifierForm());
	reply->WriteParameter(keyAEResult, theSpecifier);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEGetDataSize
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEGetDataSize(TAppleEvent* /* message */  ,
										TAppleEvent* /* reply */)
{
	// Handle the GetDataSize AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantGetDataSize);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEMove
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEMove(TAppleEvent*	/* message */ ,
								 TAppleEvent*	/* reply */)
{
	// Handle the Move AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantMove);
}


//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEOpen
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEOpen(TAppleEvent*	/* message */ ,
								 TAppleEvent*	/* reply */)
{
	// Handle the Open AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantOpen);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAEPrint
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAEPrint(TAppleEvent*	/* message */ ,
								  TAppleEvent*	/* reply */)
{
	// Handle the Print AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantPrint);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAESave
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAESave(TAppleEvent*	/* message */ ,
								 TAppleEvent*	/* reply */)
{
	// Handle the Save AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantSave);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::DoAESetData
//----------------------------------------------------------------------------------------


void MScriptableObject::DoAESetData(TAppleEvent*	/* message */  ,
									TAppleEvent*	/* reply */)
{
	// Handle the SetData AppleEvent from the Core Suite.
	::ThrowIfOSErr_AC(errAECantSetData);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetContents
//----------------------------------------------------------------------------------------


bool MScriptableObject::GetContents(CAEDesc& /*theContents*/,
										const CAEDesc& /*desiredType*/,
										long /*propertyFlags*/)
{
	// Override to return the contents of this object.
	// Default is "not found". 
	return false;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::SetContents
//----------------------------------------------------------------------------------------


void MScriptableObject::SetContents(const CAEDesc& theContents)
{
	// Sets the contents of this object. If theContents is an AERecord,
	// SetObjectProperty will be called for each property key.
	if (theContents.GetDescriptorType() == typeAERecord)
	{
		long numProps = 0;
		::ThrowIfOSErr_AC(AECountItems(theContents, &numProps));
		for (long index = 1; index <= numProps; ++index)
		{
			CTempDesc thePropData;
			DescType theKey;
			if (AEGetNthDesc(theContents, index, typeWildCard, &theKey, thePropData) == noErr)
				SetObjectProperty(thePropData, theKey);
		}
	}
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetObjectProperty
//----------------------------------------------------------------------------------------


bool MScriptableObject::GetObjectProperty(CAEDesc& thePropertyValue,
											 DescType whichProperty,
											 const CAEDesc& desiredType)
{
	bool hasProperty = true;

	try
	{
		// Returns default property values for this object.
		switch (whichProperty)
		{
			case pContents:
				hasProperty = GetContents(thePropertyValue, desiredType);
				break;
				
			case pClass:
				thePropertyValue.PutType(GetOMClass());
				break;
	
#if qAttachable
			case pScript:
				GetOSAScript(thePropertyValue, desiredType);
				break;
#endif
			case pIndex:
				{
					hasProperty = false;
					DescType myType = GetOMClass();
					MScriptableObject * itsContainer = GetObjectsContainer();
					long numObjects = itsContainer->CountContainedObjects(myType);
					for (long myIndex = 1; myIndex <= numObjects; ++myIndex)
					{
						MScriptableObject * indexedObject = itsContainer->GetIndContainedObject(myType, myIndex);
						CTempDesc objectDesc;
						objectDesc.PutObject(indexedObject);
						if (CompareScriptableObjects(kAEEquals, objectDesc))
						{
							thePropertyValue.PutLong(myIndex);
							hasProperty = true;
							break;
						}
					}
				}
				break;
	
			default:
				hasProperty = false;
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		hasProperty = false;
	}
	

	return hasProperty;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::SetObjectProperty
//----------------------------------------------------------------------------------------


void MScriptableObject::SetObjectProperty(const CAEDesc& thePropertyValue,
										  DescType whichProperty)
{
	// Override this method to set property values for this object.
	switch (whichProperty)
	{
#if qAttachable
		case pScript:
			SetOSAScript(thePropertyValue);
			break;
#endif
		case pContents:
			SetContents(thePropertyValue);
			break;

		case pClass:
		case pIndex:
			::ThrowIfOSErr_AC(errAECantSetReadOnly);
			break;

		default:
			::ThrowIfOSErr_AC(errAENoSuchObject);
			break;
	}
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetSetPropertyInfo
//----------------------------------------------------------------------------------------


void MScriptableObject::GetSetPropertyInfo(DescType /*whichProperty*/ ,
										   CommandNumber& cmdNum,
										   bool& canUndo,
										   bool& causesChange,
										   TCommandHandler* &theContext)
{
	// When an AppleEvent is received that will set a property for this object a TSetPropertyCommand
	// will be created to do it. You can override this method to provide the command number and the
	// canUndo and causesChange flags for the command.
	cmdNum = cAESetData;
	canUndo = true;
	causesChange = true;
	theContext = GetCommandContext(cmdNum);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetContainedObject
//----------------------------------------------------------------------------------------


MScriptableObject* MScriptableObject::GetContainedObject(DescType desiredType,
														 DescType selectionForm,
														 const CAEDesc& selectionData)
{
	// Returns an object contained within this one. If the desiredType is a property
	// of this object, we create a TPropertyAccessor and return it.
	// Later on, the TPropertyAccessor can act on the properties of this object.
	// For other types we try to return an object. If you would really like to reference
	// something else you can create an object class that references it and return one of those.
	// For each keyform this routine figures out which contained object is needed and gets it
	// by calling GetIndContainedObject. If you just want to provide the OSL with access to 
	// contained objects you should override GetIndContainedObject.
	MScriptableObject *result = NULL;
	CAEDesc theDataDesc = selectionData;
	if (desiredType == cApplication)
		result = gDispatcher;
	else if (desiredType == cProperty)
	{
		DescType whichProperty = theDataDesc.GetType();
		TPropertyAccessor * theAccessor = TH_new TPropertyAccessor(this, whichProperty);
		TOSADispatcher::fgDispatcher->AddTemporaryToken(theAccessor);
		result = theAccessor;
	}
	else
	{
		// Now we switch on the key forms to find the correct contained object.
		switch (selectionForm)
		{
			case formName:
				{
					// Search through all the objects of this type and look at their pName property.
					// This is hardly the most efficient way to do this, but works in a very generic fashion.
					// If you have a lot of contained objects you can override this to look at the contained
					// objects directly.
					CStr255_AC keyNameStr;
					CStr255_AC objectNameStr;
					theDataDesc.GetString(keyNameStr, 255);
					long numObjects = CountContainedObjects(desiredType);
					for (long myIndex = 1; myIndex <= numObjects; ++myIndex)
					{
						CTempDesc theNameDesc;
						MScriptableObject *indexedObject = GetIndContainedObject(desiredType, myIndex);
						if (indexedObject && indexedObject->GetObjectProperty(theNameDesc, pName, CAEDesc::fgNullDesc))
						{
							theNameDesc.GetString(objectNameStr, 255);
							if (!IUEqualString(keyNameStr, objectNameStr))
							{
								result = indexedObject;
								break;
							}
						}
					}
					break;
				}

			case formRelativePosition:
				{
					DescType whichPosition = **(DescType * *)theDataDesc.GetDataHandle();
					result = GetAdjacentObject(desiredType, whichPosition);
					break;
				}

			case formRange:
				{
					CTempDesc rangeStart;
					CTempDesc rangeStop;
					CTempDesc rangeRecord;
					MScriptableObject * startObj = NULL;
					MScriptableObject * stopObj = NULL;
					::ThrowIfOSErr_AC(AECoerceDesc(theDataDesc, typeAERecord, rangeRecord));
					rangeRecord.GetKeyDesc(keyAERangeStart, typeObjectSpecifier, rangeStart);
					rangeRecord.GetKeyDesc(keyAERangeStop, typeObjectSpecifier, rangeStop);
					startObj = TOSADispatcher::fgDispatcher->ResolveObjectSpecifier(rangeStart);
					stopObj = TOSADispatcher::fgDispatcher->ResolveObjectSpecifier(rangeStop);
					::ThrowIfNULL_AC(startObj);
					::ThrowIfNULL_AC(stopObj);
					// Now we have the bounding objects in the range
					// Call the accessor function to get the range of contained objects
					result = GetObjectsWithinRange(desiredType, startObj, stopObj);
					break;
				}

			case formUniqueID:
				{
					CStr255_AC theKeyStr;
					CStr255_AC uniqueIDStr;
					theDataDesc.GetString(theKeyStr, 255);
					long numObjects = CountContainedObjects(desiredType);
					for (long myIndex = 1; myIndex <= numObjects; ++myIndex)
					{
						CTempDesc theIDDesc;
						MScriptableObject *indexedObject = GetIndContainedObject(desiredType, myIndex);
						if (indexedObject && indexedObject->GetObjectProperty(theIDDesc, pID, CAEDesc::fgNullDesc))
						{
							theIDDesc.GetString(uniqueIDStr, 255);
							if (theKeyStr == uniqueIDStr)
							{
								result = indexedObject;
								break;
							}
						}
					}
					break;
				}

			case formAbsolutePosition:
				{
					// Gets a contained object by index. Also supports typeAbsoluteOrdinal for getting
					// objects by first, last, some, middle, and every.
					long numObjects = CountContainedObjects(desiredType);
					long objectIndex = 1;
					if (theDataDesc.GetDescriptorType() == typeAbsoluteOrdinal)
					{
						DescType whichType = **(DescType * *)theDataDesc.GetDataHandle();
						switch (whichType)
						{
							case kAEFirst:
								objectIndex = 1;
								break;
							case kAELast:
								objectIndex = numObjects;
								break;
							case kAEMiddle:
								objectIndex = (numObjects + 1) / 2;
								break;
							case kAEAny:
								objectIndex = GetRandom(1, numObjects);
								break;
							case kAEAll:
								{
									CAutoPtr_AC<TScriptableObjectList> theList(TH_new TScriptableObjectList(this));

									for (long myIndex = 1; myIndex <= numObjects; ++myIndex)
									{
										MScriptableObject *indexedObject = GetIndContainedObject(desiredType, myIndex);
										if (indexedObject)
											theList->InsertLast(indexedObject);
									}
									
									result = theList.release();
									TOSADispatcher::fgDispatcher->AddTemporaryToken(theList);
								}
								break;
						}
					}
					else
					{
						objectIndex = theDataDesc.GetLong();
						if (objectIndex < 0)
							objectIndex = objectIndex + numObjects + 1;
					}
					if (!result)				// If the result has not been set previously
						result = GetIndContainedObject(desiredType, objectIndex);
					break;
				}

			default:
				::ThrowIfOSErr_AC(errAEEventNotHandled); 	//whose clauses need errAEEventNotHandled
		}
	}

	return result;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetIndContainedObject
//----------------------------------------------------------------------------------------


MScriptableObject* MScriptableObject::GetIndContainedObject(DescType/*desiredType*/ ,
															long/*index*/)
// Returns an object contained within this one based on its absolute
// position within the container. If you override this method to return
// contained objects MScriptableObject::GetContainedObject will use this to get
// one or more objects required by the different reference forms.
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetObjectsWithinRange
//----------------------------------------------------------------------------------------


MScriptableObject* MScriptableObject::GetObjectsWithinRange(DescType desiredType,
															MScriptableObject* startBounds,
															MScriptableObject* endBounds)
// Returns an object or list of objects contained within this one
// that lie within the startBounds object and the endBounds object.
// The default method assumes that the startBounds and endBounds objects
// are of the desiredType.
// You can override this to provide custom range access.
{
	long numObjects = CountContainedObjects(desiredType);
	bool addObjects = false;

	CAutoPtr_AC<TScriptableObjectList> theList(TH_new TScriptableObjectList);

	for (long myIndex = 1; myIndex <= numObjects; ++myIndex)
	{
		MScriptableObject *indexedObject = GetIndContainedObject(desiredType, myIndex);
		CTempDesc objectDesc;
		objectDesc.PutObject(indexedObject);
		addObjects = (addObjects || (startBounds->CompareScriptableObjects(kAEEquals, objectDesc)));
		if (addObjects)
		{
			theList->InsertLast(indexedObject);
			addObjects = !endBounds->CompareScriptableObjects(kAEEquals, objectDesc);
		}
	}
	TOSADispatcher::fgDispatcher->AddTemporaryToken(theList);
	
	return theList.release();
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetObjectsContainer
//----------------------------------------------------------------------------------------


MScriptableObject* MScriptableObject::GetObjectsContainer()
{
	// By default we'll return the application.
	return gDispatcher;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetAdjacentObject
//----------------------------------------------------------------------------------------


MScriptableObject* MScriptableObject::GetAdjacentObject(DescType desiredType,
														DescType position)
// When the OSL resolves relative reference forms it first specifies an object
// then asks that object for the one before or after it. This method returns
// an object of the desiredType before or after it. 
// You can override this method by to provide custom access.
{
	MScriptableObject * result = NULL;
	CTempDesc myIndex;
	if (GetObjectProperty(myIndex, pIndex, CAEDesc::fgNullDesc))
	{
		long relativeIndex = myIndex.GetLong();
		if (position == kAENext)
			++relativeIndex;
		else if (position == kAEPrevious)
			--relativeIndex;
		MScriptableObject * itsContainer = GetObjectsContainer();
		result = itsContainer->GetIndContainedObject(desiredType, relativeIndex);
	}

	return result;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::MakeNewMarkingList
//----------------------------------------------------------------------------------------


TScriptableObjectList* MScriptableObject::MakeNewMarkingList()
{
	// Returns a TScriptableObjectList to use for marking objects contained within this one. 
	TScriptableObjectList * theList = TH_new TScriptableObjectList(this);
	TOSADispatcher::fgDispatcher->AddTemporaryToken(theList);

	return theList;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::SetProperties:
//----------------------------------------------------------------------------------------


void MScriptableObject::SetPropertiesFromEvent(TAppleEvent* theEvent)
{
	// Sets the properties of an object based on the data in the keyAEPropData parameter
	// of the event.
	if (theEvent)
	{
		CTempDesc propListDesc;
		if (AEGetParamDesc(theEvent, keyAEPropData, typeAERecord, propListDesc) == noErr)
			SetContents(propListDesc);
	}
}

#if qAttachable

//----------------------------------------------------------------------------------------
// Attached Scripts
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// MScriptableObject::HasOSAScript:
//----------------------------------------------------------------------------------------


bool MScriptableObject::HasOSAScript()
{
	return HasAppleScript_AC() && fOSAScript && fOSAScript->HasOSAScript();
}

//----------------------------------------------------------------------------------------
// MScriptableObject::FreeOSAScript:
//----------------------------------------------------------------------------------------


void MScriptableObject::FreeOSAScript()
{
	if (fOSAScript)
	{
		TOSADispatcher::fgDispatcher->ScriptDetached();
		fOSAScript = NULL;	// This will free the rep object
							// if its ref count drops to 0.
	}
}

//----------------------------------------------------------------------------------------
// MScriptableObject::SetOSAScript:
//----------------------------------------------------------------------------------------


void MScriptableObject::SetOSAScript(const CAEDesc& scriptDesc)
{
	if (HasAppleScript_AC())
	{
		if (!fOSAScript)
		{
			fOSAScript = COSAScriptCntPtr::NewOSAScript();
			TOSADispatcher::fgDispatcher->ScriptAttached();
		}
		fOSAScript->SetOSAScript(scriptDesc);
	}
}

//----------------------------------------------------------------------------------------
// MScriptableObject::GetOSAScript:
//----------------------------------------------------------------------------------------


void MScriptableObject::GetOSAScript(CAEDesc& scriptDesc,
										const CAEDesc& desiredType)
{
	if (HasAppleScript_AC() && fOSAScript)
		fOSAScript->GetOSAScript(scriptDesc, desiredType);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::HandleOSAEvent:
//----------------------------------------------------------------------------------------


bool MScriptableObject::HandleOSAEvent(CommandNumber aCommandNumber,
										  TAppleEvent* message,
										  TAppleEvent* reply)
{
	bool handled = false;
	COSAScriptCntPtr tempOSAScript(fOSAScript);	// Temporary to prevent script
												// from an early death.
	if (HasAppleScript_AC() && fOSAScript)
		handled = tempOSAScript->HandleOSAEvent(aCommandNumber, message, reply);
	
	return handled;
}

//----------------------------------------------------------------------------------------
// MScriptableObject::ReadOSAScript:
//----------------------------------------------------------------------------------------


void MScriptableObject::ReadOSAScript(CStream_AC* aStream)
{
	if (HasAppleScript_AC())
	{
		if (!fOSAScript)
		{
			fOSAScript = COSAScriptCntPtr::NewOSAScript();
			TOSADispatcher::fgDispatcher->ScriptAttached();
		}
		fOSAScript->ReadOSAScript(aStream);
	}
}

//----------------------------------------------------------------------------------------
// MScriptableObject::WriteOSAScript:
//----------------------------------------------------------------------------------------


void MScriptableObject::WriteOSAScript(CStream_AC* aStream,
									   DescType asType)
{
	if (HasAppleScript_AC() && fOSAScript)
		fOSAScript->WriteOSAScript(aStream, asType);
}

//----------------------------------------------------------------------------------------
// MScriptableObject::IdleOSAScript:
//----------------------------------------------------------------------------------------


void MScriptableObject::IdleOSAScript()
{
	COSAScriptCntPtr tempOSAScript(fOSAScript);	// Temporary to prevent script
												// from an early death.
	if (HasAppleScript_AC() && fOSAScript)
		tempOSAScript->IdleOSAScript();
}

#endif // qAttachable

//========================================================================================
// CLASS MDefaultScriptableObject
//========================================================================================
//----------------------------------------------------------------------------------------
// MDefaultScriptableObject::MDefaultScriptableObject
//----------------------------------------------------------------------------------------


MDefaultScriptableObject::MDefaultScriptableObject()
	: MScriptableObject(),
	  fRecording(false)
{
}

//----------------------------------------------------------------------------------------
// MDefaultScriptableObject::MDefaultScriptableObject
//----------------------------------------------------------------------------------------


MDefaultScriptableObject::MDefaultScriptableObject(DescType myClassID)
	: MScriptableObject(myClassID),
	  fRecording(false)
{
}

//----------------------------------------------------------------------------------------
// MDefaultScriptableObject destructor
//----------------------------------------------------------------------------------------


MDefaultScriptableObject::~MDefaultScriptableObject()
{
}

//----------------------------------------------------------------------------------------
// MDefaultScriptableObject::DoScriptCommand
//----------------------------------------------------------------------------------------


void MDefaultScriptableObject::DoScriptCommand(CommandNumber aCommandNumber,
											   TAppleEvent* message,
											   TAppleEvent* reply)
{
	switch (aCommandNumber)
	{
		case cAERecordingOn:
			fRecording = true;
			break;
		
		case cAERecordingOff:
			fRecording = false;
			break;
			
		default:
			MScriptableObject::DoScriptCommand(aCommandNumber, message, reply);
			break;
	}
}

//----------------------------------------------------------------------------------------
// End of UScriptableObject.cp


