//----------------------------------------------------------------------------------------
// UScripting.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

// ACS

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USUBSTITUTION__
#include "USubstitution.h"
#endif

// Toolbox

#ifndef __AEPACKOBJECT__
#include <AEPackObject.h>
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

// ANSI

#if qDebugMsg
#endif

// ACS

#ifndef __AEObjects_AC__
#include "AEObjects_AC.h"
#endif

//========================================================================================
// Static data allocations
//========================================================================================


#if defined(__MRC__)
	TOSADispatcher* TOSADispatcher::fgDispatcher = NULL;
#else
	CAutoPtr_AC<TOSADispatcher> TOSADispatcher::fgDispatcher;
#endif

//==================================================================================================
// class TOSADispatcher
//==================================================================================================
#undef Inherited


MA_DEFINE_CLASS(TOSADispatcher);

//----------------------------------------------------------------------------------------
// TOSADispatcher::InitUScripting
//----------------------------------------------------------------------------------------


void TOSADispatcher::InitUScripting(const char* className)
{
	if (!IsHighLevelEventAware())
	{
#if qDebug
		DebugStr((StringPtr)"\p###Not HighLevelEventAware! Check SIZE resource.");
#endif
		abort();
	}


#if defined(__MRC__)
	if (TOSADispatcher::fgDispatcher)
#else
	if (TOSADispatcher::fgDispatcher.get())
#endif
	{
#if qDebugMsg
		ProgramReport_AC("Scripting already initialized\n");
#endif
	}
	else
	{
		ObjectInit_AC();
		qd.randSeed = TickCount();
		
		fgDispatcher = dynamic_cast_AC(TOSADispatcher*,ClassDesc::NewByClassName(className));
	}
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::TOSADispatcher : 
//----------------------------------------------------------------------------------------


TOSADispatcher::TOSADispatcher() :
	fPendingReplyList(TH_new TEventList),

	fDefaultTarget(NULL),
#if qAttachable
	fAttachedScripts(0),
#endif
	fDispatchLevel(0),
	fAEResolveFlags(kAEIDoMinimum | kAEIDoWhose | kAEIDoMarking),

	fTemporaryTokens(NewList_AC())
{
	InstallDispatchHandlers();
	InstallObjectCallbacks();
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::~TOSADispatcher : 
//----------------------------------------------------------------------------------------


TOSADispatcher::~TOSADispatcher()
{
	// dispose the routine descriptors allocated by this object	
	
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::InstallDispatchHandlers : 
//----------------------------------------------------------------------------------------

// Just in case, this should be in a different segment than the glue it installs

void TOSADispatcher::InstallDispatchHandlers()
{
	if (!TAppleEvent::fgIdleProc)
	{
		TAppleEvent::fgIdleProc = NewAEIdleProc(MacAppAppleEventIdleProc);
		::ThrowIfNULL_AC(TAppleEvent::fgIdleProc);
	}

	if (!fDispatchUPP)
	{
		fDispatchUPP = NewAEEventHandlerProc(TOSADispatcher::DispatchHandlerGlue);
		::ThrowIfNULL_AC(fDispatchUPP);
	}

	short numberOfTables = MACount1Resources('aedt'); // count the number of table resources
	::ThrowIfResError_AC();
	
	for (short tableIndex = 1; tableIndex <= numberOfTables; ++tableIndex)
	{
		Handle tableHandle = MAGet1IndResource('aedt', tableIndex);
		::ThrowIfNULLResource_AC(tableHandle);
		CTempHandleLock_AC tempHandleLock(tableHandle, eDontMoveHi, eReleaseIt, eLockIt);

		Size tableSize = GetHandleSize(tableHandle);
		::ThrowIfMemError_AC();
		short tableElements = static_cast<short>(tableSize / sizeof(MAEventTableRec));
			
		MAEventTableRec *tablePtr = (MAEventTableRec*)*tableHandle;
			
		for (short eventIndex = 1; eventIndex <= tableElements; ++eventIndex)
		{
			// install the same event handler for all events
			::ThrowIfOSErr_AC(InstallEventHandler_AC(tablePtr->theClass, tablePtr->theID,
						 fDispatchUPP, tablePtr->theValue, false));
			++tablePtr;
		}
	}
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TOSADispatcher::ScriptAttached : 
//----------------------------------------------------------------------------------------


void TOSADispatcher::ScriptAttached()
{
	if (++fAttachedScripts == 1)
		InstallPreDispatchHandler();
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::ScriptDetached : 
//----------------------------------------------------------------------------------------


void TOSADispatcher::ScriptDetached()
{
#if qDebug
	::Assertion_AC(fAttachedScripts >= 0, "Too many calls to ScriptDetached");
#endif
	if (!(--fAttachedScripts))
		RemovePreDispatchHandler();
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::InstallPreDispatchHandler : 
//----------------------------------------------------------------------------------------

// Just in case, this should be in a different segment than the glue it installs

void TOSADispatcher::InstallPreDispatchHandler()
{
	if (HasAppleScript_AC())
	{	
		if  (!fPreDispatchUPP)
		{
			fPreDispatchUPP = NewAEEventHandlerProc(TOSADispatcher::PreDispatchHandlerGlue);
			::ThrowIfNULL_AC(fPreDispatchUPP);
		}
		::ThrowIfOSErr_AC(InstallPreDispatchHandler_AC(fPreDispatchUPP.get(), false));
	}
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::RemovePreDispatchHandler : 
//----------------------------------------------------------------------------------------


void TOSADispatcher::RemovePreDispatchHandler()
{
	if (HasAppleScript_AC() && fPreDispatchUPP)
		::ThrowIfOSErr_AC(RemovePreDispatchHandler_AC(fPreDispatchUPP.get(), false));
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TOSADispatcher::InstallObjectCallbacks : 
//----------------------------------------------------------------------------------------

// Just in case, this should be in a different segment than the glue it installs

void TOSADispatcher::InstallObjectCallbacks()
{
		// install the universal object accessor
	fAccessorUPP = NewOSLAccessorProc(TOSADispatcher::ObjectAccessorGlue);
	::ThrowIfOSErr_AC(InstallObjectAccessor_AC(typeWildCard, typeWildCard, fAccessorUPP, 0, false));
	
	fCompareUPP = NewOSLCompareProc(TOSADispatcher::CompareObjectsGlue);
	fCountUPP = NewOSLCountProc(TOSADispatcher::CountObjectsGlue);
	fDisposeTokenUPP = NewOSLDisposeTokenProc(TOSADispatcher::DisposeTokenGlue);
	fGetMarkTokenUPP = NewOSLGetMarkTokenProc(TOSADispatcher::GetMarkTokenGlue);
	fMarkUPP = NewOSLMarkProc(TOSADispatcher::MarkObjectGlue);
	fAdjustMarksUPP = NewOSLAdjustMarksProc(TOSADispatcher::AdjustMarksGlue);
	
	::ThrowIfOSErr_AC(SetObjectCallbacks_AC(fCompareUPP, fCountUPP, fDisposeTokenUPP,
								   fGetMarkTokenUPP, fMarkUPP, fAdjustMarksUPP, NULL));
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TOSADispatcher::PreDispatchHandler : 
//----------------------------------------------------------------------------------------


OSErr TOSADispatcher::PreDispatchHandler(CommandNumber	aCommandNumber,
										TAppleEvent* 	message,
									 	TAppleEvent*	reply)
{
	OSErr err = noErr;
		
	try
	{
		MScriptableObject* targetObject = GetTargetObject(aCommandNumber, message);
		err = targetObject->HandleOSAEvent(aCommandNumber, message, reply) ? noErr : errAEEventNotHandled;
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		// if the event is an "exists" event and the target was not found
		// return false instead of an error

		if (aCommandNumber == cAEDoObjectsExist && theException.GetError() == errAENoSuchObject)
			dynamic_cast_AC(TAppleEvent*,reply)->WriteBoolean(keyAEResult, false);
		else
			throw;
	}
	
		
	return err;
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TOSADispatcher::DispatchHandler : 
//----------------------------------------------------------------------------------------


OSErr TOSADispatcher::DispatchHandler(	CommandNumber	aCommandNumber,
										TAppleEvent* 	message,
									 	TAppleEvent*	reply)
{
	OSErr err = noErr;
	
	try
	{
		TScriptableObjectList 	*theObjectList;
		bool					targetIsContainer = false;

		MScriptableObject* targetObject = GetTargetObject(aCommandNumber, message);
	
		if (targetObject->GetOMClass() == cScriptableObjectList)
		{
			theObjectList = dynamic_cast_AC(TScriptableObjectList*,targetObject);
			targetObject = theObjectList->GetObjectsContainer();
			targetIsContainer = true;
		}
		
#if qOptimizeSelfSendAevt		
		long transactionID = message->GetTransactionID();
		if (targetObject->IsPendingAction(aCommandNumber, transactionID))
			err = errAEEventNotHandled;
		else
#endif
		{
			// print events directed at objects other than the application
			// need to be handled as cPrint commands. 
			if (aCommandNumber == cFinderPrint && targetObject != gDispatcher)
				aCommandNumber = cPrint;
				
			if (targetIsContainer)
				targetObject->DoAEOnContainedObjects(theObjectList, aCommandNumber, message, reply);
			else
				targetObject->HandleScriptCommand(aCommandNumber, message, reply);
		}
	}
	catch(CException_AC& theException)
	{	
		DoCatchMessage_AC(theException);
		// if the event is an "exists" event and the target was not found
		// return false instead of an error

		if (aCommandNumber == cAEDoObjectsExist && theException.GetError() == errAENoSuchObject)
			dynamic_cast_AC(TAppleEvent*,reply)->WriteBoolean(keyAEResult, false);
		else
			throw;
	}
	
	
	return err;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::ReportDispatchError :
// Note: This function is invoked when an error occurs during processing of an incoming
// AppleEvent. All calls to subroutines must call resident code. Apple event manager
// calls are made directly, rather than through TAppleEvent, to eliminate dependencies
// on non-resident code.
//----------------------------------------------------------------------------------------


void TOSADispatcher::ReportDispatchError(AppleEvent* 	messageEvent,
										 AppleEvent* 	replyEvent,
										 OSErr			dispatchErr,
										 long			dispatchErrMessage)
{
	Boolean sentToSelf = false;
	bool interactWithUser = false;
	bool senderWantsReply = false;
	OSErr	err;
	
	if (messageEvent)
	{
		ProcessSerialNumber eventPSN;
		DescType			actualType;
		Size				actualSize;
		AESendMode			sendMode;
		
		err = AEGetAttributePtr(messageEvent, keyAddressAttr, typeProcessSerialNumber,
			&actualType, &eventPSN, sizeof(ProcessSerialNumber), &actualSize);
		if (err == noErr)
			SameProcess(&eventPSN, &gDispatcher->GetProcessSerialNumber(), &sentToSelf);
		
		err = AEGetAttributePtr(messageEvent, keyInteractLevelAttr, typeShortInteger,
			&actualType, &sendMode, sizeof(short), &actualSize);
		if (err == noErr)
			senderWantsReply = (sendMode & kAEReplyModeMask) != kAENoReply;
	}
	
	if (!sentToSelf && !senderWantsReply)
	{
		// pass NULL for the idling proc. incoming events right now might
		// cause memory problems
		interactWithUser = MAInteractWithUserNoIdleProc() == noErr;
	}
			
	// Put up a dialog if:
	//   1) we are not responding to an event sent by ourself AND
	//   2) the client doesn't expect a reply AND
	//   3) user interaction is acceptable.
	
	if (!sentToSelf && !senderWantsReply && interactWithUser)
		gDispatcher->ShowError(dispatchErr, dispatchErrMessage);
	
	if (replyEvent && replyEvent->descriptorType != typeNull && replyEvent->dataHandle)
	{
		long 	failureError = dispatchErr;	// needs to be a signed long
		CStr255_AC	errStr;
		AEPutParamPtr(replyEvent, keyErrorNumber, typeLongInteger, &failureError, sizeof(long));
		if (!LookupErrString(dispatchErr, errReasonID, errStr))
		{
			// No error string was found, substitute the generic one.
			CStr255_AC errNumStr;
			LookupErrString(errGenericErrorWithNum, errReasonID, errStr);
			NumToString(dispatchErr, errNumStr);
			MAParamText("^ENUM", errNumStr);
			MAReplaceText(errStr);
		}
		AEPutParamPtr(replyEvent, keyErrorString, typeChar, &errStr[1], errStr.Length());
	}
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::GetTargetObject : 
//----------------------------------------------------------------------------------------


MScriptableObject* TOSADispatcher::GetTargetObject(	CommandNumber aCommandNumber,
													TAppleEvent* appleEvent)
{
	CTempDesc			objectDesc;
	OSErr				err = noErr;
	MScriptableObject 	*targetObject = NULL;
	
	// Get the direct object of the event. If the event is a create element event,
	// use the insertion object.
	if (aCommandNumber == cAECreateElement)
		::ThrowIfOSErr_AC(appleEvent->ReadParameter(keyAEInsertHere, typeWildCard, objectDesc));
	else
		err = appleEvent->ReadParameter(keyDirectObject, typeWildCard, objectDesc);
	
	if (err == errAEDescNotFound)
		targetObject = GetDefaultTarget();
	else
	{
		::ThrowIfOSErr_AC(err);
		if (objectDesc.GetDescriptorType() == typeInsertionLoc)
		{
			CTempDesc insertionDesc;

			::ThrowIfOSErr_AC(AECoerceDesc(objectDesc, typeAERecord, insertionDesc));
			::ThrowIfOSErr_AC(objectDesc.DisposeDesc());
			insertionDesc.GetKeyDesc(keyAEObject, typeWildCard, objectDesc);
			appleEvent->WriteParameter(keyDirectObject, objectDesc);
		}

		targetObject = ResolveObjectSpecifier(objectDesc);
	}
	return targetObject;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::ResolveObjectSpecifier : Ask the OSL to resolve an object specifier. 
// This will initiate a dialog between the OSL and our object resolution callbacks. 
//----------------------------------------------------------------------------------------


MScriptableObject* TOSADispatcher::ResolveObjectSpecifier(CAEDesc& 	objectDesc)
{
	MScriptableObject 	*result;
			
	if (objectDesc.GetDescriptorType() == typeObjectSpecifier)
	{
		CTempDesc 			tokenDesc;
		
		::ThrowIfOSErr_AC(Resolve_AC(objectDesc, fAEResolveFlags, tokenDesc));
		result = tokenDesc.GetObject();
	}
	else
		result = GetDefaultTarget();
	
	return result;
}

//----------------------------------------------------------------------------------------
// GetAppleEventCommandNumber: return the commandNumber associated with a specific
// event class and ID.
//----------------------------------------------------------------------------------------


CommandNumber TOSADispatcher::GetAppleEventCommandNumber(AEEventClass 	eventClass,
														AEEventID 		eventID)
{
	AEEventClass		localClass = eventClass;
	AEEventID			localID = eventID;
	OSErr				err;
	long				theCommandNumber = 0;
	AEEventHandlerUPP	eventHandler = NULL;
	
	err = GetEventHandler_AC(eventClass, eventID, &eventHandler, &theCommandNumber, false);
	if (err != noErr)
		theCommandNumber = 0;
		
	return theCommandNumber;
}


//----------------------------------------------------------------------------------------
// TOSADispatcher::ObjectAccessor : Handles the AccessContainedObjects OSL callback by
// asking the container object to return a reference to a specific contained object. If
// the object was created for object resolution, and requires disposal later, it is added
// to the temp tokens list.
//----------------------------------------------------------------------------------------


void TOSADispatcher::ObjectAccessor(DescType 	desiredClass,
									const CAEDesc& 	container,
									DescType	/* containerClass */,
									DescType 	form,
									const CAEDesc& 	selectionData,
									CAEDesc& 	value,
									long		/* accessorRefcon */)
{
	MScriptableObject *containerObject = container.GetObject();
	if (!containerObject)
		containerObject = fDefaultTarget;
		
	MScriptableObject *resultObject = containerObject->GetContainedObject(desiredClass, form, selectionData);
	if (!resultObject)
		::ThrowIfOSErr_AC(errAENoSuchObject);
	
	value.PutObject(resultObject);
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::CountObjects: Handle the OSL CountObjects callback by asking
// a container object to count the contained objects of a certain type.
//----------------------------------------------------------------------------------------


long TOSADispatcher::CountObjects(	DescType 	desiredType,
									DescType 	/*containerClass*/,
								   	const CAEDesc& 	container)
{
	return container.GetObject()->CountContainedObjects(desiredType);
} 

//----------------------------------------------------------------------------------------
// TOSADispatcher::CompareObjects: Handle the OSL CompareObjects callback by asking
// the target object to compare itself to another object.
//----------------------------------------------------------------------------------------


bool TOSADispatcher::CompareObjects(	DescType 	whichOperation,
										const CAEDesc& 	targetDesc,
										const CAEDesc& 	thingToCompare)
{
	return targetDesc.GetObject()->CompareScriptableObjects(whichOperation, thingToCompare);
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::DisposeToken: The OSL calls this method after object resolution but
// before the event is processed. After an event is processed, all temporary tokens
// are disposed.
//----------------------------------------------------------------------------------------


void TOSADispatcher::DisposeToken(CAEDesc& unneededToken)
{
	::ThrowIfOSErr_AC(unneededToken.DisposeDesc());
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::GetMarkToken: Before marking a set of objects, the OSL asks for a
// token to contain them.  MacApp provides a TScriptableObjectList.
//----------------------------------------------------------------------------------------


void TOSADispatcher::GetMarkToken(const CAEDesc& theContainerToken,
								   DescType	/*containerClass*/,
								   CAEDesc& result)
{
	MScriptableObject *containerObject = theContainerToken.GetObject();
	TScriptableObjectList *theMarkingList = containerObject->MakeNewMarkingList();
	::ThrowIfNonObject_AC(theMarkingList);
	result.PutObject(theMarkingList);
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::MarkObject: OSL is requesting the specified object be marked.
//----------------------------------------------------------------------------------------


void TOSADispatcher::MarkObject(CAEDesc& theToken,
								CAEDesc& markingListToken,
								long	/*index*/)
{
	TScriptableObjectList *theMarkingList = dynamic_cast_AC(TScriptableObjectList*,markingListToken.GetObject());
	MScriptableObject *objectToMark = theToken.GetObject();
	theMarkingList->InsertMarkedObject(objectToMark);
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::AdjustMarks: 
//----------------------------------------------------------------------------------------


void TOSADispatcher::AdjustMarks(long 		newStart,
								 long 		newStop,
								 CAEDesc& 	markToken)
{
	// Ask the TAppleObjectList marking list to adjust the items it contains.
	TScriptableObjectList *theMarkingList = dynamic_cast_AC(TScriptableObjectList*,markToken.GetObject());
	theMarkingList->AdjustMarks(newStart, newStop);
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::PostPendingReplyCommand: 
//----------------------------------------------------------------------------------------


void TOSADispatcher::PostPendingReplyCommand(TCommand* command)
{
	fPendingReplyList->Insert(command);
}


//----------------------------------------------------------------------------------------
// TOSADispatcher::MatchReplyToPending: 
//----------------------------------------------------------------------------------------


void TOSADispatcher::MatchReplyToPending(TAppleEvent* reply)
{
	TClientCommand * theMatchedCommand = NULL;

	// Higher priority events are at the end of the list.
	for (TemplateIterator_AC<TClientCommand*> iter(fPendingReplyList, kIterateBackward_AC); iter; ++iter)
		if (iter->GetReturnID() == reply->GetReturnID())
		{
			theMatchedCommand = iter;
			break;
		}

	if (theMatchedCommand)
	{
		fPendingReplyList->Delete(theMatchedCommand);
		theMatchedCommand->ProcessReply(reply);
		theMatchedCommand->GetContext()->PostCommand(theMatchedCommand);
	}
	else
		::ThrowIfOSErr_AC(errAEEventNotHandled);
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::GetTemporaryTokens
//----------------------------------------------------------------------------------------


CList_AC* TOSADispatcher::GetTemporaryTokens()
{
	return fTemporaryTokens;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::AddTemporaryToken: Insert an object into the temp tokens list.
// Temp tokens will be discarded after the target is resolved and the event is executed.
//----------------------------------------------------------------------------------------


void TOSADispatcher::AddTemporaryToken(MScriptableObject* tempToken)
{
	if (fTemporaryTokens->GetEqualItemNo(tempToken) == kEmptyIndex_AC)
		fTemporaryTokens->InsertLast(tempToken);
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::FreeTemporaryTokens: 
//----------------------------------------------------------------------------------------


void TOSADispatcher::FreeTemporaryTokens()
{
	// Frees all the objects in the temp token list.
	if (fTemporaryTokens && fDispatchLevel < 1)
	{
		try
		{
			for (TemplateIterator_AC<MScriptableObject*> iter(fTemporaryTokens); iter; ++iter)
				delete iter.Current();
			fTemporaryTokens->DeleteAll();
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
		}
	}
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TOSADispatcher::PreDispatchHandlerGlue : AppleScript does not send the refCon
// associated with the standard dispatch handler for the event.
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::PreDispatchHandlerGlue(AppleEvent* message,
													AppleEvent* reply,
													long		/* info */)
{
	OSErr dispatchErr = noErr;
	long dispatchErrMessage = 0;

	++TOSADispatcher::fgDispatcher->fDispatchLevel;

	try
	{
		TAppleEvent messageEvent(*message, false);
		TAppleEvent replyEvent(*reply, false);
		
		AEEventClass eventClass = messageEvent.ReadEventClass();
		AEEventID eventID = messageEvent.ReadEventID();
		CommandNumber eventCommand = TOSADispatcher::fgDispatcher->GetAppleEventCommandNumber(eventClass, eventID);
	
		dispatchErr = TOSADispatcher::fgDispatcher->PreDispatchHandler(eventCommand, &messageEvent, &replyEvent);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		dispatchErr = theException.GetError();				// Don't propogate the error.
		dispatchErrMessage = theException.GetMessage();	
	}
	
	if (dispatchErr != noErr && dispatchErr != errAEEventNotHandled)
		TOSADispatcher::fgDispatcher->ReportDispatchError(message, reply, dispatchErr, dispatchErrMessage);
		
	--TOSADispatcher::fgDispatcher->fDispatchLevel;
	TOSADispatcher::fgDispatcher->FreeTemporaryTokens();

	return dispatchErr;
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TOSADispatcher::DispatchHandlerGlue
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::DispatchHandlerGlue(AppleEvent* 	message,
												 AppleEvent* 	reply,
												 long			info)
{
	OSErr dispatchErr = noErr;
	long dispatchErrMessage =  0;
	bool matchedToPending = false;

	TAppleEvent* messageEvent = NULL;
	TAppleEvent* replyEvent = NULL;

	CommandNumber itsCommand = (CommandNumber)info;
	
	++TOSADispatcher::fgDispatcher->fDispatchLevel;

	try
	{
		messageEvent = TH_new TAppleEvent(*message, false);
		replyEvent = TH_new TAppleEvent(*reply, false);
		
		if (itsCommand == cAppleEventReply)		// handle incoming replies to client commands
		{
			TOSADispatcher::fgDispatcher->MatchReplyToPending(messageEvent);
			matchedToPending = true;
		}
		else
			dispatchErr = TOSADispatcher::fgDispatcher->DispatchHandler(itsCommand, messageEvent, replyEvent);
		
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		dispatchErr = theException.GetError();				// Do not propogate the error.
		dispatchErrMessage = theException.GetMessage();	
	}
	
	
	if (dispatchErr != noErr && dispatchErr != errAEEventNotHandled)
		TOSADispatcher::fgDispatcher->ReportDispatchError(message, reply, dispatchErr, dispatchErrMessage);

	--TOSADispatcher::fgDispatcher->fDispatchLevel;
	TOSADispatcher::fgDispatcher->FreeTemporaryTokens();
		
	// delete the message only if it wasn't matched to a pending command
	if (!matchedToPending)
		delete messageEvent;
	delete replyEvent;
	
	return dispatchErr;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::ObjectAccessorGlue
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::ObjectAccessorGlue(DescType 	desiredClass, 
												const CAEDesc& 	container, 
												DescType 	containerClass, 
												DescType 	form, 
												const CAEDesc& 	selectionData, 
												CAEDesc& 	value,
												long 		accessorRefcon)
{
	OSErr result = noErr;
	
	try
	{
		TOSADispatcher::fgDispatcher->ObjectAccessor(desiredClass, container, containerClass, 
				form, selectionData, value, accessorRefcon);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		result = theException.GetError();
	}
	
		
	return result;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::CountObjectsGlue
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::CountObjectsGlue(	DescType 	desiredType,
												DescType	containerClass,
												const CAEDesc&	container,
												long& 		result)
{
	OSErr err = noErr;
	
	try
	{
		result = TOSADispatcher::fgDispatcher->CountObjects(desiredType, containerClass, container);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		err = theException.GetError();
	}
	
		
	return err;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::CompareObjectsGlue
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::CompareObjectsGlue(DescType 	whichOperation,
												const CAEDesc& 	obj1,
												const CAEDesc& 	obj2,
												Boolean& 	result)
{
	OSErr err = noErr;
	
	try
	{
		result = TOSADispatcher::fgDispatcher->CompareObjects(whichOperation, obj1, obj2);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		err = theException.GetError();
	}
	
		
	return err;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::DisposeTokenGlue
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::DisposeTokenGlue(CAEDesc& unneededToken)
{
	OSErr result = noErr;
	
	try
	{
		TOSADispatcher::fgDispatcher->DisposeToken(unneededToken);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		result = theException.GetError();
	}
	
		
	return result;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::GetMarkTokenGlue
//----------------------------------------------------------------------------------------


 pascal OSErr TOSADispatcher::GetMarkTokenGlue(const CAEDesc& 	containerToken,
												DescType 	containerClass,
												CAEDesc& 	result)
{
	OSErr err = noErr;
	
	
	try
	{
		TOSADispatcher::fgDispatcher->GetMarkToken(containerToken, containerClass, result);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		err = theException.GetError();
	}
	
		
	return err;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::MarkObjectGlue
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::MarkObjectGlue(CAEDesc& dToken,
											CAEDesc& markToken,
											long 	index)
{
	OSErr result = noErr;
	

	try
	{
		TOSADispatcher::fgDispatcher->MarkObject(dToken, markToken, index);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		result = theException.GetError();
	}
	

	return result;
}

//----------------------------------------------------------------------------------------
// TOSADispatcher::AdjustMarksGlue
//----------------------------------------------------------------------------------------


pascal OSErr TOSADispatcher::AdjustMarksGlue(long newStart,
											 long newStop,
											 CAEDesc& markToken)
{
	OSErr result = noErr;
	
	
	try
	{
		TOSADispatcher::fgDispatcher->AdjustMarks(newStart, newStop, markToken);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		result = theException.GetError();
	}
	

	return result;
}

//==================================================================================================
// Class TSetPropertyCommand
//==================================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TSetPropertyCommand::TSetPropertyCommand
//----------------------------------------------------------------------------------------

TSetPropertyCommand::TSetPropertyCommand(DescType theProperty,
											  CList_AC* theObjectsToSet,
											  TAppleEvent* message,
											  TAppleEvent* /* reply */) :
	fProperty(theProperty),
	fOldPropData(),
	fNewPropData()
{
	// We take charge of the temporary objects in the disposal list so we can use them
	// during execution of this command.
	// We'll free these when we commit.
	CList_AC * theDisposalList = TOSADispatcher::fgDispatcher->GetTemporaryTokens();
	fObjectsToFree = static_cast<CList_AC*>(theDisposalList->Clone());
	theDisposalList->DeleteAll();
	// We'll clone the list so the caller can free it immediately.
	fObjectsToSet = static_cast<CList_AC*>(theObjectsToSet->Clone());
	CommandNumber cmdNum;
	bool canUndo;
	bool causesChange;
	MScriptableObject * theFirstObject = (MScriptableObject*)theObjectsToSet->At(1);
	::ThrowIfNULL_AC(theFirstObject);

	// Get the info for this command from the object.
	TCommandHandler* itsContext = NULL;
	theFirstObject->GetSetPropertyInfo(theProperty, cmdNum, canUndo, causesChange, itsContext);

	fContext = itsContext;
	fObjectToNotify = itsContext;
	fCanUndo = canUndo;
	fCausesChange = causesChange;
	fIdentifier = cmdNum;

	fOldPropData = NULL;

	message->ReadParameter(keyAEData, typeWildCard, fNewPropData);
}

//----------------------------------------------------------------------------------------
// TSetPropertyCommand::~TSetPropertyCommand
//----------------------------------------------------------------------------------------


TSetPropertyCommand::~TSetPropertyCommand()
{
}

//----------------------------------------------------------------------------------------
// TSetPropertyCommand::DoIt
//----------------------------------------------------------------------------------------


void TSetPropertyCommand::DoIt()
{
	CAEDesc theOldPropList;
	if (fCanUndo)
	{
		theOldPropList.CreateList();

		for (TemplateIterator_AC<MScriptableObject*> iter(fObjectsToSet); iter; ++iter)
		{
			CTempDesc thePropData;
			iter->GetObjectProperty(thePropData, fProperty, CAEDesc::fgNullDesc);
			::ThrowIfOSErr_AC(AEPutDesc(theOldPropList, 0, thePropData));
		}
	}
	fOldPropData = theOldPropList;
		
	RedoIt();
}

//----------------------------------------------------------------------------------------
// TSetPropertyCommand::UndoIt
//----------------------------------------------------------------------------------------


void TSetPropertyCommand::UndoIt()
{
	CAEDesc theOldPropData(fOldPropData);
	::Assertion_AC(theOldPropData.GetDataHandle() != NULL, "Trying to undo with no data!");

	short listIndex = 1;

	for (TemplateIterator_AC<MScriptableObject*> iter(fObjectsToSet); iter; ++iter)
	{
		CTempDesc theOldPropDesc;
		DescType theActualKey;
		::ThrowIfOSErr_AC(AEGetNthDesc(theOldPropData, listIndex++, typeWildCard, &theActualKey, theOldPropDesc));

		iter->SetObjectProperty(theOldPropDesc, fProperty);
	}
}

//----------------------------------------------------------------------------------------
// TSetPropertyCommand::RedoIt
//----------------------------------------------------------------------------------------


void TSetPropertyCommand::RedoIt()
{
	CAEDesc thePropData(fNewPropData);
	for (TemplateIterator_AC<MScriptableObject*> iter(fObjectsToSet); iter; ++iter)
		iter->SetObjectProperty(thePropData, fProperty);
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor
//----------------------------------------------------------------------------------------
#undef Inherited


MA_DEFINE_CLONE(TPropertyAccessor);

//----------------------------------------------------------------------------------------
// TPropertyAccessor::TPropertyAccessor
//----------------------------------------------------------------------------------------

TPropertyAccessor::TPropertyAccessor(MScriptableObject* whichObject,
										  DescType whichProperty) :
	MScriptableObject(cProperty),

	fWhichObject(whichObject),
	fWhichProperty(whichProperty)
{
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor destructor
//----------------------------------------------------------------------------------------


TPropertyAccessor::~TPropertyAccessor()
{
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor::GetCommandContext
//----------------------------------------------------------------------------------------


TCommandHandler* TPropertyAccessor::GetCommandContext(const CommandNumber aCommandNumber) const
{
	TCommandHandler *commandHandler = NULL;
	
	if (fWhichObject)
	{	
		commandHandler = dynamic_cast<TCommandHandler*>(fWhichObject);
		if (commandHandler)
			commandHandler = commandHandler->GetContext(aCommandNumber);
	}
	
	if (!commandHandler)
		commandHandler = MScriptableObject::GetCommandContext(aCommandNumber);
	
	return commandHandler;
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor::DoScriptCommand
//----------------------------------------------------------------------------------------


void TPropertyAccessor::DoScriptCommand(CommandNumber 	aCommandNumber,
										TAppleEvent*	message,
								   		TAppleEvent* 	reply)
{
	// Handles GetData and SetData for this property.
	try
	{
		switch (aCommandNumber)
		{
			case cAEGetData:
				{
					CTempDesc requestedType;
					if (message->HasParameter(keyAERequestedType))
						message->ReadParameter(keyAERequestedType, typeWildCard, requestedType);
					
					CTempDesc thePropertyValue;
					fWhichObject->GetObjectProperty(thePropertyValue, fWhichProperty, requestedType);
					reply->WriteParameter(keyAEResult, thePropertyValue);
					break;
				}
			case cAESetData:
				{
					CList_AC aOneObjectList;

					aOneObjectList.InsertLast(fWhichObject);
					TSetPropertyCommand * theCmd = TH_new TSetPropertyCommand(fWhichProperty, &aOneObjectList, message, reply);

					theCmd->Process();
					break;
				}
			default:
				MScriptableObject::DoScriptCommand(aCommandNumber, message, reply);
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		reply->WriteOSError(theException.GetError());
	}
	
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor::CompareScriptableObjects
//----------------------------------------------------------------------------------------


bool TPropertyAccessor::CompareScriptableObjects(DescType operation,
											   const CAEDesc& thingToCompare)
{
	// Compares the properties.
	return fWhichObject->CompareProperties(fWhichProperty, operation, thingToCompare);
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor::SetObjectProperty
//----------------------------------------------------------------------------------------


void TPropertyAccessor::SetObjectProperty(const CAEDesc& thePropertyValue,
										   DescType whichProperty)
{
	fWhichObject->SetObjectProperty(thePropertyValue, whichProperty);
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor::GetObjectProperty
//----------------------------------------------------------------------------------------


bool TPropertyAccessor::GetObjectProperty(CAEDesc& thePropertyValue,
											 DescType whichProperty,
											 const CAEDesc& desiredType)
{
	return fWhichObject->GetObjectProperty(thePropertyValue, whichProperty, desiredType);
}

//----------------------------------------------------------------------------------------
// TPropertyAccessor::DoAEGetDataSize
//----------------------------------------------------------------------------------------


void TPropertyAccessor::DoAEGetDataSize(TAppleEvent* message,
										TAppleEvent* reply)
{
	CTempDesc requestedType;
	if (message->HasParameter(keyAERequestedType))
		message->ReadParameter(keyAERequestedType, typeWildCard, requestedType);
	
	CTempDesc thePropertyValue;
	fWhichObject->GetObjectProperty(thePropertyValue, fWhichProperty, CAEDesc::fgNullDesc);
	long theSize = thePropertyValue.GetDataSize();
	reply->WriteLong(keyAEResult, theSize);
}

#if qAttachable

//----------------------------------------------------------------------------------------
// TPropertyAccessor::HandleOSAEvent:
//----------------------------------------------------------------------------------------


bool TPropertyAccessor::HandleOSAEvent(	CommandNumber 	aCommandNumber,
											TAppleEvent* 	message,
											TAppleEvent* 	reply)
{
	message->WriteType(keyAEProperty, fWhichProperty);
	return fWhichObject->HandleOSAEvent(aCommandNumber, message, reply);
}

#endif // qAttachable

//----------------------------------------------------------------------------------------
// TScriptableObjectList
//----------------------------------------------------------------------------------------
#undef Inherited
#define Inherited CList_AC


MA_DEFINE_CLASS(TScriptableObjectList);

//----------------------------------------------------------------------------------------
// TScriptableObjectList::TScriptableObjectList
//----------------------------------------------------------------------------------------

TScriptableObjectList::TScriptableObjectList(MScriptableObject* itsContainer) :
MScriptableObject(cScriptableObjectList),
CList_AC(),

fContainer(itsContainer)
{
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList destructor
//----------------------------------------------------------------------------------------


TScriptableObjectList::~TScriptableObjectList()
{
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::DoAECountElements
//----------------------------------------------------------------------------------------


void TScriptableObjectList::DoAECountElements(TAppleEvent* /*message*/,
										 TAppleEvent* reply)
{
	// If we say "get number of (windows whose name contains "untitled")" OSL builds a
	// TAppleObjects list of TWindows that satisfies the whose test then calls this method.
	reply->WriteLong(keyAEResult, GetSize());
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::DoScriptCommand
//----------------------------------------------------------------------------------------


void TScriptableObjectList::DoScriptCommand(CommandNumber	aCommandNumber,
											TAppleEvent* 	message,
								   			TAppleEvent* 	reply)
{
	if (aCommandNumber == cAECountElements)
		MScriptableObject::DoScriptCommand(aCommandNumber, message, reply);
	else
	{
		CTempDesc theReplyList;
		theReplyList.CreateList();
		// Calls DoScriptCommand for all the objects in the list.
		for (TemplateIterator_AC<MScriptableObject*> iter(this); iter; ++iter)
		{
			iter.Current()->DoScriptCommand(aCommandNumber, message, reply);
			CTempDesc theEventReply;
			if (reply->HasParameter(keyAEResult))
			{
				reply->ReadParameter(keyAEResult, typeWildCard, theEventReply);
				::ThrowIfOSErr_AC(AEPutDesc(theReplyList, 0, theEventReply));
			}
		}
		if (reply->dataHandle)
			reply->WriteParameter(keyAEResult, theReplyList);
	}
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::InsertMarkedObject
//----------------------------------------------------------------------------------------


void TScriptableObjectList::InsertMarkedObject(MScriptableObject* theObjectMarked)
{
	// Adds the marked object to the list
	InsertLast(theObjectMarked);
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::AdjustMarks
//----------------------------------------------------------------------------------------


void TScriptableObjectList::AdjustMarks(long newStart,
								   long newStop)
{
	// The OSL may occasionally change its mind about which objects should be marked.
	// This routine makes the adjustments.
	long numToDelete = GetSize() - newStop;
	if (numToDelete > 0)
		DeleteElementsAt(newStop + 1, numToDelete);
	numToDelete = newStart - 1;
	if (numToDelete > 0)
		DeleteElementsAt(1, numToDelete);
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::GetObjectsContainer
//----------------------------------------------------------------------------------------


MScriptableObject* TScriptableObjectList::GetObjectsContainer()
{
	return fContainer;
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::IsPropertyList
//----------------------------------------------------------------------------------------


bool TScriptableObjectList::IsPropertyList()
{
	// How do we know if this is a list of properties and not of objects?
	// We could walk down the list and make sure each element was a TPropertyAccessor,
	// but that adds overhead for large lists.
	// I haven't been able to think of a situation where objects and properties are operated
	// on at the same time and would be in the same list, so lets just check the first one.
	// I believe that even complex object specifiers can only refer to objects of the same class
	// and in the same container.

	MScriptableObject * theFirstObject = NULL;
	if (fSize > 0)
		theFirstObject = (MScriptableObject *)At(1);
	return theFirstObject && theFirstObject->GetOMClass() == cProperty;
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::GetContainedObject
//----------------------------------------------------------------------------------------


MScriptableObject* TScriptableObjectList::GetContainedObject(DescType desiredType,
											  DescType selectionForm,
											  const CAEDesc& selectionData)
{
	// This method is needed when a complex object specifier has resolved into a list of objects,
	// but then actually wants a property from each one.
	// We handle this by building up a list of contained objects from each object in this list.

	CAutoPtr_AC<TScriptableObjectList> theNewList(TH_new TScriptableObjectList(GetObjectsContainer()));

	for (TemplateIterator_AC<MScriptableObject*> iter(this); iter; ++iter)
	{
		MScriptableObject *theResult = iter->GetContainedObject(desiredType, selectionForm, selectionData);
		if (!theResult)
			::ThrowIfOSErr_AC(errAENoSuchObject);
		theNewList->InsertLast(theResult);
	}
	TOSADispatcher::fgDispatcher->AddTemporaryToken(theNewList);

	return theNewList.release();
}

//----------------------------------------------------------------------------------------
// TScriptableObjectList::CountContainedObjects
//----------------------------------------------------------------------------------------


long TScriptableObjectList::CountContainedObjects(DescType desiredType)
// Adds up CountContainedObjects for all objects in the list.
{
	long result = 0;

	for (TemplateIterator_AC<MScriptableObject*> iter(this); iter; ++iter)
		result += iter->CountContainedObjects(desiredType);

	return result;
}

//----------------------------------------------------------------------------------------
// TSetPropertyEvent
//----------------------------------------------------------------------------------------
#undef Inherited
#define Inherited TAppleEvent

//----------------------------------------------------------------------------------------
// TSetPropertyEvent::TSetPropertyEvent
//----------------------------------------------------------------------------------------


TSetPropertyEvent::TSetPropertyEvent(const CAEDesc& itsAddress,
										  long itsSendingMode,
										  MScriptableObject* objectToSet,
										  DescType propType) :
TAppleEvent(kAECoreSuite, kAESetData, (AEAddressDesc&)CAEDesc(itsAddress), itsSendingMode)
{
	// Initializes the event with the recipient, sending mode, the object you want to set and the 
	// property you want to modify. Creates an object specifier for the object's property and 
	// sticks it in the direct object.

	CAEDesc objectSpecifier;
	CAEDesc theProperty;
	objectToSet->MakeObjectSpecifier(objectSpecifier, objectToSet->GetSpecifierForm());

	theProperty.PutType(propType);
	CTempDesc propertyToSet;
	::ThrowIfOSErr_AC(CreateObjSpecifier(cProperty, objectSpecifier, formPropertyID, theProperty, true, propertyToSet));
	WriteParameter(keyDirectObject, propertyToSet);
}

//----------------------------------------------------------------------------------------
// TSetPropertyEvent destructor
//----------------------------------------------------------------------------------------


TSetPropertyEvent::~TSetPropertyEvent()
{
}

//----------------------------------------------------------------------------------------
// TMenuAccessor
//----------------------------------------------------------------------------------------
#undef Inherited

//----------------------------------------------------------------------------------------
// TMenuAccessor::TMenuAccessor
//----------------------------------------------------------------------------------------

TMenuAccessor::TMenuAccessor(MenuRef theMenu, short theIndex) :
	MScriptableObject(cMenu),

	fMenu(theMenu),
	fIndex(theIndex)
{
}

//----------------------------------------------------------------------------------------
// TMenuAccessor destructor
//----------------------------------------------------------------------------------------


TMenuAccessor::~TMenuAccessor()
{
}

//----------------------------------------------------------------------------------------
// TMenuAccessor::GetObjectProperty
//----------------------------------------------------------------------------------------


bool TMenuAccessor::GetObjectProperty(CAEDesc& thePropertyValue,
										 DescType whichProperty,
										 const CAEDesc& desiredType)
{
	bool hasProperty = true;

	try
	{
		switch (whichProperty)
		{
	
			case pName:
				{
					CStr255_AC theMenuName = (CStr255_AC&)(**fMenu).menuData;
					thePropertyValue.PutString(theMenuName);
				}
				break;
				
			case pIndex:
				{
					thePropertyValue.PutLong(fIndex);
				}
				break;
	
			case pMenuID:
				{
					thePropertyValue.PutLong(static_cast<long>((*fMenu)->menuID));
				}
				break;
	
			default:
				hasProperty = MScriptableObject::GetObjectProperty(thePropertyValue, whichProperty, desiredType);
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		hasProperty = false;
	}
	

	return hasProperty;
}

//----------------------------------------------------------------------------------------
// TMenuAccessor::SetObjectProperty: 
//----------------------------------------------------------------------------------------


void TMenuAccessor::SetObjectProperty(const CAEDesc& thePropertyValue,
									  DescType whichProperty)
{
	switch (whichProperty)
	{
		case pName:
		case pMenuID:
			::ThrowIfOSErr_AC(errAECantSetReadOnly);
			break;

		default:
			MScriptableObject::SetObjectProperty(thePropertyValue, whichProperty);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TMenuAccessor::GetContainedObject
//----------------------------------------------------------------------------------------


MScriptableObject* TMenuAccessor::GetIndContainedObject(DescType desiredType,
													   long index)
{
	MScriptableObject* returnVal = NULL;

	if (desiredType == cMenuItem)
	{
		// Create a menu item accessor to handle events for this menu item
		TMenuItemAccessor* theAccessor = TH_new TMenuItemAccessor(fMenu, static_cast<short>(index));
		TOSADispatcher::fgDispatcher->AddTemporaryToken(theAccessor);
		returnVal = theAccessor;
	}
	else
		returnVal = MScriptableObject::GetIndContainedObject(desiredType, index);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMenuAccessor::CountContainedObjects
//----------------------------------------------------------------------------------------


long TMenuAccessor::CountContainedObjects(DescType desiredType)
	// Returns the number of menu item in a menu.
{
	return desiredType == cMenuItem ? CountMItems(fMenu) : MScriptableObject::CountContainedObjects(desiredType);
}

//----------------------------------------------------------------------------------------
// TMenuItemAccessor
//----------------------------------------------------------------------------------------
#undef Inherited

//----------------------------------------------------------------------------------------
// TMenuItemAccessor::TMenuItemAccessor
//----------------------------------------------------------------------------------------

TMenuItemAccessor::TMenuItemAccessor(MenuRef theMenu, short theItem) :
	MScriptableObject(cMenuItem),

	fMenu(theMenu),
	fMenuItem(theItem)
{
}

//----------------------------------------------------------------------------------------
// TMenuItemAccessor destructor
//----------------------------------------------------------------------------------------


TMenuItemAccessor::~TMenuItemAccessor()
{
}

//----------------------------------------------------------------------------------------
// TMenuItemAccessor::GetObjectProperty
//----------------------------------------------------------------------------------------


bool TMenuItemAccessor::GetObjectProperty(CAEDesc& thePropertyValue,
											 DescType whichProperty,
											 const CAEDesc& desiredType)
{
	bool hasProperty = true;

	try
	{
		switch (whichProperty)
		{
	
			case pName:
				{
					CStr255_AC theItemName;
					GetMenuItemText(fMenu, fMenuItem, theItemName);
					thePropertyValue.PutString(theItemName);
				}
				break;
				
			case pItemNumber:
			case pIndex:
				thePropertyValue.PutLong(fMenuItem);
				break;
	
			case pEnabled:
				thePropertyValue.PutBoolean(fMenuItem > 0 && fMenuItem < 32 ? (((*fMenu)->enableFlags >> fMenuItem) & 0x01) != 0 : true);
				break;
	
			default:
				hasProperty = MScriptableObject::GetObjectProperty(thePropertyValue, whichProperty, desiredType);
				break;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		hasProperty = false;
	}
	

	return hasProperty;
}

//----------------------------------------------------------------------------------------
// TMenuItemAccessor::SetObjectProperty: 
//----------------------------------------------------------------------------------------


void TMenuItemAccessor::SetObjectProperty(const CAEDesc& thePropertyValue,
									  DescType whichProperty)
{
	switch (whichProperty)
	{
		case pName:
		case pItemNumber:
		case pEnabled:
			::ThrowIfOSErr_AC(errAECantSetReadOnly);
			break;

		default:
			MScriptableObject::SetObjectProperty(thePropertyValue, whichProperty);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TMenuItemAccessor::DoAEOpen
//----------------------------------------------------------------------------------------


void TMenuItemAccessor::DoAEOpen(TAppleEvent*	/* message */ ,
								 TAppleEvent* /* reply */)
{
//	gDispatcher->MenuEvent((static_cast<long>((*fMenu)->menuID) << 16) | fMenuItem);
	gDispatcher->MenuEvent((*fMenu)->menuID, fMenuItem);
}

//----------------------------------------------------------------------------------------
// InitUScripting
//----------------------------------------------------------------------------------------
void InitUScripting()
{
	MA_REGISTER_CLASS(TOSADispatcher);

	TOSADispatcher::InitUScripting("TOSADispatcher");
}

//----------------------------------------------------------------------------------------
// End of UScripting.cp


