//----------------------------------------------------------------------------------------
// UScroller.cp 
// Copyright  1987-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

// ACS

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#if qDrag
#ifndef __UDRAGDROP__
#include "UDragDrop.h"
#endif
#endif

#ifndef __UEVENT__
#include "UEvent.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __CONTROLS__
#include <Controls.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

// ANSI


#ifndef __STDLIB__
#include <stdlib.h>
#endif


//========================================================================================
// GLOBAL Procedures
//========================================================================================
static ViewCoordinate ProportionalScroll(ViewCoordinate diff,
									  ViewCoordinate scrollUnit,
									  ViewCoordinate maxTranslationchange);

#undef Inherited

//----------------------------------------------------------------------------------------
// ProportionalScroll: 
//----------------------------------------------------------------------------------------


ViewCoordinate ProportionalScroll(ViewCoordinate diff,
							   ViewCoordinate scrollUnit,
							   ViewCoordinate maxTranslationchange)
{
	ViewCoordinate returnVal = 0;
	if (scrollUnit)
		returnVal = MinMax_AC(scrollUnit, (((diff + (scrollUnit / 2)) / scrollUnit) * scrollUnit), maxTranslationchange);

	return returnVal;
}


//========================================================================================
// CLASS TScroller
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TScroller);

//----------------------------------------------------------------------------------------
// TScroller constructor
//----------------------------------------------------------------------------------------


TScroller::TScroller(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
								 const CViewPoint& itsMax,
								 bool wantHorzSBar,
								 bool wantVertSBar) : 
	TView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fScrollUnit(kStdScrollUnit, kStdScrollUnit),
	fRespondsToFunctionKeys(true)
{
	SetOpacity(true);

	fScrollBarIdentifiers[hSel] = kNoIdentifier;
	fScrollBarIdentifiers[vSel] = kNoIdentifier;
	fConstrain[hSel] = false;
	fConstrain[vSel] = false;

	SetScrollParameters(CViewPoint(kStdScrollUnit, kStdScrollUnit), false, false);

	SetScrollLimits(itsMax, kDontRedraw);

	if (wantHorzSBar)
		CreateScrollBar(hSel);
	if (wantVertSBar)
		CreateScrollBar(vSel);

	if (wantHorzSBar || wantVertSBar)
		AdjustScrollBars(kDontInvalidate);
}

//----------------------------------------------------------------------------------------
// TScroller copy constructor
//----------------------------------------------------------------------------------------


TScroller::TScroller(const TScroller& inScroller) :
	TView(inScroller),

	fSBarOffsets(inScroller.fSBarOffsets),
	fScrollLimit(inScroller.fScrollLimit),
	fMaxTranslation(inScroller.fMaxTranslation),
	fScrollUnit(inScroller.fScrollUnit),
	fRespondsToFunctionKeys(inScroller.fRespondsToFunctionKeys)
{
	fScrollBarIdentifiers[hSel] = inScroller.fScrollBarIdentifiers[hSel];
	fScrollBarIdentifiers[vSel] = inScroller.fScrollBarIdentifiers[vSel];
	fConstrain[hSel] = inScroller.fConstrain[hSel];
	fConstrain[vSel] = inScroller.fConstrain[vSel];

	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		if (inScroller.fScrollBars[vhs])
		{
			fScrollBars[vhs] = static_cast<TScrollerScrollBar*>(inScroller.fScrollBars[vhs]->Clone());
			fScrollBars[vhs]->AttachScroller(this);
		}
}

//----------------------------------------------------------------------------------------
// TScroller::Free: 
//----------------------------------------------------------------------------------------


TScroller::~TScroller()
{
}

//----------------------------------------------------------------------------------------
// TScroller::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TScroller::GetStandardSignature() const	// override 
{
	return kStdScroller;
}

//----------------------------------------------------------------------------------------
// TScroller::ReadFields: 
//----------------------------------------------------------------------------------------


void TScroller::ReadFields(CStream_AC* aStream)				// override 
{
	Inherited::ReadFields(aStream);
	
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		*aStream >> fScrollBarIdentifiers[vhs];

	CViewPoint scrollLimit;
	CViewPoint scrollUnits;
	Boolean horzConstraint, vertConstraint; // must be boolean b/c data in stream is boolean
	Boolean respondsToFunctionKeys; // must be boolean b/c data in stream is boolean
	
	*aStream 	>> scrollLimit 
				>> scrollUnits 
				>> vertConstraint 
				>> horzConstraint
				>> fSBarOffsets
				>> respondsToFunctionKeys;
	fRespondsToFunctionKeys = respondsToFunctionKeys;

	SetScrollLimits(scrollLimit, kDontRedraw);
	SetScrollParameters(scrollUnits, horzConstraint, vertConstraint);
}

//----------------------------------------------------------------------------------------
// TScroller::WriteFields: 
//----------------------------------------------------------------------------------------


void TScroller::WriteFields(CStream_AC* aStream) const		// override 
{
	VHSelect vhs;

	Inherited::WriteFields(aStream);

	for (vhs = vSel; vhs <= hSel; ++vhs)
		*aStream << fScrollBarIdentifiers[vhs];
	
	*aStream 	<< fScrollLimit	// write scroll limits
				<< fScrollUnit; // write scroll units

	for (vhs = vSel; vhs <= hSel; ++vhs)
		*aStream << fConstrain[vhs];

	*aStream 	<< fSBarOffsets	// write scrollbar offsets
				<< static_cast<Boolean>(fRespondsToFunctionKeys);
}

//----------------------------------------------------------------------------------------
// TScroller::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TScroller::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	bool needsAdjustScrollBar = false;

	if (fSuperView)
		for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		{
			// Case 1: scroll bar was created from a resource
			if ((fScrollBarIdentifiers[vhs] != kNoIdentifier) && !fScrollBars[vhs])
			{
				TScrollerScrollBar*	aScrollBar = dynamic_cast<TScrollerScrollBar*>(fSuperView->FindSubView(fScrollBarIdentifiers[vhs]));
				if (aScrollBar)
				{
					aScrollBar->AttachScroller(this);
					needsAdjustScrollBar = true;
				}
#if qDebugMsg
				else
				{
					IDType itsID = fScrollBarIdentifiers[vhs];
					ProgramReport_AC(false, "###In TScroller::DoPostCreate, can't find scrollbar: %.4s\n", &itsID);
				}
#endif
			}
			// Case 2: scroll bar was created procedurally (e.g. TViewServer::AddSubViewWithScroller)
			else if (fScrollBars[vhs] && (fScrollBars[vhs]->GetSuperView() == NULL))
			{
				// Add scrollbar to the superview as a subview
				fSuperView->AddSubView(fScrollBars[vhs]);
			}
		}
	
	AdjustScrollLimits(kDontRedraw);

	if (needsAdjustScrollBar)
		AdjustScrollBars(kDontInvalidate);
}

//----------------------------------------------------------------------------------------
// TScroller::AddedASubView: 
//----------------------------------------------------------------------------------------

void TScroller::AddedASubView(TView* theSubView)// override 
{
	Inherited::AddedASubView(theSubView);
	
	// if this scroller is the direct superview of the subview
	// accept ownership and recalculate scroll limits
	// (we don't want any other scrollers doing this too)
	if (theSubView->GetSuperView() == this)
	{
		theSubView->BeInScroller(this);
		AdjustScrollLimits(kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TScroller::AdjustScrollLimits: 
//----------------------------------------------------------------------------------------

void TScroller::AdjustScrollLimits(bool drawScrollBars)
{
	CViewRect aggregateExtents;
	for (CSubViewIterator iter(this); iter; ++iter)
		if (iter->GetShown())
			aggregateExtents |= iter->GetFrame();		// union the new frame into the aggregate

	SetScrollLimits(aggregateExtents.GetSize(), drawScrollBars);
}

//----------------------------------------------------------------------------------------
// TScroller::AdjustScrollBars: 
//----------------------------------------------------------------------------------------

void TScroller::AdjustScrollBars(bool invalidate)
{
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		TScrollBar * aScrollBar = fScrollBars[vhs];
		if (aScrollBar)
		{
			VHSelect ortho = gOrthogonal[vhs];
			CViewPoint loc = fLocation;
			CViewPoint size = fSize;

			loc[vhs] += fSBarOffsets[topLeft][vhs] - 1;
			loc[ortho] += size[ortho];

			size[vhs] += fSBarOffsets[botRight][vhs] - fSBarOffsets[topLeft][vhs] + 2;
			size[ortho] = aScrollBar->GetSize()[ortho];		// This does not change the width of the scroll bar as created.

			aScrollBar->SetFrame(CViewRect(loc, loc + size), invalidate);
		}
	}
}

//----------------------------------------------------------------------------------------
// TScroller::AutoScroll: 
//----------------------------------------------------------------------------------------

void TScroller::AutoScroll(const CViewPoint& viewPt, CViewPoint& delta)
{
	delta.Clear();
	CViewRect myExtent(GetExtent());

	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		if (viewPt[vhs] < myExtent[topLeft][vhs])
			delta[vhs] = -ProportionalScroll(myExtent[topLeft][vhs] - viewPt[vhs], fScrollUnit[vhs], fTranslation[vhs]);
		else if (viewPt[vhs] > myExtent[botRight][vhs])
			delta[vhs] = ProportionalScroll(viewPt[vhs] - myExtent[botRight][vhs], fScrollUnit[vhs], fMaxTranslation[vhs] - fTranslation[vhs]);
}

//----------------------------------------------------------------------------------------
// TScroller::CreateScrollBar: 
//----------------------------------------------------------------------------------------

void TScroller::CreateScrollBar(VHSelect itsDirection)
{
	TScrollerScrollBar * aScrollerScrollBar = TH_new TScrollerScrollBar(gZeroVPt,
		CViewPoint(kSBarSize,kSBarSize), sizeVariable, sizeVariable, itsDirection,
		fMaxTranslation[itsDirection], this);
	if (fSuperView)
		fSuperView->AddSubView(aScrollerScrollBar);
	aScrollerScrollBar->HandlePostCreate(fDocument); //SRF new requirement
}

//----------------------------------------------------------------------------------------
// TScroller::DoKeyEvent: 
//----------------------------------------------------------------------------------------

void TScroller::DoKeyEvent(TToolboxEvent* event)// override 
{
	bool wasHandled = false;

	if (IsEnabled() && fRespondsToFunctionKeys)
	{
		wasHandled = true;
		if (event->GetText() == chPageUp)
		{
			ViewCoordinate vCoord = ScrollStep(vSel, kControlPageUpPart);
			if (fScrollBars[vSel])
				fScrollBars[vSel]->DeltaValue(ViewCoordinate(vCoord));
		}
		else if (event->GetText() == chPageDown)
		{
			ViewCoordinate vCoord = ScrollStep(vSel, kControlPageDownPart);
			if (fScrollBars[vSel])
				fScrollBars[vSel]->DeltaValue(ViewCoordinate(vCoord));
		}
		else if (event->GetText() == chHome)
			SetLocalOrigin(gZeroVPt, kRedraw);
		else if (event->GetText() == chEnd)
			SetLocalOrigin(CViewPoint(0, fMaxTranslation.v), kRedraw);
		else
			wasHandled = false;
	}

	if (wasHandled)
		Update();
	else
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TScroller::DoScroll: 
//----------------------------------------------------------------------------------------

void TScroller::DoScroll(const CViewPoint& delta,
						bool redraw)
{
	CViewPoint localDelta(delta);

	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		if (localDelta[vhs] < 0)
			localDelta[vhs] = Max_AC(localDelta[vhs], -fTranslation[vhs]);
		else if (localDelta[vhs] > 0)
			localDelta[vhs] = Min_AC(localDelta[vhs], (fMaxTranslation[vhs] - fTranslation[vhs]));
		fTranslation[vhs] += localDelta[vhs];
	}

	if (localDelta != gZeroVPt)
	{
		InvalidateFocus();
		InvalidateCoordinates();

		if (redraw)
			ScrollDraw(localDelta, kDontInvalidate); // Don't invalidate means "draw don't invalidate"
	}
}

//----------------------------------------------------------------------------------------
// TScroller::ForceRedraw: 
//----------------------------------------------------------------------------------------

void TScroller::ForceRedraw()		// override 
{
	Inherited::ForceRedraw();
	
	if (fScrollBars[hSel])
		fScrollBars[hSel]->ForceRedraw();
	if (fScrollBars[vSel])
		fScrollBars[vSel]->ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TScroller::GetScroller: 
//----------------------------------------------------------------------------------------

TScroller* TScroller::GetScroller(bool)
{
	return this;
}

//----------------------------------------------------------------------------------------
// TScroller::Draw: 
//----------------------------------------------------------------------------------------
void TScroller::Draw(const CViewRect& area)// override 
{
	::EraseRect(&ViewToQD(area));
}

//----------------------------------------------------------------------------------------
// TScroller::HaveScrollBar: 
//----------------------------------------------------------------------------------------

void TScroller::HaveScrollBar(TScrollerScrollBar* theScrollBar,
									 VHSelect direction)
{
#if qDebug
	if ((theScrollBar) && (theScrollBar->fDirection != direction))
		::ProgramBreak_AC("Scroll bar is wrong direction.");
#endif
	fScrollBars[direction].release(); // just abandon the current scrollbar
	fScrollBars[direction] = theScrollBar;
}

//----------------------------------------------------------------------------------------
// TScroller::RemovedASubView: 
//----------------------------------------------------------------------------------------

void TScroller::RemovedASubView(TView* theSubView)// override 
{
	Inherited::RemovedASubView(theSubView);
	
	if (theSubView->GetSuperView() == this)
	{
		theSubView->BeInScroller(NULL);
		AdjustScrollLimits(kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TScroller::SetFrame: 
//----------------------------------------------------------------------------------------

void TScroller::SetFrame(const CViewRect& newFrame, bool invalidate)// override 

{
	CViewPoint oldSize(fSize);
	Inherited::SetFrame(newFrame, invalidate);
	
	AdjustScrollBars(invalidate);

	if (oldSize != fSize)
		SetScrollLimits(fScrollLimit, kDontRedraw);		// Readjust sbar maximums and fMaxTranslation
}

//----------------------------------------------------------------------------------------
// TScroller::RevealRect: 
//----------------------------------------------------------------------------------------

void TScroller::RevealRect(const CViewRect& rectToReveal,
						   const CViewPoint& minToSee,
						   bool redraw)// override 
{
	CViewRect myExtent(GetExtent());

	CViewPoint delta;
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		ViewCoordinate minAmt = Min_AC(myExtent.GetLength(vhs), minToSee[vhs]);
		ViewCoordinate coord = rectToReveal[topLeft][vhs] + minAmt - myExtent[botRight][vhs];
		if (coord <= 0)
			coord = Min_AC(0, rectToReveal[botRight][vhs] - minAmt - myExtent[topLeft][vhs]);
		delta[vhs] = coord;
	}

	ScrollBy(delta, redraw);
	Inherited::RevealRect(rectToReveal, minToSee, redraw);
}

//----------------------------------------------------------------------------------------
// TScroller::ScrollBy: 
//----------------------------------------------------------------------------------------

void TScroller::ScrollBy(const CViewPoint& delta,
								bool redraw)
{
	if (delta != gZeroVPt)
	{
		if (fScrollBars[vSel])
			fScrollBars[vSel]->DeltaValue(delta.v);
			
		if (fScrollBars[hSel])
			fScrollBars[hSel]->DeltaValue(delta.h);

		DoScroll(delta, redraw);
	}
}

//----------------------------------------------------------------------------------------
// TScroller::ScrollDraw: 
//----------------------------------------------------------------------------------------

void TScroller::ScrollDraw(const CViewPoint& delta,
						   bool invalidate)
{
	if (!GetVisibleExtent().Empty() && IsShown() && Focus())
	{
		CRect_AC visRect((*GetClipRegion(qd.thePort))->rgnBBox); // just test the clip, it's faster

#if qDebug
		if (gIntenseDebugging)
			WriteFocus();
#endif
		if (!visRect.Empty())
		{
		    const CViewRect visVRect(visRect);
		    if (((visVRect - delta) & visVRect).Empty()) // too far to scrollrect 
				Invalidate(visRect);
			else
			{
				CTempRegion_AC tempRgn;

				ScrollRect(visRect, static_cast<short>(-delta.h), static_cast<short>(-delta.v), tempRgn);
				Invalidate(tempRgn);
			}
		
			if (!invalidate)
				Update();
		}
	}
}

//----------------------------------------------------------------------------------------
// TScroller::ScrollRelative: 
//----------------------------------------------------------------------------------------

ViewCoordinate TScroller::ScrollRelative(VHSelect vhs,
										 ViewCoordinate sBarValue)
{
	ViewCoordinate newValue;

	if ((fConstrain[vhs] && (sBarValue != fMaxTranslation[vhs])))
		newValue = (sBarValue + fScrollUnit[vhs] / 2) / fScrollUnit[vhs] * fScrollUnit[vhs];
	else
		newValue = sBarValue;

	CViewPoint pixelDelta;
	pixelDelta[vhs] = newValue - fTranslation[vhs];
	DoScroll(pixelDelta, kRedraw);
	return (newValue - sBarValue);
}

//----------------------------------------------------------------------------------------
// TScroller::ScrollStep: 
//----------------------------------------------------------------------------------------

ViewCoordinate TScroller::ScrollStep(VHSelect vhs,
										 short partCode)
{
	ViewCoordinate scrollUnit = fScrollUnit[vhs];
	ViewCoordinate delta;

	switch (partCode)
	{
		case kControlUpButtonPart:
		case kControlDownButtonPart:
			delta = scrollUnit;
			break;
			
		case kControlPageUpPart:
		case kControlPageDownPart:
			delta = fSize[vhs] - scrollUnit;
			break;
			
#if qDebug
		default:
			{
				CStr255_AC theString;
				ConcatNumber("TScroller::ScrollStep: bad part code =", partCode, theString);
				::ProgramBreak_AC(theString);
			}
			break;
#endif

	}

	if ((partCode == kControlUpButtonPart) || (partCode == kControlPageUpPart))
		delta = -delta;

	// Constrain if necessary 
	if (fConstrain[vhs] && scrollUnit)
	{
		ViewCoordinate adjustment = (Max_AC(0, fTranslation[vhs] + delta)) % scrollUnit;
		if (adjustment)
			if (delta > 0)
				delta -= adjustment;
			else
				delta += (scrollUnit - adjustment);
	}

	CViewPoint deltaPt;
	deltaPt[vhs] = delta;

	DoScroll(deltaPt, kRedraw);
	return delta;
}

//----------------------------------------------------------------------------------------
// TScroller::SetLocalOrigin: 
//----------------------------------------------------------------------------------------

void TScroller::SetLocalOrigin(const CViewPoint& newOrigin, bool redraw)
{
	ScrollBy(newOrigin - fTranslation, redraw);
}

//----------------------------------------------------------------------------------------
// TScroller::SetScrollLimits: 
//----------------------------------------------------------------------------------------

void TScroller::SetScrollLimits(const CViewPoint& scrollLimit, bool drawScrollBars)
{
	CViewPoint newTranslation(fTranslation);

	fScrollLimit = scrollLimit;
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		ViewCoordinate maxCoord = Max_AC(0, scrollLimit[vhs] - fSize[vhs]);
		
		fMaxTranslation[vhs] = maxCoord;
		if (fScrollBars[vhs])
			fScrollBars[vhs]->SetLongMax(maxCoord, drawScrollBars);

		if (maxCoord < fTranslation[vhs])
			newTranslation[vhs] = maxCoord;
	}

	// This is very similar to ScrollBy except that the scrollbars
	// have to react in the opposite direction as the scrolling.
	CViewPoint		delta = newTranslation - fTranslation;
	if (delta != gZeroVPt)
	{
		if (fScrollBars[vSel])
			fScrollBars[vSel]->DeltaValue(-delta.v);
			
		if (fScrollBars[hSel])
			fScrollBars[hSel]->DeltaValue(-delta.h);
	
		DoScroll(delta, kRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TScroller::SetScrollParameters: 
//----------------------------------------------------------------------------------------

void TScroller::SetScrollParameters(const CViewPoint& units,
									bool horzConstraint,
									bool vertConstraint)
{
	fScrollUnit = units;
	fConstrain[hSel] = horzConstraint;
	fConstrain[vSel] = vertConstraint;
}

//----------------------------------------------------------------------------------------
// TScroller::SubViewChangedFrame: 
//----------------------------------------------------------------------------------------

void TScroller::SubViewChangedFrame(TView* ,
									const CViewRect&,
									const CViewRect&,
									bool invalidate)// override 
{
	AdjustScrollLimits(invalidate);
}

#if qDrag

//----------------------------------------------------------------------------------------
// TScroller::GetDragScrollDelta: 
//----------------------------------------------------------------------------------------

CViewPoint TScroller::GetDragScrollDelta(const CViewPoint& mouseInWindow)
{
	static const short kDragScrollRange = 16;			
	
	CViewPoint scrollDelta;
			
	CViewRect scrollerExtent(LocalToRootView(GetVisibleExtent()));
	
	VHSelect	vhSelect;
	for (vhSelect = vSel; vhSelect <= hSel; ++vhSelect)
	{
		if (fScrollBars[vhSelect])
		{
			CViewRect scrollBarExtent(LocalToRootView(fScrollBars[vhSelect]->GetVisibleExtent()));

			ViewCoordinate	firstDiff = scrollerExtent[topLeft][vhSelect] - scrollBarExtent[botRight][vhSelect];
			ViewCoordinate	secondDiff = scrollBarExtent[topLeft][vhSelect] - scrollerExtent[botRight][vhSelect];
			
			// if scroll bar is adjacent to the scroller, union it into the scroller extent
			if (firstDiff <= 1 || secondDiff <= 1)
				scrollerExtent |= scrollBarExtent;
		}
	}
					
	if (scrollerExtent.Contains(mouseInWindow))
	{
		CViewPoint 	extentDelta(kDragScrollRange, kDragScrollRange);
		
		scrollerExtent.Inset(extentDelta);		// inset the extent
		
		if (!scrollerExtent.Contains(mouseInWindow))
		{
			for (vhSelect = vSel; vhSelect <= hSel; ++vhSelect)
			{
				// adjust scroll delta. test scrolling in all 4 directions
				// if the scroller scrolls up AND down, it'll resolve to a
				// no vertical scroll. this handles overlapping opposing scroll areas
				if ((mouseInWindow[vhSelect] < scrollerExtent[topLeft][vhSelect]) && 
					(fTranslation[vhSelect] > 0))
						scrollDelta[vhSelect] = -fScrollUnit[vhSelect];
				
				if ((mouseInWindow[vhSelect] > scrollerExtent[botRight][vhSelect]) &&
					(fMaxTranslation[vhSelect] - fTranslation[vhSelect]) > 0)
						scrollDelta[vhSelect] = fScrollUnit[vhSelect];
			}
		}
	}
	return scrollDelta;
}
	
#endif

//----------------------------------------------------------------------------------------
// End of UScroller.cp 


