//----------------------------------------------------------------------------------------
// USpeakable.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __USPEAKABLE__
#include "USpeakable.h"
#endif

// num2dec
#ifndef __FP__
#include <FP.h>			
#endif

#ifndef __STRING__
#include <string.h>
#endif

//========================================================================================
// CLASS MSpeakable
//========================================================================================
//----------------------------------------------------------------------------------------
// MSpeakable::MSpeakable
//----------------------------------------------------------------------------------------
MSpeakable::MSpeakable()
	:	fSpeechRateModifier(1.0),
		fSpeechChannel(NULL)
{
	if (HasSpeechManager_AC())
	{
		//Passing NULL as the VoiceSpec will give the system default
		::ThrowIfOSErr_AC(NewSpeechChannel(NULL, &fSpeechChannel));
		::ThrowIfOSErr_AC(GetSpeechRate(fSpeechChannel, &fSpeechRate));
	}
}


//----------------------------------------------------------------------------------------
// MSpeakable::MSpeakable
//----------------------------------------------------------------------------------------
MSpeakable::MSpeakable(const CStr63_AC& name)
	:	fSpeechRateModifier(1.0),
		fSpeechChannel(NULL)
{
	if (HasSpeechManager_AC())
	{
		//Passing NULL as the VoiceSpec will give the system default
		::ThrowIfOSErr_AC(NewSpeechChannel(NULL, &fSpeechChannel));
		::ThrowIfOSErr_AC(GetSpeechRate(fSpeechChannel, &fSpeechRate));
		UseVoiceByName(name);
	}
}

//----------------------------------------------------------------------------------------
// MSpeakable::MSpeakable
//----------------------------------------------------------------------------------------
MSpeakable::MSpeakable(short index)
	:	fSpeechRateModifier(1.0),
		fSpeechChannel(NULL)
{
	if (HasSpeechManager_AC())
	{
		//Passing NULL as the VoiceSpec will give the system default
		::ThrowIfOSErr_AC(NewSpeechChannel(NULL, &fSpeechChannel));
		::ThrowIfOSErr_AC(GetSpeechRate(fSpeechChannel, &fSpeechRate));
		UseVoiceByIndex(index);
	}
}

//----------------------------------------------------------------------------------------
// MSpeakable::~MSpeakable
//----------------------------------------------------------------------------------------
MSpeakable::~MSpeakable()
{
	::ThrowIfOSErr_AC(DisposeSpeechChannel(fSpeechChannel));
	fSpeechChannel = NULL;
}

//----------------------------------------------------------------------------------------
// MSpeakable::UseVoiceByName
//----------------------------------------------------------------------------------------
void MSpeakable::UseVoiceByName(const CStr63_AC& name)
{
	if (HasSpeechManager_AC())
	{
		//StringPtr pascalName = c2pstr(name);
		
		for (short index = 1; index <= (GetNumberVoices()); ++index)
		{
			VoiceSpec spec;
			VoiceDescription description;
		
			::ThrowIfOSErr_AC(GetIndVoice(index, &spec));
			::ThrowIfOSErr_AC(GetVoiceDescription(&spec, &description, sizeof(VoiceDescription)));
			
			if(name == *reinterpret_cast<CStr63_AC*>(description.name))
			{
				UseVoiceByIndex(index);
				break;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// MSpeakable::UseVoiceByIndex
//----------------------------------------------------------------------------------------
void MSpeakable::UseVoiceByIndex(short index)
{
	if (HasSpeechManager_AC())
	{
		VoiceSpec spec;
		VoiceDescription description;
		long infoLength = sizeof(VoiceDescription);
		
		::ThrowIfOSErr_AC(GetIndVoice(index, &spec));
		::ThrowIfOSErr_AC(GetVoiceDescription(&spec, &description, infoLength));
		
		//Clear out the old values
		::ThrowIfOSErr_AC(DisposeSpeechChannel(fSpeechChannel));
		fSpeechChannel = NULL;
		
		//Assign the new values
		::ThrowIfOSErr_AC(NewSpeechChannel(&spec, &fSpeechChannel));
		::ThrowIfOSErr_AC(::SetSpeechRate(fSpeechChannel, fSpeechRate));
	}
}

//----------------------------------------------------------------------------------------
// MSpeakable::GetVoiceName
//----------------------------------------------------------------------------------------
CStr63_AC MSpeakable::GetVoiceName(short index) const
{
	VoiceSpec spec;
	static VoiceDescription description;
	long infoLength = sizeof(description);
	
	::ThrowIfOSErr_AC(GetIndVoice(index, &spec));
	::ThrowIfOSErr_AC(GetVoiceDescription(&spec, &description, infoLength));
	
	return description.name;
}

//----------------------------------------------------------------------------------------
// MSpeakable::GetNumberVoices
//----------------------------------------------------------------------------------------
short MSpeakable::GetNumberVoices() const
{
	short numberOfVoices;
	
	::ThrowIfOSErr_AC(CountVoices(&numberOfVoices));
	
	return numberOfVoices;
}

//----------------------------------------------------------------------------------------
// MSpeakable::IsSpeechBusy
//----------------------------------------------------------------------------------------
bool MSpeakable::IsSpeechBusy(bool systemWide) const
{
	return systemWide ? ::SpeechBusySystemWide() : ::SpeechBusy();
}

//----------------------------------------------------------------------------------------
// MSpeakable::WaitForSpeech
//----------------------------------------------------------------------------------------
void MSpeakable::WaitForSpeech()
{
	//Only concerned with waiting for the last speech command
	//issued by your application.
	
	//This method could take a parameter, and be used to wait for any speech 
	
	while (IsSpeechBusy(false))
		{};
}

//----------------------------------------------------------------------------------------
// MSpeakable::StopSpeech
//----------------------------------------------------------------------------------------
void MSpeakable::StopSpeech(long whereToStop)
{
	::ThrowIfOSErr_AC(StopSpeechAt(fSpeechChannel,whereToStop));
}

//----------------------------------------------------------------------------------------
// MSpeakable::PauseSpeech
//----------------------------------------------------------------------------------------
void MSpeakable::PauseSpeech(long whereToStop)
{
	::ThrowIfOSErr_AC(PauseSpeechAt(fSpeechChannel,whereToStop));
}

//----------------------------------------------------------------------------------------
// MSpeakable::ContinueSpeech
//----------------------------------------------------------------------------------------
void MSpeakable::ContinueSpeech()
{
	::ThrowIfOSErr_AC(::ContinueSpeech(fSpeechChannel));
}

//----------------------------------------------------------------------------------------
// MSpeakable::Say -- For Floats
//----------------------------------------------------------------------------------------
void MSpeakable::Say(float number, short digitsDesired, bool wait)
{
	decimal numberDecimal;
	decform format;
	char numberString[40];

	format.style = FIXEDDECIMAL;
	format.digits = digitsDesired;
	
	::num2dec(&format, number, &numberDecimal);
	::dec2str(&format, &numberDecimal, numberString);
	
	Say(numberString, wait);
}

//----------------------------------------------------------------------------------------
// MSpeakable::Say -- For Integers
//----------------------------------------------------------------------------------------
void MSpeakable::Say(long number, bool wait)
{
	Say(number, 0, wait);
}

//----------------------------------------------------------------------------------------
// MSpeakable::Say -- For Strings
//----------------------------------------------------------------------------------------
void MSpeakable::Say(const char* str, bool wait)
{
	if (str)
		Say(str, 0, strlen(str), wait);
}

//----------------------------------------------------------------------------------------
// MSpeakable::Say -- For TextEdit
//----------------------------------------------------------------------------------------
void MSpeakable::Say(const TEHandle TEHdl, bool wait)
{
	
	Handle textHdl = (**TEHdl).hText;
	long textBytes = ::GetHandleSize(textHdl);
	
	CTempHandleLock_AC tempHandleLock(textHdl);

	Say(*textHdl, 0, textBytes, wait);
}
  
//----------------------------------------------------------------------------------------
// MSpeakable::Say -- Say some Text
//----------------------------------------------------------------------------------------
void MSpeakable::Say(const void *textBuf, short firstByte, short byteCount, bool wait)
{	
	if (HasSpeechManager_AC())
	{
		::ThrowIfOSErr_AC(SpeakText(fSpeechChannel, static_cast<Ptr>(const_cast<void *>(textBuf)) + firstByte, byteCount));
		
		if (wait)
			WaitForSpeech();
	}
}

//----------------------------------------------------------------------------------------
// MSpeakable::SetSpeechRateModifier
//----------------------------------------------------------------------------------------
void MSpeakable::SetSpeechRateModifier(Fixed newModifier)
{

#if qDebug
	if (newModifier <= 0.0)
		DebugStr("\p Setting the modification rate to zero is not safe.");
#endif

	fSpeechRateModifier = newModifier;
}
 
//----------------------------------------------------------------------------------------
// MSpeakable::SetSpeechRate
//----------------------------------------------------------------------------------------
void MSpeakable::SetSpeechRate(Fixed newRate)
{

#if qDebug
	if (newRate <= 0.0)
		DebugStr("\p Setting the speech rate to zero is not good.");
#endif
	
	fSpeechRate = newRate;
	::ThrowIfOSErr_AC(::SetSpeechRate(fSpeechChannel,fSpeechRate));
}


//----------------------------------------------------------------------------------------
// MSpeakable::SpeakFaster
//----------------------------------------------------------------------------------------
void MSpeakable::SpeakFaster()
{
	if (HasSpeechManager_AC())
	{
		fSpeechRate *= fSpeechRateModifier;
		::ThrowIfOSErr_AC(::SetSpeechRate(fSpeechChannel, fSpeechRate));
	}
}

//----------------------------------------------------------------------------------------
// MSpeakable::SpeakSlower
//----------------------------------------------------------------------------------------
void MSpeakable::SpeakSlower()
{
	if (HasSpeechManager_AC())
	{
		fSpeechRate /= fSpeechRateModifier;
		::ThrowIfOSErr_AC(::SetSpeechRate(fSpeechChannel, fSpeechRate));
	}
	
}

//----------------------------------------------------------------------------------------
// End of USpeakable.cp



