//----------------------------------------------------------------------------------------
// UViewDragger.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UVIEWDRAGGER__
#include "UViewDragger.h"
#endif


// ACS

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif

// MacApp

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

//========================================================================================
// Global Initialization Procedure
//========================================================================================

//----------------------------------------------------------------------------------------
// InitUViewDragger
//----------------------------------------------------------------------------------------

void InitUViewDragger()
{
	// So the linker doesn't dead strip class info 
	MA_REGISTER_CLASS(TViewDragger);
	MA_REGISTER_CLASS(TDragViewCommand);
}

//========================================================================================
// CLASS TViewDragger
//========================================================================================

#undef Inherited


MA_DEFINE_CLASS(TViewDragger);

//----------------------------------------------------------------------------------------
// TViewDragger constructor
//----------------------------------------------------------------------------------------


TViewDragger::TViewDragger(IDType itsIdentifier) :
	TBehavior(itsIdentifier),
	fInitialized(false),
	fNumTargets(0),
	fDragMode(eHorizontal)
{
	for (int i=0; i<kNumTargets; ++i)
	{
		fTarget[i] = NULL;
		fChildPeerOrParent[i] = ePeerAssociation;
		fTargetViewID[i] = 0;
		fChangeTypeMode[i] = eNoMode;
		fAdjustMode[i] = eDontAdjust;
		fProportion[i] = 1;
	}
}

//----------------------------------------------------------------------------------------
// TViewDragger copy constructor
//----------------------------------------------------------------------------------------


TViewDragger::TViewDragger(const TViewDragger& inViewDragger) :
	TBehavior(inViewDragger),
	fInitialized(inViewDragger.fInitialized),
	fNumTargets(inViewDragger.fNumTargets),
	fDragMode(inViewDragger.fDragMode),
	fLimitFrame(inViewDragger.fLimitFrame)
{
	for (int i=0; i<kNumTargets; ++i)
	{
		fTarget[i] = inViewDragger.fTarget[i];
		fChildPeerOrParent[i] = inViewDragger.fChildPeerOrParent[i];
		fTargetViewID[i] = inViewDragger.fTargetViewID[i];
		fChangeTypeMode[i] = inViewDragger.fChangeTypeMode[i];
		fAdjustMode[i] = inViewDragger.fAdjustMode[i];
		fProportion[i] = inViewDragger.fProportion[i];
	}
}

//----------------------------------------------------------------------------------------
// TViewDragger::destructor
//----------------------------------------------------------------------------------------


TViewDragger::~TViewDragger()
{
}

//----------------------------------------------------------------------------------------
// TViewDragger::ReadFrom:
//----------------------------------------------------------------------------------------


void TViewDragger::ReadFrom(CStream_AC* aStream)		// Override 
{
	TBehavior::ReadFrom(aStream);

	*aStream 	>> fNumTargets
				>> StreamInEnum_AC(fDragMode)
				>> fLimitFrame;

	for (int i=0; i<kNumTargets; ++i)
	{
		*aStream 	>> fTargetViewID[i]
					>> StreamInEnum_AC(fChildPeerOrParent[i])
					>> StreamInEnum_AC(fChangeTypeMode[i])
					>> StreamInEnum_AC(fAdjustMode[i]);
	}


#if 0
	fNumTargets = aStream->ReadInteger();
	fDragMode	= DragMode(aStream->ReadInteger());
	fLimitFrame = aStream->ReadLongRect();
	for (int i=0; i<kNumTargets; ++i)
	{
		fTargetViewID[i] = aStream->ReadIDType();
		fChildPeerOrParent[i] = AssociationMode(aStream->ReadInteger());
		fChangeTypeMode[i] = ChangeTypeMode(aStream->ReadInteger());
		fAdjustMode[i] = AdjustMode(aStream->ReadInteger());
	}
#endif
}

//----------------------------------------------------------------------------------------
// TViewDragger::WriteTo:
//----------------------------------------------------------------------------------------


void TViewDragger::WriteTo(CStream_AC* aStream) const		// Override 
{
	TBehavior::WriteTo(aStream);

	*aStream 	<< fNumTargets
				<< StreamOutEnum_AC(fDragMode)
				<< fLimitFrame;

	for (int i=0; i<kNumTargets; ++i)
	{
		*aStream 	<< fTargetViewID[i]
					<< StreamOutEnum_AC(fChildPeerOrParent[i])
					<< StreamOutEnum_AC(fChangeTypeMode[i])
					<< StreamOutEnum_AC(fAdjustMode[i]);
	}


#if 0
	aStream->WriteInteger(fNumTargets);
	aStream->WriteInteger(fDragMode);
	aStream->WriteLongRect(fLimitFrame);
	for (int i=0; i<kNumTargets; ++i)
	{
		aStream->WriteIDType(fTargetViewID[i]);
		aStream->WriteInteger(fChildPeerOrParent[i]);
		aStream->WriteInteger(fChangeTypeMode[i]);
		aStream->WriteInteger(fAdjustMode[i]);
	}
#endif
}


//----------------------------------------------------------------------------------------
// TViewDragger::DoMouseCommand:
//----------------------------------------------------------------------------------------


bool TViewDragger::DoMouseCommand(CViewPoint& theMouse,
								  TToolboxEvent* /* event */,
								  CPoint_AC /* hysteresis */)
{
	if (fDragMode == eHorizontal)
		fLimitFrame.top = fLimitFrame.bottom = (dynamic_cast_AC(TView*,fOwner))->GetLocation().v;
	else if (fDragMode == eVertical)
		fLimitFrame.left = fLimitFrame.right = (dynamic_cast_AC(TView*,fOwner))->GetLocation().h;

	fOwner->PostCommand(TH_new TDragViewCommand(dynamic_cast_AC(TView*,fOwner), (dynamic_cast_AC(TView*,fOwner))->LocalToSuperView(theMouse), fLimitFrame));

	return true;
}

//----------------------------------------------------------------------------------------
// TViewDragger::DoSetCursor:
//----------------------------------------------------------------------------------------


bool TViewDragger::DoSetCursor(const CViewPoint& /* localPoint*/,
							   RgnHandle cursorRegion)
{
	// Need to set horizontal, vertical, or four way arrow pair here.
	CursHandle crossCursorHandle = ::GetCursor(crossCursor);
	::SetCursor(*crossCursorHandle);
	(dynamic_cast_AC(TView*,fOwner))->GetExtentRegion(cursorRegion);
	return true;
}

//----------------------------------------------------------------------------------------
// TViewDragger::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TViewDragger::RespondToFrameAdjustment(CViewRect&	/* newFrame */ ,
									  CViewRect& deltaFrame,
									  TView* changedView )
{
	for (int i=0; i<fNumTargets; ++i)
	{
		if (fTarget[i] == changedView)
		{
			switch (fChangeTypeMode[i])
			{
				case eTopMode:
					switch (fAdjustMode[i])
					{
						case eAdjPosByTopOrLeft:
							fLimitFrame.top += deltaFrame.top;
							fLimitFrame.bottom += deltaFrame.top;
							break;

						case eAdjNegByTopOrLeft:
							fLimitFrame.top -= deltaFrame.top;
							fLimitFrame.bottom -= deltaFrame.top;
							break;

						case eAdjPosByBotOrRight:
							fLimitFrame.top += deltaFrame.bottom;
							fLimitFrame.bottom += deltaFrame.bottom;
							break;

						case eAdjNegByBotOrRight:
							fLimitFrame.top -= deltaFrame.bottom;
							fLimitFrame.bottom -= deltaFrame.bottom;
							break;

						case eAdjProportional:
							{
								// Adjust based on the height of the parent view.
								ViewCoordinate ourHeight = fLimitFrame.GetSize()[vSel];
								double aHeight = fOldFrame[i].GetSize()[vSel] + deltaFrame.GetSize()[vSel];
								fLimitFrame.top = aHeight * fProportion[i];
								fLimitFrame.bottom = fLimitFrame.top + ourHeight;
							}
							break;
					}
					break;

				case eLeftMode:
					switch (fAdjustMode[i])
					{
						case eAdjPosByTopOrLeft:
							fLimitFrame.left += deltaFrame.left;
							fLimitFrame.right += deltaFrame.left;
							break;

						case eAdjNegByTopOrLeft:
							fLimitFrame.left -= deltaFrame.left;
							fLimitFrame.right -= deltaFrame.left;
							break;

						case eAdjPosByBotOrRight:
							fLimitFrame.left += deltaFrame.right;
							fLimitFrame.right += deltaFrame.right;
							break;

						case eAdjNegByBotOrRight:
							fLimitFrame.left -= deltaFrame.right;
							fLimitFrame.right -= deltaFrame.right;
							break;

						case eAdjProportional:		// Adjust proportional to the height/width.
							{
								// Adjust based on the width of the parent view.
								ViewCoordinate ourWidth = fLimitFrame.GetSize()[hSel];
								double aWidth = fOldFrame[i].GetSize()[hSel] + deltaFrame.GetSize()[hSel];
								fLimitFrame.left = aWidth * fProportion[i];
								fLimitFrame.right = fLimitFrame.left + ourWidth;
							}
							break;
					}
					break;

				case eHeightMode:
					switch (fAdjustMode[i])
					{
						case eAdjPosByTopOrLeft:
							fLimitFrame.bottom += deltaFrame.top;
							break;

						case eAdjNegByTopOrLeft:
							fLimitFrame.bottom -= deltaFrame.top;
							break;

						case eAdjPosByBotOrRight:
							fLimitFrame.bottom += deltaFrame.bottom;
							break;

						case eAdjNegByBotOrRight:
							fLimitFrame.bottom -= deltaFrame.bottom;
							break;

						case eAdjProportional:
							{
								// Adjust based on the height of the target view.
								double aHeight = fOldFrame[i].GetSize()[vSel] + deltaFrame.GetSize()[vSel];
								double newHeight = aHeight * fProportion[i];
								fLimitFrame.bottom = fLimitFrame.top + newHeight;
							}
							break;
					}
					break;

				case eWidthMode:
					switch (fAdjustMode[i])
					{
						case eAdjPosByTopOrLeft:
							fLimitFrame.right += deltaFrame.left;
							break;

						case eAdjNegByTopOrLeft:
							fLimitFrame.right -= deltaFrame.left;
							break;

						case eAdjPosByBotOrRight:
							fLimitFrame.right += deltaFrame.right;
							break;

						case eAdjNegByBotOrRight:
							fLimitFrame.right -= deltaFrame.right;
							break;

						case eAdjProportional:		// Adjust proportional to the height/width.
							{
								// Adjust based on the width of the target view.
								double aWidth = fOldFrame[i].GetSize()[hSel] + deltaFrame.GetSize()[hSel];
								double newWidth = aWidth * fProportion[i];
								fLimitFrame.right = fLimitFrame.left + newWidth;
							}
							break;
					}
					break;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TViewDragger::InitializeViewDragger 
//----------------------------------------------------------------------------------------


void TViewDragger::InitializeViewDragger()
{
	// Check to see if we've been initialized and if we haven't then 
	// add dependencies to the target views.
	if (!fInitialized)
	{
		TView * targetView = NULL;

		for (int i=0; i<fNumTargets; ++i)
		{
			// Figure out where the view is in which we are interested.
			switch (fChildPeerOrParent[i])
			{
				case eChildAssociation:
					targetView = (dynamic_cast_AC(TView*,fOwner))->FindSubView(fTargetViewID[i]);
					break;

				case ePeerAssociation:
					targetView = (dynamic_cast_AC(TView*,fOwner))->FindPeerView(fTargetViewID[i]);
					break;

				case eParentAssociation:
					targetView = (dynamic_cast_AC(TView*,fOwner))->FindParentView(fTargetViewID[i]);
					break;
					// Add catch here on qDebug for unrecognized fChildPeerOrParent.
			}
			::ThrowIfNonObject_AC(targetView);
			fTarget[i] = targetView;

			fOldFrame[i] = fTarget[i]->GetFrame();		// We need to remember the current frame.
			bool alreadyRegistered = false;
			for (int j=0; j<i; ++j)
			{
				if (fTarget[i] == fTarget[j]) alreadyRegistered = true;
			}
			if (!alreadyRegistered) fTarget[i]->AddDependent(this);		// And register for dependency.
		
			SetGeneralMode(i, fChangeTypeMode[i], fAdjustMode[i]);

		}
		fInitialized = true;
	}
}

//----------------------------------------------------------------------------------------
// TViewDragger::DoUpdate 
//----------------------------------------------------------------------------------------


void TViewDragger::DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject,
						  void*					/* changeData */ ,
						  CDependencySpace_AC*		/* dependencySpace */ )
{
	// Policy: If the view which has changed is one of the fTargetViews then test that
	// the frame has changed, calculate the delta frame and reset fOldFrame
	// and respond via the normal manner.  Otherwise respond using the
	// alternative RespondToOtherViewChange.  This will allow some flexibility
	// in the dependency mechanism such as with TRightSizer.

	// Initialize ourselves if we haven't been already.
	if (!fInitialized)
		InitializeViewDragger();

	// Make sure we only respond to the view we're expecting to change.
	if (theChange == mFrameChanged)
	{
		TView * changedView = dynamic_cast_AC(TView*,changedObject);
		if (changedView == fOwner)	// Alternative frame change.
			RespondToOtherViewChange(changedView);
			
		// We have a view telling us that it's frame has changed.
		else
		{
			for (int i=0; i<fNumTargets; ++i)
			{
				if (changedView == fTarget[i])
				{
					// Normal frame change response.
					CViewRect newFrame(changedView->GetFrame());
					if (newFrame != fOldFrame[i])
					{
						CViewRect deltaFrame = newFrame - fOldFrame[i];
						RespondToFrameAdjustment(newFrame, deltaFrame, changedView);
						fOldFrame[i] = newFrame;
					}
					break;	// Only call respond once per view changed.
				}
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TViewDragger::AddConstraint
//----------------------------------------------------------------------------------------


int TViewDragger::AddConstraint(TView* target,
							ChangeTypeMode theChangeType,
							AdjustMode theAdjustMode)
{
	int i = -1;
	if (!fInitialized) InitializeViewDragger();
	if (fNumTargets < kNumTargets)
	{
		::ThrowIfNonObject_AC(target);
		i = fNumTargets;
		fTarget[i] = target;
		++fNumTargets;
		fOldFrame[i] = fTarget[i]->GetFrame();		// We need to remember the current frame.
		bool alreadyRegistered = false;
		for (int j=0; j<i; ++j)
		{
			if (fTarget[i] == fTarget[j]) alreadyRegistered = true;
		}
		if (!alreadyRegistered) fTarget[i]->AddDependent(this);		// And register for dependency.
	
		SetGeneralMode(i, theChangeType, theAdjustMode);
	}
	return i;	// Return the index of the added target.
}

//----------------------------------------------------------------------------------------
// TViewDragger::AddConstraint
//----------------------------------------------------------------------------------------


void TViewDragger::SetLimitFrame(CViewRect& limitFrame, DragMode theDragMode)
{
	fDragMode = theDragMode;
	fLimitFrame = limitFrame;
}

//----------------------------------------------------------------------------------------
// TViewDragger::SetGeneralMode
//----------------------------------------------------------------------------------------


void TViewDragger::SetGeneralMode(int targetIndex,
							ChangeTypeMode theChangeType,
							AdjustMode theAdjustMode)
{
	// Add assert here that index is in range.
	
	// Call this to manually set the change type and adjustment modes.  (This is
	// also called from the initialization commands.)
	fChangeTypeMode[targetIndex] = theChangeType;
	fAdjustMode[targetIndex] = theAdjustMode;

	TView * ownerView = dynamic_cast_AC(TView*,fOwner);

	// If this is a proportional relationship then let's figure out the proportion
	// right now rather than having to do so all the time.
	if (fAdjustMode[targetIndex] == eAdjProportional)
		switch (fChildPeerOrParent[targetIndex])
		{
			case eParentAssociation:
				switch (fChangeTypeMode[targetIndex])
				{
					case eTopMode:
					case eLeftMode:
						{
							VHSelect aSel = (fChangeTypeMode[targetIndex] == eTopMode) ? vSel : hSel;
							double aSize = fOldFrame[targetIndex].GetSize()[aSel];
							CViewPoint thePoint;
							// We have to crawl up to the indicated superview here.
							
							TView * aView = ownerView;
							while (aView != fTarget[targetIndex])
							{
								::ThrowIfNonObject_AC(aView);
								thePoint = aView->LocalToSuperView(thePoint);
								aView = aView->GetSuperView();
							}
							double aCoordinate = thePoint[aSel];
							fProportion[targetIndex] = aCoordinate / aSize;
						}
						break;

					case eHeightMode:
					case eWidthMode:
						{
							VHSelect aSel = (fChangeTypeMode[targetIndex] == eHeightMode) ? vSel : hSel;
							double aSize = fOldFrame[targetIndex].GetSize()[aSel];
							double ourSize = ownerView->GetSize()[aSel];
							fProportion[targetIndex] = ourSize / aSize;
						}
						break;
						// Add catch here on qDebug for unrecognized fChangeTypeModes.
				}
				break;
				
			case ePeerAssociation:
			case eChildAssociation:
				// Not yet handled!
				break;
		}
}

//========================================================================================
// CLASS TDragViewCommand
//========================================================================================

#undef Inherited

//----------------------------------------------------------------------------------------
// TDragViewCommand constructor
//----------------------------------------------------------------------------------------


TDragViewCommand::TDragViewCommand(TView* viewToDrag,
					const CViewPoint& itsMouse, const CViewRect& constraint) :
	TTracker(cDragView,NULL,false,true,NULL,NULL,NULL,itsMouse)
{
	fConstrainsMouse = true;
	fViewToDrag = viewToDrag;
	fView = viewToDrag->GetSuperView();	// Drag takes place in context of parent view.
	fConstraint = constraint;
	fOldFrame = fViewToDrag->GetFrame();
}

//----------------------------------------------------------------------------------------
// TDragViewCommand destructor
//----------------------------------------------------------------------------------------


TDragViewCommand::~TDragViewCommand()
{
}

//----------------------------------------------------------------------------------------
// TDragViewCommand::TrackConstrain:
//----------------------------------------------------------------------------------------


void TDragViewCommand::TrackConstrain(TrackPhase /* aTrackPhase */,
										const CViewPoint& /* anchorPoint */,
									  	const CViewPoint& /* previousPoint */,
									    CViewPoint& nextPoint,
										bool /* mouseDidMove */)
{
	CViewPoint oldPoint(fOldFrame.left, fOldFrame.top);
	CViewPoint delta = nextPoint - fInitialPt;
	CViewPoint newPoint = oldPoint + delta;
	newPoint.ConstrainTo(fConstraint);
	delta = newPoint - oldPoint;
	nextPoint = fInitialPt + delta;
}

//----------------------------------------------------------------------------------------
// TDragViewCommand::TrackFeedback:
//----------------------------------------------------------------------------------------


void TDragViewCommand::TrackFeedback(TrackPhase /* aTrackPhase */,
										const CViewPoint& /* anchorPoint */,
										const CViewPoint& /* previousPoint */,
										const CViewPoint& nextPoint,
										bool mouseDidMove,
										bool /* turnItOn */)
{
	if (mouseDidMove)
	{
		CViewRect vr(fOldFrame);
		CViewPoint delta = nextPoint - fInitialPt;
		vr += delta;

		::PenPat(&qd.gray);
		::FrameRect(vr.ToRect());
	}
}

//----------------------------------------------------------------------------------------
// TDragViewCommand::TrackMouse:
//----------------------------------------------------------------------------------------


TTracker* TDragViewCommand::TrackMouse(TrackPhase aTrackPhase,
										CViewPoint& /* anchorPoint */,
										CViewPoint& /* previousPoint */,
										CViewPoint& nextPoint,
										bool /* mouseDidMove */)
{
	return (aTrackPhase == trackEnd && (!fView || !fView->ContainsMouse(nextPoint))) ? NULL : this;
}

//----------------------------------------------------------------------------------------
// TDragViewCommand::DoIt:
//----------------------------------------------------------------------------------------


void TDragViewCommand::DoIt()
{
	CViewPoint delta = fNextPoint - fInitialPt;
	fOldFrame += delta;
	fViewToDrag->SetFrame(fOldFrame, true);
}