//----------------------------------------------------------------------------------------
// UViewServer.cp
// Copyright  1987-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CStreamContext_AC__
#include "CStreamContext_AC.h"
#endif

#ifndef __CRandomAccessStream_AC__
#include "CRandomAccessStream_AC.h"
#endif

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

// MacApp

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

//	#ifndef __UAPPLICATION__
//	#include "UApplication.h"
//	#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

//	#ifndef __UERRORMGR__
//	#include "UErrorMgr.h"
//	#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __RESOURCES__
#include <Resources.h>
#endif

// ANSI



//----------------------------------------------------------------------------------------
// global declarations for globals defined in interface
CAutoPtr_AC<TViewServer> TViewServer::fgViewServer;


//========================================================================================
// CLASS TViewServer
//========================================================================================
//----------------------------------------------------------------------------------------
// TViewServer::IViewServer: 
//----------------------------------------------------------------------------------------


TViewServer::TViewServer()
{
	fgViewServer = this;							// set the global reference.
}

//----------------------------------------------------------------------------------------
// TViewServer::Free: 
//----------------------------------------------------------------------------------------


TViewServer::~TViewServer()
{
}

//----------------------------------------------------------------------------------------
// TViewServer::NewPaletteWindow: 
//----------------------------------------------------------------------------------------


TWindow* TViewServer::NewPaletteWindow(ResNumber itsRsrcID,
									   bool wantHScrollBar,
									   bool wantVScrollBar,
									   TView* itsMainView,
									   TView* itsPaletteView,
									   short sizePalette,
									   VHSelect whichWay)

{
	// create the window, adjust its resize limits, and add the palette view
	CAutoPtr_AC<TWindow> aWindow(NewTWindow(itsRsrcID));

	CRect_AC resizeLimits(aWindow->GetResizeLimits());
	resizeLimits[topLeft][whichWay] += sizePalette;
	aWindow->SetResizeLimits(resizeLimits);
	aWindow->AddSubView(itsPaletteView);

	// calculate location of itsMainView
	CViewPoint itsLocation;
	itsLocation[whichWay] = sizePalette;

	// add itsMainView to the window with optional scroller
	AddSubViewWithScroller(aWindow, itsMainView, wantHScrollBar, wantVScrollBar, itsLocation);

	return aWindow.release();
}

//----------------------------------------------------------------------------------------
// TViewServer::NewSimpleWindow: 
//----------------------------------------------------------------------------------------


TWindow* TViewServer::NewSimpleWindow(ResNumber itsRsrcID,
									  bool wantHScrollBar,
									  bool wantVScrollBar,
									  TView* itsView)

{
	// create the window
	CAutoPtr_AC<TWindow> aWindow(NewTWindow(itsRsrcID));

	// add itsView to the window with optional scroller
	AddSubViewWithScroller(aWindow, itsView, wantHScrollBar, wantVScrollBar, gZeroVPt);

	return aWindow.release();
}

//----------------------------------------------------------------------------------------
// TViewServer::AddSubViewWithScroller: 
//----------------------------------------------------------------------------------------


void TViewServer::AddSubViewWithScroller(TWindow* aWindow,
										 TView* itsView,
										 bool wantHScrollBar,
										 bool wantVScrollBar,
										 const CViewPoint& itsLocation)
{
	TScroller * aScroller = NULL;
	
	if (wantHScrollBar || wantVScrollBar)
	{
		// calculate scroller's dimensions
		CViewRect sBarOffsets;
		CViewPoint itsSize = aWindow->GetSize();
		
		// Account for scroller _not_ being located at CViewPoint (0,0)
		// as when a palette is sharing the window.
		itsSize -= itsLocation;
			
		if (wantHScrollBar)
		{
			itsSize.v -= kSBarSizeMinus1;
			if (!wantVScrollBar)
				sBarOffsets.right = -kSBarSizeMinus1;
		}
		if (wantVScrollBar)
		{
			itsSize.h -= kSBarSizeMinus1;
			if (!wantHScrollBar)
				sBarOffsets.bottom = -kSBarSizeMinus1;
		}

		// create the scroller
		aScroller = TH_new TScroller(itsLocation, itsSize, sizeRelSuperView, sizeRelSuperView, gZeroVPt, wantHScrollBar, wantVScrollBar);
		aScroller->fSBarOffsets = sBarOffsets;
		aWindow->AddSubView(aScroller);
		aScroller->HandlePostCreate(aWindow->GetDocument()); //SRF new requirement
	}

	// add the subview
	if (aScroller)
		aScroller->AddSubView(itsView);
	else
		aWindow->AddSubView(itsView);

	aWindow->SetWindowTarget(itsView);

	// make sure window and subviews are the right size 
	aWindow->Resize(((CRect_AC &) GetWindowPort(aWindow->GetWindowRef())->portRect).GetSize(), kDontInvalidate);
}

//----------------------------------------------------------------------------------------
// TViewServer::NewTWindow: 
//----------------------------------------------------------------------------------------


TWindow* TViewServer::NewTWindow(ResNumber itsRsrcID)
{
	TWindow * aWindow = NULL;
	bool canResize;
	bool canClose;

	WindowRef aWMgrWindow = gDispatcher->NewToolboxWindow(itsRsrcID, canResize, canClose);
	// NewToolboxWindow signals Failure 

	try
	{
		aWindow = TH_new TWindow(aWMgrWindow, canResize, canClose, false);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		aWMgrWindow = TWindow::FreeIfWMgrWindow(aWMgrWindow);
		throw;
	}
	
	return aWindow;
}

//----------------------------------------------------------------------------------------
// TViewServer::NewTemplateWindow: 
//----------------------------------------------------------------------------------------


TWindow* TViewServer::NewTemplateWindow(ResNumber viewRsrcID,
										TDocument* itsDocument)

{
	TWindow	*theWindow = NULL;
	
	TView * aView = DoCreateViews(itsDocument, NULL, viewRsrcID, gZeroVPt);
	if (aView)
	{
		CAutoPtr_AC<TView> tempObject(aView);
#if qDebug
		if (!dynamic_cast<TWindow*>(aView))
			::ProgramBreak_AC("In NewTemplateWindow: Root view is not a window");
#endif
		theWindow = dynamic_cast_AC(TWindow*,aView);

		if (theWindow->GetTargetID() != kNoIdentifier)
		{
			TView* theTarget = theWindow->FindSubView(theWindow->GetTargetID());
			if (theTarget)
				theWindow->SetWindowTarget(theTarget);
#if qDebug
			else
			{
				CStr255_AC		errString("TViewServer::NewTemplateWindow:\n  The window's indicated target view cannot be found.\n");
				errString += "  Unable to find view with fTargetID: '" + CStr255_AC(theWindow->GetTargetID()) + "'.";
				::ProgramBreak_AC(errString);
			}
#endif
		}
		tempObject.release();
	}

	return theWindow;
}

//----------------------------------------------------------------------------------------
// TViewServer::ReadViewsFromResource: 
//----------------------------------------------------------------------------------------


TView* TViewServer::ReadViewsFromResource(TDocument* /* itsDocument */,
										  TView* parentView,
										  ResNumber itsRsrcID,
										  const CViewPoint& subviewOffset)

{
	TView* firstView = NULL;					// Assume the worst. 
	CAutoPtr_AC<TStream> itsViewStream(DoMakeViewStream(itsRsrcID, NULL));

	try
	{
		short numViews = itsViewStream->ReadInteger();

		if (numViews == kMAThreeOhTemplateVersion)// actually, the view template version
			firstView = ReadViewsFrom(itsViewStream, parentView, subviewOffset);// the new protocol
		else
			::ProgramBreak_AC("Incompatible version of 'View' resources");
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		CRandomAccessStream_AC* raViewStream = dynamic_cast<CRandomAccessStream_AC*>(itsViewStream.get());

		// set position so that when the view stream frees itself, 
		// it won't truncate the handle (see THandleStream::Free)
		// If it did truncate the handle, we'd have no hope of ever
		// re-reading this 'View'
		if (raViewStream)
			raViewStream->SetPosition(raViewStream->GetSize());

		delete firstView;
		firstView = NULL;

		throw;
	}
	
	return firstView;
}

//----------------------------------------------------------------------------------------
// TViewServer::ReadViewsFrom: 
//----------------------------------------------------------------------------------------


TView* TViewServer::ReadViewsFrom(CStream_AC* itsStream,
								  TView* parentView,
								  const CViewPoint& subviewOffset)
{
	bool wasValid;
	TView* aView = dynamic_cast<TView*>(itsStream->ReadStreamObject(wasValid));
	if (wasValid && aView)
	{
		CAutoPtr_AC<TView> tempObject(aView);

		if (parentView)
			parentView->AddSubView(aView);
		
		if (subviewOffset != gZeroVPt)
			aView->Locate(aView->GetLocation() + subviewOffset, kDontInvalidate);

		tempObject.release();
	}
#if qDebugMsg
	else
		::ProgramBreak_AC("###Unknown TView in TViewServer::ReadViewsFrom.");
#endif

	return aView;
}

//----------------------------------------------------------------------------------------
// TViewServer::DoCreateViews: 
//----------------------------------------------------------------------------------------


TView* TViewServer::DoCreateViews(TDocument* itsDocument,
								  TView* parentView,
								  ResNumber itsRsrcID,
								  const CViewPoint& subviewOffset)
{
#if qTemplateViews
	TView * rootView = ReadViewsFromResource(itsDocument, parentView, itsRsrcID, subviewOffset);
	if (rootView)
	{
		CAutoPtr_AC<TView> tempObject(rootView);

		rootView->HandlePostCreate(itsDocument);
		rootView->AdjustFrame();				// Make sure size gets adjusted by the size determiners 

		tempObject.release();
	}

	return rootView;
#else
	return NULL;
#endif
}

//----------------------------------------------------------------------------------------
// TViewServer::DoMakeViewStream: 
//----------------------------------------------------------------------------------------


CStream_AC* TViewServer::DoMakeViewStream(ResNumber itsRsrcID,
									   Handle itsHandle)
{
	Handle viewStreamHandle;

	if (itsHandle)
		viewStreamHandle = itsHandle;
	else
	{
		// Try to get a new 'View' resource first.  If not found, then get a 'view' resource.
		viewStreamHandle = GetResource('View', itsRsrcID);
#if qDebug
		if (!viewStreamHandle)
		{
			viewStreamHandle = GetResource('view', itsRsrcID);
			if (viewStreamHandle)
			{
				::ProgramBreak_AC("Use ViewPromoter. Program contains 2.0 'view' resources");
				ReleaseResource(viewStreamHandle);
				viewStreamHandle = NULL;
			}
		}
#endif

		::ThrowIfNULLResource_AC(viewStreamHandle);
	}

	bool newEntry;
	
	// Create the stream context and its contents first, so that any memory failure here 
	// only affects the context, and doesn't truncate the view stream resource handle.
	CAutoPtr_AC<CStreamContext_AC> itsContext = TH_new CStreamContext_AC;
	itsContext->Add(&TAdorner::fgDimAdorner, kDontForceReplacement, newEntry);
	itsContext->Add(&TAdorner::fgDrawAdorner, kDontForceReplacement, newEntry);
	itsContext->Add(&TAdorner::fgEraseAdorner, kDontForceReplacement, newEntry);
	itsContext->Add(&TAdorner::fgHiliteAdorner, kDontForceReplacement, newEntry);
	itsContext->Add(&TAdorner::fgPrintAdorner, kDontForceReplacement, newEntry);
	itsContext->Add(&TAdorner::fgResizeIconAdorner, kDontForceReplacement, newEntry);
	itsContext->Add(&TAdorner::fgSelectionAdorner, kDontForceReplacement, newEntry);	

	CAutoPtr_AC<THandleStream> itsViewStream(TH_new THandleStream(viewStreamHandle, sizeof(long)));

	itsViewStream->SetContext(itsContext.release());

	return itsViewStream.release();
}

//----------------------------------------------------------------------------------------
// TViewServer::WriteViewsToResource: 
//----------------------------------------------------------------------------------------


void TViewServer::WriteViewsToResource(TView* view,
									   ResNumber itsRsrcID,
									   const CStr255_AC& name,
									   short resourceAttributes)
{
#if qDebugMsg
	if (Get1Resource('View', itsRsrcID))
		ProgramBreak_AC("###TViewServer::WriteViewsToResource warning: About to over-write 'View' resource id %d.\n", itsRsrcID);
#endif

	CTempHandle_AC viewHandle(WriteViewsToHandle(view));// write the views to the handle
	::ThrowIfNULL_AC(viewHandle);

	AddResource(viewHandle, 'View', itsRsrcID, name);// add the handle as a resource
	::ThrowIfResError_AC();
	viewHandle.DontRelease();

	SetResAttrs(viewHandle, resourceAttributes);// set the resource attributes
	::ThrowIfResError_AC();

	// Need the following ChangedResource call because the SetResAttrs call on the previous
	// line cleared the "resChanged" attribute of the resource.
	ChangedResource(viewHandle);
	::ThrowIfResError_AC();
}

//----------------------------------------------------------------------------------------
// TViewServer::WriteViewsToHandle: 
//----------------------------------------------------------------------------------------


Handle TViewServer::WriteViewsToHandle(TView* view)
{
	CTempHandle_AC aHandle(NewPermHandle(kViewRsrcExpandAmt));// create a handle for the stream

	CAutoPtr_AC<TStream> itsViewStream(DoMakeViewStream(kNoResource, aHandle));

	itsViewStream->WriteInteger(kMAThreeOhTemplateVersion);// Write version number
	itsViewStream->WriteStreamObject(view, kStandardObject);

	return aHandle.release();
}

//----------------------------------------------------------------------------------------
// End of UViewServer.cp


