// UCPlusTool.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.

//----------------------------------------------------------------------------------------
// Simple framework for an MPW tool. NOTE - tools cannot currently be built with -debug
//----------------------------------------------------------------------------------------

#ifndef __UCPLUSTOOL__
#define __UCPLUSTOOL__

// MacApp

#ifndef __UASSOCIATION__
#include "UAssociation.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------
extern const char* gCPlusToolVersion;
extern const char* kErrorMarker;

//----------------------------------------------------------------------------------------
// typedefs and structs
//----------------------------------------------------------------------------------------
struct KeyWordStruct
{
	char * keyword;
	short keyvalue;
};

// Keyword IDs. Negative numbers reserved for the framework
enum
{
	kwP = -1000,
	kwNoP,
	kwT,
	kwNoT,
	kwHelp
};

//----------------------------------------------------------------------------------------
// TCPlusTool
//----------------------------------------------------------------------------------------
class EXPORT_MA TCPlusTool
{
public:
	TCPlusTool(int argc = 0, char** argv = NULL);
		// Pass the argc and argv passed into your main.

	virtual ~TCPlusTool();
		// Destructor
		
	virtual void InstallKeyWord(const CStr255_AC& keyword, short kw);
	
	virtual bool LookupKeyword(const CStr255_AC& keyword, short& kw);
	
	virtual void InstallKeyWords();
	
	virtual void DoProcessFileArg(const CStr255_AC& arg);
	
	virtual void DoProcessOptionArg(short kw);
	
	virtual void DoShowUsage();
	
	virtual void DoStartProgress();
	
	virtual void DoToolAction();
	
	virtual void GetNextArg(CStr255_AC& nextArg);
	
	virtual void ProcessArg(const CStr255_AC& arg);
	
	virtual void Run();
	
	virtual void Stop(const CStr255_AC& msg);
	
	virtual void SyntaxError(const CStr255_AC& suffix);
	
	void SetInterrupted() { fInterrupted = true; }

public:
	static TCPlusTool* fgCPlusTool;					// The tool
//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
protected:
	TAssociation fKeyWordList;					// keywords to this command
	CStr255_AC fProgName;						// Program's file name
	long fStartTicks;							// tickcount at start of tool
	long fStartDateTime;						// Date/ Time at start of tool
	char** fArgv;								// command line arguments.
	int fArgc;									// number of command line arguments
	int fArgvIndex;								// current argument being processed
	short fCursorCount;
	
	enum 
	{											// for our spinning cursor
		RC_Normal, RC_ParmErrs, RC_DontMatch, RC_Abort
	} fRetCode;									// Return codes
	
	bool fInterrupted;							// True ==> interrupted (command "." pressed)
	bool fProgress;								// True for progress request
	bool fTime;									// True for elapsed time request
};

//----------------------------------------------------------------------------------------
// Unit intialization
//----------------------------------------------------------------------------------------
EXPORT_MA  void InitUCPlusTool();

#endif // __UCPLUSTOOL__
