


#ifndef _MEMHOOKS_
#define _MEMHOOKS_

#if MM_DEBUG

#ifndef _MEMORYHE_
#include "MemoryHe.h"
#endif


//==============================================================================
// CBlockCushionHook - A debugging memory hook which places a prefix and a
//					   suffix on each block. When the block is freed both are
//					   validated to make sure they have not been overwritten.
//
//						Block format:	PPPPXXXXPPPP...SSSS where XXXX is the
//															block size.
//
//==============================================================================

class EXPORT_MA CBlockCushionHook : public ODMemoryHook
{
public:
	CBlockCushionHook();
	virtual ~CBlockCushionHook();
	
	virtual ODBlockSize GetHeaderSize();

	virtual ODBlockSize AboutToAllocate(ODBlockSize size) const;
	virtual void* DidAllocate(void* blk, ODBlockSize size);
	virtual const void* AboutToBlockSize(const void* blk);
	virtual void* AboutToFree(void* blk);
	virtual void AboutToRealloc(void*& blk, ODBlockSize& size);
	virtual void* DidRealloc(void* oldBlk, void* blk, ODBlockSize size);

private:
	unsigned long* BasicAboutToFree(void* blk, MMBoolean zap);
	
	enum { kPrefixCushionLength = 3 * sizeof(unsigned long),
		   kSuffixCushionLength = sizeof(unsigned long) };
	enum { kPrefixCushionMagicNumber = 'PPPP',
		   kSuffixCushionMagicNumber = 'SSSS' };
};


//==============================================================================
// CBlockStackCrawlHook - A debugging memory hook that tags every block with the
//						  stack crawl as of the time it was created.
//==============================================================================

class StackCrawl;

class EXPORT_MA CBlockStackCrawlHook : public ODMemoryHook
{
public:
	virtual ODBlockSize GetHeaderSize();

	virtual ODBlockSize AboutToAllocate(ODBlockSize size) const;
	virtual void* DidAllocate(void* blk, ODBlockSize size);
	virtual const void* AboutToBlockSize(const void* blk);
	virtual void* AboutToFree(void* blk);
	virtual void AboutToRealloc(void*& block, ODBlockSize& size);
	virtual void* DidRealloc(void* oldBlk, void* blk, ODBlockSize size);
	virtual long GetType( ) const;
	enum{ kBlockStackCrawlHookType = 1 };
	
			StackCrawl* GetStackCrawl( const void* blk ) const;
			void		SetStackCrawl( const void* blk, StackCrawl* s);
	
	static MMBoolean gTrack;
	
private:
			void* BasicAboutToFree(void* blk, MMBoolean zap);
};


#endif /*MM_DEBUG*/

#endif /*_MEMHOOKS_*/