


#ifndef _MEMMGR_
#define _MEMMGR_
#pragma once

#include <stdlib.h>		// For size_t

#ifdef __cplusplus
extern "C" {
#endif

#pragma lib_export on


//========================================================================================
// Data types
//========================================================================================

typedef char MMBoolean;

typedef void* MMBlock;
typedef const void* ConstMMBlock;
typedef void* MMHandle;			// Handles are opaque as in Windows

typedef struct MemHeap MemHeap;					// An opqaue data type

typedef enum MMHeapLocation {
	kMMSysMemory,		// Shared system memory, persistent between processes
	kMMAppMemory,		// Process's local memory heap
	kMMTempMemory		// Local process mem outside its heap; may make sense only on Mac
} MMHeapLocation;


//========================================================================================
// Configuration
//========================================================================================

enum{
	kMMDebugConfigMask = 1				// Mask bit for return value from MMConfiguration
};

long	MMConfiguration( );

//========================================================================================
// Heap management
//========================================================================================

EXPORT_MA MemHeap*	MMNewHeap( MMHeapLocation, size_t initialSize, size_t growBy,
					const char *name );
EXPORT_MA void		MMDisposeHeap( MemHeap* );

EXPORT_MA MemHeap*	MMGetDefaultHeap( );
EXPORT_MA MemHeap*	MMSetDefaultHeap( MemHeap* );				// Returns old default

//========================================================================================
// Free space management
//========================================================================================

EXPORT_MA void		MMGetFreeSpace( MemHeap*, size_t *total, size_t *contig );

EXPORT_MA void		MMSystemFreeSpace( MMHeapLocation, size_t *total, size_t *contig );
				// Total free space in that area of RAM, _not_ MM heap free space.

EXPORT_MA MMBoolean	MMAllocateSlushFund( MemHeap*, size_t size, size_t allocSizeLimit );
EXPORT_MA size_t		MMSlushFundSize( MemHeap* );				// zero if none
EXPORT_MA size_t		MMFreeSlushFund( MemHeap* );				// returns size freed


//========================================================================================
// Operations on pointer based blocks
//========================================================================================

EXPORT_MA MMBlock		MMAllocate( size_t size );
EXPORT_MA MMBlock		MMAllocateClear( size_t size );

EXPORT_MA MMBlock		MMAllocateIn( size_t size, MemHeap* );
EXPORT_MA MMBlock		MMAllocateClearIn( size_t size, MemHeap* );

EXPORT_MA MMBlock		MMReallocate( MMBlock, size_t size );
EXPORT_MA MMBlock		MMSplice( MMBlock, size_t offset, long delta );
EXPORT_MA void		MMFree( MMBlock );

EXPORT_MA size_t		MMBlockSize( ConstMMBlock );
EXPORT_MA MemHeap*	MMGetHeap( ConstMMBlock );

EXPORT_MA void		MMSetIsObject( MMBlock, MMBoolean isObject );
EXPORT_MA MMBoolean	MMIsObject( ConstMMBlock );


//========================================================================================
// Operations on relocatable blocks (handles)
//========================================================================================

EXPORT_MA MMHandle	MMAllocateHandle( size_t );
EXPORT_MA MMHandle	MMAllocateHandleIn( size_t, MMHeapLocation );
EXPORT_MA void		MMFreeHandle( MMHandle );
EXPORT_MA MMHandle	MMCopyHandle (MMHandle );

EXPORT_MA size_t		MMGetHandleSize( MMHandle );
EXPORT_MA MMBoolean	MMSetHandleSize( MMHandle, size_t );

EXPORT_MA void*		MMLockHandle( MMHandle );
EXPORT_MA void		MMUnlockPtr( void* ptr );
EXPORT_MA void		MMUnlockHandle( MMHandle );

//========================================================================================
// Moving memory in general
//========================================================================================

EXPORT_MA void		MMMove( void *dst, const void *src, size_t size );


#pragma lib_export off

#ifdef __cplusplus
}
#endif


#endif /*_MEMMGR_*/