/*
	File:		PlatfMem.h

	Contains:	Platform layer interfaces

	Owned by:	Jens Alfke

	Copyright:	 1993-95 by Apple Computer, Inc., all rights reserved.
*/
#ifndef _PLATFMEM_
#define _PLATFMEM_

#ifndef _MEMCNFIG_
#include "MemCnfig.h"
#endif

#ifndef _MEMMGR_
#include "MemMgr.h"
#endif

#include <stddef.h>

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

//========================================================================================
// Defines
//========================================================================================

#ifndef __MWERKS__
	#define _FIL_ ""			/* MPW puts entire pathnames in; yuk! */
#else
	#define _FIL_ __FILE__
#endif

#define MM_ASSERT(condition)	\
	if(!MM_DEBUG || (condition)) ; else MM_ASSERT_FAILED( #condition, _FIL_ )

#define MM_WARN	\
	if(!MM_DEBUG) ; else MM_SHOW_WARNING

#define MM_PRINTING	0		// Set to 1 to enable MM_PRINT

#define MM_PRINT if( !MM_PRINTING ) ; else MMPrintf


//========================================================================================
// Types
//========================================================================================

typedef unsigned long	MMULong;

#if !defined(qMacApp)
// MacApp uses size_t exclusively
#ifdef __xlC
typedef unsigned long SIZE_T;
#else
typedef size_t SIZE_T;
#endif
#endif



//========================================================================================
// Constants
//========================================================================================

const MMBoolean kMMFalse = 0;
const MMBoolean kMMTrue  = 1;

#define			kMMNULL		(0)

// These settings should be specified by MacApp to correspond with low memory conditions without
// the low memory reserve, so that they don't trigger exceptions until after MacApp notices
// low memory conditions and after its low memory reserve etc. can be utilized.
// If these values aren't near 0 for MacApp, then MacApp's GrowZoneProc is unlikely to get called
// from here... revisit.

#if !defined(qMacApp)
const size_t	kPlatformMinFreeSpace	= 64 * 1024;	// Leave this much free
const size_t	kPlatformMinContigSpace	= 32 * 1024;	// Leave this size block available
#else
const size_t	kPlatformMinFreeSpace	= 32 * 1024;	// Leave this much free
const size_t	kPlatformMinContigSpace	= 10 * 1024;	// Leave this size block available
#endif

//========================================================================================
// Global function declarations
//========================================================================================

EXPORT_MA void PlatformZapMem( void *start, size_t size, MMULong value );
EXPORT_MA void *PlatformAllocateBlock(size_t size, MMHeapLocation);
EXPORT_MA void PlatformFreeBlock(void *ptr);

EXPORT_MA void MM_ASSERT_FAILED(const char *cond, const char *fil);
EXPORT_MA void MM_SHOW_WARNING(const char *msg, ...);

extern "C" int MMPrintf(const char *fmt, ...);

//----------------------------------------------------------------------------------------
// PlatformCopyMemory
//----------------------------------------------------------------------------------------

inline void PlatformCopyMemory(void *source, void *destination, size_t length)
{
	BlockMoveData(source, destination, (Size) length);
}

#if MM_DEBUG

#define LOWEST_POSSIBLE_ADDRESS()	((void*) &SystemZone()->heapData)
#define HIGHEST_POSSIBLE_ADDRESS()	((void*) gTotalMemory)

EXPORT_MA extern size_t gTotalMemory;

#endif

#endif  // _PLATFMEM_
