// UBeveledTargetBorderView.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __UBEVELEDTARGETBORDERVIEW__
#define __UBEVELEDTARGETBORDERVIEW__

// ACS

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

// MacApp

#ifndef __UTARGETBORDERVIEW__
#include "UTargetBorderView.h"
#endif


EXPORT_MA extern void InitUBeveledTargetBorderView();

#pragma mark TBeveledTargetBorderView

//----------------------------------------------------------------------------------------
// TBeveledTargetBorderView
// TTargetBorderView replacement subclass that draws AGA-style target border frame.
//----------------------------------------------------------------------------------------

class EXPORT_MA TBeveledTargetBorderView : public TTargetBorderView
{
MA_DECLARE_CLASS;

public:

	// TTargetBorderView-compatible constructor
	TBeveledTargetBorderView(const CViewPoint& itsLocation = gZeroVPt,
			const CViewPoint& itsSize = gZeroVPt,
			SizeDeterminer itsHSizeDet = sizeVariable,
			SizeDeterminer itsVSizeDet = sizeVariable,
			IDType itsTargetView = kNoIdentifier);

	// MacApp overrides
	virtual void DoPostCreate(TDocument* itsDocument);
	virtual void Draw(const CViewRect& area);
	virtual void ComputeBorderRegion(RgnHandle borderRegion);

protected:

	CRGBColor_AC	fEnabledBackgroundColor;
	CRGBColor_AC	fDisabledBackgroundColor;
};

#pragma mark TNotchedTargetBorderView

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderView
// Same as TBeveledTargetBorderView, but with a "notched" bottom right corner,
// such as the notch left by a pair of scroll bars
//----------------------------------------------------------------------------------------

class EXPORT_MA TNotchedTargetBorderView : public TBeveledTargetBorderView
{
MA_DECLARE_CLASS;

public:

	// TTargetBorderView-compatible constructor
	TNotchedTargetBorderView(const CViewPoint& itsLocation = gZeroVPt,
			const CViewPoint& itsSize = gZeroVPt,
			SizeDeterminer itsHSizeDet = sizeVariable,
			SizeDeterminer itsVSizeDet = sizeVariable,
			IDType itsTargetView = kNoIdentifier);

	// MacApp overrides
	virtual void ComputeBorderRegion(RgnHandle borderRegion);
};

#pragma mark TTargetBorderFrameView

//----------------------------------------------------------------------------------------
// TTargetBorderFrameView
// TTargetBorderView replacement subclass that also draws a
// TBorderFrameAdorner-type "3D" sunken frame. Provided
// for convenience: makes it unnecessary to attach an
// TBorderFrameAdorner to the subview that this target
// border view surrounds.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTargetBorderFrameView : public TBeveledTargetBorderView
{
MA_DECLARE_CLASS;

public:

	// TTargetBorderView-compatible constructor
	TTargetBorderFrameView(const CViewPoint& itsLocation = gZeroVPt,
			const CViewPoint& itsSize = gZeroVPt,
			SizeDeterminer itsHSizeDet = sizeVariable,
			SizeDeterminer itsVSizeDet = sizeVariable,
			IDType itsTargetView = kNoIdentifier);

	// MacApp overrides
	virtual void Draw(const CViewRect& area);
	virtual void Dim();
	virtual void SetEnable(bool state);
	virtual void Activate(bool entering);
};

#pragma mark TNotchedTargetBorderFrameView

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderFrameView
// Same as TTargetBorderFrameView, but with a "notched" bottom right corner.
//----------------------------------------------------------------------------------------

class EXPORT_MA TNotchedTargetBorderFrameView : public TBeveledTargetBorderView
{
MA_DECLARE_CLASS;

public:

	// TTargetBorderView-compatible constructor
	TNotchedTargetBorderFrameView(const CViewPoint& itsLocation = gZeroVPt,
			const CViewPoint& itsSize = gZeroVPt,
			SizeDeterminer itsHSizeDet = sizeVariable,
			SizeDeterminer itsVSizeDet = sizeVariable,
			IDType itsTargetView = kNoIdentifier);

	// MacApp overrides
	virtual void Draw(const CViewRect& area);
	virtual void Dim();
	virtual void Activate(bool entering);
	virtual void ComputeBorderRegion(RgnHandle borderRegion);
};

#endif // __UBEVELEDTARGETBORDERVIEW__
