//----------------------------------------------------------------------------------------
// UButton.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UBUTTON__
#define __UBUTTON__

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#if qGrayscaleAppearance

#	ifndef __CButton_AC__
#	include "CButton_AC.h"
#	endif

#endif // qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TButton: Mimics a typical toolbox button as used in a dialog.
//----------------------------------------------------------------------------------------

class EXPORT_MA TButton : public TCtlMgr
{
	MA_DECLARE_CLASS;
	
public:
	TButton(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								const CStr255_AC& itsTitle = gEmptyString);
		// Initializes the button setting the fDefaultChoice field to mButtonHit.

	virtual ~TButton();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;


	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);
		// If the hit is mButtonHit then flash the button provided you didn't hit
        // yourself.

	TAdorner* FindRRectAdorner();
		// Finds a TRRectAdorner or TCompatibilityAdorner with RRect
		// that gets attached to default buttons..

	virtual void SetDefaultOutline();
		// Sets the "default" outline state for this button.
		
#if qGrayscaleAppearance
	
	// MacApp overrides
	
	virtual void HiliteState(bool state, bool redraw);
		// Let grayscale control handle hiliting
		
	virtual void Hilite();
		// let grayscale control handle hiliting

	virtual void SetText(const CStr255_AC& itsText, bool redraw);
		// pass the title onto the grayscale control

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);
		// Adjust the AGA control bounds

	// AGA overrides

	virtual void CreateAGAControl();
		// Subclasses override to call new with an CControl_AC subclass
	
	virtual CControl_AC* GetAGAControl() const { return fAGAControl; }
		// Returns AGA control instance

	inline CButton_AC* GetAGAButton() const { return fAGAControl; }
		// Returns AGA button instance		

		
protected:
	CAutoPtr_AC<CButton_AC>	fAGAControl;

#endif // qGrayscaleAppearance

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	ResNumber	fStrListID;						// button's label:	STR# rsrc id
	short		fIndex;							//					index into STR#

};

EXPORT_MA void InitUButton();

#endif
