//----------------------------------------------------------------------------------------
// UCheckbox.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCHECKBOX__
#define __UCHECKBOX__

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#if qGrayscaleAppearance
#	ifndef __CCheckBox_AC__
#	include "CCheckBox_AC.h"
#	endif
#endif // qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TCheckBox: Provides the functionality of a toolbox checkbox.
//----------------------------------------------------------------------------------------

class EXPORT_MA TCheckBox : public TCtlMgr
{
	MA_DECLARE_CLASS;
	
public:
	TCheckBox(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								const CStr255_AC& itsTitle = gEmptyString,
								bool isTurnedOn = false);
		// Constructor

	virtual ~TCheckBox();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;


	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);
		// If the hit is mCheckBoxHit then toggle the checkbox.

	virtual bool IsOn() const;
		// Is the checkbox currently checked.

	virtual void SetState(SInt16 state, bool redraw);
		// Sets the state of the checkbox to state and if redraw is true redraws it.

	virtual void Toggle(bool redraw);
		// Toggles the state of the checkbox redraw it if redraw is true.

	virtual void ToggleIf(bool matchState, bool redraw);
		// Toggles the state of the checkbox if matchState is true redrawing if redraw is
        // true.

#if qGrayscaleAppearance
	
	// MacApp overrides

	virtual void HiliteState(bool state, bool redraw);

	virtual void Hilite();

	virtual void SetText(const CStr255_AC& itsText, bool redraw);

	// AGA overrides

	virtual void CreateAGAControl();
		// Subclasses override to call new with an CControl_AC subclass
	
	virtual CControl_AC* GetAGAControl() const { return fAGAControl; }
		// Returns AGA control instance

	inline CCheckBox_AC* GetAGACheckBox() const { return fAGAControl; }
		// Returns AGA checkbox instance

protected:
	CAutoPtr_AC<CCheckBox_AC>	fAGAControl;

#endif // qGrayscaleAppearance

public:
	ResNumber	fStrListID;						// check box's label:	STR# rsrc id
	short		fIndex;							//index into STR#
};

EXPORT_MA  void InitUCheckbox();

#endif
