//----------------------------------------------------------------------------------------
// UDialogTEView.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDIALOGTEVIEW__
#define __UDIALOGTEVIEW__

// MacApp 
#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

//----------------------------------------------------------------------------------------
// TDialogTEView: Provides a floating TEView that is placed over edit & number text fields
// when entering text.
//----------------------------------------------------------------------------------------

class EXPORT_MA TDialogTEView : public TTEView
{
	MA_DECLARE_CLASS;
	
public:
	TDialogTEView(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHDeterminer = sizeVariable,
								SizeDeterminer itsVDeterminer = sizeVariable,
								const CViewRect& itsInset = gZeroVRect,
								const CTextStyle_AC& itsTextStyle = gSystemStyle,
								short itsJustification = teFlushDefault,
								bool itsAutoWrap = true);
		// Constructor
		
	TDialogTEView(const TDialogTEView& inDialogTEView);
		// Copy Constructor

	virtual ~TDialogTEView();
		// Free associated components and then free this.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	//  Commands and Menus
	//------------------------------------------------------------------------------------

	virtual void InstallEditText(TEditText* theEditText, bool selectChars);
		// Installs the EditText adding this as a subview to EditText, then gets the text
        // from the EditText item and installs it in this.

	virtual void ComputeFrame(CViewRect& newFrame);
		// accounts for case in which horizontal determiner is sizeVariable should be
        // added to TTEView !!!

	virtual void ResignedTarget();

	virtual long WillingToResignTarget();
		// !!!

	virtual void TargetValidationFailed(long reason);

	virtual TEditText* GetEditText();
		// Get the edit text view, if any, that owns this TE view. 

	void AddToScroller();
		// Add self to the enclosing scroller.

#if qDrag
	
	//------------------------------------------------------------------------------------
	// Drag and Drop support
	//------------------------------------------------------------------------------------
	
	bool WillDrag(const CViewPoint& localMouse, const RgnHandle dragCursorRegion); // override

	virtual TView* DoGetDragProxy();
	
	inline short GetLastDropCaretOffset() {return fLastCaretOffset;}

	virtual short GetWillDragCursorID();	// Override
		// display quickdraw arrow cursor
		
	virtual short GetIsDraggingCursorID();	// Override
		// display quickdraw arrow cursor
	
#endif // qDrag

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	TEditText* fEditText;						// The edit text field to which the
                                                // floating TEView is currently tied.

	CAutoPtr_AC<TScroller> fScroller;			// The scroller that I am in for
                                                // mouseability.
};

#endif
