//----------------------------------------------------------------------------------------
// UDialogView.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDIALOGVIEW__
#define __UDIALOGVIEW__

#ifndef __UCONTROL__
#include "UControl.h"
#endif


//----------------------------------------------------------------------------------------
// TDialogView: Installs a TDialogBehavior in the window 
//----------------------------------------------------------------------------------------

class EXPORT_MA TDialogView : public TView
{
	MA_DECLARE_CLASS;
	
public:
	TDialogView(const CViewPoint& itsLocation = gZeroVPt,
								 const CViewPoint& itsSize = gZeroVPt,
								 SizeDeterminer itsHSizeDet = sizeVariable,
								 SizeDeterminer itsVSizeDet = sizeVariable,
								IDType itsDefItemID = kNoIdentifier,
								IDType itsCancelItemID = kNoIdentifier);
		// Constructor

	virtual ~TDialogView();
		// Destructor
		
	virtual void DoPostCreate(TDocument* itsDocument);	// Override
		// Overridden to install a TDialogBehavior in the window , if necessary,
		// and set its attributes (default item and cancel item)
		
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	IDType fDefaultItem;						// The identifier of the default item

	IDType fCancelItem;							// The identifier of item that cancels
                                                // dialog
};

//----------------------------------------------------------------------------------------
// Externally available function declaration.
//----------------------------------------------------------------------------------------

EXPORT_MA void InitUDialogView();
	// This routine must be called if DialogViews are used so that all the necessary
    // initialization can be performed.

#endif