//----------------------------------------------------------------------------------------
// UDisclosureBehavior.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __UDISCLOSUREBEHAVIOR__
#define __UDISCLOSUREBEHAVIOR__

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UDISCLOSURETRIANGLE__
#include "UDisclosureTriangle.h"
#endif

//----------------------------------------------------------------------------------------
// TDisclosureBehavior
//----------------------------------------------------------------------------------------
//
//	This behavior causes the view to which it is attached to change its dimensions to 
//	disclose or hide a child view, in response to events from a disclosure triangle control.
//

class EXPORT_MA TDisclosureBehavior : public TBehavior
{
	MA_DECLARE_CLASS;
	
public:
	
	TDisclosureBehavior(IDType itsTarget,			// View to hide or show.
						IDType itsControl,			// Disclosure triangle.
						EventNumber	controlEventID,	// Event sent by triangle.
						IDType itsIdentifier = kNoIdentifier);
		// Constructor

	TDisclosureBehavior(const TDisclosureBehavior& inDisclosureBehavior);
		// Copy Constructor
		
	virtual ~TDisclosureBehavior();

	virtual	void		InitializeDisclosureBehavior();
	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------
	TDisclosureBehavior(IDType itsIdentifier = kNoIdentifier);
		// Constructor for streaming
		
	virtual void ReadFrom(CStream_AC* aStream);

	virtual void WriteTo(CStream_AC* aStream) const;


	//------------------------------------------------------------------------------------
	// Event Filtering Methods
	//------------------------------------------------------------------------------------
	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);
								
private:

	IDType					fTargetViewID;	// View that gets hidden.
	IDType					fControlID;		// Disclosure triangle.
	TView*					fTarget;
	TDisclosureTriangle*	fControl;
	EventNumber				fEventNumber;
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

EXPORT_MA void InitUDisclosureBehavior();

#endif
