// UDisclosureTriangle.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __UDISCLOSURETRIANGLE__
#define __UDISCLOSURETRIANGLE__

// ACS

#ifndef __CDisclosureTriangle_AC__
#include "CDisclosureTriangle_AC.h"
#endif

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif

EXPORT_MA void InitUDisclosureTriangle();

#pragma mark TDisclosureTriangle

//----------------------------------------------------------------------------------------
// TDisclosureTriangle
// TControl subclass. Set/get the CDisclosureTriangle_AC state
// directly. Think of it as a check box -- it's either on or off.
//----------------------------------------------------------------------------------------


class EXPORT_MA TDisclosureTriangle : public TControl
{
	MA_DECLARE_CLASS;

public:

	// TControl-compatible constructor
	TDisclosureTriangle(const CViewPoint& itsLocation = gZeroVPt,
							 const CViewPoint& itsSize = gZeroVPt,
							 SizeDeterminer itsHSizeDet = sizeVariable,
							 SizeDeterminer itsVSizeDet = sizeVariable,
						     const CTextStyle_AC& itsTextStyle = gSystemStyle,
						     bool isDisclosed = CDisclosureTriangle_AC::kClosedState);

	virtual ~TDisclosureTriangle();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);

	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// MacApp overrides.
	//------------------------------------------------------------------------------------

	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void Draw(const CViewRect& area);

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate); 

	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);

	virtual void Activate(bool entering);

	virtual void DimState(bool state, bool redraw);

	virtual void Dim();

	virtual void SetEnable(bool state);

	//------------------------------------------------------------------------------------
	// Disclosure control.
	//------------------------------------------------------------------------------------

	inline CDisclosureTriangle_AC* GetAGAControl() const { return fAGAControl; }
	
	virtual void SetDisclosed(bool isDisclosed, bool redraw);
	
	inline bool IsDisclosed() const { return fIsDisclosed; }
	
protected:

	virtual void CreateAGAControl();

	CAutoPtr_AC<CDisclosureTriangle_AC>	fAGAControl;
	bool fIsDisclosed;
};


#endif // __UDISCLOSURETRIANGLE__
