//----------------------------------------------------------------------------------------
// UIcon.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UICON__
#define __UICON__

#ifndef __UCONTROL__
#include "UControl.h"
#endif

//Constants

const bool kPreferColor = true;				// argument to IIcon

typedef short TIconBitmask;

//----------------------------------------------------------------------------------------
// TIcon: Provides an icon that can behave as a button.
//----------------------------------------------------------------------------------------

class EXPORT_MA TIcon : public TControl
{
	MA_DECLARE_CLASS;
	
public:
	
	//----------------------------------------------------------------------------------------
	// Enums
	//----------------------------------------------------------------------------------------
	
	enum eAlignmentFlag
	{
		kNoAlignment			= 0x00000000,	
		kVerticalCenter			= 0x00000001,
		kTop					= 0x00000002,
		kBottom 				= 0x00000003,
		kHorizontalCenter		= 0x00000004,
		kLeft 					= 0x00000008,
		kRight				 	= 0x0000000C,
		kAbsoluteCenter 		= kVerticalCenter + kHorizontalCenter,
		kCenterTop 				= kTop + kHorizontalCenter,
		kCenterBottom 			= kBottom + kHorizontalCenter,
		kCenterLeft 			= kVerticalCenter + kLeft,
		kTopLeft 				= kTop + kLeft,
		kBottomLeft 			= kBottom + kLeft,
		kCenterRight 			= kVerticalCenter + kRight,
		kTopRight 				= kTop + kRight,
		kBottomRight 			= kBottom + kRight
	};	

	enum eTransformFlag
	{
		kNoTransform			= 0x00000000,
		kDisabled	 			= 0x00000001,
		kOffline 				= 0x00000002,
		kOpen 					= 0x00000003,
		kSelected 				= 0x00004000,
		kSelectedDisabled 		= kSelected + kDisabled,
		kSelectedOffline 		= kSelected + kOffline,
		kSelectedOpen 			= kSelected + kOpen
	};

	enum eLabelFlag
	{
		kNoLabel_AC				= 0x00000000,
		kLabel1 				= 0x00000100,
		kLabel2 				= 0x00000200,
		kLabel3 				= 0x00000300,
		kLabel4 				= 0x00000400,
		kLabel5 				= 0x00000500,
		kLabel6 				= 0x00000600,
		kLabel7 				= 0x00000700
	};

	enum eIconSelectorFlag
	{
		kLarge1Bit 				= 0x00000001,
		kLarge4Bit				= 0x00000002,
		kLarge8Bit				= 0x00000004,
		kSmall1Bit				= 0x00000100,
		kSmall4Bit				= 0x00000200,
		kSmall8Bit				= 0x00000400,
		kMini1Bit				= 0x00010000,
		kMini4Bit				= 0x00020000,
		kMini8Bit				= 0x00040000,
		kAllLargeData			= 0x000000FF,
		kAllSmallData			= 0x0000FF00,
		kAllMiniData			= 0x00FF0000,
		kAll1BitData			= kLarge1Bit + kSmall1Bit + kMini1Bit,
		kAll4BitData			= kLarge4Bit + kSmall4Bit + kMini4Bit,
		kAll8BitData			= kLarge8Bit + kSmall8Bit + kMini8Bit,
		
		kAllAvailableData		= 0xFFFFFFFF
	};
	
	//----------------------------------------------------------------------------------------
	// Interface
	//----------------------------------------------------------------------------------------
	
	TIcon(const CViewPoint& itsLocation = gZeroVPt,
				const CViewPoint& itsSize = gZeroVPt,
				SizeDeterminer itsHSizeDet = sizeVariable,
				SizeDeterminer itsVSizeDet = sizeVariable,
				ResNumber itsRsrcID = kNoResource,
				eIconSelectorFlag loadFlag = kAllAvailableData);
		// Constructor
		
	TIcon(const TIcon& inIcon);
		// Copy Constructor

	virtual ~TIcon();
		// Destructor.


	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// Data member accessors.
	//------------------------------------------------------------------------------------

	virtual void ReleaseIconSuite();
		// Releases the handle to the icon.

	virtual void SetIconSuite(Handle theIconSuite, bool redraw);
		// Sets fDataHandle to theIcon, redrawing if redraw is true.
	
	virtual Handle GetIconSuite() const;

	virtual void SetIconSuiteRsrcID(ResNumber itsRsrcID, 
										bool redraw,
				  						eIconSelectorFlag loadFlag = kAllAvailableData);
		// This loads a new icon suite, and sets fRsrcID
	
	ResNumber GetIconSuiteRsrcID() const;
		// Gets current id.
	
	virtual CRect_AC GetIconRect();
		// return the rect to use when plotting the icon or getting its icon info
	
	//------------------------------------------------------------------------------------
	// Imaging.
	//------------------------------------------------------------------------------------
		
		// Alignment
	virtual void SetAlignmentType(eAlignmentFlag newAlignment);
	
	eAlignmentFlag GetAlignmentType() const;

		// Transform
	virtual void SetTransfromType(eTransformFlag newTransform);
	
	eTransformFlag GetTransfromType() const;

		// Labels
	virtual void SetLabel(eLabelFlag newLabel);
	
	eLabelFlag GetLabel() const;

	virtual void Draw(const CViewRect& area);
		// Responsible for drawing the icon and any required adornments.

	virtual void Hilite();

	virtual void HiliteState(bool state, bool redraw);

	virtual void Dim();

	virtual void DimState(bool state, bool redraw);
	
	//------------------------------------------------------------------------------------
	// Misc.
	//------------------------------------------------------------------------------------
	
	virtual bool ContainsMouse(const CViewPoint& theMouse);
		// Returns true if 'theMouse is currently in the controls area.

	
protected:

	virtual void Plot();

	inline void SetIconSuite(Handle theIconSuite) { fIconSuite = theIconSuite; }

	inline void SetIconSuiteRsrcID(ResNumber itsRsrcID) { fRsrcID = itsRsrcID; }

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	Handle 				fIconSuite;
	ResNumber 			fRsrcID;
	TIconBitmask 		fAlign;
	TIconBitmask 		fTransform;
	TIconBitmask		fLabel;
};

//------------------------------------------------------------------------------------
// Inlines
//------------------------------------------------------------------------------------

inline ResNumber TIcon::GetIconSuiteRsrcID() const 
{
 	return fRsrcID; 
}

inline void TIcon::SetAlignmentType(eAlignmentFlag newAlignment) 
{ 
	fAlign = newAlignment; 
}

inline TIcon::eAlignmentFlag TIcon::GetAlignmentType() const
{ 
	return (eAlignmentFlag) fAlign; 
}

inline void TIcon::SetTransfromType(eTransformFlag newTransform) 
{ 	
	fTransform = newTransform; 
}

inline TIcon::eTransformFlag TIcon::GetTransfromType() const 
{ 	
	return (eTransformFlag) fTransform; 
}

inline void TIcon::SetLabel(eLabelFlag newLabel)
{
	fLabel = newLabel;
}

inline TIcon::eLabelFlag TIcon::GetLabel() const
{
	return (eLabelFlag) fLabel;
}

EXPORT_MA void InitUIcon(bool InitAllIcons);

#endif