// UIconButton.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __UICONBUTTON__
#define __UICONBUTTON__

// These classes depend on functionality derived from TCtlMgr when qGrayscaleAppearance 
// is true, so they are not defined when qGrayscaleAppearance is false.

#if qGrayscaleAppearance

// MacApp

#ifndef __UNUMBERTEXT__
#include "UNumberText.h"
#endif

#ifndef __UCLUSTER__
#include "UCluster.h"
#endif

#ifndef __UBUTTON__
#include "UButton.h"
#endif

#ifndef __UCHECKBOX__
#include "UCheckBox.h"
#endif

#ifndef __URADIO__
#include "URadio.h"
#endif

#ifndef __UPOPUP__
#include "UPopup.h"
#endif

// ACS

#ifndef __CIconButton_AC__
#include "CIconButton_AC.h"
#endif


EXPORT_MA void InitUIconButton();

#pragma mark TIconButton

//----------------------------------------------------------------------------------------
// TIconButton
// TButton replacement subclass that implements an icon button that behaves
// like a TButton -- you click it and it sends its fEventNumber. When
// instantiated from a view resource, the fUserArea is used to obtain the
// button's icon ID. You can also set the icon ID of the CIconButton_AC
// directly (cast the fAGAControl to CIconButton_AC*).
//----------------------------------------------------------------------------------------

class EXPORT_MA TIconButton : public TButton
{
MA_DECLARE_CLASS;

	// TButton-compatible constructor
	TIconButton(const CViewPoint& itsLocation = gZeroVPt,
							const CViewPoint& itsSize = gZeroVPt,
							SizeDeterminer itsHSizeDet = sizeVariable,
							SizeDeterminer itsVSizeDet = sizeVariable,
							const CStr255_AC& itsTitle = gEmptyString,
							ResNumber itsRsrcID = kNoResource);

	virtual ~TIconButton();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

protected:

	//------------------------------------------------------------------------------------
	// TButton overrides
	//------------------------------------------------------------------------------------

	virtual bool IsStandardControl() const;
	
	virtual void CreateAGAControl();
	
	ResNumber fRsrcID;
};

#pragma mark TIconCheckBox

//----------------------------------------------------------------------------------------
// TIconCheckBox
// TCheckBox replacement subclass that implements an icon button that behaves
// like a check box -- you click it and it toggles its on/off state. When
// instantiated from a view resource, the fUserArea is used to obtain the
// button's icon ID. You can also set the icon ID of the CIconCheckBox_AC
// directly (cast the fAGAControl to CIconCheckBox_AC*). If you want the on/off
// states to have different icons, you can specify different icon IDs.
//----------------------------------------------------------------------------------------

class EXPORT_MA TIconCheckBox : public TCheckBox
{
MA_DECLARE_CLASS;

	// TCheckBox-compatible constructor
	TIconCheckBox(const CViewPoint& itsLocation = gZeroVPt,
							const CViewPoint& itsSize = gZeroVPt,
							SizeDeterminer itsHSizeDet = sizeVariable,
							SizeDeterminer itsVSizeDet = sizeVariable,
							const CStr255_AC& itsTitle = gEmptyString,
							bool isTurnedOn = false,
							ResNumber itsRsrcID = kNoResource);

	virtual ~TIconCheckBox();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

protected:

	//------------------------------------------------------------------------------------
	// TButton overrides
	//------------------------------------------------------------------------------------

	virtual bool IsStandardControl() const;
	
	virtual void CreateAGAControl();

	ResNumber fRsrcID;
};

#pragma mark TIconRadio

//----------------------------------------------------------------------------------------
// TIconRadio
// TRadio replacement subclass that implements an icon button that behaves
// like a radio button -- you click it and it turns on, and notifies its
// cluster so that the others in its group are turned off. When
// instantiated from a view resource, the fUserArea is used to obtain the
// button's icon ID. You can also set the icon ID of the CIconRadioButton_AC
// directly (cast the fAGAControl to CIconRadioButton_AC*). If you want the on/off
// states to have different icons, you can specify different icon IDs.
//----------------------------------------------------------------------------------------

class EXPORT_MA TIconRadio : public TRadio
{
MA_DECLARE_CLASS;

	// TRadio-compatible constructor
	TIconRadio(const CViewPoint& itsLocation = gZeroVPt,
							const CViewPoint& itsSize = gZeroVPt,
							SizeDeterminer itsHSizeDet = sizeVariable,
							SizeDeterminer itsVSizeDet = sizeVariable,
							const CStr255_AC& itsTitle = gEmptyString,
							bool isTurnedOn = false,
							ResNumber itsRsrcID = kNoResource);

	virtual ~TIconRadio();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

protected:

	//------------------------------------------------------------------------------------
	// TButton overrides
	//------------------------------------------------------------------------------------

	virtual bool IsStandardControl() const;
	
	virtual void CreateAGAControl();

	ResNumber fRsrcID;
};

#pragma mark TIconRadio

//----------------------------------------------------------------------------------------
// TIconPopup
// TPopup replacement subclass that implements an icon button that behaves
// like a popup -- you click it and it displays a popup menu. When
// instantiated from a view resource, the fUserArea is used to obtain the
// button's icon ID. You can also set the icon ID of the CIconRadioButton_AC
// directly (cast the fAGAControl to CIconRadioButton_AC*). If you want the on/off
// states to have different icons, you can specify different icon IDs.
//----------------------------------------------------------------------------------------

class EXPORT_MA TIconPopup : public TPopup
{
MA_DECLARE_CLASS;

	// TPopup-compatible constructor
	TIconPopup(const CViewPoint& itsLocation = gZeroVPt,
							const CViewPoint& itsSize = gZeroVPt,
							SizeDeterminer itsHSizeDet = sizeVariable,
							SizeDeterminer itsVSizeDet = sizeVariable,
							short itsMenuID = kNoResource,				// rsrc ID of 'MENU' or 'CMNU' rsrc 
							short itsCurrentItem = 0,
							short itsItemOffset = 0,					// width of the popup title area 
							short itsStrListID = kNoResource,			// popup's title:	STR# rsrc id
							short itsIndex = kNoResource,				//					index into STR#
							/* Style */ short itsStyle = popupTitleNoStyle, // title's style 
							short itsJust= popupTitleLeftJust,			// title's justification 
							bool useAddResMenu = false,					// true if want to AddResMenu (can use
																		// this option iff menuID != kNoResource)
							ResType useAddResMenuResType = kNoIdentifier, // used if useAddResMenu is true
							const CTextStyle_AC& itsTextStyle = gSystemStyle,
							ResNumber itsRsrcID = kNoResource); // popup's text style
		// Initializes the popup.

	virtual ~TIconPopup();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

protected:

	//------------------------------------------------------------------------------------
	// TButton overrides
	//------------------------------------------------------------------------------------

	virtual bool IsStandardControl() const;
	
	virtual void CreateAGAControl();

	ResNumber fRsrcID;
};

#endif // qGrayscaleAppearance

#endif // __UICONBUTTON__
