//----------------------------------------------------------------------------------------
// UNumberText.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UNUMBERTEXT__
#define __UNUMBERTEXT__

#ifndef __UEDITTEXT__
#include "UEditText.h"
#endif

//----------------------------------------------------------------------------------------
// TNumberText: This subclass of TEditText that allows the handling of integer numbers.
// Range checking will be performed on the numbers entered to see if they are in the range
// fMinimum to fMaximum.
//----------------------------------------------------------------------------------------

class EXPORT_MA TNumberText : public TEditText
{
	MA_DECLARE_CLASS;
	
public:
	TNumberText(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								long itsValue = 0,
								long itsMinimum = 0,
								long itsMaximum = 0);
		// Constructor

	virtual ~TNumberText();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;


	virtual long GetValue() const;
		// Returns the current value by getting the CPascalStr_AC and converting it to a number.

	virtual void SetValue(long newValue, bool redraw);
		// Sets the current value by converting the number to a CPascalStr_AC and then calling SetText.

	virtual long GetValidationError();
		// Does a range check on the number and if it is out of range returns the
        // appropriate error code.

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	long fMinimum;

	long fMaximum;
};

EXPORT_MA  void InitUNumberText();

#endif