//----------------------------------------------------------------------------------------
// UPattern.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPATTERN__
#define __UPATTERN__

#ifndef __UCONTROL__
#include "UControl.h"
#endif

//----------------------------------------------------------------------------------------
// TPattern: Provides an object that can display a pattern and can also behave as a button
// if required.
//----------------------------------------------------------------------------------------

class EXPORT_MA TPattern : public TControl
{
	MA_DECLARE_CLASS;
	
public:
	TPattern(const CViewPoint& itsLocation = gZeroVPt,
				const CViewPoint& itsSize = gZeroVPt,
				SizeDeterminer itsHSizeDet = sizeVariable,
				SizeDeterminer itsVSizeDet = sizeVariable,
				ResNumber itsRsrcID = kNoResource,
				bool preferColor = true);
		// Constructor
		
	TPattern(const TPattern& inPattern);
		// Copy Constructor

	virtual ~TPattern();
		// Release the pattern then call Inherited::Free.

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// Data member accessors.
	//------------------------------------------------------------------------------------

	virtual void ReleasePattern();
		// Releases the pattern by disposing fDataHandle.

	virtual void SetPattern(Handle thePattern, bool redraw);
		// Sets fDataHandle to thePattern, redrawing if redraw is true.

	virtual void SetPatternRsrcID(ResNumber itsRsrcID, bool redraw);
		// setter for fRsrcID

	//------------------------------------------------------------------------------------
	// Imaging.
	//------------------------------------------------------------------------------------

	virtual void Draw(const CViewRect& area);
		// Responsible for drawing the pattern and any required adornments.

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	Handle fDataHandle;

	ResNumber fRsrcID;

	bool fPreferColor;						// tells us how to GET the pattern

	bool fIsColor;							// tells us how to DRAW or FREE the
                                                // pattern
};

EXPORT_MA  void InitUPattern();

#endif