//----------------------------------------------------------------------------------------
// UPicture.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPICTURE__
#define __UPICTURE__

#ifndef __UCONTROL__
#include "UControl.h"
#endif

//----------------------------------------------------------------------------------------
// TPicture: Provides an object that can display a PICT and if required behave like a
// button.
//----------------------------------------------------------------------------------------

class EXPORT_MA TPicture : public TControl
{
	MA_DECLARE_CLASS;
	
public:
	TPicture(const CViewPoint& itsLocation = gZeroVPt,
				const CViewPoint& itsSize = gZeroVPt,
				SizeDeterminer itsHSizeDet = sizeVariable,
				SizeDeterminer itsVSizeDet = sizeVariable,
				ResNumber itsRsrcID = kNoResource);
		// Constructor
		
	TPicture(const TPicture& inPicture);
		// Copy Constructor

	virtual ~TPicture();
		// Release the picture then call Inherited::Free.

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// Data member accessors.
	//------------------------------------------------------------------------------------

	virtual void ReleasePicture();
		// Releases the PICT by disposing of fDataHandle.

	virtual void SetPicture(PicHandle thePicture, bool redraw);
		// Sets fDataHandle to thePicture, redrawing if redraw is true.

	virtual void SetPictureRsrcID(ResNumber itsRsrcID, bool redraw);
		// setter for fRsrcID

	//------------------------------------------------------------------------------------
	// Imaging.
	//------------------------------------------------------------------------------------

	virtual void Draw(const CViewRect& area);
		// Responsible for drawing the PICT and any required adornments.
		
	//------------------------------------------------------------------------------------
	// Drag & Drop
	//------------------------------------------------------------------------------------
#if qDrag
	
	virtual bool WillDrag(const CViewPoint& theMouse, const RgnHandle dragCursorRegion);
		// return true if picture content exists, and the view is draggable

	virtual void DoAddDragContent();
		// Install PICT flavor data as a promise

	virtual bool WillAcceptDrop(CDragItemIterator& dragItemIterator);
		// return true if drag is a single item available as a 'PICT'
		
	virtual TCommand* DoMakeDragDropCommand(CommandNumber 		itsCommandNumber,
											CDragItemIterator&	dragItemIterator);
		// Create and return a command that will be posted by the drag session.
	
	virtual void DoFulfillPromise(TDragItem* promisedItem);
		// Provide promised data on demand.
		
#endif // qDrag

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	PicHandle fDataHandle;

	ResNumber fRsrcID;
};

//----------------------------------------------------------------------------------------
// TPictureCommand: Command that manages inserting and removing data from a TPicture.
//----------------------------------------------------------------------------------------

class EXPORT_MA TPictureCommand : public TCommand
{
public:
		TPictureCommand(ResNumber newRsrcID = kNoResource, PicHandle newPicHandle = NULL,
				TPicture* itsPictureView = NULL, CommandNumber itsCommandNumber = cNoCommand, 
				TCommandHandler* itsContext = NULL, bool canUndo = true, bool causesChange = true, 
				MDependable_AC* objectToNotify = NULL);
			// Constructor
			
		virtual ~TPictureCommand();	

		virtual void DoIt();
		
		virtual void UndoIt();

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	TPicture*	fPictureView;
	PicHandle	fSavedDataHandle;
	ResNumber	fSavedRsrcID;
};

EXPORT_MA void InitUPicture();

#endif