// UProgressIndicator.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __UPROGRESSINDICATOR__
#define __UPROGRESSINDICATOR__

// ACS

#ifndef __CProgressIndicator_AC__
#include "CProgressIndicator_AC.h"
#endif

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif


EXPORT_MA  void InitUProgressIndicator();


#pragma mark TProgressIndicator

//----------------------------------------------------------------------------------------
// TProgressIndicator
// TControl subclass. Access the CProgressIndicator_AC directly. For
// determinate progress, set the min/max range, and then call SetValue
// or Increment repeatedly; you can also update the origin value if
// desired. For indeterminate progress, set min=max and then call Animate
// repeatedly.
//----------------------------------------------------------------------------------------

class EXPORT_MA TProgressIndicator : public TCtlMgr
{
MA_DECLARE_CLASS;

public:

	// TControl-compatible constructor
	TProgressIndicator(const CViewPoint& itsLocation = gZeroVPt,
							 const CViewPoint& itsSize = gZeroVPt,
							 SizeDeterminer itsHSizeDet = sizeVariable,
							 SizeDeterminer itsVSizeDet = sizeVariable,
							 bool isDeterminate = true);

	virtual ~TProgressIndicator();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);

	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// MacApp overrides.
	//------------------------------------------------------------------------------------

	virtual bool DoIdle(IdlePhase phase);
	
	// For an indeterminate progress indicator, you can use these functions
	// to set it running or stop it. It will create an idler behavior to
	// ensure it gets idle time.
	
	void	StartAutoAnimate();

	void	StopAutoAnimate();
	
	virtual bool IsEmulatingControls() const;
	// Returns whether this class should emulate controls

	CControl_AC*	GetAGAControl() const { return fAGAControl; }
	
protected:
	CAutoPtr_AC<CProgressIndicator_AC>	fAGAControl;

	virtual void CreateAGAControl();
	
	bool fIsDeterminate;
};

#endif // __UPROGRESSINDICATOR__
