//----------------------------------------------------------------------------------------
// UQD3DViewer.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UQD3DViewer__
#define __UQD3DViewer__

#if !q68K

//MacApp
#ifndef __UVIEW__
#include "UView.h"
#endif

//QuickDraw3D
#ifndef  QD3DViewer_h
#include "QD3DViewer.h"
#endif

// Forward Declarations 
class TFile;

//========================================================================================
// CLASS TQD3DViewer
//========================================================================================

class EXPORT_MA TQD3DViewer: public TView
{
  public:	
	TQD3DViewer(const CViewPoint& itsLocation = gZeroVPt,
						const CViewPoint& itsSize = gZeroVPt,
						SizeDeterminer itsHSizeDet = sizeVariable,
				  		SizeDeterminer itsVSizeDet = sizeVariable); 

	virtual ~TQD3DViewer();
	
	virtual void* Clone() const;

	virtual void DoPostCreate(TDocument* itsDocument); 
						   
	virtual void Draw(const CViewRect& area); 
	
	virtual void DoSetupMenus(); 
	
	virtual void DoMenuCommand(CommandNumber aCommandNumber);
	
	virtual void DoMouseCommand(CViewPoint& theMouse,TToolboxEvent* event,CPoint_AC hysteresis);
	
	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);


	// ---------------------------------------------------------
	// 3DViewer/View Information
	// ---------------------------------------------------------
	
	virtual void Add3DViewerToView(unsigned long theViewerFlags);
	
	void Remove3DViewerFromView();
	
	virtual unsigned long GetViewerState();
	
	virtual TQ3ViewObject Get3DViewFromViewer();
	
	// ---------------------------------------------------------
	// Rendering Services
	// ---------------------------------------------------------
	
	virtual void SetRenderer(TQ3ObjectType renderType,bool redraw);
	
	virtual TQ3RendererObject* GetRenderer();

	// ---------------------------------------------------------
	// Get Picture
	// ---------------------------------------------------------
	
	virtual PicHandle AsPict();
	
	// ---------------------------------------------------------
	// Background Colors
	// ---------------------------------------------------------
	
	virtual void SetBackgroundColor(TQ3ColorARGB* newColor, bool redraw);
	
	virtual void SetBackgroundColor(CRGBColor_AC newColor, float alpha, bool redraw);
	
	virtual CRGBColor_AC	  GetRGBBackgroundColor();
	
	virtual TQ3ColorARGB* GetBackgroundColor();

	// ---------------------------------------------------------
	// Viewer Information
	// ---------------------------------------------------------
			
	inline bool HasViewer() const { return fQD3DViewer != NULL; }
			
	// ---------------------------------------------------------
	// View Flags
	// ---------------------------------------------------------
	
	virtual void SetViewerFlags(unsigned long theViewerFlags);
	
	virtual unsigned long GetViewerFlags();
	
	// ---------------------------------------------------------
	// Viewer Button Calls
	// ---------------------------------------------------------
	
	virtual void SetCurrentButton(unsigned long theButton);
	
	virtual unsigned long GetCurrentButton();

	virtual CRect_AC GetButtonRect(unsigned long theButton);

	// ---------------------------------------------------------
	// Read/Write Data Calls
	// ---------------------------------------------------------
	
	virtual void 	UseFile(const CFSSpec_AC& theFile);
	
	virtual void 	Write3DMFFile(TFile* theFile);

	
  protected:
	TQ3ViewerObject fQD3DViewer;
	bool			fUseManualCreation;
	unsigned long	fViewerFlags;
};

//========================================================================================
// CLASS TQD3DCutPasteCommand
//========================================================================================
class EXPORT_MA TQD3DCutPasteCommand: public TCommand
{
  public:	
	TQD3DCutPasteCommand(CommandNumber itsCommandNumber = cNoCommand,
						TQ3ViewerObject itsQD3DViewer = NULL);

	virtual ~TQD3DCutPasteCommand();
	
	virtual void DoIt();
	
	virtual void UndoIt();
	
  protected:
	TQ3ViewerObject fQD3DViewer;
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

EXPORT_MA  void InitUQD3DViewer();
	// Call this routine at initialization time
	
#endif
#endif
