//----------------------------------------------------------------------------------------
// URadio.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __URADIO__
#define __URADIO__

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#if qGrayscaleAppearance
#	ifndef __CRadioButton_AC__
#	include "CRadioButton_AC.h"
#	endif
#endif // qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TRadio: Provides the functionality of a toolbox radio button.
//----------------------------------------------------------------------------------------

class EXPORT_MA TRadio : public TCtlMgr
{
	MA_DECLARE_CLASS;
	
public:
	TRadio(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								const CStr255_AC& itsTitle = gEmptyString,
								bool isTurnedOn = true);
		// Constructor

	virtual ~TRadio();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	virtual void DoPostCreate(TDocument* itsDocument); // Override
		// Calls HandleEvent(mRadioHit) if the radio is on so a cluster can synch up
	
	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);
		// If the hit is mRadioHit then toggle the radio button.

	virtual bool IsOn() const;
		// Is the radio button currently on.

	virtual void SetState(SInt16 state, bool redraw);
		// Sets the state of the radio button to state, redrawing if redraw is true.

	virtual void Toggle(bool redraw);
		// Toggles the state of the radio button, redrawing if redraw is true.

	virtual void ToggleIf(bool matchState, bool redraw);
		// Only toggles the state of the radio button if matchState is true, redrawing if
        // redraw is true.

#if qGrayscaleAppearance
	
	// MacApp overrides

	virtual void HiliteState(bool state, bool redraw);

	virtual void Hilite();

	virtual void SetText(const CStr255_AC& itsText, bool redraw);

	// extension overrides

	virtual void CreateAGAControl();
		// Subclasses override to call new with an CControl_AC subclass
	
	virtual CControl_AC* GetAGAControl() const { return fAGAControl; }
		// Returns AGA control instance

	inline CRadioButton_AC* GetAGARadio() const { return fAGAControl; }
		// Returns AGA button instance

protected:
	CAutoPtr_AC<CRadioButton_AC>	fAGAControl;

#endif // qGrayscaleAppearance

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
	
public:
	ResNumber	fStrListID;						// radio's label:	STR# rsrc id
	short		fIndex;							//					index into STR#

};

EXPORT_MA  void InitURadio();

#endif
