// UScrollBarProportional.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __USCROLLBARPROPORTIONAL__
#define __USCROLLBARPROPORTIONAL__


// These classes depend on functionality derived from TCtlMgr when qGrayscaleAppearance 

// These classes are the exception to the rule (that other new classes using AGA have
// all the necessary functionality repeated in the derived classes).

#if qGrayscaleAppearance

// ACS

#ifndef __CScrollBar_AC__
#include "CScrollBar_AC.h"
#endif

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif


#pragma mark TScrollBarProportional

//----------------------------------------------------------------------------------------
// TScrollBarProportional
// TScrollBar replacement subclass. To use live scrolling, set the
// CScrollBar_AC state directly and either install a notification routine
// or handle the scroll bar's fEventNumber by checking the current value.
// To use proportional indicator, set the CScrollBar_AC state directly.
//
// Please note that TScrollBar and TScrollerScrollBar are different!
//----------------------------------------------------------------------------------------

class EXPORT_MA TScrollBarProportional : public TScrollBar
{
MA_DECLARE_CLASS;

public:
	
	// TScrollBar-compatible constructor
	TScrollBarProportional(const CViewPoint& itsLocation = gZeroVPt,
							const CViewPoint& itsSize = gZeroVPt,
							SizeDeterminer itsHSizeDet = sizeVariable,
							SizeDeterminer itsVSizeDet = sizeVariable,
							VHSelect itsDirection = hSel,
							long itsVal = 0,
							long itsMin = 0,
							long itsMax = 0);

	virtual ~TScrollBarProportional();

	// MacApp overrides
	virtual void Draw(const CViewRect& area);
	virtual void SetLongVal(ViewCoordinate itsVal, bool redraw);
	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);
	virtual void HiliteState(bool state, bool redraw);
	virtual void Hilite();
	virtual void Activate(bool entering);
	virtual void SetLongMax(ViewCoordinate itsMax, bool redraw);
	
	// Notifier override
	
	virtual void	HandleNotification(SInt32 dataValue);

	virtual CControl_AC* GetAGAControl() const { return fAGAControl; }	

	inline CScrollBar_AC* GetAGAScrollBar() { return fAGAControl; }

protected:
				
	virtual bool IsEmulatingControls() const;

	virtual void CreateAGAControl();		

	static void RealAGANotifier(CIndicator_AC* theIndicator, SInt32 dataValue, TScrollBarProportional* notifierTarget);

	CAutoPtr_AC<CScrollBar_AC>	fAGAControl;


};

#pragma mark TScrollerScrollBarProportional

//----------------------------------------------------------------------------------------
// TScrollerScrollBarProportional
// TScrollerScrollBar replacement subclass. To use live scrolling, just set
// the CScrollBar_AC state directly; the scroller will get the live scroll
// nofication messages automatically. To use proportional indicator,
// set the CScrollBar_AC state directly.
//
// Please note that TScrollBar and TScrollerScrollBar are different!
//----------------------------------------------------------------------------------------

class EXPORT_MA TScrollerScrollBarProportional : public TScrollerScrollBar
{
MA_DECLARE_CLASS;

public:
	// TScrollerScrollBar-compatible constructor
	TScrollerScrollBarProportional(const CViewPoint& itsLocation = gZeroVPt,
							const CViewPoint& itsSize = gZeroVPt,
							SizeDeterminer itsHSizeDet = sizeVariable,
							SizeDeterminer itsVSizeDet = sizeVariable,
							VHSelect itsDirection = hSel,
							long itsMax = 0,
							TScroller* itsScroller = NULL);

	virtual ~TScrollerScrollBarProportional();


	// MacApp overrides
	virtual void Draw(const CViewRect& area);
	virtual void SetFrame(const CViewRect& newFrame, bool invalidate); 
	virtual void AttachScroller(TScroller* itsScroller);
	virtual void SetLongVal(ViewCoordinate itsVal, bool redraw);
	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);
	virtual void HiliteState(bool state, bool redraw);
	virtual void Hilite();
	virtual void Activate(bool entering);
	virtual void SetLongMax(ViewCoordinate itsMax, bool redraw);
	
	virtual void	HandleNotification(SInt32 dataValue);
	virtual void	UpdateScrollAmounts();
	
	virtual bool	IsEmulatingControls() const;
	
	virtual CControl_AC* GetAGAControl() const { return fAGAControl; }

	inline CScrollBar_AC* GetAGAScrollBar() { return fAGAControl; }
	

protected:

	virtual void CreateAGAControl();

	static void RealAGANotifier(CIndicator_AC* theIndicator, SInt32 dataValue, TScrollerScrollBarProportional* notifierTarget);

	CAutoPtr_AC<CScrollBar_AC>	fAGAControl;
};


EXPORT_MA  void InitUScrollBarProportional();

#endif // qGrayscaleControls

#endif // __USCROLLBARPROPORTIONAL__
