// USlider.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __USLIDER__
#define __USLIDER__

#ifndef __CSlider_AC__
#include "CSlider_AC.h"
#endif

#ifndef __UCONTROL__
#include "UControl.h"
#endif

EXPORT_MA  void InitUSlider();


#pragma mark TSlider

//----------------------------------------------------------------------------------------
// TSlider
// TControl subclass. To set/get the value, access the CSlider_AC
// directly. When instantiated from a view resource, the fUserArea,
// if non-zero, is used as the resource ID of an 'STR#' resource
// that contains the slider labels; in this case, the range is
// automatically set to match the number of labels, and the slider
// will have a pointy indicator. If the ID is zero or invalid, the
// slider will be rectangular.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSlider : public TControl
{
MA_DECLARE_CLASS;

public:

	// TControl-compatible constructor
	TSlider(const CViewPoint& itsLocation = gZeroVPt,
							 const CViewPoint& itsSize = gZeroVPt,
							 SizeDeterminer itsHSizeDet = sizeVariable,
							 SizeDeterminer itsVSizeDet = sizeVariable,
						     const CTextStyle_AC& itsTextStyle = gSystemStyle,
						     ResNumber itsLabelsID = kNoResource);

	virtual ~TSlider();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);

	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// MacApp overrides.
	//------------------------------------------------------------------------------------

	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void Draw(const CViewRect& area);

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate); 

	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);

	virtual void Activate(bool entering);

	virtual void DimState(bool state, bool redraw);

	virtual void Dim();

	virtual void SetEnable(bool state);
	
	CSlider_AC* GetAGAControl() const { return fAGAControl; }
		
	// Notifier	callback	
	virtual void	HandleNotification(SInt32 dataValue);

protected:

	virtual void 	CreateAGAControl();

	static void RealAGANotifier(CIndicator_AC* theIndicator, SInt32 dataValue, TSlider* notifierTarget);

	CAutoPtr_AC<CSlider_AC>	fAGAControl;
	ResNumber fLabelsID;	// String list resource containing labels for this slider.
	bool fRightJustify;
	ViewCoordinate fMinimum;
	ViewCoordinate fMaximum;
	ViewCoordinate fInitialValue;
	ViewCoordinate fStep;
};


#endif // __USLIDER__
