//----------------------------------------------------------------------------------------
// UStaticText.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USTATICTEXT__
#define __USTATICTEXT__

// ACS

#if qGrayscaleAppearance

#	ifndef __StandardControls_AC__
#	include "StandardControls_AC.h"
#	endif

#endif // qGrayscaleAppearance

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif


//----------------------------------------------------------------------------------------
// TStaticText: Provides an object that represents a toolbox static text item.
//----------------------------------------------------------------------------------------

class EXPORT_MA TStaticText : public TControl
{
	MA_DECLARE_CLASS;
	
public:
	TStaticText(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								ResNumber itsRsrcID = kNoResource,
								short itsIndex = 0);
		// Constructor
		
	TStaticText(const TStaticText& inStaticText);
		// Copy Constructor

	virtual ~TStaticText();
		// Releases the text and then calls Inherited::Free.

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;


	virtual void DoSubstitution(CStr255_AC& theText);
		// Calls ReplaceText so that any paramTxt items can be handled if needed.

	virtual void Draw(const CViewRect& area);
		// Draws the text by calling ImageText.

	virtual CStr255_AC GetText() const;
		// If fDataHandle not NULL then returns the text, otherwise returns the NULL
        // CPascalStr_AC.

	virtual void ImageText(Ptr text,
								  long length,
								  const CViewRect& box,
								  short just);
		// Draws the text.

	virtual void ChangeWrap(bool newAutoWrap, bool redraw);
		// Changes auto-wrapping behavior, redrawing if requested

	virtual void ReleaseText();
	// Releases the text by disposing of fDataHandle.

	virtual void SetJustification(short theJust, bool redraw);
		// Sets the justification of the text to theJust, redrawing if required.

	virtual void SetText(const CStr255_AC& theText, bool redrawImmediately);
		// Sets the text to theText, redrawing if required.

	virtual void SetTextWithStrListID(ResNumber theStrListID, short theIndex, bool redraw);
		// Set fDataHandle to the string specified by theStrListID and theIndex and sets fStrListID
		// and fIndex accordingly.

#if qDrag
	virtual void DoAddDragContent();
		// Install flavors in the current drag
#endif // qDrag

#if qGrayscaleAppearance

	// MacApp overrides
	
	virtual void DoPostCreate(TDocument* itsDocument);
	
	virtual void SetFrame(const CViewRect& newFrame, bool invalidate); 
	
	virtual void Activate(bool entering);
	
	virtual void DimState(bool state, bool redraw);
	
	virtual void Dim();
	
	virtual void InstallTextStyle(const CTextStyle_AC& theTextStyle, bool redraw);
	
	// AGA overrides

	CStaticText_AC* GetAGAControl() const { return fAGAControl; }

	virtual bool IsEmulatingControls() const;
	
protected:

	virtual void CreateAGAControl();
	
	CAutoPtr_AC<CStaticText_AC>	fAGAControl;
		
#endif // qGrayscaleAppearance

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	CPascalStrHandle_AC fDataHandle;					// A handle to the actual CPascalStr_AC.

	ResNumber fStrListID;						// ID of the CPascalStr_AC list that the CPascalStr_AC
                                                // is in.

	short fIndex;								// Index number for the CPascalStr_AC from the
                                                // above list.

	short fJust;								// Jusitification of the text in the
                                                // field.

	bool fAutoWrap;							// True to word wrap lines at the view
                                                // boundary otherwise lines wrap at
                                                // Carriage return.

	bool fEraseFirst;						// control imaging of static text

};

EXPORT_MA  void InitUStaticText();

#endif