// USynchScroller.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __USYNCHSCROLLER__
#define __USYNCHSCROLLER__

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TSecondaryScroller;

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

enum
{
	kVDependent = 1,					// Dependent scroller is dependent on vertical translation 
	kNotVDependent = 0,					// Dependent scroller is NOT dependent on vertical translation 
	kHDependent = 1,					// Dependent scroller is dependent on horizontal translation 
	kNotHDependent = 0					// Dependent scroller is NOT dependent on horizontal translation 
};

//----------------------------------------------------------------------------------------
// TPrimaryScroller: A TPrimaryScroller has a list of secondary scrollers (if non-NULL):
// When it scrolls it makes the scrollers on the list scroll too in the very same
// ScrollRect toolbox call if possible. If some of the secondary scrollers can't be
// scrolled in the same ScrollRect call because they only scroll in one direction then
// they will be scrolled separately as necessary. 
//----------------------------------------------------------------------------------------

class EXPORT_MA TPrimaryScroller : public TScroller
{
	MA_DECLARE_CLONE;
	
public:
	inline TPrimaryScroller(const CViewPoint& itsLocation = gZeroVPt,
								  const CViewPoint& itsSize = gZeroVPt,
								  SizeDeterminer itsHSizeDet = sizeVariable,
								  SizeDeterminer itsVSizeDet = sizeVariable,
								  const CViewPoint& itsMax = gZeroVPt,
								  bool wantHorzSBar = false,
								  bool wantVertSBar = false) :
	TScroller(itsLocation, itsSize,
					itsHSizeDet, itsVSizeDet, itsMax,wantHorzSBar, wantVertSBar) { }
		// Initialization method for TPrimaryScroller.

	virtual ~TPrimaryScroller();
		// Frees its list 

	virtual void AddSecondaryScroller(TSecondaryScroller* itsSecondaryScroller,
											 ViewCoordinate itsHDependency,
											 ViewCoordinate itsVDependency);
		// Become associated with the secondary scroller(s). The V and H dependencies
        // could someday be scale factors but for now are kVDependent and kHDependent for
        // now. 

	virtual void RemoveSecondaryScroller(TSecondaryScroller* itsSecondaryScroller);
		// Remove the secondary scroller. 

	virtual void DoScroll(const CViewPoint& delta, bool redraw);
		// Assure that fTranslation is always updated in the secondary scrollers 

	virtual void ScrollDraw(const CViewPoint& delta, bool invalidate);
		// Scroll the secondary scrollers as well (unless called via SetScrollLimits) 

public:
	CList_AC fSecondaryScrollers;					// The list of secondary scrollers, if any
};


//----------------------------------------------------------------------------------------
// TSecondaryScroller: A TSecondaryScroller is controlled by a TPrimaryScroller (see
// above). When it is told to scroll, it delegates the job to the TPrimaryScroller.
// Constraints (none are checked by this unit at this time): 1. It and its subview must
// have the same length in the desired synch-scroll directions. 2. Commands for dragging
// within it must have it as their fScroller. 3. No scroll bars! 
//----------------------------------------------------------------------------------------

class EXPORT_MA TSecondaryScroller : public TScroller
{
	MA_DECLARE_CLONE;
	
public:
	TSecondaryScroller(const CViewPoint& itsLocation = gZeroVPt,
								  const CViewPoint& itsSize = gZeroVPt,
								  SizeDeterminer itsHSizeDet = sizeVariable,
								  SizeDeterminer itsVSizeDet = sizeVariable,
								  const CViewPoint& itsMax = gZeroVPt,
								  bool wantHorzSBar = false,
								  bool wantVertSBar = false);
		// Initialization method for TSecondaryScroller.

	virtual ~TSecondaryScroller();

	virtual void DoScroll(const CViewPoint& delta, bool redraw);
		// Polarize and delegate scroll attempts to the primary scroller (unless called
        // via SetScrollLimits) 

public:
	CPoint_AC fDeltaFactor;							// kVDependent and kHDependent (or not) 

	TPrimaryScroller* fPrimaryScroller;				// the primary scroller 
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

EXPORT_MA  void InitUSynchScroller();


#endif


