// UTabBehaviors.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UTABBEHAVIORS__
#define __UTABBEHAVIORS__

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TView;

//----------------------------------------------------------------------------------------
// TTabber
//----------------------------------------------------------------------------------------

// An abstract parent class for tabbing behaviors which intercept the Tab key event
// and find the next view which wishes to become the target

const IDType kTabBehavior = 'tabb';	// Behavior identifier

class EXPORT_MA TTabber : public TBehavior
{
	MA_DECLARE_CLASS;
	
public:
	//------------------------------------------------------------------------------------
	// Constructor
	//------------------------------------------------------------------------------------
	TTabber(bool recursive = true);
		// Constructor

	virtual ~TTabber();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------
	virtual void ReadFrom(CStream_AC* aStream);

	virtual void WriteTo(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// Event handling.
	//------------------------------------------------------------------------------------
	virtual void DoKeyEvent(TToolboxEvent* event);
	virtual void Tab(bool tabBackward);
	virtual void Reset();
	virtual void FindSubViewTargets(TView* parent,
										   bool tabBackward);
	virtual void FindTargets(bool tabBackward);

	virtual IDType GetStandardSignature() const;
		// Return the standard signature for this class

public:
	TView*	fNext;			// The view which becomes the new target.
	TView*	fFirst;			// Save the first potential target (to do wraparound)

	bool fRecursive;		// True if the search should include the entire view hierarchy
							// false if it should be limited to immediate subviews
	bool fFoundCurrent;		// Set to true when the current target has been found
};

//----------------------------------------------------------------------------------------
// TMultiWindowTabber
//----------------------------------------------------------------------------------------

// Attached to the application object. Handles tabbing across multiple active
// windows (i.e. when windoids are present)

class EXPORT_MA TMultiWindowTabber : public TTabber
{
	MA_DECLARE_CLASS;
	
public:
	//------------------------------------------------------------------------------------
	// Constructor
	//------------------------------------------------------------------------------------

	inline TMultiWindowTabber(bool recursive = true) : TTabber(recursive) { }
		// constructor

	virtual ~TMultiWindowTabber();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Event handling.
	//------------------------------------------------------------------------------------

	virtual void FindTargets(bool tabBackward);	// Override
};

//----------------------------------------------------------------------------------------
// TViewTabber
//----------------------------------------------------------------------------------------

// Can be attached to any view to intercept the Tab key before it gets to the
// MultiWindowTabber in the application 

class EXPORT_MA TViewTabber : public TTabber
{
	MA_DECLARE_CLASS;
	
public:
	//------------------------------------------------------------------------------------
	// Constructor
	//------------------------------------------------------------------------------------

	inline TViewTabber(bool recursive = true) : TTabber(recursive) { }
		// constructor

	virtual ~TViewTabber();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Event handling.
	//------------------------------------------------------------------------------------
	virtual void FindTargets(bool tabBackward);	// Override
};

#endif
