// UTabbedView.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __UTABBEDVIEW__
#define __UTABBEDVIEW__


// ACS

#ifndef __CTabbedView_AC__
#include "CTabbedView_AC.h"
#endif

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif

EXPORT_MA  void InitUTabbedView();


#pragma mark TTabbedView

//----------------------------------------------------------------------------------------
// TTabbedView
// TControl subclass for CTabbedView_AC set for large tabs. When instantiated
// from a view resource, the fUserArea, if non-zero, is used as the resource
// ID of an 'STR#' resource that contains the tab labels; the number of
// tabs will be the number of strings in the 'STR#' resource. If you have
// existing tabs, such as via the 'STR#', use InstallPanel to assign a
// panel to a tab. If you have no tabs yet, use AddPanel to create a new
// tab for a panel. Note that the panel views must be non-window 'View'
// resources, i.e. view hierarchies with a TView at the root, not a TWindow.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTabbedView : public TControl
{
MA_DECLARE_CLASS;

public:

	// TControl-compatible constructor
	TTabbedView(const CViewPoint& itsLocation = gZeroVPt,
							 const CViewPoint& itsSize = gZeroVPt,
							 SizeDeterminer itsHSizeDet = sizeVariable,
							 SizeDeterminer itsVSizeDet = sizeVariable,
						     const CTextStyle_AC& itsTextStyle = gSystemStyle,
						     ResNumber itsLabelsID = kNoResource);

	virtual ~TTabbedView();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);

	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// MacApp overrides.
	//------------------------------------------------------------------------------------

	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void Draw(const CViewRect& area);

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate); 

	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);

	virtual void Activate(bool entering);

	virtual bool IsActive() const;

	virtual void DimState(bool state, bool redraw);

	virtual void Dim();

	virtual void SetEnable(bool state);
	
	virtual void	HandleTabSwitch(SInt32 newTabIndex);
	// HandleTabSwitch can be used to switch to the specified panel,
	// but it will call ValidatePanel before allowing the switch.
	// This is what ClickSelf does to handle a click in a tab.
	// To switch directly without validation, just call SetValue.
	
	virtual void	AddPanel(SInt16 panelResourceID,
							StringPtr tabLabel);
	// AddPanel adds another tab, and assigns it the specified 'View' view.
	// If the specified label string is NULL, the 'View' rsrc name is used.

	virtual void	InstallPanel(SInt32 tabIndex,
							SInt16 panelResourceID,
							StringPtr tabLabel);
	// InstallPanel assigns the specified 'View' view to the specified
	// existing tab. If the specified label StringPtr is NULL, the existing
	// tab label is left untouched.

	virtual void	RemoveTab(SInt32 tabIndex, bool redraw);
	// RemoveTab removes the specified tab. It does *not* destroy the
	// panel view associated with the tab. If you want to destroy the
	// panel view, call GetPanelView first to access it.
	// You are responsible for switching tabs *beforehand* if you are
	// removing the currently selected tab.

	virtual TView*	GetPanelView(SInt32 tabIndex);
	// GetPanelView returns the TView pointer to the root of the
	// pane hierarchy of the specified tab.
	
	virtual void	SetSubviewsEnable(bool state);
	// SetSubviewsEnable is called by the SetEnable override
	// to walk all subviews of the currently installed panel
	// and set their enable state. Made public in case you
	// need to customize panel enabling behavior.

	CTabbedView_AC* GetAGAControl() const { return fAGAControl; }
	
protected:

	virtual void	CreateAGAControl();

	virtual void	CreateContainerView();

	virtual bool	ValidatePanel(SInt32 currentTabIndex);

	virtual void	SwitchPanels(SInt32 newTabIndex);

	virtual bool	SwitchPanelOut();	
		// Returns true if panel contained target.

	virtual void	SwitchPanelIn(SInt32 newTabIndex, bool targetInPanel);
	
	virtual void	DoSubviewsEnable(TView* aSuperView, bool state);
	
	CAutoPtr_AC<CTabbedView_AC>	fAGAControl;

	TView*	fPanelContainerView;
	TView*	fActivePanelView;
	ResNumber fLabelsID;
};

#pragma mark TSmallTabbedView

//----------------------------------------------------------------------------------------
// TSmallTabbedView
// TTabbedView (TControl) subclass for CTabbedView_AC set for small tabs.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSmallTabbedView : public TTabbedView
{
MA_DECLARE_CLASS;

public:

	// TControl-compatible constructor
	TSmallTabbedView(const CViewPoint& itsLocation = gZeroVPt,
							 const CViewPoint& itsSize = gZeroVPt,
							 SizeDeterminer itsHSizeDet = sizeVariable,
							 SizeDeterminer itsVSizeDet = sizeVariable,
						     const CTextStyle_AC& itsTextStyle = gSystemStyle);

	virtual ~TSmallTabbedView();

protected:

	virtual void CreateAGAControl();
};

#endif // __UTABBEDVIEW__
