// UTearOffMenuView.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UTEAROFFMENUVIEW__
#define __UTEAROFFMENUVIEW__

// ACS

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UTRACKER__
#include "UTracker.h"
#endif

#ifndef __UMENUVIEW__
#include "UMenuView.h"
#endif

// Toolbox

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TTearOffMenuView;
class TWindow;


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kTearOffMargin = 10;				// the slop around the displayed menu
                                                // within which we don't tear-off; see
                                                // Apple Human Interface Guidelines p. 91;
                                                // note: some programs (eg. HyperCard) use
                                                // a 15 pixel slop value 

const short kTearOffOffset = 3;					// pixel offset from pointer to gray rgn
                                                // being dragged around 


//----------------------------------------------------------------------------------------
// TShowTearOffWindowCommand: moves, shows and selects a TTearOffMenuView 
//----------------------------------------------------------------------------------------

class EXPORT_MA TShowTearOffWindowCommand : public TCommand
{
public:
	TShowTearOffWindowCommand(TWindow* tearOffWindow = NULL, CPoint_AC where = gZeroPt);
		// constructor

	virtual ~TShowTearOffWindowCommand();
		// Destructor
		
	virtual void DoIt();

public:
	CPoint_AC fWhere;
	TWindow* fTearOffWindow;
};


//----------------------------------------------------------------------------------------
// TTearOffTracker: abstract superclass which provides the ability to process a tracking
// command immediately rather than posting it to the queue.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTearOffMenuViewTracker : public TTracker
{
public:
	TTearOffMenuViewTracker(CommandNumber itsCommandNumber = cNoCommand,
								 TCommandHandler* itsContext = NULL,
								 bool canUndo = true,
								 bool causesChange = true,
								 MDependable_AC* objectToNotify = NULL,
							 	 TView* itsView = NULL,
								 TScroller* itsScroller = NULL,
								 const CViewPoint& itsMouse = gZeroVPt) :
	TTracker(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify,
					itsView, itsScroller, itsMouse) { }
		// constructor

	virtual ~TTearOffMenuViewTracker();
		// Destructor
		
	virtual bool IsReadyToPost();	// override
		// Process the command immediately rather than posting to the command queue.
};

//----------------------------------------------------------------------------------------
// TTearOffTracker: abstract superclass providing basic tear-off menu tracking: tracks the
// grey outline of the menu around the screen 
//----------------------------------------------------------------------------------------

class EXPORT_MA TTearOffTracker : public TTearOffMenuViewTracker
{
public:
	TTearOffTracker(CPoint_AC hitPt = gZeroPt, TTearOffMenuView* tearOffMenu = NULL);
		// Constructor
		
	virtual ~TTearOffTracker();

	virtual void DoIt();

	virtual void TrackFeedback(TrackPhase aTrackPhase,
										const CViewPoint& anchorPoint,
										const CViewPoint& previousPoint,
										const CViewPoint& nextPoint,
										bool mouseDidMove,
										bool turnItOn);

	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
										CViewPoint& anchorPoint,
										CViewPoint& previousPoint,
										CViewPoint& nextPoint,
										bool mouseDidMove);

	virtual bool IsDoneTracking();

public:
	CTempRegion_AC fTearOffWindowOutline;

	CPoint_AC fOrigin;
	TTearOffMenuView* fTearOffMenu;
	bool fExitTracking;
};


//----------------------------------------------------------------------------------------
// TTearOffMenuView: abstract superclass providing basic tear-off menu behavior 
//----------------------------------------------------------------------------------------

class EXPORT_MA TTearOffMenuView : public TMenuView
{
	MA_DECLARE_CLASS;
	
public:
	TTearOffMenuView(ResNumber rsrcID = kNoResource,
										 short menuWidth = 0,
										 short menuHeight = 0,
										 TWindow* tearOffWindow = NULL);
		// Constructor
		
	virtual ~TTearOffMenuView();

	virtual void InstallTearOffTracker();
		// For the sake of efficiency when tracking a tear-off, we'll create the tear-off
        // tracker and keep it with us, so we don't need to allocate a new tracker every
        // time we tear off a menu. If you want a different tracker in your subclass of
        // TTearOffMenuView, then override this method. 

	virtual void HandleChooseMessage(short message,
											MenuRef theMenu,
											CRect_AC& menuRect,
											CPoint_AC hitPt,
											short& whichItem);
		// initiates tracking if mouse is in fTearOffTrackingRegion 

	virtual RgnHandle GetTearOffTrackingRegion(const CRect_AC& menuRect);
		// accessor for fTearOffTrackingRegion: a rgn that represents the desktop less the
        // menu CRect_AC (with some buffer area around the menu CRect_AC). This is useful for
        // testing whether tracking should occur because the mouse just might be in the
        // menu bar (which isn't in the desktop & isn't in the menu CRect_AC) and so tracking
        // should not occur while the mouse is in the menu bar (MenuSelect does that
        // tracking just fine). 

public:
	CAutoPtr_AC<TTearOffTracker> fTearOffTracker;
	CTempRegion_AC fTearOffTrackingRegion;

	TWindow* fTearOffWindow;
};


#endif

