// UUpDownControl.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __UUPDOWNCONTROL__
#define __UUPDOWNCONTROL__

// ACS

#ifndef __CUpDownControl_AC__
#include "CUpDownControl_AC.h"
#endif

#ifndef __CIndicator_AC__
#include "CIndicator_AC.h"
#endif

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#ifndef __UNUMBERTEXT__
#include "UNumberText.h"
#endif

EXPORT_MA  void InitUUpDownControl();

#pragma mark TUpDownControl

//----------------------------------------------------------------------------------------
// TUpDownControl
// TControl subclass. You will need to either install a notification
// routine to respond to arrow tracking, or subclass and override
// the HandleNotification member function. However:
//
// If you simply want to link the little arrows to a TNumberText
// field, just put the TNumberText's view ID the little arrows view's
// fUserArea, and the arrows will automatically increment/decrement
// the TNumberText view's value.
//----------------------------------------------------------------------------------------

class EXPORT_MA TUpDownControl : public TControl
{
MA_DECLARE_CLASS;

public:

	// TControl-compatible constructor
	TUpDownControl(const CViewPoint& itsLocation = gZeroVPt,
							 const CViewPoint& itsSize = gZeroVPt,
							 SizeDeterminer itsHSizeDet = sizeVariable,
							 SizeDeterminer itsVSizeDet = sizeVariable,
						     const CTextStyle_AC& itsTextStyle = gSystemStyle,
						     IDType itsLinkedNumberTextID = kNoIdentifier);

	virtual ~TUpDownControl();

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);

	virtual void WriteFields(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// MacApp overrides.
	//------------------------------------------------------------------------------------

	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void Draw(const CViewRect& area);

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate); 

	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);

	virtual void Activate(bool entering);

	virtual void DimState(bool state, bool redraw);

	virtual void Dim();

	virtual void SetEnable(bool state);
		
	CUpDownControl_AC* GetAGAControl() const
		{ return fAGAControl; }

	// Notifier
	virtual void HandleNotification(SInt32 deltaValue);
		
protected:

	virtual void CreateAGAControl();

	static void RealAGANotifier(CIndicator_AC* theIndicator, SInt32 dataValue, TUpDownControl* notifierTarget);

	CAutoPtr_AC<CUpDownControl_AC>	fAGAControl;
	IDType			fLinkedNumberTextID;
	TNumberText*	fLinkedNumberText;
};



#endif // __UUPDOWNCONTROL__
