//----------------------------------------------------------------------------------------
// UWindowMenuBehavior.cp 
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UWINDOWMENUBEHAVIOR__
#define __UWINDOWMENUBEHAVIOR__

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

enum
{
	mInvalidateWindowMenu = 901				// need some better way of coordinating the events
};


class TWindow;

class EXPORT_MA TWindowSortedList : public CSortedList_AC
{
	public:
		TWindowSortedList(bool sortByDocument = false);

		virtual ~TWindowSortedList();

		virtual void* Clone() const;
	
		virtual CompareResult_AC Compare(const void* item1,
												const void* item2) const;
	protected:
		bool fSortByDocument;				// true to sort primarily by document and then by title
};


//
// provides a windows menu for the application; provide it with the resource ID of
// a menu you wish to use, and it will do the rest.  Override the ShouldBeIncluded
// menu to disallow specific kinds of windows from being in the menu.
//
class EXPORT_MA TWindowMenuBehavior : public TBehavior
{
	public:
	
		TWindowMenuBehavior(IDType itsIdentifier = kNoIdentifier,
												 ResNumber itsMenuNo = kNoResource,
												 EventNumber itsInvalidateEvent = mInvalidateWindowMenu,
												 bool sortByDocument = false,
												 bool documentSeparators = false,
												 bool includeHidden = false);
		// Initialize with the corresponding menu to synchronize with the behavior

		virtual ~TWindowMenuBehavior();
	
		virtual void* Clone() const;
	
		virtual void DoMenuCommand(CommandNumber aCommandNumber);
		// Selects the corresponding window
	
		virtual void DoSetupMenus();
		// Updates the window menu if necessary
	
		virtual void InvalidateWindowMenu();
		// Invalidate the windows menu
	
		virtual void BuildWindowList();
		// Construct the window list that will be used to load the menu
	
		virtual bool ShouldBeIncluded(TWindow* theWindow) const;
		// Filter function returns true if a given window should be in the windows menu
	
		virtual void UpdateWindowMenu();
		// Update the windows menu (rebuilding it if necessary)
	
		virtual bool DoToolboxEvent(TToolboxEvent* event);
		// Filter that invalidates the windows menu when activate events are detected
	
		virtual void DoEvent(EventNumber eventNumber,
									 TEventHandler* source,
									 TEvent* event);

		virtual void GetMinMaxSize(CLongPoint_AC& minSize,
										   CLongPoint_AC& maxSize,
										   CSortedList_AC* windowList);
		// return the smallest window max size, and the largest window min size; useful for
		// doing things like tiling and stacking

		inline bool IsWindowMenuInvalid() { return fWindowMenuInvalid; }
		// true if window menu needs updating
	
		inline void ValidateWindowMenu() { fWindowMenuInvalid = false; }
		// Validate the windows menu (But don't update the menu)
	
		inline CSortedList_AC* GetWindowList() { return fWindowList; }
		// Return the list that holds the windows to be in the windows menu
	
		CAutoPtr_AC<CSortedList_AC> fWindowList;	// list of windows in the menu
		ResNumber fMenuNo;							// the ID of the windows menu
		EventNumber fInvalidateEvent;				// event number that causes window menu to invalidate
		short fReserveMenuItems;					// number of items in the windows menu to leave alone
		bool fIncludeHidden;						// true to include hidden windows in the menu
		bool fDocumentSeparators;					// true to place separators between documents in the menu
		bool fWindowMenuInvalid;					// true if the windows menu is invalid and needs rebuilding
};

#endif
