// UBackgroundAdorner.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __UBACKGROUNDADORNER__
#include "UBackgroundAdorner.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

// MacApp

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

//----------------------------------------------------------------------------------------
// InitUBackgroundAdorner:
// Register all of the classes defined here. If you prefer, you can
// selectively register just the classes you will use.
//----------------------------------------------------------------------------------------


void InitUBackgroundAdorner()
{
	MA_REGISTER_CLASS(TBackgroundAdorner);
	MA_REGISTER_CLASS(TModalBackgroundAdorner);
	MA_REGISTER_CLASS(TFlatBackgroundBehavior);
	MA_REGISTER_CLASS(TWhiteBackgroundAdorner);
	MA_REGISTER_CLASS(TNotchedWhiteBackgroundAdorner);
	MA_REGISTER_CLASS(TBorderFrameAdorner);
	MA_REGISTER_CLASS(TNotchedBorderFrameAdorner);
}

//========================================================================================
// CLASS TBackgroundAdorner
//========================================================================================

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS(TBackgroundAdorner);

//----------------------------------------------------------------------------------------
// TBackgroundAdorner::TBackgroundAdorner:
//----------------------------------------------------------------------------------------


TBackgroundAdorner::TBackgroundAdorner() :
	fBackgroundKind(kRaisedModelessBackground)
{
}

//----------------------------------------------------------------------------------------
// TBackgroundAdorner::DoHighlightSelection: Draw in the specified active/inactive state.
//----------------------------------------------------------------------------------------


void TBackgroundAdorner::DoHighlightSelection(TView* itsView, const CViewRect& area, HLState /*fromHL*/, HLState toHL)
{
	DrawBackground(itsView, area, false, (toHL == hlOn));
}

//----------------------------------------------------------------------------------------
// TBackgroundAdorner::Draw: Draw in active state.
//----------------------------------------------------------------------------------------


void TBackgroundAdorner::Draw(TView* itsView, const CViewRect& area)
{
	DrawBackground(itsView, area, true, itsView->IsActive());
}

//----------------------------------------------------------------------------------------
// TBackgroundAdorner::ViewChangedFrame:
//----------------------------------------------------------------------------------------


void TBackgroundAdorner::ViewChangedFrame(TView* itsView, const CViewRect& oldFrame, const CViewRect& newFrame, bool	/*invalidate*/)
{
	// Make sure we invalidate the edges that will need to be
	// redrawn with the background edge pixel color.
	// Note that oldFrame & newFrame are in global coordinates.

	CViewRect	rectToInvalidate;
	CViewPoint	sizeDelta(newFrame.GetLength(hSel) - oldFrame.GetLength(hSel),
					newFrame.GetLength(vSel) - oldFrame.GetLength(vSel));
	
	if (sizeDelta.h != 0)
	{
		if (sizeDelta.h > 0)
			rectToInvalidate = oldFrame;
		else	// sizeDelta.h < 0
			rectToInvalidate = newFrame;

		rectToInvalidate.left = rectToInvalidate.right - 1;
		itsView->Invalidate(itsView->SuperViewToLocal(rectToInvalidate));
	}
	
	if (sizeDelta.v != 0)
	{
		if (sizeDelta.v > 0)
			rectToInvalidate = oldFrame;
		else	// sizeDelta.v < 0
			rectToInvalidate = newFrame;

		rectToInvalidate.top = rectToInvalidate.bottom - 1;
		itsView->Invalidate(itsView->SuperViewToLocal(rectToInvalidate));
	}
	
	if (HasGrowBox(itsView))
	{
		rectToInvalidate = oldFrame;
		rectToInvalidate.left = rectToInvalidate.right - 16;
		rectToInvalidate.top = rectToInvalidate.bottom - 16;
		itsView->Invalidate(itsView->SuperViewToLocal(rectToInvalidate));

		rectToInvalidate = newFrame;
		rectToInvalidate.left = rectToInvalidate.right - 16;
		rectToInvalidate.top = rectToInvalidate.bottom - 16;
		itsView->Invalidate(itsView->SuperViewToLocal(rectToInvalidate));
	}
}

//----------------------------------------------------------------------------------------
// TBackgroundAdorner::AddedToView:
//----------------------------------------------------------------------------------------


void TBackgroundAdorner::AddedToView(TView* itsView)
{
	// Install the background color if we're in a window.

	TWindow*	itsWindow = itsView->GetWindow();

	if (itsWindow != NULL)
		::ThrowIfOSErr_AC(AGAInstallGrayWCTB(itsWindow->GetWindowRef()));

	// Set the current bg color of the view's port so that any of our
	// AGA adapter views that get instantiated will pick it up.

	if (itsView->Focus())
		kRGBColorRamp[r2].SetBackColor();
}

//----------------------------------------------------------------------------------------
// TBackgroundAdorner::DrawBackground:
//----------------------------------------------------------------------------------------


void TBackgroundAdorner::DrawBackground(TView* itsView, const CViewRect& /*area*/, bool fill, bool active)
{	
	AGABackgroundPaint(itsView->GetQDExtent(), fill, fBackgroundKind, active, HasGrowBox(itsView));
}

//----------------------------------------------------------------------------------------
// TBackgroundAdorner::HasGrowBox:
//----------------------------------------------------------------------------------------


bool TBackgroundAdorner::HasGrowBox(TView* itsView)
{
	return dynamic_cast<TWindow*>(itsView) ? dynamic_cast<TWindow*>(itsView)->GetIsResizable() : false;
}

//========================================================================================
// CLASS TModalBackgroundAdorner
//========================================================================================

#undef Inherited
#define Inherited TBackgroundAdorner

MA_DEFINE_CLASS(TModalBackgroundAdorner);

//----------------------------------------------------------------------------------------
// TModalBackgroundAdorner::TModalBackgroundAdorner:
//----------------------------------------------------------------------------------------


TModalBackgroundAdorner::TModalBackgroundAdorner()
{
	fBackgroundKind = kFlatWindowBackground;
}

//========================================================================================
// CLASS TFlatBackgroundBehavior
//========================================================================================

#undef Inherited
#define Inherited TBehavior

MA_DEFINE_CLASS(TFlatBackgroundBehavior);

//----------------------------------------------------------------------------------------
// TFlatBackgroundBehavior::TFlatBackgroundBehavior:
//----------------------------------------------------------------------------------------


TFlatBackgroundBehavior::TFlatBackgroundBehavior()
{
}

//----------------------------------------------------------------------------------------
// TFlatBackgroundBehavior::~TFlatBackgroundBehavior:
//----------------------------------------------------------------------------------------


TFlatBackgroundBehavior::~TFlatBackgroundBehavior()
{
}

//----------------------------------------------------------------------------------------
// TFlatBackgroundBehavior::DoPostCreate:
//----------------------------------------------------------------------------------------


void TFlatBackgroundBehavior::DoPostCreate(TDocument* itsDocument)
{
	// Install the background color if we're in a window.

	Inherited::DoPostCreate(itsDocument);
	
	TWindow*	theWindow = (dynamic_cast_AC(TView*,fOwner))->GetWindow();
	
	if (theWindow != NULL)
	{
		::ThrowIfOSErr_AC(AGAInstallGrayWCTB(theWindow->GetWindowRef()));

		// Set the current bg color of the view's port so that any of our
		// AGA adapter views that get instantiated will pick it up.

		if (theWindow->Focus())
			kRGBColorRamp[r2].SetBackColor();
	}
}

//========================================================================================
// CLASS TWhiteBackgroundAdorner
//========================================================================================

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS(TWhiteBackgroundAdorner);

//----------------------------------------------------------------------------------------
// TWhiteBackgroundAdorner::Draw:
//----------------------------------------------------------------------------------------


void TWhiteBackgroundAdorner::Draw(TView* itsView, const CViewRect& /*area*/)
{
	// Draw the background in white, inset to avoid the border
	// frame pixels.

	CRect_AC	qdArea(itsView->GetQDExtent());
	
	kRGBColorRamp[rW].SetForeColor();
	kRGBColorRamp[rW].SetBackColor();
	::InsetRect(qdArea, 2, 2);
	::PaintRect(qdArea);

	kRGBColorRamp[rB].SetForeColor();
}

//========================================================================================
// CLASS TNotchedWhiteBackgroundAdorner
//========================================================================================

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS(TNotchedWhiteBackgroundAdorner);

//----------------------------------------------------------------------------------------
// TNotchedWhiteBackgroundAdorner::Draw
//----------------------------------------------------------------------------------------


void TNotchedWhiteBackgroundAdorner::Draw(TView* itsView, const CViewRect& /*area*/)
{
	// Draw the background in white, inset to avoid the border
	// frame pixels.

	CRect_AC	qdArea(itsView->GetQDExtent());
	
	kRGBColorRamp[rW].SetForeColor();
	::InsetRect(qdArea, 2, 2);

	qdArea.right -= 15;
	::PaintRect(qdArea);
	qdArea.right += 15;
	
	qdArea.bottom -= 15;
	::PaintRect(qdArea);

	kRGBColorRamp[rB].SetForeColor();
}

//========================================================================================
// CLASS TBorderFrameAdorner
//========================================================================================

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS(TBorderFrameAdorner);

//----------------------------------------------------------------------------------------
// TBorderFrameAdorner::DoHighlightSelection:
//----------------------------------------------------------------------------------------


void TBorderFrameAdorner::DoHighlightSelection(TView* itsView, const CViewRect& /*area*/, HLState /*fromHL*/, HLState toHL)
{
	// Draw in the specified active/inactive state.
	
	itsView->Focus();
	
	AGADrawBorderFrame(itsView->GetQDExtent(), (toHL == hlOn) && itsView->IsEnabled(), false /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);
	
	kRGBColorRamp[rB].SetForeColor();
	kRGBColorRamp[rW].SetBackColor();
}

//----------------------------------------------------------------------------------------
// TBorderFrameAdorner::Draw:
//----------------------------------------------------------------------------------------


void TBorderFrameAdorner::Draw(TView* itsView, const CViewRect& /*area*/)
{
	// Draw the frame with shading outside.
	
	AGADrawBorderFrame(itsView->GetQDExtent(), itsView->IsEnabled() && itsView->IsActive(), false /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);
	
	kRGBColorRamp[rB].SetForeColor();
	kRGBColorRamp[rW].SetBackColor();
}

//========================================================================================
// CLASS TNotchedBorderFrameAdorner
//========================================================================================

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS(TNotchedBorderFrameAdorner);

//----------------------------------------------------------------------------------------
// TNotchedBorderFrameAdorner::DoHighlightSelection:
//----------------------------------------------------------------------------------------


void TNotchedBorderFrameAdorner::DoHighlightSelection(TView* itsView, const CViewRect& /*area*/, HLState /*fromHL*/, HLState toHL)
{
	// Draw in the specified active/inactive state.

	itsView->Focus();
	
	AGADrawBorderFrame(itsView->GetQDExtent(), (toHL == hlOn) && itsView->IsEnabled(), true /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);
	
	kRGBColorRamp[rB].SetForeColor();
	kRGBColorRamp[rW].SetBackColor();
}

//----------------------------------------------------------------------------------------
// TNotchedBorderFrameAdorner::Draw:
//----------------------------------------------------------------------------------------


void TNotchedBorderFrameAdorner::Draw(TView* itsView, const CViewRect& /*area*/)
{
	// Draw the frame with shading outside.
	
	AGADrawBorderFrame(itsView->GetQDExtent(), itsView->IsEnabled() && itsView->IsActive(), true /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);
	
	kRGBColorRamp[rB].SetForeColor();
	kRGBColorRamp[rW].SetBackColor();
}
