// UBeveledTargetBorderView.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __UBEVELEDTARGETBORDERVIEW__
#include "UBeveledTargetBorderView.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

// MacApp

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

//----------------------------------------------------------------------------------------
// InitUBeveledTargetBorderView
//----------------------------------------------------------------------------------------



void InitUBeveledTargetBorderView()
{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	MA_REGISTER_CLASS(TBeveledTargetBorderView);
	MA_REGISTER_CLASS(TNotchedTargetBorderView);
	MA_REGISTER_CLASS(TTargetBorderFrameView);
	MA_REGISTER_CLASS(TNotchedTargetBorderFrameView);
}

//========================================================================================
// CLASS TBeveledTargetBorderView
//========================================================================================

#undef Inherited
#define Inherited TTargetBorderView

MA_DEFINE_CLASS(TBeveledTargetBorderView);

//----------------------------------------------------------------------------------------
//TBeveledTargetBorderView::TBeveledTargetBorderView
//----------------------------------------------------------------------------------------


TBeveledTargetBorderView::TBeveledTargetBorderView(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView) :
	TTargetBorderView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView)
{
}

//----------------------------------------------------------------------------------------
//TBeveledTargetBorderView::DoPostCreate
//----------------------------------------------------------------------------------------


void TBeveledTargetBorderView::DoPostCreate(TDocument* itsDocument)
{
	// Save the background colors we will need to erase.

	Inherited::DoPostCreate(itsDocument);
	
	Focus();
	if (! CControl_AC::DetermineBackgroundColors(fEnabledBackgroundColor, fDisabledBackgroundColor, CControl_AC::kInstallOnlyIfCustom))
	{
		fEnabledBackgroundColor = kRGBColorRamp[r2];
		fDisabledBackgroundColor = kRGBColorRamp[r2];
	}
}

//----------------------------------------------------------------------------------------
// TBeveledTargetBorderView::Draw
//----------------------------------------------------------------------------------------


void TBeveledTargetBorderView::Draw(const CViewRect& /*area*/)
{
	// Set the color, always draw. TTargetBorderView leaves
	// background unerased in 1-bit.

	CTempDrawingEnvironment_AC	env;

	CGDIterator_AC	iter;
	bool			deep;
	bool			isTarget = ContainsTarget(GetTarget());
	bool			enabled = IsEnabled() && IsActive();

	CTempRegion_AC borderRegion;
	
	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			if (isTarget && enabled)
				kRGBColorRamp[r8].SetForeColor();
			else if (enabled)
				fEnabledBackgroundColor.SetForeColor();
			else
				fDisabledBackgroundColor.SetForeColor();
		}
		else
		{
			if (isTarget && enabled)
				kRGBColorRamp[rB].SetForeColor();
			else
				kRGBColorRamp[rW].SetForeColor();
		}

		ComputeBorderRegion(borderRegion);
		PaintRgn(borderRegion);
	}
}

//----------------------------------------------------------------------------------------
// TBeveledTargetBorderView::ComputeBorderRegion:
//----------------------------------------------------------------------------------------


void TBeveledTargetBorderView::ComputeBorderRegion(RgnHandle borderRegion)
{
	// Return the border region to be painted or invalidated.

	AGAComputeTargetBorderRegion(borderRegion, GetQDExtent(), false /*isNotched*/);
}

//========================================================================================
// CLASS TNotchedTargetBorderView
//========================================================================================

#undef Inherited
#define Inherited TBeveledTargetBorderView

MA_DEFINE_CLASS(TNotchedTargetBorderView);

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderView::TNotchedTargetBorderView:
//----------------------------------------------------------------------------------------


TNotchedTargetBorderView::TNotchedTargetBorderView(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView) :
	TBeveledTargetBorderView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView)
{
}

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderView::ComputeBorderRegion:
//----------------------------------------------------------------------------------------


void TNotchedTargetBorderView::ComputeBorderRegion(RgnHandle borderRegion)
{
	// Return the border region to be painted or invalidated.

	AGAComputeTargetBorderRegion(borderRegion, GetQDExtent(), true /*isNotched*/);
}

//========================================================================================
// CLASS TTargetBorderFrameView
//========================================================================================

#undef Inherited
#define Inherited TBeveledTargetBorderView

MA_DEFINE_CLASS(TTargetBorderFrameView);

//----------------------------------------------------------------------------------------
// TTargetBorderFrameView::TTargetBorderFrameView:
//----------------------------------------------------------------------------------------


TTargetBorderFrameView::TTargetBorderFrameView(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView) :
	TBeveledTargetBorderView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView)
{
}

//----------------------------------------------------------------------------------------
// TTargetBorderFrameView::Draw:
//----------------------------------------------------------------------------------------


void TTargetBorderFrameView::Draw(const CViewRect& area)
{
	// Let the TBeveledTargetBorderView draw the target border,
	// then draw the "3D" sunken frame portion..

	Inherited::Draw(area);
	
	CRect_AC	qdArea(GetQDExtent());
	
	::InsetRect(qdArea, 3, 3);

	AGADrawBorderFrame(qdArea, IsEnabled() && IsActive(), false /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);

	kRGBColorRamp[rB].SetForeColor();
	kRGBColorRamp[rW].SetBackColor();
}

//----------------------------------------------------------------------------------------
// TTargetBorderFrameView::Dim
//----------------------------------------------------------------------------------------


void TTargetBorderFrameView::Dim()
{
	// Suppress patBic painting over our frame.
}

//----------------------------------------------------------------------------------------
// TTargetBorderFrameView::SetEnable
//----------------------------------------------------------------------------------------


void TTargetBorderFrameView::SetEnable(bool state)
{
	// Redraw now; unlike TControl, TView does not automatically
	// redraw when you disable/dim it.
	
	Inherited::SetEnable(state);

	if (Focus() && IsVisible())
		Draw(GetExtent());
}

//----------------------------------------------------------------------------------------
// TTargetBorderFrameView::Activate:
//----------------------------------------------------------------------------------------


void TTargetBorderFrameView::Activate(bool entering)
{
	Inherited::Activate(entering);
	
	if (Focus() && IsVisible())
	{
		CRect_AC	qdArea(GetQDExtent());
		
		::InsetRect(qdArea, 3, 3);

		AGADrawBorderFrame(qdArea, entering && IsEnabled(), false /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);
	}
}

//========================================================================================
// CLASS TNotchedTargetBorderFrameView
//========================================================================================

#undef Inherited
#define Inherited TBeveledTargetBorderView

MA_DEFINE_CLASS(TNotchedTargetBorderFrameView);

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderFrameView::TNotchedTargetBorderFrameView:
//----------------------------------------------------------------------------------------


TNotchedTargetBorderFrameView::TNotchedTargetBorderFrameView(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView) :
	TBeveledTargetBorderView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView)
{
}

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderFrameView::Draw:
//----------------------------------------------------------------------------------------


void TNotchedTargetBorderFrameView::Draw(const CViewRect& area)
{
	// Let the TBeveledTargetBorderView draw the target border,
	// then draw the "3D" sunken frame portion..

	Inherited::Draw(area);
	
	CRect_AC	qdArea(GetQDExtent());
	
	::InsetRect(qdArea, 3, 3);

	AGADrawBorderFrame(qdArea, IsEnabled() && IsActive(), true /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);

	kRGBColorRamp[rB].SetForeColor();
	kRGBColorRamp[rW].SetBackColor();
}

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderFrameView::Dim:
//----------------------------------------------------------------------------------------


void TNotchedTargetBorderFrameView::Dim()
{
	// Suppress patBic painting over our frame.
}

//----------------------------------------------------------------------------------------
// TNotchedTargetBorderFrameView::Activate:
//----------------------------------------------------------------------------------------


void TNotchedTargetBorderFrameView::Activate(bool entering)
{
	Inherited::Activate(entering);
	
	if (Focus())
	{
		CRect_AC	qdArea(GetQDExtent());
		
		::InsetRect(qdArea, 3, 3);

		AGADrawBorderFrame(qdArea, entering && IsEnabled(), true /*isNotched*/, true /*drawFrame*/, kRGBColorRamp[r2]);
	}
}


//----------------------------------------------------------------------------------------
// TNotchedTargetBorderFrameView::ComputeBorderRegion
//----------------------------------------------------------------------------------------


void TNotchedTargetBorderFrameView::ComputeBorderRegion(RgnHandle borderRegion)
{
	// Return the border region to be painted or invalidated.

	AGAComputeTargetBorderRegion(borderRegion, GetQDExtent(), true /*isNotched*/);
}

