//----------------------------------------------------------------------------------------
// UButton.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UBUTTON__
#include "UButton.h"
#endif

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#if qWantsAppearance
#	ifndef __APPEARANCEHELPERS__
#	include <AppearanceHelpers.h>
#	endif
#endif

void InitUButton()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TButton, kStdButton);
	}
}  

//========================================================================================
// CLASS TButton
//========================================================================================
#undef Inherited
#define Inherited TCtlMgr

MA_DEFINE_CLASS(TButton);

//----------------------------------------------------------------------------------------
// TButton constructor
//----------------------------------------------------------------------------------------

TButton::TButton(const CViewPoint& itsLocation,
					  const CViewPoint& itsSize,
					  SizeDeterminer itsHSizeDet,
					  SizeDeterminer itsVSizeDet,
					  const CStr255_AC& itsLabel) :
	TCtlMgr(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, 0, 0, 0, ((pushButProc) | kControlUsesOwningWindowsFontVariant)),

	fStrListID(kNoResource),					// buttons's label:	STR# rsrc id
	fIndex(kNoResource)							//					index into STR#
#if qGrayscaleAppearance
	, fAGAControl(NULL)
#endif
{
	fEventNumber = mButtonHit;
}

//----------------------------------------------------------------------------------------
// TButton destructor
//----------------------------------------------------------------------------------------

TButton::~TButton()
{
}

//----------------------------------------------------------------------------------------
// TButton::GetStandardSignature: 
//----------------------------------------------------------------------------------------

IDType TButton::GetStandardSignature() const			// Override 
{
	return kStdButton;
}

//----------------------------------------------------------------------------------------
// TButton::ReadFields: 
//----------------------------------------------------------------------------------------

void TButton::ReadFields(CStream_AC* aStream)		// Override 
{
	Inherited::ReadFields(aStream);

	CStr255_AC itsLabel;
	
	*aStream >> fStrListID
			 >> fIndex;
			 
	if (fStrListID != kNoResource)			// retrieve the buttons's label from the resource
		GetIndString(itsLabel, fStrListID, fIndex);
	CreateCMgrControl(itsLabel, 0, 0, 0, ((pushButProc) | kControlUsesOwningWindowsFontVariant));
	SetDefaultOutline();
}

//----------------------------------------------------------------------------------------
// TButton::WriteFields: 
//----------------------------------------------------------------------------------------

void TButton::WriteFields(CStream_AC* aStream) const		// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fStrListID
			 << fIndex;
}

//----------------------------------------------------------------------------------------
// TButton::DoEvent: 
//----------------------------------------------------------------------------------------

void TButton::DoEvent(EventNumber eventNumber,
					  TEventHandler* source,
					  TEvent* event)			// Override 

{
	if ((eventNumber == GetEventNumber()) && (source != this) && IsEnabled())
	{
		if (!IsDimmed())						// if dimmed, do absolutely nothing.
		{
			Flash();
			Inherited::DoEvent(eventNumber, this, event);
		}
	}
	else
		Inherited::DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TButton::SetDefaultOutline: 
//----------------------------------------------------------------------------------------

void TButton::SetDefaultOutline()
{
	// We implement the default button outline by checking for the
	// presence of the TRRectAdorner by std ID, tossing it, and
	// telling the control that it is a default button.
#if qGrayscaleAppearance
	TAdorner*	rrectAdorner = FindRRectAdorner();
	if (rrectAdorner != NULL)
	{
		if (IsEmulatingControls() && fAGAControl)
		{
			CRect_AC bounds(fAGAControl->GetBounds());
			this->DeleteAdorner(rrectAdorner, kDontInvalidate);
			fAGAControl->SetDefault(CControl_AC::kIsDefault, CControl_AC::kFrameInside);
			::InsetRect(bounds, -3, -3);
			fAGAControl->SetBounds(bounds, CControl_AC::kDontRedraw);
		}
		// We are the default button
#if qWantsAppearance
		else if (::HasAppearanceMgr_AC())
		{
			this->DeleteAdorner(rrectAdorner, kDontInvalidate);
			::SetPushButtonDefaultState(fCMgrControl, true);
		}
#endif // qWantsAppearance
	}
#endif // qGrayscaleAppearance
}

//----------------------------------------------------------------------------------------
// TButton::FindRRectAdorner: 
//----------------------------------------------------------------------------------------

TAdorner* TButton::FindRRectAdorner()
{
	// Return the TRRectAdorner, if any, that is attached to
	// this push button.

	TAdorner*	foundAdorner = NULL;

	if (!fAdorners.IsNULL())
	{
		CAdornerIterator	iter(this);

		for (TAdorner* theAdorner = iter.First(); iter.Current(); theAdorner = iter.Next())
		{
			if (dynamic_cast<TRRectAdorner*>(theAdorner))
				foundAdorner = theAdorner;
			else if (dynamic_cast<TCompatibilityAdorner*>(theAdorner))
			{
				TCompatibilityAdorner* theCompatibilityAdorner = dynamic_cast_AC(TCompatibilityAdorner*,theAdorner);
				if (macroIn(theCompatibilityAdorner->fAdornment, adnRRect))
					foundAdorner = theAdorner;
			}
		}
	}

	return foundAdorner;
}

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TButton::HiliteState: 
//----------------------------------------------------------------------------------------

void TButton::HiliteState(bool state, bool redraw)
{
	if (IsEmulatingControls())
	{
		if (fAGAControl && Focus())
			fAGAControl->DrawButton(state);
	}
	else
		Inherited::HiliteState(state, redraw);
}
	
//----------------------------------------------------------------------------------------
// TButton::Hilite: 
//----------------------------------------------------------------------------------------

void TButton::Hilite()
{
	if (IsEmulatingControls())
	{
		if (fAGAControl && Focus())
			fAGAControl->DrawButton(CControl_AC::kPressed);
	}
	else
		Inherited::Hilite();
}

//----------------------------------------------------------------------------------------
// TButton::SetText: 
//----------------------------------------------------------------------------------------

void TButton::SetText(const CStr255_AC& itsText, bool redraw)
{
	// Change the current text in TCtlMgr and in
	// the AGA object. Do NOT let control mgr draw.

	Inherited::SetText(itsText, kDontRedraw);

	if (fAGAControl)
	{
		fAGAControl->SetTitle(itsText, IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
}

//----------------------------------------------------------------------------------------
// TButton::CreateAGAControl: 
//----------------------------------------------------------------------------------------

void TButton::CreateAGAControl()
{
	fAGAControl = TH_new CButton_AC(ControlQDArea(), fTextStyle, GetText());
}

//----------------------------------------------------------------------------------------
// TButton::SetFrame:
//----------------------------------------------------------------------------------------

void TButton::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	
	if (fAGAControl && fAGAControl->IsDefault())
	{
		// As noted above in TButton::DoMakeControlAGA, we need to
		// outset the AGA object frame by 3x3 if it has a default outline.
		
		CRect_AC	bounds(fAGAControl->GetBounds());
		
		::InsetRect(bounds, -3, -3);
		fAGAControl->SetBounds(bounds, CControl_AC::kDontRedraw);
	}
}

#endif // qGrayscaleAppearance
