//----------------------------------------------------------------------------------------
// UCheckbox.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UCHECKBOX__
#include "UCheckbox.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

void InitUCheckbox()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TCheckBox, kStdCheckBox);
	}
}

//========================================================================================
// CLASS TCheckBox
//========================================================================================
#undef Inherited
#define Inherited TCtlMgr

MA_DEFINE_CLASS(TCheckBox);

//----------------------------------------------------------------------------------------
// TCheckBox constructor
//----------------------------------------------------------------------------------------

TCheckBox::TCheckBox(const CViewPoint& itsLocation,
								const CViewPoint& itsSize,
								SizeDeterminer itsHSizeDet,
								SizeDeterminer itsVSizeDet,
								const CStr255_AC& itsTitle,
								bool isTurnedOn) :
	TCtlMgr(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTitle, 0, 0, 1, ((checkBoxProc) | kControlUsesOwningWindowsFontVariant)),

	fStrListID(kNoResource),					// buttons's label:	STR# rsrc id
	fIndex(kNoResource)							//					index into STR#
#if qGrayscaleAppearance
	, fAGAControl(NULL)
#endif
{
	fEventNumber = mCheckBoxHit;

	SetState(isTurnedOn, kDontRedraw); 
}

//----------------------------------------------------------------------------------------
// TCheckBox destructor
//----------------------------------------------------------------------------------------

TCheckBox::~TCheckBox()
{
}

//----------------------------------------------------------------------------------------
// TCheckBox::GetStandardSignature: 
//----------------------------------------------------------------------------------------

IDType TCheckBox::GetStandardSignature() const		// Override 
{
	return kStdCheckBox;
}

//----------------------------------------------------------------------------------------
// TCheckBox::ReadFields: 
//----------------------------------------------------------------------------------------

void TCheckBox::ReadFields(CStream_AC* aStream)	// Override 
{
	Inherited::ReadFields(aStream);

	bool isOn = aStream->ReadBoolean();

	CStr255_AC itsLabel;
	fStrListID = aStream->ReadInteger();
	fIndex = aStream->ReadInteger();
	if (fStrListID != kNoResource)			// retrieve the buttons's label from the resource
		GetIndString(itsLabel, fStrListID, fIndex);

	CreateCMgrControl(itsLabel, isOn, 0, kControlCheckboxMixedValue, checkBoxProc | kControlUsesOwningWindowsFontVariant);
}

//----------------------------------------------------------------------------------------
// TCheckBox::WriteFields: 
//----------------------------------------------------------------------------------------

void TCheckBox::WriteFields(CStream_AC* aStream) const	// Override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteBoolean(IsOn());

	aStream->WriteInteger(fStrListID);
	aStream->WriteInteger(fIndex);
}

//----------------------------------------------------------------------------------------
// TCheckBox::DoEvent: 
//----------------------------------------------------------------------------------------

void TCheckBox::DoEvent(EventNumber eventNumber,
						TEventHandler* source,
						TEvent* event)			// Override 

{
	if (eventNumber == mCheckBoxHit)
		Toggle(kRedraw);
	Inherited::DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TCheckBox::IsOn: 
//----------------------------------------------------------------------------------------

bool TCheckBox::IsOn() const
{
#if qGrayscaleAppearance
	if (IsEmulatingControls() && fAGAControl)
		return fAGAControl->GetValue() == CCheckBox_AC::kCheckBoxOn;
	else
#endif // qGrayscaleAppearance
		return GetLongVal() == 1;	// was != 0
}

//----------------------------------------------------------------------------------------
// TCheckBox::SetState: 
//----------------------------------------------------------------------------------------

void TCheckBox::SetState(SInt16 state,
						 bool redraw)
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		if (fAGAControl)
			fAGAControl->SetValue(state, redraw && Focus() && IsVisible());
	}
	else
#endif // qGrayscaleAppearance
		SetLongVal((long)state, redraw);
}

//----------------------------------------------------------------------------------------
// TCheckBox::Toggle: 
//----------------------------------------------------------------------------------------

void TCheckBox::Toggle(bool redraw)

{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		// Let SetState do our work for us
		SetState(!IsOn(), redraw && Focus() && IsVisible());
	}
	else
#endif // qGrayscaleAppearance
		SetLongVal((long)(!IsOn()), redraw);
}

//----------------------------------------------------------------------------------------
// TCheckBox::ToggleIf: 
//----------------------------------------------------------------------------------------

void TCheckBox::ToggleIf(bool matchState,
						 bool redraw)
{
	bool isOn = IsOn();
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		// Let SetState do our work for us
		if (isOn == matchState)
			SetState(!isOn, redraw && Focus() && IsVisible());
	}
	else
#endif // qGrayscaleAppearance
		if (isOn == matchState)
			SetLongVal((long)(!isOn), redraw);
}

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TCheckBox::HiliteState: 
//----------------------------------------------------------------------------------------

void TCheckBox::HiliteState(bool state, bool redraw)
{
	if (IsEmulatingControls())
	{
		if (fAGAControl && Focus())
			fAGAControl->DrawButton(state);
	}
	else
		Inherited::HiliteState(state, redraw);
}

//----------------------------------------------------------------------------------------
// TCheckBox::Hilite: 
//----------------------------------------------------------------------------------------

void TCheckBox::Hilite()
{
	if (IsEmulatingControls())
	{
		if (fAGAControl && Focus())
			fAGAControl->DrawButton(CControl_AC::kPressed);
	}
	else
		Inherited::Hilite();
}

//----------------------------------------------------------------------------------------
// TCheckBox::SetText: 
//----------------------------------------------------------------------------------------

void TCheckBox::SetText(const CStr255_AC& itsText, bool redraw)
{
	// Change the current text in TCtlMgr and in
	// the AGA object. Do NOT let control mgr draw.

	Inherited::SetText(itsText, kDontRedraw);

	if (fAGAControl)
	{
		fAGAControl->SetTitle(itsText, IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
}

//----------------------------------------------------------------------------------------
// TCheckBox::CreateAGAControl: 
//----------------------------------------------------------------------------------------

void TCheckBox::CreateAGAControl()
{
	fAGAControl = TH_new CCheckBox_AC(ControlQDArea(), fTextStyle, GetText(), CControl_AC::kNoAutomaticState);
	if (GetLongVal() != 0)
	{
		fAGAControl->SetValue(CCheckBox_AC::kCheckBoxOn, CControl_AC::kDontRedraw);
	}
}

#endif // qGrayscaleAppearance
