//----------------------------------------------------------------------------------------
// UCluster.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCLUSTER__
#include "UCluster.h"
#endif

// ACS

#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif

// MacApp

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USUBSTITUTION__
#include "USubstitution.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

void InitUCluster()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TCluster, kStdCluster);
		MA_REGISTER_SIGNATURE(TClusterBehavior, kStdClusterBehavior);
#if qGrayscaleAppearance
		MA_REGISTER_CLASS(TPrimaryCluster);
		MA_REGISTER_CLASS(TSecondaryCluster);
#endif
	}
}

//========================================================================================
// CLASS TClusterBehavior
//========================================================================================
#undef Inherited
#define Inherited TBehavior

MA_DEFINE_CLASS(TClusterBehavior);

//----------------------------------------------------------------------------------------
// TClusterBehavior constructor
//----------------------------------------------------------------------------------------

TClusterBehavior::TClusterBehavior(IDType itsIdentifier) :
	TBehavior(itsIdentifier),
	fCurrentChoice(kNoIdentifier)
{
}
	
//----------------------------------------------------------------------------------------
// TClusterBehavior destructor
//----------------------------------------------------------------------------------------

TClusterBehavior::~TClusterBehavior()
{

}

//----------------------------------------------------------------------------------------
// TClusterBehavior::DoEvent:
//----------------------------------------------------------------------------------------

void TClusterBehavior::DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event)
// Responsible for resetting the radio buttons so that the last selected one is
// the only one selected.
{
	// If we got this far, a radio's changed state. Only worry about it if it's our subview!
	TView* ownerAsView = dynamic_cast<TView*>(fOwner);
	
	if (eventNumber == mRadioHit)
	{
	//  && (dynamic_cast<TView*>(source)->GetSuperView() == ownerAsView))
	
		TView* sourceView = dynamic_cast<TView*>(source);
	
		if(ownerAsView && sourceView && (sourceView->GetSuperView() == ownerAsView))
		{
			for (CSubViewIterator iter(ownerAsView); iter; ++iter)
			{
				// If the subview is a TRadio, and it's not the calling radio 
				if (iter.Current() != source)
					iter->HandleEvent(mTurnOff, ownerAsView, NULL);	// set it off and redraw it 
			}
			
			fCurrentChoice = source->GetIdentifier();
		}
	}
	
	Inherited::DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TClusterBehavior::GetCurrentChoice: 
//----------------------------------------------------------------------------------------

IDType TClusterBehavior::GetCurrentChoice()
{
	return fCurrentChoice;
}

//----------------------------------------------------------------------------------------
// TClusterBehavior::SetCurrentChoice: 
//----------------------------------------------------------------------------------------

void TClusterBehavior::SetCurrentChoice(IDType newChoice)
{
	TView* ownerAsView = dynamic_cast_AC(TView*,fOwner);
	
	for (CSubViewIterator iter(ownerAsView); iter; ++iter)
		iter->DoEvent(iter->GetIdentifier() == newChoice ? mTurnOn : mTurnOff, ownerAsView, NULL);	 

	fCurrentChoice = newChoice;
}

//========================================================================================
// CLASS TCluster
//========================================================================================
#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS(TCluster);

//----------------------------------------------------------------------------------------
// TCluster constructor
//----------------------------------------------------------------------------------------

TCluster::TCluster(const CViewPoint& itsLocation,
						const CViewPoint& itsSize,
						SizeDeterminer itsHSizeDet,
						SizeDeterminer itsVSizeDet,
						ResNumber itsRsrcID,
						short itsIndex) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

#if qGrayscaleAppearance
	fAGAControl(NULL),
#endif
	fDataHandle(NULL),
	fStrListID(itsRsrcID),
	fIndex(itsIndex)
{
	fEventNumber = mClusterHit;

	if (fStrListID != kNoResource)
	{
		CStr255_AC aString(fStrListID, fIndex);
		::ThrowIfResError_AC();

		SetLabel(aString, kDontRedraw);
	}

	TCluster::SetEnable(false);			

	fClusterBehavior = TH_new TClusterBehavior(kStdClusterBehavior); // ??? by type?  Overrideable?
	AddBehavior(fClusterBehavior);	
}

//----------------------------------------------------------------------------------------
// TCluster copy constructor
//----------------------------------------------------------------------------------------

TCluster::TCluster(const TCluster& inCluster) :
#if qGrayscaleAppearance
	fAGAControl(inCluster.fAGAControl.get()),
#endif
	fDataHandle(NULL),
	fClusterBehavior(inCluster.fClusterBehavior ? static_cast<TClusterBehavior*>(inCluster.fClusterBehavior->Clone()) : NULL),
	fStrListID(inCluster.fStrListID),
	fIndex(inCluster.fIndex)
{
	if (inCluster.fDataHandle)
		SetLabel(GetLabel(), kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TCluster::Free: 
//----------------------------------------------------------------------------------------

TCluster::~TCluster()
{
	ReleaseLabel();
}

//----------------------------------------------------------------------------------------
// TCluster::GetStandardSignature: 
//----------------------------------------------------------------------------------------

IDType TCluster::GetStandardSignature() const	// Override 
{
	return kStdCluster;
}

//----------------------------------------------------------------------------------------
// TCluster::ReadFields: 
//----------------------------------------------------------------------------------------

void TCluster::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	CStr255_AC theLabel;
	
	*aStream >> fStrListID
			 >> fIndex;
	
	if (fStrListID != kNoResource)					// retrieve the buttons's label from the resource
		GetIndString(theLabel, fStrListID, fIndex);
	SetLabel(theLabel, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TCluster::WriteFields: 
//----------------------------------------------------------------------------------------

void TCluster::WriteFields(CStream_AC* aStream) const				// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fStrListID
			 << fIndex;
}

//----------------------------------------------------------------------------------------
// TCluster::Draw: 
//----------------------------------------------------------------------------------------

void TCluster::Draw(const CViewRect& area)	// Override 
{
	if (IsDrawingBorder())
	{
#if qGrayscaleAppearance
		if (IsEmulatingControls())
		{
			if (fAGAControl)
				fAGAControl->DrawObject();
		}
		else
#endif // qGrayscaleAppearance
		{
			static const short labelOffset = 16;	// offset of label from left edge of cluster
			PenNormal();							// Normalcy in most things 
			
			CPoint_AC itsPenSize;
			GetPenSize(itsPenSize);
			PenSize(itsPenSize.h, itsPenSize.v);		 

			short theFontHeight;

			CRGBColor_AC oldColor(GetIfColor());					// Save the original pen color 
			{	// block for CWhileOutlinePreferred (which has a constructor/destructor)
				CWhileOutlinePreferred_AC setOP(fPreferOutline);

				FontInfo theFontInfo;
				GetTextStyleFontInfo(fTextStyle, theFontInfo, theFontHeight);
			}
			// retrieve the label and set it
			CStr255_AC theText(GetLabel());
			MAReplaceText(theText);
			CViewRect labelRect(labelOffset, 0, labelOffset + StringWidth(theText) + 8, theFontHeight);

			CRect_AC theQDFrame(ControlQDArea());			// Get the control's extent 
			ViewCoordinate theFrameTop = theQDFrame.top + ((theFontHeight) >> 1);

			theQDFrame.Inset(1, 1);	// Inset the frame a little.

			theQDFrame.top = theFrameTop;				// Bump top so it cuts label in half 

			// Draw the frame.  We can't use FrameRect here because top line appears behind
			// the text when printing regardless of whether we eraseRect first or not.
			::MoveTo(theQDFrame.left + labelOffset, theQDFrame.top);
			::LineToPt(theQDFrame[topLeft]);
			::LineTo(theQDFrame.left, theQDFrame.bottom);
			::LineToPt(theQDFrame[botRight]);
			::LineTo(theQDFrame.right, theQDFrame.top);
			::LineTo(theQDFrame.left + labelOffset + (short)labelRect.GetLength(hSel), theQDFrame.top);

			DrawLabel(area);					// Draw the label 

			SetIfColor(oldColor);
		}
		Inherited::Draw(area);						// Let parents have a chance to draw too 
	}
}

//----------------------------------------------------------------------------------------
// TCluster::SetEnable: 
//----------------------------------------------------------------------------------------

void TCluster::SetEnable(bool state)
{
	for (CSubViewIterator iter(this); iter; ++iter)
		iter->SetEnable(state);
	
	Inherited::SetEnable(state);
}

//----------------------------------------------------------------------------------------
// TCluster::DimState: 
//----------------------------------------------------------------------------------------

void TCluster::DimState(bool state,bool redraw)
{
	for (CSubViewIterator iter(this); iter; ++iter)
	{
		TCtlMgr *theCtlMgr = dynamic_cast<TCtlMgr*>(iter.Current());
		if (theCtlMgr)
			theCtlMgr->DimState(state,redraw); 
		else
		{
			if (state && !iter->AdornerWithID(kDimAdorner))
				iter->AddAdorner(&TAdorner::fgDimAdorner, kAdornLast - 10,redraw);
			else if (!state && iter->AdornerWithID(kDimAdorner))
				iter->DeleteAdorner(&TAdorner::fgDimAdorner,redraw);
		}
	}
												
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		fDimmed = state;

		if (fAGAControl)
			fAGAControl->SetEnable(!state, redraw && Focus() && IsVisible());
	}
	else
#endif
		Inherited::DimState(state,redraw);
}

	
//----------------------------------------------------------------------------------------
// TCluster::DrawLabel: 
//----------------------------------------------------------------------------------------

void TCluster::DrawLabel(const CViewRect& area)
{
	const short labelOffset = 16;				// offset of label from left edge of cluster

	CViewRect labelRect;

	// retrieve the label and set it
	CStr255_AC theText(GetLabel());
	MAReplaceText(theText);

	// Calculate the label's CRect_AC
	{	// block for CWhileOutlinePreferred (which has a constructor/destructor)
		CWhileOutlinePreferred_AC setOP(fPreferOutline);
		FontInfo theFontInfo;
		labelRect.Set(labelOffset, 0, labelOffset + StringWidth(theText) + 8, MAGetFontInfo(theFontInfo));
	}

	// Intersect label's CRect_AC with the area. Draw label if valid intersection.
	if (!(labelRect & area).Empty())
	{
		Ptr textPtr = (Ptr)&theText;			// get address of local CStr255_AC struct
		++textPtr;								// bump it to point to first byte in CPascalStr_AC
		TextBox_AC(textPtr, theText.Length(), ViewToQD(labelRect), teCenter, kNoAutoWrap, NULL, kNoEraseFirst,
					kNoSpaceForCaret, fPreferOutline);
	}
}

//----------------------------------------------------------------------------------------
// TCluster::IsDrawingBorder: 
//----------------------------------------------------------------------------------------

bool TCluster::IsDrawingBorder() const
{
	return fDataHandle != NULL;
}

//----------------------------------------------------------------------------------------
// TCluster::GetLabel: 
//----------------------------------------------------------------------------------------

CStr255_AC TCluster::GetLabel() const
{
	return fDataHandle ? *((StringHandle)fDataHandle) : NULL;
}

//----------------------------------------------------------------------------------------
// TCluster::ReleaseLabel: 
//----------------------------------------------------------------------------------------

void TCluster::ReleaseLabel()
{
	fDataHandle = (CPascalStrHandle_AC)::DisposeIfHandle_AC((Handle)fDataHandle);
}

//----------------------------------------------------------------------------------------
// TCluster::SetLabel: 
//----------------------------------------------------------------------------------------

void TCluster::SetLabel(const CStr255_AC& theLabel,
						bool redraw)

{
	ReleaseLabel();
	if (!theLabel.IsEmpty())
	{
		fDataHandle = (CPascalStrHandle_AC)NewString(theLabel);
		if (MemError() != noErr)
			fDataHandle = NULL;
	}
	if (redraw)
		ForceRedraw();

#if qGrayscaleAppearance	
	if (fAGAControl)
		fAGAControl->SetTitle(theLabel, CControl_AC::kDontRedraw);
#endif // qGrayscaleAppearance
}

//----------------------------------------------------------------------------------------
// TCluster::SetLabelWithStrListID: 
//----------------------------------------------------------------------------------------

void TCluster::SetLabelWithStrListID(ResNumber theStrListID, short theIndex, bool redraw)
{
	ReleaseLabel();
	
	fStrListID = theStrListID;
	fIndex = theIndex;
	if (fStrListID != kNoResource)
	{
		CStr255_AC aString(fStrListID, fIndex);
		::ThrowIfResError_AC();
		SetLabel(aString, kDontRedraw);
	}

	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TCluster::GetCurrentChoice: 
//----------------------------------------------------------------------------------------

IDType TCluster::GetCurrentChoice()
{
	return fClusterBehavior->GetCurrentChoice();
}

//----------------------------------------------------------------------------------------
// TCluster::SetCurrentChoice: 
//----------------------------------------------------------------------------------------

void TCluster::SetCurrentChoice(IDType newChoice)
{
	fClusterBehavior->SetCurrentChoice(newChoice);
}

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TCluster::DoPostCreate: 
//----------------------------------------------------------------------------------------

	
void TCluster::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	CreateAGAControl();
}

//----------------------------------------------------------------------------------------
// TCluster::SetFrame: 
//----------------------------------------------------------------------------------------

void TCluster::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	
	if (fAGAControl)
		fAGAControl->SetBounds(ControlQDArea(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TCtlMgr::Activate:
//----------------------------------------------------------------------------------------

void TCluster::Activate(bool entering)
{
	if (IsEmulatingControls())
	{
		if (fAGAControl)
			fAGAControl->SetActive(entering, Focus() && IsVisible());
	}
	Inherited::Activate(entering);
}

//----------------------------------------------------------------------------------------
// TCluster::Dim: 
//----------------------------------------------------------------------------------------

void TCluster::Dim()
{
	if (fAGAControl)
		fAGAControl->SetEnable(CControl_AC::kDisabled, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TCluster::DoMouseCommand: 
//----------------------------------------------------------------------------------------

void TCluster::DoMouseCommand(CViewPoint& /*theMouse*/, TToolboxEvent* /*event*/, CPoint_AC /*hysteresis*/)
{
	// Group box drawing is based on enable state, but we don't
	// want clicking on the group box itself to cause highlighting.
}

//----------------------------------------------------------------------------------------
// TCluster::IsPrimaryType: 
//----------------------------------------------------------------------------------------

bool TCluster::IsPrimaryType() const
{
	return true;
}

//----------------------------------------------------------------------------------------
// TCluster::DoMakeControlAGA: 
//----------------------------------------------------------------------------------------

void TCluster::CreateAGAControl()
{	
	fTextStyle.Normalize(gSystemStyle.tsFont, gApplicationStyle.tsFont);
	fAGAControl = TH_new CCluster_AC(ControlQDArea(), fTextStyle, teFlushDefault, IsPrimaryType(), GetLabel());

	if (fDimmed)
		fAGAControl->SetEnable(CControl_AC::kDisabled, CControl_AC::kDontRedraw);
}

//========================================================================================
// CLASS TPrimaryCluster
//========================================================================================

#undef Inherited
#define Inherited TCluster

MA_DEFINE_CLASS(TPrimaryCluster);

//----------------------------------------------------------------------------------------
// TPrimaryCluster constructor
//----------------------------------------------------------------------------------------

TPrimaryCluster::TPrimaryCluster(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, ResNumber itsRsrcID, short itsIndex) :
	TCluster(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsRsrcID, itsIndex),
	fTitleViewID(kNoIdentifier)
{
}

//----------------------------------------------------------------------------------------
// TPrimaryCluster destructor
//----------------------------------------------------------------------------------------

TPrimaryCluster::~TPrimaryCluster()
{
}

//----------------------------------------------------------------------------------------
// TPrimaryCluster::IsDrawingBorder: 
//----------------------------------------------------------------------------------------

bool TPrimaryCluster::IsDrawingBorder() const
{
	return true;
}

//----------------------------------------------------------------------------------------
// TPrimaryCluster::ReadFields: 
//----------------------------------------------------------------------------------------

void TPrimaryCluster::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fTitleViewID;
}

//----------------------------------------------------------------------------------------
// TPrimaryCluster::WriteFields: 
//----------------------------------------------------------------------------------------

void TPrimaryCluster::WriteFields(CStream_AC* aStream) const				// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fTitleViewID;
}

//----------------------------------------------------------------------------------------
// TPrimaryCluster::CreateAGAControl: 
//----------------------------------------------------------------------------------------

void TPrimaryCluster::CreateAGAControl()							// Override 
{	
	Inherited::CreateAGAControl();

	if ((fTitleViewID != kNoIdentifier) && (fSuperView != NULL))
	{
		TView* gapView = fSuperView->FindSubView(fTitleViewID);

		if (gapView != NULL)
			fAGAControl->SetTitleGap(5 + gapView->GetQDExtent().GetLength(hSel));
	}
}

//========================================================================================
// CLASS TSecondaryCluster
//========================================================================================

#undef Inherited
#define Inherited TPrimaryCluster

MA_DEFINE_CLASS(TSecondaryCluster);

//----------------------------------------------------------------------------------------
// TSecondaryCluster constructor
//----------------------------------------------------------------------------------------

TSecondaryCluster::TSecondaryCluster(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
								 ResNumber itsRsrcID,
								 short itsIndex) :
	TPrimaryCluster(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsRsrcID, itsIndex)
{
}

//----------------------------------------------------------------------------------------
// TSecondaryCluster destructor
//----------------------------------------------------------------------------------------

TSecondaryCluster::~TSecondaryCluster()
{
}

//----------------------------------------------------------------------------------------
// TSecondaryCluster::IsPrimaryType: 
//----------------------------------------------------------------------------------------

bool TSecondaryCluster::IsPrimaryType() const
{
	return false;
}

#endif // qGrayscaleAppearance

