//----------------------------------------------------------------------------------------
// UDialogTEView.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// macapp

#ifndef __UDIALOGTEVIEW__
#include "UDialogTEView.h"
#endif

#ifndef __UEDITTEXT__
#include "UEditText.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif


//========================================================================================
// CLASS TDialogTEView
//========================================================================================
#undef Inherited
#define Inherited TTEView

MA_DEFINE_CLASS(TDialogTEView);

//----------------------------------------------------------------------------------------
// TDialogTEView constructor
//----------------------------------------------------------------------------------------

TDialogTEView::TDialogTEView(const CViewPoint& itsLocation,
							 const CViewPoint& itsSize,
							 SizeDeterminer itsHDeterminer,
							 SizeDeterminer itsVDeterminer,
							 const CViewRect& itsInset,
							 const CTextStyle_AC& itsTextStyle,
							 short itsJustification,
							 bool itsAutoWrap) :
	TTEView(itsLocation, itsSize, itsHDeterminer, itsVDeterminer, itsInset, itsTextStyle, itsJustification, itsAutoWrap, kMakeTERecord),
	fEditText(NULL),
	fScroller(TH_new TScroller(gZeroVPt, gZeroVPt, sizeRelSuperView, sizeRelSuperView, gZeroVPt, !kWantHScrollBar, !kWantVScrollBar))
{
	::TEFeatureFlag(teFOutlineHilite, teBitClear, fHTE);
	
	// NOTE: It is important that the newly created TDialogTEView be connected
	// to its superview which is the scroller stored in fScroller.  TFloatingTEManager
	// does this for you in its MakeDialogTEView.  If you create a TDialogTEView
	// manually you should probably call TDialogTEView::AddToScroller.
}

//----------------------------------------------------------------------------------------
// TDialogTEView copy constructor
//----------------------------------------------------------------------------------------

TDialogTEView::TDialogTEView(const TDialogTEView& inDialogTEView)
	:	fEditText(NULL) 
{
	if (inDialogTEView.fScroller)
	{
		// Remove ourselves from the superviews subview list to prevent an
		// infinite recursion.
		inDialogTEView.fScroller->DeleteSubView(const_cast<TDialogTEView*>(&inDialogTEView));
		fScroller = static_cast<TScroller*>(fScroller->Clone());
		fScroller->AddSubView(this);
		inDialogTEView.fScroller->InsertSubView(const_cast<TDialogTEView*>(&inDialogTEView), true /*last*/);
	}
}

//----------------------------------------------------------------------------------------
// TDialogTEView destructor
//----------------------------------------------------------------------------------------

TDialogTEView::~TDialogTEView()
{
	if (fScroller)
		fScroller->RemoveSubView(this);
}

//----------------------------------------------------------------------------------------
// TDialogTEView::ReadFields: 
//----------------------------------------------------------------------------------------

void TDialogTEView::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	fScroller = TH_new TScroller(gZeroVPt, gZeroVPt, sizeRelSuperView, sizeRelSuperView, gZeroVPt, !kWantHScrollBar, !kWantVScrollBar);
	fScroller->AddSubView(this);

	TEFeatureFlag(teFOutlineHilite, teBitClear, fHTE);
}

//----------------------------------------------------------------------------------------
// TDialogTEView::WriteFields: 
//----------------------------------------------------------------------------------------

void TDialogTEView::WriteFields(CStream_AC* aStream) const		// Override 
{
	Inherited::WriteFields(aStream);
}

//----------------------------------------------------------------------------------------
// TDialogTEView::InstallEditText: 
//----------------------------------------------------------------------------------------

void TDialogTEView::InstallEditText(TEditText* theEditText, bool selectChars)
{
	if (fEditText)
	{
		fEditText->RemoveSubView(fScroller);
		fEditText = NULL;
	}

	if (theEditText)
	{
		fPreferOutline = theEditText->fPreferOutline;
		fControlChars = theEditText->fControlChars;
		fMaxChars = theEditText->fMaxChars;
		fInset.Clear();
		//bool hadPendingUpdate = theEditText->HasPendingUpdate();	// not used

		SetJustification(theEditText->fJust, kDontRedraw);
		ChangeWrap(theEditText->fAutoWrap, kDontRedraw);

		// was: SetOneStyle(0, 0, doAll, theEditText->fTextStyle, kDontRedraw);
		// Fix up the default style and apply it to non-empty text because
		// otherwise the default style may not stick in the real call to 
		// SetText (this is a TE "limitation" in how we are using it).
		fTextStyle = theEditText->fTextStyle;
		SetText(theEditText->GetText());			
		SetOneStyle(0, SHRT_MAX, doAll, fTextStyle, kDontRedraw);

		CViewRect theControlArea(theEditText->ControlArea());

		if (fAutoWrap)
		{
			fSizeDeterminer[hSel] = sizeSuperView;
			fSizeDeterminer[vSel] = sizeVariable;
		}
		else
			fSizeDeterminer[hSel] = sizeVariable;	// Let the width vary with the number
                                                    // of characters

		fSuperView->SetFrame(theControlArea, kDontInvalidate);
		AdjustFrame();

		// Set the drawing environment to that of theEditText or its superview
		fDrawingEnvironment.reset();			// Get rid of any lingering drawing environment
		TView* view = theEditText;
		TDrawingEnvironment* drawingEnv = view->GetDrawingEnvironment();
		while (drawingEnv == NULL)				// Find a new drawing environment
		{
			view = view->GetSuperView();
			if (view == NULL)
				break;
			drawingEnv = view->GetDrawingEnvironment();
		}
		if (drawingEnv)							// Adopt the new drawing environment
			fDrawingEnvironment = static_cast<TDrawingEnvironment*>(drawingEnv->Clone());

		// Make the scroller's thinking match the display that the user already sees
		CViewPoint newTranslation(fScroller->GetTranslation());
		newTranslation.h = 0;
		switch (GetActualJustification(fJustification))
		{
			case teFlushDefault:
			case teFlushLeft:
				newTranslation.v = 0;
				break;
			case teFlushRight:					// Right brain thinkers left brain thinkers??
				newTranslation.h = fScroller->fMaxTranslation.h;
				break;
			case teCenter:
				newTranslation.h = fScroller->fMaxTranslation.h / 2;
				break;
		}
		fScroller->SetTranslation(newTranslation);

		SetSelect(0, selectChars ? SHRT_MAX : 0, fHTE);
			
		BeInScroller(fScroller);

		theEditText->AddSubView(fScroller);		// my scroller 
		SynchView(kDontRedraw);

		// Make my enable and my scroller's enable match my new superview 
		SetEnable(theEditText->IsEnabled());
		fScroller->SetEnable(theEditText->IsEnabled());

		// Set my document to that of the EditText, so changes get recorded
		BeInDocument(theEditText->GetDocument());
	}
	else	// The floating TE is being uninstalled. 
	{
		// Remove TDialogTEView and its scroller from the document's view list
		RemoveFromDocument();
		fScroller->RemoveFromDocument();
	}

#if qDrag
	// on acquisition, clear drag and drop tracking data
	fLastCaretOffset = 0;
	fLastCaretTime = 0L;
	fCaretIsShown = false;
#endif

	fEditText = theEditText;
}

//----------------------------------------------------------------------------------------
// TDialogTEView::ResignedTarget: 
//----------------------------------------------------------------------------------------

void TDialogTEView::ResignedTarget()	// Override 
{
	if (IsActive())
	{
		// If we're deselecting a field and it's been scrolled, invalidate it
		// so that it is redrawn correctly.
		if (fScroller->GetTranslation().v)
			ForceRedraw();
		else
		{
			switch (GetActualJustification(fJustification))
			{
				case teFlushDefault:
				case teFlushLeft:
					if (fScroller->GetTranslation().h)
						ForceRedraw();
					break;
				case teFlushRight:
					if (fScroller->GetTranslation().h != fScroller->fMaxTranslation.h)
						ForceRedraw();
					break;
				case teCenter:
					if (fScroller->GetTranslation().h != (fScroller->fMaxTranslation.h / 2))
						ForceRedraw();
					break;
			}
		}
		
		TView::ResignedTarget(); // send mResignedTarget event

		if (fEditText)
			fEditText->StopEdit();
	}
	else
		Inherited::ResignedTarget();
}

//----------------------------------------------------------------------------------------
// TDialogTEView::TargetValidationFailed: 
//----------------------------------------------------------------------------------------

void TDialogTEView::TargetValidationFailed(long reason)	// Override 
{
	if (fEditText)
		fEditText->ValidationFailed(reason);
}

//----------------------------------------------------------------------------------------
// TDialogTEView::WillingToResignTarget: 
//----------------------------------------------------------------------------------------

long TDialogTEView::WillingToResignTarget()	// Override 
{
	long result = 0;

	if (fEditText)
	{
		//	GetContext(cNoCommand)->CommitLastCommand();
		result = fEditText->GetValidationError();
	}
	return result;
}

//----------------------------------------------------------------------------------------
// TDialogTEView::ComputeFrame: 
//----------------------------------------------------------------------------------------

void TDialogTEView::ComputeFrame(CViewRect& newFrame)// Override 
{
	Inherited::ComputeFrame(newFrame);

	if (fAutoWrap)
		switch (fSizeDeterminer[vSel])
		{
			case sizeVariable:
				// TTEView already computed the variable size, bump it up to at least the scroller's
				// size so that the cursor is claimed for the EditText and the user can click anywhere.
//SRF				if (!fStyleType)
					newFrame.bottom = newFrame.top + Max_AC(fScroller->GetSize().v, newFrame.GetLength(vSel));
				break;
		}
	else
		switch (fSizeDeterminer[hSel])
		{
			case sizeVariable:
				// TTEView already computed the variable size, bump it up to at least the scroller's
				// size so that the cursor is claimed for the EditText and the user can click anywhere.
//SRF				if (!fStyleType)
					newFrame.right = newFrame.left + Max_AC(fScroller->GetSize().h, newFrame.GetLength(hSel));
				break;
		}
}

//----------------------------------------------------------------------------------------
// TDialogTEView::GetEditText: 
//----------------------------------------------------------------------------------------
TEditText* TDialogTEView::GetEditText()
{
	return fEditText;
}

//----------------------------------------------------------------------------------------
// TDialogTEView::AddToScroller: 
//----------------------------------------------------------------------------------------
void TDialogTEView::AddToScroller()
{
	fScroller->AddSubView(this);
}

#if qDrag

//----------------------------------------------------------------------------------------
// TDialogTEView::WillDrag: 
//----------------------------------------------------------------------------------------

bool TDialogTEView::WillDrag(const CViewPoint& localMouse, const RgnHandle dragCursorRegion)
{
	bool	result;
	bool 	savedDraggable = GetDraggable();
	
	// temporarily set this view's draggable flag to the state of its edit text
	SetDraggable(fEditText->GetDraggable());
	
	try
	{
		result = Inherited::WillDrag(localMouse, dragCursorRegion);
		SetDraggable(savedDraggable);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		SetDraggable(savedDraggable);
		throw;
	}
	
	return result;
}

//----------------------------------------------------------------------------------------
// TDialogTEView::DoGetDragProxy: 
//----------------------------------------------------------------------------------------

TView* TDialogTEView::DoGetDragProxy()
{
	return fEditText;	// fEditText will manage the drag
}

//----------------------------------------------------------------------------------------
// TDialogTEView::GetWillDragCursorID: 
//----------------------------------------------------------------------------------------

short TDialogTEView::GetWillDragCursorID()
{
	return fEditText ? fEditText->GetWillDragCursorID() : Inherited::GetWillDragCursorID();
}

//----------------------------------------------------------------------------------------
// TDialogTEView::GetIsDraggingCursorID: 
//----------------------------------------------------------------------------------------

short TDialogTEView::GetIsDraggingCursorID()
{
	return fEditText ? fEditText->GetIsDraggingCursorID() : Inherited::GetIsDraggingCursorID();
}

#endif // qDrag
	
