//----------------------------------------------------------------------------------------
// UDialogView.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDIALOGVIEW__
#include "UDialogView.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// InitUDialogView: 
//----------------------------------------------------------------------------------------
void InitUDialogView()
{
	static bool UDialogViewInitialized;
	if (!UDialogViewInitialized)
	{	
		if (qTemplateViews)
		{
			MA_REGISTER_SIGNATURE(TDialogView, kStdDialogView);
			MA_REGISTER_SIGNATURE(TDialogBehavior, kStdDialogBehavior);
		}
	
		UDialogViewInitialized = true;
	}
}

//========================================================================================
// CLASS TDialogView
//========================================================================================
MA_DEFINE_CLASS(TDialogView);

//----------------------------------------------------------------------------------------
// TDialogView constructor
//----------------------------------------------------------------------------------------


TDialogView::TDialogView(const CViewPoint& itsLocation,
							  const CViewPoint& itsSize,
							  SizeDeterminer itsHSizeDet,
							  SizeDeterminer itsVSizeDet,
							  IDType itsDefItemID,
							  IDType itsCancelItemID) :
	TView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fDefaultItem(itsDefItemID),
	fCancelItem(itsCancelItemID)
{
}

//----------------------------------------------------------------------------------------
// TDialogView destructor
//----------------------------------------------------------------------------------------


TDialogView::~TDialogView()
{
}

//----------------------------------------------------------------------------------------
// TDialogView::DoPostCreate: 
//----------------------------------------------------------------------------------------


void TDialogView::DoPostCreate(TDocument *itsDocument)	// Override
{
	TView::DoPostCreate(itsDocument);
	
	TWindow* theWindow = GetWindow();
	if (theWindow)
		theWindow->SetDialogItems(fDefaultItem,fCancelItem);
	
}

//----------------------------------------------------------------------------------------
// TDialogView::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TDialogView::GetStandardSignature() const	// Override 
{
	return kStdDialogView;
}

//----------------------------------------------------------------------------------------
// TDialogView::ReadFields: 
//----------------------------------------------------------------------------------------


void TDialogView::ReadFields(CStream_AC* aStream)	// Override 
{
	TView::ReadFields(aStream);

	fDefaultItem = aStream->ReadIDType();
	fCancelItem = aStream->ReadIDType();
}

//----------------------------------------------------------------------------------------
// TDialogView::WriteFields: 
//----------------------------------------------------------------------------------------


void TDialogView::WriteFields(CStream_AC* aStream) const	// Override 
{
	TView::WriteFields(aStream);

	aStream->WriteIDType(fDefaultItem);
	aStream->WriteIDType(fCancelItem);
}

