//----------------------------------------------------------------------------------------
// UDisclosureBehavior.h.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDISCLOSUREBEHAVIOR__
#include "UDisclosureBehavior.h"
#endif

#ifndef __CStream_AC__
#include "CStream_AC.h"
#endif

//========================================================================================
// Global Initialization Procedure
//========================================================================================

//----------------------------------------------------------------------------------------
// InitUDisclosureBehavior.h
//----------------------------------------------------------------------------------------

void InitUDisclosureBehavior()
{
	// So the linker doesn't dead strip class info 
	MA_REGISTER_CLASS(TDisclosureBehavior);
}

//========================================================================================
// CLASS TDisclosureBehavior
//========================================================================================

#undef Inherited


MA_DEFINE_CLASS(TDisclosureBehavior);

//----------------------------------------------------------------------------------------
// TDisclosureBehavior constructors
//----------------------------------------------------------------------------------------


TDisclosureBehavior::TDisclosureBehavior(IDType itsTarget,
										IDType itsControl,
										EventNumber	controlEventID,
										IDType itsIdentifier) :
	TBehavior(itsIdentifier),
	fTargetViewID(itsTarget),
	fControlID(itsControl),
	fTarget(NULL),
	fControl(NULL),
	fEventNumber(controlEventID)
{
}



TDisclosureBehavior::TDisclosureBehavior(IDType itsIdentifier) :
	TBehavior(itsIdentifier),
	fTargetViewID(0),
	fControlID(0),
	fTarget(NULL),
	fControl(NULL),
	fEventNumber(0)
// Constructor used for streaming
{
}

//----------------------------------------------------------------------------------------
// TDisclosureBehavior copy constructor
//----------------------------------------------------------------------------------------


TDisclosureBehavior::TDisclosureBehavior(const TDisclosureBehavior& inDisclosureBehavior) :
	TBehavior(inDisclosureBehavior),
	fTargetViewID(inDisclosureBehavior.fTargetViewID),
	fControlID(inDisclosureBehavior.fControlID),
	fTarget(inDisclosureBehavior.fTarget),
	fControl(inDisclosureBehavior.fControl),
	fEventNumber(inDisclosureBehavior.fEventNumber)
{
}

//----------------------------------------------------------------------------------------
// TDisclosureBehavior::destructor
//----------------------------------------------------------------------------------------


TDisclosureBehavior::~TDisclosureBehavior()
{
}

//----------------------------------------------------------------------------------------
// TDisclosureBehavior::ReadFrom:
//----------------------------------------------------------------------------------------


void TDisclosureBehavior::ReadFrom(CStream_AC* aStream)		// Override 
{
	TBehavior::ReadFrom(aStream);

	fTargetViewID = aStream->ReadIDType();
	fControlID = aStream->ReadIDType();
	fEventNumber = aStream->ReadLong();
}

//----------------------------------------------------------------------------------------
// TDisclosureBehavior::WriteTo:
//----------------------------------------------------------------------------------------


void TDisclosureBehavior::WriteTo(CStream_AC* aStream) const		// Override 
{
	TBehavior::WriteTo(aStream);

	aStream->WriteIDType(fTargetViewID);
	aStream->WriteIDType(fControlID);
	aStream->WriteLong(fEventNumber);				// write its fEventNumber
}


//----------------------------------------------------------------------------------------
// TDisclosureBehavior::InitializeDisclosureBehavior
//----------------------------------------------------------------------------------------


void TDisclosureBehavior::InitializeDisclosureBehavior()
{
	// Check to see if we've been initialized and if we haven't then 
	// add dependencies to the target views.
	if (fTarget == NULL)
	{
		TView * view = NULL;
		TDisclosureTriangle* control = NULL;
		
		TView* owner = dynamic_cast_AC(TView*,fOwner);

		view = owner->FindSubView(fControlID);
		control = dynamic_cast_AC(TDisclosureTriangle*,view);
		
		fControl = control;

		view = owner->FindSubView(fTargetViewID);
		::ThrowIfNULL_AC(view);
		
		fTarget = view;
	}
}

//----------------------------------------------------------------------------------------
// TDisclosureBehavior::DoEvent
//----------------------------------------------------------------------------------------


void TDisclosureBehavior::DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event)
//	Upon receiving an event matching the one expected from the disclosure triangle, this
//	routing checks the current state of the triangle and adjusts the bounds of the owning 
//	view to hide or show the target view.
{
	if (fTarget == NULL) InitializeDisclosureBehavior();
	
	if (eventNumber == fEventNumber)
	{
		TView* owner = dynamic_cast_AC(TView*,fOwner);	
	
		CViewRect targetFrame(owner->LocalToSuperView(fTarget->GetFrame())); // Work in owner's parent's coordinates.
		CViewRect ownerFrame(owner->GetFrame());
		CViewRect newFrame(ownerFrame);
		bool controlState = fControl->GetAGAControl()->GetState();
		if (controlState)
		{
			newFrame.bottom = targetFrame.bottom;
		}
		else
		{
			newFrame.bottom = targetFrame.top;
		}
		if (newFrame != ownerFrame)
		{
			owner->SetFrame(newFrame, true);
		}
	}
	else
	{
		TBehavior::DoEvent(eventNumber, source, event);
	}
}

