//----------------------------------------------------------------------------------------
// UDisclosureTriangle.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UDISCLOSURETRIANGLE__
#include "UDisclosureTriangle.h"
#endif

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void InitUDisclosureTriangle()
{
	MA_REGISTER_CLASS(TDisclosureTriangle);
}

//========================================================================================
// CLASS TDisclosureTriangle
//========================================================================================

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS(TDisclosureTriangle);

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

TDisclosureTriangle::TDisclosureTriangle(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
							     const CTextStyle_AC& itsTextStyle,
							     bool isDisclosed) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle),
	fAGAControl(NULL),
	fIsDisclosed(isDisclosed)
{
	// caller must call virtual function CreateAGAControl();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

TDisclosureTriangle::~TDisclosureTriangle()
{
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::ReadFields(CStream_AC* aStream)
{
	Inherited::ReadFields(aStream);
	
	*aStream >> fIsDisclosed;
}

//----------------------------------------------------------------------------------------
// TDisclosureTriangle
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::WriteFields(CStream_AC* aStream) const
{
	Inherited::WriteFields(aStream);
	
	*aStream << fIsDisclosed;
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);
	
	CreateAGAControl();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::CreateAGAControl()
{
	// Instantiate the particular CControl_AC subclass.
	fAGAControl = TH_new CDisclosureTriangle_AC(ControlQDArea(), CControl_AC::kAutomaticState);
	SetDisclosed(fIsDisclosed, kDontRedraw);

	if (fDimmed || ! fEnabled)
		fAGAControl->SetEnable(CControl_AC::kDisabled, CControl_AC::kDontRedraw);

	// Disclosure triangle has to erase when animating, so make sure that
	// it doesn't assume we're on a gray background.
	(void) Focus();
	fAGAControl->InstallBackgroundColors(CControl_AC::kInstallOnlyIfCustom);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::Draw(const CViewRect& /*area*/)
{
	if (fAGAControl)
		fAGAControl->DrawObject();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	
	if (fAGAControl)
		fAGAControl->SetBounds(ControlQDArea(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* /*event*/, CPoint_AC /*hysteresis*/)
{
	if (fAGAControl->TrackMouse(ViewToQD(theMouse)))
		HandleEvent(fEventNumber, this, NULL);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::Activate(bool entering)
{
	if (fAGAControl)
		fAGAControl->SetActive(entering, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::DimState(bool state, bool redraw)
{
	fDimmed = state;

	if (fAGAControl)
		fAGAControl->SetEnable(!state, redraw && Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::Dim()
{
	if (fAGAControl)
		fAGAControl->SetEnable(CControl_AC::kDisabled, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::SetEnable(bool state)
{
	Inherited::SetEnable(state);

	// Set the CControl_AC's enable state, without redraw.
	if (fAGAControl)
		fAGAControl->SetEnable(state, CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TDisclosureTriangle::SetDisclosed(bool isDisclosed, bool redraw)
{
	// Set the CControl_AC's state, without redraw.
	if (fAGAControl)
		fAGAControl->SetState(isDisclosed, redraw && Focus() && IsVisible());
}

