//----------------------------------------------------------------------------------------
// UForceQuitHandler.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UFORCEQUITHANDLER__
#include "UForceQuitHandler.h"
#endif

// MacApp

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

// Toolbox

// For ExitToShell
#ifndef __SEGLOAD__
#include <SegLoad.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static pascal void ExitToShellCleanupMacApp();

//========================================================================================
// CLASS PatchExitToShell
//========================================================================================


typedef pascal void(* ExitToShellType)();

#if GENERATINGCFM
typedef UniversalProcPtr ExitToShellUPP;
#else
typedef ProcPtr ExitToShellUPP;
#endif

enum
{
	uppExitToShellProcInfo = kPascalStackBased
};

#if GENERATINGCFM
#define NewExitToShellProc(userRoutine)  \
		(ExitToShellUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppExitToShellProcInfo, GetCurrentISA())
#else
#define NewExitToShellProc(userRoutine)		\
		((ExitToShellUPP) (userRoutine))
#endif

#if GENERATINGCFM
#define CallExitToShellProc(userRoutine)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppExitToShellProcInfo)
#else
#define CallExitToShellProc(userRoutine)		\
		(*(userRoutine))()
#endif

class PatchExitToShell : public TrapPatch
{
public:
	void Install(ExitToShellType routine);
	void CallInherited();
};

//--------------------------------------------------------------------------------------------------


void PatchExitToShell::Install(ExitToShellType routine)
{
	patchRoutine = NewExitToShellProc(StripLong(routine));
	::ThrowIfNULL_AC(patchRoutine);
	PatchTrap(_ExitToShell, patchRoutine);
}

//--------------------------------------------------------------------------------------------------


void PatchExitToShell::CallInherited()
{
	CallExitToShellProc((ExitToShellType)oldTrapAddr);
}

//--------------------------------------------------------------------------------------------------

static PatchExitToShell pETSPatch;

//----------------------------------------------------------------------------------------
// ExitToShellCleanupMacApp: 
//----------------------------------------------------------------------------------------


pascal void ExitToShellCleanupMacApp()
{
	long OldA5 = ::SetCurrentA5();				// ***** Called from trap patches *****

	ExitToShellType oldTrapAddr = (ExitToShellType)pETSPatch.GetOldTrapAddr();

	::CleanupMacApp();

	CallExitToShellProc(oldTrapAddr);

	::SetA5(OldA5);
}

//----------------------------------------------------------------------------------------
// InitUForceQuitHandler: 
//----------------------------------------------------------------------------------------


void InitUForceQuitHandler()
{
	pETSPatch.Install(ExitToShellCleanupMacApp);
}


