//----------------------------------------------------------------------------------------
// UGrabberTracker.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UGRABBERTRACKER__
#include "UGrabberTracker.h"
#endif

// MacApp

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif



//========================================================================================
// CLASS TGrabberTracker
//========================================================================================
#undef Inherited
#define Inherited TTracker

//----------------------------------------------------------------------------------------
// TGrabberTracker::TGrabberTracker: 
//----------------------------------------------------------------------------------------


TGrabberTracker::TGrabberTracker(CommandNumber itsCommandNumber,
											 TDocument* itsDocument,
											 TView* itsView,
											 TScroller* itsScroller,
											 const CViewPoint& itsMouse) :
	TTracker(itsCommandNumber, itsDocument, kCantUndo, kDoesNotCauseChange, itsDocument, 
					itsView, itsScroller, itsMouse)
{
}

//----------------------------------------------------------------------------------------
// TGrabberTracker destructor
//----------------------------------------------------------------------------------------


TGrabberTracker::~TGrabberTracker()
{
}

//----------------------------------------------------------------------------------------
// TGrabberTracker::TrackMouse: 
//----------------------------------------------------------------------------------------


TTracker* TGrabberTracker::TrackMouse(TrackPhase aTrackPhase,
											 CViewPoint& anchorPoint,
											 CViewPoint& previousPoint,
											 CViewPoint& nextPoint,
											 bool mouseDidMove)// override 

{
	switch (aTrackPhase)
	{
		case trackBegin:
		case trackEnd:
			break;

		case trackContinue:
			if (mouseDidMove)
			{
				CViewPoint delta = - fScroller->GetTranslation();

				delta.ConstrainTo(CViewRect(anchorPoint - nextPoint, fScroller->fMaxTranslation - fScroller->GetTranslation()));

				if (delta != gZeroVPt)
				{
					fScroller->ScrollBy(delta, true);
					fScroller->Update();
				}

				nextPoint += delta;

				previousPoint = nextPoint;
				anchorPoint = nextPoint;
			}
			break;
	}

	return this;								// keep tracking with me 
}

//----------------------------------------------------------------------------------------
// TGrabberTracker::TrackFeedback: 
//----------------------------------------------------------------------------------------


void TGrabberTracker::TrackFeedback(TrackPhase /* aTrackPhase */,
										const CViewPoint& /* anchorPoint */,
										const CViewPoint& /* previousPoint */,
										const CViewPoint& /* nextPoint */,
										bool /* mouseDidMove */,
										bool /* turnItOn */)// override

{
	// NO feedback please 
}

//----------------------------------------------------------------------------------------
// TGrabberTracker::AutoScroll: 
//----------------------------------------------------------------------------------------


void TGrabberTracker::AutoScroll(const CViewPoint& /* delta */)// override

{
	// NO AutoScroll please 
}

//----------------------------------------------------------------------------------------
// End of UGrabberTracker.cp


