//----------------------------------------------------------------------------------------
// UIcon.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UICON__
#include "UIcon.h"
#endif

// ACS

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

#ifndef __UICONSUITE__
#include "UIconSuite.h"
#endif

#ifndef __MACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USMALLICON__
#include "USmallIcon.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

void InitUIcon(bool InitAllIcons)
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TIcon, kStdIcon);
		
		if(InitAllIcons)
		{
			InitUIconSuite();
			InitUSmallIcon();
		}	
	}
}

//========================================================================================
// CLASS TIcon
//========================================================================================
#undef Inherited
#define Inherited TControl


MA_DEFINE_CLASS(TIcon);


//----------------------------------------------------------------------------------------
// TIcon constructor
//----------------------------------------------------------------------------------------


TIcon::TIcon(const CViewPoint& itsLocation,
				const CViewPoint& itsSize,
				SizeDeterminer itsHSizeDet,
				SizeDeterminer itsVSizeDet,
				ResNumber itsRsrcID,
				eIconSelectorFlag loadFlag) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fIconSuite(NULL),
	fRsrcID(kNoResource),
	fTransform(kNoTransform),
	fAlign(kNoAlignment),
	fLabel(kNoLabel_AC)
{
	fEventNumber = mIconHit;

	SetIconSuiteRsrcID(itsRsrcID, kDontRedraw, loadFlag);
	SetEnable(false);							// Default is to not enable hit testing
}

//----------------------------------------------------------------------------------------
// TIcon copy constructor 
//----------------------------------------------------------------------------------------


TIcon::TIcon(const TIcon& inIcon) :
	fIconSuite(NULL),
	fRsrcID(kNoResource),
	fTransform(inIcon.fTransform),
	fAlign(inIcon.fAlign),
	fLabel(inIcon.fLabel)
{
	if (inIcon.fIconSuite)
		SetIconSuiteRsrcID(inIcon.fRsrcID, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TIcon::~TIcon: 
//----------------------------------------------------------------------------------------


TIcon::~TIcon()
{
	ReleaseIconSuite();
}

//----------------------------------------------------------------------------------------
// TIcon::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TIcon::GetStandardSignature() const			// Override 
{
	return kStdIcon;
}

//----------------------------------------------------------------------------------------
// TIcon::ReadFields: 
//----------------------------------------------------------------------------------------


void TIcon::ReadFields(CStream_AC* aStream)		// Override 
{
	Inherited::ReadFields(aStream);

	bool wantsColor = aStream->ReadBoolean();
		// read it to remain compatible with old TIcon, but
		// we're ignoring the value.
		
	ResNumber itsRsrcID = aStream->ReadInteger();
	
	SetIconSuiteRsrcID(itsRsrcID, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TIcon::WriteFields: 
//----------------------------------------------------------------------------------------


void TIcon::WriteFields(CStream_AC* aStream) const		// Override 
{
	Inherited::WriteFields(aStream);
		
	bool wantsColor = true;
	aStream->WriteBoolean(wantsColor);
		// write it to remain compatible with old TIcon

#if qDebugMsg
	if (fRsrcID == kNoResource)
		ProgramReport_AC("Writing a TIcon with no resource ID.");
#endif

	aStream->WriteInteger(fRsrcID);
}

//----------------------------------------------------------------------------------------
// TIcon::Plot: 
//----------------------------------------------------------------------------------------


void TIcon::Plot()
{
#if qDebug
	AssumeFocused();
#endif

	if(fIconSuite)
	{
		if(IsAResource_AC(fIconSuite))
		{
			::LoadResource(fIconSuite);
		}
	
		if(*fIconSuite)
		{
			// Get our imaging area
			CRect_AC theQDRect(ControlQDArea());
			
			// prevent scaling
			CRect_AC theDestRect = GetIconRect();
			
			theDestRect.Offset( theQDRect.left, theQDRect.top );
			
			::PlotIconSuite(theDestRect, 
							(IconAlignmentType) fAlign, 
							(IconTransformType) fTransform | fLabel, 
							fIconSuite);
				// This will do the right thing in regards to screen depth
				// and plotting the right icon based on the imaging area.
				
		}
	}
}

//----------------------------------------------------------------------------------------
// TIcon::Draw: 
//----------------------------------------------------------------------------------------



void TIcon::Draw(const CViewRect& area)				
{
	Plot();
	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TIcon::ReleaseIcon: 
//----------------------------------------------------------------------------------------


void TIcon::ReleaseIconSuite()
{
	fRsrcID = kNoResource;
	
	if(fIconSuite)
	{
		::DisposeIconSuite(fIconSuite, true);
		fIconSuite = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TIcon::SetIcon: 
//----------------------------------------------------------------------------------------


void TIcon::SetIconSuite(Handle theIconSuite,bool redraw)
{
	ReleaseIconSuite();

	// Make sure the resource is loaded since it may have been purged and we
	// want to be sure that we can get the size to determine the correct type
	// of icon.
	if(IsAResource_AC(theIconSuite))
	{
		::LoadResource(theIconSuite);
		::ThrowIfResError_AC();
	
		// get the rsrc id (this will only work for b&w icons)
		ResNumber theID;
		ResType theType;
		CStr255_AC name;
	
		::GetResInfo(theIconSuite, &theID, &theType, name);
		if (ResError() == noErr)
			fRsrcID = theID;
	}
	fIconSuite = theIconSuite;

	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TIcon::SetIconSuiteRsrcID: 
//----------------------------------------------------------------------------------------


void TIcon::SetIconSuiteRsrcID(ResNumber itsRsrcID, bool redraw, eIconSelectorFlag loadFlag)
{
	ReleaseIconSuite();

	fRsrcID = itsRsrcID;

	if (fRsrcID != kNoResource)
	{
		OSErr err = ::GetIconSuite(&fIconSuite, fRsrcID, (IconSelectorValue) loadFlag);
		::ThrowIfOSErr_AC(err);
	}
	
	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TIcon::HiliteState: 
//----------------------------------------------------------------------------------------


void TIcon::HiliteState(bool state, bool redraw)
{
	
	if (state != fHilite)
	{
		fHilite = state;
				
		if(fHilite)
		{
			fTransform |= kSelected;
		}
		else
		{
			fTransform &= ~kSelected;
		}

		if (redraw && IsDrawable())
			Hilite();
	}
}
//----------------------------------------------------------------------------------------
// TIcon::HiliteState: 
//----------------------------------------------------------------------------------------


void TIcon::Hilite()
{
	Plot();
}

//----------------------------------------------------------------------------------------
// TIcon::Dim: 
//----------------------------------------------------------------------------------------


void TIcon::Dim()
{
	Plot();
}

//----------------------------------------------------------------------------------------
// TIcon::DimState: 
//----------------------------------------------------------------------------------------


void TIcon::DimState(bool state, bool redraw)
{
	if (state != fDimmed)
	{
		fDimmed = state;
		
		if(fDimmed)
			fTransform |= kDisabled;
		else
			fTransform &= ~kDisabled;
		
		if (redraw)
			DrawContents();				// Draw change immediately
	}
}

//----------------------------------------------------------------------------------------
// TIcon::GetExtentRegion
//----------------------------------------------------------------------------------------


CRect_AC TIcon::GetIconRect()
{
	//
	// Return a rect that will cause the similarly sized icon to be retrieved
	// and displayed without any stretching. It will be clipped, however.
	//
	CPoint_AC botRightPoint;

	if (fSize.h <= 16 && fSize.v <= 12)
		botRightPoint.Set(16, 12);
	else if (fSize.h <= 16 && fSize.v <= 16)
		botRightPoint.Set(16, 16);
	else
		botRightPoint.Set(32, 32);

	return CRect_AC(gZeroPt, botRightPoint);
}

//----------------------------------------------------------------------------------------
// TIcon::GetIconSuite
//----------------------------------------------------------------------------------------


Handle TIcon::GetIconSuite() const
{
	return fIconSuite;
}


//----------------------------------------------------------------------------------------
// TIcon::ContainsMouse
//----------------------------------------------------------------------------------------


bool TIcon::ContainsMouse(const CViewPoint& theMouse)
{
	CTempRegion_AC iconRgn;
	::IconSuiteToRgn(iconRgn, &GetIconRect(), fAlign, fIconSuite);
	
	return (IsShown() && PtInRgn(ViewToQD(theMouse), iconRgn));
}
