//----------------------------------------------------------------------------------------
// UIconButton.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UICONBUTTON__
#include "UIconButton.h"
#endif

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// We need to subclass the ACS Popup in order to have MacApp's GetMenu called at
// the appropriate times by that class.

class CMacAppIconPopup : public CIconPopup_AC
{
public:
	CMacAppIconPopup(const CRect_AC& bounds, MenuRef itsMenuRef = NULL, const CTextStyle_AC& itsTextStyle = CTextStyle_AC::kSystemStyle, DisposalKind menuRefDisposalKind = kDontDispose, RefHandling refHandling = kNothing, SInt16 offIconID = kNoIconID, SInt16 onIconID = kNoIconID, ButtonFrameType frameType = kAutoFrame, ButtonImageType imageType = kAutoIconFamily) :
		CIconPopup_AC(bounds, itsMenuRef, itsTextStyle, menuRefDisposalKind, refHandling, offIconID, onIconID, frameType, imageType)
		{}
		
	virtual ~CMacAppIconPopup() 
		{}

	virtual MenuRef LoadMenuRef(SInt16 menuID);
		// overridden to call MAGetMenu
};

//----------------------------------------------------------------------------------------
// InitUIconButton
//----------------------------------------------------------------------------------------

void InitUIconButton()
{
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.

	MA_REGISTER_CLASS(TIconButton);
	MA_REGISTER_CLASS(TIconCheckBox);
	MA_REGISTER_CLASS(TIconRadio);
	MA_REGISTER_CLASS(TIconPopup);
}

//========================================================================================
// CLASS TIconButton
//========================================================================================

#undef Inherited
#define Inherited TButton

MA_DEFINE_CLASS(TIconButton);

//----------------------------------------------------------------------------------------
// TIconButton constructor
//----------------------------------------------------------------------------------------

TIconButton::TIconButton(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255_AC& itsLabel, ResNumber itsRsrcID) :
	TButton(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel),
	fRsrcID(itsRsrcID)
{
}

//----------------------------------------------------------------------------------------
// TIconButton destructor
//----------------------------------------------------------------------------------------

TIconButton::~TIconButton()
{
}

//----------------------------------------------------------------------------------------
// TIconButton::IsStandardControl:
//----------------------------------------------------------------------------------------

bool TIconButton::IsStandardControl() const
{
	return false;
}

//----------------------------------------------------------------------------------------
// TIconButton::CreateAGAControl
//----------------------------------------------------------------------------------------

void TIconButton::CreateAGAControl()
{
	fAGAControl = TH_new CIconButton_AC(ControlQDArea(), fRsrcID, kAutoFrame, MIcon_AC::kAutoIconFamily);
}

//----------------------------------------------------------------------------------------
// TIconButton::ReadFields: 
//----------------------------------------------------------------------------------------

void TIconButton::ReadFields(CStream_AC* aStream)					// Override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fRsrcID;
	dynamic_cast_AC(CIconButton_AC*, fAGAControl.get())->SetIconIDs(fRsrcID, fRsrcID); // nb, inherited will create the control
}

//----------------------------------------------------------------------------------------
// TIconButton::WriteFields: 
//----------------------------------------------------------------------------------------

void TIconButton::WriteFields(CStream_AC* aStream) const			// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fRsrcID;
}

//========================================================================================
// CLASS TIconCheckBox
//========================================================================================

#undef Inherited
#define Inherited TCheckBox

MA_DEFINE_CLASS(TIconCheckBox);

//----------------------------------------------------------------------------------------
// TIconCheckBox constructor
//----------------------------------------------------------------------------------------

TIconCheckBox::TIconCheckBox(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255_AC& itsLabel, bool isTurnedOn, ResNumber itsRsrcID) :
	TCheckBox(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, isTurnedOn),
	fRsrcID(itsRsrcID)
{
}

//----------------------------------------------------------------------------------------
// TIconCheckBox destructor
//----------------------------------------------------------------------------------------

TIconCheckBox::~TIconCheckBox()
{
}

//----------------------------------------------------------------------------------------
// TIconCheckBox::IsStandardControl
//----------------------------------------------------------------------------------------

bool TIconCheckBox::IsStandardControl() const
{
	return false;
}

//----------------------------------------------------------------------------------------
// TIconCheckBox::CreateAGAControl
//----------------------------------------------------------------------------------------

void TIconCheckBox::CreateAGAControl()
{
	fAGAControl = TH_new CIconCheckBox_AC(ControlQDArea(), CControl_AC::kNoAutomaticState, fRsrcID, fRsrcID, kAutoFrame, MIcon_AC::kAutoIconFamily);
}

//----------------------------------------------------------------------------------------
// TIconCheckBox::ReadFields: 
//----------------------------------------------------------------------------------------

void TIconCheckBox::ReadFields(CStream_AC* aStream)					// Override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fRsrcID;
	dynamic_cast_AC(CIconCheckBox_AC*, fAGAControl.get())->SetIconIDs(fRsrcID, fRsrcID); // nb, inherited will create the control
}

//----------------------------------------------------------------------------------------
// TIconCheckBox::WriteFields: 
//----------------------------------------------------------------------------------------

void TIconCheckBox::WriteFields(CStream_AC* aStream) const			// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fRsrcID;
}

//========================================================================================
// CLASS TIconRadio
//========================================================================================

#undef Inherited
#define Inherited TRadio

MA_DEFINE_CLASS(TIconRadio);

//----------------------------------------------------------------------------------------
// TIconRadio constructor
//----------------------------------------------------------------------------------------

TIconRadio::TIconRadio(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255_AC& itsTitle, bool isTurnedOn, ResNumber itsRsrcID) :
	TRadio(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTitle, isTurnedOn),
	fRsrcID(itsRsrcID)
{
}

//----------------------------------------------------------------------------------------
// TIconRadio destructor
//----------------------------------------------------------------------------------------

TIconRadio::~TIconRadio()
{
}

//----------------------------------------------------------------------------------------
// TIconRadio::IsStandardControl
//----------------------------------------------------------------------------------------

bool TIconRadio::IsStandardControl() const
{
	return false;
}

//----------------------------------------------------------------------------------------
// TIconRadio::CreateAGAControl
//----------------------------------------------------------------------------------------

void TIconRadio::CreateAGAControl()
{
	fAGAControl = TH_new CIconRadioButton_AC(ControlQDArea(), MExclusiveObject_AC::kNoGroupID, MExclusiveObject_AC::kNoGroupID, CControl_AC::kNoAutomaticState, fRsrcID, fRsrcID, kAutoFrame, MIcon_AC::kAutoIconFamily);
}

//----------------------------------------------------------------------------------------
// TIconRadio::ReadFields: 
//----------------------------------------------------------------------------------------

void TIconRadio::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fRsrcID;
	dynamic_cast_AC(CIconRadioButton_AC*, fAGAControl.get())->SetIconIDs(fRsrcID, fRsrcID); // nb, inherited will create the control
}

//----------------------------------------------------------------------------------------
// TIconRadio::WriteFields: 
//----------------------------------------------------------------------------------------

void TIconRadio::WriteFields(CStream_AC* aStream) const			// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fRsrcID;
}

//========================================================================================
// CLASS TIconPopup
//========================================================================================

#undef Inherited
#define Inherited TPopup

MA_DEFINE_CLASS(TIconPopup);

//----------------------------------------------------------------------------------------
// TIconPopup constructor
//----------------------------------------------------------------------------------------

TIconPopup::TIconPopup(const CViewPoint& itsLocation,
								const CViewPoint& itsSize,
								SizeDeterminer itsHSizeDet,
								SizeDeterminer itsVSizeDet,
								short itsMenuID,
								short itsCurrentItem,
								short itsItemOffset,
								short itsStrListID,
								short itsIndex,
								/* Style */ short itsStyle,
								short itsJust,
								bool useAddResMenu,
								ResType useAddResMenuResType,
								const CTextStyle_AC& itsTextStyle,
								ResNumber itsRsrcID) :
	TPopup(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsMenuID, itsCurrentItem, 
			itsItemOffset, itsStrListID, itsIndex, itsStyle, itsJust, useAddResMenu, 
			useAddResMenuResType, itsTextStyle),
	fRsrcID(itsRsrcID)
{
}

//----------------------------------------------------------------------------------------
// TIconPopup destructor
//----------------------------------------------------------------------------------------

TIconPopup::~TIconPopup()
{
}

//----------------------------------------------------------------------------------------
// TIconPopup::IsStandardControl
//----------------------------------------------------------------------------------------

bool TIconPopup::IsStandardControl() const
{
	return false;
}

//----------------------------------------------------------------------------------------
// TIconPopup::CreateAGAControl
//----------------------------------------------------------------------------------------

void TIconPopup::CreateAGAControl()
{
	CTextStyle_AC	textStyle(fTextStyle);
	
	textStyle.tsFace = normal;	// the actual popup can't have a style, even though AdLib allows it to be set

	fAGAControl = TH_new CMacAppIconPopup(ControlQDArea(), this->GetMenuRef(), textStyle, CPopup_AC::kDontDispose, CPopup_AC::kReload, fRsrcID, fRsrcID, kAutoFrame, MIcon_AC::kAutoIconFamily);
	fAGAControl->SetCurrentItemNo((SInt16) this->GetLongVal(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TIconPopup::ReadFields: 
//----------------------------------------------------------------------------------------

void TIconPopup::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fRsrcID;
	dynamic_cast_AC(CIconPopup_AC*, fAGAControl.get())->SetIconIDs(fRsrcID, fRsrcID); // nb, inherited will create the control
}

//----------------------------------------------------------------------------------------
// TIconPopup::WriteFields: 
//----------------------------------------------------------------------------------------

void TIconPopup::WriteFields(CStream_AC* aStream) const			// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fRsrcID;
}

//----------------------------------------------------------------------------------------
// TPopup::LoadMenuRef: 
//----------------------------------------------------------------------------------------


MenuRef CMacAppIconPopup::LoadMenuRef(SInt16 menuID)
{
	return ::MAGetMenu(menuID);
}


#endif // qGrayscaleAppearance