//----------------------------------------------------------------------------------------
// UKeySelectionBehavior.cp 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UKEYSELECTIONBEHAVIOR__
#include "UKeySelectionBehavior.h"
#endif

// MacApp

#ifndef __UEVENT__
#include "UEvent.h"
#endif

#ifndef __UEVENTHANDLER__
#include "UEventHandler.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif


//========================================================================================
// CLASS TKeySelectionBehavior
//========================================================================================
#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TKeySelectionBehavior);

//----------------------------------------------------------------------------------------
// TKeySelectionBehavior constructor
//----------------------------------------------------------------------------------------


TKeySelectionBehavior::TKeySelectionBehavior(IDType itsIdentifier) :
	TBehavior(itsIdentifier),

	fLastKeyUpTime(0),
	fTimeOutInterval(Min_AC(::LMGetKeyThresh(), static_cast<short>(255)))
	// A note on LMGetKeyThresh: the key repeat rate typically ranges
	// from 12 ticks (short) to 40 ticks (long).  However, if key repeat
	// is turned off, LMGetKeyThresh returns SHRT_MAX.  To prevent an
	// fTimeOutInterval that high, we bound it at 255 ticks (about 4 seconds).
{
}

//----------------------------------------------------------------------------------------
// TKeySelectionBehavior destructor
//----------------------------------------------------------------------------------------


TKeySelectionBehavior::~TKeySelectionBehavior()
{
}

//----------------------------------------------------------------------------------------
// TKeySelectionBehavior::DoKeyEvent: 
//----------------------------------------------------------------------------------------


void TKeySelectionBehavior::DoKeyEvent(TToolboxEvent* event)
{
	if (event->IsOptionKeyPressed() || event->IsControlKeyPressed() || 
	   event->GetText() < chSpace || event->GetText() == chFwdDelete)
		Inherited::DoKeyEvent(event);
	else
	{
		if (event->GetWhen() - fLastKeyUpTime >= fTimeOutInterval)
			fKeySequence.Empty();
		fKeySequence += event->GetText();
		DoKeySelection(fKeySequence);
		fLastKeyUpTime = event->GetWhen();
	}
}

//----------------------------------------------------------------------------------------
// TKeySelectionBehavior::DoKeySelection: 
//----------------------------------------------------------------------------------------


void TKeySelectionBehavior::DoKeySelection(const CStr255_AC& keySequence)
{
	if (fOwner)
		fOwner->DoKeySelection(keySequence);
}

//----------------------------------------------------------------------------------------
// TKeySelectionBehavior::DoKeyUp: 
//----------------------------------------------------------------------------------------


void TKeySelectionBehavior::DoKeyUp(TToolboxEvent* event)
{
	if (event->IsOptionKeyPressed() || event->IsControlKeyPressed() || 
	   event->GetText() < chSpace || event->GetText() == chFwdDelete)
		Inherited::DoKeyUp(event);
	else
		fLastKeyUpTime = event->GetWhen();
}

//----------------------------------------------------------------------------------------
// TKeySelectionBehavior::SetTimeOutInterval: 
//----------------------------------------------------------------------------------------


void TKeySelectionBehavior::SetTimeOutInterval(long timeOutInterval)
{
	fTimeOutInterval = timeOutInterval;
}

//----------------------------------------------------------------------------------------
// End of UKeySelectionBehavior.cp


