//----------------------------------------------------------------------------------------
// UMenuView.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UMENUVIEW__
#include "UMenuView.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UEVENT__
#include "UEvent.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

// Toolbox

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI

#if qDebugMsg
#endif

//----------------------------------------------------------------------------------------
bool gTrackingInMenu;

//----------------------------------------------------------------------------------------
struct MenuRec
{
	ResNumber mID;
	TMenuView* mObject;
};


typedef MenuRec MenuArray[4000];
typedef MenuRec* MenuArrayPtr;
typedef MenuRec** MenuArrayHandle;

//----------------------------------------------------------------------------------------
static CGrafPort pMenuCPort;				// Color port for compatibility. Private
                                            // grafPort used to focus the menu w/ o messing
                                            // up the Window Manager port.
static MenuArrayHandle pMenuArray;			// Used to map a MenuRef to the TMenuView
static ResNumber pMenuArraySize;
static Handle pCustDefproc;					// Replaces the menu's menuProc field 


//========================================================================================
// GLOBAL Procedures
//========================================================================================
static long Future(long delta);
static void WaitTickChange();
static TMenuView* FindTMenuView(MenuRef theMenu);
static pascal void MenuDefproc(short message,
							   MenuRef theMenu,
							   CRect_AC& menuRect,
							   Point hitPt, // CW 5.5 thinks address is passed in if this is a CPoint_AC!
							   short& whichItem);

#undef Inherited

//----------------------------------------------------------------------------------------
// Future: Returns the TickCount some time in the future. 
//----------------------------------------------------------------------------------------
long Future(long delta)
{
	return TickCount() + delta;
}

//----------------------------------------------------------------------------------------
// WaitTickChange: 
//----------------------------------------------------------------------------------------


void WaitTickChange()
{
	long now = TickCount();
	while (TickCount() != now)
		;
}

//----------------------------------------------------------------------------------------
// FindTMenuView: 
//----------------------------------------------------------------------------------------


TMenuView* FindTMenuView(MenuRef theMenu)
{
	TMenuView* returnVal = NULL;

	MenuArrayPtr p = *pMenuArray;
	ResNumber id = (*theMenu)->menuID;

	for (ResNumber i = 0; i < pMenuArraySize; ++i)
		if (p[i].mID == id)
		{
			returnVal = p[i].mObject;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// MenuDefproc: Called by the MDEF resource. 
//----------------------------------------------------------------------------------------


pascal void MenuDefproc(short message,
						MenuRef theMenu,
						CRect_AC& menuRect,
						Point hitPt, // CW 5.5 thinks address is passed in if this is a CPoint_AC!
						short& whichItem)

{
	long OldA5 = SetCurrentA5();				// ***** Called from trap patches *****
	
	
	TMenuView * menuObj = FindTMenuView(theMenu);

	if (menuObj)
	{
		CPoint_AC fixedHitPt(hitPt);
		if (message == mPopUpMsg)
		{	// The hitPt is not a point; its coordinates are switched (see TN#172).
			fixedHitPt.h = hitPt.v;
			fixedHitPt.v = hitPt.h;
		}
	
		// Dispatch to the TMenuView object 
		menuObj->HandleDefproc(message, theMenu, menuRect, fixedHitPt, whichItem);
	}
#if qDebug
	else
		::ProgramBreak_AC("MenuDefproc called with no TMenuView object");
#endif
	
	SetA5(OldA5);
}

//----------------------------------------------------------------------------------------
// InitUMenuView: 
//----------------------------------------------------------------------------------------


void InitUMenuView()

{
	if (qNeedsColorQD || HasColorQD_AC())
		OpenCPort(&pMenuCPort);
	else
		OpenPort((GrafPtr) & pMenuCPort);
	
	pMenuArraySize = 0;
	pMenuArray = (MenuArrayHandle) NewPermHandle(0);

#if qMSWindows
	Handle h = NULL;

#elif !q68K
	RoutineDescriptorHandle h =
		(RoutineDescriptorHandle)NewPermHandle(sizeof(RoutineDescriptor));
	
	RoutineDescriptor aMDEFUPP = BUILD_ROUTINE_DESCRIPTOR(uppMenuDefProcInfo,MenuDefproc);
	
	::BlockMove(&aMDEFUPP, *h, sizeof(RoutineDescriptor)); // leave BlockMove so cache flushes
#else
	JmpInstructionTemplate** h =
		(JmpInstructionTemplate**) NewPermHandle(sizeof(JmpInstructionTemplate));
	PatchJmpInstruction(*h, reinterpret_cast<void*>(StripLong(MenuDefproc)));
#endif
	
	pCustDefproc = (Handle) h;
}


//========================================================================================
// CLASS TMenuView
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TMenuView);

//----------------------------------------------------------------------------------------
// TMenuView constructor
//----------------------------------------------------------------------------------------


TMenuView::TMenuView(ResNumber rsrcID,
					 short menuWidth,
					 short menuHeight) :
	TView(gZeroVPt, CViewPoint(menuWidth, menuHeight), sizeVariable, sizeVariable),

	fFlashInterval(-1),
	fHighlighted(false),
	fMenuRef(NULL),
	fNextFlash(0)
{
	fNextHandler = gDispatcher;				// so postcommand flows to the app 

	if (rsrcID != kNoResource)
	{
		// Read in menu and set its defproc 
		MenuRef m = MAGetMenu(rsrcID);
		::ThrowIfNULLResource_AC((Handle)m);

		MenuArrayPtr p = *pMenuArray;
		Boolean foundSlot = FALSE;
		for (ResNumber i = 0; i < pMenuArraySize; ++i) {
			if (p[i].mID == kNoResource) {
				p[i].mID = (*m)->menuID;
				p[i].mObject = this;
				foundSlot = TRUE;
			}
		}
		if (!foundSlot) {
			SetPermHandleSize((Handle) pMenuArray, sizeof(MenuRec) * (pMenuArraySize + 1));
			++pMenuArraySize;
			p = *pMenuArray;	// SetPermHandleSize may have changed this value since it is relocatable
			p[pMenuArraySize-1].mID = (*m)->menuID;
			p[pMenuArraySize-1].mObject = this;
		}

		(*m)->menuProc = pCustDefproc;
		fMenuRef = m;

		short item = 0;
		CRect_AC itsRect;
		MenuDefproc(mSizeMsg, m, itsRect, gZeroPt, item);// recompute the menu size 
	}
}

//----------------------------------------------------------------------------------------
// TMenuView destructor
//----------------------------------------------------------------------------------------


TMenuView::~TMenuView()
{
	MenuArrayPtr p = *pMenuArray;

	for (ResNumber i = 0; i < pMenuArraySize; ++i) {
		if (p[i].mObject == this) {
			p[i].mObject = NULL;
			p[i].mID = kNoResource;
		}
	}
}

//----------------------------------------------------------------------------------------
// TMenuView::FindItem: 
//----------------------------------------------------------------------------------------


short TMenuView::FindItem(CPoint_AC)
{
	return 0;
}

//----------------------------------------------------------------------------------------
// TMenuView::HandleDefproc: 
//----------------------------------------------------------------------------------------


void TMenuView::HandleDefproc(short message,
							  MenuRef theMenu,
							  CRect_AC& menuRect,
							  CPoint_AC hitPt,
							  short& whichItem)
{
	GrafPtr savePort;
	
	// Save the wmgr port && set our private port 
	GetPort(&savePort);

	SetPort((GrafPtr) & pMenuCPort);
	// Match the location and size that the menu mgr gives us 
	if ((message == mDrawMsg) || (message == mChooseMsg))
	{
		MovePortTo(menuRect.left, menuRect.top);
		PortSize(menuRect.GetLength(hSel), menuRect.GetLength(vSel));
		fLocation = menuRect[topLeft];
	}

	fShown = true;	// menu is pulled down
	InvalidateCoordinates();
	if (Focus())
	{
		GlobalToLocal(hitPt);
		GlobalToLocal(menuRect[topLeft]);
		GlobalToLocal(menuRect[botRight]);

		SetEnable((((*theMenu)->enableFlags) & 1) != 0);
		switch (message)
		{
			case mDrawMsg:
#if qDebugMsg
				if (gIntenseDebugging)
					ProgramReport_AC("mDrawMsg\n");
#endif

				HandleDrawMessage(message, theMenu, menuRect, hitPt, whichItem);
				break;
					
			case mChooseMsg:
#if qDebugMsg
				if (gIntenseDebugging)
					ProgramReport_AC("mChooseMsg\n");
#endif

				HandleChooseMessage(message, theMenu, menuRect, hitPt, whichItem);
				break;
					
			case mSizeMsg:
#if qDebugMsg
				if (gIntenseDebugging)
					ProgramReport_AC("mSizeMsg\n");
#endif

				HandleSizeMessage(message, theMenu, menuRect, hitPt, whichItem);
				break;
					
			case mPopUpMsg:
#if qDebugMsg
				if (gIntenseDebugging)
					ProgramReport_AC("mPopUpMsg\n");
#endif

				HandlePopUpMessage(message, theMenu, menuRect, hitPt, whichItem);
				break;

#if qDebugMsg
			default:
				if (gIntenseDebugging)
					ProgramReport_AC("otherwise message\n");
				break;
#endif

		}

		LocalToGlobal(menuRect[topLeft]);
		LocalToGlobal(menuRect[botRight]);

		InvalidateFocus();
	}

	fShown = false;	// menu is no longer pulled down
	SetPort(savePort);
}

//----------------------------------------------------------------------------------------
// TMenuView::HandleChooseMessage: 
//----------------------------------------------------------------------------------------


void TMenuView::HandleChooseMessage(short /* message */,
									MenuRef /* theMenu */,
									CRect_AC& /* menuRect */,
									CPoint_AC hitPt,
									short& whichItem)
{
	short newItem = kNoMenuItem;
	bool saveTrackingInMenu = gTrackingInMenu;

	// so that trackers get a chance to know that they're tracking in menus 
	gTrackingInMenu = true;

	if (IsEnabled())							// menu enabled 
	{
		// see if CPoint_AC is within hit area 
		CRect_AC hitRect = GetQDExtent() + fBorder;

		if (hitRect.Contains(hitPt))			// in menu (not border) 
		{
			// NOTE: Either your subclass of TTearOffMenu should override DoMouseCommand
            // or one of TTearOffMenu's view's subview's should override DoMouseCommand so
            // that it creates and posts a TTracker. TTearOffMenu's override of
            // PostCommand will ensure that the tracker is tracked immediately. The
            // tracker, having been posted and tracked, will then get executed next time
            // PerformCommand is called. So, when we're done with HandleMouseDown below,
            // we simply tell the menu manager that no menu item was selected, ie
            // newItem == kNoMenuItem.

			EventRecord theEventRecord;
			theEventRecord.what = mouseDown;
			theEventRecord.message = 0;
			theEventRecord.when = TickCount();
			theEventRecord.where = hitPt;
			theEventRecord.modifiers = 0;
			
			if (Button())
				theEventRecord.modifiers |= btnState;
				
			if (IsCommandKeyDown())
				theEventRecord.modifiers |= cmdKey;
				
			if (IsOptionKeyDown())
				theEventRecord.modifiers |= optionKey;
				
			CTempObjectAllocation tempObjectAllocation(true);
			TToolboxEvent event(NULL, &theEventRecord);
			TempObjectAllocation(tempObjectAllocation); // change temporary allocation setting back asap
			event.SetClickCount(gDispatcher->GetClickCount());
			event.SetAffectsMenus(false);
			
			if (!HandleMouseDown(hitPt, &event, GetStdHysteresis()))
			{
				// NOTE: for backwards compatibility with MacApp 2.0 - override your
                // TMenuView subclass' HandleMouseDown method to return false, and all
                // should work as before

				newItem = FindItem(hitPt);
				UpdateHighlight(whichItem, newItem);// Update highlighting 
			}
		}
	}
	else
		UpdateHighlight(whichItem, newItem);	// Update highlighting 

	gTrackingInMenu = saveTrackingInMenu;

	// Tell MenuManager about new item 
	whichItem = newItem;
}

//----------------------------------------------------------------------------------------
// TMenuView::HandleDrawMessage: 
//----------------------------------------------------------------------------------------


void TMenuView::HandleDrawMessage(short /* message */,
								  MenuRef /* theMenu */,
								  CRect_AC& /* menuRect */,
								  CPoint_AC /* hitPt */,
								  short& /* whichItem */)
{
	DrawContents();
	fHighlighted = false;
	if (!IsEnabled())
	{
		PenPat(&qd.gray);
		PenMode(notSrcBic);
		PaintRect(&GetQDExtent());
	}
}

//----------------------------------------------------------------------------------------
// TMenuView::HandleSizeMessage: 
//----------------------------------------------------------------------------------------


void TMenuView::HandleSizeMessage(short /* message */,
								  MenuRef theMenu,
								  CRect_AC& /* menuRect */,
								  CPoint_AC /* hitPt */,
								  short& /* whichItem */)
{
	CViewRect vr(GetFrame());
	ComputeFrame(vr);
	(*theMenu)->menuWidth = static_cast<short>(vr.GetLength(hSel));
	(*theMenu)->menuHeight = static_cast<short>(vr.GetLength(vSel));
}

//----------------------------------------------------------------------------------------
// TMenuView::HandlePopUpMessage: 
//----------------------------------------------------------------------------------------


void TMenuView::HandlePopUpMessage(short /* message */,
								   MenuRef /* theMenu */,
								   CRect_AC& menuRect,
								   CPoint_AC hitPt,
								   short& /* whichItem */)
{
	CViewRect vr(GetExtent());
	ComputeFrame(vr);
	menuRect = ViewToQD(vr) + hitPt;
}

//----------------------------------------------------------------------------------------
// TMenuView::Highlight: 
//----------------------------------------------------------------------------------------


void TMenuView::Highlight(short, bool)
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TMenuView::IsItemEnabled: 
//----------------------------------------------------------------------------------------

bool TMenuView::IsItemEnabled(short item)
{
	return (*fMenuRef)->enableFlags & (1 << item);
}

//----------------------------------------------------------------------------------------
// TMenuView::GetMenuViewColors: 
//----------------------------------------------------------------------------------------


void TMenuView::GetMenuViewColors(ResNumber theMenu,
								  short theItem,
								  MenuColors& theMenuColors)
{
	enum TypeOfMenuInfo { aMenuItem, aMenuTitle, aMenuBar, noType };


	MCEntryPtr aMCEntryPtr;
	TypeOfMenuInfo typeOfRequest;
	TypeOfMenuInfo typeOfEntryFound;
	ResNumber theEntryMenu = theMenu;
	short theEntryItem = theItem;

	if (qNeedsColorQD || HasColorQD_AC())
	{
		if (theItem)
			typeOfRequest = aMenuItem;
		else if (theMenu)
			typeOfRequest = aMenuTitle;
		else
			typeOfRequest = aMenuBar;

		aMCEntryPtr = GetMCEntry(theEntryMenu, theEntryItem);
		if (!aMCEntryPtr)				// not found, try as title 
		{
			theEntryItem = 0;
			aMCEntryPtr = GetMCEntry(theEntryMenu, theEntryItem);
			if (!aMCEntryPtr)			// not found, try as menubar 
			{
				theEntryMenu = 0;
				aMCEntryPtr = GetMCEntry(theEntryMenu, theEntryItem);
			}
		}

		if (!aMCEntryPtr)
			typeOfEntryFound = noType;
		else
		{
			if (theEntryItem)
				typeOfEntryFound = aMenuItem;
			else if (theEntryMenu)
				typeOfEntryFound = aMenuTitle;
			else
				typeOfEntryFound = aMenuBar;
		}

		switch (typeOfEntryFound)
		{
			case aMenuItem:
				theMenuColors.itemColor = (*aMCEntryPtr).mctRGB1;
				theMenuColors.backgroundColor = (*aMCEntryPtr).mctRGB4;
				theMenuColors.markColor = (*aMCEntryPtr).mctRGB1;
				theMenuColors.commandColor = (*aMCEntryPtr).mctRGB1;
				break;
					
			case aMenuTitle:
				switch (typeOfRequest)
				{
					case aMenuItem:
						{
							theMenuColors.itemColor = (*aMCEntryPtr).mctRGB3;
							theMenuColors.backgroundColor = (*aMCEntryPtr).mctRGB4;
							theMenuColors.markColor = (*aMCEntryPtr).mctRGB3;
							theMenuColors.commandColor = (*aMCEntryPtr).mctRGB3;
							break;
						}
					case aMenuTitle:
						{
							theMenuColors.itemColor = (*aMCEntryPtr).mctRGB1;
							theMenuColors.backgroundColor = (*aMCEntryPtr).mctRGB2;
							theMenuColors.markColor = (*aMCEntryPtr).mctRGB1;
							theMenuColors.commandColor = (*aMCEntryPtr).mctRGB1;
							break;
						}
				}
				break;
				
			case aMenuBar:
				switch (typeOfRequest)
				{
					case aMenuItem:
						theMenuColors.itemColor = (*aMCEntryPtr).mctRGB3;
						theMenuColors.backgroundColor = (*aMCEntryPtr).mctRGB2;
						theMenuColors.markColor = (*aMCEntryPtr).mctRGB3;
						theMenuColors.commandColor = (*aMCEntryPtr).mctRGB3;
						break;
						
					case aMenuTitle:
						theMenuColors.itemColor = (*aMCEntryPtr).mctRGB1;
						theMenuColors.backgroundColor = (*aMCEntryPtr).mctRGB4;
						theMenuColors.markColor = (*aMCEntryPtr).mctRGB1;
						theMenuColors.commandColor = (*aMCEntryPtr).mctRGB1;
						break;
						
					case aMenuBar:
						theMenuColors.itemColor = (*aMCEntryPtr).mctRGB1;
						theMenuColors.backgroundColor = (*aMCEntryPtr).mctRGB4;
						theMenuColors.markColor = (*aMCEntryPtr).mctRGB1;
						theMenuColors.commandColor = (*aMCEntryPtr).mctRGB1;
						break;

					case noType:
						break;
				}
				break;
				
			case noType:
				theMenuColors.itemColor = gRGBBlack;
				theMenuColors.backgroundColor = gRGBWhite;
				theMenuColors.markColor = gRGBBlack;
				theMenuColors.commandColor = gRGBBlack;
				break;
		}
	}
	else
	{
		theMenuColors.itemColor = gRGBBlack;
		theMenuColors.backgroundColor = gRGBWhite;
		theMenuColors.markColor = gRGBBlack;
		theMenuColors.commandColor = gRGBBlack;
	}
}

//----------------------------------------------------------------------------------------
// TMenuView::UpdateHighlight: 
//----------------------------------------------------------------------------------------


void TMenuView::UpdateHighlight(short oldItem, short newItem)
{
	// Update highlighting 
	if (newItem == oldItem)
	{
		if (fFlashInterval >= 0)
			if (TickCount() > fNextFlash)
			{
				fHighlighted =!fHighlighted;
				Highlight(oldItem, fHighlighted);
				fNextFlash = Future(fFlashInterval);
			}
	}
	else
	{
		if (fHighlighted)
			if (oldItem != kNoMenuItem)
				Highlight(oldItem, false);

		fHighlighted = (newItem != kNoMenuItem);
		if (fHighlighted)
			Highlight(newItem, true);

		if (fFlashInterval >= 0)
			fNextFlash = Future(fFlashInterval);
	}
}

//----------------------------------------------------------------------------------------
// TMenuView::Focus: 
//----------------------------------------------------------------------------------------


bool TMenuView::Focus()			// override 
{
	bool returnVal = false;

	CViewPoint vorigin;
	MenuColors theMenuColors;

	if (Inherited::Focus())
	{
		// Try to make the best match for the menu colors without requiring programmer
        // intervention. by setting the color environment to be for items.

		GetMenuViewColors((*fMenuRef)->menuID, 1, theMenuColors);
		SetIfColor(theMenuColors.itemColor);
		SetIfBkColor(theMenuColors.backgroundColor);
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMenuView::IsShown: 
//----------------------------------------------------------------------------------------


bool TMenuView::IsShown() const
{
	// Menu views do not always have superviews...
	return fSuperView ? Inherited::IsShown() : fShown;
}

//----------------------------------------------------------------------------------------
// TMenuView::IsActive: 
//----------------------------------------------------------------------------------------


bool TMenuView::IsActive() const			// override 
{
	return true;							// if we are asking then we are active
}

//----------------------------------------------------------------------------------------
// TMenuView::FocusOnSuperView: 
//----------------------------------------------------------------------------------------


bool TMenuView::FocusOnSuperView()			// override 
{
	if (!IsShown())							// don't focus if the menuView isn't pulled down
		return false;

	SetPort((GrafPtr) & pMenuCPort);
	
	CPoint_AC qdOrigin(GetQDOrigin());
	SetOrigin(qdOrigin.h, qdOrigin.v);

	ClipRect(&(qd.thePort->portRect));

	return true;
}

//----------------------------------------------------------------------------------------
// TMenuView::GetGrafPort: 
//----------------------------------------------------------------------------------------


GrafPtr TMenuView::GetGrafPort() const		// override 
{
	return reinterpret_cast<GrafPtr>(&pMenuCPort);
}

//----------------------------------------------------------------------------------------
// End of UMenuView.cp


