//----------------------------------------------------------------------------------------
// UPopup.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPOPUP__
#include "UPopup.h"
#endif

// ACS
#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

// MacApp

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __CONTROLS__
#include <Controls.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


#if qGrayscaleAppearance
//----------------------------------------------------------------------------------------
// We need to subclass the ACS Popup in order to have MacApp's GetMenu called at
// the appropriate times by that class.

class CMacAppPopup : public CPopup_AC
{
public:
	CMacAppPopup(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, MenuRef itsMenuRef = NULL, DisposalKind menuRefDisposalKind = kDontDispose, RefHandling refHandling = kNothing) :
		CPopup_AC(bounds, titleOffset, title, titleJustification, titleStyle, textStyle, adjustment, itsMenuRef, menuRefDisposalKind, refHandling) 
		{}

	CMacAppPopup(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, SInt16 itemsStringListID, SInt16 newMenuID) :
		CPopup_AC(bounds, titleOffset, title, titleJustification, titleStyle, textStyle, adjustment, itemsStringListID, newMenuID) 
		{}
		
	CMacAppPopup(const CRect_AC& bounds, SInt16 titleOffset, const CStr255_AC& title, SInt32 titleJustification, const CTextStyle_AC& titleStyle, const CTextStyle_AC& textStyle, WidthAdjust adjustment, ResType appendResourceType, SInt16 newMenuID) :
		CPopup_AC(bounds, titleOffset, title, titleJustification, titleStyle, textStyle, adjustment, appendResourceType, newMenuID) 
		{}
		
	virtual ~CMacAppPopup() 
		{}

	virtual MenuRef LoadMenuRef(SInt16 menuID);
		// overridden to call MAGetMenu
};

#endif // qGrayscaleAppearance

//----------------------------------------------------------------------------------------
const short kMAPopupRsrcID = 128;				// The rsrc id of the Popup CDEF supplied
                                                // with MacApp.

//----------------------------------------------------------------------------------------

// The following data structure is a private data structure used by the popup CDEF. It
// lives as a privHdl off the control handle's contrlData field. The first two fields of
// the private data structure are "public" but the rest are private.
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

struct PopupReallyPrivateData : public PopupPrivateData
{
	short mLastMenuID;							// last menu id selected (System 7+ only)
	short mLastItem;							// last item selected (System 7+ only)
};

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef PopupReallyPrivateData **PopupReallyPrivateDataHandle;

//========================================================================================
// Global Variables
//========================================================================================

TMenuTable TPopup::fgDynamicMenus;


//========================================================================================
// Global Functions
//========================================================================================

static CRect_AC CalcControlRect(ControlHandle control);
static short MatchingCharacters(const CStr255_AC& aString, const CStr255_AC& bString);


//----------------------------------------------------------------------------------------
// Initialization  - Prevent DeadStripping
//----------------------------------------------------------------------------------------

void InitUPopup()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TPopup, kStdPopup);
	}
}  

//----------------------------------------------------------------------------------------
// CalcControlRect: 
//----------------------------------------------------------------------------------------


// Get the rect for a control by getting its region. 

CRect_AC CalcControlRect(ControlHandle control)
{
	CRect_AC box;
	if (control)
	{
		Handle contrlDefProcHandle = (*control)->contrlDefProc;		
		if (contrlDefProcHandle)
		{
			if (!*contrlDefProcHandle)
			{
				::LoadResource(contrlDefProcHandle);
				::ThrowIfResError_AC();
			}
			CTempHandleLock_AC lock(contrlDefProcHandle);
			
			ControlDefUPP proc = reinterpret_cast<ControlDefUPP> (StripLong(*contrlDefProcHandle));
			short varCode = GetControlVariant(control);
			ControlDefProcMessage message = GetMMUMode() ? calcCntlRgn : calcCRgns;
			CTempRegion_AC region;
			CallControlDefProc(proc, varCode, control, message, long(RgnHandle(region)));
			box = (*region)->rgnBBox;
		}
	}
	else
		box.Empty();
	
	return box;
}

//----------------------------------------------------------------------------------------
// MatchingCharacters: 
//----------------------------------------------------------------------------------------


short MatchingCharacters(const CStr255_AC& aString, const CStr255_AC& bString)
{
	short matching = 0;
	
	if (CPascalStr_AC::CPascalStrCompare(bString, aString) == 0)
	{
		matching = bString.Length();
	}
	else
	{
		short maxLen = Max_AC(static_cast<short> (bString.Length()), static_cast<short> (aString.Length()));
		while (matching < maxLen)
		{
			short byteType = CharacterByteType(Ptr(&bString[0]), matching + 1, smCurrentScript);
			short charSize = (byteType == 0) ? 1 : 2;
			short newLen = matching + charSize;
			if (newLen > maxLen)
				break;
			
			if (CompareText(&bString[1], &aString[1], newLen, 
							newLen, CPascalStr_AC::fgItl2Handle) != 0)
				break;

			matching = newLen;
		}
	}

	return matching;
}


//========================================================================================
// CLASS TPopup
//========================================================================================
#undef Inherited
#define Inherited TCtlMgr


MA_DEFINE_CLASS(TPopup);

//----------------------------------------------------------------------------------------
// TPopup constructor
//----------------------------------------------------------------------------------------


TPopup::TPopup(const CViewPoint& itsLocation,
						   const CViewPoint& itsSize,
						   SizeDeterminer itsHSizeDet,
						   SizeDeterminer itsVSizeDet,
						   short itsMenuID,				// rsrc ID of 'MENU' or 'CMNU' rsrc (use kDynamicMenuID if dynamic)
						   short itsCurrentItem,
						   short itsItemOffset,			// width of the popup title area 
						   short itsStrListID,			// popup's title:	STR# rsrc id
						   short itsIndex,				//					index into STR#
						   /* Style */ short itsStyle,	// title's style 
						   short itsJust,				// title's justification 
						   bool useAddResMenu,		// true if want to AppendResMenu (can use
														// this option iff menuID != kNoResource)
						   ResType useAddResMenuResType,// used if useAddResMenu is true
						   const CTextStyle_AC& itsTextStyle) :

	TCtlMgr(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsStrListID != kNoResource ? CStr255_AC(itsStrListID, itsIndex) : (CStr255_AC)gEmptyString, itsCurrentItem,
			0, 0, 0, itsTextStyle),

	fItemOffset(itsItemOffset), 
	fTitleStyle(itsStyle),
	fTitleJust(itsJust),
	fUseAddResMenu(useAddResMenu),
	fUseAddResMenuResType(useAddResMenuResType),
	fUseAddResMenuNumberInitialItems(0),
	fStrListID(itsStrListID),						// popup's title:	STR# rsrc id
	fIndex(itsIndex),								//					index into STR#
	fMacAppSetDimState(false)
#if qGrayscaleAppearance
	, fAGAControl(NULL)
#endif
	, fMenuRef(NULL)
{
	fEventNumber = mPopupHit;

	fSizeDeterminer[hSel] = itsHSizeDet;			// needed for call to GetProcID()

	// set up the rest of the instance variables 
	fMenuID = itsMenuID;
}

//----------------------------------------------------------------------------------------
// TPopup::Free: 
//----------------------------------------------------------------------------------------


TPopup::~TPopup()
{
	if (fUseAddResMenu)
	{
		MenuRef theMenuRef = GetMenuRef();
		
		if (theMenuRef)
			for (short i = CountMItems(theMenuRef); i > fUseAddResMenuNumberInitialItems; --i)
				DeleteMenuItem(theMenuRef, i);
	}

	FreeMenu();
}

//----------------------------------------------------------------------------------------
// TPopup::GetPopupTitleStyle: 
//----------------------------------------------------------------------------------------


unsigned short TPopup::GetPopupTitleStyle()
{
	unsigned short popupTitleStyle;

	// set up justification value
	if (fTitleJust == teJustCenter)
		popupTitleStyle = popupTitleCenterJust;
	else if (fTitleJust == teJustRight)
		popupTitleStyle = popupTitleRightJust;
	else
		popupTitleStyle = popupTitleLeftJust;		// default value: note sets high-bit of lo-word

	// set up style value
	if (fTitleStyle == normal)
		popupTitleStyle += popupTitleNoStyle;
	else
	{
		if (macroIn(fTitleStyle, bold))
			popupTitleStyle += popupTitleBold;
		if (macroIn(fTitleStyle, italic))
			popupTitleStyle += popupTitleItalic;
		if (macroIn(fTitleStyle, underline))
			popupTitleStyle += popupTitleUnderline;
		if (macroIn(fTitleStyle, outline))
			popupTitleStyle += popupTitleOutline;
		if (macroIn(fTitleStyle, shadow))
			popupTitleStyle += popupTitleShadow;
		if (macroIn(fTitleStyle, condense))
			popupTitleStyle += popupTitleCondense;
		if (macroIn(fTitleStyle, extend))
			popupTitleStyle += popupTitleExtend;
	}

	return popupTitleStyle;
}

//----------------------------------------------------------------------------------------
// TPopup::GetProcID: 
//----------------------------------------------------------------------------------------


short TPopup::GetProcID()
{
	short	itsProcID = popupMenuProc;

	itsProcID += popupUseWFont;						// always use window font

	if (fUseAddResMenu)
		itsProcID += popupUseAddResMenu;			// set if popup uses AppendResMenu 

	if (fSizeDeterminer[hSel] == sizeFixed)
		itsProcID += popupFixedWidth;				// set if popup has fixed width 

	return itsProcID;
}

//----------------------------------------------------------------------------------------
// TPopup::CreateCMgrControl: 
//----------------------------------------------------------------------------------------

void TPopup::CreateCMgrControl(const CStr255_AC& itsTitle,
								long itsVal,
								long /* itsMin */,
								long /* itsMax */,
								short /* itsProcID */)// override 
{
	// We may get called at ReadFields time with the correct title, and at PostCreate time
	// with an empty title, and we want to be sure we only get called once.
	if (!fCMgrControl)
	{
		short	numInitialItemsInMenu = 0;		// used if we are using AppendResMenu
		
		// NOTE: we ignore the values passed in for itsMin, itsMax,  and itsProcID since the
		// popup CDEF defines these values differently for the NewControl call. So compute instead...

		// look 'n see if it's in the menubar
		MenuRef itsMenu = ::GetMenuHandle(GetMenuID());
		if (!itsMenu)
		{	// nope, it's not in the menubar...insert it
			itsMenu = GetMenuRef();
			if (itsMenu)
				InsertMenu(itsMenu, hierMenu);						// stick it in
			else
			{
	#if qDebug
				ProgramReport_AC("TPopup::CreateCMgrControl needs menu id for a MENU or CMNU.\n");
				::ThrowException_AC(resNotFound, 0);
	#endif
			}
		}
		
		TWindow* itsTWindow = GetWindow();
		WindowRef itsWindow = itsTWindow ? itsTWindow->GetWindowRef() : gWorkPort;
#if qGrayscaleAppearance
		if (IsEmulatingControls())
		{		
			if (!GetAGAControl())
			{
				// Fix up the current item for CreateAGAControl
				SetLongValues(itsVal, 1, GetNumberOfItems(), kDontRedraw);
				
				CreateAGAControl();
				
				if (GetAGAControl())
					GetAGAControl()->SetTitle(itsTitle, kDontRedraw);
			}
		}
		else
#endif
		{
			if (fUseAddResMenu)
				numInitialItemsInMenu = GetNumberOfItems();

			// set the port appropriately
			GrafPtr savePort;
			GetPort(&savePort);
			SetPortWindowPort(itsWindow);
	
			// Set up the port's text style -- NewControl needs this.
			SetPortTextStyle(fTextStyle);
			PenNormal();
			
			// create the control
			ControlRef aCMgrControl = NewControl(itsWindow, &ControlQDArea(), itsTitle, false,
												 static_cast<short>(GetPopupTitleStyle()), fMenuID, fItemOffset, 
												 GetProcID(), fUseAddResMenuResType);
			SetPort(savePort);								// restore port before signalling failure
			::ThrowIfNULL_AC(aCMgrControl);
	
			// Need to signal failure if the CDEF's private storage couldn't be created -- this is
			// most likely due to a missing MENU or CMNU resource with fMenuID.
			::ThrowIfNULL_AC((*aCMgrControl)->contrlData);
	
			SetControlReference(aCMgrControl, reinterpret_cast<long>(this));	// Set refCon here, it has a special value at
															// NewControl time - the ResType for the case
															// in which the popup's variation code has
															// popupUseAddResMenu.
	
			fCMgrControl = aCMgrControl;

			if (fUseAddResMenu)
				fUseAddResMenuNumberInitialItems = numInitialItemsInMenu;
		}
		DimState(fDimmed, kDontRedraw);
		
		// Remember, the control was created with "magic" values, fix it up.
		SetLongValues(itsVal, 1, GetNumberOfItems(), kDontRedraw);

		SetCMgrVisibility(itsWindow != gWorkPort);
		


		fTitle.reset(); // whatever stored title there was has been used so we don't need it here anymore
	}
}

//----------------------------------------------------------------------------------------
// TPopup::GetStandardSignature: 
//----------------------------------------------------------------------------------------

IDType TPopup::GetStandardSignature() const	// override 
{
	return kStdPopup;
}

//----------------------------------------------------------------------------------------
// TPopup::ReadFields: 
//----------------------------------------------------------------------------------------

void TPopup::ReadFields(CStream_AC* aStream)				// override 
{
	Inherited::ReadFields(aStream);
	
	fMenuID = aStream->ReadInteger();
	short itsCurrentItem = aStream->ReadInteger();
	fItemOffset = aStream->ReadInteger();

	fTitleStyle = (Style)aStream->ReadByte();
	fTitleJust = aStream->ReadInteger();
	fUseAddResMenu = aStream->ReadBoolean();
	fUseAddResMenuResType = aStream->ReadLong();
	fStrListID = aStream->ReadInteger();
	fIndex = aStream->ReadInteger();

	// initialize the popup 
	CreateCMgrControl(GetMenuLabel(), itsCurrentItem, 0, 0, 0);
}

//----------------------------------------------------------------------------------------
// TPopup::WriteFields: 
//----------------------------------------------------------------------------------------

void TPopup::WriteFields(CStream_AC* aStream) const		// override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteInteger(GetMenuID());
	aStream->WriteInteger(GetCurrentItem());
	aStream->WriteInteger(fItemOffset);

	aStream->WriteByte(fTitleStyle);
	aStream->WriteInteger(fTitleJust);
	aStream->WriteBoolean(fUseAddResMenu);
	aStream->WriteLong(fUseAddResMenuResType);
	aStream->WriteInteger(fStrListID);
	aStream->WriteInteger(fIndex);
}

//----------------------------------------------------------------------------------------
// TView::Focus: 
//----------------------------------------------------------------------------------------

bool TPopup::Focus()
{
	// To Focus, must be able to focus, and be drawable, so as to avoid a bug in the
	// CDEF/MDEF in which a bus error results when drawing if the popup isn't
	// drawable (e.g. if it's off-screen).
	return Inherited::Focus() && !EmptyRgn(GetClipRegion(qd.thePort));
}

//----------------------------------------------------------------------------------------
// TPopup::Draw: 
//----------------------------------------------------------------------------------------

void TPopup::Draw(const CViewRect& area)		// override 
{
	// may need to set the lo memory globals for the font/size since the CDEF will
	// occasionally ignore the values

	if (!GetNumberOfItems())
	{
		fMacAppSetDimState = true;
		DimState(true, kDontRedraw); //no items, Dim Popup
	}
	else if (fDimmed && fMacAppSetDimState)
	{
		fMacAppSetDimState = false;
 		DimState(false,kDontRedraw); 
 	}
		
	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TPopup::CalcMinFrame: 
//----------------------------------------------------------------------------------------

CViewRect TPopup::CalcMinFrame() // override
// Compute the minimum size of this view. 
{
	CViewRect minFrame = Inherited::CalcMinFrame();

	if (fCMgrControl)
	{
		CViewPoint controlSize = CalcControlRect(fCMgrControl).GetSize();
		minFrame[botRight] = minFrame[topLeft] + controlSize;
	}
	else
	{
		short theFontHeight;
		FontInfo theFontInfo;
		GetTextStyleFontInfo(fTextStyle, theFontInfo, theFontHeight);
		minFrame.bottom = minFrame.top + theFontHeight + 3;
		if (fSuperView)
			minFrame.right = minFrame.left + fSuperView->GetSize().h;
	}

	return minFrame;
}


//----------------------------------------------------------------------------------------
// TPopup::DoMouseCommand: 
//----------------------------------------------------------------------------------------

void TPopup::DoMouseCommand(CViewPoint& theMouse,
								   TToolboxEvent* event,
								   CPoint_AC hysteresis)// override 
{
	if (!Focus())							// set up the font/size
		SetPortTextStyle(fTextStyle);

	Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// TPopup::DoKeyEvent: 
//----------------------------------------------------------------------------------------

void TPopup::DoKeyEvent(TToolboxEvent* event) // override
{
	bool wasHandled = false;

	if (IsEnabled())
	{
		wasHandled = true;
		short newItem = GetCurrentItem();
		if (event->GetText() == chHome)
			newItem = 1;
		else if (event->GetText() == chEnd)
			newItem = GetNumberOfItems();
		else if (event->GetText() == chUp)
			--newItem;
		else if (event->GetText() == chDown)
			++newItem;
		else
			wasHandled = false;

		if (wasHandled)
			SetCurrentItem(newItem, kRedraw);
	}

	if (!wasHandled)
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TPopup::DoKeySelection: 
//----------------------------------------------------------------------------------------

void TPopup::DoKeySelection(const CStr255_AC& selectionString) // override
{
	CStr255_AC ucSelection(selectionString);
	UppercaseText(Ptr(&ucSelection[1]), ucSelection.Length(), smCurrentScript);

	short matchItem = 0;
	short matchLength = 0;
	
	short n = GetNumberOfItems();
	for (short item = 1; item <= n; item++)
	{
		CStr255_AC theText(GetItemText(item));
		UppercaseText(Ptr(&theText[1]), theText.Length(), smCurrentScript);
		short newLength = MatchingCharacters(theText, ucSelection);
		if (matchLength < newLength)
		{
			matchItem = item;
			matchLength = newLength;
			if (matchLength == ucSelection.Length())
				break;
		}
	}
	
	if (matchItem)
		SetCurrentItem(matchItem, kRedraw);
}

//----------------------------------------------------------------------------------------
// TPopup::AttachMenuRef: 
//----------------------------------------------------------------------------------------

void TPopup::AttachMenuRef(MenuRef itsMenuRef
#if qGrayscaleAppearance
	, CPopup_AC::DisposalKind disposalKind, 
	  CPopup_AC::RefHandling refHandling
#endif
	)
{
	// first, a quick safety check to make sure that we do nothing if we are trying to
    // install a menu that is already installed
	if (fMenuRef != itsMenuRef)
	{
		if (fMenuRef)					// first, release the old MenuRef 
			FreeMenu();

		// set the menu handle in  the popup's private storage 
		if (itsMenuRef)
		{
			fMenuID = (*itsMenuRef)->menuID;
			CreateCMgrControl(GetMenuLabel(), 1 , 0, 0, 0);

			// set up the control's needed values 
			SetLongValues(1, 1, GetNumberOfItems(), kDontRedraw);
		}
	}	
	
#if qGrayscaleAppearance
	if (fAGAControl)
		fAGAControl->SetMenuRef(itsMenuRef, disposalKind, refHandling);
#endif
}

//----------------------------------------------------------------------------------------
// TPopup::GetMenuRef: 
//----------------------------------------------------------------------------------------

MenuRef TPopup::GetMenuRef()
{
	if (!fMenuRef)
	{
		if (GetMenuID())
			fMenuRef = MAGetMenu(GetMenuID());
		else
			CreateMenu();
	}
	else
		ReinsertMenu();
	
	return fMenuRef;
}

//----------------------------------------------------------------------------------------
// Dynamic popup methods
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TPopup::CreateMenu: 
//----------------------------------------------------------------------------------------

void TPopup::CreateMenu()
{
	if (!fMenuRef && !fMenuID)
	{
		fMenuID = UniqueMenuID();

		// Get the menu label without calling GetMenuLabel, because
		// we haven't created the menu yet and that will make it recurse. 
		CStr255_AC	itsLabel;
		if (fStrListID != kNoResource)
			::GetIndString(itsLabel, fStrListID, fIndex);
		else
			itsLabel.Empty();

		fMenuRef = ::NewMenu(fMenuID, itsLabel);
		::ThrowIfNULL_AC(fMenuRef);

#if qGrayscaleAppearance
		if (fAGAControl)
		{
			fAGAControl->SetMenuRef(fMenuRef, CPopup_AC::kDontDispose, CPopup_AC::kReload);
			// set up the control's needed values 
			SetLongValues(1, 1, GetNumberOfItems(), kDontRedraw);
		}
#endif

		AddDynamicMenu(fMenuRef);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::ReinsertMenu: 
//----------------------------------------------------------------------------------------

void TPopup::ReinsertMenu()
{
	if (fMenuRef && fMenuID)
	{
		// If we've been kicked off the menu list, get back on it
		if (!::GetMenuHandle(fMenuID))
			::InsertMenu(fMenuRef, hierMenu);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::FreeMenu: 
//----------------------------------------------------------------------------------------

void TPopup::FreeMenu()
// Dispose of fMenuRef.
{
	if (fMenuRef)
	{
		// There are 3 cases that we need to account for here:
		//	1. if fMenuRef is a converted 'CMNU' resource	=> leave MenuRef alone;
		//	2. if fMenuRef is a 'MENU' resource	 			=> call ReleaseResource;
		//	3. if fMenuRef is a menu created with NewMenu	=> call DisposeMenu.
		// N.B.: you should never call GetMenu to create the MenuRef, instead call MAGetMenu.
		if (IsManagedMenu(fMenuRef))
		{
			// case 1
			// do nothing
		}
		else
		{
			// Make sure the menu is out of the menu bar before we nuke it
			if (::GetMenuHandle((**fMenuRef).menuID))
				::DeleteMenu((**fMenuRef).menuID);

			// Remove it from the list of dynamic menus
			RemoveDynamicMenu(fMenuRef);
	
			if (::IsAResource_AC(reinterpret_cast<Handle> (fMenuRef)))
			{
				// case 2 
				::ReleaseResource(reinterpret_cast<Handle> (fMenuRef));
			}
			else
			{
				// case 3 
				::DisposeMenu(fMenuRef);
			}
		}

		fMenuRef = NULL;
		if (fCMgrControl)
		{
			SetCMgrVisibility(FALSE);
			::SizeControl(fCMgrControl, 0, 0);
			::DisposeControl(fCMgrControl);
			fCMgrControl = NULL;
		}
	}
}

//----------------------------------------------------------------------------------------
// TPopup::GetMenuLabel: 
//----------------------------------------------------------------------------------------

CStr255_AC TPopup::GetMenuLabel()
{
	CStr255_AC returnVal;

	if (fStrListID != kNoResource)					// retrieve the menu label from the resource
		GetIndString(returnVal, fStrListID, fIndex);
	else
		returnVal = GetItemText(0);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TPopup::GetCurrentCommand: 
//----------------------------------------------------------------------------------------

CommandNumber TPopup::GetCurrentCommand()
{
	return CommandFromMenuItem(GetLastMenuID(), GetLastItem());
}

//----------------------------------------------------------------------------------------
// TPopup::GetCurrentItem: 
//----------------------------------------------------------------------------------------

short TPopup::GetCurrentItem() const
{
#if qGrayscaleAppearance
	if (fAGAControl)
		return fAGAControl->GetCurrentItemNo();
	else
#endif
	return GetVal();
}

//----------------------------------------------------------------------------------------
// TPopup::GetLastItem: 
//----------------------------------------------------------------------------------------

short TPopup::GetLastItem()
{
	// the following code is only supported for the System 7 CDEF
	short lastItem = 0;

	if (fCMgrControl && (*fCMgrControl)->contrlData)
		lastItem = (*reinterpret_cast<PopupReallyPrivateDataHandle>((*fCMgrControl)->contrlData))->mLastItem;
	
	return lastItem ? lastItem : GetCurrentItem();
}

//----------------------------------------------------------------------------------------
// TPopup::GetLastMenuID: 
//----------------------------------------------------------------------------------------

short TPopup::GetLastMenuID()
{
	// the following code is only supported for the System 7 CDEF
	short lastMenuID = 0;

	if (fCMgrControl && (*fCMgrControl)->contrlData)
		lastMenuID = (*reinterpret_cast<PopupReallyPrivateDataHandle>((*fCMgrControl)->contrlData))->mLastMenuID;

	return lastMenuID ? lastMenuID : GetMenuID();
}

//----------------------------------------------------------------------------------------
// TPopup::GetItemText: 
//----------------------------------------------------------------------------------------

CStr255_AC TPopup::GetItemText(short item)
{
	CStr255_AC returnVal;

	MenuRef itsMenuRef = GetMenuRef();

	if (itsMenuRef)
	{
		if (item)
			GetMenuItemText(itsMenuRef, item, returnVal);
		else
			// Toolbox has no interface for retrieving the title of a menu!!
			returnVal = (*static_cast<MenuHandle>(itsMenuRef))->menuData;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TPopup::GetNumberOfItems: 
//----------------------------------------------------------------------------------------

short TPopup::GetNumberOfItems()
{
	// if possible, query the menu, else query the control
	MenuRef itsMenuRef = GetMenuRef();

	return itsMenuRef ? CountMItems(itsMenuRef) : GetMax();
}

//----------------------------------------------------------------------------------------
// TPopup::GetTitle: 
//----------------------------------------------------------------------------------------

CStr255_AC TPopup::GetTitle() const
{
	return GetText();
}

//----------------------------------------------------------------------------------------
// TPopup::SetCurrentItem: 
//----------------------------------------------------------------------------------------

void TPopup::SetCurrentItem(short item, bool redraw)
{
	SetLongVal(item, redraw);
	
#if qGrayscaleAppearance
	if (fAGAControl)
		fAGAControl->SetCurrentItemNo((SInt16) item, IsEmulatingControls() && redraw && Focus() && IsVisible());
#endif
}

//----------------------------------------------------------------------------------------
// Convenience methods
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TPopup::SetItemText: 
//----------------------------------------------------------------------------------------

void TPopup::SetMenuItemText(short item, const CStr255_AC& theText)
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
		::SetMenuItemText(itsMenuRef, item, theText);
}

//----------------------------------------------------------------------------------------
// TPopup::AddItem: 
//----------------------------------------------------------------------------------------

void TPopup::AddItem(const CStr255_AC& theText)
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
	{
		short n = ::CountMItems(itsMenuRef);
		::InsertMenuItem(itsMenuRef, "\p ", n);
		::SetMenuItemText(itsMenuRef, n + 1, theText);
		
		SetLongMax(n + 1, kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::InsertItem: 
//----------------------------------------------------------------------------------------

void TPopup::InsertItem(short item, const CStr255_AC& theText)
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
	{
		::InsertMenuItem(itsMenuRef, "\p ", item - 1);
		::SetMenuItemText(itsMenuRef, item, theText);
		
		SetLongMax(::CountMItems(itsMenuRef), kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::DeleteAll: 
//----------------------------------------------------------------------------------------

void TPopup::DeleteAll()
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
	{
		short n = ::CountMItems(itsMenuRef);
		while (n > 0)
			::DeleteMenuItem(itsMenuRef, n--);
		
		SetLongMax(0, kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::DeleteItem: 
//----------------------------------------------------------------------------------------

void TPopup::DeleteItem(short item)
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
	{
		::DeleteMenuItem(itsMenuRef, item);
		SetLongMax(::CountMItems(itsMenuRef), kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::SetItemStyle: 
//----------------------------------------------------------------------------------------

void TPopup::SetItemStyle(short item, short chStyle)
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
		::SetItemStyle(itsMenuRef, item, chStyle);
}

//----------------------------------------------------------------------------------------
// TPopup::EnableItem: 
//----------------------------------------------------------------------------------------

void TPopup::EnableItem(short item, bool canDo)
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
	{
		if (canDo)
			::EnableItem(itsMenuRef, item);
		else
			::DisableItem(itsMenuRef, item);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::EnableCheckItem: 
//----------------------------------------------------------------------------------------

void TPopup::EnableCheckItem(short item, bool canDo, bool checked)
{
	MenuRef itsMenuRef = GetMenuRef();
	if (itsMenuRef != NULL)
	{
		if (canDo)
			::EnableItem(itsMenuRef, item);
		else
			::DisableItem(itsMenuRef, item);

		::CheckItem(itsMenuRef, item, checked);
	}
}

//----------------------------------------------------------------------------------------
// TPopup::SetPopup: 
//----------------------------------------------------------------------------------------

void TPopup::SetPopup(MenuRef itsMenuRef,
					  short itsCurrentItem,
					  bool redraw)
{
	AttachMenuRef(itsMenuRef);
	SetCurrentItem(itsCurrentItem, redraw);
	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TPopup::SetTitle: 
//----------------------------------------------------------------------------------------

void TPopup::SetTitle(const CStr255_AC& title)
{
	SetText(title, kDontRedraw);
}

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TPopup::SetTitle: 
//----------------------------------------------------------------------------------------

void TPopup::SetText(const CStr255_AC& title, bool redraw)
{
	// Install the new popup title (label).

	Inherited::SetText(title, redraw);

	if (fAGAControl)
	{
		fAGAControl->SetTitle(title, IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
}

//----------------------------------------------------------------------------------------
// TPopup::IsEmulatingControls: 
//----------------------------------------------------------------------------------------

bool TPopup::IsEmulatingControls() const
{
	return Inherited::IsEmulatingControls() && !fUseAddResMenu;
}

//----------------------------------------------------------------------------------------
// TPopup::CreateAGAControl: 
//----------------------------------------------------------------------------------------

void TPopup::CreateAGAControl()
{
	CPopup_AC::WidthAdjust	widthAdjust;

	CTextStyle_AC	titleStyle(fTextStyle);
	CTextStyle_AC	textStyle(fTextStyle);
	
	titleStyle.tsFace = fTitleStyle;
	textStyle.tsFace = normal;	// the actual popup can't have a style, even though AdLib allows it to be set

	widthAdjust = (fSizeDeterminer[hSel] == sizeFixed) ? CPopup_AC::kFixedWidth : CPopup_AC::kSystemMDEFAdjustment;

	fAGAControl = TH_new CMacAppPopup(GetQDExtent(), fItemOffset, GetMenuLabel(), fTitleJust, 
			titleStyle, textStyle, widthAdjust, this->GetMenuRef(), CPopup_AC::kDontDispose, CPopup_AC::kNothing);
	::ThrowIfNULL_AC(fAGAControl);

	fAGAControl->SetCurrentItemNo((SInt16) this->GetLongVal(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TPopup::LoadMenuRef: 
//----------------------------------------------------------------------------------------

MenuRef CMacAppPopup::LoadMenuRef(SInt16 menuID)
{
	return ::MAGetMenu(menuID);
}

#endif // qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// Static global functions
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TPopup::AddDynamicMenu: 
//----------------------------------------------------------------------------------------

void TPopup::AddDynamicMenu(MenuRef itsMenuRef)
{
	fgDynamicMenus.Insert(reinterpret_cast<Handle> (itsMenuRef));
}

//----------------------------------------------------------------------------------------
// TPopup::RemoveDynamicMenu: 
//----------------------------------------------------------------------------------------

void TPopup::RemoveDynamicMenu(MenuRef itsMenuRef)
{
	fgDynamicMenus.Delete(reinterpret_cast<Handle> (itsMenuRef));
}

//----------------------------------------------------------------------------------------
// TPopup::GetDynamicMenuRef: 
//----------------------------------------------------------------------------------------

MenuRef TPopup::GetDynamicMenuRef(MenuID menuID)
{
	MenuRef m = NULL;

	// First see if its a dynamic menu
	m = fgDynamicMenus.GetMenu(menuID);

	// Now try the regular menus
	if (!m)
		m = MAGetMenuRef(menuID);

	return m;
}

//----------------------------------------------------------------------------------------
// TPopup::UniqueMenuID: 
//----------------------------------------------------------------------------------------

short TPopup::UniqueMenuID()
{
	// Brute force search, assuming there aren't a lot of dynamic menus. 

	// Start after mLastMenu so MacApp doesn't disable the menu
	short nextMenuID = mLastMenu + 1;

	// Make sure it isn't an ID for a menu the Toolbox or MacApp knows about

	// There's a bug in TSortedHandleList::DoSearch where it returns the last
	// handle it saw instead of NULL. We can work around the bug because we
	// should never get the same handle for different id's. 
	MenuRef prevMenuRef = NULL;
	MenuRef m;
	while (((m = GetDynamicMenuRef(nextMenuID)) != NULL) && (m != prevMenuRef))
	{
		prevMenuRef = m;
		nextMenuID++;
	}

	// Assign the ID... Good luck!
	return nextMenuID;
}

//----------------------------------------------------------------------------------------
// End of UPopup.cp


