//----------------------------------------------------------------------------------------
// UPrinting.cp
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPRINTING__
#include "UPrinting.h"
#endif

// ACS

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UFILESTREAM__
#include "UFileStream.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#if qGXPrinting
#	ifndef __GXENVIRONMENT__
#	include <GXEnvironment.h>
#	endif

#	ifndef __GXGRAPHICS__
#	include <GXGraphics.h>
#	endif

#	ifndef __GXPRINTING__
#	include <GXPrinting.h>
#	endif
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#if !qGXViews
#	ifndef __PRINTING__
#	include <Printing.h>
#	endif
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif


#ifndef __STDLIB__
#include <stdlib.h>
#endif

#ifndef __STRING__
#include <string.h>
#endif

// Handy macros for Fixed point manipulation
#ifndef __GXMATH__
#	ifndef FixedToInt
#	define FixedToInt(a) (static_cast<short>((Fixed)(a) + fixed1/2 >> 16))
#	endif

#	ifndef IntToFixed
#	define IntToFixed(a) ((Fixed)(a) << 16)
#	endif
#endif


//========================================================================================
// constants
//========================================================================================
enum {
	kClassicQDResolution = 72
};

//========================================================================================
// GLOBAL Variables
//========================================================================================

PenState gBreaksPenState = { {0, 0}, {2, 2}, {patCopy}, {0xCC, 0x66, 0x33, 0x99, 0xCC, 0x66, 0x33, 0x99}};
bool gCancelAllPrinting;
CViewRect gStdPageMargins(1 * kClassicQDResolution, 1 * kClassicQDResolution, - 1 * kClassicQDResolution, - 1 * kClassicQDResolution);// 1" margins std default;
TPrintInfo* gFinderPrintInfo;
TStdPrintHandler* gJobPrintHandler;

static PrIdleUPP pPrintIdleProc;	// For MacPrint
CAutoPtr_AC<TPrintInfo> gDefaultPrintInfo;
CFSSpec_AC gTargetPrinter;

#if qGXPrinting
static GXPrintingEventUPP pGXPrintingEventUPP; // For GX
static gxShapeSpoolUPP pgxShapeSpoolUPP; // For GX
static gxPrintingFlattenUPP pgxPrintingFlattenUPP; // For GX
static gxPrintingFlattenUPP pgxPrintingUnflattenUPP; // For GX
static gxPrintingFlattenUPP pgxGetSizeHelperUPP; // For GX
bool gUsingGXPrinting;
#endif

#if !qNeedsGXViews
short TMacPrintJob::fPrintShopOpens;
TPPrPort TMacPrintJob::fQDPort;
CStr255_AC TMacPrintJob::fgLastPrinterName;
unsigned long TMacPrintJob::fgPrinterChangeLevel;
#endif

const short kFirstGXFormatIndex = 2;
const short kFirstMacPrintFormatIndex = 1;
const ResType kPrintResType[kFirstGXFormatIndex + 1] = { kAlternatePrintInfoRsrcType, kPrintInfoRsrcType, kGXPrintInfoRsrcType };
const ResNumber kPrintResNumber[kFirstGXFormatIndex + 1] = { kAlternatePrintInfoRsrcID, kPrintInfoRsrcID, kGXPrintInfoRsrcID };

//========================================================================================
// GLOBAL Procedures
//========================================================================================

#undef Inherited

//----------------------------------------------------------------------------------------
// InitUPrinting: 
//----------------------------------------------------------------------------------------

void InitUPrinting()
{
	static bool UPrintingInitialized;
	if (!UPrintingInitialized)
	{
		TPrintHandler::gCouldPrint = true;
	
#if qGXPrinting
		if (TView::gGXGraphicsClient && HasQDGXPrinting_AC())
			gUsingGXPrinting = ::GXInitPrinting() == noErr;
	
		if (gUsingGXPrinting)
			gDefaultPrintInfo = TH_new TGXJob;
		else
#endif
			gDefaultPrintInfo = TH_new TMacPrintJob;
	
		// Install a StdPrintHandler in global variable gPrintHandler
		TStdPrintHandler* aTStdPrintHandler = TH_new TStdPrintHandler;
		aTStdPrintHandler->fFinderJobDialog = true;
		TPrintHandler::gPrintHandler = aTStdPrintHandler;

		UPrintingInitialized = true;
	}
}

//----------------------------------------------------------------------------------------
// NewTPrintInfo: 
//----------------------------------------------------------------------------------------

TPrintInfo* NewTPrintInfo(bool isValid)
{
	TPrintInfo * returnVal = NULL;

#if qGXPrinting
	if (gUsingGXPrinting)
		returnVal = TH_new TGXJob(NULL, isValid);
	else
#endif
		returnVal = TH_new TMacPrintJob(NULL, isValid);

	return returnVal;
}

//========================================================================================
// CLASS TPrintInfo
//========================================================================================
#undef Inherited

MA_DEFINE_CLASS(TPrintInfo);

//----------------------------------------------------------------------------------------
// TPrintInfo constructor
//----------------------------------------------------------------------------------------

TPrintInfo::TPrintInfo() :
	fIsValid(false),
	fNeedsDefault(false)
{
	if (gDefaultPrintInfo && gDefaultPrintInfo != this)
		gDefaultPrintInfo->AddDependent(this);
}

//----------------------------------------------------------------------------------------
// TPrintInfo destructor
//----------------------------------------------------------------------------------------

TPrintInfo::~TPrintInfo()
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::ReadFrom: 
//----------------------------------------------------------------------------------------

void TPrintInfo::ReadFrom(CStream_AC* aStream)
{
	MStreamable::ReadFrom(aStream);

	fNeedsDefault = false;

	Invalidate();
}

//----------------------------------------------------------------------------------------
// TPrintInfo::SetFrom
//----------------------------------------------------------------------------------------

void TPrintInfo::SetFrom(TPrintInfo* jobToCopy)
{
	fIsValid = jobToCopy->IsValid();
	fNeedsDefault = jobToCopy->NeedsDefault();
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetSize
//----------------------------------------------------------------------------------------

long TPrintInfo::GetSize(bool /* asResource */) const
{
	long returnVal = 0;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TPrintInfo copy constructor
//----------------------------------------------------------------------------------------

TPrintInfo::TPrintInfo(const TPrintInfo& inPrintInfo) :
	MDependable_AC(inPrintInfo),
	MStreamable(inPrintInfo),

	fIsValid(inPrintInfo.fIsValid),
	fNeedsDefault(inPrintInfo.fIsValid)
{

	if (gDefaultPrintInfo)
		gDefaultPrintInfo->AddDependent(this);
}

//----------------------------------------------------------------------------------------
// TPrintInfo::DoUpdate:
//----------------------------------------------------------------------------------------

void TPrintInfo::DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject,
						  void* changeData,
						  CDependencySpace_AC* dependencySpace)// Override
{
	switch (theChange)
	{
		case mPrinterChanged:
			Invalidate();
			break;

		case mPrinterOpened:
			Update();						// Pool's open!
			break;

		default:
			MDependable_AC::DoUpdate(theChange, changedObject, changeData, dependencySpace);
	}
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetPaperRect:
//----------------------------------------------------------------------------------------

CViewRect TPrintInfo::GetPaperRect()
{
	return gZeroVRect;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetInkRect:
//----------------------------------------------------------------------------------------

CViewRect TPrintInfo::GetInkRect()
{
	return gZeroVRect;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetDeviceRes:
//----------------------------------------------------------------------------------------

CPoint_AC TPrintInfo::GetDeviceRes()
{
	return CPoint_AC(kClassicQDResolution, kClassicQDResolution);
}

//----------------------------------------------------------------------------------------
// TPrintInfo::Invalidate:
//----------------------------------------------------------------------------------------

void TPrintInfo::Invalidate()
{
	fIsValid = false;
	Changed(mPrintInfoChanged, this);
}

//----------------------------------------------------------------------------------------
// TPrintInfo::SetToDefault:
//----------------------------------------------------------------------------------------

void TPrintInfo::SetToDefault()
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::IsSetToDefault:
//----------------------------------------------------------------------------------------

bool TPrintInfo::IsSetToDefault()
{
	return false;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetPrintPages:
//----------------------------------------------------------------------------------------

void TPrintInfo::GetPrintPages(long& fromPage,
							   long& toPage,
							   long& minFromPage,
							   long& maxToPage)
{
	fromPage = toPage = minFromPage = maxToPage = 0;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::SetPrintPages:
//----------------------------------------------------------------------------------------

void TPrintInfo::SetPrintPages(long /* fromPage */,
							   long /* toPage */,
							   long /* minFromPage */,
							   long /* maxToPage */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetNumCopies:
//----------------------------------------------------------------------------------------

long TPrintInfo::GetPrintCopies()
{
	return 1;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::SetNumCopies:
//----------------------------------------------------------------------------------------

void TPrintInfo::SetPrintCopies(long /* numCopies */)
{
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetTargetPrinter:
//----------------------------------------------------------------------------------------

void TPrintInfo::SetTargetPrinter(const CFSSpec_AC& /* targetPrinter */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::SetSpoolFile:
//----------------------------------------------------------------------------------------

void TPrintInfo::SetSpoolFile(CFSSpec_AC& /* spoolFile */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::IsSpooling:
//----------------------------------------------------------------------------------------

bool TPrintInfo::IsSpooling()
{
	return true; // default is spooled printing (normal for the Macintosh)
}

//----------------------------------------------------------------------------------------
// TPrintInfo::IsMatch
//----------------------------------------------------------------------------------------

bool TPrintInfo::IsMatch(TPrintInfo* /* thePrintInfo */)
{
	return false;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::Update:
//----------------------------------------------------------------------------------------

void TPrintInfo::Update()
{
	if (!fIsValid && TPrintHandler::gCouldPrint)
	{
		try
		{
			bool wantedDefault = fNeedsDefault;
			fNeedsDefault = false;
			TPrintInfo * defaultPrintInfo = GetDefaultPrintInfo();
			if (wantedDefault && IsSetToDefault())
				fIsValid = true;				// matches the default. No validation necessary!
			else if (wantedDefault && (defaultPrintInfo != this) && defaultPrintInfo->IsValid())
				SetFrom(defaultPrintInfo);
			else								// need to go ask Printing Management
			{
				if (wantedDefault)
					CallPrintDefault();
				else
					CallPrValidate();

				fIsValid = true;
			}
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			if (theException.GetError() != errNoPrintDrvr)
				throw;
		}
		
	}
}

//----------------------------------------------------------------------------------------
// TPrintInfo::Open: 
//----------------------------------------------------------------------------------------

void TPrintInfo::Open(void* /* byWhom */ )
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::IsOpen: 
//----------------------------------------------------------------------------------------

bool TPrintInfo::IsOpen()
{
	return true;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::Close: 
//----------------------------------------------------------------------------------------

void TPrintInfo::Close(void* /* byWhom */ )
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetError: 
//----------------------------------------------------------------------------------------

OSErr TPrintInfo::GetError()
{
	return noErr;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::SetError: 
//----------------------------------------------------------------------------------------

void TPrintInfo::SetError(OSErr /* err */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::ClearError: 
//----------------------------------------------------------------------------------------

void TPrintInfo::ClearError()
{
	SetError(noErr);
}

//----------------------------------------------------------------------------------------
// TPrintInfo::ChkPrintErr: 
//----------------------------------------------------------------------------------------

bool TPrintInfo::ChkPrintErr(OSErr& err, bool& ranOutOfSpace)
{
	bool returnVal = true;					// proceed
	err = GetError();
	if (err != noErr)
	{
#if qDebugMsg
		if (gDebugPrinting)
			ProgramReport_AC(false, "Error from PrError is %d\n", err);
#endif

		returnVal = false;
		if (err == iPrSavPFil || err == dskFulErr)
			ranOutOfSpace = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::StartJob: 
//----------------------------------------------------------------------------------------

void TPrintInfo::StartJob(const CStr255_AC& /* docTitle */ , long /* numPages */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetJobPort: 
//----------------------------------------------------------------------------------------

GrafPtr TPrintInfo::GetJobPort()
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::FinishJob: 
//----------------------------------------------------------------------------------------


void TPrintInfo::FinishJob()
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::StartPage: 
//----------------------------------------------------------------------------------------


void TPrintInfo::StartPage(long					/* pageNumber */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::FinishPage: 
//----------------------------------------------------------------------------------------


void TPrintInfo::FinishPage()
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::CallJobDialog: 
//----------------------------------------------------------------------------------------


bool TPrintInfo::CallJobDialog()
{
	return true;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::CallStyleDialog: 
//----------------------------------------------------------------------------------------


bool TPrintInfo::CallStyleDialog()
{
	return true;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::CallPrValidate: 
//----------------------------------------------------------------------------------------


bool TPrintInfo::CallPrValidate()
{
	return false;
}


//----------------------------------------------------------------------------------------
// TPrintInfo::CallPrintDefault: 
//----------------------------------------------------------------------------------------


void TPrintInfo::CallPrintDefault()
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::CallPrPicFile: 
//----------------------------------------------------------------------------------------


bool TPrintInfo::CallPrPicFile(OSErr& err)
{
	err = noErr;
	
	return true;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetDefaultPrintInfo: 
//----------------------------------------------------------------------------------------


TPrintInfo* TPrintInfo::GetDefaultPrintInfo()
{
	return gDefaultPrintInfo;
}

//----------------------------------------------------------------------------------------
// TPrintInfo::JobMerge:
//----------------------------------------------------------------------------------------


void TPrintInfo::JobMerge(TPrintInfo* /* mergeFrom */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::BanishPrintDialog: 
//----------------------------------------------------------------------------------------


void TPrintInfo::BanishPrintDialog()
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::PosePrintDialog: 
//----------------------------------------------------------------------------------------


void TPrintInfo::PosePrintDialog(const CStr255_AC& /* docName */, CommandNumber /* aCommandNumber */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::ShowDocBeingPrinted: 
//----------------------------------------------------------------------------------------


void TPrintInfo::ShowDocBeingPrinted(const CStr255_AC& /* docName */, bool/* entering */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetMaxSquareResolution:
//----------------------------------------------------------------------------------------

void TPrintInfo::GetMaxSquareResolution(Fixed& hRes, Fixed& vRes)
{
	hRes = IntToFixed(kClassicQDResolution);
	vRes = IntToFixed(kClassicQDResolution);

}

//----------------------------------------------------------------------------------------
// TPrintInfo::SetResolution:
//----------------------------------------------------------------------------------------

void TPrintInfo::SetResolution(Fixed /* hRes */, Fixed /* vRes */)
{
}

//----------------------------------------------------------------------------------------
// TPrintInfo::MaximizeResolution:
//----------------------------------------------------------------------------------------

void TPrintInfo::MaximizeResolution()
{
	Fixed hRes;
	Fixed vRes;
	GetMaxSquareResolution(hRes, vRes);
	SetResolution(hRes, vRes);
}

//----------------------------------------------------------------------------------------
// TPrintInfo::MaximizeResolution:
//----------------------------------------------------------------------------------------

short TPrintInfo::GetScale()
{
	return 100;
}

#if qGXPrinting
//========================================================================================
// CLASS TGXJob
//========================================================================================
#undef Inherited
#define Inherited TPrintInfo


MA_DEFINE_CLASS(TGXJob);

//----------------------------------------------------------------------------------------
// TGXJob constructor
//----------------------------------------------------------------------------------------


TGXJob::TGXJob(gxJob itsgxJob,
								bool isValid) :
	fgxJob(NULL),
	fPrintViewPort(NULL)
{
	if (itsgxJob)
	{
		fgxJob = itsgxJob;
		AdoptJob(fgxJob);
		fIsValid = isValid;
	}
	else
	{
		::ThrowIfOSErr_AC(::GXNewJob(&fgxJob));
		AdoptJob(fgxJob);
		SetToDefault();
	}

	if (!pgxShapeSpoolUPP)
		::ThrowIfNULL_AC(pgxShapeSpoolUPP = NewgxShapeSpoolProc(MyPrintAShape));

	if (!pGXPrintingEventUPP)
		::ThrowIfNULL_AC(pGXPrintingEventUPP = NewGXPrintingEventProc(PrintingEventOverride));
	
	if (!pgxPrintingFlattenUPP)
		::ThrowIfNULL_AC(pgxPrintingFlattenUPP = NewgxPrintingFlattenProc(MyFlattenFunction));
	
	if (!pgxPrintingUnflattenUPP)
		::ThrowIfNULL_AC(pgxPrintingUnflattenUPP = NewgxPrintingFlattenProc(MyUnflattenFunction));

	if (!pgxGetSizeHelperUPP)
		::ThrowIfNULL_AC(pgxGetSizeHelperUPP = NewgxPrintingFlattenProc(GetSizeHelper));
}

//----------------------------------------------------------------------------------------
// TGXJob destructor
//----------------------------------------------------------------------------------------


TGXJob::~TGXJob()
{
	if (fgxJob)
	{
		::GXDisposeJob(fgxJob);
		fgxJob = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TGXJob::ReadFrom: 
//----------------------------------------------------------------------------------------


void TGXJob::ReadFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);

	if (!fgxJob)
	{
		::ThrowIfOSErr_AC(::GXNewJob(&fgxJob));
		AdoptJob(fgxJob);
	}

	::GXUnflattenJob(fgxJob, (gxPrintingFlattenProc)pgxPrintingUnflattenUPP, aStream);
}

//----------------------------------------------------------------------------------------
// TGXJob::ConvertFrom: 
//----------------------------------------------------------------------------------------


void TGXJob::ConvertFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);

	THPrint aPrint = (THPrint)NewPermHandleClear(sizeof(TPrint));
	CTempHandle_AC tempHandle((Handle)aPrint);

	TemplateTempHandleLock<THPrint> tempHandleLock(aPrint, eMoveHi);
	aStream->ReadBytes(*aPrint, sizeof(TPrint));

	if (!fgxJob)
	{
		::ThrowIfOSErr_AC(::GXNewJob(&fgxJob));
		AdoptJob(fgxJob);
	}

	::GXConvertPrintRecord(fgxJob, aPrint);
}

//----------------------------------------------------------------------------------------
// TGXJob::WriteTo: 
//----------------------------------------------------------------------------------------


void TGXJob::WriteTo(CStream_AC* aStream) const
{
	MStreamable::WriteTo(aStream);

	if (fgxJob)
	{
		::GXFlattenJob(fgxJob, (gxPrintingFlattenProc)pgxPrintingFlattenUPP, aStream);
	}
}

//----------------------------------------------------------------------------------------
// TGXJob::SetFrom
//----------------------------------------------------------------------------------------


void TGXJob::SetFrom(TPrintInfo* jobToCopy)
{
	Inherited::SetFrom(jobToCopy);

	TGXJob * gxToCopy = dynamic_cast_AC(TGXJob*,jobToCopy);

	::GXCopyJob(gxToCopy->GetRawgxJob(), GetRawgxJob());

	if (fIsValid)
		Changed(mPrintInfoChanged, this);
	else
		Invalidate();
}

//----------------------------------------------------------------------------------------
// TGXJob::SetFrom
//----------------------------------------------------------------------------------------


void TGXJob::SetFrom(gxJob jobToCopy, bool isValid,
				bool needsDefault)
{
	::GXCopyJob(jobToCopy, fgxJob);

	fIsValid = isValid;
	fNeedsDefault = needsDefault;

	if (fIsValid)
		Changed(mPrintInfoChanged, this);
	else
		Invalidate();
}

//----------------------------------------------------------------------------------------
// TGXJob::GetSize
//----------------------------------------------------------------------------------------


long TGXJob::GetSize(bool				/* asResource */) const
{
	long returnVal = 0;

	// the helper function will total up the bytes
	if (GetRawgxJob())
		::GXFlattenJob(GetRawgxJob(), (gxPrintingFlattenProc)pgxGetSizeHelperUPP, &returnVal);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TGXJob::GetSize
//----------------------------------------------------------------------------------------


pascal OSErr TGXJob::GetSizeHelper(long size,
										 void* /* data */,
										 void* refCon)
{
	*static_cast<long*>(refCon) += size;

	return noErr;
}


//----------------------------------------------------------------------------------------
// TGXJob copy constructor
//----------------------------------------------------------------------------------------


TGXJob::TGXJob(const TGXJob& inGXJob) :
	TPrintInfo(inGXJob),

	fgxJob(inGXJob.GetRawgxJob() ? ::GXCopyJob(inGXJob.fgxJob, NULL) : NULL),
	fPrintViewPort(inGXJob.fPrintViewPort),

	fQDPortRec(inGXJob.fQDPortRec),
	fQDPort(inGXJob.fQDPort),
	fSpoolData(inGXJob.fSpoolData)
{
}

//----------------------------------------------------------------------------------------
// TGXJob::GetPaperRect:
//----------------------------------------------------------------------------------------


CViewRect TGXJob::GetPaperRect()
{
	gxFormat theFormat = ::GXGetJobFormat(GetgxJob(), 1);	//Get the default format for now.

	gxRectangle	pageSize;
	gxRectangle	paperSize;
	::GXGetFormatDimensions( theFormat, &pageSize, &paperSize);
	
	CViewRect thePaper;
	thePaper.top    = FixedTruncate(paperSize.top);
	thePaper.left   = FixedTruncate(paperSize.left);
	thePaper.bottom = FixedTruncate(paperSize.bottom);
	thePaper.right  = FixedTruncate(paperSize.right);

	return thePaper;
}

//----------------------------------------------------------------------------------------
// TGXJob::GetInkRect:
//----------------------------------------------------------------------------------------


CViewRect TGXJob::GetInkRect()
{

	gxFormat theFormat = ::GXGetJobFormat( GetgxJob(), 1);	//Get the default format for now.

	gxRectangle	pageSize;
	gxRectangle	paperSize;
	::GXGetFormatDimensions( theFormat, &pageSize, &paperSize);

	CViewRect ink;
	ink.top    = FixedTruncate(pageSize.top);
	ink.left   = FixedTruncate(pageSize.left);
	ink.bottom = FixedTruncate(pageSize.bottom);
	ink.right  = FixedTruncate(pageSize.right);

	return ink;
}

//----------------------------------------------------------------------------------------
// TGXJob::GetDeviceRes:
//----------------------------------------------------------------------------------------


CPoint_AC TGXJob::GetDeviceRes()
{
	// Get the formatting printer
	gxPrinter formatPrinter = ::GXGetJobFormattingPrinter(GetgxJob());

	// Get the current view device and its mapping
	gxViewDevice printerVDev = ::GXGetPrinterViewDevice(formatPrinter, 0);
	gxMapping vDevMapping;
	::GXGetViewDeviceMapping(printerVDev, &vDevMapping);

	long hRes = vDevMapping.map[0][0];
	long vRes = vDevMapping.map[1][1];

	//	Convert scaling factors (multiples of kClassicQDResolution (72) dpi) into
	//	resolutions.

	hRes = FixedToInt(FixedMultiply(hRes, IntToFixed(kClassicQDResolution)));
	vRes = FixedToInt(FixedMultiply(vRes, IntToFixed(kClassicQDResolution)));
	
	return CViewPoint(hRes, vRes).ToPoint();
}

//----------------------------------------------------------------------------------------
// TGXJob::GetPrintPages:
//----------------------------------------------------------------------------------------


void TGXJob::GetPrintPages(long& fromPage,
							   long& toPage,
							   long& minFromPage,
							   long& maxToPage)
{
// Get the job collection
	Collection jobCollection = GXGetJobCollection(GetgxJob());

	long oldDataSize;
	long index;
	::ThrowIfOSErr_AC(GetCollectionItemInfo(jobCollection,
									gxPageRangeTag,
									gxPrintingTagID,
									&index,
									&oldDataSize,
									dontWantAttributes));

	gxPageRangeInfo pageRangeInfo;
	::ThrowIfOSErr_AC(GetCollectionItem(jobCollection,
							gxPageRangeTag,
							gxPrintingTagID,
							&oldDataSize,
							&pageRangeInfo));


	fromPage = pageRangeInfo.simpleRange.fromPage;
	toPage = pageRangeInfo.simpleRange.toPage;

	minFromPage = pageRangeInfo.minFromPage;
	maxToPage = pageRangeInfo.maxToPage;
}

//----------------------------------------------------------------------------------------
// TGXJob::SetPrintPages:
//----------------------------------------------------------------------------------------


void TGXJob::SetPrintPages(long fromPage,
							   long toPage,
							   long minFromPage,
							   long maxToPage)
{
// Get the job collection
	Collection jobCollection = GXGetJobCollection(GetgxJob());

	long oldDataSize;
	long index;
	::ThrowIfOSErr_AC(GetCollectionItemInfo(jobCollection,
									gxPageRangeTag,
									gxPrintingTagID,
									&index,
									&oldDataSize,
									dontWantAttributes));

	gxPageRangeInfo pageRangeInfo;
	::ThrowIfOSErr_AC(GetCollectionItem(jobCollection,
							gxPageRangeTag,
							gxPrintingTagID,
							&oldDataSize,
							&pageRangeInfo));

	pageRangeInfo.minFromPage = minFromPage;
	pageRangeInfo.maxToPage = maxToPage;

	pageRangeInfo.simpleRange.fromPage = fromPage;
	pageRangeInfo.simpleRange.toPage = toPage;

//	pageRangeInfo.simpleRange.optionChosen = gxDefaultPageRange;
	pageRangeInfo.simpleRange.printAll = (fromPage == minFromPage) && (toPage == maxToPage);

	::ThrowIfOSErr_AC(ReplaceIndexedCollectionItem(jobCollection,
										   index,
										   oldDataSize,
										   &pageRangeInfo));
}

//----------------------------------------------------------------------------------------
// TGXJob::GetPrintCopies:
//----------------------------------------------------------------------------------------


long TGXJob::GetPrintCopies()
{
// Get the job collection
	Collection jobCollection = GXGetJobCollection(GetgxJob());

	long oldDataSize;
	::ThrowIfOSErr_AC(GetCollectionItemInfo(jobCollection,
									gxCopiesTag,
									gxPrintingTagID,
									dontWantIndex,
									&oldDataSize,
									dontWantAttributes));

	gxCopiesInfo copiesInfo;
	::ThrowIfOSErr_AC(GetCollectionItem(jobCollection,
							gxCopiesTag,
							gxPrintingTagID,
							&oldDataSize,
							&copiesInfo));

	return copiesInfo.copies;
}

//----------------------------------------------------------------------------------------
// TGXJob::SetPrintCopies:
//----------------------------------------------------------------------------------------


void TGXJob::SetPrintCopies(long numCopies)
{
// Get the job collection
	Collection jobCollection = GXGetJobCollection(GetgxJob());

	long oldDataSize;
	long index;
	::ThrowIfOSErr_AC(GetCollectionItemInfo(jobCollection,
									gxCopiesTag,
									gxPrintingTagID,
									&index,
									&oldDataSize,
									dontWantAttributes));

	gxCopiesInfo copiesInfo;
	::ThrowIfOSErr_AC(GetCollectionItem(jobCollection,
							gxCopiesTag,
							gxPrintingTagID,
							&oldDataSize,
							&copiesInfo));


	copiesInfo.copies = numCopies;
	::ThrowIfOSErr_AC(ReplaceIndexedCollectionItem(jobCollection,
										   index,
										   oldDataSize,
										   &copiesInfo));
}

//----------------------------------------------------------------------------------------
// TGXJob::SetTargetPrinter:
//----------------------------------------------------------------------------------------


void TGXJob::SetTargetPrinter(const CFSSpec_AC& targetPrinter)
{
	::GXSelectJobOutputPrinter(GetgxJob(), CStr31_AC(targetPrinter.name));
}

//----------------------------------------------------------------------------------------
// TGXJob::SetToDefault:
//----------------------------------------------------------------------------------------


void TGXJob::SetToDefault()
{
	fNeedsDefault = true;

	Invalidate();
}

//----------------------------------------------------------------------------------------
// TGXJob::IsSetToDefault:
//----------------------------------------------------------------------------------------


bool TGXJob::IsSetToDefault()
{
	// we don't really know how to compare the job to see if it's just like the default
	// so let's just assume that if we even have a job it's not default
	return fgxJob ? false : true;
}

//----------------------------------------------------------------------------------------
// TGXJob::AdoptJob: 
//----------------------------------------------------------------------------------------


void TGXJob::AdoptJob(gxJob itsgxJob)
{
	::GXInstallApplicationOverride(itsgxJob, gxPrintingEventMsg, pGXPrintingEventUPP);
}

//----------------------------------------------------------------------------------------
// TGXJob::PrintingEventOverride: 
//----------------------------------------------------------------------------------------


OSErr TGXJob::PrintingEventOverride(EventRecord* anEvent,
										  bool filterEvent)
{
	if (!filterEvent && gDispatcher)
	{
		static TToolboxEvent* bufferableEvent;
	
		try
		{
			bool needsSecondaryEvent = false;
			if (!bufferableEvent)
			{
				CTempObjectAllocation tempObjectAllocation(true);
				bufferableEvent = TH_new TToolboxEvent(gDispatcher, anEvent);
				needsSecondaryEvent = bufferableEvent->IsDoubleEvent();
			}
			else
				bufferableEvent->AddSecondaryKeyEvent(*anEvent);
			
			if (!needsSecondaryEvent)
			{
				gDispatcher->SetEventLevel(gDispatcher->GetEventLevel() + 1);
				gDispatcher->SetSysWindowActive(true);


				InvalidateMenus();
				gDispatcher->SetupTheMenus();
				gDispatcher->InvalidateFocus();
				bufferableEvent->Process();


				gDispatcher->SetSysWindowActive(false);
				gDispatcher->SetEventLevel(gDispatcher->GetEventLevel() - 1);
				bufferableEvent = NULL;
			}
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			delete bufferableEvent;
			bufferableEvent = NULL;
		}
	}
	
	return noErr;
}

//----------------------------------------------------------------------------------------
// TGXJob::GetgxJob:
//----------------------------------------------------------------------------------------


gxJob TGXJob::GetgxJob()
{
	Update();

	return fgxJob;
}

//----------------------------------------------------------------------------------------
// TGXJob::GetError: 
//----------------------------------------------------------------------------------------


OSErr TGXJob::GetError()
{
	return ::GXGetJobError(GetgxJob());
}

//----------------------------------------------------------------------------------------
// TGXJob::SetError: 
//----------------------------------------------------------------------------------------


void TGXJob::SetError(OSErr err)
{
	::GXSetJobError(GetRawgxJob(), err);
}

//----------------------------------------------------------------------------------------
// TGXJob::StartJob: 
//----------------------------------------------------------------------------------------


void TGXJob::StartJob(const CStr255_AC& docTitle,
						long numPages)
{
	::GXStartJob(GetgxJob(), CStr255_AC(docTitle), numPages);

	::ThrowIfOSErr_AC(GetError());
	
	fPrintViewPort = ::GXNewViewPort(gxScreenViewDevices);
	
	
	OpenCPort(&fQDPortRec);
	fQDPort = &fQDPortRec;

	SetPort(GetJobPort());

	CRect_AC wideopenRect(-32766, -32766, 32767, 32767);
	RectRgn(GetJobPort()->visRgn, &wideopenRect);
	ClipRect(&wideopenRect);
}

//----------------------------------------------------------------------------------------
// TGXJob::GetJobPort: 
//----------------------------------------------------------------------------------------


GrafPtr TGXJob::GetJobPort()
{
	return (GrafPtr)fQDPort;
}

//----------------------------------------------------------------------------------------
// TGXJob::FinishJob: 
//----------------------------------------------------------------------------------------


void TGXJob::FinishJob()
{
	::GXFinishJob(GetgxJob());
	OSErr err = GetError();
	
	::GXDisposeViewPort(fPrintViewPort);
	fPrintViewPort = NULL;

	SetPortWindowPort(gWorkPort);

	CloseCPort(&fQDPortRec);
	fQDPort = NULL;	// So we don't accidentally use it

	::ThrowIfOSErr_AC(err);
}

//----------------------------------------------------------------------------------------
// TGXJob::StartPage: 
//----------------------------------------------------------------------------------------


void TGXJob::StartPage(long pageNumber)
{
	
	// to get the specific pages format
	
	gxFormat pageFormat = ::GXGetJobFormat(GetgxJob(), 1);

	::GXStartPage(GetgxJob(), pageNumber, pageFormat, 1, &fPrintViewPort);
	::ThrowIfOSErr_AC(GetError());

	fSpoolData.printViewPort = fPrintViewPort;

	// Install Translators
	::GXGetFormatDimensions(pageFormat, &fSpoolData.pageArea, NULL);

	CRect_AC everywhereRect(0, 0, 32767, 32767);

	::GXInstallQDTranslator(GetJobPort(),
						  gxDefaultOptionsTranslation,
						  &everywhereRect, &everywhereRect,
						  CPoint_AC(1, 1) /* patStretch */, 
						  pgxShapeSpoolUPP,
						  &fSpoolData);
}

//----------------------------------------------------------------------------------------
// TGXJob::FinishPage: 
//----------------------------------------------------------------------------------------


void TGXJob::FinishPage()
{
// Remove Translators
		::GXRemoveQDTranslator(GetJobPort(), NULL);

// Finish the page.

		::GXFinishPage(GetgxJob());
}


//----------------------------------------------------------------------------------------
// TGXJob::PrepareEditRecord: 
//----------------------------------------------------------------------------------------


void TGXJob::PrepareEditRecord(gxEditMenuRecord* editMenuRecord)
{
	short itsItem;
	CommandToMenuItem(cCut, editMenuRecord->editMenuID, itsItem);
	short itsMenu;

	CommandToMenuItem(cCut, itsMenu, editMenuRecord->cutItem);
	CommandToMenuItem(cCopy, itsMenu, editMenuRecord->copyItem);
	CommandToMenuItem(cPaste, itsMenu, editMenuRecord->pasteItem);
	CommandToMenuItem(cClear, itsMenu, editMenuRecord->clearItem);
	CommandToMenuItem(cUndo, itsMenu, editMenuRecord->undoItem);
}


//----------------------------------------------------------------------------------------
// TGXJob::CallJobDialog: 
//----------------------------------------------------------------------------------------


bool TGXJob::CallJobDialog()
{
	::ThrowIfOSErr_AC(MAInteractWithUser());
	PullApplicationToFront();

	SetCursor(&qd.arrow);
	if (gDispatcher)
		gDispatcher->InvalidateMouseRegions();

	gDispatcher->AboutToLoseControl(true);		// so scrap gets converted and floaters get removed

	gxEditMenuRecord editMenuRecord;
	PrepareEditRecord(&editMenuRecord);

	EnableItem(MAGetMenu(editMenuRecord.editMenuID), 0); // ensure the menu is enabled for GX

	HiliteMenu(0);
	//SRF - force the even handler in place
	AdoptJob(GetgxJob());

	gxDialogResult aGXDialogResult = ::GXJobPrintDialog(GetgxJob(), &editMenuRecord);
	bool result = aGXDialogResult == gxOKSelected;

	gDispatcher->RegainControl(true);			// scrap gets converted, printer gets checked, floaters get restored

	return result;
}

//----------------------------------------------------------------------------------------
// TGXJob::CallStyleDialog: 
//----------------------------------------------------------------------------------------


bool TGXJob::CallStyleDialog()
{
	::ThrowIfOSErr_AC(MAInteractWithUser());
	PullApplicationToFront();

	SetCursor(&qd.arrow);
	if (gDispatcher)
		gDispatcher->InvalidateMouseRegions();

	gDispatcher->AboutToLoseControl(true);		// so scrap gets converted and floaters get removed

	gxEditMenuRecord editMenuRecord;
	PrepareEditRecord(&editMenuRecord);

	EnableItem(MAGetMenu(editMenuRecord.editMenuID), 0); // ensure the menu is enabled for GX

	HiliteMenu(0);
	//SRF - force the even handler in place
	AdoptJob(GetgxJob());

	gxDialogResult aGXDialogResult = ::GXJobDefaultFormatDialog(GetgxJob(), &editMenuRecord);
	bool result = aGXDialogResult == gxOKSelected;

	gDispatcher->RegainControl(true);			// scrap gets converted, printer gets checked, floaters get restored

	return result;
}

//----------------------------------------------------------------------------------------
// TGXJob::CallPrValidate: 
//----------------------------------------------------------------------------------------


bool TGXJob::CallPrValidate()
{
	static const bool kNoChange_AC = false;
	
	return kNoChange_AC;
}


//----------------------------------------------------------------------------------------
// TGXJob::CallPrintDefault: 
//----------------------------------------------------------------------------------------


void TGXJob::CallPrintDefault()
{
	if (fgxJob)
	{
		::GXDisposeJob(fgxJob);
		fgxJob = NULL;
	}

	::ThrowIfOSErr_AC(::GXNewJob(&fgxJob));
}

//----------------------------------------------------------------------------------------
// TGXJob::Update:
//----------------------------------------------------------------------------------------


void TGXJob::Update()
{
	Inherited::Update();
}

//----------------------------------------------------------------------------------------
// TGXJob::DoUpdate:
//----------------------------------------------------------------------------------------


void TGXJob::DoUpdate(ChangeID_AC theChange,
						MDependable_AC* changedObject,
						void* changeData,
						CDependencySpace_AC* dependencySpace)// Override
{
	switch (theChange)
	{
		case mRegainControl:
			TPrintHandler::gCouldPrint = true;			// Could be OK now
			InvalidateMenus();							// so the menus stay in sync.

			Invalidate();
			Changed(mPrinterChanged, this);
			break;

		default:
			Inherited::DoUpdate(theChange, changedObject, changeData, dependencySpace);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TGXJob::JobMerge:
//----------------------------------------------------------------------------------------


void TGXJob::JobMerge(TPrintInfo* /* mergeFrom */)
{
	//SRF fix this
}

//----------------------------------------------------------------------------------------
// TGXJob::MyPrintAShape:
//----------------------------------------------------------------------------------------
OSErr TGXJob::MyPrintAShape(gxShape currentShape, long refCon)
{
	// Don't waste time spooling the shape if it's being
	// drawn completely off the page.

	MySpoolDataRec* spoolData = (MySpoolDataRec*) refCon;
	gxShapeType theShapeType = ::GXGetShapeType(currentShape);
	
#if 0
// Actually, MacApp has already gone to great lengths to ensure that we only printed
// Stuff that intersects the page. This test is unnecessary, and in fact, since we are
// not updating the spoolData.pageArea... it doesn't work!
	if ((theShapeType == gxEmptyType)   || 
		(theShapeType == gxFullType)    || 
		(theShapeType == gxPictureType) ||
		(GXTouchesBoundsShape(&spoolData->pageArea, currentShape)))
#endif
	{
		::GXSetShapeViewPorts(currentShape, 1, &spoolData->printViewPort);
		::GXDrawShape(currentShape);
	}
	
	return (OSErr) ::GXGetGraphicsError(NULL);
}

//----------------------------------------------------------------------------------------
// TGXJob::MyUnflattenFunction:
//----------------------------------------------------------------------------------------
pascal OSErr TGXJob::MyFlattenFunction (long size, void *data, void *refCon)
{
	OSErr returnVal = noErr;

	try
	{
		reinterpret_cast<CStream_AC*>(refCon)->WriteBytes(data, size);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		returnVal = theException.GetError();
	}
		// drop thru
	

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TGXJob::MyUnflattenFunction:
//----------------------------------------------------------------------------------------
pascal OSErr TGXJob::MyUnflattenFunction (long size, void *data, void *refCon)
{
	OSErr returnVal = noErr;

	try
	{
		reinterpret_cast<CStream_AC*>(refCon)->ReadBytes(data, size);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		returnVal = theException.GetError();
	}
		// drop thru
	

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TGXJob::GetMaxSquareResolution:
//----------------------------------------------------------------------------------------
void TGXJob::GetMaxSquareResolution(Fixed& hRes, Fixed& vRes)
{
	hRes = 0;
	vRes = 0;

	// Get the formatting printer and the number of
	// view devices for that printer.

	gxPrinter formatPrinter = ::GXGetJobFormattingPrinter(GetgxJob());
	long numViewDevices = ::GXCountPrinterViewDevices (formatPrinter);

	// Loop through the view devices that this printer supports.
	for (long idx = 1; idx <= numViewDevices; ++idx)
	{
		gxViewDevice printerVDev = ::GXGetPrinterViewDevice(formatPrinter, idx);
		gxMapping vDevMapping;
		::GXGetViewDeviceMapping(printerVDev, &vDevMapping);

		if ((vDevMapping.map[0][0] > hRes) && (vDevMapping.map[1][1] > vRes))
		{
			hRes = vDevMapping.map[0][0];
			vRes = vDevMapping.map[1][1];
		}
	}


	//	Convert scaling factors (multiples of kClassicQDResolution (72) dpi) into
	//	resolutions.

	hRes = FixedMultiply(hRes, IntToFixed(kClassicQDResolution));
	vRes = FixedMultiply(vRes, IntToFixed(kClassicQDResolution));
}

//----------------------------------------------------------------------------------------
// TGXJob::SetResolution:
//----------------------------------------------------------------------------------------
void TGXJob::SetResolution(Fixed hRes, Fixed vRes)
{
	//	Convert scaling factors (multiples of kClassicQDResolution (72) dpi) into
	//	resolutions.

	hRes = FixedDivide(hRes, IntToFixed(kClassicQDResolution));
	vRes = FixedDivide(vRes, IntToFixed(kClassicQDResolution));


	// Get the formatting printer and the number of
	// view devices for that printer.

	gxPrinter formatPrinter = ::GXGetJobFormattingPrinter(GetgxJob());
	long numViewDevices = ::GXCountPrinterViewDevices (formatPrinter);

	// Loop through the view devices that this printer supports.
	for (long idx = 1; idx <= numViewDevices; ++idx)
	{
		gxViewDevice printerVDev = ::GXGetPrinterViewDevice(formatPrinter, idx);
		gxMapping vDevMapping;
		::GXGetViewDeviceMapping(printerVDev, &vDevMapping);

		if ((vDevMapping.map[0][0] == hRes) && (vDevMapping.map[1][1] == vRes))
			::GXSelectPrinterViewDevice(formatPrinter, idx);
	}


	Invalidate();
}

//----------------------------------------------------------------------------------------
// TGXJob::GetScale:
//----------------------------------------------------------------------------------------

short TGXJob::GetScale()
{
	CViewRect printingRect(GetPaperRect());
	VHSelect longerDirection = LongerSide(printingRect);
	ViewCoordinate projectionPaperSize = printingRect.GetSize()[longerDirection];

	gxPaperType aGXPaperType = ::GXGetFormatPaperType(GXGetJobFormat( GetgxJob(), 1));	//Get the default format for now.
	::ThrowIfOSErr_AC(GetError());

	gxRectangle	paperTypePageSize;
	gxRectangle	paperTypepaperSize;
	::GXGetPaperTypeDimensions (aGXPaperType, &paperTypePageSize, &paperTypepaperSize);

	CViewRect paperTypePaperSize;
	paperTypePaperSize.top    = FixedTruncate(paperTypepaperSize.top);
	paperTypePaperSize.left   = FixedTruncate(paperTypepaperSize.left);
	paperTypePaperSize.bottom = FixedTruncate(paperTypepaperSize.bottom);
	paperTypePaperSize.right  = FixedTruncate(paperTypepaperSize.right);

	
	short paperSizeIn_iPrPgFract = paperTypePaperSize.GetSize()[LongerSide(paperTypePaperSize)];
	short paperSizeInDeviceResDPI = paperSizeIn_iPrPgFract;
	
	return (paperSizeInDeviceResDPI * 100) / projectionPaperSize;
}
#endif

//========================================================================================
// CLASS CInMacPrint
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// CInMacPrint::CInMacPrint: 
//----------------------------------------------------------------------------------------


CInMacPrint::CInMacPrint(TPrintInfo* itsPrintinfo, void* byWhom, bool active)
{
	fIsActive = active;
	fPrintShopOpened = false;

	if (fIsActive)
	{
#if qDebug
#	if qGXPrinting
		if (gUsingGXPrinting)
			::DebugStr("\pYikes!!, making an active CInMacPrint call while using QD GX!");
#	endif
#endif

		fPrintInfo = itsPrintinfo ? itsPrintinfo : gDefaultPrintInfo;
		fByWhom = byWhom ? byWhom : fPrintInfo;


		fResFile = CurResFile_AC();
		fPrintInfo->ClearError();					// Clear printer-error flag 

		fPrintInfo->Open(fByWhom);
		fPrintShopOpened = true;
	}
}

//----------------------------------------------------------------------------------------
// CInMacPrint::~CInMacPrint: 
//----------------------------------------------------------------------------------------


CInMacPrint::~CInMacPrint()
{
	if (fIsActive)
	{
		if (fPrintInfo->GetError() == errNoPrintDrvr)
		{
			TPrintHandler::gCouldPrint = false;		// well, we didn't make it. don't bother to try again
			InvalidateMenus();						// so the menus stay in sync/
		}

		if (fPrintShopOpened)
		{
			fPrintInfo->Close(fByWhom);
			UseResFile_AC(fResFile);
			SetPortWindowPort(gWorkPort);			// Might be left looking at a dead port 
			gDispatcher->InvalidateFocus();
	
			fPrintShopOpened = false;
		}
	}
}

//========================================================================================
// CLASS TMacPrintJob
//========================================================================================
#undef Inherited
#define Inherited TPrintInfo


MA_DEFINE_CLASS(TMacPrintJob);

//----------------------------------------------------------------------------------------
// TMacPrintJob constructor
//----------------------------------------------------------------------------------------


TMacPrintJob::TMacPrintJob(THPrint itsHPrint,
								  bool isValid) :
	fPrintDialog(NULL)
{
	if (itsHPrint)
	{
		fHPrint = itsHPrint;
		fIsValid = isValid;
	}
	else
	{
		fHPrint = (THPrint)NewPermHandleClear(sizeof(TPrint));
		SetToDefault();
	}

	if (!fgPrinterChangeLevel)
		CheckPrinterChangeLevel();
	fPrinterChangeLevel = fgPrinterChangeLevel;

	if (!pPrintIdleProc)
		::ThrowIfNULL_AC(pPrintIdleProc = NewPrIdleProc(TMacPrintJob::IdleProc));
}

//----------------------------------------------------------------------------------------
// TMacPrintJob destructor
//----------------------------------------------------------------------------------------


TMacPrintJob::~TMacPrintJob()
{
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::ReadFrom: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::ReadFrom(CStream_AC* aStream)
{
	Inherited::ReadFrom(aStream);

	if (!fHPrint)
		fHPrint = (THPrint)NewPermHandleClear(sizeof(TPrint));

	TemplateTempHandleLock<THPrint> tempHandleLock(fHPrint, eMoveHi);
	aStream->ReadBytes(*fHPrint, GetSize());

	Invalidate();
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::WriteTo: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::WriteTo(CStream_AC* aStream) const
{
	MStreamable::WriteTo(aStream);

	if (fHPrint)
	{
		// Don't need to validate it before saving it b/c it will be
		// validated before USE when loaded again in the future.
		// Also, if a document is opened, modified and saved without modifying
		// the printinfo (via page setup or printing or something; then the printshop
		// never even needs to be opened at all (Saving lots & lots of time)

		TemplateTempHandleLock<THPrint> tempHandleLock(fHPrint, eMoveHi);
		aStream->WriteBytes(*fHPrint, GetSize());
	}
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetFrom
//----------------------------------------------------------------------------------------


void TMacPrintJob::SetFrom(TPrintInfo* jobToCopy)
{
	Inherited::SetFrom(jobToCopy);

	TMacPrintJob * macPrintToCopy = dynamic_cast_AC(TMacPrintJob*,jobToCopy);

	BlockMove_AC(*macPrintToCopy->GetRawHPrint(), *GetRawHPrint(), GetHandleSize((Handle)GetRawHPrint()));

	if (fIsValid)
		Changed(mPrintInfoChanged, this);
	else
		Invalidate();
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetFrom
//----------------------------------------------------------------------------------------


void TMacPrintJob::SetFrom(THPrint hPrintToCopy,
							bool isValid,
							bool needsDefault)
{
	BlockMove_AC(*hPrintToCopy, *fHPrint, GetHandleSize((Handle)hPrintToCopy));

	fIsValid = isValid;
	fNeedsDefault = needsDefault;

	if (fIsValid)
		Changed(mPrintInfoChanged, this);
	else
		Invalidate();
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetSize
//----------------------------------------------------------------------------------------


long TMacPrintJob::GetSize(bool				/* asResource */) const
{
	long returnVal = 0;
	if (fHPrint)
		returnVal = GetHandleSize(reinterpret_cast<Handle>(fHPrint.get()));

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::IsMatch
//----------------------------------------------------------------------------------------


bool TMacPrintJob::IsMatch(TPrintInfo* thePrintInfo)
{
	return fIsValid && EqualBlocks(*GetRawHPrint(), *dynamic_cast_AC(TMacPrintJob*,thePrintInfo)->GetRawHPrint(), thePrintInfo->GetSize());
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetToDefault:
//----------------------------------------------------------------------------------------


void TMacPrintJob::SetToDefault()
{
	fNeedsDefault = true;

	// Print Manager code could unavailable, but we would want some plausible things
	// anyway - set up for Portrait Tall Adj.
	TPrint & anHPrint = **fHPrint;
	anHPrint.iPrVersion = 0;					// something invalid 

	anHPrint.prInfo.iHRes = kClassicQDResolution;
	anHPrint.prInfo.iVRes = kClassicQDResolution;
	anHPrint.prInfo.rPage = CRect_AC(0, 0, 576, 752);// must have its top left & (0,0)

	anHPrint.rPaper = CRect_AC(-18, -36, 594, 756);
	anHPrint.prStl.iPageV = 1320;				// 11 inches in 120th of an inch 
	anHPrint.prStl.iPageH = 1020;				// 8.5 inches in 120th of an inch 

	Invalidate();
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::IsSetToDefault:
//----------------------------------------------------------------------------------------


bool TMacPrintJob::IsSetToDefault()
{
	return fNeedsDefault || GetDefaultPrintInfo()->IsMatch(this);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob copy constructor
//----------------------------------------------------------------------------------------


TMacPrintJob::TMacPrintJob(const TMacPrintJob& inMacPrintJob) :
	TPrintInfo(inMacPrintJob),
	
	fCurrentDocTitle(inMacPrintJob.fCurrentDocTitle),
	fPrintDialog(NULL),		// Sorry, but this just can't be cloned
	fHPrint(inMacPrintJob.fHPrint ? ((THPrint)PermCopyHandle(reinterpret_cast<Handle>(inMacPrintJob.fHPrint.get()))) : NULL),

	fMinFromPage(inMacPrintJob.fMinFromPage),
	fMaxToPage(inMacPrintJob.fMaxToPage),
	
	fPrinterChangeLevel(inMacPrintJob.fPrinterChangeLevel)
{
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetPrintPages:
//----------------------------------------------------------------------------------------


void TMacPrintJob::GetPrintPages(long& fromPage,
							   long& toPage,
							   long& minFromPage,
							   long& maxToPage)
{
	fromPage = (*GetHPrint())->prJob.iFstPage;
	toPage = (*GetHPrint())->prJob.iLstPage;

	minFromPage = fMaxToPage;
	maxToPage = fMaxToPage;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetPrintPages:
//----------------------------------------------------------------------------------------


void TMacPrintJob::SetPrintPages(long fromPage,
							   long toPage,
							   long minFromPage,
							   long maxToPage)
{
	//
	// MacPrint can only take shorts for the page range and we always tell it we're
	// printing from page 1, otherwise we have to do PrOpenPage/ClosePage up to the 
	// desired page range before MacPrint will start printing
	//
	short shortFromPage = fromPage ? 1 : 0;

	short shortToPage = 0;
	if (toPage)
	{
		shortToPage = static_cast<short>((toPage - fromPage + 1) % SHRT_MAX);
		shortToPage = shortToPage ? shortToPage : SHRT_MAX;
	}

	TPrint& print = **GetHPrint();
	print.prJob.iFstPage = shortFromPage;
	print.prJob.iLstPage = shortToPage;
	
	fMinFromPage = minFromPage;
	fMaxToPage = maxToPage;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetNumCopies:
//----------------------------------------------------------------------------------------


long TMacPrintJob::GetPrintCopies()
{
	return (*GetHPrint())->prJob.iCopies;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetNumCopies:
//----------------------------------------------------------------------------------------


void TMacPrintJob::SetPrintCopies(long numCopies)
{
	(*GetHPrint())->prJob.iCopies = numCopies;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetSpoolFile:
//----------------------------------------------------------------------------------------


void TMacPrintJob::SetSpoolFile(CFSSpec_AC& spoolFile)
{
	*((CStr31_AC*)((*GetHPrint())->prJob.pFileName)) = spoolFile.name;
	((*GetHPrint())->prJob.iFileVol = spoolFile.vRefNum);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::IsSpooling:
//----------------------------------------------------------------------------------------


bool TMacPrintJob::IsSpooling()
{
	return (*GetHPrint())->prJob.bJDocLoop == bSpoolLoop;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetPaperRect:
//----------------------------------------------------------------------------------------


CViewRect TMacPrintJob::GetPaperRect()
{
	return (*GetHPrint())->rPaper;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetInkRect:
//----------------------------------------------------------------------------------------


CViewRect TMacPrintJob::GetInkRect()
{
	return (*GetHPrint())->prInfo.rPage;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetDeviceRes:
//----------------------------------------------------------------------------------------


CPoint_AC TMacPrintJob::GetDeviceRes()
{
	return *(CPoint_AC*)&(*GetHPrint())->prInfo.iVRes;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetHPrint:
//----------------------------------------------------------------------------------------


THPrint TMacPrintJob::GetHPrint()
{
	Update();

	return fHPrint;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::CheckPrinterChangeLevel: 
//----------------------------------------------------------------------------------------


unsigned long TMacPrintJob::CheckPrinterChangeLevel()
{
	bool returnVal = false;

	CStr255_AC driverName;
	GetDriverName(driverName);

	if (!EqualString(driverName, fgLastPrinterName, false, true))
		returnVal = true;

	if (returnVal)
	{
		fgLastPrinterName = driverName;				// Save it for later
		++fgPrinterChangeLevel;						// bump the change level

		TPrintHandler::gCouldPrint = true;			// Could be OK now
		InvalidateMenus();							// so the menus stay in sync.
	}

	return fgPrinterChangeLevel;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::Open: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::Open(void* byWhom)
{
	// The Universal Headers as of 10/17/95 don't provide the following constant!!!
	static const OSErr kPAPNoPrinter = -4101;

	bool didOpen = false;

	if (!fPrintShopOpens)
	{
		PrOpen();								// Open the print shop 
		OSErr err = GetError();					// Get code 
		if (err != noErr)
		{
			if ((err == fnfErr) || (err == resFNotFound) || (err == resNotFound) || (err == dirNFErr) || (err == kPAPNoPrinter))
				err = errNoPrintDrvr;
			else if ((err == fBsyErr) || (err == opWrErr))
				err = errPrintDriverBusy;
			::ThrowException_AC(err, 0);
		}
		didOpen = true;
	}
	++fPrintShopOpens;

	if (didOpen)
		Changed(mPrinterOpened, byWhom);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::IsOpen: 
//----------------------------------------------------------------------------------------


bool TMacPrintJob::IsOpen()
{
	return fPrintShopOpens > 0;
}


//----------------------------------------------------------------------------------------
// TMacPrintJob::Close: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::Close(void* byWhom)
{
	bool didClose = false;

	if (fPrintShopOpens == 1)
	{
		::PrClose();
#if qDebugMsg
		OSErr err = GetError();
		// Now check for fresh error from Print Manager --
		// only for debugging, since ChkPrintErr
		// checks afresh in the real world
		if (err != noErr)
			ProgramReport_AC(false, "Error from Print Manager is: %1d\n", err);
#endif

		didClose = true;
	}

	--fPrintShopOpens;

	if (fPrintShopOpens < 0)
	{
		fPrintShopOpens = 0;
#if qDebugMsg
		ProgramReport_AC("Too many calls to TMacPrintJob::Close\n");
#endif

	}

	if (didClose)
		Changed(mPrinterClosed, byWhom);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetError: 
//----------------------------------------------------------------------------------------


OSErr TMacPrintJob::GetError()
{
	return ::PrError();
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetError: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::SetError(OSErr err)
{
	::PrSetError(err);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::StartJob: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::StartJob(const CStr255_AC& docTitle,
						 long					/* numPages */)
{
	CInMacPrint inMacPrint(this);

	fCurrentDocTitle = docTitle;
	fQDPort = ::PrOpenDoc(GetHPrint(), NULL, NULL);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetJobPort: 
//----------------------------------------------------------------------------------------


GrafPtr TMacPrintJob::GetJobPort()
{
	return (GrafPtr)fQDPort;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::FinishJob: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::FinishJob()
{
	::PrCloseDoc((TPPrPort)GetJobPort());			// This will close the port!
	fQDPort = NULL;
	::SetPortWindowPort(gWorkPort);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::StartPage: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::StartPage(long					/* pageNumber */)
{
	::PrOpenPage((TPPrPort)GetJobPort(), NULL);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::FinishPage: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::FinishPage()
{
	::PrClosePage((TPPrPort)GetJobPort());
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::CallJobDialog: 
//----------------------------------------------------------------------------------------


bool TMacPrintJob::CallJobDialog()
{
	::ThrowIfOSErr_AC(MAInteractWithUser());
	PullApplicationToFront();

	SetCursor(&qd.arrow);
	if (gDispatcher)
		gDispatcher->InvalidateMouseRegions();

	CInMacPrint inMacPrint(this);

	gDispatcher->AboutToLoseControl(true);		// so scrap gets converted and floaters get removed
	bool result = ::PrJobDialog(GetHPrint());
	gDispatcher->RegainControl(true);			// scrap gets converted, printer gets checked, floaters get restored

	return result;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::CallStyleDialog: 
//----------------------------------------------------------------------------------------


bool TMacPrintJob::CallStyleDialog()
{
	::ThrowIfOSErr_AC(MAInteractWithUser());
	PullApplicationToFront();

	SetCursor(&qd.arrow);
	if (gDispatcher)
		gDispatcher->InvalidateMouseRegions();

	CInMacPrint inMacPrint(this);

	gDispatcher->AboutToLoseControl(true);		// so scrap gets converted and floaters get removed
	bool result = ::PrStlDialog(GetHPrint());
	gDispatcher->RegainControl(true);			// scrap gets converted, printer gets checked, floaters get restored

	return result;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::CallPrValidate: 
//----------------------------------------------------------------------------------------


bool TMacPrintJob::CallPrValidate()
{
	CInMacPrint inMacPrint(this, this);
	bool result = ::PrValidate(GetRawHPrint());

	return result;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::CallPrintDefault: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::CallPrintDefault()
{
	CInMacPrint inMacPrint(this, this);

	PrintDefault(GetRawHPrint());
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::CallPrPicFile: 
//----------------------------------------------------------------------------------------


bool TMacPrintJob::CallPrPicFile(OSErr& err)
{
	CInMacPrint inMacPrint(this, this);

	bool returnVal = true;

	TPrStatus prStatus;
	::PrPicFile(GetRawHPrint(), NULL, NULL, NULL, &prStatus);

	bool b;

	if (returnVal)
		returnVal = ChkPrintErr(err, b);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::GetDriverName: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::GetDriverName(CStr255_AC& driverName)
{
	driverName.Empty();

	String255Handle driverHandle = (String255Handle)GetString(kPrintDriverName);// Get current driver 

	if (driverHandle)
		driverName.CopyFrom(**driverHandle);
}


//----------------------------------------------------------------------------------------
// TMacPrintJob::Update:
//----------------------------------------------------------------------------------------


void TMacPrintJob::Update()
{
	// Printer name may have changed out with the old in with the new...
	fPrinterChangeLevel = fgPrinterChangeLevel;
	
	Inherited::Update();
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::DoUpdate:
//----------------------------------------------------------------------------------------


void TMacPrintJob::DoUpdate(ChangeID_AC theChange,
						 MDependable_AC* changedObject,
						 void* changeData,
						 CDependencySpace_AC* dependencySpace)// Override
{
	switch (theChange)
	{
		case mRegainControl:
			if (CheckPrinterChangeLevel() != fPrinterChangeLevel)
			{
 				Invalidate();
				Changed(mPrinterChanged, this);

			}
			break;

		default:
			Inherited::DoUpdate(theChange, changedObject, changeData, dependencySpace);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::JobMerge:
//----------------------------------------------------------------------------------------


void TMacPrintJob::JobMerge(TPrintInfo* mergeFrom)
{
	CInMacPrint inMacPrint(this);

	::PrJobMerge(dynamic_cast_AC(TMacPrintJob*,mergeFrom)->GetHPrint(), GetHPrint());
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::IdleProc: 
//----------------------------------------------------------------------------------------


pascal void TMacPrintJob::IdleProc()
{
	if (gJobPrintHandler)
	{
		CTempUseResFile_AC tempUseResFile(CurResFile_AC()); // save it so it can be restored if someone forgets

		GrafPtr savedPort;
		GetPort(&savedPort);
		SetPortWindowPort(gWorkPort);

		TPrintHandler * currentPrintHandler = TPrintHandler::gCurrPrintHandler.release();	// So we can focus when updating and activating windows
		bool printingStatus = TView::gPrinting;
		TView::gPrinting = false;

		gDispatcher->SetEventLevel(gDispatcher->GetEventLevel() + 1);				// Increment the event level for proper busy cursor handling
		dynamic_cast_AC(TMacPrintJob*,gJobPrintHandler->fPrintInfo.get())->DoPrintIdling();		// Forward to the current print job handler
		gDispatcher->SetEventLevel(gDispatcher->GetEventLevel() - 1);

		TPrintHandler::gCurrPrintHandler = currentPrintHandler;
		TView::gPrinting = printingStatus;

		SetPort(savedPort);
	}
}


//----------------------------------------------------------------------------------------
// TMacPrintJob::DoPrintIdling: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::DoPrintIdling()
{
	const short myDlgMask = mDownMask | mUpMask | keyDownMask | keyUpMask | autoKeyMask | activMask;

	TToolboxEvent * event = gDispatcher->GetEvent(myDlgMask, 0, NULL);
	if (event)
	{
		// Workaround for LaserWriter driver bug: it tries to set GhostWindow
		// so that its status window is invisible. Unfortunately, it doesn't
		// always set it, so IsDialogEvent returns false (the status window
		// is frontmost).  If ours isn't already in front, force it there.
		if ((WindowRef)fPrintDialog != FrontWindow())
			TWindow::MASelectToolboxWindow((WindowRef)fPrintDialog);// 3.5

		switch (event->GetWhat())
		{
			case keyDown:
				{
					if (((event->GetText() == '.') && event->IsCommandKeyPressed()) || ((event->GetText() == chEscape) && (event->GetKeyCode() == kEscapeVirtualCode)))
					{
						short item = 1;			// Want the 'Cancel' button 

//						if (fPrintingCommand == cFinderPrint || fPrintingCommand == cDragPrint)
//						item = cancel;		// Want the 'Cancel All' button 

						// Flash the appropriate button 
						Handle theItem;
						short itemType;
						CRect_AC box;
						GetDialogItem(fPrintDialog, item, &itemType, &theItem, box);

						HiliteControl((ControlRef)theItem, 10);

						long dontCare;
						Delay(8, &dontCare);

						HiliteControl((ControlRef)theItem, 0);

						SetError(iPrAbort);
						gCancelAllPrinting = true;
					}
					break;
				}

			default:
				{
					EventRecord anEventRecord = event->GetEventRecord(0);
					DialogRef aDialog;
					short item;
					if (IsDialogEvent(&anEventRecord) && DialogSelect(&anEventRecord, &aDialog, &item) && (aDialog == fPrintDialog))
					{
						switch (item)
						{
							case 1:
								SetError(iPrAbort);// Cancel
								break;

							case cancel:		// Cancel All Printing 
								SetError(iPrAbort);
								gCancelAllPrinting = true;
								break;
						}
					}
					else if (event->GetWhat() == mouseDown)
					{
						WindowRef aWindow;
						short itsPart = FindWindow(event->GetWhere(), &aWindow);
						if (itsPart == inDrag && aWindow == (WindowRef)fPrintDialog)
						{
							CRect_AC moveBounds(-32000, -32000, 32000, 32000);
							TWindow::MADragWindow((WindowRef)fPrintDialog, event->GetWhere(), moveBounds);
						}
					}
					break;
				}
		}
		if (event->ShouldFreeOnCompletion())
			delete event;
	}
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::PosePrintDialog: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::PosePrintDialog(const CStr255_AC& docName, CommandNumber aCommandNumber)
{
	short dlgNumber;
	DialogTHndl dlogTemplate;
	short itemNo;
	Handle theItem;
	short itemType;
	CRect_AC box;
	CStr255_AC itemText;
	short preDocName;
	short constTitle;

	::ThrowIfOSErr_AC(MAInteractWithUser());

	if (aCommandNumber == cFinderPrint || aCommandNumber == cDragPrint)
	{
		dlgNumber = phFinderPrintDialog;
		itemNo = 3;
	}
	else
	{
		dlgNumber = phSpoolPrintDialog;
		itemNo = 2;
	}

	// Can't use GetNewCenteredDialog because vertically centering the dialog may cause it
	// to interfere with the Print Manager's status windows.  Therefore, leave the vertical
	// location of the dialog fixed.

	SetCursor(&(qd.arrow));
	gDispatcher->InvalidateMouseRegions();
	dlogTemplate = (DialogTHndl)GetResource('DLOG', dlgNumber);
	if (dlogTemplate)
	{
		CenterRectOnScreen(((CRect_AC &)(*dlogTemplate)->boundsRect), true, false, false);
		fPrintDialog = GetNewDialog(dlgNumber, NULL, (WindowRef) - 1);
		::ThrowIfNULL_AC(fPrintDialog);
	}
	else
	{
#if qDebugMsg
		ProgramReport_AC("You may have forgotten to include Printing.rsrc in your .r file\n");
		::ProgramBreak_AC("The print job dialog resource can't be found.");
#endif

		::ThrowIfNULLResource_AC((Handle)dlogTemplate);
	}

	// Substitute the document name for '<<!=>>' in 'Document <<!=>> is being printed'.
	// ParamText would be a lot simpler, but the Print Mgr. also uses ParamText, and
	// the substitution doesn't happen until draw time.
	GetDialogItem(fPrintDialog, itemNo, &itemType, &theItem, &box);
	if (theItem)
	{
		GetDialogItemText(theItem, itemText);
		if (ParseTitleTemplate(itemText, preDocName, constTitle) && SubstituteInTitle(itemText, docName, preDocName, constTitle))
			SetDialogItemText(theItem, itemText);
	}

	(*GetHPrint())->prJob.pIdleProc = pPrintIdleProc;

	SetWTitle((WindowRef)fPrintDialog, docName);// In case Print Mgr. needs it. 
	DrawDialog(fPrintDialog);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::BanishPrintDialog: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::BanishPrintDialog()
{
	if (fPrintDialog)
	{
		if (GetWindowPort(fPrintDialog) == (CGrafPtr)qd.thePort)// Only need to invalidate focus if freed
		// dialog is the current port
		{
			gDispatcher->InvalidateFocus();
			SetPortWindowPort(gWorkPort);
		}
		DisposeDialog(fPrintDialog);
		fPrintDialog = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::ShowDocBeingPrinted: 
//----------------------------------------------------------------------------------------


void TMacPrintJob::ShowDocBeingPrinted(const CStr255_AC& docName, bool entering)
{
	if (entering)
	{
		fPrintDialog = GetNewDialog(phWhichDoc, NULL, (WindowRef) - 1);
		if (fPrintDialog)
		{
			SetWTitle((WindowRef)fPrintDialog, docName);
			DrawDialog(fPrintDialog);
		}
#if qDebug
		else
		{
			CStr255_AC theString;
			ConcatNumber("Unable to load dialog ", phWhichDoc, theString);
			::ProgramBreak_AC(theString);
		}
#endif

	}
	else
		BanishPrintDialog();
}


//----------------------------------------------------------------------------------------
// TMacPrintJob::GetMaxSquareResolution:
//----------------------------------------------------------------------------------------
void TMacPrintJob::GetMaxSquareResolution(Fixed& hRes, Fixed& vRes)
{
	CInMacPrint inMacPrint(this);

	TGetRslBlk aGetRslData;
	aGetRslData.iOpCode = getRslDataOp;
	aGetRslData.iError = 0;

	::PrGeneral((Ptr) & aGetRslData);
	if (GetError() == noErr)
	{
		short dpi = 0;
		for (short i = 0; i < aGetRslData.iRslRecCnt; ++i)
			if (aGetRslData.rgRslRec[i].iXRsl == aGetRslData.rgRslRec[i].iYRsl)
			// only square 
				dpi = Max_AC(dpi, aGetRslData.rgRslRec[i].iXRsl);

		hRes = dpi;
		vRes = dpi;
	}
	else
		Inherited::GetMaxSquareResolution(hRes, vRes);
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::SetResolution:
//----------------------------------------------------------------------------------------
void TMacPrintJob::SetResolution(Fixed hRes, Fixed vRes)
{
	TSetRslBlk aSetRslData;
	aSetRslData.iOpCode = setRslOp;
	aSetRslData.iError = 0;
	aSetRslData.hPrint = GetHPrint();
	aSetRslData.iXRsl = hRes;
	aSetRslData.iYRsl = vRes;

	::PrGeneral((Ptr) & aSetRslData);
	::ThrowIfOSErr_AC(GetError());

	Invalidate();
}

//----------------------------------------------------------------------------------------
// TMacPrintJob::MaximizeResolution:
//----------------------------------------------------------------------------------------
void TMacPrintJob::MaximizeResolution()
{
	// Thank you, A.D.
	CInMacPrint inMacPrint(this); // for grouping the two calls together

	long hRes;
	long vRes;
	GetMaxSquareResolution(hRes, vRes);
	SetResolution(hRes, vRes);
}

//----------------------------------------------------------------------------------------
// TPrintInfo::GetScale:
//----------------------------------------------------------------------------------------

short TMacPrintJob::GetScale()
{
	CViewRect printingRect(GetPaperRect());
	VHSelect longerDirection = LongerSide(printingRect);
	ViewCoordinate projectionPaperSize = printingRect.GetSize()[longerDirection];

	//
	// NOTE: iPageV/H are currently undocumented but have a long history of safe usage. I would gladly
	// use any other means to calculate the scaling percentage if it was available. Any ideas?
	//
	short iPageV = (*GetHPrint())->prStl.iPageV;
	short iPageH = (*GetHPrint())->prStl.iPageH;
	
	short paperSizeIn_iPrPgFract = Max_AC(iPageH, iPageV);
	short paperSizeInDeviceResDPI = (paperSizeIn_iPrPgFract * GetDeviceRes()[longerDirection]) / iPrPgFract;
	
	return (paperSizeInDeviceResDPI * 100) / projectionPaperSize;
}

//========================================================================================
// CLASS TStdPrintHandler
//========================================================================================
#undef Inherited
#define Inherited TPrintHandler


MA_DEFINE_CLASS(TStdPrintHandler);

//----------------------------------------------------------------------------------------
// TStdPrintHandler constructor
//----------------------------------------------------------------------------------------


TStdPrintHandler::TStdPrintHandler(TDocument* itsDocument,
										TView* itsView,
										bool itsHFixedSize,
										bool itsVFixedSize) :

	TPrintHandler(itsView),						// sets fView to itsView

	fLastStrip(CViewPoint(SHRT_MAX, SHRT_MAX)),
	fPrintingCommand(cNoCommand),				// Will be set in DoPrintCommand
	fStartPage(1),
	fPageDirection(vSel),						// Page 2 is below page 1, etc. 

	fFinderSetup(false),
	fFinderJobDialog(false),
	fMinimalMargins(false),
	fUseMaxResolution(false),

	fMarginsRect(gStdPageMargins),
	fScale(100),

	fIsValid(false)
{
	fFixedSizePages[hSel] = itsHFixedSize;
	fFixedSizePages[vSel] = itsVFixedSize;

	fDocument = itsDocument;
	try
	{
		if (fView)
		{
			SetDefaultPrintInfo();

			if (itsDocument)
				itsDocument->AttachPrintHandler(this);

			fView->AttachPrintHandler(this);

			fPrintInfo->AddDependent(this);
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		// We may have failed AFTER attaching the printhandler to the view.  If we were
		// to free ourself without detaching we would leave the view with a
		// reference to a freed behavior.  So remove ourselves before freeing.  We don't
		// have to worry about detaching from the document since that will occur
		// in our free method. In the same vein, RemoveBehavior will call SetOwner
		// which will detach us from the window. 
		if (fView)
			fView->RemoveBehavior(this);

		throw;
	}
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::~TStdPrintHandler: 
//----------------------------------------------------------------------------------------


TStdPrintHandler::~TStdPrintHandler()
{
	bool dontDispose = fView != NULL;
	if (dontDispose)
	{
		// If you are freeing a print handler, it is up to you to detach it
		// from the view first.

		fView->DeleteAdorner(&TAdorner::fgPrintAdorner, false);
		fView = NULL;

		if (fDocument)
		{
			fDocument->DetachPrintHandler(this);
			dontDispose = fDocument->GetSharePrintInfo() && (fDocument->GetPrintInfo() == fPrintInfo);
		}
		else
			dontDispose = false;
		fDocument = NULL;
	}

	if (fPrintInfo)
		fPrintInfo->BanishPrintDialog();

	if (dontDispose)
		fPrintInfo.release();

	if (gJobPrintHandler == this)				// let's be safe 
		gJobPrintHandler = NULL;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::ChkPrintErr: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::ChkPrintErr(OSErr& err,
									  bool& ranOutOfSpace)
{
	return fPrintInfo->ChkPrintErr(err, ranOutOfSpace);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::PrintSpoolFile: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::PrintSpoolFile(OSErr& err)
{
	return fPrintInfo->CallPrPicFile(err);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandle copy constructor
//----------------------------------------------------------------------------------------


TStdPrintHandler::TStdPrintHandler(const TStdPrintHandler& inStdPrintHandler) :
	fPrintExtent(inStdPrintHandler.fPrintExtent),
	fViewedRect(inStdPrintHandler.fViewedRect),
	fViewToQDOffset(inStdPrintHandler.fViewToQDOffset),
	fPageStrips(inStdPrintHandler.fPageStrips),
	fLastStrip(inStdPrintHandler.fLastStrip),
	fLastBreak(inStdPrintHandler.fLastBreak),
	fQDRectToClipTo(inStdPrintHandler.fQDRectToClipTo),
	fQDOrigin(inStdPrintHandler.fQDOrigin),
	fPrintingCommand(inStdPrintHandler.fPrintingCommand),
	fStartPage(inStdPrintHandler.fStartPage),
	fPageDirection(inStdPrintHandler.fPageDirection),

	fFinderSetup(inStdPrintHandler.fFinderSetup),
	fFinderJobDialog(inStdPrintHandler.fFinderJobDialog),
	fMinimalMargins(inStdPrintHandler.fMinimalMargins),
	fUseMaxResolution(inStdPrintHandler.fUseMaxResolution),

	fPaperRect(inStdPrintHandler.fPaperRect),
	fInkRect(inStdPrintHandler.fInkRect),
	fMarginsRect(inStdPrintHandler.fMarginsRect),
	fInteriorRect(inStdPrintHandler.fInteriorRect),
	fScale(inStdPrintHandler.fScale),

	fIsValid(inStdPrintHandler.fIsValid)
{
	fFixedSizePages[hSel] = inStdPrintHandler.fFixedSizePages[hSel];
	fFixedSizePages[vSel] = inStdPrintHandler.fFixedSizePages[vSel];

	if (inStdPrintHandler.fPrintInfo)
		fPrintInfo = static_cast<TPrintInfo*>(inStdPrintHandler.fPrintInfo->Clone());
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::AdornPage: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::AdornPage()
{
#if qDebug
	if (gDebugPrinting)							// Print extra stuff if debugging 
	{
		const short botSlop = 8;

		CViewRect thePaperRect(GetPaperRect());
		long rPlusL = thePaperRect.right + thePaperRect.left;

		long itsBottom = GetInkRect().bottom - botSlop;

		// draw the heading
		TextFont(applFont);
		TextFace(normal);
		TextSize(12);
		FontInfo theFontInfo;
		GetFontInfo(&theFontInfo);

		CStr255_AC pgStr;
		NumToString(fFocusedPage, pgStr);
		CStr255_AC heading = "-" + pgStr + "-";

		long itsWidth = StringWidth(heading);

		CViewRect theTextRect;
		theTextRect.left = (rPlusL - itsWidth) / 2;
		theTextRect.top = itsBottom - theFontInfo.ascent;
		theTextRect.right = theTextRect.left + itsWidth;
		theTextRect.bottom = itsBottom + theFontInfo.descent;
		MADrawString(heading, theTextRect.ToRect(), teFlushDefault);

		// Additionally frame the printable area of the page if gDebugPrinting 
		PenSize(1, 1);
		FrameRect(&fView->ViewToQD(GetInkRect()));

		// Frame the 'interior' of the page 
		PenSize(2, 2);
		FrameRect(&fView->ViewToQD(GetInteriorRect()));
	}
#endif

}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::BreakFollowing: Called from fView->DoBreakFollowing(),
//----------------------------------------------------------------------------------------


ViewCoordinate TStdPrintHandler::BreakFollowing(VHSelect vhs,
											 ViewCoordinate previousBreak,
											 bool& automatic)// override 
{
	VHSelect orthoVHS = gOrthogonal[vhs];
	automatic = true;
	ViewCoordinate newLoc = Min_AC(previousBreak + GetViewPerPage()[orthoVHS], fPrintExtent[botRight][orthoVHS]);

#if qDebugMsg
	if (newLoc <= previousBreak)
	{
		ProgramReport_AC(false, "No advance in BreakFollowing; vhs: %1d previousBreak: %1d newLoc: %1d view size: \n", static_cast<short>(vhs), previousBreak, newLoc, fPrintExtent[botRight][orthoVHS]);
		::ProgramBreak_AC("No advance in BreakFollowing");
	}
#endif

	return newLoc;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::CalcPageStrips: 
//----------------------------------------------------------------------------------------
CViewPoint TStdPrintHandler::CalcPageStrips()// override 
{
	CViewPoint pageStrips;
	// If pages are of fixed size, then simple divide the total size by the
	// page size.  Otherwise, count up the page breaks one by one.

	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		VHSelect ortho = gOrthogonal[vhs];
		if (fFixedSizePages[ortho])
			pageStrips[vhs] = (fPrintExtent[botRight][ortho] - fPrintExtent[topLeft][ortho] + GetViewPerPage()[ortho] - 1) / GetViewPerPage()[ortho];
		else
		{
			ViewCoordinate previousBreak = 0;

			long nStrips = 0;
			ViewCoordinate startLoc = fPrintExtent[topLeft][gOrthogonal[vhs]];
			ViewCoordinate endLoc = fPrintExtent[botRight][gOrthogonal[vhs]];

			ViewCoordinate loc = startLoc;
			bool automatic = true;
			bool done = false;
			while (loc < endLoc)
			{
				if (loc != startLoc)
					++nStrips;

#if qDebug
				previousBreak = loc;
#endif

				loc = fView->DoBreakFollowing(vhs, loc, automatic);

#if qDebug
				if (loc <= previousBreak)
					::ProgramBreak_AC("thisBreak (loc) <= previousBreak");// Thanks much to Larry T. ! 
#endif

			}

			if (loc != startLoc)
				++nStrips;

			pageStrips[vhs] = nStrips;
		}
	}

	return pageStrips;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::CalcViewPerPage: 
//----------------------------------------------------------------------------------------


CViewPoint TStdPrintHandler::CalcViewPerPage()// override 
{
	return (GetPaperRect() + GetMarginsRect()).GetSize();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::Invalidate:
//----------------------------------------------------------------------------------------


void TStdPrintHandler::Invalidate()
{
	fIsValid = false;

	//SRF get a message assigned!!!	Changed(mPrinterChanged, this);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::Update: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::Update()
{
	if (!fIsValid)
	{
		fPrintInfo->Update();

		fIsValid = true;

		CPoint_AC oldDevRes(fDeviceRes);
		CViewRect oldPaper(fPaperRect);
		CViewRect oldInk(fInkRect);
		CViewRect oldMargins(fMarginsRect);
		short oldScale(fScale);

		// Store latest data from fPrintInfo into my own instance variables
		fDeviceRes = fPrintInfo->GetDeviceRes();
		fPaperRect = fPrintInfo->GetPaperRect();
		fInkRect = fPrintInfo->GetInkRect();
		fScale = fPrintInfo->GetScale();

		if (fMinimalMargins)
			fMarginsRect = fInkRect - fPaperRect;
		else
			fMarginsRect.Set(
			(oldMargins.left * fDeviceRes.h) / oldDevRes.h * oldScale / fScale,
			(oldMargins.top * fDeviceRes.v) / oldDevRes.v * oldScale / fScale,
			(oldMargins.right * fDeviceRes.h) / oldDevRes.h * oldScale / fScale,
			(oldMargins.bottom * fDeviceRes.v) / oldDevRes.v * oldScale / fScale
			);

		// and while we are at it update fViewPerPage
		fViewPerPage = fView->DoCalcViewPerPage();

		if (fDeviceRes != oldDevRes || fPaperRect != oldPaper || fInkRect != oldInk || fMarginsRect != oldMargins || fScale != oldScale)
		{
			// something important to our projection model changed...
			if (fDocument && fDocument->GetSharePrintInfo())
				for (TemplateIterator_AC<TView*> iter(fDocument->GetViewList()); iter; ++iter)
					iter->DoPrinterChanged();
			else
				fView->DoPrinterChanged();
				// didn't send msg to all views, current company included,
				// so now we need to tell must my local view
		}
	}
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::ChooseSpoolFile: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::ChooseSpoolFile(CFSSpec_AC& theSpoolFile,
								 long& pagesPerSubjob)
{
	// Default choices tell Print Manager to use its standard choice algorithm
	theSpoolFile.name[0] = 0;
	theSpoolFile.vRefNum = 0;
	theSpoolFile.parID = 0;

	// Print Shop suggests attempting to print entire document.  If it
	// runs out of space, then PerformPrinting will retry with a
	// smaller number of pages.
	pagesPerSubjob = GetDefaultSubJobSize();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetPaperRect: 
//----------------------------------------------------------------------------------------


CViewRect TStdPrintHandler::GetPaperRect()
{
	Update();

	return fPaperRect;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetInkRect: 
//----------------------------------------------------------------------------------------


CViewRect TStdPrintHandler::GetInkRect()
{
	Update();

	return fInkRect;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetMarginsRect: 
//----------------------------------------------------------------------------------------


CViewRect TStdPrintHandler::GetMarginsRect()
{
	Update();

	return fMarginsRect;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetInteriorRect: 
//----------------------------------------------------------------------------------------


CViewRect TStdPrintHandler::GetInteriorRect()
{
	Update();

	return fInteriorRect;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetViewPerPage: 
//----------------------------------------------------------------------------------------


CViewPoint TStdPrintHandler::GetViewPerPage()
{
	Update();

	return fViewPerPage;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DoScriptCommand: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::DoScriptCommand(CommandNumber aCommandNumber,
										  TAppleEvent* message,
										  TAppleEvent* reply)
{
	bool wasHandled;

	if (aCommandNumber == cPrint)
	{
		bool setupSuccess;
		// if user interaction is allowed, bring up the job dialog. If no UI is allowed,
		// perform a print one.
		if (MAInteractWithUser() == noErr)
			setupSuccess = PoseJobDialog();
		else
		{
			aCommandNumber = cPrintOne;
			setupSuccess = SetupPrintOne(aCommandNumber);
		}

		if (setupSuccess)
			Print(aCommandNumber);
		wasHandled = true;
	}
	else
		wasHandled = Inherited::DoScriptCommand(aCommandNumber, message, reply);

	return wasHandled;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DoMenuCommand: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::DoMenuCommand(CommandNumber aCommandNumber)// override 
{
	if (!DoPrintCommand(aCommandNumber))
		Inherited::DoMenuCommand(aCommandNumber);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DoPrintCommand: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::DoPrintCommand(CommandNumber aCommandNumber)
{
	bool handled = true;						// assume we handle it

	fPrintingCommand = aCommandNumber;

	switch (aCommandNumber)
	{
		case cPageSetup:
			PosePageSetupDialog(true);
			break;

		case cPrint:
			{
				TPrintCommand * aPrintCommand = TH_new TPrintCommand(aCommandNumber, this);
				aPrintCommand->SetUseAppleEvent(true);
				if (fOwner)
					fOwner->PostCommand(aPrintCommand);
				else
					gDispatcher->PostCommand(aPrintCommand);
			}
			break;

		case cPrintOne:
			{
				TPrintCommand * aPrintCommand = TH_new TPrintCommand(aCommandNumber, this);
				// aPrintCommand->fUseAppleEvent = true;		// no way to record this
				if (fOwner)
					fOwner->PostCommand(aPrintCommand);
				else
					gDispatcher->PostCommand(aPrintCommand);
			}
			break;

		case cShowBreaks:						// Toggle state of "Show Breaks" 
			fShowBreaks = !fShowBreaks;
			InvalPageFeedback();				// force redraw of area the breaks did or will occupy
			break;

		case cFinderPrint:						// handle Finder printing 
			if (SetupForFinder(aCommandNumber))
				gFinderPrintingProceed = Print(cFinderPrint);
			break;

		case cDragPrint:						// handle Drag printing 
			if (SetupPrintOne(aCommandNumber))
				gFinderPrintingProceed = Print(cDragPrint);
			break;

		default:
			handled = false;
			break;
	}

	return handled;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DoSetupMenus: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::DoSetupMenus()			// override 
{
	Inherited::DoSetupMenus();

	DoSetupPrintMenus();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DoSetupPrintMenus: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::DoSetupPrintMenus()
{
	if (gCouldPrint && fView && !MemSpaceIsLow())
	{
		Enable(cPageSetup, true);
		Enable(cPrint, true);
		Enable(cPrintOne, true);
		Enable(cPrintSelection, fView != NULL);
	}
	EnableCheck(cShowBreaks, true, fShowBreaks);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DrawPrintFeedback: Draws page breaks and page numbers
//----------------------------------------------------------------------------------------

void TStdPrintHandler::DrawPrintFeedback(const CViewRect& area)// override 
{
#if qDebug
	fView->AssumeFocused();
#endif

	if (fShowBreaks || gDebugPrinting)
	{
		SetPrintExtent();						// Make sure print extent is accurate before starting

#if qDebug
		if (gDebugPrinting)						// Now draw Page numbers in the corners of pages, if desired
			SetPortTextStyle(CTextStyle_AC(applFont, bold, 9, gRGBBlack));
#endif

		SetPenState(&gBreaksPenState);

		for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		{
			VHSelect orthoVHS = gOrthogonal[vhs];
			short whichBreak = 0;
			ViewCoordinate previousBreak = 0;
			bool automatic = true;
			bool done = false;

			ViewCoordinate startLoc = fPrintExtent[topLeft][gOrthogonal[vhs]];
			ViewCoordinate endLoc = fPrintExtent[botRight][gOrthogonal[vhs]];
			ViewCoordinate loc = startLoc;

			while ((loc < endLoc) && !done)
			{
				if (loc != startLoc)
				{
					if (loc > area[botRight][orthoVHS])
						done = true;
					else
					{
						++whichBreak;
						if (loc > (area[topLeft][orthoVHS] - ((CPoint_AC &)gBreaksPenState.pnSize)[orthoVHS]))
							fView->DoDrawPageBreak(vhs, whichBreak, loc, automatic);
						done = false;
					}
				}

#if qDebug
				previousBreak = loc;
#endif

				loc = fView->DoBreakFollowing(vhs, loc, automatic);

#if qDebug
				if (loc <= previousBreak)
					::ProgramBreak_AC("thisBreak (loc) <= previousBreak");// Thanks much to Larry T. ! 
#endif

			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DrawPageBreak: 
//----------------------------------------------------------------------------------------


#if qDebug
void TStdPrintHandler::DrawPageBreak(VHSelect vhs,
									 long whichBreak,
									 ViewCoordinate loc,
									 bool)	// override 
#else
void TStdPrintHandler::DrawPageBreak(VHSelect vhs,
									 long		/* whichBreak */,
									 ViewCoordinate loc,
									 bool)	// multiple declarations to eliminate compiler warnings 
#endif

{
	CViewPoint vPt;
	vPt[gOrthogonal[vhs]] = loc;

	// Get the origin of the view, while accounting for any translation.
	vPt[vhs] = fView->GetExtent()[topLeft][vhs];

	CPoint_AC qdStartPt = fView->ViewToQD(vPt);
	vPt[vhs] = fView->GetSize()[vhs];
	CPoint_AC qdEndPt = fView->ViewToQD(vPt);
	qdEndPt[vhs] -= ((CPoint_AC &)gBreaksPenState.pnSize)[vhs];
	// Subtract the pen size _AFTER__ converting to qd coords. Thank you L.J.!

	if (fShowBreaks)
	{
		::MoveToPt(qdStartPt);
		::LineToPt(qdEndPt);
	}

#if qDebug
	if (gDebugPrinting)
		if (vhs == hSel)
		{
			ViewCoordinate hLoc;
			CStr255_AC aString;
			FontInfo theFontInfo;
			CRect_AC theTextRect;

			for (long i = 0; i <= fPageStrips.v; ++i)
			{
				if (i)
					GetBreakCoord(vSel, i, hLoc);
				else
					hLoc = 0;

				GetFontInfo(&theFontInfo);

				NumToString(StripToPage(CViewPoint(whichBreak - 1, i)), aString);
				// top computed based on the bottom - text height
				theTextRect.Set(static_cast<short>(hLoc) + 3, qdStartPt.v - 3 - theFontInfo.ascent, static_cast<short>(hLoc) + 3 + StringWidth(aString), qdStartPt.v - 3 + theFontInfo.descent);
				MADrawString(aString, theTextRect, teFlushDefault);

				NumToString(StripToPage(CViewPoint(whichBreak, i)), aString);
				// top computed based on the bottom - text height
				theTextRect.Set(static_cast<short>(hLoc) + 3, qdEndPt.v + 10 - theFontInfo.ascent, static_cast<short>(hLoc) + 3 + StringWidth(aString), qdEndPt.v + 10 + theFontInfo.descent);
				MADrawString(aString, theTextRect, teFlushDefault);

				NumToString(StripToPage(CViewPoint(whichBreak - 1, i - 1)), aString);
				// top computed based on the bottom - text height
				theTextRect.Set(static_cast<short>(hLoc) - StringWidth(aString) - 3, qdStartPt.v - 3 - theFontInfo.ascent, static_cast<short>(hLoc) - 3, qdStartPt.v - 3 + theFontInfo.descent);
				MADrawString(aString, theTextRect, teFlushDefault);

				NumToString(StripToPage(CViewPoint(whichBreak, i - 1)), aString);
				// top computed based on the bottom - text height
				theTextRect.Set(static_cast<short>(hLoc) - StringWidth(aString) - 3, qdEndPt.v + 10 - theFontInfo.ascent, static_cast<short>(hLoc) - 3, qdEndPt.v + 10 + theFontInfo.descent);
				MADrawString(aString, theTextRect, teFlushDefault);
			}
		}
#endif

}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DrawPageInterior: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::DrawPageInterior()
{
	// i.e., by default, the same code used for drawing on the screen
	fView->HandleDraw(fView->QDToView(fQDRectToClipTo));
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::Focus: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::Focus()				// override
{
	bool returnVal = false;

	GrafPtr printingPort = fPrintInfo->GetJobPort();
	if (printingPort)
	{
		SetPort(printingPort);

		SetOrigin(fQDOrigin.h, fQDOrigin.v);
		CRect_AC clipTo = fQDRectToClipTo;
		ClipRect(clipTo);
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::FocusOnBorder: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::FocusOnBorder()
{
	CViewRect rectToClipTo(GetInkRect());

	// Only works for newer LaserWriter drivers 
	fQDOrigin = rectToClipTo[topLeft].ToPoint();
	SetOrigin(fQDOrigin.h, fQDOrigin.v);
	fView->InvalidateCoordinates();				// Need to update the coordinates to reflect the
	// change in the port's origin.

	fQDRectToClipTo = fView->ViewToQD(rectToClipTo);
	ClipRect(&fQDRectToClipTo);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::FocusOnInterior: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::FocusOnInterior()		// override 
{
	const short kMaxOriginFixup = 1024;

	CViewPoint theVOrigin;

	CViewRect aRect = GetInkRect();
	CViewPoint theOrigin = aRect[topLeft];
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		if (fView->GetSize()[vhs] > kMaxQDCoord)
			theVOrigin[vhs] = gPageOffset[vhs];
		else
		{
			theVOrigin[vhs] = 0;
			theOrigin[vhs] += gPageOffset[vhs];
		}
		aRect[topLeft][vhs] += gPageOffset[vhs];
		aRect[botRight][vhs] += gPageOffset[vhs];
	}

	if (CViewRect(-kMaxQDCoord, -kMaxQDCoord, kMaxQDCoord, kMaxQDCoord).Contains(fView->GetExtent()))
	{
		fQDOrigin = theOrigin.ToPoint();
		fViewToQDOffset = theVOrigin;
	}
	else
	{
		CPoint_AC deltaOrigin(static_cast<short>(theVOrigin.h % kMaxOriginFixup), static_cast<short>(theVOrigin.v % kMaxOriginFixup));
		fViewToQDOffset = theVOrigin - CViewPoint(deltaOrigin);
		fQDOrigin = deltaOrigin;
	}
	SetOrigin(fQDOrigin.h, fQDOrigin.v);
	fView->InvalidateCoordinates();				// Need to update the coordinates to reflect the
	// change in the port's origin.

	// Clip the page to the macroIntersection of the visible part of the view and the
	// printable area of the page.  Note that in some cases (e.g., a WYSIWYG
	// word processor which showed the complete page margin) parts of
	// the interior might lie outside theInk.  We must clip to theInk to
	// avoid slowing down PostScript printers.
	fQDRectToClipTo = fView->ViewToQD(fViewedRect & aRect);
	ClipRect(&fQDRectToClipTo);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetDefaultSubJobSize: 
//----------------------------------------------------------------------------------------


long TStdPrintHandler::GetDefaultSubJobSize()
{
	return iPFMaxPgs;
	//SRF - is there any subjobbing necessary with GX, ever?
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetQDOrigin: 
//----------------------------------------------------------------------------------------


CPoint_AC TStdPrintHandler::GetQDOrigin()			// override
{
	return fQDOrigin;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetViewToQDOffset: 
//----------------------------------------------------------------------------------------


CViewPoint TStdPrintHandler::GetViewToQDOffset()// override
{
	return fViewToQDOffset;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetBreakCoord: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::GetBreakCoord(VHSelect vhs,
									 long whichBreak,
									 ViewCoordinate& loc)
{
	bool automatic;
	long startBreak;
	VHSelect orthoVHS;
	ViewCoordinate previousBreak = 0;


	orthoVHS = gOrthogonal[vhs];
	if (fFixedSizePages[orthoVHS])
		loc = fPrintExtent[topLeft][orthoVHS] + GetViewPerPage()[orthoVHS] * whichBreak;
	else if (whichBreak == fLastStrip[vhs])
		loc = fLastBreak[vhs];
	else
	{
		if (whichBreak > fLastStrip[vhs])
		{
			startBreak = fLastStrip[vhs] + 1;
			loc = fLastBreak[vhs];
		}
		else
		{
			startBreak = 1;
			loc = fPrintExtent[topLeft][gOrthogonal[vhs]];
		}

		for (long i = startBreak; i <= whichBreak; ++i)
		{
#if qDebug
			previousBreak = loc;
#endif

			loc = fView->DoBreakFollowing(vhs, loc, automatic);

#if qDebug
			if (loc <= previousBreak)
				::ProgramBreak_AC("thisBreak (loc) <= previousBreak");
#endif

		}
	}
	loc = Min_AC(loc, fPrintExtent[botRight][orthoVHS]);

	fLastStrip[vhs] = whichBreak;
	fLastBreak[vhs] = loc;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetDocName: 
//----------------------------------------------------------------------------------------


CStr255_AC TStdPrintHandler::GetDocName()
{
	CStr255_AC docName;

	if (fDocument)
		docName = fDocument->GetTitle();

	if (docName.IsEmpty())
	{
		TWindow * aWindow = fView->GetWindow();
		if (aWindow)
			docName = aWindow->GetTitle();
	}

#if qDebug
	if (docName.IsEmpty())
		::ProgramBreak_AC("GetDocName can't get a document or window name");
#endif

	return docName;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetPrintInfo: 
//----------------------------------------------------------------------------------------


TPrintInfo* TStdPrintHandler::GetPrintInfo()
{
	return fPrintInfo;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::InstallMargins: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::InstallMargins(const CViewRect& newMargins,
									  bool areMinimalMargins)
{
	if (fMarginsRect != newMargins || fMinimalMargins != areMinimalMargins)
	{
		fMinimalMargins = areMinimalMargins;

		if (fMinimalMargins)
		{
			CViewRect theInkRect = GetInkRect();
			fInteriorRect = theInkRect;
			fMarginsRect = theInkRect - GetPaperRect();
		}
		else
		{
			CViewRect theInterior = GetPaperRect() + newMargins;
			if (theInterior.Empty())
			{
#if qDebug
				if (gDebugPrinting)
					ProgramReport_AC("margins too big for paper!\n");
#endif
				CViewRect copyMargins(newMargins);
				MarginsTooBig(copyMargins);

				InstallMargins(copyMargins, areMinimalMargins);// try again
			}
			else
			{
				fMarginsRect = newMargins;
				fInteriorRect = theInterior;
			}
		}

		Invalidate();
	}
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::MarginsTooBig: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::MarginsTooBig(CViewRect& margins)
{
	// if you wanted, you could put up an alert here, or change the margins differently.
	// We'll set the margins to the minimal amount, as they would be set with minimal margins

	margins = GetInkRect() - GetPaperRect();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::InvalPageFeedback: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::InvalPageFeedback()
{
	if (ShowsOnScreen())
		fView->ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::LocatePageInterior: 
//----------------------------------------------------------------------------------------


CViewPoint TStdPrintHandler::LocatePageInterior(long /* pageNumber */)// override 
{
	return GetPaperRect()[topLeft] + GetMarginsRect()[topLeft];
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::MaxPageNumber: 
//----------------------------------------------------------------------------------------


long TStdPrintHandler::MaxPageNumber()			// override 

{
	// respect the maximum page number supported by the printing manager
	
	Update(); // Ensure the fPageStrips is valid!!

	return Min_AC(fStartPage + (fPageStrips.v * fPageStrips.h) - 1, iPrPgMax);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::OneSubJob: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::OneSubJob(long subjobFirstPage,
									long subjobLastPage,
									long minFromPage,
									long maxToPage,
									bool justSpool,
									bool		/* partialJob */,
									bool& ranOutOfSpace,
									long& lastPageTried)
{
	bool proceed = true;

#if qDebugMsg
	if (gDebugPrinting)
		ProgramReport_AC(false, "OneSubJob entered for pages %1d through %1d, proceed=%s\n", subjobFirstPage, subjobLastPage, proceed ? "true" : "false");
#endif

	ranOutOfSpace = false;
	lastPageTried = subjobFirstPage - 1;

	fPrintInfo->SetPrintPages(subjobFirstPage, subjobLastPage, minFromPage, maxToPage);

	long numberOfTimesToImage = fPrintInfo->IsSpooling() ? 1 : fPrintInfo->GetPrintCopies();

	fPrintInfo->StartJob(GetDocName(), subjobLastPage - subjobFirstPage + 1);

	OSErr err;
	if (proceed)
		proceed = ChkPrintErr(err, ranOutOfSpace);

	if (proceed)
	{
		fView->InvalidateFocus();
		SetPort(fPrintInfo->GetJobPort());

		gCurrPrintHandler = this;
		TView::gPrinting = true;

		fView->BeInPort(fPrintInfo->GetJobPort());

		if (!fView->Focus())
		{
#if qDebug
			::ProgramBreak_AC("Can't focus view while printing");
#endif
		}

		for (long pass = 1; pass <= numberOfTimesToImage; ++pass)
			for (long aPageNumber = subjobFirstPage; aPageNumber <= subjobLastPage; ++aPageNumber)
				if (proceed)
				{
					lastPageTried = aPageNumber;

					try
					{
						PrintPage(aPageNumber);
					}
					catch(CException_AC& theException)
					{
						DoCatchMessage_AC(theException);
						proceed = false;
						err = theException.GetError();		// pass along client's error code
					}
					

					if (proceed)
						proceed = ChkPrintErr(err, ranOutOfSpace);
				}

		TView::gPrinting = false;
		gCurrPrintHandler.release();
		fView->InvalidateFocus();
		fView->BeInPort(fView->GetGrafPort());
	}

	fPrintInfo->FinishJob();			// This will close the port! 

	if (proceed)
		proceed = ChkPrintErr(err, ranOutOfSpace);

	if (!ranOutOfSpace)
	{
		if (proceed)
		{
			if (fPrintInfo->IsSpooling() && !justSpool)
				proceed = PrintSpoolFile(err);
		}
		if ((!proceed) && (err != iPrAbort))
			::ThrowException_AC(err, 0);
	}

	return proceed;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::PageToStrip: 
//----------------------------------------------------------------------------------------


CViewPoint TStdPrintHandler::PageToStrip(long pageNumber)
{
	long normalizedPageNum;
	VHSelect ortho;
	CViewPoint strip;

	normalizedPageNum = pageNumber - fStartPage + 1;

	ortho = gOrthogonal[fPageDirection];

	CViewPoint pageStrip;
	pageStrip[ortho] = (normalizedPageNum - 1) / fPageStrips[ortho];
	pageStrip[fPageDirection] = normalizedPageNum - ((pageStrip[ortho]) * fPageStrips[ortho]) - 1;
	
	return pageStrip;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::PoseJobDialog: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::PoseJobDialog()
{
	// PrepareForDialog;
	bool proceed = fPrintInfo->CallJobDialog();

	OSErr err;
	bool b;
	if (proceed)
		proceed = ChkPrintErr(err, b);

	// Rectify the range as a public service if needed
	long firstPage;
	long lastPage;
	long minPage;
	long maxPage;
	fPrintInfo->GetPrintPages(firstPage, lastPage, minPage, maxPage);
	if (firstPage > lastPage)
		fPrintInfo->SetPrintPages(firstPage, lastPage, minPage, maxPage);

	gDispatcher->UpdateAllWindows();

	return proceed;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::PosePageSetupDialog: 
//----------------------------------------------------------------------------------------

// Needs to be here because it may be called from Print

bool TStdPrintHandler::PosePageSetupDialog(bool isUndoable)
{
	bool returnVal = true;					// proceed

	// Must setup command before putting up page setup dialog because the
	// command records the current print record to make it undoable.
	CAutoPtr_AC<TPrintStyleChangeCommand> aPrintStyleChangeCommand(TH_new TPrintStyleChangeCommand(this, isUndoable));

	// Put up the Page Setup Dialog 
	returnVal = fPrintInfo->CallStyleDialog();

	if (returnVal)								// User specified a change 
	{
		aPrintStyleChangeCommand->fNewPrintInfo->SetFrom(fPrintInfo);
		if (fView)
			fView->PostCommand(aPrintStyleChangeCommand.release());
		else
			gDispatcher->PostCommand(aPrintStyleChangeCommand.release());
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::Print: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::Print(CommandNumber itsCommandNumber)// override 
{
	bool active = true;

#if qGXPrinting
	active = !gUsingGXPrinting;
#endif
	CInMacPrint inMacPrint(fPrintInfo, fPrintInfo, active);

	gCancelAllPrinting = false;

	SetPrintExtent();							// Make sure we've got the right area. 

	try
	{
		bool proceedWithJob = true;
		bool justSpool = (itsCommandNumber == cPrintToFile);

		gJobPrintHandler = this;				// be visible to the idleProc 

		CViewPoint pageStrips;
		fPageStrips = fView->DoCalcPageStrips();

		long firstPage;
		long lastPage;
		long aMinPage;
		long aMaxPage;
		fPrintInfo->GetPrintPages(firstPage, lastPage, aMinPage, aMaxPage);
		if (!lastPage)
			lastPage = aMaxPage;

		firstPage = Max_AC(firstPage, fStartPage);
		lastPage = Min_AC(lastPage, MaxPageNumber());

		if (lastPage < firstPage)
		{
			OSErr anErr;
			anErr = MAInteractWithUser();
			if (anErr == noErr)
				anErr = Alert(phNoPages, NULL);
			else if (anErr == errAENoUserInteraction)
				::ThrowIfOSErr_AC(errNoPagesToPrint);
			else
				::ThrowIfOSErr_AC(anErr);
		}
		else
		{
			long pagesPerSubjob = GetDefaultSubJobSize();
			long totalPages = lastPage - firstPage + 1;

			if (fPrintInfo->IsSpooling())
			{
				CFSSpec_AC spoolFile;
				ChooseSpoolFile(spoolFile, pagesPerSubjob);
				// if justSpool is true, then the spool filename and vRefNum will already
				// have been stuffed into the prJob record before this method is called
				if (!justSpool && spoolFile.name[0])
					fPrintInfo->SetSpoolFile(spoolFile);;
			}

			long lastPrinted = firstPage - 1;

			pagesPerSubjob = Min_AC(pagesPerSubjob, totalPages);

			fPrintInfo->PosePrintDialog(GetDocName(), itsCommandNumber);

			try
			{
				do
				{
					long firstSubJobPage = lastPrinted + 1;
					bool ranOutOfSpace = false;
					long lastAttempted;
					proceedWithJob = OneSubJob(firstSubJobPage, Min_AC(firstSubJobPage + pagesPerSubjob - 1, lastPage), fStartPage, MaxPageNumber(), justSpool, (pagesPerSubjob < totalPages), ranOutOfSpace, lastAttempted);
					if (proceedWithJob)
						lastPrinted = lastAttempted;
					if (ranOutOfSpace)
					{
						pagesPerSubjob = lastAttempted - 1 - firstSubJobPage;
						proceedWithJob = true;
					}
				} while ((lastPrinted != lastPage) && (pagesPerSubjob >= 1) && proceedWithJob);

				if (pagesPerSubjob < 1)
					::ThrowException_AC(errSpooling, 0);
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				fPrintInfo->BanishPrintDialog();
				throw;
			}
			

			fPrintInfo->BanishPrintDialog();
		}
		gJobPrintHandler = NULL;
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);

		// Certain Print Manager errors should not result in any alert,
		// since the Print Manager will have already put one up.
		// !!! sure would be nice if there were constants for these in the headers!
		if ((theException.GetError() >= -8160) && (theException.GetError() <= -8150))
			::ThrowException_AC(0, messagePrintFailed);

		if (!theException.GetMessage())
			gErrorParm3 = GetDocName();

		::ThrowNewMessage_AC(	theException.GetError(), 
								theException.GetMessage(), 
								messagePrintFailed);
	}
	
	return !gCancelAllPrinting;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::PrinterChanged: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::PrinterChanged()			// override 
{
	fView->DoPagination();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::PrintPage: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::PrintPage(long aPageNumber)// Print a single page 
{
	SetPage(aPageNumber);						// gets gPage set up correctly for coordinate transformations

	try
	{
#if qDebug
		GrafPtr aPort;
#endif
		fPrintInfo->StartPage(aPageNumber);
		::ThrowIfOSErr_AC(fPrintInfo->GetError());

		FocusOnInterior();
		DrawPageInterior();
#if qDebug
		GetPort(&aPort);
		if (aPort != fPrintInfo->GetJobPort())
			::ProgramBreak_AC("The view's DrawPageInterior method changed the grafPort");
#endif
		::ThrowIfOSErr_AC(fPrintInfo->GetError());
		FocusOnBorder();
		AdornPage();
		::ThrowIfOSErr_AC(fPrintInfo->GetError());
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		fPrintInfo->FinishPage();
		throw;
	}
	

	fPrintInfo->FinishPage();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::RedoPageBreaks: 
//----------------------------------------------------------------------------------------


// Called from fView.DoPagination. 
void TStdPrintHandler::RedoPageBreaks()			// override 
{
	bool worryAboutBreaks = (fView->GetGrafPort()) && gInitialized && (fShowBreaks || gDebugPrinting);

	if (worryAboutBreaks)
		InvalPageFeedback();					// invalidate old page breaks, if relevant 

	SetPrintExtent();
	CViewRect oldInterior(GetInteriorRect());
	CViewPoint oldViewPerPage(GetViewPerPage());

	SetMargins();								// will synch with fViewPerPage automatically

	SetPageInterior(kUsualPages);
	CViewRect newInterior = GetInteriorRect();

#if qDebug
	if (gDebugPrinting)
		if (oldInterior != newInterior)
			ProgramReport_AC("Setting new interior");
#endif

	if (oldInterior != newInterior)
		fView->PageInteriorChanged(newInterior);

#if qDebug
	if (gDebugPrinting)
		if (oldViewPerPage != GetViewPerPage())
			ProgramReport_AC("Setting new view per page");
#endif

	if ((oldInterior != newInterior) || (oldViewPerPage != GetViewPerPage()))
		fView->AdjustFrame();

	fPageStrips = fView->DoCalcPageStrips();

	if (worryAboutBreaks)
		InvalPageFeedback();					// force redraw of new page breaks, if relevant
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::Reset: 
//----------------------------------------------------------------------------------------
void TStdPrintHandler::Reset()
{
	if (fPrintInfo)
		fPrintInfo->SetToDefault();
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetPrintExtent: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::SetPrintExtent()
{
	CViewRect printExtent(fView->GetPrintExtent());

	// Make sure the bottom or right is not smaller than the top or left.
	// This can happen when views are being created and before the window
	// has been resized to its actual size.
	printExtent.bottom = Max_AC(printExtent.bottom, printExtent.top);
	printExtent.right = Max_AC(printExtent.right, printExtent.left);
	fPrintExtent = printExtent;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetDefaultPrintInfo: 
//----------------------------------------------------------------------------------------


void TStdPrintHandler::SetDefaultPrintInfo()
{
	if (!fDocument || (fDocument && !fDocument->GetSharePrintInfo()))
		fPrintInfo.reset();
	fPrintInfo = NULL;

	if (fView && fDocument && (fDocument->GetSharePrintInfo() && fDocument->GetPrintInfo()))
		fPrintInfo = fDocument->GetPrintInfo();
	else
	{
		fPrintInfo = static_cast<TPrintInfo*>(gDefaultPrintInfo->Clone());
		Reset();
	}

	try								// this will fail if no printer selected, so allow the app to still run
	{
		if (fUseMaxResolution)
			fPrintInfo->MaximizeResolution();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
	}
	
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetMargins: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::SetMargins()
{
	InstallMargins(GetMarginsRect(), fMinimalMargins);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetPage: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::SetPage(long aPageNumber)
{
	fFocusedPage = aPageNumber;
	CViewPoint strip(PageToStrip(aPageNumber));

	CViewRect viewedRect;
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
	{
		GetBreakCoord(gOrthogonal[vhs], strip[vhs], viewedRect[topLeft][vhs]);
		GetBreakCoord(gOrthogonal[vhs], strip[vhs] + 1, viewedRect[botRight][vhs]);
	}

	SetPageInterior(aPageNumber);
	fView->DoSetPageOffset(viewedRect[topLeft]);
	fViewedRect = viewedRect;

#if qDebugMsg
	if (gDebugPrinting)
		ProgramReport_AC(false, "pg #: %1d, coord = %s Page Interior: %s\n", aPageNumber, (const char*)(viewedRect[topLeft]), (const char*)(GetInteriorRect()));
#endif

}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetPageInterior: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::SetPageInterior(long pageNumber)// override 
{
	CViewPoint pegPoint(LocatePageInterior(pageNumber));

	fInteriorRect.Set(pegPoint, pegPoint + GetViewPerPage());
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetPageOffset: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::SetPageOffset(const CViewPoint& coord)// override 
{
	gPageOffset = coord - GetInteriorRect()[topLeft];
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetupForFinder: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::SetupForFinder(CommandNumber /* aCommandNumber */)		// override 
{
	bool proceed = true;

	if (fFinderSetup)
		proceed = PosePageSetupDialog(false);

	if (proceed)
	{
		fPrintInfo->ShowDocBeingPrinted(GetDocName(), true);

		if (fFinderJobDialog || !gFinderPrintInfo)
		{
			proceed = PoseJobDialog();
			
			if (proceed)
			{
				// Put into gFinderPrintInfo, in case next document needs it 
				if (!gFinderPrintInfo)
					gFinderPrintInfo = NewTPrintInfo();

				gFinderPrintInfo->SetFrom(fPrintInfo);
			}
		}
		else
			fPrintInfo->JobMerge(gFinderPrintInfo);

		fPrintInfo->ShowDocBeingPrinted(GetDocName(), false);
	}

	return proceed;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SetupPrintOne: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::SetupPrintOne(CommandNumber aCommandNumber)
{
	// Call PrValidate to 1) make sure the record is OK, and
	// 2) Get the LaserWriter driver to get the name of the front window.

	fPrintInfo->ClearError();
	fPrintInfo->SetPrintPages(fStartPage, MaxPageNumber(), fStartPage, MaxPageNumber());
	
	if (aCommandNumber == cDragPrint && gTargetPrinter.name[0])
		fPrintInfo->SetTargetPrinter(gTargetPrinter);

	return true;								// Should always be able to continue from here
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::ShowsOnScreen: 
//----------------------------------------------------------------------------------------


bool TStdPrintHandler::ShowsOnScreen()
{
	return fView ? fView->IsVisible() : false;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::StripToPage: 
//----------------------------------------------------------------------------------------


long TStdPrintHandler::StripToPage(CViewPoint pageStrip)
{
	VHSelect ortho = gOrthogonal[fPageDirection];

	return pageStrip[fPageDirection] * fPageStrips[ortho] + pageStrip[ortho] + fStartPage;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DoUpdate:
//----------------------------------------------------------------------------------------


void TStdPrintHandler::DoUpdate(ChangeID_AC theChange,
								MDependable_AC*		/*changedObject*/ ,
								void*				/*changeData*/ ,
								CDependencySpace_AC*/*dependencySpace*/  )// Override
{
	switch (theChange)
	{
		case mPrintInfoChanged:
			bool wasValid = fIsValid;
			Invalidate();

			if (fShowBreaks || wasValid)		// someone probably wants to see this
				Update();

			break;
	}

}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::TerminateUPrinting: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::TerminateUPrinting()
{
#if qGXPrinting
	OSErr printingErr = noErr;
	if (gUsingGXPrinting)
	{
		printingErr = ::GXExitPrinting();
		gUsingGXPrinting = false;
	}
#endif
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::PrepareForFinderPrinting: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::PrepareForFinderPrinting(const CFSSpec_AC* targetPrinter)
{
	// If we have previously done a finder print we need to toss the old print information
	// or we won't ask the user for some new stuff.
	delete gFinderPrintInfo;
	gFinderPrintInfo = NULL;

	gTargetPrinter.name[0] = 0;

	if (targetPrinter)
		gTargetPrinter = *targetPrinter;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DispatcherIsAvailable: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::DispatcherIsAvailable()
{
	if (gDispatcher && gDefaultPrintInfo)
		gDispatcher->AddDependent(gDefaultPrintInfo);
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::GetSavePrintInfoSize: 
//----------------------------------------------------------------------------------------

long TStdPrintHandler::GetSavePrintInfoSize(TPrintInfo* itsPrintInfo, TFile* itsFile, bool useRsrcFork)
{
	return itsPrintInfo->GetSize(useRsrcFork && itsFile->HasRsrcFork());
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::SavePrintInfo: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::SavePrintInfo(TPrintInfo* itsPrintInfo, TFile* itsFile, bool useRsrcFork)
{
	if (useRsrcFork && itsFile->HasRsrcFork())
	{
		if (itsFile->IsRsrcForkOpen())
		{
			CTempUseResFile_AC tempUseResFile(itsFile->GetRsrcRefNum());
			if (ResError() == noErr)
			{

				if (itsPrintInfo->IsSetToDefault())
				{
					//
					// PrintInfo matches default for printer, User hasn't expressed any
					// specific preference. Don't write the resource so that it will be
					// default next time as well (Not just needing validation). Remove
					// any resource if present
					//
					// NOTE: that we don't do any of this if the print info is going in
					// the data fork b/c we don't have any standardized way of identifying
					// the print info and wouldn't know how to test for its presence when
					// opening the document later.
					//
					Handle printResource = NULL;

					short idx;
#if qGXPrinting
					if (gUsingGXPrinting)
						idx = kFirstGXFormatIndex;	// Try a GX Format record first
					else
#endif
					idx = kFirstMacPrintFormatIndex;

					for (; idx >=0; --idx)
					{
						short numResources = Count1Resources(kPrintResType[idx]);

						if (numResources) // Any at all?
						{
							// Try preferred ID first
							printResource = Get1Resource(kPrintResType[idx], kPrintResNumber[idx]);

							// Try any ID next
							if (!printResource)
								printResource = Get1IndResource(kPrintResType[idx], 1);

							if (printResource)
							{
								RemoveResource(printResource);
								printResource = NULL;
							}
						}
					}
				}
				else
				{
					CTempHandle_AC aHandle = NewPermHandle(256);

					THandleStream aHandleStream(aHandle, 256);
					
					itsPrintInfo->WriteTo(&aHandleStream);
						
					short idx;
#if qGXPrinting
					if (gUsingGXPrinting)
						idx = kFirstGXFormatIndex;	// Try a GX Format record first
					else
#endif
					idx = kFirstMacPrintFormatIndex;

					MAAddResource(aHandle, kPrintResType[idx], kPrintResNumber[idx], gEmptyString);
					::ThrowIfResError_AC();

					WriteResource(aHandle);
					ReleaseResource(aHandle.DontRelease());
					aHandleStream.SetHandle(NULL);
				}
			}
		}
	}
	else
	{
		// This should be changed to have the count in the file before the data.
		// And to identify the type of print record

		
		itsPrintInfo->WriteTo(&TFileStream(itsFile));
	}
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::RestorePrintInfo: 
//----------------------------------------------------------------------------------------

TPrintInfo* TStdPrintHandler::RestorePrintInfo(TFile* itsFile, bool useRsrcFork)
{
	TPrintInfo * returnVal = NULL;

	Handle printResource = NULL;
#if qGXPrinting
	bool isGXFormat = false;
#endif

	if (useRsrcFork && itsFile->HasRsrcFork())			// Read the print info resource 
	{
		if (itsFile->IsRsrcForkOpen())
		{
			CTempUseResFile_AC tempUseResFile(itsFile->GetRsrcRefNum());

			if (ResError() == noErr)
			{
				short idx;
#if qGXPrinting
				if (gUsingGXPrinting)
					idx = kFirstGXFormatIndex;	// Try a GX Format record first
				else
#endif
				idx = kFirstMacPrintFormatIndex;

				for (; idx >=0; --idx)
				{
					short numResources = Count1Resources(kPrintResType[idx]);

					if (numResources) // Any at all?
					{
						// Try preferred ID first
						printResource = Get1Resource(kPrintResType[idx], kPrintResNumber[idx]);

						// Try any ID next
						if (!printResource)
							printResource = Get1IndResource(kPrintResType[idx], 1);

						if (printResource)
							break;
					}
				}
#if qGXPrinting
				if (idx > kFirstMacPrintFormatIndex)
					isGXFormat = true;
#endif
			}
		}
		if (printResource)
		{
			THandleStream* aHandleStream = TH_new THandleStream(printResource, 0);
		
			try
			{
				
				TPrintInfo* aPrintInfo = NewTPrintInfo();
				
#if qGXPrinting
				// NOT native format (THPrint for MacPrint, gxJob for GX)
				if (gUsingGXPrinting && !isGXFormat)
					dynamic_cast_AC(TGXJob*,aPrintInfo)->ConvertFrom(aHandleStream);
				else
#endif
					aPrintInfo->ReadFrom(aHandleStream);

				returnVal = aPrintInfo;
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				aHandleStream->SetPosition(aHandleStream->GetSize());	// prevent truncation
				delete aHandleStream;
				aHandleStream = NULL;
				ReleaseResource(printResource);
				throw;
			}
			
			
			aHandleStream->SetPosition(aHandleStream->GetSize());	// prevent truncation
			delete aHandleStream;
			aHandleStream = NULL;
			ReleaseResource(printResource);
		}
	}
	else
	{
		// This should be changed to have the count in the file before the data.
		// And to identify the type of print record

		
		TPrintInfo* aPrintInfo = NewTPrintInfo();
		aPrintInfo->ReadFrom(&TFileStream(itsFile));
		returnVal = aPrintInfo;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DeleteIfPrintInfo: 
//----------------------------------------------------------------------------------------

void TStdPrintHandler::DeleteIfPrintInfo(TPrintInfo* inPrintInfo)
{
	if (inPrintInfo)
		delete inPrintInfo;
}

//========================================================================================
// CLASS TPrintCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TPrintCommand constructor
//----------------------------------------------------------------------------------------

TPrintCommand::TPrintCommand(CommandNumber itsCommandNumber,
							 TStdPrintHandler* itsStdPrintHandler) :
	TCommand(itsCommandNumber, itsStdPrintHandler->fDocument ?
	itsStdPrintHandler->fDocument :
	dynamic_cast<TCommandHandler*>((itsStdPrintHandler->fView ?
		(itsStdPrintHandler->fView->GetWindow() ?
			itsStdPrintHandler->fView->GetWindow() :
			itsStdPrintHandler->fView) :
		NULL)), kCantUndo, kDoesNotCauseChange, NULL),

	fStdPrintHandler(itsStdPrintHandler)
{
	fObjectToNotify = fContext;
}

//----------------------------------------------------------------------------------------
// TPrintCommand destructor
//----------------------------------------------------------------------------------------

TPrintCommand::~TPrintCommand()
{
}

//----------------------------------------------------------------------------------------
// TPrintCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------

TAppleEvent* TPrintCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kCoreEventClass, kAEPrint, gServerAddress, kAEWaitReply));

	// Send an AppleEvent for recording
	CTempDesc theTargetDesc;
	if (fStdPrintHandler->fDocument)
	{
		TDocument * theDocument = fStdPrintHandler->fDocument;
		theDocument->MakeObjectSpecifier(theTargetDesc, theDocument->GetSpecifierForm());
	}
	else
	{
		::ThrowIfNonObject_AC(fStdPrintHandler->fView);
		TWindow * theWindow = fStdPrintHandler->fView->GetWindow();
		::ThrowIfNonObject_AC(theWindow);
		theWindow->MakeObjectSpecifier(theTargetDesc, theWindow->GetSpecifierForm());
	}

	theEvent->WriteParameter(keyDirectObject, theTargetDesc);

	return theEvent.release();
}

//----------------------------------------------------------------------------------------
// TPrintCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TPrintCommand::DoIt()
{
	bool setupSuccess = false;

	switch (fIdentifier)
	{
		case cPrint:
			setupSuccess = fStdPrintHandler->PoseJobDialog();
			break;
		
		case cPrintOne:
		case cDragPrint:
			setupSuccess = fStdPrintHandler->SetupPrintOne(fIdentifier);
			break;
	}

	if (setupSuccess)
		fStdPrintHandler->Print(fIdentifier);
}

//========================================================================================
// CLASS TPrintStyleChangeCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TPrintStyleChangeCommand constructor
//----------------------------------------------------------------------------------------

TPrintStyleChangeCommand::TPrintStyleChangeCommand(TStdPrintHandler* itsPrintHandler,
														bool isUndoable) :
	TCommand(cChangePrinterStyle, itsPrintHandler->fView ? itsPrintHandler->fView->GetContext(cChangePrinterStyle) : itsPrintHandler->fDocument->GetContext(cChangePrinterStyle), isUndoable, kCausesChange, itsPrintHandler->fDocument),

	fStdPrintHandler(itsPrintHandler)
{
	TDocument* changedDocument = itsPrintHandler->fDocument;

	fCausesChange = changedDocument && changedDocument->GetSavePrintInfo();

	if (isUndoable)
	{
		fOldPrintInfo = NewTPrintInfo();
		// Make a copy of the old version of the PrintInfo record 
		fOldPrintInfo->SetFrom(itsPrintHandler->fPrintInfo);
	}

	fNewPrintInfo = NewTPrintInfo();
}

//----------------------------------------------------------------------------------------
// TPrintStyleChangeCommand::Free: 
//----------------------------------------------------------------------------------------

TPrintStyleChangeCommand::~TPrintStyleChangeCommand()
{
}

//----------------------------------------------------------------------------------------
// TPrintStyleChangeCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TPrintStyleChangeCommand::DoIt()			// override 
{
	fStdPrintHandler->fPrintInfo->SetFrom(fNewPrintInfo);
}

//----------------------------------------------------------------------------------------
// TPrintStyleChangeCommand::UndoIt: 
//----------------------------------------------------------------------------------------

void TPrintStyleChangeCommand::UndoIt()			// override 
{
	fStdPrintHandler->fPrintInfo->SetFrom(fOldPrintInfo);
}

//----------------------------------------------------------------------------------------
// TPrintStyleChangeCommand::SetupDependencies:
//----------------------------------------------------------------------------------------

void TPrintStyleChangeCommand::SetupDependencies()
{
	Inherited::SetupDependencies();

	if (fStdPrintHandler && fStdPrintHandler->fView && CanBeUndone())
	{
		fStdPrintHandler->fView->AddDependent(this);
		fStdPrintHandler->fPrintInfo->AddDependent(this);
	}
}

//----------------------------------------------------------------------------------------
// TPrintStyleChangeCommand::DoUpdate:
//----------------------------------------------------------------------------------------

void TPrintStyleChangeCommand::DoUpdate(ChangeID_AC theChange,
										MDependable_AC* changedObject,
										void* changeData,
										CDependencySpace_AC* dependencySpace)// Override
{
	if ((theChange == mClosed) && (changedObject == fStdPrintHandler->fView))
		fContext->CommitLastCommand();
	else if (theChange == mPrinterChanged && changedObject == fStdPrintHandler->fPrintInfo.get())
		fCanUndo = false;
	else
		Inherited::DoUpdate(theChange, changedObject, changeData, dependencySpace);
}

//----------------------------------------------------------------------------------------
// End of UPrinting.cp


