//----------------------------------------------------------------------------------------
// UProgressIndicator.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UPROGRESSINDICATOR__
#include "UProgressIndicator.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#if qWantsAppearance

#	ifndef __APPEARANCE__
#	include <Appearance.h>
#	endif

#	ifndef __APPEARANCEHELPERS__
#	include <AppearanceHelpers.h>
#	endif

#endif

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __UViewIdler__
#include "UViewIdler.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

//----------------------------------------------------------------------------------------
// InitUProgressIndicator
//----------------------------------------------------------------------------------------


void InitUProgressIndicator()
{
	MA_REGISTER_CLASS(TProgressIndicator);
}

//========================================================================================
// CLASS TProgressIndicator
//========================================================================================

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS(TProgressIndicator);

//----------------------------------------------------------------------------------------
// TProgressIndicator constructor
//----------------------------------------------------------------------------------------


TProgressIndicator::TProgressIndicator(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
								 bool isDeterminate) :
#if qWantsAppearance
	TCtlMgr(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, gEmptyString, 0, 0, 0, kControlProgressBarProc)
#else
	TCtlMgr(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, gEmptyString, 0, 0, 0, pushButProc /* dummy type; control will be hidden */)
#endif
	, fAGAControl(NULL),
	fIsDeterminate(isDeterminate)
{
}

//----------------------------------------------------------------------------------------
// TProgressIndicator destructor
//----------------------------------------------------------------------------------------


TProgressIndicator::~TProgressIndicator()
{
	StopAutoAnimate();
}

//----------------------------------------------------------------------------------------
// TProgressIndicator::ReadFields
//----------------------------------------------------------------------------------------


void TProgressIndicator::ReadFields(TStream* aStream)
{
	Inherited::ReadFields(aStream);

	*aStream >> fLongVal
			 >> fLongMin
			 >> fLongMax
			 >> fIsDeterminate;
	
#if qWantsAppearance
	CreateCMgrControl(gEmptyString, fLongVal, fLongMin, fLongMax, kControlProgressBarProc);
#else
	CreateCMgrControl(gEmptyString, fLongVal, fLongMin, fLongMax, pushButProc /* dummy type; control will be hidden */);
#endif

	if (!fIsDeterminate)
		StartAutoAnimate();
}

//----------------------------------------------------------------------------------------
// TProgressIndicator::WriteFields
//----------------------------------------------------------------------------------------


void TProgressIndicator::WriteFields(TStream* aStream) const
{
	Inherited::WriteFields(aStream);

	*aStream << fLongVal
			 << fLongMin
			 << fLongMax
			 << fIsDeterminate;
}

//----------------------------------------------------------------------------------------
// TProgressIndicator::DoIdle
//----------------------------------------------------------------------------------------


bool TProgressIndicator::DoIdle(IdlePhase /*phase*/)
{
#if qWantsAppearance
	if (!IsEmulatingControls())
	{
		if (fCMgrControl && Focus())
			::IdleControls(GetWindow()->GetWindowRef());
	}
	else
#endif
	{
	// Let the CControl_AC do one animation step.
		if (fAGAControl && Focus())
			fAGAControl->QuantizedAnimate();
	}
	
	return false;	// did not free self
}

//----------------------------------------------------------------------------------------
// TProgressIndicator::StartAutoAnimate
//----------------------------------------------------------------------------------------


void TProgressIndicator::StartAutoAnimate()
{
	SetIdleFreq(0);		// idle often
	SetEnable(true);	// we won't get idle time if we're disabled!

#if qWantsAppearance
	if (!IsEmulatingControls())
	{
		(void) ::SetProgressIndicatorState( fCMgrControl, false );
	} 
	else
#endif
	{
		// Kludge: the AGA control auto-animates only when min==max
		SetLongMax(GetLongMin(), kDontRedraw);
	}

	// This creates an idler behavior for the dispatcher, so we always get idle time.
	TH_new TViewIdler(this);
	
	fIsDeterminate = false;
}

//----------------------------------------------------------------------------------------
// TProgressIndicator::StopAutoAnimate
//----------------------------------------------------------------------------------------


void TProgressIndicator::StopAutoAnimate()
{
#if qWantsAppearance
	if (!IsEmulatingControls())
	{
		(void) ::SetProgressIndicatorState( fCMgrControl, true );
	}	
#endif
	
	// This finds and removes the idler behavior from the dispatcher.
	for (CBehaviorIterator iter(gDispatcher); iter; ++iter)
	{
		TViewIdler* idler = dynamic_cast<TViewIdler*> (iter.Current());
		if (idler && idler->GetView() == this)
		{
			delete idler;
			break;
		}
	}
	
	fIsDeterminate = true;
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

bool TProgressIndicator::IsEmulatingControls() const
{
#if qWantsAppearance
	return !::HasAppearanceMgr_AC();
#else
	return true;
#endif
}

//----------------------------------------------------------------------------------------
// TProgressIndicator::CreateAGAControl
//----------------------------------------------------------------------------------------


void TProgressIndicator::CreateAGAControl()
{
	// Instantiate the particular CControl_AC subclass.
	fAGAControl = TH_new CProgressIndicator_AC(ControlQDArea(), 0, 100);
}

