//----------------------------------------------------------------------------------------
// UQD3DViewer.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#if !q68K

#ifndef __UQD3DViewer__
#include "UQD3DViewer.h"
#endif

// ACS

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

//MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef  __UQD3DCOMMON__
#include "UQD3DCommon.h"
#endif

#ifndef __UWINDOW__
#include "UWIndow.h"
#endif

//QuickDraw3D

#ifndef  QD3D_h
#include "QD3D.h"
#endif

#ifndef   QD3DView_h
#include "QD3DView.h" 
#endif

#ifndef   QD3DViewer_h
#include "QD3DViewer.h"
#endif


//========================================================================================
// Global Initialization Procedure
//========================================================================================
//----------------------------------------------------------------------------------------
// InitUQD3DViewer: 
//----------------------------------------------------------------------------------------
void InitUQD3DViewer()
{
	// So the linker doesn't dead strip class info 
	// !!! NOTE: really need to assign a standard id for the standard viewer
	MA_REGISTER_CLASS(TQD3DViewer);
	
#if qDebug
	if (!HasQuickDraw3DViewer_AC())
		DebugStr("\p InitUQD3DViewer called but QD3D isn't present.");

#if qDrag
	if(!HasDragManager_AC())
		DebugStr("\p Due to a bug in QD3D (1.0.6 and prior), the viewer will crash without the DragManager installed.");
#endif
#endif
} 

//========================================================================================
// CLASS TQD3DViewer
//========================================================================================
MA_DEFINE_CLASS(TQD3DViewer);

//----------------------------------------------------------------------------------------
// TQD3DViewer constructor 
//----------------------------------------------------------------------------------------
TQD3DViewer::TQD3DViewer(const CViewPoint& itsLocation,
						 				const CViewPoint& itsSize,
						 				SizeDeterminer itsHSizeDet,
				  						SizeDeterminer itsVSizeDet) :
	TView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fQD3DViewer(NULL),
	fUseManualCreation(false),
	fViewerFlags(kQ3ViewerDefault)
{
}

//----------------------------------------------------------------------------------------
// TQD3DViewer destructor
//----------------------------------------------------------------------------------------
TQD3DViewer::~TQD3DViewer()
{
	Remove3DViewerFromView();
}
	
//----------------------------------------------------------------------------------------
// TQD3DViewer::DoPostCreate: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::DoPostCreate(TDocument* itsDocument) 
{
	TView::DoPostCreate(itsDocument);
	
	if (!fUseManualCreation)
		Add3DViewerToView(fViewerFlags);
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::Draw: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::Draw(const CViewRect& area)
{
	if (HasViewer())
	{
		GetWindow()->Focus();
		
		::ThrowIfOSErr_AC(::Q3ViewerDraw(fQD3DViewer));
		
		Focus();
	}
	
	TView::Draw(area);
}


//----------------------------------------------------------------------------------------
// TQD3DViewer::DoSetupMenus: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::DoSetupMenus()
{
	TView::DoSetupMenus();
	
	if (HasViewer())
	{
		bool hasImage = GetViewerState() == kQ3ViewerHasModel;
		
		Enable(cCut,  hasImage);
		Enable(cCopy, hasImage);
		gClipboardMgr->CanPaste(kMac3DMFFileType); //can paste a 3DMF file
		Enable(cClear, hasImage);
	}
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::DoMenuCommand:
//----------------------------------------------------------------------------------------
void TQD3DViewer::DoMenuCommand(CommandNumber aCommandNumber)
{
	switch (aCommandNumber) 
	{
		case cCut:
		case cPaste:
			PostCommand(TH_new TQD3DCutPasteCommand(aCommandNumber, fQD3DViewer));
			break;
		
		case cCopy:
			::ThrowIfOSErr_AC(::Q3ViewerCopy(fQD3DViewer));
			break;
		
		case cClear:
			::ThrowIfOSErr_AC(::Q3ViewerClear(fQD3DViewer));
			break;
		
		default:
			TView::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::DoMouseCommand: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::DoMouseCommand(CViewPoint& theMouse,
								  TToolboxEvent*  event,
								  CPoint_AC  hysteresis)
{
	bool handledEvent = false;

	if (fQD3DViewer)
	{
		GetWindow()->Focus();
		
		EventRecord theEventRecord = const_cast<EventRecord&>(event->GetEventRecord(0));
		handledEvent = ::Q3ViewerEvent(fQD3DViewer, &theEventRecord);
		Focus();
	}

	//Pass the event off to MacApp if QD3D did not handle it
	
	if (!handledEvent)
		TView::DoMouseCommand(theMouse, event, hysteresis);
}


//----------------------------------------------------------------------------------------
// TQD3DViewer::SetFrame: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	CViewRect oldFrame(GetFrame());

	::ThrowIfOSErr_AC(::Q3ViewerSetBounds(fQD3DViewer, ViewToQD(newFrame)));
	TView::SetFrame(newFrame, invalidate);
	
	//MacApp only invalidates the new region, not the entire thing. We need to 
	//invalidate the region that was the original image.
	
	if (oldFrame != newFrame && invalidate)
	{
		CTempRegion_AC oldFrameRgn;
		CRect_AC oldQDFrame(fSuperView->ViewToQD(oldFrame));
		RectRgn(oldFrameRgn, &oldQDFrame);
		fSuperView->Invalidate(oldFrameRgn);
	}
}


//----------------------------------------------------------------------------------------
// TQD3DViewer::AddViewerToView: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::Add3DViewerToView(unsigned long theViewerFlags)
{
	//Note: If you have a frame adorner of any kind, you will want to override this
	//      method and inset the Frame accordingly. (i.e. theFrame.Inset(1,1);) If you
	//      just want a normal frame, there is a viewer flag you can set.
	
	fQD3DViewer = ::Q3ViewerNew(reinterpret_cast<CGrafPtr>(GetWindow()->GetGrafPort()), ViewToQD(GetFrame()), theViewerFlags);
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::RemoveViewerFromView: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::Remove3DViewerFromView()
{
	if (HasViewer())
		::ThrowIfOSErr_AC(::Q3ViewerDispose(fQD3DViewer));
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::GetViewerState: 
//----------------------------------------------------------------------------------------
unsigned long TQD3DViewer::GetViewerState()
{
	return HasViewer() ? ::Q3ViewerGetState(fQD3DViewer) : 0;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::Get3DViewFromViewer: 
//----------------------------------------------------------------------------------------
TQ3ViewObject TQD3DViewer::Get3DViewFromViewer()
{
	return HasViewer() ? ::Q3ViewerGetView(fQD3DViewer) : NULL;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::SetRenderer: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::SetRenderer(TQ3ObjectType renderType, bool redraw)
{
	if (HasViewer())
	{
		TQ3ViewObject 	theView		= ::Q3ViewerGetView(fQD3DViewer);

		if (Q3View_SetRendererByType(theView, renderType) != kQ3Success)
			Fail3DErr(::Q3Object_Dispose(theView));  //make sure to decrement the ref count.

		if (redraw)
			ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::GetRenderer: 
//----------------------------------------------------------------------------------------
TQ3RendererObject* TQD3DViewer::GetRenderer()
{
	TQ3RendererObject* theRenderer = NULL;

	if (HasViewer())
	{
		TQ3ViewObject 	theView = Get3DViewFromViewer();
		if (Q3View_GetRenderer(theView, theRenderer) != kQ3Success )
			Fail3DErr(::Q3Object_Dispose(theView));  //make sure to decrement the ref count.
	}
#if qDebug
	else
		DebugStr("\p Attempted to GetRenderer without a viewer object.");
#endif

	return theRenderer;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::AsPict: 
//----------------------------------------------------------------------------------------
PicHandle TQD3DViewer::AsPict()
{
#if qDebug
	if (!HasViewer())
		DebugStr("\p Attempted to get a Pict without a viewer object.");
#endif

	return HasViewer() ? ::Q3ViewerGetPict(fQD3DViewer) : NULL;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::SetBackgroundColor: 
/*
	Note: QuickDraw3D uses it's own color structures. When calling this method, if you have
		  an RGB value handy that you want to use (maybe you called GetColor), the best thing
		  to do is to get each value from your RGB.x, divide it by 65535, and stuff it back in
		  the TQ3ColorARGB structure.
		  
		  Or, you can just use the easy overloaded version of this function...
	  
*/
//----------------------------------------------------------------------------------------
void TQD3DViewer::SetBackgroundColor(TQ3ColorARGB* newColor,bool redraw)
{
	if (fQD3DViewer)
	{	
		::ThrowIfOSErr_AC(::Q3ViewerSetBackgroundColor(fQD3DViewer,newColor));

		if (redraw)
			ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::SetBackgroundColor: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::SetBackgroundColor(CRGBColor_AC newColor, float alpha, bool redraw)
{
	TQ3ColorARGB theQ3Color;
	float maxColor = 65535.0;
	 
	theQ3Color.r = newColor.red/maxColor;
	theQ3Color.g = newColor.green/maxColor;
	theQ3Color.b = newColor.blue/maxColor;
	
	theQ3Color.a = alpha;

	SetBackgroundColor(&theQ3Color, redraw);
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::GetRGBBackgroundColor: 
//----------------------------------------------------------------------------------------
CRGBColor_AC TQD3DViewer::GetRGBBackgroundColor()
{
	CRGBColor_AC 	 theRGBColor = gRGBWhite; //white by default
	TQ3ColorARGB theQ3Color;
	
	if(HasViewer())
	{
		::ThrowIfOSErr_AC(::Q3ViewerGetBackgroundColor(fQD3DViewer, &theQ3Color));
		
		float maxColor = 65535.0;
		 
		theRGBColor.red 	= theQ3Color.r * maxColor;
		theRGBColor.green 	= theQ3Color.g * maxColor;
		theRGBColor.blue 	= theQ3Color.b * maxColor;
	}

	return theRGBColor;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::GetBackgroundColor: 
//----------------------------------------------------------------------------------------
TQ3ColorARGB* TQD3DViewer::GetBackgroundColor()
{
	TQ3ColorARGB* theQ3Color = NULL;
	
	if (HasViewer())
		::ThrowIfOSErr_AC(::Q3ViewerGetBackgroundColor(fQD3DViewer, theQ3Color));

	return theQ3Color;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::SetViewerFlags: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::SetViewerFlags(unsigned long theViewerFlags)
{
	if (HasViewer())
	{
		fViewerFlags = theViewerFlags;
		::ThrowIfOSErr_AC(::Q3ViewerSetFlags(fQD3DViewer,fViewerFlags));
	}
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::GetViewerFlags: 
//----------------------------------------------------------------------------------------
unsigned long TQD3DViewer::GetViewerFlags()
{
	unsigned long theFlags = 0;
	
	if (HasViewer())
		theFlags = fViewerFlags = ::Q3ViewerGetFlags(fQD3DViewer);

 	return theFlags;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::SetCurrentButton: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::SetCurrentButton(unsigned long theButton)
{
	if (HasViewer())
		::ThrowIfOSErr_AC(::Q3ViewerSetCurrentButton(fQD3DViewer, theButton));
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::GetCurrentButton: 
//----------------------------------------------------------------------------------------
unsigned long TQD3DViewer::GetCurrentButton()
{
	unsigned long theButton = 0;
	
	if (HasViewer())
		theButton = ::Q3ViewerGetCurrentButton(fQD3DViewer);
	
	return theButton;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::GetButtonRect: 
//----------------------------------------------------------------------------------------
CRect_AC TQD3DViewer::GetButtonRect(unsigned long theButton)
{
	CRect_AC theCRect;

	if (HasViewer())
		::ThrowIfOSErr_AC(::Q3ViewerGetButtonRect(fQD3DViewer, theButton, &theCRect));
	
	return theCRect;
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::UseFile: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::UseFile(const CFSSpec_AC& theFile)
{
	if (HasViewer())
	{
		short theFileRef;
		
		::ThrowIfOSErr_AC(::FSpOpenDF( theFile, fsRdPerm, &theFileRef));
		::ThrowIfOSErr_AC(::Q3ViewerUseFile(fQD3DViewer, theFileRef));
		::ThrowIfOSErr_AC(::FSClose(theFileRef));
	}
}

//----------------------------------------------------------------------------------------
// TQD3DViewer::Write3DMFFile: 
//----------------------------------------------------------------------------------------
void TQD3DViewer::Write3DMFFile(TFile* theFile)
{
	if (HasViewer())
	{
		short theFileRef;
		
		::ThrowIfOSErr_AC(::FSpOpenDF( theFile->GetFileSpec(), fsWrPerm, &theFileRef));
		::ThrowIfOSErr_AC(::Q3ViewerWriteFile(fQD3DViewer,theFileRef));
		::ThrowIfOSErr_AC(::FSClose(theFileRef));
	}
}

//========================================================================================
// CLASS TQD3DCutPasteCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TQD3DCutPasteCommand constructor 
//----------------------------------------------------------------------------------------
TQD3DCutPasteCommand::TQD3DCutPasteCommand(CommandNumber itsCommandNumber, TQ3ViewerObject itsQD3DViewer) :
	TCommand(itsCommandNumber,NULL, kCanUndo, kCausesChange, NULL),
	fQD3DViewer(itsQD3DViewer)
{
}

//----------------------------------------------------------------------------------------
// TQD3DCutPasteCommand destructor
//----------------------------------------------------------------------------------------
TQD3DCutPasteCommand::~TQD3DCutPasteCommand()
{
}

//----------------------------------------------------------------------------------------
// TQD3DCutPasteCommand::DoIt: 
//----------------------------------------------------------------------------------------
void TQD3DCutPasteCommand::DoIt()
{
	if (fIdentifier == cCut)
		::ThrowIfOSErr_AC(::Q3ViewerCut(fQD3DViewer));
	else if (fIdentifier == cPaste)
		::ThrowIfOSErr_AC(::Q3ViewerPaste(fQD3DViewer));
}

//----------------------------------------------------------------------------------------
// TQD3DCutPasteCommand::UndoIt: 
//----------------------------------------------------------------------------------------
void TQD3DCutPasteCommand::UndoIt()
{
	if (fIdentifier == cCut)
		::ThrowIfOSErr_AC(::Q3ViewerPaste(fQD3DViewer));
	else if (fIdentifier == cPaste)
		::ThrowIfOSErr_AC(::Q3ViewerCut(fQD3DViewer));
}

//----------------------------------------------------------------------------------------
// End of UQD3DViewer.cp

#endif
