//----------------------------------------------------------------------------------------
// URadio.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __URADIO__
#include "URadio.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

void InitURadio()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TRadio, kStdRadio);
	}
}  

//========================================================================================
// CLASS TRadio
//========================================================================================
#undef Inherited
#define Inherited TCtlMgr

MA_DEFINE_CLASS(TRadio);

//----------------------------------------------------------------------------------------
// TRadio constructor
//----------------------------------------------------------------------------------------

TRadio::TRadio(const CViewPoint& itsLocation,
					const CViewPoint& itsSize,
					SizeDeterminer itsHSizeDet,
					SizeDeterminer itsVSizeDet,
					const CStr255_AC& itsLabel,
					bool isTurnedOn) :
	TCtlMgr(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, 0, 0, 1, ((radioButProc) | kControlUsesOwningWindowsFontVariant)),

	fStrListID(kNoResource),					// buttons's label:	STR# rsrc id
	fIndex(kNoResource)							//					index into STR#
#if qGrayscaleAppearance
	, fAGAControl(NULL)
#endif
{
	fEventNumber = mRadioHit;

	SetState(isTurnedOn, kDontRedraw); 

	if (IsOn()) 
		HandleEvent(fEventNumber, this, NULL); 
}

//----------------------------------------------------------------------------------------
// TRadio destructor
//----------------------------------------------------------------------------------------

TRadio::~TRadio()
{
}

//----------------------------------------------------------------------------------------
// TRadio::GetStandardSignature: 
//----------------------------------------------------------------------------------------

IDType TRadio::GetStandardSignature() const			// Override 
{
	return kStdRadio;
}

//----------------------------------------------------------------------------------------
// TRadio::ReadFields: 
//----------------------------------------------------------------------------------------

void TRadio::ReadFields(CStream_AC* aStream)		// Override 
{
	Inherited::ReadFields(aStream);

	bool isOn = aStream->ReadBoolean();

	CStr255_AC itsLabel;
	fStrListID = aStream->ReadInteger();
	fIndex = aStream->ReadInteger();
	if (fStrListID != kNoResource)			// retrieve the buttons's label from the resource
		GetIndString(itsLabel, fStrListID, fIndex);

	CreateCMgrControl(itsLabel, isOn, 0, kControlRadioButtonMixedValue, radioButProc | kControlUsesOwningWindowsFontVariant);
}

//----------------------------------------------------------------------------------------
// TRadio::WriteFields: 
//----------------------------------------------------------------------------------------

void TRadio::WriteFields(CStream_AC* aStream) const		// Override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteBoolean(IsOn());

	aStream->WriteInteger(fStrListID);
	aStream->WriteInteger(fIndex);
}

//----------------------------------------------------------------------------------------
// TRadio::DoPostCreate: 
//----------------------------------------------------------------------------------------

void TRadio::DoPostCreate(TDocument* itsDocument)// Override
{
	Inherited::DoPostCreate(itsDocument);

	if (IsOn())
		HandleEvent(fEventNumber, this, NULL);
}

//----------------------------------------------------------------------------------------
// TRadio::DoEvent: 
//----------------------------------------------------------------------------------------

void TRadio::DoEvent(EventNumber eventNumber,
					 TEventHandler* source,
					 TEvent* event)				// Override 

{
	switch (eventNumber)
	{
		case mTurnOn:
		case mTurnOff:
			SetState(eventNumber == mTurnOn, kRedraw);
			break;

		case mRadioHit:
			if (!IsOn())
				Toggle(kRedraw);
			// no break - drop through

		default:
			Inherited::DoEvent(eventNumber, source, event);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TRadio::IsOn: 
//----------------------------------------------------------------------------------------

bool TRadio::IsOn() const
{
#if qGrayscaleAppearance
	if (IsEmulatingControls() && fAGAControl)
		return fAGAControl->GetValue() == CRadioButton_AC::kRadioButtonOn;
	else
#endif // qGrayscaleAppearance
		return GetLongVal() == 1; // was: != 0
}

//----------------------------------------------------------------------------------------
// TRadio::SetState: 
//----------------------------------------------------------------------------------------

void TRadio::SetState(SInt16 state,
					  bool redraw)
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		if (fAGAControl)
			fAGAControl->SetValue(state, redraw && Focus() && IsVisible());
	}
	else
#endif // qGrayscaleAppearance
		SetLongVal((long)(state), redraw);
}

//----------------------------------------------------------------------------------------
// TRadio::Toggle: 
//----------------------------------------------------------------------------------------

void TRadio::Toggle(bool redraw)
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		// Let SetState do our work for us
		SetState(!IsOn(), redraw && Focus() && IsVisible());
	}
	else
#endif // qGrayscaleAppearance
		SetLongVal((long)(!IsOn()), redraw);
}

//----------------------------------------------------------------------------------------
// TRadio::ToggleIf: 
//----------------------------------------------------------------------------------------

void TRadio::ToggleIf(bool matchState,
					  bool redraw)
{
	bool isOn = IsOn();
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		// Let SetState do our work for us
		if (isOn == matchState)
			SetState(!isOn, redraw && Focus() && IsVisible());
	}
	else
#endif // qGrayscaleAppearance
		if (isOn == matchState)
			SetLongVal((long)(!isOn), redraw);
}

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TRadio::HiliteState: 
//----------------------------------------------------------------------------------------

void TRadio::HiliteState(bool state, bool redraw)
{
	if (IsEmulatingControls())
	{
		if (fAGAControl && Focus())
			fAGAControl->DrawButton(state);
	}
	else
		Inherited::HiliteState(state, redraw);
}

//----------------------------------------------------------------------------------------
// TRadio::Hilite: 
//----------------------------------------------------------------------------------------

void TRadio::Hilite()
{
	if (IsEmulatingControls())
	{
		if (fAGAControl && Focus())
			fAGAControl->DrawButton(CControl_AC::kPressed);
	}
	else
		Inherited::Hilite();
}

//----------------------------------------------------------------------------------------
// TRadio::SetText: 
//----------------------------------------------------------------------------------------

void TRadio::SetText(const CStr255_AC& itsText, bool redraw)
{
	// Change the current text in TCtlMgr and in
	// the AGA object. Do NOT let control mgr draw.

	Inherited::SetText(itsText, kDontRedraw);

	if (fAGAControl)
	{
		fAGAControl->SetTitle(itsText, IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
}

//----------------------------------------------------------------------------------------
// TRadio::DoMakeControlAGA: 
//----------------------------------------------------------------------------------------

void TRadio::CreateAGAControl()
{
	fAGAControl = TH_new CRadioButton_AC(ControlQDArea(), fTextStyle, 
		GetText(), MExclusiveObject_AC::kNoGroupID, MExclusiveObject_AC::kNoGroupID, CControl_AC::kNoAutomaticState);
	if (GetLongVal() != 0)
	{
		fAGAControl->SetValue(CRadioButton_AC::kRadioButtonOn, CControl_AC::kDontRedraw);
	}
}

#endif // qGrayscaleAppearance
