// UScrollBarProportional.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.


// These classes depend on functionality derived from TCtlMgr when qGrayscaleAppearance 

// These classes are the exception to the rule (that other new classes using AGA have
// all the necessary functionality repeated in the derived classes).

#if qGrayscaleAppearance

#ifndef __USCROLLBARPROPORTIONAL__
#include "UScrollBarProportional.h"
#endif

// ACS

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

// MacApp

#ifndef __USCROLLER__
#include "UScroller.h"
#endif


void InitUScrollBarProportional()
{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	MA_REGISTER_CLASS(TScrollBarProportional);
	MA_REGISTER_CLASS(TScrollerScrollBarProportional);
}

//========================================================================================
// CLASS TScrollBarProportional
//========================================================================================

#undef Inherited
#define Inherited TScrollBar

MA_DEFINE_CLASS(TScrollBarProportional);

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

TScrollBarProportional::TScrollBarProportional(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, VHSelect itsDirection, long itsVal, long itsMin, long itsMax) :
	TScrollBar(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsDirection, itsVal, itsMin, itsMax),
	fAGAControl(NULL)
{
	// caller must call virtual function this->CreateAGAControl();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

TScrollBarProportional::~TScrollBarProportional()
{
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::HandleNotification(SInt32 dataValue)
{
	// Update the scrollbar value, and propagate our event number.

	SetLongVal(dataValue, kDontRedraw);
	
	HandleEvent(fEventNumber, this, NULL);

	(void) Focus();	// HandleEvent may have caused another view to draw
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

bool TScrollBarProportional::IsEmulatingControls() const
{
	return EmulateControls_AC();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::CreateAGAControl()
{
	fAGAControl = TH_new CScrollBar_AC(ControlQDArea(), GetLongMin(), GetLongMax(), GetLongVal());
	::ThrowIfNULL_AC(fAGAControl);
	
	fAGAControl->InstallNotificationRoutine((CIndicatorCallback_AC) RealAGANotifier, this);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::Draw(const CViewRect& area)	// override 
{
	if (IsEmulatingControls())
	{
		// Let the CControl_AC draw itself.
		if (GetAGAControl() != NULL)
			GetAGAControl()->DrawObject();
	}
	else 
		Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::SetLongVal(ViewCoordinate itsVal, bool redraw)
{
	// Catch all value changes here and synchronize the
	// CScrollBar_AC accordingly.

	Inherited::SetLongVal(itsVal, redraw && !IsEmulatingControls());
	
	if (fAGAControl)
	{
		fAGAControl->SetValue(itsVal, IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis)
{
	if (IsEmulatingControls())
	{	
		// Let the CControl_AC track the mouse. Don't call HandleEvent;
		// it has already been handled through HandleNotification.
		(void) fAGAControl->TrackMouse(ViewToQD(theMouse));
	}
	else
		Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::HiliteState(bool state, bool redraw)
{
	if (!IsEmulatingControls())
		Inherited::HiliteState(state, redraw);

	// Suppress Control Manager scroll bar drawing.
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::Hilite()
{
	if (!IsEmulatingControls())
		Inherited::Hilite();
	
	// Suppress Control Manager scroll bar drawing.
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::Activate(bool entering)
{
	if (IsEmulatingControls())
	{	
		// Set the CScrollBar_AC's activated state.

		if (fAGAControl)
			fAGAControl->SetActive(entering, Focus() && IsVisible());
	}
	else
		Inherited::Activate(entering);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollBarProportional::SetLongMax(ViewCoordinate itsMax, bool redraw)
{
	// Set the CScrollBar_AC's limit.
	
	Inherited::SetLongMax(itsMax, redraw && ! IsEmulatingControls());

	if (fAGAControl)
	{
		fAGAControl->SetRange(fAGAControl->GetMinimum(), GetLongMax(), IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
}

//----------------------------------------------------------------------------------------
// TScrollBarProportional
//----------------------------------------------------------------------------------------

void TScrollBarProportional::RealAGANotifier(CIndicator_AC* /*theIndicator*/, SInt32 dataValue, TScrollBarProportional* notifierTarget)
{
	// This is a static function.

	notifierTarget->HandleNotification(dataValue);
}

//========================================================================================
// CLASS TScrollBarProportional
//========================================================================================

#undef Inherited
#define Inherited TScrollerScrollBar

MA_DEFINE_CLASS(TScrollerScrollBarProportional);

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

TScrollerScrollBarProportional::TScrollerScrollBarProportional(const CViewPoint& itsLocation, const CViewPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, VHSelect itsDirection, long itsMax, TScroller* itsScroller) :
	TScrollerScrollBar(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsDirection, itsMax, itsScroller),
	fAGAControl(NULL)
{
	// caller must call virtual function this->CreateAGAControl();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

TScrollerScrollBarProportional::~TScrollerScrollBarProportional()
{
}


//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::HandleNotification(SInt32 dataValue)
{
	// Scroll our scrollers and call the installed notification
	// function.

	for (TemplateIterator_AC<TScroller*> iter(&fScrollers); iter; ++iter)
		(void) iter->ScrollRelative(fDirection, dataValue);

	(void) Focus();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::CreateAGAControl()
{
	// Instantiate the particular CControl_AC subclass.
	fAGAControl = TH_new CScrollBar_AC(ControlQDArea(), GetLongMin(), GetLongMax(), GetLongVal());
	::ThrowIfNULL_AC(fAGAControl);
	
	fAGAControl->InstallNotificationRoutine((CIndicatorCallback_AC) RealAGANotifier, this);

	UpdateScrollAmounts();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::Draw(const CViewRect& area)	// override 
{
	if (IsEmulatingControls())
	{
		// Let the CControl_AC draw itself.
		if (GetAGAControl() != NULL)
			GetAGAControl()->DrawObject();
	}
	else 
		Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	// Let the MacApp stuff relocate and recalculate,
	// then resynchronize the CScrollBar_AC to the scrollers.

	Inherited::SetFrame(newFrame, invalidate);
	
	UpdateScrollAmounts();
	
	if (GetAGAControl() != NULL)
		GetAGAControl()->SetBounds(ControlQDArea(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::AttachScroller(TScroller* itsScroller)
{
	// Synchronize the CScrollBar_AC to the supplied scroller.

	Inherited::AttachScroller(itsScroller);
	
	UpdateScrollAmounts();
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::SetLongVal(ViewCoordinate itsVal, bool redraw)
{
	// Catch all value changes here and synchronize the
	// CScrollBar_AC accordingly.

	Inherited::SetLongVal(itsVal, redraw && !IsEmulatingControls());
	
	if (fAGAControl)
	{
		fAGAControl->SetValue(itsVal, IsEmulatingControls() && redraw && Focus() && IsVisible());
	}
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis)
{
	if (IsEmulatingControls())
	{	
		// Let the CControl_AC track the mouse. Don't call HandleEvent;
		// it has already been handled through HandleNotification.
		(void) fAGAControl->TrackMouse(ViewToQD(theMouse));
	}
	else
		Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::HiliteState(bool state, bool redraw)
{
	if (!IsEmulatingControls())
		Inherited::HiliteState(state, redraw);

	// Suppress Control Manager scroll bar drawing.
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::Hilite()
{
	if (!IsEmulatingControls())
		Inherited::Hilite();
	
	// Suppress Control Manager scroll bar drawing.
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::Activate(bool entering)
{
	if (IsEmulatingControls())
	{
		// Set the CScrollBar_AC's activated state.

		if (fAGAControl)
			fAGAControl->SetActive(entering, Focus() && IsVisible());
	}
	else
		Inherited::Activate(entering);
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::SetLongMax(ViewCoordinate itsMax, bool redraw)
	{
	// Set the CScrollBar_AC's limit.
	
	Inherited::SetLongMax(itsMax, redraw && ! EmulateControls_AC());

	if (fAGAControl)
	{
		fAGAControl->SetRange(fAGAControl->GetMinimum(), GetLongMax(), IsEmulatingControls() && redraw && Focus() && IsVisible());

		UpdateScrollAmounts();
	}
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::UpdateScrollAmounts()
{
	// Synchronize the CScrollBar_AC to reflect the scroller state.

	if ((fAGAControl) && (fScrollers.GetSize() > 0))
	{
		TScroller*	aScroller = static_cast<TScroller*>(fScrollers.At(1));
		ViewCoordinate	maxTranslation = aScroller->fMaxTranslation[fDirection];
		ViewCoordinate	viewSize = aScroller->GetSize()[fDirection];
		ViewCoordinate	singleStepSize = aScroller->fScrollUnit[fDirection];
		ViewCoordinate	pageStepSize = Max_AC(0, viewSize - singleStepSize);
		ViewCoordinate	pageSize = maxTranslation * (((float) (Max_AC(0, viewSize - singleStepSize))) / ((float) (viewSize + maxTranslation)));

		(void) Focus();

		fAGAControl->SetRange(0, maxTranslation, kDontRedraw);
		fAGAControl->SetPageSize(pageSize, EmulateControls_AC());
		fAGAControl->SetStepSizes(singleStepSize, pageStepSize);
	}
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

bool TScrollerScrollBarProportional::IsEmulatingControls() const
{
	return EmulateControls_AC();
}

//----------------------------------------------------------------------------------------
// TScrollerScrollBarProportional
//----------------------------------------------------------------------------------------

void TScrollerScrollBarProportional::RealAGANotifier(CIndicator_AC* /*theIndicator*/, SInt32 dataValue, TScrollerScrollBarProportional* notifierTarget)
{
	// This is a static function.

	notifierTarget->HandleNotification(dataValue);
}

#endif // qGrayscaleControls

