//----------------------------------------------------------------------------------------
// USlider.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __USLIDER__
#include "USlider.h"
#endif

//----------------------------------------------------------------------------------------
// Unit initialization
//----------------------------------------------------------------------------------------

void InitUSlider()
{
	MA_REGISTER_CLASS(TSlider);
}

//========================================================================================
// CLASS TSlider
//========================================================================================

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS(TSlider);

//----------------------------------------------------------------------------------------
// TSlider constructor
//----------------------------------------------------------------------------------------

TSlider::TSlider(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
							     const CTextStyle_AC& itsTextStyle,
							     ResNumber itsLabelsID) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle),
	fAGAControl(NULL),
	fLabelsID(itsLabelsID)
{
	// caller must call virtual function CreateAGAControl or HandlePostCreate.
}

//----------------------------------------------------------------------------------------
// TSlider destructor
//----------------------------------------------------------------------------------------

TSlider::~TSlider()
{
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::HandleNotification(SInt32 /*dataValue*/)
{
	// Propagate our event number.
	
	HandleEvent(fEventNumber, this, NULL);

	(void) Focus();	// HandleEvent may have caused another view to draw
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);
	
	CreateAGAControl();
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::CreateAGAControl()
{
	fTextStyle.Normalize(gSystemStyle.tsFont, gApplicationStyle.tsFont);
	fAGAControl = TH_new CSlider_AC(ControlQDArea(), fMinimum, fMaximum, fInitialValue, fTextStyle, fLabelsID);
	if (fRightJustify)
		fAGAControl->SetJustification(teFlushRight);
	fAGAControl->SetPageSize(fStep, CControl_AC::kDontRedraw);
	
	fAGAControl->InstallNotificationRoutine((CIndicatorCallback_AC) RealAGANotifier, this);

	if (fDimmed || ! fEnabled)
		fAGAControl->SetEnable(CControl_AC::kDisabled, CControl_AC::kDontRedraw);

	// Slider has to erase when drawing, so make sure that
	// it doesn't assume we're on a gray background.
	
	(void) Focus();
	fAGAControl->InstallBackgroundColors(CControl_AC::kInstallOnlyIfCustom);
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::Draw(const CViewRect& /*area*/)
{
	if (fAGAControl)
		fAGAControl->DrawObject();
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	
	if (fAGAControl)
		fAGAControl->SetBounds(ControlQDArea(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* /*event*/, CPoint_AC /*hysteresis*/)
{
	if (fAGAControl->TrackMouse(ViewToQD(theMouse)))
		HandleEvent(fEventNumber, this, NULL);
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::Activate(bool entering)
{
	if (fAGAControl)
		fAGAControl->SetActive(entering, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::DimState(bool state, bool redraw)
{
	fDimmed = state;

	if (fAGAControl)
		fAGAControl->SetEnable(!state, redraw && Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::Dim()
{
	if (fAGAControl)
		fAGAControl->SetEnable(CControl_AC::kDisabled, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::SetEnable(bool state)
{
	Inherited::SetEnable(state);

	// Set the CControl_AC's enable state, without redraw.
	if (fAGAControl)
		fAGAControl->SetEnable(state, CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::ReadFields(CStream_AC* aStream)
{
	Inherited::ReadFields(aStream);
	
	*aStream >> fLabelsID
			 >> fRightJustify
			 >> fMinimum
			 >> fMaximum
			 >> fInitialValue
			 >> fStep;
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::WriteFields(CStream_AC* aStream) const
{
	Inherited::WriteFields(aStream);
	
	*aStream << fLabelsID
			 << fRightJustify
			 << fMinimum
			 << fMaximum
			 << fInitialValue
			 << fStep;
}

//----------------------------------------------------------------------------------------
// TSlider
//----------------------------------------------------------------------------------------

void TSlider::RealAGANotifier(CIndicator_AC* /*theIndicator*/, SInt32 dataValue, TSlider* notifierTarget)
{
	// This is a static function.

	notifierTarget->HandleNotification(dataValue);
}
