//----------------------------------------------------------------------------------------
// UStaticText.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USTATICTEXT__
#include "UStaticText.h"
#endif

#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif

//temp
#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __USUBSTITUTION__
#include "USubstitution.h"
#endif

#ifndef __UDRAGDROP__
#include "UDragDrop.h"
#endif

#if qGrayscaleAppearance

#	ifndef __AGA_AC__
#	include "AGA_AC.h"
#	endif

#endif // qGrayscaleAppearance

void InitUStaticText()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TStaticText, kStdStaticText);
	}
}
	
//========================================================================================
// CLASS TStaticText
//========================================================================================
#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS(TStaticText);

//----------------------------------------------------------------------------------------
// TStaticText constructor
//----------------------------------------------------------------------------------------

TStaticText::TStaticText(const CViewPoint& itsLocation,
							  const CViewPoint& itsSize,
							  SizeDeterminer itsHSizeDet,
							  SizeDeterminer itsVSizeDet,
							  ResNumber itsRsrcID,
							  short itsIndex):
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),
#if qGrayscaleAppearance
	fAGAControl(NULL),
#endif
	fDataHandle(NULL),
	fStrListID(itsRsrcID),
	fIndex(itsIndex),
	fJust(teFlushDefault),
	fAutoWrap(true),
	fEraseFirst(kNoEraseFirst)
{
	fEventNumber = mStaticTextHit;

	if (fStrListID != kNoResource)
	{
		CStr255_AC aString(fStrListID, fIndex);
		::ThrowIfResError_AC();
		SetText(aString, kDontRedraw);
	}
	SetEnable(false);						// Default is to not enable hit testing 
}

//----------------------------------------------------------------------------------------
// TStaticText copy constructor
//----------------------------------------------------------------------------------------

TStaticText::TStaticText(const TStaticText& inStaticText) :
#if qGrayscaleAppearance
	fAGAControl(inStaticText.fAGAControl.get()),
#endif
	fDataHandle(NULL),
	fStrListID(inStaticText.fStrListID),
	fIndex(inStaticText.fIndex),
	fJust(inStaticText.fJust),
	fAutoWrap(inStaticText.fAutoWrap),
	fEraseFirst(inStaticText.fEraseFirst)
{	
	fDataHandle = (CPascalStrHandle_AC)PermCopyHandle((Handle) inStaticText.fDataHandle);
}

//----------------------------------------------------------------------------------------
// TStaticText::Free: 
//----------------------------------------------------------------------------------------

TStaticText::~TStaticText()
{
	ReleaseText();
}

//----------------------------------------------------------------------------------------
// TStaticText::GetStandardSignature: 
//----------------------------------------------------------------------------------------

IDType TStaticText::GetStandardSignature() const	// Override 
{
	return kStdStaticText;
}

//----------------------------------------------------------------------------------------
// TStaticText::ReadFields: 
//----------------------------------------------------------------------------------------

void TStaticText::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	fAutoWrap = aStream->ReadBoolean();
	fEraseFirst = aStream->ReadBoolean();

	fJust = aStream->ReadInteger();
	
	CStr255_AC itsText;
	fStrListID = aStream->ReadInteger();
	fIndex = aStream->ReadInteger();
	if (fStrListID != kNoResource)					// retrieve the static text from the resource
		GetIndString(itsText, fStrListID, fIndex);
	SetText(itsText, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TStaticText::WriteFields: 
//----------------------------------------------------------------------------------------

void TStaticText::WriteFields(CStream_AC* aStream) const				// Override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteBoolean(fAutoWrap);
	aStream->WriteBoolean(fEraseFirst);

	aStream->WriteInteger(fJust);

	aStream->WriteInteger(fStrListID);
	aStream->WriteInteger(fIndex);
}

//----------------------------------------------------------------------------------------
// TStaticText::ChangeWrap: 
//----------------------------------------------------------------------------------------

void TStaticText::ChangeWrap(bool newAutoWrap,
							 bool redraw)
{
	fAutoWrap = newAutoWrap;
	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TStaticText::DoSubstitution: 
//----------------------------------------------------------------------------------------

void TStaticText::DoSubstitution(CStr255_AC& theText)

{
	MAReplaceText(theText);
}

//----------------------------------------------------------------------------------------
// TStaticText::Draw: 
//----------------------------------------------------------------------------------------

void TStaticText::Draw(const CViewRect& area)	// Override 
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
	// To properly mimic TStaticText, we need to do text substitution
	// and use the potentially modified text.	
		if (fAGAControl)
		{
			CStr255_AC	theText(GetText());
			this->DoSubstitution(theText);

			fAGAControl->SetTitle(theText, kDontRedraw);
			fAGAControl->DrawObject();
		}
	}
	else
#endif
	if (fDataHandle)
	{
		CStr255_AC theText(GetText());

		DoSubstitution(theText);			// Make the substitution if desired 

		ImageText((Ptr)(((long) & theText) + 1), theText.Length(), ControlArea(), fJust);
	}
	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TStaticText::GetText: 
//----------------------------------------------------------------------------------------

CStr255_AC TStaticText::GetText() const
{
	return fDataHandle ? *reinterpret_cast<StringHandle>(fDataHandle) : NULL;
}

//----------------------------------------------------------------------------------------
// TStaticText::ImageText: 
//----------------------------------------------------------------------------------------

void TStaticText::ImageText(Ptr text,
							long Length,
							const CViewRect& box,
							short just)
{
	TextBox_AC(text, Length, ViewToQD(box), just, fAutoWrap, NULL, fEraseFirst,
		kSpaceForCaret, fPreferOutline);
}

//----------------------------------------------------------------------------------------
// TStaticText::ReleaseText: 
//----------------------------------------------------------------------------------------

void TStaticText::ReleaseText()
{
	fDataHandle = (CPascalStrHandle_AC)::DisposeIfHandle_AC((Handle)fDataHandle);
}

//----------------------------------------------------------------------------------------
// TStaticText::SetJustification: 
//----------------------------------------------------------------------------------------

void TStaticText::SetJustification(short theJust,
								   bool redraw)
{
	fJust = theJust;
	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TStaticText::SetText: 
//----------------------------------------------------------------------------------------

void TStaticText::SetText(const CStr255_AC& theText,
						  bool redrawImmediately)
{
#if qGrayscaleAppearance
	if (IsEmulatingControls())
	{
		// Do NOT let TStaticText redraw, because it erases to white.
		bool redraw = redrawImmediately;
		redrawImmediately = false;
		if (fAGAControl)
			fAGAControl->SetTitle(theText, redraw && Focus() && IsVisible());
	}
#endif

	if (!fDataHandle || (theText != **(CStr255_AC**)fDataHandle))
	{
		ReleaseText();
		fDataHandle = (CPascalStrHandle_AC)NewString(theText);
		if (MemError() != noErr)
			fDataHandle = NULL;

		if (IsDrawable())
		{
			Invalidate(ControlQDArea());
			
			if (redrawImmediately)
				Update();
		}
	}
}

//----------------------------------------------------------------------------------------
// TStaticText::SetTextWithStrListID: 
//----------------------------------------------------------------------------------------

void TStaticText::SetTextWithStrListID(ResNumber theStrListID,
									   short theIndex,
									   bool redraw)
{
	CStr255_AC		aString;
	fStrListID = theStrListID;
	fIndex = theIndex;
	if (fStrListID != kNoResource)
	{
		aString.CopyFrom(fStrListID, fIndex, aString.kMaxLength);
		::ThrowIfResError_AC();
	}
	SetText(aString, redraw);
}

#if qDrag

//----------------------------------------------------------------------------------------
// TStaticText::DoAddDragContent: 
//----------------------------------------------------------------------------------------

void TStaticText::DoAddDragContent()
{
	TDragItem 		*dragItem = TDragDropSession::fgDragDropSession->AddDragItem(1);
	CFlavorFlags	flags;
	
	CStr255_AC theText(GetText());
	DoSubstitution(theText);
	
	dragItem->AddFlavor(kTextType_AC, flags, (Ptr)(((long)&theText)+1), theText.Length());
}

#endif // qDrag

#if qGrayscaleAppearance

//----------------------------------------------------------------------------------------
// TStaticText::DoPostCreate: 
//----------------------------------------------------------------------------------------

void TStaticText::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	CreateAGAControl();
}

//----------------------------------------------------------------------------------------
// TStaticText::Activate: 
//----------------------------------------------------------------------------------------

void TStaticText::Activate(bool entering)
{
	if (IsEmulatingControls())
	{
		if (fAGAControl)
			fAGAControl->SetActive(entering, Focus() && IsVisible());
	}
	
	Inherited::Activate(entering);
}

//----------------------------------------------------------------------------------------
// TStaticText::DimState: 
//----------------------------------------------------------------------------------------

void TStaticText::DimState(bool state, bool redraw)
{
	if (IsEmulatingControls())
	{
		fDimmed = state;

		if (fAGAControl)
			fAGAControl->SetEnable(!state, redraw && Focus() && IsVisible());
	}
	else
		Inherited::DimState(state, redraw);
}

//----------------------------------------------------------------------------------------
// TStaticText::Dim: 
//----------------------------------------------------------------------------------------

void TStaticText::Dim()
{
	if (IsEmulatingControls())
	{
		if (fAGAControl)
			fAGAControl->SetEnable(CControl_AC::kDisabled, Focus() && IsVisible());
	}
	else
		Inherited::Dim();
}

//----------------------------------------------------------------------------------------
// TStaticText::InstallTextStyle: 
//----------------------------------------------------------------------------------------

void TStaticText::InstallTextStyle(const CTextStyle_AC& theTextStyle, bool redraw)
{
	// Change the text style of the AGA object.
	
	if (fAGAControl)
		fAGAControl->SetTextStyle(theTextStyle, kDontRedraw);

	Inherited::InstallTextStyle(theTextStyle, redraw);
}

//----------------------------------------------------------------------------------------
// TStaticText::SetFrame: 
//----------------------------------------------------------------------------------------

void TStaticText::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	
	if (fAGAControl)
		fAGAControl->SetBounds(ControlQDArea(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TStaticText::CreateAGAControl: 
//----------------------------------------------------------------------------------------

void TStaticText::CreateAGAControl()
{
	// Instantiate the particular CControl_AC subclass.
	fTextStyle.Normalize(gSystemStyle.tsFont, gApplicationStyle.tsFont);
	fAGAControl = TH_new CStaticText_AC(ControlQDArea(), fTextStyle, fJust, fEraseFirst, GetText());

	// Note that unlike other controls, for static text, we allow
	// "disabled" text to draw normal. This is because static text
	// views typically are "disabled" but not "dimmed".
	if (fDimmed)
		fAGAControl->SetEnable(CControl_AC::kDisabled, CControl_AC::kDontRedraw);

	// A static text paints its background, so make sure that
	// it doesn't assume we're on a gray background.

	this->Focus();
	fAGAControl->InstallBackgroundColors(CControl_AC::kInstallOnlyIfCustom);
}

//----------------------------------------------------------------------------------------
// TStaticText::IsEmulatingControls: 
//----------------------------------------------------------------------------------------

bool TStaticText::IsEmulatingControls() const
{
	// Using EmulateControls_AC() prevents us from using the behavior under Mac OS 8
	return /* EmulateControls_AC() && */ GetStandardSignature() == kStdStaticText;
}

#endif // qGrayscaleAppearance
