//----------------------------------------------------------------------------------------
// UTECommands.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UTECOMMANDS__
#include "UTECommands.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UDIALOGTEVIEW__
#include "UDialogTEView.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UEDITTEXT__
#include "UEditText.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USTATICTEXT__
#include "UStaticText.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __ALIASES__
#include <Aliases.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


#ifndef __STRING__
#include <string.h>
#endif


//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

#if qDebug

//----------------------------------------------------------------------------------------
// DumpTTECommand: 
//----------------------------------------------------------------------------------------

void DumpTTECommand(TTECommand* theTTECommand)
{
	long oldStyleSize;
	long newStyleSize;
	long last;

	long oldDiff = Max_AC(theTTECommand->fOldEnd - theTTECommand->fOldStart, 0L);
	long newDiff = Max_AC(theTTECommand->fNewEnd - theTTECommand->fNewStart, 0L);

	ProgramReport_AC(false, "old start/end, TH_new start/end, diff :%1d/%1d,  %1d/%1d,  %1d \n", theTTECommand->fOldStart, theTTECommand->fOldEnd, theTTECommand->fNewStart, theTTECommand->fNewEnd, Max_AC(oldDiff, newDiff));
	ProgramReport_AC(false, "fTextPad :%1d\n", theTTECommand->fTextPad);
	ProgramReport_AC(false, "fPadding size :%1d\n", GetHandleSize(theTTECommand->fPadding));

	oldStyleSize = 0;
	newStyleSize = 0;
	ProgramReport_AC("** Old styles:\n");

	if (!theTTECommand->fOldStyles)
		ProgramReport_AC("NONE!\n");
	else
	{

		last = (*theTTECommand->fOldStyles)->scrpNStyles;
		ProgramReport_AC(false, "Number of table entries: %1d\n", last);
		for (long i = 0; i <= last - 1; ++i)
		{
			ScrpSTElement & theScrpSTElement = (*theTTECommand->fOldStyles)->scrpStyleTab[i];

			ProgramReport_AC(false, "%1d, ofs:%1d ", theScrpSTElement.scrpStartChar);
			ProgramReport_AC("\n");
		}
		oldStyleSize = GetHandleSize((Handle)(theTTECommand->fOldStyles));
	}

	ProgramReport_AC("** New styles:\n");

	if (!theTTECommand->fNewStyles)
		ProgramReport_AC("NONE!\n");
	else
	{
		last = (*theTTECommand->fNewStyles)->scrpNStyles;
		ProgramReport_AC(false, "Number of table entries: %1d\n", last);
		for (long i = 0; i <= last - 1; ++i)
		{
			ScrpSTElement & theScrpSTElement = (*theTTECommand->fNewStyles)->scrpStyleTab[i];

			ProgramReport_AC(false, "%1d, ofs:%1d ", theScrpSTElement.scrpStartChar);
			ProgramReport_AC("\n");
		}
		newStyleSize = GetHandleSize((Handle)(theTTECommand->fNewStyles));
	}

	ProgramReport_AC(false, "Styles size diff: %1d\n", Max_AC(newStyleSize, oldStyleSize));
	ProgramReport_AC(false, "fStylePad :%1d\n", theTTECommand->fStylePad);
}

#endif


//========================================================================================
// CLASS TTECommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TTECommand constructor
//----------------------------------------------------------------------------------------

TTECommand::TTECommand(TTEView* itsTEView,
						CommandNumber itsCommandNumber,
						bool itsSaveText) :
	TCommand(itsCommandNumber, NULL, kCanUndo, kCausesChange, NULL),

	fHTE(NULL),
	fNewEnd(0),
	fNewStart(0),
	fNewStyles(NULL),
	fNewText(NULL),
	fOldEnd(0),
	fOldStart(0),
	fOldStyles(NULL),
	fOldText(NULL),
	fOldStaticText(),
	fPadding(NULL),
	fStylePad(0),
	fTEView(NULL),
	fEditText(NULL),
	fTextPad(0)
{
	CommonInit(itsTEView, itsCommandNumber, itsSaveText);
}

//----------------------------------------------------------------------------------------
// TTECommand::ITECommand: 
//----------------------------------------------------------------------------------------

TTECommand::TTECommand(TEditText* itsEditText,
						CommandNumber itsCommandNumber,
						bool itsSaveText) :
	TCommand(itsCommandNumber, NULL, kCanUndo, kCausesChange, NULL),

	fHTE(NULL),
	fNewEnd(0),
	fNewStart(0),
	fNewStyles(NULL),
	fNewText(NULL),
	fOldEnd(0),
	fOldStart(0),
	fOldStyles(NULL),
	fOldText(NULL),
	fOldStaticText(),
	fPadding(NULL),
	fStylePad(0),
	fTEView(NULL),
	fEditText(NULL),
	fTextPad(0)
{
	fEditText = itsEditText;
	AssureTEView();

	CommonInit(fTEView, itsCommandNumber, itsSaveText);

	ForgetTEView();
}

//----------------------------------------------------------------------------------------
// TTECommand::ITECommand: 
//----------------------------------------------------------------------------------------

void TTECommand::CommonInit(TTEView* itsTEView,
							CommandNumber itsCommandNumber,
							bool itsSaveText)
{
	fContext = itsTEView ? itsTEView->GetContext(itsCommandNumber) : NULL;
	fObjectToNotify = fContext;

	fTEView = itsTEView;
	fEditText = itsTEView->GetEditText();

	fHTE = itsTEView->fHTE;

	short selChars;
	{
		TERec & theTERec = **fHTE;

		fOldStart = theTERec.selStart;
		fOldEnd = theTERec.selEnd;
		selChars = theTERec.selEnd - theTERec.selStart;
	}

	if (itsSaveText)
	{
		Handle h = NewPermHandle(selChars);

		if (selChars > 0)
			BlockMove_AC((Ptr)((*((*fHTE)->hText)) + fOldStart), (*h), selChars);

		fOldText = h;
		fTextPad = fOldStart - fOldEnd;
		fPadding = NewPermHandle(0);

		if (fEditText)
			fOldStaticText = fEditText->TStaticText::GetText();
	}

	// TextEdit has this "feature" which it exercises if it runs out of memory. It's
	// called DS number 25. We'll try to avoid it by assuring that enough memory
	// exists to fulfill the request, but we won't die because of it. This is a
	// particularly ugly situation - there could be >600K of style information
	// associated with a 32K block of text. And to support undo, we've got to assume
	// that there may momentarily be THREE copies floating around, adding up to a
	// total potential liability of almost 2 Meg for a single TE record. The worst
	// that can happen, though, is that the text will be safe, but it won't have any
	// styles associated with it.

	if (itsTEView->SpaceForStyles((*fHTE)->selStart, (*fHTE)->selEnd))
	{
		CTempAllocation tempAllocation(false);
		fOldStyles = TEGetStyleScrapHandle(fHTE);

		::ThrowIfNULL_AC(fOldStyles);

		fStylePad = GetHandleSize((Handle)fOldStyles);
	}
}

//----------------------------------------------------------------------------------------
// TTECommand destructor 
//----------------------------------------------------------------------------------------

TTECommand::~TTECommand()
{
	fOldText = ::DisposeIfHandle_AC(fOldText);
	fOldStyles = (StScrpHandle)::DisposeIfHandle_AC((Handle)fOldStyles);
	fNewText = ::DisposeIfHandle_AC(fNewText);
	fNewStyles = (StScrpHandle)::DisposeIfHandle_AC((Handle)fNewStyles);
	fPadding = ::DisposeIfHandle_AC(fPadding);
}

//----------------------------------------------------------------------------------------
// TTECommand::AssureTEView: 
//----------------------------------------------------------------------------------------

void TTECommand::AssureTEView()
{
	if (fEditText)
	{
		if (!fEditText->fTEView)
			fEditText->InstallFloatingTEView(false);

		fTEView = fEditText->fTEView;
		fHTE = fTEView->fHTE;
	}
}

//----------------------------------------------------------------------------------------
// TTECommand::ForgetTEView: 
//----------------------------------------------------------------------------------------

void TTECommand::ForgetTEView()
{
	if (fEditText)
	{
		fTEView = NULL;							// null these references so dependency
		fHTE = NULL;							// notifications work correctly.
	}
}

//----------------------------------------------------------------------------------------
// TTECommand::DoPostProcess
//----------------------------------------------------------------------------------------

void TTECommand::DoPostProcess()
{
	if (fEditText)
	{
		TWindow * theWindow = fEditText->GetWindow();
		TEventHandler * windowTarget = theWindow ? theWindow->GetWindowTarget() : NULL;

		if (!theWindow || (windowTarget != fTEView && windowTarget != fEditText))
		{
			if (!fEditText->IsValid())
				SetValidationError((OSErr)kInvalidValue);
			fEditText->StopEdit();
		}
		ForgetTEView();
	}
}

//----------------------------------------------------------------------------------------
// TTECommand::BanishOldText: 
//----------------------------------------------------------------------------------------

void TTECommand::BanishOldText()
{
	if (fOldEnd > fOldStart)
		TEDelete(fHTE);
	if (fPadding)
		SetPermHandleSize(fPadding, Max_AC(-(fTextPad + fStylePad), 0));
}

//----------------------------------------------------------------------------------------
// TTECommand::InstallNewText: 
//----------------------------------------------------------------------------------------

void TTECommand::InstallNewText()
{
	if (fNewEnd > fNewStart)
	{
		Handle itsText = fTEView->fText;
		long savedSize = GetHandleSize(itsText);

#if qDebug
		if (!fNewText)
			::ProgramBreak_AC("InstallNewText called with fNewText == nil!");
#endif

		CTempHandleLock_AC tempHandleLock(fNewText, eMoveHi); // Prevent heap fragmentation for TEInsert 

		// Replace invalid characters with spaces.
		OSErr err;
		CStr15_AC key;

		bool allocationState = TemporaryAllocation(false);
		Handle space;
		unsigned char charSpace = chSpace;
		err = PtrToHand((Ptr)&charSpace, &space, 1);
		TemporaryAllocation(allocationState);

		::ThrowIfOSErr_AC(err);
		
		for (unsigned char i = 0; i < chSpace; ++i) {
			if (!macroIn(fTEView->fControlChars, macroAsSetElem(i)))
			{
				key = i;
				err = ReplaceText(fNewText, space, key);
				if (err < 0) ::ThrowIfOSErr_AC(err);
			}
		}
		fTEView->StyleInsert((*fNewText), GetHandleSize(fNewText), fNewStyles);

		if (GetHandleSize(itsText) <= savedSize)
			::ThrowIfOSErr_AC(memFullErr);
	}
}

//----------------------------------------------------------------------------------------
// TTECommand::RemoveAdditions: 
//----------------------------------------------------------------------------------------

void TTECommand::RemoveAdditions()
{
	if (fNewText)
	{
		fTEView->SetSelection(fNewStart, fNewEnd, !kRedraw);
		TEDelete(fHTE);
	}
	if (fPadding)
		SetPermHandleSize(fPadding, Max_AC(fTextPad + fStylePad, 0));
}

//----------------------------------------------------------------------------------------
// TTECommand::RestoreSelection: 
//----------------------------------------------------------------------------------------

void TTECommand::RestoreSelection()
{
	fTEView->SetSelection(fOldStart, fOldEnd, kRedraw);
}

//----------------------------------------------------------------------------------------
// TTECommand::PrepareForUndoRedo:
//----------------------------------------------------------------------------------------

void TTECommand::PrepareForUndoRedo()
{
	if (fEditText)
		fEditText->PrepareForUndoRedo(this);
	else if (fTEView)
		fTEView->PrepareForUndoRedo(this);
	else
		Inherited::PrepareForUndoRedo();
}

//----------------------------------------------------------------------------------------
// TTECommand::RevealUndoRedo:
//----------------------------------------------------------------------------------------

void TTECommand::RevealUndoRedo()
{
	if (fEditText)
		fEditText->RevealUndoRedo(this);
	else if (fTEView)
		fTEView->RevealUndoRedo(this);
	else
		Inherited::RevealUndoRedo();
}

//----------------------------------------------------------------------------------------
// TTECommand::ReviveDeletions: 
//----------------------------------------------------------------------------------------

void TTECommand::ReviveDeletions()
{
	fTEView->SetSelection(fOldStart, fOldStart, !kRedraw);	// so insert will take place at right point 
	short nChars = static_cast<short>(GetHandleSize(fOldText));
	if (nChars > 0)
	{
		Handle itsText = fTEView->fText;
		long savedSize = GetHandleSize(itsText);

		CTempHandleLock_AC tempHandleLock(fOldText, eMoveHi); // Prevent heap fragmentation 

		fTEView->StyleInsert((*fOldText), nChars, fOldStyles);

		if (GetHandleSize(itsText) <= savedSize)
			::ThrowIfOSErr_AC(memFullErr);
	}

	if (fEditText)
		fEditText->TStaticText::SetText(fOldStaticText, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TTECommand::DoMainFunction: 
//----------------------------------------------------------------------------------------

void TTECommand::DoMainFunction()
{
	if (fIdentifier != cCopy)
		BanishOldText();
	InstallNewText();
	if (fIdentifier != cCopy)
		fTEView->SynchView(kRedraw);
}

//----------------------------------------------------------------------------------------
// TTECommand::SetupDependencies:
//----------------------------------------------------------------------------------------

void TTECommand::SetupDependencies()
{
	Inherited::SetupDependencies();

	if (CanBeUndone())
	{
		if (fEditText)
			fEditText->AddDependent(this);
		else if (fTEView)
			fTEView->AddDependent(this);
	}
}

//----------------------------------------------------------------------------------------
// TCommand::DoUpdate:
//----------------------------------------------------------------------------------------

void TTECommand::DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject,
						  void* changeData,
						  CDependencySpace_AC* dependencySpace)// Override
{
	if ((theChange == mClosed) && ((changedObject == fTEView) || changedObject == fEditText))
		fContext->CommitLastCommand();
	else
		Inherited::DoUpdate(theChange, changedObject, changeData, dependencySpace);
}

//----------------------------------------------------------------------------------------
// TTECommand::DoIt: 
//----------------------------------------------------------------------------------------

void TTECommand::DoIt()							// override 
{
	AssureTEView();
	fTEView->Focus();

	DoMainFunction();

#if qDebug
	if (TTEView::fgTEIntenseDebugging)
		DumpTTECommand(this);
#endif

	DoPostProcess();
}

//----------------------------------------------------------------------------------------
// TTECommand::UndoIt: 
//----------------------------------------------------------------------------------------

void TTECommand::UndoIt()						// override 
{
	AssureTEView();
	fTEView->Focus();

	RemoveAdditions();
	if (fOldText || !fOldStaticText.IsEmpty())
		ReviveDeletions();

	RestoreSelection();
	if (fIdentifier != cCopy)
		fTEView->SynchView(kRedraw);

#if qDebug
	if (TTEView::fgTEIntenseDebugging)
		DumpTTECommand(this);
#endif

	DoPostProcess();
}

//----------------------------------------------------------------------------------------
// TTECommand::RedoIt: 
//----------------------------------------------------------------------------------------

void TTECommand::RedoIt()						// override 
{
	AssureTEView();
	fTEView->Focus();

	RestoreSelection();
	DoMainFunction();

#if qDebug
	if (TTEView::fgTEIntenseDebugging)
		DumpTTECommand(this);
#endif

	DoPostProcess();
}


//========================================================================================
// CLASS TTECutCopyCommand
//========================================================================================
#undef Inherited
#define Inherited TTECommand

//----------------------------------------------------------------------------------------
// TTECutCopyCommand::TTECutCopyCommand
//----------------------------------------------------------------------------------------

TTECutCopyCommand::TTECutCopyCommand(TTEView* itsTEView, CommandNumber itsCommandNumber) :
	TTECommand(itsTEView, itsCommandNumber, true),

	fClipCreated(false)
{
	fChangesClipboard = true;
	fCausesChange = (itsCommandNumber != cCopy);
	fCanUndo = fCausesChange;
}

//----------------------------------------------------------------------------------------
// TTECutCopyCommand::Free: 
//----------------------------------------------------------------------------------------

TTECutCopyCommand::~TTECutCopyCommand()
{
	if (fClipCreated)
		fOldText = NULL;
}

//----------------------------------------------------------------------------------------
// TTECutCopyCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TTECutCopyCommand::DoIt()					// override 
{
	fTEView->Focus();

	CTextStyle_AC clipStyle(applFont, 0, 12, gRGBBlack);// Initial style same as virgin TEView 

	CViewPoint itsSize(100, 50);					// An arbitrary initial size.
	CViewRect itsMargins(10, 8, 10, 0);				// No bottom margin.

	CAutoPtr_AC<TTEView> clipTEView = TH_new TTEView(gZeroVPt, itsSize, sizeSuperView, sizeVariable, itsMargins, clipStyle, teFlushDefault, true, true);
	clipTEView->fAcceptsChanges = false;		// This is a read-only view 

	// Cut can eat into temp memory so users can rescue text from overweight documents 	
	if (!WillCauseChange())						// If Copy-ing, assure there's enough room 
		FailSpaceIsLow();

	clipTEView->StuffText(fOldText);
	FailSpaceIsLow();

	TEFeatureFlag( teFOutlineHilite, teBitClear, clipTEView->fHTE ); // Turn off gray caret.
	
	TEUseStyleScrap(0, SHRT_MAX, fOldStyles,// then put in the styles 
						kDontRedraw, clipTEView->fHTE);
	FailSpaceIsLow();

	clipTEView->fFreeText = true;				// Let TEView know it has to free the text 

	ClaimClipboard(clipTEView);					// Okay to claim (will call RecalcText!) 
	clipTEView.release();

	fClipCreated = true;						// We be done 
	DoMainFunction();							// Do the actual cut/copy 

#if qDebug
	if (TTEView::fgTEIntenseDebugging)
	{
		TTEView::DumpTERecord(clipTEView->fHTE);
		DumpTTECommand(this);
	}
#endif
}

//----------------------------------------------------------------------------------------
// TTECutCopyCommand::ReviveDeletions: 
//----------------------------------------------------------------------------------------

void TTECutCopyCommand::ReviveDeletions()		// override 
{
	if (fIdentifier == cCut)
		Inherited::ReviveDeletions();			// Don't do it for COPY 
}


//========================================================================================
// CLASS TTEPasteCommand
//========================================================================================
#undef Inherited
#define Inherited TTECommand

//----------------------------------------------------------------------------------------
// TTEPasteCommand::ITEPasteCommand: We can't use TEPaste because it clobbers the
// DeskScrap; the text would be recoverable from the special TextEdit Scrap, but other
// types of non-TEXT scrap are permanently lost, it seems
//----------------------------------------------------------------------------------------

TTEPasteCommand::TTEPasteCommand(TTEView* itsTEView) :
	TTECommand(itsTEView, cPaste, true)		// Perform stock initializations 
{
	Handle newText = NULL;
	StScrpHandle newStyles = NULL;

	try
	{
		newText = NewPermHandle(0);				// Create handle to receive clipboard data 
		newStyles = (StScrpHandle)NewPermHandle(0);// Same for handle to receive style info 

		ResType dataType = '%%%%';
		short newLength = static_cast<short>(gClipboardMgr->GetDataToPaste(newText, dataType));

		if (newLength > 0)
		{
#if qDebug
			if (dataType != kTextType_AC)
				::ProgramBreak_AC("TEPasteCommand given some non-text from clipboard");
			else
#endif

				{								// Prime "new" values 
					fNewText = newText;
					fNewStart = (*fHTE)->selStart;
					fNewEnd = fNewStart + newLength;
					fTextPad = newLength - (fOldEnd - fOldStart);

					long newStyleLen = gClipboardMgr->fClipView->GivePasteData((Handle)newStyles, kStyleType_AC);
					if (newStyleLen > 0)
					{
						fNewStyles = newStyles;
						fStylePad =			// Difference between old and new styles 
								   newStyleLen - fStylePad;
					}
					else
						newStyles = (StScrpHandle)::DisposeIfHandle_AC((Handle)newStyles);

					if (fPadding)
						SetPermHandleSize(fPadding, Max_AC(fTextPad + fStylePad, 0));

					FailSpaceIsLow();
				}
		}
		else
		{
			newText = ::DisposeIfHandle_AC(newText);
			newStyles = (StScrpHandle)::DisposeIfHandle_AC((Handle)newStyles);
		}

		if (fEditText)
			fEditText->TStaticText::SetText(fEditText->GetText(), kDontRedraw);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		if (newText != fNewText)			// newText is assigned to fNewText 
			newText = ::DisposeIfHandle_AC(newText);// so avoid disposing twice. 
		if (newStyles != fNewStyles)		// Ditto for newStyles. 
			newStyles = (StScrpHandle)::DisposeIfHandle_AC((Handle)newStyles);
		throw;
	}
}

//----------------------------------------------------------------------------------------
// TTEPasteCommand destructor
//----------------------------------------------------------------------------------------

TTEPasteCommand::~TTEPasteCommand()
{
}

//========================================================================================
// CLASS TTEStyleCommand
//========================================================================================
#undef Inherited
#define Inherited TTECommand

//----------------------------------------------------------------------------------------
// TTEStyleCommand constructor
//----------------------------------------------------------------------------------------

TTEStyleCommand::TTEStyleCommand(TTEView* itsTEView,
								 const CTextStyle_AC& itsNewStyle,
								 CommandNumber itsCommandNumber,
								 short itsMode) :
	TTECommand(itsTEView, itsCommandNumber, false),// Perform stock initialization, sans text 

	fOldTextStyle(itsTEView ? itsTEView->fTextStyle : itsNewStyle),
	fNewTextStyle(itsNewStyle),

	// Only do color change if we can 
	fMode((qNeedsColorQD || HasColorQD_AC()) ? itsMode : itsMode & ~doColor)
{
}

//----------------------------------------------------------------------------------------
// TTEStyleCommand destructor
//----------------------------------------------------------------------------------------

TTEStyleCommand::~TTEStyleCommand()
{
}

//----------------------------------------------------------------------------------------
// TTEStyleCommand::InstallOneStyle: 
//----------------------------------------------------------------------------------------

void TTEStyleCommand::InstallOneStyle(const CTextStyle_AC& newStyl)
{
	fTEView->SetOneStyle(fOldStart, fOldEnd, fMode, newStyl, kRedraw);// Focus'es for us 
}

//----------------------------------------------------------------------------------------
// TTEStyleCommand::InstallManyStyles: 
//----------------------------------------------------------------------------------------

void TTEStyleCommand::InstallManyStyles(StScrpHandle newStyls)
{
	fTEView->Focus();
	TEUseStyleScrap(fOldStart, fOldEnd, newStyls, kRedraw, fHTE);

	if (fOldEnd == fOldStart)
	{
		CTextStyle_AC theStyle(*(TextStyle*)&(*newStyls)->scrpStyleTab[0].scrpFont);

		TESetStyle(doAll, &theStyle, kRedraw, fHTE);
	}

	fTEView->RecalcText();						// Might have changed number of lines 
	fTEView->SynchView(kRedraw);				// Show corrected view 
}

//----------------------------------------------------------------------------------------
// TTEStyleCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TTEStyleCommand::DoIt()					// override 
{
	InstallOneStyle(fNewTextStyle);

#if qDebug
	if (TTEView::fgTEIntenseDebugging)
		DumpTTECommand(this);
#endif

}

//----------------------------------------------------------------------------------------
// TTEStyleCommand::UndoIt: 
//----------------------------------------------------------------------------------------

void TTEStyleCommand::UndoIt()					// override 
{
	fTEView->Focus();

	RestoreSelection();

	InstallManyStyles(fOldStyles);
#if qDebug
	if (TTEView::fgTEIntenseDebugging)
		DumpTTECommand(this);
#endif

}

//----------------------------------------------------------------------------------------
// TTEStyleCommand::RedoIt: 
//----------------------------------------------------------------------------------------

void TTEStyleCommand::RedoIt()					// override 
{
	fTEView->Focus();
	RestoreSelection();
	DoIt();
}


//========================================================================================
// CLASS TTETypingCommand
//========================================================================================
#undef Inherited
#define Inherited TTECommand

//----------------------------------------------------------------------------------------
// TTETypingCommand constructor
//----------------------------------------------------------------------------------------

TTETypingCommand::TTETypingCommand(TTEView* itsTEView, const CStr2_AC& itsFirstCh) :
	TTECommand(itsTEView, cTyping, true),

	fFirstChar(itsFirstCh),				// Save character[s] for Doit 
	fCompleted(false)
{
	fNewStart = (*fHTE)->selStart;		// Start and end are the same 
	fNewEnd = fNewStart;

	fNewText = NewPermHandle(0);		// Allocate an empty block for text 

	AddCharacter(itsFirstCh);			// [1115111] moved here from DoIt
}

//----------------------------------------------------------------------------------------
// TTETypingCommand destructor 
//----------------------------------------------------------------------------------------

TTETypingCommand::~TTETypingCommand()
{
	if (fTEView)
	{
		fTEView->ForgetIfTypingCommand(this);
	}
	if (fEditText)
	{
		TDialogTEView* teView = fEditText->fTEView;
		if (teView)
			teView->ForgetIfTypingCommand(this);
	}
}

//----------------------------------------------------------------------------------------
// TTETypingCommand::DoNormalChar: 
//----------------------------------------------------------------------------------------

void TTETypingCommand::DoNormalChar(const CStr2_AC& ch)
{
	// Append char[s] to end of fNewText

	CTempAllocation tempAllocation(false);
	::ThrowIfOSErr_AC(PtrAndHand((Ptr) & ch[1], fNewText, ch.Length()));

	fNewEnd += ch.Length();						// Bump both end of "selection" 
	fTextPad += ch.Length();					// and padding value 

	// This SetPermHandleSize can't grow the handle,
	if (fPadding)
		SetPermHandleSize(fPadding, Max_AC(-(fTextPad + fStylePad), 0));// so it shouldn't fail. 
}

//----------------------------------------------------------------------------------------
// TTETypingCommand::BkSpcLeft: User has backspaced to the left of the original starting
// point. First, copy the character (which may be more than one byte long if we are using
// a non-Roman script) to a temporary buffer. The assumption is that no character will
// ever be longer than two bytes. Next, copy the character to the front of
// fOldText, and adjust fOldStart, fNewStart, and fNewEnd. Note that we do !check for
// MemSpaceIsLow, since we want to let the user delete characters.
//----------------------------------------------------------------------------------------

void TTETypingCommand::BkSpcLeft(Handle theText,
								 short curStart)
{
	unsigned char saveLength = 1;
	if (MACharacterByteType((*theText), curStart - 1, smCurrentScript) == smLastByte)
		++saveLength;
	curStart -= saveLength;

	CStr2_AC savedCh(static_cast<void*>(*theText + curStart), saveLength);

	CTextStyle_AC delStyle;
	short theHeight;
	short theAscent;
	TEGetStyle(curStart, &delStyle,			// Get the style of the deleted character 
			   &theHeight, &theAscent, fHTE);// (1 or 4 bytes, it's all only one style) 

	if (!EqualBlocks(&delStyle,		// If style doesn't match first in the list 
					 &(*fOldStyles)->scrpStyleTab[0].scrpFont, sizeof(delStyle)))
	{
		// then insert new style at head of list 

		long oldSize =						// Make room for the new style element 
					  GetHandleSize((Handle)fOldStyles);
		SetPermHandleSize((Handle)fOldStyles, oldSize + sizeof(ScrpSTElement));
		fStylePad += sizeof(ScrpSTElement);

		{
			//ScrpSTElement & theScrpSTElement = (*fOldStyles)->scrpStyleTab[0];
			ScrpSTElement * theScrpSTElement = &(*fOldStyles)->scrpStyleTab[0];

			//	BlockMove_AC(&(theScrpSTElement.scrpStartChar),// Move entire array up one element's size 
			//				(Ptr)&(theScrpSTElement.scrpStartChar) + sizeof(ScrpSTElement),
			//				oldSize - sizeof((*fOldStyles)->scrpNStyles));
			BlockMove_AC(theScrpSTElement, (Ptr)theScrpSTElement + sizeof(ScrpSTElement), oldSize - sizeof(ScrpSTElement));
		}

		++(*fOldStyles)->scrpNStyles;		// One more style 

		{
			ScrpSTElement & theScrpSTElement = (*fOldStyles)->scrpStyleTab[0];

			theScrpSTElement.scrpHeight = theHeight;// Fill in the blanks 
			theScrpSTElement.scrpAscent = theAscent;
			BlockMove_AC(&delStyle, &theScrpSTElement.scrpFont, sizeof(TextStyle));
		}
	}

	--(*fOldStyles)->scrpStyleTab[0].scrpStartChar;// Regardless, back off offset by one 

	if (fPadding)
		SetPermHandleSize(fPadding, GetHandleSize(fOldText) + savedCh.Length() + fStylePad);
	Munger(fOldText, 0, NULL, 0, &savedCh[1], savedCh.Length());
	::ThrowIfMemError_AC();
	fOldStart = curStart;						// Treat this as though original selection 
	fNewStart = curStart;						// had included this character 
	fNewEnd = curStart;
	fTextPad -= savedCh.Length();
}

//----------------------------------------------------------------------------------------
// TTETypingCommand::BkSpcRight: 
//----------------------------------------------------------------------------------------

void TTETypingCommand::BkSpcRight(Handle theText,
								  short curStart)
{
	short savedSize = 1;
	if (MACharacterByteType((*theText), curStart - savedSize, smCurrentScript) == smLastByte)
		++savedSize;
	if (fPadding)
		SetPermHandleSize(fPadding, Max_AC(-(fTextPad - savedSize + fStylePad), 0));
	fNewEnd -= savedSize;
	fTextPad -= savedSize;

	SetPermHandleSize(fNewText, fNewEnd - fNewStart);// Shouldn't fail as we're only shrinking it
}

//----------------------------------------------------------------------------------------
// TTETypingCommand::FwdDelete: Forward delete courtesy of: Larry Goldman. Used by
// permission.
//----------------------------------------------------------------------------------------

void TTETypingCommand::FwdDelete(Handle theText,
								 short curStart,
								 short curEnd)
{
	short theHeight;
	short theAscent;
	long oldSize;
	CTextStyle_AC delStyle;
	long oldTextSize;

	CStr2_AC savedCh;

	long textSize = GetHandleSize(theText);
	if ((curStart == curEnd) && (curStart < textSize))
	{
		unsigned char saveLength = 0;			//Get the complete character
		if ((curStart + saveLength <= textSize) && (MACharacterByteType((*theText), curStart + saveLength, smCurrentScript) == smLastByte))
			++saveLength;
		++saveLength;

		savedCh.CopyFrom(static_cast<void*>(*theText + curStart), saveLength);

		if ((curStart >= fNewStart) && (curStart < fNewEnd))// char is within fNewText 
		{
			// Remove the char from fNewText and update fNewEnd and fTextPad
			if (fPadding)
				SetPermHandleSize(fPadding, Max_AC(-(fTextPad - savedCh.Length() + fStylePad), 0));
			fNewEnd -= savedCh.Length();
			fTextPad -= savedCh.Length();

			// Shouldn't fail as we're only shrinking it 
			Munger(fNewText, curStart - fNewStart, NULL, savedCh.Length(), &savedCh[1], 0);
			::ThrowIfMemError_AC();
		}
		else									// add char to the end of fOldChars, don't
			// update fOldEnd, but update fPadding
			{
				oldTextSize = GetHandleSize(fOldText);
				TEGetStyle(curStart, &delStyle,// Get the style of the deleted character 
						   &theHeight, &theAscent, fHTE);// (1 or 4 bytes, it's all only one style) 

				if (!EqualBlocks(&delStyle,// If style doesn't match last in the list 
								 &(*fOldStyles)->scrpStyleTab[(*fOldStyles)->scrpNStyles - 1].scrpFont, sizeof(delStyle)))
				{
					// then insert new style at end of list 

					oldSize = GetHandleSize((Handle)fOldStyles);// Make room for the new style element 
					SetPermHandleSize((Handle)fOldStyles, oldSize + sizeof(ScrpSTElement));
					fStylePad += sizeof(ScrpSTElement);

					++(*fOldStyles)->scrpNStyles;// One more style 

					{
						ScrpSTElement & theScrpSTElement = (*fOldStyles)->scrpStyleTab[(*fOldStyles)->scrpNStyles - 1];

						theScrpSTElement.scrpStartChar = oldTextSize;
						theScrpSTElement.scrpHeight = theHeight;// Fill in the blanks 
						theScrpSTElement.scrpAscent = theAscent;
						BlockMove_AC(&delStyle, &theScrpSTElement.scrpFont, sizeof(TextStyle));
					}
				}

				if (fPadding)
					SetPermHandleSize(fPadding, oldTextSize + savedCh.Length() + fStylePad);
				Munger(fOldText, oldTextSize, NULL, 0, &savedCh[1], savedCh.Length());
				::ThrowIfMemError_AC();
				fTextPad = fTextPad - savedCh.Length();
			}
	}
}

//----------------------------------------------------------------------------------------
// TTETypingCommand::AddCharacter: 
//----------------------------------------------------------------------------------------

void TTETypingCommand::AddCharacter(const CStr2_AC& ch)
{
	Handle theText;
	short curSelStart;
	short curSelEnd;

	fTEView->Update();							// Makes sure that all of TE's actions are Visible
	fTEView->Focus();

	{
		TERec & theTERec = **fHTE;

		curSelStart = theTERec.selStart;
		curSelEnd = theTERec.selEnd;
		theText = theTERec.hText;
	}

	CWhileOutlinePreferred_AC setOP(fTEView->fPreferOutline);
	{
		CTempAllocation tempAllocation(false);

		// Update the fNewText handle and other information. Note that because of
		// backspace, this can be tricky.*/
		if (ch == chFwdDelete)					// User types forward delete, so keep in  synch
			FwdDelete(theText, curSelStart, curSelEnd);

		else if (ch != chBackspace)				// Not a backspace. Do the right thing 
			DoNormalChar(ch);

		else if ((curSelStart <= fOldStart) && (curSelStart > 0) && (curSelStart == curSelEnd))// User typed backspace so keep in synch 
			BkSpcLeft(theText, curSelStart);	// Handle backspace to left of start 

		else if (fNewEnd > fNewStart)			// Delete 1 character from end of fNewText 
			BkSpcRight(theText, curSelStart);	// Handle backspace to right of start 
	}

	if (ch != chFwdDelete)
	{
		// Let TextEdit have the character, as either 1) we're adding a byte, so we know
		// there is a reserve tank, so the worst this will do is eat into it a little, or
		// 2) we're deleting a character, which can only decrease memory usage.

		// NOTE: here is where we insert multibyte characters all at once
		TEKey(ch[1], fHTE);
		if (ch.Length() > 1)
			TEKey(ch[2], fHTE);
	}
	else if (curSelStart != curSelEnd)			// forward delete with chars selected
		TEDelete(fHTE);
	else if (curSelStart < GetHandleSize(theText))
	{
		// forward delete with insertion point
		TEKey(chRight, fHTE);
		TEKey(chBackspace, fHTE);
	}

	fTEView->SynchView(kRedraw);				// Now clean up the view. 

#if qDebugMsg
	if (TTEView::fgTEIntenseDebugging)
	{
		WrLblHandleContents("fOldText", fOldText);
		ProgramReport_AC("\n");
		WrLblHandleContents("fNewText", fNewText);
		ProgramReport_AC("\n");
		DumpTTECommand(this);
	}
#endif

}

//----------------------------------------------------------------------------------------
// TTETypingCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TTETypingCommand::DoIt()					// override 
{
//	AddCharacter(fFirstChar);	[1115111] moved to constructor
#if qDebug
	if (TTEView::fgTEIntenseDebugging)
		DumpTTECommand(this);
#endif

}

//----------------------------------------------------------------------------------------
// TTETypingCommand::RedoIt: 
//----------------------------------------------------------------------------------------

void TTETypingCommand::RedoIt()					// override 
{
	AssureTEView();		// MEB make sure fHTE is valid before referencing it!

	if ((fOldEnd - fOldStart) == GetHandleSize(fOldText))
	{	// No chars were vacuumed
		bool resetStyle = false;

		if (fOldEnd == fOldStart)
		{
			CTextStyle_AC currentStyle;
			short lineHeight;
			short fontAscent;
			TEGetStyle(fOldStart, &currentStyle, &lineHeight, &fontAscent, fHTE);
			resetStyle = !EqualBlocks(&currentStyle, &(*fOldStyles)->scrpStyleTab[0].scrpFont, sizeof(currentStyle));
		}

		if (resetStyle)							// The new text has a style of its own 
			fNewStyles = fOldStyles;			// Make InstallNewText insert styles, too 
		Inherited::RedoIt();
		if (resetStyle)
			fNewStyles = NULL;					// So fNewStyles doesn't get disposed 
	}
	else
	{
		fTEView->Focus();
		// select vacuumed chars, too
		fTEView->SetSelection(fOldStart, fOldStart + GetHandleSize(fOldText), !kRedraw);
		TEDelete(fHTE);							// Remove old text, including vacuumed chars
		if (fPadding)
			SetPermHandleSize(fPadding, Max_AC(-(fTextPad + fStylePad), 0));
		InstallNewText();
		fTEView->SynchView(kRedraw);
#if qDebug
		if (TTEView::fgTEIntenseDebugging)
			DumpTTECommand(this);
#endif

	}
}

//----------------------------------------------------------------------------------------
// TTETypingCommand::UndoIt: 
//----------------------------------------------------------------------------------------

void TTETypingCommand::UndoIt()					// override 
{
	CompleteTyping();
	Inherited::UndoIt();
}

//----------------------------------------------------------------------------------------
// TTETypingCommand::CompleteTyping: 
//----------------------------------------------------------------------------------------

void TTETypingCommand::CompleteTyping()
{
	fCompleted = true;

	StScrpRec & theStScrpRec = **fOldStyles;
	long offset = -theStScrpRec.scrpStyleTab[0].scrpStartChar;
	if (offset > 0)
		for (short i = 0; i <= theStScrpRec.scrpNStyles - 1; ++i)
			theStScrpRec.scrpStyleTab[i].scrpStartChar += offset;
#if qDebug
	if (TTEView::fgTEIntenseDebugging)
		DumpTTECommand(this);
#endif

}

#if qDrag

//========================================================================================
// CLASS TTEDragDropCommand
//========================================================================================
#undef Inherited
#define Inherited TTECommand

//----------------------------------------------------------------------------------------
// TTEDragDropCommand constructor 
//----------------------------------------------------------------------------------------

TTEDragDropCommand::TTEDragDropCommand(TTEView* itsTEView,
									   CommandNumber itsCommandNumber) :
	TTECommand(itsTEView, itsCommandNumber, false)
{
}

//----------------------------------------------------------------------------------------
// TTEDragDropCommand constructor
//----------------------------------------------------------------------------------------

TTEDragDropCommand::TTEDragDropCommand(TEditText* itsEditText,
									   CommandNumber itsCommandNumber) :
	TTECommand(itsEditText, itsCommandNumber, false)
{
}

//----------------------------------------------------------------------------------------
// TTEDragDropCommand destructor
//----------------------------------------------------------------------------------------

TTEDragDropCommand::~TTEDragDropCommand()
{
}

//----------------------------------------------------------------------------------------
// TTEDragDropCommand::DoMainFunction: 
//----------------------------------------------------------------------------------------

void TTEDragDropCommand::DoMainFunction()
{
	InstallNewText();
	fTEView->SynchView(kRedraw);
}

//----------------------------------------------------------------------------------------
// TTEDragDropCommand::SetNewText: 
//----------------------------------------------------------------------------------------

void TTEDragDropCommand::SetNewText(Handle newText,
									StScrpHandle newStyles)
{
	fNewText = newText;
	fNewEnd = fNewStart + static_cast<short>(GetHandleSize(newText));// correct for text size
	fNewStyles = newStyles;
}

//----------------------------------------------------------------------------------------
// TTEDragDropCommand::SetNewStart: 
//----------------------------------------------------------------------------------------

void TTEDragDropCommand::SetNewStart(short newStart)
{
	short textSize = fNewEnd - fNewStart;
	fNewStart = newStart;
	fNewEnd = fNewStart + textSize;
}

//----------------------------------------------------------------------------------------
// TTEDragDropCommand::InstallNewText: 
//----------------------------------------------------------------------------------------

void TTEDragDropCommand::InstallNewText()
{
	CViewRect textEditArea(fTEView->GetExtent());
	fTEView->SetSelection(fNewStart, fNewStart, kRedraw);
	fTEView->Draw(textEditArea);
	Inherited::InstallNewText();
	// fTEView->SetSelection(fNewStart, fNewEnd, kRedraw);
	fTEView->Draw(textEditArea);
}

//========================================================================================
// CLASS TTEDragMoveCommand
//========================================================================================
#undef Inherited
#define Inherited TTECommand

//----------------------------------------------------------------------------------------
// TTEDragMoveCommand constructor 
//----------------------------------------------------------------------------------------

TTEDragMoveCommand::TTEDragMoveCommand(TTEView* itsTEView,
									   CommandNumber itsCommandNumber,
									   short newStart) :
	TTECommand(itsTEView, itsCommandNumber, true)
{
	// initialize the command, and save the selection into fOldText

	// fOldStart and fOldEnd represent the text being dragged
	// fNewStart takes its impending relocation into account
	fNewStart = (fOldStart < newStart) ? (newStart - (fOldEnd - fOldStart)) : newStart;
	fNewEnd = fNewStart + (fOldEnd - fOldStart);

	// since new and old text are the same, the text handle can be shared
	fNewText = fOldText;
	fNewStyles = fOldStyles;
}

//----------------------------------------------------------------------------------------
// TTEDragMoveCommand constructor
//----------------------------------------------------------------------------------------

TTEDragMoveCommand::TTEDragMoveCommand(TEditText* itsEditText,
									   CommandNumber itsCommandNumber,
									   short newStart) :
	TTECommand(itsEditText, itsCommandNumber, true)
{
	// initialize the command, and save the selection into fOldText

	// fOldStart and fOldEnd represent the text being dragged
	// fNewStart takes its impending relocation into account
	fNewStart = (fOldStart < newStart) ? (newStart - (fOldEnd - fOldStart)) : newStart;
	fNewEnd = fNewStart + (fOldEnd - fOldStart);

	// since new and old text are the same, the text handle can be shared
	fNewText = fOldText;
	fNewStyles = fOldStyles;
}

//----------------------------------------------------------------------------------------
// TTEDragMoveCommand destructor
//----------------------------------------------------------------------------------------
TTEDragMoveCommand::~TTEDragMoveCommand()
{
	// fNewText and fOldText share a handle in this command. fNewText is NULLed to
	// prevent double disposal in the base destructor.
	fNewText = NULL;
	fNewStyles = NULL;
}

//----------------------------------------------------------------------------------------
// TTEDragMoveCommand::InstallNewText: 
//----------------------------------------------------------------------------------------

void TTEDragMoveCommand::InstallNewText()
{
	// set the text insertion point
	fTEView->SetSelection(fNewStart, fNewStart, !kRedraw);

	Inherited::InstallNewText();

	fTEView->SetSelection(fNewStart, fNewEnd, !kRedraw);
}

#endif // qDrag

//----------------------------------------------------------------------------------------
// End of UTECommands.cp



