//----------------------------------------------------------------------------------------
// UTEView.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UTEVIEW__
#include "UTEView.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __ImagingStreaming_AC__
#include "ImagingStreaming_AC.h"
#endif

// MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#if qDrag
#ifndef __UDRAGDROP__
#include "UDragDrop.h"
#endif

#endif

#ifndef __UDRAGDROPBEHAVIOR__
#include "UDragDropBehavior.h"
#endif

#ifndef __UERRORMGR__
#include "UErrorMgr.h"
#endif

#ifndef __UFILE__
#include "UFile.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UTECOMMANDS__
#include "UTECommands.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __ALIASES__
#include <Aliases.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif


#ifndef __STRING__
#include <string.h>
#endif

//========================================================================================
// static data members
//========================================================================================

TEClickLoopUPP TTEView::fgDefClickLoopProc;		//  Standard TextEdit click loop routine
TTEView* TTEView::fgCurrTEView;
TEClickLoopUPP TTEView::fgClickLoopProc = NewTEClickLoopProc(TTEView::ClickLoopForTTEView); // static so it's always done
#if qDebug
bool TTEView::fgTEIntenseDebugging;			// (has external) For the benefit of ClickLoopForTTEView only
#endif

//----------------------------------------------------------------------------------------
// InitUTEView:
//----------------------------------------------------------------------------------------

void InitUTEView()
{
	static bool UTEViewInitialized;

	if (!UTEViewInitialized)
	{
		if (qTemplateViews)
		{
			// So we can create TTEView objects from view resources
			MA_REGISTER_SIGNATURE(TTEView, kStdTEView);
		}

		UTEViewInitialized = true;
	}
}

//========================================================================================
// CLASS TTEView
//========================================================================================
#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS(TTEView);

//----------------------------------------------------------------------------------------
// TTEView constructor
//----------------------------------------------------------------------------------------

TTEView::TTEView(const CViewPoint& itsLocation,
					  const CViewPoint& itsSize,
					  SizeDeterminer itsHDeterminer,
					  SizeDeterminer itsVDeterminer,
					  const CViewRect& itsInset,
					  const CTextStyle_AC& itsTextStyle,
					  short itsJustification,
					  bool itsAutoWrap,
					  bool itsMakeTERecord) :
	TView(itsLocation, itsSize, itsHDeterminer, itsVDeterminer),

	fTextStyle(itsTextStyle),
	fTypingCommand(NULL),
	fHTE(NULL),
	fText(NULL),
	fSavedTEHandle(NULL),
	fLastHeight(0),
	fLastWidth(0),
	fMinAhead(kMinAhead),
	fLastPageBreak(0),
	fKeyCommandNumber(cTyping),
	// fControlChars([chLeft, chRight, chUp, chDown, chBackspace, chReturn]),
	fControlChars(macroAsSetElem(chLeft) | macroAsSetElem(chRight) | macroAsSetElem(chUp) | macroAsSetElem(chDown) | macroAsSetElem(chBackspace) | macroAsSetElem(chReturn)),
	fMaxChars(kUnlimited),
	fTextStyleRsrcID(kNoResource),
	fJustification(itsJustification),
	fLastLine(0),
	fSelAnchor(0),
	fUpDownH(0),
	fPreferOutline(kDontPreferOutline),
	fAcceptsChanges(true),						// Stuff to false if you don't want to
	// allow Cut, Paste, or Typing
	fAutoWrap(itsAutoWrap),
	fFreeText(true),
	fUpDown(false),
	fInset(itsInset)
#if qDrag
	,
	fHiliteRgn(NULL),
	fLastCaretTime(0L),
	fLastCaretOffset(0),
	fCaretIsShown(false)
#endif // qDrag
{
	fWantsToBeTarget = true;
	fCursorID = iBeamCursor;

	if (itsMakeTERecord == kMakeTERecord)
	{
		MakeTERecord();

		fSelAnchor = (*fHTE)->selStart;
		::TESetClickLoop(fgClickLoopProc, fHTE);
		fText = (*fHTE)->hText;

		SetIdleFreq(0);							// Idle ASAP

		::TEFeatureFlag(teFOutlineHilite, teBitSet, fHTE);
	}
//SRF only add adorner if wanted.	AddAdorner(gSelectionAdorner, kDrawView, false);// Wants DoHighlightSelection called
}

//----------------------------------------------------------------------------------------
// TTEView copy constructor
//----------------------------------------------------------------------------------------

TTEView::TTEView(const TTEView& inTEView) :
	TView(inTEView),

	fTextStyle(inTEView.fTextStyle),
	fTypingCommand(NULL),
	fHTE(NULL),
	fText(NULL),
	fSavedTEHandle(NULL),
	fLastHeight(inTEView.fLastHeight),
	fLastWidth(inTEView.fLastWidth),
	fMinAhead(inTEView.fMinAhead),
	fLastPageBreak(inTEView.fLastPageBreak),
	fKeyCommandNumber(inTEView.fKeyCommandNumber),
	fControlChars(inTEView.fControlChars),
	fMaxChars(inTEView.fMaxChars),
	fTextStyleRsrcID(inTEView.fTextStyleRsrcID),
	fJustification(inTEView.fJustification),
	fLastLine(inTEView.fLastLine),
	fSelAnchor(inTEView.fSelAnchor),
	fUpDownH(inTEView.fUpDownH),
	fPreferOutline(inTEView.fPreferOutline),
	fAcceptsChanges(inTEView.fAcceptsChanges),
	fAutoWrap(inTEView.fAutoWrap),
	fFreeText(inTEView.fFreeText),
	fUpDown(inTEView.fUpDown),
	fInset(inTEView.fInset)
#if qDrag
	,
	fHiliteRgn(inTEView.fHiliteRgn),
	fLastCaretTime(inTEView.fLastCaretTime),
	fLastCaretOffset(inTEView.fLastCaretOffset),
	fCaretIsShown(inTEView.fCaretIsShown)
#endif // qDrag
{
	if (inTEView.fHTE)
	{
		TEStyleHandle theStyles = NULL;
		STHandle theElements = NULL;
		LHHandle theLineHeights = NULL;
		NullStHandle theNullStyles = NULL;
		Handle theText = NULL;

		NullStHandle oldNullStyles = NULL;
		StScrpHandle oldNullScrap = NULL;
		LHHandle oldLineHeights = NULL;
		STHandle oldElements = NULL;
		TEStyleHandle oldStyles = NULL;
		STHandle inputElements = NULL;

		GrafPtr oldPort;

		CWhileOutlinePreferred_AC setOP(inTEView.fPreferOutline);

		GetPort(&oldPort);
		SetPortWindowPort(gWorkPort);
		SetPortTextStyle(fTextStyle);

		CTempHandleLock_AC tempHandleLock((Handle)(inTEView.fHTE));
		{
			CRect_AC destRect((*inTEView.fHTE)->destRect);
			CRect_AC viewRect((*inTEView.fHTE)->viewRect);

			fHTE = ::TEStyleNew(&destRect, &viewRect);
		}

		SetPort(oldPort);

		::ThrowIfNULL_AC(fHTE);

		// Copy some fields from the original TERecord into the cloned TERecord
		(*(fHTE))->just = (*inTEView.fHTE)->just;
		(*(fHTE))->crOnly = (*inTEView.fHTE)->crOnly;
#if qPowerPC
		// This doesn't work on 68K. The cloned TTEView should call TESetWordBreak
		// and pass it the address of the word break routine
		::TESetWordBreak((*inTEView.fHTE)->wordBreak, fHTE);
#endif
		::TESetClickLoop((*inTEView.fHTE)->clickLoop, fHTE);

		// These internal fields need to be cloned too
		(*(fHTE))->selRect = (*inTEView.fHTE)->selRect;
		(*(fHTE))->selPoint = (*inTEView.fHTE)->selPoint;

		// Clone the styles record since clone or stuffstyles doesn't do it for us
		TEStyleHandle inputStyles;

		ExtractStyles(inputStyles, inputElements);
		oldStyles = ::TEGetStyleHandle(fHTE);

		// Remember pointers to the memory blocks allocated by TEStyleNew
		oldElements = (*oldStyles)->styleTab;
		oldLineHeights = (*oldStyles)->lhTab;
		oldNullStyles = (*oldStyles)->nullStyle;

		oldNullScrap = oldNullStyles ? (*oldNullStyles)->nullScrap : NULL;

		theStyles = (TEStyleHandle)PermCopyHandle((Handle)inputStyles);
		CTempHandle_AC tempHandle((Handle)theStyles);

		theElements = (STHandle)PermCopyHandle((Handle)inputElements);
		CTempHandle_AC tempHandle2((Handle)theElements);

		theLineHeights = (LHHandle)PermCopyHandle((Handle)(*inputStyles)->lhTab);
		CTempHandle_AC tempHandle3((Handle)theLineHeights);

		theNullStyles = (*inputStyles)->nullStyle;
		CTempHandle_AC tempHandle4;
		CTempHandle_AC tempHandle5;
		if (theNullStyles)
		{
			PermHandToHand((Handle &)theNullStyles);
			
			tempHandle4 = (Handle)theNullStyles;
			if (oldNullScrap)
			{
				StScrpHandle theNullScrap = oldNullScrap;
				PermHandToHand((Handle &)theNullScrap);
				tempHandle5 = (Handle)theNullScrap;
				(*theNullStyles)->nullScrap = theNullScrap;
			}
		}

		(*theStyles)->nullStyle = theNullStyles;// Replace NULL style handle
		(*theStyles)->lhTab = theLineHeights;// Replace line heights table handle
		(*theStyles)->styleTab = theElements;// Replace STElements handle

		(*theStyles)->teRefCon = (*oldStyles)->teRefCon;// clone the refCon

		// Now that the cloned elements have been added to the
		// TE Record, delete the old versions (allocated by
		// TEStyleNew) and clear out the local variables that
		// point to the cloned items, so that they failure
		// handler will not try to free the same item twice.
		tempHandle2.DontRelease();
		tempHandle3.DontRelease();
		tempHandle4.DontRelease();
		tempHandle5.DontRelease();

		oldElements = (STHandle)(::DisposeIfHandle_AC((Handle)oldElements));
		oldLineHeights = (LHHandle)(::DisposeIfHandle_AC((Handle)oldLineHeights));
		oldNullStyles = (NullStHandle)(::DisposeIfHandle_AC((Handle)oldNullStyles));
		oldNullScrap = (StScrpHandle)(::DisposeIfHandle_AC((Handle)oldNullScrap));

		// NOTE!! TESetStyleHandle will dispose of oldStyles for us, but
		// not any of the handles contained INSIDE of oldStyles (argh!)
		// TextEdit is an Evil Dog.
		::TESetStyleHandle(theStyles, fHTE);
		tempHandle.DontRelease();

		fSavedTEHandle = (*fHTE)->hText;// Save existing handle
		theText = inTEView.fText;
		PermHandToHand(theText);
		(*fHTE)->hText = theText;// Install new handle
		fText = theText;	// Make a local copy, too
		theText = NULL;

		// Don't forget to update the length field of the new TE Record
		(*fHTE)->teLength = (*inTEView.fHTE)->teLength;
		
		// Now copy the TEFeatureFlag settings
		short flags[5] = { teFAutoScr, teFTextBuffering, teFOutlineHilite, teFInlineInput, teFUseTextServices };
		for (int idx = 0; idx <= 5; ++idx)
			::TEFeatureFlag(flags[idx], ::TEFeatureFlag(flags[idx], teBitTest, inTEView.fHTE), fHTE);

		// Copy the insertion point of the old TE record
		SetSelection((*inTEView.fHTE)->selStart, (*inTEView.fHTE)->selEnd, kDontRedraw);
	}

	RecalcText();
}

//----------------------------------------------------------------------------------------
// TTEView::Free:
//----------------------------------------------------------------------------------------

TTEView::~TTEView()
{
	if (fgCurrTEView == this)
		fgCurrTEView = NULL;

	if (fHTE)
	{
		if (fSavedTEHandle)
		{
			// Worry about fText separately. Put back the handle which TE allocated.
			(*fHTE)->hText = fSavedTEHandle;

			// This is here because it only makes sense if fSavedTEHandle is not nil.
			(*fHTE)->teLength = static_cast<short>(GetHandleSize((*fHTE)->hText));

			if (fFreeText)
				fText = ::DisposeIfHandle_AC(fText);
			else
				fText = NULL;					// Always drop my reference
		}
		::TEDispose(fHTE);
		fHTE = NULL;

		fSavedTEHandle = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TTEView::GetStandardSignature:
//----------------------------------------------------------------------------------------

IDType TTEView::GetStandardSignature() const			// override
{
	return kStdTEView;
}

//----------------------------------------------------------------------------------------
// TTEView::ReadFields:
//----------------------------------------------------------------------------------------

void TTEView::ReadFields(CStream_AC* aStream)		// override
{
	Inherited::ReadFields(aStream);

	bool ignoreStyleTypeSinceWeAlwaysUseStyles;

	*aStream 	>> ignoreStyleTypeSinceWeAlwaysUseStyles
				>> fAutoWrap
				>> fAcceptsChanges
				>> fFreeText
				>> fKeyCommandNumber
				>> fMaxChars
				>> fInset
				>> fJustification
				>> fTextStyleRsrcID
				>> fPreferOutline;

				if (fTextStyleRsrcID != kNoResource)
					fTextStyle = MAGetTextStyle(fTextStyleRsrcID);

#if 0
	bool ignoreStyleTypeSinceWeAlwaysUseStyles = aStream->ReadBoolean();

	fAutoWrap = aStream->ReadBoolean();
	fAcceptsChanges = aStream->ReadBoolean();
	fFreeText = aStream->ReadBoolean();
	fKeyCommandNumber = aStream->ReadLong();
	fMaxChars = aStream->ReadInteger();

	fInset = aStream->ReadLongRect();

	fJustification = aStream->ReadInteger();
	fTextStyleRsrcID = aStream->ReadInteger();
	if (fTextStyleRsrcID != kNoResource)
		fTextStyle = MAGetTextStyle(fTextStyleRsrcID);

	fPreferOutline = aStream->ReadBoolean();
#endif

	MakeTERecord();

	fSelAnchor = (*fHTE)->selStart;
	::TESetClickLoop(fgClickLoopProc, fHTE);
	fText = (*fHTE)->hText;

	::TEFeatureFlag(teFOutlineHilite, teBitSet, fHTE);
}

//----------------------------------------------------------------------------------------
// TTEView::WriteFields:
//----------------------------------------------------------------------------------------

void TTEView::WriteFields(CStream_AC* aStream) const		// override
{
	Inherited::WriteFields(aStream);

	*aStream 	<< true
				<< fAutoWrap
				<< fAcceptsChanges
				<< fFreeText
				<< fKeyCommandNumber
				<< fMaxChars
				<< fInset
				<< fJustification
				<< fTextStyleRsrcID
				<< fPreferOutline;

#if 0
	aStream->WriteBoolean(true); // ignoreStyleTypeSinceWeAlwaysUseStyles
	aStream->WriteBoolean(fAutoWrap);
	aStream->WriteBoolean(fAcceptsChanges);
	aStream->WriteBoolean(fFreeText);
	aStream->WriteLong(fKeyCommandNumber);
	aStream->WriteInteger(fMaxChars);
	aStream->WriteLongRect(fInset);
	aStream->WriteInteger(fJustification);
	aStream->WriteInteger(fTextStyleRsrcID);
	aStream->WriteBoolean(fPreferOutline);
#endif
}

//----------------------------------------------------------------------------------------
// TTEView::BecameWindowTarget:
//----------------------------------------------------------------------------------------

void TTEView::BecameWindowTarget()				// Override
{
	DoHighlightSelection(fActiveHL, hlOn);
	fActiveHL = hlOn;

	Inherited::BecameWindowTarget();
}

//----------------------------------------------------------------------------------------
// TTEView::BecameTarget:
//----------------------------------------------------------------------------------------

void TTEView::BecameTarget()					// Override
{
	SetActive(true);

	Inherited::BecameTarget();
}

//----------------------------------------------------------------------------------------
// TTEView::ClickLoop:
//----------------------------------------------------------------------------------------

bool TTEView::ClickLoop()
{
	if (StillDown())
	{
		TScroller * scroller = GetScroller(kAnySuperView);
		if (scroller && scroller->Focus())
		{
			CPoint_AC msePt;
			GetMouse(msePt);

			CViewPoint delta;
			scroller->AutoScroll(scroller->QDToView(msePt), delta);// find how much should scroll
			if (Focus())
			{
				for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
					delta[vhs] = (delta[vhs] < 0) ? MinMax_AC(-GetVisibleQDRect()[topLeft][vhs], delta[vhs], 0) : MinMax_AC(0, delta[vhs], fSize[vhs] - GetVisibleQDRect()[botRight][vhs]);
				// The intent of the above is not to do autoscrolling that would scroll
				// beyond the subview boundary in any direction

				if (delta != gZeroVPt)
				{
					scroller->ScrollBy(delta, kRedraw);
					Update();					// make sure the scrolling was visible
				}
			}
		}

		// Focus may have changed, which could change lots of things, thus requiring us,
		// tiresomely, to take some or all of the following restorative precautions b/ c
		// clickloop expects us to be clipped to the destrect.
		if (Focus())
		{
			CTempRegion_AC rRgn;
			RectRgn(rRgn, &CRect_AC((*fHTE)->destRect));
			ClipFurtherTo(rRgn, gZeroPt);
		}
	}

	return true;								// Still consider the mouse to be down
}

//----------------------------------------------------------------------------------------
// TTEView::AutoScrolling:
//----------------------------------------------------------------------------------------

void TTEView::AutoScrolling(bool doScrolling)
{
	if (fHTE)
		::TEAutoView(doScrolling, fHTE);
}

//----------------------------------------------------------------------------------------
// TTEView::BeInPort:
//----------------------------------------------------------------------------------------

void TTEView::BeInPort(GrafPtr itsPort)			// override
{
	if (fHTE)
	{
		if (itsPort)
			(*fHTE)->inPort = itsPort;
		else
		{
			(*fHTE)->inPort = (GrafPtr)GetWindowPort(gWorkPort);
			DoneTyping();
		}
	}

	Inherited::BeInPort(itsPort);
}

//----------------------------------------------------------------------------------------
// TTEView::BeInScroller:
//----------------------------------------------------------------------------------------

void TTEView::BeInScroller(TScroller* itsScroller)// override
{
	if (fHTE && itsScroller)
	{
		short vertScrollUnit;

		if ((*fHTE)->lineHeight > 0)			// This works for both old && new TextEdit
			vertScrollUnit = (*fHTE)->lineHeight;
		else
			vertScrollUnit = GetDefFontSize();	// Ask for system default size

		itsScroller->SetScrollParameters(CViewPoint(kStdScrollUnit, vertScrollUnit), false, true);
	}

	Inherited::BeInScroller(itsScroller);
}

//----------------------------------------------------------------------------------------
// TTEView::CalcMinFrame:
//----------------------------------------------------------------------------------------

CViewRect TTEView::CalcMinFrame()		// override
{
	// Note that we omit the margins here, so that if TView::ComputeFrame rounds up to a
	// page multiple, the margins will get tacked on after.
	CViewPoint minSize(fSize.h - fInset.left - fInset.right, CalcRealHeight());

//SRF	if (fSizeDeterminer[hSel] == sizeVariable && (!fStyleType && !fAutoWrap))
	if (fSizeDeterminer[hSel] == sizeVariable && !fAutoWrap)
		minSize.Set(fLastWidth, minSize.v);

	CViewRect minFrame(Inherited::CalcMinFrame());
	minFrame[botRight] = minFrame[topLeft] + minSize;

	return minFrame;
}

//----------------------------------------------------------------------------------------
// TTEView::CalcRealHeight:
//----------------------------------------------------------------------------------------

long TTEView::CalcRealHeight()
{
	long theHeight = 0;							// default return value.
	bool lastIsCR;
	short nLines;
	short lineHeight;

	CWhileOutlinePreferred_AC setOP(fPreferOutline);

	{
		TERec & theTERec = **fHTE;

		lastIsCR = (theTERec.teLength <= 0) || ((*(CharsHandle)(theTERec.hText))[theTERec.teLength - 1] == chReturn);
		nLines = theTERec.nLines;
		lineHeight = theTERec.lineHeight;
	}

	if (nLines > 0)
		theHeight = ::TEGetHeight(SHRT_MAX, 0, fHTE);

	if (lastIsCR)							// if lastIsCR, then add to result of TEGetHeight
	{
		short theMode = doAll;
		CTextStyle_AC theStyle;
		ContinuousStyle(SHRT_MAX, SHRT_MAX, theMode, theStyle);

		FontInfo theFontInfo;
		short theFontHeight;
		GetTextStyleFontInfo(theStyle, theFontInfo, theFontHeight);

		theHeight += theFontHeight;
	}

#if qDebugMsg
	if (gIntenseDebugging)
		ProgramReport_AC(false, "CalcRealHeight=%1d\n", theHeight);
#endif

	return theHeight;
}

//----------------------------------------------------------------------------------------
// TTEView::CalcRealWidth:
//----------------------------------------------------------------------------------------

long TTEView::CalcRealWidth()
{
	short aWidth = 0;

	if (Focus())
	{
		SetPortTextStyle(fTextStyle);

		CTempHandleLock_AC tempHandleLock((Handle)(*fHTE)->hText);
		short lineCount = (*fHTE)->nLines;
		short fromChar = (*fHTE)->lineStarts[0];
		short firstChar;

		for (short index = 1; index <= lineCount; ++index)
		{
			firstChar = (*fHTE)->lineStarts[index];	// offset of 1st char in line
			aWidth = Max_AC(aWidth, TextWidth(*((*fHTE)->hText), fromChar, firstChar-fromChar));
			fromChar = firstChar;
		}
	}

	return aWidth;
}

//----------------------------------------------------------------------------------------
// TTEView::ChangeWrap:
//----------------------------------------------------------------------------------------

void TTEView::ChangeWrap(bool newAutoWrap,
						 bool redraw)
{
	fAutoWrap = newAutoWrap;
	(*fHTE)->crOnly = newAutoWrap ? 0 : -1;

	if (redraw)
	{
		RecalcText();
		SynchView(kRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TTEView::ComputeFrame:
//----------------------------------------------------------------------------------------

void TTEView::ComputeFrame(CViewRect& newFrame)		// override
{
	Inherited::ComputeFrame(newFrame);

	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		switch (fSizeDeterminer[vhs])
		{
			case sizeFixed:
			case sizeRelSuperView:
				break;

			case sizeSuperView:
				if (fSuperView)
					break;
				// else drop through to default
			default:
				newFrame[botRight][vhs] += fInset[topLeft][vhs] + fInset[botRight][vhs];
				break;
		}

	// we must limit the frame to kMaxCoord, since Text Edit doesn't allow height to exceed 32K
	newFrame.bottom = newFrame.top + Min_AC(static_cast<long>(kMaxQDCoord), newFrame.bottom - newFrame.top);
}

//----------------------------------------------------------------------------------------
// TTEView::CalcSelLoc:
//----------------------------------------------------------------------------------------

void TTEView::CalcSelLoc(CViewRect& selectionRect)
{
	CharsHandle handleToText;
	short startOfSelection;
	short endOfSelection;
	short charCount;
	short lineHeight;
	short fontAscent;
	CTextStyle_AC theStyle;
	bool selectionIsTheLastReturn;
	short theMode;
	FontInfo theFontInfo;
	short theFontHeight;

	CWhileOutlinePreferred_AC setOP(fPreferOutline);

	{
		TERec & theTERec = **fHTE;

		charCount = theTERec.teLength;
		startOfSelection = theTERec.selStart;
		endOfSelection = theTERec.selEnd;
		handleToText = (CharsHandle)(theTERec.hText);

		if (!(theTERec.selEnd - theTERec.selStart) && fIdleFreq == kMaxIdleTime)
			SetIdleFreq(0);						// Idle ASAP
	}

	selectionIsTheLastReturn = (startOfSelection == charCount) && (charCount > 0) && ((*handleToText)[charCount - 1] == chReturn);

	::TEGetStyle(startOfSelection, &theStyle, &lineHeight, &fontAscent, fHTE);

	CPoint_AC thePoint(::TEGetPoint(startOfSelection, fHTE));
	thePoint.v -= lineHeight;
	selectionRect[topLeft] = QDToView(thePoint);
	selectionRect[botRight] = QDToView(::TEGetPoint(endOfSelection, fHTE));

	// Through System 6.0.4 TE TEGetPoint returns
	// bogus numbers when the character count is 0.
	if (charCount <= 0)
	{
		selectionRect.top = 0;
		selectionRect.bottom = lineHeight;
	}

	if (selectionIsTheLastReturn)
	{
		theMode = doAll;

		// Get the style so we know how tall to make the selection when its just the last return
		ContinuousStyle(SHRT_MAX, SHRT_MAX, theMode, theStyle);

		GetTextStyleFontInfo(theStyle, theFontInfo, theFontHeight);

		selectionRect.top = selectionRect.bottom;
		selectionRect.bottom += theFontHeight;
	}
	else
	{
		// Correct errors by CalcSelLoc. If there is no selection then the "selection"
		// consists of the bits enclosed by the insertion bar.
		if (!((*fHTE)->selEnd - (*fHTE)->selStart))
			selectionRect.left = selectionRect.right - 1;

	}
}

//----------------------------------------------------------------------------------------
// TTEView::ContinuousStyle:
//----------------------------------------------------------------------------------------

bool TTEView::ContinuousStyle(short firstChar,
								 short lastChar,
								 short& mode,
								 CTextStyle_AC& aStyle)
{
	short oldSelStart = (*fHTE)->selStart;
	short oldSelEnd = (*fHTE)->selEnd;

	SetSelect(firstChar, lastChar, fHTE);		// Use SetSelect so this is invisible
	bool result = ::TEContinuousStyle(&mode, &aStyle, fHTE);
	SetSelect(oldSelStart, oldSelEnd, fHTE);

	return result;
}

//----------------------------------------------------------------------------------------
// TTEView::OffsetToScript:
//----------------------------------------------------------------------------------------

short TTEView::OffsetToScript(short offset)
{
	CTextStyle_AC textStyle;
	short mode = doFont;
	ContinuousStyle(offset, offset, mode, textStyle);

	return FontToScript(textStyle.tsFont);
}

//----------------------------------------------------------------------------------------
// TTEView::ContainsClipType:
//----------------------------------------------------------------------------------------

bool TTEView::ContainsClipType(ResType aType)// override
{
	return aType == kTextType_AC;
}

//----------------------------------------------------------------------------------------
// TTEView::DoBreakFollowing: Put in resident segment since this gets called while typing
//----------------------------------------------------------------------------------------

ViewCoordinate TTEView::DoBreakFollowing(VHSelect vhs,
									  ViewCoordinate previousBreak,
									  bool& automatic)// override
{
	VHSelect orthoVhs;
	ViewCoordinate possibleLoc;
	TEStyleHandle theStyles;
	LHHandle lhTab;
	long height;
	short lineHeight;
	short lineNo;
	short countLines;

	orthoVhs = gOrthogonal[vhs];
	automatic = true;

	TPrintHandler* itsPrintHandler = GetPrintHandler();
	if (itsPrintHandler)
		possibleLoc = Min_AC(static_cast<long>(kMaxQDCoord), previousBreak + itsPrintHandler->GetViewPerPage()[orthoVhs]);

	// We want to get rid of the on-screen margin represented by fMargin when printing, so
	// adjust things so that the portion of the view occupied by the screen margin doesn't
	// get printed.
	if (!previousBreak)
		possibleLoc += fInset[topLeft][orthoVhs];//!!! ViewCoordinate->short

	if (vhs == hSel)
	{
		if (fLastPageBreak == previousBreak)
		{
			height = fLastPageBreak;
			lineNo = fLastLine;
		}
		else
		{
			height = fInset[topLeft][orthoVhs];//!!! ViewCoordinate->short
			lineNo = 0;
		}

		theStyles = ::TEGetStyleHandle(fHTE);
		lhTab = (*theStyles)->lhTab;

		countLines = (*fHTE)->nLines;
		if ((*fHTE)->teLength > 0)
		{
			// we want to see if the last line is empty (only a return)
			// Can't just look at the last byte in hText, because it could be
			// a multi-byte character.
			short offsetOfLastLine = (*fHTE)->lineStarts[countLines - 1];
			char firstByteInLastLine = (*(*fHTE)->hText)[offsetOfLastLine];
			if (firstByteInLastLine == chReturn)
				++countLines;
		}

		while (lineNo < countLines)
		{
			lineHeight = (*lhTab)[lineNo].lhHeight;
			if (height + lineHeight <= possibleLoc)
				height += lineHeight;
			else
				break;
			++lineNo;
		}
		if (lineNo >= countLines)
			possibleLoc = Max_AC(possibleLoc, height);
		else if (height > previousBreak)
		{
			possibleLoc = height;
			fLastPageBreak = possibleLoc;
			fLastLine = lineNo;
		}
		// else the current line height was bigger than this page--print what we can.
		// next break we'll recalculate from the start again.
		fLastPageBreak = possibleLoc;
		fLastLine = lineNo;
	}

	return (possibleLoc + fInset[topLeft][orthoVhs]) >= fSize[orthoVhs] ? fSize[orthoVhs] : possibleLoc;
}

//----------------------------------------------------------------------------------------
// TTEView::DoCalcViewPerPage:
//----------------------------------------------------------------------------------------

CViewPoint TTEView::DoCalcViewPerPage()// override
{
	CViewPoint returnVal(Inherited::DoCalcViewPerPage());	// Get max amount allowed given margins

#if qDebugMsg
	if (gDebugPrinting)
		ProgramReport_AC(false, "TTEView: incoming generic viewPerPage: %s\n", (const char*)(returnVal));
#endif

	// Adjust for integral # of lines per page (unless the view is too small)
//SRF	if ((fStyleType != kWithStyle) && fHTE && (*fHTE)->lineHeight > returnVal.v)
	if (fHTE && (*fHTE)->lineHeight > returnVal.v)
		returnVal.v = (*fHTE)->lineHeight * (returnVal.v / (*fHTE)->lineHeight);

#if qDebug
	if (gDebugPrinting)
		ProgramReport_AC(false, "TTEView: computed viewPerPage: %s\n", (const char*)(returnVal));
#endif

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TTEView::DoCommandKeyEvent - handle command-arrow key event
//----------------------------------------------------------------------------------------
void TTEView::DoCommandKeyEvent(TToolboxEvent* event)	// Override
{
	bool handledCharacter = false;
	CStr2_AC text(event->GetText());

	// Check for a valid control character
	if (macroIn(fControlChars, macroAsSetElem(text[1])))
	{
		// Check for an arrow key
		if ((text >= chLeft) && (text <= chDown))
		{
			DoKeyEvent(event);
			handledCharacter = true;
		}
	}

	if (!handledCharacter)
		TView::DoCommandKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TTEView::DoIdle:
//----------------------------------------------------------------------------------------

bool TTEView::DoIdle(IdlePhase phase)		// override
{
	if (phase == idleContinue)
		if (fHTE && !((*fHTE)->selEnd - (*fHTE)->selStart) && IsEnabled())
		{
			if (fAcceptsChanges && IsDrawable())
			{
				CWhileOutlinePreferred_AC setOP(fPreferOutline);
				::TEIdle(fHTE);
			}
			SetIdleFreq(Max_AC(GetCaretTime() / 2, 1));// Reset idle frequency in case
			// user changed it
		}
		else
			SetIdleFreq(kMaxIdleTime);			// No need to bother anyone.

	return false;								// Didn't free myself
}

//----------------------------------------------------------------------------------------
// TTEView::DoKeyEvent: The Tab character has no width in some fonts, and so can cause
// confusing screen feedback. It is filtered out by default. If you want to include it in
// your text union it into fControlChars.
//----------------------------------------------------------------------------------------

void TTEView::DoKeyEvent(TToolboxEvent* event)	// override
{
	TTETypingCommand*	aTypingCommand = NULL;
	bool				needNewCommand = false;
	bool				handledCharacter = false;
	short				goToPos = (*fHTE)->selStart;	// The new character position we are moving to.
	short				i = 0;
	short				nonAnchor;						// The character position at the other end of the selection from fAnchor.
	short				newStart;
	short				newEnd;
	bool				towardStart;

	CStr2_AC text(event->GetText());

	if (IsEnabled())							// if view is not enabled then we don't take ANY keystrokes
	{
		if (text >= chSpace || (text < chSpace && macroIn(fControlChars, macroAsSetElem(text[1]))))// Check that the character should be accepted
		{
			if (((text >= chLeft) && (text <= chDown)) && Focus())	// Check for pure movement keys
			{
				bool	selecting = event->IsShiftKeyPressed();		// Are we extending the selection?

				DoneTyping();					// Like mousedown, further typing == new command

				if ((text != chUp) && (text != chDown))
					fUpDown = false;

				if ((text == chLeft) || (text == chRight))
				{
					towardStart = (text == chLeft);		// We're moving toward the start.
					if (GetScriptVariable(static_cast<short>(GetScriptManagerVariable(smKeyScript)), smScriptRight))
						towardStart = !towardStart;		// But we might be in a script system in which right is left.
				}
				else
					towardStart = false;				// We're not moving towards the start if we're going up or down.

				// Figure out what our selection anchor and 'non' selection anchor are.
				{
					TERec& theTERec = **fHTE;

					if (theTERec.selStart == theTERec.selEnd)
						fSelAnchor = theTERec.selStart;	// Nothing in the selection.
					if (fSelAnchor < 0)					// How can the anchor ever be < zero?
						if ((text == chUp) || (towardStart))
							fSelAnchor = theTERec.selEnd;
						else
							fSelAnchor = theTERec.selStart;
					if (fSelAnchor == theTERec.selEnd)
						nonAnchor = theTERec.selStart;
					else
						nonAnchor = theTERec.selEnd;
				}

				short nonAnchorLine = OffsetToLine(nonAnchor) + 1;	// 1-based line number
				if (text == chUp && (nonAnchorLine <= 1))
				{
					// We're moving up and there is no place to go.
					// Same as command-Up - Go to the very top of the text buffer.
					handledCharacter = true;
					goToPos = 0;
					fUpDown = false;					// We can no longer go up.
				}
				else if (text == chDown && (nonAnchorLine >= (*fHTE)->nLines))
				{
					// We're moving down and there is no place to go.
					// Same as command-Down - go to the very bottom of the buffer.
					handledCharacter = true;
					goToPos = (*fHTE)->teLength;
					fUpDown = false;					// We can no longer go down.
				}

				if (!handledCharacter)
				{
					// We haven't figured out yet what to do with the movement key.  Keep analyzing.
					CTempHandleLock_AC tempHandleLock((Handle)fHTE);

					handledCharacter = true;

					// The following doesn't appear to be functional.  We never seem to get key events
					// with the command key down.
					if (event->IsCommandKeyPressed())
					{
						// Having the command key down means to make a BIG jump.
						if (text == chUp)
							goToPos = 0;								// Cmd-up: go to very top of buffer.
						else if (text == chDown)
							goToPos = (*fHTE)->teLength;				// Cmd-down: go to very bottom of buffer.
						else if (towardStart)
							goToPos = OffsetToLineStart(nonAnchor);		// Cmd-right: go to beginning of line.
						else
							goToPos = OffsetToLineEnd(nonAnchor);		// Cmd-left: go to end of line.
						fUpDown = false;
					}
					else if ((text == chUp) || (text == chDown))
					{
						// We're moving up or down.
						CViewPoint aVPoint;

						if (!fUpDown)
						{
							// If we're moving left or right remember our horizontal position
							// so that we can set it when we begin moving up or down.
							fUpDown = true;
							aVPoint = OffsetToPt(nonAnchor);
							fUpDownH = static_cast<short>(aVPoint.h);
						}

						// Calculate the line to which we are going
						i = nonAnchorLine-1;	// need 0-based line number for indexing into lineStarts array
						if (text == chUp)
							i--;
						else	// if (text == chDown)
							i++;

						// Calculate the new character position to which we are going
						short lineOffset = (*fHTE)->lineStarts[i];		// offset of 1st char of nonAnchorLine
						aVPoint = OffsetToPt(lineOffset);
						aVPoint.h = fUpDownH;
						aVPoint.v -= 1;		// The -1 is needed so we don't end up just below the next line up.
						goToPos = PtToOffset(aVPoint);
					}
					else if (event->IsOptionKeyPressed())
					{
						// We're moving left or right.
						// Option key down means to move a word at a time.
						
						// we're moving to the right but to not include the selection if we're moving to the left.
						// I think we should be including the selection if moving to the left the same way as right.
						bool		expanding;			

						if (selecting)
							if (towardStart)
								expanding = (nonAnchor <= fSelAnchor);
							else
								expanding = (nonAnchor >= fSelAnchor);
						else
							expanding = true;

						goToPos = nonAnchor;
						if (!expanding)
							if (towardStart)
							{
								newEnd = goToPos;
								while ((goToPos > fSelAnchor) && ((!WordBounds(goToPos, newStart, newEnd)) || (newEnd + 1 >= nonAnchor)))
									goToPos = newStart - 1;
								if (goToPos <= fSelAnchor)
								{
									expanding = true;
									goToPos = fSelAnchor;
								}
								else
									goToPos = newEnd + 1;
							}
							else				//!towardStart
								{
									newStart = goToPos;
									while ((goToPos < fSelAnchor) && ((!WordBounds(goToPos, newStart, newEnd)) || (newStart <= nonAnchor)))
										goToPos = newEnd + 1;
									if (goToPos >= fSelAnchor)
									{
										expanding = true;
										goToPos = fSelAnchor;
									}
								}
						if (expanding)
							if (towardStart)
							{
								i = goToPos;
								newStart = goToPos;
								while ((goToPos > 0) && ((!WordBounds(goToPos, newStart, newEnd)) || (newStart >= i)))
									--goToPos;
								goToPos = Min_AC(goToPos, newStart);
							}
							else				//!towardStart
								{
									newEnd = goToPos;
									i = (*fHTE)->teLength - 1;
									while ((goToPos < i) && !WordBounds(goToPos + 1, newStart, newEnd))
										++goToPos;
									goToPos = Max_AC(goToPos, newEnd) + 1;
								}
					}
					else if (selecting || ((*fHTE)->selStart == (*fHTE)->selEnd))
					{
						if (towardStart)
							goToPos = GetPrevCharacterOffset(nonAnchor);
						else
							goToPos = GetNextCharacterOffset(nonAnchor);
					}
					else		// selStart != selEnd && ((text == chLeft) || (text == chRight))
					{
						// If there is a selection then move to the beginning or end of selection.
						if (towardStart)
							goToPos = (*fHTE)->selStart;
						else
							goToPos = (*fHTE)->selEnd;
					}
				}

				if (selecting)
					SetSelection(Min_AC(goToPos, fSelAnchor), Max_AC(goToPos, fSelAnchor), kRedraw);
				else
					SetSelection(goToPos, goToPos, kRedraw);
				ScrollSelectionIntoView(kRedraw);
			}
			else if (fAcceptsChanges && Focus())
			{
				// Check max size for text, and that we're not running out of memory
				if ((text != chBackspace && text != chFwdDelete && (*fHTE)->selStart == (*fHTE)->selEnd)
					&& (((fMaxChars - GetHandleSize(fText)) < 1) || MemSpaceIsLow()))
						StdAlert(phTooManyChars);
				else
				{
					// keep the keyscript and font in synch
					if (event->GetKeyScript() != FontToScript(fTextStyle.tsFont) && Focus())
					{
						DoneTyping();
						fTextStyle = GetPortTextStyle();
						fTextStyle.tsFont = GetScriptVariable(event->GetKeyScript(), smScriptAppFond);
						SetPortTextStyle(fTextStyle);

						if ((*fHTE)->selStart == (*fHTE)->selEnd)
						{
							TEStyleHandle oldStyles = ::TEGetStyleHandle(fHTE);
							TEStyleRec & theTEStyleRec = **oldStyles;

							NullStHandle theNullStyles = theTEStyleRec.nullStyle;
							StScrpHandle theScrpHandle = (*theTEStyleRec.nullStyle)->nullScrap;

							if (event->GetKeyScript() != FontToScript((*theScrpHandle)->scrpStyleTab[0].scrpFont))
								SetOneStyle((*fHTE)->selStart, (*fHTE)->selEnd, doFont, fTextStyle, false);
						}
					}

					// This key will change the horizontal position of the insertion point
					fUpDown = false;

					// Pass the character to the typing command, creating a new one if necessary
					needNewCommand = fTypingCommand == NULL;
					if (!needNewCommand)
						needNewCommand = fTypingCommand->fCompleted;

					if (needNewCommand)
					{
						aTypingCommand = DoMakeTypingCommand(text);
						fTypingCommand = aTypingCommand;
						PostCommand(aTypingCommand);
					}
					else
					{
						short oldLength = (*fHTE)->teLength;
						fTypingCommand->AddCharacter(text);
						// Once you're typing (first character already processed) collecting
						// subsequent characters really shouldn't affect the menus unless
						// you're keeping a character count in them or something that depends
						// on the aggregate of the characters you've typed. The CTextStyle_AC for
						// the first character would certainly apply to subsequent characters.
						// So, this is one ideal place to say that the event _DOES NOT_ affect
						// the menus. If the menus are already invalid they will stay so, but
						// if they are valid then there is no need to invalidate them in the
						// character collection process. When the user terminates the addition
						// of characters with another event, the menus will be setup from that
						// event. If you really feel you must, then you can always override,
						// call Inherited and then set event->fAffectMenus back to true.
						// (Rhymes with rue)
						event->SetAffectsMenus(!(*fHTE)->teLength || !oldLength);
					}
				}
				handledCharacter = true;
			}
		}
		if (fIdleFreq == kMaxIdleTime)
			SetIdleFreq(0);						// Idle ASAP, since someone may somehow set an
		// insertion point and want it to flash. (the
		// idle time will be reset to match the caret
		// time in doidle.)
	}

	if (!handledCharacter)
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TTEView::SetSelection:
//----------------------------------------------------------------------------------------

void TTEView::SetSelection(short newSelStart,
						   short newSelEnd,
						   bool redraw)
{
	if (redraw && Focus())
	{
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		// MEB pure hack. Make sure clikStuff is set to the magic value so that the cursor is positioned correctly.
		if ((*fHTE)->clikStuff != -1)
			(*fHTE)->clikStuff = -1;

		::TESetSelect(Max_AC(newSelStart, static_cast<short>(0)), Min_AC(newSelEnd, (*fHTE)->teLength), fHTE);

#if qDrag
		if (GetDraggable())
			gDispatcher->InvalidateMouseRegions();
#endif

	}
	else
		SetSelect(newSelStart, newSelEnd, fHTE);

	if (newSelStart == newSelEnd)
		fSelAnchor = newSelStart;
}

//----------------------------------------------------------------------------------------
// TTEView::DoMakeEditCommand:
//----------------------------------------------------------------------------------------

TTECommand* TTEView::DoMakeEditCommand(CommandNumber aCommandNumber)
{
	TTECommand * aTECommand = NULL;

	switch (aCommandNumber)
	{
		case cCut:
		case cCopy:
			aTECommand = TH_new TTECutCopyCommand(this, aCommandNumber);
			break;

		case cPaste:
			aTECommand = TH_new TTEPasteCommand(this);
			break;

		case cClear:
			aTECommand = TH_new TTECommand(this, aCommandNumber, true);
			break;

	}

	return aTECommand;
}

//----------------------------------------------------------------------------------------
// TTEView::DoMakeStyleCommand:
//----------------------------------------------------------------------------------------

TTEStyleCommand* TTEView::DoMakeStyleCommand(const CTextStyle_AC& aStyle,
											 CommandNumber itsCommandNumber,
											 short itsMode)
{
	return TH_new TTEStyleCommand(this, aStyle, itsCommandNumber, itsMode);
}

//----------------------------------------------------------------------------------------
// TTEView::DoMakeTypingCommand:
//----------------------------------------------------------------------------------------

TTETypingCommand* TTEView::DoMakeTypingCommand(const CStr2_AC& ch)
{
	return TH_new TTETypingCommand(this, ch);
}

//----------------------------------------------------------------------------------------
// TTEView::DoMenuCommand:
//----------------------------------------------------------------------------------------

void TTEView::DoMenuCommand(CommandNumber aCommandNumber)// override
{
	switch (aCommandNumber)
	{
		case cCut:
		case cCopy:
		case cClear:
			fUpDown = false;
			PostCommand(DoMakeEditCommand(aCommandNumber));
			break;

		case cPaste:
			{
				ResType dataType = '%%%%';

				long nChars = gClipboardMgr->GetDataToPaste(NULL, dataType);
				if (dataType == kTextType_AC)
				{
					if (nChars < 0)
					{
#if qDebug
						::ProgramBreak_AC("Couldn't get data to paste");
#endif
					}
					else
					{
						if (nChars - ((*fHTE)->selEnd - (*fHTE)->selStart) > fMaxChars - GetHandleSize(fText))
						{
							OSErr anErr;
							anErr = MAInteractWithUser();
							if (anErr == noErr)
								StdAlert(phTooManyChars);
							else if (anErr == errAENoUserInteraction)
								::ThrowIfOSErr_AC(errTooManyChars);
							else
								::ThrowIfOSErr_AC(anErr);
						}
						else
						{
							fUpDown = false;
							PostCommand(DoMakeEditCommand(aCommandNumber));
						}
					}
				}
				else
					Inherited::DoMenuCommand(aCommandNumber);
				break;
			}

		case cSelectAll:
			if (Focus())
			{
				fUpDown = false;
				SetSelection(0, (*fHTE)->teLength, kRedraw);
				DoneTyping();
				ScrollSelectionIntoView(kRedraw);
			}
			break;

		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TTEView::DoMouseCommand:
//----------------------------------------------------------------------------------------

void TTEView::DoMouseCommand(CViewPoint& theMouse,
							 TToolboxEvent* event,
							 CPoint_AC)			// override
{
	if (Focus() && IsVisible())
	{
		CWhileOutlinePreferred_AC setOP(fPreferOutline);

		fgCurrTEView = this;						// So the global clickLoop routine can forward
		DoneTyping();							// Mousedown terminates the Typing command
		fUpDown = false;						// and resets the horizontal location for up
		// and down arrow cursor movement
		::TEClick(ViewToQD(theMouse), event->IsShiftKeyPressed(), fHTE);

		// force a re-focus b/c the focusing in ClickLoop clips down to the destrect.
		if (IsFocused())
			InvalidateFocus();

#if qDrag
		// force an update of the current sleep region and cursor
		gDispatcher->InvalidateCursorRgn();
#endif // qDrag

		fSelAnchor = (*fHTE)->selStart;			// Remember the new selection anchor

		if (fIdleFreq == kMaxIdleTime)
			SetIdleFreq(0);						// Idle ASAP
	}
}

//----------------------------------------------------------------------------------------
// TTEView::DoneTyping:
//----------------------------------------------------------------------------------------

void TTEView::DoneTyping()
{
	if (fTypingCommand)
	{
		fTypingCommand->CompleteTyping();
		fTypingCommand = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TTEView::DoSetPageOffset:
//----------------------------------------------------------------------------------------

void TTEView::DoSetPageOffset(const CViewPoint& coord)// override

{
	Inherited::DoSetPageOffset(coord);
	for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		if (!coord[vhs])
			TPrintHandler::gPageOffset[vhs] += fInset[topLeft][vhs];
}

//----------------------------------------------------------------------------------------
// TTEView::DoSetupMenus:
//----------------------------------------------------------------------------------------

void TTEView::DoSetupMenus()					// override
{
	bool manyChars;

	Inherited::DoSetupMenus();

	manyChars = (*fHTE)->selStart < (*fHTE)->selEnd;
	if (!MemSpaceIsLow())
	{
		if (fAcceptsChanges)					// One way or another, we can paste text
			gClipboardMgr->CanPaste(kTextType_AC);	// If styles exist, all the better

		Enable(cCopy, manyChars);
	}
	Enable(cSelectAll, (*fHTE)->teLength > 0);

	// We enable Cut even if space is low, since it's nice to be able to rescue some of
	// the stuff you have to delete even if space is low. Note that it is possible to get
	// into the "can't do any commands" situation as a result. You should be able to close
	// and save the big document, then save the rescued text elsewhere, however.
	Enable(cCut, manyChars && fAcceptsChanges);
	Enable(cClear, manyChars && fAcceptsChanges);
}

//----------------------------------------------------------------------------------------
// TTEView::Draw:
//----------------------------------------------------------------------------------------

void TTEView::Draw(const CViewRect& area)			// override
{
	short oldHilite;
	short oldCaretState;

	CWhileOutlinePreferred_AC setOP(fPreferOutline);

	bool wasActive = (*fHTE)->active != 0;

	// Hide selection if we're printing or drawing to scrap
	// or if we cannot be changed and our selection is an insertion point.
	bool hideSelection = gPrinting
						|| gDrawingPictScrap
						|| (!fAcceptsChanges
							&& (*fHTE)->selStart == (*fHTE)->selEnd);

	if (hideSelection)
	{
		// we don't TEDeactivate to draw the selection as an outline
		oldHilite = ::TEFeatureFlag(teFOutlineHilite, teBitClear, fHTE);

		// we have to remember what the caret state was, because
		// TEDeactivate is going to blast it
		oldCaretState = (*fHTE)->caretState;

		// prevent selection from being drawn.
		CTempRegion_AC tempRgn;

		::GetClip(tempRgn);
		::ClipRect(gZeroRect);
		::TEDeactivate(fHTE);
		::SetClip(tempRgn);
	}

	::TEUpdate(&ViewToQD(area), fHTE);		// normal screen update handled by
	// TextEdit directly

	if (hideSelection && wasActive)
	{
		// Reactivate the TE record
		CTempRegion_AC tempRgn;

		::GetClip(tempRgn);
		::ClipRect(gZeroRect);
		::TEActivate(fHTE);
		::SetClip(tempRgn);

		::TEFeatureFlag(teFOutlineHilite, oldHilite, fHTE);

		// Put the caret state back to what it was
		(*fHTE)->caretState = oldCaretState;
	}

	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TTEView::ExtractStyles:
//----------------------------------------------------------------------------------------

void TTEView::ExtractStyles(TEStyleHandle& theStyles,
							STHandle& theElements) const
{
	theStyles = ::TEGetStyleHandle(fHTE);
	theElements = (*theStyles)->styleTab;
}

//----------------------------------------------------------------------------------------
// TTEView::ExtractText:
//----------------------------------------------------------------------------------------

Handle TTEView::ExtractText() const
{
	return fText;
}

//----------------------------------------------------------------------------------------
// TTEView::GetPrintExtent:
//----------------------------------------------------------------------------------------

CViewRect TTEView::GetPrintExtent()// override
{
	CViewRect printExtent(Inherited::GetPrintExtent());

	printExtent.Inset(fInset);

	return printExtent;
}

//----------------------------------------------------------------------------------------
// TTEView::GetSelectionString:
//----------------------------------------------------------------------------------------

void TTEView::GetSelectionString(CStr255_AC& selection)
{
	selection.Empty();
	if (fText && fHTE)
	{
		short start = (*fHTE)->selStart;
		short length = Min_AC(static_cast<short>(kStr255Len), static_cast<short>((*fHTE)->selEnd - start));
		if (length > 0)
			selection.CopyFrom(static_cast<void*>(*fText + start), length);
	}
}

//----------------------------------------------------------------------------------------
// TTEView::GetNumberOfChars:
//----------------------------------------------------------------------------------------

short TTEView::GetNumberOfChars()
{
	short numOfChars = 0;

	if (fText)
	{
		short textSize = static_cast<short>(GetHandleSize(fText));

		for (short numOfBytes = 0; numOfBytes < textSize; ++numOfBytes)
			if (MACharacterByteType(*fText, numOfBytes, OffsetToScript(numOfBytes)) != smFirstByte)
				++numOfChars;
	}

	return numOfChars;
}

//----------------------------------------------------------------------------------------
// TTEView::GetCharacterOffset:
//----------------------------------------------------------------------------------------

short TTEView::GetCharacterOffset(short fromOffset)
{
	return MACharacterByteType(*fText, fromOffset, OffsetToScript(fromOffset)) != smLastByte ? fromOffset : --fromOffset;
}

//----------------------------------------------------------------------------------------
// TTEView::GetBytesInCharacter:
//----------------------------------------------------------------------------------------

short TTEView::GetBytesInCharacter(short inOffset)
{
	// return the correct number of bytes even if the offset points into the middle
	// of a two byte character
	return MACharacterByteType(*fText, inOffset, OffsetToScript(inOffset)) == smSingleByte ? 1 : 2;
}

//----------------------------------------------------------------------------------------
// TTEView::GetNextCharacterOffset:
//----------------------------------------------------------------------------------------

short TTEView::GetNextCharacterOffset(short fromOffset)
{
	short correctedCharacterOffset = GetCharacterOffset(fromOffset);

	return Min_AC(static_cast<short>(correctedCharacterOffset + GetBytesInCharacter(correctedCharacterOffset + 1)), (*fHTE)->teLength);
}

//----------------------------------------------------------------------------------------
// TTEView::GetPrevCharacterOffset:
//----------------------------------------------------------------------------------------

short TTEView::GetPrevCharacterOffset(short fromOffset)
{
	short correctedCharacterOffset = GetCharacterOffset(fromOffset);

	return Max_AC(static_cast<short>(correctedCharacterOffset - GetBytesInCharacter(correctedCharacterOffset - 1)), static_cast<short>(0));
}

//----------------------------------------------------------------------------------------
// TTEView::GivePasteData:
//----------------------------------------------------------------------------------------

long TTEView::GivePasteData(Handle aDataHandle,
							ResType dataType)	// override
{
	long aSize = 0;
	OSErr err;
	bool savedPerm = false;

	if (dataType == kTextType_AC)
	{
		aSize = GetHandleSize(fText);
		if (aDataHandle)
		{
			SetPermHandleSize(aDataHandle, aSize);// Don't forget. This can fail
			BlockMove_AC((*fText), (*aDataHandle), aSize);
		}
	}
	else if (dataType == kStyleType_AC)
	{
		if (!SpaceForStyles(0, SHRT_MAX))
			::ThrowException_AC(noErr, 0);			// We'll accept this error in worst case
		else
		{
			short oldStart = (*fHTE)->selStart;
			short oldEnd = (*fHTE)->selEnd;
			SetSelect(0, SHRT_MAX, fHTE);

			CTempHandle_AC aHandle((Handle)::TEGetStyleScrapHandle(fHTE));
			SetSelect(oldStart, oldEnd, fHTE);

			if ((Handle)aHandle)
			{
				aSize = GetHandleSize(aHandle);
				if (aDataHandle)
				{
					CTempAllocation tempAllocation(false);
					// Try to prevent fragmentation, in case can't move while we're copying it!
					CTempHandleLock_AC tempHandleLock(aHandle, eMoveHi);
					// Copy styles into user-supplied handle
					err = PtrToXHand((*(Handle)aHandle), aDataHandle, aSize);

					if (err != noErr)		// Maybe enough for one copy, but not two!
						::ThrowException_AC(phStylesTooBig, phStylesTooBig + messageAlert);
				}
			}
			// Hmm. There _was_ enough memory, but the heap is probably pretty fragmented
			else if (aDataHandle)
				::ThrowException_AC(phStylesTooBig, phStylesTooBig + messageAlert);
		}
	}
	else
		::ThrowException_AC(noTypeErr, 0);

	FailSpaceIsLow();

	return aSize;
}

//----------------------------------------------------------------------------------------
// TTEView::MakeTERecord:
//----------------------------------------------------------------------------------------

void TTEView::MakeTERecord()
{
	if(!fHTE)
	{
		GrafPtr oldPort;

		GetPort(&oldPort);
		SetPortWindowPort(gWorkPort);
		SetPortTextStyle(fTextStyle);

		CViewRect itsExtent(GetExtent());
		itsExtent.Inset(fInset);

		CRect_AC dest(ViewToQD(itsExtent));

		fHTE = ::TEStyleNew(dest, &dest); // Open a styled record if requested

		::SetPort(oldPort);					// Make sure we restore the port before we
											// might throw an exception.
		::ThrowIfNULL_AC(fHTE);				// Make sure we have a TEHandle

		TEStyleHandle styleHandle = ::TEGetStyleHandle(fHTE);
		::ThrowIfNULL_AC(styleHandle);			// Make sure we have a TEStyleHandle

		(*styleHandle)->teRefCon = reinterpret_cast<long>(this);	// set the refcon to this view

		fgDefClickLoopProc = (*fHTE)->clickLoop;	// Just in case we want to restore it
		// Note that the system call SetClickLoop
		// _cannot_ be used to set the clickProc of
		// the TERecord. This is because the default
		// clickproc does not follow the Pascal Parameter
		// passing conventions that this procedure expects.

		SetJustification(fJustification, kDontRedraw);// Set justification to requested value
		ChangeWrap(fAutoWrap, false);				// Install auto wrap (or CR only)
		FailNoReserve();							// Got to have some reserve tank

		BeInPort(GetGrafPort());					// Associate with real port
	}
}

//----------------------------------------------------------------------------------------
// TTEView::OffsetToLine:
//----------------------------------------------------------------------------------------

short TTEView::OffsetToLine(short offset)
{
	// Given a character offset figure out what line it is in.
	// NOTE: resulting line number is 0-based, used for indexing into the lineStarts array
	short returnVal = 0;

	TERec & theTERec = **fHTE;

	if (offset == theTERec.teLength)
	{
		// special case for when offset is at end of text
		returnVal = theTERec.nLines-1;
		short lastOffset = offset-1;	// check whether last character is a chReturn
		if ((*((*fHTE)->hText))[lastOffset] == chReturn &&
				(MACharacterByteType(*(theTERec.hText), lastOffset, smCurrentScript) == smSingleByte))
					++returnVal;
	}
	else if (theTERec.nLines > 1 && theTERec.lineStarts[0] < offset)
	{
		short i;
		for (i = 1; i <= theTERec.nLines; ++i)
		{
			if (theTERec.lineStarts[i] == offset)
			{
				returnVal = i;
				break;
			}
			else if (theTERec.lineStarts[i] > offset)
			{
				returnVal = i-1;
				break;
			}
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TTEView::OffsetToLineStart:
//----------------------------------------------------------------------------------------

short TTEView::OffsetToLineStart(short offset)
{
	short returnVal = 0;

	if ((*fHTE)->nLines > 1)
	{
		short i = OffsetToLine(offset);
		returnVal = (*fHTE)->lineStarts[i];
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TTEView::OffsetToLineEnd:
//----------------------------------------------------------------------------------------

short TTEView::OffsetToLineEnd(short offset)
{
	TERec & theTERec = **fHTE;
	short returnVal = theTERec.teLength;

	if (theTERec.nLines > 1)
	{
		short i;
		for (i = 0; i <= theTERec.nLines; ++i)
		{
			if (theTERec.lineStarts[i] > offset)
				break;
		}

		// Last entry in lineStarts array (lineStarts[nLines]) is the teLength
		if (i < theTERec.nLines)
			returnVal = (theTERec.lineStarts[i] - 1);
		else if (i == theTERec.nLines)
		{	// check whether last character is a chReturn (i.e. blank line)
			short lastOffset = theTERec.teLength-1;
			if ((*((*fHTE)->hText))[lastOffset] == chReturn &&
				(MACharacterByteType(*(theTERec.hText), lastOffset, smCurrentScript) == smSingleByte))
					--returnVal;
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TTEView::OffsetToPt:
//----------------------------------------------------------------------------------------

CViewPoint TTEView::OffsetToPt(short offset)
{
	// Given a character offset figure out the view coordinates of the character.
	CPoint_AC thePoint;

	// Through System 6.0.4 TEGetPoint returns bogus numbers when the character count is 0.
	if ((*fHTE)->teLength <= 0)
	{
		short height;
		CTextStyle_AC theStyle;
		short ascent;
		::TEGetStyle(offset, &theStyle, &height, &ascent, fHTE);

		thePoint = fInset[topLeft].ToPoint();
		thePoint.v += ascent;
	}
	else
	{
		thePoint = ::TEGetPoint(offset, fHTE);
	}

	return QDToView(thePoint);
}

//----------------------------------------------------------------------------------------
// TTEView::PtToOffset:
//----------------------------------------------------------------------------------------

short TTEView::PtToOffset(const CViewPoint& aPoint)
{
	// Given a point in the view figure out what the character
	// offset is of the character containing the point.
	CPoint_AC aQDPoint(ViewToQD(aPoint));
	short offset = ::TEGetOffset(aQDPoint, fHTE);
	return offset;
}

//----------------------------------------------------------------------------------------
// TTEView::RecalcText:
//----------------------------------------------------------------------------------------

void TTEView::RecalcText()
{
	fUpDown = false;
	::TECalText(fHTE);
}

//----------------------------------------------------------------------------------------
// TTEView::ResignedWindowTarget:
//----------------------------------------------------------------------------------------

void TTEView::ResignedWindowTarget()			// override
{
	DoHighlightSelection(fActiveHL, hlDim);
	fActiveHL = hlDim;

	Inherited::ResignedWindowTarget();
}

//----------------------------------------------------------------------------------------
// TTEView::ResignedTarget:
//----------------------------------------------------------------------------------------

void TTEView::ResignedTarget()					// override
{
	SetActive(false);

	Inherited::ResignedTarget();
}

//----------------------------------------------------------------------------------------
// TTEView::SetFrame:
//----------------------------------------------------------------------------------------

void TTEView::SetFrame(const CViewRect& newFrame,
					   bool invalidate)		// override
{
	CViewPoint oldSize(fSize);

	Inherited::SetFrame(newFrame, invalidate);

	if (fHTE && fSize != oldSize)
	{
		CRect_AC itsQDExtent(GetQDExtent());

		itsQDExtent.Inset(fInset.ToRect());

		bool needCalText = (fSize.h != oldSize.h) || (itsQDExtent.right != (*fHTE)->destRect.right);

		StuffTERects(itsQDExtent);
		if (needCalText)
		{
			RecalcText();
			SynchView(kDontRedraw);
			short actualJust = GetActualJustification(fJustification);
			if (invalidate && ((fAutoWrap && (fSize != oldSize)) || ((actualJust == teFlushRight) || (actualJust == teCenter))))
				ForceRedraw();
		}
	}
}

//----------------------------------------------------------------------------------------
// TTEView::UpdateCoordinates:
//----------------------------------------------------------------------------------------

void TTEView::UpdateCoordinates()				// override
{
	CPoint_AC oldQDOrigin(fQDOrigin);
	CViewPoint oldViewToQDOffset(fViewToQDOffset);

	Inherited::UpdateCoordinates();

	if (fQDOrigin != oldQDOrigin || fViewToQDOffset != oldViewToQDOffset)
		if (fHTE)
		{
			CRect_AC itsQDExtent(GetQDExtent());

			itsQDExtent.Inset(fInset.ToRect());

			StuffTERects(itsQDExtent);
		}
}

//----------------------------------------------------------------------------------------
// TTEView::ScrollSelectionIntoView:
//----------------------------------------------------------------------------------------

void TTEView::ScrollSelectionIntoView(bool redraw)// Override
{
	if (GetScroller(kAnySuperView) && Focus())// Can't scroll selection if we don't have
	{
		//  a scroller!
		if (fIdleFreq == kMaxIdleTime)
			SetIdleFreq(0);						// Idle ASAP

		CViewRect selectionRect;
		CalcSelLoc(selectionRect);

#if qDebugMsg
		if (gIntenseDebugging)
		{
			ProgramReport_AC(false, "Visible CRect_AC was: %s", (const char*)(GetVisibleRect()));
			ProgramReport_AC(false, "; Sel CRect_AC was: %s\n", (const char*)(selectionRect));
		}
#endif

		if (!GetVisibleRect().Contains(selectionRect))
		{
			// Scroll the selection into view. accounting for the user's preference for
			// how much to jump at a time with fMinAhead. */
			CViewPoint minToSee(Min_AC(fMinAhead, fSize.h - selectionRect.left), selectionRect.GetLength(vSel));
#if qDebug
			if (gIntenseDebugging)
			{
				ProgramReport_AC(false, "RevealRect: r = %s", (const char*)(selectionRect));
				ProgramReport_AC(false, ", minToSee = %s\n", (const char*)(minToSee));
			}
#endif

			RevealRect(selectionRect, minToSee, redraw);
			Focus();							// Refocus in newly-scrolled position.
		}
	}
	else if (!fAutoWrap && fHTE)
		::TESelView(fHTE);
}

//----------------------------------------------------------------------------------------
// TTEView::SetJustification:
//----------------------------------------------------------------------------------------

void TTEView::SetJustification(short newJust,
							   bool redraw)
{
	::TESetAlignment(newJust, fHTE);
	fJustification = newJust;

	CRect_AC itsQDExtent(GetQDExtent());

	itsQDExtent.Inset(fInset.ToRect());

	StuffTERects(itsQDExtent);

	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TTEView::SetOneStyle:
//----------------------------------------------------------------------------------------

void TTEView::SetOneStyle(short theStart,
						  short theEnd,
						  short theMode,
						  const CTextStyle_AC& theStyle,
						  bool redraw)
{
	CWhileOutlinePreferred_AC setOP(fPreferOutline);

	InvalidateFocus();
	Focus();

	short saveStart = (*fHTE)->selStart;
	short saveEnd = (*fHTE)->selEnd;
	SetSelect(theStart, theEnd, fHTE);
	::TESetStyle(theMode, &theStyle, redraw, fHTE);
	SetSelect(saveStart, saveEnd, fHTE);

	RecalcText();
	SynchView(redraw && true);
//SRF	if (redraw && false)
//SRF		ForceRedraw();

}

//----------------------------------------------------------------------------------------
// TTEView::SetText:
//----------------------------------------------------------------------------------------

void TTEView::SetText(const CStr255_AC& theText)
{
	if (fHTE)									// If we're replacing text, styles are kaput
	{
		CStr255_AC localText(theText);
		OSErr err;

		bool allocationState = TemporaryAllocation(false);
		Handle theTextHandle;
		err = PtrToHand((Ptr) & localText[1], &theTextHandle, theText.Length());
		TemporaryAllocation(allocationState);

		::ThrowIfOSErr_AC(err);

		StuffText(theTextHandle);

		// Set the insertion point to the end of the text.
		// (This is consistant with the TextEdit SetText routine)
		short textLength = static_cast<short>(GetHandleSize(theTextHandle));
		SetSelection(textLength, textLength, kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TTEView::ShowReverted:
//----------------------------------------------------------------------------------------

void TTEView::ShowReverted()					// override
{
	RecalcText();
	fLastHeight = 0;
	fLastWidth = 0;
	Inherited::ShowReverted();
}

//----------------------------------------------------------------------------------------
// TTEView::SpaceForStyles:
//----------------------------------------------------------------------------------------

bool TTEView::SpaceForStyles(long rangeStart,
								long rangeEnd)
{
	bool returnVal = false;

	try
	{
		Handle h = NewPermHandle(::TENumStyles(rangeStart, rangeEnd, fHTE) * sizeof(ScrpSTElement) + 2);

		h = ::DisposeIfHandle_AC(h);					// Release memory back to the system
		returnVal = true;
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		StdAlert(phStylesTooBig);
		// Don't resignal in this case
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TTEView::StuffStyles:
//----------------------------------------------------------------------------------------

void TTEView::StuffStyles(TEStyleHandle theStyles,
						  STHandle theElements)
{
	if (fHTE)
	{
		LHHandle oldLineHeights;
		NullStHandle theNullStyles;
		StScrpHandle theScrpHandle;
		long savedRefCon;

		{
			TEStyleHandle oldStyles = TEGetStyleHandle(fHTE);
			TEStyleRec & theTEStyleRec = **oldStyles;

			STHandle oldElements = theTEStyleRec.styleTab;
			oldLineHeights = theTEStyleRec.lhTab;
			theNullStyles = theTEStyleRec.nullStyle;
			theScrpHandle = (*theTEStyleRec.nullStyle)->nullScrap;
			savedRefCon = theTEStyleRec.teRefCon;
			oldElements = (STHandle)::DisposeIfHandle_AC((Handle)oldElements);
		}

		{
			TEStyleRec & theTEStyleRec = **theStyles;

			theTEStyleRec.styleTab = theElements;// Replace STElements handle
			theTEStyleRec.lhTab = oldLineHeights;// Replace line heights table handle
			theTEStyleRec.nullStyle = theNullStyles;// Replace NULL style handle
			(*theTEStyleRec.nullStyle)->nullScrap = theScrpHandle;
			theTEStyleRec.teRefCon = savedRefCon;// restore the refcon reference
		}

		// NOTE!! TESetStyleHandle will dispose of oldStyles for us!
		::TESetStyleHandle(theStyles, fHTE);

		RecalcText();
	}
}

//----------------------------------------------------------------------------------------
// TTEView::StuffText:
//----------------------------------------------------------------------------------------

void TTEView::StuffText(Handle theText)
{
	if (fHTE)									// If we're replacing text, styles are kaput
	{
		if (GetHandleSize(theText) > fMaxChars)
		{
#if qDebug
			::ProgramBreak_AC("Text size exceeds maximum for this view");
#endif

			::ThrowException_AC(minErr, 0);
		}

		short textLength = static_cast<short>(GetHandleSize(theText));// Check size of new text

		// Remember where the selection range was & try to keep it
		// in the same state when the new text is added.  Usually, the
		// selection range should be in one of three states:  at the
		// beginning of the text, at the end of the text, or covering
		// all of the text (everything selected).  This isn't enforced,
		// though.

		short oldStart = (*fHTE)->selStart;
		short oldEnd = (*fHTE)->selEnd;
		if ((oldStart >= textLength) || (oldStart >= (*fHTE)->teLength))
			oldStart = textLength;
		if ((oldEnd >= textLength) || (oldEnd >= (*fHTE)->teLength))
			oldEnd = textLength;

		// Set the selection range to cover the entire text,
		// then call TESetStyle to fix up the style table.
		// It is _very important_ to reduce the number of style runs
		// down to one _before_ changing (*fHTE)->hText because
		// TESetStyle is not smart enough to fix up the style
		// run table if its length differs from (*fHTE)->teLength.

		SetSelect(0, SHRT_MAX, fHTE);
		::TESetStyle(doAll, &fTextStyle, false, fHTE);

		if ((fSavedTEHandle != theText) && ((*fHTE)->hText != theText))
		{
			fSavedTEHandle = ::DisposeIfHandle_AC(fSavedTEHandle);// we have no choice but to dispose it
			fSavedTEHandle = (*fHTE)->hText;	// Save existing handle
			(*fHTE)->hText = theText;			// Install new handle
		}

		fText = theText;						// Make a local copy, too
		(*fHTE)->teLength = textLength;	// Tell TE how long we are

		// At this point we are back to one plain-text
		// style, but TECalText is not smart enough to change
		// the total length of the style run table, so we
		// must do it manually here.

		TEStyleHandle styles = ::TEGetStyleHandle(fHTE);
		TEStyleRec & aTEStyleRec = **styles;

		// This next line sets the "dummy record" at the end of the table. See IM V-261.
		aTEStyleRec.runs[1].startChar = (*fHTE)->teLength + 1;

		RecalcText();							// Recalculate the line breaks and line-height tables
		SetSelect(oldStart, oldEnd, fHTE);		// Restore the old selection
	}
}

//----------------------------------------------------------------------------------------
// TTEView::StyleInsert:
//----------------------------------------------------------------------------------------

void TTEView::StyleInsert(const void *text, long length, StScrpHandle hST)
{
		if (hST)
		{
			TemplateTempHandle<StScrpHandle> singleStyle((StScrpHandle)::NewPermHandle(sizeof(short) + sizeof(ScrpSTElement)));
			(*singleStyle)->scrpNStyles = 1;

			short numRuns = (*hST)->scrpNStyles;
			Ptr singleText = static_cast<Ptr>(const_cast<void*>(text));
			for(long i = 0; i < numRuns; ++i)
			{
				(*singleStyle)->scrpStyleTab[0] = (*hST)->scrpStyleTab[i];
				// Here's the real key, setting the KeyScript to match the font in the style run
				SetKeyScript(FontToScript((*singleStyle)->scrpStyleTab[0].scrpFont));

				long runLength;
				if (i + 1 < numRuns)
					runLength = (*hST)->scrpStyleTab[i + 1].scrpStartChar - (*singleStyle)->scrpStyleTab[0].scrpStartChar;
				else
					runLength = length - (*singleStyle)->scrpStyleTab[0].scrpStartChar;

				(*singleStyle)->scrpStyleTab[0].scrpStartChar = 0;
				::TEStyleInsert(singleText, runLength, singleStyle, fHTE);
				singleText += runLength;
			}
		}
		else
			::TEStyleInsert(text, length, hST, fHTE);
}

//----------------------------------------------------------------------------------------
// TTEView::StuffTERects:
//----------------------------------------------------------------------------------------

void TTEView::StuffTERects(const CRect_AC& newTERect)
{
	CWhileOutlinePreferred_AC setOP(fPreferOutline);

	FontInfo aFontInfo;
	short theFontHeight;
	GetTextStyleFontInfo(fTextStyle, aFontInfo, theFontHeight);

	CRect_AC localRect(newTERect);
	short actualJust = GetActualJustification(fJustification);
//SRF	if ((fSizeDeterminer[hSel] == sizeVariable) && (actualJust != teJustCenter) && !fStyleType && !fAutoWrap)
	if ((fSizeDeterminer[hSel] == sizeVariable) && (actualJust != teJustCenter) && !fAutoWrap)
	{
		if ((actualJust == teJustLeft) || (actualJust == teForceLeft))
			localRect.right = localRect.right + aFontInfo.widMax;
		else
			localRect.left = localRect.left - aFontInfo.widMax;
	}
	else
		localRect.right = Max_AC(localRect.right, static_cast<short>(localRect.left + aFontInfo.widMax));

	(*fHTE)->destRect = localRect;
	(*fHTE)->viewRect = localRect;
}

//----------------------------------------------------------------------------------------
// TTEView::SynchView:
//----------------------------------------------------------------------------------------

void TTEView::SynchView(bool redraw)
{
	const short kInsertionBarWidth = 1;			// insertion bar is one pixel wide

	long theHeight = CalcRealHeight();

	long theWidth;
//SRF	bool doRealWidth = (fSizeDeterminer[hSel] == sizeVariable) && !fStyleType && !fAutoWrap;
	bool doRealWidth = (fSizeDeterminer[hSel] == sizeVariable) && !fAutoWrap;
	if (doRealWidth)
		theWidth = CalcRealWidth() + kInsertionBarWidth * 2;

	if ((fLastHeight != theHeight) || (doRealWidth && (fLastWidth != theWidth)))
	{
		if (doRealWidth)
			fLastWidth = theWidth;				// Width is expensive to calculate. Cache for CalcMinFrame

		AdjustFrame();							// may need to grow view
		fLastHeight = theHeight;				// Remember new height value
	}

	// First, make sure selection is visible (this conveniently focuses). Then, repair any
	// extra feedback which TextEdit may have mashed.
	if (redraw && Focus())
	{
		ScrollSelectionIntoView(redraw);
		DoHighlightSelection(hlOff, GetCurrentHL());
		if (GetPrintHandler())
			DoDrawPrintFeedback(GetDrawableRect());
	}
}

//----------------------------------------------------------------------------------------
// TTEView::WriteToDeskScrap:
//----------------------------------------------------------------------------------------

void TTEView::WriteToDeskScrap()				// override
{
	::ThrowIfOSErr_AC(gClipboardMgr->PutDeskScrapData(kTextType_AC, fText));

	if (SpaceForStyles(0, SHRT_MAX))
	{
		short saveStart = (*fHTE)->selStart;
		short saveEnd = (*fHTE)->selEnd;
		SetSelect(0, SHRT_MAX, fHTE);
		Handle aHandle = (Handle)(::TEGetStyleScrapHandle(fHTE));
		SetSelect(saveStart, saveEnd, fHTE);
		::ThrowIfNULL_AC(aHandle);
		::ThrowIfOSErr_AC(gClipboardMgr->PutDeskScrapData(kStyleType_AC, aHandle));
	}
}

//----------------------------------------------------------------------------------------
// TTEView::SetActive:
//----------------------------------------------------------------------------------------

void TTEView::SetActive(bool state)
{
	CWhileOutlinePreferred_AC setOP(fPreferOutline);

	if (fHTE && Focus())						// Try to focus because TEActivate/TEDeactivate may draw
		if (state)
		{
			bool selectionWasHidden = (!fAcceptsChanges
									 && (*fHTE)->selStart == (*fHTE)->selEnd);
			short oldHilite;
			if (selectionWasHidden)
				oldHilite = TEFeatureFlag(teFOutlineHilite, teBitClear, fHTE);
			if (fIdleFreq == kMaxIdleTime)
				SetIdleFreq(0);					// Idle ASAP
			SetKeyScript(FontToScript(fTextStyle.tsFont));
			::TEActivate(fHTE);
			fgCurrTEView = this;					// So the global clickLoop routine can forward
			if (selectionWasHidden)
				::TEFeatureFlag(teFOutlineHilite, oldHilite, fHTE);
		}
		else
		{
			bool hideSelection = (!fAcceptsChanges
									 && (*fHTE)->selStart == (*fHTE)->selEnd);
			short oldHilite;
			if (hideSelection)
				oldHilite = TEFeatureFlag(teFOutlineHilite, teBitClear, fHTE);
			TEDeactivate(fHTE);
			DoneTyping();
			if (hideSelection)
				::TEFeatureFlag(teFOutlineHilite, oldHilite, fHTE);
		}

}

//----------------------------------------------------------------------------------------
// TTEView::SetEnable:
//----------------------------------------------------------------------------------------

void TTEView::SetEnable(bool state)
{
	if (state && (fIdleFreq == kMaxIdleTime))
		SetIdleFreq(0);							// Get correct idle set ASAP

	Inherited::SetEnable(state);
}

//----------------------------------------------------------------------------------------
// TTEView::GetEditText:
//----------------------------------------------------------------------------------------

TEditText* TTEView::GetEditText()
{
	// TDialogTEView overrides this to return fEditText.
	return NULL;
}

//----------------------------------------------------------------------------------------
// TTEView::ForgetIfTypingCommand
//----------------------------------------------------------------------------------------
void TTEView::ForgetIfTypingCommand(TTETypingCommand* typingCommand)
{
	if (fTypingCommand == typingCommand)
		fTypingCommand = NULL;
}

//----------------------------------------------------------------------------------------
// TTEView::WordBounds:
//----------------------------------------------------------------------------------------

bool TTEView::WordBounds(short charPos,
						 short& wordStart,
						 short& wordEnd)
{
	bool returnVal = false;

	wordStart = charPos;
	wordEnd = charPos;

	if (!((charPos < 0) || (charPos > (*fHTE)->teLength - 1)))
	{
		// Start on a character boundary
		if (MACharacterByteType((*fText), charPos, OffsetToScript(charPos)) == smFirstByte)
			++charPos;

		OffsetTable offs;
		FindWord((*fText), (*fHTE)->teLength, charPos, true, NULL, offs);
		wordStart = Min_AC(offs[0].offFirst, static_cast<short>(offs[0].offSecond - 1));
		wordEnd = Max_AC(offs[0].offFirst, static_cast<short>(offs[0].offSecond - 1));

		returnVal = ((wordStart < wordEnd) || (MACharacterType((*fText), wordStart, OffsetToScript(wordStart)) % 8 != smCharPunct));
	}

	return returnVal;
}

#if qDrag

//----------------------------------------------------------------------------------------
// TTEView::DoMakeDragCursorRegion:
//----------------------------------------------------------------------------------------

RgnHandle TTEView::DoMakeDragCursorRegion()
{
	RgnHandle dragCursorRegion = MakeNewRgn();

	if (HasDragManager_AC())
	{
		CWhileOutlinePreferred_AC setOP(fPreferOutline);
		::TEGetHiliteRgn(dragCursorRegion, fHTE);
	}

	return dragCursorRegion;
}

//----------------------------------------------------------------------------------------
// TTEView::WillDrag:
//----------------------------------------------------------------------------------------

bool TTEView::WillDrag(const CViewPoint& localMouse, const RgnHandle dragCursorRegion)
{
	return !IsShiftKeyDown() && Inherited::WillDrag(localMouse, dragCursorRegion);
}

//----------------------------------------------------------------------------------------
// TTEView::GetWillDragCursorID:
//----------------------------------------------------------------------------------------

short TTEView::GetWillDragCursorID()
{
	return kNoResource;
}

//----------------------------------------------------------------------------------------
// TTEView::GetIsDraggingCursorID:
//----------------------------------------------------------------------------------------

short TTEView::GetIsDraggingCursorID()
{
	return kNoResource;
}

//----------------------------------------------------------------------------------------
// TTEView::DoAddDragContent:
//----------------------------------------------------------------------------------------

void TTEView::DoAddDragContent()
{
	TDragItem * dragItem = TDragDropSession::fgDragDropSession->AddDragItem(1);

	CFlavorFlags flags;
	dragItem->PromiseFlavor(kTextType_AC, flags);
	StScrpHandle styleHandle = ::TEGetStyleScrapHandle(fHTE);
	if (styleHandle)
	{
		dragItem->PromiseFlavor(kStyleType_AC, flags);
		::DisposeIfHandle_AC((Handle)styleHandle);
	}
}

//----------------------------------------------------------------------------------------
// TTEView::WillAcceptDrop:
//----------------------------------------------------------------------------------------

bool TTEView::WillAcceptDrop(CDragItemIterator& dragItemIterator)
{
	bool returnVal = false;

	if (fAcceptsChanges
		&& dragItemIterator.GetItemCount() == 1)
		returnVal = dragItemIterator.First()->FlavorExists(kTextType_AC);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TTEView::DoDragEnter:
//----------------------------------------------------------------------------------------

void TTEView::DoDragEnter()
{
	fLastCaretOffset = 0;
	fLastCaretTime = 0L;
	fCaretIsShown = false;

	if (TDragDropSession::fgDragDropSession->GetCurrentDragSource() == this)
	{
		fHiliteRgn = MakeNewRgn();
		CWhileOutlinePreferred_AC setOP(fPreferOutline);
		::TEGetHiliteRgn(fHiliteRgn, fHTE);
		::ThrowIfNULL_AC(fHiliteRgn);
	}
}

//----------------------------------------------------------------------------------------
// TTEView::DoDragWithin:
//----------------------------------------------------------------------------------------

void TTEView::DoDragWithin(const CViewPoint& localMouse)
{
	unsigned long currentTime = TickCount();
	short offset = -1;
	bool caretMoved = true;

	// if this view is the source and the target of the drag, don't allow dropping
	// into the selection
	if (fHiliteRgn && PtInRgn(ViewToQD(localMouse), fHiliteRgn))
	{
		if (fLastCaretOffset != -1)
		{
			offset = -1;
			caretMoved = true;					// force redraw
		}
	}
	else
	{
		offset = PtToOffset(localMouse);
		caretMoved = (fLastCaretOffset != offset);
	}

	if (caretMoved && fCaretIsShown)
		DrawCaret(fLastCaretOffset);

	if (caretMoved || (currentTime - fLastCaretTime) > GetCaretTime())
	{
		if (offset != -1)
			DrawCaret(offset);
		fLastCaretOffset = offset;
		fLastCaretTime = currentTime;
	}
}

//----------------------------------------------------------------------------------------
// TTEView::DoDragLeave:
//----------------------------------------------------------------------------------------

void TTEView::DoDragLeave()
{
	if (fCaretIsShown)
		DrawCaret(fLastCaretOffset);

	if (fHiliteRgn)
		fHiliteRgn = DisposeIfRgnHandle(fHiliteRgn);
}

//----------------------------------------------------------------------------------------
// TTEView::DoMakeDragDropCommand:
//----------------------------------------------------------------------------------------

TCommand* TTEView::DoMakeDragDropCommand(CommandNumber itsCommandNumber,
										 CDragItemIterator& dragItemIterator)
{
	TCommand * returnCommand = NULL;

	switch (itsCommandNumber)
	{
		case cDrag:
			returnCommand = TH_new TTECommand(this, itsCommandNumber, true);
			break;

		case cDrop:
			{
				Handle textHandle = NULL;
				StScrpHandle textStyles = NULL;

				TDragItem * firstItem = dragItemIterator.First();
				firstItem->FocusOnFlavor(kTextType_AC);
				textHandle = firstItem->GetDataAsHandle();
				::ThrowIfNULL_AC(textHandle);

				if (firstItem->FlavorExists(kStyleType_AC))
				{
					firstItem->FocusOnFlavor(kStyleType_AC);
					textStyles = (StScrpHandle)firstItem->GetDataAsHandle();
				}

				TTEDragDropCommand * dropCommand = TH_new TTEDragDropCommand(this, itsCommandNumber);
				dropCommand->SetNewText(textHandle, textStyles);
				dropCommand->SetNewStart(fLastCaretOffset);
				returnCommand = dropCommand;
			}
			break;

		case cDragMove:
			if (fLastCaretOffset != -1)		// can't drop into selection
				returnCommand = TH_new TTEDragMoveCommand(this, itsCommandNumber, fLastCaretOffset);
			break;

		default:
			returnCommand = Inherited::DoMakeDragDropCommand(itsCommandNumber, dragItemIterator);
			break;
	}

	return returnCommand;
}

//----------------------------------------------------------------------------------------
// TTEView::DoFulfillPromise
//----------------------------------------------------------------------------------------

void TTEView::DoFulfillPromise(TDragItem* promisedItem)
{
	switch (promisedItem->GetFlavorType())
	{
		case kTextType_AC:
			{
				CTempHandleLock_AC tempHandleLock(fText);

				try
				{
					long byteCount = (*fHTE)->selEnd - (*fHTE)->selStart;
					char* textPtr = *fText + (*fHTE)->selStart;
					promisedItem->SetData(textPtr, byteCount);
				}
				catch(CException_AC& theException)
				{
					DoCatchMessage_AC(theException);
					theException.SetError(cantGetFlavorErr);
					throw;
				}

			}
			break;

		case kStyleType_AC:
			{
				CTempHandle_AC tempHandle;

				try
				{
					tempHandle = (Handle)::TEGetStyleScrapHandle(fHTE);
					::ThrowIfNULL_AC(tempHandle);

					promisedItem->SetDataFromHandle(tempHandle);
				}
				catch(CException_AC& theException)
				{
					DoCatchMessage_AC(theException);
					theException.SetError(cantGetFlavorErr);
					throw;
				}

			}
			break;

		default:
			Inherited::DoFulfillPromise(promisedItem);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TTEView::DrawCaret:
//----------------------------------------------------------------------------------------

void TTEView::DrawCaret(short offset)
{
	//	Get the coordinates and the line of the offset to draw the caret.
	CPoint_AC theLoc(::TEGetPoint(offset, fHTE));
	short theLine = TEGetLine(offset, fHTE);

	long lineHeight = ::TEGetHeight(theLine, theLine, fHTE);

	//	For some reason, TextEdit doesn't return the proper coordinates
	//	of the last offset in the field if the last character in the record
	//	is a carriage return. TEGetPoint returns a point that is one line
	//	higher than expected. The following code fixes this problem.

	if ((offset == (*fHTE)->teLength) && (*((*fHTE)->hText))[(*fHTE)->teLength - 1] == 0x0D)
		theLoc.v += static_cast<short>(lineHeight);

	PenNormal();
	// The caret supposed to be nice and dark (which the white pattern will accomplish)
	PenPat(&qd.white);
	PenMode(notPatXor);							//	set invert mode

	::MoveToPt(--theLoc);							//	Draw the caret image
	Line(0, 1 - lineHeight);

	PenNormal();								// reset the pen

	fCaretIsShown = !fCaretIsShown;
}

#endif // qDrag

//----------------------------------------------------------------------------------------
// WriteChar:
//----------------------------------------------------------------------------------------
#if qDebug

const short kMaxIndex = 2047;
typedef char x[kMaxIndex];
typedef x* XPtr;
typedef XPtr* XHandle;

void TTEView::WriteChar(short index, Handle hText)
{
	if (index <= kMaxIndex)
		ProgramReport_AC(false, "[%1d:%1d]", index, *((XHandle)hText)[index]);
}
#endif

//----------------------------------------------------------------------------------------
// DumpTERecord:
//----------------------------------------------------------------------------------------
#if qDebug

void TTEView::DumpTERecord(TEHandle aTEH)
{
	const short kMaxCharsToPrint = 100;

	ProgramReport_AC(false, "TE -- dest CRect_AC: %s\n", (const char*)(CRect_AC((*aTEH)->destRect)));

	short size = (*aTEH)->teLength;
	ProgramReport_AC(false, "Line ht: %1d", (*aTEH)->lineHeight);
	ProgramReport_AC(false, "; teLength: %1d", size);
	ProgramReport_AC(false, "; hText length: %1d", GetHandleSize((*aTEH)->hText));
	ProgramReport_AC(false, "; lines: %1d", (*aTEH)->nLines);
	ProgramReport_AC("\n");

	if (gIntenseDebugging)
	{
		Handle hText = (*aTEH)->hText;

		ProgramReport_AC("Chars: ");
		short minSize = Min_AC(static_cast<short>(2047), static_cast<short>(size - 1));
		for (short i = 0; i <= Min_AC(kMaxCharsToPrint, minSize); ++i)
		{
			WriteChar(i, hText);
			if (i % 10 == 9)
				ProgramReport_AC("\n");
		}
		ProgramReport_AC("\n");

		ProgramReport_AC(false, "InPort vis bbox: %s", (const char*)(CRect_AC((*TWindow::GetVisRegion((*aTEH)->inPort))->rgnBBox)));
		ProgramReport_AC(false, "; clip bbox: %s\n", (const char*)(CRect_AC((*TView::GetClipRegion((*aTEH)->inPort))->rgnBBox)));

		if ((*aTEH)->inPort != qd.thePort)
		{
			ProgramReport_AC(false, "thePort vis bbox: %s", (const char*)(CRect_AC((*TWindow::GetVisRegion(qd.thePort))->rgnBBox)));
			ProgramReport_AC(false, "; clip bbox: %s\n", (const char*)(CRect_AC((*TView::GetClipRegion(qd.thePort))->rgnBBox)));
		}
	}

}
#endif

//----------------------------------------------------------------------------------------
// SetSelect:  Usually, TTEView::SetSelection should be used instead.  This
// procedure is used by MacApp to set the selection 'behind the scenes' (that is,
// in instances where an O.S. routine operates on the current selection, but we
// want to change a piece of the text without changing the selection).
//----------------------------------------------------------------------------------------

void TTEView::SetSelect(short theStart,
						short theEnd,
						TEHandle hTE)
{
	theStart = Max_AC(theStart, static_cast<short>(0));

	(*hTE)->selStart = theStart;
	(*hTE)->selEnd = MinMax_AC(theStart, theEnd, (*hTE)->teLength);
}

//----------------------------------------------------------------------------------------
// ClickLoopForTTEView:
//----------------------------------------------------------------------------------------

pascal Boolean TTEView::ClickLoopForTTEView()
{
	return fgCurrTEView ? fgCurrTEView->ClickLoop() : true;
}

//----------------------------------------------------------------------------------------
// TEGetLine - static function
//----------------------------------------------------------------------------------------

short TTEView::TEGetLine(short offset, TEHandle theTE)
//	TEGetLine, given an offset and a TextEdit handle, returns the line number
//	of the line that contains the offset.
//	NOTE: the resulting line number is 1-based
{
	short returnVal;

	if (offset > (*theTE)->teLength)
		returnVal = (*theTE)->nLines;
	else
	{
		short line = 0;
		while ((*theTE)->lineStarts[line] < offset)
			++line;

		returnVal = line;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEIsFrontOfLine:
//----------------------------------------------------------------------------------------

short TTEView::TEIsFrontOfLine(short offset, TEHandle theTE)
//	TEIsFrontOfLine, given an offset and a TextEdit handle, returns true if
//	the given offset is at the beginning of a line start.
{
	bool returnVal = false;

	short line = 0;

	if (!(*theTE)->teLength)
		returnVal = true;
	else if (offset >= (*theTE)->teLength)
		returnVal = ((*((*theTE)->hText))[(*theTE)->teLength - 1] == chReturn);
	else
	{
		while ((*theTE)->lineStarts[line] < offset)
			++line;

		returnVal = ((*theTE)->lineStarts[line] == offset);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// IsBlank:
//----------------------------------------------------------------------------------------

bool TTEView::IsBlank(Handle theText,
					  short offset,
					  short& length, short inScript)
{
	bool returnVal = false;

	short i = MACharacterType(*theText, offset, inScript);
	length = 0;

	if ((i & smcClassMask) == smPunctBlank)
	{
		length = MACharacterByteType(*theText, offset, inScript) == smSingleByte ? 1 : 2;

		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// End of UTEView.cp
