//----------------------------------------------------------------------------------------
// UTabBehaviors.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UTABBEHAVIORS__
#include "UTabBehaviors.h"
#endif

// MacApp

#ifndef __UEVENT__
#include "UEvent.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//========================================================================================
// CLASS TTabber
//========================================================================================
#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TTabber);

//----------------------------------------------------------------------------------------
// TTabber constructor
//----------------------------------------------------------------------------------------


TTabber::TTabber(bool recursive) :
	TBehavior(kTabBehavior),

	fNext(NULL),
	fFirst(NULL),
	fRecursive(recursive),
	fFoundCurrent(false)
{
}

//----------------------------------------------------------------------------------------
// TTabber destructor
//----------------------------------------------------------------------------------------


TTabber::~TTabber()
{
}

//----------------------------------------------------------------------------------------
// TTabber::ReadFrom: 
//----------------------------------------------------------------------------------------


void TTabber::ReadFrom(CStream_AC* aStream)				// override 
{
	Inherited::ReadFrom(aStream);

	fRecursive = aStream->ReadBoolean();
}

//----------------------------------------------------------------------------------------
// TTabber::WriteTo: 
//----------------------------------------------------------------------------------------


void TTabber::WriteTo(CStream_AC* aStream) const				// override 
{
	Inherited::WriteTo(aStream);

	aStream->WriteBoolean(fRecursive);
}

//----------------------------------------------------------------------------------------
// TTabber::DoKeyEvent: 
//----------------------------------------------------------------------------------------


void TTabber::DoKeyEvent(TToolboxEvent* event)
{
	if (event && event->GetText() == chTab)
		Tab(event->IsShiftKeyPressed());
	else
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TTabber::Tab: 
//----------------------------------------------------------------------------------------


void TTabber::Tab(bool tabBackward)
{
	Reset();

	FindTargets(tabBackward);
	
	if (!fNext)
	    fNext = fFirst;
	
	if (fNext)
	{
		if (fNext->BecomeTarget())
			fNext->SetTargetSelection(kRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TTabber::Reset: 
//----------------------------------------------------------------------------------------


void TTabber::Reset()
{
	fFirst = NULL;
	fNext = NULL;
	fFoundCurrent = false;
}

//----------------------------------------------------------------------------------------
// TTabber::FindSubViewTargets: 
//----------------------------------------------------------------------------------------


void TTabber::FindSubViewTargets(TView* parent,
										bool tabBackward)
{
	for (CSubViewIterator iter(parent,!tabBackward); iter; ++iter)
	{
		bool canBeTarget = iter->IsEnabled() && iter->IsShown() && iter->WantsToBeTarget();

		if (!fFirst && canBeTarget)
			fFirst = iter;

		if (TEventHandler::GetTarget() == iter.Current())
			fFoundCurrent = true;
		else if (fFoundCurrent && !fNext && canBeTarget)
		{
			fNext = iter;
			break;
		}

		if (fRecursive)
			FindSubViewTargets(iter,tabBackward);
	}
}

//----------------------------------------------------------------------------------------
// TTabber::FindTargets: 
//----------------------------------------------------------------------------------------


void TTabber::FindTargets(bool /* tabBackward */)
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TTabber::GetStandardSignature:
//----------------------------------------------------------------------------------------


IDType TTabber::GetStandardSignature() const
{
	return kTabBehavior;
}

//========================================================================================
// CLASS TMultiWindowTabber
//========================================================================================
#undef Inherited
#define Inherited TTabber


MA_DEFINE_CLASS(TMultiWindowTabber);

//----------------------------------------------------------------------------------------
// TMultiWindowTabber destructor
//----------------------------------------------------------------------------------------


TMultiWindowTabber::~TMultiWindowTabber()
{
}

//----------------------------------------------------------------------------------------
// TMultiWindowTabber::FindTargets: 
//----------------------------------------------------------------------------------------


void TMultiWindowTabber::FindTargets(bool tabBackward)	// Override
{
	TWindow* theFrontWindow = TWindow::WMgrToWindow(FrontWindow());
	
	if (theFrontWindow)	
		if (theFrontWindow->IsModal())
			FindSubViewTargets(theFrontWindow,tabBackward);
		else
			for (CWMgrIterator iter(!tabBackward); iter; ++iter)
			{
				TWindow* aWindow = TWindow::WMgrToWindow(iter);
			
				if (aWindow && aWindow->IsShown() && aWindow->IsActive())
					FindSubViewTargets(aWindow,tabBackward);
			}
}


//========================================================================================
// CLASS TViewTabber
//========================================================================================
#undef Inherited
#define Inherited TTabber


MA_DEFINE_CLASS(TViewTabber);

//----------------------------------------------------------------------------------------
// TViewTabber destructor
//----------------------------------------------------------------------------------------


TViewTabber::~TViewTabber()
{
}

//----------------------------------------------------------------------------------------
// TViewTabber::FindTargets: 
//----------------------------------------------------------------------------------------


void TViewTabber::FindTargets(bool tabBackward)	// Override
{
	FindSubViewTargets( dynamic_cast_AC(TView*,fOwner), tabBackward);
}


//----------------------------------------------------------------------------------------
// End of UTabBehaviors.cp


