//----------------------------------------------------------------------------------------
// UTargetBorderView.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UTARGETBORDERVIEW__
#include "UTargetBorderView.h"
#endif

// ACS

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDISPATCHER__ 
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

void InitUTargetBorderView()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TTargetBorderView, kStdTargetBorderView);
	}
}

//========================================================================================
// CLASS TTargetBorderView
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TTargetBorderView);

//----------------------------------------------------------------------------------------
// TTargetBorderView constructor 
//----------------------------------------------------------------------------------------


TTargetBorderView::TTargetBorderView(const CViewPoint& itsLocation,
										  const CViewPoint& itsSize,
										  SizeDeterminer itsHSizeDet,
										  SizeDeterminer itsVSizeDet,
										  IDType itsTargetView) :
	TView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fTargetView(itsTargetView)
{
	fActiveHL = hlOff;	 			// Gets turned on when a subview becomes the target 
	fInactiveHL = hlOff; 			// Don't do dim highlighting 
}

//----------------------------------------------------------------------------------------
// TTargetBorderView destructor
//----------------------------------------------------------------------------------------


TTargetBorderView::~TTargetBorderView()
{
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TTargetBorderView::GetStandardSignature() const	// Override 
{
	return kStdTargetBorderView;
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::ReadFields: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::ReadFields(CStream_AC* aStream) // Override
{
	Inherited::ReadFields(aStream);
	
	fTargetView = aStream->ReadIDType();
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::WriteFields: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::WriteFields(CStream_AC* aStream) const // Override
{
	Inherited::WriteFields(aStream);
	aStream->WriteIDType(fTargetView);
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::DoEvent: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event)	// Override
{
	switch (eventNumber)
	{
		case mBecameTarget:  
			if (ContainsTarget(source))
			{
				InvalidateBorderRegion();
			}
			Inherited::DoEvent(eventNumber, source, event);
			break;
		case mResignedTarget:  
			if (ContainsTarget(source))
			{
				InvalidateBorderRegion();
			}
			Inherited::DoEvent(eventNumber, source, event);
			break;
			
		default:	
			Inherited::DoEvent(eventNumber, source, event);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::DoMouseCommand: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::DoMouseCommand( CViewPoint& 		/* theMouse */,
										TToolboxEvent* 	/* event */,
										CPoint_AC			/* hysteresis */)
{
	TView *targetView = FindSubView(fTargetView);
	if (targetView && targetView->IsEnabled() && targetView->WantsToBeTarget())
		targetView->BecomeTarget();
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::ComputeBorderRegion: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::ComputeBorderRegion(RgnHandle borderRegion)
{
	CRect_AC QDArea(GetQDExtent());
	RectRgn(borderRegion, &QDArea);
	QDArea.Inset(2, 2);

	CTempRegion_AC innerRegion;
	RectRgn(innerRegion, &QDArea );
	DiffRgn(borderRegion, innerRegion, borderRegion);
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::InvalidateBorderRegion: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::InvalidateBorderRegion()
{
	CTempRegion_AC borderRegion;
	
	ComputeBorderRegion(borderRegion);
	Invalidate(borderRegion);
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::Draw: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::Draw(const CViewRect& /*area*/)
{
	if (ContainsTarget(GetTarget()))
	{
		CTempRegion_AC borderRegion;
		
		ComputeBorderRegion(borderRegion);
		PaintRgn(borderRegion);
	}
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::SetFrame: 
//----------------------------------------------------------------------------------------


void TTargetBorderView::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	// make sure the entire old frame is invalidated
	if (invalidate)
		InvalidateBorderRegion();
	
	Inherited::SetFrame(newFrame, invalidate);
	
	// make sure the entire new frame is invalidated
	if (invalidate)
		InvalidateBorderRegion();
}

//----------------------------------------------------------------------------------------
// TTargetBorderView::ContainsTarget: 
//----------------------------------------------------------------------------------------


bool TTargetBorderView::ContainsTarget(TEventHandler *target)
{
	TEventHandler 	*localTarget = target;
	bool			containsTarget = false;
	
	while (localTarget && !containsTarget)
	{
		if (localTarget == this)
			containsTarget = true;
		localTarget = localTarget->GetNextHandler();
	}
	return containsTarget;
}
