//----------------------------------------------------------------------------------------
// UTearOffMenu.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UTEAROFFMENUVIEW__
#include "UTearOffMenuView.h"
#endif

// MacApp

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UPOINTEROBJECT__
#include "UPointerObject.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

//========================================================================================
// CLASS TShowTearOffWindowCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TShowTearOffWindowCommand constructor
//----------------------------------------------------------------------------------------

TShowTearOffWindowCommand::TShowTearOffWindowCommand(TWindow* tearOffWindow,
														 CPoint_AC where) :
	TCommand(cNoCommand, NULL, kCantUndo, kDoesNotCauseChange, NULL),

	fTearOffWindow(tearOffWindow),
	fWhere(where)
{
}

//----------------------------------------------------------------------------------------
// TShowTearOffWindowCommand destructor
//----------------------------------------------------------------------------------------

TShowTearOffWindowCommand::~TShowTearOffWindowCommand()
{
}

//----------------------------------------------------------------------------------------
// TShowTearOffWindowCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TShowTearOffWindowCommand::DoIt()	// override 
{
	if (fTearOffWindow)
	{
		fTearOffWindow->Show(false, false);			// Hide the window so that Open will call
		fTearOffWindow->SetMustForceOnScreen(true);	// ForceOnScreen before showing
		fTearOffWindow->SetForcedOnScreen(false);
		
		fTearOffWindow->Locate(fWhere, kDontInvalidate);
		
		fTearOffWindow->Open();						// show the torn-off window 
		
		fTearOffWindow->Select();					// select the torn-off window 
	}
}

//========================================================================================
// CLASS TTearOffMenuViewTracker
//========================================================================================
#undef Inherited
#define Inherited TTracker

//----------------------------------------------------------------------------------------
// TTearOffMenuViewTracker destructor
//----------------------------------------------------------------------------------------

TTearOffMenuViewTracker::~TTearOffMenuViewTracker()
{
}

//----------------------------------------------------------------------------------------
// TTearOffMenuViewTracker::IsReadyToPost: 
//----------------------------------------------------------------------------------------

bool TTearOffMenuViewTracker::IsReadyToPost()	// override
{
	gDispatcher->SetEventLevel(gDispatcher->GetEventLevel() + 1);
	Process();				// perform this right away
	gDispatcher->SetEventLevel(gDispatcher->GetEventLevel() - 1);

	return false;
}


//========================================================================================
// CLASS TTearOffTracker
//========================================================================================
#undef Inherited
#define Inherited TTearOffMenuViewTracker

//----------------------------------------------------------------------------------------
// TTearOffTracker constructor
//----------------------------------------------------------------------------------------

TTearOffTracker::TTearOffTracker(CPoint_AC hitPt, TTearOffMenuView* tearOffMenu) :
	TTearOffMenuViewTracker(cNoCommand, NULL, kCantUndo, kDoesNotCauseChange, NULL,
					NULL, NULL, CViewPoint(hitPt)),

	fTearOffWindowOutline(MakeNewRgn()),
	fTearOffMenu(tearOffMenu),
	fExitTracking(false)
{
	fFreeOnCompletion = false;
}

//----------------------------------------------------------------------------------------
// TTearOffTracker::Free: 
//----------------------------------------------------------------------------------------

TTearOffTracker::~TTearOffTracker()
{
}

//----------------------------------------------------------------------------------------
// TTearOffTracker::DoIt: 
//----------------------------------------------------------------------------------------

void TTearOffTracker::DoIt()
{
	if (!fExitTracking)					// ensure that we didn't exit the tracking loop
	{
		TWindow * aTearOffWindow = fTearOffMenu->fTearOffWindow;

		CPoint_AC where = ((CRect_AC &) (*fTearOffWindowOutline)->rgnBBox)[topLeft] + aTearOffWindow->GetContentRegionInset();

		if (aTearOffWindow)
			fTearOffMenu->PostCommand(TH_new TShowTearOffWindowCommand(aTearOffWindow, where));
			// NOTE: we can't just move and show the tear off window here because the menu
            // that we want to tear off is currently displayed on the screen and the menu
            // manager has cached the bits underneath the currently displayed menu and we
            // might be moving our floating window into that territory thus invalidating
            // the menu manager's cache (which the menu manager will blit back in place
            // momentarily). To work-around this "feature" we'll simply post a command
            // which will do the work later. Isn't MacApp wonderful!
	}
}

//----------------------------------------------------------------------------------------
// TTearOffTracker::TrackFeedback: 
//----------------------------------------------------------------------------------------

void TTearOffTracker::TrackFeedback(TrackPhase /* aTrackPhase */,
									const CViewPoint& /* anchorPoint */,
									const CViewPoint& /* previousPoint */,
									const CViewPoint& /* nextPoint */,
									bool mouseDidMove,
									bool /* turnItOn */)
{
	if (mouseDidMove && !EmptyRgn(fTearOffWindowOutline))
	{
		PenPat(&qd.gray);
		FrameRgn(fTearOffWindowOutline);		// draw/erase 
	}
}

//----------------------------------------------------------------------------------------
// TTearOffTracker::TrackMouse: 
//----------------------------------------------------------------------------------------

TTracker* TTearOffTracker::TrackMouse(TrackPhase aTrackPhase,
									  CViewPoint& ,
									  CViewPoint& previousPoint,
									  CViewPoint& nextPoint,
									  bool)
{
	TWindow * aTearOffWindow;
	short verticalOffset;
	short horizontalOffset;
	bool rgnsWereBuilt;

	CPoint_AC qdPt = nextPoint.ToPoint();
	fExitTracking =!PtInRgn(qdPt, fTearOffMenu->fTearOffTrackingRegion);

	if (aTrackPhase == trackBegin)
	{
		aTearOffWindow = fTearOffMenu->fTearOffWindow;
		if (aTearOffWindow)
		{
			rgnsWereBuilt = aTearOffWindow->BuildWindowRegions(kBuild);
			GetWindowStructureRgn(aTearOffWindow->GetWindowRef(), fTearOffWindowOutline);
			aTearOffWindow->BuildWindowRegions(rgnsWereBuilt);

			// move the outline to the topleft of the window mgr graf port 
			OffsetRgn(fTearOffWindowOutline, -(*fTearOffWindowOutline)->rgnBBox.left, -(*fTearOffWindowOutline)->rgnBBox.top);

			// move the outline so that it's centered at the mouse 
			verticalOffset = (((CRect_AC &) (*fTearOffWindowOutline)->rgnBBox).GetLength(vSel)) / 2;
			horizontalOffset = (((CRect_AC &) (*fTearOffWindowOutline)->rgnBBox).GetLength(hSel)) / 2;
			OffsetRgn(fTearOffWindowOutline, static_cast<short>(previousPoint.h - horizontalOffset), static_cast<short>(previousPoint.v - verticalOffset));
		}
	}
	else
		OffsetRgn(fTearOffWindowOutline, static_cast<short>(nextPoint.h - previousPoint.h), static_cast<short>(nextPoint.v - previousPoint.v));
	return this;
}

//----------------------------------------------------------------------------------------
// TTearOffTracker::IsDoneTracking: 
//----------------------------------------------------------------------------------------

bool TTearOffTracker::IsDoneTracking()	// override 
{
	return ((!StillDown()) || fExitTracking);
}

//========================================================================================
// CLASS TTearOffMenuView
//========================================================================================
#undef Inherited
#define Inherited TMenuView

MA_DEFINE_CLASS(TTearOffMenuView);

//----------------------------------------------------------------------------------------
// TTearOffMenuView constructor
//----------------------------------------------------------------------------------------

TTearOffMenuView::TTearOffMenuView(ResNumber rsrcID,
								   short menuWidth,
								   short menuHeight,
								   TWindow* tearOffWindow) :
	TMenuView(rsrcID, menuWidth, menuHeight),
	fTearOffTrackingRegion(MakeNewRgn()),
	fTearOffWindow(tearOffWindow)
{
	InstallTearOffTracker();
}

//----------------------------------------------------------------------------------------
// TTearOffMenuView::InstallTearOffTracker: For the sake of efficiency when tracking a
// tear-off, we'll create the tear-off tracker and keep it with us, so we don't need to
// allocate a new tracker every time we tear off a menu. If you want a different tracker
// in your subclass of TTearOffMenuView, then override this method.
//----------------------------------------------------------------------------------------

void TTearOffMenuView::InstallTearOffTracker()
{
	fTearOffTracker = TH_new TTearOffTracker(gZeroPt, this);
}

//----------------------------------------------------------------------------------------
// TTearOffMenuView::Free: 
//----------------------------------------------------------------------------------------

TTearOffMenuView::~TTearOffMenuView()
{
}

//----------------------------------------------------------------------------------------
// TTearOffMenuView::HandleChooseMessage: 
//----------------------------------------------------------------------------------------

void TTearOffMenuView::HandleChooseMessage(short,
										   MenuRef,
										   CRect_AC& menuRect,
										   CPoint_AC hitPt,
										   short& whichItem)// override 
{
	short newItem = kNoMenuItem;						// default return 

	// so that trackers know that they're tracking in menus 
	bool saveTrackingInMenu = gTrackingInMenu;
	gTrackingInMenu = true;

	if (IsEnabled())							// menu enabled 
	{
		// see if CPoint_AC is within hit area 
		CRect_AC hitRect = GetQDExtent() + fBorder;
		if (hitRect.Contains(hitPt))			// in menu view (not border) 
		{
			// NOTE: Either your subclass of TTearOffMenuView should override
            // DoMouseCommand or one of TTearOffMenuView's view's subview's should
            // override DoMouseCommand so that it creates and posts a TTracker. SubClasses
            // of TTearOffMenuViewTrackers will be tracked immediately because of the its
            // override of IsReadyToPost. So, when we're done with HandleMouseDown below,
            // we simply tell the menu manager that no menu item was selected, ie
			// newItem == kNoMenuItem.

			EventRecord theEventRecord;
			theEventRecord.what = mouseDown;
			theEventRecord.message = 0;
			theEventRecord.when = TickCount();
			theEventRecord.where = hitPt;
			theEventRecord.modifiers = 0;
			
			if (Button())
				theEventRecord.modifiers |= btnState;
				
			if (IsCommandKeyDown())
				theEventRecord.modifiers |= cmdKey;
				
			if (IsOptionKeyDown())
				theEventRecord.modifiers |= optionKey;
			
			CTempObjectAllocation tempObjectAllocation(true);
			TToolboxEvent event(NULL, &theEventRecord);
			TempObjectAllocation(tempObjectAllocation); // set allocation state back now, before handling the mouse down
			
			event.SetClickCount(gDispatcher->GetClickCount());
			event.SetAffectsMenus(false);
			HandleMouseDown(hitPt, &event, GetStdHysteresis());
		}
		else
		{										// determine if we should track a tear off image
			LocalToGlobal(hitPt);
			if (PtInRgn(hitPt, GetTearOffTrackingRegion(menuRect)))
			{
				fTearOffTracker->fInitialPt = hitPt;
				PostCommand(fTearOffTracker);
			}
		}
	}
	else
		UpdateHighlight(whichItem, newItem);	// Update highlighting 

	gTrackingInMenu = saveTrackingInMenu;

	// Tell MenuManager about new item 
	whichItem = newItem;
}

//----------------------------------------------------------------------------------------
// TTearOffMenuView::GetTearOffTrackingRegion: This function returns
// fTearOffTrackingRegion
//----------------------------------------------------------------------------------------

RgnHandle TTearOffMenuView::GetTearOffTrackingRegion(const CRect_AC& menuRect)
{
	if (EmptyRgn(fTearOffTrackingRegion))
	{
		CRect_AC rgnRect(menuRect);
		
		// calculate the tracking region for the tearoff image
		LocalToGlobal(rgnRect[topLeft]);
		LocalToGlobal(rgnRect[botRight]);
		rgnRect.Inset(-kTearOffMargin, -kTearOffMargin);

		CTempRegion_AC tempRgn;

		RectRgn(tempRgn, rgnRect);
		DiffRgn(GetGrayRgn(), tempRgn, fTearOffTrackingRegion);
		InsetRgn(fTearOffTrackingRegion, 4, 4);
	}
	return fTearOffTrackingRegion;
}

//----------------------------------------------------------------------------------------
// End of UTearOffMenuView.cp


