//----------------------------------------------------------------------------------------
// UUpDownControl.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UUPDOWNCONTROL__
#include "UUpDownControl.h"
#endif

// MacApp

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//----------------------------------------------------------------------------------------
// Unit initialization
//----------------------------------------------------------------------------------------

void InitUUpDownControl()
{
	MA_REGISTER_CLASS(TUpDownControl);
}

//========================================================================================
// CLASS TUpDownControl
//========================================================================================

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS(TUpDownControl);

//----------------------------------------------------------------------------------------
// TUpDownControl constructor
//----------------------------------------------------------------------------------------

TUpDownControl::TUpDownControl(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
							     const CTextStyle_AC& itsTextStyle,
							     IDType itsLinkedNumberTextID) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle),
	fAGAControl(NULL),
	fLinkedNumberTextID(itsLinkedNumberTextID),
	fLinkedNumberText(NULL)
{
	// caller must call virtual function CreateControlAGA();
}

//----------------------------------------------------------------------------------------
// TUpDownControl destructor
//----------------------------------------------------------------------------------------

TUpDownControl::~TUpDownControl()
{
}

//----------------------------------------------------------------------------------------
// TUpDownControl::ReadFields: 
//----------------------------------------------------------------------------------------

void TUpDownControl::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fLinkedNumberTextID;
}

//----------------------------------------------------------------------------------------
// TUpDownControl::WriteFields: 
//----------------------------------------------------------------------------------------

void TUpDownControl::WriteFields(CStream_AC* aStream) const			// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fLinkedNumberTextID;
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::HandleNotification(SInt32 deltaValue)
{
	// Update the linked TNumberText, if any, call the
	// installed notification function, and propagate our
	// event number.

	if (fLinkedNumberText != NULL)
		fLinkedNumberText->SetValue(fLinkedNumberText->GetValue() + deltaValue, kRedraw);

	HandleEvent(fEventNumber, this, NULL);

	(void) Focus();
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);
	
	CreateAGAControl();
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::CreateAGAControl()
{
	fAGAControl = TH_new CUpDownControl_AC(ControlQDArea());
	fAGAControl->InstallNotificationRoutine((CUpDownControlCallback_AC) RealAGANotifier, this);

	if (fDimmed || ! fEnabled)
		fAGAControl->SetEnable(CControl_AC::kDisabled, CControl_AC::kDontRedraw);

	if (fLinkedNumberTextID != kNoIdentifier)
	{
		fLinkedNumberText = dynamic_cast<TNumberText*>(GetRootView()->FindSubView(fLinkedNumberTextID));
		if (fLinkedNumberText != NULL)
			if (!dynamic_cast<TNumberText*>(fLinkedNumberText))
				fLinkedNumberText = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::Draw(const CViewRect& /*area*/)
{
	if (fAGAControl)
		fAGAControl->DrawObject();
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);
	
	if (fAGAControl)
		fAGAControl->SetBounds(ControlQDArea(), CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* /*event*/, CPoint_AC /*hysteresis*/)
{
	if (fAGAControl->TrackMouse(ViewToQD(theMouse)))
		HandleEvent(fEventNumber, this, NULL);
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::Activate(bool entering)
{
	if (fAGAControl)
		fAGAControl->SetActive(entering, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::DimState(bool state, bool redraw)
{
	fDimmed = state;

	if (fAGAControl)
		fAGAControl->SetEnable(!state, redraw && Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::Dim()
{
	if (fAGAControl)
		fAGAControl->SetEnable(CControl_AC::kDisabled, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::SetEnable(bool state)
{
	Inherited::SetEnable(state);

	// Set the CControl_AC's enable state, without redraw.
	if (fAGAControl)
		fAGAControl->SetEnable(state, CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TUpDownControl
//----------------------------------------------------------------------------------------

void TUpDownControl::RealAGANotifier(CIndicator_AC* /*theIndicator*/, SInt32 dataValue, TUpDownControl* notifierTarget)
{
	// This is a static function.

	notifierTarget->HandleNotification(dataValue);
}
