//----------------------------------------------------------------------------------------
// UViewIdler.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UViewIdler__
#include "UViewIdler.h"
#endif

// MacApp

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

//========================================================================================
// CLASS TViewIdler
//========================================================================================

#undef Inherited



// MA_DEFINE_CLASS(TViewIdler);

//----------------------------------------------------------------------------------------
// TViewIdler constructor
//----------------------------------------------------------------------------------------


TViewIdler::TViewIdler(TView* itsView) :
	fView(itsView)
{
	// Add me as a dependent of this view so I am notified when it is closed.
	fView->AddDependent(this);

	// Add me as a behavior of the application so I always get idle time
	if (gDispatcher)
		gDispatcher->AddBehavior(this);	

	// Set my idle frequency to that of my view, if one has been specified,
	// otherwise maximize amount of idle time by using smallest possible idle freq.
	const long kMagicIdleNumber = 1;
	long myIdleFreq = fView->GetIdleFreq();
	if (myIdleFreq == kMaxIdleTime)
		myIdleFreq = kMagicIdleNumber;
	SetIdleFreq(myIdleFreq); // NB, a virtual function call, any overrides won't get called.
	fView->SetIdleFreq(myIdleFreq); // Make sure my view has the same value.
}

//----------------------------------------------------------------------------------------
// TViewIdler::Free:
//----------------------------------------------------------------------------------------


TViewIdler::~TViewIdler()
{
	if (gDispatcher)
		gDispatcher->RemoveBehavior(this);
}

//----------------------------------------------------------------------------------------
// TViewIdler::DoIdle:
//----------------------------------------------------------------------------------------


bool TViewIdler::DoIdle(IdlePhase phase)
{
	if (fView != NULL)
	{
		// I only need to idle the view if it isn't idling itself.
		if (fView != gDispatcher->GetTarget())
			(void) fView->DoIdle(phase);
	}
#if qDebug
	else
		ProgramBreak_AC("My view has been freed and nobody told me about it (TView::Close() was not called or was overridden) :-(");
#endif

	return false;								// Did not free myself 
}

//----------------------------------------------------------------------------------------
// TViewIdler::DoUpdate:
//----------------------------------------------------------------------------------------


void TViewIdler::DoUpdate(ChangeID_AC theChange, 
								MDependable_AC* changedObject,
								void* /*changeData*/,
								CDependencySpace_AC* /*dependencySpace*/)
{
	if (changedObject == fView && theChange == mClosed)
	{
		delete this;	// self-destruct (destructor removes self as behavior & MDependable removes the dependency)	
	}
}

//----------------------------------------------------------------------------------------
// End of UViewIdler.cp


