//----------------------------------------------------------------------------------------
// CreatorAndBundle.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
// CreatorAndBundle -- post process application for bundle resource and shared bit
//----------------------------------------------------------------------------------------

// MacApp

#ifndef __UCPLUSTOOL__
#include <UCPlusTool.h>
#endif

// ToolBox

#ifndef __FINDER__
#include <Finder.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif


class TCreatorAndBundleTool : public TCPlusTool
{
  public:
	inline TCreatorAndBundleTool(int argc, char** argv) : TCPlusTool(argc, argv) {}	// constructor
	
	virtual ~TCreatorAndBundleTool();
		// Destructor
	
	virtual void DoProcessFileArg(const CStr255& arg);	// override
	
	virtual void DoToolAction(); // override
	
	void DoAutoMagicCreatorAndBundleAndSharedBit();

  protected:
	CStr255 fResFileName;

};


//========================================================================================
// CLASS TCreatorAndBundleTool
//========================================================================================
//----------------------------------------------------------------------------------------
// TCreatorAndBundleTool destructor
//----------------------------------------------------------------------------------------
TCreatorAndBundleTool::~TCreatorAndBundleTool()
{
}

//----------------------------------------------------------------------------------------
// TCreatorAndBundleTool::DoProcessFileArg: 
//----------------------------------------------------------------------------------------
void TCreatorAndBundleTool::DoProcessFileArg(const CStr255& arg)
{
	if (fResFileName.Length() > 0)
		Stop("Sorry only smart enough to process one file");
	else
		fResFileName = arg;
}

//----------------------------------------------------------------------------------------
// TCreatorAndBundleTool::DoAutoMagicCreatorAndBundleAndSharedBit: 
//----------------------------------------------------------------------------------------
typedef OSType** OSTypeHandle;

void TCreatorAndBundleTool::DoAutoMagicCreatorAndBundleAndSharedBit()
{
	CStr255 aName;
	FInfo theFInfo;
	OSTypeHandle theBundle;
	
	aName = fResFileName;
	FailOSErr(GetFInfo(aName, 0, &theFInfo));
	theBundle = (OSTypeHandle)Get1IndResource('BNDL', 1);
	if (theBundle)
	{
		theFInfo.fdCreator = **theBundle;
		theFInfo.fdFlags |= kHasBundle;
	}
	else
	{
		theFInfo.fdCreator = '????';
		theFInfo.fdFlags &= (~kHasBundle);
	}
	theFInfo.fdFlags |= kIsShared;	// by default MacApp apps can be shared cause they don't write back to themselves.

	FailOSErr(SetFInfo(aName, 0, &theFInfo));
}

//----------------------------------------------------------------------------------------
// TCreatorAndBundleTool::DoToolAction: 
//----------------------------------------------------------------------------------------
void TCreatorAndBundleTool::DoToolAction()
{
	CStr255 aName;
	short ref;
	
	if (fResFileName.IsEmpty())
		Stop("No filename specified");

	aName = fResFileName;

	SetResLoad(false);			// avoid preloads
	ref = OpenResFile(aName);
	SetResLoad(true);
	if (ref == -1)
		FailResError();

	DoAutoMagicCreatorAndBundleAndSharedBit();

	CloseResFile(ref);
	FailResError();
}

//----------------------------------------------------------------------------------------
// main: 
//----------------------------------------------------------------------------------------
void main(int argc, char** argv)
{
	InitUCPlusTool();
	
	TCreatorAndBundleTool aCreatorAndBundleTool(argc,argv);
	gCreatorAndBundleTool.Run();
}

//----------------------------------------------------------------------------------------
// End of CreatorAndBundle.cp
