//----------------------------------------------------------------------------------------
// MABuildTool.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCPLUSTOOL__
#include "UCPlusTool.h"
#endif

// MacApp

#ifndef __MACAPPVERSION__
#include "MacAppVersion.h"
#endif

#ifndef __UASSOCIATION__
#include "UAssociation.h"
#endif

#ifndef __USTRINGHANDLE__
#include "UStringHandle.h"
#endif

// ACS

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// Toolbox

#ifndef __RESOURCES__
#include <Resources.h>
#endif

// MPW

#ifndef __INTENV__
#include <IntEnv.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

#ifndef __STRING__
#include <string.h>
#endif


//========================================================================================
// CONSTANTS
//========================================================================================

//----------------------------------------------------------------------------------------
// Resource IDs
//----------------------------------------------------------------------------------------
enum
{
	kHelpStr				= 129	// Resource ID of the stringlist printed for help
};

//----------------------------------------------------------------------------------------
// Keyword IDs
//----------------------------------------------------------------------------------------
//	set i 0
//	loop
//		find /'= '[0-9]+';'/
//		break if {status}
//		replace /[0-9]+/ "{i}"
//		evaluate i={i}+1
//	end

enum
{
	kwUnspecified,
	
	kwAsm,
	kwBuildFlags,
	kwBuildFolder,	// Optional folder to place the object folders for a build in
	kwC,
	kwCP,
	kwCPlusPlus,
	kwCreatorAndBundle,
	kwd,
	kwE,
	kwFat,
	kwFolderName,
	kwOutputFile,
	
	kwLib,
	kwLink,
	kwMake,
	kwMakeSym,
	kwMrC,
	kwMWC68K,
	kwMWLink68K,
	kwMWCPPC,
	kwMWLinkPPC,
	kwPPCAsm,
	kwPPCLib,
	kwPPCLink,
	kwR,
	kwRAMDisk,
	kwRenameFlag,
	kwRez,
	kwRSRC,
	kwS,
	kwSCpp,
	kwSymantec,
	kwVersion,
	
	kwAlign,
	kwNoAlign,
	
	kwAttachable,
	kwNoAttachable,
	
	kwAutoBuild,
	kwNoAutoBuild,
	
	kwClassic,
	kwNoClassic,
	
	kwContainer,
	kwNoContainer,
	
	kwCPlusLoad,
	kwNoCPlusLoad,
	
	kwCreateMake,
	kwNoCreateMake,
	
	kwDebug,
	kwNoDebug,
	
	kwDebugMsg,
	kwNoDebugMsg,
	
	kwDeluxe,
	kwNoDeluxe,
	
	kwDrag,
	kwNoDrag,
	
	kwExecute,
	kwNoExecute,
	
	kwExpandEnvVars,
	kwNoExpandEnvVars,
	
	kwFail,
	kwNoFail,
	
	kwFast,
	kwNoFast,
	
	kwGXPrinting,
	kwNoGXPrinting,
	
	kwGXViews,
	kwNoGXViews,
	
	kwInspector,
	kwNoInspector,
	
	kwLinkMap,
	kwNoLinkMap,
	
	kwLinkXRef,
	kwNoLinkXRef,
	
	kwMacApp,
	kwNoMacApp,
	
	kwMALibrary,
	kwNoMALibrary,
	
	kwModelCFM,
	kwNoModelCFM,
	
	kwModelFar,
	kwNoModelFar,
	
	//CW added
	kwModelFarCode,
	kwNoModelFarCode,
	
	//CW added
	kwModelFarData,
	kwNoModelFarData,
	
	kwNames,
	kwNoNames,
	
	kwNeedsColorQD,
	kwNoNeedsColorQD,
	
	kwNeedsFPU,
	kwNoNeedsFPU,
	
	kwNeedsGX,
	kwNoNeedsGX,
	
	kwNeedsMC68020,
	kwNoNeedsMC68020,
	
	kwNeedsMC68030,
	kwNoNeedsMC68030,
	
	kwNeedsMC68040,
	kwNoNeedsMC68040,
	
	kwNeedsSystem7,
	kwNoNeedsSystem7,
	
	kwNeedsSystem7_5,
	kwNoNeedsSystem7_5,
	
	kwNeedsVU,	// Optional Virtual User support.
	kwNoNeedsVU,
	
	kwPerform,
	kwNoPerform,
	
	kwPowerTalk,
	kwNoPowerTalk,
	
	kwPP,
	kwNoPP,
	
	kwRangeCheck,
	kwNoRangeCheck,
	
	kwRun,
	kwNoRun,
	
	kwSave,
	kwNoSave,
	
	kwSeparateObjects,
	kwNoSeparateObjects,
	
	kwSmallSym,
	kwNoSmallSym,
	
	kwStatusOnly,
	kwNoStatusOnly,
	
	kwSym,
	kwNoSym,
	
	kwTemplateViews,
	kwNoTemplateViews,
	
	kwTheDebugger,	// Optional The Debugger support
	kwNoTheDebugger,
	
	kwTT,
	kwNoTT,
	
	kwUserAutoBuild,
	kwNoUserAutoBuild,
	
	kwVerbose,
	kwNoVerbose,
	
	kwWriteTemplateViews,
	kwNoWriteTemplateViews
};

//========================================================================================
// CLASS TMABuildTool
//========================================================================================
class TMABuildTool : public TCPlusTool
{
  public:
	TMABuildTool(int argc = 0, char** argv = NULL);	// constructor
	
	virtual ~TMABuildTool();
		// Destructor
	
	void IMABuildTool(int argc, char** argv);
	
	void GetRenameFlags();
	
	virtual void DoProcessFileArg(const CStr255_AC& arg);	// override
	
	virtual void DoProcessOptionArg(short kw); // override
	
	virtual void DoShowUsage(); // override
	
	virtual void DoStartProgress(); // override
	
	virtual void DoToolAction(); // override
	
	virtual void InstallKeyWords(); // override

	void CatenateToSourceOptionStrings(const CStr255_AC& newText);
	
	void Echo(const CStr255_AC& aStr);
	
	void Execute(const CStr255_AC& aStr);
	
	void SetIE(const CStr255_AC& theVariable,
			   const TStringHandle& theValue);
	
	void SetIE(const CStr255_AC& theVariable,
			   const CStr255_AC& theValue);
	
	void CreateFolder(const CStr255_AC& name);
	
	bool Exists(const CStr255_AC& theFile);

	void Failed(const CStr255_AC& aStr);
	
	void CheckKeywords();

	void GetAutoBuild();
	void AddAutoBuilds();
	void AutoBuildTarget(const CStr255_AC& autoBuildExtension);
	void AddTargets(CStr255_AC& targetNames);
	
	void DoOptionFlags_Part1();

	void DoOptionFlags_Part2();
	
	void OutputOptionFlags();

	void ParseTargetName(CStr255_AC& targetName);
	
//	bool CheckSources(const CStr255_AC& srcVar, const CStr255_AC& srcFile,
//						 const CStr255_AC& objVar, const CStr255_AC& objFile);
//	
	void CreateBuildFolder();
	
	void CreateObjectsFolder();
	
	void DoAllTargets();
	
	void DoATarget_Part1();
	void DoATarget_Part2();
	
	void ExpandEnvironmentVars();
	
	void ExportEnvironmentVars();
	
	void CreateMakeFile();
	void CatenateMake(const CStr255_AC& fileName);
	
	void MakeTarget();
	
	void DoOverrides();
	
	void DoDefinitions();
	
	void DoDependencies();
	
	void SetMakeVariable(const CStr255_AC& theVariable,
						 const TStringHandle& theValue);

	void SetMakeVariable(const CStr255_AC& theVariable,
						 const CStr255_AC& theValue);
	
	void SetMakeVariable(const CStr255_AC& theVariable,
						 bool theValue);
	
	void SetMakeVariable(const CStr255_AC& aString);

	void SetMakeIE(const CStr255_AC& theVariable);
		// Set a make variable to the same as the corresponding integrated
		// environment variable. 
	
	void SetMakeIE(const CStr255_AC& theVariable,
				   const CStr255_AC& theIEVariable);
		// Set a make variable to the same as the corresponding integrated
		// environment variable. 
	
	void OutputMakeVariables();

	void OutputTheEnd();
	
	void SetOutputFile(const CStr255_AC& outputFileName);

protected:
	bool fGetBuildFlags;
	bool fGetFolderName;
	
	bool fMacApp;

	bool fDebug;
	bool fInspector;
	bool fDebugMsg;
	bool fNames;
	bool fRangeCheck;
	bool fPerform;
	bool fSym;
	bool fSmallSym;
	bool fAlign;
	bool fAttachable;
	bool fSeparateObjects;
	CStr255_AC fSeparateObjectsFolder;
	CStr255_AC fBuildFolder;
	CStr255_AC fMAObj;
	bool fMALibrary;
	bool fCPlusLoad;					// Create load/dump files for C++
	bool fCreateMake;

	bool fModelFar;
	bool fModelFarCode;
	bool fModelFarData;

	bool fNeedsColorQD;
	
	// Kind of CPU
	bool fNeedsMC68020;
	bool fNeedsMC68030;
	bool fNeedsMC68040;
	bool fPowerPC;
	
	bool fNeedsFPU;
	
	bool fNeedsSystem7;
	bool fNeedsSystem7_5;
	
	bool fTemplateViews;
	bool fWriteTemplateViews;

	bool fAllProgress;
	bool fExecute;
	bool fAutoBuild;
	bool fUserAutoBuild;
	bool fVerbose;
	bool fNoFail;

	bool fFast;
	bool fLinkMap;
	bool fLinkXRef;
	bool fStatusOnly;
	bool fTimes;
	bool fRunAfterBuild;
	bool fSaveBeforeBuild;
	bool fExpandEnvironmentVars;

	bool fEverExported;				// only need to export once
	
	bool fNeedsVU;					// Virtual User option.

	bool fTheDebugger;				// The Debugger option.
	
	bool fClassic;
	bool fModelCFM;
	bool fRSRC;
	
	bool fFat;
	bool fRAMDisk;
	
	bool fDrag;
	bool fGXPrinting;
	bool fPowerTalk;
	
	short fCompiler;

	CStr255_AC fAppName68K;
	CStr255_AC fRAMDiskName;
	
	TStringHandle fAsmOptions;
	TStringHandle fEchoOptions;
	TStringHandle fLibOptions;
	TStringHandle fLinkOptions;

	TStringHandle fCOptions;

	TStringHandle fMWC68KOptions;
	TStringHandle fMWLink68KOptions;
	TStringHandle fMWCPPCOptions;
	TStringHandle fMWLinkPPCOptions;
	// The following is split out from other compiler options because UMacAppUniversal.cp
	// must be compiled to run on 68000 machines. The CPlus compiler does not have compiler
	// directives to control machine options. It can only be controlled from the command
	// line.
	TStringHandle fCPlusCPUOptions;
	TStringHandle fCPlusOptions;
	TStringHandle fSCOptions;
	
	TStringHandle fPPCAsmOptions;
	TStringHandle fPPCCOptions;
	TStringHandle fMrCOptions;
	TStringHandle fPPCLibOptions;
	TStringHandle fPPCLinkOptions;
	TStringHandle fMakeSymOptions;

	TStringHandle fRunTimeModel;
	
	TStringHandle fMakeOptions;
	TStringHandle fRezOptions;
	TStringHandle fCreatorAndBundleOptions;

	CList_AC fTargStringList;
	TStringHandle fOptionFlags;

	FILE* fOutputFile;
	FILE* fMakeFile;

	CStr255_AC fStartPath;

	TAssociation fRenameFlagsPairs;
	
	CStr255_AC fXLinkMap;
	CStr255_AC fXLinkXRef;
	CStr255_AC fXUAppName_h;
	CStr255_AC fXUAppName_cp;
	CStr255_AC fXUAppName_cp_o;
	CStr255_AC fXMAppName_cp;
	CStr255_AC fXMAppName_cp_o;
	CStr255_AC fXAppName_cp;
	CStr255_AC fXAppName_cp_o;
	
	CStr255_AC fBuildFlags;

	CStr255_AC fAppPath;
	CStr255_AC fAppName;
	CStr255_AC fBuildFolderAppName;
	CStr255_AC fIncApp;
	CStr255_AC fSrcApp;
	CStr255_AC fObjApp;
	CStr255_AC fMAMakeFileExtension;
	bool fAutomake;
	bool fAutorez;
	bool fAnyCPlus;

	CStr255_AC fDirectorySeparator;	// : or / as required by host filesystem }
};



//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

static bool IEgetenv(const char* envName, CStr255_AC& envValue);
static void SubstituteInString(CStr255_AC& s);
static bool FindVariable(const CStr255_AC& s, short& varBegin, short& varLength);

static void CatenateFile(FILE* fromFile, FILE* toFile);
static void CatenateFile(const CStr255_AC& fileName, FILE* toFile);
static unsigned long GetFileModDate(const CStr255_AC& theFile);
static void PathNameFromDirID(long dirID, short vRefNum, CStr255_AC& fullPathName);

//----------------------------------------------------------------------------------------
// IEgetenv: 
//----------------------------------------------------------------------------------------
// our own version to map to the Pascal routine with the same name
bool IEgetenv(const char* envName, CStr255_AC& envValue)
{
	envValue.Empty();	// initialize envValue
	char * p = getenv(envName);	// try and get the variable
	if (p)
		envValue = p;		// copy value if it exists

	return p != NULL;
}

//----------------------------------------------------------------------------------------
// SubstituteInString: 
//----------------------------------------------------------------------------------------
void SubstituteInString(CStr255_AC& s)
{
	short varBegin = 0;
	short varLength;
	while (FindVariable(s, varBegin, varLength))
	{
		CStr255_AC varName = s.Copy(varBegin + 1, varLength - 2);
		CStr255_AC value;
		if (IEgetenv(varName, value))
		{
			s.Delete(varBegin, varLength);
			s.Insert(value, varBegin);
			varBegin = 0;
		}
		else
			varBegin += varLength - 1;	// skip unknown variable
	}
}

//----------------------------------------------------------------------------------------
// FindVariable: 
//----------------------------------------------------------------------------------------
bool FindVariable(const CStr255_AC& s, short& varBegin, short& varLength)
{
	// Returns the position and size of an embedded variable, including the {}
	
	bool found = false;
	varLength = 0;
	short sLength = s.Length();
	for (short i = varBegin + 1; i <= sLength; ++i)
	{
		if (s[i] == '{')
		{
			varBegin = i;
		}
		else if ((varBegin > 0) && (s[i]  == '}'))
		{
			varLength = i - varBegin + 1;
			found = true;
			break;
		}
	}
	return found;
}

//----------------------------------------------------------------------------------------
// CatenateFile: 
//----------------------------------------------------------------------------------------
void CatenateFile(FILE* fromFile, FILE* toFile)
{
	char* buffer = (char*)operator new(BUFSIZ);
	::ThrowIfNULL_AC(buffer);
	while (!feof(fromFile))
	{
		size_t n = fread(buffer, 1, BUFSIZ, fromFile);
		if (n > 0)
			fwrite(buffer, 1, n, toFile);
	}
	operator delete(buffer);
}

//----------------------------------------------------------------------------------------
// CatenateFile: 
//----------------------------------------------------------------------------------------
void CatenateFile(const CStr255_AC& fileName, FILE* toFile)
{
	FILE* fromFile = fopen(fileName, "r");
	CatenateFile(fromFile, toFile);
	fclose(fromFile);
}

//----------------------------------------------------------------------------------------
// GetFileModDate
//----------------------------------------------------------------------------------------
unsigned long GetFileModDate(const CStr255_AC& theFile)
{
	unsigned long modDate = 0;

	CStr255_AC fileName = theFile;
	
	// Return true if the file or directory exists
	CInfoPBRec aCInfoPBRec;

	aCInfoPBRec.hFileInfo.ioCompletion = NULL;
	aCInfoPBRec.hFileInfo.ioNamePtr = (StringPtr)fileName;
	aCInfoPBRec.hFileInfo.ioVRefNum = 0;
	aCInfoPBRec.hFileInfo.ioFRefNum = 0;
	aCInfoPBRec.hFileInfo.ioFDirIndex = 0;
	aCInfoPBRec.hFileInfo.ioDirID = 0;

	if ((PBGetCatInfoSync(&aCInfoPBRec) == noErr) && (aCInfoPBRec.hFileInfo.ioResult == noErr))
		modDate = aCInfoPBRec.hFileInfo.ioFlMdDat;
	
	return modDate;
}

//========================================================================================
// CLASS TMABuildTool
//========================================================================================
#undef Inherited
#define Inherited TCPlusTool

//----------------------------------------------------------------------------------------
// TMABuildTool destructor
//----------------------------------------------------------------------------------------
TMABuildTool::~TMABuildTool()
{
}

//----------------------------------------------------------------------------------------
// TMABuildTool::TMABuildTool: 
//----------------------------------------------------------------------------------------
TMABuildTool::TMABuildTool(int argc, char** argv) :
	TCPlusTool(argc, argv),

	// setup the default options
	fGetBuildFlags(false),
	fGetFolderName(false),
	
	fMacApp(true),
	fDebug(false),
	fInspector(false),
	fDebugMsg(false),
	fNames(false),
	fRangeCheck(false),
	fPerform(false),
	fSym(false),
	fSmallSym(false),
	fAlign(true),
	fAttachable(false),
	fSeparateObjects(true),

	fMALibrary(false),
	fCPlusLoad(true),
	fCreateMake(true),
	
	fModelFar(false),
	fModelFarCode(false),
	fModelFarData(false),
		
	fNeedsColorQD(true),
	fNeedsMC68020(true),
	fNeedsMC68030(false),
	fNeedsMC68040(false),
	fPowerPC(false),
	fNeedsFPU(false),
	fNeedsSystem7(true),
	fNeedsSystem7_5(false),

	fTemplateViews(true),
	fWriteTemplateViews(false),
	fAllProgress(false),
	fExecute(true),
	fAutoBuild(false),
	fUserAutoBuild(true),
	fVerbose(false),
	fNoFail(false),
	fFast(false),
	fLinkMap(false),
	fLinkXRef(false),
	fStatusOnly(false),
	fTimes(false),
	fRunAfterBuild(false),
	fSaveBeforeBuild(false),
	fExpandEnvironmentVars(false),
	fEverExported(false),
	fNeedsVU(false),
	fTheDebugger(false),
	
	fCompiler(kwUnspecified),

	fClassic(false),
	fModelCFM(false),
	fRSRC(false),
	fFat(false),
	fRAMDisk(false),
	
	fDrag(false),
	fGXPrinting(false),
	fPowerTalk(false),
		
	fOutputFile(stdout)
{
	// Try to get the appropriate separator character for directories
	if (!IEgetenv("MADirectorySeparator", fDirectorySeparator))
		fDirectorySeparator = ":";
	
	GetRenameFlags();
}

//----------------------------------------------------------------------------------------
// TMABuildTool::GetRenameFlags: 
//----------------------------------------------------------------------------------------
void TMABuildTool::GetRenameFlags()
{
	CStr255_AC uglyName;
	CStr255_AC niceName;
	
	Handle renameFlags = Get1Resource('STR#', 1000);
	if (renameFlags)
	{
		HLock(renameFlags);
		short count = **reinterpret_cast<short **>(renameFlags);
		count >>= 1;	// divide by 2 to get number of pairs
		StringPtr s = reinterpret_cast<StringPtr>(*renameFlags + sizeof(short));
		while (--count >= 0)
		{
			uglyName = s;
			s += *s + 1;
			
			niceName = s;
			s += *s + 1;
			
			fRenameFlagsPairs.InsertEntry(uglyName, niceName);
		}
		HUnlock(renameFlags);
		ReleaseResource(renameFlags);
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoShowUsage: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoShowUsage()
{
	// output each string in the stringlist
	for (short i = 1;;++i)
	{
		CStr255_AC theHelpString(kHelpStr, i);
		if (!theHelpString.Length())
			break;

		fprintf(stdout, "%s\n", (const char*)(theHelpString));
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoProcessFileArg: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoProcessFileArg(const CStr255_AC& arg)
{
	// Special case for MacApp as target for backward compatibility (until post 2.0)
	if (::EqualString(arg, (StringPtr) "\pMacApp", false, true))
		fAutoBuild = true;
	else
		fTargStringList.InsertLast(TH_new TStringHandle(arg));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoStartProgress: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoStartProgress()
{
	// Don't show the copyright information if all we're getting is the folder name.
	if (fProgress && !fGetFolderName)
	{
		CStr255_AC theDateTimeString;
	
		IUTimeString(fStartDateTime, true, theDateTimeString);
		fprintf(stdout, "# MABuildTool         Start: %s",
				(const char*)(theDateTimeString));
	
		IUDateString(fStartDateTime, shortDate, theDateTimeString);
		fprintf(stdout," %s\n",(const char*)(theDateTimeString));
		fprintf(stdout, "\n# %s\n\n", MA_COPYRIGHT);
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::CatenateToSourceOptionStrings: 
//----------------------------------------------------------------------------------------
void TMABuildTool::CatenateToSourceOptionStrings(const CStr255_AC& newText)
{
	fAsmOptions.Catenate(newText);
	fCOptions.Catenate(newText);

	fMWC68KOptions.Catenate(newText);
	fMWCPPCOptions.Catenate(newText);
	
	fCPlusOptions.Catenate(newText);
	fSCOptions.Catenate(newText);
	fPPCAsmOptions.Catenate(newText);
	fPPCCOptions.Catenate(newText);
	fMrCOptions.Catenate(newText);
	fRezOptions.Catenate(newText);
	
	//fMakeOptions.Catenate(newText);	// Don't include Make
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoProcessOptionArg: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoProcessOptionArg(short kw)
{
	CStr255_AC theNextArg;
	
	switch (kw)
	{
		// "    -Asm option            # Pass through options to Assembler";
		case kwAsm:
			fAsmOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fAsmOptions.Catenate(theNextArg);
			break;

		// "    -BuildFolder           # Specify root object folder for this build";
		case kwBuildFolder:
			GetNextArg(theNextArg);
			fBuildFolder = theNextArg;
			break;
				
		// "    -BuildFlags             # Get the build flags for these options";
		case kwBuildFlags:
			fGetBuildFlags = true;
			fProgress = false;
			break;
				
		// "    -C option              # Pass through options to C compiler";
		case kwC:
			fCOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fCOptions.Catenate(theNextArg);
			break;
		
		// "    -CP compiler            # Specify the C++ compiler to use:";
		// "                            # MrC, MWC68K, MWCPPC, SCpp";
		case kwCP:
			GetNextArg(theNextArg);
			{
				if (theNextArg[1] == '-')
				{
					SyntaxError("'" + theNextArg + "' < missing compiler name > ");
				}
				else
				{
					bool found = false;
					short akw;
					if (LookupKeyword(theNextArg, akw))
					{
						if (akw == kwSymantec)
							akw = kwSCpp;
						
						if ((akw == kwMrC) || (akw == kwMWC68K) ||
							(akw == kwMWCPPC) || (akw == kwSCpp))
						{
							fCompiler = akw;
							found = true;
						}
					}
					
					if (!found)
						SyntaxError("'" + theNextArg + "' < not a compiler name > ");
				}
			}
			break;

		// "    -CPlusPlus             # Pass through generic C++ compiler options";
		case kwCPlusPlus:
			GetNextArg(theNextArg);
			
			fCPlusOptions.Catenate(" ");
			fCPlusOptions.Catenate(theNextArg);
			
			fSCOptions.Catenate(" ");
			fSCOptions.Catenate(theNextArg);
			
			fPPCCOptions.Catenate(" ");
			fPPCCOptions.Catenate(theNextArg);
			
			fMrCOptions.Catenate(" ");
			fMrCOptions.Catenate(theNextArg);
			break;
		
		// "    -CreatorAndBundle option # Pass through options to CreatorAndBundle tool";
		case kwCreatorAndBundle:
			fCreatorAndBundleOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fCreatorAndBundleOptions.Catenate(theNextArg);
			break;

		// "    -d name=(true|false)    # Set compile time variable name in all compilers";
		case kwd:
			GetNextArg(theNextArg);
			
			CatenateToSourceOptionStrings(" -d ");
			CatenateToSourceOptionStrings(theNextArg);
			break;

		// "    -E                      # Pass through -E to Make";
		case kwE:
			fMakeOptions.Catenate(" -e");
			break;
		
		// "    -Fat appname            # Build a fat binary by adding the code and resources";
		// "                            # from a 68K application file";
		case kwFat:
			fFat = true;
			GetNextArg(fAppName68K);
			break;

		// "    -FolderName             # Get the folder name for these options";
		case kwFolderName:
			fGetFolderName = true;
			fProgress = false;
			break;
				
		// "    -Help                   # Prints this list of options to stdout";
		
		// "    -Lib option            # Pass through options to Librarian";
		case kwLib:
			fLibOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fLibOptions.Catenate(theNextArg);
			break;

		// "    -Link option           # Pass through options to Linker";
		case kwLink:
			fLinkOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fLinkOptions.Catenate(theNextArg);
			break;

		// "    -Make option           # Pass through options to Make";
		case kwMake:
			fMakeOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fMakeOptions.Catenate(theNextArg);
			break;

		// "    -MakeSYM option        # Pass through options to MakeSYM";
		case kwMakeSym:
			fMakeSymOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fMakeSymOptions.Catenate(theNextArg);
			break;

		// "    -MrC option            # Pass through options to Macintosh RISC C/C++ compiler";
		case kwMrC:
			fMrCOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fMrCOptions.Catenate(theNextArg);
			break;

		// "    -MWC68K option         # Pass through options to Metrowerks C/C++ 68K compiler";
		case kwMWC68K:
			fMWC68KOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fMWC68KOptions.Catenate(theNextArg);
			break;

		// "    -MWLink68K option         # Pass through options to Metrowerks C/C++ 68K linker";
		case kwMWLink68K:
			fMWLink68KOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fMWLink68KOptions.Catenate(theNextArg);
			break;

		// "    -MWCPPC option         # Pass through options to Metrowerks C/C++ PPC compiler";
		case kwMWCPPC:
			fMWCPPCOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fMWCPPCOptions.Catenate(theNextArg);
			break;

		// "    -MWLinkPPC option         # Pass through options to Metrowerks C/C++ PPC Linker";
		case kwMWLinkPPC:
			fMWLinkPPCOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fMWLinkPPCOptions.Catenate(theNextArg);
			break;

		// "    -O outputFileName       # Specify where the mabuild results ought to go.
		case kwOutputFile:
			GetNextArg(theNextArg);
			SetOutputFile(theNextArg);
			break;
		
		// "    -PPCAsm option         # Pass through options to PPC Assembler";
		case kwPPCAsm:
			fPPCAsmOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fPPCAsmOptions.Catenate(theNextArg);
			break;

		// "    -PPCLib option         # Pass through options to PPC Librarian";
		case kwPPCLib:
			fPPCLibOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fPPCLibOptions.Catenate(theNextArg);
			break;

		// "    -PPCLink option        # Pass through options to PPC Linker";
		case kwPPCLink:
			fPPCLinkOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fPPCLinkOptions.Catenate(theNextArg);
			break;

		// "    -R                      # Pass through -R to Make and write results of Make to stdout";
		case kwR:
			fExecute = false;
			fStatusOnly = true;
			fMakeOptions.Catenate(" -r");
			break;

		// "    -RAMDisk name           # Use a RAM Disk for pre-compiled headers";
		case kwRAMDisk:
			fRAMDisk = true;
			GetNextArg(fRAMDiskName);
			break;

		// "    -RenameFlag oldname newname # Rename separate object folders";
		case kwRenameFlag:
			{
				CStr255_AC anotherArg;
				
				GetNextArg(theNextArg);
				GetNextArg(anotherArg);
				
				fRenameFlagsPairs.InsertEntry(theNextArg, anotherArg);
			}
			break;

		// "    -Rez option            # Pass through options to Resource compiler";
		case kwRez:
			fRezOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fRezOptions.Catenate(theNextArg);
			break;

		// "    -RSRC                   # Build only an application .rsrc file";
		case kwRSRC:
			fRSRC = (kw == kwRSRC);
			break;

		// "    -S                      # Pass through -S to Make and write results of Make to stdout";
		case kwS:
			fExecute = false;
			fStatusOnly = true;
			fMakeOptions.Catenate(" -s");
			break;

		// "    -SCpp option           # Pass through options to the Symantec C++ compiler";
		case kwSCpp:
			fSCOptions.Catenate(" ");
			GetNextArg(theNextArg);
			fSCOptions.Catenate(theNextArg);
			break;

		// "    -Symantec               # Equivalent to -CP SCpp (for compatibility)";
		case kwSymantec:
			fCompiler = kwSCpp;
			break;

		// "    -Version                # Display the tool version";
		case kwVersion:
			{
				fprintf(stdout,"\n# MABuildTool - Release %s", MA_SHORT_VERSION);
#if qPowerPC
				fprintf(stdout," - PPC Mac OS version");
#elif q68K
				fprintf(stdout," - 68K Mac OS version");
#else
				fprintf(stdout," - some other platform version");
#endif
				fprintf(stdout,"\n");
			}
			break;

		// "    -[No]Align              # Longword align all code and data for speed";
		case kwAlign:
		case kwNoAlign:
			fAlign = (kw == kwAlign);
			break;
			
		// "    -[No]Attachable         # Support for attaching scripts to objects";
		case kwAttachable:
		case kwNoAttachable:
			fAttachable = (kw == kwAttachable);
			break;
			
		// "    -[No]AutoBuild          # Auto-build the MacApp libraries (and any others)";
		case kwAutoBuild:
		case kwNoAutoBuild:
			fAutoBuild = (kw == kwAutoBuild);
			break;

		// "    -[No]Classic            # Build for classic 68K runtime";
		case kwClassic:
		case kwNoClassic:
			fClassic = (kw == kwClassic);
			if (fClassic)
				fModelCFM = false;
			break;

		// "    -[No]CPlusLoad          # Make the C++ compiler use load/dump files";
		case kwCPlusLoad:
		case kwNoCPlusLoad:
			fCPlusLoad = (kw == kwCPlusLoad);
			break;

		// "    -[No]CreateMake         # Create a make file, even if one already exists";
		case kwCreateMake:
		case kwNoCreateMake:
			fCreateMake = (kw == kwCreateMake);
			break;

		// "    -[No]Debug              # Turn on debugging and install debugging gear";
		case kwDebug:
		case kwNoDebug:
			fPerform = fDebug = fDebugMsg  = fRangeCheck = fNames = (kw == kwDebug);
			
			// Default is to also turn on source debugging symbols,
			// but it can be turned off
			if (fDebug)
				fSym = true;
			break;
				
		// "    -[No]DebugMsg           # Turn on extra debugging messages";
		case kwDebugMsg:
		case kwNoDebugMsg:
			fDebugMsg = (kw == kwDebugMsg);
			break;

		// "    -[No]Deluxe             # Include support for the coolest features";
		case kwDeluxe:
		case kwNoDeluxe:
			fAttachable = fDrag = (kw == kwDeluxe);
			break;

		// "    -[No]Drag               # Include support for Drag & Drop";
		case kwDrag:
		case kwNoDrag:
			fDrag = (kw == kwDrag);
			break;

		// "    -[No]Execute            # Execute the results of Make";
		case kwExecute:
		case kwNoExecute:
			fExecute = (kw == kwExecute);
			break;
			
		// "    -[No]ExpandEnvVars      # Expand environment vars in AppName.MakeIt";
		case kwExpandEnvVars:
		case kwNoExpandEnvVars:
			fExpandEnvironmentVars = (kw == kwExpandEnvVars);
			break;

		// "    -[No]Fail               # Stop executing after first failure";
		case kwNoFail:
		case kwFail:
			fNoFail = (kw == kwNoFail);
			break;
		
		// "    -[No]Fast               # Compile files in one batch (faster)";
		case kwNoFast:
		case kwFast:
			fFast = (kw == kwFast);
			break;
		
		// "    -[No]GXPrinting         # Include support for GX Printing";
		case kwGXPrinting:
		case kwNoGXPrinting:
			fGXPrinting = (kw == kwGXPrinting);
			break;

		// "    -[No]Inspector          # Include hooks for the inspector";
		case kwInspector:
		case kwNoInspector:
			fInspector = (kw == kwInspector);
			break;

		// "    -[No]LinkMap            # Generate a link map in AppName.map";
		case kwLinkMap:
		case kwNoLinkMap:
			fLinkMap = (kw == kwLinkMap);
			break;
			
		// "    -[No]LinkXRef           # Generate a linker cross reference in AppName.xref";
		case kwLinkXRef:
		case kwNoLinkXRef:
			fLinkXRef = (kw == kwLinkXRef);
			break;

		// "    -[No]MacApp             # Build for use with MacApp";
		case kwMacApp:
		case kwNoMacApp:
			fMacApp = (kw == kwMacApp);
			break;

		// "    -[No]MALibrary          # Use library, not object files";
		case kwMALibrary:
		case kwNoMALibrary:
			fMALibrary = (kw == kwMALibrary);
			break;

		// "    -[No]ModelCFM           # Build for model CFM-68K support";
		case kwModelCFM:
		case kwNoModelCFM:
			fModelCFM = (kw == kwModelCFM);
			if (fModelCFM)
			{
				fClassic = false;
				fModelFar = false;
//CW added
				fModelFarCode = false;
				fModelFarData = false;
			}
			break;

		// "    -[No]ModelFar           # Build for model far support";
		case kwModelFar:
		case kwNoModelFar:
			fModelFar = (kw == kwModelFar);
//CW added
			fModelFarCode = (kw == kwModelFar);
			fModelFarData = (kw == kwModelFar);
			break;
				
//CW added
		// "    -[No]ModelFarCode           # Build for model far code support";
		case kwModelFarCode:
		case kwNoModelFarCode:
			fModelFarCode = (kw == kwModelFarCode);
			break;
				
//CW added
		// "    -[No]ModelFarData           # Build for model far data support";
		case kwModelFarData:
		case kwNoModelFarData:
			fModelFarData = (kw == kwModelFarData);
			break;
				
		// "    -[No]Names              # Include embedded debugging symbols";
		case kwNames:
		case kwNoNames:
			fNames = (kw == kwNames);
			
			// Default is to also turn on source debugging symbols,
			// but it can be turned off
			if (fNames)
				fSym = true;
			break;

		// "    -[No]NeedsColorQD       # Build to require Color QuickDraw";
		case kwNeedsColorQD:
		case kwNoNeedsColorQD:
			fNeedsColorQD = (kw == kwNeedsColorQD);
			break;

		// "    -[No]NeedsFPU           # Build to require a Floating point unit";
		case kwNeedsFPU:
		case kwNoNeedsFPU:
			fNeedsFPU = (kw == kwNeedsFPU);
			break;

		// "    -[No]NeedsMC68020       # Build to require a 68020";
		case kwNeedsMC68020:
		case kwNoNeedsMC68020:
			fNeedsMC68020 = (kw == kwNeedsMC68020);
			if (fNeedsMC68020)
			{
				fPowerPC = false;
			}
			break;

		// "    -[No]NeedsMC68030       # Build to require a 68030";
		case kwNeedsMC68030:
		case kwNoNeedsMC68030:
			fNeedsMC68030 = (kw == kwNeedsMC68030);
			if (fNeedsMC68030)
			{
				fPowerPC = false;
			}
			break;

		// "    -[No]NeedsMC68040       # Build to require a 68040";
		case kwNeedsMC68040:
		case kwNoNeedsMC68040:
			fNeedsMC68040 = (kw == kwNeedsMC68040);
			if (fNeedsMC68040)
			{
				fPowerPC = false;
			}
			break;

		// "    -[No]NeedsSystem7       # Build to require System 7.0 or better";
		case kwNeedsSystem7:
		case kwNoNeedsSystem7:
			fNeedsSystem7 = (kw == kwNeedsSystem7);
			break;

		// "    -[No]NeedsSystem7       # Build to require System 7.0 or better";
		case kwNeedsSystem7_5:
		case kwNoNeedsSystem7_5:
			fNeedsSystem7_5 = (kw == kwNeedsSystem7_5);
			break;

		// "    -[No]NeedsVU            # Build for VU compatibility";
		case kwNeedsVU:
		case kwNoNeedsVU:
			fNeedsVU = (kw == kwNeedsVU);
			break;

		// "    -[No]P                  # Show progress for the MABuild system";
		
		// "    -[No]Perform            # Include performance monitor code";
		case kwPerform:
		case kwNoPerform:
			fPerform = (kw == kwPerform);
			break;

		// "    -[No]PowerTalk          # Include support for PowerTalk Mailers";
		case kwPowerTalk:
		case kwNoPowerTalk:
			fPowerTalk = (kw == kwPowerTalk);
			break;

		// "    -[No]PP                 # Have all invoked tools show progress too";
		case kwPP:
		case kwNoPP:
			fAllProgress = fProgress = (kw == kwPP);
			break;

		// "    -[No]RangeCheck         # Force range/overflow checking";
		case kwRangeCheck:
		case kwNoRangeCheck:
			fRangeCheck = (kw == kwRangeCheck);
			break;

		// "    -[No]Run                # Run after build";
		case kwRun:
		case kwNoRun:
			fRunAfterBuild = (kw == kwRun);
			break;

		// "    -[No]Save               # Auto-save all open windows before starting build";
		case kwSave:
		case kwNoSave:
			fSaveBeforeBuild = (kw == kwSave);
			break;

		// "    -[No]SeparateObjects    # Keep separate object folders for each build variation";
		case kwSeparateObjects:
		case kwNoSeparateObjects:
			fSeparateObjects = (kw == kwSeparateObjects);
			break;

		// "    -[No]SmallSym           # Try to generate smaller symbol files (slow!)";
		case kwSmallSym:
		case kwNoSmallSym:
			fSmallSym = (kw == kwSmallSym);
			fSym = true;
			break;
			
		// "    -[No]StatusOnly         # Write results of Make to stdout";
		case kwStatusOnly:
		case kwNoStatusOnly:
			fExecute = (kw == kwNoStatusOnly);
			fStatusOnly = (kw == kwStatusOnly);
			break;

		// "    -[No]Sym                # Generate SADE-compatible symbol information";
		case kwSym:
		case kwNoSym:
			fSym = (kw == kwSym);
			if (!fSym)
				fSmallSym = false;
			break;
		
		// "    -[No]T                  # Show elapsed time for the MABuild system";
		
		// "    -[No]TemplateViews      # Include code to create views from templates";
		case kwTemplateViews:
		case kwNoTemplateViews:
			fTemplateViews = (kw == kwTemplateViews);
			break;

		// "    -[No]TT                 # Have all invoked tools show elapsed time too";
		case kwTT:
		case kwNoTT:
			fTimes = fTime = (kw == kwTT);
			break;

		// "    -[No]UserAutoBuild      # Auto-build user-specified libraries {MAUserAutoBuild}";
		case kwUserAutoBuild:
		case kwNoUserAutoBuild:
			fUserAutoBuild = (kw == kwUserAutoBuild);
			break;

		// "    -[No]Verbose            # Show all kinds of progress for the MABuild system";
		case kwVerbose:
		case kwNoVerbose:
			fVerbose = (kw == kwVerbose);
			fAllProgress = fProgress = fTimes = fTime = fVerbose;
			break;

		// "    -[No]WriteTemplateViews # Include code to write view resources"
		case kwWriteTemplateViews:
		case kwNoWriteTemplateViews:
			fWriteTemplateViews = (kw == kwWriteTemplateViews);
			break;

		case kwTheDebugger:
		case kwNoTheDebugger:
			fTheDebugger = (kw == kwTheDebugger);
			break;
		
		default:
			Inherited::DoProcessOptionArg(kw);
			break;
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::InstallKeyWords: 
//----------------------------------------------------------------------------------------
void TMABuildTool::InstallKeyWords()
{
	KeyWordStruct keywords [] =
	{
		{"Asm", kwAsm},
		{"BuildFlags", kwBuildFlags},
		{"BuildFolder", kwBuildFolder},
		{"C", kwC},
		{"CP", kwCP},
		{"CPlusPlus", kwCPlusPlus},
		{"CreatorAndBundle", kwCreatorAndBundle},
		{"d", kwd},
		{"E", kwE},
		{"Fat", kwFat},
		{"FolderName", kwFolderName},
		{"Lib", kwLib},
		{"Link", kwLink},
		{"Make", kwMake},
		{"MakeSym", kwMakeSym},
		{"MrC", kwMrC},
		{"MWC68K", kwMWC68K},
		{"MWCPPC", kwMWCPPC},

		{"MWLink68K", kwMWLink68K},
		{"MWLinkPPC", kwMWLinkPPC},

		{"O", kwOutputFile},
				
		{"PPCAsm", kwPPCAsm},
		{"PPCLib", kwPPCLib},
		{"PPCLink", kwPPCLink},
		{"R", kwR},
		{"RAMDisk", kwRAMDisk},
		{"RenameFlag", kwRenameFlag},
		{"Rez", kwRez},
		{"RSRC", kwRSRC},
		{"S", kwS},
		{"SCpp", kwSCpp},
		{"Symantec", kwSymantec},
		{"Version", kwVersion},

		{"Align", kwAlign},						{"NoAlign", kwNoAlign},
		{"Attachable", kwAttachable},			{"NoAttachable", kwNoAttachable},
		{"AutoBuild", kwAutoBuild},				{"NoAutoBuild", kwNoAutoBuild},
		{"Classic", kwClassic},					{"NoClassic", kwNoClassic},
		{"Container", kwContainer},				{"NoContainer", kwNoContainer},
		{"CPlusLoad", kwCPlusLoad},				{"NoCPlusLoad", kwNoCPlusLoad},
		{"CreateMake", kwCreateMake},			{"NoCreateMake", kwNoCreateMake},
		{"Debug", kwDebug},						{"NoDebug", kwNoDebug},
		{"DebugMsg", kwDebugMsg},				{"NoDebugMsg", kwNoDebugMsg},
		{"Deluxe", kwDeluxe},					{"NoDeluxe", kwNoDeluxe},
		{"Drag", kwDrag},						{"NoDrag", kwNoDrag},
		{"Execute", kwExecute},					{"NoExecute", kwNoExecute},
		{"ExpandEnvVars", kwExpandEnvVars},		{"NoExpandEnvVars", kwNoExpandEnvVars},
		{"Fail", kwFail},						{"NoFail", kwNoFail},
		{"Fast", kwFast},						{"NoFast", kwNoFast},
		{"GXPrinting", kwGXPrinting},			{"NoGXPrinting", kwNoGXPrinting},
		{"GXViews", kwGXViews},					{"NoGXViews", kwNoGXViews},
		{"Inspector", kwInspector},				{"NoInspector", kwNoInspector},
		{"LinkMap", kwLinkMap},					{"NoLinkMap", kwNoLinkMap},
		{"LinkXRef", kwLinkXRef},				{"NoLinkXRef", kwNoLinkXRef},
		{"MacApp", kwMacApp},					{"NoMacApp", kwNoMacApp},
		{"MALibrary", kwMALibrary},				{"NoMALibrary", kwNoMALibrary},
		{"ModelCFM", kwModelCFM},				{"NoModelCFM", kwNoModelCFM},
		{"ModelFar", kwModelFar},				{"NoModelFar", kwNoModelFar},

		{"ModelFarCode", kwModelFarCode},		{"NoModelFarCode", kwNoModelFarCode},
		{"ModelFarData", kwModelFarData},		{"NoModelFarData", kwNoModelFarData},

		{"Names", kwNames},						{"NoNames", kwNoNames},
		{"NeedsColorQD", kwNeedsColorQD},		{"NoNeedsColorQD", kwNoNeedsColorQD},
		{"NeedsFPU", kwNeedsFPU},				{"NoNeedsFPU", kwNoNeedsFPU},
		{"NeedsGX", kwNeedsGX},					{"NoNeedsGX", kwNoNeedsGX},
		{"NeedsMC68020", kwNeedsMC68020},		{"NoNeedsMC68020", kwNoNeedsMC68020},
		{"NeedsMC68030", kwNeedsMC68030},		{"NoNeedsMC68030", kwNoNeedsMC68030},
		{"NeedsMC68040", kwNeedsMC68040},		{"NoNeedsMC68040", kwNoNeedsMC68040},
		{"NeedsSystem7", kwNeedsSystem7},		{"NoNeedsSystem7", kwNoNeedsSystem7},
		{"NeedsSystem7_5", kwNeedsSystem7_5},	{"NoNeedsSystem7_5", kwNoNeedsSystem7_5},
		{"NeedsVU", kwNeedsVU},					{"NoNeedsVU", kwNoNeedsVU},
		{"Perform", kwPerform},					{"NoPerform", kwNoPerform},
		{"PowerTalk", kwPowerTalk},				{"NoPowerTalk", kwNoPowerTalk},
		{"PP", kwPP},							{"NoPP", kwNoPP},
		{"RangeCheck", kwRangeCheck},			{"NoRangeCheck", kwNoRangeCheck},
		{"Run", kwRun},							{"NoRun", kwNoRun},
		{"Save", kwSave},						{"NoSave", kwNoSave},
		{"SeparateObjects", kwSeparateObjects},	{"NoSeparateObjects", kwNoSeparateObjects},
		{"SmallSym", kwSmallSym},				{"NoSmallSym", kwNoSmallSym},
		{"StatusOnly", kwStatusOnly},			{"NoStatusOnly", kwNoStatusOnly},
		{"Sym", kwSym},							{"NoSym", kwNoSym},
		{"TemplateViews", kwTemplateViews},		{"NoTemplateViews", kwNoTemplateViews},
		{"TheDebugger", kwTheDebugger},			{"NoTheDebugger", kwNoTheDebugger},
		{"TT", kwTT},							{"NoTT", kwNoTT},
		{"UserAutoBuild", kwUserAutoBuild},		{"NoUserAutoBuild", kwNoUserAutoBuild},
		{"Verbose", kwVerbose},					{"NoVerbose", kwNoVerbose},
		{"WriteTemplateViews", kwWriteTemplateViews},	{"NoWriteTemplateViews", kwNoWriteTemplateViews}
	};
	
	Inherited::InstallKeyWords();
	
	for (short i = 0; i < sizeof(keywords) / sizeof(KeyWordStruct); ++i)
		InstallKeyWord(keywords[i].keyword, keywords[i].keyvalue);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::Execute: 
//----------------------------------------------------------------------------------------
void TMABuildTool::Execute(const CStr255_AC& aStr)
{
	fprintf(fOutputFile,"%s\n", (const char*)(aStr));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetIE: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetIE(const CStr255_AC& theVariable,
						 const TStringHandle& theValue)
{
	fprintf(fOutputFile, "SET %s \"", (const char*)(theVariable));
	theValue.WriteToFile(fOutputFile);
	fprintf(fOutputFile, "\"\n");
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetIE: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetIE(const CStr255_AC& theVariable,
						 const CStr255_AC& theValue)
{
	fprintf(fOutputFile, "SET %s \"%s\"\n", (const char*)(theVariable), (const char*)(theValue));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::Echo: 
//----------------------------------------------------------------------------------------
void TMABuildTool::Echo(const CStr255_AC& aStr)
{
	fprintf(fOutputFile,"{MAEcho} %s\n",(const char*)(aStr));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::CreateFolder: 
//----------------------------------------------------------------------------------------
void TMABuildTool::CreateFolder(const CStr255_AC& name)
{
	long dirID;
	if ((name.Length() > 0) && !Exists(name) && DirCreate(0, 0, name, &dirID) != noErr)
		Failed("Unable to create folder: " + name);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::Exists: 
//----------------------------------------------------------------------------------------
bool TMABuildTool::Exists(const CStr255_AC& theFile)
{
	CStr255_AC fileName = theFile;
	
	// Return true if the file or directory exists
	CInfoPBRec aCInfoPBRec;

	aCInfoPBRec.hFileInfo.ioCompletion = NULL;
	aCInfoPBRec.hFileInfo.ioNamePtr = (StringPtr)&fileName;
	aCInfoPBRec.hFileInfo.ioVRefNum = 0;
	aCInfoPBRec.hFileInfo.ioFRefNum = 0;
	aCInfoPBRec.hFileInfo.ioFDirIndex = 0;
	aCInfoPBRec.hFileInfo.ioDirID = 0;

	return (PBGetCatInfoSync(&aCInfoPBRec) == noErr) && (aCInfoPBRec.hFileInfo.ioResult == noErr);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::Failed: 
//----------------------------------------------------------------------------------------
void TMABuildTool::Failed(const CStr255_AC& aStr)
{
	Echo("MABuild: " + aStr);
	Echo("MABuild of {XAppName} failed: `DATE`");
	Execute("{MAFailed}");

	if (fNoFail)
		SetIE("XRunStatus", "1");
	else
		Execute("EXIT 1");
}

//----------------------------------------------------------------------------------------
// PathNameFromDirID: 
//----------------------------------------------------------------------------------------
void PathNameFromDirID(long dirID, short vRefNum, CStr255_AC& fullPathName)
{
	const short fsRtDir = 2;
	CInfoPBRec block;
	CStr255_AC directoryName;
	OSErr err;

	fullPathName.Empty();
	block.hFileInfo.ioNamePtr = (StringPtr)&directoryName;
	block.dirInfo.ioDrParID = dirID;

	do
	{
		block.hFileInfo.ioVRefNum = vRefNum;
		block.hFileInfo.ioFDirIndex = - 1;
		block.dirInfo.ioDrDirID = block.dirInfo.ioDrParID;
		err = PBGetCatInfoSync(&block);
	
		directoryName = directoryName + ":";
		
		fullPathName = directoryName + fullPathName;
	} while (block.dirInfo.ioDrDirID != fsRtDir);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::CheckKeywords: 
//----------------------------------------------------------------------------------------
void TMABuildTool::CheckKeywords()
{
	// Resolve matrix of options.
	
	if (fNeedsSystem7_5)
		fNeedsSystem7 = true;

	if (fModelCFM)
	{
		if (fCompiler == kwUnspecified)
			fCompiler = kwSCpp;
		
		fModelFar = false;

		fModelFarCode = false;
		fModelFarData = false;

		fNeedsMC68030 = true;
		fPowerPC = false;
		fNeedsSystem7 = true;
	}
	
	if (fClassic)
	{
		if (fCompiler == kwUnspecified)
			fCompiler = kwSCpp;
		fPowerPC = false;
	}
	
	// Default the compiler if none specified
	
	if (fCompiler == kwUnspecified)
	{
		if (fPowerPC)
			fCompiler = kwMrC;
		else
		{
			fCompiler = kwSCpp;
			if (!fModelCFM)
				fClassic = true;
		}
	}
	
	// Force other options that are dependent on the compiler
	
	if ((fCompiler == kwMrC) || (fCompiler == kwMWCPPC))
	{
		fNames = false;
		fPerform = false;
		fModelFar = false;

		fModelFarCode = false;
		fModelFarData = false;

		fNeedsColorQD = true;
		fNeedsMC68020 = false;
		fNeedsMC68030 = false;
		fNeedsMC68040 = false;
		fPowerPC = true;
		fNeedsFPU = true;
		fNeedsSystem7 = true;
	}
	
	if (fCompiler == kwMWC68K)
		fPerform = false;

	if ((fCompiler == kwSCpp) || (fCompiler == kwMWC68K))
		fPowerPC = false;

	if ((fCompiler != kwMWCPPC) && (fCompiler != kwMWC68K))
		fFast = false;

	if (!fProgress)
		fEchoOptions.Catenate("  Dev:NULL ");
	
	if (!fPowerPC)
	{
		if (fDebug)
			fNames = true;
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::GetAutoBuild: 
//----------------------------------------------------------------------------------------
void TMABuildTool::GetAutoBuild()
{
	if (fUserAutoBuild)
	{
		CStr255_AC MAUserAutoBuild;
		if (IEgetenv("MAUserAutoBuild", MAUserAutoBuild) && !MAUserAutoBuild.IsEmpty())
			AddTargets(MAUserAutoBuild);
	}

	if (fAutoBuild)
	{
		CStr255_AC targetNames;
		bool defined;
		if (fPowerPC)
			defined = IEgetenv("MAAutoBuildPowerPC", targetNames);
		else
			defined = IEgetenv("MAAutoBuild", targetNames);
		
		if (defined && !targetNames.IsEmpty())
			AddTargets(targetNames);
		
		AddAutoBuilds();
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::AddAutoBuilds: 
//----------------------------------------------------------------------------------------
void TMABuildTool::AddAutoBuilds()
{
	CStr255_AC autoBuildExtension;
	if (IEgetenv("MAAutoBuildExtension", autoBuildExtension))
	{
		// Loop through the targets and get their autobuild requirements. 
		// The autobuild targets are added to the front of the list. 
		// Because we're using an iterator, the current index will be
		// adjusted when we insert new targets. However, we will not process
		// autobuild targets recursively, because the iterator also adjusts
		// the low bound when we insert. 
		
		for (TemplateIterator_AC<TStringHandle*> iter(&fTargStringList, kIterateBackward_AC); iter; ++iter)
		{
			ParseTargetName(iter->AsStr255());
			AutoBuildTarget(autoBuildExtension);
		}
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::AutoBuildTarget: 
//----------------------------------------------------------------------------------------
void TMABuildTool::AutoBuildTarget(const CStr255_AC& autoBuildExtension)
{
	// Look for a target.autobuild file
	CStr255_AC autobuild(fAppPath + fAppName + autoBuildExtension);
	if (Exists(autobuild))
	{
		FILE* autobuildFile = fopen(autobuild, "r");
		if (autobuildFile)
		{
			char targetNames[256];
			while (fgets(targetNames, 255, autobuildFile))
			{
				short l = strlen(targetNames) - 1;
				if ((l > 0) && (targetNames[l] == '\n') || (targetNames[l] == '\r'))
					targetNames[l] = 0;
				
				AddTargets(CStr255_AC(targetNames));
			}
			fclose(autobuildFile);
		}
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::AddTargets: 
//----------------------------------------------------------------------------------------
void TMABuildTool::AddTargets(CStr255_AC& targetNames)
{
	while (targetNames.Length() > 0)
	{
		short i = targetNames.Pos(",");
		i = (!i) ? targetNames.Length() : i - 1;
		
		fTargStringList.InsertFirst(TH_new TStringHandle(targetNames.Copy(1, i)));
		// Had to break up the following because SCPP was complaining.
//		targetNames.Delete(1, Min(i + 1, static_cast<short>(targetNames.Length()))); // toss the target
		short j = static_cast<short>(targetNames.Length());
		j = j <= i ? j : i + 1;
		targetNames.Delete(1, j); 		// toss the target
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoOptionFlags_Part1: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoOptionFlags_Part1()
{
	if (fRSRC || fPowerPC)
	{
		// Using MetroWerks or Symantec, so define a variable stating this.  This will only be used in strategic places in some
		// resources where CODE's are included.  Since we don't want this, we'll condition them with #ifdef qNoCode.
		
		//CatenateToSourceOptionStrings(" -d qNoCode=true");
		fRezOptions.Catenate(" -d qNoCode=true");
	}
	else
		fRezOptions.Catenate(" -d qNoCode=false");

	// Building for use with MacApp?
	if (fMacApp)
		CatenateToSourceOptionStrings(" -d qMacApp=true");
	else
		CatenateToSourceOptionStrings(" -d qMacApp=false");

	if (fAlign)
	{
		fLinkOptions.Catenate(" -ac 4 -ad 4");
		fRezOptions.Catenate(" -align longword");
	}

	// process elapsed time indication
	if (fTimes)
	{
		CStr255_AC aString(" -t");
		
		fAsmOptions.Catenate(aString);
		fCreatorAndBundleOptions.Catenate(aString);
		fCOptions.Catenate(aString);
		fCPlusOptions.Catenate(aString);
		fPPCAsmOptions.Catenate(aString);
	}

	// Progress indication
	if (fAllProgress)
	{
		CStr255_AC aString(" -p");
		
		CatenateToSourceOptionStrings(aString);
		fLibOptions.Catenate(aString);
		fLinkOptions.Catenate(aString);
//CW added
		fMWC68KOptions.Catenate(aString);
		fMWLink68KOptions.Catenate(aString);
		fMWCPPCOptions.Catenate(aString);
		fMWLinkPPCOptions.Catenate(aString);

		fCreatorAndBundleOptions.Catenate(aString);
		fPPCLibOptions.Catenate(aString);
		fPPCLinkOptions.Catenate(aString);
		fMakeSymOptions.Catenate(aString);
	}

	if (fPowerPC)
	{
		// PreSystem 7.5 support
		if (fNeedsSystem7_5)
		{
			fOptionFlags.Catenate("75");
			CatenateToSourceOptionStrings(" -d qNeedsSystem7_5=true -d SystemSevenFiveOrLater=true");
		}
		else
			CatenateToSourceOptionStrings(" -d qNeedsSystem7_5=false -d SystemSevenFiveOrLater=false");
	
		// System 7.0, ColorQD support is always on, FPU is always off
		if (!fNeedsSystem7_5)
			fOptionFlags.Catenate("S7");
		CatenateToSourceOptionStrings(" -d qNeedsAppleEventMgr=true");
		CatenateToSourceOptionStrings(" -d qNeedsEditionMgr=true");
		CatenateToSourceOptionStrings(" -d qNeedsHelpMgr=true");
		CatenateToSourceOptionStrings(" -d qNeedsProcessMgr=true");
		CatenateToSourceOptionStrings(" -d qNeedsAliasMgr=true");
		CatenateToSourceOptionStrings(" -d qNeedsFolderMgr=true");
		CatenateToSourceOptionStrings(" -d qNeedsSystem7=true");
		CatenateToSourceOptionStrings(" -d SystemSevenOrLater=true");
		fOptionFlags.Catenate("Cq");
		CatenateToSourceOptionStrings(" -d qNeedsColorQD=true");
		CatenateToSourceOptionStrings(" -d qNeedsFPU=false");
	}
	else
	{
		// PreSystem 7.5 support
		if (fNeedsSystem7_5)
		{
			fOptionFlags.Catenate("75");
			CatenateToSourceOptionStrings(" -d qNeedsSystem7_5=true -d SystemSevenFiveOrLater=true");
		}
		else
			CatenateToSourceOptionStrings(" -d qNeedsSystem7_5=false -d SystemSevenFiveOrLater=false");
	
		// PreSystem 7.0 support
		if (fNeedsSystem7 || fNeedsSystem7_5)
		{
			if (!fNeedsSystem7_5)
				fOptionFlags.Catenate("S7");
			CatenateToSourceOptionStrings(" -d qNeedsAppleEventMgr=true -d qNeedsEditionMgr=true -d qNeedsHelpMgr=true -d qNeedsProcessMgr=true -d qNeedsAliasMgr=true -d qNeedsFolderMgr=true -d qNeedsSystem7=true -d SystemSevenOrLater=true");
		}
		else
		{
			CatenateToSourceOptionStrings(" -d qNeedsAppleEventMgr=false -d qNeedsEditionMgr=false -d qNeedsHelpMgr=false -d qNeedsProcessMgr=false -d qNeedsAliasMgr=false -d qNeedsFolderMgr=false -d qNeedsSystem7=false -d SystemSevenOrLater=false");
			
			// Require System 6!
			CatenateToSourceOptionStrings(" -d SystemSixOrLater=false");
		}
	
		// ColorQD support
		if (fNeedsColorQD)
		{
			fOptionFlags.Catenate("Cq");
			CatenateToSourceOptionStrings(" -d qNeedsColorQD=true");
		}
		else
			CatenateToSourceOptionStrings(" -d qNeedsColorQD=false");
	
		// Processor dependent conditional compilation
		// 020 support
		if (fNeedsMC68020)
			CatenateToSourceOptionStrings(" -d qNeedsMC68020=true");
		else
			CatenateToSourceOptionStrings(" -d qNeedsMC68020=false");
	
		// 030 support
		if (fNeedsMC68030)
			CatenateToSourceOptionStrings(" -d qNeedsMC68030=true");
		else
			CatenateToSourceOptionStrings(" -d qNeedsMC68030=false");
	
		// 040 support
		if (fNeedsMC68040)
			CatenateToSourceOptionStrings(" -d qNeedsMC68040=true");
		else
			CatenateToSourceOptionStrings(" -d qNeedsMC68040=false");
	
		// Compiler targets only one processor!
		if (fNeedsMC68040)
		{
			fOptionFlags.Catenate("40");
			fCPlusCPUOptions.Catenate(" -mc68040");
		}
		else if (fNeedsMC68030)
		{
			fOptionFlags.Catenate("30");
			fCPlusCPUOptions.Catenate(" -mc68030");
		}
		else if (fNeedsMC68020)
		{
			fOptionFlags.Catenate("20");
			fCPlusCPUOptions.Catenate(" -mc68020");
		}
	
		// FPU support
		if (fNeedsFPU)
		{
			fOptionFlags.Catenate("Fp");
			CatenateToSourceOptionStrings(" -d qNeedsFPU=true");
			if (fCompiler == kwMWC68K)
				fCPlusCPUOptions.Catenate(" -mc68881");
			else
				fCPlusCPUOptions.Catenate(" -mc68881 -elems881");
		}
		else
			CatenateToSourceOptionStrings(" -d qNeedsFPU=false");
	
	} // fPowerPC

	// Virtual User support
	if (fNeedsVU)
	{
		fOptionFlags.Catenate("Vu");

		CatenateToSourceOptionStrings(" -d qNeedsVU=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qNeedsVU=false");

	// The Debugger support
	if (fTheDebugger)
	{
		fOptionFlags.Catenate("J");

		CatenateToSourceOptionStrings(" -d qTheDebugger=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qTheDebugger=false");

	// Embedded debugger names
	if (fNames)
	{
		fOptionFlags.Catenate("Nm");

		CatenateToSourceOptionStrings(" -d qNames=true");

		fMWC68KOptions.Catenate(" -mbg full");
	}
	else
	{
		CatenateToSourceOptionStrings(" -d qNames=false");
		if (!fPowerPC)
		{
			fCOptions.Catenate(" -mbg off");
			fMWC68KOptions.Catenate(" -mbg off");
			fCPlusOptions.Catenate(" -mbg off");
			fSCOptions.Catenate(" -mbg off");
		}
	}

	// Debugging support
	if (fDebug)
	{
		fOptionFlags.Catenate("Db");
		CatenateToSourceOptionStrings(" -d qDebug=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qDebug=false");

	// Inspector support
	if (fInspector)
	{
		fOptionFlags.Catenate("In");
		CatenateToSourceOptionStrings(" -d qInspector=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qInspector=false");

	// Debug messages support
	if (fDebugMsg)
	{
		fOptionFlags.Catenate("Dm");
		CatenateToSourceOptionStrings(" -d qDebugMsg=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qDebugMsg=false");

	// Perform support
	if (fPerform)
	{
		fOptionFlags.Catenate("Pe");
		CatenateToSourceOptionStrings(" -d qPerform=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qPerform=false");

	// RangeCheck support
	if (fRangeCheck)
	{
		fOptionFlags.Catenate("Ra");
		CatenateToSourceOptionStrings(" -d qRangeCheck=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qRangeCheck=false");

	// Attachability support
	if (fAttachable)
	{
		fOptionFlags.Catenate("At");
		CatenateToSourceOptionStrings(" -d qAttachable=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qAttachable=false");

	// Drag & Drop support
	if (fDrag)
	{
		fOptionFlags.Catenate("Dr");
		CatenateToSourceOptionStrings(" -d qDrag=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qDrag=false");

	// GX Printing support
	if (fGXPrinting)
	{
		fOptionFlags.Catenate("Gp");
		CatenateToSourceOptionStrings(" -d qGXPrinting=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qGXPrinting=false");

	// template views support
	if (fTemplateViews)
	{
		fOptionFlags.Catenate("Te");
		CatenateToSourceOptionStrings(" -d qTemplateViews=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qTemplateViews=false");

	// writing template views support
	if (fWriteTemplateViews)
	{
		fOptionFlags.Catenate("Wr");
		CatenateToSourceOptionStrings(" -d qWriteTemplateViews=true");
	}
	else
		CatenateToSourceOptionStrings(" -d qWriteTemplateViews=false");

	// symbolic debugging support
	if (fSym)
	{
		fOptionFlags.Catenate("Sm");
		CatenateToSourceOptionStrings(" -d qSym=true");
		
//CW added symOn and fullPath
		const CStr255_AC symOn = " -sym on";
		const CStr255_AC fullpath = "  -sym full";
		if (!fPowerPC)
		{
			CStr255_AC aString;

			if (fSmallSym)
				aString = " -sym on,notypes,novars";
			else
				aString = symOn; 

			fAsmOptions.Catenate(aString);
			fCOptions.Catenate(aString);

			if (fTheDebugger)
				fMWLink68KOptions.Catenate(" -sym tdb");
			fMWC68KOptions.Catenate(fullpath);
			fMWC68KOptions.Catenate(" -a6");
			fMWLink68KOptions.Catenate(fullpath);

			fCPlusOptions.Catenate(aString);
			fSCOptions.Catenate(aString);
			fLinkOptions.Catenate(aString);
		}
		else
		{
			fAsmOptions.Catenate(symOn);
			fPPCCOptions.Catenate(symOn);
			fMrCOptions.Catenate(symOn);
		
			// big sym support
			if (fCompiler == kwMWCPPC)
			{
				fMWCPPCOptions.Catenate(fullpath);
				fMWLinkPPCOptions.Catenate(fullpath);
			}
			else if (fSmallSym)
			{
				fPPCLibOptions.Catenate(symOn);
				fPPCLinkOptions.Catenate(symOn);
			}
			else
			{
				fPPCLibOptions.Catenate(" -sym big");
				fPPCLinkOptions.Catenate(" -sym big");
				fMakeSymOptions.Catenate(" -sym big");
			}
		}
	}
	else
		CatenateToSourceOptionStrings(" -d qSym=false");

}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoOptionFlags_Part2: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoOptionFlags_Part2()
{
	// Model far support
	if (fCompiler == kwMWC68K)
	{
		fMWLink68KOptions.Catenate("-br 020");
		if ((fModelFarCode && fModelFarData) || fModelFar)
		{
			fOptionFlags.Catenate("fCfD");
			fMWC68KOptions.Catenate(" -model far");
			fMWLink68KOptions.Catenate(" -model far");
		}
		else if (fModelFarCode)
		{
			fOptionFlags.Catenate("fC");
			
			fMWC68KOptions.Catenate(" -model farCode");
			fMWLink68KOptions.Catenate(" -model far");
		}
		else if (fModelFarData)
		{
			fOptionFlags.Catenate("fD");
			
			fMWC68KOptions.Catenate(" -model farData");
			fMWC68KOptions.Catenate(" -model farstrings");
			fMWC68KOptions.Catenate(" -model farvtables");
		}
	}
	else
	{
		fLinkOptions.Catenate("-br 020");
		if (fModelFarCode || fModelFarData || fModelFar)
		{
			fModelFar = true;
			fOptionFlags.Catenate("fCfD");
			fCOptions.Catenate(" -model farCode -model farData");
			fRunTimeModel.Catenate(" -model far");
		}
		else
		{
			if (!fModelCFM)
				fLinkOptions.Catenate(" -wrap");
		}
	}

	// Model CFM-68K support
	if (fModelCFM)
	{
		fOptionFlags.Catenate("CF");
		CatenateToSourceOptionStrings(" -d qModelCFM=true");
		fRunTimeModel.Catenate(" -model cfmseg");
	}
	else
	{
		CatenateToSourceOptionStrings(" -d qModelCFM=false");
	}
	
	// Classic 68K runtime support
	if (fClassic)
	{
		fOptionFlags.Catenate("Cl");
		CatenateToSourceOptionStrings(" -d qClassic=true");
	}
	else
	{
		CatenateToSourceOptionStrings(" -d qClassic=false");
	}
	
	// PowerPC support
	if (fPowerPC)
	{
		CatenateToSourceOptionStrings(" -d qPowerPC=true");
	}
	else
	{
		// We're not building a native app
		CatenateToSourceOptionStrings(" -d qPowerPC=false");
	}

	// Add the compiler
	switch (fCompiler)
	{
		case kwSCpp:
			fOptionFlags.Catenate("SC");
			break;
		
		case kwMrC:
			fOptionFlags.Catenate("Mr");
			break;
		
		case kwMWCPPC:
			fOptionFlags.Catenate("MW");
			break;
		
		case kwMWC68K:
			fOptionFlags.Catenate("M8");
			break;
		
		default:
			break;
	}

	// Process separate objects
	// Get basic name
	if (fSeparateObjects)
	{
		CStr255_AC tempString(fOptionFlags.AsStr255());
		if (!fRenameFlagsPairs.ValueAt(tempString, fSeparateObjectsFolder))
			fSeparateObjectsFolder = tempString;
	}
	else
	{
		if (!IEgetenv("MANoSeparateObjectsFolder", fSeparateObjectsFolder))
			fSeparateObjectsFolder.Empty();
	}

	// Add the prefix
	CStr255_AC MASeparateObjectsPrefix;
	if (IEgetenv("MASeparateObjectsPrefix", MASeparateObjectsPrefix))
		fSeparateObjectsFolder = MASeparateObjectsPrefix + fSeparateObjectsFolder;

	if (fSeparateObjectsFolder.Length() > 31)
	{
		fprintf(stderr, "\nMABuild: Warning: SeparateObjectsFolder '%s'; truncated to 31 characters\n",
				(const char*)(fSeparateObjectsFolder));
		fSeparateObjectsFolder.Length() = 31;
	}
	
	fSeparateObjectsFolder += fDirectorySeparator;

	SetIE("SeparateObjectsFolder", fSeparateObjectsFolder);

	if (fRAMDisk)
	{
		fPPCCOptions.Catenate(" -y '");
		fPPCCOptions.Catenate(fRAMDiskName);
		fPPCCOptions.Catenate("'");

		fMrCOptions.Catenate(" -y '");
		fMrCOptions.Catenate(fRAMDiskName);
		fMrCOptions.Catenate("'");
	}
	
	// Include any unconditional options that we might want to force on unsuspecting
	// tools but haven't the gall to include in the startup file.
}

//----------------------------------------------------------------------------------------
// TMABuildTool::OutputOptionFlags: 
//----------------------------------------------------------------------------------------
void TMABuildTool::OutputOptionFlags()
{
	// output the results.
	if (fTimes)
		fprintf(fOutputFile,"SET XStartTime %li\n",fStartDateTime);
	
	if (fRAMDisk && fCPlusLoad)
		SetIE("XRAMDiskLoad", "0");
	
	if (fFast)
	{
		fprintf(fOutputFile, "SET All_Cpp_Files \"\"\n");
		fprintf(fOutputFile, "SET All_Cpp_Names \"\"\n");
		fprintf(fOutputFile, "SET All_C_Files \"\"\n");
		fprintf(fOutputFile, "SET All_C_Names \"\"\n");
	}

	SetIE("MakeOptions", fMakeOptions);
	SetIE("XOptionFlags", fOptionFlags);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::ParseTargetName: 
//----------------------------------------------------------------------------------------
void TMABuildTool::ParseTargetName(CStr255_AC& targetName)
{
	// find the pathname and filename

	SubstituteInString(targetName);
	
	fAppName = targetName;
	fAppPath.Empty();
	
	for (short i = targetName.Length(); i >= 1; --i)
	{
		if (targetName.Copy(i, fDirectorySeparator.Length()) == fDirectorySeparator)
		{
			fAppPath = targetName.Copy(1, i);
			fAppName = targetName.Copy(i + 1, targetName.Length() - i);
			break;
		}
	}
	
	// Automatically trim .MAMake off the target name if it was specified
	if (IEgetenv("MAMakeFileExtension", fMAMakeFileExtension) && !fMAMakeFileExtension.IsEmpty())
	{
		CStr255_AC aString(fMAMakeFileExtension);
		CStr255_AC bString(fAppName);
		UprStr255(aString);
		UprStr255(bString);
		short j = aString.Pos(bString);
		if (j && (j + aString.Length() - 1 == fAppName.Length()))
			fAppName.Delete(j, aString.Length());
	}

	// Blank path is current path
	if (fAppPath.IsEmpty())
		fAppPath = fStartPath;
}

//----------------------------------------------------------------------------------------
// TMABuildTool::CheckSources: 
//----------------------------------------------------------------------------------------
//	bool TMABuildTool::CheckSources(const CStr255_AC& srcVar, const CStr255_AC& srcFile,
//									   const CStr255_AC& objVar, const CStr255_AC& objFile)
//	{
//		bool exists = Exists(srcFile);
//		if (exists)
//		{
//			SetIE(srcVar, srcFile);
//			if (objVar.Length() > 0)
//				SetIE(objVar, objFile);
//		}
//		else
//		{
//			SetIE(srcVar, "");
//			if (objVar.Length() > 0)
//				SetIE(objVar, "");
//		}
//		return exists;
//	}

//----------------------------------------------------------------------------------------
// TMABuildTool::CreateBuildFolder: 
//----------------------------------------------------------------------------------------
void TMABuildTool::CreateBuildFolder()
{
	// Define the MAObj folder. We do this here because its used in building
	// each target. see 'Basic Definitions'/'Basic DefinitionsFT'
		
	SetIE("BuildFolder", fBuildFolder);
	
	if (fBuildFolder.Length() > 0)
	{
		fMAObj = fBuildFolder;
		CreateFolder(fMAObj);
		
		fMAObj += "MacApp.Lib";
		CreateFolder(fMAObj);
			
		fMAObj += fDirectorySeparator + fSeparateObjectsFolder;
		SetIE("MAObj", fMAObj);
		CreateFolder(fMAObj);
	}
	else
	{
		IEgetenv("MALibraries", fMAObj);
		fMAObj += fSeparateObjectsFolder;
		SetIE("MAObj", fMAObj);
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::CreateObjectsFolder: 
//----------------------------------------------------------------------------------------
void TMABuildTool::CreateObjectsFolder()
{
	// Make sure alternate build folder exists and AppName folder in build folder.
	if (fBuildFolder.Length() > 0)
		CreateFolder(fBuildFolderAppName);

	// Make sure separate objects folder exists
	CreateFolder(fObjApp);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoAllTargets: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoAllTargets()
{
	// Now process every target in fTargStringList
	
	TStringHandle* stringHandle = (TStringHandle*)(fTargStringList.First());
	while (stringHandle)
	{
		fTargStringList.Delete(stringHandle);

		ParseTargetName(stringHandle->AsStr255());// Paths can't currently be longer than this anyways! (MPW 3.1)
		delete stringHandle;

		//CW had to split these up because of 32K local data limitation
		DoATarget_Part1();
		DoATarget_Part2();
		
		stringHandle = (TStringHandle*)(fTargStringList.First());
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoATarget_Part1: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoATarget_Part1()
{
	SetIE("XAppPath", fAppPath);
	SetIE("XAppName", fAppName);

	// Check for the  MASetupExtension file
	{
		CStr255_AC MASetupExtension;
		if (IEgetenv("MASetupExtension", MASetupExtension))
		{
			CStr255_AC aString = fAppPath + fAppName + MASetupExtension;
			if (Exists(aString))
				fprintf(fOutputFile,"EXECUTE \"%s\"\n",(const char*)(aString));
		}
	}

	// Process separate objects into the source and object pathnames
	if (fBuildFolder.Length() > 0)
	{
		fBuildFolderAppName = fBuildFolder;
		if (fAppName == "MacApp.lib.xcoff")
			fBuildFolderAppName += "MacApp.Lib";
		else
			fBuildFolderAppName += fAppName;
		fObjApp = fBuildFolderAppName + fDirectorySeparator + fSeparateObjectsFolder;
	}
	else
	{
		fBuildFolderAppName.Empty();
		fObjApp = fAppPath + fSeparateObjectsFolder;
	}

	fSrcApp = fAppPath;
	
	{
		CStr255_AC MASourcesFolder;
		if (IEgetenv("MASourcesFolder", MASourcesFolder))
		{
			CStr255_AC srcAppSrc = fSrcApp + MASourcesFolder;
			if (Exists(srcAppSrc))
				fSrcApp = srcAppSrc;
		}
	}
	
	SetIE("ObjApp", fObjApp);
	//SetIE("LoadFileDir", fObjApp);
	SetIE("SrcApp", fSrcApp);
	
	{
		fIncApp = fAppPath;
		CStr255_AC MAIncludesFolder;
		if (IEgetenv("MAIncludesFolder", MAIncludesFolder))
		{
			short saveLen = fIncApp.Length();
			fIncApp += MAIncludesFolder;
			if (!Exists(fIncApp))
				fIncApp.Length() = saveLen;
		}
		SetIE("IncApp", fIncApp);
	}
	
	if (fProgress)
		Echo("\"Target Folder: \"{ObjApp}\"\"");
	 
	// Linkmap and LinkXref
	if (fLinkMap)
	{
		if (fCompiler == kwMWC68K || fCompiler == kwMWCPPC)
			fXLinkMap = "-map > \"{ObjApp}{XAppName}.map\" ";
		else if (!fPowerPC && fPerform)
			fXLinkMap = "-la -lf -l > \"{ObjApp}{XAppName}.map\" ";
		else
			fXLinkMap = "-map  \"{ObjApp}{XAppName}.map\" ";
	}
	else
		fXLinkMap.Empty();
	
	if (fLinkXRef && (fCompiler != kwMWC68K && fCompiler != kwMWCPPC))
		fXLinkXRef = "-x \"{ObjApp}{XAppName}.xref\" ";
	else
		fXLinkXRef.Empty();
	
	// in C++
	fAnyCPlus = false;

	fXUAppName_h.Empty();
	fXUAppName_cp.Empty();
	fXUAppName_cp_o.Empty();
	fXMAppName_cp.Empty();
	fXMAppName_cp_o.Empty();
	fXAppName_cp.Empty();
	fXAppName_cp_o.Empty();
	
	if (!fRSRC)
	{
		CStr255_AC fileName;
		
		fileName = fSrcApp + "U" + fAppName + ".h";
		if (Exists(fileName))
		{
			fAnyCPlus = true;
			fXUAppName_h = fileName;
		}
	
		fileName = "U" + fAppName + ".cp";
		if (Exists(fSrcApp + fileName))
		{
			fAnyCPlus = true;
			fXUAppName_cp = fSrcApp + fileName;
			fXUAppName_cp_o = fObjApp + fileName + ".o";
		}
	
		fileName = "M" + fAppName + ".cp";
		if (Exists(fSrcApp + fileName))
		{
			fAnyCPlus = true;
			fXMAppName_cp = fSrcApp + fileName;
			fXMAppName_cp_o = fObjApp + fileName + ".o";
		}
	
		fileName = fAppName + ".cp";
		if (Exists(fSrcApp + fileName))
		{
			fAnyCPlus = true;
			fXAppName_cp = fSrcApp + fileName;
			fXAppName_cp_o = fObjApp + fileName + ".o";
		}
	}
	
	SetIE("XUAppName.h", fXUAppName_h);
	SetIE("XUAppName.cp", fXUAppName_cp);
	SetIE("XUAppName.cp.o", fXUAppName_cp_o);
	SetIE("XMAppName.cp", fXMAppName_cp);
	SetIE("XMAppName.cp.o", fXMAppName_cp_o);
	SetIE("XAppName.cp", fXAppName_cp);
	SetIE("XAppName.cp.o", fXAppName_cp_o);

	// See if we can automake it
	if (!Exists(fAppPath + fAppName + fMAMakeFileExtension))
	{
		fAutomake = true;
		if (!fAnyCPlus && !fRSRC)
		{
			Failed("Bad parameter: Unable to access file: \"{XAppPath}{XAppName}{MAMakeFileExtension}\"");
		}
	}
	else
		fAutomake = false;

	// See if we can autorez it
	if (!Exists(fAppPath + fAppName + ".r"))
	{
		fAutorez = true;
		SetIE("XAutoRez", "1");
		SetIE("XAppRezSrc", "{MAFramework}Includes:Default.r");
	}
	else
	{
		fAutorez = false;
		SetIE("XAutoRez", "0");
		SetIE("XAppRezSrc", "{XAppPath}{XAppName}.r");
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoATarget_Part2: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoATarget_Part2()
{
	// Make sure alternate build folder exists and AppName folder in build folder.
	CreateObjectsFolder();

	// SET the BuildFlags
	{
		CStr255_AC MABuildFlagsExtension;
		IEgetenv("MABuildFlagsExtension", MABuildFlagsExtension);
		fBuildFlags = fObjApp + fAppName + MABuildFlagsExtension;
	}
	SetIE("BuildFlags", fBuildFlags);

	// Find out what the last option flags were
	bool existsBuildFlags = Exists(fBuildFlags);
	if (existsBuildFlags)
		Execute("EXECUTE \"{BuildFlags}\"  Dev:NULL || SET Status 0");
	else
		SetIE("XLastOptionFlags", "InvalidString");
	
	if (fExecute)
	{
		Execute("IF {XLastOptionFlags} != {XOptionFlags}");
		// the file BuildFlags will contain a SET command to set the value of XLastOptionFlags
		Execute("ECHO \"SET XLastOptionFlags {XOptionFlags}\"  >  \"{BuildFlags}\"");
		Execute("END");
	}
	else if (!existsBuildFlags)
	{
		// the file BuildFlags will contain a SET command to set the value of XLastOptionFlags
		Echo("\"SET XLastOptionFlags {XOptionFlags}\"  >  \"{BuildFlags}\"");
	}

	// Rebuild the application by creating the "MakeIt" file and then executing it
	SetIE("XMakeIt", "{ObjApp}{XAppName}{MAMakeOutfileExtension}");

	// SET the failure processing mode in the makeit files and (OPTIONALLY) the
	// active environment vars into the makefile
	if (fNoFail)
		Execute("ECHO \"SET EXIT 0\"  > \"{XMakeIt}\"");
	else
		Execute("ECHO \"SET EXIT 1\"  > \"{XMakeIt}\"");
	
	//	if (fExpandEnvironmentVars)
	//		ExpandEnvironmentVars();

	//	// Export the various and sundry environment variables
	//	if (!fEverExported)
	//	{
	//		fEverExported = true;					// only need to export once
	//		ExportEnvironmentVars();
	//	}

	//if (!fRSRC)
	//{
	// Give user makefile processing status message
	if (fProgress)
	{
		if (fAutomake)
			Echo("\"AutoMaking:    {XAppName}\"");
		else
			Echo("\"Making:        {XAppName}{MAMakeFileExtension}\"");
	}
	//}

	if (fTimes)
		SetIE("XMakeStartTime", "`DATE -n`");
	SetIE("EXIT", "0");

	// Create the make file
	CreateMakeFile();

	// run make
	MakeTarget();

	SetIE("XMakeStatus", "{Status}");
	SetIE("EXIT", "1");
	Execute("IF \"{XMakeStatus}\"");
	Echo("MAKE of {XAppName} failed: `DATE`");
	if (fTimes)
		Echo("Elapsed time: `evaluate `DATE -n` - {XMakeStartTime}` seconds");
	Execute("{MAFailed}");
	if (fNoFail)
		SetIE("XExitStatus", "{XMakeStatus}");
	else
		Execute("EXIT \"{XMakeStatus}\"");
	
	if (fTimes)
	{
		Execute("ELSE");
		Echo("Elapsed time: `evaluate `DATE -n` - {XMakeStartTime}` seconds");
	}
	Execute("END");

	// Attempt Execution and let the user know how it all came out
	if (fExecute)
	{
		SetIE("EXIT", "0");
		Execute("\"{XMakeIt}\"");
		SetIE("XRunStatus", "{Status}");
		SetIE("EXIT", "1");
		Execute("IF \"{XRunStatus}\"");
		Execute("{MAFailed}");
		if (fNoFail)
			SetIE("XExitStatus", "{XRunStatus}");
		else
			Execute("EXIT \"{XRunStatus}\"");
		Execute("END");
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::ExpandEnvironmentVars: 
//----------------------------------------------------------------------------------------
void TMABuildTool::ExpandEnvironmentVars()
{
}

//----------------------------------------------------------------------------------------
// TMABuildTool::ExportEnvironmentVars: 
//----------------------------------------------------------------------------------------
void TMABuildTool::ExportEnvironmentVars()
{
}

//----------------------------------------------------------------------------------------
// TMABuildTool::CreateMakeFile: 
//----------------------------------------------------------------------------------------
void TMABuildTool::CreateMakeFile()
{
	CStr255_AC makeFileName = fObjApp + fAppName + ".make";
	
	// See if we can get away without writing a new make file
	bool writeThisMakeFile = true;
	if (!fCreateMake && !fAutomake && Exists(makeFileName))
	{
		// We're not auto-making, which means there must be a MAMake file, 
		// and the make file does exist. Is it up to date?
		unsigned long maMakeFileModDate = GetFileModDate(fAppPath + fAppName + fMAMakeFileExtension);
		unsigned long makeFileModDate = GetFileModDate(makeFileName);
		if (makeFileModDate > maMakeFileModDate)
		{
			writeThisMakeFile = false;
			if (fVerbose)
				fprintf(stdout, "Using:         %s\n", (const char*)((fAppName + ".make")));
		}
	}
	
	if (writeThisMakeFile)
	{
		if (fVerbose)
			fprintf(stdout, "Creating:      %s\n", (const char*)((fAppName + ".make")));

		fMakeFile = fopen(makeFileName, "w");
		
		fprintf(fMakeFile, "# %s\n", (const char*)(makeFileName));
		{
			unsigned long theDateTime;
			CStr255_AC theDateTimeString;
			
			GetDateTime(&theDateTime);
		
			IUTimeString(theDateTime, true, theDateTimeString);
			fprintf(fMakeFile, "# Created by MABuild: %s", (const char*)(theDateTimeString));
		
			IUDateString(theDateTime, shortDate, theDateTimeString);
			fprintf(fMakeFile, " %s\n\n", (const char*)(theDateTimeString));
		}
	
		OutputMakeVariables();
	
		DoDefinitions();
		
		if (fAutomake)
			SetMakeVariable("AppName", fAppName); 	// the default rules supply the rest
		
		if (fFat)
			SetMakeVariable("Merge68K", fAppName68K);
		
		DoOverrides();
		
		if (!fAutomake)
			CatenateMake(fAppPath + fAppName + fMAMakeFileExtension); // a makefile was supplied
		
		DoDependencies();
	
		fclose(fMakeFile);
		
		// The makefile is opened with a creator of 'TTXT'
		// The easiest way to patch it is in the script
		Execute("SetFile -c 'MPS ' \"" + makeFileName + "\"");
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::CatenateMake: 
//----------------------------------------------------------------------------------------
void TMABuildTool::CatenateMake(const CStr255_AC& fileName)
{
	CatenateFile(fileName, fMakeFile);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::MakeTarget: 
//----------------------------------------------------------------------------------------
void TMABuildTool::MakeTarget()
{
	// run make
	Execute("{MAMake} {MakeOptions} ");
	Execute("-f \"{ObjApp}{XAppName}.make\" ");

	// Supply a target
	if (fRSRC)
	{
		if (fAppName == "MacApp.lib.xcoff")
			Execute("\"{ObjApp}MacApp.Lib..rsrc\" ");
		else
			Execute("\"{ObjApp}{XAppName}..rsrc\" ");
	}
	else
		Execute("\"{ObjApp}{XAppName}\" ");
	
	if (!fStatusOnly)
		Execute(">> \"{XMakeIt}\"");
	else
		Execute("");
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoOverrides: 
//----------------------------------------------------------------------------------------


void TMABuildTool::DoOverrides()
{
	// MacApp as library support
	if (fMALibrary)
		SetMakeVariable("MacAppObjs=");
	else
		SetMakeVariable("MacAppLibrary=");

	if (fCPlusLoad)
	{
		// If we don't have a RAM disk, just use the original C++ load files
		if (fRAMDisk)
			SetMakeVariable("CPlusLoadOriginal=");
		else
			SetMakeVariable("CPlusLoadCopy=");
	}
	else
	{
		SetMakeVariable("CPlusLoad=");
		SetMakeVariable("CPlusLoadObj=");
		SetMakeVariable("CPlusLoadOptions="); // Remove the definitions
		SetMakeVariable("CPlusLoadOriginal=");
		SetMakeVariable("CPlusLoadCopy=");
	}
	
	// Debugging support
	if (!fDebug)
	{
		SetMakeVariable("DebugFiles=");
		SetMakeVariable("DebugRsrcs=");
		SetMakeVariable("DebugLib="); // Eliminate debug files as targets in the makefiles
	}
	
	// Performance analysis support
	if (!fPerform)
		SetMakeVariable("PerformLib=");
	
	if (fCompiler == kwMWC68K || fCompiler == kwMWCPPC)
	{
		SetMakeVariable("MAPrefixFile=");
		SetMakeVariable("LoadFileOutput=");
		SetMakeVariable("Stubs=");
	}
	else
	{
		SetMakeVariable("CLoadOptions=");
		SetMakeVariable("CWPrefixFile=");
	}

	if (fCompiler == kwMWC68K)
	{
		// Select the proper CodeWarrior floating point support
		if (fNeedsFPU)
		{
			SetMakeVariable("CWLibrariesNoFPU=");
			SetMakeVariable("CWNearNonFPUStdCLib=");
			SetMakeVariable("CWFarNonFPUStdCLib=");
			SetMakeVariable("CWToolNearNonFPUStdCLib=");
			SetMakeVariable("CWToolFarNonFPUStdCLib=");
		}
		else
		{
			SetMakeVariable("CWFPUSANELib=");
			SetMakeVariable("CWNearFPUStdCLib=");
			SetMakeVariable("CWFarFPUStdCLib=");
			SetMakeVariable("CWToolNearFPUStdCLib=");
			SetMakeVariable("CWToolFarFPUStdCLib=");
		}
		
		// Select the proper CodeWarrior run-time model
		if (fModelFarData || fModelFar)
		{
			SetMakeVariable("CWNearFPUStdCLib=");
			SetMakeVariable("CWNearNonFPUStdCLib=");
			SetMakeVariable("CWToolNearFPUStdCLib=");
			SetMakeVariable("CWToolNearNonFPUStdCLib=");
		}
		else
		{
			SetMakeVariable("CWFarNonFPUStdCLib=");
			SetMakeVariable("CWFarFPUStdCLib=");
			SetMakeVariable("CWToolFarNonFPUStdCLib=");
			SetMakeVariable("CWToolFarFPUStdCLib=");
		}
	}
	else if (!fPowerPC)
	{
		// Select the proper floating point support
		if (fNeedsFPU)
			SetMakeVariable("LibrariesNoFPU=");
		else
			SetMakeVariable("LibrariesFPU=");

		// Select the proper link libraries for modelfar support by eliminating
		// the definition of ModelFarSupport if it's not used
		if (!fModelFar)
			SetMakeVariable("ModelFarSupport=");
	}

	// Drag & Drop Support
	if (!fDrag)
	{
		// Knock out the Drag & Drop support
		SetMakeVariable("DragLib=");
		if (fPowerPC || fModelCFM)
			SetMakeVariable("DragLibMapping=");
	}
	
	// PowerTalk Mailer Support
	if (!fPowerTalk)
	{
		// Knock out the AOCE support
		SetMakeVariable("AOCELib=");
		if (fPowerPC || fModelCFM)
			SetMakeVariable("AOCELibMapping=");
	}
		
	//GXPrinting Support
	if (!fGXPrinting)
	{
		// Knock out the GX support
		SetMakeVariable("GXLib=");
		if (fPowerPC || fModelCFM)
			SetMakeVariable("GXLibMapping=");
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoDefinitions: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoDefinitions()
{
	CStr255_AC MADependencies;
	IEgetenv("MADependencies", MADependencies);
	
	// RSRC has to go first because it overrides PPC or 68K
	if (fRSRC)
		CatenateMake(MADependencies + "Definitions_RSRC");
	else
	{
		CatenateMake(MADependencies + "Definitions_Common");
		
		if (fPowerPC)
		{
			CatenateMake(MADependencies + "Definitions_PPC");
			if (fCompiler == kwMrC)
				CatenateMake(MADependencies + "Definitions_MrC");
			else if (fCompiler == kwMWCPPC)
				CatenateMake(MADependencies + "Definitions_MWPPC");
			else
				CatenateMake(MADependencies + "Definitions_PPCC");
		}
		else
		{
			CatenateMake(MADependencies + "Definitions_68K");
			if (fModelCFM)
				CatenateMake(MADependencies + "Definitions_CFM68K");
			else if (fClassic)
				CatenateMake(MADependencies + "Definitions_Classic");
			else if (fCompiler == kwSCpp)
				CatenateMake(MADependencies + "Definitions_SC");
			else if (fCompiler == kwMWC68K)
				CatenateMake(MADependencies + "Definitions_MW68K");
		}
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoDependencies: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoDependencies()
{
	CStr255_AC MADependencies;
	IEgetenv("MADependencies", MADependencies);
	
	// RSRC has to go first because it overrides PPC or 68K
	if (fRSRC)
		CatenateMake(MADependencies + "Dependencies_RSRC");
	else if (fPowerPC)
	{
		if (fFast)
			CatenateMake(MADependencies + "Fast_Dependencies_PPC");
		else
			CatenateMake(MADependencies + "Dependencies_PPC");
	}
	else
	{
		if (fFast)
			CatenateMake(MADependencies + "Fast_Dependencies_68K");
		else
			CatenateMake(MADependencies + "Dependencies_68K");
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetMakeVariable: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetMakeVariable(const CStr255_AC& theVariable,
								   const TStringHandle& theValue)
{
	fprintf(fMakeFile, "%s = ", (const char*)(theVariable));
	theValue.WriteToFile(fMakeFile);
	fprintf(fMakeFile, "\n");
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetMakeVariable: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetMakeVariable(const CStr255_AC& theVariable,
								   const CStr255_AC& theValue)
{
	fprintf(fMakeFile, "%s = %s\n", (const char*)(theVariable), (const char*)(theValue));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetMakeVariable: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetMakeVariable(const CStr255_AC& theVariable,
								   bool theValue)
{
	short boolAsShort = theValue;
	fprintf(fMakeFile, "%s = %i\n", (const char*)(theVariable), boolAsShort);
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetMakeVariable: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetMakeVariable(const CStr255_AC& aString)
{
	fprintf(fMakeFile, "%s\n", (const char*)(aString));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetMakeIE: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetMakeIE(const CStr255_AC& theVariable)
{
	CStr255_AC theValue;
	IEgetenv(theVariable, theValue);
	fprintf(fMakeFile, "%s = %s\n", (const char*)(theVariable), (const char*)(theValue));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetMakeIE: 
//----------------------------------------------------------------------------------------
void TMABuildTool::SetMakeIE(const CStr255_AC& theVariable,
							 const CStr255_AC& theIEVariable)
{
	CStr255_AC theValue;
	IEgetenv(theIEVariable, theValue);
	fprintf(fMakeFile, "%s = %s\n", (const char*)(theVariable), (const char*)(theValue));
}

//----------------------------------------------------------------------------------------
// TMABuildTool::OutputMakeVariables: 
//----------------------------------------------------------------------------------------
void TMABuildTool::OutputMakeVariables()
{
	// output the results.
	//	if (fTimes)
	//		fprintf(fOutputFile," Echo XStartTime = %li\n", fStartDateTime);
	
	SetMakeVariable("XTimes", fTimes);
	SetMakeVariable("XRunAfterBuild", fRunAfterBuild);
	SetMakeVariable("XUseRAMDisk", fRAMDisk && fCPlusLoad);
	SetMakeVariable("XRAMDiskName", fRAMDiskName);
	//	if (fRAMDisk && fCPlusLoad)
	//		SetMakeVariable("XRAMDiskLoad = 0");
	SetMakeVariable("LoadFileDir", fObjApp);
	
	SetMakeIE("CIncludes");
	SetMakeIE("MACPlusIncludes");
	SetMakeIE("MARIncludes");
	SetMakeIE("MALibraries");

	SetMakeIE("MAAsmPaths");
	SetMakeIE("MACPlusPaths");
	SetMakeIE("MARezPaths");

	SetMakeIE("MAApp");
	SetMakeIE("MACore");
	SetMakeIE("MADocs");
	SetMakeIE("MAFramework");
	SetMakeIE("MAMail");
	SetMakeIE("MATool");
	SetMakeIE("MAViews");

	SetMakeIE("MAAsm");
	SetMakeIE("MAC");
	SetMakeIE("MACPlus");
	SetMakeIE("MAEcho");
	SetMakeIE("MALib");
	SetMakeIE("MARez");
	
	SetMakeVariable("AsmOptions", fAsmOptions);
		
	if (!fPowerPC)
	{
		if (fCompiler == kwMWC68K )
		{
			SetMakeVariable("CPlusOptions", fMWC68KOptions);
			SetMakeVariable("COptions", fMWC68KOptions);
			SetMakeVariable("SCOptions =");
			SetMakeIE("MAC", "MAmwc68K");
			SetMakeIE("MACPlus", "MAmwc68K");
		}
		else
		{
			SetMakeVariable("COptions", fCOptions);

			if (fCompiler == kwSCpp)
			{
				SetMakeVariable("CPlusOptions =");
				SetMakeVariable("MAC = SC");
				SetMakeIE("MACPlus", "MASCpp");
				SetMakeVariable("SCOptions", fSCOptions);
			}
			else
			{
				SetMakeVariable("SCOptions =");
				SetMakeVariable("CPlusOptions", fCPlusOptions);
			}
		}
		SetMakeVariable("CPlusCPUOptions", fCPlusCPUOptions);
	}
	else // fPowerPC
	{
		SetMakeVariable("PPCAsmOptions", fPPCAsmOptions);
		
		if (fCompiler == kwMWCPPC)
		{
			SetMakeIE("MAPPCC", "MAmwCPPC");
			SetMakeIE("MAPPCLink", "MAmwLinkPPC");
			SetMakeVariable("MrCOptions =");
			SetMakeVariable("PPCCOptions", fMWCPPCOptions);
		}
		else if (fCompiler == kwMrC)
		{
			SetMakeVariable("PPCCOptions =");
			SetMakeIE("MAPPCC", "MAMrCPlus");
			SetMakeVariable("MrCOptions", fMrCOptions);
		}
		else
		{
			SetMakeVariable("MrCOptions =");
			SetMakeVariable("PPCCOptions", fPPCCOptions);
		}
	}

	SetMakeVariable("EchoOptions", fEchoOptions);
			
	if (fCompiler == kwMWC68K)
	{
		SetMakeIE("MALink", "MAmwLink68K");
		SetMakeVariable("LinkOptions", fMWLink68KOptions);
		SetMakeIE("MALib", "MAmwLink68K");
		
		fprintf(fMakeFile, "LibOptions = -library ");
		fMWLink68KOptions.WriteToFile(fMakeFile);
		fprintf(fMakeFile, "\n");
	}
	else
	{
		SetMakeVariable("LibOptions", fLibOptions);
		SetMakeVariable("LinkOptions", fLinkOptions);
	}
	
	if (fModelCFM)
	{
		SetMakeIE("MALink", "MAILink");
	}
	
	if (fPowerPC)
	{
		if (fCompiler == kwMWCPPC)
		{
			fprintf(fMakeFile, "PPCLibOptions =  -xm library ");
			fMWLinkPPCOptions.WriteToFile(fMakeFile);
			fprintf(fMakeFile, "\n");

			SetMakeVariable("PPCLinkOptions", fMWLinkPPCOptions);
		}
		else
		{
			SetMakeVariable("PPCLibOptions", fPPCLibOptions);
			SetMakeVariable("PPCLinkOptions", fPPCLinkOptions);
		}
		
		SetMakeVariable("MakeSymOptions", fMakeSymOptions);
	}
	
	SetMakeVariable("RunTimeModel", fRunTimeModel);
	
	if (fCompiler == kwMWC68K || fCompiler == kwMWCPPC)
		SetMakeIE("CIncludes", "CWExtraFiles");

	//	if (fFast)
	//	{
	//		fprintf(fOutputFile, "SET All_Cpp_Files \"\"\n");
	//		fprintf(fOutputFile, "SET All_Cpp_Names \"\"\n");
	//		fprintf(fOutputFile, "SET All_C_Files \"\"\n");
	//		fprintf(fOutputFile, "SET All_C_Names \"\"\n");
	//	}

	//SetMakeVariable("MakeOptions", fMakeOptions);
	SetMakeVariable("RezOptions", fRezOptions);
	SetMakeVariable("CreatorAndBundleOptions", fCreatorAndBundleOptions);
	SetMakeVariable("XOptionFlags", fOptionFlags);
	
	SetMakeVariable("SeparateObjectsFolder", fSeparateObjectsFolder);
	SetMakeVariable("BuildFolder", fBuildFolder);
	SetMakeVariable("MAObj",& fMAObj);
	SetMakeVariable("XAppPath", fAppPath);
	SetMakeVariable("ObjApp", fObjApp);
	SetMakeVariable("SrcApp", fSrcApp);
	SetMakeVariable("IncApp", fIncApp);

	SetMakeVariable("XLinkMap", fXLinkMap);
	SetMakeVariable("XLinkXRef", fXLinkXRef);

	SetMakeVariable("XAppName", fAppName);
	SetMakeVariable("XUAppName.h", fXUAppName_h);
	SetMakeVariable("XUAppName.cp", fXUAppName_cp);
	SetMakeVariable("XUAppName.cp.o", fXUAppName_cp_o);
	SetMakeVariable("XMAppName.cp", fXMAppName_cp);
	SetMakeVariable("XMAppName.cp.o", fXMAppName_cp_o);
	SetMakeVariable("XAppName.cp", fXAppName_cp);
	SetMakeVariable("XAppName.cp.o", fXAppName_cp_o);
	
	SetMakeVariable("XAutoRez", fAutorez);
	if (fAutorez)
		SetMakeVariable("XAppRezSrc", "{MAFramework}Includes:Default.r");
	else
		SetMakeVariable("XAppRezSrc", fAppPath + fAppName + ".r");

	SetMakeVariable("BuildFlags", fBuildFlags);
	SetMakeVariable("XMakeIt", fObjApp + fAppName + "{MAMakeOutfileExtension}");
}

//----------------------------------------------------------------------------------------
// TMABuildTool::OutputTheEnd: 
//----------------------------------------------------------------------------------------
void TMABuildTool::OutputTheEnd()
{
	// Close the output file.
	if (fOutputFile != stdout)
		fclose(fOutputFile);
		
	// termination messages
	if (fTimes)
	{
		Echo("\" \"");
		Echo("MABuild:   Elapsed time: `evaluate `DATE -n` - {XStartTime}` seconds");
	}

	Execute("IF \"{XExitStatus}\"");
	Execute("{MAFailed}");
	Execute("EXIT \"{XExitStatus}\"");
	Execute("END");

	if (fProgress)
	{
		Echo("Completion time for MABuild is `DATE`");
		Execute("{MADone}");
	}
}

//----------------------------------------------------------------------------------------
// TMABuildTool::SetOutputFile:
//----------------------------------------------------------------------------------------
void TMABuildTool::SetOutputFile(const CStr255_AC& outputFileName)
{
	fOutputFile = fopen(outputFileName, "w");
	if (!fOutputFile)
		Stop("MABuild: Unable to create or open output file.");
}

//----------------------------------------------------------------------------------------
// TMABuildTool::DoToolAction: 
//----------------------------------------------------------------------------------------
void TMABuildTool::DoToolAction()
{
	{
		CStr255_AC MAShortVersion;
		if (!IEgetenv("MAShortVersion", MAShortVersion))
		{
			Stop("MABuild: Whoops You have not executed the Startup file in the MacApp directory");
		}
		else if (MAShortVersion != MA_SHORT_VERSION)
		{
			fprintf(stderr, "\nMABuild: Warning: The MacApp version seems to be '%s'; should be '%s'\n",
					(const char*)(MAShortVersion), MA_SHORT_VERSION);
		}
	}
	
	// Resolve matrix of options.
	CheckKeywords();

	// Set up the option flags.
	DoOptionFlags_Part1();
	DoOptionFlags_Part2();
	
	if (fGetBuildFlags || fGetFolderName)
	{
		if (fGetFolderName)
			Execute("ECHO \"{SeparateObjectsFolder}\"");
		else
		{
			fprintf(fOutputFile, "SET XOptionFlags \"");
			fOptionFlags.WriteToFile(fOutputFile);
			fprintf(fOutputFile, "\"\n");
			Execute("ECHO \"{XOptionFlags}\"");
		}
		return;
	}
	
	// Get the autobuild targets, if any.
	GetAutoBuild();

	// output the results.
	OutputOptionFlags();

	// Process autosave
	if (fSaveBeforeBuild)
	{
		if (fProgress)
			Echo("\"AutoSaving\"");
		Execute("Save -a");
	}

	// Remember the current directory
	short vRefNum;
	long dirID;
	if (HGetVol(NULL, &vRefNum, &dirID) == noErr)
		PathNameFromDirID(dirID, vRefNum, fStartPath);

	// Create the alternate build folder
	CreateBuildFolder();
	
	// process each target in the list
	DoAllTargets();

	// termination messages
	OutputTheEnd();
}

//----------------------------------------------------------------------------------------
// main: 
//----------------------------------------------------------------------------------------
void main(int argc, char** argv)
{
	InitUCPlusTool();

	TMABuildTool* theMABuildTool = TH_new TMABuildTool(argc, argv);

	theMABuildTool->Run();
}

//----------------------------------------------------------------------------------------
// End of MABuildTool.cp
